
// clang-format off

/*******************************************************************************
* Copyright 2017-2018 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/
//
// !!!!!!!!!!!!!!THIS FILE IS AUTOGENERATED OUTSIDE OF THE BUILD PROCESS!!!!!!!!!!!!!!
//
// It takes quite awhile to compute the results, so doing it at cmake time is not a good option.
//
// If you want to add new tests, you should edit test/ref_generators/generate_convolution_ref.py
// and regenerate this file.
//
// To regenerate:
//
//   $ cd <ngraph source dir>/test
//   $ ./update_reference.sh
//
// !!!!!!!!!!!!!!THIS FILE IS AUTOGENERATED OUTSIDE OF THE BUILD PROCESS!!!!!!!!!!!!!!
//

#include <cmath>

#include "gtest/gtest.h"

#include "ngraph/ngraph.hpp"
#include "util/test_tools.hpp"
#include "util/autodiff/numeric_compare.hpp"

using namespace std;
using namespace ngraph;

template<typename T>
static bool all_close(const std::vector<T>& a,
                      const std::vector<T>& b,
                      T rtol = T(1e-4),
                      T atol = T(1e-7))
{
    assert(a.size() == b.size());

    bool rc = true;

    for (size_t i = 0; i < a.size(); ++i)
    {
        if (std::abs(a[i] - b[i]) > atol + rtol * std::abs(b[i]))
        {
            rc = false;
        }
    }
    return rc;
}


TEST (${BACKEND_NAME}, convolution_2d_1item)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,2,2,4};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.30069997668564363291210383977158926427364349365234375f,-0.1735999867632986859433685822295956313610076904296875f,0.642799983334541469304213023860938847064971923828125f,0.6127999761819840074394960538484156131744384765625f,-0.4669000021502374142556845981744118034839630126953125f,-0.1287999965891242215576539820176549255847930908203125f,-0.21679999537467953274472165503539144992828369140625f,0.43640000984668692041168469586409628391265869140625f,-0.5798999691963200309174908397835679352283477783203125f,-0.156800002080201839049777845502831041812896728515625f,-0.3561999741300942989852273967699147760868072509765625f,0.5735999830007549871169203470344655215740203857421875f,0.844099985437094968432347741327248513698577880859375f,-0.44620000281482941062449754099361598491668701171875f,-1.358800014470517680820194073021411895751953125f,-0.160699963200091833215310543891973793506622314453125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_padded_1_1x1_1)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,2,4,6};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{1,1}, // below_pads
                                                                  CoordinateDiff{1,1}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.1825999953389167984596497262828052043914794921875f,0.327299984288215828343027169466949999332427978515625f,0.00450000229477875990369284409098327159881591796875f,0.25089998874664320993588262354023754596710205078125f,-0.0989999968409538411151515902020037174224853515625f,-0.40499997854232816507646930404007434844970703125f,-0.39209999197125444680978034739382565021514892578125f,0.30069997668564363291210383977158926427364349365234375f,-0.1735999867632986859433685822295956313610076904296875f,0.642799983334541469304213023860938847064971923828125f,0.6127999761819840074394960538484156131744384765625f,-0.989999968409538411151515902020037174224853515625f,0.40149999341368669547591707669198513031005859375f,-0.4669000021502374142556845981744118034839630126953125f,-0.1287999965891242215576539820176549255847930908203125f,-0.21679999537467953274472165503539144992828369140625f,0.43640000984668692041168469586409628391265869140625f,-0.335700011777877449503648676909506320953369140625f,-0.1847999932289123847795053734444081783294677734375f,0.7576999795019627725878308410756289958953857421875f,-0.05539999551177032799387234263122081756591796875f,-0.0227000025987624809431508765555918216705322265625f,-0.78289997971653946251535671763122081756591796875f,0.20749999582767486572265625f,0.257299996805191000248669297434389591217041015625f,-0.0153000005826353646654069962096400558948516845703125f,0.126899998052418194749435542689752764999866485595703125f,0.0951999988362193239055386584368534386157989501953125f,-0.26579999512135976669213732748175971210002899169921875f,-0.053999997362494500663387952954508364200592041015625f,-1.128599996709823471974232234060764312744140625f,-0.5798999691963200309174908397835679352283477783203125f,-0.156800002080201839049777845502831041812896728515625f,-0.3561999741300942989852273967699147760868072509765625f,0.5735999830007549871169203470344655215740203857421875f,0.7775999565213924658024780001142062246799468994140625f,1.25290001950263985008859890513122081756591796875f,0.844099985437094968432347741327248513698577880859375f,-0.44620000281482941062449754099361598491668701171875f,-1.358800014470517680820194073021411895751953125f,-0.160699963200091833215310543891973793506622314453125f,0.50100000610202499728984548710286617279052734375f,-0.871200003671646072689327411353588104248046875f,-0.6632999782085420292787603102624416351318359375f,0.3644999949932099525540252216160297393798828125f,1.096199997711181595150264911353588104248046875f,0.5444999723434449379055877216160297393798828125f,-0.2249999940395355224609375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_padded_2_3x4_5)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,2,8,12};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{2,3}, // below_pads
                                                                  CoordinateDiff{4,5}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1825999953389167984596497262828052043914794921875f,0.327299984288215828343027169466949999332427978515625f,0.00450000229477875990369284409098327159881591796875f,0.25089998874664320993588262354023754596710205078125f,-0.0989999968409538411151515902020037174224853515625f,-0.40499997854232816507646930404007434844970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.39209999197125444680978034739382565021514892578125f,0.30069997668564363291210383977158926427364349365234375f,-0.1735999867632986859433685822295956313610076904296875f,0.642799983334541469304213023860938847064971923828125f,0.6127999761819840074394960538484156131744384765625f,-0.989999968409538411151515902020037174224853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.40149999341368669547591707669198513031005859375f,-0.4669000021502374142556845981744118034839630126953125f,-0.1287999965891242215576539820176549255847930908203125f,-0.21679999537467953274472165503539144992828369140625f,0.43640000984668692041168469586409628391265869140625f,-0.335700011777877449503648676909506320953369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1847999932289123847795053734444081783294677734375f,0.7576999795019627725878308410756289958953857421875f,-0.05539999551177032799387234263122081756591796875f,-0.0227000025987624809431508765555918216705322265625f,-0.78289997971653946251535671763122081756591796875f,0.20749999582767486572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.257299996805191000248669297434389591217041015625f,-0.0153000005826353646654069962096400558948516845703125f,0.126899998052418194749435542689752764999866485595703125f,0.0951999988362193239055386584368534386157989501953125f,-0.26579999512135976669213732748175971210002899169921875f,-0.053999997362494500663387952954508364200592041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.128599996709823471974232234060764312744140625f,-0.5798999691963200309174908397835679352283477783203125f,-0.156800002080201839049777845502831041812896728515625f,-0.3561999741300942989852273967699147760868072509765625f,0.5735999830007549871169203470344655215740203857421875f,0.7775999565213924658024780001142062246799468994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.25290001950263985008859890513122081756591796875f,0.844099985437094968432347741327248513698577880859375f,-0.44620000281482941062449754099361598491668701171875f,-1.358800014470517680820194073021411895751953125f,-0.160699963200091833215310543891973793506622314453125f,0.50100000610202499728984548710286617279052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.871200003671646072689327411353588104248046875f,-0.6632999782085420292787603102624416351318359375f,0.3644999949932099525540252216160297393798828125f,1.096199997711181595150264911353588104248046875f,0.5444999723434449379055877216160297393798828125f,-0.2249999940395355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,2,4};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.30069997668564363291210383977158926427364349365234375f,-0.1735999867632986859433685822295956313610076904296875f,0.642799983334541469304213023860938847064971923828125f,0.6127999761819840074394960538484156131744384765625f,-0.4669000021502374142556845981744118034839630126953125f,-0.1287999965891242215576539820176549255847930908203125f,-0.21679999537467953274472165503539144992828369140625f,0.43640000984668692041168469586409628391265869140625f,-0.5798999691963200309174908397835679352283477783203125f,-0.156800002080201839049777845502831041812896728515625f,-0.3561999741300942989852273967699147760868072509765625f,0.5735999830007549871169203470344655215740203857421875f,0.844099985437094968432347741327248513698577880859375f,-0.44620000281482941062449754099361598491668701171875f,-1.358800014470517680820194073021411895751953125f,-0.160699963200091833215310543891973793506622314453125f,-0.553499987512827029689788105315528810024261474609375f,-0.8014999864101408633132450631819665431976318359375f,0.958099981868266947770962360664270818233489990234375f,-0.326000004015862765438527048900141380727291107177734375f,-0.5104999683499340523695764204603619873523712158203125f,-0.071000000774860350105655015795491635799407958984375f,0.08210000851154308776358448085375130176544189453125f,0.76689997238516838251598528586328029632568359375f,1.542800020475685318643854770925827324390411376953125f,0.301299981389939575127101534235407598316669464111328125f,-0.23019997778832912427304790980997495353221893310546875f,0.1088999921515583224618239910341799259185791015625f,0.45619995982497985043124799631186760962009429931640625f,-0.834300012175738903152932834927923977375030517578125f,-0.5861999916389581954945242614485323429107666015625f,0.82200002501904945173549776882282458245754241943359375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items_strided)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,1,2};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,2},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.30069997668564363291210383977158926427364349365234375f,0.642799983334541469304213023860938847064971923828125f,-0.5798999691963200309174908397835679352283477783203125f,-0.3561999741300942989852273967699147760868072509765625f,-0.553499987512827029689788105315528810024261474609375f,0.958099981868266947770962360664270818233489990234375f,1.542800020475685318643854770925827324390411376953125f,-0.23019997778832912427304790980997495353221893310546875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items_strided_padded)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,6,7};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,2},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{4,2}, // below_pads
                                                                  CoordinateDiff{5,7}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.30069997668564363291210383977158926427364349365234375f,0.642799983334541469304213023860938847064971923828125f,-0.989999968409538411151515902020037174224853515625f,0.0f,0.0f,0.0f,0.0f,0.7576999795019627725878308410756289958953857421875f,-0.0227000025987624809431508765555918216705322265625f,0.20749999582767486572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5798999691963200309174908397835679352283477783203125f,-0.3561999741300942989852273967699147760868072509765625f,0.7775999565213924658024780001142062246799468994140625f,0.0f,0.0f,0.0f,0.0f,-0.6632999782085420292787603102624416351318359375f,1.096199997711181595150264911353588104248046875f,-0.2249999940395355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.553499987512827029689788105315528810024261474609375f,0.958099981868266947770962360664270818233489990234375f,-0.3540999903544783666831108348560519516468048095703125f,0.0f,0.0f,0.0f,0.0f,0.0516000023126601536915813994710333645343780517578125f,-0.710699981975555505897546026972122490406036376953125f,0.771899990415573000746007892303168773651123046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.542800020475685318643854770925827324390411376953125f,-0.23019997778832912427304790980997495353221893310546875f,-0.026999999873340119194153885473497211933135986328125f,0.0f,0.0f,0.0f,0.0f,-0.18990000153183927977806888520717620849609375f,0.25919999109506619561216211877763271331787109375f,-0.83699998426437360876661841757595539093017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items_strided_padded_same)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,3,4};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,2},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{2,2}, // below_pads
                                                                  CoordinateDiff{2,2}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.30069997668564363291210383977158926427364349365234375f,0.642799983334541469304213023860938847064971923828125f,-0.989999968409538411151515902020037174224853515625f,0.0f,0.7576999795019627725878308410756289958953857421875f,-0.0227000025987624809431508765555918216705322265625f,0.20749999582767486572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5798999691963200309174908397835679352283477783203125f,-0.3561999741300942989852273967699147760868072509765625f,0.7775999565213924658024780001142062246799468994140625f,0.0f,-0.6632999782085420292787603102624416351318359375f,1.096199997711181595150264911353588104248046875f,-0.2249999940395355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.553499987512827029689788105315528810024261474609375f,0.958099981868266947770962360664270818233489990234375f,-0.3540999903544783666831108348560519516468048095703125f,0.0f,0.0516000023126601536915813994710333645343780517578125f,-0.710699981975555505897546026972122490406036376953125f,0.771899990415573000746007892303168773651123046875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.542800020475685318643854770925827324390411376953125f,-0.23019997778832912427304790980997495353221893310546875f,-0.026999999873340119194153885473497211933135986328125f,0.0f,-0.18990000153183927977806888520717620849609375f,0.25919999109506619561216211877763271331787109375f,-0.83699998426437360876661841757595539093017578125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,1,3};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{2,2},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.93539996541142489405729065765626728534698486328125f,-0.472599983566999615192116834805347025394439697265625f,0.5049999743700031018533991300500929355621337890625f,-1.217199959743023196523381557199172675609588623046875f,-0.309400011985003775549785132170654833316802978515625f,0.54850000736862369876689626835286617279052734375f,-0.7882999953448770469321971177123486995697021484375f,-0.608399997836351236202290237997658550739288330078125f,0.49379999666362994759083449025638401508331298828125f,-0.42060001051574980035496764685376547276973724365234375f,1.364399988515674788658316174405626952648162841796875f,-0.41189997693002222689528935006819665431976318359375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2items_dilated_padded)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,10,12};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{2,2},        // filter_dilation
                                                                  CoordinateDiff{4,2}, // below_pads
                                                                  CoordinateDiff{5,7}, // above_pads
                                                                  Strides{1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1825999953389167984596497262828052043914794921875f,-0.046199998307228096194876343361102044582366943359375f,0.4724999794363977656530551030300557613372802734375f,-0.0460999950706959538138107745908200740814208984375f,0.0044999955892564003079314716160297393798828125f,0.0989999968409538411151515902020037174224853515625f,-0.40499997854232816507646930404007434844970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.21780000091791151817233185283839702606201171875f,0.01319999963343143623006881171022541821002960205078125f,-0.3772999923348425710400988464243710041046142578125f,0.2030000003501772898317767612752504646778106689453125f,0.0206999933004379244039228069595992565155029296875f,0.359999995827674723614109097979962825775146484375f,-0.243000003218650562075708876363933086395263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.01930000684857358095314339152537286281585693359375f,0.0154999984502792731433373774052597582340240478515625f,0.93539996541142489405729065765626728534698486328125f,-0.472599983566999615192116834805347025394439697265625f,0.5049999743700031018533991300500929355621337890625f,-0.2133999920248985660009566345252096652984619140625f,-0.63449996817112008784533827565610408782958984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.207899995511770185885325190611183643341064453125f,-0.012599999324977406711667526906239800155162811279296875f,-0.88679998986124974891254169051535427570343017578125f,-0.1181999993726610131972165618208236992359161376953125f,0.37069999777078610492253574193455278873443603515625f,0.663999996542930404075377737171947956085205078125f,-0.448200008797645210734117426909506320953369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1847999932289123847795053734444081783294677734375f,0.0272999981462955787492319359444081783294677734375f,0.78289997971653946251535671763122081756591796875f,0.0768999993562697881088752183131873607635498046875f,-0.2599999941885471343994140625f,-0.73039998135566719383859890513122081756591796875f,0.20749999582767486572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.257299996805191000248669297434389591217041015625f,-0.06509999846816061364052075077779591083526611328125f,0.1892999952629208504362168241641484200954437255859375f,0.055600000829994666862177155053359456360340118408203125f,-0.2519999960735439525905121627147309482097625732421875f,0.01319999963343143623006881171022541821002960205078125f,-0.053999997362494500663387952954508364200592041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3069000053167343367022112943232059478759765625f,0.0185999997273087469551455797045491635799407958984375f,0.03670000223368408143187480163760483264923095703125f,0.251600005441904084335646984982304275035858154296875f,-0.1488000082120299882859626450226642191410064697265625f,0.0479999996423721153604446953977458178997039794921875f,-0.0324000005632638643504606079659424722194671630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.54889999077320084097664221189916133880615234375f,0.167599996680021234851665212772786617279052734375f,-1.217199959743023196523381557199172675609588623046875f,-0.309400011985003775549785132170654833316802978515625f,0.54850000736862369876689626835286617279052734375f,-0.250799992215633427150578427244909107685089111328125f,0.8249999567493802032913663424551486968994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.98010001888275155579322017729282379150390625f,-0.05939999924451111468215458444319665431976318359375f,0.5840999796628949525256757624447345733642578125f,-0.846000016793608811127569424570538103580474853515625f,0.25560003163814570825707050971686840057373046875f,-0.71999999165534944722821819595992565155029296875f,0.48600000643730112415141775272786617279052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.871200003671646072689327411353588104248046875f,0.128699996519088699642452411353588104248046875f,-0.5444999723434449379055877216160297393798828125f,0.9881999962806702342277276329696178436279296875f,-0.0225000083446502685546875f,0.7919999747276307289212127216160297393798828125f,-0.2249999940395355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.134200002419948560827833716757595539093017578125f,-0.21780000091791151817233185283839702606201171875f,-0.10950000005960447424513404257595539093017578125f,-0.52029999287128436691318711382336914539337158203125f,0.33309999118149280494804997942992486059665679931640625f,-0.1529999975562095215764202293939888477325439453125f,-0.0089999995604157501105646588257513940334320068359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.142999993979930906107256305404007434844970703125f,0.037400000190734861149621792719699442386627197265625f,-0.19129998023510008664516135468147695064544677734375f,0.13369999637007712323111263685859739780426025390625f,0.0419999991655348736685482435859739780426025390625f,0.1169999926090241615384002216160297393798828125f,-0.33749999105930328369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.14789999968409528463553215260617434978485107421875f,0.232099995249509749584149176371283829212188720703125f,-0.7882999953448770469321971177123486995697021484375f,-0.608399997836351236202290237997658550739288330078125f,0.49379999666362994759083449025638401508331298828125f,-0.09320000820159890508875832892954349517822265625f,0.4018999928370117213916046239319257438182830810546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1364999907314778937461596797220408916473388671875f,-0.0356999992609023930612011099583469331264495849609375f,-0.636099968099594459403078872128389775753021240234375f,0.08650000235438337625737403868697583675384521484375f,0.5392999943315981425939753535203635692596435546875f,0.2157999877452851933412603102624416351318359375f,-0.62249998748302459716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.018900000160932517445644407416693866252899169921875f,-0.0230999991536140480974381716805510222911834716796875f,0.2321999965488910078192930086515843868255615234375f,0.003100003176927490500247586169280111789703369140625f,-0.81779998288750643808953100233338773250579833984375f,0.3485999821066858572748969891108572483062744140625f,0.771899990415573000746007892303168773651123046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.18910000588893893791464506648480892181396484375f,-0.3069000053167343367022112943232059478759765625f,0.195900001747906227222983943647705018520355224609375f,-0.16480000116378068497624553856439888477325439453125f,0.0387999990850687037635680098901502788066864013671875f,-0.02039999975860118386350450236932374536991119384765625f,-0.00119999994635582030166887079758453182876110076904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.20149999415874475516829988919198513031005859375f,0.05270000095963478514704547706060111522674560546875f,0.103600005985796439489377007703296840190887451171875f,0.090799997542798493999072206861455924808979034423828125f,-0.20490000190436841176477855697157792747020721435546875f,0.01559999907910825100998408743180334568023681640625f,-0.044999998994171619415283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6318000213027001876753274700604379177093505859375f,1.01420001896023759258014251827262341976165771484375f,-0.42060001051574980035496764685376547276973724365234375f,1.364399988515674788658316174405626952648162841796875f,-0.41189997693002222689528935006819665431976318359375f,0.33119999376237385657617551260045729577541351318359375f,0.0737999983027577410865660567651502788066864013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.643499982595443498212262056767940521240234375f,-0.168300003391504304772752220742404460906982421875f,0.12959995039701510677332407794892787933349609375f,-0.4103999905943869208613250521011650562286376953125f,0.328500010609626968971497262828052043914794921875f,-0.233999985218048323076800443232059478759765625f,0.6749999821186065673828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.08910000439882281852987944148480892181396484375f,-0.108900000458955759086165926419198513031005859375f,0.661500006079673852354972041212022304534912109375f,-0.5147999878644942128858019714243710041046142578125f,-0.2457000338315964427238213829696178436279296875f,-0.377999978184700324845834984444081783294677734375f,-0.83699998426437360876661841757595539093017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_3d_2items)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2,3};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,2,4,6};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1},        // move_strides
                                                                  Strides{1,1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0}, // above_pads
                                                                  Strides{1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.959999953955411822192900217487476766109466552734375f,-2.3211999967426066859843558631837368011474609375f,0.7014000389039514171685141263878904283046722412109375f,2.33690000583827472979692174703814089298248291015625f,0.5572000018760558948116567989927716553211212158203125f,-0.3628999787479638694520645003649406135082244873046875f,-0.43580001616925034824134854716248810291290283203125f,-0.936799995002150165390730762737803161144256591796875f,0.8076999713063239827448569485568441450595855712890625f,0.23889999386519245039295356036745943129062652587890625f,0.0587000115692617452367585428874008357524871826171875f,-2.0543999675422899287013933644630014896392822265625f,-1.306799969632923552609327089157886803150177001953125f,0.7130999959871171522962640665355138480663299560546875f,-0.5407999646484855649219980477937497198581695556640625f,-1.5998000093966726620209328757482580840587615966796875f,-1.4261999794200068691907290485687553882598876953125f,-1.6161000297904006828275669249705970287322998046875f,-1.5903000183150173274526650857296772301197052001953125f,-1.26379999454021429983185953460633754730224609375f,-1.928000048704445301694931913516484200954437255859375f,-0.8300999278366563149944568067439831793308258056640625f,0.1585000311359754388007559100515209138393402099609375f,0.25269995916187770745864327182061970233917236328125f,0.8076999914228915056213509160443209111690521240234375f,0.5184999466836448522855107512441463768482208251953125f,1.152699913564325218118256088928319513797760009765625f,-1.6964999792575827797236343030817806720733642578125f,0.2409999142363663526822392668691463768482208251953125f,0.8176999599814422925447843226720578968524932861328125f,0.4823000367030498836129481787793338298797607421875f,2.520699989724158651682728304876945912837982177734375f,-0.386799979393183779308174052857793867588043212890625f,1.178900024673341651038072086521424353122711181640625f,1.159299983192979777157916032592765986919403076171875f,1.0041999801054595220506371333613060414791107177734375f,2.20250000189989858512262799195013940334320068359375f,-0.28880006621480003303759076516143977642059326171875f,1.758400048053265773972952956683002412319183349609375f,0.031399991555511486129859122229390777647495269775390625f,-1.1566999849110839448940168949775397777557373046875f,-0.7459999538734560786679139710031449794769287109375f,-1.7084000086769464132885332219302654266357421875f,-1.439699967268109670470721539459191262722015380859375f,0.8811999906703829044118947422248311340808868408203125f,-0.733399984797835191585591019247658550739288330078125f,-0.57709998534023687621896669952548108994960784912109375f,0.30950000378489461727582465755403973162174224853515625f,0.557100000837445374912704210146330296993255615234375f,-1.005499978184700271555129802436567842960357666015625f,-1.978999975755810769584286390454508364200592041015625f,-0.182499995231628187042360877967439591884613037109375f,2.0620000173449515301626888685859739780426025390625f,0.5044000061362978737378170990268699824810028076171875f,0.5886000051662325649459717169520445168018341064453125f,-0.62910001900196010904409149588900618255138397216796875f,-0.4466999789506196361088541380013339221477508544921875f,0.87269997626095985321370562814990989863872528076171875f,0.9447999931171537912888425125856883823871612548828125f,-0.490600048588216264278116796049289405345916748046875f,-0.72779998825192449007914774483651854097843170166015625f,-0.821800001558661730172161696827970445156097412109375f,0.460300006468594258279836139990948140621185302734375f,1.1980999917030334955825310316868126392364501953125f,0.173299999889731382030078066236455924808979034423828125f,-0.67330008776932981362506325240246951580047607421875f,1.4415999910995365151933356173685751855373382568359375f,0.26359997510314026047950619613402523100376129150390625f,0.8712999584048979340167306872899644076824188232421875f,-0.4687000136926771087786391944973729550838470458984375f,-1.733599951186775900424663632293231785297393798828125f,-1.93010002592354990014200666337274014949798583984375f,-0.50640001793056743650822681956924498081207275390625f,1.3859000081926584613256636657752096652984619140625f,2.6903000031530854840866595623083412647247314453125f,1.6003999752834443537352626663050614297389984130859375f,-0.3532999783575541119517993138288147747516632080078125f,-1.0304000180497769889598203008063137531280517578125f,-1.289900009131431435349668390699662268161773681640625f,-0.197699950742722396768158432678319513797760009765625f,0.8554999876096849309448089115903712809085845947265625f,-0.02429995238482973718419088982045650482177734375f,-0.6881999777287248765134108907659538090229034423828125f,-0.6599999879300593175202038764837197959423065185546875f,0.26880004945099333912850170236197300255298614501953125f,0.6576999778255820938710485279443673789501190185546875f,-0.260299999912083190878320237970910966396331787109375f,1.5140000333860512693462396782706491649150848388671875f,-0.96089997086524991942724227556027472019195556640625f,-0.6148000079065558676205682786530815064907073974609375f,0.8490000172555440816068994536180980503559112548828125f,-0.7388000526547429114287979246000759303569793701171875f,1.2809999916106462602982674070517532527446746826171875f,1.74730000414401320796287109260447323322296142578125f,-0.383800026018917217474069047966622747480869293212890625f,0.50130002713650501089404087906586937606334686279296875f,0.652500013224780683884773679892532527446746826171875f,0.010799983088671805120384306064806878566741943359375f,2.590399974836408159006850837613455951213836669921875f,0.18859993354752757621639602803043089807033538818359375f,1.077999972745775636440157541073858737945556640625f,-0.5096000508680933460681217184173874557018280029296875f,0.4992999478027229098842099119792692363262176513671875f,-0.5629999824687834664160845932201482355594635009765625f,-1.03429995529055673841867246665060520172119140625f,-1.31119998057484554720986125175841152667999267578125f,-1.2652999061465275421056730920099653303623199462890625f,1.7999999956041572435339048752211965620517730712890625f,-1.815699991624058373673733512987382709980010986328125f,-1.7695000081062317587310417366097681224346160888671875f,0.4333000924631951189525125300860963761806488037109375f,0.8102999776348471794307215532171539962291717529296875f,-0.8667999956354492940846512283314950764179229736328125f,1.4345000278130175530577616882510483264923095703125f,-1.5904999491274356415715374168939888477325439453125f,-1.1945000274032351317288203063071705400943756103515625f,-2.2387999745726592237815566477365791797637939453125f,0.87460000139623794712662174788420088589191436767578125f,-1.769300032404065081692579042282886803150177001953125f,-0.2708999533191314146307604460162110626697540283203125f,0.6918000049114223504176379719865508377552032470703125f,-0.786200074489414202361103889415971934795379638671875f,0.48560004964992398157619390985928475856781005859375f,-0.975800056272000393420285035972483456134796142578125f,-2.132299997270851843467198705184273421764373779296875f,-0.067899993276597125912985575268976390361785888671875f,-2.45459996038079264479847552138380706310272216796875f,3.6036000506892804651215556077659130096435546875f,1.1337999978929758082557555098901502788066864013671875f,2.01449994596838966032237294712103903293609619140625f,-0.41050006639957370424554028431884944438934326171875f,1.4040000705644490874846042061108164489269256591796875f,2.08089997634142687132907667546533048152923583984375f,1.6371999610096212851573227453627623617649078369140625f,1.533100034618378071371580517734400928020477294921875f,-0.22879995561093124933904618956148624420166015625f,0.9493000159755335953803978554788045585155487060546875f,0.5544999909698962170523373060859739780426025390625f,-0.0474000223189599001472060990636236965656280517578125f,0.40570000198036393346257000303012318909168243408203125f,-1.4361000261768690489105892993393354117870330810546875f,-0.38029993524104332980328990743146277964115142822265625f,-0.4722000375643384817436754019581712782382965087890625f,-2.82260000424235979465947821154259145259857177734375f,-1.9813000133529303781187991262413561344146728515625f,-1.0606000475823886830539777292869985103607177734375f,-0.1128999393902716530391217020223848521709442138671875f,1.0285000095665457475746507043368183076381683349609375f,1.2643000225320453200339443355915136635303497314453125f,2.51599999850243261079185685957781970500946044921875f,1.7315999680772422575358859830885194242000579833984375f,-0.9453000246852638621675168906222097575664520263671875f,-0.4708999725788836077100540933315642178058624267578125f,-0.203299998697637984701458435665699653327465057373046875f,-1.205700001570582546861487571732141077518463134765625f,0.3128999616116292514789165579713881015777587890625f,0.9631000168859957799583071391680277884006500244140625f,-0.5962000614508988771689246277674101293087005615234375f,-2.08600003240257425574100125231780111789703369140625f,0.8760000055059802281931524703395552933216094970703125f,-0.72230000757128021149355845409445464611053466796875f,0.08000001639127674035734116841922514140605926513671875f,0.7403999720051881450189057431998662650585174560546875f,-0.2299000232502816931656752785784192383289337158203125f,0.12889998421669002226508382591418921947479248046875f,0.421299994949996037263417747453786432743072509765625f,1.2374000149056316733009452946134842932224273681640625f,1.548000012494624488823546926141716539859771728515625f,-0.6367000078216198399871927904314361512660980224609375f,0.99259996628165148013778207314317114651203155517578125f,0.63610003539696347019116728915832936763763427734375f,0.458800015662610416455891026998870074748992919921875f,-0.146900040464848391508212444023229181766510009765625f,1.60530003109276275807815181906335055828094482421875f,-1.29230005056113039785259388736449182033538818359375f,-0.9017000330045814582291541228187270462512969970703125f,-0.338399952313304108741931486292742192745208740234375f,1.058299996499717732234557843185029923915863037109375f,0.1546999466300007508579028581152670085430145263671875f,0.502700002971291137754406008752994239330291748046875f,-1.3905000122338539991773131987429223954677581787109375f,0.523100030893087097894067483139224350452423095703125f,-0.0502999756604427528117184920120052993297576904296875f,0.8820999762877816952055809451849199831485748291015625f,1.4557000290632249317468449589796364307403564453125f,0.181000043354928752403765201961505226790904998779296875f,-1.327000008665024655130082464893348515033721923828125f,0.0308999641411004943858387150612543337047100067138671875f,2.68339994303733053726546131656505167484283447265625f,0.64899998287111493322498745328630320727825164794921875f,2.16680009226948033784765357268042862415313720703125f,-0.38320002593398061474516680391388945281505584716796875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_2items)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,1,3,5,8,7};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,2,2,4,6,7};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1,1},        // move_strides
                                                                  Strides{1,1,1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0,0}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{-0.191800018620490231313624462927691638469696044921875f,-0.68540000203102746656469435038161464035511016845703125f,0.7844999844878912664825065803597681224346160888671875f,0.878799965371191316165777607238851487636566162109375f,-1.431100053520501180770452265278436243534088134765625f,1.706700035798550008081519990810193121433258056640625f,-1.240800004583597182516996326739899814128875732421875f,0.2708000497341156442843157492461614310741424560546875f,1.4280000424459569074997489224188029766082763671875f,0.01050000984966825701150128224981017410755157470703125f,0.1577000447317959963555722424644045531749725341796875f,-0.84869996880292852825533600480412133038043975830078125f,0.5696000066488984270307582846726290881633758544921875f,-0.0093999686226247103348896416719071567058563232421875f,-0.0609000163510444725289971756865270435810089111328125f,2.24019998099207828801127106999047100543975830078125f,-1.136699961218238019711179731530137360095977783203125f,2.5808999514937394081925958744250237941741943359375f,-0.3882999844297769787004881436587311327457427978515625f,-0.3011000138089061639590227059670723974704742431640625f,0.02580005528479734078217688875156454741954803466796875f,-0.133100029551983300990514180739410221576690673828125f,-0.3276001001313326721486873793764971196651458740234375f,-0.362899993909894913457492293673567473888397216796875f,-3.85979998421370940064889509812928736209869384765625f,0.3916999944105741260358399813412688672542572021484375f,-0.2974999827519064776737423017038963735103607177734375f,-0.967599984274805091644111598725430667400360107421875f,-2.468899961045384117142020841129124164581298828125f,-0.117499995529651235504076112192706204950809478759765625f,-1.55290001030117341684899656684137880802154541015625f,0.9163000625640151941553313008625991642475128173828125f,-1.2186000170126549679849858875968493521213531494140625f,0.6040999289244410874744062311947345733642578125f,-0.0483999957129357394336466313689015805721282958984375f,-1.1471000281065715231676449548103846609592437744140625f,1.389199995472282456177026688237674534320831298828125f,0.0589999999329451352281239451258443295955657958984375f,1.0071000329494477210090508378925733268260955810546875f,1.95720008550882340614407439716160297393798828125f,0.833599952341616212692088083713315427303314208984375f,-0.919299947337062217656011853250674903392791748046875f,1.862999970100819968621408406761474907398223876953125f,-1.434100010955333903694963737507350742816925048828125f,0.09750005958601892785964082577265799045562744140625f,-1.2954999910742042157352216236176900565624237060546875f,-0.39859998419284836668197158360271714627742767333984375f,0.6121000590637326599363632340100593864917755126953125f,1.251399979448317711927529671811498701572418212890625f,0.443100069040060606795350395259447395801544189453125f,0.46240003852397204564539379134657792747020721435546875f,-0.1270000065416099221948798003722913563251495361328125f,-0.257099958369135794811199957621283829212188720703125f,-1.0787999941676857940109357514302246272563934326171875f,0.5838999762073153743102693624678067862987518310546875f,-2.6600000809133046431043112534098327159881591796875f,2.122699989895522332261634801398031413555145263671875f,-0.8201000616118314656688426111941225826740264892578125f,-0.241999982997775031634546394343487918376922607421875f,1.873999965474009599120108759962022304534912109375f,2.467599956148862627713924666750244796276092529296875f,-1.038800002437829217427633921033702790737152099609375f,-0.2788999694272875640166375887929461896419525146484375f,-0.2127000121355051565075200414867140352725982666015625f,0.47350001172721445374946824813378043472766876220703125f,0.5870999630630009402665336892823688685894012451171875f,-1.0248000264212482335324239102192223072052001953125f,-1.0246000705003732722531140097999013960361480712890625f,0.222499985657631349500462647483800537884235382080078125f,-2.75579998837858308746717739268206059932708740234375f,-0.1856999924063684748176683569909073412418365478515625f,0.205500058166682375571099328226409852504730224609375f,0.6941999691531062222082937296363525092601776123046875f,1.0238000386849044343051673422451131045818328857421875f,-0.0264000175952905369314294148352928459644317626953125f,-0.6931000665813680239324412468704394996166229248046875f,-0.95970004158765054302904218275216408073902130126953125f,1.286599947923421805029420283972285687923431396484375f,0.287199983026087790616287520606420002877712249755859375f,1.9868000519245867518947079588542692363262176513671875f,-0.1755000881366435283581495241378434002399444580078125f,0.6585000635832554127802040966344065964221954345703125f,-0.5093999128550290578232306870631873607635498046875f,-0.807899927040934162647545235813595354557037353515625f,-0.97920002845972742289148982308688573539257049560546875f,1.451000041753053526605299339280463755130767822265625f,-0.6624999859184033113024270278401672840118408203125f,1.191999977521598363949806298478506505489349365234375f,2.066600024701654714220921960077248513698577880859375f,-0.18980000865161350542820173359359614551067352294921875f,1.6036999843746431171354061007150448858737945556640625f,0.30630006487369609846638240924221463501453399658203125f,-2.614800008688867638539932158892042934894561767578125f,-0.1104000479638573839480386595823802053928375244140625f,-0.0693000060290092090298230687039904296398162841796875f,-0.136700054574013407648891416101832874119281768798828125f,2.868299980704486440430400762124918401241302490234375f,1.2948999356314541131496298476122319698333740234375f,1.011299993310867950668807679903693497180938720703125f,1.75580003531724226917276610038243234157562255859375f,-0.1134000184744590189467317031812854111194610595703125f,-0.35589996419698100016404396228608675301074981689453125f,0.4546000316828491261134104206576012074947357177734375f,-1.2768000325903285929740604842663742601871490478515625f,-0.06500001396983901003068240243010222911834716796875f,-0.9413999812364572949974217408453114330768585205078125f,-1.407599965573847811128871398977935314178466796875f,-0.61650003599375435836549286250374279916286468505859375f,0.18400000514835124221235673758201301097869873046875f,1.1316999830856915298937792613287456333637237548828125f,-2.35150000600516850823851200402714312076568603515625f,-0.2206000513449304367696868212078697979450225830078125f,1.726300010335445112019669977598823606967926025390625f,-0.32690002569407194954465012415312230587005615234375f,0.2401000499144203548240739110042341053485870361328125f,-0.744900041504204413200795897864736616611480712890625f,0.271900035864114375527833544765599071979522705078125f,-0.8686000395320359057649284295621328055858612060546875f,0.915400081603229498483642601058818399906158447265625f,-0.16889997587203964268809386339853517711162567138671875f,-3.22969994931220849565534081193618476390838623046875f,-0.3341999787643550323679164648638106882572174072265625f,0.134999965615570705068648749147541821002960205078125f,-0.856599959583579817490317509509623050689697265625f,-2.01499997418373855140316663892008364200592041015625f,0.04730003482177813634024232669617049396038055419921875f,0.83029997383505127483971364199533127248287200927734375f,-1.6004000528812405423906284340773709118366241455078125f,-0.2129999799355870582218130948604084551334381103515625f,-0.93970000101924000546205206774175167083740234375f,-2.01330000961273913873128549312241375446319580078125f,-1.0658000012680888257676770081161521375179290771484375f,0.93749999590218136003016979884705506265163421630859375f,-0.1065000112950802435562991377082653343677520751953125f,-0.6895999917849902072219947513076476752758026123046875f,0.888300011959672008998722958494909107685089111328125f,1.4116000463455922186994939693249762058258056640625f,0.8445000114962459836220887154922820627689361572265625f,0.1902000464081761155199501445167697966098785400390625f,-0.398099951320887601724507476319558918476104736328125f,-0.5226000518515707682354332064278423786163330078125f,-1.6451000569179650145912319203489460051059722900390625f,1.16359998784363227741778246127068996429443359375f,0.04500003751367198656652135468902997672557830810546875f,-1.4982000486955049378678950233734212815761566162109375f,0.5730999680288133557581886634579859673976898193359375f,-1.881200023452937575285659477231092751026153564453125f,0.22100000006705589061795080851879902184009552001953125f,1.65020004849433821192405957845039665699005126953125f,-0.1232999475866558558578844895237125456333160400390625f,-0.097200054439902927594374659747700206935405731201171875f,0.0518000058427456888665574297192506492137908935546875f,0.1337999908521771974534431137726642191410064697265625f,-2.945699965398014708028995300992392003536224365234375f,-1.0005000566393140726262345197028480470180511474609375f,0.387299980302155066880231970571912825107574462890625f,1.21179994623809950127224510652013123035430908203125f,0.410800030325353393578780014649964869022369384765625f,0.41999998379498737488546566964942030608654022216796875f,-0.1398999989613891603568163191084749996662139892578125f,-1.6042999939218152150033347425051033496856689453125f,-2.39579998327493459697734579094685614109039306640625f,-0.1260000119507314053635127493180334568023681640625f,-1.741500009581447017126265564002096652984619140625f,1.064199984240530749701747481594793498516082763671875f,0.52420001460164922235662743332795798778533935546875f,1.2580000196658078781553058433928526937961578369140625f,-0.888399953132868791527698704157955944538116455078125f,-0.21360004498958584662204884807579219341278076171875f,1.29139995750635794280469781369902193546295166015625f,-1.2492000053256753222541419745539315044879913330078125f,1.2299000107333053488645191464456729590892791748046875f,0.9435000068843368836724039283581078052520751953125f,-0.75859995006918989890465354619664140045642852783203125f,1.71460004855841408044625495676882565021514892578125f,-0.433100026124716208641984849236905574798583984375f,0.08589991822689813094626742895343340933322906494140625f,-0.930400000391900849905368886538781225681304931640625f,-0.50990007629841482827970366997760720551013946533203125f,0.946500059127807435999102381174452602863311767578125f,-1.0864999965056778563621264765970408916473388671875f,3.508199972587823456393607557401992380619049072265625f,-1.4846000054940571288852879661135375499725341796875f,-1.2775999582722785685717781234416179358959197998046875f,-1.997300032828749127844503163942135870456695556640625f,-1.3494999788999562095881401546648703515529632568359375f,-1.2136000030800697668809107199194841086864471435546875f,0.46840000364780376429507668945007026195526123046875f,-0.27930004022717458411761981551535427570343017578125f,-0.42579996092319571943107803235761821269989013671875f,1.51709996712356787185171924647875130176544189453125f,0.088200059312581569059830144396983087062835693359375f,-0.50250003878027216241974883814691565930843353271484375f,-0.08219990494400253311368942377157509326934814453125f,0.06690001723766414443161920644342899322509765625f,0.660599987977743552391984849236905574798583984375f,0.2867999873280531186736652671243064105510711669921875f,-0.22570002151951218394287934643216431140899658203125f,1.0938999792248005338279881470953114330768585205078125f,0.945700027126073816674534100457094609737396240234375f,0.1711000109404319236006131177418865263462066650390625f,-2.682399951836467177912481929524801671504974365234375f,1.66469998989924761190195567905902862548828125f,1.030599954599141998556888211169280111789703369140625f,-1.809800016493349428259307387634180486202239990234375f,-2.055200001890957306471818810678087174892425537109375f,0.18950006253272277945143287070095539093017578125f,-1.500399995064734870453548865043558180332183837890625f,0.4910000443235045253231874085031449794769287109375f,-0.04560006006211059226274073807871900498867034912109375f,1.5068000261828304875422190889366902410984039306640625f,0.94859998765736819326122031270642764866352081298828125f,-1.502699962607770611811019989545457065105438232421875f,-1.098999994773418364957251469604671001434326171875f,0.20360001053065091003446696049650199711322784423828125f,-0.4433999948561193615859110650490038096904754638671875f,0.44360003072172526916716606137924827635288238525390625f,0.78090001366883488032044624560512602329254150390625f,-0.66969999725669671075678479610360227525234222412109375f,-0.092599984665959922391920144946197979152202606201171875f,1.70949994425475626513843963039107620716094970703125f,-0.9790000120773909575433435747982002794742584228515625f,-0.35630003961622735975112163941957987844944000244140625f,-0.26939996702075064138881543840398080646991729736328125f,1.2374999660998582395876610462437383830547332763671875f,0.119700001615286044653885255684144794940948486328125f,1.281399947373568881658911777776665985584259033203125f,-1.986699939574301243538911876385100185871124267578125f,0.2319000006601215080337397012044675648212432861328125f,-1.6597999818295239293775011901743710041046142578125f,-1.693200003209710313711866547237150371074676513671875f,-0.91219999084919844989371995325200259685516357421875f,-0.26059993664324287987454908943618647754192352294921875f,1.1214999360069644041004721657373011112213134765625f,1.608799984072148081537534380913712084293365478515625f,-1.3398999778389921733179335205932147800922393798828125f,1.180200013253093427323392461403273046016693115234375f,0.05230003986954745442972125601954758167266845703125f,0.375799980406463729476485013947240076959133148193359375f,1.2523000068634744241791167951305396854877471923828125f,-1.314800028395653352930594337522052228450775146484375f,1.716300045017898678878509599599055945873260498046875f,-0.842500014528633212052000089897774159908294677734375f,1.81890007213354021331497278879396617412567138671875f,-0.703100032234192706681596973794512450695037841796875f,-0.63089991096258246638939226613729260861873626708984375f,0.595099946947395874730091236415319144725799560546875f,-0.7143000268533816754512599800364114344120025634765625f,-1.6531000447139145848041152930818498134613037109375f,-1.29270003448724679628867306746542453765869140625f,0.8440999494507914757690514306887052953243255615234375f,-1.964899987450241969355602122959680855274200439453125f,1.4341000071927905157309623973560519516468048095703125f,0.7805999987065794787355343942181207239627838134765625f,-1.3518999869883054909536213017418049275875091552734375f,-1.717700022715330288036739148083142936229705810546875f,-1.910400008326768261923689351533539593219757080078125f,-0.1465000200495121251975660925381816923618316650390625f,-2.19530001289472043168871095986105501651763916015625f,0.748600028412044427028604332008399069309234619140625f,0.4400000260025256348939137751585803925991058349609375f,1.5346000200226896925670416749198921024799346923828125f,-0.543199957723915272111980812042020261287689208984375f,-0.16489999169707292292486044971155934035778045654296875f,-1.988999954223633626071432445314712822437286376953125f,3.021399993529915395384932708111591637134552001953125f,-0.96819999412000168348413353669457137584686279296875f,1.618700036007166165319404171896167099475860595703125f,0.5180999983713032719379043555818498134613037109375f,-1.453799974833428709786176113993860781192779541015625f,-0.0994999610930686895216012999298982322216033935546875f,2.00509999887794254647133129765279591083526611328125f,0.806600033791363468793633728637360036373138427734375f,0.627800007697940198880814932635985314846038818359375f,1.6334000469356764195794085026136599481105804443359375f,1.4027000196233387896427302621304988861083984375f,0.25909999009370709988075986984767951071262359619140625f,0.10429994821250476721985478434362448751926422119140625f,0.4343999306246641989304180242470465600490570068359375f,1.9012999374873935654051138044451363384723663330078125f,2.52099997957795896041943706222809851169586181640625f,1.622299952623247776273274212144315242767333984375f,-0.2713000088825829880789797243778593838214874267578125f,-1.6198000213548537917773728622705675661563873291015625f,1.186600025856494600162704955437220633029937744140625f,-1.87769998715743380302001241943798959255218505859375f,1.1205999763548373238108979421667754650115966796875f,0.9501999383375043795041392513667233288288116455078125f,-0.1247999947935334308368737765704281628131866455078125f,0.3550000508502122453791116640786640346050262451171875f,1.03600008222833306348320547840557992458343505859375f,0.524899909815192078355039484449662268161773681640625f,-0.252400003804266415130541645339690148830413818359375f,0.211399963336438734273770023719407618045806884765625f,-0.3128999436184767279911511650425381958484649658203125f,-2.4560000388845804053516985732130706310272216796875f,-0.669099949227273338436816629837267100811004638671875f,0.3726999426960941885766942505142651498317718505859375f,-0.7040000151321288424099975600256584584712982177734375f,-0.427700064401328017282821747357957065105438232421875f,-1.4811999568819997907809238313348032534122467041015625f,-1.89070003122389262983915614313445985317230224609375f,-0.3012999150052662056253893752000294625759124755859375f,0.74119998642355133799952682238654233515262603759765625f,-1.0877000231623645820633328185067512094974517822265625f,1.24360003794878704042048411793075501918792724609375f,-2.0924999758228661761449984624050557613372802734375f,1.35849995674192935979363028309307992458343505859375f,-0.960699997733533539445716087357141077518463134765625f,0.85380001864284249535330673097632825374603271484375f,-2.271500004515051074349685222841799259185791015625f,-0.5887000171944489768094399551046080887317657470703125f,-0.76850001604855133763294361415319144725799560546875f,1.69900000799819839158999457140453159809112548828125f,-0.625599998185037975417799316346645355224609375f,2.0670000231564049641974634141661226749420166015625f,0.0016999503776435886948092957027256488800048828125f,-0.380199969145655902735825293348170816898345947265625f,0.1287000212177635827259791767573915421962738037109375f,1.2513000283285984348680131006403826177120208740234375f,-0.5999999832734468352413159664138220250606536865234375f,0.06930001098364557687858678036718629300594329833984375f,-0.4374999460950494967192980766412802040576934814453125f,-1.888999992668628724601376234204508364200592041015625f,-1.700300025616585219978560417075641453266143798828125f,0.14409995157420618649979360270663164556026458740234375f,-0.1476999585658316238578890988719649612903594970703125f,2.124200044944136944735646466142497956752777099609375f,0.34480001269355409210248808449250645935535430908203125f,1.068899954824148235132952322601340711116790771484375f,-0.54920002320706817755535666947253048419952392578125f,0.2339000896796592510185064384131692349910736083984375f,-1.047900017882138712366213439963757991790771484375f,1.2278999937236305672882963335723616182804107666015625f,-1.6014000620938844132723488655756227672100067138671875f,2.24319997132122495031580911017954349517822265625f,-0.0409000478297472103150766997714526951313018798828125f,0.2332999599918725852631951056537218391895294189453125f,0.5542000319242477157644088947563432157039642333984375f,0.2194000032678247169570795449544675648212432861328125f,1.4159999839738006688349969408591277897357940673828125f,0.4124000146448618675520947363111190497875213623046875f,-0.26160001043379332852367724626674316823482513427734375f,-0.083300013189017807491154599119909107685089111328125f,-0.265499954193831211313181484001688659191131591796875f,1.7978000055931515621665539583773352205753326416015625f,-1.4143999840825796088239485470694489777088165283203125f,0.13089993376135822256145502251456491649150848388671875f,1.292400026826560743842264855629764497280120849609375f,-1.292700027148426134004921550513245165348052978515625f,1.054499999985099467636473491438664495944976806640625f,-0.1283999627068634907800515065900981426239013671875f,-0.277600015641749298112017640960402786731719970703125f,2.49370001589059864244291020440869033336639404296875f,0.6065000144243242186092857082257978618144989013671875f,0.798300012406706560597058341954834759235382080078125f,1.513600026102364548563627977273426949977874755859375f,0.35780001939535110988543920029769651591777801513671875f,0.78769999291300774046931110206060111522674560546875f,-1.258300051112472228709293631254695355892181396484375f,1.2274999874085195106232504258514381945133209228515625f,1.309300026369095260037056505098007619380950927734375f,-0.309599994392692590139404273941181600093841552734375f,-0.44309997572153803080396983204991556704044342041015625f,-0.795200004163384210897902448778040707111358642578125f,-1.0591000103779142360593823468661867082118988037109375f,-1.12379997319430113833504947251640260219573974609375f,-1.6725999743282786713649556986638344824314117431640625f,0.4037999606028199739427009262726642191410064697265625f,0.39419998975396086660083483366179279983043670654296875f,-0.0106000435031952111586406317655928432941436767578125f,-1.04099998229742052302526644780300557613372802734375f,1.2348999661415807072017969403532333672046661376953125f,-1.614399984045327141046755059505812823772430419921875f,-2.245100003571807167901397406239993870258331298828125f,-1.051699945795535828807487632730044424533843994140625f,1.658600006879865862430278866668231785297393798828125f,-0.563999959364533509642569697462022304534912109375f,0.11130002132505223411129691157839260995388031005859375f,1.3098999759763481076646485234959982335567474365234375f,0.83979999017417428230913856168626807630062103271484375f,0.9138999839931731816733417872455902397632598876953125f,-0.376500018447637696539231910719536244869232177734375f,-0.5862999925658105571102396424976177513599395751953125f,0.4399000512272126517387960120686329901218414306640625f,-0.5233999737709751354231002551387064158916473388671875f,-1.4822999855920671041786818022956140339374542236328125f,-0.1653999961450700606491182043100707232952117919921875f,-2.45200000199675560708101329510100185871124267578125f,-1.30960001406222570352611000998876988887786865234375f,1.7626000565826895893906112178228795528411865234375f,1.908999919988215054900138056837022304534912109375f,-0.38829995526075411493849287580815143883228302001953125f,1.790499977663158492902084617526270449161529541015625f,-2.88039997904598710221080182236619293689727783203125f,1.68739998555034365068650004104711115360260009765625f,0.0405000406205651142244050788576714694499969482421875f,1.758199991177022969424115217407234013080596923828125f,1.4859000127002592872571540283388458192348480224609375f,0.485399962840974696920426367796608246862888336181640625f,0.405199934500455327679446781985461711883544921875f,-0.9395999556496741345057444050326012074947357177734375f,0.4541999927714466256389869158738292753696441650390625f,2.395099982039629704644312369055114686489105224609375f,-1.5030999445408588233163982295081950724124908447265625f,-0.28750001776963440835999108458054251968860626220703125f,-0.717399991808831405393220848054625093936920166015625f,0.097900008773804170214560826934757642447948455810546875f,-0.76219995579421517817308995290659368038177490234375f,-1.0226000013738871796675766745465807616710662841796875f,0.56949998709559412901626274106092751026153564453125f,0.666599967518448632830541100702248513698577880859375f,0.64409996438920469330469131818972527980804443359375f,-0.34539997226595897661383105514687485992908477783203125f,0.6621000331729651744439024696475826203823089599609375f,-1.775399983814358595424209852353669703006744384765625f,0.62950001740082994761138479589135386049747467041015625f,-1.5385999585628507002610376730444841086864471435546875f,-1.698800006759167313674652177724055945873260498046875f,-0.355400030939280409114644498913548886775970458984375f,0.14819996565878312821951112709939479827880859375f,-0.01850000319629885314753892089356668293476104736328125f,0.2821000520229339780797772618825547397136688232421875f,-0.6284999940320847855446118046529591083526611328125f,0.07030002997517625917822670089663006365299224853515625f,-0.480600026310980421584417854319326579570770263671875f,1.22479997385740180249058539629913866519927978515625f,-1.0682000417664649916105190641246736049652099609375f,-0.7264000186011194681867664257879368960857391357421875f,-2.161500048920512284666983759962022304534912109375f,0.20359996172934768932094584670267067849636077880859375f,1.77200003552436857745533416164107620716094970703125f,1.4026000196650618700999757493264041841030120849609375f,0.61790001700669439088642320712096989154815673828125f,1.667800047819316677788492597755976021289825439453125f,-1.354800052274763633164411658071912825107574462890625f,-0.42500002857297614067277891081175766885280609130859375f,0.98969998228102940007744336980977095663547515869140625f,-0.3964000338375568421867001234204508364200592041015625f,0.49140000932514649178983745514415204524993896484375f,1.1871000255361201514148206115351058542728424072265625f,-0.6677999920517201370984139430220238864421844482421875f,-0.3089999997466803538515023319632746279239654541015625f,-1.4727000269994146020025027610245160758495330810546875f,0.301799992096424551935029967353329993784427642822265625f,0.8328999944686883427635848420322872698307037353515625f,-0.22180002569854362359791366543504409492015838623046875f,-0.07610001038163949349524273202405311167240142822265625f,-0.81229993422031465666321992102894000709056854248046875f,0.28090004589259631728737076628021895885467529296875f,-0.22610004428178032132024100064882077276706695556640625f,0.30109999138265763907185146308620460331439971923828125f,1.00350006399303737225636723451316356658935546875f,0.500999958008527368491513698245398700237274169921875f,-1.09169998677372870332646925817243754863739013671875f,-0.53469999416470515019028653114219196140766143798828125f,-0.539900097830593228565021490794606506824493408203125f,-0.7160000097528100848620624674367718398571014404296875f,1.900200029364973897116897205705754458904266357421875f,-1.73190004614591597231765263131819665431976318359375f,-1.242400002239644862811474013142287731170654296875f,-1.49499999105930303500144873396493494510650634765625f,0.59289997144639461357229492932674475014209747314453125f,0.6932999468043437030218001382309012115001678466796875f,0.07309999019429069821995881284237839281558990478515625f,-0.02499999247491391241027258729445748031139373779296875f,1.8962000174552200437716464875848032534122467041015625f,-0.1033000008583073725532130993087776005268096923828125f,-0.15140002309009392700289708955097012221813201904296875f,1.70690005415529011401076786569319665431976318359375f,-0.8205000382736329367361349795828573405742645263671875f,-1.445500020634382654094451936543919146060943603515625f,-0.27430002253204566198263592013972811400890350341796875f,-0.38320000514686169967859541429788805544376373291015625f,-0.855400044201314546654657533508725464344024658203125f,1.384700019980967145016848007799126207828521728515625f,0.751999986909329454221051491913385689258575439453125f,-1.40800004859268756973733616177923977375030517578125f,1.30279997886419263153356951079331338405609130859375f,0.54710006051659598735881218090071342885494232177734375f,-0.13950001894682706460315557706053368747234344482421875f,0.66639995732009416062879836317733861505985260009765625f,2.720299984079599386888048684340901672840118408203125f,-1.1589000464290382108600852006929926574230194091796875f,0.710100056172907390106274760910309851169586181640625f,0.13439995774477775380972843777271918952465057373046875f,-0.44729995087236151807275064129498787224292755126953125f,-0.0397000115618106885762017554952763020992279052734375f,0.524300006252527595052015385590493679046630859375f,0.157300004516542468735451620887033641338348388671875f,2.504100010657311248252199220587499439716339111328125f,1.3392000307321556551443109128740616142749786376953125f,2.3754000671118493670519455918110907077789306640625f,-0.966200041744112780861541978083550930023193359375f,0.596899980655312134558698744513094425201416015625f,-0.800600015246868412788217028719373047351837158203125f,0.2892999537259346443107688173768110573291778564453125f,-0.69870003347396814064040881930850446224212646484375f,0.45350001593679223343968942572246305644512176513671875f,0.77680005691647568966118342359550297260284423828125f,2.744800045066326799059197583119384944438934326171875f,-1.461700005772709420881483310949988663196563720703125f,-0.9202000218212604831791168180643580853939056396484375f,-2.229199959784001094931227271445095539093017578125f,2.0647000296294688581610898836515843868255615234375f,0.65240002496391469311021182875265367329120635986328125f,1.389800002821534263830471900291740894317626953125f,0.225100074128805782702755777791026048362255096435546875f,-2.062200018416345148608570525539107620716094970703125f,-0.12870000892430499828122947292285971343517303466796875f,-1.1174000169545410532379037249484099447727203369140625f,1.0163000464335081307609698342275805771350860595703125f,1.30660000141859011790756994741968810558319091796875f,0.1971000077478581857803874299861490726470947265625f,1.968199979386478393195147873484529554843902587890625f,-1.391599956901371282214086022577248513698577880859375f,-0.96350001761317238813120411577983759343624114990234375f,0.12790007202923281415252176884678192436695098876953125f,0.079999938458203967428516989457421004772186279296875f,-0.183800008137524617524150016834028065204620361328125f,-0.7455000212490556776856465148739516735076904296875f,-1.044299983789027663760862196795642375946044921875f,1.70129996991604581779711224953643977642059326171875f,-0.7855000211000440213382489673676900565624237060546875f,0.6718000214517119150769985935767181217670440673828125f,-1.0093000101268290169542751755216158926486968994140625f,-0.9030999587342147805202330346219241619110107421875f,-1.658700018200277792601582405040971934795379638671875f,0.876399976766109478631960882921703159809112548828125f,-1.0165000066012142010407615089206956326961517333984375f,1.1091999741822482494768564720288850367069244384765625f,-1.1002999886617070846028809683048166334629058837890625f,-1.3233000220179553796384652741835452616214752197265625f,3.02990001209303816409601495251990854740142822265625f,0.693800072957575064691582156228832900524139404296875f,1.372199988092482225710000420804135501384735107421875f,1.2734000455200682466738726361654698848724365234375f,0.41669998535811891837710163599695079028606414794921875f,1.685400013858824319612494946341030299663543701171875f,1.5806000139057625464289458250277675688266754150390625f,-0.583800007355212979831549091613851487636566162109375f,-0.020500001616775787027791011496447026729583740234375f,-0.0002000495001668323169496943592093884944915771484375f,-0.8946000053822988906659929853049106895923614501953125f,0.4294000021502382136162623282871209084987640380859375f,-0.45089995432496088323404137554462067782878875732421875f,0.96900002671778207030683915945701301097869873046875f,1.84000001393258560966614822973497211933135986328125f,0.7320000164508828621734437547274865210056304931640625f,0.600800015237927365063796969479881227016448974609375f,-0.202900016596913201905039159100851975381374359130859375f,-0.1825999822258947968833808772615157067775726318359375f,-1.7872000426307319553842489767703227698802947998046875f,0.4637000542610885123195885171298868954181671142578125f,0.288399977570772236834528712279279716312885284423828125f,-1.425600024727731618412462921696715056896209716796875f,1.440300008368491813115497279795818030834197998046875f,1.0903999702915549985249299425049684941768646240234375f,0.34750000320374951900959104023058898746967315673828125f,2.398699944979697296076892598648555576801300048828125f,-0.53059999673217494997601306749857030808925628662109375f,0.3686999748378989583130760365747846662998199462890625f,-1.00680000055283347393242365797050297260284423828125f,-1.687500006295740373474245643592439591884613037109375f,0.6797000077620147795443017457728274166584014892578125f,-0.9758000246629119356356341086211614310741424560546875f,1.566300014358759096211315409163944423198699951171875f,-1.1896999738618732056494309290428645908832550048828125f,-0.525799967852234484411155790439806878566741943359375f,-0.15809999833405063984770322349504567682743072509765625f,0.00229998365491634881863092232379131019115447998046875f,0.52709996153563276610753973727696575224399566650390625f,-0.67500007331371325580704478852567262947559356689453125f,0.4146000107839709070134404100826941430568695068359375f,-0.02960002032071373800903302253573201596736907958984375f,-0.049599959896504675072037571226246654987335205078125f,1.806200012277066502264233349706046283245086669921875f,-0.525000016354024712228465432417578995227813720703125f,1.2157000375196338470829005018458701670169830322265625f,-1.158499930553138046462890997645445168018341064453125f,2.265400037793815268827302134013734757900238037109375f,-0.1583000264883034358120994511409662663936614990234375f,-1.3190999975815416167534976921160705387592315673828125f,-0.9604000529930001306411213590763509273529052734375f,1.6856000261619683566749472447554580867290496826171875f,-1.4431000150978565699233513441868126392364501953125f,1.18440005918443258536854045814834535121917724609375f,-0.5481000397779050015145685392781160771846771240234375f,0.5829000307902696409456666515325196087360382080078125f,0.8843999672815205936871052472270093858242034912109375f,-1.165700034539402185629342056927271187305450439453125f,-0.72480001125931770022958744448260404169559478759765625f,0.42719998716115947789973006365471519529819488525390625f,2.22359998404383674142081872560083866119384765625f,-1.3857999761596320187351238928386010229587554931640625f,0.8540000248551378181360860253334976732730865478515625f,-1.0638999763190752911867775765131227672100067138671875f,-0.7677999683216214510395047909696586430072784423828125f,0.702299937610328495196654330356977880001068115234375f,0.8285999610215422084280589842819608747959136962890625f,-0.15490000127106906546714526484720408916473388671875f,-0.04459996887445516478010176797397434711456298828125f,-1.1314000469505784973733852893928997218608856201171875f,1.077800008890033467423563706688582897186279296875f,-1.0139000083193183687768623713054694235324859619140625f,0.532999987386167273228920748806558549404144287109375f,-0.71339998654872172068053259863518178462982177734375f,-1.1639000004217023853669843447278253734111785888671875f,-0.471399968607723263147590841981582343578338623046875f,0.18189997659474588243000425791251473128795623779296875f,0.08500008113682344657036082935519516468048095703125f,1.218999969087541490608828098629601299762725830078125f,-0.65799997647106678311246241719345562160015106201171875f,0.38680002283006909014062557616853155195713043212890625f,0.6315000072345136050699920815532095730304718017578125f,0.42600005147606179178865204448811709880828857421875f,0.146100035303831254918094373351777903735637664794921875f,-0.435599926212429711114282326889224350452423095703125f,1.83730003178566647648040088824927806854248046875f,-0.296300032947957647788683743783622048795223236083984375f,-0.4656999582082033128926923382095992565155029296875f,0.918099985984712763098514187731780111789703369140625f,0.9537000310897827404232884873636066913604736328125f,-1.293699934306741017309150265646167099475860595703125f,0.73209997345656219192022717834333889186382293701171875f,-0.24389998894780851035335444976226426661014556884765625f,-0.2282999671258021390229941971483640372753143310546875f,-0.37529996172785773378421936286031268537044525146484375f,1.7296999980203811020373905193991959095001220703125f,0.36519995461702325911801381153054535388946533203125f,-0.093199960070848242299490493678604252636432647705078125f,-0.3014000083938241925096690465579740703105926513671875f,-0.154300024059414564447934026247821748256683349609375f,0.2659000100925563714326926856301724910736083984375f,-0.21350000684708358988928011967800557613372802734375f,2.31209999778270702819327198085375130176544189453125f,-0.9598999717481431215304610304883681237697601318359375f,-0.1082000295661394162483048830836196430027484893798828125f,-1.5553000584736460520929313133819960057735443115234375f,0.5827000191017983610208830214105546474456787109375f,-0.3952999914184207597145359613932669162750244140625f,-0.5504999901428819786275425940402783453464508056640625f,-1.5195999847441914454293510061688721179962158203125f,-0.491099986651539754944906235323287546634674072265625f,-0.7066000509276992858076482661999762058258056640625f,0.385800034274161429692640012945048511028289794921875f,-0.65660002928376137720789529339526779949665069580078125f,-1.1033000711172828989248273501289077103137969970703125f,-1.57809999509304788034569355659186840057373046875f,-0.8075999734900893844979918867466039955615997314453125f,-1.2794000199198729550431608004146255552768707275390625f,-0.659599994243681209127316833473742008209228515625f,2.322700040820240729999568429775536060333251953125f,0.0454999781474469955355743877589702606201171875f,-0.850100028829277487574245242285542190074920654296875f,-0.31979999471902786911670091285486705601215362548828125f,0.30719994456246524361375804801355116069316864013671875f,0.0388999924682076336779346092953346669673919677734375f,0.664099996240437473460360706667415797710418701171875f,-0.1940999729573731347187504070461727678775787353515625f,-0.54420001601725831452682768940576352179050445556640625f,1.195499984033406182248882032581605017185211181640625f,-1.2768999648600816154697668025619350373744964599609375f,0.26520003605187003614673812990076839923858642578125f,-0.014999978505074895718252037113416008651256561279296875f,1.85560005536824501604087345185689628124237060546875f,1.1135999786794190669070303556509315967559814453125f,-2.85199999771267176384981212322600185871124267578125f,0.8666999810926612202166552378912456333637237548828125f,-0.33140000817030623725401028423220850527286529541015625f,2.389900013042986159916836186312139034271240234375f,0.969499991994350995838658491265960037708282470703125f,-0.8573999568283550143377169661107473075389862060546875f,0.027899990557134657365168095566332340240478515625f,-0.2303999932393432370503205675049684941768646240234375f,-0.4350000448524948826189984174561686813831329345703125f,-2.4354999788552529338403473957441747188568115234375f,-0.331199948686361267391475848853588104248046875f,-1.9167999512672426742909692620742134749889373779296875f,1.5862000532552595899460357031784951686859130859375f,-0.37590004350840988678328358219005167484283447265625f,-0.21839999172687540873738498703460209071636199951171875f,0.47040004937946822138883362640626728534698486328125f,-2.02999993819743362877261461107991635799407958984375f,0.606599983611701976116137302597053349018096923828125f,1.01550008562207150220046969479881227016448974609375f,-0.6156000389397149330505953912506811320781707763671875f,-0.93499996904283799725732251317822374403476715087890625f,-0.50040005409196053332010478698066435754299163818359375f,-1.3855999887228016564932886467431671917438507080078125f,-1.5986999702744191242942406461224891245365142822265625f,2.70889997322708442339944667764939367771148681640625f,-1.79130006774216976594971129088662564754486083984375f,-0.050199939082563815873072599060833454132080078125f,0.1328999376952648692196135016274638473987579345703125f,0.13610000639557873203244753312901593744754791259765625f,1.0213000287011266831171951707801781594753265380859375f,-1.6754999776817853796018198409001342952251434326171875f,-0.17020002018213364891607852769084274768829345703125f,-0.68330004926845422374981353641487658023834228515625f,-0.2463000319421293493604707691702060401439666748046875f,0.3498999846562742543909507730859331786632537841796875f,1.3786999189957980238574464237899519503116607666015625f,-1.4210999730542306362934823482646606862545013427734375f,0.7704999976493420721368465819978155195713043212890625f,0.1548999638319019656051978017785586416721343994140625f,-2.62530000561177789819566896767355501651763916015625f,-1.8691999658003453088639389534364454448223114013671875f,-1.5804999996051185551237949766800738871097564697265625f,-1.8893000221595170895483306594542227685451507568359375f,0.50190004581063940580776261413120664656162261962890625f,-2.858799959336220997130340037983842194080352783203125f,-0.649299980473518001389265918987803161144256591796875f,1.4601999901190396702332918721367605030536651611328125f,1.32609998512417082139336343971081078052520751953125f,-1.995899978613854042208686223602853715419769287109375f,2.06670000914409701664453677949495613574981689453125f,0.545199977453053374887304016738198697566986083984375f,-1.626799949665368227869066686253063380718231201171875f,-2.35369997316151735589073723531328141689300537109375f,-0.1758999486625185681987204588949680328369140625f,-1.923500041902065138543775901780463755130767822265625f,-1.8876999811597163780874097938067279756069183349609375f,-0.464599998676776149153511141776107251644134521484375f,-2.24740002954602235973879942321218550205230712890625f,1.2276999512642621237290541102993302047252655029296875f,-0.253499982781708599333114761975593864917755126953125f,-0.76060004050433605016223737038671970367431640625f,0.98660003475993907073871014290489256381988525390625f,-0.934699922862649135169021974434144794940948486328125f,0.3426000117301928327151472331024706363677978515625f,1.24290004952848054387004594900645315647125244140625f,1.6956999704644086790494839078746736049652099609375f,-0.25609993613660286815303379626129753887653350830078125f,-0.1383000049188733981964105623774230480194091796875f,-0.7556000114470713402425872118328697979450225830078125f,0.34649997232854345963204423242132179439067840576171875f,-0.678799956765770762245892910868860781192779541015625f,1.55630000642389010323540787794627249240875244140625f,0.8583999811097982046703691594302654266357421875f,1.99960003894716376038331873132847249507904052734375f,-0.054199983011185448589230873039923608303070068359375f,1.1059999760016798742157106971717439591884613037109375f,-2.09930001701861623786271593417041003704071044921875f,0.8254999526292083888989736806252039968967437744140625f,-0.03600006723403925690263349679298698902130126953125f,-1.5827999604985125703393578078248538076877593994140625f,-0.9915000374093647650397542747668921947479248046875f,0.9045000202134245714802318616420961916446685791015625f,-0.488200035209954474879623376182280480861663818359375f,-0.829799993638694832753799346392042934894561767578125f,-0.53359999834895166515025266562588512897491455078125f,-1.2785999859437346959367687304620631039142608642578125f,3.93079997779876055830072800745256245136260986328125f,0.40030003068298081192466497668647207319736480712890625f,-0.1157000043086713336748516667285002768039703369140625f,0.058999985087663020522086299024522304534912109375f,0.131799966317414085636983145377598702907562255859375f,-0.30279998407959929362931461582775227725505828857421875f,-1.266599978135526161082680118852294981479644775390625f,-1.2353999684289098670575413052574731409549713134765625f,-1.8210999917551873306109655459295026957988739013671875f,0.36150001684576238414337012727628462016582489013671875f,0.7803999227568498842089184108772315084934234619140625f,-0.01180006071627115460387358325533568859100341796875f,-2.340200016214697864569416196900419890880584716796875f,-1.284999987334014104334301009657792747020721435546875f,1.509999972693622627417653347947634756565093994140625f,0.8702000056162468855092129160766489803791046142578125f,0.81649998417496616465172110110870562493801116943359375f,0.804900005443395460957844989025034010410308837890625f,2.020400039265304759084074248676188290119171142578125f,1.415899997910857077698665307252667844295501708984375f,-2.0199999622255564446504649822600185871124267578125f,-0.184600010150670679198725565584027208387851715087890625f,0.19059998671114453117070297594182193279266357421875f,0.3389999789595601331626539831631816923618316650390625f,-0.0819999705553058344520422906498424708843231201171875f,-0.07180005997121285954420955022214911878108978271484375f,-2.595400054110587273470400759833864867687225341796875f,0.9701999990224841052821602715994231402873992919921875f,0.0793999607995152789641224444494582712650299072265625f,0.245999905668199847497135124285705387592315673828125f,-0.2866999818935986876766719433362595736980438232421875f,-1.7064999412968766545617427254910580813884735107421875f,0.3244000576943164571019906361470930278301239013671875f,-0.02210000571757486653723390190862119197845458984375f,-1.961800040040909909322408566367812454700469970703125f,0.1521999784067276095811394043266773223876953125f,-0.64980002802312331056100447312928736209869384765625f,1.9363000417396420971272164024412631988525390625f,0.8039999859258528669414545220206491649150848388671875f,0.942500017397104539185193061712197959423065185546875f,0.05470004188567345604354841270833276212215423583984375f,-0.21190002759396964204796631747740320861339569091796875f,0.8623999910265214285942647620686329901218414306640625f,-2.202900081603229143212274721008725464344024658203125f,-2.2077999652296309562871101661585271358489990234375f,1.31939994327202381185770718730054795742034912109375f,0.31220000895410660035622640862129628658294677734375f,-0.30569992596581574417768933926708996295928955078125f,0.64239996766895102187078236966044642031192779541015625f,-0.2416999516442419615458447879063896834850311279296875f,1.2087000259116293943151276835124008357524871826171875f,-0.3600999827846884659976467446540482342243194580078125f,-1.2963000232435761827076703411876223981380462646484375f,-0.3870999840736388808437595798750407993793487548828125f,3.057400014477222871533967918367125093936920166015625f,0.166300016705692055296594844548963010311126708984375f,0.574700004781782514129417904769070446491241455078125f,1.662799923841654869960393625660799443721771240234375f,-0.01609990085810475424210608252906240522861480712890625f,0.36640000796541671856942912199883721768856048583984375f,0.7906000228092062709350784643902443349361419677734375f,0.179199954773487046821855983580462634563446044921875f,1.7604999491274355705172638408839702606201171875f,2.392100016818939689500211898121051490306854248046875f,-0.1736000247612594638013661096920259296894073486328125f,0.31020000006556447402772391797043383121490478515625f,-1.2975000423006708416551191476173698902130126953125f,-0.097300039534270243990476956241764128208160400390625f,-0.66799996183067644750508407014422118663787841796875f,-0.37069999624341731792043219684273935854434967041015625f,1.36019995886385469674451087485067546367645263671875f,-1.0191000564783811999092222322360612452030181884765625f,-0.123099960298836297312163878814317286014556884765625f,-1.5430999514326446675482884529628790915012359619140625f,-2.37060002843439487918431041180156171321868896484375f,0.7756000156193978423146973000257275998592376708984375f,1.4786999542742957913787904544733464717864990234375f,-1.3509000153452148662580611926387064158916473388671875f,-1.18909999832659973861836988362483680248260498046875f,-0.0143000256240368717186584035516716539859771728515625f,1.089799947296083981740366652957163751125335693359375f,-0.51869997487515195899732134421356022357940673828125f,-0.7135999705210327448412499506957828998565673828125f,-0.39380001320391866403980429822695441544055938720703125f,-0.7468998681887988322358751247520558536052703857421875f,-1.4538999957650899919059384046704508364200592041015625f,-2.31970000215545280042306330869905650615692138671875f,-1.0293999628856782546648673815070651471614837646484375f,-0.7336999591730535286160375108011066913604736328125f,-2.229699966746569117503895540721714496612548828125f,1.2305000083595503124200831734924577176570892333984375f,1.23359991933554358212177248788066208362579345703125f,0.0107999742224829109460415565990842878818511962890625f,-0.18690006797611713462714533307007513940334320068359375f,-0.18279996530413666899761437889537774026393890380859375f,-0.4255999302357438285326907134731300175189971923828125f,1.7827999841727313334871496408595703542232513427734375f,1.17439997055977674023097279132343828678131103515625f,-1.01460002007856875394509188481606543064117431640625f,-0.7613000103726978995410945572075434029102325439453125f,-0.806300020281970564184348404523916542530059814453125f,-0.079199975821376167317566796555183827877044677734375f,0.86569998517930546455545481876470148563385009765625f,-0.422599985373765196783324427087791264057159423828125f,-0.4728000148132445179527394429896958172321319580078125f,0.785500014021992765123059143661521375179290771484375f,-0.922200039240717917010670134914107620716094970703125f,2.149499954368919940606019736151210963726043701171875f,-1.0824000524938106604366794272209517657756805419921875f,0.188600025171040908134045821498148143291473388671875f,-1.2329999389015140476288934223703108727931976318359375f,-0.9182999868884682559411203328636474907398223876953125f,-0.226400050787627815651603668811731040477752685546875f,-0.52730004050284551286864598296233452856540679931640625f,-0.694199979379027443116001450107432901859283447265625f,-0.2735999739855528023468878018320538103580474853515625f,1.4815999936699866257328039864660240709781646728515625f,1.60819999061822915820130219799466431140899658203125f,0.789899944013356947181137002189643681049346923828125f,-0.1847000021368263578125379353878088295459747314453125f,0.459000008538364756560667956364341080188751220703125f,-0.21260001033321007657406198632088489830493927001953125f,0.6033999935522678104149463251815177500247955322265625f,1.0625999810338020434841155292815528810024261474609375f,2.36180002692788892915132237249054014682769775390625f,-0.794800021429358682922838852391578257083892822265625f,-1.757000019114464794967034322326071560382843017578125f,0.05719997740089954874065369949676096439361572265625f,-0.704799968977272595083150008576922118663787841796875f,0.34050004885345774852112299413420259952545166015625f,1.875400017714500489063311761128716170787811279296875f,-0.917700023180991752980162345920689404010772705078125f,2.258400042875110980133968041627667844295501708984375f,-2.068499957896769547716075976495631039142608642578125f,0.15619993646740937265349202789366245269775390625f,1.27900001226365578332888617296703159809112548828125f,0.29589998390376492931608254366437904536724090576171875f,0.08660000451058114112612429380533285439014434814453125f,0.15849999138712878021806318429298698902130126953125f,-1.0676000306174151521787507590488530695438385009765625f,0.1550999996602531183498285827226936817169189453125f,-1.033899972593784521990301072946749627590179443359375f,-0.09869996768534228248626050117309205234050750732421875f,1.4004999833256006613879662836552597582340240478515625f,-1.229200042764842404352521043620072305202484130859375f,0.26490001069605406680551595854922197759151458740234375f,1.39150003036856606541959990863688290119171142578125f,-0.836999975956976616231486332253552973270416259765625f,-1.401999990373849147573537265998311340808868408203125f,0.71720000675618589269788571982644498348236083984375f,-0.5016000038772825408983635497861541807651519775390625f,-1.96059994893670097582116795820184051990509033203125f,-3.590399988470972747478526798659004271030426025390625f,-2.199400042159854518075690066325478255748748779296875f,0.7449000482097261066627424952457658946514129638671875f,0.11150006542354840011199712535017170011997222900390625f,-1.925899955479800862434558439417742192745208740234375f,0.60789993858933399906163685955107212066650390625f,-0.496099994400143184947182817268185317516326904296875f,0.809700019980966967381164067774079740047454833984375f,2.24220005119889975730984588153660297393798828125f,-1.0809999986514451375541057132068090140819549560546875f,1.03109999782741024176857536076568067073822021484375f,-0.08450003477930956297115017150645144283771514892578125f,-1.7424999904260030092473243712447583675384521484375f,-1.7332000049605966740529083836008794605731964111328125f,0.00100002584606517785914547857828438282012939453125f,0.3634999361410737872546405924367718398571014404296875f,0.3291999863266938941563921616761945188045501708984375f,0.7001000482007857517174898021039552986621856689453125f,-1.075599975199997171415589036769233644008636474609375f,-0.485699975977837095086897534201852977275848388671875f,0.37410008713752151976450477377511560916900634765625f,-0.563900021711737142737774775014258921146392822265625f,-0.8474000351712105416623899145633913576602935791015625f,-0.145700011181830568762762823098455555737018585205078125f,-0.2390000611022113918835430013132281601428985595703125f,-2.8830000158101309892799690715037286281585693359375f,0.3624000440001484690810684696771204471588134765625f,1.214999982845037518330855164094828069210052490234375f,-0.614200017675011533668794072582386434078216552734375f,0.94270001653134816166357268230058252811431884765625f,-1.83849992492795077936307279742322862148284912109375f,-1.6666999865502116762172590824775397777557373046875f,-1.731899982368946933064535187440924346446990966796875f,0.70050005690008398939738754052086733281612396240234375f,-2.37240003153905210098173483856953680515289306640625f,0.2942999814607214492667708327644504606723785400390625f,-1.3577999842286114784428718849085271358489990234375f,-0.6247000079855322685062901655328460037708282470703125f,-0.0434999360293151227097041555680334568023681640625f,1.0660999656781562361373971725697629153728485107421875f,2.013999947965145143058407484204508364200592041015625f,1.03190002537742220312111385283060371875762939453125f,-0.5081000393122436786796924934606067836284637451171875f,0.423000047661364231288416704046539962291717529296875f,-2.18350000312179393091582824126817286014556884765625f,-0.01030001783072986842881846314412541687488555908203125f,-0.20519998017251517286041462284629233181476593017578125f,-0.423599994176626248876260660836123861372470855712890625f,1.83869990659058135662462518666870892047882080078125f,-1.64580004451870909321087310672737658023834228515625f,0.6922000993669026325250115405651740729808807373046875f,0.4857999346598991241563680887338705360889434814453125f,-2.866700047458708322523079914390109479427337646484375f,3.0456000078335403458140717702917754650115966796875f,1.0629999837353825142827190575189888477325439453125f,0.43239997988790346283849430619738996028900146484375f,0.203699990390987295540980994701385498046875f,0.04260000811666220243978386861272156238555908203125f,-0.4223999908402562564191384808509610593318939208984375f,-2.03239999724775532996545734931714832782745361328125f,0.4618999277934430214287431226694025099277496337890625f,0.7167000028669843203488198923878371715545654296875f,1.0625999798789624861683478229679167270660400390625f,0.35830001937300026693122845244943164288997650146484375f,0.39729998976439262481363812185009010136127471923828125f,0.331300014768541206677809896063990890979766845703125f,2.174300032553076800212465968797914683818817138671875f,1.7301999403864147808462803368456661701202392578125f,0.9250000506266946587174970773048698902130126953125f,1.4268999614328148339836843661032617092132568359375f,0.6016000324502577267793412829632870852947235107421875f,0.0804000001311304668405455231550149619579315185546875f,0.57830000286996374736503412350430153310298919677734375f,-0.3051000593766579616072931457892991602420806884765625f,2.37919995786547655569620474125258624553680419921875f,-0.0163999479696155248120703618042171001434326171875f,-1.58120002758801003750477320863865315914154052734375f,1.130700034166872480767551678582094609737396240234375f,-0.64550001808255930146884793430217541754245758056640625f,-0.886700038592517447710861233645118772983551025390625f,-1.2468999819591635169757637413567863404750823974609375f,-0.09410006728172248813280020840466022491455078125f,0.98490000691488344131130361347459256649017333984375f,0.3646000316083430892177830173750407993793487548828125f,1.5522999950170512928337984703830443322658538818359375f,-1.6374000049591057592834886236232705414295196533203125f,1.8679000430464742787961540670949034392833709716796875f,1.361099965380132115200240150443278253078460693359375f,2.005200042831897899731075085583142936229705810546875f,-1.4889999545961618565570461214520037174224853515625f,1.004799998407065242389535342226736247539520263671875f,-0.612499939650298141913253857637755572795867919921875f,-0.403999952435493359104157207184471189975738525390625f,0.4294999761059876686175584836746565997600555419921875f,-1.6107000386372207767493591745733283460140228271484375f,0.697700057248770821871630687383003532886505126953125f,-1.046900001405180358204916046815924346446990966796875f,-1.8750000312924377521284213798935525119304656982421875f,1.575999983452260000404976381105370819568634033203125f,2.05159995693862384769090567715466022491455078125f,1.346999987356364503199301907443441450595855712890625f,2.81749996252357970405455489526502788066864013671875f,1.065399953304230873385449740453623235225677490234375f,-0.0598000754460683570101764416904188692569732666015625f,0.921400011560320475467733558616600930690765380859375f,2.65849999071657716598338083713315427303314208984375f,1.703999981790780271495577835594303905963897705078125f,0.5826999937698242870709464114042930305004119873046875f,-0.305799941477178816029436347889713943004608154296875f,0.0825999831199641931789301452226936817169189453125f,-1.1391000108808280177896676832460798323154449462890625f,1.870699986259638958330242530792020261287689208984375f,0.007899999218434317072023986838757991790771484375f,0.5779000344783071785315087254275567829608917236328125f,-0.132400048209726561854182591559947468340396881103515625f,-1.81789991873353784512801212258636951446533203125f,-0.3368999970093360385448022498167119920253753662109375f,2.671199990595876538890252049895934760570526123046875f,-0.2217999633371829926176133085391484200954437255859375f,0.72640001841485446476553988759405910968780517578125f,-0.5326999901190399011596809941693209111690521240234375f,-0.8902000242613257174895124990143813192844390869140625f,0.8358999887466429523641409105039201676845550537109375f,-0.2484999629817898902928163806791417300701141357421875f,-0.8117999376326798799397010952816344797611236572265625f,-0.2468999987602240597794889254146255552768707275390625f,-1.1270000126510864024709235309273935854434967041015625f,0.327099982695281621403182725771330296993255615234375f,-0.980400023823976507486577247618697583675384521484375f,0.432200109387933739668596899718977510929107666015625f,0.40099998930096536486900049567339010536670684814453125f,1.6389999848827725426048118606559000909328460693359375f,0.4060000199601050407949287546216510236263275146484375f,0.34889998991638382364044446148909628391265869140625f,-0.7344000180199741212305752924294210970401763916015625f,-0.57110002779737134570581247317022643983364105224609375f,0.65930002946108601236119284294545650482177734375f,-0.68500004604458919477139033915591426193714141845703125f,-0.51549999900907261807248005425208248198032379150390625f,0.9599999834224579586816616938449442386627197265625f,0.654300045628845250433869296102784574031829833984375f,-1.2025999821886419471894669186440296471118927001953125f,0.7626999544307591083480701854568906128406524658203125f,-1.2211999902233479087243495087022893130779266357421875f,0.34699998821318178698902556789107620716094970703125f,-0.01889993169009673845692987015354447066783905029296875f,-0.7557000041037797988252577852108515799045562744140625f,1.0645000541135669624992488024872727692127227783203125f,0.3170000066906222269125237289699725806713104248046875f,0.2227000053182248695549105832469649612903594970703125f,-1.696200025166571645485191766056232154369354248046875f,-2.331700034382939268340351191000081598758697509765625f,0.190299979348480530205733884940855205059051513671875f,0.339199916253984046221603421145118772983551025390625f,-0.20269999924600090590587342376238666474819183349609375f,1.8243999927252527726295738830231130123138427734375f,-1.3822999819040295843564081224030815064907073974609375f,0.38389997654259200299264875866356305778026580810546875f,-0.93610003236085237432462236029095947742462158203125f,-1.2836999737575649493948048984748311340808868408203125f,-1.2192000067412853692161434082663618028163909912109375f,2.192900033342093824018093073391355574131011962890625f,-1.71310003651827624793213544762693345546722412109375f,0.336099985682964452760046469848020933568477630615234375f,-0.80930001768916792759256395584088750183582305908203125f,0.13409997835904352569968978059478104114532470703125f,-0.421300009664893337912872084416449069976806640625f,1.6175999914087355602276829813490621745586395263671875f,-1.0015999991089101772701042136759497225284576416015625f,1.083100045794248789121638765209354460239410400390625f,0.991300009031593543085136843728832900524139404296875f,1.1131000460550193320585776746156625449657440185546875f,-0.2112000750109555013978024362586438655853271484375f,-0.354100009129941939445274101672112010419368743896484375f,1.9473999989613888228490168330608867108821868896484375f,0.72509996702671009050078509972081519663333892822265625f,0.583400025664270405201250468962825834751129150390625f,-1.7629000036701556464180384864448569715023040771484375f,-0.32119999789744546614400633188779465854167938232421875f,-0.1556999758660790877939916754257865250110626220703125f,0.75920006113052374008276501626824028789997100830078125f,0.5126000097930434318271863958216272294521331787109375f,0.1949000144585968374855156071134842932224273681640625f,-0.6245999854892494429492444396601058542728424072265625f,-0.3803000319123268635479462318471632897853851318359375f,-1.0253000193580985577312958412221632897853851318359375f,0.4687000291340053248489994075498543679714202880859375f,1.7450000273250048099527020895038731396198272705078125f,0.629300034937262608991659362800419330596923828125f,1.8062000125378363524220048930146731436252593994140625f,1.1472999756455422026846235894481651484966278076171875f,-1.737399979664385085698086186312139034271240234375f,-1.3474000282794234006900069289258681237697601318359375f,0.006899970373511365995256028327275998890399932861328125f,0.804800038845092213790621826774440705776214599609375f,-0.7351000310085715749863766177440993487834930419921875f,0.806799996995180901393496242235414683818817138671875f,0.433700016859174031225165890646167099475860595703125f,-0.618400028793514078273574341437779366970062255859375f,1.2135999922767284342484117587446235120296478271484375f,0.0764999777302144412516327065532095730304718017578125f,-0.5624000262118877824235596563084982335567474365234375f,0.010400067260861345364020280612749047577381134033203125f,0.181800061088800146702482152250013314187526702880859375f,-1.6085999812126148267310554729192517697811126708984375f,1.184800002616643865849255234934389591217041015625f,-2.30909997154176238609579741023480892181396484375f,-0.15709994115829484240265401240321807563304901123046875f,1.561900011985004166348289800225757062435150146484375f,0.2619000262156128133739230179344303905963897705078125f,-0.237599962593615376960087814950384199619293212890625f,-0.69970000701695678202440831228159368038177490234375f,1.024599974350631104158537709736265242099761962890625f,0.91409996275007632249298694659955799579620361328125f,-1.535100047194957628704514718265272676944732666015625f,-0.460999941952525749133684485059347935020923614501953125f,-2.0644999857917429864073710632510483264923095703125f,-0.39790004882067442348869690249557606875896453857421875f,0.0424999520927664686809066552086733281612396240234375f,-0.2817999565571553954868022628943435847759246826171875f,2.10760000249147338990951539017260074615478515625f,0.6313000436395406200773550153826363384723663330078125f,-0.2438999801188710403465620402130298316478729248046875f,-0.21159994464516584944391297540278173983097076416015625f,-1.1241000080868606136874632284161634743213653564453125f,-1.5331000409886248458946056416607461869716644287109375f,-0.2884000271171329732311505722464062273502349853515625f,1.7112000130593771896059251957922242581844329833984375f,-0.426299949613212714183418938773684203624725341796875f,-0.1571999535776675838150140407378785312175750732421875f,0.5413000404357906081287410415825434029102325439453125f,-0.8972000258296721764139647348201833665370941162109375f,-0.81989999126866452971995613552280701696872711181640625f,-2.770799957607687158400722182705067098140716552734375f,-1.5295999910771842511536533493199385702610015869140625f,0.415600029551982164122136964579112827777862548828125f,-0.5027000257328158028258258127607405185699462890625f,-0.0795000002458687848871932146721519529819488525390625f,-1.4333999684624367798591038081212900578975677490234375f,-1.881100005650520490263488682103343307971954345703125f,-0.0574000179044897862468133098445832729339599609375f,-0.0214999749362476677561062388122081756591796875f,-2.4110999644488089188598678447306156158447265625f,0.168999953981489647869551617986871860921382904052734375f,0.357300046463310483346731416531838476657867431640625f,1.13869997830241942438078694976866245269775390625f,0.12570004474669660421426442553638480603694915771484375f,-0.8726999442607166201923973858356475830078125f,1.5490000189319246270969188117305748164653778076171875f,-0.4065999907642596777890275916433893144130706787109375f,1.580599968568980440153382005519233644008636474609375f,0.8051999935835609800705015004496090114116668701171875f,0.0613999898046257985839702087105251848697662353515625f,-1.84169998252689826045980225899256765842437744140625f,-0.2449000032454724529173972769058309495449066162109375f,0.4994999875053760529652890909346751868724822998046875f,-1.7912999983772632806022784279775805771350860595703125f,0.329700009699165519361230280992458574473857879638671875f,-0.1753000536307691703541422612033784389495849609375f,-0.87620004856139388493829756043851375579833984375f,0.07249999698251474722354714685934595763683319091796875f,1.320099921913444607213250492350198328495025634765625f,0.0065000088363882380093627944006584584712982177734375f,1.5113999934554105397666035059955902397632598876953125f,-0.9312000381119551750686014202074147760868072509765625f,1.13819997802674688358592902659438550472259521484375f,-1.8003000154837971447108202482922933995723724365234375f,-2.2987999986752871706130463280715048313140869140625f,-1.83499998081475457212263791006989777088165283203125f,-2.099000037595630363540522012044675648212432861328125f,-0.71359997825101029178540557040832936763763427734375f,0.36630001689195645919738808515830896794795989990234375f,-0.378700031536816916766241547520621679723262786865234375f,-0.66520002390742349884789064162760041654109954833984375f,-0.3384999392330636514003572301589883863925933837890625f,1.58900005352124562563176368712447583675384521484375f,1.7077999818071720738998919841833412647247314453125f,1.5953999943569303621160315742599777877330780029296875f,-1.468000003591180213646794072701595723628997802734375f,-1.0223999979555602468650477021583355963230133056640625f,-0.882400004642457158610113765462301671504974365234375f,0.000399980722367576646547604468651115894317626953125f,0.5297000085443259731476928209303878247737884521484375f,0.9532999919548628131593659418285824358463287353515625f,-0.838800018829108040563369286246597766876220703125f,-0.196599984282254780509902047924697399139404296875f,0.6042999780334532555770010731066577136516571044921875f,0.343900023443997060468291238066740334033966064453125f,1.1238999862655993933202580592478625476360321044921875f,0.62650000235438341178451082669198513031005859375f,0.74650004679709613952809377224184572696685791015625f,0.73339997330531492281124883447773754596710205078125f,-0.0925999954134233416169763586367480456829071044921875f,0.450100017616152536792384353248053230345249176025390625f,-1.6141999870903787073217472425312735140323638916015625f,-0.34800000772625150347039379994384944438934326171875f,1.970199989858269162823489750735461711883544921875f,-0.3301000184729694364449414933915250003337860107421875f,-0.21209997442513728316271226503886282444000244140625f,-1.4923999885305765378262776721385307610034942626953125f,1.107700025881827254892186829238198697566986083984375f,-1.3666000246271494678040880899061448872089385986328125f,0.5942999624803662062078046801616437733173370361328125f,-0.66949999899789691681917247478850185871124267578125f,-1.2372999516919256368652213495806790888309478759765625f,0.996100009431689858985237151500768959522247314453125f,0.5923000355482106460414115645107813179492950439453125f,0.6466000892981889069943690628861077129840850830078125f,0.8298000660210844703357224716455675661563873291015625f,1.778899997180700243148976369411684572696685791015625f,1.130099993476271880155081817065365612506866455078125f,-1.12760003076642778552240997669287025928497314453125f,0.454900027050077593315791091299615800380706787109375f,0.03249998189508962465055219581699930131435394287109375f,-0.15649998467788106548681525964639149606227874755859375f,-1.141099984658509480794918999890796840190887451171875f,-0.50839999109357625339811193043715320527553558349609375f,-0.131300022368133151307034722776734270155429840087890625f,1.524699980604648619220142791164107620716094970703125f,-0.62569996729791199641113053075969219207763671875f,-0.2249999622255562936601336332387290894985198974609375f,1.3343000383459042712530617791344411671161651611328125f,-0.56460002965256606000110650711576454341411590576171875f,0.124399966387450600091568730931612662971019744873046875f,-0.3429999817982316567821499120327644050121307373046875f,1.0870999787092205934868616168387234210968017578125f,-2.1179999819099908364705697749741375446319580078125f,1.6767000303223731361157433639164082705974578857421875f,1.541200000188500229825194764998741447925567626953125f,2.266299983643741189354159359936602413654327392578125f,1.130200057546794756291319572483189404010772705078125f,-0.2744999551698568129864952425123192369937896728515625f,1.514199966396391427991829914390109479427337646484375f,1.4260999685838822159666960942558944225311279296875f,0.1731999870419509424124271390610374510288238525390625f,-0.389400002429634628509091953674214892089366912841796875f,-0.0105999889835717286956651150831021368503570556640625f,-1.16529999961406094399762878310866653919219970703125f,-1.7643999803945418225481489571393467485904693603515625f,-1.2864000260144472509438173801754601299762725830078125f,0.26799998496472887676844720772351138293743133544921875f,0.62289999346286040182718579671927727758884429931640625f,-0.5419999918639659686192544540972448885440826416015625f,-2.328499994181095988921015305095352232456207275390625f,-1.6253000117585063577507753507234156131744384765625f,0.5623999498620630976830625513684935867786407470703125f,-0.0789000243380664212367037180229090154170989990234375f,1.31849998397380119286026456393301486968994140625f,1.540699962604045847314182537957094609737396240234375f,1.129199971239268851519454983645118772983551025390625f,1.758399964644015600612192429252900183200836181640625f,1.4496999931588769872092825607978738844394683837890625f,1.0598000285074113957506369843031279742717742919921875f,-0.5742999701172106785662663241964764893054962158203125f,-0.08780002032667388878195424695149995386600494384765625f,-1.269800023627281060356608577421866357326507568359375f,-1.4755000618919733579303965598228387534618377685546875f,-0.556200017548352576568504446186125278472900390625f,-0.173500026926399097959574646665714681148529052734375f,0.55420001128613927132704475297941826283931732177734375f,1.7877999629944556492233687094994820654392242431640625f,0.85949994503706739212844922803924418985843658447265625f,-1.290600016774237079886233914294280111789703369140625f,1.715800037422031021350221635657362639904022216796875f,0.14290000393092683328433167844195850193500518798828125f,-0.671899987100065043676977438735775649547576904296875f,0.2139000016883023658209594941581599414348602294921875f,1.56189996664822050576049150549806654453277587890625f,-0.6232999976173043688021380148711614310741424560546875f,-1.7736999731615175068810685843345709145069122314453125f,-0.39870000579506170712562607150175608694553375244140625f,0.1828999375648796554827413274324499070644378662109375f,0.08330003591328871692667235038243234157562255859375f,0.7774999457970270810847068787552416324615478515625f,-0.8189000185638661832854268141090869903564453125f,-1.16560000265538743491333661950193345546722412109375f,-0.2960999810263522658004831100697629153728485107421875f,-1.4096999940343197810221909094252623617649078369140625f,0.122999980866908220633604287286289036273956298828125f,-2.46090000171065259593206064892001450061798095703125f,0.1084999983534216116964898901642300188541412353515625f,0.8959000363931064470079945749603211879730224609375f,-0.34779997681528396213934684055857360363006591796875f,0.47109996981322732612085246728383935987949371337890625f,0.2550000334903594900737289208336733281612396240234375f,0.2521999894708393252784617288853041827678680419921875f,-2.504099993260204382039546544547192752361297607421875f,-0.8005000099614267838887826655991375446319580078125f,1.0477999878048895876503365798271261155605316162109375f,-1.627099947156011694460175931453704833984375f,-0.570199991683661977504016249440610408782958984375f,0.5192999485850327001656978609389625489711761474609375f,-0.052300053243339306163761648349463939666748046875f,1.152199955831469058153970763669349253177642822265625f,0.263200007084011478042384624131955206394195556640625f,0.50769999991655279103497377946041524410247802734375f,0.480099962854385164945369979250244796276092529296875f,-2.379600089313090194309552316553890705108642578125f,-0.2973000028401620031814900357858277857303619384765625f,-0.0268999783083796462079817501944489777088165283203125f,-1.0012999986566601950954691346851177513599395751953125f,1.0291999852277344817963466994115151464939117431640625f,-0.493400001487135586586418867227621376514434814453125f,-0.1402000096395614736621837437269277870655059814453125f,-0.413399980513751097443986282087280414998531341552734375f,2.63469999468624482830136912525631487369537353515625f,-1.6381999586924909095841940143145620822906494140625f,0.98399995634704884306387384640402160584926605224609375f,1.306300041739642647797836616518907248973846435546875f,1.2816000225409867052661638808785937726497650146484375f,-0.12829996602684301532093513742438517510890960693359375f,-1.50699998635053589168819598853588104248046875f,0.2848000228822231694181255079456605017185211181640625f,-2.46460000940561219096025524777360260486602783203125f,-0.1139000116348267699351026749354787170886993408203125f,-0.3711000014781957734300021911622025072574615478515625f,-1.5677999839678407045795438534696586430072784423828125f,0.31919996106922621237345083500258624553680419921875f,1.568799990461022364485188518301583826541900634765625f,-0.8641000200822948063006379015860147774219512939453125f,2.284999964386224657886259592487476766109466552734375f,0.94369994987845462475206659291870892047882080078125f,-2.432400004176795693666690567624755203723907470703125f,0.9455000092908736331054342372226528823375701904296875f,0.52359999160617609614831735598272643983364105224609375f,1.93049999064579669294516861555166542530059814453125f,1.189800001722574496199058557976968586444854736328125f,-0.831399973078072118681802749051712453365325927734375f,-0.177200030896067506347435482894070446491241455078125f,-0.71649996215850053626894577973871491849422454833984375f,-1.53780001332312821205050568096339702606201171875f,-0.091299970698356547327279031378566287457942962646484375f,0.59499999692663518846558190489304251968860626220703125f,2.56929995607286709713434902369044721126556396484375f,-1.6037999823398880749891759478487074375152587890625f,0.58870003302693374447329688337049447000026702880859375f,0.9577999974533926152986396118649281561374664306640625f,-0.6449999943748121467024247976951301097869873046875f,0.28149998562782985178643002655007876455783843994140625f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,3,5,8,7};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,2,4,6,7};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1,1},        // move_strides
                                                                  Strides{1,1,1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0,0}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.20289988379031631549054282004362903535366058349609375f,0.13019999593049325081750566823757253587245941162109375f,2.989100064897538100439078334602527320384979248046875f,2.7403999176532014558915761881507933139801025390625f,0.73939990573972347309705810403102077543735504150390625f,3.32020010647848362594913851353339850902557373046875f,-1.137100040586292326594275436946190893650054931640625f,0.26990007749050770602394777597510255873203277587890625f,2.4770000157803284679403077461756765842437744140625f,0.50800001929327998073659955480252392590045928955078125f,-0.52329987136721545226691887364722788333892822265625f,-0.52909997373968298273183563651400618255138397216796875f,0.586900149615853905515905353240668773651123046875f,-1.815800052230059247193594273994676768779754638671875f,5.23659996174424779979972299770452082157135009765625f,4.39350007628649397162234890856780111789703369140625f,1.2066000982016318321399239721358753740787506103515625f,1.9846999736942354530810916912741959095001220703125f,-1.1419999631233508363692408238421194255352020263671875f,-1.14160008685812375262003115494735538959503173828125f,-0.66119988183602718123665908933617174625396728515625f,-0.3103999343052510351270711908000521361827850341796875f,0.3130999425664546809144894723431207239627838134765625f,-1.7154999895840881141140243926201947033405303955078125f,-3.43670001251548473675256900605745613574981689453125f,0.0259000097200268175612336563062854111194610595703125f,-0.6702998929217469648023097761324606835842132568359375f,0.20779999909251767320483850198797881603240966796875f,-1.939899987040459183873508663964457809925079345703125f,-1.317700028526783029292346327565610408782958984375f,-3.1571000030726192875363267376087605953216552734375f,0.7551999692760411608816184525494463741779327392578125f,-2.430700040145963303217513384879566729068756103515625f,3.1357999614819878075877568335272371768951416015625f,-0.11050006911903753614723200371372513473033905029296875f,-1.5370000213496393115519822458736598491668701171875f,0.41389995033517490430341467799735255539417266845703125f,-0.1494000060617930447648404879146255552768707275390625f,1.0671000083252788481757988847675733268260955810546875f,1.820700002576411069554751520627178251743316650390625f,0.5623999284788971930737488946761004626750946044921875f,-1.914999942388385445468657053424976766109466552734375f,2.154500004455447648155086426413618028163909912109375f,-2.2410000658929352113091226783581078052520751953125f,3.190200142204016930946863794815726578235626220703125f,-2.1877000264964987508164995233528316020965576171875f,0.2650000340677802768141191336326301097869873046875f,-0.41059994506239860800889118763734586536884307861328125f,-2.5066000876963148158438343671150505542755126953125f,0.135500058688223123937177660991437733173370361328125f,0.441900000920892210842794156633317470550537109375f,1.1243000091210018087650723828119225800037384033203125f,0.7222000155292442880039516239776276051998138427734375f,-4.08990006679892648122631726437248289585113525390625f,0.9134999443739644231499141824315302073955535888671875f,-2.1320001028776172091738771996460855007171630859375f,1.2328000070273870125703297162544913589954376220703125f,-2.215800047051906584982816639239899814128875732421875f,2.831400060492009540524804833694361150264739990234375f,-0.04560007351040962220167784835211932659149169921875f,4.1785999483555542610702104866504669189453125f,-3.29680005944967202680118134594522416591644287109375f,0.1633000322625033362555768690072000026702880859375f,0.671400006046892539046666570357047021389007568359375f,5.52930000481828987091148519539274275302886962890625f,-0.4078000063568361088783831291948445141315460205078125f,-0.1231999561779186791454776539467275142669677734375f,2.39729996975958403737649859976954758167266845703125f,2.381099987582862009816153658903203904628753662109375f,-3.89060003178715607674575949204154312610626220703125f,0.653300000634789324038820268469862639904022216796875f,3.573000067405402990772245175321586430072784423828125f,-0.55240005230754596343700768557027913630008697509765625f,2.6002001702181996023455212707631289958953857421875f,-3.290099987217782739890026277862489223480224609375f,-1.0623000572666516649888990286854095757007598876953125f,-0.238100047098098677178512616592342965304851531982421875f,3.009499997489152978147330941283144056797027587890625f,-0.40209997975230182820638447083183564245700836181640625f,2.25290002028495184305256771040149033069610595703125f,-0.794200071728974332785355727537535130977630615234375f,-0.6691999681845308689531748314038850367069244384765625f,-2.844699981163442803477892084629274904727935791015625f,-1.411399884434788365439317203708924353122711181640625f,-0.3418999886460600823312461216119118034839630126953125f,3.17270001114830346722328613395802676677703857421875f,2.2492000240825102963526660460047423839569091796875f,0.858199951279163375517100575962103903293609619140625f,0.17170006303042129047753405757248401641845703125f,1.3201999497368930303053957686643116176128387451171875f,1.1246999541178348547276755198254249989986419677734375f,-1.2485998997591425396791464663692750036716461181640625f,-3.45210000961050500478677349747158586978912353515625f,1.3508999575287108374510580688365735113620758056640625f,-1.9159000083789232693476378699415363371372222900390625f,-1.5450000504776841836473977309651672840118408203125f,6.50810000388473230970021177199669182300567626953125f,-0.54220008815377906952193143297336064279079437255859375f,-1.42240013234540985109788380214013159275054931640625f,0.776000183574855117996094122645445168018341064453125f,0.0688000453531742994783826361526735126972198486328125f,-3.75709997807592088747696834616363048553466796875f,1.586800054383278535397039377130568027496337890625f,-4.4442999680310482091272206162102520465850830078125f,-0.5525999760419129103183877305127680301666259765625f,1.0823000470593580057965255036833696067333221435546875f,-1.0852999914556737959259180570370517671108245849609375f,-1.4998000237017869995526098136906512081623077392578125f,-2.91090003054439971919009622070007026195526123046875f,2.624299957562981955305758674512617290019989013671875f,-2.019000081069769247932299549574963748455047607421875f,-2.549700157369673814145016876864247024059295654296875f,1.95070013445168921606409639935009181499481201171875f,-3.917800058287381315125230685225687921047210693359375f,-0.29069998782426031969094992746249772608280181884765625f,-1.200500116802751815470173823996447026729583740234375f,0.02260002387463921902366337235434912145137786865234375f,1.258099941691009693300884464406408369541168212890625f,0.4280000650770963144253755672252736985683441162109375f,-2.03180001176595670386859637801535427570343017578125f,-5.636499883331357096949432161636650562286376953125f,-0.63009992772489820112014058395288884639739990234375f,0.6095999296471463768654075465747155249118804931640625f,1.176600005143881855218523924122564494609832763671875f,-0.281799958084524015955452114212675951421260833740234375f,-1.0533998909950248457079169384087435901165008544921875f,-1.3512999899394804170782435903674922883510589599609375f,-2.07280002569109189636265000444836914539337158203125f,-1.55439998551681668459423235617578029632568359375f,-3.09450000805035241313589722267352044582366943359375f,-4.0139999058321134128846097155474126338958740234375f,-1.851500070415436738358039292506873607635498046875f,0.81220005782991666620773685281164944171905517578125f,-1.0256001046910891982832936264458112418651580810546875f,-2.1325999921351677102165922406129539012908935546875f,-1.3347999608002598659339810183155350387096405029296875f,-0.961300012272596315909822806133888661861419677734375f,0.5294000200688839807838803608319722115993499755859375f,3.252500028274951215934152060071937739849090576171875f,-5.06859990711659147422096793889068067073822021484375f,1.2113998821623628288079999038018286228179931640625f,-0.982899988731741558467547292821109294891357421875f,2.795299954761564453775690708425827324390411376953125f,2.0806000909447650570882615284062922000885009765625f,0.0238000276766714424780957415350712835788726806640625f,0.25030001114383193527856974469614215195178985595703125f,-2.72410001292973635855787506443448364734649658203125f,-0.38870005824714848330359018291346728801727294921875f,2.993300063275545763730178805417381227016448974609375f,-0.06329992471337286819021983319544233381748199462890625f,-1.0703000221148142312443951595923863351345062255859375f,-0.2833999914847311618615322004188783466815948486328125f,-0.168900043858587534106874272765708155930042266845703125f,-6.4299000021651391278965093079023063182830810546875f,-3.1336000041976586061309717479161918163299560546875f,-0.4185000750757754683917255533742718398571014404296875f,-1.152900095126033530590348163968883454799652099609375f,-0.9039998684301966402898642627405934035778045654296875f,0.14589989341646425469178893763455562293529510498046875f,0.35499995980411735985171617357991635799407958984375f,-2.258700074712931549214545157155953347682952880859375f,-5.07770000094100648624362293048761785030364990234375f,0.4265999858655027399123582654283381998538970947265625f,-3.983900053823740794456398361944593489170074462890625f,-1.4891999837003659923340137538616545498371124267578125f,-1.447199970695374648954611984663642942905426025390625f,-0.379299983379245253534151061103329993784427642822265625f,-2.128099933905156060376384630217216908931732177734375f,0.31079999243915035211927033742540515959262847900390625f,2.66679998086467406892552389763295650482177734375f,-1.850999972276389460290602073655463755130767822265625f,2.298900054345279553302816566429100930690765380859375f,2.8794000721484422911089495755732059478759765625f,-0.56439997931420837762317432861891575157642364501953125f,4.82810001841336511319013879983685910701751708984375f,-3.2412999890267855107595096342265605926513671875f,-0.493400095234065527716182941730949096381664276123046875f,-0.055700008369237163918796795769594609737396240234375f,-1.6465000633746382785460582454106770455837249755859375f,0.8299000738769770624259081159834749996662139892578125f,-0.40139999751597665866853503757738508284091949462890625f,1.985799947698412726282413132139481604099273681640625f,-0.5980000006481993590767842761124484241008758544921875f,-0.44469999705180551785588249913416802883148193359375f,-2.2167000003337857805263411137275397777557373046875f,0.653300039917974917358378661447204649448394775390625f,-0.85090000886321071060791609852458350360393524169921875f,3.09740002444237294554341133334673941135406494140625f,-1.0447999907515941320212959908531047403812408447265625f,1.2388000533625465049425429242546670138835906982421875f,2.33519998713880649887641993700526654720306396484375f,-4.62359995198771489555156222195364534854888916015625f,0.6142999418422581126009163199341855943202972412109375f,2.064900027273595117804916299064643681049346923828125f,0.127299972143770201427059873822145164012908935546875f,-0.827899961127341210698205031803809106349945068359375f,-0.3749000039532768280281516126706264913082122802734375f,1.1165000290088358259055212329258210957050323486328125f,0.6768998986989254174062580204918049275875091552734375f,0.5531999805971976602592121707857586443424224853515625f,-0.584799949475379055030543895554728806018829345703125f,-4.66149986433237817351482590311206877231597900390625f,1.43119989604800945670604050974361598491668701171875f,0.2581999718055119075188486021943390369415283203125f,-1.49609984529539730147007503546774387359619140625f,1.5340000701546661066032584130880422890186309814453125f,0.97610003321766836137385325855575501918792724609375f,-0.0904999834001054548338061067624948918819427490234375f,-0.7442999949648971824700538491015322506427764892578125f,1.3993999580055469511563615014893002808094024658203125f,-0.78500000637024613947545503833680413663387298583984375f,1.555299986463785533175041564390994608402252197265625f,-2.057499936632812165271388948895037174224853515625f,-2.461699967961013868489317246712744235992431640625f,0.402999994780868231458725858828984200954437255859375f,0.664399982070922323629247330245561897754669189453125f,-0.60939988003522049808680094429291784763336181640625f,-0.19009996423274266863501225088839419186115264892578125f,-0.0107999111533156888498297121259383857250213623046875f,-1.4770000073798004169844944044598378241062164306640625f,1.040299943157286133299521679873578250408172607421875f,-0.9375999506540593220194068635464645922183990478515625f,-1.41360006443560148881033455836586654186248779296875f,-3.357099913553894143802835969836451113224029541015625f,3.7134000491708523128409069613553583621978759765625f,0.8301000154927373753110941834165714681148529052734375f,3.31619993987604999574614339508116245269775390625f,-1.3289999243281780394454472116194665431976318359375f,-0.54419995192363834757287577303941361606121063232421875f,0.10819995384961467177475924472673796117305755615234375f,-0.75060007321238497723925320315174758434295654296875f,-2.273900067178905803899624515906907618045806884765625f,-0.5600999658346179632673056403291411697864532470703125f,1.3194999327436101044241922863875515758991241455078125f,2.64999999154358878428183743380941450595855712890625f,-1.911799970508365031918174281599931418895721435546875f,1.54709992975890653355008907965384423732757568359375f,1.05620002065897100607116954051889479160308837890625f,-0.8088001000776883220311219702125526964664459228515625f,0.20039997118562358213722518485155887901782989501953125f,-3.57019997972548086551114465692080557346343994140625f,0.9705000074654825947817471387679688632488250732421875f,0.8849999150261276259499254592810757458209991455078125f,2.724200045074521181476256970199756324291229248046875f,0.8135999264322213431199770639068447053432464599609375f,1.4411000420838588897964882562519051134586334228515625f,0.3761999877646584611312619017553515732288360595703125f,0.334899949713053590727440678165294229984283447265625f,1.160999950148164483465507146320305764675140380859375f,0.450599990920723225684696444659493863582611083984375f,1.96020002651512736946415316197089850902557373046875f,-3.65620004111081353670442695147357881069183349609375f,0.8854000162988910904715567085077054798603057861328125f,-1.929700077071041075527091379626654088497161865234375f,-3.8116999613672479796377956517972052097320556640625f,-0.7644000444322838205835068947635591030120849609375f,-2.304299973879754048766699270345270633697509765625f,4.44239992102831582343469563056714832782745361328125f,-1.18640003501102331284755564411170780658721923828125f,1.5555000405833132592903211843804456293582916259765625f,2.12500006629154025716843534610234200954437255859375f,2.1997999994128942802262827171944081783294677734375f,-0.32659989189505633788712657405994832515716552734375f,0.304500011663884517787437289371155202388763427734375f,-1.37619998813718691366148050292395055294036865234375f,3.210000013634563220676909622852690517902374267578125f,-3.054599915565548773344062283285893499851226806640625f,2.081900025209785010105179026140831410884857177734375f,1.74839991753771872851075386279262602329254150390625f,-2.14749995868652998609604765078984200954437255859375f,0.5420000331774363910852798653650097548961639404296875f,-0.476200001809001349784011836163699626922607421875f,-0.04129996712207872899824678825098089873790740966796875f,-1.985199962030349762898140397737734019756317138671875f,1.9118000550538292170443810391589067876338958740234375f,0.9781000257149330412431709191878326237201690673828125f,-1.437799949993193049380124648450873792171478271484375f,0.7870999614797538068700077928951941430568695068359375f,2.183399965202809323017163478652946650981903076171875f,2.31789999683424863263780935085378587245941162109375f,0.435899942329525169615322965910308994352817535400390625f,2.577399998737870401299687728169374167919158935546875f,-0.9712000380560770196325393044389784336090087890625f,-4.83550002121180266811961701023392379283905029296875f,2.135000054314731610105582149117253720760345458984375f,-1.214899991380422594744459274807013571262359619140625f,-0.6262999364815644565140928534674458205699920654296875f,2.130899951083957688524606055580079555511474609375f,-1.7351000262402005436257468318217433989048004150390625f,1.3062999666377894669722081744112074375152587890625f,1.6504999797865753663472787593491375446319580078125f,1.5033999991774560367474578015389852225780487060546875f,0.42289998007044127259490551296039484441280364990234375f,1.0207998988226065772977335655014030635356903076171875f,-1.855600041435659708355387920164503157138824462890625f,-3.31160005606859986215795288444496691226959228515625f,-0.6109999218173325363778758401167578995227813720703125f,0.53329998646006038232059154324815608561038970947265625f,1.2002999179370714699643940548412501811981201171875f,-0.503700134094058338263266705325804650783538818359375f,-2.662299951021372290682620587176643311977386474609375f,-0.998800013967602762221531520481221377849578857421875f,1.72410008825510718821760747232474386692047119140625f,1.234200065489113651295838280930183827877044677734375f,2.5271000141367334634878716315142810344696044921875f,4.19060009543374167861884416197426617145538330078125f,-0.72399991182982947890423019998706877231597900390625f,-0.765199986095727258117449309793300926685333251953125f,-2.261499981790781266255407899734564125537872314453125f,1.2314000466525556465313684384454973042011260986328125f,-5.35929997522085699301896966062486171722412109375f,1.31239997977614475388463688432238996028900146484375f,1.0405999871768034648056300284224562346935272216796875f,0.0389001125715677797956004724255762994289398193359375f,1.025500007763505028179906730656512081623077392578125f,3.28500006562099056139913955121301114559173583984375f,0.4042998597554861017755456487066112458705902099609375f,-0.665200019548833676452659346978180110454559326171875f,0.84889996018856839210542375440127216279506683349609375f,0.04040000223591977412951337100821547210216522216796875f,1.29520000334381979456566114095039665699005126953125f,1.1546999790400267471568440669216215610504150390625f,2.8786000739164645523260332993231713771820068359375f,0.63140003005638778432739854906685650348663330078125f,-3.9377000856727359945352873182855546474456787109375f,0.40879990934282528503018738774699158966541290283203125f,0.257700109104812946636542392297997139394283294677734375f,2.116600008776038155389187522814609110355377197265625f,-0.05349996871873707871003489344730041921138763427734375f,2.5151999971412113410451638628728687763214111328125f,2.153999947611243470646513742394745349884033203125f,-1.26769988333359151511103846132755279541015625f,-1.506800055072456956395399174652993679046630859375f,2.62720005965530933877971619949676096439361572265625f,-2.3204000679686682673263931064866483211517333984375f,2.56139997866600754861110544879920780658721923828125f,-0.96139994465410705259955648216418921947479248046875f,0.045099956037104593775666216970421373844146728515625f,-1.2218000632867209720444634513114579021930694580078125f,-1.5384999667257075461890281076193787157535552978515625f,-1.0313000185683371956457676787977106869220733642578125f,-0.055700028187782635313851642422378063201904296875f,1.468199999745189554545277133001945912837982177734375f,1.7099999193102120020881784512312151491641998291015625f,0.61019995815604965994083386249258182942867279052734375f,0.32979997415542727789983246111660264432430267333984375f,1.938699944942443575968127333908341825008392333984375f,-0.8648999596595763339479390197084285318851470947265625f,-2.445600141794980419973626339924521744251251220703125f,-2.262599875980617181170373442000709474086761474609375f,-0.962699980954826006751545719453133642673492431640625f,1.8964999867826701773765307734720408916473388671875f,0.124099981190264774255638258182443678379058837890625f,0.0861999978229395225071129971183836460113525390625f,1.078899978125840153353465211694128811359405517578125f,-1.5251000459842385392761343609890900552272796630859375f,-1.514399920175225400242879914003424346446990966796875f,5.36729999915212285799270830466412007808685302734375f,0.52669999355375718419480790544184856116771697998046875f,3.1843000175960352038373457617126405239105224609375f,-0.30189997106269050863858183220145292580127716064453125f,0.439500025223941381824488416896201670169830322265625f,2.07319996517449656181497630313970148563385009765625f,-1.0922000041857369279085787638905458152294158935546875f,1.0787000066891303529104106928571127355098724365234375f,2.63400001535564687316082199686206877231597900390625f,3.101699939946831019454975830740295350551605224609375f,2.27980012773051754493280895985662937164306640625f,-0.4182999872796233642446850353735499083995819091796875f,-1.503700001883507386679639239446260035037994384765625f,-0.02419996540918845084888744167983531951904296875f,0.4629000035025168102720272145234048366546630859375f,-0.9289000181913380504994393049855716526508331298828125f,1.30949988938122974246880403370596468448638916015625f,0.5159000644072875285672807876835577189922332763671875f,0.293199921514093464480765760526992380619049072265625f,-1.99029994872659443672091583721339702606201171875f,-1.3575000580400244931666975389816798269748687744140625f,0.515000031813977354744338299497030675411224365234375f,-0.6697998973399410527207464838284067809581756591796875f,-2.25150000650808213009668179438449442386627197265625f,-2.053099935767800143793238021316938102245330810546875f,-1.964199980496615705050089673022739589214324951171875f,-1.1137000825732943365409255420672707259654998779296875f,0.1789000455722218330123496343730948865413665771484375f,-2.73180002618283079840466598398052155971527099609375f,-2.2796999293446535972407218650914728641510009765625f,1.8205000020451838071977590516326017677783966064453125f,-2.322900061374902680455534209613688290119171142578125f,-2.790399959897996495783445425331592559814453125f,-1.42679999567270243687744368799030780792236328125f,-1.8545999186947927928059698388096876442432403564453125f,0.286099883200228166657552719698287546634674072265625f,-3.00230001965984616418836594675667583942413330078125f,2.124099925497173746435919383657164871692657470703125f,-2.53629999606758271823991890414617955684661865234375f,-2.74569997389167408385901580913923680782318115234375f,-0.8092999662056563625611715906416065990924835205078125f,-0.3903000720709550730447290334268473088741302490234375f,-1.9858999907210463931050981045700609683990478515625f,-3.30519994060993127504843869246542453765869140625f,1.38709994335621633609889613580889999866485595703125f,-2.59980006326436896557652289629913866519927978515625f,-1.799300043207406218925825669430196285247802734375f,0.0627000573977831709981956009869463741779327392578125f,-0.56980008013993443949374295698362402617931365966796875f,-0.28559999654590983997337616528966464102268218994140625f,-1.12520003839880278206919683725573122501373291015625f,0.7635000138506295996876360732130706310272216796875f,2.10649992637708916021210825419984757900238037109375f,2.5639999756067997083164300420321524143218994140625f,-0.3815998964399096582411630151909776031970977783203125f,1.014600030900538296663171422551386058330535888671875f,1.7538999583631758039103942792280577123165130615234375f,-0.7681000472843664272915020774235017597675323486328125f,-1.060300054301321903693633430521003901958465576171875f,1.113499891586602785764625878073275089263916015625f,3.00319994032680970263982089818455278873443603515625f,0.1179999679028984616024899878539144992828369140625f,1.1051000193856646003354171625687740743160247802734375f,-2.810899963638184839709310836042277514934539794921875f,0.175900033543258782486873315065167844295501708984375f,-0.17670000210329794843033823781297542154788970947265625f,2.040600023144482744186234413064084947109222412109375f,1.9399999443441624524808730711811222136020660400390625f,-0.8555000631958236123608685375074855983257293701171875f,-2.983600069017707578922227185103110969066619873046875f,0.04769994031190794014918310494977049529552459716796875f,-0.64699997113272711413145543701830320060253143310546875f,-0.3031999725133178724689742011833004653453826904296875f,2.4984999675825232401393805048428475856781005859375f,1.5651000262960803599554537868243642151355743408203125f,-2.1387000073224289309337109443731606006622314453125f,0.1135999968960887951396898642997257411479949951171875f,-0.4304000401593750524398274137638509273529052734375f,2.40279999860823156865308192209340631961822509765625f,-0.38159999484345286102637828662409447133541107177734375f,1.835800079014896279971935655339621007442474365234375f,0.081000006753951137028479934087954461574554443359375f,1.83609998992979495113786470028571784496307373046875f,-0.19419999172836466261315990777802653610706329345703125f,0.503699960011244396440588388941250741481781005859375f,-1.431900034094601981138339397148229181766510009765625f,0.9671000271379954771333586904802359640598297119140625f,1.26420007897466302182465369696728885173797607421875f,-3.99049997919052845674059426528401672840118408203125f,-3.69699999968707526676325869630090892314910888671875f,-1.65950000527501106972749767010100185871124267578125f,0.866299915526806518784042054903693497180938720703125f,1.1984000654689968001775923767127096652984619140625f,0.130800036174058387228313904415699653327465057373046875f,-2.811499943271278834089343945379368960857391357421875f,-0.127900006985664338454711241865879856050014495849609375f,1.1359000459671022564833720025490038096904754638671875f,1.5524999709613613863012915317085571587085723876953125f,-1.5704000144734988619887872118852101266384124755859375f,0.52350004450976772574222195544280111789703369140625f,0.83089992493242181836876625311560928821563720703125f,-2.182099930354951311528566293418407440185546875f,-1.069599961125851539378572852001525461673736572265625f,0.49480000447928862428170759812928736209869384765625f,1.425699970110503755904574063606560230255126953125f,1.514799981009959051192481638281606137752532958984375f,0.5925999459788211520816503252717666327953338623046875f,-0.139200010205805302376802501385100185871124267578125f,-1.2218999750301253737205797733622603118419647216796875f,-2.41750007489696105977827755850739777088165283203125f,3.264400092563032984571691486053168773651123046875f,0.51210000074431238914485220448113977909088134765625f,1.3449000067658725132702102200710214674472808837890625f,6.742200061294436608250180142931640148162841796875f,1.53359993826001872463393738144077360630035400390625f,-1.769599925661088146711108493036590516567230224609375f,0.03640003713443895616563850126112811267375946044921875f,-2.32749988686293374229308028589002788066864013671875f,1.3165000563338387085110525731579400599002838134765625f,0.1181001587636785199464384277234785258769989013671875f,0.49809987521320575609706793329678475856781005859375f,-0.232899998529255913837232583318836987018585205078125f,0.895700002762675584477847223752178251743316650390625f,1.41889994647949979622580940485931932926177978515625f,-0.74649995422363335517701443677651695907115936279296875f,-0.5772999005809438255454324462334625422954559326171875f,0.43100004518032053457687879927107132971286773681640625f,0.49670000046417162042899917651084251701831817626953125f,1.1780999897658828157176458262256346642971038818359375f,2.259999989345668591766980171087197959423065185546875f,2.0346000760324312039983851718716323375701904296875f,3.6888000100746740628210318391211330890655517578125f,4.39400011672079582325523006147705018520355224609375f,1.0321999565206481097590085482806898653507232666015625f,3.390899963004887052164804117637686431407928466796875f,2.138899982242286501588068858836777508258819580078125f,0.7953000433862207341206840283120982348918914794921875f,1.321599928775430754512854036875069141387939453125f,-2.023000011134892250908023925148881971836090087890625f,0.948600113385915211239307609503157436847686767578125f,-0.014000107631087388426749384962022304534912109375f,2.192099927877634524264749416033737361431121826171875f,1.876699996981025631015427279635332524776458740234375f,3.31709999828562107637708322727121412754058837890625f,4.099499934904276443603521329350769519805908203125f,-0.59620003036335134094514387470553629100322723388671875f,2.487999921206384357219576486386358737945556640625f,0.09719994782209351313895240309648215770721435546875f,-2.4228999732345339879202583688311278820037841796875f,-3.70800008688866977735187901998870074748992919921875f,0.7630000505857175152613081081653945147991180419921875f,2.05920000532567382123261268134228885173797607421875f,0.88090003017186990774689547833986580371856689453125f,-3.13949996370077233365236679674126207828521728515625f,-0.0512000471830365011527419483172707259654998779296875f,-1.2820999443992977528949950283276848495006561279296875f,1.2683000667959454244027028835262171924114227294921875f,-2.721000022232532788990511107840575277805328369140625f,0.69799999755620889541063434080570004880428314208984375f,0.3073999642416833655289565285784192383289337158203125f,0.5868999572604900460959242991521023213863372802734375f,-0.8341998731896287377907128757215104997158050537109375f,-2.091899995165316994416571105830371379852294921875f,2.51410009345188623086642110138200223445892333984375f,-1.3654000474981942137020496375043876469135284423828125f,-5.05340002186447367904520433512516319751739501953125f,0.255099961736798608313847580575384199619293212890625f,-0.271600025651603349530205377959646284580230712890625f,-1.6405999360471952286388841457664966583251953125f,-0.8245000956133001768222356986370868980884552001953125f,4.2618000263504693947425039368681609630584716796875f,-1.3384999146834015704143894254229962825775146484375f,-1.9310999967470774407729550148360431194305419921875f,0.6923000337228177869519640807993710041046142578125f,0.10959983806088613444984503075829707086086273193359375f,0.1944999705553047686379386504995636641979217529296875f,-2.52609995133578735249102464877068996429443359375f,1.1214999944940202691867625617305748164653778076171875f,-0.85449993535131252286873859702609479427337646484375f,2.571900037596374755111128251883201301097869873046875f,-1.354700117099285616717452285229228436946868896484375f,-1.619099941124765518196682023699395358562469482421875f,1.23859991806894509380754243466071784496307373046875f,3.787799967502057274515436802175827324390411376953125f,-3.473699959713220319912352351821027696132659912109375f,2.377400013639031417511660038144327700138092041015625f,1.9376999437764277356421871445490978658199310302734375f,-1.0039999925568701133471449793432839214801788330078125f,1.014699975351989191807433599024079740047454833984375f,-1.0822000239670275245629227356403134763240814208984375f,-3.04839997824132513670747357537038624286651611328125f,2.14560003107935148847218442824669182300567626953125f,-0.75479994403645367384569908608682453632354736328125f,-1.6895999546252189471573501577950082719326019287109375f,-1.4195000378340474345151278612320311367511749267578125f,4.2116999554440379682773709646426141262054443359375f,1.880299933266638134909953805617988109588623046875f,-0.37489995975270884454033648580661974847316741943359375f,-0.042800075162947592843920574523508548736572265625f,2.04130003190487485653648036532104015350341796875f,-0.8615000332929196869002907988033257424831390380859375f,0.8555999782174839030091106906183995306491851806640625f,0.179200003034621691000438659102655947208404541015625f,0.47109996910542140113875575480051338672637939453125f,2.349199922642856819265944068320095539093017578125f,0.3661999449424453434431825371575541794300079345703125f,2.24460001173391798801048935274593532085418701171875f,1.90659998670369379425437728059478104114532470703125f,-1.293099974901974036356477881781756877899169921875f,0.261300017897782677067652912228368222713470458984375f,-0.7294999917149527224324856433668173849582672119140625f,1.306599938684701811553168226964771747589111328125f,0.3946999333307152912908577491180039942264556884765625f,-0.3716999427035452274736826439038850367069244384765625f,-0.2904000017516319420707304743700660765171051025390625f,-1.644599955016375236738213061471469700336456298828125f,1.8814000506013623947865198715589940547943115234375f,0.012800039549171859931675498955883085727691650390625f,0.15190005758255697987380017366376705467700958251953125f,-0.31889994126409160202229031710885465145111083984375f,-1.6855999981105327378827496431767940521240234375f,-0.567099932478368540245128315291367471218109130859375f,-1.5382000130444752894476323490380309522151947021484375f,-1.4332999377891415537789043810334987938404083251953125f,-2.8342999353230009518256338196806609630584716796875f,-0.67209996716082198719988127777469344437122344970703125f,-0.002300043185055056227383829536847770214080810546875f,-4.31040004081129968227514837053604423999786376953125f,0.73510000413060172519408297375775873661041259765625f,0.6381999025896174515537495608441531658172607421875f,0.4489000381402672434916212296229787170886993408203125f,-0.489500002406537404464614837706903927028179168701171875f,1.3287000034295022299346555882948450744152069091796875f,0.52559999192655137978391621800255961716175079345703125f,1.9604000442758191535830292195896618068218231201171875f,2.177599934206903231626029082690365612506866455078125f,-1.6183999042570604842694592662155628204345703125f,-0.88250003490597095368030977624584920704364776611328125f,0.9053000613793733553080755882547236979007720947265625f,-0.73450003703311061986624963537906296551227569580078125f,0.8716000359334052927096081475610844790935516357421875f,-1.215799970832467380432717618532478809356689453125f,0.9589000063449149013905525862355716526508331298828125f,0.67590000644177050670435846768668852746486663818359375f,-0.765000066198407768780498372507281601428985595703125f,0.8939000409156083737372000541654415428638458251953125f,-3.008999994251877918571835834882222115993499755859375f,-3.907199930480867777760067838244140148162841796875f,-1.20650006654113628457025697571225464344024658203125f,0.354199980757385457508235049317590892314910888671875f,1.245199946346878139280534014687873423099517822265625f,-0.14749999430030502622912536025978624820709228515625f,2.3627999895930287266310187987983226776123046875f,-0.86260002493634857501092483289539813995361328125f,-3.192199935416876410698705512913875281810760498046875f,4.525100048163533728029506164602935314178466796875f,0.714899918662756928000590050942264497280120849609375f,2.21489998651891983882933345739729702472686767578125f,0.90949998027831302493950715870596468448638916015625f,3.7969999565482144276984399766661226749420166015625f,2.3076999411873515555271296761929988861083984375f,1.56859993390142893332495077629573643207550048828125f,-0.7455000834614036175906903736176900565624237060546875f,0.021900006918610603889874255401082336902618408203125f,-0.732799994696676559868819822440855205059051513671875f,0.410199969648569418012584719690494239330291748046875f,4.3891000179216295151718441047705709934234619140625f,1.025099945326894523844885043217800557613372802734375f,-0.325100029239058674335183241055347025394439697265625f,-0.60559988124817498711394136989838443696498870849609375f,0.05329999031573573464726223392062820494174957275390625f,-1.169699980492890034611264127306640148162841796875f,-1.1422999698154632230995275676832534372806549072265625f,-1.2730000050626681940002526971511542797088623046875f,2.11869999640733031043282608152367174625396728515625f,1.957400031352787284077976437401957809925079345703125f,-0.57000000407919182432436855378909967839717864990234375f,-2.73210003632083697766574914567172527313232421875f,-1.606999992981553493365254325908608734607696533203125f,-0.609699978015571186773513545631431043148040771484375f,-0.3280000021010638366902867346652783453464508056640625f,-0.98839998058825617732736645848490297794342041015625f,0.719399950927495535069056131760589778423309326171875f,1.4140000875890257336919830777333118021488189697265625f,-0.149100027458369144284233698272146284580230712890625f,-0.030399963586031919504648612928576767444610595703125f,2.44159991786032914973247898160479962825775146484375f,1.5358000152379280844883169265813194215297698974609375f,-0.62420003788471267736071013132459484040737152099609375f,1.6215000641196961606027571178856305778026580810546875f,-0.5128000255793325745656829894869588315486907958984375f,-0.9359000539392212658640346489846706390380859375f,2.860599992187321305436853435821831226348876953125f,2.3483999335378431538856602855958044528961181640625f,1.6404999819472425581778907144325785338878631591796875f,-4.07230004428401581861862723599188029766082763671875f,-0.8755000078007568742322064281324855983257293701171875f,-2.11969998727291741857925444492138922214508056640625f,-2.521999980557710241413360563456080853939056396484375f,-3.29079997694194492652286498923785984516143798828125f,-0.7296000428214650668223839602433145046234130859375f,-1.7551000478282581251932015220518223941326141357421875f,-3.7766999628573643121853820048272609710693359375f,1.1971999765068292997938215194153599441051483154296875f,0.9785000946149242651017630123533308506011962890625f,-0.823200000490249550466614891774952411651611328125f,0.03420000303462078505845056497491896152496337890625f,-0.809799989205598702568522639921866357326507568359375f,-2.754600013000517488848117864108644425868988037109375f,-1.6874998435564345644621653264039196074008941650390625f,-1.70889999835416706019941557315178215503692626953125f,-1.947499975766986413105996689409948885440826416015625f,-2.026200019112975869717274690628983080387115478515625f,-2.518500038791447348529572991537861526012420654296875f,0.65250007908791207711374227073974907398223876953125f,-1.9755999386921505589498337940312922000885009765625f,-1.6168000388488172891499061734066344797611236572265625f,-0.421700098159910552464424426943878643214702606201171875f,1.12149998160451591644459767849184572696685791015625f,-0.39889999326914449273573382015456445515155792236328125f,-2.2226000198699527032886180677451193332672119140625f,-0.734100039751826471956519526429474353790283203125f,0.5560999986469727929261352983303368091583251953125f,-0.5232000435546033667577603409881703555583953857421875f,-0.74129998911991723531400566571392118930816650390625f,-0.58289997152090011756087051253416575491428375244140625f,-0.55360005836337666806912238826043903827667236328125f,1.9780000997409217244893397946725599467754364013671875f,1.0407999314934015533395950114936567842960357666015625f,0.22989987435042902319537461153231561183929443359375f,1.349000039532780359508024048409424722194671630859375f,-2.82219996998757149953007683507166802883148193359375f,0.94800005066022041688711397000588476657867431640625f,-3.51679996063634892067284454242326319217681884765625f,1.683899908015878299494261227664537727832794189453125f,0.564700053750722741341405708226375281810760498046875f,-0.82020002666413827085989396437071263790130615234375f,-0.28200001939386165616951984702609479427337646484375f,1.29439998154938251673229387961328029632568359375f,2.0104998301975438579347610357217490673065185546875f,2.4596002017423526098127695149742066860198974609375f,-2.28709997695833333608561588334850966930389404296875f,0.6740001134425395168392469713580794632434844970703125f,0.7997999914407747379385682506836019456386566162109375f,-1.848999932467936258007057404029183089733123779296875f,-1.4174999490752806519111572924884967505931854248046875f,1.949700031305849901031024273834191262722015380859375f,0.5609000198341902443388562460313551127910614013671875f,5.6091000449672332450745670939795672893524169921875f,-1.622200064013897158332611070363782346248626708984375f,0.261100000006704302535354145220480859279632568359375f,-0.4709000228330475490423623341484926640987396240234375f,-0.52949999912828016324084501320612616837024688720703125f,-0.5722998678728947741234378554509021341800689697265625f,-2.0895999467089776402417555800639092922210693359375f,0.68879996354579997586142781074158847332000732421875f,2.772000043831764681812046546838246285915374755859375f,1.32879997982531694589170001563616096973419189453125f,-1.563800063830613762405619127093814313411712646484375f,3.006899964804201630386160104535520076751708984375f,2.67430009716823757770498559693805873394012451171875f,0.42849994958937143163524297051480971276760101318359375f,-2.370000004656613867837222642265260219573974609375f,0.21790009352415840027106241905130445957183837890625f,3.16750000651925933681241076556034386157989501953125f,1.240899953151493928515947118285112082958221435546875f,0.01079996157512008725376517759286798536777496337890625f,-4.27460004066080134776939303264953196048736572265625f,-2.638699996630848421119708291371352970600128173828125f,-1.55079993931651216598766040988266468048095703125f,-1.675899929812551203411885580862872302532196044921875f,0.22779999387711313829640857875347137451171875f,-1.608799949054421585259433413739316165447235107421875f,0.2620999327950166435385881413822062313556671142578125f,3.59199998700246236893462992156855762004852294921875f,-0.55880003454983206978567977785132825374603271484375f,-1.0231000236660221514028989986400119960308074951171875f,2.0415001353099935244017615332268178462982177734375f,0.5991998458087433032659419041010551154613494873046875f,0.50949996032938560386327253581839613616466522216796875f,-0.195999966353177457190071208970039151608943939208984375f,-0.039100042881073715506090593407861888408660888671875f,2.1591000189647076012988691218197345733642578125f,-0.267499960921706136929287822567857801914215087890625f,-1.004700082193312749012648055213503539562225341796875f,0.9005000776126992345638200276880525052547454833984375f,-2.097500132992862642566933573107235133647918701171875f,0.01119987716078674200304021724150516092777252197265625f,2.361100036887080033665142764220945537090301513671875f,-5.5831000748515133835780943627469241619110107421875f,3.219499870494009652333033955073915421962738037109375f,-1.292100030490756612522318391711451113224029541015625f,-0.0640999067217122853179489538888446986675262451171875f,0.57140007250607016686672068317420780658721923828125f,1.3787999959930774540595166399725712835788726806640625f,0.056700017991661244565193555899895727634429931640625f,-0.23030004942118953437102391035296022891998291015625f,-3.370699838253854441205703551531769335269927978515625f,0.02990004757642472643652808983461000025272369384765625f,2.739500035822391321715940648573450744152069091796875f,-0.42209990446418588216914713484584353864192962646484375f,4.88729999065846154593373285024426877498626708984375f,0.0407000453799966077639282957534305751323699951171875f,0.670200103591381601830789804807864129543304443359375f,-4.19220001379698459231804008595645427703857421875f,2.193600149074197869225599788478575646877288818359375f,-2.753900004240126708765501462039537727832794189453125f,-1.22039998323693854587190799065865576267242431640625f,-0.191900079505890552855618125249748118221759796142578125f,-2.188400102740526875777504756115376949310302734375f,-2.299599986159801101592847771826200187206268310546875f,2.74829997496753808405856034369207918643951416015625f,0.378799899955094776515807097894139587879180908203125f,0.031499946046622095963130050222389400005340576171875f,-1.711299834352729920539104568888433277606964111328125f,1.902300005187092413194704931811429560184478759765625f,-0.5675999592781051372725187320611439645290374755859375f,3.934599973307550957457578988396562635898590087890625f,1.36430004838555962720647585229016840457916259765625f,4.0130000385344022362232863088138401508331298828125f,0.5954000336959968109340479713864624500274658203125f,6.51299987257272139373753816471435129642486572265625f,0.181500071607529012140958002419210970401763916015625f,0.8751999742865554932080840444541536271572113037109375f,-0.37870006975829351159035240925732068717479705810546875f,1.389699989582599215509617351926863193511962890625f,2.449700053471328686782726435922086238861083984375f,1.269300126653907678786481483257375657558441162109375f,-1.281799876388905801150031038559973239898681640625f,0.252899969397484547517507280645077116787433624267578125f,3.05149995958805053675177987315692007541656494140625f,0.166200050647556440974739189186948351562023162841796875f,-3.07060001133531290662403989699669182300567626953125f,2.1234999404065302286426231148652732372283935546875f,3.23639996886849434787336576846428215503692626953125f,0.5055001067444653273241783608682453632354736328125f,1.6334000983446828048073484751512296497821807861328125f,-0.3039001164272410182576322767999954521656036376953125f,1.7314999288916601560828212313936091959476470947265625f,-1.370600144365429429171854280866682529449462890625f,-0.12539993950203209838178963764221407473087310791015625f,0.6460999792195838420383324773865751922130584716796875f,-0.16950002773851247095393546260311268270015716552734375f,1.457100046416373029245505676954053342342376708984375f,1.318299972303955680530407335027121007442474365234375f,-2.89939995763301805453693305025808513164520263671875f,-1.187000042192635884674700719187967479228973388671875f,0.1708999533936388370847225814941339194774627685546875f,-1.091600090229510211514707407332025468349456787109375f,-2.908999823689460750841817571199499070644378662109375f,-0.51930003298148630364039490814320743083953857421875f,-0.6559999807327996368400135906995274126529693603515625f,-1.833499989636241611634659420815296471118927001953125f,2.52900000850111172923107005772180855274200439453125f,-1.950899903325737483328339294530451297760009765625f,0.854299925544115268394307349808514118194580078125f,0.0512999111309644373335459022200666368007659912109375f,0.5689000668063777244043421887909062206745147705078125f,-3.129000104092062084504277663654647767543792724609375f,0.88729989648312113104111631400883197784423828125f,-0.128100051382185053938655983074568212032318115234375f,0.4973000281721342563656662605353631079196929931640625f,-1.001299958721549199225364645826630294322967529296875f,-0.31779994669630962089712511442485265433788299560546875f,-1.571899991235139015799404660356231033802032470703125f,1.453099958883223852268429254763759672641754150390625f,0.47330003349185201244608833803795278072357177734375f,1.227399995943903920903039761469699442386627197265625f,-3.737399974039197214636942590004764497280120849609375f,-2.2990998918950555918172540259547531604766845703125f,-1.476599862353503045397928872262127697467803955078125f,-1.7984999513775090651535037977737374603748321533203125f,-1.394699962984027319379265463794581592082977294921875f,0.7948999083623309314106109013664536178112030029296875f,5.04109996731728404739669713308103382587432861328125f,0.2799000825569051276175969178439117968082427978515625f,1.60149994183704347250341015751473605632781982421875f,-1.375999902538956209951948039815761148929595947265625f,4.29649996113404863962159652146510779857635498046875f,0.64610005186274765964782318405923433601856231689453125f,-2.859900021834671957066120739909820258617401123046875f,-0.85079999294206742899149276126991026103496551513671875f,-1.1024999139644238166368950260221026837825775146484375f,-1.44219997226000007373158950940705835819244384765625f,2.2469000144250710349069777294062077999114990234375f,1.8517000425226963766789367582532577216625213623046875f,-0.7560999319829040388185603660531342029571533203125f,0.90889985824599772712417689035646617412567138671875f,-2.418399958683550732274625261197797954082489013671875f,3.6472998551696544922151588252745568752288818359375f,-1.0057000440761445503312643268145620822906494140625f,-2.29859997916370684123421597178094089031219482421875f,1.0652999733321391051532600613427348434925079345703125f,3.35949992262944530097001916146837174892425537109375f,0.8335999869868164768860197000321932137012481689453125f,-2.9978999438792452991719983401708304882049560546875f,-0.2783998813465251576104719788418151438236236572265625f,-1.36310002511888850307286702445708215236663818359375f,1.895900078544765943178163070115260779857635498046875f,-0.590800047275423789727710754959844052791595458984375f,-1.0256000331282606463645379335503093898296356201171875f,1.4190000593885769841762112264405004680156707763671875f,-0.9229999788552529782492683807504363358020782470703125f,2.557799953625352173958162893541157245635986328125f,0.944499936282635843554089660756289958953857421875f,-5.12929996095299767233655074960552155971527099609375f,-2.6778999392040052640595604316331446170806884765625f,-1.558900028547644200926924895611591637134552001953125f,-1.250400014138221660431327109108678996562957763671875f,-1.10249999817460686557524240924976766109466552734375f,-1.057300013829767681983184957061894237995147705078125f,-4.73379999234229398297202351386658847332000732421875f,0.363099956014751423793995854794047772884368896484375f,0.26790004155635804838908597957924939692020416259765625f,0.3543999825924657454123689603875391185283660888671875f,-0.7537000930599864201298032639897428452968597412109375f,-1.121799997013810834545211037038825452327728271484375f,3.0257000916667298184847822994925081729888916015625f,1.06810006907731303016362289781682193279266357421875f,-1.916700033265351432731904424144886434078216552734375f,4.4225999508962043904602978727780282497406005859375f,-1.08120001145750332938177962205372750759124755859375f,0.041900100460647404787550840410403907299041748046875f,-1.0829000211045152202160579690826125442981719970703125f,-2.638700048449637680647583692916668951511383056640625f,-3.6924000673204648848013675888068974018096923828125f,1.84119998830482334284397438750602304935455322265625f,-2.3273999755479390927348504192195832729339599609375f,-2.40120001408383387797584873624145984649658203125f,2.373500095713883428771850958582945168018341064453125f,0.0592999985784297134472353718592785298824310302734375f,-1.387499956190586747339921203092671930789947509765625f,-2.2405999681592003724972528289072215557098388671875f,-1.740799982287736469288574880920350551605224609375f,-0.233400018791109165050556839560158550739288330078125f,0.1975999690569938405104721823590807616710662841796875f,0.417500010598450099763567777699790894985198974609375f,0.469700052428245751201529856189154088497161865234375f,-2.40289991362988875067685512476600706577301025390625f,2.768499900229272725482587702572345733642578125f,0.03870001639351239308695085128420032560825347900390625f,-1.1709000213243074828284306931891478598117828369140625f,2.606599928458781079143591341562569141387939453125f,-0.72669995663613118352941455668769776821136474609375f,-0.4467001029282797741615240738610737025737762451171875f,-0.9751000081911678218915540128364227712154388427734375f,0.7231000313028699277850819271407090127468109130859375f,-2.70580003012046166332993379910476505756378173828125f,-1.467499886080625426387769039138220250606536865234375f,-1.023800038237869802770774185773916542530059814453125f,-1.296500028021633443842119959299452602863311767578125f,-1.227899975357950257404127114568836987018585205078125f,-2.15150005201250138497925945557653903961181640625f,-2.25190006689578314791333468747325241565704345703125f,-0.965499991931020762336856932961381971836090087890625f,-0.195799966883658527194711496122181415557861328125f,5.16350002364814297806105969357304275035858154296875f,-0.7680999529600149866581659807707183063030242919921875f,-0.8505000379011027344944295691675506532192230224609375f,2.32739995341971539488667986006475985050201416015625f,-0.628100070455670955738014527014456689357757568359375f,0.97289990770667600372689776122570037841796875f,-0.104699981386959795059965472319163382053375244140625f,1.5851998332098122812539031656342558562755584716796875f,2.01330001743585018658677654457278549671173095703125f,-0.1785998888254172189959945171722210943698883056640625f,-2.599299997255951399921514166635461151599884033203125f,-1.966000067979099430459655195591039955615997314453125f,0.7045000676177444187686660370673052966594696044921875f,-1.86480006875917414532750626676715910434722900390625f,-3.98680006954521015671843997552059590816497802734375f,-1.95489995310306508002895498066209256649017333984375f,-0.867299999263882792632784912711940705776214599609375f,-1.6198000219881532046883876319043338298797607421875f,3.54860004815608220951617113314568996429443359375f,0.7854999291412518847010915123973973095417022705078125f,2.737300005839021554976397965219803154468536376953125f,3.351099967764318510177190546528436243534088134765625f,-4.090299927771837928958120755851268768310546875f,-0.550000058040023276362262549810111522674560546875f,-1.383800014377385867447856071521528065204620361328125f,1.509400046257673100313922986970283091068267822265625f,1.85479992852062114394584568799473345279693603515625f,1.351499917026609320913621559157036244869232177734375f,-0.656099918851258845364782246178947389125823974609375f,0.5074999807961277920043130507110618054866790771484375f,-1.2993999526411290634797524035093374550342559814453125f,-1.6202001756757498096561675993143580853939056396484375f,-1.4797999152958396518187100809882394969463348388671875f,-2.40330005692392578708904693485237658023834228515625f,0.2414000929579132037616773232002742588520050048828125f,-1.491499964058398131072635806049220263957977294921875f,-2.42080000547766704954710803576745092868804931640625f,0.509799988721311603256935995887033641338348388671875f,-0.4099999792128807829527659123414196074008941650390625f,-0.8234000308796767253483039894490502774715423583984375f,0.1984000704608848142385113533237017691135406494140625f,-5.00000000268220912147398848901502788066864013671875f,2.97000010151415860804036128683947026729583740234375f,0.8349999231845159908260711745242588222026824951171875f,0.490499946221708338356393142021261155605316162109375f,3.453899987066536336755007141618989408016204833984375f,0.238700008663537577291435809456743299961090087890625f,-1.791999934419989504164050231338478624820709228515625f,0.964699902205915282848991409991867840290069580078125f,-0.0265999214552319340754138465854339301586151123046875f,-1.136300047774613464213189217844046652317047119140625f,-1.9614999023303403280493739657686091959476470947265625f,0.2153000022590150219770066541968844830989837646484375f,-0.65690002887919263230287469923496246337890625f,-0.340200068648158993056540566612966358661651611328125f,-4.25380004738345629533569081104360520839691162109375f,0.8377999009869994129218184752971865236759185791015625f,-0.7994000520505011309069232083857059478759765625f,4.710399957551061334015685133635997772216796875f,-0.829900073802470661377128635649569332599639892578125f,0.74340009037256160606688126790686510503292083740234375f,1.716999903723597231675057628308422863483428955078125f,5.08320000697225093944098261999897658824920654296875f,-0.5009999057799576771543570430367253720760345458984375f,1.159800105416028070948186723398976027965545654296875f,-3.6205000960901383422196886385791003704071044921875f,-1.3300999355480109809235500506474636495113372802734375f,-5.5689999978095290344981549424119293689727783203125f,-0.86280003835708196646692158537916839122772216796875f,-5.48919996889233541281782891019247472286224365234375f,0.89179995145350599017319837003014981746673583984375f,1.831599922777711331178807085962034761905670166015625f,1.250900064034759839870503128622658550739288330078125f,-3.076900034299490105382801630184985697269439697265625f,-2.368599985794725082399736493243835866451263427734375f,1.941500063430517730722613123361952602863311767578125f,4.116200115393102265670677297748625278472900390625f,-3.891899868896602665557793443440459668636322021484375f,0.89249989813193764121024287305772304534912109375f,3.22169994603767850804842964862473309040069580078125f,1.9798999905794847364859379013068974018096923828125f,-1.0185000024698662723920961070689372718334197998046875f,0.4994999181777235452983632058021612465381622314453125f,-2.052400051450728568624981562606990337371826171875f,-2.922199954117834952427301686839200556278228759765625f,0.6682000911638137363723899397882632911205291748046875f,-0.8549000342398895835316352531663142144680023193359375f,-1.6891999892696747398446177612640894949436187744140625f,3.34170003089979417154609109275043010711669921875f,1.047700004331023393433497403748333454132080078125f,0.886199916741998716673833769164048135280609130859375f,1.4293000690981749745134266049717552959918975830078125f,-0.561000021226705225529940435080789029598236083984375f,1.5811000748850394526101581504917703568935394287109375f,1.67029993090107797826249225181527435779571533203125f,1.955399942165613413891378513653762638568878173828125f,-1.1046999429419646077121797134168446063995361328125f,-0.334799994830787872590605047662393189966678619384765625f,-4.70080004576668120108706716564483940601348876953125f,-2.6215999964639546959688232163898646831512451171875f,0.70209991564005680420024191334960050880908966064453125f,-2.67780011565685160945804454968310892581939697265625f,1.503599994847178322743275202810764312744140625f,0.7472000091031183455214659261400811374187469482421875f,2.383300088458508270150559837929904460906982421875f,-0.59190002796650109662124350506928749382495880126953125f,-0.24020005821734657924793054917245171964168548583984375f,-0.33700005459785475903089491112041287124156951904296875f,3.54419992478489920273432289832271635532379150390625f,-3.285599967339635885110737945069558918476104736328125f,-0.0691000108435739779366713264607824385166168212890625f,0.65269993032813022892923981999047100543975830078125f,0.10669999217540204750065413463744334876537322998046875f,-0.7331000083364553887577130808494985103607177734375f,-1.6913999878115972830272539795259945094585418701171875f,4.44409987716153498382709585712291300296783447265625f,1.2414000073879936092424713933723978698253631591796875f,-2.150799925644695687054763766354881227016448974609375f,-0.728299917337298818864610439050011336803436279296875f,-2.970099909489602563184007522067986428737640380859375f,0.07540002084374552993040197179652750492095947265625f,-1.848499998129903065802182027255184948444366455078125f,-0.7792999981686483668141818270669318735599517822265625f,2.657800045919419318352083791978657245635986328125f,0.149899971040337476324566523544490337371826171875f,1.1210999538317334422998783338698558509349822998046875f,2.2678000492162997403511326410807669162750244140625f,1.0947999915525310488106924822204746305942535400390625f,-2.853200008536877163578537874855101108551025390625f,-2.639799931407722066722953968564979732036590576171875f,-0.17310002403855317965053473017178475856781005859375f,0.01239998130351249727709728176705539226531982421875f,-1.1796000236734747179667692762450315058231353759765625f,2.372499955408276406387813040055334568023681640625f,-0.72900001312047357782120116098667494952678680419921875f,-1.130699917602538295824388114851899445056915283203125f,1.624199929031730160744473323575221002101898193359375f,1.7360000136643660706425862372270785272121429443359375f,0.58280001112520773975944621270173229277133941650390625f,1.10330001341253502999961710884235799312591552734375f,4.39709999936595519187676472938619554042816162109375f,-0.7875000528246165476531359672662802040576934814453125f,-0.2435000038668502231331558505189605057239532470703125f,1.924399956068397532504832270205952227115631103515625f,0.35509991058856271717303343393723480403423309326171875f,2.63849995972216078143901540897786617279052734375f,2.67300003080442660774451724137179553508758544921875f,-1.5437999701581899270053099826327525079250335693359375f,2.8430999692022798086554757901467382907867431640625f,-3.65490002529919166818217490799725055694580078125f,0.9556000765651464501360123904305510222911834716796875f,-0.083600111299754420457475134753622114658355712890625f,2.514999883268030256289193857810460031032562255859375f,-2.614600042542071012263704687939025461673736572265625f,0.1761999887704830403123423820943571627140045166015625f,0.2521000820115195661941243088222108781337738037109375f,-0.71809999442249505818836041726171970367431640625f,-1.2702999559909102789134749400545842945575714111328125f,0.340500042036175454285285013611428439617156982421875f,-0.8433000129841270453567858567112125456333160400390625f,-0.080299976852535703830682223269832320511341094970703125f,-1.6663999597415337650119226964307017624378204345703125f,-2.29619994719624376244837549165822565555572509765625f,1.160200091558696744442613635328598320484161376953125f,2.063500104375183052951570061850361526012420654296875f,-2.990199868227541468712615824188105762004852294921875f,-3.789999961368739622713519565877504646778106689453125f,-0.87529997726232000587742732022888958454132080078125f,-0.67549988477304478973195500657311640679836273193359375f,-0.96209999726116623985916476158308796584606170654296875f,2.810800082702936020240258585545234382152557373046875f,-0.5517000577591366639040870722965337336063385009765625f,-2.06160005424320669220605850568972527980804443359375f,2.986199937659500758257991037680767476558685302734375f,0.5488999736741175805576631319127045571804046630859375f,-5.03719992047846343297123894444666802883148193359375f,1.3591999245241284111074264728813432157039642333984375f,0.8149999863281853773600005297339521348476409912109375f,-0.84529990666806675836397744205896742641925811767578125f,-0.9446999554589374792357148180599324405193328857421875f,-0.9804998264163720111952216029749251902103424072265625f,0.64610009220763864323799907651846297085285186767578125f,-1.97829999724477278988388206926174461841583251953125f,-1.248499993361532833802129971445538103580474853515625f,-0.09410003405213362981385216698981821537017822265625f,0.2651000453695646630336568705388344824314117431640625f,-0.61330004215315003790465198107995092868804931640625f,2.13649992566928315085306167020462453365325927734375f,-3.8555000036284337028291702154092490673065185546875f,-1.810900094619392941552860065712593495845794677734375f,-1.032200008227676324423782716621644794940948486328125f,-1.4732999859571460632423622882924973964691162109375f,-2.40730006419569075859499207581393420696258544921875f,2.08870008626133074614017459680326282978057861328125f,1.716599981339276492775525184697471559047698974609375f,-0.2888999344095586341296666432754136621952056884765625f,1.2289999219998717361335138775757513940334320068359375f,-1.258799985897539830403957239468581974506378173828125f,2.973500077962876186887797302915714681148529052734375f,1.8717999656096111493752687238156795501708984375f,0.061599971430004618611064870492555201053619384765625f,-0.0230999391578131374469506909008487127721309661865234375f,3.9419000512495614430008572526276111602783203125f,-0.4027000099748370143259990072692744433879852294921875f,-0.68250009376555798024099885878968052566051483154296875f,-2.405000041052699710775186758837662637233734130859375f,-2.1309999011233440313617393258027732372283935546875f,-2.599400048492848558367995792650617659091949462890625f,5.559899944944678651381764211691915988922119140625f,-1.588299987335502816421239913324825465679168701171875f,1.010399972750246622155145814758725464344024658203125f,2.436799997013807672630036904592998325824737548828125f,-1.437600026202947578468638312187977135181427001953125f,1.3694999718405294064638155759894289076328277587890625f,-0.30010005658268912309694087525713257491588592529296875f,0.6536000842168945279553327054600231349468231201171875f,1.5464000598773350869663545381627045571804046630859375f,0.0256999720476581428041384924654266797006130218505859375f,0.4115999981775859684063334498205222189426422119140625f,-0.66919996619150257100017142875003628432750701904296875f,1.038999997492879856508807279169559478759765625f,-1.027399983147530004856662344536744058132171630859375f,0.9182000261202454272080331065808422863483428955078125f,-1.199299950745701170973234184202738106250762939453125f,0.3638999328635639241014132494456134736537933349609375f,3.202199885702878390247860806994140148162841796875f,2.632200046356023559468439998454414308071136474609375f,2.088599919894338352577278783428482711315155029296875f,-1.031199946376681442217204676126129925251007080078125f,0.138599969701471981142759659633156843483448028564453125f,-2.24160002220570930830945144407451152801513671875f,2.013599905030427184016161845647729933261871337890625f,1.7302999514460559726813926317845471203327178955078125f,3.08839985020309892860268519143573939800262451171875f,2.11559997233152241591369602247141301631927490234375f,0.0409000423163166093587506111362017691135406494140625f,-1.755099997723104099378588216495700180530548095703125f,-0.3348999679669739659715332891209982335567474365234375f,0.13169997131377453314371450687758624553680419921875f,-0.67789997839406013202534495576401241123676300048828125f,-4.21029999404475230306843513972125947475433349609375f,-1.8056999569043503672816086691454984247684478759765625f,-1.840299968396126306657833993085660040378570556640625f,0.7499999585188934503321434021927416324615478515625f,-4.48010002290606479391499306075274944305419921875f,2.03140000463128078678209931240417063236236572265625f,1.5331001253291969277370299096219241619110107421875f,-2.14809995694607636096407077275216579437255859375f,-1.02770003668516807238120236434042453765869140625f,0.0475999443583188242001824619364924728870391845703125f,-0.6250999171264466358621803010464645922183990478515625f,1.1407000120386487918011653164285235106945037841796875f,1.5412999407470220347704525920562446117401123046875f,-0.05329992434084374064440225993166677653789520263671875f,-1.05449990761652667714542985777370631694793701171875f,1.6947000521674742135047608826425857841968536376953125f,0.3953000687740750862531058373861014842987060546875f,-3.06750004209578097658095430233515799045562744140625f,3.75440001313984428321646191761828958988189697265625f,1.75530003755614227856085562962107360363006591796875f,-1.24390003514140889961936409235931932926177978515625f,2.05829997714683354814724225434474647045135498046875f,0.497499930635095122255506794317625463008880615234375f,-3.057299975608289255291083463816903531551361083984375f,-2.257599959743767481512577433022670447826385498046875f,-1.60880002817958622784999533905647695064544677734375f,-0.922100033470989455253175037796609103679656982421875f,1.545200071144102960118971168412826955318450927734375f,-0.19310004523545465371370255525107495486736297607421875f,-4.3242000074490913874569741892628371715545654296875f,-1.22730000757127921673372838995419442653656005859375f,-1.221399944132566250942772967391647398471832275390625f,1.931399989674240647019587413524277508258819580078125f,0.68120003222599756487198874310706742107868194580078125f,-0.730499938417226157838513245224021375179290771484375f,-0.623999962642788563726981010404415428638458251953125f,-0.3042000226110219696096237385063432157039642333984375f,3.278099889816342482617983478121459484100341796875f,-0.56399996132031116058413999780896119773387908935546875f,0.649599981279672444856032598181627690792083740234375f,0.408899914274365006150446788524277508258819580078125f,-1.5569001359626637448485553250066004693508148193359375f,-1.679800041247904829333492671139538288116455078125f,0.55580001048818150621855238568969070911407470703125f,0.3614000687435263703406462809653021395206451416015625f,-3.085900014004109959131483265082351863384246826171875f,2.24209999569654439000032652984373271465301513671875f,-0.946600014159081570852549702976830303668975830078125f,0.18259994094967757494174520616070367395877838134765625f,-0.6972999368280188292601451394148170948028564453125f,0.313800019015371134400993469171226024627685546875f,-1.4202999776721003666324349978822283446788787841796875f,3.55969993655309036029166236403398215770721435546875f,2.26739991316795386211424556677229702472686767578125f,0.66150001408904690602952314293361268937587738037109375f,5.00019999747648835608515582862310111522674560546875f,2.614399917362629111039495910517871379852294921875f,0.19779997038915608786879829494864679872989654541015625f,0.9733999215424062878554423150490038096904754638671875f,2.99380011423379155388602157472632825374603271484375f,2.814199986363945971135080981184728443622589111328125f,-1.6168000291258095568736052882741205394268035888671875f,-0.1953999507710333372045852229348383843898773193359375f,-0.1694999678730957359817921314970590174198150634765625f,-1.06210000514015323602734497399069368839263916015625f,0.446399937874078778232700415173894725739955902099609375f,-1.1919000164553523735122553262044675648212432861328125f,1.3031000446207829934763822166132740676403045654296875f,2.550199937331676824925352775608189404010772705078125f,-0.797900019260496140560690037091262638568878173828125f,-1.629499891914426346062327866093255579471588134765625f,1.592499958202242549276661520707421004772186279296875f,-2.077700076098739856433894601650536060333251953125f,-1.948300003410129210834611512837000191211700439453125f,-1.4468999809719633731219801120460033416748046875f,-0.0511000345215186779768146152491681277751922607421875f,-1.0535000010170041395696216568467207252979278564453125f,2.180199987772106329231291965697892010211944580078125f,2.637700007590652706568334906478412449359893798828125f,-4.0171000220343469067074693157337605953216552734375f,-1.6286999933525898587305391629342921078205108642578125f,-3.581200015331804653584413244971074163913726806640625f,-1.141100120929629202493060802225954830646514892578125f,0.45810006373152034608864369147340767085552215576171875f,-0.4589000439330941372162442348781041800975799560546875f,-2.76800008249282836203519764239899814128875732421875f,-3.18869990092813804238858210737816989421844482421875f,-0.23340001702159851593165740268887020647525787353515625f,1.2029000842481867294964104075916111469268798828125f,0.11809991876185021997258672854513861238956451416015625f,0.8576000564709309603728115689591504633426666259765625f,1.761799995933473805820312918513081967830657958984375f,-1.041100032174586687006012653000652790069580078125f,1.7635999616175883630830867332406342029571533203125f,1.3321999491259453041180904619977809488773345947265625f,1.432699951003491900536346292938105762004852294921875f,1.16960006249099901509680421440862119197845458984375f,-1.867899927115439506764005272998474538326263427734375f,0.0939001299872981309846409203601069748401641845703125f,1.1926999165445570927346352618769742548465728759765625f,0.4633999036982661134942418357240967452526092529296875f,0.679499965526164206863768413313664495944976806640625f,-1.393199945765733804847741339472122490406036376953125f,-0.18980003301501380263971441308967769145965576171875f,-1.273000037342309820331820446881465613842010498046875f,1.5604999972954398668179010201129131019115447998046875f,1.877499976120891034270243835635483264923095703125f,0.9789000351041554637276931316591799259185791015625f,0.87340000955238938917801760908332653343677520751953125f,-0.946900061810761695113569658133201301097869873046875f,-0.083499982073904188695223638205789029598236083984375f,-0.2357999795496461548083289017085917294025421142578125f,-3.404900034761428884166889474727213382720947265625f,2.33689995711147791013218011357821524143218994140625f,1.94570003688633530458673703833483159542083740234375f,-1.4806000435777011414728576710331253707408905029296875f,-0.1886999947495764384797212187550030648708343505859375f,2.797899940638242100021670921705663204193115234375f,-1.1137000565335153368096143822185695171356201171875f,-0.853200080770254221107506964472122490406036376953125f,0.409899896180629497166592045687139034271240234375f,1.278300031293927663256226878729648888111114501953125f,0.7660999023109678063292449223808944225311279296875f,-1.19270002740919611738945604884065687656402587890625f,3.53590005121976158619645502767525613307952880859375f,-0.3563000140048544484017156719346530735492706298828125f,1.582400009019673259302862788899801671504974365234375f,2.84269989031851277871965066879056394100189208984375f,-0.19560003134012049486045725643634796142578125f,-0.306900046034158269225855519835022278130054473876953125f,0.55789998821020247987689799629151821136474609375f,2.8930999839544302432159383897669613361358642578125f,-0.8809000087700791237210751205566339194774627685546875f,-0.162199956837296543898929712668177671730518341064453125f,-0.0138999752573685242396095418371260166168212890625f,0.0838000021770592962155888017150573432445526123046875f,1.6322999725721769603836719397804699838161468505859375f,-5.20350005508959423394799159723334014415740966796875f,3.121199967871607139358047788846306502819061279296875f,0.3135000329986208367216704573365859687328338623046875f,-2.429100071491302248460897317272610962390899658203125f,-1.5621999883905051120791540597565472126007080078125f,0.7490001831799750764417922255233861505985260009765625f,0.61700008151307639270299887357396073639392852783203125f,1.0494999473467470085807917712372727692127227783203125f,-3.213899864634871761381873511709272861480712890625f,0.28370007698535892881608333482290618121623992919921875f,-0.4637000131711364048214818467386066913604736328125f,-1.7314999891296005696261772754951380193233489990234375f,4.05350008900836211722662483225576579570770263671875f,1.7295000267326816612012407858856022357940673828125f,-0.089300015696138768106493444065563380718231201171875f,0.91980002422332896028223103712662123143672943115234375f,-5.725899998953938307977296062745153903961181640625f,0.7170001049265251058528747307718731462955474853515625f,-1.919899962230026790876991071854718029499053955078125f,1.20949998383596390993943714420311152935028076171875f,0.7941999454602584052054226049222052097320556640625f,0.391800085601210046348796822712756693363189697265625f,1.6306000404298313100071027292869985103607177734375f,1.539300020315498951362087609595619142055511474609375f,1.76210000825077184316569400834850966930389404296875f,6.17209991295784643483557374565862119197845458984375f,-0.39759995272159509571707758368575014173984527587890625f,-1.349799992595613229440232316846959292888641357421875f,-1.17099997328221672177051004837267100811004638671875f,-1.8350999186351903968983378945267759263515472412109375f,0.7376000466734178306893454646342433989048004150390625f,2.526599935480951852895259435172192752361297607421875f,2.263700093078611974561908937175758183002471923828125f,1.8700999527588493265284341759979724884033203125f,0.3143000524088730429639326757751405239105224609375f,0.65360003686845402626204304397106170654296875f,-1.555700060914456006599948523216880857944488525390625f,3.50280010222717930190583501826040446758270263671875f,1.3015998660042884527143769446411170065402984619140625f,-0.63240004006996830554498956189490854740142822265625f,-1.1657001191593707734028839695383794605731964111328125f,1.876900017498434269924700856790877878665924072265625f,3.42540002561211576193045402760617434978485107421875f,-1.0577000131413323291695860461913980543613433837890625f,3.28989994760826309772028253064490854740142822265625f,-0.5031998825289321075615589506924152374267578125f,1.59180005218535569611049140803515911102294921875f,0.0249999688938284947425927384756505489349365234375f,1.9784000060133644094406690783216618001461029052734375f,4.6860000418461851268148166127502918243408203125f,0.649199995211512170811829491867683827877044677734375f,-3.365600067438183184975741824018768966197967529296875f,0.23079995894879046236525255153537727892398834228515625f,-1.99650006847828453970805639983154833316802978515625f,-0.3871000539600839562837109042447991669178009033203125f,1.49030003928840226734564566868357360363006591796875f,1.48460000752433973758570573409087955951690673828125f,-0.605799880084395336865554781979881227016448974609375f,1.89880001525282882113287996617145836353302001953125f,2.15619997442811683896479735267348587512969970703125f,0.596200047797709853369951815693639218807220458984375f,-1.9766000518403956665025589245487935841083526611328125f,-2.305999959610402161302999957115389406681060791015625f,0.449200073815136935451164390542544424533843994140625f,-1.271900064623356474413640171405859291553497314453125f,0.2852000167548653930538193890242837369441986083984375f,-2.898800094676019067918559812824241816997528076171875f,-0.980199993602184793672904561390168964862823486328125f,0.28209994380325031926304291118867695331573486328125f,2.247699935692548223187259281985461711883544921875f,-1.413900025660544290673215073184110224246978759765625f,2.657399909144640748337451441329903900623321533203125f,-2.968500000663101712206071169930510222911834716796875f,-5.7159999619573351736789845745079219341278076171875f,-1.850599945099652199331785595859400928020477294921875f,2.871199982549248463925550822750665247440338134765625f,-1.603099955157934797256302772439084947109222412109375f,1.914100068362057260884512288612313568592071533203125f,1.873700003336370656370490905828773975372314453125f,-0.3273000454761094690780964810983277857303619384765625f,-0.7571999098986397225274913580506108701229095458984375f,1.51829998966380852465363204828463494777679443359375f,-0.6400999408751726349464661325328052043914794921875f,-2.074399889087676029930662480182945728302001953125f,1.5962999978929748312594938397523947060108184814453125f,5.379000014759601100422514718957245349884033203125f,4.119899965657292995047100703231990337371826171875f,1.5147000905752190380582078432780690491199493408203125f,1.246299994074554273737476250971667468547821044921875f,3.0875000518560415940783059340901672840118408203125f,1.89760000554621210966388389351777732372283935546875f,0.1824000231385227710489971286733634769916534423828125f,-1.1213000100746752796254668282927013933658599853515625f,1.8106999992236507690535063375136815011501312255859375f,-0.72359989523291545765459886752068996429443359375f,2.68610006188377870017802706570364534854888916015625f,0.360100071744621441638400938245467841625213623046875f,-0.97440005564540610460966263417503796517848968505859375f,4.865499961365014058856104384176433086395263671875f,4.072199981461466222754097543656826019287109375f,1.011699923667312095432180285570211708545684814453125f,2.06219999744296078603156274766661226749420166015625f,0.8791000903971506463818741394788958132266998291015625f,2.354100019560754208924890917842276394367218017578125f,-3.72849996195733535131466851453296840190887451171875f,1.513599963554739691318218319793231785297393798828125f,-1.14980003038868172637876341468654572963714599609375f,0.0188000067219153133013520573513233102858066558837890625f,-0.66130002648457886227362223507952876389026641845703125f,-2.186300014451890039168802104541100561618804931640625f,-1.03050001499056964604505992610938847064971923828125f,1.1759000488728295064078110954142175614833831787109375f,1.48900007011741575269070381182245910167694091796875f,-2.053500041194260550270200837985612452030181884765625f,-1.8645999137029047876268350591999478638172149658203125f,-0.7635000321604292761179522130987606942653656005859375f,-3.5322999913476422761959838680922985076904296875f,-1.2175001530349260914221076745889149606227874755859375f,-2.26379991716593398365375833236612379550933837890625f,-1.581599939504266583156777414842508733272552490234375f,-2.33029997705742797364791840664111077785491943359375f,1.7045001071989542840157128011924214661121368408203125f,-0.5318000514589258376219049750943668186664581298828125f,-0.90200005012750583688330152654089033603668212890625f,-0.0619000708818442380021451754146255552768707275390625f,2.52959996937736786293271507020108401775360107421875f,-1.899700071464476724969472343218512833118438720703125f,-0.062699985872209096360307967188418842852115631103515625f,3.795800046474486322267694049514830112457275390625f,-1.4396000965029005502771042301901616156101226806640625f,-0.3139000334650268175806786530301906168460845947265625f,2.94549997691810094835318523109890520572662353515625f,0.578700018405168492563461768440902233123779296875f,0.492500110398979007442221700330264866352081298828125f,1.84829997578710436556548302178271114826202392578125f,1.052400016675143934463676487212069332599639892578125f,1.1739000133857133700843178303330205380916595458984375f,1.2360000698603690327814774718717671930789947509765625f,2.7825000848807395215089854900725185871124267578125f,-0.91559993854314125627524845185689628124237060546875f,-0.42239995094239801698421388209681026637554168701171875f,-0.363299976830185167298026271964772604405879974365234375f,0.07449994205683407866303014088771305978298187255859375f,5.50809996674358703927509850473143160343170166015625f,0.869600045932083620670027812593616545200347900390625f,1.714700133844465401722345632151700556278228759765625f,-0.15359995353370992088315460932790301740169525146484375f,0.61130001847520443192252059816382825374603271484375f,-5.91210008903369388377768700593151152133941650390625f,1.9793999714538443868150352500379085540771484375f,0.18820006130561228729902722989208996295928955078125f,-2.5099999842047697740099465590901672840118408203125f,-0.6336000509873027652929522446356713771820068359375f,1.1800000511109820511279622223810292780399322509765625f,0.206300089237093697835945249607902951538562774658203125f,-3.1299999522976573729238225496374070644378662109375f,2.325100077276676557858081650920212268829345703125f,-0.981200013878941401657129972591064870357513427734375f,0.752000061322003698904836710426025092601776123046875f,-3.87550004460662567140616374672390520572662353515625f,0.4351001052722340833867065157392062246799468994140625f,-0.60939997566342329893274154528626240789890289306640625f,0.497300017722695830713064424344338476657867431640625f,2.410099894104153595009165655937977135181427001953125f,0.9284999474845816447299284845939837396144866943359375f,0.2245998999752092384341040087747387588024139404296875f,-0.277199974681436867030015491764061152935028076171875f,2.189500031705945648496935973525978624820709228515625f,-2.660599995744970858169153871131129562854766845703125f,-0.417099932105839510398936909041367471218109130859375f,-1.3398999652102585589119598807883448898792266845703125f,-0.528600020086020361276268886285834014415740966796875f,-1.638700010023267594760909560136497020721435546875f,1.279700018155575325096151573234237730503082275390625f,1.49919995205402312166143019567243754863739013671875f,1.917899967870116739732111454941332340240478515625f,4.52379999435394974938162704347632825374603271484375f,1.3826000368386506256257462155190296471118927001953125f,-0.418299985323848044771466447855345904827117919921875f,-2.009400117522477824394400158780626952648162841796875f,-1.002599970602989554180339837330393493175506591796875f,0.766499951578677229946379156899638473987579345703125f,-1.021100070507824142396202660165727138519287109375f,1.545699992536753608618482758174650371074676513671875f,-0.297200044139475760829327555256895720958709716796875f,3.050299962808193043883875361643731594085693359375f,-0.84990000782757968433855921830399893224239349365234375f,-0.830199965699762731929922665585763752460479736328125f,3.406599928775432051253346799057908356189727783203125f,-1.087900023041665775735964416526257991790771484375f,-0.5207999569557610897163613117299973964691162109375f,0.7473999036312106714063929757685400545597076416015625f,2.882399963980912360739239375106990337371826171875f,0.614799918648600840498374964226968586444854736328125f,-0.99940008222535203774583578706369735300540924072265625f,-1.517099846982955302365780880791135132312774658203125f,1.5852998983047894565601154681644402444362640380859375f,-0.7549001025244599372143738946761004626750946044921875f,1.42260003529265421917671119445003569126129150390625f,3.3596999693170186418456069077365100383758544921875f,-1.9834000193685288326861382302013225853443145751953125f,-1.7397999921672056000687689447659067809581756591796875f,3.9422000516086814769778357003815472126007080078125f,-1.5931000033445650299057660959078930318355560302734375f,0.396499918349086488689181351219303905963897705078125f,0.04809987644255098171441886734100989997386932373046875f,1.072400129346551267417453345842659473419189453125f,0.2941000788673762134095568399061448872089385986328125f,-0.4909000217154633194382995498017407953739166259765625f,-1.667500010170041679913310872507281601428985595703125f,3.2427999701097593998611046117730438709259033203125f,-1.8965999892741438603849246646859683096408843994140625f,-3.375000070687383502132661305950023233890533447265625f,-0.9283000079922392178133350171265192329883575439453125f,-1.770200086566807495813691275543533265590667724609375f,-2.2230001059621571357638458721339702606201171875f,1.1892000023826962973316767602227628231048583984375f,1.565299931627512020071435472345910966396331787109375f,3.20639997125268028099753792048431932926177978515625f,-1.2995999451629829390952863832353614270687103271484375f,-0.5207999508462848314849225062062032520771026611328125f,-0.990299958691746251560061864438466727733612060546875f,-2.457699940051139009256075951270759105682373046875f,-0.24539989591613486918930675528827123343944549560546875f,0.0073999135404818616734701208770275115966796875f,-3.14920001387521697466809200705029070377349853515625f,-0.43649994183704377004318075705668888986110687255859375f,-0.1309999185204502314405772267491556704044342041015625f,0.6561999110050493744239474835922010242938995361328125f,0.1911998106084757953482267112121917307376861572265625f,2.548500106703489365145287592895328998565673828125f,-3.4176999234735969679377376451157033443450927734375f,-2.134799903486669148122700789826922118663787841796875f,1.621199975806474125050726797780953347682952880859375f,6.83490004698038067232346293167211115360260009765625f,-0.1366001152634639392857707207440398633480072021484375f,-2.892199940427392590436284081079065799713134765625f,1.290899969486890341840990004129707813262939453125f,1.5682000764489174571281182579696178436279296875f,-2.039600036916883407656087001669220626354217529296875f,-1.68110000455156072263207533978857100009918212890625f,1.8702999566242084483747021295130252838134765625f,3.835499954957514834319454166688956320285797119140625f,1.1649000679723915396834854618646204471588134765625f,-2.57130000240057743354782360256649553775787353515625f,0.891000010646879392339769765385426580905914306640625f,-0.45050002156570678746305702588870190083980560302734375f,-0.1610000649802392214127166880643926560878753662109375f,3.6658999411746862051586504094302654266357421875f,0.7795000137873007162170324590988457202911376953125f,0.4169999609626824366159780765883624553680419921875f,0.78590013606324848893791568116284906864166259765625f,-0.18830007279366289818511859266436658799648284912109375f,-0.56339997740313396690225999918766319751739501953125f,0.58409999476894913783553420216776430606842041015625f,-0.6228999773696080044516065754578448832035064697265625f,0.6844000520505015838779172554495744407176971435546875f,-0.2834000889569525671873861938365735113620758056640625f,0.7073999954782419496979173345607705414295196533203125f,1.3465999971531330547946936349035240709781646728515625f,-0.40000000704079885505137781365192495286464691162109375f,-0.20480002321749968263731034312513656914234161376953125f,-3.8452000658541916067179045057855546474456787109375f,-1.0544000622391693777757382122217677533626556396484375f,-4.10030000166297181607433230965398252010345458984375f,2.1889000689670439214751240797340869903564453125f,2.28780002487152867018949109478853642940521240234375f,-3.21509994422793443646924060885794460773468017578125f,1.993900071164965037695537830586545169353485107421875f,0.0036999631591145831777112107374705374240875244140625f,0.1542999617911868792674567885114811360836029052734375f,-1.253400101846456227150383710977621376514434814453125f,4.90250003226101416231585972127504646778106689453125f,-0.5171999390490338299741779337637126445770263671875f,4.51530000944882825564263839623890817165374755859375f,2.336100006507336956929066218435764312744140625f,3.38560002018287686809117076336406171321868896484375f,2.11199992734193831012134978664107620716094970703125f,-6.629800009825078888070493121631443500518798828125f,2.7982999825671317495334733393974602222442626953125f,1.7113999734133482633069434086792171001434326171875f,2.63000003773718926680658114491961896419525146484375f,0.3568999345734715777922474444494582712650299072265625f,3.107800051097573845737542796996422111988067626953125f,0.0998000357344757116351274817134253680706024169921875f,1.0599999500811094765850839394261129200458526611328125f,-3.89510001102238856418580326135270297527313232421875f,1.7216000323757523204903918667696416378021240234375f,-1.808200072835384641933842431171797215938568115234375f,0.99759997861236282545860376558266580104827880859375f,0.0515000494420529975059253047220408916473388671875f,1.23230004359483746867454101447947323322296142578125f,-0.5214001111373314234498366204206831753253936767578125f,-2.685999929100276073512532093445770442485809326171875f,4.04999993197619811979848236660473048686981201171875f,0.436500017143787832907264601089991629123687744140625f,-0.48669999675750752476943716828827746212482452392578125f,-0.9849000258207316615965964956558309495449066162109375f,2.587000065065919951479145311168394982814788818359375f,-5.42330004314035107881863950751721858978271484375f,3.472500072382390445824285052367486059665679931640625f,-0.0727000765644010282784392984467558562755584716796875f,1.2760999413147560144210501675843261182308197021484375f,-1.80180004290938455824289121665060520172119140625f,-5.760499930351972608377764117904007434844970703125f,-4.15580004347562681488170710508711636066436767578125f,0.0988999297715709435596664889089879579842090606689453125f,0.6033999929189681754593266305164434015750885009765625f,1.0801000119909651697724939367617480456829071044921875f,-1.13559990982115355251380606205202639102935791015625f,0.5048000301465387362753745037480257451534271240234375f,1.0878999773323538846625524456612765789031982421875f,-0.651999949432908731949964931118302047252655029296875f,-1.40760007777959028629766180529259145259857177734375f,2.15020000807494060524049928062595427036285400390625f,0.6026000525780006444875880333711393177509307861328125f,-0.224200087431071448662578404764644801616668701171875f,-3.704800022379309165643235246534459292888641357421875f,-1.2372000503048303787778650075779296457767486572265625f,2.47069991346746764548925057169981300830841064453125f,2.824500089615582520963243950973264873027801513671875f,-2.8283000089794398945741704665124416351318359375f,2.338400047066062636957894937950186431407928466796875f,4.6679000020310272844881183118559420108795166015625f,-1.672199988129733139174959433148615062236785888671875f,-1.3772000585004684491963189429952763020992279052734375f,0.4404000322431309566439949776395224034786224365234375f,2.2191001169584705365878107841126620769500732421875f,-4.8542000979363901791430180310271680355072021484375f,-0.517199871360510154971734664286486804485321044921875f,-0.0070999613493665236774177174083888530731201171875f,-0.36810001811534320825813892952282913029193878173828125f,0.9993000198125840949359144360641948878765106201171875f,2.91820005446970487383850922924466431140899658203125f,-2.101800010126830375867257316713221371173858642578125f,0.49919992463588658271333997618057765066623687744140625f,-0.5224000212758788119771224955911748111248016357421875f,0.39229995854645871578014748592977412045001983642578125f,2.62929990458935503028214952792041003704071044921875f,2.6264999734088778637897121370770037174224853515625f,3.01840001746863340059690017369575798511505126953125f,-2.3868000294983406917026513838209211826324462890625f,4.36919996695518619844733620993793010711669921875f,-3.52669999187737648327356509980745613574981689453125f,-2.079300019291043799540830150363035500049591064453125f,2.193500044323502873311326766270212829113006591796875f,-3.5535998706087479348525448585860431194305419921875f,-1.3758999605461965654029654615442268550395965576171875f,-2.419700013573466979011072908178903162479400634765625f,2.009400037745386224941057662363164126873016357421875f,-0.57429996842220465413930696740862913429737091064453125f,-0.88690008114501839298071672601508907973766326904296875f,1.054800069131703654790044311084784567356109619140625f,-1.3358000235080720319302827192586846649646759033203125f,-2.030200005169212840172576761688105762004852294921875f,1.5346000436410296430267408140935003757476806640625f,0.6454000013232217281000657749245874583721160888671875f,-0.3638000449433900218565440809470601379871368408203125f,-3.041400045907497684538611792959272861480712890625f,-1.433699901021271205792118053068406879901885986328125f,-0.243299978618324475032608233959763310849666595458984375f,1.102399996726215025688588866614736616611480712890625f,-1.1076999689035120777447218642919324338436126708984375f,2.459200165550411210091397151700221002101898193359375f,0.4899000874370338376451172734959982335567474365234375f,3.68959996391981892571720891282893717288970947265625f,-0.733099982576070896556075240368954837322235107421875f,0.5604999966993926552305538280052132904529571533203125f,2.40539994723200845072597076068632304668426513671875f,0.05149996521323962905825055713648907840251922607421875f,-1.82939988741129599958412654814310371875762939453125f,1.72829998469054846310655193519778549671173095703125f,-0.52039996466636739835820435473578982055187225341796875f,-0.00039998459666978902760092751123011112213134765625f,-3.458800058801472232516971416771411895751953125f,-1.3241000304758561156859286711551249027252197265625f,0.8107000848494478884020963960210792720317840576171875f,0.873499996490777252944326392025686800479888916015625f,-0.448600028542429585254325274945585988461971282958984375f,-2.544299987328052292667734946007840335369110107421875f,-1.004100026117265453962090759887360036373138427734375f,-0.1186000065073369302837136274320073425769805908203125f,2.270600072728097273966341163031756877899169921875f,0.9024000018484896745718515376211144030094146728515625f,-1.96780002183541657956311610178090631961822509765625f,1.306999923840165056532214293838478624820709228515625f,-0.293600109828263011735316467820666730403900146484375f,1.6096000053822994146912606083787977695465087890625f,0.9146000186443330459695744139025919139385223388671875f,-0.81539992357641377651589209563098847866058349609375f,-0.21179987823292745208192400241387076675891876220703125f,1.645800002944469042631681077182292938232421875f,2.141499952044338073164908564649522304534912109375f,-0.8317000151976963184807800644193775951862335205078125f,1.023200113254784326954904827289283275604248046875f,2.624700044195353410003690441953949630260467529296875f,-0.12770001751854975768907252131612040102481842041015625f,0.809000028748065869166339325602166354656219482421875f,-0.27570003647655294543028503539972007274627685546875f,0.49729994688630074772817124539869837462902069091796875f,0.33360001611858536296750799010624177753925323486328125f,1.7354999237582082738384769982076250016689300537109375f,2.63019997547864914366755328956060111522674560546875f,-2.19889993271455264078895197599194943904876708984375f,-2.678599953235687447516966130933724343776702880859375f,-1.567199978630245738031590008176863193511962890625f,-0.8126000054530819749487591252545826137065887451171875f,1.239900013080239116192160508944652974605560302734375f,2.115500021062791002890435265726409852504730224609375f,0.216699944566189461880867384024895727634429931640625f,-1.2122999931171543064323259386583231389522552490234375f,-1.6914000763803735960522089953883551061153411865234375f,-1.4543998974695819814684227821999229490756988525390625f,-0.649299920496344551423817392787896096706390380859375f,-2.487500030510127135841003109817393124103546142578125f,2.558800016668438814804176217876374721527099609375f,-1.4696000246047977721985944299376569688320159912109375f,0.689900006002188259657259550294838845729827880859375f,1.497999965928495047506885384791530668735504150390625f,0.9870001185610879357312796855694614350795745849609375f,-1.0844000266626470097008905213442631065845489501953125f,-2.51869996246993554933624182012863457202911376953125f,0.5956000073865073130008340740459971129894256591796875f,0.6884000319600109829565326435840688645839691162109375f,0.85590000644177133271028878880315460264682769775390625f,1.51400000529736278309655972407199442386627197265625f,-0.5491999660983679643067034703562967479228973388671875f,-0.06009998986273945575931065832264721393585205078125f,-1.29670000879019386985646633547730743885040283203125f,-2.717399980111420365602725723874755203723907470703125f,0.74689999104887283554177201949642039835453033447265625f,0.34129993540123060302704516288940794765949249267578125f,-3.036500040203334638277965495944954454898834228515625f,1.1779000556379546349461406862246803939342498779296875f,-2.090699986706673119130073246196843683719635009765625f,-2.409000039942562398920244959299452602863311767578125f,-0.807200018751621062307322063134051859378814697265625f,-2.54910001684129117194288483005948364734649658203125f,0.99860002336427822200448645162396132946014404296875f,-0.3567999300703410536783621864742599427700042724609375f,-0.635000054650009193579762722947634756565093994140625f,-2.28039997165128571765535525628365576267242431640625f,-0.1000000003725285413480605711811222136020660400390625f,0.13550011873990219779528842991567216813564300537109375f,1.0842000949189074532341692247427999973297119140625f,-0.2140999929994344963546382132335565984249114990234375f,-1.9678999791577462818992216853075660765171051025390625f,-0.5859999718666071277795026617241092026233673095703125f,0.6540000086873754003846670457278378307819366455078125f,-3.56920002181008388930649743997491896152496337890625f,-0.34829993074834408872675339807756245136260986328125f,0.20860005333423614626298103758017532527446746826171875f,2.62239995748028231759008122025988996028900146484375f,-3.126899965120851110356170465820468962192535400390625f,0.618800056268273390713829940068535506725311279296875f,-1.493800151021032807108213091851212084293365478515625f,1.765700026101618913543234157259576022624969482421875f,0.622099982807039264542936507496051490306854248046875f,2.113499951209873994883992054383270442485809326171875f,0.069699942848831852870716829784214496612548828125f,-2.097699974100291075984614508342929184436798095703125f,-2.228999994680284313375295823789201676845550537109375f,-0.9372000048562885154979085200466215610504150390625f,-0.8381000373378391099521422802354209125041961669921875f,-1.1120999721899631484944848125451244413852691650390625f,0.251100026623905847600326524116098880767822265625f,-1.523499932136387347014760962338186800479888916015625f,-2.332699885848165788360120131983421742916107177734375f,0.6755999675631532142006108188070356845855712890625f,-0.44300005930289698685697885593981482088565826416015625f,1.437399991361797635391894800704903900623321533203125f,0.92219999651163764298189562396146357059478759765625f,0.6184000334873800408530541972140781581401824951171875f,-2.955299991753696620122582316980697214603424072265625f,-0.7279000219427047557729792970349080860614776611328125f,-0.395700013603269784567828537547029554843902587890625f,-1.6220000575967141731581477870349772274494171142578125f,-2.135699977244437608447924503707326948642730712890625f,-1.000000014100222767154946268419735133647918701171875f,-0.4661000611573447027780048301792703568935394287109375f,-0.913199916801603617244609267800115048885345458984375f,-2.569600020059942924177676104591228067874908447265625f,2.95179998147934696817173971794545650482177734375f,0.18830000039264493949531242833472788333892822265625f,2.23130004682466420717901200987398624420166015625f,-0.3299000470176329802285408732132054865360260009765625f,0.7406999471999713424708033926435746252536773681640625f,0.5436000643424685474514035377069376409053802490234375f,-1.33850000599026675018876630929298698902130126953125f,0.220099911855161423090976313687860965728759765625f,-2.4685000675693142824229653342626988887786865234375f,1.672400037182867560403565221349708735942840576171875f,2.441699942889810959201213336200453341007232666015625f,-0.726799946033211075047120175440795719623565673828125f,1.104900013210623921366959621082060039043426513671875f,-0.4218000189743940087083728940342552959918975830078125f,1.4864999171197410543499017876456491649150848388671875f,-0.01230004365071579730539497177232988178730010986328125f,-1.190600027484447043235604724031873047351837158203125f,1.55979997583180729492369209765456616878509521484375f,-2.0525998861692844599247109727002680301666259765625f,0.200399993686377175805546357878483831882476806640625f,-0.25259999399781207785053993575274944305419921875f,1.25149988323822558555775685817934572696685791015625f,2.183800031960756005133816870511509478092193603515625f,1.527299958769976040429128261166624724864959716796875f,3.16449992749094999311409992515109479427337646484375f,-0.615799893756211691453472667490132153034210205078125f,0.1620000139176855924461051472462713718414306640625f,-0.2237000842869287797753941049450077116489410400390625f,0.550900003033131913099396115285344421863555908203125f,-0.366400027914344672552715564961545169353485107421875f,-2.54569994978904734495017692097462713718414306640625f,-0.53090001353845028830846786149777472019195556640625f,-1.285399944084137491273622799781151115894317626953125f,0.5106001024372870400469537344179116189479827880859375f,-1.0542999518260349756104687912738882005214691162109375f,0.182299924460054274533149509807117283344268798828125f,-2.015000015422701462597387944697402417659759521484375f,1.314299940799176003025650061317719519138336181640625f,0.044000014051795321989857256994582712650299072265625f,-1.586899941204488850843290492775849997997283935546875f,0.80600009364634761510615135193802416324615478515625f,1.11359995867460970231377359596081078052520751953125f,0.2288000521704558476443480685702525079250335693359375f,-1.192500015832482773703304701484739780426025390625f,1.4929000213652858786161914395052008330821990966796875f,-0.78939985222592967684107634340762160718441009521484375f,-1.965299925462156149791326242848299443721771240234375f,0.7817999796539540913187238402315415441989898681640625f,1.46720005693584720773969820584170520305633544921875f,0.11269998743683096176226854367996565997600555419921875f,-0.323499991964548261336176437907852232456207275390625f,3.53339998043924463644316347199492156505584716796875f,0.848799944528191741000000547501258552074432373046875f,-3.866399962796271072562603876576758921146392822265625f,-2.318100017482042307648271162179298698902130126953125f,-0.082100099315493668683529904228635132312774658203125f,1.722800047223268737894841251545585691928863525390625f,-3.335799966734647181709760843659751117229461669921875f,-0.2807999562665821624563022851361893117427825927734375f,-3.004699922005830359239553217776119709014892578125f,-5.36210004632324110929175731143914163112640380859375f,-3.97279995673596797445270567550323903560638427734375f,-0.0508000348702071224948895178386010229587554931640625f,-1.026400008282065900999668883741833269596099853515625f,0.81869994099363641026911864173598587512969970703125f,2.737399925051629434591404788079671561717987060546875f,0.79700010405108390187933764536865055561065673828125f,0.033400037268547233537674401304684579372406005859375f,-3.2970000459179278351484754239208996295928955078125f,-1.1649999170005316262432870644261129200458526611328125f,-0.7878999911159281221983974319300614297389984130859375f,-1.24749998670071438056083934498019516468048095703125f,0.6547999829471109478618018329143524169921875f,1.161499915741384114653556025587022304534912109375f,-0.71289990759491994598562314422451891005039215087890625f,0.187800059386342699241367881768383085727691650390625f,1.5071999832682312359111165278591215610504150390625f,1.269300141629576383905941838747821748256683349609375f,-0.4456000286206605043304307400831021368503570556640625f,0.4651001112513253321623096780967898666858673095703125f,0.587300086528807607777480370714329183101654052734375f,-1.6688000115461651962078803990152664482593536376953125f,0.81689998026415755472129376357770524919033050537109375f,-1.155699881970137621323146959184668958187103271484375f,2.08440000386386969921659328974783420562744140625f,2.664799998220802823567510131397284567356109619140625f,2.550400027847289141647024734993465244770050048828125f,-2.61109996429979762666562237427569925785064697265625f,0.2308999685183168448787682791589759290218353271484375f,0.28870001477301154846344388715806417167186737060546875f,0.199100021833180651498196311877109110355377197265625f,0.8942999586433175718269694698392413556575775146484375f,0.981599956938624007563021223177202045917510986328125f,0.069799979165940495562381329364143311977386474609375f,0.95479996435791303355244963313452899456024169921875f,-1.178100026012957268761738305329345166683197021484375f,-0.46639990216717119864142659935168921947479248046875f,-0.93879990695864068417364478591480292379856109619140625f,1.1310000237971540304471318449941463768482208251953125f,3.108299942073225796690394417964853346347808837890625f,-0.5708000243648883031966079215635545551776885986328125f,-1.3903000252068034026109444312169216573238372802734375f,-1.346199931959807916115323678241111338138580322265625f,-2.5604000803738831137934539583511650562286376953125f,-3.6981999336585413828970558824948966503143310546875f,-3.181900016026945277047843774198554456233978271484375f,0.44029997979551549480703442895901389420032501220703125f,2.334099944663792225441056871204636991024017333984375f,-0.9920000246651465847236295303446240723133087158203125f,-1.180700021165609125972650872427038848400115966796875f,-1.51189998443648132564476327388547360897064208984375f,-3.096099931740762034593217322253622114658355712890625f,-0.9298000176295639906953738318406976759433746337890625f,1.6357999481454494894450135689112357795238494873046875f,-0.6224000194504843985754405366606079041957855224609375f,-0.2390000337213280534598425219883210957050323486328125f,-5.27650000902265414737257742672227323055267333984375f,-0.8743000111326575396475391244166530668735504150390625f,2.805499997686595303747481011669151484966278076171875f,2.24599991792440345506065568770281970500946044921875f,-0.3406999883085484182032587341382168233394622802734375f,1.68679994217753392860004169051535427570343017578125f,0.400800002720952175394586447509936988353729248046875f,0.8035999800577762197661968457396142184734344482421875f,-4.03900006164237712624753839918412268161773681640625f,2.75960014027506250755550354369916021823883056640625f,-4.235200047767907705065226764418184757232666015625f,0.7380999109014876324863507761619985103607177734375f,-0.3020000461787011314385154037154279649257659912109375f,-0.2980999667063353175677775652729906141757965087890625f,-3.6270000749938162698526866734027862548828125f,1.3330000024735932928621195969753898680210113525390625f,3.76450001661851985090834205038845539093017578125f,-3.055400047591328327456494662328623235225677490234375f,1.0915000047385692294454884176957421004772186279296875f,0.6231998724147675972062643268145620822906494140625f,-2.38510001068711385840970251592807471752166748046875f,-2.462500074170530073303098106407560408115386962890625f,-1.040300016415119710444514566916041076183319091796875f,1.99740004299432083456622422090731561183929443359375f,-0.87980003137588569384064385303645394742488861083984375f,-0.018300058469922220183434546925127506256103515625f,0.8616000065222382264806810781010426580905914306640625f,2.7105999594420193687938080984167754650115966796875f,-1.1723999895364058954072561391512863337993621826171875f,0.516400052314997193292356314486823976039886474609375f,3.110900013389438445443602176965214312076568603515625f,-0.2921000499740245004431926645338535308837890625f,-0.364299980920552712859716848470270633697509765625f,2.155200040149688245350034776492975652217864990234375f,1.75819999859035025480125113972462713718414306640625f,-2.7736999174311751659161018324084579944610595703125f,-2.5435000914111736136646868544630706310272216796875f,3.93490006273835835060026511200703680515289306640625f,-1.800399961388109648652289251913316547870635986328125f,-1.42489995017871251548058353364467620849609375f,0.2088999809011824471127738434006460011005401611328125f,1.240700039121509501427453869837336242198944091796875f,4.540399902193247072546000708825886249542236328125f,1.255800008755922458902887228759936988353729248046875f,-0.17490005586519696834102433058433234691619873046875f,-0.5674000245168790979022332976455800235271453857421875f,-3.551399963429571204898138603311963379383087158203125f,3.062499917075039324032559306942857801914215087890625f,-0.65540007808283029522300466851447708904743194580078125f,0.62060004042983063499150375719182193279266357421875f,0.0026999234735967103659959320793859660625457763671875f,0.142000011831521444349846206023357808589935302734375f,-1.309999940134585205697703713667578995227813720703125f,-0.70220000765025647471162528745480813086032867431640625f,4.18809995459914308213456024532206356525421142578125f,-0.3250999897323547660477061072015203535556793212890625f,-1.8592999347642054086549023850238882005214691162109375f,-2.235199926863611352700900170020759105682373046875f,3.60110002658665084851463689119555056095123291015625f,-1.2344998698979612328230359707958996295928955078125f,1.033499886818229906992883115890435874462127685546875f,-1.2205000222548818555878824554383754730224609375f,0.630800085236131291566152867744676768779754638671875f,-3.9709999904185533381451023160479962825775146484375f,0.301500019975007749195583528489805757999420166015625f,2.8338000373810512400041261571459472179412841796875f,-0.44599989899993008890533019439317286014556884765625f,2.5915999344192446329770973534323275089263916015625f,-0.5244000331074001319819899435970000922679901123046875f,1.329300051626564282969411578960716724395751953125f,0.197300049406290867182178772054612636566162109375f,0.0105999858915806211001608971855603158473968505859375f,0.186999973796307283890882899868302047252655029296875f,2.66180001713037572841358269215561449527740478515625f,0.401999908454715448868910243618302047252655029296875f,-3.05990004671961113302813828340731561183929443359375f,-2.039999935217201798565156423137523233890533447265625f,-1.6507999592840667535398324616835452616214752197265625f,-1.67439999015480278643508427194319665431976318359375f,2.294500029433518495380894819390960037708282470703125f,-0.28420002499520780059327762501197867095470428466796875f,1.7382999954193809610814014376956038177013397216796875f,2.900000035986304514068478965782560408115386962890625f,-1.728600013324617545862338374718092381954193115234375f,-2.18300006259977852351994442869909107685089111328125f,2.3003999522611504602309651090763509273529052734375f,-0.8015999402865769951631591538898646831512451171875f,-2.805500019572675807211226128856651484966278076171875f,-0.651700042708963866999738456797786056995391845703125f,0.7061000496275717797090010208194144070148468017578125f,1.76580004064440831967885969788767397403717041015625f,-0.09549996138364080788818455403088591992855072021484375f,-1.0701999201580878473549773843842558562755584716796875f,2.187800047893822874556235547061078250408172607421875f,0.34939995748400753949880481741274707019329071044921875f,1.10559995575398062328531523235142230987548828125f,-1.85049991103634159372859357972629368305206298828125f,3.80589996335878932853802325553260743618011474609375f,1.466199978339673304361667760531418025493621826171875f,-0.7799999666959041011438102941610850393772125244140625f,1.685799984355271785574359455495141446590423583984375f,-1.52420009177103654707252644584514200687408447265625f,-0.28560006058364972858498731511645019054412841796875f,-0.392699836897849596795140314497984945774078369140625f,-0.7363000441238296556178966056904755532741546630859375f,1.0745999746486549764057372158276848495006561279296875f,1.022899996368586972295133818988688290119171142578125f,-0.0684000568449494306122460329788736999034881591796875f,-0.25310001658797265378098018118180334568023681640625f,-1.607000048637388545103021897375583648681640625f,1.1121000173404806954380319439223967492580413818359375f,1.7514000114485621661941650017979554831981658935546875f,1.1456000029906625581332946239854209125041961669921875f,-0.06540000966787307135064111207611858844757080078125f,2.168599932653457518227924083475954830646514892578125f,-1.0978000031530867897089365214924328029155731201171875f,-0.683700089316068737588238946045748889446258544921875f,2.13470002158284177795621872064657509326934814453125f,2.600900053939222633658800987177528440952301025390625f,0.4656000523880130526066523088957183063030242919921875f,0.385199987213313388423330252408050000667572021484375f,4.00309985658675504538450695690698921680450439453125f,-0.77949991357699122129787383528309874236583709716796875f,0.174200062713772307887438728357665240764617919921875f,1.5001000372484314393517479402362369000911712646484375f,-0.8301999331966045669872755752294324338436126708984375f,0.5658998930439362862188090730342082679271697998046875f,0.15739997821152129642285899535636417567729949951171875f,1.197400067469476692139096485334448516368865966796875f,-4.32610007155090681152387332986108958721160888671875f,-2.67590001007392874754486911115236580371856689453125f,0.418800048221646736834600233123637735843658447265625f,-1.68689995614290122460943166515789926052093505859375f,0.821700047607718975228863200754858553409576416015625f,0.90470007181838052456868126682820729911327362060546875f,-3.59870011440589987472549182712100446224212646484375f,-0.260299963404239331765666065621189773082733154296875f,1.5457999417752008586290912717231549322605133056640625f,-3.63800003711879060830369780887849628925323486328125f,-2.899799954709412208586627457407303154468536376953125f,-1.7678999431714410572880069594248197972774505615234375f,0.603400032239405970102552601019851863384246826171875f,0.86870007251501046852126819430850446224212646484375f,1.14029991285204790329999013920314610004425048828125f,-0.75340000377819027477244162582792341709136962890625f,0.6608000401973714854619856851059012115001678466796875f,-0.2008000170633206238335333182476460933685302734375f,1.167400113476813938717668861499987542629241943359375f,3.579599998266993932105606290861032903194427490234375f,-0.4562000285938392352136361296288669109344482421875f,1.460799924676120298272508080117404460906982421875f,-2.251800112311542534371255896985530853271484375f,2.519199928919971132046384809655137360095977783203125f,-4.2849000327497730467030123691074550151824951171875f,2.046500045493245156791317640454508364200592041015625f,-3.584399891471862442671181270270608365535736083984375f,-3.725099938453733994236927173915319144725799560546875f,1.1538000063121318650161128971376456320285797119140625f,-3.460799938683213117229797717300243675708770751953125f,-1.611799911667406437487670700647868216037750244140625f,3.098399927074460880049855404649861156940460205078125f,-0.0118000094190247040160102187655866146087646484375f,1.465700039587170433463825247599743306636810302734375f,2.634400000883639680893111290060915052890777587890625f,-1.3002999927595271234537221971550025045871734619140625f,2.43580003969445879619115657988004386425018310546875f,-1.144100058229267080633917430532164871692657470703125f,-0.0725000685453427440307905271765775978565216064453125f,4.41109993196428007422582595609128475189208984375f,3.35129998066350776042554571176879107952117919921875f,2.89309983434677153724123854772187769412994384765625f,-0.105699954445659383139854980981908738613128662109375f,-0.2265000344850125468809665107983164489269256591796875f,3.568899849621951947398201809846796095371246337890625f,-0.2269001348078252267015386678394861519336700439453125f,0.7454000188693392647820701313321478664875030517578125f,2.62410004615932646032661068602465093135833740234375f,-4.91189997555166524279002260300330817699432373046875f,0.5800000841543087037877057809964753687381744384765625f,0.205299969308821772528972360305488109588623046875f,-0.12869992713555478136555620949366129934787750244140625f,0.36709994898140363428495902553549967706203460693359375f,-1.765299854588507688646359383710660040378570556640625f,2.63560001090690310121544825960882008075714111328125f,2.399699966932832939647823877749033272266387939453125f,0.99560000466704412502849663724191486835479736328125f,0.4484999818131309989865940224262885749340057373046875f,0.9328000094674511188941323780454695224761962890625f,-1.757999955274164083363075405941344797611236572265625f,0.12270001677349184898702105783740989863872528076171875f,0.626100110834092671296957632876001298427581787109375f,-0.55220006759017525865829156828112900257110595703125f,1.793200076709686729969916996196843683719635009765625f,0.7568000465974213941677817274467088282108306884765625f,-1.94129993456304017485081203631125390529632568359375f,2.626300012160836416796882986091077327728271484375f,5.493100080420820319204722181893885135650634765625f,-0.7039999202303592884533145479508675634860992431640625f,-1.26069999881386873852306962362490594387054443359375f,3.12660001967251321985941103775985538959503173828125f,-2.23879990744292722837371911737136542797088623046875f,0.821499935634433686715283329249359667301177978515625f,-0.02350004704296659863160812165006063878536224365234375f,0.3328999583333729361811492708511650562286376953125f,-0.3147999469049285181654340703971683979034423828125f,4.8412998917967087209035526029765605926513671875f,-3.00010003983750994649426502292044460773468017578125f,1.4759999432377510775182827273965813219547271728515625f,-4.92790002017319306304443671251647174358367919921875f,-0.984200082159788269819955530692823231220245361328125f,6.0382999579802163481190291349776089191436767578125f,0.533000032480807295343083751504309475421905517578125f,2.964299926289169828663716543815098702907562255859375f,-1.2863999284863487648777891081408597528934478759765625f,-1.349599945479631468714387665386311709880828857421875f,-1.43520011297911320724551842431537806987762451171875f,1.135499961122869994056827636086381971836090087890625f,-1.385099900530280425670071053900755941867828369140625f,2.486500000510364127848106363671831786632537841796875f,2.341600052417812438676492092781700193881988525390625f,-1.127099907369909725929346677730791270732879638671875f,-2.485699920303374721441969086299650371074676513671875f,-0.9694999070204790569960096036083996295928955078125f,-0.27630001320391783803387397711048834025859832763671875f,-2.425700044504555119573296906310133635997772216796875f,-2.36130001052170968023347086273133754730224609375f,-0.959299984254686766149688992300070822238922119140625f,3.605400031088290990766154209268279373645782470703125f,-1.3312000207521019756740088268998079001903533935546875f,-3.875900000779331389111348471487872302532196044921875f,-3.420699921402336141085243070847354829311370849609375f,-0.589100090564788292368803013232536613941192626953125f,-2.26149996195361069339924142695963382720947265625f,-0.8810000159740429825916407935437746345996856689453125f,0.5149000728711483620969602270633913576602935791015625f,-0.06699983286857580377926524306531064212322235107421875f,1.785600017016381269030489420401863753795623779296875f,1.78050001374259370123809276265092194080352783203125f,1.451300095327944017498111861641518771648406982421875f,-2.00999992903322066695182002149522304534912109375f,-1.7658999946698548111356785739189945161342620849609375f,0.89549991289898667190527703496627509593963623046875f,-1.778900066545604286005755056976340711116790771484375f,-0.7598999960556620525409243782632984220981597900390625f,0.0570000009536737461957045525196008384227752685546875f,-3.178900073865801090278182527981698513031005859375f,0.5884000160902751463964932554517872631549835205078125f,0.5093999811768537000489232013933360576629638671875f,0.75109999324530551945144907222129404544830322265625f,1.8448000681817529056161220069043338298797607421875f,1.85999992467462949008449868415482342243194580078125f,2.00050007273256635897951127844862639904022216796875f,3.22239999179020575326148900785483419895172119140625f,-0.0723000290103261722407523848232813179492950439453125f,0.0801000629901889027451034053228795528411865234375f,-1.0305999929696325612127338899881578981876373291015625f,1.5921999641388644430861631917650811374187469482421875f,-0.16210012561604292269379357094294391572475433349609375f,-1.6119000755012049541647911610198207199573516845703125f,1.5094999412931482396516003063879907131195068359375f,3.22719983151853018199517464381642639636993408203125f,-0.908800009739400227459782399819232523441314697265625f,4.40829998387098509482484587351791560649871826171875f,-1.12139983367770579292255206382833421230316162109375f,-0.0223999838180828358957796808681450784206390380859375f,-2.16790011259764270334926550276577472686767578125f,1.35740006411671654262818265124224126338958740234375f,1.3301000097185344817063423761283047497272491455078125f,-1.125599985332784935820882310508750379085540771484375f,-3.03290001644790141455132470582611858844757080078125f,0.58359994893297340023963215571711771190166473388671875f,-1.927200073625149112643839544034563004970550537109375f,0.044800016027688460695799221866764128208160400390625f,2.621099952788652043267347835353575646877288818359375f,-0.555999940201641340564719939720816910266876220703125f,-1.314599950508772874258056617691181600093841552734375f,-0.1109999887049202271782633033581078052520751953125f,0.7854000627532624623228230120730586349964141845703125f,0.104199993404747726088999115745536983013153076171875f,1.448099950128794777270968552329577505588531494140625f,3.65509989982247507356305504799820482730865478515625f,0.31669983660727663021816624677740037441253662109375f,-0.247699961248040612105114632868207991123199462890625f,0.10909991469383506501600322735612280666828155517578125f,-2.44969999798312887406837035086937248706817626953125f,0.5519000540807847965396604195120744407176971435546875f,-1.307299907528609583806655791704542934894561767578125f,-1.4934999707303955318593580159358680248260498046875f,3.371600126998126167876534964307211339473724365234375f,1.3018999804131681230501271784305572509765625f,1.92300007597356970023838584893383085727691650390625f,0.340000011175873861901663985918276011943817138671875f,1.309899937848002071660857836832292377948760986328125f,1.7851998957201853368559341106447391211986541748046875f,1.060499979190527852779268869198858737945556640625f,-3.583200146428491716932285271468572318553924560546875f,-3.91559994372129605011423336691223084926605224609375f,-0.7821000553011894229626932428800500929355621337890625f,2.90519994800463177142546555842272937297821044921875f,0.3961999850451951576957299039349891245365142822265625f,-0.80989999914765553601370129399583674967288970947265625f,-0.27740005439370929440912050267797894775867462158203125f,1.585900047438591808912633496220223605632781982421875f,4.03329989068284699982314123190008103847503662109375f,0.927899932312218567176387296058237552642822265625f,-2.30619997878670712765369898988865315914154052734375f,-1.6872000289961677221839408957748673856258392333984375f,1.18129991090744734805184634751640260219573974609375f,-0.63179999448061041622537459261366166174411773681640625f,-3.959599983477589457692147334455512464046478271484375f,2.04300000215694321781256803660653531551361083984375f,3.1633999238893384386983598233200609683990478515625f,0.382400004288552697317982165259309113025665283203125f,2.90300007129833037566868370049633085727691650390625f,1.107200094002484203059566425508819520473480224609375f,2.698499927498400285941215770435519516468048095703125f,0.79420011695400061224603405207744799554347991943359375f,3.284199989716707346332214001449756324291229248046875f,-2.08039989936202829312605899758636951446533203125f,-1.6054999643266203435132410959340631961822509765625f,0.691999828062953792340294967289082705974578857421875f,-0.68920004988014937907792045734822750091552734375f,-3.20490001512914890469119200133718550205230712890625f,2.244900012670458355756863966234959661960601806640625f,3.159599927318841761092471642768941819667816162109375f,-2.01130008426383266595394161413423717021942138671875f,-4.26539997368156686974316471605561673641204833984375f,-0.25999993272125776488934434382827021181583404541015625f,-1.0537000684544441941881132152047939598560333251953125f,3.00739998707771416519562990288250148296356201171875f,4.758700114536285497024437063373625278472900390625f,-3.461300124366581076884585854713805019855499267578125f,-0.21049995758384298927268218903918750584125518798828125f,-2.383900062224270310906604208867065608501434326171875f,5.38389993884265294354918296448886394500732421875f,-2.30569994543045897472666183602996170520782470703125f,1.0636998546227811601738721947185695171356201171875f,0.6712000228196373097233617954771034419536590576171875f,-4.40509989995285788921819403185509145259857177734375f,0.704700076456365565036321640945971012115478515625f,2.93260008530467786158624221570789813995361328125f,0.6922999672263856485443511701305396854877471923828125f,3.54909998297840356684673679410479962825775146484375f,2.31830015109926534933038055896759033203125f,-3.656400071220843539521183629403822124004364013671875f,0.802900029337408671636922008474357426166534423828125f,2.8861999718204156550882544252090156078338623046875f,-0.7823000145375740288500310271047055721282958984375f,-1.830999919787047236496846380759961903095245361328125f,1.74049996633827586123288710950873792171478271484375f,0.10849995543807733344721100365859456360340118408203125f,-5.232399971617756051500691683031618595123291015625f,-2.23159997658953113131019563297741115093231201171875f,3.074300017018615438502138204057700932025909423828125f,-1.69109987360760616326160743483342230319976806640625f,0.347199944096804102855458040721714496612548828125f,2.279400033889711796319943459820933640003204345703125f,-2.95399999009817815220912962104193866252899169921875f,1.9871999643996360873643425293266773223876953125f,1.56869995319396249300325507647357881069183349609375f,0.63090000524967937423070907243527472019195556640625f,0.85019983592927506066416754038073122501373291015625f,2.5465000463873170843953630537725985050201416015625f,-1.4563000421494247671461153004202060401439666748046875f,0.5105999490484582903349064508802257478237152099609375f,-1.477699958603083896235830252408050000667572021484375f,-1.331000108286736871576749763335101306438446044921875f,-2.56400007659942019699883530847728252410888671875f,-0.0492999363660791001251482157385908067226409912109375f,1.40680002059489428489769125008024275302886962890625f,-0.6527998944863682151407147102872841060161590576171875f,3.59330001177340818685479462146759033203125f,4.25909999799132332753970331395976245403289794921875f,-3.820699945728481594642289564944803714752197265625f,1.2601999535366896765253841294907033443450927734375f,1.14999980950727920259168968186713755130767822265625f,2.25399997270107466107447180547751486301422119140625f,1.4259000946938982234968307238887064158916473388671875f,-4.71509986931234692519865348003804683685302734375f,-2.61129995002299519768484969972632825374603271484375f,-3.363599907116590514988274662755429744720458984375f,-0.910400026059150935253683201153762638568878173828125f,3.014500026583672553925907777738757431507110595703125f,-2.5679000202849504574942329782061278820037841796875f,3.03830010283812868010500096715986728668212890625f,1.17690000997334909271785363671369850635528564453125f,-0.9584999900460251520684096249169670045375823974609375f,0.6617999324545262851415827753953635692596435546875f,0.36869998992532515558195882476866245269775390625f,-3.26839994303733050173832452856004238128662109375f,0.27170016260743068414740264415740966796875f,-1.416399960933626633874382605426944792270660400390625f,-1.508900040543081022548221881152130663394927978515625f,0.785099958793818775149020439130254089832305908203125f,-2.023199975717066667613153185811825096607208251953125f,-0.67639999371618042545151183730922639369964599609375f,-2.27389999762773609148780451505444943904876708984375f,2.47619986650645973469409000244922935962677001953125f,1.727299973261354981701742872246541082859039306640625f,-1.437499860264361384309950153692625463008880615234375f,0.158800011900069204529017952154390513896942138671875f,0.999699809390306892709077146719209849834442138671875f,-0.07640001979321209280016091724974103271961212158203125f,-1.3311000495716911462551479417015798389911651611328125f,2.57850006045401158871754887513816356658935546875f,-0.637900098031760620642671710811555385589599609375f,-2.43250003069639131325629932689480483531951904296875f,-1.175300008256735306844120714231394231319427490234375f,2.051800025661290050038587651215493679046630859375f,-0.0073000687405448960731746410601772367954254150390625f,2.126800080832838890643188278772868216037750244140625f,3.5847999450609098204267866094596683979034423828125f,2.735500025160610082508583218441344797611236572265625f,-0.3087000816047218432203180782380513846874237060546875f,3.39049983561784262775518072885461151599884033203125f,-3.690499993532895484094069615821354091167449951171875f,3.546799863946437536554867619997821748256683349609375f,-2.445900022833048748083228929317556321620941162109375f,-1.361100009040531855220024226582609117031097412109375f,-1.6480999409645804565371918215532787144184112548828125f,-3.84730000082850676079715412924997508525848388671875f,-0.3055998954042793425145418950705789029598236083984375f,-0.562399985587596074054772543604485690593719482421875f,0.089999955035744871878478079452179372310638427734375f,0.84099999667704106176557843355112709105014801025390625f,-1.1852999701283846167854107989114709198474884033203125f,1.1103998859882369121265810463228262960910797119140625f,2.141599973627923869656797251082025468349456787109375f,-2.203300007005035698881556527339853346347808837890625f,-1.184000040389598940038240471039898693561553955078125f,-1.43919992646127781199538731016218662261962890625f,0.02519993628859518164375685955747030675411224365234375f,-0.7876000474184741140248888768837787210941314697265625f,-3.7276999835439053043728563352487981319427490234375f,-2.247799861927330056232676724903285503387451171875f,1.73160000283420156819147450732998549938201904296875f,1.069399925670027950985740972100757062435150146484375f,-2.637600092941523310940965529880486428737640380859375f,-0.62770001083165549715658926288597285747528076171875f,0.7693998469918967852976265930919907987117767333984375f,0.9223999805212008684662805535481311380863189697265625f,-4.4411999560996893166020527132786810398101806640625f,-3.717800081086156893661609501577913761138916015625f,-0.372799913112821013072561981971375644207000732421875f,1.2179000502735386479713497465127147734165191650390625f,-0.43939992395639304589849416515789926052093505859375f,-0.0866000460848213027276187858660705387592315673828125f,-1.6060000323653209619578774436376988887786865234375f,1.57920002898126909940401674248278141021728515625f,1.3555999282613397927121923203230835497379302978515625f,0.21939988323897097188819316215813159942626953125f,-0.2346000684887161735758809300023131072521209716796875f,1.4146000675201417795534553079050965607166290283203125f,-1.0215001017823805984363616516930051147937774658203125f,-3.881599953772127964413130030152387917041778564453125f,-3.818699982735515252585400958196260035037994384765625f,1.08599984323233567096167462295852601528167724609375f,-2.548300038465114436547764853457920253276824951171875f,0.42150009110942521228793111731647513806819915771484375f,-4.52510006699487465908759986632503569126129150390625f,-0.0610999479085208019313313343445770442485809326171875f,1.9434999325647950296769295164267532527446746826171875f,-1.3909999875873335639653305406682193279266357421875f,0.6911001115977752906616160544217564165592193603515625f,-1.4128999670878046313049480886547826230525970458984375f,-0.866200000505150224938688552356325089931488037109375f,-0.9838000800728787798021812704973854124546051025390625f,0.3601998771309860014611103906645439565181732177734375f,-0.83210006673783087904894273378886282444000244140625f,-1.7039999275132995393278179108165204524993896484375f,3.139699967100471500458525042631663382053375244140625f,-1.5898999300435188164470901028835214674472808837890625f,-0.27840008195340704144626897686976008117198944091796875f,1.17300001096725470262072121840901672840118408203125f,-3.03719996536821223998003915767185389995574951171875f,-4.59200003921240540449844047543592751026153564453125f,-3.63050000265985772784915752708911895751953125f,0.345300037127734160691261422471143305301666259765625f,1.8695999608278282000384251659852452576160430908203125f,-1.787300063785909554070485683041624724864959716796875f,-2.766000084649772983169668805203400552272796630859375f,-1.73329997023642068398885385249741375446319580078125f,1.232899916889518276974513355526141822338104248046875f,3.18199991162121254006933668279089033603668212890625f,-0.698799940001964881020057873683981597423553466796875f,-4.98910011768490146977228505420498549938201904296875f,4.09910000026375076487283877213485538959503173828125f,-2.090900052635372485809739373507909476757049560546875f,1.13590000245571243198128286167047917842864990234375f,-0.0031999966532001433705545423435978591442108154296875f,2.77369996023476250712747059878893196582794189453125f,-0.0380000552609545128035506422747857868671417236328125f,1.308799936276674902302374903229065239429473876953125f,2.07690005657672838168537055025808513164520263671875f,1.38150001863390325951286285999231040477752685546875f,-0.34920000756085034510078912717290222644805908203125f,-0.3189999179020528430328340618871152400970458984375f,1.0469999865368013747257691647973842918872833251953125f,-0.69990005199089744269969060042058117687702178955078125f,1.6174000249639164383808065394987352192401885986328125f,0.331199998605253398409331566654145717620849609375f,1.81409995824247527451689165900461375713348388671875f,-1.090899982525408429268054533167742192745208740234375f,2.1315999467313293536108176340349018573760986328125f,1.3280000608488922164696077743428759276866912841796875f,-1.9133999833822254998949574655853211879730224609375f,-1.834899954481425066177280314150266349315643310546875f,-2.7632999708324685883553684107027947902679443359375f,-1.9060000359602273789505488821305334568023681640625f,-1.5440000463128082941466345801018178462982177734375f,-1.324399926228821744444985597510822117328643798828125f,-0.2098999537363639422693495362182147800922393798828125f,2.27460005317777369526766051421873271465301513671875f,-3.745400119340420719282747086253948509693145751953125f,-0.2971999398499718392230306562851183116436004638671875f,-0.6477999762378632286896618097671307623386383056640625f,1.994100054578483049994019893347285687923431396484375f,3.550799988266825746308086308999918401241302490234375f,1.17739996204376229371746376273222267627716064453125f,0.4806000766210256625043939493480138480663299560546875f,-0.4526000659331683717567784697166644036769866943359375f,2.346199949245154403598689896170981228351593017578125f,-2.2931999695703400021784545970149338245391845703125f,-2.2684000803329045226064408780075609683990478515625f,3.5584000163882958389649502350948750972747802734375f,0.20260008860155920284995545443962328135967254638671875f,-1.6147999505370858042141435362282209098339080810546875f,-0.514699957321585355174420328694395720958709716796875f,1.1113000513136395142055334872566163539886474609375f,-1.7474999303743228296070810756646096706390380859375f,-0.6540999654881660685390443177311681210994720458984375f,2.66910003286004116063168112304992973804473876953125f,1.55340003810673987771906467969529330730438232421875f,-1.3387000469036396399502564236172474920749664306640625f,-0.75950002878159306174410403400543145835399627685546875f,-0.704199944826962909161238712840713560581207275390625f,-0.7872001903384935328489291350706480443477630615234375f,-1.0811998896405110581753206133726052939891815185546875f,-1.75609998519867627209123384091071784496307373046875f,2.384399906820059644729781211935915052890777587890625f,0.6285999598667035836996319630998186767101287841796875f,0.68260001266151693055661553444224409759044647216796875f,-3.24290005338415365798709899536333978176116943359375f,0.49409999525323566871293223812244832515716552734375f,-1.4188999710664145226246546371839940547943115234375f,-3.162399971897155825928393824142403900623321533203125f,4.3953001203879704661403593490831553936004638671875f,0.72620005567670009583025603205896914005279541015625f,-0.700099952740223852032386275823228061199188232421875f,0.24799995013326558979116498449002392590045928955078125f,2.27699985913187230579524111817590892314910888671875f,-3.5073000639349221074780871276743710041046142578125f,-0.47149994885921497900227450372767634689807891845703125f,-4.6458999863252046225170488469302654266357421875f,2.142399971673636382973882064106874167919158935546875f,-0.449699925619362783635324376518838107585906982421875f,-0.05499997016042545983083300598082132637500762939453125f,1.4116000529952348330198219628073275089263916015625f,2.3365000846460457495368245872668921947479248046875f,-2.35439998687655016595954293734394013881683349609375f,-1.4110000096969301797145135424216277897357940673828125f,0.088100069896877020170222749584354460239410400390625f,-3.003999929040671812430218778899870812892913818359375f,-0.1870999252885579000604820976150222122669219970703125f,-3.44129992606937928911747803795151412487030029296875f,1.094599996702374511414745938964188098907470703125f,-1.192699974137544582930559045053087174892425537109375f,1.1311999826982621630833136805449612438678741455078125f,0.4557998602658515086005763805587776005268096923828125f,-3.954200090411304824300486870924942195415496826171875f,1.006400033912062230712081145611591637134552001953125f,1.408300026208908040104006431647576391696929931640625f,1.647000016618520579214646204491145908832550048828125f,-1.5598999499179428962492011123686097562313079833984375f,1.121299990963935844234811156638897955417633056640625f,-2.658300088793784876628478741622529923915863037109375f,-3.08510002730190624475881122634746134281158447265625f,-2.190100011674314117726680706255137920379638671875f,0.585800119229406224263811964192427694797515869140625f,1.5810999393776048460580341270542703568935394287109375f,1.611599983481318165701168254599906504154205322265625f,-0.89100005628168421623058748082257807254791259765625f,-2.02979996107965643403758804197423160076141357421875f,-2.04750003635883270902695585391484200954437255859375f,-1.3116999587595474796586358934291638433933258056640625f,1.75540011550337116119635538780130445957183837890625f,0.689099979551136332389660310582257807254791259765625f,2.206599955523014244818114093504846096038818359375f,5.14719997600391376835204937378875911235809326171875f,3.270499922472983822530068209744058549404144287109375f,0.715800054353475712076715353759936988353729248046875f,0.8401000015228989870053055710741318762302398681640625f,0.762199933964013975895568364649079740047454833984375f,0.0093999586574715632281140642589889466762542724609375f,-3.61080007340833386564327156520448625087738037109375f,1.153399953058362736868502906872890889644622802734375f,-1.3884998734816937915326207075850106775760650634765625f,-0.9628000090204180683173262877971865236759185791015625f,1.221700147147476211983985194819979369640350341796875f,1.9872999822765595734352928047883324325084686279296875f,2.19990004005134043296720847138203680515289306640625f,2.818299933281540159413225410389713943004608154296875f,0.227200013722480331779252082924358546733856201171875f,-2.266199939559400799993227337836287915706634521484375f,-1.31199998031556663846686205943115055561065673828125f,0.1405999033018974220254904139437712728977203369140625f,-0.81359998121261778436519307433627545833587646484375f,-5.342400038710234611016858252696692943572998046875f,0.9949999189935632148973354560439474880695343017578125f,-3.63149995571375061587104937643744051456451416015625f,1.69649989860504746275182696990668773651123046875f,-1.6173000109799213763750458383583463728427886962890625f,1.34750002749264208290469468920491635799407958984375f,0.2144000665605088062903860190999694168567657470703125f,-1.90680009696334717972376893158070743083953857421875f,0.58849999794363949678910330476355738937854766845703125f,-1.6016999872580177655123634394840337336063385009765625f,0.09920009540319529861562841688282787799835205078125f,0.49129991716593390815859265785547904670238494873046875f,-1.3942000032395132080864641466177999973297119140625f,-3.54120000002086054990968477795831859111785888671875f,2.5752999970808634344621168565936386585235595703125f,3.75260008809864675782819176674820482730865478515625f,1.906599924342334606564008936402387917041778564453125f,1.041799891215564155544370805728249251842498779296875f,1.6180000366717575133179707336239516735076904296875f,0.4776999749571100295497672050260007381439208984375f,-0.532699960987269349743655766360461711883544921875f,0.769200012366473284686207989579997956752777099609375f,0.8630999068409190844164413647376932203769683837890625f,0.7586999298736454289127095762523822486400604248046875f,-0.4726000439539561615021057150443084537982940673828125f,-0.6457000369235863512784590056980960071086883544921875f,1.15799996941164184960371130728162825107574462890625f,-1.955600001426043288432765621109865605831146240234375f,3.37590001165717890074802198796533048152923583984375f,3.528799960528312684715501745813526213169097900390625f,1.93419993480592911083704166230745613574981689453125f,1.024400031617284323459671213640831410884857177734375f,-0.07889992250725830214008738039410673081874847412109375f,-2.17680003897920215649719466455280780792236328125f,3.459200023318828254303980429540388286113739013671875f,1.7111999532312147653811962300096638500690460205078125f,-0.502699930328132094103921190253458917140960693359375f,-1.22340005915463034824597343686036765575408935546875f,0.522499943077566086913066101260483264923095703125f,2.127900021421164300505779465311206877231597900390625f,-2.43919988954365418720726665924303233623504638671875f,0.992899983441831057717763542314060032367706298828125f,0.52299990477785474052296876834589056670665740966796875f,2.13830006640478931245752391987480223178863525390625f,-0.3036000623494370387334129191003739833831787109375f,-0.2758999814637010938866978904115967452526092529296875f,-1.8879000280149274004770632018335163593292236328125f,1.083999964803456617090660074609331786632537841796875f,-4.30030003008693473276480290223844349384307861328125f,-2.86040001058057047345073442556895315647125244140625f,-1.084900020679832710612799928640015423297882080078125f,-3.21490005562305380948373567662201821804046630859375f,-0.98129992701932788889962466782890260219573974609375f,-1.111799979002029648000871020485647022724151611328125f,-2.332700073695928466577242943458259105682373046875f,-2.890199984270334088165554931038059294223785400390625f,0.8426999668359755890634232855518348515033721923828125f,-0.6950000260956594733130486929439939558506011962890625f,0.108799950134755629704841339844278991222381591796875f,1.5920000497177244636048953907447867095470428466796875f,0.702299929712712600604618273791857063770294189453125f,-1.6414999834485353158441967025282792747020721435546875f,-0.81599993707239804052733234129846096038818359375f,0.046100263813135311519175729699782095849514007568359375f,-1.1943000359430886447853481513448059558868408203125f,1.721299954754112793153808524948544800281524658203125f,-0.508599998777358397461512140580452978610992431640625f,-0.73549994216114189526933841989375650882720947265625f,0.238399919120967584973413977422751486301422119140625f,0.4191000740990047379597172039211727678775787353515625f,-1.167100041592120884814676173846237361431121826171875f,-4.36969996957778850088516264804638922214508056640625f,-0.411399788769332364068986862548626959323883056640625f,-2.416399974046646637049207129166461527347564697265625f,3.081599989087881130700452558812685310840606689453125f,1.027799877163767749976841514580883085727691650390625f,-2.221399906693398040857800879166461527347564697265625f,-3.17260006440579900299781002104282379150390625f,-1.6001000759169461407083190351841039955615997314453125f,0.3609999775662984244917197429458610713481903076171875f,2.7760999650821087669783082674257457256317138671875f,-0.71989994574561766427223119535483419895172119140625f,0.151999929651619236636861387523822486400604248046875f,1.139999877512457349126862027333118021488189697265625f,-3.37669998062699949770149032701738178730010986328125f,-2.86589993402212872553036504541523754596710205078125f,-1.1624000332340589380919482209719717502593994140625f,2.575299988847971288663529776385985314846038818359375f,0.37100004260986907400621248598326928913593292236328125f,1.22919998878538638820145934005267918109893798828125f,-2.378500046297908010473065587575547397136688232421875f,0.5995999774426223893897258676588535308837890625f,-2.27470000843256769940126105211675167083740234375f,0.5832999469533550751521033816970884799957275390625f,-0.714500016190110276426139535033144056797027587890625f,-2.4686998757079248179024943965487182140350341796875f,-0.0071000247910630864822678631753660738468170166015625f,-1.383799996626375428121491495403461158275604248046875f,2.46220011981874886686227910104207694530487060546875f,0.80530010060668055782429064493044279515743255615234375f,-0.4911999054357394367542610780219547450542449951171875f,-0.6108999271675952247306895515066571533679962158203125f,1.8971000442370762328891942161135375499725341796875f,2.307599979059397288239097179030068218708038330078125f,0.670799957365544319287664620787836611270904541015625f,-0.8144000049442039479430377468816004693508148193359375f,-2.409400010047853424310915215755812823772430419921875f,-0.572100097434221854797442574636079370975494384765625f,-1.7632999093651762212431322041084058582782745361328125f,-3.015900038684160566759828725480474531650543212890625f,-0.590199967841803729839966763393022119998931884765625f,-0.60450000646710322360632972049643285572528839111328125f,-0.6711999864980586760765390863525681197643280029296875f,-0.88439991410300111684961166247376240789890289306640625f,1.195400111405550802601283066906034946441650390625f,3.25979986634552520996521707274951040744781494140625f,2.810699919427932957916027589817531406879425048828125f,0.8935000394061207718010564349242486059665679931640625f,-3.04830008644685168661681018420495092868804931640625f,4.5043000120639806738154220511205494403839111328125f,-2.613800034791978532666689716279506683349609375f,-1.288399958162008918094443288282491266727447509765625f,2.3599000688478355414190446026623249053955078125f,1.781499995015561221833877425524406135082244873046875f,1.650199917811156158364838120178319513797760009765625f,-3.177200020539761471383144453284330666065216064453125f,0.386900055719912761009737778294947929680347442626953125f,1.272600038142503109384051640518009662628173828125f,3.198299966324866350220190724940039217472076416015625f,0.786900000958144829610318993218243122100830078125f,-0.9818000990681350348410205697291530668735504150390625f,0.5018000596731908569125835128943435847759246826171875f,-0.328500005152076113290604553185403347015380859375f,-0.399200002047419122419569248449988663196563720703125f,0.5399999514222140195585097899311222136020660400390625f,0.28520010277181706559446183746331371366977691650390625f,1.4827000365734102071968436575843952596187591552734375f,1.140299959790705752737949296715669333934783935546875f,-1.26229995824843665985781626659445464611053466796875f,1.513400024360419759972273823223076760768890380859375f,-1.9288000470295540988985294461599551141262054443359375f,0.94439997689276911074784948141314089298248291015625f,-2.943900044827162165717027164646424353122711181640625f,-3.1984999496638781835144982323981821537017822265625f,-0.840500055521725908391772463801316916942596435546875f,-1.45349997721239976300466878456063568592071533203125f,1.2754998605400327260639414816978387534618377685546875f,1.8155999350413676296511766850017011165618896484375f,-0.819699999976157389625086580053903162479400634765625f,-1.2551001389861122792268588455044664442539215087890625f,1.8710999392285947440228710547671653330326080322265625f,4.1755999777704477793349724379368126392364501953125f,1.011000018302351310950371043873019516468048095703125f,-1.4392000029787421766513944021426141262054443359375f,2.35620000670775908702125889249145984649658203125f,-0.0339999355971816186183787067420780658721923828125f,-0.79879997107088573926603203290142118930816650390625f,2.218100022175908403454513972974382340908050537109375f,2.443299990129470966593316916259936988353729248046875f,0.8105000360012055704572730974177829921245574951171875f,0.8140000341683617524068949933280237019062042236328125f,4.790399979045986356140929274260997772216796875f,1.3788000197045509054305512108840048313140869140625f,-2.613500007964671301152748128515668213367462158203125f,1.2940999785453097814524880959652364253997802734375f,1.24600002467259596272697308450005948543548583984375f,-3.711200019057095733643336643581278622150421142578125f,1.2052999535322204138765300740487873554229736328125f,-1.1960000102370964558673449573689140379428863525390625f,-1.9077999594554302387194866241770796477794647216796875f,-3.239100123719870882865734529332257807254791259765625f,-2.05840002622306261770290802814997732639312744140625f,2.41210001687481945253921367111615836620330810546875f,1.703900093442202834381760112592019140720367431640625f,1.453900100408494378001478253281675279140472412109375f,1.0033000030189744844477672813809476792812347412109375f,-0.615899921095372615553742434713058173656463623046875f,-0.122299941670894529721635990426875650882720947265625f,0.7456999437540774433585966107784770429134368896484375f,-0.07839996632039702983973938898998312652111053466796875f,-4.2285000459998851596310487366281449794769287109375f,-0.26300004008039767899873595524695701897144317626953125f,3.235099954919517184492860906175337731838226318359375f,1.0929998916462067004573555095703341066837310791015625f,-0.89260001232624031075602033524774014949798583984375f,1.09069996863901597095036777318455278873443603515625f,0.4367999622941012294319307329715229570865631103515625f,2.85120002045407883173311347491107881069183349609375f,-0.05499990426004064403286974993534386157989501953125f,2.2467999431833636236888196435756981372833251953125f,-0.1110999647095807230101627283147536218166351318359375f,-3.1393000166833413544509312487207353115081787109375f,0.80749999448656961664738673789543099701404571533203125f,-3.5252999196320775610047348891384899616241455078125f,1.9090000645667313250442020944319665431976318359375f,0.3771000551149243928961141136824153363704681396484375f,-0.5068000119522206858846402610652148723602294921875f,0.13899995705485201913376158699975349009037017822265625f,0.42420004041791015136908526983461342751979827880859375f,2.166200011550635462498348715598694980144500732421875f,2.384699927910417738274873045156709849834442138671875f,-2.935399977313725816685519021120853722095489501953125f,0.94930006354749296715311857042252086102962493896484375f,0.7373999277710898692106411544955335557460784912109375f,0.45029994447976318117099481241893954575061798095703125f,-1.163499925393610912038866445072926580905914306640625f,-2.117600090184807992699234091560356318950653076171875f,-0.6851000025287283801134208260918967425823211669921875f,-0.8654000336214882072027876347419805824756622314453125f,5.13140004993080811601657842402346432209014892578125f,3.9828999480143192357672887737862765789031982421875f,-0.28009995599985160197320510633289813995361328125f,-2.900000024419278066289962225710041821002960205078125f,2.29740007542297330900282759102992713451385498046875f,1.186000045944004543940764051512815058231353759765625f,1.9804000249601898531182087026536464691162109375f,1.91050000560283717021548000047914683818817138671875f,-0.23420007159858957646036969890701584517955780029296875f,1.68670001255273849238847105880267918109893798828125f,-1.3147000347517427965016167945577763020992279052734375f,5.31760011279731958921956902486272156238555908203125f,-2.46610001246780097261535047437064349651336669921875f,2.502600009420515192459788522683084011077880859375f,1.704699964325131045228545190184377133846282958984375f,0.3489000057116149466907017995254136621952056884765625f,0.76719999889582324303916038843453861773014068603515625f,-4.09429998943284179091506302938796579837799072265625f,0.0038999952435484763757500559222535230219364166259765625f,2.66129996213018937822880616295151412487030029296875f,1.6504000399544820343322726330370642244815826416015625f,-0.908099939064682448730536634684540331363677978515625f,0.5486000530734649327513352545793168246746063232421875f,1.9779001133427029923694817625801078975200653076171875f,-1.528499957244843887593788167578168213367462158203125f,-3.01429995972365105671997298486530780792236328125f,-2.20980007686168011815652789664454758167266845703125f,2.65319999331906419826054843724705278873443603515625f,2.9649999577179553256200961186550557613372802734375f,-2.287700013402103049742208895622752606868743896484375f,-2.053899996864051669120954102254472672939300537109375f,-0.44589997917264667659509314034949056804180145263671875f,0.8296999535590410257412941064103506505489349365234375f,-0.83430004227608467459731400595046579837799072265625f,1.571900043817608061402779640047810971736907958984375f,0.3662999530404815740070034735254012048244476318359375f,0.69150001300871288822946780783240683376789093017578125f,-1.247899966584891107146404465311206877231597900390625f,-1.484399998722970348552507857675664126873016357421875f,-1.95709998552650343839331981143914163112640380859375f,3.9054999878704546034668965148739516735076904296875f,-1.30359999477267241019262655754573643207550048828125f,1.0328999852672235970629799339803867042064666748046875f,0.7444999656006690003096082364208996295928955078125f,-0.36120001605823570667297417458030395209789276123046875f,2.017199961493909565746207590564154088497161865234375f,0.436100010437519447492604740546084940433502197265625f,0.02560006842538775639894765845383517444133758544921875f,3.2259000110983837572575794183649122714996337890625f,1.255100060233473424631256420980207622051239013671875f,2.02419993055909941404024721123278141021728515625f,0.518199998739361955557569672237150371074676513671875f,3.078399960732460360901541207567788660526275634765625f,-2.180100041476638050852443484473042190074920654296875f,2.943899951843917062177524712751619517803192138671875f,2.53350002551078912205184678896330296993255615234375f,1.2774999644234770190820427160360850393772125244140625f,1.0739000572323795790197209498728625476360321044921875f,1.3112000445939602233380583129473961889743804931640625f,2.80589995477199405371493412530981004238128662109375f,-1.89669997986331662076509019243530929088592529296875f,1.2021998836353426920453557613654993474483489990234375f,-1.5596000186815868193690448606503196060657501220703125f,0.72739997757822294488505576737225055694580078125f,1.8190000111460686849085277572157792747020721435546875f,2.590699993151426294701877850457094609737396240234375f,1.1471999799340981684991902511683292686939239501953125f,-1.805999973021446436405312852002680301666259765625f,1.1805999103054405363621981450705789029598236083984375f,-1.09560002664625688595378960599191486835479736328125f,-2.897299953945726702642105010454542934894561767578125f,-1.4538000481285153764332562786876223981380462646484375f,-1.6722999993942675445168788428418338298797607421875f,-3.5054000509068377056109966360963881015777587890625f,-1.1413000569388256355551902743172831833362579345703125f,2.540499992117286609527582186274230480194091796875f,-1.173799987760186791518890458974055945873260498046875f,-5.3683998912185426632959206472150981426239013671875f,-0.0743998885288845723806616661022417247295379638671875f,0.30149996778368848282525505055673420429229736328125f,0.09860000691711878761935849979636259377002716064453125f,-5.33790000877380510502234756131656467914581298828125f,-0.40140002310872180402157027856446802616119384765625f,-1.9459000210821628673585337310214526951313018798828125f,0.7307000229164943849724522806354798376560211181640625f,1.1048999870777134191257573547773063182830810546875f,-2.890200126893073928613375755958259105682373046875f,-0.28899992496147774989623258079518564045429229736328125f,-0.70680004954039876707838629954494535923004150390625f,3.955100062599033350352328852750360965728759765625f,2.7144999951049673114766846993006765842437744140625f,1.3745999098658554515850482857786118984222412109375f,-3.415400012536348395286722734454087913036346435546875f,-2.62130005697607781911528945784084498882293701171875f,-2.22550005272775752729330633883364498615264892578125f,0.81270003277361346771812122824485413730144500732421875f,1.96699996565654888769358876743353903293609619140625f,3.485799965319037685418379624024964869022369384765625f,-1.6600000199303017733853948811884038150310516357421875f,-3.913799914297459991985306260176002979278564453125f,0.410100096350162868219513256917707622051239013671875f,0.758499945305287415209249957115389406681060791015625f,-3.33940013618618269930493624997325241565704345703125f,-2.7801999594412745153704236145131289958953857421875f,1.6259999542459848687059320582193322479724884033203125f,-0.69200008127093248777583767150645144283771514892578125f,0.9067000745788209581377259382861666381359100341796875f,0.539000049777329071076792388339526951313018798828125f,-0.6195000677108766229395087066222913563251495361328125f,0.4440000273324555823961645728559233248233795166015625f,0.13649994278699129068854745128192007541656494140625f,-3.854999885894358158111572265625f,-0.0548000768803067472845214069820940494537353515625f,0.9154999626129869394475235822028480470180511474609375f,-2.042599955478310302936506559490226209163665771484375f,-0.192799926635622842496786688570864498615264892578125f,-2.400400001025198459814191664918325841426849365234375f,0.6859000784516329218831742764450609683990478515625f,1.367899989961088191847693451563827693462371826171875f,-0.0376000012621278134616886745789088308811187744140625f,-0.4639000120073557553723730961792171001434326171875f,-1.405399985118211514389940930414013564586639404296875f,0.0238999503910550803453816115506924688816070556640625f,2.397100087785720035071790334768593311309814453125f,-0.37099997529387496175701244283118285238742828369140625f,1.5063000214740636106824922535452060401439666748046875f,0.7346999340012663193277830941951833665370941162109375f,-0.8246000012844791893229512425023131072521209716796875f,2.4873000079810605456032135407440364360809326171875f,0.54850000898912620872494017021381296217441558837890625f,1.1438999463118619814849807880818843841552734375f,1.45350004316866421305576295708306133747100830078125f,-0.98210001048594808281677615013904869556427001953125f,2.0726999537229513492775367922149598598480224609375f,1.0653999853417268628419378728722222149372100830078125f,-3.676699820923803319061562433489598333835601806640625f,-1.326800003179161802080443521845154464244842529296875f,0.059100036138297007415332018354092724621295928955078125f,-0.14940003083497377378563442107406444847583770751953125f,-1.24780005407780425485952946473844349384307861328125f,-5.48970003950148832672084608930163085460662841796875f,-1.1076999500721689262405789122567512094974517822265625f,0.30670011809021158200039280927740037441253662109375f,-3.6379999954514214977052688482217490673065185546875f,3.00370002987906303815179853700101375579833984375f,-3.09190008265376103935295759583823382854461669921875f,-2.484099914358557814608730041072703897953033447265625f,-1.1166999310433853853652408361085690557956695556640625f,-0.999800026011467224407169851474463939666748046875f,0.146399993474035650109499329118989408016204833984375f,1.1607000374451283963850301006459631025791168212890625f,1.5347999903790647824308734925580210983753204345703125f,0.519199985618889581218127204920165240764617919921875f,-0.23279993540868282320133175744558684527873992919921875f,-3.70450000471621621045414940454065799713134765625f,-0.92160014044642313280064627178944647312164306640625f,2.938200039903818616693342846701852977275848388671875f,0.3550000142864881969018142626737244427204132080078125f,-1.4752000175558028782774044884718023240566253662109375f,2.110299984042345755597125389613211154937744140625f,-1.0061000098228454202597959010745398700237274169921875f,-1.58050008152425380814065647427923977375030517578125f,3.120200030948221847637569226208142936229705810546875f,0.44529996366500856908032801584340631961822509765625f,2.48790005515366718924497035914100706577301025390625f,2.141600014829635245661165754427202045917510986328125f,-3.17919995145797873448145764996297657489776611328125f,1.100700040127336354345288782496936619281768798828125f,2.276699923443794570943055077805183827877044677734375f,-1.0977999735370271849888013093732297420501708984375f,-0.3888999765053393620206634295755065977573394775390625f,-2.39879993536397950748551011201925575733184814453125f,3.13509993668422026757980347611010074615478515625f,0.8169001349754625440624522525467909872531890869140625f,-0.7518000191234042173249463303363882005214691162109375f,-2.016000038176773756504189805127680301666259765625f,-0.2431001010030506837011898824130184948444366455078125f,0.962500007674097712850880270707421004772186279296875f,-5.24460000050216734734931378625333309173583984375f,-0.37149999689683232961812109351740218698978424072265625f,0.639200053605438966997098759748041629791259765625f,-2.20359999503344550220163000631146132946014404296875f,0.76599989236891286026320813107304275035858154296875f,-4.56210001339167359191151263075880706310272216796875f,-0.69379994877502271233282726825564168393611907958984375f,-0.54850003691017634732673968755989335477352142333984375f,-0.24770003424510511624845321421162225306034088134765625f,-1.1374000750690702599143833140260539948940277099609375f,-3.661699992454795715701720837387256324291229248046875f,0.61680001146793450583771800665999762713909149169921875f,-3.000399945704639304011607237043790519237518310546875f,-2.6297999519154426906197841162793338298797607421875f,-1.0916000205293296954067727710935287177562713623046875f,-1.80350005333870644363969404366798698902130126953125f,1.368400072323530114459799733594991266727447509765625f,-2.20610002304762442548735634773038327693939208984375f,-0.9243999035418035692401872438495047390460968017578125f,1.6518999754026537640783089955220930278301239013671875f,-0.215500066995620187526583322323858737945556640625f,-0.84569995813369691628480495637631975114345550537109375f,-2.215899948625265292179165044217370450496673583984375f,-0.48599996037408688120962096945731900632381439208984375f,0.28089990325123082026692600265960209071636199951171875f,1.03039999035224294487989027402363717555999755859375f,2.327400079055129911154153887764550745487213134765625f,2.7882000163599851561002651578746736049652099609375f,4.15920003324672649114290834404528141021728515625f,0.603500015806406775453751833993010222911834716796875f,0.87019994169026571029235128662548959255218505859375f,0.51160005744695646878739125895663164556026458740234375f,5.5757000510610641441644474980421364307403564453125f,2.18890010551214242440210000495426356792449951171875f,3.622499965131281474128854824812151491641998291015625f,1.437399960888921413015850703231990337371826171875f,0.844100051058083611366100740269757807254791259765625f,-2.41590000744760136086597412941046059131622314453125f,-0.698300090898573699860207852907478809356689453125f,-1.6427999417044230057172171655111014842987060546875f,0.879700022923945113006993778981268405914306640625f,2.607699982631206392369449531543068587779998779296875f,0.24819988081529775758582445632782764732837677001953125f,1.01250004412606386949846637435257434844970703125f,0.50420005904436104859911438325070776045322418212890625f,-4.08550017702207046710327631444670259952545166015625f,-5.01849987694621102463088391232304275035858154296875f,0.3745999534517527873589415321475826203823089599609375f,-0.2762000191316016373122010918450541794300079345703125f,4.5393000437289483528502387343905866146087646484375f,-2.340099996307493324110282628680579364299774169921875f,-1.49009990090280819430290648597292602062225341796875f,2.11830006076097543399328060331754386425018310546875f,2.02540002069473334955773680121637880802154541015625f,-1.964200044701993164153464022092521190643310546875f,-2.21050006541237120671894444967620074748992919921875f,-1.223900008878114231691824898007325828075408935546875f,1.759899877777695298419757818919606506824493408203125f,-0.251599931960552691823096438383799977600574493408203125f,-2.9616999483287340666493037133477628231048583984375f,-2.76799997546523801617013305076397955417633056640625f,-0.771799933199584398124670769902877509593963623046875f,0.1272999825745821933509205337031744420528411865234375f,1.5077000169038770049922959515242837369441986083984375f,1.242200007836520736503871376044116914272308349609375f,-2.9847000603444886479564956971444189548492431640625f,-1.4497999499782909804679320586728863418102264404296875f,-1.2781000554427495163878347739228047430515289306640625f,-1.134700079213082357654229781473986804485321044921875f,0.2342000889584421319256080096238292753696441650390625f,-0.95699993821978524000115839953650720417499542236328125f,1.565000094473361524904930774937383830547332763671875f,2.73239988308623527046847812016494572162628173828125f,-0.087000048413874153663272181802312843501567840576171875f,1.0551000202238542957644540365436114370822906494140625f,-0.7418999773584318280228444564272649586200714111328125f,-0.3719000207394362877977300740894861519336700439453125f,-3.4657000444114220982783081126399338245391845703125f,0.9122000108912586835430147402803413569927215576171875f,-0.5383000076755857232768676112755201756954193115234375f,-1.1229000588044522057629137634648941457271575927734375f,-2.688799894553423275311843099188990890979766845703125f,0.652800002892316211244860824081115424633026123046875f,-4.166300011117755275336094200611114501953125f,0.9688999774739135428802683236426673829555511474609375f,0.160699912126362753905794988895650021731853485107421875f,2.235999987959861545760986700770445168018341064453125f,-2.0016999248966573787811285001225769519805908203125f,-1.1048999595478183888275225399411283433437347412109375f,-1.4809000290170306612935746670700609683990478515625f,-4.07470003342926379019672822323627769947052001953125f,-0.05260004449412225557836109146592207252979278564453125f,-0.99129999910369548388189286924898624420166015625f,1.123799976919591880886173385079018771648406982421875f,1.535000079032033948323032745975069701671600341796875f,-0.4299999583512554846009834363940171897411346435546875f,0.5898000268682836999545315848081372678279876708984375f,-1.00839994665086241099061226123012602329254150390625f,-2.92129999293833986229174115578643977642059326171875f,0.7115000492557899125500853187986649572849273681640625f,-0.0204999578446150021449057021527551114559173583984375f,2.322600043432414107513750423095189034938812255859375f,1.1316999745920295339374206378124654293060302734375f,-1.3333000569425534909129282823414541780948638916015625f,-0.3534000265583390198997904008137993514537811279296875f,1.786699987313896453855477375327609479427337646484375f,1.3354999295324085917258116751327179372310638427734375f,-2.7192999968647963981993598281405866146087646484375f,0.6572001296535143755050967229180969297885894775390625f,0.5818999522313486050251185588422231376171112060546875f,1.8107999922715141138951366883702576160430908203125f,-0.793199985291063836001512754592113196849822998046875f,-0.0448000440418721002089341709506697952747344970703125f,-1.184299967509506945617658857372589409351348876953125f,0.750799938906728581144989220774732530117034912109375f,-2.53650004951655905216512110200710594654083251953125f,1.987500050850213195730020743212662637233734130859375f,1.27990000061914432905041394405998289585113525390625f,0.341899957484006478125593275763094425201416015625f,-0.329200027453900367913064428648795001208782196044921875f,2.62680009949654280632103109383024275302886962890625f,-0.3657000124484304226513131652609445154666900634765625f,-1.4701000221237550125152893087943084537982940673828125f,1.742000003859402124106736664543859660625457763671875f,-0.7157000993594540005915405345149338245391845703125f,-1.2500000742450356749912998566287569701671600341796875f,2.88669994031935939204913665889762341976165771484375f,-4.3366000931538639662221612525172531604766845703125f,-1.2038000056415796290565367598901502788066864013671875f,0.369099946880340812338516798263299278914928436279296875f,-1.324899909461290103962483044597320258617401123046875f,-0.7107000178687286062739758563111536204814910888671875f,-4.39819994844794148747268991428427398204803466796875f,2.965600011111796607821133875404484570026397705078125f,-0.1671000274918970074367052802699618041515350341796875f,4.201400035774707220070922630839049816131591796875f,-3.868099964955448388792547120829112827777862548828125f,-0.705700069091469206483679954544641077518463134765625f,3.5461999837785942446544140693731606006622314453125f,2.64049995333701392752345782355405390262603759765625f,-1.7578001020036648771593945639324374496936798095703125f,-0.9147000448100273128915205234079621732234954833984375f,-1.28179998982399734330783758196048438549041748046875f,0.729799914830178142466365898144431412220001220703125f,-0.98300003074854569451446195671451278030872344970703125f,0.7716998734131461645802119164727628231048583984375f,-0.4120998725757007985492919033276848495006561279296875f,0.4629999469108880827405982927302829921245574951171875f,-0.38290003609880762791561892299796454608440399169921875f,1.4883000609658640911447946564294397830963134765625f,0.0786999873362488333583542043925262987613677978515625f,-0.695600075633824754817169377929531037807464599609375f,-1.166300020244717128292677443823777139186859130859375f,-2.124099992142618464896486329962499439716339111328125f,0.9089999320395281134921106058754958212375640869140625f,-1.8721999505229280913454203982837498188018798828125f,0.967300088875739927374297622009180486202239990234375f,1.85850008269399413762812400818802416324615478515625f,2.397099940077961122142369276843965053558349609375f,-3.436400045907496814123760486836545169353485107421875f,-0.42590002517998215569861031326581723988056182861328125f,0.884400048809498517954352792003192007541656494140625f,-2.394000008035451987353781078127212822437286376953125f,-0.297599934225529716425029391757561825215816497802734375f,-3.55650005244091094169789357692934572696685791015625f,-0.11160010773837658604179523536004126071929931640625f,1.2993998962029815746888061767094768583774566650390625f,-1.040000077672302847275886961142532527446746826171875f,0.5270000765770674888699431903660297393798828125f,-2.461000027168543713429471608833409845829010009765625f,-0.942199960674346304045911892899312078952789306640625f,1.451700120548903516493055576574988663196563720703125f,3.3356999856263396253552855341695249080657958984375f,-2.100699959661067151017732612672261893749237060546875f,-0.5461999697342518889087159550399519503116607666015625f,4.70549998513236555908179070684127509593963623046875f,4.20469997259527428212777522276155650615692138671875f,-1.3797999021828186716476238871109671890735626220703125f,-1.4472000598602001719683585179154761135578155517578125f,0.229599918620289056292449458851478993892669677734375f,-0.53229993597120095483177237838390283286571502685546875f,-0.24810001874864184845392856004764325916767120361328125f,1.958699966139345605142807471565902233123779296875f,-0.0859000302463777121175780848716385662555694580078125f,3.958399975968898143463547967257909476757049560546875f,-2.875500014431774165046817870461381971836090087890625f,-0.932400004810094884533100412227213382720947265625f,0.724500045023857897064090138883329927921295166015625f,1.0115999905034895878230827293009497225284576416015625f,1.492500031478704247689393014297820627689361572265625f,-3.07459995512813311080435596522875130176544189453125f,2.09299997699260842409785254858434200286865234375f,2.548000025402755941428267760784365236759185791015625f,-0.71509998837262234161471496918238699436187744140625f,1.1792000129066402269728541796212084591388702392578125f,-0.8411999400064338328064650340820662677288055419921875f,1.553899962274730039979431239771656692028045654296875f,2.381600053982437348309986191452480852603912353515625f,0.14279999018907574281911365687847137451171875f,-1.6654999934211378587178842280991375446319580078125f,-4.186399988947808736838851473294198513031005859375f,-2.099000033311545276859533259994350373744964599609375f,-1.2100000717677172890063275190186686813831329345703125f,-2.96309997125119028993367464863695204257965087890625f,0.56059998862966808896857173749594949185848236083984375f,0.62569996189624144111718351268791593611240386962890625f,0.3977000207781793417183280325843952596187591552734375f,-3.18709994968920984348415004205889999866485595703125f,-0.846099972048401927082750262343324720859527587890625f,1.68459999018311545881942947744391858577728271484375f,-2.3386999999836088903748532175086438655853271484375f,-3.300799983796476322339685793849639594554901123046875f,-0.0697000418111672903620501529076136648654937744140625f,-0.51070001125633712746321180020458996295928955078125f,-1.83169988350868351290046120993793010711669921875f,2.716100054079294334741234706598334014415740966796875f,-1.716700030285121414408422424457967281341552734375f,1.4089999548941862794748658416210673749446868896484375f,0.81900004446879126618341615539975464344024658203125f,-0.30870004504099401998473695130087435245513916015625f,1.36840002957582473896991359652020037174224853515625f,-0.2544000121340166398198334718472324311733245849609375f,-2.747800017197430832993632066063582897186279296875f,-0.537399984302371791500263498164713382720947265625f,1.147299934406578625356587508576922118663787841796875f,5.0824999165534965328561156638897955417633056640625f,-4.20800000720470990245303255505859851837158203125f,-0.09129987257421134927426464855670928955078125f,-0.9926999096341428963086173098417930305004119873046875f,-2.101200007434190553112784982658922672271728515625f,-3.83829997450187843099911333411000669002532958984375f,0.0879999850019803631084869266487658023834228515625f,0.0797000278040760345987791879451833665370941162109375f,0.1954999688714755823326640893355943262577056884765625f,0.245100008042156536358646690132445655763149261474609375f,-0.56499996319413270384757197462022304534912109375f,-1.2754000053465361919080578445573337376117706298828125f,-0.9643999751605090153816490783356130123138427734375f,0.7182000219851720768104996750480495393276214599609375f,-0.562299998332559169966771150939166545867919921875f,0.4271000174522410208766132200253196060657501220703125f,3.529500009596348686358169288723729550838470458984375f,0.940599968345463111063509131781756877899169921875f,1.555700009617207779655245758476667106151580810546875f,-0.344399912240355654535761686929618008434772491455078125f,0.35530000982135589993049507029354572296142578125f,-0.613000034287572770352880979771725833415985107421875f,-3.5962000399559741481425589881837368011474609375f,-2.294800064307449982692332923761568963527679443359375f,-1.138999930605291144303237160784192383289337158203125f,-1.5831999693095675230125607413356192409992218017578125f,-1.035499885462225932997171184979379177093505859375f,-0.597700004498661119356484050513245165348052978515625f,3.47839991152137617547168702003546059131622314453125f,-0.28689998203367095275240217233658768236637115478515625f,-0.811499933362008096793260847334749996662139892578125f,-2.737100012348592681377112967311404645442962646484375f,0.356799993996321784806013965862803161144256591796875f,-0.691299884662777142096956595196388661861419677734375f,0.9858000223346066182017466417164541780948638916015625f,-1.5267999925434576624638793873600661754608154296875f,-2.506199888839572675891531616798602044582366943359375f,-1.143199973183870898907343871542252600193023681640625f,-2.604899867644906663599613239057362079620361328125f,3.58930001225024586375411672634072601795196533203125f,-2.3835999718248839229772784165106713771820068359375f,-0.12349992053210712139588167701731435954570770263671875f,0.35090001553148086888000989347347058355808258056640625f,3.359100106248258388319527512066997587680816650390625f,3.7379999457374228910566671402193605899810791015625f,0.890299951241166009907601619488559663295745849609375f,-0.80249997422098939825474417375517077744007110595703125f,0.9692999444499601491287421595188789069652557373046875f,2.263099978105722609456051941378973424434661865234375f,0.06339992794990501057128540196572430431842803955078125f,1.05540000068992423365443755756132304668426513671875f,-1.428599982814490854110545114963315427303314208984375f,5.1817999660566460562449719873256981372833251953125f,0.22609995107501779099123950800276361405849456787109375f,0.941299955610930272342784519423730671405792236328125f,-2.444199963676929154843264768715016543865203857421875f,-0.0472000454246981338002342454274185001850128173828125f,3.11830006903111911498172048595733940601348876953125f,3.1215000247433781765948879183270037174224853515625f,-0.6623000217646362042245300472131930291652679443359375f,-4.0146999486789116673435273696668446063995361328125f,0.04770004398673710621636701034731231629848480224609375f,2.439199948813023155480550485663115978240966796875f,-0.02670001303702651540561419096775352954864501953125f,-0.0955000174120061995353125894325785338878631591796875f,2.4083999527230854909021218190900981426239013671875f,-0.81849996873736241553842774010263383388519287109375f,-0.33599992086738417196301043077255599200725555419921875f,1.153900051756201694530545864836312830448150634765625f,2.827699989448487993826120145968161523342132568359375f,-1.64349998689815368635436243494041264057159423828125f,1.6550999985426673166699629291542805731296539306640625f,-3.147800117035210565319403031026013195514678955078125f,-0.2180999773606657754498883150517940521240234375f,-1.095699962194264909243202055222354829311370849609375f,-1.074199957753717793451642137370072305202484130859375f,0.51110004824921551058736213235533796250820159912109375f,1.2329000696264216241360145431826822459697723388671875f,2.048399937337636256273754042922519147396087646484375f,-0.05669995179325415524118625398841686546802520751953125f,-0.9028000050343567561839108748245052993297576904296875f,1.5844000078871849357398104984895326197147369384765625f,2.047599906360357824297580009442754089832305908203125f,1.1883999879270781274698265406186692416667938232421875f,-0.72020005354210692161132101318798959255218505859375f,1.627099998565018523777325754053890705108642578125f,-1.0644000927492978103572340842220000922679901123046875f,-0.8150999234408147220420914891292341053485870361328125f,1.408400065189599814630128093995153903961181640625f,1.540500089291484631104367508669383823871612548828125f,-2.310799950790406231959650540375150740146636962890625f,3.698099948564171679521450641914270818233489990234375f,0.39930004056245049781637135311029851436614990234375f,-3.91649993220716652331248042173683643341064453125f,-2.39850000360608106575455167330801486968994140625f,1.925699939022958329104540098342113196849822998046875f,1.1090999540328978500980383614660240709781646728515625f,-1.277200071091950661639202735386788845062255859375f,0.2381999406248329620439108111895620822906494140625f,-0.7626999804891652079419372967095114290714263916015625f,-2.702500001601874313195139620802365243434906005859375f,-0.652899831170587585660314289270900189876556396484375f,4.7885000049471866390149443759582936763763427734375f,0.68389994294047273815095877580461092293262481689453125f,1.559199958554656095799373360932804644107818603515625f,-0.03670002505108704848879597193445079028606414794921875f,-1.024799953778087857614309541531838476657867431640625f,-0.4850999552361676681044855286017991602420806884765625f,0.1141000328227865789898487491882406175136566162109375f,2.25770001705288958504524998716078698635101318359375f,0.9463000529527665349860399146564304828643798828125f,0.8246001053690907856008607268449850380420684814453125f,1.565400042729824026110918566701002418994903564453125f,-0.1637000645801440779081303844577632844448089599609375f,-0.70639998322948682751842852667323313653469085693359375f,-2.776099918292462831459488370455801486968994140625f,2.557600026929379044560164402355439960956573486328125f,-1.0992999761894346466561955821816809475421905517578125f,1.153200042381137269131841094349510967731475830078125f,0.3208000290028760748128888735664077103137969970703125f,1.6751000232785953070191453662118874490261077880859375f,2.68509994070753332806589241954497992992401123046875f,0.321499949492513703575014005764387547969818115234375f,4.09660002570748371653053254703991115093231201171875f,0.025100055018067113365987097495235502719879150390625f,1.115400038166343588130757780163548886775970458984375f,-2.253199968545883447035294011584483087062835693359375f,1.5419999589696533615068574363249354064464569091796875f,1.378999948617070980816379233147017657756805419921875f,0.047899944028258811812293060938827693462371826171875f,0.32929998349100342291961851515225134789943695068359375f,0.80729996055811559241277564069605432450771331787109375f,0.863299944345653269550666664144955575466156005859375f,1.6578000293232502126983263224246911704540252685546875f,-1.4137000095948568656467614346183836460113525390625f,1.040799892042577834416761106695048511028289794921875f,0.79160000672712926839125202604918740689754486083984375f,-4.7103000414863220868255666573531925678253173828125f,-2.45830004489123776778569663292728364467620849609375f,-0.851499995946883192488030545064248144626617431640625f,2.085499997612088929344054122339002788066864013671875f,1.6045000196918850488003727150498889386653900146484375f,0.245599940368534053192206556559540331363677978515625f,-1.4875000452250246585350623718113638460636138916015625f,-2.41570000350773295139106267015449702739715576171875f,-0.46870007929503909593904609209857881069183349609375f,0.51789998124390879130629627979942597448825836181640625f,-0.4845000519342719247362083478947170078754425048828125f,2.21999991038814226129716189461760222911834716796875f,-0.79929997827559795542384790678624995052814483642578125f,0.87049999664351229711201085592620074748992919921875f,-0.567000031165779017072736678528599441051483154296875f,-1.971300027397274501339552443823777139186859130859375f,1.85630003231465945390255001257173717021942138671875f,0.019600017284601367606455823988653719425201416015625f,-1.177800046999752936471850262023508548736572265625f,1.168900066173076179865120138856582343578338623046875f,-0.153699947643280321329939397401176393032073974609375f,-0.35590004719644896535868383580236695706844329833984375f,-0.7092999623313540613622762975865043699741363525390625f,1.995499999642373012420648592524230480194091796875f,2.16270000451728794388372989487834274768829345703125f,3.0899999583140012049398137605749070644378662109375f,1.1161000592946999088184156789793632924556732177734375f,-2.29459995665550398058485370711423456668853759765625f,0.0720000042319284272451795914093963801860809326171875f,-1.827400008349120863471171105629764497280120849609375f,3.321899929916857896472492939210496842861175537109375f,0.22459997124001251922464916788158006966114044189453125f,0.1013000293716779243169412438874132931232452392578125f,-0.704900061397254251716049111564643681049346923828125f,0.392499987222255430907580375787802040576934814453125f,0.3685999891288582741566415279521606862545013427734375f,0.96520004473179465964705059377592988312244415283203125f,2.653999974675477435681614224449731409549713134765625f,0.083900021693110726772601992706768214702606201171875f,2.1520999980807307139230033499188721179962158203125f,0.44760004272460918173237587325274944305419921875f,0.47450002740323438121805565970134921371936798095703125f,1.2590000419542197729327881461358629167079925537109375f,-0.1168000000312916508704574880539439618587493896484375f,0.64179997708350544627364797634072601795196533203125f,-0.771400011634825855111330383806489408016204833984375f,-2.2042998884774736012559515074826776981353759765625f,-1.1826999725729219203884667876991443336009979248046875f,-0.09880002724826386639733755146153271198272705078125f,-1.06670006796643068724961267434991896152496337890625f,-3.2984999625533806266730607603676617145538330078125f,0.55710005816966290392855398749816231429576873779296875f,1.6110000130310642152409172922489233314990997314453125f,1.26109997294247033750025366316549479961395263671875f,0.73159995570927804919847403652966022491455078125f,-1.555899949482082877949551402707584202289581298828125f,-0.326600006298721101671844735392369329929351806640625f,-2.050199959310889941122013624408282339572906494140625f,3.33210000649988469234585863887332379817962646484375f,-2.459400014927982880408308119513094425201416015625f,2.32779986561536755829138201079331338405609130859375f,1.773599942953883346064003490027971565723419189453125f,0.173599946939944960178081601043231785297393798828125f,-1.81730001870989799073186077293939888477325439453125f,-3.214499991081656293800961066153831779956817626953125f,0.622199973973631248469473575823940336704254150390625f,-0.9179999674931151876222656937898136675357818603515625f,0.5042999555513272280649061940493993461132049560546875f,-1.7198000135689976985986504587344825267791748046875f,-0.807600077705085350743274830165319144725799560546875f,3.527400049532206427471692222752608358860015869140625f,1.680300098334253089404910497250966727733612060546875f,0.8216000783458345946286272010183893144130706787109375f,-0.1352000520333647148873978949268348515033721923828125f,-1.7721000218108289292473500609048642218112945556640625f,-1.909800025154649194547573642921634018421173095703125f,1.91190004692822679288610743242315948009490966796875f,0.96820004079788990480892607592977583408355712890625f,-0.3735999550983313266527829910046420991420745849609375f,-0.17470000163391308234395182807929813861846923828125f,-1.688199934142828428917937344522215425968170166015625f,-0.041000103853641889362080519276787526905536651611328125f,1.626199895228445324590893505956046283245086669921875f,-0.27110002196729243717499002741533331573009490966796875f,-1.1716000177726153541613030029111541807651519775390625f,-0.22800001692771931782743877192842774093151092529296875f,-1.10189994902759824668692090199328958988189697265625f,-1.00679997604042359427012343076057732105255126953125f,-2.677699927347897190799130839877761900424957275390625f,0.3226000854536887896273356091114692389965057373046875f,-1.337599990719556597440487166750244796276092529296875f,1.0717000238589931004895561272860504686832427978515625f,1.658500083029271099377410791930742561817169189453125f,2.30869998098462847480050186277367174625396728515625f,-1.6909999994710085413629485628916881978511810302734375f,-0.91529996969997906575144952512346208095550537109375f,-2.06379993553161700248210763675160706043243408203125f,-0.033400007596612102389599385787732899188995361328125f,-1.822599916921555429638601708575151860713958740234375f,-2.115099985168871654650502023287117481231689453125f,-1.578600008817016231432717177085578441619873046875f,0.60190001923069313960468207369558513164520263671875f,-1.0628000479310759640583228247123770415782928466796875f,-0.16380000817477669006194673784193582832813262939453125f,1.9693000245064506970038564759306609630584716796875f,0.59470003773197621654844624572433531284332275390625f,1.4169999692514536615561837606946937739849090576171875f,0.688699984784423957506760416436009109020233154296875f,-0.095599856400489235941364540849463082849979400634765625f,-0.5236999122157717234671281403279863297939300537109375f,1.26289991828650283167689849506132304668426513671875f,3.34109999875873331376396890846081078052520751953125f,-0.306000073809176598160775029100477695465087890625f,-0.54110000505447464203001572968787513673305511474609375f,-0.92130005000978798523902923989226110279560089111328125f,-2.88690001956596997700899009942077100276947021484375f,-2.169599960008264272204314693226478993892669677734375f,-1.7679999021701504613446331859449855983257293701171875f,2.789200001190603739331663746270351111888885498046875f,-0.6025999208703627374461575527675449848175048828125f,-5.40570018228441373508985634543932974338531494140625f,1.617199966821075474143754036049358546733856201171875f,-0.679599921730905887073959092958830296993255615234375f,-0.975600120504946044519556380691938102245330810546875f,-3.321100030423700832926670045708306133747100830078125f,1.7967999631881692579327136627398431301116943359375f,-2.811900017833709686243537362315692007541656494140625f,-1.2726999677672983235510173471993766725063323974609375f,1.38450007409974862326862421468831598758697509765625f,-1.71120002509206425855836641858331859111785888671875f,-1.084499972715973203918338185758329927921295166015625f,0.03720007049590368097824466531164944171905517578125f,-2.395599971474705824903139728121459484100341796875f,0.9145999123990538048900589274126105010509490966796875f,2.02829996621310915116964679327793419361114501953125f,-1.6732000018127255458466606796719133853912353515625f,-1.4493999392859635566566112174768932163715362548828125f,2.82550005095824641188073655939660966396331787109375f,0.7668998736463488175019165282719768583774566650390625f,3.120999909207224476404007873497903347015380859375f,-4.4870001231804490515742145362310111522674560546875f,1.425699926804004480374032937106676399707794189453125f,-0.87880003879666190869102138094604015350341796875f,2.02689987943917504509272475843317806720733642578125f,-0.4393000989377480181019564042799174785614013671875f,-1.412800018464027917985958993085660040378570556640625f,-0.51679997555613521598161241854541003704071044921875f,-0.35080009521991062371171210543252527713775634765625f,0.877000043421983566105382124078460037708282470703125f,-6.71280004992410628261723104515112936496734619140625f,-2.47939999354481965809782195719890296459197998046875f,0.6909999901577823511189535565790720283985137939453125f,1.132500074766576769746961872442625463008880615234375f,1.3062999714061607203774428853648714721202850341796875f,0.283799940523504545808464172296226024627685546875f,-0.619500132493675526035303846583701670169830322265625f,-1.357400011944025930432644599932245910167694091796875f,3.96029995863586758986230051959864795207977294921875f,-0.000699998068810270268613749067299067974090576171875f,3.9066999799564481321567654958926141262054443359375f,3.35959993162155168278104611090384423732757568359375f,-0.9543999824061992764967499169870279729366302490234375f,0.668900010964274205349511248641647398471832275390625f,-2.19699997990578399509331575245596468448638916015625f,-0.5075998982951037685751316530513577163219451904296875f,0.7704998856484888136719746398739516735076904296875f,-0.971799861543626075643942385795526206493377685546875f,1.7120999185085314930887534501380287110805511474609375f,0.0780000079870217799538067993125878274440765380859375f,-0.37239980910196879282381132725276984274387359619140625f,-3.1546000050842764039771282114088535308837890625f,-1.64230002392530405330717258038930594921112060546875f,-1.224900010509789805013269869959913194179534912109375f,3.287499920949339848874615199747495353221893310546875f,2.48560000984519735567346288007684051990509033203125f,3.555599951097370148289655844564549624919891357421875f,0.024200039281697005577598247327841818332672119140625f,0.7473998802363868154685633271583355963230133056640625f,0.04330003727301912075375867061666212975978851318359375f,0.293900077162682737963450563256628811359405517578125f,-1.276799982559679857985202033887617290019989013671875f,2.206999929018318429285727688693441450595855712890625f,-0.035300159913301154546161342295818030834197998046875f,0.0204000589162125489650634335703216493129730224609375f,-1.050699975381792938833314110524952411651611328125f,2.77589998986423136528856048244051635265350341796875f,-0.572799969048052748377131138113327324390411376953125f,0.3498998982295391524388605830608867108821868896484375f,3.213399998097124399265567262773402035236358642578125f,2.75250009264797057539908564649522304534912109375f,4.6732999403782162772813535411842167377471923828125f,-6.2321999961763623332444694824516773223876953125f,1.25430014025121838727727663354016840457916259765625f,-0.0162999508798111758522964009898714721202850341796875f,0.732699997308850381472211665823124349117279052734375f,-1.4751000966668141867188523974618874490261077880859375f,3.114699858228861639730666865943931043148040771484375f,1.6397999651961028266811126741231419146060943603515625f,0.9793999055534587938609547563828527927398681640625f,-1.674500009223817809100864906213246285915374755859375f,1.3008998797073960673742476501502096652984619140625f,-1.403700004006922963384340619086287915706634521484375f,1.810399956526607123663552556536160409450531005859375f,-1.731400015099349598557409990462474524974822998046875f,0.6238000119186930536585578010999597609043121337890625f,-1.4750999923959382531535311500192619860172271728515625f,1.5637999295711517699913883916451595723628997802734375f,1.417599959799646480718138263910077512264251708984375f,3.31930001035034738521289909840561449527740478515625f,-1.0429999705478547600279171092552132904529571533203125f,0.348900042014568700921017807559110224246978759765625f,0.1644000645861016796089870695141144096851348876953125f,0.407299949233232094769618925056420266628265380859375f,-1.473400021193921549667038561892695724964141845703125f,1.421900148032605226688929178635589778423309326171875f,-2.27699999704211908380102613591589033603668212890625f,-1.624899921102821220841860849759541451930999755859375f,2.1314999388754358733422122895717620849609375f,-1.101000084314494475989931743242777884006500244140625f,-4.8354999722614895318884009611792862415313720703125f,-1.575700033272803057826649819617159664630889892578125f,0.343499981962144484981536152190528810024261474609375f,4.503399964085222251242157653905451297760009765625f,1.04899990136176679556001545279286801815032958984375f,1.29889984626919119392596257966943085193634033203125f,2.004299981181322909407072074827738106250762939453125f,-3.91229984542802089464430537191219627857208251953125f,-3.016999879956246299883559913723729550838470458984375f,0.569899985215067683697043321444652974605560302734375f,-2.33080005826502922872123235720209777355194091796875f,0.7319999622479091971882780853775329887866973876953125f,1.0299001039773205690153190516866743564605712890625f,2.8894999612234553154621607973240315914154052734375f,-3.42640002579092683987482814700342714786529541015625f,4.0639999062046410216453296015970408916473388671875f,-1.062400100214777154405965120531618595123291015625f,2.07770000941604404687268470297567546367645263671875f,-0.6764000018000597691525399568490684032440185546875f,-2.58060012134313598863855077070184051990509033203125f,-0.90830005606710972898554246057756245136260986328125f,4.502999936386942181343329139053821563720703125f,-0.487399979683010631248407662496902048587799072265625f,-1.17330009950399460905146042932756245136260986328125f,-5.7829999935328970650516566820442676544189453125f,-2.27849996640905683165101436316035687923431396484375f,0.39429999597072684647258711265749298036098480224609375f,-2.891800001409649656380906890262849628925323486328125f,0.385000033043322531511876150034368038177490234375f,-3.15239999981820684382682884461246430873870849609375f,1.848600017683207141772072645835578441619873046875f,0.0470000782161947849857597248046658933162689208984375f,-2.199299923290312097634568999637849628925323486328125f,-0.1344999313279995334369232296012341976165771484375f,0.8419999210461959204820914237643592059612274169921875f,-1.3826999685868612832706503468216396868228912353515625f,-2.1096999259918920444079049048013985157012939453125f,-0.5729000350370991867521297535859048366546630859375f,0.9191999170735491997419330800767056643962860107421875f,2.24879998023509841686973231844604015350341796875f,1.8957998428687456993202431476674973964691162109375f,2.6324999244511122498124677804298698902130126953125f,0.96580011537372900676245990325696766376495361328125f,-0.089900138976425569836692375247366726398468017578125f,-0.73320005502849838752155164911528117954730987548828125f,2.1874000233620431998815547558479011058807373046875f,1.156199995103481370506415260024368762969970703125f,-0.945499880209566878619398266891948878765106201171875f,0.735400000615415994076329297968186438083648681640625f,-1.9619000508025283835422669653780758380889892578125f,0.8346999815918485499111056924448348581790924072265625f,2.121300012496114550941683773999102413654327392578125f,-1.744500134505331256917770588188432157039642333984375f,1.76010015837624589352117254748009145259857177734375f,0.84059996812194326398781640818924643099308013916015625f,2.685500039577482400687813424156047403812408447265625f,2.39890007284134920695350956520996987819671630859375f,-1.3640999923661336623581519233994185924530029296875f,-0.88600001854449406124558663577772676944732666015625f,0.63919997041970522122511511042830534279346466064453125f,0.421800055575371146687757573090493679046630859375f,-0.7490999637931599064444299074239097535610198974609375f,-3.368400012756137318348237386089749634265899658203125f,-1.097999987646936848051382185076363384723663330078125f,1.1941999264053979601385435671545565128326416015625f,0.513499899558720329650896019302308559417724609375f,3.80480008950903947351207534666173160076141357421875f,0.80189994831979394263044014223851263523101806640625f,0.453400028830766022025500205927528440952301025390625f,-0.8120000196546310888834341312758624553680419921875f,-1.4387000401794918236220155449700541794300079345703125f,-0.7439999781586219551599015176179818809032440185546875f,0.553499979969114264122254098765552043914794921875f,-2.210099972800910972381416286225430667400360107421875f,-1.325199969108403319495437244768254458904266357421875f,2.086400035364925020786586173926480114459991455078125f,-2.61880001160204667343123219325207173824310302734375f,-2.5440000220239173955860678688623011112213134765625f,0.369300057419389649027152700000442564487457275390625f,0.10779994488954425246873825017246417701244354248046875f,0.7270999544948335557847940435749478638172149658203125f,0.20599996024370359481991954453405924141407012939453125f,0.4716999489247808430292252523940987884998321533203125f,-2.974299915467203181407285228488035500049591064453125f,-0.26530006182640708534137274909880943596363067626953125f,0.4118000035144391279828823826392181217670440673828125f,-2.60900007680430956469308512168936431407928466796875f,-0.90579989563748153358346826280467212200164794921875f,-0.837799967986347926540702246711589395999908447265625f,2.8420001121535900523440432152710855007171630859375f,1.999200029838084535782627426669932901859283447265625f,-1.207200052763522268861606789869256317615509033203125f,0.0488999987080684928031359959277324378490447998046875f,2.84500001296401006101177699747495353221893310546875f,-1.231800048348306919621109045692719519138336181640625f,1.2427999335646635614693877869285643100738525390625f,1.529900108745694264911207937984727323055267333984375f,1.874000086210667603836554917506873607635498046875f,-1.2477999938398605106470995451672933995723724365234375f,1.028200011871010399744363894569687545299530029296875f,1.1524000248894099573959692861535586416721343994140625f,-4.84539994169995313910703771398402750492095947265625f,0.5611999742791062839586402333225123584270477294921875f,-1.1835999770402889641474075688165612518787384033203125f,-0.622899952931700529035197178018279373645782470703125f,-2.627799911287428624717676939326338469982147216796875f,-1.797299971473214963424425150151364505290985107421875f,4.03629995548054676390847816946916282176971435546875f,0.094100139477847566382706645526923239231109619140625f,-0.0819999893307699068145666387863457202911376953125f,0.26030002459213086307698858945514075458049774169921875f,0.4584000599928204078281623878865502774715423583984375f,1.7433000386327484054760361686930991709232330322265625f,-1.2677000268131506377500272719771601259708404541015625f,-1.720200022622198066102328084525652229785919189453125f,-2.198200086991488166887620536726899445056915283203125f,-0.41039999947920147160829174026730470359325408935546875f,1.764900002351402985567574432934634387493133544921875f,-0.9053999512925743342606210717349313199520111083984375f,2.0431999475911251096249543479643762111663818359375f,1.3720999678499996576874764286912977695465087890625f,1.134300010834634608869464500457979738712310791015625f,1.230100039856136362459437805227935314178466796875f,3.37929989434480493315504645579494535923004150390625f,1.5519999843388794236176408958272077143192291259765625f,0.42250004727393442038874127320013940334320068359375f,0.90260004300400653587388433152227662503719329833984375f,-0.64440000853910983646954946379992179572582244873046875f,-1.088399980588257154323628128622658550739288330078125f,1.020199893037976313081571788643486797809600830078125f,1.560000092163681184587176176137290894985198974609375f,1.1243000693961981983903797299717552959918975830078125f,4.4718000014841567946177747217006981372833251953125f,1.13099989233166109414696620660834014415740966796875f,3.6419999851211901642500379239208996295928955078125f,-0.945199947762491721192645854898728430271148681640625f,-4.5364000037744620641433357377536594867706298828125f,-0.78510000047981520765461027622222900390625f,-0.73420009063482183364612865261733531951904296875f,3.343700050386785971312519905040971934795379638671875f,-0.4864999449290341448914887223509140312671661376953125f,-2.014900018928944103180356250959448516368865966796875f,-4.420400028927627289476731675677001476287841796875f,-1.2985000322535633632270446469192393124103546142578125f,2.426500095188616423769190078019164502620697021484375f,-0.81039997140914099560404793010093271732330322265625f,-0.5767999874398110904394343378953635692596435546875f,-1.8778000205874452177567945909686386585235595703125f,0.503800153367968572837298779631964862346649169921875f,2.63769991876110321271653447183780372142791748046875f,-1.17109989594072327889762163977138698101043701171875f,-0.57410007964968645666203883592970669269561767578125f,0.0345000067465001947208946830869535915553569793701171875f,-1.46900009485334148706670021056197583675384521484375f,-0.80620009922534141555416908886400051414966583251953125f,1.992999961495399485755797286401502788066864013671875f,-0.18439989504814124909870542978751473128795623779296875f,-0.6982000587910410782654935246682725846767425537109375f,-1.324600033508240404245270838146097958087921142578125f,-0.402899921937284855033567509963177144527435302734375f,0.9110000013709071975398501308518461883068084716796875f,3.027300080922243896708323518396355211734771728515625f,4.08959997192919200159622050705365836620330810546875f,1.2676999565541759995568327212822623550891876220703125f,-0.15759999030977434042455342932953499257564544677734375f,1.231299890120327145126566392718814313411712646484375f,-1.63909996379315980874480374041013419628143310546875f,0.3526998807817687264787309686653316020965576171875f,-0.02909994445890162761969577331910841166973114013671875f,0.7389000026568748324962143669836223125457763671875f,-1.1396000127956262826245392716373316943645477294921875f,-0.9770000193566088508845268734148703515529632568359375f,0.03530006383806405523273497237823903560638427734375f,1.0310999972313654726718823440023697912693023681640625f,1.7633000980511308597442621248774230480194091796875f,-1.813600111970303796482539837597869336605072021484375f,1.867500007115304061500182797317393124103546142578125f,-0.8520999619454141527086221685749478638172149658203125f,-1.921299962465464972183326608501374721527099609375f,2.423900064347682015153395695961080491542816162109375f,-2.462400061713904353410953262937255203723907470703125f,-0.38809995892047854937345618964172899723052978515625f,3.822499995604157874140582862310111522674560546875f,-1.79570007806941855932336693513207137584686279296875f,0.083399879260358744659242802299559116363525390625f,-0.0147999256893986963490306152380071580410003662109375f,0.368799954307080124493722905754111707210540771484375f,-1.858499994665383070469033555127680301666259765625f,-4.68609999499618989915461497730575501918792724609375f,-5.979599955351648787882368196733295917510986328125f,1.854300002974272221223372980603016912937164306640625f,0.4083000534221532529244313991512171924114227294921875f,-3.66960001149177461599037997075356543064117431640625f,-1.399099976459145633356229154742322862148284912109375f,2.43110010394230702246431974344886839389801025390625f,-1.851600062140822178236021500197239220142364501953125f,-1.7563000069454297591420299795572645962238311767578125f,1.7183999346926823736936285058618523180484771728515625f,-0.9183999211326234757279962650500237941741943359375f,0.5520000322461122710393510715221054852008819580078125f,0.602500000298024307454625159152783453464508056640625f,-2.09840004235357024953145810286514461040496826171875f,0.827300091390311909123056466341949999332427978515625f,2.401700028906762707237021459150128066539764404296875f,0.5967000447392454276496209786273539066314697265625f,1.151200055676699918194572092033922672271728515625f,-0.724499938517808583782198184053413569927215576171875f,0.0929999697655434776066840640851296484470367431640625f,0.55530007486492294344060383082251064479351043701171875f,0.2782000903442491956951698739430867135524749755859375f,1.4648999529168011957125372646260075271129608154296875f,3.38019991095662231117557894322089850902557373046875f,0.7148998680174347697402481571771204471588134765625f,-0.28389991246759915810571328620426356792449951171875f,1.91739995708912491778619369142688810825347900390625f,-0.8468000958457586602179389956290833652019500732421875f,-0.346599985735116522533871830091811716556549072265625f,-0.158000030711290317952943951240740716457366943359375f,-0.992199966206403161095295217819511890411376953125f,0.419399941539765119813409910420887172222137451171875f,-0.41019990600198441565993334734230302274227142333984375f,1.00500002410262556651332488399930298328399658203125f,0.799100026936829355150848641642369329929351806640625f,2.111300139379501228376057042623870074748992919921875f,-2.13950000110268501174459743197076022624969482421875f,1.7320999971121568972165505329030565917491912841796875f,1.567700005765256587864087123307399451732635498046875f,3.773000047400592027457832955406047403812408447265625f,0.8508001228429378226536528018186800181865692138671875f,0.9768999241054052706800803207443095743656158447265625f,0.411699963490663289888971121399663388729095458984375f,-1.857999908521771548208789681666530668735504150390625f,2.79889992915689678198987166979350149631500244140625f,1.4334999302551150179851902066729962825775146484375f,-2.512400017382950156985543799237348139286041259765625f,2.162100013242661322721005490166135132312774658203125f,-0.9815000250041467921846560784615576267242431640625f,-0.7312000427126876811456668292521499097347259521484375f,-1.03649997059628251605545301572419703006744384765625f,0.447100017061085264202802136423997581005096435546875f,-0.2116000662572685708795461323461495339870452880859375f,-0.9181998882658779503884716177708469331264495849609375f,2.299899969792367215859485440887510776519775390625f,-2.5056001034989971998356850235722959041595458984375f,2.57929985787421411913555857609026134014129638671875f,-1.0864000390343362401068816325278021395206451416015625f,1.16290001585185454047177699976600706577301025390625f,3.315699914138021231480024653137661516666412353515625f,3.0528999926060453873333244700916111469268798828125f,-0.790800030027330791426720679737627506256103515625f,-3.258100068518519520921472576446831226348876953125f,-0.56079993909299441501303817858570255339145660400390625f,2.314299971719085302623852840042673051357269287109375f,-2.46200010293349524914674475439824163913726806640625f,1.6464000300750136762673037083004601299762725830078125f,-1.6582999960340567557892654804163612425327301025390625f,-4.1453000348553050713462653220631182193756103515625f,-1.784200032557548443179484820575453341007232666015625f,-0.829200014825165254706007544882595539093017578125f,-1.250199915650486826024234687793068587779998779296875f,-3.49439999235272491517889648093841969966888427734375f,0.0054000212907805433815155993215739727020263671875f,1.791300026801230682593768506194464862346649169921875f,1.7051999967873092689529812560067512094974517822265625f,3.06229996737539789108950571971945464611053466796875f,1.1836999767377973658000200884998776018619537353515625f,0.6575998713426283170946362588438205420970916748046875f,0.68330002913326015612227593010175041854381561279296875f,-0.3351001032605769314187682539341039955615997314453125f,-1.708599982181192356023302636458538472652435302734375f,1.031999996408820674531625627423636615276336669921875f,0.230000067092476445651527683367021381855010986328125f,-1.126200051355362052873942957376129925251007080078125f,0.60059996653869607285969323129393160343170166015625f,0.69279996798634513321957228981773369014263153076171875f,0.0829999883919942949006554044899530708789825439453125f,3.049499970220029165801634007948450744152069091796875f,1.1258000137105594884445736170164309442043304443359375f,-0.908699963252246334377559833228588104248046875f,0.4889000296279795154674729928956367075443267822265625f,1.907300073136389162442583256051875650882720947265625f,-0.651000036500394063665453359135426580905914306640625f,3.653500013105570598526128378580324351787567138671875f,2.275300026380272289117101536248810589313507080078125f,0.753400042912363954172860758262686431407928466796875f,0.2444999228715907246822780507500283420085906982421875f,-0.99180000466927931146443597754114307463169097900390625f,-0.06350004473328663134878979690256528556346893310546875f,-0.180200057770311783400529748178087174892425537109375f,-0.10889998343437923278287371431360952556133270263671875f,-1.1107000111445781698193968622945249080657958984375f,-0.0136000863403065164192184965941123664379119873046875f,1.291199986498057672434924825211055576801300048828125f,-0.7348999120876176238681409813580103218555450439453125f,-1.8364000228479524512437137673259712755680084228515625f,-0.74049998982623355114895957740372978150844573974609375f,-1.356300001357495066400815630913712084293365478515625f,1.04029999562799790879807915189303457736968994140625f,3.61709995281845397840925215859897434711456298828125f,0.34360008734613500802623775598476640880107879638671875f,0.388399998655914568956859511672519147396087646484375f,0.4973999158129112085902079343213699758052825927734375f,1.5889999839328228237178564086207188665866851806640625f,-4.412899899213016396970488131046295166015625f,-0.7168999940663571823762367785093374550342559814453125f,2.732700032885373442326226722798310220241546630859375f,1.080300090473891305720144373481161892414093017578125f,-0.1461000161558392296257125053671188652515411376953125f,-2.755200057099761234979951041168533265590667724609375f,4.37099996266514079223952649044804275035858154296875f,-2.15149986977130236454058831441216170787811279296875f,0.78580001773387220254107887740246951580047607421875f,-0.0384000119179510601696847515995614230632781982421875f,-0.82100001027435098865936424772371537983417510986328125f,1.0320000742487616118836513123824261128902435302734375f,0.5466000399008377197418440118781290948390960693359375f,-2.010999969370663276180266620940528810024261474609375f,-1.3684000140786187760255643297568894922733306884765625f,0.921099891470372522661591574433259665966033935546875f,2.9742999576374895553954047500155866146087646484375f,-1.2537000225588676993737635712022893130779266357421875f,-3.930100016964227283011723557137884199619293212890625f,-2.25060015386492029421106053632684051990509033203125f,1.6562000700190664392863482134998776018619537353515625f,-0.035499860167502728103272602311335504055023193359375f,3.35940003947764775915629797964356839656829833984375f,2.13769993729442564500686785322614014148712158203125f,-2.1020000041201711837857146747410297393798828125f,-1.1027000908136355139532724933815188705921173095703125f,-2.157300032642484044487218852736987173557281494140625f,-0.166000035233795861788763659205869771540164947509765625f,0.8353999531924731769549907767213881015777587890625f,2.44869997122436888759011708316393196582794189453125f,-2.4192000536426885304308598279021680355072021484375f,1.4906000130862000485620910694706253707408905029296875f,0.266100001403690777923571886276477016508579254150390625f,-0.19279992607682927552303908669273369014263153076171875f,-0.6875000471621761644058778983890078961849212646484375f,-2.649300026909261251972793616005219519138336181640625f,-2.545199892590940038417102186940610408782958984375f,-2.9501999068960547134565786109305918216705322265625f,0.079900048023461067714379169046878814697265625f,3.412099946112931547759217210114002227783203125f,-2.499500084251163212201163332792930305004119873046875f,0.7892000110998760309399813195341266691684722900390625f,-1.6687000382423395894448958642897196114063262939453125f,-1.531100081311166594133510443498380482196807861328125f,0.077800073486567811187342158518731594085693359375f,3.775299989891051755108719589770771563053131103515625f,4.0250000372901553191695711575448513031005859375f,1.665599970692395626059578717104159295558929443359375f,0.461300056529044599784583624568767845630645751953125f,-1.647800058007983903252124946448020637035369873046875f,-4.99329995689988148654947508475743234157562255859375f,-0.27550004088133750457245696452446281909942626953125f,-0.263100075037776992559201971744187176227569580078125f,1.13239998461902313664495522971265017986297607421875f,2.057400075776876224864508913015015423297882080078125f,-0.4549999903887507191058148237061686813831329345703125f,0.5358000166535370478726463261409662663936614990234375f,0.4537000202491878741994923984748311340808868408203125f,-1.14540003079026941890106172650121152400970458984375f,-1.598299851287900086305171498679555952548980712890625f,-1.0794000001013301925922860391438007354736328125f,-3.262700015003980436034680678858421742916107177734375f,-2.443200037258864032452265746542252600193023681640625f,-0.1639000009804953972292196340276859700679779052734375f,-1.4642000086411834924859931561513803899288177490234375f,-0.14619994052797535832866060445667244493961334228515625f,-0.2752000457003724864790683568571694195270538330078125f,1.3544999873563632331041617362643592059612274169921875f,1.5397000356644401364292207290418446063995361328125f,0.9337999921187754015505788629525341093540191650390625f,-2.656999986909330591089428708073683083057403564453125f,-2.020500019125639479256051345146261155605316162109375f,-1.5732001251384606366201523997006006538867950439453125f,-1.399999952316285156683761670137755572795867919921875f,-0.2296000309005387374217121987385326065123081207275390625f,-1.24139993100091761135672641103155910968780517578125f,0.754500013358891497006197823793627321720123291015625f,-1.964499967947601621887088185758329927921295166015625f,2.4921999756872654785411214106716215610504150390625f,-0.850499962016938670927856946946121752262115478515625f,1.41939994977265460107673789025284349918365478515625f,0.9576000676468032768440252766595222055912017822265625f,1.679899965433777619949751169770024716854095458984375f,2.700299927175787928490535705350339412689208984375f,-0.21579995913505545246380279422737658023834228515625f,-0.88480002362728293885396624318673275411128997802734375f,1.36150002168864148188731633126735687255859375f,2.7693000561714171681160223670303821563720703125f,3.468299940881133469616770526045002043247222900390625f,1.663399986939877894798200941295363008975982666015625f,-0.9160999697946003994530883574043400585651397705078125f,-0.5663999783307323010461686862981878221035003662109375f,4.51949993266910521327872629626654088497161865234375f,-1.006999879900366767770947262761183083057403564453125f,1.9711999007418754548126571535249240696430206298828125f,-4.2421000795714558506688263150863349437713623046875f,0.24079994531422865833292235038243234157562255859375f,-2.6360000958070148868728210800327360630035400390625f,0.333399985617399696735674297087825834751129150390625f,2.022499873992054997273726257844828069210052490234375f,1.354799976129829719440067492541857063770294189453125f,-1.1716999690785983023744165620882995426654815673828125f,-2.0744999857172370383295856299810111522674560546875f,1.145699897057562477442616000189445912837982177734375f,0.1452999820120635110498596986872144043445587158203125f,0.4898000305563201539627016245503909885883331298828125f,-0.22639998488724144554140593754709698259830474853515625f,-0.52379991792961722030241844549891538918018341064453125f,0.40510001932978501049120723109808750450611114501953125f,-2.796100059741735588403344081598334014415740966796875f,-4.11859999838620449708059823024086654186248779296875f,-1.685000040568410906161034290562383830547332763671875f,2.482100023692101586902936105616390705108642578125f,-0.3604000303059800014437996651395224034786224365234375f,-1.56729991835057891336191460140980780124664306640625f,0.9697000673294080552722107313456945121288299560546875f,-2.325999972686172867497589322738349437713623046875f,1.463699992961434492855232747388072311878204345703125f,-2.562999998040497384721447815536521375179290771484375f,1.60759995523616527890453653526492416858673095703125f,-0.4485999058686205298585036871372722089290618896484375f,-2.641999980725345142928972563822753727436065673828125f,-1.4426998993337150078986041989992372691631317138671875f,-1.174199931006131958355354072409681975841522216796875f,3.844700087334215421464023165754042565822601318359375f,0.10540007156357233952803653664886951446533203125f,-0.6367999337583778451943317122641019523143768310546875f,0.71680009245574527465549863336491398513317108154296875f,0.7993999279797068258091030656942166388034820556640625f,2.2262999780371810487622497021220624446868896484375f,0.760400031442195700748243325506336987018585205078125f,0.79340001083761391154069997355691157281398773193359375f,1.99680004773363517500683883554302155971527099609375f,-2.398800017301738307651248760521411895751953125f,-1.91080011190921172925527571351267397403717041015625f,0.526100057655574548931554090813733637332916259765625f,1.2256000243738298660645114068756811320781707763671875f,-0.08909997992366402996822216664440929889678955078125f,2.192399922294914649256725169834680855274200439453125f,-2.5098000066772101490641944110393524169921875f,-1.392299994234740712073516988311894237995147705078125f,-2.060899939870833197375077361357398331165313720703125f,-0.0640999051570887434792211934109218418598175048828125f,1.4902000041447589495646752766333520412445068359375f,0.34890001696199046676838406710885465145111083984375f,1.3861000451944764932932230294682085514068603515625f,2.200899963824451077698540757410228252410888671875f,-0.13869999951794798054294233224936760962009429931640625f,-2.972000011011959319517927724518813192844390869140625f,1.3662999418459822464910757844336330890655517578125f,0.297899890086054164317630466030095703899860382080078125f,-4.64000007219612609077330489526502788066864013671875f,-0.859200021195408947249916309374384582042694091796875f,3.89419999595657007063209675834514200687408447265625f,-0.01399998003989566530691490697790868580341339111328125f,-0.45140005391687054725480265915393829345703125f,-4.76510001640543290335472192964516580104827880859375f,0.89180000621527477644434611647739075124263763427734375f,-2.61469995840191860025925052468664944171905517578125f,1.089200007430463745805582220782525837421417236328125f,0.70019991684258020558928592436132021248340606689453125f,3.03129996872395057749827174120582640171051025390625f,-0.2050999025791881980040898270090110599994659423828125f,2.1869000556081541475350604741834104061126708984375f,-3.2294000253587942239619223983027040958404541015625f,-0.371499975681305105723595261224545538425445556640625f,-2.904399891136586919770934400730766355991363525390625f,1.035200048252196136644442958640865981578826904296875f,1.750899914911388410843073870637454092502593994140625f,-0.6801000690065333031952832243405282497406005859375f,1.57630006483644091730411673779599368572235107421875f,-1.82270008421987395053065483807586133480072021484375f,-1.15080001797601472190990534727461636066436767578125f,-2.220699980857969269010254720342345535755157470703125f,0.244899929317085707225487567484378814697265625f,-2.972800048936902417295868872315622866153717041015625f,2.1400000105053180732284090481698513031005859375f,-4.10900004951655706264546097372658550739288330078125f,-2.812100018495322206746322990511544048786163330078125f,0.271200016747414007767247312585823237895965576171875f,1.6741000340521339229127306680311448872089385986328125f,-2.711299929962305288455581830930896103382110595703125f,0.940600019735842618473498077946715056896209716796875f,0.7627999803915628262984682805836200714111328125f,4.43809993373751598966236997512169182300567626953125f,-1.7801999185934642699891128359013237059116363525390625f,-1.090100077146292978369501724955625832080841064453125f,-3.452100026355686690493485002662055194377899169921875f,3.625500156588852718186899437569081783294677734375f,0.6620000051446266464694190290174447000026702880859375f,-0.6210999819204221505941632130998186767101287841796875f,-0.29900002790987445511206033188500441610813140869140625f,0.29330009440034576773115304604289121925830841064453125f,2.86190007669329649075962151982821524143218994140625f,5.7545999663412583657873256015591323375701904296875f,-0.0257000447094444250950573405134491622447967529296875f,-0.89090000755935871978863360709510743618011474609375f,3.16640001878738441831728778197430074214935302734375f,0.02609987578689942910870058767613954842090606689453125f,-0.27920001378208303410843882375047542154788970947265625f,-0.50320005283057656075840213816263712942600250244140625f,4.41280005467385105788480359478853642940521240234375f,-0.3271000084742912150659321923740208148956298828125f,-2.0192999343916770982332309358753263950347900390625f,-3.30170002924203931371494036284275352954864501953125f,2.597700002449749856481275855912826955318450927734375f,-0.3360000371709468414138655134593136608600616455078125f,1.6946999832496045801377704265178181231021881103515625f,-1.4384999688491226788045196371967904269695281982421875f,1.997300049350410944271061453036963939666748046875f,0.5192999927669761373039136742590926587581634521484375f,-1.7937000496231016999360008412622846662998199462890625f,-0.85300003689527414962867624126374721527099609375f,1.3762000041931881089141143093002028763294219970703125f,2.334700100931526467462617802084423601627349853515625f,-0.474999935422093955139644094742834568023681640625f,-3.098100044769794170207433126051910221576690673828125f,0.1691999990299333234133882797323167324066162109375f,-1.4862999698415377825000405209721066057682037353515625f,1.6164999844729892597428033695905469357967376708984375f,-1.514899911584705716194321212242357432842254638671875f,-0.430500019013880574902941589243710041046142578125f,2.5790998983398072397221767459996044635772705078125f,3.046800007891655059921731663052923977375030517578125f,-1.2267000127017524402361914326320402324199676513671875f,-1.5957000421389933020321905132732354104518890380859375f,0.2060999935805793992216194965294562280178070068359375f,2.789000101502985717871752058272249996662139892578125f,-0.343599964635074306418260903228656388819217681884765625f,-1.2763000890880820126227490618475712835788726806640625f,-1.192300044600664410410217897151596844196319580078125f,0.666600024031103632893291432992555201053619384765625f,-0.16190006041377813517101458273828029632568359375f,0.9852999946966758404442998653394170105457305908203125f,-3.1250001392699768842931007384322583675384521484375f,-0.14800009009242087731905712644220329821109771728515625f,1.597700040335953364234455875703133642673492431640625f,1.17559996402412547666926911915652453899383544921875f,0.31389995135963022487857188025373034179210662841796875f,-0.002700035195052574721330529428087174892425537109375f,-1.317600046934188195990600434015505015850067138671875f,-1.9465000260099751816511570723378099501132965087890625f,-0.092899963437020449674719202448613941669464111328125f,-0.890600015544892098517948397784493863582611083984375f,-0.5205999231576907693153088985127396881580352783203125f,0.907299979706108761234872872591949999332427978515625f,-0.113100036182998575640112903784029185771942138671875f,-3.035099935846032881414657822460867464542388916015625f,2.491699958983064622231040630140341818332672119140625f,0.47710010354369902785265367128886282444000244140625f,-2.171799973320962084954999227193184196949005126953125f,-0.09510010152831627205927134127705357968807220458984375f,1.827099974238871826770491679781116545200347900390625f,-1.289599941568076335585146807716228067874908447265625f,2.56189986137151937128919598762877285480499267578125f,2.272399972455948269356440505362115800380706787109375f,2.63069998927712589420480071566998958587646484375f,0.35930000016167706888836619327776134014129638671875f,3.341800091841070408094083177275024354457855224609375f,-0.65210010924339290028228788287378847599029541015625f,4.1355999796703457604962750338017940521240234375f,1.50820010496601497607116471044719219207763671875f,-2.4732999917127198585831138188950717449188232421875f,-3.096900040297211997852855347446165978908538818359375f,-3.848399997761847490806985661038197576999664306640625f,-0.445699959176778737202084812452085316181182861328125f,1.0430000996105370791866562285576947033405303955078125f,2.10350008098036056480850675143301486968994140625f,1.86620007232874485225693206302821636199951171875f,1.4372000502675768007776468948577530682086944580078125f,-2.37920008327737431130799450329504907131195068359375f,2.3464000212833298064651899039745330810546875f,-0.60719997976645867510114840115420520305633544921875f,0.583499971233308212248402924160473048686981201171875f,1.5916999979123473973885438681463710963726043701171875f,2.0440000865459442280780422152020037174224853515625f,2.097800022971628930434917492675594985485076904296875f,-1.0341001273147771843952114068088121712207794189453125f,2.69210002829283556735617821686901152133941650390625f,0.072500019818543481875394718372263014316558837890625f,0.4609000308424235914372957267914898693561553955078125f,-2.0105000934824346359164337627589702606201171875f,1.032600015027075102125309058465063571929931640625f,0.7878999781705446903146139447926543653011322021484375f,-1.118699939037860691115611189161427319049835205078125f,-2.16390006710439930515121886855922639369964599609375f,0.4794999957755206043685802796971984207630157470703125f,0.164900052605569502617299804114736616611480712890625f,-1.576600022764504505090599195682443678379058837890625f,0.6364000461310144185489434676128439605236053466796875f,1.2330000227764250642081833575502969324588775634765625f,1.765599937518686157744696174631826579570770263671875f,-2.09160003941655237014174417709000408649444580078125f,-0.43520004465729034137666531023569405078887939453125f,0.73670004749596096349506524347816593945026397705078125f,5.76050008293986426366473097004927694797515869140625f,-0.42219991461113071107291716543841175734996795654296875f,-1.2183999829351901400542601550114341080188751220703125f,3.1737000209942465289714164100587368011474609375f,-0.6384000001087783982001155891339294612407684326171875f,-1.5285999702602615091251436751917935907840728759765625f,-3.2011000276483603244059850112535059452056884765625f,-1.228799988244474672427486439119093120098114013671875f,2.264400013437867897891919710673391819000244140625f,1.31789993631690638409281746135093271732330322265625f,-0.90239992859065498720383402542211115360260009765625f,3.31679995223581869367990293540060520172119140625f,-0.7974998996779325555195327979163266718387603759765625f,0.17520005227550949911119460011832416057586669921875f,0.166599913464484716829616672839620150625705718994140625f,1.1853999400049433088355499421595595777034759521484375f,1.772199923346936767387660438544116914272308349609375f,3.93710000631362344591934743220917880535125732421875f,1.3652999806523318238049569117720238864421844482421875f,-0.951999930266291816138846115791238844394683837890625f,-0.57849999406933771961547563478234224021434783935546875f,2.70859999905675774556357282563112676143646240234375f,-0.7591000311464066729882915751659311354160308837890625f,2.63899998069182029070134376524947583675384521484375f,2.06820002362430077624821933568455278873443603515625f,-1.996199870157243427826188053586520254611968994140625f,-3.55460000337064219166904877056367695331573486328125f,-0.0893000914871693307617306345491670072078704833984375f,-3.6059999939203262186993015347979962825775146484375f,2.54089997693747449147849692963063716888427734375f,-1.545399921285361966027949165436439216136932373046875f,2.08929983976930344624634017236530780792236328125f,-2.859600010187923313509372746921144425868988037109375f,0.5243000339686876287714767386205494403839111328125f,-0.42079998416900654678585169676807709038257598876953125f,1.061700070033966714078133009024895727634429931640625f,-1.1838000144891449583184339644503779709339141845703125f,-1.9026000333555028731780112138949334621429443359375f,3.8041000534608944150249953963793814182281494140625f,-0.28850003376230703988625236888765357434749603271484375f,-1.86110002908259541953839288908056914806365966796875f,0.8312998987257496086300534443580545485019683837890625f,0.28660006269961602054507920911419205367565155029296875f,-1.7502000644385820038451129221357405185699462890625f,-0.52310000319555338688104484390350989997386932373046875f,2.543499911535530788597725404542870819568634033203125f,2.143899949217586975436233842629007995128631591796875f,2.68169999549090931623140932060778141021728515625f,-1.4425999263837929742493315643514506518840789794921875f,0.5878000649556518464322607542271725833415985107421875f,3.909500013936311635376341655501164495944976806640625f,0.90049993880838130611010683423955924808979034423828125f,-0.31209993165880300924897028380655683577060699462890625f,-1.363500012826175389335503496113233268260955810546875f,-0.86059999140501142189663141834898851811885833740234375f,-1.669000033665448246011919763986952602863311767578125f,-1.0604000366762280638255333542474545538425445556640625f,-2.64679991561621275053539648070000112056732177734375f,-0.04440005205050134851063603491638787090778350830078125f,-0.97579992545843108331382609321735799312591552734375f,1.269400051640719784273869663593359291553497314453125f,-2.217000011943280224357977203908376395702362060546875f,-0.485699968974291351742067490704357624053955078125f,3.213099875809252381486658123321831226348876953125f,-0.96659988392293438863589472020976245403289794921875f,1.22190008840933472811229876242578029632568359375f,0.6797001296907669853908373625017702579498291015625f,-1.99500004131347008495822592522017657756805419921875f,2.6324999505653980946817682706750929355621337890625f,0.5693000260151921398943386520841158926486968994140625f,0.144699988129734169461926285293884575366973876953125f,0.8001999372757968576053144715842790901660919189453125f,0.95960005477964926967615610919892787933349609375f,-0.72349998218566058394429774125455878674983978271484375f,-1.008900068072973166266592670581303536891937255859375f,3.42070000019222408838004412245936691761016845703125f,1.299199964682759667056188845890574157238006591796875f,-0.4812999798439439036457088150200434029102325439453125f,-3.477700009620935617249415372498333454132080078125f,-0.851100016640126799671861590468324720859527587890625f,-1.550499997407197572130144180846400558948516845703125f,-1.26929988283365968726457140292041003704071044921875f,0.3921000202462086026145016148802824318408966064453125f,2.9615000041052699231158840120770037174224853515625f,0.3445999088600277815430672490037977695465087890625f,-1.53339998667910748508802498690783977508544921875f,0.88759998192787070703246854463941417634487152099609375f,-1.8412000971391808690214020316489040851593017578125f,1.359300014187394989306767456582747399806976318359375f,-3.325100091823936399038075251155532896518707275390625f,1.226799950503557834480261590215377509593963623046875f,-1.771800025661291133616259685368277132511138916015625f,-2.4217000599384306269712396897375583648681640625f,-1.06209996311888144049362381338141858577728271484375f,-2.986399979839472695886115616303868591785430908203125f,1.054199924971161550502074533142149448394775390625f,-4.3065999033689497110799493384547531604766845703125f,-2.127599947115034861866433857358060777187347412109375f,-3.32950000875815721457229301449842751026153564453125f,-0.265200042645632461102422894327901303768157958984375f,2.35190014237016242759636952541768550872802734375f,-0.2487999160483485017181237708427943289279937744140625f,-0.65079997519105658998483932009548880159854888916015625f,-0.0557999846905474772285060680587776005268096923828125f,0.18870016488358476447473321968573145568370819091796875f,3.365799936820567328510378501960076391696929931640625f,1.4446000587657115143969122073031030595302581787109375f,2.1325000186823306336236782954074442386627197265625f,1.20960002795755716675785151892341673374176025390625f,1.8262999429449440214057176490314304828643798828125f,3.205300007474424006659319275058805942535400390625f,-1.35610003472641249544494712608866393566131591796875f,1.0543000219732518996096359842340461909770965576171875f,0.82889994020238333405359298922121524810791015625f,-1.30220007615834365566342967213131487369537353515625f,2.0574000004142529718365040025673806667327880859375f,-0.30289994466155778507499007901060394942760467529296875f,3.63529993561357311904203015728853642940521240234375f,2.279199924486875250551065619220025837421417236328125f,1.49940003126338172734222098370082676410675048828125f,-0.81689999361932164045896342940977774560451507568359375f,0.2894000709936024318125191712169907987117767333984375f,-1.2175999973133209142162058924441225826740264892578125f,3.22780000451281612328102710307575762271881103515625f,-1.69650000395625877303018569364212453365325927734375f,-0.854999920539557312082479256787337362766265869140625f,1.926699987276644154832183630787767469882965087890625f,-1.7587000123329463718846454867161810398101806640625f,1.1575000237673529479565104338689707219600677490234375f,-0.36240008576065252388076487477519549429416656494140625f,-3.70700002319365484737545557436533272266387939453125f,-1.80749991562217449114768896833993494510650634765625f,3.160599984792619832063564899726770818233489990234375f,-2.5075000392273079086180587182752788066864013671875f,-1.8206001067399977255689691446605138480663299560546875f,-0.39799996153265360110395931769744493067264556884765625f,-2.71570007685869807545486764865927398204803466796875f,5.064100034610927281164549640379846096038818359375f,-2.510800033789871488210110328509472310543060302734375f,-2.30860004228875137499699121690355241298675537109375f,3.7222000111520276988130717654712498188018798828125f,-2.59639997855424820016878584283404052257537841796875f,-3.74839998472332869283718537189997732639312744140625f,-2.37670003961697062777602695859968662261962890625f,1.9750999394595625258119753198116086423397064208984375f,-2.2341000645995148232714200275950133800506591796875f,-2.01850004596263143952228347188793122768402099609375f,1.0970999303549529013679375566425733268260955810546875f,-1.1039999911412590183346082994830794632434844970703125f,-0.8980999023333191555451548993005417287349700927734375f,5.80079993965178619674816218321211636066436767578125f,-1.4617000640735031691264111941563896834850311279296875f,-0.4636999713175005144449869476375170052051544189453125f,0.9520000532567498563452090820646844804286956787109375f,3.366100012318789413967579093878157436847686767578125f,-0.2029999646432691928765734701300971210002899169921875f,-0.907499940786510084222982186474837362766265869140625f,-0.096600056627391683861105775577016174793243408203125f,-1.20690005111917830760148717672564089298248291015625f,-0.744599977852403061007180440356023609638214111328125f,0.348499954245982568323825034894980490207672119140625f,-1.7818000623181458053068126901052892208099365234375f,1.011000080347061125252139390795491635799407958984375f,-1.9025999378390612282174743086216039955615997314453125f,-1.90099992060661282522460169275291264057159423828125f,-2.654099977185577330374144366942346096038818359375f,0.742899849219619934359570834203623235225677490234375f,2.614699995207785843120973368058912456035614013671875f,0.955000190213322053267575029167346656322479248046875f,0.94320006687492108365944432080141268670558929443359375f,-0.07820008220449048419453674796386621892452239990234375f,0.50350011566281260844135658771847374737262725830078125f,0.36290002220347628014707197507959790527820587158203125f,-1.547000040385871244552618009038269519805908203125f,-1.1943999441660932969710984252742491662502288818359375f,-2.220100043059886729679419659078121185302734375f,-0.18819997625723505763062348705716431140899658203125f,4.54129994510561285636640604934655129909515380859375f,3.686400039798021222026136456406675279140472412109375f,-0.655499930016696819023991338326595723628997802734375f,-4.17060007172226843152884612209163606166839599609375f,0.86389993293806865981565579204470850527286529541015625f,-0.113700019317866385382131966252927668392658233642578125f,0.6063000118255617021389980436651967465877532958984375f,1.4178999237440519376463043954572640359401702880859375f,0.450400009313970173252528184093534946441650390625f,-1.0896000170424577913053099109674803912639617919921875f,-1.231399960151315031708918468211777508258819580078125f,0.9911999543488032582416735749575309455394744873046875f,-0.1051999914228913812763721580267883837223052978515625f,3.42070000369399718209706406923942267894744873046875f,-3.0320999773494907714166402001865208148956298828125f,-0.76529996496886110168844652434927411377429962158203125f,1.1245999850422137011918266580323688685894012451171875f,1.955399924507736475476349369273521006107330322265625f,0.0731000736221669722425531290355138480663299560546875f,-0.4552000291973354872965273898444138467311859130859375f,0.8750998720131810681976958221639506518840789794921875f,0.0912999378785503967748127251979894936084747314453125f,-2.13290008398741548489851993508636951446533203125f,-0.4018000190302732921310280289617367088794708251953125f,0.39749997925013202948463231223286129534244537353515625f,1.1143000066250561275182917597703635692596435546875f,-1.5226999084606764878202511681593023240566253662109375f,-1.2857000201784085646750099840573966503143310546875f,-1.242100026094913989282986221951432526111602783203125f,-1.99170005841105979271787873585708439350128173828125f,0.677400004232673769166694910381920635700225830078125f,0.18140000880360551693826209884718991816043853759765625f,-0.2218999927252529591470420200494118034839630126953125f,-2.452600006142259037034136781585402786731719970703125f,-0.297099927989393386429384236180339939892292022705078125f,1.9049999924376599391706577080185525119304656982421875f,5.0782000543020675564775956445373594760894775390625f,2.20419991630986356057064767810516059398651123046875f,3.11919998170733503428664334933273494243621826171875f,0.1677999341979632408339284665999002754688262939453125f,3.04409990210235203988986540935002267360687255859375f,-0.953299988806991738243823419907130300998687744140625f,2.25819995656907668291069057886488735675811767578125f,4.4508999238520861041479292907752096652984619140625f,-1.982599997890741594375185741228051483631134033203125f,-2.64679991444274786971391222323291003704071044921875f,1.2798000581793491647886185091920197010040283203125f,1.4524999567121275045877837328589521348476409912109375f,2.24539993411898564090733998455107212066650390625f,0.7215000061914327300627292061108164489269256591796875f,1.3190999797374025614971060349489562213420867919921875f,3.455400080578773458483965441701002418994903564453125f,-0.73619998325705449104816580074839293956756591796875f,0.84580001926124037670007282940787263214588165283203125f,-1.6331001548707499981816226863884367048740386962890625f,0.430099919752775949888246032060123980045318603515625f,-2.448200003992022200094424988492392003536224365234375f,-3.188699988137184693215431252610869705677032470703125f,1.7353999969273790071611074381507933139801025390625f,-1.9229999912232162984082606271840631961822509765625f,3.368200000900029689177017644396983087062835693359375f,-1.034599950918553279421985280350781977176666259765625f,-2.6277999911203977490004035644233226776123046875f,-2.733800046843289255349418453988619148731231689453125f,-2.07640005987733644587933667935431003570556640625f,0.56999997742474184381222812589840032160282135009765625f,-4.07920010937303079145976880681701004505157470703125f,-0.23709996656477583254485352881602011620998382568359375f,-1.3136999321460722800480880323448218405246734619140625f,-0.48790000866204596974284868338145315647125244140625f,-2.03400009347498400558151843142695724964141845703125f,-2.80339998692125025314680897281505167484283447265625f,-2.474799932394921686551469974801875650882720947265625f,0.30910001406595000172927711901138536632061004638671875f,-1.203400000481306353350419158232398331165313720703125f,-1.6076999805822964617618708871304988861083984375f,-1.38920008784085613484649002202786505222320556640625f,-3.013700043681261764305645556305535137653350830078125f,3.08089996872320615040052871336229145526885986328125f,-2.255400114832819102872463190578855574131011962890625f,2.11440005195737068532935154507867991924285888671875f,-0.9065999553740031746684735480812378227710723876953125f,-0.88020006671100869954926793070626445114612579345703125f,-1.8029000230044134323037496869801543653011322021484375f,-0.10630004656389335782051830392447300255298614501953125f,2.85099998239055185678125781123526394367218017578125f,-4.49150003889948123969588777981698513031005859375f,2.2339999858885999373114827903918921947479248046875f,1.835200035083293101934032165445387363433837890625f,2.604199914428592332882317350595258176326751708984375f,0.0430999239772571485218577436171472072601318359375f,-0.5829999535232774920956444475450552999973297119140625f,0.15569992544427646574689561020932160317897796630859375f,-0.596000006008893024045391939580440521240234375f,1.400600101561843668918072580709122121334075927734375f,3.09309999624788733996183509589172899723052978515625f,1.45450004077702832461227444582618772983551025390625f,0.5030000972636041733920819751801900565624237060546875f,-0.2193999564968052151670008242945186793804168701171875f,-1.642099928604066061410549082211218774318695068359375f,-1.977800086487828234993457954260520637035369873046875f,-3.182699986412376613742480913060717284679412841796875f,1.459499943640084840268400512286461889743804931640625f,0.26530005784034538240945266807102598249912261962890625f,1.7192000170975934025818787631578743457794189453125f,1.5156999615609638709656792343594133853912353515625f,-2.45150000973045667507221878622658550739288330078125f,1.430800052863359095312034696689806878566741943359375f,0.32670002220123894876024905897793360054492950439453125f,-1.8176999840095628968583696405403316020965576171875f,-2.85880006725788238242103034281171858310699462890625f,0.22819988545328495632702470174990594387054443359375f,2.422800003134458535214434959925711154937744140625f,1.196100090549887529078887382638640701770782470703125f,-1.6502000034369521674904035535291768610477447509765625f,0.387000016562641879769302022396004758775234222412109375f,0.006700037735700714591757787275128066539764404296875f,-0.5815999867968255632177942970884032547473907470703125f,-5.0059001151084903113996915635652840137481689453125f,4.41600006417930135427241111756302416324615478515625f,-1.0427000172205265204894431008142419159412384033203125f,-0.82700002129376126713822259262087754905223846435546875f,-2.240800016430021468494260261650197207927703857421875f,0.4275999261230227777019763379939831793308258056640625f,2.852100092591346669479435149696655571460723876953125f,0.158900040729343938750162124051712453365325927734375f,-2.09329990623593431564586353488266468048095703125f,-5.37800008670240625718861338100396096706390380859375f,-0.7456000825628648076559557011933065950870513916015625f,2.2114999597370630368686761357821524143218994140625f,-0.27610021924004002347174946407903917133808135986328125f,0.202100000464917162190658928011544048786163330078125f,-2.1417000475704668360776850022375583648681640625f,0.238600063914059745684426161460578441619873046875f,-0.85809992118328815013938992706243880093097686767578125f,-0.2569000287115574820973051828332245349884033203125f,-0.71960003926381421734959076275117695331573486328125f,-2.795900053678451957495099122752435505390167236328125f,-3.144100061656533995346762822009623050689697265625f,-2.34220002229064760967958136461675167083740234375f,0.50339994331672721639137080273940227925777435302734375f,1.889200012608618362008883195812813937664031982421875f,0.0011000123932962679873526212759315967559814453125f,-1.6993999596819289177318523798021487891674041748046875f,2.18729999669343300183754763565957546234130859375f,0.67590008329451134461152150834095664322376251220703125f,-3.222899992345273911809044875553809106349945068359375f,-2.984999949373304151123420524527318775653839111328125f,2.950199952400476721692257342510856688022613525390625f,-2.423000026650726557164716723491437733173370361328125f,-2.93920003922581685174009180627763271331787109375f,-1.229500103063882132659045964828692376613616943359375f,-1.11180000383108801287335154484026134014129638671875f,-0.97979998670965517959530188818462193012237548828125f,-0.77750002220272962372149549992172978818416595458984375f,0.30820004372224152344728054231381975114345550537109375f,1.4622000647589568078643651460879482328891754150390625f,0.427899911431969481867554350174032151699066162109375f,-1.40980000250488668456227969727478921413421630859375f,-0.8510999665722260854039404875948093831539154052734375f,-0.547500051259994346963821953977458178997039794921875f,0.7071999207206072224352055854978971183300018310546875f,0.8976999046236269208520752727054059505462646484375f,-1.4062000523798172668676897956174798309803009033203125f,0.89220001137554783010585879310383461415767669677734375f,-0.692600011078269073294677582453005015850067138671875f,-2.007999979488552799722356212441809475421905517578125f,-0.856899984008073989372178402845747768878936767578125f,-0.13260007863640765890522743575274944305419921875f,3.40839994005709900903866582666523754596710205078125f,1.3597999729633325483035832803579978644847869873046875f,-1.1366999731391669303803837465238757431507110595703125f,-3.529899976069479361484582113916985690593719482421875f,0.4169999620988973365598440068424679338932037353515625f,-1.78670007849037570935024632490240037441253662109375f,0.268399921635538885045235701909405179321765899658203125f,0.27280000760480704879995528244762681424617767333984375f,-0.4812999942421918309065631547127850353717803955078125f,3.42590004267022063544345655827783048152923583984375f,-1.8233999685555684067139736725948750972747802734375f,-2.024399938634036999474119511432945728302001953125f,-1.5460000814273955338506993939517997205257415771484375f,-1.0367000439941878031646638191887177526950836181640625f,2.019500013284385442347002026508562266826629638671875f,0.3318000192165377892905553380842320621013641357421875f,3.472900016038119819228313644998706877231597900390625f,-0.93330003732889821321805357001721858978271484375f,-0.582300076451897741236507499706931412220001220703125f,0.20340000443011518882485688664019107818603515625f,-0.071600054392218392962377038202248513698577880859375f,-1.51509997388124428852052005822770297527313232421875f,0.55380006703436246429106404320918954908847808837890625f,2.7763999789267774787049347651191055774688720703125f,2.55739992158711260117343044839799404144287109375f,1.7447000172615059199188181082718074321746826171875f,-1.9658000655665990308307300438173115253448486328125f,-2.64529991786256335473126455326564610004425048828125f,-1.745400122357905647874076748848892748355865478515625f,-0.57790003017559665732250095970812253654003143310546875f,-0.9964999558255076550494777620770037174224853515625f,1.4548999804466962171289878824609331786632537841796875f,-3.815400017453729919481020260718651115894317626953125f,-2.39710001905411473899221164174377918243408203125f,-2.40390001197010239053497571148909628391265869140625f,-2.92790007053911693901682156138122081756591796875f,1.71779997081756619081716053187847137451171875f,0.8797999819412820610153858069679699838161468505859375f,2.31259992362707844648639365914277732372283935546875f,-0.8343000469326964374516819589189253747463226318359375f,-3.19490007359758010352379642426967620849609375f,0.168099992969632072714603054919280111789703369140625f,-2.41320011079609297865999906207434833049774169921875f,0.797499973978847975786266033537685871124267578125f,1.2615999850645664981385607461561448872089385986328125f,-0.79899999054521286812047264902503229677677154541015625f,0.235800010581315167002003363450057804584503173828125f,-0.7329999670833353686560940332128666341304779052734375f,-1.156500012319534587135194669826887547969818115234375f,-0.01420008080005541462043083811295218765735626220703125f,1.545599945248663953378809310379438102245330810546875f,-1.59190003500729648777678448823280632495880126953125f,-2.006899981493503570817438230733387172222137451171875f,-2.807800024610756839393843620200641453266143798828125f,-0.9111000800073139060941684874705970287322998046875f,-2.6433000613383956789448347990401089191436767578125f,-2.26340000420659759328145810286514461040496826171875f,-1.454100065940617358961617355816997587680816650390625f,2.57509999153912172431546423467807471752166748046875f,1.0079999885968862205487539540627039968967437744140625f,1.48750010829418943814062004094012081623077392578125f,1.842600066007673564882907157880254089832305908203125f,2.58010002309232877593103694380261003971099853515625f,-0.416999890908598569438936465303413569927215576171875f,0.127600011730193418912904235185123980045318603515625f,-1.140200070846079771769154831417836248874664306640625f,-1.719199971053004816212705918587744235992431640625f,-0.62960000359415924098271943876170553267002105712890625f,2.72760007893443212623196814092807471752166748046875f,1.8622000329636023696622260104049928486347198486328125f,0.10040007717013299526342962053604423999786376953125f,-0.62349999283999146815204994709347374737262725830078125f,-1.393799939741194204856356009258888661861419677734375f,-0.31899995850771689642755291060893796384334564208984375f,3.25439999920725853144176653586328029632568359375f,0.819100034443289271024468689574860036373138427734375f,0.9293999534979462406880656999419443309307098388671875f,-0.43389993183910746665787883102893829345703125f,0.119999933354557108966531586702330969274044036865234375f,-0.82489999184608375770721977460198104381561279296875f,-0.01999994650483216673819697462022304534912109375f,1.19589995311424335255878759198822081089019775390625f,-2.984200024548173235672265946050174534320831298828125f,1.42039995790496487160226024570874869823455810546875f,0.6130999797075975532578695492702536284923553466796875f,-2.72469997054636348110534527222625911235809326171875f,1.74470000794827928558561325189657509326934814453125f,3.05159994369521836432568306918255984783172607421875f,-1.26290001572146959318843073560856282711029052734375f,-0.00709998150318791321211620015674270689487457275390625f,3.151100012132525218788714482798241078853607177734375f,3.25509984755665104927402353496290743350982666015625f,-1.7003999290339650851677788523375056684017181396484375f,2.141999961055815138166735778213478624820709228515625f,1.0490999759003518665423371203360147774219512939453125f,-2.27739993797838646827358388691209256649017333984375f,-0.2409999713450685643323367912671528756618499755859375f,0.83210001814141865139617948443628847599029541015625f,0.795799973477422373235867780749686062335968017578125f,-3.08639989305883499781657519633881747722625732421875f,-2.61410003051310813049212811165489256381988525390625f,-1.948500076677649683887239007162861526012420654296875f,-0.83310006036013406482965137911378405988216400146484375f,-1.155600010534375332582612827536650002002716064453125f,-1.23799996622651864441877478384412825107574462890625f,0.9158999964952461159128915824112482368946075439453125f,0.79010000249147427364704299179720692336559295654296875f,-0.5782999448671943820698970739613287150859832763671875f,-0.7671999939411879854134213019278831779956817626953125f,0.750799934026598947411912376992404460906982421875f,-0.267899916740506061074711396940983831882476806640625f,-1.91249992212280606196372900740243494510650634765625f,0.55009998248666658327010736684314906597137451171875f,-0.8383000628471382054129890093463473021984100341796875f,-1.13540003492534236073652209597639739513397216796875f,-0.44839999355226733968038388411514461040496826171875f,0.1085999882534152138902072692872025072574615478515625f,1.116000053618103837749231388443149626255035400390625f,-1.6747998642966155369293801413732580840587615966796875f,4.823799984444679722628279705531895160675048828125f,2.3563999958768473419468136853538453578948974609375f,-0.0413999396994706447827638839953579008579254150390625f,-0.7161999985493718856588429844123311340808868408203125f,0.4494999434910715851998475045547820627689361572265625f,1.286200051169096969516658646170981228351593017578125f,-2.2923999701276418505813126103021204471588134765625f,0.959700063753129217758441882324405014514923095703125f,-1.581300017916409839102698242641054093837738037109375f,1.1391999787643538599724024606985040009021759033203125f,2.89310002401992694132104588788934051990509033203125f,-1.5828000423803920693188729273970238864421844482421875f,-1.3237001836776727259348263032734394073486328125f,-0.4037999545864756623103630772675387561321258544921875f,-2.0272000210054219593303059809841215610504150390625f,1.5348000010892752786872961223707534372806549072265625f,2.85599996065348538110129084088839590549468994140625f,1.3918000406369570587372663794667460024356842041015625f,-1.6612000386893741588067996417521499097347259521484375f,4.837499975077808045398342073895037174224853515625f,-0.98740009017512131439531231080763973295688629150390625f,-2.87679988469630387726283515803515911102294921875f,0.940299925294517446872077925945632159709930419921875f,0.10399995999783318012532618013210594654083251953125f,2.0640999954581271680353893316350877285003662109375f,1.17810000593364261334272669046185910701751708984375f,1.597899955241381508841413960908539593219757080078125f,0.4211999535664918692390301657724194228649139404296875f,-1.1168000166088332125724491561413742601871490478515625f,1.950799980574100089825151371769607067108154296875f,1.229799991738795394979888442321680486202239990234375f,-1.7843000422760837420099733208189718425273895263671875f,-1.189600098514556947293385746888816356658935546875f,0.48669997250586660708648878426174633204936981201171875f,0.5102999269708992091665322732296772301197052001953125f,3.62299999960511787122641180758364498615264892578125f,-0.00569991093426847061920170744997449219226837158203125f,-0.3281000063247969666946346478653140366077423095703125f,0.98870004345774697984694512342684902250766754150390625f,3.980699947479367839520136840292252600193023681640625f,-0.832299961694328960248867588234134018421173095703125f,1.45480006361827207683745655231177806854248046875f,-2.1088000687107442132628420949913561344146728515625f,-1.2152999993532909162041732997749932110309600830078125f,0.814300112944841725237665741587989032268524169921875f,-0.13280012778267291650990955531597137451171875f,0.46960007828622973846677268738858401775360107421875f,-2.006499966628849218608365845284424722194671630859375f,2.18609994045793865069526873412542045116424560546875f,-1.428600127355753812707916949875652790069580078125f,3.9588000060513603983736174996010959148406982421875f,-4.26059991842657392879800681839697062969207763671875f,-0.5420999879293135759183996924548409879207611083984375f,-0.56029990517795058568850663505145348608493804931640625f,-0.9940999819353215638528808995033614337444305419921875f,1.4131000529840591628527590728481300175189971923828125f,-0.35189996802657841357131474069319665431976318359375f,1.537000037685036790691128771868534386157989501953125f,-1.892800008778273923581991766695864498615264892578125f,0.91149989327788427662113690530532039701938629150390625f,0.287199954527615819444008593563921749591827392578125f,-3.331700034829974566008559122565202414989471435546875f,2.5158999877594414584791593370027840137481689453125f,-2.35640006393790191197012973134405910968780517578125f,1.571400095714628353249509018496610224246978759765625f,0.0326000555768610755791314659290947020053863525390625f,0.64969993020147043250034357697586528956890106201171875f,-1.70759996467977703815677159582264721393585205078125f,0.6303000545620911498900795777444727718830108642578125f,0.07020006075054296967863365352968685328960418701171875f,-0.0640000221729284390903558232821524143218994140625f,-1.589499986313283930172701730043627321720123291015625f,1.1951998724371211135775183720397762954235076904296875f,-0.1160999320760378594741268898360431194305419921875f,1.3417999261960371626400956301949918270111083984375f,-0.8950999857276682458717687040916644036769866943359375f,1.519499950103462726502812074613757431507110595703125f,-0.3760999443694965638229632531874813139438629150390625f,0.93319994015917284713168555754236876964569091796875f,-2.4513000075042246095335940481163561344146728515625f,-2.173599888229370780123872464173473417758941650390625f,-2.89460000828802588301869036513380706310272216796875f,-0.60559999406859199577723984475596807897090911865234375f,-0.53150007436424384632545070417108945548534393310546875f,1.755999900937080582252747262828052043914794921875f,-0.74989996696114491481210961865144781768321990966796875f,-0.49400001822412165086717550366302020847797393798828125f,-2.951099845612049410448207709123380482196807861328125f,-0.08989993931949158412209044399787671864032745361328125f,0.552799997732788650495194815448485314846038818359375f,-3.912000008217990654202367295511066913604736328125f,0.1513000165194287038872289485880173742771148681640625f,-3.961700079216807779403097811155021190643310546875f,2.998699997487665580564453193801455199718475341796875f,0.323400018809735456670750863850116729736328125f,-3.47220008360892595788982362137176096439361572265625f,-0.69080008311271734466885163783445023000240325927734375f,1.20029998005628613100270740687847137451171875f,-0.218699943619967474006671181996352970600128173828125f,-0.61580003760829737924353821654221974313259124755859375f,-0.5352000053927301603806654384243302047252655029296875f,1.758900027765333273777059730491600930690765380859375f,-1.43119998832344830219653886160813271999359130859375f,0.11640005857348445506005418792483396828174591064453125f,3.9965999990716571943494273000396788120269775390625f,-0.118500025585293489172045156010426580905914306640625f,-1.00939998598247715477782548987306654453277587890625f,-1.277200003068148514984159191953949630260467529296875f,0.794400007440148936410651003825478255748748779296875f,-2.594099918419122019486167118884623050689697265625f,-2.272399979850650009183254951494745910167694091796875f,0.5685000805705791027122586456243880093097686767578125f,-0.0291999717235553735150688225985504686832427978515625f,-3.1663000306010236073461783234961330890655517578125f,-3.4653999030314395213281386531889438629150390625f,-1.530200072000919231385296370717696845531463623046875f,-0.21119992573857349071886346791870892047882080078125f,-1.2647999231934534680732440392603166401386260986328125f,0.11370006122738285370132871321402490139007568359375f,3.27619993594586755847331005497835576534271240234375f,-1.246100038954616184838641856913454830646514892578125f,0.373799997967483477179229112152825109660625457763671875f,1.0347999090746047290423348385957069694995880126953125f,0.7502000695794828200035908594145439565181732177734375f,1.53949997770786328743497506366111338138580322265625f,0.7484000941537323203789355829940177500247955322265625f,0.53969995121210967425895432825200259685516357421875f,0.4134000745028256762481078112614341080188751220703125f,1.31769995893836000533383412403054535388946533203125f,4.52409997478276437732347403652966022491455078125f,0.1850999722421164950247884917189367115497589111328125f,0.711799917031824325164279798627831041812896728515625f,1.9388001232489937564196225139312446117401123046875f,-2.184100006820262063200743796187452971935272216796875f,3.40150002098083437118702931911684572696685791015625f,2.54609999626278948881008545868098735809326171875f,-1.955300098911672446178044992848299443721771240234375f,0.354499976664781168977924608043394982814788818359375f,-0.817499974034727738825267806532792747020721435546875f,-2.23949997603148265312711373553611338138580322265625f,-2.76839996009916067265521633089520037174224853515625f,-3.001000112626702076568108168430626392364501953125f,3.422399962751568214258668376714922487735748291015625f,-2.56200002535432513894875228288583457469940185546875f,1.381299968835711045045400169328786432743072509765625f,0.27289995699226798908654245678917504847049713134765625f,0.874700011785327635749354158178903162479400634765625f,1.172199989619852278366352038574405014514923095703125f,-1.955399998342989942301528571988455951213836669921875f,2.88159999927654819629196936148218810558319091796875f,-0.1542999855771671757764806898194365203380584716796875f,2.0617998280346387929284901474602520465850830078125f,0.4779000264316788371132815882447175681591033935546875f,3.66979990371018605088693220750428736209869384765625f,-2.6898999435476955710555557743646204471588134765625f,1.1567001002833252609747205497114919126033782958984375f,5.5471000240460028862798935733735561370849609375f,0.09880003060102404699449607505812309682369232177734375f,2.5387999824702749407379087642766535282135009765625f,-0.727300014891473889377948580658994615077972412109375f,-0.3285000758022065259211785814841277897357940673828125f,0.0007999996103349449327879483462311327457427978515625f,2.38330003397613676696664697374217212200164794921875f,0.41200005193427113869830691328388638794422149658203125f,-0.186599934791773502951173213659785687923431396484375f,2.535499924689532225130506049026735126972198486328125f,-1.304599907779693257481312684831209480762481689453125f,-1.89980004996508444747860266943462193012237548828125f,0.659500004939735706699366346583701670169830322265625f,0.1523999654337762077460638465709052979946136474609375f,1.265999984513968001920147798955440521240234375f,1.30130003350675149675907960045151412487030029296875f,3.82570005655586697912440286017954349517822265625f,-0.055099919752776838066665732185356318950653076171875f,-0.118700075662881321392205791198648512363433837890625f,-0.980500016741452373736365188960917294025421142578125f,-0.944600025815514943161588234943337738513946533203125f,-1.92789997278749947184905977337621152400970458984375f,1.0086000216692665443218857035390101373195648193359375f,-3.17949989705532853889735633856616914272308349609375f,3.31759992824643656916805412038229405879974365234375f,-0.494600041778384902357856844901107251644134521484375f,1.4027000157304119198897751630283892154693603515625f,-2.4074001297190790182867203839123249053955078125f,0.5539000404499463403595882482477463781833648681640625f,0.3197999573171141918237481149844825267791748046875f,2.981299989436567710043846091139130294322967529296875f,0.500300015781820928140177784371189773082733154296875f,-1.6760000740885734327179079627967439591884613037109375f,-5.2838998814918110014104968286119401454925537109375f,2.0687000133201483009770527132786810398101806640625f,0.437399877293408234635307962889783084392547607421875f,-1.9863000306755314028350767330266535282135009765625f,0.6162000589922069782033986484748311340808868408203125f,0.351199999946356644642264654976315796375274658203125f,-0.8499000077158205268545998478657566010951995849609375f,-0.583600021594762541354839413543231785297393798828125f,3.5170000376850367729275603778660297393798828125f,-1.253699997301400248517211366561241447925567626953125f,2.603700007843969377319126579095609486103057861328125f,-2.25560006725191986021172851906158030033111572265625f,-1.6826999288752677585989658837206661701202392578125f,0.803100020350516530243112356401979923248291015625f,2.485299912572653369124964228831231594085693359375f,-1.8735999337896711569584340395522303879261016845703125f,3.44959996283948289175214085844345390796661376953125f,-2.49249999340623329402433228096924722194671630859375f,4.34039999398439935163196423673070967197418212890625f,-0.0776000479027636469453454992617480456829071044921875f,-0.5813999014221116201639460996375419199466705322265625f,-1.9571000599950565401741187088191509246826171875f,-2.70360001302659380684190182364545762538909912109375f,-0.37950008719414551894288933908683247864246368408203125f,-0.0817000177867708288204084965400397777557373046875f,0.7158999885603785084953187833889387547969818115234375f,-1.5725000003911542290069291993859224021434783935546875f,-1.830399956116825155305605221656151115894317626953125f,-1.823199932093917396969118271954357624053955078125f,0.08810006788521895781940429515088908374309539794921875f,0.9531000600621089002828512093401513993740081787109375f,-0.993599956551195706566659282543696463108062744140625f,-2.30409996573030984023944256477989256381988525390625f,-0.2089000411763792808272910406230948865413665771484375f,-1.4571000613175335569593471518601290881633758544921875f,-3.093700049939752005911941523663699626922607421875f,0.1630999800801287147322682358208112418651580810546875f,3.2214001225180926013536009122617542743682861328125f,-0.1963998727187534765192822305834852159023284912109375f,1.58240002593249240447903503081761300563812255859375f,-0.46879997777640880496363706697593443095684051513671875f,1.2364000590763974951613590747001580893993377685546875f,3.156899954280258047134566368185915052890777587890625f,1.6907000846818078798605711199343204498291015625f,0.148099989654123742610636327299289405345916748046875f,0.45959996835291594408090531942434608936309814453125f,-1.5868997914850719865853534429334104061126708984375f,-0.59119996895194082409119573640055023133754730224609375f,-0.6247999744720760251226465697982348501682281494140625f,1.950999984793366781588019875925965607166290283203125f,-0.5030000162199128599382902393699623644351959228515625f,0.732700100313126601037083673872984945774078369140625f,-2.725600009770690501653689352679066359996795654296875f,0.28680009055584765320645601605065166950225830078125f,0.751499963350592548039230678114108741283416748046875f,-0.208299962469188582048218449926935136318206787109375f,3.239899985624850042853495324379764497280120849609375f,-0.38790003680661289120479295888799242675304412841796875f,2.574400004646182349432592673110775649547576904296875f,-2.01519996173232751601744894287548959255218505859375f,5.9975999763399361341953408555127680301666259765625f,2.117000005405395679503044448210857808589935302734375f,0.49889996987432283503949292935431003570556640625f,0.484499960385260663997541996650397777557373046875f,-1.047600014300643866960172090330161154270172119140625f,0.5627000215232380586627414231770671904087066650390625f,-2.20260004604011783158057369291782379150390625f,-2.62759998900592339765580618404783308506011962890625f,1.8053999090291561469001635487074963748455047607421875f,-3.29060001571252680463430806412361562252044677734375f,-4.83590006716400377939635291113518178462982177734375f,5.98889993340373205654714183765463531017303466796875f,1.028599991494414833681503296247683465480804443359375f,-2.56049994478747322546041687019169330596923828125f,1.10390003048479723446462230640463531017303466796875f,-2.214499901860952579824015629128552973270416259765625f,-0.5161000157274295840892364140017889440059661865234375f,2.660900059676170936739936223602853715419769287109375f,0.42640003501102408112188868472003377974033355712890625f,-3.49870000244230094921249474282376468181610107421875f,-2.98239988800361910392666686675511300563812255859375f,1.884000033833085563372833348694257438182830810546875f,0.3055999272927645282749153921031393110752105712890625f,-1.19400002150237671827426311210729181766510009765625f,1.673599915312229402530874722287990152835845947265625f,2.058799985636769580565896831103600561618804931640625f,-0.6646000846937294426197695429436862468719482421875f,1.10509998256117381032481716829352080821990966796875f,-0.192699938151239535955028259195387363433837890625f,-0.778199976238610258150174558977596461772918701171875f,-0.191000010944902154363944646320305764675140380859375f,0.458899933701755191606252992642112076282501220703125f,1.086700054425000505631260239169932901859283447265625f,1.0488999665215619216951381531544029712677001953125f,-0.41010014565438013267595351862837560474872589111328125f,1.34719991105347869364550206228159368038177490234375f,2.50670003965422427683051864732988178730010986328125f,-3.12780004083439866491289649275131523609161376953125f,4.1583000144928679020495110307820141315460205078125f,1.3911999664187424396999404052621684968471527099609375f,-0.8510000535622257444856586516834795475006103515625f,2.608700057539345085189097517286427319049835205078125f,0.1606000061757864028066933315130881965160369873046875f,0.6881000490166255811885775983682833611965179443359375f,1.03529993524104479973857451113872230052947998046875f,-0.151299899582564734146217233501374721527099609375f,-0.5157000235125421827575564748258329927921295166015625f,1.04450017328560296192563328077085316181182861328125f,-1.131700049507618377475637316820211708545684814453125f,-0.8144000051677211526879318626015447080135345458984375f,1.657599998468161306419688116875477135181427001953125f,-0.855200046091528331970721410471014678478240966796875f,0.0406000047415482345769532912527211010456085205078125f,1.1644999494329095757194636462372727692127227783203125f,-4.33920013641863988596014678478240966796875f,-4.233199889569728924243463552556931972503662109375f,-1.2075999740488818634531753559713251888751983642578125f,-0.4244999014139156923164364343392662703990936279296875f,-1.522899972526730483224355339189060032367706298828125f,-0.2116999072015295357829245403991080820560455322265625f,3.329600085140765219904324112576432526111602783203125f,-4.05050005112588618061408851644955575466156005859375f,1.367900047144293029788286730763502418994903564453125f,0.8442001022204757010314324361388571560382843017578125f,1.781100023643668261996708679362200200557708740234375f,1.34209986477121656633926249924115836620330810546875f,4.36969996898174173338702530600130558013916015625f,3.1511999644815933407926422660239040851593017578125f,3.17379983967989875992543602478690445423126220703125f,-1.254699918448179740693149142316542565822601318359375f,2.001599914656579937144442737917415797710418701171875f,2.289899978490918641682583256624639034271240234375f,0.7279001263439648372610690785222686827182769775390625f,0.7306999769464130878304786165244877338409423828125f,-1.449999948963523355160987193812616169452667236328125f,-0.380399966417253931894038032623939216136932373046875f,-1.0740999989226451827306618724833242595195770263671875f,0.269100079910455480813880058121867477893829345703125f,2.991100012877583314008234083303250372409820556640625f,1.64229990581497276025402243249118328094482421875f,0.76040002717673882415283515001647174358367919921875f,-0.008099919506907304889153920157696120440959930419921875f,-2.60559999255985008659308732603676617145538330078125f,0.5120999780759210029401629071799106895923614501953125f,4.827899912475050570037637953646481037139892578125f,1.8766000578753647687335615046322345733642578125f,2.80620000814199688221606265869922935962677001953125f,-1.17509992139190444504492916166782379150390625f,-0.6600000147894020674499415690661408007144927978515625f,0.432100110062957565304486706736497581005096435546875f,-0.558999972645194187492734272382222115993499755859375f,-3.43470009862184522120287510915659368038177490234375f,-1.3183999581992613681080683818436227738857269287109375f,3.67359989556819233058604368125088512897491455078125f,0.74440010902881592613766770227812230587005615234375f,3.60169997742325076472980072139762341976165771484375f,-3.02679997875988693323279221658594906330108642578125f,1.905999789997935511820514875580556690692901611328125f,3.40299986825138400803325566812418401241302490234375f,-1.6139999398067583769034172291867434978485107421875f,1.7856000865675503153084946461603976786136627197265625f,-0.374500038895757558776722362381406128406524658203125f,-1.25029983987360981956271643866784870624542236328125f,1.504099940696359993808073340915143489837646484375f,0.7913001558080328123878643964417278766632080078125f,2.8441000693678848421086513553746044635772705078125f,-3.33349995547533151096786241396330296993255615234375f,4.0855001306235774194419718696735799312591552734375f,2.20819997743070217666172538883984088897705078125f,-2.367600185440480675680419153650291264057159423828125f,2.887700036629288202760790227330289781093597412109375f,-1.3804999388083822875472606028779409825801849365234375f,1.6959999819621434102856483150389976799488067626953125f,-0.3310999824047089479250871590920723974704742431640625f,-1.2811999435454606288686818515998311340808868408203125f,-0.1866000543177122050764182858983986079692840576171875f,-0.794499988120047628825659558060579001903533935546875f,-1.3260999944001437000906662433408200740814208984375f,1.9246000576481223642844042842625640332698822021484375f,1.7968000022664669668159831417142413556575775146484375f,-2.53640006753280911055981050594709813594818115234375f,0.4426999532386670654915405975771136581897735595703125f,-4.65949989195168168265581698506139218807220458984375f,-1.2193000354401777318713584463694132864475250244140625f,-0.7206999622315155562546351575292646884918212890625f,-0.0457000415056938447122547586332075297832489013671875f,-2.582599969783427429348421355825848877429962158203125f,5.4242000433236370326994801871478557586669921875f,-5.97889992897063482502062470302917063236236572265625f,-0.9000999485492702234523676452226936817169189453125f,-0.52389991399496682600300800913828425109386444091796875f,-3.6919001061603413660350270220078527927398681640625f,2.688199969737976591233064027619548141956329345703125f,2.149899947738648453565701856859959661960601806640625f,1.5434000472337012777046538758440874516963958740234375f,-2.12400003305077689219615422189235687255859375f,-2.5720998654976501285318590817041695117950439453125f,-1.3259000099249167448078878805972635746002197265625f,0.2642001374617206632677834932110272347927093505859375f,-0.06969999306574392594626488062203861773014068603515625f,0.1604000376448044828947558926302008330821990966796875f,-1.245699917304515569327350021922029554843902587890625f,1.64719993886277027428377550677396357059478759765625f,-3.1902999247916046243744858657009899616241455078125f,2.432700089770555518242645121063105762004852294921875f,0.708400088174642394989177773823030292987823486328125f,1.693199976387621319418030907399952411651611328125f,-2.5869999339729563558876179740764200687408447265625f,-1.3411001149505370477754695457406342029571533203125f,-0.548399858659505579083770498982630670070648193359375f,3.5873999511286651653563239960931241512298583984375f,-4.29190013722926355654863073141314089298248291015625f,-0.1096999222293515430237675900571048259735107421875f,-0.1898000307053331336959445252432487905025482177734375f,1.6356000436335822456612731912173330783843994140625f,0.692499998956918272341454212437383830547332763671875f,0.6411999325186010789678903165622614324092864990234375f,-3.234600019203127896361138482461683452129364013671875f,2.803000147461892854749976322636939585208892822265625f,-0.3043000323295574904847171637811698019504547119140625f,1.61029994757175476394195356988348066806793212890625f,0.3075999583095299083623785918462090194225311279296875f,0.9472999290421604268175315155531279742717742919921875f,-3.52129984821080999068954042741097509860992431640625f,0.88279998255223102177069449680857360363006591796875f,-0.5846999625928699373389463289640843868255615234375f,1.56929997745603078129761343006975948810577392578125f,5.27809998384267142768067060387693345546722412109375f,2.69829997623413664342706397292204201221466064453125f,4.9829000734075901135611275094561278820037841796875f,3.919699949833750363126227966859005391597747802734375f,-2.839399982182683412901269548456184566020965576171875f,3.9000999422535311822457515518181025981903076171875f,-0.037400004102288608720527918194420635700225830078125f,0.4335000099018209152035296938265673816204071044921875f,-0.97730006475448616409806845695129595696926116943359375f,1.3351000732533637727073028145241551101207733154296875f,2.401699916924537436813125168555416166782379150390625f,3.13399987839535043576688622124493122100830078125f,0.5788999233268212751823966755182482302188873291015625f,-2.186899992427229211244821271975524723529815673828125f,0.4389999070055792262934346581459976732730865478515625f,0.9198999676316976792378454774734564125537872314453125f,-0.2283001185029764901202042892691679298877716064453125f,0.75689993785172748896883376801270060241222381591796875f,0.1595000182576484792917881350149400532245635986328125f,0.06140002186074777767998966737650334835052490234375f,-1.97460000918209654940937980427406728267669677734375f,1.6520998885571973602282014326192438602447509765625f,1.1536000480070722762349078038823790848255157470703125f,-0.06890009381472828042802802883670665323734283447265625f,-2.739699951919913889497593117994256317615509033203125f,-0.3439000405430789886196407678653486073017120361328125f,-0.6290999785453068593454872825532220304012298583984375f,3.158399976918846885354241749155335128307342529296875f,3.150100022123755394432009779848158359527587890625f,-1.69070002094209481668940497911535203456878662109375f,0.3683999849095924705721927239210344851016998291015625f,-0.4556999848067759817382693654508329927921295166015625f,-2.04049994316697169693952673696912825107574462890625f,1.9120000453591343703152460875571705400943756103515625f,-3.472200080069901328982950872159563004970550537109375f,3.8256999889418477067692947457544505596160888671875f,-0.763600091071427566902229955303482711315155029296875f,-3.3361999146372074420696662855334579944610595703125f,-5.51009987812265666917710404959507286548614501953125f,-1.306299996551871345218387432396411895751953125f,4.67610005258917826864717426360584795475006103515625f,0.289599992250652638059449373031384311616420745849609375f,1.93200012329220616180691649788059294223785400390625f,1.444900009392203532598841775325126945972442626953125f,-2.58869990681409856136951930238865315914154052734375f,3.09209995583593855172921394114382565021514892578125f,1.51209987924397015746080796816386282444000244140625f,-1.315700080807506555657937497016973793506622314453125f,-1.307000166319312040030808930168859660625457763671875f,-2.034199961292743719099007648765109479427337646484375f,-1.510900068318843825210251452517695724964141845703125f,-0.9438000498980290142725380064803175628185272216796875f,-0.81470002100542071588051840080879628658294677734375f,3.79050002981722400363651104271411895751953125f,-1.1067999751091008864278819601167924702167510986328125f,-1.720400023153423330057876228238455951213836669921875f,2.39970000709146091821821755729615688323974609375f,-3.260099996419249190893197010154835879802703857421875f,1.2431998394459486956264981927233748137950897216796875f,1.165799957533177888535647070966660976409912109375f,3.0548000054106108080986814456991851329803466796875f,0.41690002425983563316691515865386463701725006103515625f,-1.0804000408112999220833216895698569715023040771484375f,0.9675999937370429737626409405493177473545074462890625f,-1.02040002892762426967010469525121152400970458984375f,-0.628099936568737415854002392734400928020477294921875f,-1.749499995365740634412077270098961889743804931640625f,-3.29669993104338576728196130716241896152496337890625f,-1.133199977542460157309278656612150371074676513671875f,1.327900082180649921070880736806429922580718994140625f,1.763099945472180962724451092071831226348876953125f,1.76470000385045722168797510676085948944091796875f,-3.038700058135391568470140555291436612606048583984375f,-1.588499994460491837600102371652610599994659423828125f,0.504600005400923645737520928378216922283172607421875f,0.987199870634079434950081122224219143390655517578125f,-0.61319995235949686929188828798942267894744873046875f,-0.89030000441968493163358289166353642940521240234375f,1.708400023987889593257705200812779366970062255859375f,0.891800017093122843192531945533119142055511474609375f,-2.04120007234737244061761884950101375579833984375f,-1.3424999345839001563263082061894237995147705078125f,2.347300057788193061725223742541857063770294189453125f,2.7711000091522937083254873869009315967559814453125f,-0.179700187730788218942734602023847401142120361328125f,-0.2609000048398966153939682044438086450099945068359375f,-2.56150000641494823838684169459156692028045654296875f,-0.830500040620567148152986192144453525543212890625f,0.8102999228544540688545794182573445141315460205078125f,-0.02129993443265754127224909098003990948200225830078125f,-1.185599895851312535199895137338899075984954833984375f,-0.4010000369474300274674760657944716513156890869140625f,0.1698999121993762795312932212254963815212249755859375f,-1.0598000659093271824673365699709393084049224853515625f,0.366500056333840529276812958414666354656219482421875f,-1.809399829781053181676497843000106513500213623046875f,-0.60999996963888258871833158991648815572261810302734375f,0.51790005053430820947113488728064112365245819091796875f,-0.92140008712783572075721849614637903869152069091796875f,-1.7879000504225486256615340607822872698307037353515625f,0.54989997374117327577636160640395246446132659912109375f,1.2150999446377170887245711128343828022480010986328125f,1.1646000460810963517133131972514092922210693359375f,-0.7605000000894073242108106569503434002399444580078125f,1.386699982377887607754018972627818584442138671875f,-0.61049997606128492133592544632847420871257781982421875f,-0.825299971190095238426920332130976021289825439453125f,1.6843000147089355333918092583189718425273895263671875f,-0.7605999737471342836414578414405696094036102294921875f,0.385799994245915822688175467192195355892181396484375f,-0.1991999228477485672783586778677999973297119140625f,2.267099835099279925287873993511311709880828857421875f,-2.2460999242901795724947078269906342029571533203125f,4.52090001087486825781525112688541412353515625f,-2.494200093670934759160218163742683827877044677734375f,-0.90649995116889403590221263584680855274200439453125f,-0.5763999554574497086179007965256460011005401611328125f,-2.101499978847800154113656390109099447727203369140625f,-0.90470000232309299992294882031274028122425079345703125f,0.40479997732192185555533114893478341400623321533203125f,0.397900072215496336536233457081834785640239715576171875f,-0.80290004585906871170664089731872081756591796875f,-2.83900004068761990794200755772180855274200439453125f,-3.1474998663738364967912275460548698902130126953125f,-1.1227999929271634815819425057270564138889312744140625f,-0.3830999722570183063652393684606067836284637451171875f,2.402499934434889627965503677842207252979278564453125f,-2.403000085696578747018747890251688659191131591796875f,-2.0081999570906159391370238154195249080657958984375f,0.5926000050805513286178438647766597568988800048828125f,-3.170700107145310031597773559042252600193023681640625f,-3.24249994490295723181816356373019516468048095703125f,-1.6492998728871359759295955882407724857330322265625f,-1.88840003477260420794436868163757026195526123046875f,-5.4044998942613613479579726117663085460662841796875f,-0.8550000165961677556225595253636129200458526611328125f,-5.27989996796697180769797341781668365001678466796875f,0.7357000159502036229497434760560281574726104736328125f,-0.13350005263090081175647583222598768770694732666015625f,2.440300013956427704897578223608434200286865234375f,0.076200089465082221362735026559676043689250946044921875f,0.4050999567076563057099747311440296471118927001953125f,-2.72050001201033619935287788393907248973846435546875f,-4.44229999009966913803282295702956616878509521484375f,-1.824999981708824048354244951042346656322479248046875f,-1.7785999825537202045921958415419794619083404541015625f,4.6616000333443281533618574030697345733642578125f,-1.8479000172674642232806263564270921051502227783203125f,-0.018799863857029919245178462006151676177978515625f,-1.2888000880822556570848291812581010162830352783203125f,0.450099959799648008385020148125477135181427001953125f,-0.44580003054887118452143113245256245136260986328125f,3.02789982247203592891082735150121152400970458984375f,2.5040001226067527539953516679815948009490966796875f,-2.8199999065324643510166424675844609737396240234375f,0.2510999294310829466070345006301067769527435302734375f,-1.28959997869059339592467949842102825641632080078125f,0.57699997198954289245875770575366914272308349609375f,2.18370005701780200979555957019329071044921875f,0.250000017508864136317470183712430298328399658203125f,2.355500010520222620158392601297236979007720947265625f,-2.58699989306926791954310829169116914272308349609375f,1.5694000000268222283494878865894861519336700439453125f,2.711100089991094197472421001293696463108062744140625f,4.22749999750405702769739946234039962291717529296875f,-2.464099988640845229070919231162406504154205322265625f,-0.5634000582791871547527762231766246259212493896484375f,-0.396699973995985555319521154160611331462860107421875f,-1.396599925534427466544684648397378623485565185546875f,1.6199001181706762508838437497615814208984375f,1.547799944703279972912923767580650746822357177734375f,-2.954899931664019430144207944977097213268280029296875f,-4.07800006761029454338540745084173977375030517578125f,-4.0144000078685593280169996432960033416748046875f,-1.529899970574675638346207051654346287250518798828125f,1.2195000035241216895798288533114828169345855712890625f,-1.372399901321530446551832937984727323055267333984375f,0.247800007716565406923336922773160040378570556640625f,-3.47290016702413328886223098379559814929962158203125f,-1.0744000143319372142514112056232988834381103515625f,3.115300068047644810320662145386449992656707763671875f,0.15069990188181370260878111366764642298221588134765625f,-3.61979993999451465924721560440957546234130859375f,0.6061999742418524572684646045672707259654998779296875f,1.92619993255585431057852474623359739780426025390625f,-1.0418000153422328768471061266609467566013336181640625f,3.219200073833762498765054260729812085628509521484375f,0.7971999297358094427323749187053181231021881103515625f,2.60059997546076804297854323522187769412994384765625f,1.06079985786304131778479131753556430339813232421875f,-2.307199958476423784503595015848986804485321044921875f,0.0429999303147194922303242492489516735076904296875f,-1.5248000987105079229166904042358510196208953857421875f,4.6787999622046942960196247440762817859649658203125f,-1.3692998724773541852073321933858096599578857421875f,-0.3298000465005639370019707712344825267791748046875f,-0.68430000083222874973643001794698648154735565185546875f,-0.17680005993396008534546126611530780792236328125f,4.1521999737501147365037468262016773223876953125f,-1.734099969027190102366375867859460413455963134765625f,-0.25439992894828444836008429774665273725986480712890625f,1.20610002852380393534303948399610817432403564453125f,-0.59579998409450229956973998923785984516143798828125f,-1.063600004830955736423447888228110969066619873046875f,0.75300003287196037859985153772868216037750244140625f,0.958000019665806501478755308198742568492889404296875f,-2.461000066433100652574239575187675654888153076171875f,1.6160000541210164382022185236564837396144866943359375f,1.2008999753728504256145015460788272321224212646484375f,1.3034999950379144717516055607120506465435028076171875f,-0.59289989861696834605453432232025079429149627685546875f,-2.633199930380281994501956432941369712352752685546875f,-0.693100016513465977396890593809075653553009033203125f,-2.35320014361590157392356559284962713718414306640625f,-0.505799850188939803530274730292148888111114501953125f,1.798799865682422449708610656671226024627685546875f,3.069000029101967452760391097399406135082244873046875f,-1.520299909611046995649985547061078250408172607421875f,0.1055000342875711538681571255438029766082763671875f,1.3138000257954001170901392470113933086395263671875f,2.8965999299302698233304909081198275089263916015625f,2.748700016300380521983015569276176393032073974609375f,-0.8530999655514956980795204799505881965160369873046875f,0.41039995684325603075848221124033443629741668701171875f,1.083500005245208175352900070720352232456207275390625f,1.235499963954091118267797355656512081623077392578125f,-0.626099866212905364903917870833538472652435302734375f,1.23199992689490311903455221909098327159881591796875f,-0.4521000332847238123434863155125640332698822021484375f,3.337500038631258547638935851864516735076904296875f,-0.75810012778788848919475640286691486835479736328125f,-0.512099992232023293325937629560939967632293701171875f,0.9475000063702465613602043958962894976139068603515625f,2.60649991495907418226352092460729181766510009765625f,-0.5654999373182665323156470549292862415313720703125f,-0.219000046350062316236062542884610593318939208984375f,0.7486002279013395099838135138270445168018341064453125f,0.3338000851765265952764139001374132931232452392578125f,-2.150799987726657480635594765772111713886260986328125f,-2.160900003610552122523813523002900183200836181640625f,2.17059996840134150630774456658400595188140869140625f,1.423299978990853364990698537440039217472076416015625f,-1.01879992811829023224845514050684869289398193359375f,-2.237900002478062422284210697398521006107330322265625f,-1.3608999668046817266287007441860623657703399658203125f,0.8832999724343413294747051622834987938404083251953125f,-0.42820004312619630582048557698726654052734375f,0.62840000506341520125630495385848917067050933837890625f,1.101499890781939772210762384929694235324859619140625f,-2.66059992177933413159962583449669182300567626953125f,-1.7450000841729347289543738952488638460636138916015625f,-4.120499991819261964565157541073858737945556640625f,0.8988000364869821456892395872273482382297515869140625f,-0.9990999202817671953624767411383800208568572998046875f,1.68170011168271482659974935813806951045989990234375f,4.23960008722692993643477166187949478626251220703125f,-3.7038000179350358820329347508959472179412841796875f,3.05450004930794261071014261688105762004852294921875f,0.6886999285697934514161033803247846662998199462890625f,-3.36159992916658456607592597720213234424591064453125f,-0.54250012569129590911387595042469911277294158935546875f,1.805099914313853748382143749040551483631134033203125f,0.8828999529317027583630306253326125442981719970703125f,2.01220008368343261651034481474198400974273681640625f,-0.1334000229820595517793435647035948932170867919921875f,-2.19089991647601056712346689892001450061798095703125f,1.36240000229552293831147835589945316314697265625f,-2.64629990181774044089024755521677434444427490234375f,-4.54219991354942376204917309223674237728118896484375f,3.33010005796104646691446760087274014949798583984375f,2.4198999835759398280288223759271204471588134765625f,-3.46859992015510609775219563744030892848968505859375f,1.54100007565319518931801212602294981479644775390625f,4.8647999919623128306511716800741851329803466796875f,-3.1413998688444504381322985864244401454925537109375f,-2.368699898096917610956779753905721008777618408203125f,0.895600075708330667367818023194558918476104736328125f,0.54330004511475582518187366076745092868804931640625f,-1.5568001405119902269547083051293157041072845458984375f,2.607599972894041773230355829582549631595611572265625f,-3.323600043797492187280795405968092381954193115234375f,-0.7835999184042228282720543575123883783817291259765625f,-1.789700028400122988614384667016565799713134765625f,-0.81850000768527397809748435975052416324615478515625f,0.0039000856935966776717350512626580893993377685546875f,3.063500063024460207117272148025222122669219970703125f,0.4830000452771792129880168431554920971393585205078125f,-0.1784000517040469357965548624633811414241790771484375f,-3.267599990458788017377855794620700180530548095703125f,-0.2225001354143021359277554438449442386627197265625f,-1.18449998255074095965255764895118772983551025390625f,4.44079995358437340513546587317250669002532958984375f,1.6077999612621971525783237666473723948001861572265625f,-2.928599998348950794735401359503157436847686767578125f,0.9801999942541101962234506572713144123554229736328125f,0.6917998819023376544379289043718017637729644775390625f,2.4699000536486526158341803238727152347564697265625f,-0.954400002802163083259756604093126952648162841796875f,-5.1207000338315964427238213829696178436279296875f,0.73179995199367464664419458131305873394012451171875f,-0.4535000194571907616847283861716277897357940673828125f,1.1545000155754381321315804598270915448665618896484375f,3.11710000776648410436564518022350966930389404296875f,1.94370005619824137710338618489913642406463623046875f,-3.818900025269389431059607886709272861480712890625f,-1.6926999527916313237341228159493766725063323974609375f,-0.9205000365599980138853197786374948918819427490234375f,6.1842999334231034680442462558858096599578857421875f,-0.405199937685579936186286431620828807353973388671875f,-2.45929998859465204219532097340561449527740478515625f,0.4627999922379861796883915303624235093593597412109375f,-4.98599999216943867708096149726770818233489990234375f,-2.29220002441406212057017910410650074481964111328125f,1.990700011479855646712167072109878063201904296875f,1.815700001011788078386643974226899445056915283203125f,-2.650299872395395883728497210540808737277984619140625f,-0.90229996838122661362291410114266909658908843994140625f,-2.69500011626631152950039904681034386157989501953125f,2.672199944301695939685714620281942188739776611328125f,-1.4249000387474899387285631746635772287845611572265625f,-0.9540000516027216104220087800058536231517791748046875f,-1.51179992984682343148961081169545650482177734375f,1.707499859370290806737102684564888477325439453125f,-1.57430000424087079835544500383548438549041748046875f,0.782300065927951759903180573019199073314666748046875f,-0.660599991852045764773038172279484570026397705078125f,3.2563999151125546660523468744941055774688720703125f,2.881400004109739398927558795548975467681884765625f,-3.257100008623301956589557448751293122768402099609375f,4.4388000098139013260833962704055011272430419921875f,0.6417999422520377361678356464835815131664276123046875f,2.451700015868246484984638300375081598758697509765625f,0.7902000178724535306429288539220578968524932861328125f,-3.121899951784313120839442490250803530216217041015625f,-2.01739999464005137497224495746195316314697265625f,0.9705998568102727208639635136933065950870513916015625f,1.14479999393671949547979238559491932392120361328125f,5.28170002346783729052503986167721450328826904296875f,2.124200005791337719784905857522971928119659423828125f,-1.703299972210822854634670875384472310543060302734375f,-3.96250003781169635175274379434995353221893310546875f,0.770099990049003313430375783354975283145904541015625f,-0.626599943732471853508059211890213191509246826171875f,0.7725999907940632738245767541229724884033203125f,-1.37040000434070829982147188275121152400970458984375f,-0.847299964451041720536750290193594992160797119140625f,-2.945199978235365279033430851995944976806640625f,-1.0048999262809721333411516752676106989383697509765625f,2.17980008447989792585985924233682453632354736328125f,0.50019999425411310056688307668082416057586669921875f,0.48739990610852979013856156598194502294063568115234375f,-3.272899914002417975922298865043558180332183837890625f,-4.00950000449270049784900038503110408782958984375f,0.46430003229230731864873860104125924408435821533203125f,0.520599913769959954379373812116682529449462890625f,-1.2055000845789913821448635644628666341304779052734375f,-0.8907999379008997298257099828333593904972076416015625f,2.361399945287407486915753906941972672939300537109375f,-0.364099974261225423077803498017601668834686279296875f,0.292400046943125602183499722741544246673583984375f,0.22889998783022147410548541301977820694446563720703125f,-1.807100002253053450118613909580744802951812744140625f,1.184199929553269914350721592199988663196563720703125f,-0.8373999710217123748634548974223434925079345703125f,-0.6293999566644441490126382632297463715076446533203125f,-0.77960004766434476408676346181891858577728271484375f,1.211799930815398607109045769902877509593963623046875f,0.1953000820547325933063120828592218458652496337890625f,0.3311998435214167368911830635624937713146209716796875f,0.4906000247836110883525861936504952609539031982421875f,0.789499955076724546643163193948566913604736328125f,-2.54429992034733398753587607643567025661468505859375f,4.3139000497817985291248987778089940547943115234375f,-1.5128999798655531350277669844217598438262939453125f,-1.230799939577280621705313023994676768779754638671875f,1.4300999869197601466197511399514041841030120849609375f,0.834700058872999139936155188479460775852203369140625f,3.92360004150644048337426283978857100009918212890625f,-1.6296998534604905106704109130077995359897613525390625f,3.700500122502446753713911675731651484966278076171875f,2.24570003785490968084559426642954349517822265625f,-0.1032999834798278282477212997036986052989959716796875f,0.449400026532260188361078689922578632831573486328125f,-2.41870005146712241383966102148406207561492919921875f,1.0613999482676399477298900819732807576656341552734375f,-1.8518999047338962071762580308131873607635498046875f,-0.831000006213785002984195671160705387592315673828125f,-0.565900129413604791039915653527714312076568603515625f,2.293000018436463616211540283984504640102386474609375f,0.8114999773204341071419776199036277830600738525390625f,-0.69239998208582365091245947041898034512996673583984375f,-0.46369990100264768528859349316917359828948974609375f,-1.2902999517440798538103763348772190511226654052734375f,-3.493399986660481015832147022592835128307342529296875f,1.200700027535854363947009915136732161045074462890625f,2.23610001004636416155335609801113605499267578125f,-0.4946999162875103106529195429175160825252532958984375f,-1.094300053749979184658513986505568027496337890625f,-1.730400108052789764911949532688595354557037353515625f,-0.3875000195205207109694356404361315071582794189453125f,-1.637900066143275434882298213778994977474212646484375f,-0.6657999943949277632526673187385313212871551513671875f,2.95070001934021686196274458779953420162200927734375f,0.797800029751659423027376760728657245635986328125f,-0.1792000356867900767721124566378421150147914886474609375f,4.57810003702119150403859748621471226215362548828125f,-1.0624999761395150787990360186086036264896392822265625f,-3.637599961140752125032804542570374906063079833984375f,-4.6798998861595979548155810334719717502593994140625f,-1.107199990215895990530725612188689410686492919921875f,2.229099940174820826399582074373029172420501708984375f,1.7988000126451260118898289874778129160404205322265625f,0.65230000164806778428783218259923160076141357421875f,0.859799950686097158580878385691903531551361083984375f,3.8983999830469482361650079838000237941741943359375f,-0.6895999747604140139856099267490208148956298828125f,-4.2526000801824039854182046838104724884033203125f,1.34990000955983990849063047789968550205230712890625f,0.84070000678598721943757254848605953156948089599609375f,-1.6132999010018986840719890096806921064853668212890625f,-0.6846000407353038941238310144399292767047882080078125f,3.20380008817538541876501767546869814395904541015625f,4.15639997266829208655281036044470965862274169921875f,-1.031299952444434619991397994454018771648406982421875f,-0.7725000980310137155271377196186222136020660400390625f,0.03150000000745134531854318993282504379749298095703125f,2.2103999372854818972200519056059420108795166015625f,-0.265800001975894151229340423014946281909942626953125f,0.4530999518051743724100788313080556690692901611328125f,0.9139999956488598087389618740417063236236572265625f,-1.8707999209508301419191411696374416351318359375f,4.9974000723496079245933287893421947956085205078125f,-1.526799913977087808092392151593230664730072021484375f,1.088900011038780757388622077996842563152313232421875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_padded_neg)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,3,5,8,7};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,2,6,3,6};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1,1},        // move_strides
                                                                  Strides{1,1,1,1},        // filter_dilation
                                                                  CoordinateDiff{-1,2,-3,2}, // below_pads
                                                                  CoordinateDiff{1,0,0,-3}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.01170000670403315012890743673779070377349853515625f,-1.02729999240934777304801173158921301364898681640625f,-1.719799970113486065059760221629403531551361083984375f,-0.0036000744193793732250696848495863378047943115234375f,0.0f,0.0f,-1.8938999921515584645703711430542171001434326171875f,2.35600000921264385311815203749574720859527587890625f,1.7772000180810696434718920500017702579498291015625f,-0.1864000910207626038328498907503671944141387939453125f,0.0f,0.0f,1.488900024934113108798783287056721746921539306640625f,-0.53959991955160990784179375623352825641632080078125f,3.72060005380362301963259596959687769412994384765625f,-2.66530003973543561102133025997318327426910400390625f,0.0f,0.0f,1.2388000533625465049425429242546670138835906982421875f,2.33519998713880649887641993700526654720306396484375f,-4.62359995198771489555156222195364534854888916015625f,0.6142999418422581126009163199341855943202972412109375f,0.0f,0.0f,-0.3749000039532768280281516126706264913082122802734375f,1.1165000290088358259055212329258210957050323486328125f,0.6768998986989254174062580204918049275875091552734375f,0.5531999805971976602592121707857586443424224853515625f,0.0f,0.0f,0.2581999718055119075188486021943390369415283203125f,-1.49609984529539730147007503546774387359619140625f,1.5340000701546661066032584130880422890186309814453125f,0.97610003321766836137385325855575501918792724609375f,0.0f,0.0f,-2.273900067178905803899624515906907618045806884765625f,-0.5600999658346179632673056403291411697864532470703125f,1.3194999327436101044241922863875515758991241455078125f,2.64999999154358878428183743380941450595855712890625f,0.0f,0.0f,-0.8088001000776883220311219702125526964664459228515625f,0.20039997118562358213722518485155887901782989501953125f,-3.57019997972548086551114465692080557346343994140625f,0.9705000074654825947817471387679688632488250732421875f,0.0f,0.0f,1.4411000420838588897964882562519051134586334228515625f,0.3761999877646584611312619017553515732288360595703125f,0.334899949713053590727440678165294229984283447265625f,1.160999950148164483465507146320305764675140380859375f,0.0f,0.0f,-1.985199962030349762898140397737734019756317138671875f,1.9118000550538292170443810391589067876338958740234375f,0.9781000257149330412431709191878326237201690673828125f,-1.437799949993193049380124648450873792171478271484375f,0.0f,0.0f,0.435899942329525169615322965910308994352817535400390625f,2.577399998737870401299687728169374167919158935546875f,-0.9712000380560770196325393044389784336090087890625f,-4.83550002121180266811961701023392379283905029296875f,0.0f,0.0f,2.130899951083957688524606055580079555511474609375f,-1.7351000262402005436257468318217433989048004150390625f,1.3062999666377894669722081744112074375152587890625f,1.6504999797865753663472787593491375446319580078125f,0.0f,0.0f,3.28500006562099056139913955121301114559173583984375f,0.4042998597554861017755456487066112458705902099609375f,-0.665200019548833676452659346978180110454559326171875f,0.84889996018856839210542375440127216279506683349609375f,0.0f,0.0f,2.8786000739164645523260332993231713771820068359375f,0.63140003005638778432739854906685650348663330078125f,-3.9377000856727359945352873182855546474456787109375f,0.40879990934282528503018738774699158966541290283203125f,0.0f,0.0f,2.5151999971412113410451638628728687763214111328125f,2.153999947611243470646513742394745349884033203125f,-1.26769988333359151511103846132755279541015625f,-1.506800055072456956395399174652993679046630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.5913000672578814231883370666764676570892333984375f,1.686900019323825716810461017303168773651123046875f,0.62949997726082784765821998007595539093017578125f,-0.45140000597238572055402983096428215503692626953125f,0.0f,0.0f,0.307099991300702868102234788239002227783203125f,-0.059199971798063355521435369155369699001312255859375f,0.800299973946810183633715496398508548736572265625f,-1.28569998182654376961409070645458996295928955078125f,0.0f,0.0f,1.02760009689033093849275246611796319484710693359375f,1.0399999034404758813110447590588591992855072021484375f,-0.607499964088201949152789893560111522674560546875f,1.366300005082786039878328665508888661861419677734375f,0.0f,0.0f,-0.30090003836750955912293647998012602329254150390625f,1.5047999524556103789763028544257394969463348388671875f,-0.5605000301897522518856931128539144992828369140625f,-1.71100004726648347030959484982304275035858154296875f,0.0f,0.0f,2.20589995216429191060569792171008884906768798828125f,-2.75619994514733601675970930955372750759124755859375f,1.398900059169530862845931551419198513031005859375f,-1.71209994980096791294954527984373271465301513671875f,0.0f,0.0f,0.43040006361007687729625104111619293689727783203125f,0.0452999336019159759558760924846865236759185791015625f,-1.78339998680949207710000337101519107818603515625f,0.737400031967461178084022321854718029499053955078125f,0.0f,0.0f,0.8108998629808426006349009185214526951313018798828125f,-0.0474000107891854316477520114858634769916534423828125f,-0.3014000602126127859037296730093657970428466796875f,-1.008700034964084668587247506366111338138580322265625f,0.0f,0.0f,0.465799997878074467649867074214853346347808837890625f,-1.66539998344182915701594538404606282711029052734375f,1.258000065952539880953509054961614310741424560546875f,-1.254699942457676797857857309281826019287109375f,0.0f,0.0f,-1.99229999978542338112674769945442676544189453125f,-1.060199970225990551142558615538291633129119873046875f,0.235700030776857971659410395659506320953369140625f,-0.755600081408024148998947566724382340908050537109375f,0.0f,0.0f,0.03409995976984436794055000063963234424591064453125f,2.18970004102885784647014588699676096439361572265625f,-1.021700028422474648692741538980044424533843994140625f,1.837400045862793707129867470939643681049346923828125f,0.0f,0.0f,-0.2391999475091701743423300285940058529376983642578125f,0.2739000012785191007225193970953114330768585205078125f,2.429599964106083032078231553896330296993255615234375f,-0.720800024029613961573659253190271556377410888671875f,0.0f,0.0f,2.165299967092276123281635591411031782627105712890625f,1.2085999971568579525182940415106713771820068359375f,-1.6238999918349090734182027517817914485931396484375f,2.75959997543096591243738657794892787933349609375f,0.0f,0.0f,-0.09529997111558952127552402089349925518035888671875f,1.227500030100344918793098258902318775653839111328125f,-1.32559996361434517808675082051195204257965087890625f,0.8570999354958537264081996909226290881633758544921875f,0.0f,0.0f,-0.7343000525206322404159209327190183103084564208984375f,0.97219995065331499262129000271670520305633544921875f,1.425199990901349966776479050167836248874664306640625f,-0.29670000610798641371701478419709019362926483154296875f,0.0f,0.0f,-1.273000006943940665138370604836381971836090087890625f,0.360999992057681851065353839658200740814208984375f,-0.2223000113151964995950038428418338298797607421875f,-2.399700036521255075427916381158865988254547119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.040200008559227118354328922578133642673492431640625f,0.934199988189339247668385723954997956752777099609375f,-0.2079000573702154408550768494023941457271575927734375f,0.319199982526898740076148897060193121433258056640625f,0.0f,0.0f,0.9669000117242332326128462227643467485904693603515625f,0.5152999822914592353839680072269402444362640380859375f,-0.801100000733137473929446059628389775753021240234375f,-2.769699988800287115964238182641565799713134765625f,0.0f,0.0f,-1.056999959826470103507745079696178436279296875f,1.1943999703735102002610801719129085540771484375f,1.95399996308982348836025266791693866252899169921875f,0.10320004239976388049626621068455278873443603515625f,0.0f,0.0f,-1.6405999360471952286388841457664966583251953125f,-0.8245000956133001768222356986370868980884552001953125f,4.2618000263504693947425039368681609630584716796875f,-1.3384999146834015704143894254229962825775146484375f,0.0f,0.0f,0.1944999705553047686379386504995636641979217529296875f,-2.52609995133578735249102464877068996429443359375f,1.1214999944940202691867625617305748164653778076171875f,-0.85449993535131252286873859702609479427337646484375f,0.0f,0.0f,1.23859991806894509380754243466071784496307373046875f,3.787799967502057274515436802175827324390411376953125f,-3.473699959713220319912352351821027696132659912109375f,2.377400013639031417511660038144327700138092041015625f,0.0f,0.0f,-0.7294999917149527224324856433668173849582672119140625f,1.306599938684701811553168226964771747589111328125f,0.3946999333307152912908577491180039942264556884765625f,-0.3716999427035452274736826439038850367069244384765625f,0.0f,0.0f,0.012800039549171859931675498955883085727691650390625f,0.15190005758255697987380017366376705467700958251953125f,-0.31889994126409160202229031710885465145111083984375f,-1.6855999981105327378827496431767940521240234375f,0.0f,0.0f,-2.8342999353230009518256338196806609630584716796875f,-0.67209996716082198719988127777469344437122344970703125f,-0.002300043185055056227383829536847770214080810546875f,-4.31040004081129968227514837053604423999786376953125f,0.0f,0.0f,2.3627999895930287266310187987983226776123046875f,-0.86260002493634857501092483289539813995361328125f,-3.192199935416876410698705512913875281810760498046875f,4.525100048163533728029506164602935314178466796875f,0.0f,0.0f,3.7969999565482144276984399766661226749420166015625f,2.3076999411873515555271296761929988861083984375f,1.56859993390142893332495077629573643207550048828125f,-0.7455000834614036175906903736176900565624237060546875f,0.0f,0.0f,4.3891000179216295151718441047705709934234619140625f,1.025099945326894523844885043217800557613372802734375f,-0.325100029239058674335183241055347025394439697265625f,-0.60559988124817498711394136989838443696498870849609375f,0.0f,0.0f,1.6404999819472425581778907144325785338878631591796875f,-4.07230004428401581861862723599188029766082763671875f,-0.8755000078007568742322064281324855983257293701171875f,-2.11969998727291741857925444492138922214508056640625f,0.0f,0.0f,-1.7551000478282581251932015220518223941326141357421875f,-3.7766999628573643121853820048272609710693359375f,1.1971999765068292997938215194153599441051483154296875f,0.9785000946149242651017630123533308506011962890625f,0.0f,0.0f,-2.754600013000517488848117864108644425868988037109375f,-1.6874998435564345644621653264039196074008941650390625f,-1.70889999835416706019941557315178215503692626953125f,-1.947499975766986413105996689409948885440826416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9123999583184723149997807922773063182830810546875f,-0.77170004190802554688843883923254907131195068359375f,-0.7607999929606925615388490768964402377605438232421875f,-0.76340000230670046477143841912038624286651611328125f,0.0f,0.0f,0.20360002532005327946507122760522179305553436279296875f,-0.7528000405550001605803345228196121752262115478515625f,-1.6658999745905400668988249890389852225780487060546875f,0.39839998651146846242454557796008884906768798828125f,0.0f,0.0f,0.97289998269677102626928899553604423999786376953125f,-0.3431999687880278315788018517196178436279296875f,-0.152199997442961088012225445709191262722015380859375f,0.990599981495738024506181318429298698902130126953125f,0.0f,0.0f,1.854200046117604028239611579920165240764617919921875f,-0.063700025371461299616981932558701373636722564697265625f,-1.5806000087276095289468003102228976786136627197265625f,-0.2422999787002793059542682385654188692569732666015625f,0.0f,0.0f,-0.7405999695375566904687048008781857788562774658203125f,-0.048600027629732256428951586713083088397979736328125f,0.099699939104915369370019107009284198284149169921875f,0.4480999896913770985662495149881578981876373291015625f,0.0f,0.0f,-1.0788000010594729349833187370677478611469268798828125f,-0.60029994585812118668144421462784521281719207763671875f,0.70509996408373165621696898597292602062225341796875f,-0.5446999716639526756267741802730597555637359619140625f,0.0f,0.0f,1.067400012521446672764113827724941074848175048828125f,0.019100016971676847532535248319618403911590576171875f,0.36780003485530665674474448678665794432163238525390625f,-1.2873999462857839137797100192983634769916534423828125f,0.0f,0.0f,-0.1820000150352705237111194946919567883014678955078125f,1.6833999938502903770398688720888458192348480224609375f,-0.8274000493273139245076208680984564125537872314453125f,0.9393999475747343286258228545193560421466827392578125f,0.0f,0.0f,0.227800003488361380021842705900780856609344482421875f,3.050099921913446365806521498598158359527587890625f,0.11259997898489182421855048232828266918659210205078125f,-0.633199989388882666929703191271983087062835693359375f,0.0f,0.0f,-0.74309996741414063858854888167115859687328338623046875f,0.1125999907940624655822148270090110599994659423828125f,0.8310000223815443565200666853343136608600616455078125f,-0.510500008694827034361196638201363384723663330078125f,0.0f,0.0f,-0.047300004591047439816975384019315242767333984375f,-0.314199990387260985613693264895118772983551025390625f,-0.917099966900050578288983160746283829212188720703125f,-0.36340008273571677843705174382193945348262786865234375f,0.0f,0.0f,0.048999994084239606451092186034657061100006103515625f,-0.64629998615831174557655458556837402284145355224609375f,0.0352000004753472151008963919593952596187591552734375f,1.8628999667897818692807732077199034392833709716796875f,0.0f,0.0f,0.3850999580115079812259182290290482342243194580078125f,0.143499975331127682665055544930510222911834716796875f,0.8802999591574065529897552551119588315486907958984375f,-0.354199989046156016314625958329997956752777099609375f,0.0f,0.0f,0.7151000110223881822690827902988530695438385009765625f,-0.3551999958559866055196607703692279756069183349609375f,0.339999990984797051396526512689888477325439453125f,0.18480003789514454215492378352792002260684967041015625f,0.0f,0.0f,0.3108000382229685332191593261086381971836090087890625f,-0.99450000538676963657280793995596468448638916015625f,0.43259996209070028516663342088577337563037872314453125f,-0.6866999631553891703106273780576884746551513671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8812999558717014014774804309126920998096466064453125f,-3.7453999860122788589933406910859048366546630859375f,0.2023999946586789722147159409360028803348541259765625f,0.09900003532320322818804925191216170787811279296875f,0.0f,0.0f,0.07190003210157325241880243993364274501800537109375f,1.275000003911554102842274005524814128875732421875f,-1.2471000566795458208702029878622852265834808349609375f,2.3501000510692602318840727093629539012908935546875f,0.0f,0.0f,-1.652600051926076840658197397715412080287933349609375f,-1.1930999752186230988826309840078465640544891357421875f,1.5764000089898697609669397934339940547943115234375f,-0.265100009457768237552954815328121185302734375f,0.0f,0.0f,0.944499936282635843554089660756289958953857421875f,-5.12929996095299767233655074960552155971527099609375f,-2.6778999392040052640595604316331446170806884765625f,-1.558900028547644200926924895611591637134552001953125f,0.0f,0.0f,-4.73379999234229398297202351386658847332000732421875f,0.363099956014751423793995854794047772884368896484375f,0.26790004155635804838908597957924939692020416259765625f,0.3543999825924657454123689603875391185283660888671875f,0.0f,0.0f,1.06810006907731303016362289781682193279266357421875f,-1.916700033265351432731904424144886434078216552734375f,4.4225999508962043904602978727780282497406005859375f,-1.08120001145750332938177962205372750759124755859375f,0.0f,0.0f,0.7231000313028699277850819271407090127468109130859375f,-2.70580003012046166332993379910476505756378173828125f,-1.467499886080625426387769039138220250606536865234375f,-1.023800038237869802770774185773916542530059814453125f,0.0f,0.0f,-2.25190006689578314791333468747325241565704345703125f,-0.965499991931020762336856932961381971836090087890625f,-0.195799966883658527194711496122181415557861328125f,5.16350002364814297806105969357304275035858154296875f,0.0f,0.0f,-0.628100070455670955738014527014456689357757568359375f,0.97289990770667600372689776122570037841796875f,-0.104699981386959795059965472319163382053375244140625f,1.5851998332098122812539031656342558562755584716796875f,0.0f,0.0f,-1.4797999152958396518187100809882394969463348388671875f,-2.40330005692392578708904693485237658023834228515625f,0.2414000929579132037616773232002742588520050048828125f,-1.491499964058398131072635806049220263957977294921875f,0.0f,0.0f,-0.8234000308796767253483039894490502774715423583984375f,0.1984000704608848142385113533237017691135406494140625f,-5.00000000268220912147398848901502788066864013671875f,2.97000010151415860804036128683947026729583740234375f,0.0f,0.0f,0.238700008663537577291435809456743299961090087890625f,-1.791999934419989504164050231338478624820709228515625f,0.964699902205915282848991409991867840290069580078125f,-0.0265999214552319340754138465854339301586151123046875f,0.0f,0.0f,-2.368599985794725082399736493243835866451263427734375f,1.941500063430517730722613123361952602863311767578125f,4.116200115393102265670677297748625278472900390625f,-3.891899868896602665557793443440459668636322021484375f,0.0f,0.0f,-1.0185000024698662723920961070689372718334197998046875f,0.4994999181777235452983632058021612465381622314453125f,-2.052400051450728568624981562606990337371826171875f,-2.922199954117834952427301686839200556278228759765625f,0.0f,0.0f,3.34170003089979417154609109275043010711669921875f,1.047700004331023393433497403748333454132080078125f,0.886199916741998716673833769164048135280609130859375f,1.4293000690981749745134266049717552959918975830078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.64679995938837464564130641520023345947265625f,0.062300011706352353968441093456931412220001220703125f,0.6616000119239089372058515436947345733642578125f,-0.0490999793276192253443923618760891258716583251953125f,0.0f,0.0f,-1.4641000324502584373220770430634729564189910888671875f,1.0942000516310337321357337714289315044879913330078125f,1.9733999975755811551181295726564712822437286376953125f,-0.9742999574512245519741782118217088282108306884765625f,0.0f,0.0f,1.225800000932812050535858361399732530117034912109375f,-0.81270000449865975600260981082101352512836456298828125f,0.1638000137627122487771202941075898706912994384765625f,0.4486999883607027950205292654572986066341400146484375f,0.0f,0.0f,-2.3811000233456471875115312286652624607086181640625f,0.693700006391107404368767674895934760570526123046875f,1.11819996696263590507669505313970148563385009765625f,-2.236199963848292782131466083228588104248046875f,0.0f,0.0f,1.94669998837560331850227157701738178730010986328125f,-0.686399975983797983047907109721563756465911865234375f,0.7611000438347452767828826836193911731243133544921875f,-0.0417999767109760167471677050343714654445648193359375f,0.0f,0.0f,-0.02690000293254957153266104796784929931163787841796875f,-0.8396000624164929870829610081273131072521209716796875f,-1.4604000282570712698770876158960163593292236328125f,0.5967000004827978099086749352863989770412445068359375f,0.0f,0.0f,0.83279988349676159486989490687847137451171875f,-0.19809993565604155918435935745947062969207763671875f,-2.02079997902810504228909849189221858978271484375f,0.4637000031501063990191369157400913536548614501953125f,0.0f,0.0f,1.31199997811764479394014415447600185871124267578125f,-0.253700008551775990639498559175990521907806396484375f,0.04339998852312454413748810111428610980510711669921875f,0.42830001922398697189464655821211636066436767578125f,0.0f,0.0f,0.0596000308632846920176007188274525105953216552734375f,-1.9773999832406639054482866413309238851070404052734375f,-0.6516000090926887811093592972611077129840850830078125f,-0.53979996473044167260013637132942676544189453125f,0.0f,0.0f,0.423799993012847320272840079269371926784515380859375f,1.2756000383809193987616481535951606929302215576171875f,-0.04530004204511561827217747122631408274173736572265625f,0.0295999572142953137898757631774060428142547607421875f,0.0f,0.0f,1.2607001025259492177355014064232818782329559326171875f,-1.015999978274106485542915834230370819568634033203125f,1.735899999065696253097712542512454092502593994140625f,-0.733800039169192341859115913393907248973846435546875f,0.0f,0.0f,-0.8783000501811490945414107045507989823818206787109375f,0.19290006007105120477973514425684697926044464111328125f,0.80569996685087719168194553276407532393932342529296875f,-0.12019989419281529308136668987572193145751953125f,0.0f,0.0f,-0.050399956135452672612018432118929922580718994140625f,0.21589997403174676637860329719842411577701568603515625f,1.1563000088825823308269491462851874530315399169921875f,0.68690003448575753797200604822137393057346343994140625f,0.0f,0.0f,-0.6612000352248539325472620475920848548412322998046875f,-0.355700022357701595154821916366927325725555419921875f,0.613800007914007128562161597074009478092193603515625f,-1.4965000221356745679912592095206491649150848388671875f,0.0f,0.0f,-1.0466000043988221701596330603933893144130706787109375f,-0.178599984304607328766678620013408362865447998046875f,0.66170000340715084963250092187081463634967803955078125f,-1.2990000339820986230421340223983861505985260009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,3.13790004945397438262943978770636022090911865234375f,-0.805399962691961679439600629848428070545196533203125f,3.139899987934529068667188766994513571262359619140625f,0.770600032979249949249833662179298698902130126953125f,0.0f,0.0f,-0.563999991923570309637625541654415428638458251953125f,0.40689999842494695148076289115124382078647613525390625f,-1.12989996193200337160078561282716691493988037109375f,-0.1220999210044750871162477778852917253971099853515625f,0.0f,0.0f,-0.5170000755339863030002334198798052966594696044921875f,-1.17450005981326111026419312111102044582366943359375f,-1.4613000108197322646219618036411702632904052734375f,0.4209000022321938150327014227514155209064483642578125f,0.0f,0.0f,0.3614000687435263703406462809653021395206451416015625f,-3.085900014004109959131483265082351863384246826171875f,2.24209999569654439000032652984373271465301513671875f,-0.946600014159081570852549702976830303668975830078125f,0.0f,0.0f,-1.4202999776721003666324349978822283446788787841796875f,3.55969993655309036029166236403398215770721435546875f,2.26739991316795386211424556677229702472686767578125f,0.66150001408904690602952314293361268937587738037109375f,0.0f,0.0f,0.9733999215424062878554423150490038096904754638671875f,2.99380011423379155388602157472632825374603271484375f,2.814199986363945971135080981184728443622589111328125f,-1.6168000291258095568736052882741205394268035888671875f,0.0f,0.0f,-3.18869990092813804238858210737816989421844482421875f,-0.23340001702159851593165740268887020647525787353515625f,1.2029000842481867294964104075916111469268798828125f,0.11809991876185021997258672854513861238956451416015625f,0.0f,0.0f,1.7635999616175883630830867332406342029571533203125f,1.3321999491259453041180904619977809488773345947265625f,1.432699951003491900536346292938105762004852294921875f,1.16960006249099901509680421440862119197845458984375f,0.0f,0.0f,0.4633999036982661134942418357240967452526092529296875f,0.679499965526164206863768413313664495944976806640625f,-1.393199945765733804847741339472122490406036376953125f,-0.18980003301501380263971441308967769145965576171875f,0.0f,0.0f,-0.19560003134012049486045725643634796142578125f,-0.306900046034158269225855519835022278130054473876953125f,0.55789998821020247987689799629151821136474609375f,2.8930999839544302432159383897669613361358642578125f,0.0f,0.0f,0.0838000021770592962155888017150573432445526123046875f,1.6322999725721769603836719397804699838161468505859375f,-5.20350005508959423394799159723334014415740966796875f,3.121199967871607139358047788846306502819061279296875f,0.0f,0.0f,0.7490001831799750764417922255233861505985260009765625f,0.61700008151307639270299887357396073639392852783203125f,1.0494999473467470085807917712372727692127227783203125f,-3.213899864634871761381873511709272861480712890625f,0.0f,0.0f,1.8700999527588493265284341759979724884033203125f,0.3143000524088730429639326757751405239105224609375f,0.65360003686845402626204304397106170654296875f,-1.555700060914456006599948523216880857944488525390625f,0.0f,0.0f,-1.1657001191593707734028839695383794605731964111328125f,1.876900017498434269924700856790877878665924072265625f,3.42540002561211576193045402760617434978485107421875f,-1.0577000131413323291695860461913980543613433837890625f,0.0f,0.0f,0.0249999688938284947425927384756505489349365234375f,1.9784000060133644094406690783216618001461029052734375f,4.6860000418461851268148166127502918243408203125f,0.649199995211512170811829491867683827877044677734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.658799993646145054526641615666449069976806640625f,-1.7985999709308144733910239665419794619083404541015625f,-0.634300017428397922003568965010344982147216796875f,0.113299978208541762825234400224871933460235595703125f,0.0f,0.0f,1.08990004787445027290004873066209256649017333984375f,-0.1112000360071652149684950927621684968471527099609375f,-1.5609999662041669221679285328718833625316619873046875f,1.76649997597932806314702247618697583675384521484375f,0.0f,0.0f,-1.95480007645189690634879298158921301364898681640625f,-0.642199944916368448133425772539339959621429443359375f,0.262399961298704464951470072264783084392547607421875f,-0.904799993601441254753581233671866357326507568359375f,0.0f,0.0f,-0.149799953591824408505317478557117283344268798828125f,-1.519899948539584766393772952142171561717987060546875f,0.08160008531212792348696893895976245403289794921875f,0.7258000249981879647975802072323858737945556640625f,0.0f,0.0f,-2.48789994715750228948536459938623011112213134765625f,-0.205800020043551246118340714019723236560821533203125f,-0.769400065666437260603061076835729181766510009765625f,0.595400017416477300713495424133725464344024658203125f,0.0f,0.0f,1.35980000321268956753328893682919442653656005859375f,1.040000064745544960942424950189888477325439453125f,-1.0049999527633206586330061327316798269748687744140625f,1.065799903739989229478624110925011336803436279296875f,0.0f,0.0f,-0.63839992079734830809911727556027472019195556640625f,-0.25590000180378502658840034200693480670452117919921875f,1.5850999950408943828250585283967666327953338623046875f,0.1630999665200698611755569800152443349361419677734375f,0.0f,0.0f,-0.630299930584431233882014566916041076183319091796875f,0.49599998974800119100336814881302416324615478515625f,-1.5513999837696559325195266865193843841552734375f,-0.1527000240564344579752287245355546474456787109375f,0.0f,0.0f,-0.7558999790608884605802586520439945161342620849609375f,0.0815000196397315246343850958510302007198333740234375f,1.02529997722506660551289314753375947475433349609375f,0.463400005584954755022408789955079555511474609375f,0.0f,0.0f,2.258799967792631147034398964024148881435394287109375f,-3.13919996859431194025091826915740966796875f,0.3443000362038615147497466750792227685451507568359375f,-0.956799954098464855434258424793370068073272705078125f,0.0f,0.0f,0.20729997326135585211659417836926877498626708984375f,0.21840001243948936604510890902020037174224853515625f,-0.811599978470802785324167416547425091266632080078125f,-0.68559994536042179191781542613171041011810302734375f,0.0f,0.0f,0.464200059193371483701184843084774911403656005859375f,0.42959996149838008960841762018389999866485595703125f,-0.11049993918091105538081819759099744260311126708984375f,-1.134000021725893869728452045819722115993499755859375f,0.0f,0.0f,-1.278600019359587225409313759882934391498565673828125f,-0.6630999808251860638819152882206253707408905029296875f,0.5884999114423994370781656471081078052520751953125f,0.629200033265350899824852604069747030735015869140625f,0.0f,0.0f,-1.092799998533725780447412034845910966396331787109375f,-2.794299917143583389389505100552923977375030517578125f,-0.0665000342428692103879939168109558522701263427734375f,1.4381000251561399938537988418829627335071563720703125f,0.0f,0.0f,0.5968999640405174833546197987743653357028961181640625f,-0.90370002468228260994465017574839293956756591796875f,-1.973899914386123555232188664376735687255859375f,3.036399981199204400894586797221563756465911865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6041999601751568693686067490489222109317779541015625f,-0.2254999979659917386243250803090631961822509765625f,0.358699992570280290493656139005906879901885986328125f,0.40499998129904157817549048559158109128475189208984375f,0.0f,0.0f,2.29420002268552725155359439668245613574981689453125f,1.375699960257113385608818134642206132411956787109375f,-2.533800089088082518884448290918953716754913330078125f,-0.8059999998807898524688653196790255606174468994140625f,0.0f,0.0f,-1.2871000339180234117719692221726290881633758544921875f,-1.069500039584934558689610639703460037708282470703125f,-1.059899971692264131206684396602213382720947265625f,-0.19059994275271918695580097846686840057373046875f,0.0f,0.0f,1.1649000679723915396834854618646204471588134765625f,-2.57130000240057743354782360256649553775787353515625f,0.891000010646879392339769765385426580905914306640625f,-0.45050002156570678746305702588870190083980560302734375f,0.0f,0.0f,0.4169999609626824366159780765883624553680419921875f,0.78590013606324848893791568116284906864166259765625f,-0.18830007279366289818511859266436658799648284912109375f,-0.56339997740313396690225999918766319751739501953125f,0.0f,0.0f,-0.2834000889569525671873861938365735113620758056640625f,0.7073999954782419496979173345607705414295196533203125f,1.3465999971531330547946936349035240709781646728515625f,-0.40000000704079885505137781365192495286464691162109375f,0.0f,0.0f,1.0599999500811094765850839394261129200458526611328125f,-3.89510001102238856418580326135270297527313232421875f,1.7216000323757523204903918667696416378021240234375f,-1.808200072835384641933842431171797215938568115234375f,0.0f,0.0f,-0.5214001111373314234498366204206831753253936767578125f,-2.685999929100276073512532093445770442485809326171875f,4.04999993197619811979848236660473048686981201171875f,0.436500017143787832907264601089991629123687744140625f,0.0f,0.0f,-5.42330004314035107881863950751721858978271484375f,3.472500072382390445824285052367486059665679931640625f,-0.0727000765644010282784392984467558562755584716796875f,1.2760999413147560144210501675843261182308197021484375f,0.0f,0.0f,2.2191001169584705365878107841126620769500732421875f,-4.8542000979363901791430180310271680355072021484375f,-0.517199871360510154971734664286486804485321044921875f,-0.0070999613493665236774177174083888530731201171875f,0.0f,0.0f,-2.101800010126830375867257316713221371173858642578125f,0.49919992463588658271333997618057765066623687744140625f,-0.5224000212758788119771224955911748111248016357421875f,0.39229995854645871578014748592977412045001983642578125f,0.0f,0.0f,-2.3868000294983406917026513838209211826324462890625f,4.36919996695518619844733620993793010711669921875f,-3.52669999187737648327356509980745613574981689453125f,-2.079300019291043799540830150363035500049591064453125f,0.0f,0.0f,0.05149996521323962905825055713648907840251922607421875f,-1.82939988741129599958412654814310371875762939453125f,1.72829998469054846310655193519778549671173095703125f,-0.52039996466636739835820435473578982055187225341796875f,0.0f,0.0f,0.8107000848494478884020963960210792720317840576171875f,0.873499996490777252944326392025686800479888916015625f,-0.448600028542429585254325274945585988461971282958984375f,-2.544299987328052292667734946007840335369110107421875f,0.0f,0.0f,0.9024000018484896745718515376211144030094146728515625f,-1.96780002183541657956311610178090631961822509765625f,1.306999923840165056532214293838478624820709228515625f,-0.293600109828263011735316467820666730403900146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.378599951317160954289420260465703904628753662109375f,1.1886999834060674174907035194337368011474609375f,-0.355199979874491145181991669232957065105438232421875f,-0.001999956957995874251565737722557969391345977783203125f,0.0f,0.0f,0.4169999501407151143439477891661226749420166015625f,0.63920003727003937843420544595574028789997100830078125f,2.178300096672774355255342015880160033702850341796875f,0.43900003042444613043926437967456877231597900390625f,0.0f,0.0f,-3.576200043010711482338592759333550930023193359375f,1.54290001275986465856249196804128587245941162109375f,1.339900026565789659116489929147064685821533203125f,0.716499991476535313950080308131873607635498046875f,0.0f,0.0f,-0.50800001607090283783918494009412825107574462890625f,0.8323000538766383016309191589243710041046142578125f,1.1337000356346360607773249284946359694004058837890625f,1.7314000396303830253685873685753904283046722412109375f,0.0f,0.0f,-1.4344999900572001916287945277872495353221893310546875f,1.7381000120617446924597970792092382907867431640625f,0.808300050777197132134688217774964869022369384765625f,-0.93700004575028916775636389502324163913726806640625f,0.0f,0.0f,-0.6085999259293084495681114276521839201450347900390625f,-1.1125000005029141991741425954387523233890533447265625f,0.1193999983131874831343566256691701710224151611328125f,-3.55790001365393404597625703900121152400970458984375f,0.0f,0.0f,-4.3454999853372573426213421043939888477325439453125f,0.7939999487102031849872219027020037174224853515625f,0.250599988722801203522294599679298698902130126953125f,-2.706299941864610314468109208974055945873260498046875f,0.0f,0.0f,0.16280004780069090575267409803927876055240631103515625f,2.889699971738458383896386294509284198284149169921875f,1.4895999839246290008532014326192438602447509765625f,-1.040199983525276206108856058563105762004852294921875f,0.0f,0.0f,2.485200011613964310441815541707910597324371337890625f,-1.163499995596707581313467017025686800479888916015625f,0.981200030195712002978325472213327884674072265625f,0.231400046913325496689139981754124164581298828125f,0.0f,0.0f,-0.845399976643175232737803526106290519237518310546875f,-1.59620000456571542457595569430850446224212646484375f,-0.38579998282790217700721768778748810291290283203125f,-1.034200037661194837568245930015109479427337646484375f,0.0f,0.0f,-1.1592000489488245573710401004063896834850311279296875f,1.427400033718348115741036963299848139286041259765625f,1.00450008589029327055186513462103903293609619140625f,-1.734200081507861312957174959592521190643310546875f,0.0f,0.0f,0.40149998503178352393661043606698513031005859375f,0.35450006508454656728446252600406296551227569580078125f,-1.739599946373702632484992136596702039241790771484375f,0.7490000038445003838205593638122081756591796875f,0.0f,0.0f,1.45690000425502663716770257451571524143218994140625f,-1.821899999877810927273458219133317470550537109375f,-1.21260007580518713865558311226777732372283935546875f,-1.323099998818337841299808133044280111789703369140625f,0.0f,0.0f,0.19039998904839183779813538421876728534698486328125f,1.745500016257166553401702913106419146060943603515625f,0.70500002512708304092114985905936919152736663818359375f,-1.454299986992775739480521224322728812694549560546875f,0.0f,0.0f,1.350100016256421842086865581222809851169586181640625f,-0.96839993895813847313291944374213926494121551513671875f,1.9316999899402260698622058043838478624820709228515625f,0.41000002518296252418394942651502788066864013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2726999998792998258778652598266489803791046142578125f,-1.0262999148935076920707842873525805771350860595703125f,-0.7599999476969234724066382113960571587085723876953125f,-0.3539999728314573435028478343156166374683380126953125f,0.0f,0.0f,-1.4322000271707759910810864312225021421909332275390625f,-1.9809999343901874002682461650692857801914215087890625f,0.4415000342056156767966967890970408916473388671875f,-0.76300001711398390824570014956407248973846435546875f,0.0f,0.0f,0.873399965575337500212071972782723605632781982421875f,-1.76760001163706181870338696171529591083526611328125f,-0.24750003725290270484293841946055181324481964111328125f,0.63360000993460319218542053931741975247859954833984375f,0.0f,0.0f,-0.93879990695864068417364478591480292379856109619140625f,1.1310000237971540304471318449941463768482208251953125f,3.108299942073225796690394417964853346347808837890625f,-0.5708000243648883031966079215635545551776885986328125f,0.0f,0.0f,-3.6981999336585413828970558824948966503143310546875f,-3.181900016026945277047843774198554456233978271484375f,0.44029997979551549480703442895901389420032501220703125f,2.334099944663792225441056871204636991024017333984375f,0.0f,0.0f,-3.096099931740762034593217322253622114658355712890625f,-0.9298000176295639906953738318406976759433746337890625f,1.6357999481454494894450135689112357795238494873046875f,-0.6224000194504843985754405366606079041957855224609375f,0.0f,0.0f,1.99740004299432083456622422090731561183929443359375f,-0.87980003137588569384064385303645394742488861083984375f,-0.018300058469922220183434546925127506256103515625f,0.8616000065222382264806810781010426580905914306640625f,0.0f,0.0f,3.110900013389438445443602176965214312076568603515625f,-0.2921000499740245004431926645338535308837890625f,-0.364299980920552712859716848470270633697509765625f,2.155200040149688245350034776492975652217864990234375f,0.0f,0.0f,3.93490006273835835060026511200703680515289306640625f,-1.800399961388109648652289251913316547870635986328125f,-1.42489995017871251548058353364467620849609375f,0.2088999809011824471127738434006460011005401611328125f,0.0f,0.0f,2.8338000373810512400041261571459472179412841796875f,-0.44599989899993008890533019439317286014556884765625f,2.5915999344192446329770973534323275089263916015625f,-0.5244000331074001319819899435970000922679901123046875f,0.0f,0.0f,0.186999973796307283890882899868302047252655029296875f,2.66180001713037572841358269215561449527740478515625f,0.401999908454715448868910243618302047252655029296875f,-3.05990004671961113302813828340731561183929443359375f,0.0f,0.0f,2.294500029433518495380894819390960037708282470703125f,-0.28420002499520780059327762501197867095470428466796875f,1.7382999954193809610814014376956038177013397216796875f,2.900000035986304514068478965782560408115386962890625f,0.0f,0.0f,-0.0684000568449494306122460329788736999034881591796875f,-0.25310001658797265378098018118180334568023681640625f,-1.607000048637388545103021897375583648681640625f,1.1121000173404806954380319439223967492580413818359375f,0.0f,0.0f,2.168599932653457518227924083475954830646514892578125f,-1.0978000031530867897089365214924328029155731201171875f,-0.683700089316068737588238946045748889446258544921875f,2.13470002158284177795621872064657509326934814453125f,0.0f,0.0f,4.00309985658675504538450695690698921680450439453125f,-0.77949991357699122129787383528309874236583709716796875f,0.174200062713772307887438728357665240764617919921875f,1.5001000372484314393517479402362369000911712646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.477899986030906054335076760253286920487880706787109375f,-1.854600008809565991896306513808667659759521484375f,-0.5502999694019561349733749011647887527942657470703125f,-0.99299998572841363664309710657107643783092498779296875f,0.0f,0.0f,-1.4387999554991726114394623436965048313140869140625f,-0.1123000349894167637643249690881930291652679443359375f,0.728499976187943598660012867185287177562713623046875f,-0.79620000745281682696941061294637620449066162109375f,0.0f,0.0f,-1.198399929738045432969784087617881596088409423828125f,0.38949999415501890265289830495021305978298187255859375f,0.868199949491023748038287521922029554843902587890625f,-0.067800032806396348661337469820864498615264892578125f,0.0f,0.0f,-1.004599983906000915112599614076316356658935546875f,-0.40519994511753321120295368018560111522674560546875f,1.2956000022083522527083232489530928432941436767578125f,0.004699989843369445363663317039026878774166107177734375f,0.0f,0.0f,-0.1559999994151295421573877320042811334133148193359375f,-1.5748000113151963130775357058155350387096405029296875f,-0.70840002167820959044775008806027472019195556640625f,-2.44999995546415494374059562687762081623077392578125f,0.0f,0.0f,2.696300017786026348431960286688990890979766845703125f,-0.55760003981888239810160712295328266918659210205078125f,-2.505499953784048106086856932961381971836090087890625f,-1.19819998964965268584137447760440409183502197265625f,0.0f,0.0f,1.0415000636912885578766463368083350360393524169921875f,0.5729000056818127628588399602449499070644378662109375f,0.868999968528747945839540989254601299762725830078125f,-0.25690000904202470177750683433259837329387664794921875f,0.0f,0.0f,1.4960000544004137257303455044166184961795806884765625f,-2.1030999370530256697975346469320356845855712890625f,1.1716999777212731004283341462723910808563232421875f,-0.795999945081772519728247061721049249172210693359375f,0.0f,0.0f,-0.866800040636956925510503424447961151599884033203125f,-0.05340000733584149283927899887203238904476165771484375f,-0.377400052829086174455142099759541451930999755859375f,-0.78689998687654794107260158853023312985897064208984375f,0.0f,0.0f,1.3828999683916574081621320146950893104076385498046875f,2.029900008684396350844281187164597213268280029296875f,-0.275899989752470986559274024330079555511474609375f,0.51559994722306712322534849590738303959369659423828125f,0.0f,0.0f,1.3574999742209914455060015825438313186168670654296875f,-1.049199924263359395837369447690434753894805908203125f,-1.03269999321103167488899998716078698635101318359375f,0.841499998368323787190092843957245349884033203125f,0.0f,0.0f,0.447000052008779125145565558341331779956817626953125f,-0.79800000679492943955750661189085803925991058349609375f,-0.0258000480204814142126679143984802067279815673828125f,-2.004199934843182262511618318967521190643310546875f,0.0f,0.0f,0.9924999845400446218945944565348327159881591796875f,0.50869999290555723003848243024549447000026702880859375f,2.07049989835172976171406844514422118663787841796875f,-1.5134999638944857114353226279490627348423004150390625f,0.0f,0.0f,-0.24299996892735398201779162263846956193447113037109375f,0.61299997308105247384446556679904460906982421875f,0.124099892863631655615108684287406504154205322265625f,1.07520000859647968383114857715554535388946533203125f,0.0f,0.0f,0.24180000303015081186686074943281710147857666015625f,1.4592999123565852936934561512316577136516571044921875f,1.85699995679035811946278045070357620716094970703125f,0.7728000127643339656202670084894634783267974853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.18180000126063722287739210514700971543788909912109375f,0.573899988128244498142294105491600930690765380859375f,0.1316999782800680751648769728490151464939117431640625f,2.3300999540627014283700191299431025981903076171875f,0.0f,0.0f,1.95409998938590323547259686165489256381988525390625f,0.73880004546493338590806843058089725673198699951171875f,-0.885599977658686920989339341758750379085540771484375f,0.22990001050979091967718659361707977950572967529296875f,0.0f,0.0f,0.614899920991063719810654220054857432842254638671875f,0.05040000821501078309694321433198638260364532470703125f,-1.9022999927446255785667972304509021341800689697265625f,0.9769000432029353309104635627591051161289215087890625f,0.0f,0.0f,3.284199989716707346332214001449756324291229248046875f,-2.08039989936202829312605899758636951446533203125f,-1.6054999643266203435132410959340631961822509765625f,0.691999828062953792340294967289082705974578857421875f,0.0f,0.0f,3.159599927318841761092471642768941819667816162109375f,-2.01130008426383266595394161413423717021942138671875f,-4.26539997368156686974316471605561673641204833984375f,-0.25999993272125776488934434382827021181583404541015625f,0.0f,0.0f,-3.461300124366581076884585854713805019855499267578125f,-0.21049995758384298927268218903918750584125518798828125f,-2.383900062224270310906604208867065608501434326171875f,5.38389993884265294354918296448886394500732421875f,0.0f,0.0f,1.56869995319396249300325507647357881069183349609375f,0.63090000524967937423070907243527472019195556640625f,0.85019983592927506066416754038073122501373291015625f,2.5465000463873170843953630537725985050201416015625f,0.0f,0.0f,-1.331000108286736871576749763335101306438446044921875f,-2.56400007659942019699883530847728252410888671875f,-0.0492999363660791001251482157385908067226409912109375f,1.40680002059489428489769125008024275302886962890625f,0.0f,0.0f,-3.820699945728481594642289564944803714752197265625f,1.2601999535366896765253841294907033443450927734375f,1.14999980950727920259168968186713755130767822265625f,2.25399997270107466107447180547751486301422119140625f,0.0f,0.0f,0.999699809390306892709077146719209849834442138671875f,-0.07640001979321209280016091724974103271961212158203125f,-1.3311000495716911462551479417015798389911651611328125f,2.57850006045401158871754887513816356658935546875f,0.0f,0.0f,2.051800025661290050038587651215493679046630859375f,-0.0073000687405448960731746410601772367954254150390625f,2.126800080832838890643188278772868216037750244140625f,3.5847999450609098204267866094596683979034423828125f,0.0f,0.0f,-3.690499993532895484094069615821354091167449951171875f,3.546799863946437536554867619997821748256683349609375f,-2.445900022833048748083228929317556321620941162109375f,-1.361100009040531855220024226582609117031097412109375f,0.0f,0.0f,-0.372799913112821013072561981971375644207000732421875f,1.2179000502735386479713497465127147734165191650390625f,-0.43939992395639304589849416515789926052093505859375f,-0.0866000460848213027276187858660705387592315673828125f,0.0f,0.0f,0.21939988323897097188819316215813159942626953125f,-0.2346000684887161735758809300023131072521209716796875f,1.4146000675201417795534553079050965607166290283203125f,-1.0215001017823805984363616516930051147937774658203125f,0.0f,0.0f,-2.548300038465114436547764853457920253276824951171875f,0.42150009110942521228793111731647513806819915771484375f,-4.52510006699487465908759986632503569126129150390625f,-0.0610999479085208019313313343445770442485809326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.14799997524172081586613103354466147720813751220703125f,1.177599993811548007016654082690365612506866455078125f,-0.366500005893409142032624004059471189975738525390625f,-0.185800008998066257248638066812418401241302490234375f,0.0f,0.0f,0.9482999800339342311161772158811800181865692138671875f,-0.016199998661130532440211027278564870357513427734375f,-0.0821000033706429377389213186688721179962158203125f,0.9357000317826866808701424815808422863483428955078125f,0.0f,0.0f,0.39390000351369336417661770610720850527286529541015625f,0.815199992596358224972163952770642936229705810546875f,-0.17029999722987432253518136349157430231571197509765625f,-0.3362999660044907468403607708751223981380462646484375f,0.0f,0.0f,0.79859996169209512828501829062588512897491455078125f,-1.606499985963105725517152677639387547969818115234375f,-0.12589997477680447968140242664958350360393524169921875f,0.529500007323920218738066978403367102146148681640625f,0.0f,0.0f,-0.281300004877895304389312514103949069976806640625f,1.124899982365220818536499791662208735942840576171875f,0.17310000500231870201872652614838443696498870849609375f,0.77189998845979446162601789183099754154682159423828125f,0.0f,0.0f,-0.859599921898543950504745225771330296993255615234375f,0.891599936002493098641252800007350742816925048828125f,0.7898999640926718246447535420884378254413604736328125f,-0.66619998780190936571443671709857881069183349609375f,0.0f,0.0f,-1.5905999744176864130196236146730370819568634033203125f,-0.240799999144673648743264493532478809356689453125f,0.9316999389037494694321139832027256488800048828125f,-0.4464999804496769808537237622658722102642059326171875f,0.0f,0.0f,-0.51810001757517498699456837130128405988216400146484375f,2.13419995760470637691241790889762341976165771484375f,0.333999969422816622710570300114341080188751220703125f,-1.5100999891176822842453475459478795528411865234375f,0.0f,0.0f,2.02919995056390689569525420665740966796875f,0.1966999845013024383888478041626513004302978515625f,1.9965000155791639002700321725569665431976318359375f,-0.4200999687403437210519996369839645922183990478515625f,0.0f,0.0f,-0.2178999921590090149692287013749592006206512451171875f,-0.8574999925494191099772933739586733281612396240234375f,-0.873599978306888846901756551233120262622833251953125f,0.33810007917284845557759354051086120307445526123046875f,0.0f,0.0f,-0.07169998504146934958924930469947867095470428466796875f,1.1288999544516202799826487535028718411922454833984375f,1.08029994485229252632052521221339702606201171875f,0.6887999606028210042296677784179337322711944580078125f,0.0f,0.0f,-0.06229999471902747387730414629913866519927978515625f,0.202599986975640522501862506032921373844146728515625f,-1.5318998697161678723688282843795605003833770751953125f,0.7559000225350243074018408151459880173206329345703125f,0.0f,0.0f,-0.685099999418109728566150806727819144725799560546875f,-1.277099964925646258251390463556163012981414794921875f,-1.285699975791573024253011681139469146728515625f,-0.32500006109476042848172028243425302207469940185546875f,0.0f,0.0f,0.9756999655470253163258576023508794605731964111328125f,2.41619999301731613883248428464867174625396728515625f,0.578299982734769013603681742097251117229461669921875f,-0.2322999736711384155540827123331837356090545654296875f,0.0f,0.0f,1.2740000522732730647845755811431445181369781494140625f,-2.343099940722435992057626208406873047351837158203125f,-1.5836999513126908123439307018998079001903533935546875f,-1.125999973133207543440903464215807616710662841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.70239993093758734676868016322259791195392608642578125f,1.0557000497385871096156506609986536204814910888671875f,1.5403000367924573055233850027434527873992919921875f,0.50190002673715372605300899522262625396251678466796875f,0.0f,0.0f,-2.06989998800903496345426901825703680515289306640625f,-1.181800012306124525451878071180544793605804443359375f,0.6985000385493032037942384704365395009517669677734375f,0.96440002036690530839990742606460116803646087646484375f,0.0f,0.0f,1.7180999786645159144171657317201606929302215576171875f,-1.696300004598498833985331657459028065204620361328125f,-0.3559000821769247036030492381541989743709564208984375f,2.635399983069298901483534791623242199420928955078125f,0.0f,0.0f,-1.8879000280149274004770632018335163593292236328125f,1.083999964803456617090660074609331786632537841796875f,-4.30030003008693473276480290223844349384307861328125f,-2.86040001058057047345073442556895315647125244140625f,0.0f,0.0f,-1.111799979002029648000871020485647022724151611328125f,-2.332700073695928466577242943458259105682373046875f,-2.890199984270334088165554931038059294223785400390625f,0.8426999668359755890634232855518348515033721923828125f,0.0f,0.0f,0.702299929712712600604618273791857063770294189453125f,-1.6414999834485353158441967025282792747020721435546875f,-0.81599993707239804052733234129846096038818359375f,0.046100263813135311519175729699782095849514007568359375f,0.0f,0.0f,0.37100004260986907400621248598326928913593292236328125f,1.22919998878538638820145934005267918109893798828125f,-2.378500046297908010473065587575547397136688232421875f,0.5995999774426223893897258676588535308837890625f,0.0f,0.0f,-2.4686998757079248179024943965487182140350341796875f,-0.0071000247910630864822678631753660738468170166015625f,-1.383799996626375428121491495403461158275604248046875f,2.46220011981874886686227910104207694530487060546875f,0.0f,0.0f,1.8971000442370762328891942161135375499725341796875f,2.307599979059397288239097179030068218708038330078125f,0.670799957365544319287664620787836611270904541015625f,-0.8144000049442039479430377468816004693508148193359375f,0.0f,0.0f,-0.9818000990681350348410205697291530668735504150390625f,0.5018000596731908569125835128943435847759246826171875f,-0.328500005152076113290604553185403347015380859375f,-0.399200002047419122419569248449988663196563720703125f,0.0f,0.0f,1.140299959790705752737949296715669333934783935546875f,-1.26229995824843665985781626659445464611053466796875f,1.513400024360419759972273823223076760768890380859375f,-1.9288000470295540988985294461599551141262054443359375f,0.0f,0.0f,-0.840500055521725908391772463801316916942596435546875f,-1.45349997721239976300466878456063568592071533203125f,1.2754998605400327260639414816978387534618377685546875f,1.8155999350413676296511766850017011165618896484375f,0.0f,0.0f,2.41210001687481945253921367111615836620330810546875f,1.703900093442202834381760112592019140720367431640625f,1.453900100408494378001478253281675279140472412109375f,1.0033000030189744844477672813809476792812347412109375f,0.0f,0.0f,-0.07839996632039702983973938898998312652111053466796875f,-4.2285000459998851596310487366281449794769287109375f,-0.26300004008039767899873595524695701897144317626953125f,3.235099954919517184492860906175337731838226318359375f,0.0f,0.0f,0.4367999622941012294319307329715229570865631103515625f,2.85120002045407883173311347491107881069183349609375f,-0.05499990426004064403286974993534386157989501953125f,2.2467999431833636236888196435756981372833251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0561999674804509741221636431873776018619537353515625f,-0.867499966621398499455608543939888477325439453125f,1.589100009092689003153964222292415797710418701171875f,0.2105999532021585185503909087856300175189971923828125f,0.0f,0.0f,-1.16689995480179842246570842689834535121917724609375f,-0.3572000136107202905577651108615100383758544921875f,-1.3691000556588168723948228944209404289722442626953125f,-0.5794000801391894217573508285568095743656158447265625f,0.0f,0.0f,2.221000016421080136552745898370631039142608642578125f,-1.8187999993644659024738530206377618014812469482421875f,-1.68870005448460513974850982776843011379241943359375f,-0.28390001521110530546820882591418921947479248046875f,0.0f,0.0f,0.3907999954566354627871760385460220277309417724609375f,1.214599919849634890312017887481488287448883056640625f,-0.425500031903386144449541461654007434844970703125f,-1.813700007769466271412284186226315796375274658203125f,0.0f,0.0f,0.527200042854249151247358895489014685153961181640625f,-2.610399982547759289985833675018511712551116943359375f,0.213499991238116582081829619710333645343780517578125f,0.18789994147643496180677402662695385515689849853515625f,0.0f,0.0f,-1.59419999251067689982619413058273494243621826171875f,0.54310005235075919927112408913671970367431640625f,1.84470003126859705133711031521670520305633544921875f,2.134100007677078192358521846472285687923431396484375f,0.0f,0.0f,5.12199995047598921615872313850559294223785400390625f,-0.37929998449683122174747040844522416591644287109375f,-1.930399972712993683643389886128716170787811279296875f,3.165899945682286276138484026887454092502593994140625f,0.0f,0.0f,1.53890000794678893925038209999911487102508544921875f,-1.33860003202557553692031433456577360630035400390625f,-1.50109993069767977402761971461586654186248779296875f,2.087699990528822713287127044168300926685333251953125f,0.0f,0.0f,-0.4810999929845329869948500345344655215740203857421875f,-0.902299956609308395627522259019315242767333984375f,-3.9190999948620781623276343452744185924530029296875f,-1.7144999985694890920484567686798982322216033935546875f,0.0f,0.0f,-1.08089998490959349197737537906505167484283447265625f,2.3483999592423447921873957966454327106475830078125f,1.811400054699181083606163156218826770782470703125f,-0.093400034679472554444146226160228252410888671875f,0.0f,0.0f,1.7074999995529649421399653874686919152736663818359375f,0.568599997901916509590591886080801486968994140625f,-0.39980006028413672680699164629913866519927978515625f,-0.262099899546803793981553099001757800579071044921875f,0.0f,0.0f,1.5144000241667026074310342664830386638641357421875f,-1.7653999686896799659052703646011650562286376953125f,1.0903998932898044671446768916212022304534912109375f,1.379099986219404883058814448304474353790283203125f,0.0f,0.0f,1.13949994628503947780018279445357620716094970703125f,3.1489999727755790814853753545321524143218994140625f,-0.527499945908785239367944086552597582340240478515625f,0.958900097018479780075495000346563756465911865234375f,0.0f,0.0f,-1.5261999391496174904858662557671777904033660888671875f,-1.57690002722144040347984628169797360897064208984375f,-1.507599970975518122173752999515272676944732666015625f,-0.6239999309405692695662537516909651458263397216796875f,0.0f,0.0f,-1.651800035924462672909385219099931418895721435546875f,1.439699945586920382112339211744256317615509033203125f,-1.3461999917693432404774966926197521388530731201171875f,-0.65289999521374664226414097356610000133514404296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.48580002786666132141846219383296556770801544189453125f,1.67220004816278855486189058865420520305633544921875f,-2.4084999846816064206223018118180334568023681640625f,1.1543000375822189962349284542142413556575775146484375f,0.0f,0.0f,-1.90100004812330070080861332826316356658935546875f,-0.647200050174445262740619000396691262722015380859375f,0.123700054559112260221809265203773975372314453125f,-1.2099999991804357701852268291986547410488128662109375f,0.0f,0.0f,-0.5995001098439105557957873315899632871150970458984375f,-3.31519998611435173785366714582778513431549072265625f,-1.8364999101534487113696059168432839214801788330078125f,-1.5252999717861401851592972889193333685398101806640625f,0.0f,0.0f,0.8169001349754625440624522525467909872531890869140625f,-0.7518000191234042173249463303363882005214691162109375f,-2.016000038176773756504189805127680301666259765625f,-0.2431001010030506837011898824130184948444366455078125f,0.0f,0.0f,0.639200053605438966997098759748041629791259765625f,-2.20359999503344550220163000631146132946014404296875f,0.76599989236891286026320813107304275035858154296875f,-4.56210001339167359191151263075880706310272216796875f,0.0f,0.0f,-1.1374000750690702599143833140260539948940277099609375f,-3.661699992454795715701720837387256324291229248046875f,0.61680001146793450583771800665999762713909149169921875f,-3.000399945704639304011607237043790519237518310546875f,0.0f,0.0f,-2.41590000744760136086597412941046059131622314453125f,-0.698300090898573699860207852907478809356689453125f,-1.6427999417044230057172171655111014842987060546875f,0.879700022923945113006993778981268405914306640625f,0.0f,0.0f,0.50420005904436104859911438325070776045322418212890625f,-4.08550017702207046710327631444670259952545166015625f,-5.01849987694621102463088391232304275035858154296875f,0.3745999534517527873589415321475826203823089599609375f,0.0f,0.0f,-1.49009990090280819430290648597292602062225341796875f,2.11830006076097543399328060331754386425018310546875f,2.02540002069473334955773680121637880802154541015625f,-1.964200044701993164153464022092521190643310546875f,0.0f,0.0f,-0.5383000076755857232768676112755201756954193115234375f,-1.1229000588044522057629137634648941457271575927734375f,-2.688799894553423275311843099188990890979766845703125f,0.652800002892316211244860824081115424633026123046875f,0.0f,0.0f,2.235999987959861545760986700770445168018341064453125f,-2.0016999248966573787811285001225769519805908203125f,-1.1048999595478183888275225399411283433437347412109375f,-1.4809000290170306612935746670700609683990478515625f,0.0f,0.0f,1.123799976919591880886173385079018771648406982421875f,1.535000079032033948323032745975069701671600341796875f,-0.4299999583512554846009834363940171897411346435546875f,0.5898000268682836999545315848081372678279876708984375f,0.0f,0.0f,-0.3657000124484304226513131652609445154666900634765625f,-1.4701000221237550125152893087943084537982940673828125f,1.742000003859402124106736664543859660625457763671875f,-0.7157000993594540005915405345149338245391845703125f,0.0f,0.0f,-1.2038000056415796290565367598901502788066864013671875f,0.369099946880340812338516798263299278914928436279296875f,-1.324899909461290103962483044597320258617401123046875f,-0.7107000178687286062739758563111536204814910888671875f,0.0f,0.0f,4.201400035774707220070922630839049816131591796875f,-3.868099964955448388792547120829112827777862548828125f,-0.705700069091469206483679954544641077518463134765625f,3.5461999837785942446544140693731606006622314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.475400048708916500572740915231406688690185546875f,-0.30750005748122877236028216429986059665679931640625f,1.59070000939369204928652834496460855007171630859375f,-0.1045999958455556821945720002986490726470947265625f,0.0f,0.0f,-1.025699987936018686696115764789283275604248046875f,-2.936100009673834332346586961648426949977874755859375f,0.0520000347793097006388052250258624553680419921875f,0.9390000567436214140570882591418921947479248046875f,0.0f,0.0f,3.69489998886585180315478282864205539226531982421875f,1.082600022235512771118237651535309851169586181640625f,-1.9945000522136684395491101895458996295928955078125f,0.031199967610834988818169222213327884674072265625f,0.0f,0.0f,-0.7677000151343642553314339238568209111690521240234375f,-1.345400009630619564404696575365960597991943359375f,-0.72329999521449206412171406554989516735076904296875f,-0.27289997286200406989564726245589554309844970703125f,0.0f,0.0f,-3.176199975061416491683985441341064870357513427734375f,-1.725299976964294490500151368905790150165557861328125f,0.0192000933915377203220486990176141262054443359375f,0.1857000257849696911449655090109445154666900634765625f,0.0f,0.0f,3.69419998289942608238334287307225167751312255859375f,0.6502000500217082201714902112144045531749725341796875f,-1.4600000981986511749965984563459642231464385986328125f,0.981899971044064567848863589460961520671844482421875f,0.0f,0.0f,-2.07320000233426693370120119652710855007171630859375f,-0.38619996306598114443886515800841152667999267578125f,-0.194400079245121037985200018738396465778350830078125f,0.8884000477179887500511767939315177500247955322265625f,0.0f,0.0f,-0.93649995005130737268927987315692007541656494140625f,3.249800067902356648374961878289468586444854736328125f,1.80870000747144299424462587921880185604095458984375f,-1.098099890449643734058327027014456689357757568359375f,0.0f,0.0f,3.66869998221397342064165059127844870090484619140625f,1.51400005037337503921435200027190148830413818359375f,-1.385399961760640508146025240421295166015625f,-0.4201000667154790590984703158028423786163330078125f,0.0f,0.0f,-0.77770006324648921491871078615076839923858642578125f,3.0177000739380712701631637173704802989959716796875f,-2.34560006628334516420864019892178475856781005859375f,-0.8619000303134323193177124267094768583774566650390625f,0.0f,0.0f,1.108400000220537862105629756115376949310302734375f,1.29469997950568771472035223268903791904449462890625f,0.2103998950779430998636598815210163593292236328125f,-1.109199965558201217419309614342637360095977783203125f,0.0f,0.0f,2.16089997071623773905457710498012602329254150390625f,0.223600010176748131840440692030824720859527587890625f,-2.294099998289345965218899436877109110355377197265625f,1.434799981252104839057892604614607989788055419921875f,0.0f,0.0f,-2.045199989783764049633418835583142936229705810546875f,3.038900038643180590014480912941507995128631591796875f,-0.08400002701580522312951870844699442386627197265625f,1.162199944990874644901168721844442188739776611328125f,0.0f,0.0f,0.2476999924659732954523860826157033443450927734375f,0.53939997454583643587966435006819665431976318359375f,-2.7722000174105172476402003667317330837249755859375f,0.758400005286932188397486243047751486301422119140625f,0.0f,0.0f,1.11540004557967176168631340260617434978485107421875f,-0.6052000297188764310618580566369928419589996337890625f,-1.4393999810278419460018994868732988834381103515625f,-0.5901999816440046675580788360093720257282257080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3915999642401930547208621646859683096408843994140625f,-2.169399982853234210011805771500803530216217041015625f,-0.03869999923854994250405070488341152667999267578125f,-0.925899974814057724614713151822797954082489013671875f,0.0f,0.0f,1.553900051569938600692921681911684572696685791015625f,-2.77329990180283747491785106831230223178863525390625f,-0.213700014325977516449484028271399438381195068359375f,-0.50829995708614550409265575581230223178863525390625f,0.0f,0.0f,-0.181899947202205591878509949310682713985443115234375f,0.1043000505276026057543958813766948878765106201171875f,1.1750999720186001695765298791229724884033203125f,-1.21650002239644461354828308685682713985443115234375f,0.0f,0.0f,-1.024799953778087857614309541531838476657867431640625f,-0.4850999552361676681044855286017991602420806884765625f,0.1141000328227865789898487491882406175136566162109375f,2.25770001705288958504524998716078698635101318359375f,0.0f,0.0f,-0.1637000645801440779081303844577632844448089599609375f,-0.70639998322948682751842852667323313653469085693359375f,-2.776099918292462831459488370455801486968994140625f,2.557600026929379044560164402355439960956573486328125f,0.0f,0.0f,1.6751000232785953070191453662118874490261077880859375f,2.68509994070753332806589241954497992992401123046875f,0.321499949492513703575014005764387547969818115234375f,4.09660002570748371653053254703991115093231201171875f,0.0f,0.0f,2.21999991038814226129716189461760222911834716796875f,-0.79929997827559795542384790678624995052814483642578125f,0.87049999664351229711201085592620074748992919921875f,-0.567000031165779017072736678528599441051483154296875f,0.0f,0.0f,-1.177800046999752936471850262023508548736572265625f,1.168900066173076179865120138856582343578338623046875f,-0.153699947643280321329939397401176393032073974609375f,-0.35590004719644896535868383580236695706844329833984375f,0.0f,0.0f,3.0899999583140012049398137605749070644378662109375f,1.1161000592946999088184156789793632924556732177734375f,-2.29459995665550398058485370711423456668853759765625f,0.0720000042319284272451795914093963801860809326171875f,0.0f,0.0f,1.26109997294247033750025366316549479961395263671875f,0.73159995570927804919847403652966022491455078125f,-1.555899949482082877949551402707584202289581298828125f,-0.326600006298721101671844735392369329929351806640625f,0.0f,0.0f,2.32779986561536755829138201079331338405609130859375f,1.773599942953883346064003490027971565723419189453125f,0.173599946939944960178081601043231785297393798828125f,-1.81730001870989799073186077293939888477325439453125f,0.0f,0.0f,0.5042999555513272280649061940493993461132049560546875f,-1.7198000135689976985986504587344825267791748046875f,-0.807600077705085350743274830165319144725799560546875f,3.527400049532206427471692222752608358860015869140625f,0.0f,0.0f,-0.91529996969997906575144952512346208095550537109375f,-2.06379993553161700248210763675160706043243408203125f,-0.033400007596612102389599385787732899188995361328125f,-1.822599916921555429638601708575151860713958740234375f,0.0f,0.0f,-1.0628000479310759640583228247123770415782928466796875f,-0.16380000817477669006194673784193582832813262939453125f,1.9693000245064506970038564759306609630584716796875f,0.59470003773197621654844624572433531284332275390625f,0.0f,0.0f,-0.5236999122157717234671281403279863297939300537109375f,1.26289991828650283167689849506132304668426513671875f,3.34109999875873331376396890846081078052520751953125f,-0.306000073809176598160775029100477695465087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.153599984192847927033653832040727138519287109375f,-1.2145999913007015091892526470473967492580413818359375f,-0.53720000832080838648607823415659368038177490234375f,-0.230000027567148546125963548547588288784027099609375f,0.0f,0.0f,0.554100057148933178297056656447239220142364501953125f,0.3571999660015114930189383812830783426761627197265625f,-0.67440003944039350614048089482821524143218994140625f,0.003299947959184645895902576739899814128875732421875f,0.0f,0.0f,-0.370899982041120512832321765017695724964141845703125f,0.50259996110349935971584045546478591859340667724609375f,-0.466899971193075469244604391860775649547576904296875f,-0.00520006503462777658342020004056394100189208984375f,0.0f,0.0f,1.202500010523945395135569924605078995227813720703125f,0.59120005809813547248410259271622635424137115478515625f,-0.2472999967493116901806615715031512081623077392578125f,0.005600009025632513015580116189084947109222412109375f,0.0f,0.0f,1.65249996151775047081855518626980483531951904296875f,-0.54829990936517791766391383134759962558746337890625f,1.6824999482184648247340419402462430298328399658203125f,-1.2500000343471768804448629452963359653949737548828125f,0.0f,0.0f,-0.2301999971225849872524804595741443336009979248046875f,0.7975000099837779021783035204862244427204132080078125f,0.0003000699475397272664167758193798363208770751953125f,-2.449799971994757630255890035186894237995147705078125f,0.0f,0.0f,-0.25869991305396045078168754116632044315338134765625f,1.21549999513477136048322790884412825107574462890625f,-1.20389999488964605234286864288151264190673828125f,-0.97939999961703971820270453463308513164520263671875f,0.0f,0.0f,2.126599972212314781927489093504846096038818359375f,1.957999977923929435519312391988933086395263671875f,-1.4108000082716340983779446105472743511199951171875f,0.810600007386505172490842596744187176227569580078125f,0.0f,0.0f,-0.599599982471762604774312421795912086963653564453125f,0.2712999234244228841106405525351874530315399169921875f,1.5472999684929844566028123153955675661563873291015625f,-1.30899993392080116194620131864212453365325927734375f,0.0f,0.0f,-0.6300999901607642339484982585418038070201873779296875f,1.475599985109269152161459714989177882671356201171875f,0.7542000411257150371824309331714175641536712646484375f,-1.360600001649558965510777852614410221576690673828125f,0.0f,0.0f,-0.76729996510297215905893608578480780124664306640625f,-0.24990004020035250231757117944653145968914031982421875f,-1.4305999529600139652529833256267011165618896484375f,0.2752999799817807957680315666948445141315460205078125f,0.0f,0.0f,-0.883699988919496970396494361921213567256927490234375f,-0.181000021710991720880201683030463755130767822265625f,0.31880001175105621147309875595965422689914703369140625f,1.548700019615888212598520112805999815464019775390625f,0.0f,0.0f,1.244700011301041353561913638259284198284149169921875f,-0.5819000604137769538937163815717212855815887451171875f,-0.973799965035915260358478917623870074748992919921875f,0.428300031182170304333567401045002043247222900390625f,0.0f,0.0f,0.5215999979168171041266077736509032547473907470703125f,-1.0276000005915759683006172053865157067775726318359375f,-0.9763999299392092101612661281251348555088043212890625f,0.39209997734949009196014912959071807563304901123046875f,0.0f,0.0f,0.2981000679224725669058670973754487931728363037109375f,-0.788400033412873568750001140870153903961181640625f,0.2120000160783528375674222843372263014316558837890625f,0.1677999835766847969154014208470471203327178955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8183000836342582129390166301163844764232635498046875f,1.7324000211641188418099090995383448898792266845703125f,-0.09229989491850043581422369243227876722812652587890625f,1.94930003665089746078820098773576319217681884765625f,0.0f,0.0f,0.217500016391279249461376821273006498813629150390625f,3.057200000814346285693545723916031420230865478515625f,0.6702999277532091237929989802069030702114105224609375f,2.53340005233734810730084063834510743618011474609375f,0.0f,0.0f,-0.87610005270093882501214466174133121967315673828125f,-0.08529989409521210319553574663586914539337158203125f,-0.4562999186560501296838765483698807656764984130859375f,-1.0398999623045324103287612160784192383289337158203125f,0.0f,0.0f,-2.60900007680430956469308512168936431407928466796875f,-0.90579989563748153358346826280467212200164794921875f,-0.837799967986347926540702246711589395999908447265625f,2.8420001121535900523440432152710855007171630859375f,0.0f,0.0f,2.84500001296401006101177699747495353221893310546875f,-1.231800048348306919621109045692719519138336181640625f,1.2427999335646635614693877869285643100738525390625f,1.529900108745694264911207937984727323055267333984375f,0.0f,0.0f,1.1524000248894099573959692861535586416721343994140625f,-4.84539994169995313910703771398402750492095947265625f,0.5611999742791062839586402333225123584270477294921875f,-1.1835999770402889641474075688165612518787384033203125f,0.0f,0.0f,-1.088399980588257154323628128622658550739288330078125f,1.020199893037976313081571788643486797809600830078125f,1.560000092163681184587176176137290894985198974609375f,1.1243000693961981983903797299717552959918975830078125f,0.0f,0.0f,-0.945199947762491721192645854898728430271148681640625f,-4.5364000037744620641433357377536594867706298828125f,-0.78510000047981520765461027622222900390625f,-0.73420009063482183364612865261733531951904296875f,0.0f,0.0f,-4.420400028927627289476731675677001476287841796875f,-1.2985000322535633632270446469192393124103546142578125f,2.426500095188616423769190078019164502620697021484375f,-0.81039997140914099560404793010093271732330322265625f,0.0f,0.0f,-1.1396000127956262826245392716373316943645477294921875f,-0.9770000193566088508845268734148703515529632568359375f,0.03530006383806405523273497237823903560638427734375f,1.0310999972313654726718823440023697912693023681640625f,0.0f,0.0f,-0.8520999619454141527086221685749478638172149658203125f,-1.921299962465464972183326608501374721527099609375f,2.423900064347682015153395695961080491542816162109375f,-2.462400061713904353410953262937255203723907470703125f,0.0f,0.0f,0.083399879260358744659242802299559116363525390625f,-0.0147999256893986963490306152380071580410003662109375f,0.368799954307080124493722905754111707210540771484375f,-1.858499994665383070469033555127680301666259765625f,0.0f,0.0f,0.7148998680174347697402481571771204471588134765625f,-0.28389991246759915810571328620426356792449951171875f,1.91739995708912491778619369142688810825347900390625f,-0.8468000958457586602179389956290833652019500732421875f,0.0f,0.0f,0.419399941539765119813409910420887172222137451171875f,-0.41019990600198441565993334734230302274227142333984375f,1.00500002410262556651332488399930298328399658203125f,0.799100026936829355150848641642369329929351806640625f,0.0f,0.0f,1.567700005765256587864087123307399451732635498046875f,3.773000047400592027457832955406047403812408447265625f,0.8508001228429378226536528018186800181865692138671875f,0.9768999241054052706800803207443095743656158447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.00610003239810497088768670437275432050228118896484375f,0.84870001309663056776599887598422355949878692626953125f,-0.1986000315785407810409424200770445168018341064453125f,0.31680002461820866432873344820109196007251739501953125f,0.0f,0.0f,-1.693700013282894989430360510596074163913726806640625f,-0.6652999793484808410681807799846865236759185791015625f,1.400300059479474779067231793305836617946624755859375f,0.566700006256997568243605201132595539093017578125f,0.0f,0.0f,0.62399996517598577128183023887686431407928466796875f,-0.43690001880228457320498591798241250216960906982421875f,-0.7682999976918096773914612640510313212871551513671875f,0.6740000112205748461491339185158722102642059326171875f,0.0f,0.0f,0.08480003936663293995223966703633777797222137451171875f,-2.8759999866560104209156634169630706310272216796875f,0.378599936173856121968128718435764312744140625f,-2.175600008057058776245185072184540331363677978515625f,0.0f,0.0f,-0.2362999480858458678511624384555034339427947998046875f,-0.331899992762505036125730839557945728302001953125f,-1.7639999375343331511345468243234790861606597900390625f,-0.00160006538182511093282300862483680248260498046875f,0.0f,0.0f,0.6752999121814979854860894192825071513652801513671875f,0.8135999830380080677372234276845119893550872802734375f,0.381999933451414808160961911198683083057403564453125f,0.8518999797239903415402295649982988834381103515625f,0.0f,0.0f,-2.681999955635518517738091759383678436279296875f,-1.1733999992892147901102362084202468395233154296875f,0.580100013276190207278659727307967841625213623046875f,0.9275999586448075984179695296916179358959197998046875f,0.0f,0.0f,-1.2586999573290362786082141610677354037761688232421875f,0.49600002443045365652096734265796840190887451171875f,1.2697999661833050399906142047257162630558013916015625f,-1.145099972093105211712327218265272676944732666015625f,0.0f,0.0f,1.925199912558496873060676080058328807353973388671875f,0.8065000220797953200957408625981770455837249755859375f,-1.7452999496951708824354909665999002754688262939453125f,-0.0872999801158906851838992224656976759433746337890625f,0.0f,0.0f,0.8464000652790064638253397788503207266330718994140625f,0.66390003909021444439275683180312626063823699951171875f,-2.02459998843222788167395265190862119197845458984375f,2.20350001247227123002403459395281970500946044921875f,0.0f,0.0f,3.226400023406743144960273639298975467681884765625f,1.0228999685406687802924352581612765789031982421875f,1.110299956922232933465011228690855205059051513671875f,-0.78659995712488939290096823242492973804473876953125f,0.0f,0.0f,0.89640000206083081391028599682613275945186614990234375f,-0.49130000158101327922821610627579502761363983154296875f,-1.0059000096082690856036379045690409839153289794921875f,-0.14130003579780403555332668474875390529632568359375f,0.0f,0.0f,-1.280400041779876207925781272933818399906158447265625f,1.3578999674975869016435581215773709118366241455078125f,1.13809996514171363202194697805680334568023681640625f,-0.22460004069805084014888052479363977909088134765625f,0.0f,0.0f,1.156200003448127677785350897465832531452178955078125f,-0.21129999152272926910001160649699158966541290283203125f,-0.541199999983607238362992575275711715221405029296875f,1.729699985168128506529683363623917102813720703125f,0.0f,0.0f,-2.0493999376282090452150441706180572509765625f,1.5075999933645132244919295771978795528411865234375f,-2.15349989702552679915470434934832155704498291015625f,-1.47819996102079809219276285148225724697113037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.291400078280271035424675574176944792270660400390625f,-1.499199996757507591382818645797669887542724609375f,1.900200019399822526366961028543300926685333251953125f,0.176200065809488659596127035911194980144500732421875f,0.0f,0.0f,3.117900070241093057887837858288548886775970458984375f,1.605400084378570735310631789616309106349945068359375f,0.363099891977010980070872392388992011547088623046875f,0.7068000288277875853282239404506981372833251953125f,0.0f,0.0f,-1.01809993038475621318639241508208215236663818359375f,2.849200032781066038722883604350499808788299560546875f,1.97350002335012053578111590468324720859527587890625f,1.642200007911027714868623661459423601627349853515625f,0.0f,0.0f,1.60759995523616527890453653526492416858673095703125f,-0.4485999058686205298585036871372722089290618896484375f,-2.641999980725345142928972563822753727436065673828125f,-1.4426998993337150078986041989992372691631317138671875f,0.0f,0.0f,-0.6367999337583778451943317122641019523143768310546875f,0.71680009245574527465549863336491398513317108154296875f,0.7993999279797068258091030656942166388034820556640625f,2.2262999780371810487622497021220624446868896484375f,0.0f,0.0f,-2.398800017301738307651248760521411895751953125f,-1.91080011190921172925527571351267397403717041015625f,0.526100057655574548931554090813733637332916259765625f,1.2256000243738298660645114068756811320781707763671875f,0.0f,0.0f,3.03129996872395057749827174120582640171051025390625f,-0.2050999025791881980040898270090110599994659423828125f,2.1869000556081541475350604741834104061126708984375f,-3.2294000253587942239619223983027040958404541015625f,0.0f,0.0f,1.750899914911388410843073870637454092502593994140625f,-0.6801000690065333031952832243405282497406005859375f,1.57630006483644091730411673779599368572235107421875f,-1.82270008421987395053065483807586133480072021484375f,0.0f,0.0f,-2.972800048936902417295868872315622866153717041015625f,2.1400000105053180732284090481698513031005859375f,-4.10900004951655706264546097372658550739288330078125f,-2.812100018495322206746322990511544048786163330078125f,0.0f,0.0f,-2.0192999343916770982332309358753263950347900390625f,-3.30170002924203931371494036284275352954864501953125f,2.597700002449749856481275855912826955318450927734375f,-0.3360000371709468414138655134593136608600616455078125f,0.0f,0.0f,0.5192999927669761373039136742590926587581634521484375f,-1.7937000496231016999360008412622846662998199462890625f,-0.85300003689527414962867624126374721527099609375f,1.3762000041931881089141143093002028763294219970703125f,0.0f,0.0f,0.1691999990299333234133882797323167324066162109375f,-1.4862999698415377825000405209721066057682037353515625f,1.6164999844729892597428033695905469357967376708984375f,-1.514899911584705716194321212242357432842254638671875f,0.0f,0.0f,0.907299979706108761234872872591949999332427978515625f,-0.113100036182998575640112903784029185771942138671875f,-3.035099935846032881414657822460867464542388916015625f,2.491699958983064622231040630140341818332672119140625f,0.0f,0.0f,1.827099974238871826770491679781116545200347900390625f,-1.289599941568076335585146807716228067874908447265625f,2.56189986137151937128919598762877285480499267578125f,2.272399972455948269356440505362115800380706787109375f,0.0f,0.0f,-0.65210010924339290028228788287378847599029541015625f,4.1355999796703457604962750338017940521240234375f,1.50820010496601497607116471044719219207763671875f,-2.4732999917127198585831138188950717449188232421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.498399997985363274466408256557770073413848876953125f,-0.55799999777972708603357432366465218365192413330078125f,-0.595499983102083074726351696881465613842010498046875f,-0.265800050330162296319258530274964869022369384765625f,0.0f,0.0f,2.522200016516446208214574653538875281810760498046875f,2.510099982076884117532244999893009662628173828125f,-1.354700050789117593552646212629042565822601318359375f,-0.857500088959932238452665842487476766109466552734375f,0.0f,0.0f,-2.74149997538328182855593695421703159809112548828125f,0.439100034499168234702892732457257807254791259765625f,2.201700014340877142871022442704997956752777099609375f,-1.1193000335216520824133112910203635692596435546875f,0.0f,0.0f,-0.599800020311773263159693669877015054225921630859375f,2.0689999789968140930795925669372081756591796875f,0.2847999948494150945776937078335322439670562744140625f,2.42599992731213642827015064540319144725799560546875f,0.0f,0.0f,1.30399993857741502978342396090738475322723388671875f,0.675800027158856675413289849529974162578582763671875f,0.0214999488592140242104733260930515825748443603515625f,0.7612000057578089950993671664036810398101806640625f,0.0f,0.0f,-0.7184999732077130829566158354282379150390625f,0.333599902273715276379562055808492004871368408203125f,-0.35049996715784015322014965931884944438934326171875f,1.0895999553143980165259563364088535308837890625f,0.0f,0.0f,3.29609997700303747336647575139068067073822021484375f,1.4246000531405211830815460416488349437713623046875f,2.01030000887140669618702304433099925518035888671875f,-0.212299983952939808062865267856977880001068115234375f,0.0f,0.0f,0.209000046312809217852191068232059478759765625f,-2.09910004899054758453758040559478104114532470703125f,-0.03749998413026478516485440195538103580474853515625f,1.166199923726916498623040752136148512363433837890625f,0.0f,0.0f,-1.205699991363285761281076702289283275604248046875f,-2.029599972003698127309689880348742008209228515625f,1.4918999409437192493754764655022881925106048583984375f,1.6555999621987336212214358965866267681121826171875f,0.0f,0.0f,-0.22649996286630624808822176419198513031005859375f,-1.776000099271535237477337432210333645343780517578125f,-0.082700014594197401862629703828133642673492431640625f,0.0690000141635531694106475697481073439121246337890625f,0.0f,0.0f,-0.9558999552190299286991148619563318789005279541015625f,-0.5721000102810565035582612836151383817195892333984375f,0.380399982696771221668541329563595354557037353515625f,0.9254000716939561055340845996397547423839569091796875f,0.0f,0.0f,-1.116100025916097582268093901802785694599151611328125f,0.924099968636036717128945383592508733272552490234375f,1.1591000614702695603597248918958939611911773681640625f,-0.431900045754759265292932468582876026630401611328125f,0.0f,0.0f,0.152800073933601066045184779795818030834197998046875f,-0.8939999765008703747071194811724126338958740234375f,0.0614000155836347261129049002192914485931396484375f,-3.222299850326777459486038424074649810791015625f,0.0f,0.0f,-2.624799989205598205188607607851736247539520263671875f,0.4870000185370439993448599125258624553680419921875f,3.155399971073865383885959090548567473888397216796875f,-2.864899987692385963100605295039713382720947265625f,0.0f,0.0f,-0.68180005527734710568665832397527992725372314453125f,0.197800001141429593332077274681068956851959228515625f,3.83989993179440514126099515124224126338958740234375f,0.671799930610507800565756042487919330596923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.052000010974706523114718947908841073513031005859375f,-0.9380999761685722138082610399578697979450225830078125f,0.51620000639110819040666910950676538050174713134765625f,-0.4601999972343440159505689734942279756069183349609375f,0.0f,0.0f,-1.0508999497055999672312509574112482368946075439453125f,-2.10890000437051039483549175201915204524993896484375f,1.68889999163001736093292493023909628391265869140625f,-0.820599991516769211585824450594373047351837158203125f,0.0f,0.0f,0.91700011533871295110742494216538034379482269287109375f,1.70080000283271193239897911553271114826202392578125f,0.8237999462231992975347338870051316916942596435546875f,-0.318500020481644996461767505024909041821956634521484375f,0.0f,0.0f,0.348499954245982568323825034894980490207672119140625f,-1.7818000623181458053068126901052892208099365234375f,1.011000080347061125252139390795491635799407958984375f,-1.9025999378390612282174743086216039955615997314453125f,0.0f,0.0f,2.614699995207785843120973368058912456035614013671875f,0.955000190213322053267575029167346656322479248046875f,0.94320006687492108365944432080141268670558929443359375f,-0.07820008220449048419453674796386621892452239990234375f,0.0f,0.0f,-1.1943999441660932969710984252742491662502288818359375f,-2.220100043059886729679419659078121185302734375f,-0.18819997625723505763062348705716431140899658203125f,4.54129994510561285636640604934655129909515380859375f,0.0f,0.0f,1.1143000066250561275182917597703635692596435546875f,-1.5226999084606764878202511681593023240566253662109375f,-1.2857000201784085646750099840573966503143310546875f,-1.242100026094913989282986221951432526111602783203125f,0.0f,0.0f,-0.2218999927252529591470420200494118034839630126953125f,-2.452600006142259037034136781585402786731719970703125f,-0.297099927989393386429384236180339939892292022705078125f,1.9049999924376599391706577080185525119304656982421875f,0.0f,0.0f,0.1677999341979632408339284665999002754688262939453125f,3.04409990210235203988986540935002267360687255859375f,-0.953299988806991738243823419907130300998687744140625f,2.25819995656907668291069057886488735675811767578125f,0.0f,0.0f,-0.23709996656477583254485352881602011620998382568359375f,-1.3136999321460722800480880323448218405246734619140625f,-0.48790000866204596974284868338145315647125244140625f,-2.03400009347498400558151843142695724964141845703125f,0.0f,0.0f,-1.203400000481306353350419158232398331165313720703125f,-1.6076999805822964617618708871304988861083984375f,-1.38920008784085613484649002202786505222320556640625f,-3.013700043681261764305645556305535137653350830078125f,0.0f,0.0f,-0.9065999553740031746684735480812378227710723876953125f,-0.88020006671100869954926793070626445114612579345703125f,-1.8029000230044134323037496869801543653011322021484375f,-0.10630004656389335782051830392447300255298614501953125f,0.0f,0.0f,-1.8176999840095628968583696405403316020965576171875f,-2.85880006725788238242103034281171858310699462890625f,0.22819988545328495632702470174990594387054443359375f,2.422800003134458535214434959925711154937744140625f,0.0f,0.0f,0.006700037735700714591757787275128066539764404296875f,-0.5815999867968255632177942970884032547473907470703125f,-5.0059001151084903113996915635652840137481689453125f,4.41600006417930135427241111756302416324615478515625f,0.0f,0.0f,0.4275999261230227777019763379939831793308258056640625f,2.852100092591346669479435149696655571460723876953125f,0.158900040729343938750162124051712453365325927734375f,-2.09329990623593431564586353488266468048095703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.419200021344423578995019852300174534320831298828125f,0.895400029672682062908961597713641822338104248046875f,0.02529997327625732594924556906335055828094482421875f,-0.684399995630979418592687579803168773651123046875f,0.0f,0.0f,-0.29280003042221114384346947190351784229278564453125f,1.39449993270635541620094954851083457469940185546875f,0.456900005912781015382506666355766355991363525390625f,0.832699994142353627779584712698124349117279052734375f,0.0f,0.0f,1.125900020486117369955536560155451297760009765625f,-1.06529998303651751712095574475824832916259765625f,-0.08499999806284908032694147550500929355621337890625f,0.62860001228153716823499053134582936763763427734375f,0.0f,0.0f,-1.80010002648234301858565231668762862682342529296875f,-0.109200051854549951002582020009867846965789794921875f,1.5499000552505253214263802874484099447727203369140625f,-3.21850001321732914760787025443278253078460693359375f,0.0f,0.0f,-0.030399938142300353405289570218883454799652099609375f,1.58929994187951084683163571753539144992828369140625f,-1.1141000082358718525910035168635658919811248779296875f,1.06540004591494774643933851621113717555999755859375f,0.0f,0.0f,-2.11790004777759310883311627549119293689727783203125f,-2.7749999884515990089539627660997211933135986328125f,0.93719996697008511876703096277196891605854034423828125f,3.198100019530951687585229592514224350452423095703125f,0.0f,0.0f,-2.65980002869367648798970549250952899456024169921875f,-1.551600056478381617353079491294920444488525390625f,1.080500032275915511803532353951595723628997802734375f,2.335899966804684257937196889542974531650543212890625f,0.0f,0.0f,-1.6857000057429072370496214716695249080657958984375f,-4.54229996864199581096954716485925018787384033203125f,-2.062500001974403307514194239047355949878692626953125f,2.6099000347241751995852609979920089244842529296875f,0.0f,0.0f,-0.618800004561245398093660696758888661861419677734375f,0.86099999943375571387349509677733294665813446044921875f,0.49219999748021325824964833373087458312511444091796875f,0.9956999676704401114335496458807028830051422119140625f,0.0f,0.0f,-2.15369999540150214301093001267872750759124755859375f,-0.63840001698434445387420055340044200420379638671875f,1.64229996107965714458032380207441747188568115234375f,2.49110001513138445972117551718838512897491455078125f,0.0f,0.0f,-1.8151000752836470297779669635929167270660400390625f,-3.52660007503032613129789751837961375713348388671875f,0.165000033080577868105365269002504646778106689453125f,-2.8986999100364752024461267865262925624847412109375f,0.0f,0.0f,-0.267599962407350400184213867760263383388519287109375f,-0.65869994414150767614302139918436296284198760986328125f,2.0243000365018843922371161170303821563720703125f,1.1783000732593240655887711909599602222442626953125f,0.0f,0.0f,-1.48310000470429681485029504983685910701751708984375f,-1.624100019560754670777669161907397210597991943359375f,0.89769998404681672354854526929557323455810546875f,3.47719998881891445563496745307929813861846923828125f,0.0f,0.0f,-3.021999991305170940592006445513106882572174072265625f,-0.60230003668442311237640751642175018787384033203125f,2.97810007097721207713902913383208215236663818359375f,-1.54260002506673377098422861308790743350982666015625f,0.0f,0.0f,-0.1273999342530967027187216444872319698333740234375f,0.51170009089559176373995796893723309040069580078125f,1.3466999696373929129578073116135783493518829345703125f,-0.5472000498391675904485964565537869930267333984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.061200014325976770379611480166204273700714111328125f,-0.41079995578229466790531887454562820494174957275390625f,0.056399987718462718522260956888203509151935577392578125f,-0.34209995136559034900614051366574130952358245849609375f,0.0f,0.0f,1.110299996373056874432450058520771563053131103515625f,-0.8886000242024658657413738183095119893550872802734375f,0.39779998489916368331620333265163935720920562744140625f,0.244499998662620843248305391171015799045562744140625f,0.0f,0.0f,1.8944999505504955816803658308344893157482147216796875f,-0.4144999664574868791788730959524400532245635986328125f,-0.1074999906867739607463363427086733281612396240234375f,-1.689700002732873240773869838449172675609588623046875f,0.0f,0.0f,-1.7843000422760837420099733208189718425273895263671875f,-1.189600098514556947293385746888816356658935546875f,0.48669997250586660708648878426174633204936981201171875f,0.5102999269708992091665322732296772301197052001953125f,0.0f,0.0f,0.98870004345774697984694512342684902250766754150390625f,3.980699947479367839520136840292252600193023681640625f,-0.832299961694328960248867588234134018421173095703125f,1.45480006361827207683745655231177806854248046875f,0.0f,0.0f,-0.13280012778267291650990955531597137451171875f,0.46960007828622973846677268738858401775360107421875f,-2.006499966628849218608365845284424722194671630859375f,2.18609994045793865069526873412542045116424560546875f,0.0f,0.0f,-0.1160999320760378594741268898360431194305419921875f,1.3417999261960371626400956301949918270111083984375f,-0.8950999857276682458717687040916644036769866943359375f,1.519499950103462726502812074613757431507110595703125f,0.0f,0.0f,-2.173599888229370780123872464173473417758941650390625f,-2.89460000828802588301869036513380706310272216796875f,-0.60559999406859199577723984475596807897090911865234375f,-0.53150007436424384632545070417108945548534393310546875f,0.0f,0.0f,-2.951099845612049410448207709123380482196807861328125f,-0.08989993931949158412209044399787671864032745361328125f,0.552799997732788650495194815448485314846038818359375f,-3.912000008217990654202367295511066913604736328125f,0.0f,0.0f,-1.530200072000919231385296370717696845531463623046875f,-0.21119992573857349071886346791870892047882080078125f,-1.2647999231934534680732440392603166401386260986328125f,0.11370006122738285370132871321402490139007568359375f,0.0f,0.0f,1.0347999090746047290423348385957069694995880126953125f,0.7502000695794828200035908594145439565181732177734375f,1.53949997770786328743497506366111338138580322265625f,0.7484000941537323203789355829940177500247955322265625f,0.0f,0.0f,4.52409997478276437732347403652966022491455078125f,0.1850999722421164950247884917189367115497589111328125f,0.711799917031824325164279798627831041812896728515625f,1.9388001232489937564196225139312446117401123046875f,0.0f,0.0f,0.09880003060102404699449607505812309682369232177734375f,2.5387999824702749407379087642766535282135009765625f,-0.727300014891473889377948580658994615077972412109375f,-0.3285000758022065259211785814841277897357940673828125f,0.0f,0.0f,-0.186599934791773502951173213659785687923431396484375f,2.535499924689532225130506049026735126972198486328125f,-1.304599907779693257481312684831209480762481689453125f,-1.89980004996508444747860266943462193012237548828125f,0.0f,0.0f,1.30130003350675149675907960045151412487030029296875f,3.82570005655586697912440286017954349517822265625f,-0.055099919752776838066665732185356318950653076171875f,-0.118700075662881321392205791198648512363433837890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.085099951380491756225410426850430667400360107421875f,0.604199949409067738059775365400128066539764404296875f,-0.43969997859299159159718328737653791904449462890625f,-0.258600014293194124803676459123380482196807861328125f,0.0f,0.0f,-1.0305999821662903404018152286880649626255035400390625f,0.3606999702036386423742442275397479534149169921875f,-0.657499976158141929971634453977458178997039794921875f,-0.1505000178962945245331184196402318775653839111328125f,0.0f,0.0f,0.5439000240027895838323956922977231442928314208984375f,1.14999997869133974148780907853506505489349365234375f,-0.4432999831259252232484868727624416351318359375f,0.9730999690532684542887409406830556690692901611328125f,0.0f,0.0f,-0.6404000038936727978722274201572872698307037353515625f,-0.906099968509376463288162995013408362865447998046875f,0.1682000118151301126800944985006935894489288330078125f,1.1514000264242287840943390619941055774688720703125f,0.0f,0.0f,-0.427299952399730453844739486157777719199657440185546875f,-1.187099945665895273094747608411125838756561279296875f,-1.7710000160858034856659060096717439591884613037109375f,1.6279000315353273631302499779849313199520111083984375f,0.0f,0.0f,-1.130299972791970919416826291126199066638946533203125f,-1.6792999349132184949695556497317738831043243408203125f,-1.7800000070035457699901826345012523233890533447265625f,0.0913999919280403805288415242102928459644317626953125f,0.0f,0.0f,-0.77939997070878774820812395773828029632568359375f,-1.7050999552920469426453564665280282497406005859375f,-1.073300029356777596234451266354881227016448974609375f,2.4113999563515182700257355463691055774688720703125f,0.0f,0.0f,-0.1662000536650400928095905328518711030483245849609375f,1.32100004469603238277386481058783829212188720703125f,-0.513600031988322935916357891983352601528167724609375f,0.2080999340355400573088218152406625449657440185546875f,0.0f,0.0f,1.749599969433248691785820483346469700336456298828125f,1.5616000391185280715689032149384729564189910888671875f,-2.617600031660496373575597317540086805820465087890625f,-0.9515000486597420259471391545957885682582855224609375f,0.0f,0.0f,0.7222000098854299832140668513602577149868011474609375f,-1.5999000223189578928639775767805986106395721435546875f,-0.1201999602049592663632893163594417273998260498046875f,-1.4202000117257238542833874817006289958953857421875f,0.0f,0.0f,-0.0976999599441887855988397859618999063968658447265625f,-0.2731999658077963744062799378298223018646240234375f,-0.7457999832004305051924575309385545551776885986328125f,-0.804300045330821777866958655067719519138336181640625f,0.0f,0.0f,-1.02289994753003110616873527760617434978485107421875f,1.604400012655556206908613603445701301097869873046875f,-1.18219998172223572652228540391661226749420166015625f,0.47329995594993301555319931139820255339145660400390625f,0.0f,0.0f,-0.151999974727630604576233963598497211933135986328125f,-1.605799934659898564603963677654974162578582763671875f,0.2907999866090718033007078702212311327457427978515625f,-0.683500047732144455636671409592963755130767822265625f,0.0f,0.0f,0.748900053674727672614608309231698513031005859375f,-0.84919992411434674917103393454453907907009124755859375f,0.806299949315190556120569453923963010311126708984375f,0.0883999711073943927885920857079327106475830078125f,0.0f,0.0f,1.5038000110059979164134347229264676570892333984375f,0.290199933196605197593953562318347394466400146484375f,-2.35380005835443650852312202914617955684661865234375f,-0.6284000178225335186965594402863644063472747802734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.370500051014125642012686512316577136516571044921875f,0.23310009604841452901524689877987839281558990478515625f,-1.363600065217913215320777453598566353321075439453125f,1.0821999908305712789768904258380644023418426513671875f,0.0f,0.0f,-1.4969000763952735599815468958695419132709503173828125f,-2.26270004059672391605317898211069405078887939453125f,0.08790002546310471398527397468569688498973846435546875f,-1.1728999924197787141366688956622965633869171142578125f,0.0f,0.0f,-0.41599996020644780525543637850205413997173309326171875f,0.2791000281289222062497401566361077129840850830078125f,1.1773999711334706130827498782309703528881072998046875f,-1.7234999610260128832806003629229962825775146484375f,0.0f,0.0f,1.9246000576481223642844042842625640332698822021484375f,1.7968000022664669668159831417142413556575775146484375f,-2.53640006753280911055981050594709813594818115234375f,0.4426999532386670654915405975771136581897735595703125f,0.0f,0.0f,-0.0457000415056938447122547586332075297832489013671875f,-2.582599969783427429348421355825848877429962158203125f,5.4242000433236370326994801871478557586669921875f,-5.97889992897063482502062470302917063236236572265625f,0.0f,0.0f,2.688199969737976591233064027619548141956329345703125f,2.149899947738648453565701856859959661960601806640625f,1.5434000472337012777046538758440874516963958740234375f,-2.12400003305077689219615422189235687255859375f,0.0f,0.0f,1.61029994757175476394195356988348066806793212890625f,0.3075999583095299083623785918462090194225311279296875f,0.9472999290421604268175315155531279742717742919921875f,-3.52129984821080999068954042741097509860992431640625f,0.0f,0.0f,5.27809998384267142768067060387693345546722412109375f,2.69829997623413664342706397292204201221466064453125f,4.9829000734075901135611275094561278820037841796875f,3.919699949833750363126227966859005391597747802734375f,0.0f,0.0f,0.4335000099018209152035296938265673816204071044921875f,-0.97730006475448616409806845695129595696926116943359375f,1.3351000732533637727073028145241551101207733154296875f,2.401699916924537436813125168555416166782379150390625f,0.0f,0.0f,3.8256999889418477067692947457544505596160888671875f,-0.763600091071427566902229955303482711315155029296875f,-3.3361999146372074420696662855334579944610595703125f,-5.51009987812265666917710404959507286548614501953125f,0.0f,0.0f,1.93200012329220616180691649788059294223785400390625f,1.444900009392203532598841775325126945972442626953125f,-2.58869990681409856136951930238865315914154052734375f,3.09209995583593855172921394114382565021514892578125f,0.0f,0.0f,-2.034199961292743719099007648765109479427337646484375f,-1.510900068318843825210251452517695724964141845703125f,-0.9438000498980290142725380064803175628185272216796875f,-0.81470002100542071588051840080879628658294677734375f,0.0f,0.0f,-0.89030000441968493163358289166353642940521240234375f,1.708400023987889593257705200812779366970062255859375f,0.891800017093122843192531945533119142055511474609375f,-2.04120007234737244061761884950101375579833984375f,0.0f,0.0f,-0.179700187730788218942734602023847401142120361328125f,-0.2609000048398966153939682044438086450099945068359375f,-2.56150000641494823838684169459156692028045654296875f,-0.830500040620567148152986192144453525543212890625f,0.0f,0.0f,-0.4010000369474300274674760657944716513156890869140625f,0.1698999121993762795312932212254963815212249755859375f,-1.0598000659093271824673365699709393084049224853515625f,0.366500056333840529276812958414666354656219482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.277299999226630067283849712111987173557281494140625f,0.3457000107347969475313220755197107791900634765625f,0.145699984993040487779580871574580669403076171875f,-0.21529998195618349488000831115641631186008453369140625f,0.0f,0.0f,0.4762000037461520118853286476223729550838470458984375f,0.5380999570578339774584719634731300175189971923828125f,0.698200002166629030142530609737150371074676513671875f,0.138599985273182735312502700253389775753021240234375f,0.0f,0.0f,-0.365200012321770295375955583949689753353595733642578125f,0.2940000222846868016546295621083118021488189697265625f,-0.65450000367313643234723485875292681157588958740234375f,0.287199989396333510427439250634051859378814697265625f,0.0f,0.0f,-0.4131999930769196804902776420931331813335418701171875f,0.2707999229252340267493082137661986052989959716796875f,1.4130000390931962694907042532577179372310638427734375f,0.512099988096952785099347238428890705108642578125f,0.0f,0.0f,0.908000039018690419112544987001456320285797119140625f,2.70549997401237529714990159845910966396331787109375f,0.2316000063359739957746796790161170065402984619140625f,-0.87980004247725063226681641026516444981098175048828125f,0.0f,0.0f,0.0883999833449717886679763978463597595691680908203125f,-1.59650003085285430159956376883201301097869873046875f,-0.1945999706253405125977451461949385702610015869140625f,1.42089994970560073994647609652020037174224853515625f,0.0f,0.0f,-2.037599936591089289095179992727935314178466796875f,2.188000023521482706456708910991437733173370361328125f,-1.1087000087380414203863665534299798309803009033203125f,-0.259800000865757052537219351506792008876800537109375f,0.0f,0.0f,-0.0201000359818335194717064950964413583278656005859375f,-2.04679998494386605756290009594522416591644287109375f,-1.6252999977886675164739926913171075284481048583984375f,3.066599913315474879027533461339771747589111328125f,0.0f,0.0f,-1.4590999654322864831357264847611077129840850830078125f,-0.57249999240040894132874882416217587888240814208984375f,0.8672999378338464016025000091758556663990020751953125f,0.7183000375524162262053096128511242568492889404296875f,0.0f,0.0f,-1.269000034764407924825491136289201676845550537109375f,0.3593000559106476199389135217643342912197113037109375f,-0.20070001330524689375778280009399168193340301513671875f,0.78159993887096723685914412271813489496707916259765625f,0.0f,0.0f,-0.8612999553874136626774316027876920998096466064453125f,-0.693499955289065272978632492595352232456207275390625f,-0.6623000486612318216117500924156047403812408447265625f,-1.2203999724335965471055942543898709118366241455078125f,0.0f,0.0f,0.570599951916933800788456210284493863582611083984375f,0.76079999616444116128377572749741375446319580078125f,1.6023000180020925409252186000230722129344940185546875f,-0.670799962096660262744762803777121007442474365234375f,0.0f,0.0f,-2.30159999719038577126184463850222527980804443359375f,-0.7657999900177119290134442053386010229587554931640625f,1.2055999331414712916199505343683995306491851806640625f,2.293799981259553977253062839736230671405792236328125f,0.0f,0.0f,1.282199986378849576595939652179367840290069580078125f,-0.4403999800518152429873453002073802053928375244140625f,-0.39699998348206266829407695695408619940280914306640625f,-1.1349999348074195637536831782199442386627197265625f,0.0f,0.0f,-1.560699975754320956156107058632187545299530029296875f,0.4856001113407302227642503567039966583251953125f,1.9364000040352333797954997862689197063446044921875f,0.53389995946213619770759351013111881911754608154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.399100169950724836809285989147610962390899658203125f,1.75850002681464001597078095073811709880828857421875f,-1.46710004732906806879100258811376988887786865234375f,2.8070000353008506266405674978159368038177490234375f,0.0f,0.0f,0.668399949705600793237181278527714312076568603515625f,-1.98210008003711646296096660080365836620330810546875f,-0.10029998221695357596416897649760358035564422607421875f,0.3603000230833888206660731157171539962291717529296875f,0.0f,0.0f,0.4364999317601319095416556592681445181369781494140625f,0.874000001572073248468086603679694235324859619140625f,-0.7904999752789743094893992747529409825801849365234375f,-2.01930003612935582424370295484550297260284423828125f,0.0f,0.0f,-0.1784000517040469357965548624633811414241790771484375f,-3.267599990458788017377855794620700180530548095703125f,-0.2225001354143021359277554438449442386627197265625f,-1.18449998255074095965255764895118772983551025390625f,0.0f,0.0f,0.9801999942541101962234506572713144123554229736328125f,0.6917998819023376544379289043718017637729644775390625f,2.4699000536486526158341803238727152347564697265625f,-0.954400002802163083259756604093126952648162841796875f,0.0f,0.0f,1.1545000155754381321315804598270915448665618896484375f,3.11710000776648410436564518022350966930389404296875f,1.94370005619824137710338618489913642406463623046875f,-3.818900025269389431059607886709272861480712890625f,0.0f,0.0f,4.4388000098139013260833962704055011272430419921875f,0.6417999422520377361678356464835815131664276123046875f,2.451700015868246484984638300375081598758697509765625f,0.7902000178724535306429288539220578968524932861328125f,0.0f,0.0f,1.14479999393671949547979238559491932392120361328125f,5.28170002346783729052503986167721450328826904296875f,2.124200005791337719784905857522971928119659423828125f,-1.703299972210822854634670875384472310543060302734375f,0.0f,0.0f,0.7725999907940632738245767541229724884033203125f,-1.37040000434070829982147188275121152400970458984375f,-0.847299964451041720536750290193594992160797119140625f,-2.945199978235365279033430851995944976806640625f,0.0f,0.0f,-2.54429992034733398753587607643567025661468505859375f,4.3139000497817985291248987778089940547943115234375f,-1.5128999798655531350277669844217598438262939453125f,-1.230799939577280621705313023994676768779754638671875f,0.0f,0.0f,-1.6296998534604905106704109130077995359897613525390625f,3.700500122502446753713911675731651484966278076171875f,2.24570003785490968084559426642954349517822265625f,-0.1032999834798278282477212997036986052989959716796875f,0.0f,0.0f,-1.8518999047338962071762580308131873607635498046875f,-0.831000006213785002984195671160705387592315673828125f,-0.565900129413604791039915653527714312076568603515625f,2.293000018436463616211540283984504640102386474609375f,0.0f,0.0f,0.859799950686097158580878385691903531551361083984375f,3.8983999830469482361650079838000237941741943359375f,-0.6895999747604140139856099267490208148956298828125f,-4.2526000801824039854182046838104724884033203125f,0.0f,0.0f,-0.6846000407353038941238310144399292767047882080078125f,3.20380008817538541876501767546869814395904541015625f,4.15639997266829208655281036044470965862274169921875f,-1.031299952444434619991397994454018771648406982421875f,0.0f,0.0f,-0.265800001975894151229340423014946281909942626953125f,0.4530999518051743724100788313080556690692901611328125f,0.9139999956488598087389618740417063236236572265625f,-1.8707999209508301419191411696374416351318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.17619994645118719489573777536861598491668701171875f,-0.1486000146657231457680836683721281588077545166015625f,-0.387499965578317340231251364457421004772186279296875f,1.32489997202754050675821417826227843761444091796875f,0.0f,0.0f,0.33710001104474063282623319537378847599029541015625f,-2.054999950230121275041028638952411711215972900390625f,-0.3213999988198283919160758159705437719821929931640625f,-0.27529996612370066788599842766416259109973907470703125f,0.0f,0.0f,-0.2491999866247176509403971067513339221477508544921875f,0.242099977666139665188893559388816356658935546875f,-0.063800006908178108488982616108842194080352783203125f,-0.876300000947713098042868296033702790737152099609375f,0.0f,0.0f,-0.829299950224161808165490583633072674274444580078125f,0.545399980387092142564142704941332340240478515625f,-2.64620006536245266914875173824839293956756591796875f,1.776799989861249873257520448532886803150177001953125f,0.0f,0.0f,0.5452999322235576595829797952319495379924774169921875f,0.25199999335408307388206594623625278472900390625f,-0.322600030505657553447917962330393493175506591796875f,1.5531999666646125746183315641246736049652099609375f,0.0f,0.0f,0.407100044497847424196379506611265242099761962890625f,2.37470001353621373851865428150631487369537353515625f,-1.187599976451695571455502431490458548069000244140625f,-1.983000006198881948193957214243710041046142578125f,0.0f,0.0f,1.638399947172403781081584384082816541194915771484375f,0.2676999683260934403961073257960379123687744140625f,-1.71269996311068606331673436216078698635101318359375f,-1.7666999818936002242253380245529115200042724609375f,0.0f,0.0f,-0.6063999152243149115548703775857575237751007080078125f,3.26809998767971965349943275214172899723052978515625f,0.3178999879121777727419839720823802053928375244140625f,-2.690899964383245190902016474865376949310302734375f,0.0f,0.0f,0.73139996585100941484114400736871175467967987060546875f,-0.67789995244741430013846184010617434978485107421875f,-0.79249992489814780061152532653068192303180694580078125f,0.0342000002592806051637808195664547383785247802734375f,0.0f,0.0f,0.358399970641732235066001521772705018520355224609375f,-1.238199951130151799105760801467113196849822998046875f,1.1123000146120778364178249830729328095912933349609375f,0.394999984204769560847125831060111522674560546875f,0.0f,0.0f,3.76789992868006340387410091352649033069610595703125f,1.48720005794167509094449997064657509326934814453125f,-0.379299917292595711870717423153109848499298095703125f,2.676699951029569479743486226652748882770538330078125f,0.0f,0.0f,1.77370001618862094261430684127844870090484619140625f,1.932499975115061108255076760542578995227813720703125f,-2.37230007801651954224553264793939888477325439453125f,0.51689996482282862810819779042503796517848968505859375f,0.0f,0.0f,3.08869997968077836247857703710906207561492919921875f,0.559699961605667706265876404359005391597747802734375f,0.39460001074671791077008720094454474747180938720703125f,-3.5286000296972677148232833133079111576080322265625f,0.0f,0.0f,0.714499996855855190602824222878552973270416259765625f,-0.429399997381865350387641910856473259627819061279296875f,-2.387800004047155422171044847345910966396331787109375f,0.63539999838024474598796587088145315647125244140625f,0.0f,0.0f,1.937199908334017894873113618814386427402496337890625f,-1.53020004923939634267071596696041524410247802734375f,-2.09929996601939183875629169051535427570343017578125f,0.076900040744245057311445634695701301097869873046875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_strided)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,3,5,8,7};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,1,4,2,4};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,1,3,2},        // move_strides
                                                                  Strides{1,1,1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0,0}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.20289988379031631549054282004362903535366058349609375f,2.989100064897538100439078334602527320384979248046875f,0.73939990573972347309705810403102077543735504150390625f,-1.137100040586292326594275436946190893650054931640625f,-0.3103999343052510351270711908000521361827850341796875f,-1.7154999895840881141140243926201947033405303955078125f,0.0259000097200268175612336563062854111194610595703125f,0.20779999909251767320483850198797881603240966796875f,2.154500004455447648155086426413618028163909912109375f,3.190200142204016930946863794815726578235626220703125f,0.2650000340677802768141191336326301097869873046875f,-2.5066000876963148158438343671150505542755126953125f,0.671400006046892539046666570357047021389007568359375f,-0.4078000063568361088783831291948445141315460205078125f,2.39729996975958403737649859976954758167266845703125f,-3.89060003178715607674575949204154312610626220703125f,-0.3418999886460600823312461216119118034839630126953125f,2.2492000240825102963526660460047423839569091796875f,0.17170006303042129047753405757248401641845703125f,1.1246999541178348547276755198254249989986419677734375f,1.0823000470593580057965255036833696067333221435546875f,-1.4998000237017869995526098136906512081623077392578125f,2.624299957562981955305758674512617290019989013671875f,-2.549700157369673814145016876864247024059295654296875f,-1.3512999899394804170782435903674922883510589599609375f,-1.55439998551681668459423235617578029632568359375f,-4.0139999058321134128846097155474126338958740234375f,0.81220005782991666620773685281164944171905517578125f,-0.38870005824714848330359018291346728801727294921875f,-0.06329992471337286819021983319544233381748199462890625f,-0.2833999914847311618615322004188783466815948486328125f,-6.4299000021651391278965093079023063182830810546875f,-0.96139994465410705259955648216418921947479248046875f,-1.2218000632867209720444634513114579021930694580078125f,-1.0313000185683371956457676787977106869220733642578125f,1.468199999745189554545277133001945912837982177734375f,5.36729999915212285799270830466412007808685302734375f,3.1843000175960352038373457617126405239105224609375f,0.439500025223941381824488416896201670169830322265625f,-1.0922000041857369279085787638905458152294158935546875f,0.515000031813977354744338299497030675411224365234375f,-2.25150000650808213009668179438449442386627197265625f,-1.964199980496615705050089673022739589214324951171875f,0.1789000455722218330123496343730948865413665771484375f,-1.9858999907210463931050981045700609683990478515625f,1.38709994335621633609889613580889999866485595703125f,-1.799300043207406218925825669430196285247802734375f,-0.56980008013993443949374295698362402617931365966796875f,-2.810899963638184839709310836042277514934539794921875f,-0.17670000210329794843033823781297542154788970947265625f,1.9399999443441624524808730711811222136020660400390625f,-2.983600069017707578922227185103110969066619873046875f,0.503699960011244396440588388941250741481781005859375f,0.9671000271379954771333586904802359640598297119140625f,-3.99049997919052845674059426528401672840118408203125f,-1.65950000527501106972749767010100185871124267578125f,1.514799981009959051192481638281606137752532958984375f,-0.139200010205805302376802501385100185871124267578125f,-2.41750007489696105977827755850739777088165283203125f,0.51210000074431238914485220448113977909088134765625f,0.43100004518032053457687879927107132971286773681640625f,1.1780999897658828157176458262256346642971038818359375f,2.0346000760324312039983851718716323375701904296875f,4.39400011672079582325523006147705018520355224609375f,-1.9755999386921505589498337940312922000885009765625f,-0.421700098159910552464424426943878643214702606201171875f,-0.39889999326914449273573382015456445515155792236328125f,-0.734100039751826471956519526429474353790283203125f,-0.82020002666413827085989396437071263790130615234375f,1.29439998154938251673229387961328029632568359375f,2.4596002017423526098127695149742066860198974609375f,0.6740001134425395168392469713580794632434844970703125f,1.32879997982531694589170001563616096973419189453125f,3.006899964804201630386160104535520076751708984375f,0.42849994958937143163524297051480971276760101318359375f,0.21790009352415840027106241905130445957183837890625f,0.5991998458087433032659419041010551154613494873046875f,-0.195999966353177457190071208970039151608943939208984375f,2.1591000189647076012988691218197345733642578125f,-1.004700082193312749012648055213503539562225341796875f,2.739500035822391321715940648573450744152069091796875f,4.88729999065846154593373285024426877498626708984375f,0.670200103591381601830789804807864129543304443359375f,2.193600149074197869225599788478575646877288818359375f,0.5954000336959968109340479713864624500274658203125f,0.181500071607529012140958002419210970401763916015625f,-0.37870006975829351159035240925732068717479705810546875f,2.449700053471328686782726435922086238861083984375f,0.6460999792195838420383324773865751922130584716796875f,1.457100046416373029245505676954053342342376708984375f,-2.89939995763301805453693305025808513164520263671875f,0.1708999533936388370847225814941339194774627685546875f,-1.001299958721549199225364645826630294322967529296875f,-1.571899991235139015799404660356231033802032470703125f,0.47330003349185201244608833803795278072357177734375f,-3.737399974039197214636942590004764497280120849609375f,1.955399942165613413891378513653762638568878173828125f,-0.334799994830787872590605047662393189966678619384765625f,-2.6215999964639546959688232163898646831512451171875f,-2.67780011565685160945804454968310892581939697265625f,1.2414000073879936092424713933723978698253631591796875f,-0.728299917337298818864610439050011336803436279296875f,0.07540002084374552993040197179652750492095947265625f,-0.7792999981686483668141818270669318735599517822265625f,1.7360000136643660706425862372270785272121429443359375f,1.10330001341253502999961710884235799312591552734375f,-0.7875000528246165476531359672662802040576934814453125f,1.924399956068397532504832270205952227115631103515625f,0.340500042036175454285285013611428439617156982421875f,-0.080299976852535703830682223269832320511341094970703125f,-2.29619994719624376244837549165822565555572509765625f,2.063500104375183052951570061850361526012420654296875f,-0.9446999554589374792357148180599324405193328857421875f,0.64610009220763864323799907651846297085285186767578125f,-1.248499993361532833802129971445538103580474853515625f,0.2651000453695646630336568705388344824314117431640625f,0.061599971430004618611064870492555201053619384765625f,3.9419000512495614430008572526276111602783203125f,-0.68250009376555798024099885878968052566051483154296875f,-2.1309999011233440313617393258027732372283935546875f,-1.027399983147530004856662344536744058132171630859375f,-1.199299950745701170973234184202738106250762939453125f,3.202199885702878390247860806994140148162841796875f,2.088599919894338352577278783428482711315155029296875f,-1.840299968396126306657833993085660040378570556640625f,-4.48010002290606479391499306075274944305419921875f,1.5331001253291969277370299096219241619110107421875f,-1.02770003668516807238120236434042453765869140625f,-0.3871000539600839562837109042447991669178009033203125f,1.48460000752433973758570573409087955951690673828125f,1.89880001525282882113287996617145836353302001953125f,0.596200047797709853369951815693639218807220458984375f,2.871199982549248463925550822750665247440338134765625f,1.914100068362057260884512288612313568592071533203125f,-0.3273000454761094690780964810983277857303619384765625f,1.51829998966380852465363204828463494777679443359375f,0.360100071744621441638400938245467841625213623046875f,4.865499961365014058856104384176433086395263671875f,1.011699923667312095432180285570211708545684814453125f,0.8791000903971506463818741394788958132266998291015625f,-1.2175001530349260914221076745889149606227874755859375f,-1.581599939504266583156777414842508733272552490234375f,1.7045001071989542840157128011924214661121368408203125f,-0.90200005012750583688330152654089033603668212890625f,2.7825000848807395215089854900725185871124267578125f,-0.42239995094239801698421388209681026637554168701171875f,0.07449994205683407866303014088771305978298187255859375f,0.869600045932083620670027812593616545200347900390625f,-3.87550004460662567140616374672390520572662353515625f,-0.60939997566342329893274154528626240789890289306640625f,2.410099894104153595009165655937977135181427001953125f,0.2245998999752092384341040087747387588024139404296875f,-1.002599970602989554180339837330393493175506591796875f,-1.021100070507824142396202660165727138519287109375f,-0.297200044139475760829327555256895720958709716796875f,-0.84990000782757968433855921830399893224239349365234375f,-1.7397999921672056000687689447659067809581756591796875f,-1.5931000033445650299057660959078930318355560302734375f,0.04809987644255098171441886734100989997386932373046875f,0.2941000788673762134095568399061448872089385986328125f,-0.21179987823292745208192400241387076675891876220703125f,2.141499952044338073164908564649522304534912109375f,1.023200113254784326954904827289283275604248046875f,-0.12770001751854975768907252131612040102481842041015625f,-1.6914000763803735960522089953883551061153411865234375f,-0.649299920496344551423817392787896096706390380859375f,2.558800016668438814804176217876374721527099609375f,0.689900006002188259657259550294838845729827880859375f,-3.036500040203334638277965495944954454898834228515625f,-2.090699986706673119130073246196843683719635009765625f,-0.807200018751621062307322063134051859378814697265625f,0.99860002336427822200448645162396132946014404296875f,-3.126899965120851110356170465820468962192535400390625f,-1.493800151021032807108213091851212084293365478515625f,0.622099982807039264542936507496051490306854248046875f,0.069699942848831852870716829784214496612548828125f,-0.7279000219427047557729792970349080860614776611328125f,-1.6220000575967141731581477870349772274494171142578125f,-1.000000014100222767154946268419735133647918701171875f,-0.913199916801603617244609267800115048885345458984375f,-0.4218000189743940087083728940342552959918975830078125f,-0.01230004365071579730539497177232988178730010986328125f,1.55979997583180729492369209765456616878509521484375f,0.200399993686377175805546357878483831882476806640625f,-1.0542999518260349756104687912738882005214691162109375f,-2.015000015422701462597387944697402417659759521484375f,0.044000014051795321989857256994582712650299072265625f,0.80600009364634761510615135193802416324615478515625f,-0.082100099315493668683529904228635132312774658203125f,-3.335799966734647181709760843659751117229461669921875f,-3.004699922005830359239553217776119709014892578125f,-3.97279995673596797445270567550323903560638427734375f,1.197400067469476692139096485334448516368865966796875f,-2.67590001007392874754486911115236580371856689453125f,-1.68689995614290122460943166515789926052093505859375f,0.90470007181838052456868126682820729911327362060546875f,-0.4562000285938392352136361296288669109344482421875f,-2.251800112311542534371255896985530853271484375f,-4.2849000327497730467030123691074550151824951171875f,-3.584399891471862442671181270270608365535736083984375f,2.89309983434677153724123854772187769412994384765625f,-0.2265000344850125468809665107983164489269256591796875f,-0.2269001348078252267015386678394861519336700439453125f,2.62410004615932646032661068602465093135833740234375f,-0.55220006759017525865829156828112900257110595703125f,0.7568000465974213941677817274467088282108306884765625f,2.626300012160836416796882986091077327728271484375f,-0.7039999202303592884533145479508675634860992431640625f,2.964299926289169828663716543815098702907562255859375f,-1.349599945479631468714387665386311709880828857421875f,1.135499961122869994056827636086381971836090087890625f,2.486500000510364127848106363671831786632537841796875f,-0.8810000159740429825916407935437746345996856689453125f,-0.06699983286857580377926524306531064212322235107421875f,1.78050001374259370123809276265092194080352783203125f,-2.00999992903322066695182002149522304534912109375f,0.0801000629901889027451034053228795528411865234375f,1.5921999641388644430861631917650811374187469482421875f,-1.6119000755012049541647911610198207199573516845703125f,3.22719983151853018199517464381642639636993408203125f,-1.314599950508772874258056617691181600093841552734375f,0.7854000627532624623228230120730586349964141845703125f,1.448099950128794777270968552329577505588531494140625f,0.31669983660727663021816624677740037441253662109375f,-1.4128999670878046313049480886547826230525970458984375f,-0.9838000800728787798021812704973854124546051025390625f,-0.83210006673783087904894273378886282444000244140625f,3.139699967100471500458525042631663382053375244140625f,-4.98910011768490146977228505420498549938201904296875f,-2.090900052635372485809739373507909476757049560546875f,-0.0031999966532001433705545423435978591442108154296875f,-0.0380000552609545128035506422747857868671417236328125f,-1.834899954481425066177280314150266349315643310546875f,-1.9060000359602273789505488821305334568023681640625f,-1.324399926228821744444985597510822117328643798828125f,2.27460005317777369526766051421873271465301513671875f,-0.514699957321585355174420328694395720958709716796875f,-1.7474999303743228296070810756646096706390380859375f,2.66910003286004116063168112304992973804473876953125f,-1.3387000469036396399502564236172474920749664306640625f,-0.700099952740223852032386275823228061199188232421875f,2.27699985913187230579524111817590892314910888671875f,-0.47149994885921497900227450372767634689807891845703125f,2.142399971673636382973882064106874167919158935546875f,-3.954200090411304824300486870924942195415496826171875f,1.408300026208908040104006431647576391696929931640625f,-1.5598999499179428962492011123686097562313079833984375f,-2.658300088793784876628478741622529923915863037109375f,0.715800054353475712076715353759936988353729248046875f,0.762199933964013975895568364649079740047454833984375f,-3.61080007340833386564327156520448625087738037109375f,-1.3884998734816937915326207075850106775760650634765625f,-1.6173000109799213763750458383583463728427886962890625f,0.2144000665605088062903860190999694168567657470703125f,0.58849999794363949678910330476355738937854766845703125f,0.09920009540319529861562841688282787799835205078125f,-3.5252999196320775610047348891384899616241455078125f,0.3771000551149243928961141136824153363704681396484375f,0.13899995705485201913376158699975349009037017822265625f,2.166200011550635462498348715598694980144500732421875f,1.186000045944004543940764051512815058231353759765625f,1.91050000560283717021548000047914683818817138671875f,1.68670001255273849238847105880267918109893798828125f,5.31760011279731958921956902486272156238555908203125f,-2.20980007686168011815652789664454758167266845703125f,2.9649999577179553256200961186550557613372802734375f,-2.053899996864051669120954102254472672939300537109375f,0.8296999535590410257412941064103506505489349365234375f,0.02560006842538775639894765845383517444133758544921875f,1.255100060233473424631256420980207622051239013671875f,0.518199998739361955557569672237150371074676513671875f,-2.180100041476638050852443484473042190074920654296875f,1.1805999103054405363621981450705789029598236083984375f,-2.897299953945726702642105010454542934894561767578125f,-1.6722999993942675445168788428418338298797607421875f,-1.1413000569388256355551902743172831833362579345703125f,3.955100062599033350352328852750360965728759765625f,1.3745999098658554515850482857786118984222412109375f,-2.62130005697607781911528945784084498882293701171875f,0.81270003277361346771812122824485413730144500732421875f,0.13649994278699129068854745128192007541656494140625f,-0.0548000768803067472845214069820940494537353515625f,-2.042599955478310302936506559490226209163665771484375f,-2.400400001025198459814191664918325841426849365234375f,1.45350004316866421305576295708306133747100830078125f,2.0726999537229513492775367922149598598480224609375f,-3.676699820923803319061562433489598333835601806640625f,0.059100036138297007415332018354092724621295928955078125f,-1.28179998982399734330783758196048438549041748046875f,-0.98300003074854569451446195671451278030872344970703125f,-0.4120998725757007985492919033276848495006561279296875f,-0.38290003609880762791561892299796454608440399169921875f,-0.297599934225529716425029391757561825215816497802734375f,-0.11160010773837658604179523536004126071929931640625f,-1.040000077672302847275886961142532527446746826171875f,-2.461000027168543713429471608833409845829010009765625f,3.958399975968898143463547967257909476757049560546875f,-0.932400004810094884533100412227213382720947265625f,1.0115999905034895878230827293009497225284576416015625f,-3.07459995512813311080435596522875130176544189453125f,0.62569996189624144111718351268791593611240386962890625f,-3.18709994968920984348415004205889999866485595703125f,1.68459999018311545881942947744391858577728271484375f,-3.300799983796476322339685793849639594554901123046875f,-4.20800000720470990245303255505859851837158203125f,-0.9926999096341428963086173098417930305004119873046875f,-3.83829997450187843099911333411000669002532958984375f,0.0797000278040760345987791879451833665370941162109375f,-3.5962000399559741481425589881837368011474609375f,-1.138999930605291144303237160784192383289337158203125f,-1.035499885462225932997171184979379177093505859375f,3.47839991152137617547168702003546059131622314453125f,3.359100106248258388319527512066997587680816650390625f,0.890299951241166009907601619488559663295745849609375f,0.9692999444499601491287421595188789069652557373046875f,0.06339992794990501057128540196572430431842803955078125f,-0.0955000174120061995353125894325785338878631591796875f,-0.81849996873736241553842774010263383388519287109375f,1.153900051756201694530545864836312830448150634765625f,-1.64349998689815368635436243494041264057159423828125f,-2.169599960008264272204314693226478993892669677734375f,2.789200001190603739331663746270351111888885498046875f,-5.40570018228441373508985634543932974338531494140625f,-0.679599921730905887073959092958830296993255615234375f,2.82550005095824641188073655939660966396331787109375f,3.120999909207224476404007873497903347015380859375f,1.425699926804004480374032937106676399707794189453125f,2.02689987943917504509272475843317806720733642578125f,-0.000699998068810270268613749067299067974090576171875f,3.35959993162155168278104611090384423732757568359375f,0.668900010964274205349511248641647398471832275390625f,-0.5075998982951037685751316530513577163219451904296875f,0.293900077162682737963450563256628811359405517578125f,2.206999929018318429285727688693441450595855712890625f,0.0204000589162125489650634335703216493129730224609375f,2.77589998986423136528856048244051635265350341796875f,1.3008998797073960673742476501502096652984619140625f,1.810399956526607123663552556536160409450531005859375f,0.6238000119186930536585578010999597609043121337890625f,1.5637999295711517699913883916451595723628997802734375f,0.343499981962144484981536152190528810024261474609375f,1.04899990136176679556001545279286801815032958984375f,2.004299981181322909407072074827738106250762939453125f,-3.016999879956246299883559913723729550838470458984375f,-1.17330009950399460905146042932756245136260986328125f,-2.27849996640905683165101436316035687923431396484375f,-2.891800001409649656380906890262849628925323486328125f,-3.15239999981820684382682884461246430873870849609375f,-0.73320005502849838752155164911528117954730987548828125f,1.156199995103481370506415260024368762969970703125f,0.735400000615415994076329297968186438083648681640625f,0.8346999815918485499111056924448348581790924072265625f,1.4334999302551150179851902066729962825775146484375f,2.162100013242661322721005490166135132312774658203125f,-0.7312000427126876811456668292521499097347259521484375f,0.447100017061085264202802136423997581005096435546875f,1.6464000300750136762673037083004601299762725830078125f,-4.1453000348553050713462653220631182193756103515625f,-0.829200014825165254706007544882595539093017578125f,-3.49439999235272491517889648093841969966888427734375f,0.0829999883919942949006554044899530708789825439453125f,1.1258000137105594884445736170164309442043304443359375f,0.4889000296279795154674729928956367075443267822265625f,-0.651000036500394063665453359135426580905914306640625f,-1.356300001357495066400815630913712084293365478515625f,3.61709995281845397840925215859897434711456298828125f,0.388399998655914568956859511672519147396087646484375f,1.5889999839328228237178564086207188665866851806640625f,-1.3684000140786187760255643297568894922733306884765625f,2.9742999576374895553954047500155866146087646484375f,-3.930100016964227283011723557137884199619293212890625f,1.6562000700190664392863482134998776018619537353515625f,-2.649300026909261251972793616005219519138336181640625f,-2.9501999068960547134565786109305918216705322265625f,3.412099946112931547759217210114002227783203125f,0.7892000110998760309399813195341266691684722900390625f,0.5358000166535370478726463261409662663936614990234375f,-1.14540003079026941890106172650121152400970458984375f,-1.0794000001013301925922860391438007354736328125f,-2.443200037258864032452265746542252600193023681640625f,-1.964499967947601621887088185758329927921295166015625f,-0.850499962016938670927856946946121752262115478515625f,0.9576000676468032768440252766595222055912017822265625f,2.700299927175787928490535705350339412689208984375f,1.0430000996105370791866562285576947033405303955078125f,1.86620007232874485225693206302821636199951171875f,-2.37920008327737431130799450329504907131195068359375f,-0.60719997976645867510114840115420520305633544921875f,0.164900052605569502617299804114736616611480712890625f,0.6364000461310144185489434676128439605236053466796875f,1.765599937518686157744696174631826579570770263671875f,-0.43520004465729034137666531023569405078887939453125f,0.17520005227550949911119460011832416057586669921875f,1.1853999400049433088355499421595595777034759521484375f,3.93710000631362344591934743220917880535125732421875f,-0.951999930266291816138846115791238844394683837890625f,-0.42079998416900654678585169676807709038257598876953125f,-1.1838000144891449583184339644503779709339141845703125f,3.8041000534608944150249953963793814182281494140625f,-1.86110002908259541953839288908056914806365966796875f,-1.669000033665448246011919763986952602863311767578125f,-2.64679991561621275053539648070000112056732177734375f,-0.97579992545843108331382609321735799312591552734375f,-2.217000011943280224357977203908376395702362060546875f,1.299199964682759667056188845890574157238006591796875f,-3.477700009620935617249415372498333454132080078125f,-1.550499997407197572130144180846400558948516845703125f,0.3921000202462086026145016148802824318408966064453125f,-2.127599947115034861866433857358060777187347412109375f,-0.265200042645632461102422894327901303768157958984375f,-0.2487999160483485017181237708427943289279937744140625f,-0.0557999846905474772285060680587776005268096923828125f,2.279199924486875250551065619220025837421417236328125f,-0.81689999361932164045896342940977774560451507568359375f,-1.2175999973133209142162058924441225826740264892578125f,-1.69650000395625877303018569364212453365325927734375f,-0.27610021924004002347174946407903917133808135986328125f,-2.1417000475704668360776850022375583648681640625f,-0.85809992118328815013938992706243880093097686767578125f,-0.71960003926381421734959076275117695331573486328125f,-1.229500103063882132659045964828692376613616943359375f,-0.97979998670965517959530188818462193012237548828125f,0.30820004372224152344728054231381975114345550537109375f,0.427899911431969481867554350174032151699066162109375f,-3.529899976069479361484582113916985690593719482421875f,-1.78670007849037570935024632490240037441253662109375f,0.27280000760480704879995528244762681424617767333984375f,3.42590004267022063544345655827783048152923583984375f,2.55739992158711260117343044839799404144287109375f,-1.9658000655665990308307300438173115253448486328125f,-1.745400122357905647874076748848892748355865478515625f,-0.9964999558255076550494777620770037174224853515625f,-0.79899999054521286812047264902503229677677154541015625f,-0.7329999670833353686560940332128666341304779052734375f,-0.01420008080005541462043083811295218765735626220703125f,-1.59190003500729648777678448823280632495880126953125f,-1.719199971053004816212705918587744235992431640625f,2.72760007893443212623196814092807471752166748046875f,0.10040007717013299526342962053604423999786376953125f,-1.393799939741194204856356009258888661861419677734375f,3.05159994369521836432568306918255984783172607421875f,-0.00709998150318791321211620015674270689487457275390625f,3.25509984755665104927402353496290743350982666015625f,2.141999961055815138166735778213478624820709228515625f,-0.7671999939411879854134213019278831779956817626953125f,-0.267899916740506061074711396940983831882476806640625f,0.55009998248666658327010736684314906597137451171875f,-1.13540003492534236073652209597639739513397216796875f,1.0086000216692665443218857035390101373195648193359375f,3.31759992824643656916805412038229405879974365234375f,1.4027000157304119198897751630283892154693603515625f,0.5539000404499463403595882482477463781833648681640625f,2.603700007843969377319126579095609486103057861328125f,-1.6826999288752677585989658837206661701202392578125f,2.485299912572653369124964228831231594085693359375f,3.44959996283948289175214085844345390796661376953125f,-0.993599956551195706566659282543696463108062744140625f,-0.2089000411763792808272910406230948865413665771484375f,-3.093700049939752005911941523663699626922607421875f,3.2214001225180926013536009122617542743682861328125f,-2.725600009770690501653689352679066359996795654296875f,0.751499963350592548039230678114108741283416748046875f,3.239899985624850042853495324379764497280120849609375f,2.574400004646182349432592673110775649547576904296875f,-2.56049994478747322546041687019169330596923828125f,-2.214499901860952579824015629128552973270416259765625f,2.660900059676170936739936223602853715419769287109375f,-3.49870000244230094921249474282376468181610107421875f,-0.41010014565438013267595351862837560474872589111328125f,2.50670003965422427683051864732988178730010986328125f,4.1583000144928679020495110307820141315460205078125f,-0.8510000535622257444856586516834795475006103515625f,-4.233199889569728924243463552556931972503662109375f,-0.4244999014139156923164364343392662703990936279296875f,-0.2116999072015295357829245403991080820560455322265625f,-4.05050005112588618061408851644955575466156005859375f,-1.0740999989226451827306618724833242595195770263671875f,2.991100012877583314008234083303250372409820556640625f,0.76040002717673882415283515001647174358367919921875f,-2.60559999255985008659308732603676617145538330078125f,-0.92140008712783572075721849614637903869152069091796875f,0.54989997374117327577636160640395246446132659912109375f,1.1646000460810963517133131972514092922210693359375f,1.386699982377887607754018972627818584442138671875f,0.40479997732192185555533114893478341400623321533203125f,-0.80290004585906871170664089731872081756591796875f,-3.1474998663738364967912275460548698902130126953125f,-0.3830999722570183063652393684606067836284637451171875f,0.076200089465082221362735026559676043689250946044921875f,-2.72050001201033619935287788393907248973846435546875f,-1.824999981708824048354244951042346656322479248046875f,4.6616000333443281533618574030697345733642578125f,-2.58699989306926791954310829169116914272308349609375f,2.711100089991094197472421001293696463108062744140625f,-2.464099988640845229070919231162406504154205322265625f,-0.396699973995985555319521154160611331462860107421875f,-3.61979993999451465924721560440957546234130859375f,1.92619993255585431057852474623359739780426025390625f,3.219200073833762498765054260729812085628509521484375f,2.60059997546076804297854323522187769412994384765625f,-1.063600004830955736423447888228110969066619873046875f,0.958000019665806501478755308198742568492889404296875f,1.6160000541210164382022185236564837396144866943359375f,1.3034999950379144717516055607120506465435028076171875f,1.083500005245208175352900070720352232456207275390625f,-0.626099866212905364903917870833538472652435302734375f,-0.4521000332847238123434863155125640332698822021484375f,-0.75810012778788848919475640286691486835479736328125f,0.8832999724343413294747051622834987938404083251953125f,0.62840000506341520125630495385848917067050933837890625f,-2.66059992177933413159962583449669182300567626953125f,-4.120499991819261964565157541073858737945556640625f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,3,5,8,7};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,1,4,2,7};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1,1},        // move_strides
                                                                  Strides{2,1,3,2},        // filter_dilation
                                                                  CoordinateDiff{0,0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0,0}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{5.41479995798766555736847294610925018787384033203125f,0.30130006185621038827804341053706593811511993408203125f,-1.75299994853138851880203219479881227016448974609375f,3.747399964688718387861854353104718029499053955078125f,-0.210899899507314969593352316223899833858013153076171875f,0.641799988147616407019313555792905390262603759765625f,-0.133599968323112161527888019918464124202728271484375f,-2.010100053285807319980449392460286617279052734375f,1.2793999854795636839099870485370047390460968017578125f,-2.19269993613958380507256151759065687656402587890625f,0.2436999820649623504920100458548404276371002197265625f,1.705799902510642507280635982169769704341888427734375f,2.019200092571974192168227091315202414989471435546875f,-5.2607999584831315331712175975553691387176513671875f,2.328800053250790202952202889719046652317047119140625f,-0.697699988386780045601653910125605762004852294921875f,1.9473001255139708032260159598081372678279876708984375f,1.0302999429523946872677697683684527873992919921875f,-0.3663000594906498097458324991748668253421783447265625f,0.05919994668960659733869533738470636308193206787109375f,-0.6459001095578076334646766554214991629123687744140625f,-2.39479999244660124446681948029436171054840087890625f,-1.7706000082060688871621323414728976786136627197265625f,0.94689999924451218049625822459347546100616455078125f,-0.0219999662712214438808899785726680420339107513427734375f,-0.526600013488530027672140931827016174793243408203125f,1.670400034888089191298377045313827693462371826171875f,-1.402400049383192737195713561959564685821533203125f,0.4710000417158013430452001557569019496440887451171875f,-2.25430006619244949206404271535575389862060546875f,1.19439996020346850258420090540312230587005615234375f,-2.357199991519749904256286754389293491840362548828125f,0.2725000304356222091684003316913731396198272705078125f,-0.0235000710710882021459156021592207252979278564453125f,2.382299978290498554400755892856977880001068115234375f,0.69960008316636201897154023754410445690155029296875f,-0.7273999448329206973795635349233634769916534423828125f,-0.8146999801948660024919490751926787197589874267578125f,0.209899940194935741288873032317496836185455322265625f,1.5872999544113868264361144611029885709285736083984375f,3.032900011269747064801549640833400189876556396484375f,-0.710700015167891052669801865704357624053955078125f,-0.794099990540742428635212490917183458805084228515625f,1.302200020167231908629901226959191262722015380859375f,-1.4313999820560214804032739266403950750827789306640625f,1.056600142979623679906353572732768952846527099609375f,1.490299966868757763904795865528285503387451171875f,3.915399987316131369396998707088641822338104248046875f,1.516400064049661988718753491411916911602020263671875f,1.8894998857676974512997958299820311367511749267578125f,2.02459998452067235774620712618343532085418701171875f,0.75429998812899101690021552713005803525447845458984375f,-1.7806000191956756761868518879055045545101165771484375f,0.172200004800409534055916083161719143390655517578125f,1.34449992462247536195718566887080669403076171875f,-2.70039993460327565344414324499666690826416015625f,-2.140899965500831836351380843552760779857635498046875f,-1.574900069984048744942128905677236616611480712890625f,3.4535999897062783503542959806509315967559814453125f,-0.7731999817706654010152078626560978591442108154296875f,0.44089999608546381892892895848490297794342041015625f,-1.2195000524371868433348709004349075257778167724609375f,-1.47140005682259822350488320807926356792449951171875f,2.174099991043658253175863137585110962390899658203125f,2.274599976194650707839173264801502227783203125f,0.679899979738890269942430677474476397037506103515625f,2.078300055955349723291192276519723236560821533203125f,1.138100050245970162876574249821715056896209716796875f,0.077100002178549065234847148531116545200347900390625f,1.2384000085279340819255367023288272321224212646484375f,0.6593999662756926394280299064121209084987640380859375f,0.582099953414501047888052198686636984348297119140625f,1.19530001775622363169304662733338773250579833984375f,-2.037899977593124134500612854026257991790771484375f,-0.70819997314661609078001447414862923324108123779296875f,-1.1138999827638269568552686905604787170886993408203125f,1.0089000473976146299293077390757389366626739501953125f,-0.571199981133640566355325063341297209262847900390625f,-3.2688998774498703170365843106992542743682861328125f,-0.8958000073030587540046099093160592019557952880859375f,-1.9022000050425524886321682060952298343181610107421875f,-4.84659992553442720009115873835980892181396484375f,-0.6538000128686434209157596342265605926513671875f,2.58529997720644022507485715323127806186676025390625f,-1.8119000846281652084002189440070651471614837646484375f,2.32770006663650264044918003492057323455810546875f,-1.9105000767186257615293243361520580947399139404296875f,0.61980002757608776420283902552910149097442626953125f,3.3446999966606494325560561264865100383758544921875f,-0.50189999764263648618367597009637393057346343994140625f,1.15419998509735055591818309039808809757232666015625f,3.275900036616624877439107876853086054325103759765625f,1.67069998633414318334189374581910669803619384765625f,-0.44140003573745456577626100624911487102508544921875f,0.66290012619495375378875223759678192436695098876953125f,-1.224800002281367650169840999296866357326507568359375f,-0.069400015151499960808223477215506136417388916015625f,-4.1518000242643058328440019977279007434844970703125f,-2.58349998311698225705868026125244796276092529296875f,3.055000020973383190181493773707188665866851806640625f,0.8129999996423717956162136033526621758937835693359375f,-2.00320006937086514398060899111442267894744873046875f,-0.58450005012750583244240942804026417434215545654296875f,1.246099961505830133745575949433259665966033935546875f,-0.6494999707415700651580436897347681224346160888671875f,-1.643400004709511197376059499219991266727447509765625f,-0.363800008547305875339361591613851487636566162109375f,1.400699956960230974090109157259576022624969482421875f,-0.1703999361678960600130494640325196087360382080078125f,-3.62299989742040562390457125729881227016448974609375f,-1.81149998059868888589107882580719888210296630859375f,1.81740000937357581989317623083479702472686767578125f,1.46939985131323336275954716256819665431976318359375f,-1.2180999560147547366995013362611643970012664794921875f,-0.4930000199452049258752595051191747188568115234375f,-3.76749998766928850812973905703984200954437255859375f,1.005899948643892205524252858594991266727447509765625f,0.064599966751038906398463268487830646336078643798828125f,1.268599995741249930603089524083770811557769775390625f,0.53100011873617791735568971489556133747100830078125f,-2.67610001738518388236798273283056914806365966796875f,2.886899982555209742685065066325478255748748779296875f,1.0812999961979687935098581874626688659191131591796875f,1.08890005302280190591091013629920780658721923828125f,-3.54959995605945533014846660080365836620330810546875f,-0.9325999381184590930615740944631397724151611328125f,0.5962999544933451634420862319529987871646881103515625f,-4.3542999694094053353410345152951776981353759765625f,1.692599979431927526007939377450384199619293212890625f,0.5349999308958641730527006075135432183742523193359375f,-1.56929992377460081343087949790060520172119140625f,0.0535999390795820485067224581143818795680999755859375f,-0.2209999805465334699050572453415952622890472412109375f,0.26150013992935239315329454257152974605560302734375f,-1.1677001171141860869084894147817976772785186767578125f,-4.113799892504513167068580514751374721527099609375f,-0.612400039548426367019828830962069332599639892578125f,-1.3652000375419846012192692796816118061542510986328125f,3.514700060512121471134605599218048155307769775390625f,1.00130001301765592103265589685179293155670166015625f,1.4168000124365069947174333719885908067226409912109375f,-3.75679996717423136942670680582523345947265625f,2.949600051575897996514186161221005022525787353515625f,-1.69720000101924117785756607190705835819244384765625f,-0.4175000187382116312306834515766240656375885009765625f,-1.964800078649819869980319708702154457569122314453125f,0.699799857032299410519726734491996467113494873046875f,0.3154000279590480904090554759022779762744903564453125f,0.04779994503855478082954277851968072354793548583984375f,-1.38779994406998152811638647108338773250579833984375f,-0.7203999917492260873785880903596989810466766357421875f,-0.30569996320009196200118140041013248264789581298828125f,4.3962999796763053694803602411411702632904052734375f,2.143300012853741609575308757484890520572662353515625f,-2.41419997146278575428368640132248401641845703125f,1.2787999369472264898917046593851409852504730224609375f,-2.5632000874385223454510196461342275142669677734375f,-0.2598999624535467400932020609616301953792572021484375f,0.38629998463094306071496930599096231162548065185546875f,-0.034200099091231894732345608645118772983551025390625f,-1.45160003766566347849220619536936283111572265625f,2.791299956393242798213805144769139587879180908203125f,-0.6894999524876486152180632416275329887866973876953125f,2.676999951500445007468442781828343868255615234375f,-1.53559987949728959932826910517178475856781005859375f,-0.47080004144534226551144229233614169061183929443359375f,3.9745998862102620563518939889036118984222412109375f,-0.936399936439842317525972248404286801815032958984375f,0.3836999735340473893785429027047939598560333251953125f,-1.5048999953478567892517503423732705414295196533203125f,1.234799976241589103409523886512033641338348388671875f,-2.3085000640302890673183355829678475856781005859375f,2.830400015423446546947161550633609294891357421875f,-3.89150010774284549341928141075186431407928466796875f,1.1195999209299676291351488544023595750331878662109375f,0.146000012733040751555790848215110599994659423828125f,-1.5884999891892082057864854505169205367565155029296875f,-1.648299982827901910553691777749918401241302490234375f,1.1877000801034267585265524758142419159412384033203125f,0.7917999948903926554777399360318668186664581298828125f,-0.2109000631034376738170976750552654266357421875f,1.93170000664815422197762018186040222644805908203125f,-1.01529998437762269958284377935342490673065185546875f,1.12339992983117920033464542939327657222747802734375f,-0.64839993838071763310182404893566854298114776611328125f,-1.0627000050947088549690988656948320567607879638671875f,-1.19979990611299935920897041796706616878509521484375f,0.82090009003728692249524101498536765575408935546875f,0.4762000209942442996435829627444036304950714111328125f,-1.45770004193782742873963798047043383121490478515625f,4.09449993281811419620908054639585316181182861328125f,0.3291999386802333393831077046343125402927398681640625f,-1.4464999186843652267953075352124869823455810546875f,-0.3205999394372103239447824307717382907867431640625f,1.3617999987646947079866777130519039928913116455078125f,0.9234001345358791024153788384865038096904754638671875f,-0.8721999028205871251628877871553413569927215576171875f,3.9314999779537327384559830534271895885467529296875f,-4.28669994374662710612255978048779070377349853515625f,2.916499956347047284310747272684238851070404052734375f,0.2102999679863459814299631034373305737972259521484375f,-1.6160000692456966131516082896268926560878753662109375f,-0.817899909643829747807330932118929922580718994140625f,0.9311002266347421230108238887623883783817291259765625f,-4.14460005988329616144483225070871412754058837890625f,1.8541000801712268497567492886446416378021240234375f,0.41020012126788596429349809113773517310619354248046875f,0.04059986642152113578418948236503638327121734619140625f,-0.137100013019145539061582894646562635898590087890625f,-0.4263000661589193551748167010373435914516448974609375f,-1.41099998158961437155767271178774535655975341796875f,-0.384900035841762910848729006829671561717987060546875f,-1.6420000068396325421105075292871333658695220947265625f,-1.5197000638276338957410871444153599441051483154296875f,-1.612599964959173792777846756507642567157745361328125f,-3.03999999646097496253105418873019516468048095703125f,1.356399923196434098571216964046470820903778076171875f,-2.131699940095841849796443057130090892314910888671875f,1.5948999602556230836825079677510075271129608154296875f,-0.94469992530271251762741258062305860221385955810546875f,0.84670006012469656297980691306293010711669921875f,2.348300006613878831984720818581990897655487060546875f,0.0774999570474041998835446065641008317470550537109375f,0.7786999160155654298165472937398590147495269775390625f,-3.3709999914430088807648644433356821537017822265625f,1.1943000133305787802129316332866437733173370361328125f,-2.507600041271746960092059453018009662628173828125f,1.11709997485354595170292668626643717288970947265625f,1.291999989461154019210198384826071560382843017578125f,-0.96429993375837874669542770789121277630329132080078125f,0.1075999495550983287017743350588716566562652587890625f,0.592999950822440613507069429033435881137847900390625f,-2.315599938971548699129243686911650002002716064453125f,-0.93629995236992780149876125506125390529632568359375f,-0.5905000015981489713823293641326017677783966064453125f,1.495800100398062770779006314114667475223541259765625f,1.799300057438014466271170022082515060901641845703125f,2.709799984586239940398399994592182338237762451171875f,-1.271600095854700906983225650037638843059539794921875f,4.37180005864873511001178485457785427570343017578125f,3.264299956855178930226202282938174903392791748046875f,-1.639599961237608649611274813651107251644134521484375f,-0.92679991183504484286714841800858266651630401611328125f,1.262399942541869268808341075782664120197296142578125f,0.75120005310624760053173076812527142465114593505859375f,0.28309997583329715720168451298377476632595062255859375f,-3.0416999108150601927036404958926141262054443359375f,-1.2263999665774394376427380848326720297336578369140625f,-0.01640006974935459549413963031838648021221160888671875f,-0.330999984681605796676961972480057738721370697021484375f,1.2577000126384210165753074761596508324146270751953125f,0.6922000072777283463665298768319189548492431640625f,0.67519994297549157380444739828817546367645263671875f,1.4291000252790748703546341857872903347015380859375f,3.070800067801774613229781607515178620815277099609375f,0.0993000472493452690514459391124546527862548828125f,-1.976099962753802063986086068325676023960113525390625f,5.05849994275346404037918546237051486968994140625f,0.57750005666166581708154126317822374403476715087890625f,2.121000100091100737387250774190761148929595947265625f,1.7549000267148018306073709027259610593318939208984375f,-0.8480999510042384059715914190746843814849853515625f,-3.78829999854862631281093854340724647045135498046875f,1.74529996971860423826683472725562751293182373046875f,1.75300002799183030077756484388373792171478271484375f,-2.316199970833213495069458076613955199718475341796875f,1.06610005648210570683431797078810632228851318359375f,1.861400081409513074248707198421470820903778076171875f,-0.9176000363357366129690717571065761148929595947265625f,2.01289997967928702138351582107134163379669189453125f,-0.31129992495179148459527596060070209205150604248046875f,0.1663999996207665699188282815157435834407806396484375f,0.794899977131187096546227621729485690593719482421875f,1.24529997383505008912152334232814610004425048828125f,1.5128000217609109956384827455622144043445587158203125f,1.4127998942069714960467763376072980463504791259765625f,1.0227999324657017776729617253295145928859710693359375f,-1.0243000862158828656589548700139857828617095947265625f,0.5915001059919606785086898526060394942760467529296875f,2.2218999904342009443780625588260591030120849609375f,-3.043600010288507018429982053930871188640594482421875f,0.1381000159546734717963545335805974900722503662109375f,0.54070009872615276691476537962444126605987548828125f,-2.91999994361773218543021357618272304534912109375f,0.452799978845567052676557295853854157030582427978515625f,3.467699971790612067934489459730684757232666015625f,4.0724000776767734777195073547773063182830810546875f,-0.9140999139860266353707629605196416378021240234375f,0.8784000741675488388437997855362482368946075439453125f,2.362999952927230307153649846441112458705902099609375f,-2.095999960765243752547348776715807616710662841796875f,2.36449997975677206341060809791088104248046875f,0.3989999617487185457065379523555748164653778076171875f,-2.832399968693405600816959122312255203723907470703125f,1.198000035051256961793342270539142191410064697265625f,0.48079997303038890521520443144254386425018310546875f,2.42410000998675823069561374722979962825775146484375f,3.011400011094659046051447148784063756465911865234375f,-0.0097999918878082237227999939932487905025482177734375f,0.6326000190317635496484172108466736972332000732421875f,1.3320999631375087801643530838191509246826171875f,0.8604000725880276423396253449027426540851593017578125f,0.46279998033568092719036712878732942044734954833984375f,-1.885500113915651976270737577578984200954437255859375f,-0.2546000189796095991567881355877034366130828857421875f,0.34930000915825354379506961777224205434322357177734375f,-0.8459000339902942311454125956515781581401824951171875f,0.99700001283735151247356043313629925251007080078125f,-1.9717000231511885299795494574937038123607635498046875f,-0.17819998553320759437923470613895915448665618896484375f,-0.86360012359321114416843556682579219341278076171875f,-0.60850008489191420579800251289270818233489990234375f,1.526399960803241473428215613239444792270660400390625f,-2.021600011495501281189035580609925091266632080078125f,-1.1012000068008898079341406628373079001903533935546875f,1.0542999322682595764177904129610396921634674072265625f,-0.7977999784171585506697965683997608721256256103515625f,-0.09040001916736317699729852392920292913913726806640625f,0.0321000130191448906913365135551430284976959228515625f,-0.008099980266390360839068307541310787200927734375f,-1.8726000183984634617218034691177308559417724609375f,1.0129000189810997145656301654526032507419586181640625f,-0.413699990707636633402444203966297209262847900390625f,-1.117400140373407513294523596414364874362945556640625f,-3.622800016098469910730273113586008548736572265625f,1.600600065948068806420678811264224350452423095703125f,2.393199952843784839018326238146983087062835693359375f,0.159300071910767560012800458935089409351348876953125f,1.596700014005600909428039813064970076084136962890625f,-5.985900028681754747594823129475116729736328125f,1.195400074841826754123985665501095354557037353515625f,-0.09920000685304442011869241468957625329494476318359375f,-0.73589993724450442069695554891950450837612152099609375f,0.121500062424689136975075598456896841526031494140625f,0.2720999907977870169162315505673177540302276611328125f,-1.639199949502199604722818548907525837421417236328125f,0.629000032715499202140563284046947956085205078125f,0.17489991918429625972208896200754679739475250244140625f,-1.872399930620937791303504127427004277706146240234375f,5.5220999522224065714226526324637234210968017578125f,4.21479992002695791342148368130438029766082763671875f,0.4632998307988052122396993581787683069705963134765625f,-0.7622999087765809367311931055155582726001739501953125f,-3.622900055693835330572483144351281225681304931640625f,-1.9373999918274573683873995832982473075389862060546875f,-1.25340002566426989005776704289019107818603515625f,3.3130998198553935907284540007822215557098388671875f,-0.05709986575841963230004694196395576000213623046875f,0.342399964688716362815057436819188296794891357421875f,0.355600016420336562106285782647319138050079345703125f,-1.81509995851442251790786031051538884639739990234375f,0.87150001040100999460946695762686431407928466796875f,0.3874000970855373537915511406026780605316162109375f,3.20750004319473891456482306239195168018341064453125f,4.73080001553595064223145527648739516735076904296875f,-0.5275999062113445603472428047098219394683837890625f,-1.9076999935835605715084284383920021355152130126953125f,1.6763000595465309405796006103628315031528472900390625f,1.43950006160139931665753465495072305202484130859375f,-0.491400148334355524326610975549556314945220947265625f,1.2978000139191774309210813953541219234466552734375f,1.3548998975589849891321136965416371822357177734375f,2.38209991973787271746232363511808216571807861328125f,0.4178000689789651556083072136971168220043182373046875f,-0.8289999505914746880108623372507281601428985595703125f,2.553699912346153677589200015063397586345672607421875f,0.8839000761568538422352503403089940547943115234375f,-0.550000099427997657386413266067393124103546142578125f,-0.4173999892197552963324369557085447013378143310546875f,0.1953000365503125834720776765607297420501708984375f,-1.16299994212389012915309649542905390262603759765625f,0.915199930998682464888815957237966358661651611328125f,4.81649998123198752608686845633201301097869873046875f,-3.0838001496240483589872383163310587406158447265625f,2.12110011588185987108090557740069925785064697265625f,-1.1172000143371523250834798091091215610504150390625f,-2.351700050736965419417856537620536983013153076171875f,-0.9780999876983447194334075902588665485382080078125f,-0.7422000350311404037739748673629947006702423095703125f,3.15529989280998801604027903522364795207977294921875f,2.510300101910528436377489924780093133449554443359375f,1.733499878585341047454448926146142184734344482421875f,-3.03749990450218465554144131601788103580474853515625f,-2.952800025299937569656094638048671185970306396484375f,0.6281999905064703515478186091058887541294097900390625f,-0.95310007697492959977125792647711932659149169921875f,6.11920001897886489672373500070534646511077880859375f,0.0843999616190795887149533882620744407176971435546875f,-0.42299998308345632125337942852638661861419677734375f,-2.07500001169741121742617906420491635799407958984375f,1.9869000341504794437952341468189842998981475830078125f,1.0717999907739450549115645117126405239105224609375f,1.854800011650473923197068870649673044681549072265625f,-1.874499912273141166707546290126629173755645751953125f,2.3823999712452295085540754371322691440582275390625f,-1.0199000187054305666123354967567138373851776123046875f,-0.123700017604231493617561454811948351562023162841796875f,1.73239995440691796346754927071742713451385498046875f,1.9060999774873266243702119027148000895977020263671875f,-3.050500026632101846502109765424393117427825927734375f,2.2504999989159397699722831021063029766082763671875f,-0.721100101092459144780377755523659288883209228515625f,-0.6285999488584697036941406622645445168018341064453125f,0.411699952091276577537115599625394679605960845947265625f,-0.16589999742656791337225286042666994035243988037109375f,-2.93439994865506914578645591973327100276947021484375f,-3.685899877434967830680534461862407624721527099609375f,-0.202099959449471400230180506696342490613460540771484375f,-2.16929991088509410701590240933001041412353515625f,2.00040004444346042333791046985425055027008056640625f,-0.53299995061755101044553839528816752135753631591796875f,0.8810000714994974391203186314669437706470489501953125f,1.1216999611064786268599391405587084591388702392578125f,0.6243001032590875798433671661769039928913116455078125f,1.758399942627550505136468927958048880100250244140625f,-0.480699983093141991474794849636964499950408935546875f,1.728099975423515122230355700594373047351837158203125f,0.5787999353453512352274401564500294625759124755859375f,-2.1347000129401703105713750119321048259735107421875f,1.051599938945474210783004309632815420627593994140625f,-2.8267999995656314382586060673929750919342041015625f,-0.8097999473147090565561256880755536258220672607421875f,-1.442800032173096713705717775155790150165557861328125f,1.81149997940659535089480414171703159809112548828125f,2.14670006871148988381037270301021635532379150390625f,2.252299951282143997133289303747005760669708251953125f,1.0063999141812318693922634338377974927425384521484375f,-2.868400092011690549753666346077807247638702392578125f,2.139300154537707054913653337280265986919403076171875f,-3.52299997421726818203069342416711151599884033203125f,4.2203000816449520726791888591833412647247314453125f,-1.756800006606429320754614309407770633697509765625f,-0.7483000098735093086332881284761242568492889404296875f,2.6949000928014523736919727525673806667327880859375f,0.24459992830604104785408026145887561142444610595703125f,-1.5819999831467874695789532779599539935588836669921875f,1.8547000212289395104647837797529064118862152099609375f,2.59380008160024910779384299530647695064544677734375f,2.050199983450771146209490325418300926685333251953125f,-1.3655001258179531031800024720723740756511688232421875f,2.176300002735853666990806232206523418426513671875f,1.1554000453189008368326540221460163593292236328125f,0.69240012325570088336235130554996430873870849609375f,2.461000034488737053806062249350361526012420654296875f,-1.2215999765336518212421879070461727678775787353515625f,4.97209996913522456196687926421873271465301513671875f,-0.4924000264912848923160026970435865223407745361328125f,-0.4884000159040080557559804219636134803295135498046875f,-2.2770000442415483377089913119561970233917236328125f,0.5170999700106679863864655999350361526012420654296875f,1.884299986843764873611917209927923977375030517578125f,-1.67310010275393761247642032685689628124237060546875f,-1.218199975772947585284100568969734013080596923828125f,-1.0643999454699464290996502313646487891674041748046875f,-0.38249998895451431391023788819438777863979339599609375f,2.304300063361227390856811325647868216037750244140625f,3.269300082267076135167371830902993679046630859375f,0.6570999793425194024365509903873316943645477294921875f,-1.123200032080710730753025927697308361530303955078125f,-2.56429995242208264016881003044545650482177734375f,1.234399915555865590022222022525966167449951171875f,0.993300020173937259215790618327446281909942626953125f,3.12480004055872573331953390152193605899810791015625f,2.99510002337172664965692092664539813995361328125f,2.7708000506095604720258052111603319644927978515625f,2.3071999487906698078631961834616959095001220703125f,2.3115000117607404916952873463742434978485107421875f,1.526399898609519301118098155711777508258819580078125f,0.4673999937832340467736003120080567896366119384765625f,1.450499912917614597773763307486660778522491455078125f,-1.4224000488244001871862565167248249053955078125f,0.31799996689707199504226764474879018962383270263671875f,-0.69540000303685556293231684321654029190540313720703125f,-0.1693000473424783602904852841675165109336376190185546875f,-1.190800002218037167978081924957223236560821533203125f,-0.171600077377260351596532927942462265491485595703125f,3.017999989025295093370004906319081783294677734375f,3.27740005439370651885155893978662788867950439453125f,0.515900050344317406825211946852505207061767578125f,-2.713700009520350509006902939290739595890045166015625f,-0.9201000383101403112817706642090342938899993896484375f,-1.0282000698737812083294329568161629140377044677734375f,-1.4662000507034356200364300093497149646282196044921875f,-1.5003999990135437059279865934513509273529052734375f,0.189699955719709212331736125634051859378814697265625f,-0.229799980153142957561840376001782715320587158203125f,-0.76119998690783974293339042560546658933162689208984375f,-1.48209994866475280872464281856082379817962646484375f,1.4706999637961402083163875431637279689311981201171875f,-0.862699955864997036769636906683444976806640625f,2.86269996348321331680608636816032230854034423828125f,-1.6901999547287822611707497344468720257282257080078125f,-2.42250002624467075662550996639765799045562744140625f,-2.263400085958092944338204688392579555511474609375f,-0.5069998561516395607640106391045264899730682373046875f,-3.466899920808522939097429116372950375080108642578125f,1.230800016467273660936143642175011336803436279296875f,1.4219000955432650190601862050243653357028961181640625f,-2.614799993303419167745005324832163751125335693359375f,0.027799951334298800276201291126199066638946533203125f,-3.15709999103993244062849043984897434711456298828125f,-1.817499925997108078945529996417462825775146484375f,2.01110000578090630796168625238351523876190185546875f,0.148000057402998763667056891790707595646381378173828125f,-1.2612999293476343520836735478951595723628997802734375f,2.259500010751189780222603076254017651081085205078125f,-0.02369998908713366603251415654085576534271240234375f,1.579800031413139294045322458259761333465576171875f,0.2757000735990715600820522013236768543720245361328125f,2.5441999250270423971187483402900397777557373046875f,1.65270007861330991971726689371280372142791748046875f,-2.48619996198564852107892875210382044315338134765625f,-4.60760007854327557907936352421529591083526611328125f,3.32269993263780971659571150667034089565277099609375f,0.04510005032420139059468056075274944305419921875f,0.3604000217005616235610432340763509273529052734375f,-3.2530999376863238836676828213967382907867431640625f,-1.418200054246187225004405263462103903293609619140625f,-0.0067999909788369894414472582866437733173370361328125f,0.038700020454078387643903624848462641239166259765625f,-3.701600011439619919428878347389400005340576171875f,-2.865000019595027591634561758837662637233734130859375f,2.378599920341371909415784102748148143291473388671875f,-1.24629998191148150255003201891668140888214111328125f,-0.711399965385348220792138818069361150264739990234375f,-0.904600089350341196592353298910893499851226806640625f,2.0810000022836021571492892690002918243408203125f,-1.805399957811832933884943486191332340240478515625f,-1.791900006583333482268471925635822117328643798828125f,-1.969000031933188399335676876944489777088165283203125f,0.793200017477572405510954922647215425968170166015625f,-0.7556999535515902532978316230583004653453826904296875f,-4.52690000753328281035692270961590111255645751953125f,-0.1547000694900757533645219155005179345607757568359375f,-0.1018000104062253274861404861439950764179229736328125f,-2.13080000450909157194701037951745092868804931640625f,-4.162199947132915411884823697619140148162841796875f,-1.515699920620023899431316749542020261287689208984375f,0.0441000677101310856187410536222159862518310546875f,1.6088999953553067001621457166038453578948974609375f,-1.84989992587119278510954245575703680515289306640625f,-0.957999984405935034459389498806558549404144287109375f,0.4228000198423860211960345623083412647247314453125f,-1.0011998948559177602390946049126796424388885498046875f,0.92500006869435225098641240037977695465087890625f,-2.02750002831220488275221214280463755130767822265625f,-1.18559998387992404644819544046185910701751708984375f,-1.376099975810944897602894343435764312744140625f,-0.67919986382052233153672204934991896152496337890625f,2.622100114645063317908579847426153719425201416015625f,-0.9049999427795414330688572590588591992855072021484375f,1.883900021748989406233931731549091637134552001953125f,1.3433000649333013409858494924264959990978240966796875f,1.19479996234625662765438391943462193012237548828125f,1.0147999209955340660371803096495568752288818359375f,0.2106999931886786026069557919981889426708221435546875f,-3.3901999871015551235586826805956661701202392578125f,-0.18880004015639417769278907144325785338878631591796875f,-2.0744999968186039751572025124914944171905517578125f,-2.192999995358289044844468662631697952747344970703125f,2.36469995151013012701923798886127769947052001953125f,-1.533600036477297035020228577195666730403900146484375f,-1.9573999577969285468270754790864884853363037109375f,2.984200035090745384280808139010332524776458740234375f,-1.815799965170026286642723789555020630359649658203125f,-2.37650009302794895660326801589690148830413818359375f,1.368500079397110713586016572662629187107086181640625f,2.58989996815323930690055931336246430873870849609375f,-0.35509999969750793979983427561819553375244140625f,-0.455199839729070188099058213992975652217864990234375f,1.060800012145938442387205213890410959720611572265625f,0.2428000218540413701617808328592218458652496337890625f,0.88859999939203226926309753253008238971233367919921875f,1.751600093284249481939696124754846096038818359375f,-0.1190999799050380669740434314007870852947235107421875f,0.08609993989914688228282102500088512897491455078125f,4.650799992439150543077630572952330112457275390625f,2.70429997756779183504249886027537286281585693359375f,0.4209000430986282736967041273601353168487548828125f,0.333400028309225771039336905232630670070648193359375f,4.02930004285350396031617492553777992725372314453125f,-0.47149994785338711356104113292531110346317291259765625f,-0.5856000353075565545424296942655928432941436767578125f,0.045100054906308084667898583575151860713958740234375f,-0.6880000135377033476657970822998322546482086181640625f,-2.145000013336539002040126433712430298328399658203125f,1.257699998202918667544736308627761900424957275390625f,4.258700028854608632400413625873625278472900390625f,3.97839994855076017898909412906505167484283447265625f,0.805400096876919047161891285213641822338104248046875f,1.2124000030778347980486842061509378254413604736328125f,-0.2039999756999308910820900564431212842464447021484375f,2.694499946359545372587263045716099441051483154296875f,-1.162400004512070150752833797014318406581878662109375f,0.974400040576606674136428409838117659091949462890625f,2.276899950070678801949952685390599071979522705078125f,0.02260007431507293773620403953827917575836181640625f,1.7298999773174525973473691919934935867786407470703125f,-0.121599982233344672266639463487081229686737060546875f,0.8954000586926931237741200675372965633869171142578125f,2.4292000853262845083690990577451884746551513671875f,3.0925000392459338627304532565176486968994140625f,1.5378999962568300485798999943654052913188934326171875f,-1.347799999912083546149688118021003901958465576171875f,4.0771999317847207322529357043094933032989501953125f,0.59840007353425084257736443760222755372524261474609375f,-1.085300001066924924231216209591366350650787353515625f,-1.7633000893339509929091946105472743511199951171875f,2.3546000526189825308165382011793553829193115234375f,2.954600039766727359591413915040902793407440185546875f,0.211200020323692339019316932535730302333831787109375f,-1.332599911557138749884643402765505015850067138671875f,-0.6289999247565882800614645020687021315097808837890625f,-0.43089999968037062672010506503283977508544921875f,1.08290004643648796189836502890102565288543701171875f,1.091999991118905999343269286327995359897613525390625f,2.6176000591717656362789057311601936817169189453125f,0.131000107411295640957860086928121745586395263671875f,1.176000027485189836085055503644980490207672119140625f,2.61960009575784358304417764884419739246368408203125f,0.4442999820195148164003740021144039928913116455078125f,0.28499987998977049930005023270496167242527008056640625f,4.202000013805925249243955477140843868255615234375f,1.6737998726114631242722907700226642191410064697265625f,-5.49900002990290648341442647506482899188995361328125f,-3.645300040331481472577479507890529930591583251953125f,0.5828001041457040454218940794817171990871429443359375f,0.3654998880699285024320488446392118930816650390625f,-3.1434998883083462573040378629229962825775146484375f,-1.55819997553080202834507872466929256916046142578125f,-0.87669995549991686090152143151499330997467041015625f,-0.87880005390271431764404042041860520839691162109375f,1.0361999875970193674135089167975820600986480712890625f,-0.4202999821797013257906883154646493494510650634765625f,0.3126000295184556687644317207741551101207733154296875f,1.51710004829764155687144011608324944972991943359375f,-0.25140005585402303012187985586933791637420654296875f,-2.32429993610531138159558395273052155971527099609375f,1.68180005518421626931058199261315166950225830078125f,0.588099946142734086862446929444558918476104736328125f,-0.532200084093212399949379687313921749591827392578125f,-3.92809999911636253955293796025216579437255859375f,1.5082000243321065369173084036447107791900634765625f,3.1426000635489828027857583947479724884033203125f,1.37499996624887099727629902190528810024261474609375f,-0.9321000534199168630067333651822991669178009033203125f,2.775399975693225496087279680068604648113250732421875f,-0.0917999695956692729481574133387766778469085693359375f,1.294999967329204526578223521937616169452667236328125f,3.586099977282434014824730184045620262622833251953125f,3.387100059678405106211585007258690893650054931640625f,-1.7987000167846673992499972882797010242938995361328125f,-3.162700033704935353995324476272799074649810791015625f,-1.71419998695999442617221575346775352954864501953125f,-1.72110001317560712408294421038590371608734130859375f,5.44529996463358401115328888408839702606201171875f,0.0219999791979799130814399177324958145618438720703125f,1.94010007532089989723544931621290743350982666015625f,-4.5222000642746689180739849689416587352752685546875f,-0.46239991763830168292059852319653145968914031982421875f,1.41450000102817963210100060678087174892425537109375f,2.681800062634797310323619967675767838954925537109375f,0.2199999745935203865343510187813080847263336181640625f,-3.265599976522475689222346773021854460239410400390625f,2.989100176507234696288151098997332155704498291015625f,0.627600107954444208502309265895746648311614990234375f,0.589799978253245260617632084176875650882720947265625f,-1.40229980469197013093207715428434312343597412109375f,-0.3642001176804308126833120695664547383785247802734375f,-2.613300021328778033335993313812650740146636962890625f,1.0486999007046211573879190837033092975616455078125f,0.8203999250851570224085662630386650562286376953125f,-2.1355000172071161301801112131215631961822509765625f,0.219400131529570163735343157895840704441070556640625f,1.91160010505616728693212280631996691226959228515625f,0.046599925739318592832205467857420444488525390625f,4.00739996707290746513763224356807768344879150390625f,0.6206000426277515913398019620217382907867431640625f,-1.2495999796405421466971574773197062313556671142578125f,-2.460100042575598333627340252860449254512786865234375f,-2.26809996741414199306063892436213791370391845703125f,1.6416000427693135943485458483337424695491790771484375f,1.71170010730549648769738269038498401641845703125f,1.18179999507665467461947628180496394634246826171875f,-0.9457000068791210356522469737683422863483428955078125f,-2.42570003934502675946305316756479442119598388671875f,1.7509999493658536806606207392178475856781005859375f,-1.5202000846669090350360420416109263896942138671875f,-0.5070001276507969389939489701646380126476287841796875f,-0.92800005889311509843508929407107643783092498779296875f,2.188499892666935675578088194015435874462127685546875f,0.552399948222934700226005588774569332599639892578125f,-0.6971999267183253135016229862230829894542694091796875f,-0.958999876849351284846534326788969337940216064453125f,-0.426200011681019841347506371675990521907806396484375f,-1.038599983075260357878732975223101675510406494140625f,-1.16150003388896561062892942572943866252899169921875f,1.859200079943240435653706299490295350551605224609375f,-2.14060005194097957570420476258732378482818603515625f,0.1662999348983174030536247300915420055389404296875f,2.12799994504824230290296327439136803150177001953125f,0.3891999515697375233713728448492474853992462158203125f,-1.34790004312098066208136515342630445957183837890625f,-4.60279998231008580233947213855572044849395751953125f,2.77959991724193056228386922157369554042816162109375f,1.0051000365406295156134319768170826137065887451171875f,-0.220699988532066182500557260937057435512542724609375f,-3.2693999687902621786861345754005014896392822265625f,1.153299970050155831557958663324825465679168701171875f,-3.925000066142529941970451545785181224346160888671875f,0.151300055802613631072972566471435129642486572265625f,-0.589399985870718179370442157960496842861175537109375f,-1.2741001161575322697672163485549390316009521484375f,2.115999896876513464150093568605370819568634033203125f,0.466200020733474662648632147465832531452178955078125f,0.27109995077699389209868741090758703649044036865234375f,-2.5638000177487736408465934800915420055389404296875f,2.312400063520669402095109035144560039043426513671875f,1.670699949714542054124422065797261893749237060546875f,0.8805000565275544754939573977026157081127166748046875f,0.48090000208318173235966241918504238128662109375f,1.51629997392073345707785847480408847332000732421875f,1.277300070044398605517699252231977880001068115234375f,-2.1677999819934345993033275590278208255767822265625f,2.08460000433921788243196715484373271465301513671875f,-0.303700067858396760556161098065786063671112060546875f,1.802800016750395695197539680521003901958465576171875f,-1.7338999931573848556354278116486966609954833984375f,3.292400047483295733030672636232338845729827880859375f,3.72120003056824089071596972644329071044921875f,-3.154500090137124512779109863913618028163909912109375f,-1.890499906677752139927406460628844797611236572265625f,0.2874998925812539685153979007736779749393463134765625f,-0.38430002513974892419668094589724205434322357177734375f,-1.9503999496720723083598159064422361552715301513671875f,-0.27419996226131948535709170755580998957157135009765625f,0.66999999798834408704095721986959688365459442138671875f,-0.3563000177860258244066926636151038110256195068359375f,1.0813998610213391149414974279352463781833648681640625f,-0.7820000218152987070396875424194149672985076904296875f,-0.07060002185925828843693352609989233314990997314453125f,-1.129900061117857124060037676827050745487213134765625f,1.982499975375831891000188988982699811458587646484375f,-1.3494000083714718929428499905043281614780426025390625f,-0.041000000253319957010944563080556690692901611328125f,0.87480004320368232040294742546393536031246185302734375f,-0.50790004321411197363289602435543201863765716552734375f,1.482000025913119856113553396426141262054443359375f,1.2041999157510694384853877636487595736980438232421875f,-0.433999939024448000424172278144396841526031494140625f,0.39399993224442042549782172500272281467914581298828125f,-4.42560006535202088429059585905633866786956787109375f,-0.965700016620754997376252504182048141956329345703125f,0.5092999745875588235577424711664207279682159423828125f,0.7181999571651218605694566576858051121234893798828125f,0.9483001166030768924741778391762636601924896240234375f,-1.21799993504583881076541729271411895751953125f,-2.286399877151847448430999065749347209930419921875f,-1.292199937875569304424061556346714496612548828125f,0.864899948614091584886409691534936428070068359375f,-0.527899995921549791688676123158074915409088134765625f,-1.810599944615363998678958523669280111789703369140625f,-0.699299944841117682159392643370665609836578369140625f,0.77550004397332639172191193210892379283905029296875f,0.195799975526332215025604455149732530117034912109375f,0.4919999819546936326020158958272077143192291259765625f,4.21410005128160047860319536994211375713348388671875f,1.80079999068826257513364907936193048954010009765625f,-1.1966999629318706421798879091511480510234832763671875f,-1.6580999920755614684964029947877861559391021728515625f,0.137599916008861011817998587503097951412200927734375f,1.170399923110754247801423844066448509693145751953125f,0.4468999241799120891727170601370744407176971435546875f,-1.789300028492510019617611760622821748256683349609375f,-1.277099943616985644467831662041135132312774658203125f,1.6688000156626101588841493139625526964664459228515625f,-1.37370000471472675229733795276843011379241943359375f,-2.095599908237904429597620037384331226348876953125f,1.4409999793544419066648742955294437706470489501953125f,1.1104000190742298048718339487095363438129425048828125f,-1.619500006243586920362531600403599441051483154296875f,-0.7544000542111695128966175616369582712650299072265625f,-0.874299995784461270176279867882840335369110107421875f,-0.3824999203160412974966675392352044582366943359375f,-1.70819997120946798219165430054999887943267822265625f,-1.888200137543678547302761216997168958187103271484375f,2.084900144545736022649862206890247762203216552734375f,-0.50030001201927465359631241881288588047027587890625f,-0.4738000597327953755666385404765605926513671875f,2.1036000946849586767939399578608572483062744140625f,-0.2777001162387431332234655201318673789501190185546875f,7.15929992142766824514410473057068884372711181640625f,0.89730005275905189154883601077017374336719512939453125f,-0.54199990916252271944131280179135501384735107421875f,-0.267999954491854541771544973016716539859771728515625f,1.8033999359220278524418290544417686760425567626953125f,-1.7232999688021859441278138547204434871673583984375f,1.576800033484399676808607182465493679046630859375f,-1.1257000722020855665306271475856192409992218017578125f,0.2926999328613291151413022816996090114116668701171875f,0.6369001622632148862379608544870279729366302490234375f,-0.64119995798096096617513239834806881844997406005859375f,1.2096999755755053485017924685962498188018798828125f,3.85860013793557943273526689154095947742462158203125f,0.427799964465946036540344721288420259952545166015625f,1.8665000472441317658223169928533025085926055908203125f,-1.705400081156193348164151757373474538326263427734375f,2.30209997021556045382340016658417880535125732421875f,1.961900075706094259686551595223136246204376220703125f,-0.4530000122338526669096836485550738871097564697265625f,1.25540010171979599107316971640102565288543701171875f,-2.3554999905526639025765689439140260219573974609375f,0.28109994312152419837502748123370110988616943359375f,0.3433000631079086151231649637338705360889434814453125f,-0.781799924892187192426717956550419330596923828125f,-0.982199979654699628639491493231616914272308349609375f,1.322600017411261763555785364587791264057159423828125f,1.77049996179342183921789910527877509593963623046875f,-0.686100017310680154736246549873612821102142333984375f,2.483600086377559801320558108272962272167205810546875f,0.666999870307742437347542363568209111690521240234375f,-3.3628000101380042252685598214156925678253173828125f,-3.93810001474395310339104980812408030033111572265625f,0.1146999234586945082270403872826136648654937744140625f,0.1419999505504974113279104130924679338932037353515625f,1.355400061747431994518819919903762638568878173828125f,0.226199936690927305704690297716297209262847900390625f,-0.2667001104719919357677326843258924782276153564453125f,2.1041999678120024697136614122427999973297119140625f,-1.97689995990544620241280426853336393833160400390625f,4.5409000798299903323140824795700609683990478515625f,-2.821000010646880440390305011533200740814208984375f,-0.891799998801947158000302806613035500049591064453125f,1.657200043617933271633546610246412456035614013671875f,-0.6476000545710325351222991230315528810024261474609375f,-1.43769994858205318877253375831060111522674560546875f,0.4257000174775715795050246015307493507862091064453125f,3.240200051586330065589436344453133642673492431640625f,0.3639000135347243425343322087428532540798187255859375f,2.82390003581195880855148061527870595455169677734375f,-0.756400016757101223419113011914305388927459716796875f,-1.568799875144662792081362567842006683349609375f,-0.94820004370361488099661073647439479827880859375f,-0.87539994622617844921563801108277402818202972412109375f,-2.891900024427473514521125252940692007541656494140625f,0.91609999324530377862174645997583866119384765625f,-0.1719998729340741494553412849199958145618438720703125f,2.109200121740997246888582594692707061767578125f,0.16300006245076625877032938660704530775547027587890625f,1.3032999936684961372890256825485266745090484619140625f,-3.378199863287805637668270719586871564388275146484375f,-2.2077999390035873972237823181785643100738525390625f,0.3800999622210861073057230896665714681148529052734375f,4.49309990888983268320089337066747248172760009765625f,-0.88229995031356711621128852129913866519927978515625f,0.411499989539383381753623325494118034839630126953125f,0.42580001803189559961282384392688982188701629638671875f,-2.4722000332057465499246973195113241672515869140625f,0.0577999984219685902786523001850582659244537353515625f,1.248300116640328472072951626614667475223541259765625f,-0.790999995019287638342575519345700740814208984375f,0.74610010306686103120910047437064349651336669921875f,-1.28899995008856027567389901378192007541656494140625f,1.528299987372756962855646634125150740146636962890625f,2.6369999815449123303778833360411226749420166015625f,-2.69140007354915145043605662067420780658721923828125f,-0.1669000454381099896039586383267305791378021240234375f,1.6027999163165704832323399386950768530368804931640625f,-0.17139994146898462901162929483689367771148681640625f,-1.6889999868944272076731749621103517711162567138671875f,2.95439994835704577980095564271323382854461669921875f,-1.226599960459022309322563160094432532787322998046875f,1.004700093592703513678543458809144794940948486328125f,-1.2267000050649041753558776690624654293060302734375f,1.952500036600977040279758512042462825775146484375f,0.907300081145764547585486070602200925350189208984375f,0.421799960841236887265637278687790967524051666259765625f,1.083199950105696363067409038194455206394195556640625f,-0.2233999208718540074869451927952468395233154296875f,1.763900035439431857042791307321749627590179443359375f,0.16170007942169906112184207813697867095470428466796875f,0.625499976433813831278030193061567842960357666015625f,-0.2690999730691310265484617048059590160846710205078125f,-1.6760999297849838995944082853384315967559814453125f,-1.32719992379397044857114451588131487369537353515625f,-1.7223998737543817494355380404158495366573333740234375f,-0.9317000081568933556042111376882530748844146728515625f,0.778500078186393551504806964658200740814208984375f,-1.200299952191115959720946193556301295757293701171875f,-2.23079986970946020363726347568444907665252685546875f,2.87289993829131073965754694654606282711029052734375f,1.088199919129909432058411766774952411651611328125f,0.674000003173947259682563526439480483531951904296875f,1.4628000409089023658992800847045145928859710693359375f,-3.18570005987137516711982243577949702739715576171875f,1.75020001258254165321659456822089850902557373046875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_strided_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,8,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,2,4,1,3};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{3,2,2,3},        // move_strides
                                                                  Strides{2,1,3,2},        // filter_dilation
                                                                  CoordinateDiff{0,0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0,0}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f,.81f,-.90f,.66f,-.06f,-.42f,.47f,.12f,.59f,-.26f,.12f,-.93f,.61f,-.19f,-.96f,-.63f,-.86f,-.96f,.33f,-.80f,-.31f,-.22f,-.73f,-.39f,.69f,-.59f,.17f,-.24f,.75f,-.61f,-.90f,.23f,.75f,-.51f,-.08f,-.47f,.37f,-.30f,.36f,.75f,.93f,.97f,.52f,.64f,.98f,-.05f,.34f,-.66f,.60f,.44f,.48f,.53f,-.61f,-.57f,.18f,-.28f,.80f,.14f,.31f,-.54f,-.56f,.48f,.28f,-.86f,-.04f,-.12f,-.97f,.87f,-.82f,-.33f,-.75f,.67f,-.05f,.83f,.14f,.84f,-.47f,.25f,-.52f,-.48f,-.86f,.15f,.19f,-.45f,-.48f,-.33f,-.98f,-.61f,.13f,.83f,-.45f,.78f,.16f,-.52f,.37f,.07f,-.98f,.78f,.91f,-.26f,-.30f,.10f,-.92f,.17f,.10f,.44f,-.31f,-.50f,-.85f,-.29f,.85f,-.47f,.13f,-.59f,-.71f,.34f,.39f,-.72f,-.46f,-.66f,.44f,.95f,.09f,-.10f,.81f,.37f,-.82f,.24f,-.58f,-.79f,.61f,-.73f,.45f,-.76f,-.55f,-.06f,-.59f,-.04f,.42f,.08f,-.51f,-.87f,-.54f,-.25f,.51f,-.14f,.78f,.06f,.44f,-.40f,.11f,-.47f,.36f,.23f,.12f,.49f,-.40f,-.39f,.45f,.74f,-.99f,-.21f,.03f,.85f,.59f,.67f,.15f,.74f,.34f,-.49f,-.59f,-.82f,-.82f,.60f,.12f,-.61f,-.19f,.52f,-.32f,.96f,.46f,.95f,.27f,.97f,-.01f,.18f,.81f,-.55f,.05f,-.76f,-.74f,.08f,.58f,-.77f,.43f,-.55f,-.78f,.37f,.39f,-.37f,-.03f,-.75f,-.46f,.96f,.50f,.55f,-.81f,.39f,-.54f,.42f,-.92f,-.40f,.31f,.16f,-.87f,-.58f,.41f,-.31f,-.74f,-.36f,.18f,.00f,.32f,.32f,.39f,-.91f,.50f,.75f,.38f,.60f,.11f,-.56f,.61f,-.92f,.38f,.10f,-.30f,-.42f,.35f,-.51f,.58f,.84f,-.37f,-.00f,.82f,.15f,.82f,-.89f,.34f,.29f,-.45f,-.31f,-.54f,.02f,-.69f,.15f,-.77f,-.14f,-.89f,-.19f,-.03f,-.41f,.20f,-.28f,-.60f,.25f,-.18f,.82f,-.38f,-.20f,-.25f,-.18f,-.85f,.88f,-.02f,.97f,.62f,-.97f,.63f,.36f,-.13f,-.93f,-.84f,.63f,.87f,-.18f,-.34f,-.72f,.63f,.13f,-.05f,-.56f,.81f,-.19f,-.89f,.61f,.71f,.22f,.45f,-.44f,.89f,.67f,.47f,.85f,-.50f,-.03f,-.73f,.20f,-.86f,.71f,-.12f,-.16f,.50f,.47f,-.95f,-.37f,.81f,.85f,.69f,.46f,-.77f,.25f,.15f,.09f,-.12f,-.32f,.93f,-.04f,-.66f,-.41f,-.91f,-.84f,-.32f,-.51f,.65f,.09f,-.18f,.80f,.55f,-.30f,.56f,.22f,.28f,-.48f,-.61f,-.15f,-.65f,-.38f,.03f,-.83f,.61f,.83f,-.14f,.39f,-.44f,-.49f,-.70f,.35f,-.72f,.51f,.09f,.31f,.98f,.63f,.74f,.25f,-.30f,.20f,.08f,.81f,.69f,.16f,-.85f,.69f,.20f,-.68f,.99f,.14f,-.65f,-.21f,.03f,.06f,-.16f,.80f,.95f,.90f,-.13f,.05f,.84f,.56f,-.42f,-.58f,.02f,-.16f,-.68f,.03f,.94f,-.20f,-.99f,.99f,-.20f,-.69f,.25f,-.00f,.55f,-.11f,.74f,.96f,-.12f,-.50f,.81f,.97f,-.56f,.40f,.94f,.34f,-.98f,-.04f,-.45f,-.89f,-.04f,-.07f,-.35f,.58f,-.52f,.85f,.94f,-.92f,-.46f,-.51f,-.30f,-.98f,.04f,.45f,-.08f,.53f,-.78f,.43f,.90f,-.48f,-.31f,-.84f,-.70f,-.79f,-.33f,.13f,.68f,-.44f,.21f,.62f,.73f,.15f,.72f,.31f,.21f,-.78f,-.89f,-.96f,-.69f,.12f,.45f,-.47f,-.91f,-.33f,.75f,.87f,.35f,-.33f,.08f,.34f,-.93f,.55f,-.30f,.93f,.60f,-.06f,.89f,-.84f,-.34f,.59f,.20f,-.63f,-.40f,.10f,-.47f,-.79f,-.05f,-.28f,-.97f,.68f,.58f,-.96f,-.32f,.09f,-.14f,.20f,.86f,.33f,-.27f,-.62f,.99f,-.56f,.87f,-.75f,-.53f,.86f,-.44f,-.38f,-.30f,-.73f,-.95f,-.19f,.16f,-.16f,-.87f,-.41f,.77f,-.40f,.90f,-.10f,.67f,-.28f,-.81f,.66f,-.97f,-.21f,-.08f,-.57f,-.80f,.07f,-.86f,-.94f,-.33f,-.43f,-.63f,.52f,-.74f,-.07f,-.17f,.75f,.10f,.97f,.70f,-.01f,.13f,.28f,-.59f,-.25f,.39f,.44f,.18f,.38f,-.95f,.93f,-.80f,.37f,.08f,.42f,-.84f,-.05f,.21f,.96f,.93f,-.60f,.96f,-.50f,.17f,-.74f,.47f,-.16f,-.05f,-.36f,-.46f,-.62f,.95f,.44f,-.45f,.42f,-.31f,.79f,.37f,-.21f,-.78f,-.08f,-.70f,-.83f,.02f,.10f,-.45f,-.12f,-.45f,-.90f,.95f,.60f,.34f,-.38f,-.22f,.37f,.04f,-.21f,-.63f,.10f,.61f,-.76f,-.48f,-.45f,.27f,.96f,.75f,.09f,.77f,.99f,-.35f,.81f,.95f,.14f,-.41f,-.54f,-.43f,.25f,.36f,-.61f,-.11f,.83f,.14f,.56f,-.42f,.80f,.35f,.11f,-.51f,-.68f,-.28f,.05f,.54f,-.01f,-.74f,-.19f,-.91f,.86f,-.05f,.19f,-.71f,-.22f,.39f,.97f,.08f,.92f,-.36f,.29f,.08f,.99f,-.31f,-.94f,-.37f,.17f,.41f,.41f,-.57f,-.56f,-.05f,.88f,.19f,.91f,.34f,.07f,.39f,-.56f,.76f,.87f,-.82f,.79f,-.29f,.29f,.79f,-.00f,-.68f,.22f,.75f,-.29f,-.92f,-.57f,.88f,-.78f,-.99f,.88f,.34f,.82f,.86f,-.17f,.79f,.12f,.42f,-.50f,.23f,.83f,.25f,.77f,.62f,-.81f,-.24f,.47f,-.49f,-.45f,.06f,-.38f,-.01f,.15f,-.16f,-.63f,-.25f,-.99f,.24f,.85f,-.69f,-.37f,-.27f,.01f,-.22f,.73f,.63f,-.48f,.87f,.90f,-.64f,.68f,.59f,.11f,-.83f,.17f,.87f,-.88f,.76f,-.85f,-.20f,.94f,.71f,.16f,-.40f,-.73f,.91f,.61f,.89f,-.73f,.79f,-.48f,-.18f,.74f,.64f,-.74f,-.89f,-.84f,-.28f,-.00f,.45f,.50f,.78f,.84f,-.54f,.66f,.67f,-.58f,-.67f,.09f,-.93f,-.50f,-.71f,-.57f,.44f,-.20f,.42f,-.65f,-.81f,.37f,.21f,-.82f,-.16f,.84f,.04f,.51f,.33f,-.43f,.44f,-.82f,.92f,-.33f,-.92f,.76f,-.33f,.83f,-.21f,-.25f,.28f,.78f,.70f,.04f,.65f,-.44f,.28f,-.61f,-.85f,.26f,-.86f,.41f,.02f,-.44f,.93f,.27f,.79f,-.60f,.91f,-.25f,-.92f,.13f,.87f,-.29f,-.89f,-.56f,-.91f,-.95f,-.72f,-.62f,-.99f,-.89f,-.89f,.64f,-.08f,.70f,-.78f,.82f,-.54f,.97f,-.26f,.12f,.46f,-.26f,-.46f,-.65f,.04f,-.50f,.34f,-.78f,-.54f,-.29f,.13f,.19f,.01f,-.77f,-.20f,-.98f,.43f,.37f,.80f,-.97f,-.99f,-.66f,.55f,.85f,.45f,.58f,-.29f,-.54f,-.95f,.41f,.14f,-.84f,.13f,.06f,.37f,.23f,.76f,.88f,.12f,-.84f,-.53f,-.26f,.26f,.67f,-.24f,.76f,.76f,.12f,-.13f,-.05f,-.12f,.92f,.40f,-.68f,.68f,.00f,.19f,-.72f,.06f,-.66f,.55f,.77f,-.71f,.84f,.59f,.15f,.27f,.38f,-.99f,-.03f,.65f,-.94f,.55f,-.51f,.40f,.32f,.33f,-.93f,.49f,.83f,.98f,.32f,-.45f,.32f,-.22f,.37f,-.35f,-.30f,-.63f,-.72f,.86f,-.95f,.70f,.44f,.12f,-.66f,.39f,.67f,-.56f,-.25f,-.64f,-.15f,.33f,.37f,-.26f,.65f,.13f,-.98f,-.27f,-.94f,.90f,-.30f,-.35f,.24f,.20f,-.93f,.24f,-.17f,-.80f,.24f,.92f,-.81f,-.61f,.05f,-.72f,.12f,.95f,-.07f,-.04f,.74f,-.10f,-.07f,-.55f,-.07f,.21f,-.77f,-.79f,.52f,-.76f,-.02f,-.18f,-.88f,-.76f,.78f,-.74f,-.22f,.26f,-.51f,.91f,.33f,.82f,.67f,.89f,-.57f,.47f,-.30f,.06f,.46f,.40f,.33f,.36f,-.12f,.80f,-.46f,.69f,.60f,.96f,.69f,-.85f,-.16f,-.44f,-.13f,-.58f,-.93f,.05f,.51f,-.02f,.79f,-.36f,.27f,.74f,.67f,.61f,-.67f,-.97f,-.01f,.78f,-.94f,.11f,-.35f,-.45f,-.27f,.51f,.07f,-.34f,.46f,.85f,-.25f,.84f,.40f,-.68f,-.66f,-.89f,.72f,-.52f,-.46f,-.19f,.91f,.09f,-.56f,.16f,.01f,-.29f,.86f,-.05f,.50f,-.98f,.98f,-.25f,.21f,-.52f,-.45f,.07f,.97f,.62f,.51f,.91f,-.42f,.80f,-.33f,-.77f,-.98f,.82f,-.35f,.96f,.25f,-.54f,.50f,-.66f,.54f,-.14f,.53f,-.96f,-.05f,-.85f,-.03f,.12f,-.71f,-.06f,.47f,-.80f,.70f,.88f,.57f,-.07f,-.04f,-.13f,.80f,.59f,-.23f,-.66f,-.34f,-.78f,.95f,.14f,.24f,-.37f,-.29f,.51f,-.94f,-.93f,-.62f,-.68f,.58f,-.13f,.16f,.50f,-.23f,-.01f,-.24f,.82f,-.07f,-.01f,-.49f,.23f,-.00f,.83f,-.60f,.72f,.34f,-.12f,-.11f,-.07f,.01f,.15f,-.23f,.58f,-.60f,.74f,.43f,-.15f,-.41f,-.29f,.66f,.60f,.74f,-.13f,.48f,-.13f,-.35f,.29f,-.64f,-.76f,.27f,.56f,.72f,.51f,-.75f,-.30f,.53f,-.62f,.64f,.11f,-.93f,-.12f,-.43f,.96f,.49f,-.66f,.47f,-.96f,-.60f,.72f,.64f,.39f,-.53f,-.61f,-.37f,.99f,-.74f,.50f,-.32f,.82f,-.98f,-.06f,-.68f,.32f,-.28f,-.64f,.30f,-.44f,.60f,.16f,-.29f,-.21f,-.12f,.19f,.26f,-.32f,-.90f,.88f,.96f,-.28f,.51f,-.86f,.28f,.43f,.82f,-.21f,-.15f,.15f,-.70f,-.70f,.05f,-.82f,-.85f,-.57f,.13f,-.22f,.53f,.80f,.43f,.30f,-.77f,-.95f,.95f,.95f,-.21f,.56f,.56f,-.66f,-.70f,-.03f,.51f,.79f,.00f,-.68f,.25f,-.15f,.12f,.07f,-.18f,-.57f,.81f,.22f,-.80f,.14f,.14f,.19f,.65f,-.06f,.72f,-.96f,-.93f,-.96f,-.35f,.06f,.92f,-.01f,-.16f,.13f,.00f,-.24f,.79f,-.15f,-.29f,-.83f,-.38f,-.91f,-.17f,-.21f,-.89f,-.24f,.40f,-.43f,-.31f,.45f,.74f,.77f,-.98f,.53f,.87f,-.42f,-.37f,-.95f,.02f,-.03f,.21f,-.39f,-.27f,-.66f,.62f,-.34f,.34f,.89f,-.64f,-.65f,-.51f,-.98f,.14f,-.49f,.27f,.10f,-.71f,-.20f,-.01f,-.51f,.50f,-.30f,.98f,-.46f,.10f,-.60f,-.71f,-.95f,.49f,.59f,.00f,-.19f,.11f,-.41f,.08f,-.50f,.48f,.19f,-.80f,.55f,-.27f,.14f,-.66f,.04f,-.16f,-.40f,.18f,-.50f,.89f,-.98f,.27f,-.39f,.91f,-.07f,-.88f,-.76f,-.34f,.10f,.80f,-.05f,-.31f,-.87f,.65f,.26f,-.62f,.55f,.57f,-.98f,.11f,.14f,-.63f,-.39f,.13f,-.87f,-.60f,.92f,-.08f,-.88f,.08f,-.82f,-.12f,.71f,.41f,-.81f,.17f,.16f,-.52f,-.44f,-.58f,.24f,-.16f,.42f,-.87f,-.41f,.86f,.51f,-.42f,.57f,.44f,-.90f,-.37f,-.11f,-.13f,.12f,-.13f,-.80f,-.44f,-.12f,.76f,-.65f,-.00f,.94f,-.49f,.64f,-.20f,-.02f,-.30f,-.70f,.81f,.37f,-.12f,.06f,-.73f,-.99f,-.24f,.66f,.32f,-.94f,.29f,.47f,.32f,-.20f,.90f,.37f,.14f,-.65f,-.30f,.03f,-.81f,-.85f,-.45f,.95f,.52f,-.64f,.70f,-.32f,-.50f,.34f,.33f,.44f,.94f,.39f,.78f,-.99f,.35f,-.04f,.56f,-.65f,.33f,-.57f,.23f,-.17f,-.97f,.70f,-.68f,-.23f,-.60f,.20f,-.93f,.00f,.16f,.92f,-.19f,.72f,.28f,.55f,.97f,.44f,-.40f,-.03f,.86f,.27f,.31f,-.50f,.16f,.03f,-.49f,-.09f,-.64f,-.17f,-.01f,.88f,.07f,.45f,-.45f,-.36f,-.40f,.69f,-.54f,-.44f,-.74f,.00f,-.49f,-.40f,-.92f,.16f,-.21f,-.39f,-.26f,-.63f,.14f,-.53f,.31f,-.83f,.33f,.31f,.06f,-.88f,.12f,-.20f,.63f,.35f,.79f,-.25f,.77f,-.79f,.16f,.64f,-.49f,-.14f,.09f,-.00f,.46f,-.96f,-.00f,.67f,.74f,-.23f,-.25f,-.16f,.45f,-.87f,.73f,.89f,-.40f,.85f,.70f,-.98f,.11f,.27f,.64f,.22f,-.38f,.80f,-.38f,.98f,.87f,.96f,.54f,.88f,.46f,.00f,-.08f,.94f,.02f,-.97f,.16f,-.04f,.44f,-.45f,-.58f,.38f,-.49f,.93f,.78f,.84f,-.35f,.28f,.96f,.16f,-.58f,.32f,.34f,-.43f,.34f,.68f,.92f,-.69f,.48f,.42f,-.13f,-.54f,.09f,.70f,-.85f,.74f,-.55f,-.18f,.45f,-.49f,.12f,.33f,-.49f,-.68f,-.67f,.60f,.61f,-.87f,.13f,.68f,-.56f,.83f,-.16f,.26f,-.18f,-.80f,-.47f,.80f,-.25f,.36f,-.12f,-.98f,.78f,-.58f,.76f,-.25f,.34f,.42f,.76f,.09f,-.97f,.57f,.01f,.82f,.02f,-.67f,-.15f,.99f,-.27f,.19f,-.16f,-.28f,-.70f,.17f,.68f,.00f,.81f,-.88f,-.43f,-.71f,-.90f,-.12f,-.31f,-.15f,-.12f,-.44f,.58f,.26f,-.41f,-.23f,-.06f,-.71f,-.21f,.66f,-.27f,.31f,.55f,-.43f,-.23f,-.98f,.17f,-.03f,.66f,.13f,.41f,-.20f,-.93f,-.33f,-.25f,.23f,-.15f,.19f,.73f,.24f,-.49f,.16f,.24f,-.12f,-.21f,-.61f,-.11f,-.23f,-.98f,-.51f,.19f,.09f,.32f,.99f,.28f,.19f,.66f,.74f,.89f,-.10f,.04f,-.12f,.79f,.36f,.17f,.51f,-.66f,-.09f,.75f,-.88f,-.67f,-.74f,-.63f,-.36f,-.93f,.61f,-.12f,-.00f,.06f,.58f,-.27f,.73f,-.97f,-.03f,-.97f,.99f,-.11f,-.55f,-.59f,-.44f,-.90f,-.13f,.20f,.85f,.90f,.38f,-.46f,-.41f,.18f,-.87f,-.82f,.61f,-.75f,.20f,-.74f,-.68f,-.22f,-.21f,.68f,-.00f,.38f,-.50f,-.21f,.11f,.94f,-.98f,.59f,-.69f,-.36f,.92f,-.76f,-.73f,-.29f,.23f,-.52f,.18f,.26f,.37f,.08f,.35f,-.61f,.71f,.36f,.00f,-.67f,.57f,.63f,-.04f,.06f,-.96f,.40f,.37f,.86f,.05f,.49f,.78f,-.47f,-.90f,.53f,-.30f,-.57f,-.72f,.25f,.06f,.98f,.89f,-.66f,-.83f,.55f,.35f,.84f,.27f,-.56f,-.40f,-.56f,-.40f,-.83f,.69f,-.05f,-.39f,.87f,-.81f,-.29f,-.94f,.10f,-.32f,-.48f,.43f,-.40f,-.64f,.31f,.98f,-.96f,-.99f,-.70f,.23f,.95f,-.59f,-.04f,-.06f,.95f,-.52f,.18f,.79f,-.28f,-.30f,-.23f,.66f,.49f,-.21f,-.84f,-.27f,.85f,.12f,.05f,.39f,-.21f,.49f,-.28f,-.07f,.96f,.00f,.78f,-.54f,.75f,.15f,.14f,-.21f,-.19f,.06f,.42f,.34f,-.41f,.11f,.44f,-.02f,.89f,.56f,-.78f,.75f,.06f,-.22f,-.32f,-.78f,-.52f,-.46f,.64f,-.31f,-.53f,-.20f,.66f,.29f,.91f,-.84f,-.82f,.67f,-.92f,-.56f,.26f,-.42f,-.93f,.13f,.90f,-.20f,-.40f,.56f,-.60f,.39f,-.01f,.67f,-.77f,-.70f,.30f,-.21f,.04f,-.05f,-.70f,.87f,-.99f,.23f,-.11f,-.99f,-.72f,.22f,.82f,.59f,-.20f,-.40f,.28f,-.91f,-.44f,-.49f,.67f,.33f,.15f,-.09f,-.77f,-.65f,-.95f,.39f,-.02f,-.08f,-.60f,.10f,-.72f,.51f,-.17f,-.80f,-.07f,.92f,-.81f,.65f,.86f,.51f,-.99f,.32f,-.47f,-.37f,-.95f,.59f,-.46f,-.63f,.84f,.84f,-.61f,.97f,.17f,.48f,.94f,-.02f,.98f,.19f,.16f,.40f,.16f,-.40f,-.94f,-.61f,.49f,-.60f,.52f,-.62f,.91f,.67f,.25f,.68f,-.03f,-.03f,.31f,-.16f,.72f,.28f,-.13f,-.10f,-.47f,-.49f,-.79f,-.24f,-.39f,-.93f,-.93f,-.64f,.79f,.06f,-.73f,-.05f,-.77f,-.42f,-.10f,-.28f,-.26f,.58f,.07f,.80f,.52f,-.68f,.86f,-.35f,.79f,.50f,-.83f,-.52f,.75f,-.54f,.49f,.17f,.29f,.45f,-.50f,-.08f,.81f,.98f,.57f,-.57f,-.73f,-.54f,-.94f,-.91f,.13f,.74f,-.31f,-.23f,-.85f,-.98f,-.88f,-.98f,.70f,-.92f,.85f,-.96f,-.71f,.46f,.07f,-.41f,.80f,.35f,.64f,.50f,-.57f,.86f,.13f,.09f,-.13f,.78f,.51f,.66f,.18f,.63f,.66f,.05f,.06f,-.87f,-.33f,-.19f,-.55f,.94f,-.52f,-.74f,.21f,.33f,.50f,.90f,.53f,.80f,.53f,-.15f,.02f,-.84f,-.14f,.19f,.51f,.26f,.16f,-.88f,.98f,-.73f,.28f,.64f,-.76f,.20f,-.74f,.09f,.21f,-.67f,.63f,-.74f,.18f,-.03f,-.25f,-.42f,.76f,-.79f,.98f,-.48f,.94f,.10f,.36f,.47f,-.56f,-.69f,-.74f,.37f,.49f,.78f,-.57f,.14f,-.83f,.15f,-.88f,.91f,.25f,-.89f,-.98f,-.67f,.74f,.53f,-.50f,-.54f,-.94f,-.93f,.69f,.15f,.76f,.60f,.86f,-.96f,-.52f,.84f,-.73f,.10f,.35f,-.13f,.53f,.43f,.19f,-.59f,-.02f,.43f,.50f,-.18f,.03f,-.13f,-.21f,.46f,-.07f,-.99f,.94f,-.16f,-.43f,.25f,-.55f,-.77f,-.13f,-.99f,.98f,.69f,.92f,-.77f,-.89f,-.03f,.03f,-.10f,.48f,.47f,-.95f,.87f,-.12f,.56f,.36f,.56f,-.04f,-.97f,.57f,-.85f,-.45f,.48f,-.54f,.90f,-.46f,.49f,-.02f,.25f,-.83f,.77f,-.38f,.09f,.35f,.89f,-.79f,-.79f,.13f,-.14f,-.27f,-.49f,.46f,.43f,-.91f,-.79f,-.67f,-.15f,.73f,.88f,.17f,-.03f,.22f,.43f,.47f,-.72f,.38f,.23f,-.28f,-.14f,-.02f,-.23f,.47f,.48f,-.11f,-.42f,-.19f,.20f,-.84f,.91f,-.17f,-.52f,.46f,-.53f,.55f,-.29f,.80f,-.94f,-.24f,.53f,.18f,.52f,-.91f,-.47f,-.98f,.82f,.81f,.89f,-.94f,-.18f,.26f,.98f,-.97f,-.99f,-.82f,.67f,-.88f,.76f,-.73f,.25f,.82f,-.27f,-.16f,.35f,-.61f,.39f,.79f,-.57f,-.41f,-.56f,-.35f,.26f,-.34f,.74f,-.10f,.98f,-.55f,-.38f,-.91f,-.06f,-.85f,.62f,-.34f,.51f,-.83f,.91f,-.09f,-.54f,-.79f,-.37f,.79f,.74f,-.86f,-.52f,.45f,.04f,.65f,-.26f,.70f,.88f,-.13f,.36f,.23f,.06f,.73f,.16f,-.53f,-.91f,-.56f,-.07f,.72f,-.17f,-.13f,.56f,.11f,.19f,.46f,-.31f,.76f,.95f,.68f,.12f,.60f,-.20f,-.89f,.80f,.78f,-.21f,-.04f,.79f,-.18f,.28f,-.37f,-.39f,.92f,-.73f,-.55f,-.32f,-.15f,-.20f,-.59f,.71f,.57f,-.60f,-.58f,-.36f,.05f,.22f,.08f,-.37f,-.73f,.00f,.99f,-.44f,-.83f,.30f,-.39f,-.49f,.40f,-.33f,-.64f,-.07f,.24f,-.80f,.38f,.65f,.18f,.88f,.99f,-.18f,-.08f,.45f,-.76f,-.30f,.87f,.24f,-.21f,-.28f,.62f,-.71f,.57f,.92f,.01f,.70f,.44f,-.77f,-.47f,-.70f,.72f,-.67f,-.60f,.08f,-.26f,.26f,.16f,-.94f,-.06f,.46f,.98f,-.90f,.17f,-.45f,-.43f,-.00f,.71f,-.82f,-.59f,-.44f,.03f,.98f,-.85f,-.33f,.45f,.22f,-.04f,.59f,.87f,.91f,.97f,-.97f,.69f,-.60f,.61f,-.59f,-.56f,-.71f,-.35f,.13f,.06f,.81f,-.19f,.45f,.56f,-.03f,.09f,.28f,-.75f,.97f,.70f,.98f,-.56f,-.62f,.52f,-.57f,.48f,.04f,-.31f,-.60f,-.22f,-.39f,.05f,-.27f,.15f,-.40f,-.47f,.74f,.14f,.96f,-.35f,-.12f,-.95f,.32f,-.99f,.06f,-.22f,-.82f,-.89f,.89f,.00f,.98f,-.90f,.48f,.79f,.87f,.10f,-.29f,.79f,-.88f,-.07f,-.95f,-.70f,.06f,-.44f,.17f,.94f,.54f,.56f,.83f,-.89f,.80f,-.41f,.40f,-.13f,.44f,-.20f,-.29f,.59f,-.58f,-.80f,-.37f,-.33f,-.10f,-.51f,-.90f,-.16f,-.14f,.97f,.06f,.77f,-.70f,-.70f,.30f,-.36f,-.17f,-.17f,.70f,.25f,.24f,-.14f,.22f,.81f,-.16f,.07f,-.89f,.29f,-.60f,-.95f,.26f,-.20f,-.04f,-.90f,.55f,.80f,-.66f,.51f,.59f,.71f,-.51f,.73f,.89f,-.74f,.88f,.70f,.08f,.61f,-.83f,-.89f,-.93f,.34f,-.41f,.85f,-.74f,-.46f,-.48f,.05f,.86f,-.23f,.82f,.92f,-.41f,-.61f,-.76f,.22f,-.80f,-.29f,.18f,-.05f,-.96f,-.47f,.74f,.81f,-.73f,.69f,-.74f,-.05f,-.80f,-.89f,.13f,-.50f,-.40f,.16f,.03f,.85f,-.23f,.63f,.88f,-.77f,-.58f,.47f,-.70f,-.40f,.61f,.62f,-.23f,-.46f,-.19f,.14f,-.18f,.67f,.21f,-.57f,-.75f,-.04f,.20f,.82f,-.18f,-.44f,-.48f,.11f,.50f,-.60f,.67f,-.60f,.48f,.58f,-.92f,.20f,-.86f,.48f,-.87f,.47f,-.43f,.04f,-.90f,-.50f,-.79f,-.28f,-.25f,-.20f,.49f,-.01f,.28f,.83f,-.03f,-.32f,-.72f,.76f,.80f,-.53f,-.64f,-.34f,.59f,-.81f,.20f,.10f,-.13f,-.40f,-.51f,-.57f,.13f,.93f,.89f,-.03f,-.09f,-.96f,-.41f,-.32f,.14f,-.94f,-.21f,.51f,.03f,-.50f,.54f,-.52f,-.23f,.22f,-.73f,-.49f,-.80f,-.84f,-.26f,.53f,-.61f,.81f,-.80f,.83f,.35f,.49f,-.39f,-.10f,.64f,.12f,.65f,.78f,-.84f,-.32f,.01f,-.38f,-.98f,-.75f,.68f,.16f,.56f,-.23f,-.87f,.37f,.19f,.71f,-.73f,.07f,-.17f,.20f,.08f,.33f,-.49f,.67f,-.58f,-.14f,-.60f,-.89f,-.58f,-.06f,.11f,-.14f,.94f,-.81f,-.54f,-.63f,-.89f,-.35f,.90f,.73f,.67f,-.27f,-.36f,.00f,.11f,.48f,.38f,-.78f,.90f,-.36f,-.74f,-.50f,.90f,-.77f,.78f,-.08f,.76f,-.91f,-.97f,-.38f,-.00f,.59f,.79f,-.53f,.21f,.34f,-.79f,.90f,-.42f,-.14f,-.44f,-.25f,-.29f,.34f,-.08f,.76f,.53f,.24f,.07f,.01f,-.29f,-.76f,-.43f,-.40f,.23f,-.66f,.62f,.79f,-.07f,-.26f,.62f,-.18f,-.66f,.90f,.19f,.58f,.59f,-.24f,-.75f,-.30f,.13f,.68f,.08f,.00f,-.00f,.81f,-.56f,.41f,.58f,.50f,-.40f,.59f,-.45f,-.56f,.66f,.02f,.74f,-.27f,.70f,-.50f,-.76f,.24f,.94f,-.61f,.74f,-.95f,.76f,-.24f,.09f,.87f,-.02f,-.73f,-.43f,-.26f,.39f,-.78f,-.50f,-.52f,-.84f,-.81f,-.51f,.86f,-.27f,.27f,-.92f,.39f,.02f,.12f,-.47f,-.75f,-.69f,.00f,-.46f,.19f,.21f,-.75f,.65f,.12f,.74f,-.92f,-.88f,.68f,.85f,-.38f,-.68f,.43f,-.88f,.12f,-.02f,-.77f,.27f,.40f,.53f,.95f,.99f,.56f,.62f,-.68f,-.69f,-.87f,.09f,-.91f,-.25f,-.66f,.19f,-.69f,-.54f,.46f,.21f,-.76f,.84f,-.83f,-.30f,-.30f,.16f,.38f,.39f,-.04f,.32f,-.68f,-.43f,.27f,.08f,.41f,.93f,-.61f,-.54f,.52f,-.99f,.04f,.74f,-.26f,.25f,-.28f,-.60f,.52f,-.68f,.75f,-.62f,-.32f,-.51f,-.69f,-.61f,.90f,-.71f,.81f,-.20f,.24f,.92f,-.25f,-.76f,.88f,.88f,.42f,-.21f,.11f,.33f,-.09f,.01f,-.72f,.96f,.20f,-.70f,-.59f,.48f,-.03f,.42f,.54f,.67f,-.21f,-.13f,.55f,-.11f,.26f,.52f,.83f,.76f,.21f,.51f,-.35f,.49f,.04f,-.62f,.88f,.04f,.00f,.33f,-.68f,.72f,.78f,-.69f,.54f,.04f,-.69f,-.51f,.56f,-.05f,-.68f,.16f,.62f,-.90f,.77f,-.25f,-.78f,.25f,-.86f,.72f,-.82f,-.13f,-.23f,.77f,-.06f,.13f,.29f,-.79f,.13f,-.77f,-.56f,-.22f,-.17f,-.20f,-.35f,.91f,-.92f,.70f,-.07f,.99f,-.44f,.00f,-.91f,.36f,-.69f,-.97f,.97f,-.15f,.76f,.78f,-.36f,-.46f,.43f,.78f,-.08f,-.36f,.91f,-.15f,-.49f,.89f,-.47f,-.88f,.32f,-.35f,.46f,.86f,.19f,.87f,.76f,.41f,.87f,.71f,.11f,.78f,-.87f,.70f,-.29f,-.32f,.01f,-.68f,.19f,-.59f,-.04f,.89f,-.46f,.46f,-.82f,.02f,.35f,-.93f,-.46f,.58f,.03f,-.81f,.27f,-.90f,-.33f,-.93f,-.47f,-.42f,-.27f,-.49f,.82f,.12f,.82f,.29f,-.35f,.74f,.00f,-.70f,.45f,-.19f,.05f,-.39f,-.84f,-.99f,-.22f,-.71f,-.60f,-.54f,.21f,.61f,.18f,.08f,.19f,-.06f,-.98f,.21f,-.78f,.15f,.68f,-.53f,.31f,.89f,-.01f,-.73f,.02f,.40f,-.37f,.23f,-.95f,.62f,.84f,-.35f,-.63f,.96f,-.47f,.14f,.80f,-.68f,.29f,.99f,.49f,-.01f,.16f,.19f,-.38f,.63f,-.93f,-.31f,.83f,-.64f,-.72f,.49f,-.87f,.09f,.89f,-.76f,-.98f,.36f,.71f,.07f,.99f,-.47f,-.41f,.68f,.86f,.77f,.92f,-.79f,-.22f,.24f,.09f,-.45f,.00f,.22f,-.96f,-.50f,.87f,.06f,.93f,-.39f,-.39f,-.39f,-.36f,-.11f,.77f,-.29f,.32f,-.60f,-.69f,-.35f,.12f,-.30f,.66f,.29f,-.45f,-.18f,-.95f,.75f,-.69f,-.71f,-.93f,-.25f,-.36f,.81f,-.79f,.17f,.93f,-.60f,-.98f,.55f,-.87f,-.67f,.32f,.80f,.79f,.62f,.44f,-.19f,-.48f,.34f,.89f,.72f,.37f,-.82f,-.49f,-.71f,-.40f,-.08f,-.05f,-.30f,-.46f,.50f,.24f,.85f,-.95f,-.62f,-.45f,-.09f,.42f,-.95f,-.20f,.21f,-.50f,.39f,-.38f,-.74f,.69f,-.65f,.43f,.60f,-.48f,-.81f,-.14f,-.90f,.56f,.42f,.39f,-.13f,.76f,.06f,-.46f,-.55f,.60f,.72f,-.05f,-.20f,-.17f,-.40f,.23f,-.55f,.90f,-.49f,.58f,.34f,.72f,-.72f,-.54f,-.43f,-.80f,-.49f,-.76f,-.76f,-.11f,.55f,-.68f,.14f,-.98f,.20f,.93f,.19f,.52f,-.26f,.89f,-.82f,-.02f,-.47f,.74f,-.94f,-.01f,.90f,.68f,.20f,.36f,-.50f,-.99f,.20f,.05f,-.18f,-.06f,-.39f,.60f,-.12f,-.94f,.89f,-.31f,.00f,.35f,.74f,-.24f,-.88f,.41f,-.78f,-.04f,.38f,-.17f,.93f,.48f,.31f,.60f,-.20f,-.78f,.85f,-.03f,.26f,.88f,.17f,.82f,.39f,-.03f,.10f,.13f,.72f,.15f,.40f,-.39f,.32f,.44f,-.75f,-.64f,-.29f,.77f,-.50f,.74f,.71f,-.08f,-.30f,-.42f,.17f,-.52f,.84f,-.07f,-.89f,-.60f,-.04f,.64f,-.73f,-.10f,-.82f,.87f,-.78f,.23f,.67f,-.98f,-.68f,.66f,.82f,.64f,.40f,.55f,.91f,-.02f,.01f,.13f,-.82f,-.86f,.79f,-.06f,-.58f,.75f,.99f,-.05f,.32f,-.07f,.97f,.49f,.40f,.42f,.86f,-.62f,.54f,.57f,.90f,-.76f,.54f,-.61f,.32f,.66f,.21f,-.24f,-.70f,.02f,.27f,.78f,-.45f,.29f,.18f,-.06f,.78f,.38f,-.41f,.14f,.50f,-.43f,-.45f,.68f,-.30f,-.79f,-.88f,.57f,-.06f,.36f,.16f,.14f,.48f,-.27f,.46f,-.78f,-.23f,.20f,.34f,.26f,.34f,-.40f,.07f,.00f,.28f,-.92f,-.23f,.46f,-.60f,-.69f,-.98f,.49f,.21f,-.65f,-.01f,.56f,-.42f,.88f,.96f,.40f,.60f,-.24f,-.93f,-.82f,.64f,.23f,-.24f,.28f,-.50f,.48f,.23f,.52f,.75f,.30f,.59f,-.63f,.46f,.47f,-.91f,.70f,.29f,-.47f,.12f,-.82f,.72f,-.20f,-.48f,.07f,.39f,.72f,.24f,.44f,.79f,-.44f,-.59f,.25f,-.95f,-.60f,.24f,-.40f,.50f,-.05f,-.86f,.49f,.95f,.92f,-.87f,.29f,-.71f,-.14f,.37f,.24f,.91f,.82f,-.64f,.12f,-.39f,.53f,-.95f,.32f,.87f,-.90f,.97f,-.85f,.09f,-.54f,-.75f,.74f,-.59f,.51f,-.23f,-.14f,-.64f,.51f,.39f,.60f,.36f,.16f,-.64f,.27f,-.16f,.32f,-.99f,.77f,.48f,-.71f,.55f,.73f,.44f,-.92f,-.29f,.44f,.10f,-.53f,-.71f,-.09f,-.46f,-.66f,-.31f,.05f,.92f,-.52f,.04f,.94f,.91f,.09f,.03f,.23f,-.88f,-.92f,.91f,-.47f,.04f,.37f,-.75f,.74f,.77f,-.69f,-.02f,-.90f,.64f,-.38f,-.21f,.76f,.71f,.38f,-.89f,.36f,-.17f,.86f,.13f,.45f,.36f,-.06f,.40f,.33f,-.53f,-.47f,.71f,.10f,.63f,.84f,-.22f,.07f,.76f,.98f,-.43f,-.54f,.42f,-.23f,-.75f,-.67f,.62f,.55f,.24f,-.77f,-.45f,.77f,.30f,-.80f,-.47f,.70f,.03f,-.26f,-.10f,.16f,.98f,-.33f,-.64f,.67f,.31f,-.61f,-.09f,.54f,-.23f,-.18f,-.41f,-.32f,-.43f,.35f,-.85f,-.45f,.53f,.24f,.86f,-.45f,.37f,.37f,-.87f,-.44f,-.67f,.66f,-.78f,.84f,.10f,.74f,-.33f,-.74f,.28f,.30f,.43f,.57f,.48f,-.36f,-.84f,.82f,.03f,.36f,.04f,.93f,-.42f,.73f,-.33f,.54f,-.85f,.28f,.19f,.31f,-.06f,.59f,.01f,.46f,-.68f,-.79f,-.91f,.95f,-.04f,.59f,.66f,-.03f,-.35f,.77f,.27f,.48f,.04f,-.91f,-.90f,-.38f,-.43f,.19f,.47f,.10f,.01f,-.15f,.38f,.74f,.24f,.68f,.66f,-.06f,-.09f,.79f,.06f,-.98f,-.26f,-.11f,-.01f,-.63f,.17f,-.20f,.94f,.04f,.15f,-.41f,-.88f,.62f,-.03f,-.95f,-.99f,-.35f,-.79f,-.08f,.13f,.45f,-.79f,.30f,-.97f,.53f,-.87f,-.00f,.41f,-.90f,.46f,.14f,-.71f,-.34f,.06f,-.93f,-.31f,-.91f,.88f,.76f,-.69f,.51f,-.97f,.40f,-.95f,.75f,-.95f,-.33f,.74f,-.86f,.17f,.73f,.70f,-.41f,.11f,.91f,.94f,-.67f,.40f,.18f,.01f,.17f,-.10f,.24f,.54f,-.01f,.30f,.21f,.54f,-.37f,-.56f,.76f,-.57f,.23f,-.87f,.35f,.35f,.84f,-.47f,.32f,-.51f,.45f,.03f,.84f,.02f,.74f,.40f,-.56f,.08f,.36f,.97f,.25f,.03f,.48f,-.03f,.65f,.71f,-.04f,.85f,.05f,.13f,-.08f,.81f,.32f,-.34f,-.79f,-.45f,.77f,-.89f,.79f,-.23f,-.65f,-.43f,.55f,.99f,.68f,-.43f,.25f,-.19f,-.32f,-.03f,-.61f,.01f,.10f,.14f,.82f,.50f,.96f,.24f,-.44f,-.21f,.05f,-.28f,.58f,.29f,-.08f,.83f,-.95f,-.21f,-.49f,.61f,-.37f,-.47f,.50f,-.33f,-.83f,.00f,-.68f,.06f,-.96f,-.74f,.57f,.34f,-.54f,-.77f,-.73f,.84f,.81f,-.90f,.95f,.57f,-.79f,.35f,.54f,-.50f,-.86f,.13f,.16f,.75f,.64f,-.74f,-.72f,-.85f,.87f,-.71f,.43f,-.06f,-.69f,.12f,-.06f,.36f,-.34f,.79f,.02f,-.83f,.91f,.40f,.46f,.90f,-.72f,.02f,.01f,-.45f,.30f,-.12f,-.09f,-.14f,.55f,-.33f,.34f,-.82f,.90f,.45f,.95f,-.62f,-.49f,.06f,-.15f,.66f,.10f,-.92f,.71f,.04f,-.28f,-.83f,-.37f,.18f,.37f,.49f,.82f,-.53f,-.30f,.17f,-.65f,-.06f,.29f,-.89f,-.74f,-.21f,-.85f,.93f,-.51f,.29f,-.86f,-.20f,-.19f,.39f,-.51f,.54f,-.67f,.27f,-.23f,.23f,.43f,.65f,.71f,.95f,-.84f,.84f,-.44f,-.78f,.05f,.84f,.05f,-.54f,-.18f,-.41f,-.80f,.05f,-.41f,-.97f,.34f,.64f,-.94f,.19f,.22f,.85f,.58f,.92f,-.07f,-.41f,.30f,-.01f,.72f,-.28f,.91f,.82f,.92f,.17f,-.08f,-.69f,-.92f,.05f,.45f,-.72f,-.34f,.36f,.35f,-.18f,.55f,-.21f,-.69f,-.60f,.86f,.85f,.47f,.48f,.49f,-.31f,.03f,-.42f,-.28f,.38f,-.94f,.82f,.25f,-.40f,-.95f,.81f,-.21f,.50f,-.55f,-.02f,-.91f,-.10f,.45f,-.30f,.31f,.80f,-.57f,.41f,-.83f,.28f,-.02f,.11f,-.45f,.84f,.03f,.14f,.76f,-.21f,-.04f,-.52f,-.55f,-.03f,-.98f,.48f,.91f,-.39f,.57f,.45f,.29f,-.05f,-.33f,-.65f,-.62f,.98f,-.35f,.61f,.81f,.61f,-.51f,.43f,-.36f,.73f,-.20f,-.18f,.05f,.04f,-.83f,.38f,-.98f,-.62f,.07f,.65f,-.23f,.49f,-.33f,-.85f,.33f,-.51f,-.08f,-.21f,.66f,.84f,-.36f,-.01f,.99f,.48f,-.79f,.18f,.82f,.11f,-.38f,-.49f,.24f,.20f,.16f,.60f,-.33f,.58f,.77f,-.43f,-.65f,.81f,.14f,.36f,.00f,-.87f,-.57f,-.01f,-.02f,.30f,.14f,-.17f,.18f,.17f,.64f,.64f,.57f,-.72f,.12f,-.33f,.73f,.54f,-.36f,.16f,.62f,.01f,.09f,.36f,-.65f,-.72f,-.02f,-.40f,.55f,-.54f,-.20f,.24f,.97f,-.31f,.96f,.74f,.71f,.10f,.79f,-.64f,.17f,-.56f,.27f,-.62f,-.65f,.73f,.17f,-.06f,.67f,-.39f,-.46f,.25f,.64f,-.22f,-.13f,.35f,.70f,-.70f,-.60f,-.14f,-.53f,-.09f,.59f,-.09f,-.12f,-.28f,-.65f,.69f,.48f,.11f,-.82f,-.15f,-.01f,.06f,.96f,-.14f,.51f,.39f,.66f,-.27f,-.52f,.54f,-.87f,-.18f,-.69f,-.19f,-.20f,.19f,-.06f,.27f,.18f,-.26f,-.45f,-.09f,.47f,-.15f,-.05f,.54f,.80f,.86f,-.60f,-.02f,.74f,.07f,-.10f,-.73f,.56f,.67f,.51f,.23f,.69f,.85f,.21f,-.92f,.15f,.53f,.12f,.43f,-.26f,-.50f,.29f,.92f,-.29f,-.25f,.94f,-.92f,.53f,.14f,.93f,.88f,.09f,-.29f,.59f,.98f,.49f,.64f,-.18f,-.75f,-.26f,-.47f,.64f,.87f,.46f,-.51f,.04f,-.46f,.00f,-.67f,-.78f,.05f,-.35f,.21f,-.42f,-.85f,.02f,.78f,.86f,-.93f,-.28f,.54f,-.41f,-.83f,.95f,-.72f,.31f,.67f,-.22f,-.07f,.74f,.26f,.27f,-.78f,-.23f,.57f,.55f,-.27f,.17f,.57f,.63f,-.05f,-.76f,-.25f,.37f,-.29f,-.48f,.92f,-.87f,-.08f,-.68f,.61f,.24f,-.48f,-.70f,.61f,-.01f,-.96f,-.54f,.08f,-.75f,.06f,.60f,-.89f,.34f,.76f,-.25f,.34f,-.94f,.00f,.60f,-.24f,-.10f,-.98f,-.56f,-.74f,-.36f,.27f,.74f,.88f,-.94f,.25f,-.85f,-.39f,-.37f,.25f,-.80f,.10f,.52f,-.53f,.20f,.59f,-.17f,-.57f,.45f,.39f,-.63f,.37f,.45f,-.81f,-.16f,.18f,.66f,-.23f,-.98f,-.81f,.47f,.18f,.24f,.38f,-.79f,-.73f,.28f,-.81f,.39f,-.15f,.93f,-.83f,-.42f,-.70f,-.09f,-.95f,.23f,.25f,-.44f,-.79f,-.68f,-.26f,-.04f,.05f,-.39f,-.23f,.22f,-.25f,-.30f,-.00f,-.03f,-.42f,.94f,.27f,.98f,-.32f,.88f,.49f,.75f,.76f,.02f,-.10f,.26f,-.54f,-.00f,-.91f,.30f,.48f,.30f,-.54f,.58f,.13f,.95f,.43f,-.90f,-.79f,.82f,-.03f,.28f,-.69f,.87f,-.35f,-.34f,.97f,-.17f,.03f,.61f,.32f,-.05f,.47f,-.35f,-.48f,-.50f,.37f,.84f,.50f,-.82f,.26f,.47f,-.36f,-.42f,.58f,.71f,.87f,.59f,.92f,.31f,.66f,-.28f,-.13f,-.19f,-.95f,.57f,-.81f,.95f,.16f,.62f,-.60f,-.03f,.75f,.40f,.40f,.78f,.95f,.59f,-.00f,.79f,.49f,-.76f,.91f,-.76f,-.74f,-.68f,.97f,.98f,-.97f,.91f,-.93f,.47f,.86f,.88f,-.56f,.84f,-.00f,.02f,-.23f,.95f,.58f,-.00f,.84f,-.65f,.77f,-.27f,-.42f,-.54f,.19f,-.77f,-.59f,.81f,.45f,.96f,.39f,-.46f,-.74f,-.59f,.93f,-.12f,.26f,-.74f,.28f,-.83f,.58f,.20f,.25f,-.39f,.85f,.29f,.90f,-.42f,.95f,.49f,-.11f,-.66f,.65f,.91f,-.98f,-.61f,-.72f,.20f,.25f,.39f,.76f,-.13f,.56f,.78f,.90f,.10f,.54f,.49f,-.85f,.50f,.91f,-.72f,-.00f,.80f,-.25f,.07f,-.18f,.87f,-.93f,.02f,.03f,-.15f,.37f,.50f,-.21f,-.17f,.78f,-.40f,.70f,-.96f,-.74f,.05f,.22f,-.04f,.78f,.99f,.65f,.63f,.66f,.40f,.53f,-.91f,.46f,.43f,.18f,.65f,-.58f,.73f,.60f,.57f,.72f,.71f,.15f,.70f,-.47f,-.82f,-.38f,.89f,-.91f,.72f,.13f,-.81f,-.79f,-.58f,-.13f,.28f,.79f,.46f,.34f,.27f,.26f,.00f,.39f,.89f,.24f,.83f,.91f,-.87f,-.49f,.30f,-.11f,.77f,-.97f,-.36f,-.41f,-.68f,.85f,.79f,.30f,.81f,-.57f,.91f,-.04f,-.22f,-.16f,.95f,.60f,.27f,-.73f,.26f,-.18f,.50f,-.79f,.10f,-.69f,-.23f,.07f,.76f,-.41f,-.13f,-.64f,-.15f,.47f,-.03f,.93f,.95f,-.67f,.72f,-.80f,-.17f,.98f,.45f,-.91f,.94f,-.67f,.51f,.81f,-.37f,.49f,.43f,.26f,.83f,-.88f,-.10f,.05f,.41f,-.13f,-.32f,.75f,.37f,-.26f,-.42f,-.41f,.79f,.37f,-.06f,.77f,-.61f,.23f,.57f,.28f,.63f,-.43f,-.09f,.22f,-.86f,-.07f,.22f,.48f,-.05f,.71f,-.98f,.47f,.46f,.02f,-.52f,-.24f,.35f,-.02f,-.30f,-.80f,-.18f,-.81f,.19f,-.74f,-.04f,-.58f,-.51f,-.71f,-.28f,.23f,-.51f,-.05f,.88f,-.55f,.54f,.41f,-.06f,-.58f,.32f,.93f,-.14f,-.61f,-.05f,.13f,-.73f,-.19f,-.49f,.59f,.53f,.97f,.64f,.02f,.39f,-.95f,-.87f,-.87f,.96f,-.90f,-.68f,.74f,-.76f,-.08f,-.19f,-.11f,-.32f,.95f,-.13f,.70f,-.81f,.67f,.09f,-.64f,-.24f,.70f,.46f,.28f,.21f,-.10f,.49f,-.22f,.64f,.34f,-.10f,-.78f,-.11f,-.04f,-.31f,-.63f,-.45f,.12f,.71f,.88f,-.41f,.30f,.51f,-.73f,.54f,-.00f,.39f,.62f,.81f,.65f,-.26f,-.35f,.84f,-.12f,.77f,.65f,.77f,-.42f,.54f,.95f,-.82f,.46f,.83f,.15f,-.11f,-.06f,.41f,.87f,-.74f,-.24f,.37f,.94f,-.42f,-.56f,-.01f,-.15f,.03f,.49f,-.75f,.18f,.42f,.54f,.39f,.70f,-.49f,.08f,.68f,-.25f,.36f,.38f,.44f,-.52f,.86f,-.02f,.69f,.90f,.89f,-.77f,.14f,-.78f,-.64f,.40f,-.09f,.78f,-.81f,.94f,.46f,-.53f,.04f,-.46f,.71f,-.85f,.66f,-.51f,-.09f,.91f,.38f,-.26f,.91f,-.04f,-.86f,-.03f,.83f,-.69f,.35f,.65f,-.03f,.65f,.68f,.56f,.30f,-.46f,.73f,.02f,-.21f,.85f,.48f,.91f,.05f,.15f,-.32f,.39f,-.33f,-.84f,-.66f,-.41f,.98f,-.03f,.56f,.30f,.37f,-.86f,.60f,.47f,-.24f,.53f,-.85f,.66f,.53f,-.31f,-.44f,.88f,.34f,-.73f,-.48f,-.33f,-.73f,.56f,.45f,.18f,-.76f,.56f,.23f,-.83f,.97f,-.48f,-.80f,-.66f,.17f,.74f,.39f,-.02f,.49f,-.33f,-.73f,-.81f,.27f,.21f,.38f,-.68f,-.20f,-.09f,-.96f,-.61f,-.93f,-.99f,-.16f,.15f,.62f,.24f,.74f,-.65f,.35f,.26f,.38f,-.64f,-.67f,.80f,.26f,.30f,-.56f,.20f,-.82f,.06f,.81f,.75f,.82f,-.82f,.55f,-.91f,.37f,.16f,-.03f,-.13f,.09f,-.67f,-.33f,-.18f,-.13f,-.98f,-.81f,-.72f,.49f,-.96f,-.22f,.65f,.35f,-.35f,-.60f,-.64f,.19f,-.57f,.73f,-.70f,-.89f,.46f,-.77f,-.32f,.55f,-.81f,-.61f,.97f,-.49f,.30f,.42f,.66f,.19f,.55f,-.72f,-.95f,-.65f,-.28f,-.75f,-.22f,.29f,.56f,-.02f,-.15f,-.80f,.68f,-.05f,-.35f,.68f,-.84f,.95f,-.14f,.04f,.64f,.49f,.02f,.74f,.57f,.58f,.01f,.62f,-.87f,.76f,.37f,.27f,-.51f,.07f,.31f,.39f,.61f,.60f,-.14f,.62f,.01f,-.10f,-.43f,-.23f,-.91f,.99f,.58f,.81f,-.39f,-.98f,.96f,.02f,.38f,-.94f,-.46f,-.02f,.46f,.48f,-.39f,-.46f,.89f,-.62f,.10f,.81f,.53f,-.96f,-.33f,-.58f,-.17f,-.73f,.12f,.05f,.63f,-.83f,.01f,-.02f,-.00f,-.01f,-.79f,.78f,.48f,-.44f,.46f,.48f,-.72f,-.53f,.08f,.00f,-.12f,-.93f,-.87f,-.32f,-.49f,.35f,-.63f,-.20f,.99f,.01f,-.23f,.41f,-.32f,.12f,.85f,.31f,.43f,.21f,.60f,-.33f,-.71f,-.84f,-.23f,-.06f,-.23f,.08f,.89f,.99f,.64f,.65f,-.55f,.60f,-.07f,-.62f,.74f,.57f,.85f,.27f,.56f,-.87f,.00f,-.37f,-.67f,.07f,.49f,.44f,-.92f,.48f,.70f,-.86f,.33f,.39f,.71f,.43f,.80f,-.52f,.01f,-.54f,-.95f,-.39f,.40f,-.75f,.08f,.76f,-.62f,-.39f,.94f,.31f,.09f,-.15f,.83f,-.77f,.05f,.85f,.68f,-.32f,.94f,-.09f,-.57f,.46f,-.01f,-.77f,.68f,.44f,.01f,-.38f,.82f,-.22f,.74f,.75f,-.43f,-.26f,.01f,.87f,.15f,-.93f,-.51f,-.23f,-.57f,.88f,.76f,-.39f,.78f,-.40f,.78f,-.86f,-.15f,.47f,-.75f,-.26f,-.54f,-.00f,-.42f,.40f,.53f,-.34f,.26f,-.20f,-.09f,-.89f,.96f,.44f,-.56f,-.58f,-.40f,-.69f,.21f,-.74f,.42f,.94f,.95f,-.01f,.93f,.22f,-.40f,-.56f,.42f,-.97f,-.72f,.59f,-.69f,.52f,.62f,-.13f,-.47f,.18f,.00f,.37f,.58f,-.61f,.28f,-.85f,.52f,-.69f,-.62f,.54f,-.90f,.89f,.86f,-.83f,-.57f,.32f,.43f,.35f,-.35f,.02f,.93f,.77f,.32f,.52f,.58f,.76f,.35f,-.55f,-.10f,-.07f,-.82f,-.40f,.25f,.91f,.81f,.60f,-.58f,.64f,.20f,.97f,.61f,-.57f,.93f,.69f,-.42f,.99f,-.80f,.59f,.48f,.13f,.42f,-.78f,-.83f,.72f,.49f,.56f,-.69f,-.89f,.95f,-.45f,-.15f,-.39f,.12f,-.85f,-.85f,-.81f,-.63f,.33f,-.00f,.14f,-.46f,-.13f,.44f,.97f,-.03f,.58f,.91f,-.34f,.56f,.79f,.36f,.34f,.00f,-.16f,.98f,-.32f,-.91f,.51f,.65f,-.71f,-.53f,.00f,.41f,.83f,-.42f,-.52f,-.82f,.46f,.20f,-.61f,.50f,.82f,.43f,.05f,-.29f,.92f,.87f,-.57f,-.56f,.10f,.06f,-.95f,-.26f,.35f,-.83f,-.41f,.85f,-.43f,.61f,-.71f,.70f,.70f,-.50f,-.94f,.89f,-.78f,-.64f,.66f,-.37f,-.24f,-.74f,-.40f,.62f,-.01f,.11f,.47f,-.85f,-.10f,-.11f,-.91f,.48f,.43f,.01f,.71f,.66f,.15f,-.33f,-.07f,-.92f,-.77f,-.44f,.33f,-.89f,-.52f,.46f,-.68f,.37f,-.54f,.11f,-.08f,.60f,-.13f,-.01f,-.38f,-.31f,-.32f,.10f,-.58f,-.12f,-.62f,.97f,-.59f,-.26f,-.26f,.08f,-.59f,.91f,.33f,-.12f,-.22f,-.08f,-.86f,.02f,.65f,.56f,-.13f,.34f,.17f,.59f,.62f,.70f,.57f,.73f,-.86f,-.21f,.64f,-.23f,-.68f,.92f,.75f,-.05f,-.63f,-.61f,.52f,.77f,-.92f,-.71f,.73f,-.62f,.32f,-.38f,.99f,-.58f,-.77f,.58f,-.25f,.08f,.87f,.12f,.03f,.60f,-.07f,-.31f,-.70f,.34f,.71f,-.94f,-.72f,.37f,.18f,.71f,.17f,-.75f,-.90f,.24f,-.53f,-.05f,.68f,-.19f,.62f,-.62f,.67f,.02f,.78f,-.85f,.24f,-.57f,.02f,-.31f,-.23f,-.95f,-.78f,-.51f,.46f,.33f,-.06f,-.05f,.75f,-.07f,-.69f,-.55f,-.79f,.39f,-.28f,.92f,-.47f,-.34f,-.04f,.32f,-.35f,.97f,-.45f,.65f,.51f,-.90f,-.87f,.48f,.40f,.46f,.82f,-.02f,-.55f,.85f,.12f,-.10f,.65f,-.23f,.99f,.91f,-.02f,-.89f,-.67f,-.92f,.06f,-.16f,-.97f,-.17f,.11f,.37f,.59f,-.85f,-.43f,-.24f,-.78f,-.35f,.07f,-.26f,.65f,-.86f,-.29f,-.82f,-.03f,.25f,-.23f,.51f,-.61f,.68f,.88f,-.35f,-.17f,.36f,.36f,.80f,-.80f,-.52f,.90f,.65f,.82f,-.19f,-.10f,.62f,.87f,-.11f,-.67f,-.24f,.28f,.80f,.69f,.36f,.06f,.02f,-.15f,-.85f,-.94f,.18f,-.67f,-.03f,.58f,.95f,.70f,.04f,-.66f,-.21f,.77f,-.87f,-.96f,-.86f,-.96f,-.33f,.46f,-.08f,-.15f,.24f,.83f,-.10f,-.90f,-.47f,.56f,-.10f,.96f,-.13f,.12f,.15f,-.14f,-.88f,-.27f,.86f,-.51f,-.00f,.13f,.56f,.00f,.03f,-.64f,.05f,.44f,.85f,.15f,-.70f,.26f,-.57f,-.59f,-.90f,-.42f,.46f,.90f,-.47f,-.05f,-.29f,.77f,-.47f,.63f,-.72f,.24f,-.90f,-.96f,.61f,.74f,-.90f,.12f,-.92f,-.76f,.34f,-.53f,-.50f,.75f,.78f,.13f,.39f,.10f,.57f,-.10f,.00f,.61f,-.29f,.92f,-.89f,.70f,-.25f,.01f,.56f,.17f,-.51f,.78f,.03f,-.45f,.37f,-.65f,.65f,-.18f,.28f,-.04f,-.68f,-.48f,.06f,.94f,.56f,.92f,.96f,-.46f,-.69f,.89f,-.43f,-.68f,.87f,-.73f,.93f,.02f,.99f,.41f,-.78f,.06f,-.56f,.04f,.68f,-.79f,.26f,.39f,.92f,-.71f,-.17f,-.36f,-.48f,-.17f,.97f,-.94f,-.46f,-.84f,-.79f,.40f,.20f,-.71f,-.70f,.28f,-.27f,-.63f,-.39f,-.36f,.35f,-.69f,-.94f,-.17f,-.97f,-.07f,-.78f,.84f,.17f,.54f,.29f,.76f,.73f,-.58f,-.03f,.27f,.44f,-.03f,-.83f,-.04f,-.93f,-.42f,.82f,-.01f,-.62f,-.01f,.61f,-.05f,-.69f,-.63f,-.02f,-.25f,-.27f,.86f,.48f,-.92f,.71f,.69f,-.93f,.23f,-.48f,.22f,.05f,-.00f,-.97f,.82f,-.47f,.53f,.68f,.65f,-.13f,-.29f,-.74f,.66f,-.47f,-.01f,-.62f,.18f,.78f,.35f,.34f,.34f,-.20f,.65f,.40f,-.78f,-.11f,.81f,-.36f,.74f,-.83f,-.66f,-.26f,.90f,.02f,.55f,-.97f,.54f,.50f,.25f,.57f,-.49f,-.15f,-.45f,.50f,-.79f,-.90f,-.91f,.31f,.97f,.77f,.76f,.73f,.40f,-.04f,-.25f,.71f,.44f,.71f,.59f,.88f,.39f,-.23f,.99f,.94f,.65f,.52f,-.55f,-.30f,-.81f,.21f,.37f,.47f,.26f,-.07f,-.08f,.62f,.99f,-.52f,-.04f,.75f,-.67f,-.23f,-.16f,-.25f,.50f,-.17f,-.09f,.16f,-.92f,.62f,.70f,.28f,.00f,.87f,-.73f,.97f,-.31f,.94f,.56f,-.00f,.37f,.05f,-.70f,-.03f,.80f,.95f,.08f,-.34f,.85f,.88f,.95f,.88f,.14f,-.22f,.94f,.47f,-.85f,-.07f,.67f,.55f,.74f,.87f,-.39f,.99f,.49f,-.14f,-.20f,.79f,-.88f,.79f,-.14f,.40f,.57f,-.32f,.47f,-.93f,-.97f,-.18f,.71f,-.96f,-.50f,-.65f,.69f,-.26f,-.94f,-.07f,-.20f,.55f,-.84f,.14f,.88f,-.94f,-.62f,.42f,-.49f,.70f,-.61f,-.36f,-.92f,-.96f,-.42f,-.15f,-.55f,.51f,.56f,-.03f,.16f,.27f,.06f,-.46f,-.54f,-.74f,.48f,.79f,-.30f,-.31f,.37f,-.63f,-.53f,.65f,-.54f,-.68f,.10f,.19f,-.51f,.68f,-.81f,.20f,.38f,.83f,.14f,.81f,.29f,.95f,-.57f,.83f,-.25f,-.26f,.10f,-.23f,.00f,-.77f,-.47f,-.36f,-.33f,-.38f,.48f,.15f,-.61f,-.07f,-.35f,-.52f,.46f,-.51f,-.56f,.32f,-.51f,-.87f,-.76f,.95f,-.88f,-.53f,-.51f,.18f,.95f,-.72f,-.06f,.85f,.07f,-.84f,.90f,-.19f,.14f,.54f,-.84f,-.57f,-.38f,.23f,.45f,.66f,.73f,-.87f,-.29f,.25f,-.74f,-.87f,-.54f,.08f,-.26f,-.15f,.40f,-.62f,-.73f,-.38f,.13f,.37f,.20f,.61f,-.78f,.84f,.99f,.80f,-.99f,.52f,.85f,-.61f,-.74f,-.52f,.03f,-.17f,-.91f,.64f,.61f,.82f,.49f,-.90f,.13f,.92f,.63f,-.38f,.40f,-.12f,-.46f,-.66f,.45f,-.68f,-.32f,-.41f,.97f,-.78f,-.82f,-.93f,-.19f,.33f,-.56f,-.21f,-.37f,.61f,-.37f,-.59f,.50f,.88f,.39f,.11f,.66f,-.67f,.49f,-.07f,.51f,-.20f,.65f,-.63f,-.87f,.96f,-.58f,-.24f,.29f,-.67f,-.39f,-.18f,.48f,.79f,.97f,.24f,.07f,.19f,.54f,-.45f,-.34f,-.13f,.43f,-.55f,-.31f,-.55f,.77f,-.52f,-.20f,.30f,.81f,.74f,.63f,.01f,-.70f,.17f,.50f,.34f,.73f,.01f,-.74f,.37f,.88f,-.40f,.39f,-.07f,-.03f,.14f,.89f,.70f,-.49f,-.41f,-.66f,.20f,.95f,.34f,.69f,-.01f,-.63f,.57f,.60f,.99f,.49f,-.45f,.77f,-.26f,-.83f,.29f,.69f,-.26f,.97f,-.25f,-.96f,.68f,-.17f,.99f,-.61f,.40f,-.56f,.50f,.07f,.74f,-.16f,-.44f,.72f,.50f,.02f,.74f,.95f,-.14f,-.40f,.82f,-.82f,-.58f,.06f,.77f,-.26f,.63f,.17f,-.44f,-.08f,.72f,.16f,-.91f,-.56f,.86f,.34f,.70f,.85f,-.93f,.99f,.33f,.32f,.23f,-.40f,-.96f,.07f,.01f,-.81f,-.39f,-.82f,-.50f,-.15f,.52f,.34f,-.96f,-.28f,.16f,.72f,-.91f,.03f,-.22f,.27f,-.53f,.27f,-.53f,.95f,.48f,.22f,.61f,-.30f,.67f,-.22f,-.06f,-.56f,-.74f,.98f,-.97f,.15f,.97f,-.71f,.53f,-.09f,.58f,-.80f,.85f,-.96f,-.50f,.22f,-.43f,-.73f,.36f,-.27f,.46f,.36f,.92f,.02f,.00f,.55f,-.00f,-.68f,-.39f,-.67f,-.58f,-.50f,.39f,-.18f,.48f,-.66f,.05f,-.38f,-.48f,.42f,-.72f,.30f,.30f,.13f,-.52f,-.47f,-.07f,-.92f,-.28f,.25f,-.19f,.83f,-.30f,.66f,.57f,-.98f,-.47f,.03f,.22f,-.55f,.14f,-.04f,-.81f,.99f,-.48f,-.36f,.24f,.08f,-.82f,-.74f,-.68f,-.12f,-.57f,-.55f,-.43f,-.57f,-.37f,-.84f,-.83f,.44f,.26f,-.04f,.03f,-.58f,-.84f,.78f,-.30f,-.00f,-.82f,.12f,-.00f,.13f,.49f,-.26f,.11f,.50f,-.44f,-.87f,-.15f,-.22f,-.21f,-.76f,-.79f,.80f,.97f,.97f,.06f,.87f,.13f,-.22f,.49f,-.11f,-.36f,-.12f,-.39f,.43f,.17f,.54f,-.17f,.93f,-.21f,.80f,.60f,-.40f,.02f,.85f,.32f,-.52f,-.62f,.54f,.01f,-.93f,.07f,.97f,-.46f,-.94f,-.94f,-.93f,-.41f,.80f,-.54f,-.23f,-.08f,.60f,-.56f,.26f,.12f,-.48f,.53f,.90f,.93f,-.84f,.71f,-.60f,.18f,.52f,-.59f,-.74f,.36f,-.74f,-.25f,-.77f,.13f,-.88f,.88f,-.23f,-.29f,.82f,.26f,.57f,-.29f,.07f,-.40f,.99f,-.89f,.56f,-.19f,.95f,.50f,-.70f,.31f,.75f,.63f,.70f,-.46f,-.34f,.88f,.03f,.56f,.71f,.34f,.17f,-.79f,-.78f,.47f,.14f,.00f,.49f,.10f,.42f,.35f,-.39f,.02f,-.02f,.19f,.36f,-.71f,.30f,-.58f,-.42f,-.23f,.24f,.17f,-.42f,.77f,-.88f,.07f,-.33f,.20f,-.05f,.37f,-.74f,.87f,.00f,-.23f,-.48f,-.95f,-.97f,.12f,-.86f,.58f,-.92f,.62f,.72f,-.53f,.03f,.70f,.26f,.95f,-.44f,.91f,.09f,-.31f,.53f,.90f,.92f,-.68f,.75f,-.85f,-.04f,-.42f,-.77f,-.67f,.94f,-.57f,-.93f,-.43f,-.74f,.95f,-.53f,-.50f,.69f,.93f,.61f,-.62f,.66f,.44f,-.98f,.54f,.33f,-.61f,-.95f,.92f,-.95f,.63f,-.25f,-.36f,-.70f,-.57f,-.36f,-.42f,.34f,-.01f,.27f,-.76f,-.22f,.46f,-.89f,-.02f,.57f,.82f,-.58f,-.52f,.49f,.02f,-.04f,.73f,-.47f,-.08f,.47f,-.98f,.94f,.82f,-.90f,-.88f,-.82f,.04f,.25f,.84f,-.26f,.19f,-.43f,.32f,-.66f,.25f,.44f,.47f,-.69f,-.14f,.64f,-.15f,-.50f,-.12f,.29f,-.08f,.83f,.48f,.07f,-.51f,.24f,-.81f,-.65f,-.54f,.12f,.52f,.15f,-.55f,-.94f,.47f,-.32f,.04f,.17f,-.13f,-.13f,.46f,.13f,-.85f,-.55f,.41f,-.90f,.84f,-.17f,-.73f,.29f,-.70f,.16f,.38f,.39f,-.93f,.96f,-.45f,.54f,-.76f,-.07f,-.02f,.84f,.91f,-.09f,.94f,-.19f,.88f,-.93f,.74f,.19f,-.39f,.11f,-.36f,-.57f,.01f,-.71f,-.67f,-.54f,.54f,-.22f,.06f,-.67f,.68f,-.45f,.64f,.46f,.69f,.75f,.94f,-.40f,-.05f,-.89f,-.07f,-.47f,.03f,-.31f,.54f,-.88f,.90f,-.37f,.01f,-.23f,-.96f,.12f,-.42f,.91f,-.62f,-.06f,.78f,.63f,-.72f,.28f,.11f,.11f,.15f,.73f,-.91f,.36f,.42f,.25f,-.36f,-.27f,.42f,-.51f,-.44f,.87f,.33f,.47f,.84f,.05f,.37f,-.60f,-.66f,.83f,.84f,.43f,.15f,-.90f,-.48f,-.59f,-.98f,.80f,-.43f,-.01f,.43f,.62f,-.52f,.36f,-.55f,.14f,.08f,-.32f,.65f,-.51f,.26f,.69f,-.71f,-.69f,.28f,-.66f,-.91f,-.15f,.47f,.34f,.56f,-.66f,.26f,.56f,.21f,-.67f,.85f,.30f,.83f,.17f,-.30f,.09f,.23f,.64f,.26f,.78f,-.09f,-.37f,-.45f,-.74f,-.96f,-.13f,.16f,-.94f,-.78f,.75f,.40f,.83f,.99f,-.91f,-.26f,.66f,-.55f,.89f,.86f,-.64f,-.06f,.33f,-.09f,-.87f,.63f,-.40f,.91f,-.89f,.76f,-.39f,-.93f,.77f,.61f,-.10f,-.78f,-.79f,-.93f,.44f,.58f,.16f,-.39f,-.97f,-.90f,.01f,-.36f,.70f,.26f,.90f,-.65f,.94f,-.19f,-.00f,.26f,-.39f,.25f,-.27f,.68f,.06f,.11f,.88f,.73f,.17f,.83f,-.39f,-.16f,-.43f,-.16f,.59f,.68f,.45f,.15f,.53f,-.56f,.99f,-.46f,-.50f,-.47f,-.63f,-.07f,.32f,.09f,.40f,-.42f,-.66f,.89f,-.73f,-.62f,.43f,.67f,-.31f,.65f,-.04f,-.37f,.77f,.10f,-.78f,.67f,.29f,-.37f,-.94f,-.00f,-.79f,-.15f,.08f,-.53f,.88f,.21f,.73f,.17f,-.13f,.93f,.86f,-.65f,-.12f,-.69f,.57f,.23f,-.62f,.03f,-.47f,.02f,.20f,-.42f,.97f,.85f,-.16f,-.87f,-.78f,.50f,.34f,-.81f,.87f,.29f,.61f,-.82f,-.50f,-.86f,.56f,-.14f,-.50f,-.35f,-.55f,-.37f,.37f,-.55f,.59f,-.27f,.47f,-.88f,.78f,.09f,-.61f,-.20f,-.81f,.10f,.11f,-.93f,-.80f,.74f,.65f,.27f,-.97f,.97f,-.04f,.49f,-.77f,.36f,.82f,.09f,-.32f,.08f,.67f,.08f,.48f,-.02f,-.07f,.55f,-.47f,.03f,-.17f,-.72f,-.05f,.79f,.60f,-.86f,.68f,.07f,.77f,-.60f,.83f,.47f,.65f,-.95f,-.00f,.71f,.97f,.63f,.75f,.83f,-.08f,-.89f,-.46f,-.46f,-.51f,.69f,.83f,.33f,.25f,.36f,-.47f,-.57f,.36f,-.15f,.03f,.72f,-.91f,-.46f,-.45f,-.08f,.21f,.39f,-.59f,.73f,.34f,-.42f,.35f,-.11f,-.38f,-.51f,.84f,-.07f,-.21f,-.58f,-.96f,-.91f,.00f,-.85f,-.29f,-.73f,-.45f,-.52f,-.13f,-.44f,-.79f,-.90f,.32f,.67f,.66f,.51f,-.57f,-.31f,.85f,-.46f,.91f,-.03f,-.13f,.81f,-.84f,.07f,.37f,-.51f,.73f,-.11f,.59f,-.38f,.64f,-.41f,-.80f,.72f,-.72f,-.70f,.50f,-.19f,-.42f,-.10f,.59f,.79f,-.69f,-.58f,.15f,.98f,.44f,.19f,-.18f,-.06f,.80f,-.27f,-.02f,-.02f,.25f,-.20f,.29f,-.80f,.39f,.89f,.99f,.47f,.43f,.71f,-.43f,-.26f,.62f,.10f,.79f,-.26f,.18f,.27f,-.75f,-.82f,-.09f,-.63f,.66f,-.29f,-.54f,-.21f,-.83f,-.42f,.85f,.69f,-.95f,-.85f,.04f,-.60f,.02f,-.00f,.19f,.01f,.89f,-.69f,.85f,-.70f,.46f,-.47f,-.43f,-.03f,.37f,-.66f,-.44f,.90f,.03f,-.07f,-.09f,.17f,-.42f,.16f,-.81f,-.05f,.56f,-.19f,-.23f,-.89f,-.20f,-.95f,-.47f,-.84f,.84f,-.91f,-.33f,-.69f,.53f,-.99f,.93f,.03f,-.88f,.03f,-.01f,.88f,.84f,-.71f,-.25f,-.04f,-.15f,-.92f,-.12f,-.74f,-.04f,.08f,-.61f,.18f,.93f,-.30f,-.81f,.94f,-.39f,-.41f,.21f,-.08f,-.97f,-.41f,.10f,-.44f,.14f,.30f,-.75f,.68f,-.09f,.43f,.49f,-.58f,.94f,-.01f,.26f,.75f,.77f,-.26f,.56f,.48f,-.50f,-.45f,.56f,-.13f,.53f,-.75f,-.18f,.05f,-.00f,.66f,.23f,-.50f,.57f,.37f,-.23f,.86f,-.05f,-.61f,-.22f,.72f,-.35f,.47f,-.50f,.78f,.81f,.05f,-.04f,-.05f,.41f,-.74f,.52f,.34f,-.02f,.30f,.91f,-.52f,-.66f,-.57f,-.49f,-.76f,-.29f,-.92f,.87f,-.26f,-.88f,-.78f,-.98f,-.61f,-.72f,.09f,.82f,.47f,-.19f,-.71f,.56f,.15f,-.07f,-.94f,.85f,.85f,.40f,-.13f,-.20f,.23f,-.02f,-.74f,.12f,.99f,-.44f,.34f,-.54f,.90f,-.95f,.54f,-.27f,.36f,.78f,-.86f,-.21f,.17f,.28f,-.15f,.66f,-.43f,.64f,.70f,-.95f,.27f,.71f,.27f,-.99f,-.11f,-.56f,.21f,-.84f,-.64f,-.26f,.05f,.31f,.56f,-.41f,-.77f,-.32f,.72f,.25f,.81f,.06f,-.92f,.89f,-.23f,.42f,.53f,-.15f,.11f,.88f,-.36f,-.43f,-.68f,.37f,-.62f,-.77f,-.03f,-.93f,.50f,.57f,-.90f,-.00f,.82f,.61f,-.62f,.38f,-.06f,-.32f,.31f,.66f,-.25f,.90f,.15f,.12f,.60f,.82f,-.86f,.54f,-.99f,.05f,.92f,-.75f,-.26f,-.32f,-.18f,.41f,.38f,-.32f,.27f,.08f,.74f,-.71f,-.26f,.34f,.99f,.46f,.11f,-.41f,-.09f,.19f,-.83f,.43f,.12f,.74f,.31f,-.35f,-.76f,.13f,.05f,.65f,.32f,-.62f,-.34f,-.37f,.94f,-.18f,.03f,-.30f,-.91f,.95f,.76f,.67f,.85f,.15f,.84f,-.21f,.87f,.83f,.56f,-.67f,.72f,.05f,-.70f,-.28f,-.87f,-.25f,.95f,.34f,-.01f,.20f,-.72f,.57f,.06f,.99f,.05f,.36f,.77f,-.89f,-.52f,-.65f,-.95f,-.49f,-.62f,.12f,.17f,-.89f,.27f,-.72f,.09f,.94f,.46f,-.39f,-.27f,-.34f,.05f,-.43f,.76f,.59f,.41f,-.69f,.50f,.30f,.69f,.31f,.31f,.60f,-.81f,-.34f,-.31f,.32f,.72f,-.37f,.08f,-.61f,-.77f,-.86f,.13f,.52f,-.28f,-.43f,-.70f,.34f,.41f,-.85f,-.19f,.09f,.88f,-.40f,-.92f,-.62f,-.97f,.11f,-.54f,-.51f,-.99f,-.03f,.69f,.57f,-.56f,-.87f,.39f,.37f,.72f,.17f,-.28f,-.86f,.46f,-.77f,.49f,-.73f,-.65f,-.81f,-.78f,.02f,.22f,-.58f,.30f,-.95f,-.91f,.94f,-.73f,.67f,.45f,-.49f,.37f,.48f,-.76f,-.59f,.32f,.97f,.01f,.46f,-.42f,.58f,.26f,.62f,.58f,-.26f,-.52f,-.24f,-.72f,-.88f,.89f,.99f,.20f,.48f,.11f,.82f,-.51f,.46f,-.07f,.19f,.66f,-.22f,-.68f,.37f,.81f,.51f,-.81f,-.75f,-.51f,-.26f,-.13f,.47f,.59f,-.71f,-.43f,-.33f,.82f,-.15f,-.18f,.80f,-.09f,.19f,.75f,.20f,.31f,.28f,-.44f,-.43f,.18f,.53f,.76f,.15f,-.63f,-.39f,-.84f,.01f,-.05f,-.33f,.32f,.10f,.00f,.80f,.46f,-.78f,.84f,-.20f,.60f,.62f,.52f,-.79f,.21f,-.97f,-.94f,.45f,-.46f,-.97f,-.53f,-.37f,-.97f,-.68f,.09f,-.80f,.50f,.09f,.56f,.10f,.81f,.33f,.81f,-.32f,-.51f,-.00f,-.15f,.92f,.94f,-.79f,-.72f,.43f,-.04f,.42f,.21f,.99f,.39f,.06f,-.01f,-.59f,.18f,-.94f,-.50f,-.55f,-.13f,.32f,.94f,-.87f,.08f,-.72f,-.17f,-.37f,.21f,.01f,-.28f,.60f,.05f,-.99f,-.50f,-.74f,.32f,.49f,-.29f,.12f,-.52f,-.90f,-.68f,-.45f,.73f,.52f,.03f,-.71f,.39f,.92f,-.95f,.83f,-.60f,-.44f,-.66f,-.59f,-.76f,-.79f,-.10f,-.77f,.35f,.88f,-.49f,.08f,.06f,-.97f,-.84f,-.47f,.40f,.51f,.13f,-.06f,-.35f,.81f,-.99f,-.09f,-.92f,-.38f,-.78f,-.10f,.76f,-.08f,-.31f,-.59f,-.86f,-.34f,-.86f,.54f,.10f,.16f,-.01f,-.68f,-.22f,-.15f,.46f,.92f,-.75f,-.84f,-.62f,-.74f,.40f,.90f,-.41f,-.14f,-.73f,.27f,.86f,.53f,-.41f,.35f,.96f,.34f,.96f,-.83f,.88f,-.35f,.34f,-.71f,-.27f,-.28f,.38f,-.98f,-.30f,-.67f,.52f,-.03f,.63f,-.54f,.26f,.41f,-.01f,-.10f,-.38f,.61f,-.77f,.92f,.58f,-.98f,.93f,-.27f,-.08f,.78f,-.64f,-.93f,.79f,-.36f,.94f,-.10f,.48f,.73f,-.44f,.00f,-.94f,-.90f,.04f,-.72f,.09f,-.57f,.46f,-.62f,-.41f,.09f,-.21f,-.56f,.79f,.49f,-.78f,.43f,.61f,-.29f,-.10f,-.54f,.64f,-.95f,.35f,-.50f,.99f,.08f,-.07f,-.00f,.31f,.69f,.78f,.68f,-.34f,-.39f,.17f,.51f,-.25f,.71f,.83f,.58f,.87f,.79f,.14f,.26f,.54f,.26f,.06f,.85f,-.29f,-.79f,-.57f,-.02f,-.07f,.77f,.16f,.17f,-.35f,.63f,.61f,.89f,.52f,.14f,.77f,.57f,.80f,-.81f,-.93f,-.18f,-.32f,-.88f,.24f,-.25f,-.75f,.88f,.43f,.09f,-.08f,-.15f,.53f,-.93f,.02f,-.48f,.82f,-.81f,.86f,-.84f,.15f,.64f,-.20f,.92f,.10f,.26f,.75f,-.50f,.50f,-.75f,.48f,-.27f,-.39f,-.82f,-.71f,.60f,.99f,-.84f,.42f,-.01f,.39f,-.86f,-.06f,.66f,.70f,.17f,.75f,-.97f,.57f,-.44f,-.68f,.13f,-.20f,-.69f,.99f,.73f,.38f,-.51f,-.23f,.26f,-.85f,.57f,-.62f,.72f,.69f,.88f,-.09f,-.30f,-.30f,.37f,-.62f,-.74f,-.02f,-.33f,-.69f,.00f,-.53f,-.99f,-.39f,-.88f,.56f,.09f,.33f,.56f,.75f,.01f,-.68f,.87f,.80f,-.23f,-.48f,-.20f,.08f,.64f,.10f,-.14f,.13f,.50f,-.92f,.43f,-.61f,-.04f,-.90f,.65f,.76f,.58f,-.21f,.63f,.93f,-.61f,.01f,-.13f,.09f,.89f,.46f,.82f,.95f,.23f,-.86f,-.16f,.38f,.00f,.70f,-.42f,.56f,-.55f,.05f,.88f,-.22f,-.51f,-.84f,.28f,-.03f,.74f,.53f,.40f,-.98f,-.52f,-.50f,-.67f,.84f,-.72f,-.78f,.14f,.08f,-.44f,-.23f,.38f,-.01f,.76f,.26f,.97f,.80f,-.84f,-.20f,-.19f,.51f,.42f,-.51f,-.81f,.69f,.10f,-.98f,-.62f,-.82f,.09f,-.43f,.54f,.47f,.43f,-.85f,.87f,-.86f,.24f,.23f,-.46f,-.33f,.86f,-.90f,.26f,-.42f,-.48f,-.12f,.55f,-.28f,-.49f,-.40f,.45f,.72f,.69f,.10f,-.20f,.02f,.65f,-.55f,.70f,.27f,-.11f,.44f,.87f,-.83f,.09f,.60f,.11f,.05f,-.33f,-.35f,.74f,.23f,-.91f,.35f,-.47f,.52f,-.70f,.56f,.24f,-.60f,-.60f,.42f,-.40f,-.30f,-.87f,.61f,-.53f,.61f,.85f,.00f,-.19f,-.10f,-.72f,-.07f,-.62f,.46f,-.16f,-.75f,-.96f,-.92f,.87f,-.95f,-.25f,-.40f,.63f,.45f,-.22f,-.49f,.27f,.05f,-.96f,-.89f,.43f,.38f,.50f,.09f,.78f,-.73f,.67f,.55f,-.53f,.89f,-.34f,.99f,-.29f,-.04f,-.42f,-.34f,-.74f,-.36f,.38f,.94f,.29f,-.11f,.10f,-.08f,-.21f,-.18f,.88f,-.64f,-.58f,-.18f,.77f,-.47f,-.53f,-.00f,.13f,-.41f,.71f,.52f,-.39f,-.25f,.15f,.09f,.49f,.99f,-.09f,.97f,.21f,-.24f,.76f,-.25f,.81f,-.44f,.96f,-.85f,.48f,-.86f,.12f,.94f,-.09f,-.69f,.98f,.26f,.63f,.65f,.70f,.95f,.96f,.49f,-.25f,.66f,-.97f,-.62f,.85f,.04f,.16f,.22f,.71f,-.52f,-.55f,.08f,-.85f,-.87f,.73f,.08f,.94f,-.34f,-.82f,.22f,-.25f,-.92f,-.96f,.94f,-.47f,.76f,-.83f,.22f,.42f,-.80f,-.22f,-.84f,-.52f,.15f,.20f,.96f,.20f,.63f,-.32f,-.76f,.32f,-.71f,-.68f,.55f,-.45f,.59f,-.41f,-.10f,-.62f,.56f,.07f,-.22f,-.90f,.39f,-.53f,.98f,-.55f,-.31f,-.48f,.22f,-.78f,-.57f,-.88f,-.97f,.26f,-.74f,.20f,-.82f,-.34f,.27f,.96f,-.14f,-.60f,-.20f,-.34f,.68f,-.00f,.57f,.87f,-.48f,.10f,.39f,-.22f,.97f,.45f,-.41f,.85f,-.18f,-.22f,-.40f,.67f,-.09f,-.61f,-.28f,.71f,.45f,-.27f,-.67f,.50f,-.10f,-.09f,-.75f,.96f,.86f,.27f,-.83f,-.35f,-.30f,.37f,.44f,.78f,-.50f,-.53f,-.40f,-.43f,-.04f,.46f,-.57f,-.60f,-.13f,.90f,.32f,.92f,.79f,.48f,-.76f,.87f,.11f,-.17f,-.50f,-.00f,.02f,.61f,.33f,.64f,-.80f,-.48f,.91f,.81f,-.73f,-.02f,.94f,-.63f,.19f,.39f,-.50f,-.91f,-.99f,.95f,-.75f,.67f,.82f,-.54f,-.76f,.50f,.90f,.46f,-.74f,.69f,-.86f,.72f,.52f,.39f,-.46f,-.36f,.34f,.21f,-.44f,.14f,.35f,.13f,-.84f,.95f,.32f,-.82f,-.62f,-.69f,-.50f,.59f,.47f,-.97f,.15f,.59f,.65f,-.26f,.50f,-.03f,-.96f,.97f,-.99f,-.67f,-.73f,-.48f,-.64f,-.84f,-.94f,.62f,.14f,-.32f,-.47f,-.74f,.07f,.05f,.87f,-.83f,.49f,-.32f,-.32f,.03f,.51f,.06f,-.77f,.75f,-.39f,-.78f,.10f,.95f,.38f,-.08f,-.66f,-.07f,.09f,-.09f,-.09f,.83f,-.12f,.43f,-.65f,.41f,.68f,.93f,.88f,-.24f,.70f,-.97f,-.40f,-.96f,-.45f,-.35f,-.29f,.91f,-.28f,.92f,-.57f,-.37f,-.02f,.37f,-.37f,-.01f,-.48f,.74f,-.28f,.21f,.74f,-.99f,.45f,.91f,-.28f,-.11f,-.86f,-.22f,.48f,-.34f,.52f,.17f,.04f,.23f,-.54f,-.80f,-.60f,.04f,.47f,-.14f,.33f,.57f,.22f,-.67f,.02f,.69f,.93f,.69f,.93f,.99f,.75f,.47f,.21f,-.42f,.02f,.20f,.03f,.12f,.76f,.04f,-.43f,-.10f,-.52f,-.26f,-.10f,.78f,-.81f,-.72f,-.76f,.87f,.88f,.83f,-.27f,.82f,-.97f,-.59f,.80f,.66f,.66f,.59f,-.73f,.03f,-.16f,.69f,-.52f,.22f,.82f,-.11f,-.25f,-.39f,.56f,.93f,-.43f,-.73f,-.28f,-.06f,-.49f,-.95f,.39f,.51f,.29f,.40f,-.46f,-.11f,.60f,.03f,-.57f,.24f,-.55f,.88f,-.33f,.64f,-.79f,.59f,-.05f,.94f,.00f,.74f,.44f,-.86f,.23f,-.03f,-.66f,.45f,.08f,-.53f,.09f,.98f,-.28f,-.10f,.89f,.18f,.45f,-.98f,-.89f,.47f,.78f,-.11f,.53f,-.35f,.46f,.00f,.49f,-.13f,-.58f,-.56f,.73f,.21f,.20f,.72f,-.71f,-.68f,-.54f,.61f,-.80f,.53f,.90f,.20f,.25f,-.29f,.91f,-.88f,.04f,.98f,-.18f,-.16f,.28f,.28f,.60f,.35f,-.32f,.48f,-.99f,-.57f,.41f,.93f,-.33f,-.69f,.68f,-.99f,.40f,-.31f,.58f,.70f,-.95f,-.97f,.59f,-.71f,-.43f,.78f,-.92f,-.01f,.79f,-.34f,-.14f,.25f,.86f,.62f,-.12f,-.53f,.77f,-.30f,-.50f,-.82f,.76f,.27f,-.59f,.69f,-.87f,-.03f,.82f,.05f,.58f,.96f,.52f,.04f,.80f,-.66f,-.06f,.88f,-.59f,-.19f,-.47f,.84f,.15f,.90f,-.08f,.69f,-.34f,.12f,.14f,.86f,-.56f,-.59f,.08f,.27f,-.13f,.00f,-.00f,-.47f,.34f,.44f,.17f,-.57f,-.19f,-.09f,-.31f,.03f,-.60f,.40f,.58f,-.43f,-.81f,.08f,.04f,-.28f,.74f,.00f,-.59f,-.57f,-.82f,.99f,-.36f,.44f,-.51f,.26f,.75f,-.56f,-.86f,.85f,-.63f,.49f,.81f,-.53f,.23f,-.35f,-.25f,-.44f,-.03f,.23f,.98f,-.37f,-.25f,-.60f,.81f,.16f,.95f,.77f,-.18f,.45f,-.35f,.47f,.74f,.65f,.81f,-.02f,-.39f,-.75f,.33f,.35f,-.76f,-.12f,-.62f,.06f,.47f,.47f,-.74f,.58f,-.68f,-.77f,-.12f,-.41f,.26f,.39f,-.37f,.47f,.81f,.00f,.80f,-.59f,.74f,-.11f,-.81f,-.10f,-.03f,.88f,-.65f,.58f,-.93f,-.09f,-.30f,.72f,-.75f,-.10f,.01f,.21f,-.17f,.37f,-.96f,-.44f,-.69f,.36f,.89f,.24f,.06f,.20f,-.51f,.20f,-.25f,.64f,.06f,-.91f,.19f,-.71f,-.09f,.85f,.49f,-.39f,.80f,.42f,.61f,-.52f,.61f,-.13f,.48f,.67f,.60f,.93f,-.02f,.58f,.01f,-.17f,-.50f,-.67f,-.74f,-.12f,.92f,-.61f,-.21f,.30f,-.70f,-.56f,-.91f,-.65f,.13f,-.36f,-.91f,.93f,.69f,-.95f,.67f,-.46f,.26f,.59f,.11f,.64f,-.83f,-.00f,-.85f,.50f,.25f,.80f,-.83f,-.54f,-.96f,.68f,-.39f,.84f,.80f,.84f,.86f,-.60f,-.73f,-.90f,.22f,.52f,-.70f,-.70f,.35f,.57f,.55f,-.94f,.51f,.07f,-.57f,-.20f,.16f,.66f,.71f,.46f,-.03f,.11f,-.61f,-.84f,-.57f,.12f,-.37f,-.97f,-.83f,.47f,.40f,.54f,.79f,.95f,.40f,.23f,.45f,.66f,.33f,.61f,.90f,-.66f,-.18f,.95f,.91f,-.89f,-.66f,.37f,-.80f,-.36f,.02f,.23f,-.57f,-.19f,-.11f,-.01f,-.47f,-.67f,-.64f,.80f,-.34f,-.08f,-.54f,.96f,-.41f,-.43f,.52f,.30f,-.02f,.68f,-.82f,-.06f,-.24f,.53f,-.12f,-.64f,-.57f,-.93f,.17f,-.62f,-.57f,-.18f,.44f,-.93f,.27f,.47f,.02f,-.66f,.84f,-.56f,-.86f,-.30f,.45f,-.87f,-.18f,-.78f,-.58f,-.87f,-.01f,-.39f,-.82f,-.88f,.73f,-.96f,-.08f,-.83f,-.78f,-.40f,.55f,-.96f,-.72f,-.48f,-.58f,.56f,.17f,.51f,.81f,-.72f,-.46f,.98f,.59f,-.78f,-.00f,-.97f,.94f,.53f,.29f,.90f,.85f,-.83f,-.66f,-.22f,-.23f,.41f,-.13f,-.10f,.66f,-.18f,.83f,-.50f,-.14f,-.22f,.75f,-.81f,.56f,-.60f,-.11f,-.24f,-.37f,-.44f,.27f,-.84f,.21f,-.47f,.87f,.91f,-.95f,-.09f,.97f,.08f,-.49f,-.27f,.22f,.50f,-.54f,.41f,.84f,.25f,.24f,.12f,.76f,-.50f,.78f,-.35f,.66f,-.78f,.10f,.81f,.32f,-.41f,-.20f,-.99f,.80f,-.91f,-.80f,.53f,.83f,-.43f,-.12f,.26f,-.28f,.33f,-.88f,.39f,.15f,-.11f,-.35f,.46f,-.47f,-.30f,.42f,-.33f,.95f,.59f,-.43f,-.12f,.57f,-.57f,-.47f,.60f,.34f,.80f,.80f,-.64f,-.72f,.01f,.49f,-.03f,.09f,.38f,-.65f,-.22f,.61f,-.82f,.42f,-.33f,.91f,.55f,.45f,.85f,-.06f,-.58f,.64f,.12f,-.57f,.68f,.24f,-.88f,-.67f,-.65f,-.22f,-.11f,-.83f,-.82f,.76f,.33f,-.14f,.97f,-.18f,-.04f,.97f,.00f,.08f,.15f,.52f,-.19f,-.46f,-.50f,-.59f,.69f,.06f,-.14f,.27f,-.75f,.27f,-.14f,.15f,-.57f,-.42f,-.18f,.43f,.44f,-.65f,-.87f,-.49f,-.37f,.82f,-.72f,.01f,-.88f,.14f,-.92f,-.50f,.83f,-.14f,-.16f,-.50f,-.05f,-.63f,-.90f,.83f,.74f,.00f,.44f,-.66f,.93f,.27f,-.28f,.18f,-.80f,-.54f,-.08f,.22f,.43f,.16f,-.25f,-.14f,.25f,.55f,-.85f,.72f,.33f,.12f,-.65f,-.17f,.26f,-.40f,.78f,-.50f,.07f,-.98f,-.60f,-.38f,.95f,.75f,-.10f,-.30f,.57f,-.67f,.12f,.46f,-.23f,-.93f,-.34f,-.63f,.42f,.20f,.25f,-.83f,.09f,-.35f,-.70f,.73f,-.71f,-.65f,.53f,.93f,-.30f,.42f,.10f,.73f,-.96f,-.08f,.64f,.66f,-.20f,.62f,-.90f,-.63f,-.55f,.72f,.64f,-.42f,-.01f,-.62f,-.82f,.68f,-.86f,.48f,-.04f,-.02f,-.91f,-.99f,-.33f,-.46f,-.06f,-.92f,-.23f,.15f,.26f,.37f,.23f,.08f,.47f,-.78f,.86f,.52f,-.27f,-.01f,-.54f,-.42f,-.25f,-.20f,-.13f,.64f,.32f,.45f,.34f,-.13f,.99f,.45f,.81f,-.63f,-.27f,-.59f,-.29f,-.44f,.89f,-.01f,-.17f,.79f,.08f,.59f,-.01f,-.26f,.10f,.27f,.28f,-.12f,-.73f,.19f,.16f,-.12f,.49f,.43f,.74f,.56f,.90f,.28f,-.46f,-.62f,.47f,-.35f,-.36f,-.99f,.89f,-.64f,-.33f,-.43f,.97f,-.34f,.37f,-.16f,-.39f,.18f,-.95f,-.41f,.41f,-.21f,-.82f,-.93f,-.90f,.53f,.25f,.29f,-.84f,-.39f,-.72f,-.21f,-.78f,-.97f,-.92f,-.34f,.81f,-.08f,.34f,.26f,-.74f,-.26f,-.20f,.71f,.67f,-.08f,.35f,-.63f,-.11f,-.50f,-.71f,-.26f,-.46f,.01f,-.24f,.73f,-.22f,-.85f,-.68f,-.21f,-.56f,.16f,-.04f,-.77f,.24f,-.66f,.77f,.25f,-.27f,-.24f,-.36f,.89f,-.65f,.62f,.00f,-.74f,.22f,.82f,.72f,.58f,-.44f,.72f,-.55f,-.20f,.11f,-.62f,.48f,-.24f,-.02f,-.81f,.69f,-.58f,.26f,-.64f,-.48f,.79f,.38f,.68f,.70f,-.19f,.79f,.89f,-.57f,-.24f,-.57f,-.20f,-.80f,.54f,-.12f,-.94f,-.04f,.40f,-.16f,-.99f,.12f,.73f,-.02f,.63f,-.76f,-.40f,-.82f,.60f,.48f,.26f,-.77f,-.68f,.10f,-.81f,.90f,.97f,.10f,-.12f,-.07f,-.64f,.36f,-.37f,.25f,-.38f,-.65f,-.30f,-.71f,.35f,.69f,.29f,-.60f,.66f,.53f,.69f,-.27f,.04f,-.38f,-.47f,-.00f,-.30f,.71f,.44f,-.50f,.52f,.75f,.18f,-.47f,-.15f,.46f,-.42f,.32f,-.97f,.16f,.67f,-.13f,-.59f,.49f,-.02f,-.91f,-.05f,-.89f,.68f,-.21f,.57f,-.17f,.37f,-.36f,-.44f,.40f,.31f,-.51f,-.56f,.09f,.80f,-.61f,-.36f,-.66f,.29f,-.00f,-.72f,.14f,-.95f,-.18f,-.43f,-.25f,.28f,-.64f,-.37f,-.29f,.15f,.48f,.29f,-.86f,.43f,.85f,.56f,-.66f,-.93f,-.00f,.06f,.00f,-.73f,-.29f,-.29f,.59f,-.60f,-.24f,-.88f,.99f,.39f,.71f,-.58f,-.46f,-.86f,-.65f,-.17f,.28f,.85f,.74f,.32f,-.14f,.59f,.37f,-.94f,.28f,-.50f,-.88f,.36f,-.21f,-.22f,.55f,-.49f,-.22f,.33f,.34f,.59f,-.94f,.82f,-.06f,.82f,.71f,.03f,-.11f,-.02f,.85f,-.25f,.86f,-.30f,-.99f,.27f,.17f,.78f,.24f,-.85f,.97f,.35f,.44f,-.86f,.22f,.69f,.89f,-.11f,.94f,-.97f,-.73f,-.46f,-.85f,.33f,-.01f,-.37f,.23f,-.67f,.02f,-.89f,.99f,-.46f,.40f,-.60f,-.35f,-.13f,-.38f,-.18f,-.50f,-.52f,.06f,.01f,.68f,.69f,.17f,-.36f,.68f,-.01f,.22f,.96f,.58f,-.28f,.10f,-.49f,-.21f,-.74f,.06f,-.03f,.36f,.47f,-.94f,-.13f,.66f,-.03f,.81f,-.10f,-.34f,-.74f,-.02f,.36f,.13f,-.09f,-.23f,-.15f,-.84f,.61f,.63f,-.06f,-.59f,.49f,-.83f,.43f,-.51f,-.26f,-.49f,.79f,.92f,.69f,-.51f,.43f,.57f,.59f,.62f,.04f,-.66f,.12f,-.61f,.15f,-.29f,-.66f,-.05f,.27f,.83f,.62f,-.89f,.37f,.08f,-.41f,.60f,.57f,.27f,-.08f,.62f,-.75f,.71f,-.56f,-.08f,.98f,.63f,-.46f,-.76f,.75f,-.99f,-.80f,-.58f,-.16f,-.55f,.09f,.42f,-.11f,.18f,-.87f,-.89f,.04f,.24f,-.86f,-.20f,.09f,-.76f,.77f,-.39f,.60f,-.61f,-.04f,.16f,.57f,-.39f,-.75f,-.99f,-.77f,.91f,.58f,-.29f,.48f,.22f,.73f,-.29f,-.28f,.93f,-.78f,-.19f,.27f,-.76f,-.93f,-.95f,-.98f,.74f,.50f,.09f,-.76f,-.31f,-.95f,.18f,.95f,-.90f,-.87f,.87f,.39f,.91f,.91f,.43f,.21f,-.33f,-.82f,-.85f,.79f,.25f,.62f,.44f,-.19f,.80f,.77f,-.08f,.74f,-.25f,.90f,.93f,.55f,-.46f,-.63f,.73f,.51f,.31f,-.30f,-.46f,-.26f,.82f,-.44f,-.96f,-.05f,.25f,.67f,.37f,-.96f,-.70f,-.50f,-.33f,-.30f,.19f,.45f,.96f,.18f,-.44f,.10f,-.29f,-.83f,.18f,.14f,.91f,-.76f,.33f,-.75f,-.06f,.35f,.35f,-.22f,-.42f,-.34f,.60f,-.56f,.95f,-.35f,-.13f,-.58f,-.84f,.89f,-.67f,-.84f,.78f,-.29f,.83f,-.03f,.74f,-.98f,.54f,-.33f,.40f,.79f,.78f,.47f,-.60f,.66f,.69f,-.25f,-.38f,.90f,.91f,-.22f,.55f,-.30f,-.48f,-.16f,-.09f,-.23f,-.68f,-.62f,.49f,.34f,.18f,.88f,.08f,.95f,-.59f,.40f,.52f,.19f,.94f,.36f,.96f,-.31f,.71f,-.19f,.67f,.87f,-.39f,-.55f,-.01f,-.14f,.28f,-.75f,.70f,-.17f,.18f,.64f,.50f,-.83f,.85f,.85f,.38f,.04f,-.46f,.41f,-.21f,-.78f,.32f,.46f,.95f,-.16f,-.31f,-.24f,.06f,-.25f,-.94f,.99f,.75f,-.00f,-.42f,-.72f,.61f,-.06f,.78f,.92f,.25f,.90f,-.37f,.72f,.21f,-.72f,.36f,.76f,-.53f,-.36f,.45f,-.28f,-.47f,-.46f,.47f,-.30f,.30f,-.72f,.73f,-.61f,-.39f,.06f,.35f,.77f,-.38f,.52f,-.03f,-.85f,-.20f,.02f,.64f,-.72f,.10f,.11f,.81f,-.07f,.72f,.87f,.81f,-.21f,-.03f,-.86f,-.18f,.01f,-.77f,-.60f,.16f,.32f,-.46f,.00f,-.00f,-.77f,-.04f,.92f,.48f,-.84f,-.65f,-.93f,-.77f,-.11f,.77f,.86f,.96f,.31f,-.18f,-.94f,.66f,-.97f,.51f,.39f,-.66f,-.74f,.82f,.91f,.90f,-.22f,-.89f,.72f,.98f,-.14f,.73f,.68f,.68f,-.20f,.44f,-.76f,.40f,-.93f,-.43f,-.89f,-.74f,-.18f,-.79f,-.69f,.32f,.73f,-.85f,-.06f,.72f,-.25f,.37f,.76f,-.20f,-.70f,.27f,-.94f,.75f,-.29f,-.25f,-.82f,.38f,.50f,.56f,.62f,-.41f,-.10f,-.38f,.76f,-.42f,.89f,.16f,-.83f,-.05f,-.08f,-.41f,.55f,.01f,.93f,-.79f,-.46f,.63f,-.93f,-.26f,.51f,.57f,-.76f,-.94f,.21f,-.83f,.51f,-.38f,-.00f,-.35f,-.94f,.32f,-.70f,-.05f,-.73f,.19f,-.52f,.32f,-.48f,.99f,.18f,.36f,-.73f,.48f,.74f,.53f,.41f,.85f,-.42f,.46f,.18f,.66f,.99f,-.55f,-.02f,.54f,-.89f,-.78f,.68f,.34f,.90f,-.34f,.55f,-.50f,-.80f,-.26f,-.91f,-.72f,.75f,.23f,.87f,.15f,.77f,-.64f,-.78f,.03f,-.29f,-.31f,.02f,-.16f,.01f,.50f,-.35f,-.65f,.22f,.53f,-.25f,-.31f,.61f,.59f,-.83f,-.30f,-.53f,-.76f,-.28f,.71f,.52f,-.77f,.86f,-.01f,.92f,.90f,.20f,.20f,.49f,.38f,-.95f,.59f,-.25f,.42f,-.33f,.29f,.17f,-.17f,.78f,-.75f,.99f,-.43f,-.33f,-.61f,.41f,-.22f,-.25f,-.16f,-.74f,-.06f,.24f,.60f,-.96f,.06f,-.13f,-.27f,-.04f,-.04f,.45f,.25f,.83f,-.31f,.27f,-.06f,-.40f,.68f,-.52f,.92f,-.71f,-.78f,.93f,-.84f,-.51f,-.30f,-.15f,.02f,-.59f,-.64f,.06f,-.18f,-.55f,-.96f,.12f,-.89f,.97f,.91f,-.73f,.06f,.57f,-.82f,.59f,-.86f,.08f,.45f,-.95f,.67f,-.65f,-.29f,-.36f,.04f,-.82f,.10f,.55f,.32f,.55f,.32f,.81f,-.92f,.09f,-.35f,.88f,.31f,.06f,.89f,.00f,.37f,-.45f,.58f,.97f,.95f,.34f,-.41f,-.84f,.45f,-.81f,.73f,-.11f,.06f,-.68f,-.92f,.59f,.80f,.60f,-.04f,-.18f,.44f,-.83f,.71f,.68f,.02f,.62f,.83f,.60f,.81f,-.92f,.86f,-.57f,.26f,.30f,-.26f,.29f,-.74f,-.21f,.65f,.40f,.43f,-.95f,.36f,-.28f,.12f,-.89f,-.23f,.86f,-.11f,.70f,.52f,-.22f,-.83f,-.63f,.08f,.58f,-.88f,-.34f,-.06f,.44f,.10f,.36f,.99f,.04f,.41f,-.73f,.41f,.57f,-.41f,.35f,.36f,-.37f,.64f,-.70f,-.33f,-.57f,-.38f,.09f,.50f,-.09f,-.95f,.59f,.07f,.25f,.75f,-.29f,-.46f,-.69f,-.93f,-.49f,-.03f,-.25f,.65f,-.15f,.79f,.76f,-.73f,.57f,-.90f,.74f,.99f,.20f,.09f,.33f,-.55f,.42f,-.98f,-.26f,-.48f,.96f,.83f,-.75f,-.64f,-.09f,-.73f,.96f,-.26f,-.76f,.83f,.54f,.66f,.54f,.64f,-.58f,.45f,-.67f,.81f,.00f,-.77f,.66f,.36f,.93f,.75f,-.62f,.65f,-.52f,.08f,.02f,.12f,.35f,.85f,.83f,-.07f,-.62f,-.78f,-.98f,-.78f,.43f,.53f,-.55f,.04f,.99f,-.57f,-.52f,.78f,.11f,-.00f,.88f,.10f,-.43f,-.44f,-.51f,.73f,-.74f,.53f,-.26f,-.58f,.07f,-.48f,.23f,.94f,-.47f,-.53f,.04f,.18f,-.16f,-.86f,-.90f,-.04f,.05f,.03f,.53f,-.11f,-.86f,.80f,.34f,.61f,-.00f,-.99f,.55f,-.20f,-.84f,.21f,-.66f,-.43f,-.93f,-.83f,.00f,-.62f,-.59f,-.16f,-.56f,-.11f,.55f,-.30f,.96f,-.67f,.05f,.17f,.30f,-.86f,-.44f,.05f,.75f,-.44f,-.03f,-.61f,-.89f,.40f,.35f,-.42f,.35f,-.97f,.58f,.32f,.84f,-.13f,.01f,.89f,.60f,.06f,.80f,.67f,-.54f,.59f,-.52f,.06f,.86f,.19f,.04f,.75f,.86f,.11f,.00f,-.72f,-.85f,.09f,.78f,-.68f,.90f,.06f,.38f,-.14f,.97f,.07f,-.60f,.60f,-.78f,-.38f,.01f,-.99f,.61f,.53f,-.65f,.16f,.32f,.61f,-.94f,.88f,.75f,-.05f,-.24f,-.84f,-.49f,.89f,.39f,.22f,.20f,-.29f,.74f,-.01f,.08f,-.24f,-.80f,.57f,-.16f,.00f,-.48f,.68f,-.06f,.78f,-.66f,-.91f,.30f,-.72f,-.20f,.28f,-.94f,-.44f,-.29f,.27f,-.84f,.86f,-.47f,-.62f,.29f,-.59f,.71f,-.10f,-.47f,-.40f,-.62f,-.45f,.01f,-.88f,-.47f,-.11f,-.27f,.33f,-.12f,.68f,-.96f,.30f,-.73f,.13f,.70f,.78f,.18f,.35f,-.82f,-.65f,-.72f,.44f,-.03f,.12f,-.68f,-.68f,.48f,.89f,-.32f,.84f,.32f,.14f,-.68f,.44f,.17f,.56f,-.76f,.24f,-.88f,.04f,-.46f,-.34f,.28f,-.31f,-.97f,-.21f,-.78f,-.84f,-.22f,-.73f,.07f,-.56f,.01f,-.75f,-.19f,-.41f,.78f,-.23f,.93f,.91f,.96f,-.41f,-.27f,-.20f,-.59f,.25f,.38f,-.63f,.24f,.62f,.44f,.71f,-.55f,.61f,.50f,-.56f,.77f,.44f,.25f,-.17f,-.32f,.83f,-.33f,.16f,.90f,-.58f,-.63f,.28f,-.65f,.02f,.60f,.67f,.30f,.09f,-.48f,.54f,-.07f,.32f,-.51f,-.83f,.67f,.86f,.48f,-.14f,-.03f,-.49f,.17f,-.59f,-.21f,-.26f,-.93f,.80f,-.35f,.52f,-.40f,-.89f,-.77f,-.53f,.56f,-.58f,.45f,.54f,.81f,.05f,.54f,.11f,.78f,.53f,.88f,.13f,-.95f,-.23f,-.40f,.29f,-.37f,-.57f,.39f,.24f,.14f,.86f,.36f,-.72f,.12f,-.54f,.60f,.22f,.32f,-.02f,.69f,-.43f,-.64f,.92f,.37f,.07f,.16f,.05f,.11f,.19f,.21f,-.60f,-.02f,.43f,.84f,-.97f,-.12f,-.06f,-.11f,-.35f,.77f,-.12f,-.44f,-.02f,-.09f,.34f,.37f,.17f,-.63f,.07f,.74f,.57f,-.97f,.57f,.22f,.47f,-.92f,-.97f,-.01f,.47f,.08f,-.31f,.74f,.79f,-.68f,-.89f,-.10f,-.80f,-.91f,.06f,-.40f,.76f,.27f,.79f,.96f,-.95f,.62f,-.09f,.41f,.03f,-.03f,-.85f,-.51f,.92f,-.67f,.44f,.20f,-.29f,-.01f,.48f,.76f,-.74f,-.95f,-.62f,-.46f,.55f,.61f,-.51f,.26f,.53f,-.99f,.28f,.53f,-.41f,-.92f,.38f,-.53f,-.14f,.87f,.80f,.07f,-.76f,.25f,-.75f,-.83f,-.83f,-.01f,-.08f,.92f,-.33f,.67f,.08f,-.11f,.35f,-.17f,.70f,.23f,-.64f,-.91f,.56f,.82f,.42f,-.48f,.54f,-.04f,.00f,.05f,.00f,-.11f,.02f,-.26f,.46f,.04f,.84f,-.54f,-.33f,-.55f,-.00f,-.90f,.73f,-.19f,-.35f,-.40f,-.09f,-.15f,.92f,-.20f,-.95f,-.85f,.92f,.10f,.61f,.02f,-.01f,.54f,.07f,.85f,.50f,-.84f,.14f,-.38f,.40f,.22f,-.49f,.11f,-.49f,-.23f,-.39f,-.25f,-.31f,.65f,-.52f,-.38f,.46f,-.73f,.91f,-.71f,.77f,.65f,-.05f,.31f,-.60f,.08f,-.26f,.25f,.38f,-.68f,.55f,.83f,-.63f,-.75f,-.55f,.08f,-.42f,.35f,-.33f,-.45f,.00f,.67f,.78f,-.59f,.55f,-.90f,-.47f,.91f,.00f,-.15f,.34f,.75f,-.92f,.15f,.21f,.14f,-.14f,.22f,-.19f,.42f,.31f,.75f,-.66f,-.24f,.37f,.40f,.25f,-.15f,.47f,.03f,.14f,-.32f,.56f,-.58f,.88f,-.67f,.96f,.14f,-.78f,.98f,-.77f,-.52f,-.27f,.94f,.55f,.03f,.90f,-.56f,-.43f,-.58f,-.27f,-.79f,-.87f,-.71f,-.22f,-.48f,-.38f,-.96f,.58f,.24f,-.28f,-.47f,.33f,.10f,-.44f,-.94f,.29f,-.16f,.43f,-.79f,.81f,-.89f,-.63f,-.76f,-.15f,-.71f,-.77f,.13f,.67f,-.15f,.86f,.12f,-.37f,.42f,-.82f,-.64f,.42f,.21f,-.37f,-.88f,.14f,-.80f,.22f,.52f,-.84f,-.39f,.89f,.23f,-.21f,-.62f,.49f,.35f,-.13f,-.40f,.94f,-.10f,-.71f,-.54f,.75f,.28f,-.00f,-.78f,-.12f,.42f,.28f,-.99f,.82f,-.99f,-.97f,.61f,.29f,-.76f,.97f,-.57f,-.50f,.20f,.38f,-.95f,.08f,-.57f,-.98f,.24f,-.41f,.40f,.50f,.59f,.44f,-.76f,-.97f,-.65f,-.07f,-.12f,-.03f,.79f,.87f,-.63f,-.09f,.16f,-.47f,.62f,.55f,.17f,.32f,-.44f,.44f,.61f,-.52f,.51f,-.49f,.87f,-.34f,.63f,-.53f,-.30f,-.50f,-.11f,-.82f,-.63f,-.27f,.31f,-.60f,-.83f,.84f,.90f,-.13f,.61f,-.29f,-.62f,.51f,-.32f,.01f,-.35f,-.22f,-.03f,-.65f,.25f,.23f,-.70f,.09f,-.34f,-.82f,-.60f,.27f,.48f,-.04f,.91f,.40f,-.66f,.85f,-.20f,.85f,.61f,-.96f,-.25f,-.55f,-.75f,.00f,.78f,-.92f,.29f,-.00f,.66f,.03f,.47f,-.44f,-.43f,-.63f,.82f,-.00f,-.88f,-.14f,-.55f,.37f,.00f,.95f,.12f,.95f,.22f,-.98f,.82f,-.44f,.37f,-.09f,-.92f,.52f,-.26f,-.37f,.95f,.74f,.84f,-.97f,.09f,-.33f,-.87f,-.06f,.81f,-.85f,-.68f,-.67f,.65f,-.95f,.60f,-.55f,-.45f,.50f,.54f,-.18f,-.00f,-.53f,.68f,.88f,.36f,-.37f,.58f,-.90f,.69f,-.69f,.10f,.40f,-.88f,-.91f,-.90f,-.18f,.47f,-.60f,.17f,-.13f,.64f,-.24f,-.42f,-.89f,-.47f,.24f,-.09f,.13f,.22f,.35f,.49f,.73f,-.97f,-.58f,.49f,.20f,.21f,-.24f,.16f,-.77f,.02f,-.35f,.06f,.83f,-.04f,-.87f,-.61f,.78f,.25f,-.49f,-.73f,-.13f,.95f,-.59f,.41f,-.97f,-.53f,-.10f,-.45f,-.86f,.57f,-.58f,-.78f,.51f,-.05f,-.57f,.11f,.46f,.66f,-.48f,.51f,-.01f,-.53f,-.76f,.19f,-.01f,.62f,-.76f,.49f,.13f,-.44f,.01f,.98f,-.33f,-.18f,-.00f,.27f,.58f,.94f,-.21f,-.79f,.78f,.15f,-.19f,-.60f,.66f,.53f,-.13f,-.91f,-.96f,.16f,-.19f,-.40f,-.79f,-.18f,.66f,-.17f,.80f,.90f,.91f,.43f,-.83f,.74f,-.88f,-.41f,.07f,-.25f,-.53f,-.75f,-.43f,-.62f,-.35f,.90f,-.05f,.75f,-.76f,.87f,-.51f,.16f,.11f,-.79f,.29f,.36f,.56f,-.85f,-.51f,-.04f,.63f,.75f,.66f,-.65f,-.33f,-.58f,.09f,-.50f,-.70f,-.75f,.45f,-.75f,.21f,-.20f,-.39f,.48f,.97f,-.03f,.24f,-.18f,-.48f,.46f,-.04f,-.71f,-.11f,.89f,.56f,-.52f,.76f,.44f,.19f,.83f,.93f,-.51f,.83f,.73f,.12f,.52f,-.97f,.42f,.91f,-.17f,.13f,.44f,-.03f,.54f,.48f,-.25f,-.92f,-.46f,.39f,.82f,-.23f,.05f,.66f,-.08f,.60f,.14f,-.20f,-.32f,-.10f,-.21f,.12f,.56f,-.73f,.63f,.09f,.97f,.10f,-.88f,.71f,.25f,-.62f,-.30f,-.25f,-.64f,.35f,.91f,.57f,.08f,-.97f,-.41f,-.94f,-.91f,-.11f,.57f,-.68f,-.37f,-.39f,.83f,.31f,-.15f,.81f,-.53f,.63f,-.65f,.92f,-.40f,-.35f,.05f,.84f,.63f,.12f,-.97f,.98f,.21f,.91f,-.86f,-.12f,-.77f,.56f,.66f,-.60f,.67f,-.36f,-.33f,.39f,-.21f,.22f,.82f,-.51f,.86f,.28f,-.07f,.13f,-.91f,-.83f,-.60f,-.88f,.42f,.63f,-.70f,.54f,.70f,.38f,-.71f,-.15f,.87f,.07f,.81f,-.33f,-.87f,-.86f,.85f,.37f,-.17f,.71f,.87f,-.56f,.06f,-.66f,.79f,-.17f,.13f,-.22f,-.75f,.65f,.14f,.25f,.56f,.63f,-.68f,-.11f,.91f,.12f,-.01f,-.25f,-.76f,-.93f,.66f,.55f,.43f,-.35f,.60f,-.01f,-.68f,-.94f,-.34f,-.47f,.87f,.78f,.10f,.61f,-.34f,.19f,-.24f,-.20f,.65f,-.04f,-.46f,.08f,.10f,.54f,.28f,-.33f,-.89f,.88f,-.54f,-.04f,.22f,.53f,.70f,.07f,-.65f,-.86f,-.72f,-.61f,.15f,-.33f,-.46f,.96f,.51f,.86f,.84f,.03f,-.62f,.54f,-.40f,-.73f,.04f,-.40f,.23f,-.40f,.28f,-.66f,.71f,.33f,.00f,.59f,.08f,.81f,-.02f,.35f,-.57f,.92f,-.76f,.55f,-.28f,.53f,-.84f,-.76f,.62f,-.86f,.97f,.33f,-.31f,-.70f,.51f,.21f,-.88f,-.14f,.17f,.93f,.34f,-.38f,-.71f,-.68f,-.99f,-.80f,-.26f,.80f,.39f,.65f,.89f,-.11f,.27f,.59f,.72f,.88f,.41f,.27f,-.54f,.61f,-.24f,.64f,.39f,.06f,-.65f,-.23f,-.43f,-.74f,.23f,.06f,-.18f,-.72f,.30f,.07f,-.90f,-.49f,-.50f,.90f,.73f,-.00f,.12f,-.03f,.50f,-.15f,.70f,-.82f,.54f,.73f,-.49f,-.50f,-.22f,-.57f,.58f,-.73f,-.69f,-.58f,-.81f,-.48f,.92f,-.83f,-.67f,-.43f,-.64f,.53f,.00f,-.71f,-.36f,.38f,.92f,.98f,-.60f,-.52f,.11f,.78f,-.40f,.81f,-.50f,-.93f,-.04f,-.65f,.44f,-.66f,-.23f,.09f,-.14f,-.40f,.26f,.82f,.68f,.61f,.93f,.02f,-.86f,-.74f,-.81f,-.84f,-.64f,-.70f,-.36f,.83f,-.57f,-.74f,-.22f,.51f,-.96f,-.07f,-.57f,.18f,.95f,.25f,.76f,-.59f,-.09f,-.74f,.33f,.64f,-.90f,.62f,-.16f,-.69f,.46f,-.86f,.21f,-.25f,.27f,.66f,.63f,-.19f,-.16f,-.31f,-.37f,.99f,.96f,-.84f,-.05f,-.21f,.05f,-.22f,-.09f,.32f,.61f,.94f,-.30f,.73f,-.79f,-.84f,.36f,-.34f,.00f,.19f,.14f,-.94f,-.98f,.93f,.09f,-.75f,.53f,-.39f,-.95f,-.21f,.17f,.87f,.21f,-.29f,.15f,-.96f,-.30f,-.13f,.43f,-.01f,-.08f,.57f,.77f,.98f,-.74f,-.89f,.79f,-.10f,-.15f,.28f,.47f,-.62f,.86f,.77f,.21f,-.22f,.63f,-.42f,-.10f,.42f,.02f,-.15f,-.74f,.66f,-.69f,-.82f,-.39f,.17f,-.16f,-.04f,-.69f,.97f,-.33f,.74f,-.69f,-.01f,-.87f,.43f,.99f,-.18f,-.37f,.86f,-.96f,.97f,-.62f,-.02f,.81f,-.21f,.78f,-.36f,.79f,.14f,.04f,.13f,.77f,.84f,-.21f,-.24f,.50f,-.46f,-.64f,-.43f,-.26f,-.08f,-.14f,-.70f,.77f,-.59f,-.39f,.60f,-.06f,.42f,-.83f,-.43f,.90f,.76f,-.52f,-.92f,-.22f,-.15f,-.54f,.87f,.20f,.80f,.93f,-.14f,-.33f,-.83f,.20f,-.98f,.18f,.32f,-.65f,.00f,-.22f,.89f,-.07f,-.31f,.05f,-.49f,-.58f,.11f,-.23f,.18f,-.58f,-.30f,-.83f,.39f,.56f,-.49f,.45f,-.79f,.79f,-.67f,.67f,-.15f,.32f,-.91f,.46f,.11f,.49f,.78f,.33f,-.66f,-.64f,.63f,-.42f,-.25f,-.80f,-.24f,-.69f,.56f,.84f,.77f,.74f,-.39f,.54f,.83f,.22f,-.98f,.26f,.17f,.63f,.00f,.55f,-.92f,.94f,-.32f,-.87f,-.74f,.11f,.61f,.55f,-.72f,.19f,-.44f,-.97f,-.25f,-.94f,.75f,-.41f,-.40f,-.83f,-.82f,-.51f,-.26f,.49f,.98f,.26f,.91f,.30f,-.37f,.50f,-.77f,.39f,-.02f,.22f,-.91f,.25f,.98f,.92f,.85f,-.23f,-.00f,-.35f,-.63f,.88f,.39f,.74f,-.04f,.34f,.02f,-.82f,.73f,.33f,-.59f,.47f,.56f,.48f,-.34f,.84f,-.68f,-.28f,.96f,-.06f,.30f,-.75f,.91f,.09f,.29f,-.84f,.97f,-.66f,.52f,-.00f,.56f,-.46f,.29f,.49f,.64f,.77f,-.24f,.20f,.34f,.40f,.20f,.85f,-.80f,.28f,.93f,.72f,-.30f,.62f,.61f,-.63f,-.66f,-.46f,.95f,.05f,-.47f,.83f,.02f,-.33f,-.35f,.38f,-.51f,.11f,.52f,.14f,.87f,-.71f,-.87f,-.22f,.52f,.82f,-.77f,.72f,.48f,-.11f,-.77f,.64f,-.61f,.09f,-.16f,-.30f,.56f,.50f,-.66f,-.39f,.81f,.42f,.29f,-.32f,.67f,-.59f,-.60f,.74f,.93f,-.49f,-.34f,-.35f,.32f,-.75f,.32f,-.03f,-.39f,.61f,-.84f,-.46f,-.33f,.99f,-.71f,.71f,-.48f,.27f,.65f,-.08f,.86f,-.69f,.58f,.10f,-.64f,.82f,-.66f,-.65f,.58f,-.42f,.85f,-.34f,.99f,-.02f,-.05f,.51f,-.40f,.24f,.61f,-.13f,-.78f,-.31f,-.89f,.62f,.30f,-.27f,-.68f,-.41f,-.77f,.64f,.94f,.01f,.29f,-.45f,-.15f,-.73f,.78f,-.27f,-.80f,-.70f,-.30f,-.69f,.38f,-.31f,.37f,.08f,.97f,-.32f,-.01f,.70f,.61f,.39f,.58f,.35f,.32f,.17f,.86f,-.04f,-.75f,.98f,.81f,-.58f,.78f,.94f,.65f,.89f,-.36f,-.43f,.27f,.99f,-.99f,.11f,.04f,-.08f,-.16f,.20f,-.66f,.87f,-.05f,-.57f,.27f,.90f,-.41f,-.60f,.52f,-.95f,.30f,.26f,.54f,-.71f,.44f,-.70f,-.29f,-.84f,-.21f,-.59f,.55f,-.98f,-.85f,-.98f,.84f,.99f,.25f,.14f,-.42f,-.91f,-.48f,-.52f,-.49f,.33f,.05f,-.46f,.97f,.89f,-.12f,-.45f,.91f,-.13f,-.22f,-.74f,-.34f,.00f,.34f,.16f,-.31f,-.11f,.77f,-.70f,.81f,.84f,-.30f,.18f,.98f,-.12f,-.36f,.48f,.26f,-.87f,.21f,.64f,.36f,-.85f,-.04f,.44f,-.89f,.94f,-.98f,.19f,-.40f,-.48f,-.36f,.77f,.29f,.18f,-.02f,-.72f,.44f,.35f,.42f,-.27f,.51f,.94f,.14f,.53f,-.56f,-.07f,.37f,-.33f,.83f,-.53f,.33f,.86f,-.13f,-.52f,.11f,-.23f,.25f,-.65f,-.06f,.42f,.71f,.94f,-.78f,.66f,.96f,.54f,.20f,-.36f,-.52f,-.26f,-.59f,.42f,.34f,-.23f,-.35f,.67f,.49f,.22f,-.31f,.17f,-.34f,.13f,.06f,.78f,-.24f,-.50f,.41f,-.18f,-.93f,-.98f,-.02f,-.16f,-.11f,-.37f,-.66f,-.63f,.14f,-.87f,-.60f,.60f,-.45f,.75f,-.41f,.62f,.24f,.71f,.63f,.77f,.11f,-.99f,.03f,.50f,-.72f,-.77f,-.25f,.00f,.51f,-.77f,-.35f,-.91f,.94f,.40f,.89f,.25f,-.42f,-.24f,-.15f,-.85f,.32f,-.42f,.66f,.57f,-.89f,-.08f,.76f,-.57f,.09f,-.26f,-.30f,-.46f,-.44f,-.06f,.79f,.67f,-.60f,-.59f,-.05f,.11f,-.60f,-.22f,-.99f,.51f,-.30f,.70f,.85f,.00f,-.55f,-.26f,-.28f,-.55f,.85f,.49f,-.33f,-.95f,-.34f,-.26f,-.41f,-.37f,-.07f,.84f,-.94f,.36f,-.97f,.04f,.52f,.36f,.36f,-.41f,-.66f,.91f,.99f,-.70f,-.36f,.20f,.97f,-.87f,-.32f,-.86f,-.85f,-.33f,.69f,.44f,-.25f,-.44f,-.44f,-.90f,.93f,.74f,-.52f,.71f,.97f,.13f,.04f,.42f,.08f,-.71f,.08f,.13f,-.42f,-.08f,-.49f,-.13f,.91f,.08f,.02f,.42f,-.44f,-.32f,.34f,.27f,.70f,.49f,-.30f,-.00f,-.87f,.08f,.29f,.64f,.02f,-.29f,.23f,-.53f,-.93f,.42f,-.47f,-.79f,.66f,.75f,-.30f,.16f,.81f,-.81f,.36f,.71f,.90f,.15f,.98f,-.77f,.96f,.10f,.40f,.05f,-.76f,.71f,-.12f,-.30f,.93f,-.00f,-.53f,-.25f,-.10f,.04f,-.07f,.04f,-.89f,.53f,-.76f,.60f,.07f,-.31f,.06f,-.35f,.33f,.01f,-.25f,-.71f,-.38f,-.39f,.15f,-.28f,-.44f,-.64f,-.80f,-.61f,-.57f,.46f,.61f,-.49f,-.07f,-.17f,-.94f,.73f,-.06f,-.02f,.19f,.54f,.38f,.69f,-.67f,-.27f,.05f,.75f,-.48f,-.13f,-.18f,-.02f,.37f,.63f,-.97f,.30f,.82f,.82f,.88f,.25f,-.64f,.60f,-.41f,-.12f,-.73f,-.65f,-.53f,.02f,.58f,-.36f,-.53f,.88f,-.51f,.66f,-.79f,-.23f,.65f,-.38f,.99f,-.51f,-.62f,.72f,-.31f,-.87f,.56f,.18f,.71f,.01f,-.72f,.59f,.15f,-.73f,.43f,-.81f,-.62f,.08f,.82f,.98f,.94f,-.97f,.97f,-.27f,-.17f,-.41f,-.20f,-.61f,.31f,-.25f,-.58f,.95f,-.36f,.28f,-.54f,-.92f,-.87f,-.61f,.12f,-.24f,.71f,.90f,-.29f,-.75f,.78f,.30f,.51f,-.48f,-.39f,.75f,-.90f,.65f,-.07f,.17f,-.55f,.27f,-.66f,-.23f,-.13f,.06f,-.27f,.13f,.53f,-.06f,.88f,.81f,-.56f,-.09f,-.26f,-.84f,-.22f,-.59f,.23f,.60f,-.60f,.90f,-.86f,.00f,-.43f,-.15f,-.33f,-.89f,.20f,-.08f,.37f,.24f,.86f,.67f,-.52f,-.88f,.45f,-.56f,.00f,-.45f,.07f,-.82f,.08f,-.27f,-.64f,-.88f,.92f,-.99f,-.12f,.89f,.35f,-.29f,.48f,-.35f,-.50f,-.60f,.31f,-.46f,.60f,.47f,.54f,.45f,.86f,.54f,-.99f,.59f,-.01f,.86f,-.10f,.54f,-.76f,-.88f,.75f,-.61f,.40f,-.25f,.92f,.14f,-.18f,.04f,-.31f,.29f,-.80f,.67f,-.60f,-.76f,-.01f,-.19f,.18f,.55f,.03f,.23f,-.30f,-.66f,-.12f,-.16f,.97f,.40f,.52f,-.15f,-.89f,.71f,.31f,.96f,.90f,-.19f,-.35f,.28f,-.64f,-.45f,-.78f,-.98f,-.36f,.00f,-.53f,-.91f,-.41f,.21f,.43f,-.24f,-.37f,-.91f,.66f,.19f,.76f,.72f,-.44f,.55f,.51f,.45f,.68f,-.26f,.09f,-.87f,.16f,.52f,.85f,-.61f,-.28f,-.50f,.32f,-.24f,.09f,.18f,.46f,.80f,-.04f,.07f,.29f,-.57f,-.22f,-.92f,-.49f,.98f,.99f,.61f,.26f,.85f,-.24f,.47f,-.40f,-.59f,-.00f,-.89f,-.33f,.37f,.26f,.31f,-.77f,-.89f,-.03f,-.26f,.85f,.71f,-.12f,.64f,.92f,-.54f,.96f,.42f,.85f,-.85f,.22f,.61f,.64f,-.82f,.01f,-.07f,.58f,-.33f,.25f,.26f,-.52f,.44f,-.84f,.50f,-.02f,-.29f,.57f,-.43f,-.62f,.66f,-.01f,-.86f,.83f,.90f,-.16f,.66f,-.81f,-.30f,.15f,-.62f,.38f,.92f,-.11f,.39f,.51f,-.27f,.62f,-.86f,.00f,-.16f,.70f,-.77f,-.92f,-.73f,-.28f,-.47f,-.70f,-.66f,-.78f,-.86f,.56f,.28f,.31f,.96f,-.64f,.18f,.49f,-.81f,-.56f,-.98f,.99f,-.93f,.04f,-.63f,-.22f,-.63f,-.49f,.08f,.47f,.71f,-.35f,-.76f,.01f,-.92f,.80f,.23f,.69f,-.66f,.14f,-.21f,.11f,-.38f,-.11f,-.12f,-.77f,-.90f,-.24f,.78f,.54f,.84f,.18f,-.26f,-.08f,-.66f,-.83f,-.28f,.90f,-.89f,.98f,-.75f,-.92f,-.56f,.29f,.03f,.25f,-.64f,.23f,-.30f,.44f,.98f,-.48f,.66f,.80f,-.31f,.17f,.45f,-.43f,.59f,-.78f,.10f,.27f,.04f,.85f,-.25f,-.22f,-.07f,-.64f,-.86f,-.39f,.73f,.81f,-.71f,.63f,.87f,-.41f,.45f,-.00f,-.01f,.40f,.10f,-.59f,.13f,-.11f,.13f,.18f,-.06f,-.18f,.28f,-.42f,-.62f,-.82f,.34f,-.61f,.61f,-.47f,.34f,-.68f,.92f,-.08f,.90f,.52f,.71f,-.14f,.73f,-.72f,-.43f,.15f,-.10f,.49f,-.33f,.71f,-.13f,-.48f,-.39f,.08f,.50f,.47f,-.79f,-.21f,-.75f,.89f,.15f,.21f,.62f,-.05f,.62f,-.83f,.87f,-.34f,.61f,-.60f,-.42f,.11f,.03f,-.66f,-.05f,.42f,.19f,.82f,.69f,-.33f,-.92f,-.02f,-.00f,.82f,.83f,-.02f,.44f,.11f,.19f,.01f,-.09f,.38f,-.32f,.30f,.84f,-.65f,-.01f,-.08f,-.02f,-.83f,-.47f,.95f,-.99f,.37f,-.38f,.21f,.35f,-.38f,.92f,.60f,.46f,-.14f,.28f,.20f,-.15f,-.03f,.96f,-.33f,.08f,-.48f,-.56f,.96f,-.03f,-.56f,-.25f,.90f,.16f,.21f,-.45f,.77f,.21f,.61f,.74f,-.07f,.93f,-.35f,-.26f,.52f,.74f,-.91f,.33f,.09f,-.45f,.58f,-.53f,.31f,.42f,.70f,-.85f,-.19f,.63f,-.87f,.67f,-.49f,.51f,-.08f,.96f,-.80f,.00f,.79f,.44f,.69f,-.37f,-.86f,-.63f,.90f,.79f,-.56f,-.26f,.66f,.62f,.22f,.27f,.75f,-.86f,.23f,.34f,-.50f,-.40f,-.05f,.12f,-.26f,.40f,-.74f,-.94f,-.65f,-.36f,-.66f,-.96f,-.34f,.57f,-.44f,.09f,-.88f,.66f,-.04f,.46f,-.33f,-.72f,.69f,.58f,-.57f,-.75f,.48f,.31f,-.04f,.34f,-.95f,.49f,-.63f,-.27f,.61f,-.96f,-.00f,-.27f,-.26f,.26f,-.33f,.30f,.61f,.81f,.12f,.24f,.24f,-.07f,-.23f,-.50f,-.55f,-.22f,-.89f,-.97f,.05f,.46f,.11f,.62f,-.02f,-.34f,-.94f,.04f,.97f,.43f,.58f,-.46f,.59f,-.36f,.47f,-.06f,-.57f,-.57f,.10f,.38f,-.43f,.23f,-.91f,.35f,.09f,.91f,.30f,.84f,.09f,.40f,-.75f,.50f,-.36f,-.95f,-.86f,-.78f,-.69f,.28f,.93f,.19f,-.15f,.90f,-.19f,.71f,.98f,.15f,.31f,-.87f,-.61f,.51f,.71f,.67f,-.40f,.70f,.34f,.61f,.61f,.47f,.49f,-.13f,-.43f,.79f,-.45f,-.57f,.40f,-.23f,.53f,.27f,-.58f,.43f,-.10f,-.72f,.27f,-.09f,.56f,.64f,.11f,-.57f,-.27f,.60f,-.29f,.16f,.77f,-.96f,.40f,.13f,.86f,-.42f,.62f,-.21f,-.11f,.92f,.07f,-.08f,.70f,.34f,-.82f,-.46f,.86f,-.01f,-.42f,.41f,.74f,-.03f,-.69f,.38f,.03f,.94f,-.81f,-.44f,.26f,-.48f,-.01f,.81f,.41f,-.87f,-.35f,-.23f,-.32f,.19f,-.31f,-.36f,-.39f,.96f,-.03f,.64f,.65f,-.40f,.80f,.61f,.83f,.37f,.63f,.42f,-.01f,.95f,-.67f,.59f,-.02f,-.06f,.63f,.20f,.27f,-.15f,.13f,.49f,-.43f,.92f,-.76f,.20f,-.35f,.20f,-.82f,-.79f,.09f,-.62f,.60f,-.20f,-.89f,-.22f,-.63f,.31f,.27f,-.67f,.23f,.13f,.18f,-.82f,.95f,.52f,-.26f,.55f,-.77f,-.29f,.43f,-.34f,-.61f,.72f,.21f,-.21f,.21f,.47f,-.02f,-.13f,.10f,.81f,.73f,-.50f,-.59f,-.32f,-.18f,-.02f,.19f,-.58f,-.35f,-.52f,.40f,-.88f,-.87f,.75f,-.08f,-.36f,.49f,.46f,.72f,.47f,-.69f,-.53f,-.67f,.27f,.51f,.73f,-.70f,-.52f,.83f,.03f,-.03f,-.72f,.22f,.29f,-.92f,-.07f,-.58f,.21f,-.62f,.08f,.52f,-.70f,.68f,-.72f,-.66f,.24f,.80f,.48f,.46f,-.41f,-.47f,.06f,-.11f,-.30f,.92f,-.28f,.53f,-.08f,.28f,.55f,.87f,.65f,-.00f,-.44f,-.17f,.35f,-.43f,.01f,-.92f,.01f,-.33f,.21f,.34f,.67f,.48f,-.23f,-.08f,.07f,-.17f,-.02f,-.62f,-.56f,.47f,-.10f,.49f,-.33f,.04f,-.92f,-.68f,.80f,.49f,.42f,.86f,-.23f,-.74f,-.64f,.83f,.39f,-.53f,-.69f,-.86f,-.30f,.22f,-.85f,.78f,.00f,-.86f,.94f,.33f,.85f,.33f,-.34f,.06f,-.25f,.26f,-.88f,.90f,-.47f,-.29f,-.56f,-.59f,.66f,-.06f,-.03f,.75f,-.64f,.03f,-.15f,.69f,-.59f,.66f,.37f,-.21f,.04f,-.24f,.49f,-.30f,-.27f,-.93f,-.16f,.73f,-.22f,-.63f,.98f,-.47f,.60f,-.67f,-.31f,-.24f,-.57f,.10f,.49f,-.52f,-.81f,-.28f,.13f,-.36f,.87f,-.61f,.38f,-.35f,.05f,-.01f,.38f,.92f,-.43f,-.87f,.35f,.84f,.47f,.19f,.71f,-.16f,.32f,.64f,-.27f,.30f,-.88f,.24f,.97f,-.02f,.59f,-.45f,-.13f,.32f,.40f,-.69f,.24f,-.06f,-.29f,-.74f,.01f,.36f,.16f,.18f,-.89f,-.62f,.63f,-.83f,.37f,.88f,.53f,.43f,.16f,.72f,.94f,-.04f,.86f,.64f,.58f,-.03f,-.83f,.78f,-.96f,-.11f,-.30f,.20f,-.60f,-.02f,.52f,.62f,-.99f,-.96f,.17f,-.79f,-.87f,-.32f,.39f,.31f,.02f,-.84f,-.77f,.49f,.65f,.83f,-.30f,.98f,-.13f,-.94f,.14f,.40f,-.47f,.81f,.88f,-.75f,.88f,.54f,.89f,-.52f,.92f,-.10f,.11f,.83f,.11f,-.99f,.22f,-.41f,.00f,.80f,-.53f,.19f,.26f,-.17f,-.70f,-.56f,-.20f,.26f,.91f,-.83f,.67f,.56f,.83f,.43f,-.39f,.28f,.83f,-.43f,.92f,.88f,-.68f,-.37f,.62f,-.68f,.70f,-.16f,-.78f,.88f,-.84f,.32f,-.22f,.72f,.58f,.40f,.24f,.90f,-.08f,-.03f,-.89f,.55f,-.76f,-.34f,-.15f,.19f,.36f,.46f,-.60f,-.43f,.54f,-.38f,-.02f,-.21f,-.60f,-.74f,.92f,.65f,.47f,.75f,.58f,.95f,-.22f,-.64f,-.28f,-.12f,.84f,-.37f,-.18f,-.50f,.08f,.14f,.31f,-.56f,-.18f,.32f,.52f,.46f,.19f,.31f,-.48f,-.95f,-.31f,.83f,-.31f,.22f,.41f,-.02f,.05f,.60f,.36f,.78f,.50f,.50f,.61f,-.59f,-.00f,.50f,.96f,-.80f,-.56f,.92f,-.00f,.25f,.79f,.76f,.14f,-.00f,-.99f,-.85f,.81f,-.26f,-.58f,.21f,-.68f,-.90f,.68f,-.48f,.59f,.08f,.62f,.38f,.64f,-.54f,.65f,.60f,-.45f,-.54f,-.40f,.13f,-.94f,-.86f,-.67f,.09f,-.53f,-.45f,.23f,.19f,.62f,-.18f,-.51f,.56f,.81f,.55f,-.62f,.60f,.44f,.61f,.58f,.13f,.13f,.46f,-.04f,-.01f,-.61f,-.76f,-.87f,-.25f,.69f,-.29f,.53f,-.59f,-.22f,.22f,-.08f,.77f,-.08f,.65f,-.90f,.19f,.82f,-.26f,.00f,.12f,.22f,-.88f,.52f,-.37f,-.97f,.34f,.94f,.84f,.88f,.35f,.31f,-.22f,.41f,-.96f,.93f,-.43f,.30f,-.20f,-.41f,.96f,.36f,-.88f,.72f,.32f,-.23f,.37f,-.80f,.05f,.52f,.50f,-.94f,-.43f,-.05f,-.44f,.87f,-.11f,-.01f,.85f,.41f,.30f,-.13f,-.02f,-.49f,-.68f,-.15f,-.49f,-.04f,-.40f,-.56f,.32f,.70f,-.38f,-.12f,-.64f,-.69f,-.22f,.40f,-.77f,.44f,-.36f,-.68f,-.45f,-.39f,.00f,-.44f,.73f,.14f,-.21f,-.50f,-.28f,.04f,.25f,.24f,-.70f,-.04f,.78f,.90f,-.37f,.14f,-.31f,-.98f,-.23f,.15f,-.04f,-.43f,-.51f,.34f,-.69f,-.33f,-.00f,.33f,.77f,.55f,.88f,.56f,-.06f,.86f,.61f,-.97f,.42f,.96f,-.23f,.56f,-.82f,.41f,.87f,-.93f,-.40f,-.71f,-.88f,.49f,-.39f,-.66f,-.16f,.57f,-.77f,.15f,-.36f,-.13f,.69f,.36f,.75f,-.84f,-.44f,.77f,.69f,-.89f,.90f,-.91f,.09f,.86f,.27f,.83f,.75f,-.17f,.80f,.40f,.20f,-.40f,.87f,-.85f,-.47f,-.40f,-.47f,-.17f,-.42f,-.14f,-.83f,.04f,.80f,.72f,-.07f,-.24f,-.32f,.91f,-.67f,.27f,.65f,.68f,-.94f,-.88f,-.27f,.74f,-.36f,.88f,-.78f,.44f,-.31f,.21f,.57f,.01f,.25f,-.02f,.44f,-.88f,.96f,-.91f,.66f,.88f,.97f,.53f,-.90f,.07f,.70f,-.41f,-.52f,-.00f,-.48f,.27f,-.19f,.96f,-.00f,.23f,.13f,-.56f,-.69f,.66f,.19f,-.79f,.87f,.91f,-.35f,-.67f,.01f,-.64f,-.48f,.77f,.29f,-.32f,-.36f,-.07f,-.36f,-.03f,.51f,-.99f,.42f,-.95f,.31f,.96f,.58f,.91f,.96f,.42f,-.76f,-.73f,-.52f,.64f,.24f,-.28f,-.50f,.56f,.50f,.41f,.54f,.10f,.10f,-.57f,-.73f,.50f,.36f,-.71f,.49f,-.08f,-.33f,-.78f,-.77f,.84f,.40f,.91f,-.39f,.26f,-.76f,-.25f,-.15f,-.07f,-.34f,.82f,-.94f,.21f,-.99f,.33f,.79f,-.41f,.71f,-.40f,.87f,.82f,-.34f,-.99f,.35f,-.41f,.10f,.60f,.84f,-.28f,.46f,.63f,-.47f,-.24f,-.05f,-.92f,.55f,.59f,-.23f,.92f,.86f,-.16f,-.51f,-.63f,-.34f,.56f,-.98f,-.07f,.64f,-.83f,.71f,.40f,-.47f,-.67f,.80f,-.64f,.86f,-.00f,-.64f,-.54f,.53f,-.54f,.13f,-.73f,.71f,.44f,.62f,.17f,.80f,.82f,-.87f,-.54f,-.56f,.35f,.02f,.18f,-.06f,-.00f,.13f,.16f,.18f,-.11f,.35f,.70f,.84f,.48f,-.03f,.07f,.20f,-.84f,-.18f,-.30f,.79f,-.88f,-.28f,.64f,.70f,-.05f,-.18f,.59f,-.87f,-.98f,.57f,.11f,.84f,-.33f,.55f,-.89f,-.07f,-.64f,-.17f,.74f,.70f,-.54f,.42f,-.64f,-.74f,-.90f,-.40f,.69f,.62f,.26f,.50f,-.61f,.60f,.54f,.23f,.30f,.68f,.96f,-.19f,.38f,-.73f,.04f,-.44f,.51f,.87f,.56f,.72f,.26f,-.61f,-.36f,.67f,.64f,-.67f,-.85f,-.30f,.37f,-.34f,-.00f,.30f,.86f,-.19f,.08f,.46f,-.14f,.45f,-.52f,.40f,-.69f,-.60f,-.53f,-.35f,-.15f,-.01f,-.93f,-.97f,.48f,.10f,.64f,.61f,-.02f,-.32f,.67f,.30f,-.73f,-.75f,-.24f,.43f,-.68f,-.09f,.76f,-.50f,-.85f,-.94f,-.73f,.85f,-.73f,-.93f,.52f,-.67f,.67f,.15f,.25f,.24f,-.06f,-.20f,-.43f,.77f,-.09f,.81f,.28f,.32f,-.34f,.15f,.95f,.48f,.37f,-.39f,.92f,.06f,.97f,-.59f,.93f,-.22f,-.16f,-.29f,.00f,.70f,-.78f,.56f,-.05f,.39f,.23f,-.89f,.29f,-.08f,-.67f,-.51f,-.09f,.64f,-.75f,-.89f,.91f,-.05f,-.33f,.84f,-.74f,-.95f,-.12f,.59f,.27f,.03f,.98f,-.16f,.63f,.24f,.65f,-.64f,-.89f,.27f,.55f,-.22f,-.27f,.68f,.89f,-.78f,.36f,-.98f,-.53f,.44f,.72f,-.00f,-.08f,-.58f,.96f,.05f,-.75f,.69f,.14f,-.55f,.83f,-.66f,.30f,-.85f,.63f,.03f,-.10f,.73f,.04f,.70f,-.95f,-.52f,-.45f,.97f,-.16f,-.31f,-.61f,-.55f,.58f,.20f,-.17f,-.80f,.75f,-.31f,-.48f,.81f,-.61f,.59f,-.21f,-.16f,.15f,-.84f,.90f,-.53f,.82f,.17f,-.75f,-.51f,-.00f,-.00f,-.07f,-.25f,-.40f,-.09f,-.39f,.50f,-.63f,.76f,-.80f,.03f,.44f,-.18f,-.75f,.08f,.55f,.22f,-.88f,.47f,.32f,.40f,.58f,-.08f,.50f,.40f,.41f,-.28f,.40f,-.30f,-.39f,.32f,.78f,.56f,-.55f,.08f,-.50f,-.94f,-.69f,.88f,-.97f,-.67f,.98f,.44f,-.78f,.08f,-.66f,.83f,-.84f,.93f,-.55f,.03f,-.25f,-.91f,-.95f,.88f,-.19f,-.23f,.67f,-.21f,-.03f,-.10f,-.77f,-.27f,.19f,-.37f,.95f,.29f,-.43f,.94f,.47f,.96f,-.92f,.26f,-.20f,-.00f,-.55f,.37f,-.76f,-.02f,-.30f,-.47f,.65f,-.14f,.09f,-.26f,-.19f,-.63f,-.28f,.58f,.87f,.87f,.50f,.73f,.96f,-.16f,.55f,.68f,-.62f,-.81f,.32f,-.47f,.29f,.53f,-.19f,-.98f,-.41f,.29f,-.48f,-.16f,.11f,.09f,-.27f,-.97f,.95f,.36f,-.66f,-.68f,.41f,.01f,.07f,-.85f,.13f,-.02f,.31f,-.53f,-.13f,-.29f,-.43f,.68f,.14f,.85f,-.57f,-.63f,-.46f,-.44f,-.70f,-.27f,.68f,.68f,.90f,-.59f,.33f,.63f,.40f,.69f,.26f,.44f,-.91f,-.00f,-.60f,-.45f,-.51f,.42f,.53f,-.19f,-.97f,-.79f,.98f,.99f,-.05f,-.17f,.16f,-.77f,.14f,.34f,.15f,.15f,-.00f,.27f,-.17f,.68f,.37f,.50f,-.14f,-.03f,-.70f,-.06f,-.33f,.50f,.57f,-.37f,-.96f,-.89f,-.43f,-.45f,.11f,-.33f,-.19f,-.62f,.81f,.56f,-.38f,.69f,.61f,-.30f,-.58f,-.67f,-.72f,-.83f,.64f,.94f,-.61f,.83f,-.43f,-.73f,-.67f,.96f,-.73f,.61f,.89f,-.97f,.80f,.30f,-.42f,.92f,.81f,.03f,.48f,.98f,-.04f,-.68f,.13f,.90f,.17f,.59f,.48f,-.40f,-.70f,-.25f,-.46f,.69f,-.02f,-.53f,-.60f,.55f,-.49f,-.47f,-.83f,.59f,-.26f,.21f,.86f,-.51f,-.26f,.59f,-.18f,-.93f,.55f,-.20f,-.92f,.87f,.95f,-.32f,.84f,.78f,.43f,.56f,.43f,.36f,.14f,-.19f,-.84f,-.30f,.48f,.63f,-.62f,-.54f,-.28f,-.08f,.06f,.71f,.64f,.60f,.88f,-.68f,-.91f,-.03f,.79f,.69f,-.51f,-.24f,.95f,-.41f,.01f,.97f,.94f,-.74f,.28f,.55f,-.48f,-.63f,-.85f,-.70f,.56f,.37f,.96f,-.80f,-.08f,.39f,.92f,-.02f,.85f,-.28f,-.14f,.81f,.35f,-.17f,-.39f,.12f,.36f,-.06f,.88f,-.59f,-.25f,-.50f,-.34f,.71f,-.72f,-.75f,.40f,.29f,.29f,.61f,.49f,-.79f,.05f,.03f,.48f,.58f,-.38f,.98f,.60f,-.79f,-.15f,-.24f,.37f,.51f,.95f,-.94f,.30f,-.58f,.62f,.57f,-.86f,.69f,-.18f,-.29f,.33f,.56f,-.37f,.02f,.12f,-.97f,.80f,-.46f,.55f,-.82f,-.60f,-.86f,-.80f,.60f,-.09f,-.51f,-.46f,-.86f,-.46f,-.30f,-.86f,.33f,.31f,-.73f,-.83f,.33f,.35f,-.26f,.58f,.75f,.89f,-.10f,-.55f,-.37f,.51f,.14f,-.37f,.58f,.08f,.54f,.37f,.43f,-.03f,.18f,-.48f,.27f,-.22f,.57f,-.27f,-.89f,.05f,.53f,-.71f,-.12f,-.53f,.57f,-.38f,-.22f,.49f,-.47f,.58f,-.26f,-.15f,.47f,.32f,.97f,-.82f,.70f,.15f,-.08f,.16f,-.76f,.14f,-.75f,-.49f,.68f,-.31f,.02f,.20f,.66f,.16f,-.10f,.13f,-.48f,-.78f,-.81f,-.61f,-.84f,-.23f,-.65f,.18f,.72f,.50f,-.09f,.56f,.97f,-.98f,-.49f,.59f,.37f,.32f,-.61f,.24f,-.76f,-.36f,-.69f,.02f,-.29f,.22f,.70f,.49f,-.99f,-.88f,.28f,-.38f,.17f,.73f,-.86f,-.17f,-.79f,.84f,.47f,.86f,-.23f,-.03f,-.69f,-.38f,-.92f,-.86f,-.46f,-.11f,.32f,.37f,-.29f,-.41f,-.05f,-.11f,.48f,-.16f,-.74f,-.83f,-.07f,-.04f,.14f,.31f,.39f,-.91f,.64f,-.05f,.22f,-.61f,-.70f,-.82f,-.45f,.31f,-.48f,-.31f,.61f,-.31f,.40f,.86f,-.92f,.16f,.02f,.49f,-.60f,-.72f,.71f,.03f,-.67f,.27f,.94f,.06f,.47f,.24f,-.72f,-.41f,.08f,.83f,-.81f,-.61f,-.48f,.08f,-.06f,.35f,-.65f,.91f,-.77f,-.90f,.38f,-.42f,-.48f,.37f,-.71f,.36f,.57f,-.08f,.36f,-.87f,.70f,-.53f,-.00f,.65f,-.73f,.19f,-.09f,.42f,.74f,-.29f,-.31f,-.66f,-.01f,-.40f,.71f,-.36f,.69f,.67f,-.86f,.64f,-.02f,.06f,.49f,.42f,.34f,-.35f,-.58f,-.94f,-.17f,.54f,-.91f,-.31f,.54f,-.19f,.69f,-.39f,.65f,.18f,-.54f,.64f,.43f,-.12f,.44f,-.95f,-.62f,.15f,-.71f,.00f,-.33f,-.30f,-.32f,-.36f,.44f,-.96f,-.65f,.02f,-.21f,.34f,.03f,.10f,-.13f,.58f,.40f,.10f,.75f,-.32f,-.22f,-.48f,.87f,-.03f,-.02f,.82f,.06f,.56f,.41f,.16f,.41f,-.52f,.38f,-.09f,.07f,-.42f,.70f,-.16f,-.13f,.22f,.83f,.56f,-.03f,.20f,-.34f,-.52f,.50f,.73f,.34f,.05f,-.39f,.61f,.67f,-.63f,.27f,-.73f,-.73f,.84f,-.65f,-.51f,-.05f,-.46f,-.38f,-.00f,.38f,-.40f,-.52f,.51f,-.63f,-.40f,-.02f,.39f,.18f,.60f,.30f,.32f,.23f,.77f,.21f,.12f,.49f,.30f,.72f,-.85f,.96f,.23f,-.08f,.57f,.20f,.87f,.99f,.54f,-.76f,.11f,.97f,-.10f,.19f,-.45f,-.12f,.85f,-.21f,.69f,.71f,-.61f,.01f,-.73f,-.61f,.51f,.33f,-.71f,.76f,.67f,.35f,-.51f,-.62f,-.46f,.28f,-.80f,.62f,.65f,-.11f,.87f,.24f,-.97f,.60f,-.03f,-.83f,.32f,-.49f,-.94f,-.42f,-.47f,.50f,.34f,.84f,.11f,-.86f,-.24f,.69f,-.53f,.89f,.00f,.08f,-.81f,-.07f,-.18f,-.52f,-.39f,-.94f,.04f,.89f,-.30f,.44f,-.68f,.49f,.40f,.65f,-.80f,-.72f,.42f,-.49f,.91f,.45f,.26f,.60f,.00f,.20f,.27f,-.00f,.27f,.91f,.44f,.84f,.81f,-.09f,.38f,-.49f,.56f,.04f,.54f,-.79f,.05f,.56f,.34f,-.20f,.12f,.00f,.48f,.18f,.15f,-.86f,.57f,-.03f,-.12f,.56f,-.33f,-.80f,.03f,.00f,.90f,.00f,.50f,.15f,-.09f,-.43f,.17f,-.28f,.21f,-.86f,-.41f,.69f,-.40f,.11f,-.95f,-.08f,-.98f,-.67f,.73f,-.97f,-.67f,-.09f,.52f,-.25f,.26f,.09f,-.66f,-.16f,.01f,.86f,-.11f,-.78f,.15f,-.02f,-.23f,.18f,.91f,.29f,.21f,-.53f,-.65f,.79f,-.68f,.87f,-.22f,-.07f,-.43f,.50f,.26f,.05f,.55f,-.30f,-.16f,-.43f,.56f,-.16f,.03f,-.92f,.32f,-.69f,.05f,-.34f,-.90f,.67f,-.24f,-.24f,.97f,.18f,-.31f,-.45f,-.98f,.94f,.84f,.52f,.80f,.71f,-.26f,.04f,-.78f,.10f,-.21f,-.21f,.42f,-.36f,-.98f,-.29f,-.79f,-.29f,-.89f,-.82f,.17f,-.11f,.59f,-.30f,.43f,.40f,.54f,-.35f,-.60f,-.30f,.61f,.52f,.20f,-.40f,.07f,.81f,.50f,.00f,.26f,.06f,.01f,-.43f,.68f,-.25f,-.28f,.01f,.29f,.87f,.68f,.18f,-.26f,-.16f,-.43f,-.59f,.74f,.91f,.33f,.88f,.07f,-.67f,.26f,-.88f,-.31f,-.25f,.63f,.08f,-.60f,-.08f,-.29f,-.41f,-.12f,-.65f,-.20f,-.73f,.50f,-.50f,-.55f,.80f,.01f,-.47f,-.33f,.52f,-.59f,-.61f,.99f,-.94f,.32f,-.50f,-.41f,.66f,-.06f,-.30f,.79f,.33f,.21f,-.79f,.09f,.66f,-.13f,-.32f,.05f,.32f,.07f,-.11f,-.16f,.72f,.69f,.76f,-.14f,-.96f,-.26f,.93f,.84f,-.00f,.23f,-.68f,.65f,.64f,-.37f,.94f,-.70f,-.04f,-.39f,.90f,-.62f,.02f,.88f,.77f,-.40f,.01f,-.45f,-.92f,-.18f,.79f,.52f,.86f,.57f,-.39f,-.39f,-.47f,.47f,-.70f,-.10f,.27f,.99f,-.62f,-.11f,-.99f,-.68f,.81f,-.94f,.08f,-.01f,.42f,.50f,.05f,.67f,.87f,-.11f,-.55f,-.20f,.12f,-.77f,.50f,-.46f,-.69f,-.76f,-.21f,-.49f,.08f,-.29f,-.44f,.59f,.17f,-.34f,.57f,-.00f,-.21f,.65f,.04f,-.26f,-.73f,.95f,.28f,-.33f,-.95f,-.55f,-.52f,-.93f,.57f,-.56f,-.50f,-.62f,-.73f,.94f,-.37f,.04f,-.09f,-.12f,-.71f,-.38f,-.08f,-.56f,.27f,.45f,-.47f,.84f,.24f,-.76f,-.10f,.96f,.72f,.67f,.04f,.91f,.50f,.29f,-.34f,-.87f,.00f,-.74f,.76f,-.20f,.38f,.54f,-.69f,.54f,-.24f,-.16f,.79f,-.17f,.53f,-.25f,-.54f,.40f,-.78f,-.41f,-.25f,.85f,.70f,-.77f,-.23f,-.96f,.75f,.52f,.59f,-.85f,.70f,-.23f,-.71f,-.22f,.87f,.44f,.57f,.75f,.44f,-.44f,.45f,.41f,-.90f,-.86f,.51f,.56f,-.41f,-.55f,.53f,.78f,-.71f,-.43f,-.20f,.98f,-.39f,-.42f,-.52f,.40f,-.67f,-.32f,.12f,.95f,.99f,-.75f,.97f,-.33f,-.24f,.31f,-.50f,-.54f,-.20f,.92f,-.81f,.38f,-.62f,-.39f,.82f,.91f,-.75f,.03f,-.06f,-.61f,-.17f,-.91f,.27f,.30f,-.75f,-.51f,.50f,.60f,-.96f,-.91f,-.52f,.34f,.92f,.31f,-.02f,.89f,.93f,-.77f,.53f,.98f,.01f,.45f,.56f,-.41f,.38f,-.96f,.72f,-.11f,-.05f,.88f,-.78f,-.27f,-.68f,-.41f,-.16f,-.08f,.70f,.16f,.13f,-.29f,.53f,-.52f,-.75f,.99f,.73f,.11f,.11f,-.05f,.27f,-.87f,-.19f,-.59f,-.73f,.74f,-.18f,.13f,-.32f,.55f,.03f,-.20f,.45f,-.06f,-.10f,-.86f,.94f,-.63f,.88f,.23f,.28f,-.62f,-.59f,.76f,.41f,-.29f,.55f,.01f,.36f,.33f,.06f,.67f,-.63f,-.36f,.77f,-.72f,.76f,-.87f,.95f,-.28f,-.83f,-.67f,-.64f,-.49f,.55f,.06f,.90f,-.01f,-.58f,.94f,-.76f,-.24f,.58f,-.99f,.17f,-.14f,.65f,-.80f,.77f,-.18f,-.37f,.07f,-.71f,-.31f,.43f,.59f,-.52f,-.73f,-.17f,.04f,-.25f,-.62f,.90f,.40f,-.60f,-.31f,.52f,-.60f,.90f,.78f,.13f,-.27f,-.71f,.71f,.24f,.93f,-.39f,.33f,-.02f,.45f,.61f,-.50f,.81f,.95f,-.70f,.72f,.03f,-.16f,.07f,-.28f,-.83f,.84f,-.48f,-.42f,.62f,.62f,.39f,.88f,.03f,-.41f,-.49f,-.80f,-.60f,.55f,.83f,-.02f,-.69f,-.19f,.65f,.49f,-.32f,-.28f,.05f,-.40f,-.03f,-.39f,.48f,-.75f,-.02f,-.91f,-.25f,.33f,.80f,-.30f,-.14f,-.36f,.61f,.82f,.48f,.31f,-.87f,.45f,.06f,.15f,.08f,-.39f,.10f,-.21f,.63f,-.47f,.55f,.47f,.90f,.32f,-.29f,.79f,-.76f,.75f,-.90f,.56f,.92f,-.04f,-.73f,-.75f,-.68f,-.74f,.99f,-.68f,-.49f,.31f,.76f,-.76f,-.03f,-.95f,-.75f,-.89f,-.32f,.91f,-.34f,.58f,-.43f,.33f,-.88f,-.32f,.90f,.23f,-.40f,-.86f,.81f,.26f,-.90f,.78f,-.61f,.65f,-.75f,.69f,-.29f,-.17f,-.92f,.84f,.80f,-.35f,.53f,.73f,-.60f,-.75f,-.46f,-.68f,-.13f,.80f,-.21f,.38f,.89f,-.53f,.34f,-.32f,.71f,.08f,-.43f,.27f,-.17f,.48f,.75f,-.89f,-.47f,-.94f,-.23f,.52f,-.47f,.34f,-.84f,-.81f,-.14f,.36f,-.73f,-.35f,-.06f,-.66f,-.16f,-.84f,-.95f,-.52f,-.02f,-.28f,.85f,.91f,-.61f,-.16f,-.82f,.10f,.19f,-.51f,.04f,-.23f,.74f,-.18f,.31f,-.63f,-.37f,-.27f,-.95f,-.36f,-.32f,-.85f,.52f,.16f,.75f,-.68f,-.99f,-.18f,-.82f,.90f,.61f,-.96f,.55f,.61f,-.82f,.15f,.21f,-.46f,.06f,.35f,.95f,.25f,-.10f,-.08f,.58f,.14f,-.54f,.21f,.48f,.53f,.09f,.75f,.82f,-.12f,-.45f,.65f,.29f,-.04f,.43f,.84f,-.97f,.04f,-.14f,-.02f,.09f,-.46f,.01f,-.90f,-.54f,-.71f,-.03f,-.45f,-.02f,-.49f,-.71f,.06f,-.48f,.90f,.83f,.20f,.42f,-.99f,-.11f,.53f,-.35f,-.01f,.67f,-.68f,-.08f,-.52f,.44f,-.61f,-.58f,-.15f,-.47f,.90f,.37f,-.05f,-.52f,.54f,.88f,.19f,-.36f,.19f,.82f,.04f,-.37f,.36f,.97f,-.93f,.97f,-.82f,.28f,.17f,-.19f,.17f,-.24f,-.69f,.48f,-.01f,.43f,-.70f,.98f,-.49f,-.51f,.97f,.05f,-.87f,.57f,-.93f,-.45f,.78f,-.22f,.70f,.91f,-.54f,.67f,-.53f,-.01f,.38f,.99f,-.56f,.85f,.67f,.88f,-.02f,.33f,-.00f,.74f,.33f,-.37f,-.92f,.79f,-.93f,.24f,.18f,-.56f,-.83f,-.22f,.79f,-.04f,.54f,-.11f,-.40f,.27f,.44f,.50f,.24f,.96f,-.02f,.24f,.18f,-.95f,-.03f,.93f,-.15f,.86f,-.24f,-.30f,.80f,-.05f,.91f,.55f,.05f,.44f,-.43f,.00f,-.96f,-.55f,.52f,-.11f,-.56f,-.08f,.91f,.05f,-.70f,.13f,.43f,.07f,.81f,-.49f,.14f,-.89f,-.38f,.19f,.57f,.94f,.66f,.98f,.75f,.37f,-.12f,.12f,.96f,.40f,-.10f,-.64f,-.88f,-.64f,.95f,-.81f,.61f,-.97f,-.68f,-.73f,-.00f,-.69f,-.99f,.73f,-.09f,-.49f,.22f,.04f,.77f,.79f,-.06f,-.35f,.06f,.81f,-.03f,-.05f,.47f,.27f,-.74f,-.59f,.82f,.47f,.80f,.80f,.58f,-.87f,.05f,.50f,-.26f,-.50f,-.84f,.12f,.62f,.66f,-.77f,-.10f,.74f,.66f,-.07f,.73f,.74f,.52f,.10f,-.78f,.24f,.92f,.02f,-.12f,.80f,.60f,.76f,.96f,.57f,.94f,-.37f,-.01f,-.20f,-.95f,-.37f,-.61f,.74f,.82f,.31f,.32f,-.47f,-.91f,-.60f,-.78f,.19f,-.65f,.44f,-.63f,-.19f,-.62f,.39f,-.58f,-.80f,-.47f,.29f,-.31f,.19f,.13f,.59f,-.52f,.32f,.71f,-.69f,-.21f,-.94f,-.80f,-.82f,-.81f,-.92f,-.08f,-.68f,.19f,.31f,-.02f,-.60f,.97f,-.71f,.21f,-.18f,.19f,.83f,.79f,-.18f,-.91f,-.77f,-.80f,-.00f,.08f,.30f,-.86f,-.88f,-.01f,-.88f,.71f,-.80f,-.54f,.43f,-.68f,-.56f,.32f,.05f,.21f,-.51f,.17f,-.42f,.01f,.00f,.03f,.16f,-.77f,.62f,-.91f,-.70f,-.42f,-.49f,.46f,-.82f,.97f,-.49f,-.49f,.42f,.23f,-.19f,.23f,.99f,-.57f,-.93f,-.46f,.05f,-.86f,.23f,.94f,-.68f,.73f,.08f,.33f,.63f,.65f,.11f,.41f,-.16f,-.05f,.80f,.65f,.07f,-.76f,-.69f,.34f,.18f,.69f,.58f,.67f,.56f,.02f,-.72f,.07f,.38f,-.30f,.59f,.00f,-.67f,.52f,.40f,.60f,-.26f,-.16f,-.99f,-.89f,.48f,.03f,-.59f,-.62f,-.51f,.10f,.79f,.47f,.66f,.23f,-.51f,-.08f,-.84f,-.38f,.27f,.11f,-.46f,.50f,-.75f,-.49f,-.44f,-.86f,-.46f,.70f,-.21f,-.33f,-.69f,.91f,.79f,.29f,-.90f,.70f,-.92f,-.49f,.85f,-.58f,-.71f,.53f,.88f,.29f,.35f,.75f,-.23f,.90f,-.49f,.04f,.07f,.13f,-.12f,-.84f,-.09f,-.19f,-.26f,-.73f,-.66f,-.83f,-.14f,.38f,-.95f,.39f,-.69f,.46f,.44f,.53f,-.38f,.01f,-.41f,.10f,.90f,-.36f,.58f,.41f,.11f,-.73f,.14f,.97f,.89f,-.24f,.49f,.65f,-.52f,.63f,-.82f,.34f,.95f,-.90f,-.19f,-.48f,-.05f,-.92f,-.48f,-.09f,-.72f,.64f,-.92f,.64f,-.85f,-.78f,-.44f,-.67f,-.48f,.91f,-.45f,.23f,-.35f,-.10f,.49f,.65f,.29f,.03f,.59f,-.92f,.24f,.48f,.86f,.29f,.77f,.40f,-.56f,-.27f,-.99f,.95f,.99f,-.65f,.71f,.21f,-.14f,.51f,.25f,-.31f,.73f,.37f,.62f,.58f,-.81f,-.68f,.41f,-.88f,-.20f,.40f,-.71f,-.71f,.97f,-.28f,-.73f,.36f,-.57f,-.56f,-.33f,.98f,-.32f,.25f,-.85f,-.16f,-.14f,-.17f,-.57f,.12f,.03f,-.77f,.12f,-.58f,-.90f,.51f,-.29f,-.47f,.89f,-.81f,.69f,.70f,-.40f,.15f,-.66f,.54f,-.75f,-.81f,-.87f,-.26f,.31f,-.56f,-.15f,.04f,-.86f,.07f,.19f,-.80f,-.94f,.04f,-.01f,.87f,.83f,-.44f,-.77f,-.73f,.24f,-.87f,-.61f,.85f,.40f,-.83f,.18f,-.88f,.49f,-.27f,-.25f,-.59f,-.47f,-.91f,-.81f,-.60f,-.73f,.78f,-.81f,.70f,.20f,.44f,-.60f,-.82f,-.56f,.90f,.08f,-.00f,.06f,.17f,-.42f,.34f,.80f,-.92f,.02f,.44f,-.54f,.45f,.69f,-.19f,-.06f,.32f,-.38f,.72f,.96f,.47f,-.56f,-.50f,.09f,-.60f,-.92f,.10f,-.34f,.16f,.74f,.03f,-.54f,.63f,.60f,.69f,.20f,-.43f,.31f,-.68f,-.64f,-.46f,.61f,-.74f,.56f,.11f,-.44f,-.02f,-.38f,-.96f,.95f,.38f,.24f,-.74f,.91f,-.54f,-.61f,.67f,.38f,.91f,.05f,-.53f,-.08f,.04f,-.86f,-.90f,-.92f,.08f,-.49f,.22f,.92f,.76f,-.30f,.85f,-.76f,.15f,.28f,-.52f,-.16f,.84f,.48f,.70f,.62f,-.52f,.37f,-.43f,.52f,-.87f,.91f,.13f,.15f,.10f,.62f,.72f,-.69f,-.53f,-.09f,-.37f,.76f,-.76f,-.50f,.86f,-.17f,-.73f,-.39f,-.05f,.91f,-.26f,.39f,-.69f,.26f,.97f,-.23f,.61f,.77f,-.28f,.17f,.32f,.85f,-.80f,-.85f,-.03f,-.59f,-.91f,.28f,-.14f,.46f,-.53f,-.75f,-.84f,.52f,-.35f,-.97f,.94f,-.44f,-.26f,.54f,-.51f,.41f,.41f,.89f,-.54f,-.27f,.72f,-.34f,.01f,.19f,.23f,-.30f,-.61f,-.39f,-.11f,.45f,-.22f,-.66f,.06f,.15f,.46f,-.00f,.91f,.25f,.37f,-.13f,.26f,.08f,.78f,-.77f,-.68f,.21f,-.80f,.02f,.33f,.65f,.33f,.02f,-.97f,.01f,-.47f,-.48f,.46f,-.70f,.47f,.01f,-.98f,.49f,-.12f,-.62f,-.45f,.57f,-.05f,.91f,.29f,-.85f,-.14f,-.51f,-.31f,.48f,.81f,-.11f,.10f,.72f,.27f,-.97f,.24f,-.18f,-.09f,-.77f,-.70f,.98f,.50f,.58f,.10f,-.19f,.12f,.05f,.77f,.17f,-.38f,-.17f,.15f,-.18f,.45f,.10f,.86f,.23f,-.74f,-.18f,.41f,.30f,-.75f,-.25f,-.29f,.86f,-.21f,.34f,-.62f,.89f,-.04f,-.34f,-.04f,.38f,-.59f,.02f,.52f,.84f,.14f,.48f,-.89f,-.86f,.12f,-.07f,-.21f,-.94f,-.36f,-.49f,.26f,.70f,.45f,.16f,-.98f,-.85f,-.38f,-.01f,-.31f,-.23f,-.16f,.93f,-.68f,.12f,-.92f,-.55f,-.15f,-.68f,-.59f,-.57f,.79f,-.21f,.17f,-.14f,-.30f,-.35f,.07f,-.15f,-.25f,-.04f,.01f,.28f,-.54f,.85f,-.59f,-.69f,-.18f,.42f,-.93f,-.02f,-.69f,.94f,.11f,-.33f,.34f,.25f,-.77f,-.82f,.33f,-.29f,-.68f,-.52f,-.49f,.57f,.47f,-.60f,-.45f,.69f,.82f,-.15f,-.83f,-.07f,-.79f,-.63f,-.42f,-.62f,-.25f,.78f,.69f,.32f,.36f,.73f,.52f,-.74f,-.11f,.04f,-.96f,-.50f,-.47f,-.88f,-.26f,.67f,-.59f,-.33f,.78f,-.59f,.21f,.13f,-.31f,-.19f,.15f,.26f,-.40f,.43f,-.96f,.75f,-.61f,-.04f,.66f,-.10f,.65f,.12f,-.20f,.98f,.99f,-.75f,.51f,.93f,-.27f,.02f,-.85f,-.62f,-.35f,-.04f,.96f,.85f,.89f,-.38f,-.68f,.02f,-.88f,-.67f,.00f,.92f,-.88f,.56f,.46f,-.72f,.66f,.80f,.48f,.19f,.27f,.15f,-.77f,-.45f,-.94f,.11f,.37f,.22f,-.54f,-.27f,-.67f,-.76f,-.15f,-.20f,.93f,-.05f,.27f,.20f,.72f,-.40f,-.72f,-.49f,.81f,.06f,.83f,.09f,.08f,-.65f,-.26f,.74f,-.22f,-.96f,-.62f,-.45f,.60f,-.10f,.17f,.12f,-.17f,-.74f,.08f,.12f,-.64f,.68f,.92f,.00f,-.04f,.31f,-.83f,-.47f,-.55f,-.89f,-.42f,-.96f,-.62f,.19f,-.47f,.19f,-.46f,-.70f,.39f,-.10f,-.82f,.19f,-.06f,-.47f,-.03f,-.46f,.22f,-.76f,.34f,-.80f,.31f,.41f,.17f,.90f,.47f,.85f,-.94f,.31f,-.81f,.24f,.42f,.20f,.15f,-.24f,-.69f,-.87f,-.65f,.09f,-.25f,.36f,-.38f,.04f,-.80f,.54f,.82f,-.95f,.86f,.40f,.70f,-.34f,-.58f,-.75f,-.37f,.39f,-.62f,.05f,-.76f,-.91f,.65f,.38f,.76f,.36f,-.73f,.06f,-.01f,-.05f,.41f,-.66f,.75f,.56f,.55f,-.09f,.82f,.49f,-.35f,.91f,-.54f,.63f,.00f,-.34f,-.30f,-.08f,.90f,.76f,.41f,-.48f,.95f,.61f,.28f,.41f,-.86f,-.79f,.25f,-.48f,.35f,.22f,-.38f,.33f,.81f,-.85f,-.60f,.87f,.77f,.13f,.90f,.46f,.74f,-.66f,.31f,.00f,.87f,.60f,-.04f,.30f,.61f,.93f,.59f,.19f,.26f,-.93f,.24f,.64f,-.72f,.62f,.31f,-.11f,-.24f,.00f,.79f,.98f,-.43f,-.60f,-.10f,.31f,.38f,-.21f,.81f,-.96f,-.87f,-.66f,.92f,.21f,.53f,.22f,-.25f,-.20f,.02f,.93f,.39f,.38f,.71f,-.38f,.73f,.58f,-.81f,.78f,-.92f,-.26f,.39f,.83f,.85f,.90f,-.26f,.12f,-.40f,-.17f,.54f,.85f,-.38f,-.68f,-.77f,.90f,.08f,.26f,.93f,.52f,-.05f,-.19f,-.64f,.96f,.92f,-.90f,.77f,.05f,-.27f,.76f,.63f,-.15f,.14f,.24f,-.37f,-.39f,.62f,-.68f,-.46f,-.21f,-.72f,.67f,-.75f,-.48f,-.85f,-.06f,-.37f,.07f,.63f,.94f,-.28f,-.88f,-.16f,-.20f,-.23f,.45f,-.02f,-.78f,.76f,-.63f,.09f,-.35f,.91f,.41f,-.89f,.67f,.57f,-.95f,.41f,.62f,-.61f,.17f,-.24f,.34f,.11f,-.25f,-.29f,-.22f,.99f,.73f,-.54f,.39f,-.74f,.32f,.39f,-.40f,.18f,.94f,.94f,.47f,.27f,-.61f,-.62f,.78f,.77f,-.91f,.41f,.67f,-.17f,-.16f,.86f,.50f,.71f,.68f,.54f,.68f,.27f,.35f,-.84f,-.04f,-.06f,.14f,.64f,-.61f,-.51f,-.17f,-.22f,-.55f,.68f,.52f,-.38f,-.18f,.21f,.34f,-.29f,-.85f,-.88f,.92f,.44f,-.02f,.59f,-.06f,-.62f,-.11f,-.72f,-.88f,-.91f,-.59f,-.77f,.67f,.19f,.34f,.05f,.77f,.09f,-.08f,-.60f,.03f,.02f,.00f,.98f,.75f,.96f,.05f,-.09f,.25f,-.53f,.26f,.00f,.26f,.77f,.74f,.38f,.27f,.49f,-.85f,.67f,.59f,.45f,.70f,-.21f,.78f,.74f,-.83f,.85f,-.38f,.78f,-.82f,-.70f,-.36f,.21f,.32f,-.66f,-.33f,.50f,.26f,.59f,-.13f,.86f,-.01f,.52f,-.65f,-.02f,-.82f,.04f,.64f,.62f,.94f,.70f,-.55f,-.03f,.38f,-.45f,.70f,-.17f,.76f,.35f,-.91f,-.85f,.72f,-.18f,-.09f,.54f,-.54f,-.19f,-.97f,-.98f,-.60f,.38f,-.36f,.58f,.56f,.29f,.16f,.27f,-.91f,.53f,-.06f,-.42f,.05f,-.05f,.25f,-.46f,.92f,.37f,-.87f,.14f,.98f,-.33f,.07f,.98f,.16f,-.19f,-.22f,-.45f,.61f,-.77f,.59f,.32f,.34f,.85f,.98f,-.56f,-.46f,-.83f,.64f,-.78f,.33f,.38f,.49f,.90f,-.73f,.50f,.51f,-.25f,-.80f,.17f,-.88f,-.30f,.57f,-.48f,-.83f,-.05f,-.27f,-.28f,.80f,-.26f,-.17f,-.61f,.25f,-.47f,-.65f,-.29f,-.43f,-.88f,-.02f,.97f,.04f,.94f,-.47f,.46f,.19f,-.00f,.70f,.36f,.93f,-.47f,-.85f,-.44f,-.26f,-.44f,.41f,-.50f,-.77f,-.98f,.30f,-.75f,-.53f,-.77f,-.89f,.92f,-.35f,.22f,-.14f,-.50f,.41f,.78f,-.89f,.59f,-.62f,-.82f,.50f,.98f,.85f,.84f,.63f,.40f,.04f,.15f,-.70f,.95f,.10f,.25f,-.02f,-.70f,.88f,-.56f,.56f,-.58f,.52f,-.11f,.02f,-.35f,-.62f,-.15f,-.06f,.15f,.45f,-.99f,.57f,-.19f,.11f,.84f,.02f,-.05f,-.53f,.74f,-.85f,-.07f,.33f,.45f,-.45f,-.74f,.07f,.38f,.60f,.36f,.57f,-.27f,-.54f,-.01f,.68f,-.97f,.67f,-.06f,.67f,.01f,.03f,.65f,.18f,-.81f,.23f,.06f,.45f,.42f,-.17f,.76f,-.27f,-.29f,-.17f,.02f,-.42f,-.46f,-.93f,-.17f,-.78f,-.43f,-.04f,-.69f,.29f,.35f,-.91f,-.97f,.09f,.63f,-.20f,.15f,-.63f,-.96f,-.95f,.60f,.91f,.09f,-.70f,.50f,-.07f,.44f,.01f,.75f,.73f,-.73f,.16f,-.80f,.52f,.64f,-.02f,-.17f,.04f,-.50f,.72f,-.06f,.80f,-.50f,-.78f,-.37f,-.16f,-.06f,-.73f,-.68f,-.45f,.43f,-.41f,-.01f,-.75f,.41f,-.94f,.86f,-.33f,.65f,-.82f,-.53f,-.19f,-.21f,.75f,.39f,-.52f,-.82f,.05f,.29f,-.84f,-.03f,-.05f,-.92f,.66f,.14f,.19f,-.98f,.09f,.48f,.96f,.39f,.67f,-.49f,.42f,.30f,.84f,-.46f,-.41f,.92f,-.93f,.51f,-.90f,-.54f,-.32f,.18f,-.83f,-.05f,.78f,-.76f,.76f,-.50f,-.94f,-.58f,.52f,.45f,-.17f,.62f,.48f,.85f,.11f,.74f,.99f,.94f,-.77f,.44f,-.89f,.97f,-.77f,.16f,-.35f,.16f,-.45f,.50f,-.96f,.74f,-.52f,-.64f,.13f,-.21f,-.03f,-.16f,.35f,.53f,-.82f,.34f,.52f,-.59f,-.17f,-.53f,-.88f,.67f,.13f,-.21f,.93f,-.45f,-.11f,-.60f,.84f,.16f,.76f,.01f,.89f,.87f,.04f,.86f,.20f,.20f,.61f,-.21f,.04f,.26f,.91f,-.40f,.00f,-.26f,-.12f,.35f,.21f,-.87f,.81f,-.03f,-.82f,.19f,-.71f,.93f,.39f,-.70f,-.97f,.20f,.92f,.13f,-.81f,.88f,.34f,-.27f,-.46f,.78f,.94f,-.50f,.57f,.06f,-.52f,.14f,.33f,-.34f,-.31f,-.05f,-.73f,.13f,-.63f,.88f,-.44f,-.15f,.11f,.90f,-.36f,.51f,-.38f,-.64f,-.21f,.13f,.89f,.81f,-.69f,.15f,-.67f,-.89f,-.47f,.73f,-.15f,-.97f,-.24f,-.57f,.91f,-.64f,-.30f,.67f,-.55f,-.68f,.98f,-.98f,.96f,-.36f,-.55f,.11f,-.73f,.57f,.73f,-.46f,-.11f,.52f,.48f,.06f,.43f,-.41f,-.43f,.58f,-.69f,.57f,.71f,-.73f,.19f,-.45f,-.24f,.96f,.61f,.66f,-.50f,-.46f,.32f,.43f,.15f,-.93f,.77f,.60f,-.27f,-.37f,.31f,-.69f,.20f,-.22f,.90f,.51f,-.56f,-.50f,-.55f,.64f,-.03f,.70f,.90f,-.83f,-.11f,-.70f,.82f,.62f,.66f,.26f,.24f,-.55f,.72f,.42f,.58f,.60f,-.02f,-.29f,.73f,-.04f,-.45f,.01f,-.04f,-.59f,.86f,-.29f,.82f,.56f,-.76f,-.58f,.77f,.96f,-.28f,.71f,.19f,-.26f,.98f,.32f,-.75f,-.24f,.85f,-.26f,-.73f,.49f,.26f,-.74f,.08f,-.53f,-.58f,.52f,.41f,.06f,.20f,.47f,-.41f,.14f,.00f,.94f,.76f,-.41f,-.90f,.92f,-.29f,.52f,-.94f,.37f,.87f,-.06f,-.44f,-.33f,-.04f,.78f,-.55f,-.23f,.25f,.08f,.41f,-.48f,.05f,-.37f,.39f,-.74f,-.89f,.51f,-.80f,-.70f,-.94f,.32f,-.43f,-.75f,.63f,-.33f,-.67f,-.33f,.18f,.65f,.34f,.92f,.08f,.92f,.04f,-.33f,-.59f,.62f,.85f,-.49f,.39f,-.37f,-.04f,-.11f,.23f,-.41f,-.84f,.71f,-.35f,.88f,.70f,.94f,.19f,.85f,-.35f,.84f,-.99f,-.70f,.57f,-.78f,-.96f,-.55f,.79f,.67f,.09f,-.28f,-.11f,.22f,.92f,.79f,-.70f,-.62f,.56f,-.70f,-.15f,.18f,.65f,.97f,-.87f,-.04f,-.75f,-.65f,.12f,-.09f,.85f,.98f,.58f,.09f,-.06f,.70f,.19f,-.89f,-.53f,-.26f,-.79f,-.07f,.19f,-.68f,.72f,.87f,-.10f,.68f,-.12f,-.44f,.26f,-.30f,.36f,-.98f,-.76f,-.93f,.93f,-.68f,-.39f,-.06f,.87f,-.19f,.60f,.96f,.91f,.05f,-.95f,.13f,.35f,.87f,.67f,-.48f,.26f,.19f,.74f,-.53f,-.54f,.60f,.08f,.50f,.18f,.89f,.89f,-.19f,.10f,.72f,-.13f,-.38f,-.45f,.19f,.33f,-.01f,-.34f,-.28f,-.42f,-.39f,.48f,.11f,-.66f,.22f,-.08f,.71f,.86f,.02f,-.02f,.39f,-.86f,-.51f,.50f,-.42f,.12f,.44f,-.47f,-.43f,.11f,-.92f,-.07f,.83f,-.69f,-.28f,-.13f,.05f,-.49f,.07f,-.34f,.18f,-.42f,.57f,.44f,.57f,.45f,.97f,-.57f,.30f,-.87f,-.90f,-.11f,-.14f,.89f,.74f,.35f,.30f,-.16f,-.17f,.23f,-.63f,-.95f,.79f,-.39f,.38f,.34f,-.69f,-.45f,-.34f,.96f,-.02f,-.77f,.15f,.37f,-.61f,-.83f,.48f,-.57f,.52f,-.71f,-.02f,-.83f,.34f,.29f,-.05f,.73f,-.15f,.89f,-.17f,-.81f,.59f,.11f,.92f,.31f,.71f,-.89f,-.70f,.31f,.75f,-.59f,-.89f,-.81f,-.81f,.23f,.12f,.26f,.56f,.01f,.24f,-.26f,.61f,-.58f,.57f,-.95f,-.46f,.78f,.06f,-.32f,.88f,.84f,-.25f,-.25f,-.32f,.48f,.28f,-.14f,.48f,.29f,.67f,.89f,.97f,.54f,-.46f,.19f,.67f,-.27f,.19f,-.64f,.68f,.30f,.24f,-.61f,-.28f,-.26f,.32f,-.30f,-.72f,.01f,.19f,-.35f,-.56f,-.02f,.17f,.45f,.07f,-.07f,-.75f,.21f,-.19f,.21f,.89f,-.13f,.51f,.70f,-.00f,-.92f,-.64f,.26f,-.13f,.69f,.28f,-.47f,-.78f,-.32f,.01f,.15f,.36f,-.28f,-.35f,.31f,-.47f,.53f,.51f,.23f,.35f,.97f,.23f,-.09f,.81f,-.26f,.08f,-.17f,-.41f,.78f,-.51f,.16f,-.27f,.05f,-.30f,.32f,.51f,.19f,.94f,.60f,.10f,.21f,.49f,-.69f,.23f,-.87f,.64f,-.36f,-.53f,-.63f,-.05f,-.95f,.23f,.85f,-.32f,-.11f,-.89f,-.13f,.31f,.28f,-.69f,-.37f,-.47f,-.79f,.78f,-.90f,.91f,-.86f,.05f,-.86f,-.74f,.19f,.68f,-.56f,.04f,.10f,-.35f,.47f,-.79f,-.28f,.46f,-.11f,.60f,.72f,-.47f,.03f,.63f,.75f,-.84f,.40f,-.25f,-.72f,.63f,-.22f,-.96f,.43f,-.61f,.73f,-.58f,.78f,-.11f,-.30f,-.59f,-.11f,.01f,.44f,.35f,.42f,-.84f,.13f,.63f,.37f,-.45f,.86f,.90f,.08f,-.50f,.89f,.54f,.50f,-.17f,-.89f,-.72f,.60f,-.75f,.75f,-.79f,.60f,.56f,-.67f,.75f,-.70f,-.15f,-.09f,-.27f,.44f,.74f,-.97f,-.72f,.47f,.83f,.60f,-.97f,.32f,.24f,.33f,.65f,.74f,-.10f,.19f,.30f,-.94f,.07f,-.04f,-.47f,.56f,-.94f,.01f,-.65f,-.18f,.41f,-.34f,-.42f,.56f,.40f,-.17f,-.98f,.62f,.27f,-.27f,.25f,.26f,.41f,-.17f,.18f,-.67f,-.17f,-.78f,.52f,.83f,.04f,-.14f,.35f,.73f,.14f,.72f,-.26f,.83f,-.60f,-.61f,.89f,.30f,.07f,.82f,-.79f,-.68f,-.42f,-.40f,-.99f,-.86f,.48f,.44f,.50f,.51f,.16f,-.77f,-.15f,-.70f,.64f,.97f,-.71f,-.54f,.23f,-.23f,.83f,.53f,-.58f,.62f,-.32f,.84f,-.66f,.95f,.76f,.92f,.48f,-.81f,.43f,.08f,-.67f,.46f,.30f,.23f,.01f,.55f,.02f,.81f,.56f,-.34f,-.69f,.05f,-.99f,.96f,-.84f,.34f,.88f,.01f,.44f,-.19f,.28f,.39f,-.70f,-.24f,-.83f,.44f,.09f,-.08f,.42f,.09f,.24f,-.86f,-.21f,-.90f,-.87f,-.91f,.29f,-.61f,-.84f,.36f,-.40f,-.45f,.46f,.05f,-.55f,.24f,.99f,-.20f,.18f,-.16f,.45f,.90f,.38f,-.37f,.24f,.90f,-.55f,.09f,-.53f,-.14f,.66f,-.59f,-.70f,.24f,.17f,.24f,-.65f,.76f,.32f,.49f,-.65f,.55f,-.59f,-.31f,.78f,-.67f,-.29f,-.60f,.31f,.26f,-.02f,.87f,-.31f,.95f,.14f,.76f,-.89f,.26f,.74f,-.41f,.10f,-.93f,-.42f,-.60f,-.96f,-.31f,.90f,-.20f,.68f,-.57f,.86f,-.71f,.68f,.26f,.75f,.79f,.09f,.94f,.20f,.11f,-.27f,-.06f,.23f,.43f,-.44f,-.87f,.56f,-.34f,.37f,-.49f,-.47f,.10f,.53f,.80f,.53f,-.44f,-.92f,.79f,-.89f,.50f,.82f,-.09f,-.14f,-.88f,.16f,-.15f,.81f,-.96f,.99f,.30f,.09f,-.84f,-.61f,.41f,-.68f,.54f,-.86f,-.87f,-.55f,.69f,.67f,-.43f,.09f,.47f,.85f,-.36f,.27f,-.10f,.93f,.34f,-.77f,.45f,-.37f,.71f,.82f,-.98f,.42f,-.07f,.50f,.66f,.62f,.13f,-.53f,-.83f,-.38f,-.03f,-.49f,-.10f,.21f,.89f,-.90f,.55f,-.62f,.42f,.45f,-.17f,-.04f,-.22f,-.20f,-.10f,.42f,.97f,-.08f,-.58f,-.25f,.02f,-.08f,-.38f,.89f,-.65f,.46f,-.35f,-.54f,.92f,.03f,.58f,-.00f,-.40f,-.55f,.99f,-.46f,-.17f,.17f,-.05f,-.28f,.57f,-.15f,-.41f,.24f,.64f,.55f,-.43f,-.52f,.79f,.59f,.23f,-.41f,-.87f,.82f,-.26f,-.10f,-.14f,-.93f,-.03f,-.51f,.17f,-.53f,-.27f,-.64f,.28f,-.98f,.03f,-.06f,.22f,.65f,.93f,.18f,-.60f,.74f,.13f,.79f,-.20f,-.12f,.08f,-.93f,-.82f,.61f,-.68f,.17f,-.66f,.23f,-.62f,-.23f,.65f,.42f,.29f,-.46f,.97f,-.42f,.85f,-.18f,-.68f,.13f,-.50f,.83f,.59f,-.44f,-.14f,.74f,.46f,-.27f,.38f,-.06f,-.34f,-.52f,.66f,.95f,.69f,-.09f,-.37f,-.35f,.57f,.19f,-.65f,-.70f,.43f,-.01f,.24f,.83f,.28f,.37f,-.18f,-.50f,.10f,.80f,.04f,.11f,.02f,-.44f,.30f,.91f,-.42f,.04f,-.48f,-.25f,-.79f,-.20f,.33f,.22f,-.55f,-.73f,.25f,.53f,.57f,.40f,.53f,-.69f,.86f,.78f,.90f,.79f,-.52f,-.90f,-.16f,.88f,.40f,.75f,.53f,-.91f,-.39f,-.79f,-.26f,.77f,-.56f,-.82f,-.17f,.94f,-.86f,.57f,-.61f,-.86f,.11f,.90f,-.50f,-.38f,.06f,.46f,.60f,-.81f,-.81f,.64f,.13f,.41f,-.12f,-.44f,-.66f,-.86f,-.08f,.16f,.18f,.45f,.84f,-.18f,-.09f,.78f,.67f,.86f,.08f,-.06f,.77f,-.88f,.45f,-.77f,-.06f,-.48f,.81f,-.48f,-.01f,.75f,.43f,-.63f,.65f,-.52f,-.72f,-.56f,-.05f,-.63f,.08f,-.34f,-.72f,-.70f,.84f,-.84f,-.64f,-.80f,.40f,.19f,.07f,.49f,.62f,.17f,-.79f,.62f,-.59f,-.82f,-.40f,-.07f,-.35f,.42f,-.23f,-.92f,.70f,.12f,.93f,.58f,.94f,-.67f,-.13f,-.96f,-.04f,-.57f,.67f,.36f,-.03f,.51f,-.87f,.58f,.85f,.54f,-.10f,-.98f,.64f,-.33f,.70f,.94f,-.53f,-.44f,-.39f,.17f,.34f,.22f,.50f,.63f,-.61f,.15f,-.82f,-.20f,-.79f,-.24f,.58f,-.48f,-.76f,.39f,-.85f,-.52f,-.90f,.00f,.06f,.40f,-.89f,.17f,-.30f,-.79f,-.86f,.44f,.55f,.49f,.54f,.12f,.72f,.17f,.65f,-.74f,-.33f,-.82f,.30f,.24f,-.39f,.47f,.79f,-.31f,.97f,-.17f,.57f,-.63f,.23f,.43f,-.55f,.61f,-.88f,.51f,-.93f,.26f,.34f,-.32f,.22f,-.38f,-.93f,.76f,-.90f,-.02f,.08f,-.07f,-.71f,.61f,-.80f,.81f,-.26f,-.82f,-.18f,.98f,.57f,.17f,-.93f,.67f,-.80f,-.07f,-.43f,.66f,.17f,-.26f,.35f,.15f,.83f,.74f,-.88f,.01f,.81f,-.90f,-.75f,.59f,.00f,-.42f,-.11f,-.54f,-.99f,.49f,.46f,.45f,.27f,.57f,-.88f,.54f,-.59f,.66f,-.71f,-.79f,-.51f,.73f,.53f,-.00f,.52f,-.84f,-.39f,.58f,.17f,-.29f,-.36f,-.57f,-.25f,.36f,.58f,-.06f,-.71f,-.35f,-.87f,-.38f,-.60f,-.75f,.30f,.97f,.55f,.31f,.61f,.59f,-.24f,-.13f,-.91f,-.07f,-.17f,-.82f,-.40f,.68f,.78f,.96f,-.70f,.19f,-.98f,.83f,.77f,-.17f,.26f,-.98f,.62f,-.43f,.74f,.50f,-.36f,-.78f,-.34f,.42f,.62f,.43f,-.17f,.83f,.25f,.51f,.07f,-.40f,.05f,-.69f,.44f,-.88f,.12f,.53f,.24f,.16f,.08f,-.97f,.00f,-.58f,.19f,.59f,.32f,-.46f,.65f,-.27f,-.30f,.63f,-.85f,-.99f,.76f,-.62f,.90f,.68f,-.98f,.40f,.15f,-.34f,-.03f,-.64f,-.03f,.99f,.14f,-.48f,-.91f,-.29f,.43f,-.40f,.36f,.15f,.73f,.00f,-.39f,-.55f,.15f,.12f,.48f,.47f,.47f,-.25f,-.45f,.68f,.48f,-.92f,-.23f,.36f,.78f,-.20f,-.15f,-.14f,.82f,-.49f,-.91f,-.92f,-.92f,-.25f,-.22f,.77f,.44f,-.86f,.16f,.76f,-.06f,.73f,-.94f,-.99f,-.14f,.52f,-.63f,-.18f,.77f,-.70f,-.92f,.36f,.06f,.57f,-.96f,.10f,.12f,-.64f,.38f,-.75f,-.09f,-.81f,-.17f,-.82f,.39f,.16f,-.49f,-.82f,-.95f,.16f,.20f,.12f,.40f,.26f,.94f,.55f,-.83f,.06f,-.04f,.42f,-.97f,.49f,-.15f,-.18f,-.40f,.43f,-.56f,.33f,.29f,-.94f,-.74f,-.20f,.47f,-.08f,-.95f,.08f,-.32f,.75f,-.67f,.36f,-.29f,-.41f,.85f,-.15f,.09f,.26f,-.40f,-.64f,-.43f,-.45f,.87f,-.62f,-.96f,.26f,.87f,-.36f,-.69f,-.93f,-.32f,.09f,.94f,-.85f,.31f,-.59f,-.08f,.10f,.33f,-.97f,-.54f,-.50f,.25f,.26f,.67f,.30f,-.98f,.67f,-.92f,-.56f,-.80f,-.53f,-.34f,-.68f,-.28f,.59f,-.56f,.94f,-.10f,-.26f,-.93f,.54f,.13f,.23f,-.81f,.36f,-.26f,.90f,.53f,-.43f,.68f,.28f,.31f,.25f,.45f,-.21f,.67f,-.56f,-.63f,.05f,.39f,.81f,-.37f,-.92f,.53f,.40f,-.31f,.23f,-.00f,-.83f,.46f,-.54f,.98f,.09f,-.54f,-.89f,.82f,-.62f,-.09f,.02f,-.24f,.42f,-.85f,.15f,-.59f,-.73f,.73f,-.19f,-.58f,-.45f,.98f,-.12f,-.24f,-.49f,-.49f,.46f,.59f,-.74f,.79f,.22f,-.11f,-.64f,-.68f,.31f,.28f,.23f,.77f,-.36f,.30f,.68f,-.51f,-.52f,.59f,.06f,.29f,.69f,-.11f,-.35f,.67f,-.18f,-.59f,-.77f,-.09f,.30f,.78f,-.36f,.06f,-.08f,-.25f,-.69f,.82f,.64f,-.41f,.40f,-.34f,.01f,-.26f,.40f,-.73f,.84f,.39f,-.42f,-.31f,-.28f,-.96f,-.75f,-.64f,.23f,-.76f,-.42f,.92f,.52f,.92f,-.58f,.69f,-.15f,.24f,-.27f,-.86f,-.09f,-.42f,-.74f,-.01f,-.18f,-.41f,.64f,.92f,-.58f,.66f,-.59f,-.64f,.20f,-.80f,-.58f,-.44f,-.34f,-.11f,-.07f,.18f,.31f,-.05f,.90f,-.10f,-.59f,.33f,.89f,-.78f,.66f,.59f,.76f,.52f,.04f,.37f,.79f,.40f,-.97f,-.41f,.04f,.10f,.72f,-.20f,-.95f,.69f,.26f,-.56f,.88f,-.09f,-.73f,-.61f,-.88f,-.53f,-.61f,-.52f,-.29f,.96f,-.61f,-.63f,.18f,.04f,-.31f,.47f,-.72f,.57f,-.69f,-.30f,.32f,.38f,-.31f,-.05f,.83f,-.17f,.40f,.33f,-.27f,.09f,.85f,-.99f,-.96f,-.57f,-.56f,-.73f,.69f,.42f,-.89f,-.24f,.99f,-.18f,.91f,-.66f,-.63f,-.24f,-.24f,-.63f,.80f,.96f,.94f,.59f,-.76f,.26f,.74f,-.89f,.50f,.16f,.21f,-.48f,.32f,-.33f,.13f,-.61f,-.01f,.63f,-.62f,.45f,-.03f,-.25f,.33f,.09f,-.96f,.00f,-.63f,.41f,.48f,.90f,-.61f,.98f,-.02f,-.45f,-.28f,.14f,-.22f,-.68f,-.46f,.64f,-.95f,-.31f,-.24f,.88f,.34f,-.34f,-.42f,.56f,-.40f,-.27f,-.09f,-.78f,.84f,.25f,.34f,-.74f,.46f,-.95f,.05f,-.72f,-.15f,.12f,.49f,-.74f,.91f,.65f,.89f,-.33f,-.57f,-.49f,-.19f,-.08f,-.36f,.74f,-.48f,.07f,.09f,-.48f,-.16f,.65f,-.29f,-.13f,.04f,.37f,-.66f,.06f,-.38f,.10f,.07f,.92f,-.41f,-.17f,.99f,.36f,-.38f,.21f,.66f,-.25f,-.32f,-.31f,.85f,-.47f,-.38f,-.40f,-.10f,.08f,.27f,-.50f,-.31f,.96f,.51f,-.83f,.63f,.65f,-.70f,.09f,-.48f,.48f,.22f,.72f,-.90f,-.06f,.19f,-.16f,.41f,-.69f,-.16f,-.71f,.78f,-.83f,-.76f,-.20f,-.81f,.64f,-.49f,.08f,-.15f,-.33f,.77f,.51f,-.24f,-.08f,.82f,-.58f,.64f,.49f,-.18f,.43f,-.54f,-.85f,-.24f,-.71f,-.90f,.69f,-.73f,.72f,.47f,-.42f,-.75f,-.51f,.80f,.18f,-.08f,-.41f,.61f,-.31f,-.39f,.97f,-.32f,.51f,-.61f,-.35f,.10f,-.08f,-.28f,-.04f,.94f,.71f,.61f,-.13f,.21f,.96f,-.61f,.85f,-.55f,.96f,-.18f,-.05f,-.85f,-.60f,.92f,-.88f,-.71f,-.40f,.14f,.69f,-.21f,.63f,.86f,.44f,-.25f,-.23f,-.05f,-.29f,.89f,-.53f,-.06f,.03f,.36f,-.83f,.79f,-.40f,-.10f,.90f,-.92f,-.49f,-.38f,-.43f,.02f,.98f,-.99f,.13f,-.75f,-.99f,-.37f,.52f,-.20f,-.56f,-.38f,.80f,-.45f,.13f,.71f,-.60f,-.46f,-.90f,.83f,-.01f,-.47f,.08f,-.52f,-.27f,-.83f,-.06f,-.26f,-.43f,.02f,.71f,.26f,-.39f,-.20f,-.12f,.95f,.15f,.71f,-.82f,-.91f,-.98f,.99f,.60f,-.20f,-.89f,-.33f,-.22f,.25f,.04f,.74f,-.92f,-.19f,-.23f,.84f,.34f,.83f,.41f,.77f,.24f,.32f,-.31f,-.36f,-.08f,.58f,-.45f,.63f,-.28f,.15f,-.19f,.58f,.67f,-.21f,-.79f,.23f,-.33f,-.05f,-.60f,-.09f,-.98f,.73f,-.36f,.18f,.70f,.86f,-.18f,-.36f,.06f,-.30f,-.48f,-.08f,.70f,.57f,.00f,.95f,.69f,-.70f,-.98f,-.30f,-.78f,-.63f,-.91f,.83f,.72f,.45f,-.50f,-.30f,-.41f,-.17f,-.39f,-.65f,-.97f,.25f,-.03f,-.38f,-.24f,.42f,.27f,-.18f,.50f,-.04f,-.33f,-.54f,.69f,-.97f,.45f,.03f,.78f,.83f,-.91f,-.37f,-.55f,-.28f,-.15f,-.19f,.38f,.11f,-.39f,-.63f,.84f,-.19f,.61f,-.36f,.34f,-.54f,-.68f,-.68f,.47f,-.08f,.79f,.21f,-.50f,-.18f,-.40f,.80f,.00f,-.40f,-.90f,-.52f,-.21f,-.10f,.99f,.97f,-.71f,-.98f,-.80f,.20f,.73f,.95f,-.47f,.52f,.98f,.01f,.16f,-.50f,.43f,.51f,.81f,.69f,-.34f,-.49f,-.92f,.68f,.79f,-.56f,-.46f,.69f,.51f,-.45f,-.30f,.95f,-.46f,-.88f,-.92f,-.69f,.91f,-.62f,-.58f,.70f,.51f,-.70f,-.87f,-.01f,-.64f,-.22f,-.81f,-.65f,-.55f,-.06f,.84f,.54f,.29f,-.85f,-.99f,.87f,.98f,.61f,.64f,.25f,.87f,.89f,.48f,-.25f,-.78f,-.77f,.49f,.53f,-.06f,-.93f,.21f,.92f,-.51f,.55f,.83f,.14f,.17f,.42f,-.09f,.52f,.95f,.46f,.81f,-.75f,.88f,.99f,-.53f,-.93f,-.46f,-.67f,.61f,-.02f,-.57f,-.58f,.09f,.29f,.27f,.73f,.90f,-.72f,-.68f,-.00f,.64f,.24f,.18f,.73f,.03f,-.06f,-.84f,-.07f,-.25f,-.61f,.71f,-.10f,-.27f,-.83f,.24f,-.11f,-.97f,-.99f,.13f,-.84f,-.51f,.85f,-.11f,-.67f,.45f,.95f,.76f,-.63f,.95f,-.68f,-.15f,.77f,-.44f,.10f,.11f,.76f,.56f,.48f,.35f,-.11f,-.16f,-.00f,.63f,-.01f,.51f,.13f,-.28f,-.01f,.58f,.89f,-.69f,.43f,-.90f,.28f,.86f,.93f,.22f,-.16f,.47f,.33f,-.56f,.12f,-.26f,-.72f,-.51f,.24f,-.19f,-.88f,-.70f,.45f,-.93f,-.61f,-.76f,-.49f,.27f,-.11f,-.06f,.10f,.04f,.62f,.51f,-.27f,-.19f,.90f,-.39f,.18f,-.10f,.84f,.53f,-.71f,.17f,.29f,.72f,-.80f,.27f,.57f,-.53f,-.95f,.88f,.84f,.09f,.11f,-.13f,-.64f,-.36f,-.48f,.82f,.08f,-.35f,.03f,.76f,.14f,.79f,-.94f,-.35f,.89f,-.25f,-.48f,.37f,.82f,-.30f,.57f,.94f,.47f,.17f,-.39f,-.62f,.65f,.37f,.41f,-.85f,.74f,-.59f,.44f,-.63f,.45f,.90f,.99f,-.73f,-.03f,.00f,.49f,-.29f,-.63f,.59f,.24f,.66f,-.05f,.06f,-.95f,.06f,.62f,-.83f,-.46f,-.25f,-.35f,-.91f,.07f,-.77f,.54f,-.82f,.82f,.68f,.49f,-.63f,-.36f,-.31f,-.48f,-.95f,-.55f,-.62f,.52f,.26f,-.39f,.45f,-.09f,-.07f,.56f,.32f,.37f,.57f,-.83f,-.90f,.03f,.47f,.97f,.26f,.37f,-.24f,.72f,.38f,-.73f,.86f,.88f,-.59f,.17f,-.80f,-.86f,.64f,.83f,-.15f,-.31f,-.87f,.92f,-.24f,.45f,.42f,-.61f,-.37f,.02f,.82f,.84f,.06f,.67f,-.12f,.11f,.94f,-.81f,.75f,-.04f,.72f,-.50f,.30f,.90f,.93f,-.05f,-.10f,-.02f,.51f,.03f,-.67f,.18f,.37f,.65f,-.20f,-.45f,.52f,.46f,.95f,-.78f,.20f,.54f,.92f,-.64f,.51f,.63f,-.58f,-.50f,-.40f,.82f,.15f,.15f,.21f,.01f,.89f,.10f,-.05f,-.92f,-.18f,.30f,.00f,-.12f,.71f,-.58f,.15f,.17f,-.31f,-.36f,.95f,-.21f,-.82f,-.31f,-.78f,-.17f,.47f,-.41f,.90f,.75f,.28f,.57f,.16f,-.73f,.66f,-.27f,.76f,-.74f,.35f,-.21f,.39f,-.70f,-.31f,-.14f,-.85f,.80f,.58f,-.43f,.02f,.01f,.31f,-.89f,-.06f,.04f,.28f,-.00f,.74f,.32f,.66f,-.98f,-.53f,-.40f,-.94f,.35f,.53f,-.57f,.60f,.86f,-.94f,-.67f,.95f,-.35f,-.79f,.56f,.10f,-.28f,-.53f,-.27f,.25f,.83f,.77f,.18f,-.33f,.49f,-.64f,.57f,.80f,.98f,.56f,.03f,.58f,.76f,.87f,.61f,.91f,-.30f,-.60f,-.76f,-.30f,.23f,.06f,.03f,.72f,.11f,-.23f,-.21f,-.20f,-.44f,.78f,.15f,.87f,-.31f,.43f,-.05f,-.16f,.04f,-.79f,.87f,.09f,.32f,-.58f,.65f,-.62f,.23f,.20f,-.29f,.88f,-.58f,-.03f,-.33f,-.48f,.27f,.47f,.81f,.91f,-.41f,.67f,-.94f,.31f,.25f,-.00f,.99f,-.06f,.22f,-.19f,.53f,-.62f,-.63f,.28f,-.49f,.37f,-.69f,.59f,-.30f,.57f,.52f,.55f,-.67f,-.15f,.71f,.11f,-.34f,-.06f,.10f,.70f,.98f,.52f,.16f,.87f,-.62f,-.58f,.57f,.65f,-.35f,-.31f,.96f,.21f,.31f,-.82f,.14f,.57f,.24f,-.26f,.77f,-.09f,.20f,.40f,.12f,-.17f,.65f,.64f,-.63f,-.75f,.50f,.31f,-.06f,-.19f,-.63f,.69f,-.54f,.27f,.95f,-.44f,.18f,-.91f,-.02f,.77f,-.66f,-.33f,-.20f,-.22f,.65f,.31f,-.00f,-.26f,.07f,.05f,-.47f,-.52f,-.06f,.49f,.65f,.54f,.10f,-.10f,-.56f,-.87f,.35f,-.51f,.90f,-.04f,-.90f,-.16f,-.75f,-.06f,-.04f,-.11f,.44f,.73f,.72f,.80f,.73f,-.95f,.83f,.72f,.46f,-.94f,-.56f,.00f,-.41f,-.66f,.56f,.92f,-.23f,-.32f,-.16f,-.04f,-.37f,-.81f,-.29f,-.79f,-.45f,-.37f,.65f,-.07f,-.97f,-.93f,.15f,.01f,-.90f,.05f,-.63f,.43f,-.67f,.04f,.86f,-.64f,.80f,.66f,-.02f,.84f,-.46f,.19f,-.42f,-.20f,.86f,.12f,.32f,-.72f,-.40f,-.32f,-.26f,.85f,-.40f,.15f,-.69f,-.77f,-.48f,-.10f,-.16f,.61f,-.12f,-.76f,.21f,-.15f,-.15f,.00f,-.31f,.74f,-.12f,.75f,-.31f,.89f,.75f,-.17f,-.58f,-.33f,-.72f,-.43f,-.36f,-.74f,-.17f,-.61f,-.53f,.62f,-.24f,-.37f,-.16f,-.74f,.39f,-.41f,.57f,-.64f,-.51f,.65f,.40f,.65f,-.04f,-.08f,.30f,-.86f,.37f,-.49f,.24f,.36f,-.66f,.53f,-.42f,.47f,-.10f,-.41f,-.92f,-.88f,.90f,.59f,.00f,.70f,-.55f,.45f,.81f,.42f,-.81f,-.90f,-.66f,.47f,-.49f,-.21f,-.71f,-.62f,.20f,-.90f,-.49f,-.83f,-.94f,.16f,.04f,.84f,-.49f,-.23f,-.75f,-.13f,.91f,.36f,-.51f,.56f,-.26f,-.34f,-.27f,.13f,.45f,-.89f,.69f,-.42f,.49f,-.93f,.46f,.68f,.63f,-.86f,-.26f,.14f,-.29f,.80f,-.74f,.73f,.19f,-.26f,.01f,-.71f,-.54f,-.64f,-.59f,-.37f,.76f,.15f,.90f,-.28f,.59f,.16f,-.55f,.48f,-.49f,-.43f,.56f,-.30f,.12f,.08f,-.28f,.78f,.73f,-.06f,.13f,.31f,-.30f,.90f,.39f,-.77f,-.89f,.91f,-.09f,.76f,.30f,-.80f,.31f,-.94f,.88f,-.38f,-.10f,.02f,-.93f,-.27f,.83f,-.33f,-.55f,.04f,-.31f,-.21f,-.38f,-.04f,-.21f,-.58f,.57f,-.50f,.45f,-.88f,-.95f,.66f,-.15f,-.65f,.04f,-.91f,-.02f,-.40f,.60f,.17f,.07f,-.11f,-.55f,.36f,.47f,.09f,.37f,.75f,-.98f,-.39f,.79f,.61f,-.75f,.51f,-.49f,-.99f,-.72f,-.56f,.97f,-.42f,.06f,-.14f,-.96f,-.28f,-.48f,.99f,.64f,-.12f,.63f,-.95f,-.09f,-.81f,.83f,.99f,-.01f,.17f,.03f,-.52f,.20f,.77f,-.42f,.96f,.79f,-.64f,-.29f,-.84f,-.79f,.83f,-.88f,-.24f,.22f,.55f,.16f,-.85f,.49f,.61f,-.79f,-.75f,.27f,.36f,.36f,-.98f,-.34f,.13f,.38f,-.25f,.74f,.08f,.46f,.13f,.32f,.78f,.44f,.11f,-.57f,-.34f,-.92f,-.03f,-.74f,-.06f,.30f,-.51f,.06f,.74f,.13f,.34f,.05f,.81f,.43f,.80f,-.77f,-.54f,.59f,-.05f,.14f,.29f,-.70f,-.12f,.03f,-.04f,-.07f,-.84f,.86f,.87f,-.24f,.02f,.71f,.31f,-.62f,.62f,-.65f,.26f,.47f,.57f,.84f,.32f,.54f,-.50f,.93f,-.91f,.35f,-.74f,-.56f,-.46f,-.78f,-.60f,-.89f,-.88f,-.16f,-.83f,-.63f,.59f,.85f,.32f,.74f,.84f,-.83f,-.46f,-.77f,-.74f,.81f,-.95f,.99f,-.87f,-.28f,.66f,-.58f,.20f,-.17f,-.95f,.23f,.86f,-.81f,.93f,.38f,.49f,.68f,-.91f,-.58f,-.41f,.31f,.23f,.92f,.71f,.33f,.18f,-.33f,.64f,.75f,-.72f,.70f,-.49f,-.49f,-.18f,-.59f,-.23f,.71f,.42f,-.53f,-.83f,-.04f,-.75f,-.77f,-.59f,.53f,-.30f,.22f,-.78f,.64f,-.64f,-.37f,.79f,.72f,-.06f,.94f,.93f,-.75f,.90f,-.79f,.96f,.28f,-.74f,.70f,-.81f,.18f,-.96f,.84f,-.18f,.05f,-.23f,.26f,-.85f,-.12f,.50f,.12f,-.94f,.64f,.86f,.81f,.89f,-.62f,.74f,-.19f,-.78f,.67f,-.04f,.36f,.62f,-.02f,-.49f,.72f,-.02f,.42f,-.78f,.79f,.26f,-.63f,-.31f,-.38f,-.37f,-.43f,-.97f,.27f,-.34f,-.59f,-.07f,.44f,-.62f,-.64f,-.09f,-.70f,-.11f,-.89f,-.35f,.49f,-.73f,-.89f,.76f,.90f,.04f,.44f,.58f,.18f,-.06f,.68f,.14f,-.48f,-.15f,-.53f,-.31f,.37f,.71f,-.10f,-.83f,-.88f,-.26f,-.88f,.82f,-.04f,.84f,-.92f,-.10f,.00f,.18f,.21f,.74f,-.25f,.88f,-.75f,-.47f,.72f,-.54f,.04f,.10f,.29f,.20f,.08f,-.17f,.71f,-.46f,.96f,-.21f,.10f,.58f,.65f,.06f,.58f,.68f,.27f,-.07f,-.05f,.97f,.25f,-.42f,-.59f,.42f,.87f,-.06f,-.61f,.06f,.84f,.58f,-.68f,.65f,.40f,-.32f,-.38f,.93f,.58f,-.65f,.73f,.96f,-.97f,.08f,.80f,.39f,-.21f,-.47f,.56f,-.17f,-.46f,-.52f,-.27f,.76f,.60f,-.41f,-.73f,.93f,-.06f,.30f,-.64f,-.02f,-.23f,.08f,.72f,-.58f,-.29f,-.49f,.85f,-.42f,.40f,-.37f,-.74f,.51f,-.52f,-.52f,-.63f,-.66f,.81f,-.39f,.33f,-.46f,-.76f,-.06f,.45f,-.05f,-.64f,.42f,.76f,-.55f,-.98f,.78f,-.82f,-.25f,.24f,.35f,.68f,-.68f,.28f,.60f,-.50f,.20f,.15f,-.92f,.09f,-.99f,.49f,.24f,.69f,-.50f,.47f,.94f,-.30f,.96f,-.45f,-.57f,.25f,.77f,.82f,-.88f,-.33f,-.32f,-.65f,-.85f,.83f,.06f,.71f,.53f,-.42f,.25f,-.50f,-.02f,.76f,-.13f,.39f,-.99f,-.20f,.20f,-.80f,-.59f,-.59f,.13f,.57f,.55f,.44f,-.11f,.30f,-.16f,.78f,-.91f,.02f,.68f,.59f,-.28f,-.79f,.48f,-.17f,-.86f,-.68f,.96f,-.61f,-.69f,.95f,.81f,-.89f,.02f,.97f,-.13f,-.10f,.07f,.67f,.30f,-.26f,-.07f,-.55f,-.58f,-.81f,.06f,.65f,.97f,-.29f,.85f,-.49f,-.01f,-.90f,.23f,-.39f,-.71f,.28f,.38f,-.26f,.19f,-.94f,.38f,-.80f,.15f,-.59f,.18f,.87f,-.54f,-.65f,.47f,.74f,.43f,.30f,.99f,-.66f,.89f,-.72f,.25f,.73f,-.26f,-.60f,.10f,.55f,-.86f,-.96f,-.54f,-.81f,.84f,.67f,.34f,.98f,-.50f,-.64f,-.40f,-.61f,-.72f,-.68f,.80f,-.15f,-.99f,-.79f,.71f,-.66f,-.11f,.06f,.24f,-.58f,.60f,-.21f,.24f,-.75f,-.12f,-.91f,-.67f,-.19f,-.47f,-.87f,.40f,.42f,.69f,-.61f,-.98f,-.63f,.97f,.05f,-.19f,-.47f,.41f,-.45f,.94f,.85f,-.10f,.09f,.00f,.20f,.49f,-.81f,.31f,.94f,-.41f,-.71f,.99f,.82f,-.34f,-.08f,.85f,.61f,-.93f,-.54f,.37f,-.90f,-.67f,-.02f,-.43f,-.52f,-.58f,-.61f,.81f,.53f,.86f,-.09f,.26f,.23f,-.76f,-.58f,-.90f,.33f,.82f,-.44f,.18f,-.18f,.72f,.74f,-.84f,.83f,-.26f,-.32f,-.99f,-.07f,.57f,.71f,-.64f,.56f,-.08f,-.78f,.83f,-.60f,-.54f,-.65f,.26f,.67f,.10f,.72f,-.12f,.71f,.90f,-.37f,-.55f,-.42f,-.89f,.87f,.70f,.47f,.07f,.46f,.48f,.53f,-.95f,.82f,.35f,.48f,.99f,-.64f,.45f,-.33f,.94f,.93f,.55f,.95f,.23f,.71f,.42f,.48f,-.45f,.54f,.38f,.58f,.94f,.82f,.65f,-.51f,.53f,-.47f,-.78f,-.51f,.67f,.72f,-.20f,.09f,.41f,.69f,.21f,.95f,-.26f,.23f,-.04f,-.30f,-.18f,-.78f,.06f,.04f,.83f,-.97f,-.55f,-.86f,-.99f,-.12f,.17f,-.50f,-.77f,-.20f,-.47f,-.95f,.05f,-.37f,.88f,.09f,-.54f,-.64f,-.72f,.57f,.16f,-.31f,-.28f,-.36f,.71f,-.82f,.23f,.88f,.69f,.99f,-.99f,.94f,.24f,-.45f,.61f,.85f,-.92f,.96f,.98f,.34f,.03f,-.25f,.15f,-.71f,.48f,-.02f,-.29f,.34f,.48f,-.14f,-.46f,-.32f,.67f,.04f,-.90f,-.82f,.44f,-.46f,-.33f,.63f,-.46f,.16f,.02f,.57f,-.02f,.50f,.26f,.51f,-.15f,.25f,.07f,-.59f,-.87f,.64f,-.92f,.05f,.79f,.14f,-.34f,-.77f,.72f,.53f,-.42f,.95f,-.86f,-.35f,.90f,.36f,.25f,.63f,.59f,-.32f,-.95f,-.98f,-.84f,-.56f,.65f,-.81f,-.06f,-.45f,-.60f,-.84f,.09f,.77f,.17f,.11f,-.57f,-.67f,-.23f,-.98f,.12f,.32f,.35f,.37f,-.81f,-.07f,-.21f,.72f,-.35f,-.02f,.33f,.58f,-.52f,.17f,-.49f,.30f,-.30f,-.78f,-.38f,.25f,.14f,-.20f,.75f,-.51f,.46f,.15f,-.07f,.19f,-.47f,.70f,-.47f,-.03f,.14f,-.25f,.67f,.83f,.22f,-.80f,.15f,.88f,.42f,.76f,-.53f,.85f,.89f,.92f,-.73f,-.88f,.05f,.09f,.19f,.55f,.74f,-.08f,.41f,.06f,-.35f,.66f,.05f,.36f,.28f,.77f,.65f,.94f,-.28f,-.31f,.47f,-.62f,-.99f,.90f,.70f,.88f,-.68f,.69f,-.23f,-.01f,.97f,.87f,.55f,.07f,.59f,-.16f,-.05f,.28f,-.48f,.56f,-.64f,.22f,.55f,-.66f,.29f,.12f,.73f,-.98f,-.13f,-.39f,.33f,.95f,.38f,-.34f,.73f,.25f,.63f,.13f,-.18f,-.23f,-.68f,.39f,.65f,-.08f,-.96f,.35f,-.06f,.81f,-.34f,-.91f,.96f,-.75f,.61f,.81f,-.21f,.21f,-.89f,-.13f,-.92f,-.11f,-.25f,.28f,.31f,.83f,.96f,-.13f,.54f,.95f,.16f,.49f,.12f,.13f,.20f,.37f,.34f,.87f,-.11f,-.28f,-.05f,.72f,-.27f,.44f,.84f,-.03f,.69f,-.79f,.63f,.81f,.88f,.98f,.51f,-.20f,.54f,.73f,-.95f,-.61f,.33f,.26f,.51f,.46f,.43f,-.04f,-.29f,.77f,.34f,-.43f,.48f,.42f,-.14f,.89f,-.93f,-.40f,.77f,-.04f,.85f,.98f,-.26f,.05f,-.24f,-.22f,-.50f,-.82f,-.42f,.63f,.79f,-.84f,-.60f,.20f,-.41f,.86f,.51f,-.17f,-.20f,.94f,-.89f,-.51f,-.35f,.73f,-.53f,.09f,.49f,-.72f,-.47f,-.72f,-.58f,-.28f,.05f,-.08f,-.86f,-.69f,-.22f,-.32f,.17f,-.58f,-.49f,-.62f,.19f,-.28f,-.40f,-.61f,.75f,-.87f,-.96f,-.47f,-.04f,.64f,.80f,.90f,-.57f,.40f,.34f,-.31f,-.94f,.79f,.12f,.55f,.62f,.44f,.58f,-.10f,-.07f,.35f,.73f,.42f,-.05f,-.16f,.89f,-.05f,-.22f,-.25f,-.81f,-.57f,.78f,.49f,.06f,.87f,.61f,-.01f,.77f,.63f,-.15f,-.17f,-.42f,-.59f,.64f,.85f,-.70f,.03f,-.58f,.25f,.31f,.72f,.26f,-.83f,-.37f,-.10f,-.30f,-.75f,-.54f,.13f,.10f,.75f,-.66f,-.73f,-.41f,.15f,.51f,-.72f,.76f,.98f,-.41f,.61f,-.67f,-.82f,-.90f,.57f,.57f,.82f,.50f,.74f,-.06f,-.19f,-.49f,.71f,.82f,.19f,-.04f,.55f,.03f,.59f,-.65f,.99f,.74f,.81f,-.59f,-.60f,.73f,.47f,.73f,.72f,-.33f,.29f,.66f,.72f,-.21f,-.90f,.69f,.40f,.67f,.07f,-.39f,.34f,.94f,.56f,.61f,.24f,-.76f,.53f,-.15f,-.70f,.93f,.42f,-.12f,-.50f,.67f,-.55f,.73f,.46f,-.74f,.90f,-.68f,.57f,-.62f,-.19f,-.57f,.37f,.81f,-.84f,.73f,.20f,-.89f,-.55f,.01f,-.12f,.24f,-.14f,-.35f,.99f,.40f,.11f,-.83f,-.25f,.84f,.89f,-.51f,.75f,.44f,.45f,-.18f,-.70f,.83f,.02f,-.84f,.46f,-.83f,-.02f,.10f,.44f,.53f,-.96f,.70f,.55f,.37f,.84f,-.64f,-.06f,.94f,-.58f,-.83f,.92f,.11f,.34f,-.67f,-.42f,-.27f,-.55f,-.97f,-.92f,-.81f,-.21f,-.40f,.73f,.49f,.60f,-.56f,.95f,-.09f,-.20f,-.88f,-.09f,-.70f,-.30f,-.85f,-.39f,.94f,.06f,.06f,-.09f,.05f,.61f,.51f,.60f,-.78f,-.89f,.92f,-.03f,-.03f,.41f,-.10f,.97f,.67f,.92f,.70f,-.22f,.63f,.77f,-.46f,.25f,.17f,.00f,-.14f,-.87f,.49f,.99f,.42f,-.67f,.61f,.34f,.25f,-.71f,-.18f,.24f,-.27f,.24f,-.06f,-.05f,-.08f,.70f,-.19f,-.29f,.41f,.78f,.08f,-.88f,.39f,.33f,-.50f,.00f,-.35f,.56f,-.85f,.06f,-.02f,-.91f,-.58f,.18f,.66f,.46f,.96f,.90f,.74f,-.17f,-.80f,-.38f,-.01f,.25f,-.53f,.21f,.26f,-.63f,-.78f,.32f,.42f,-.60f,-.31f,-.93f,.36f,.41f,.47f,-.41f,.03f,.37f,-.93f,-.75f,-.10f,-.44f,.74f,.96f,.47f,.69f,.44f,.07f,.28f,-.84f,-.00f,-.65f,.23f,-.17f,.65f,-.05f,-.32f,-.98f,.67f,-.45f,.16f,.26f,.81f,.81f,-.72f,-.23f,.57f,-.87f,.76f,-.57f,.14f,-.44f,.31f,.80f,-.73f,.83f,.00f,-.54f,.10f,.42f,.54f,-.18f,.56f,.35f,.44f,-.52f,.69f,.69f,.94f,.39f,.96f,.58f,-.71f,.98f,-.92f,-.11f,.98f,.90f,.79f,.63f,.84f,-.95f,.61f,-.52f,-.10f,.33f,.10f,.56f,.74f,.11f,-.27f,-.36f,.88f,.33f,-.30f,.67f,.98f,.19f,.56f,.85f,.13f,-.65f,-.67f,.86f,-.03f,.83f,-.50f,-.54f,-.65f,-.64f,-.73f,-.60f,.68f,-.71f,-.36f,-.81f,-.27f,-.06f,-.04f,.47f,-.99f,-.35f,-.89f,.55f,-.39f,-.94f,.32f,-.23f,-.53f,.37f,-.17f,.85f,.27f,-.10f,-.81f,-.31f,-.22f,-.30f,-.29f,.93f,.08f,.98f,.32f,.97f,.86f,.67f,.39f,.50f,.06f,.25f,-.73f,-.88f,-.02f,.88f,.85f,.38f,-.29f,-.99f,.30f,-.06f,.14f,-.02f,.09f,-.62f,.41f,-.46f,.90f,.06f,-.78f,-.10f,.97f,.66f,-.96f,.46f,.12f,.84f,.89f,-.28f,-.65f,-.71f,.04f,.34f,-.98f,-.27f,.21f,.13f,-.56f,.56f,.71f,-.55f,.14f,.81f,.03f,-.19f,.82f,.48f,-.42f,.73f,.04f,.48f,-.10f,-.78f,.75f,-.89f,-.02f,.60f,.15f,-.88f,.01f,.93f,-.59f,.43f,.57f,.14f,.96f,.75f,.58f,.69f,-.92f,.14f,.94f,-.42f,.54f,-.73f,.91f,-.89f,-.95f,.41f,.26f,-.46f,-.64f,-.65f,.53f,-.61f,-.95f,-.41f,-.28f,-.78f,-.01f,-.55f,.69f,.60f,-.29f,.81f,.74f,.39f,.58f,-.65f,.37f,-.88f,-.11f,.08f,-.78f,.01f,-.46f,-.12f,.38f,-.63f,.66f,-.33f,-.51f,-.16f,.32f,.40f,-.03f,-.35f,.49f,-.52f,-.26f,-.99f,-.94f,.18f,.94f,.37f,.94f,.85f,.36f,-.00f,.36f,-.94f,-.47f,.92f,.75f,-.29f,-.82f,.95f,.29f,-.47f,.12f,-.96f,.06f,-.56f,-.20f,.47f,.15f,.08f,-.20f,-.20f,-.31f,.44f,-.83f,-.03f,.75f,-.87f,.90f,-.39f,-.06f,.56f,.03f,-.03f,-.37f,-.08f,-.25f,.15f,-.90f,.29f,-.94f,.20f,.71f,.14f,-.70f,.98f,-.77f,-.97f,-.43f,.25f,-.30f,.28f,-.04f,.19f,.40f,.56f,-.18f,-.66f,.66f,-.21f,-.48f,.76f,.87f,-.63f,-.12f,.22f,-.48f,-.41f,-.87f,-.16f,-.53f,.25f,-.31f,.45f,-.25f,.46f,.35f,.64f,-.45f,.84f,-.70f,.19f,-.96f,-.68f,.92f,-.07f,.10f,.89f,.25f,.70f,.52f,.63f,-.36f,.11f,-.39f,.77f,.59f,-.01f,-.60f,.66f,-.46f,.64f,-.21f,.37f,-.55f,.27f,.09f,-.35f,.07f,.80f,-.51f,-.80f,-.04f,.59f,.59f,-.96f,-.12f,-.28f,.71f,.17f,-.36f,.35f,.99f,-.30f,-.98f,.52f,-.62f,.69f,-.48f,.34f,.30f,-.78f,-.84f,.46f,-.36f,.31f,-.61f,-.71f,.46f,-.97f,-.97f,-.18f,-.42f,-.22f,-.42f,-.08f,.84f,-.48f,-.22f,-.95f,-.42f,-.45f,.17f,-.58f,.82f,-.81f,.23f,.07f,-.16f,.93f,-.82f,-.78f,.13f,-.80f,-.24f,-.00f,.75f,-.59f,.01f,.71f,.53f,.02f,-.97f,.81f,-.35f,.15f,-.32f,-.11f,.79f,-.08f,.82f,-.99f,-.68f,-.48f,-.08f,-.03f,.64f,.48f,-.69f,-.82f,.98f,.86f,-.32f,-.86f,-.08f,.25f,-.37f,-.56f,.04f,-.53f,.52f,.89f,-.26f,.79f,.97f,.46f,.07f,-.99f,-.91f,-.03f,.81f,.06f,-.56f,-.68f,.14f,-.15f,.33f,-.26f,.94f,.76f,.70f,.70f,-.88f,.20f,.02f,-.32f,.75f,.34f,.47f,.63f,.01f,-.80f,.31f,.01f,-.25f,.20f,.40f,-.95f,.52f,.24f,-.46f,.61f,.58f,-.97f,-.90f,-.24f,.18f,.16f,.56f,.21f,-.71f,-.96f,-.86f,.04f,-.11f,-.09f,.07f,-.01f,-.58f,-.12f,-.43f,-.95f,.45f,-.87f,-.07f,.31f,.02f,-.22f,-.62f,.48f,.19f,.42f,-.23f,.70f,-.56f,-.19f,-.54f,-.15f,.78f,-.39f,-.95f,.36f,.12f,-.31f,.93f,.16f,.92f,.58f,-.80f,.76f,-.16f,.15f,.93f,-.18f,.11f,.97f,.36f,-.29f,.56f,.67f,-.46f,-.94f,.64f,-.87f,-.72f,-.86f,.64f,.34f,-.00f,.03f,-.48f,-.11f,.09f,-.89f,-.85f,.71f,-.36f,-.28f,.04f,-.88f,.91f,.19f,.37f,-.58f,.29f,.17f,.33f,-.62f,-.32f,.83f,.00f,-.95f,-.03f,-.80f,.64f,.10f,-.11f,.46f,.54f,.64f,-.17f,-.31f,-.46f,.85f,-.42f,-.27f,.45f,.06f,-.11f,-.20f,-.30f,.29f,.73f,.43f,-.95f,.46f,-.04f,-.51f,.30f,.82f,-.93f,-.86f,.33f,-.39f,-.07f,-.67f,-.86f,-.61f,.48f,-.35f,-.57f,.31f,.05f,-.72f,.46f,-.62f,-.81f,-.74f,.59f,.79f,.77f,-.31f,-.38f,-.85f,.70f,.39f,.57f,.52f,-.08f,.49f,-.49f,-.53f,.61f,-.21f,-.04f,.03f,.54f,-.49f,.95f,-.45f,-.99f,-.39f,.19f,-.74f,-.43f,.97f,-.46f,-.93f,-.56f,.17f,-.74f,-.87f,-.20f,-.87f,-.60f,-.66f,.12f,-.88f,-.25f,-.27f,.36f,-.22f,.97f,-.24f,.35f,-.30f,-.10f,.66f,-.06f,-.21f,-.20f,.87f,-.13f,-.25f,.10f,-.47f,-.51f,.55f,.20f,.14f,.80f,-.25f,.81f,-.19f,-.04f,-.05f,.63f,-.03f,-.60f,.39f,-.25f,.47f,-.51f,-.42f,-.31f,-.08f,.76f,-.64f,.28f,-.15f,.89f,-.39f,.49f,-.31f,.39f,-.48f,-.24f,-.97f,.76f,-.34f,-.35f,-.83f,-.70f,-.92f,.37f,-.05f,.95f,.49f,-.04f,.51f,-.84f,.56f,.50f,.85f,.78f,-.03f,-.32f,.31f,-.13f,-.38f,-.34f,-.15f,.22f,-.91f,.97f,.12f,.71f,.22f,.98f,.45f,-.21f,-.27f,.31f,-.47f,-.90f,-.90f,-.83f,.07f,.20f,.68f,-.42f,-.36f,-.14f,.29f,.57f,.76f,-.18f,-.89f,-.30f,.92f,-.09f,-.05f,.96f,.29f,.07f,-.65f,-.78f,.55f,.17f,-.39f,.17f,.62f,.51f,.00f,-.61f,-.52f,-.57f,-.99f,-.65f,.94f,-.54f,-.00f,.99f,.40f,-.98f,.38f,-.15f,-.91f,-.47f,.61f,-.67f,.79f,-.01f,-.04f,.27f,.27f,.49f,.71f,.56f,-.98f,.42f,-.30f,-.48f,.85f,-.28f,.02f,-.26f,.27f,.40f,-.61f,-.48f,.79f,.08f,-.04f,.96f,.86f,.99f,-.97f,.79f,-.96f,.53f,.61f,-.16f,.39f,.02f,.76f,-.51f,.19f,.79f,.20f,.76f,.26f,-.95f,.03f,.06f,.86f,-.69f,.46f,-.55f,-.71f,-.91f,.69f,.53f,-.27f,-.02f,-.34f,-.78f,.51f,.04f,.92f,-.72f,-.84f,.01f,-.95f,.58f,-.85f,.86f,-.70f,.34f,.14f,-.22f,.89f,.44f,.91f,.51f,.15f,.56f,.75f,-.10f,.34f,.14f,-.34f,-.55f,.77f,.59f,.11f,-.16f,-.12f,-.89f,-.45f,-.56f,-.93f,.51f,-.19f,.82f,.17f,.65f,.18f,.69f,-.72f,.75f,.40f,-.37f,-.55f,.87f,.58f,.33f,-.78f,.29f,-.17f,.62f,-.05f,.39f,.69f,.90f,-.53f,.98f,.74f,-.34f,-.34f,-.25f,.62f,.96f,-.30f,-.54f,.46f,-.99f,.13f,-.40f,.15f,-.59f,-.85f,.18f,.89f,-.85f,.56f,-.23f,.73f,.17f,-.76f,-.23f,.77f,.82f,-.72f,-.94f,-.95f,.28f,.74f,-.97f,.44f,-.02f,.40f,-.53f,-.83f,-.57f,-.07f,.20f,.27f,.10f,-.19f,.99f,.18f,-.24f,.73f,-.30f,-.83f,-.34f,-.16f,-.23f,.18f,-.69f,-.70f,.52f,-.33f,.45f,-.81f,-.17f,.03f,.49f,.80f,.88f,-.30f,.17f,-.69f,.19f,.67f,-.78f,-.84f,-.75f,-.81f,.65f,.50f,.62f,.19f,-.91f,.26f,-.17f,-.33f,.19f,.13f,.32f,.59f,-.31f,.88f,-.33f,-.42f,-.34f,-.05f,-.35f,.20f,.31f,-.21f,-.30f,.70f,.10f,-.28f,-.27f,.48f,.45f,-.67f,-.14f,-.41f,-.80f,.02f,.03f,-.49f,-.73f,-.94f,-.28f,-.11f,.94f,-.02f,.06f,-.64f,-.87f,.59f,-.99f,-.19f,.50f,-.12f,.84f,.98f,-.69f,.64f,.93f,-.93f,-.42f,-.96f,-.26f,.21f,.48f,-.64f,.58f,-.86f,-.85f,-.73f,-.81f,.08f,-.89f,.00f,-.16f,.78f,.25f,-.87f,-.30f,-.19f,.58f,.30f,-.10f,.54f,.06f,-.52f,-.88f,.63f,-.34f,-.19f,-.92f,.60f,-.90f,-.21f,-.90f,-.28f,-.14f,-.71f,.49f,.80f,-.49f,-.38f,.57f,.10f,-.80f,.25f,-.38f,-.12f,-.52f,-.01f,-.72f,.74f,-.35f,.88f,.02f,-.14f,-.40f,-.42f,.12f,.93f,-.98f,.95f,-.34f,.53f,.00f,.22f,.35f,.90f,-.39f,-.04f,-.27f,-.20f,.90f,-.00f,.36f,-.68f,-.95f,.21f,.76f,.29f,.99f,-.28f,.02f,-.79f,-.39f,.63f,-.25f,.64f,.72f,-.93f,-.64f,.34f,.36f,.76f,.77f,.27f,.63f,.28f,.94f,.80f,.45f,.49f,.09f,.22f,.50f,.44f,.94f,-.85f,.32f,.34f,.02f,-.04f,-.50f,-.54f,-.96f,.77f,-.09f,-.20f,-.24f,-.78f,.36f,-.73f,.87f,.23f,-.99f,.72f,-.56f,.79f,.14f,-.48f,.37f,.69f,.91f,-.58f,.37f,-.86f,-.76f,-.15f,-.60f,-.16f,.13f,-.97f,-.08f,.84f,-.34f,-.46f,.23f,.61f,-.71f,.71f,-.07f,-.72f,.17f,-.25f,.58f,.10f,.90f,-.80f,.26f,.21f,.48f,-.24f,-.28f,.27f,-.80f,.52f,-.76f,.87f,-.71f,.20f,-.14f,.17f,.87f,-.37f,-.87f,.62f,.50f,.89f,-.66f,.43f,-.85f,.70f,.73f,-.69f,-.95f,-.85f,-.43f,.94f,-.15f,-.69f,.23f,-.29f,.76f,-.62f,.05f,-.59f,.59f,.47f,.27f,-.13f,.47f,-.18f,-.20f,.02f,-.43f,.60f,.11f,-.26f,-.41f,.82f,.66f,-.31f,-.32f,.58f,.86f,.49f,-.90f,.40f,.35f,-.16f,.48f,-.46f,-.02f,-.07f,.32f,-.98f,-.33f,.33f,.22f,-.26f,.09f,-.79f,.06f,.54f,-.87f,.02f,.83f,-.33f,.10f,.50f,-.44f,-.45f,-.97f,-.51f,-.90f,-.56f,.79f,-.48f,.41f,.72f,-.23f,.91f,-.07f,-.38f,.77f,-.55f,-.81f,-.55f,.75f,.66f,-.47f,.44f,-.90f,-.06f,.68f,-.31f,-.25f,-.00f,.47f,-.12f,.12f,.99f,-.62f,-.84f,.17f,.92f,.38f,-.19f,-.97f,.48f,-.89f,.30f,-.08f,.03f,.42f,.14f,.36f,-.26f,-.09f,.41f,.76f,.66f,-.11f,-.19f,.22f,-.11f,.39f,.68f,-.81f,.87f,.04f,.54f,.02f,.05f,-.24f,.05f,.57f,-.81f,-.77f,-.09f,-.35f,-.00f,-.61f,.03f,-.48f,.40f,-.22f,-.19f,-.55f,-.53f,-.32f,-.17f,-.08f,.57f,.18f,.41f,.54f,.21f,-.25f,.07f,.76f,-.97f,-.79f,-.03f,-.56f,-.50f,-.55f,-.58f,.59f,.82f,-.18f,-.50f,-.32f,-.83f,-.44f,-.94f,-.70f,.90f,-.23f,-.23f,-.83f,.35f,-.45f,.31f,.52f,-.14f,.79f,-.30f,-.51f,.99f,.47f,.00f,-.42f,.43f,-.52f,.68f,-.11f,.54f,.98f,.66f,-.32f,.13f,-.87f,-.20f,.27f,-.74f,.64f,-.08f,-.68f,-.78f,.45f,-.37f,.23f,.79f,-.03f,-.09f,.27f,-.46f,-.22f,-.39f,.92f,.80f,-.83f,.89f,.42f,.75f,-.59f,-.26f,-.62f,-.37f,-.07f,.97f,-.90f,.39f,-.50f,-.20f,-.54f,.34f,-.43f,-.82f,-.35f,.42f,-.46f,.48f,-.69f,.33f,.19f,.99f,.06f,-.30f,-.99f,.13f,.47f,-.23f,-.47f,.07f,.82f,-.58f,.42f,-.83f,-.19f,-.98f,-.80f,.28f,.15f,.44f,-.87f,.99f,-.46f,-.85f,-.07f,-.46f,-.93f,.87f,.49f,-.09f,-.74f,-.43f,-.94f,.44f,.50f,-.84f,-.51f,.50f,-.33f,.58f,.22f,-.89f,-.65f,.39f,.01f,.94f,-.73f,-.30f,.69f,.47f,.11f,-.30f,-.16f,-.58f,.76f,-.78f,-.84f,.83f,.96f,-.44f,-.40f,-.28f,.76f,-.47f,-.00f,.91f,.12f,.58f,.64f,.61f,.30f,-.55f,-.33f,.40f,.17f,.35f,.93f,-.63f,.33f,.21f,-.03f,-.55f,.37f,.55f,-.14f,-.37f,-.32f,-.41f,-.50f,-.30f,.49f,-.16f,-.13f,-.77f,-.82f,-.16f,.67f,-.83f,-.87f,.17f,.39f,-.72f,-.09f,.28f,.43f,.44f,-.28f,.23f,.33f,-.95f,.94f,-.70f,.03f,-.73f,-.06f,.90f,-.13f,.39f,-.57f,.13f,-.00f,.30f,-.83f,.53f,-.61f,.27f,.84f,.98f,.98f,-.34f,-.83f,-.36f,.16f,-.93f,-.45f,.23f,.22f,.18f,.80f,-.56f,-.34f,-.40f,-.84f,-.63f,.98f,-.07f,-.36f,.52f,.06f,-.61f,-.19f,.10f,.60f,.37f,-.74f,.84f,-.86f,.63f,.75f,-.48f,.93f,-.39f,-.01f,.43f,.24f,-.49f,-.36f,-.97f,-.56f,-.08f,.61f,.38f,-.25f,.81f,.42f,.42f,.95f,.18f,-.14f,.12f,-.32f,-.62f,.84f,-.92f,-.57f,.57f,-.18f,-.70f,.39f,.48f,.37f,-.34f,-.65f,-.42f,-.81f,.50f,-.27f,-.63f,.09f,-.97f,.76f,-.86f,-.93f,-.67f,-.86f,-.69f,.15f,-.21f,.20f,-.52f,-.65f,.80f,-.69f,-.78f,.23f,-.69f,-.84f,.09f,.53f,-.24f,.44f,.16f,-.10f,.46f,-.63f,-.50f,-.08f,.33f,.76f,-.37f,-.16f,.81f,.44f,.02f,.33f,-.73f,.72f,-.08f,-.47f,.59f,.76f,-.83f,.09f,-.90f,-.91f,-.53f,.80f,-.51f,.33f,-.78f,.47f,-.71f,-.58f,.22f,.53f,-.10f,-.00f,.10f,-.71f,.94f,.18f,-.67f,-.10f,-.95f,.52f,.87f,-.45f,.68f,-.00f,-.78f,-.01f,-.04f,.39f,.02f,.77f,.82f,-.98f,-.90f,-.63f,-.31f,-.97f,.29f,-.00f,.55f,-.58f,.97f,-.35f,.16f,-.56f,.06f,.97f,.54f,.80f,-.59f,.87f,.74f,-.31f,-.24f,.39f,.43f,.33f,-.12f,-.30f,.61f,.77f,.23f,-.05f,-.13f,.14f,.76f,-.12f,.53f,-.56f,-.75f,-.84f,-.97f,.33f,.65f,.16f,-.46f,.28f,-.39f,.40f,.03f,-.36f,.97f,-.27f,-.31f,-.34f,.68f,-.15f,-.09f,.84f,.29f,.90f,-.42f,-.82f,.89f,.15f,.20f,-.32f,-.03f,.29f,.61f,-.52f,-.62f,-.93f,.79f,.28f,-.03f,.03f,-.54f,-.61f,.31f,-.91f,.92f,-.18f,-.12f,.79f,-.24f,-.40f,.46f,-.63f,-.93f,.82f,.40f,-.09f,.12f,-.77f,-.13f,-.58f,.86f,-.86f,.50f,-.72f,-.46f,-.81f,.94f,-.66f,.24f,-.78f,-.79f,-.48f,.49f,.47f,-.04f,-.83f,-.49f,-.48f,-.16f,-.18f,.03f,-.90f,-.47f,-.26f,.38f,.90f,-.81f,-.64f,-.29f,.69f,.62f,-.99f,.50f,.94f,.68f,-.00f,.51f,.26f,-.78f,-.71f,.63f,-.81f,-.18f,-.30f,.28f,.18f,-.20f,-.44f,.26f,-.13f,-.86f,.77f,.78f,-.19f,-.15f,-.77f,.37f,-.01f,-.21f,-.07f,-.78f,.54f,.78f,.38f,.69f,.32f,.04f,.42f,-.40f,.06f,.79f,-.26f,-.15f,-.27f,.23f,-.37f,.90f,-.22f,.73f,-.87f,.59f,-.53f,-.35f,-.88f,.82f,-.57f,-.86f,-.38f,-.87f,.72f,.51f,.56f,.32f,.27f,.86f,.96f,-.40f,-.54f,-.54f,.64f,.99f,-.50f,.29f,.58f,.66f,-.58f,.23f,.24f,-.67f,-.23f,.31f,.47f,-.91f,.56f,.97f,.06f,-.68f,-.36f,.47f,.98f,.41f,-.71f,-.38f,.94f,-.28f,.10f,.87f,.63f,.19f,-.93f,-.64f,.35f,.85f,-.33f,.62f,-.54f,.37f,.92f,-.22f,-.55f,-.59f,-.06f,-.81f,.81f,-.05f,.70f,.47f,.62f,-.69f,-.19f,.37f,-.57f,.66f,-.14f,-.97f,-.71f,.57f,.66f,-.46f,.66f,-.11f,.91f,.75f,-.02f,.14f,-.25f,-.32f,.92f,-.58f,.48f,.62f,.23f,.06f,.17f,-.28f,.81f,.78f,-.14f,.09f,.12f,.27f,.36f,.92f,-.06f,-.52f,-.17f,.36f,-.22f,-.41f,-.64f,.64f,-.69f,-.68f,-.07f,-.12f,.73f,.88f,-.49f,.00f,-.93f,-.45f,-.39f,.67f,-.02f,-.55f,.61f,.51f,.41f,.27f,.19f,-.49f,.14f,.16f,-.45f,-.31f,.70f,-.17f,.52f,-.34f,.66f,.84f,.47f,-.90f,-.38f,.21f,.55f,.86f,.86f,-.79f,-.44f,-.51f,-.09f,-.30f,-.39f,-.18f,-.81f,.32f,-.90f,-.14f,-.24f,.77f,-.68f,.09f,.64f,.08f,.15f,-.36f,.51f,-.54f,-.34f,-.73f,-.25f,.66f,-.33f,.23f,-.93f,.40f,.62f,.12f,.23f,.20f,.57f,-.52f,.08f,.53f,.10f,.79f,-.63f,.46f,-.12f,.21f,.60f,.84f,.84f,-.54f,-.24f,-.75f,.71f,.94f,.81f,-.07f,-.96f,-.51f,.40f,-.26f,.34f,-.79f,-.95f,-.58f,.54f,-.48f,-.83f,-.71f,-.45f,-.68f,.05f,.28f,.58f,.16f,.48f,.09f,.09f,.13f,.76f,-.54f,.14f,-.47f,.95f,-.40f,.96f,-.26f,-.38f,-.54f,.05f,.77f,-.84f,-.88f,.81f,-.69f,-.93f,-.03f,.32f,.60f,-.59f,-.07f,-.66f,.68f,-.77f,-.55f,.31f,-.22f,-.20f,.13f,.32f,.32f,.32f,-.86f,-.19f,.12f,.82f,.38f,.73f,.07f,.09f,-.04f,-.62f,.13f,-.51f,-.87f,-.48f,.07f,.13f,-.16f,.68f,-.97f,-.59f,.00f,.24f,-.67f,-.57f,.59f,.64f,.36f,-.50f,.01f,.94f,.33f,.69f,.24f,-.96f,.31f,-.21f,.77f,-.78f,-.87f,.73f,.55f,.46f,.95f,.99f,.64f,.03f,.04f,-.19f,-.03f,.54f,.19f,-.09f,.57f,.25f,.75f,-.22f,.45f,.23f,-.52f,-.27f,-.83f,.01f,-.08f,-.51f,.26f,-.36f,-.30f,-.77f,.17f,-.02f,-.57f,-.29f,-.47f,-.53f,.97f,.49f,.03f,.52f,.93f,.71f,.82f,-.35f,.38f,.47f,-.92f,-.36f,.69f,-.88f,.58f,-.02f,-.80f,-.69f,.56f,-.15f,-.74f,.88f,.35f,.24f,.03f,-.43f,-.03f,-.97f,.17f,-.92f,.44f,-.57f,-.90f,.50f,.26f,-.87f,.25f,-.70f,-.19f,.56f,.92f,.09f,.55f,-.38f,.28f,.91f,-.25f,.55f,.46f,-.80f,.24f,-.66f,.32f,-.05f,-.09f,.12f,.86f,.66f,-.93f,.88f,-.31f,-.13f,.68f,.84f,-.58f,.19f,-.61f,-.06f,-.01f,.10f,-.77f,.36f,-.05f,.96f,.83f,-.66f,.81f,.54f,.46f,-.93f,.19f,-.75f,.85f,-.26f,-.03f,-.77f,.54f,-.54f,.72f,-.05f,-.81f,.58f,-.34f,-.53f,.98f,.96f,-.55f,.99f,-.48f,-.29f,-.01f,.09f,-.02f,-.34f,-.15f,.44f,-.54f,-.82f,-.35f,-.87f,-.45f,.80f,.74f,.86f,.75f,.66f,-.69f,-.29f,-.18f,.17f,-.50f,.66f,.37f,.75f,-.44f,-.07f,.06f,-.36f,-.10f,-.63f,.12f,-.25f,-.71f,-.51f,-.75f,-.46f,.52f,-.36f,-.97f,.75f,-.23f,.26f,-.27f,.78f,-.93f,-.54f,.00f,.73f,.05f,.00f,-.95f,.19f,.53f,-.57f,.82f,-.48f,-.46f,.37f,.74f,-.99f,.46f,-.10f,.91f,.83f,-.45f,-.37f,-.05f,-.75f,-.23f,-.88f,-.16f,-.97f,.80f,.64f,-.57f,.53f,-.70f,-.19f,.96f,.26f,-.70f,.59f,-.32f,.49f,-.25f,-.01f,.57f,-.21f,.04f,-.32f,.23f,.19f,-.17f,-.58f,-.27f,-.50f,-.59f,-.77f,.07f,-.37f,.13f,-.83f,.31f,.63f,-.55f,-.32f,-.66f,-.61f,-.95f,.04f,.58f,.19f,.50f,-.79f,.32f,.89f,.58f,-.28f,.21f,-.36f,.84f,.24f,-.18f,.96f,-.43f,.17f,-.38f,-.07f,.05f,.91f,-.41f,-.24f,.93f,-.46f,-.51f,.93f,.49f,.17f,.72f,-.07f,-.94f,-.04f,.87f,.86f,-.73f,-.76f,-.54f,-.03f,.83f,.62f,.76f,-.72f,-.76f,.95f,.32f,.61f,.45f,.63f,.89f,-.73f,.63f,.30f,-.75f,-.03f,.64f,.48f,-.94f,-.40f,.28f,.91f,.29f,-.02f,.62f,-.06f,.07f,.86f,.31f,.93f,.41f,.58f,.67f,-.30f,.64f,.59f,-.33f,-.77f,-.52f,.74f,-.58f,.93f,-.17f,-.49f,-.91f,.82f,-.57f,.05f,.17f,.44f,.72f,-.95f,-.29f,.70f,-.96f,-.11f,.31f,-.52f,-.97f,-.58f,.46f,-.06f,-.30f,.25f,.16f,-.38f,.72f,.47f,.47f,.84f,.64f,-.61f,-.39f,-.78f,.58f,.19f,-.05f,.38f,.35f,.11f,-.16f,.85f,-.63f,.88f,.22f,.79f,.30f,-.50f,-.90f,.41f,.16f,.40f,-.38f,-.94f,-.61f,-.02f,.13f,-.84f,.86f,.61f,.08f,.00f,.13f,-.58f,.09f,-.10f,-.21f,-.89f,-.53f,.41f,.02f,.93f,.94f,-.95f,.14f,-.81f,.51f,.00f,-.23f,.11f,-.50f,-.62f,-.57f,.46f,.65f,.98f,.61f,.67f,-.32f,-.09f,-.70f,.62f,.40f,-.69f,.15f,.73f,.76f,.49f,-.10f,-.13f,.93f,.72f,.08f,.96f,.44f,.63f,-.68f,-.69f,.35f,.50f,-.73f,.58f,.85f,-.21f,.32f,-.87f,.05f,-.05f,.35f,.63f,-.24f,-.96f,-.42f,-.05f,.75f,.07f,-.91f,-.35f,-.80f,-.48f,-.86f,-.35f,.42f,.88f,.38f,.53f,-.74f,-.61f,.14f,-.43f,.68f,-.02f,-.30f,-.81f,-.39f,.32f,-.39f,.23f,-.82f,.17f,-.05f,-.45f,-.37f,.57f,-.44f,.72f,-.00f,.84f,.98f,-.03f,-.34f,-.51f,-.27f,.46f,-.49f,.86f,-.15f,-.70f,.91f,-.87f,-.44f,.68f,.96f,.20f,-.22f,.96f,-.46f,-.18f,.42f,-.02f,-.66f,.85f,.23f,.03f,.28f,-.56f,-.20f,-.94f,.90f,-.11f,-.63f,-.19f,-.37f,.29f,.18f,-.32f,.41f,-.42f,.95f,-.88f,.74f,-.55f,-.36f,-.70f,.09f,.08f,-.33f,.16f,.67f,.46f,-.94f,-.47f,.94f,.61f,-.40f,.97f,-.08f,.29f,.61f,-.88f,.66f,.83f,-.84f,.61f,.66f,.92f,.26f,-.12f,-.25f,-.01f,-.07f,-.98f,-.45f,.92f,-.29f,-.48f,.08f,.16f,.53f,.96f,.55f,.39f,-.61f,.45f,-.45f,-.09f,.10f,-.07f,-.11f,-.87f,-.43f,.32f,.91f,-.72f,-.75f,.55f,-.80f,.33f,.03f,-.19f,-.93f,-.65f,.92f,-.26f,.89f,.50f,.28f,-.98f,-.55f,-.80f,-.50f,.33f,.08f,.85f,-.33f,.57f,-.59f,.97f,.95f,.51f,.60f,-.80f,.92f,-.55f,.10f,.13f,-.89f,.04f,-.75f,.73f,.80f,.87f,.23f,.59f,-.63f,.70f,-.95f,.82f,-.27f,.54f,.19f,.96f,.95f,-.81f,.02f,.45f,.40f,-.52f,-.74f,.27f,.37f,-.27f,-.23f,.24f,.28f,.27f,.38f,.62f,-.84f,.22f,-.88f,-.77f,.61f,-.22f,-.26f,.59f,.63f,.49f,-.77f,.30f,-.57f,.39f,-.21f,-.68f,.43f,.60f,.30f,-.51f,-.28f,.26f,-.06f,-.68f,.50f,-.07f,.16f,.25f,-.39f,-.08f,.81f,.83f,-.72f,-.67f,.35f,-.08f,.37f,-.24f,.64f,.37f,-.17f,.61f,-.35f,.21f,.72f,.98f,.38f,-.14f,.59f,.45f,.30f,.61f,.62f,.56f,-.66f,-.39f,.11f,-.43f,-.08f,.75f,.31f,-.64f,-.53f,.41f,.48f,.54f,.62f,-.64f,-.58f,-.00f,.05f,-.62f,-.65f,.72f,-.90f,.29f,-.56f,-.68f,.98f,.84f,-.43f,.16f,-.43f,.16f,-.86f,.23f,.70f,-.94f,-.74f,.41f,.20f,.02f,-.35f,-.99f,-.87f,-.65f,.77f,.32f,.92f,.14f,.95f,-.80f,-.14f,.10f,.60f,.06f,.19f,.28f,.32f,-.87f,.32f,-.61f,-.94f,-.76f,.91f,-.77f,-.80f,-.33f,.36f,-.66f,.50f,.06f,-.79f,-.14f,.38f,.54f,-.34f,-.51f,-.35f,-.78f,.40f,.78f,-.55f,-.93f,.77f,-.93f,.69f,.86f,-.07f,-.78f,-.16f,.30f,-.95f,-.53f,.73f,-.75f,-.55f,-.16f,-.30f,-.74f,.95f,-.18f,-.65f,.99f,-.87f,.49f,.24f,-.06f,.58f,-.93f,-.93f,-.36f,.74f,.54f,-.62f,.09f,.09f,-.05f,-.06f,.15f,-.69f,-.22f,-.20f,.70f,.56f,-.30f,-.70f,-.69f,-.10f,.44f,-.01f,-.95f,-.85f,.53f,-.05f,.00f,-.66f,.05f,-.52f,.64f,-.21f,.22f,-.25f,-.22f,.28f,-.70f,.57f,-.90f,-.33f,-.69f,-.14f,.08f,-.38f,-.14f,-.97f,.84f,.69f,-.38f,.36f,.42f,-.39f,.29f,.50f,-.36f,-.58f,-.40f,.88f,.70f,.59f,.88f,.90f,-.10f,-.22f,-.68f,-.92f,-.55f,-.46f,.96f,-.50f,.08f,.92f,-.68f,-.27f,.29f,-.95f,.58f,.69f,-.62f,-.25f,.02f,-.50f,.01f,.33f,-.93f,-.85f,-.51f,.04f,.29f,-.32f,-.66f,.08f,.94f,-.81f,-.70f,.44f,-.13f,.72f,-.81f,-.55f,-.53f,-.27f,-.76f,.04f,.91f,.80f,-.91f,-.79f,-.11f,.17f,-.42f,.57f,.72f,-.52f,-.12f,-.00f,-.17f,.98f,-.30f,.20f,.64f,-.29f,-.42f,.45f,.85f,.53f,.47f,-.29f,-.04f,.97f,.86f,-.13f,-.96f,-.34f,.75f,-.04f,-.21f,.59f,.68f,.36f,-.11f,.76f,-.59f,-.28f,.07f,.49f,.23f,-.01f,-.00f,-.33f,-.14f,.17f,.68f,.41f,-.02f,-.40f,.75f,.39f,-.07f,.11f,.66f,.47f,.90f,.17f,-.15f,.48f,-.10f,-.81f,.47f,.72f,.51f,-.64f,.41f,-.21f,.97f,.42f,.81f,.00f,-.99f,.43f,.83f,.78f,-.92f,.32f,.59f,-.96f,-.46f,.32f,.53f,-.28f,-.00f,-.04f,-.99f,.67f,-.13f,-.43f,.50f,-.65f,.96f,.71f,-.71f,-.40f,.44f,.25f,.90f,.45f,.71f,-.66f,.00f,.52f,.93f,.58f,.29f,.62f,.27f,.56f,.42f,.26f,.78f,-.92f,-.29f,.98f,.97f,.80f,.53f,.40f,-.28f,-.01f,-.15f,.00f,.18f,.01f,-.46f,-.81f,-.42f,.58f,-.47f,.40f,-.81f,.67f,.70f,-.64f,.31f,-.75f,-.07f,.73f,.31f,.20f,.07f,-.31f,-.50f,-.91f,-.53f,.80f,.13f,-.31f,.46f,.99f,-.52f,.09f,-.21f,.46f,-.76f,.27f,.88f,.30f,-.83f,-.85f,.98f,-.34f,-.84f,.50f,-.78f,.17f,-.72f,-.06f,-.53f,-.53f,-.74f,.95f,.42f,.25f,-.73f,.10f,-.06f,-.87f,.01f,.54f,.87f,.68f,-.85f,.53f,-.02f,.86f,-.15f,.23f,.18f,-.34f,.67f,-.82f,-.70f,.35f,-.90f,.09f,-.03f,.94f,.69f,.43f,.60f,-.87f,.45f,.61f,-.31f,.54f,-.91f,-.73f,.58f,.27f,-.81f,-.94f,-.63f,-.91f,.10f,.25f,.08f,.47f,-.75f,-.34f,.69f,.37f,.90f,.21f,.81f,-.53f,.80f,-.67f,-.95f,-.55f,.21f,-.04f,-.12f,-.64f,.78f,.15f,-.49f,.26f,-.29f,.09f,-.84f,-.93f,.11f,.58f,-.67f,.60f,-.40f,-.61f,.72f,-.36f,-.24f,-.89f,-.21f,-.82f,-.37f,-.57f,.72f,-.66f,.63f,.74f,.37f,-.51f,.37f,-.71f,.92f,-.34f,.02f,-.95f,-.37f,-.92f,.04f,.12f,-.26f,-.43f,-.81f,-.26f,-.48f,.44f,-.12f,-.75f,.98f,-.31f,-.01f,.29f,.77f,.29f,.87f,.47f,.18f,.07f,-.99f,.81f,.14f,.53f,-.03f,-.22f,.28f,-.56f,-.86f,.52f,.83f,.89f,-.54f,-.48f,-.91f,.80f,-.12f,-.06f,.09f,.75f,-.36f,.55f,.55f,.46f,.74f,.53f,.99f,-.33f,-.45f,-.06f,-.12f,.19f,-.17f,-.38f,-.78f,-.58f,-.74f,-.65f,-.03f,-.77f,-.38f,-.97f,.24f,.57f,.75f,.45f,.39f,.89f,-.81f,-.42f,.93f,-.87f,.52f,.44f,-.34f,-.16f,-.02f,-.85f,.97f,-.41f,-.57f,.96f,-.94f,-.66f,.92f,-.94f,.23f,-.06f,-.68f,.37f,.30f,.75f,-.51f,.12f,.52f,-.17f,.55f,-.17f,.40f,.60f,.97f,.83f,-.13f,-.28f,.73f,-.77f,.36f,-.03f,.96f,.41f,.97f,-.65f,-.78f,-.68f,.71f,-.25f,-.02f,-.14f,.32f,.48f,-.85f,-.12f,-.11f,.90f,-.59f,-.70f,.19f,.53f,-.83f,.34f,-.22f,.34f,-.37f,.01f,-.64f,-.40f,-.64f,-.04f,-.41f,-.77f,-.63f,.16f,-.17f,.29f,-.19f,.87f,.49f,-.09f,.27f,-.24f,.73f,-.55f,.97f,-.62f,.64f,.27f,-.66f,-.34f,.39f,.81f,-.13f,.14f,.88f,-.43f,.79f,-.88f,.91f,-.25f,-.24f,-.40f,-.51f,-.61f,.11f,.45f,.23f,.60f,-.73f,.20f,.65f,-.25f,-.67f,.76f,-.83f,.64f,-.01f,-.51f,-.27f,-.03f,.33f,.27f,-.69f,-.85f,.06f,.53f,-.78f,.85f,-.26f,.46f,-.79f,.86f,-.25f,-.67f,-.88f,-.69f,-.71f,.58f,.09f,-.60f,.83f,.40f,-.88f,-.15f,.46f,.05f,.37f,.25f,-.85f,-.39f,.69f,-.68f,-.21f,.97f,-.03f,-.13f,-.59f,.65f,-.72f,-.84f,-.80f,.42f,-.36f,-.11f,-.84f,-.27f,-.13f,-.03f,.82f,.33f,-.24f,.30f,-.28f,.41f,-.56f,-.47f,-.18f,-.57f,-.56f,-.87f,-.05f,-.17f,-.31f,-.73f,-.89f,.01f,.20f,.06f,-.85f,-.75f,.78f,-.60f,.87f,.91f,-.13f,.75f,-.49f,.96f,-.60f,-.87f,-.49f,.83f,.42f,-.29f,.32f,-.46f,-.84f,.37f,-.36f,.39f,-.72f,.24f,.28f,-.63f,.40f,.92f,-.75f,-.68f,.74f,.07f,.60f,.22f,-.50f,-.10f,-.08f,.50f,-.34f,.24f,-.77f,-.23f,.77f,.89f,-.20f,.89f,-.95f,.35f,.19f,.70f,-.10f,.07f,.71f,.16f,.33f,.37f,.13f,-.98f,-.12f,-.50f,.68f,-.62f,-.38f,.13f,.86f,.22f,-.56f,.35f,-.98f,-.72f,.86f,-.44f,.73f,.57f,.84f,-.93f,-.62f,-.67f,.14f,-.73f,-.85f,-.38f,.01f,-.87f,.05f,-.98f,-.18f,.56f,.90f,-.66f,-.81f,.09f,-.63f,.04f,-.88f,.74f,.73f,.71f,.68f,.27f,-.52f,.16f,-.72f,.10f,-.71f,.97f,-.55f,-.06f,-.68f,-.74f,-.66f,-.93f,.23f,-.35f,.37f,.54f,.60f,.10f,-.76f,.45f,-.66f,-.08f,.98f,-.09f,.96f,-.54f,-.33f,-.04f,.36f,.86f,-.42f,-.50f,.45f,.33f,.78f,.25f,.58f,.15f,-.89f,-.49f,.42f,.78f,-.86f,-.65f,.81f,-.67f,.62f,.14f,.62f,.05f,-.93f,.34f,-.40f,.01f,.46f,-.34f,.51f,-.86f,.94f,-.20f,-.75f,.27f,-.09f,.41f,-.86f,.80f,-.13f,-.31f,.52f,-.12f,.46f,.58f,.60f,-.12f,-.45f,-.61f,-.33f,-.80f,-.25f,-.00f,-.20f,-.48f,-.98f,.96f,-.86f,-.21f,.30f,.26f,.53f,.83f,.82f,-.39f,-.66f,-.69f,-.42f,.35f,-.88f,.27f,-.34f,-.21f,-.89f,.74f,-.98f,-.35f,-.72f,-.37f,-.32f,.39f,-.24f,-.83f,.95f,-.43f,.72f,.24f,-.85f,-.19f,.55f,.02f,.34f,-.41f,-.08f,-.25f,.40f,-.00f,.58f,.43f,.79f,.29f,-.57f,-.02f,-.31f,-.01f,.33f,-.09f,-.07f,.53f,-.58f,.94f,-.64f,.37f,.63f,-.79f,.21f,.72f,.24f,.09f,.34f,.15f,.41f,.94f,.69f,.51f,.52f,-.03f,-.81f,-.99f,-.76f,.33f,.61f,.03f,-.61f,.78f,.69f,.62f,-.22f,-.70f,-.64f,.55f,-.56f,.24f,-.39f,.33f,-.08f,.92f,.88f,-.94f,-.85f,-.47f,-.96f,-.51f,.22f,.87f,-.85f,.51f,.27f,.12f,-.83f,-.13f,-.63f,.72f,.51f,.35f,-.94f,.60f,.79f,-.57f,-.70f,.21f,-.68f,-.84f,.10f,.76f,.92f,.60f,-.16f,.85f,-.28f,-.00f,.25f,.61f,.69f,-.62f,.17f,.05f,.08f,-.87f,.23f,.95f,-.82f,.84f,-.58f,.76f,-.46f,-.35f,.76f,-.57f,.70f,-.14f,.55f,.57f,.57f,-.28f,-.18f,.46f,.89f,.07f,.45f,-.65f,-.46f,.01f,.32f,-.96f,-.71f,.05f,.35f,-.90f,-.56f,.72f,-.33f,-.48f,.42f,.32f,.16f,-.55f,-.83f,.42f,.74f,.41f,.93f,.61f,-.39f,-.03f,-.81f,-.37f,-.55f,-.06f,.62f,.69f,.74f,.21f,.12f,-.18f,.25f,-.66f,.40f,-.33f,.95f,.72f,.29f,.71f,-.46f,-.56f,-.79f,.59f,-.71f,-.02f,.61f,.66f,.23f,-.44f,.99f,.66f,-.47f,.99f,.68f,-.21f,-.96f,-.67f,.39f,.22f,-.72f,.01f,.62f,.68f,-.87f,-.92f,-.64f,-.60f,-.51f,-.45f,.72f,-.86f,.99f,-.77f,.36f,.20f,.17f,-.32f,-.06f,.10f,.63f,-.08f,-.01f,.58f,-.31f,.18f,-.09f,.10f,-.72f,.98f,-.76f,-.21f,-.61f,.19f,-.52f,.13f,.39f,-.53f,-.86f,-.91f,-.57f,.61f,.47f,-.73f,.63f,.07f,.90f,-.58f,.52f,.45f,-.51f,-.87f,.31f,.25f,-.08f,.53f,-.73f,-.08f,-.86f,-.63f,.63f,-.40f,-.66f,-.40f,-.71f,-.76f,.66f,-.22f,-.58f,.49f,-.97f,.84f,.96f,-.09f,-.92f,-.24f,.01f,-.97f,-.80f,.00f,-.99f,.62f,.62f,-.32f,.11f,.30f,.80f,.62f,-.57f,.89f,.57f,-.87f,-.95f,.74f,-.57f,-.31f,.05f,-.78f,-.51f,.84f,-.40f,-.97f,-.31f,.89f,-.41f,.15f,.52f,.19f,-.33f,.07f,.74f,.06f,-.64f,-.70f,.58f,-.31f,-.99f,-.46f,.28f,.08f,-.73f,-.31f,.70f,-.90f,.28f,.88f,-.92f,-.21f,-.63f,.47f,.72f,.86f,.95f,.79f,.34f,.16f,.91f,.65f,-.86f,-.81f,.67f,.18f,-.58f,.25f,-.49f,.52f,-.86f,-.25f,.11f,-.97f,.44f,.43f,.64f,.27f,-.64f,-.44f,.96f,-.44f,-.79f,.17f,-.21f,.86f,-.61f,-.16f,-.68f,.95f,.67f,.58f,.10f,.61f,-.99f,-.01f,.86f,-.80f,.35f,-.44f,.84f,.92f,-.35f,.62f,-.40f,.71f,-.38f,.43f,.02f,.74f,-.66f,.68f,.29f,-.79f,.36f,.07f,-.04f,.43f,-.51f,-.47f,.55f,.15f,.82f,-.94f,-.59f,-.85f,-.74f,.85f,.41f,.59f,.53f,.91f,.12f,.92f,-.52f,.41f,.41f,-.25f,.64f,-.45f,.25f,-.56f,-.53f,.16f,-.75f,-.85f,-.16f,-.84f,-.85f,-.57f,-.57f,-.00f,.41f,-.96f,.34f,.47f,-.53f,-.84f,.00f,.39f,-.38f,.87f,-.84f,.56f,-.49f,.32f,.23f,-.04f,-.16f,-.01f,-.47f,-.02f,.57f,-.69f,-.71f,-.71f,.72f,.82f,.65f,.73f,-.31f,.76f,-.64f,.15f,-.50f,-.59f,-.94f,.81f,.78f,-.19f,-.56f,.87f,-.71f,.65f,.43f,.79f,.33f,.07f,.33f,-.77f,.11f,.88f,.47f,.66f,-.62f,.89f,.22f,-.49f,-.71f,-.87f,.77f,-.86f,.08f,-.53f,-.58f,.89f,.57f,-.65f,-.25f,-.48f,-.28f,-.59f,.60f,-.56f,-.06f,.33f,.29f,-.00f,-.66f,.86f,-.90f,-.59f,-.32f,.00f,-.16f,-.59f,.23f,.45f,-.86f,.59f,.37f,-.79f,-.57f,.77f,.34f,-.08f,-.98f,.01f,.69f,.28f,-.84f,.74f,-.02f,.49f,.29f,.07f,.09f,.47f,.87f,.25f,.38f,-.31f,-.12f,-.23f,.23f,-.94f,-.48f,.38f,-.12f,-.66f,-.72f,-.21f,.83f,.78f,-.27f,.81f,.58f,-.93f,-.63f,-.42f,-.16f,-.96f,-.73f,.76f,.32f,-.60f,.50f,.24f,-.48f,-.20f,.99f,-.24f,.59f,.07f,-.80f,-.01f,.45f,-.48f,.34f,-.60f,.15f,-.40f,.68f,.69f,.13f,-.99f,-.21f,-.17f,.29f,-.20f,.86f,-.80f,.82f,-.36f,.79f,.75f,-.37f,-.54f,.16f,-.86f,.36f,-.46f,.33f,-.31f,.75f,-.62f,.79f,-.45f,.34f,-.72f,-.67f,.06f,.17f,.57f,.89f,-.08f,.58f,-.01f,.27f,.05f,-.42f,-.22f,.86f,.92f,-.66f,-.01f,-.62f,.62f,.57f,.17f,.47f,-.50f,-.72f,-.90f,.05f,-.40f,-.77f,-.71f,-.03f,-.81f,.11f,-.34f,-.70f,.05f,.74f,-.27f,-.50f,-.47f,.87f,-.15f,.14f,.31f,-.67f,.74f,-.55f,.41f,-.80f,.84f,-.23f,.10f,-.17f,.65f,.79f,-.67f,.67f,-.17f,.32f,-.06f,.44f,.13f,.26f,-.67f,.63f,-.17f,.40f,.26f,.55f,.29f,.62f,.83f,-.41f,-.07f,.40f,-.40f,-.21f,.45f,-.88f,-.83f,.85f,-.84f,.43f,.44f,.24f,-.32f,.29f,-.62f,-.24f,.38f,-.84f,-.97f,-.68f,.44f,-.70f,-.95f,-.70f,-.39f,-.30f,-.72f,-.55f,-.09f,-.15f,.77f,.40f,-.16f,-.94f,-.24f,-.79f,-.94f,.46f,-.35f,.93f,.10f,.39f,-.37f,-.63f,.03f,-.13f,.64f,.49f,-.52f,.24f,-.59f,.13f,.42f,-.19f,.33f,-.33f,-.67f,.43f,.22f,-.85f,.83f,-.51f,.60f,.15f,-.90f,-.24f,-.39f,.90f,-.87f,.39f,-.70f,-.05f,-.09f,-.95f,-.05f,-.08f,.92f,.95f,-.15f,-.85f,-.87f,.53f,-.31f,.49f,-.59f,.92f,-.82f,-.97f,.40f,-.60f,.80f,-.48f,-.68f,.33f,.73f,-.37f,-.31f,.26f,.62f,.11f,.65f,-.73f,.26f,-.88f,.33f,-.03f,-.54f,-.92f,-.74f,-.80f,-.82f,-.61f,-.97f,-.09f,-.81f,-.83f,-.84f,.02f,-.26f,.60f,-.30f,.41f,-.59f,-.07f,-.70f,.74f,.25f,-.50f,.88f,.73f,-.40f,.49f,.84f,.27f,.80f,-.06f,-.26f,.16f,-.87f,-.86f,.19f,.88f,.00f,.19f,.89f,.94f,.11f,-.09f,.04f,-.15f,-.07f,.56f,.71f,-.94f,-.13f,.72f,.50f,-.17f,-.58f,-.35f,.20f,-.18f,-.31f,-.80f,-.95f,-.00f,.84f,.91f,.34f,.89f,-.08f,.90f,.80f,-.41f,-.50f,.95f,.41f,.52f,-.21f,-.99f,-.77f,.41f,-.32f,-.99f,.21f,.29f,.82f,.99f,.57f,-.46f,.77f,-.43f,-.71f,.13f,-.61f,-.38f,.89f,-.15f,-.46f,-.81f,-.75f,.54f,.45f,.88f,-.25f,-.74f,.38f,.52f,-.93f,.54f,.50f,.52f,.41f,-.64f,.64f,.04f,.33f,.38f,-.97f,.08f,-.90f,.49f,.54f,-.09f,.90f,.66f,.20f,-.75f,-.42f,.64f,.33f,.21f,-.08f,.02f,.92f,-.18f,-.28f,.75f,.13f,.37f,.17f,.89f,.23f,-.47f,.90f,-.24f,.13f,-.81f,.73f,-.12f,.76f,-.54f,.19f,.98f,.81f,.99f,.76f,-.33f,.12f,.22f,.02f,-.43f,-.39f,-.74f,-.79f,.26f,-.14f,.52f,-.96f,-.60f,-.79f,-.53f,.11f,.65f,-.12f,-.92f,-.85f,-.06f,.31f,-.55f,.27f,-.87f,-.46f,-.95f,.23f,-.52f,-.96f,.06f,-.22f,-.44f,-.11f,.34f,-.89f,.43f,-.24f,.48f,-.07f,.08f,-.85f,-.25f,.92f,-.27f,-.03f,-.06f,-.85f,.12f,-.55f,-.70f,-.11f,.70f,.69f,.76f,-.91f,-.63f,-.61f,-.98f,.73f,-.90f,-.62f,.68f,-.87f,-.56f,-.27f,-.70f,.99f,.79f,-.41f,.54f,-.64f,.61f,-.35f,.90f,-.33f,.51f,-.59f,.95f,-.73f,.15f,-.22f,.26f,.72f,.51f,.61f,-.00f,.37f,.58f,.26f,-.13f,.41f,.41f,-.95f,-.79f,.58f,-.20f,-.86f,-.20f,-.60f,-.44f,.11f,-.23f,.91f,.01f,.87f,-.19f,.24f,.01f,-.88f,.09f,.08f,.83f,.02f,-.57f,-.21f,.93f,.91f,.51f,-.74f,.29f,.31f,.41f,.55f,.72f,-.93f,-.47f,.95f,-.73f,-.76f,-.98f,-.84f,.31f,-.89f,-.27f,-.70f,-.22f,.39f,.81f,-.85f,.69f,-.50f,-.53f,-.14f,-.02f,.31f,-.74f,-.19f,.84f,.99f,-.92f,-.35f,-.30f,.85f,-.64f,-.84f,.69f,.77f,.91f,.04f,.94f,.44f,.28f,.13f,-.65f,-.84f,.61f,.01f,.78f,-.17f,.27f,.72f,-.14f,-.18f,.70f,-.19f,.57f,-.89f,.48f,.36f,.39f,.46f,.67f,.16f,-.86f,-.52f,.20f,-.18f,.70f,-.11f,-.57f,-.96f,-.27f,.41f,.72f,-.33f,-.93f,.99f,-.90f,.79f,.17f,-.40f,-.13f,.92f,.74f,-.58f,-.76f,-.93f,.65f,.97f,.27f,-.84f,-.61f,.83f,-.56f,-.76f,-.69f,-.44f,-.84f,-.95f,.33f,.88f,-.70f,.30f,-.69f,.50f,-.65f,-.16f,-.51f,.81f,.13f,-.45f,.35f,-.10f,.31f,-.21f,.39f,.49f,-.66f,.44f,.71f,.87f,-.61f,.86f,-.65f,.87f,-.89f,.70f,-.20f,-.83f,.61f,.43f,-.98f,-.09f,-.20f,-.09f,.99f,.45f,.28f,-.28f,-.46f,.05f,.96f,.66f,.72f,.27f,-.07f,-.27f,.54f,.16f,.54f,-.60f,-.50f,-.94f,-.28f,-.05f,.52f,.65f,-.99f,.50f,-.51f,.28f,-.93f,-.15f,-.64f,-.62f,.36f,-.83f,.86f,.42f,.60f,.59f,.42f,.89f,.50f,.73f,-.41f,-.61f,-.51f,-.32f,-.53f,.64f,.98f,-.94f,-.13f,.06f,.20f,-.18f,-.05f,.50f,-.39f,-.08f,.94f,-.22f,-.01f,.51f,.43f,-.18f,-.40f,.08f,-.40f,.44f,-.45f,-.39f,.39f,.02f,.81f,-.75f,.91f,-.83f,.06f,-.36f,.03f,.11f,-.19f,-.29f,.21f,-.61f,.44f,-.86f,-.69f,-.18f,.51f,.90f,-.05f,.39f,-.89f,-.61f,.66f,-.06f,.90f,.20f,.91f,-.50f,-.63f,.68f,.40f,-.30f,.28f,.98f,-.56f,-.13f,-.12f,.29f,.66f,.07f,.48f,.60f,.09f,.09f,.38f,.63f,-.27f,.35f,-.26f,-.83f,-.63f,-.84f,.87f,-.94f,.07f,.17f,-.98f,.76f,-.98f,.81f,.18f,-.77f,-.80f,-.26f,.76f,.41f,-.40f,-.80f,.16f,.96f,.81f,.47f,-.29f,.57f,-.06f,-.04f,-.30f,-.92f,-.18f,.32f,-.66f,-.90f,.74f,.61f,.37f,.34f,.67f,.08f,.80f,-.27f,-.49f,-.15f,.84f,-.71f,.56f,.24f,.89f,-.00f,-.44f,.67f,-.27f,.31f,-.86f,.52f,.87f,-.03f,-.34f,-.38f,.82f,.84f,-.30f,-.13f,.86f,-.35f,.67f,.45f,-.24f,.95f,.70f,-.83f,-.17f,-.09f,-.35f,-.47f,.71f,-.75f,.33f,.64f,-.68f,.43f,.38f,.48f,.21f,.85f,-.77f,.11f,-.19f,.05f,-.17f,-.67f,-.84f,-.54f,-.54f,-.70f,-.39f,-.52f,-.42f,-.78f,.39f,.70f,-.06f,-.79f,.41f,.94f,.83f,-.33f,.98f,-.14f,-.40f,-.72f,.66f,.79f,.79f,.29f,-.14f,-.34f,.40f,-.95f,.90f,-.38f,-.29f,.95f,.72f,.17f,-.54f,.29f,-.10f,.93f,.69f,-.19f,.20f,.02f,.96f,-.05f,-.49f,-.93f,.32f,-.31f,.07f,.71f,-.58f,.08f,.95f,-.55f,.95f,.05f,.42f,-.98f,.95f,-.09f,-.77f,.42f,.60f,-.16f,.29f,.25f,.39f,-.81f,-.90f,.46f,-.30f,.10f,.07f,.81f,-.57f,-.31f,-.21f,-.12f,.01f,-.11f,.66f,.01f,.90f,-.38f,.52f,.08f,-.77f,-.66f,-.54f,.00f,-.44f,.17f,.38f,-.33f,.80f,-.80f,.15f,-.51f,-.23f,-.52f,.37f,.58f,-.35f,-.71f,-.79f,-.47f,-.69f,-.88f,-.48f,.28f,.71f,.46f,-.28f,-.13f,.64f,-.43f,-.44f,.44f,-.11f,.22f,.10f,-.94f,.42f,-.15f,.25f,-.25f,-.88f,-.29f,.92f,-.13f,.06f,-.35f,-.66f,-.77f,.02f,-.55f,.51f,-.57f,.50f,.68f,-.22f,.23f,.00f,-.66f,-.36f,.44f,.25f,.28f,.96f,.68f,-.22f,-.33f,-.67f,.11f,-.33f,-.15f,-.16f,-.87f,-.37f,.18f,.62f,-.25f,-.13f,.65f,-.03f,-.00f,.30f,.77f,.63f,.57f,.93f,-.37f,-.06f,-.72f,.39f,.36f,-.28f,.53f,.00f,.71f,-.57f,-.64f,.30f,-.40f,-.30f,.05f,-.90f,.36f,.87f,.77f,.03f,-.41f,-.91f,.81f,.16f,-.12f,-.18f,.30f,-.51f,.26f,-.46f,-.62f,.79f,-.98f,-.60f,-.90f,.01f,-.52f,.66f,-.49f,.75f,-.68f,-.78f,-.70f,.44f,.07f,-.90f,-.50f,.22f,.04f,-.54f,.35f,-.38f,.18f,-.04f,-.09f,-.83f,-.16f,-.99f,-.78f,-.73f,.71f,.27f,.15f,.19f,.29f,-.20f,.00f,.38f,.84f,-.20f,.28f,.19f,-.02f,-.50f,-.66f,-.23f,.79f,.62f,.20f,-.24f,.60f,-.39f,-.61f,-.53f,-.93f,.99f,.78f,-.92f,.92f,.43f,.46f,.87f,.32f,-.53f,.85f,-.73f,-.71f,-.39f,-.90f,-.63f,.01f,-.38f,-.62f,-.25f,-.37f,.60f,-.01f,.18f,.95f,.99f,-.70f,-.76f,-.15f,-.78f,.02f,.08f,.27f,-.61f,.62f,.59f,-.04f,.46f,-.63f,-.05f,.85f,-.05f,-.84f,-.35f,-.35f,.63f,.55f,-.33f,-.53f,.62f,.64f,-.02f,.06f,-.39f,-.55f,-.09f,-.17f,-.09f,-.84f,.94f,-.08f,-.95f,-.25f,.12f,-.26f,-.94f,.32f,-.60f,-.11f,-.27f,-.32f,.31f,.97f,.64f,.99f,-.99f,-.56f,.96f,.11f,-.57f,-.51f,.82f,-.35f,.13f,.70f,-.97f,.92f,.59f,-.03f,.15f,-.09f,.46f,.28f,.73f,-.17f,.26f,-.23f,.45f,-.96f,.21f,-.28f,-.64f,-.09f,.68f,.49f,-.30f,-.75f,.98f,-.71f,-.42f,-.52f,.19f,.52f,.30f,.92f,.61f,-.53f,.62f,-.15f,.19f,.07f,.67f,.92f,.20f,-.69f,-.29f,-.75f,.54f,.78f,.92f,-.70f,-.70f,-.40f,-.41f,.89f,-.29f,.02f,.34f,-.24f,.48f,.97f,.75f,-.66f,.17f,-.32f,-.41f,.24f,.85f,-.70f,.39f,.46f,-.84f,.86f,.45f,-.78f,-.61f,.74f,.73f,.78f,.66f,-.15f,-.85f,.46f,.46f,-.21f,-.78f,-.16f,.12f,.83f,.05f,-.95f,.41f,.45f,-.02f,.56f,-.91f,-.90f,-.33f,-.94f,-.72f,-.90f,-.22f,-.01f,-.59f,.58f,-.37f,.23f,-.96f,.41f,-.65f,.69f,-.79f,.14f,.39f,-.99f,-.21f,-.73f,-.33f,.53f,-.15f,-.04f,-.00f,.33f,.12f,-.44f,-.88f,-.93f,.05f,.73f,-.66f,-.85f,-.17f,-.47f,.15f,.65f,.91f,-.72f,.32f,.64f,-.30f,-.40f,-.79f,-.77f,.27f,.09f,.65f,-.85f,.37f,.57f,-.79f,.28f,.02f,-.92f,.23f,.66f,-.38f,.24f,-.21f,.84f,-.71f,-.12f,.26f,-.29f,.26f,-.08f,.75f,.54f,.46f,.11f,-.81f,-.53f,.85f,-.24f,-.38f,.95f,.36f,.77f,-.37f,.58f,.91f,.03f,.13f,-.98f,.68f,.74f,.52f,.42f,.83f,-.76f,.52f,-.79f,-.57f,.95f,-.58f,-.11f,-.08f,.57f,.75f,-.80f,.33f,.42f,-.85f,-.74f,-.91f,-.99f,-.08f,-.06f,-.42f,.33f,-.45f,-.09f,.56f,-.95f,.98f,-.24f,.48f,-.85f,-.05f,.73f,.75f,.43f,.00f,-.29f,-.75f,-.39f,-.76f,.35f,.29f,-.19f,-.61f,-.65f,-.35f,.74f,-.12f,.62f,.59f,-.11f,-.36f,-.37f,-.59f,.28f,-.28f,-.00f,.60f,-.40f,.10f,-.94f,-.25f,-.50f,-.12f,-.90f,.13f,-.39f,.30f,-.87f,.76f,-.61f,.55f,.17f,.74f,-.36f,.52f,.71f,-.20f,.07f,.40f,.61f,.06f,.68f,.71f,-.36f,-.52f,-.61f,-.70f,-.40f,.00f,.69f,-.44f,-.15f,.45f,-.63f,.88f,.23f,-.98f,.29f,-.86f,-.24f,.65f,-.61f,-.61f,.81f,.10f,.24f,.84f,-.98f,-.26f,-.17f,-.05f,.88f,-.93f,-.34f,.10f,.89f,-.92f,.88f,-.01f,.30f,.01f,.65f,-.87f,-.51f,.42f,.60f,.02f,.56f,-.56f,-.00f,.63f,.40f,.32f,-.12f,.05f,.61f,-.57f,-.84f,.20f,-.91f,.48f,.64f,-.60f,-.24f,.84f,.21f,-.48f,.26f,-.71f,.31f,.21f,.59f,.90f,-.38f,.56f,-.72f,.22f,.01f,.74f,.19f,-.40f,-.63f,.76f,-.67f,.27f,.09f,-.04f,.49f,.33f,.55f,.73f,-.41f,-.49f,.16f,-.93f,.37f,.62f,.68f,-.51f,.08f,.89f,-.35f,.26f,.77f,.33f,.42f,-.66f,.65f,-.34f,.87f,-.97f,-.16f,.43f,-.93f,-.06f,.00f,.40f,.06f,-.16f,.79f,.46f,-.60f,-.68f,.11f,-.19f,.33f,-.37f,-.94f,-.08f,.11f,.20f,.78f,.34f,-.61f,.60f,-.93f,-.36f,.96f,-.37f,-.79f,-.56f,.86f,.36f,-.68f,-.18f,.54f,.14f,.00f,.23f,.14f,.83f,-.45f,.01f,.59f,-.02f,.43f,-.03f,.97f,.62f,-.12f,.90f,-.90f,-.08f,.36f,-.72f,.02f,.22f,-.24f,.96f,-.33f,-.13f,.44f,-.65f,.95f,.68f,-.76f,.18f,-.06f,.26f,.74f,-.38f,-.08f,.15f,.45f,-.97f,.43f,-.82f,-.84f,.99f,-.55f,-.93f,-.07f,-.29f,-.59f,-.91f,.49f,-.60f,-.56f,.63f,.86f,.49f,.76f,.69f,.12f,.67f,.60f,.12f,-.79f,.78f,.35f,.52f,-.80f,-.12f,.25f,.43f,.69f,.12f,-.01f,.47f,.65f,-.38f,.99f,.28f,-.37f,-.18f,.69f,.56f,.80f,.89f,-.13f,.57f,.45f,-.11f,.62f,.83f,.14f,-.80f,-.12f,-.04f,.25f,.59f,-.17f,.04f,-.75f,.75f,-.56f,-.46f,-.27f,.22f,.53f,.58f,.03f,.44f,-.14f,.26f,-.22f,.74f,.28f,.50f,-.21f,-.51f,-.50f,.73f,-.67f,.19f,-.58f,-.29f,.56f,-.77f,.18f,-.46f,.59f,.80f,.95f,-.66f,-.67f,-.57f,.01f,-.20f,.27f,.67f,-.12f,-.34f,.72f,-.05f,.95f,-.26f,.30f,.87f,-.61f,.91f,-.68f,.20f,-.75f,-.04f,.12f,.37f,-.34f,.06f,-.92f,-.76f,-.04f,-.17f,-.73f,.50f,-.32f,-.77f,-.25f,-.72f,-.32f,-.88f,.97f,.63f,-.05f,.05f,.33f,.46f,.65f,-.38f,.36f,.52f,-.33f,-.09f,.57f,-.44f,-.80f,.18f,-.14f,.83f,.53f,.45f,-.82f,.11f,.24f,.56f,-.47f,-.35f,-.33f,.97f,-.77f,-.42f,.61f,-.45f,.57f,.43f,.69f,.46f,.22f,.10f,-.24f,.52f,.55f,-.97f,.09f,-.52f,.78f,.50f,.87f,-.04f,.21f,.92f,.15f,-.85f,-.64f,.25f,.20f,.35f,.85f,-.95f,-.36f,-.75f,.85f,.62f,.04f,-.87f,.41f,-.07f,-.71f,-.30f,.07f,.37f,-.16f,.39f,-.62f,-.02f,.57f,-.37f,.31f,-.55f,-.41f,-.11f,-.32f,-.19f,.31f,.29f,.44f,.09f,-.54f,.38f,.07f,.10f,-.71f,-.97f,-.48f,-.55f,.49f,-.95f,.92f,.69f,.24f,-.82f,.33f,-.03f,-.20f,-.14f,-.77f,-.16f,.57f,.39f,.84f,-.73f,-.48f,.65f,.33f,.24f,-.56f,.31f,.23f,.44f,-.97f,-.87f,.14f,-.73f,-.02f,.18f,-.09f,-.02f,-.27f,.09f,.96f,-.01f,-.79f,.29f,-.34f,.99f,.87f,-.45f,.55f,-.25f,-.69f,-.75f,-.45f,-.27f,-.00f,-.99f,.26f,.07f,-.60f,-.50f,-.11f,.20f,.01f,-.09f,-.56f,.85f,.96f,.02f,.09f,-.63f,.13f,.61f,.46f,-.97f,.29f,.09f,-.41f,-.73f,.43f,-.49f,-.33f,-.35f,-.36f,.63f,.41f,-.57f,-.39f,.65f,-.03f,-.29f,.06f,.04f,.97f,-.50f,-.54f,.86f,-.41f,.57f,-.12f,.51f,-.41f,.77f,-.89f,-.74f,.45f,.99f,.03f,.96f,.48f,.47f,-.71f,-.85f,.62f,-.83f,-.71f,-.87f,.97f,-.41f,-.20f,-.84f,.55f,-.01f,-.76f,.15f,.55f,.26f,.19f,.70f,.07f,-.56f,-.51f,.48f,-.55f,.14f,.53f,-.53f,-.99f,-.22f,-.61f,-.88f,-.26f,.46f,-.19f,-.22f,.20f,-.02f,.71f,-.82f,-.67f,.80f,.31f,-.58f,-.59f,-.63f,.48f,-.73f,.45f,.49f,.74f,.59f,.72f,-.48f,.18f,-.12f,.30f,.81f,.55f,.97f,-.66f,-.52f,-.99f,.14f,-.62f,-.02f,.90f,.78f,.77f,-.05f,-.06f,.51f,-.26f,.64f,.91f,.66f,.52f,.19f,.21f,-.52f,.96f,.71f,.28f,-.89f,-.01f,.83f,.07f,.39f,-.88f,.28f,.68f,-.81f,-.46f,.42f,.47f,-.43f,-.93f,.43f,-.76f,.15f,.88f,.21f,.30f,-.85f,-.26f,-.12f,-.96f,-.84f,-.97f,-.49f,.64f,.42f,-.50f,.87f,.19f,-.13f,-.46f,-.45f,-.95f,.22f,.41f,.83f,-.14f,-.33f,.87f,.76f,-.06f,-.94f,.72f,.73f,-.78f,.90f,.89f,.84f,.90f,.28f,.88f,.86f,-.62f,.02f,-.81f,-.52f,-.82f,-.14f,-.42f,-.30f,-.02f,.79f,.10f,-.58f,-.00f,.95f,.55f,-.75f,-.09f,.36f,.44f,.30f,.03f,-.49f,-.60f,.95f,.86f,-.67f,-.23f,.28f,-.45f,-.58f,-.52f,.77f,-.19f,-.88f,-.16f,.08f,.82f,-.02f,.45f,-.66f,.73f,.94f,-.55f,-.54f,.02f,.62f,.71f,-.99f,.10f,-.06f,.08f,.79f,-.00f,-.77f,-.47f,.42f,.83f,.21f,.08f,-.43f,-.14f,.22f,.32f,-.03f,.11f,.12f,-.64f,-.58f,-.54f,-.08f,-.38f,.41f,.61f,-.65f,.27f,.98f,-.40f,-.13f,.83f,.74f,-.68f,.22f,-.69f,.42f,.77f,-.87f,-.42f,-.77f,-.11f,.25f,-.69f,.84f,-.25f,-.38f,-.56f,-.20f,-.35f,.83f,-.89f,-.51f,.82f,-.77f,.05f,.78f,.71f,.12f,-.82f,-.32f,.09f,.63f,.70f,-.49f,.80f,.25f,-.85f,.04f,-.72f,.56f,.14f,.62f,.17f,-.88f,-.26f,.20f,.72f,-.55f,-.57f,-.33f,.66f,-.12f,.46f,-.14f,-.87f,-.12f,.08f,.92f,.41f,.36f,-.58f,-.17f,.25f,-.97f,-.87f,-.86f,-.43f,.85f,-.38f,-.45f,.47f,.41f,.98f,-.67f,-.13f,-.36f,.08f,-.33f,.92f,-.96f,-.13f,.71f,.77f,.17f,-.03f,-.92f,-.06f,.00f,.29f,-.14f,-.79f,.64f,.53f,.82f,-.02f,-.39f,.94f,-.97f,-.67f,-.98f,-.38f,-.25f,-.76f,.21f,.80f,.24f,-.69f,-.44f,-.80f,.17f,-.87f,.33f,.72f,-.23f,-.39f,.52f,-.39f,-.14f,-.70f,.86f,-.58f,.05f,.82f,.19f,.58f,-.76f,.00f,.09f,.97f,-.60f,.90f,-.73f,-.47f,-.47f,.18f,.93f,-.53f,.71f,.34f,.48f,-.12f,.94f,.12f,-.44f,-.35f,-.93f,-.00f,-.91f,.40f,.37f,-.87f,-.86f,.68f,.55f,.35f,-.30f,.31f,-.37f,.81f,.89f,-.00f,-.72f,-.24f,-.33f,-.09f,-.79f,.41f,.15f,-.30f,-.99f,-.00f,-.07f,-.55f,.22f,.54f,-.15f,-.45f,-.79f,-.75f,-.50f,-.69f,-.59f,-.67f,.70f,.29f,.74f,.83f,-.83f,.41f,-.04f,-.94f,.59f,-.38f,.85f,.34f,-.06f,.08f,.79f,.41f,-.19f,-.15f,.25f,-.66f,.03f,.57f,-.17f,.69f,.32f,.07f,-.95f,.49f,.42f,-.75f,.70f,-.09f,-.98f,-.93f,.40f,-.38f,.36f,-.49f,.65f,-.30f,.14f,-.99f,.66f,-.27f,.94f,.08f,-.70f,.78f,-.93f,-.97f,-.89f,-.94f,-.77f,-.75f,-.87f,.89f,-.25f,.61f,.59f,-.55f,-.73f,.26f,.93f,-.42f,.66f,-.91f,-.72f,.05f,.26f,-.49f,-.00f,-.88f,-.66f,.34f,.93f,.91f,.94f,.25f,.72f,.13f,-.23f,-.67f,-.99f,-.77f,.45f,.33f,.68f,.34f,.43f,.60f,.36f,-.72f,-.56f,-.41f,-.72f,.65f,.34f,.91f,.95f,-.90f,-.85f,.91f,.80f,-.60f,-.96f,.29f,-.36f,-.54f,.81f,-.95f,-.54f,.67f,.13f,.16f,.73f,.92f,.00f,.08f,-.05f,.84f,-.42f,-.82f,-.31f,-.82f,-.14f,-.65f,-.15f,.55f,.56f,-.93f,-.06f,.59f,.48f,.60f,.08f,-.81f,-.72f,-.69f,-.19f,.02f,.25f,-.38f,-.41f,.31f,-.84f,-.09f,.64f,.62f,-.00f,.78f,-.87f,-.50f,-.85f,.61f,-.90f,.96f,.51f,.99f,-.33f,-.44f,-.23f,-.27f,-.03f,-.75f,.12f,-.78f,-.25f,.44f,.49f,.33f,.33f,-.77f,-.35f,-.39f,-.49f,.00f,-.20f,-.06f,-.78f,.41f,.31f,.62f,-.78f,-.36f,.98f,-.03f,-.41f,-.01f,-.81f,.30f,-.81f,.60f,-.05f,.20f,.70f,.81f,.64f,.78f,.84f,.07f,.86f,.72f,.59f,-.58f,-.85f,.23f,.03f,-.40f,-.29f,-.16f,.59f,.79f,.07f,-.29f,.45f,-.94f,-.48f,-.65f,.57f,.43f,-.31f,-.18f,-.83f,-.85f,-.27f,.28f,.58f,-.07f,.74f,.09f,-.80f,.93f,-.84f,.74f,.37f,.85f,.70f,-.95f,.35f,-.81f,-.54f,.81f,.05f,.02f,.47f,-.70f,.45f,.44f,-.12f,.46f,-.76f,.42f,-.38f,-.47f,-.80f,-.96f,-.97f,-.95f,-.91f,.79f,-.41f,-.87f,.60f,.72f,.19f,.45f,.99f,-.02f,-.70f,-.89f,-.75f,-.54f,.11f,-.69f,.94f,-.86f,-.34f,-.79f,-.75f,.84f,-.56f,.63f,.22f,-.69f,.34f,.34f,.38f,.03f,.43f,-.38f,-.70f,-.34f,-.01f,-.47f,-.88f,.97f,-.99f,.33f,-.79f,-.79f,-.78f,-.14f,-.58f,-.93f,.41f,.14f,.40f,-.06f,.31f,-.69f,-.83f,.29f,.08f,.66f,.71f,.11f,.28f,.98f,.94f,.11f,.91f,.05f,-.07f,.33f,-.97f,.48f,-.77f,.93f,.68f,-.00f,.36f,.37f,-.76f,.51f,.55f,-.92f,.15f,.84f,-.82f,.02f,-.15f,.06f,.05f,-.10f,-.57f,.93f,.03f,-.61f,.47f,-.23f,.39f,-.54f,.33f,-.48f,.45f,.65f,.64f,.35f,.64f,-.39f,.15f,.54f,.97f,-.12f,-.88f,.57f,.95f,.91f,.83f,.64f,-.16f,-.85f,.52f,.03f,.81f,.10f,.47f,.39f,-.66f,.86f,.69f,.29f,-.43f,-.76f,.58f,-.76f,-.89f,-.12f,.37f,.83f,-.02f,.36f,.79f,-.29f,.63f,.18f,-.24f,.62f,-.07f,-.25f,.07f,-.17f,-.60f,-.66f,-.21f,.83f,.04f,-.63f,-.35f,.00f,.22f,.05f,-.18f,.26f,.49f,.51f,.54f,-.43f,.10f,.07f,-.70f,.69f,-.19f,-.24f,.94f,.22f,.16f,-.23f,-.81f,-.72f,.58f,.44f,-.58f,-.89f,.59f,.64f,.72f,.43f,.82f,-.57f,.69f,.27f,-.20f,.24f,.07f,.41f,-.49f,.72f,.27f,-.40f,.11f,-.58f,.78f,-.71f,.97f,.12f,.00f,.76f,.77f,-.70f,-.13f,-.41f,.57f,.42f,.48f,.59f,.47f,.16f,-.10f,.80f,.53f,.50f,-.34f,.78f,-.20f,-.70f,.56f,-.23f,-.11f,-.51f,.44f,.21f,-.59f,.08f,-.43f,-.87f,-.60f,-.35f,.79f,-.62f,-.85f,-.39f,-.39f,-.74f,.80f,-.19f,-.93f,.55f,-.26f,-.86f,-.62f,-.00f,-.20f,-.11f,.95f,-.27f,.59f,.61f,-.78f,-.64f,-.09f,-.51f,.10f,.83f,-.22f,.37f,.40f,-.74f,-.68f,-.22f,-.36f,-.82f,.29f,.29f,-.04f,.88f,-.62f,.49f,.89f,-.87f,.30f,.90f,.54f,.07f,-.52f,-.82f,.65f,.28f,.73f,.43f,.34f,.24f,-.61f,-.20f,-.07f,.85f,.34f,-.33f,.14f,.64f,-.58f,.32f,-.46f,.07f,.88f,.64f,.12f,-.26f,-.11f,.72f,-.58f,-.58f,-.95f,.38f,.11f,-.01f,.15f,-.10f,.50f,.66f,.60f,-.81f,.75f,.48f,.65f,-.83f,-.81f,-.92f,-.31f,.81f,-.35f,.49f,.95f,-.99f,.17f,.34f,.60f,.65f,-.49f,-.18f,-.09f,.35f,.06f,.92f,.78f,-.18f,-.66f,.46f,.58f,-.56f,-.45f,.63f,.03f,.28f,-.84f,.39f,.06f,.85f,.04f,-.50f,.53f,.62f,.46f,-.86f,-.90f,-.72f,.12f,.05f,-.39f,-.41f,.59f,-.19f,-.22f,.50f,.32f,.83f,-.33f,-.83f,-.81f,-.46f,-.63f,-.58f,-.78f,-.49f,-.78f,.19f,.69f,-.66f,.27f,-.10f,-.09f,.88f,.08f,.53f,-.64f,-.38f,.46f,-.64f,-.55f,.33f,-.41f,-.80f,.11f,.37f,-.05f,-.73f,-.57f,.93f,.90f,-.60f,.93f,.81f,.94f,.23f,.06f,.68f,-.06f,-.91f,-.05f,.40f,-.11f,-.83f,-.67f,.96f,-.89f,-.56f,.96f,.82f,-.29f,.80f,.42f,.23f,-.05f,-.06f,.16f,-.54f,.08f,.69f,.97f,.19f,-.65f,.87f,.70f,.67f,.44f,.10f,-.12f,.97f,.07f,.92f,-.00f,-.58f,-.44f,.66f,-.63f,-.32f,.08f,.53f,-.11f,.80f,-.35f,.98f,-.66f,-.63f,-.80f,.87f,-.82f,.09f,.95f,-.25f,-.15f,.72f,-.97f,-.45f,.96f,.49f,.92f,-.19f,-.10f,-.12f,-.00f,-.96f,.76f,.99f,-.37f,.84f,.66f,-.63f,-.41f,.37f,-.29f,.69f,.61f,.54f,-.95f,.97f,.76f,.02f,.06f,.83f,-.94f,-.77f,.04f,-.25f,.57f,.75f,.46f,-.96f,-.49f,.76f,.35f,-.80f,-.74f,.52f,-.57f,-.38f,-.75f,-.88f,.76f,-.69f,.22f,-.64f,.65f,.79f,-.79f,.78f,-.13f,.07f,-.98f,.63f,.94f,-.54f,.52f,.02f,-.64f,-.05f,.04f,-.91f,-.05f,-.54f,.67f,.44f,-.43f,-.37f,.03f,.37f,-.51f,-.36f,.21f,.65f,-.62f,.20f,-.90f,-.37f,-.11f,-.40f,-.11f,.50f,.81f,.46f,.03f,-.09f,.05f,-.20f,.98f,.45f,.26f,-.15f,-.40f,.70f,-.17f,-.59f,-.23f,-.80f,.65f,-.47f,.39f,.68f,-.64f,.15f,-.89f,.98f,-.36f,.15f,-.59f,.88f,.47f,-.01f,.26f,-.94f,.39f,-.76f,-.43f,-.29f,-.88f,-.52f,-.42f,.12f,-.56f,.09f,.38f,.32f,-.22f,-.29f,-.46f,.59f,-.96f,.30f,.95f,-.76f,-.90f,-.76f,.56f,.35f,-.76f,.42f,-.72f,-.67f,.74f,-.60f,.90f,.74f,-.87f,-.36f,-.12f,.29f,-.83f,.60f,.35f,.44f,-.50f,.19f,.19f,.42f,-.47f,-.50f,-.35f,.49f,.41f,-.19f,-.08f,-.70f,.76f,-.44f,.69f,.16f,-.01f,.56f,-.85f,-.11f,-.87f,.11f,.17f,-.05f,.50f,.24f,-.47f,.57f,.59f,-.90f,.95f,-.05f,.02f,.51f,.82f,.28f,.91f,-.59f,.65f,-.42f,-.82f,.84f,-.31f,-.41f,.75f,.68f,-.67f,.52f,-.18f,.79f,-.38f,.63f,.33f,-.93f,-.59f,.76f,-.60f,.03f,-.77f,-.34f,.17f,-.35f,.81f,.50f,.82f,.57f,.61f,-.38f,.69f,.69f,.75f,-.61f,-.73f,.53f,.66f,-.83f,-.69f,.67f,-.41f,.84f,-.23f,.70f,.95f,.66f,.81f,.64f,-.55f,-.80f,.45f,-.11f,-.77f,-.47f,-.36f,-.41f,-.82f,-.60f,-.32f,-.58f,.49f,.32f,-.38f,.35f,-.88f,-.11f,.49f,.24f,.97f,.68f,.90f,.62f,-.40f,.94f,-.16f,-.57f,-.00f,-.18f,.55f,-.88f,.95f,-.49f,-.75f,.54f,.45f,.03f,.59f,.17f,-.22f,-.44f,-.63f,.69f,.08f,.06f,-.43f,.00f,-.50f,-.86f,-.48f,.91f,-.19f,-.18f,-.61f,.85f,-.41f,-.73f,.83f,-.86f,.66f,.94f,-.02f,-.07f,.21f,-.68f,-.33f,-.40f,.13f,-.11f,.59f,-.91f,-.28f,.72f,.18f,.46f,-.20f,-.36f,-.93f,-.38f,.91f,-.03f,-.85f,-.05f,.23f,-.03f,-.28f,.68f,.93f,.07f,.69f,-.76f,-.55f,.42f,-.27f,-.28f,.39f,-.02f,.95f,-.43f,.38f,.70f,.93f,-.55f,-.25f,-.72f,-.44f,.97f,-.72f,-.24f,-.66f,-.99f,-.29f,-.93f,.97f,.64f,.65f,.95f,.58f,.13f,.18f,.42f,-.38f,.11f,.60f,.30f,-.01f,.79f,-.31f,-.70f,.23f,-.17f,-.58f,-.82f,.13f,-.75f,-.44f,.99f,.26f,-.55f,.12f,-.40f,.85f,-.93f,.28f,-.78f,-.94f,.99f,-.71f,.55f,-.30f,.20f,.33f,-.84f,-.93f,-.49f,-.83f,-.30f,-.44f,-.14f,.96f,-.71f,-.68f,.60f,.74f,-.86f,-.10f,-.04f,.07f,-.30f,.28f,-.10f,.15f,-.34f,.94f,.71f,.17f,-.48f,-.53f,.93f,.73f,.29f,-.17f,-.34f,-.61f,-.93f,.94f,-.14f,.27f,.49f,.78f,-.69f,.36f,.38f,.49f,-.93f,.83f,-.85f,.51f,.43f,.61f,.40f,-.95f,-.81f,.09f,-.87f,-.42f,-.85f,.34f,-.43f,.72f,-.40f,-.51f,-.39f,-.50f,-.74f,.91f,.49f,.92f,.84f,.68f,-.63f,-.77f,.25f,.63f,-.60f,.32f,-.94f,.69f,-.81f,.55f,.07f,.58f,-.76f,-.82f,.53f,-.87f,-.97f,-.92f,-.19f,.12f,-.70f,.05f,.99f,-.88f,-.76f,-.81f,-.81f,.94f,-.22f,.44f,.10f,-.09f,.34f,-.72f,-.19f,-.44f,.29f,.74f,.45f,.77f,.36f,.37f,-.59f,.46f,.41f,-.86f,-.10f,-.42f,.66f,.11f,.50f,.17f,.78f,.92f,.93f,.64f,.61f,.53f,-.51f,.75f,-.89f,-.01f,.50f,-.27f,-.40f,.06f,.66f,-.02f,-.70f,-.13f,.03f,-.72f,-.64f,.67f,.12f,-.97f,.97f,-.74f,-.56f,-.11f,.81f,-.13f,-.32f,.06f,.10f,.77f,-.80f,-.68f,-.14f,-.53f,-.70f,-.18f,-.37f,-.61f,.12f,.65f,.40f,-.16f,-.92f,.92f,.94f,-.20f,-.24f,.33f,-.75f,.83f,.67f,-.10f,-.18f,.84f,-.23f,-.80f,.77f,-.80f,.60f,-.51f,.64f,-.63f,.89f,-.61f,.00f,.43f,.86f,.23f,-.51f,.05f,-.23f,.05f,.32f,.36f,.78f,.12f,-.52f,.52f,-.86f,-.37f,-.63f,.50f,-.82f,-.93f,.49f,-.20f,-.35f,.20f,-.03f,.09f,.01f,.26f,.14f,.82f,.15f,.26f,-.83f,-.22f,.36f,.26f,-.40f,.90f,.87f,-.96f,.87f,.73f,.67f,.08f,.46f,.59f,.59f,.82f,.99f,-.13f,.18f,-.04f,.60f,-.60f,.84f,-.80f,-.54f,.40f,-.57f,-.43f,-.08f,-.09f,.62f,.58f,.89f,-.38f,.07f,-.72f,-.15f,.20f,.34f,-.81f,-.04f,-.06f,.77f,-.95f,-.99f,-.05f,-.13f,-.37f,-.21f,-.31f,-.77f,-.82f,.67f,.71f,.03f,.98f,.46f,-.87f,.55f,-.14f,.14f,.26f,.05f,.31f,.67f,-.00f,-.49f,-.71f,-.75f,-.66f,.27f,-.29f,.04f,-.91f,.02f,-.68f,-.03f,-.47f,.34f,-.89f,-.98f,-.00f,.11f,-.52f,.28f,-.42f,.60f,.28f,.42f,.91f,-.09f,.23f,-.44f,-.39f,-.20f,.61f,.82f,-.43f,.57f,.15f,-.03f,-.71f,-.19f,.56f,.54f,.99f,-.59f,-.76f,-.90f,.05f,.00f,-.56f,-.85f,-.74f,.31f,-.92f,.38f,.13f,.44f,.94f,.95f,.35f,.18f,.35f,-.16f,.15f,.21f,-.90f,.80f,.50f,-.68f,-.20f,-.99f,.20f,.71f,.65f,.46f,.21f,.19f,-.64f,.79f,-.23f,.69f,-.71f,.29f,-.55f,.61f,-.31f,.18f,-.00f,.40f,-.89f,-.38f,-.08f,-.43f,-.87f,.53f,-.05f,.55f,.80f,-.60f,-.87f,-.15f,-.22f,-.55f,-.41f,-.43f,.69f,.32f,-.10f,-.29f,.26f,.88f,-.86f,-.16f,-.36f,-.94f,.47f,-.75f,-.15f,-.10f,.97f,.93f,.39f,.29f,.12f,.49f,-.38f,.22f,.24f,-.33f,.18f,.37f,-.80f,-.77f,.28f,-.26f,.05f,.60f,-.85f,-.14f,.90f,-.44f,.36f,-.85f,-.89f,.36f,-.93f,.44f,-.95f,-.18f,.62f,-.50f,-.58f,-.08f,.84f,.33f,-.96f,.92f,.41f,-.61f,.90f,.09f,-.70f,-.73f,-.64f,-.09f,-.09f,-.42f,-.90f,.61f,.71f,.82f,-.42f,.56f,.97f,.64f,-.49f,-.22f,-.94f,.24f,-.29f,-.88f,-.54f,.97f,-.76f,.66f,-.71f,-.64f,-.64f,-.53f,.20f,.23f,-.56f,-.62f,-.55f,.18f,.18f,-.77f,.19f,-.68f,.89f,.50f,.42f,-.10f,-.28f,.41f,-.39f,-.51f,.70f,-.98f,.44f,.04f,-.46f,-.63f,-.93f,-.31f,.41f,.35f,.53f,.77f,.60f,.26f,-.21f,.82f,-.92f,.92f,-.70f,-.10f,.64f,-.83f,-.55f,.61f,.77f,-.72f,-.30f,-.85f,.33f,-.60f,.19f,.86f,.70f,.77f,.21f,.86f,.04f,.22f,.95f,.57f,.36f,-.41f,.37f,-.17f,.29f,.70f,-.40f,-.39f,-.83f,-.11f,.84f,-.43f,-.48f,.13f,-.27f,-.75f,-.54f,.79f,.75f,-.85f,.90f,-.41f,.24f,.29f,.61f,.69f,-.03f,-.72f,-.36f,-.33f,-.32f,-.11f,.19f,-.84f,-.24f,-.77f,-.96f,.70f,.04f,.95f,-.97f,-.11f,-.24f,.69f,-.13f,.85f,.43f,-.41f,-.89f,.70f,.09f,-.63f,.57f,.80f,.56f,-.62f,.32f,-.28f,.54f,.14f,-.67f,-.24f,-.79f,.23f,-.95f,-.16f,-.85f,-.18f,.46f,-.60f,.36f,.22f,-.08f,.15f,.62f,-.20f,.88f,-.26f,-.13f,-.11f,.56f,.17f,.67f,-.56f,.49f,.87f,-.35f,-.73f,-.93f,-.80f,-.06f,-.95f,-.36f,-.26f,.34f,.19f,.57f,.56f,-.65f,.60f,.94f,.86f,-.16f,-.95f,.36f,.12f,-.42f,.34f,.75f,-.14f,.15f,.03f,-.17f,.70f,.95f,.95f,.79f,.11f,-.82f,-.30f,.75f,-.72f,-.91f,.31f,-.07f,.42f,.85f,.00f,-.47f,.37f,-.62f,.22f,.10f,.75f,-.15f,-.13f,.18f,-.21f,.45f,-.35f,.77f,-.40f,-.75f,.59f,.87f,-.62f,-.91f,.55f,-.09f,.06f,.30f,.52f,-.51f,-.03f,.89f,-.05f,.88f,.83f,-.66f,-.01f,.34f,-.83f,.60f,.47f,.65f,.51f,.70f,.62f,.99f,.34f,.41f,-.31f,.75f,.61f,.04f,.54f,-.42f,.47f,-.43f,.39f,-.58f,-.44f,-.02f,-.08f,.38f,-.77f,-.55f,-.20f,.63f,-.93f,-.96f,-.04f,-.26f,.83f,.34f,.31f,-.32f,.37f,-.74f,.60f,-.77f,-.04f,.65f,.94f,-.39f,.01f,-.76f,-.25f,.44f,.87f,.06f,-.58f,-.21f,.67f,-.97f,.22f,.24f,-.16f,.82f,-.19f,.66f,.00f,.50f,-.85f,.32f,-.66f,.11f,-.48f,.66f,-.60f,-.50f,.29f,-.93f,.94f,-.02f,.41f,-.13f,-.04f,.77f,.31f,-.23f,.84f,.25f,-.48f,-.51f,.19f,-.42f,-.03f,-.13f,.78f,.76f,.23f,.63f,.23f,-.82f,.41f,.10f,.73f,.49f,-.94f,.80f,-.40f,.78f,-.12f,-.49f,-.80f,.37f,.96f,.22f,-.93f,.78f,-.74f,-.71f,.46f,-.38f,.86f,.03f,.48f,.22f,-.05f,.13f,.91f,-.20f,.36f,.64f,.26f,.38f,-.10f,.57f,-.14f,-.72f,.08f,.55f,-.91f,.62f,.27f,-.49f,-.22f,.22f,-.12f,-.32f,-.28f,.15f,-.92f,.58f,-.51f,-.07f,-.45f,-.24f,-.73f,.63f,-.70f,-.11f,-.46f,-.75f,-.58f,-.15f,.38f,-.22f,.23f,-.26f,.41f,.14f,.87f,.44f,.26f,.79f,.28f,.19f,.06f,.84f,.19f,.91f,-.25f,.24f,.07f,-.46f,-.57f,.86f,.98f,-.33f,.77f,.04f,.55f,-.04f,-.72f,-.44f,.36f,-.73f,.74f,-.86f,.71f,.89f,-.94f,-.03f,.07f,.12f,-.14f,-.20f,-.71f,-.51f,-.23f,-.35f,-.89f,.75f,-.95f,-.48f,-.34f,-.16f,.45f,.34f,-.50f,-.82f,.12f,.18f,.35f,-.26f,-.16f,-.98f,.80f,.58f,-.65f,-.17f,.77f,.80f,-.06f,-.19f,-.43f,-.88f,.08f,.23f,-.70f,-.69f,-.76f,-.11f,.44f,-.87f,.70f,-.73f,-.55f,.05f,.30f,-.02f,.19f,.20f,.14f,.08f,.16f,.44f,.58f,-.51f,-.63f,.32f,.98f,-.48f,-.66f,-.00f,-.83f,.39f,-.95f,-.32f,-.28f,.60f,.59f,-.94f,.34f,-.68f,-.62f,.46f,-.87f,.25f,-.00f,.12f,.38f,.69f,-.54f,.11f,.03f,-.76f,.92f,.52f,-.33f,-.36f,-.33f,.46f,-.60f,.33f,-.56f,-.77f,.13f,.31f,-.65f,-.67f,-.58f,.54f,.41f,.80f,-.92f,.77f,-.44f,.69f,-.60f,.13f,-.82f,.13f,.27f,-.36f,-.88f,-.77f,-.72f,-.94f,.69f,.18f,-.29f,-.12f,-.81f,-.51f,.72f,-.03f,.32f,-.25f,-.83f,.11f,.14f,-.76f,-.62f,.51f,.87f,.76f,-.23f,.52f,.89f,-.02f,-.57f,.81f,-.83f,-.10f,-.88f,.71f,.30f,-.66f,-.38f,-.56f,.37f,.02f,.08f,-.88f,-.86f,.01f,-.61f,-.44f,.60f,.34f,-.90f,-.50f,-.11f,.52f,-.76f,.04f,-.53f,-.10f,.03f,.07f,.32f,.98f,.28f,.13f,.05f,.62f,-.15f,-.39f,.04f,-.81f,.84f,-.48f,.56f,-.74f,.39f,.79f,-.40f,-.67f,-.52f,.67f,-.68f,-.55f,-.65f,.36f,.27f,-.14f,-.14f,-.58f,.34f,.14f,-.33f,-.67f,.92f,-.73f,-.29f,.96f,-.43f,-.85f,.96f,.65f,.21f,.76f,-.74f,.42f,-.78f,-.06f,.21f,-.03f,-.83f,-.42f,.44f,-.79f,-.55f,-.22f,-.54f,-.24f,.68f,.03f,-.90f,.81f,.68f,.70f,-.84f,-.41f,-.26f,-.67f,.21f,-.49f,.00f,-.99f,.73f,.96f,-.50f,.82f,.83f,-.41f,.83f,-.48f,-.20f,.84f,-.79f,-.93f,.89f,-.24f,.59f,-.10f,.95f,.43f,.44f,-.81f,.61f,-.49f,-.78f,-.97f,.83f,-.33f,.26f,.46f,-.74f,-.64f,.49f,.38f,.92f,.70f,.51f,-.64f,.57f,.55f,.82f,-.91f,-.75f,-.50f,-.39f,-.79f,-.96f,-.72f,-.96f,.78f,-.81f,.07f,-.08f,-.34f,-.90f,.26f,.88f,-.69f,-.64f,-.41f,.46f,.93f,-.60f,-.81f,-.11f,-.50f,-.30f,-.27f,.59f,-.02f,-.71f,-.78f,.72f,.11f,-.84f,-.84f,.52f,.45f,-.90f,-.64f,-.38f,.32f,-.20f,.20f,.33f,-.61f,.79f,.33f,-.75f,-.21f,-.04f,.65f,-.27f,.59f,.60f,-.49f,-.38f,.72f,.87f,-.36f,-.97f,.13f,-.62f,-.86f,-.91f,-.52f,.45f,-.41f,-.38f,-.29f,-.07f,-.69f,-.86f,.46f,.60f,.37f,.14f,-.12f,-.75f,.64f,-.74f,.00f,-.91f,-.44f,-.75f,.94f,.91f,-.12f,.76f,.25f,-.75f,-.42f,-.71f,.41f,.55f,-.99f,.12f,.83f,.98f,-.45f,.16f,-.89f,-.38f,-.36f,-.89f,.36f,.27f,.88f,.64f,.31f,-.51f,.48f,-.31f,-.94f,.83f,-.51f,.51f,-.32f,.45f,-.34f,.62f,-.88f,.37f,.48f,-.25f,-.87f,-.40f,-.35f,-.60f,-.48f,-.62f,-.14f,.16f,-.60f,-.95f,.70f,.57f,.38f,-.42f,-.76f,-.37f,-.13f,.67f,.46f,-.54f,.62f,.87f,.93f,-.72f,.56f,.83f,-.24f,-.34f,-.52f,.65f,-.64f,.10f,-.02f,-.63f,-.59f,-.90f,-.98f,.28f,.52f,-.41f,.27f,-.97f,.53f,.29f,-.58f,-.44f,.87f,.32f,-.02f,-.49f,.33f,.61f,.93f,-.95f,.65f,.37f,-.42f,-.82f,.95f,-.96f,.72f,-.14f,-.07f,.03f,.20f,.87f,.52f,-.40f,-.72f,-.95f,.49f,-.29f,.17f,.93f,.04f,-.55f,.37f,.92f,-.99f,.94f,.24f,-.90f,-.60f,-.87f,.48f,.19f,-.27f,.12f,-.50f,-.97f,.85f,-.02f,.68f,.03f,.30f,-.53f,.66f,.38f,.72f,.12f,-.62f,-.46f,-.61f,-.89f,.24f,-.80f,.13f,-.52f,.45f,-.02f,.13f,.03f,.02f,.22f,.50f,.70f,.46f,.40f,-.00f,.49f,-.96f,.75f,-.31f,.60f,.06f,-.57f,.20f,-.79f,-.70f,.08f,.71f,.89f,.75f,-.65f,.50f,-.89f,.15f,.05f,.95f,.77f,-.03f,-.52f,.52f,.60f,.00f,-.92f,.34f,.38f,-.69f,.26f,-.74f,.29f,.39f,.18f,-.06f,.19f,-.67f,-.79f,.96f,-.63f,-.55f,-.86f,-.29f,.68f,-.57f,-.10f,-.87f,.22f,.70f,.09f,.17f,.27f,-.30f,-.53f,.50f,.80f,.07f,.09f,-.85f,.07f,.52f,-.71f,-.71f,.13f,-.57f,.39f,-.45f,.45f,.93f,-.82f,.09f,.94f,-.77f,-.58f,-.46f,-.40f,.47f,-.85f,-.92f,-.01f,-.35f,.33f,-.74f,.79f,.10f,-.24f,.07f,.97f,-.67f,-.69f,-.64f,-.31f,-.19f,.39f,-.73f,.18f,.10f,.14f,-.97f,-.37f,-.36f,-.33f,.64f,-.16f,-.60f,-.78f,.84f,-.77f,-.40f,.68f,-.49f,.13f,.39f,.98f,-.35f,.05f,-.97f,-.12f,.93f,.88f,.33f,.02f,.17f,.80f,-.71f,.98f,.92f,-.97f,.75f,.48f,.22f,.74f,-.51f,.67f,-.87f,.44f,.84f,.28f,-.04f,.41f,.33f,-.68f,.52f,.55f,-.81f,-.63f,.57f,-.48f,-.29f,-.51f,.76f,-.78f,-.39f,-.15f,-.24f,-.01f,-.37f,-.87f,.64f,.07f,.19f,-.96f,.77f,.48f,.95f,.27f,-.55f,-.47f,.00f,-.99f,-.89f,-.31f,.05f,-.08f,-.56f,.30f,.41f,-.63f,.44f,.80f,.11f,.14f,-.62f,-.41f,-.94f,-.64f,.44f,-.49f,-.89f,.42f,-.82f,-.97f,-.83f,-.41f,.68f,.89f,.40f,.09f,.28f,.09f,.01f,.32f,.33f,-.31f,.29f,-.57f,.26f,.70f,.28f,.34f,.12f,-.39f,.77f,.83f,.30f,.52f,-.97f,.80f,-.94f,-.53f,-.33f,-.38f,-.25f,-.20f,-.22f,.37f,-.64f,.95f,-.26f,-.46f,-.97f,-.86f,-.73f,-.19f,.62f,.45f,.33f,-.14f,.87f,-.41f,.01f,-.67f,-.14f,.59f,-.29f,.20f,-.33f,.19f,.50f,.68f,.60f,.21f,.91f,.29f,-.77f,.48f,.54f,-.72f,.26f,.94f,-.82f,.26f,.08f,-.69f,.20f,.66f,.65f,-.77f,.22f,-.17f,-.31f,.56f,.86f,.53f,-.06f,.73f,-.52f,.27f,.99f,-.73f,-.31f,-.09f,.74f,-.15f,-.14f,-.70f,.86f,-.83f,-.02f,-.18f,-.53f,.53f,-.23f,.94f,-.23f,-.32f,.23f,-.73f,-.43f,.06f,-.75f,-.88f,.43f,.50f,-.74f,-.17f,.03f,.74f,-.18f,-.97f,-.09f,.88f,.63f,-.88f,-.42f,.03f,-.19f,.54f,-.26f,.30f,-.13f,.74f,.36f,.05f,-.31f,.77f,-.24f,-.95f,-.52f,.95f,.23f,.87f,-.09f,.08f,.24f,.16f,.31f,.88f,.48f,-.12f,.30f,-.42f,-.10f,.44f,-.91f,-.43f,-.95f,.62f,.36f,.91f,-.74f,.74f,-.08f,.55f,.12f,-.14f,.82f,.22f,.77f,-.11f,.81f,.29f,-.52f,-.47f,.75f,-.08f,.23f,.45f,.87f,.48f,-.41f,.09f,.46f,.71f,.73f,.23f,-.87f,-.90f,-.38f,-.72f,.23f,.36f,.64f,-.87f,-.82f,-.97f,.96f,.97f,-.63f,-.92f,.73f,.29f,-.54f,-.20f,-.96f,-.15f,.98f,-.50f,-.42f,.29f,-.21f,.71f,-.71f,.58f,.12f,-.61f,-.37f,-.28f,-.00f,.66f,-.80f,-.29f,-.83f,-.15f,.70f,.27f,-.37f,.02f,-.28f,.85f,.14f,.26f,.06f,-.72f,.07f,.33f,.18f,-.56f,.66f,.51f,-.21f,.41f,-.69f,-.38f,.42f,-.04f,.71f,.66f,-.26f,.28f,.74f,.61f,.22f,.47f,-.44f,.49f,-.26f,.61f,-.24f,.07f,-.63f,-.24f,-.44f,-.41f,.74f,-.27f,.60f,.47f,-.69f,.13f,-.10f,.36f,-.54f,-.05f,-.26f,.50f,-.62f,.19f,-.05f,.56f,-.12f,.33f,.60f,-.50f,-.40f,.12f,-.06f,.95f,.06f,.60f,.00f,.32f,-.73f,-.05f,.42f,.24f,-.51f,.37f,-.86f,.49f,-.35f,.07f,.21f,-.03f,.13f,-.03f,-.19f,.48f,-.29f,-.56f,-.35f,.57f,-.98f,-.21f,.50f,.95f,.51f,-.40f,.77f,.26f,.77f,.71f,-.28f,.98f,-.78f,.55f,-.96f,-.30f,-.06f,.72f,.79f,.32f,.46f,-.42f,.38f,-.22f,.82f,.65f,.63f,-.92f,-.99f,.07f,-.28f,-.48f,-.22f,.29f,.61f,.04f,.76f,.77f,-.89f,-.84f,-.18f,.27f,-.14f,.13f,.75f,-.53f,.88f,-.46f,.22f,-.57f,-.19f,-.11f,-.96f,.16f,.69f,.37f,.06f,.59f,.21f,-.74f,.53f,.74f,.68f,-.60f,.81f,-.74f,-.82f,-.84f,-.64f,-.09f,.31f,.54f,.34f,.86f,.30f,-.72f,.25f,.51f,.62f,.47f,-.67f,.33f,-.68f,.63f,.84f,.91f,-.85f,-.24f,-.48f,-.40f,.29f,.24f,-.76f,.27f,.60f,-.51f,-.27f,-.99f,.58f,.87f,.58f,-.42f,.18f,.97f,-.42f,-.96f,-.30f,-.98f,-.05f,.98f,.35f,-.82f,-.55f,.87f,.13f,.13f,-.14f,-.61f,-.75f,-.58f,.24f,-.33f,-.79f,.95f,.19f,.01f,.54f,-.40f,-.60f,.08f,-.59f,.71f,-.98f,-.94f,.73f,.46f,-.54f,.23f,.49f,-.26f,.26f,.63f,-.32f,-.34f,-.14f,.01f,.68f,-.03f,.53f,.40f,-.90f,.70f,-.13f,.31f,.12f,-.57f,.52f,-.55f,.98f,-.71f,-.75f,-.71f,-.23f,.54f,.19f,.78f,-.41f,-.81f,-.34f,-.03f,-.78f,-.71f,-.98f,.69f,.08f,.16f,.64f,-.23f,-.71f,.35f,-.53f,-.45f,-.01f,-.99f,.38f,-.20f,-.62f,.56f,.82f,.98f,.26f,.00f,.36f,.29f,-.78f,-.88f,.46f,.00f,.13f,-.22f,.73f,-.42f,.67f,-.75f,.76f,.72f,.95f,.46f,-.81f,-.98f,-.38f,-.05f,-.02f,.77f,-.53f,-.53f,-.68f,-.98f,.07f,-.08f,.40f,-.59f,-.03f,-.13f,.70f,.07f,-.82f,.06f,-.09f,.66f,.15f,.73f,.39f,-.84f,-.33f,-.48f,-.81f,-.01f,-.43f,.42f,.55f,.31f,-.78f,-.15f,-.04f,.81f,.26f,-.38f,.00f,.54f,.21f,-.60f,-.75f,.92f,-.60f,-.51f,.95f,-.37f,-.81f,.44f,.68f,.71f,-.37f,.61f,.29f,-.27f,-.96f,-.14f,-.06f,-.74f,-.09f,.51f,-.89f,.47f,-.75f,.40f,.50f,.45f,-.68f,.77f,.31f,-.50f,-.45f,.52f,.97f,.05f,-.91f,.61f,-.36f,-.26f,-.36f,.47f,-.34f,.97f,-.45f,.19f,-.08f,.39f,.35f,-.28f,.81f,-.73f,.37f,-.39f,.15f,-.79f,-.73f,.37f,-.65f,.72f,-.47f,.57f,-.57f,-.82f,-.87f,-.62f,-.92f,.92f,.85f,-.32f,.04f,-.79f,.96f,.77f,.13f,-.19f,.44f,-.10f,-.65f,-.49f,.33f,.72f,.96f,.13f,-.66f,-.78f,.76f,-.44f,-.69f,.87f,.67f,-.34f,.87f,.92f,.02f,-.28f,-.21f,-.56f,.36f,.96f,.53f,-.19f,-.15f,-.73f,.39f,.53f,-.78f,.26f,-.56f,.17f,.69f,.20f,-.08f,.45f,.19f,.51f,-.96f,-.42f,.59f,.16f,.48f,-.75f,-.71f,.11f,-.24f,-.31f,-.67f,-.96f,-.05f,-.53f,.62f,.81f,.37f,.07f,.80f,.40f,-.38f,-.84f,-.57f,.96f,-.23f,.62f,-.88f,.49f,.49f,.37f,.82f,-.83f,-.00f,.93f,-.25f,.13f,-.69f,.87f,.94f,.71f,.58f,.51f,.97f,-.44f,-.76f,.48f,.01f,.11f,-.57f,.53f,-.85f,-.45f,.31f,-.02f,-.27f,.39f,.86f,-.42f,-.67f,-.34f,.39f,.42f,-.08f,.25f,.01f,-.45f,-.11f,.65f,-.89f,-.44f,.54f,.66f,.53f,-.96f,-.09f,.57f,.20f,-.16f,.90f,-.02f,.92f,.67f,-.06f,.51f,-.54f,-.79f,.08f,.89f,-.96f,.77f,-.83f,-.72f,.95f,.06f,.38f,.80f,.20f,.17f,-.30f,.76f,-.39f,-.81f,-.59f,-.66f,.35f,.01f,-.53f,.07f,.94f,.88f,.97f,-.18f,.14f,.41f,-.83f,-.00f,.59f,-.05f,.03f,.84f,-.17f,-.32f,-.79f,-.86f,-.55f,-.09f,.57f,.01f,.80f,-.11f,.68f,.92f,-.57f,.60f,-.07f,-.10f,.66f,.47f,.62f,.11f,-.50f,.01f,-.03f,.84f,-.15f,-.45f,-.07f,.19f,.94f,-.10f,-.46f,-.47f,-.78f,.05f,.70f,-.96f,.88f,-.84f,.72f,-.40f,.46f,.44f,.31f,.10f,.36f,.42f,.42f,-.29f,-.87f,.16f,-.24f,.62f,.05f,.13f,-.11f,.80f,-.23f,.18f,.43f,-.91f,.75f,.38f,.84f,.61f,-.83f,-.34f,-.01f,.05f,-.39f,.05f,-.64f,.62f,-.29f,.56f,-.06f,.23f,-.39f,.18f,.44f,.04f,.36f,.89f,.93f,.24f,.42f,.25f,-.45f,-.63f,-.22f,.09f,-.27f,.27f,.28f,-.12f,-.97f,-.20f,.55f,-.43f,-.35f,-.64f,.63f,.42f,-.93f,-.76f,-.98f,.90f,.80f,-.64f,-.92f,.34f,-.81f,.79f,-.77f,.56f,.60f,-.05f,-.63f,-.32f,.51f,-.52f,-.20f,.38f,.80f,-.63f,-.52f,-.13f,-.06f,.01f,-.66f,.14f,-.45f,.78f,.07f,.00f,.17f,-.46f,.47f,-.10f,.31f,-.16f,-.62f,-.32f,-.85f,.24f,-.01f,-.63f,.80f,.92f,.54f,-.85f,-.74f,-.93f,-.98f,.54f,-.23f,-.36f,.20f,.75f,.22f,.71f,-.07f,-.13f,.40f,-.82f,.46f,-.39f,.34f,-.35f,.12f,-.42f,.63f,.68f,.04f,.97f,-.04f,-.38f,.87f,.12f,.02f,-.71f,-.81f,-.43f,.56f,.80f,.88f,.51f,-.78f,-.52f,.35f,-.52f,.21f,-.51f,.19f,-.05f,.64f,.72f,-.69f,.45f,-.94f,.41f,-.80f,-.17f,.67f,-.41f,.08f,.15f,-.55f,.13f,-.25f,-.85f,.77f,.47f,-.44f,.39f,-.71f,-.62f,.65f,.35f,.20f,.25f,.62f,-.53f,-.97f,-.36f,-.12f,-.93f,-.46f,.39f,.07f,.06f,-.88f,.07f,.01f,-.95f,-.02f,.52f,.41f,-.88f,.55f,.99f,.11f,.76f,-.90f,.97f,-.25f,-.92f,.65f,.94f,.71f,-.82f,-.46f,.44f,.21f,.84f,-.49f,.55f,-.20f,.45f,.48f,-.58f,.96f,.52f,.01f,.31f,-.67f,.52f,-.66f,-.77f,.18f,.36f,-.32f,.51f,.23f,.58f,-.45f,-.52f,.22f,-.46f,.79f,.49f,.40f,.81f,-.44f,.79f,-.50f,.90f,.13f,.34f,.93f,.31f,.56f,-.83f,.90f,-.18f,.55f,-.05f,-.82f,.16f,.67f,.08f,-.82f,-.04f,-.05f,.69f,.75f,.74f,-.37f,.09f,.82f,-.32f,.16f,.67f,.50f,.37f,.63f,-.65f,.28f,.34f,.09f,.41f,-.64f,.57f,.80f,-.11f,-.43f,-.16f,-.29f,-.79f,-.52f,-.54f,-.22f,-.94f,-.77f,.36f,.75f,-.42f,.71f,-.53f,-.23f,-.26f,-.23f,-.19f,-.13f,.85f,-.15f,.70f,-.27f,-.43f,.63f,.13f,.95f,-.63f,-.28f,-.42f,-.91f,-.42f,.79f,-.05f,-.43f,.24f,.39f,.22f,.63f,-.40f,.12f,.10f,-.23f,-.86f,-.12f,-.42f,.31f,-.87f,-.38f,.04f,.54f,.37f,-.51f,.09f,.58f,.27f,.27f,.17f,-.16f,.36f,.35f,.88f,-.79f,.34f,.34f,-.51f,-.15f,-.84f,.07f,.46f,.46f,.44f,-.70f,-.41f,.98f,.44f,.66f,.31f,.84f,.95f,-.25f,-.52f,-.07f,-.78f,-.72f,-.55f,-.84f,-.38f,-.77f,.90f,-.08f,-.62f,.74f,-.78f,-.46f,-.71f,.80f,.21f,.66f,.19f,.80f,.05f,.83f,.28f,.46f,.75f,.33f,.31f,-.62f,-.20f,.13f,-.79f,-.17f,.32f,-.26f,-.48f,.40f,.93f,-.08f,.49f,.71f,.34f,-.05f,.62f,.30f,.22f,.12f,.60f,-.02f,-.56f,.39f,-.15f,.78f,.80f,-.11f,-.61f,-.66f,-.25f,-.80f,-.44f,-.39f,-.99f,.05f,.57f,-.36f,.82f,-.74f,.37f,-.50f,.68f,-.37f,.56f,.80f,-.50f,.45f,-.50f,.58f,-.01f,.79f,-.48f,.05f,.97f,.32f,.82f,.46f,.49f,-.12f,-.71f,-.04f,-.65f,.60f,-.81f,.78f,-.85f,.90f,-.95f,-.46f,.62f,.36f,-.43f,.10f,.57f,.52f,-.60f,-.05f,-.11f,.12f,.32f,.52f,.91f,.53f,.24f,.04f,-.44f,-.41f,.89f,-.17f,-.45f,-.83f,.28f,-.70f,.46f,.58f,.81f,-.23f,-.97f,-.61f,-.82f,.83f,.45f,.74f,-.55f,-.51f,-.74f,.56f,.00f,.23f,.76f,-.30f,.07f,.91f,.26f,.82f,.32f,-.40f,.82f,.64f,-.83f,-.99f,-.19f,.34f,.09f,.31f,-.47f,.38f,.33f,.68f,.71f,-.20f,-.86f,-.10f,-.53f,-.04f,-.99f,.35f,.49f,.52f,.81f,-.51f,.76f,-.50f,-.31f,-.93f,-.89f,-.88f,.97f,.22f,-.41f,.25f,-.28f,-.29f,-.60f,-.23f,-.75f,-.39f,.81f,.17f,.18f,-.92f,-.94f,.82f,-.83f,-.71f,.77f,-.65f,.15f,.17f,.09f,.23f,.21f,.05f,.14f,-.56f,-.31f,.32f,.45f,-.41f,.40f,-.01f,-.59f,-.07f,.96f,.15f,.98f,-.32f,-.66f,.58f,-.07f,.01f,-.67f,-.17f,-.37f,.76f,-.57f,.18f,-.97f,-.14f,.78f,.13f,.78f,-.58f,.33f,.45f,-.31f,.29f,.50f,-.20f,-.78f,-.82f,-.55f,.46f,.55f,-.73f,-.63f,.01f,.12f,.62f,.58f,.95f,.08f,-.94f,-.90f,-.70f,-.22f,.57f,.69f,.80f,.52f,-.38f,-.38f,.59f,.17f,.53f,.46f,-.55f,.81f,.59f,.54f,.57f,-.38f,.07f,-.23f,.16f,.18f,.57f,-.94f,.95f,-.83f,.75f,-.75f,-.97f,-.57f,-.61f,-.15f,-.12f,-.67f,.72f,-.90f,-.26f,.39f,.65f,-.34f,-.43f,.13f,-.23f,-.22f,-.80f,.60f,-.54f,.31f,-.98f,-.12f,.48f,.94f,-.96f,.42f,-.33f,-.89f,-.03f,.94f,.04f,.36f,-.75f,.28f,.28f,-.77f,.32f,-.33f,-.09f,-.30f,.96f,-.30f,-.17f,.54f,.48f,.61f,-.50f,-.01f,.49f,-.88f,-.28f,.50f,.66f,.03f,-.19f,-.01f,-.44f,-.73f,-.11f,.07f,.71f,.05f,-.38f,-.50f,-.47f,-.92f,.75f,.48f,-.46f,-.08f,-.15f,.59f,.32f,.87f,.87f,.80f,-.79f,.20f,-.08f,-.64f,.45f,-.26f,.42f,-.95f,.03f,.91f,-.11f,.14f,.63f,.55f,.59f,.04f,.79f,.30f,-.13f,.20f,.48f,-.07f,-.69f,.29f,-.02f,.45f,-.61f,-.52f,.46f,-.28f,.81f,-.93f,-.73f,.43f,-.96f,.45f,.09f,.51f,.56f,-.39f,.20f,-.18f,.16f,-.28f,.02f,.05f,-.45f,.31f,-.61f,.82f,-.92f,-.94f,-.20f,.10f,.44f,-.02f,-.84f,.16f,.08f,-.66f,.06f,.72f,.09f,-.91f,.70f,-.96f,.53f,-.30f,.89f,.76f,.11f,.61f,.89f,-.02f,-.65f,-.98f,-.93f,-.66f,-.11f,-.97f,.52f,-.75f,-.15f,.30f,.39f,.32f,.49f,-.72f,.44f,.62f,.64f,-.76f,.63f,-.18f,.29f,-.74f,-.10f,-.35f,-.11f,-.42f,.29f,-.07f,.06f,-.26f,-.91f,.85f,-.31f,-.73f,.84f,.89f,.46f,.96f,.92f,-.76f,.87f,-.83f,-.57f,.42f,-.79f,-.55f,-.32f,-.14f,.36f,-.37f,.14f,.17f,.60f,.48f,-.66f,.24f,-.27f,.30f,-.76f,-.73f,-.14f,-.86f,-.43f,.93f,.61f,-.03f,.70f,.74f,-.91f,.45f,-.78f,-.07f,-.60f,-.57f,.40f,.51f,.53f,-.02f,.27f,.04f,.94f,.49f,-.16f,-.94f,-.16f,.41f,-.45f,.31f,.56f,.25f,-.56f,.87f,.15f,-.83f,-.66f,-.69f,-.16f,.32f,-.61f,.76f,-.59f,-.69f,-.70f,.42f,.65f,.90f,-.45f,.59f,-.25f,-.94f,.74f,-.87f,-.24f,.69f,-.85f,.33f,-.89f,.35f,-.29f,-.67f,-.28f,-.85f,-.91f,-.68f,.38f,.48f,-.21f,-.00f,-.39f,.93f,.00f,-.43f,-.25f,.67f,.82f,.00f,.27f,-.76f,.44f,-.83f,-.61f,-.80f,.98f,-.07f,-.86f,.03f,.69f,-.88f,-.87f,-.65f,.88f,.61f,.32f,-.82f,.76f,-.57f,-.59f,.02f,-.80f,.34f,-.98f,-.25f,-.34f,.83f,.40f,-.73f,-.63f,-.73f,-.59f,-.15f,-.19f,.07f,-.21f,-.87f,-.52f,.03f,.69f,-.73f,.99f,.25f,-.28f,.50f,-.17f,.81f,.70f,-.62f,.19f,-.64f,-.14f,.87f,.37f,-.98f,-.33f,-.41f,-.21f,.51f,.84f,-.93f,-.44f,.40f,.86f,-.39f,-.46f,.01f,-.06f,.74f,.43f,-.97f,.69f,-.14f,.06f,.46f,.72f,.48f,-.33f,.91f,.85f,.07f,.51f,.34f,-.16f,.46f,.88f,-.24f,.17f,.55f,-.61f,.16f,-.32f,.02f,.53f,.91f,.04f,.26f,.33f,-.64f,-.50f,.32f,.55f,-.47f,.37f,.69f,.09f,-.29f,.75f,-.19f,.93f,.92f,.65f,.85f,-.45f,.17f,-.36f,-.08f,-.01f,.08f,.69f,-.98f,-.46f,-.32f,-.66f,.98f,.08f,-.71f,-.98f,-.51f,-.95f,.23f,.26f,-.35f,-.72f,.69f,-.64f,-.13f,-.23f,.95f,-.74f,-.25f,.34f,.36f,.25f,-.51f,.00f,-.60f,-.92f,.81f,-.84f,.85f,-.55f,.26f,.20f,.18f,-.78f,-.27f,.99f,-.36f,-.97f,.76f,.10f,-.80f,.16f,-.90f,.48f,-.38f,-.15f,.97f,.75f,.17f,.60f,.15f,-.76f,-.58f,.72f,.24f,.16f,-.37f,.07f,.45f,-.29f,.17f,-.30f,-.37f,-.15f,.43f,-.90f,-.33f,.90f,-.29f,.53f,.96f,.83f,.11f,.28f,-.70f,.87f,-.26f,.90f,.17f,.89f,-.25f,-.92f,.62f,.10f,.14f,-.91f,-.59f,-.54f,-.34f,.10f,.25f,.69f,-.60f,.57f,.82f,-.12f,-.97f,-.72f,.34f,-.43f,-.02f,-.16f,-.49f,-.10f,-.22f,-.95f,-.29f,.15f,.15f,.55f,-.55f,.89f,-.28f,-.15f,-.22f,-.63f,.28f,.64f,-.82f,-.09f,-.14f,.96f,-.07f,-.58f,-.29f,.77f,.18f,.47f,-.33f,-.14f,-.80f,-.80f,-.52f,.52f,-.37f,-.09f,.13f,.85f,.01f,.25f,.47f,-.67f,.29f,-.52f,.92f,.87f,-.25f,-.87f,.67f,.44f,-.85f,.81f,-.29f,-.51f,-.11f,-.86f,-.92f,.90f,.87f,.57f,.45f,.74f,.02f,.81f,-.50f,-.32f,.24f,-.70f,-.71f,-.30f,.71f,.56f,.60f,-.64f,.56f,-.88f,.75f,-.04f,.90f,.40f,-.39f,-.16f,.99f,-.55f,.55f,.77f,-.76f,.25f,-.18f,-.78f,.71f,.05f,-.03f,.80f,-.55f,-.13f,.63f,-.33f,.18f,-.66f,.27f,-.83f,-.21f,-.79f,-.46f,-.15f,-.61f,-.28f,-.84f,.58f,-.43f,.85f,-.22f,.54f,.73f,-.87f,-.48f,.53f,-.97f,-.93f,.85f,-.96f,.37f,.78f,.61f,.55f,.43f,-.75f,.09f,-.06f,-.23f,.68f,-.42f,-.88f,-.81f,-.36f,.33f,-.50f,.63f,-.74f,-.61f,-.29f,.14f,.78f,-.19f,-.13f,-.73f,.97f,.87f,-.15f,-.62f,-.16f,.83f,-.78f,.87f,.48f,.32f,.77f,.59f,-.02f,.06f,.72f,.84f,.18f,-.24f,-.53f,.26f,.56f,.50f,.62f,-.29f,-.08f,-.39f,-.29f,-.36f,-.98f,.53f,.53f,-.21f,-.08f,.47f,-.77f,-.73f,-.48f,.84f,.12f,.09f,-.22f,.77f,-.10f,-.69f,.10f,.13f,-.42f,.96f,-.67f,-.78f,.10f,.66f,.85f,.52f,.89f,.50f,.38f,-.03f,.75f,.29f,.53f,-.71f,.99f,-.58f,.75f,.74f,-.66f,.50f,.76f,-.73f,-.84f,-.86f,.71f,.07f,-.82f,-.43f,.96f,.94f,-.36f,-.27f,.43f,.03f,-.60f,-.35f,.41f,-.30f,.06f,-.51f,.49f,-.34f,.99f,-.86f,.99f,-.03f,-.43f,-.27f,-.72f,-.92f,.17f,-.78f,.43f,.93f,.12f,-.15f,.73f,.88f,.44f,-.58f,-.20f,.27f,.45f,.40f,.02f,.94f,-.68f,.88f,-.76f,-.16f,.08f,.92f,-.88f,.03f,-.66f,-.59f,-.05f,-.95f,-.54f,.97f,.94f,-.84f,.24f,-.03f,-.14f,.58f,-.75f,.38f,.00f,-.90f,-.34f,-.70f,-.79f,-.33f,.48f,.92f,-.68f,-.88f,.99f,-.58f,.32f,.81f,-.57f,.23f,-.70f,-.26f,.49f,-.13f,.93f,.66f,-.31f,-.58f,.56f,-.57f,-.38f,.59f,.10f,.10f,-.55f,-.41f,.13f,-.22f,-.53f,.30f,.39f,.66f,.93f,-.38f,.93f,.35f,-.47f,.21f,.14f,.86f,.71f,-.51f,-.99f,.72f,-.37f,-.24f,-.04f,.19f,.03f,-.18f,.38f,-.68f,.88f,.98f,-.57f,.26f,-.18f,.34f,.99f,.04f,-.05f,.84f,-.83f,.45f,-.93f,-.94f,-.65f,.91f,-.23f,.34f,-.34f,-.11f,.10f,.96f,.22f,.59f,.67f,.73f,-.05f,.34f,-.69f,-.74f,-.32f,-.47f,.96f,-.38f,-.77f,-.04f,-.30f,-.29f,-.82f,.22f,-.05f,-.56f,.10f,-.78f,-.06f,-.14f,-.46f,-.11f,.66f,.83f,-.46f,-.26f,-.19f,-.55f,.28f,.00f,-.02f,.02f,-.64f,.14f,-.57f,-.49f,.07f,-.38f,.18f,.75f,-.97f,.20f,-.14f,.26f,.58f,-.31f,-.64f,.90f,.52f,-.49f,.98f,.78f,-.84f,-.36f,.64f,-.63f,.46f,-.95f,.05f,.27f,.65f,.69f,-.80f,-.31f,-.57f,.70f,-.34f,-.65f,-.46f,-.06f,-.12f,-.35f,-.03f,-.49f,.38f,-.07f,-.15f,-.46f,.65f,.70f,-.05f,-.98f,-.75f,.07f,-.61f,-.69f,.05f,-.39f,-.93f,-.93f,.66f,-.68f,.52f,.76f,-.24f,-.78f,-.06f,-.77f,.28f,-.65f,-.74f,.42f,-.04f,-.59f,.97f,-.96f,.87f,.30f,-.31f,-.24f,.99f,-.15f,-.06f,.87f,-.86f,.34f,-.68f,-.15f,.45f,-.83f,-.33f,.56f,-.53f,.45f,-.95f,.48f,.89f,.93f,-.74f,-.21f,.15f,-.17f,-.81f,-.07f,.99f,-.32f,-.66f,.64f,-.89f,-.74f,.14f,.04f,-.82f,-.74f,-.30f,.61f,.35f,-.72f,.52f,-.60f,-.44f,.42f,.58f,.29f,.91f,-.44f,.94f,.65f,-.32f,-.26f,-.02f,.79f,.45f,-.40f,.10f,.53f,-.11f,-.15f,-.99f,-.24f,.86f,.85f,-.42f,-.00f,.28f,.07f,-.86f,-.18f,.75f,-.38f,.16f,-.62f,.97f,.28f,.30f,.80f,.14f,.24f,.82f,-.25f,.55f,-.82f,.42f,-.39f,-.24f,-.58f,.54f,.27f,.95f,-.98f,-.30f,-.55f,.51f,-.52f,-.32f,-.50f,.49f,-.90f,-.15f,.38f,-.92f,.38f,-.48f,.39f,.45f,-.56f,-.96f,-.96f,.72f,-.25f,-.77f,-.31f,.59f,-.89f,.42f,.48f,.62f,-.11f,-.78f,.70f,.46f,.88f,-.32f,-.62f,-.87f,-.61f,.14f,.99f,.44f,.81f,-.66f,.24f,-.56f,.69f,-.97f,.76f,-.75f,-.00f,-.94f,.58f,-.32f,.82f,.74f,-.24f,-.88f,.30f,-.07f,.04f,.75f,-.70f,.70f,.23f,-.21f,-.51f,.36f,.63f,-.70f,.10f,.09f,.64f,-.51f,-.12f,-.23f,-.39f,.35f,.74f,-.58f,.39f,-.84f,.81f,-.93f,-.40f,.31f,-.81f,.41f,-.89f,-.27f,-.39f,.13f,-.51f,-.72f,-.55f,.79f,.49f,-.29f,.00f,-.92f,-.10f,-.77f,-.75f,-.33f,-.20f,.21f,.22f,-.31f,.14f,-.58f,.82f,.38f,.33f,-.01f,.54f,-.67f,.24f,.60f,.53f,.52f,.21f,-.73f,-.72f,-.60f,-.37f,-.34f,-.06f,.15f,-.63f,-.99f,-.28f,.04f,-.36f,-.28f,.35f,.88f,.24f,.52f,-.54f,.55f,-.91f,.36f,.59f,.48f,-.43f,-.54f,-.36f,.12f,.56f,-.72f,.54f,-.75f,-.77f,.12f,-.89f,-.30f,.93f,-.11f,-.18f,-.67f,-.15f,.02f,-.76f,-.51f,.45f,.79f,-.49f,.90f,-.98f,-.70f,.10f,.35f,-.22f,.66f,-.62f,.87f,.44f,-.55f,-.63f,.09f,-.42f,-.38f,.81f,.69f,-.56f,-.51f,.25f,.31f,.06f,-.99f,.45f,-.85f,.84f,.32f,.21f,.61f,-.44f,-.63f,.46f,-.87f,-.03f,.40f,-.52f,.89f,-.92f,-.14f,.30f,.39f,.30f,-.63f,-.01f,-.66f,-.02f,-.51f,.14f,.02f,.28f,.85f,.77f,-.55f,.69f,-.13f,-.72f,-.04f,-.35f,.05f,.40f,.49f,.27f,-.64f,-.46f,-.59f,-.33f,.43f,.53f,-.50f,-.60f,.28f,.82f,.51f,.90f,.48f,-.55f,-.08f,-.32f,.68f,-.93f,-.09f,-.85f,-.29f,-.92f,.15f,.84f,-.97f,.12f,.72f,.81f,-.88f,-.98f,-.26f,.85f,.56f,.95f,-.10f,.67f,-.14f,.03f,.25f,.72f,-.22f,.18f,-.72f,-.85f,.83f,.00f,.98f,-.34f,.35f,.62f,-.65f,-.66f,.78f,.09f,-.09f,-.98f,.89f,-.27f,-.54f,-.82f,.06f,-.12f,-.89f,.45f,-.07f,.73f,-.66f,-.00f,-.67f,.56f,-.41f,-.99f,.09f,.30f,-.30f,.76f,-.33f,.02f,.29f,-.36f,.69f,.83f,-.16f,.76f,-.36f,-.47f,-.22f,.18f,-.49f,.57f,-.02f,-.56f,-.93f,-.19f,-.98f,-.98f,-.17f,-.42f,.55f,.49f,-.37f,.02f,-.22f,-.61f,-.36f,.94f,-.65f,-.87f,.56f,-.93f,-.53f,-.23f,-.30f,.34f,.53f,-.55f,-.62f,-.86f,.22f,-.57f,.47f,.12f,-.93f,-.83f,-.90f,.77f,.36f,.73f,-.59f,.87f,.88f,-.54f,.88f,.39f,.42f,.50f,.68f,-.66f,.07f,.22f,-.39f,-.47f,.41f,-.79f,-.38f,.79f,-.09f,.37f,-.07f,.05f,.66f,-.54f,.45f,.39f,.84f,.72f,.12f,.30f,-.55f,.54f,.42f,-.14f,-.85f,-.56f,-.55f,.27f,.70f,.98f,-.28f,-.29f,.48f,-.02f,-.83f,-.86f,-.92f,.88f,-.11f,.53f,.06f,-.45f,.05f,.78f,.69f,.22f,.52f,.11f,.38f,-.52f,.47f,-.02f,-.43f,-.19f,-.39f,-.45f,.81f,.29f,-.97f,-.04f,.25f,-.29f,-.77f,.46f,-.62f,-.92f,.85f,-.89f,-.84f,.03f,.16f,.47f,-.18f,-.60f,.04f,.71f,-.99f,.57f,.12f,-.83f,-.26f,.29f,-.81f,-.48f,.91f,.01f,-.38f,.50f,.69f,.38f,.18f,.03f,-.64f,.19f,.10f,.76f,.27f,-.34f,-.28f,-.74f,-.26f,.28f,-.60f,.95f,.52f,.84f,-.20f,.97f,.93f,-.56f,-.29f,.87f,-.71f,.20f,.83f,-.83f,.69f,-.86f,.40f,.34f,.78f,-.50f,-.19f,.50f,-.78f,-.11f,.77f,-.80f,.95f,-.22f,.03f,-.68f,-.63f,.74f,.01f,-.70f,-.59f,-.64f,-.72f,.83f,.41f,-.65f,-.16f,-.29f,.55f,-.63f,.80f,-.85f,-.49f,.45f,-.54f,-.00f,-.36f,-.04f,.66f,-.14f,.15f,-.44f,.79f,-.44f,.81f,.90f,.65f,.27f,-.08f,.28f,-.88f,.89f,-.80f,-.96f,-.97f,.56f,-.72f,-.68f,-.26f,.91f,.43f,.93f,-.85f,-.97f,-.00f,.04f,-.68f,.76f,.61f,-.66f,-.08f,-.75f,-.95f,.84f,.47f,-.66f,-.92f,-.50f,.11f,-.99f,.22f,.20f,.00f,.33f,.14f,-.35f,.48f,.42f,.30f,.89f,-.93f,.88f,-.44f,.08f,.01f,.03f,-.14f,.71f,.24f,-.17f,-.68f,.17f,.53f,.79f,.97f,.32f,.58f,.40f,-.77f,-.77f,.14f,.38f,.69f,.73f,.88f,.81f,.67f,.13f,-.53f,-.13f,-.79f,.70f,.46f,.71f,.06f,-.42f,.96f,.24f,.02f,-.31f,-.18f,-.87f,-.70f,.05f,-.46f,-.59f,-.84f,.12f,.41f,-.90f,.45f,.11f,.02f,-.49f,-.26f,-.41f,.43f,-.34f,-.19f,-.41f,-.64f,-.20f,.22f,-.20f,-.05f,.33f,-.19f,.69f,-.85f,-.01f,.33f,.07f,.80f,-.45f,-.53f,-.57f,.88f,-.80f,.12f,.60f,-.31f,.11f,.33f,-.10f,-.09f,.21f,-.93f,-.44f,.18f,-.13f,-.33f,.17f,.38f,-.43f,.07f,.30f,-.10f,-.25f,.96f,-.23f,.91f,-.47f,.37f,.52f,-.96f,.33f,.37f,-.75f,-.91f,.33f,.11f,-.92f,-.98f,-.74f,-.26f,.04f,-.06f,.38f,-.73f,.27f,.88f,.25f,-.79f,.48f,.82f,.98f,-.97f,.21f,-.16f,.56f,-.03f,-.53f,-.13f,-.73f,-.31f,.37f,-.68f,-.36f,.42f,-.21f,-.02f,-.57f,.50f,.00f,-.87f,.28f,.27f,.98f,-.96f,.21f,.81f,-.33f,-.94f,-.15f,-.45f,.93f,-.36f,.23f,.64f,-.04f,-.13f,.40f,-.72f,.17f,.87f,-.60f,-.82f,-.20f,.32f,.92f,.99f,.28f,-.47f,.50f,.27f,.58f,-.98f,.36f,.90f,.45f,-.65f,.26f,.54f,-.54f,.35f,.07f,-.94f,.23f,.02f,.87f,.83f,.09f,-.83f,.53f,.44f,-.93f,.35f,-.86f,-.01f,.38f,.56f,-.65f,.72f,-.04f,.09f,.17f,.95f,-.64f,.91f,.34f,-.60f,-.40f,.73f,-.66f,-.65f,-.21f,-.17f,-.12f,.50f,.27f,-.68f,-.74f,-.27f,-.64f,-.75f,-.01f,-.50f,.47f,-.50f,-.08f,-.00f,-.79f,.99f,-.01f,-.69f,-.60f,-.95f,-.11f,.25f,-.52f,-.49f,.31f,.76f,-.83f,-.39f,-.42f,-.18f,.06f,-.04f,-.74f,-.27f,-.23f,.76f,.33f,.59f,-.72f,-.70f,-.04f,.63f,-.21f,.25f,.16f,-.03f,.77f,.03f,-.74f,.94f,-.91f,-.23f,.47f,.90f,.82f,-.58f,.77f,-.37f,.05f,-.51f,.09f,-.80f,.95f,.73f,.28f,.56f,-.74f,-.31f,.37f,.65f,.32f,-.50f,-.52f,-.68f,.78f,.82f,.94f,.27f,-.44f,-.05f,.39f,-.80f,-.88f,.42f,.72f,.02f,.26f,-.58f,.21f,-.90f,.00f,.36f,.77f,-.96f,-.18f,.99f,-.71f,.94f,-.81f,-.08f,-.64f,-.71f,-.84f,-.28f,.44f,.44f,.80f,-.17f,-.50f,.38f,.75f,.29f,.54f,-.56f,-.40f,.35f,.59f,.88f,-.87f,.03f,.88f,-.02f,.26f,.68f,.38f,.24f,.03f,.15f,.28f,.73f,.67f,.94f,-.04f,.18f,-.19f,.68f,-.91f,.41f,-.50f,-.36f,.03f,-.65f,.44f,-.73f,-.18f,.39f,.40f,-.53f,-.94f,.11f,.71f,.82f,-.17f,.65f,.60f,.93f,-.99f,-.11f,-.83f,-.31f,-.09f,-.08f,.91f,-.51f,-.25f,.52f,.54f,.23f,-.00f,.69f,.88f,.49f,-.50f,.99f,.82f,-.56f,-.68f,.18f,-.38f,-.97f,.27f,-.83f,.63f,.98f,-.59f,.43f,.07f,-.68f,-.00f,-.25f,-.69f,-.03f,.86f,.72f,.53f,-.95f,-.88f,.15f,.43f,-.61f,-.03f,-.56f,-.44f,-.35f,-.46f,-.21f,-.80f,-.63f,.39f,-.24f,-.91f,.24f,.32f,-.22f,.73f,-.24f,-.53f,-.22f,-.62f,-.39f,-.40f,.60f,.78f,-.11f,.32f,-.34f,.53f,.60f,.10f,-.59f,.31f,.77f,-.54f,.52f,.68f,.84f,-.55f,-.15f,.16f,.15f,-.64f,.73f,.77f,-.57f,-.05f,.36f,-.03f,.27f,.93f,-.88f,.82f,.40f,-.69f,.23f,.01f,.30f,-.21f,.09f,-.18f,.97f,.33f,.48f,-.15f,-.05f,-.86f,.81f,-.33f,-.92f,.58f,.47f,.22f,.81f,.77f,-.82f,-.15f,.82f,-.53f,.31f,.30f,-.16f,-.24f,-.24f,-.80f,.72f,-.91f,-.53f,-.73f,.85f,.82f,-.97f,-.80f,-.13f,-.20f,.40f,-.70f,-.35f,.88f,.08f,-.48f,.77f,.62f,-.16f,-.47f,.53f,.11f,.43f,-.36f,-.13f,-.01f,-.36f,.88f,-.89f,.03f,.69f,-.36f,-.98f,.52f,.28f,.59f,.27f,-.43f,-.75f,.61f,.67f,.46f,-.50f,-.11f,-.77f,-.43f,-.00f,-.80f,.91f,.22f,-.50f,-.83f,-.52f,.94f,-.92f,-.36f,-.96f,-.40f,.11f,-.37f,.08f,.52f,.53f,.02f,-.83f,.54f,-.48f,.02f,-.05f,.32f,-.78f,.02f,-.79f,.34f,-.54f,.93f,-.80f,.32f,.70f,-.82f,.92f,.04f,.83f,-.26f,-.68f,.76f,.78f,.35f,.71f,.83f,.52f,.57f,-.20f,.12f,.26f,-.99f,.47f,.60f,-.06f,-.17f,.26f,-.60f,.70f,-.22f,-.87f,-.78f,.29f,.47f,.76f,.09f,.73f,.07f,-.91f,-.53f,-.40f,.78f,.81f,-.57f,-.05f,-.54f,-.04f,-.14f,.80f,.11f,-.75f,.56f,.63f,-.73f,-.32f,.86f,-.98f,-.64f,.59f,-.95f,.63f,.10f,.24f,-.83f,-.60f,-.67f,.40f,-.96f,-.30f,-.69f,.70f,-.89f,.40f,-.21f,.83f,.44f,.71f,-.85f,-.05f,-.95f,-.83f,.26f,-.63f,.37f,-.05f,-.76f,-.47f,.37f,.01f,.42f,-.12f,.99f,-.43f,.16f,-.20f,-.17f,-.18f,-.13f,-.98f,-.05f,-.86f,.40f,.70f,-.68f,.93f,-.62f,.08f,-.54f,-.01f,.82f,.05f,-.45f,-.83f,-.22f,-.35f,.54f,.48f,.12f,.27f,-.87f,-.67f,-.15f,.02f,-.67f,.52f,.24f,.08f,-.80f,.09f,.99f,.31f,.01f,.83f,-.99f,-.90f,-.23f,.39f,.75f,-.51f,.75f,.37f,-.28f,.27f,-.72f,.71f,.78f,.95f,-.05f,.08f,-.10f,-.19f,.43f,.38f,-.69f,-.38f,.67f,.58f,.86f,.14f,.86f,-.06f,.41f,-.17f,-.15f,.96f,.10f,.20f,.22f,-.92f,-.51f,-.08f,.84f,.83f,-.90f,.51f,-.07f,.69f,.84f,.30f,.35f,-.01f,-.01f,.85f,.76f,.77f,-.50f,.67f,.65f,-.49f,-.47f,-.64f,.63f,-.12f,-.84f,.76f,.50f,.00f,-.83f,-.71f,.43f,-.10f,.95f,.86f,-.77f,.93f,-.61f,-.67f,-.18f,-.53f,-.12f,-.59f,.95f,-.97f,-.08f,-.20f,-.65f,-.78f,-.02f,-.47f,-.55f,-.77f,.00f,-.01f,-.97f,.29f,-.68f,.66f,-.86f,.70f,-.53f,-.99f,-.29f,-.87f,.34f,.13f,-.12f,-.64f,.65f,-.08f,.80f,-.37f,-.01f,.63f,-.85f,.37f,.23f,.06f,-.57f,-.99f,.28f,-.48f,.70f,.34f,.08f,.96f,-.27f,.36f,.31f,-.39f,-.22f,-.06f,-.67f,-.22f,-.49f,.77f,-.16f,.95f,-.03f,-.40f,.67f,.17f,.73f,.44f,-.69f,.78f,.57f,-.14f,-.61f,-.31f,.73f,-.71f,.03f,.95f,.33f,-.50f,.69f,-.11f,.81f,-.64f,.67f,.66f,-.47f,-.28f,-.11f,.11f,.97f,.67f,.98f,.38f,-.07f,-.38f,.12f,-.59f,.89f,.12f,.05f,-.03f,.58f,-.68f,-.74f,.57f,.81f,.23f,-.35f,-.27f,.29f,-.43f,.03f,-.60f,.18f,.07f,.74f,-.56f,.70f,-.11f,.28f,-.73f,.14f,-.60f,.00f,-.71f,-.38f,.39f,.93f,-.32f,.75f,-.39f,.63f,-.44f,-.10f,.10f,.83f,.56f,-.07f,-.25f,.75f,.96f,.34f,-.78f,-.95f,-.18f,.36f,.57f,-.82f,.44f,-.57f,.65f,.40f,-.02f,.97f,.31f,.06f,-.91f,-.80f,-.74f,-.29f,.46f,-.58f,.47f,.25f,-.06f,.28f,.98f,-.56f,-.13f,.90f,.28f,.99f,.78f,-.46f,.96f,-.34f,-.94f,.22f,.99f,-.48f,.18f,.97f,-.52f,.98f,.74f,-.18f,.04f,.98f,-.85f,-.36f,-.29f,-.18f,-.39f,-.89f,.49f,-.71f,.24f,.59f,-.85f,.58f,.86f,-.68f,.07f,.72f,-.32f,-.76f,.84f,.05f,.16f,.68f,-.76f,.22f,-.65f,-.18f,-.49f,-.17f,.70f,-.42f,.98f,.18f,.18f,.22f,.14f,.35f,.26f,-.36f,.98f,.84f,-.54f,.13f,.90f,.46f,.53f,-.94f,-.15f,-.14f,.77f,.68f,.16f,-.49f,-.43f,-.83f,-.33f,.14f,-.55f,-.39f,-.14f,.08f,-.78f,-.79f,-.65f,-.65f,.73f,.97f,.65f,.50f,-.51f,-.24f,-.88f,.59f,.74f,.03f,-.91f,.33f,.03f,-.66f,-.20f,-.13f,.94f,.46f,.41f,.50f,.45f,.47f,-.10f,-.28f,-.36f,-.02f,-.52f,.11f,-.19f,-.70f,.33f,.20f,-.85f,-.15f,-.89f,.11f,.68f,-.64f,-.70f,.90f,-.02f,-.26f,-.49f,.88f,.46f,.75f,.96f,.46f,.08f,.43f,-.14f,-.17f,-.74f,-.75f,-.19f,.61f,-.87f,.84f,-.04f,.85f,.16f,-.73f,.14f,-.21f,.21f,.73f,.92f,.10f,-.66f,.37f,-.64f,.90f,.32f,-.46f,.04f,.80f,.88f,.13f,-.88f,-.48f,.37f,.42f,-.35f,-.92f,-.16f,.61f,-.15f,.17f,.14f,.89f,-.15f,.18f,.94f,-.13f,.51f,-.67f,.64f,.41f,.45f,.00f,-.81f,.06f,-.06f,-.48f,-.92f,-.42f,.83f,-.87f,.29f,.72f,.70f,.40f,.79f,-.80f,.17f,.32f,.29f,-.30f,-.13f,.82f,.72f,-.96f,-.84f,-.13f,-.36f,.10f,.40f,.59f,.92f,-.30f,-.16f,-.53f,-.47f,.14f,-.82f,.02f,.81f,.27f,-.77f,.63f,.47f,.24f,.19f,-.29f,-.56f,-.49f,-.03f,.60f,-.84f,.10f,-.30f,-.09f,.78f,.31f,-.75f,.19f,-.10f,.02f,.45f,.60f,-.36f,.68f,.49f,-.29f,-.21f,.54f,-.66f,.55f,.03f,.53f,.25f,-.47f,-.13f,.41f,.74f,.02f,-.67f,-.68f,-.61f,-.92f,-.37f,-.65f,-.76f,-.86f,.83f,-.84f,-.79f,-.36f,.09f,.43f,.68f,-.81f,-.25f,.81f,-.66f,.28f,.33f,.74f,.56f,-.29f,-.84f,-.76f,.74f,-.03f,.52f,.30f,.18f,.11f,-.46f,.28f,-.15f,-.27f,.21f,.79f,-.46f,-.41f,-.70f,.60f,-.35f,-.33f,-.56f,-.14f,.17f,.01f,-.81f,-.62f,-.60f,.39f,-.62f,.87f,.83f,-.00f,-.47f,-.39f,-.97f,.11f,.22f,-.32f,-.21f,-.25f,.70f,.70f,.01f,.04f,-.38f,-.12f,.80f,.85f,.40f,.87f,-.59f,-.99f,-.17f,.63f,.30f,.36f,.77f,-.25f,-.50f,.22f,-.26f,.49f,-.97f,.18f,-.29f,-.19f,-.28f,.34f,-.69f,-.15f,-.50f,.18f,.68f,-.31f,.52f,-.18f,-.67f,-.22f,.11f,-.58f,-.17f,-.21f,-.01f,.71f,.93f,.00f,.41f,-.72f,-.66f,-.26f,-.57f,-.06f,-.44f,-.06f,-.79f,-.14f,-.09f,.52f,.67f,-.65f,-.52f,-.85f,-.52f,-.18f,-.70f,.75f,-.51f,-.49f,-.06f,.81f,.06f,-.95f,.53f,-.80f,-.16f,.84f,.97f,-.53f,.00f,-.75f,-.99f,-.38f,.07f,-.44f,.25f,.31f,.16f,-.39f,-.92f,-.17f,-.38f,-.63f,.27f,.53f,.56f,-.82f,.95f,.69f,-.25f,.62f,-.36f,-.21f,.15f,-.82f,-.73f,-.77f,-.78f,.46f,-.15f,.44f,.82f,-.82f,-.50f,-.73f,-.39f,-.85f,.89f,-.30f,-.48f,-.96f,.95f,.66f,-.08f,.82f,-.00f,-.16f,.99f,-.49f,.27f,.88f,-.69f,-.36f,-.81f,.86f,-.53f,.79f,.76f,.56f,.59f,-.70f,.31f,.61f,.80f,.93f,.90f,.05f,.42f,.21f,-.75f,-.87f,.48f,-.83f,-.58f,-.53f,-.20f,-.34f,-.08f,.25f,-.21f,-.18f,-.82f,.59f,.18f,-.73f,.50f,.36f,.01f,-.54f,-.40f,.24f,-.99f,-.30f,-.43f,-.26f,.53f,.98f,.90f,-.68f,-.57f,-.02f,.63f,-.84f,-.96f,-.52f,.32f,.09f,-.98f,-.52f,.64f,.17f,.82f,.07f,.92f,.63f,-.43f,-.52f,.06f,-.85f,.02f,-.71f,-.86f,-.58f,-.32f,.79f,.68f,.02f,.50f,-.76f,-.26f,-.23f,.03f,.42f,-.26f,-.09f,-.40f,.84f,.79f,-.57f,-.07f,-.61f,-.79f,-.97f,.16f,.10f,.45f,-.04f,.66f,-.52f,-.39f,-.77f,.94f,.70f,.61f,-.96f,-.33f,.85f,.53f,-.74f,.30f,.31f,-.33f,.74f,-.56f,-.00f,-.95f,-.91f,-.42f,.85f,-.02f,-.06f,-.73f,.07f,-.52f,.10f,.39f,.90f,.78f,-.11f,.58f,.61f,.30f,.53f,.51f,.06f,.62f,.58f,.39f,-.78f,.47f,-.36f,-.48f,.29f,-.98f,.90f,-.63f,-.40f,.77f,.93f,.32f,.94f,.76f,-.86f,.84f,-.57f,-.35f,-.68f,.58f,-.70f,-.87f,-.13f,-.99f,-.46f,.51f,.28f,-.22f,-.92f,-.02f,.77f,.91f,.41f,-.13f,-.18f,-.80f,-.12f,.99f,-.62f,.71f,-.39f,-.53f,-.75f,-.88f,-.05f,.84f,.32f,.87f,.18f,.71f,.03f,-.48f,-.59f,.43f,-.44f,.23f,-.36f,.79f,-.27f,-.30f,-.35f,.56f,.99f,-.35f,-.89f,-.45f,.45f,-.55f,.27f,-.29f,.90f,-.95f,-.63f,.31f,-.28f,-.56f,.38f,-.08f,-.18f,.51f,.60f,-.93f,-.00f,.89f,-.52f,.08f,-.36f,-.61f,.71f,-.06f,.53f,-.77f,-.46f,-.30f,-.79f,-.36f,.59f,-.32f,-.28f,.88f,-.15f,.99f,.31f,.84f,.72f,.45f,.61f,-.69f,.73f,-.28f,.88f,-.21f,.65f,-.54f,-.76f,-.33f,-.53f,.87f,-.36f,.96f,.60f,-.07f,.43f,.52f,.68f,.72f,-.53f,-.23f,.09f,.47f,-.92f,-.31f,-.63f,-.26f,-.86f,.28f,.77f,-.14f,.63f,.48f,.81f,.31f,.94f,.37f,.93f,-.70f,.64f,-.63f,-.31f,-.10f,-.64f,.61f,.02f,-.86f,-.40f,-.95f,-.80f,.98f,.96f,-.07f,.33f,-.59f,-.13f,-.54f,.67f,-.50f,.14f,-.30f,.30f,-.80f,.71f,.86f,.26f,-.97f,-.57f,-.88f,-.95f,.84f,-.25f,.58f,-.28f,-.61f,-.55f,-.03f,.14f,-.05f,.66f,.98f,-.01f,-.41f,.06f,.74f,.09f,.42f,.34f,-.06f,.27f,-.26f,-.82f,-.07f,.67f,-.12f,-.46f,.16f,-.07f,.05f,-.21f,.35f,-.77f,-.39f,.11f,.15f,.06f,.17f,.69f,.25f,-.89f,.93f,-.29f,-.25f,-.95f,.74f,-.29f,-.22f,.82f,-.71f,.39f,-.54f,.58f,.41f,-.82f,-.65f,-.41f,-.74f,-.09f,-.08f,-.53f,-.06f,-.02f,.51f,.76f,.40f,.23f,-.14f,-.71f,-.59f,.33f,.14f,-.98f,-.01f,-.45f,.25f,-.33f,-.21f,.47f,-.28f,-.43f,.52f,-.04f,.04f,-.63f,.53f,-.15f,-.23f,-.15f,.04f,-.99f,-.69f,.16f,.26f,-.21f,.90f,.99f,-.91f,-.69f,.40f,-.69f,-.52f,.18f,.86f,-.41f,.69f,.44f,-.13f,-.16f,.89f,.60f,-.12f,.06f,.84f,-.58f,.71f,.19f,.81f,.58f,-.64f,-.39f,-.92f,-.70f,-.98f,-.13f,.07f,-.99f,.14f,.69f,-.20f,.18f,-.80f,.34f,-.79f,.94f,.34f,-.81f,-.69f,-.72f,-.07f,.33f,.06f,.87f,-.77f,.72f,.16f,.74f,.87f,-.87f,.57f,.87f,.65f,.63f,-.78f,.71f,.64f,-.94f,.45f,.52f,.61f,.80f,.56f,.89f,-.99f,.31f,.11f,-.63f,.21f,.97f,.04f,-.59f,-.81f,.13f,.46f,.28f,.01f,-.22f,.62f,-.12f,.64f,.82f,-.67f,-.76f,.87f,-.23f,-.43f,.72f,-.64f,.08f,.53f,-.41f,-.03f,-.68f,.62f,.08f,.37f,-.12f,-.52f,.07f,.06f,-.28f,.02f,.35f,.11f,.38f,.90f,-.19f,-.36f,.95f,-.53f,-.66f,-.30f,.46f,.44f,.63f,.15f,.75f,-.88f,-.80f,-.39f,-.33f,.45f,.56f,-.34f,.98f,-.50f,.22f,.15f,.12f,.62f,.06f,.82f,-.40f,.86f,.56f,.18f,.13f,-.30f,-.98f,.57f,-.49f,-.32f,-.74f,.91f,-.07f,.09f,.35f,-.22f,-.42f,.37f,.41f,-.01f,-.45f,.96f,-.97f,-.37f,.58f,-.74f,.61f,.37f,-.25f,.82f,.99f,-.22f,.60f,-.95f,.35f,.63f,-.73f,-.96f,.40f,.17f,-.04f,.01f,-.45f,-.52f,.86f,.32f,-.97f,-.67f,.21f,.60f,-.88f,-.17f,.11f,-.89f,.52f,-.93f,-.17f,-.94f,.72f,-.45f,.81f,-.85f,-.63f,.27f,.72f,-.99f,-.74f,-.70f,-.06f,-.28f,.28f,.89f,-.38f,-.36f,.28f,.51f,-.29f,-.04f,-.22f,-.77f,.62f,.68f,.73f,-.29f,.96f,.26f,.36f,-.79f,.93f,.23f,.06f,-.99f,.93f,.34f,-.40f,.43f,-.80f,-.60f,-.64f,-.17f,-.89f,-.56f,-.81f,.70f,-.67f,.57f,.57f,-.77f,-.28f,-.39f,-.58f,-.35f,-.02f,.21f,-.43f,.60f,-.95f,-.44f,.40f,-.22f,.44f,-.94f,.21f,-.21f,-.38f,-.06f,.43f,-.48f,.49f,-.79f,.31f,-.91f,.34f,.89f,.10f,.50f,.48f,.29f,.00f,.24f,.50f,.56f,-.73f,-.10f,.23f,.29f,.59f,-.02f,-.76f,-.46f,-.97f,-.06f,-.46f,.51f,.14f,.97f,-.37f,.65f,-.74f,-.20f,-.29f,.57f,-.80f,-.34f,-.33f,.17f,-.86f,.58f,-.85f,-.08f,.09f,-.88f,.57f,-.78f,-.55f,-.72f,.18f,-.45f,.37f,-.51f,-.77f,.83f,-.39f,.73f,-.87f,-.18f,-.95f,.94f,-.21f,.70f,.56f,.25f,.99f,.14f,.18f,-.35f,.72f,.09f,.98f,.85f,.10f,.97f,.42f,-.15f,-.85f,.89f,.67f,.99f,-.08f,.32f,.08f,.55f,.96f,-.52f,-.63f,-.21f,-.50f,.79f,.10f,.61f,-.96f,-.47f,.59f,-.02f,-.39f,.89f,.31f,.94f,-.84f,.47f,.79f,.82f,-.18f,-.41f,-.22f,-.99f,-.68f,-.35f,.86f,.16f,.68f,.05f,-.11f,-.65f,.23f,-.81f,-.92f,-.70f,.60f,-.65f,-.37f,-.23f,.10f,.15f,.19f,-.03f,.74f,-.07f,.12f,-.49f,.67f,-.91f,.38f,.53f,.86f,-.41f,-.39f,-.79f,-.48f,-.83f,-.58f,.18f,-.96f,-.77f,-.47f,-.38f,.32f,.23f,.85f,.35f,.40f,-.27f,.66f,.62f,.65f,-.94f,-.69f,-.93f,.69f,-.20f,.05f,-.67f,-.18f,-.36f,-.31f,.15f,-.01f,.86f,-.72f,-.94f,.41f,-.22f,-.86f,-.56f,.69f,.09f,-.00f,-.80f,.05f,.98f,.95f,.46f,-.33f,-.38f,-.04f,-.53f,-.12f,.82f,-.55f,-.79f,.10f,-.14f,-.68f,-.29f,.14f,.93f,.64f,.97f,.37f,.69f,.73f,-.57f,.11f,-.75f,-.07f,.74f,.25f,-.57f,.67f,.60f,-.26f,-.35f,-.19f,.55f,.42f,-.62f,.63f,-.87f,-.82f,.60f,-.10f,-.67f,.89f,-.38f,.29f,.30f,.81f,.95f,.43f,-.64f,.91f,.37f,.96f,-.71f,.55f,.06f,-.95f,-.56f,.34f,.25f,.05f,.26f,.28f,.95f,.68f,-.63f,-.80f,.63f,-.09f,-.06f,.26f,-.34f,-.27f,-.57f,.99f,-.48f,.10f,.96f,-.17f,.70f,-.76f,-.49f,.37f,-.81f,-.05f,-.67f,.70f,-.94f,-.15f,-.98f,.03f,-.77f,.97f,-.61f,-.32f,.63f,-.01f,-.42f,-.89f,.41f,.14f,.11f,-.61f,-.05f,.32f,.21f,.64f,.36f,-.44f,.05f,-.33f,.19f,-.43f,-.40f,-.89f,.90f,.67f,.66f,-.37f,-.67f,-.63f,.93f,.29f,.28f,.41f,.17f,-.76f,-.61f,-.48f,-.86f,-.13f,-.55f,.96f,-.06f,-.14f,.97f,.86f,-.20f,.88f,.40f,.64f,.79f,.47f,-.09f,-.26f,-.31f,.74f,-.09f,-.49f,.64f,-.40f,-.31f,-.87f,-.68f,.40f,-.44f,.19f,.10f,-.69f,.42f,-.59f,.84f,-.39f,-.85f,.33f,.44f,.12f,.69f,.02f,-.71f,.57f,-.16f,-.67f,.53f,-.95f,.37f,.42f,.40f,.76f,-.61f,.02f,.77f,.66f,.54f,-.37f,-.76f,-.56f,.77f,.57f,.99f,-.02f,.00f,.54f,.19f,.89f,-.32f,-.00f,-.74f,-.82f,.43f,.42f,.35f,-.81f,.68f,-.52f,-.17f,.18f,.29f,.76f,-.73f,.01f,-.28f,.04f,.21f,.89f,-.40f,.04f,.13f,-.95f,.33f,.47f,.18f,.61f,-.83f,-.08f,-.08f,-.75f,.30f,.42f,.76f,.25f,.18f,-.84f,.11f,-.94f,-.42f,-.16f,-.97f,.54f,.11f,.37f,.10f,-.39f,-.20f,.09f,.04f,-.92f,.97f,.98f,-.78f,.58f,-.74f,.92f,-.27f,.16f,-.42f,-.31f,.39f,.23f,-.65f,.24f,-.92f,.52f,.98f,.39f,.51f,.98f,-.98f,.34f,.68f,-.84f,.41f,-.87f,.55f,.39f,.16f,.37f,.01f,.82f,.91f,-.82f,-.76f,.65f,-.45f,.44f,.07f,-.46f,.64f,-.63f,-.11f,.54f,-.92f,-.11f,-.94f,-.78f,-.88f,.68f,-.11f,.78f,.28f,-.08f,.99f,.27f,.73f,-.55f,.05f,.69f,-.69f,-.83f,.70f,-.24f,.26f,-.86f,.69f,.91f,.43f,.84f,-.60f,.31f,.74f,.57f,-.93f,-.85f,-.74f,.73f,-.15f,.30f,.70f,-.15f,.09f,-.46f,.59f,.05f,.34f,.33f,.17f,-.08f,.03f,-.60f,.93f,.11f,-.13f,-.65f,-.19f,.49f,.82f,-.77f,.22f,-.02f,.79f,.06f,.16f,-.79f,-.30f,-.48f,.62f,-.12f,-.95f,-.87f,-.94f,-.90f,-.33f,.60f,-.21f,.88f,-.28f,.46f,-.14f,.13f,-.11f,-.17f,-.03f,.19f,-.04f,-.34f,-.14f,.65f,-.35f,-.66f,-.75f,.25f,-.91f,-.45f,-.59f,.32f,.75f,-.24f,-.14f,.10f,-.24f,.03f,-.83f,.90f,.08f,.09f,.65f,-.39f,-.80f,.26f,.36f,-.14f,-.53f,.21f,-.06f,.63f,.94f,-.86f,.30f,.91f,-.01f,-.61f,.67f,-.44f,-.57f,.28f,.83f,-.46f,-.18f,-.27f,-.25f,.43f,.55f,-.74f,-.39f,.51f,.62f,.70f,.17f,.61f,-.87f,-.76f,-.28f,.80f,-.09f,.48f,-.68f,.46f,.76f,.73f,-.06f,.15f,.01f,.84f,-.71f,.43f,-.82f,.77f,-.86f,.30f,.01f,-.03f,-.79f,-.96f,.75f,.36f,-.18f,-.23f,.79f,.36f,-.77f,.96f,.60f,.11f,-.73f,-.44f,-.15f,-.54f,.94f,-.54f,.73f,.43f,.70f,-.02f,-.47f,.14f,-.07f,-.78f,-.12f,.61f,-.65f,.28f,-.66f,-.56f,.76f,.56f,.52f,-.43f,.31f,.11f,-.94f,.04f,-.52f,-.37f,.86f,-.74f,-.38f,-.40f,.41f,-.40f,.09f,-.07f,-.79f,-.08f,-.32f,-.87f,-.31f,.21f,-.58f,-.18f,-.86f,.29f,-.28f,.30f,-.52f,.41f,.66f,.10f,-.40f,.70f,.29f,-.20f,-.25f,.33f,.32f,-.14f,.39f,-.41f,.65f,-.88f,.82f,.34f,-.80f,.06f,.86f,-.69f,-.84f,-.36f,.43f,.25f,-.80f,.84f,.19f,-.80f,.53f,-.53f,.04f,-.54f,-.26f,-.59f,.21f,-.45f,.53f,-.17f,.52f,.35f,.72f,.16f,-.31f,.12f,-.03f,-.92f,.02f,.27f,.52f,.34f,-.66f,-.46f,-.14f,.81f,.00f,.92f,-.49f,.80f,.40f,-.90f,.34f,.12f,.47f,.16f,-.66f,-.83f,.30f,.10f,-.99f,.95f,.61f,.92f,.90f,-.46f,.99f,.38f,.38f,.79f,-.33f,.28f,.69f,.44f,-.18f,-.66f,.07f,.65f,-.06f,-.24f,-.70f,.01f,-.31f,-.40f,.19f,.93f,-.52f,-.29f,-.59f,-.55f,-.57f,-.61f,-.51f,.99f,.22f,.51f,.71f,.39f,.59f,.38f,.02f,-.98f,.56f,-.04f,-.02f,.44f,-.85f,-.70f,-.70f,-.02f,-.09f,-.96f,.72f,.25f,-.20f,.15f,.68f,-.90f,-.91f,-.99f,-.21f,-.59f,-.38f,-.02f,-.84f,-.50f,.68f,-.22f,-.88f,.25f,.98f,-.10f,-.37f,-.14f,.70f,.03f,.45f,.17f,-.01f,.33f,-.15f,.03f,-.66f,-.05f,-.10f,.35f,-.00f,-.12f,-.79f,.76f,.29f,-.27f,.79f,-.57f,.62f,.65f,.84f,-.17f,.23f,.74f,.98f,-.24f,-.96f,-.16f,.76f,-.22f,.35f,-.39f,.67f,.59f,.91f,.88f,.69f,-.72f,.43f,.55f,-.55f,.94f,.34f,-.74f,-.44f,-.48f,-.92f,-.62f,.79f,.52f,-.59f,-.14f,-.20f,.14f,.65f,-.27f,.71f,-.80f,.29f,-.15f,.15f,.30f,-.65f,-.17f,-.72f,-.90f,.99f,-.20f,.40f,-.59f,.46f,-.69f,.97f,-.86f,.35f,-.96f,.22f,.26f,-.87f,-.46f,-.54f,.30f,.28f,.63f,.43f,-.87f,-.16f,.40f,.07f,-.04f,.67f,-.30f,-.45f,.88f,.92f,.05f,.24f,.18f,-.29f,.26f,.57f,-.35f,.49f,-.16f,.32f,-.79f,-.29f,-.87f,-.72f,.40f,-.44f,-.69f,-.93f,-.85f,.92f,.00f,.66f,-.32f,.94f,.40f,.81f,.67f,-.72f,-.11f,.82f,-.83f,-.53f,-.01f,.18f,-.95f,-.41f,-.98f,-.08f,.24f,-.33f,.72f,-.81f,.21f,.46f,.25f,-.74f,.78f,.68f,-.20f,-.66f,.73f,-.09f,.98f,.50f,.38f,.95f,-.79f,.47f,.95f,-.28f,-.03f,.15f,.25f,.61f,.10f,-.42f,.98f,-.93f,-.47f,-.03f,.31f,.50f,-.24f,.42f,-.03f,-.75f,.75f,.86f,.07f,.79f,.39f,.58f,-.09f,-.04f,.63f,-.49f,.89f,.64f,.14f,-.70f,.04f,-.24f,.42f,.98f,-.88f,-.86f,.43f,-.17f,-.79f,-.15f,-.43f,.97f,-.88f,-.97f,.87f,-.61f,.56f,.53f,.94f,.26f,-.71f,.68f,-.19f,.87f,.00f,-.02f,.33f,.91f,-.36f,.60f,-.55f,-.04f,-.58f,-.07f,.18f,.42f,-.77f,-.66f,-.90f,-.23f,-.94f,.90f,-.37f,-.63f,.57f,.98f,-.46f,-.47f,.79f,.63f,-.05f,-.39f,-.51f,.22f,.23f,-.89f,-.01f,.98f,-.59f,-.85f,.12f,.24f,.63f,-.69f,.84f,-.15f,.48f,-.16f,.14f,.78f,.84f,.86f,-.80f,-.05f,-.42f,.32f,-.39f,-.27f,.85f,-.58f,.96f,-.53f,-.40f,-.75f,.76f,-.87f,-.39f,.73f,.48f,.87f,-.49f,.94f,-.25f,-.19f,-.30f,.56f,-.35f,.09f,.62f,.02f,.07f,-.32f,.83f,.87f,.96f,.50f,.16f,.12f,-.21f,-.77f,-.69f,.95f,-.84f,-.93f,-.83f,.68f,.06f,.48f,-.95f,.44f,.62f,.21f,-.62f,-.11f,.21f,-.74f,-.04f,.07f,.47f,-.07f,-.27f,.71f,.70f,-.55f,.13f,-.50f,.63f,-.51f,.48f,-.19f,.17f,-.34f,-.87f,-.28f,-.71f,.16f,-.56f,.39f,-.14f,.26f,-.09f,-.48f,-.10f,.50f,-.83f,.02f,-.39f,.19f,.10f,.73f,-.43f,-.47f,.79f,.75f,.89f,-.38f,-.72f,-.93f,-.22f,-.00f,.43f,.07f,-.96f,.08f,.63f,-.55f,-.55f,.83f,-.12f,-.70f,.03f,-.51f,.61f,.49f,.95f,.93f,.69f,.88f,-.10f,.50f,-.05f,-.59f,.36f,-.85f,-.02f,-.02f,.75f,-.77f,-.49f,.16f,.75f,-.08f,-.44f,.17f,-.87f,-.19f,-.60f,.03f,-.30f,-.21f,-.58f,.08f,-.47f,-.03f,.39f,.61f,.71f,.10f,.18f,.53f,-.62f,-.72f,.72f,-.87f,-.45f,-.57f,.14f,.65f,-.14f,-.65f,.17f,-.89f,-.35f,.22f,-.82f,-.23f,-.56f,.25f,.82f,.98f,.24f,.76f,-.27f,-.97f,-.14f,-.99f,.95f,.73f,-.37f,-.26f,.73f,-.51f,.42f,-.25f,.76f,.59f,-.54f,.18f,.12f,.41f,.41f,-.45f,.44f,-.26f,-.10f,-.21f,-.33f,.82f,-.88f,.33f,-.83f,-.47f,.08f,.36f,-.59f,-.77f,.70f,-.23f,.72f,.77f,-.99f,.36f,.16f,.62f,.35f,-.99f,.99f,-.32f,.39f,-.09f,.15f,.97f,-.42f,-.51f,-.98f,-.23f,-.81f,-.62f,.36f,-.63f,.33f,-.67f,-.84f,.17f,.26f,-.57f,.77f,-.86f,.54f,-.11f,.97f,-.29f,.45f,-.59f,-.39f,.21f,-.40f,.03f,.36f,.41f,.56f,.76f,.71f,-.62f,-.49f,.99f,.03f,.55f,-.15f,.03f,.62f,-.62f,-.02f,-.53f,.64f,-.24f,.14f,.67f,-.58f,-.90f,.66f,.44f,-.15f,.16f,.92f,.84f,.61f,.45f,.01f,-.23f,.51f,.65f,-.04f,-.61f,-.66f,.76f,-.52f,.63f,.82f,.67f,.81f,-.08f,.14f,.91f,.20f,-.82f,-.90f,.70f,.65f,.87f,.38f,.77f,.52f,.40f,-.57f,.19f,.58f,.50f,.78f,-.09f,-.93f,.67f,.06f,.11f,.36f,-.40f,.37f,.53f,.01f,-.09f,.89f,-.87f,.99f,-.54f,.92f,-.45f,.69f,-.22f,.92f,-.01f,.04f,.68f,-.96f,-.56f,.26f,.24f,.77f,-.31f,.23f,.26f,-.09f,-.12f,-.57f,.35f,-.80f,.36f,.36f,.55f,.85f,-.60f,.12f,-.96f,-.54f,-.58f,-.21f,.62f,.60f,.64f,.70f,-.14f,-.63f,-.14f,.55f,.25f,.21f,-.19f,.42f,-.61f,-.08f,-.38f,-.86f,.08f,.48f,-.58f,.82f,.14f,.50f,.81f,.07f,.24f,.03f,.54f,.47f,-.97f,-.80f,-.76f,-.86f,.80f,-.14f,-.04f,-.54f,.61f,-.42f,-.90f,.94f,-.84f,.15f,-.72f,.52f,.10f,-.48f,.42f,-.63f,.93f,-.80f,-.17f,-.29f,-.87f,-.24f,.99f,.75f,.78f,-.39f,-.94f,.46f,.34f,-.93f,.87f,-.18f,.20f,.33f,.91f,.12f,.62f,-.19f,-.50f,-.20f,.73f,.81f,-.04f,-.26f,.16f,-.24f,.60f,-.69f,.59f,.18f,.02f,-.00f,-.28f,.48f,-.29f,-.97f,.11f,-.34f,-.22f,.74f,.53f,.11f,-.97f,-.50f,-.46f,-.96f,-.93f,-.02f,-.23f,.04f,-.30f,-.79f,-.46f,.03f,-.18f,.71f,.73f,.27f,.76f,.65f,.52f,.34f,.54f,-.36f,.02f,.90f,-.24f,-.44f,-.81f,-.22f,.50f,-.25f,-.94f,.91f,-.57f,-.38f,.95f,-.05f,.46f,-.74f,-.88f,.59f,.86f,-.52f,-.32f,.30f,-.43f,.68f,.69f,-.88f,.92f,-.29f,-.78f,.89f,-.96f,-.91f,-.80f,.14f,-.36f,-.86f,.79f,-.39f,-.81f,.86f,.43f,-.77f,.44f,-.40f,-.91f,-.43f,-.29f,-.23f,-.07f,-.76f,.14f,.88f,.59f,-.36f,-.80f,-.81f,.74f,.49f,-.41f,.10f,-.45f,.89f,.99f,.95f,-.84f,-.19f,-.50f,-.00f,.17f,-.27f,-.14f,-.27f,-.34f,-.22f,.10f,.78f,-.90f,.64f,.98f,-.78f,.06f,.61f,.51f,-.34f,.70f,-.17f,-.65f,.84f,-.56f,-.81f,.26f,-.08f,.84f,.64f,.96f,-.26f,.73f,.68f,.71f,-.53f,-.04f,.32f,.99f,-.73f,.65f,-.57f,.44f,-.66f,-.20f,-.43f,-.54f,-.97f,.41f,.24f,-.53f,-.55f,-.26f,-.71f,.51f,.83f,.53f,.01f,.95f,.45f,-.40f,.31f,.59f,.34f,-.45f,-.92f,-.27f,.54f,-.14f,-.87f,-.60f,.72f,-.70f,-.04f,.18f,.91f,.68f,-.15f,.21f,.86f,.54f,-.86f,.70f,.40f,.64f,-.82f,-.70f,.68f,.96f,.71f,.84f,-.89f,-.47f,-.63f,-.75f,.79f,-.77f,.14f,.02f,.19f,-.84f,-.19f,.63f,.32f,.65f,-.16f,.44f,-.23f,-.01f,.67f,.85f,.08f,.32f,.14f,.70f,.40f,-.03f,-.29f,.78f,-.17f,-.68f,.36f,.28f,-.48f,.65f,.74f,-.37f,.81f,-.95f,-.21f,.04f,-.23f,-.24f,-.27f,-.93f,.19f,-.27f,-.83f,.19f,-.21f,-.08f,-.19f,.73f,.26f,-.97f,.51f,.92f,-.53f,-.18f,-.54f,.89f,-.49f,-.61f,-.54f,-.63f,-.01f,-.77f,.96f,.44f,-.64f,.63f,.24f,-.51f,.67f,-.16f,-.99f,.61f,.73f,.53f,.84f,-.06f,-.39f,.13f,.40f,-.26f,.90f,.94f,.15f,.71f,.70f,-.71f,-.53f,.74f,.47f,-.45f,.45f,-.24f,.44f,.90f,-.71f,.22f,.32f,-.61f,-.78f,-.66f,-.44f,.88f,-.58f,.57f,.08f,.57f,-.61f,.71f,-.12f,.49f,-.41f,-.69f,-.47f,-.46f,.73f,-.19f,-.01f,.44f,-.41f,.74f,.15f,.15f,.10f,.10f,.59f,.16f,-.80f,-.75f,.79f,-.22f,-.75f,-.05f,.22f,-.92f,-.11f,-.86f,.81f,-.29f,-.08f,-.78f,-.65f,.19f,-.35f,-.76f,.91f,-.55f,-.39f,-.23f,.34f,-.07f,-.95f,.10f,.98f,-.12f,-.49f,-.79f,-.00f,.57f,.83f,.21f,.28f,.33f,-.87f,.60f,-.00f,-.82f,-.00f,.61f,.06f,.45f,.65f,.75f,.04f,-.21f,-.92f,.25f,-.11f,-.52f,.79f,.19f,.48f,-.79f,-.26f,.47f,.05f,.98f,.89f,-.58f,-.03f,.22f,.20f,.17f,-.96f,.13f,.92f,-.31f,-.81f,-.30f,-.60f,.30f,.78f,.86f,-.40f,.23f,-.36f,-.35f,-.69f,.93f,.94f,.52f,.38f,.33f,-.97f,-.28f,.80f,.47f,-.27f,.06f,.25f,-.74f,.57f,.04f,-.93f,.88f,.65f,.68f,.95f,-.69f,.58f,-.67f,.73f,.85f,-.35f,-.10f,-.38f,-.57f,.67f,.79f,-.24f,.60f,.89f,-.85f,.20f,.11f,.14f,-.02f,.53f,-.56f,.39f,-.27f,.43f,.43f,-.41f,-.25f,.00f,-.13f,.26f,.08f,.39f,.07f,.04f,.70f,.49f,-.28f,.19f,-.48f,.29f,.52f,-.99f,.40f,.04f,.47f,.54f,.32f,.48f,.66f,.51f,.29f,-.58f,-.93f,.03f,-.23f,.62f,-.33f,-.51f,-.63f,.22f,.47f,.26f,-.04f,-.50f,.25f,-.61f,.92f,-.52f,.97f,-.12f,.98f,-.12f,.20f,-.04f,-.78f,-.69f,-.06f,-.55f,.38f,-.19f,-.35f,.84f,.46f,.68f,.17f,.27f,.32f,.79f,.37f,-.16f,-.39f,.55f,-.49f,-.70f,-.67f,-.53f,.80f,.35f,-.96f,-.06f,.63f,-.50f,-.16f,.35f,.39f,.51f,.65f,-.12f,.24f,-.88f,.08f,.56f,.92f,-.77f,.93f,.79f,-.06f,-.82f,.00f,-.67f,.49f,.83f,.35f,-.47f,-.67f,-.94f,-.23f,.71f,-.06f,-.06f,.98f,-.94f,-.79f,.03f,-.43f,-.64f,-.27f,-.32f,-.78f,.40f,-.39f,.96f,.81f,.36f,-.64f,.41f,.97f,-.08f,.01f,.31f,-.08f,-.85f,-.09f,-.40f,-.91f,-.11f,-.29f,.79f,-.40f,-.20f,-.92f,.72f,-.25f,.40f,-.20f,.47f,-.52f,.86f,.68f,-.68f,.15f,.99f,.30f,-.99f,-.51f,.44f,-.96f,-.71f,.83f,-.37f,-.12f,-.58f,-.11f,.60f,-.96f,-.35f,-.13f,.52f,.99f,-.94f,-.49f,-.88f,-.50f,.11f,-.79f,-.04f,-.03f,-.60f,-.16f,-.93f,-.85f,-.73f,-.47f,-.22f,-.27f,-.52f,.19f,-.39f,.96f,-.23f,.72f,-.19f,-.71f,.00f,-.78f,.55f,.90f,.47f,-.33f,-.02f,.04f,-.66f,-.34f,-.32f,-.34f,.34f,-.93f,.38f,-.43f,.70f,.90f,-.76f,-.16f,.54f,-.30f,.57f,.20f,-.68f,.62f,.03f,.58f,.41f,-.59f,.54f,.96f,-.04f,-.84f,-.34f,-.76f,.89f,-.86f,-.84f,-.22f,-.61f,-.64f,-.25f,.07f,-.37f,-.85f,-.41f,.59f,.11f,-.97f,.68f,.28f,-.42f,-.19f,.79f,-.46f,.42f,.83f,.97f,-.18f,.16f,-.04f,-.22f,.04f,.14f,.96f,.61f,.72f,-.71f,.23f,-.85f,.06f,.20f,-.72f,.31f,.84f,-.57f,-.23f,-.31f,.17f,.39f,.56f,.31f,.46f,-.15f,-.11f,-.86f,.51f,-.43f,-.63f,.65f,.51f,-.54f,-.58f,.13f,.55f,-.77f,-.64f,-.81f,.18f,-.08f,.21f,-.13f,.18f,.64f,.55f,-.73f,-.35f,.39f,-.79f,.68f,.52f,-.68f,.69f,.42f,-.64f,-.98f,.76f,-.56f,.76f,-.06f,-.64f,-.99f,-.16f,.19f,.90f,-.31f,-.98f,-.71f,-.36f,-.19f,-.03f,.98f,-.34f,-.50f,.91f,-.38f,.20f,-.13f,.62f,-.09f,.66f,-.69f,.25f,-.81f,-.20f,-.16f,-.23f,.26f,.69f,.05f,-.88f,-.71f,-.02f,.96f,.56f,.48f,-.50f,-.68f,-.83f,-.17f,.38f,.48f,.04f,-.05f,-.47f,.46f,.85f,.13f,.08f,.39f,.89f,-.36f,.05f,-.60f,-.95f,-.95f,.08f,.76f,.82f,-.62f,.32f,-.09f,.55f,-.26f,.08f,-.88f,-.93f,-.96f,.26f,.08f,.98f,.40f,.34f,-.34f,-.38f,.35f,.68f,.28f,-.58f,.66f,.30f,-.66f,-.25f,.42f,.05f,-.19f,.33f,.47f,.39f,-.03f,.63f,.09f,.65f,.83f,-.67f,-.86f,.72f,-.20f,.35f,-.88f,-.37f,-.52f,-.29f,-.83f,-.02f,-.08f,.25f,-.75f,.42f,-.23f,-.37f,.48f,-.75f,-.11f,.87f,.87f,.03f,.33f,-.26f,-.43f,.65f,.45f,.77f,.99f,-.52f,.97f,-.72f,.74f,.38f,.86f,-.26f,.81f,.94f,.28f,.16f,-.69f,-.50f,-.78f,-.40f,.11f,-.10f,.89f,.88f,.57f,-.75f,-.55f,-.49f,.06f,.18f,-.56f,.18f,.92f,-.63f,.94f,.01f,.39f,.46f,.44f,.90f,.16f,.77f,-.58f,-.69f,.88f,.12f,.10f,.27f,-.89f,-.94f,-.64f,.55f,-.87f,-.70f,-.47f,.93f,.73f,-.87f,-.20f,-.21f,.61f,-.98f,-.67f,-.45f,.82f,.77f,-.38f,-.29f,-.61f,-.78f,.93f,-.74f,-.48f,.51f,.03f,.73f,-.53f,-.55f,.99f,.08f,.99f,.88f,.57f,.04f,-.98f,.12f,-.39f,.47f,-.54f,-.65f,-.29f,-.51f,-.92f,-.23f,.73f,-.76f,.50f,.11f,.38f,-.44f,.04f,-.40f,-.27f,-.95f,-.83f,.77f,-.20f,-.27f,-.36f,.59f,.65f,-.69f,-.53f,-.86f,-.55f,-.68f,.80f,.55f,.69f,-.41f,-.49f,.95f,.99f,.01f,-.17f,-.72f,-.70f,-.05f,-.56f,.92f,-.37f,.00f,-.38f,-.18f,-.63f,-.46f,-.45f,.82f,.54f,.31f,.41f,-.00f,.08f,-.91f,.16f,.35f,-.62f,-.36f,.22f,.26f,-.31f,-.91f,.48f,.41f,.87f,-.34f,.69f,.24f,-.83f,.90f,.76f,.17f,.11f,.85f,.03f,-.66f,-.29f,.58f,.31f,-.87f,-.90f,-.48f,.21f,-.34f,-.87f,.60f,.91f,-.85f,-.59f,.20f,.64f,-.55f,-.20f,.77f,-.70f,.36f,.44f,-.66f,-.12f,-.56f,-.47f,.37f,-.45f,.43f,.93f,.76f,.03f,-.42f,-.12f,.91f,-.25f,-.93f,-.71f,.00f,.32f,.41f,-.72f,.38f,-.29f,-.83f,.71f,-.73f,-.68f,.81f,.83f,.08f,.74f,-.37f,-.17f,.01f,-.97f,-.35f,-.79f,.80f,-.27f,.28f,-.50f,-.55f,.70f,.36f,.68f,-.25f,-.08f,.84f,-.28f,-.92f,.54f,-.50f,.82f,.23f,-.01f,.09f,.61f,.42f,-.48f,.01f,.56f,.96f,.28f,.67f,-.48f,-.98f,-.56f,-.05f,-.55f,.91f,.77f,-.98f,.55f,.89f,.56f,.58f,.42f,.87f,.40f,.28f,-.51f,.74f,.99f,.77f,.55f,.48f,.58f,-.28f,-.16f,.69f,.19f,-.15f,-.67f,-.89f,-.63f,-.49f,.69f,.14f,.41f,.37f,.96f,.11f,-.89f,-.19f,.83f,.67f,-.51f,.17f,-.79f,.53f,-.27f,-.41f,-.93f,-.01f,.31f,.17f,.20f,.79f,.84f,.90f,-.02f,.59f,-.77f,-.47f,.59f,-.99f,-.88f,.78f,.43f,.90f,.08f,.71f,-.12f,.91f,-.82f,-.86f,-.61f,.28f,.35f,.60f,.55f,-.60f,-.67f,.67f,.99f,-.30f,.24f,.47f,-.17f,-.82f,-.22f,.40f,.01f,-.75f,.45f,.38f,.70f,-.44f,.96f,.72f,.12f,-.58f,.96f,-.41f,-.56f,-.49f,-.95f,-.36f,.62f,-.10f,-.65f,-.51f,-.13f,-.44f,.21f,.75f,-.82f,.75f,-.76f,.20f,-.63f,-.95f,-.10f,-.30f,.85f,.59f,-.76f,-.36f,.75f,-.05f,-.47f,-.67f,-.84f,-.24f,-.22f,.24f,.88f,-.33f,-.60f,-.81f,-.22f,-.55f,-.10f,.84f,-.52f,.31f,.31f,.73f,-.47f,.08f,-.78f,.59f,.27f,.06f,.74f,-.58f,.48f,-.74f,-.60f,-.57f,.16f,-.91f,.48f,-.08f,.42f,.56f,-.31f,-.42f,-.33f,.88f,.84f,.72f,.92f,.36f,.95f,-.40f,.77f,.12f,-.63f,-.67f,-.02f,-.65f,.53f,-.98f,.23f,-.92f,-.88f,-.64f,.84f,.44f,-.08f,-.62f,-.85f,-.16f,.61f,-.11f,.23f,.01f,-.37f,-.36f,.42f,-.73f,.43f,.96f,-.67f,.41f,.94f,-.27f,.04f,-.46f,.26f,.01f,.75f,-.10f,-.41f,-.28f,.57f,-.55f,.18f,-.36f,-.96f,-.15f,-.04f,.80f,.09f,-.84f,.06f,-.63f,.82f,.47f,-.17f,.01f,-.81f,.44f,.19f,-.26f,.88f,.57f,-.14f,-.00f,.34f,.44f,-.20f,-.58f,-.32f,.17f,-.98f,-.29f,-.08f,-.53f,.89f,-.96f,-.43f,.87f,.90f,.25f,-.00f,.55f,.73f,-.28f,.63f,-.73f,.61f,.14f,.51f,-.15f,.90f,.15f,-.24f,.48f,-.74f,.54f,.75f,-.59f,-.22f,.54f,-.99f,.29f,.47f,.87f,-.05f,.65f,.46f,.76f,-.89f,.63f,.65f,-.28f,-.63f,.54f,.37f,.03f,.05f,-.16f,-.38f,.60f,.70f,-.74f,.86f,-.41f,.22f,-.45f,.75f,-.30f,.32f,-.03f,.42f,.48f,.01f,-.79f,-.87f,-.76f,-.51f,-.02f,.00f,.40f,.35f,.78f,-.89f,-.66f,-.15f,-.41f,-.28f,.41f,.37f,.47f,-.56f,.27f,.58f,-.44f,.73f,-.55f,-.54f,.87f,-.67f,-.62f,-.95f,.52f,-.20f,.81f,-.34f,-.48f,.49f,-.23f,.62f,.74f,-.60f,-.39f,-.60f,.34f,-.81f,.17f,.63f,.37f,.11f,-.74f,.60f,-.47f,-.96f,-.42f,-.54f,.20f,.85f,.13f,.83f,.17f,-.56f,-.25f,.69f,-.11f,.11f,.09f,-.83f,-.02f,-.44f,-.50f,-.00f,-.35f,.25f,-.41f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.618199953607470487071395837119780480861663818359375f,-0.06389989937320461610426036713761277496814727783203125f,-3.89999998692423144319718630868010222911834716796875f,-1.450500045929104420139310605009086430072784423828125f,-2.17999985365197002096238065860234200954437255859375f,-0.51189986908286855271654758325894363224506378173828125f,3.577699910062551058587132501997984945774078369140625f,-2.250200014352051969268586617545224726200103759765625f,-2.5099000343143931246459032990969717502593994140625f,-0.6730000213980662504553720282274298369884490966796875f,0.702300014165043950953304374706931412220001220703125f,2.122699901270866718050456256605684757232666015625f,-0.770099962258338877063579275272786617279052734375f,1.6702000853002083591292148412321694195270538330078125f,0.4507999328531322458246677342685870826244354248046875f,-3.265199947389959334032027982175350189208984375f,-0.4861000270523108124365307958214543759822845458984375f,-2.0486000483795994142610652488656342029571533203125f,0.707300072801114954046397542697377502918243408203125f,1.6329000269904743181115236438927240669727325439453125f,1.2462999362021676130751757227699272334575653076171875f,-0.5837000796303171146206523189903236925601959228515625f,-0.5396000196874151111359196875127963721752166748046875f,0.85940005713552303223679018628899939358234405517578125f,-1.08160001805201044788873332436196506023406982421875f,1.68950001641362934634571502101607620716094970703125f,3.773399983419476910739831509999930858612060546875f,-1.224599980702250956454690822283737361431121826171875f,-2.73200011105462703397961377049796283245086669921875f,-0.020499964252113755946993478573858737945556640625f,0.58750004885718298819341498528956435620784759521484375f,-0.4742000119380662681578542105853557586669921875f,0.64729998525679033338775525407982058823108673095703125f,-1.4689000944852834695808496690005995333194732666015625f,2.16949998055770976179701392538845539093017578125f,-3.03649996625632123681270968518219888210296630859375f,3.719999976120889062514152101357467472553253173828125f,-0.858199986259639668872978290892206132411956787109375f,0.5504000382222253850983406664454378187656402587890625f,1.059699930257348654549787170253694057464599609375f,-0.79369989710971733387623316957615315914154052734375f,-2.8935000758580873281289314036257565021514892578125f,3.57269999946951788416527051595039665699005126953125f,-1.9553999356091029682147564017213881015777587890625f,0.0845000578016055214902735315263271331787109375f,-0.51820002015978072762436568154953420162200927734375f,0.7091000539079310716061854691361077129840850830078125f,1.3938999825030575774320595883182249963283538818359375f,-0.57229990773350059019008995164767839014530181884765625f,0.300399929965289747002543663256801664829254150390625f,-0.5855999114975336539146155701018869876861572265625f,1.7347999579504147504849242977797985076904296875f,0.0068999602407229243539177332422696053981781005859375f,0.35139998388141435725628980435431003570556640625f,-0.4149000509105620349004084346233867108821868896484375f,3.53700003839284082829408362158574163913726806640625f,1.353000022888182218849806304206140339374542236328125f,-0.071599975714089936218442744575440883636474609375f,1.4383999492771926131950976923690177500247955322265625f,0.38930002554953180293750847340561449527740478515625f,0.580099880748987661860383013845421373844146728515625f,0.45249995306134227490701960050500929355621337890625f,-2.9152000174812968680271296761929988861083984375f,-0.92110003694296072040970102534629404544830322265625f,0.1675999211870127236778671431238763034343719482421875f,-3.47370003407001615158833374152891337871551513671875f,-3.3733000667586932053154669119976460933685302734375f,0.429399964413044177291567393695004284381866455078125f,3.026499865114689047373985886224545538425445556640625f,-1.3677999185517411007140253786928951740264892578125f,-0.3117000153094531267328193280263803899288177490234375f,0.581399965515732475296317716129124164581298828125f,-4.1367999156162138518766369088552892208099365234375f,0.675400034683196093254764491575770080089569091796875f,3.256199969212710154664591755135916173458099365234375f,-1.2805999611929035353341532754711806774139404296875f,0.712900056736916365451861565816216170787811279296875f,0.697600012400744429186261186259798705577850341796875f,-0.206099923601002377182567215641029179096221923828125f,-2.8741999713696539942020535818301141262054443359375f,-1.42419994702488228455194985144771635532379150390625f,-2.73390009075999085297326018917374312877655029296875f,-1.424599940953406917998336211894638836383819580078125f,-0.5055000218078493645634807762689888477325439453125f,0.102200036558508511319587341859005391597747802734375f,1.046999971188604217076090208138339221477508544921875f,-1.32049992293864448811291367746889591217041015625f,1.607400074473023465770893380977213382720947265625f,0.9296000199481848635940650638076476752758026123046875f,-1.5670000295452763960923903141519986093044281005859375f,0.0993000772565582057183064534910954535007476806640625f,0.846400080925224784778038156218826770782470703125f,-0.60460002810657031968588626114069484174251556396484375f,0.9928000080332155530271620591520331799983978271484375f,3.272799956549702127261980422190390527248382568359375f,-0.832599964419007410043604977545328438282012939453125f,3.7424000225052242996071072411723434925079345703125f,-4.3864000545874244352262394386343657970428466796875f,-0.9423000023558725235517385954153724014759063720703125f,-0.3134000219762318462102257399237714707851409912109375f,-1.9983000513732436242975154527812264859676361083984375f,-0.036100036756693987261002121158526279032230377197265625f,1.806199969734251897790500152041204273700714111328125f,0.21709998450204637254756789843668229877948760986328125f,-1.442499998807907690689944502082653343677520751953125f,1.310499960247427875259518259554170072078704833984375f,2.979999953061341244620052748359739780426025390625f,3.438399952127038972093941993080079555511474609375f,-1.1435000146329397097133551142178475856781005859375f,-0.29189997948184698284279647850780747830867767333984375f,-3.272900048150121588008687467663548886775970458984375f,0.215900102293492768268379222718067467212677001953125f,2.1521999172002086453403535415418446063995361328125f,0.2322999874174607182197860311134718358516693115234375f,0.0708999912239625107446272522793151438236236572265625f,-0.7955999200284491479351345333270728588104248046875f,6.185100034379960476371707045473158359527587890625f,0.704799917903543349240180759807117283344268798828125f,-3.40090000398307967799382822704501450061798095703125f,-1.64809999326765588278931318200193345546722412109375f,-1.565199979576469058173415760393254458904266357421875f,1.480600052388013398996235991944558918476104736328125f,1.2765000033788374000920384787605144083499908447265625f,-0.89809997322559365073857406969182193279266357421875f,1.497099973270297201821676935651339590549468994140625f,-1.860899999885261113519163700402714312076568603515625f,-0.13640000235885396673296554581611417233943939208984375f,-0.4052000105336317492543685148120857775211334228515625f,-4.2081000969797361932478452217765152454376220703125f,2.435299979385733504244626601575873792171478271484375f,-0.16389994057491386048042159018223173916339874267578125f,-2.96709980980455956256491845124401152133941650390625f,-0.8944998957142231699180001669446937739849090576171875f,0.15100000546872716089552568519138731062412261962890625f,-0.46139999047517721297850812334218062460422515869140625f,-2.16170000607073387044465562212280929088592529296875f,-0.474900014179199470021330853342078626155853271484375f,-4.2459999535754331390080551500432193279266357421875f,-2.500699932447820739156441049999557435512542724609375f,-1.0384999812915918226963185588829219341278076171875f,-1.094900024423747897373004889232106506824493408203125f,0.6689999719560153490505172158009372651576995849609375f,3.16739995867237578153208232834003865718841552734375f,-0.49690004465580006609570773434825241565704345703125f,1.793799914856255117712180435773916542530059814453125f,-0.1017999662056556786637884215451776981353759765625f,-0.04729993422031497640745101307402364909648895263671875f,-2.46970000038594061919638988911174237728118896484375f,-2.70700010872632379488322840188629925251007080078125f,3.21850009893625799151095634442754089832305908203125f,3.6796999232351783604144657147116959095001220703125f,0.9723999677807084740521759158582426607608795166015625f,-1.162699897098543377893520300858654081821441650390625f,-1.31039995110630957952935204957611858844757080078125f,-2.58080014463588636175472856848500669002532958984375f,0.945899903474749503828888919088058173656463623046875f,0.39809998389855028033679218424367718398571014404296875f,-0.7374000199347729989085564739070832729339599609375f,-0.771499937273562697015449884929694235324859619140625f,-0.2223999260760856078178449024562723934650421142578125f,-1.380599937667697840737446313141845166683197021484375f,1.835099925191700176441145231365226209163665771484375f,0.6770999148577441228979978404822759330272674560546875f,0.272699988740682908172630050103180110454559326171875f,2.68879999558329618736252086819149553775787353515625f,3.72510004978403497233330199378542602062225341796875f,-1.0417999445244674472377255369792692363262176513671875f,1.99119999491721433315660760854370892047882080078125f,-3.09219998565241827037652910803444683551788330078125f,3.3112000555649405697522524860687553882598876953125f,0.32489991925880445933216833509504795074462890625f,1.525099913606048129821601833100430667400360107421875f,0.92580005535930365301311439907294698059558868408203125f,2.29149988280981897759147614124231040477752685546875f,-2.942899830667673910511439316906034946441650390625f,-4.6436000349685553345580046880058944225311279296875f,-2.6475000213086605782564220135100185871124267578125f,-2.12309991566985889477336968411691486835479736328125f,-0.74130002272203565638619693345390260219573974609375f,1.65280003392398366912630081060342490673065185546875f,1.610500065319241169703445848426781594753265380859375f,2.916999995868653883235310786403715610504150390625f,1.70160002939552068568218601285479962825775146484375f,2.0107999439731241153594964998774230480194091796875f,1.8411999064974502449132387482677586376667022705078125f,2.434199932049215586715718018240295350551605224609375f,0.33039997070133797496538363702711649239063262939453125f,0.1786999496176828738924768913420848548412322998046875f,-0.2618999412603688625722497818060219287872314453125f,-2.57299995814263837701219017617404460906982421875f,0.43049995570257337806907571575720794498920440673828125f,-2.0889999592527743743630708195269107818603515625f,2.43820007941052363520384460571222007274627685546875f,0.8072000045768914500143864643177948892116546630859375f,-0.334899931719898680260172341149882413446903228759765625f,1.7308000687330966638199925000662915408611297607421875f,2.950099978090822361309619736857712268829345703125f,1.9102000157117846601551036656019277870655059814453125f,0.47409997750222798718056083089322783052921295166015625f,1.7720000334568322131190143409185111522674560546875f,0.3326999041020866076934225930017419159412384033203125f,-1.46809996266439579670759485452435910701751708984375f,4.53400003530457684775001325760968029499053955078125f,-1.3231000918202102667464714613743126392364501953125f,-2.26949996660649855328983903746120631694793701171875f,0.80380001618415108932680368525325320661067962646484375f,-2.117799951334296881810814738855697214603424072265625f,0.6252999424867324851362582194269634783267974853515625f,-1.76629993849322186605377282830886542797088623046875f,0.214899968339503255609912457657628692686557769775390625f,0.68789998882487501941085383805329911410808563232421875f,0.560299978975951518123110872693359851837158203125f,-2.084299898368119130509512615390121936798095703125f,1.49390011458322380377694571507163345813751220703125f,1.24320000838786359764753797207958996295928955078125f,-0.3680000534355654107088184900931082665920257568359375f,-2.39830003161057891958307664026506245136260986328125f,-1.3280000287368884936967106114025227725505828857421875f,0.3295999320313338643018141738139092922210693359375f,-0.222299963445216963719985869829542934894561767578125f,-0.614900000078975939032943642814643681049346923828125f,1.199100046159326193873084775987081229686737060546875f,2.132300103572010829822147570666857063770294189453125f,-1.498599916746468085904098188620992004871368408203125f,1.3205999562755226239829653422930277884006500244140625f,3.819199997316301331551358089200221002101898193359375f,-0.5852999651923784796281324815936386585235595703125f,-0.12109996590167160235296250903047621250152587890625f,3.8767999583080410275215399451553821563720703125f,-0.2576999577462661950022493329015560448169708251953125f,1.694099938032775032326071595889516174793243408203125f,-2.23180000604763595362101114005781710147857666015625f,-0.93489992646723985014745039734407328069210052490234375f,1.751300028999148938879670822643674910068511962890625f,2.473899926307052243856787754339165985584259033203125f,-4.2182999695658676131415631971322000026702880859375f,-1.2680999694257983900769204410607926547527313232421875f,1.07439986906424156387629409437067806720733642578125f,-0.5982000495709467458027575048618018627166748046875f,-2.3729999545291047269301998312585055828094482421875f,3.855999995075165998059674166142940521240234375f,-3.6458000206582230617868845001794397830963134765625f,-0.427999950356781511118242633529007434844970703125f,-0.037899981281162897772674114094115793704986572265625f,2.559799986858667608657924574799835681915283203125f,1.403399871921539698860215139575302600860595703125f,-0.0886000187262883098782140223192982375621795654296875f,-0.5370000410191713147156633567647077143192291259765625f,2.5332001013338558692566948593594133853912353515625f,-3.008100033929197447690739863901399075984954833984375f,-0.185300120797752043699802015908062458038330078125f,-2.892400102598964739541997914784587919712066650390625f,-3.472199950243532118321354573708958923816680908203125f,-0.4006999557122592481306355693959631025791168212890625f,-2.024899929037690782251956989057362079620361328125f,3.282700014482438977125866585993207991123199462890625f,1.7687000584147869464146651807823218405246734619140625f,0.7889000636026246571219644465600140392780303955078125f,-2.562499931398779740021609541145153343677520751953125f,-1.2175000935792896772369431346305646002292633056640625f,0.3503001311168054687783524059341289103031158447265625f,1.959000013548882090930192134692333638668060302734375f,2.177699963445961284236318533658050000667572021484375f,0.095700005109609520559388329274952411651611328125f,0.81099997467920115212791642989031970500946044921875f,-1.0846000147327792273443947124178521335124969482421875f,2.31709995378702870993947726674377918243408203125f,-0.6441999535255151787538352436968125402927398681640625f,-0.0425000251084590718875233505968935787677764892578125f,-0.786999975733459589122276156558655202388763427734375f,-0.3585999142691489449674691059044562280178070068359375f,1.687999918486922990012999434839002788066864013671875f,-0.7972000542536370470969586676801554858684539794921875f,0.420699986557661331776358792922110296785831451416015625f,-1.9861000464051965508360808598808944225311279296875f,-0.4761000350616872989206740385270677506923675537109375f,0.5452000254720434924138317001052200794219970703125f,-0.696599971728027345108102963422425091266632080078125f,-1.80350017210096069675273611210286617279052734375f,1.3748999976947897661005981717607937753200531005859375f,2.277400088182092385835630921064876019954681396484375f,-0.61260006876438877210233613368473015725612640380859375f,-1.010500038906931852267234717146493494510650634765625f,0.1330999471858127503054447515751235187053680419921875f,3.4783999128252265364835693617351353168487548828125f,-0.8969999599754816532737322631874121725559234619140625f,0.4659999933987861719941747651319019496440887451171875f,0.73980005674511328805920129525475203990936279296875f,-0.2943000911518927065202433368540368974208831787109375f,-1.109999940358102232806913889362476766109466552734375f,-1.6493999812327315712678910131216980516910552978515625f,-2.056300016370414596167393028736114501953125f,0.8356000117450947950459294588654302060604095458984375f,0.47389988419264506003258929922594688832759857177734375f,-0.41130009262710753858982570818625390529632568359375f,0.55980012711584503648509780759923160076141357421875f,7.371600035523623972721907193772494792938232421875f,-1.6259999818317592001903904019854962825775146484375f,-1.954599989158660289234603624208830296993255615234375f,4.2537998894125212956396353547461330890655517578125f,1.449199999905375957354181082337163388729095458984375f,-4.1569000040687615893375550513155758380889892578125f,-3.772899980554729637560740229673683643341064453125f,-1.341400053302198802640532449004240334033966064453125f,0.000199997085332082047415269698831252753734588623046875f,-2.4340001004971565379264575312845408916473388671875f,-0.09130006783530164593543076989590190351009368896484375f,0.94899994304776225106934361974708735942840576171875f,0.9377998864583674265560375715722329914569854736328125f,-2.376200068845598867284252264653332531452178955078125f,3.02689996851086728923974078497849404811859130859375f,-0.73139995657503698023305105380131863057613372802734375f,-1.832199983100593509988129881094209849834442138671875f,-0.839900018891693189715397238614968955516815185546875f,-0.14950004625320467699367554814671166241168975830078125f,1.680400022780894619955915914033539593219757080078125f,1.1691999902754994788978137876256369054317474365234375f,-2.45220000422298678444121833308599889278411865234375f,0.2323999752685417252706656654481776058673858642578125f,-0.826300001469255196440144572989083826541900634765625f,0.463699847228080219707635478698648512363433837890625f,-1.446900016958267087829881347715854644775390625f,4.80200006683543367813626900897361338138580322265625f,0.57949995427578659956679985043592751026153564453125f,-0.7202000202566380959723346677492372691631317138671875f,1.75620000262856468253858110983856022357940673828125f,-0.3458999738082295660746012799791060388088226318359375f,-1.925100001224875345684495187015272676944732666015625f,-0.6612000731483098636687145699397660791873931884765625f,0.2474999914132054446014308268786408007144927978515625f,-0.0564999791644513749133693636395037174224853515625f,-2.76010001078024469478577884729020297527313232421875f,-2.26959997625052967151759730768389999866485595703125f,-1.9451999833762652070134890891495160758495330810546875f,0.54159999735802433207254580338485538959503173828125f,0.4241000479474668072299436971661634743213653564453125f,1.6488000493951151526061948970891535282135009765625f,0.2112000081792457795160089517594315111637115478515625f,3.11149997098743913426233120844699442386627197265625f,-4.210500003442167127332140807993710041046142578125f,-4.32100016841292511315941737848334014415740966796875f,-1.59809998082518500694959584507159888744354248046875f,-1.573699929314853473982793730101548135280609130859375f,1.963900019606946489858501081471331417560577392578125f,1.786799895816295613570900968625210225582122802734375f,1.68940006587132796056494044023565948009490966796875f,-2.385499956279994560048862695111893117427825927734375f,3.173799936109034991460475794156081974506378173828125f,-0.960799952913823585021191320265643298625946044921875f,0.69360010392367854592521325685083866119384765625f,-2.7447998557657005136434236192144453525543212890625f,1.389399984976649182755181755055673420429229736328125f,-0.50479995221346596334655032478622160851955413818359375f,2.43779997977688989152511567226611077785491943359375f,-0.3189999385029071987673887633718550205230712890625f,-2.645200016680359311749271000735461711883544921875f,-0.5224999414198112646090521593578159809112548828125f,1.253599945505708301851655050995759665966033935546875f,-0.937600072862206257440220724674873054027557373046875f,-3.6566000400871043751749311923049390316009521484375f,0.86660005401968820404334792328882031142711639404296875f,0.3509000384792688720381192979402840137481689453125f,2.052100024865568084209144217311404645442962646484375f,-0.2227000473022451298987789414240978658199310302734375f,0.8030997884884480964018393933656625449657440185546875f,-0.54639991935938569955766297425725497305393218994140625f,-0.311400113409756595928001843276433646678924560546875f,2.93779998877346582020209098118357360363006591796875f,-5.1799000533133749968328629620373249053955078125f,2.7541998925797646080582126160152256488800048828125f,0.0627999727360916415452862793245003558695316314697265625f,-1.0179999810531723536399795193574391305446624755859375f,-2.531900080921501317021693466813303530216217041015625f,-1.367399974858759303941724283504299819469451904296875f,3.84519989098906567193125738413073122501373291015625f,-1.655700037575511540666184373549185693264007568359375f,-1.629200019761175344257253527757711708545684814453125f,1.5652000741988427723327959029120393097400665283203125f,-2.55429996572136719379386704531498253345489501953125f,-1.643600033571569696277947514317929744720458984375f,-2.159100029321014968530789701617322862148284912109375f,1.04570001464635264909475154126994311809539794921875f,2.495100085118411303852781202294863760471343994140625f,1.824799998779592868913823622278869152069091796875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_strided_dilated_padded)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,8,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,3,7,7,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{3,2,2,3},        // move_strides
                                                                  Strides{2,1,3,2},        // filter_dilation
                                                                  CoordinateDiff{2,4,6,8}, // below_pads
                                                                  CoordinateDiff{1,3,5,7}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f,.81f,-.90f,.66f,-.06f,-.42f,.47f,.12f,.59f,-.26f,.12f,-.93f,.61f,-.19f,-.96f,-.63f,-.86f,-.96f,.33f,-.80f,-.31f,-.22f,-.73f,-.39f,.69f,-.59f,.17f,-.24f,.75f,-.61f,-.90f,.23f,.75f,-.51f,-.08f,-.47f,.37f,-.30f,.36f,.75f,.93f,.97f,.52f,.64f,.98f,-.05f,.34f,-.66f,.60f,.44f,.48f,.53f,-.61f,-.57f,.18f,-.28f,.80f,.14f,.31f,-.54f,-.56f,.48f,.28f,-.86f,-.04f,-.12f,-.97f,.87f,-.82f,-.33f,-.75f,.67f,-.05f,.83f,.14f,.84f,-.47f,.25f,-.52f,-.48f,-.86f,.15f,.19f,-.45f,-.48f,-.33f,-.98f,-.61f,.13f,.83f,-.45f,.78f,.16f,-.52f,.37f,.07f,-.98f,.78f,.91f,-.26f,-.30f,.10f,-.92f,.17f,.10f,.44f,-.31f,-.50f,-.85f,-.29f,.85f,-.47f,.13f,-.59f,-.71f,.34f,.39f,-.72f,-.46f,-.66f,.44f,.95f,.09f,-.10f,.81f,.37f,-.82f,.24f,-.58f,-.79f,.61f,-.73f,.45f,-.76f,-.55f,-.06f,-.59f,-.04f,.42f,.08f,-.51f,-.87f,-.54f,-.25f,.51f,-.14f,.78f,.06f,.44f,-.40f,.11f,-.47f,.36f,.23f,.12f,.49f,-.40f,-.39f,.45f,.74f,-.99f,-.21f,.03f,.85f,.59f,.67f,.15f,.74f,.34f,-.49f,-.59f,-.82f,-.82f,.60f,.12f,-.61f,-.19f,.52f,-.32f,.96f,.46f,.95f,.27f,.97f,-.01f,.18f,.81f,-.55f,.05f,-.76f,-.74f,.08f,.58f,-.77f,.43f,-.55f,-.78f,.37f,.39f,-.37f,-.03f,-.75f,-.46f,.96f,.50f,.55f,-.81f,.39f,-.54f,.42f,-.92f,-.40f,.31f,.16f,-.87f,-.58f,.41f,-.31f,-.74f,-.36f,.18f,.00f,.32f,.32f,.39f,-.91f,.50f,.75f,.38f,.60f,.11f,-.56f,.61f,-.92f,.38f,.10f,-.30f,-.42f,.35f,-.51f,.58f,.84f,-.37f,-.00f,.82f,.15f,.82f,-.89f,.34f,.29f,-.45f,-.31f,-.54f,.02f,-.69f,.15f,-.77f,-.14f,-.89f,-.19f,-.03f,-.41f,.20f,-.28f,-.60f,.25f,-.18f,.82f,-.38f,-.20f,-.25f,-.18f,-.85f,.88f,-.02f,.97f,.62f,-.97f,.63f,.36f,-.13f,-.93f,-.84f,.63f,.87f,-.18f,-.34f,-.72f,.63f,.13f,-.05f,-.56f,.81f,-.19f,-.89f,.61f,.71f,.22f,.45f,-.44f,.89f,.67f,.47f,.85f,-.50f,-.03f,-.73f,.20f,-.86f,.71f,-.12f,-.16f,.50f,.47f,-.95f,-.37f,.81f,.85f,.69f,.46f,-.77f,.25f,.15f,.09f,-.12f,-.32f,.93f,-.04f,-.66f,-.41f,-.91f,-.84f,-.32f,-.51f,.65f,.09f,-.18f,.80f,.55f,-.30f,.56f,.22f,.28f,-.48f,-.61f,-.15f,-.65f,-.38f,.03f,-.83f,.61f,.83f,-.14f,.39f,-.44f,-.49f,-.70f,.35f,-.72f,.51f,.09f,.31f,.98f,.63f,.74f,.25f,-.30f,.20f,.08f,.81f,.69f,.16f,-.85f,.69f,.20f,-.68f,.99f,.14f,-.65f,-.21f,.03f,.06f,-.16f,.80f,.95f,.90f,-.13f,.05f,.84f,.56f,-.42f,-.58f,.02f,-.16f,-.68f,.03f,.94f,-.20f,-.99f,.99f,-.20f,-.69f,.25f,-.00f,.55f,-.11f,.74f,.96f,-.12f,-.50f,.81f,.97f,-.56f,.40f,.94f,.34f,-.98f,-.04f,-.45f,-.89f,-.04f,-.07f,-.35f,.58f,-.52f,.85f,.94f,-.92f,-.46f,-.51f,-.30f,-.98f,.04f,.45f,-.08f,.53f,-.78f,.43f,.90f,-.48f,-.31f,-.84f,-.70f,-.79f,-.33f,.13f,.68f,-.44f,.21f,.62f,.73f,.15f,.72f,.31f,.21f,-.78f,-.89f,-.96f,-.69f,.12f,.45f,-.47f,-.91f,-.33f,.75f,.87f,.35f,-.33f,.08f,.34f,-.93f,.55f,-.30f,.93f,.60f,-.06f,.89f,-.84f,-.34f,.59f,.20f,-.63f,-.40f,.10f,-.47f,-.79f,-.05f,-.28f,-.97f,.68f,.58f,-.96f,-.32f,.09f,-.14f,.20f,.86f,.33f,-.27f,-.62f,.99f,-.56f,.87f,-.75f,-.53f,.86f,-.44f,-.38f,-.30f,-.73f,-.95f,-.19f,.16f,-.16f,-.87f,-.41f,.77f,-.40f,.90f,-.10f,.67f,-.28f,-.81f,.66f,-.97f,-.21f,-.08f,-.57f,-.80f,.07f,-.86f,-.94f,-.33f,-.43f,-.63f,.52f,-.74f,-.07f,-.17f,.75f,.10f,.97f,.70f,-.01f,.13f,.28f,-.59f,-.25f,.39f,.44f,.18f,.38f,-.95f,.93f,-.80f,.37f,.08f,.42f,-.84f,-.05f,.21f,.96f,.93f,-.60f,.96f,-.50f,.17f,-.74f,.47f,-.16f,-.05f,-.36f,-.46f,-.62f,.95f,.44f,-.45f,.42f,-.31f,.79f,.37f,-.21f,-.78f,-.08f,-.70f,-.83f,.02f,.10f,-.45f,-.12f,-.45f,-.90f,.95f,.60f,.34f,-.38f,-.22f,.37f,.04f,-.21f,-.63f,.10f,.61f,-.76f,-.48f,-.45f,.27f,.96f,.75f,.09f,.77f,.99f,-.35f,.81f,.95f,.14f,-.41f,-.54f,-.43f,.25f,.36f,-.61f,-.11f,.83f,.14f,.56f,-.42f,.80f,.35f,.11f,-.51f,-.68f,-.28f,.05f,.54f,-.01f,-.74f,-.19f,-.91f,.86f,-.05f,.19f,-.71f,-.22f,.39f,.97f,.08f,.92f,-.36f,.29f,.08f,.99f,-.31f,-.94f,-.37f,.17f,.41f,.41f,-.57f,-.56f,-.05f,.88f,.19f,.91f,.34f,.07f,.39f,-.56f,.76f,.87f,-.82f,.79f,-.29f,.29f,.79f,-.00f,-.68f,.22f,.75f,-.29f,-.92f,-.57f,.88f,-.78f,-.99f,.88f,.34f,.82f,.86f,-.17f,.79f,.12f,.42f,-.50f,.23f,.83f,.25f,.77f,.62f,-.81f,-.24f,.47f,-.49f,-.45f,.06f,-.38f,-.01f,.15f,-.16f,-.63f,-.25f,-.99f,.24f,.85f,-.69f,-.37f,-.27f,.01f,-.22f,.73f,.63f,-.48f,.87f,.90f,-.64f,.68f,.59f,.11f,-.83f,.17f,.87f,-.88f,.76f,-.85f,-.20f,.94f,.71f,.16f,-.40f,-.73f,.91f,.61f,.89f,-.73f,.79f,-.48f,-.18f,.74f,.64f,-.74f,-.89f,-.84f,-.28f,-.00f,.45f,.50f,.78f,.84f,-.54f,.66f,.67f,-.58f,-.67f,.09f,-.93f,-.50f,-.71f,-.57f,.44f,-.20f,.42f,-.65f,-.81f,.37f,.21f,-.82f,-.16f,.84f,.04f,.51f,.33f,-.43f,.44f,-.82f,.92f,-.33f,-.92f,.76f,-.33f,.83f,-.21f,-.25f,.28f,.78f,.70f,.04f,.65f,-.44f,.28f,-.61f,-.85f,.26f,-.86f,.41f,.02f,-.44f,.93f,.27f,.79f,-.60f,.91f,-.25f,-.92f,.13f,.87f,-.29f,-.89f,-.56f,-.91f,-.95f,-.72f,-.62f,-.99f,-.89f,-.89f,.64f,-.08f,.70f,-.78f,.82f,-.54f,.97f,-.26f,.12f,.46f,-.26f,-.46f,-.65f,.04f,-.50f,.34f,-.78f,-.54f,-.29f,.13f,.19f,.01f,-.77f,-.20f,-.98f,.43f,.37f,.80f,-.97f,-.99f,-.66f,.55f,.85f,.45f,.58f,-.29f,-.54f,-.95f,.41f,.14f,-.84f,.13f,.06f,.37f,.23f,.76f,.88f,.12f,-.84f,-.53f,-.26f,.26f,.67f,-.24f,.76f,.76f,.12f,-.13f,-.05f,-.12f,.92f,.40f,-.68f,.68f,.00f,.19f,-.72f,.06f,-.66f,.55f,.77f,-.71f,.84f,.59f,.15f,.27f,.38f,-.99f,-.03f,.65f,-.94f,.55f,-.51f,.40f,.32f,.33f,-.93f,.49f,.83f,.98f,.32f,-.45f,.32f,-.22f,.37f,-.35f,-.30f,-.63f,-.72f,.86f,-.95f,.70f,.44f,.12f,-.66f,.39f,.67f,-.56f,-.25f,-.64f,-.15f,.33f,.37f,-.26f,.65f,.13f,-.98f,-.27f,-.94f,.90f,-.30f,-.35f,.24f,.20f,-.93f,.24f,-.17f,-.80f,.24f,.92f,-.81f,-.61f,.05f,-.72f,.12f,.95f,-.07f,-.04f,.74f,-.10f,-.07f,-.55f,-.07f,.21f,-.77f,-.79f,.52f,-.76f,-.02f,-.18f,-.88f,-.76f,.78f,-.74f,-.22f,.26f,-.51f,.91f,.33f,.82f,.67f,.89f,-.57f,.47f,-.30f,.06f,.46f,.40f,.33f,.36f,-.12f,.80f,-.46f,.69f,.60f,.96f,.69f,-.85f,-.16f,-.44f,-.13f,-.58f,-.93f,.05f,.51f,-.02f,.79f,-.36f,.27f,.74f,.67f,.61f,-.67f,-.97f,-.01f,.78f,-.94f,.11f,-.35f,-.45f,-.27f,.51f,.07f,-.34f,.46f,.85f,-.25f,.84f,.40f,-.68f,-.66f,-.89f,.72f,-.52f,-.46f,-.19f,.91f,.09f,-.56f,.16f,.01f,-.29f,.86f,-.05f,.50f,-.98f,.98f,-.25f,.21f,-.52f,-.45f,.07f,.97f,.62f,.51f,.91f,-.42f,.80f,-.33f,-.77f,-.98f,.82f,-.35f,.96f,.25f,-.54f,.50f,-.66f,.54f,-.14f,.53f,-.96f,-.05f,-.85f,-.03f,.12f,-.71f,-.06f,.47f,-.80f,.70f,.88f,.57f,-.07f,-.04f,-.13f,.80f,.59f,-.23f,-.66f,-.34f,-.78f,.95f,.14f,.24f,-.37f,-.29f,.51f,-.94f,-.93f,-.62f,-.68f,.58f,-.13f,.16f,.50f,-.23f,-.01f,-.24f,.82f,-.07f,-.01f,-.49f,.23f,-.00f,.83f,-.60f,.72f,.34f,-.12f,-.11f,-.07f,.01f,.15f,-.23f,.58f,-.60f,.74f,.43f,-.15f,-.41f,-.29f,.66f,.60f,.74f,-.13f,.48f,-.13f,-.35f,.29f,-.64f,-.76f,.27f,.56f,.72f,.51f,-.75f,-.30f,.53f,-.62f,.64f,.11f,-.93f,-.12f,-.43f,.96f,.49f,-.66f,.47f,-.96f,-.60f,.72f,.64f,.39f,-.53f,-.61f,-.37f,.99f,-.74f,.50f,-.32f,.82f,-.98f,-.06f,-.68f,.32f,-.28f,-.64f,.30f,-.44f,.60f,.16f,-.29f,-.21f,-.12f,.19f,.26f,-.32f,-.90f,.88f,.96f,-.28f,.51f,-.86f,.28f,.43f,.82f,-.21f,-.15f,.15f,-.70f,-.70f,.05f,-.82f,-.85f,-.57f,.13f,-.22f,.53f,.80f,.43f,.30f,-.77f,-.95f,.95f,.95f,-.21f,.56f,.56f,-.66f,-.70f,-.03f,.51f,.79f,.00f,-.68f,.25f,-.15f,.12f,.07f,-.18f,-.57f,.81f,.22f,-.80f,.14f,.14f,.19f,.65f,-.06f,.72f,-.96f,-.93f,-.96f,-.35f,.06f,.92f,-.01f,-.16f,.13f,.00f,-.24f,.79f,-.15f,-.29f,-.83f,-.38f,-.91f,-.17f,-.21f,-.89f,-.24f,.40f,-.43f,-.31f,.45f,.74f,.77f,-.98f,.53f,.87f,-.42f,-.37f,-.95f,.02f,-.03f,.21f,-.39f,-.27f,-.66f,.62f,-.34f,.34f,.89f,-.64f,-.65f,-.51f,-.98f,.14f,-.49f,.27f,.10f,-.71f,-.20f,-.01f,-.51f,.50f,-.30f,.98f,-.46f,.10f,-.60f,-.71f,-.95f,.49f,.59f,.00f,-.19f,.11f,-.41f,.08f,-.50f,.48f,.19f,-.80f,.55f,-.27f,.14f,-.66f,.04f,-.16f,-.40f,.18f,-.50f,.89f,-.98f,.27f,-.39f,.91f,-.07f,-.88f,-.76f,-.34f,.10f,.80f,-.05f,-.31f,-.87f,.65f,.26f,-.62f,.55f,.57f,-.98f,.11f,.14f,-.63f,-.39f,.13f,-.87f,-.60f,.92f,-.08f,-.88f,.08f,-.82f,-.12f,.71f,.41f,-.81f,.17f,.16f,-.52f,-.44f,-.58f,.24f,-.16f,.42f,-.87f,-.41f,.86f,.51f,-.42f,.57f,.44f,-.90f,-.37f,-.11f,-.13f,.12f,-.13f,-.80f,-.44f,-.12f,.76f,-.65f,-.00f,.94f,-.49f,.64f,-.20f,-.02f,-.30f,-.70f,.81f,.37f,-.12f,.06f,-.73f,-.99f,-.24f,.66f,.32f,-.94f,.29f,.47f,.32f,-.20f,.90f,.37f,.14f,-.65f,-.30f,.03f,-.81f,-.85f,-.45f,.95f,.52f,-.64f,.70f,-.32f,-.50f,.34f,.33f,.44f,.94f,.39f,.78f,-.99f,.35f,-.04f,.56f,-.65f,.33f,-.57f,.23f,-.17f,-.97f,.70f,-.68f,-.23f,-.60f,.20f,-.93f,.00f,.16f,.92f,-.19f,.72f,.28f,.55f,.97f,.44f,-.40f,-.03f,.86f,.27f,.31f,-.50f,.16f,.03f,-.49f,-.09f,-.64f,-.17f,-.01f,.88f,.07f,.45f,-.45f,-.36f,-.40f,.69f,-.54f,-.44f,-.74f,.00f,-.49f,-.40f,-.92f,.16f,-.21f,-.39f,-.26f,-.63f,.14f,-.53f,.31f,-.83f,.33f,.31f,.06f,-.88f,.12f,-.20f,.63f,.35f,.79f,-.25f,.77f,-.79f,.16f,.64f,-.49f,-.14f,.09f,-.00f,.46f,-.96f,-.00f,.67f,.74f,-.23f,-.25f,-.16f,.45f,-.87f,.73f,.89f,-.40f,.85f,.70f,-.98f,.11f,.27f,.64f,.22f,-.38f,.80f,-.38f,.98f,.87f,.96f,.54f,.88f,.46f,.00f,-.08f,.94f,.02f,-.97f,.16f,-.04f,.44f,-.45f,-.58f,.38f,-.49f,.93f,.78f,.84f,-.35f,.28f,.96f,.16f,-.58f,.32f,.34f,-.43f,.34f,.68f,.92f,-.69f,.48f,.42f,-.13f,-.54f,.09f,.70f,-.85f,.74f,-.55f,-.18f,.45f,-.49f,.12f,.33f,-.49f,-.68f,-.67f,.60f,.61f,-.87f,.13f,.68f,-.56f,.83f,-.16f,.26f,-.18f,-.80f,-.47f,.80f,-.25f,.36f,-.12f,-.98f,.78f,-.58f,.76f,-.25f,.34f,.42f,.76f,.09f,-.97f,.57f,.01f,.82f,.02f,-.67f,-.15f,.99f,-.27f,.19f,-.16f,-.28f,-.70f,.17f,.68f,.00f,.81f,-.88f,-.43f,-.71f,-.90f,-.12f,-.31f,-.15f,-.12f,-.44f,.58f,.26f,-.41f,-.23f,-.06f,-.71f,-.21f,.66f,-.27f,.31f,.55f,-.43f,-.23f,-.98f,.17f,-.03f,.66f,.13f,.41f,-.20f,-.93f,-.33f,-.25f,.23f,-.15f,.19f,.73f,.24f,-.49f,.16f,.24f,-.12f,-.21f,-.61f,-.11f,-.23f,-.98f,-.51f,.19f,.09f,.32f,.99f,.28f,.19f,.66f,.74f,.89f,-.10f,.04f,-.12f,.79f,.36f,.17f,.51f,-.66f,-.09f,.75f,-.88f,-.67f,-.74f,-.63f,-.36f,-.93f,.61f,-.12f,-.00f,.06f,.58f,-.27f,.73f,-.97f,-.03f,-.97f,.99f,-.11f,-.55f,-.59f,-.44f,-.90f,-.13f,.20f,.85f,.90f,.38f,-.46f,-.41f,.18f,-.87f,-.82f,.61f,-.75f,.20f,-.74f,-.68f,-.22f,-.21f,.68f,-.00f,.38f,-.50f,-.21f,.11f,.94f,-.98f,.59f,-.69f,-.36f,.92f,-.76f,-.73f,-.29f,.23f,-.52f,.18f,.26f,.37f,.08f,.35f,-.61f,.71f,.36f,.00f,-.67f,.57f,.63f,-.04f,.06f,-.96f,.40f,.37f,.86f,.05f,.49f,.78f,-.47f,-.90f,.53f,-.30f,-.57f,-.72f,.25f,.06f,.98f,.89f,-.66f,-.83f,.55f,.35f,.84f,.27f,-.56f,-.40f,-.56f,-.40f,-.83f,.69f,-.05f,-.39f,.87f,-.81f,-.29f,-.94f,.10f,-.32f,-.48f,.43f,-.40f,-.64f,.31f,.98f,-.96f,-.99f,-.70f,.23f,.95f,-.59f,-.04f,-.06f,.95f,-.52f,.18f,.79f,-.28f,-.30f,-.23f,.66f,.49f,-.21f,-.84f,-.27f,.85f,.12f,.05f,.39f,-.21f,.49f,-.28f,-.07f,.96f,.00f,.78f,-.54f,.75f,.15f,.14f,-.21f,-.19f,.06f,.42f,.34f,-.41f,.11f,.44f,-.02f,.89f,.56f,-.78f,.75f,.06f,-.22f,-.32f,-.78f,-.52f,-.46f,.64f,-.31f,-.53f,-.20f,.66f,.29f,.91f,-.84f,-.82f,.67f,-.92f,-.56f,.26f,-.42f,-.93f,.13f,.90f,-.20f,-.40f,.56f,-.60f,.39f,-.01f,.67f,-.77f,-.70f,.30f,-.21f,.04f,-.05f,-.70f,.87f,-.99f,.23f,-.11f,-.99f,-.72f,.22f,.82f,.59f,-.20f,-.40f,.28f,-.91f,-.44f,-.49f,.67f,.33f,.15f,-.09f,-.77f,-.65f,-.95f,.39f,-.02f,-.08f,-.60f,.10f,-.72f,.51f,-.17f,-.80f,-.07f,.92f,-.81f,.65f,.86f,.51f,-.99f,.32f,-.47f,-.37f,-.95f,.59f,-.46f,-.63f,.84f,.84f,-.61f,.97f,.17f,.48f,.94f,-.02f,.98f,.19f,.16f,.40f,.16f,-.40f,-.94f,-.61f,.49f,-.60f,.52f,-.62f,.91f,.67f,.25f,.68f,-.03f,-.03f,.31f,-.16f,.72f,.28f,-.13f,-.10f,-.47f,-.49f,-.79f,-.24f,-.39f,-.93f,-.93f,-.64f,.79f,.06f,-.73f,-.05f,-.77f,-.42f,-.10f,-.28f,-.26f,.58f,.07f,.80f,.52f,-.68f,.86f,-.35f,.79f,.50f,-.83f,-.52f,.75f,-.54f,.49f,.17f,.29f,.45f,-.50f,-.08f,.81f,.98f,.57f,-.57f,-.73f,-.54f,-.94f,-.91f,.13f,.74f,-.31f,-.23f,-.85f,-.98f,-.88f,-.98f,.70f,-.92f,.85f,-.96f,-.71f,.46f,.07f,-.41f,.80f,.35f,.64f,.50f,-.57f,.86f,.13f,.09f,-.13f,.78f,.51f,.66f,.18f,.63f,.66f,.05f,.06f,-.87f,-.33f,-.19f,-.55f,.94f,-.52f,-.74f,.21f,.33f,.50f,.90f,.53f,.80f,.53f,-.15f,.02f,-.84f,-.14f,.19f,.51f,.26f,.16f,-.88f,.98f,-.73f,.28f,.64f,-.76f,.20f,-.74f,.09f,.21f,-.67f,.63f,-.74f,.18f,-.03f,-.25f,-.42f,.76f,-.79f,.98f,-.48f,.94f,.10f,.36f,.47f,-.56f,-.69f,-.74f,.37f,.49f,.78f,-.57f,.14f,-.83f,.15f,-.88f,.91f,.25f,-.89f,-.98f,-.67f,.74f,.53f,-.50f,-.54f,-.94f,-.93f,.69f,.15f,.76f,.60f,.86f,-.96f,-.52f,.84f,-.73f,.10f,.35f,-.13f,.53f,.43f,.19f,-.59f,-.02f,.43f,.50f,-.18f,.03f,-.13f,-.21f,.46f,-.07f,-.99f,.94f,-.16f,-.43f,.25f,-.55f,-.77f,-.13f,-.99f,.98f,.69f,.92f,-.77f,-.89f,-.03f,.03f,-.10f,.48f,.47f,-.95f,.87f,-.12f,.56f,.36f,.56f,-.04f,-.97f,.57f,-.85f,-.45f,.48f,-.54f,.90f,-.46f,.49f,-.02f,.25f,-.83f,.77f,-.38f,.09f,.35f,.89f,-.79f,-.79f,.13f,-.14f,-.27f,-.49f,.46f,.43f,-.91f,-.79f,-.67f,-.15f,.73f,.88f,.17f,-.03f,.22f,.43f,.47f,-.72f,.38f,.23f,-.28f,-.14f,-.02f,-.23f,.47f,.48f,-.11f,-.42f,-.19f,.20f,-.84f,.91f,-.17f,-.52f,.46f,-.53f,.55f,-.29f,.80f,-.94f,-.24f,.53f,.18f,.52f,-.91f,-.47f,-.98f,.82f,.81f,.89f,-.94f,-.18f,.26f,.98f,-.97f,-.99f,-.82f,.67f,-.88f,.76f,-.73f,.25f,.82f,-.27f,-.16f,.35f,-.61f,.39f,.79f,-.57f,-.41f,-.56f,-.35f,.26f,-.34f,.74f,-.10f,.98f,-.55f,-.38f,-.91f,-.06f,-.85f,.62f,-.34f,.51f,-.83f,.91f,-.09f,-.54f,-.79f,-.37f,.79f,.74f,-.86f,-.52f,.45f,.04f,.65f,-.26f,.70f,.88f,-.13f,.36f,.23f,.06f,.73f,.16f,-.53f,-.91f,-.56f,-.07f,.72f,-.17f,-.13f,.56f,.11f,.19f,.46f,-.31f,.76f,.95f,.68f,.12f,.60f,-.20f,-.89f,.80f,.78f,-.21f,-.04f,.79f,-.18f,.28f,-.37f,-.39f,.92f,-.73f,-.55f,-.32f,-.15f,-.20f,-.59f,.71f,.57f,-.60f,-.58f,-.36f,.05f,.22f,.08f,-.37f,-.73f,.00f,.99f,-.44f,-.83f,.30f,-.39f,-.49f,.40f,-.33f,-.64f,-.07f,.24f,-.80f,.38f,.65f,.18f,.88f,.99f,-.18f,-.08f,.45f,-.76f,-.30f,.87f,.24f,-.21f,-.28f,.62f,-.71f,.57f,.92f,.01f,.70f,.44f,-.77f,-.47f,-.70f,.72f,-.67f,-.60f,.08f,-.26f,.26f,.16f,-.94f,-.06f,.46f,.98f,-.90f,.17f,-.45f,-.43f,-.00f,.71f,-.82f,-.59f,-.44f,.03f,.98f,-.85f,-.33f,.45f,.22f,-.04f,.59f,.87f,.91f,.97f,-.97f,.69f,-.60f,.61f,-.59f,-.56f,-.71f,-.35f,.13f,.06f,.81f,-.19f,.45f,.56f,-.03f,.09f,.28f,-.75f,.97f,.70f,.98f,-.56f,-.62f,.52f,-.57f,.48f,.04f,-.31f,-.60f,-.22f,-.39f,.05f,-.27f,.15f,-.40f,-.47f,.74f,.14f,.96f,-.35f,-.12f,-.95f,.32f,-.99f,.06f,-.22f,-.82f,-.89f,.89f,.00f,.98f,-.90f,.48f,.79f,.87f,.10f,-.29f,.79f,-.88f,-.07f,-.95f,-.70f,.06f,-.44f,.17f,.94f,.54f,.56f,.83f,-.89f,.80f,-.41f,.40f,-.13f,.44f,-.20f,-.29f,.59f,-.58f,-.80f,-.37f,-.33f,-.10f,-.51f,-.90f,-.16f,-.14f,.97f,.06f,.77f,-.70f,-.70f,.30f,-.36f,-.17f,-.17f,.70f,.25f,.24f,-.14f,.22f,.81f,-.16f,.07f,-.89f,.29f,-.60f,-.95f,.26f,-.20f,-.04f,-.90f,.55f,.80f,-.66f,.51f,.59f,.71f,-.51f,.73f,.89f,-.74f,.88f,.70f,.08f,.61f,-.83f,-.89f,-.93f,.34f,-.41f,.85f,-.74f,-.46f,-.48f,.05f,.86f,-.23f,.82f,.92f,-.41f,-.61f,-.76f,.22f,-.80f,-.29f,.18f,-.05f,-.96f,-.47f,.74f,.81f,-.73f,.69f,-.74f,-.05f,-.80f,-.89f,.13f,-.50f,-.40f,.16f,.03f,.85f,-.23f,.63f,.88f,-.77f,-.58f,.47f,-.70f,-.40f,.61f,.62f,-.23f,-.46f,-.19f,.14f,-.18f,.67f,.21f,-.57f,-.75f,-.04f,.20f,.82f,-.18f,-.44f,-.48f,.11f,.50f,-.60f,.67f,-.60f,.48f,.58f,-.92f,.20f,-.86f,.48f,-.87f,.47f,-.43f,.04f,-.90f,-.50f,-.79f,-.28f,-.25f,-.20f,.49f,-.01f,.28f,.83f,-.03f,-.32f,-.72f,.76f,.80f,-.53f,-.64f,-.34f,.59f,-.81f,.20f,.10f,-.13f,-.40f,-.51f,-.57f,.13f,.93f,.89f,-.03f,-.09f,-.96f,-.41f,-.32f,.14f,-.94f,-.21f,.51f,.03f,-.50f,.54f,-.52f,-.23f,.22f,-.73f,-.49f,-.80f,-.84f,-.26f,.53f,-.61f,.81f,-.80f,.83f,.35f,.49f,-.39f,-.10f,.64f,.12f,.65f,.78f,-.84f,-.32f,.01f,-.38f,-.98f,-.75f,.68f,.16f,.56f,-.23f,-.87f,.37f,.19f,.71f,-.73f,.07f,-.17f,.20f,.08f,.33f,-.49f,.67f,-.58f,-.14f,-.60f,-.89f,-.58f,-.06f,.11f,-.14f,.94f,-.81f,-.54f,-.63f,-.89f,-.35f,.90f,.73f,.67f,-.27f,-.36f,.00f,.11f,.48f,.38f,-.78f,.90f,-.36f,-.74f,-.50f,.90f,-.77f,.78f,-.08f,.76f,-.91f,-.97f,-.38f,-.00f,.59f,.79f,-.53f,.21f,.34f,-.79f,.90f,-.42f,-.14f,-.44f,-.25f,-.29f,.34f,-.08f,.76f,.53f,.24f,.07f,.01f,-.29f,-.76f,-.43f,-.40f,.23f,-.66f,.62f,.79f,-.07f,-.26f,.62f,-.18f,-.66f,.90f,.19f,.58f,.59f,-.24f,-.75f,-.30f,.13f,.68f,.08f,.00f,-.00f,.81f,-.56f,.41f,.58f,.50f,-.40f,.59f,-.45f,-.56f,.66f,.02f,.74f,-.27f,.70f,-.50f,-.76f,.24f,.94f,-.61f,.74f,-.95f,.76f,-.24f,.09f,.87f,-.02f,-.73f,-.43f,-.26f,.39f,-.78f,-.50f,-.52f,-.84f,-.81f,-.51f,.86f,-.27f,.27f,-.92f,.39f,.02f,.12f,-.47f,-.75f,-.69f,.00f,-.46f,.19f,.21f,-.75f,.65f,.12f,.74f,-.92f,-.88f,.68f,.85f,-.38f,-.68f,.43f,-.88f,.12f,-.02f,-.77f,.27f,.40f,.53f,.95f,.99f,.56f,.62f,-.68f,-.69f,-.87f,.09f,-.91f,-.25f,-.66f,.19f,-.69f,-.54f,.46f,.21f,-.76f,.84f,-.83f,-.30f,-.30f,.16f,.38f,.39f,-.04f,.32f,-.68f,-.43f,.27f,.08f,.41f,.93f,-.61f,-.54f,.52f,-.99f,.04f,.74f,-.26f,.25f,-.28f,-.60f,.52f,-.68f,.75f,-.62f,-.32f,-.51f,-.69f,-.61f,.90f,-.71f,.81f,-.20f,.24f,.92f,-.25f,-.76f,.88f,.88f,.42f,-.21f,.11f,.33f,-.09f,.01f,-.72f,.96f,.20f,-.70f,-.59f,.48f,-.03f,.42f,.54f,.67f,-.21f,-.13f,.55f,-.11f,.26f,.52f,.83f,.76f,.21f,.51f,-.35f,.49f,.04f,-.62f,.88f,.04f,.00f,.33f,-.68f,.72f,.78f,-.69f,.54f,.04f,-.69f,-.51f,.56f,-.05f,-.68f,.16f,.62f,-.90f,.77f,-.25f,-.78f,.25f,-.86f,.72f,-.82f,-.13f,-.23f,.77f,-.06f,.13f,.29f,-.79f,.13f,-.77f,-.56f,-.22f,-.17f,-.20f,-.35f,.91f,-.92f,.70f,-.07f,.99f,-.44f,.00f,-.91f,.36f,-.69f,-.97f,.97f,-.15f,.76f,.78f,-.36f,-.46f,.43f,.78f,-.08f,-.36f,.91f,-.15f,-.49f,.89f,-.47f,-.88f,.32f,-.35f,.46f,.86f,.19f,.87f,.76f,.41f,.87f,.71f,.11f,.78f,-.87f,.70f,-.29f,-.32f,.01f,-.68f,.19f,-.59f,-.04f,.89f,-.46f,.46f,-.82f,.02f,.35f,-.93f,-.46f,.58f,.03f,-.81f,.27f,-.90f,-.33f,-.93f,-.47f,-.42f,-.27f,-.49f,.82f,.12f,.82f,.29f,-.35f,.74f,.00f,-.70f,.45f,-.19f,.05f,-.39f,-.84f,-.99f,-.22f,-.71f,-.60f,-.54f,.21f,.61f,.18f,.08f,.19f,-.06f,-.98f,.21f,-.78f,.15f,.68f,-.53f,.31f,.89f,-.01f,-.73f,.02f,.40f,-.37f,.23f,-.95f,.62f,.84f,-.35f,-.63f,.96f,-.47f,.14f,.80f,-.68f,.29f,.99f,.49f,-.01f,.16f,.19f,-.38f,.63f,-.93f,-.31f,.83f,-.64f,-.72f,.49f,-.87f,.09f,.89f,-.76f,-.98f,.36f,.71f,.07f,.99f,-.47f,-.41f,.68f,.86f,.77f,.92f,-.79f,-.22f,.24f,.09f,-.45f,.00f,.22f,-.96f,-.50f,.87f,.06f,.93f,-.39f,-.39f,-.39f,-.36f,-.11f,.77f,-.29f,.32f,-.60f,-.69f,-.35f,.12f,-.30f,.66f,.29f,-.45f,-.18f,-.95f,.75f,-.69f,-.71f,-.93f,-.25f,-.36f,.81f,-.79f,.17f,.93f,-.60f,-.98f,.55f,-.87f,-.67f,.32f,.80f,.79f,.62f,.44f,-.19f,-.48f,.34f,.89f,.72f,.37f,-.82f,-.49f,-.71f,-.40f,-.08f,-.05f,-.30f,-.46f,.50f,.24f,.85f,-.95f,-.62f,-.45f,-.09f,.42f,-.95f,-.20f,.21f,-.50f,.39f,-.38f,-.74f,.69f,-.65f,.43f,.60f,-.48f,-.81f,-.14f,-.90f,.56f,.42f,.39f,-.13f,.76f,.06f,-.46f,-.55f,.60f,.72f,-.05f,-.20f,-.17f,-.40f,.23f,-.55f,.90f,-.49f,.58f,.34f,.72f,-.72f,-.54f,-.43f,-.80f,-.49f,-.76f,-.76f,-.11f,.55f,-.68f,.14f,-.98f,.20f,.93f,.19f,.52f,-.26f,.89f,-.82f,-.02f,-.47f,.74f,-.94f,-.01f,.90f,.68f,.20f,.36f,-.50f,-.99f,.20f,.05f,-.18f,-.06f,-.39f,.60f,-.12f,-.94f,.89f,-.31f,.00f,.35f,.74f,-.24f,-.88f,.41f,-.78f,-.04f,.38f,-.17f,.93f,.48f,.31f,.60f,-.20f,-.78f,.85f,-.03f,.26f,.88f,.17f,.82f,.39f,-.03f,.10f,.13f,.72f,.15f,.40f,-.39f,.32f,.44f,-.75f,-.64f,-.29f,.77f,-.50f,.74f,.71f,-.08f,-.30f,-.42f,.17f,-.52f,.84f,-.07f,-.89f,-.60f,-.04f,.64f,-.73f,-.10f,-.82f,.87f,-.78f,.23f,.67f,-.98f,-.68f,.66f,.82f,.64f,.40f,.55f,.91f,-.02f,.01f,.13f,-.82f,-.86f,.79f,-.06f,-.58f,.75f,.99f,-.05f,.32f,-.07f,.97f,.49f,.40f,.42f,.86f,-.62f,.54f,.57f,.90f,-.76f,.54f,-.61f,.32f,.66f,.21f,-.24f,-.70f,.02f,.27f,.78f,-.45f,.29f,.18f,-.06f,.78f,.38f,-.41f,.14f,.50f,-.43f,-.45f,.68f,-.30f,-.79f,-.88f,.57f,-.06f,.36f,.16f,.14f,.48f,-.27f,.46f,-.78f,-.23f,.20f,.34f,.26f,.34f,-.40f,.07f,.00f,.28f,-.92f,-.23f,.46f,-.60f,-.69f,-.98f,.49f,.21f,-.65f,-.01f,.56f,-.42f,.88f,.96f,.40f,.60f,-.24f,-.93f,-.82f,.64f,.23f,-.24f,.28f,-.50f,.48f,.23f,.52f,.75f,.30f,.59f,-.63f,.46f,.47f,-.91f,.70f,.29f,-.47f,.12f,-.82f,.72f,-.20f,-.48f,.07f,.39f,.72f,.24f,.44f,.79f,-.44f,-.59f,.25f,-.95f,-.60f,.24f,-.40f,.50f,-.05f,-.86f,.49f,.95f,.92f,-.87f,.29f,-.71f,-.14f,.37f,.24f,.91f,.82f,-.64f,.12f,-.39f,.53f,-.95f,.32f,.87f,-.90f,.97f,-.85f,.09f,-.54f,-.75f,.74f,-.59f,.51f,-.23f,-.14f,-.64f,.51f,.39f,.60f,.36f,.16f,-.64f,.27f,-.16f,.32f,-.99f,.77f,.48f,-.71f,.55f,.73f,.44f,-.92f,-.29f,.44f,.10f,-.53f,-.71f,-.09f,-.46f,-.66f,-.31f,.05f,.92f,-.52f,.04f,.94f,.91f,.09f,.03f,.23f,-.88f,-.92f,.91f,-.47f,.04f,.37f,-.75f,.74f,.77f,-.69f,-.02f,-.90f,.64f,-.38f,-.21f,.76f,.71f,.38f,-.89f,.36f,-.17f,.86f,.13f,.45f,.36f,-.06f,.40f,.33f,-.53f,-.47f,.71f,.10f,.63f,.84f,-.22f,.07f,.76f,.98f,-.43f,-.54f,.42f,-.23f,-.75f,-.67f,.62f,.55f,.24f,-.77f,-.45f,.77f,.30f,-.80f,-.47f,.70f,.03f,-.26f,-.10f,.16f,.98f,-.33f,-.64f,.67f,.31f,-.61f,-.09f,.54f,-.23f,-.18f,-.41f,-.32f,-.43f,.35f,-.85f,-.45f,.53f,.24f,.86f,-.45f,.37f,.37f,-.87f,-.44f,-.67f,.66f,-.78f,.84f,.10f,.74f,-.33f,-.74f,.28f,.30f,.43f,.57f,.48f,-.36f,-.84f,.82f,.03f,.36f,.04f,.93f,-.42f,.73f,-.33f,.54f,-.85f,.28f,.19f,.31f,-.06f,.59f,.01f,.46f,-.68f,-.79f,-.91f,.95f,-.04f,.59f,.66f,-.03f,-.35f,.77f,.27f,.48f,.04f,-.91f,-.90f,-.38f,-.43f,.19f,.47f,.10f,.01f,-.15f,.38f,.74f,.24f,.68f,.66f,-.06f,-.09f,.79f,.06f,-.98f,-.26f,-.11f,-.01f,-.63f,.17f,-.20f,.94f,.04f,.15f,-.41f,-.88f,.62f,-.03f,-.95f,-.99f,-.35f,-.79f,-.08f,.13f,.45f,-.79f,.30f,-.97f,.53f,-.87f,-.00f,.41f,-.90f,.46f,.14f,-.71f,-.34f,.06f,-.93f,-.31f,-.91f,.88f,.76f,-.69f,.51f,-.97f,.40f,-.95f,.75f,-.95f,-.33f,.74f,-.86f,.17f,.73f,.70f,-.41f,.11f,.91f,.94f,-.67f,.40f,.18f,.01f,.17f,-.10f,.24f,.54f,-.01f,.30f,.21f,.54f,-.37f,-.56f,.76f,-.57f,.23f,-.87f,.35f,.35f,.84f,-.47f,.32f,-.51f,.45f,.03f,.84f,.02f,.74f,.40f,-.56f,.08f,.36f,.97f,.25f,.03f,.48f,-.03f,.65f,.71f,-.04f,.85f,.05f,.13f,-.08f,.81f,.32f,-.34f,-.79f,-.45f,.77f,-.89f,.79f,-.23f,-.65f,-.43f,.55f,.99f,.68f,-.43f,.25f,-.19f,-.32f,-.03f,-.61f,.01f,.10f,.14f,.82f,.50f,.96f,.24f,-.44f,-.21f,.05f,-.28f,.58f,.29f,-.08f,.83f,-.95f,-.21f,-.49f,.61f,-.37f,-.47f,.50f,-.33f,-.83f,.00f,-.68f,.06f,-.96f,-.74f,.57f,.34f,-.54f,-.77f,-.73f,.84f,.81f,-.90f,.95f,.57f,-.79f,.35f,.54f,-.50f,-.86f,.13f,.16f,.75f,.64f,-.74f,-.72f,-.85f,.87f,-.71f,.43f,-.06f,-.69f,.12f,-.06f,.36f,-.34f,.79f,.02f,-.83f,.91f,.40f,.46f,.90f,-.72f,.02f,.01f,-.45f,.30f,-.12f,-.09f,-.14f,.55f,-.33f,.34f,-.82f,.90f,.45f,.95f,-.62f,-.49f,.06f,-.15f,.66f,.10f,-.92f,.71f,.04f,-.28f,-.83f,-.37f,.18f,.37f,.49f,.82f,-.53f,-.30f,.17f,-.65f,-.06f,.29f,-.89f,-.74f,-.21f,-.85f,.93f,-.51f,.29f,-.86f,-.20f,-.19f,.39f,-.51f,.54f,-.67f,.27f,-.23f,.23f,.43f,.65f,.71f,.95f,-.84f,.84f,-.44f,-.78f,.05f,.84f,.05f,-.54f,-.18f,-.41f,-.80f,.05f,-.41f,-.97f,.34f,.64f,-.94f,.19f,.22f,.85f,.58f,.92f,-.07f,-.41f,.30f,-.01f,.72f,-.28f,.91f,.82f,.92f,.17f,-.08f,-.69f,-.92f,.05f,.45f,-.72f,-.34f,.36f,.35f,-.18f,.55f,-.21f,-.69f,-.60f,.86f,.85f,.47f,.48f,.49f,-.31f,.03f,-.42f,-.28f,.38f,-.94f,.82f,.25f,-.40f,-.95f,.81f,-.21f,.50f,-.55f,-.02f,-.91f,-.10f,.45f,-.30f,.31f,.80f,-.57f,.41f,-.83f,.28f,-.02f,.11f,-.45f,.84f,.03f,.14f,.76f,-.21f,-.04f,-.52f,-.55f,-.03f,-.98f,.48f,.91f,-.39f,.57f,.45f,.29f,-.05f,-.33f,-.65f,-.62f,.98f,-.35f,.61f,.81f,.61f,-.51f,.43f,-.36f,.73f,-.20f,-.18f,.05f,.04f,-.83f,.38f,-.98f,-.62f,.07f,.65f,-.23f,.49f,-.33f,-.85f,.33f,-.51f,-.08f,-.21f,.66f,.84f,-.36f,-.01f,.99f,.48f,-.79f,.18f,.82f,.11f,-.38f,-.49f,.24f,.20f,.16f,.60f,-.33f,.58f,.77f,-.43f,-.65f,.81f,.14f,.36f,.00f,-.87f,-.57f,-.01f,-.02f,.30f,.14f,-.17f,.18f,.17f,.64f,.64f,.57f,-.72f,.12f,-.33f,.73f,.54f,-.36f,.16f,.62f,.01f,.09f,.36f,-.65f,-.72f,-.02f,-.40f,.55f,-.54f,-.20f,.24f,.97f,-.31f,.96f,.74f,.71f,.10f,.79f,-.64f,.17f,-.56f,.27f,-.62f,-.65f,.73f,.17f,-.06f,.67f,-.39f,-.46f,.25f,.64f,-.22f,-.13f,.35f,.70f,-.70f,-.60f,-.14f,-.53f,-.09f,.59f,-.09f,-.12f,-.28f,-.65f,.69f,.48f,.11f,-.82f,-.15f,-.01f,.06f,.96f,-.14f,.51f,.39f,.66f,-.27f,-.52f,.54f,-.87f,-.18f,-.69f,-.19f,-.20f,.19f,-.06f,.27f,.18f,-.26f,-.45f,-.09f,.47f,-.15f,-.05f,.54f,.80f,.86f,-.60f,-.02f,.74f,.07f,-.10f,-.73f,.56f,.67f,.51f,.23f,.69f,.85f,.21f,-.92f,.15f,.53f,.12f,.43f,-.26f,-.50f,.29f,.92f,-.29f,-.25f,.94f,-.92f,.53f,.14f,.93f,.88f,.09f,-.29f,.59f,.98f,.49f,.64f,-.18f,-.75f,-.26f,-.47f,.64f,.87f,.46f,-.51f,.04f,-.46f,.00f,-.67f,-.78f,.05f,-.35f,.21f,-.42f,-.85f,.02f,.78f,.86f,-.93f,-.28f,.54f,-.41f,-.83f,.95f,-.72f,.31f,.67f,-.22f,-.07f,.74f,.26f,.27f,-.78f,-.23f,.57f,.55f,-.27f,.17f,.57f,.63f,-.05f,-.76f,-.25f,.37f,-.29f,-.48f,.92f,-.87f,-.08f,-.68f,.61f,.24f,-.48f,-.70f,.61f,-.01f,-.96f,-.54f,.08f,-.75f,.06f,.60f,-.89f,.34f,.76f,-.25f,.34f,-.94f,.00f,.60f,-.24f,-.10f,-.98f,-.56f,-.74f,-.36f,.27f,.74f,.88f,-.94f,.25f,-.85f,-.39f,-.37f,.25f,-.80f,.10f,.52f,-.53f,.20f,.59f,-.17f,-.57f,.45f,.39f,-.63f,.37f,.45f,-.81f,-.16f,.18f,.66f,-.23f,-.98f,-.81f,.47f,.18f,.24f,.38f,-.79f,-.73f,.28f,-.81f,.39f,-.15f,.93f,-.83f,-.42f,-.70f,-.09f,-.95f,.23f,.25f,-.44f,-.79f,-.68f,-.26f,-.04f,.05f,-.39f,-.23f,.22f,-.25f,-.30f,-.00f,-.03f,-.42f,.94f,.27f,.98f,-.32f,.88f,.49f,.75f,.76f,.02f,-.10f,.26f,-.54f,-.00f,-.91f,.30f,.48f,.30f,-.54f,.58f,.13f,.95f,.43f,-.90f,-.79f,.82f,-.03f,.28f,-.69f,.87f,-.35f,-.34f,.97f,-.17f,.03f,.61f,.32f,-.05f,.47f,-.35f,-.48f,-.50f,.37f,.84f,.50f,-.82f,.26f,.47f,-.36f,-.42f,.58f,.71f,.87f,.59f,.92f,.31f,.66f,-.28f,-.13f,-.19f,-.95f,.57f,-.81f,.95f,.16f,.62f,-.60f,-.03f,.75f,.40f,.40f,.78f,.95f,.59f,-.00f,.79f,.49f,-.76f,.91f,-.76f,-.74f,-.68f,.97f,.98f,-.97f,.91f,-.93f,.47f,.86f,.88f,-.56f,.84f,-.00f,.02f,-.23f,.95f,.58f,-.00f,.84f,-.65f,.77f,-.27f,-.42f,-.54f,.19f,-.77f,-.59f,.81f,.45f,.96f,.39f,-.46f,-.74f,-.59f,.93f,-.12f,.26f,-.74f,.28f,-.83f,.58f,.20f,.25f,-.39f,.85f,.29f,.90f,-.42f,.95f,.49f,-.11f,-.66f,.65f,.91f,-.98f,-.61f,-.72f,.20f,.25f,.39f,.76f,-.13f,.56f,.78f,.90f,.10f,.54f,.49f,-.85f,.50f,.91f,-.72f,-.00f,.80f,-.25f,.07f,-.18f,.87f,-.93f,.02f,.03f,-.15f,.37f,.50f,-.21f,-.17f,.78f,-.40f,.70f,-.96f,-.74f,.05f,.22f,-.04f,.78f,.99f,.65f,.63f,.66f,.40f,.53f,-.91f,.46f,.43f,.18f,.65f,-.58f,.73f,.60f,.57f,.72f,.71f,.15f,.70f,-.47f,-.82f,-.38f,.89f,-.91f,.72f,.13f,-.81f,-.79f,-.58f,-.13f,.28f,.79f,.46f,.34f,.27f,.26f,.00f,.39f,.89f,.24f,.83f,.91f,-.87f,-.49f,.30f,-.11f,.77f,-.97f,-.36f,-.41f,-.68f,.85f,.79f,.30f,.81f,-.57f,.91f,-.04f,-.22f,-.16f,.95f,.60f,.27f,-.73f,.26f,-.18f,.50f,-.79f,.10f,-.69f,-.23f,.07f,.76f,-.41f,-.13f,-.64f,-.15f,.47f,-.03f,.93f,.95f,-.67f,.72f,-.80f,-.17f,.98f,.45f,-.91f,.94f,-.67f,.51f,.81f,-.37f,.49f,.43f,.26f,.83f,-.88f,-.10f,.05f,.41f,-.13f,-.32f,.75f,.37f,-.26f,-.42f,-.41f,.79f,.37f,-.06f,.77f,-.61f,.23f,.57f,.28f,.63f,-.43f,-.09f,.22f,-.86f,-.07f,.22f,.48f,-.05f,.71f,-.98f,.47f,.46f,.02f,-.52f,-.24f,.35f,-.02f,-.30f,-.80f,-.18f,-.81f,.19f,-.74f,-.04f,-.58f,-.51f,-.71f,-.28f,.23f,-.51f,-.05f,.88f,-.55f,.54f,.41f,-.06f,-.58f,.32f,.93f,-.14f,-.61f,-.05f,.13f,-.73f,-.19f,-.49f,.59f,.53f,.97f,.64f,.02f,.39f,-.95f,-.87f,-.87f,.96f,-.90f,-.68f,.74f,-.76f,-.08f,-.19f,-.11f,-.32f,.95f,-.13f,.70f,-.81f,.67f,.09f,-.64f,-.24f,.70f,.46f,.28f,.21f,-.10f,.49f,-.22f,.64f,.34f,-.10f,-.78f,-.11f,-.04f,-.31f,-.63f,-.45f,.12f,.71f,.88f,-.41f,.30f,.51f,-.73f,.54f,-.00f,.39f,.62f,.81f,.65f,-.26f,-.35f,.84f,-.12f,.77f,.65f,.77f,-.42f,.54f,.95f,-.82f,.46f,.83f,.15f,-.11f,-.06f,.41f,.87f,-.74f,-.24f,.37f,.94f,-.42f,-.56f,-.01f,-.15f,.03f,.49f,-.75f,.18f,.42f,.54f,.39f,.70f,-.49f,.08f,.68f,-.25f,.36f,.38f,.44f,-.52f,.86f,-.02f,.69f,.90f,.89f,-.77f,.14f,-.78f,-.64f,.40f,-.09f,.78f,-.81f,.94f,.46f,-.53f,.04f,-.46f,.71f,-.85f,.66f,-.51f,-.09f,.91f,.38f,-.26f,.91f,-.04f,-.86f,-.03f,.83f,-.69f,.35f,.65f,-.03f,.65f,.68f,.56f,.30f,-.46f,.73f,.02f,-.21f,.85f,.48f,.91f,.05f,.15f,-.32f,.39f,-.33f,-.84f,-.66f,-.41f,.98f,-.03f,.56f,.30f,.37f,-.86f,.60f,.47f,-.24f,.53f,-.85f,.66f,.53f,-.31f,-.44f,.88f,.34f,-.73f,-.48f,-.33f,-.73f,.56f,.45f,.18f,-.76f,.56f,.23f,-.83f,.97f,-.48f,-.80f,-.66f,.17f,.74f,.39f,-.02f,.49f,-.33f,-.73f,-.81f,.27f,.21f,.38f,-.68f,-.20f,-.09f,-.96f,-.61f,-.93f,-.99f,-.16f,.15f,.62f,.24f,.74f,-.65f,.35f,.26f,.38f,-.64f,-.67f,.80f,.26f,.30f,-.56f,.20f,-.82f,.06f,.81f,.75f,.82f,-.82f,.55f,-.91f,.37f,.16f,-.03f,-.13f,.09f,-.67f,-.33f,-.18f,-.13f,-.98f,-.81f,-.72f,.49f,-.96f,-.22f,.65f,.35f,-.35f,-.60f,-.64f,.19f,-.57f,.73f,-.70f,-.89f,.46f,-.77f,-.32f,.55f,-.81f,-.61f,.97f,-.49f,.30f,.42f,.66f,.19f,.55f,-.72f,-.95f,-.65f,-.28f,-.75f,-.22f,.29f,.56f,-.02f,-.15f,-.80f,.68f,-.05f,-.35f,.68f,-.84f,.95f,-.14f,.04f,.64f,.49f,.02f,.74f,.57f,.58f,.01f,.62f,-.87f,.76f,.37f,.27f,-.51f,.07f,.31f,.39f,.61f,.60f,-.14f,.62f,.01f,-.10f,-.43f,-.23f,-.91f,.99f,.58f,.81f,-.39f,-.98f,.96f,.02f,.38f,-.94f,-.46f,-.02f,.46f,.48f,-.39f,-.46f,.89f,-.62f,.10f,.81f,.53f,-.96f,-.33f,-.58f,-.17f,-.73f,.12f,.05f,.63f,-.83f,.01f,-.02f,-.00f,-.01f,-.79f,.78f,.48f,-.44f,.46f,.48f,-.72f,-.53f,.08f,.00f,-.12f,-.93f,-.87f,-.32f,-.49f,.35f,-.63f,-.20f,.99f,.01f,-.23f,.41f,-.32f,.12f,.85f,.31f,.43f,.21f,.60f,-.33f,-.71f,-.84f,-.23f,-.06f,-.23f,.08f,.89f,.99f,.64f,.65f,-.55f,.60f,-.07f,-.62f,.74f,.57f,.85f,.27f,.56f,-.87f,.00f,-.37f,-.67f,.07f,.49f,.44f,-.92f,.48f,.70f,-.86f,.33f,.39f,.71f,.43f,.80f,-.52f,.01f,-.54f,-.95f,-.39f,.40f,-.75f,.08f,.76f,-.62f,-.39f,.94f,.31f,.09f,-.15f,.83f,-.77f,.05f,.85f,.68f,-.32f,.94f,-.09f,-.57f,.46f,-.01f,-.77f,.68f,.44f,.01f,-.38f,.82f,-.22f,.74f,.75f,-.43f,-.26f,.01f,.87f,.15f,-.93f,-.51f,-.23f,-.57f,.88f,.76f,-.39f,.78f,-.40f,.78f,-.86f,-.15f,.47f,-.75f,-.26f,-.54f,-.00f,-.42f,.40f,.53f,-.34f,.26f,-.20f,-.09f,-.89f,.96f,.44f,-.56f,-.58f,-.40f,-.69f,.21f,-.74f,.42f,.94f,.95f,-.01f,.93f,.22f,-.40f,-.56f,.42f,-.97f,-.72f,.59f,-.69f,.52f,.62f,-.13f,-.47f,.18f,.00f,.37f,.58f,-.61f,.28f,-.85f,.52f,-.69f,-.62f,.54f,-.90f,.89f,.86f,-.83f,-.57f,.32f,.43f,.35f,-.35f,.02f,.93f,.77f,.32f,.52f,.58f,.76f,.35f,-.55f,-.10f,-.07f,-.82f,-.40f,.25f,.91f,.81f,.60f,-.58f,.64f,.20f,.97f,.61f,-.57f,.93f,.69f,-.42f,.99f,-.80f,.59f,.48f,.13f,.42f,-.78f,-.83f,.72f,.49f,.56f,-.69f,-.89f,.95f,-.45f,-.15f,-.39f,.12f,-.85f,-.85f,-.81f,-.63f,.33f,-.00f,.14f,-.46f,-.13f,.44f,.97f,-.03f,.58f,.91f,-.34f,.56f,.79f,.36f,.34f,.00f,-.16f,.98f,-.32f,-.91f,.51f,.65f,-.71f,-.53f,.00f,.41f,.83f,-.42f,-.52f,-.82f,.46f,.20f,-.61f,.50f,.82f,.43f,.05f,-.29f,.92f,.87f,-.57f,-.56f,.10f,.06f,-.95f,-.26f,.35f,-.83f,-.41f,.85f,-.43f,.61f,-.71f,.70f,.70f,-.50f,-.94f,.89f,-.78f,-.64f,.66f,-.37f,-.24f,-.74f,-.40f,.62f,-.01f,.11f,.47f,-.85f,-.10f,-.11f,-.91f,.48f,.43f,.01f,.71f,.66f,.15f,-.33f,-.07f,-.92f,-.77f,-.44f,.33f,-.89f,-.52f,.46f,-.68f,.37f,-.54f,.11f,-.08f,.60f,-.13f,-.01f,-.38f,-.31f,-.32f,.10f,-.58f,-.12f,-.62f,.97f,-.59f,-.26f,-.26f,.08f,-.59f,.91f,.33f,-.12f,-.22f,-.08f,-.86f,.02f,.65f,.56f,-.13f,.34f,.17f,.59f,.62f,.70f,.57f,.73f,-.86f,-.21f,.64f,-.23f,-.68f,.92f,.75f,-.05f,-.63f,-.61f,.52f,.77f,-.92f,-.71f,.73f,-.62f,.32f,-.38f,.99f,-.58f,-.77f,.58f,-.25f,.08f,.87f,.12f,.03f,.60f,-.07f,-.31f,-.70f,.34f,.71f,-.94f,-.72f,.37f,.18f,.71f,.17f,-.75f,-.90f,.24f,-.53f,-.05f,.68f,-.19f,.62f,-.62f,.67f,.02f,.78f,-.85f,.24f,-.57f,.02f,-.31f,-.23f,-.95f,-.78f,-.51f,.46f,.33f,-.06f,-.05f,.75f,-.07f,-.69f,-.55f,-.79f,.39f,-.28f,.92f,-.47f,-.34f,-.04f,.32f,-.35f,.97f,-.45f,.65f,.51f,-.90f,-.87f,.48f,.40f,.46f,.82f,-.02f,-.55f,.85f,.12f,-.10f,.65f,-.23f,.99f,.91f,-.02f,-.89f,-.67f,-.92f,.06f,-.16f,-.97f,-.17f,.11f,.37f,.59f,-.85f,-.43f,-.24f,-.78f,-.35f,.07f,-.26f,.65f,-.86f,-.29f,-.82f,-.03f,.25f,-.23f,.51f,-.61f,.68f,.88f,-.35f,-.17f,.36f,.36f,.80f,-.80f,-.52f,.90f,.65f,.82f,-.19f,-.10f,.62f,.87f,-.11f,-.67f,-.24f,.28f,.80f,.69f,.36f,.06f,.02f,-.15f,-.85f,-.94f,.18f,-.67f,-.03f,.58f,.95f,.70f,.04f,-.66f,-.21f,.77f,-.87f,-.96f,-.86f,-.96f,-.33f,.46f,-.08f,-.15f,.24f,.83f,-.10f,-.90f,-.47f,.56f,-.10f,.96f,-.13f,.12f,.15f,-.14f,-.88f,-.27f,.86f,-.51f,-.00f,.13f,.56f,.00f,.03f,-.64f,.05f,.44f,.85f,.15f,-.70f,.26f,-.57f,-.59f,-.90f,-.42f,.46f,.90f,-.47f,-.05f,-.29f,.77f,-.47f,.63f,-.72f,.24f,-.90f,-.96f,.61f,.74f,-.90f,.12f,-.92f,-.76f,.34f,-.53f,-.50f,.75f,.78f,.13f,.39f,.10f,.57f,-.10f,.00f,.61f,-.29f,.92f,-.89f,.70f,-.25f,.01f,.56f,.17f,-.51f,.78f,.03f,-.45f,.37f,-.65f,.65f,-.18f,.28f,-.04f,-.68f,-.48f,.06f,.94f,.56f,.92f,.96f,-.46f,-.69f,.89f,-.43f,-.68f,.87f,-.73f,.93f,.02f,.99f,.41f,-.78f,.06f,-.56f,.04f,.68f,-.79f,.26f,.39f,.92f,-.71f,-.17f,-.36f,-.48f,-.17f,.97f,-.94f,-.46f,-.84f,-.79f,.40f,.20f,-.71f,-.70f,.28f,-.27f,-.63f,-.39f,-.36f,.35f,-.69f,-.94f,-.17f,-.97f,-.07f,-.78f,.84f,.17f,.54f,.29f,.76f,.73f,-.58f,-.03f,.27f,.44f,-.03f,-.83f,-.04f,-.93f,-.42f,.82f,-.01f,-.62f,-.01f,.61f,-.05f,-.69f,-.63f,-.02f,-.25f,-.27f,.86f,.48f,-.92f,.71f,.69f,-.93f,.23f,-.48f,.22f,.05f,-.00f,-.97f,.82f,-.47f,.53f,.68f,.65f,-.13f,-.29f,-.74f,.66f,-.47f,-.01f,-.62f,.18f,.78f,.35f,.34f,.34f,-.20f,.65f,.40f,-.78f,-.11f,.81f,-.36f,.74f,-.83f,-.66f,-.26f,.90f,.02f,.55f,-.97f,.54f,.50f,.25f,.57f,-.49f,-.15f,-.45f,.50f,-.79f,-.90f,-.91f,.31f,.97f,.77f,.76f,.73f,.40f,-.04f,-.25f,.71f,.44f,.71f,.59f,.88f,.39f,-.23f,.99f,.94f,.65f,.52f,-.55f,-.30f,-.81f,.21f,.37f,.47f,.26f,-.07f,-.08f,.62f,.99f,-.52f,-.04f,.75f,-.67f,-.23f,-.16f,-.25f,.50f,-.17f,-.09f,.16f,-.92f,.62f,.70f,.28f,.00f,.87f,-.73f,.97f,-.31f,.94f,.56f,-.00f,.37f,.05f,-.70f,-.03f,.80f,.95f,.08f,-.34f,.85f,.88f,.95f,.88f,.14f,-.22f,.94f,.47f,-.85f,-.07f,.67f,.55f,.74f,.87f,-.39f,.99f,.49f,-.14f,-.20f,.79f,-.88f,.79f,-.14f,.40f,.57f,-.32f,.47f,-.93f,-.97f,-.18f,.71f,-.96f,-.50f,-.65f,.69f,-.26f,-.94f,-.07f,-.20f,.55f,-.84f,.14f,.88f,-.94f,-.62f,.42f,-.49f,.70f,-.61f,-.36f,-.92f,-.96f,-.42f,-.15f,-.55f,.51f,.56f,-.03f,.16f,.27f,.06f,-.46f,-.54f,-.74f,.48f,.79f,-.30f,-.31f,.37f,-.63f,-.53f,.65f,-.54f,-.68f,.10f,.19f,-.51f,.68f,-.81f,.20f,.38f,.83f,.14f,.81f,.29f,.95f,-.57f,.83f,-.25f,-.26f,.10f,-.23f,.00f,-.77f,-.47f,-.36f,-.33f,-.38f,.48f,.15f,-.61f,-.07f,-.35f,-.52f,.46f,-.51f,-.56f,.32f,-.51f,-.87f,-.76f,.95f,-.88f,-.53f,-.51f,.18f,.95f,-.72f,-.06f,.85f,.07f,-.84f,.90f,-.19f,.14f,.54f,-.84f,-.57f,-.38f,.23f,.45f,.66f,.73f,-.87f,-.29f,.25f,-.74f,-.87f,-.54f,.08f,-.26f,-.15f,.40f,-.62f,-.73f,-.38f,.13f,.37f,.20f,.61f,-.78f,.84f,.99f,.80f,-.99f,.52f,.85f,-.61f,-.74f,-.52f,.03f,-.17f,-.91f,.64f,.61f,.82f,.49f,-.90f,.13f,.92f,.63f,-.38f,.40f,-.12f,-.46f,-.66f,.45f,-.68f,-.32f,-.41f,.97f,-.78f,-.82f,-.93f,-.19f,.33f,-.56f,-.21f,-.37f,.61f,-.37f,-.59f,.50f,.88f,.39f,.11f,.66f,-.67f,.49f,-.07f,.51f,-.20f,.65f,-.63f,-.87f,.96f,-.58f,-.24f,.29f,-.67f,-.39f,-.18f,.48f,.79f,.97f,.24f,.07f,.19f,.54f,-.45f,-.34f,-.13f,.43f,-.55f,-.31f,-.55f,.77f,-.52f,-.20f,.30f,.81f,.74f,.63f,.01f,-.70f,.17f,.50f,.34f,.73f,.01f,-.74f,.37f,.88f,-.40f,.39f,-.07f,-.03f,.14f,.89f,.70f,-.49f,-.41f,-.66f,.20f,.95f,.34f,.69f,-.01f,-.63f,.57f,.60f,.99f,.49f,-.45f,.77f,-.26f,-.83f,.29f,.69f,-.26f,.97f,-.25f,-.96f,.68f,-.17f,.99f,-.61f,.40f,-.56f,.50f,.07f,.74f,-.16f,-.44f,.72f,.50f,.02f,.74f,.95f,-.14f,-.40f,.82f,-.82f,-.58f,.06f,.77f,-.26f,.63f,.17f,-.44f,-.08f,.72f,.16f,-.91f,-.56f,.86f,.34f,.70f,.85f,-.93f,.99f,.33f,.32f,.23f,-.40f,-.96f,.07f,.01f,-.81f,-.39f,-.82f,-.50f,-.15f,.52f,.34f,-.96f,-.28f,.16f,.72f,-.91f,.03f,-.22f,.27f,-.53f,.27f,-.53f,.95f,.48f,.22f,.61f,-.30f,.67f,-.22f,-.06f,-.56f,-.74f,.98f,-.97f,.15f,.97f,-.71f,.53f,-.09f,.58f,-.80f,.85f,-.96f,-.50f,.22f,-.43f,-.73f,.36f,-.27f,.46f,.36f,.92f,.02f,.00f,.55f,-.00f,-.68f,-.39f,-.67f,-.58f,-.50f,.39f,-.18f,.48f,-.66f,.05f,-.38f,-.48f,.42f,-.72f,.30f,.30f,.13f,-.52f,-.47f,-.07f,-.92f,-.28f,.25f,-.19f,.83f,-.30f,.66f,.57f,-.98f,-.47f,.03f,.22f,-.55f,.14f,-.04f,-.81f,.99f,-.48f,-.36f,.24f,.08f,-.82f,-.74f,-.68f,-.12f,-.57f,-.55f,-.43f,-.57f,-.37f,-.84f,-.83f,.44f,.26f,-.04f,.03f,-.58f,-.84f,.78f,-.30f,-.00f,-.82f,.12f,-.00f,.13f,.49f,-.26f,.11f,.50f,-.44f,-.87f,-.15f,-.22f,-.21f,-.76f,-.79f,.80f,.97f,.97f,.06f,.87f,.13f,-.22f,.49f,-.11f,-.36f,-.12f,-.39f,.43f,.17f,.54f,-.17f,.93f,-.21f,.80f,.60f,-.40f,.02f,.85f,.32f,-.52f,-.62f,.54f,.01f,-.93f,.07f,.97f,-.46f,-.94f,-.94f,-.93f,-.41f,.80f,-.54f,-.23f,-.08f,.60f,-.56f,.26f,.12f,-.48f,.53f,.90f,.93f,-.84f,.71f,-.60f,.18f,.52f,-.59f,-.74f,.36f,-.74f,-.25f,-.77f,.13f,-.88f,.88f,-.23f,-.29f,.82f,.26f,.57f,-.29f,.07f,-.40f,.99f,-.89f,.56f,-.19f,.95f,.50f,-.70f,.31f,.75f,.63f,.70f,-.46f,-.34f,.88f,.03f,.56f,.71f,.34f,.17f,-.79f,-.78f,.47f,.14f,.00f,.49f,.10f,.42f,.35f,-.39f,.02f,-.02f,.19f,.36f,-.71f,.30f,-.58f,-.42f,-.23f,.24f,.17f,-.42f,.77f,-.88f,.07f,-.33f,.20f,-.05f,.37f,-.74f,.87f,.00f,-.23f,-.48f,-.95f,-.97f,.12f,-.86f,.58f,-.92f,.62f,.72f,-.53f,.03f,.70f,.26f,.95f,-.44f,.91f,.09f,-.31f,.53f,.90f,.92f,-.68f,.75f,-.85f,-.04f,-.42f,-.77f,-.67f,.94f,-.57f,-.93f,-.43f,-.74f,.95f,-.53f,-.50f,.69f,.93f,.61f,-.62f,.66f,.44f,-.98f,.54f,.33f,-.61f,-.95f,.92f,-.95f,.63f,-.25f,-.36f,-.70f,-.57f,-.36f,-.42f,.34f,-.01f,.27f,-.76f,-.22f,.46f,-.89f,-.02f,.57f,.82f,-.58f,-.52f,.49f,.02f,-.04f,.73f,-.47f,-.08f,.47f,-.98f,.94f,.82f,-.90f,-.88f,-.82f,.04f,.25f,.84f,-.26f,.19f,-.43f,.32f,-.66f,.25f,.44f,.47f,-.69f,-.14f,.64f,-.15f,-.50f,-.12f,.29f,-.08f,.83f,.48f,.07f,-.51f,.24f,-.81f,-.65f,-.54f,.12f,.52f,.15f,-.55f,-.94f,.47f,-.32f,.04f,.17f,-.13f,-.13f,.46f,.13f,-.85f,-.55f,.41f,-.90f,.84f,-.17f,-.73f,.29f,-.70f,.16f,.38f,.39f,-.93f,.96f,-.45f,.54f,-.76f,-.07f,-.02f,.84f,.91f,-.09f,.94f,-.19f,.88f,-.93f,.74f,.19f,-.39f,.11f,-.36f,-.57f,.01f,-.71f,-.67f,-.54f,.54f,-.22f,.06f,-.67f,.68f,-.45f,.64f,.46f,.69f,.75f,.94f,-.40f,-.05f,-.89f,-.07f,-.47f,.03f,-.31f,.54f,-.88f,.90f,-.37f,.01f,-.23f,-.96f,.12f,-.42f,.91f,-.62f,-.06f,.78f,.63f,-.72f,.28f,.11f,.11f,.15f,.73f,-.91f,.36f,.42f,.25f,-.36f,-.27f,.42f,-.51f,-.44f,.87f,.33f,.47f,.84f,.05f,.37f,-.60f,-.66f,.83f,.84f,.43f,.15f,-.90f,-.48f,-.59f,-.98f,.80f,-.43f,-.01f,.43f,.62f,-.52f,.36f,-.55f,.14f,.08f,-.32f,.65f,-.51f,.26f,.69f,-.71f,-.69f,.28f,-.66f,-.91f,-.15f,.47f,.34f,.56f,-.66f,.26f,.56f,.21f,-.67f,.85f,.30f,.83f,.17f,-.30f,.09f,.23f,.64f,.26f,.78f,-.09f,-.37f,-.45f,-.74f,-.96f,-.13f,.16f,-.94f,-.78f,.75f,.40f,.83f,.99f,-.91f,-.26f,.66f,-.55f,.89f,.86f,-.64f,-.06f,.33f,-.09f,-.87f,.63f,-.40f,.91f,-.89f,.76f,-.39f,-.93f,.77f,.61f,-.10f,-.78f,-.79f,-.93f,.44f,.58f,.16f,-.39f,-.97f,-.90f,.01f,-.36f,.70f,.26f,.90f,-.65f,.94f,-.19f,-.00f,.26f,-.39f,.25f,-.27f,.68f,.06f,.11f,.88f,.73f,.17f,.83f,-.39f,-.16f,-.43f,-.16f,.59f,.68f,.45f,.15f,.53f,-.56f,.99f,-.46f,-.50f,-.47f,-.63f,-.07f,.32f,.09f,.40f,-.42f,-.66f,.89f,-.73f,-.62f,.43f,.67f,-.31f,.65f,-.04f,-.37f,.77f,.10f,-.78f,.67f,.29f,-.37f,-.94f,-.00f,-.79f,-.15f,.08f,-.53f,.88f,.21f,.73f,.17f,-.13f,.93f,.86f,-.65f,-.12f,-.69f,.57f,.23f,-.62f,.03f,-.47f,.02f,.20f,-.42f,.97f,.85f,-.16f,-.87f,-.78f,.50f,.34f,-.81f,.87f,.29f,.61f,-.82f,-.50f,-.86f,.56f,-.14f,-.50f,-.35f,-.55f,-.37f,.37f,-.55f,.59f,-.27f,.47f,-.88f,.78f,.09f,-.61f,-.20f,-.81f,.10f,.11f,-.93f,-.80f,.74f,.65f,.27f,-.97f,.97f,-.04f,.49f,-.77f,.36f,.82f,.09f,-.32f,.08f,.67f,.08f,.48f,-.02f,-.07f,.55f,-.47f,.03f,-.17f,-.72f,-.05f,.79f,.60f,-.86f,.68f,.07f,.77f,-.60f,.83f,.47f,.65f,-.95f,-.00f,.71f,.97f,.63f,.75f,.83f,-.08f,-.89f,-.46f,-.46f,-.51f,.69f,.83f,.33f,.25f,.36f,-.47f,-.57f,.36f,-.15f,.03f,.72f,-.91f,-.46f,-.45f,-.08f,.21f,.39f,-.59f,.73f,.34f,-.42f,.35f,-.11f,-.38f,-.51f,.84f,-.07f,-.21f,-.58f,-.96f,-.91f,.00f,-.85f,-.29f,-.73f,-.45f,-.52f,-.13f,-.44f,-.79f,-.90f,.32f,.67f,.66f,.51f,-.57f,-.31f,.85f,-.46f,.91f,-.03f,-.13f,.81f,-.84f,.07f,.37f,-.51f,.73f,-.11f,.59f,-.38f,.64f,-.41f,-.80f,.72f,-.72f,-.70f,.50f,-.19f,-.42f,-.10f,.59f,.79f,-.69f,-.58f,.15f,.98f,.44f,.19f,-.18f,-.06f,.80f,-.27f,-.02f,-.02f,.25f,-.20f,.29f,-.80f,.39f,.89f,.99f,.47f,.43f,.71f,-.43f,-.26f,.62f,.10f,.79f,-.26f,.18f,.27f,-.75f,-.82f,-.09f,-.63f,.66f,-.29f,-.54f,-.21f,-.83f,-.42f,.85f,.69f,-.95f,-.85f,.04f,-.60f,.02f,-.00f,.19f,.01f,.89f,-.69f,.85f,-.70f,.46f,-.47f,-.43f,-.03f,.37f,-.66f,-.44f,.90f,.03f,-.07f,-.09f,.17f,-.42f,.16f,-.81f,-.05f,.56f,-.19f,-.23f,-.89f,-.20f,-.95f,-.47f,-.84f,.84f,-.91f,-.33f,-.69f,.53f,-.99f,.93f,.03f,-.88f,.03f,-.01f,.88f,.84f,-.71f,-.25f,-.04f,-.15f,-.92f,-.12f,-.74f,-.04f,.08f,-.61f,.18f,.93f,-.30f,-.81f,.94f,-.39f,-.41f,.21f,-.08f,-.97f,-.41f,.10f,-.44f,.14f,.30f,-.75f,.68f,-.09f,.43f,.49f,-.58f,.94f,-.01f,.26f,.75f,.77f,-.26f,.56f,.48f,-.50f,-.45f,.56f,-.13f,.53f,-.75f,-.18f,.05f,-.00f,.66f,.23f,-.50f,.57f,.37f,-.23f,.86f,-.05f,-.61f,-.22f,.72f,-.35f,.47f,-.50f,.78f,.81f,.05f,-.04f,-.05f,.41f,-.74f,.52f,.34f,-.02f,.30f,.91f,-.52f,-.66f,-.57f,-.49f,-.76f,-.29f,-.92f,.87f,-.26f,-.88f,-.78f,-.98f,-.61f,-.72f,.09f,.82f,.47f,-.19f,-.71f,.56f,.15f,-.07f,-.94f,.85f,.85f,.40f,-.13f,-.20f,.23f,-.02f,-.74f,.12f,.99f,-.44f,.34f,-.54f,.90f,-.95f,.54f,-.27f,.36f,.78f,-.86f,-.21f,.17f,.28f,-.15f,.66f,-.43f,.64f,.70f,-.95f,.27f,.71f,.27f,-.99f,-.11f,-.56f,.21f,-.84f,-.64f,-.26f,.05f,.31f,.56f,-.41f,-.77f,-.32f,.72f,.25f,.81f,.06f,-.92f,.89f,-.23f,.42f,.53f,-.15f,.11f,.88f,-.36f,-.43f,-.68f,.37f,-.62f,-.77f,-.03f,-.93f,.50f,.57f,-.90f,-.00f,.82f,.61f,-.62f,.38f,-.06f,-.32f,.31f,.66f,-.25f,.90f,.15f,.12f,.60f,.82f,-.86f,.54f,-.99f,.05f,.92f,-.75f,-.26f,-.32f,-.18f,.41f,.38f,-.32f,.27f,.08f,.74f,-.71f,-.26f,.34f,.99f,.46f,.11f,-.41f,-.09f,.19f,-.83f,.43f,.12f,.74f,.31f,-.35f,-.76f,.13f,.05f,.65f,.32f,-.62f,-.34f,-.37f,.94f,-.18f,.03f,-.30f,-.91f,.95f,.76f,.67f,.85f,.15f,.84f,-.21f,.87f,.83f,.56f,-.67f,.72f,.05f,-.70f,-.28f,-.87f,-.25f,.95f,.34f,-.01f,.20f,-.72f,.57f,.06f,.99f,.05f,.36f,.77f,-.89f,-.52f,-.65f,-.95f,-.49f,-.62f,.12f,.17f,-.89f,.27f,-.72f,.09f,.94f,.46f,-.39f,-.27f,-.34f,.05f,-.43f,.76f,.59f,.41f,-.69f,.50f,.30f,.69f,.31f,.31f,.60f,-.81f,-.34f,-.31f,.32f,.72f,-.37f,.08f,-.61f,-.77f,-.86f,.13f,.52f,-.28f,-.43f,-.70f,.34f,.41f,-.85f,-.19f,.09f,.88f,-.40f,-.92f,-.62f,-.97f,.11f,-.54f,-.51f,-.99f,-.03f,.69f,.57f,-.56f,-.87f,.39f,.37f,.72f,.17f,-.28f,-.86f,.46f,-.77f,.49f,-.73f,-.65f,-.81f,-.78f,.02f,.22f,-.58f,.30f,-.95f,-.91f,.94f,-.73f,.67f,.45f,-.49f,.37f,.48f,-.76f,-.59f,.32f,.97f,.01f,.46f,-.42f,.58f,.26f,.62f,.58f,-.26f,-.52f,-.24f,-.72f,-.88f,.89f,.99f,.20f,.48f,.11f,.82f,-.51f,.46f,-.07f,.19f,.66f,-.22f,-.68f,.37f,.81f,.51f,-.81f,-.75f,-.51f,-.26f,-.13f,.47f,.59f,-.71f,-.43f,-.33f,.82f,-.15f,-.18f,.80f,-.09f,.19f,.75f,.20f,.31f,.28f,-.44f,-.43f,.18f,.53f,.76f,.15f,-.63f,-.39f,-.84f,.01f,-.05f,-.33f,.32f,.10f,.00f,.80f,.46f,-.78f,.84f,-.20f,.60f,.62f,.52f,-.79f,.21f,-.97f,-.94f,.45f,-.46f,-.97f,-.53f,-.37f,-.97f,-.68f,.09f,-.80f,.50f,.09f,.56f,.10f,.81f,.33f,.81f,-.32f,-.51f,-.00f,-.15f,.92f,.94f,-.79f,-.72f,.43f,-.04f,.42f,.21f,.99f,.39f,.06f,-.01f,-.59f,.18f,-.94f,-.50f,-.55f,-.13f,.32f,.94f,-.87f,.08f,-.72f,-.17f,-.37f,.21f,.01f,-.28f,.60f,.05f,-.99f,-.50f,-.74f,.32f,.49f,-.29f,.12f,-.52f,-.90f,-.68f,-.45f,.73f,.52f,.03f,-.71f,.39f,.92f,-.95f,.83f,-.60f,-.44f,-.66f,-.59f,-.76f,-.79f,-.10f,-.77f,.35f,.88f,-.49f,.08f,.06f,-.97f,-.84f,-.47f,.40f,.51f,.13f,-.06f,-.35f,.81f,-.99f,-.09f,-.92f,-.38f,-.78f,-.10f,.76f,-.08f,-.31f,-.59f,-.86f,-.34f,-.86f,.54f,.10f,.16f,-.01f,-.68f,-.22f,-.15f,.46f,.92f,-.75f,-.84f,-.62f,-.74f,.40f,.90f,-.41f,-.14f,-.73f,.27f,.86f,.53f,-.41f,.35f,.96f,.34f,.96f,-.83f,.88f,-.35f,.34f,-.71f,-.27f,-.28f,.38f,-.98f,-.30f,-.67f,.52f,-.03f,.63f,-.54f,.26f,.41f,-.01f,-.10f,-.38f,.61f,-.77f,.92f,.58f,-.98f,.93f,-.27f,-.08f,.78f,-.64f,-.93f,.79f,-.36f,.94f,-.10f,.48f,.73f,-.44f,.00f,-.94f,-.90f,.04f,-.72f,.09f,-.57f,.46f,-.62f,-.41f,.09f,-.21f,-.56f,.79f,.49f,-.78f,.43f,.61f,-.29f,-.10f,-.54f,.64f,-.95f,.35f,-.50f,.99f,.08f,-.07f,-.00f,.31f,.69f,.78f,.68f,-.34f,-.39f,.17f,.51f,-.25f,.71f,.83f,.58f,.87f,.79f,.14f,.26f,.54f,.26f,.06f,.85f,-.29f,-.79f,-.57f,-.02f,-.07f,.77f,.16f,.17f,-.35f,.63f,.61f,.89f,.52f,.14f,.77f,.57f,.80f,-.81f,-.93f,-.18f,-.32f,-.88f,.24f,-.25f,-.75f,.88f,.43f,.09f,-.08f,-.15f,.53f,-.93f,.02f,-.48f,.82f,-.81f,.86f,-.84f,.15f,.64f,-.20f,.92f,.10f,.26f,.75f,-.50f,.50f,-.75f,.48f,-.27f,-.39f,-.82f,-.71f,.60f,.99f,-.84f,.42f,-.01f,.39f,-.86f,-.06f,.66f,.70f,.17f,.75f,-.97f,.57f,-.44f,-.68f,.13f,-.20f,-.69f,.99f,.73f,.38f,-.51f,-.23f,.26f,-.85f,.57f,-.62f,.72f,.69f,.88f,-.09f,-.30f,-.30f,.37f,-.62f,-.74f,-.02f,-.33f,-.69f,.00f,-.53f,-.99f,-.39f,-.88f,.56f,.09f,.33f,.56f,.75f,.01f,-.68f,.87f,.80f,-.23f,-.48f,-.20f,.08f,.64f,.10f,-.14f,.13f,.50f,-.92f,.43f,-.61f,-.04f,-.90f,.65f,.76f,.58f,-.21f,.63f,.93f,-.61f,.01f,-.13f,.09f,.89f,.46f,.82f,.95f,.23f,-.86f,-.16f,.38f,.00f,.70f,-.42f,.56f,-.55f,.05f,.88f,-.22f,-.51f,-.84f,.28f,-.03f,.74f,.53f,.40f,-.98f,-.52f,-.50f,-.67f,.84f,-.72f,-.78f,.14f,.08f,-.44f,-.23f,.38f,-.01f,.76f,.26f,.97f,.80f,-.84f,-.20f,-.19f,.51f,.42f,-.51f,-.81f,.69f,.10f,-.98f,-.62f,-.82f,.09f,-.43f,.54f,.47f,.43f,-.85f,.87f,-.86f,.24f,.23f,-.46f,-.33f,.86f,-.90f,.26f,-.42f,-.48f,-.12f,.55f,-.28f,-.49f,-.40f,.45f,.72f,.69f,.10f,-.20f,.02f,.65f,-.55f,.70f,.27f,-.11f,.44f,.87f,-.83f,.09f,.60f,.11f,.05f,-.33f,-.35f,.74f,.23f,-.91f,.35f,-.47f,.52f,-.70f,.56f,.24f,-.60f,-.60f,.42f,-.40f,-.30f,-.87f,.61f,-.53f,.61f,.85f,.00f,-.19f,-.10f,-.72f,-.07f,-.62f,.46f,-.16f,-.75f,-.96f,-.92f,.87f,-.95f,-.25f,-.40f,.63f,.45f,-.22f,-.49f,.27f,.05f,-.96f,-.89f,.43f,.38f,.50f,.09f,.78f,-.73f,.67f,.55f,-.53f,.89f,-.34f,.99f,-.29f,-.04f,-.42f,-.34f,-.74f,-.36f,.38f,.94f,.29f,-.11f,.10f,-.08f,-.21f,-.18f,.88f,-.64f,-.58f,-.18f,.77f,-.47f,-.53f,-.00f,.13f,-.41f,.71f,.52f,-.39f,-.25f,.15f,.09f,.49f,.99f,-.09f,.97f,.21f,-.24f,.76f,-.25f,.81f,-.44f,.96f,-.85f,.48f,-.86f,.12f,.94f,-.09f,-.69f,.98f,.26f,.63f,.65f,.70f,.95f,.96f,.49f,-.25f,.66f,-.97f,-.62f,.85f,.04f,.16f,.22f,.71f,-.52f,-.55f,.08f,-.85f,-.87f,.73f,.08f,.94f,-.34f,-.82f,.22f,-.25f,-.92f,-.96f,.94f,-.47f,.76f,-.83f,.22f,.42f,-.80f,-.22f,-.84f,-.52f,.15f,.20f,.96f,.20f,.63f,-.32f,-.76f,.32f,-.71f,-.68f,.55f,-.45f,.59f,-.41f,-.10f,-.62f,.56f,.07f,-.22f,-.90f,.39f,-.53f,.98f,-.55f,-.31f,-.48f,.22f,-.78f,-.57f,-.88f,-.97f,.26f,-.74f,.20f,-.82f,-.34f,.27f,.96f,-.14f,-.60f,-.20f,-.34f,.68f,-.00f,.57f,.87f,-.48f,.10f,.39f,-.22f,.97f,.45f,-.41f,.85f,-.18f,-.22f,-.40f,.67f,-.09f,-.61f,-.28f,.71f,.45f,-.27f,-.67f,.50f,-.10f,-.09f,-.75f,.96f,.86f,.27f,-.83f,-.35f,-.30f,.37f,.44f,.78f,-.50f,-.53f,-.40f,-.43f,-.04f,.46f,-.57f,-.60f,-.13f,.90f,.32f,.92f,.79f,.48f,-.76f,.87f,.11f,-.17f,-.50f,-.00f,.02f,.61f,.33f,.64f,-.80f,-.48f,.91f,.81f,-.73f,-.02f,.94f,-.63f,.19f,.39f,-.50f,-.91f,-.99f,.95f,-.75f,.67f,.82f,-.54f,-.76f,.50f,.90f,.46f,-.74f,.69f,-.86f,.72f,.52f,.39f,-.46f,-.36f,.34f,.21f,-.44f,.14f,.35f,.13f,-.84f,.95f,.32f,-.82f,-.62f,-.69f,-.50f,.59f,.47f,-.97f,.15f,.59f,.65f,-.26f,.50f,-.03f,-.96f,.97f,-.99f,-.67f,-.73f,-.48f,-.64f,-.84f,-.94f,.62f,.14f,-.32f,-.47f,-.74f,.07f,.05f,.87f,-.83f,.49f,-.32f,-.32f,.03f,.51f,.06f,-.77f,.75f,-.39f,-.78f,.10f,.95f,.38f,-.08f,-.66f,-.07f,.09f,-.09f,-.09f,.83f,-.12f,.43f,-.65f,.41f,.68f,.93f,.88f,-.24f,.70f,-.97f,-.40f,-.96f,-.45f,-.35f,-.29f,.91f,-.28f,.92f,-.57f,-.37f,-.02f,.37f,-.37f,-.01f,-.48f,.74f,-.28f,.21f,.74f,-.99f,.45f,.91f,-.28f,-.11f,-.86f,-.22f,.48f,-.34f,.52f,.17f,.04f,.23f,-.54f,-.80f,-.60f,.04f,.47f,-.14f,.33f,.57f,.22f,-.67f,.02f,.69f,.93f,.69f,.93f,.99f,.75f,.47f,.21f,-.42f,.02f,.20f,.03f,.12f,.76f,.04f,-.43f,-.10f,-.52f,-.26f,-.10f,.78f,-.81f,-.72f,-.76f,.87f,.88f,.83f,-.27f,.82f,-.97f,-.59f,.80f,.66f,.66f,.59f,-.73f,.03f,-.16f,.69f,-.52f,.22f,.82f,-.11f,-.25f,-.39f,.56f,.93f,-.43f,-.73f,-.28f,-.06f,-.49f,-.95f,.39f,.51f,.29f,.40f,-.46f,-.11f,.60f,.03f,-.57f,.24f,-.55f,.88f,-.33f,.64f,-.79f,.59f,-.05f,.94f,.00f,.74f,.44f,-.86f,.23f,-.03f,-.66f,.45f,.08f,-.53f,.09f,.98f,-.28f,-.10f,.89f,.18f,.45f,-.98f,-.89f,.47f,.78f,-.11f,.53f,-.35f,.46f,.00f,.49f,-.13f,-.58f,-.56f,.73f,.21f,.20f,.72f,-.71f,-.68f,-.54f,.61f,-.80f,.53f,.90f,.20f,.25f,-.29f,.91f,-.88f,.04f,.98f,-.18f,-.16f,.28f,.28f,.60f,.35f,-.32f,.48f,-.99f,-.57f,.41f,.93f,-.33f,-.69f,.68f,-.99f,.40f,-.31f,.58f,.70f,-.95f,-.97f,.59f,-.71f,-.43f,.78f,-.92f,-.01f,.79f,-.34f,-.14f,.25f,.86f,.62f,-.12f,-.53f,.77f,-.30f,-.50f,-.82f,.76f,.27f,-.59f,.69f,-.87f,-.03f,.82f,.05f,.58f,.96f,.52f,.04f,.80f,-.66f,-.06f,.88f,-.59f,-.19f,-.47f,.84f,.15f,.90f,-.08f,.69f,-.34f,.12f,.14f,.86f,-.56f,-.59f,.08f,.27f,-.13f,.00f,-.00f,-.47f,.34f,.44f,.17f,-.57f,-.19f,-.09f,-.31f,.03f,-.60f,.40f,.58f,-.43f,-.81f,.08f,.04f,-.28f,.74f,.00f,-.59f,-.57f,-.82f,.99f,-.36f,.44f,-.51f,.26f,.75f,-.56f,-.86f,.85f,-.63f,.49f,.81f,-.53f,.23f,-.35f,-.25f,-.44f,-.03f,.23f,.98f,-.37f,-.25f,-.60f,.81f,.16f,.95f,.77f,-.18f,.45f,-.35f,.47f,.74f,.65f,.81f,-.02f,-.39f,-.75f,.33f,.35f,-.76f,-.12f,-.62f,.06f,.47f,.47f,-.74f,.58f,-.68f,-.77f,-.12f,-.41f,.26f,.39f,-.37f,.47f,.81f,.00f,.80f,-.59f,.74f,-.11f,-.81f,-.10f,-.03f,.88f,-.65f,.58f,-.93f,-.09f,-.30f,.72f,-.75f,-.10f,.01f,.21f,-.17f,.37f,-.96f,-.44f,-.69f,.36f,.89f,.24f,.06f,.20f,-.51f,.20f,-.25f,.64f,.06f,-.91f,.19f,-.71f,-.09f,.85f,.49f,-.39f,.80f,.42f,.61f,-.52f,.61f,-.13f,.48f,.67f,.60f,.93f,-.02f,.58f,.01f,-.17f,-.50f,-.67f,-.74f,-.12f,.92f,-.61f,-.21f,.30f,-.70f,-.56f,-.91f,-.65f,.13f,-.36f,-.91f,.93f,.69f,-.95f,.67f,-.46f,.26f,.59f,.11f,.64f,-.83f,-.00f,-.85f,.50f,.25f,.80f,-.83f,-.54f,-.96f,.68f,-.39f,.84f,.80f,.84f,.86f,-.60f,-.73f,-.90f,.22f,.52f,-.70f,-.70f,.35f,.57f,.55f,-.94f,.51f,.07f,-.57f,-.20f,.16f,.66f,.71f,.46f,-.03f,.11f,-.61f,-.84f,-.57f,.12f,-.37f,-.97f,-.83f,.47f,.40f,.54f,.79f,.95f,.40f,.23f,.45f,.66f,.33f,.61f,.90f,-.66f,-.18f,.95f,.91f,-.89f,-.66f,.37f,-.80f,-.36f,.02f,.23f,-.57f,-.19f,-.11f,-.01f,-.47f,-.67f,-.64f,.80f,-.34f,-.08f,-.54f,.96f,-.41f,-.43f,.52f,.30f,-.02f,.68f,-.82f,-.06f,-.24f,.53f,-.12f,-.64f,-.57f,-.93f,.17f,-.62f,-.57f,-.18f,.44f,-.93f,.27f,.47f,.02f,-.66f,.84f,-.56f,-.86f,-.30f,.45f,-.87f,-.18f,-.78f,-.58f,-.87f,-.01f,-.39f,-.82f,-.88f,.73f,-.96f,-.08f,-.83f,-.78f,-.40f,.55f,-.96f,-.72f,-.48f,-.58f,.56f,.17f,.51f,.81f,-.72f,-.46f,.98f,.59f,-.78f,-.00f,-.97f,.94f,.53f,.29f,.90f,.85f,-.83f,-.66f,-.22f,-.23f,.41f,-.13f,-.10f,.66f,-.18f,.83f,-.50f,-.14f,-.22f,.75f,-.81f,.56f,-.60f,-.11f,-.24f,-.37f,-.44f,.27f,-.84f,.21f,-.47f,.87f,.91f,-.95f,-.09f,.97f,.08f,-.49f,-.27f,.22f,.50f,-.54f,.41f,.84f,.25f,.24f,.12f,.76f,-.50f,.78f,-.35f,.66f,-.78f,.10f,.81f,.32f,-.41f,-.20f,-.99f,.80f,-.91f,-.80f,.53f,.83f,-.43f,-.12f,.26f,-.28f,.33f,-.88f,.39f,.15f,-.11f,-.35f,.46f,-.47f,-.30f,.42f,-.33f,.95f,.59f,-.43f,-.12f,.57f,-.57f,-.47f,.60f,.34f,.80f,.80f,-.64f,-.72f,.01f,.49f,-.03f,.09f,.38f,-.65f,-.22f,.61f,-.82f,.42f,-.33f,.91f,.55f,.45f,.85f,-.06f,-.58f,.64f,.12f,-.57f,.68f,.24f,-.88f,-.67f,-.65f,-.22f,-.11f,-.83f,-.82f,.76f,.33f,-.14f,.97f,-.18f,-.04f,.97f,.00f,.08f,.15f,.52f,-.19f,-.46f,-.50f,-.59f,.69f,.06f,-.14f,.27f,-.75f,.27f,-.14f,.15f,-.57f,-.42f,-.18f,.43f,.44f,-.65f,-.87f,-.49f,-.37f,.82f,-.72f,.01f,-.88f,.14f,-.92f,-.50f,.83f,-.14f,-.16f,-.50f,-.05f,-.63f,-.90f,.83f,.74f,.00f,.44f,-.66f,.93f,.27f,-.28f,.18f,-.80f,-.54f,-.08f,.22f,.43f,.16f,-.25f,-.14f,.25f,.55f,-.85f,.72f,.33f,.12f,-.65f,-.17f,.26f,-.40f,.78f,-.50f,.07f,-.98f,-.60f,-.38f,.95f,.75f,-.10f,-.30f,.57f,-.67f,.12f,.46f,-.23f,-.93f,-.34f,-.63f,.42f,.20f,.25f,-.83f,.09f,-.35f,-.70f,.73f,-.71f,-.65f,.53f,.93f,-.30f,.42f,.10f,.73f,-.96f,-.08f,.64f,.66f,-.20f,.62f,-.90f,-.63f,-.55f,.72f,.64f,-.42f,-.01f,-.62f,-.82f,.68f,-.86f,.48f,-.04f,-.02f,-.91f,-.99f,-.33f,-.46f,-.06f,-.92f,-.23f,.15f,.26f,.37f,.23f,.08f,.47f,-.78f,.86f,.52f,-.27f,-.01f,-.54f,-.42f,-.25f,-.20f,-.13f,.64f,.32f,.45f,.34f,-.13f,.99f,.45f,.81f,-.63f,-.27f,-.59f,-.29f,-.44f,.89f,-.01f,-.17f,.79f,.08f,.59f,-.01f,-.26f,.10f,.27f,.28f,-.12f,-.73f,.19f,.16f,-.12f,.49f,.43f,.74f,.56f,.90f,.28f,-.46f,-.62f,.47f,-.35f,-.36f,-.99f,.89f,-.64f,-.33f,-.43f,.97f,-.34f,.37f,-.16f,-.39f,.18f,-.95f,-.41f,.41f,-.21f,-.82f,-.93f,-.90f,.53f,.25f,.29f,-.84f,-.39f,-.72f,-.21f,-.78f,-.97f,-.92f,-.34f,.81f,-.08f,.34f,.26f,-.74f,-.26f,-.20f,.71f,.67f,-.08f,.35f,-.63f,-.11f,-.50f,-.71f,-.26f,-.46f,.01f,-.24f,.73f,-.22f,-.85f,-.68f,-.21f,-.56f,.16f,-.04f,-.77f,.24f,-.66f,.77f,.25f,-.27f,-.24f,-.36f,.89f,-.65f,.62f,.00f,-.74f,.22f,.82f,.72f,.58f,-.44f,.72f,-.55f,-.20f,.11f,-.62f,.48f,-.24f,-.02f,-.81f,.69f,-.58f,.26f,-.64f,-.48f,.79f,.38f,.68f,.70f,-.19f,.79f,.89f,-.57f,-.24f,-.57f,-.20f,-.80f,.54f,-.12f,-.94f,-.04f,.40f,-.16f,-.99f,.12f,.73f,-.02f,.63f,-.76f,-.40f,-.82f,.60f,.48f,.26f,-.77f,-.68f,.10f,-.81f,.90f,.97f,.10f,-.12f,-.07f,-.64f,.36f,-.37f,.25f,-.38f,-.65f,-.30f,-.71f,.35f,.69f,.29f,-.60f,.66f,.53f,.69f,-.27f,.04f,-.38f,-.47f,-.00f,-.30f,.71f,.44f,-.50f,.52f,.75f,.18f,-.47f,-.15f,.46f,-.42f,.32f,-.97f,.16f,.67f,-.13f,-.59f,.49f,-.02f,-.91f,-.05f,-.89f,.68f,-.21f,.57f,-.17f,.37f,-.36f,-.44f,.40f,.31f,-.51f,-.56f,.09f,.80f,-.61f,-.36f,-.66f,.29f,-.00f,-.72f,.14f,-.95f,-.18f,-.43f,-.25f,.28f,-.64f,-.37f,-.29f,.15f,.48f,.29f,-.86f,.43f,.85f,.56f,-.66f,-.93f,-.00f,.06f,.00f,-.73f,-.29f,-.29f,.59f,-.60f,-.24f,-.88f,.99f,.39f,.71f,-.58f,-.46f,-.86f,-.65f,-.17f,.28f,.85f,.74f,.32f,-.14f,.59f,.37f,-.94f,.28f,-.50f,-.88f,.36f,-.21f,-.22f,.55f,-.49f,-.22f,.33f,.34f,.59f,-.94f,.82f,-.06f,.82f,.71f,.03f,-.11f,-.02f,.85f,-.25f,.86f,-.30f,-.99f,.27f,.17f,.78f,.24f,-.85f,.97f,.35f,.44f,-.86f,.22f,.69f,.89f,-.11f,.94f,-.97f,-.73f,-.46f,-.85f,.33f,-.01f,-.37f,.23f,-.67f,.02f,-.89f,.99f,-.46f,.40f,-.60f,-.35f,-.13f,-.38f,-.18f,-.50f,-.52f,.06f,.01f,.68f,.69f,.17f,-.36f,.68f,-.01f,.22f,.96f,.58f,-.28f,.10f,-.49f,-.21f,-.74f,.06f,-.03f,.36f,.47f,-.94f,-.13f,.66f,-.03f,.81f,-.10f,-.34f,-.74f,-.02f,.36f,.13f,-.09f,-.23f,-.15f,-.84f,.61f,.63f,-.06f,-.59f,.49f,-.83f,.43f,-.51f,-.26f,-.49f,.79f,.92f,.69f,-.51f,.43f,.57f,.59f,.62f,.04f,-.66f,.12f,-.61f,.15f,-.29f,-.66f,-.05f,.27f,.83f,.62f,-.89f,.37f,.08f,-.41f,.60f,.57f,.27f,-.08f,.62f,-.75f,.71f,-.56f,-.08f,.98f,.63f,-.46f,-.76f,.75f,-.99f,-.80f,-.58f,-.16f,-.55f,.09f,.42f,-.11f,.18f,-.87f,-.89f,.04f,.24f,-.86f,-.20f,.09f,-.76f,.77f,-.39f,.60f,-.61f,-.04f,.16f,.57f,-.39f,-.75f,-.99f,-.77f,.91f,.58f,-.29f,.48f,.22f,.73f,-.29f,-.28f,.93f,-.78f,-.19f,.27f,-.76f,-.93f,-.95f,-.98f,.74f,.50f,.09f,-.76f,-.31f,-.95f,.18f,.95f,-.90f,-.87f,.87f,.39f,.91f,.91f,.43f,.21f,-.33f,-.82f,-.85f,.79f,.25f,.62f,.44f,-.19f,.80f,.77f,-.08f,.74f,-.25f,.90f,.93f,.55f,-.46f,-.63f,.73f,.51f,.31f,-.30f,-.46f,-.26f,.82f,-.44f,-.96f,-.05f,.25f,.67f,.37f,-.96f,-.70f,-.50f,-.33f,-.30f,.19f,.45f,.96f,.18f,-.44f,.10f,-.29f,-.83f,.18f,.14f,.91f,-.76f,.33f,-.75f,-.06f,.35f,.35f,-.22f,-.42f,-.34f,.60f,-.56f,.95f,-.35f,-.13f,-.58f,-.84f,.89f,-.67f,-.84f,.78f,-.29f,.83f,-.03f,.74f,-.98f,.54f,-.33f,.40f,.79f,.78f,.47f,-.60f,.66f,.69f,-.25f,-.38f,.90f,.91f,-.22f,.55f,-.30f,-.48f,-.16f,-.09f,-.23f,-.68f,-.62f,.49f,.34f,.18f,.88f,.08f,.95f,-.59f,.40f,.52f,.19f,.94f,.36f,.96f,-.31f,.71f,-.19f,.67f,.87f,-.39f,-.55f,-.01f,-.14f,.28f,-.75f,.70f,-.17f,.18f,.64f,.50f,-.83f,.85f,.85f,.38f,.04f,-.46f,.41f,-.21f,-.78f,.32f,.46f,.95f,-.16f,-.31f,-.24f,.06f,-.25f,-.94f,.99f,.75f,-.00f,-.42f,-.72f,.61f,-.06f,.78f,.92f,.25f,.90f,-.37f,.72f,.21f,-.72f,.36f,.76f,-.53f,-.36f,.45f,-.28f,-.47f,-.46f,.47f,-.30f,.30f,-.72f,.73f,-.61f,-.39f,.06f,.35f,.77f,-.38f,.52f,-.03f,-.85f,-.20f,.02f,.64f,-.72f,.10f,.11f,.81f,-.07f,.72f,.87f,.81f,-.21f,-.03f,-.86f,-.18f,.01f,-.77f,-.60f,.16f,.32f,-.46f,.00f,-.00f,-.77f,-.04f,.92f,.48f,-.84f,-.65f,-.93f,-.77f,-.11f,.77f,.86f,.96f,.31f,-.18f,-.94f,.66f,-.97f,.51f,.39f,-.66f,-.74f,.82f,.91f,.90f,-.22f,-.89f,.72f,.98f,-.14f,.73f,.68f,.68f,-.20f,.44f,-.76f,.40f,-.93f,-.43f,-.89f,-.74f,-.18f,-.79f,-.69f,.32f,.73f,-.85f,-.06f,.72f,-.25f,.37f,.76f,-.20f,-.70f,.27f,-.94f,.75f,-.29f,-.25f,-.82f,.38f,.50f,.56f,.62f,-.41f,-.10f,-.38f,.76f,-.42f,.89f,.16f,-.83f,-.05f,-.08f,-.41f,.55f,.01f,.93f,-.79f,-.46f,.63f,-.93f,-.26f,.51f,.57f,-.76f,-.94f,.21f,-.83f,.51f,-.38f,-.00f,-.35f,-.94f,.32f,-.70f,-.05f,-.73f,.19f,-.52f,.32f,-.48f,.99f,.18f,.36f,-.73f,.48f,.74f,.53f,.41f,.85f,-.42f,.46f,.18f,.66f,.99f,-.55f,-.02f,.54f,-.89f,-.78f,.68f,.34f,.90f,-.34f,.55f,-.50f,-.80f,-.26f,-.91f,-.72f,.75f,.23f,.87f,.15f,.77f,-.64f,-.78f,.03f,-.29f,-.31f,.02f,-.16f,.01f,.50f,-.35f,-.65f,.22f,.53f,-.25f,-.31f,.61f,.59f,-.83f,-.30f,-.53f,-.76f,-.28f,.71f,.52f,-.77f,.86f,-.01f,.92f,.90f,.20f,.20f,.49f,.38f,-.95f,.59f,-.25f,.42f,-.33f,.29f,.17f,-.17f,.78f,-.75f,.99f,-.43f,-.33f,-.61f,.41f,-.22f,-.25f,-.16f,-.74f,-.06f,.24f,.60f,-.96f,.06f,-.13f,-.27f,-.04f,-.04f,.45f,.25f,.83f,-.31f,.27f,-.06f,-.40f,.68f,-.52f,.92f,-.71f,-.78f,.93f,-.84f,-.51f,-.30f,-.15f,.02f,-.59f,-.64f,.06f,-.18f,-.55f,-.96f,.12f,-.89f,.97f,.91f,-.73f,.06f,.57f,-.82f,.59f,-.86f,.08f,.45f,-.95f,.67f,-.65f,-.29f,-.36f,.04f,-.82f,.10f,.55f,.32f,.55f,.32f,.81f,-.92f,.09f,-.35f,.88f,.31f,.06f,.89f,.00f,.37f,-.45f,.58f,.97f,.95f,.34f,-.41f,-.84f,.45f,-.81f,.73f,-.11f,.06f,-.68f,-.92f,.59f,.80f,.60f,-.04f,-.18f,.44f,-.83f,.71f,.68f,.02f,.62f,.83f,.60f,.81f,-.92f,.86f,-.57f,.26f,.30f,-.26f,.29f,-.74f,-.21f,.65f,.40f,.43f,-.95f,.36f,-.28f,.12f,-.89f,-.23f,.86f,-.11f,.70f,.52f,-.22f,-.83f,-.63f,.08f,.58f,-.88f,-.34f,-.06f,.44f,.10f,.36f,.99f,.04f,.41f,-.73f,.41f,.57f,-.41f,.35f,.36f,-.37f,.64f,-.70f,-.33f,-.57f,-.38f,.09f,.50f,-.09f,-.95f,.59f,.07f,.25f,.75f,-.29f,-.46f,-.69f,-.93f,-.49f,-.03f,-.25f,.65f,-.15f,.79f,.76f,-.73f,.57f,-.90f,.74f,.99f,.20f,.09f,.33f,-.55f,.42f,-.98f,-.26f,-.48f,.96f,.83f,-.75f,-.64f,-.09f,-.73f,.96f,-.26f,-.76f,.83f,.54f,.66f,.54f,.64f,-.58f,.45f,-.67f,.81f,.00f,-.77f,.66f,.36f,.93f,.75f,-.62f,.65f,-.52f,.08f,.02f,.12f,.35f,.85f,.83f,-.07f,-.62f,-.78f,-.98f,-.78f,.43f,.53f,-.55f,.04f,.99f,-.57f,-.52f,.78f,.11f,-.00f,.88f,.10f,-.43f,-.44f,-.51f,.73f,-.74f,.53f,-.26f,-.58f,.07f,-.48f,.23f,.94f,-.47f,-.53f,.04f,.18f,-.16f,-.86f,-.90f,-.04f,.05f,.03f,.53f,-.11f,-.86f,.80f,.34f,.61f,-.00f,-.99f,.55f,-.20f,-.84f,.21f,-.66f,-.43f,-.93f,-.83f,.00f,-.62f,-.59f,-.16f,-.56f,-.11f,.55f,-.30f,.96f,-.67f,.05f,.17f,.30f,-.86f,-.44f,.05f,.75f,-.44f,-.03f,-.61f,-.89f,.40f,.35f,-.42f,.35f,-.97f,.58f,.32f,.84f,-.13f,.01f,.89f,.60f,.06f,.80f,.67f,-.54f,.59f,-.52f,.06f,.86f,.19f,.04f,.75f,.86f,.11f,.00f,-.72f,-.85f,.09f,.78f,-.68f,.90f,.06f,.38f,-.14f,.97f,.07f,-.60f,.60f,-.78f,-.38f,.01f,-.99f,.61f,.53f,-.65f,.16f,.32f,.61f,-.94f,.88f,.75f,-.05f,-.24f,-.84f,-.49f,.89f,.39f,.22f,.20f,-.29f,.74f,-.01f,.08f,-.24f,-.80f,.57f,-.16f,.00f,-.48f,.68f,-.06f,.78f,-.66f,-.91f,.30f,-.72f,-.20f,.28f,-.94f,-.44f,-.29f,.27f,-.84f,.86f,-.47f,-.62f,.29f,-.59f,.71f,-.10f,-.47f,-.40f,-.62f,-.45f,.01f,-.88f,-.47f,-.11f,-.27f,.33f,-.12f,.68f,-.96f,.30f,-.73f,.13f,.70f,.78f,.18f,.35f,-.82f,-.65f,-.72f,.44f,-.03f,.12f,-.68f,-.68f,.48f,.89f,-.32f,.84f,.32f,.14f,-.68f,.44f,.17f,.56f,-.76f,.24f,-.88f,.04f,-.46f,-.34f,.28f,-.31f,-.97f,-.21f,-.78f,-.84f,-.22f,-.73f,.07f,-.56f,.01f,-.75f,-.19f,-.41f,.78f,-.23f,.93f,.91f,.96f,-.41f,-.27f,-.20f,-.59f,.25f,.38f,-.63f,.24f,.62f,.44f,.71f,-.55f,.61f,.50f,-.56f,.77f,.44f,.25f,-.17f,-.32f,.83f,-.33f,.16f,.90f,-.58f,-.63f,.28f,-.65f,.02f,.60f,.67f,.30f,.09f,-.48f,.54f,-.07f,.32f,-.51f,-.83f,.67f,.86f,.48f,-.14f,-.03f,-.49f,.17f,-.59f,-.21f,-.26f,-.93f,.80f,-.35f,.52f,-.40f,-.89f,-.77f,-.53f,.56f,-.58f,.45f,.54f,.81f,.05f,.54f,.11f,.78f,.53f,.88f,.13f,-.95f,-.23f,-.40f,.29f,-.37f,-.57f,.39f,.24f,.14f,.86f,.36f,-.72f,.12f,-.54f,.60f,.22f,.32f,-.02f,.69f,-.43f,-.64f,.92f,.37f,.07f,.16f,.05f,.11f,.19f,.21f,-.60f,-.02f,.43f,.84f,-.97f,-.12f,-.06f,-.11f,-.35f,.77f,-.12f,-.44f,-.02f,-.09f,.34f,.37f,.17f,-.63f,.07f,.74f,.57f,-.97f,.57f,.22f,.47f,-.92f,-.97f,-.01f,.47f,.08f,-.31f,.74f,.79f,-.68f,-.89f,-.10f,-.80f,-.91f,.06f,-.40f,.76f,.27f,.79f,.96f,-.95f,.62f,-.09f,.41f,.03f,-.03f,-.85f,-.51f,.92f,-.67f,.44f,.20f,-.29f,-.01f,.48f,.76f,-.74f,-.95f,-.62f,-.46f,.55f,.61f,-.51f,.26f,.53f,-.99f,.28f,.53f,-.41f,-.92f,.38f,-.53f,-.14f,.87f,.80f,.07f,-.76f,.25f,-.75f,-.83f,-.83f,-.01f,-.08f,.92f,-.33f,.67f,.08f,-.11f,.35f,-.17f,.70f,.23f,-.64f,-.91f,.56f,.82f,.42f,-.48f,.54f,-.04f,.00f,.05f,.00f,-.11f,.02f,-.26f,.46f,.04f,.84f,-.54f,-.33f,-.55f,-.00f,-.90f,.73f,-.19f,-.35f,-.40f,-.09f,-.15f,.92f,-.20f,-.95f,-.85f,.92f,.10f,.61f,.02f,-.01f,.54f,.07f,.85f,.50f,-.84f,.14f,-.38f,.40f,.22f,-.49f,.11f,-.49f,-.23f,-.39f,-.25f,-.31f,.65f,-.52f,-.38f,.46f,-.73f,.91f,-.71f,.77f,.65f,-.05f,.31f,-.60f,.08f,-.26f,.25f,.38f,-.68f,.55f,.83f,-.63f,-.75f,-.55f,.08f,-.42f,.35f,-.33f,-.45f,.00f,.67f,.78f,-.59f,.55f,-.90f,-.47f,.91f,.00f,-.15f,.34f,.75f,-.92f,.15f,.21f,.14f,-.14f,.22f,-.19f,.42f,.31f,.75f,-.66f,-.24f,.37f,.40f,.25f,-.15f,.47f,.03f,.14f,-.32f,.56f,-.58f,.88f,-.67f,.96f,.14f,-.78f,.98f,-.77f,-.52f,-.27f,.94f,.55f,.03f,.90f,-.56f,-.43f,-.58f,-.27f,-.79f,-.87f,-.71f,-.22f,-.48f,-.38f,-.96f,.58f,.24f,-.28f,-.47f,.33f,.10f,-.44f,-.94f,.29f,-.16f,.43f,-.79f,.81f,-.89f,-.63f,-.76f,-.15f,-.71f,-.77f,.13f,.67f,-.15f,.86f,.12f,-.37f,.42f,-.82f,-.64f,.42f,.21f,-.37f,-.88f,.14f,-.80f,.22f,.52f,-.84f,-.39f,.89f,.23f,-.21f,-.62f,.49f,.35f,-.13f,-.40f,.94f,-.10f,-.71f,-.54f,.75f,.28f,-.00f,-.78f,-.12f,.42f,.28f,-.99f,.82f,-.99f,-.97f,.61f,.29f,-.76f,.97f,-.57f,-.50f,.20f,.38f,-.95f,.08f,-.57f,-.98f,.24f,-.41f,.40f,.50f,.59f,.44f,-.76f,-.97f,-.65f,-.07f,-.12f,-.03f,.79f,.87f,-.63f,-.09f,.16f,-.47f,.62f,.55f,.17f,.32f,-.44f,.44f,.61f,-.52f,.51f,-.49f,.87f,-.34f,.63f,-.53f,-.30f,-.50f,-.11f,-.82f,-.63f,-.27f,.31f,-.60f,-.83f,.84f,.90f,-.13f,.61f,-.29f,-.62f,.51f,-.32f,.01f,-.35f,-.22f,-.03f,-.65f,.25f,.23f,-.70f,.09f,-.34f,-.82f,-.60f,.27f,.48f,-.04f,.91f,.40f,-.66f,.85f,-.20f,.85f,.61f,-.96f,-.25f,-.55f,-.75f,.00f,.78f,-.92f,.29f,-.00f,.66f,.03f,.47f,-.44f,-.43f,-.63f,.82f,-.00f,-.88f,-.14f,-.55f,.37f,.00f,.95f,.12f,.95f,.22f,-.98f,.82f,-.44f,.37f,-.09f,-.92f,.52f,-.26f,-.37f,.95f,.74f,.84f,-.97f,.09f,-.33f,-.87f,-.06f,.81f,-.85f,-.68f,-.67f,.65f,-.95f,.60f,-.55f,-.45f,.50f,.54f,-.18f,-.00f,-.53f,.68f,.88f,.36f,-.37f,.58f,-.90f,.69f,-.69f,.10f,.40f,-.88f,-.91f,-.90f,-.18f,.47f,-.60f,.17f,-.13f,.64f,-.24f,-.42f,-.89f,-.47f,.24f,-.09f,.13f,.22f,.35f,.49f,.73f,-.97f,-.58f,.49f,.20f,.21f,-.24f,.16f,-.77f,.02f,-.35f,.06f,.83f,-.04f,-.87f,-.61f,.78f,.25f,-.49f,-.73f,-.13f,.95f,-.59f,.41f,-.97f,-.53f,-.10f,-.45f,-.86f,.57f,-.58f,-.78f,.51f,-.05f,-.57f,.11f,.46f,.66f,-.48f,.51f,-.01f,-.53f,-.76f,.19f,-.01f,.62f,-.76f,.49f,.13f,-.44f,.01f,.98f,-.33f,-.18f,-.00f,.27f,.58f,.94f,-.21f,-.79f,.78f,.15f,-.19f,-.60f,.66f,.53f,-.13f,-.91f,-.96f,.16f,-.19f,-.40f,-.79f,-.18f,.66f,-.17f,.80f,.90f,.91f,.43f,-.83f,.74f,-.88f,-.41f,.07f,-.25f,-.53f,-.75f,-.43f,-.62f,-.35f,.90f,-.05f,.75f,-.76f,.87f,-.51f,.16f,.11f,-.79f,.29f,.36f,.56f,-.85f,-.51f,-.04f,.63f,.75f,.66f,-.65f,-.33f,-.58f,.09f,-.50f,-.70f,-.75f,.45f,-.75f,.21f,-.20f,-.39f,.48f,.97f,-.03f,.24f,-.18f,-.48f,.46f,-.04f,-.71f,-.11f,.89f,.56f,-.52f,.76f,.44f,.19f,.83f,.93f,-.51f,.83f,.73f,.12f,.52f,-.97f,.42f,.91f,-.17f,.13f,.44f,-.03f,.54f,.48f,-.25f,-.92f,-.46f,.39f,.82f,-.23f,.05f,.66f,-.08f,.60f,.14f,-.20f,-.32f,-.10f,-.21f,.12f,.56f,-.73f,.63f,.09f,.97f,.10f,-.88f,.71f,.25f,-.62f,-.30f,-.25f,-.64f,.35f,.91f,.57f,.08f,-.97f,-.41f,-.94f,-.91f,-.11f,.57f,-.68f,-.37f,-.39f,.83f,.31f,-.15f,.81f,-.53f,.63f,-.65f,.92f,-.40f,-.35f,.05f,.84f,.63f,.12f,-.97f,.98f,.21f,.91f,-.86f,-.12f,-.77f,.56f,.66f,-.60f,.67f,-.36f,-.33f,.39f,-.21f,.22f,.82f,-.51f,.86f,.28f,-.07f,.13f,-.91f,-.83f,-.60f,-.88f,.42f,.63f,-.70f,.54f,.70f,.38f,-.71f,-.15f,.87f,.07f,.81f,-.33f,-.87f,-.86f,.85f,.37f,-.17f,.71f,.87f,-.56f,.06f,-.66f,.79f,-.17f,.13f,-.22f,-.75f,.65f,.14f,.25f,.56f,.63f,-.68f,-.11f,.91f,.12f,-.01f,-.25f,-.76f,-.93f,.66f,.55f,.43f,-.35f,.60f,-.01f,-.68f,-.94f,-.34f,-.47f,.87f,.78f,.10f,.61f,-.34f,.19f,-.24f,-.20f,.65f,-.04f,-.46f,.08f,.10f,.54f,.28f,-.33f,-.89f,.88f,-.54f,-.04f,.22f,.53f,.70f,.07f,-.65f,-.86f,-.72f,-.61f,.15f,-.33f,-.46f,.96f,.51f,.86f,.84f,.03f,-.62f,.54f,-.40f,-.73f,.04f,-.40f,.23f,-.40f,.28f,-.66f,.71f,.33f,.00f,.59f,.08f,.81f,-.02f,.35f,-.57f,.92f,-.76f,.55f,-.28f,.53f,-.84f,-.76f,.62f,-.86f,.97f,.33f,-.31f,-.70f,.51f,.21f,-.88f,-.14f,.17f,.93f,.34f,-.38f,-.71f,-.68f,-.99f,-.80f,-.26f,.80f,.39f,.65f,.89f,-.11f,.27f,.59f,.72f,.88f,.41f,.27f,-.54f,.61f,-.24f,.64f,.39f,.06f,-.65f,-.23f,-.43f,-.74f,.23f,.06f,-.18f,-.72f,.30f,.07f,-.90f,-.49f,-.50f,.90f,.73f,-.00f,.12f,-.03f,.50f,-.15f,.70f,-.82f,.54f,.73f,-.49f,-.50f,-.22f,-.57f,.58f,-.73f,-.69f,-.58f,-.81f,-.48f,.92f,-.83f,-.67f,-.43f,-.64f,.53f,.00f,-.71f,-.36f,.38f,.92f,.98f,-.60f,-.52f,.11f,.78f,-.40f,.81f,-.50f,-.93f,-.04f,-.65f,.44f,-.66f,-.23f,.09f,-.14f,-.40f,.26f,.82f,.68f,.61f,.93f,.02f,-.86f,-.74f,-.81f,-.84f,-.64f,-.70f,-.36f,.83f,-.57f,-.74f,-.22f,.51f,-.96f,-.07f,-.57f,.18f,.95f,.25f,.76f,-.59f,-.09f,-.74f,.33f,.64f,-.90f,.62f,-.16f,-.69f,.46f,-.86f,.21f,-.25f,.27f,.66f,.63f,-.19f,-.16f,-.31f,-.37f,.99f,.96f,-.84f,-.05f,-.21f,.05f,-.22f,-.09f,.32f,.61f,.94f,-.30f,.73f,-.79f,-.84f,.36f,-.34f,.00f,.19f,.14f,-.94f,-.98f,.93f,.09f,-.75f,.53f,-.39f,-.95f,-.21f,.17f,.87f,.21f,-.29f,.15f,-.96f,-.30f,-.13f,.43f,-.01f,-.08f,.57f,.77f,.98f,-.74f,-.89f,.79f,-.10f,-.15f,.28f,.47f,-.62f,.86f,.77f,.21f,-.22f,.63f,-.42f,-.10f,.42f,.02f,-.15f,-.74f,.66f,-.69f,-.82f,-.39f,.17f,-.16f,-.04f,-.69f,.97f,-.33f,.74f,-.69f,-.01f,-.87f,.43f,.99f,-.18f,-.37f,.86f,-.96f,.97f,-.62f,-.02f,.81f,-.21f,.78f,-.36f,.79f,.14f,.04f,.13f,.77f,.84f,-.21f,-.24f,.50f,-.46f,-.64f,-.43f,-.26f,-.08f,-.14f,-.70f,.77f,-.59f,-.39f,.60f,-.06f,.42f,-.83f,-.43f,.90f,.76f,-.52f,-.92f,-.22f,-.15f,-.54f,.87f,.20f,.80f,.93f,-.14f,-.33f,-.83f,.20f,-.98f,.18f,.32f,-.65f,.00f,-.22f,.89f,-.07f,-.31f,.05f,-.49f,-.58f,.11f,-.23f,.18f,-.58f,-.30f,-.83f,.39f,.56f,-.49f,.45f,-.79f,.79f,-.67f,.67f,-.15f,.32f,-.91f,.46f,.11f,.49f,.78f,.33f,-.66f,-.64f,.63f,-.42f,-.25f,-.80f,-.24f,-.69f,.56f,.84f,.77f,.74f,-.39f,.54f,.83f,.22f,-.98f,.26f,.17f,.63f,.00f,.55f,-.92f,.94f,-.32f,-.87f,-.74f,.11f,.61f,.55f,-.72f,.19f,-.44f,-.97f,-.25f,-.94f,.75f,-.41f,-.40f,-.83f,-.82f,-.51f,-.26f,.49f,.98f,.26f,.91f,.30f,-.37f,.50f,-.77f,.39f,-.02f,.22f,-.91f,.25f,.98f,.92f,.85f,-.23f,-.00f,-.35f,-.63f,.88f,.39f,.74f,-.04f,.34f,.02f,-.82f,.73f,.33f,-.59f,.47f,.56f,.48f,-.34f,.84f,-.68f,-.28f,.96f,-.06f,.30f,-.75f,.91f,.09f,.29f,-.84f,.97f,-.66f,.52f,-.00f,.56f,-.46f,.29f,.49f,.64f,.77f,-.24f,.20f,.34f,.40f,.20f,.85f,-.80f,.28f,.93f,.72f,-.30f,.62f,.61f,-.63f,-.66f,-.46f,.95f,.05f,-.47f,.83f,.02f,-.33f,-.35f,.38f,-.51f,.11f,.52f,.14f,.87f,-.71f,-.87f,-.22f,.52f,.82f,-.77f,.72f,.48f,-.11f,-.77f,.64f,-.61f,.09f,-.16f,-.30f,.56f,.50f,-.66f,-.39f,.81f,.42f,.29f,-.32f,.67f,-.59f,-.60f,.74f,.93f,-.49f,-.34f,-.35f,.32f,-.75f,.32f,-.03f,-.39f,.61f,-.84f,-.46f,-.33f,.99f,-.71f,.71f,-.48f,.27f,.65f,-.08f,.86f,-.69f,.58f,.10f,-.64f,.82f,-.66f,-.65f,.58f,-.42f,.85f,-.34f,.99f,-.02f,-.05f,.51f,-.40f,.24f,.61f,-.13f,-.78f,-.31f,-.89f,.62f,.30f,-.27f,-.68f,-.41f,-.77f,.64f,.94f,.01f,.29f,-.45f,-.15f,-.73f,.78f,-.27f,-.80f,-.70f,-.30f,-.69f,.38f,-.31f,.37f,.08f,.97f,-.32f,-.01f,.70f,.61f,.39f,.58f,.35f,.32f,.17f,.86f,-.04f,-.75f,.98f,.81f,-.58f,.78f,.94f,.65f,.89f,-.36f,-.43f,.27f,.99f,-.99f,.11f,.04f,-.08f,-.16f,.20f,-.66f,.87f,-.05f,-.57f,.27f,.90f,-.41f,-.60f,.52f,-.95f,.30f,.26f,.54f,-.71f,.44f,-.70f,-.29f,-.84f,-.21f,-.59f,.55f,-.98f,-.85f,-.98f,.84f,.99f,.25f,.14f,-.42f,-.91f,-.48f,-.52f,-.49f,.33f,.05f,-.46f,.97f,.89f,-.12f,-.45f,.91f,-.13f,-.22f,-.74f,-.34f,.00f,.34f,.16f,-.31f,-.11f,.77f,-.70f,.81f,.84f,-.30f,.18f,.98f,-.12f,-.36f,.48f,.26f,-.87f,.21f,.64f,.36f,-.85f,-.04f,.44f,-.89f,.94f,-.98f,.19f,-.40f,-.48f,-.36f,.77f,.29f,.18f,-.02f,-.72f,.44f,.35f,.42f,-.27f,.51f,.94f,.14f,.53f,-.56f,-.07f,.37f,-.33f,.83f,-.53f,.33f,.86f,-.13f,-.52f,.11f,-.23f,.25f,-.65f,-.06f,.42f,.71f,.94f,-.78f,.66f,.96f,.54f,.20f,-.36f,-.52f,-.26f,-.59f,.42f,.34f,-.23f,-.35f,.67f,.49f,.22f,-.31f,.17f,-.34f,.13f,.06f,.78f,-.24f,-.50f,.41f,-.18f,-.93f,-.98f,-.02f,-.16f,-.11f,-.37f,-.66f,-.63f,.14f,-.87f,-.60f,.60f,-.45f,.75f,-.41f,.62f,.24f,.71f,.63f,.77f,.11f,-.99f,.03f,.50f,-.72f,-.77f,-.25f,.00f,.51f,-.77f,-.35f,-.91f,.94f,.40f,.89f,.25f,-.42f,-.24f,-.15f,-.85f,.32f,-.42f,.66f,.57f,-.89f,-.08f,.76f,-.57f,.09f,-.26f,-.30f,-.46f,-.44f,-.06f,.79f,.67f,-.60f,-.59f,-.05f,.11f,-.60f,-.22f,-.99f,.51f,-.30f,.70f,.85f,.00f,-.55f,-.26f,-.28f,-.55f,.85f,.49f,-.33f,-.95f,-.34f,-.26f,-.41f,-.37f,-.07f,.84f,-.94f,.36f,-.97f,.04f,.52f,.36f,.36f,-.41f,-.66f,.91f,.99f,-.70f,-.36f,.20f,.97f,-.87f,-.32f,-.86f,-.85f,-.33f,.69f,.44f,-.25f,-.44f,-.44f,-.90f,.93f,.74f,-.52f,.71f,.97f,.13f,.04f,.42f,.08f,-.71f,.08f,.13f,-.42f,-.08f,-.49f,-.13f,.91f,.08f,.02f,.42f,-.44f,-.32f,.34f,.27f,.70f,.49f,-.30f,-.00f,-.87f,.08f,.29f,.64f,.02f,-.29f,.23f,-.53f,-.93f,.42f,-.47f,-.79f,.66f,.75f,-.30f,.16f,.81f,-.81f,.36f,.71f,.90f,.15f,.98f,-.77f,.96f,.10f,.40f,.05f,-.76f,.71f,-.12f,-.30f,.93f,-.00f,-.53f,-.25f,-.10f,.04f,-.07f,.04f,-.89f,.53f,-.76f,.60f,.07f,-.31f,.06f,-.35f,.33f,.01f,-.25f,-.71f,-.38f,-.39f,.15f,-.28f,-.44f,-.64f,-.80f,-.61f,-.57f,.46f,.61f,-.49f,-.07f,-.17f,-.94f,.73f,-.06f,-.02f,.19f,.54f,.38f,.69f,-.67f,-.27f,.05f,.75f,-.48f,-.13f,-.18f,-.02f,.37f,.63f,-.97f,.30f,.82f,.82f,.88f,.25f,-.64f,.60f,-.41f,-.12f,-.73f,-.65f,-.53f,.02f,.58f,-.36f,-.53f,.88f,-.51f,.66f,-.79f,-.23f,.65f,-.38f,.99f,-.51f,-.62f,.72f,-.31f,-.87f,.56f,.18f,.71f,.01f,-.72f,.59f,.15f,-.73f,.43f,-.81f,-.62f,.08f,.82f,.98f,.94f,-.97f,.97f,-.27f,-.17f,-.41f,-.20f,-.61f,.31f,-.25f,-.58f,.95f,-.36f,.28f,-.54f,-.92f,-.87f,-.61f,.12f,-.24f,.71f,.90f,-.29f,-.75f,.78f,.30f,.51f,-.48f,-.39f,.75f,-.90f,.65f,-.07f,.17f,-.55f,.27f,-.66f,-.23f,-.13f,.06f,-.27f,.13f,.53f,-.06f,.88f,.81f,-.56f,-.09f,-.26f,-.84f,-.22f,-.59f,.23f,.60f,-.60f,.90f,-.86f,.00f,-.43f,-.15f,-.33f,-.89f,.20f,-.08f,.37f,.24f,.86f,.67f,-.52f,-.88f,.45f,-.56f,.00f,-.45f,.07f,-.82f,.08f,-.27f,-.64f,-.88f,.92f,-.99f,-.12f,.89f,.35f,-.29f,.48f,-.35f,-.50f,-.60f,.31f,-.46f,.60f,.47f,.54f,.45f,.86f,.54f,-.99f,.59f,-.01f,.86f,-.10f,.54f,-.76f,-.88f,.75f,-.61f,.40f,-.25f,.92f,.14f,-.18f,.04f,-.31f,.29f,-.80f,.67f,-.60f,-.76f,-.01f,-.19f,.18f,.55f,.03f,.23f,-.30f,-.66f,-.12f,-.16f,.97f,.40f,.52f,-.15f,-.89f,.71f,.31f,.96f,.90f,-.19f,-.35f,.28f,-.64f,-.45f,-.78f,-.98f,-.36f,.00f,-.53f,-.91f,-.41f,.21f,.43f,-.24f,-.37f,-.91f,.66f,.19f,.76f,.72f,-.44f,.55f,.51f,.45f,.68f,-.26f,.09f,-.87f,.16f,.52f,.85f,-.61f,-.28f,-.50f,.32f,-.24f,.09f,.18f,.46f,.80f,-.04f,.07f,.29f,-.57f,-.22f,-.92f,-.49f,.98f,.99f,.61f,.26f,.85f,-.24f,.47f,-.40f,-.59f,-.00f,-.89f,-.33f,.37f,.26f,.31f,-.77f,-.89f,-.03f,-.26f,.85f,.71f,-.12f,.64f,.92f,-.54f,.96f,.42f,.85f,-.85f,.22f,.61f,.64f,-.82f,.01f,-.07f,.58f,-.33f,.25f,.26f,-.52f,.44f,-.84f,.50f,-.02f,-.29f,.57f,-.43f,-.62f,.66f,-.01f,-.86f,.83f,.90f,-.16f,.66f,-.81f,-.30f,.15f,-.62f,.38f,.92f,-.11f,.39f,.51f,-.27f,.62f,-.86f,.00f,-.16f,.70f,-.77f,-.92f,-.73f,-.28f,-.47f,-.70f,-.66f,-.78f,-.86f,.56f,.28f,.31f,.96f,-.64f,.18f,.49f,-.81f,-.56f,-.98f,.99f,-.93f,.04f,-.63f,-.22f,-.63f,-.49f,.08f,.47f,.71f,-.35f,-.76f,.01f,-.92f,.80f,.23f,.69f,-.66f,.14f,-.21f,.11f,-.38f,-.11f,-.12f,-.77f,-.90f,-.24f,.78f,.54f,.84f,.18f,-.26f,-.08f,-.66f,-.83f,-.28f,.90f,-.89f,.98f,-.75f,-.92f,-.56f,.29f,.03f,.25f,-.64f,.23f,-.30f,.44f,.98f,-.48f,.66f,.80f,-.31f,.17f,.45f,-.43f,.59f,-.78f,.10f,.27f,.04f,.85f,-.25f,-.22f,-.07f,-.64f,-.86f,-.39f,.73f,.81f,-.71f,.63f,.87f,-.41f,.45f,-.00f,-.01f,.40f,.10f,-.59f,.13f,-.11f,.13f,.18f,-.06f,-.18f,.28f,-.42f,-.62f,-.82f,.34f,-.61f,.61f,-.47f,.34f,-.68f,.92f,-.08f,.90f,.52f,.71f,-.14f,.73f,-.72f,-.43f,.15f,-.10f,.49f,-.33f,.71f,-.13f,-.48f,-.39f,.08f,.50f,.47f,-.79f,-.21f,-.75f,.89f,.15f,.21f,.62f,-.05f,.62f,-.83f,.87f,-.34f,.61f,-.60f,-.42f,.11f,.03f,-.66f,-.05f,.42f,.19f,.82f,.69f,-.33f,-.92f,-.02f,-.00f,.82f,.83f,-.02f,.44f,.11f,.19f,.01f,-.09f,.38f,-.32f,.30f,.84f,-.65f,-.01f,-.08f,-.02f,-.83f,-.47f,.95f,-.99f,.37f,-.38f,.21f,.35f,-.38f,.92f,.60f,.46f,-.14f,.28f,.20f,-.15f,-.03f,.96f,-.33f,.08f,-.48f,-.56f,.96f,-.03f,-.56f,-.25f,.90f,.16f,.21f,-.45f,.77f,.21f,.61f,.74f,-.07f,.93f,-.35f,-.26f,.52f,.74f,-.91f,.33f,.09f,-.45f,.58f,-.53f,.31f,.42f,.70f,-.85f,-.19f,.63f,-.87f,.67f,-.49f,.51f,-.08f,.96f,-.80f,.00f,.79f,.44f,.69f,-.37f,-.86f,-.63f,.90f,.79f,-.56f,-.26f,.66f,.62f,.22f,.27f,.75f,-.86f,.23f,.34f,-.50f,-.40f,-.05f,.12f,-.26f,.40f,-.74f,-.94f,-.65f,-.36f,-.66f,-.96f,-.34f,.57f,-.44f,.09f,-.88f,.66f,-.04f,.46f,-.33f,-.72f,.69f,.58f,-.57f,-.75f,.48f,.31f,-.04f,.34f,-.95f,.49f,-.63f,-.27f,.61f,-.96f,-.00f,-.27f,-.26f,.26f,-.33f,.30f,.61f,.81f,.12f,.24f,.24f,-.07f,-.23f,-.50f,-.55f,-.22f,-.89f,-.97f,.05f,.46f,.11f,.62f,-.02f,-.34f,-.94f,.04f,.97f,.43f,.58f,-.46f,.59f,-.36f,.47f,-.06f,-.57f,-.57f,.10f,.38f,-.43f,.23f,-.91f,.35f,.09f,.91f,.30f,.84f,.09f,.40f,-.75f,.50f,-.36f,-.95f,-.86f,-.78f,-.69f,.28f,.93f,.19f,-.15f,.90f,-.19f,.71f,.98f,.15f,.31f,-.87f,-.61f,.51f,.71f,.67f,-.40f,.70f,.34f,.61f,.61f,.47f,.49f,-.13f,-.43f,.79f,-.45f,-.57f,.40f,-.23f,.53f,.27f,-.58f,.43f,-.10f,-.72f,.27f,-.09f,.56f,.64f,.11f,-.57f,-.27f,.60f,-.29f,.16f,.77f,-.96f,.40f,.13f,.86f,-.42f,.62f,-.21f,-.11f,.92f,.07f,-.08f,.70f,.34f,-.82f,-.46f,.86f,-.01f,-.42f,.41f,.74f,-.03f,-.69f,.38f,.03f,.94f,-.81f,-.44f,.26f,-.48f,-.01f,.81f,.41f,-.87f,-.35f,-.23f,-.32f,.19f,-.31f,-.36f,-.39f,.96f,-.03f,.64f,.65f,-.40f,.80f,.61f,.83f,.37f,.63f,.42f,-.01f,.95f,-.67f,.59f,-.02f,-.06f,.63f,.20f,.27f,-.15f,.13f,.49f,-.43f,.92f,-.76f,.20f,-.35f,.20f,-.82f,-.79f,.09f,-.62f,.60f,-.20f,-.89f,-.22f,-.63f,.31f,.27f,-.67f,.23f,.13f,.18f,-.82f,.95f,.52f,-.26f,.55f,-.77f,-.29f,.43f,-.34f,-.61f,.72f,.21f,-.21f,.21f,.47f,-.02f,-.13f,.10f,.81f,.73f,-.50f,-.59f,-.32f,-.18f,-.02f,.19f,-.58f,-.35f,-.52f,.40f,-.88f,-.87f,.75f,-.08f,-.36f,.49f,.46f,.72f,.47f,-.69f,-.53f,-.67f,.27f,.51f,.73f,-.70f,-.52f,.83f,.03f,-.03f,-.72f,.22f,.29f,-.92f,-.07f,-.58f,.21f,-.62f,.08f,.52f,-.70f,.68f,-.72f,-.66f,.24f,.80f,.48f,.46f,-.41f,-.47f,.06f,-.11f,-.30f,.92f,-.28f,.53f,-.08f,.28f,.55f,.87f,.65f,-.00f,-.44f,-.17f,.35f,-.43f,.01f,-.92f,.01f,-.33f,.21f,.34f,.67f,.48f,-.23f,-.08f,.07f,-.17f,-.02f,-.62f,-.56f,.47f,-.10f,.49f,-.33f,.04f,-.92f,-.68f,.80f,.49f,.42f,.86f,-.23f,-.74f,-.64f,.83f,.39f,-.53f,-.69f,-.86f,-.30f,.22f,-.85f,.78f,.00f,-.86f,.94f,.33f,.85f,.33f,-.34f,.06f,-.25f,.26f,-.88f,.90f,-.47f,-.29f,-.56f,-.59f,.66f,-.06f,-.03f,.75f,-.64f,.03f,-.15f,.69f,-.59f,.66f,.37f,-.21f,.04f,-.24f,.49f,-.30f,-.27f,-.93f,-.16f,.73f,-.22f,-.63f,.98f,-.47f,.60f,-.67f,-.31f,-.24f,-.57f,.10f,.49f,-.52f,-.81f,-.28f,.13f,-.36f,.87f,-.61f,.38f,-.35f,.05f,-.01f,.38f,.92f,-.43f,-.87f,.35f,.84f,.47f,.19f,.71f,-.16f,.32f,.64f,-.27f,.30f,-.88f,.24f,.97f,-.02f,.59f,-.45f,-.13f,.32f,.40f,-.69f,.24f,-.06f,-.29f,-.74f,.01f,.36f,.16f,.18f,-.89f,-.62f,.63f,-.83f,.37f,.88f,.53f,.43f,.16f,.72f,.94f,-.04f,.86f,.64f,.58f,-.03f,-.83f,.78f,-.96f,-.11f,-.30f,.20f,-.60f,-.02f,.52f,.62f,-.99f,-.96f,.17f,-.79f,-.87f,-.32f,.39f,.31f,.02f,-.84f,-.77f,.49f,.65f,.83f,-.30f,.98f,-.13f,-.94f,.14f,.40f,-.47f,.81f,.88f,-.75f,.88f,.54f,.89f,-.52f,.92f,-.10f,.11f,.83f,.11f,-.99f,.22f,-.41f,.00f,.80f,-.53f,.19f,.26f,-.17f,-.70f,-.56f,-.20f,.26f,.91f,-.83f,.67f,.56f,.83f,.43f,-.39f,.28f,.83f,-.43f,.92f,.88f,-.68f,-.37f,.62f,-.68f,.70f,-.16f,-.78f,.88f,-.84f,.32f,-.22f,.72f,.58f,.40f,.24f,.90f,-.08f,-.03f,-.89f,.55f,-.76f,-.34f,-.15f,.19f,.36f,.46f,-.60f,-.43f,.54f,-.38f,-.02f,-.21f,-.60f,-.74f,.92f,.65f,.47f,.75f,.58f,.95f,-.22f,-.64f,-.28f,-.12f,.84f,-.37f,-.18f,-.50f,.08f,.14f,.31f,-.56f,-.18f,.32f,.52f,.46f,.19f,.31f,-.48f,-.95f,-.31f,.83f,-.31f,.22f,.41f,-.02f,.05f,.60f,.36f,.78f,.50f,.50f,.61f,-.59f,-.00f,.50f,.96f,-.80f,-.56f,.92f,-.00f,.25f,.79f,.76f,.14f,-.00f,-.99f,-.85f,.81f,-.26f,-.58f,.21f,-.68f,-.90f,.68f,-.48f,.59f,.08f,.62f,.38f,.64f,-.54f,.65f,.60f,-.45f,-.54f,-.40f,.13f,-.94f,-.86f,-.67f,.09f,-.53f,-.45f,.23f,.19f,.62f,-.18f,-.51f,.56f,.81f,.55f,-.62f,.60f,.44f,.61f,.58f,.13f,.13f,.46f,-.04f,-.01f,-.61f,-.76f,-.87f,-.25f,.69f,-.29f,.53f,-.59f,-.22f,.22f,-.08f,.77f,-.08f,.65f,-.90f,.19f,.82f,-.26f,.00f,.12f,.22f,-.88f,.52f,-.37f,-.97f,.34f,.94f,.84f,.88f,.35f,.31f,-.22f,.41f,-.96f,.93f,-.43f,.30f,-.20f,-.41f,.96f,.36f,-.88f,.72f,.32f,-.23f,.37f,-.80f,.05f,.52f,.50f,-.94f,-.43f,-.05f,-.44f,.87f,-.11f,-.01f,.85f,.41f,.30f,-.13f,-.02f,-.49f,-.68f,-.15f,-.49f,-.04f,-.40f,-.56f,.32f,.70f,-.38f,-.12f,-.64f,-.69f,-.22f,.40f,-.77f,.44f,-.36f,-.68f,-.45f,-.39f,.00f,-.44f,.73f,.14f,-.21f,-.50f,-.28f,.04f,.25f,.24f,-.70f,-.04f,.78f,.90f,-.37f,.14f,-.31f,-.98f,-.23f,.15f,-.04f,-.43f,-.51f,.34f,-.69f,-.33f,-.00f,.33f,.77f,.55f,.88f,.56f,-.06f,.86f,.61f,-.97f,.42f,.96f,-.23f,.56f,-.82f,.41f,.87f,-.93f,-.40f,-.71f,-.88f,.49f,-.39f,-.66f,-.16f,.57f,-.77f,.15f,-.36f,-.13f,.69f,.36f,.75f,-.84f,-.44f,.77f,.69f,-.89f,.90f,-.91f,.09f,.86f,.27f,.83f,.75f,-.17f,.80f,.40f,.20f,-.40f,.87f,-.85f,-.47f,-.40f,-.47f,-.17f,-.42f,-.14f,-.83f,.04f,.80f,.72f,-.07f,-.24f,-.32f,.91f,-.67f,.27f,.65f,.68f,-.94f,-.88f,-.27f,.74f,-.36f,.88f,-.78f,.44f,-.31f,.21f,.57f,.01f,.25f,-.02f,.44f,-.88f,.96f,-.91f,.66f,.88f,.97f,.53f,-.90f,.07f,.70f,-.41f,-.52f,-.00f,-.48f,.27f,-.19f,.96f,-.00f,.23f,.13f,-.56f,-.69f,.66f,.19f,-.79f,.87f,.91f,-.35f,-.67f,.01f,-.64f,-.48f,.77f,.29f,-.32f,-.36f,-.07f,-.36f,-.03f,.51f,-.99f,.42f,-.95f,.31f,.96f,.58f,.91f,.96f,.42f,-.76f,-.73f,-.52f,.64f,.24f,-.28f,-.50f,.56f,.50f,.41f,.54f,.10f,.10f,-.57f,-.73f,.50f,.36f,-.71f,.49f,-.08f,-.33f,-.78f,-.77f,.84f,.40f,.91f,-.39f,.26f,-.76f,-.25f,-.15f,-.07f,-.34f,.82f,-.94f,.21f,-.99f,.33f,.79f,-.41f,.71f,-.40f,.87f,.82f,-.34f,-.99f,.35f,-.41f,.10f,.60f,.84f,-.28f,.46f,.63f,-.47f,-.24f,-.05f,-.92f,.55f,.59f,-.23f,.92f,.86f,-.16f,-.51f,-.63f,-.34f,.56f,-.98f,-.07f,.64f,-.83f,.71f,.40f,-.47f,-.67f,.80f,-.64f,.86f,-.00f,-.64f,-.54f,.53f,-.54f,.13f,-.73f,.71f,.44f,.62f,.17f,.80f,.82f,-.87f,-.54f,-.56f,.35f,.02f,.18f,-.06f,-.00f,.13f,.16f,.18f,-.11f,.35f,.70f,.84f,.48f,-.03f,.07f,.20f,-.84f,-.18f,-.30f,.79f,-.88f,-.28f,.64f,.70f,-.05f,-.18f,.59f,-.87f,-.98f,.57f,.11f,.84f,-.33f,.55f,-.89f,-.07f,-.64f,-.17f,.74f,.70f,-.54f,.42f,-.64f,-.74f,-.90f,-.40f,.69f,.62f,.26f,.50f,-.61f,.60f,.54f,.23f,.30f,.68f,.96f,-.19f,.38f,-.73f,.04f,-.44f,.51f,.87f,.56f,.72f,.26f,-.61f,-.36f,.67f,.64f,-.67f,-.85f,-.30f,.37f,-.34f,-.00f,.30f,.86f,-.19f,.08f,.46f,-.14f,.45f,-.52f,.40f,-.69f,-.60f,-.53f,-.35f,-.15f,-.01f,-.93f,-.97f,.48f,.10f,.64f,.61f,-.02f,-.32f,.67f,.30f,-.73f,-.75f,-.24f,.43f,-.68f,-.09f,.76f,-.50f,-.85f,-.94f,-.73f,.85f,-.73f,-.93f,.52f,-.67f,.67f,.15f,.25f,.24f,-.06f,-.20f,-.43f,.77f,-.09f,.81f,.28f,.32f,-.34f,.15f,.95f,.48f,.37f,-.39f,.92f,.06f,.97f,-.59f,.93f,-.22f,-.16f,-.29f,.00f,.70f,-.78f,.56f,-.05f,.39f,.23f,-.89f,.29f,-.08f,-.67f,-.51f,-.09f,.64f,-.75f,-.89f,.91f,-.05f,-.33f,.84f,-.74f,-.95f,-.12f,.59f,.27f,.03f,.98f,-.16f,.63f,.24f,.65f,-.64f,-.89f,.27f,.55f,-.22f,-.27f,.68f,.89f,-.78f,.36f,-.98f,-.53f,.44f,.72f,-.00f,-.08f,-.58f,.96f,.05f,-.75f,.69f,.14f,-.55f,.83f,-.66f,.30f,-.85f,.63f,.03f,-.10f,.73f,.04f,.70f,-.95f,-.52f,-.45f,.97f,-.16f,-.31f,-.61f,-.55f,.58f,.20f,-.17f,-.80f,.75f,-.31f,-.48f,.81f,-.61f,.59f,-.21f,-.16f,.15f,-.84f,.90f,-.53f,.82f,.17f,-.75f,-.51f,-.00f,-.00f,-.07f,-.25f,-.40f,-.09f,-.39f,.50f,-.63f,.76f,-.80f,.03f,.44f,-.18f,-.75f,.08f,.55f,.22f,-.88f,.47f,.32f,.40f,.58f,-.08f,.50f,.40f,.41f,-.28f,.40f,-.30f,-.39f,.32f,.78f,.56f,-.55f,.08f,-.50f,-.94f,-.69f,.88f,-.97f,-.67f,.98f,.44f,-.78f,.08f,-.66f,.83f,-.84f,.93f,-.55f,.03f,-.25f,-.91f,-.95f,.88f,-.19f,-.23f,.67f,-.21f,-.03f,-.10f,-.77f,-.27f,.19f,-.37f,.95f,.29f,-.43f,.94f,.47f,.96f,-.92f,.26f,-.20f,-.00f,-.55f,.37f,-.76f,-.02f,-.30f,-.47f,.65f,-.14f,.09f,-.26f,-.19f,-.63f,-.28f,.58f,.87f,.87f,.50f,.73f,.96f,-.16f,.55f,.68f,-.62f,-.81f,.32f,-.47f,.29f,.53f,-.19f,-.98f,-.41f,.29f,-.48f,-.16f,.11f,.09f,-.27f,-.97f,.95f,.36f,-.66f,-.68f,.41f,.01f,.07f,-.85f,.13f,-.02f,.31f,-.53f,-.13f,-.29f,-.43f,.68f,.14f,.85f,-.57f,-.63f,-.46f,-.44f,-.70f,-.27f,.68f,.68f,.90f,-.59f,.33f,.63f,.40f,.69f,.26f,.44f,-.91f,-.00f,-.60f,-.45f,-.51f,.42f,.53f,-.19f,-.97f,-.79f,.98f,.99f,-.05f,-.17f,.16f,-.77f,.14f,.34f,.15f,.15f,-.00f,.27f,-.17f,.68f,.37f,.50f,-.14f,-.03f,-.70f,-.06f,-.33f,.50f,.57f,-.37f,-.96f,-.89f,-.43f,-.45f,.11f,-.33f,-.19f,-.62f,.81f,.56f,-.38f,.69f,.61f,-.30f,-.58f,-.67f,-.72f,-.83f,.64f,.94f,-.61f,.83f,-.43f,-.73f,-.67f,.96f,-.73f,.61f,.89f,-.97f,.80f,.30f,-.42f,.92f,.81f,.03f,.48f,.98f,-.04f,-.68f,.13f,.90f,.17f,.59f,.48f,-.40f,-.70f,-.25f,-.46f,.69f,-.02f,-.53f,-.60f,.55f,-.49f,-.47f,-.83f,.59f,-.26f,.21f,.86f,-.51f,-.26f,.59f,-.18f,-.93f,.55f,-.20f,-.92f,.87f,.95f,-.32f,.84f,.78f,.43f,.56f,.43f,.36f,.14f,-.19f,-.84f,-.30f,.48f,.63f,-.62f,-.54f,-.28f,-.08f,.06f,.71f,.64f,.60f,.88f,-.68f,-.91f,-.03f,.79f,.69f,-.51f,-.24f,.95f,-.41f,.01f,.97f,.94f,-.74f,.28f,.55f,-.48f,-.63f,-.85f,-.70f,.56f,.37f,.96f,-.80f,-.08f,.39f,.92f,-.02f,.85f,-.28f,-.14f,.81f,.35f,-.17f,-.39f,.12f,.36f,-.06f,.88f,-.59f,-.25f,-.50f,-.34f,.71f,-.72f,-.75f,.40f,.29f,.29f,.61f,.49f,-.79f,.05f,.03f,.48f,.58f,-.38f,.98f,.60f,-.79f,-.15f,-.24f,.37f,.51f,.95f,-.94f,.30f,-.58f,.62f,.57f,-.86f,.69f,-.18f,-.29f,.33f,.56f,-.37f,.02f,.12f,-.97f,.80f,-.46f,.55f,-.82f,-.60f,-.86f,-.80f,.60f,-.09f,-.51f,-.46f,-.86f,-.46f,-.30f,-.86f,.33f,.31f,-.73f,-.83f,.33f,.35f,-.26f,.58f,.75f,.89f,-.10f,-.55f,-.37f,.51f,.14f,-.37f,.58f,.08f,.54f,.37f,.43f,-.03f,.18f,-.48f,.27f,-.22f,.57f,-.27f,-.89f,.05f,.53f,-.71f,-.12f,-.53f,.57f,-.38f,-.22f,.49f,-.47f,.58f,-.26f,-.15f,.47f,.32f,.97f,-.82f,.70f,.15f,-.08f,.16f,-.76f,.14f,-.75f,-.49f,.68f,-.31f,.02f,.20f,.66f,.16f,-.10f,.13f,-.48f,-.78f,-.81f,-.61f,-.84f,-.23f,-.65f,.18f,.72f,.50f,-.09f,.56f,.97f,-.98f,-.49f,.59f,.37f,.32f,-.61f,.24f,-.76f,-.36f,-.69f,.02f,-.29f,.22f,.70f,.49f,-.99f,-.88f,.28f,-.38f,.17f,.73f,-.86f,-.17f,-.79f,.84f,.47f,.86f,-.23f,-.03f,-.69f,-.38f,-.92f,-.86f,-.46f,-.11f,.32f,.37f,-.29f,-.41f,-.05f,-.11f,.48f,-.16f,-.74f,-.83f,-.07f,-.04f,.14f,.31f,.39f,-.91f,.64f,-.05f,.22f,-.61f,-.70f,-.82f,-.45f,.31f,-.48f,-.31f,.61f,-.31f,.40f,.86f,-.92f,.16f,.02f,.49f,-.60f,-.72f,.71f,.03f,-.67f,.27f,.94f,.06f,.47f,.24f,-.72f,-.41f,.08f,.83f,-.81f,-.61f,-.48f,.08f,-.06f,.35f,-.65f,.91f,-.77f,-.90f,.38f,-.42f,-.48f,.37f,-.71f,.36f,.57f,-.08f,.36f,-.87f,.70f,-.53f,-.00f,.65f,-.73f,.19f,-.09f,.42f,.74f,-.29f,-.31f,-.66f,-.01f,-.40f,.71f,-.36f,.69f,.67f,-.86f,.64f,-.02f,.06f,.49f,.42f,.34f,-.35f,-.58f,-.94f,-.17f,.54f,-.91f,-.31f,.54f,-.19f,.69f,-.39f,.65f,.18f,-.54f,.64f,.43f,-.12f,.44f,-.95f,-.62f,.15f,-.71f,.00f,-.33f,-.30f,-.32f,-.36f,.44f,-.96f,-.65f,.02f,-.21f,.34f,.03f,.10f,-.13f,.58f,.40f,.10f,.75f,-.32f,-.22f,-.48f,.87f,-.03f,-.02f,.82f,.06f,.56f,.41f,.16f,.41f,-.52f,.38f,-.09f,.07f,-.42f,.70f,-.16f,-.13f,.22f,.83f,.56f,-.03f,.20f,-.34f,-.52f,.50f,.73f,.34f,.05f,-.39f,.61f,.67f,-.63f,.27f,-.73f,-.73f,.84f,-.65f,-.51f,-.05f,-.46f,-.38f,-.00f,.38f,-.40f,-.52f,.51f,-.63f,-.40f,-.02f,.39f,.18f,.60f,.30f,.32f,.23f,.77f,.21f,.12f,.49f,.30f,.72f,-.85f,.96f,.23f,-.08f,.57f,.20f,.87f,.99f,.54f,-.76f,.11f,.97f,-.10f,.19f,-.45f,-.12f,.85f,-.21f,.69f,.71f,-.61f,.01f,-.73f,-.61f,.51f,.33f,-.71f,.76f,.67f,.35f,-.51f,-.62f,-.46f,.28f,-.80f,.62f,.65f,-.11f,.87f,.24f,-.97f,.60f,-.03f,-.83f,.32f,-.49f,-.94f,-.42f,-.47f,.50f,.34f,.84f,.11f,-.86f,-.24f,.69f,-.53f,.89f,.00f,.08f,-.81f,-.07f,-.18f,-.52f,-.39f,-.94f,.04f,.89f,-.30f,.44f,-.68f,.49f,.40f,.65f,-.80f,-.72f,.42f,-.49f,.91f,.45f,.26f,.60f,.00f,.20f,.27f,-.00f,.27f,.91f,.44f,.84f,.81f,-.09f,.38f,-.49f,.56f,.04f,.54f,-.79f,.05f,.56f,.34f,-.20f,.12f,.00f,.48f,.18f,.15f,-.86f,.57f,-.03f,-.12f,.56f,-.33f,-.80f,.03f,.00f,.90f,.00f,.50f,.15f,-.09f,-.43f,.17f,-.28f,.21f,-.86f,-.41f,.69f,-.40f,.11f,-.95f,-.08f,-.98f,-.67f,.73f,-.97f,-.67f,-.09f,.52f,-.25f,.26f,.09f,-.66f,-.16f,.01f,.86f,-.11f,-.78f,.15f,-.02f,-.23f,.18f,.91f,.29f,.21f,-.53f,-.65f,.79f,-.68f,.87f,-.22f,-.07f,-.43f,.50f,.26f,.05f,.55f,-.30f,-.16f,-.43f,.56f,-.16f,.03f,-.92f,.32f,-.69f,.05f,-.34f,-.90f,.67f,-.24f,-.24f,.97f,.18f,-.31f,-.45f,-.98f,.94f,.84f,.52f,.80f,.71f,-.26f,.04f,-.78f,.10f,-.21f,-.21f,.42f,-.36f,-.98f,-.29f,-.79f,-.29f,-.89f,-.82f,.17f,-.11f,.59f,-.30f,.43f,.40f,.54f,-.35f,-.60f,-.30f,.61f,.52f,.20f,-.40f,.07f,.81f,.50f,.00f,.26f,.06f,.01f,-.43f,.68f,-.25f,-.28f,.01f,.29f,.87f,.68f,.18f,-.26f,-.16f,-.43f,-.59f,.74f,.91f,.33f,.88f,.07f,-.67f,.26f,-.88f,-.31f,-.25f,.63f,.08f,-.60f,-.08f,-.29f,-.41f,-.12f,-.65f,-.20f,-.73f,.50f,-.50f,-.55f,.80f,.01f,-.47f,-.33f,.52f,-.59f,-.61f,.99f,-.94f,.32f,-.50f,-.41f,.66f,-.06f,-.30f,.79f,.33f,.21f,-.79f,.09f,.66f,-.13f,-.32f,.05f,.32f,.07f,-.11f,-.16f,.72f,.69f,.76f,-.14f,-.96f,-.26f,.93f,.84f,-.00f,.23f,-.68f,.65f,.64f,-.37f,.94f,-.70f,-.04f,-.39f,.90f,-.62f,.02f,.88f,.77f,-.40f,.01f,-.45f,-.92f,-.18f,.79f,.52f,.86f,.57f,-.39f,-.39f,-.47f,.47f,-.70f,-.10f,.27f,.99f,-.62f,-.11f,-.99f,-.68f,.81f,-.94f,.08f,-.01f,.42f,.50f,.05f,.67f,.87f,-.11f,-.55f,-.20f,.12f,-.77f,.50f,-.46f,-.69f,-.76f,-.21f,-.49f,.08f,-.29f,-.44f,.59f,.17f,-.34f,.57f,-.00f,-.21f,.65f,.04f,-.26f,-.73f,.95f,.28f,-.33f,-.95f,-.55f,-.52f,-.93f,.57f,-.56f,-.50f,-.62f,-.73f,.94f,-.37f,.04f,-.09f,-.12f,-.71f,-.38f,-.08f,-.56f,.27f,.45f,-.47f,.84f,.24f,-.76f,-.10f,.96f,.72f,.67f,.04f,.91f,.50f,.29f,-.34f,-.87f,.00f,-.74f,.76f,-.20f,.38f,.54f,-.69f,.54f,-.24f,-.16f,.79f,-.17f,.53f,-.25f,-.54f,.40f,-.78f,-.41f,-.25f,.85f,.70f,-.77f,-.23f,-.96f,.75f,.52f,.59f,-.85f,.70f,-.23f,-.71f,-.22f,.87f,.44f,.57f,.75f,.44f,-.44f,.45f,.41f,-.90f,-.86f,.51f,.56f,-.41f,-.55f,.53f,.78f,-.71f,-.43f,-.20f,.98f,-.39f,-.42f,-.52f,.40f,-.67f,-.32f,.12f,.95f,.99f,-.75f,.97f,-.33f,-.24f,.31f,-.50f,-.54f,-.20f,.92f,-.81f,.38f,-.62f,-.39f,.82f,.91f,-.75f,.03f,-.06f,-.61f,-.17f,-.91f,.27f,.30f,-.75f,-.51f,.50f,.60f,-.96f,-.91f,-.52f,.34f,.92f,.31f,-.02f,.89f,.93f,-.77f,.53f,.98f,.01f,.45f,.56f,-.41f,.38f,-.96f,.72f,-.11f,-.05f,.88f,-.78f,-.27f,-.68f,-.41f,-.16f,-.08f,.70f,.16f,.13f,-.29f,.53f,-.52f,-.75f,.99f,.73f,.11f,.11f,-.05f,.27f,-.87f,-.19f,-.59f,-.73f,.74f,-.18f,.13f,-.32f,.55f,.03f,-.20f,.45f,-.06f,-.10f,-.86f,.94f,-.63f,.88f,.23f,.28f,-.62f,-.59f,.76f,.41f,-.29f,.55f,.01f,.36f,.33f,.06f,.67f,-.63f,-.36f,.77f,-.72f,.76f,-.87f,.95f,-.28f,-.83f,-.67f,-.64f,-.49f,.55f,.06f,.90f,-.01f,-.58f,.94f,-.76f,-.24f,.58f,-.99f,.17f,-.14f,.65f,-.80f,.77f,-.18f,-.37f,.07f,-.71f,-.31f,.43f,.59f,-.52f,-.73f,-.17f,.04f,-.25f,-.62f,.90f,.40f,-.60f,-.31f,.52f,-.60f,.90f,.78f,.13f,-.27f,-.71f,.71f,.24f,.93f,-.39f,.33f,-.02f,.45f,.61f,-.50f,.81f,.95f,-.70f,.72f,.03f,-.16f,.07f,-.28f,-.83f,.84f,-.48f,-.42f,.62f,.62f,.39f,.88f,.03f,-.41f,-.49f,-.80f,-.60f,.55f,.83f,-.02f,-.69f,-.19f,.65f,.49f,-.32f,-.28f,.05f,-.40f,-.03f,-.39f,.48f,-.75f,-.02f,-.91f,-.25f,.33f,.80f,-.30f,-.14f,-.36f,.61f,.82f,.48f,.31f,-.87f,.45f,.06f,.15f,.08f,-.39f,.10f,-.21f,.63f,-.47f,.55f,.47f,.90f,.32f,-.29f,.79f,-.76f,.75f,-.90f,.56f,.92f,-.04f,-.73f,-.75f,-.68f,-.74f,.99f,-.68f,-.49f,.31f,.76f,-.76f,-.03f,-.95f,-.75f,-.89f,-.32f,.91f,-.34f,.58f,-.43f,.33f,-.88f,-.32f,.90f,.23f,-.40f,-.86f,.81f,.26f,-.90f,.78f,-.61f,.65f,-.75f,.69f,-.29f,-.17f,-.92f,.84f,.80f,-.35f,.53f,.73f,-.60f,-.75f,-.46f,-.68f,-.13f,.80f,-.21f,.38f,.89f,-.53f,.34f,-.32f,.71f,.08f,-.43f,.27f,-.17f,.48f,.75f,-.89f,-.47f,-.94f,-.23f,.52f,-.47f,.34f,-.84f,-.81f,-.14f,.36f,-.73f,-.35f,-.06f,-.66f,-.16f,-.84f,-.95f,-.52f,-.02f,-.28f,.85f,.91f,-.61f,-.16f,-.82f,.10f,.19f,-.51f,.04f,-.23f,.74f,-.18f,.31f,-.63f,-.37f,-.27f,-.95f,-.36f,-.32f,-.85f,.52f,.16f,.75f,-.68f,-.99f,-.18f,-.82f,.90f,.61f,-.96f,.55f,.61f,-.82f,.15f,.21f,-.46f,.06f,.35f,.95f,.25f,-.10f,-.08f,.58f,.14f,-.54f,.21f,.48f,.53f,.09f,.75f,.82f,-.12f,-.45f,.65f,.29f,-.04f,.43f,.84f,-.97f,.04f,-.14f,-.02f,.09f,-.46f,.01f,-.90f,-.54f,-.71f,-.03f,-.45f,-.02f,-.49f,-.71f,.06f,-.48f,.90f,.83f,.20f,.42f,-.99f,-.11f,.53f,-.35f,-.01f,.67f,-.68f,-.08f,-.52f,.44f,-.61f,-.58f,-.15f,-.47f,.90f,.37f,-.05f,-.52f,.54f,.88f,.19f,-.36f,.19f,.82f,.04f,-.37f,.36f,.97f,-.93f,.97f,-.82f,.28f,.17f,-.19f,.17f,-.24f,-.69f,.48f,-.01f,.43f,-.70f,.98f,-.49f,-.51f,.97f,.05f,-.87f,.57f,-.93f,-.45f,.78f,-.22f,.70f,.91f,-.54f,.67f,-.53f,-.01f,.38f,.99f,-.56f,.85f,.67f,.88f,-.02f,.33f,-.00f,.74f,.33f,-.37f,-.92f,.79f,-.93f,.24f,.18f,-.56f,-.83f,-.22f,.79f,-.04f,.54f,-.11f,-.40f,.27f,.44f,.50f,.24f,.96f,-.02f,.24f,.18f,-.95f,-.03f,.93f,-.15f,.86f,-.24f,-.30f,.80f,-.05f,.91f,.55f,.05f,.44f,-.43f,.00f,-.96f,-.55f,.52f,-.11f,-.56f,-.08f,.91f,.05f,-.70f,.13f,.43f,.07f,.81f,-.49f,.14f,-.89f,-.38f,.19f,.57f,.94f,.66f,.98f,.75f,.37f,-.12f,.12f,.96f,.40f,-.10f,-.64f,-.88f,-.64f,.95f,-.81f,.61f,-.97f,-.68f,-.73f,-.00f,-.69f,-.99f,.73f,-.09f,-.49f,.22f,.04f,.77f,.79f,-.06f,-.35f,.06f,.81f,-.03f,-.05f,.47f,.27f,-.74f,-.59f,.82f,.47f,.80f,.80f,.58f,-.87f,.05f,.50f,-.26f,-.50f,-.84f,.12f,.62f,.66f,-.77f,-.10f,.74f,.66f,-.07f,.73f,.74f,.52f,.10f,-.78f,.24f,.92f,.02f,-.12f,.80f,.60f,.76f,.96f,.57f,.94f,-.37f,-.01f,-.20f,-.95f,-.37f,-.61f,.74f,.82f,.31f,.32f,-.47f,-.91f,-.60f,-.78f,.19f,-.65f,.44f,-.63f,-.19f,-.62f,.39f,-.58f,-.80f,-.47f,.29f,-.31f,.19f,.13f,.59f,-.52f,.32f,.71f,-.69f,-.21f,-.94f,-.80f,-.82f,-.81f,-.92f,-.08f,-.68f,.19f,.31f,-.02f,-.60f,.97f,-.71f,.21f,-.18f,.19f,.83f,.79f,-.18f,-.91f,-.77f,-.80f,-.00f,.08f,.30f,-.86f,-.88f,-.01f,-.88f,.71f,-.80f,-.54f,.43f,-.68f,-.56f,.32f,.05f,.21f,-.51f,.17f,-.42f,.01f,.00f,.03f,.16f,-.77f,.62f,-.91f,-.70f,-.42f,-.49f,.46f,-.82f,.97f,-.49f,-.49f,.42f,.23f,-.19f,.23f,.99f,-.57f,-.93f,-.46f,.05f,-.86f,.23f,.94f,-.68f,.73f,.08f,.33f,.63f,.65f,.11f,.41f,-.16f,-.05f,.80f,.65f,.07f,-.76f,-.69f,.34f,.18f,.69f,.58f,.67f,.56f,.02f,-.72f,.07f,.38f,-.30f,.59f,.00f,-.67f,.52f,.40f,.60f,-.26f,-.16f,-.99f,-.89f,.48f,.03f,-.59f,-.62f,-.51f,.10f,.79f,.47f,.66f,.23f,-.51f,-.08f,-.84f,-.38f,.27f,.11f,-.46f,.50f,-.75f,-.49f,-.44f,-.86f,-.46f,.70f,-.21f,-.33f,-.69f,.91f,.79f,.29f,-.90f,.70f,-.92f,-.49f,.85f,-.58f,-.71f,.53f,.88f,.29f,.35f,.75f,-.23f,.90f,-.49f,.04f,.07f,.13f,-.12f,-.84f,-.09f,-.19f,-.26f,-.73f,-.66f,-.83f,-.14f,.38f,-.95f,.39f,-.69f,.46f,.44f,.53f,-.38f,.01f,-.41f,.10f,.90f,-.36f,.58f,.41f,.11f,-.73f,.14f,.97f,.89f,-.24f,.49f,.65f,-.52f,.63f,-.82f,.34f,.95f,-.90f,-.19f,-.48f,-.05f,-.92f,-.48f,-.09f,-.72f,.64f,-.92f,.64f,-.85f,-.78f,-.44f,-.67f,-.48f,.91f,-.45f,.23f,-.35f,-.10f,.49f,.65f,.29f,.03f,.59f,-.92f,.24f,.48f,.86f,.29f,.77f,.40f,-.56f,-.27f,-.99f,.95f,.99f,-.65f,.71f,.21f,-.14f,.51f,.25f,-.31f,.73f,.37f,.62f,.58f,-.81f,-.68f,.41f,-.88f,-.20f,.40f,-.71f,-.71f,.97f,-.28f,-.73f,.36f,-.57f,-.56f,-.33f,.98f,-.32f,.25f,-.85f,-.16f,-.14f,-.17f,-.57f,.12f,.03f,-.77f,.12f,-.58f,-.90f,.51f,-.29f,-.47f,.89f,-.81f,.69f,.70f,-.40f,.15f,-.66f,.54f,-.75f,-.81f,-.87f,-.26f,.31f,-.56f,-.15f,.04f,-.86f,.07f,.19f,-.80f,-.94f,.04f,-.01f,.87f,.83f,-.44f,-.77f,-.73f,.24f,-.87f,-.61f,.85f,.40f,-.83f,.18f,-.88f,.49f,-.27f,-.25f,-.59f,-.47f,-.91f,-.81f,-.60f,-.73f,.78f,-.81f,.70f,.20f,.44f,-.60f,-.82f,-.56f,.90f,.08f,-.00f,.06f,.17f,-.42f,.34f,.80f,-.92f,.02f,.44f,-.54f,.45f,.69f,-.19f,-.06f,.32f,-.38f,.72f,.96f,.47f,-.56f,-.50f,.09f,-.60f,-.92f,.10f,-.34f,.16f,.74f,.03f,-.54f,.63f,.60f,.69f,.20f,-.43f,.31f,-.68f,-.64f,-.46f,.61f,-.74f,.56f,.11f,-.44f,-.02f,-.38f,-.96f,.95f,.38f,.24f,-.74f,.91f,-.54f,-.61f,.67f,.38f,.91f,.05f,-.53f,-.08f,.04f,-.86f,-.90f,-.92f,.08f,-.49f,.22f,.92f,.76f,-.30f,.85f,-.76f,.15f,.28f,-.52f,-.16f,.84f,.48f,.70f,.62f,-.52f,.37f,-.43f,.52f,-.87f,.91f,.13f,.15f,.10f,.62f,.72f,-.69f,-.53f,-.09f,-.37f,.76f,-.76f,-.50f,.86f,-.17f,-.73f,-.39f,-.05f,.91f,-.26f,.39f,-.69f,.26f,.97f,-.23f,.61f,.77f,-.28f,.17f,.32f,.85f,-.80f,-.85f,-.03f,-.59f,-.91f,.28f,-.14f,.46f,-.53f,-.75f,-.84f,.52f,-.35f,-.97f,.94f,-.44f,-.26f,.54f,-.51f,.41f,.41f,.89f,-.54f,-.27f,.72f,-.34f,.01f,.19f,.23f,-.30f,-.61f,-.39f,-.11f,.45f,-.22f,-.66f,.06f,.15f,.46f,-.00f,.91f,.25f,.37f,-.13f,.26f,.08f,.78f,-.77f,-.68f,.21f,-.80f,.02f,.33f,.65f,.33f,.02f,-.97f,.01f,-.47f,-.48f,.46f,-.70f,.47f,.01f,-.98f,.49f,-.12f,-.62f,-.45f,.57f,-.05f,.91f,.29f,-.85f,-.14f,-.51f,-.31f,.48f,.81f,-.11f,.10f,.72f,.27f,-.97f,.24f,-.18f,-.09f,-.77f,-.70f,.98f,.50f,.58f,.10f,-.19f,.12f,.05f,.77f,.17f,-.38f,-.17f,.15f,-.18f,.45f,.10f,.86f,.23f,-.74f,-.18f,.41f,.30f,-.75f,-.25f,-.29f,.86f,-.21f,.34f,-.62f,.89f,-.04f,-.34f,-.04f,.38f,-.59f,.02f,.52f,.84f,.14f,.48f,-.89f,-.86f,.12f,-.07f,-.21f,-.94f,-.36f,-.49f,.26f,.70f,.45f,.16f,-.98f,-.85f,-.38f,-.01f,-.31f,-.23f,-.16f,.93f,-.68f,.12f,-.92f,-.55f,-.15f,-.68f,-.59f,-.57f,.79f,-.21f,.17f,-.14f,-.30f,-.35f,.07f,-.15f,-.25f,-.04f,.01f,.28f,-.54f,.85f,-.59f,-.69f,-.18f,.42f,-.93f,-.02f,-.69f,.94f,.11f,-.33f,.34f,.25f,-.77f,-.82f,.33f,-.29f,-.68f,-.52f,-.49f,.57f,.47f,-.60f,-.45f,.69f,.82f,-.15f,-.83f,-.07f,-.79f,-.63f,-.42f,-.62f,-.25f,.78f,.69f,.32f,.36f,.73f,.52f,-.74f,-.11f,.04f,-.96f,-.50f,-.47f,-.88f,-.26f,.67f,-.59f,-.33f,.78f,-.59f,.21f,.13f,-.31f,-.19f,.15f,.26f,-.40f,.43f,-.96f,.75f,-.61f,-.04f,.66f,-.10f,.65f,.12f,-.20f,.98f,.99f,-.75f,.51f,.93f,-.27f,.02f,-.85f,-.62f,-.35f,-.04f,.96f,.85f,.89f,-.38f,-.68f,.02f,-.88f,-.67f,.00f,.92f,-.88f,.56f,.46f,-.72f,.66f,.80f,.48f,.19f,.27f,.15f,-.77f,-.45f,-.94f,.11f,.37f,.22f,-.54f,-.27f,-.67f,-.76f,-.15f,-.20f,.93f,-.05f,.27f,.20f,.72f,-.40f,-.72f,-.49f,.81f,.06f,.83f,.09f,.08f,-.65f,-.26f,.74f,-.22f,-.96f,-.62f,-.45f,.60f,-.10f,.17f,.12f,-.17f,-.74f,.08f,.12f,-.64f,.68f,.92f,.00f,-.04f,.31f,-.83f,-.47f,-.55f,-.89f,-.42f,-.96f,-.62f,.19f,-.47f,.19f,-.46f,-.70f,.39f,-.10f,-.82f,.19f,-.06f,-.47f,-.03f,-.46f,.22f,-.76f,.34f,-.80f,.31f,.41f,.17f,.90f,.47f,.85f,-.94f,.31f,-.81f,.24f,.42f,.20f,.15f,-.24f,-.69f,-.87f,-.65f,.09f,-.25f,.36f,-.38f,.04f,-.80f,.54f,.82f,-.95f,.86f,.40f,.70f,-.34f,-.58f,-.75f,-.37f,.39f,-.62f,.05f,-.76f,-.91f,.65f,.38f,.76f,.36f,-.73f,.06f,-.01f,-.05f,.41f,-.66f,.75f,.56f,.55f,-.09f,.82f,.49f,-.35f,.91f,-.54f,.63f,.00f,-.34f,-.30f,-.08f,.90f,.76f,.41f,-.48f,.95f,.61f,.28f,.41f,-.86f,-.79f,.25f,-.48f,.35f,.22f,-.38f,.33f,.81f,-.85f,-.60f,.87f,.77f,.13f,.90f,.46f,.74f,-.66f,.31f,.00f,.87f,.60f,-.04f,.30f,.61f,.93f,.59f,.19f,.26f,-.93f,.24f,.64f,-.72f,.62f,.31f,-.11f,-.24f,.00f,.79f,.98f,-.43f,-.60f,-.10f,.31f,.38f,-.21f,.81f,-.96f,-.87f,-.66f,.92f,.21f,.53f,.22f,-.25f,-.20f,.02f,.93f,.39f,.38f,.71f,-.38f,.73f,.58f,-.81f,.78f,-.92f,-.26f,.39f,.83f,.85f,.90f,-.26f,.12f,-.40f,-.17f,.54f,.85f,-.38f,-.68f,-.77f,.90f,.08f,.26f,.93f,.52f,-.05f,-.19f,-.64f,.96f,.92f,-.90f,.77f,.05f,-.27f,.76f,.63f,-.15f,.14f,.24f,-.37f,-.39f,.62f,-.68f,-.46f,-.21f,-.72f,.67f,-.75f,-.48f,-.85f,-.06f,-.37f,.07f,.63f,.94f,-.28f,-.88f,-.16f,-.20f,-.23f,.45f,-.02f,-.78f,.76f,-.63f,.09f,-.35f,.91f,.41f,-.89f,.67f,.57f,-.95f,.41f,.62f,-.61f,.17f,-.24f,.34f,.11f,-.25f,-.29f,-.22f,.99f,.73f,-.54f,.39f,-.74f,.32f,.39f,-.40f,.18f,.94f,.94f,.47f,.27f,-.61f,-.62f,.78f,.77f,-.91f,.41f,.67f,-.17f,-.16f,.86f,.50f,.71f,.68f,.54f,.68f,.27f,.35f,-.84f,-.04f,-.06f,.14f,.64f,-.61f,-.51f,-.17f,-.22f,-.55f,.68f,.52f,-.38f,-.18f,.21f,.34f,-.29f,-.85f,-.88f,.92f,.44f,-.02f,.59f,-.06f,-.62f,-.11f,-.72f,-.88f,-.91f,-.59f,-.77f,.67f,.19f,.34f,.05f,.77f,.09f,-.08f,-.60f,.03f,.02f,.00f,.98f,.75f,.96f,.05f,-.09f,.25f,-.53f,.26f,.00f,.26f,.77f,.74f,.38f,.27f,.49f,-.85f,.67f,.59f,.45f,.70f,-.21f,.78f,.74f,-.83f,.85f,-.38f,.78f,-.82f,-.70f,-.36f,.21f,.32f,-.66f,-.33f,.50f,.26f,.59f,-.13f,.86f,-.01f,.52f,-.65f,-.02f,-.82f,.04f,.64f,.62f,.94f,.70f,-.55f,-.03f,.38f,-.45f,.70f,-.17f,.76f,.35f,-.91f,-.85f,.72f,-.18f,-.09f,.54f,-.54f,-.19f,-.97f,-.98f,-.60f,.38f,-.36f,.58f,.56f,.29f,.16f,.27f,-.91f,.53f,-.06f,-.42f,.05f,-.05f,.25f,-.46f,.92f,.37f,-.87f,.14f,.98f,-.33f,.07f,.98f,.16f,-.19f,-.22f,-.45f,.61f,-.77f,.59f,.32f,.34f,.85f,.98f,-.56f,-.46f,-.83f,.64f,-.78f,.33f,.38f,.49f,.90f,-.73f,.50f,.51f,-.25f,-.80f,.17f,-.88f,-.30f,.57f,-.48f,-.83f,-.05f,-.27f,-.28f,.80f,-.26f,-.17f,-.61f,.25f,-.47f,-.65f,-.29f,-.43f,-.88f,-.02f,.97f,.04f,.94f,-.47f,.46f,.19f,-.00f,.70f,.36f,.93f,-.47f,-.85f,-.44f,-.26f,-.44f,.41f,-.50f,-.77f,-.98f,.30f,-.75f,-.53f,-.77f,-.89f,.92f,-.35f,.22f,-.14f,-.50f,.41f,.78f,-.89f,.59f,-.62f,-.82f,.50f,.98f,.85f,.84f,.63f,.40f,.04f,.15f,-.70f,.95f,.10f,.25f,-.02f,-.70f,.88f,-.56f,.56f,-.58f,.52f,-.11f,.02f,-.35f,-.62f,-.15f,-.06f,.15f,.45f,-.99f,.57f,-.19f,.11f,.84f,.02f,-.05f,-.53f,.74f,-.85f,-.07f,.33f,.45f,-.45f,-.74f,.07f,.38f,.60f,.36f,.57f,-.27f,-.54f,-.01f,.68f,-.97f,.67f,-.06f,.67f,.01f,.03f,.65f,.18f,-.81f,.23f,.06f,.45f,.42f,-.17f,.76f,-.27f,-.29f,-.17f,.02f,-.42f,-.46f,-.93f,-.17f,-.78f,-.43f,-.04f,-.69f,.29f,.35f,-.91f,-.97f,.09f,.63f,-.20f,.15f,-.63f,-.96f,-.95f,.60f,.91f,.09f,-.70f,.50f,-.07f,.44f,.01f,.75f,.73f,-.73f,.16f,-.80f,.52f,.64f,-.02f,-.17f,.04f,-.50f,.72f,-.06f,.80f,-.50f,-.78f,-.37f,-.16f,-.06f,-.73f,-.68f,-.45f,.43f,-.41f,-.01f,-.75f,.41f,-.94f,.86f,-.33f,.65f,-.82f,-.53f,-.19f,-.21f,.75f,.39f,-.52f,-.82f,.05f,.29f,-.84f,-.03f,-.05f,-.92f,.66f,.14f,.19f,-.98f,.09f,.48f,.96f,.39f,.67f,-.49f,.42f,.30f,.84f,-.46f,-.41f,.92f,-.93f,.51f,-.90f,-.54f,-.32f,.18f,-.83f,-.05f,.78f,-.76f,.76f,-.50f,-.94f,-.58f,.52f,.45f,-.17f,.62f,.48f,.85f,.11f,.74f,.99f,.94f,-.77f,.44f,-.89f,.97f,-.77f,.16f,-.35f,.16f,-.45f,.50f,-.96f,.74f,-.52f,-.64f,.13f,-.21f,-.03f,-.16f,.35f,.53f,-.82f,.34f,.52f,-.59f,-.17f,-.53f,-.88f,.67f,.13f,-.21f,.93f,-.45f,-.11f,-.60f,.84f,.16f,.76f,.01f,.89f,.87f,.04f,.86f,.20f,.20f,.61f,-.21f,.04f,.26f,.91f,-.40f,.00f,-.26f,-.12f,.35f,.21f,-.87f,.81f,-.03f,-.82f,.19f,-.71f,.93f,.39f,-.70f,-.97f,.20f,.92f,.13f,-.81f,.88f,.34f,-.27f,-.46f,.78f,.94f,-.50f,.57f,.06f,-.52f,.14f,.33f,-.34f,-.31f,-.05f,-.73f,.13f,-.63f,.88f,-.44f,-.15f,.11f,.90f,-.36f,.51f,-.38f,-.64f,-.21f,.13f,.89f,.81f,-.69f,.15f,-.67f,-.89f,-.47f,.73f,-.15f,-.97f,-.24f,-.57f,.91f,-.64f,-.30f,.67f,-.55f,-.68f,.98f,-.98f,.96f,-.36f,-.55f,.11f,-.73f,.57f,.73f,-.46f,-.11f,.52f,.48f,.06f,.43f,-.41f,-.43f,.58f,-.69f,.57f,.71f,-.73f,.19f,-.45f,-.24f,.96f,.61f,.66f,-.50f,-.46f,.32f,.43f,.15f,-.93f,.77f,.60f,-.27f,-.37f,.31f,-.69f,.20f,-.22f,.90f,.51f,-.56f,-.50f,-.55f,.64f,-.03f,.70f,.90f,-.83f,-.11f,-.70f,.82f,.62f,.66f,.26f,.24f,-.55f,.72f,.42f,.58f,.60f,-.02f,-.29f,.73f,-.04f,-.45f,.01f,-.04f,-.59f,.86f,-.29f,.82f,.56f,-.76f,-.58f,.77f,.96f,-.28f,.71f,.19f,-.26f,.98f,.32f,-.75f,-.24f,.85f,-.26f,-.73f,.49f,.26f,-.74f,.08f,-.53f,-.58f,.52f,.41f,.06f,.20f,.47f,-.41f,.14f,.00f,.94f,.76f,-.41f,-.90f,.92f,-.29f,.52f,-.94f,.37f,.87f,-.06f,-.44f,-.33f,-.04f,.78f,-.55f,-.23f,.25f,.08f,.41f,-.48f,.05f,-.37f,.39f,-.74f,-.89f,.51f,-.80f,-.70f,-.94f,.32f,-.43f,-.75f,.63f,-.33f,-.67f,-.33f,.18f,.65f,.34f,.92f,.08f,.92f,.04f,-.33f,-.59f,.62f,.85f,-.49f,.39f,-.37f,-.04f,-.11f,.23f,-.41f,-.84f,.71f,-.35f,.88f,.70f,.94f,.19f,.85f,-.35f,.84f,-.99f,-.70f,.57f,-.78f,-.96f,-.55f,.79f,.67f,.09f,-.28f,-.11f,.22f,.92f,.79f,-.70f,-.62f,.56f,-.70f,-.15f,.18f,.65f,.97f,-.87f,-.04f,-.75f,-.65f,.12f,-.09f,.85f,.98f,.58f,.09f,-.06f,.70f,.19f,-.89f,-.53f,-.26f,-.79f,-.07f,.19f,-.68f,.72f,.87f,-.10f,.68f,-.12f,-.44f,.26f,-.30f,.36f,-.98f,-.76f,-.93f,.93f,-.68f,-.39f,-.06f,.87f,-.19f,.60f,.96f,.91f,.05f,-.95f,.13f,.35f,.87f,.67f,-.48f,.26f,.19f,.74f,-.53f,-.54f,.60f,.08f,.50f,.18f,.89f,.89f,-.19f,.10f,.72f,-.13f,-.38f,-.45f,.19f,.33f,-.01f,-.34f,-.28f,-.42f,-.39f,.48f,.11f,-.66f,.22f,-.08f,.71f,.86f,.02f,-.02f,.39f,-.86f,-.51f,.50f,-.42f,.12f,.44f,-.47f,-.43f,.11f,-.92f,-.07f,.83f,-.69f,-.28f,-.13f,.05f,-.49f,.07f,-.34f,.18f,-.42f,.57f,.44f,.57f,.45f,.97f,-.57f,.30f,-.87f,-.90f,-.11f,-.14f,.89f,.74f,.35f,.30f,-.16f,-.17f,.23f,-.63f,-.95f,.79f,-.39f,.38f,.34f,-.69f,-.45f,-.34f,.96f,-.02f,-.77f,.15f,.37f,-.61f,-.83f,.48f,-.57f,.52f,-.71f,-.02f,-.83f,.34f,.29f,-.05f,.73f,-.15f,.89f,-.17f,-.81f,.59f,.11f,.92f,.31f,.71f,-.89f,-.70f,.31f,.75f,-.59f,-.89f,-.81f,-.81f,.23f,.12f,.26f,.56f,.01f,.24f,-.26f,.61f,-.58f,.57f,-.95f,-.46f,.78f,.06f,-.32f,.88f,.84f,-.25f,-.25f,-.32f,.48f,.28f,-.14f,.48f,.29f,.67f,.89f,.97f,.54f,-.46f,.19f,.67f,-.27f,.19f,-.64f,.68f,.30f,.24f,-.61f,-.28f,-.26f,.32f,-.30f,-.72f,.01f,.19f,-.35f,-.56f,-.02f,.17f,.45f,.07f,-.07f,-.75f,.21f,-.19f,.21f,.89f,-.13f,.51f,.70f,-.00f,-.92f,-.64f,.26f,-.13f,.69f,.28f,-.47f,-.78f,-.32f,.01f,.15f,.36f,-.28f,-.35f,.31f,-.47f,.53f,.51f,.23f,.35f,.97f,.23f,-.09f,.81f,-.26f,.08f,-.17f,-.41f,.78f,-.51f,.16f,-.27f,.05f,-.30f,.32f,.51f,.19f,.94f,.60f,.10f,.21f,.49f,-.69f,.23f,-.87f,.64f,-.36f,-.53f,-.63f,-.05f,-.95f,.23f,.85f,-.32f,-.11f,-.89f,-.13f,.31f,.28f,-.69f,-.37f,-.47f,-.79f,.78f,-.90f,.91f,-.86f,.05f,-.86f,-.74f,.19f,.68f,-.56f,.04f,.10f,-.35f,.47f,-.79f,-.28f,.46f,-.11f,.60f,.72f,-.47f,.03f,.63f,.75f,-.84f,.40f,-.25f,-.72f,.63f,-.22f,-.96f,.43f,-.61f,.73f,-.58f,.78f,-.11f,-.30f,-.59f,-.11f,.01f,.44f,.35f,.42f,-.84f,.13f,.63f,.37f,-.45f,.86f,.90f,.08f,-.50f,.89f,.54f,.50f,-.17f,-.89f,-.72f,.60f,-.75f,.75f,-.79f,.60f,.56f,-.67f,.75f,-.70f,-.15f,-.09f,-.27f,.44f,.74f,-.97f,-.72f,.47f,.83f,.60f,-.97f,.32f,.24f,.33f,.65f,.74f,-.10f,.19f,.30f,-.94f,.07f,-.04f,-.47f,.56f,-.94f,.01f,-.65f,-.18f,.41f,-.34f,-.42f,.56f,.40f,-.17f,-.98f,.62f,.27f,-.27f,.25f,.26f,.41f,-.17f,.18f,-.67f,-.17f,-.78f,.52f,.83f,.04f,-.14f,.35f,.73f,.14f,.72f,-.26f,.83f,-.60f,-.61f,.89f,.30f,.07f,.82f,-.79f,-.68f,-.42f,-.40f,-.99f,-.86f,.48f,.44f,.50f,.51f,.16f,-.77f,-.15f,-.70f,.64f,.97f,-.71f,-.54f,.23f,-.23f,.83f,.53f,-.58f,.62f,-.32f,.84f,-.66f,.95f,.76f,.92f,.48f,-.81f,.43f,.08f,-.67f,.46f,.30f,.23f,.01f,.55f,.02f,.81f,.56f,-.34f,-.69f,.05f,-.99f,.96f,-.84f,.34f,.88f,.01f,.44f,-.19f,.28f,.39f,-.70f,-.24f,-.83f,.44f,.09f,-.08f,.42f,.09f,.24f,-.86f,-.21f,-.90f,-.87f,-.91f,.29f,-.61f,-.84f,.36f,-.40f,-.45f,.46f,.05f,-.55f,.24f,.99f,-.20f,.18f,-.16f,.45f,.90f,.38f,-.37f,.24f,.90f,-.55f,.09f,-.53f,-.14f,.66f,-.59f,-.70f,.24f,.17f,.24f,-.65f,.76f,.32f,.49f,-.65f,.55f,-.59f,-.31f,.78f,-.67f,-.29f,-.60f,.31f,.26f,-.02f,.87f,-.31f,.95f,.14f,.76f,-.89f,.26f,.74f,-.41f,.10f,-.93f,-.42f,-.60f,-.96f,-.31f,.90f,-.20f,.68f,-.57f,.86f,-.71f,.68f,.26f,.75f,.79f,.09f,.94f,.20f,.11f,-.27f,-.06f,.23f,.43f,-.44f,-.87f,.56f,-.34f,.37f,-.49f,-.47f,.10f,.53f,.80f,.53f,-.44f,-.92f,.79f,-.89f,.50f,.82f,-.09f,-.14f,-.88f,.16f,-.15f,.81f,-.96f,.99f,.30f,.09f,-.84f,-.61f,.41f,-.68f,.54f,-.86f,-.87f,-.55f,.69f,.67f,-.43f,.09f,.47f,.85f,-.36f,.27f,-.10f,.93f,.34f,-.77f,.45f,-.37f,.71f,.82f,-.98f,.42f,-.07f,.50f,.66f,.62f,.13f,-.53f,-.83f,-.38f,-.03f,-.49f,-.10f,.21f,.89f,-.90f,.55f,-.62f,.42f,.45f,-.17f,-.04f,-.22f,-.20f,-.10f,.42f,.97f,-.08f,-.58f,-.25f,.02f,-.08f,-.38f,.89f,-.65f,.46f,-.35f,-.54f,.92f,.03f,.58f,-.00f,-.40f,-.55f,.99f,-.46f,-.17f,.17f,-.05f,-.28f,.57f,-.15f,-.41f,.24f,.64f,.55f,-.43f,-.52f,.79f,.59f,.23f,-.41f,-.87f,.82f,-.26f,-.10f,-.14f,-.93f,-.03f,-.51f,.17f,-.53f,-.27f,-.64f,.28f,-.98f,.03f,-.06f,.22f,.65f,.93f,.18f,-.60f,.74f,.13f,.79f,-.20f,-.12f,.08f,-.93f,-.82f,.61f,-.68f,.17f,-.66f,.23f,-.62f,-.23f,.65f,.42f,.29f,-.46f,.97f,-.42f,.85f,-.18f,-.68f,.13f,-.50f,.83f,.59f,-.44f,-.14f,.74f,.46f,-.27f,.38f,-.06f,-.34f,-.52f,.66f,.95f,.69f,-.09f,-.37f,-.35f,.57f,.19f,-.65f,-.70f,.43f,-.01f,.24f,.83f,.28f,.37f,-.18f,-.50f,.10f,.80f,.04f,.11f,.02f,-.44f,.30f,.91f,-.42f,.04f,-.48f,-.25f,-.79f,-.20f,.33f,.22f,-.55f,-.73f,.25f,.53f,.57f,.40f,.53f,-.69f,.86f,.78f,.90f,.79f,-.52f,-.90f,-.16f,.88f,.40f,.75f,.53f,-.91f,-.39f,-.79f,-.26f,.77f,-.56f,-.82f,-.17f,.94f,-.86f,.57f,-.61f,-.86f,.11f,.90f,-.50f,-.38f,.06f,.46f,.60f,-.81f,-.81f,.64f,.13f,.41f,-.12f,-.44f,-.66f,-.86f,-.08f,.16f,.18f,.45f,.84f,-.18f,-.09f,.78f,.67f,.86f,.08f,-.06f,.77f,-.88f,.45f,-.77f,-.06f,-.48f,.81f,-.48f,-.01f,.75f,.43f,-.63f,.65f,-.52f,-.72f,-.56f,-.05f,-.63f,.08f,-.34f,-.72f,-.70f,.84f,-.84f,-.64f,-.80f,.40f,.19f,.07f,.49f,.62f,.17f,-.79f,.62f,-.59f,-.82f,-.40f,-.07f,-.35f,.42f,-.23f,-.92f,.70f,.12f,.93f,.58f,.94f,-.67f,-.13f,-.96f,-.04f,-.57f,.67f,.36f,-.03f,.51f,-.87f,.58f,.85f,.54f,-.10f,-.98f,.64f,-.33f,.70f,.94f,-.53f,-.44f,-.39f,.17f,.34f,.22f,.50f,.63f,-.61f,.15f,-.82f,-.20f,-.79f,-.24f,.58f,-.48f,-.76f,.39f,-.85f,-.52f,-.90f,.00f,.06f,.40f,-.89f,.17f,-.30f,-.79f,-.86f,.44f,.55f,.49f,.54f,.12f,.72f,.17f,.65f,-.74f,-.33f,-.82f,.30f,.24f,-.39f,.47f,.79f,-.31f,.97f,-.17f,.57f,-.63f,.23f,.43f,-.55f,.61f,-.88f,.51f,-.93f,.26f,.34f,-.32f,.22f,-.38f,-.93f,.76f,-.90f,-.02f,.08f,-.07f,-.71f,.61f,-.80f,.81f,-.26f,-.82f,-.18f,.98f,.57f,.17f,-.93f,.67f,-.80f,-.07f,-.43f,.66f,.17f,-.26f,.35f,.15f,.83f,.74f,-.88f,.01f,.81f,-.90f,-.75f,.59f,.00f,-.42f,-.11f,-.54f,-.99f,.49f,.46f,.45f,.27f,.57f,-.88f,.54f,-.59f,.66f,-.71f,-.79f,-.51f,.73f,.53f,-.00f,.52f,-.84f,-.39f,.58f,.17f,-.29f,-.36f,-.57f,-.25f,.36f,.58f,-.06f,-.71f,-.35f,-.87f,-.38f,-.60f,-.75f,.30f,.97f,.55f,.31f,.61f,.59f,-.24f,-.13f,-.91f,-.07f,-.17f,-.82f,-.40f,.68f,.78f,.96f,-.70f,.19f,-.98f,.83f,.77f,-.17f,.26f,-.98f,.62f,-.43f,.74f,.50f,-.36f,-.78f,-.34f,.42f,.62f,.43f,-.17f,.83f,.25f,.51f,.07f,-.40f,.05f,-.69f,.44f,-.88f,.12f,.53f,.24f,.16f,.08f,-.97f,.00f,-.58f,.19f,.59f,.32f,-.46f,.65f,-.27f,-.30f,.63f,-.85f,-.99f,.76f,-.62f,.90f,.68f,-.98f,.40f,.15f,-.34f,-.03f,-.64f,-.03f,.99f,.14f,-.48f,-.91f,-.29f,.43f,-.40f,.36f,.15f,.73f,.00f,-.39f,-.55f,.15f,.12f,.48f,.47f,.47f,-.25f,-.45f,.68f,.48f,-.92f,-.23f,.36f,.78f,-.20f,-.15f,-.14f,.82f,-.49f,-.91f,-.92f,-.92f,-.25f,-.22f,.77f,.44f,-.86f,.16f,.76f,-.06f,.73f,-.94f,-.99f,-.14f,.52f,-.63f,-.18f,.77f,-.70f,-.92f,.36f,.06f,.57f,-.96f,.10f,.12f,-.64f,.38f,-.75f,-.09f,-.81f,-.17f,-.82f,.39f,.16f,-.49f,-.82f,-.95f,.16f,.20f,.12f,.40f,.26f,.94f,.55f,-.83f,.06f,-.04f,.42f,-.97f,.49f,-.15f,-.18f,-.40f,.43f,-.56f,.33f,.29f,-.94f,-.74f,-.20f,.47f,-.08f,-.95f,.08f,-.32f,.75f,-.67f,.36f,-.29f,-.41f,.85f,-.15f,.09f,.26f,-.40f,-.64f,-.43f,-.45f,.87f,-.62f,-.96f,.26f,.87f,-.36f,-.69f,-.93f,-.32f,.09f,.94f,-.85f,.31f,-.59f,-.08f,.10f,.33f,-.97f,-.54f,-.50f,.25f,.26f,.67f,.30f,-.98f,.67f,-.92f,-.56f,-.80f,-.53f,-.34f,-.68f,-.28f,.59f,-.56f,.94f,-.10f,-.26f,-.93f,.54f,.13f,.23f,-.81f,.36f,-.26f,.90f,.53f,-.43f,.68f,.28f,.31f,.25f,.45f,-.21f,.67f,-.56f,-.63f,.05f,.39f,.81f,-.37f,-.92f,.53f,.40f,-.31f,.23f,-.00f,-.83f,.46f,-.54f,.98f,.09f,-.54f,-.89f,.82f,-.62f,-.09f,.02f,-.24f,.42f,-.85f,.15f,-.59f,-.73f,.73f,-.19f,-.58f,-.45f,.98f,-.12f,-.24f,-.49f,-.49f,.46f,.59f,-.74f,.79f,.22f,-.11f,-.64f,-.68f,.31f,.28f,.23f,.77f,-.36f,.30f,.68f,-.51f,-.52f,.59f,.06f,.29f,.69f,-.11f,-.35f,.67f,-.18f,-.59f,-.77f,-.09f,.30f,.78f,-.36f,.06f,-.08f,-.25f,-.69f,.82f,.64f,-.41f,.40f,-.34f,.01f,-.26f,.40f,-.73f,.84f,.39f,-.42f,-.31f,-.28f,-.96f,-.75f,-.64f,.23f,-.76f,-.42f,.92f,.52f,.92f,-.58f,.69f,-.15f,.24f,-.27f,-.86f,-.09f,-.42f,-.74f,-.01f,-.18f,-.41f,.64f,.92f,-.58f,.66f,-.59f,-.64f,.20f,-.80f,-.58f,-.44f,-.34f,-.11f,-.07f,.18f,.31f,-.05f,.90f,-.10f,-.59f,.33f,.89f,-.78f,.66f,.59f,.76f,.52f,.04f,.37f,.79f,.40f,-.97f,-.41f,.04f,.10f,.72f,-.20f,-.95f,.69f,.26f,-.56f,.88f,-.09f,-.73f,-.61f,-.88f,-.53f,-.61f,-.52f,-.29f,.96f,-.61f,-.63f,.18f,.04f,-.31f,.47f,-.72f,.57f,-.69f,-.30f,.32f,.38f,-.31f,-.05f,.83f,-.17f,.40f,.33f,-.27f,.09f,.85f,-.99f,-.96f,-.57f,-.56f,-.73f,.69f,.42f,-.89f,-.24f,.99f,-.18f,.91f,-.66f,-.63f,-.24f,-.24f,-.63f,.80f,.96f,.94f,.59f,-.76f,.26f,.74f,-.89f,.50f,.16f,.21f,-.48f,.32f,-.33f,.13f,-.61f,-.01f,.63f,-.62f,.45f,-.03f,-.25f,.33f,.09f,-.96f,.00f,-.63f,.41f,.48f,.90f,-.61f,.98f,-.02f,-.45f,-.28f,.14f,-.22f,-.68f,-.46f,.64f,-.95f,-.31f,-.24f,.88f,.34f,-.34f,-.42f,.56f,-.40f,-.27f,-.09f,-.78f,.84f,.25f,.34f,-.74f,.46f,-.95f,.05f,-.72f,-.15f,.12f,.49f,-.74f,.91f,.65f,.89f,-.33f,-.57f,-.49f,-.19f,-.08f,-.36f,.74f,-.48f,.07f,.09f,-.48f,-.16f,.65f,-.29f,-.13f,.04f,.37f,-.66f,.06f,-.38f,.10f,.07f,.92f,-.41f,-.17f,.99f,.36f,-.38f,.21f,.66f,-.25f,-.32f,-.31f,.85f,-.47f,-.38f,-.40f,-.10f,.08f,.27f,-.50f,-.31f,.96f,.51f,-.83f,.63f,.65f,-.70f,.09f,-.48f,.48f,.22f,.72f,-.90f,-.06f,.19f,-.16f,.41f,-.69f,-.16f,-.71f,.78f,-.83f,-.76f,-.20f,-.81f,.64f,-.49f,.08f,-.15f,-.33f,.77f,.51f,-.24f,-.08f,.82f,-.58f,.64f,.49f,-.18f,.43f,-.54f,-.85f,-.24f,-.71f,-.90f,.69f,-.73f,.72f,.47f,-.42f,-.75f,-.51f,.80f,.18f,-.08f,-.41f,.61f,-.31f,-.39f,.97f,-.32f,.51f,-.61f,-.35f,.10f,-.08f,-.28f,-.04f,.94f,.71f,.61f,-.13f,.21f,.96f,-.61f,.85f,-.55f,.96f,-.18f,-.05f,-.85f,-.60f,.92f,-.88f,-.71f,-.40f,.14f,.69f,-.21f,.63f,.86f,.44f,-.25f,-.23f,-.05f,-.29f,.89f,-.53f,-.06f,.03f,.36f,-.83f,.79f,-.40f,-.10f,.90f,-.92f,-.49f,-.38f,-.43f,.02f,.98f,-.99f,.13f,-.75f,-.99f,-.37f,.52f,-.20f,-.56f,-.38f,.80f,-.45f,.13f,.71f,-.60f,-.46f,-.90f,.83f,-.01f,-.47f,.08f,-.52f,-.27f,-.83f,-.06f,-.26f,-.43f,.02f,.71f,.26f,-.39f,-.20f,-.12f,.95f,.15f,.71f,-.82f,-.91f,-.98f,.99f,.60f,-.20f,-.89f,-.33f,-.22f,.25f,.04f,.74f,-.92f,-.19f,-.23f,.84f,.34f,.83f,.41f,.77f,.24f,.32f,-.31f,-.36f,-.08f,.58f,-.45f,.63f,-.28f,.15f,-.19f,.58f,.67f,-.21f,-.79f,.23f,-.33f,-.05f,-.60f,-.09f,-.98f,.73f,-.36f,.18f,.70f,.86f,-.18f,-.36f,.06f,-.30f,-.48f,-.08f,.70f,.57f,.00f,.95f,.69f,-.70f,-.98f,-.30f,-.78f,-.63f,-.91f,.83f,.72f,.45f,-.50f,-.30f,-.41f,-.17f,-.39f,-.65f,-.97f,.25f,-.03f,-.38f,-.24f,.42f,.27f,-.18f,.50f,-.04f,-.33f,-.54f,.69f,-.97f,.45f,.03f,.78f,.83f,-.91f,-.37f,-.55f,-.28f,-.15f,-.19f,.38f,.11f,-.39f,-.63f,.84f,-.19f,.61f,-.36f,.34f,-.54f,-.68f,-.68f,.47f,-.08f,.79f,.21f,-.50f,-.18f,-.40f,.80f,.00f,-.40f,-.90f,-.52f,-.21f,-.10f,.99f,.97f,-.71f,-.98f,-.80f,.20f,.73f,.95f,-.47f,.52f,.98f,.01f,.16f,-.50f,.43f,.51f,.81f,.69f,-.34f,-.49f,-.92f,.68f,.79f,-.56f,-.46f,.69f,.51f,-.45f,-.30f,.95f,-.46f,-.88f,-.92f,-.69f,.91f,-.62f,-.58f,.70f,.51f,-.70f,-.87f,-.01f,-.64f,-.22f,-.81f,-.65f,-.55f,-.06f,.84f,.54f,.29f,-.85f,-.99f,.87f,.98f,.61f,.64f,.25f,.87f,.89f,.48f,-.25f,-.78f,-.77f,.49f,.53f,-.06f,-.93f,.21f,.92f,-.51f,.55f,.83f,.14f,.17f,.42f,-.09f,.52f,.95f,.46f,.81f,-.75f,.88f,.99f,-.53f,-.93f,-.46f,-.67f,.61f,-.02f,-.57f,-.58f,.09f,.29f,.27f,.73f,.90f,-.72f,-.68f,-.00f,.64f,.24f,.18f,.73f,.03f,-.06f,-.84f,-.07f,-.25f,-.61f,.71f,-.10f,-.27f,-.83f,.24f,-.11f,-.97f,-.99f,.13f,-.84f,-.51f,.85f,-.11f,-.67f,.45f,.95f,.76f,-.63f,.95f,-.68f,-.15f,.77f,-.44f,.10f,.11f,.76f,.56f,.48f,.35f,-.11f,-.16f,-.00f,.63f,-.01f,.51f,.13f,-.28f,-.01f,.58f,.89f,-.69f,.43f,-.90f,.28f,.86f,.93f,.22f,-.16f,.47f,.33f,-.56f,.12f,-.26f,-.72f,-.51f,.24f,-.19f,-.88f,-.70f,.45f,-.93f,-.61f,-.76f,-.49f,.27f,-.11f,-.06f,.10f,.04f,.62f,.51f,-.27f,-.19f,.90f,-.39f,.18f,-.10f,.84f,.53f,-.71f,.17f,.29f,.72f,-.80f,.27f,.57f,-.53f,-.95f,.88f,.84f,.09f,.11f,-.13f,-.64f,-.36f,-.48f,.82f,.08f,-.35f,.03f,.76f,.14f,.79f,-.94f,-.35f,.89f,-.25f,-.48f,.37f,.82f,-.30f,.57f,.94f,.47f,.17f,-.39f,-.62f,.65f,.37f,.41f,-.85f,.74f,-.59f,.44f,-.63f,.45f,.90f,.99f,-.73f,-.03f,.00f,.49f,-.29f,-.63f,.59f,.24f,.66f,-.05f,.06f,-.95f,.06f,.62f,-.83f,-.46f,-.25f,-.35f,-.91f,.07f,-.77f,.54f,-.82f,.82f,.68f,.49f,-.63f,-.36f,-.31f,-.48f,-.95f,-.55f,-.62f,.52f,.26f,-.39f,.45f,-.09f,-.07f,.56f,.32f,.37f,.57f,-.83f,-.90f,.03f,.47f,.97f,.26f,.37f,-.24f,.72f,.38f,-.73f,.86f,.88f,-.59f,.17f,-.80f,-.86f,.64f,.83f,-.15f,-.31f,-.87f,.92f,-.24f,.45f,.42f,-.61f,-.37f,.02f,.82f,.84f,.06f,.67f,-.12f,.11f,.94f,-.81f,.75f,-.04f,.72f,-.50f,.30f,.90f,.93f,-.05f,-.10f,-.02f,.51f,.03f,-.67f,.18f,.37f,.65f,-.20f,-.45f,.52f,.46f,.95f,-.78f,.20f,.54f,.92f,-.64f,.51f,.63f,-.58f,-.50f,-.40f,.82f,.15f,.15f,.21f,.01f,.89f,.10f,-.05f,-.92f,-.18f,.30f,.00f,-.12f,.71f,-.58f,.15f,.17f,-.31f,-.36f,.95f,-.21f,-.82f,-.31f,-.78f,-.17f,.47f,-.41f,.90f,.75f,.28f,.57f,.16f,-.73f,.66f,-.27f,.76f,-.74f,.35f,-.21f,.39f,-.70f,-.31f,-.14f,-.85f,.80f,.58f,-.43f,.02f,.01f,.31f,-.89f,-.06f,.04f,.28f,-.00f,.74f,.32f,.66f,-.98f,-.53f,-.40f,-.94f,.35f,.53f,-.57f,.60f,.86f,-.94f,-.67f,.95f,-.35f,-.79f,.56f,.10f,-.28f,-.53f,-.27f,.25f,.83f,.77f,.18f,-.33f,.49f,-.64f,.57f,.80f,.98f,.56f,.03f,.58f,.76f,.87f,.61f,.91f,-.30f,-.60f,-.76f,-.30f,.23f,.06f,.03f,.72f,.11f,-.23f,-.21f,-.20f,-.44f,.78f,.15f,.87f,-.31f,.43f,-.05f,-.16f,.04f,-.79f,.87f,.09f,.32f,-.58f,.65f,-.62f,.23f,.20f,-.29f,.88f,-.58f,-.03f,-.33f,-.48f,.27f,.47f,.81f,.91f,-.41f,.67f,-.94f,.31f,.25f,-.00f,.99f,-.06f,.22f,-.19f,.53f,-.62f,-.63f,.28f,-.49f,.37f,-.69f,.59f,-.30f,.57f,.52f,.55f,-.67f,-.15f,.71f,.11f,-.34f,-.06f,.10f,.70f,.98f,.52f,.16f,.87f,-.62f,-.58f,.57f,.65f,-.35f,-.31f,.96f,.21f,.31f,-.82f,.14f,.57f,.24f,-.26f,.77f,-.09f,.20f,.40f,.12f,-.17f,.65f,.64f,-.63f,-.75f,.50f,.31f,-.06f,-.19f,-.63f,.69f,-.54f,.27f,.95f,-.44f,.18f,-.91f,-.02f,.77f,-.66f,-.33f,-.20f,-.22f,.65f,.31f,-.00f,-.26f,.07f,.05f,-.47f,-.52f,-.06f,.49f,.65f,.54f,.10f,-.10f,-.56f,-.87f,.35f,-.51f,.90f,-.04f,-.90f,-.16f,-.75f,-.06f,-.04f,-.11f,.44f,.73f,.72f,.80f,.73f,-.95f,.83f,.72f,.46f,-.94f,-.56f,.00f,-.41f,-.66f,.56f,.92f,-.23f,-.32f,-.16f,-.04f,-.37f,-.81f,-.29f,-.79f,-.45f,-.37f,.65f,-.07f,-.97f,-.93f,.15f,.01f,-.90f,.05f,-.63f,.43f,-.67f,.04f,.86f,-.64f,.80f,.66f,-.02f,.84f,-.46f,.19f,-.42f,-.20f,.86f,.12f,.32f,-.72f,-.40f,-.32f,-.26f,.85f,-.40f,.15f,-.69f,-.77f,-.48f,-.10f,-.16f,.61f,-.12f,-.76f,.21f,-.15f,-.15f,.00f,-.31f,.74f,-.12f,.75f,-.31f,.89f,.75f,-.17f,-.58f,-.33f,-.72f,-.43f,-.36f,-.74f,-.17f,-.61f,-.53f,.62f,-.24f,-.37f,-.16f,-.74f,.39f,-.41f,.57f,-.64f,-.51f,.65f,.40f,.65f,-.04f,-.08f,.30f,-.86f,.37f,-.49f,.24f,.36f,-.66f,.53f,-.42f,.47f,-.10f,-.41f,-.92f,-.88f,.90f,.59f,.00f,.70f,-.55f,.45f,.81f,.42f,-.81f,-.90f,-.66f,.47f,-.49f,-.21f,-.71f,-.62f,.20f,-.90f,-.49f,-.83f,-.94f,.16f,.04f,.84f,-.49f,-.23f,-.75f,-.13f,.91f,.36f,-.51f,.56f,-.26f,-.34f,-.27f,.13f,.45f,-.89f,.69f,-.42f,.49f,-.93f,.46f,.68f,.63f,-.86f,-.26f,.14f,-.29f,.80f,-.74f,.73f,.19f,-.26f,.01f,-.71f,-.54f,-.64f,-.59f,-.37f,.76f,.15f,.90f,-.28f,.59f,.16f,-.55f,.48f,-.49f,-.43f,.56f,-.30f,.12f,.08f,-.28f,.78f,.73f,-.06f,.13f,.31f,-.30f,.90f,.39f,-.77f,-.89f,.91f,-.09f,.76f,.30f,-.80f,.31f,-.94f,.88f,-.38f,-.10f,.02f,-.93f,-.27f,.83f,-.33f,-.55f,.04f,-.31f,-.21f,-.38f,-.04f,-.21f,-.58f,.57f,-.50f,.45f,-.88f,-.95f,.66f,-.15f,-.65f,.04f,-.91f,-.02f,-.40f,.60f,.17f,.07f,-.11f,-.55f,.36f,.47f,.09f,.37f,.75f,-.98f,-.39f,.79f,.61f,-.75f,.51f,-.49f,-.99f,-.72f,-.56f,.97f,-.42f,.06f,-.14f,-.96f,-.28f,-.48f,.99f,.64f,-.12f,.63f,-.95f,-.09f,-.81f,.83f,.99f,-.01f,.17f,.03f,-.52f,.20f,.77f,-.42f,.96f,.79f,-.64f,-.29f,-.84f,-.79f,.83f,-.88f,-.24f,.22f,.55f,.16f,-.85f,.49f,.61f,-.79f,-.75f,.27f,.36f,.36f,-.98f,-.34f,.13f,.38f,-.25f,.74f,.08f,.46f,.13f,.32f,.78f,.44f,.11f,-.57f,-.34f,-.92f,-.03f,-.74f,-.06f,.30f,-.51f,.06f,.74f,.13f,.34f,.05f,.81f,.43f,.80f,-.77f,-.54f,.59f,-.05f,.14f,.29f,-.70f,-.12f,.03f,-.04f,-.07f,-.84f,.86f,.87f,-.24f,.02f,.71f,.31f,-.62f,.62f,-.65f,.26f,.47f,.57f,.84f,.32f,.54f,-.50f,.93f,-.91f,.35f,-.74f,-.56f,-.46f,-.78f,-.60f,-.89f,-.88f,-.16f,-.83f,-.63f,.59f,.85f,.32f,.74f,.84f,-.83f,-.46f,-.77f,-.74f,.81f,-.95f,.99f,-.87f,-.28f,.66f,-.58f,.20f,-.17f,-.95f,.23f,.86f,-.81f,.93f,.38f,.49f,.68f,-.91f,-.58f,-.41f,.31f,.23f,.92f,.71f,.33f,.18f,-.33f,.64f,.75f,-.72f,.70f,-.49f,-.49f,-.18f,-.59f,-.23f,.71f,.42f,-.53f,-.83f,-.04f,-.75f,-.77f,-.59f,.53f,-.30f,.22f,-.78f,.64f,-.64f,-.37f,.79f,.72f,-.06f,.94f,.93f,-.75f,.90f,-.79f,.96f,.28f,-.74f,.70f,-.81f,.18f,-.96f,.84f,-.18f,.05f,-.23f,.26f,-.85f,-.12f,.50f,.12f,-.94f,.64f,.86f,.81f,.89f,-.62f,.74f,-.19f,-.78f,.67f,-.04f,.36f,.62f,-.02f,-.49f,.72f,-.02f,.42f,-.78f,.79f,.26f,-.63f,-.31f,-.38f,-.37f,-.43f,-.97f,.27f,-.34f,-.59f,-.07f,.44f,-.62f,-.64f,-.09f,-.70f,-.11f,-.89f,-.35f,.49f,-.73f,-.89f,.76f,.90f,.04f,.44f,.58f,.18f,-.06f,.68f,.14f,-.48f,-.15f,-.53f,-.31f,.37f,.71f,-.10f,-.83f,-.88f,-.26f,-.88f,.82f,-.04f,.84f,-.92f,-.10f,.00f,.18f,.21f,.74f,-.25f,.88f,-.75f,-.47f,.72f,-.54f,.04f,.10f,.29f,.20f,.08f,-.17f,.71f,-.46f,.96f,-.21f,.10f,.58f,.65f,.06f,.58f,.68f,.27f,-.07f,-.05f,.97f,.25f,-.42f,-.59f,.42f,.87f,-.06f,-.61f,.06f,.84f,.58f,-.68f,.65f,.40f,-.32f,-.38f,.93f,.58f,-.65f,.73f,.96f,-.97f,.08f,.80f,.39f,-.21f,-.47f,.56f,-.17f,-.46f,-.52f,-.27f,.76f,.60f,-.41f,-.73f,.93f,-.06f,.30f,-.64f,-.02f,-.23f,.08f,.72f,-.58f,-.29f,-.49f,.85f,-.42f,.40f,-.37f,-.74f,.51f,-.52f,-.52f,-.63f,-.66f,.81f,-.39f,.33f,-.46f,-.76f,-.06f,.45f,-.05f,-.64f,.42f,.76f,-.55f,-.98f,.78f,-.82f,-.25f,.24f,.35f,.68f,-.68f,.28f,.60f,-.50f,.20f,.15f,-.92f,.09f,-.99f,.49f,.24f,.69f,-.50f,.47f,.94f,-.30f,.96f,-.45f,-.57f,.25f,.77f,.82f,-.88f,-.33f,-.32f,-.65f,-.85f,.83f,.06f,.71f,.53f,-.42f,.25f,-.50f,-.02f,.76f,-.13f,.39f,-.99f,-.20f,.20f,-.80f,-.59f,-.59f,.13f,.57f,.55f,.44f,-.11f,.30f,-.16f,.78f,-.91f,.02f,.68f,.59f,-.28f,-.79f,.48f,-.17f,-.86f,-.68f,.96f,-.61f,-.69f,.95f,.81f,-.89f,.02f,.97f,-.13f,-.10f,.07f,.67f,.30f,-.26f,-.07f,-.55f,-.58f,-.81f,.06f,.65f,.97f,-.29f,.85f,-.49f,-.01f,-.90f,.23f,-.39f,-.71f,.28f,.38f,-.26f,.19f,-.94f,.38f,-.80f,.15f,-.59f,.18f,.87f,-.54f,-.65f,.47f,.74f,.43f,.30f,.99f,-.66f,.89f,-.72f,.25f,.73f,-.26f,-.60f,.10f,.55f,-.86f,-.96f,-.54f,-.81f,.84f,.67f,.34f,.98f,-.50f,-.64f,-.40f,-.61f,-.72f,-.68f,.80f,-.15f,-.99f,-.79f,.71f,-.66f,-.11f,.06f,.24f,-.58f,.60f,-.21f,.24f,-.75f,-.12f,-.91f,-.67f,-.19f,-.47f,-.87f,.40f,.42f,.69f,-.61f,-.98f,-.63f,.97f,.05f,-.19f,-.47f,.41f,-.45f,.94f,.85f,-.10f,.09f,.00f,.20f,.49f,-.81f,.31f,.94f,-.41f,-.71f,.99f,.82f,-.34f,-.08f,.85f,.61f,-.93f,-.54f,.37f,-.90f,-.67f,-.02f,-.43f,-.52f,-.58f,-.61f,.81f,.53f,.86f,-.09f,.26f,.23f,-.76f,-.58f,-.90f,.33f,.82f,-.44f,.18f,-.18f,.72f,.74f,-.84f,.83f,-.26f,-.32f,-.99f,-.07f,.57f,.71f,-.64f,.56f,-.08f,-.78f,.83f,-.60f,-.54f,-.65f,.26f,.67f,.10f,.72f,-.12f,.71f,.90f,-.37f,-.55f,-.42f,-.89f,.87f,.70f,.47f,.07f,.46f,.48f,.53f,-.95f,.82f,.35f,.48f,.99f,-.64f,.45f,-.33f,.94f,.93f,.55f,.95f,.23f,.71f,.42f,.48f,-.45f,.54f,.38f,.58f,.94f,.82f,.65f,-.51f,.53f,-.47f,-.78f,-.51f,.67f,.72f,-.20f,.09f,.41f,.69f,.21f,.95f,-.26f,.23f,-.04f,-.30f,-.18f,-.78f,.06f,.04f,.83f,-.97f,-.55f,-.86f,-.99f,-.12f,.17f,-.50f,-.77f,-.20f,-.47f,-.95f,.05f,-.37f,.88f,.09f,-.54f,-.64f,-.72f,.57f,.16f,-.31f,-.28f,-.36f,.71f,-.82f,.23f,.88f,.69f,.99f,-.99f,.94f,.24f,-.45f,.61f,.85f,-.92f,.96f,.98f,.34f,.03f,-.25f,.15f,-.71f,.48f,-.02f,-.29f,.34f,.48f,-.14f,-.46f,-.32f,.67f,.04f,-.90f,-.82f,.44f,-.46f,-.33f,.63f,-.46f,.16f,.02f,.57f,-.02f,.50f,.26f,.51f,-.15f,.25f,.07f,-.59f,-.87f,.64f,-.92f,.05f,.79f,.14f,-.34f,-.77f,.72f,.53f,-.42f,.95f,-.86f,-.35f,.90f,.36f,.25f,.63f,.59f,-.32f,-.95f,-.98f,-.84f,-.56f,.65f,-.81f,-.06f,-.45f,-.60f,-.84f,.09f,.77f,.17f,.11f,-.57f,-.67f,-.23f,-.98f,.12f,.32f,.35f,.37f,-.81f,-.07f,-.21f,.72f,-.35f,-.02f,.33f,.58f,-.52f,.17f,-.49f,.30f,-.30f,-.78f,-.38f,.25f,.14f,-.20f,.75f,-.51f,.46f,.15f,-.07f,.19f,-.47f,.70f,-.47f,-.03f,.14f,-.25f,.67f,.83f,.22f,-.80f,.15f,.88f,.42f,.76f,-.53f,.85f,.89f,.92f,-.73f,-.88f,.05f,.09f,.19f,.55f,.74f,-.08f,.41f,.06f,-.35f,.66f,.05f,.36f,.28f,.77f,.65f,.94f,-.28f,-.31f,.47f,-.62f,-.99f,.90f,.70f,.88f,-.68f,.69f,-.23f,-.01f,.97f,.87f,.55f,.07f,.59f,-.16f,-.05f,.28f,-.48f,.56f,-.64f,.22f,.55f,-.66f,.29f,.12f,.73f,-.98f,-.13f,-.39f,.33f,.95f,.38f,-.34f,.73f,.25f,.63f,.13f,-.18f,-.23f,-.68f,.39f,.65f,-.08f,-.96f,.35f,-.06f,.81f,-.34f,-.91f,.96f,-.75f,.61f,.81f,-.21f,.21f,-.89f,-.13f,-.92f,-.11f,-.25f,.28f,.31f,.83f,.96f,-.13f,.54f,.95f,.16f,.49f,.12f,.13f,.20f,.37f,.34f,.87f,-.11f,-.28f,-.05f,.72f,-.27f,.44f,.84f,-.03f,.69f,-.79f,.63f,.81f,.88f,.98f,.51f,-.20f,.54f,.73f,-.95f,-.61f,.33f,.26f,.51f,.46f,.43f,-.04f,-.29f,.77f,.34f,-.43f,.48f,.42f,-.14f,.89f,-.93f,-.40f,.77f,-.04f,.85f,.98f,-.26f,.05f,-.24f,-.22f,-.50f,-.82f,-.42f,.63f,.79f,-.84f,-.60f,.20f,-.41f,.86f,.51f,-.17f,-.20f,.94f,-.89f,-.51f,-.35f,.73f,-.53f,.09f,.49f,-.72f,-.47f,-.72f,-.58f,-.28f,.05f,-.08f,-.86f,-.69f,-.22f,-.32f,.17f,-.58f,-.49f,-.62f,.19f,-.28f,-.40f,-.61f,.75f,-.87f,-.96f,-.47f,-.04f,.64f,.80f,.90f,-.57f,.40f,.34f,-.31f,-.94f,.79f,.12f,.55f,.62f,.44f,.58f,-.10f,-.07f,.35f,.73f,.42f,-.05f,-.16f,.89f,-.05f,-.22f,-.25f,-.81f,-.57f,.78f,.49f,.06f,.87f,.61f,-.01f,.77f,.63f,-.15f,-.17f,-.42f,-.59f,.64f,.85f,-.70f,.03f,-.58f,.25f,.31f,.72f,.26f,-.83f,-.37f,-.10f,-.30f,-.75f,-.54f,.13f,.10f,.75f,-.66f,-.73f,-.41f,.15f,.51f,-.72f,.76f,.98f,-.41f,.61f,-.67f,-.82f,-.90f,.57f,.57f,.82f,.50f,.74f,-.06f,-.19f,-.49f,.71f,.82f,.19f,-.04f,.55f,.03f,.59f,-.65f,.99f,.74f,.81f,-.59f,-.60f,.73f,.47f,.73f,.72f,-.33f,.29f,.66f,.72f,-.21f,-.90f,.69f,.40f,.67f,.07f,-.39f,.34f,.94f,.56f,.61f,.24f,-.76f,.53f,-.15f,-.70f,.93f,.42f,-.12f,-.50f,.67f,-.55f,.73f,.46f,-.74f,.90f,-.68f,.57f,-.62f,-.19f,-.57f,.37f,.81f,-.84f,.73f,.20f,-.89f,-.55f,.01f,-.12f,.24f,-.14f,-.35f,.99f,.40f,.11f,-.83f,-.25f,.84f,.89f,-.51f,.75f,.44f,.45f,-.18f,-.70f,.83f,.02f,-.84f,.46f,-.83f,-.02f,.10f,.44f,.53f,-.96f,.70f,.55f,.37f,.84f,-.64f,-.06f,.94f,-.58f,-.83f,.92f,.11f,.34f,-.67f,-.42f,-.27f,-.55f,-.97f,-.92f,-.81f,-.21f,-.40f,.73f,.49f,.60f,-.56f,.95f,-.09f,-.20f,-.88f,-.09f,-.70f,-.30f,-.85f,-.39f,.94f,.06f,.06f,-.09f,.05f,.61f,.51f,.60f,-.78f,-.89f,.92f,-.03f,-.03f,.41f,-.10f,.97f,.67f,.92f,.70f,-.22f,.63f,.77f,-.46f,.25f,.17f,.00f,-.14f,-.87f,.49f,.99f,.42f,-.67f,.61f,.34f,.25f,-.71f,-.18f,.24f,-.27f,.24f,-.06f,-.05f,-.08f,.70f,-.19f,-.29f,.41f,.78f,.08f,-.88f,.39f,.33f,-.50f,.00f,-.35f,.56f,-.85f,.06f,-.02f,-.91f,-.58f,.18f,.66f,.46f,.96f,.90f,.74f,-.17f,-.80f,-.38f,-.01f,.25f,-.53f,.21f,.26f,-.63f,-.78f,.32f,.42f,-.60f,-.31f,-.93f,.36f,.41f,.47f,-.41f,.03f,.37f,-.93f,-.75f,-.10f,-.44f,.74f,.96f,.47f,.69f,.44f,.07f,.28f,-.84f,-.00f,-.65f,.23f,-.17f,.65f,-.05f,-.32f,-.98f,.67f,-.45f,.16f,.26f,.81f,.81f,-.72f,-.23f,.57f,-.87f,.76f,-.57f,.14f,-.44f,.31f,.80f,-.73f,.83f,.00f,-.54f,.10f,.42f,.54f,-.18f,.56f,.35f,.44f,-.52f,.69f,.69f,.94f,.39f,.96f,.58f,-.71f,.98f,-.92f,-.11f,.98f,.90f,.79f,.63f,.84f,-.95f,.61f,-.52f,-.10f,.33f,.10f,.56f,.74f,.11f,-.27f,-.36f,.88f,.33f,-.30f,.67f,.98f,.19f,.56f,.85f,.13f,-.65f,-.67f,.86f,-.03f,.83f,-.50f,-.54f,-.65f,-.64f,-.73f,-.60f,.68f,-.71f,-.36f,-.81f,-.27f,-.06f,-.04f,.47f,-.99f,-.35f,-.89f,.55f,-.39f,-.94f,.32f,-.23f,-.53f,.37f,-.17f,.85f,.27f,-.10f,-.81f,-.31f,-.22f,-.30f,-.29f,.93f,.08f,.98f,.32f,.97f,.86f,.67f,.39f,.50f,.06f,.25f,-.73f,-.88f,-.02f,.88f,.85f,.38f,-.29f,-.99f,.30f,-.06f,.14f,-.02f,.09f,-.62f,.41f,-.46f,.90f,.06f,-.78f,-.10f,.97f,.66f,-.96f,.46f,.12f,.84f,.89f,-.28f,-.65f,-.71f,.04f,.34f,-.98f,-.27f,.21f,.13f,-.56f,.56f,.71f,-.55f,.14f,.81f,.03f,-.19f,.82f,.48f,-.42f,.73f,.04f,.48f,-.10f,-.78f,.75f,-.89f,-.02f,.60f,.15f,-.88f,.01f,.93f,-.59f,.43f,.57f,.14f,.96f,.75f,.58f,.69f,-.92f,.14f,.94f,-.42f,.54f,-.73f,.91f,-.89f,-.95f,.41f,.26f,-.46f,-.64f,-.65f,.53f,-.61f,-.95f,-.41f,-.28f,-.78f,-.01f,-.55f,.69f,.60f,-.29f,.81f,.74f,.39f,.58f,-.65f,.37f,-.88f,-.11f,.08f,-.78f,.01f,-.46f,-.12f,.38f,-.63f,.66f,-.33f,-.51f,-.16f,.32f,.40f,-.03f,-.35f,.49f,-.52f,-.26f,-.99f,-.94f,.18f,.94f,.37f,.94f,.85f,.36f,-.00f,.36f,-.94f,-.47f,.92f,.75f,-.29f,-.82f,.95f,.29f,-.47f,.12f,-.96f,.06f,-.56f,-.20f,.47f,.15f,.08f,-.20f,-.20f,-.31f,.44f,-.83f,-.03f,.75f,-.87f,.90f,-.39f,-.06f,.56f,.03f,-.03f,-.37f,-.08f,-.25f,.15f,-.90f,.29f,-.94f,.20f,.71f,.14f,-.70f,.98f,-.77f,-.97f,-.43f,.25f,-.30f,.28f,-.04f,.19f,.40f,.56f,-.18f,-.66f,.66f,-.21f,-.48f,.76f,.87f,-.63f,-.12f,.22f,-.48f,-.41f,-.87f,-.16f,-.53f,.25f,-.31f,.45f,-.25f,.46f,.35f,.64f,-.45f,.84f,-.70f,.19f,-.96f,-.68f,.92f,-.07f,.10f,.89f,.25f,.70f,.52f,.63f,-.36f,.11f,-.39f,.77f,.59f,-.01f,-.60f,.66f,-.46f,.64f,-.21f,.37f,-.55f,.27f,.09f,-.35f,.07f,.80f,-.51f,-.80f,-.04f,.59f,.59f,-.96f,-.12f,-.28f,.71f,.17f,-.36f,.35f,.99f,-.30f,-.98f,.52f,-.62f,.69f,-.48f,.34f,.30f,-.78f,-.84f,.46f,-.36f,.31f,-.61f,-.71f,.46f,-.97f,-.97f,-.18f,-.42f,-.22f,-.42f,-.08f,.84f,-.48f,-.22f,-.95f,-.42f,-.45f,.17f,-.58f,.82f,-.81f,.23f,.07f,-.16f,.93f,-.82f,-.78f,.13f,-.80f,-.24f,-.00f,.75f,-.59f,.01f,.71f,.53f,.02f,-.97f,.81f,-.35f,.15f,-.32f,-.11f,.79f,-.08f,.82f,-.99f,-.68f,-.48f,-.08f,-.03f,.64f,.48f,-.69f,-.82f,.98f,.86f,-.32f,-.86f,-.08f,.25f,-.37f,-.56f,.04f,-.53f,.52f,.89f,-.26f,.79f,.97f,.46f,.07f,-.99f,-.91f,-.03f,.81f,.06f,-.56f,-.68f,.14f,-.15f,.33f,-.26f,.94f,.76f,.70f,.70f,-.88f,.20f,.02f,-.32f,.75f,.34f,.47f,.63f,.01f,-.80f,.31f,.01f,-.25f,.20f,.40f,-.95f,.52f,.24f,-.46f,.61f,.58f,-.97f,-.90f,-.24f,.18f,.16f,.56f,.21f,-.71f,-.96f,-.86f,.04f,-.11f,-.09f,.07f,-.01f,-.58f,-.12f,-.43f,-.95f,.45f,-.87f,-.07f,.31f,.02f,-.22f,-.62f,.48f,.19f,.42f,-.23f,.70f,-.56f,-.19f,-.54f,-.15f,.78f,-.39f,-.95f,.36f,.12f,-.31f,.93f,.16f,.92f,.58f,-.80f,.76f,-.16f,.15f,.93f,-.18f,.11f,.97f,.36f,-.29f,.56f,.67f,-.46f,-.94f,.64f,-.87f,-.72f,-.86f,.64f,.34f,-.00f,.03f,-.48f,-.11f,.09f,-.89f,-.85f,.71f,-.36f,-.28f,.04f,-.88f,.91f,.19f,.37f,-.58f,.29f,.17f,.33f,-.62f,-.32f,.83f,.00f,-.95f,-.03f,-.80f,.64f,.10f,-.11f,.46f,.54f,.64f,-.17f,-.31f,-.46f,.85f,-.42f,-.27f,.45f,.06f,-.11f,-.20f,-.30f,.29f,.73f,.43f,-.95f,.46f,-.04f,-.51f,.30f,.82f,-.93f,-.86f,.33f,-.39f,-.07f,-.67f,-.86f,-.61f,.48f,-.35f,-.57f,.31f,.05f,-.72f,.46f,-.62f,-.81f,-.74f,.59f,.79f,.77f,-.31f,-.38f,-.85f,.70f,.39f,.57f,.52f,-.08f,.49f,-.49f,-.53f,.61f,-.21f,-.04f,.03f,.54f,-.49f,.95f,-.45f,-.99f,-.39f,.19f,-.74f,-.43f,.97f,-.46f,-.93f,-.56f,.17f,-.74f,-.87f,-.20f,-.87f,-.60f,-.66f,.12f,-.88f,-.25f,-.27f,.36f,-.22f,.97f,-.24f,.35f,-.30f,-.10f,.66f,-.06f,-.21f,-.20f,.87f,-.13f,-.25f,.10f,-.47f,-.51f,.55f,.20f,.14f,.80f,-.25f,.81f,-.19f,-.04f,-.05f,.63f,-.03f,-.60f,.39f,-.25f,.47f,-.51f,-.42f,-.31f,-.08f,.76f,-.64f,.28f,-.15f,.89f,-.39f,.49f,-.31f,.39f,-.48f,-.24f,-.97f,.76f,-.34f,-.35f,-.83f,-.70f,-.92f,.37f,-.05f,.95f,.49f,-.04f,.51f,-.84f,.56f,.50f,.85f,.78f,-.03f,-.32f,.31f,-.13f,-.38f,-.34f,-.15f,.22f,-.91f,.97f,.12f,.71f,.22f,.98f,.45f,-.21f,-.27f,.31f,-.47f,-.90f,-.90f,-.83f,.07f,.20f,.68f,-.42f,-.36f,-.14f,.29f,.57f,.76f,-.18f,-.89f,-.30f,.92f,-.09f,-.05f,.96f,.29f,.07f,-.65f,-.78f,.55f,.17f,-.39f,.17f,.62f,.51f,.00f,-.61f,-.52f,-.57f,-.99f,-.65f,.94f,-.54f,-.00f,.99f,.40f,-.98f,.38f,-.15f,-.91f,-.47f,.61f,-.67f,.79f,-.01f,-.04f,.27f,.27f,.49f,.71f,.56f,-.98f,.42f,-.30f,-.48f,.85f,-.28f,.02f,-.26f,.27f,.40f,-.61f,-.48f,.79f,.08f,-.04f,.96f,.86f,.99f,-.97f,.79f,-.96f,.53f,.61f,-.16f,.39f,.02f,.76f,-.51f,.19f,.79f,.20f,.76f,.26f,-.95f,.03f,.06f,.86f,-.69f,.46f,-.55f,-.71f,-.91f,.69f,.53f,-.27f,-.02f,-.34f,-.78f,.51f,.04f,.92f,-.72f,-.84f,.01f,-.95f,.58f,-.85f,.86f,-.70f,.34f,.14f,-.22f,.89f,.44f,.91f,.51f,.15f,.56f,.75f,-.10f,.34f,.14f,-.34f,-.55f,.77f,.59f,.11f,-.16f,-.12f,-.89f,-.45f,-.56f,-.93f,.51f,-.19f,.82f,.17f,.65f,.18f,.69f,-.72f,.75f,.40f,-.37f,-.55f,.87f,.58f,.33f,-.78f,.29f,-.17f,.62f,-.05f,.39f,.69f,.90f,-.53f,.98f,.74f,-.34f,-.34f,-.25f,.62f,.96f,-.30f,-.54f,.46f,-.99f,.13f,-.40f,.15f,-.59f,-.85f,.18f,.89f,-.85f,.56f,-.23f,.73f,.17f,-.76f,-.23f,.77f,.82f,-.72f,-.94f,-.95f,.28f,.74f,-.97f,.44f,-.02f,.40f,-.53f,-.83f,-.57f,-.07f,.20f,.27f,.10f,-.19f,.99f,.18f,-.24f,.73f,-.30f,-.83f,-.34f,-.16f,-.23f,.18f,-.69f,-.70f,.52f,-.33f,.45f,-.81f,-.17f,.03f,.49f,.80f,.88f,-.30f,.17f,-.69f,.19f,.67f,-.78f,-.84f,-.75f,-.81f,.65f,.50f,.62f,.19f,-.91f,.26f,-.17f,-.33f,.19f,.13f,.32f,.59f,-.31f,.88f,-.33f,-.42f,-.34f,-.05f,-.35f,.20f,.31f,-.21f,-.30f,.70f,.10f,-.28f,-.27f,.48f,.45f,-.67f,-.14f,-.41f,-.80f,.02f,.03f,-.49f,-.73f,-.94f,-.28f,-.11f,.94f,-.02f,.06f,-.64f,-.87f,.59f,-.99f,-.19f,.50f,-.12f,.84f,.98f,-.69f,.64f,.93f,-.93f,-.42f,-.96f,-.26f,.21f,.48f,-.64f,.58f,-.86f,-.85f,-.73f,-.81f,.08f,-.89f,.00f,-.16f,.78f,.25f,-.87f,-.30f,-.19f,.58f,.30f,-.10f,.54f,.06f,-.52f,-.88f,.63f,-.34f,-.19f,-.92f,.60f,-.90f,-.21f,-.90f,-.28f,-.14f,-.71f,.49f,.80f,-.49f,-.38f,.57f,.10f,-.80f,.25f,-.38f,-.12f,-.52f,-.01f,-.72f,.74f,-.35f,.88f,.02f,-.14f,-.40f,-.42f,.12f,.93f,-.98f,.95f,-.34f,.53f,.00f,.22f,.35f,.90f,-.39f,-.04f,-.27f,-.20f,.90f,-.00f,.36f,-.68f,-.95f,.21f,.76f,.29f,.99f,-.28f,.02f,-.79f,-.39f,.63f,-.25f,.64f,.72f,-.93f,-.64f,.34f,.36f,.76f,.77f,.27f,.63f,.28f,.94f,.80f,.45f,.49f,.09f,.22f,.50f,.44f,.94f,-.85f,.32f,.34f,.02f,-.04f,-.50f,-.54f,-.96f,.77f,-.09f,-.20f,-.24f,-.78f,.36f,-.73f,.87f,.23f,-.99f,.72f,-.56f,.79f,.14f,-.48f,.37f,.69f,.91f,-.58f,.37f,-.86f,-.76f,-.15f,-.60f,-.16f,.13f,-.97f,-.08f,.84f,-.34f,-.46f,.23f,.61f,-.71f,.71f,-.07f,-.72f,.17f,-.25f,.58f,.10f,.90f,-.80f,.26f,.21f,.48f,-.24f,-.28f,.27f,-.80f,.52f,-.76f,.87f,-.71f,.20f,-.14f,.17f,.87f,-.37f,-.87f,.62f,.50f,.89f,-.66f,.43f,-.85f,.70f,.73f,-.69f,-.95f,-.85f,-.43f,.94f,-.15f,-.69f,.23f,-.29f,.76f,-.62f,.05f,-.59f,.59f,.47f,.27f,-.13f,.47f,-.18f,-.20f,.02f,-.43f,.60f,.11f,-.26f,-.41f,.82f,.66f,-.31f,-.32f,.58f,.86f,.49f,-.90f,.40f,.35f,-.16f,.48f,-.46f,-.02f,-.07f,.32f,-.98f,-.33f,.33f,.22f,-.26f,.09f,-.79f,.06f,.54f,-.87f,.02f,.83f,-.33f,.10f,.50f,-.44f,-.45f,-.97f,-.51f,-.90f,-.56f,.79f,-.48f,.41f,.72f,-.23f,.91f,-.07f,-.38f,.77f,-.55f,-.81f,-.55f,.75f,.66f,-.47f,.44f,-.90f,-.06f,.68f,-.31f,-.25f,-.00f,.47f,-.12f,.12f,.99f,-.62f,-.84f,.17f,.92f,.38f,-.19f,-.97f,.48f,-.89f,.30f,-.08f,.03f,.42f,.14f,.36f,-.26f,-.09f,.41f,.76f,.66f,-.11f,-.19f,.22f,-.11f,.39f,.68f,-.81f,.87f,.04f,.54f,.02f,.05f,-.24f,.05f,.57f,-.81f,-.77f,-.09f,-.35f,-.00f,-.61f,.03f,-.48f,.40f,-.22f,-.19f,-.55f,-.53f,-.32f,-.17f,-.08f,.57f,.18f,.41f,.54f,.21f,-.25f,.07f,.76f,-.97f,-.79f,-.03f,-.56f,-.50f,-.55f,-.58f,.59f,.82f,-.18f,-.50f,-.32f,-.83f,-.44f,-.94f,-.70f,.90f,-.23f,-.23f,-.83f,.35f,-.45f,.31f,.52f,-.14f,.79f,-.30f,-.51f,.99f,.47f,.00f,-.42f,.43f,-.52f,.68f,-.11f,.54f,.98f,.66f,-.32f,.13f,-.87f,-.20f,.27f,-.74f,.64f,-.08f,-.68f,-.78f,.45f,-.37f,.23f,.79f,-.03f,-.09f,.27f,-.46f,-.22f,-.39f,.92f,.80f,-.83f,.89f,.42f,.75f,-.59f,-.26f,-.62f,-.37f,-.07f,.97f,-.90f,.39f,-.50f,-.20f,-.54f,.34f,-.43f,-.82f,-.35f,.42f,-.46f,.48f,-.69f,.33f,.19f,.99f,.06f,-.30f,-.99f,.13f,.47f,-.23f,-.47f,.07f,.82f,-.58f,.42f,-.83f,-.19f,-.98f,-.80f,.28f,.15f,.44f,-.87f,.99f,-.46f,-.85f,-.07f,-.46f,-.93f,.87f,.49f,-.09f,-.74f,-.43f,-.94f,.44f,.50f,-.84f,-.51f,.50f,-.33f,.58f,.22f,-.89f,-.65f,.39f,.01f,.94f,-.73f,-.30f,.69f,.47f,.11f,-.30f,-.16f,-.58f,.76f,-.78f,-.84f,.83f,.96f,-.44f,-.40f,-.28f,.76f,-.47f,-.00f,.91f,.12f,.58f,.64f,.61f,.30f,-.55f,-.33f,.40f,.17f,.35f,.93f,-.63f,.33f,.21f,-.03f,-.55f,.37f,.55f,-.14f,-.37f,-.32f,-.41f,-.50f,-.30f,.49f,-.16f,-.13f,-.77f,-.82f,-.16f,.67f,-.83f,-.87f,.17f,.39f,-.72f,-.09f,.28f,.43f,.44f,-.28f,.23f,.33f,-.95f,.94f,-.70f,.03f,-.73f,-.06f,.90f,-.13f,.39f,-.57f,.13f,-.00f,.30f,-.83f,.53f,-.61f,.27f,.84f,.98f,.98f,-.34f,-.83f,-.36f,.16f,-.93f,-.45f,.23f,.22f,.18f,.80f,-.56f,-.34f,-.40f,-.84f,-.63f,.98f,-.07f,-.36f,.52f,.06f,-.61f,-.19f,.10f,.60f,.37f,-.74f,.84f,-.86f,.63f,.75f,-.48f,.93f,-.39f,-.01f,.43f,.24f,-.49f,-.36f,-.97f,-.56f,-.08f,.61f,.38f,-.25f,.81f,.42f,.42f,.95f,.18f,-.14f,.12f,-.32f,-.62f,.84f,-.92f,-.57f,.57f,-.18f,-.70f,.39f,.48f,.37f,-.34f,-.65f,-.42f,-.81f,.50f,-.27f,-.63f,.09f,-.97f,.76f,-.86f,-.93f,-.67f,-.86f,-.69f,.15f,-.21f,.20f,-.52f,-.65f,.80f,-.69f,-.78f,.23f,-.69f,-.84f,.09f,.53f,-.24f,.44f,.16f,-.10f,.46f,-.63f,-.50f,-.08f,.33f,.76f,-.37f,-.16f,.81f,.44f,.02f,.33f,-.73f,.72f,-.08f,-.47f,.59f,.76f,-.83f,.09f,-.90f,-.91f,-.53f,.80f,-.51f,.33f,-.78f,.47f,-.71f,-.58f,.22f,.53f,-.10f,-.00f,.10f,-.71f,.94f,.18f,-.67f,-.10f,-.95f,.52f,.87f,-.45f,.68f,-.00f,-.78f,-.01f,-.04f,.39f,.02f,.77f,.82f,-.98f,-.90f,-.63f,-.31f,-.97f,.29f,-.00f,.55f,-.58f,.97f,-.35f,.16f,-.56f,.06f,.97f,.54f,.80f,-.59f,.87f,.74f,-.31f,-.24f,.39f,.43f,.33f,-.12f,-.30f,.61f,.77f,.23f,-.05f,-.13f,.14f,.76f,-.12f,.53f,-.56f,-.75f,-.84f,-.97f,.33f,.65f,.16f,-.46f,.28f,-.39f,.40f,.03f,-.36f,.97f,-.27f,-.31f,-.34f,.68f,-.15f,-.09f,.84f,.29f,.90f,-.42f,-.82f,.89f,.15f,.20f,-.32f,-.03f,.29f,.61f,-.52f,-.62f,-.93f,.79f,.28f,-.03f,.03f,-.54f,-.61f,.31f,-.91f,.92f,-.18f,-.12f,.79f,-.24f,-.40f,.46f,-.63f,-.93f,.82f,.40f,-.09f,.12f,-.77f,-.13f,-.58f,.86f,-.86f,.50f,-.72f,-.46f,-.81f,.94f,-.66f,.24f,-.78f,-.79f,-.48f,.49f,.47f,-.04f,-.83f,-.49f,-.48f,-.16f,-.18f,.03f,-.90f,-.47f,-.26f,.38f,.90f,-.81f,-.64f,-.29f,.69f,.62f,-.99f,.50f,.94f,.68f,-.00f,.51f,.26f,-.78f,-.71f,.63f,-.81f,-.18f,-.30f,.28f,.18f,-.20f,-.44f,.26f,-.13f,-.86f,.77f,.78f,-.19f,-.15f,-.77f,.37f,-.01f,-.21f,-.07f,-.78f,.54f,.78f,.38f,.69f,.32f,.04f,.42f,-.40f,.06f,.79f,-.26f,-.15f,-.27f,.23f,-.37f,.90f,-.22f,.73f,-.87f,.59f,-.53f,-.35f,-.88f,.82f,-.57f,-.86f,-.38f,-.87f,.72f,.51f,.56f,.32f,.27f,.86f,.96f,-.40f,-.54f,-.54f,.64f,.99f,-.50f,.29f,.58f,.66f,-.58f,.23f,.24f,-.67f,-.23f,.31f,.47f,-.91f,.56f,.97f,.06f,-.68f,-.36f,.47f,.98f,.41f,-.71f,-.38f,.94f,-.28f,.10f,.87f,.63f,.19f,-.93f,-.64f,.35f,.85f,-.33f,.62f,-.54f,.37f,.92f,-.22f,-.55f,-.59f,-.06f,-.81f,.81f,-.05f,.70f,.47f,.62f,-.69f,-.19f,.37f,-.57f,.66f,-.14f,-.97f,-.71f,.57f,.66f,-.46f,.66f,-.11f,.91f,.75f,-.02f,.14f,-.25f,-.32f,.92f,-.58f,.48f,.62f,.23f,.06f,.17f,-.28f,.81f,.78f,-.14f,.09f,.12f,.27f,.36f,.92f,-.06f,-.52f,-.17f,.36f,-.22f,-.41f,-.64f,.64f,-.69f,-.68f,-.07f,-.12f,.73f,.88f,-.49f,.00f,-.93f,-.45f,-.39f,.67f,-.02f,-.55f,.61f,.51f,.41f,.27f,.19f,-.49f,.14f,.16f,-.45f,-.31f,.70f,-.17f,.52f,-.34f,.66f,.84f,.47f,-.90f,-.38f,.21f,.55f,.86f,.86f,-.79f,-.44f,-.51f,-.09f,-.30f,-.39f,-.18f,-.81f,.32f,-.90f,-.14f,-.24f,.77f,-.68f,.09f,.64f,.08f,.15f,-.36f,.51f,-.54f,-.34f,-.73f,-.25f,.66f,-.33f,.23f,-.93f,.40f,.62f,.12f,.23f,.20f,.57f,-.52f,.08f,.53f,.10f,.79f,-.63f,.46f,-.12f,.21f,.60f,.84f,.84f,-.54f,-.24f,-.75f,.71f,.94f,.81f,-.07f,-.96f,-.51f,.40f,-.26f,.34f,-.79f,-.95f,-.58f,.54f,-.48f,-.83f,-.71f,-.45f,-.68f,.05f,.28f,.58f,.16f,.48f,.09f,.09f,.13f,.76f,-.54f,.14f,-.47f,.95f,-.40f,.96f,-.26f,-.38f,-.54f,.05f,.77f,-.84f,-.88f,.81f,-.69f,-.93f,-.03f,.32f,.60f,-.59f,-.07f,-.66f,.68f,-.77f,-.55f,.31f,-.22f,-.20f,.13f,.32f,.32f,.32f,-.86f,-.19f,.12f,.82f,.38f,.73f,.07f,.09f,-.04f,-.62f,.13f,-.51f,-.87f,-.48f,.07f,.13f,-.16f,.68f,-.97f,-.59f,.00f,.24f,-.67f,-.57f,.59f,.64f,.36f,-.50f,.01f,.94f,.33f,.69f,.24f,-.96f,.31f,-.21f,.77f,-.78f,-.87f,.73f,.55f,.46f,.95f,.99f,.64f,.03f,.04f,-.19f,-.03f,.54f,.19f,-.09f,.57f,.25f,.75f,-.22f,.45f,.23f,-.52f,-.27f,-.83f,.01f,-.08f,-.51f,.26f,-.36f,-.30f,-.77f,.17f,-.02f,-.57f,-.29f,-.47f,-.53f,.97f,.49f,.03f,.52f,.93f,.71f,.82f,-.35f,.38f,.47f,-.92f,-.36f,.69f,-.88f,.58f,-.02f,-.80f,-.69f,.56f,-.15f,-.74f,.88f,.35f,.24f,.03f,-.43f,-.03f,-.97f,.17f,-.92f,.44f,-.57f,-.90f,.50f,.26f,-.87f,.25f,-.70f,-.19f,.56f,.92f,.09f,.55f,-.38f,.28f,.91f,-.25f,.55f,.46f,-.80f,.24f,-.66f,.32f,-.05f,-.09f,.12f,.86f,.66f,-.93f,.88f,-.31f,-.13f,.68f,.84f,-.58f,.19f,-.61f,-.06f,-.01f,.10f,-.77f,.36f,-.05f,.96f,.83f,-.66f,.81f,.54f,.46f,-.93f,.19f,-.75f,.85f,-.26f,-.03f,-.77f,.54f,-.54f,.72f,-.05f,-.81f,.58f,-.34f,-.53f,.98f,.96f,-.55f,.99f,-.48f,-.29f,-.01f,.09f,-.02f,-.34f,-.15f,.44f,-.54f,-.82f,-.35f,-.87f,-.45f,.80f,.74f,.86f,.75f,.66f,-.69f,-.29f,-.18f,.17f,-.50f,.66f,.37f,.75f,-.44f,-.07f,.06f,-.36f,-.10f,-.63f,.12f,-.25f,-.71f,-.51f,-.75f,-.46f,.52f,-.36f,-.97f,.75f,-.23f,.26f,-.27f,.78f,-.93f,-.54f,.00f,.73f,.05f,.00f,-.95f,.19f,.53f,-.57f,.82f,-.48f,-.46f,.37f,.74f,-.99f,.46f,-.10f,.91f,.83f,-.45f,-.37f,-.05f,-.75f,-.23f,-.88f,-.16f,-.97f,.80f,.64f,-.57f,.53f,-.70f,-.19f,.96f,.26f,-.70f,.59f,-.32f,.49f,-.25f,-.01f,.57f,-.21f,.04f,-.32f,.23f,.19f,-.17f,-.58f,-.27f,-.50f,-.59f,-.77f,.07f,-.37f,.13f,-.83f,.31f,.63f,-.55f,-.32f,-.66f,-.61f,-.95f,.04f,.58f,.19f,.50f,-.79f,.32f,.89f,.58f,-.28f,.21f,-.36f,.84f,.24f,-.18f,.96f,-.43f,.17f,-.38f,-.07f,.05f,.91f,-.41f,-.24f,.93f,-.46f,-.51f,.93f,.49f,.17f,.72f,-.07f,-.94f,-.04f,.87f,.86f,-.73f,-.76f,-.54f,-.03f,.83f,.62f,.76f,-.72f,-.76f,.95f,.32f,.61f,.45f,.63f,.89f,-.73f,.63f,.30f,-.75f,-.03f,.64f,.48f,-.94f,-.40f,.28f,.91f,.29f,-.02f,.62f,-.06f,.07f,.86f,.31f,.93f,.41f,.58f,.67f,-.30f,.64f,.59f,-.33f,-.77f,-.52f,.74f,-.58f,.93f,-.17f,-.49f,-.91f,.82f,-.57f,.05f,.17f,.44f,.72f,-.95f,-.29f,.70f,-.96f,-.11f,.31f,-.52f,-.97f,-.58f,.46f,-.06f,-.30f,.25f,.16f,-.38f,.72f,.47f,.47f,.84f,.64f,-.61f,-.39f,-.78f,.58f,.19f,-.05f,.38f,.35f,.11f,-.16f,.85f,-.63f,.88f,.22f,.79f,.30f,-.50f,-.90f,.41f,.16f,.40f,-.38f,-.94f,-.61f,-.02f,.13f,-.84f,.86f,.61f,.08f,.00f,.13f,-.58f,.09f,-.10f,-.21f,-.89f,-.53f,.41f,.02f,.93f,.94f,-.95f,.14f,-.81f,.51f,.00f,-.23f,.11f,-.50f,-.62f,-.57f,.46f,.65f,.98f,.61f,.67f,-.32f,-.09f,-.70f,.62f,.40f,-.69f,.15f,.73f,.76f,.49f,-.10f,-.13f,.93f,.72f,.08f,.96f,.44f,.63f,-.68f,-.69f,.35f,.50f,-.73f,.58f,.85f,-.21f,.32f,-.87f,.05f,-.05f,.35f,.63f,-.24f,-.96f,-.42f,-.05f,.75f,.07f,-.91f,-.35f,-.80f,-.48f,-.86f,-.35f,.42f,.88f,.38f,.53f,-.74f,-.61f,.14f,-.43f,.68f,-.02f,-.30f,-.81f,-.39f,.32f,-.39f,.23f,-.82f,.17f,-.05f,-.45f,-.37f,.57f,-.44f,.72f,-.00f,.84f,.98f,-.03f,-.34f,-.51f,-.27f,.46f,-.49f,.86f,-.15f,-.70f,.91f,-.87f,-.44f,.68f,.96f,.20f,-.22f,.96f,-.46f,-.18f,.42f,-.02f,-.66f,.85f,.23f,.03f,.28f,-.56f,-.20f,-.94f,.90f,-.11f,-.63f,-.19f,-.37f,.29f,.18f,-.32f,.41f,-.42f,.95f,-.88f,.74f,-.55f,-.36f,-.70f,.09f,.08f,-.33f,.16f,.67f,.46f,-.94f,-.47f,.94f,.61f,-.40f,.97f,-.08f,.29f,.61f,-.88f,.66f,.83f,-.84f,.61f,.66f,.92f,.26f,-.12f,-.25f,-.01f,-.07f,-.98f,-.45f,.92f,-.29f,-.48f,.08f,.16f,.53f,.96f,.55f,.39f,-.61f,.45f,-.45f,-.09f,.10f,-.07f,-.11f,-.87f,-.43f,.32f,.91f,-.72f,-.75f,.55f,-.80f,.33f,.03f,-.19f,-.93f,-.65f,.92f,-.26f,.89f,.50f,.28f,-.98f,-.55f,-.80f,-.50f,.33f,.08f,.85f,-.33f,.57f,-.59f,.97f,.95f,.51f,.60f,-.80f,.92f,-.55f,.10f,.13f,-.89f,.04f,-.75f,.73f,.80f,.87f,.23f,.59f,-.63f,.70f,-.95f,.82f,-.27f,.54f,.19f,.96f,.95f,-.81f,.02f,.45f,.40f,-.52f,-.74f,.27f,.37f,-.27f,-.23f,.24f,.28f,.27f,.38f,.62f,-.84f,.22f,-.88f,-.77f,.61f,-.22f,-.26f,.59f,.63f,.49f,-.77f,.30f,-.57f,.39f,-.21f,-.68f,.43f,.60f,.30f,-.51f,-.28f,.26f,-.06f,-.68f,.50f,-.07f,.16f,.25f,-.39f,-.08f,.81f,.83f,-.72f,-.67f,.35f,-.08f,.37f,-.24f,.64f,.37f,-.17f,.61f,-.35f,.21f,.72f,.98f,.38f,-.14f,.59f,.45f,.30f,.61f,.62f,.56f,-.66f,-.39f,.11f,-.43f,-.08f,.75f,.31f,-.64f,-.53f,.41f,.48f,.54f,.62f,-.64f,-.58f,-.00f,.05f,-.62f,-.65f,.72f,-.90f,.29f,-.56f,-.68f,.98f,.84f,-.43f,.16f,-.43f,.16f,-.86f,.23f,.70f,-.94f,-.74f,.41f,.20f,.02f,-.35f,-.99f,-.87f,-.65f,.77f,.32f,.92f,.14f,.95f,-.80f,-.14f,.10f,.60f,.06f,.19f,.28f,.32f,-.87f,.32f,-.61f,-.94f,-.76f,.91f,-.77f,-.80f,-.33f,.36f,-.66f,.50f,.06f,-.79f,-.14f,.38f,.54f,-.34f,-.51f,-.35f,-.78f,.40f,.78f,-.55f,-.93f,.77f,-.93f,.69f,.86f,-.07f,-.78f,-.16f,.30f,-.95f,-.53f,.73f,-.75f,-.55f,-.16f,-.30f,-.74f,.95f,-.18f,-.65f,.99f,-.87f,.49f,.24f,-.06f,.58f,-.93f,-.93f,-.36f,.74f,.54f,-.62f,.09f,.09f,-.05f,-.06f,.15f,-.69f,-.22f,-.20f,.70f,.56f,-.30f,-.70f,-.69f,-.10f,.44f,-.01f,-.95f,-.85f,.53f,-.05f,.00f,-.66f,.05f,-.52f,.64f,-.21f,.22f,-.25f,-.22f,.28f,-.70f,.57f,-.90f,-.33f,-.69f,-.14f,.08f,-.38f,-.14f,-.97f,.84f,.69f,-.38f,.36f,.42f,-.39f,.29f,.50f,-.36f,-.58f,-.40f,.88f,.70f,.59f,.88f,.90f,-.10f,-.22f,-.68f,-.92f,-.55f,-.46f,.96f,-.50f,.08f,.92f,-.68f,-.27f,.29f,-.95f,.58f,.69f,-.62f,-.25f,.02f,-.50f,.01f,.33f,-.93f,-.85f,-.51f,.04f,.29f,-.32f,-.66f,.08f,.94f,-.81f,-.70f,.44f,-.13f,.72f,-.81f,-.55f,-.53f,-.27f,-.76f,.04f,.91f,.80f,-.91f,-.79f,-.11f,.17f,-.42f,.57f,.72f,-.52f,-.12f,-.00f,-.17f,.98f,-.30f,.20f,.64f,-.29f,-.42f,.45f,.85f,.53f,.47f,-.29f,-.04f,.97f,.86f,-.13f,-.96f,-.34f,.75f,-.04f,-.21f,.59f,.68f,.36f,-.11f,.76f,-.59f,-.28f,.07f,.49f,.23f,-.01f,-.00f,-.33f,-.14f,.17f,.68f,.41f,-.02f,-.40f,.75f,.39f,-.07f,.11f,.66f,.47f,.90f,.17f,-.15f,.48f,-.10f,-.81f,.47f,.72f,.51f,-.64f,.41f,-.21f,.97f,.42f,.81f,.00f,-.99f,.43f,.83f,.78f,-.92f,.32f,.59f,-.96f,-.46f,.32f,.53f,-.28f,-.00f,-.04f,-.99f,.67f,-.13f,-.43f,.50f,-.65f,.96f,.71f,-.71f,-.40f,.44f,.25f,.90f,.45f,.71f,-.66f,.00f,.52f,.93f,.58f,.29f,.62f,.27f,.56f,.42f,.26f,.78f,-.92f,-.29f,.98f,.97f,.80f,.53f,.40f,-.28f,-.01f,-.15f,.00f,.18f,.01f,-.46f,-.81f,-.42f,.58f,-.47f,.40f,-.81f,.67f,.70f,-.64f,.31f,-.75f,-.07f,.73f,.31f,.20f,.07f,-.31f,-.50f,-.91f,-.53f,.80f,.13f,-.31f,.46f,.99f,-.52f,.09f,-.21f,.46f,-.76f,.27f,.88f,.30f,-.83f,-.85f,.98f,-.34f,-.84f,.50f,-.78f,.17f,-.72f,-.06f,-.53f,-.53f,-.74f,.95f,.42f,.25f,-.73f,.10f,-.06f,-.87f,.01f,.54f,.87f,.68f,-.85f,.53f,-.02f,.86f,-.15f,.23f,.18f,-.34f,.67f,-.82f,-.70f,.35f,-.90f,.09f,-.03f,.94f,.69f,.43f,.60f,-.87f,.45f,.61f,-.31f,.54f,-.91f,-.73f,.58f,.27f,-.81f,-.94f,-.63f,-.91f,.10f,.25f,.08f,.47f,-.75f,-.34f,.69f,.37f,.90f,.21f,.81f,-.53f,.80f,-.67f,-.95f,-.55f,.21f,-.04f,-.12f,-.64f,.78f,.15f,-.49f,.26f,-.29f,.09f,-.84f,-.93f,.11f,.58f,-.67f,.60f,-.40f,-.61f,.72f,-.36f,-.24f,-.89f,-.21f,-.82f,-.37f,-.57f,.72f,-.66f,.63f,.74f,.37f,-.51f,.37f,-.71f,.92f,-.34f,.02f,-.95f,-.37f,-.92f,.04f,.12f,-.26f,-.43f,-.81f,-.26f,-.48f,.44f,-.12f,-.75f,.98f,-.31f,-.01f,.29f,.77f,.29f,.87f,.47f,.18f,.07f,-.99f,.81f,.14f,.53f,-.03f,-.22f,.28f,-.56f,-.86f,.52f,.83f,.89f,-.54f,-.48f,-.91f,.80f,-.12f,-.06f,.09f,.75f,-.36f,.55f,.55f,.46f,.74f,.53f,.99f,-.33f,-.45f,-.06f,-.12f,.19f,-.17f,-.38f,-.78f,-.58f,-.74f,-.65f,-.03f,-.77f,-.38f,-.97f,.24f,.57f,.75f,.45f,.39f,.89f,-.81f,-.42f,.93f,-.87f,.52f,.44f,-.34f,-.16f,-.02f,-.85f,.97f,-.41f,-.57f,.96f,-.94f,-.66f,.92f,-.94f,.23f,-.06f,-.68f,.37f,.30f,.75f,-.51f,.12f,.52f,-.17f,.55f,-.17f,.40f,.60f,.97f,.83f,-.13f,-.28f,.73f,-.77f,.36f,-.03f,.96f,.41f,.97f,-.65f,-.78f,-.68f,.71f,-.25f,-.02f,-.14f,.32f,.48f,-.85f,-.12f,-.11f,.90f,-.59f,-.70f,.19f,.53f,-.83f,.34f,-.22f,.34f,-.37f,.01f,-.64f,-.40f,-.64f,-.04f,-.41f,-.77f,-.63f,.16f,-.17f,.29f,-.19f,.87f,.49f,-.09f,.27f,-.24f,.73f,-.55f,.97f,-.62f,.64f,.27f,-.66f,-.34f,.39f,.81f,-.13f,.14f,.88f,-.43f,.79f,-.88f,.91f,-.25f,-.24f,-.40f,-.51f,-.61f,.11f,.45f,.23f,.60f,-.73f,.20f,.65f,-.25f,-.67f,.76f,-.83f,.64f,-.01f,-.51f,-.27f,-.03f,.33f,.27f,-.69f,-.85f,.06f,.53f,-.78f,.85f,-.26f,.46f,-.79f,.86f,-.25f,-.67f,-.88f,-.69f,-.71f,.58f,.09f,-.60f,.83f,.40f,-.88f,-.15f,.46f,.05f,.37f,.25f,-.85f,-.39f,.69f,-.68f,-.21f,.97f,-.03f,-.13f,-.59f,.65f,-.72f,-.84f,-.80f,.42f,-.36f,-.11f,-.84f,-.27f,-.13f,-.03f,.82f,.33f,-.24f,.30f,-.28f,.41f,-.56f,-.47f,-.18f,-.57f,-.56f,-.87f,-.05f,-.17f,-.31f,-.73f,-.89f,.01f,.20f,.06f,-.85f,-.75f,.78f,-.60f,.87f,.91f,-.13f,.75f,-.49f,.96f,-.60f,-.87f,-.49f,.83f,.42f,-.29f,.32f,-.46f,-.84f,.37f,-.36f,.39f,-.72f,.24f,.28f,-.63f,.40f,.92f,-.75f,-.68f,.74f,.07f,.60f,.22f,-.50f,-.10f,-.08f,.50f,-.34f,.24f,-.77f,-.23f,.77f,.89f,-.20f,.89f,-.95f,.35f,.19f,.70f,-.10f,.07f,.71f,.16f,.33f,.37f,.13f,-.98f,-.12f,-.50f,.68f,-.62f,-.38f,.13f,.86f,.22f,-.56f,.35f,-.98f,-.72f,.86f,-.44f,.73f,.57f,.84f,-.93f,-.62f,-.67f,.14f,-.73f,-.85f,-.38f,.01f,-.87f,.05f,-.98f,-.18f,.56f,.90f,-.66f,-.81f,.09f,-.63f,.04f,-.88f,.74f,.73f,.71f,.68f,.27f,-.52f,.16f,-.72f,.10f,-.71f,.97f,-.55f,-.06f,-.68f,-.74f,-.66f,-.93f,.23f,-.35f,.37f,.54f,.60f,.10f,-.76f,.45f,-.66f,-.08f,.98f,-.09f,.96f,-.54f,-.33f,-.04f,.36f,.86f,-.42f,-.50f,.45f,.33f,.78f,.25f,.58f,.15f,-.89f,-.49f,.42f,.78f,-.86f,-.65f,.81f,-.67f,.62f,.14f,.62f,.05f,-.93f,.34f,-.40f,.01f,.46f,-.34f,.51f,-.86f,.94f,-.20f,-.75f,.27f,-.09f,.41f,-.86f,.80f,-.13f,-.31f,.52f,-.12f,.46f,.58f,.60f,-.12f,-.45f,-.61f,-.33f,-.80f,-.25f,-.00f,-.20f,-.48f,-.98f,.96f,-.86f,-.21f,.30f,.26f,.53f,.83f,.82f,-.39f,-.66f,-.69f,-.42f,.35f,-.88f,.27f,-.34f,-.21f,-.89f,.74f,-.98f,-.35f,-.72f,-.37f,-.32f,.39f,-.24f,-.83f,.95f,-.43f,.72f,.24f,-.85f,-.19f,.55f,.02f,.34f,-.41f,-.08f,-.25f,.40f,-.00f,.58f,.43f,.79f,.29f,-.57f,-.02f,-.31f,-.01f,.33f,-.09f,-.07f,.53f,-.58f,.94f,-.64f,.37f,.63f,-.79f,.21f,.72f,.24f,.09f,.34f,.15f,.41f,.94f,.69f,.51f,.52f,-.03f,-.81f,-.99f,-.76f,.33f,.61f,.03f,-.61f,.78f,.69f,.62f,-.22f,-.70f,-.64f,.55f,-.56f,.24f,-.39f,.33f,-.08f,.92f,.88f,-.94f,-.85f,-.47f,-.96f,-.51f,.22f,.87f,-.85f,.51f,.27f,.12f,-.83f,-.13f,-.63f,.72f,.51f,.35f,-.94f,.60f,.79f,-.57f,-.70f,.21f,-.68f,-.84f,.10f,.76f,.92f,.60f,-.16f,.85f,-.28f,-.00f,.25f,.61f,.69f,-.62f,.17f,.05f,.08f,-.87f,.23f,.95f,-.82f,.84f,-.58f,.76f,-.46f,-.35f,.76f,-.57f,.70f,-.14f,.55f,.57f,.57f,-.28f,-.18f,.46f,.89f,.07f,.45f,-.65f,-.46f,.01f,.32f,-.96f,-.71f,.05f,.35f,-.90f,-.56f,.72f,-.33f,-.48f,.42f,.32f,.16f,-.55f,-.83f,.42f,.74f,.41f,.93f,.61f,-.39f,-.03f,-.81f,-.37f,-.55f,-.06f,.62f,.69f,.74f,.21f,.12f,-.18f,.25f,-.66f,.40f,-.33f,.95f,.72f,.29f,.71f,-.46f,-.56f,-.79f,.59f,-.71f,-.02f,.61f,.66f,.23f,-.44f,.99f,.66f,-.47f,.99f,.68f,-.21f,-.96f,-.67f,.39f,.22f,-.72f,.01f,.62f,.68f,-.87f,-.92f,-.64f,-.60f,-.51f,-.45f,.72f,-.86f,.99f,-.77f,.36f,.20f,.17f,-.32f,-.06f,.10f,.63f,-.08f,-.01f,.58f,-.31f,.18f,-.09f,.10f,-.72f,.98f,-.76f,-.21f,-.61f,.19f,-.52f,.13f,.39f,-.53f,-.86f,-.91f,-.57f,.61f,.47f,-.73f,.63f,.07f,.90f,-.58f,.52f,.45f,-.51f,-.87f,.31f,.25f,-.08f,.53f,-.73f,-.08f,-.86f,-.63f,.63f,-.40f,-.66f,-.40f,-.71f,-.76f,.66f,-.22f,-.58f,.49f,-.97f,.84f,.96f,-.09f,-.92f,-.24f,.01f,-.97f,-.80f,.00f,-.99f,.62f,.62f,-.32f,.11f,.30f,.80f,.62f,-.57f,.89f,.57f,-.87f,-.95f,.74f,-.57f,-.31f,.05f,-.78f,-.51f,.84f,-.40f,-.97f,-.31f,.89f,-.41f,.15f,.52f,.19f,-.33f,.07f,.74f,.06f,-.64f,-.70f,.58f,-.31f,-.99f,-.46f,.28f,.08f,-.73f,-.31f,.70f,-.90f,.28f,.88f,-.92f,-.21f,-.63f,.47f,.72f,.86f,.95f,.79f,.34f,.16f,.91f,.65f,-.86f,-.81f,.67f,.18f,-.58f,.25f,-.49f,.52f,-.86f,-.25f,.11f,-.97f,.44f,.43f,.64f,.27f,-.64f,-.44f,.96f,-.44f,-.79f,.17f,-.21f,.86f,-.61f,-.16f,-.68f,.95f,.67f,.58f,.10f,.61f,-.99f,-.01f,.86f,-.80f,.35f,-.44f,.84f,.92f,-.35f,.62f,-.40f,.71f,-.38f,.43f,.02f,.74f,-.66f,.68f,.29f,-.79f,.36f,.07f,-.04f,.43f,-.51f,-.47f,.55f,.15f,.82f,-.94f,-.59f,-.85f,-.74f,.85f,.41f,.59f,.53f,.91f,.12f,.92f,-.52f,.41f,.41f,-.25f,.64f,-.45f,.25f,-.56f,-.53f,.16f,-.75f,-.85f,-.16f,-.84f,-.85f,-.57f,-.57f,-.00f,.41f,-.96f,.34f,.47f,-.53f,-.84f,.00f,.39f,-.38f,.87f,-.84f,.56f,-.49f,.32f,.23f,-.04f,-.16f,-.01f,-.47f,-.02f,.57f,-.69f,-.71f,-.71f,.72f,.82f,.65f,.73f,-.31f,.76f,-.64f,.15f,-.50f,-.59f,-.94f,.81f,.78f,-.19f,-.56f,.87f,-.71f,.65f,.43f,.79f,.33f,.07f,.33f,-.77f,.11f,.88f,.47f,.66f,-.62f,.89f,.22f,-.49f,-.71f,-.87f,.77f,-.86f,.08f,-.53f,-.58f,.89f,.57f,-.65f,-.25f,-.48f,-.28f,-.59f,.60f,-.56f,-.06f,.33f,.29f,-.00f,-.66f,.86f,-.90f,-.59f,-.32f,.00f,-.16f,-.59f,.23f,.45f,-.86f,.59f,.37f,-.79f,-.57f,.77f,.34f,-.08f,-.98f,.01f,.69f,.28f,-.84f,.74f,-.02f,.49f,.29f,.07f,.09f,.47f,.87f,.25f,.38f,-.31f,-.12f,-.23f,.23f,-.94f,-.48f,.38f,-.12f,-.66f,-.72f,-.21f,.83f,.78f,-.27f,.81f,.58f,-.93f,-.63f,-.42f,-.16f,-.96f,-.73f,.76f,.32f,-.60f,.50f,.24f,-.48f,-.20f,.99f,-.24f,.59f,.07f,-.80f,-.01f,.45f,-.48f,.34f,-.60f,.15f,-.40f,.68f,.69f,.13f,-.99f,-.21f,-.17f,.29f,-.20f,.86f,-.80f,.82f,-.36f,.79f,.75f,-.37f,-.54f,.16f,-.86f,.36f,-.46f,.33f,-.31f,.75f,-.62f,.79f,-.45f,.34f,-.72f,-.67f,.06f,.17f,.57f,.89f,-.08f,.58f,-.01f,.27f,.05f,-.42f,-.22f,.86f,.92f,-.66f,-.01f,-.62f,.62f,.57f,.17f,.47f,-.50f,-.72f,-.90f,.05f,-.40f,-.77f,-.71f,-.03f,-.81f,.11f,-.34f,-.70f,.05f,.74f,-.27f,-.50f,-.47f,.87f,-.15f,.14f,.31f,-.67f,.74f,-.55f,.41f,-.80f,.84f,-.23f,.10f,-.17f,.65f,.79f,-.67f,.67f,-.17f,.32f,-.06f,.44f,.13f,.26f,-.67f,.63f,-.17f,.40f,.26f,.55f,.29f,.62f,.83f,-.41f,-.07f,.40f,-.40f,-.21f,.45f,-.88f,-.83f,.85f,-.84f,.43f,.44f,.24f,-.32f,.29f,-.62f,-.24f,.38f,-.84f,-.97f,-.68f,.44f,-.70f,-.95f,-.70f,-.39f,-.30f,-.72f,-.55f,-.09f,-.15f,.77f,.40f,-.16f,-.94f,-.24f,-.79f,-.94f,.46f,-.35f,.93f,.10f,.39f,-.37f,-.63f,.03f,-.13f,.64f,.49f,-.52f,.24f,-.59f,.13f,.42f,-.19f,.33f,-.33f,-.67f,.43f,.22f,-.85f,.83f,-.51f,.60f,.15f,-.90f,-.24f,-.39f,.90f,-.87f,.39f,-.70f,-.05f,-.09f,-.95f,-.05f,-.08f,.92f,.95f,-.15f,-.85f,-.87f,.53f,-.31f,.49f,-.59f,.92f,-.82f,-.97f,.40f,-.60f,.80f,-.48f,-.68f,.33f,.73f,-.37f,-.31f,.26f,.62f,.11f,.65f,-.73f,.26f,-.88f,.33f,-.03f,-.54f,-.92f,-.74f,-.80f,-.82f,-.61f,-.97f,-.09f,-.81f,-.83f,-.84f,.02f,-.26f,.60f,-.30f,.41f,-.59f,-.07f,-.70f,.74f,.25f,-.50f,.88f,.73f,-.40f,.49f,.84f,.27f,.80f,-.06f,-.26f,.16f,-.87f,-.86f,.19f,.88f,.00f,.19f,.89f,.94f,.11f,-.09f,.04f,-.15f,-.07f,.56f,.71f,-.94f,-.13f,.72f,.50f,-.17f,-.58f,-.35f,.20f,-.18f,-.31f,-.80f,-.95f,-.00f,.84f,.91f,.34f,.89f,-.08f,.90f,.80f,-.41f,-.50f,.95f,.41f,.52f,-.21f,-.99f,-.77f,.41f,-.32f,-.99f,.21f,.29f,.82f,.99f,.57f,-.46f,.77f,-.43f,-.71f,.13f,-.61f,-.38f,.89f,-.15f,-.46f,-.81f,-.75f,.54f,.45f,.88f,-.25f,-.74f,.38f,.52f,-.93f,.54f,.50f,.52f,.41f,-.64f,.64f,.04f,.33f,.38f,-.97f,.08f,-.90f,.49f,.54f,-.09f,.90f,.66f,.20f,-.75f,-.42f,.64f,.33f,.21f,-.08f,.02f,.92f,-.18f,-.28f,.75f,.13f,.37f,.17f,.89f,.23f,-.47f,.90f,-.24f,.13f,-.81f,.73f,-.12f,.76f,-.54f,.19f,.98f,.81f,.99f,.76f,-.33f,.12f,.22f,.02f,-.43f,-.39f,-.74f,-.79f,.26f,-.14f,.52f,-.96f,-.60f,-.79f,-.53f,.11f,.65f,-.12f,-.92f,-.85f,-.06f,.31f,-.55f,.27f,-.87f,-.46f,-.95f,.23f,-.52f,-.96f,.06f,-.22f,-.44f,-.11f,.34f,-.89f,.43f,-.24f,.48f,-.07f,.08f,-.85f,-.25f,.92f,-.27f,-.03f,-.06f,-.85f,.12f,-.55f,-.70f,-.11f,.70f,.69f,.76f,-.91f,-.63f,-.61f,-.98f,.73f,-.90f,-.62f,.68f,-.87f,-.56f,-.27f,-.70f,.99f,.79f,-.41f,.54f,-.64f,.61f,-.35f,.90f,-.33f,.51f,-.59f,.95f,-.73f,.15f,-.22f,.26f,.72f,.51f,.61f,-.00f,.37f,.58f,.26f,-.13f,.41f,.41f,-.95f,-.79f,.58f,-.20f,-.86f,-.20f,-.60f,-.44f,.11f,-.23f,.91f,.01f,.87f,-.19f,.24f,.01f,-.88f,.09f,.08f,.83f,.02f,-.57f,-.21f,.93f,.91f,.51f,-.74f,.29f,.31f,.41f,.55f,.72f,-.93f,-.47f,.95f,-.73f,-.76f,-.98f,-.84f,.31f,-.89f,-.27f,-.70f,-.22f,.39f,.81f,-.85f,.69f,-.50f,-.53f,-.14f,-.02f,.31f,-.74f,-.19f,.84f,.99f,-.92f,-.35f,-.30f,.85f,-.64f,-.84f,.69f,.77f,.91f,.04f,.94f,.44f,.28f,.13f,-.65f,-.84f,.61f,.01f,.78f,-.17f,.27f,.72f,-.14f,-.18f,.70f,-.19f,.57f,-.89f,.48f,.36f,.39f,.46f,.67f,.16f,-.86f,-.52f,.20f,-.18f,.70f,-.11f,-.57f,-.96f,-.27f,.41f,.72f,-.33f,-.93f,.99f,-.90f,.79f,.17f,-.40f,-.13f,.92f,.74f,-.58f,-.76f,-.93f,.65f,.97f,.27f,-.84f,-.61f,.83f,-.56f,-.76f,-.69f,-.44f,-.84f,-.95f,.33f,.88f,-.70f,.30f,-.69f,.50f,-.65f,-.16f,-.51f,.81f,.13f,-.45f,.35f,-.10f,.31f,-.21f,.39f,.49f,-.66f,.44f,.71f,.87f,-.61f,.86f,-.65f,.87f,-.89f,.70f,-.20f,-.83f,.61f,.43f,-.98f,-.09f,-.20f,-.09f,.99f,.45f,.28f,-.28f,-.46f,.05f,.96f,.66f,.72f,.27f,-.07f,-.27f,.54f,.16f,.54f,-.60f,-.50f,-.94f,-.28f,-.05f,.52f,.65f,-.99f,.50f,-.51f,.28f,-.93f,-.15f,-.64f,-.62f,.36f,-.83f,.86f,.42f,.60f,.59f,.42f,.89f,.50f,.73f,-.41f,-.61f,-.51f,-.32f,-.53f,.64f,.98f,-.94f,-.13f,.06f,.20f,-.18f,-.05f,.50f,-.39f,-.08f,.94f,-.22f,-.01f,.51f,.43f,-.18f,-.40f,.08f,-.40f,.44f,-.45f,-.39f,.39f,.02f,.81f,-.75f,.91f,-.83f,.06f,-.36f,.03f,.11f,-.19f,-.29f,.21f,-.61f,.44f,-.86f,-.69f,-.18f,.51f,.90f,-.05f,.39f,-.89f,-.61f,.66f,-.06f,.90f,.20f,.91f,-.50f,-.63f,.68f,.40f,-.30f,.28f,.98f,-.56f,-.13f,-.12f,.29f,.66f,.07f,.48f,.60f,.09f,.09f,.38f,.63f,-.27f,.35f,-.26f,-.83f,-.63f,-.84f,.87f,-.94f,.07f,.17f,-.98f,.76f,-.98f,.81f,.18f,-.77f,-.80f,-.26f,.76f,.41f,-.40f,-.80f,.16f,.96f,.81f,.47f,-.29f,.57f,-.06f,-.04f,-.30f,-.92f,-.18f,.32f,-.66f,-.90f,.74f,.61f,.37f,.34f,.67f,.08f,.80f,-.27f,-.49f,-.15f,.84f,-.71f,.56f,.24f,.89f,-.00f,-.44f,.67f,-.27f,.31f,-.86f,.52f,.87f,-.03f,-.34f,-.38f,.82f,.84f,-.30f,-.13f,.86f,-.35f,.67f,.45f,-.24f,.95f,.70f,-.83f,-.17f,-.09f,-.35f,-.47f,.71f,-.75f,.33f,.64f,-.68f,.43f,.38f,.48f,.21f,.85f,-.77f,.11f,-.19f,.05f,-.17f,-.67f,-.84f,-.54f,-.54f,-.70f,-.39f,-.52f,-.42f,-.78f,.39f,.70f,-.06f,-.79f,.41f,.94f,.83f,-.33f,.98f,-.14f,-.40f,-.72f,.66f,.79f,.79f,.29f,-.14f,-.34f,.40f,-.95f,.90f,-.38f,-.29f,.95f,.72f,.17f,-.54f,.29f,-.10f,.93f,.69f,-.19f,.20f,.02f,.96f,-.05f,-.49f,-.93f,.32f,-.31f,.07f,.71f,-.58f,.08f,.95f,-.55f,.95f,.05f,.42f,-.98f,.95f,-.09f,-.77f,.42f,.60f,-.16f,.29f,.25f,.39f,-.81f,-.90f,.46f,-.30f,.10f,.07f,.81f,-.57f,-.31f,-.21f,-.12f,.01f,-.11f,.66f,.01f,.90f,-.38f,.52f,.08f,-.77f,-.66f,-.54f,.00f,-.44f,.17f,.38f,-.33f,.80f,-.80f,.15f,-.51f,-.23f,-.52f,.37f,.58f,-.35f,-.71f,-.79f,-.47f,-.69f,-.88f,-.48f,.28f,.71f,.46f,-.28f,-.13f,.64f,-.43f,-.44f,.44f,-.11f,.22f,.10f,-.94f,.42f,-.15f,.25f,-.25f,-.88f,-.29f,.92f,-.13f,.06f,-.35f,-.66f,-.77f,.02f,-.55f,.51f,-.57f,.50f,.68f,-.22f,.23f,.00f,-.66f,-.36f,.44f,.25f,.28f,.96f,.68f,-.22f,-.33f,-.67f,.11f,-.33f,-.15f,-.16f,-.87f,-.37f,.18f,.62f,-.25f,-.13f,.65f,-.03f,-.00f,.30f,.77f,.63f,.57f,.93f,-.37f,-.06f,-.72f,.39f,.36f,-.28f,.53f,.00f,.71f,-.57f,-.64f,.30f,-.40f,-.30f,.05f,-.90f,.36f,.87f,.77f,.03f,-.41f,-.91f,.81f,.16f,-.12f,-.18f,.30f,-.51f,.26f,-.46f,-.62f,.79f,-.98f,-.60f,-.90f,.01f,-.52f,.66f,-.49f,.75f,-.68f,-.78f,-.70f,.44f,.07f,-.90f,-.50f,.22f,.04f,-.54f,.35f,-.38f,.18f,-.04f,-.09f,-.83f,-.16f,-.99f,-.78f,-.73f,.71f,.27f,.15f,.19f,.29f,-.20f,.00f,.38f,.84f,-.20f,.28f,.19f,-.02f,-.50f,-.66f,-.23f,.79f,.62f,.20f,-.24f,.60f,-.39f,-.61f,-.53f,-.93f,.99f,.78f,-.92f,.92f,.43f,.46f,.87f,.32f,-.53f,.85f,-.73f,-.71f,-.39f,-.90f,-.63f,.01f,-.38f,-.62f,-.25f,-.37f,.60f,-.01f,.18f,.95f,.99f,-.70f,-.76f,-.15f,-.78f,.02f,.08f,.27f,-.61f,.62f,.59f,-.04f,.46f,-.63f,-.05f,.85f,-.05f,-.84f,-.35f,-.35f,.63f,.55f,-.33f,-.53f,.62f,.64f,-.02f,.06f,-.39f,-.55f,-.09f,-.17f,-.09f,-.84f,.94f,-.08f,-.95f,-.25f,.12f,-.26f,-.94f,.32f,-.60f,-.11f,-.27f,-.32f,.31f,.97f,.64f,.99f,-.99f,-.56f,.96f,.11f,-.57f,-.51f,.82f,-.35f,.13f,.70f,-.97f,.92f,.59f,-.03f,.15f,-.09f,.46f,.28f,.73f,-.17f,.26f,-.23f,.45f,-.96f,.21f,-.28f,-.64f,-.09f,.68f,.49f,-.30f,-.75f,.98f,-.71f,-.42f,-.52f,.19f,.52f,.30f,.92f,.61f,-.53f,.62f,-.15f,.19f,.07f,.67f,.92f,.20f,-.69f,-.29f,-.75f,.54f,.78f,.92f,-.70f,-.70f,-.40f,-.41f,.89f,-.29f,.02f,.34f,-.24f,.48f,.97f,.75f,-.66f,.17f,-.32f,-.41f,.24f,.85f,-.70f,.39f,.46f,-.84f,.86f,.45f,-.78f,-.61f,.74f,.73f,.78f,.66f,-.15f,-.85f,.46f,.46f,-.21f,-.78f,-.16f,.12f,.83f,.05f,-.95f,.41f,.45f,-.02f,.56f,-.91f,-.90f,-.33f,-.94f,-.72f,-.90f,-.22f,-.01f,-.59f,.58f,-.37f,.23f,-.96f,.41f,-.65f,.69f,-.79f,.14f,.39f,-.99f,-.21f,-.73f,-.33f,.53f,-.15f,-.04f,-.00f,.33f,.12f,-.44f,-.88f,-.93f,.05f,.73f,-.66f,-.85f,-.17f,-.47f,.15f,.65f,.91f,-.72f,.32f,.64f,-.30f,-.40f,-.79f,-.77f,.27f,.09f,.65f,-.85f,.37f,.57f,-.79f,.28f,.02f,-.92f,.23f,.66f,-.38f,.24f,-.21f,.84f,-.71f,-.12f,.26f,-.29f,.26f,-.08f,.75f,.54f,.46f,.11f,-.81f,-.53f,.85f,-.24f,-.38f,.95f,.36f,.77f,-.37f,.58f,.91f,.03f,.13f,-.98f,.68f,.74f,.52f,.42f,.83f,-.76f,.52f,-.79f,-.57f,.95f,-.58f,-.11f,-.08f,.57f,.75f,-.80f,.33f,.42f,-.85f,-.74f,-.91f,-.99f,-.08f,-.06f,-.42f,.33f,-.45f,-.09f,.56f,-.95f,.98f,-.24f,.48f,-.85f,-.05f,.73f,.75f,.43f,.00f,-.29f,-.75f,-.39f,-.76f,.35f,.29f,-.19f,-.61f,-.65f,-.35f,.74f,-.12f,.62f,.59f,-.11f,-.36f,-.37f,-.59f,.28f,-.28f,-.00f,.60f,-.40f,.10f,-.94f,-.25f,-.50f,-.12f,-.90f,.13f,-.39f,.30f,-.87f,.76f,-.61f,.55f,.17f,.74f,-.36f,.52f,.71f,-.20f,.07f,.40f,.61f,.06f,.68f,.71f,-.36f,-.52f,-.61f,-.70f,-.40f,.00f,.69f,-.44f,-.15f,.45f,-.63f,.88f,.23f,-.98f,.29f,-.86f,-.24f,.65f,-.61f,-.61f,.81f,.10f,.24f,.84f,-.98f,-.26f,-.17f,-.05f,.88f,-.93f,-.34f,.10f,.89f,-.92f,.88f,-.01f,.30f,.01f,.65f,-.87f,-.51f,.42f,.60f,.02f,.56f,-.56f,-.00f,.63f,.40f,.32f,-.12f,.05f,.61f,-.57f,-.84f,.20f,-.91f,.48f,.64f,-.60f,-.24f,.84f,.21f,-.48f,.26f,-.71f,.31f,.21f,.59f,.90f,-.38f,.56f,-.72f,.22f,.01f,.74f,.19f,-.40f,-.63f,.76f,-.67f,.27f,.09f,-.04f,.49f,.33f,.55f,.73f,-.41f,-.49f,.16f,-.93f,.37f,.62f,.68f,-.51f,.08f,.89f,-.35f,.26f,.77f,.33f,.42f,-.66f,.65f,-.34f,.87f,-.97f,-.16f,.43f,-.93f,-.06f,.00f,.40f,.06f,-.16f,.79f,.46f,-.60f,-.68f,.11f,-.19f,.33f,-.37f,-.94f,-.08f,.11f,.20f,.78f,.34f,-.61f,.60f,-.93f,-.36f,.96f,-.37f,-.79f,-.56f,.86f,.36f,-.68f,-.18f,.54f,.14f,.00f,.23f,.14f,.83f,-.45f,.01f,.59f,-.02f,.43f,-.03f,.97f,.62f,-.12f,.90f,-.90f,-.08f,.36f,-.72f,.02f,.22f,-.24f,.96f,-.33f,-.13f,.44f,-.65f,.95f,.68f,-.76f,.18f,-.06f,.26f,.74f,-.38f,-.08f,.15f,.45f,-.97f,.43f,-.82f,-.84f,.99f,-.55f,-.93f,-.07f,-.29f,-.59f,-.91f,.49f,-.60f,-.56f,.63f,.86f,.49f,.76f,.69f,.12f,.67f,.60f,.12f,-.79f,.78f,.35f,.52f,-.80f,-.12f,.25f,.43f,.69f,.12f,-.01f,.47f,.65f,-.38f,.99f,.28f,-.37f,-.18f,.69f,.56f,.80f,.89f,-.13f,.57f,.45f,-.11f,.62f,.83f,.14f,-.80f,-.12f,-.04f,.25f,.59f,-.17f,.04f,-.75f,.75f,-.56f,-.46f,-.27f,.22f,.53f,.58f,.03f,.44f,-.14f,.26f,-.22f,.74f,.28f,.50f,-.21f,-.51f,-.50f,.73f,-.67f,.19f,-.58f,-.29f,.56f,-.77f,.18f,-.46f,.59f,.80f,.95f,-.66f,-.67f,-.57f,.01f,-.20f,.27f,.67f,-.12f,-.34f,.72f,-.05f,.95f,-.26f,.30f,.87f,-.61f,.91f,-.68f,.20f,-.75f,-.04f,.12f,.37f,-.34f,.06f,-.92f,-.76f,-.04f,-.17f,-.73f,.50f,-.32f,-.77f,-.25f,-.72f,-.32f,-.88f,.97f,.63f,-.05f,.05f,.33f,.46f,.65f,-.38f,.36f,.52f,-.33f,-.09f,.57f,-.44f,-.80f,.18f,-.14f,.83f,.53f,.45f,-.82f,.11f,.24f,.56f,-.47f,-.35f,-.33f,.97f,-.77f,-.42f,.61f,-.45f,.57f,.43f,.69f,.46f,.22f,.10f,-.24f,.52f,.55f,-.97f,.09f,-.52f,.78f,.50f,.87f,-.04f,.21f,.92f,.15f,-.85f,-.64f,.25f,.20f,.35f,.85f,-.95f,-.36f,-.75f,.85f,.62f,.04f,-.87f,.41f,-.07f,-.71f,-.30f,.07f,.37f,-.16f,.39f,-.62f,-.02f,.57f,-.37f,.31f,-.55f,-.41f,-.11f,-.32f,-.19f,.31f,.29f,.44f,.09f,-.54f,.38f,.07f,.10f,-.71f,-.97f,-.48f,-.55f,.49f,-.95f,.92f,.69f,.24f,-.82f,.33f,-.03f,-.20f,-.14f,-.77f,-.16f,.57f,.39f,.84f,-.73f,-.48f,.65f,.33f,.24f,-.56f,.31f,.23f,.44f,-.97f,-.87f,.14f,-.73f,-.02f,.18f,-.09f,-.02f,-.27f,.09f,.96f,-.01f,-.79f,.29f,-.34f,.99f,.87f,-.45f,.55f,-.25f,-.69f,-.75f,-.45f,-.27f,-.00f,-.99f,.26f,.07f,-.60f,-.50f,-.11f,.20f,.01f,-.09f,-.56f,.85f,.96f,.02f,.09f,-.63f,.13f,.61f,.46f,-.97f,.29f,.09f,-.41f,-.73f,.43f,-.49f,-.33f,-.35f,-.36f,.63f,.41f,-.57f,-.39f,.65f,-.03f,-.29f,.06f,.04f,.97f,-.50f,-.54f,.86f,-.41f,.57f,-.12f,.51f,-.41f,.77f,-.89f,-.74f,.45f,.99f,.03f,.96f,.48f,.47f,-.71f,-.85f,.62f,-.83f,-.71f,-.87f,.97f,-.41f,-.20f,-.84f,.55f,-.01f,-.76f,.15f,.55f,.26f,.19f,.70f,.07f,-.56f,-.51f,.48f,-.55f,.14f,.53f,-.53f,-.99f,-.22f,-.61f,-.88f,-.26f,.46f,-.19f,-.22f,.20f,-.02f,.71f,-.82f,-.67f,.80f,.31f,-.58f,-.59f,-.63f,.48f,-.73f,.45f,.49f,.74f,.59f,.72f,-.48f,.18f,-.12f,.30f,.81f,.55f,.97f,-.66f,-.52f,-.99f,.14f,-.62f,-.02f,.90f,.78f,.77f,-.05f,-.06f,.51f,-.26f,.64f,.91f,.66f,.52f,.19f,.21f,-.52f,.96f,.71f,.28f,-.89f,-.01f,.83f,.07f,.39f,-.88f,.28f,.68f,-.81f,-.46f,.42f,.47f,-.43f,-.93f,.43f,-.76f,.15f,.88f,.21f,.30f,-.85f,-.26f,-.12f,-.96f,-.84f,-.97f,-.49f,.64f,.42f,-.50f,.87f,.19f,-.13f,-.46f,-.45f,-.95f,.22f,.41f,.83f,-.14f,-.33f,.87f,.76f,-.06f,-.94f,.72f,.73f,-.78f,.90f,.89f,.84f,.90f,.28f,.88f,.86f,-.62f,.02f,-.81f,-.52f,-.82f,-.14f,-.42f,-.30f,-.02f,.79f,.10f,-.58f,-.00f,.95f,.55f,-.75f,-.09f,.36f,.44f,.30f,.03f,-.49f,-.60f,.95f,.86f,-.67f,-.23f,.28f,-.45f,-.58f,-.52f,.77f,-.19f,-.88f,-.16f,.08f,.82f,-.02f,.45f,-.66f,.73f,.94f,-.55f,-.54f,.02f,.62f,.71f,-.99f,.10f,-.06f,.08f,.79f,-.00f,-.77f,-.47f,.42f,.83f,.21f,.08f,-.43f,-.14f,.22f,.32f,-.03f,.11f,.12f,-.64f,-.58f,-.54f,-.08f,-.38f,.41f,.61f,-.65f,.27f,.98f,-.40f,-.13f,.83f,.74f,-.68f,.22f,-.69f,.42f,.77f,-.87f,-.42f,-.77f,-.11f,.25f,-.69f,.84f,-.25f,-.38f,-.56f,-.20f,-.35f,.83f,-.89f,-.51f,.82f,-.77f,.05f,.78f,.71f,.12f,-.82f,-.32f,.09f,.63f,.70f,-.49f,.80f,.25f,-.85f,.04f,-.72f,.56f,.14f,.62f,.17f,-.88f,-.26f,.20f,.72f,-.55f,-.57f,-.33f,.66f,-.12f,.46f,-.14f,-.87f,-.12f,.08f,.92f,.41f,.36f,-.58f,-.17f,.25f,-.97f,-.87f,-.86f,-.43f,.85f,-.38f,-.45f,.47f,.41f,.98f,-.67f,-.13f,-.36f,.08f,-.33f,.92f,-.96f,-.13f,.71f,.77f,.17f,-.03f,-.92f,-.06f,.00f,.29f,-.14f,-.79f,.64f,.53f,.82f,-.02f,-.39f,.94f,-.97f,-.67f,-.98f,-.38f,-.25f,-.76f,.21f,.80f,.24f,-.69f,-.44f,-.80f,.17f,-.87f,.33f,.72f,-.23f,-.39f,.52f,-.39f,-.14f,-.70f,.86f,-.58f,.05f,.82f,.19f,.58f,-.76f,.00f,.09f,.97f,-.60f,.90f,-.73f,-.47f,-.47f,.18f,.93f,-.53f,.71f,.34f,.48f,-.12f,.94f,.12f,-.44f,-.35f,-.93f,-.00f,-.91f,.40f,.37f,-.87f,-.86f,.68f,.55f,.35f,-.30f,.31f,-.37f,.81f,.89f,-.00f,-.72f,-.24f,-.33f,-.09f,-.79f,.41f,.15f,-.30f,-.99f,-.00f,-.07f,-.55f,.22f,.54f,-.15f,-.45f,-.79f,-.75f,-.50f,-.69f,-.59f,-.67f,.70f,.29f,.74f,.83f,-.83f,.41f,-.04f,-.94f,.59f,-.38f,.85f,.34f,-.06f,.08f,.79f,.41f,-.19f,-.15f,.25f,-.66f,.03f,.57f,-.17f,.69f,.32f,.07f,-.95f,.49f,.42f,-.75f,.70f,-.09f,-.98f,-.93f,.40f,-.38f,.36f,-.49f,.65f,-.30f,.14f,-.99f,.66f,-.27f,.94f,.08f,-.70f,.78f,-.93f,-.97f,-.89f,-.94f,-.77f,-.75f,-.87f,.89f,-.25f,.61f,.59f,-.55f,-.73f,.26f,.93f,-.42f,.66f,-.91f,-.72f,.05f,.26f,-.49f,-.00f,-.88f,-.66f,.34f,.93f,.91f,.94f,.25f,.72f,.13f,-.23f,-.67f,-.99f,-.77f,.45f,.33f,.68f,.34f,.43f,.60f,.36f,-.72f,-.56f,-.41f,-.72f,.65f,.34f,.91f,.95f,-.90f,-.85f,.91f,.80f,-.60f,-.96f,.29f,-.36f,-.54f,.81f,-.95f,-.54f,.67f,.13f,.16f,.73f,.92f,.00f,.08f,-.05f,.84f,-.42f,-.82f,-.31f,-.82f,-.14f,-.65f,-.15f,.55f,.56f,-.93f,-.06f,.59f,.48f,.60f,.08f,-.81f,-.72f,-.69f,-.19f,.02f,.25f,-.38f,-.41f,.31f,-.84f,-.09f,.64f,.62f,-.00f,.78f,-.87f,-.50f,-.85f,.61f,-.90f,.96f,.51f,.99f,-.33f,-.44f,-.23f,-.27f,-.03f,-.75f,.12f,-.78f,-.25f,.44f,.49f,.33f,.33f,-.77f,-.35f,-.39f,-.49f,.00f,-.20f,-.06f,-.78f,.41f,.31f,.62f,-.78f,-.36f,.98f,-.03f,-.41f,-.01f,-.81f,.30f,-.81f,.60f,-.05f,.20f,.70f,.81f,.64f,.78f,.84f,.07f,.86f,.72f,.59f,-.58f,-.85f,.23f,.03f,-.40f,-.29f,-.16f,.59f,.79f,.07f,-.29f,.45f,-.94f,-.48f,-.65f,.57f,.43f,-.31f,-.18f,-.83f,-.85f,-.27f,.28f,.58f,-.07f,.74f,.09f,-.80f,.93f,-.84f,.74f,.37f,.85f,.70f,-.95f,.35f,-.81f,-.54f,.81f,.05f,.02f,.47f,-.70f,.45f,.44f,-.12f,.46f,-.76f,.42f,-.38f,-.47f,-.80f,-.96f,-.97f,-.95f,-.91f,.79f,-.41f,-.87f,.60f,.72f,.19f,.45f,.99f,-.02f,-.70f,-.89f,-.75f,-.54f,.11f,-.69f,.94f,-.86f,-.34f,-.79f,-.75f,.84f,-.56f,.63f,.22f,-.69f,.34f,.34f,.38f,.03f,.43f,-.38f,-.70f,-.34f,-.01f,-.47f,-.88f,.97f,-.99f,.33f,-.79f,-.79f,-.78f,-.14f,-.58f,-.93f,.41f,.14f,.40f,-.06f,.31f,-.69f,-.83f,.29f,.08f,.66f,.71f,.11f,.28f,.98f,.94f,.11f,.91f,.05f,-.07f,.33f,-.97f,.48f,-.77f,.93f,.68f,-.00f,.36f,.37f,-.76f,.51f,.55f,-.92f,.15f,.84f,-.82f,.02f,-.15f,.06f,.05f,-.10f,-.57f,.93f,.03f,-.61f,.47f,-.23f,.39f,-.54f,.33f,-.48f,.45f,.65f,.64f,.35f,.64f,-.39f,.15f,.54f,.97f,-.12f,-.88f,.57f,.95f,.91f,.83f,.64f,-.16f,-.85f,.52f,.03f,.81f,.10f,.47f,.39f,-.66f,.86f,.69f,.29f,-.43f,-.76f,.58f,-.76f,-.89f,-.12f,.37f,.83f,-.02f,.36f,.79f,-.29f,.63f,.18f,-.24f,.62f,-.07f,-.25f,.07f,-.17f,-.60f,-.66f,-.21f,.83f,.04f,-.63f,-.35f,.00f,.22f,.05f,-.18f,.26f,.49f,.51f,.54f,-.43f,.10f,.07f,-.70f,.69f,-.19f,-.24f,.94f,.22f,.16f,-.23f,-.81f,-.72f,.58f,.44f,-.58f,-.89f,.59f,.64f,.72f,.43f,.82f,-.57f,.69f,.27f,-.20f,.24f,.07f,.41f,-.49f,.72f,.27f,-.40f,.11f,-.58f,.78f,-.71f,.97f,.12f,.00f,.76f,.77f,-.70f,-.13f,-.41f,.57f,.42f,.48f,.59f,.47f,.16f,-.10f,.80f,.53f,.50f,-.34f,.78f,-.20f,-.70f,.56f,-.23f,-.11f,-.51f,.44f,.21f,-.59f,.08f,-.43f,-.87f,-.60f,-.35f,.79f,-.62f,-.85f,-.39f,-.39f,-.74f,.80f,-.19f,-.93f,.55f,-.26f,-.86f,-.62f,-.00f,-.20f,-.11f,.95f,-.27f,.59f,.61f,-.78f,-.64f,-.09f,-.51f,.10f,.83f,-.22f,.37f,.40f,-.74f,-.68f,-.22f,-.36f,-.82f,.29f,.29f,-.04f,.88f,-.62f,.49f,.89f,-.87f,.30f,.90f,.54f,.07f,-.52f,-.82f,.65f,.28f,.73f,.43f,.34f,.24f,-.61f,-.20f,-.07f,.85f,.34f,-.33f,.14f,.64f,-.58f,.32f,-.46f,.07f,.88f,.64f,.12f,-.26f,-.11f,.72f,-.58f,-.58f,-.95f,.38f,.11f,-.01f,.15f,-.10f,.50f,.66f,.60f,-.81f,.75f,.48f,.65f,-.83f,-.81f,-.92f,-.31f,.81f,-.35f,.49f,.95f,-.99f,.17f,.34f,.60f,.65f,-.49f,-.18f,-.09f,.35f,.06f,.92f,.78f,-.18f,-.66f,.46f,.58f,-.56f,-.45f,.63f,.03f,.28f,-.84f,.39f,.06f,.85f,.04f,-.50f,.53f,.62f,.46f,-.86f,-.90f,-.72f,.12f,.05f,-.39f,-.41f,.59f,-.19f,-.22f,.50f,.32f,.83f,-.33f,-.83f,-.81f,-.46f,-.63f,-.58f,-.78f,-.49f,-.78f,.19f,.69f,-.66f,.27f,-.10f,-.09f,.88f,.08f,.53f,-.64f,-.38f,.46f,-.64f,-.55f,.33f,-.41f,-.80f,.11f,.37f,-.05f,-.73f,-.57f,.93f,.90f,-.60f,.93f,.81f,.94f,.23f,.06f,.68f,-.06f,-.91f,-.05f,.40f,-.11f,-.83f,-.67f,.96f,-.89f,-.56f,.96f,.82f,-.29f,.80f,.42f,.23f,-.05f,-.06f,.16f,-.54f,.08f,.69f,.97f,.19f,-.65f,.87f,.70f,.67f,.44f,.10f,-.12f,.97f,.07f,.92f,-.00f,-.58f,-.44f,.66f,-.63f,-.32f,.08f,.53f,-.11f,.80f,-.35f,.98f,-.66f,-.63f,-.80f,.87f,-.82f,.09f,.95f,-.25f,-.15f,.72f,-.97f,-.45f,.96f,.49f,.92f,-.19f,-.10f,-.12f,-.00f,-.96f,.76f,.99f,-.37f,.84f,.66f,-.63f,-.41f,.37f,-.29f,.69f,.61f,.54f,-.95f,.97f,.76f,.02f,.06f,.83f,-.94f,-.77f,.04f,-.25f,.57f,.75f,.46f,-.96f,-.49f,.76f,.35f,-.80f,-.74f,.52f,-.57f,-.38f,-.75f,-.88f,.76f,-.69f,.22f,-.64f,.65f,.79f,-.79f,.78f,-.13f,.07f,-.98f,.63f,.94f,-.54f,.52f,.02f,-.64f,-.05f,.04f,-.91f,-.05f,-.54f,.67f,.44f,-.43f,-.37f,.03f,.37f,-.51f,-.36f,.21f,.65f,-.62f,.20f,-.90f,-.37f,-.11f,-.40f,-.11f,.50f,.81f,.46f,.03f,-.09f,.05f,-.20f,.98f,.45f,.26f,-.15f,-.40f,.70f,-.17f,-.59f,-.23f,-.80f,.65f,-.47f,.39f,.68f,-.64f,.15f,-.89f,.98f,-.36f,.15f,-.59f,.88f,.47f,-.01f,.26f,-.94f,.39f,-.76f,-.43f,-.29f,-.88f,-.52f,-.42f,.12f,-.56f,.09f,.38f,.32f,-.22f,-.29f,-.46f,.59f,-.96f,.30f,.95f,-.76f,-.90f,-.76f,.56f,.35f,-.76f,.42f,-.72f,-.67f,.74f,-.60f,.90f,.74f,-.87f,-.36f,-.12f,.29f,-.83f,.60f,.35f,.44f,-.50f,.19f,.19f,.42f,-.47f,-.50f,-.35f,.49f,.41f,-.19f,-.08f,-.70f,.76f,-.44f,.69f,.16f,-.01f,.56f,-.85f,-.11f,-.87f,.11f,.17f,-.05f,.50f,.24f,-.47f,.57f,.59f,-.90f,.95f,-.05f,.02f,.51f,.82f,.28f,.91f,-.59f,.65f,-.42f,-.82f,.84f,-.31f,-.41f,.75f,.68f,-.67f,.52f,-.18f,.79f,-.38f,.63f,.33f,-.93f,-.59f,.76f,-.60f,.03f,-.77f,-.34f,.17f,-.35f,.81f,.50f,.82f,.57f,.61f,-.38f,.69f,.69f,.75f,-.61f,-.73f,.53f,.66f,-.83f,-.69f,.67f,-.41f,.84f,-.23f,.70f,.95f,.66f,.81f,.64f,-.55f,-.80f,.45f,-.11f,-.77f,-.47f,-.36f,-.41f,-.82f,-.60f,-.32f,-.58f,.49f,.32f,-.38f,.35f,-.88f,-.11f,.49f,.24f,.97f,.68f,.90f,.62f,-.40f,.94f,-.16f,-.57f,-.00f,-.18f,.55f,-.88f,.95f,-.49f,-.75f,.54f,.45f,.03f,.59f,.17f,-.22f,-.44f,-.63f,.69f,.08f,.06f,-.43f,.00f,-.50f,-.86f,-.48f,.91f,-.19f,-.18f,-.61f,.85f,-.41f,-.73f,.83f,-.86f,.66f,.94f,-.02f,-.07f,.21f,-.68f,-.33f,-.40f,.13f,-.11f,.59f,-.91f,-.28f,.72f,.18f,.46f,-.20f,-.36f,-.93f,-.38f,.91f,-.03f,-.85f,-.05f,.23f,-.03f,-.28f,.68f,.93f,.07f,.69f,-.76f,-.55f,.42f,-.27f,-.28f,.39f,-.02f,.95f,-.43f,.38f,.70f,.93f,-.55f,-.25f,-.72f,-.44f,.97f,-.72f,-.24f,-.66f,-.99f,-.29f,-.93f,.97f,.64f,.65f,.95f,.58f,.13f,.18f,.42f,-.38f,.11f,.60f,.30f,-.01f,.79f,-.31f,-.70f,.23f,-.17f,-.58f,-.82f,.13f,-.75f,-.44f,.99f,.26f,-.55f,.12f,-.40f,.85f,-.93f,.28f,-.78f,-.94f,.99f,-.71f,.55f,-.30f,.20f,.33f,-.84f,-.93f,-.49f,-.83f,-.30f,-.44f,-.14f,.96f,-.71f,-.68f,.60f,.74f,-.86f,-.10f,-.04f,.07f,-.30f,.28f,-.10f,.15f,-.34f,.94f,.71f,.17f,-.48f,-.53f,.93f,.73f,.29f,-.17f,-.34f,-.61f,-.93f,.94f,-.14f,.27f,.49f,.78f,-.69f,.36f,.38f,.49f,-.93f,.83f,-.85f,.51f,.43f,.61f,.40f,-.95f,-.81f,.09f,-.87f,-.42f,-.85f,.34f,-.43f,.72f,-.40f,-.51f,-.39f,-.50f,-.74f,.91f,.49f,.92f,.84f,.68f,-.63f,-.77f,.25f,.63f,-.60f,.32f,-.94f,.69f,-.81f,.55f,.07f,.58f,-.76f,-.82f,.53f,-.87f,-.97f,-.92f,-.19f,.12f,-.70f,.05f,.99f,-.88f,-.76f,-.81f,-.81f,.94f,-.22f,.44f,.10f,-.09f,.34f,-.72f,-.19f,-.44f,.29f,.74f,.45f,.77f,.36f,.37f,-.59f,.46f,.41f,-.86f,-.10f,-.42f,.66f,.11f,.50f,.17f,.78f,.92f,.93f,.64f,.61f,.53f,-.51f,.75f,-.89f,-.01f,.50f,-.27f,-.40f,.06f,.66f,-.02f,-.70f,-.13f,.03f,-.72f,-.64f,.67f,.12f,-.97f,.97f,-.74f,-.56f,-.11f,.81f,-.13f,-.32f,.06f,.10f,.77f,-.80f,-.68f,-.14f,-.53f,-.70f,-.18f,-.37f,-.61f,.12f,.65f,.40f,-.16f,-.92f,.92f,.94f,-.20f,-.24f,.33f,-.75f,.83f,.67f,-.10f,-.18f,.84f,-.23f,-.80f,.77f,-.80f,.60f,-.51f,.64f,-.63f,.89f,-.61f,.00f,.43f,.86f,.23f,-.51f,.05f,-.23f,.05f,.32f,.36f,.78f,.12f,-.52f,.52f,-.86f,-.37f,-.63f,.50f,-.82f,-.93f,.49f,-.20f,-.35f,.20f,-.03f,.09f,.01f,.26f,.14f,.82f,.15f,.26f,-.83f,-.22f,.36f,.26f,-.40f,.90f,.87f,-.96f,.87f,.73f,.67f,.08f,.46f,.59f,.59f,.82f,.99f,-.13f,.18f,-.04f,.60f,-.60f,.84f,-.80f,-.54f,.40f,-.57f,-.43f,-.08f,-.09f,.62f,.58f,.89f,-.38f,.07f,-.72f,-.15f,.20f,.34f,-.81f,-.04f,-.06f,.77f,-.95f,-.99f,-.05f,-.13f,-.37f,-.21f,-.31f,-.77f,-.82f,.67f,.71f,.03f,.98f,.46f,-.87f,.55f,-.14f,.14f,.26f,.05f,.31f,.67f,-.00f,-.49f,-.71f,-.75f,-.66f,.27f,-.29f,.04f,-.91f,.02f,-.68f,-.03f,-.47f,.34f,-.89f,-.98f,-.00f,.11f,-.52f,.28f,-.42f,.60f,.28f,.42f,.91f,-.09f,.23f,-.44f,-.39f,-.20f,.61f,.82f,-.43f,.57f,.15f,-.03f,-.71f,-.19f,.56f,.54f,.99f,-.59f,-.76f,-.90f,.05f,.00f,-.56f,-.85f,-.74f,.31f,-.92f,.38f,.13f,.44f,.94f,.95f,.35f,.18f,.35f,-.16f,.15f,.21f,-.90f,.80f,.50f,-.68f,-.20f,-.99f,.20f,.71f,.65f,.46f,.21f,.19f,-.64f,.79f,-.23f,.69f,-.71f,.29f,-.55f,.61f,-.31f,.18f,-.00f,.40f,-.89f,-.38f,-.08f,-.43f,-.87f,.53f,-.05f,.55f,.80f,-.60f,-.87f,-.15f,-.22f,-.55f,-.41f,-.43f,.69f,.32f,-.10f,-.29f,.26f,.88f,-.86f,-.16f,-.36f,-.94f,.47f,-.75f,-.15f,-.10f,.97f,.93f,.39f,.29f,.12f,.49f,-.38f,.22f,.24f,-.33f,.18f,.37f,-.80f,-.77f,.28f,-.26f,.05f,.60f,-.85f,-.14f,.90f,-.44f,.36f,-.85f,-.89f,.36f,-.93f,.44f,-.95f,-.18f,.62f,-.50f,-.58f,-.08f,.84f,.33f,-.96f,.92f,.41f,-.61f,.90f,.09f,-.70f,-.73f,-.64f,-.09f,-.09f,-.42f,-.90f,.61f,.71f,.82f,-.42f,.56f,.97f,.64f,-.49f,-.22f,-.94f,.24f,-.29f,-.88f,-.54f,.97f,-.76f,.66f,-.71f,-.64f,-.64f,-.53f,.20f,.23f,-.56f,-.62f,-.55f,.18f,.18f,-.77f,.19f,-.68f,.89f,.50f,.42f,-.10f,-.28f,.41f,-.39f,-.51f,.70f,-.98f,.44f,.04f,-.46f,-.63f,-.93f,-.31f,.41f,.35f,.53f,.77f,.60f,.26f,-.21f,.82f,-.92f,.92f,-.70f,-.10f,.64f,-.83f,-.55f,.61f,.77f,-.72f,-.30f,-.85f,.33f,-.60f,.19f,.86f,.70f,.77f,.21f,.86f,.04f,.22f,.95f,.57f,.36f,-.41f,.37f,-.17f,.29f,.70f,-.40f,-.39f,-.83f,-.11f,.84f,-.43f,-.48f,.13f,-.27f,-.75f,-.54f,.79f,.75f,-.85f,.90f,-.41f,.24f,.29f,.61f,.69f,-.03f,-.72f,-.36f,-.33f,-.32f,-.11f,.19f,-.84f,-.24f,-.77f,-.96f,.70f,.04f,.95f,-.97f,-.11f,-.24f,.69f,-.13f,.85f,.43f,-.41f,-.89f,.70f,.09f,-.63f,.57f,.80f,.56f,-.62f,.32f,-.28f,.54f,.14f,-.67f,-.24f,-.79f,.23f,-.95f,-.16f,-.85f,-.18f,.46f,-.60f,.36f,.22f,-.08f,.15f,.62f,-.20f,.88f,-.26f,-.13f,-.11f,.56f,.17f,.67f,-.56f,.49f,.87f,-.35f,-.73f,-.93f,-.80f,-.06f,-.95f,-.36f,-.26f,.34f,.19f,.57f,.56f,-.65f,.60f,.94f,.86f,-.16f,-.95f,.36f,.12f,-.42f,.34f,.75f,-.14f,.15f,.03f,-.17f,.70f,.95f,.95f,.79f,.11f,-.82f,-.30f,.75f,-.72f,-.91f,.31f,-.07f,.42f,.85f,.00f,-.47f,.37f,-.62f,.22f,.10f,.75f,-.15f,-.13f,.18f,-.21f,.45f,-.35f,.77f,-.40f,-.75f,.59f,.87f,-.62f,-.91f,.55f,-.09f,.06f,.30f,.52f,-.51f,-.03f,.89f,-.05f,.88f,.83f,-.66f,-.01f,.34f,-.83f,.60f,.47f,.65f,.51f,.70f,.62f,.99f,.34f,.41f,-.31f,.75f,.61f,.04f,.54f,-.42f,.47f,-.43f,.39f,-.58f,-.44f,-.02f,-.08f,.38f,-.77f,-.55f,-.20f,.63f,-.93f,-.96f,-.04f,-.26f,.83f,.34f,.31f,-.32f,.37f,-.74f,.60f,-.77f,-.04f,.65f,.94f,-.39f,.01f,-.76f,-.25f,.44f,.87f,.06f,-.58f,-.21f,.67f,-.97f,.22f,.24f,-.16f,.82f,-.19f,.66f,.00f,.50f,-.85f,.32f,-.66f,.11f,-.48f,.66f,-.60f,-.50f,.29f,-.93f,.94f,-.02f,.41f,-.13f,-.04f,.77f,.31f,-.23f,.84f,.25f,-.48f,-.51f,.19f,-.42f,-.03f,-.13f,.78f,.76f,.23f,.63f,.23f,-.82f,.41f,.10f,.73f,.49f,-.94f,.80f,-.40f,.78f,-.12f,-.49f,-.80f,.37f,.96f,.22f,-.93f,.78f,-.74f,-.71f,.46f,-.38f,.86f,.03f,.48f,.22f,-.05f,.13f,.91f,-.20f,.36f,.64f,.26f,.38f,-.10f,.57f,-.14f,-.72f,.08f,.55f,-.91f,.62f,.27f,-.49f,-.22f,.22f,-.12f,-.32f,-.28f,.15f,-.92f,.58f,-.51f,-.07f,-.45f,-.24f,-.73f,.63f,-.70f,-.11f,-.46f,-.75f,-.58f,-.15f,.38f,-.22f,.23f,-.26f,.41f,.14f,.87f,.44f,.26f,.79f,.28f,.19f,.06f,.84f,.19f,.91f,-.25f,.24f,.07f,-.46f,-.57f,.86f,.98f,-.33f,.77f,.04f,.55f,-.04f,-.72f,-.44f,.36f,-.73f,.74f,-.86f,.71f,.89f,-.94f,-.03f,.07f,.12f,-.14f,-.20f,-.71f,-.51f,-.23f,-.35f,-.89f,.75f,-.95f,-.48f,-.34f,-.16f,.45f,.34f,-.50f,-.82f,.12f,.18f,.35f,-.26f,-.16f,-.98f,.80f,.58f,-.65f,-.17f,.77f,.80f,-.06f,-.19f,-.43f,-.88f,.08f,.23f,-.70f,-.69f,-.76f,-.11f,.44f,-.87f,.70f,-.73f,-.55f,.05f,.30f,-.02f,.19f,.20f,.14f,.08f,.16f,.44f,.58f,-.51f,-.63f,.32f,.98f,-.48f,-.66f,-.00f,-.83f,.39f,-.95f,-.32f,-.28f,.60f,.59f,-.94f,.34f,-.68f,-.62f,.46f,-.87f,.25f,-.00f,.12f,.38f,.69f,-.54f,.11f,.03f,-.76f,.92f,.52f,-.33f,-.36f,-.33f,.46f,-.60f,.33f,-.56f,-.77f,.13f,.31f,-.65f,-.67f,-.58f,.54f,.41f,.80f,-.92f,.77f,-.44f,.69f,-.60f,.13f,-.82f,.13f,.27f,-.36f,-.88f,-.77f,-.72f,-.94f,.69f,.18f,-.29f,-.12f,-.81f,-.51f,.72f,-.03f,.32f,-.25f,-.83f,.11f,.14f,-.76f,-.62f,.51f,.87f,.76f,-.23f,.52f,.89f,-.02f,-.57f,.81f,-.83f,-.10f,-.88f,.71f,.30f,-.66f,-.38f,-.56f,.37f,.02f,.08f,-.88f,-.86f,.01f,-.61f,-.44f,.60f,.34f,-.90f,-.50f,-.11f,.52f,-.76f,.04f,-.53f,-.10f,.03f,.07f,.32f,.98f,.28f,.13f,.05f,.62f,-.15f,-.39f,.04f,-.81f,.84f,-.48f,.56f,-.74f,.39f,.79f,-.40f,-.67f,-.52f,.67f,-.68f,-.55f,-.65f,.36f,.27f,-.14f,-.14f,-.58f,.34f,.14f,-.33f,-.67f,.92f,-.73f,-.29f,.96f,-.43f,-.85f,.96f,.65f,.21f,.76f,-.74f,.42f,-.78f,-.06f,.21f,-.03f,-.83f,-.42f,.44f,-.79f,-.55f,-.22f,-.54f,-.24f,.68f,.03f,-.90f,.81f,.68f,.70f,-.84f,-.41f,-.26f,-.67f,.21f,-.49f,.00f,-.99f,.73f,.96f,-.50f,.82f,.83f,-.41f,.83f,-.48f,-.20f,.84f,-.79f,-.93f,.89f,-.24f,.59f,-.10f,.95f,.43f,.44f,-.81f,.61f,-.49f,-.78f,-.97f,.83f,-.33f,.26f,.46f,-.74f,-.64f,.49f,.38f,.92f,.70f,.51f,-.64f,.57f,.55f,.82f,-.91f,-.75f,-.50f,-.39f,-.79f,-.96f,-.72f,-.96f,.78f,-.81f,.07f,-.08f,-.34f,-.90f,.26f,.88f,-.69f,-.64f,-.41f,.46f,.93f,-.60f,-.81f,-.11f,-.50f,-.30f,-.27f,.59f,-.02f,-.71f,-.78f,.72f,.11f,-.84f,-.84f,.52f,.45f,-.90f,-.64f,-.38f,.32f,-.20f,.20f,.33f,-.61f,.79f,.33f,-.75f,-.21f,-.04f,.65f,-.27f,.59f,.60f,-.49f,-.38f,.72f,.87f,-.36f,-.97f,.13f,-.62f,-.86f,-.91f,-.52f,.45f,-.41f,-.38f,-.29f,-.07f,-.69f,-.86f,.46f,.60f,.37f,.14f,-.12f,-.75f,.64f,-.74f,.00f,-.91f,-.44f,-.75f,.94f,.91f,-.12f,.76f,.25f,-.75f,-.42f,-.71f,.41f,.55f,-.99f,.12f,.83f,.98f,-.45f,.16f,-.89f,-.38f,-.36f,-.89f,.36f,.27f,.88f,.64f,.31f,-.51f,.48f,-.31f,-.94f,.83f,-.51f,.51f,-.32f,.45f,-.34f,.62f,-.88f,.37f,.48f,-.25f,-.87f,-.40f,-.35f,-.60f,-.48f,-.62f,-.14f,.16f,-.60f,-.95f,.70f,.57f,.38f,-.42f,-.76f,-.37f,-.13f,.67f,.46f,-.54f,.62f,.87f,.93f,-.72f,.56f,.83f,-.24f,-.34f,-.52f,.65f,-.64f,.10f,-.02f,-.63f,-.59f,-.90f,-.98f,.28f,.52f,-.41f,.27f,-.97f,.53f,.29f,-.58f,-.44f,.87f,.32f,-.02f,-.49f,.33f,.61f,.93f,-.95f,.65f,.37f,-.42f,-.82f,.95f,-.96f,.72f,-.14f,-.07f,.03f,.20f,.87f,.52f,-.40f,-.72f,-.95f,.49f,-.29f,.17f,.93f,.04f,-.55f,.37f,.92f,-.99f,.94f,.24f,-.90f,-.60f,-.87f,.48f,.19f,-.27f,.12f,-.50f,-.97f,.85f,-.02f,.68f,.03f,.30f,-.53f,.66f,.38f,.72f,.12f,-.62f,-.46f,-.61f,-.89f,.24f,-.80f,.13f,-.52f,.45f,-.02f,.13f,.03f,.02f,.22f,.50f,.70f,.46f,.40f,-.00f,.49f,-.96f,.75f,-.31f,.60f,.06f,-.57f,.20f,-.79f,-.70f,.08f,.71f,.89f,.75f,-.65f,.50f,-.89f,.15f,.05f,.95f,.77f,-.03f,-.52f,.52f,.60f,.00f,-.92f,.34f,.38f,-.69f,.26f,-.74f,.29f,.39f,.18f,-.06f,.19f,-.67f,-.79f,.96f,-.63f,-.55f,-.86f,-.29f,.68f,-.57f,-.10f,-.87f,.22f,.70f,.09f,.17f,.27f,-.30f,-.53f,.50f,.80f,.07f,.09f,-.85f,.07f,.52f,-.71f,-.71f,.13f,-.57f,.39f,-.45f,.45f,.93f,-.82f,.09f,.94f,-.77f,-.58f,-.46f,-.40f,.47f,-.85f,-.92f,-.01f,-.35f,.33f,-.74f,.79f,.10f,-.24f,.07f,.97f,-.67f,-.69f,-.64f,-.31f,-.19f,.39f,-.73f,.18f,.10f,.14f,-.97f,-.37f,-.36f,-.33f,.64f,-.16f,-.60f,-.78f,.84f,-.77f,-.40f,.68f,-.49f,.13f,.39f,.98f,-.35f,.05f,-.97f,-.12f,.93f,.88f,.33f,.02f,.17f,.80f,-.71f,.98f,.92f,-.97f,.75f,.48f,.22f,.74f,-.51f,.67f,-.87f,.44f,.84f,.28f,-.04f,.41f,.33f,-.68f,.52f,.55f,-.81f,-.63f,.57f,-.48f,-.29f,-.51f,.76f,-.78f,-.39f,-.15f,-.24f,-.01f,-.37f,-.87f,.64f,.07f,.19f,-.96f,.77f,.48f,.95f,.27f,-.55f,-.47f,.00f,-.99f,-.89f,-.31f,.05f,-.08f,-.56f,.30f,.41f,-.63f,.44f,.80f,.11f,.14f,-.62f,-.41f,-.94f,-.64f,.44f,-.49f,-.89f,.42f,-.82f,-.97f,-.83f,-.41f,.68f,.89f,.40f,.09f,.28f,.09f,.01f,.32f,.33f,-.31f,.29f,-.57f,.26f,.70f,.28f,.34f,.12f,-.39f,.77f,.83f,.30f,.52f,-.97f,.80f,-.94f,-.53f,-.33f,-.38f,-.25f,-.20f,-.22f,.37f,-.64f,.95f,-.26f,-.46f,-.97f,-.86f,-.73f,-.19f,.62f,.45f,.33f,-.14f,.87f,-.41f,.01f,-.67f,-.14f,.59f,-.29f,.20f,-.33f,.19f,.50f,.68f,.60f,.21f,.91f,.29f,-.77f,.48f,.54f,-.72f,.26f,.94f,-.82f,.26f,.08f,-.69f,.20f,.66f,.65f,-.77f,.22f,-.17f,-.31f,.56f,.86f,.53f,-.06f,.73f,-.52f,.27f,.99f,-.73f,-.31f,-.09f,.74f,-.15f,-.14f,-.70f,.86f,-.83f,-.02f,-.18f,-.53f,.53f,-.23f,.94f,-.23f,-.32f,.23f,-.73f,-.43f,.06f,-.75f,-.88f,.43f,.50f,-.74f,-.17f,.03f,.74f,-.18f,-.97f,-.09f,.88f,.63f,-.88f,-.42f,.03f,-.19f,.54f,-.26f,.30f,-.13f,.74f,.36f,.05f,-.31f,.77f,-.24f,-.95f,-.52f,.95f,.23f,.87f,-.09f,.08f,.24f,.16f,.31f,.88f,.48f,-.12f,.30f,-.42f,-.10f,.44f,-.91f,-.43f,-.95f,.62f,.36f,.91f,-.74f,.74f,-.08f,.55f,.12f,-.14f,.82f,.22f,.77f,-.11f,.81f,.29f,-.52f,-.47f,.75f,-.08f,.23f,.45f,.87f,.48f,-.41f,.09f,.46f,.71f,.73f,.23f,-.87f,-.90f,-.38f,-.72f,.23f,.36f,.64f,-.87f,-.82f,-.97f,.96f,.97f,-.63f,-.92f,.73f,.29f,-.54f,-.20f,-.96f,-.15f,.98f,-.50f,-.42f,.29f,-.21f,.71f,-.71f,.58f,.12f,-.61f,-.37f,-.28f,-.00f,.66f,-.80f,-.29f,-.83f,-.15f,.70f,.27f,-.37f,.02f,-.28f,.85f,.14f,.26f,.06f,-.72f,.07f,.33f,.18f,-.56f,.66f,.51f,-.21f,.41f,-.69f,-.38f,.42f,-.04f,.71f,.66f,-.26f,.28f,.74f,.61f,.22f,.47f,-.44f,.49f,-.26f,.61f,-.24f,.07f,-.63f,-.24f,-.44f,-.41f,.74f,-.27f,.60f,.47f,-.69f,.13f,-.10f,.36f,-.54f,-.05f,-.26f,.50f,-.62f,.19f,-.05f,.56f,-.12f,.33f,.60f,-.50f,-.40f,.12f,-.06f,.95f,.06f,.60f,.00f,.32f,-.73f,-.05f,.42f,.24f,-.51f,.37f,-.86f,.49f,-.35f,.07f,.21f,-.03f,.13f,-.03f,-.19f,.48f,-.29f,-.56f,-.35f,.57f,-.98f,-.21f,.50f,.95f,.51f,-.40f,.77f,.26f,.77f,.71f,-.28f,.98f,-.78f,.55f,-.96f,-.30f,-.06f,.72f,.79f,.32f,.46f,-.42f,.38f,-.22f,.82f,.65f,.63f,-.92f,-.99f,.07f,-.28f,-.48f,-.22f,.29f,.61f,.04f,.76f,.77f,-.89f,-.84f,-.18f,.27f,-.14f,.13f,.75f,-.53f,.88f,-.46f,.22f,-.57f,-.19f,-.11f,-.96f,.16f,.69f,.37f,.06f,.59f,.21f,-.74f,.53f,.74f,.68f,-.60f,.81f,-.74f,-.82f,-.84f,-.64f,-.09f,.31f,.54f,.34f,.86f,.30f,-.72f,.25f,.51f,.62f,.47f,-.67f,.33f,-.68f,.63f,.84f,.91f,-.85f,-.24f,-.48f,-.40f,.29f,.24f,-.76f,.27f,.60f,-.51f,-.27f,-.99f,.58f,.87f,.58f,-.42f,.18f,.97f,-.42f,-.96f,-.30f,-.98f,-.05f,.98f,.35f,-.82f,-.55f,.87f,.13f,.13f,-.14f,-.61f,-.75f,-.58f,.24f,-.33f,-.79f,.95f,.19f,.01f,.54f,-.40f,-.60f,.08f,-.59f,.71f,-.98f,-.94f,.73f,.46f,-.54f,.23f,.49f,-.26f,.26f,.63f,-.32f,-.34f,-.14f,.01f,.68f,-.03f,.53f,.40f,-.90f,.70f,-.13f,.31f,.12f,-.57f,.52f,-.55f,.98f,-.71f,-.75f,-.71f,-.23f,.54f,.19f,.78f,-.41f,-.81f,-.34f,-.03f,-.78f,-.71f,-.98f,.69f,.08f,.16f,.64f,-.23f,-.71f,.35f,-.53f,-.45f,-.01f,-.99f,.38f,-.20f,-.62f,.56f,.82f,.98f,.26f,.00f,.36f,.29f,-.78f,-.88f,.46f,.00f,.13f,-.22f,.73f,-.42f,.67f,-.75f,.76f,.72f,.95f,.46f,-.81f,-.98f,-.38f,-.05f,-.02f,.77f,-.53f,-.53f,-.68f,-.98f,.07f,-.08f,.40f,-.59f,-.03f,-.13f,.70f,.07f,-.82f,.06f,-.09f,.66f,.15f,.73f,.39f,-.84f,-.33f,-.48f,-.81f,-.01f,-.43f,.42f,.55f,.31f,-.78f,-.15f,-.04f,.81f,.26f,-.38f,.00f,.54f,.21f,-.60f,-.75f,.92f,-.60f,-.51f,.95f,-.37f,-.81f,.44f,.68f,.71f,-.37f,.61f,.29f,-.27f,-.96f,-.14f,-.06f,-.74f,-.09f,.51f,-.89f,.47f,-.75f,.40f,.50f,.45f,-.68f,.77f,.31f,-.50f,-.45f,.52f,.97f,.05f,-.91f,.61f,-.36f,-.26f,-.36f,.47f,-.34f,.97f,-.45f,.19f,-.08f,.39f,.35f,-.28f,.81f,-.73f,.37f,-.39f,.15f,-.79f,-.73f,.37f,-.65f,.72f,-.47f,.57f,-.57f,-.82f,-.87f,-.62f,-.92f,.92f,.85f,-.32f,.04f,-.79f,.96f,.77f,.13f,-.19f,.44f,-.10f,-.65f,-.49f,.33f,.72f,.96f,.13f,-.66f,-.78f,.76f,-.44f,-.69f,.87f,.67f,-.34f,.87f,.92f,.02f,-.28f,-.21f,-.56f,.36f,.96f,.53f,-.19f,-.15f,-.73f,.39f,.53f,-.78f,.26f,-.56f,.17f,.69f,.20f,-.08f,.45f,.19f,.51f,-.96f,-.42f,.59f,.16f,.48f,-.75f,-.71f,.11f,-.24f,-.31f,-.67f,-.96f,-.05f,-.53f,.62f,.81f,.37f,.07f,.80f,.40f,-.38f,-.84f,-.57f,.96f,-.23f,.62f,-.88f,.49f,.49f,.37f,.82f,-.83f,-.00f,.93f,-.25f,.13f,-.69f,.87f,.94f,.71f,.58f,.51f,.97f,-.44f,-.76f,.48f,.01f,.11f,-.57f,.53f,-.85f,-.45f,.31f,-.02f,-.27f,.39f,.86f,-.42f,-.67f,-.34f,.39f,.42f,-.08f,.25f,.01f,-.45f,-.11f,.65f,-.89f,-.44f,.54f,.66f,.53f,-.96f,-.09f,.57f,.20f,-.16f,.90f,-.02f,.92f,.67f,-.06f,.51f,-.54f,-.79f,.08f,.89f,-.96f,.77f,-.83f,-.72f,.95f,.06f,.38f,.80f,.20f,.17f,-.30f,.76f,-.39f,-.81f,-.59f,-.66f,.35f,.01f,-.53f,.07f,.94f,.88f,.97f,-.18f,.14f,.41f,-.83f,-.00f,.59f,-.05f,.03f,.84f,-.17f,-.32f,-.79f,-.86f,-.55f,-.09f,.57f,.01f,.80f,-.11f,.68f,.92f,-.57f,.60f,-.07f,-.10f,.66f,.47f,.62f,.11f,-.50f,.01f,-.03f,.84f,-.15f,-.45f,-.07f,.19f,.94f,-.10f,-.46f,-.47f,-.78f,.05f,.70f,-.96f,.88f,-.84f,.72f,-.40f,.46f,.44f,.31f,.10f,.36f,.42f,.42f,-.29f,-.87f,.16f,-.24f,.62f,.05f,.13f,-.11f,.80f,-.23f,.18f,.43f,-.91f,.75f,.38f,.84f,.61f,-.83f,-.34f,-.01f,.05f,-.39f,.05f,-.64f,.62f,-.29f,.56f,-.06f,.23f,-.39f,.18f,.44f,.04f,.36f,.89f,.93f,.24f,.42f,.25f,-.45f,-.63f,-.22f,.09f,-.27f,.27f,.28f,-.12f,-.97f,-.20f,.55f,-.43f,-.35f,-.64f,.63f,.42f,-.93f,-.76f,-.98f,.90f,.80f,-.64f,-.92f,.34f,-.81f,.79f,-.77f,.56f,.60f,-.05f,-.63f,-.32f,.51f,-.52f,-.20f,.38f,.80f,-.63f,-.52f,-.13f,-.06f,.01f,-.66f,.14f,-.45f,.78f,.07f,.00f,.17f,-.46f,.47f,-.10f,.31f,-.16f,-.62f,-.32f,-.85f,.24f,-.01f,-.63f,.80f,.92f,.54f,-.85f,-.74f,-.93f,-.98f,.54f,-.23f,-.36f,.20f,.75f,.22f,.71f,-.07f,-.13f,.40f,-.82f,.46f,-.39f,.34f,-.35f,.12f,-.42f,.63f,.68f,.04f,.97f,-.04f,-.38f,.87f,.12f,.02f,-.71f,-.81f,-.43f,.56f,.80f,.88f,.51f,-.78f,-.52f,.35f,-.52f,.21f,-.51f,.19f,-.05f,.64f,.72f,-.69f,.45f,-.94f,.41f,-.80f,-.17f,.67f,-.41f,.08f,.15f,-.55f,.13f,-.25f,-.85f,.77f,.47f,-.44f,.39f,-.71f,-.62f,.65f,.35f,.20f,.25f,.62f,-.53f,-.97f,-.36f,-.12f,-.93f,-.46f,.39f,.07f,.06f,-.88f,.07f,.01f,-.95f,-.02f,.52f,.41f,-.88f,.55f,.99f,.11f,.76f,-.90f,.97f,-.25f,-.92f,.65f,.94f,.71f,-.82f,-.46f,.44f,.21f,.84f,-.49f,.55f,-.20f,.45f,.48f,-.58f,.96f,.52f,.01f,.31f,-.67f,.52f,-.66f,-.77f,.18f,.36f,-.32f,.51f,.23f,.58f,-.45f,-.52f,.22f,-.46f,.79f,.49f,.40f,.81f,-.44f,.79f,-.50f,.90f,.13f,.34f,.93f,.31f,.56f,-.83f,.90f,-.18f,.55f,-.05f,-.82f,.16f,.67f,.08f,-.82f,-.04f,-.05f,.69f,.75f,.74f,-.37f,.09f,.82f,-.32f,.16f,.67f,.50f,.37f,.63f,-.65f,.28f,.34f,.09f,.41f,-.64f,.57f,.80f,-.11f,-.43f,-.16f,-.29f,-.79f,-.52f,-.54f,-.22f,-.94f,-.77f,.36f,.75f,-.42f,.71f,-.53f,-.23f,-.26f,-.23f,-.19f,-.13f,.85f,-.15f,.70f,-.27f,-.43f,.63f,.13f,.95f,-.63f,-.28f,-.42f,-.91f,-.42f,.79f,-.05f,-.43f,.24f,.39f,.22f,.63f,-.40f,.12f,.10f,-.23f,-.86f,-.12f,-.42f,.31f,-.87f,-.38f,.04f,.54f,.37f,-.51f,.09f,.58f,.27f,.27f,.17f,-.16f,.36f,.35f,.88f,-.79f,.34f,.34f,-.51f,-.15f,-.84f,.07f,.46f,.46f,.44f,-.70f,-.41f,.98f,.44f,.66f,.31f,.84f,.95f,-.25f,-.52f,-.07f,-.78f,-.72f,-.55f,-.84f,-.38f,-.77f,.90f,-.08f,-.62f,.74f,-.78f,-.46f,-.71f,.80f,.21f,.66f,.19f,.80f,.05f,.83f,.28f,.46f,.75f,.33f,.31f,-.62f,-.20f,.13f,-.79f,-.17f,.32f,-.26f,-.48f,.40f,.93f,-.08f,.49f,.71f,.34f,-.05f,.62f,.30f,.22f,.12f,.60f,-.02f,-.56f,.39f,-.15f,.78f,.80f,-.11f,-.61f,-.66f,-.25f,-.80f,-.44f,-.39f,-.99f,.05f,.57f,-.36f,.82f,-.74f,.37f,-.50f,.68f,-.37f,.56f,.80f,-.50f,.45f,-.50f,.58f,-.01f,.79f,-.48f,.05f,.97f,.32f,.82f,.46f,.49f,-.12f,-.71f,-.04f,-.65f,.60f,-.81f,.78f,-.85f,.90f,-.95f,-.46f,.62f,.36f,-.43f,.10f,.57f,.52f,-.60f,-.05f,-.11f,.12f,.32f,.52f,.91f,.53f,.24f,.04f,-.44f,-.41f,.89f,-.17f,-.45f,-.83f,.28f,-.70f,.46f,.58f,.81f,-.23f,-.97f,-.61f,-.82f,.83f,.45f,.74f,-.55f,-.51f,-.74f,.56f,.00f,.23f,.76f,-.30f,.07f,.91f,.26f,.82f,.32f,-.40f,.82f,.64f,-.83f,-.99f,-.19f,.34f,.09f,.31f,-.47f,.38f,.33f,.68f,.71f,-.20f,-.86f,-.10f,-.53f,-.04f,-.99f,.35f,.49f,.52f,.81f,-.51f,.76f,-.50f,-.31f,-.93f,-.89f,-.88f,.97f,.22f,-.41f,.25f,-.28f,-.29f,-.60f,-.23f,-.75f,-.39f,.81f,.17f,.18f,-.92f,-.94f,.82f,-.83f,-.71f,.77f,-.65f,.15f,.17f,.09f,.23f,.21f,.05f,.14f,-.56f,-.31f,.32f,.45f,-.41f,.40f,-.01f,-.59f,-.07f,.96f,.15f,.98f,-.32f,-.66f,.58f,-.07f,.01f,-.67f,-.17f,-.37f,.76f,-.57f,.18f,-.97f,-.14f,.78f,.13f,.78f,-.58f,.33f,.45f,-.31f,.29f,.50f,-.20f,-.78f,-.82f,-.55f,.46f,.55f,-.73f,-.63f,.01f,.12f,.62f,.58f,.95f,.08f,-.94f,-.90f,-.70f,-.22f,.57f,.69f,.80f,.52f,-.38f,-.38f,.59f,.17f,.53f,.46f,-.55f,.81f,.59f,.54f,.57f,-.38f,.07f,-.23f,.16f,.18f,.57f,-.94f,.95f,-.83f,.75f,-.75f,-.97f,-.57f,-.61f,-.15f,-.12f,-.67f,.72f,-.90f,-.26f,.39f,.65f,-.34f,-.43f,.13f,-.23f,-.22f,-.80f,.60f,-.54f,.31f,-.98f,-.12f,.48f,.94f,-.96f,.42f,-.33f,-.89f,-.03f,.94f,.04f,.36f,-.75f,.28f,.28f,-.77f,.32f,-.33f,-.09f,-.30f,.96f,-.30f,-.17f,.54f,.48f,.61f,-.50f,-.01f,.49f,-.88f,-.28f,.50f,.66f,.03f,-.19f,-.01f,-.44f,-.73f,-.11f,.07f,.71f,.05f,-.38f,-.50f,-.47f,-.92f,.75f,.48f,-.46f,-.08f,-.15f,.59f,.32f,.87f,.87f,.80f,-.79f,.20f,-.08f,-.64f,.45f,-.26f,.42f,-.95f,.03f,.91f,-.11f,.14f,.63f,.55f,.59f,.04f,.79f,.30f,-.13f,.20f,.48f,-.07f,-.69f,.29f,-.02f,.45f,-.61f,-.52f,.46f,-.28f,.81f,-.93f,-.73f,.43f,-.96f,.45f,.09f,.51f,.56f,-.39f,.20f,-.18f,.16f,-.28f,.02f,.05f,-.45f,.31f,-.61f,.82f,-.92f,-.94f,-.20f,.10f,.44f,-.02f,-.84f,.16f,.08f,-.66f,.06f,.72f,.09f,-.91f,.70f,-.96f,.53f,-.30f,.89f,.76f,.11f,.61f,.89f,-.02f,-.65f,-.98f,-.93f,-.66f,-.11f,-.97f,.52f,-.75f,-.15f,.30f,.39f,.32f,.49f,-.72f,.44f,.62f,.64f,-.76f,.63f,-.18f,.29f,-.74f,-.10f,-.35f,-.11f,-.42f,.29f,-.07f,.06f,-.26f,-.91f,.85f,-.31f,-.73f,.84f,.89f,.46f,.96f,.92f,-.76f,.87f,-.83f,-.57f,.42f,-.79f,-.55f,-.32f,-.14f,.36f,-.37f,.14f,.17f,.60f,.48f,-.66f,.24f,-.27f,.30f,-.76f,-.73f,-.14f,-.86f,-.43f,.93f,.61f,-.03f,.70f,.74f,-.91f,.45f,-.78f,-.07f,-.60f,-.57f,.40f,.51f,.53f,-.02f,.27f,.04f,.94f,.49f,-.16f,-.94f,-.16f,.41f,-.45f,.31f,.56f,.25f,-.56f,.87f,.15f,-.83f,-.66f,-.69f,-.16f,.32f,-.61f,.76f,-.59f,-.69f,-.70f,.42f,.65f,.90f,-.45f,.59f,-.25f,-.94f,.74f,-.87f,-.24f,.69f,-.85f,.33f,-.89f,.35f,-.29f,-.67f,-.28f,-.85f,-.91f,-.68f,.38f,.48f,-.21f,-.00f,-.39f,.93f,.00f,-.43f,-.25f,.67f,.82f,.00f,.27f,-.76f,.44f,-.83f,-.61f,-.80f,.98f,-.07f,-.86f,.03f,.69f,-.88f,-.87f,-.65f,.88f,.61f,.32f,-.82f,.76f,-.57f,-.59f,.02f,-.80f,.34f,-.98f,-.25f,-.34f,.83f,.40f,-.73f,-.63f,-.73f,-.59f,-.15f,-.19f,.07f,-.21f,-.87f,-.52f,.03f,.69f,-.73f,.99f,.25f,-.28f,.50f,-.17f,.81f,.70f,-.62f,.19f,-.64f,-.14f,.87f,.37f,-.98f,-.33f,-.41f,-.21f,.51f,.84f,-.93f,-.44f,.40f,.86f,-.39f,-.46f,.01f,-.06f,.74f,.43f,-.97f,.69f,-.14f,.06f,.46f,.72f,.48f,-.33f,.91f,.85f,.07f,.51f,.34f,-.16f,.46f,.88f,-.24f,.17f,.55f,-.61f,.16f,-.32f,.02f,.53f,.91f,.04f,.26f,.33f,-.64f,-.50f,.32f,.55f,-.47f,.37f,.69f,.09f,-.29f,.75f,-.19f,.93f,.92f,.65f,.85f,-.45f,.17f,-.36f,-.08f,-.01f,.08f,.69f,-.98f,-.46f,-.32f,-.66f,.98f,.08f,-.71f,-.98f,-.51f,-.95f,.23f,.26f,-.35f,-.72f,.69f,-.64f,-.13f,-.23f,.95f,-.74f,-.25f,.34f,.36f,.25f,-.51f,.00f,-.60f,-.92f,.81f,-.84f,.85f,-.55f,.26f,.20f,.18f,-.78f,-.27f,.99f,-.36f,-.97f,.76f,.10f,-.80f,.16f,-.90f,.48f,-.38f,-.15f,.97f,.75f,.17f,.60f,.15f,-.76f,-.58f,.72f,.24f,.16f,-.37f,.07f,.45f,-.29f,.17f,-.30f,-.37f,-.15f,.43f,-.90f,-.33f,.90f,-.29f,.53f,.96f,.83f,.11f,.28f,-.70f,.87f,-.26f,.90f,.17f,.89f,-.25f,-.92f,.62f,.10f,.14f,-.91f,-.59f,-.54f,-.34f,.10f,.25f,.69f,-.60f,.57f,.82f,-.12f,-.97f,-.72f,.34f,-.43f,-.02f,-.16f,-.49f,-.10f,-.22f,-.95f,-.29f,.15f,.15f,.55f,-.55f,.89f,-.28f,-.15f,-.22f,-.63f,.28f,.64f,-.82f,-.09f,-.14f,.96f,-.07f,-.58f,-.29f,.77f,.18f,.47f,-.33f,-.14f,-.80f,-.80f,-.52f,.52f,-.37f,-.09f,.13f,.85f,.01f,.25f,.47f,-.67f,.29f,-.52f,.92f,.87f,-.25f,-.87f,.67f,.44f,-.85f,.81f,-.29f,-.51f,-.11f,-.86f,-.92f,.90f,.87f,.57f,.45f,.74f,.02f,.81f,-.50f,-.32f,.24f,-.70f,-.71f,-.30f,.71f,.56f,.60f,-.64f,.56f,-.88f,.75f,-.04f,.90f,.40f,-.39f,-.16f,.99f,-.55f,.55f,.77f,-.76f,.25f,-.18f,-.78f,.71f,.05f,-.03f,.80f,-.55f,-.13f,.63f,-.33f,.18f,-.66f,.27f,-.83f,-.21f,-.79f,-.46f,-.15f,-.61f,-.28f,-.84f,.58f,-.43f,.85f,-.22f,.54f,.73f,-.87f,-.48f,.53f,-.97f,-.93f,.85f,-.96f,.37f,.78f,.61f,.55f,.43f,-.75f,.09f,-.06f,-.23f,.68f,-.42f,-.88f,-.81f,-.36f,.33f,-.50f,.63f,-.74f,-.61f,-.29f,.14f,.78f,-.19f,-.13f,-.73f,.97f,.87f,-.15f,-.62f,-.16f,.83f,-.78f,.87f,.48f,.32f,.77f,.59f,-.02f,.06f,.72f,.84f,.18f,-.24f,-.53f,.26f,.56f,.50f,.62f,-.29f,-.08f,-.39f,-.29f,-.36f,-.98f,.53f,.53f,-.21f,-.08f,.47f,-.77f,-.73f,-.48f,.84f,.12f,.09f,-.22f,.77f,-.10f,-.69f,.10f,.13f,-.42f,.96f,-.67f,-.78f,.10f,.66f,.85f,.52f,.89f,.50f,.38f,-.03f,.75f,.29f,.53f,-.71f,.99f,-.58f,.75f,.74f,-.66f,.50f,.76f,-.73f,-.84f,-.86f,.71f,.07f,-.82f,-.43f,.96f,.94f,-.36f,-.27f,.43f,.03f,-.60f,-.35f,.41f,-.30f,.06f,-.51f,.49f,-.34f,.99f,-.86f,.99f,-.03f,-.43f,-.27f,-.72f,-.92f,.17f,-.78f,.43f,.93f,.12f,-.15f,.73f,.88f,.44f,-.58f,-.20f,.27f,.45f,.40f,.02f,.94f,-.68f,.88f,-.76f,-.16f,.08f,.92f,-.88f,.03f,-.66f,-.59f,-.05f,-.95f,-.54f,.97f,.94f,-.84f,.24f,-.03f,-.14f,.58f,-.75f,.38f,.00f,-.90f,-.34f,-.70f,-.79f,-.33f,.48f,.92f,-.68f,-.88f,.99f,-.58f,.32f,.81f,-.57f,.23f,-.70f,-.26f,.49f,-.13f,.93f,.66f,-.31f,-.58f,.56f,-.57f,-.38f,.59f,.10f,.10f,-.55f,-.41f,.13f,-.22f,-.53f,.30f,.39f,.66f,.93f,-.38f,.93f,.35f,-.47f,.21f,.14f,.86f,.71f,-.51f,-.99f,.72f,-.37f,-.24f,-.04f,.19f,.03f,-.18f,.38f,-.68f,.88f,.98f,-.57f,.26f,-.18f,.34f,.99f,.04f,-.05f,.84f,-.83f,.45f,-.93f,-.94f,-.65f,.91f,-.23f,.34f,-.34f,-.11f,.10f,.96f,.22f,.59f,.67f,.73f,-.05f,.34f,-.69f,-.74f,-.32f,-.47f,.96f,-.38f,-.77f,-.04f,-.30f,-.29f,-.82f,.22f,-.05f,-.56f,.10f,-.78f,-.06f,-.14f,-.46f,-.11f,.66f,.83f,-.46f,-.26f,-.19f,-.55f,.28f,.00f,-.02f,.02f,-.64f,.14f,-.57f,-.49f,.07f,-.38f,.18f,.75f,-.97f,.20f,-.14f,.26f,.58f,-.31f,-.64f,.90f,.52f,-.49f,.98f,.78f,-.84f,-.36f,.64f,-.63f,.46f,-.95f,.05f,.27f,.65f,.69f,-.80f,-.31f,-.57f,.70f,-.34f,-.65f,-.46f,-.06f,-.12f,-.35f,-.03f,-.49f,.38f,-.07f,-.15f,-.46f,.65f,.70f,-.05f,-.98f,-.75f,.07f,-.61f,-.69f,.05f,-.39f,-.93f,-.93f,.66f,-.68f,.52f,.76f,-.24f,-.78f,-.06f,-.77f,.28f,-.65f,-.74f,.42f,-.04f,-.59f,.97f,-.96f,.87f,.30f,-.31f,-.24f,.99f,-.15f,-.06f,.87f,-.86f,.34f,-.68f,-.15f,.45f,-.83f,-.33f,.56f,-.53f,.45f,-.95f,.48f,.89f,.93f,-.74f,-.21f,.15f,-.17f,-.81f,-.07f,.99f,-.32f,-.66f,.64f,-.89f,-.74f,.14f,.04f,-.82f,-.74f,-.30f,.61f,.35f,-.72f,.52f,-.60f,-.44f,.42f,.58f,.29f,.91f,-.44f,.94f,.65f,-.32f,-.26f,-.02f,.79f,.45f,-.40f,.10f,.53f,-.11f,-.15f,-.99f,-.24f,.86f,.85f,-.42f,-.00f,.28f,.07f,-.86f,-.18f,.75f,-.38f,.16f,-.62f,.97f,.28f,.30f,.80f,.14f,.24f,.82f,-.25f,.55f,-.82f,.42f,-.39f,-.24f,-.58f,.54f,.27f,.95f,-.98f,-.30f,-.55f,.51f,-.52f,-.32f,-.50f,.49f,-.90f,-.15f,.38f,-.92f,.38f,-.48f,.39f,.45f,-.56f,-.96f,-.96f,.72f,-.25f,-.77f,-.31f,.59f,-.89f,.42f,.48f,.62f,-.11f,-.78f,.70f,.46f,.88f,-.32f,-.62f,-.87f,-.61f,.14f,.99f,.44f,.81f,-.66f,.24f,-.56f,.69f,-.97f,.76f,-.75f,-.00f,-.94f,.58f,-.32f,.82f,.74f,-.24f,-.88f,.30f,-.07f,.04f,.75f,-.70f,.70f,.23f,-.21f,-.51f,.36f,.63f,-.70f,.10f,.09f,.64f,-.51f,-.12f,-.23f,-.39f,.35f,.74f,-.58f,.39f,-.84f,.81f,-.93f,-.40f,.31f,-.81f,.41f,-.89f,-.27f,-.39f,.13f,-.51f,-.72f,-.55f,.79f,.49f,-.29f,.00f,-.92f,-.10f,-.77f,-.75f,-.33f,-.20f,.21f,.22f,-.31f,.14f,-.58f,.82f,.38f,.33f,-.01f,.54f,-.67f,.24f,.60f,.53f,.52f,.21f,-.73f,-.72f,-.60f,-.37f,-.34f,-.06f,.15f,-.63f,-.99f,-.28f,.04f,-.36f,-.28f,.35f,.88f,.24f,.52f,-.54f,.55f,-.91f,.36f,.59f,.48f,-.43f,-.54f,-.36f,.12f,.56f,-.72f,.54f,-.75f,-.77f,.12f,-.89f,-.30f,.93f,-.11f,-.18f,-.67f,-.15f,.02f,-.76f,-.51f,.45f,.79f,-.49f,.90f,-.98f,-.70f,.10f,.35f,-.22f,.66f,-.62f,.87f,.44f,-.55f,-.63f,.09f,-.42f,-.38f,.81f,.69f,-.56f,-.51f,.25f,.31f,.06f,-.99f,.45f,-.85f,.84f,.32f,.21f,.61f,-.44f,-.63f,.46f,-.87f,-.03f,.40f,-.52f,.89f,-.92f,-.14f,.30f,.39f,.30f,-.63f,-.01f,-.66f,-.02f,-.51f,.14f,.02f,.28f,.85f,.77f,-.55f,.69f,-.13f,-.72f,-.04f,-.35f,.05f,.40f,.49f,.27f,-.64f,-.46f,-.59f,-.33f,.43f,.53f,-.50f,-.60f,.28f,.82f,.51f,.90f,.48f,-.55f,-.08f,-.32f,.68f,-.93f,-.09f,-.85f,-.29f,-.92f,.15f,.84f,-.97f,.12f,.72f,.81f,-.88f,-.98f,-.26f,.85f,.56f,.95f,-.10f,.67f,-.14f,.03f,.25f,.72f,-.22f,.18f,-.72f,-.85f,.83f,.00f,.98f,-.34f,.35f,.62f,-.65f,-.66f,.78f,.09f,-.09f,-.98f,.89f,-.27f,-.54f,-.82f,.06f,-.12f,-.89f,.45f,-.07f,.73f,-.66f,-.00f,-.67f,.56f,-.41f,-.99f,.09f,.30f,-.30f,.76f,-.33f,.02f,.29f,-.36f,.69f,.83f,-.16f,.76f,-.36f,-.47f,-.22f,.18f,-.49f,.57f,-.02f,-.56f,-.93f,-.19f,-.98f,-.98f,-.17f,-.42f,.55f,.49f,-.37f,.02f,-.22f,-.61f,-.36f,.94f,-.65f,-.87f,.56f,-.93f,-.53f,-.23f,-.30f,.34f,.53f,-.55f,-.62f,-.86f,.22f,-.57f,.47f,.12f,-.93f,-.83f,-.90f,.77f,.36f,.73f,-.59f,.87f,.88f,-.54f,.88f,.39f,.42f,.50f,.68f,-.66f,.07f,.22f,-.39f,-.47f,.41f,-.79f,-.38f,.79f,-.09f,.37f,-.07f,.05f,.66f,-.54f,.45f,.39f,.84f,.72f,.12f,.30f,-.55f,.54f,.42f,-.14f,-.85f,-.56f,-.55f,.27f,.70f,.98f,-.28f,-.29f,.48f,-.02f,-.83f,-.86f,-.92f,.88f,-.11f,.53f,.06f,-.45f,.05f,.78f,.69f,.22f,.52f,.11f,.38f,-.52f,.47f,-.02f,-.43f,-.19f,-.39f,-.45f,.81f,.29f,-.97f,-.04f,.25f,-.29f,-.77f,.46f,-.62f,-.92f,.85f,-.89f,-.84f,.03f,.16f,.47f,-.18f,-.60f,.04f,.71f,-.99f,.57f,.12f,-.83f,-.26f,.29f,-.81f,-.48f,.91f,.01f,-.38f,.50f,.69f,.38f,.18f,.03f,-.64f,.19f,.10f,.76f,.27f,-.34f,-.28f,-.74f,-.26f,.28f,-.60f,.95f,.52f,.84f,-.20f,.97f,.93f,-.56f,-.29f,.87f,-.71f,.20f,.83f,-.83f,.69f,-.86f,.40f,.34f,.78f,-.50f,-.19f,.50f,-.78f,-.11f,.77f,-.80f,.95f,-.22f,.03f,-.68f,-.63f,.74f,.01f,-.70f,-.59f,-.64f,-.72f,.83f,.41f,-.65f,-.16f,-.29f,.55f,-.63f,.80f,-.85f,-.49f,.45f,-.54f,-.00f,-.36f,-.04f,.66f,-.14f,.15f,-.44f,.79f,-.44f,.81f,.90f,.65f,.27f,-.08f,.28f,-.88f,.89f,-.80f,-.96f,-.97f,.56f,-.72f,-.68f,-.26f,.91f,.43f,.93f,-.85f,-.97f,-.00f,.04f,-.68f,.76f,.61f,-.66f,-.08f,-.75f,-.95f,.84f,.47f,-.66f,-.92f,-.50f,.11f,-.99f,.22f,.20f,.00f,.33f,.14f,-.35f,.48f,.42f,.30f,.89f,-.93f,.88f,-.44f,.08f,.01f,.03f,-.14f,.71f,.24f,-.17f,-.68f,.17f,.53f,.79f,.97f,.32f,.58f,.40f,-.77f,-.77f,.14f,.38f,.69f,.73f,.88f,.81f,.67f,.13f,-.53f,-.13f,-.79f,.70f,.46f,.71f,.06f,-.42f,.96f,.24f,.02f,-.31f,-.18f,-.87f,-.70f,.05f,-.46f,-.59f,-.84f,.12f,.41f,-.90f,.45f,.11f,.02f,-.49f,-.26f,-.41f,.43f,-.34f,-.19f,-.41f,-.64f,-.20f,.22f,-.20f,-.05f,.33f,-.19f,.69f,-.85f,-.01f,.33f,.07f,.80f,-.45f,-.53f,-.57f,.88f,-.80f,.12f,.60f,-.31f,.11f,.33f,-.10f,-.09f,.21f,-.93f,-.44f,.18f,-.13f,-.33f,.17f,.38f,-.43f,.07f,.30f,-.10f,-.25f,.96f,-.23f,.91f,-.47f,.37f,.52f,-.96f,.33f,.37f,-.75f,-.91f,.33f,.11f,-.92f,-.98f,-.74f,-.26f,.04f,-.06f,.38f,-.73f,.27f,.88f,.25f,-.79f,.48f,.82f,.98f,-.97f,.21f,-.16f,.56f,-.03f,-.53f,-.13f,-.73f,-.31f,.37f,-.68f,-.36f,.42f,-.21f,-.02f,-.57f,.50f,.00f,-.87f,.28f,.27f,.98f,-.96f,.21f,.81f,-.33f,-.94f,-.15f,-.45f,.93f,-.36f,.23f,.64f,-.04f,-.13f,.40f,-.72f,.17f,.87f,-.60f,-.82f,-.20f,.32f,.92f,.99f,.28f,-.47f,.50f,.27f,.58f,-.98f,.36f,.90f,.45f,-.65f,.26f,.54f,-.54f,.35f,.07f,-.94f,.23f,.02f,.87f,.83f,.09f,-.83f,.53f,.44f,-.93f,.35f,-.86f,-.01f,.38f,.56f,-.65f,.72f,-.04f,.09f,.17f,.95f,-.64f,.91f,.34f,-.60f,-.40f,.73f,-.66f,-.65f,-.21f,-.17f,-.12f,.50f,.27f,-.68f,-.74f,-.27f,-.64f,-.75f,-.01f,-.50f,.47f,-.50f,-.08f,-.00f,-.79f,.99f,-.01f,-.69f,-.60f,-.95f,-.11f,.25f,-.52f,-.49f,.31f,.76f,-.83f,-.39f,-.42f,-.18f,.06f,-.04f,-.74f,-.27f,-.23f,.76f,.33f,.59f,-.72f,-.70f,-.04f,.63f,-.21f,.25f,.16f,-.03f,.77f,.03f,-.74f,.94f,-.91f,-.23f,.47f,.90f,.82f,-.58f,.77f,-.37f,.05f,-.51f,.09f,-.80f,.95f,.73f,.28f,.56f,-.74f,-.31f,.37f,.65f,.32f,-.50f,-.52f,-.68f,.78f,.82f,.94f,.27f,-.44f,-.05f,.39f,-.80f,-.88f,.42f,.72f,.02f,.26f,-.58f,.21f,-.90f,.00f,.36f,.77f,-.96f,-.18f,.99f,-.71f,.94f,-.81f,-.08f,-.64f,-.71f,-.84f,-.28f,.44f,.44f,.80f,-.17f,-.50f,.38f,.75f,.29f,.54f,-.56f,-.40f,.35f,.59f,.88f,-.87f,.03f,.88f,-.02f,.26f,.68f,.38f,.24f,.03f,.15f,.28f,.73f,.67f,.94f,-.04f,.18f,-.19f,.68f,-.91f,.41f,-.50f,-.36f,.03f,-.65f,.44f,-.73f,-.18f,.39f,.40f,-.53f,-.94f,.11f,.71f,.82f,-.17f,.65f,.60f,.93f,-.99f,-.11f,-.83f,-.31f,-.09f,-.08f,.91f,-.51f,-.25f,.52f,.54f,.23f,-.00f,.69f,.88f,.49f,-.50f,.99f,.82f,-.56f,-.68f,.18f,-.38f,-.97f,.27f,-.83f,.63f,.98f,-.59f,.43f,.07f,-.68f,-.00f,-.25f,-.69f,-.03f,.86f,.72f,.53f,-.95f,-.88f,.15f,.43f,-.61f,-.03f,-.56f,-.44f,-.35f,-.46f,-.21f,-.80f,-.63f,.39f,-.24f,-.91f,.24f,.32f,-.22f,.73f,-.24f,-.53f,-.22f,-.62f,-.39f,-.40f,.60f,.78f,-.11f,.32f,-.34f,.53f,.60f,.10f,-.59f,.31f,.77f,-.54f,.52f,.68f,.84f,-.55f,-.15f,.16f,.15f,-.64f,.73f,.77f,-.57f,-.05f,.36f,-.03f,.27f,.93f,-.88f,.82f,.40f,-.69f,.23f,.01f,.30f,-.21f,.09f,-.18f,.97f,.33f,.48f,-.15f,-.05f,-.86f,.81f,-.33f,-.92f,.58f,.47f,.22f,.81f,.77f,-.82f,-.15f,.82f,-.53f,.31f,.30f,-.16f,-.24f,-.24f,-.80f,.72f,-.91f,-.53f,-.73f,.85f,.82f,-.97f,-.80f,-.13f,-.20f,.40f,-.70f,-.35f,.88f,.08f,-.48f,.77f,.62f,-.16f,-.47f,.53f,.11f,.43f,-.36f,-.13f,-.01f,-.36f,.88f,-.89f,.03f,.69f,-.36f,-.98f,.52f,.28f,.59f,.27f,-.43f,-.75f,.61f,.67f,.46f,-.50f,-.11f,-.77f,-.43f,-.00f,-.80f,.91f,.22f,-.50f,-.83f,-.52f,.94f,-.92f,-.36f,-.96f,-.40f,.11f,-.37f,.08f,.52f,.53f,.02f,-.83f,.54f,-.48f,.02f,-.05f,.32f,-.78f,.02f,-.79f,.34f,-.54f,.93f,-.80f,.32f,.70f,-.82f,.92f,.04f,.83f,-.26f,-.68f,.76f,.78f,.35f,.71f,.83f,.52f,.57f,-.20f,.12f,.26f,-.99f,.47f,.60f,-.06f,-.17f,.26f,-.60f,.70f,-.22f,-.87f,-.78f,.29f,.47f,.76f,.09f,.73f,.07f,-.91f,-.53f,-.40f,.78f,.81f,-.57f,-.05f,-.54f,-.04f,-.14f,.80f,.11f,-.75f,.56f,.63f,-.73f,-.32f,.86f,-.98f,-.64f,.59f,-.95f,.63f,.10f,.24f,-.83f,-.60f,-.67f,.40f,-.96f,-.30f,-.69f,.70f,-.89f,.40f,-.21f,.83f,.44f,.71f,-.85f,-.05f,-.95f,-.83f,.26f,-.63f,.37f,-.05f,-.76f,-.47f,.37f,.01f,.42f,-.12f,.99f,-.43f,.16f,-.20f,-.17f,-.18f,-.13f,-.98f,-.05f,-.86f,.40f,.70f,-.68f,.93f,-.62f,.08f,-.54f,-.01f,.82f,.05f,-.45f,-.83f,-.22f,-.35f,.54f,.48f,.12f,.27f,-.87f,-.67f,-.15f,.02f,-.67f,.52f,.24f,.08f,-.80f,.09f,.99f,.31f,.01f,.83f,-.99f,-.90f,-.23f,.39f,.75f,-.51f,.75f,.37f,-.28f,.27f,-.72f,.71f,.78f,.95f,-.05f,.08f,-.10f,-.19f,.43f,.38f,-.69f,-.38f,.67f,.58f,.86f,.14f,.86f,-.06f,.41f,-.17f,-.15f,.96f,.10f,.20f,.22f,-.92f,-.51f,-.08f,.84f,.83f,-.90f,.51f,-.07f,.69f,.84f,.30f,.35f,-.01f,-.01f,.85f,.76f,.77f,-.50f,.67f,.65f,-.49f,-.47f,-.64f,.63f,-.12f,-.84f,.76f,.50f,.00f,-.83f,-.71f,.43f,-.10f,.95f,.86f,-.77f,.93f,-.61f,-.67f,-.18f,-.53f,-.12f,-.59f,.95f,-.97f,-.08f,-.20f,-.65f,-.78f,-.02f,-.47f,-.55f,-.77f,.00f,-.01f,-.97f,.29f,-.68f,.66f,-.86f,.70f,-.53f,-.99f,-.29f,-.87f,.34f,.13f,-.12f,-.64f,.65f,-.08f,.80f,-.37f,-.01f,.63f,-.85f,.37f,.23f,.06f,-.57f,-.99f,.28f,-.48f,.70f,.34f,.08f,.96f,-.27f,.36f,.31f,-.39f,-.22f,-.06f,-.67f,-.22f,-.49f,.77f,-.16f,.95f,-.03f,-.40f,.67f,.17f,.73f,.44f,-.69f,.78f,.57f,-.14f,-.61f,-.31f,.73f,-.71f,.03f,.95f,.33f,-.50f,.69f,-.11f,.81f,-.64f,.67f,.66f,-.47f,-.28f,-.11f,.11f,.97f,.67f,.98f,.38f,-.07f,-.38f,.12f,-.59f,.89f,.12f,.05f,-.03f,.58f,-.68f,-.74f,.57f,.81f,.23f,-.35f,-.27f,.29f,-.43f,.03f,-.60f,.18f,.07f,.74f,-.56f,.70f,-.11f,.28f,-.73f,.14f,-.60f,.00f,-.71f,-.38f,.39f,.93f,-.32f,.75f,-.39f,.63f,-.44f,-.10f,.10f,.83f,.56f,-.07f,-.25f,.75f,.96f,.34f,-.78f,-.95f,-.18f,.36f,.57f,-.82f,.44f,-.57f,.65f,.40f,-.02f,.97f,.31f,.06f,-.91f,-.80f,-.74f,-.29f,.46f,-.58f,.47f,.25f,-.06f,.28f,.98f,-.56f,-.13f,.90f,.28f,.99f,.78f,-.46f,.96f,-.34f,-.94f,.22f,.99f,-.48f,.18f,.97f,-.52f,.98f,.74f,-.18f,.04f,.98f,-.85f,-.36f,-.29f,-.18f,-.39f,-.89f,.49f,-.71f,.24f,.59f,-.85f,.58f,.86f,-.68f,.07f,.72f,-.32f,-.76f,.84f,.05f,.16f,.68f,-.76f,.22f,-.65f,-.18f,-.49f,-.17f,.70f,-.42f,.98f,.18f,.18f,.22f,.14f,.35f,.26f,-.36f,.98f,.84f,-.54f,.13f,.90f,.46f,.53f,-.94f,-.15f,-.14f,.77f,.68f,.16f,-.49f,-.43f,-.83f,-.33f,.14f,-.55f,-.39f,-.14f,.08f,-.78f,-.79f,-.65f,-.65f,.73f,.97f,.65f,.50f,-.51f,-.24f,-.88f,.59f,.74f,.03f,-.91f,.33f,.03f,-.66f,-.20f,-.13f,.94f,.46f,.41f,.50f,.45f,.47f,-.10f,-.28f,-.36f,-.02f,-.52f,.11f,-.19f,-.70f,.33f,.20f,-.85f,-.15f,-.89f,.11f,.68f,-.64f,-.70f,.90f,-.02f,-.26f,-.49f,.88f,.46f,.75f,.96f,.46f,.08f,.43f,-.14f,-.17f,-.74f,-.75f,-.19f,.61f,-.87f,.84f,-.04f,.85f,.16f,-.73f,.14f,-.21f,.21f,.73f,.92f,.10f,-.66f,.37f,-.64f,.90f,.32f,-.46f,.04f,.80f,.88f,.13f,-.88f,-.48f,.37f,.42f,-.35f,-.92f,-.16f,.61f,-.15f,.17f,.14f,.89f,-.15f,.18f,.94f,-.13f,.51f,-.67f,.64f,.41f,.45f,.00f,-.81f,.06f,-.06f,-.48f,-.92f,-.42f,.83f,-.87f,.29f,.72f,.70f,.40f,.79f,-.80f,.17f,.32f,.29f,-.30f,-.13f,.82f,.72f,-.96f,-.84f,-.13f,-.36f,.10f,.40f,.59f,.92f,-.30f,-.16f,-.53f,-.47f,.14f,-.82f,.02f,.81f,.27f,-.77f,.63f,.47f,.24f,.19f,-.29f,-.56f,-.49f,-.03f,.60f,-.84f,.10f,-.30f,-.09f,.78f,.31f,-.75f,.19f,-.10f,.02f,.45f,.60f,-.36f,.68f,.49f,-.29f,-.21f,.54f,-.66f,.55f,.03f,.53f,.25f,-.47f,-.13f,.41f,.74f,.02f,-.67f,-.68f,-.61f,-.92f,-.37f,-.65f,-.76f,-.86f,.83f,-.84f,-.79f,-.36f,.09f,.43f,.68f,-.81f,-.25f,.81f,-.66f,.28f,.33f,.74f,.56f,-.29f,-.84f,-.76f,.74f,-.03f,.52f,.30f,.18f,.11f,-.46f,.28f,-.15f,-.27f,.21f,.79f,-.46f,-.41f,-.70f,.60f,-.35f,-.33f,-.56f,-.14f,.17f,.01f,-.81f,-.62f,-.60f,.39f,-.62f,.87f,.83f,-.00f,-.47f,-.39f,-.97f,.11f,.22f,-.32f,-.21f,-.25f,.70f,.70f,.01f,.04f,-.38f,-.12f,.80f,.85f,.40f,.87f,-.59f,-.99f,-.17f,.63f,.30f,.36f,.77f,-.25f,-.50f,.22f,-.26f,.49f,-.97f,.18f,-.29f,-.19f,-.28f,.34f,-.69f,-.15f,-.50f,.18f,.68f,-.31f,.52f,-.18f,-.67f,-.22f,.11f,-.58f,-.17f,-.21f,-.01f,.71f,.93f,.00f,.41f,-.72f,-.66f,-.26f,-.57f,-.06f,-.44f,-.06f,-.79f,-.14f,-.09f,.52f,.67f,-.65f,-.52f,-.85f,-.52f,-.18f,-.70f,.75f,-.51f,-.49f,-.06f,.81f,.06f,-.95f,.53f,-.80f,-.16f,.84f,.97f,-.53f,.00f,-.75f,-.99f,-.38f,.07f,-.44f,.25f,.31f,.16f,-.39f,-.92f,-.17f,-.38f,-.63f,.27f,.53f,.56f,-.82f,.95f,.69f,-.25f,.62f,-.36f,-.21f,.15f,-.82f,-.73f,-.77f,-.78f,.46f,-.15f,.44f,.82f,-.82f,-.50f,-.73f,-.39f,-.85f,.89f,-.30f,-.48f,-.96f,.95f,.66f,-.08f,.82f,-.00f,-.16f,.99f,-.49f,.27f,.88f,-.69f,-.36f,-.81f,.86f,-.53f,.79f,.76f,.56f,.59f,-.70f,.31f,.61f,.80f,.93f,.90f,.05f,.42f,.21f,-.75f,-.87f,.48f,-.83f,-.58f,-.53f,-.20f,-.34f,-.08f,.25f,-.21f,-.18f,-.82f,.59f,.18f,-.73f,.50f,.36f,.01f,-.54f,-.40f,.24f,-.99f,-.30f,-.43f,-.26f,.53f,.98f,.90f,-.68f,-.57f,-.02f,.63f,-.84f,-.96f,-.52f,.32f,.09f,-.98f,-.52f,.64f,.17f,.82f,.07f,.92f,.63f,-.43f,-.52f,.06f,-.85f,.02f,-.71f,-.86f,-.58f,-.32f,.79f,.68f,.02f,.50f,-.76f,-.26f,-.23f,.03f,.42f,-.26f,-.09f,-.40f,.84f,.79f,-.57f,-.07f,-.61f,-.79f,-.97f,.16f,.10f,.45f,-.04f,.66f,-.52f,-.39f,-.77f,.94f,.70f,.61f,-.96f,-.33f,.85f,.53f,-.74f,.30f,.31f,-.33f,.74f,-.56f,-.00f,-.95f,-.91f,-.42f,.85f,-.02f,-.06f,-.73f,.07f,-.52f,.10f,.39f,.90f,.78f,-.11f,.58f,.61f,.30f,.53f,.51f,.06f,.62f,.58f,.39f,-.78f,.47f,-.36f,-.48f,.29f,-.98f,.90f,-.63f,-.40f,.77f,.93f,.32f,.94f,.76f,-.86f,.84f,-.57f,-.35f,-.68f,.58f,-.70f,-.87f,-.13f,-.99f,-.46f,.51f,.28f,-.22f,-.92f,-.02f,.77f,.91f,.41f,-.13f,-.18f,-.80f,-.12f,.99f,-.62f,.71f,-.39f,-.53f,-.75f,-.88f,-.05f,.84f,.32f,.87f,.18f,.71f,.03f,-.48f,-.59f,.43f,-.44f,.23f,-.36f,.79f,-.27f,-.30f,-.35f,.56f,.99f,-.35f,-.89f,-.45f,.45f,-.55f,.27f,-.29f,.90f,-.95f,-.63f,.31f,-.28f,-.56f,.38f,-.08f,-.18f,.51f,.60f,-.93f,-.00f,.89f,-.52f,.08f,-.36f,-.61f,.71f,-.06f,.53f,-.77f,-.46f,-.30f,-.79f,-.36f,.59f,-.32f,-.28f,.88f,-.15f,.99f,.31f,.84f,.72f,.45f,.61f,-.69f,.73f,-.28f,.88f,-.21f,.65f,-.54f,-.76f,-.33f,-.53f,.87f,-.36f,.96f,.60f,-.07f,.43f,.52f,.68f,.72f,-.53f,-.23f,.09f,.47f,-.92f,-.31f,-.63f,-.26f,-.86f,.28f,.77f,-.14f,.63f,.48f,.81f,.31f,.94f,.37f,.93f,-.70f,.64f,-.63f,-.31f,-.10f,-.64f,.61f,.02f,-.86f,-.40f,-.95f,-.80f,.98f,.96f,-.07f,.33f,-.59f,-.13f,-.54f,.67f,-.50f,.14f,-.30f,.30f,-.80f,.71f,.86f,.26f,-.97f,-.57f,-.88f,-.95f,.84f,-.25f,.58f,-.28f,-.61f,-.55f,-.03f,.14f,-.05f,.66f,.98f,-.01f,-.41f,.06f,.74f,.09f,.42f,.34f,-.06f,.27f,-.26f,-.82f,-.07f,.67f,-.12f,-.46f,.16f,-.07f,.05f,-.21f,.35f,-.77f,-.39f,.11f,.15f,.06f,.17f,.69f,.25f,-.89f,.93f,-.29f,-.25f,-.95f,.74f,-.29f,-.22f,.82f,-.71f,.39f,-.54f,.58f,.41f,-.82f,-.65f,-.41f,-.74f,-.09f,-.08f,-.53f,-.06f,-.02f,.51f,.76f,.40f,.23f,-.14f,-.71f,-.59f,.33f,.14f,-.98f,-.01f,-.45f,.25f,-.33f,-.21f,.47f,-.28f,-.43f,.52f,-.04f,.04f,-.63f,.53f,-.15f,-.23f,-.15f,.04f,-.99f,-.69f,.16f,.26f,-.21f,.90f,.99f,-.91f,-.69f,.40f,-.69f,-.52f,.18f,.86f,-.41f,.69f,.44f,-.13f,-.16f,.89f,.60f,-.12f,.06f,.84f,-.58f,.71f,.19f,.81f,.58f,-.64f,-.39f,-.92f,-.70f,-.98f,-.13f,.07f,-.99f,.14f,.69f,-.20f,.18f,-.80f,.34f,-.79f,.94f,.34f,-.81f,-.69f,-.72f,-.07f,.33f,.06f,.87f,-.77f,.72f,.16f,.74f,.87f,-.87f,.57f,.87f,.65f,.63f,-.78f,.71f,.64f,-.94f,.45f,.52f,.61f,.80f,.56f,.89f,-.99f,.31f,.11f,-.63f,.21f,.97f,.04f,-.59f,-.81f,.13f,.46f,.28f,.01f,-.22f,.62f,-.12f,.64f,.82f,-.67f,-.76f,.87f,-.23f,-.43f,.72f,-.64f,.08f,.53f,-.41f,-.03f,-.68f,.62f,.08f,.37f,-.12f,-.52f,.07f,.06f,-.28f,.02f,.35f,.11f,.38f,.90f,-.19f,-.36f,.95f,-.53f,-.66f,-.30f,.46f,.44f,.63f,.15f,.75f,-.88f,-.80f,-.39f,-.33f,.45f,.56f,-.34f,.98f,-.50f,.22f,.15f,.12f,.62f,.06f,.82f,-.40f,.86f,.56f,.18f,.13f,-.30f,-.98f,.57f,-.49f,-.32f,-.74f,.91f,-.07f,.09f,.35f,-.22f,-.42f,.37f,.41f,-.01f,-.45f,.96f,-.97f,-.37f,.58f,-.74f,.61f,.37f,-.25f,.82f,.99f,-.22f,.60f,-.95f,.35f,.63f,-.73f,-.96f,.40f,.17f,-.04f,.01f,-.45f,-.52f,.86f,.32f,-.97f,-.67f,.21f,.60f,-.88f,-.17f,.11f,-.89f,.52f,-.93f,-.17f,-.94f,.72f,-.45f,.81f,-.85f,-.63f,.27f,.72f,-.99f,-.74f,-.70f,-.06f,-.28f,.28f,.89f,-.38f,-.36f,.28f,.51f,-.29f,-.04f,-.22f,-.77f,.62f,.68f,.73f,-.29f,.96f,.26f,.36f,-.79f,.93f,.23f,.06f,-.99f,.93f,.34f,-.40f,.43f,-.80f,-.60f,-.64f,-.17f,-.89f,-.56f,-.81f,.70f,-.67f,.57f,.57f,-.77f,-.28f,-.39f,-.58f,-.35f,-.02f,.21f,-.43f,.60f,-.95f,-.44f,.40f,-.22f,.44f,-.94f,.21f,-.21f,-.38f,-.06f,.43f,-.48f,.49f,-.79f,.31f,-.91f,.34f,.89f,.10f,.50f,.48f,.29f,.00f,.24f,.50f,.56f,-.73f,-.10f,.23f,.29f,.59f,-.02f,-.76f,-.46f,-.97f,-.06f,-.46f,.51f,.14f,.97f,-.37f,.65f,-.74f,-.20f,-.29f,.57f,-.80f,-.34f,-.33f,.17f,-.86f,.58f,-.85f,-.08f,.09f,-.88f,.57f,-.78f,-.55f,-.72f,.18f,-.45f,.37f,-.51f,-.77f,.83f,-.39f,.73f,-.87f,-.18f,-.95f,.94f,-.21f,.70f,.56f,.25f,.99f,.14f,.18f,-.35f,.72f,.09f,.98f,.85f,.10f,.97f,.42f,-.15f,-.85f,.89f,.67f,.99f,-.08f,.32f,.08f,.55f,.96f,-.52f,-.63f,-.21f,-.50f,.79f,.10f,.61f,-.96f,-.47f,.59f,-.02f,-.39f,.89f,.31f,.94f,-.84f,.47f,.79f,.82f,-.18f,-.41f,-.22f,-.99f,-.68f,-.35f,.86f,.16f,.68f,.05f,-.11f,-.65f,.23f,-.81f,-.92f,-.70f,.60f,-.65f,-.37f,-.23f,.10f,.15f,.19f,-.03f,.74f,-.07f,.12f,-.49f,.67f,-.91f,.38f,.53f,.86f,-.41f,-.39f,-.79f,-.48f,-.83f,-.58f,.18f,-.96f,-.77f,-.47f,-.38f,.32f,.23f,.85f,.35f,.40f,-.27f,.66f,.62f,.65f,-.94f,-.69f,-.93f,.69f,-.20f,.05f,-.67f,-.18f,-.36f,-.31f,.15f,-.01f,.86f,-.72f,-.94f,.41f,-.22f,-.86f,-.56f,.69f,.09f,-.00f,-.80f,.05f,.98f,.95f,.46f,-.33f,-.38f,-.04f,-.53f,-.12f,.82f,-.55f,-.79f,.10f,-.14f,-.68f,-.29f,.14f,.93f,.64f,.97f,.37f,.69f,.73f,-.57f,.11f,-.75f,-.07f,.74f,.25f,-.57f,.67f,.60f,-.26f,-.35f,-.19f,.55f,.42f,-.62f,.63f,-.87f,-.82f,.60f,-.10f,-.67f,.89f,-.38f,.29f,.30f,.81f,.95f,.43f,-.64f,.91f,.37f,.96f,-.71f,.55f,.06f,-.95f,-.56f,.34f,.25f,.05f,.26f,.28f,.95f,.68f,-.63f,-.80f,.63f,-.09f,-.06f,.26f,-.34f,-.27f,-.57f,.99f,-.48f,.10f,.96f,-.17f,.70f,-.76f,-.49f,.37f,-.81f,-.05f,-.67f,.70f,-.94f,-.15f,-.98f,.03f,-.77f,.97f,-.61f,-.32f,.63f,-.01f,-.42f,-.89f,.41f,.14f,.11f,-.61f,-.05f,.32f,.21f,.64f,.36f,-.44f,.05f,-.33f,.19f,-.43f,-.40f,-.89f,.90f,.67f,.66f,-.37f,-.67f,-.63f,.93f,.29f,.28f,.41f,.17f,-.76f,-.61f,-.48f,-.86f,-.13f,-.55f,.96f,-.06f,-.14f,.97f,.86f,-.20f,.88f,.40f,.64f,.79f,.47f,-.09f,-.26f,-.31f,.74f,-.09f,-.49f,.64f,-.40f,-.31f,-.87f,-.68f,.40f,-.44f,.19f,.10f,-.69f,.42f,-.59f,.84f,-.39f,-.85f,.33f,.44f,.12f,.69f,.02f,-.71f,.57f,-.16f,-.67f,.53f,-.95f,.37f,.42f,.40f,.76f,-.61f,.02f,.77f,.66f,.54f,-.37f,-.76f,-.56f,.77f,.57f,.99f,-.02f,.00f,.54f,.19f,.89f,-.32f,-.00f,-.74f,-.82f,.43f,.42f,.35f,-.81f,.68f,-.52f,-.17f,.18f,.29f,.76f,-.73f,.01f,-.28f,.04f,.21f,.89f,-.40f,.04f,.13f,-.95f,.33f,.47f,.18f,.61f,-.83f,-.08f,-.08f,-.75f,.30f,.42f,.76f,.25f,.18f,-.84f,.11f,-.94f,-.42f,-.16f,-.97f,.54f,.11f,.37f,.10f,-.39f,-.20f,.09f,.04f,-.92f,.97f,.98f,-.78f,.58f,-.74f,.92f,-.27f,.16f,-.42f,-.31f,.39f,.23f,-.65f,.24f,-.92f,.52f,.98f,.39f,.51f,.98f,-.98f,.34f,.68f,-.84f,.41f,-.87f,.55f,.39f,.16f,.37f,.01f,.82f,.91f,-.82f,-.76f,.65f,-.45f,.44f,.07f,-.46f,.64f,-.63f,-.11f,.54f,-.92f,-.11f,-.94f,-.78f,-.88f,.68f,-.11f,.78f,.28f,-.08f,.99f,.27f,.73f,-.55f,.05f,.69f,-.69f,-.83f,.70f,-.24f,.26f,-.86f,.69f,.91f,.43f,.84f,-.60f,.31f,.74f,.57f,-.93f,-.85f,-.74f,.73f,-.15f,.30f,.70f,-.15f,.09f,-.46f,.59f,.05f,.34f,.33f,.17f,-.08f,.03f,-.60f,.93f,.11f,-.13f,-.65f,-.19f,.49f,.82f,-.77f,.22f,-.02f,.79f,.06f,.16f,-.79f,-.30f,-.48f,.62f,-.12f,-.95f,-.87f,-.94f,-.90f,-.33f,.60f,-.21f,.88f,-.28f,.46f,-.14f,.13f,-.11f,-.17f,-.03f,.19f,-.04f,-.34f,-.14f,.65f,-.35f,-.66f,-.75f,.25f,-.91f,-.45f,-.59f,.32f,.75f,-.24f,-.14f,.10f,-.24f,.03f,-.83f,.90f,.08f,.09f,.65f,-.39f,-.80f,.26f,.36f,-.14f,-.53f,.21f,-.06f,.63f,.94f,-.86f,.30f,.91f,-.01f,-.61f,.67f,-.44f,-.57f,.28f,.83f,-.46f,-.18f,-.27f,-.25f,.43f,.55f,-.74f,-.39f,.51f,.62f,.70f,.17f,.61f,-.87f,-.76f,-.28f,.80f,-.09f,.48f,-.68f,.46f,.76f,.73f,-.06f,.15f,.01f,.84f,-.71f,.43f,-.82f,.77f,-.86f,.30f,.01f,-.03f,-.79f,-.96f,.75f,.36f,-.18f,-.23f,.79f,.36f,-.77f,.96f,.60f,.11f,-.73f,-.44f,-.15f,-.54f,.94f,-.54f,.73f,.43f,.70f,-.02f,-.47f,.14f,-.07f,-.78f,-.12f,.61f,-.65f,.28f,-.66f,-.56f,.76f,.56f,.52f,-.43f,.31f,.11f,-.94f,.04f,-.52f,-.37f,.86f,-.74f,-.38f,-.40f,.41f,-.40f,.09f,-.07f,-.79f,-.08f,-.32f,-.87f,-.31f,.21f,-.58f,-.18f,-.86f,.29f,-.28f,.30f,-.52f,.41f,.66f,.10f,-.40f,.70f,.29f,-.20f,-.25f,.33f,.32f,-.14f,.39f,-.41f,.65f,-.88f,.82f,.34f,-.80f,.06f,.86f,-.69f,-.84f,-.36f,.43f,.25f,-.80f,.84f,.19f,-.80f,.53f,-.53f,.04f,-.54f,-.26f,-.59f,.21f,-.45f,.53f,-.17f,.52f,.35f,.72f,.16f,-.31f,.12f,-.03f,-.92f,.02f,.27f,.52f,.34f,-.66f,-.46f,-.14f,.81f,.00f,.92f,-.49f,.80f,.40f,-.90f,.34f,.12f,.47f,.16f,-.66f,-.83f,.30f,.10f,-.99f,.95f,.61f,.92f,.90f,-.46f,.99f,.38f,.38f,.79f,-.33f,.28f,.69f,.44f,-.18f,-.66f,.07f,.65f,-.06f,-.24f,-.70f,.01f,-.31f,-.40f,.19f,.93f,-.52f,-.29f,-.59f,-.55f,-.57f,-.61f,-.51f,.99f,.22f,.51f,.71f,.39f,.59f,.38f,.02f,-.98f,.56f,-.04f,-.02f,.44f,-.85f,-.70f,-.70f,-.02f,-.09f,-.96f,.72f,.25f,-.20f,.15f,.68f,-.90f,-.91f,-.99f,-.21f,-.59f,-.38f,-.02f,-.84f,-.50f,.68f,-.22f,-.88f,.25f,.98f,-.10f,-.37f,-.14f,.70f,.03f,.45f,.17f,-.01f,.33f,-.15f,.03f,-.66f,-.05f,-.10f,.35f,-.00f,-.12f,-.79f,.76f,.29f,-.27f,.79f,-.57f,.62f,.65f,.84f,-.17f,.23f,.74f,.98f,-.24f,-.96f,-.16f,.76f,-.22f,.35f,-.39f,.67f,.59f,.91f,.88f,.69f,-.72f,.43f,.55f,-.55f,.94f,.34f,-.74f,-.44f,-.48f,-.92f,-.62f,.79f,.52f,-.59f,-.14f,-.20f,.14f,.65f,-.27f,.71f,-.80f,.29f,-.15f,.15f,.30f,-.65f,-.17f,-.72f,-.90f,.99f,-.20f,.40f,-.59f,.46f,-.69f,.97f,-.86f,.35f,-.96f,.22f,.26f,-.87f,-.46f,-.54f,.30f,.28f,.63f,.43f,-.87f,-.16f,.40f,.07f,-.04f,.67f,-.30f,-.45f,.88f,.92f,.05f,.24f,.18f,-.29f,.26f,.57f,-.35f,.49f,-.16f,.32f,-.79f,-.29f,-.87f,-.72f,.40f,-.44f,-.69f,-.93f,-.85f,.92f,.00f,.66f,-.32f,.94f,.40f,.81f,.67f,-.72f,-.11f,.82f,-.83f,-.53f,-.01f,.18f,-.95f,-.41f,-.98f,-.08f,.24f,-.33f,.72f,-.81f,.21f,.46f,.25f,-.74f,.78f,.68f,-.20f,-.66f,.73f,-.09f,.98f,.50f,.38f,.95f,-.79f,.47f,.95f,-.28f,-.03f,.15f,.25f,.61f,.10f,-.42f,.98f,-.93f,-.47f,-.03f,.31f,.50f,-.24f,.42f,-.03f,-.75f,.75f,.86f,.07f,.79f,.39f,.58f,-.09f,-.04f,.63f,-.49f,.89f,.64f,.14f,-.70f,.04f,-.24f,.42f,.98f,-.88f,-.86f,.43f,-.17f,-.79f,-.15f,-.43f,.97f,-.88f,-.97f,.87f,-.61f,.56f,.53f,.94f,.26f,-.71f,.68f,-.19f,.87f,.00f,-.02f,.33f,.91f,-.36f,.60f,-.55f,-.04f,-.58f,-.07f,.18f,.42f,-.77f,-.66f,-.90f,-.23f,-.94f,.90f,-.37f,-.63f,.57f,.98f,-.46f,-.47f,.79f,.63f,-.05f,-.39f,-.51f,.22f,.23f,-.89f,-.01f,.98f,-.59f,-.85f,.12f,.24f,.63f,-.69f,.84f,-.15f,.48f,-.16f,.14f,.78f,.84f,.86f,-.80f,-.05f,-.42f,.32f,-.39f,-.27f,.85f,-.58f,.96f,-.53f,-.40f,-.75f,.76f,-.87f,-.39f,.73f,.48f,.87f,-.49f,.94f,-.25f,-.19f,-.30f,.56f,-.35f,.09f,.62f,.02f,.07f,-.32f,.83f,.87f,.96f,.50f,.16f,.12f,-.21f,-.77f,-.69f,.95f,-.84f,-.93f,-.83f,.68f,.06f,.48f,-.95f,.44f,.62f,.21f,-.62f,-.11f,.21f,-.74f,-.04f,.07f,.47f,-.07f,-.27f,.71f,.70f,-.55f,.13f,-.50f,.63f,-.51f,.48f,-.19f,.17f,-.34f,-.87f,-.28f,-.71f,.16f,-.56f,.39f,-.14f,.26f,-.09f,-.48f,-.10f,.50f,-.83f,.02f,-.39f,.19f,.10f,.73f,-.43f,-.47f,.79f,.75f,.89f,-.38f,-.72f,-.93f,-.22f,-.00f,.43f,.07f,-.96f,.08f,.63f,-.55f,-.55f,.83f,-.12f,-.70f,.03f,-.51f,.61f,.49f,.95f,.93f,.69f,.88f,-.10f,.50f,-.05f,-.59f,.36f,-.85f,-.02f,-.02f,.75f,-.77f,-.49f,.16f,.75f,-.08f,-.44f,.17f,-.87f,-.19f,-.60f,.03f,-.30f,-.21f,-.58f,.08f,-.47f,-.03f,.39f,.61f,.71f,.10f,.18f,.53f,-.62f,-.72f,.72f,-.87f,-.45f,-.57f,.14f,.65f,-.14f,-.65f,.17f,-.89f,-.35f,.22f,-.82f,-.23f,-.56f,.25f,.82f,.98f,.24f,.76f,-.27f,-.97f,-.14f,-.99f,.95f,.73f,-.37f,-.26f,.73f,-.51f,.42f,-.25f,.76f,.59f,-.54f,.18f,.12f,.41f,.41f,-.45f,.44f,-.26f,-.10f,-.21f,-.33f,.82f,-.88f,.33f,-.83f,-.47f,.08f,.36f,-.59f,-.77f,.70f,-.23f,.72f,.77f,-.99f,.36f,.16f,.62f,.35f,-.99f,.99f,-.32f,.39f,-.09f,.15f,.97f,-.42f,-.51f,-.98f,-.23f,-.81f,-.62f,.36f,-.63f,.33f,-.67f,-.84f,.17f,.26f,-.57f,.77f,-.86f,.54f,-.11f,.97f,-.29f,.45f,-.59f,-.39f,.21f,-.40f,.03f,.36f,.41f,.56f,.76f,.71f,-.62f,-.49f,.99f,.03f,.55f,-.15f,.03f,.62f,-.62f,-.02f,-.53f,.64f,-.24f,.14f,.67f,-.58f,-.90f,.66f,.44f,-.15f,.16f,.92f,.84f,.61f,.45f,.01f,-.23f,.51f,.65f,-.04f,-.61f,-.66f,.76f,-.52f,.63f,.82f,.67f,.81f,-.08f,.14f,.91f,.20f,-.82f,-.90f,.70f,.65f,.87f,.38f,.77f,.52f,.40f,-.57f,.19f,.58f,.50f,.78f,-.09f,-.93f,.67f,.06f,.11f,.36f,-.40f,.37f,.53f,.01f,-.09f,.89f,-.87f,.99f,-.54f,.92f,-.45f,.69f,-.22f,.92f,-.01f,.04f,.68f,-.96f,-.56f,.26f,.24f,.77f,-.31f,.23f,.26f,-.09f,-.12f,-.57f,.35f,-.80f,.36f,.36f,.55f,.85f,-.60f,.12f,-.96f,-.54f,-.58f,-.21f,.62f,.60f,.64f,.70f,-.14f,-.63f,-.14f,.55f,.25f,.21f,-.19f,.42f,-.61f,-.08f,-.38f,-.86f,.08f,.48f,-.58f,.82f,.14f,.50f,.81f,.07f,.24f,.03f,.54f,.47f,-.97f,-.80f,-.76f,-.86f,.80f,-.14f,-.04f,-.54f,.61f,-.42f,-.90f,.94f,-.84f,.15f,-.72f,.52f,.10f,-.48f,.42f,-.63f,.93f,-.80f,-.17f,-.29f,-.87f,-.24f,.99f,.75f,.78f,-.39f,-.94f,.46f,.34f,-.93f,.87f,-.18f,.20f,.33f,.91f,.12f,.62f,-.19f,-.50f,-.20f,.73f,.81f,-.04f,-.26f,.16f,-.24f,.60f,-.69f,.59f,.18f,.02f,-.00f,-.28f,.48f,-.29f,-.97f,.11f,-.34f,-.22f,.74f,.53f,.11f,-.97f,-.50f,-.46f,-.96f,-.93f,-.02f,-.23f,.04f,-.30f,-.79f,-.46f,.03f,-.18f,.71f,.73f,.27f,.76f,.65f,.52f,.34f,.54f,-.36f,.02f,.90f,-.24f,-.44f,-.81f,-.22f,.50f,-.25f,-.94f,.91f,-.57f,-.38f,.95f,-.05f,.46f,-.74f,-.88f,.59f,.86f,-.52f,-.32f,.30f,-.43f,.68f,.69f,-.88f,.92f,-.29f,-.78f,.89f,-.96f,-.91f,-.80f,.14f,-.36f,-.86f,.79f,-.39f,-.81f,.86f,.43f,-.77f,.44f,-.40f,-.91f,-.43f,-.29f,-.23f,-.07f,-.76f,.14f,.88f,.59f,-.36f,-.80f,-.81f,.74f,.49f,-.41f,.10f,-.45f,.89f,.99f,.95f,-.84f,-.19f,-.50f,-.00f,.17f,-.27f,-.14f,-.27f,-.34f,-.22f,.10f,.78f,-.90f,.64f,.98f,-.78f,.06f,.61f,.51f,-.34f,.70f,-.17f,-.65f,.84f,-.56f,-.81f,.26f,-.08f,.84f,.64f,.96f,-.26f,.73f,.68f,.71f,-.53f,-.04f,.32f,.99f,-.73f,.65f,-.57f,.44f,-.66f,-.20f,-.43f,-.54f,-.97f,.41f,.24f,-.53f,-.55f,-.26f,-.71f,.51f,.83f,.53f,.01f,.95f,.45f,-.40f,.31f,.59f,.34f,-.45f,-.92f,-.27f,.54f,-.14f,-.87f,-.60f,.72f,-.70f,-.04f,.18f,.91f,.68f,-.15f,.21f,.86f,.54f,-.86f,.70f,.40f,.64f,-.82f,-.70f,.68f,.96f,.71f,.84f,-.89f,-.47f,-.63f,-.75f,.79f,-.77f,.14f,.02f,.19f,-.84f,-.19f,.63f,.32f,.65f,-.16f,.44f,-.23f,-.01f,.67f,.85f,.08f,.32f,.14f,.70f,.40f,-.03f,-.29f,.78f,-.17f,-.68f,.36f,.28f,-.48f,.65f,.74f,-.37f,.81f,-.95f,-.21f,.04f,-.23f,-.24f,-.27f,-.93f,.19f,-.27f,-.83f,.19f,-.21f,-.08f,-.19f,.73f,.26f,-.97f,.51f,.92f,-.53f,-.18f,-.54f,.89f,-.49f,-.61f,-.54f,-.63f,-.01f,-.77f,.96f,.44f,-.64f,.63f,.24f,-.51f,.67f,-.16f,-.99f,.61f,.73f,.53f,.84f,-.06f,-.39f,.13f,.40f,-.26f,.90f,.94f,.15f,.71f,.70f,-.71f,-.53f,.74f,.47f,-.45f,.45f,-.24f,.44f,.90f,-.71f,.22f,.32f,-.61f,-.78f,-.66f,-.44f,.88f,-.58f,.57f,.08f,.57f,-.61f,.71f,-.12f,.49f,-.41f,-.69f,-.47f,-.46f,.73f,-.19f,-.01f,.44f,-.41f,.74f,.15f,.15f,.10f,.10f,.59f,.16f,-.80f,-.75f,.79f,-.22f,-.75f,-.05f,.22f,-.92f,-.11f,-.86f,.81f,-.29f,-.08f,-.78f,-.65f,.19f,-.35f,-.76f,.91f,-.55f,-.39f,-.23f,.34f,-.07f,-.95f,.10f,.98f,-.12f,-.49f,-.79f,-.00f,.57f,.83f,.21f,.28f,.33f,-.87f,.60f,-.00f,-.82f,-.00f,.61f,.06f,.45f,.65f,.75f,.04f,-.21f,-.92f,.25f,-.11f,-.52f,.79f,.19f,.48f,-.79f,-.26f,.47f,.05f,.98f,.89f,-.58f,-.03f,.22f,.20f,.17f,-.96f,.13f,.92f,-.31f,-.81f,-.30f,-.60f,.30f,.78f,.86f,-.40f,.23f,-.36f,-.35f,-.69f,.93f,.94f,.52f,.38f,.33f,-.97f,-.28f,.80f,.47f,-.27f,.06f,.25f,-.74f,.57f,.04f,-.93f,.88f,.65f,.68f,.95f,-.69f,.58f,-.67f,.73f,.85f,-.35f,-.10f,-.38f,-.57f,.67f,.79f,-.24f,.60f,.89f,-.85f,.20f,.11f,.14f,-.02f,.53f,-.56f,.39f,-.27f,.43f,.43f,-.41f,-.25f,.00f,-.13f,.26f,.08f,.39f,.07f,.04f,.70f,.49f,-.28f,.19f,-.48f,.29f,.52f,-.99f,.40f,.04f,.47f,.54f,.32f,.48f,.66f,.51f,.29f,-.58f,-.93f,.03f,-.23f,.62f,-.33f,-.51f,-.63f,.22f,.47f,.26f,-.04f,-.50f,.25f,-.61f,.92f,-.52f,.97f,-.12f,.98f,-.12f,.20f,-.04f,-.78f,-.69f,-.06f,-.55f,.38f,-.19f,-.35f,.84f,.46f,.68f,.17f,.27f,.32f,.79f,.37f,-.16f,-.39f,.55f,-.49f,-.70f,-.67f,-.53f,.80f,.35f,-.96f,-.06f,.63f,-.50f,-.16f,.35f,.39f,.51f,.65f,-.12f,.24f,-.88f,.08f,.56f,.92f,-.77f,.93f,.79f,-.06f,-.82f,.00f,-.67f,.49f,.83f,.35f,-.47f,-.67f,-.94f,-.23f,.71f,-.06f,-.06f,.98f,-.94f,-.79f,.03f,-.43f,-.64f,-.27f,-.32f,-.78f,.40f,-.39f,.96f,.81f,.36f,-.64f,.41f,.97f,-.08f,.01f,.31f,-.08f,-.85f,-.09f,-.40f,-.91f,-.11f,-.29f,.79f,-.40f,-.20f,-.92f,.72f,-.25f,.40f,-.20f,.47f,-.52f,.86f,.68f,-.68f,.15f,.99f,.30f,-.99f,-.51f,.44f,-.96f,-.71f,.83f,-.37f,-.12f,-.58f,-.11f,.60f,-.96f,-.35f,-.13f,.52f,.99f,-.94f,-.49f,-.88f,-.50f,.11f,-.79f,-.04f,-.03f,-.60f,-.16f,-.93f,-.85f,-.73f,-.47f,-.22f,-.27f,-.52f,.19f,-.39f,.96f,-.23f,.72f,-.19f,-.71f,.00f,-.78f,.55f,.90f,.47f,-.33f,-.02f,.04f,-.66f,-.34f,-.32f,-.34f,.34f,-.93f,.38f,-.43f,.70f,.90f,-.76f,-.16f,.54f,-.30f,.57f,.20f,-.68f,.62f,.03f,.58f,.41f,-.59f,.54f,.96f,-.04f,-.84f,-.34f,-.76f,.89f,-.86f,-.84f,-.22f,-.61f,-.64f,-.25f,.07f,-.37f,-.85f,-.41f,.59f,.11f,-.97f,.68f,.28f,-.42f,-.19f,.79f,-.46f,.42f,.83f,.97f,-.18f,.16f,-.04f,-.22f,.04f,.14f,.96f,.61f,.72f,-.71f,.23f,-.85f,.06f,.20f,-.72f,.31f,.84f,-.57f,-.23f,-.31f,.17f,.39f,.56f,.31f,.46f,-.15f,-.11f,-.86f,.51f,-.43f,-.63f,.65f,.51f,-.54f,-.58f,.13f,.55f,-.77f,-.64f,-.81f,.18f,-.08f,.21f,-.13f,.18f,.64f,.55f,-.73f,-.35f,.39f,-.79f,.68f,.52f,-.68f,.69f,.42f,-.64f,-.98f,.76f,-.56f,.76f,-.06f,-.64f,-.99f,-.16f,.19f,.90f,-.31f,-.98f,-.71f,-.36f,-.19f,-.03f,.98f,-.34f,-.50f,.91f,-.38f,.20f,-.13f,.62f,-.09f,.66f,-.69f,.25f,-.81f,-.20f,-.16f,-.23f,.26f,.69f,.05f,-.88f,-.71f,-.02f,.96f,.56f,.48f,-.50f,-.68f,-.83f,-.17f,.38f,.48f,.04f,-.05f,-.47f,.46f,.85f,.13f,.08f,.39f,.89f,-.36f,.05f,-.60f,-.95f,-.95f,.08f,.76f,.82f,-.62f,.32f,-.09f,.55f,-.26f,.08f,-.88f,-.93f,-.96f,.26f,.08f,.98f,.40f,.34f,-.34f,-.38f,.35f,.68f,.28f,-.58f,.66f,.30f,-.66f,-.25f,.42f,.05f,-.19f,.33f,.47f,.39f,-.03f,.63f,.09f,.65f,.83f,-.67f,-.86f,.72f,-.20f,.35f,-.88f,-.37f,-.52f,-.29f,-.83f,-.02f,-.08f,.25f,-.75f,.42f,-.23f,-.37f,.48f,-.75f,-.11f,.87f,.87f,.03f,.33f,-.26f,-.43f,.65f,.45f,.77f,.99f,-.52f,.97f,-.72f,.74f,.38f,.86f,-.26f,.81f,.94f,.28f,.16f,-.69f,-.50f,-.78f,-.40f,.11f,-.10f,.89f,.88f,.57f,-.75f,-.55f,-.49f,.06f,.18f,-.56f,.18f,.92f,-.63f,.94f,.01f,.39f,.46f,.44f,.90f,.16f,.77f,-.58f,-.69f,.88f,.12f,.10f,.27f,-.89f,-.94f,-.64f,.55f,-.87f,-.70f,-.47f,.93f,.73f,-.87f,-.20f,-.21f,.61f,-.98f,-.67f,-.45f,.82f,.77f,-.38f,-.29f,-.61f,-.78f,.93f,-.74f,-.48f,.51f,.03f,.73f,-.53f,-.55f,.99f,.08f,.99f,.88f,.57f,.04f,-.98f,.12f,-.39f,.47f,-.54f,-.65f,-.29f,-.51f,-.92f,-.23f,.73f,-.76f,.50f,.11f,.38f,-.44f,.04f,-.40f,-.27f,-.95f,-.83f,.77f,-.20f,-.27f,-.36f,.59f,.65f,-.69f,-.53f,-.86f,-.55f,-.68f,.80f,.55f,.69f,-.41f,-.49f,.95f,.99f,.01f,-.17f,-.72f,-.70f,-.05f,-.56f,.92f,-.37f,.00f,-.38f,-.18f,-.63f,-.46f,-.45f,.82f,.54f,.31f,.41f,-.00f,.08f,-.91f,.16f,.35f,-.62f,-.36f,.22f,.26f,-.31f,-.91f,.48f,.41f,.87f,-.34f,.69f,.24f,-.83f,.90f,.76f,.17f,.11f,.85f,.03f,-.66f,-.29f,.58f,.31f,-.87f,-.90f,-.48f,.21f,-.34f,-.87f,.60f,.91f,-.85f,-.59f,.20f,.64f,-.55f,-.20f,.77f,-.70f,.36f,.44f,-.66f,-.12f,-.56f,-.47f,.37f,-.45f,.43f,.93f,.76f,.03f,-.42f,-.12f,.91f,-.25f,-.93f,-.71f,.00f,.32f,.41f,-.72f,.38f,-.29f,-.83f,.71f,-.73f,-.68f,.81f,.83f,.08f,.74f,-.37f,-.17f,.01f,-.97f,-.35f,-.79f,.80f,-.27f,.28f,-.50f,-.55f,.70f,.36f,.68f,-.25f,-.08f,.84f,-.28f,-.92f,.54f,-.50f,.82f,.23f,-.01f,.09f,.61f,.42f,-.48f,.01f,.56f,.96f,.28f,.67f,-.48f,-.98f,-.56f,-.05f,-.55f,.91f,.77f,-.98f,.55f,.89f,.56f,.58f,.42f,.87f,.40f,.28f,-.51f,.74f,.99f,.77f,.55f,.48f,.58f,-.28f,-.16f,.69f,.19f,-.15f,-.67f,-.89f,-.63f,-.49f,.69f,.14f,.41f,.37f,.96f,.11f,-.89f,-.19f,.83f,.67f,-.51f,.17f,-.79f,.53f,-.27f,-.41f,-.93f,-.01f,.31f,.17f,.20f,.79f,.84f,.90f,-.02f,.59f,-.77f,-.47f,.59f,-.99f,-.88f,.78f,.43f,.90f,.08f,.71f,-.12f,.91f,-.82f,-.86f,-.61f,.28f,.35f,.60f,.55f,-.60f,-.67f,.67f,.99f,-.30f,.24f,.47f,-.17f,-.82f,-.22f,.40f,.01f,-.75f,.45f,.38f,.70f,-.44f,.96f,.72f,.12f,-.58f,.96f,-.41f,-.56f,-.49f,-.95f,-.36f,.62f,-.10f,-.65f,-.51f,-.13f,-.44f,.21f,.75f,-.82f,.75f,-.76f,.20f,-.63f,-.95f,-.10f,-.30f,.85f,.59f,-.76f,-.36f,.75f,-.05f,-.47f,-.67f,-.84f,-.24f,-.22f,.24f,.88f,-.33f,-.60f,-.81f,-.22f,-.55f,-.10f,.84f,-.52f,.31f,.31f,.73f,-.47f,.08f,-.78f,.59f,.27f,.06f,.74f,-.58f,.48f,-.74f,-.60f,-.57f,.16f,-.91f,.48f,-.08f,.42f,.56f,-.31f,-.42f,-.33f,.88f,.84f,.72f,.92f,.36f,.95f,-.40f,.77f,.12f,-.63f,-.67f,-.02f,-.65f,.53f,-.98f,.23f,-.92f,-.88f,-.64f,.84f,.44f,-.08f,-.62f,-.85f,-.16f,.61f,-.11f,.23f,.01f,-.37f,-.36f,.42f,-.73f,.43f,.96f,-.67f,.41f,.94f,-.27f,.04f,-.46f,.26f,.01f,.75f,-.10f,-.41f,-.28f,.57f,-.55f,.18f,-.36f,-.96f,-.15f,-.04f,.80f,.09f,-.84f,.06f,-.63f,.82f,.47f,-.17f,.01f,-.81f,.44f,.19f,-.26f,.88f,.57f,-.14f,-.00f,.34f,.44f,-.20f,-.58f,-.32f,.17f,-.98f,-.29f,-.08f,-.53f,.89f,-.96f,-.43f,.87f,.90f,.25f,-.00f,.55f,.73f,-.28f,.63f,-.73f,.61f,.14f,.51f,-.15f,.90f,.15f,-.24f,.48f,-.74f,.54f,.75f,-.59f,-.22f,.54f,-.99f,.29f,.47f,.87f,-.05f,.65f,.46f,.76f,-.89f,.63f,.65f,-.28f,-.63f,.54f,.37f,.03f,.05f,-.16f,-.38f,.60f,.70f,-.74f,.86f,-.41f,.22f,-.45f,.75f,-.30f,.32f,-.03f,.42f,.48f,.01f,-.79f,-.87f,-.76f,-.51f,-.02f,.00f,.40f,.35f,.78f,-.89f,-.66f,-.15f,-.41f,-.28f,.41f,.37f,.47f,-.56f,.27f,.58f,-.44f,.73f,-.55f,-.54f,.87f,-.67f,-.62f,-.95f,.52f,-.20f,.81f,-.34f,-.48f,.49f,-.23f,.62f,.74f,-.60f,-.39f,-.60f,.34f,-.81f,.17f,.63f,.37f,.11f,-.74f,.60f,-.47f,-.96f,-.42f,-.54f,.20f,.85f,.13f,.83f,.17f,-.56f,-.25f,.69f,-.11f,.11f,.09f,-.83f,-.02f,-.44f,-.50f,-.00f,-.35f,.25f,-.41f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1993000055074691800882646930404007434844970703125f,-0.956799974215030157864703141967765986919403076171875f,0.33320001341700500319120692438445985317230224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.22530001544654343348383918055333197116851806640625f,-1.469699995505809653195683495141565799713134765625f,-0.4057999984741211818572992342524230480194091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4320000252798201767490127167548052966594696044921875f,-1.15859994686543910091813813778571784496307373046875f,1.7572999796316028930931452123331837356090545654296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2460999610587959463003926430246792733669281005859375f,-1.0153999967038627172399856135598383843898773193359375f,0.5274000048846010368919223765260539948940277099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8876000217512245438200579883414320647716522216796875f,0.4402000493325302077352034757495857775211334228515625f,-1.052000020883977260410802045953460037708282470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1013999843657018562481653134454973042011260986328125f,-0.7279000543341040962985744044999592006206512451171875f,2.23990002023279632936691996292211115360260009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.258300069887936967205632754485122859477996826171875f,-0.61610000203698833676213553189882077276706695556640625f,1.11350002551078741674928096472285687923431396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.10279999138116835410983185283839702606201171875f,-0.4631000034749508387221794691868126392364501953125f,1.0385999837458133043810448725707828998565673828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4296999754637482737962272949516773223876953125f,-0.1741000222802160379842462134547531604766845703125f,0.11460004785060817766861873678863048553466796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.274100034797191138835614765412174165248870849609375f,0.61489998927563416231123483157716691493988037109375f,-0.81110000397413983819205895997583866119384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.29990001322925208260272711413563229143619537353515625f,-0.152099967104941280471308573396527208387851715087890625f,0.527800012578069299706839956343173980712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.32169998710900504779175435032811947166919708251953125f,-0.888700077506900498747199890203773975372314453125f,-0.30750002043321755795801664135069586336612701416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0328999818027024826250226396950893104076385498046875f,1.098900067029893623526959345326758921146392822265625f,1.353400017431378277166231782757677137851715087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.40490005696415920510844443924725055694580078125f,0.5074000259511175503490676419460214674472808837890625f,-0.9468000086739654097556240230915136635303497314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.121000022232532700172669137828052043914794921875f,-0.647900026655197081026926753111183643341064453125f,0.362899996517598300016516077448613941669464111328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.78319997974038102483973489142954349517822265625f,-0.06859998449087090222064944100566208362579345703125f,0.23090000394582776976903915056027472019195556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.80570001364052312720076542973401956260204315185546875f,0.1113000294834372017049872738425619900226593017578125f,-0.10880001879185441371333809001953341066837310791015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.480899992881715032666534170857630670070648193359375f,2.3068999458238490518624530523084104061126708984375f,1.0037000653438268304995517610223032534122467041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.34580002745687987708578248202684335410594940185546875f,0.33169995749294789444405751055455766618251800537109375f,-0.998499967917799491345931528485380113124847412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.36840000372231018754831666228710673749446868896484375f,0.5838000193506476254157178118475712835788726806640625f,-1.437399986369908067018741348874755203723907470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.16060000734925239385120221413671970367431640625f,-0.33479999408572835850605997620732523500919342041015625f,-0.5360999769285310634359120740555226802825927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.00730001230239896159446288947947323322296142578125f,-1.47100000464916291775807621888816356658935546875f,0.21660002619177054494770118253654800355434417724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.265299988996982705202754004858434200286865234375f,-0.4564999715834854310259061094257049262523651123046875f,0.4383000306606295026767838862724602222442626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.35759999515265139535813432303257286548614501953125f,-0.372300013531745133121830804157070815563201904296875f,-1.483700014549493406690316987805999815464019775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6672999804697929970842551483656279742717742919921875f,-1.479399890968948749758737903903238475322723388671875f,0.2832000015519564595223300784709863364696502685546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.81929999066218683889673002340714447200298309326171875f,-0.4244000578433275716605521665769629180431365966796875f,-1.7139000343590986563668820963357575237751007080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5063999135479324564812486642040312290191650390625f,0.9844000463321809224481739875045605003833770751953125f,-0.94360003008842496807773159162024967372417449951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.45260000437274516826846593176014721393585205078125f,-0.9113000028476114788844597569550387561321258544921875f,-0.73750003114342721488583265454508364200592041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0795999970376490839640837293700315058231353759765625f,-0.031499981008469735144217338529415428638458251953125f,0.00090002752691542564633664369466714560985565185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.10739998894035807097679935395717620849609375f,-0.9768000025272367548012653060141019523143768310546875f,-0.51250004306435581469258977449499070644378662109375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.183700020919740580893630976788699626922607421875f,1.4871999162502593794243921365705318748950958251953125f,-0.333099974529444953219581293524242937564849853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.78879994899854199985611558076925575733184814453125f,-1.6831999529182921637726622066111303865909576416015625f,0.4256999577239157783736800411134026944637298583984375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.08069992572367201688621207722462713718414306640625f,0.6646999580666423934616204860503785312175750732421875f,-0.146499981008471003018911460458184592425823211669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0477000133648513013895353651605546474456787109375f,0.123999993208796233101764983075554482638835906982421875f,-1.464400011351704389284122953540645539760589599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.475700038935244429172399804883752949535846710205078125f,-0.40380000053793219105813250280334614217281341552734375f,1.651200056868791676834007375873625278472900390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.481600035896152522951751961954869329929351806640625f,-0.5019999767392870904103574503096751868724822998046875f,0.82100001746416051418009374174289405345916748046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.413499969482421736444166526780463755130767822265625f,-0.08200003790855436847095916164107620716094970703125f,-1.804900001122057506819373884354718029499053955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.378000048033893421006723656319081783294677734375f,-2.455999938823282935373981672455556690692901611328125f,1.42579995159134309545834184973500669002532958984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.70130010395199082751105379429645836353302001953125f,0.6622999157242481782503773501957766711711883544921875f,-2.1663999363653356766690194490365684032440185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8010999861672518651545260581769980490207672119140625f,-1.2151999514505262833807819333742372691631317138671875f,2.486799993921815055131219196482561528682708740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.93079994192421366960843442939221858978271484375f,-1.82110003310590951031144868466071784496307373046875f,0.14899998916685597283304787197266705334186553955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7832000747725367251206307628308422863483428955078125f,1.2853999885454772122983513327199034392833709716796875f,0.337799952582269913392565285903401672840118408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3652000443406409058155759339570067822933197021484375f,-0.20030001511126671598361781434505246579647064208984375f,1.511099998851865411353401214000768959522247314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7892000065922732954248886017012409865856170654296875f,0.93280001671314227706943711382336914539337158203125f,0.70290001899972498478064153459854423999786376953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5253000469252453452639883835217915475368499755859375f,2.6631998892158268432694967486895620822906494140625f,-0.9510000142976637871328193796216510236263275146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.786799980603903659215347943245433270931243896484375f,-0.25840007308721701928533320824499242007732391357421875f,-2.3821999245762821573180190171115100383758544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.990000033006073199004504203912802040576934814453125f,-1.2688999423444273784156166584580205380916595458984375f,-0.9029999285079528537067972138174809515476226806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.55320003900974989985428464933647774159908294677734375f,1.3971000229284182836408945149742066860198974609375f,0.898100094632804601957332124584354460239410400390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4102999815836556596337914015748538076877593994140625f,-1.474200001563132023107982604415155947208404541015625f,1.0046999829888341171368892901227809488773345947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4418000396683809949394117211340926587581634521484375f,-1.2263000149548053396841851281351409852504730224609375f,-1.5323000629663463545426793643855489790439605712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.160799941477180130533497504075057804584503173828125f,-1.3020999983787537868096251258975826203823089599609375f,0.3163000353693963262458055396564304828643798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.238900008580088307752475884626619517803192138671875f,1.643499986059963280382589800865389406681060791015625f,0.84669993052184533421922196794184856116771697998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.701299971462041593639469283516518771648406982421875f,-0.25649980051815568504736120303277857601642608642578125f,0.30950010209530531835042665989021770656108856201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2448999839484702345515643173712305724620819091796875f,0.413400060980021744061474464615457691252231597900390625f,-1.3992000038355587587801664994913153350353240966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6265001000314966983495423846761696040630340576171875f,2.049200042857975301302531079272739589214324951171875f,3.45630003715753542081756677362136542797088623046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.43250004515051865272567965803318656980991363525390625f,-0.53879996682405384422764882401679642498493194580078125f,-0.848499979875982024424274641205556690692901611328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.34030006620362363012333162259892560541629791259765625f,-0.8131000628560787202303572485106997191905975341796875f,0.334699953484534962200314112124033272266387939453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.471900032809376757114705469575710594654083251953125f,-0.6730000230371953051644595689140260219573974609375f,-0.91479994504228245855159684651880525052547454833984375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2117000114165239477159730085986666381359100341796875f,-2.657799976666272101510912762023508548736572265625f,-0.23740001276359035120577800626051612198352813720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.4767999198257939497125335037708282470703125f,0.25910014957338400876096784486435353755950927734375f,2.173500066526235396935362587100826203823089599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5941999926783154695186794924666173756122589111328125f,3.4663999871224167037553343106992542743682861328125f,1.0532000320248295910374736195080913603305816650390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.406900018634647420157079977798275649547576904296875f,-0.2094000003620986039010176682495512068271636962890625f,-0.2761999985307443949977823649533092975616455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.936000036351382735944071100675500929355621337890625f,-0.5048000343374912102234475241857580840587615966796875f,-0.51049999058991668565710142502211965620517730712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0184999656453725958016320873866789042949676513671875f,-1.873999943997711081777879371657036244869232177734375f,0.5474999941885470011726511074812151491641998291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.109200013223290426367384498007595539093017578125f,1.22919995816350091644153508241288363933563232421875f,-0.56409997899234287643821517121978104114532470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.12139990222305119260681749437935650348663330078125f,0.17039991694539846633915658458136022090911865234375f,2.48229994019940480853847475373186171054840087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.32949991445243487220295719453133642673492431640625f,-2.142400112079827945166243807761929929256439208984375f,0.3433000350750969875690543631208129227161407470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.85099999801814518729514702499727718532085418701171875f,1.6976999036923043906455177420866675674915313720703125f,-0.12719999204129006642460808507166802883148193359375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.591299981799720431041578194708563387393951416015625f,0.81009995821639868562868969092960469424724578857421875f,0.59019998818188745826063268395955674350261688232421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.67929999096020932114470269880257546901702880859375f,0.26309999621063429042777670474606566131114959716796875f,0.74360000848174079290942017905763350427150726318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.55820001166611898923974877106957137584686279296875f,0.8283000065952546009384604985825717449188232421875f,1.171100002819299934486707570613361895084381103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.31560003508403922101166472202748991549015045166015625f,0.903800029446184538528541452251374721527099609375f,-0.04990002243071867127355289994738996028900146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.56029996401891057900002124370075762271881103515625f,-0.3435000525936489079725788542418740689754486083984375f,-3.71630001901537099229244631715118885040283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5269999816939232584189767294446937739849090576171875f,-0.835599999861419195923417646554298698902130126953125f,-1.779699907961487692631408208399079740047454833984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.09589998641833730186334605605225078761577606201171875f,-1.44870014940500357880637238849885761737823486328125f,1.084399975402653648615114434505812823772430419921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.7106999619707463633488941923133097589015960693359375f,-1.373399970977008166528321453370153903961181640625f,1.7637000440165404580028507552924565970897674560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.40419999534189710033160736202262341976165771484375f,-0.21620000586956800159299518782063387334346771240234375f,-1.0840000344291329525958644808270037174224853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.94789997465014419475437534856610000133514404296875f,-1.51250001698732372545919133699499070644378662109375f,1.561900074383616665585350347100757062435150146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.064999958314001293757655730587430298328399658203125f,0.71749996438622520855687980656512081623077392578125f,0.683100013384223103685144451446831226348876953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3350000024586901048451181850396096706390380859375f,-3.022899984149634899921466058003716170787811279296875f,0.6483999903671449072106724997865967452526092529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9003000791303801264575668028555810451507568359375f,-0.5908999996617432248768864155863411724567413330078125f,-5.80310014864951284607741399668157100677490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7849999593570817779664139379747211933135986328125f,-0.7510000589825214234451777883805334568023681640625f,0.132099982807040217114291635880363173782825469970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.34849990667402674926478312045219354331493377685546875f,-0.451900007924436575290627615686389617621898651123046875f,0.069499959826470447676882713494705967605113983154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.458300022539496421103422107989899814128875732421875f,1.4305999786645162696885336117702536284923553466796875f,0.9371999891728162168647031649015843868255615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.32710001477003114445096798590384423732757568359375f,1.0853000766158105161451885578571818768978118896484375f,-0.5018000072956088075670777470804750919342041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.91990003379285401052811721456237137317657470703125f,-0.8188000082492834508229861967265605926513671875f,1.08590000752210613654824555851519107818603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3268000273004172395729938216391019523143768310546875f,1.68199998307228071325880591757595539093017578125f,0.86599999693781215537313755703507922589778900146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.514799972441793318722602634807117283344268798828125f,2.685499996662139121639256700291298329830169677734375f,-1.943299974855779765903207589872181415557861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8512999537110326198074972126050852239131927490234375f,-0.21840003788322170930058518933947198092937469482421875f,1.700400005868077268900151466368697583675384521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.38859997879117680330551820588880218565464019775390625f,0.539200027528405900767438652110286056995391845703125f,-0.40179999660402521133306663614348508417606353759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1979000219613311628563678823411464691162109375f,0.9915000082030893668871840418432839214801788330078125f,-1.343200010958314205566921373247168958187103271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3993000266671185016775780241005122661590576171875f,-0.8609999861717227975788091498543508350849151611328125f,0.3841999630808832222328419447876513004302978515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.367999984145164216187140482361428439617156982421875f,-0.5563999711036675677178209298290312290191650390625f,1.2801999693691730186628774390555918216705322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.849199947658181297782675756025128066539764404296875f,0.21019997212588759083473632927052676677703857421875f,1.242099970513582007924924255348742008209228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.151400018638372879653530844734632410109043121337890625f,-0.34040001899972605059474517474882304668426513671875f,0.60490002023279654252974069095216691493988037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7733000262089071430438025345210917294025421142578125f,1.6789000368177884325859849923290312290191650390625f,2.038899965776502565262262578471563756465911865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.15540000214278659740330112981610000133514404296875f,-0.959500004827976482602025498636066913604736328125f,-0.574099989125132825762420907267369329929351806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4639000137209892127287957919179461896419525146484375f,-0.7719000248186291823770943665294907987117767333984375f,0.49959998444616804391671394114382565021514892578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.811300049525499389346805401146411895751953125f,-1.211399999415874617625377140939235687255859375f,0.0500000095367436614424150320701301097869873046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8706000206112864958640784607268869876861572265625f,-0.698300013226270888111457679769955575466156005859375f,0.11980003405809380723212598240934312343597412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.66829995615482307158572439220733940601348876953125f,0.19559999896734925339814026301610283553600311279296875f,-1.032199954155087251450595431379042565822601318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9954999787062401850334936170838773250579833984375f,-1.6290000107735396195351995629607699811458587646484375f,-0.6236000221908100105139283186872489750385284423828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0267999684780837466036018668091855943202972412109375f,1.2655000473260875626380084213451482355594635009765625f,-0.636800041940808636553583710337989032268524169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.182999985784292551471708065946586430072784423828125f,0.150800031629203967753483084379695355892181396484375f,2.44289993803054183985068448237143456935882568359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.582699983264505672053701346158050000667572021484375f,-0.467500031739473076441981902462430298328399658203125f,-0.1094000560551879885906600975431501865386962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.11050000917911528830472889239899814128875732421875f,0.090599977248907226368146439199335873126983642578125f,0.1237000139906998530392456814297474920749664306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.066299992752075098678687936626374721527099609375f,0.428200005742907396921737017692066729068756103515625f,0.75890001103878024224513865192420780658721923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.597100033545494124354036102886311709880828857421875f,0.2169000084459782318191400918294675648212432861328125f,-1.01259998162984832248412203625775873661041259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.36879995328262449305611880845390260219573974609375f,2.273800002251566088062872950104065239429473876953125f,-1.2402000196605917903269755697692744433879852294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.08119999365061525420372845474048517644405364990234375f,1.2617000232815740456970843297312967479228973388671875f,1.1718000112071640383959447717643342912197113037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.529600068823992575772763302666135132312774658203125f,-1.3363000620424745346070949381100945174694061279296875f,1.75169998140931237884387883241288363933563232421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.636500008296221775339063242427073419094085693359375f,0.7481999758474524586659981650882400572299957275390625f,1.0592000304527584564340259021264500916004180908203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8874999876320368574766916935914196074008941650390625f,1.3282000609144570812958363603684119880199432373046875f,1.866099997045099545545099317678250372409820556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.19180003747046026063571844133548438549041748046875f,0.808400018846988199783254458452574908733367919921875f,-1.7134999854266641516886693352716974914073944091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1600000150501739160091574376565404236316680908203125f,-0.7141999811299155620503142927191220223903656005859375f,-1.45450001198053424644740516669116914272308349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4547998752489699203493955792509950697422027587890625f,2.563500026665627817834547386155463755130767822265625f,0.9651999252989893474108384907594881951808929443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.9522998956821862037713799509219825267791748046875f,0.5550999826729299257976890658028423786163330078125f,-1.83129996704757136427588193328119814395904541015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.195300017383695401207432951196096837520599365234375f,1.512400034146755611885737380362115800380706787109375f,-0.0924999701231714599458655357011593878269195556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.70680000278800736435158569292980246245861053466796875f,0.729099949562549642934072835487313568592071533203125f,-0.723600032807885984453832861618138849735260009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.967200029778480629971681992174126207828521728515625f,0.086400002023577560095191074651665985584259033203125f,-1.12730004672408146149109597899951040744781494140625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.640799977463483827477830345742404460906982421875f,-0.7906999781012533645707662799395620822906494140625f,1.4037000238627201920138531932025216519832611083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.26240001598596496279469647561199963092803955078125f,-0.30840002760142193327652648804360069334506988525390625f,-1.72340001053959124277525916113518178462982177734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.586100082242489772710314355208538472652435302734375f,0.11500008249655468972605376620776951313018798828125f,-1.396099924960732874268387604388408362865447998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.48679998166561111450079124551848508417606353759765625f,-1.3101999410197142648115686824894510209560394287109375f,-0.0333999992147087088056878201314248144626617431640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.40970001320093896168827996007166802883148193359375f,-1.4905999969929462078965798355056904256343841552734375f,-1.54299998136982363661218187189660966396331787109375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.447200000199674985168485363828949630260467529296875f,-1.5128000051274888004826379983569495379924774169921875f,0.960499991819262266545820239116437733173370361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.664299969297647674437712339567951858043670654296875f,1.08940003932863493929517062497325241565704345703125f,1.8920000182762743978770458852523006498813629150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8375000361353162059430133012938313186168670654296875f,-1.0163999711036684203691038419492542743682861328125f,0.79380002359747925400057511069462634623050689697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.008700019317864349233104803715832531452178955078125f,0.5063999762818227612370947099407203495502471923828125f,0.253000073552132054288676954456605017185211181640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.602900048504024343998253243626095354557037353515625f,-1.877599953764675699829922450589947402477264404296875f,3.0492999830998481769483987591229379177093505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8779999931603679197422707147779874503612518310546875f,-1.62760004555583037699761916883289813995361328125f,0.01629992284700210181114243823685683310031890869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3560999523416155110311365206143818795680999755859375f,0.7190999429315321211930722711258567869663238525390625f,-2.07400001643598042022631489089690148830413818359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.587499968707561048830712024937383830547332763671875f,0.7575000175461175100366517654038034379482269287109375f,-0.47670000070631479704985622447566129267215728759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6372000342115757831606970285065472126007080078125f,-0.68199998381733928365378005764796398580074310302734375f,1.5334000595271586320222922950051724910736083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9650000078231097955239192742737941443920135498046875f,0.701599998140335134877432210487313568592071533203125f,-1.657499972432851631509720391477458178997039794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.50600007896870291546065345755778253078460693359375f,0.13069996172860387506631241194554604589939117431640625f,-1.3508999749258163713960811946890316903591156005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.38400002783537023898219331385917030274868011474609375f,0.0563998992055656511723782386980019509792327880859375f,0.5665999401003121160869113737135194242000579833984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.01219994551241387892304146589594893157482147216796875f,2.11489994870722330944090572302229702472686767578125f,-1.26210004073530424051341469748876988887786865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.26720002243965979005224653519690036773681640625f,-0.12049995724856898959842510521411895751953125f,-0.5515999572739007650312714758911170065402984619140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6913000293903059922939746684278361499309539794921875f,0.72039997435212033138896003947593271732330322265625f,-0.5912000246077777632081051706336438655853271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.547499970588833040352483294554986059665679931640625f,-1.68329999184310441506795541499741375446319580078125f,0.2438999690175054357865747078903950750827789306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4526000004053114533775215022615157067775726318359375f,0.67420000241994859635497050476260483264923095703125f,-1.5118999925576150911155082212644629180431365966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1070999758780011301695367365027777850627899169921875f,2.40309998417794634661959207733161747455596923828125f,-1.3539001351468262779320639310753904283046722412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.42889997581616068345056191901676356792449951171875f,1.7277999642796817436618539431947283446788787841796875f,-3.8310998460404590559846838004887104034423828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0392999564453955230192150338552892208099365234375f,-0.2888999858930707542725713210529647767543792724609375f,-2.08000003842636882467331815860234200954437255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.1874999923817828317851308383978903293609619140625f,0.352099967794119894648474655696190893650054931640625f,-1.0799999965168531890213898805086500942707061767578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.83190004283040785981739873022888787090778350830078125f,0.6677000071808694148245422184118069708347320556640625f,0.0341999999240041319126248708926141262054443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.26919998807758138692491911569959484040737152099609375f,-0.760800016877055451658407037029974162578582763671875f,0.48230004448890706481023471496882848441600799560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.6788000633090742752528967685066163539886474609375f,1.606800036948919707668892442598007619380950927734375f,0.09769996348321452472873716033063828945159912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,3.146000000700354792826374250580556690692901611328125f,-2.851399916416406110641901250346563756465911865234375f,0.258599941799044774626281650853343307971954345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.286599915662408388783433110802434384822845458984375f,2.307000031705945364279841669485904276371002197265625f,1.93149991331994552723472224897705018520355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.095499939870089978199985125684179365634918212890625f,2.27699998281151128054489163332618772983551025390625f,0.65749994859099469834973206161521375179290771484375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.286100024500488547829490926233120262622833251953125f,-1.4477999748781316124990325988619588315486907958984375f,-2.914999982267618694464772488572634756565093994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.11619995714277042697659680925426073372364044189453125f,-1.659100026229022972756865783594548702239990234375f,-0.8030000135935839722378659644164144992828369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.201700022163987580103849950319272466003894805908203125f,-1.4570999854147432817086382783600129187107086181640625f,1.823600022637844819684005415183492004871368408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.2229000218123200482978063519112765789031982421875f,-0.0978999878749247098852492854348383843898773193359375f,-0.4474000029847025050599995665834285318851470947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.77410004310458813137074685073457658290863037109375f,-1.1999000293225048352496742154471576213836669921875f,0.795200042459368372504968647263012826442718505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.68240003610253185950540455451118759810924530029296875f,0.5775000516138961703660470448085106909275054931640625f,1.1665999894976619710718068745336495339870452880859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5315000026710336999968831150908954441547393798828125f,-0.63919998804032740480352003942243754863739013671875f,0.98210008279383220752833949518389999866485595703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.7143999869696797677676158855319954454898834228515625f,0.7840999574974183428111018656636588275432586669921875f,0.65779996426105480367141353781335055828094482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8478999837771052927593018466723151504993438720703125f,0.4295999756917361356300943953101523220539093017578125f,0.12810001524686820406628839918994344770908355712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3803999949902294730463836458511650562286376953125f,-0.428899970805645125437877140939235687255859375f,1.13239999303817739928490482270717620849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.488900072692334930479773902334272861480712890625f,-1.0820000037848951368602001821273006498813629150390625f,1.4339999990016216724342257293756119906902313232421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.059600104326009262223351470311172306537628173828125f,0.1005000104457132170665545345400460064411163330078125f,-0.3002000214673585798408339542220346629619598388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,4.60259995644688668647859230986796319484710693359375f,1.700000051856040084174992443877272307872772216796875f,1.058400031587481304740094856242649257183074951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.91229998495876873931820227880962193012237548828125f,-2.4081999944552787695783990784548223018646240234375f,-0.17840007170885752163513870982569642364978790283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8023999901697040915138359196134842932224273681640625f,1.56170000506490413982874088105745613574981689453125f,0.13289992858693022714788867233437485992908477783203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.819199995863436836174287236644886434078216552734375f,-0.607099970774351849200911601656116545200347900390625f,0.42169997742325104894689502543769776821136474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.21189999637603751381220718030817806720733642578125f,1.2748000537276269472641843094606883823871612548828125f,-0.74830003602504735482625619624741375446319580078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8837999515503636160929090692661702632904052734375f,1.4573999805212007885302227805368602275848388671875f,-0.3891999717235563593931146897375583648681640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.424900014235078060664818622171878814697265625f,-2.268000011116267256028322663041763007640838623046875f,-3.178499997384846675885228250990621745586395263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.193599982330202369240623738733120262622833251953125f,-0.0272999765396118032612093884381465613842010498046875f,1.485799977165461971395643558935262262821197509765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.01500001832842912108390009962022304534912109375f,-1.110300000321867486263727187179028987884521484375f,-2.764500035226345797667590886703692376613616943359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.555500046618283338517585434601642191410064697265625f,-1.8084999856501813297882108599878847599029541015625f,-1.2166000239938494775771005151909776031970977783203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2143000253260129994714588974602520465850830078125f,0.843800015252828217171554570086300373077392578125f,0.447500019073487464993377216160297393798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.84660004552602874383637754363007843494415283203125f,-0.223599998423458234952931888983584940433502197265625f,-0.578900010442733758964095613919198513031005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6426999926149843389566740370355546474456787109375f,-0.60829995675086845352552700205706059932708740234375f,-1.341900008744001837612813687883317470550537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.638600001478195178350460992078296840190887451171875f,1.28599997957795775249678627005778253078460693359375f,-0.51740001047253514343537972308695316314697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.627800007027389028735342435538768768310546875f,0.47899991553649112763224593436461873352527618408203125f,3.034399965530635512322987779043614864349365234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.60800001007318427159731299980194307863712310791015625f,-1.31229993921220255970183643512427806854248046875f,-0.16770001869201733857295266716391779482364654541015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.229599937880038851290009915828704833984375f,0.09439994662478501386004836604115553200244903564453125f,1.476799945008753667252676677890121936798095703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4525000197440378624236245741485618054866790771484375f,1.158599989296495547108634127653203904628753662109375f,-1.227899964964390022714724182151257991790771484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.447600034528971235658900695852935314178466796875f,1.19150001770257940592045997618697583675384521484375f,0.92389999716207460433992082471377216279506683349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.12629999752044707150844260468147695064544677734375f,-0.757300059241056811032422046991996467113494873046875f,0.80650002038478785237884949310682713985443115234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.383300048393010961422788795971428044140338897705078125f,2.75039993434250362724924343638122081756591796875f,-0.97499994151294189226319986119051463901996612548828125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3075001106411214824021271851961500942707061767578125f,1.34160005044341179569755695411004126071929931640625f,1.7399000601865350912333951782784424722194671630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.34740000928044256767179831513203680515289306640625f,-0.5709001058697698027799560804851353168487548828125f,2.81700001853704407039913348853588104248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6695999793797733357081369831576012074947357177734375f,1.150599989467858907943309532129205763339996337890625f,-0.84180004097223160020035948036820627748966217041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1942999917238938945018844606238417327404022216796875f,-0.2383999902367595069563321885652840137481689453125f,-0.55549989920854603298039364744909107685089111328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.73550002974271766476022094138897955417633056640625f,1.98860004498958620189341672812588512897491455078125f,0.9188999795600765185810132607002742588520050048828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5188999949455261884168066899292171001434326171875f,0.358299999144672920436960339429788291454315185546875f,-1.12650002396106696522792844916693866252899169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0928000267714266247054410996497608721256256103515625f,-1.5256000025063738423369841257226653397083282470703125f,-0.027399964278936206341086290194652974605560302734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.20890000207945791377284194823005236685276031494140625f,-0.24999998383224031339722159827942959964275360107421875f,1.8586001458704470845617606755695305764675140380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.42300003063678570658368016665917821228504180908203125f,-0.413199977952241948031542051467113196849822998046875f,-0.817099985377490067861572242691181600093841552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8879000069111582238434721148223616182804107666015625f,2.394600056083501016246373183093965053558349609375f,0.50310004743337533028579855454154312610626220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.29489999128729016408811958172009326517581939697265625f,-1.253400024062395257118396330042742192745208740234375f,0.947800041896104783489818146335892379283905029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9148000236459079559381279977969825267791748046875f,-2.23450002408772707696016368572600185871124267578125f,0.312100095199048244154482745216228067874908447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2915999952346093326838172288262285292148590087890625f,0.2039000295162196607634541578590869903564453125f,0.04490001375079177847737810225225985050201416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.42370004818886553010770512628369033336639404296875f,-0.8650999582164000667461323246243409812450408935546875f,0.504899940176307904238228729809634387493133544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2517999267734591928302734231692738831043243408203125f,-3.566999918773770428259695108863525092601776123046875f,2.65849998740106929773219235357828438282012939453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.93149993138760545097198928488069213926792144775390625f,-3.05509996855408072491400162107311189174652099609375f,2.92219997814595711105312147992663085460662841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4510000063627952471279058954678475856781005859375f,-0.30080003395751220285347926619579084217548370361328125f,-5.05560004735886980853365457733161747455596923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.820999962329864718668659406830556690692901611328125f,-1.27479996566176367878142627887427806854248046875f,0.180399999982117975605433457531034946441650390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0364000141307714297767006428330205380916595458984375f,1.1455000223666420566814849735237658023834228515625f,-0.942100062602758470120534184388816356658935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8704000168949359572678758922847919166088104248046875f,0.1379000010251996144461372750811278820037841796875f,2.01420005636215382338605195400305092334747314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.044599919402598775519663831801153719425201416015625f,0.25969992457628066073738182240049354732036590576171875f,2.822700002337992231105090468190610408782958984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.9342999700799570206299904384650290012359619140625f,1.542100059752911089816507228533737361431121826171875f,-2.488899965217709731035711229196749627590179443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9181000006809829638854125732905231416225433349609375f,-3.16899996618181400975799988373182713985443115234375f,1.53590007640272485645027700229547917842864990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.94549995765835159744483462418429553508758544921875f,-2.8312999973341828052753044175915420055389404296875f,-0.55789997155815329588079976019798777997493743896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.2556999575376490696498876786790788173675537109375f,1.1217000008180735282081741388537921011447906494140625f,0.495499978091569204252664349041879177093505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0267999564081440322382832164294086396694183349609375f,1.60490005841702298994277953170239925384521484375f,-1.477599928842483922863948464510031044483184814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.634900016209483997187135173589922487735748291015625f,1.2980999557912353825628315462381578981876373291015625f,1.55720002635121357315028944867663085460662841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.065099936106800537771732706460170447826385498046875f,-1.1425000584125513114486238919198513031005859375f,-1.05580004820674577814543226850219070911407470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9146000711336730315537124624825082719326019287109375f,-0.937999965555965875552146826521493494510650634765625f,-1.905299963068960433787424335605464875698089599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.6772000315666186764929079799912869930267333984375f,0.4564000546991835438603857255657203495502471923828125f,-3.0005999100074181029640385531820356845855712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.936099923135340628022049713763408362865447998046875f,-0.836100045660139290504275777493603527545928955078125f,2.03100000695884386203715621377341449260711669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.512499976977705440361887667677365243434906005859375f,-0.1380000219196071409299975130124948918819427490234375f,-0.497200020018219834838646420394070446491241455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.55609998601824006669858135865069925785064697265625f,0.51480002295672810941340458157355897128582000732421875f,-1.21829994754940162948741999571211636066436767578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9626000270783894308124217786826193332672119140625f,0.901400021097064296782264136709272861480712890625f,-0.0732999661386006806651494116522371768951416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0514999072104684874062741073430515825748443603515625f,-0.762799993970244560870241912198252975940704345703125f,0.1703999759167438288187668149475939571857452392578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.92330003088414702716590909403748810291290283203125f,-1.9465000242218390980042386217974126338958740234375f,-3.7450999767124653061500794137828052043914794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.237199999566374941650792607106268405914306640625f,1.29830001419484641900226051802746951580047607421875f,1.0584000282533472248047701214090920984745025634765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.86169999722316992230020105125731788575649261474609375f,0.43259998835399804395507317167357541620731353759765625f,0.42739992967098938692060983157716691493988037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0446000127211207963995320824324153363704681396484375f,-0.359499996557831824617323945858515799045562744140625f,-0.284399992650747446276682239840738475322723388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.205199999376386166272112632213975302875041961669921875f,-0.3223000581234689576604068861342966556549072265625f,0.0279999601542950671273501939140260219573974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.680199975255131761286975233815610408782958984375f,-0.2734999963790176735756176640279591083526611328125f,0.32029998949989713086239362382912077009677886962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2485000067353253516699851388693787157535552978515625f,-0.434599976548552202615383066586218774318695068359375f,-1.1092999449901288500086593558080494403839111328125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.41829997833892473835248893010430037975311279296875f,0.5963000564731644193017245925148017704486846923828125f,0.88909992743432375572609771552379243075847625732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.086999955598264566702937372610904276371002197265625f,0.32910002108812275167792904539965093135833740234375f,1.27610000253990474305965108214877545833587646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.192600057812035263538064100430347025394439697265625f,-0.59819990206807904797869923640973865985870361328125f,0.132999949406832573828296517604030668735504150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.19940000993609441337639509583823382854461669921875f,0.436499999765307844512562951422296464443206787109375f,-0.263400016760825916861676887492649257183074951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6822000020995730995565509147127158939838409423828125f,0.6394999856799830073583734701969660818576812744140625f,-0.13929994620084673595528101941454224288463592529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8171999265134319845316213104524649679660797119140625f,-2.42419996352791766724976696423254907131195068359375f,-0.618000009477138068092472167336381971836090087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.116900018876792355371208032011054456233978271484375f,4.18299997736513606838570922263897955417633056640625f,1.58010006153732529554645225289277732372283935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.1225000257976347484145662747323513031005859375f,-1.7371999779224387427944975570426322519779205322265625f,-0.301800020334123786369673325680196285247802734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.45040006513744612615113283027312718331813812255859375f,-1.427400005443393293802500920719467103481292724609375f,-1.4937999948382369996835450365324504673480987548828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.683100074255467148987008840776979923248291015625f,-1.232700007963179356096361516392789781093597412109375f,1.434200040939450193633319941000081598758697509765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.15409999748841018973877226017066277563571929931640625f,0.48080000309348047693447369965724647045135498046875f,0.194799985927343488611995780956931412220001220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.39059999917223908738606041879393160343170166015625f,0.111899959532916870585950164240784943103790283203125f,0.182600000554324015666907143895514309406280517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.247499940693377717959577921646996401250362396240234375f,-2.01130000269859987582776739145629107952117919921875f,-0.736700005586444550687019727774895727634429931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.057800038915872153921782228280790150165557861328125f,0.6523000728756185306878023766330443322658538818359375f,2.4901999420255425121695225243456661701202392578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.05930005402937421621345492894761264324188232421875f,-3.368199972010402110100812933524139225482940673828125f,-2.02669993443339979677375595201738178730010986328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.251499889031053580623620291589759290218353271484375f,-2.92289998929053407294986755005083978176116943359375f,1.2747999771356590681392617625533603131771087646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3762001154489809096048702485859394073486328125f,-0.559499943211676065146775727043859660625457763671875f,-3.036299995899944814681248317356221377849578857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.7046000989615919252173625864088535308837890625f,-0.61650000190734832727912362315692007541656494140625f,-1.2045999880969520123841220993199385702610015869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.067999966599046945958662035991437733173370361328125f,-0.37059998529553350721243987209163606166839599609375f,1.2231999619334941176163056297809816896915435791015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0100000165030371679364407100365497171878814697265625f,1.648599981808661407711724677938036620616912841796875f,-0.1941999647945171769691796725965104997158050537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5246999333307156287986572351655922830104827880859375f,0.1630999833583832714367645166930742561817169189453125f,2.2311999260552237700494515593163669109344482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.433400024826079199868900104775093495845794677734375f,-0.11440005791783336430711415232508443295955657958984375f,-0.94619977285265921551626888685859739780426025390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.61440007527768714101767955071409232914447784423828125f,-0.15880002277791571696496930599096231162548065185546875f,1.199400032474100985524501084000803530216217041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4854000046387298983319169565220363438129425048828125f,1.0904000228367731750722668948583304882049560546875f,-1.56879992912411747596479472122155129909515380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.25480004001855860451541957445442676544189453125f,1.881399987047909672099876843276433646678924560546875f,-1.2315000075697906112281998503021895885467529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.386000022195279790526711849452112801373004913330078125f,1.0828000372767452308409019678947515785694122314453125f,0.036699972952902282230525088380090892314910888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.09640003439635069781843412783928215503692626953125f,1.155000011287629302358936911332421004772186279296875f,0.928999991551040960047203043359331786632537841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3888999530732639708929809785331599414348602294921875f,-1.2280000098496677285453415606752969324588775634765625f,-0.833699989590048762266860649106092751026153564453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4168999901548036124410145930596627295017242431640625f,-0.3550000845640892688237499896786175668239593505859375f,1.203200045268240092610767533187754452228546142578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.16679997943043645136640407145023345947265625f,-0.54510006960257817443249450661824084818363189697265625f,-2.59660003174245357371319187222979962825775146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6957999953448774643760543767712078988552093505859375f,-0.7858999998852611579280846854089759290218353271484375f,-0.0094000453263525240998887966270558536052703857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9276000492811211639576640664017759263515472412109375f,0.44880001495480559281503474267083220183849334716796875f,0.16209997878372639501520779958809725940227508544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0793000136658552623458717789617367088794708251953125f,-0.5842000128880144860232803694088943302631378173828125f,-0.658599997343122911530599594698287546634674072265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.82129998474270049513279445818625390529632568359375f,-0.76430000014305132793879238306544721126556396484375f,0.1376999696671958872684626840054988861083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.88060004333555763622598533402197062969207763671875f,0.0870000231750314156897729844786226749420166015625f,0.10679994884580235048332497171941213309764862060546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0859000594899053115938158953213132917881011962890625f,-0.932500063683837421280031776404939591884613037109375f,2.39210006208121672699462578748352825641632080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0481999784365287897713869824656285345554351806640625f,0.5434999539107072674681830903864465653896331787109375f,-0.4678000275723641632197313811047933995723724365234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.10249994266778283957819439820013940334320068359375f,0.1730000148788095604146519690402783453464508056640625f,0.0114000414744015454715508894878439605236053466796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3799000320047143386403831755160354077816009521484375f,1.2161000002861026558775847661308944225311279296875f,-1.5197000260531903226279837326728738844394683837890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7514000347688798431278200951055623590946197509765625f,-0.2099999939277765736989067590911872684955596923828125f,0.5835000359416013804292333588819019496440887451171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3077000255838042708234070232720114290714263916015625f,-0.871400028137862658894619016791693866252899169921875f,-0.280999975107609767377425669110380113124847412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.57380001711547290454973335727117955684661865234375f,0.50069994904398973378789605703786946833133697509765625f,-0.95320000496059609229604348001885227859020233154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.49339995283484461285894440152333118021488189697265625f,0.3923999925538901312194184356485493481159210205078125f,0.7075000661425294712358891047188080847263336181640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2426000480517751523024116977467201650142669677734375f,-1.40220002158284184901049229665659368038177490234375f,1.78780002285987205112860465305857360363006591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3017000005573027276994935164111666381359100341796875f,0.548800031793117870648757161688990890979766845703125f,-0.7800999806240203060525573164341039955615997314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.669400002261996807106925189145840704441070556640625f,0.197399991846084699176344656734727323055267333984375f,0.13409998413324419885839233756996691226959228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4217000276029114758102878113277256488800048828125f,1.31190000673234496986196973011828958988189697265625f,1.0033000099480149369668424697010777890682220458984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4099000113666060318706740872585214674472808837890625f,0.284000001572072446887062824316672049462795257568359375f,-0.4284000322952870032366945451940409839153289794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.46490001639574740632809835005900822579860687255859375f,-0.2225000129640106738548865905613638460636138916015625f,-0.4016000423222794157851467389264144003391265869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.019500014457850600724242440264788456261157989501953125f,0.552900104457885621656032526516355574131011962890625f,1.2406000264786190001586874132044613361358642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.53950001106783751492201872679288499057292938232421875f,0.484499946489930000126378217828460037708282470703125f,-1.046300013539195106204715557396411895751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3137000803381209124154338496737182140350341796875f,0.66430004208981952018575611873529851436614990234375f,-0.2005999730393284163909584094653837382793426513671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6468999756634230546836761277518235146999359130859375f,-1.64390005257576721220402760081924498081207275390625f,0.9720000082924962203634322577272541821002960205078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.219500011943280615156481871963478624820709228515625f,1.5969000435754650002451171530992724001407623291015625f,0.2295999844834204772769936653276090510189533233642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0222000520184642002874397803680039942264556884765625f,0.1724000022768965578734423615969717502593994140625f,-0.673000032238662004857587817241437733173370361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8584999681040648145113891587243415415287017822265625f,-2.366699968519807129041510052047669887542724609375f,-0.914899985252319414286148457904346287250518798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.44080000180825695821340559632517397403717041015625f,-0.95800000236183535662348731420934200286865234375f,1.701300017562508148927236106828786432743072509765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.93980004556924090497460611004498787224292755126953125f,0.9731998889550574904916402374510653316974639892578125f,2.037899968875944267665545339696109294891357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.86680005594789888423434831565828062593936920166015625f,-2.502100043994933020741200380143709480762481689453125f,-2.3695999876499183045552854309789836406707763671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2879000537753102495486245970823802053928375244140625f,-1.1385999770402914776923353201709687709808349609375f,-1.5738999988198283386253706339630298316478729248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1520000369586054222992288487148471176624298095703125f,1.7010000153779980536228322307579219341278076171875f,-1.9157000197499984839311082396307028830051422119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1211999857529997282057365737273357808589935302734375f,-1.4379999332576984816256526755751110613346099853515625f,2.727900016373395519764244454563595354557037353515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.419900017774104572509941135649569332599639892578125f,-2.221800018732249970554448736947961151599884033203125f,-1.542499959822743793580457349889911711215972900390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7528999908179050404299914589500986039638519287109375f,0.4488999729104332647722230831277556717395782470703125f,-0.40740006456374988630386724253185093402862548828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.09849999295175138058766606263816356658935546875f,0.66989997858405114783408862422220408916473388671875f,-2.479299971011280856458824928267858922481536865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.86040000676214545283215784365893341600894927978515625f,-0.1758000141948465877561602610512636601924896240234375f,1.614399943067132969787280671880580484867095947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.2997999908447273043066161335445940494537353515625f,-0.0853999872788782088406378534273244440555572509765625f,0.3908999895289551052002252617967315018177032470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.479400045903771943045512671233154833316802978515625f,0.2679999709576359467888551080250181257724761962890625f,-0.9319999433606849237321512191556394100189208984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.075000000875442562886519226594828069210052490234375f,-1.221399937725067719185290116001851856708526611328125f,0.864100027626008238001986683229915797710418701171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9431999826088546257096822955645620822906494140625f,-1.95939994194954625328364272718317806720733642578125f,3.16979998358041026307319043553434312343597412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.254999967943877248188755402225069701671600341796875f,-0.8908000047326083414844788421760313212871551513671875f,2.10479999088197899936858448199927806854248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.158199991922081562023549849982373416423797607421875f,1.254500048898159914045891127898357808589935302734375f,-2.250500028457493417732848683954216539859771728515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.118600023084880046297939770738594233989715576171875f,0.0524999881908299048660637708962894976139068603515625f,1.862199973526596608053296222351491451263427734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.154899988567828383878577369614504277706146240234375f,-1.0578999493181715507716944557614624500274658203125f,-0.59880000728070736926156314439140260219573974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.19590003535002498136208259893464855849742889404296875f,1.771600011551381026464468959602527320384979248046875f,1.419300042760372360106657652067951858043670654296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.05080004708915897282395235379226505756378173828125f,0.014699999864399071469733826234005391597747802734375f,-1.1882000102505083649617745322757400572299957275390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.476699960845710091206228753435425460338592529296875f,0.0819000316917897208668364328332245349884033203125f,-0.32730003241896532539811914830352179706096649169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.642699964656680666763577391975559294223785400390625f,1.44640002728104644091899899649433791637420654296875f,-0.3359000465258950018920813818112947046756744384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.780800037589668871618187040439806878566741943359375f,1.32440006156861844743843903415836393833160400390625f,-0.12610005318522399253566845800378359854221343994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3232999941155318968100118581787683069705963134765625f,0.767799982663988078712691276450641453266143798828125f,1.6133999563366163698674426996149122714996337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1893000484228128943442470699665136635303497314453125f,-1.038300014418363392820765511714853346347808837890625f,-0.40160004194974963098729858756996691226959228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.052200024376809095461027254714281298220157623291015625f,1.0818000092700115200017307870439253747463226318359375f,-0.05560003841817484637743973507895134389400482177734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.326800011728704209446050299447961151599884033203125f,1.7955999875307082636055611146730370819568634033203125f,1.053599942655861543272521885228343307971954345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2344999662525950601121849103947170078754425048828125f,0.33540006659030818791933370448532514274120330810546875f,-2.760000043772160349675459656282328069210052490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.648999947275965549664533682516776025295257568359375f,-1.39000005057081477133351654629223048686981201171875f,1.057400002686680107188976762699894607067108154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.07460001046732067653266540219192393124103546142578125f,0.80119999905228789227606966960593126714229583740234375f,-3.631200020454079524512280841008760035037994384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.090799987242368818129989449516870081424713134765625f,-1.33899999404698544225311707123182713985443115234375f,3.5411000389732425475131094572134315967559814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.337200035031140377128622276359237730503082275390625f,1.4232999963507051432998196105472743511199951171875f,-0.0727000193625684243414752927492372691631317138671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.90459998083263570833878475241363048553466796875f,0.356299953617900566626985892071388661861419677734375f,-0.06400001267343602417980719110346399247646331787109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9612000632017849710564405540935695171356201171875f,-0.8607000000059594757573222523205913603305816650390625f,2.09469995337277570257583647617138922214508056640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.61459995252042975977246896945871412754058837890625f,1.620400008773805300421599895344115793704986572265625f,-0.43699997878819640817482650163583457469940185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4899999518319966274049193089012987911701202392578125f,2.051400050619987869282567771733738481998443603515625f,-0.4591001169212158838917048342409543693065643310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.26720006680786501096491747375694103538990020751953125f,-1.85680002655535947297948951018042862415313720703125f,-1.43839996497929067942322944873012602329254150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0912000095576050995305195101536810398101806640625f,0.2728000198796387554267539599095471203327178955078125f,-1.186300067071616837210967787541449069976806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.072300002635270033124470501206815242767333984375f,-0.309600005680322976076013219426386058330535888671875f,-1.169799947445093746267730239196680486202239990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0267999718308444823122727029840461909770965576171875f,-0.36999994894489507402113304124213755130767822265625f,-0.15069990575611524885601966161630116403102874755859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0935000280253588389456353979767300188541412353515625f,0.32079999184310381554752211741288192570209503173828125f,-1.28999998804181803535584549535997211933135986328125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.151800024320185134030225526657886803150177001953125f,1.351600004287063949703906473587267100811004638671875f,3.867800064750760657972250555758364498615264892578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.254699873651563990506474510766565799713134765625f,1.07819994636178062563658386352472007274627685546875f,-0.4033999206110838731120793454465456306934356689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5129999386034906905251773423515260219573974609375f,0.06610004424452620153118687085225246846675872802734375f,-0.255999928690491262983641718165017664432525634765625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.152899954794347703312951125553809106349945068359375f,-0.7772999674499041500297380480333231389522552490234375f,-4.450799973030388656525246915407478809356689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.36650002820789795432432356392382644116878509521484375f,-0.0788999927848583748613009447581134736537933349609375f,-0.68360003463327867478938060230575501918792724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1010000135339794713473793308367021381855010986328125f,-0.6759000728636979093977288357564248144626617431640625f,-0.43569997988939379585104916259297169744968414306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.77790001585185475363459772779606282711029052734375f,-1.4190999828293922035982177476398646831512451171875f,1.8312000333994629119871433431399054825305938720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.586000013105571593285958442720584571361541748046875f,1.8317000228159121544280196758336387574672698974609375f,0.18500003369524942176127524362527765333652496337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.681499955713750438235365436412394046783447265625f,1.24329989785403061119950507418252527713775634765625f,-1.713299949449301085024899293784983456134796142578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.303700004155933900307218209491111338138580322265625f,-1.2072000449590392090470913899480365216732025146484375f,0.9451999842703331378146458519040606915950775146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.21520000569075481422487428062595427036285400390625f,-2.07460002792030540064160959445871412754058837890625f,-0.964599974834917883725893261726014316082000732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.69600005233287820516352439881302416324615478515625f,0.5967000078588722278283285049838013947010040283203125f,-0.43969996898174290578253931016661226749420166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.049899988120794436241567382239736616611480712890625f,-0.4309999924302108098572716698981821537017822265625f,-0.232800045770407137979418621398508548736572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.603199949379265465410071556107141077518463134765625f,1.582700009602308721667895952123217284679412841796875f,-0.25459999070465588744127671816386282444000244140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.29389997143894441844480525105609558522701263427734375f,0.0501000455930820098870981382788158953189849853515625f,0.30219998157322403020685896990471519529819488525390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.14700004107504971440079089006758294999599456787109375f,0.090700104481728216487823601710260845720767974853515625f,0.8291000192068531671196751631214283406734466552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2804000061661000575696789383073337376117706298828125f,-1.2087000028520822336730589086073450744152069091796875f,-1.4514000169992453681544475330156274139881134033203125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.2880000430792559740211800090037286281585693359375f,-0.31480001818835734450630070568877272307872772216796875f,0.4485000445097686583295626405742950737476348876953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1645000382065777699835962266661226749420166015625f,-0.0350000067614019416684101315695443190634250640869140625f,0.9869000125065439110727538718492724001407623291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.067299998146295170187158873886801302433013916015625f,1.279199991393089153035589333740063011646270751953125f,-0.28779995431453009313571556049282662570476531982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.328100031489133303619354364855098538100719451904296875f,-0.70539998731613184901334534515626728534698486328125f,0.9501999888896950352545900386758148670196533203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.439800026458500969983589357070741243660449981689453125f,-0.26650000600516843274334632951649837195873260498046875f,0.07760005166530647979783452683477662503719329833984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.45959999927282269016615146028925664722919464111328125f,-0.7397999545976521318380036973394453525543212890625f,0.5991999447524547495191882262588478624820709228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.06290001704394876380632695145322941243648529052734375f,1.4377000152647501085567682821420021355152130126953125f,0.357900040923059492570246220566332340240478515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.056899967132509221556802003760822117328643798828125f,0.50380003831237607503368280958966352045536041259765625f,-0.70060000686496504140876595556619577109813690185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.109600010169297323869841420673765242099761962890625f,-0.65120001257508963732334450469352304935455322265625f,1.153000021528452379016016493551433086395263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4478999996840957731336629876750521361827850341796875f,-0.0775999835669995974285484408028423786163330078125f,-1.05440001544952366430152324028313159942626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1632000031352038860177344758994877338409423828125f,1.207200021918117815999949016259051859378814697265625f,-0.155599998855590282431649029604159295558929443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.18520002219378894903201171473483555018901824951171875f,1.0441999803289772064118778871488757431507110595703125f,-0.703700008533150178635651172953657805919647216796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.69289994622617978592415965977124869823455810546875f,-2.96180004875809022024668593076057732105255126953125f,1.052299986821413124715718367951922118663787841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8470999909281733408761283499188721179962158203125f,-0.165000011809170121779288820107467472553253173828125f,2.2402000041261320717467242502607405185699462890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4016000159472226105350500802160240709781646728515625f,-0.5554000278845434568353311988175846636295318603515625f,0.8950999974250797297514736783341504633426666259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.34760001232624071487720129880472086369991302490234375f,-0.42580001907497655455614449238055385649204254150390625f,0.065099991576373472668137765140272676944732666015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.770399988061189588961497065611183643341064453125f,0.6714000132739545989579710294492542743682861328125f,-0.541499982535838864095012468169443309307098388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2881999900408087622594166532508097589015960693359375f,-0.28949998426437417720080702565610408782958984375f,-0.28070002340078392677469310001470148563385009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4648999742627151210427882688236422836780548095703125f,1.2877999580398202805753271604771725833415985107421875f,2.61989997366666838019000351778231561183929443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.706200042787194615101498129661194980144500732421875f,1.2587001157656312333443793249898590147495269775390625f,-0.9671000155150901722578282715403474867343902587890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.317500018402933736894055982702411711215972900390625f,-0.326000011056661431751990676275454461574554443359375f,0.2652000179097058207844384014606475830078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.448500052481889621702748627285473048686981201171875f,3.228700012761355875312574426061473786830902099609375f,1.815000031664967483635564349242486059665679931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.07249998666346069153831876974436454474925994873046875f,0.5300000375136739716452893844689242541790008544921875f,-1.1637000391736631588202044440549798309803009033203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8968000353470435559444240425364114344120025634765625f,-1.5276000205218789318450944847427308559417724609375f,0.23479999740123691509552372735925018787384033203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.281200003038346579842254868708550930023193359375f,-0.0562999615341421844050273648463189601898193359375f,-0.989099993669986243816083515412174165248870849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0193000273935501898137090392992831766605377197265625f,0.09860011040568306128761832951568067073822021484375f,-1.1468999786436551158175234377267770469188690185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.401399884974956489003261594916693866252899169921875f,0.5664000050596891266962984445854090154170989990234375f,1.7257000453427433495079412750783376395702362060546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.54360006439834684943690490399603731930255889892578125f,0.289399909278749867258540007242118008434772491455078125f,0.626499987900255295159013257944025099277496337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.65709995788485020540292680379934608936309814453125f,-0.9852000118374821813205244325217790901660919189453125f,-0.9116999799564482476199600569088943302631378173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7313999554201953134935365596902556717395782470703125f,-0.108799946185945739518530217537772841751575469970703125f,0.227199989843368310715732150129042565822601318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.309299953167139651810657596797682344913482666015625f,0.6722000568240888984661296490230597555637359619140625f,-0.57950001484900737214189803125918842852115631103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.225599997365474536081819678656756877899169921875f,1.856100046516954815700728431693278253078460693359375f,0.228800049413741268811151030604378320276737213134765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.496499965958297817536504226154647767543792724609375f,2.5586000047191976847216210444457828998565673828125f,0.7841999427780506071172794690937735140323638916015625f,0.0f,0.0f,0.0f,0.0f,0.0f,3.302599997853487234777958292397670447826385498046875f,2.929899980260431302525603314279578626155853271484375f,1.3842999456420541637413634816766716539859771728515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.224500018797816114357601691153831779956817626953125f,-1.0128000108644366061838582027121447026729583740234375f,-1.7837000921845433953905057933297939598560333251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.31210009370893254665446647777571342885494232177734375f,-2.4740000290274632988030134583823382854461669921875f,3.2401000319212673872470986680127680301666259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.547000042658300156261930169421248137950897216796875f,0.15870002656355464143445033187163062393665313720703125f,0.588600006991625068764051320613361895084381103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3279999834001063874211467918939888477325439453125f,0.287499986216426695051495698862709105014801025390625f,-1.716200042265654257533924464951269328594207763671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.037600049988925743349454933195374906063079833984375f,-0.2050000188313421400909675185175728984177112579345703125f,0.3507000029303137811353963115834631025791168212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.21619999514073118263013384421356022357940673828125f,-0.615400021272151764861746414680965244770050048828125f,-1.545900014451145665361764258705079555511474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.829399999039619917340360188973136246204376220703125f,2.08759998578354721843197694397531449794769287109375f,-0.6552000803828239572368374865618534386157989501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.268599969664216953191271386458538472652435302734375f,2.85520007008239584678221945068798959255218505859375f,2.069699950411169719899362462456338107585906982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.93189988871514817247998507809825241565704345703125f,0.5108000216268011595133202717988751828670501708984375f,0.34120006234496835784142376724048517644405364990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.49350002975761864121295730001293122768402099609375f,-0.557300005503745143897731395554728806018829345703125f,0.7436000068426140696686843511997722089290618896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3623000308170920558126226751483045518398284912109375f,-1.4866999888598915191550986492075026035308837890625f,-1.0791000200450415746900034719146788120269775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.40450003582239180133228728664107620716094970703125f,-1.23170002782642828975667725899256765842437744140625f,0.420899982674419970152257519657723605632781982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3925999567076556839850809410563670098781585693359375f,2.28609993608072503690209487103857100009918212890625f,-2.427199927370250254199390838039107620716094970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.17780002189129451295457329251803457736968994140625f,-0.978700053106248191170379868708550930023193359375f,1.2804000026270740963951766389072872698307037353515625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1822999724045402114569469631533138453960418701171875f,0.408900027839839363963392315781675279140472412109375f,-1.790900066418945613122559734620153903961181640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3777999547243113820371718247770331799983978271484375f,-3.907500018030404032032265604357235133647918701171875f,1.1399999231845132641183226951397955417633056640625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.13930001839697325749511946924030780792236328125f,-1.200800063778461268526598360040225088596343994140625f,0.8766000472769146512064253329299390316009521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.48119995714276964093869537464343011379241943359375f,0.751300045725703302679221451398916542530059814453125f,-0.05470004719421306393911663690232671797275543212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.19300000441074438572996996299480088055133819580078125f,2.3596999759852881339838859275914728641510009765625f,0.968099994273483588358431006781756877899169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5099999957904213232495749252848327159881591796875f,-1.3980000312328335621003816413576714694499969482421875f,1.516799973749368479758459216100163757801055908203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8706000524811436935834763062302954494953155517578125f,-1.93110001772046135926075294264592230319976806640625f,0.8269000670261694718732314868248067796230316162109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4284999808445564273284844603040255606174468994140625f,-0.22410002097636383400214299399522133171558380126953125f,1.1930999894678591122243460631580092012882232666015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.71300001957267600261047846288420259952545166015625f,0.715599937872589908494092014734633266925811767578125f,1.799799911347031322605971581651829183101654052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4714000255115333715139058767817914485931396484375f,-0.7497999866351483522208809517906047403812408447265625f,0.565900015456974880834195573697797954082489013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.059999992009252434144173093955032527446746826171875f,-0.88149999669194212259526466368697583675384521484375f,0.50420002000331809455246911966241896152496337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.89850001008808622060541893006302416324615478515625f,0.987500034272670301760399524937383830547332763671875f,-1.601499970931560046238928407547064125537872314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.810300013025103016417460821685381233692169189453125f,-3.98250001430511435529524533194489777088165283203125f,1.2226999495878823065453389062895439565181732177734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.26010004436373801439685848890803754329681396484375f,2.73389996048659167371397415990941226482391357421875f,-1.4261000577673319522631345535046420991420745849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.04390000206083044531624182127416133880615234375f,-3.070999943330884551784265568130649626255035400390625f,0.6101999211080364471371240142616443336009979248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0388999902144078202326227255980484187602996826171875f,1.2343999725714318582703299398417584598064422607421875f,0.23490005163699356760531600230024196207523345947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5310999489329752965005582154844887554645538330078125f,0.399599927076696559424817678518593311309814453125f,0.3673000261604799643322394331335090100765228271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.244899995254723723547840563696809113025665283203125f,-0.99199998135864808634920564145431853830814361572265625f,-2.41819999046921640939444841933436691761016845703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.70110003277063359661269714706577360630035400390625f,0.70679999574720919763848314687493257224559783935546875f,1.921799958643317207673817392787896096706390380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.950600042180716808815077456529252231121063232421875f,1.9075999522000544050825965314288623631000518798828125f,-2.2683999695427718990003995713777840137481689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.452599945345522058914866647683084011077880859375f,-0.7737001216143368775846056450973264873027801513671875f,0.3078998972386124410860475109075196087360382080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.07650002731382787857228322536684572696685791015625f,-0.79720006148070010620898528941324912011623382568359375f,1.288900017967819611186541806091554462909698486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.89760007707178601776831783354282379150390625f,-0.6807999296121296506356657118885777890682220458984375f,0.25490001660063821287138807747396640479564666748046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.37199998627602948175763231120072305202484130859375f,-0.03010000062882856042278945096768438816070556640625f,1.4333000609472399400345921094412915408611297607421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.03009995855167446610067827350576408207416534423828125f,-0.25730002705454868561218972899951040744781494140625f,-1.4081000043690199419188502361066639423370361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5170000096708531334144254287821240723133087158203125f,2.115700010436773137456611948437057435512542724609375f,-1.199799988609552059415364055894315242767333984375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2054999809786675069034345142426900565624237060546875f,1.045900057757645384981515235267579555511474609375f,-0.1836999814316626622456851691822521388530731201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.2840999769620591308694201870821416378021240234375f,2.284900006281584960987629528972320258617401123046875f,5.9912999319553392751913634128868579864501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.99070001460909917767594379256479442119598388671875f,-2.544800016511976625821489506051875650882720947265625f,1.2402999956279987525675778670120052993297576904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3095000033006065631724368358845822513103485107421875f,-2.54790004476010789602469230885617434978485107421875f,2.8650000201538201594075871980749070644378662109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2357000264927739952014462687657214701175689697265625f,0.94290001559108527207087036003940738737583160400390625f,-0.5920000642463565387885182644822634756565093994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3847000184535982469924420001916587352752685546875f,0.5804999810159205964055217918939888477325439453125f,-1.1879999987483025325474272904102690517902374267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.42000001616775950452620236319489777088165283203125f,-0.08079996405243861090639256872236728668212890625f,0.14750001668930057263651178800500929355621337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2174999958276748746044404470012523233890533447265625f,2.4364000396490101962854168959893286228179931640625f,-0.574899998328089356647296881419606506824493408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.41369995798096059758108822279609739780426025390625f,-0.800400052713603482601456562406383454799652099609375f,0.24799990468472221660789500674582086503505706787109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3044999749138954658889133497723378241062164306640625f,1.3434999881833793455854220155742950737476348876953125f,1.4934999369233850075033842585980892181396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.398600028896331171068823095993138849735260009765625f,0.3863999711781733736870592110790312290191650390625f,0.054099997022748258501678719767369329929351806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.61020001161396475453102539177052676677703857421875f,-0.21199998014792809097883719005039893090724945068359375f,-0.7854999699890612419039825908839702606201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.257599968516826205444658626220189034938812255859375f,1.6871000395059592147362081959727220237255096435546875f,0.3829999706968660877492993677151389420032501220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.44659996803998947001446140347979962825775146484375f,0.20369998370409003740633124834857881069183349609375f,0.60510006075501454603227102779783308506011962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.40990001591146085768713192010181955993175506591796875f,0.4762999542325729862568550743162631988525390625f,0.5018000070348376251416766535839997231960296630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0943000474542372568720338676939718425273895263671875f,0.1165999673694375626809005552786402404308319091796875f,1.49260008865743909467482808395288884639739990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6181000260874622842521830534678883850574493408203125f,-0.0774000050336123290861678469809703528881072998046875f,-1.1756999969512218484624099801294505596160888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3402000502452249275364692948642186820507049560546875f,-0.128900033129751623306447072536684572696685791015625f,2.624399974024296255947774625383317470550537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0429000044152143544806676800362765789031982421875f,1.536900009638070940809484454803168773651123046875f,-1.216300001469254876695913480944000184535980224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3170999939084051533200181438587605953216552734375f,0.2167000220894816919781078468076884746551513671875f,0.175700005221366861718479412957094609737396240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.05549998548626877692413472686894237995147705078125f,1.047199990476667341710026448708958923816680908203125f,0.14080000652074797784507609321735799312591552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1730000233352182448243183898739516735076904296875f,-1.700500000536441458365288781351409852504730224609375f,-1.121899977488815647319597701425664126873016357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.761800028157235686876447289250791072845458984375f,2.22069999359846104169946556794457137584686279296875f,0.3260000191777956413119454737170599400997161865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0259000175058841097808226550114341080188751220703125f,1.0516999611064796749104743867064826190471649169921875f,-0.485699979814886884099678354687057435512542724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.749400023719668073596267277025617659091949462890625f,0.1162000015854840473394915534299798309803009033203125f,0.90300002413615576557504027732647955417633056640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.46870001201629651887969885137863457202911376953125f,0.395700006711483087684655401972122490406036376953125f,-0.7624000252991915971279013319872319698333740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.36940002334713906151364426477812230587005615234375f,0.290699995237588382224203087389469146728515625f,0.6156000410258772870264465382206253707408905029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.559099962712824360977492688107304275035858154296875f,0.3715999866664407846883477759547531604766845703125f,-0.215700002986192629350625793449580669403076171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.00149998924136129918593951515504159033298492431640625f,0.03099997931718778687582016573287546634674072265625f,-1.94070005156397851209248983650468289852142333984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0737999951362608985760971336276270449161529541015625f,-0.4396000141739839772725417788024060428142547607421875f,1.79360003146678170793393292115069925785064697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.49259998645409919237891926968586631119251251220703125f,1.56310000846684005182396504096686840057373046875f,-0.23610000393688768127731236745603382587432861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0225000230409211354754006606526672840118408203125f,-2.677899993854015558980563582736067473888397216796875f,-0.43629999677538899849338349667959846556186676025390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.02649999535083769064414127569762058556079864501953125f,-0.9952000219702721306447301685693673789501190185546875f,1.1773000099927184436410243506543338298797607421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.609599989736080427604747455916367471218109130859375f,1.73869999517798401456047940882854163646697998046875f,-1.3599999685585490993844359763897955417633056640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6202000128433109438219617004506289958953857421875f,1.056800037246942824964435203582979738712310791015625f,-0.3191000340521334965870892119710333645343780517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.32810001595467308543874196402612142264842987060546875f,-0.131100096324085146903826171183027327060699462890625f,1.2417000664763147010916100043687038123607635498046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.551399941115082725673346430994570255279541015625f,-0.23259998395815539851128050941042602062225341796875f,3.31819998078346234393620761693455278873443603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.374199924188852950379668982350267469882965087890625f,0.1013000421121714733629914917401038110256195068359375f,-0.2514000351414085177026436213054694235324859619140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.419999921321869695844952730112709105014801025390625f,-1.1325999679952871712629303146968595683574676513671875f,0.03989999311268410675523909958428703248500823974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.48900003822893012284112046472728252410888671875f,1.0324999805912373762595279913512058556079864501953125f,-0.227299982760846841500779191846959292888641357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6149999951198694692067192590911872684955596923828125f,2.45550004754960493613680228008888661861419677734375f,-0.382700016903877393570354570329072885215282440185546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8803000153720379472588319913484156131744384765625f,0.7408999548092485287753561351564712822437286376953125f,0.3097999790169294609398775719455443322658538818359375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.827000030457972545860911850468255579471588134765625f,-1.574800036218760634909585860441438853740692138671875f,-1.298000033020972665553927072323858737945556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.589700030244143302837755982181988656520843505859375f,0.30830006074234728163929730726522393524646759033203125f,-1.910999898999929502707573192310519516468048095703125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.5295999960318198418462998233735561370849609375f,0.60420004233643342939075182584929279983043670654296875f,1.7878000141054399829698695612023584544658660888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.45659995284080512334412560448981821537017822265625f,2.0379000373840323590002299170009791851043701171875f,0.8832999804809684718520657042972743511199951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.70380001135990066796210840038838796317577362060546875f,-0.758699959601461859648452445981092751026153564453125f,0.627100014695525231900319340638816356658935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.944399990266561406571099723805673420429229736328125f,-0.1565000182241196302612706858781166374683380126953125f,-1.951600046420097545052385612507350742816925048828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9121000138387083122637477572425268590450286865234375f,1.221599988473207165640133098349906504154205322265625f,-0.75590002551525803387022506285575218498706817626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.36640012421309808843261635047383606433868408203125f,0.92140004993081081607897431240417063236236572265625f,3.210500064499675954010626810486428439617156982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.953999990582465873245610055164434015750885009765625f,-4.79719992420375351827033227891661226749420166015625f,-0.769699973526597691630968256504274904727935791015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.02740003520846290285817303811199963092803955078125f,-0.31990004118755432926946014049462974071502685546875f,-0.052299947016686498812987338169477880001068115234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.62829997101873136244876150158233940601348876953125f,1.5754000149764113558603639830835163593292236328125f,1.0590999734789132702417191467247903347015380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0559999457336963768483428793842904269695281982421875f,0.7144000342808667358696084193070419132709503173828125f,0.42450000095367423913472748608910478651523590087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3389000593930493465677500353194773197174072265625f,-0.4373999399900441176214371807873249053955078125f,0.127200048702955204049658277654089033603668212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.24500002399086984183895765454508364200592041015625f,-0.5421999884277581838176729434053413569927215576171875f,-0.09860005702227336854548411793075501918792724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.18960004624873416645414181402884423732757568359375f,-2.239699972409010086948910611681640148162841796875f,3.74739996189475110810462865629233419895172119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.686000058628617903622171070310287177562713623046875f,-0.3679999503195283683254501738701947033405303955078125f,0.486600030252337223402037125197239220142364501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.11239991359636203327454495592974126338958740234375f,0.03059995784014557518304400218767113983631134033203125f,0.9032000372588633307913141834433190524578094482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2947000262022017036400711731403134763240814208984375f,0.861600017828493935923006574739702045917510986328125f,0.021500041842461126151420103269629180431365966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.79900001107156237711848234539502300322055816650390625f,1.5258000069677830623504632967524230480194091796875f,0.79889998186975719196567524704732932150363922119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1845999830305569133770404732786118984222412109375f,0.618600004756450605469808579073287546634674072265625f,-1.23180005583614171627004907350055873394012451171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.11779999853372569162957006483338773250579833984375f,-0.513500041678547791690334634040482342243194580078125f,-0.158100032457709982480764665524475276470184326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7650999961584805664216446530190296471118927001953125f,-0.3496000070959330852105040321475826203823089599609375f,-0.83880009877383709948617251939140260219573974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3584999693892896299729500242392532527446746826171875f,1.212800030459463496157468398450873792171478271484375f,-0.2309000336922715224119428967242129147052764892578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7613000398397444801190658836276270449161529541015625f,0.4122000391662101748124769073911011219024658203125f,-0.50059999814778521898261942624230869114398956298828125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.432699989150465480491902781068347394466400146484375f,-0.36140000304803254760344088936108164489269256591796875f,-1.3374999313801525513412116197287105023860931396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.10779999862685840650300406196038238704204559326171875f,0.640799989831447813770637367269955575466156005859375f,-0.05460001558959459710962391909561119973659515380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.274999991003423982505182721070013940334320068359375f,-0.8817000009298328055962201688089407980442047119140625f,2.036299995583295441292648320086300373077392578125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.415000031962990956202474990277551114559173583984375f,0.6108000151634211949414066111785359680652618408203125f,-0.0110999744884663353872156221768818795680999755859375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.6010999350190164847163032391108572483062744140625f,0.908100101226568146017825711169280111789703369140625f,0.7829000264316789969853971342672593891620635986328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.407600072377919619981412324705161154270172119140625f,-0.1496999622017138875662567443214356899261474609375f,-2.46880004311800060889936503372155129909515380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0693999897450210401217418620944954454898834228515625f,-1.617199922639132037005538222729228436946868896484375f,0.986200063276290617153563289321027696132659912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1623000776626175589711920110858045518398284912109375f,-0.11940003623664308118890176046988926827907562255859375f,-1.416500015765428610592380209709517657756805419921875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.200400010245293191957216549781151115894317626953125f,1.0983999816685925399184498019167222082614898681640625f,-0.190099994631112156895369480480439960956573486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.553200008015334976363419627887196838855743408203125f,0.0424999823421241540444270867737941443920135498046875f,0.623999997437000519795446962234564125537872314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.183000014245509934340816471376456320285797119140625f,-1.2606000166624784508684342654305510222911834716796875f,1.5945999705880884445008405236876569688320159912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2432999963879580729297913421760313212871551513671875f,-1.343299990129470433686265096184797585010528564453125f,1.778399985840918073876082416973076760768890380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.369799918015301987139764605672098696231842041015625f,-1.2936001273185000481902307001291774213314056396484375f,1.6003999758049853863184353031101636588573455810546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.22329995060414009611093888452160172164440155029296875f,-0.73149997590482207510120815641130320727825164794921875f,-1.4078000420078626842013136410969309508800506591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3767000035375345223798149163485504686832427978515625f,0.367399939878283898764266268699429929256439208984375f,1.1900000197999178741525838631787337362766265869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1219999834448091124983193367370404303073883056640625f,-0.36439998352378621948588488521636463701725006103515625f,-0.597799991157650811857138251070864498615264892578125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1708000393964355101417140758712776005268096923828125f,1.17350000077486082972200165386311709880828857421875f,-0.284200009572506129273961050785146653652191162109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.617600018957257024254658972495235502719879150390625f,0.55489999162256697928796711494214832782745361328125f,0.383399983382224807115790099487639963626861572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2585999634802345070738738286308944225311279296875f,-0.2813999578975139304048980193329043686389923095703125f,1.61429999276697611065856108325533568859100341796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.60970009066834929267741927105817012488842010498046875f,-1.50760000975579000481729963212274014949798583984375f,-3.714799910080433420489498530514538288116455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.082800021891296982090580058866180479526519775390625f,-0.57599998799711471519913175143301486968994140625f,-1.34799998541176346833481147768907248973846435546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.26889993724078120607146047404967248439788818359375f,0.34060009535923574386373502420610748231410980224609375f,0.886299967196582993977926889783702790737152099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3649000275343661225946334525360725820064544677734375f,-0.68179998922795104210337058248114772140979766845703125f,-1.8521999785557385909129379797377623617649078369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.330400030529499844078600290231406688690185546875f,1.31270003105997989933939606999047100543975830078125f,-0.428199921104311709285639153677038848400115966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4530999750137321147036573165678419172763824462890625f,-2.11050001856684676937447875388897955417633056640625f,0.122500018626451545600275494507513940334320068359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6998999997436998743438607561984099447727203369140625f,-2.441500000715254969918532879091799259185791015625f,0.859600036153197688548743826686404645442962646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.135700003321470585859742641332559287548065185546875f,1.238699955447762679483503234223462641239166259765625f,0.09510003966987168322333445757976733148097991943359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.4139999980330468787315112422220408916473388671875f,-1.83780006201267109844366132165305316448211669921875f,-0.34719995825290705937504753819666802883148193359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2050000203400854648094764343113638460636138916015625f,0.9133000116616489805210221675224602222442626953125f,-0.163300069850682572081268517649732530117034912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.217499957680701072604279033839702606201171875f,0.14969995016902759576993275913991965353488922119140625f,1.131600012743472660758925485424697399139404296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.630999967396258742979853195720352232456207275390625f,-0.7215999462842928036110379252932034432888031005859375f,-0.6162000614508993390217028718325309455394744873046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.92490001330375637422775980667211115360260009765625f,0.79360000058412527579321249504573643207550048828125f,0.31410002302527484374650157405994832515716552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5799000337183459752310454859980382025241851806640625f,-0.0537000087380408164250411573448218405246734619140625f,-0.7971000195384034814338747310102917253971099853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1797999616943304346250442904420197010040283203125f,-1.3287999540090549288606780464760959148406982421875f,-0.8761000149078663756796458983444608747959136962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.028499935768544037983929229085333645343780517578125f,-0.08790001190304774780770458164624869823455810546875f,0.621200086224079850438783978461287915706634521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.05659991383701468148359481347142718732357025146484375f,0.5538000238955020915199156661401502788066864013671875f,-2.571499924123287428301409818232059478759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.62589996537044634283830646381829865276813507080078125f,0.507899962282181149930693209171295166015625f,0.67530001352802015990306472303927876055240631103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.66709997990131342238129263932933099567890167236328125f,-1.4809000021576874672746271244250237941741943359375f,0.371400065875053808639449925976805388927459716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.931499990433454883032027282752096652984619140625f,-1.385699995274842333259357474162243306636810302734375f,0.58030000970959694228668013238348066806793212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8061999595642088678459913353435695171356201171875f,-0.12410002108812367982437763203051872551441192626953125f,2.33849996985495156565093566314317286014556884765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6528999339699748105658727581612765789031982421875f,-3.512900109505652235242223468958400189876556396484375f,-0.9402000189900405757725820876657962799072265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.702700040149687676915846168412826955318450927734375f,-0.548899988314509368336757688666693866252899169921875f,2.090299986687303768206902532256208360195159912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.456500023365020712873274533194489777088165283203125f,0.7939999579489238623608571288059465587139129638671875f,-1.930099947357178269413680027355439960956573486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.18960000165701007546203982201404869556427001953125f,-1.47280000211000494658719617291353642940521240234375f,-1.05090009871721346002004793263040482997894287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.612000005275010749983266578055918216705322265625f,0.54309996443390762266290039406158030033111572265625f,0.6171999989330771985152068737079389393329620361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.29669999400079227758197930597816593945026397705078125f,-1.14549998369813010867801494896411895751953125f,0.8346000328004361534794952603988349437713623046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.81300003801286202520515189462457783520221710205078125f,-2.16419998587965967118407206726260483264923095703125f,0.5224999366700640024419044493697583675384521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.40619993080496907822407592902891337871551513671875f,1.02530003235936106875669793225824832916259765625f,-1.81220001424401999656765838153660297393798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.607799992275238398775627501890994608402252197265625f,-1.300100121402742114895545455510728061199188232421875f,0.785599961900711463158586411736905574798583984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.500799960401653532215959785389713943004608154296875f,0.14430003333538798937496494545484893023967742919921875f,1.4203999792695041204382278010598383843898773193359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.14330003647208133799040297162719070911407470703125f,-0.4993999656796450636875306372530758380889892578125f,-0.77520002448484304213849327425123192369937896728515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2688000061631201642597943646251223981380462646484375f,-0.413699974018336025238795627956278622150421142578125f,0.833400028532745640319490121328271925449371337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5925999540999526971063460223376750946044921875f,-1.0597999772846697918993186249281279742717742919921875f,-0.5410000350475303587671760396915487945079803466796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9759999975711093345154267808538861572742462158203125f,1.095899879185857184182850687648169696331024169921875f,-1.5049000024072818337828039148007519543170928955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.11199993468076030467273085378110408782958984375f,-1.8317999820753929629546519208815880119800567626953125f,1.631000053115190695507408236153423786163330078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.076799972501397473223505585337989032268524169921875f,-1.45799996097385786697486764751374721527099609375f,0.90089990399628927519870558171533048152923583984375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9144999423548565431474344222806394100189208984375f,0.034199993143977369669528343365527689456939697265625f,-1.968799972430616218588284027646295726299285888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.375699994492531885725838947109878063201904296875f,0.05680007047653123919417339493520557880401611328125f,0.19620004852414130169790951185859739780426025390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.164700027357041367537249243468977510929107666015625f,-0.5392999373346558655839544371701776981353759765625f,1.483300026190280451743319645174778997898101806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6448000299602743012883365736342966556549072265625f,1.1897000105187294227704342119977809488773345947265625f,-0.6747999657176444454620423130108974874019622802734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1162999346375455100854878764948807656764984130859375f,1.0623999496757992044848606383311562240123748779296875f,-0.748700024235247507675694578210823237895965576171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3704999759308968965143549212370999157428741455078125f,0.2940999262608603448398980617639608681201934814453125f,1.016699995714427817716796198510564863681793212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.37220003871917750615239128819666802883148193359375f,1.7409000057525940707847666999441571533679962158203125f,0.78250005984678849824121016354183666408061981201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1352999637395146947937973891384899616241455078125f,-3.30570003465115913598992847255431115627288818359375f,2.98279997834265397926856167032383382320404052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.39890008601024795620304530530120246112346649169921875f,0.83819998784288796134234189594280906021595001220703125f,1.3962000399187213250939976205700077116489410400390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1626999199718246469359428374445997178554534912109375f,1.353199944350123029579435751657001674175262451171875f,-0.154499992832542343279556007473729550838470458984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5969999504759908592887995837372727692127227783203125f,-0.02649997372552703733816770181874744594097137451171875f,0.66509989721477047464048837355221621692180633544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3642999736189853621226575342006981372833251953125f,-3.039100006652623253700085115269757807254791259765625f,-1.2939999815672624805529267177917063236236572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9346000889405601252946098611573688685894012451171875f,2.10679977439045895692970589152537286281585693359375f,2.677199882517754314648072977433912456035614013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2178000372022401709415362347499467432498931884765625f,0.764899951780587006311407094472087919712066650390625f,0.9028999214343722545805803747498430311679840087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.123099979893862343516275359434075653553009033203125f,-1.396999944254755821049229780328460037708282470703125f,-1.79089999191313875570585878449492156505584716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.209700036893785135561074639554135501384735107421875f,-0.840700002501905796492565059452317655086517333984375f,0.5155000340268018899081425843178294599056243896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.283799991299210407902364750043489038944244384765625f,-0.78189994355142200532782226218841969966888427734375f,0.36380001391172445135424595719086937606334686279296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1499999722093352527707565968739800155162811279296875f,-0.7861999968729913579323920203023590147495269775390625f,-0.32240001028627229828771305619738996028900146484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9155999082937829047779132451978512108325958251953125f,3.360599928261340352264596731401979923248291015625f,-0.5541998901769533691918923068442381918430328369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.47340007759481483873287288588471710681915283203125f,-4.05730000816732516710771960788406431674957275390625f,-0.73839997097700926786956188152544200420379638671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2559001295626177618913743572193197906017303466796875f,-0.99249999225139606817691628748434595763683319091796875f,-2.468800128352643508122810089844278991222381591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.741800028380750209322513910592533648014068603515625f,-0.2929000213466583790733466230449266731739044189453125f,2.149599920743704206671509382431395351886749267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.31590008381605105824974089046008884906768798828125f,-1.001200018870830543704641968361102044582366943359375f,-0.78949996697902680153902110760100185871124267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5431000739574418645361220114864408969879150390625f,-1.3978000208854677310910119558684527873992919921875f,1.577600011972335902754593917052261531352996826171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.32020002137422576282688169158063828945159912109375f,0.284499969549476983132763052708469331264495849609375f,-1.2382000427722943403097133341361768543720245361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0745999888047581549699316383339464664459228515625f,-1.588799941510707913749911313061602413654327392578125f,1.863599972197414356145372948958538472652435302734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.360500054795295010734434981714002788066864013671875f,-2.25380002709925264525736565701663494110107421875f,-3.912999970175327302257528572226874530315399169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.80699993970990302472046096227131783962249755859375f,0.98170001219883484822048558271490037441253662109375f,3.026699935234336891198836383409798145294189453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.436000067867338803040411221445538103580474853515625f,-2.662399945615229857054373496794141829013824462890625f,-0.3499999999441183451409642657381482422351837158203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.241299997986108127889792740461416542530059814453125f,-0.3074999403953544430123656638897955417633056640625f,0.46769999142289098159608329297043383121490478515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6365999810598792496563191889435984194278717041015625f,-1.096399988314509688080988780711777508258819580078125f,0.58080003252327383478359479340724647045135498046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.593699933487176867430434867856092751026153564453125f,-0.0121000756412750209989326322101987898349761962890625f,1.063800017245859930170581719721667468547821044921875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.020100048796833025477326373220421373844146728515625f,0.32199997506290778392923357387189753353595733642578125f,0.03260001444965610062354244291782379150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.83660008773356597799875089549459517002105712890625f,-1.651999921977519658611299746553413569927215576171875f,-0.79360002442598531757766977534629404544830322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.024399936473371131029352909536100924015045166015625f,-2.2310999159082793852348913787864148616790771484375f,0.983400007168204748353446120745502412319183349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.36560008151978296009332325411378405988216400146484375f,-1.4622999579131590319747147077578119933605194091796875f,-0.6768998987361800612205797733622603118419647216796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.155000036042184152762501980760134756565093994140625f,-1.1766000472769129192585069176857359707355499267578125f,-0.10919998945593878403315102332271635532379150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.00530005225241125810242692750762216746807098388671875f,-2.291999941274524221768160714418627321720123291015625f,1.079500066816806214120560980518348515033721923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5083000528633601078354331548325717449188232421875f,-0.457200041624904240933346954989247024059295654296875f,-0.6779999029338359406438030418939888477325439453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.15590007468909039545224004541523754596710205078125f,-4.309499973572787467901434865780174732208251953125f,-0.31489990590363614586522089666686952114105224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3098999269142759249717755665187723934650421142578125f,-3.556399990028144220133299313602037727832794189453125f,1.1614000260889538740372017855406738817691802978515625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.894399988329410700060861927340738475322723388671875f,1.2145999774426206041511022704071365296840667724609375f,0.336100117483736138268568538478575646877288818359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8860998878195875949614901401218958199024200439453125f,-3.62979992531686956880321304197423160076141357421875f,1.3136000285424305200621120093273930251598358154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.676500023439527087276701422524638473987579345703125f,-0.426000088319182790286276940605603158473968505859375f,1.97919996967464584258777904324233531951904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0766000393979255900234193177311681210994720458984375f,-0.95439995770752350523480345145799219608306884765625f,1.7021000018805256814857784775085747241973876953125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.254700016292929820593826661934144794940948486328125f,0.35589998468607675352615160591085441410541534423828125f,0.2293999562174076078946427514893002808094024658203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.245299895603955153688957580015994608402252197265625f,0.35829992797300302953544814954511821269989013671875f,2.38199999853223598478280109702609479427337646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.36179993858262804451442207209765911102294921875f,0.1348000981889667659885390094132162630558013916015625f,4.500399898169934687075510737486183643341064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.46799996868520832293825151282362639904022216796875f,0.1652000738263124102189749464741908013820648193359375f,2.88219988691359763066657251329161226749420166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.410099988223614264626348813180811703205108642578125f,-2.323299988434463170250410257722251117229461669921875f,3.79419995998889358901351442909799516201019287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.517500000149011096794993136427365243434906005859375f,1.402999985560775098036856434191577136516571044921875f,-0.9390999820470806724870271864347159862518310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.016400008580089053822348432731814682483673095703125f,0.8342000234678395020893049149890430271625518798828125f,-0.8374999894574279846182207620586268603801727294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7581999880105254607798315191757865250110626220703125f,-1.177799966235458128949176170863211154937744140625f,0.4421000001668924372921765097999013960361480712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.24589998312145466830003215363831259310245513916015625f,-0.4849000306263555604147086341981776058673858642578125f,1.1396999984487898860407995016430504620075225830078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.54940010299384767211705593581427820026874542236328125f,-0.74729999373182642852242452136124484241008758544921875f,0.36639997197911089443778109853155910968780517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8009000433966519949535722844302654266357421875f,-1.2322999887585639466891507254331372678279876708984375f,-0.1256000062689188734310619111056439578533172607421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3362999970734119603577028101426549255847930908203125f,0.7164999934509399537319040973670780658721923828125f,-1.306199981729685877240854097180999815464019775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.330099987366795577514722026535309851169586181640625f,-0.19609998352229585094619324081577360630035400390625f,0.80140003733932985152677019868860952556133270263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.08039997956752653607281899894587695598602294921875f,0.614300051589310669442056678235530853271484375f,0.25229994718432380285122462737490423023700714111328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.79260005216822049245450898524723015725612640380859375f,0.55479999315440675644595103221945464611053466796875f,0.8823000915765770724163985505583696067333221435546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1893000036634505389709914879858843050897121429443359375f,-0.623899996547401247681818858836777508258819580078125f,0.5843000220850107506720405581290833652019500732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.298999954223633235272927777259610593318939208984375f,-2.14619997211843571705003341776318848133087158203125f,-0.411899994625149812321751596755348145961761474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.12119993788301919312999643807415850460529327392578125f,-0.02880000828653550559721452373196370899677276611328125f,0.3237000404030092592932987827225588262081146240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.69759988760352176218049180533853359520435333251953125f,0.7120000450611119191535181016661226749420166015625f,-1.727499966919421314770488606882281601428985595703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.466899973744899110528194796643219888210296630859375f,0.248800014656782142452584594138897955417633056640625f,-0.003899980323762097800255332913366146385669708251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.14579999910742003077501749430666677653789520263671875f,-0.0981000161409377735566295086755417287349700927734375f,0.167800000694394046973911827080883085727691650390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.10820000391751494195347049753763712942600250244140625f,-0.9100000380352144535578418071963824331760406494140625f,0.0052000014066695410264173915493302047252655029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.57740000644922329087194157182238996028900146484375f,-0.5740000061541792319985688664019107818603515625f,1.8199000397346913882046237631584517657756805419921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.596599957683680681697069303481839597225189208984375f,-2.389900010807812780200265478924848139286041259765625f,-0.02269997063577122720090528673608787357807159423828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6699000228032467862959720150684006512165069580078125f,-0.21419994896203231604658867581747472286224365234375f,1.7565000244826076691850857969257049262523651123046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.2510000089332464767721830867230892181396484375f,0.17280003831982659878718777690664865076541900634765625f,-0.279600032539665033226583545911125838756561279296875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2347999991893765514561209784005768597126007080078125f,-1.6141999945968390495210087465238757431507110595703125f,-0.958900058722496506646848501986823976039886474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.85819998517930462522684820214635692536830902099609375f,0.4212000209569932707154293893836438655853271484375f,0.90180000684857353210333030801848508417606353759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2007999996848404133942267435486428439617156982421875f,-0.70680004656016859332368085233611054718494415283203125f,-0.0077000089690091311211972424644045531749725341796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.15460002963393859598539847866049967706203460693359375f,-0.8604000026270741674494502149173058569431304931640625f,0.4283999621108172117800449996138922870159149169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.316799946722388536812786696827970445156097412109375f,1.993000058241189975660745403729379177093505859375f,-0.15310008798316143696638391702435910701751708984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.74679994385391434974508229061029851436614990234375f,-0.725499996960162985004672009381465613842010498046875f,0.3056000009417534357680779066868126392364501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.87109997635111113822858897037804126739501953125f,0.62190005605518983156088097530300728976726531982421875f,0.9259000152707097530679902774863876402378082275390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.97799999196827414049693061315338127315044403076171875f,-1.19259997604191259057415663846768438816070556640625f,-0.82510003568381107275087060770601965487003326416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.527800029751658517085388666600920259952545166015625f,2.34000007100403362159113385132513940334320068359375f,1.2239000286594035049603235165704973042011260986328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4384000180274247870926274117664434015750885009765625f,-0.20730002265870695055127725936472415924072265625f,-1.332900068788230196759059253963641822338104248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.81590000644177174127236185086076147854328155517578125f,-2.28349991650879413640495840809307992458343505859375f,-0.39470003219991889320539257823838852345943450927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,4.756199895600975224851936218328773975372314453125f,-0.02850007028877765691277090809307992458343505859375f,-1.6132999298170209723224388653761707246303558349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2185000275969499750061686427216045558452606201171875f,2.0740000128597007034159105387516319751739501953125f,-1.9695999263316394944922649301588535308837890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.892799986389277488996185638825409114360809326171875f,0.06160008192211297028961780597455799579620361328125f,1.37949997722357498020073762745596468448638916015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4023999641299251806003667297773063182830810546875f,-1.701699978354572539984701506909914314746856689453125f,-0.4108999999970190941667169681750237941741943359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1509999985769375496769129085805616341531276702880859375f,0.039999939538539253103266446487396024167537689208984375f,0.184100001157820936104059228455298580229282379150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.897800056387483724762432757415808737277984619140625f,-0.5964999725520601092654260355629958212375640869140625f,0.621099953999370679724734145565889775753021240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,3.39539996012151323867556129698641598224639892578125f,-1.34120002665668636154805426485836505889892578125f,-0.12129994465857751873016923127579502761363983154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.45009993228837907874861912205233238637447357177734375f,1.244699911761284027988949674181640148162841796875f,-0.0333000299155708778808815395677811466157436370849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.552299983617663858836976942257024347782135009765625f,1.7556000460363925252949002242530696094036102294921875f,-0.10569996025711214659992265296750701963901519775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1439999602586039539886542115709744393825531005859375f,-2.303299977165460976635813494795002043247222900390625f,-0.92689995342343944884078155155293643474578857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.58700000575929767165916928206570446491241455078125f,0.183400036281347655897633330823737196624279022216796875f,2.09560001405477525082687861868180334568023681640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.48109998743385151698248591856099665164947509765625f,-0.24149999687820622451539520625374279916286468505859375f,1.235699949640034933651122628361918032169342041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.334000039495526923616353087709285318851470947265625f,0.62270002636611454516213370879995636641979217529296875f,-0.62080002205520889990708610639558173716068267822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7935999637782582549760945767047815024852752685546875f,2.87760004700869398419627032126300036907196044921875f,-0.64379993769228349709266012723674066364765167236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1455000483319162096762511282577179372310638427734375f,0.110300011851635293425033523817546665668487548828125f,-0.1404000014349815472769478219561278820037841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.786399911722540867486941351671703159809112548828125f,-2.668499998260289718388094115653075277805328369140625f,-0.78590004272609892410628162906505167484283447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.70579995336085499246792096528224647045135498046875f,-0.22910003347471386003775251083425246179103851318359375f,0.9516000222429628063736117837834171950817108154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.468199985030292253895822796039283275604248046875f,0.1626999510966233852826690053916536271572113037109375f,1.67870006778389235790882594301365315914154052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.05440001042038267176081944853649474680423736572265625f,0.3512999449193487944143043932854197919368743896484375f,0.465600015135108957764487058739177882671356201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4296000363394620880086449687951244413852691650390625f,1.1054999718517060092182191510801203548908233642578125f,-2.95430004767775500340576400049030780792236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.189900011925399514467471817624755203723907470703125f,-1.4839999690130358800388421514071524143218994140625f,1.4033999804019925417009062584838829934597015380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.65319997866004708697573732933960855007171630859375f,-1.7812000805616363674488411561469547450542449951171875f,1.01030008395463344328391031012870371341705322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.40239995742440282100460535730235278606414794921875f,-2.210599998408555588724766494124196469783782958984375f,-0.3217999888554217147174085766891948878765106201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6212999686494453666085746590397320687770843505859375f,2.166300022852421403030120927724055945873260498046875f,0.184600015887617541210374838556163012981414794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.696399959778786215025547789991833269596099853515625f,-0.993300014269351994045109677244909107685089111328125f,-0.562400011739134786381555386469699442386627197265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2865000165104862883680425511556677520275115966796875f,-2.019900049699843602724058655439876019954681396484375f,-2.01169996449649257641567601240240037441253662109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8348999812662596387013991261483170092105865478515625f,-3.171799975239485203104550237185321748256683349609375f,-0.11760007282495521518939085581223480403423309326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.555400034515559415382313090958632528781890869140625f,0.951800056134165739507579928613267838954925537109375f,-2.60059997054338420952035448863171041011810302734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.193400049133599039041797595928073860704898834228515625f,-0.90229995515644556558498834419879131019115447998046875f,0.5738999186702089527756243114708922803401947021484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.059400073117017893054025989840738475322723388671875f,2.811299959969521200520148340729065239429473876953125f,-1.2233000553220494399653262007632292807102203369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.8848999522089950886538645136170089244842529296875f,-2.961200094009934336014566724770702421665191650390625f,-0.2007999189205469381391822025761939585208892822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.25309997587055022005841919963131658732891082763671875f,-0.4199999134242542453421265236102044582366943359375f,0.401600006112456109352848443450056947767734527587890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3837999755598604512130123112001456320285797119140625f,0.435699962976574983741784308222122490406036376953125f,-0.6265999757513405210573864678735844790935516357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.667500044144691440095584766822867095470428466796875f,1.5225000004097817285497740158461965620517730712890625f,1.817899992233515060746640301658771932125091552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1176999833762655267577201811945997178554534912109375f,1.642599978649617487036493912455625832080841064453125f,-0.754899987114966197765397737384773790836334228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.928199964429437418544921456486918032169342041015625f,0.51539993598163302834791466011665761470794677734375f,-1.2766999704197055098831015129690058529376983642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.8912999809056518785155276418663561344146728515625f,-0.5999000090569264198592236425611190497875213623046875f,-2.288099966575949384406385433976538479328155517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2355000801458955184131127680302597582340240478515625f,-3.45630001420974952708320415695197880268096923828125f,2.6064999518394476041294183232821524143218994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.16919997934177555265478076762519776821136474609375f,-0.0568000304669140643198943507741205394268035888671875f,0.83710003101229712996200760244391858577728271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.27470003922209240698748544673435389995574951171875f,-0.4450000621378424892071734575438313186168670654296875f,2.9463000399142504903693406959064304828643798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.740700026194752059183201708947308361530303955078125f,-0.783900028026104589429223779006861150264739990234375f,-0.91089995216429187063766903520445339381694793701171875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.39559994763284844765394154819659888744354248046875f,-2.949400013400613307368303139810450375080108642578125f,0.3004000178635133355697917068027891218662261962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6695000506490489389221920646377839148044586181640625f,-4.6989999300092453182742247008718550205230712890625f,0.8895000228956322363416120424517430365085601806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3443000647395848545784247107803821563720703125f,0.941199910539389339447780002956278622150421142578125f,0.4024000524193034333819696257705800235271453857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8334000198900692879533380619250237941741943359375f,-0.83850000939890634299445082433521747589111328125f,0.67070004154294782239276173640973865985870361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.244799944651127976413818032597191631793975830078125f,0.2666000152766712094631884610862471163272857666015625f,0.7004999956190578469517049597925506532192230224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5209000024929639760529198611038736999034881591796875f,1.5728000324338673454604986545746214687824249267578125f,2.0767000226110230443055115756578743457794189453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6648000438556074254137229218031279742717742919921875f,-0.0445999796032909845422409489401616156101226806640625f,1.762599994556605764728374197147786617279052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.79810000015944115858701479737646877765655517578125f,-0.0362000513367360110539294737463933415710926055908203125f,2.588100080327689678227898184559307992458343505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.389799962327627813607477946789003908634185791015625f,-1.73329996187314261391065883799456059932708740234375f,-3.239699991706013637582373121404089033603668212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.46299998261779562458428927129716612398624420166015625f,-1.6202000161588172577609157087863422930240631103515625f,0.829399997903407015797938584000803530216217041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0910998736262325081725066411308944225311279296875f,0.76600001206249046514784595274250023066997528076171875f,1.454799911049009519814489976852200925350189208984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.28429995178878275652323281974531710147857666015625f,-0.36669997783303254212938782075070776045322418212890625f,0.336799996939301848897230229340493679046630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2913999954670665193390277636353857815265655517578125f,0.36220001592040063087551970966160297393798828125f,0.0764000369668007550671973149292171001434326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.22619998854696721224399880156852304935455322265625f,-0.96539998880624811050665812217630445957183837890625f,-0.542599980437756368445434418390505015850067138671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3682000109583132285706597031094133853912353515625f,-0.306800008562207349172012982307933270931243896484375f,-0.938399967922269429010384556022472679615020751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.159700023054332174154978929436765611171722412109375f,-0.9977999967828397043234645025222562253475189208984375f,-0.94009999839365487872555604553781449794769287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.07380004334151823996990060550160706043243408203125f,0.50010003344863707130940611023106612265110015869140625f,0.9194999792352323009225756322848610579967498779296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.719599988692998682182633274351246654987335205078125f,-0.549300009381771037197950136032886803150177001953125f,1.8885000071078525962775529478676617145538330078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.67660004604756895929540405631996691226959228515625f,-0.25730001267492774719158887819503434002399444580078125f,-0.75370003917366312773395975455059669911861419677734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.199999980628490447998046875f,0.423499997012317219713395388680510222911834716796875f,-0.3376000078745187149564799256040714681148529052734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.171600012855231742747719181352294981479644775390625f,-0.3637000012874604948365231393836438655853271484375f,-0.719899991846084663649207868729718029499053955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.154899967147410055900991210364736616611480712890625f,-0.66740000499635865338632356724701821804046630859375f,-0.323599976965784907889656096813268959522247314453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.26320006911009541372692410732270218431949615478515625f,0.32649998227506771275585606417735107243061065673828125f,-0.73659993656128663896964781088172458112239837646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.67540001372843960769642990271677263081073760986328125f,-0.90630004419833465334477295982651412487030029296875f,-0.6307999789535994050737599536660127341747283935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.31230002306848714255238519399426877498626708984375f,0.5130999757960441609583313038456253707408905029296875f,0.398300029449909998024992319187731482088565826416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0944999760687343925980030689970590174198150634765625f,-0.31519999205619086968255260217119939625263214111328125f,0.15199999085813720167692508766776882112026214599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.38509997090101266703499049981473945081233978271484375f,0.5238999982237810826291024568490684032440185546875f,0.90779999540448130801451043225824832916259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3170000167489046560831411625258624553680419921875f,-0.8500999771595001419655091012828052043914794921875f,0.172300022584199297170926001854240894317626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.59409996513426310826844201073981821537017822265625f,0.65239998467490067923790775239467620849609375f,0.42330002566874014302555906397174112498760223388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.6632000307023522278626614934182725846767425537109375f,-0.1769999404177051705033818507217802107334136962890625f,1.3912000266566881823138146501150913536548614501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3888000564172864326195622197701595723628997802734375f,-1.120400009481608893935344894998706877231597900390625f,-0.10640001554638145453424158404232002794742584228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.30230000846534921254971095549990423023700714111328125f,0.6939999681748452164953278042958118021488189697265625f,-0.8837000034108764001672398080700077116489410400390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.91840001866072407121777132488205097615718841552734375f,0.30179999500214993446434164070524275302886962890625f,0.0113999944239856765282326023225323297083377838134765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.187899973011017351609552861191332340240478515625f,-1.189999978542327863095806605997495353221893310546875f,0.571599991583823463514590912382118403911590576171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.06220001875162051163670184905640780925750732421875f,-0.9446000151425602009425119831576012074947357177734375f,-0.396799986881017474615873652510344982147216796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.128300012388079809344532122850068844854831695556640625f,-0.51020001381188506694996931400964967906475067138671875f,1.169999995417892346694088701042346656322479248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.361800018993019278923384263180196285247802734375f,0.460499970510600764583841737476177513599395751953125f,2.087900046510994922499548920313827693462371826171875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.11180002532601474030116150970570743083953857421875f,-1.0018999701872470797781033979845233261585235595703125f,-0.0190000462383028700941167699056677520275115966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.090799987428635819952660312992520630359649658203125f,-0.29650002727657553958096059432136826217174530029296875f,1.0308000214964161411757004316314123570919036865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6519000305369500036789531804970465600490570068359375f,0.70040003369599634908126972732134163379669189453125f,-0.5847000263139607856288648690679110586643218994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.35259997440278567637506057508289813995361328125f,-2.0125999454200265148529069847427308559417724609375f,-0.13940001544952362877438645227812230587005615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.597799961280822689246861045830883085727691650390625f,0.23180000167042014158624851916101761162281036376953125f,3.1658000098735090688251148094423115253448486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.099499994210898723423497358453460037708282470703125f,-0.8082000894129268697696488743531517684459686279296875f,1.560100086683035680579223480890505015850067138671875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.615799971447140936930964016937650740146636962890625f,-1.7558000262461626039822704115067608654499053955078125f,-0.32740006558820533566489530130638740956783294677734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.538999979257583472502801669179461896419525146484375f,-0.11229993809461635834168191649951040744781494140625f,0.12890000258236999908234565737075172364711761474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.7639999949783078392329116468317806720733642578125f,0.347299941149353863512061479923431761562824249267578125f,-0.7591001014612623887245490550412796437740325927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.705300060224534064445833791978657245635986328125f,0.32430008537769416943064015868003480136394500732421875f,0.62520001729503282827948851263499818742275238037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1054999462589625291997208478278480470180511474609375f,-1.525599940368532969614534522406756877899169921875f,2.37489994058609088511957452283240854740142822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3024999776855106237150039305561222136020660400390625f,0.4898999870590865501895905254059471189975738525390625f,-1.6964999182745812422723474810482002794742584228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.95509999772310294474664260633289813995361328125f,-0.2846000973224642738301781719201244413852691650390625f,-0.5396000720277418505332889253622852265834808349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1736000478580608241685467874049209058284759521484375f,0.66190000839009865973849855436128564178943634033203125f,-1.846499954335392157389605927164666354656219482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0909000280484557610094498159014619886875152587890625f,-2.261000045161694060880108736455440521240234375f,1.521399933329224740674590066191740334033966064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4074999828264107559760986987384967505931854248046875f,-2.83330002097487554379995344788767397403717041015625f,0.80180002465546096335202719274093396961688995361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9338000325940551960002267151139676570892333984375f,0.5385000637322667760287231430993415415287017822265625f,0.1095000212565062636116408611997030675411224365234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.35179997406601959397676182561554014682769775390625f,0.8814000053018329339238334796391427516937255859375f,-0.10680001973807851101128107984550297260284423828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.115899988597631509179564091027714312076568603515625f,-0.4660999919414516856619457030319608747959136962890625f,0.10069996260404590060488771996460855007171630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.28600000610202602757681233924813568592071533203125f,-0.2997999868586658234193009775481186807155609130859375f,-0.7374999836459761315410332827013917267322540283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.33159989383220800363005764666013419628143310546875f,0.989399955397842756354975790600292384624481201171875f,2.6555000639408827822762759751640260219573974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1379000161126271439826496134628541767597198486328125f,-0.4814000102564686134343219237052835524082183837890625f,1.2993999659031629789751605130732059478759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.463499948695301977608096422045491635799407958984375f,0.5185999487280839037595114859868772327899932861328125f,0.6513001204922790332574322746950201690196990966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.251999956436455452291056644753552973270416259765625f,-0.477799975734949811112528550438582897186279296875f,0.27520000993758453322612922420375980436801910400390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.99519997160434769956083300712634809315204620361328125f,1.026899978028982474143049330450594425201416015625f,-1.332099998937546825317213006201200187206268310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1016999730646610533568718892638571560382843017578125f,0.8343999796867376250730785613995976746082305908203125f,0.5293000215262164687146650976501405239105224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5288000417396439001294083936954848468303680419921875f,2.753500045217574410116867511533200740814208984375f,-1.058100052946806091114240189199335873126983642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,5.31899992891028627894911551265977323055267333984375f,1.632100043808668576872378253028728067874908447265625f,4.88850000446289723043946651159785687923431396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.20530002308338968219914022483862936496734619140625f,-0.47469998593181317314559919395833276212215423583984375f,0.1028000615283841678859744206420145928859710693359375f,0.0f,0.0f,0.0f,0.0f,0.0f,3.105400045095382921545024146325886249542236328125f,-1.673400056807696767435800211387686431407928466796875f,2.47240007201433087402620003558695316314697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.871900049852581293663433825713582336902618408203125f,-1.204100016263871975041865880484692752361297607421875f,0.661300023187697316728872465319000184535980224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.371299925249815032657352276146411895751953125f,0.63030004025697650149595574475824832916259765625f,0.88440000531673401695798020227812230587005615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.88920000057593051590032473541214130818843841552734375f,-0.335400018310547221034312315168790519237518310546875f,0.05849997499585157800794377180864103138446807861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.308799973455071263828131122863851487636566162109375f,3.084400027146936285049605430685915052890777587890625f,-0.8708999234795571364742272635339759290218353271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.355300093826652840789392939768731594085693359375f,-2.699700025680659809523831427213735878467559814453125f,-0.24419999169111250569841331525822170078754425048828125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0677999852158137628066469915211200714111328125f,-0.981000022903084190062372726970352232456207275390625f,3.599899932334573282588507936452515423297882080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.92789996749758696381604750058613717555999755859375f,1.090600071945785121130256811738945543766021728515625f,-1.8230999893188475358130062886630184948444366455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.054299994704126997024928868995630182325839996337890625f,0.6648999849915508963960064647835679352283477783203125f,-0.254900054728984581942086151684634387493133544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.894299997311831074142673969618044793605804443359375f,1.0796999805301432662218985569779761135578155517578125f,0.009600042039155542994421921321190893650054931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0716999952673920137868890378740616142749786376953125f,0.17410002160966353557114416616968810558319091796875f,-1.4331000260129573842249328663456253707408905029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.59660002056658267832744968472979962825775146484375f,-1.133800039057433650668826885521411895751953125f,-3.477800008610635540406974541838280856609344482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.171300113116205299235161874094046652317047119140625f,2.71269996216073661088330482016317546367645263671875f,1.5452000566899768418949179249466396868228912353515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.7590999348104003896509084370336495339870452880859375f,-1.333400017692147887515830007032491266727447509765625f,1.3952999787896873673531672466197051107883453369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0462999481976031912466851281351409852504730224609375f,2.5078000769510868650513657485134899616241455078125f,1.2459000536412012660747450354392640292644500732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,3.22710000813901398686311949859373271465301513671875f,2.14509997157156373503994473139755427837371826171875f,-0.05829995876252669351202939651557244360446929931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.40170002015233041703368144226260483264923095703125f,-0.580599955046177118589412202709354460239410400390625f,-1.284800025378167287470887458766810595989227294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.716499932691454066713276915834285318851470947265625f,-1.236899955826252561763567427988164126873016357421875f,0.09790001629888944734148026327602565288543701171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.151799958531558587537801940925419330596923828125f,0.68060003284886594343561228015460073947906494140625f,0.65120001391619497521645598681061528623104095458984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.46599999339878450665963782739709131419658660888671875f,0.5247999935641878277436944699729792773723602294921875f,-3.9417000845625995708587652188725769519805908203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.711800011281669586793441339978016912937164306640625f,-1.473400058130174716808369339560158550739288330078125f,-2.881099914343653978221482248045504093170166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.49740002261698224028663162243901751935482025146484375f,-1.211100024258346508077011094428598880767822265625f,0.4736999667167667293909971704124473035335540771484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4330000384226437848411705999751575291156768798828125f,-0.7459999941438433257445694835041649639606475830078125f,1.1374000684380529690997718716971576213836669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.05620000694990157086294857435859739780426025390625f,1.120200035083293688131789167528040707111358642578125f,-1.3073999618947509393507289132685400545597076416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3918999632954598411060942453332245349884033203125f,-1.4877999646708364167579929926432669162750244140625f,0.40540006562173491744260900304652750492095947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1476000026404860054896062138141132891178131103515625f,0.568099984010308478588058278546668589115142822265625f,-1.1050000578165051212664593549561686813831329345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5352001287370919957453452298068441450595855712890625f,-1.203400012383609496424696772010065615177154541015625f,-1.1618999804317942103892846716917119920253753662109375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3168000578105452103017114495742134749889373779296875f,0.6351000026032316281288103709812276065349578857421875f,-0.137300042253732268449084585881792008876800537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.696700018885731964246588177047669887542724609375f,0.43290002278089489973211811957298777997493743896484375f,2.809199937078357134367934122565202414989471435546875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8328000089272860062550307702622376382350921630859375f,-0.237399983166158012881652439318713732063770294189453125f,0.1661999929055568703262224516947753727436065673828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.551800004091858919963442531297914683818817138671875f,-0.66959999435543959833694316330365836620330810546875f,0.63940000054836243492673020227812230587005615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.24040002486705791540089194313623011112213134765625f,0.66060003655552890222679707221686840057373046875f,0.991699974107742576734381145797669887542724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.39740001885443820839327599969692528247833251953125f,-0.0237000055715441693138245682348497211933135986328125f,0.71710001685619406686100774095393717288970947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3856000019475820383973996285931207239627838134765625f,-0.1004000287413600034369665081612765789031982421875f,2.2079999578446152241895106271840631961822509765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.14620001991391173845613593584857881069183349609375f,0.19349996516108458610716525072348304092884063720703125f,-1.0392999993979932238374885855591855943202972412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.152300007869303755114742671139538288116455078125f,-1.01769999473840044856842723675072193145751953125f,-1.0612999682024113479172910956549458205699920654296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.53640000455677494528572424314916133880615234375f,0.8237000282585622557007809518836438655853271484375f,1.25740003096163288631714749499224126338958740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1448999869287010966445450321771204471588134765625f,1.1977000317677861662701843670220114290714263916015625f,-0.47030001078993077801015942895901389420032501220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.289299984161555556738676386885344982147216796875f,2.18420001836419164220615130034275352954864501953125f,-1.40680001891851436113256568205542862415313720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.82779997297823448842990501361782662570476531982421875f,-0.60109992995262206960660478216595947742462158203125f,-0.0036000448405737461854414505069144070148468017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.17699999149143696985930773735162802040576934814453125f,2.172000042267143715690735916723497211933135986328125f,-1.569100033996253973356260758009739220142364501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5999999936297533142948168460861779749393463134765625f,0.242100039878487383049332493101246654987335205078125f,1.0573000053733585534843086861656047403812408447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.05500000901520252227783203125f,0.185400070166587216391462789033539593219757080078125f,-0.62799997904151660943483648225083015859127044677734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.282699999842047855480586804333142936229705810546875f,0.57229999859333047229625890395254828035831451416015625f,0.08370002818405619660069305609795264899730682373046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.63330001359879961597698638797737658023834228515625f,-0.62600002402067200790725109982304275035858154296875f,0.80189997149109881746653627487830817699432373046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0683999877035628145449663861654698848724365234375f,-0.627999999344349024710254525416530668735504150390625f,1.25229995761513723806501729995943605899810791015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.026799959760903657723929427447728812694549560546875f,-0.70580001067444708695575172896496951580047607421875f,-2.31520000434964945412730230600573122501373291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0208000174731015619045138009823858737945556640625f,0.24249994277954112220641036401502788066864013671875f,-1.360700011703372513949261701782234013080596923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.619900014644862107360268055344931781291961669921875f,1.032299967971443965808475695666857063770294189453125f,0.7918000268161293764279662354965694248676300048828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6253999943941830341742615928524173796176910400390625f,1.452499979082495062954194509075023233890533447265625f,0.6241000135257841474611950616235844790935516357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.92469999438822247928015940487966872751712799072265625f,0.03530002140700805313144883257336914539337158203125f,-0.6568000053584572839326938265003263950347900390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.14020000468492543888032741961069405078887939453125f,-1.1939000159561634628602178054279647767543792724609375f,-0.5555000675916674790499882874428294599056243896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.816799967956542527502961092977784574031829833984375f,0.01709998802244694360297216917388141155242919921875f,-0.391399976915121072806869051419198513031005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3977999951250847932016085906070657074451446533203125f,-0.614199952463806209124186352710239589214324951171875f,-0.981200033250452019473186737741343677043914794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.351399974903463085951216271496377885341644287109375f,0.6484000270053746906029346064315177500247955322265625f,-0.3655999837867922597212100299657322466373443603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3187999954342839448173663186025805771350860595703125f,1.891700012552738119353534784750081598758697509765625f,-0.443499969333410515304194632335565984249114990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.95290001039430460849644077825360000133514404296875f,-0.179099966959654910425570051302202045917510986328125f,-0.1974000033944849352707251455285586416721343994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8287000241234900865805457215174101293087005615234375f,-1.351200043271482353901546957786194980144500732421875f,0.15959997401535463978916595806367695331573486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.185000014752148889130012321402318775653839111328125f,-0.9804999865293499983209812853601761162281036376953125f,0.43839998055100437568398774601519107818603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.559599987053870595588023206801153719425201416015625f,-0.3001000240609050795370649211690761148929595947265625f,-0.5999999790638685936983165447600185871124267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9600999796181917123050197915290482342243194580078125f,-0.2531999619334939222170532957534305751323699951171875f,-0.469400001367926977824396317373611964285373687744140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.37640000749975666138880114885978400707244873046875f,-0.20450011319667094067398238621535710990428924560546875f,-0.544200052264331990414802930899895727634429931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6899000054806467829848770634271204471588134765625f,-0.31670003374964028619586997592705301940441131591796875f,-1.63639997847974161260253822547383606433868408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2982000015333294840047528850845992565155029296875f,0.1666000921666623835193377090035937726497650146484375f,1.1677999702587731345460042575723491609096527099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.877700041714310597029680138803087174892425537109375f,-1.7756000471539798990505687470431439578533172607421875f,0.614100033809990009103785268962383270263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6388000014320009967150326701812446117401123046875f,0.730899983531236419054266661987639963626861572265625f,-0.5681999786227940330007868396933190524578094482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.37110000170171275613029138185083866119384765625f,0.42109995947554701434256685388390906155109405517578125f,3.224200043547153171630270662717521190643310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7286999272286893702954557738848961889743804931640625f,-2.75279991043061134092795327887870371341705322265625f,-1.44349995908886175044472111039794981479644775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5250999871060251233956250871415250003337860107421875f,1.906499893650412946755068333004601299762725830078125f,1.0600000737980008747030069571337662637233734130859375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.22020002763271318002580301254056394100189208984375f,2.381499947052450050222205391037277877330780029296875f,0.33220003549680132692856204812414944171905517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2985999714151039707843438009149394929409027099609375f,-1.1659999681413177885502818753593601286411285400390625f,0.49050006479769958644965299754403531551361083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0342000673331330773407898959703743457794189453125f,-0.03120000879392048620530886182677932083606719970703125f,0.737199995058775758849378689774312078952789306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.569499937176704218444456273573450744152069091796875f,-0.6660000577718019343365085660479962825775146484375f,-0.6944000418245794836735740318545140326023101806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.850100037211179770935132182785309851169586181640625f,1.138800045949220685059799507143907248973846435546875f,1.717100023412704512537629852886311709880828857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.1926999070823196547053157701157033443450927734375f,1.785499972000717416875659182551316916942596435546875f,2.327199994611740319072623606189154088497161865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.041500044189392237825586562394164502620697021484375f,3.021599991639704274604127931524999439716339111328125f,-2.7142000517800450865024686208926141262054443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.455499904312194558997362037189304828643798828125f,-0.5921999480083586231415893053053878247737884521484375f,0.4447999752730142919432410053559578955173492431640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1904000184223055835985860539949499070644378662109375f,-1.662000029377638465888367136358283460140228271484375f,1.89389990583658285316914771101437509059906005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.34169993204921544904806296472088433802127838134765625f,0.19710000020414586430206327349878847599029541015625f,-1.614999977052211210803989160922355949878692626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.342599988894164708685963205425650812685489654541015625f,-0.95040001929774875488732277517556212842464447021484375f,-0.7891000141963360192676191218197345733642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9303000254303217531059999600984156131744384765625f,-0.562299963575601413623417101916857063770294189453125f,-1.3907000189304354886843384520034305751323699951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.7857000380039220743810801650397479534149169921875f,1.280200020498781565692070216755382716655731201171875f,-1.9555999759823079475751228528679348528385162353515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.123000005304813697648569359444081783294677734375f,0.08939993980750404745094783720560371875762939453125f,0.407699922840297990234148528543300926685333251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.179800027017294805631308918236754834651947021484375f,-0.1559000208586460711757126773591153323650360107421875f,-2.53909996491074618774064219905994832515716552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.640499981239438742619540789746679365634918212890625f,-0.970499900810420701446901148301549255847930908203125f,-1.4279000232279290827364093274809420108795166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4072999551378189142525343413581140339374542236328125f,-1.4118000250093647451876677223481237888336181640625f,-0.5608000139713287257592355672386474907398223876953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3212999924540518303928138266201131045818328857421875f,0.27079995496273034927270373373175971210002899169921875f,0.8993999728322024100890530462493188679218292236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8052000536724925883191872344468720257282257080078125f,-1.392200040507316050053532308083958923816680908203125f,0.2683999634146687629510097394813783466815948486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.099999980404972266256891089142300188541412353515625f,-1.008699957496673516033069972763769328594207763671875f,-0.9784999805837861774904240519390441477298736572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.9672999920926990125025213274057023227214813232421875f,-1.3263000242680320628352319545228965580463409423828125f,-0.14589998006671667241107570589520037174224853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5672999692939246774159300912288017570972442626953125f,-1.89509996207207453977616751217283308506011962890625f,0.0334999890029425984039335162378847599029541015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0837000141769641547995206565246917307376861572265625f,-2.16839998707026015978271971107460558414459228515625f,2.1038000376231966725981692434288561344146728515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.244400023943185384922571756760589778423309326171875f,0.219299969893694324252209071346442215144634246826171875f,0.6775999986916783068835457015666179358959197998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.22969995720982527842579656862653791904449462890625f,-0.005200022342801258190547741833142936229705810546875f,-0.069699961531161758188090971088968217372894287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.033999967485666360289542353712022304534912109375f,0.92619997997880021411987172541557811200618743896484375f,-1.8259000202625987885340919092413969337940216064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.319300035440175822287756091100163757801055908203125f,0.3221000125348567610927830173750407993793487548828125f,0.25760003279671106479753461826476268470287322998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.852999968573450839670613277121447026729583740234375f,2.758800034214555552125602844171226024627685546875f,0.121599975043535035723607506952248513698577880859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.2081000021710988079348680912517011165618896484375f,1.158500058665870913188200574950315058231353759765625f,-2.074499999761581836565937919658608734607696533203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.77579993249922996056966439937241375446319580078125f,0.56669994460344319531230894426698796451091766357421875f,0.299400010513514391874423381523229181766510009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.221699997949599991731162162977852858603000640869140625f,-0.97729995783865408753854353562928736209869384765625f,-0.8021000193893905727549054063274525105953216552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.03780004651546509109039106988348066806793212890625f,2.00390000641942034320663879043422639369964599609375f,0.09160001564919983962909100227989256381988525390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5079000324666491650305033545009791851043701171875f,0.2152000173322854426061212507192976772785186767578125f,-1.38390003860592880613467059447430074214935302734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.47270002092719121122854630812071263790130615234375f,1.4695000234544262074365406078868545591831207275390625f,-2.12019999477565335155304637737572193145751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.084700027804076594151183599024079740047454833984375f,1.0196999302014717603270810286630876362323760986328125f,0.458100036443769287330951556214131414890289306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.85640005032196686585166389704681932926177978515625f,1.0179999529831114113420653666253201663494110107421875f,0.788799875666200467350108738173730671405792236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.669100079984962459178632343537174165248870849609375f,-1.92929997728839452975080348551273345947265625f,1.0443000092700120973177035921253263950347900390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.699699978443980352693642998929135501384735107421875f,-0.071600017884373645671303165727294981479644775390625f,-0.44449998169392390678922311053611338138580322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.51160000655949122716492638573981821537017822265625f,1.290299987804889081388637350755743682384490966796875f,-0.485599997514486236838138211169280111789703369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0822000051915647844680279376916587352752685546875f,0.39230001569241235159779535024426877498626708984375f,-1.400300020885467677800306773860938847064971923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.563699976663290858169830244150944054126739501953125f,-3.2073000139042733280803076922893524169921875f,0.8603999950647349681531750320573337376117706298828125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9722000425562258296707796034752391278743743896484375f,-1.177300066728891092537878648727200925350189208984375f,0.9130999702081084024030133150517940521240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2423999615967280529815752743161283433437347412109375f,0.9556999764248728101989627248258329927921295166015625f,0.20269998933672950247597555062384344637393951416015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.443800007541478525041611646884121000766754150390625f,1.47380005163028737769081999431364238262176513671875f,-2.275899955759197457183518054080195724964141845703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.98090000465363302861732108794967643916606903076171875f,-1.545400020322202916389642268768511712551116943359375f,0.439200010466575196943495029699988663196563720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.255000030025839929948006101767532527446746826171875f,-0.48790004110932283509782791952602565288543701171875f,-0.77250002175569587592462994507513940334320068359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.392900010412931433023686622618697583675384521484375f,1.52640006304383302193627969245426356792449951171875f,0.46610000535249707098728322307579219341278076171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0842000064060091091278081876225769519805908203125f,0.8240999556347727850180717723560519516468048095703125f,0.565600003623962788168455517734400928020477294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6265000874772679306801137499860487878322601318359375f,1.2822999466627840181587316692457534372806549072265625f,-4.03959998154044086504654842428863048553466796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.081699977982044202917677466757595539093017578125f,0.351100012542308714813543701893649995326995849609375f,1.5115999394297592317570888553746044635772705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.04790004297196848170869998284615576267242431640625f,-0.2975000366568572029990491500939242541790008544921875f,2.38809998697191705474551781662739813327789306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.79900004154443760029380428022705018520355224609375f,0.36249997943639744590882401098497211933135986328125f,-0.52360003411173838827608051360584795475006103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.19489998961091004048284958116710186004638671875f,-0.3985999776735891231282948865555226802825927734375f,0.66179999921172860677387461691978387534618377685546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.058299949039518850213426048867404460906982421875f,0.24770004745125806522310085711069405078887939453125f,0.02589997161030765937539399601519107818603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.96109999358057951468481405754573643207550048828125f,-0.23049996224045798953738994896411895751953125f,-1.0164999437928214920390246334136463701725006103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,3.066700047123432693041422680835239589214324951171875f,0.14730002124309660160861312760971486568450927734375f,2.774200003537535597075702753500081598758697509765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.16089997585713877725765996729023754596710205078125f,-1.935300073337554493946299771778285503387451171875f,-1.31470003104507870972383898333646357059478759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.58670003132820180979933866183273494243621826171875f,2.0618999832630162671875950763933360576629638671875f,3.2557999570488931340150884352624416351318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.73670003490448010552427149377763271331787109375f,1.083800064501165838493079718318767845630645751953125f,-0.00320003360807863401760187116451561450958251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.435599974454939253831753376289270818233489990234375f,-0.92360001608133313055759572307579219341278076171875f,-0.60149997940659449824352122959680855274200439453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4825000096857561260321745066903531551361083984375f,0.682499992847442005228231209912337362766265869140625f,-0.686699985730647721737796018715016543865203857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4893999489903457789097274144296534359455108642578125f,-0.414200024268776445524053997360169887542724609375f,2.3309000077083705804170676856301724910736083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.44209994441792321850925873150117695331573486328125f,-1.060499990105629564141054288484156131744384765625f,0.10400003204494634445609335671178996562957763671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.963099984773994943765274001634679734706878662109375f,0.51520001660585368341571665951050817966461181640625f,-1.2703999683916560314855814795009791851043701171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.15340004984140254151725457631982862949371337890625f,-0.2469000170141468775142357117147184908390045166015625f,0.5467000219404709593362667874316684901714324951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0215999944895504114583673072047531604766845703125f,0.2931000030279165002866648137569427490234375f,-0.78650000572577105639737737874384038150310516357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.84509998282194143115475526428781449794769287109375f,0.7755999341845509764681310116429813206195831298828125f,-0.6104000466227541554786739652627147734165191650390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.480200045216082838095417173462919890880584716796875f,0.895800049212575277834957887534983456134796142578125f,0.0607999995172026075351823237724602222442626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.09189999456927100407455100139486603438854217529296875f,1.13080003785043903263840547879226505756378173828125f,2.7369999501034616429251400404609739780426025390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.706100039178132465877979484503157436847686767578125f,0.5667999752208585650947725298465229570865631103515625f,-2.93330010039806410304663586430251598358154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3102999433994302336259352159686386585235595703125f,0.746599981749057395319368879427202045917510986328125f,1.63150002217292833250894545926712453365325927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.681600023658573750395817114622332155704498291015625f,-0.6944999388158319408859142640721984207630157470703125f,2.032400010267644052674995691631920635700225830078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.31069995630830471444738805075758136808872222900390625f,-0.0852999954789870340476909404969774186611175537109375f,-1.37200001049041730993849341757595539093017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.525000040605664963777599041350185871124267578125f,-0.7516000230252737335234769489034079015254974365234375f,-0.72479995932877105957459207274951040744781494140625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4650999589055777594381879680440761148929595947265625f,-0.5740999932602040001228260734933428466320037841796875f,-0.42430007487237464403051490080542862415313720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0012999753922227075264572704327292740345001220703125f,0.138300021682679297185813993564806878566741943359375f,-0.365599938729406381821007698817993514239788055419921875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.72550004492327690996944511425681412220001220703125f,-1.0873999669611447149719651861232705414295196533203125f,-0.53560003074407713796745156287215650081634521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.32799995717406194017939924378879368305206298828125f,1.2957000351727006037805267624207772314548492431640625f,-2.193300021347404094029798216070048511028289794921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4721000195011495481622887382400222122669219970703125f,-1.3295000578016049619378691204474307596683502197265625f,-0.016100055532155987947362518752925097942352294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.559900008833409401631797663867473602294921875f,0.5852000076651571180974542585317976772785186767578125f,1.6546999834731226375339474543579854071140289306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1373999324277053180054508629837073385715484619140625f,-0.83289993255436467478602935443632304668426513671875f,0.49429997585415907934702772763557732105255126953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0404000178262593490075005320250056684017181396484375f,-0.426999981973321229844486879301257431507110595703125f,-1.187499982491135863682529816287569701671600341796875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5817998911038042564314309856854379177093505859375f,-0.848000082418321365906876962981186807155609130859375f,0.0005000137239685731316285455250181257724761962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0399000771924846731053548865020275115966796875f,2.92090001992732251068218829459510743618011474609375f,0.386400028193742084425821303739212453365325927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.453700077208875285350586636923253536224365234375f,-0.21230000978782903597874565093661658465862274169921875f,-0.85420001102536990966740404473966918885707855224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2269999863877905088571651504025794565677642822265625f,-1.4243000635102396334019658752367831766605377197265625f,0.8185999895945184068324351756018586456775665283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.421600052510946843309369569396949373185634613037109375f,1.1298999925166366420370422929408960044384002685546875f,0.909600102388861131430530804209411144256591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.08299993154406681838963777408935129642486572265625f,-1.83930003605485037354583255364559590816497802734375f,-1.34479999144077400075047989957965910434722900390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.822100016036630076854407889186404645442962646484375f,-0.58210001380145559579659675364382565021514892578125f,-0.91509995793700227295630611479282379150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,5.30599982538819414656927619944326579570770263671875f,-0.49820003230422837958712989347986876964569091796875f,-2.26989998163878947678995245951227843761444091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.992999995358287979030365022481419146060943603515625f,-1.46460003924518833429146980051882565021514892578125f,1.3591000353559856694829477419261820614337921142578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2563000626012676974596615764312446117401123046875f,0.297099902527035109045527860871516168117523193359375f,0.29990005700141164624028533580712974071502685546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3406999874144795992236822712584398686885833740234375f,2.701799980585276816924533704877831041812896728515625f,1.522199943128228039057603382389061152935028076171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.69720003663301444163380438112653791904449462890625f,0.08229999084472705561665861750952899456024169921875f,0.2431000528723001874453757409355603158473968505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.631600013227760381795405919547192752361297607421875f,-2.45719999883994422162913906504400074481964111328125f,-0.846900063766538924170390600920654833316802978515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6402999545752992016645066541968844830989837646484375f,-0.228800003704427989958958278293721377849578857421875f,0.17639997259378414895536479889415204524993896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1654000424504278310422478170949034392833709716796875f,1.6919000504299994691592701201443560421466827392578125f,3.549599975281954034045384105411358177661895751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.39690015909671882976539336596033535897731781005859375f,-1.16300002313032901923861572868190705776214599609375f,1.843699968504903807797745685093104839324951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.896300040696562660968993441201746463775634765625f,-2.173400044924020502179473623982630670070648193359375f,-0.0475999141275877946100081317126750946044921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2132999530255796916122790207737125456333160400390625f,4.04089999070242100742689217440783977508544921875f,-0.343999981008468846965797638404183089733123779296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.147100043641030797658686424256302416324615478515625f,-1.2237999384373434708095373935066163539886474609375f,0.711100033888221627620396247948519885540008544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.834100037628411339341027996852062642574310302734375f,-1.3552999799817817550007248428300954401493072509765625f,-0.13969996533095940804969359305687248706817626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3263999969013025026498553415876813232898712158203125f,0.32870005038678673514596084714867174625396728515625f,0.37170002797543855166395587730221450328826904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.26699985097348832852048872155137360095977783203125f,0.7377999647080886180816605701693333685398101806640625f,-0.1162999545305971604847172784502618014812469482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.323900004370510696816154450061731040477752685546875f,-3.0842999805107726984942928538657724857330322265625f,0.6601000771090375085492496509687043726444244384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0245000638738275267058952522347681224346160888671875f,0.15940005281418478944033267907798290252685546875f,1.91630000992566262851823921664617955684661865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3011000197693698421375074758543632924556732177734375f,-2.111499998182058579487829774734564125537872314453125f,0.851299995471537673807915780344046652317047119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.69259998483359908050260855816304683685302734375f,0.343799973455071405936678274883888661861419677734375f,-1.73770004134178090993145815446041524410247802734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.13369995293319192342096357606351375579833984375f,-3.241000062316656826766347876400686800479888916015625f,0.51319998108148734417000014218501746654510498046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.035000006034971331558836027397774159908294677734375f,-0.9535000064373004846629555686376988887786865234375f,2.29459993940740947238055014167912304401397705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.56120001570433419857408807729370892047882080078125f,-0.78119993240684382218574910439201630651950836181640625f,-0.37619991387352247347308775715646333992481231689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.794900128601492728108723895275034010410308837890625f,1.88999975152313570703199729905463755130767822265625f,-3.095399936093390369507005743798799812793731689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.08990006773024816766337607987225055694580078125f,-0.0873999571636312477806995957507751882076263427734375f,1.41149994908273157534495112486183643341064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2442999962314955642028735383064486086368560791015625f,-1.0169999541640286633281675676698796451091766357421875f,1.187800058250129797698946276796050369739532470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.167300011333823217540839323191903531551361083984375f,0.5782000391960142593461569049395620822906494140625f,1.05580003893077201126970976474694907665252685546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.501200052025914555287045004661194980144500732421875f,-0.3682999601036309744728214354836381971836090087890625f,1.4695000502020132415736952680163085460662841796875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.192600048685072966492271007155068218708038330078125f,-1.31349989629909469357471607509069144725799560546875f,-0.9069000559061770605495667041395790874958038330078125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9411000868618482062544217114918865263462066650390625f,-1.0415999954953800710200084722600877285003662109375f,-0.03480000496357771311295437044464051723480224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9669999692887041309319329229765571653842926025390625f,2.96710006100088374836332150152884423732757568359375f,-1.11270008135139875093955197371542453765869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5724001166619345770669724515755660831928253173828125f,-1.548099932899324127077989032841287553310394287109375f,-1.0610000439509743586086187860928475856781005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.00129994316846282487176722497679293155670166015625f,1.9343999721430247262787815998308360576629638671875f,1.812299949866531534325986285693943500518798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3045000203251841952578615746460855007171630859375f,0.67589999172687509343404599349014461040496826171875f,0.737799969774483255235963952145539224147796630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.644199937003851363925832629320211708545684814453125f,0.073800057348609282570350842433981597423553466796875f,-1.307400001233815611811905910144560039043426513671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.13670004101395694107168310438282787799835205078125f,0.827200008879600989786240461398847401142120361328125f,0.193699954348802716452837557881139218807220458984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.014199938009681556394525614450685679912567138671875f,0.4530999878659844881667595473118126392364501953125f,0.047999985523523580610572025761939585208892822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4676999592736377220347776528797112405300140380859375f,4.32910004928857095052308068261481821537017822265625f,-0.98489999527335125861782216816209256649017333984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2102999276041976894902063577319495379924774169921875f,0.95910006553083804181625282581080682575702667236328125f,0.32839996229708301456184926792047917842864990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.3967999422147858723519675550051033496856689453125f,-0.39030011174529699591317921658628620207309722900390625f,0.40140000828206578997736642122617922723293304443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.53459997643679380008308044125442393124103546142578125f,0.671000027000903997986824833787977695465087890625f,1.16659993183016741369328883592970669269561767578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.092900010599196114480236019517178647220134735107421875f,-0.89250000808387974782220908309682272374629974365234375f,0.082999986529350255892722998396493494510650634765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1791000018656256909110879860236309468746185302734375f,1.3614000444546350276908697196631692349910736083984375f,0.949899990430474350233680524979718029499053955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9527999828875064469713151993346400558948516845703125f,0.4759999872893094785553103065467439591884613037109375f,-0.23119994164556267879362394523923285305500030517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6468000071465969114115068805404007434844970703125f,-0.1751000001072886558262098333216272294521331787109375f,-2.029400055235624389382564913830719888210296630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.134099966587126306905020101112313568592071533203125f,0.0467999706946311544442096419516019523143768310546875f,0.0479999955818060097811894593178294599056243896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1095000278130175974666826732573099434375762939453125f,-0.67500007145106810657608775727567262947559356689453125f,2.4754000243455163143835306982509791851043701171875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.136700039374828552496410338790155947208404541015625f,0.071200036752224082903239832376129925251007080078125f,-0.626100000137090884066992657608352601528167724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2806999941945076404437031669658608734607696533203125f,0.37940000960081821101965715570258907973766326904296875f,0.06419999994263048126441617569071240723133087158203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8176999841213221653646314734942279756069183349609375f,1.89120004487335702236805445863865315914154052734375f,-0.745300027292966404957041959278285503387451171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.57349997425079213275722622711327858269214630126953125f,-0.05920001143515130070937857453827746212482452392578125f,-0.38890000608414343474805718869902193546295166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.99460008003711752877507024095393717288970947265625f,0.164300017391144859146834278362803161144256591796875f,0.82739999385774110063351827193400822579860687255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.8676000152692198152948321876465342938899993896484375f,-1.7715000586807738702788128648535348474979400634765625f,-1.156899955956637082721272236085496842861175537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.142300050300360414468059389037080109119415283203125f,2.2229999832510944912655759253539144992828369140625f,1.334400040390342656593247738783247768878936767578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.59670001270175010432694762130267918109893798828125f,0.590199994552135454028984895558096468448638916015625f,-0.2164999912157654637923087648232467472553253173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1026000266127286841832955133213545195758342742919921875f,-0.0585999951824545117773368474445305764675140380859375f,-0.396600005143880718350146707962267100811004638671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4169000279665004971008102074847556650638580322265625f,-0.45720001491457207265511897276155650615692138671875f,0.2053999802380801487089456713874824345111846923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0516000060379495639750757618458010256290435791015625f,-1.4068000367440287590170555631630122661590576171875f,1.0600999966427675502700367360375821590423583984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.50120003317594419289804363870644010603427886962890625f,-0.2433999446392050014509322863887064158916473388671875f,0.03840004829540799580200882701319642364978790283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.80500005051493506158521995530463755130767822265625f,0.75689998739808805883200193420634604990482330322265625f,-0.56069999609440657195591484196484088897705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5904999639913446340955260893679223954677581787109375f,0.3405000097751618159946929154102690517902374267578125f,0.524399998313188842047338766860775649547576904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.027300002393126465705108785186894237995147705078125f,0.0708999751865857508192902969312854111194610595703125f,-0.44699999301880566360267721393029205501079559326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.820200005765258310930221341550350189208984375f,0.0260999531239265447624120497494004666805267333984375f,0.15700000736117392108326384914107620716094970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9038999927476047968610828320379368960857391357421875f,0.178600001962482879402926982947974465787410736083984375f,-0.0018999985739587277322470981744118034839630126953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.64370000644698688319067514385096728801727294921875f,0.83809995217770405506740871715010143816471099853515625f,1.372299946159869055151148131699301302433013916015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.624100029674918399535954449675045907497406005859375f,0.0259000147119170798237774988592718727886676788330078125f,-1.927500003557652608066064203740097582340240478515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2747999890193337790833538747392594814300537109375f,0.228900010964273814551006580586545169353485107421875f,1.29709997610151805957912074518389999866485595703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.31439997715353917850933385125244967639446258544921875f,-0.0723000005677338908327556055155582726001739501953125f,0.89240004049837640121012327654170803725719451904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.68699996235966664936967163157532922923564910888671875f,-0.287799985793232959707665941095910966396331787109375f,-0.6791000117748975384301957092247903347015380859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.050899938566982161347596047562547028064727783203125f,1.2961999755084525620674185120151378214359283447265625f,-0.241399990475178238824582876986823976039886474609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.06520005311370002942794599221087992191314697265625f,-0.1301000470459465763184425668441690504550933837890625f,-2.391899984976650017642896273173391819000244140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.69159999244064185308644709948566742241382598876953125f,1.817699988889693418769866184447892010211944580078125f,0.053699971038103477827263532162760384380817413330078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8548999358363433831442534938105382025241851806640625f,-3.54409997541606447413187197525985538959503173828125f,-1.052399944441766432845497547532431781291961669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.75679997678548005524135078303515911102294921875f,-1.304899964986741434103123538079671561717987060546875f,-3.605200019846857006911022835993207991123199462890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4311998816311366500286794689600355923175811767578125f,-0.9585000367611649085830549665843136608600616455078125f,-0.3100999837532636416170817028614692389965057373046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.168400039932131928566150236292742192745208740234375f,0.5655000307112931778874553856439888477325439453125f,-0.5386999991454188840833694484899751842021942138671875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.453999981343745862005789604154415428638458251953125f,2.730299956996738774961386297945864498615264892578125f,-2.2392999548621475014442694373428821563720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3100000052154061602749379744636826217174530029296875f,-1.8663000140793624126445138244889676570892333984375f,3.73789995162784993709692571428604423999786376953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3582000024273994664980591551284305751323699951171875f,0.3016999449014665657387013197876513004302978515625f,0.1811000388428560814446655058418400585651397705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.39889999412596244265927225569612346589565277099609375f,-2.7708998618818814207998002530075609683990478515625f,1.25960002363622347587579497485421597957611083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.842300069820879571125260554254055023193359375f,-1.3970998932510620971214621022227220237255096435546875f,-2.141499997474252126039573340676724910736083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.115000039488076577498532060417346656322479248046875f,0.8349999488517647616703243329538963735103607177734375f,-0.177200029666721992072098146309144794940948486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.007799990114569776977759829605929553508758544921875f,0.269099997805059165667529441634542308747768402099609375f,0.0283000200808043445022121886722743511199951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.203000070907175089729435057961381971836090087890625f,-0.074900009243191345564838456994039006531238555908203125f,0.52189998143762383886468114724266342818737030029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.514000078462065079776266429689712822437286376953125f,0.73080000134259581745510558903333730995655059814453125f,2.046000038847327839874878918635658919811248779296875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.778999989129601555504223142634145915508270263671875f,-1.7383999735847106560271413400187157094478607177734375f,0.6092999961197376190824570585391484200954437255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.816499966070057592304465288179926574230194091796875f,1.5112999850779775812270599999465048313140869140625f,-2.985099991464614799241417131270281970500946044921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.350299949471652549704003831720910966396331787109375f,-0.1405999946832661162687827527406625449657440185546875f,1.701600018741192155147246012347750365734100341796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.351999962434172086744865737273357808589935302734375f,-2.77419991971850254941500679706223309040069580078125f,-0.86609995260238814562825382381561212241649627685546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.949099930340052022614827365032397210597991943359375f,-2.298700009483099382379123198916204273700714111328125f,1.76240003081262219808422742062248289585113525390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7828000284105540718115889831096865236759185791015625f,-1.606400061237067244945819766144268214702606201171875f,1.4882999842062580864165965977008454501628875732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7242999945178627552166972236591391265392303466796875f,-0.30619998959004846028619795106351375579833984375f,-0.6839000475466241457667138092801906168460845947265625f,0.0f,0.0f,0.0f,0.0f,0.0f,3.3769000778853897060116651118732988834381103515625f,0.4990999452225854948750338735408149659633636474609375f,2.012100002886354932485346580506302416324615478515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.447800034017117543072572516393847763538360595703125f,-2.172699965085089335303791813203133642673492431640625f,-1.66240002440512046888443364878185093402862548828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8823999378666289583605930602061562240123748779296875f,-1.072499999664724423809047948452644050121307373046875f,1.261500013940036524218157865107059478759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8281000260502098253567737629055045545101165771484375f,2.14560000785216775653907461673952639102935791015625f,0.3227999717488876374460460283444263041019439697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.355800011903793400591666795662604272365570068359375f,-0.8755000354982909183121364549151621758937835693359375f,0.747399976497888030024796535144560039043426513671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2178000263243908829480233180220238864421844482421875f,-0.76189997501149686609522859725984744727611541748046875f,0.1100000026822083309951949559035710990428924560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.614500017605722259617095915018580853939056396484375f,-1.582499946914613264681293003377504646778106689453125f,-0.03609998262822655679116223836899735033512115478515625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.700500064052640869505239606951363384723663330078125f,-0.35350009576976215708299378093215636909008026123046875f,1.1453999883219603272976883090450428426265716552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.14230000365972605980147136506275273859500885009765625f,-2.29889999416321533232121510081924498081207275390625f,-2.17710001782476769704999242094345390796661376953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.36830003682598455583274699165485799312591552734375f,1.8394000695407377587997643786366097629070281982421875f,-1.5995000140294433066401325049810111522674560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.849600020581484205450806257431395351886749267578125f,3.649000007346272500541317640454508364200592041015625f,2.215400000298768379281000306946225464344024658203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.62040004252493385905609102337621152400970458984375f,0.37209999025762130475669664519955404102802276611328125f,-1.642100012460350200171887991018593311309814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2761000211477269150606161929317750036716461181640625f,-2.31460006867498169214059089426882565021514892578125f,-0.881899987621604708465383737348020076751708984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5509999700039618364399984784540720283985137939453125f,-0.5638999755740166097695009739254601299762725830078125f,-0.307199988241494192919844863354228436946868896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3952000610113139966728112995042465627193450927734375f,1.863099956424535630361560833989642560482025146484375f,0.456400020147115892399369840859435498714447021484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7869000538945190026396403482067398726940155029296875f,1.68159996166974234910185259650461375713348388671875f,-2.060499878831209219498532547731883823871612548828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.617800015707314020829699074965901672840118408203125f,-0.94699992585182091175965979346074163913726806640625f,-0.294500019989907713124921428970992565155029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.07469990404993342991701865685172379016876220703125f,0.3689999845847486970029649455682374536991119384765625f,0.4410000828430040709093873374513350427150726318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.792700020107627523202609154395759105682373046875f,0.322600021862984309706234853365458548069000244140625f,1.0280000008344651885039411354227922856807708740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.403899982614813968240241592866368591785430908203125f,0.54459995296746466220128013446810655295848846435546875f,1.026900013251602405972562337410636246204376220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.72149998894333666754619116545654833316802978515625f,-0.17159997437298246669712398215779103338718414306640625f,-0.215800030921398988681403352529741823673248291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.32929992533922358699527421777020208537578582763671875f,0.731099924439191806158078179578296840190887451171875f,-0.8746000528797512885148535133339464664459228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1518000213772070505768851944594644010066986083984375f,-1.7204999474138020776337043571402318775653839111328125f,1.11009997213035749297205256880261003971099853515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3141000057213003682221597046009264886379241943359375f,-0.12119996170625013665045344168902374804019927978515625f,-3.600799967554210834208561209379695355892181396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.04310007898658430036675781593658030033111572265625f,2.076699998806417202246166198165155947208404541015625f,1.95740008345097304953696948359720408916473388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.745599995353817934784501630929298698902130126953125f,-0.54459997356831990611425453607807867228984832763671875f,0.515999995857476800864560573245398700237274169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.47439997871816164121128167607821524143218994140625f,-1.78520003683418071460664577898569405078887939453125f,1.19660000391453547052833528141491115093231201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.917499985657631356161800795234739780426025390625f,1.0229000123500835428558275452814996242523193359375f,1.0426999803215257767163848257041536271572113037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3512999574363229182694112751050852239131927490234375f,-0.2672000738114110873766549048013985157012939453125f,-0.045500004224480417036602375446818768978118896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.404299994573741994230431373580358922481536865234375f,0.09430006228089415909465742515749298036098480224609375f,-0.5713001127064225581619894001050852239131927490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.71990001859366881120649850345216691493988037109375f,0.90829997560083974850186905314330942928791046142578125f,-0.056299920369685452214980614371597766876220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.18709997926801502643456842633895576000213623046875f,2.20639995042830694416124970302917063236236572265625f,1.270100007539241016019104790757410228252410888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.88689999078810177746134968401747755706310272216796875f,1.187800059404969577059318908140994608402252197265625f,0.6307000783115623843144703641883097589015960693359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.30900003916025209349527358426712453365325927734375f,0.508800017786026348431960286688990890979766845703125f,0.4769000013865529741252657913719303905963897705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.00699998739361706867612156202085316181182861328125f,-0.660700007307528647260141951846890151500701904296875f,0.9710000150799746876373319537378847599029541015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.290800043717771183882092600470059551298618316650390625f,1.56600002983212416296510127722285687923431396484375f,-0.493300035056472463423915542080067098140716552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.14439999715834961335758634959347546100616455078125f,-2.004299934987723741386389519902877509593963623046875f,1.171499976649880370160872189444489777088165283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.37900001390278259538746397083741612732410430908203125f,0.8716999608829605339366253247135318815708160400390625f,0.5476000296860947802457530997344292700290679931640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9258999972961847735319906860240735113620758056640625f,-0.34929998911619197787814528055605478584766387939453125f,-0.64440002155899966940211243127123452723026275634765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.23019999648928657354218785258126445114612579345703125f,-0.63710001223683365711991655189194716513156890869140625f,0.570099989266692919187562438310123980045318603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.11810002120733198438529143459163606166839599609375f,-0.64019999738335642280873116760631091892719268798828125f,0.557399974709749113088719241204671561717987060546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.12020000155568144606377245509065687656402587890625f,0.621600014829635227897597360424697399139404296875f,0.64100004160404200348466474679298698902130126953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.60679993703663359649880248980480246245861053466796875f,-1.1385999664977191070391882021795026957988739013671875f,-0.6576000340633088026010000248788855969905853271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.82970001681447114183498570127994753420352935791015625f,-1.2801999831527466344738286352367140352725982666015625f,-0.3822000137969843169827299789176322519779205322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.40689998810589422806316406422411091625690460205078125f,-1.7285000158622854105061605878290720283985137939453125f,0.4848000128984450984859222444356419146060943603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.08199999435991023322145565543905831873416900634765625f,-0.732300038155913934900809181272052228450775146484375f,0.2368000364832585358243477458017878234386444091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.49760002096891431389025228781974874436855316162109375f,-0.36250004138797653485681848906097002327442169189453125f,-1.1231000566348441349617814921657554805278778076171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.11430000550746921561540148104541003704071044921875f,-0.4200000101141629915701969366637058556079864501953125f,0.325200020983070159363848006250918842852115631103515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.962899946635961345720033932593651115894317626953125f,-0.04590001646280228442265070043504238128662109375f,-0.7230999899148937704040918106329627335071563720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.01799998589605100729471587328589521348476409912109375f,0.25950002220645618233874074576306156814098358154296875f,2.752499995306134650263629737310111522674560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.47570003640204594486107225748128257691860198974609375f,1.04130004960000377423057216219604015350341796875f,-0.2671999562226237134865414191153831779956817626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.17910004791021361114644605549983680248260498046875f,1.563699982213973438405218985280953347682952880859375f,0.756900022732466037922449686448089778423309326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6952000181704758308143254907918162643909454345703125f,0.27639995944350859158333832965581677854061126708984375f,-1.291300002288817250217789478483609855175018310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.224700065802038384532579584629274904727935791015625f,0.212500046454370927762056453502736985683441162109375f,0.250900011694431324116294490522705018520355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9366000143826003210278940969146788120269775390625f,-0.0202999885499484111761603344348259270191192626953125f,0.2449000043630604750433121807873249053955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.57499997407197867005379521287977695465087890625f,1.2114000015020369716012282879091799259185791015625f,0.418000001560896505825581925819278694689273834228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.628499973766506681016608126810751855373382568359375f,1.8395999733448034074712040819576941430568695068359375f,-0.6116000050693759515496594758587889373302459716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.355400066404044867596212498028762638568878173828125f,0.9754000201731936048332727295928634703159332275390625f,0.9081000248208628278234755271114408969879150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.43830005826503093402379818144254386425018310546875f,0.6458000138223163322237496686284430325031280517578125f,1.4152000222682954433395252635818906128406524658203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8110999661996964871235604732646606862545013427734375f,1.474800002132356180339911588816903531551361083984375f,0.39969999543130352304132202334585599601268768310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9788000214740630866572246304713189601898193359375f,-1.1695000542625793027440295190899632871150970458984375f,0.46560001904666403760302273440174758434295654296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.659699998709559309872929588891565799713134765625f,0.2634999444484709130165356327779591083526611328125f,2.00170003482997405086507569649256765842437744140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5106999775797127494314509021933190524578094482421875f,0.32379999550879023928473543492145836353302001953125f,1.376400009883939734578461866476573050022125244140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2814000054694709707092670214478857815265655517578125f,0.159999960660934303913194298729649744927883148193359375f,1.33600000334531099355217520496807992458343505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.703200014422835639749109759577549993991851806640625f,2.252000041708348998525934803183190524578094482421875f,-1.647399994751810314852491501369513571262359619140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2120999420896170217787357614724896848201751708984375f,-0.23199994809180568555717627532430924475193023681640625f,-2.806899971211702204954008266213349997997283935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.30870006577223563226652913726866245269775390625f,-1.3899999471381299098737827080185525119304656982421875f,-0.7381000356987115207374472447554580867290496826171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.087800030981003640562221335130743682384490966796875f,-0.063699942874908987988646913436241447925567626953125f,-0.492600060904025360830615909435437060892581939697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.266699999104440532704529687180183827877044677734375f,0.88039995699226825109917626832611858844757080078125f,0.15570002319589237860242292299517430365085601806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6606000025063754055310027979430742561817169189453125f,-3.086899998611211781707197587820701301097869873046875f,-1.67879998107328987089204019866883754730224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.23449996690451957448431130615063011646270751953125f,0.696199996612220761704747928888536989688873291015625f,-2.978199986147880284903521896922029554843902587890625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.84459993279501777152518116054125130176544189453125f,1.054500102095306868221769036608748137950897216796875f,-1.0399000328615304322710244377958588302135467529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.531200042749941925279699717066250741481781005859375f,0.649699922024458142999492338276468217372894287109375f,-0.2528000234000391532873663891223259270191192626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.212500089071692155329174056532792747020721435546875f,-0.7235000237971542791370893610292114317417144775390625f,-1.0997999858900919800674955695285461843013763427734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.16539996710643212196600870811380445957183837890625f,0.5166999961614611169835598047939129173755645751953125f,-1.0822000006467102917184774923953227698802947998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.68800003074854654272485277033410966396331787109375f,-2.49979998562931893246741310576908290386199951171875f,-1.0720000098571180746631625879672355949878692626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.64420005487203557681397114720311947166919708251953125f,0.91540003809183811966931898496113717555999755859375f,-1.4264999970644709481604195389081723988056182861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2095999689675875909955493625602684915065765380859375f,-0.68069997344464050481604999731644056737422943115234375f,1.477699947352707710024333209730684757232666015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7639999449849124157907453991356305778026580810546875f,1.1399999020993709653026826345012523233890533447265625f,-2.0949000229522578564456125604920089244842529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7163000173017388672036531716003082692623138427734375f,-1.4044999403432019136062081088311970233917236328125f,0.63790002393573519423597417699056677520275115966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.852000008367002958920011224108748137950897216796875f,0.72130001432150614437688318503205664455890655517578125f,-0.2258000788658856006207997779711149632930755615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6563999575808645570162980220629833638668060302734375f,1.630500066697598615661490839556790888309478759765625f,0.7663000275090336987915407007676549255847930908203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0965999665498724624512760783545672893524169921875f,-0.21909999061524920715982034380431286990642547607421875f,0.81229999155253285181288447347469627857208251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.71950001192465473653925300823175348341464996337890625f,-1.506600037181377249595470857457257807254791259765625f,-0.1197999551191930578397659701295197010040283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.170199960540234673800341624883003532886505126953125f,-1.377199985894561162780291851959191262722015380859375f,0.2848000092476612454817086472758091986179351806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.3187000820145009516437539787148125469684600830078125f,-1.7719999204874028464473667554557323455810546875f,2.626999963998795095676541677676141262054443359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.320900021808596835892757326291757635772228240966796875f,-1.2385999952569601845198121736757457256317138671875f,0.09189995353519930798569248509011231362819671630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.9736000315040342911743209697306156158447265625f,3.14139991876333990461489520384930074214935302734375f,1.91940002439022094904430559836328029632568359375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1236000061348090373058994373423047363758087158203125f,-0.13549995665252101417763697099871933460235595703125f,-0.13350004380196345277198588519240729510784149169921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.61630008327662988421025147545151412487030029296875f,0.7717999924130738431671261423616670072078704833984375f,1.3958999764531851983662136262864805757999420166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.5389000224567952468390785725205205380916595458984375f,-0.631000029757618374759431389975361526012420654296875f,-2.9170000179409978358080479665659368038177490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.82499996703118139151911236695013940334320068359375f,2.992899984131008661591977215721271932125091552734375f,-1.64629997822344353863854848896153271198272705078125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2828999611645952594329855855903588235378265380859375f,0.42030000926256116944301766125136055052280426025390625f,-0.105299961578846412724175252151326276361942291259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.07419999508112684605265485515701584517955780029296875f,-0.95759995551556376103263801269349642097949981689453125f,1.9492000675380223295718451481661759316921234130859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.15089999630898187632510598632507026195526123046875f,-0.7492999945178622223096454035839997231960296630859375f,3.722300020311772872361189001821912825107574462890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.02170004574507522487891719720209948718547821044921875f,-0.1349000266402964332002056835335679352283477783203125f,-0.42890005216598492410184917389415204524993896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.919500022746623013603084473288618028163909912109375f,1.849099969828127854043486877344548702239990234375f,0.69539991146922108811168072861619293689727783203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.92779999406337676504108458175323903560638427734375f,-1.016300055262446822013089331449009478092193603515625f,0.72249996662139981395966970012523233890533447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.18830003215074497813219522868166677653789520263671875f,-0.32699996703490474825315459384000860154628753662109375f,0.83549999926984319653655575166339986026287078857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.096999904058873820389408137998543679714202880859375f,4.44940009051412488361165742389857769012451171875f,3.1054999612346296800069467280991375446319580078125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.0078000000238436228983118780888617038726806640625f,-0.15199998102337108907278206970659084618091583251953125f,0.7571999561667446254631386182154528796672821044921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.54609992455095035523982005543075501918792724609375f,1.7128999147847299155955624883063137531280517578125f,-0.2399000687547021559709037319407798349857330322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.41070003368258500842102876049466431140899658203125f,-0.083100016289949341086185086169280111789703369140625f,0.020300058771670403956477457541041076183319091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1079999626874925677810779234278015792369842529296875f,0.758599999354780063498537856503389775753021240234375f,1.459700054514408318340201731189154088497161865234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4794000229746107066830518306232988834381103515625f,0.46109997342675956844004758750088512897491455078125f,0.319300000590085897300696160527877509593963623046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.286500009637326413791669210695545189082622528076171875f,3.061399934633076469481238746084272861480712890625f,-2.08979995510056593133185742772184312343597412109375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.49679990518540062538477286580018699169158935546875f,-2.05909994386285699619065781007520854473114013671875f,-0.582900091885032001215449781739152967929840087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9278999808713788155500878929160535335540771484375f,0.76630005582123905671920738313929177820682525634765625f,-1.0624999618902801756803455646149814128875732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.65609991063699146796039940454647876322269439697265625f,0.67459998059049286478483509199577383697032928466796875f,-1.991499950628728310420001434977166354656219482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.595600043633580877866506853024475276470184326171875f,-1.8885000445842738958646123137441463768482208251953125f,0.333700035634636460457613793550990521907806396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.139599982881545958690594488871283829212188720703125f,-1.873699985045195415267471616971306502819061279296875f,0.3615000077188013083429041216732002794742584228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4354999807178980830713044269941747188568115234375f,-0.7564000302612778892097367133828811347484588623046875f,-0.2333000026464462361985852112411521375179290771484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1741000269554555668349848929210565984249114990234375f,-2.39179998088329881511526764370501041412353515625f,0.5503000506505362210418752511031925678253173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.336700024231522565543173186597414314746856689453125f,2.302900085142255637293828840483911335468292236328125f,-2.220200011427702069255474270903505384922027587890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.94760000722259307703865260918973945081233978271484375f,1.5395999039798964336256403839797712862491607666015625f,-2.25810002057403291786386034800671041011810302734375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.9306000031210481804322398602380417287349700927734375f,-1.7138000815816212263342777077923528850078582763671875f,0.3881999946042908877785748700262047350406646728515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.70059995782151796017700462471111677587032318115234375f,-2.050700011144578116528691680287010967731475830078125f,-0.471300012235343235289519725483842194080352783203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.958500015377998781929136384860612452030181884765625f,0.096000009715556888778564825770445168018341064453125f,-0.0979000142686070606856674203299917280673980712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.672600032293795724314122708165086805820465087890625f,-1.4231999740779397445322729254257865250110626220703125f,1.18190000859498933749591742525808513164520263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.726899998871981978254552814178168773651123046875f,-0.269800016772746342752498094341717660427093505859375f,-1.184200037363171365001335288980044424533843994140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.869100026378035739327287956257350742816925048828125f,0.61410001445710715728409923030994832515716552734375f,-1.55900003686919763623563994769938290119171142578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.11399998797476251777283096089377067983150482177734375f,-0.5125999612525105053606466753990389406681060791015625f,-2.97000003658235112169450076180510222911834716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.476599954312294737945876477169804275035858154296875f,1.89819994636178090985367816756479442119598388671875f,0.016699996012448981019815619220025837421417236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.63350002051889919840732545708306133747100830078125f,1.130200018058717059687978689908050000667572021484375f,-0.3083999655380844640006898771389387547969818115234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6787000026285650022828122018836438655853271484375f,-0.275599985891580245844778573882649652659893035888671875f,-0.91739998856782900560347115970216691493988037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.456200003820657951081329883891157805919647216796875f,0.15069997497200926517280095140449702739715576171875f,1.965400008177757396055085337138734757900238037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.725999981626868073902869582525454461574554443359375f,0.6612999380648136860116892421501688659191131591796875f,-1.546699954084307382373708605882711708545684814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.825900099089742045777029488817788660526275634765625f,1.60230001297295121531760742072947323322296142578125f,-1.253399985542893890766436015837825834751129150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2878999674975879496940933677251450717449188232421875f,1.79229999002516304784649037173949182033538818359375f,-0.29709996351003653547451222038944251835346221923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.014400019696354515730263301520608365535736083984375f,1.1591000252604484188623246154747903347015380859375f,0.939399990322440370249523766688071191310882568359375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3397000405818226820287009104504249989986419677734375f,-0.105899963898956617214253128622658550739288330078125f,1.70410005372166661885557914501987397670745849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.442099999570846335927853942848742008209228515625f,0.24219999232068634142933660768903791904449462890625f,0.79879999407455304361036496629822067916393280029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.37340000364780490116345390561036765575408935546875f,0.6208000391542913831699479487724602222442626953125f,-0.59199999305605899291293781061540357768535614013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.898000035516918426736765468376688659191131591796875f,1.44689994090646489866003321367315948009490966796875f,0.0175000233948221506352638243697583675384521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.260199972945452717709713397198356688022613525390625f,1.599399938056617731518826985848136246204376220703125f,-0.817099978429824513881385428248904645442962646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.7431000308930872932933198171667754650115966796875f,0.420200027781724383402206512982957065105438232421875f,1.49009998889416461764767518616281449794769287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.8605999882385135801854403325705789029598236083984375f,1.7767000168740754162399753113277256488800048828125f,1.1001999699652194042442943100468255579471588134765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.007000021368265496590765906148590147495269775390625f,0.724599984185397172353759742691181600093841552734375f,0.383999991029500886696723682689480483531951904296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5558000007092953609344476717524230480194091796875f,-0.26330004392266281865175869825179688632488250732421875f,0.11159996908307068252952376496978104114532470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1730999595165254589801406837068498134613037109375f,1.31230000485181808045354046043939888477325439453125f,0.380100000368058577038254952640272676944732666015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.93649997482448821273237626883201301097869873046875f,0.61930001146793378641319804955855943262577056884765625f,-0.3788000037044281231857212333125062286853790283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.85880004084557270971345133148133754730224609375f,0.54720007559955075038260474684648215770721435546875f,1.716700000035763284955692142830230295658111572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.857300003659725806670621750527061522006988525390625f,-0.1928999815046792409845011206925846636295318603515625f,-0.08429999259933840249914283049292862415313720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.58670002313256353065895609688595868647098541259765625f,-1.584699997815489869168459335924126207828521728515625f,-0.577399982272088774237772668129764497280120849609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.463599970409273698379593042773194611072540283203125f,0.8005000263899564316716350731439888477325439453125f,0.2324000114411111095336082144058309495449066162109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.150300021854043119873267642105929553508758544921875f,-0.69149997687339759266933469916693866252899169921875f,0.4446000028491017275200647418387234210968017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.694399989372491699413103560800664126873016357421875f,1.9450000095367432351167735760100185871124267578125f,-0.15439997368901980934197126771323382854461669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.21769994794875369148456911716493777930736541748046875f,-0.2550000090897082127838757514837197959423065185546875f,-1.5038999916300179737760345233255065977573394775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.638199949826300017008406939567066729068756103515625f,0.52270008064359441135593442595563828945159912109375f,2.602399988158046628683450762764550745487213134765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.35890000049620784938753104142961092293262481689453125f,-0.36249996583908894454140181551338173449039459228515625f,1.70149996424466376510054033133201301097869873046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.41680006380826295497854516725055873394012451171875f,0.0450999891921863849120200029574334621429443359375f,0.15919999139308960156569128230330534279346466064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.89590003050714639432072772251558490097522735595703125f,-0.799699995394051565966719863354228436946868896484375f,0.6125999778300528220142950885929167270660400390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.38550001402199285394090111367404460906982421875f,-0.5588000068336722581108233498525805771350860595703125f,1.4630999990791082598917682844330556690692901611328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0979000163733958483902597436099313199520111083984375f,0.5523000368893151712512690210132859647274017333984375f,-0.01849997724965224410453856762615032494068145751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.291600010918080965183207808877341449260711669921875f,0.864200017898528738413688188302330672740936279296875f,1.97180006064176627234019179013557732105255126953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.344299959015847090171291711158119142055511474609375f,0.3468000790074474704027807092643342912197113037109375f,1.57019996283128815406371359131298959255218505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.87819996988698945106222026879549957811832427978515625f,0.9562999881327154394483613941702060401439666748046875f,-0.0587999852456146943069370536250062286853790283203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.926799981963635222115271972143091261386871337890625f,0.67700007450953070264887401208397932350635528564453125f,3.91430001418739603735730270273052155971527099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5049999852292235136275166951236315071582794189453125f,0.3782999863669276319200207581161521375179290771484375f,-1.646799998429417488665649216272868216037750244140625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.07430003400593943041485545109026134014129638671875f,0.28280002803802517519216053187847137451171875f,0.2999999908730384134969426668249070644378662109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.262399977391959027261947312581469304859638214111328125f,-0.9001999645635485602923608894343487918376922607421875f,1.09520000483393697976453040610067546367645263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.589199996738879239188690917217172682285308837890625f,1.5287999703258268180405821112799458205699920654296875f,0.037800060336291796403429543715901672840118408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.062500056289136463050226666382513940334320068359375f,1.62560000626891820729724713601171970367431640625f,4.1558999883741147840510166133753955364227294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.8738999549359096619127740268595516681671142578125f,0.443300002124906111777846717814099974930286407470703125f,-0.8882999911539253279357808423810638487339019775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4364999854043121185753761892556212842464447021484375f,0.40190001615732862116914247962995432317256927490234375f,-0.49989992708191366421743850878556258976459503173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3403000135093929845453430971247144043445587158203125f,-0.83010005818456422677087402917095459997653961181640625f,0.6806000405415895126992609220906160771846771240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.591199943321942811280678142793476581573486328125f,0.1938000194996594149898783143726177513599395751953125f,0.02940003754049536066617065444006584584712982177734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6994000526651744653605646817595697939395904541015625f,1.0497000662863253950263242586515843868255615234375f,1.0570000028908257405646509141661226749420166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.529300004836916304640226371702738106250762939453125f,0.7320999992728236538397368349251337349414825439453125f,-1.94610001399517162923302748822607100009918212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.8350999865472321648240949798491783440113067626953125f,-1.110399991674719810674787368043325841426849365234375f,-4.844599932254851637480896897614002227783203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.0806999949954434470100750331766903400421142578125f,0.42270007207542692473367651473381556570529937744140625f,-0.52380000362992351625734954723156988620758056640625f,0.0f,0.0f,0.0f,0.0f,0.0f,3.915200035083292728899095891392789781093597412109375f,0.26349997872114183650182894780300557613372802734375f,-2.89690000226199639854485212708823382854461669921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.6298000272408119659672820489504374563694000244140625f,-0.38250004030764095386274448173935525119304656982421875f,0.9660999509632588466701008655945770442485809326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.07470001234412204649260047517600469291210174560546875f,0.744000005222857030418026624829508364200592041015625f,1.2327000113159414240726619027554988861083984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.301799947113543520771372641320340335369110107421875f,-0.4795999617218968946730228708474896848201751708984375f,1.270600029402971120617849010159261524677276611328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.885499947451055202662928422796539962291717529296875f,0.16560003461837846661097728429012931883335113525390625f,-1.023300000038744439478932690690271556377410888671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.4829999824315314160827483647153712809085845947265625f,3.427499940898269148448207488399930298328399658203125f,0.8964999881610273835264024455682374536991119384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.619699987440557098494764431961812078952789306640625f,-0.12510005572587135080908637974062003195285797119140625f,0.052499970123172534641753372852690517902374267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.99799997255951122809847220196388661861419677734375f,-0.3590001084506517781136381017859093844890594482421875f,-2.32520001437068035698985113413073122501373291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.074499997861684319477859617109061218798160552978515625f,-0.67889998254030903268585461773909628391265869140625f,1.984299951006472273462577504687942564487457275390625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.53069998968690601515163507428951561450958251953125f,0.41359997361302358598322825855575501918792724609375f,0.81059997262954741614748854772187769412994384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.597399987822770928147519953199662268161773681640625f,-0.67699999409913991232912167106405831873416900634765625f,0.41970002858638810749880576622672379016876220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.018499979354441364876038278453052043914794921875f,-0.5335000016130513333934004549519158899784088134765625f,-0.3206000104412443096890683591482229530811309814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.454799961563944865616804236196912825107574462890625f,1.53989989465325916029314612387679517269134521484375f,-3.401599852406978374830259781447239220142364501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.21789993557184939021453828900121152400970458984375f,-0.64510002145320155531038608387461863458156585693359375f,-1.019600030304490534405204016366042196750640869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.20030000123456004335054103648872114717960357666015625f,1.687900072159618236611322572571225464344024658203125f,1.563399994427710826272459598840214312076568603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1986000864893198158966924893320538103580474853515625f,0.66170000633150427571393947800970636308193206787109375f,-0.29710005023479535424257846898399293422698974609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.96190000665783925715146551738143898546695709228515625f,0.935600010888278266207862543524242937564849853515625f,1.2882000301063059044537340014358051121234893798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.404999990016222444211280162562616169452667236328125f,-0.3496999823927879891272141321678645908832550048828125f,-2.7824999658763402976546785794198513031005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.10769999248459882768003126329858787357807159423828125f,-1.403700041818618071687296833260916173458099365234375f,1.0280000305250289738268065775628201663494110107421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5188999854832889724320921231992542743682861328125f,1.08359993412494670650403350009582936763763427734375f,1.3249000345379136067691661082790233194828033447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.00260005777478244937128692981787025928497314453125f,-2.123000022552906873585243374691344797611236572265625f,-1.01649996290355915107284090481698513031005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2766999524079258865327801686362363398075103759765625f,-1.309999972097575238194622215814888477325439453125f,0.112899930002540449525127996821538545191287994384765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1697000338017942144830385586828924715518951416015625f,-0.54149995236098791284717890448519028723239898681640625f,-0.92289997748136498589843768058926798403263092041015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2331000218033789650462495046667754650115966796875f,2.116499974191188737648872120189480483531951904296875f,-1.440300050836801570852685472345910966396331787109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.182200031864642841838985987124033272266387939453125f,0.303900033539533875881488711456768214702606201171875f,-0.13020000513196006153293637908063828945159912109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.18609998437911212665341054162126965820789337158203125f,4.34749998971819717752396172727458178997039794921875f,-1.51140002184212107749772258102893829345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.924200052879006417327900635427795350551605224609375f,-0.2032999577753245024069883584161289036273956298828125f,-1.0301000283822421277335479317116551101207733154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.961399933813512408420365318306721746921539306640625f,-2.42599995448812766341006863513030111789703369140625f,-0.6239999252781271987799982525757513940334320068359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4877999537184838363401695460197515785694122314453125f,0.5550000297278157024294387156260199844837188720703125f,-2.035599938170614020549464839859865605831146240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.35740000166222485322720103795290924608707427978515625f,0.63400000160932556969584084072266705334186553955078125f,1.4321000202089546604611314251087605953216552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.498400047457217443280796942417509853839874267578125f,-0.611200040924549714560498614446260035037994384765625f,-0.7319000057078899335039068319019861519336700439453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.978000045835970777119428021251223981380462646484375f,0.4869999483525753181112349921022541821002960205078125f,-0.43300000202655919423477826057933270931243896484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.434500018779188312834094176650978624820709228515625f,-0.086199979531764281404093708260916173458099365234375f,-1.05439991948604561144975377828814089298248291015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.31360002442598489125202831928618252277374267578125f,0.3432999804437144586444219385157339274883270263671875f,0.543899913287163361275133866001851856708526611328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.4776000522613512710989880361012183129787445068359375f,0.792699916283786887305495838518254458904266357421875f,1.1387999680161460247518334654159843921661376953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.12020010210126608196645747739239595830440521240234375f,-1.34600002238154292655281096813268959522247314453125f,-1.44569995582401755740420412621460855007171630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6319000741600984127899209852330386638641357421875f,0.23700002628564931228538625873625278472900390625f,-0.175200058031081962184316580533050000667572021484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.64530004398226736839205841533839702606201171875f,-0.034499965339898353011705012249876745045185089111328125f,0.07950002717971837995492023765109479427337646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.567200021582841440448419234598986804485321044921875f,-1.56079999236464583844963271985761821269989013671875f,0.2174000119626526839056168682873249053955078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2766000253349549797832196418312378227710723876953125f,-1.1429000482618814071855695146950893104076385498046875f,0.10859998786225844025210562904248945415019989013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.769700114342570795855635878979228436946868896484375f,1.509399987360835471150721787125803530216217041015625f,0.0215000606179245323801296763122081756591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,2.15760000547170616158609846024774014949798583984375f,-1.97279998750686669239939874387346208095550537109375f,0.21920002819895945123107594554312527179718017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.60739996234178494916022827965207397937774658203125f,-0.5985000226795680333680138573981821537017822265625f,1.058999949343502056109400655259378254413604736328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.925399999386071758777916329563595354557037353515625f,-0.48599997339397738027599871202255599200725555419921875f,-0.067700006175041238520861952565610408782958984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2132999770164492048252213862724602222442626953125f,1.510899995582550392470011502155102789402008056640625f,-0.07950001926347660674565531735424883663654327392578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.19719995884895258342339730006642639636993408203125f,1.13300002571940439111131126992404460906982421875f,0.54779997424483262147276718678767792880535125732421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.6609000153452149195487663746462203562259674072265625f,-2.012200025345385778763329653884284198284149169921875f,-1.7156000153958803622344930772669613361358642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.143399895912409203191373308072797954082489013671875f,-0.2513999361231935569804818442207761108875274658203125f,0.49049996250122729346543337669572792947292327880859375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.102399921214580746919864395749755203723907470703125f,0.176900013065338868045728304423391819000244140625f,-2.131300006312131234409434910048730671405792236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3235999501436965797296352320699952542781829833984375f,-0.157500082999467938549287282512523233890533447265625f,-1.4520999892145400878007421852089464664459228515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.0524000067472467634388522128574550151824951171875f,0.22340000931024572849992182455025613307952880859375f,0.90449997979402496639522723853588104248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0451000181376937092636580928228795528411865234375f,-0.10300000604987158414616033041966147720813751220703125f,-0.73280002878308270197749152430333197116851806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.27049998742341951896150931133888661861419677734375f,-0.5420999912261965647530814749188721179962158203125f,-0.63699999581277422233682727892301045358180999755859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.2214999627545477522971850703470408916473388671875f,-0.35210004397630678685260363636189140379428863525390625f,1.9802999254435320608536130748689174652099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.118300075699389051209209355874918401241302490234375f,1.700599980378150188897734551574103534221649169921875f,0.41199993367493181750660369289107620716094970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.6803000232696536642151841078884899616241455078125f,0.444499997377395317244008765555918216705322265625f,-1.2519000456616282690447405911982059478759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.584299981479347252388834021985530853271484375f,-0.391799956184625042254765503457747399806976318359375f,-0.251800116446613930776265988242812454700469970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5063000088825819755555812662350945174694061279296875f,-0.897100012683868186513791442848742008209228515625f,1.0906999900221823196488912799395620822906494140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,3.208499983415007417164588332525454461574554443359375f,0.885000037699938957302947528660297393798828125f,-1.8422000274688012932955416545155458152294158935546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0037000250920653510178226497373543679714202880859375f,-0.310799960178135936672561001614667475223541259765625f,0.6352999578163036620281900468398816883563995361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.53230004357621130139932574820704758167266845703125f,0.92370006804466331384873001297819428145885467529296875f,0.53309998831301985688924105488695204257965087890625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.36659990120828300774746821844018995761871337890625f,2.434700051720442903757657404639758169651031494140625f,-1.7466999509364369291830598740489222109317779541015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.65359993930309923104715608133119530975818634033203125f,1.062700021709502617994758111308328807353973388671875f,-1.66049988956004401785548907355405390262603759765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.779800031189619602400853182189166545867919921875f,0.98460005466789024541895969377947039902210235595703125f,-2.0706000101804722390852475655265152454376220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0705999838799236112407697874004952609539031982421875f,1.024100082778932829796758596785366535186767578125f,0.59000000003725305841584258814691565930843353271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5082000104367720982878608992905355989933013916015625f,1.234499941572547498935819021426141262054443359375f,-2.36989996046051398792542386217974126338958740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3137999896228311769164065481163561344146728515625f,0.3890000397562987100030795772909186780452728271484375f,-0.56999996488913862169312096739304251968860626220703125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.168599955619871622047867276705801486968994140625f,-0.695799926799535395360862821689806878566741943359375f,-2.94489998089373017364778206683695316314697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.464099933841826572944455620017834007740020751953125f,2.001299994949996108317691323463805019855499267578125f,-0.7362999762862916242056599003262817859649658203125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.39009993349909866111602241289801895618438720703125f,-1.10440000900477119927245439612306654453277587890625f,1.984200055542587382007013729889877140522003173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6155999889090677523739714160910807549953460693359375f,-0.3192000281244513626432990349712781608104705810546875f,-3.405599956424534280330362889799289405345916748046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5552001066274934171218546907766722142696380615234375f,2.61280000717639904195266353781335055828094482421875f,-0.01079999951720278517086626379750669002532958984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.155900026334822694451531788217835128307342529296875f,0.9870999940574176623186986034852452576160430908203125f,-0.436099980802834519266752977273426949977874755859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.106800051403044327091862442102865315973758697509765625f,1.923599981920420365355539615848101675510406494140625f,0.6963000725477932473239661703701131045818328857421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0595999927908164028877990858745761215686798095703125f,-3.153200001160799903487941264756955206394195556640625f,0.9758000324860220953127054599463008344173431396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.8109000694736832048903352188062854111194610595703125f,1.1515000860989095254893754827207885682582855224609375f,-0.5171000750079743912834828734048642218112945556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1684000409938384290597923609311692416667938232421875f,-0.4505000086948272031150963812251575291156768798828125f,2.934399984324723931194967008195817470550537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.18169996997266935068182647228240966796875f,-0.3790000778287645477604428378981538116931915283203125f,-1.3751999643214052326811724924482405185699462890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.20410000443607512199406528452527709305286407470703125f,0.569699999976158277803506280179135501384735107421875f,0.4073000449359422958650611690245568752288818359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.594900003468989968524738287669606506824493408203125f,1.2682000222086902141427344758994877338409423828125f,-1.5516000149413937680975550392759032547473907470703125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7199999910406769121351544526987709105014801025390625f,0.547300052610040221878762167762033641338348388671875f,-1.4597999758690585192510980050428770482540130615234375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.666599976496397683689565383247099816799163818359375f,-0.15819997009187980463451594914658926427364349365234375f,1.5059000996857871967904429766349494457244873046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3826999468684195715439955165493302047252655029296875f,-1.221799978387355878339803894050419330596923828125f,0.608199939171970793694299572962336242198944091796875f,0.0f,0.0f,0.0f,0.0f,0.0f,3.326100041376054416986107753473334014415740966796875f,1.07009999917596587692969478666782379150390625f,-1.3180999903246750637464401734177954494953155517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.70609997169449822962405960424803197383880615234375f,-1.852499962113796527773956768214702606201171875f,3.2449000649362798043284783489070832729339599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0357999462455520500725469901226460933685302734375f,0.8818000734940178997334214727743528783321380615234375f,0.3013000814512374336828770537977106869220733642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.3003999781146646785856546557624824345111846923828125f,-1.743600022041797981131594497128389775753021240234375f,0.0500000075250859321585039651836268603801727294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.180999991815536720451973451417870819568634033203125f,0.3288000067591674291378467387403361499309539794921875f,0.38730009805858056637362096807919442653656005859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.9662000807479020902945876514422707259654998779296875f,-6.03930002208501104377091905917041003704071044921875f,-0.0489999990388765294113682102761231362819671630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.85690000706762248228187672793865203857421875f,1.032200033727287635798575138323940336704254150390625f,3.94289999055713291653546548332087695598602294921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.093699994209409442902369846706278622150421142578125f,-2.2356999790325762234033390996046364307403564453125f,-0.7491999386802314919719947283738292753696441650390625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.67499995119869726778460972127504646778106689453125f,1.421499950349330188004159936099313199520111083984375f,2.646700023691355596611174405552446842193603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.04220003111958536834436017670668661594390869140625f,-1.801199983704090623604088250431232154369354248046875f,-0.75679999507665574043357992195524275302886962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.08760007457584162438024577568285167217254638671875f,0.679500061750411443739494643523357808589935302734375f,2.087900053998828386880859397933818399906158447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.458600004085897960948159379768185317516326904296875f,0.17950002166628831190564596909098327159881591796875f,0.23149998961389162133173158508725464344024658203125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.44929996128827465184940592735074460506439208984375f,0.276299977850913425214685048558749258518218994140625f,0.645199898067116084376948492717929184436798095703125f,0.0f,0.0f,0.0f,0.0f,0.0f,3.473199955563246899004070655792020261287689208984375f,-1.7044998442679666350585421241703443229198455810546875f,4.3832000652357923087265589856542646884918212890625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.9313999842539413265285475063137710094451904296875f,-0.840500082716346685884900580276735126972198486328125f,1.1195999678313739433832552094827406108379364013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.01950000007823149417163222096860408782958984375f,0.60879998060762918754562633694149553775787353515625f,-1.5254998951666054463061072965501807630062103271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.96960000180602090580350704840384423732757568359375f,0.76910001870393696066230404539965093135833740234375f,-0.201799961102008040825239731930196285247802734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.172599984833598174560620464035309851169586181640625f,0.1475999284699553992794562873314134776592254638671875f,0.1893000113934271144700005606864579021930694580078125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.51680000871121922756401545484550297260284423828125f,0.8721999657779935244406033234554342925548553466796875f,0.962199977028370678766577839269302785396575927734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.44059993848726108911506571530480869114398956298828125f,1.393699846408515963247509716893546283245086669921875f,-0.6356999145291755137776590345310978591442108154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.830499956466258293374949062126688659191131591796875f,-2.99590002946406830375281060696579515933990478515625f,-0.900199966537952089851160053513012826442718505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.374799954709409188780000476981513202190399169921875f,-1.53679993685036908601659888518042862415313720703125f,0.69319998048543940427634879597462713718414306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.5277000267013904899471299358992837369441986083984375f,-2.491499968435616718664959989837370812892913818359375f,-1.5604999468736349133024532420677132904529571533203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0395000345557929843920419443747960031032562255859375f,-1.2811000726312402608897400568821467459201812744140625f,-0.0225999735087174524750253112870268523693084716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.3894999897778035347073455341160297393798828125f,-0.419999997690319926135771311237476766109466552734375f,-0.52240006538331495988813912845216691493988037109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.02559995027780548326745702070184051990509033203125f,-2.0429000096678731068777779000811278820037841796875f,0.0544999594539401499559971853159368038177490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.423299890515209664698659253190271556377410888671875f,-2.75169999398216713615283879335038363933563232421875f,-0.73449998445063879870531309279613196849822998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.967899990873784421552272760891355574131011962890625f,0.55030007456690033240676029890892095863819122314453125f,4.331299983736872860617950209416449069976806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.59299998068064496692386455833911895751953125f,-0.61720005110576803275534985004924237728118896484375f,-2.77119995844662181383455390459857881069183349609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0968000095680345840065683660213835537433624267578125f,-0.643900004463643593766164485714398324489593505859375f,1.36029993499889911845457390882074832916259765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.87780004252940357911683122438262216746807098388671875f,0.72550004466250628265555633333860896527767181396484375f,-0.455400031572580576977316013653762638568878173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5445000111237170425937392792548052966594696044921875f,-0.70380001555085180964255187063827179372310638427734375f,-0.95610002984628106315057038955274038016796112060546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.95040001153051800741877741529606282711029052734375f,-0.1320999878361821533445663590100593864917755126953125f,0.4761000199183824665993824964971281588077545166015625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3548000215783722044449177701608277857303619384765625f,-0.3279000086992971407795494087622500956058502197265625f,-0.74290000765621666545257539837621152400970458984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.374700049299000692570871251518838107585906982421875f,0.2520000539273026252118370393873192369937896728515625f,0.62039996883869086286011906850035302340984344482421875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1464999362304817420721292364760302007198333740234375f,0.98579990683197904477452766514034010469913482666015625f,-0.092200080405174578146443309378810226917266845703125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.30070003699809344510640585212968289852142333984375f,-1.526599983201921251207977547892369329929351806640625f,0.417499996237456316716674109557061456143856048583984375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.09550000187754648095506126992404460906982421875f,0.12009997827708696060966531149460934102535247802734375f,-0.2269999940991403175605256592461955733597278594970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2719000379130249722692269642720930278301239013671875f,0.111900002932548814360558253611088730394840240478515625f,-1.0782999908745285466693530906923115253448486328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7161000073641539120927745898370631039142608642578125f,-0.608000000201165402557990091736428439617156982421875f,0.889000026419759503681916612549684941768646240234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2968999833375212116237662485218606889247894287109375f,-0.7639000168502334542353082724730484187602996826171875f,0.36059996732100929595077332123764790594577789306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.353500041082501825773221071358420886099338531494140625f,2.138700017734616931619484603288583457469940185546875f,1.406000003866851866263232295750640332698822021484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.80219996864646692102951419656164944171905517578125f,-0.58139997430741718886793023557402193546295166015625f,0.5151000249549750886757237822166644036769866943359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.57669998699724656088250185348442755639553070068359375f,-0.88530002028942111902409806134528480470180511474609375f,0.77709998649507705525962819592677988111972808837890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.889800017964839984330183142446912825107574462890625f,-0.74070003226697467280104092424153350293636322021484375f,0.36879999178349986976854779641143977642059326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1474000001534816473025557570508681237697601318359375f,1.103200009188800567727639645454473793506622314453125f,0.42219999772235705659539917178335599601268768310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.05839994903504786005754567668191157281398773193359375f,1.0247000552967191477904407292953692376613616943359375f,0.22510000327378520967869235391844995319843292236328125f,0.0f,0.0f,0.0f,0.0f,0.0f,2.031999989628790359574850299395620822906494140625f,-1.879100037814676493752585884067229926586151123046875f,-0.343300044555961303416324881254695355892181396484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.1289999275132995837367388958227820694446563720703125f,1.5493000681668525064793584533617831766605377197265625f,-0.6765000633373861038677432588883675634860992431640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3695000148862590538811900842119939625263214111328125f,1.2072999644696704724111668838304467499256134033203125f,-0.685100031716376900448040032642893493175506591796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2395000254288313090711426411871798336505889892578125f,-0.1255999993026255534545043701655231416225433349609375f,-1.3892999996587622746346823987551033496856689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1284000286817548186490967054851353168487548828125f,-0.42009998751580734932531413505785167217254638671875f,1.092599990123510611539359160815365612506866455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0923000285819171040202490985393524169921875f,-0.77430004205256697247250485816039144992828369140625f,-0.708400032183528427509600078337825834751129150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4282999919548629019772079118411056697368621826171875f,0.007900008662044655238787527196109294891357421875f,0.302300025601685451448474850622005760669708251953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.84920000156313213413028506693080998957157135009765625f,-0.77639996644705344674974867302807979285717010498046875f,0.2741000249996770055105343999457545578479766845703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.614299948473274515237108062137849628925323486328125f,3.11140002903193302330464575788937509059906005859375f,0.26539997353255717627007470582611858844757080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.00640004274100025583038586773909628391265869140625f,0.899500016115604150712670161738060414791107177734375f,-0.21740003681033870375216565662412904202938079833984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7682000585302717343694212104310281574726104736328125f,0.15009998416304615176386505481787025928497314453125f,0.457099966266749646592870703898370265960693359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1335000214874748625248912503593601286411285400390625f,-1.329100062289834571771507398807443678379058837890625f,0.340500043600797663856383223901502788066864013671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9093000701040041544587211319594644010066986083984375f,1.49560001822710031404994879267178475856781005859375f,-0.89320004862099822151577654949505813419818878173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.88020002079680581541509809540002606809139251708984375f,1.44070002455562384824361288337968289852142333984375f,0.3875000493228430986647481404361315071582794189453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2424999963864691299164633164764381945133209228515625f,-1.31519992315694711493279100977815687656402587890625f,-3.04489994729161228548264261917211115360260009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.85419996092021488465206857654266059398651123046875f,2.877299960260095712527572686667554080486297607421875f,2.92789988470301221212821474182419478893280029296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.84010001225173380756672258939943276345729827880859375f,-0.910700042548775900996815835242159664630889892578125f,-0.5103999951578661598006192434695549309253692626953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.140699974897503299331447124131955206394195556640625f,1.37479998473525011348783664288930594921112060546875f,0.73770003139525675006638039121753536164760589599609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.7354999383240941046580019246903248131275177001953125f,0.8294000223785633618689416834968142211437225341796875f,-1.281899997493624265842981913010589778423309326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.313600064063070949060829661902971565723419189453125f,-1.7850999295502898434051530784927308559417724609375f,1.644399977712333704715774729265831410884857177734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.15920002536773603107889130114926956593990325927734375f,1.414100019709765287956315660267136991024017333984375f,1.0233999738268548806985336341313086450099945068359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.02149996763467765248378782416693866252899169921875f,-0.261600029339640993697457815869711339473724365234375f,4.2393000336334107913671687128953635692596435546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.821799872924387830863679482717998325824737548828125f,1.047900020694731981762970463023521006107330322265625f,-0.2253000173836949837635756921372376382350921630859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6314999905452125528171336554805748164653778076171875f,-2.795199908460677118426929155248217284679412841796875f,0.76050007869303215546352703313459642231464385986328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.729300029610097766408216557465493679046630859375f,0.282400002109258974059002866852097213268280029296875f,-2.214100064301491865847992812632583081722259521484375f,0.0f,0.0f,0.0f,0.0f,0.0f,1.526800009717046879842428097617812454700469970703125f,1.398400023019313653804829300497658550739288330078125f,1.5078999667525294636760690991650335490703582763671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1507999926626675257779197636409662663936614990234375f,0.595300060597061087008796675945632159709930419921875f,0.764999993294476876570797685417346656322479248046875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.69840010110139782995020141243003308773040771484375f,-0.3752999478325238502662841710844077169895172119140625f,1.0028000130623586638733968356973491609096527099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.2955999971047038599181178142316639423370361328125f,-0.437100003221632615879599370600772090256214141845703125f,-0.3085999913081527434854933744645677506923675537109375f,0.0f,0.0f,0.0f,0.0f,0.0f,2.46659997765123772950346392462961375713348388671875f,-0.538599954167008565519836338353343307971954345703125f,1.555300017122922984214028474525548517704010009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.08120001875907245647567833657376468181610107421875f,0.76839994583129811456245761291938833892345428466796875f,0.3528000132672481914397621949319727718830108642578125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1380000007227042413404660692322067916393280029296875f,-1.09659995999336334904228351661004126071929931640625f,0.3876999447636297890795731291291303932666778564453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.59030003731399693922554661185131408274173736572265625f,-0.80989998588561984416145378418150357902050018310546875f,-0.8442998944938182592778730395366437733173370361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.465400046809018252957912409328855574131011962890625f,0.47810000258088114488685960168368183076381683349609375f,-1.1394999945089221426997028174810111522674560546875f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2513999772876511773489482948207296431064605712890625f,0.668499971680342586211054367595352232456207275390625f,0.8835999829635026170393530264846049249172210693359375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.030999929361044120668111645500175654888153076171875f,0.11389999490827396044778652139939367771148681640625f,-0.739100008720159795672088876017369329929351806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,2.54380002009570471699362315121106803417205810546875f,2.718200005817412279185418810811825096607208251953125f,-0.3006999684900062419501409749500453472137451171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9977000554792567754702758975327014923095703125f,1.442500016167759913088275425252504646778106689453125f,-0.1466000254094599331011750109610147774219512939453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1591000408135361254835515865124762058258056640625f,-1.131900017014146442306810058653354644775390625f,1.588299856950344146611087126075290143489837646484375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.22279995621442783004795273882336914539337158203125f,0.871100055643916793002290432923473417758941650390625f,0.08599998673051612030349133419804275035858154296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.11120002948790908003928734615328721702098846435546875f,0.67210001396909291315751033835113048553466796875f,0.883099993118643400435985313379205763339996337890625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5899000170849273416706637362949550151824951171875f,0.03830003565251793862245222044293768703937530517578125f,1.8639000038489701438493284513242542743682861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2658999989911909889173102783388458192348480224609375f,-0.8657000454172496528570945883984677493572235107421875f,0.8278999661192312231605683336965739727020263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.13400000771879927441432300838641822338104248046875f,-2.43629999588131962440229472122155129909515380859375f,-0.12220001863986118539884273559437133371829986572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.7033999446392058541022151985089294612407684326171875f,0.110800020006299515529235577560029923915863037109375f,0.01319989160001266981225853669457137584686279296875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.35100002141296737789843973587267100811004638671875f,-0.88640003678053513880286118364892899990081787109375f,-0.30449998672306410885113336917129345238208770751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.9799000419139864970219377937610261142253875732421875f,-0.4053000073999164509785941845620982348918914794921875f,-0.148999991923570274110488753649406135082244873046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.922700053709745393604180208058096468448638916015625f,-0.683799923014640764762361868633888661861419677734375f,-1.3692999719798566449213694795616902410984039306640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.549100023025274719401522816042415797710418701171875f,0.141899991980194872809306616545654833316802978515625f,-1.140799916927516477471726830117404460906982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7614999901726842157501096153282560408115386962890625f,0.44400002787262204950735622333013452589511871337890625f,-2.42560001804083480436702302540652453899383544921875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2743999516353026191239905529073439538478851318359375f,-1.714700035142897593942734602023847401142120361328125f,3.072100044050813938412147763301618397235870361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,1.708900027877092497874400578439235687255859375f,-0.94220002202987640504971977861714549362659454345703125f,-4.93579986913204304954660983639769256114959716796875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6863000244729235976848258360405452549457550048828125f,-2.2470999491676675319240530370734632015228271484375f,-0.2559000022508184368774664108059369027614593505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.32969999572932717768480870290659368038177490234375f,0.811400026610494595757927527301944792270660400390625f,0.63560001681149014274296860094182193279266357421875f,0.0f,0.0f,0.0f,0.0f,0.0f,1.94709998779892945464098374941386282444000244140625f,-0.0439000107035045772363446303643286228179931640625f,0.1913000169292098373574617653503082692623138427734375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.686099941426515425035859152558259665966033935546875f,-0.3470999542340640342530377893126569688320159912109375f,-0.2035000197365872676158460308215580880641937255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.2755999695934352455850557817029766738414764404296875f,-1.00629997881949062588091692305169999599456787109375f,0.06430005602240596029872676808736287057399749755859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.945499957919121669647211092524230480194091796875f,0.8404998490288850820917332384851761162281036376953125f,0.47960003257691752320823752597789280116558074951171875f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.14499996412545501645041667870827950537204742431640625f,-0.1001000714465967167399185200338251888751983642578125f,4.30120002598613471178623512969352304935455322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.105500010184944148505792327341623604297637939453125f,0.1513000191457571919073643584852106869220733642578125f,0.09429997151419389211923771654255688190460205078125f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.339699964660405751004645935609005391597747802734375f,-1.8998000088751307856682615238241851329803466796875f,-0.4868000590026380081098977825604379177093505859375f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8974999643862238141167608773685060441493988037109375f,2.9700000077858561553512117825448513031005859375f,-0.28800004646927135354417259804904460906982421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.796100005985795444729546943563036620616912841796875f,-0.1409000387027872225331748268217779695987701416015625f,-0.99619997770637258582837603171356022357940673828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.53120000996738647724981774445041082799434661865234375f,2.109899887090921577481594795244745910167694091796875f,-1.083599969757349068544272086001001298427581787109375f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.00199998394772382681594535824842751026153564453125f,-0.899200016985833183724707851069979369640350341796875f,-0.594600014173984892096314069931395351886749267578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.99989998994618645156151615083217620849609375f,-1.3875000771321357451171252250787802040576934814453125f,-0.51740002894997605409344032523222267627716064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.7105999824270607945209121680818498134613037109375f,0.586799933665246076230914695770479738712310791015625f,3.9646999270722265151789542869664728641510009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0164000304102902560998700209893286228179931640625f,0.63549999096244558227652987625333480536937713623046875f,-0.4939999837279316352578462101519107818603515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_strided_dilated_padded_neg)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,8,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,2,7,1,3};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{3,2,2,3},        // move_strides
                                                                  Strides{2,1,3,2},        // filter_dilation
                                                                  CoordinateDiff{-2,4,0,5}, // below_pads
                                                                  CoordinateDiff{1,3,-1,-4}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f,.81f,-.90f,.66f,-.06f,-.42f,.47f,.12f,.59f,-.26f,.12f,-.93f,.61f,-.19f,-.96f,-.63f,-.86f,-.96f,.33f,-.80f,-.31f,-.22f,-.73f,-.39f,.69f,-.59f,.17f,-.24f,.75f,-.61f,-.90f,.23f,.75f,-.51f,-.08f,-.47f,.37f,-.30f,.36f,.75f,.93f,.97f,.52f,.64f,.98f,-.05f,.34f,-.66f,.60f,.44f,.48f,.53f,-.61f,-.57f,.18f,-.28f,.80f,.14f,.31f,-.54f,-.56f,.48f,.28f,-.86f,-.04f,-.12f,-.97f,.87f,-.82f,-.33f,-.75f,.67f,-.05f,.83f,.14f,.84f,-.47f,.25f,-.52f,-.48f,-.86f,.15f,.19f,-.45f,-.48f,-.33f,-.98f,-.61f,.13f,.83f,-.45f,.78f,.16f,-.52f,.37f,.07f,-.98f,.78f,.91f,-.26f,-.30f,.10f,-.92f,.17f,.10f,.44f,-.31f,-.50f,-.85f,-.29f,.85f,-.47f,.13f,-.59f,-.71f,.34f,.39f,-.72f,-.46f,-.66f,.44f,.95f,.09f,-.10f,.81f,.37f,-.82f,.24f,-.58f,-.79f,.61f,-.73f,.45f,-.76f,-.55f,-.06f,-.59f,-.04f,.42f,.08f,-.51f,-.87f,-.54f,-.25f,.51f,-.14f,.78f,.06f,.44f,-.40f,.11f,-.47f,.36f,.23f,.12f,.49f,-.40f,-.39f,.45f,.74f,-.99f,-.21f,.03f,.85f,.59f,.67f,.15f,.74f,.34f,-.49f,-.59f,-.82f,-.82f,.60f,.12f,-.61f,-.19f,.52f,-.32f,.96f,.46f,.95f,.27f,.97f,-.01f,.18f,.81f,-.55f,.05f,-.76f,-.74f,.08f,.58f,-.77f,.43f,-.55f,-.78f,.37f,.39f,-.37f,-.03f,-.75f,-.46f,.96f,.50f,.55f,-.81f,.39f,-.54f,.42f,-.92f,-.40f,.31f,.16f,-.87f,-.58f,.41f,-.31f,-.74f,-.36f,.18f,.00f,.32f,.32f,.39f,-.91f,.50f,.75f,.38f,.60f,.11f,-.56f,.61f,-.92f,.38f,.10f,-.30f,-.42f,.35f,-.51f,.58f,.84f,-.37f,-.00f,.82f,.15f,.82f,-.89f,.34f,.29f,-.45f,-.31f,-.54f,.02f,-.69f,.15f,-.77f,-.14f,-.89f,-.19f,-.03f,-.41f,.20f,-.28f,-.60f,.25f,-.18f,.82f,-.38f,-.20f,-.25f,-.18f,-.85f,.88f,-.02f,.97f,.62f,-.97f,.63f,.36f,-.13f,-.93f,-.84f,.63f,.87f,-.18f,-.34f,-.72f,.63f,.13f,-.05f,-.56f,.81f,-.19f,-.89f,.61f,.71f,.22f,.45f,-.44f,.89f,.67f,.47f,.85f,-.50f,-.03f,-.73f,.20f,-.86f,.71f,-.12f,-.16f,.50f,.47f,-.95f,-.37f,.81f,.85f,.69f,.46f,-.77f,.25f,.15f,.09f,-.12f,-.32f,.93f,-.04f,-.66f,-.41f,-.91f,-.84f,-.32f,-.51f,.65f,.09f,-.18f,.80f,.55f,-.30f,.56f,.22f,.28f,-.48f,-.61f,-.15f,-.65f,-.38f,.03f,-.83f,.61f,.83f,-.14f,.39f,-.44f,-.49f,-.70f,.35f,-.72f,.51f,.09f,.31f,.98f,.63f,.74f,.25f,-.30f,.20f,.08f,.81f,.69f,.16f,-.85f,.69f,.20f,-.68f,.99f,.14f,-.65f,-.21f,.03f,.06f,-.16f,.80f,.95f,.90f,-.13f,.05f,.84f,.56f,-.42f,-.58f,.02f,-.16f,-.68f,.03f,.94f,-.20f,-.99f,.99f,-.20f,-.69f,.25f,-.00f,.55f,-.11f,.74f,.96f,-.12f,-.50f,.81f,.97f,-.56f,.40f,.94f,.34f,-.98f,-.04f,-.45f,-.89f,-.04f,-.07f,-.35f,.58f,-.52f,.85f,.94f,-.92f,-.46f,-.51f,-.30f,-.98f,.04f,.45f,-.08f,.53f,-.78f,.43f,.90f,-.48f,-.31f,-.84f,-.70f,-.79f,-.33f,.13f,.68f,-.44f,.21f,.62f,.73f,.15f,.72f,.31f,.21f,-.78f,-.89f,-.96f,-.69f,.12f,.45f,-.47f,-.91f,-.33f,.75f,.87f,.35f,-.33f,.08f,.34f,-.93f,.55f,-.30f,.93f,.60f,-.06f,.89f,-.84f,-.34f,.59f,.20f,-.63f,-.40f,.10f,-.47f,-.79f,-.05f,-.28f,-.97f,.68f,.58f,-.96f,-.32f,.09f,-.14f,.20f,.86f,.33f,-.27f,-.62f,.99f,-.56f,.87f,-.75f,-.53f,.86f,-.44f,-.38f,-.30f,-.73f,-.95f,-.19f,.16f,-.16f,-.87f,-.41f,.77f,-.40f,.90f,-.10f,.67f,-.28f,-.81f,.66f,-.97f,-.21f,-.08f,-.57f,-.80f,.07f,-.86f,-.94f,-.33f,-.43f,-.63f,.52f,-.74f,-.07f,-.17f,.75f,.10f,.97f,.70f,-.01f,.13f,.28f,-.59f,-.25f,.39f,.44f,.18f,.38f,-.95f,.93f,-.80f,.37f,.08f,.42f,-.84f,-.05f,.21f,.96f,.93f,-.60f,.96f,-.50f,.17f,-.74f,.47f,-.16f,-.05f,-.36f,-.46f,-.62f,.95f,.44f,-.45f,.42f,-.31f,.79f,.37f,-.21f,-.78f,-.08f,-.70f,-.83f,.02f,.10f,-.45f,-.12f,-.45f,-.90f,.95f,.60f,.34f,-.38f,-.22f,.37f,.04f,-.21f,-.63f,.10f,.61f,-.76f,-.48f,-.45f,.27f,.96f,.75f,.09f,.77f,.99f,-.35f,.81f,.95f,.14f,-.41f,-.54f,-.43f,.25f,.36f,-.61f,-.11f,.83f,.14f,.56f,-.42f,.80f,.35f,.11f,-.51f,-.68f,-.28f,.05f,.54f,-.01f,-.74f,-.19f,-.91f,.86f,-.05f,.19f,-.71f,-.22f,.39f,.97f,.08f,.92f,-.36f,.29f,.08f,.99f,-.31f,-.94f,-.37f,.17f,.41f,.41f,-.57f,-.56f,-.05f,.88f,.19f,.91f,.34f,.07f,.39f,-.56f,.76f,.87f,-.82f,.79f,-.29f,.29f,.79f,-.00f,-.68f,.22f,.75f,-.29f,-.92f,-.57f,.88f,-.78f,-.99f,.88f,.34f,.82f,.86f,-.17f,.79f,.12f,.42f,-.50f,.23f,.83f,.25f,.77f,.62f,-.81f,-.24f,.47f,-.49f,-.45f,.06f,-.38f,-.01f,.15f,-.16f,-.63f,-.25f,-.99f,.24f,.85f,-.69f,-.37f,-.27f,.01f,-.22f,.73f,.63f,-.48f,.87f,.90f,-.64f,.68f,.59f,.11f,-.83f,.17f,.87f,-.88f,.76f,-.85f,-.20f,.94f,.71f,.16f,-.40f,-.73f,.91f,.61f,.89f,-.73f,.79f,-.48f,-.18f,.74f,.64f,-.74f,-.89f,-.84f,-.28f,-.00f,.45f,.50f,.78f,.84f,-.54f,.66f,.67f,-.58f,-.67f,.09f,-.93f,-.50f,-.71f,-.57f,.44f,-.20f,.42f,-.65f,-.81f,.37f,.21f,-.82f,-.16f,.84f,.04f,.51f,.33f,-.43f,.44f,-.82f,.92f,-.33f,-.92f,.76f,-.33f,.83f,-.21f,-.25f,.28f,.78f,.70f,.04f,.65f,-.44f,.28f,-.61f,-.85f,.26f,-.86f,.41f,.02f,-.44f,.93f,.27f,.79f,-.60f,.91f,-.25f,-.92f,.13f,.87f,-.29f,-.89f,-.56f,-.91f,-.95f,-.72f,-.62f,-.99f,-.89f,-.89f,.64f,-.08f,.70f,-.78f,.82f,-.54f,.97f,-.26f,.12f,.46f,-.26f,-.46f,-.65f,.04f,-.50f,.34f,-.78f,-.54f,-.29f,.13f,.19f,.01f,-.77f,-.20f,-.98f,.43f,.37f,.80f,-.97f,-.99f,-.66f,.55f,.85f,.45f,.58f,-.29f,-.54f,-.95f,.41f,.14f,-.84f,.13f,.06f,.37f,.23f,.76f,.88f,.12f,-.84f,-.53f,-.26f,.26f,.67f,-.24f,.76f,.76f,.12f,-.13f,-.05f,-.12f,.92f,.40f,-.68f,.68f,.00f,.19f,-.72f,.06f,-.66f,.55f,.77f,-.71f,.84f,.59f,.15f,.27f,.38f,-.99f,-.03f,.65f,-.94f,.55f,-.51f,.40f,.32f,.33f,-.93f,.49f,.83f,.98f,.32f,-.45f,.32f,-.22f,.37f,-.35f,-.30f,-.63f,-.72f,.86f,-.95f,.70f,.44f,.12f,-.66f,.39f,.67f,-.56f,-.25f,-.64f,-.15f,.33f,.37f,-.26f,.65f,.13f,-.98f,-.27f,-.94f,.90f,-.30f,-.35f,.24f,.20f,-.93f,.24f,-.17f,-.80f,.24f,.92f,-.81f,-.61f,.05f,-.72f,.12f,.95f,-.07f,-.04f,.74f,-.10f,-.07f,-.55f,-.07f,.21f,-.77f,-.79f,.52f,-.76f,-.02f,-.18f,-.88f,-.76f,.78f,-.74f,-.22f,.26f,-.51f,.91f,.33f,.82f,.67f,.89f,-.57f,.47f,-.30f,.06f,.46f,.40f,.33f,.36f,-.12f,.80f,-.46f,.69f,.60f,.96f,.69f,-.85f,-.16f,-.44f,-.13f,-.58f,-.93f,.05f,.51f,-.02f,.79f,-.36f,.27f,.74f,.67f,.61f,-.67f,-.97f,-.01f,.78f,-.94f,.11f,-.35f,-.45f,-.27f,.51f,.07f,-.34f,.46f,.85f,-.25f,.84f,.40f,-.68f,-.66f,-.89f,.72f,-.52f,-.46f,-.19f,.91f,.09f,-.56f,.16f,.01f,-.29f,.86f,-.05f,.50f,-.98f,.98f,-.25f,.21f,-.52f,-.45f,.07f,.97f,.62f,.51f,.91f,-.42f,.80f,-.33f,-.77f,-.98f,.82f,-.35f,.96f,.25f,-.54f,.50f,-.66f,.54f,-.14f,.53f,-.96f,-.05f,-.85f,-.03f,.12f,-.71f,-.06f,.47f,-.80f,.70f,.88f,.57f,-.07f,-.04f,-.13f,.80f,.59f,-.23f,-.66f,-.34f,-.78f,.95f,.14f,.24f,-.37f,-.29f,.51f,-.94f,-.93f,-.62f,-.68f,.58f,-.13f,.16f,.50f,-.23f,-.01f,-.24f,.82f,-.07f,-.01f,-.49f,.23f,-.00f,.83f,-.60f,.72f,.34f,-.12f,-.11f,-.07f,.01f,.15f,-.23f,.58f,-.60f,.74f,.43f,-.15f,-.41f,-.29f,.66f,.60f,.74f,-.13f,.48f,-.13f,-.35f,.29f,-.64f,-.76f,.27f,.56f,.72f,.51f,-.75f,-.30f,.53f,-.62f,.64f,.11f,-.93f,-.12f,-.43f,.96f,.49f,-.66f,.47f,-.96f,-.60f,.72f,.64f,.39f,-.53f,-.61f,-.37f,.99f,-.74f,.50f,-.32f,.82f,-.98f,-.06f,-.68f,.32f,-.28f,-.64f,.30f,-.44f,.60f,.16f,-.29f,-.21f,-.12f,.19f,.26f,-.32f,-.90f,.88f,.96f,-.28f,.51f,-.86f,.28f,.43f,.82f,-.21f,-.15f,.15f,-.70f,-.70f,.05f,-.82f,-.85f,-.57f,.13f,-.22f,.53f,.80f,.43f,.30f,-.77f,-.95f,.95f,.95f,-.21f,.56f,.56f,-.66f,-.70f,-.03f,.51f,.79f,.00f,-.68f,.25f,-.15f,.12f,.07f,-.18f,-.57f,.81f,.22f,-.80f,.14f,.14f,.19f,.65f,-.06f,.72f,-.96f,-.93f,-.96f,-.35f,.06f,.92f,-.01f,-.16f,.13f,.00f,-.24f,.79f,-.15f,-.29f,-.83f,-.38f,-.91f,-.17f,-.21f,-.89f,-.24f,.40f,-.43f,-.31f,.45f,.74f,.77f,-.98f,.53f,.87f,-.42f,-.37f,-.95f,.02f,-.03f,.21f,-.39f,-.27f,-.66f,.62f,-.34f,.34f,.89f,-.64f,-.65f,-.51f,-.98f,.14f,-.49f,.27f,.10f,-.71f,-.20f,-.01f,-.51f,.50f,-.30f,.98f,-.46f,.10f,-.60f,-.71f,-.95f,.49f,.59f,.00f,-.19f,.11f,-.41f,.08f,-.50f,.48f,.19f,-.80f,.55f,-.27f,.14f,-.66f,.04f,-.16f,-.40f,.18f,-.50f,.89f,-.98f,.27f,-.39f,.91f,-.07f,-.88f,-.76f,-.34f,.10f,.80f,-.05f,-.31f,-.87f,.65f,.26f,-.62f,.55f,.57f,-.98f,.11f,.14f,-.63f,-.39f,.13f,-.87f,-.60f,.92f,-.08f,-.88f,.08f,-.82f,-.12f,.71f,.41f,-.81f,.17f,.16f,-.52f,-.44f,-.58f,.24f,-.16f,.42f,-.87f,-.41f,.86f,.51f,-.42f,.57f,.44f,-.90f,-.37f,-.11f,-.13f,.12f,-.13f,-.80f,-.44f,-.12f,.76f,-.65f,-.00f,.94f,-.49f,.64f,-.20f,-.02f,-.30f,-.70f,.81f,.37f,-.12f,.06f,-.73f,-.99f,-.24f,.66f,.32f,-.94f,.29f,.47f,.32f,-.20f,.90f,.37f,.14f,-.65f,-.30f,.03f,-.81f,-.85f,-.45f,.95f,.52f,-.64f,.70f,-.32f,-.50f,.34f,.33f,.44f,.94f,.39f,.78f,-.99f,.35f,-.04f,.56f,-.65f,.33f,-.57f,.23f,-.17f,-.97f,.70f,-.68f,-.23f,-.60f,.20f,-.93f,.00f,.16f,.92f,-.19f,.72f,.28f,.55f,.97f,.44f,-.40f,-.03f,.86f,.27f,.31f,-.50f,.16f,.03f,-.49f,-.09f,-.64f,-.17f,-.01f,.88f,.07f,.45f,-.45f,-.36f,-.40f,.69f,-.54f,-.44f,-.74f,.00f,-.49f,-.40f,-.92f,.16f,-.21f,-.39f,-.26f,-.63f,.14f,-.53f,.31f,-.83f,.33f,.31f,.06f,-.88f,.12f,-.20f,.63f,.35f,.79f,-.25f,.77f,-.79f,.16f,.64f,-.49f,-.14f,.09f,-.00f,.46f,-.96f,-.00f,.67f,.74f,-.23f,-.25f,-.16f,.45f,-.87f,.73f,.89f,-.40f,.85f,.70f,-.98f,.11f,.27f,.64f,.22f,-.38f,.80f,-.38f,.98f,.87f,.96f,.54f,.88f,.46f,.00f,-.08f,.94f,.02f,-.97f,.16f,-.04f,.44f,-.45f,-.58f,.38f,-.49f,.93f,.78f,.84f,-.35f,.28f,.96f,.16f,-.58f,.32f,.34f,-.43f,.34f,.68f,.92f,-.69f,.48f,.42f,-.13f,-.54f,.09f,.70f,-.85f,.74f,-.55f,-.18f,.45f,-.49f,.12f,.33f,-.49f,-.68f,-.67f,.60f,.61f,-.87f,.13f,.68f,-.56f,.83f,-.16f,.26f,-.18f,-.80f,-.47f,.80f,-.25f,.36f,-.12f,-.98f,.78f,-.58f,.76f,-.25f,.34f,.42f,.76f,.09f,-.97f,.57f,.01f,.82f,.02f,-.67f,-.15f,.99f,-.27f,.19f,-.16f,-.28f,-.70f,.17f,.68f,.00f,.81f,-.88f,-.43f,-.71f,-.90f,-.12f,-.31f,-.15f,-.12f,-.44f,.58f,.26f,-.41f,-.23f,-.06f,-.71f,-.21f,.66f,-.27f,.31f,.55f,-.43f,-.23f,-.98f,.17f,-.03f,.66f,.13f,.41f,-.20f,-.93f,-.33f,-.25f,.23f,-.15f,.19f,.73f,.24f,-.49f,.16f,.24f,-.12f,-.21f,-.61f,-.11f,-.23f,-.98f,-.51f,.19f,.09f,.32f,.99f,.28f,.19f,.66f,.74f,.89f,-.10f,.04f,-.12f,.79f,.36f,.17f,.51f,-.66f,-.09f,.75f,-.88f,-.67f,-.74f,-.63f,-.36f,-.93f,.61f,-.12f,-.00f,.06f,.58f,-.27f,.73f,-.97f,-.03f,-.97f,.99f,-.11f,-.55f,-.59f,-.44f,-.90f,-.13f,.20f,.85f,.90f,.38f,-.46f,-.41f,.18f,-.87f,-.82f,.61f,-.75f,.20f,-.74f,-.68f,-.22f,-.21f,.68f,-.00f,.38f,-.50f,-.21f,.11f,.94f,-.98f,.59f,-.69f,-.36f,.92f,-.76f,-.73f,-.29f,.23f,-.52f,.18f,.26f,.37f,.08f,.35f,-.61f,.71f,.36f,.00f,-.67f,.57f,.63f,-.04f,.06f,-.96f,.40f,.37f,.86f,.05f,.49f,.78f,-.47f,-.90f,.53f,-.30f,-.57f,-.72f,.25f,.06f,.98f,.89f,-.66f,-.83f,.55f,.35f,.84f,.27f,-.56f,-.40f,-.56f,-.40f,-.83f,.69f,-.05f,-.39f,.87f,-.81f,-.29f,-.94f,.10f,-.32f,-.48f,.43f,-.40f,-.64f,.31f,.98f,-.96f,-.99f,-.70f,.23f,.95f,-.59f,-.04f,-.06f,.95f,-.52f,.18f,.79f,-.28f,-.30f,-.23f,.66f,.49f,-.21f,-.84f,-.27f,.85f,.12f,.05f,.39f,-.21f,.49f,-.28f,-.07f,.96f,.00f,.78f,-.54f,.75f,.15f,.14f,-.21f,-.19f,.06f,.42f,.34f,-.41f,.11f,.44f,-.02f,.89f,.56f,-.78f,.75f,.06f,-.22f,-.32f,-.78f,-.52f,-.46f,.64f,-.31f,-.53f,-.20f,.66f,.29f,.91f,-.84f,-.82f,.67f,-.92f,-.56f,.26f,-.42f,-.93f,.13f,.90f,-.20f,-.40f,.56f,-.60f,.39f,-.01f,.67f,-.77f,-.70f,.30f,-.21f,.04f,-.05f,-.70f,.87f,-.99f,.23f,-.11f,-.99f,-.72f,.22f,.82f,.59f,-.20f,-.40f,.28f,-.91f,-.44f,-.49f,.67f,.33f,.15f,-.09f,-.77f,-.65f,-.95f,.39f,-.02f,-.08f,-.60f,.10f,-.72f,.51f,-.17f,-.80f,-.07f,.92f,-.81f,.65f,.86f,.51f,-.99f,.32f,-.47f,-.37f,-.95f,.59f,-.46f,-.63f,.84f,.84f,-.61f,.97f,.17f,.48f,.94f,-.02f,.98f,.19f,.16f,.40f,.16f,-.40f,-.94f,-.61f,.49f,-.60f,.52f,-.62f,.91f,.67f,.25f,.68f,-.03f,-.03f,.31f,-.16f,.72f,.28f,-.13f,-.10f,-.47f,-.49f,-.79f,-.24f,-.39f,-.93f,-.93f,-.64f,.79f,.06f,-.73f,-.05f,-.77f,-.42f,-.10f,-.28f,-.26f,.58f,.07f,.80f,.52f,-.68f,.86f,-.35f,.79f,.50f,-.83f,-.52f,.75f,-.54f,.49f,.17f,.29f,.45f,-.50f,-.08f,.81f,.98f,.57f,-.57f,-.73f,-.54f,-.94f,-.91f,.13f,.74f,-.31f,-.23f,-.85f,-.98f,-.88f,-.98f,.70f,-.92f,.85f,-.96f,-.71f,.46f,.07f,-.41f,.80f,.35f,.64f,.50f,-.57f,.86f,.13f,.09f,-.13f,.78f,.51f,.66f,.18f,.63f,.66f,.05f,.06f,-.87f,-.33f,-.19f,-.55f,.94f,-.52f,-.74f,.21f,.33f,.50f,.90f,.53f,.80f,.53f,-.15f,.02f,-.84f,-.14f,.19f,.51f,.26f,.16f,-.88f,.98f,-.73f,.28f,.64f,-.76f,.20f,-.74f,.09f,.21f,-.67f,.63f,-.74f,.18f,-.03f,-.25f,-.42f,.76f,-.79f,.98f,-.48f,.94f,.10f,.36f,.47f,-.56f,-.69f,-.74f,.37f,.49f,.78f,-.57f,.14f,-.83f,.15f,-.88f,.91f,.25f,-.89f,-.98f,-.67f,.74f,.53f,-.50f,-.54f,-.94f,-.93f,.69f,.15f,.76f,.60f,.86f,-.96f,-.52f,.84f,-.73f,.10f,.35f,-.13f,.53f,.43f,.19f,-.59f,-.02f,.43f,.50f,-.18f,.03f,-.13f,-.21f,.46f,-.07f,-.99f,.94f,-.16f,-.43f,.25f,-.55f,-.77f,-.13f,-.99f,.98f,.69f,.92f,-.77f,-.89f,-.03f,.03f,-.10f,.48f,.47f,-.95f,.87f,-.12f,.56f,.36f,.56f,-.04f,-.97f,.57f,-.85f,-.45f,.48f,-.54f,.90f,-.46f,.49f,-.02f,.25f,-.83f,.77f,-.38f,.09f,.35f,.89f,-.79f,-.79f,.13f,-.14f,-.27f,-.49f,.46f,.43f,-.91f,-.79f,-.67f,-.15f,.73f,.88f,.17f,-.03f,.22f,.43f,.47f,-.72f,.38f,.23f,-.28f,-.14f,-.02f,-.23f,.47f,.48f,-.11f,-.42f,-.19f,.20f,-.84f,.91f,-.17f,-.52f,.46f,-.53f,.55f,-.29f,.80f,-.94f,-.24f,.53f,.18f,.52f,-.91f,-.47f,-.98f,.82f,.81f,.89f,-.94f,-.18f,.26f,.98f,-.97f,-.99f,-.82f,.67f,-.88f,.76f,-.73f,.25f,.82f,-.27f,-.16f,.35f,-.61f,.39f,.79f,-.57f,-.41f,-.56f,-.35f,.26f,-.34f,.74f,-.10f,.98f,-.55f,-.38f,-.91f,-.06f,-.85f,.62f,-.34f,.51f,-.83f,.91f,-.09f,-.54f,-.79f,-.37f,.79f,.74f,-.86f,-.52f,.45f,.04f,.65f,-.26f,.70f,.88f,-.13f,.36f,.23f,.06f,.73f,.16f,-.53f,-.91f,-.56f,-.07f,.72f,-.17f,-.13f,.56f,.11f,.19f,.46f,-.31f,.76f,.95f,.68f,.12f,.60f,-.20f,-.89f,.80f,.78f,-.21f,-.04f,.79f,-.18f,.28f,-.37f,-.39f,.92f,-.73f,-.55f,-.32f,-.15f,-.20f,-.59f,.71f,.57f,-.60f,-.58f,-.36f,.05f,.22f,.08f,-.37f,-.73f,.00f,.99f,-.44f,-.83f,.30f,-.39f,-.49f,.40f,-.33f,-.64f,-.07f,.24f,-.80f,.38f,.65f,.18f,.88f,.99f,-.18f,-.08f,.45f,-.76f,-.30f,.87f,.24f,-.21f,-.28f,.62f,-.71f,.57f,.92f,.01f,.70f,.44f,-.77f,-.47f,-.70f,.72f,-.67f,-.60f,.08f,-.26f,.26f,.16f,-.94f,-.06f,.46f,.98f,-.90f,.17f,-.45f,-.43f,-.00f,.71f,-.82f,-.59f,-.44f,.03f,.98f,-.85f,-.33f,.45f,.22f,-.04f,.59f,.87f,.91f,.97f,-.97f,.69f,-.60f,.61f,-.59f,-.56f,-.71f,-.35f,.13f,.06f,.81f,-.19f,.45f,.56f,-.03f,.09f,.28f,-.75f,.97f,.70f,.98f,-.56f,-.62f,.52f,-.57f,.48f,.04f,-.31f,-.60f,-.22f,-.39f,.05f,-.27f,.15f,-.40f,-.47f,.74f,.14f,.96f,-.35f,-.12f,-.95f,.32f,-.99f,.06f,-.22f,-.82f,-.89f,.89f,.00f,.98f,-.90f,.48f,.79f,.87f,.10f,-.29f,.79f,-.88f,-.07f,-.95f,-.70f,.06f,-.44f,.17f,.94f,.54f,.56f,.83f,-.89f,.80f,-.41f,.40f,-.13f,.44f,-.20f,-.29f,.59f,-.58f,-.80f,-.37f,-.33f,-.10f,-.51f,-.90f,-.16f,-.14f,.97f,.06f,.77f,-.70f,-.70f,.30f,-.36f,-.17f,-.17f,.70f,.25f,.24f,-.14f,.22f,.81f,-.16f,.07f,-.89f,.29f,-.60f,-.95f,.26f,-.20f,-.04f,-.90f,.55f,.80f,-.66f,.51f,.59f,.71f,-.51f,.73f,.89f,-.74f,.88f,.70f,.08f,.61f,-.83f,-.89f,-.93f,.34f,-.41f,.85f,-.74f,-.46f,-.48f,.05f,.86f,-.23f,.82f,.92f,-.41f,-.61f,-.76f,.22f,-.80f,-.29f,.18f,-.05f,-.96f,-.47f,.74f,.81f,-.73f,.69f,-.74f,-.05f,-.80f,-.89f,.13f,-.50f,-.40f,.16f,.03f,.85f,-.23f,.63f,.88f,-.77f,-.58f,.47f,-.70f,-.40f,.61f,.62f,-.23f,-.46f,-.19f,.14f,-.18f,.67f,.21f,-.57f,-.75f,-.04f,.20f,.82f,-.18f,-.44f,-.48f,.11f,.50f,-.60f,.67f,-.60f,.48f,.58f,-.92f,.20f,-.86f,.48f,-.87f,.47f,-.43f,.04f,-.90f,-.50f,-.79f,-.28f,-.25f,-.20f,.49f,-.01f,.28f,.83f,-.03f,-.32f,-.72f,.76f,.80f,-.53f,-.64f,-.34f,.59f,-.81f,.20f,.10f,-.13f,-.40f,-.51f,-.57f,.13f,.93f,.89f,-.03f,-.09f,-.96f,-.41f,-.32f,.14f,-.94f,-.21f,.51f,.03f,-.50f,.54f,-.52f,-.23f,.22f,-.73f,-.49f,-.80f,-.84f,-.26f,.53f,-.61f,.81f,-.80f,.83f,.35f,.49f,-.39f,-.10f,.64f,.12f,.65f,.78f,-.84f,-.32f,.01f,-.38f,-.98f,-.75f,.68f,.16f,.56f,-.23f,-.87f,.37f,.19f,.71f,-.73f,.07f,-.17f,.20f,.08f,.33f,-.49f,.67f,-.58f,-.14f,-.60f,-.89f,-.58f,-.06f,.11f,-.14f,.94f,-.81f,-.54f,-.63f,-.89f,-.35f,.90f,.73f,.67f,-.27f,-.36f,.00f,.11f,.48f,.38f,-.78f,.90f,-.36f,-.74f,-.50f,.90f,-.77f,.78f,-.08f,.76f,-.91f,-.97f,-.38f,-.00f,.59f,.79f,-.53f,.21f,.34f,-.79f,.90f,-.42f,-.14f,-.44f,-.25f,-.29f,.34f,-.08f,.76f,.53f,.24f,.07f,.01f,-.29f,-.76f,-.43f,-.40f,.23f,-.66f,.62f,.79f,-.07f,-.26f,.62f,-.18f,-.66f,.90f,.19f,.58f,.59f,-.24f,-.75f,-.30f,.13f,.68f,.08f,.00f,-.00f,.81f,-.56f,.41f,.58f,.50f,-.40f,.59f,-.45f,-.56f,.66f,.02f,.74f,-.27f,.70f,-.50f,-.76f,.24f,.94f,-.61f,.74f,-.95f,.76f,-.24f,.09f,.87f,-.02f,-.73f,-.43f,-.26f,.39f,-.78f,-.50f,-.52f,-.84f,-.81f,-.51f,.86f,-.27f,.27f,-.92f,.39f,.02f,.12f,-.47f,-.75f,-.69f,.00f,-.46f,.19f,.21f,-.75f,.65f,.12f,.74f,-.92f,-.88f,.68f,.85f,-.38f,-.68f,.43f,-.88f,.12f,-.02f,-.77f,.27f,.40f,.53f,.95f,.99f,.56f,.62f,-.68f,-.69f,-.87f,.09f,-.91f,-.25f,-.66f,.19f,-.69f,-.54f,.46f,.21f,-.76f,.84f,-.83f,-.30f,-.30f,.16f,.38f,.39f,-.04f,.32f,-.68f,-.43f,.27f,.08f,.41f,.93f,-.61f,-.54f,.52f,-.99f,.04f,.74f,-.26f,.25f,-.28f,-.60f,.52f,-.68f,.75f,-.62f,-.32f,-.51f,-.69f,-.61f,.90f,-.71f,.81f,-.20f,.24f,.92f,-.25f,-.76f,.88f,.88f,.42f,-.21f,.11f,.33f,-.09f,.01f,-.72f,.96f,.20f,-.70f,-.59f,.48f,-.03f,.42f,.54f,.67f,-.21f,-.13f,.55f,-.11f,.26f,.52f,.83f,.76f,.21f,.51f,-.35f,.49f,.04f,-.62f,.88f,.04f,.00f,.33f,-.68f,.72f,.78f,-.69f,.54f,.04f,-.69f,-.51f,.56f,-.05f,-.68f,.16f,.62f,-.90f,.77f,-.25f,-.78f,.25f,-.86f,.72f,-.82f,-.13f,-.23f,.77f,-.06f,.13f,.29f,-.79f,.13f,-.77f,-.56f,-.22f,-.17f,-.20f,-.35f,.91f,-.92f,.70f,-.07f,.99f,-.44f,.00f,-.91f,.36f,-.69f,-.97f,.97f,-.15f,.76f,.78f,-.36f,-.46f,.43f,.78f,-.08f,-.36f,.91f,-.15f,-.49f,.89f,-.47f,-.88f,.32f,-.35f,.46f,.86f,.19f,.87f,.76f,.41f,.87f,.71f,.11f,.78f,-.87f,.70f,-.29f,-.32f,.01f,-.68f,.19f,-.59f,-.04f,.89f,-.46f,.46f,-.82f,.02f,.35f,-.93f,-.46f,.58f,.03f,-.81f,.27f,-.90f,-.33f,-.93f,-.47f,-.42f,-.27f,-.49f,.82f,.12f,.82f,.29f,-.35f,.74f,.00f,-.70f,.45f,-.19f,.05f,-.39f,-.84f,-.99f,-.22f,-.71f,-.60f,-.54f,.21f,.61f,.18f,.08f,.19f,-.06f,-.98f,.21f,-.78f,.15f,.68f,-.53f,.31f,.89f,-.01f,-.73f,.02f,.40f,-.37f,.23f,-.95f,.62f,.84f,-.35f,-.63f,.96f,-.47f,.14f,.80f,-.68f,.29f,.99f,.49f,-.01f,.16f,.19f,-.38f,.63f,-.93f,-.31f,.83f,-.64f,-.72f,.49f,-.87f,.09f,.89f,-.76f,-.98f,.36f,.71f,.07f,.99f,-.47f,-.41f,.68f,.86f,.77f,.92f,-.79f,-.22f,.24f,.09f,-.45f,.00f,.22f,-.96f,-.50f,.87f,.06f,.93f,-.39f,-.39f,-.39f,-.36f,-.11f,.77f,-.29f,.32f,-.60f,-.69f,-.35f,.12f,-.30f,.66f,.29f,-.45f,-.18f,-.95f,.75f,-.69f,-.71f,-.93f,-.25f,-.36f,.81f,-.79f,.17f,.93f,-.60f,-.98f,.55f,-.87f,-.67f,.32f,.80f,.79f,.62f,.44f,-.19f,-.48f,.34f,.89f,.72f,.37f,-.82f,-.49f,-.71f,-.40f,-.08f,-.05f,-.30f,-.46f,.50f,.24f,.85f,-.95f,-.62f,-.45f,-.09f,.42f,-.95f,-.20f,.21f,-.50f,.39f,-.38f,-.74f,.69f,-.65f,.43f,.60f,-.48f,-.81f,-.14f,-.90f,.56f,.42f,.39f,-.13f,.76f,.06f,-.46f,-.55f,.60f,.72f,-.05f,-.20f,-.17f,-.40f,.23f,-.55f,.90f,-.49f,.58f,.34f,.72f,-.72f,-.54f,-.43f,-.80f,-.49f,-.76f,-.76f,-.11f,.55f,-.68f,.14f,-.98f,.20f,.93f,.19f,.52f,-.26f,.89f,-.82f,-.02f,-.47f,.74f,-.94f,-.01f,.90f,.68f,.20f,.36f,-.50f,-.99f,.20f,.05f,-.18f,-.06f,-.39f,.60f,-.12f,-.94f,.89f,-.31f,.00f,.35f,.74f,-.24f,-.88f,.41f,-.78f,-.04f,.38f,-.17f,.93f,.48f,.31f,.60f,-.20f,-.78f,.85f,-.03f,.26f,.88f,.17f,.82f,.39f,-.03f,.10f,.13f,.72f,.15f,.40f,-.39f,.32f,.44f,-.75f,-.64f,-.29f,.77f,-.50f,.74f,.71f,-.08f,-.30f,-.42f,.17f,-.52f,.84f,-.07f,-.89f,-.60f,-.04f,.64f,-.73f,-.10f,-.82f,.87f,-.78f,.23f,.67f,-.98f,-.68f,.66f,.82f,.64f,.40f,.55f,.91f,-.02f,.01f,.13f,-.82f,-.86f,.79f,-.06f,-.58f,.75f,.99f,-.05f,.32f,-.07f,.97f,.49f,.40f,.42f,.86f,-.62f,.54f,.57f,.90f,-.76f,.54f,-.61f,.32f,.66f,.21f,-.24f,-.70f,.02f,.27f,.78f,-.45f,.29f,.18f,-.06f,.78f,.38f,-.41f,.14f,.50f,-.43f,-.45f,.68f,-.30f,-.79f,-.88f,.57f,-.06f,.36f,.16f,.14f,.48f,-.27f,.46f,-.78f,-.23f,.20f,.34f,.26f,.34f,-.40f,.07f,.00f,.28f,-.92f,-.23f,.46f,-.60f,-.69f,-.98f,.49f,.21f,-.65f,-.01f,.56f,-.42f,.88f,.96f,.40f,.60f,-.24f,-.93f,-.82f,.64f,.23f,-.24f,.28f,-.50f,.48f,.23f,.52f,.75f,.30f,.59f,-.63f,.46f,.47f,-.91f,.70f,.29f,-.47f,.12f,-.82f,.72f,-.20f,-.48f,.07f,.39f,.72f,.24f,.44f,.79f,-.44f,-.59f,.25f,-.95f,-.60f,.24f,-.40f,.50f,-.05f,-.86f,.49f,.95f,.92f,-.87f,.29f,-.71f,-.14f,.37f,.24f,.91f,.82f,-.64f,.12f,-.39f,.53f,-.95f,.32f,.87f,-.90f,.97f,-.85f,.09f,-.54f,-.75f,.74f,-.59f,.51f,-.23f,-.14f,-.64f,.51f,.39f,.60f,.36f,.16f,-.64f,.27f,-.16f,.32f,-.99f,.77f,.48f,-.71f,.55f,.73f,.44f,-.92f,-.29f,.44f,.10f,-.53f,-.71f,-.09f,-.46f,-.66f,-.31f,.05f,.92f,-.52f,.04f,.94f,.91f,.09f,.03f,.23f,-.88f,-.92f,.91f,-.47f,.04f,.37f,-.75f,.74f,.77f,-.69f,-.02f,-.90f,.64f,-.38f,-.21f,.76f,.71f,.38f,-.89f,.36f,-.17f,.86f,.13f,.45f,.36f,-.06f,.40f,.33f,-.53f,-.47f,.71f,.10f,.63f,.84f,-.22f,.07f,.76f,.98f,-.43f,-.54f,.42f,-.23f,-.75f,-.67f,.62f,.55f,.24f,-.77f,-.45f,.77f,.30f,-.80f,-.47f,.70f,.03f,-.26f,-.10f,.16f,.98f,-.33f,-.64f,.67f,.31f,-.61f,-.09f,.54f,-.23f,-.18f,-.41f,-.32f,-.43f,.35f,-.85f,-.45f,.53f,.24f,.86f,-.45f,.37f,.37f,-.87f,-.44f,-.67f,.66f,-.78f,.84f,.10f,.74f,-.33f,-.74f,.28f,.30f,.43f,.57f,.48f,-.36f,-.84f,.82f,.03f,.36f,.04f,.93f,-.42f,.73f,-.33f,.54f,-.85f,.28f,.19f,.31f,-.06f,.59f,.01f,.46f,-.68f,-.79f,-.91f,.95f,-.04f,.59f,.66f,-.03f,-.35f,.77f,.27f,.48f,.04f,-.91f,-.90f,-.38f,-.43f,.19f,.47f,.10f,.01f,-.15f,.38f,.74f,.24f,.68f,.66f,-.06f,-.09f,.79f,.06f,-.98f,-.26f,-.11f,-.01f,-.63f,.17f,-.20f,.94f,.04f,.15f,-.41f,-.88f,.62f,-.03f,-.95f,-.99f,-.35f,-.79f,-.08f,.13f,.45f,-.79f,.30f,-.97f,.53f,-.87f,-.00f,.41f,-.90f,.46f,.14f,-.71f,-.34f,.06f,-.93f,-.31f,-.91f,.88f,.76f,-.69f,.51f,-.97f,.40f,-.95f,.75f,-.95f,-.33f,.74f,-.86f,.17f,.73f,.70f,-.41f,.11f,.91f,.94f,-.67f,.40f,.18f,.01f,.17f,-.10f,.24f,.54f,-.01f,.30f,.21f,.54f,-.37f,-.56f,.76f,-.57f,.23f,-.87f,.35f,.35f,.84f,-.47f,.32f,-.51f,.45f,.03f,.84f,.02f,.74f,.40f,-.56f,.08f,.36f,.97f,.25f,.03f,.48f,-.03f,.65f,.71f,-.04f,.85f,.05f,.13f,-.08f,.81f,.32f,-.34f,-.79f,-.45f,.77f,-.89f,.79f,-.23f,-.65f,-.43f,.55f,.99f,.68f,-.43f,.25f,-.19f,-.32f,-.03f,-.61f,.01f,.10f,.14f,.82f,.50f,.96f,.24f,-.44f,-.21f,.05f,-.28f,.58f,.29f,-.08f,.83f,-.95f,-.21f,-.49f,.61f,-.37f,-.47f,.50f,-.33f,-.83f,.00f,-.68f,.06f,-.96f,-.74f,.57f,.34f,-.54f,-.77f,-.73f,.84f,.81f,-.90f,.95f,.57f,-.79f,.35f,.54f,-.50f,-.86f,.13f,.16f,.75f,.64f,-.74f,-.72f,-.85f,.87f,-.71f,.43f,-.06f,-.69f,.12f,-.06f,.36f,-.34f,.79f,.02f,-.83f,.91f,.40f,.46f,.90f,-.72f,.02f,.01f,-.45f,.30f,-.12f,-.09f,-.14f,.55f,-.33f,.34f,-.82f,.90f,.45f,.95f,-.62f,-.49f,.06f,-.15f,.66f,.10f,-.92f,.71f,.04f,-.28f,-.83f,-.37f,.18f,.37f,.49f,.82f,-.53f,-.30f,.17f,-.65f,-.06f,.29f,-.89f,-.74f,-.21f,-.85f,.93f,-.51f,.29f,-.86f,-.20f,-.19f,.39f,-.51f,.54f,-.67f,.27f,-.23f,.23f,.43f,.65f,.71f,.95f,-.84f,.84f,-.44f,-.78f,.05f,.84f,.05f,-.54f,-.18f,-.41f,-.80f,.05f,-.41f,-.97f,.34f,.64f,-.94f,.19f,.22f,.85f,.58f,.92f,-.07f,-.41f,.30f,-.01f,.72f,-.28f,.91f,.82f,.92f,.17f,-.08f,-.69f,-.92f,.05f,.45f,-.72f,-.34f,.36f,.35f,-.18f,.55f,-.21f,-.69f,-.60f,.86f,.85f,.47f,.48f,.49f,-.31f,.03f,-.42f,-.28f,.38f,-.94f,.82f,.25f,-.40f,-.95f,.81f,-.21f,.50f,-.55f,-.02f,-.91f,-.10f,.45f,-.30f,.31f,.80f,-.57f,.41f,-.83f,.28f,-.02f,.11f,-.45f,.84f,.03f,.14f,.76f,-.21f,-.04f,-.52f,-.55f,-.03f,-.98f,.48f,.91f,-.39f,.57f,.45f,.29f,-.05f,-.33f,-.65f,-.62f,.98f,-.35f,.61f,.81f,.61f,-.51f,.43f,-.36f,.73f,-.20f,-.18f,.05f,.04f,-.83f,.38f,-.98f,-.62f,.07f,.65f,-.23f,.49f,-.33f,-.85f,.33f,-.51f,-.08f,-.21f,.66f,.84f,-.36f,-.01f,.99f,.48f,-.79f,.18f,.82f,.11f,-.38f,-.49f,.24f,.20f,.16f,.60f,-.33f,.58f,.77f,-.43f,-.65f,.81f,.14f,.36f,.00f,-.87f,-.57f,-.01f,-.02f,.30f,.14f,-.17f,.18f,.17f,.64f,.64f,.57f,-.72f,.12f,-.33f,.73f,.54f,-.36f,.16f,.62f,.01f,.09f,.36f,-.65f,-.72f,-.02f,-.40f,.55f,-.54f,-.20f,.24f,.97f,-.31f,.96f,.74f,.71f,.10f,.79f,-.64f,.17f,-.56f,.27f,-.62f,-.65f,.73f,.17f,-.06f,.67f,-.39f,-.46f,.25f,.64f,-.22f,-.13f,.35f,.70f,-.70f,-.60f,-.14f,-.53f,-.09f,.59f,-.09f,-.12f,-.28f,-.65f,.69f,.48f,.11f,-.82f,-.15f,-.01f,.06f,.96f,-.14f,.51f,.39f,.66f,-.27f,-.52f,.54f,-.87f,-.18f,-.69f,-.19f,-.20f,.19f,-.06f,.27f,.18f,-.26f,-.45f,-.09f,.47f,-.15f,-.05f,.54f,.80f,.86f,-.60f,-.02f,.74f,.07f,-.10f,-.73f,.56f,.67f,.51f,.23f,.69f,.85f,.21f,-.92f,.15f,.53f,.12f,.43f,-.26f,-.50f,.29f,.92f,-.29f,-.25f,.94f,-.92f,.53f,.14f,.93f,.88f,.09f,-.29f,.59f,.98f,.49f,.64f,-.18f,-.75f,-.26f,-.47f,.64f,.87f,.46f,-.51f,.04f,-.46f,.00f,-.67f,-.78f,.05f,-.35f,.21f,-.42f,-.85f,.02f,.78f,.86f,-.93f,-.28f,.54f,-.41f,-.83f,.95f,-.72f,.31f,.67f,-.22f,-.07f,.74f,.26f,.27f,-.78f,-.23f,.57f,.55f,-.27f,.17f,.57f,.63f,-.05f,-.76f,-.25f,.37f,-.29f,-.48f,.92f,-.87f,-.08f,-.68f,.61f,.24f,-.48f,-.70f,.61f,-.01f,-.96f,-.54f,.08f,-.75f,.06f,.60f,-.89f,.34f,.76f,-.25f,.34f,-.94f,.00f,.60f,-.24f,-.10f,-.98f,-.56f,-.74f,-.36f,.27f,.74f,.88f,-.94f,.25f,-.85f,-.39f,-.37f,.25f,-.80f,.10f,.52f,-.53f,.20f,.59f,-.17f,-.57f,.45f,.39f,-.63f,.37f,.45f,-.81f,-.16f,.18f,.66f,-.23f,-.98f,-.81f,.47f,.18f,.24f,.38f,-.79f,-.73f,.28f,-.81f,.39f,-.15f,.93f,-.83f,-.42f,-.70f,-.09f,-.95f,.23f,.25f,-.44f,-.79f,-.68f,-.26f,-.04f,.05f,-.39f,-.23f,.22f,-.25f,-.30f,-.00f,-.03f,-.42f,.94f,.27f,.98f,-.32f,.88f,.49f,.75f,.76f,.02f,-.10f,.26f,-.54f,-.00f,-.91f,.30f,.48f,.30f,-.54f,.58f,.13f,.95f,.43f,-.90f,-.79f,.82f,-.03f,.28f,-.69f,.87f,-.35f,-.34f,.97f,-.17f,.03f,.61f,.32f,-.05f,.47f,-.35f,-.48f,-.50f,.37f,.84f,.50f,-.82f,.26f,.47f,-.36f,-.42f,.58f,.71f,.87f,.59f,.92f,.31f,.66f,-.28f,-.13f,-.19f,-.95f,.57f,-.81f,.95f,.16f,.62f,-.60f,-.03f,.75f,.40f,.40f,.78f,.95f,.59f,-.00f,.79f,.49f,-.76f,.91f,-.76f,-.74f,-.68f,.97f,.98f,-.97f,.91f,-.93f,.47f,.86f,.88f,-.56f,.84f,-.00f,.02f,-.23f,.95f,.58f,-.00f,.84f,-.65f,.77f,-.27f,-.42f,-.54f,.19f,-.77f,-.59f,.81f,.45f,.96f,.39f,-.46f,-.74f,-.59f,.93f,-.12f,.26f,-.74f,.28f,-.83f,.58f,.20f,.25f,-.39f,.85f,.29f,.90f,-.42f,.95f,.49f,-.11f,-.66f,.65f,.91f,-.98f,-.61f,-.72f,.20f,.25f,.39f,.76f,-.13f,.56f,.78f,.90f,.10f,.54f,.49f,-.85f,.50f,.91f,-.72f,-.00f,.80f,-.25f,.07f,-.18f,.87f,-.93f,.02f,.03f,-.15f,.37f,.50f,-.21f,-.17f,.78f,-.40f,.70f,-.96f,-.74f,.05f,.22f,-.04f,.78f,.99f,.65f,.63f,.66f,.40f,.53f,-.91f,.46f,.43f,.18f,.65f,-.58f,.73f,.60f,.57f,.72f,.71f,.15f,.70f,-.47f,-.82f,-.38f,.89f,-.91f,.72f,.13f,-.81f,-.79f,-.58f,-.13f,.28f,.79f,.46f,.34f,.27f,.26f,.00f,.39f,.89f,.24f,.83f,.91f,-.87f,-.49f,.30f,-.11f,.77f,-.97f,-.36f,-.41f,-.68f,.85f,.79f,.30f,.81f,-.57f,.91f,-.04f,-.22f,-.16f,.95f,.60f,.27f,-.73f,.26f,-.18f,.50f,-.79f,.10f,-.69f,-.23f,.07f,.76f,-.41f,-.13f,-.64f,-.15f,.47f,-.03f,.93f,.95f,-.67f,.72f,-.80f,-.17f,.98f,.45f,-.91f,.94f,-.67f,.51f,.81f,-.37f,.49f,.43f,.26f,.83f,-.88f,-.10f,.05f,.41f,-.13f,-.32f,.75f,.37f,-.26f,-.42f,-.41f,.79f,.37f,-.06f,.77f,-.61f,.23f,.57f,.28f,.63f,-.43f,-.09f,.22f,-.86f,-.07f,.22f,.48f,-.05f,.71f,-.98f,.47f,.46f,.02f,-.52f,-.24f,.35f,-.02f,-.30f,-.80f,-.18f,-.81f,.19f,-.74f,-.04f,-.58f,-.51f,-.71f,-.28f,.23f,-.51f,-.05f,.88f,-.55f,.54f,.41f,-.06f,-.58f,.32f,.93f,-.14f,-.61f,-.05f,.13f,-.73f,-.19f,-.49f,.59f,.53f,.97f,.64f,.02f,.39f,-.95f,-.87f,-.87f,.96f,-.90f,-.68f,.74f,-.76f,-.08f,-.19f,-.11f,-.32f,.95f,-.13f,.70f,-.81f,.67f,.09f,-.64f,-.24f,.70f,.46f,.28f,.21f,-.10f,.49f,-.22f,.64f,.34f,-.10f,-.78f,-.11f,-.04f,-.31f,-.63f,-.45f,.12f,.71f,.88f,-.41f,.30f,.51f,-.73f,.54f,-.00f,.39f,.62f,.81f,.65f,-.26f,-.35f,.84f,-.12f,.77f,.65f,.77f,-.42f,.54f,.95f,-.82f,.46f,.83f,.15f,-.11f,-.06f,.41f,.87f,-.74f,-.24f,.37f,.94f,-.42f,-.56f,-.01f,-.15f,.03f,.49f,-.75f,.18f,.42f,.54f,.39f,.70f,-.49f,.08f,.68f,-.25f,.36f,.38f,.44f,-.52f,.86f,-.02f,.69f,.90f,.89f,-.77f,.14f,-.78f,-.64f,.40f,-.09f,.78f,-.81f,.94f,.46f,-.53f,.04f,-.46f,.71f,-.85f,.66f,-.51f,-.09f,.91f,.38f,-.26f,.91f,-.04f,-.86f,-.03f,.83f,-.69f,.35f,.65f,-.03f,.65f,.68f,.56f,.30f,-.46f,.73f,.02f,-.21f,.85f,.48f,.91f,.05f,.15f,-.32f,.39f,-.33f,-.84f,-.66f,-.41f,.98f,-.03f,.56f,.30f,.37f,-.86f,.60f,.47f,-.24f,.53f,-.85f,.66f,.53f,-.31f,-.44f,.88f,.34f,-.73f,-.48f,-.33f,-.73f,.56f,.45f,.18f,-.76f,.56f,.23f,-.83f,.97f,-.48f,-.80f,-.66f,.17f,.74f,.39f,-.02f,.49f,-.33f,-.73f,-.81f,.27f,.21f,.38f,-.68f,-.20f,-.09f,-.96f,-.61f,-.93f,-.99f,-.16f,.15f,.62f,.24f,.74f,-.65f,.35f,.26f,.38f,-.64f,-.67f,.80f,.26f,.30f,-.56f,.20f,-.82f,.06f,.81f,.75f,.82f,-.82f,.55f,-.91f,.37f,.16f,-.03f,-.13f,.09f,-.67f,-.33f,-.18f,-.13f,-.98f,-.81f,-.72f,.49f,-.96f,-.22f,.65f,.35f,-.35f,-.60f,-.64f,.19f,-.57f,.73f,-.70f,-.89f,.46f,-.77f,-.32f,.55f,-.81f,-.61f,.97f,-.49f,.30f,.42f,.66f,.19f,.55f,-.72f,-.95f,-.65f,-.28f,-.75f,-.22f,.29f,.56f,-.02f,-.15f,-.80f,.68f,-.05f,-.35f,.68f,-.84f,.95f,-.14f,.04f,.64f,.49f,.02f,.74f,.57f,.58f,.01f,.62f,-.87f,.76f,.37f,.27f,-.51f,.07f,.31f,.39f,.61f,.60f,-.14f,.62f,.01f,-.10f,-.43f,-.23f,-.91f,.99f,.58f,.81f,-.39f,-.98f,.96f,.02f,.38f,-.94f,-.46f,-.02f,.46f,.48f,-.39f,-.46f,.89f,-.62f,.10f,.81f,.53f,-.96f,-.33f,-.58f,-.17f,-.73f,.12f,.05f,.63f,-.83f,.01f,-.02f,-.00f,-.01f,-.79f,.78f,.48f,-.44f,.46f,.48f,-.72f,-.53f,.08f,.00f,-.12f,-.93f,-.87f,-.32f,-.49f,.35f,-.63f,-.20f,.99f,.01f,-.23f,.41f,-.32f,.12f,.85f,.31f,.43f,.21f,.60f,-.33f,-.71f,-.84f,-.23f,-.06f,-.23f,.08f,.89f,.99f,.64f,.65f,-.55f,.60f,-.07f,-.62f,.74f,.57f,.85f,.27f,.56f,-.87f,.00f,-.37f,-.67f,.07f,.49f,.44f,-.92f,.48f,.70f,-.86f,.33f,.39f,.71f,.43f,.80f,-.52f,.01f,-.54f,-.95f,-.39f,.40f,-.75f,.08f,.76f,-.62f,-.39f,.94f,.31f,.09f,-.15f,.83f,-.77f,.05f,.85f,.68f,-.32f,.94f,-.09f,-.57f,.46f,-.01f,-.77f,.68f,.44f,.01f,-.38f,.82f,-.22f,.74f,.75f,-.43f,-.26f,.01f,.87f,.15f,-.93f,-.51f,-.23f,-.57f,.88f,.76f,-.39f,.78f,-.40f,.78f,-.86f,-.15f,.47f,-.75f,-.26f,-.54f,-.00f,-.42f,.40f,.53f,-.34f,.26f,-.20f,-.09f,-.89f,.96f,.44f,-.56f,-.58f,-.40f,-.69f,.21f,-.74f,.42f,.94f,.95f,-.01f,.93f,.22f,-.40f,-.56f,.42f,-.97f,-.72f,.59f,-.69f,.52f,.62f,-.13f,-.47f,.18f,.00f,.37f,.58f,-.61f,.28f,-.85f,.52f,-.69f,-.62f,.54f,-.90f,.89f,.86f,-.83f,-.57f,.32f,.43f,.35f,-.35f,.02f,.93f,.77f,.32f,.52f,.58f,.76f,.35f,-.55f,-.10f,-.07f,-.82f,-.40f,.25f,.91f,.81f,.60f,-.58f,.64f,.20f,.97f,.61f,-.57f,.93f,.69f,-.42f,.99f,-.80f,.59f,.48f,.13f,.42f,-.78f,-.83f,.72f,.49f,.56f,-.69f,-.89f,.95f,-.45f,-.15f,-.39f,.12f,-.85f,-.85f,-.81f,-.63f,.33f,-.00f,.14f,-.46f,-.13f,.44f,.97f,-.03f,.58f,.91f,-.34f,.56f,.79f,.36f,.34f,.00f,-.16f,.98f,-.32f,-.91f,.51f,.65f,-.71f,-.53f,.00f,.41f,.83f,-.42f,-.52f,-.82f,.46f,.20f,-.61f,.50f,.82f,.43f,.05f,-.29f,.92f,.87f,-.57f,-.56f,.10f,.06f,-.95f,-.26f,.35f,-.83f,-.41f,.85f,-.43f,.61f,-.71f,.70f,.70f,-.50f,-.94f,.89f,-.78f,-.64f,.66f,-.37f,-.24f,-.74f,-.40f,.62f,-.01f,.11f,.47f,-.85f,-.10f,-.11f,-.91f,.48f,.43f,.01f,.71f,.66f,.15f,-.33f,-.07f,-.92f,-.77f,-.44f,.33f,-.89f,-.52f,.46f,-.68f,.37f,-.54f,.11f,-.08f,.60f,-.13f,-.01f,-.38f,-.31f,-.32f,.10f,-.58f,-.12f,-.62f,.97f,-.59f,-.26f,-.26f,.08f,-.59f,.91f,.33f,-.12f,-.22f,-.08f,-.86f,.02f,.65f,.56f,-.13f,.34f,.17f,.59f,.62f,.70f,.57f,.73f,-.86f,-.21f,.64f,-.23f,-.68f,.92f,.75f,-.05f,-.63f,-.61f,.52f,.77f,-.92f,-.71f,.73f,-.62f,.32f,-.38f,.99f,-.58f,-.77f,.58f,-.25f,.08f,.87f,.12f,.03f,.60f,-.07f,-.31f,-.70f,.34f,.71f,-.94f,-.72f,.37f,.18f,.71f,.17f,-.75f,-.90f,.24f,-.53f,-.05f,.68f,-.19f,.62f,-.62f,.67f,.02f,.78f,-.85f,.24f,-.57f,.02f,-.31f,-.23f,-.95f,-.78f,-.51f,.46f,.33f,-.06f,-.05f,.75f,-.07f,-.69f,-.55f,-.79f,.39f,-.28f,.92f,-.47f,-.34f,-.04f,.32f,-.35f,.97f,-.45f,.65f,.51f,-.90f,-.87f,.48f,.40f,.46f,.82f,-.02f,-.55f,.85f,.12f,-.10f,.65f,-.23f,.99f,.91f,-.02f,-.89f,-.67f,-.92f,.06f,-.16f,-.97f,-.17f,.11f,.37f,.59f,-.85f,-.43f,-.24f,-.78f,-.35f,.07f,-.26f,.65f,-.86f,-.29f,-.82f,-.03f,.25f,-.23f,.51f,-.61f,.68f,.88f,-.35f,-.17f,.36f,.36f,.80f,-.80f,-.52f,.90f,.65f,.82f,-.19f,-.10f,.62f,.87f,-.11f,-.67f,-.24f,.28f,.80f,.69f,.36f,.06f,.02f,-.15f,-.85f,-.94f,.18f,-.67f,-.03f,.58f,.95f,.70f,.04f,-.66f,-.21f,.77f,-.87f,-.96f,-.86f,-.96f,-.33f,.46f,-.08f,-.15f,.24f,.83f,-.10f,-.90f,-.47f,.56f,-.10f,.96f,-.13f,.12f,.15f,-.14f,-.88f,-.27f,.86f,-.51f,-.00f,.13f,.56f,.00f,.03f,-.64f,.05f,.44f,.85f,.15f,-.70f,.26f,-.57f,-.59f,-.90f,-.42f,.46f,.90f,-.47f,-.05f,-.29f,.77f,-.47f,.63f,-.72f,.24f,-.90f,-.96f,.61f,.74f,-.90f,.12f,-.92f,-.76f,.34f,-.53f,-.50f,.75f,.78f,.13f,.39f,.10f,.57f,-.10f,.00f,.61f,-.29f,.92f,-.89f,.70f,-.25f,.01f,.56f,.17f,-.51f,.78f,.03f,-.45f,.37f,-.65f,.65f,-.18f,.28f,-.04f,-.68f,-.48f,.06f,.94f,.56f,.92f,.96f,-.46f,-.69f,.89f,-.43f,-.68f,.87f,-.73f,.93f,.02f,.99f,.41f,-.78f,.06f,-.56f,.04f,.68f,-.79f,.26f,.39f,.92f,-.71f,-.17f,-.36f,-.48f,-.17f,.97f,-.94f,-.46f,-.84f,-.79f,.40f,.20f,-.71f,-.70f,.28f,-.27f,-.63f,-.39f,-.36f,.35f,-.69f,-.94f,-.17f,-.97f,-.07f,-.78f,.84f,.17f,.54f,.29f,.76f,.73f,-.58f,-.03f,.27f,.44f,-.03f,-.83f,-.04f,-.93f,-.42f,.82f,-.01f,-.62f,-.01f,.61f,-.05f,-.69f,-.63f,-.02f,-.25f,-.27f,.86f,.48f,-.92f,.71f,.69f,-.93f,.23f,-.48f,.22f,.05f,-.00f,-.97f,.82f,-.47f,.53f,.68f,.65f,-.13f,-.29f,-.74f,.66f,-.47f,-.01f,-.62f,.18f,.78f,.35f,.34f,.34f,-.20f,.65f,.40f,-.78f,-.11f,.81f,-.36f,.74f,-.83f,-.66f,-.26f,.90f,.02f,.55f,-.97f,.54f,.50f,.25f,.57f,-.49f,-.15f,-.45f,.50f,-.79f,-.90f,-.91f,.31f,.97f,.77f,.76f,.73f,.40f,-.04f,-.25f,.71f,.44f,.71f,.59f,.88f,.39f,-.23f,.99f,.94f,.65f,.52f,-.55f,-.30f,-.81f,.21f,.37f,.47f,.26f,-.07f,-.08f,.62f,.99f,-.52f,-.04f,.75f,-.67f,-.23f,-.16f,-.25f,.50f,-.17f,-.09f,.16f,-.92f,.62f,.70f,.28f,.00f,.87f,-.73f,.97f,-.31f,.94f,.56f,-.00f,.37f,.05f,-.70f,-.03f,.80f,.95f,.08f,-.34f,.85f,.88f,.95f,.88f,.14f,-.22f,.94f,.47f,-.85f,-.07f,.67f,.55f,.74f,.87f,-.39f,.99f,.49f,-.14f,-.20f,.79f,-.88f,.79f,-.14f,.40f,.57f,-.32f,.47f,-.93f,-.97f,-.18f,.71f,-.96f,-.50f,-.65f,.69f,-.26f,-.94f,-.07f,-.20f,.55f,-.84f,.14f,.88f,-.94f,-.62f,.42f,-.49f,.70f,-.61f,-.36f,-.92f,-.96f,-.42f,-.15f,-.55f,.51f,.56f,-.03f,.16f,.27f,.06f,-.46f,-.54f,-.74f,.48f,.79f,-.30f,-.31f,.37f,-.63f,-.53f,.65f,-.54f,-.68f,.10f,.19f,-.51f,.68f,-.81f,.20f,.38f,.83f,.14f,.81f,.29f,.95f,-.57f,.83f,-.25f,-.26f,.10f,-.23f,.00f,-.77f,-.47f,-.36f,-.33f,-.38f,.48f,.15f,-.61f,-.07f,-.35f,-.52f,.46f,-.51f,-.56f,.32f,-.51f,-.87f,-.76f,.95f,-.88f,-.53f,-.51f,.18f,.95f,-.72f,-.06f,.85f,.07f,-.84f,.90f,-.19f,.14f,.54f,-.84f,-.57f,-.38f,.23f,.45f,.66f,.73f,-.87f,-.29f,.25f,-.74f,-.87f,-.54f,.08f,-.26f,-.15f,.40f,-.62f,-.73f,-.38f,.13f,.37f,.20f,.61f,-.78f,.84f,.99f,.80f,-.99f,.52f,.85f,-.61f,-.74f,-.52f,.03f,-.17f,-.91f,.64f,.61f,.82f,.49f,-.90f,.13f,.92f,.63f,-.38f,.40f,-.12f,-.46f,-.66f,.45f,-.68f,-.32f,-.41f,.97f,-.78f,-.82f,-.93f,-.19f,.33f,-.56f,-.21f,-.37f,.61f,-.37f,-.59f,.50f,.88f,.39f,.11f,.66f,-.67f,.49f,-.07f,.51f,-.20f,.65f,-.63f,-.87f,.96f,-.58f,-.24f,.29f,-.67f,-.39f,-.18f,.48f,.79f,.97f,.24f,.07f,.19f,.54f,-.45f,-.34f,-.13f,.43f,-.55f,-.31f,-.55f,.77f,-.52f,-.20f,.30f,.81f,.74f,.63f,.01f,-.70f,.17f,.50f,.34f,.73f,.01f,-.74f,.37f,.88f,-.40f,.39f,-.07f,-.03f,.14f,.89f,.70f,-.49f,-.41f,-.66f,.20f,.95f,.34f,.69f,-.01f,-.63f,.57f,.60f,.99f,.49f,-.45f,.77f,-.26f,-.83f,.29f,.69f,-.26f,.97f,-.25f,-.96f,.68f,-.17f,.99f,-.61f,.40f,-.56f,.50f,.07f,.74f,-.16f,-.44f,.72f,.50f,.02f,.74f,.95f,-.14f,-.40f,.82f,-.82f,-.58f,.06f,.77f,-.26f,.63f,.17f,-.44f,-.08f,.72f,.16f,-.91f,-.56f,.86f,.34f,.70f,.85f,-.93f,.99f,.33f,.32f,.23f,-.40f,-.96f,.07f,.01f,-.81f,-.39f,-.82f,-.50f,-.15f,.52f,.34f,-.96f,-.28f,.16f,.72f,-.91f,.03f,-.22f,.27f,-.53f,.27f,-.53f,.95f,.48f,.22f,.61f,-.30f,.67f,-.22f,-.06f,-.56f,-.74f,.98f,-.97f,.15f,.97f,-.71f,.53f,-.09f,.58f,-.80f,.85f,-.96f,-.50f,.22f,-.43f,-.73f,.36f,-.27f,.46f,.36f,.92f,.02f,.00f,.55f,-.00f,-.68f,-.39f,-.67f,-.58f,-.50f,.39f,-.18f,.48f,-.66f,.05f,-.38f,-.48f,.42f,-.72f,.30f,.30f,.13f,-.52f,-.47f,-.07f,-.92f,-.28f,.25f,-.19f,.83f,-.30f,.66f,.57f,-.98f,-.47f,.03f,.22f,-.55f,.14f,-.04f,-.81f,.99f,-.48f,-.36f,.24f,.08f,-.82f,-.74f,-.68f,-.12f,-.57f,-.55f,-.43f,-.57f,-.37f,-.84f,-.83f,.44f,.26f,-.04f,.03f,-.58f,-.84f,.78f,-.30f,-.00f,-.82f,.12f,-.00f,.13f,.49f,-.26f,.11f,.50f,-.44f,-.87f,-.15f,-.22f,-.21f,-.76f,-.79f,.80f,.97f,.97f,.06f,.87f,.13f,-.22f,.49f,-.11f,-.36f,-.12f,-.39f,.43f,.17f,.54f,-.17f,.93f,-.21f,.80f,.60f,-.40f,.02f,.85f,.32f,-.52f,-.62f,.54f,.01f,-.93f,.07f,.97f,-.46f,-.94f,-.94f,-.93f,-.41f,.80f,-.54f,-.23f,-.08f,.60f,-.56f,.26f,.12f,-.48f,.53f,.90f,.93f,-.84f,.71f,-.60f,.18f,.52f,-.59f,-.74f,.36f,-.74f,-.25f,-.77f,.13f,-.88f,.88f,-.23f,-.29f,.82f,.26f,.57f,-.29f,.07f,-.40f,.99f,-.89f,.56f,-.19f,.95f,.50f,-.70f,.31f,.75f,.63f,.70f,-.46f,-.34f,.88f,.03f,.56f,.71f,.34f,.17f,-.79f,-.78f,.47f,.14f,.00f,.49f,.10f,.42f,.35f,-.39f,.02f,-.02f,.19f,.36f,-.71f,.30f,-.58f,-.42f,-.23f,.24f,.17f,-.42f,.77f,-.88f,.07f,-.33f,.20f,-.05f,.37f,-.74f,.87f,.00f,-.23f,-.48f,-.95f,-.97f,.12f,-.86f,.58f,-.92f,.62f,.72f,-.53f,.03f,.70f,.26f,.95f,-.44f,.91f,.09f,-.31f,.53f,.90f,.92f,-.68f,.75f,-.85f,-.04f,-.42f,-.77f,-.67f,.94f,-.57f,-.93f,-.43f,-.74f,.95f,-.53f,-.50f,.69f,.93f,.61f,-.62f,.66f,.44f,-.98f,.54f,.33f,-.61f,-.95f,.92f,-.95f,.63f,-.25f,-.36f,-.70f,-.57f,-.36f,-.42f,.34f,-.01f,.27f,-.76f,-.22f,.46f,-.89f,-.02f,.57f,.82f,-.58f,-.52f,.49f,.02f,-.04f,.73f,-.47f,-.08f,.47f,-.98f,.94f,.82f,-.90f,-.88f,-.82f,.04f,.25f,.84f,-.26f,.19f,-.43f,.32f,-.66f,.25f,.44f,.47f,-.69f,-.14f,.64f,-.15f,-.50f,-.12f,.29f,-.08f,.83f,.48f,.07f,-.51f,.24f,-.81f,-.65f,-.54f,.12f,.52f,.15f,-.55f,-.94f,.47f,-.32f,.04f,.17f,-.13f,-.13f,.46f,.13f,-.85f,-.55f,.41f,-.90f,.84f,-.17f,-.73f,.29f,-.70f,.16f,.38f,.39f,-.93f,.96f,-.45f,.54f,-.76f,-.07f,-.02f,.84f,.91f,-.09f,.94f,-.19f,.88f,-.93f,.74f,.19f,-.39f,.11f,-.36f,-.57f,.01f,-.71f,-.67f,-.54f,.54f,-.22f,.06f,-.67f,.68f,-.45f,.64f,.46f,.69f,.75f,.94f,-.40f,-.05f,-.89f,-.07f,-.47f,.03f,-.31f,.54f,-.88f,.90f,-.37f,.01f,-.23f,-.96f,.12f,-.42f,.91f,-.62f,-.06f,.78f,.63f,-.72f,.28f,.11f,.11f,.15f,.73f,-.91f,.36f,.42f,.25f,-.36f,-.27f,.42f,-.51f,-.44f,.87f,.33f,.47f,.84f,.05f,.37f,-.60f,-.66f,.83f,.84f,.43f,.15f,-.90f,-.48f,-.59f,-.98f,.80f,-.43f,-.01f,.43f,.62f,-.52f,.36f,-.55f,.14f,.08f,-.32f,.65f,-.51f,.26f,.69f,-.71f,-.69f,.28f,-.66f,-.91f,-.15f,.47f,.34f,.56f,-.66f,.26f,.56f,.21f,-.67f,.85f,.30f,.83f,.17f,-.30f,.09f,.23f,.64f,.26f,.78f,-.09f,-.37f,-.45f,-.74f,-.96f,-.13f,.16f,-.94f,-.78f,.75f,.40f,.83f,.99f,-.91f,-.26f,.66f,-.55f,.89f,.86f,-.64f,-.06f,.33f,-.09f,-.87f,.63f,-.40f,.91f,-.89f,.76f,-.39f,-.93f,.77f,.61f,-.10f,-.78f,-.79f,-.93f,.44f,.58f,.16f,-.39f,-.97f,-.90f,.01f,-.36f,.70f,.26f,.90f,-.65f,.94f,-.19f,-.00f,.26f,-.39f,.25f,-.27f,.68f,.06f,.11f,.88f,.73f,.17f,.83f,-.39f,-.16f,-.43f,-.16f,.59f,.68f,.45f,.15f,.53f,-.56f,.99f,-.46f,-.50f,-.47f,-.63f,-.07f,.32f,.09f,.40f,-.42f,-.66f,.89f,-.73f,-.62f,.43f,.67f,-.31f,.65f,-.04f,-.37f,.77f,.10f,-.78f,.67f,.29f,-.37f,-.94f,-.00f,-.79f,-.15f,.08f,-.53f,.88f,.21f,.73f,.17f,-.13f,.93f,.86f,-.65f,-.12f,-.69f,.57f,.23f,-.62f,.03f,-.47f,.02f,.20f,-.42f,.97f,.85f,-.16f,-.87f,-.78f,.50f,.34f,-.81f,.87f,.29f,.61f,-.82f,-.50f,-.86f,.56f,-.14f,-.50f,-.35f,-.55f,-.37f,.37f,-.55f,.59f,-.27f,.47f,-.88f,.78f,.09f,-.61f,-.20f,-.81f,.10f,.11f,-.93f,-.80f,.74f,.65f,.27f,-.97f,.97f,-.04f,.49f,-.77f,.36f,.82f,.09f,-.32f,.08f,.67f,.08f,.48f,-.02f,-.07f,.55f,-.47f,.03f,-.17f,-.72f,-.05f,.79f,.60f,-.86f,.68f,.07f,.77f,-.60f,.83f,.47f,.65f,-.95f,-.00f,.71f,.97f,.63f,.75f,.83f,-.08f,-.89f,-.46f,-.46f,-.51f,.69f,.83f,.33f,.25f,.36f,-.47f,-.57f,.36f,-.15f,.03f,.72f,-.91f,-.46f,-.45f,-.08f,.21f,.39f,-.59f,.73f,.34f,-.42f,.35f,-.11f,-.38f,-.51f,.84f,-.07f,-.21f,-.58f,-.96f,-.91f,.00f,-.85f,-.29f,-.73f,-.45f,-.52f,-.13f,-.44f,-.79f,-.90f,.32f,.67f,.66f,.51f,-.57f,-.31f,.85f,-.46f,.91f,-.03f,-.13f,.81f,-.84f,.07f,.37f,-.51f,.73f,-.11f,.59f,-.38f,.64f,-.41f,-.80f,.72f,-.72f,-.70f,.50f,-.19f,-.42f,-.10f,.59f,.79f,-.69f,-.58f,.15f,.98f,.44f,.19f,-.18f,-.06f,.80f,-.27f,-.02f,-.02f,.25f,-.20f,.29f,-.80f,.39f,.89f,.99f,.47f,.43f,.71f,-.43f,-.26f,.62f,.10f,.79f,-.26f,.18f,.27f,-.75f,-.82f,-.09f,-.63f,.66f,-.29f,-.54f,-.21f,-.83f,-.42f,.85f,.69f,-.95f,-.85f,.04f,-.60f,.02f,-.00f,.19f,.01f,.89f,-.69f,.85f,-.70f,.46f,-.47f,-.43f,-.03f,.37f,-.66f,-.44f,.90f,.03f,-.07f,-.09f,.17f,-.42f,.16f,-.81f,-.05f,.56f,-.19f,-.23f,-.89f,-.20f,-.95f,-.47f,-.84f,.84f,-.91f,-.33f,-.69f,.53f,-.99f,.93f,.03f,-.88f,.03f,-.01f,.88f,.84f,-.71f,-.25f,-.04f,-.15f,-.92f,-.12f,-.74f,-.04f,.08f,-.61f,.18f,.93f,-.30f,-.81f,.94f,-.39f,-.41f,.21f,-.08f,-.97f,-.41f,.10f,-.44f,.14f,.30f,-.75f,.68f,-.09f,.43f,.49f,-.58f,.94f,-.01f,.26f,.75f,.77f,-.26f,.56f,.48f,-.50f,-.45f,.56f,-.13f,.53f,-.75f,-.18f,.05f,-.00f,.66f,.23f,-.50f,.57f,.37f,-.23f,.86f,-.05f,-.61f,-.22f,.72f,-.35f,.47f,-.50f,.78f,.81f,.05f,-.04f,-.05f,.41f,-.74f,.52f,.34f,-.02f,.30f,.91f,-.52f,-.66f,-.57f,-.49f,-.76f,-.29f,-.92f,.87f,-.26f,-.88f,-.78f,-.98f,-.61f,-.72f,.09f,.82f,.47f,-.19f,-.71f,.56f,.15f,-.07f,-.94f,.85f,.85f,.40f,-.13f,-.20f,.23f,-.02f,-.74f,.12f,.99f,-.44f,.34f,-.54f,.90f,-.95f,.54f,-.27f,.36f,.78f,-.86f,-.21f,.17f,.28f,-.15f,.66f,-.43f,.64f,.70f,-.95f,.27f,.71f,.27f,-.99f,-.11f,-.56f,.21f,-.84f,-.64f,-.26f,.05f,.31f,.56f,-.41f,-.77f,-.32f,.72f,.25f,.81f,.06f,-.92f,.89f,-.23f,.42f,.53f,-.15f,.11f,.88f,-.36f,-.43f,-.68f,.37f,-.62f,-.77f,-.03f,-.93f,.50f,.57f,-.90f,-.00f,.82f,.61f,-.62f,.38f,-.06f,-.32f,.31f,.66f,-.25f,.90f,.15f,.12f,.60f,.82f,-.86f,.54f,-.99f,.05f,.92f,-.75f,-.26f,-.32f,-.18f,.41f,.38f,-.32f,.27f,.08f,.74f,-.71f,-.26f,.34f,.99f,.46f,.11f,-.41f,-.09f,.19f,-.83f,.43f,.12f,.74f,.31f,-.35f,-.76f,.13f,.05f,.65f,.32f,-.62f,-.34f,-.37f,.94f,-.18f,.03f,-.30f,-.91f,.95f,.76f,.67f,.85f,.15f,.84f,-.21f,.87f,.83f,.56f,-.67f,.72f,.05f,-.70f,-.28f,-.87f,-.25f,.95f,.34f,-.01f,.20f,-.72f,.57f,.06f,.99f,.05f,.36f,.77f,-.89f,-.52f,-.65f,-.95f,-.49f,-.62f,.12f,.17f,-.89f,.27f,-.72f,.09f,.94f,.46f,-.39f,-.27f,-.34f,.05f,-.43f,.76f,.59f,.41f,-.69f,.50f,.30f,.69f,.31f,.31f,.60f,-.81f,-.34f,-.31f,.32f,.72f,-.37f,.08f,-.61f,-.77f,-.86f,.13f,.52f,-.28f,-.43f,-.70f,.34f,.41f,-.85f,-.19f,.09f,.88f,-.40f,-.92f,-.62f,-.97f,.11f,-.54f,-.51f,-.99f,-.03f,.69f,.57f,-.56f,-.87f,.39f,.37f,.72f,.17f,-.28f,-.86f,.46f,-.77f,.49f,-.73f,-.65f,-.81f,-.78f,.02f,.22f,-.58f,.30f,-.95f,-.91f,.94f,-.73f,.67f,.45f,-.49f,.37f,.48f,-.76f,-.59f,.32f,.97f,.01f,.46f,-.42f,.58f,.26f,.62f,.58f,-.26f,-.52f,-.24f,-.72f,-.88f,.89f,.99f,.20f,.48f,.11f,.82f,-.51f,.46f,-.07f,.19f,.66f,-.22f,-.68f,.37f,.81f,.51f,-.81f,-.75f,-.51f,-.26f,-.13f,.47f,.59f,-.71f,-.43f,-.33f,.82f,-.15f,-.18f,.80f,-.09f,.19f,.75f,.20f,.31f,.28f,-.44f,-.43f,.18f,.53f,.76f,.15f,-.63f,-.39f,-.84f,.01f,-.05f,-.33f,.32f,.10f,.00f,.80f,.46f,-.78f,.84f,-.20f,.60f,.62f,.52f,-.79f,.21f,-.97f,-.94f,.45f,-.46f,-.97f,-.53f,-.37f,-.97f,-.68f,.09f,-.80f,.50f,.09f,.56f,.10f,.81f,.33f,.81f,-.32f,-.51f,-.00f,-.15f,.92f,.94f,-.79f,-.72f,.43f,-.04f,.42f,.21f,.99f,.39f,.06f,-.01f,-.59f,.18f,-.94f,-.50f,-.55f,-.13f,.32f,.94f,-.87f,.08f,-.72f,-.17f,-.37f,.21f,.01f,-.28f,.60f,.05f,-.99f,-.50f,-.74f,.32f,.49f,-.29f,.12f,-.52f,-.90f,-.68f,-.45f,.73f,.52f,.03f,-.71f,.39f,.92f,-.95f,.83f,-.60f,-.44f,-.66f,-.59f,-.76f,-.79f,-.10f,-.77f,.35f,.88f,-.49f,.08f,.06f,-.97f,-.84f,-.47f,.40f,.51f,.13f,-.06f,-.35f,.81f,-.99f,-.09f,-.92f,-.38f,-.78f,-.10f,.76f,-.08f,-.31f,-.59f,-.86f,-.34f,-.86f,.54f,.10f,.16f,-.01f,-.68f,-.22f,-.15f,.46f,.92f,-.75f,-.84f,-.62f,-.74f,.40f,.90f,-.41f,-.14f,-.73f,.27f,.86f,.53f,-.41f,.35f,.96f,.34f,.96f,-.83f,.88f,-.35f,.34f,-.71f,-.27f,-.28f,.38f,-.98f,-.30f,-.67f,.52f,-.03f,.63f,-.54f,.26f,.41f,-.01f,-.10f,-.38f,.61f,-.77f,.92f,.58f,-.98f,.93f,-.27f,-.08f,.78f,-.64f,-.93f,.79f,-.36f,.94f,-.10f,.48f,.73f,-.44f,.00f,-.94f,-.90f,.04f,-.72f,.09f,-.57f,.46f,-.62f,-.41f,.09f,-.21f,-.56f,.79f,.49f,-.78f,.43f,.61f,-.29f,-.10f,-.54f,.64f,-.95f,.35f,-.50f,.99f,.08f,-.07f,-.00f,.31f,.69f,.78f,.68f,-.34f,-.39f,.17f,.51f,-.25f,.71f,.83f,.58f,.87f,.79f,.14f,.26f,.54f,.26f,.06f,.85f,-.29f,-.79f,-.57f,-.02f,-.07f,.77f,.16f,.17f,-.35f,.63f,.61f,.89f,.52f,.14f,.77f,.57f,.80f,-.81f,-.93f,-.18f,-.32f,-.88f,.24f,-.25f,-.75f,.88f,.43f,.09f,-.08f,-.15f,.53f,-.93f,.02f,-.48f,.82f,-.81f,.86f,-.84f,.15f,.64f,-.20f,.92f,.10f,.26f,.75f,-.50f,.50f,-.75f,.48f,-.27f,-.39f,-.82f,-.71f,.60f,.99f,-.84f,.42f,-.01f,.39f,-.86f,-.06f,.66f,.70f,.17f,.75f,-.97f,.57f,-.44f,-.68f,.13f,-.20f,-.69f,.99f,.73f,.38f,-.51f,-.23f,.26f,-.85f,.57f,-.62f,.72f,.69f,.88f,-.09f,-.30f,-.30f,.37f,-.62f,-.74f,-.02f,-.33f,-.69f,.00f,-.53f,-.99f,-.39f,-.88f,.56f,.09f,.33f,.56f,.75f,.01f,-.68f,.87f,.80f,-.23f,-.48f,-.20f,.08f,.64f,.10f,-.14f,.13f,.50f,-.92f,.43f,-.61f,-.04f,-.90f,.65f,.76f,.58f,-.21f,.63f,.93f,-.61f,.01f,-.13f,.09f,.89f,.46f,.82f,.95f,.23f,-.86f,-.16f,.38f,.00f,.70f,-.42f,.56f,-.55f,.05f,.88f,-.22f,-.51f,-.84f,.28f,-.03f,.74f,.53f,.40f,-.98f,-.52f,-.50f,-.67f,.84f,-.72f,-.78f,.14f,.08f,-.44f,-.23f,.38f,-.01f,.76f,.26f,.97f,.80f,-.84f,-.20f,-.19f,.51f,.42f,-.51f,-.81f,.69f,.10f,-.98f,-.62f,-.82f,.09f,-.43f,.54f,.47f,.43f,-.85f,.87f,-.86f,.24f,.23f,-.46f,-.33f,.86f,-.90f,.26f,-.42f,-.48f,-.12f,.55f,-.28f,-.49f,-.40f,.45f,.72f,.69f,.10f,-.20f,.02f,.65f,-.55f,.70f,.27f,-.11f,.44f,.87f,-.83f,.09f,.60f,.11f,.05f,-.33f,-.35f,.74f,.23f,-.91f,.35f,-.47f,.52f,-.70f,.56f,.24f,-.60f,-.60f,.42f,-.40f,-.30f,-.87f,.61f,-.53f,.61f,.85f,.00f,-.19f,-.10f,-.72f,-.07f,-.62f,.46f,-.16f,-.75f,-.96f,-.92f,.87f,-.95f,-.25f,-.40f,.63f,.45f,-.22f,-.49f,.27f,.05f,-.96f,-.89f,.43f,.38f,.50f,.09f,.78f,-.73f,.67f,.55f,-.53f,.89f,-.34f,.99f,-.29f,-.04f,-.42f,-.34f,-.74f,-.36f,.38f,.94f,.29f,-.11f,.10f,-.08f,-.21f,-.18f,.88f,-.64f,-.58f,-.18f,.77f,-.47f,-.53f,-.00f,.13f,-.41f,.71f,.52f,-.39f,-.25f,.15f,.09f,.49f,.99f,-.09f,.97f,.21f,-.24f,.76f,-.25f,.81f,-.44f,.96f,-.85f,.48f,-.86f,.12f,.94f,-.09f,-.69f,.98f,.26f,.63f,.65f,.70f,.95f,.96f,.49f,-.25f,.66f,-.97f,-.62f,.85f,.04f,.16f,.22f,.71f,-.52f,-.55f,.08f,-.85f,-.87f,.73f,.08f,.94f,-.34f,-.82f,.22f,-.25f,-.92f,-.96f,.94f,-.47f,.76f,-.83f,.22f,.42f,-.80f,-.22f,-.84f,-.52f,.15f,.20f,.96f,.20f,.63f,-.32f,-.76f,.32f,-.71f,-.68f,.55f,-.45f,.59f,-.41f,-.10f,-.62f,.56f,.07f,-.22f,-.90f,.39f,-.53f,.98f,-.55f,-.31f,-.48f,.22f,-.78f,-.57f,-.88f,-.97f,.26f,-.74f,.20f,-.82f,-.34f,.27f,.96f,-.14f,-.60f,-.20f,-.34f,.68f,-.00f,.57f,.87f,-.48f,.10f,.39f,-.22f,.97f,.45f,-.41f,.85f,-.18f,-.22f,-.40f,.67f,-.09f,-.61f,-.28f,.71f,.45f,-.27f,-.67f,.50f,-.10f,-.09f,-.75f,.96f,.86f,.27f,-.83f,-.35f,-.30f,.37f,.44f,.78f,-.50f,-.53f,-.40f,-.43f,-.04f,.46f,-.57f,-.60f,-.13f,.90f,.32f,.92f,.79f,.48f,-.76f,.87f,.11f,-.17f,-.50f,-.00f,.02f,.61f,.33f,.64f,-.80f,-.48f,.91f,.81f,-.73f,-.02f,.94f,-.63f,.19f,.39f,-.50f,-.91f,-.99f,.95f,-.75f,.67f,.82f,-.54f,-.76f,.50f,.90f,.46f,-.74f,.69f,-.86f,.72f,.52f,.39f,-.46f,-.36f,.34f,.21f,-.44f,.14f,.35f,.13f,-.84f,.95f,.32f,-.82f,-.62f,-.69f,-.50f,.59f,.47f,-.97f,.15f,.59f,.65f,-.26f,.50f,-.03f,-.96f,.97f,-.99f,-.67f,-.73f,-.48f,-.64f,-.84f,-.94f,.62f,.14f,-.32f,-.47f,-.74f,.07f,.05f,.87f,-.83f,.49f,-.32f,-.32f,.03f,.51f,.06f,-.77f,.75f,-.39f,-.78f,.10f,.95f,.38f,-.08f,-.66f,-.07f,.09f,-.09f,-.09f,.83f,-.12f,.43f,-.65f,.41f,.68f,.93f,.88f,-.24f,.70f,-.97f,-.40f,-.96f,-.45f,-.35f,-.29f,.91f,-.28f,.92f,-.57f,-.37f,-.02f,.37f,-.37f,-.01f,-.48f,.74f,-.28f,.21f,.74f,-.99f,.45f,.91f,-.28f,-.11f,-.86f,-.22f,.48f,-.34f,.52f,.17f,.04f,.23f,-.54f,-.80f,-.60f,.04f,.47f,-.14f,.33f,.57f,.22f,-.67f,.02f,.69f,.93f,.69f,.93f,.99f,.75f,.47f,.21f,-.42f,.02f,.20f,.03f,.12f,.76f,.04f,-.43f,-.10f,-.52f,-.26f,-.10f,.78f,-.81f,-.72f,-.76f,.87f,.88f,.83f,-.27f,.82f,-.97f,-.59f,.80f,.66f,.66f,.59f,-.73f,.03f,-.16f,.69f,-.52f,.22f,.82f,-.11f,-.25f,-.39f,.56f,.93f,-.43f,-.73f,-.28f,-.06f,-.49f,-.95f,.39f,.51f,.29f,.40f,-.46f,-.11f,.60f,.03f,-.57f,.24f,-.55f,.88f,-.33f,.64f,-.79f,.59f,-.05f,.94f,.00f,.74f,.44f,-.86f,.23f,-.03f,-.66f,.45f,.08f,-.53f,.09f,.98f,-.28f,-.10f,.89f,.18f,.45f,-.98f,-.89f,.47f,.78f,-.11f,.53f,-.35f,.46f,.00f,.49f,-.13f,-.58f,-.56f,.73f,.21f,.20f,.72f,-.71f,-.68f,-.54f,.61f,-.80f,.53f,.90f,.20f,.25f,-.29f,.91f,-.88f,.04f,.98f,-.18f,-.16f,.28f,.28f,.60f,.35f,-.32f,.48f,-.99f,-.57f,.41f,.93f,-.33f,-.69f,.68f,-.99f,.40f,-.31f,.58f,.70f,-.95f,-.97f,.59f,-.71f,-.43f,.78f,-.92f,-.01f,.79f,-.34f,-.14f,.25f,.86f,.62f,-.12f,-.53f,.77f,-.30f,-.50f,-.82f,.76f,.27f,-.59f,.69f,-.87f,-.03f,.82f,.05f,.58f,.96f,.52f,.04f,.80f,-.66f,-.06f,.88f,-.59f,-.19f,-.47f,.84f,.15f,.90f,-.08f,.69f,-.34f,.12f,.14f,.86f,-.56f,-.59f,.08f,.27f,-.13f,.00f,-.00f,-.47f,.34f,.44f,.17f,-.57f,-.19f,-.09f,-.31f,.03f,-.60f,.40f,.58f,-.43f,-.81f,.08f,.04f,-.28f,.74f,.00f,-.59f,-.57f,-.82f,.99f,-.36f,.44f,-.51f,.26f,.75f,-.56f,-.86f,.85f,-.63f,.49f,.81f,-.53f,.23f,-.35f,-.25f,-.44f,-.03f,.23f,.98f,-.37f,-.25f,-.60f,.81f,.16f,.95f,.77f,-.18f,.45f,-.35f,.47f,.74f,.65f,.81f,-.02f,-.39f,-.75f,.33f,.35f,-.76f,-.12f,-.62f,.06f,.47f,.47f,-.74f,.58f,-.68f,-.77f,-.12f,-.41f,.26f,.39f,-.37f,.47f,.81f,.00f,.80f,-.59f,.74f,-.11f,-.81f,-.10f,-.03f,.88f,-.65f,.58f,-.93f,-.09f,-.30f,.72f,-.75f,-.10f,.01f,.21f,-.17f,.37f,-.96f,-.44f,-.69f,.36f,.89f,.24f,.06f,.20f,-.51f,.20f,-.25f,.64f,.06f,-.91f,.19f,-.71f,-.09f,.85f,.49f,-.39f,.80f,.42f,.61f,-.52f,.61f,-.13f,.48f,.67f,.60f,.93f,-.02f,.58f,.01f,-.17f,-.50f,-.67f,-.74f,-.12f,.92f,-.61f,-.21f,.30f,-.70f,-.56f,-.91f,-.65f,.13f,-.36f,-.91f,.93f,.69f,-.95f,.67f,-.46f,.26f,.59f,.11f,.64f,-.83f,-.00f,-.85f,.50f,.25f,.80f,-.83f,-.54f,-.96f,.68f,-.39f,.84f,.80f,.84f,.86f,-.60f,-.73f,-.90f,.22f,.52f,-.70f,-.70f,.35f,.57f,.55f,-.94f,.51f,.07f,-.57f,-.20f,.16f,.66f,.71f,.46f,-.03f,.11f,-.61f,-.84f,-.57f,.12f,-.37f,-.97f,-.83f,.47f,.40f,.54f,.79f,.95f,.40f,.23f,.45f,.66f,.33f,.61f,.90f,-.66f,-.18f,.95f,.91f,-.89f,-.66f,.37f,-.80f,-.36f,.02f,.23f,-.57f,-.19f,-.11f,-.01f,-.47f,-.67f,-.64f,.80f,-.34f,-.08f,-.54f,.96f,-.41f,-.43f,.52f,.30f,-.02f,.68f,-.82f,-.06f,-.24f,.53f,-.12f,-.64f,-.57f,-.93f,.17f,-.62f,-.57f,-.18f,.44f,-.93f,.27f,.47f,.02f,-.66f,.84f,-.56f,-.86f,-.30f,.45f,-.87f,-.18f,-.78f,-.58f,-.87f,-.01f,-.39f,-.82f,-.88f,.73f,-.96f,-.08f,-.83f,-.78f,-.40f,.55f,-.96f,-.72f,-.48f,-.58f,.56f,.17f,.51f,.81f,-.72f,-.46f,.98f,.59f,-.78f,-.00f,-.97f,.94f,.53f,.29f,.90f,.85f,-.83f,-.66f,-.22f,-.23f,.41f,-.13f,-.10f,.66f,-.18f,.83f,-.50f,-.14f,-.22f,.75f,-.81f,.56f,-.60f,-.11f,-.24f,-.37f,-.44f,.27f,-.84f,.21f,-.47f,.87f,.91f,-.95f,-.09f,.97f,.08f,-.49f,-.27f,.22f,.50f,-.54f,.41f,.84f,.25f,.24f,.12f,.76f,-.50f,.78f,-.35f,.66f,-.78f,.10f,.81f,.32f,-.41f,-.20f,-.99f,.80f,-.91f,-.80f,.53f,.83f,-.43f,-.12f,.26f,-.28f,.33f,-.88f,.39f,.15f,-.11f,-.35f,.46f,-.47f,-.30f,.42f,-.33f,.95f,.59f,-.43f,-.12f,.57f,-.57f,-.47f,.60f,.34f,.80f,.80f,-.64f,-.72f,.01f,.49f,-.03f,.09f,.38f,-.65f,-.22f,.61f,-.82f,.42f,-.33f,.91f,.55f,.45f,.85f,-.06f,-.58f,.64f,.12f,-.57f,.68f,.24f,-.88f,-.67f,-.65f,-.22f,-.11f,-.83f,-.82f,.76f,.33f,-.14f,.97f,-.18f,-.04f,.97f,.00f,.08f,.15f,.52f,-.19f,-.46f,-.50f,-.59f,.69f,.06f,-.14f,.27f,-.75f,.27f,-.14f,.15f,-.57f,-.42f,-.18f,.43f,.44f,-.65f,-.87f,-.49f,-.37f,.82f,-.72f,.01f,-.88f,.14f,-.92f,-.50f,.83f,-.14f,-.16f,-.50f,-.05f,-.63f,-.90f,.83f,.74f,.00f,.44f,-.66f,.93f,.27f,-.28f,.18f,-.80f,-.54f,-.08f,.22f,.43f,.16f,-.25f,-.14f,.25f,.55f,-.85f,.72f,.33f,.12f,-.65f,-.17f,.26f,-.40f,.78f,-.50f,.07f,-.98f,-.60f,-.38f,.95f,.75f,-.10f,-.30f,.57f,-.67f,.12f,.46f,-.23f,-.93f,-.34f,-.63f,.42f,.20f,.25f,-.83f,.09f,-.35f,-.70f,.73f,-.71f,-.65f,.53f,.93f,-.30f,.42f,.10f,.73f,-.96f,-.08f,.64f,.66f,-.20f,.62f,-.90f,-.63f,-.55f,.72f,.64f,-.42f,-.01f,-.62f,-.82f,.68f,-.86f,.48f,-.04f,-.02f,-.91f,-.99f,-.33f,-.46f,-.06f,-.92f,-.23f,.15f,.26f,.37f,.23f,.08f,.47f,-.78f,.86f,.52f,-.27f,-.01f,-.54f,-.42f,-.25f,-.20f,-.13f,.64f,.32f,.45f,.34f,-.13f,.99f,.45f,.81f,-.63f,-.27f,-.59f,-.29f,-.44f,.89f,-.01f,-.17f,.79f,.08f,.59f,-.01f,-.26f,.10f,.27f,.28f,-.12f,-.73f,.19f,.16f,-.12f,.49f,.43f,.74f,.56f,.90f,.28f,-.46f,-.62f,.47f,-.35f,-.36f,-.99f,.89f,-.64f,-.33f,-.43f,.97f,-.34f,.37f,-.16f,-.39f,.18f,-.95f,-.41f,.41f,-.21f,-.82f,-.93f,-.90f,.53f,.25f,.29f,-.84f,-.39f,-.72f,-.21f,-.78f,-.97f,-.92f,-.34f,.81f,-.08f,.34f,.26f,-.74f,-.26f,-.20f,.71f,.67f,-.08f,.35f,-.63f,-.11f,-.50f,-.71f,-.26f,-.46f,.01f,-.24f,.73f,-.22f,-.85f,-.68f,-.21f,-.56f,.16f,-.04f,-.77f,.24f,-.66f,.77f,.25f,-.27f,-.24f,-.36f,.89f,-.65f,.62f,.00f,-.74f,.22f,.82f,.72f,.58f,-.44f,.72f,-.55f,-.20f,.11f,-.62f,.48f,-.24f,-.02f,-.81f,.69f,-.58f,.26f,-.64f,-.48f,.79f,.38f,.68f,.70f,-.19f,.79f,.89f,-.57f,-.24f,-.57f,-.20f,-.80f,.54f,-.12f,-.94f,-.04f,.40f,-.16f,-.99f,.12f,.73f,-.02f,.63f,-.76f,-.40f,-.82f,.60f,.48f,.26f,-.77f,-.68f,.10f,-.81f,.90f,.97f,.10f,-.12f,-.07f,-.64f,.36f,-.37f,.25f,-.38f,-.65f,-.30f,-.71f,.35f,.69f,.29f,-.60f,.66f,.53f,.69f,-.27f,.04f,-.38f,-.47f,-.00f,-.30f,.71f,.44f,-.50f,.52f,.75f,.18f,-.47f,-.15f,.46f,-.42f,.32f,-.97f,.16f,.67f,-.13f,-.59f,.49f,-.02f,-.91f,-.05f,-.89f,.68f,-.21f,.57f,-.17f,.37f,-.36f,-.44f,.40f,.31f,-.51f,-.56f,.09f,.80f,-.61f,-.36f,-.66f,.29f,-.00f,-.72f,.14f,-.95f,-.18f,-.43f,-.25f,.28f,-.64f,-.37f,-.29f,.15f,.48f,.29f,-.86f,.43f,.85f,.56f,-.66f,-.93f,-.00f,.06f,.00f,-.73f,-.29f,-.29f,.59f,-.60f,-.24f,-.88f,.99f,.39f,.71f,-.58f,-.46f,-.86f,-.65f,-.17f,.28f,.85f,.74f,.32f,-.14f,.59f,.37f,-.94f,.28f,-.50f,-.88f,.36f,-.21f,-.22f,.55f,-.49f,-.22f,.33f,.34f,.59f,-.94f,.82f,-.06f,.82f,.71f,.03f,-.11f,-.02f,.85f,-.25f,.86f,-.30f,-.99f,.27f,.17f,.78f,.24f,-.85f,.97f,.35f,.44f,-.86f,.22f,.69f,.89f,-.11f,.94f,-.97f,-.73f,-.46f,-.85f,.33f,-.01f,-.37f,.23f,-.67f,.02f,-.89f,.99f,-.46f,.40f,-.60f,-.35f,-.13f,-.38f,-.18f,-.50f,-.52f,.06f,.01f,.68f,.69f,.17f,-.36f,.68f,-.01f,.22f,.96f,.58f,-.28f,.10f,-.49f,-.21f,-.74f,.06f,-.03f,.36f,.47f,-.94f,-.13f,.66f,-.03f,.81f,-.10f,-.34f,-.74f,-.02f,.36f,.13f,-.09f,-.23f,-.15f,-.84f,.61f,.63f,-.06f,-.59f,.49f,-.83f,.43f,-.51f,-.26f,-.49f,.79f,.92f,.69f,-.51f,.43f,.57f,.59f,.62f,.04f,-.66f,.12f,-.61f,.15f,-.29f,-.66f,-.05f,.27f,.83f,.62f,-.89f,.37f,.08f,-.41f,.60f,.57f,.27f,-.08f,.62f,-.75f,.71f,-.56f,-.08f,.98f,.63f,-.46f,-.76f,.75f,-.99f,-.80f,-.58f,-.16f,-.55f,.09f,.42f,-.11f,.18f,-.87f,-.89f,.04f,.24f,-.86f,-.20f,.09f,-.76f,.77f,-.39f,.60f,-.61f,-.04f,.16f,.57f,-.39f,-.75f,-.99f,-.77f,.91f,.58f,-.29f,.48f,.22f,.73f,-.29f,-.28f,.93f,-.78f,-.19f,.27f,-.76f,-.93f,-.95f,-.98f,.74f,.50f,.09f,-.76f,-.31f,-.95f,.18f,.95f,-.90f,-.87f,.87f,.39f,.91f,.91f,.43f,.21f,-.33f,-.82f,-.85f,.79f,.25f,.62f,.44f,-.19f,.80f,.77f,-.08f,.74f,-.25f,.90f,.93f,.55f,-.46f,-.63f,.73f,.51f,.31f,-.30f,-.46f,-.26f,.82f,-.44f,-.96f,-.05f,.25f,.67f,.37f,-.96f,-.70f,-.50f,-.33f,-.30f,.19f,.45f,.96f,.18f,-.44f,.10f,-.29f,-.83f,.18f,.14f,.91f,-.76f,.33f,-.75f,-.06f,.35f,.35f,-.22f,-.42f,-.34f,.60f,-.56f,.95f,-.35f,-.13f,-.58f,-.84f,.89f,-.67f,-.84f,.78f,-.29f,.83f,-.03f,.74f,-.98f,.54f,-.33f,.40f,.79f,.78f,.47f,-.60f,.66f,.69f,-.25f,-.38f,.90f,.91f,-.22f,.55f,-.30f,-.48f,-.16f,-.09f,-.23f,-.68f,-.62f,.49f,.34f,.18f,.88f,.08f,.95f,-.59f,.40f,.52f,.19f,.94f,.36f,.96f,-.31f,.71f,-.19f,.67f,.87f,-.39f,-.55f,-.01f,-.14f,.28f,-.75f,.70f,-.17f,.18f,.64f,.50f,-.83f,.85f,.85f,.38f,.04f,-.46f,.41f,-.21f,-.78f,.32f,.46f,.95f,-.16f,-.31f,-.24f,.06f,-.25f,-.94f,.99f,.75f,-.00f,-.42f,-.72f,.61f,-.06f,.78f,.92f,.25f,.90f,-.37f,.72f,.21f,-.72f,.36f,.76f,-.53f,-.36f,.45f,-.28f,-.47f,-.46f,.47f,-.30f,.30f,-.72f,.73f,-.61f,-.39f,.06f,.35f,.77f,-.38f,.52f,-.03f,-.85f,-.20f,.02f,.64f,-.72f,.10f,.11f,.81f,-.07f,.72f,.87f,.81f,-.21f,-.03f,-.86f,-.18f,.01f,-.77f,-.60f,.16f,.32f,-.46f,.00f,-.00f,-.77f,-.04f,.92f,.48f,-.84f,-.65f,-.93f,-.77f,-.11f,.77f,.86f,.96f,.31f,-.18f,-.94f,.66f,-.97f,.51f,.39f,-.66f,-.74f,.82f,.91f,.90f,-.22f,-.89f,.72f,.98f,-.14f,.73f,.68f,.68f,-.20f,.44f,-.76f,.40f,-.93f,-.43f,-.89f,-.74f,-.18f,-.79f,-.69f,.32f,.73f,-.85f,-.06f,.72f,-.25f,.37f,.76f,-.20f,-.70f,.27f,-.94f,.75f,-.29f,-.25f,-.82f,.38f,.50f,.56f,.62f,-.41f,-.10f,-.38f,.76f,-.42f,.89f,.16f,-.83f,-.05f,-.08f,-.41f,.55f,.01f,.93f,-.79f,-.46f,.63f,-.93f,-.26f,.51f,.57f,-.76f,-.94f,.21f,-.83f,.51f,-.38f,-.00f,-.35f,-.94f,.32f,-.70f,-.05f,-.73f,.19f,-.52f,.32f,-.48f,.99f,.18f,.36f,-.73f,.48f,.74f,.53f,.41f,.85f,-.42f,.46f,.18f,.66f,.99f,-.55f,-.02f,.54f,-.89f,-.78f,.68f,.34f,.90f,-.34f,.55f,-.50f,-.80f,-.26f,-.91f,-.72f,.75f,.23f,.87f,.15f,.77f,-.64f,-.78f,.03f,-.29f,-.31f,.02f,-.16f,.01f,.50f,-.35f,-.65f,.22f,.53f,-.25f,-.31f,.61f,.59f,-.83f,-.30f,-.53f,-.76f,-.28f,.71f,.52f,-.77f,.86f,-.01f,.92f,.90f,.20f,.20f,.49f,.38f,-.95f,.59f,-.25f,.42f,-.33f,.29f,.17f,-.17f,.78f,-.75f,.99f,-.43f,-.33f,-.61f,.41f,-.22f,-.25f,-.16f,-.74f,-.06f,.24f,.60f,-.96f,.06f,-.13f,-.27f,-.04f,-.04f,.45f,.25f,.83f,-.31f,.27f,-.06f,-.40f,.68f,-.52f,.92f,-.71f,-.78f,.93f,-.84f,-.51f,-.30f,-.15f,.02f,-.59f,-.64f,.06f,-.18f,-.55f,-.96f,.12f,-.89f,.97f,.91f,-.73f,.06f,.57f,-.82f,.59f,-.86f,.08f,.45f,-.95f,.67f,-.65f,-.29f,-.36f,.04f,-.82f,.10f,.55f,.32f,.55f,.32f,.81f,-.92f,.09f,-.35f,.88f,.31f,.06f,.89f,.00f,.37f,-.45f,.58f,.97f,.95f,.34f,-.41f,-.84f,.45f,-.81f,.73f,-.11f,.06f,-.68f,-.92f,.59f,.80f,.60f,-.04f,-.18f,.44f,-.83f,.71f,.68f,.02f,.62f,.83f,.60f,.81f,-.92f,.86f,-.57f,.26f,.30f,-.26f,.29f,-.74f,-.21f,.65f,.40f,.43f,-.95f,.36f,-.28f,.12f,-.89f,-.23f,.86f,-.11f,.70f,.52f,-.22f,-.83f,-.63f,.08f,.58f,-.88f,-.34f,-.06f,.44f,.10f,.36f,.99f,.04f,.41f,-.73f,.41f,.57f,-.41f,.35f,.36f,-.37f,.64f,-.70f,-.33f,-.57f,-.38f,.09f,.50f,-.09f,-.95f,.59f,.07f,.25f,.75f,-.29f,-.46f,-.69f,-.93f,-.49f,-.03f,-.25f,.65f,-.15f,.79f,.76f,-.73f,.57f,-.90f,.74f,.99f,.20f,.09f,.33f,-.55f,.42f,-.98f,-.26f,-.48f,.96f,.83f,-.75f,-.64f,-.09f,-.73f,.96f,-.26f,-.76f,.83f,.54f,.66f,.54f,.64f,-.58f,.45f,-.67f,.81f,.00f,-.77f,.66f,.36f,.93f,.75f,-.62f,.65f,-.52f,.08f,.02f,.12f,.35f,.85f,.83f,-.07f,-.62f,-.78f,-.98f,-.78f,.43f,.53f,-.55f,.04f,.99f,-.57f,-.52f,.78f,.11f,-.00f,.88f,.10f,-.43f,-.44f,-.51f,.73f,-.74f,.53f,-.26f,-.58f,.07f,-.48f,.23f,.94f,-.47f,-.53f,.04f,.18f,-.16f,-.86f,-.90f,-.04f,.05f,.03f,.53f,-.11f,-.86f,.80f,.34f,.61f,-.00f,-.99f,.55f,-.20f,-.84f,.21f,-.66f,-.43f,-.93f,-.83f,.00f,-.62f,-.59f,-.16f,-.56f,-.11f,.55f,-.30f,.96f,-.67f,.05f,.17f,.30f,-.86f,-.44f,.05f,.75f,-.44f,-.03f,-.61f,-.89f,.40f,.35f,-.42f,.35f,-.97f,.58f,.32f,.84f,-.13f,.01f,.89f,.60f,.06f,.80f,.67f,-.54f,.59f,-.52f,.06f,.86f,.19f,.04f,.75f,.86f,.11f,.00f,-.72f,-.85f,.09f,.78f,-.68f,.90f,.06f,.38f,-.14f,.97f,.07f,-.60f,.60f,-.78f,-.38f,.01f,-.99f,.61f,.53f,-.65f,.16f,.32f,.61f,-.94f,.88f,.75f,-.05f,-.24f,-.84f,-.49f,.89f,.39f,.22f,.20f,-.29f,.74f,-.01f,.08f,-.24f,-.80f,.57f,-.16f,.00f,-.48f,.68f,-.06f,.78f,-.66f,-.91f,.30f,-.72f,-.20f,.28f,-.94f,-.44f,-.29f,.27f,-.84f,.86f,-.47f,-.62f,.29f,-.59f,.71f,-.10f,-.47f,-.40f,-.62f,-.45f,.01f,-.88f,-.47f,-.11f,-.27f,.33f,-.12f,.68f,-.96f,.30f,-.73f,.13f,.70f,.78f,.18f,.35f,-.82f,-.65f,-.72f,.44f,-.03f,.12f,-.68f,-.68f,.48f,.89f,-.32f,.84f,.32f,.14f,-.68f,.44f,.17f,.56f,-.76f,.24f,-.88f,.04f,-.46f,-.34f,.28f,-.31f,-.97f,-.21f,-.78f,-.84f,-.22f,-.73f,.07f,-.56f,.01f,-.75f,-.19f,-.41f,.78f,-.23f,.93f,.91f,.96f,-.41f,-.27f,-.20f,-.59f,.25f,.38f,-.63f,.24f,.62f,.44f,.71f,-.55f,.61f,.50f,-.56f,.77f,.44f,.25f,-.17f,-.32f,.83f,-.33f,.16f,.90f,-.58f,-.63f,.28f,-.65f,.02f,.60f,.67f,.30f,.09f,-.48f,.54f,-.07f,.32f,-.51f,-.83f,.67f,.86f,.48f,-.14f,-.03f,-.49f,.17f,-.59f,-.21f,-.26f,-.93f,.80f,-.35f,.52f,-.40f,-.89f,-.77f,-.53f,.56f,-.58f,.45f,.54f,.81f,.05f,.54f,.11f,.78f,.53f,.88f,.13f,-.95f,-.23f,-.40f,.29f,-.37f,-.57f,.39f,.24f,.14f,.86f,.36f,-.72f,.12f,-.54f,.60f,.22f,.32f,-.02f,.69f,-.43f,-.64f,.92f,.37f,.07f,.16f,.05f,.11f,.19f,.21f,-.60f,-.02f,.43f,.84f,-.97f,-.12f,-.06f,-.11f,-.35f,.77f,-.12f,-.44f,-.02f,-.09f,.34f,.37f,.17f,-.63f,.07f,.74f,.57f,-.97f,.57f,.22f,.47f,-.92f,-.97f,-.01f,.47f,.08f,-.31f,.74f,.79f,-.68f,-.89f,-.10f,-.80f,-.91f,.06f,-.40f,.76f,.27f,.79f,.96f,-.95f,.62f,-.09f,.41f,.03f,-.03f,-.85f,-.51f,.92f,-.67f,.44f,.20f,-.29f,-.01f,.48f,.76f,-.74f,-.95f,-.62f,-.46f,.55f,.61f,-.51f,.26f,.53f,-.99f,.28f,.53f,-.41f,-.92f,.38f,-.53f,-.14f,.87f,.80f,.07f,-.76f,.25f,-.75f,-.83f,-.83f,-.01f,-.08f,.92f,-.33f,.67f,.08f,-.11f,.35f,-.17f,.70f,.23f,-.64f,-.91f,.56f,.82f,.42f,-.48f,.54f,-.04f,.00f,.05f,.00f,-.11f,.02f,-.26f,.46f,.04f,.84f,-.54f,-.33f,-.55f,-.00f,-.90f,.73f,-.19f,-.35f,-.40f,-.09f,-.15f,.92f,-.20f,-.95f,-.85f,.92f,.10f,.61f,.02f,-.01f,.54f,.07f,.85f,.50f,-.84f,.14f,-.38f,.40f,.22f,-.49f,.11f,-.49f,-.23f,-.39f,-.25f,-.31f,.65f,-.52f,-.38f,.46f,-.73f,.91f,-.71f,.77f,.65f,-.05f,.31f,-.60f,.08f,-.26f,.25f,.38f,-.68f,.55f,.83f,-.63f,-.75f,-.55f,.08f,-.42f,.35f,-.33f,-.45f,.00f,.67f,.78f,-.59f,.55f,-.90f,-.47f,.91f,.00f,-.15f,.34f,.75f,-.92f,.15f,.21f,.14f,-.14f,.22f,-.19f,.42f,.31f,.75f,-.66f,-.24f,.37f,.40f,.25f,-.15f,.47f,.03f,.14f,-.32f,.56f,-.58f,.88f,-.67f,.96f,.14f,-.78f,.98f,-.77f,-.52f,-.27f,.94f,.55f,.03f,.90f,-.56f,-.43f,-.58f,-.27f,-.79f,-.87f,-.71f,-.22f,-.48f,-.38f,-.96f,.58f,.24f,-.28f,-.47f,.33f,.10f,-.44f,-.94f,.29f,-.16f,.43f,-.79f,.81f,-.89f,-.63f,-.76f,-.15f,-.71f,-.77f,.13f,.67f,-.15f,.86f,.12f,-.37f,.42f,-.82f,-.64f,.42f,.21f,-.37f,-.88f,.14f,-.80f,.22f,.52f,-.84f,-.39f,.89f,.23f,-.21f,-.62f,.49f,.35f,-.13f,-.40f,.94f,-.10f,-.71f,-.54f,.75f,.28f,-.00f,-.78f,-.12f,.42f,.28f,-.99f,.82f,-.99f,-.97f,.61f,.29f,-.76f,.97f,-.57f,-.50f,.20f,.38f,-.95f,.08f,-.57f,-.98f,.24f,-.41f,.40f,.50f,.59f,.44f,-.76f,-.97f,-.65f,-.07f,-.12f,-.03f,.79f,.87f,-.63f,-.09f,.16f,-.47f,.62f,.55f,.17f,.32f,-.44f,.44f,.61f,-.52f,.51f,-.49f,.87f,-.34f,.63f,-.53f,-.30f,-.50f,-.11f,-.82f,-.63f,-.27f,.31f,-.60f,-.83f,.84f,.90f,-.13f,.61f,-.29f,-.62f,.51f,-.32f,.01f,-.35f,-.22f,-.03f,-.65f,.25f,.23f,-.70f,.09f,-.34f,-.82f,-.60f,.27f,.48f,-.04f,.91f,.40f,-.66f,.85f,-.20f,.85f,.61f,-.96f,-.25f,-.55f,-.75f,.00f,.78f,-.92f,.29f,-.00f,.66f,.03f,.47f,-.44f,-.43f,-.63f,.82f,-.00f,-.88f,-.14f,-.55f,.37f,.00f,.95f,.12f,.95f,.22f,-.98f,.82f,-.44f,.37f,-.09f,-.92f,.52f,-.26f,-.37f,.95f,.74f,.84f,-.97f,.09f,-.33f,-.87f,-.06f,.81f,-.85f,-.68f,-.67f,.65f,-.95f,.60f,-.55f,-.45f,.50f,.54f,-.18f,-.00f,-.53f,.68f,.88f,.36f,-.37f,.58f,-.90f,.69f,-.69f,.10f,.40f,-.88f,-.91f,-.90f,-.18f,.47f,-.60f,.17f,-.13f,.64f,-.24f,-.42f,-.89f,-.47f,.24f,-.09f,.13f,.22f,.35f,.49f,.73f,-.97f,-.58f,.49f,.20f,.21f,-.24f,.16f,-.77f,.02f,-.35f,.06f,.83f,-.04f,-.87f,-.61f,.78f,.25f,-.49f,-.73f,-.13f,.95f,-.59f,.41f,-.97f,-.53f,-.10f,-.45f,-.86f,.57f,-.58f,-.78f,.51f,-.05f,-.57f,.11f,.46f,.66f,-.48f,.51f,-.01f,-.53f,-.76f,.19f,-.01f,.62f,-.76f,.49f,.13f,-.44f,.01f,.98f,-.33f,-.18f,-.00f,.27f,.58f,.94f,-.21f,-.79f,.78f,.15f,-.19f,-.60f,.66f,.53f,-.13f,-.91f,-.96f,.16f,-.19f,-.40f,-.79f,-.18f,.66f,-.17f,.80f,.90f,.91f,.43f,-.83f,.74f,-.88f,-.41f,.07f,-.25f,-.53f,-.75f,-.43f,-.62f,-.35f,.90f,-.05f,.75f,-.76f,.87f,-.51f,.16f,.11f,-.79f,.29f,.36f,.56f,-.85f,-.51f,-.04f,.63f,.75f,.66f,-.65f,-.33f,-.58f,.09f,-.50f,-.70f,-.75f,.45f,-.75f,.21f,-.20f,-.39f,.48f,.97f,-.03f,.24f,-.18f,-.48f,.46f,-.04f,-.71f,-.11f,.89f,.56f,-.52f,.76f,.44f,.19f,.83f,.93f,-.51f,.83f,.73f,.12f,.52f,-.97f,.42f,.91f,-.17f,.13f,.44f,-.03f,.54f,.48f,-.25f,-.92f,-.46f,.39f,.82f,-.23f,.05f,.66f,-.08f,.60f,.14f,-.20f,-.32f,-.10f,-.21f,.12f,.56f,-.73f,.63f,.09f,.97f,.10f,-.88f,.71f,.25f,-.62f,-.30f,-.25f,-.64f,.35f,.91f,.57f,.08f,-.97f,-.41f,-.94f,-.91f,-.11f,.57f,-.68f,-.37f,-.39f,.83f,.31f,-.15f,.81f,-.53f,.63f,-.65f,.92f,-.40f,-.35f,.05f,.84f,.63f,.12f,-.97f,.98f,.21f,.91f,-.86f,-.12f,-.77f,.56f,.66f,-.60f,.67f,-.36f,-.33f,.39f,-.21f,.22f,.82f,-.51f,.86f,.28f,-.07f,.13f,-.91f,-.83f,-.60f,-.88f,.42f,.63f,-.70f,.54f,.70f,.38f,-.71f,-.15f,.87f,.07f,.81f,-.33f,-.87f,-.86f,.85f,.37f,-.17f,.71f,.87f,-.56f,.06f,-.66f,.79f,-.17f,.13f,-.22f,-.75f,.65f,.14f,.25f,.56f,.63f,-.68f,-.11f,.91f,.12f,-.01f,-.25f,-.76f,-.93f,.66f,.55f,.43f,-.35f,.60f,-.01f,-.68f,-.94f,-.34f,-.47f,.87f,.78f,.10f,.61f,-.34f,.19f,-.24f,-.20f,.65f,-.04f,-.46f,.08f,.10f,.54f,.28f,-.33f,-.89f,.88f,-.54f,-.04f,.22f,.53f,.70f,.07f,-.65f,-.86f,-.72f,-.61f,.15f,-.33f,-.46f,.96f,.51f,.86f,.84f,.03f,-.62f,.54f,-.40f,-.73f,.04f,-.40f,.23f,-.40f,.28f,-.66f,.71f,.33f,.00f,.59f,.08f,.81f,-.02f,.35f,-.57f,.92f,-.76f,.55f,-.28f,.53f,-.84f,-.76f,.62f,-.86f,.97f,.33f,-.31f,-.70f,.51f,.21f,-.88f,-.14f,.17f,.93f,.34f,-.38f,-.71f,-.68f,-.99f,-.80f,-.26f,.80f,.39f,.65f,.89f,-.11f,.27f,.59f,.72f,.88f,.41f,.27f,-.54f,.61f,-.24f,.64f,.39f,.06f,-.65f,-.23f,-.43f,-.74f,.23f,.06f,-.18f,-.72f,.30f,.07f,-.90f,-.49f,-.50f,.90f,.73f,-.00f,.12f,-.03f,.50f,-.15f,.70f,-.82f,.54f,.73f,-.49f,-.50f,-.22f,-.57f,.58f,-.73f,-.69f,-.58f,-.81f,-.48f,.92f,-.83f,-.67f,-.43f,-.64f,.53f,.00f,-.71f,-.36f,.38f,.92f,.98f,-.60f,-.52f,.11f,.78f,-.40f,.81f,-.50f,-.93f,-.04f,-.65f,.44f,-.66f,-.23f,.09f,-.14f,-.40f,.26f,.82f,.68f,.61f,.93f,.02f,-.86f,-.74f,-.81f,-.84f,-.64f,-.70f,-.36f,.83f,-.57f,-.74f,-.22f,.51f,-.96f,-.07f,-.57f,.18f,.95f,.25f,.76f,-.59f,-.09f,-.74f,.33f,.64f,-.90f,.62f,-.16f,-.69f,.46f,-.86f,.21f,-.25f,.27f,.66f,.63f,-.19f,-.16f,-.31f,-.37f,.99f,.96f,-.84f,-.05f,-.21f,.05f,-.22f,-.09f,.32f,.61f,.94f,-.30f,.73f,-.79f,-.84f,.36f,-.34f,.00f,.19f,.14f,-.94f,-.98f,.93f,.09f,-.75f,.53f,-.39f,-.95f,-.21f,.17f,.87f,.21f,-.29f,.15f,-.96f,-.30f,-.13f,.43f,-.01f,-.08f,.57f,.77f,.98f,-.74f,-.89f,.79f,-.10f,-.15f,.28f,.47f,-.62f,.86f,.77f,.21f,-.22f,.63f,-.42f,-.10f,.42f,.02f,-.15f,-.74f,.66f,-.69f,-.82f,-.39f,.17f,-.16f,-.04f,-.69f,.97f,-.33f,.74f,-.69f,-.01f,-.87f,.43f,.99f,-.18f,-.37f,.86f,-.96f,.97f,-.62f,-.02f,.81f,-.21f,.78f,-.36f,.79f,.14f,.04f,.13f,.77f,.84f,-.21f,-.24f,.50f,-.46f,-.64f,-.43f,-.26f,-.08f,-.14f,-.70f,.77f,-.59f,-.39f,.60f,-.06f,.42f,-.83f,-.43f,.90f,.76f,-.52f,-.92f,-.22f,-.15f,-.54f,.87f,.20f,.80f,.93f,-.14f,-.33f,-.83f,.20f,-.98f,.18f,.32f,-.65f,.00f,-.22f,.89f,-.07f,-.31f,.05f,-.49f,-.58f,.11f,-.23f,.18f,-.58f,-.30f,-.83f,.39f,.56f,-.49f,.45f,-.79f,.79f,-.67f,.67f,-.15f,.32f,-.91f,.46f,.11f,.49f,.78f,.33f,-.66f,-.64f,.63f,-.42f,-.25f,-.80f,-.24f,-.69f,.56f,.84f,.77f,.74f,-.39f,.54f,.83f,.22f,-.98f,.26f,.17f,.63f,.00f,.55f,-.92f,.94f,-.32f,-.87f,-.74f,.11f,.61f,.55f,-.72f,.19f,-.44f,-.97f,-.25f,-.94f,.75f,-.41f,-.40f,-.83f,-.82f,-.51f,-.26f,.49f,.98f,.26f,.91f,.30f,-.37f,.50f,-.77f,.39f,-.02f,.22f,-.91f,.25f,.98f,.92f,.85f,-.23f,-.00f,-.35f,-.63f,.88f,.39f,.74f,-.04f,.34f,.02f,-.82f,.73f,.33f,-.59f,.47f,.56f,.48f,-.34f,.84f,-.68f,-.28f,.96f,-.06f,.30f,-.75f,.91f,.09f,.29f,-.84f,.97f,-.66f,.52f,-.00f,.56f,-.46f,.29f,.49f,.64f,.77f,-.24f,.20f,.34f,.40f,.20f,.85f,-.80f,.28f,.93f,.72f,-.30f,.62f,.61f,-.63f,-.66f,-.46f,.95f,.05f,-.47f,.83f,.02f,-.33f,-.35f,.38f,-.51f,.11f,.52f,.14f,.87f,-.71f,-.87f,-.22f,.52f,.82f,-.77f,.72f,.48f,-.11f,-.77f,.64f,-.61f,.09f,-.16f,-.30f,.56f,.50f,-.66f,-.39f,.81f,.42f,.29f,-.32f,.67f,-.59f,-.60f,.74f,.93f,-.49f,-.34f,-.35f,.32f,-.75f,.32f,-.03f,-.39f,.61f,-.84f,-.46f,-.33f,.99f,-.71f,.71f,-.48f,.27f,.65f,-.08f,.86f,-.69f,.58f,.10f,-.64f,.82f,-.66f,-.65f,.58f,-.42f,.85f,-.34f,.99f,-.02f,-.05f,.51f,-.40f,.24f,.61f,-.13f,-.78f,-.31f,-.89f,.62f,.30f,-.27f,-.68f,-.41f,-.77f,.64f,.94f,.01f,.29f,-.45f,-.15f,-.73f,.78f,-.27f,-.80f,-.70f,-.30f,-.69f,.38f,-.31f,.37f,.08f,.97f,-.32f,-.01f,.70f,.61f,.39f,.58f,.35f,.32f,.17f,.86f,-.04f,-.75f,.98f,.81f,-.58f,.78f,.94f,.65f,.89f,-.36f,-.43f,.27f,.99f,-.99f,.11f,.04f,-.08f,-.16f,.20f,-.66f,.87f,-.05f,-.57f,.27f,.90f,-.41f,-.60f,.52f,-.95f,.30f,.26f,.54f,-.71f,.44f,-.70f,-.29f,-.84f,-.21f,-.59f,.55f,-.98f,-.85f,-.98f,.84f,.99f,.25f,.14f,-.42f,-.91f,-.48f,-.52f,-.49f,.33f,.05f,-.46f,.97f,.89f,-.12f,-.45f,.91f,-.13f,-.22f,-.74f,-.34f,.00f,.34f,.16f,-.31f,-.11f,.77f,-.70f,.81f,.84f,-.30f,.18f,.98f,-.12f,-.36f,.48f,.26f,-.87f,.21f,.64f,.36f,-.85f,-.04f,.44f,-.89f,.94f,-.98f,.19f,-.40f,-.48f,-.36f,.77f,.29f,.18f,-.02f,-.72f,.44f,.35f,.42f,-.27f,.51f,.94f,.14f,.53f,-.56f,-.07f,.37f,-.33f,.83f,-.53f,.33f,.86f,-.13f,-.52f,.11f,-.23f,.25f,-.65f,-.06f,.42f,.71f,.94f,-.78f,.66f,.96f,.54f,.20f,-.36f,-.52f,-.26f,-.59f,.42f,.34f,-.23f,-.35f,.67f,.49f,.22f,-.31f,.17f,-.34f,.13f,.06f,.78f,-.24f,-.50f,.41f,-.18f,-.93f,-.98f,-.02f,-.16f,-.11f,-.37f,-.66f,-.63f,.14f,-.87f,-.60f,.60f,-.45f,.75f,-.41f,.62f,.24f,.71f,.63f,.77f,.11f,-.99f,.03f,.50f,-.72f,-.77f,-.25f,.00f,.51f,-.77f,-.35f,-.91f,.94f,.40f,.89f,.25f,-.42f,-.24f,-.15f,-.85f,.32f,-.42f,.66f,.57f,-.89f,-.08f,.76f,-.57f,.09f,-.26f,-.30f,-.46f,-.44f,-.06f,.79f,.67f,-.60f,-.59f,-.05f,.11f,-.60f,-.22f,-.99f,.51f,-.30f,.70f,.85f,.00f,-.55f,-.26f,-.28f,-.55f,.85f,.49f,-.33f,-.95f,-.34f,-.26f,-.41f,-.37f,-.07f,.84f,-.94f,.36f,-.97f,.04f,.52f,.36f,.36f,-.41f,-.66f,.91f,.99f,-.70f,-.36f,.20f,.97f,-.87f,-.32f,-.86f,-.85f,-.33f,.69f,.44f,-.25f,-.44f,-.44f,-.90f,.93f,.74f,-.52f,.71f,.97f,.13f,.04f,.42f,.08f,-.71f,.08f,.13f,-.42f,-.08f,-.49f,-.13f,.91f,.08f,.02f,.42f,-.44f,-.32f,.34f,.27f,.70f,.49f,-.30f,-.00f,-.87f,.08f,.29f,.64f,.02f,-.29f,.23f,-.53f,-.93f,.42f,-.47f,-.79f,.66f,.75f,-.30f,.16f,.81f,-.81f,.36f,.71f,.90f,.15f,.98f,-.77f,.96f,.10f,.40f,.05f,-.76f,.71f,-.12f,-.30f,.93f,-.00f,-.53f,-.25f,-.10f,.04f,-.07f,.04f,-.89f,.53f,-.76f,.60f,.07f,-.31f,.06f,-.35f,.33f,.01f,-.25f,-.71f,-.38f,-.39f,.15f,-.28f,-.44f,-.64f,-.80f,-.61f,-.57f,.46f,.61f,-.49f,-.07f,-.17f,-.94f,.73f,-.06f,-.02f,.19f,.54f,.38f,.69f,-.67f,-.27f,.05f,.75f,-.48f,-.13f,-.18f,-.02f,.37f,.63f,-.97f,.30f,.82f,.82f,.88f,.25f,-.64f,.60f,-.41f,-.12f,-.73f,-.65f,-.53f,.02f,.58f,-.36f,-.53f,.88f,-.51f,.66f,-.79f,-.23f,.65f,-.38f,.99f,-.51f,-.62f,.72f,-.31f,-.87f,.56f,.18f,.71f,.01f,-.72f,.59f,.15f,-.73f,.43f,-.81f,-.62f,.08f,.82f,.98f,.94f,-.97f,.97f,-.27f,-.17f,-.41f,-.20f,-.61f,.31f,-.25f,-.58f,.95f,-.36f,.28f,-.54f,-.92f,-.87f,-.61f,.12f,-.24f,.71f,.90f,-.29f,-.75f,.78f,.30f,.51f,-.48f,-.39f,.75f,-.90f,.65f,-.07f,.17f,-.55f,.27f,-.66f,-.23f,-.13f,.06f,-.27f,.13f,.53f,-.06f,.88f,.81f,-.56f,-.09f,-.26f,-.84f,-.22f,-.59f,.23f,.60f,-.60f,.90f,-.86f,.00f,-.43f,-.15f,-.33f,-.89f,.20f,-.08f,.37f,.24f,.86f,.67f,-.52f,-.88f,.45f,-.56f,.00f,-.45f,.07f,-.82f,.08f,-.27f,-.64f,-.88f,.92f,-.99f,-.12f,.89f,.35f,-.29f,.48f,-.35f,-.50f,-.60f,.31f,-.46f,.60f,.47f,.54f,.45f,.86f,.54f,-.99f,.59f,-.01f,.86f,-.10f,.54f,-.76f,-.88f,.75f,-.61f,.40f,-.25f,.92f,.14f,-.18f,.04f,-.31f,.29f,-.80f,.67f,-.60f,-.76f,-.01f,-.19f,.18f,.55f,.03f,.23f,-.30f,-.66f,-.12f,-.16f,.97f,.40f,.52f,-.15f,-.89f,.71f,.31f,.96f,.90f,-.19f,-.35f,.28f,-.64f,-.45f,-.78f,-.98f,-.36f,.00f,-.53f,-.91f,-.41f,.21f,.43f,-.24f,-.37f,-.91f,.66f,.19f,.76f,.72f,-.44f,.55f,.51f,.45f,.68f,-.26f,.09f,-.87f,.16f,.52f,.85f,-.61f,-.28f,-.50f,.32f,-.24f,.09f,.18f,.46f,.80f,-.04f,.07f,.29f,-.57f,-.22f,-.92f,-.49f,.98f,.99f,.61f,.26f,.85f,-.24f,.47f,-.40f,-.59f,-.00f,-.89f,-.33f,.37f,.26f,.31f,-.77f,-.89f,-.03f,-.26f,.85f,.71f,-.12f,.64f,.92f,-.54f,.96f,.42f,.85f,-.85f,.22f,.61f,.64f,-.82f,.01f,-.07f,.58f,-.33f,.25f,.26f,-.52f,.44f,-.84f,.50f,-.02f,-.29f,.57f,-.43f,-.62f,.66f,-.01f,-.86f,.83f,.90f,-.16f,.66f,-.81f,-.30f,.15f,-.62f,.38f,.92f,-.11f,.39f,.51f,-.27f,.62f,-.86f,.00f,-.16f,.70f,-.77f,-.92f,-.73f,-.28f,-.47f,-.70f,-.66f,-.78f,-.86f,.56f,.28f,.31f,.96f,-.64f,.18f,.49f,-.81f,-.56f,-.98f,.99f,-.93f,.04f,-.63f,-.22f,-.63f,-.49f,.08f,.47f,.71f,-.35f,-.76f,.01f,-.92f,.80f,.23f,.69f,-.66f,.14f,-.21f,.11f,-.38f,-.11f,-.12f,-.77f,-.90f,-.24f,.78f,.54f,.84f,.18f,-.26f,-.08f,-.66f,-.83f,-.28f,.90f,-.89f,.98f,-.75f,-.92f,-.56f,.29f,.03f,.25f,-.64f,.23f,-.30f,.44f,.98f,-.48f,.66f,.80f,-.31f,.17f,.45f,-.43f,.59f,-.78f,.10f,.27f,.04f,.85f,-.25f,-.22f,-.07f,-.64f,-.86f,-.39f,.73f,.81f,-.71f,.63f,.87f,-.41f,.45f,-.00f,-.01f,.40f,.10f,-.59f,.13f,-.11f,.13f,.18f,-.06f,-.18f,.28f,-.42f,-.62f,-.82f,.34f,-.61f,.61f,-.47f,.34f,-.68f,.92f,-.08f,.90f,.52f,.71f,-.14f,.73f,-.72f,-.43f,.15f,-.10f,.49f,-.33f,.71f,-.13f,-.48f,-.39f,.08f,.50f,.47f,-.79f,-.21f,-.75f,.89f,.15f,.21f,.62f,-.05f,.62f,-.83f,.87f,-.34f,.61f,-.60f,-.42f,.11f,.03f,-.66f,-.05f,.42f,.19f,.82f,.69f,-.33f,-.92f,-.02f,-.00f,.82f,.83f,-.02f,.44f,.11f,.19f,.01f,-.09f,.38f,-.32f,.30f,.84f,-.65f,-.01f,-.08f,-.02f,-.83f,-.47f,.95f,-.99f,.37f,-.38f,.21f,.35f,-.38f,.92f,.60f,.46f,-.14f,.28f,.20f,-.15f,-.03f,.96f,-.33f,.08f,-.48f,-.56f,.96f,-.03f,-.56f,-.25f,.90f,.16f,.21f,-.45f,.77f,.21f,.61f,.74f,-.07f,.93f,-.35f,-.26f,.52f,.74f,-.91f,.33f,.09f,-.45f,.58f,-.53f,.31f,.42f,.70f,-.85f,-.19f,.63f,-.87f,.67f,-.49f,.51f,-.08f,.96f,-.80f,.00f,.79f,.44f,.69f,-.37f,-.86f,-.63f,.90f,.79f,-.56f,-.26f,.66f,.62f,.22f,.27f,.75f,-.86f,.23f,.34f,-.50f,-.40f,-.05f,.12f,-.26f,.40f,-.74f,-.94f,-.65f,-.36f,-.66f,-.96f,-.34f,.57f,-.44f,.09f,-.88f,.66f,-.04f,.46f,-.33f,-.72f,.69f,.58f,-.57f,-.75f,.48f,.31f,-.04f,.34f,-.95f,.49f,-.63f,-.27f,.61f,-.96f,-.00f,-.27f,-.26f,.26f,-.33f,.30f,.61f,.81f,.12f,.24f,.24f,-.07f,-.23f,-.50f,-.55f,-.22f,-.89f,-.97f,.05f,.46f,.11f,.62f,-.02f,-.34f,-.94f,.04f,.97f,.43f,.58f,-.46f,.59f,-.36f,.47f,-.06f,-.57f,-.57f,.10f,.38f,-.43f,.23f,-.91f,.35f,.09f,.91f,.30f,.84f,.09f,.40f,-.75f,.50f,-.36f,-.95f,-.86f,-.78f,-.69f,.28f,.93f,.19f,-.15f,.90f,-.19f,.71f,.98f,.15f,.31f,-.87f,-.61f,.51f,.71f,.67f,-.40f,.70f,.34f,.61f,.61f,.47f,.49f,-.13f,-.43f,.79f,-.45f,-.57f,.40f,-.23f,.53f,.27f,-.58f,.43f,-.10f,-.72f,.27f,-.09f,.56f,.64f,.11f,-.57f,-.27f,.60f,-.29f,.16f,.77f,-.96f,.40f,.13f,.86f,-.42f,.62f,-.21f,-.11f,.92f,.07f,-.08f,.70f,.34f,-.82f,-.46f,.86f,-.01f,-.42f,.41f,.74f,-.03f,-.69f,.38f,.03f,.94f,-.81f,-.44f,.26f,-.48f,-.01f,.81f,.41f,-.87f,-.35f,-.23f,-.32f,.19f,-.31f,-.36f,-.39f,.96f,-.03f,.64f,.65f,-.40f,.80f,.61f,.83f,.37f,.63f,.42f,-.01f,.95f,-.67f,.59f,-.02f,-.06f,.63f,.20f,.27f,-.15f,.13f,.49f,-.43f,.92f,-.76f,.20f,-.35f,.20f,-.82f,-.79f,.09f,-.62f,.60f,-.20f,-.89f,-.22f,-.63f,.31f,.27f,-.67f,.23f,.13f,.18f,-.82f,.95f,.52f,-.26f,.55f,-.77f,-.29f,.43f,-.34f,-.61f,.72f,.21f,-.21f,.21f,.47f,-.02f,-.13f,.10f,.81f,.73f,-.50f,-.59f,-.32f,-.18f,-.02f,.19f,-.58f,-.35f,-.52f,.40f,-.88f,-.87f,.75f,-.08f,-.36f,.49f,.46f,.72f,.47f,-.69f,-.53f,-.67f,.27f,.51f,.73f,-.70f,-.52f,.83f,.03f,-.03f,-.72f,.22f,.29f,-.92f,-.07f,-.58f,.21f,-.62f,.08f,.52f,-.70f,.68f,-.72f,-.66f,.24f,.80f,.48f,.46f,-.41f,-.47f,.06f,-.11f,-.30f,.92f,-.28f,.53f,-.08f,.28f,.55f,.87f,.65f,-.00f,-.44f,-.17f,.35f,-.43f,.01f,-.92f,.01f,-.33f,.21f,.34f,.67f,.48f,-.23f,-.08f,.07f,-.17f,-.02f,-.62f,-.56f,.47f,-.10f,.49f,-.33f,.04f,-.92f,-.68f,.80f,.49f,.42f,.86f,-.23f,-.74f,-.64f,.83f,.39f,-.53f,-.69f,-.86f,-.30f,.22f,-.85f,.78f,.00f,-.86f,.94f,.33f,.85f,.33f,-.34f,.06f,-.25f,.26f,-.88f,.90f,-.47f,-.29f,-.56f,-.59f,.66f,-.06f,-.03f,.75f,-.64f,.03f,-.15f,.69f,-.59f,.66f,.37f,-.21f,.04f,-.24f,.49f,-.30f,-.27f,-.93f,-.16f,.73f,-.22f,-.63f,.98f,-.47f,.60f,-.67f,-.31f,-.24f,-.57f,.10f,.49f,-.52f,-.81f,-.28f,.13f,-.36f,.87f,-.61f,.38f,-.35f,.05f,-.01f,.38f,.92f,-.43f,-.87f,.35f,.84f,.47f,.19f,.71f,-.16f,.32f,.64f,-.27f,.30f,-.88f,.24f,.97f,-.02f,.59f,-.45f,-.13f,.32f,.40f,-.69f,.24f,-.06f,-.29f,-.74f,.01f,.36f,.16f,.18f,-.89f,-.62f,.63f,-.83f,.37f,.88f,.53f,.43f,.16f,.72f,.94f,-.04f,.86f,.64f,.58f,-.03f,-.83f,.78f,-.96f,-.11f,-.30f,.20f,-.60f,-.02f,.52f,.62f,-.99f,-.96f,.17f,-.79f,-.87f,-.32f,.39f,.31f,.02f,-.84f,-.77f,.49f,.65f,.83f,-.30f,.98f,-.13f,-.94f,.14f,.40f,-.47f,.81f,.88f,-.75f,.88f,.54f,.89f,-.52f,.92f,-.10f,.11f,.83f,.11f,-.99f,.22f,-.41f,.00f,.80f,-.53f,.19f,.26f,-.17f,-.70f,-.56f,-.20f,.26f,.91f,-.83f,.67f,.56f,.83f,.43f,-.39f,.28f,.83f,-.43f,.92f,.88f,-.68f,-.37f,.62f,-.68f,.70f,-.16f,-.78f,.88f,-.84f,.32f,-.22f,.72f,.58f,.40f,.24f,.90f,-.08f,-.03f,-.89f,.55f,-.76f,-.34f,-.15f,.19f,.36f,.46f,-.60f,-.43f,.54f,-.38f,-.02f,-.21f,-.60f,-.74f,.92f,.65f,.47f,.75f,.58f,.95f,-.22f,-.64f,-.28f,-.12f,.84f,-.37f,-.18f,-.50f,.08f,.14f,.31f,-.56f,-.18f,.32f,.52f,.46f,.19f,.31f,-.48f,-.95f,-.31f,.83f,-.31f,.22f,.41f,-.02f,.05f,.60f,.36f,.78f,.50f,.50f,.61f,-.59f,-.00f,.50f,.96f,-.80f,-.56f,.92f,-.00f,.25f,.79f,.76f,.14f,-.00f,-.99f,-.85f,.81f,-.26f,-.58f,.21f,-.68f,-.90f,.68f,-.48f,.59f,.08f,.62f,.38f,.64f,-.54f,.65f,.60f,-.45f,-.54f,-.40f,.13f,-.94f,-.86f,-.67f,.09f,-.53f,-.45f,.23f,.19f,.62f,-.18f,-.51f,.56f,.81f,.55f,-.62f,.60f,.44f,.61f,.58f,.13f,.13f,.46f,-.04f,-.01f,-.61f,-.76f,-.87f,-.25f,.69f,-.29f,.53f,-.59f,-.22f,.22f,-.08f,.77f,-.08f,.65f,-.90f,.19f,.82f,-.26f,.00f,.12f,.22f,-.88f,.52f,-.37f,-.97f,.34f,.94f,.84f,.88f,.35f,.31f,-.22f,.41f,-.96f,.93f,-.43f,.30f,-.20f,-.41f,.96f,.36f,-.88f,.72f,.32f,-.23f,.37f,-.80f,.05f,.52f,.50f,-.94f,-.43f,-.05f,-.44f,.87f,-.11f,-.01f,.85f,.41f,.30f,-.13f,-.02f,-.49f,-.68f,-.15f,-.49f,-.04f,-.40f,-.56f,.32f,.70f,-.38f,-.12f,-.64f,-.69f,-.22f,.40f,-.77f,.44f,-.36f,-.68f,-.45f,-.39f,.00f,-.44f,.73f,.14f,-.21f,-.50f,-.28f,.04f,.25f,.24f,-.70f,-.04f,.78f,.90f,-.37f,.14f,-.31f,-.98f,-.23f,.15f,-.04f,-.43f,-.51f,.34f,-.69f,-.33f,-.00f,.33f,.77f,.55f,.88f,.56f,-.06f,.86f,.61f,-.97f,.42f,.96f,-.23f,.56f,-.82f,.41f,.87f,-.93f,-.40f,-.71f,-.88f,.49f,-.39f,-.66f,-.16f,.57f,-.77f,.15f,-.36f,-.13f,.69f,.36f,.75f,-.84f,-.44f,.77f,.69f,-.89f,.90f,-.91f,.09f,.86f,.27f,.83f,.75f,-.17f,.80f,.40f,.20f,-.40f,.87f,-.85f,-.47f,-.40f,-.47f,-.17f,-.42f,-.14f,-.83f,.04f,.80f,.72f,-.07f,-.24f,-.32f,.91f,-.67f,.27f,.65f,.68f,-.94f,-.88f,-.27f,.74f,-.36f,.88f,-.78f,.44f,-.31f,.21f,.57f,.01f,.25f,-.02f,.44f,-.88f,.96f,-.91f,.66f,.88f,.97f,.53f,-.90f,.07f,.70f,-.41f,-.52f,-.00f,-.48f,.27f,-.19f,.96f,-.00f,.23f,.13f,-.56f,-.69f,.66f,.19f,-.79f,.87f,.91f,-.35f,-.67f,.01f,-.64f,-.48f,.77f,.29f,-.32f,-.36f,-.07f,-.36f,-.03f,.51f,-.99f,.42f,-.95f,.31f,.96f,.58f,.91f,.96f,.42f,-.76f,-.73f,-.52f,.64f,.24f,-.28f,-.50f,.56f,.50f,.41f,.54f,.10f,.10f,-.57f,-.73f,.50f,.36f,-.71f,.49f,-.08f,-.33f,-.78f,-.77f,.84f,.40f,.91f,-.39f,.26f,-.76f,-.25f,-.15f,-.07f,-.34f,.82f,-.94f,.21f,-.99f,.33f,.79f,-.41f,.71f,-.40f,.87f,.82f,-.34f,-.99f,.35f,-.41f,.10f,.60f,.84f,-.28f,.46f,.63f,-.47f,-.24f,-.05f,-.92f,.55f,.59f,-.23f,.92f,.86f,-.16f,-.51f,-.63f,-.34f,.56f,-.98f,-.07f,.64f,-.83f,.71f,.40f,-.47f,-.67f,.80f,-.64f,.86f,-.00f,-.64f,-.54f,.53f,-.54f,.13f,-.73f,.71f,.44f,.62f,.17f,.80f,.82f,-.87f,-.54f,-.56f,.35f,.02f,.18f,-.06f,-.00f,.13f,.16f,.18f,-.11f,.35f,.70f,.84f,.48f,-.03f,.07f,.20f,-.84f,-.18f,-.30f,.79f,-.88f,-.28f,.64f,.70f,-.05f,-.18f,.59f,-.87f,-.98f,.57f,.11f,.84f,-.33f,.55f,-.89f,-.07f,-.64f,-.17f,.74f,.70f,-.54f,.42f,-.64f,-.74f,-.90f,-.40f,.69f,.62f,.26f,.50f,-.61f,.60f,.54f,.23f,.30f,.68f,.96f,-.19f,.38f,-.73f,.04f,-.44f,.51f,.87f,.56f,.72f,.26f,-.61f,-.36f,.67f,.64f,-.67f,-.85f,-.30f,.37f,-.34f,-.00f,.30f,.86f,-.19f,.08f,.46f,-.14f,.45f,-.52f,.40f,-.69f,-.60f,-.53f,-.35f,-.15f,-.01f,-.93f,-.97f,.48f,.10f,.64f,.61f,-.02f,-.32f,.67f,.30f,-.73f,-.75f,-.24f,.43f,-.68f,-.09f,.76f,-.50f,-.85f,-.94f,-.73f,.85f,-.73f,-.93f,.52f,-.67f,.67f,.15f,.25f,.24f,-.06f,-.20f,-.43f,.77f,-.09f,.81f,.28f,.32f,-.34f,.15f,.95f,.48f,.37f,-.39f,.92f,.06f,.97f,-.59f,.93f,-.22f,-.16f,-.29f,.00f,.70f,-.78f,.56f,-.05f,.39f,.23f,-.89f,.29f,-.08f,-.67f,-.51f,-.09f,.64f,-.75f,-.89f,.91f,-.05f,-.33f,.84f,-.74f,-.95f,-.12f,.59f,.27f,.03f,.98f,-.16f,.63f,.24f,.65f,-.64f,-.89f,.27f,.55f,-.22f,-.27f,.68f,.89f,-.78f,.36f,-.98f,-.53f,.44f,.72f,-.00f,-.08f,-.58f,.96f,.05f,-.75f,.69f,.14f,-.55f,.83f,-.66f,.30f,-.85f,.63f,.03f,-.10f,.73f,.04f,.70f,-.95f,-.52f,-.45f,.97f,-.16f,-.31f,-.61f,-.55f,.58f,.20f,-.17f,-.80f,.75f,-.31f,-.48f,.81f,-.61f,.59f,-.21f,-.16f,.15f,-.84f,.90f,-.53f,.82f,.17f,-.75f,-.51f,-.00f,-.00f,-.07f,-.25f,-.40f,-.09f,-.39f,.50f,-.63f,.76f,-.80f,.03f,.44f,-.18f,-.75f,.08f,.55f,.22f,-.88f,.47f,.32f,.40f,.58f,-.08f,.50f,.40f,.41f,-.28f,.40f,-.30f,-.39f,.32f,.78f,.56f,-.55f,.08f,-.50f,-.94f,-.69f,.88f,-.97f,-.67f,.98f,.44f,-.78f,.08f,-.66f,.83f,-.84f,.93f,-.55f,.03f,-.25f,-.91f,-.95f,.88f,-.19f,-.23f,.67f,-.21f,-.03f,-.10f,-.77f,-.27f,.19f,-.37f,.95f,.29f,-.43f,.94f,.47f,.96f,-.92f,.26f,-.20f,-.00f,-.55f,.37f,-.76f,-.02f,-.30f,-.47f,.65f,-.14f,.09f,-.26f,-.19f,-.63f,-.28f,.58f,.87f,.87f,.50f,.73f,.96f,-.16f,.55f,.68f,-.62f,-.81f,.32f,-.47f,.29f,.53f,-.19f,-.98f,-.41f,.29f,-.48f,-.16f,.11f,.09f,-.27f,-.97f,.95f,.36f,-.66f,-.68f,.41f,.01f,.07f,-.85f,.13f,-.02f,.31f,-.53f,-.13f,-.29f,-.43f,.68f,.14f,.85f,-.57f,-.63f,-.46f,-.44f,-.70f,-.27f,.68f,.68f,.90f,-.59f,.33f,.63f,.40f,.69f,.26f,.44f,-.91f,-.00f,-.60f,-.45f,-.51f,.42f,.53f,-.19f,-.97f,-.79f,.98f,.99f,-.05f,-.17f,.16f,-.77f,.14f,.34f,.15f,.15f,-.00f,.27f,-.17f,.68f,.37f,.50f,-.14f,-.03f,-.70f,-.06f,-.33f,.50f,.57f,-.37f,-.96f,-.89f,-.43f,-.45f,.11f,-.33f,-.19f,-.62f,.81f,.56f,-.38f,.69f,.61f,-.30f,-.58f,-.67f,-.72f,-.83f,.64f,.94f,-.61f,.83f,-.43f,-.73f,-.67f,.96f,-.73f,.61f,.89f,-.97f,.80f,.30f,-.42f,.92f,.81f,.03f,.48f,.98f,-.04f,-.68f,.13f,.90f,.17f,.59f,.48f,-.40f,-.70f,-.25f,-.46f,.69f,-.02f,-.53f,-.60f,.55f,-.49f,-.47f,-.83f,.59f,-.26f,.21f,.86f,-.51f,-.26f,.59f,-.18f,-.93f,.55f,-.20f,-.92f,.87f,.95f,-.32f,.84f,.78f,.43f,.56f,.43f,.36f,.14f,-.19f,-.84f,-.30f,.48f,.63f,-.62f,-.54f,-.28f,-.08f,.06f,.71f,.64f,.60f,.88f,-.68f,-.91f,-.03f,.79f,.69f,-.51f,-.24f,.95f,-.41f,.01f,.97f,.94f,-.74f,.28f,.55f,-.48f,-.63f,-.85f,-.70f,.56f,.37f,.96f,-.80f,-.08f,.39f,.92f,-.02f,.85f,-.28f,-.14f,.81f,.35f,-.17f,-.39f,.12f,.36f,-.06f,.88f,-.59f,-.25f,-.50f,-.34f,.71f,-.72f,-.75f,.40f,.29f,.29f,.61f,.49f,-.79f,.05f,.03f,.48f,.58f,-.38f,.98f,.60f,-.79f,-.15f,-.24f,.37f,.51f,.95f,-.94f,.30f,-.58f,.62f,.57f,-.86f,.69f,-.18f,-.29f,.33f,.56f,-.37f,.02f,.12f,-.97f,.80f,-.46f,.55f,-.82f,-.60f,-.86f,-.80f,.60f,-.09f,-.51f,-.46f,-.86f,-.46f,-.30f,-.86f,.33f,.31f,-.73f,-.83f,.33f,.35f,-.26f,.58f,.75f,.89f,-.10f,-.55f,-.37f,.51f,.14f,-.37f,.58f,.08f,.54f,.37f,.43f,-.03f,.18f,-.48f,.27f,-.22f,.57f,-.27f,-.89f,.05f,.53f,-.71f,-.12f,-.53f,.57f,-.38f,-.22f,.49f,-.47f,.58f,-.26f,-.15f,.47f,.32f,.97f,-.82f,.70f,.15f,-.08f,.16f,-.76f,.14f,-.75f,-.49f,.68f,-.31f,.02f,.20f,.66f,.16f,-.10f,.13f,-.48f,-.78f,-.81f,-.61f,-.84f,-.23f,-.65f,.18f,.72f,.50f,-.09f,.56f,.97f,-.98f,-.49f,.59f,.37f,.32f,-.61f,.24f,-.76f,-.36f,-.69f,.02f,-.29f,.22f,.70f,.49f,-.99f,-.88f,.28f,-.38f,.17f,.73f,-.86f,-.17f,-.79f,.84f,.47f,.86f,-.23f,-.03f,-.69f,-.38f,-.92f,-.86f,-.46f,-.11f,.32f,.37f,-.29f,-.41f,-.05f,-.11f,.48f,-.16f,-.74f,-.83f,-.07f,-.04f,.14f,.31f,.39f,-.91f,.64f,-.05f,.22f,-.61f,-.70f,-.82f,-.45f,.31f,-.48f,-.31f,.61f,-.31f,.40f,.86f,-.92f,.16f,.02f,.49f,-.60f,-.72f,.71f,.03f,-.67f,.27f,.94f,.06f,.47f,.24f,-.72f,-.41f,.08f,.83f,-.81f,-.61f,-.48f,.08f,-.06f,.35f,-.65f,.91f,-.77f,-.90f,.38f,-.42f,-.48f,.37f,-.71f,.36f,.57f,-.08f,.36f,-.87f,.70f,-.53f,-.00f,.65f,-.73f,.19f,-.09f,.42f,.74f,-.29f,-.31f,-.66f,-.01f,-.40f,.71f,-.36f,.69f,.67f,-.86f,.64f,-.02f,.06f,.49f,.42f,.34f,-.35f,-.58f,-.94f,-.17f,.54f,-.91f,-.31f,.54f,-.19f,.69f,-.39f,.65f,.18f,-.54f,.64f,.43f,-.12f,.44f,-.95f,-.62f,.15f,-.71f,.00f,-.33f,-.30f,-.32f,-.36f,.44f,-.96f,-.65f,.02f,-.21f,.34f,.03f,.10f,-.13f,.58f,.40f,.10f,.75f,-.32f,-.22f,-.48f,.87f,-.03f,-.02f,.82f,.06f,.56f,.41f,.16f,.41f,-.52f,.38f,-.09f,.07f,-.42f,.70f,-.16f,-.13f,.22f,.83f,.56f,-.03f,.20f,-.34f,-.52f,.50f,.73f,.34f,.05f,-.39f,.61f,.67f,-.63f,.27f,-.73f,-.73f,.84f,-.65f,-.51f,-.05f,-.46f,-.38f,-.00f,.38f,-.40f,-.52f,.51f,-.63f,-.40f,-.02f,.39f,.18f,.60f,.30f,.32f,.23f,.77f,.21f,.12f,.49f,.30f,.72f,-.85f,.96f,.23f,-.08f,.57f,.20f,.87f,.99f,.54f,-.76f,.11f,.97f,-.10f,.19f,-.45f,-.12f,.85f,-.21f,.69f,.71f,-.61f,.01f,-.73f,-.61f,.51f,.33f,-.71f,.76f,.67f,.35f,-.51f,-.62f,-.46f,.28f,-.80f,.62f,.65f,-.11f,.87f,.24f,-.97f,.60f,-.03f,-.83f,.32f,-.49f,-.94f,-.42f,-.47f,.50f,.34f,.84f,.11f,-.86f,-.24f,.69f,-.53f,.89f,.00f,.08f,-.81f,-.07f,-.18f,-.52f,-.39f,-.94f,.04f,.89f,-.30f,.44f,-.68f,.49f,.40f,.65f,-.80f,-.72f,.42f,-.49f,.91f,.45f,.26f,.60f,.00f,.20f,.27f,-.00f,.27f,.91f,.44f,.84f,.81f,-.09f,.38f,-.49f,.56f,.04f,.54f,-.79f,.05f,.56f,.34f,-.20f,.12f,.00f,.48f,.18f,.15f,-.86f,.57f,-.03f,-.12f,.56f,-.33f,-.80f,.03f,.00f,.90f,.00f,.50f,.15f,-.09f,-.43f,.17f,-.28f,.21f,-.86f,-.41f,.69f,-.40f,.11f,-.95f,-.08f,-.98f,-.67f,.73f,-.97f,-.67f,-.09f,.52f,-.25f,.26f,.09f,-.66f,-.16f,.01f,.86f,-.11f,-.78f,.15f,-.02f,-.23f,.18f,.91f,.29f,.21f,-.53f,-.65f,.79f,-.68f,.87f,-.22f,-.07f,-.43f,.50f,.26f,.05f,.55f,-.30f,-.16f,-.43f,.56f,-.16f,.03f,-.92f,.32f,-.69f,.05f,-.34f,-.90f,.67f,-.24f,-.24f,.97f,.18f,-.31f,-.45f,-.98f,.94f,.84f,.52f,.80f,.71f,-.26f,.04f,-.78f,.10f,-.21f,-.21f,.42f,-.36f,-.98f,-.29f,-.79f,-.29f,-.89f,-.82f,.17f,-.11f,.59f,-.30f,.43f,.40f,.54f,-.35f,-.60f,-.30f,.61f,.52f,.20f,-.40f,.07f,.81f,.50f,.00f,.26f,.06f,.01f,-.43f,.68f,-.25f,-.28f,.01f,.29f,.87f,.68f,.18f,-.26f,-.16f,-.43f,-.59f,.74f,.91f,.33f,.88f,.07f,-.67f,.26f,-.88f,-.31f,-.25f,.63f,.08f,-.60f,-.08f,-.29f,-.41f,-.12f,-.65f,-.20f,-.73f,.50f,-.50f,-.55f,.80f,.01f,-.47f,-.33f,.52f,-.59f,-.61f,.99f,-.94f,.32f,-.50f,-.41f,.66f,-.06f,-.30f,.79f,.33f,.21f,-.79f,.09f,.66f,-.13f,-.32f,.05f,.32f,.07f,-.11f,-.16f,.72f,.69f,.76f,-.14f,-.96f,-.26f,.93f,.84f,-.00f,.23f,-.68f,.65f,.64f,-.37f,.94f,-.70f,-.04f,-.39f,.90f,-.62f,.02f,.88f,.77f,-.40f,.01f,-.45f,-.92f,-.18f,.79f,.52f,.86f,.57f,-.39f,-.39f,-.47f,.47f,-.70f,-.10f,.27f,.99f,-.62f,-.11f,-.99f,-.68f,.81f,-.94f,.08f,-.01f,.42f,.50f,.05f,.67f,.87f,-.11f,-.55f,-.20f,.12f,-.77f,.50f,-.46f,-.69f,-.76f,-.21f,-.49f,.08f,-.29f,-.44f,.59f,.17f,-.34f,.57f,-.00f,-.21f,.65f,.04f,-.26f,-.73f,.95f,.28f,-.33f,-.95f,-.55f,-.52f,-.93f,.57f,-.56f,-.50f,-.62f,-.73f,.94f,-.37f,.04f,-.09f,-.12f,-.71f,-.38f,-.08f,-.56f,.27f,.45f,-.47f,.84f,.24f,-.76f,-.10f,.96f,.72f,.67f,.04f,.91f,.50f,.29f,-.34f,-.87f,.00f,-.74f,.76f,-.20f,.38f,.54f,-.69f,.54f,-.24f,-.16f,.79f,-.17f,.53f,-.25f,-.54f,.40f,-.78f,-.41f,-.25f,.85f,.70f,-.77f,-.23f,-.96f,.75f,.52f,.59f,-.85f,.70f,-.23f,-.71f,-.22f,.87f,.44f,.57f,.75f,.44f,-.44f,.45f,.41f,-.90f,-.86f,.51f,.56f,-.41f,-.55f,.53f,.78f,-.71f,-.43f,-.20f,.98f,-.39f,-.42f,-.52f,.40f,-.67f,-.32f,.12f,.95f,.99f,-.75f,.97f,-.33f,-.24f,.31f,-.50f,-.54f,-.20f,.92f,-.81f,.38f,-.62f,-.39f,.82f,.91f,-.75f,.03f,-.06f,-.61f,-.17f,-.91f,.27f,.30f,-.75f,-.51f,.50f,.60f,-.96f,-.91f,-.52f,.34f,.92f,.31f,-.02f,.89f,.93f,-.77f,.53f,.98f,.01f,.45f,.56f,-.41f,.38f,-.96f,.72f,-.11f,-.05f,.88f,-.78f,-.27f,-.68f,-.41f,-.16f,-.08f,.70f,.16f,.13f,-.29f,.53f,-.52f,-.75f,.99f,.73f,.11f,.11f,-.05f,.27f,-.87f,-.19f,-.59f,-.73f,.74f,-.18f,.13f,-.32f,.55f,.03f,-.20f,.45f,-.06f,-.10f,-.86f,.94f,-.63f,.88f,.23f,.28f,-.62f,-.59f,.76f,.41f,-.29f,.55f,.01f,.36f,.33f,.06f,.67f,-.63f,-.36f,.77f,-.72f,.76f,-.87f,.95f,-.28f,-.83f,-.67f,-.64f,-.49f,.55f,.06f,.90f,-.01f,-.58f,.94f,-.76f,-.24f,.58f,-.99f,.17f,-.14f,.65f,-.80f,.77f,-.18f,-.37f,.07f,-.71f,-.31f,.43f,.59f,-.52f,-.73f,-.17f,.04f,-.25f,-.62f,.90f,.40f,-.60f,-.31f,.52f,-.60f,.90f,.78f,.13f,-.27f,-.71f,.71f,.24f,.93f,-.39f,.33f,-.02f,.45f,.61f,-.50f,.81f,.95f,-.70f,.72f,.03f,-.16f,.07f,-.28f,-.83f,.84f,-.48f,-.42f,.62f,.62f,.39f,.88f,.03f,-.41f,-.49f,-.80f,-.60f,.55f,.83f,-.02f,-.69f,-.19f,.65f,.49f,-.32f,-.28f,.05f,-.40f,-.03f,-.39f,.48f,-.75f,-.02f,-.91f,-.25f,.33f,.80f,-.30f,-.14f,-.36f,.61f,.82f,.48f,.31f,-.87f,.45f,.06f,.15f,.08f,-.39f,.10f,-.21f,.63f,-.47f,.55f,.47f,.90f,.32f,-.29f,.79f,-.76f,.75f,-.90f,.56f,.92f,-.04f,-.73f,-.75f,-.68f,-.74f,.99f,-.68f,-.49f,.31f,.76f,-.76f,-.03f,-.95f,-.75f,-.89f,-.32f,.91f,-.34f,.58f,-.43f,.33f,-.88f,-.32f,.90f,.23f,-.40f,-.86f,.81f,.26f,-.90f,.78f,-.61f,.65f,-.75f,.69f,-.29f,-.17f,-.92f,.84f,.80f,-.35f,.53f,.73f,-.60f,-.75f,-.46f,-.68f,-.13f,.80f,-.21f,.38f,.89f,-.53f,.34f,-.32f,.71f,.08f,-.43f,.27f,-.17f,.48f,.75f,-.89f,-.47f,-.94f,-.23f,.52f,-.47f,.34f,-.84f,-.81f,-.14f,.36f,-.73f,-.35f,-.06f,-.66f,-.16f,-.84f,-.95f,-.52f,-.02f,-.28f,.85f,.91f,-.61f,-.16f,-.82f,.10f,.19f,-.51f,.04f,-.23f,.74f,-.18f,.31f,-.63f,-.37f,-.27f,-.95f,-.36f,-.32f,-.85f,.52f,.16f,.75f,-.68f,-.99f,-.18f,-.82f,.90f,.61f,-.96f,.55f,.61f,-.82f,.15f,.21f,-.46f,.06f,.35f,.95f,.25f,-.10f,-.08f,.58f,.14f,-.54f,.21f,.48f,.53f,.09f,.75f,.82f,-.12f,-.45f,.65f,.29f,-.04f,.43f,.84f,-.97f,.04f,-.14f,-.02f,.09f,-.46f,.01f,-.90f,-.54f,-.71f,-.03f,-.45f,-.02f,-.49f,-.71f,.06f,-.48f,.90f,.83f,.20f,.42f,-.99f,-.11f,.53f,-.35f,-.01f,.67f,-.68f,-.08f,-.52f,.44f,-.61f,-.58f,-.15f,-.47f,.90f,.37f,-.05f,-.52f,.54f,.88f,.19f,-.36f,.19f,.82f,.04f,-.37f,.36f,.97f,-.93f,.97f,-.82f,.28f,.17f,-.19f,.17f,-.24f,-.69f,.48f,-.01f,.43f,-.70f,.98f,-.49f,-.51f,.97f,.05f,-.87f,.57f,-.93f,-.45f,.78f,-.22f,.70f,.91f,-.54f,.67f,-.53f,-.01f,.38f,.99f,-.56f,.85f,.67f,.88f,-.02f,.33f,-.00f,.74f,.33f,-.37f,-.92f,.79f,-.93f,.24f,.18f,-.56f,-.83f,-.22f,.79f,-.04f,.54f,-.11f,-.40f,.27f,.44f,.50f,.24f,.96f,-.02f,.24f,.18f,-.95f,-.03f,.93f,-.15f,.86f,-.24f,-.30f,.80f,-.05f,.91f,.55f,.05f,.44f,-.43f,.00f,-.96f,-.55f,.52f,-.11f,-.56f,-.08f,.91f,.05f,-.70f,.13f,.43f,.07f,.81f,-.49f,.14f,-.89f,-.38f,.19f,.57f,.94f,.66f,.98f,.75f,.37f,-.12f,.12f,.96f,.40f,-.10f,-.64f,-.88f,-.64f,.95f,-.81f,.61f,-.97f,-.68f,-.73f,-.00f,-.69f,-.99f,.73f,-.09f,-.49f,.22f,.04f,.77f,.79f,-.06f,-.35f,.06f,.81f,-.03f,-.05f,.47f,.27f,-.74f,-.59f,.82f,.47f,.80f,.80f,.58f,-.87f,.05f,.50f,-.26f,-.50f,-.84f,.12f,.62f,.66f,-.77f,-.10f,.74f,.66f,-.07f,.73f,.74f,.52f,.10f,-.78f,.24f,.92f,.02f,-.12f,.80f,.60f,.76f,.96f,.57f,.94f,-.37f,-.01f,-.20f,-.95f,-.37f,-.61f,.74f,.82f,.31f,.32f,-.47f,-.91f,-.60f,-.78f,.19f,-.65f,.44f,-.63f,-.19f,-.62f,.39f,-.58f,-.80f,-.47f,.29f,-.31f,.19f,.13f,.59f,-.52f,.32f,.71f,-.69f,-.21f,-.94f,-.80f,-.82f,-.81f,-.92f,-.08f,-.68f,.19f,.31f,-.02f,-.60f,.97f,-.71f,.21f,-.18f,.19f,.83f,.79f,-.18f,-.91f,-.77f,-.80f,-.00f,.08f,.30f,-.86f,-.88f,-.01f,-.88f,.71f,-.80f,-.54f,.43f,-.68f,-.56f,.32f,.05f,.21f,-.51f,.17f,-.42f,.01f,.00f,.03f,.16f,-.77f,.62f,-.91f,-.70f,-.42f,-.49f,.46f,-.82f,.97f,-.49f,-.49f,.42f,.23f,-.19f,.23f,.99f,-.57f,-.93f,-.46f,.05f,-.86f,.23f,.94f,-.68f,.73f,.08f,.33f,.63f,.65f,.11f,.41f,-.16f,-.05f,.80f,.65f,.07f,-.76f,-.69f,.34f,.18f,.69f,.58f,.67f,.56f,.02f,-.72f,.07f,.38f,-.30f,.59f,.00f,-.67f,.52f,.40f,.60f,-.26f,-.16f,-.99f,-.89f,.48f,.03f,-.59f,-.62f,-.51f,.10f,.79f,.47f,.66f,.23f,-.51f,-.08f,-.84f,-.38f,.27f,.11f,-.46f,.50f,-.75f,-.49f,-.44f,-.86f,-.46f,.70f,-.21f,-.33f,-.69f,.91f,.79f,.29f,-.90f,.70f,-.92f,-.49f,.85f,-.58f,-.71f,.53f,.88f,.29f,.35f,.75f,-.23f,.90f,-.49f,.04f,.07f,.13f,-.12f,-.84f,-.09f,-.19f,-.26f,-.73f,-.66f,-.83f,-.14f,.38f,-.95f,.39f,-.69f,.46f,.44f,.53f,-.38f,.01f,-.41f,.10f,.90f,-.36f,.58f,.41f,.11f,-.73f,.14f,.97f,.89f,-.24f,.49f,.65f,-.52f,.63f,-.82f,.34f,.95f,-.90f,-.19f,-.48f,-.05f,-.92f,-.48f,-.09f,-.72f,.64f,-.92f,.64f,-.85f,-.78f,-.44f,-.67f,-.48f,.91f,-.45f,.23f,-.35f,-.10f,.49f,.65f,.29f,.03f,.59f,-.92f,.24f,.48f,.86f,.29f,.77f,.40f,-.56f,-.27f,-.99f,.95f,.99f,-.65f,.71f,.21f,-.14f,.51f,.25f,-.31f,.73f,.37f,.62f,.58f,-.81f,-.68f,.41f,-.88f,-.20f,.40f,-.71f,-.71f,.97f,-.28f,-.73f,.36f,-.57f,-.56f,-.33f,.98f,-.32f,.25f,-.85f,-.16f,-.14f,-.17f,-.57f,.12f,.03f,-.77f,.12f,-.58f,-.90f,.51f,-.29f,-.47f,.89f,-.81f,.69f,.70f,-.40f,.15f,-.66f,.54f,-.75f,-.81f,-.87f,-.26f,.31f,-.56f,-.15f,.04f,-.86f,.07f,.19f,-.80f,-.94f,.04f,-.01f,.87f,.83f,-.44f,-.77f,-.73f,.24f,-.87f,-.61f,.85f,.40f,-.83f,.18f,-.88f,.49f,-.27f,-.25f,-.59f,-.47f,-.91f,-.81f,-.60f,-.73f,.78f,-.81f,.70f,.20f,.44f,-.60f,-.82f,-.56f,.90f,.08f,-.00f,.06f,.17f,-.42f,.34f,.80f,-.92f,.02f,.44f,-.54f,.45f,.69f,-.19f,-.06f,.32f,-.38f,.72f,.96f,.47f,-.56f,-.50f,.09f,-.60f,-.92f,.10f,-.34f,.16f,.74f,.03f,-.54f,.63f,.60f,.69f,.20f,-.43f,.31f,-.68f,-.64f,-.46f,.61f,-.74f,.56f,.11f,-.44f,-.02f,-.38f,-.96f,.95f,.38f,.24f,-.74f,.91f,-.54f,-.61f,.67f,.38f,.91f,.05f,-.53f,-.08f,.04f,-.86f,-.90f,-.92f,.08f,-.49f,.22f,.92f,.76f,-.30f,.85f,-.76f,.15f,.28f,-.52f,-.16f,.84f,.48f,.70f,.62f,-.52f,.37f,-.43f,.52f,-.87f,.91f,.13f,.15f,.10f,.62f,.72f,-.69f,-.53f,-.09f,-.37f,.76f,-.76f,-.50f,.86f,-.17f,-.73f,-.39f,-.05f,.91f,-.26f,.39f,-.69f,.26f,.97f,-.23f,.61f,.77f,-.28f,.17f,.32f,.85f,-.80f,-.85f,-.03f,-.59f,-.91f,.28f,-.14f,.46f,-.53f,-.75f,-.84f,.52f,-.35f,-.97f,.94f,-.44f,-.26f,.54f,-.51f,.41f,.41f,.89f,-.54f,-.27f,.72f,-.34f,.01f,.19f,.23f,-.30f,-.61f,-.39f,-.11f,.45f,-.22f,-.66f,.06f,.15f,.46f,-.00f,.91f,.25f,.37f,-.13f,.26f,.08f,.78f,-.77f,-.68f,.21f,-.80f,.02f,.33f,.65f,.33f,.02f,-.97f,.01f,-.47f,-.48f,.46f,-.70f,.47f,.01f,-.98f,.49f,-.12f,-.62f,-.45f,.57f,-.05f,.91f,.29f,-.85f,-.14f,-.51f,-.31f,.48f,.81f,-.11f,.10f,.72f,.27f,-.97f,.24f,-.18f,-.09f,-.77f,-.70f,.98f,.50f,.58f,.10f,-.19f,.12f,.05f,.77f,.17f,-.38f,-.17f,.15f,-.18f,.45f,.10f,.86f,.23f,-.74f,-.18f,.41f,.30f,-.75f,-.25f,-.29f,.86f,-.21f,.34f,-.62f,.89f,-.04f,-.34f,-.04f,.38f,-.59f,.02f,.52f,.84f,.14f,.48f,-.89f,-.86f,.12f,-.07f,-.21f,-.94f,-.36f,-.49f,.26f,.70f,.45f,.16f,-.98f,-.85f,-.38f,-.01f,-.31f,-.23f,-.16f,.93f,-.68f,.12f,-.92f,-.55f,-.15f,-.68f,-.59f,-.57f,.79f,-.21f,.17f,-.14f,-.30f,-.35f,.07f,-.15f,-.25f,-.04f,.01f,.28f,-.54f,.85f,-.59f,-.69f,-.18f,.42f,-.93f,-.02f,-.69f,.94f,.11f,-.33f,.34f,.25f,-.77f,-.82f,.33f,-.29f,-.68f,-.52f,-.49f,.57f,.47f,-.60f,-.45f,.69f,.82f,-.15f,-.83f,-.07f,-.79f,-.63f,-.42f,-.62f,-.25f,.78f,.69f,.32f,.36f,.73f,.52f,-.74f,-.11f,.04f,-.96f,-.50f,-.47f,-.88f,-.26f,.67f,-.59f,-.33f,.78f,-.59f,.21f,.13f,-.31f,-.19f,.15f,.26f,-.40f,.43f,-.96f,.75f,-.61f,-.04f,.66f,-.10f,.65f,.12f,-.20f,.98f,.99f,-.75f,.51f,.93f,-.27f,.02f,-.85f,-.62f,-.35f,-.04f,.96f,.85f,.89f,-.38f,-.68f,.02f,-.88f,-.67f,.00f,.92f,-.88f,.56f,.46f,-.72f,.66f,.80f,.48f,.19f,.27f,.15f,-.77f,-.45f,-.94f,.11f,.37f,.22f,-.54f,-.27f,-.67f,-.76f,-.15f,-.20f,.93f,-.05f,.27f,.20f,.72f,-.40f,-.72f,-.49f,.81f,.06f,.83f,.09f,.08f,-.65f,-.26f,.74f,-.22f,-.96f,-.62f,-.45f,.60f,-.10f,.17f,.12f,-.17f,-.74f,.08f,.12f,-.64f,.68f,.92f,.00f,-.04f,.31f,-.83f,-.47f,-.55f,-.89f,-.42f,-.96f,-.62f,.19f,-.47f,.19f,-.46f,-.70f,.39f,-.10f,-.82f,.19f,-.06f,-.47f,-.03f,-.46f,.22f,-.76f,.34f,-.80f,.31f,.41f,.17f,.90f,.47f,.85f,-.94f,.31f,-.81f,.24f,.42f,.20f,.15f,-.24f,-.69f,-.87f,-.65f,.09f,-.25f,.36f,-.38f,.04f,-.80f,.54f,.82f,-.95f,.86f,.40f,.70f,-.34f,-.58f,-.75f,-.37f,.39f,-.62f,.05f,-.76f,-.91f,.65f,.38f,.76f,.36f,-.73f,.06f,-.01f,-.05f,.41f,-.66f,.75f,.56f,.55f,-.09f,.82f,.49f,-.35f,.91f,-.54f,.63f,.00f,-.34f,-.30f,-.08f,.90f,.76f,.41f,-.48f,.95f,.61f,.28f,.41f,-.86f,-.79f,.25f,-.48f,.35f,.22f,-.38f,.33f,.81f,-.85f,-.60f,.87f,.77f,.13f,.90f,.46f,.74f,-.66f,.31f,.00f,.87f,.60f,-.04f,.30f,.61f,.93f,.59f,.19f,.26f,-.93f,.24f,.64f,-.72f,.62f,.31f,-.11f,-.24f,.00f,.79f,.98f,-.43f,-.60f,-.10f,.31f,.38f,-.21f,.81f,-.96f,-.87f,-.66f,.92f,.21f,.53f,.22f,-.25f,-.20f,.02f,.93f,.39f,.38f,.71f,-.38f,.73f,.58f,-.81f,.78f,-.92f,-.26f,.39f,.83f,.85f,.90f,-.26f,.12f,-.40f,-.17f,.54f,.85f,-.38f,-.68f,-.77f,.90f,.08f,.26f,.93f,.52f,-.05f,-.19f,-.64f,.96f,.92f,-.90f,.77f,.05f,-.27f,.76f,.63f,-.15f,.14f,.24f,-.37f,-.39f,.62f,-.68f,-.46f,-.21f,-.72f,.67f,-.75f,-.48f,-.85f,-.06f,-.37f,.07f,.63f,.94f,-.28f,-.88f,-.16f,-.20f,-.23f,.45f,-.02f,-.78f,.76f,-.63f,.09f,-.35f,.91f,.41f,-.89f,.67f,.57f,-.95f,.41f,.62f,-.61f,.17f,-.24f,.34f,.11f,-.25f,-.29f,-.22f,.99f,.73f,-.54f,.39f,-.74f,.32f,.39f,-.40f,.18f,.94f,.94f,.47f,.27f,-.61f,-.62f,.78f,.77f,-.91f,.41f,.67f,-.17f,-.16f,.86f,.50f,.71f,.68f,.54f,.68f,.27f,.35f,-.84f,-.04f,-.06f,.14f,.64f,-.61f,-.51f,-.17f,-.22f,-.55f,.68f,.52f,-.38f,-.18f,.21f,.34f,-.29f,-.85f,-.88f,.92f,.44f,-.02f,.59f,-.06f,-.62f,-.11f,-.72f,-.88f,-.91f,-.59f,-.77f,.67f,.19f,.34f,.05f,.77f,.09f,-.08f,-.60f,.03f,.02f,.00f,.98f,.75f,.96f,.05f,-.09f,.25f,-.53f,.26f,.00f,.26f,.77f,.74f,.38f,.27f,.49f,-.85f,.67f,.59f,.45f,.70f,-.21f,.78f,.74f,-.83f,.85f,-.38f,.78f,-.82f,-.70f,-.36f,.21f,.32f,-.66f,-.33f,.50f,.26f,.59f,-.13f,.86f,-.01f,.52f,-.65f,-.02f,-.82f,.04f,.64f,.62f,.94f,.70f,-.55f,-.03f,.38f,-.45f,.70f,-.17f,.76f,.35f,-.91f,-.85f,.72f,-.18f,-.09f,.54f,-.54f,-.19f,-.97f,-.98f,-.60f,.38f,-.36f,.58f,.56f,.29f,.16f,.27f,-.91f,.53f,-.06f,-.42f,.05f,-.05f,.25f,-.46f,.92f,.37f,-.87f,.14f,.98f,-.33f,.07f,.98f,.16f,-.19f,-.22f,-.45f,.61f,-.77f,.59f,.32f,.34f,.85f,.98f,-.56f,-.46f,-.83f,.64f,-.78f,.33f,.38f,.49f,.90f,-.73f,.50f,.51f,-.25f,-.80f,.17f,-.88f,-.30f,.57f,-.48f,-.83f,-.05f,-.27f,-.28f,.80f,-.26f,-.17f,-.61f,.25f,-.47f,-.65f,-.29f,-.43f,-.88f,-.02f,.97f,.04f,.94f,-.47f,.46f,.19f,-.00f,.70f,.36f,.93f,-.47f,-.85f,-.44f,-.26f,-.44f,.41f,-.50f,-.77f,-.98f,.30f,-.75f,-.53f,-.77f,-.89f,.92f,-.35f,.22f,-.14f,-.50f,.41f,.78f,-.89f,.59f,-.62f,-.82f,.50f,.98f,.85f,.84f,.63f,.40f,.04f,.15f,-.70f,.95f,.10f,.25f,-.02f,-.70f,.88f,-.56f,.56f,-.58f,.52f,-.11f,.02f,-.35f,-.62f,-.15f,-.06f,.15f,.45f,-.99f,.57f,-.19f,.11f,.84f,.02f,-.05f,-.53f,.74f,-.85f,-.07f,.33f,.45f,-.45f,-.74f,.07f,.38f,.60f,.36f,.57f,-.27f,-.54f,-.01f,.68f,-.97f,.67f,-.06f,.67f,.01f,.03f,.65f,.18f,-.81f,.23f,.06f,.45f,.42f,-.17f,.76f,-.27f,-.29f,-.17f,.02f,-.42f,-.46f,-.93f,-.17f,-.78f,-.43f,-.04f,-.69f,.29f,.35f,-.91f,-.97f,.09f,.63f,-.20f,.15f,-.63f,-.96f,-.95f,.60f,.91f,.09f,-.70f,.50f,-.07f,.44f,.01f,.75f,.73f,-.73f,.16f,-.80f,.52f,.64f,-.02f,-.17f,.04f,-.50f,.72f,-.06f,.80f,-.50f,-.78f,-.37f,-.16f,-.06f,-.73f,-.68f,-.45f,.43f,-.41f,-.01f,-.75f,.41f,-.94f,.86f,-.33f,.65f,-.82f,-.53f,-.19f,-.21f,.75f,.39f,-.52f,-.82f,.05f,.29f,-.84f,-.03f,-.05f,-.92f,.66f,.14f,.19f,-.98f,.09f,.48f,.96f,.39f,.67f,-.49f,.42f,.30f,.84f,-.46f,-.41f,.92f,-.93f,.51f,-.90f,-.54f,-.32f,.18f,-.83f,-.05f,.78f,-.76f,.76f,-.50f,-.94f,-.58f,.52f,.45f,-.17f,.62f,.48f,.85f,.11f,.74f,.99f,.94f,-.77f,.44f,-.89f,.97f,-.77f,.16f,-.35f,.16f,-.45f,.50f,-.96f,.74f,-.52f,-.64f,.13f,-.21f,-.03f,-.16f,.35f,.53f,-.82f,.34f,.52f,-.59f,-.17f,-.53f,-.88f,.67f,.13f,-.21f,.93f,-.45f,-.11f,-.60f,.84f,.16f,.76f,.01f,.89f,.87f,.04f,.86f,.20f,.20f,.61f,-.21f,.04f,.26f,.91f,-.40f,.00f,-.26f,-.12f,.35f,.21f,-.87f,.81f,-.03f,-.82f,.19f,-.71f,.93f,.39f,-.70f,-.97f,.20f,.92f,.13f,-.81f,.88f,.34f,-.27f,-.46f,.78f,.94f,-.50f,.57f,.06f,-.52f,.14f,.33f,-.34f,-.31f,-.05f,-.73f,.13f,-.63f,.88f,-.44f,-.15f,.11f,.90f,-.36f,.51f,-.38f,-.64f,-.21f,.13f,.89f,.81f,-.69f,.15f,-.67f,-.89f,-.47f,.73f,-.15f,-.97f,-.24f,-.57f,.91f,-.64f,-.30f,.67f,-.55f,-.68f,.98f,-.98f,.96f,-.36f,-.55f,.11f,-.73f,.57f,.73f,-.46f,-.11f,.52f,.48f,.06f,.43f,-.41f,-.43f,.58f,-.69f,.57f,.71f,-.73f,.19f,-.45f,-.24f,.96f,.61f,.66f,-.50f,-.46f,.32f,.43f,.15f,-.93f,.77f,.60f,-.27f,-.37f,.31f,-.69f,.20f,-.22f,.90f,.51f,-.56f,-.50f,-.55f,.64f,-.03f,.70f,.90f,-.83f,-.11f,-.70f,.82f,.62f,.66f,.26f,.24f,-.55f,.72f,.42f,.58f,.60f,-.02f,-.29f,.73f,-.04f,-.45f,.01f,-.04f,-.59f,.86f,-.29f,.82f,.56f,-.76f,-.58f,.77f,.96f,-.28f,.71f,.19f,-.26f,.98f,.32f,-.75f,-.24f,.85f,-.26f,-.73f,.49f,.26f,-.74f,.08f,-.53f,-.58f,.52f,.41f,.06f,.20f,.47f,-.41f,.14f,.00f,.94f,.76f,-.41f,-.90f,.92f,-.29f,.52f,-.94f,.37f,.87f,-.06f,-.44f,-.33f,-.04f,.78f,-.55f,-.23f,.25f,.08f,.41f,-.48f,.05f,-.37f,.39f,-.74f,-.89f,.51f,-.80f,-.70f,-.94f,.32f,-.43f,-.75f,.63f,-.33f,-.67f,-.33f,.18f,.65f,.34f,.92f,.08f,.92f,.04f,-.33f,-.59f,.62f,.85f,-.49f,.39f,-.37f,-.04f,-.11f,.23f,-.41f,-.84f,.71f,-.35f,.88f,.70f,.94f,.19f,.85f,-.35f,.84f,-.99f,-.70f,.57f,-.78f,-.96f,-.55f,.79f,.67f,.09f,-.28f,-.11f,.22f,.92f,.79f,-.70f,-.62f,.56f,-.70f,-.15f,.18f,.65f,.97f,-.87f,-.04f,-.75f,-.65f,.12f,-.09f,.85f,.98f,.58f,.09f,-.06f,.70f,.19f,-.89f,-.53f,-.26f,-.79f,-.07f,.19f,-.68f,.72f,.87f,-.10f,.68f,-.12f,-.44f,.26f,-.30f,.36f,-.98f,-.76f,-.93f,.93f,-.68f,-.39f,-.06f,.87f,-.19f,.60f,.96f,.91f,.05f,-.95f,.13f,.35f,.87f,.67f,-.48f,.26f,.19f,.74f,-.53f,-.54f,.60f,.08f,.50f,.18f,.89f,.89f,-.19f,.10f,.72f,-.13f,-.38f,-.45f,.19f,.33f,-.01f,-.34f,-.28f,-.42f,-.39f,.48f,.11f,-.66f,.22f,-.08f,.71f,.86f,.02f,-.02f,.39f,-.86f,-.51f,.50f,-.42f,.12f,.44f,-.47f,-.43f,.11f,-.92f,-.07f,.83f,-.69f,-.28f,-.13f,.05f,-.49f,.07f,-.34f,.18f,-.42f,.57f,.44f,.57f,.45f,.97f,-.57f,.30f,-.87f,-.90f,-.11f,-.14f,.89f,.74f,.35f,.30f,-.16f,-.17f,.23f,-.63f,-.95f,.79f,-.39f,.38f,.34f,-.69f,-.45f,-.34f,.96f,-.02f,-.77f,.15f,.37f,-.61f,-.83f,.48f,-.57f,.52f,-.71f,-.02f,-.83f,.34f,.29f,-.05f,.73f,-.15f,.89f,-.17f,-.81f,.59f,.11f,.92f,.31f,.71f,-.89f,-.70f,.31f,.75f,-.59f,-.89f,-.81f,-.81f,.23f,.12f,.26f,.56f,.01f,.24f,-.26f,.61f,-.58f,.57f,-.95f,-.46f,.78f,.06f,-.32f,.88f,.84f,-.25f,-.25f,-.32f,.48f,.28f,-.14f,.48f,.29f,.67f,.89f,.97f,.54f,-.46f,.19f,.67f,-.27f,.19f,-.64f,.68f,.30f,.24f,-.61f,-.28f,-.26f,.32f,-.30f,-.72f,.01f,.19f,-.35f,-.56f,-.02f,.17f,.45f,.07f,-.07f,-.75f,.21f,-.19f,.21f,.89f,-.13f,.51f,.70f,-.00f,-.92f,-.64f,.26f,-.13f,.69f,.28f,-.47f,-.78f,-.32f,.01f,.15f,.36f,-.28f,-.35f,.31f,-.47f,.53f,.51f,.23f,.35f,.97f,.23f,-.09f,.81f,-.26f,.08f,-.17f,-.41f,.78f,-.51f,.16f,-.27f,.05f,-.30f,.32f,.51f,.19f,.94f,.60f,.10f,.21f,.49f,-.69f,.23f,-.87f,.64f,-.36f,-.53f,-.63f,-.05f,-.95f,.23f,.85f,-.32f,-.11f,-.89f,-.13f,.31f,.28f,-.69f,-.37f,-.47f,-.79f,.78f,-.90f,.91f,-.86f,.05f,-.86f,-.74f,.19f,.68f,-.56f,.04f,.10f,-.35f,.47f,-.79f,-.28f,.46f,-.11f,.60f,.72f,-.47f,.03f,.63f,.75f,-.84f,.40f,-.25f,-.72f,.63f,-.22f,-.96f,.43f,-.61f,.73f,-.58f,.78f,-.11f,-.30f,-.59f,-.11f,.01f,.44f,.35f,.42f,-.84f,.13f,.63f,.37f,-.45f,.86f,.90f,.08f,-.50f,.89f,.54f,.50f,-.17f,-.89f,-.72f,.60f,-.75f,.75f,-.79f,.60f,.56f,-.67f,.75f,-.70f,-.15f,-.09f,-.27f,.44f,.74f,-.97f,-.72f,.47f,.83f,.60f,-.97f,.32f,.24f,.33f,.65f,.74f,-.10f,.19f,.30f,-.94f,.07f,-.04f,-.47f,.56f,-.94f,.01f,-.65f,-.18f,.41f,-.34f,-.42f,.56f,.40f,-.17f,-.98f,.62f,.27f,-.27f,.25f,.26f,.41f,-.17f,.18f,-.67f,-.17f,-.78f,.52f,.83f,.04f,-.14f,.35f,.73f,.14f,.72f,-.26f,.83f,-.60f,-.61f,.89f,.30f,.07f,.82f,-.79f,-.68f,-.42f,-.40f,-.99f,-.86f,.48f,.44f,.50f,.51f,.16f,-.77f,-.15f,-.70f,.64f,.97f,-.71f,-.54f,.23f,-.23f,.83f,.53f,-.58f,.62f,-.32f,.84f,-.66f,.95f,.76f,.92f,.48f,-.81f,.43f,.08f,-.67f,.46f,.30f,.23f,.01f,.55f,.02f,.81f,.56f,-.34f,-.69f,.05f,-.99f,.96f,-.84f,.34f,.88f,.01f,.44f,-.19f,.28f,.39f,-.70f,-.24f,-.83f,.44f,.09f,-.08f,.42f,.09f,.24f,-.86f,-.21f,-.90f,-.87f,-.91f,.29f,-.61f,-.84f,.36f,-.40f,-.45f,.46f,.05f,-.55f,.24f,.99f,-.20f,.18f,-.16f,.45f,.90f,.38f,-.37f,.24f,.90f,-.55f,.09f,-.53f,-.14f,.66f,-.59f,-.70f,.24f,.17f,.24f,-.65f,.76f,.32f,.49f,-.65f,.55f,-.59f,-.31f,.78f,-.67f,-.29f,-.60f,.31f,.26f,-.02f,.87f,-.31f,.95f,.14f,.76f,-.89f,.26f,.74f,-.41f,.10f,-.93f,-.42f,-.60f,-.96f,-.31f,.90f,-.20f,.68f,-.57f,.86f,-.71f,.68f,.26f,.75f,.79f,.09f,.94f,.20f,.11f,-.27f,-.06f,.23f,.43f,-.44f,-.87f,.56f,-.34f,.37f,-.49f,-.47f,.10f,.53f,.80f,.53f,-.44f,-.92f,.79f,-.89f,.50f,.82f,-.09f,-.14f,-.88f,.16f,-.15f,.81f,-.96f,.99f,.30f,.09f,-.84f,-.61f,.41f,-.68f,.54f,-.86f,-.87f,-.55f,.69f,.67f,-.43f,.09f,.47f,.85f,-.36f,.27f,-.10f,.93f,.34f,-.77f,.45f,-.37f,.71f,.82f,-.98f,.42f,-.07f,.50f,.66f,.62f,.13f,-.53f,-.83f,-.38f,-.03f,-.49f,-.10f,.21f,.89f,-.90f,.55f,-.62f,.42f,.45f,-.17f,-.04f,-.22f,-.20f,-.10f,.42f,.97f,-.08f,-.58f,-.25f,.02f,-.08f,-.38f,.89f,-.65f,.46f,-.35f,-.54f,.92f,.03f,.58f,-.00f,-.40f,-.55f,.99f,-.46f,-.17f,.17f,-.05f,-.28f,.57f,-.15f,-.41f,.24f,.64f,.55f,-.43f,-.52f,.79f,.59f,.23f,-.41f,-.87f,.82f,-.26f,-.10f,-.14f,-.93f,-.03f,-.51f,.17f,-.53f,-.27f,-.64f,.28f,-.98f,.03f,-.06f,.22f,.65f,.93f,.18f,-.60f,.74f,.13f,.79f,-.20f,-.12f,.08f,-.93f,-.82f,.61f,-.68f,.17f,-.66f,.23f,-.62f,-.23f,.65f,.42f,.29f,-.46f,.97f,-.42f,.85f,-.18f,-.68f,.13f,-.50f,.83f,.59f,-.44f,-.14f,.74f,.46f,-.27f,.38f,-.06f,-.34f,-.52f,.66f,.95f,.69f,-.09f,-.37f,-.35f,.57f,.19f,-.65f,-.70f,.43f,-.01f,.24f,.83f,.28f,.37f,-.18f,-.50f,.10f,.80f,.04f,.11f,.02f,-.44f,.30f,.91f,-.42f,.04f,-.48f,-.25f,-.79f,-.20f,.33f,.22f,-.55f,-.73f,.25f,.53f,.57f,.40f,.53f,-.69f,.86f,.78f,.90f,.79f,-.52f,-.90f,-.16f,.88f,.40f,.75f,.53f,-.91f,-.39f,-.79f,-.26f,.77f,-.56f,-.82f,-.17f,.94f,-.86f,.57f,-.61f,-.86f,.11f,.90f,-.50f,-.38f,.06f,.46f,.60f,-.81f,-.81f,.64f,.13f,.41f,-.12f,-.44f,-.66f,-.86f,-.08f,.16f,.18f,.45f,.84f,-.18f,-.09f,.78f,.67f,.86f,.08f,-.06f,.77f,-.88f,.45f,-.77f,-.06f,-.48f,.81f,-.48f,-.01f,.75f,.43f,-.63f,.65f,-.52f,-.72f,-.56f,-.05f,-.63f,.08f,-.34f,-.72f,-.70f,.84f,-.84f,-.64f,-.80f,.40f,.19f,.07f,.49f,.62f,.17f,-.79f,.62f,-.59f,-.82f,-.40f,-.07f,-.35f,.42f,-.23f,-.92f,.70f,.12f,.93f,.58f,.94f,-.67f,-.13f,-.96f,-.04f,-.57f,.67f,.36f,-.03f,.51f,-.87f,.58f,.85f,.54f,-.10f,-.98f,.64f,-.33f,.70f,.94f,-.53f,-.44f,-.39f,.17f,.34f,.22f,.50f,.63f,-.61f,.15f,-.82f,-.20f,-.79f,-.24f,.58f,-.48f,-.76f,.39f,-.85f,-.52f,-.90f,.00f,.06f,.40f,-.89f,.17f,-.30f,-.79f,-.86f,.44f,.55f,.49f,.54f,.12f,.72f,.17f,.65f,-.74f,-.33f,-.82f,.30f,.24f,-.39f,.47f,.79f,-.31f,.97f,-.17f,.57f,-.63f,.23f,.43f,-.55f,.61f,-.88f,.51f,-.93f,.26f,.34f,-.32f,.22f,-.38f,-.93f,.76f,-.90f,-.02f,.08f,-.07f,-.71f,.61f,-.80f,.81f,-.26f,-.82f,-.18f,.98f,.57f,.17f,-.93f,.67f,-.80f,-.07f,-.43f,.66f,.17f,-.26f,.35f,.15f,.83f,.74f,-.88f,.01f,.81f,-.90f,-.75f,.59f,.00f,-.42f,-.11f,-.54f,-.99f,.49f,.46f,.45f,.27f,.57f,-.88f,.54f,-.59f,.66f,-.71f,-.79f,-.51f,.73f,.53f,-.00f,.52f,-.84f,-.39f,.58f,.17f,-.29f,-.36f,-.57f,-.25f,.36f,.58f,-.06f,-.71f,-.35f,-.87f,-.38f,-.60f,-.75f,.30f,.97f,.55f,.31f,.61f,.59f,-.24f,-.13f,-.91f,-.07f,-.17f,-.82f,-.40f,.68f,.78f,.96f,-.70f,.19f,-.98f,.83f,.77f,-.17f,.26f,-.98f,.62f,-.43f,.74f,.50f,-.36f,-.78f,-.34f,.42f,.62f,.43f,-.17f,.83f,.25f,.51f,.07f,-.40f,.05f,-.69f,.44f,-.88f,.12f,.53f,.24f,.16f,.08f,-.97f,.00f,-.58f,.19f,.59f,.32f,-.46f,.65f,-.27f,-.30f,.63f,-.85f,-.99f,.76f,-.62f,.90f,.68f,-.98f,.40f,.15f,-.34f,-.03f,-.64f,-.03f,.99f,.14f,-.48f,-.91f,-.29f,.43f,-.40f,.36f,.15f,.73f,.00f,-.39f,-.55f,.15f,.12f,.48f,.47f,.47f,-.25f,-.45f,.68f,.48f,-.92f,-.23f,.36f,.78f,-.20f,-.15f,-.14f,.82f,-.49f,-.91f,-.92f,-.92f,-.25f,-.22f,.77f,.44f,-.86f,.16f,.76f,-.06f,.73f,-.94f,-.99f,-.14f,.52f,-.63f,-.18f,.77f,-.70f,-.92f,.36f,.06f,.57f,-.96f,.10f,.12f,-.64f,.38f,-.75f,-.09f,-.81f,-.17f,-.82f,.39f,.16f,-.49f,-.82f,-.95f,.16f,.20f,.12f,.40f,.26f,.94f,.55f,-.83f,.06f,-.04f,.42f,-.97f,.49f,-.15f,-.18f,-.40f,.43f,-.56f,.33f,.29f,-.94f,-.74f,-.20f,.47f,-.08f,-.95f,.08f,-.32f,.75f,-.67f,.36f,-.29f,-.41f,.85f,-.15f,.09f,.26f,-.40f,-.64f,-.43f,-.45f,.87f,-.62f,-.96f,.26f,.87f,-.36f,-.69f,-.93f,-.32f,.09f,.94f,-.85f,.31f,-.59f,-.08f,.10f,.33f,-.97f,-.54f,-.50f,.25f,.26f,.67f,.30f,-.98f,.67f,-.92f,-.56f,-.80f,-.53f,-.34f,-.68f,-.28f,.59f,-.56f,.94f,-.10f,-.26f,-.93f,.54f,.13f,.23f,-.81f,.36f,-.26f,.90f,.53f,-.43f,.68f,.28f,.31f,.25f,.45f,-.21f,.67f,-.56f,-.63f,.05f,.39f,.81f,-.37f,-.92f,.53f,.40f,-.31f,.23f,-.00f,-.83f,.46f,-.54f,.98f,.09f,-.54f,-.89f,.82f,-.62f,-.09f,.02f,-.24f,.42f,-.85f,.15f,-.59f,-.73f,.73f,-.19f,-.58f,-.45f,.98f,-.12f,-.24f,-.49f,-.49f,.46f,.59f,-.74f,.79f,.22f,-.11f,-.64f,-.68f,.31f,.28f,.23f,.77f,-.36f,.30f,.68f,-.51f,-.52f,.59f,.06f,.29f,.69f,-.11f,-.35f,.67f,-.18f,-.59f,-.77f,-.09f,.30f,.78f,-.36f,.06f,-.08f,-.25f,-.69f,.82f,.64f,-.41f,.40f,-.34f,.01f,-.26f,.40f,-.73f,.84f,.39f,-.42f,-.31f,-.28f,-.96f,-.75f,-.64f,.23f,-.76f,-.42f,.92f,.52f,.92f,-.58f,.69f,-.15f,.24f,-.27f,-.86f,-.09f,-.42f,-.74f,-.01f,-.18f,-.41f,.64f,.92f,-.58f,.66f,-.59f,-.64f,.20f,-.80f,-.58f,-.44f,-.34f,-.11f,-.07f,.18f,.31f,-.05f,.90f,-.10f,-.59f,.33f,.89f,-.78f,.66f,.59f,.76f,.52f,.04f,.37f,.79f,.40f,-.97f,-.41f,.04f,.10f,.72f,-.20f,-.95f,.69f,.26f,-.56f,.88f,-.09f,-.73f,-.61f,-.88f,-.53f,-.61f,-.52f,-.29f,.96f,-.61f,-.63f,.18f,.04f,-.31f,.47f,-.72f,.57f,-.69f,-.30f,.32f,.38f,-.31f,-.05f,.83f,-.17f,.40f,.33f,-.27f,.09f,.85f,-.99f,-.96f,-.57f,-.56f,-.73f,.69f,.42f,-.89f,-.24f,.99f,-.18f,.91f,-.66f,-.63f,-.24f,-.24f,-.63f,.80f,.96f,.94f,.59f,-.76f,.26f,.74f,-.89f,.50f,.16f,.21f,-.48f,.32f,-.33f,.13f,-.61f,-.01f,.63f,-.62f,.45f,-.03f,-.25f,.33f,.09f,-.96f,.00f,-.63f,.41f,.48f,.90f,-.61f,.98f,-.02f,-.45f,-.28f,.14f,-.22f,-.68f,-.46f,.64f,-.95f,-.31f,-.24f,.88f,.34f,-.34f,-.42f,.56f,-.40f,-.27f,-.09f,-.78f,.84f,.25f,.34f,-.74f,.46f,-.95f,.05f,-.72f,-.15f,.12f,.49f,-.74f,.91f,.65f,.89f,-.33f,-.57f,-.49f,-.19f,-.08f,-.36f,.74f,-.48f,.07f,.09f,-.48f,-.16f,.65f,-.29f,-.13f,.04f,.37f,-.66f,.06f,-.38f,.10f,.07f,.92f,-.41f,-.17f,.99f,.36f,-.38f,.21f,.66f,-.25f,-.32f,-.31f,.85f,-.47f,-.38f,-.40f,-.10f,.08f,.27f,-.50f,-.31f,.96f,.51f,-.83f,.63f,.65f,-.70f,.09f,-.48f,.48f,.22f,.72f,-.90f,-.06f,.19f,-.16f,.41f,-.69f,-.16f,-.71f,.78f,-.83f,-.76f,-.20f,-.81f,.64f,-.49f,.08f,-.15f,-.33f,.77f,.51f,-.24f,-.08f,.82f,-.58f,.64f,.49f,-.18f,.43f,-.54f,-.85f,-.24f,-.71f,-.90f,.69f,-.73f,.72f,.47f,-.42f,-.75f,-.51f,.80f,.18f,-.08f,-.41f,.61f,-.31f,-.39f,.97f,-.32f,.51f,-.61f,-.35f,.10f,-.08f,-.28f,-.04f,.94f,.71f,.61f,-.13f,.21f,.96f,-.61f,.85f,-.55f,.96f,-.18f,-.05f,-.85f,-.60f,.92f,-.88f,-.71f,-.40f,.14f,.69f,-.21f,.63f,.86f,.44f,-.25f,-.23f,-.05f,-.29f,.89f,-.53f,-.06f,.03f,.36f,-.83f,.79f,-.40f,-.10f,.90f,-.92f,-.49f,-.38f,-.43f,.02f,.98f,-.99f,.13f,-.75f,-.99f,-.37f,.52f,-.20f,-.56f,-.38f,.80f,-.45f,.13f,.71f,-.60f,-.46f,-.90f,.83f,-.01f,-.47f,.08f,-.52f,-.27f,-.83f,-.06f,-.26f,-.43f,.02f,.71f,.26f,-.39f,-.20f,-.12f,.95f,.15f,.71f,-.82f,-.91f,-.98f,.99f,.60f,-.20f,-.89f,-.33f,-.22f,.25f,.04f,.74f,-.92f,-.19f,-.23f,.84f,.34f,.83f,.41f,.77f,.24f,.32f,-.31f,-.36f,-.08f,.58f,-.45f,.63f,-.28f,.15f,-.19f,.58f,.67f,-.21f,-.79f,.23f,-.33f,-.05f,-.60f,-.09f,-.98f,.73f,-.36f,.18f,.70f,.86f,-.18f,-.36f,.06f,-.30f,-.48f,-.08f,.70f,.57f,.00f,.95f,.69f,-.70f,-.98f,-.30f,-.78f,-.63f,-.91f,.83f,.72f,.45f,-.50f,-.30f,-.41f,-.17f,-.39f,-.65f,-.97f,.25f,-.03f,-.38f,-.24f,.42f,.27f,-.18f,.50f,-.04f,-.33f,-.54f,.69f,-.97f,.45f,.03f,.78f,.83f,-.91f,-.37f,-.55f,-.28f,-.15f,-.19f,.38f,.11f,-.39f,-.63f,.84f,-.19f,.61f,-.36f,.34f,-.54f,-.68f,-.68f,.47f,-.08f,.79f,.21f,-.50f,-.18f,-.40f,.80f,.00f,-.40f,-.90f,-.52f,-.21f,-.10f,.99f,.97f,-.71f,-.98f,-.80f,.20f,.73f,.95f,-.47f,.52f,.98f,.01f,.16f,-.50f,.43f,.51f,.81f,.69f,-.34f,-.49f,-.92f,.68f,.79f,-.56f,-.46f,.69f,.51f,-.45f,-.30f,.95f,-.46f,-.88f,-.92f,-.69f,.91f,-.62f,-.58f,.70f,.51f,-.70f,-.87f,-.01f,-.64f,-.22f,-.81f,-.65f,-.55f,-.06f,.84f,.54f,.29f,-.85f,-.99f,.87f,.98f,.61f,.64f,.25f,.87f,.89f,.48f,-.25f,-.78f,-.77f,.49f,.53f,-.06f,-.93f,.21f,.92f,-.51f,.55f,.83f,.14f,.17f,.42f,-.09f,.52f,.95f,.46f,.81f,-.75f,.88f,.99f,-.53f,-.93f,-.46f,-.67f,.61f,-.02f,-.57f,-.58f,.09f,.29f,.27f,.73f,.90f,-.72f,-.68f,-.00f,.64f,.24f,.18f,.73f,.03f,-.06f,-.84f,-.07f,-.25f,-.61f,.71f,-.10f,-.27f,-.83f,.24f,-.11f,-.97f,-.99f,.13f,-.84f,-.51f,.85f,-.11f,-.67f,.45f,.95f,.76f,-.63f,.95f,-.68f,-.15f,.77f,-.44f,.10f,.11f,.76f,.56f,.48f,.35f,-.11f,-.16f,-.00f,.63f,-.01f,.51f,.13f,-.28f,-.01f,.58f,.89f,-.69f,.43f,-.90f,.28f,.86f,.93f,.22f,-.16f,.47f,.33f,-.56f,.12f,-.26f,-.72f,-.51f,.24f,-.19f,-.88f,-.70f,.45f,-.93f,-.61f,-.76f,-.49f,.27f,-.11f,-.06f,.10f,.04f,.62f,.51f,-.27f,-.19f,.90f,-.39f,.18f,-.10f,.84f,.53f,-.71f,.17f,.29f,.72f,-.80f,.27f,.57f,-.53f,-.95f,.88f,.84f,.09f,.11f,-.13f,-.64f,-.36f,-.48f,.82f,.08f,-.35f,.03f,.76f,.14f,.79f,-.94f,-.35f,.89f,-.25f,-.48f,.37f,.82f,-.30f,.57f,.94f,.47f,.17f,-.39f,-.62f,.65f,.37f,.41f,-.85f,.74f,-.59f,.44f,-.63f,.45f,.90f,.99f,-.73f,-.03f,.00f,.49f,-.29f,-.63f,.59f,.24f,.66f,-.05f,.06f,-.95f,.06f,.62f,-.83f,-.46f,-.25f,-.35f,-.91f,.07f,-.77f,.54f,-.82f,.82f,.68f,.49f,-.63f,-.36f,-.31f,-.48f,-.95f,-.55f,-.62f,.52f,.26f,-.39f,.45f,-.09f,-.07f,.56f,.32f,.37f,.57f,-.83f,-.90f,.03f,.47f,.97f,.26f,.37f,-.24f,.72f,.38f,-.73f,.86f,.88f,-.59f,.17f,-.80f,-.86f,.64f,.83f,-.15f,-.31f,-.87f,.92f,-.24f,.45f,.42f,-.61f,-.37f,.02f,.82f,.84f,.06f,.67f,-.12f,.11f,.94f,-.81f,.75f,-.04f,.72f,-.50f,.30f,.90f,.93f,-.05f,-.10f,-.02f,.51f,.03f,-.67f,.18f,.37f,.65f,-.20f,-.45f,.52f,.46f,.95f,-.78f,.20f,.54f,.92f,-.64f,.51f,.63f,-.58f,-.50f,-.40f,.82f,.15f,.15f,.21f,.01f,.89f,.10f,-.05f,-.92f,-.18f,.30f,.00f,-.12f,.71f,-.58f,.15f,.17f,-.31f,-.36f,.95f,-.21f,-.82f,-.31f,-.78f,-.17f,.47f,-.41f,.90f,.75f,.28f,.57f,.16f,-.73f,.66f,-.27f,.76f,-.74f,.35f,-.21f,.39f,-.70f,-.31f,-.14f,-.85f,.80f,.58f,-.43f,.02f,.01f,.31f,-.89f,-.06f,.04f,.28f,-.00f,.74f,.32f,.66f,-.98f,-.53f,-.40f,-.94f,.35f,.53f,-.57f,.60f,.86f,-.94f,-.67f,.95f,-.35f,-.79f,.56f,.10f,-.28f,-.53f,-.27f,.25f,.83f,.77f,.18f,-.33f,.49f,-.64f,.57f,.80f,.98f,.56f,.03f,.58f,.76f,.87f,.61f,.91f,-.30f,-.60f,-.76f,-.30f,.23f,.06f,.03f,.72f,.11f,-.23f,-.21f,-.20f,-.44f,.78f,.15f,.87f,-.31f,.43f,-.05f,-.16f,.04f,-.79f,.87f,.09f,.32f,-.58f,.65f,-.62f,.23f,.20f,-.29f,.88f,-.58f,-.03f,-.33f,-.48f,.27f,.47f,.81f,.91f,-.41f,.67f,-.94f,.31f,.25f,-.00f,.99f,-.06f,.22f,-.19f,.53f,-.62f,-.63f,.28f,-.49f,.37f,-.69f,.59f,-.30f,.57f,.52f,.55f,-.67f,-.15f,.71f,.11f,-.34f,-.06f,.10f,.70f,.98f,.52f,.16f,.87f,-.62f,-.58f,.57f,.65f,-.35f,-.31f,.96f,.21f,.31f,-.82f,.14f,.57f,.24f,-.26f,.77f,-.09f,.20f,.40f,.12f,-.17f,.65f,.64f,-.63f,-.75f,.50f,.31f,-.06f,-.19f,-.63f,.69f,-.54f,.27f,.95f,-.44f,.18f,-.91f,-.02f,.77f,-.66f,-.33f,-.20f,-.22f,.65f,.31f,-.00f,-.26f,.07f,.05f,-.47f,-.52f,-.06f,.49f,.65f,.54f,.10f,-.10f,-.56f,-.87f,.35f,-.51f,.90f,-.04f,-.90f,-.16f,-.75f,-.06f,-.04f,-.11f,.44f,.73f,.72f,.80f,.73f,-.95f,.83f,.72f,.46f,-.94f,-.56f,.00f,-.41f,-.66f,.56f,.92f,-.23f,-.32f,-.16f,-.04f,-.37f,-.81f,-.29f,-.79f,-.45f,-.37f,.65f,-.07f,-.97f,-.93f,.15f,.01f,-.90f,.05f,-.63f,.43f,-.67f,.04f,.86f,-.64f,.80f,.66f,-.02f,.84f,-.46f,.19f,-.42f,-.20f,.86f,.12f,.32f,-.72f,-.40f,-.32f,-.26f,.85f,-.40f,.15f,-.69f,-.77f,-.48f,-.10f,-.16f,.61f,-.12f,-.76f,.21f,-.15f,-.15f,.00f,-.31f,.74f,-.12f,.75f,-.31f,.89f,.75f,-.17f,-.58f,-.33f,-.72f,-.43f,-.36f,-.74f,-.17f,-.61f,-.53f,.62f,-.24f,-.37f,-.16f,-.74f,.39f,-.41f,.57f,-.64f,-.51f,.65f,.40f,.65f,-.04f,-.08f,.30f,-.86f,.37f,-.49f,.24f,.36f,-.66f,.53f,-.42f,.47f,-.10f,-.41f,-.92f,-.88f,.90f,.59f,.00f,.70f,-.55f,.45f,.81f,.42f,-.81f,-.90f,-.66f,.47f,-.49f,-.21f,-.71f,-.62f,.20f,-.90f,-.49f,-.83f,-.94f,.16f,.04f,.84f,-.49f,-.23f,-.75f,-.13f,.91f,.36f,-.51f,.56f,-.26f,-.34f,-.27f,.13f,.45f,-.89f,.69f,-.42f,.49f,-.93f,.46f,.68f,.63f,-.86f,-.26f,.14f,-.29f,.80f,-.74f,.73f,.19f,-.26f,.01f,-.71f,-.54f,-.64f,-.59f,-.37f,.76f,.15f,.90f,-.28f,.59f,.16f,-.55f,.48f,-.49f,-.43f,.56f,-.30f,.12f,.08f,-.28f,.78f,.73f,-.06f,.13f,.31f,-.30f,.90f,.39f,-.77f,-.89f,.91f,-.09f,.76f,.30f,-.80f,.31f,-.94f,.88f,-.38f,-.10f,.02f,-.93f,-.27f,.83f,-.33f,-.55f,.04f,-.31f,-.21f,-.38f,-.04f,-.21f,-.58f,.57f,-.50f,.45f,-.88f,-.95f,.66f,-.15f,-.65f,.04f,-.91f,-.02f,-.40f,.60f,.17f,.07f,-.11f,-.55f,.36f,.47f,.09f,.37f,.75f,-.98f,-.39f,.79f,.61f,-.75f,.51f,-.49f,-.99f,-.72f,-.56f,.97f,-.42f,.06f,-.14f,-.96f,-.28f,-.48f,.99f,.64f,-.12f,.63f,-.95f,-.09f,-.81f,.83f,.99f,-.01f,.17f,.03f,-.52f,.20f,.77f,-.42f,.96f,.79f,-.64f,-.29f,-.84f,-.79f,.83f,-.88f,-.24f,.22f,.55f,.16f,-.85f,.49f,.61f,-.79f,-.75f,.27f,.36f,.36f,-.98f,-.34f,.13f,.38f,-.25f,.74f,.08f,.46f,.13f,.32f,.78f,.44f,.11f,-.57f,-.34f,-.92f,-.03f,-.74f,-.06f,.30f,-.51f,.06f,.74f,.13f,.34f,.05f,.81f,.43f,.80f,-.77f,-.54f,.59f,-.05f,.14f,.29f,-.70f,-.12f,.03f,-.04f,-.07f,-.84f,.86f,.87f,-.24f,.02f,.71f,.31f,-.62f,.62f,-.65f,.26f,.47f,.57f,.84f,.32f,.54f,-.50f,.93f,-.91f,.35f,-.74f,-.56f,-.46f,-.78f,-.60f,-.89f,-.88f,-.16f,-.83f,-.63f,.59f,.85f,.32f,.74f,.84f,-.83f,-.46f,-.77f,-.74f,.81f,-.95f,.99f,-.87f,-.28f,.66f,-.58f,.20f,-.17f,-.95f,.23f,.86f,-.81f,.93f,.38f,.49f,.68f,-.91f,-.58f,-.41f,.31f,.23f,.92f,.71f,.33f,.18f,-.33f,.64f,.75f,-.72f,.70f,-.49f,-.49f,-.18f,-.59f,-.23f,.71f,.42f,-.53f,-.83f,-.04f,-.75f,-.77f,-.59f,.53f,-.30f,.22f,-.78f,.64f,-.64f,-.37f,.79f,.72f,-.06f,.94f,.93f,-.75f,.90f,-.79f,.96f,.28f,-.74f,.70f,-.81f,.18f,-.96f,.84f,-.18f,.05f,-.23f,.26f,-.85f,-.12f,.50f,.12f,-.94f,.64f,.86f,.81f,.89f,-.62f,.74f,-.19f,-.78f,.67f,-.04f,.36f,.62f,-.02f,-.49f,.72f,-.02f,.42f,-.78f,.79f,.26f,-.63f,-.31f,-.38f,-.37f,-.43f,-.97f,.27f,-.34f,-.59f,-.07f,.44f,-.62f,-.64f,-.09f,-.70f,-.11f,-.89f,-.35f,.49f,-.73f,-.89f,.76f,.90f,.04f,.44f,.58f,.18f,-.06f,.68f,.14f,-.48f,-.15f,-.53f,-.31f,.37f,.71f,-.10f,-.83f,-.88f,-.26f,-.88f,.82f,-.04f,.84f,-.92f,-.10f,.00f,.18f,.21f,.74f,-.25f,.88f,-.75f,-.47f,.72f,-.54f,.04f,.10f,.29f,.20f,.08f,-.17f,.71f,-.46f,.96f,-.21f,.10f,.58f,.65f,.06f,.58f,.68f,.27f,-.07f,-.05f,.97f,.25f,-.42f,-.59f,.42f,.87f,-.06f,-.61f,.06f,.84f,.58f,-.68f,.65f,.40f,-.32f,-.38f,.93f,.58f,-.65f,.73f,.96f,-.97f,.08f,.80f,.39f,-.21f,-.47f,.56f,-.17f,-.46f,-.52f,-.27f,.76f,.60f,-.41f,-.73f,.93f,-.06f,.30f,-.64f,-.02f,-.23f,.08f,.72f,-.58f,-.29f,-.49f,.85f,-.42f,.40f,-.37f,-.74f,.51f,-.52f,-.52f,-.63f,-.66f,.81f,-.39f,.33f,-.46f,-.76f,-.06f,.45f,-.05f,-.64f,.42f,.76f,-.55f,-.98f,.78f,-.82f,-.25f,.24f,.35f,.68f,-.68f,.28f,.60f,-.50f,.20f,.15f,-.92f,.09f,-.99f,.49f,.24f,.69f,-.50f,.47f,.94f,-.30f,.96f,-.45f,-.57f,.25f,.77f,.82f,-.88f,-.33f,-.32f,-.65f,-.85f,.83f,.06f,.71f,.53f,-.42f,.25f,-.50f,-.02f,.76f,-.13f,.39f,-.99f,-.20f,.20f,-.80f,-.59f,-.59f,.13f,.57f,.55f,.44f,-.11f,.30f,-.16f,.78f,-.91f,.02f,.68f,.59f,-.28f,-.79f,.48f,-.17f,-.86f,-.68f,.96f,-.61f,-.69f,.95f,.81f,-.89f,.02f,.97f,-.13f,-.10f,.07f,.67f,.30f,-.26f,-.07f,-.55f,-.58f,-.81f,.06f,.65f,.97f,-.29f,.85f,-.49f,-.01f,-.90f,.23f,-.39f,-.71f,.28f,.38f,-.26f,.19f,-.94f,.38f,-.80f,.15f,-.59f,.18f,.87f,-.54f,-.65f,.47f,.74f,.43f,.30f,.99f,-.66f,.89f,-.72f,.25f,.73f,-.26f,-.60f,.10f,.55f,-.86f,-.96f,-.54f,-.81f,.84f,.67f,.34f,.98f,-.50f,-.64f,-.40f,-.61f,-.72f,-.68f,.80f,-.15f,-.99f,-.79f,.71f,-.66f,-.11f,.06f,.24f,-.58f,.60f,-.21f,.24f,-.75f,-.12f,-.91f,-.67f,-.19f,-.47f,-.87f,.40f,.42f,.69f,-.61f,-.98f,-.63f,.97f,.05f,-.19f,-.47f,.41f,-.45f,.94f,.85f,-.10f,.09f,.00f,.20f,.49f,-.81f,.31f,.94f,-.41f,-.71f,.99f,.82f,-.34f,-.08f,.85f,.61f,-.93f,-.54f,.37f,-.90f,-.67f,-.02f,-.43f,-.52f,-.58f,-.61f,.81f,.53f,.86f,-.09f,.26f,.23f,-.76f,-.58f,-.90f,.33f,.82f,-.44f,.18f,-.18f,.72f,.74f,-.84f,.83f,-.26f,-.32f,-.99f,-.07f,.57f,.71f,-.64f,.56f,-.08f,-.78f,.83f,-.60f,-.54f,-.65f,.26f,.67f,.10f,.72f,-.12f,.71f,.90f,-.37f,-.55f,-.42f,-.89f,.87f,.70f,.47f,.07f,.46f,.48f,.53f,-.95f,.82f,.35f,.48f,.99f,-.64f,.45f,-.33f,.94f,.93f,.55f,.95f,.23f,.71f,.42f,.48f,-.45f,.54f,.38f,.58f,.94f,.82f,.65f,-.51f,.53f,-.47f,-.78f,-.51f,.67f,.72f,-.20f,.09f,.41f,.69f,.21f,.95f,-.26f,.23f,-.04f,-.30f,-.18f,-.78f,.06f,.04f,.83f,-.97f,-.55f,-.86f,-.99f,-.12f,.17f,-.50f,-.77f,-.20f,-.47f,-.95f,.05f,-.37f,.88f,.09f,-.54f,-.64f,-.72f,.57f,.16f,-.31f,-.28f,-.36f,.71f,-.82f,.23f,.88f,.69f,.99f,-.99f,.94f,.24f,-.45f,.61f,.85f,-.92f,.96f,.98f,.34f,.03f,-.25f,.15f,-.71f,.48f,-.02f,-.29f,.34f,.48f,-.14f,-.46f,-.32f,.67f,.04f,-.90f,-.82f,.44f,-.46f,-.33f,.63f,-.46f,.16f,.02f,.57f,-.02f,.50f,.26f,.51f,-.15f,.25f,.07f,-.59f,-.87f,.64f,-.92f,.05f,.79f,.14f,-.34f,-.77f,.72f,.53f,-.42f,.95f,-.86f,-.35f,.90f,.36f,.25f,.63f,.59f,-.32f,-.95f,-.98f,-.84f,-.56f,.65f,-.81f,-.06f,-.45f,-.60f,-.84f,.09f,.77f,.17f,.11f,-.57f,-.67f,-.23f,-.98f,.12f,.32f,.35f,.37f,-.81f,-.07f,-.21f,.72f,-.35f,-.02f,.33f,.58f,-.52f,.17f,-.49f,.30f,-.30f,-.78f,-.38f,.25f,.14f,-.20f,.75f,-.51f,.46f,.15f,-.07f,.19f,-.47f,.70f,-.47f,-.03f,.14f,-.25f,.67f,.83f,.22f,-.80f,.15f,.88f,.42f,.76f,-.53f,.85f,.89f,.92f,-.73f,-.88f,.05f,.09f,.19f,.55f,.74f,-.08f,.41f,.06f,-.35f,.66f,.05f,.36f,.28f,.77f,.65f,.94f,-.28f,-.31f,.47f,-.62f,-.99f,.90f,.70f,.88f,-.68f,.69f,-.23f,-.01f,.97f,.87f,.55f,.07f,.59f,-.16f,-.05f,.28f,-.48f,.56f,-.64f,.22f,.55f,-.66f,.29f,.12f,.73f,-.98f,-.13f,-.39f,.33f,.95f,.38f,-.34f,.73f,.25f,.63f,.13f,-.18f,-.23f,-.68f,.39f,.65f,-.08f,-.96f,.35f,-.06f,.81f,-.34f,-.91f,.96f,-.75f,.61f,.81f,-.21f,.21f,-.89f,-.13f,-.92f,-.11f,-.25f,.28f,.31f,.83f,.96f,-.13f,.54f,.95f,.16f,.49f,.12f,.13f,.20f,.37f,.34f,.87f,-.11f,-.28f,-.05f,.72f,-.27f,.44f,.84f,-.03f,.69f,-.79f,.63f,.81f,.88f,.98f,.51f,-.20f,.54f,.73f,-.95f,-.61f,.33f,.26f,.51f,.46f,.43f,-.04f,-.29f,.77f,.34f,-.43f,.48f,.42f,-.14f,.89f,-.93f,-.40f,.77f,-.04f,.85f,.98f,-.26f,.05f,-.24f,-.22f,-.50f,-.82f,-.42f,.63f,.79f,-.84f,-.60f,.20f,-.41f,.86f,.51f,-.17f,-.20f,.94f,-.89f,-.51f,-.35f,.73f,-.53f,.09f,.49f,-.72f,-.47f,-.72f,-.58f,-.28f,.05f,-.08f,-.86f,-.69f,-.22f,-.32f,.17f,-.58f,-.49f,-.62f,.19f,-.28f,-.40f,-.61f,.75f,-.87f,-.96f,-.47f,-.04f,.64f,.80f,.90f,-.57f,.40f,.34f,-.31f,-.94f,.79f,.12f,.55f,.62f,.44f,.58f,-.10f,-.07f,.35f,.73f,.42f,-.05f,-.16f,.89f,-.05f,-.22f,-.25f,-.81f,-.57f,.78f,.49f,.06f,.87f,.61f,-.01f,.77f,.63f,-.15f,-.17f,-.42f,-.59f,.64f,.85f,-.70f,.03f,-.58f,.25f,.31f,.72f,.26f,-.83f,-.37f,-.10f,-.30f,-.75f,-.54f,.13f,.10f,.75f,-.66f,-.73f,-.41f,.15f,.51f,-.72f,.76f,.98f,-.41f,.61f,-.67f,-.82f,-.90f,.57f,.57f,.82f,.50f,.74f,-.06f,-.19f,-.49f,.71f,.82f,.19f,-.04f,.55f,.03f,.59f,-.65f,.99f,.74f,.81f,-.59f,-.60f,.73f,.47f,.73f,.72f,-.33f,.29f,.66f,.72f,-.21f,-.90f,.69f,.40f,.67f,.07f,-.39f,.34f,.94f,.56f,.61f,.24f,-.76f,.53f,-.15f,-.70f,.93f,.42f,-.12f,-.50f,.67f,-.55f,.73f,.46f,-.74f,.90f,-.68f,.57f,-.62f,-.19f,-.57f,.37f,.81f,-.84f,.73f,.20f,-.89f,-.55f,.01f,-.12f,.24f,-.14f,-.35f,.99f,.40f,.11f,-.83f,-.25f,.84f,.89f,-.51f,.75f,.44f,.45f,-.18f,-.70f,.83f,.02f,-.84f,.46f,-.83f,-.02f,.10f,.44f,.53f,-.96f,.70f,.55f,.37f,.84f,-.64f,-.06f,.94f,-.58f,-.83f,.92f,.11f,.34f,-.67f,-.42f,-.27f,-.55f,-.97f,-.92f,-.81f,-.21f,-.40f,.73f,.49f,.60f,-.56f,.95f,-.09f,-.20f,-.88f,-.09f,-.70f,-.30f,-.85f,-.39f,.94f,.06f,.06f,-.09f,.05f,.61f,.51f,.60f,-.78f,-.89f,.92f,-.03f,-.03f,.41f,-.10f,.97f,.67f,.92f,.70f,-.22f,.63f,.77f,-.46f,.25f,.17f,.00f,-.14f,-.87f,.49f,.99f,.42f,-.67f,.61f,.34f,.25f,-.71f,-.18f,.24f,-.27f,.24f,-.06f,-.05f,-.08f,.70f,-.19f,-.29f,.41f,.78f,.08f,-.88f,.39f,.33f,-.50f,.00f,-.35f,.56f,-.85f,.06f,-.02f,-.91f,-.58f,.18f,.66f,.46f,.96f,.90f,.74f,-.17f,-.80f,-.38f,-.01f,.25f,-.53f,.21f,.26f,-.63f,-.78f,.32f,.42f,-.60f,-.31f,-.93f,.36f,.41f,.47f,-.41f,.03f,.37f,-.93f,-.75f,-.10f,-.44f,.74f,.96f,.47f,.69f,.44f,.07f,.28f,-.84f,-.00f,-.65f,.23f,-.17f,.65f,-.05f,-.32f,-.98f,.67f,-.45f,.16f,.26f,.81f,.81f,-.72f,-.23f,.57f,-.87f,.76f,-.57f,.14f,-.44f,.31f,.80f,-.73f,.83f,.00f,-.54f,.10f,.42f,.54f,-.18f,.56f,.35f,.44f,-.52f,.69f,.69f,.94f,.39f,.96f,.58f,-.71f,.98f,-.92f,-.11f,.98f,.90f,.79f,.63f,.84f,-.95f,.61f,-.52f,-.10f,.33f,.10f,.56f,.74f,.11f,-.27f,-.36f,.88f,.33f,-.30f,.67f,.98f,.19f,.56f,.85f,.13f,-.65f,-.67f,.86f,-.03f,.83f,-.50f,-.54f,-.65f,-.64f,-.73f,-.60f,.68f,-.71f,-.36f,-.81f,-.27f,-.06f,-.04f,.47f,-.99f,-.35f,-.89f,.55f,-.39f,-.94f,.32f,-.23f,-.53f,.37f,-.17f,.85f,.27f,-.10f,-.81f,-.31f,-.22f,-.30f,-.29f,.93f,.08f,.98f,.32f,.97f,.86f,.67f,.39f,.50f,.06f,.25f,-.73f,-.88f,-.02f,.88f,.85f,.38f,-.29f,-.99f,.30f,-.06f,.14f,-.02f,.09f,-.62f,.41f,-.46f,.90f,.06f,-.78f,-.10f,.97f,.66f,-.96f,.46f,.12f,.84f,.89f,-.28f,-.65f,-.71f,.04f,.34f,-.98f,-.27f,.21f,.13f,-.56f,.56f,.71f,-.55f,.14f,.81f,.03f,-.19f,.82f,.48f,-.42f,.73f,.04f,.48f,-.10f,-.78f,.75f,-.89f,-.02f,.60f,.15f,-.88f,.01f,.93f,-.59f,.43f,.57f,.14f,.96f,.75f,.58f,.69f,-.92f,.14f,.94f,-.42f,.54f,-.73f,.91f,-.89f,-.95f,.41f,.26f,-.46f,-.64f,-.65f,.53f,-.61f,-.95f,-.41f,-.28f,-.78f,-.01f,-.55f,.69f,.60f,-.29f,.81f,.74f,.39f,.58f,-.65f,.37f,-.88f,-.11f,.08f,-.78f,.01f,-.46f,-.12f,.38f,-.63f,.66f,-.33f,-.51f,-.16f,.32f,.40f,-.03f,-.35f,.49f,-.52f,-.26f,-.99f,-.94f,.18f,.94f,.37f,.94f,.85f,.36f,-.00f,.36f,-.94f,-.47f,.92f,.75f,-.29f,-.82f,.95f,.29f,-.47f,.12f,-.96f,.06f,-.56f,-.20f,.47f,.15f,.08f,-.20f,-.20f,-.31f,.44f,-.83f,-.03f,.75f,-.87f,.90f,-.39f,-.06f,.56f,.03f,-.03f,-.37f,-.08f,-.25f,.15f,-.90f,.29f,-.94f,.20f,.71f,.14f,-.70f,.98f,-.77f,-.97f,-.43f,.25f,-.30f,.28f,-.04f,.19f,.40f,.56f,-.18f,-.66f,.66f,-.21f,-.48f,.76f,.87f,-.63f,-.12f,.22f,-.48f,-.41f,-.87f,-.16f,-.53f,.25f,-.31f,.45f,-.25f,.46f,.35f,.64f,-.45f,.84f,-.70f,.19f,-.96f,-.68f,.92f,-.07f,.10f,.89f,.25f,.70f,.52f,.63f,-.36f,.11f,-.39f,.77f,.59f,-.01f,-.60f,.66f,-.46f,.64f,-.21f,.37f,-.55f,.27f,.09f,-.35f,.07f,.80f,-.51f,-.80f,-.04f,.59f,.59f,-.96f,-.12f,-.28f,.71f,.17f,-.36f,.35f,.99f,-.30f,-.98f,.52f,-.62f,.69f,-.48f,.34f,.30f,-.78f,-.84f,.46f,-.36f,.31f,-.61f,-.71f,.46f,-.97f,-.97f,-.18f,-.42f,-.22f,-.42f,-.08f,.84f,-.48f,-.22f,-.95f,-.42f,-.45f,.17f,-.58f,.82f,-.81f,.23f,.07f,-.16f,.93f,-.82f,-.78f,.13f,-.80f,-.24f,-.00f,.75f,-.59f,.01f,.71f,.53f,.02f,-.97f,.81f,-.35f,.15f,-.32f,-.11f,.79f,-.08f,.82f,-.99f,-.68f,-.48f,-.08f,-.03f,.64f,.48f,-.69f,-.82f,.98f,.86f,-.32f,-.86f,-.08f,.25f,-.37f,-.56f,.04f,-.53f,.52f,.89f,-.26f,.79f,.97f,.46f,.07f,-.99f,-.91f,-.03f,.81f,.06f,-.56f,-.68f,.14f,-.15f,.33f,-.26f,.94f,.76f,.70f,.70f,-.88f,.20f,.02f,-.32f,.75f,.34f,.47f,.63f,.01f,-.80f,.31f,.01f,-.25f,.20f,.40f,-.95f,.52f,.24f,-.46f,.61f,.58f,-.97f,-.90f,-.24f,.18f,.16f,.56f,.21f,-.71f,-.96f,-.86f,.04f,-.11f,-.09f,.07f,-.01f,-.58f,-.12f,-.43f,-.95f,.45f,-.87f,-.07f,.31f,.02f,-.22f,-.62f,.48f,.19f,.42f,-.23f,.70f,-.56f,-.19f,-.54f,-.15f,.78f,-.39f,-.95f,.36f,.12f,-.31f,.93f,.16f,.92f,.58f,-.80f,.76f,-.16f,.15f,.93f,-.18f,.11f,.97f,.36f,-.29f,.56f,.67f,-.46f,-.94f,.64f,-.87f,-.72f,-.86f,.64f,.34f,-.00f,.03f,-.48f,-.11f,.09f,-.89f,-.85f,.71f,-.36f,-.28f,.04f,-.88f,.91f,.19f,.37f,-.58f,.29f,.17f,.33f,-.62f,-.32f,.83f,.00f,-.95f,-.03f,-.80f,.64f,.10f,-.11f,.46f,.54f,.64f,-.17f,-.31f,-.46f,.85f,-.42f,-.27f,.45f,.06f,-.11f,-.20f,-.30f,.29f,.73f,.43f,-.95f,.46f,-.04f,-.51f,.30f,.82f,-.93f,-.86f,.33f,-.39f,-.07f,-.67f,-.86f,-.61f,.48f,-.35f,-.57f,.31f,.05f,-.72f,.46f,-.62f,-.81f,-.74f,.59f,.79f,.77f,-.31f,-.38f,-.85f,.70f,.39f,.57f,.52f,-.08f,.49f,-.49f,-.53f,.61f,-.21f,-.04f,.03f,.54f,-.49f,.95f,-.45f,-.99f,-.39f,.19f,-.74f,-.43f,.97f,-.46f,-.93f,-.56f,.17f,-.74f,-.87f,-.20f,-.87f,-.60f,-.66f,.12f,-.88f,-.25f,-.27f,.36f,-.22f,.97f,-.24f,.35f,-.30f,-.10f,.66f,-.06f,-.21f,-.20f,.87f,-.13f,-.25f,.10f,-.47f,-.51f,.55f,.20f,.14f,.80f,-.25f,.81f,-.19f,-.04f,-.05f,.63f,-.03f,-.60f,.39f,-.25f,.47f,-.51f,-.42f,-.31f,-.08f,.76f,-.64f,.28f,-.15f,.89f,-.39f,.49f,-.31f,.39f,-.48f,-.24f,-.97f,.76f,-.34f,-.35f,-.83f,-.70f,-.92f,.37f,-.05f,.95f,.49f,-.04f,.51f,-.84f,.56f,.50f,.85f,.78f,-.03f,-.32f,.31f,-.13f,-.38f,-.34f,-.15f,.22f,-.91f,.97f,.12f,.71f,.22f,.98f,.45f,-.21f,-.27f,.31f,-.47f,-.90f,-.90f,-.83f,.07f,.20f,.68f,-.42f,-.36f,-.14f,.29f,.57f,.76f,-.18f,-.89f,-.30f,.92f,-.09f,-.05f,.96f,.29f,.07f,-.65f,-.78f,.55f,.17f,-.39f,.17f,.62f,.51f,.00f,-.61f,-.52f,-.57f,-.99f,-.65f,.94f,-.54f,-.00f,.99f,.40f,-.98f,.38f,-.15f,-.91f,-.47f,.61f,-.67f,.79f,-.01f,-.04f,.27f,.27f,.49f,.71f,.56f,-.98f,.42f,-.30f,-.48f,.85f,-.28f,.02f,-.26f,.27f,.40f,-.61f,-.48f,.79f,.08f,-.04f,.96f,.86f,.99f,-.97f,.79f,-.96f,.53f,.61f,-.16f,.39f,.02f,.76f,-.51f,.19f,.79f,.20f,.76f,.26f,-.95f,.03f,.06f,.86f,-.69f,.46f,-.55f,-.71f,-.91f,.69f,.53f,-.27f,-.02f,-.34f,-.78f,.51f,.04f,.92f,-.72f,-.84f,.01f,-.95f,.58f,-.85f,.86f,-.70f,.34f,.14f,-.22f,.89f,.44f,.91f,.51f,.15f,.56f,.75f,-.10f,.34f,.14f,-.34f,-.55f,.77f,.59f,.11f,-.16f,-.12f,-.89f,-.45f,-.56f,-.93f,.51f,-.19f,.82f,.17f,.65f,.18f,.69f,-.72f,.75f,.40f,-.37f,-.55f,.87f,.58f,.33f,-.78f,.29f,-.17f,.62f,-.05f,.39f,.69f,.90f,-.53f,.98f,.74f,-.34f,-.34f,-.25f,.62f,.96f,-.30f,-.54f,.46f,-.99f,.13f,-.40f,.15f,-.59f,-.85f,.18f,.89f,-.85f,.56f,-.23f,.73f,.17f,-.76f,-.23f,.77f,.82f,-.72f,-.94f,-.95f,.28f,.74f,-.97f,.44f,-.02f,.40f,-.53f,-.83f,-.57f,-.07f,.20f,.27f,.10f,-.19f,.99f,.18f,-.24f,.73f,-.30f,-.83f,-.34f,-.16f,-.23f,.18f,-.69f,-.70f,.52f,-.33f,.45f,-.81f,-.17f,.03f,.49f,.80f,.88f,-.30f,.17f,-.69f,.19f,.67f,-.78f,-.84f,-.75f,-.81f,.65f,.50f,.62f,.19f,-.91f,.26f,-.17f,-.33f,.19f,.13f,.32f,.59f,-.31f,.88f,-.33f,-.42f,-.34f,-.05f,-.35f,.20f,.31f,-.21f,-.30f,.70f,.10f,-.28f,-.27f,.48f,.45f,-.67f,-.14f,-.41f,-.80f,.02f,.03f,-.49f,-.73f,-.94f,-.28f,-.11f,.94f,-.02f,.06f,-.64f,-.87f,.59f,-.99f,-.19f,.50f,-.12f,.84f,.98f,-.69f,.64f,.93f,-.93f,-.42f,-.96f,-.26f,.21f,.48f,-.64f,.58f,-.86f,-.85f,-.73f,-.81f,.08f,-.89f,.00f,-.16f,.78f,.25f,-.87f,-.30f,-.19f,.58f,.30f,-.10f,.54f,.06f,-.52f,-.88f,.63f,-.34f,-.19f,-.92f,.60f,-.90f,-.21f,-.90f,-.28f,-.14f,-.71f,.49f,.80f,-.49f,-.38f,.57f,.10f,-.80f,.25f,-.38f,-.12f,-.52f,-.01f,-.72f,.74f,-.35f,.88f,.02f,-.14f,-.40f,-.42f,.12f,.93f,-.98f,.95f,-.34f,.53f,.00f,.22f,.35f,.90f,-.39f,-.04f,-.27f,-.20f,.90f,-.00f,.36f,-.68f,-.95f,.21f,.76f,.29f,.99f,-.28f,.02f,-.79f,-.39f,.63f,-.25f,.64f,.72f,-.93f,-.64f,.34f,.36f,.76f,.77f,.27f,.63f,.28f,.94f,.80f,.45f,.49f,.09f,.22f,.50f,.44f,.94f,-.85f,.32f,.34f,.02f,-.04f,-.50f,-.54f,-.96f,.77f,-.09f,-.20f,-.24f,-.78f,.36f,-.73f,.87f,.23f,-.99f,.72f,-.56f,.79f,.14f,-.48f,.37f,.69f,.91f,-.58f,.37f,-.86f,-.76f,-.15f,-.60f,-.16f,.13f,-.97f,-.08f,.84f,-.34f,-.46f,.23f,.61f,-.71f,.71f,-.07f,-.72f,.17f,-.25f,.58f,.10f,.90f,-.80f,.26f,.21f,.48f,-.24f,-.28f,.27f,-.80f,.52f,-.76f,.87f,-.71f,.20f,-.14f,.17f,.87f,-.37f,-.87f,.62f,.50f,.89f,-.66f,.43f,-.85f,.70f,.73f,-.69f,-.95f,-.85f,-.43f,.94f,-.15f,-.69f,.23f,-.29f,.76f,-.62f,.05f,-.59f,.59f,.47f,.27f,-.13f,.47f,-.18f,-.20f,.02f,-.43f,.60f,.11f,-.26f,-.41f,.82f,.66f,-.31f,-.32f,.58f,.86f,.49f,-.90f,.40f,.35f,-.16f,.48f,-.46f,-.02f,-.07f,.32f,-.98f,-.33f,.33f,.22f,-.26f,.09f,-.79f,.06f,.54f,-.87f,.02f,.83f,-.33f,.10f,.50f,-.44f,-.45f,-.97f,-.51f,-.90f,-.56f,.79f,-.48f,.41f,.72f,-.23f,.91f,-.07f,-.38f,.77f,-.55f,-.81f,-.55f,.75f,.66f,-.47f,.44f,-.90f,-.06f,.68f,-.31f,-.25f,-.00f,.47f,-.12f,.12f,.99f,-.62f,-.84f,.17f,.92f,.38f,-.19f,-.97f,.48f,-.89f,.30f,-.08f,.03f,.42f,.14f,.36f,-.26f,-.09f,.41f,.76f,.66f,-.11f,-.19f,.22f,-.11f,.39f,.68f,-.81f,.87f,.04f,.54f,.02f,.05f,-.24f,.05f,.57f,-.81f,-.77f,-.09f,-.35f,-.00f,-.61f,.03f,-.48f,.40f,-.22f,-.19f,-.55f,-.53f,-.32f,-.17f,-.08f,.57f,.18f,.41f,.54f,.21f,-.25f,.07f,.76f,-.97f,-.79f,-.03f,-.56f,-.50f,-.55f,-.58f,.59f,.82f,-.18f,-.50f,-.32f,-.83f,-.44f,-.94f,-.70f,.90f,-.23f,-.23f,-.83f,.35f,-.45f,.31f,.52f,-.14f,.79f,-.30f,-.51f,.99f,.47f,.00f,-.42f,.43f,-.52f,.68f,-.11f,.54f,.98f,.66f,-.32f,.13f,-.87f,-.20f,.27f,-.74f,.64f,-.08f,-.68f,-.78f,.45f,-.37f,.23f,.79f,-.03f,-.09f,.27f,-.46f,-.22f,-.39f,.92f,.80f,-.83f,.89f,.42f,.75f,-.59f,-.26f,-.62f,-.37f,-.07f,.97f,-.90f,.39f,-.50f,-.20f,-.54f,.34f,-.43f,-.82f,-.35f,.42f,-.46f,.48f,-.69f,.33f,.19f,.99f,.06f,-.30f,-.99f,.13f,.47f,-.23f,-.47f,.07f,.82f,-.58f,.42f,-.83f,-.19f,-.98f,-.80f,.28f,.15f,.44f,-.87f,.99f,-.46f,-.85f,-.07f,-.46f,-.93f,.87f,.49f,-.09f,-.74f,-.43f,-.94f,.44f,.50f,-.84f,-.51f,.50f,-.33f,.58f,.22f,-.89f,-.65f,.39f,.01f,.94f,-.73f,-.30f,.69f,.47f,.11f,-.30f,-.16f,-.58f,.76f,-.78f,-.84f,.83f,.96f,-.44f,-.40f,-.28f,.76f,-.47f,-.00f,.91f,.12f,.58f,.64f,.61f,.30f,-.55f,-.33f,.40f,.17f,.35f,.93f,-.63f,.33f,.21f,-.03f,-.55f,.37f,.55f,-.14f,-.37f,-.32f,-.41f,-.50f,-.30f,.49f,-.16f,-.13f,-.77f,-.82f,-.16f,.67f,-.83f,-.87f,.17f,.39f,-.72f,-.09f,.28f,.43f,.44f,-.28f,.23f,.33f,-.95f,.94f,-.70f,.03f,-.73f,-.06f,.90f,-.13f,.39f,-.57f,.13f,-.00f,.30f,-.83f,.53f,-.61f,.27f,.84f,.98f,.98f,-.34f,-.83f,-.36f,.16f,-.93f,-.45f,.23f,.22f,.18f,.80f,-.56f,-.34f,-.40f,-.84f,-.63f,.98f,-.07f,-.36f,.52f,.06f,-.61f,-.19f,.10f,.60f,.37f,-.74f,.84f,-.86f,.63f,.75f,-.48f,.93f,-.39f,-.01f,.43f,.24f,-.49f,-.36f,-.97f,-.56f,-.08f,.61f,.38f,-.25f,.81f,.42f,.42f,.95f,.18f,-.14f,.12f,-.32f,-.62f,.84f,-.92f,-.57f,.57f,-.18f,-.70f,.39f,.48f,.37f,-.34f,-.65f,-.42f,-.81f,.50f,-.27f,-.63f,.09f,-.97f,.76f,-.86f,-.93f,-.67f,-.86f,-.69f,.15f,-.21f,.20f,-.52f,-.65f,.80f,-.69f,-.78f,.23f,-.69f,-.84f,.09f,.53f,-.24f,.44f,.16f,-.10f,.46f,-.63f,-.50f,-.08f,.33f,.76f,-.37f,-.16f,.81f,.44f,.02f,.33f,-.73f,.72f,-.08f,-.47f,.59f,.76f,-.83f,.09f,-.90f,-.91f,-.53f,.80f,-.51f,.33f,-.78f,.47f,-.71f,-.58f,.22f,.53f,-.10f,-.00f,.10f,-.71f,.94f,.18f,-.67f,-.10f,-.95f,.52f,.87f,-.45f,.68f,-.00f,-.78f,-.01f,-.04f,.39f,.02f,.77f,.82f,-.98f,-.90f,-.63f,-.31f,-.97f,.29f,-.00f,.55f,-.58f,.97f,-.35f,.16f,-.56f,.06f,.97f,.54f,.80f,-.59f,.87f,.74f,-.31f,-.24f,.39f,.43f,.33f,-.12f,-.30f,.61f,.77f,.23f,-.05f,-.13f,.14f,.76f,-.12f,.53f,-.56f,-.75f,-.84f,-.97f,.33f,.65f,.16f,-.46f,.28f,-.39f,.40f,.03f,-.36f,.97f,-.27f,-.31f,-.34f,.68f,-.15f,-.09f,.84f,.29f,.90f,-.42f,-.82f,.89f,.15f,.20f,-.32f,-.03f,.29f,.61f,-.52f,-.62f,-.93f,.79f,.28f,-.03f,.03f,-.54f,-.61f,.31f,-.91f,.92f,-.18f,-.12f,.79f,-.24f,-.40f,.46f,-.63f,-.93f,.82f,.40f,-.09f,.12f,-.77f,-.13f,-.58f,.86f,-.86f,.50f,-.72f,-.46f,-.81f,.94f,-.66f,.24f,-.78f,-.79f,-.48f,.49f,.47f,-.04f,-.83f,-.49f,-.48f,-.16f,-.18f,.03f,-.90f,-.47f,-.26f,.38f,.90f,-.81f,-.64f,-.29f,.69f,.62f,-.99f,.50f,.94f,.68f,-.00f,.51f,.26f,-.78f,-.71f,.63f,-.81f,-.18f,-.30f,.28f,.18f,-.20f,-.44f,.26f,-.13f,-.86f,.77f,.78f,-.19f,-.15f,-.77f,.37f,-.01f,-.21f,-.07f,-.78f,.54f,.78f,.38f,.69f,.32f,.04f,.42f,-.40f,.06f,.79f,-.26f,-.15f,-.27f,.23f,-.37f,.90f,-.22f,.73f,-.87f,.59f,-.53f,-.35f,-.88f,.82f,-.57f,-.86f,-.38f,-.87f,.72f,.51f,.56f,.32f,.27f,.86f,.96f,-.40f,-.54f,-.54f,.64f,.99f,-.50f,.29f,.58f,.66f,-.58f,.23f,.24f,-.67f,-.23f,.31f,.47f,-.91f,.56f,.97f,.06f,-.68f,-.36f,.47f,.98f,.41f,-.71f,-.38f,.94f,-.28f,.10f,.87f,.63f,.19f,-.93f,-.64f,.35f,.85f,-.33f,.62f,-.54f,.37f,.92f,-.22f,-.55f,-.59f,-.06f,-.81f,.81f,-.05f,.70f,.47f,.62f,-.69f,-.19f,.37f,-.57f,.66f,-.14f,-.97f,-.71f,.57f,.66f,-.46f,.66f,-.11f,.91f,.75f,-.02f,.14f,-.25f,-.32f,.92f,-.58f,.48f,.62f,.23f,.06f,.17f,-.28f,.81f,.78f,-.14f,.09f,.12f,.27f,.36f,.92f,-.06f,-.52f,-.17f,.36f,-.22f,-.41f,-.64f,.64f,-.69f,-.68f,-.07f,-.12f,.73f,.88f,-.49f,.00f,-.93f,-.45f,-.39f,.67f,-.02f,-.55f,.61f,.51f,.41f,.27f,.19f,-.49f,.14f,.16f,-.45f,-.31f,.70f,-.17f,.52f,-.34f,.66f,.84f,.47f,-.90f,-.38f,.21f,.55f,.86f,.86f,-.79f,-.44f,-.51f,-.09f,-.30f,-.39f,-.18f,-.81f,.32f,-.90f,-.14f,-.24f,.77f,-.68f,.09f,.64f,.08f,.15f,-.36f,.51f,-.54f,-.34f,-.73f,-.25f,.66f,-.33f,.23f,-.93f,.40f,.62f,.12f,.23f,.20f,.57f,-.52f,.08f,.53f,.10f,.79f,-.63f,.46f,-.12f,.21f,.60f,.84f,.84f,-.54f,-.24f,-.75f,.71f,.94f,.81f,-.07f,-.96f,-.51f,.40f,-.26f,.34f,-.79f,-.95f,-.58f,.54f,-.48f,-.83f,-.71f,-.45f,-.68f,.05f,.28f,.58f,.16f,.48f,.09f,.09f,.13f,.76f,-.54f,.14f,-.47f,.95f,-.40f,.96f,-.26f,-.38f,-.54f,.05f,.77f,-.84f,-.88f,.81f,-.69f,-.93f,-.03f,.32f,.60f,-.59f,-.07f,-.66f,.68f,-.77f,-.55f,.31f,-.22f,-.20f,.13f,.32f,.32f,.32f,-.86f,-.19f,.12f,.82f,.38f,.73f,.07f,.09f,-.04f,-.62f,.13f,-.51f,-.87f,-.48f,.07f,.13f,-.16f,.68f,-.97f,-.59f,.00f,.24f,-.67f,-.57f,.59f,.64f,.36f,-.50f,.01f,.94f,.33f,.69f,.24f,-.96f,.31f,-.21f,.77f,-.78f,-.87f,.73f,.55f,.46f,.95f,.99f,.64f,.03f,.04f,-.19f,-.03f,.54f,.19f,-.09f,.57f,.25f,.75f,-.22f,.45f,.23f,-.52f,-.27f,-.83f,.01f,-.08f,-.51f,.26f,-.36f,-.30f,-.77f,.17f,-.02f,-.57f,-.29f,-.47f,-.53f,.97f,.49f,.03f,.52f,.93f,.71f,.82f,-.35f,.38f,.47f,-.92f,-.36f,.69f,-.88f,.58f,-.02f,-.80f,-.69f,.56f,-.15f,-.74f,.88f,.35f,.24f,.03f,-.43f,-.03f,-.97f,.17f,-.92f,.44f,-.57f,-.90f,.50f,.26f,-.87f,.25f,-.70f,-.19f,.56f,.92f,.09f,.55f,-.38f,.28f,.91f,-.25f,.55f,.46f,-.80f,.24f,-.66f,.32f,-.05f,-.09f,.12f,.86f,.66f,-.93f,.88f,-.31f,-.13f,.68f,.84f,-.58f,.19f,-.61f,-.06f,-.01f,.10f,-.77f,.36f,-.05f,.96f,.83f,-.66f,.81f,.54f,.46f,-.93f,.19f,-.75f,.85f,-.26f,-.03f,-.77f,.54f,-.54f,.72f,-.05f,-.81f,.58f,-.34f,-.53f,.98f,.96f,-.55f,.99f,-.48f,-.29f,-.01f,.09f,-.02f,-.34f,-.15f,.44f,-.54f,-.82f,-.35f,-.87f,-.45f,.80f,.74f,.86f,.75f,.66f,-.69f,-.29f,-.18f,.17f,-.50f,.66f,.37f,.75f,-.44f,-.07f,.06f,-.36f,-.10f,-.63f,.12f,-.25f,-.71f,-.51f,-.75f,-.46f,.52f,-.36f,-.97f,.75f,-.23f,.26f,-.27f,.78f,-.93f,-.54f,.00f,.73f,.05f,.00f,-.95f,.19f,.53f,-.57f,.82f,-.48f,-.46f,.37f,.74f,-.99f,.46f,-.10f,.91f,.83f,-.45f,-.37f,-.05f,-.75f,-.23f,-.88f,-.16f,-.97f,.80f,.64f,-.57f,.53f,-.70f,-.19f,.96f,.26f,-.70f,.59f,-.32f,.49f,-.25f,-.01f,.57f,-.21f,.04f,-.32f,.23f,.19f,-.17f,-.58f,-.27f,-.50f,-.59f,-.77f,.07f,-.37f,.13f,-.83f,.31f,.63f,-.55f,-.32f,-.66f,-.61f,-.95f,.04f,.58f,.19f,.50f,-.79f,.32f,.89f,.58f,-.28f,.21f,-.36f,.84f,.24f,-.18f,.96f,-.43f,.17f,-.38f,-.07f,.05f,.91f,-.41f,-.24f,.93f,-.46f,-.51f,.93f,.49f,.17f,.72f,-.07f,-.94f,-.04f,.87f,.86f,-.73f,-.76f,-.54f,-.03f,.83f,.62f,.76f,-.72f,-.76f,.95f,.32f,.61f,.45f,.63f,.89f,-.73f,.63f,.30f,-.75f,-.03f,.64f,.48f,-.94f,-.40f,.28f,.91f,.29f,-.02f,.62f,-.06f,.07f,.86f,.31f,.93f,.41f,.58f,.67f,-.30f,.64f,.59f,-.33f,-.77f,-.52f,.74f,-.58f,.93f,-.17f,-.49f,-.91f,.82f,-.57f,.05f,.17f,.44f,.72f,-.95f,-.29f,.70f,-.96f,-.11f,.31f,-.52f,-.97f,-.58f,.46f,-.06f,-.30f,.25f,.16f,-.38f,.72f,.47f,.47f,.84f,.64f,-.61f,-.39f,-.78f,.58f,.19f,-.05f,.38f,.35f,.11f,-.16f,.85f,-.63f,.88f,.22f,.79f,.30f,-.50f,-.90f,.41f,.16f,.40f,-.38f,-.94f,-.61f,-.02f,.13f,-.84f,.86f,.61f,.08f,.00f,.13f,-.58f,.09f,-.10f,-.21f,-.89f,-.53f,.41f,.02f,.93f,.94f,-.95f,.14f,-.81f,.51f,.00f,-.23f,.11f,-.50f,-.62f,-.57f,.46f,.65f,.98f,.61f,.67f,-.32f,-.09f,-.70f,.62f,.40f,-.69f,.15f,.73f,.76f,.49f,-.10f,-.13f,.93f,.72f,.08f,.96f,.44f,.63f,-.68f,-.69f,.35f,.50f,-.73f,.58f,.85f,-.21f,.32f,-.87f,.05f,-.05f,.35f,.63f,-.24f,-.96f,-.42f,-.05f,.75f,.07f,-.91f,-.35f,-.80f,-.48f,-.86f,-.35f,.42f,.88f,.38f,.53f,-.74f,-.61f,.14f,-.43f,.68f,-.02f,-.30f,-.81f,-.39f,.32f,-.39f,.23f,-.82f,.17f,-.05f,-.45f,-.37f,.57f,-.44f,.72f,-.00f,.84f,.98f,-.03f,-.34f,-.51f,-.27f,.46f,-.49f,.86f,-.15f,-.70f,.91f,-.87f,-.44f,.68f,.96f,.20f,-.22f,.96f,-.46f,-.18f,.42f,-.02f,-.66f,.85f,.23f,.03f,.28f,-.56f,-.20f,-.94f,.90f,-.11f,-.63f,-.19f,-.37f,.29f,.18f,-.32f,.41f,-.42f,.95f,-.88f,.74f,-.55f,-.36f,-.70f,.09f,.08f,-.33f,.16f,.67f,.46f,-.94f,-.47f,.94f,.61f,-.40f,.97f,-.08f,.29f,.61f,-.88f,.66f,.83f,-.84f,.61f,.66f,.92f,.26f,-.12f,-.25f,-.01f,-.07f,-.98f,-.45f,.92f,-.29f,-.48f,.08f,.16f,.53f,.96f,.55f,.39f,-.61f,.45f,-.45f,-.09f,.10f,-.07f,-.11f,-.87f,-.43f,.32f,.91f,-.72f,-.75f,.55f,-.80f,.33f,.03f,-.19f,-.93f,-.65f,.92f,-.26f,.89f,.50f,.28f,-.98f,-.55f,-.80f,-.50f,.33f,.08f,.85f,-.33f,.57f,-.59f,.97f,.95f,.51f,.60f,-.80f,.92f,-.55f,.10f,.13f,-.89f,.04f,-.75f,.73f,.80f,.87f,.23f,.59f,-.63f,.70f,-.95f,.82f,-.27f,.54f,.19f,.96f,.95f,-.81f,.02f,.45f,.40f,-.52f,-.74f,.27f,.37f,-.27f,-.23f,.24f,.28f,.27f,.38f,.62f,-.84f,.22f,-.88f,-.77f,.61f,-.22f,-.26f,.59f,.63f,.49f,-.77f,.30f,-.57f,.39f,-.21f,-.68f,.43f,.60f,.30f,-.51f,-.28f,.26f,-.06f,-.68f,.50f,-.07f,.16f,.25f,-.39f,-.08f,.81f,.83f,-.72f,-.67f,.35f,-.08f,.37f,-.24f,.64f,.37f,-.17f,.61f,-.35f,.21f,.72f,.98f,.38f,-.14f,.59f,.45f,.30f,.61f,.62f,.56f,-.66f,-.39f,.11f,-.43f,-.08f,.75f,.31f,-.64f,-.53f,.41f,.48f,.54f,.62f,-.64f,-.58f,-.00f,.05f,-.62f,-.65f,.72f,-.90f,.29f,-.56f,-.68f,.98f,.84f,-.43f,.16f,-.43f,.16f,-.86f,.23f,.70f,-.94f,-.74f,.41f,.20f,.02f,-.35f,-.99f,-.87f,-.65f,.77f,.32f,.92f,.14f,.95f,-.80f,-.14f,.10f,.60f,.06f,.19f,.28f,.32f,-.87f,.32f,-.61f,-.94f,-.76f,.91f,-.77f,-.80f,-.33f,.36f,-.66f,.50f,.06f,-.79f,-.14f,.38f,.54f,-.34f,-.51f,-.35f,-.78f,.40f,.78f,-.55f,-.93f,.77f,-.93f,.69f,.86f,-.07f,-.78f,-.16f,.30f,-.95f,-.53f,.73f,-.75f,-.55f,-.16f,-.30f,-.74f,.95f,-.18f,-.65f,.99f,-.87f,.49f,.24f,-.06f,.58f,-.93f,-.93f,-.36f,.74f,.54f,-.62f,.09f,.09f,-.05f,-.06f,.15f,-.69f,-.22f,-.20f,.70f,.56f,-.30f,-.70f,-.69f,-.10f,.44f,-.01f,-.95f,-.85f,.53f,-.05f,.00f,-.66f,.05f,-.52f,.64f,-.21f,.22f,-.25f,-.22f,.28f,-.70f,.57f,-.90f,-.33f,-.69f,-.14f,.08f,-.38f,-.14f,-.97f,.84f,.69f,-.38f,.36f,.42f,-.39f,.29f,.50f,-.36f,-.58f,-.40f,.88f,.70f,.59f,.88f,.90f,-.10f,-.22f,-.68f,-.92f,-.55f,-.46f,.96f,-.50f,.08f,.92f,-.68f,-.27f,.29f,-.95f,.58f,.69f,-.62f,-.25f,.02f,-.50f,.01f,.33f,-.93f,-.85f,-.51f,.04f,.29f,-.32f,-.66f,.08f,.94f,-.81f,-.70f,.44f,-.13f,.72f,-.81f,-.55f,-.53f,-.27f,-.76f,.04f,.91f,.80f,-.91f,-.79f,-.11f,.17f,-.42f,.57f,.72f,-.52f,-.12f,-.00f,-.17f,.98f,-.30f,.20f,.64f,-.29f,-.42f,.45f,.85f,.53f,.47f,-.29f,-.04f,.97f,.86f,-.13f,-.96f,-.34f,.75f,-.04f,-.21f,.59f,.68f,.36f,-.11f,.76f,-.59f,-.28f,.07f,.49f,.23f,-.01f,-.00f,-.33f,-.14f,.17f,.68f,.41f,-.02f,-.40f,.75f,.39f,-.07f,.11f,.66f,.47f,.90f,.17f,-.15f,.48f,-.10f,-.81f,.47f,.72f,.51f,-.64f,.41f,-.21f,.97f,.42f,.81f,.00f,-.99f,.43f,.83f,.78f,-.92f,.32f,.59f,-.96f,-.46f,.32f,.53f,-.28f,-.00f,-.04f,-.99f,.67f,-.13f,-.43f,.50f,-.65f,.96f,.71f,-.71f,-.40f,.44f,.25f,.90f,.45f,.71f,-.66f,.00f,.52f,.93f,.58f,.29f,.62f,.27f,.56f,.42f,.26f,.78f,-.92f,-.29f,.98f,.97f,.80f,.53f,.40f,-.28f,-.01f,-.15f,.00f,.18f,.01f,-.46f,-.81f,-.42f,.58f,-.47f,.40f,-.81f,.67f,.70f,-.64f,.31f,-.75f,-.07f,.73f,.31f,.20f,.07f,-.31f,-.50f,-.91f,-.53f,.80f,.13f,-.31f,.46f,.99f,-.52f,.09f,-.21f,.46f,-.76f,.27f,.88f,.30f,-.83f,-.85f,.98f,-.34f,-.84f,.50f,-.78f,.17f,-.72f,-.06f,-.53f,-.53f,-.74f,.95f,.42f,.25f,-.73f,.10f,-.06f,-.87f,.01f,.54f,.87f,.68f,-.85f,.53f,-.02f,.86f,-.15f,.23f,.18f,-.34f,.67f,-.82f,-.70f,.35f,-.90f,.09f,-.03f,.94f,.69f,.43f,.60f,-.87f,.45f,.61f,-.31f,.54f,-.91f,-.73f,.58f,.27f,-.81f,-.94f,-.63f,-.91f,.10f,.25f,.08f,.47f,-.75f,-.34f,.69f,.37f,.90f,.21f,.81f,-.53f,.80f,-.67f,-.95f,-.55f,.21f,-.04f,-.12f,-.64f,.78f,.15f,-.49f,.26f,-.29f,.09f,-.84f,-.93f,.11f,.58f,-.67f,.60f,-.40f,-.61f,.72f,-.36f,-.24f,-.89f,-.21f,-.82f,-.37f,-.57f,.72f,-.66f,.63f,.74f,.37f,-.51f,.37f,-.71f,.92f,-.34f,.02f,-.95f,-.37f,-.92f,.04f,.12f,-.26f,-.43f,-.81f,-.26f,-.48f,.44f,-.12f,-.75f,.98f,-.31f,-.01f,.29f,.77f,.29f,.87f,.47f,.18f,.07f,-.99f,.81f,.14f,.53f,-.03f,-.22f,.28f,-.56f,-.86f,.52f,.83f,.89f,-.54f,-.48f,-.91f,.80f,-.12f,-.06f,.09f,.75f,-.36f,.55f,.55f,.46f,.74f,.53f,.99f,-.33f,-.45f,-.06f,-.12f,.19f,-.17f,-.38f,-.78f,-.58f,-.74f,-.65f,-.03f,-.77f,-.38f,-.97f,.24f,.57f,.75f,.45f,.39f,.89f,-.81f,-.42f,.93f,-.87f,.52f,.44f,-.34f,-.16f,-.02f,-.85f,.97f,-.41f,-.57f,.96f,-.94f,-.66f,.92f,-.94f,.23f,-.06f,-.68f,.37f,.30f,.75f,-.51f,.12f,.52f,-.17f,.55f,-.17f,.40f,.60f,.97f,.83f,-.13f,-.28f,.73f,-.77f,.36f,-.03f,.96f,.41f,.97f,-.65f,-.78f,-.68f,.71f,-.25f,-.02f,-.14f,.32f,.48f,-.85f,-.12f,-.11f,.90f,-.59f,-.70f,.19f,.53f,-.83f,.34f,-.22f,.34f,-.37f,.01f,-.64f,-.40f,-.64f,-.04f,-.41f,-.77f,-.63f,.16f,-.17f,.29f,-.19f,.87f,.49f,-.09f,.27f,-.24f,.73f,-.55f,.97f,-.62f,.64f,.27f,-.66f,-.34f,.39f,.81f,-.13f,.14f,.88f,-.43f,.79f,-.88f,.91f,-.25f,-.24f,-.40f,-.51f,-.61f,.11f,.45f,.23f,.60f,-.73f,.20f,.65f,-.25f,-.67f,.76f,-.83f,.64f,-.01f,-.51f,-.27f,-.03f,.33f,.27f,-.69f,-.85f,.06f,.53f,-.78f,.85f,-.26f,.46f,-.79f,.86f,-.25f,-.67f,-.88f,-.69f,-.71f,.58f,.09f,-.60f,.83f,.40f,-.88f,-.15f,.46f,.05f,.37f,.25f,-.85f,-.39f,.69f,-.68f,-.21f,.97f,-.03f,-.13f,-.59f,.65f,-.72f,-.84f,-.80f,.42f,-.36f,-.11f,-.84f,-.27f,-.13f,-.03f,.82f,.33f,-.24f,.30f,-.28f,.41f,-.56f,-.47f,-.18f,-.57f,-.56f,-.87f,-.05f,-.17f,-.31f,-.73f,-.89f,.01f,.20f,.06f,-.85f,-.75f,.78f,-.60f,.87f,.91f,-.13f,.75f,-.49f,.96f,-.60f,-.87f,-.49f,.83f,.42f,-.29f,.32f,-.46f,-.84f,.37f,-.36f,.39f,-.72f,.24f,.28f,-.63f,.40f,.92f,-.75f,-.68f,.74f,.07f,.60f,.22f,-.50f,-.10f,-.08f,.50f,-.34f,.24f,-.77f,-.23f,.77f,.89f,-.20f,.89f,-.95f,.35f,.19f,.70f,-.10f,.07f,.71f,.16f,.33f,.37f,.13f,-.98f,-.12f,-.50f,.68f,-.62f,-.38f,.13f,.86f,.22f,-.56f,.35f,-.98f,-.72f,.86f,-.44f,.73f,.57f,.84f,-.93f,-.62f,-.67f,.14f,-.73f,-.85f,-.38f,.01f,-.87f,.05f,-.98f,-.18f,.56f,.90f,-.66f,-.81f,.09f,-.63f,.04f,-.88f,.74f,.73f,.71f,.68f,.27f,-.52f,.16f,-.72f,.10f,-.71f,.97f,-.55f,-.06f,-.68f,-.74f,-.66f,-.93f,.23f,-.35f,.37f,.54f,.60f,.10f,-.76f,.45f,-.66f,-.08f,.98f,-.09f,.96f,-.54f,-.33f,-.04f,.36f,.86f,-.42f,-.50f,.45f,.33f,.78f,.25f,.58f,.15f,-.89f,-.49f,.42f,.78f,-.86f,-.65f,.81f,-.67f,.62f,.14f,.62f,.05f,-.93f,.34f,-.40f,.01f,.46f,-.34f,.51f,-.86f,.94f,-.20f,-.75f,.27f,-.09f,.41f,-.86f,.80f,-.13f,-.31f,.52f,-.12f,.46f,.58f,.60f,-.12f,-.45f,-.61f,-.33f,-.80f,-.25f,-.00f,-.20f,-.48f,-.98f,.96f,-.86f,-.21f,.30f,.26f,.53f,.83f,.82f,-.39f,-.66f,-.69f,-.42f,.35f,-.88f,.27f,-.34f,-.21f,-.89f,.74f,-.98f,-.35f,-.72f,-.37f,-.32f,.39f,-.24f,-.83f,.95f,-.43f,.72f,.24f,-.85f,-.19f,.55f,.02f,.34f,-.41f,-.08f,-.25f,.40f,-.00f,.58f,.43f,.79f,.29f,-.57f,-.02f,-.31f,-.01f,.33f,-.09f,-.07f,.53f,-.58f,.94f,-.64f,.37f,.63f,-.79f,.21f,.72f,.24f,.09f,.34f,.15f,.41f,.94f,.69f,.51f,.52f,-.03f,-.81f,-.99f,-.76f,.33f,.61f,.03f,-.61f,.78f,.69f,.62f,-.22f,-.70f,-.64f,.55f,-.56f,.24f,-.39f,.33f,-.08f,.92f,.88f,-.94f,-.85f,-.47f,-.96f,-.51f,.22f,.87f,-.85f,.51f,.27f,.12f,-.83f,-.13f,-.63f,.72f,.51f,.35f,-.94f,.60f,.79f,-.57f,-.70f,.21f,-.68f,-.84f,.10f,.76f,.92f,.60f,-.16f,.85f,-.28f,-.00f,.25f,.61f,.69f,-.62f,.17f,.05f,.08f,-.87f,.23f,.95f,-.82f,.84f,-.58f,.76f,-.46f,-.35f,.76f,-.57f,.70f,-.14f,.55f,.57f,.57f,-.28f,-.18f,.46f,.89f,.07f,.45f,-.65f,-.46f,.01f,.32f,-.96f,-.71f,.05f,.35f,-.90f,-.56f,.72f,-.33f,-.48f,.42f,.32f,.16f,-.55f,-.83f,.42f,.74f,.41f,.93f,.61f,-.39f,-.03f,-.81f,-.37f,-.55f,-.06f,.62f,.69f,.74f,.21f,.12f,-.18f,.25f,-.66f,.40f,-.33f,.95f,.72f,.29f,.71f,-.46f,-.56f,-.79f,.59f,-.71f,-.02f,.61f,.66f,.23f,-.44f,.99f,.66f,-.47f,.99f,.68f,-.21f,-.96f,-.67f,.39f,.22f,-.72f,.01f,.62f,.68f,-.87f,-.92f,-.64f,-.60f,-.51f,-.45f,.72f,-.86f,.99f,-.77f,.36f,.20f,.17f,-.32f,-.06f,.10f,.63f,-.08f,-.01f,.58f,-.31f,.18f,-.09f,.10f,-.72f,.98f,-.76f,-.21f,-.61f,.19f,-.52f,.13f,.39f,-.53f,-.86f,-.91f,-.57f,.61f,.47f,-.73f,.63f,.07f,.90f,-.58f,.52f,.45f,-.51f,-.87f,.31f,.25f,-.08f,.53f,-.73f,-.08f,-.86f,-.63f,.63f,-.40f,-.66f,-.40f,-.71f,-.76f,.66f,-.22f,-.58f,.49f,-.97f,.84f,.96f,-.09f,-.92f,-.24f,.01f,-.97f,-.80f,.00f,-.99f,.62f,.62f,-.32f,.11f,.30f,.80f,.62f,-.57f,.89f,.57f,-.87f,-.95f,.74f,-.57f,-.31f,.05f,-.78f,-.51f,.84f,-.40f,-.97f,-.31f,.89f,-.41f,.15f,.52f,.19f,-.33f,.07f,.74f,.06f,-.64f,-.70f,.58f,-.31f,-.99f,-.46f,.28f,.08f,-.73f,-.31f,.70f,-.90f,.28f,.88f,-.92f,-.21f,-.63f,.47f,.72f,.86f,.95f,.79f,.34f,.16f,.91f,.65f,-.86f,-.81f,.67f,.18f,-.58f,.25f,-.49f,.52f,-.86f,-.25f,.11f,-.97f,.44f,.43f,.64f,.27f,-.64f,-.44f,.96f,-.44f,-.79f,.17f,-.21f,.86f,-.61f,-.16f,-.68f,.95f,.67f,.58f,.10f,.61f,-.99f,-.01f,.86f,-.80f,.35f,-.44f,.84f,.92f,-.35f,.62f,-.40f,.71f,-.38f,.43f,.02f,.74f,-.66f,.68f,.29f,-.79f,.36f,.07f,-.04f,.43f,-.51f,-.47f,.55f,.15f,.82f,-.94f,-.59f,-.85f,-.74f,.85f,.41f,.59f,.53f,.91f,.12f,.92f,-.52f,.41f,.41f,-.25f,.64f,-.45f,.25f,-.56f,-.53f,.16f,-.75f,-.85f,-.16f,-.84f,-.85f,-.57f,-.57f,-.00f,.41f,-.96f,.34f,.47f,-.53f,-.84f,.00f,.39f,-.38f,.87f,-.84f,.56f,-.49f,.32f,.23f,-.04f,-.16f,-.01f,-.47f,-.02f,.57f,-.69f,-.71f,-.71f,.72f,.82f,.65f,.73f,-.31f,.76f,-.64f,.15f,-.50f,-.59f,-.94f,.81f,.78f,-.19f,-.56f,.87f,-.71f,.65f,.43f,.79f,.33f,.07f,.33f,-.77f,.11f,.88f,.47f,.66f,-.62f,.89f,.22f,-.49f,-.71f,-.87f,.77f,-.86f,.08f,-.53f,-.58f,.89f,.57f,-.65f,-.25f,-.48f,-.28f,-.59f,.60f,-.56f,-.06f,.33f,.29f,-.00f,-.66f,.86f,-.90f,-.59f,-.32f,.00f,-.16f,-.59f,.23f,.45f,-.86f,.59f,.37f,-.79f,-.57f,.77f,.34f,-.08f,-.98f,.01f,.69f,.28f,-.84f,.74f,-.02f,.49f,.29f,.07f,.09f,.47f,.87f,.25f,.38f,-.31f,-.12f,-.23f,.23f,-.94f,-.48f,.38f,-.12f,-.66f,-.72f,-.21f,.83f,.78f,-.27f,.81f,.58f,-.93f,-.63f,-.42f,-.16f,-.96f,-.73f,.76f,.32f,-.60f,.50f,.24f,-.48f,-.20f,.99f,-.24f,.59f,.07f,-.80f,-.01f,.45f,-.48f,.34f,-.60f,.15f,-.40f,.68f,.69f,.13f,-.99f,-.21f,-.17f,.29f,-.20f,.86f,-.80f,.82f,-.36f,.79f,.75f,-.37f,-.54f,.16f,-.86f,.36f,-.46f,.33f,-.31f,.75f,-.62f,.79f,-.45f,.34f,-.72f,-.67f,.06f,.17f,.57f,.89f,-.08f,.58f,-.01f,.27f,.05f,-.42f,-.22f,.86f,.92f,-.66f,-.01f,-.62f,.62f,.57f,.17f,.47f,-.50f,-.72f,-.90f,.05f,-.40f,-.77f,-.71f,-.03f,-.81f,.11f,-.34f,-.70f,.05f,.74f,-.27f,-.50f,-.47f,.87f,-.15f,.14f,.31f,-.67f,.74f,-.55f,.41f,-.80f,.84f,-.23f,.10f,-.17f,.65f,.79f,-.67f,.67f,-.17f,.32f,-.06f,.44f,.13f,.26f,-.67f,.63f,-.17f,.40f,.26f,.55f,.29f,.62f,.83f,-.41f,-.07f,.40f,-.40f,-.21f,.45f,-.88f,-.83f,.85f,-.84f,.43f,.44f,.24f,-.32f,.29f,-.62f,-.24f,.38f,-.84f,-.97f,-.68f,.44f,-.70f,-.95f,-.70f,-.39f,-.30f,-.72f,-.55f,-.09f,-.15f,.77f,.40f,-.16f,-.94f,-.24f,-.79f,-.94f,.46f,-.35f,.93f,.10f,.39f,-.37f,-.63f,.03f,-.13f,.64f,.49f,-.52f,.24f,-.59f,.13f,.42f,-.19f,.33f,-.33f,-.67f,.43f,.22f,-.85f,.83f,-.51f,.60f,.15f,-.90f,-.24f,-.39f,.90f,-.87f,.39f,-.70f,-.05f,-.09f,-.95f,-.05f,-.08f,.92f,.95f,-.15f,-.85f,-.87f,.53f,-.31f,.49f,-.59f,.92f,-.82f,-.97f,.40f,-.60f,.80f,-.48f,-.68f,.33f,.73f,-.37f,-.31f,.26f,.62f,.11f,.65f,-.73f,.26f,-.88f,.33f,-.03f,-.54f,-.92f,-.74f,-.80f,-.82f,-.61f,-.97f,-.09f,-.81f,-.83f,-.84f,.02f,-.26f,.60f,-.30f,.41f,-.59f,-.07f,-.70f,.74f,.25f,-.50f,.88f,.73f,-.40f,.49f,.84f,.27f,.80f,-.06f,-.26f,.16f,-.87f,-.86f,.19f,.88f,.00f,.19f,.89f,.94f,.11f,-.09f,.04f,-.15f,-.07f,.56f,.71f,-.94f,-.13f,.72f,.50f,-.17f,-.58f,-.35f,.20f,-.18f,-.31f,-.80f,-.95f,-.00f,.84f,.91f,.34f,.89f,-.08f,.90f,.80f,-.41f,-.50f,.95f,.41f,.52f,-.21f,-.99f,-.77f,.41f,-.32f,-.99f,.21f,.29f,.82f,.99f,.57f,-.46f,.77f,-.43f,-.71f,.13f,-.61f,-.38f,.89f,-.15f,-.46f,-.81f,-.75f,.54f,.45f,.88f,-.25f,-.74f,.38f,.52f,-.93f,.54f,.50f,.52f,.41f,-.64f,.64f,.04f,.33f,.38f,-.97f,.08f,-.90f,.49f,.54f,-.09f,.90f,.66f,.20f,-.75f,-.42f,.64f,.33f,.21f,-.08f,.02f,.92f,-.18f,-.28f,.75f,.13f,.37f,.17f,.89f,.23f,-.47f,.90f,-.24f,.13f,-.81f,.73f,-.12f,.76f,-.54f,.19f,.98f,.81f,.99f,.76f,-.33f,.12f,.22f,.02f,-.43f,-.39f,-.74f,-.79f,.26f,-.14f,.52f,-.96f,-.60f,-.79f,-.53f,.11f,.65f,-.12f,-.92f,-.85f,-.06f,.31f,-.55f,.27f,-.87f,-.46f,-.95f,.23f,-.52f,-.96f,.06f,-.22f,-.44f,-.11f,.34f,-.89f,.43f,-.24f,.48f,-.07f,.08f,-.85f,-.25f,.92f,-.27f,-.03f,-.06f,-.85f,.12f,-.55f,-.70f,-.11f,.70f,.69f,.76f,-.91f,-.63f,-.61f,-.98f,.73f,-.90f,-.62f,.68f,-.87f,-.56f,-.27f,-.70f,.99f,.79f,-.41f,.54f,-.64f,.61f,-.35f,.90f,-.33f,.51f,-.59f,.95f,-.73f,.15f,-.22f,.26f,.72f,.51f,.61f,-.00f,.37f,.58f,.26f,-.13f,.41f,.41f,-.95f,-.79f,.58f,-.20f,-.86f,-.20f,-.60f,-.44f,.11f,-.23f,.91f,.01f,.87f,-.19f,.24f,.01f,-.88f,.09f,.08f,.83f,.02f,-.57f,-.21f,.93f,.91f,.51f,-.74f,.29f,.31f,.41f,.55f,.72f,-.93f,-.47f,.95f,-.73f,-.76f,-.98f,-.84f,.31f,-.89f,-.27f,-.70f,-.22f,.39f,.81f,-.85f,.69f,-.50f,-.53f,-.14f,-.02f,.31f,-.74f,-.19f,.84f,.99f,-.92f,-.35f,-.30f,.85f,-.64f,-.84f,.69f,.77f,.91f,.04f,.94f,.44f,.28f,.13f,-.65f,-.84f,.61f,.01f,.78f,-.17f,.27f,.72f,-.14f,-.18f,.70f,-.19f,.57f,-.89f,.48f,.36f,.39f,.46f,.67f,.16f,-.86f,-.52f,.20f,-.18f,.70f,-.11f,-.57f,-.96f,-.27f,.41f,.72f,-.33f,-.93f,.99f,-.90f,.79f,.17f,-.40f,-.13f,.92f,.74f,-.58f,-.76f,-.93f,.65f,.97f,.27f,-.84f,-.61f,.83f,-.56f,-.76f,-.69f,-.44f,-.84f,-.95f,.33f,.88f,-.70f,.30f,-.69f,.50f,-.65f,-.16f,-.51f,.81f,.13f,-.45f,.35f,-.10f,.31f,-.21f,.39f,.49f,-.66f,.44f,.71f,.87f,-.61f,.86f,-.65f,.87f,-.89f,.70f,-.20f,-.83f,.61f,.43f,-.98f,-.09f,-.20f,-.09f,.99f,.45f,.28f,-.28f,-.46f,.05f,.96f,.66f,.72f,.27f,-.07f,-.27f,.54f,.16f,.54f,-.60f,-.50f,-.94f,-.28f,-.05f,.52f,.65f,-.99f,.50f,-.51f,.28f,-.93f,-.15f,-.64f,-.62f,.36f,-.83f,.86f,.42f,.60f,.59f,.42f,.89f,.50f,.73f,-.41f,-.61f,-.51f,-.32f,-.53f,.64f,.98f,-.94f,-.13f,.06f,.20f,-.18f,-.05f,.50f,-.39f,-.08f,.94f,-.22f,-.01f,.51f,.43f,-.18f,-.40f,.08f,-.40f,.44f,-.45f,-.39f,.39f,.02f,.81f,-.75f,.91f,-.83f,.06f,-.36f,.03f,.11f,-.19f,-.29f,.21f,-.61f,.44f,-.86f,-.69f,-.18f,.51f,.90f,-.05f,.39f,-.89f,-.61f,.66f,-.06f,.90f,.20f,.91f,-.50f,-.63f,.68f,.40f,-.30f,.28f,.98f,-.56f,-.13f,-.12f,.29f,.66f,.07f,.48f,.60f,.09f,.09f,.38f,.63f,-.27f,.35f,-.26f,-.83f,-.63f,-.84f,.87f,-.94f,.07f,.17f,-.98f,.76f,-.98f,.81f,.18f,-.77f,-.80f,-.26f,.76f,.41f,-.40f,-.80f,.16f,.96f,.81f,.47f,-.29f,.57f,-.06f,-.04f,-.30f,-.92f,-.18f,.32f,-.66f,-.90f,.74f,.61f,.37f,.34f,.67f,.08f,.80f,-.27f,-.49f,-.15f,.84f,-.71f,.56f,.24f,.89f,-.00f,-.44f,.67f,-.27f,.31f,-.86f,.52f,.87f,-.03f,-.34f,-.38f,.82f,.84f,-.30f,-.13f,.86f,-.35f,.67f,.45f,-.24f,.95f,.70f,-.83f,-.17f,-.09f,-.35f,-.47f,.71f,-.75f,.33f,.64f,-.68f,.43f,.38f,.48f,.21f,.85f,-.77f,.11f,-.19f,.05f,-.17f,-.67f,-.84f,-.54f,-.54f,-.70f,-.39f,-.52f,-.42f,-.78f,.39f,.70f,-.06f,-.79f,.41f,.94f,.83f,-.33f,.98f,-.14f,-.40f,-.72f,.66f,.79f,.79f,.29f,-.14f,-.34f,.40f,-.95f,.90f,-.38f,-.29f,.95f,.72f,.17f,-.54f,.29f,-.10f,.93f,.69f,-.19f,.20f,.02f,.96f,-.05f,-.49f,-.93f,.32f,-.31f,.07f,.71f,-.58f,.08f,.95f,-.55f,.95f,.05f,.42f,-.98f,.95f,-.09f,-.77f,.42f,.60f,-.16f,.29f,.25f,.39f,-.81f,-.90f,.46f,-.30f,.10f,.07f,.81f,-.57f,-.31f,-.21f,-.12f,.01f,-.11f,.66f,.01f,.90f,-.38f,.52f,.08f,-.77f,-.66f,-.54f,.00f,-.44f,.17f,.38f,-.33f,.80f,-.80f,.15f,-.51f,-.23f,-.52f,.37f,.58f,-.35f,-.71f,-.79f,-.47f,-.69f,-.88f,-.48f,.28f,.71f,.46f,-.28f,-.13f,.64f,-.43f,-.44f,.44f,-.11f,.22f,.10f,-.94f,.42f,-.15f,.25f,-.25f,-.88f,-.29f,.92f,-.13f,.06f,-.35f,-.66f,-.77f,.02f,-.55f,.51f,-.57f,.50f,.68f,-.22f,.23f,.00f,-.66f,-.36f,.44f,.25f,.28f,.96f,.68f,-.22f,-.33f,-.67f,.11f,-.33f,-.15f,-.16f,-.87f,-.37f,.18f,.62f,-.25f,-.13f,.65f,-.03f,-.00f,.30f,.77f,.63f,.57f,.93f,-.37f,-.06f,-.72f,.39f,.36f,-.28f,.53f,.00f,.71f,-.57f,-.64f,.30f,-.40f,-.30f,.05f,-.90f,.36f,.87f,.77f,.03f,-.41f,-.91f,.81f,.16f,-.12f,-.18f,.30f,-.51f,.26f,-.46f,-.62f,.79f,-.98f,-.60f,-.90f,.01f,-.52f,.66f,-.49f,.75f,-.68f,-.78f,-.70f,.44f,.07f,-.90f,-.50f,.22f,.04f,-.54f,.35f,-.38f,.18f,-.04f,-.09f,-.83f,-.16f,-.99f,-.78f,-.73f,.71f,.27f,.15f,.19f,.29f,-.20f,.00f,.38f,.84f,-.20f,.28f,.19f,-.02f,-.50f,-.66f,-.23f,.79f,.62f,.20f,-.24f,.60f,-.39f,-.61f,-.53f,-.93f,.99f,.78f,-.92f,.92f,.43f,.46f,.87f,.32f,-.53f,.85f,-.73f,-.71f,-.39f,-.90f,-.63f,.01f,-.38f,-.62f,-.25f,-.37f,.60f,-.01f,.18f,.95f,.99f,-.70f,-.76f,-.15f,-.78f,.02f,.08f,.27f,-.61f,.62f,.59f,-.04f,.46f,-.63f,-.05f,.85f,-.05f,-.84f,-.35f,-.35f,.63f,.55f,-.33f,-.53f,.62f,.64f,-.02f,.06f,-.39f,-.55f,-.09f,-.17f,-.09f,-.84f,.94f,-.08f,-.95f,-.25f,.12f,-.26f,-.94f,.32f,-.60f,-.11f,-.27f,-.32f,.31f,.97f,.64f,.99f,-.99f,-.56f,.96f,.11f,-.57f,-.51f,.82f,-.35f,.13f,.70f,-.97f,.92f,.59f,-.03f,.15f,-.09f,.46f,.28f,.73f,-.17f,.26f,-.23f,.45f,-.96f,.21f,-.28f,-.64f,-.09f,.68f,.49f,-.30f,-.75f,.98f,-.71f,-.42f,-.52f,.19f,.52f,.30f,.92f,.61f,-.53f,.62f,-.15f,.19f,.07f,.67f,.92f,.20f,-.69f,-.29f,-.75f,.54f,.78f,.92f,-.70f,-.70f,-.40f,-.41f,.89f,-.29f,.02f,.34f,-.24f,.48f,.97f,.75f,-.66f,.17f,-.32f,-.41f,.24f,.85f,-.70f,.39f,.46f,-.84f,.86f,.45f,-.78f,-.61f,.74f,.73f,.78f,.66f,-.15f,-.85f,.46f,.46f,-.21f,-.78f,-.16f,.12f,.83f,.05f,-.95f,.41f,.45f,-.02f,.56f,-.91f,-.90f,-.33f,-.94f,-.72f,-.90f,-.22f,-.01f,-.59f,.58f,-.37f,.23f,-.96f,.41f,-.65f,.69f,-.79f,.14f,.39f,-.99f,-.21f,-.73f,-.33f,.53f,-.15f,-.04f,-.00f,.33f,.12f,-.44f,-.88f,-.93f,.05f,.73f,-.66f,-.85f,-.17f,-.47f,.15f,.65f,.91f,-.72f,.32f,.64f,-.30f,-.40f,-.79f,-.77f,.27f,.09f,.65f,-.85f,.37f,.57f,-.79f,.28f,.02f,-.92f,.23f,.66f,-.38f,.24f,-.21f,.84f,-.71f,-.12f,.26f,-.29f,.26f,-.08f,.75f,.54f,.46f,.11f,-.81f,-.53f,.85f,-.24f,-.38f,.95f,.36f,.77f,-.37f,.58f,.91f,.03f,.13f,-.98f,.68f,.74f,.52f,.42f,.83f,-.76f,.52f,-.79f,-.57f,.95f,-.58f,-.11f,-.08f,.57f,.75f,-.80f,.33f,.42f,-.85f,-.74f,-.91f,-.99f,-.08f,-.06f,-.42f,.33f,-.45f,-.09f,.56f,-.95f,.98f,-.24f,.48f,-.85f,-.05f,.73f,.75f,.43f,.00f,-.29f,-.75f,-.39f,-.76f,.35f,.29f,-.19f,-.61f,-.65f,-.35f,.74f,-.12f,.62f,.59f,-.11f,-.36f,-.37f,-.59f,.28f,-.28f,-.00f,.60f,-.40f,.10f,-.94f,-.25f,-.50f,-.12f,-.90f,.13f,-.39f,.30f,-.87f,.76f,-.61f,.55f,.17f,.74f,-.36f,.52f,.71f,-.20f,.07f,.40f,.61f,.06f,.68f,.71f,-.36f,-.52f,-.61f,-.70f,-.40f,.00f,.69f,-.44f,-.15f,.45f,-.63f,.88f,.23f,-.98f,.29f,-.86f,-.24f,.65f,-.61f,-.61f,.81f,.10f,.24f,.84f,-.98f,-.26f,-.17f,-.05f,.88f,-.93f,-.34f,.10f,.89f,-.92f,.88f,-.01f,.30f,.01f,.65f,-.87f,-.51f,.42f,.60f,.02f,.56f,-.56f,-.00f,.63f,.40f,.32f,-.12f,.05f,.61f,-.57f,-.84f,.20f,-.91f,.48f,.64f,-.60f,-.24f,.84f,.21f,-.48f,.26f,-.71f,.31f,.21f,.59f,.90f,-.38f,.56f,-.72f,.22f,.01f,.74f,.19f,-.40f,-.63f,.76f,-.67f,.27f,.09f,-.04f,.49f,.33f,.55f,.73f,-.41f,-.49f,.16f,-.93f,.37f,.62f,.68f,-.51f,.08f,.89f,-.35f,.26f,.77f,.33f,.42f,-.66f,.65f,-.34f,.87f,-.97f,-.16f,.43f,-.93f,-.06f,.00f,.40f,.06f,-.16f,.79f,.46f,-.60f,-.68f,.11f,-.19f,.33f,-.37f,-.94f,-.08f,.11f,.20f,.78f,.34f,-.61f,.60f,-.93f,-.36f,.96f,-.37f,-.79f,-.56f,.86f,.36f,-.68f,-.18f,.54f,.14f,.00f,.23f,.14f,.83f,-.45f,.01f,.59f,-.02f,.43f,-.03f,.97f,.62f,-.12f,.90f,-.90f,-.08f,.36f,-.72f,.02f,.22f,-.24f,.96f,-.33f,-.13f,.44f,-.65f,.95f,.68f,-.76f,.18f,-.06f,.26f,.74f,-.38f,-.08f,.15f,.45f,-.97f,.43f,-.82f,-.84f,.99f,-.55f,-.93f,-.07f,-.29f,-.59f,-.91f,.49f,-.60f,-.56f,.63f,.86f,.49f,.76f,.69f,.12f,.67f,.60f,.12f,-.79f,.78f,.35f,.52f,-.80f,-.12f,.25f,.43f,.69f,.12f,-.01f,.47f,.65f,-.38f,.99f,.28f,-.37f,-.18f,.69f,.56f,.80f,.89f,-.13f,.57f,.45f,-.11f,.62f,.83f,.14f,-.80f,-.12f,-.04f,.25f,.59f,-.17f,.04f,-.75f,.75f,-.56f,-.46f,-.27f,.22f,.53f,.58f,.03f,.44f,-.14f,.26f,-.22f,.74f,.28f,.50f,-.21f,-.51f,-.50f,.73f,-.67f,.19f,-.58f,-.29f,.56f,-.77f,.18f,-.46f,.59f,.80f,.95f,-.66f,-.67f,-.57f,.01f,-.20f,.27f,.67f,-.12f,-.34f,.72f,-.05f,.95f,-.26f,.30f,.87f,-.61f,.91f,-.68f,.20f,-.75f,-.04f,.12f,.37f,-.34f,.06f,-.92f,-.76f,-.04f,-.17f,-.73f,.50f,-.32f,-.77f,-.25f,-.72f,-.32f,-.88f,.97f,.63f,-.05f,.05f,.33f,.46f,.65f,-.38f,.36f,.52f,-.33f,-.09f,.57f,-.44f,-.80f,.18f,-.14f,.83f,.53f,.45f,-.82f,.11f,.24f,.56f,-.47f,-.35f,-.33f,.97f,-.77f,-.42f,.61f,-.45f,.57f,.43f,.69f,.46f,.22f,.10f,-.24f,.52f,.55f,-.97f,.09f,-.52f,.78f,.50f,.87f,-.04f,.21f,.92f,.15f,-.85f,-.64f,.25f,.20f,.35f,.85f,-.95f,-.36f,-.75f,.85f,.62f,.04f,-.87f,.41f,-.07f,-.71f,-.30f,.07f,.37f,-.16f,.39f,-.62f,-.02f,.57f,-.37f,.31f,-.55f,-.41f,-.11f,-.32f,-.19f,.31f,.29f,.44f,.09f,-.54f,.38f,.07f,.10f,-.71f,-.97f,-.48f,-.55f,.49f,-.95f,.92f,.69f,.24f,-.82f,.33f,-.03f,-.20f,-.14f,-.77f,-.16f,.57f,.39f,.84f,-.73f,-.48f,.65f,.33f,.24f,-.56f,.31f,.23f,.44f,-.97f,-.87f,.14f,-.73f,-.02f,.18f,-.09f,-.02f,-.27f,.09f,.96f,-.01f,-.79f,.29f,-.34f,.99f,.87f,-.45f,.55f,-.25f,-.69f,-.75f,-.45f,-.27f,-.00f,-.99f,.26f,.07f,-.60f,-.50f,-.11f,.20f,.01f,-.09f,-.56f,.85f,.96f,.02f,.09f,-.63f,.13f,.61f,.46f,-.97f,.29f,.09f,-.41f,-.73f,.43f,-.49f,-.33f,-.35f,-.36f,.63f,.41f,-.57f,-.39f,.65f,-.03f,-.29f,.06f,.04f,.97f,-.50f,-.54f,.86f,-.41f,.57f,-.12f,.51f,-.41f,.77f,-.89f,-.74f,.45f,.99f,.03f,.96f,.48f,.47f,-.71f,-.85f,.62f,-.83f,-.71f,-.87f,.97f,-.41f,-.20f,-.84f,.55f,-.01f,-.76f,.15f,.55f,.26f,.19f,.70f,.07f,-.56f,-.51f,.48f,-.55f,.14f,.53f,-.53f,-.99f,-.22f,-.61f,-.88f,-.26f,.46f,-.19f,-.22f,.20f,-.02f,.71f,-.82f,-.67f,.80f,.31f,-.58f,-.59f,-.63f,.48f,-.73f,.45f,.49f,.74f,.59f,.72f,-.48f,.18f,-.12f,.30f,.81f,.55f,.97f,-.66f,-.52f,-.99f,.14f,-.62f,-.02f,.90f,.78f,.77f,-.05f,-.06f,.51f,-.26f,.64f,.91f,.66f,.52f,.19f,.21f,-.52f,.96f,.71f,.28f,-.89f,-.01f,.83f,.07f,.39f,-.88f,.28f,.68f,-.81f,-.46f,.42f,.47f,-.43f,-.93f,.43f,-.76f,.15f,.88f,.21f,.30f,-.85f,-.26f,-.12f,-.96f,-.84f,-.97f,-.49f,.64f,.42f,-.50f,.87f,.19f,-.13f,-.46f,-.45f,-.95f,.22f,.41f,.83f,-.14f,-.33f,.87f,.76f,-.06f,-.94f,.72f,.73f,-.78f,.90f,.89f,.84f,.90f,.28f,.88f,.86f,-.62f,.02f,-.81f,-.52f,-.82f,-.14f,-.42f,-.30f,-.02f,.79f,.10f,-.58f,-.00f,.95f,.55f,-.75f,-.09f,.36f,.44f,.30f,.03f,-.49f,-.60f,.95f,.86f,-.67f,-.23f,.28f,-.45f,-.58f,-.52f,.77f,-.19f,-.88f,-.16f,.08f,.82f,-.02f,.45f,-.66f,.73f,.94f,-.55f,-.54f,.02f,.62f,.71f,-.99f,.10f,-.06f,.08f,.79f,-.00f,-.77f,-.47f,.42f,.83f,.21f,.08f,-.43f,-.14f,.22f,.32f,-.03f,.11f,.12f,-.64f,-.58f,-.54f,-.08f,-.38f,.41f,.61f,-.65f,.27f,.98f,-.40f,-.13f,.83f,.74f,-.68f,.22f,-.69f,.42f,.77f,-.87f,-.42f,-.77f,-.11f,.25f,-.69f,.84f,-.25f,-.38f,-.56f,-.20f,-.35f,.83f,-.89f,-.51f,.82f,-.77f,.05f,.78f,.71f,.12f,-.82f,-.32f,.09f,.63f,.70f,-.49f,.80f,.25f,-.85f,.04f,-.72f,.56f,.14f,.62f,.17f,-.88f,-.26f,.20f,.72f,-.55f,-.57f,-.33f,.66f,-.12f,.46f,-.14f,-.87f,-.12f,.08f,.92f,.41f,.36f,-.58f,-.17f,.25f,-.97f,-.87f,-.86f,-.43f,.85f,-.38f,-.45f,.47f,.41f,.98f,-.67f,-.13f,-.36f,.08f,-.33f,.92f,-.96f,-.13f,.71f,.77f,.17f,-.03f,-.92f,-.06f,.00f,.29f,-.14f,-.79f,.64f,.53f,.82f,-.02f,-.39f,.94f,-.97f,-.67f,-.98f,-.38f,-.25f,-.76f,.21f,.80f,.24f,-.69f,-.44f,-.80f,.17f,-.87f,.33f,.72f,-.23f,-.39f,.52f,-.39f,-.14f,-.70f,.86f,-.58f,.05f,.82f,.19f,.58f,-.76f,.00f,.09f,.97f,-.60f,.90f,-.73f,-.47f,-.47f,.18f,.93f,-.53f,.71f,.34f,.48f,-.12f,.94f,.12f,-.44f,-.35f,-.93f,-.00f,-.91f,.40f,.37f,-.87f,-.86f,.68f,.55f,.35f,-.30f,.31f,-.37f,.81f,.89f,-.00f,-.72f,-.24f,-.33f,-.09f,-.79f,.41f,.15f,-.30f,-.99f,-.00f,-.07f,-.55f,.22f,.54f,-.15f,-.45f,-.79f,-.75f,-.50f,-.69f,-.59f,-.67f,.70f,.29f,.74f,.83f,-.83f,.41f,-.04f,-.94f,.59f,-.38f,.85f,.34f,-.06f,.08f,.79f,.41f,-.19f,-.15f,.25f,-.66f,.03f,.57f,-.17f,.69f,.32f,.07f,-.95f,.49f,.42f,-.75f,.70f,-.09f,-.98f,-.93f,.40f,-.38f,.36f,-.49f,.65f,-.30f,.14f,-.99f,.66f,-.27f,.94f,.08f,-.70f,.78f,-.93f,-.97f,-.89f,-.94f,-.77f,-.75f,-.87f,.89f,-.25f,.61f,.59f,-.55f,-.73f,.26f,.93f,-.42f,.66f,-.91f,-.72f,.05f,.26f,-.49f,-.00f,-.88f,-.66f,.34f,.93f,.91f,.94f,.25f,.72f,.13f,-.23f,-.67f,-.99f,-.77f,.45f,.33f,.68f,.34f,.43f,.60f,.36f,-.72f,-.56f,-.41f,-.72f,.65f,.34f,.91f,.95f,-.90f,-.85f,.91f,.80f,-.60f,-.96f,.29f,-.36f,-.54f,.81f,-.95f,-.54f,.67f,.13f,.16f,.73f,.92f,.00f,.08f,-.05f,.84f,-.42f,-.82f,-.31f,-.82f,-.14f,-.65f,-.15f,.55f,.56f,-.93f,-.06f,.59f,.48f,.60f,.08f,-.81f,-.72f,-.69f,-.19f,.02f,.25f,-.38f,-.41f,.31f,-.84f,-.09f,.64f,.62f,-.00f,.78f,-.87f,-.50f,-.85f,.61f,-.90f,.96f,.51f,.99f,-.33f,-.44f,-.23f,-.27f,-.03f,-.75f,.12f,-.78f,-.25f,.44f,.49f,.33f,.33f,-.77f,-.35f,-.39f,-.49f,.00f,-.20f,-.06f,-.78f,.41f,.31f,.62f,-.78f,-.36f,.98f,-.03f,-.41f,-.01f,-.81f,.30f,-.81f,.60f,-.05f,.20f,.70f,.81f,.64f,.78f,.84f,.07f,.86f,.72f,.59f,-.58f,-.85f,.23f,.03f,-.40f,-.29f,-.16f,.59f,.79f,.07f,-.29f,.45f,-.94f,-.48f,-.65f,.57f,.43f,-.31f,-.18f,-.83f,-.85f,-.27f,.28f,.58f,-.07f,.74f,.09f,-.80f,.93f,-.84f,.74f,.37f,.85f,.70f,-.95f,.35f,-.81f,-.54f,.81f,.05f,.02f,.47f,-.70f,.45f,.44f,-.12f,.46f,-.76f,.42f,-.38f,-.47f,-.80f,-.96f,-.97f,-.95f,-.91f,.79f,-.41f,-.87f,.60f,.72f,.19f,.45f,.99f,-.02f,-.70f,-.89f,-.75f,-.54f,.11f,-.69f,.94f,-.86f,-.34f,-.79f,-.75f,.84f,-.56f,.63f,.22f,-.69f,.34f,.34f,.38f,.03f,.43f,-.38f,-.70f,-.34f,-.01f,-.47f,-.88f,.97f,-.99f,.33f,-.79f,-.79f,-.78f,-.14f,-.58f,-.93f,.41f,.14f,.40f,-.06f,.31f,-.69f,-.83f,.29f,.08f,.66f,.71f,.11f,.28f,.98f,.94f,.11f,.91f,.05f,-.07f,.33f,-.97f,.48f,-.77f,.93f,.68f,-.00f,.36f,.37f,-.76f,.51f,.55f,-.92f,.15f,.84f,-.82f,.02f,-.15f,.06f,.05f,-.10f,-.57f,.93f,.03f,-.61f,.47f,-.23f,.39f,-.54f,.33f,-.48f,.45f,.65f,.64f,.35f,.64f,-.39f,.15f,.54f,.97f,-.12f,-.88f,.57f,.95f,.91f,.83f,.64f,-.16f,-.85f,.52f,.03f,.81f,.10f,.47f,.39f,-.66f,.86f,.69f,.29f,-.43f,-.76f,.58f,-.76f,-.89f,-.12f,.37f,.83f,-.02f,.36f,.79f,-.29f,.63f,.18f,-.24f,.62f,-.07f,-.25f,.07f,-.17f,-.60f,-.66f,-.21f,.83f,.04f,-.63f,-.35f,.00f,.22f,.05f,-.18f,.26f,.49f,.51f,.54f,-.43f,.10f,.07f,-.70f,.69f,-.19f,-.24f,.94f,.22f,.16f,-.23f,-.81f,-.72f,.58f,.44f,-.58f,-.89f,.59f,.64f,.72f,.43f,.82f,-.57f,.69f,.27f,-.20f,.24f,.07f,.41f,-.49f,.72f,.27f,-.40f,.11f,-.58f,.78f,-.71f,.97f,.12f,.00f,.76f,.77f,-.70f,-.13f,-.41f,.57f,.42f,.48f,.59f,.47f,.16f,-.10f,.80f,.53f,.50f,-.34f,.78f,-.20f,-.70f,.56f,-.23f,-.11f,-.51f,.44f,.21f,-.59f,.08f,-.43f,-.87f,-.60f,-.35f,.79f,-.62f,-.85f,-.39f,-.39f,-.74f,.80f,-.19f,-.93f,.55f,-.26f,-.86f,-.62f,-.00f,-.20f,-.11f,.95f,-.27f,.59f,.61f,-.78f,-.64f,-.09f,-.51f,.10f,.83f,-.22f,.37f,.40f,-.74f,-.68f,-.22f,-.36f,-.82f,.29f,.29f,-.04f,.88f,-.62f,.49f,.89f,-.87f,.30f,.90f,.54f,.07f,-.52f,-.82f,.65f,.28f,.73f,.43f,.34f,.24f,-.61f,-.20f,-.07f,.85f,.34f,-.33f,.14f,.64f,-.58f,.32f,-.46f,.07f,.88f,.64f,.12f,-.26f,-.11f,.72f,-.58f,-.58f,-.95f,.38f,.11f,-.01f,.15f,-.10f,.50f,.66f,.60f,-.81f,.75f,.48f,.65f,-.83f,-.81f,-.92f,-.31f,.81f,-.35f,.49f,.95f,-.99f,.17f,.34f,.60f,.65f,-.49f,-.18f,-.09f,.35f,.06f,.92f,.78f,-.18f,-.66f,.46f,.58f,-.56f,-.45f,.63f,.03f,.28f,-.84f,.39f,.06f,.85f,.04f,-.50f,.53f,.62f,.46f,-.86f,-.90f,-.72f,.12f,.05f,-.39f,-.41f,.59f,-.19f,-.22f,.50f,.32f,.83f,-.33f,-.83f,-.81f,-.46f,-.63f,-.58f,-.78f,-.49f,-.78f,.19f,.69f,-.66f,.27f,-.10f,-.09f,.88f,.08f,.53f,-.64f,-.38f,.46f,-.64f,-.55f,.33f,-.41f,-.80f,.11f,.37f,-.05f,-.73f,-.57f,.93f,.90f,-.60f,.93f,.81f,.94f,.23f,.06f,.68f,-.06f,-.91f,-.05f,.40f,-.11f,-.83f,-.67f,.96f,-.89f,-.56f,.96f,.82f,-.29f,.80f,.42f,.23f,-.05f,-.06f,.16f,-.54f,.08f,.69f,.97f,.19f,-.65f,.87f,.70f,.67f,.44f,.10f,-.12f,.97f,.07f,.92f,-.00f,-.58f,-.44f,.66f,-.63f,-.32f,.08f,.53f,-.11f,.80f,-.35f,.98f,-.66f,-.63f,-.80f,.87f,-.82f,.09f,.95f,-.25f,-.15f,.72f,-.97f,-.45f,.96f,.49f,.92f,-.19f,-.10f,-.12f,-.00f,-.96f,.76f,.99f,-.37f,.84f,.66f,-.63f,-.41f,.37f,-.29f,.69f,.61f,.54f,-.95f,.97f,.76f,.02f,.06f,.83f,-.94f,-.77f,.04f,-.25f,.57f,.75f,.46f,-.96f,-.49f,.76f,.35f,-.80f,-.74f,.52f,-.57f,-.38f,-.75f,-.88f,.76f,-.69f,.22f,-.64f,.65f,.79f,-.79f,.78f,-.13f,.07f,-.98f,.63f,.94f,-.54f,.52f,.02f,-.64f,-.05f,.04f,-.91f,-.05f,-.54f,.67f,.44f,-.43f,-.37f,.03f,.37f,-.51f,-.36f,.21f,.65f,-.62f,.20f,-.90f,-.37f,-.11f,-.40f,-.11f,.50f,.81f,.46f,.03f,-.09f,.05f,-.20f,.98f,.45f,.26f,-.15f,-.40f,.70f,-.17f,-.59f,-.23f,-.80f,.65f,-.47f,.39f,.68f,-.64f,.15f,-.89f,.98f,-.36f,.15f,-.59f,.88f,.47f,-.01f,.26f,-.94f,.39f,-.76f,-.43f,-.29f,-.88f,-.52f,-.42f,.12f,-.56f,.09f,.38f,.32f,-.22f,-.29f,-.46f,.59f,-.96f,.30f,.95f,-.76f,-.90f,-.76f,.56f,.35f,-.76f,.42f,-.72f,-.67f,.74f,-.60f,.90f,.74f,-.87f,-.36f,-.12f,.29f,-.83f,.60f,.35f,.44f,-.50f,.19f,.19f,.42f,-.47f,-.50f,-.35f,.49f,.41f,-.19f,-.08f,-.70f,.76f,-.44f,.69f,.16f,-.01f,.56f,-.85f,-.11f,-.87f,.11f,.17f,-.05f,.50f,.24f,-.47f,.57f,.59f,-.90f,.95f,-.05f,.02f,.51f,.82f,.28f,.91f,-.59f,.65f,-.42f,-.82f,.84f,-.31f,-.41f,.75f,.68f,-.67f,.52f,-.18f,.79f,-.38f,.63f,.33f,-.93f,-.59f,.76f,-.60f,.03f,-.77f,-.34f,.17f,-.35f,.81f,.50f,.82f,.57f,.61f,-.38f,.69f,.69f,.75f,-.61f,-.73f,.53f,.66f,-.83f,-.69f,.67f,-.41f,.84f,-.23f,.70f,.95f,.66f,.81f,.64f,-.55f,-.80f,.45f,-.11f,-.77f,-.47f,-.36f,-.41f,-.82f,-.60f,-.32f,-.58f,.49f,.32f,-.38f,.35f,-.88f,-.11f,.49f,.24f,.97f,.68f,.90f,.62f,-.40f,.94f,-.16f,-.57f,-.00f,-.18f,.55f,-.88f,.95f,-.49f,-.75f,.54f,.45f,.03f,.59f,.17f,-.22f,-.44f,-.63f,.69f,.08f,.06f,-.43f,.00f,-.50f,-.86f,-.48f,.91f,-.19f,-.18f,-.61f,.85f,-.41f,-.73f,.83f,-.86f,.66f,.94f,-.02f,-.07f,.21f,-.68f,-.33f,-.40f,.13f,-.11f,.59f,-.91f,-.28f,.72f,.18f,.46f,-.20f,-.36f,-.93f,-.38f,.91f,-.03f,-.85f,-.05f,.23f,-.03f,-.28f,.68f,.93f,.07f,.69f,-.76f,-.55f,.42f,-.27f,-.28f,.39f,-.02f,.95f,-.43f,.38f,.70f,.93f,-.55f,-.25f,-.72f,-.44f,.97f,-.72f,-.24f,-.66f,-.99f,-.29f,-.93f,.97f,.64f,.65f,.95f,.58f,.13f,.18f,.42f,-.38f,.11f,.60f,.30f,-.01f,.79f,-.31f,-.70f,.23f,-.17f,-.58f,-.82f,.13f,-.75f,-.44f,.99f,.26f,-.55f,.12f,-.40f,.85f,-.93f,.28f,-.78f,-.94f,.99f,-.71f,.55f,-.30f,.20f,.33f,-.84f,-.93f,-.49f,-.83f,-.30f,-.44f,-.14f,.96f,-.71f,-.68f,.60f,.74f,-.86f,-.10f,-.04f,.07f,-.30f,.28f,-.10f,.15f,-.34f,.94f,.71f,.17f,-.48f,-.53f,.93f,.73f,.29f,-.17f,-.34f,-.61f,-.93f,.94f,-.14f,.27f,.49f,.78f,-.69f,.36f,.38f,.49f,-.93f,.83f,-.85f,.51f,.43f,.61f,.40f,-.95f,-.81f,.09f,-.87f,-.42f,-.85f,.34f,-.43f,.72f,-.40f,-.51f,-.39f,-.50f,-.74f,.91f,.49f,.92f,.84f,.68f,-.63f,-.77f,.25f,.63f,-.60f,.32f,-.94f,.69f,-.81f,.55f,.07f,.58f,-.76f,-.82f,.53f,-.87f,-.97f,-.92f,-.19f,.12f,-.70f,.05f,.99f,-.88f,-.76f,-.81f,-.81f,.94f,-.22f,.44f,.10f,-.09f,.34f,-.72f,-.19f,-.44f,.29f,.74f,.45f,.77f,.36f,.37f,-.59f,.46f,.41f,-.86f,-.10f,-.42f,.66f,.11f,.50f,.17f,.78f,.92f,.93f,.64f,.61f,.53f,-.51f,.75f,-.89f,-.01f,.50f,-.27f,-.40f,.06f,.66f,-.02f,-.70f,-.13f,.03f,-.72f,-.64f,.67f,.12f,-.97f,.97f,-.74f,-.56f,-.11f,.81f,-.13f,-.32f,.06f,.10f,.77f,-.80f,-.68f,-.14f,-.53f,-.70f,-.18f,-.37f,-.61f,.12f,.65f,.40f,-.16f,-.92f,.92f,.94f,-.20f,-.24f,.33f,-.75f,.83f,.67f,-.10f,-.18f,.84f,-.23f,-.80f,.77f,-.80f,.60f,-.51f,.64f,-.63f,.89f,-.61f,.00f,.43f,.86f,.23f,-.51f,.05f,-.23f,.05f,.32f,.36f,.78f,.12f,-.52f,.52f,-.86f,-.37f,-.63f,.50f,-.82f,-.93f,.49f,-.20f,-.35f,.20f,-.03f,.09f,.01f,.26f,.14f,.82f,.15f,.26f,-.83f,-.22f,.36f,.26f,-.40f,.90f,.87f,-.96f,.87f,.73f,.67f,.08f,.46f,.59f,.59f,.82f,.99f,-.13f,.18f,-.04f,.60f,-.60f,.84f,-.80f,-.54f,.40f,-.57f,-.43f,-.08f,-.09f,.62f,.58f,.89f,-.38f,.07f,-.72f,-.15f,.20f,.34f,-.81f,-.04f,-.06f,.77f,-.95f,-.99f,-.05f,-.13f,-.37f,-.21f,-.31f,-.77f,-.82f,.67f,.71f,.03f,.98f,.46f,-.87f,.55f,-.14f,.14f,.26f,.05f,.31f,.67f,-.00f,-.49f,-.71f,-.75f,-.66f,.27f,-.29f,.04f,-.91f,.02f,-.68f,-.03f,-.47f,.34f,-.89f,-.98f,-.00f,.11f,-.52f,.28f,-.42f,.60f,.28f,.42f,.91f,-.09f,.23f,-.44f,-.39f,-.20f,.61f,.82f,-.43f,.57f,.15f,-.03f,-.71f,-.19f,.56f,.54f,.99f,-.59f,-.76f,-.90f,.05f,.00f,-.56f,-.85f,-.74f,.31f,-.92f,.38f,.13f,.44f,.94f,.95f,.35f,.18f,.35f,-.16f,.15f,.21f,-.90f,.80f,.50f,-.68f,-.20f,-.99f,.20f,.71f,.65f,.46f,.21f,.19f,-.64f,.79f,-.23f,.69f,-.71f,.29f,-.55f,.61f,-.31f,.18f,-.00f,.40f,-.89f,-.38f,-.08f,-.43f,-.87f,.53f,-.05f,.55f,.80f,-.60f,-.87f,-.15f,-.22f,-.55f,-.41f,-.43f,.69f,.32f,-.10f,-.29f,.26f,.88f,-.86f,-.16f,-.36f,-.94f,.47f,-.75f,-.15f,-.10f,.97f,.93f,.39f,.29f,.12f,.49f,-.38f,.22f,.24f,-.33f,.18f,.37f,-.80f,-.77f,.28f,-.26f,.05f,.60f,-.85f,-.14f,.90f,-.44f,.36f,-.85f,-.89f,.36f,-.93f,.44f,-.95f,-.18f,.62f,-.50f,-.58f,-.08f,.84f,.33f,-.96f,.92f,.41f,-.61f,.90f,.09f,-.70f,-.73f,-.64f,-.09f,-.09f,-.42f,-.90f,.61f,.71f,.82f,-.42f,.56f,.97f,.64f,-.49f,-.22f,-.94f,.24f,-.29f,-.88f,-.54f,.97f,-.76f,.66f,-.71f,-.64f,-.64f,-.53f,.20f,.23f,-.56f,-.62f,-.55f,.18f,.18f,-.77f,.19f,-.68f,.89f,.50f,.42f,-.10f,-.28f,.41f,-.39f,-.51f,.70f,-.98f,.44f,.04f,-.46f,-.63f,-.93f,-.31f,.41f,.35f,.53f,.77f,.60f,.26f,-.21f,.82f,-.92f,.92f,-.70f,-.10f,.64f,-.83f,-.55f,.61f,.77f,-.72f,-.30f,-.85f,.33f,-.60f,.19f,.86f,.70f,.77f,.21f,.86f,.04f,.22f,.95f,.57f,.36f,-.41f,.37f,-.17f,.29f,.70f,-.40f,-.39f,-.83f,-.11f,.84f,-.43f,-.48f,.13f,-.27f,-.75f,-.54f,.79f,.75f,-.85f,.90f,-.41f,.24f,.29f,.61f,.69f,-.03f,-.72f,-.36f,-.33f,-.32f,-.11f,.19f,-.84f,-.24f,-.77f,-.96f,.70f,.04f,.95f,-.97f,-.11f,-.24f,.69f,-.13f,.85f,.43f,-.41f,-.89f,.70f,.09f,-.63f,.57f,.80f,.56f,-.62f,.32f,-.28f,.54f,.14f,-.67f,-.24f,-.79f,.23f,-.95f,-.16f,-.85f,-.18f,.46f,-.60f,.36f,.22f,-.08f,.15f,.62f,-.20f,.88f,-.26f,-.13f,-.11f,.56f,.17f,.67f,-.56f,.49f,.87f,-.35f,-.73f,-.93f,-.80f,-.06f,-.95f,-.36f,-.26f,.34f,.19f,.57f,.56f,-.65f,.60f,.94f,.86f,-.16f,-.95f,.36f,.12f,-.42f,.34f,.75f,-.14f,.15f,.03f,-.17f,.70f,.95f,.95f,.79f,.11f,-.82f,-.30f,.75f,-.72f,-.91f,.31f,-.07f,.42f,.85f,.00f,-.47f,.37f,-.62f,.22f,.10f,.75f,-.15f,-.13f,.18f,-.21f,.45f,-.35f,.77f,-.40f,-.75f,.59f,.87f,-.62f,-.91f,.55f,-.09f,.06f,.30f,.52f,-.51f,-.03f,.89f,-.05f,.88f,.83f,-.66f,-.01f,.34f,-.83f,.60f,.47f,.65f,.51f,.70f,.62f,.99f,.34f,.41f,-.31f,.75f,.61f,.04f,.54f,-.42f,.47f,-.43f,.39f,-.58f,-.44f,-.02f,-.08f,.38f,-.77f,-.55f,-.20f,.63f,-.93f,-.96f,-.04f,-.26f,.83f,.34f,.31f,-.32f,.37f,-.74f,.60f,-.77f,-.04f,.65f,.94f,-.39f,.01f,-.76f,-.25f,.44f,.87f,.06f,-.58f,-.21f,.67f,-.97f,.22f,.24f,-.16f,.82f,-.19f,.66f,.00f,.50f,-.85f,.32f,-.66f,.11f,-.48f,.66f,-.60f,-.50f,.29f,-.93f,.94f,-.02f,.41f,-.13f,-.04f,.77f,.31f,-.23f,.84f,.25f,-.48f,-.51f,.19f,-.42f,-.03f,-.13f,.78f,.76f,.23f,.63f,.23f,-.82f,.41f,.10f,.73f,.49f,-.94f,.80f,-.40f,.78f,-.12f,-.49f,-.80f,.37f,.96f,.22f,-.93f,.78f,-.74f,-.71f,.46f,-.38f,.86f,.03f,.48f,.22f,-.05f,.13f,.91f,-.20f,.36f,.64f,.26f,.38f,-.10f,.57f,-.14f,-.72f,.08f,.55f,-.91f,.62f,.27f,-.49f,-.22f,.22f,-.12f,-.32f,-.28f,.15f,-.92f,.58f,-.51f,-.07f,-.45f,-.24f,-.73f,.63f,-.70f,-.11f,-.46f,-.75f,-.58f,-.15f,.38f,-.22f,.23f,-.26f,.41f,.14f,.87f,.44f,.26f,.79f,.28f,.19f,.06f,.84f,.19f,.91f,-.25f,.24f,.07f,-.46f,-.57f,.86f,.98f,-.33f,.77f,.04f,.55f,-.04f,-.72f,-.44f,.36f,-.73f,.74f,-.86f,.71f,.89f,-.94f,-.03f,.07f,.12f,-.14f,-.20f,-.71f,-.51f,-.23f,-.35f,-.89f,.75f,-.95f,-.48f,-.34f,-.16f,.45f,.34f,-.50f,-.82f,.12f,.18f,.35f,-.26f,-.16f,-.98f,.80f,.58f,-.65f,-.17f,.77f,.80f,-.06f,-.19f,-.43f,-.88f,.08f,.23f,-.70f,-.69f,-.76f,-.11f,.44f,-.87f,.70f,-.73f,-.55f,.05f,.30f,-.02f,.19f,.20f,.14f,.08f,.16f,.44f,.58f,-.51f,-.63f,.32f,.98f,-.48f,-.66f,-.00f,-.83f,.39f,-.95f,-.32f,-.28f,.60f,.59f,-.94f,.34f,-.68f,-.62f,.46f,-.87f,.25f,-.00f,.12f,.38f,.69f,-.54f,.11f,.03f,-.76f,.92f,.52f,-.33f,-.36f,-.33f,.46f,-.60f,.33f,-.56f,-.77f,.13f,.31f,-.65f,-.67f,-.58f,.54f,.41f,.80f,-.92f,.77f,-.44f,.69f,-.60f,.13f,-.82f,.13f,.27f,-.36f,-.88f,-.77f,-.72f,-.94f,.69f,.18f,-.29f,-.12f,-.81f,-.51f,.72f,-.03f,.32f,-.25f,-.83f,.11f,.14f,-.76f,-.62f,.51f,.87f,.76f,-.23f,.52f,.89f,-.02f,-.57f,.81f,-.83f,-.10f,-.88f,.71f,.30f,-.66f,-.38f,-.56f,.37f,.02f,.08f,-.88f,-.86f,.01f,-.61f,-.44f,.60f,.34f,-.90f,-.50f,-.11f,.52f,-.76f,.04f,-.53f,-.10f,.03f,.07f,.32f,.98f,.28f,.13f,.05f,.62f,-.15f,-.39f,.04f,-.81f,.84f,-.48f,.56f,-.74f,.39f,.79f,-.40f,-.67f,-.52f,.67f,-.68f,-.55f,-.65f,.36f,.27f,-.14f,-.14f,-.58f,.34f,.14f,-.33f,-.67f,.92f,-.73f,-.29f,.96f,-.43f,-.85f,.96f,.65f,.21f,.76f,-.74f,.42f,-.78f,-.06f,.21f,-.03f,-.83f,-.42f,.44f,-.79f,-.55f,-.22f,-.54f,-.24f,.68f,.03f,-.90f,.81f,.68f,.70f,-.84f,-.41f,-.26f,-.67f,.21f,-.49f,.00f,-.99f,.73f,.96f,-.50f,.82f,.83f,-.41f,.83f,-.48f,-.20f,.84f,-.79f,-.93f,.89f,-.24f,.59f,-.10f,.95f,.43f,.44f,-.81f,.61f,-.49f,-.78f,-.97f,.83f,-.33f,.26f,.46f,-.74f,-.64f,.49f,.38f,.92f,.70f,.51f,-.64f,.57f,.55f,.82f,-.91f,-.75f,-.50f,-.39f,-.79f,-.96f,-.72f,-.96f,.78f,-.81f,.07f,-.08f,-.34f,-.90f,.26f,.88f,-.69f,-.64f,-.41f,.46f,.93f,-.60f,-.81f,-.11f,-.50f,-.30f,-.27f,.59f,-.02f,-.71f,-.78f,.72f,.11f,-.84f,-.84f,.52f,.45f,-.90f,-.64f,-.38f,.32f,-.20f,.20f,.33f,-.61f,.79f,.33f,-.75f,-.21f,-.04f,.65f,-.27f,.59f,.60f,-.49f,-.38f,.72f,.87f,-.36f,-.97f,.13f,-.62f,-.86f,-.91f,-.52f,.45f,-.41f,-.38f,-.29f,-.07f,-.69f,-.86f,.46f,.60f,.37f,.14f,-.12f,-.75f,.64f,-.74f,.00f,-.91f,-.44f,-.75f,.94f,.91f,-.12f,.76f,.25f,-.75f,-.42f,-.71f,.41f,.55f,-.99f,.12f,.83f,.98f,-.45f,.16f,-.89f,-.38f,-.36f,-.89f,.36f,.27f,.88f,.64f,.31f,-.51f,.48f,-.31f,-.94f,.83f,-.51f,.51f,-.32f,.45f,-.34f,.62f,-.88f,.37f,.48f,-.25f,-.87f,-.40f,-.35f,-.60f,-.48f,-.62f,-.14f,.16f,-.60f,-.95f,.70f,.57f,.38f,-.42f,-.76f,-.37f,-.13f,.67f,.46f,-.54f,.62f,.87f,.93f,-.72f,.56f,.83f,-.24f,-.34f,-.52f,.65f,-.64f,.10f,-.02f,-.63f,-.59f,-.90f,-.98f,.28f,.52f,-.41f,.27f,-.97f,.53f,.29f,-.58f,-.44f,.87f,.32f,-.02f,-.49f,.33f,.61f,.93f,-.95f,.65f,.37f,-.42f,-.82f,.95f,-.96f,.72f,-.14f,-.07f,.03f,.20f,.87f,.52f,-.40f,-.72f,-.95f,.49f,-.29f,.17f,.93f,.04f,-.55f,.37f,.92f,-.99f,.94f,.24f,-.90f,-.60f,-.87f,.48f,.19f,-.27f,.12f,-.50f,-.97f,.85f,-.02f,.68f,.03f,.30f,-.53f,.66f,.38f,.72f,.12f,-.62f,-.46f,-.61f,-.89f,.24f,-.80f,.13f,-.52f,.45f,-.02f,.13f,.03f,.02f,.22f,.50f,.70f,.46f,.40f,-.00f,.49f,-.96f,.75f,-.31f,.60f,.06f,-.57f,.20f,-.79f,-.70f,.08f,.71f,.89f,.75f,-.65f,.50f,-.89f,.15f,.05f,.95f,.77f,-.03f,-.52f,.52f,.60f,.00f,-.92f,.34f,.38f,-.69f,.26f,-.74f,.29f,.39f,.18f,-.06f,.19f,-.67f,-.79f,.96f,-.63f,-.55f,-.86f,-.29f,.68f,-.57f,-.10f,-.87f,.22f,.70f,.09f,.17f,.27f,-.30f,-.53f,.50f,.80f,.07f,.09f,-.85f,.07f,.52f,-.71f,-.71f,.13f,-.57f,.39f,-.45f,.45f,.93f,-.82f,.09f,.94f,-.77f,-.58f,-.46f,-.40f,.47f,-.85f,-.92f,-.01f,-.35f,.33f,-.74f,.79f,.10f,-.24f,.07f,.97f,-.67f,-.69f,-.64f,-.31f,-.19f,.39f,-.73f,.18f,.10f,.14f,-.97f,-.37f,-.36f,-.33f,.64f,-.16f,-.60f,-.78f,.84f,-.77f,-.40f,.68f,-.49f,.13f,.39f,.98f,-.35f,.05f,-.97f,-.12f,.93f,.88f,.33f,.02f,.17f,.80f,-.71f,.98f,.92f,-.97f,.75f,.48f,.22f,.74f,-.51f,.67f,-.87f,.44f,.84f,.28f,-.04f,.41f,.33f,-.68f,.52f,.55f,-.81f,-.63f,.57f,-.48f,-.29f,-.51f,.76f,-.78f,-.39f,-.15f,-.24f,-.01f,-.37f,-.87f,.64f,.07f,.19f,-.96f,.77f,.48f,.95f,.27f,-.55f,-.47f,.00f,-.99f,-.89f,-.31f,.05f,-.08f,-.56f,.30f,.41f,-.63f,.44f,.80f,.11f,.14f,-.62f,-.41f,-.94f,-.64f,.44f,-.49f,-.89f,.42f,-.82f,-.97f,-.83f,-.41f,.68f,.89f,.40f,.09f,.28f,.09f,.01f,.32f,.33f,-.31f,.29f,-.57f,.26f,.70f,.28f,.34f,.12f,-.39f,.77f,.83f,.30f,.52f,-.97f,.80f,-.94f,-.53f,-.33f,-.38f,-.25f,-.20f,-.22f,.37f,-.64f,.95f,-.26f,-.46f,-.97f,-.86f,-.73f,-.19f,.62f,.45f,.33f,-.14f,.87f,-.41f,.01f,-.67f,-.14f,.59f,-.29f,.20f,-.33f,.19f,.50f,.68f,.60f,.21f,.91f,.29f,-.77f,.48f,.54f,-.72f,.26f,.94f,-.82f,.26f,.08f,-.69f,.20f,.66f,.65f,-.77f,.22f,-.17f,-.31f,.56f,.86f,.53f,-.06f,.73f,-.52f,.27f,.99f,-.73f,-.31f,-.09f,.74f,-.15f,-.14f,-.70f,.86f,-.83f,-.02f,-.18f,-.53f,.53f,-.23f,.94f,-.23f,-.32f,.23f,-.73f,-.43f,.06f,-.75f,-.88f,.43f,.50f,-.74f,-.17f,.03f,.74f,-.18f,-.97f,-.09f,.88f,.63f,-.88f,-.42f,.03f,-.19f,.54f,-.26f,.30f,-.13f,.74f,.36f,.05f,-.31f,.77f,-.24f,-.95f,-.52f,.95f,.23f,.87f,-.09f,.08f,.24f,.16f,.31f,.88f,.48f,-.12f,.30f,-.42f,-.10f,.44f,-.91f,-.43f,-.95f,.62f,.36f,.91f,-.74f,.74f,-.08f,.55f,.12f,-.14f,.82f,.22f,.77f,-.11f,.81f,.29f,-.52f,-.47f,.75f,-.08f,.23f,.45f,.87f,.48f,-.41f,.09f,.46f,.71f,.73f,.23f,-.87f,-.90f,-.38f,-.72f,.23f,.36f,.64f,-.87f,-.82f,-.97f,.96f,.97f,-.63f,-.92f,.73f,.29f,-.54f,-.20f,-.96f,-.15f,.98f,-.50f,-.42f,.29f,-.21f,.71f,-.71f,.58f,.12f,-.61f,-.37f,-.28f,-.00f,.66f,-.80f,-.29f,-.83f,-.15f,.70f,.27f,-.37f,.02f,-.28f,.85f,.14f,.26f,.06f,-.72f,.07f,.33f,.18f,-.56f,.66f,.51f,-.21f,.41f,-.69f,-.38f,.42f,-.04f,.71f,.66f,-.26f,.28f,.74f,.61f,.22f,.47f,-.44f,.49f,-.26f,.61f,-.24f,.07f,-.63f,-.24f,-.44f,-.41f,.74f,-.27f,.60f,.47f,-.69f,.13f,-.10f,.36f,-.54f,-.05f,-.26f,.50f,-.62f,.19f,-.05f,.56f,-.12f,.33f,.60f,-.50f,-.40f,.12f,-.06f,.95f,.06f,.60f,.00f,.32f,-.73f,-.05f,.42f,.24f,-.51f,.37f,-.86f,.49f,-.35f,.07f,.21f,-.03f,.13f,-.03f,-.19f,.48f,-.29f,-.56f,-.35f,.57f,-.98f,-.21f,.50f,.95f,.51f,-.40f,.77f,.26f,.77f,.71f,-.28f,.98f,-.78f,.55f,-.96f,-.30f,-.06f,.72f,.79f,.32f,.46f,-.42f,.38f,-.22f,.82f,.65f,.63f,-.92f,-.99f,.07f,-.28f,-.48f,-.22f,.29f,.61f,.04f,.76f,.77f,-.89f,-.84f,-.18f,.27f,-.14f,.13f,.75f,-.53f,.88f,-.46f,.22f,-.57f,-.19f,-.11f,-.96f,.16f,.69f,.37f,.06f,.59f,.21f,-.74f,.53f,.74f,.68f,-.60f,.81f,-.74f,-.82f,-.84f,-.64f,-.09f,.31f,.54f,.34f,.86f,.30f,-.72f,.25f,.51f,.62f,.47f,-.67f,.33f,-.68f,.63f,.84f,.91f,-.85f,-.24f,-.48f,-.40f,.29f,.24f,-.76f,.27f,.60f,-.51f,-.27f,-.99f,.58f,.87f,.58f,-.42f,.18f,.97f,-.42f,-.96f,-.30f,-.98f,-.05f,.98f,.35f,-.82f,-.55f,.87f,.13f,.13f,-.14f,-.61f,-.75f,-.58f,.24f,-.33f,-.79f,.95f,.19f,.01f,.54f,-.40f,-.60f,.08f,-.59f,.71f,-.98f,-.94f,.73f,.46f,-.54f,.23f,.49f,-.26f,.26f,.63f,-.32f,-.34f,-.14f,.01f,.68f,-.03f,.53f,.40f,-.90f,.70f,-.13f,.31f,.12f,-.57f,.52f,-.55f,.98f,-.71f,-.75f,-.71f,-.23f,.54f,.19f,.78f,-.41f,-.81f,-.34f,-.03f,-.78f,-.71f,-.98f,.69f,.08f,.16f,.64f,-.23f,-.71f,.35f,-.53f,-.45f,-.01f,-.99f,.38f,-.20f,-.62f,.56f,.82f,.98f,.26f,.00f,.36f,.29f,-.78f,-.88f,.46f,.00f,.13f,-.22f,.73f,-.42f,.67f,-.75f,.76f,.72f,.95f,.46f,-.81f,-.98f,-.38f,-.05f,-.02f,.77f,-.53f,-.53f,-.68f,-.98f,.07f,-.08f,.40f,-.59f,-.03f,-.13f,.70f,.07f,-.82f,.06f,-.09f,.66f,.15f,.73f,.39f,-.84f,-.33f,-.48f,-.81f,-.01f,-.43f,.42f,.55f,.31f,-.78f,-.15f,-.04f,.81f,.26f,-.38f,.00f,.54f,.21f,-.60f,-.75f,.92f,-.60f,-.51f,.95f,-.37f,-.81f,.44f,.68f,.71f,-.37f,.61f,.29f,-.27f,-.96f,-.14f,-.06f,-.74f,-.09f,.51f,-.89f,.47f,-.75f,.40f,.50f,.45f,-.68f,.77f,.31f,-.50f,-.45f,.52f,.97f,.05f,-.91f,.61f,-.36f,-.26f,-.36f,.47f,-.34f,.97f,-.45f,.19f,-.08f,.39f,.35f,-.28f,.81f,-.73f,.37f,-.39f,.15f,-.79f,-.73f,.37f,-.65f,.72f,-.47f,.57f,-.57f,-.82f,-.87f,-.62f,-.92f,.92f,.85f,-.32f,.04f,-.79f,.96f,.77f,.13f,-.19f,.44f,-.10f,-.65f,-.49f,.33f,.72f,.96f,.13f,-.66f,-.78f,.76f,-.44f,-.69f,.87f,.67f,-.34f,.87f,.92f,.02f,-.28f,-.21f,-.56f,.36f,.96f,.53f,-.19f,-.15f,-.73f,.39f,.53f,-.78f,.26f,-.56f,.17f,.69f,.20f,-.08f,.45f,.19f,.51f,-.96f,-.42f,.59f,.16f,.48f,-.75f,-.71f,.11f,-.24f,-.31f,-.67f,-.96f,-.05f,-.53f,.62f,.81f,.37f,.07f,.80f,.40f,-.38f,-.84f,-.57f,.96f,-.23f,.62f,-.88f,.49f,.49f,.37f,.82f,-.83f,-.00f,.93f,-.25f,.13f,-.69f,.87f,.94f,.71f,.58f,.51f,.97f,-.44f,-.76f,.48f,.01f,.11f,-.57f,.53f,-.85f,-.45f,.31f,-.02f,-.27f,.39f,.86f,-.42f,-.67f,-.34f,.39f,.42f,-.08f,.25f,.01f,-.45f,-.11f,.65f,-.89f,-.44f,.54f,.66f,.53f,-.96f,-.09f,.57f,.20f,-.16f,.90f,-.02f,.92f,.67f,-.06f,.51f,-.54f,-.79f,.08f,.89f,-.96f,.77f,-.83f,-.72f,.95f,.06f,.38f,.80f,.20f,.17f,-.30f,.76f,-.39f,-.81f,-.59f,-.66f,.35f,.01f,-.53f,.07f,.94f,.88f,.97f,-.18f,.14f,.41f,-.83f,-.00f,.59f,-.05f,.03f,.84f,-.17f,-.32f,-.79f,-.86f,-.55f,-.09f,.57f,.01f,.80f,-.11f,.68f,.92f,-.57f,.60f,-.07f,-.10f,.66f,.47f,.62f,.11f,-.50f,.01f,-.03f,.84f,-.15f,-.45f,-.07f,.19f,.94f,-.10f,-.46f,-.47f,-.78f,.05f,.70f,-.96f,.88f,-.84f,.72f,-.40f,.46f,.44f,.31f,.10f,.36f,.42f,.42f,-.29f,-.87f,.16f,-.24f,.62f,.05f,.13f,-.11f,.80f,-.23f,.18f,.43f,-.91f,.75f,.38f,.84f,.61f,-.83f,-.34f,-.01f,.05f,-.39f,.05f,-.64f,.62f,-.29f,.56f,-.06f,.23f,-.39f,.18f,.44f,.04f,.36f,.89f,.93f,.24f,.42f,.25f,-.45f,-.63f,-.22f,.09f,-.27f,.27f,.28f,-.12f,-.97f,-.20f,.55f,-.43f,-.35f,-.64f,.63f,.42f,-.93f,-.76f,-.98f,.90f,.80f,-.64f,-.92f,.34f,-.81f,.79f,-.77f,.56f,.60f,-.05f,-.63f,-.32f,.51f,-.52f,-.20f,.38f,.80f,-.63f,-.52f,-.13f,-.06f,.01f,-.66f,.14f,-.45f,.78f,.07f,.00f,.17f,-.46f,.47f,-.10f,.31f,-.16f,-.62f,-.32f,-.85f,.24f,-.01f,-.63f,.80f,.92f,.54f,-.85f,-.74f,-.93f,-.98f,.54f,-.23f,-.36f,.20f,.75f,.22f,.71f,-.07f,-.13f,.40f,-.82f,.46f,-.39f,.34f,-.35f,.12f,-.42f,.63f,.68f,.04f,.97f,-.04f,-.38f,.87f,.12f,.02f,-.71f,-.81f,-.43f,.56f,.80f,.88f,.51f,-.78f,-.52f,.35f,-.52f,.21f,-.51f,.19f,-.05f,.64f,.72f,-.69f,.45f,-.94f,.41f,-.80f,-.17f,.67f,-.41f,.08f,.15f,-.55f,.13f,-.25f,-.85f,.77f,.47f,-.44f,.39f,-.71f,-.62f,.65f,.35f,.20f,.25f,.62f,-.53f,-.97f,-.36f,-.12f,-.93f,-.46f,.39f,.07f,.06f,-.88f,.07f,.01f,-.95f,-.02f,.52f,.41f,-.88f,.55f,.99f,.11f,.76f,-.90f,.97f,-.25f,-.92f,.65f,.94f,.71f,-.82f,-.46f,.44f,.21f,.84f,-.49f,.55f,-.20f,.45f,.48f,-.58f,.96f,.52f,.01f,.31f,-.67f,.52f,-.66f,-.77f,.18f,.36f,-.32f,.51f,.23f,.58f,-.45f,-.52f,.22f,-.46f,.79f,.49f,.40f,.81f,-.44f,.79f,-.50f,.90f,.13f,.34f,.93f,.31f,.56f,-.83f,.90f,-.18f,.55f,-.05f,-.82f,.16f,.67f,.08f,-.82f,-.04f,-.05f,.69f,.75f,.74f,-.37f,.09f,.82f,-.32f,.16f,.67f,.50f,.37f,.63f,-.65f,.28f,.34f,.09f,.41f,-.64f,.57f,.80f,-.11f,-.43f,-.16f,-.29f,-.79f,-.52f,-.54f,-.22f,-.94f,-.77f,.36f,.75f,-.42f,.71f,-.53f,-.23f,-.26f,-.23f,-.19f,-.13f,.85f,-.15f,.70f,-.27f,-.43f,.63f,.13f,.95f,-.63f,-.28f,-.42f,-.91f,-.42f,.79f,-.05f,-.43f,.24f,.39f,.22f,.63f,-.40f,.12f,.10f,-.23f,-.86f,-.12f,-.42f,.31f,-.87f,-.38f,.04f,.54f,.37f,-.51f,.09f,.58f,.27f,.27f,.17f,-.16f,.36f,.35f,.88f,-.79f,.34f,.34f,-.51f,-.15f,-.84f,.07f,.46f,.46f,.44f,-.70f,-.41f,.98f,.44f,.66f,.31f,.84f,.95f,-.25f,-.52f,-.07f,-.78f,-.72f,-.55f,-.84f,-.38f,-.77f,.90f,-.08f,-.62f,.74f,-.78f,-.46f,-.71f,.80f,.21f,.66f,.19f,.80f,.05f,.83f,.28f,.46f,.75f,.33f,.31f,-.62f,-.20f,.13f,-.79f,-.17f,.32f,-.26f,-.48f,.40f,.93f,-.08f,.49f,.71f,.34f,-.05f,.62f,.30f,.22f,.12f,.60f,-.02f,-.56f,.39f,-.15f,.78f,.80f,-.11f,-.61f,-.66f,-.25f,-.80f,-.44f,-.39f,-.99f,.05f,.57f,-.36f,.82f,-.74f,.37f,-.50f,.68f,-.37f,.56f,.80f,-.50f,.45f,-.50f,.58f,-.01f,.79f,-.48f,.05f,.97f,.32f,.82f,.46f,.49f,-.12f,-.71f,-.04f,-.65f,.60f,-.81f,.78f,-.85f,.90f,-.95f,-.46f,.62f,.36f,-.43f,.10f,.57f,.52f,-.60f,-.05f,-.11f,.12f,.32f,.52f,.91f,.53f,.24f,.04f,-.44f,-.41f,.89f,-.17f,-.45f,-.83f,.28f,-.70f,.46f,.58f,.81f,-.23f,-.97f,-.61f,-.82f,.83f,.45f,.74f,-.55f,-.51f,-.74f,.56f,.00f,.23f,.76f,-.30f,.07f,.91f,.26f,.82f,.32f,-.40f,.82f,.64f,-.83f,-.99f,-.19f,.34f,.09f,.31f,-.47f,.38f,.33f,.68f,.71f,-.20f,-.86f,-.10f,-.53f,-.04f,-.99f,.35f,.49f,.52f,.81f,-.51f,.76f,-.50f,-.31f,-.93f,-.89f,-.88f,.97f,.22f,-.41f,.25f,-.28f,-.29f,-.60f,-.23f,-.75f,-.39f,.81f,.17f,.18f,-.92f,-.94f,.82f,-.83f,-.71f,.77f,-.65f,.15f,.17f,.09f,.23f,.21f,.05f,.14f,-.56f,-.31f,.32f,.45f,-.41f,.40f,-.01f,-.59f,-.07f,.96f,.15f,.98f,-.32f,-.66f,.58f,-.07f,.01f,-.67f,-.17f,-.37f,.76f,-.57f,.18f,-.97f,-.14f,.78f,.13f,.78f,-.58f,.33f,.45f,-.31f,.29f,.50f,-.20f,-.78f,-.82f,-.55f,.46f,.55f,-.73f,-.63f,.01f,.12f,.62f,.58f,.95f,.08f,-.94f,-.90f,-.70f,-.22f,.57f,.69f,.80f,.52f,-.38f,-.38f,.59f,.17f,.53f,.46f,-.55f,.81f,.59f,.54f,.57f,-.38f,.07f,-.23f,.16f,.18f,.57f,-.94f,.95f,-.83f,.75f,-.75f,-.97f,-.57f,-.61f,-.15f,-.12f,-.67f,.72f,-.90f,-.26f,.39f,.65f,-.34f,-.43f,.13f,-.23f,-.22f,-.80f,.60f,-.54f,.31f,-.98f,-.12f,.48f,.94f,-.96f,.42f,-.33f,-.89f,-.03f,.94f,.04f,.36f,-.75f,.28f,.28f,-.77f,.32f,-.33f,-.09f,-.30f,.96f,-.30f,-.17f,.54f,.48f,.61f,-.50f,-.01f,.49f,-.88f,-.28f,.50f,.66f,.03f,-.19f,-.01f,-.44f,-.73f,-.11f,.07f,.71f,.05f,-.38f,-.50f,-.47f,-.92f,.75f,.48f,-.46f,-.08f,-.15f,.59f,.32f,.87f,.87f,.80f,-.79f,.20f,-.08f,-.64f,.45f,-.26f,.42f,-.95f,.03f,.91f,-.11f,.14f,.63f,.55f,.59f,.04f,.79f,.30f,-.13f,.20f,.48f,-.07f,-.69f,.29f,-.02f,.45f,-.61f,-.52f,.46f,-.28f,.81f,-.93f,-.73f,.43f,-.96f,.45f,.09f,.51f,.56f,-.39f,.20f,-.18f,.16f,-.28f,.02f,.05f,-.45f,.31f,-.61f,.82f,-.92f,-.94f,-.20f,.10f,.44f,-.02f,-.84f,.16f,.08f,-.66f,.06f,.72f,.09f,-.91f,.70f,-.96f,.53f,-.30f,.89f,.76f,.11f,.61f,.89f,-.02f,-.65f,-.98f,-.93f,-.66f,-.11f,-.97f,.52f,-.75f,-.15f,.30f,.39f,.32f,.49f,-.72f,.44f,.62f,.64f,-.76f,.63f,-.18f,.29f,-.74f,-.10f,-.35f,-.11f,-.42f,.29f,-.07f,.06f,-.26f,-.91f,.85f,-.31f,-.73f,.84f,.89f,.46f,.96f,.92f,-.76f,.87f,-.83f,-.57f,.42f,-.79f,-.55f,-.32f,-.14f,.36f,-.37f,.14f,.17f,.60f,.48f,-.66f,.24f,-.27f,.30f,-.76f,-.73f,-.14f,-.86f,-.43f,.93f,.61f,-.03f,.70f,.74f,-.91f,.45f,-.78f,-.07f,-.60f,-.57f,.40f,.51f,.53f,-.02f,.27f,.04f,.94f,.49f,-.16f,-.94f,-.16f,.41f,-.45f,.31f,.56f,.25f,-.56f,.87f,.15f,-.83f,-.66f,-.69f,-.16f,.32f,-.61f,.76f,-.59f,-.69f,-.70f,.42f,.65f,.90f,-.45f,.59f,-.25f,-.94f,.74f,-.87f,-.24f,.69f,-.85f,.33f,-.89f,.35f,-.29f,-.67f,-.28f,-.85f,-.91f,-.68f,.38f,.48f,-.21f,-.00f,-.39f,.93f,.00f,-.43f,-.25f,.67f,.82f,.00f,.27f,-.76f,.44f,-.83f,-.61f,-.80f,.98f,-.07f,-.86f,.03f,.69f,-.88f,-.87f,-.65f,.88f,.61f,.32f,-.82f,.76f,-.57f,-.59f,.02f,-.80f,.34f,-.98f,-.25f,-.34f,.83f,.40f,-.73f,-.63f,-.73f,-.59f,-.15f,-.19f,.07f,-.21f,-.87f,-.52f,.03f,.69f,-.73f,.99f,.25f,-.28f,.50f,-.17f,.81f,.70f,-.62f,.19f,-.64f,-.14f,.87f,.37f,-.98f,-.33f,-.41f,-.21f,.51f,.84f,-.93f,-.44f,.40f,.86f,-.39f,-.46f,.01f,-.06f,.74f,.43f,-.97f,.69f,-.14f,.06f,.46f,.72f,.48f,-.33f,.91f,.85f,.07f,.51f,.34f,-.16f,.46f,.88f,-.24f,.17f,.55f,-.61f,.16f,-.32f,.02f,.53f,.91f,.04f,.26f,.33f,-.64f,-.50f,.32f,.55f,-.47f,.37f,.69f,.09f,-.29f,.75f,-.19f,.93f,.92f,.65f,.85f,-.45f,.17f,-.36f,-.08f,-.01f,.08f,.69f,-.98f,-.46f,-.32f,-.66f,.98f,.08f,-.71f,-.98f,-.51f,-.95f,.23f,.26f,-.35f,-.72f,.69f,-.64f,-.13f,-.23f,.95f,-.74f,-.25f,.34f,.36f,.25f,-.51f,.00f,-.60f,-.92f,.81f,-.84f,.85f,-.55f,.26f,.20f,.18f,-.78f,-.27f,.99f,-.36f,-.97f,.76f,.10f,-.80f,.16f,-.90f,.48f,-.38f,-.15f,.97f,.75f,.17f,.60f,.15f,-.76f,-.58f,.72f,.24f,.16f,-.37f,.07f,.45f,-.29f,.17f,-.30f,-.37f,-.15f,.43f,-.90f,-.33f,.90f,-.29f,.53f,.96f,.83f,.11f,.28f,-.70f,.87f,-.26f,.90f,.17f,.89f,-.25f,-.92f,.62f,.10f,.14f,-.91f,-.59f,-.54f,-.34f,.10f,.25f,.69f,-.60f,.57f,.82f,-.12f,-.97f,-.72f,.34f,-.43f,-.02f,-.16f,-.49f,-.10f,-.22f,-.95f,-.29f,.15f,.15f,.55f,-.55f,.89f,-.28f,-.15f,-.22f,-.63f,.28f,.64f,-.82f,-.09f,-.14f,.96f,-.07f,-.58f,-.29f,.77f,.18f,.47f,-.33f,-.14f,-.80f,-.80f,-.52f,.52f,-.37f,-.09f,.13f,.85f,.01f,.25f,.47f,-.67f,.29f,-.52f,.92f,.87f,-.25f,-.87f,.67f,.44f,-.85f,.81f,-.29f,-.51f,-.11f,-.86f,-.92f,.90f,.87f,.57f,.45f,.74f,.02f,.81f,-.50f,-.32f,.24f,-.70f,-.71f,-.30f,.71f,.56f,.60f,-.64f,.56f,-.88f,.75f,-.04f,.90f,.40f,-.39f,-.16f,.99f,-.55f,.55f,.77f,-.76f,.25f,-.18f,-.78f,.71f,.05f,-.03f,.80f,-.55f,-.13f,.63f,-.33f,.18f,-.66f,.27f,-.83f,-.21f,-.79f,-.46f,-.15f,-.61f,-.28f,-.84f,.58f,-.43f,.85f,-.22f,.54f,.73f,-.87f,-.48f,.53f,-.97f,-.93f,.85f,-.96f,.37f,.78f,.61f,.55f,.43f,-.75f,.09f,-.06f,-.23f,.68f,-.42f,-.88f,-.81f,-.36f,.33f,-.50f,.63f,-.74f,-.61f,-.29f,.14f,.78f,-.19f,-.13f,-.73f,.97f,.87f,-.15f,-.62f,-.16f,.83f,-.78f,.87f,.48f,.32f,.77f,.59f,-.02f,.06f,.72f,.84f,.18f,-.24f,-.53f,.26f,.56f,.50f,.62f,-.29f,-.08f,-.39f,-.29f,-.36f,-.98f,.53f,.53f,-.21f,-.08f,.47f,-.77f,-.73f,-.48f,.84f,.12f,.09f,-.22f,.77f,-.10f,-.69f,.10f,.13f,-.42f,.96f,-.67f,-.78f,.10f,.66f,.85f,.52f,.89f,.50f,.38f,-.03f,.75f,.29f,.53f,-.71f,.99f,-.58f,.75f,.74f,-.66f,.50f,.76f,-.73f,-.84f,-.86f,.71f,.07f,-.82f,-.43f,.96f,.94f,-.36f,-.27f,.43f,.03f,-.60f,-.35f,.41f,-.30f,.06f,-.51f,.49f,-.34f,.99f,-.86f,.99f,-.03f,-.43f,-.27f,-.72f,-.92f,.17f,-.78f,.43f,.93f,.12f,-.15f,.73f,.88f,.44f,-.58f,-.20f,.27f,.45f,.40f,.02f,.94f,-.68f,.88f,-.76f,-.16f,.08f,.92f,-.88f,.03f,-.66f,-.59f,-.05f,-.95f,-.54f,.97f,.94f,-.84f,.24f,-.03f,-.14f,.58f,-.75f,.38f,.00f,-.90f,-.34f,-.70f,-.79f,-.33f,.48f,.92f,-.68f,-.88f,.99f,-.58f,.32f,.81f,-.57f,.23f,-.70f,-.26f,.49f,-.13f,.93f,.66f,-.31f,-.58f,.56f,-.57f,-.38f,.59f,.10f,.10f,-.55f,-.41f,.13f,-.22f,-.53f,.30f,.39f,.66f,.93f,-.38f,.93f,.35f,-.47f,.21f,.14f,.86f,.71f,-.51f,-.99f,.72f,-.37f,-.24f,-.04f,.19f,.03f,-.18f,.38f,-.68f,.88f,.98f,-.57f,.26f,-.18f,.34f,.99f,.04f,-.05f,.84f,-.83f,.45f,-.93f,-.94f,-.65f,.91f,-.23f,.34f,-.34f,-.11f,.10f,.96f,.22f,.59f,.67f,.73f,-.05f,.34f,-.69f,-.74f,-.32f,-.47f,.96f,-.38f,-.77f,-.04f,-.30f,-.29f,-.82f,.22f,-.05f,-.56f,.10f,-.78f,-.06f,-.14f,-.46f,-.11f,.66f,.83f,-.46f,-.26f,-.19f,-.55f,.28f,.00f,-.02f,.02f,-.64f,.14f,-.57f,-.49f,.07f,-.38f,.18f,.75f,-.97f,.20f,-.14f,.26f,.58f,-.31f,-.64f,.90f,.52f,-.49f,.98f,.78f,-.84f,-.36f,.64f,-.63f,.46f,-.95f,.05f,.27f,.65f,.69f,-.80f,-.31f,-.57f,.70f,-.34f,-.65f,-.46f,-.06f,-.12f,-.35f,-.03f,-.49f,.38f,-.07f,-.15f,-.46f,.65f,.70f,-.05f,-.98f,-.75f,.07f,-.61f,-.69f,.05f,-.39f,-.93f,-.93f,.66f,-.68f,.52f,.76f,-.24f,-.78f,-.06f,-.77f,.28f,-.65f,-.74f,.42f,-.04f,-.59f,.97f,-.96f,.87f,.30f,-.31f,-.24f,.99f,-.15f,-.06f,.87f,-.86f,.34f,-.68f,-.15f,.45f,-.83f,-.33f,.56f,-.53f,.45f,-.95f,.48f,.89f,.93f,-.74f,-.21f,.15f,-.17f,-.81f,-.07f,.99f,-.32f,-.66f,.64f,-.89f,-.74f,.14f,.04f,-.82f,-.74f,-.30f,.61f,.35f,-.72f,.52f,-.60f,-.44f,.42f,.58f,.29f,.91f,-.44f,.94f,.65f,-.32f,-.26f,-.02f,.79f,.45f,-.40f,.10f,.53f,-.11f,-.15f,-.99f,-.24f,.86f,.85f,-.42f,-.00f,.28f,.07f,-.86f,-.18f,.75f,-.38f,.16f,-.62f,.97f,.28f,.30f,.80f,.14f,.24f,.82f,-.25f,.55f,-.82f,.42f,-.39f,-.24f,-.58f,.54f,.27f,.95f,-.98f,-.30f,-.55f,.51f,-.52f,-.32f,-.50f,.49f,-.90f,-.15f,.38f,-.92f,.38f,-.48f,.39f,.45f,-.56f,-.96f,-.96f,.72f,-.25f,-.77f,-.31f,.59f,-.89f,.42f,.48f,.62f,-.11f,-.78f,.70f,.46f,.88f,-.32f,-.62f,-.87f,-.61f,.14f,.99f,.44f,.81f,-.66f,.24f,-.56f,.69f,-.97f,.76f,-.75f,-.00f,-.94f,.58f,-.32f,.82f,.74f,-.24f,-.88f,.30f,-.07f,.04f,.75f,-.70f,.70f,.23f,-.21f,-.51f,.36f,.63f,-.70f,.10f,.09f,.64f,-.51f,-.12f,-.23f,-.39f,.35f,.74f,-.58f,.39f,-.84f,.81f,-.93f,-.40f,.31f,-.81f,.41f,-.89f,-.27f,-.39f,.13f,-.51f,-.72f,-.55f,.79f,.49f,-.29f,.00f,-.92f,-.10f,-.77f,-.75f,-.33f,-.20f,.21f,.22f,-.31f,.14f,-.58f,.82f,.38f,.33f,-.01f,.54f,-.67f,.24f,.60f,.53f,.52f,.21f,-.73f,-.72f,-.60f,-.37f,-.34f,-.06f,.15f,-.63f,-.99f,-.28f,.04f,-.36f,-.28f,.35f,.88f,.24f,.52f,-.54f,.55f,-.91f,.36f,.59f,.48f,-.43f,-.54f,-.36f,.12f,.56f,-.72f,.54f,-.75f,-.77f,.12f,-.89f,-.30f,.93f,-.11f,-.18f,-.67f,-.15f,.02f,-.76f,-.51f,.45f,.79f,-.49f,.90f,-.98f,-.70f,.10f,.35f,-.22f,.66f,-.62f,.87f,.44f,-.55f,-.63f,.09f,-.42f,-.38f,.81f,.69f,-.56f,-.51f,.25f,.31f,.06f,-.99f,.45f,-.85f,.84f,.32f,.21f,.61f,-.44f,-.63f,.46f,-.87f,-.03f,.40f,-.52f,.89f,-.92f,-.14f,.30f,.39f,.30f,-.63f,-.01f,-.66f,-.02f,-.51f,.14f,.02f,.28f,.85f,.77f,-.55f,.69f,-.13f,-.72f,-.04f,-.35f,.05f,.40f,.49f,.27f,-.64f,-.46f,-.59f,-.33f,.43f,.53f,-.50f,-.60f,.28f,.82f,.51f,.90f,.48f,-.55f,-.08f,-.32f,.68f,-.93f,-.09f,-.85f,-.29f,-.92f,.15f,.84f,-.97f,.12f,.72f,.81f,-.88f,-.98f,-.26f,.85f,.56f,.95f,-.10f,.67f,-.14f,.03f,.25f,.72f,-.22f,.18f,-.72f,-.85f,.83f,.00f,.98f,-.34f,.35f,.62f,-.65f,-.66f,.78f,.09f,-.09f,-.98f,.89f,-.27f,-.54f,-.82f,.06f,-.12f,-.89f,.45f,-.07f,.73f,-.66f,-.00f,-.67f,.56f,-.41f,-.99f,.09f,.30f,-.30f,.76f,-.33f,.02f,.29f,-.36f,.69f,.83f,-.16f,.76f,-.36f,-.47f,-.22f,.18f,-.49f,.57f,-.02f,-.56f,-.93f,-.19f,-.98f,-.98f,-.17f,-.42f,.55f,.49f,-.37f,.02f,-.22f,-.61f,-.36f,.94f,-.65f,-.87f,.56f,-.93f,-.53f,-.23f,-.30f,.34f,.53f,-.55f,-.62f,-.86f,.22f,-.57f,.47f,.12f,-.93f,-.83f,-.90f,.77f,.36f,.73f,-.59f,.87f,.88f,-.54f,.88f,.39f,.42f,.50f,.68f,-.66f,.07f,.22f,-.39f,-.47f,.41f,-.79f,-.38f,.79f,-.09f,.37f,-.07f,.05f,.66f,-.54f,.45f,.39f,.84f,.72f,.12f,.30f,-.55f,.54f,.42f,-.14f,-.85f,-.56f,-.55f,.27f,.70f,.98f,-.28f,-.29f,.48f,-.02f,-.83f,-.86f,-.92f,.88f,-.11f,.53f,.06f,-.45f,.05f,.78f,.69f,.22f,.52f,.11f,.38f,-.52f,.47f,-.02f,-.43f,-.19f,-.39f,-.45f,.81f,.29f,-.97f,-.04f,.25f,-.29f,-.77f,.46f,-.62f,-.92f,.85f,-.89f,-.84f,.03f,.16f,.47f,-.18f,-.60f,.04f,.71f,-.99f,.57f,.12f,-.83f,-.26f,.29f,-.81f,-.48f,.91f,.01f,-.38f,.50f,.69f,.38f,.18f,.03f,-.64f,.19f,.10f,.76f,.27f,-.34f,-.28f,-.74f,-.26f,.28f,-.60f,.95f,.52f,.84f,-.20f,.97f,.93f,-.56f,-.29f,.87f,-.71f,.20f,.83f,-.83f,.69f,-.86f,.40f,.34f,.78f,-.50f,-.19f,.50f,-.78f,-.11f,.77f,-.80f,.95f,-.22f,.03f,-.68f,-.63f,.74f,.01f,-.70f,-.59f,-.64f,-.72f,.83f,.41f,-.65f,-.16f,-.29f,.55f,-.63f,.80f,-.85f,-.49f,.45f,-.54f,-.00f,-.36f,-.04f,.66f,-.14f,.15f,-.44f,.79f,-.44f,.81f,.90f,.65f,.27f,-.08f,.28f,-.88f,.89f,-.80f,-.96f,-.97f,.56f,-.72f,-.68f,-.26f,.91f,.43f,.93f,-.85f,-.97f,-.00f,.04f,-.68f,.76f,.61f,-.66f,-.08f,-.75f,-.95f,.84f,.47f,-.66f,-.92f,-.50f,.11f,-.99f,.22f,.20f,.00f,.33f,.14f,-.35f,.48f,.42f,.30f,.89f,-.93f,.88f,-.44f,.08f,.01f,.03f,-.14f,.71f,.24f,-.17f,-.68f,.17f,.53f,.79f,.97f,.32f,.58f,.40f,-.77f,-.77f,.14f,.38f,.69f,.73f,.88f,.81f,.67f,.13f,-.53f,-.13f,-.79f,.70f,.46f,.71f,.06f,-.42f,.96f,.24f,.02f,-.31f,-.18f,-.87f,-.70f,.05f,-.46f,-.59f,-.84f,.12f,.41f,-.90f,.45f,.11f,.02f,-.49f,-.26f,-.41f,.43f,-.34f,-.19f,-.41f,-.64f,-.20f,.22f,-.20f,-.05f,.33f,-.19f,.69f,-.85f,-.01f,.33f,.07f,.80f,-.45f,-.53f,-.57f,.88f,-.80f,.12f,.60f,-.31f,.11f,.33f,-.10f,-.09f,.21f,-.93f,-.44f,.18f,-.13f,-.33f,.17f,.38f,-.43f,.07f,.30f,-.10f,-.25f,.96f,-.23f,.91f,-.47f,.37f,.52f,-.96f,.33f,.37f,-.75f,-.91f,.33f,.11f,-.92f,-.98f,-.74f,-.26f,.04f,-.06f,.38f,-.73f,.27f,.88f,.25f,-.79f,.48f,.82f,.98f,-.97f,.21f,-.16f,.56f,-.03f,-.53f,-.13f,-.73f,-.31f,.37f,-.68f,-.36f,.42f,-.21f,-.02f,-.57f,.50f,.00f,-.87f,.28f,.27f,.98f,-.96f,.21f,.81f,-.33f,-.94f,-.15f,-.45f,.93f,-.36f,.23f,.64f,-.04f,-.13f,.40f,-.72f,.17f,.87f,-.60f,-.82f,-.20f,.32f,.92f,.99f,.28f,-.47f,.50f,.27f,.58f,-.98f,.36f,.90f,.45f,-.65f,.26f,.54f,-.54f,.35f,.07f,-.94f,.23f,.02f,.87f,.83f,.09f,-.83f,.53f,.44f,-.93f,.35f,-.86f,-.01f,.38f,.56f,-.65f,.72f,-.04f,.09f,.17f,.95f,-.64f,.91f,.34f,-.60f,-.40f,.73f,-.66f,-.65f,-.21f,-.17f,-.12f,.50f,.27f,-.68f,-.74f,-.27f,-.64f,-.75f,-.01f,-.50f,.47f,-.50f,-.08f,-.00f,-.79f,.99f,-.01f,-.69f,-.60f,-.95f,-.11f,.25f,-.52f,-.49f,.31f,.76f,-.83f,-.39f,-.42f,-.18f,.06f,-.04f,-.74f,-.27f,-.23f,.76f,.33f,.59f,-.72f,-.70f,-.04f,.63f,-.21f,.25f,.16f,-.03f,.77f,.03f,-.74f,.94f,-.91f,-.23f,.47f,.90f,.82f,-.58f,.77f,-.37f,.05f,-.51f,.09f,-.80f,.95f,.73f,.28f,.56f,-.74f,-.31f,.37f,.65f,.32f,-.50f,-.52f,-.68f,.78f,.82f,.94f,.27f,-.44f,-.05f,.39f,-.80f,-.88f,.42f,.72f,.02f,.26f,-.58f,.21f,-.90f,.00f,.36f,.77f,-.96f,-.18f,.99f,-.71f,.94f,-.81f,-.08f,-.64f,-.71f,-.84f,-.28f,.44f,.44f,.80f,-.17f,-.50f,.38f,.75f,.29f,.54f,-.56f,-.40f,.35f,.59f,.88f,-.87f,.03f,.88f,-.02f,.26f,.68f,.38f,.24f,.03f,.15f,.28f,.73f,.67f,.94f,-.04f,.18f,-.19f,.68f,-.91f,.41f,-.50f,-.36f,.03f,-.65f,.44f,-.73f,-.18f,.39f,.40f,-.53f,-.94f,.11f,.71f,.82f,-.17f,.65f,.60f,.93f,-.99f,-.11f,-.83f,-.31f,-.09f,-.08f,.91f,-.51f,-.25f,.52f,.54f,.23f,-.00f,.69f,.88f,.49f,-.50f,.99f,.82f,-.56f,-.68f,.18f,-.38f,-.97f,.27f,-.83f,.63f,.98f,-.59f,.43f,.07f,-.68f,-.00f,-.25f,-.69f,-.03f,.86f,.72f,.53f,-.95f,-.88f,.15f,.43f,-.61f,-.03f,-.56f,-.44f,-.35f,-.46f,-.21f,-.80f,-.63f,.39f,-.24f,-.91f,.24f,.32f,-.22f,.73f,-.24f,-.53f,-.22f,-.62f,-.39f,-.40f,.60f,.78f,-.11f,.32f,-.34f,.53f,.60f,.10f,-.59f,.31f,.77f,-.54f,.52f,.68f,.84f,-.55f,-.15f,.16f,.15f,-.64f,.73f,.77f,-.57f,-.05f,.36f,-.03f,.27f,.93f,-.88f,.82f,.40f,-.69f,.23f,.01f,.30f,-.21f,.09f,-.18f,.97f,.33f,.48f,-.15f,-.05f,-.86f,.81f,-.33f,-.92f,.58f,.47f,.22f,.81f,.77f,-.82f,-.15f,.82f,-.53f,.31f,.30f,-.16f,-.24f,-.24f,-.80f,.72f,-.91f,-.53f,-.73f,.85f,.82f,-.97f,-.80f,-.13f,-.20f,.40f,-.70f,-.35f,.88f,.08f,-.48f,.77f,.62f,-.16f,-.47f,.53f,.11f,.43f,-.36f,-.13f,-.01f,-.36f,.88f,-.89f,.03f,.69f,-.36f,-.98f,.52f,.28f,.59f,.27f,-.43f,-.75f,.61f,.67f,.46f,-.50f,-.11f,-.77f,-.43f,-.00f,-.80f,.91f,.22f,-.50f,-.83f,-.52f,.94f,-.92f,-.36f,-.96f,-.40f,.11f,-.37f,.08f,.52f,.53f,.02f,-.83f,.54f,-.48f,.02f,-.05f,.32f,-.78f,.02f,-.79f,.34f,-.54f,.93f,-.80f,.32f,.70f,-.82f,.92f,.04f,.83f,-.26f,-.68f,.76f,.78f,.35f,.71f,.83f,.52f,.57f,-.20f,.12f,.26f,-.99f,.47f,.60f,-.06f,-.17f,.26f,-.60f,.70f,-.22f,-.87f,-.78f,.29f,.47f,.76f,.09f,.73f,.07f,-.91f,-.53f,-.40f,.78f,.81f,-.57f,-.05f,-.54f,-.04f,-.14f,.80f,.11f,-.75f,.56f,.63f,-.73f,-.32f,.86f,-.98f,-.64f,.59f,-.95f,.63f,.10f,.24f,-.83f,-.60f,-.67f,.40f,-.96f,-.30f,-.69f,.70f,-.89f,.40f,-.21f,.83f,.44f,.71f,-.85f,-.05f,-.95f,-.83f,.26f,-.63f,.37f,-.05f,-.76f,-.47f,.37f,.01f,.42f,-.12f,.99f,-.43f,.16f,-.20f,-.17f,-.18f,-.13f,-.98f,-.05f,-.86f,.40f,.70f,-.68f,.93f,-.62f,.08f,-.54f,-.01f,.82f,.05f,-.45f,-.83f,-.22f,-.35f,.54f,.48f,.12f,.27f,-.87f,-.67f,-.15f,.02f,-.67f,.52f,.24f,.08f,-.80f,.09f,.99f,.31f,.01f,.83f,-.99f,-.90f,-.23f,.39f,.75f,-.51f,.75f,.37f,-.28f,.27f,-.72f,.71f,.78f,.95f,-.05f,.08f,-.10f,-.19f,.43f,.38f,-.69f,-.38f,.67f,.58f,.86f,.14f,.86f,-.06f,.41f,-.17f,-.15f,.96f,.10f,.20f,.22f,-.92f,-.51f,-.08f,.84f,.83f,-.90f,.51f,-.07f,.69f,.84f,.30f,.35f,-.01f,-.01f,.85f,.76f,.77f,-.50f,.67f,.65f,-.49f,-.47f,-.64f,.63f,-.12f,-.84f,.76f,.50f,.00f,-.83f,-.71f,.43f,-.10f,.95f,.86f,-.77f,.93f,-.61f,-.67f,-.18f,-.53f,-.12f,-.59f,.95f,-.97f,-.08f,-.20f,-.65f,-.78f,-.02f,-.47f,-.55f,-.77f,.00f,-.01f,-.97f,.29f,-.68f,.66f,-.86f,.70f,-.53f,-.99f,-.29f,-.87f,.34f,.13f,-.12f,-.64f,.65f,-.08f,.80f,-.37f,-.01f,.63f,-.85f,.37f,.23f,.06f,-.57f,-.99f,.28f,-.48f,.70f,.34f,.08f,.96f,-.27f,.36f,.31f,-.39f,-.22f,-.06f,-.67f,-.22f,-.49f,.77f,-.16f,.95f,-.03f,-.40f,.67f,.17f,.73f,.44f,-.69f,.78f,.57f,-.14f,-.61f,-.31f,.73f,-.71f,.03f,.95f,.33f,-.50f,.69f,-.11f,.81f,-.64f,.67f,.66f,-.47f,-.28f,-.11f,.11f,.97f,.67f,.98f,.38f,-.07f,-.38f,.12f,-.59f,.89f,.12f,.05f,-.03f,.58f,-.68f,-.74f,.57f,.81f,.23f,-.35f,-.27f,.29f,-.43f,.03f,-.60f,.18f,.07f,.74f,-.56f,.70f,-.11f,.28f,-.73f,.14f,-.60f,.00f,-.71f,-.38f,.39f,.93f,-.32f,.75f,-.39f,.63f,-.44f,-.10f,.10f,.83f,.56f,-.07f,-.25f,.75f,.96f,.34f,-.78f,-.95f,-.18f,.36f,.57f,-.82f,.44f,-.57f,.65f,.40f,-.02f,.97f,.31f,.06f,-.91f,-.80f,-.74f,-.29f,.46f,-.58f,.47f,.25f,-.06f,.28f,.98f,-.56f,-.13f,.90f,.28f,.99f,.78f,-.46f,.96f,-.34f,-.94f,.22f,.99f,-.48f,.18f,.97f,-.52f,.98f,.74f,-.18f,.04f,.98f,-.85f,-.36f,-.29f,-.18f,-.39f,-.89f,.49f,-.71f,.24f,.59f,-.85f,.58f,.86f,-.68f,.07f,.72f,-.32f,-.76f,.84f,.05f,.16f,.68f,-.76f,.22f,-.65f,-.18f,-.49f,-.17f,.70f,-.42f,.98f,.18f,.18f,.22f,.14f,.35f,.26f,-.36f,.98f,.84f,-.54f,.13f,.90f,.46f,.53f,-.94f,-.15f,-.14f,.77f,.68f,.16f,-.49f,-.43f,-.83f,-.33f,.14f,-.55f,-.39f,-.14f,.08f,-.78f,-.79f,-.65f,-.65f,.73f,.97f,.65f,.50f,-.51f,-.24f,-.88f,.59f,.74f,.03f,-.91f,.33f,.03f,-.66f,-.20f,-.13f,.94f,.46f,.41f,.50f,.45f,.47f,-.10f,-.28f,-.36f,-.02f,-.52f,.11f,-.19f,-.70f,.33f,.20f,-.85f,-.15f,-.89f,.11f,.68f,-.64f,-.70f,.90f,-.02f,-.26f,-.49f,.88f,.46f,.75f,.96f,.46f,.08f,.43f,-.14f,-.17f,-.74f,-.75f,-.19f,.61f,-.87f,.84f,-.04f,.85f,.16f,-.73f,.14f,-.21f,.21f,.73f,.92f,.10f,-.66f,.37f,-.64f,.90f,.32f,-.46f,.04f,.80f,.88f,.13f,-.88f,-.48f,.37f,.42f,-.35f,-.92f,-.16f,.61f,-.15f,.17f,.14f,.89f,-.15f,.18f,.94f,-.13f,.51f,-.67f,.64f,.41f,.45f,.00f,-.81f,.06f,-.06f,-.48f,-.92f,-.42f,.83f,-.87f,.29f,.72f,.70f,.40f,.79f,-.80f,.17f,.32f,.29f,-.30f,-.13f,.82f,.72f,-.96f,-.84f,-.13f,-.36f,.10f,.40f,.59f,.92f,-.30f,-.16f,-.53f,-.47f,.14f,-.82f,.02f,.81f,.27f,-.77f,.63f,.47f,.24f,.19f,-.29f,-.56f,-.49f,-.03f,.60f,-.84f,.10f,-.30f,-.09f,.78f,.31f,-.75f,.19f,-.10f,.02f,.45f,.60f,-.36f,.68f,.49f,-.29f,-.21f,.54f,-.66f,.55f,.03f,.53f,.25f,-.47f,-.13f,.41f,.74f,.02f,-.67f,-.68f,-.61f,-.92f,-.37f,-.65f,-.76f,-.86f,.83f,-.84f,-.79f,-.36f,.09f,.43f,.68f,-.81f,-.25f,.81f,-.66f,.28f,.33f,.74f,.56f,-.29f,-.84f,-.76f,.74f,-.03f,.52f,.30f,.18f,.11f,-.46f,.28f,-.15f,-.27f,.21f,.79f,-.46f,-.41f,-.70f,.60f,-.35f,-.33f,-.56f,-.14f,.17f,.01f,-.81f,-.62f,-.60f,.39f,-.62f,.87f,.83f,-.00f,-.47f,-.39f,-.97f,.11f,.22f,-.32f,-.21f,-.25f,.70f,.70f,.01f,.04f,-.38f,-.12f,.80f,.85f,.40f,.87f,-.59f,-.99f,-.17f,.63f,.30f,.36f,.77f,-.25f,-.50f,.22f,-.26f,.49f,-.97f,.18f,-.29f,-.19f,-.28f,.34f,-.69f,-.15f,-.50f,.18f,.68f,-.31f,.52f,-.18f,-.67f,-.22f,.11f,-.58f,-.17f,-.21f,-.01f,.71f,.93f,.00f,.41f,-.72f,-.66f,-.26f,-.57f,-.06f,-.44f,-.06f,-.79f,-.14f,-.09f,.52f,.67f,-.65f,-.52f,-.85f,-.52f,-.18f,-.70f,.75f,-.51f,-.49f,-.06f,.81f,.06f,-.95f,.53f,-.80f,-.16f,.84f,.97f,-.53f,.00f,-.75f,-.99f,-.38f,.07f,-.44f,.25f,.31f,.16f,-.39f,-.92f,-.17f,-.38f,-.63f,.27f,.53f,.56f,-.82f,.95f,.69f,-.25f,.62f,-.36f,-.21f,.15f,-.82f,-.73f,-.77f,-.78f,.46f,-.15f,.44f,.82f,-.82f,-.50f,-.73f,-.39f,-.85f,.89f,-.30f,-.48f,-.96f,.95f,.66f,-.08f,.82f,-.00f,-.16f,.99f,-.49f,.27f,.88f,-.69f,-.36f,-.81f,.86f,-.53f,.79f,.76f,.56f,.59f,-.70f,.31f,.61f,.80f,.93f,.90f,.05f,.42f,.21f,-.75f,-.87f,.48f,-.83f,-.58f,-.53f,-.20f,-.34f,-.08f,.25f,-.21f,-.18f,-.82f,.59f,.18f,-.73f,.50f,.36f,.01f,-.54f,-.40f,.24f,-.99f,-.30f,-.43f,-.26f,.53f,.98f,.90f,-.68f,-.57f,-.02f,.63f,-.84f,-.96f,-.52f,.32f,.09f,-.98f,-.52f,.64f,.17f,.82f,.07f,.92f,.63f,-.43f,-.52f,.06f,-.85f,.02f,-.71f,-.86f,-.58f,-.32f,.79f,.68f,.02f,.50f,-.76f,-.26f,-.23f,.03f,.42f,-.26f,-.09f,-.40f,.84f,.79f,-.57f,-.07f,-.61f,-.79f,-.97f,.16f,.10f,.45f,-.04f,.66f,-.52f,-.39f,-.77f,.94f,.70f,.61f,-.96f,-.33f,.85f,.53f,-.74f,.30f,.31f,-.33f,.74f,-.56f,-.00f,-.95f,-.91f,-.42f,.85f,-.02f,-.06f,-.73f,.07f,-.52f,.10f,.39f,.90f,.78f,-.11f,.58f,.61f,.30f,.53f,.51f,.06f,.62f,.58f,.39f,-.78f,.47f,-.36f,-.48f,.29f,-.98f,.90f,-.63f,-.40f,.77f,.93f,.32f,.94f,.76f,-.86f,.84f,-.57f,-.35f,-.68f,.58f,-.70f,-.87f,-.13f,-.99f,-.46f,.51f,.28f,-.22f,-.92f,-.02f,.77f,.91f,.41f,-.13f,-.18f,-.80f,-.12f,.99f,-.62f,.71f,-.39f,-.53f,-.75f,-.88f,-.05f,.84f,.32f,.87f,.18f,.71f,.03f,-.48f,-.59f,.43f,-.44f,.23f,-.36f,.79f,-.27f,-.30f,-.35f,.56f,.99f,-.35f,-.89f,-.45f,.45f,-.55f,.27f,-.29f,.90f,-.95f,-.63f,.31f,-.28f,-.56f,.38f,-.08f,-.18f,.51f,.60f,-.93f,-.00f,.89f,-.52f,.08f,-.36f,-.61f,.71f,-.06f,.53f,-.77f,-.46f,-.30f,-.79f,-.36f,.59f,-.32f,-.28f,.88f,-.15f,.99f,.31f,.84f,.72f,.45f,.61f,-.69f,.73f,-.28f,.88f,-.21f,.65f,-.54f,-.76f,-.33f,-.53f,.87f,-.36f,.96f,.60f,-.07f,.43f,.52f,.68f,.72f,-.53f,-.23f,.09f,.47f,-.92f,-.31f,-.63f,-.26f,-.86f,.28f,.77f,-.14f,.63f,.48f,.81f,.31f,.94f,.37f,.93f,-.70f,.64f,-.63f,-.31f,-.10f,-.64f,.61f,.02f,-.86f,-.40f,-.95f,-.80f,.98f,.96f,-.07f,.33f,-.59f,-.13f,-.54f,.67f,-.50f,.14f,-.30f,.30f,-.80f,.71f,.86f,.26f,-.97f,-.57f,-.88f,-.95f,.84f,-.25f,.58f,-.28f,-.61f,-.55f,-.03f,.14f,-.05f,.66f,.98f,-.01f,-.41f,.06f,.74f,.09f,.42f,.34f,-.06f,.27f,-.26f,-.82f,-.07f,.67f,-.12f,-.46f,.16f,-.07f,.05f,-.21f,.35f,-.77f,-.39f,.11f,.15f,.06f,.17f,.69f,.25f,-.89f,.93f,-.29f,-.25f,-.95f,.74f,-.29f,-.22f,.82f,-.71f,.39f,-.54f,.58f,.41f,-.82f,-.65f,-.41f,-.74f,-.09f,-.08f,-.53f,-.06f,-.02f,.51f,.76f,.40f,.23f,-.14f,-.71f,-.59f,.33f,.14f,-.98f,-.01f,-.45f,.25f,-.33f,-.21f,.47f,-.28f,-.43f,.52f,-.04f,.04f,-.63f,.53f,-.15f,-.23f,-.15f,.04f,-.99f,-.69f,.16f,.26f,-.21f,.90f,.99f,-.91f,-.69f,.40f,-.69f,-.52f,.18f,.86f,-.41f,.69f,.44f,-.13f,-.16f,.89f,.60f,-.12f,.06f,.84f,-.58f,.71f,.19f,.81f,.58f,-.64f,-.39f,-.92f,-.70f,-.98f,-.13f,.07f,-.99f,.14f,.69f,-.20f,.18f,-.80f,.34f,-.79f,.94f,.34f,-.81f,-.69f,-.72f,-.07f,.33f,.06f,.87f,-.77f,.72f,.16f,.74f,.87f,-.87f,.57f,.87f,.65f,.63f,-.78f,.71f,.64f,-.94f,.45f,.52f,.61f,.80f,.56f,.89f,-.99f,.31f,.11f,-.63f,.21f,.97f,.04f,-.59f,-.81f,.13f,.46f,.28f,.01f,-.22f,.62f,-.12f,.64f,.82f,-.67f,-.76f,.87f,-.23f,-.43f,.72f,-.64f,.08f,.53f,-.41f,-.03f,-.68f,.62f,.08f,.37f,-.12f,-.52f,.07f,.06f,-.28f,.02f,.35f,.11f,.38f,.90f,-.19f,-.36f,.95f,-.53f,-.66f,-.30f,.46f,.44f,.63f,.15f,.75f,-.88f,-.80f,-.39f,-.33f,.45f,.56f,-.34f,.98f,-.50f,.22f,.15f,.12f,.62f,.06f,.82f,-.40f,.86f,.56f,.18f,.13f,-.30f,-.98f,.57f,-.49f,-.32f,-.74f,.91f,-.07f,.09f,.35f,-.22f,-.42f,.37f,.41f,-.01f,-.45f,.96f,-.97f,-.37f,.58f,-.74f,.61f,.37f,-.25f,.82f,.99f,-.22f,.60f,-.95f,.35f,.63f,-.73f,-.96f,.40f,.17f,-.04f,.01f,-.45f,-.52f,.86f,.32f,-.97f,-.67f,.21f,.60f,-.88f,-.17f,.11f,-.89f,.52f,-.93f,-.17f,-.94f,.72f,-.45f,.81f,-.85f,-.63f,.27f,.72f,-.99f,-.74f,-.70f,-.06f,-.28f,.28f,.89f,-.38f,-.36f,.28f,.51f,-.29f,-.04f,-.22f,-.77f,.62f,.68f,.73f,-.29f,.96f,.26f,.36f,-.79f,.93f,.23f,.06f,-.99f,.93f,.34f,-.40f,.43f,-.80f,-.60f,-.64f,-.17f,-.89f,-.56f,-.81f,.70f,-.67f,.57f,.57f,-.77f,-.28f,-.39f,-.58f,-.35f,-.02f,.21f,-.43f,.60f,-.95f,-.44f,.40f,-.22f,.44f,-.94f,.21f,-.21f,-.38f,-.06f,.43f,-.48f,.49f,-.79f,.31f,-.91f,.34f,.89f,.10f,.50f,.48f,.29f,.00f,.24f,.50f,.56f,-.73f,-.10f,.23f,.29f,.59f,-.02f,-.76f,-.46f,-.97f,-.06f,-.46f,.51f,.14f,.97f,-.37f,.65f,-.74f,-.20f,-.29f,.57f,-.80f,-.34f,-.33f,.17f,-.86f,.58f,-.85f,-.08f,.09f,-.88f,.57f,-.78f,-.55f,-.72f,.18f,-.45f,.37f,-.51f,-.77f,.83f,-.39f,.73f,-.87f,-.18f,-.95f,.94f,-.21f,.70f,.56f,.25f,.99f,.14f,.18f,-.35f,.72f,.09f,.98f,.85f,.10f,.97f,.42f,-.15f,-.85f,.89f,.67f,.99f,-.08f,.32f,.08f,.55f,.96f,-.52f,-.63f,-.21f,-.50f,.79f,.10f,.61f,-.96f,-.47f,.59f,-.02f,-.39f,.89f,.31f,.94f,-.84f,.47f,.79f,.82f,-.18f,-.41f,-.22f,-.99f,-.68f,-.35f,.86f,.16f,.68f,.05f,-.11f,-.65f,.23f,-.81f,-.92f,-.70f,.60f,-.65f,-.37f,-.23f,.10f,.15f,.19f,-.03f,.74f,-.07f,.12f,-.49f,.67f,-.91f,.38f,.53f,.86f,-.41f,-.39f,-.79f,-.48f,-.83f,-.58f,.18f,-.96f,-.77f,-.47f,-.38f,.32f,.23f,.85f,.35f,.40f,-.27f,.66f,.62f,.65f,-.94f,-.69f,-.93f,.69f,-.20f,.05f,-.67f,-.18f,-.36f,-.31f,.15f,-.01f,.86f,-.72f,-.94f,.41f,-.22f,-.86f,-.56f,.69f,.09f,-.00f,-.80f,.05f,.98f,.95f,.46f,-.33f,-.38f,-.04f,-.53f,-.12f,.82f,-.55f,-.79f,.10f,-.14f,-.68f,-.29f,.14f,.93f,.64f,.97f,.37f,.69f,.73f,-.57f,.11f,-.75f,-.07f,.74f,.25f,-.57f,.67f,.60f,-.26f,-.35f,-.19f,.55f,.42f,-.62f,.63f,-.87f,-.82f,.60f,-.10f,-.67f,.89f,-.38f,.29f,.30f,.81f,.95f,.43f,-.64f,.91f,.37f,.96f,-.71f,.55f,.06f,-.95f,-.56f,.34f,.25f,.05f,.26f,.28f,.95f,.68f,-.63f,-.80f,.63f,-.09f,-.06f,.26f,-.34f,-.27f,-.57f,.99f,-.48f,.10f,.96f,-.17f,.70f,-.76f,-.49f,.37f,-.81f,-.05f,-.67f,.70f,-.94f,-.15f,-.98f,.03f,-.77f,.97f,-.61f,-.32f,.63f,-.01f,-.42f,-.89f,.41f,.14f,.11f,-.61f,-.05f,.32f,.21f,.64f,.36f,-.44f,.05f,-.33f,.19f,-.43f,-.40f,-.89f,.90f,.67f,.66f,-.37f,-.67f,-.63f,.93f,.29f,.28f,.41f,.17f,-.76f,-.61f,-.48f,-.86f,-.13f,-.55f,.96f,-.06f,-.14f,.97f,.86f,-.20f,.88f,.40f,.64f,.79f,.47f,-.09f,-.26f,-.31f,.74f,-.09f,-.49f,.64f,-.40f,-.31f,-.87f,-.68f,.40f,-.44f,.19f,.10f,-.69f,.42f,-.59f,.84f,-.39f,-.85f,.33f,.44f,.12f,.69f,.02f,-.71f,.57f,-.16f,-.67f,.53f,-.95f,.37f,.42f,.40f,.76f,-.61f,.02f,.77f,.66f,.54f,-.37f,-.76f,-.56f,.77f,.57f,.99f,-.02f,.00f,.54f,.19f,.89f,-.32f,-.00f,-.74f,-.82f,.43f,.42f,.35f,-.81f,.68f,-.52f,-.17f,.18f,.29f,.76f,-.73f,.01f,-.28f,.04f,.21f,.89f,-.40f,.04f,.13f,-.95f,.33f,.47f,.18f,.61f,-.83f,-.08f,-.08f,-.75f,.30f,.42f,.76f,.25f,.18f,-.84f,.11f,-.94f,-.42f,-.16f,-.97f,.54f,.11f,.37f,.10f,-.39f,-.20f,.09f,.04f,-.92f,.97f,.98f,-.78f,.58f,-.74f,.92f,-.27f,.16f,-.42f,-.31f,.39f,.23f,-.65f,.24f,-.92f,.52f,.98f,.39f,.51f,.98f,-.98f,.34f,.68f,-.84f,.41f,-.87f,.55f,.39f,.16f,.37f,.01f,.82f,.91f,-.82f,-.76f,.65f,-.45f,.44f,.07f,-.46f,.64f,-.63f,-.11f,.54f,-.92f,-.11f,-.94f,-.78f,-.88f,.68f,-.11f,.78f,.28f,-.08f,.99f,.27f,.73f,-.55f,.05f,.69f,-.69f,-.83f,.70f,-.24f,.26f,-.86f,.69f,.91f,.43f,.84f,-.60f,.31f,.74f,.57f,-.93f,-.85f,-.74f,.73f,-.15f,.30f,.70f,-.15f,.09f,-.46f,.59f,.05f,.34f,.33f,.17f,-.08f,.03f,-.60f,.93f,.11f,-.13f,-.65f,-.19f,.49f,.82f,-.77f,.22f,-.02f,.79f,.06f,.16f,-.79f,-.30f,-.48f,.62f,-.12f,-.95f,-.87f,-.94f,-.90f,-.33f,.60f,-.21f,.88f,-.28f,.46f,-.14f,.13f,-.11f,-.17f,-.03f,.19f,-.04f,-.34f,-.14f,.65f,-.35f,-.66f,-.75f,.25f,-.91f,-.45f,-.59f,.32f,.75f,-.24f,-.14f,.10f,-.24f,.03f,-.83f,.90f,.08f,.09f,.65f,-.39f,-.80f,.26f,.36f,-.14f,-.53f,.21f,-.06f,.63f,.94f,-.86f,.30f,.91f,-.01f,-.61f,.67f,-.44f,-.57f,.28f,.83f,-.46f,-.18f,-.27f,-.25f,.43f,.55f,-.74f,-.39f,.51f,.62f,.70f,.17f,.61f,-.87f,-.76f,-.28f,.80f,-.09f,.48f,-.68f,.46f,.76f,.73f,-.06f,.15f,.01f,.84f,-.71f,.43f,-.82f,.77f,-.86f,.30f,.01f,-.03f,-.79f,-.96f,.75f,.36f,-.18f,-.23f,.79f,.36f,-.77f,.96f,.60f,.11f,-.73f,-.44f,-.15f,-.54f,.94f,-.54f,.73f,.43f,.70f,-.02f,-.47f,.14f,-.07f,-.78f,-.12f,.61f,-.65f,.28f,-.66f,-.56f,.76f,.56f,.52f,-.43f,.31f,.11f,-.94f,.04f,-.52f,-.37f,.86f,-.74f,-.38f,-.40f,.41f,-.40f,.09f,-.07f,-.79f,-.08f,-.32f,-.87f,-.31f,.21f,-.58f,-.18f,-.86f,.29f,-.28f,.30f,-.52f,.41f,.66f,.10f,-.40f,.70f,.29f,-.20f,-.25f,.33f,.32f,-.14f,.39f,-.41f,.65f,-.88f,.82f,.34f,-.80f,.06f,.86f,-.69f,-.84f,-.36f,.43f,.25f,-.80f,.84f,.19f,-.80f,.53f,-.53f,.04f,-.54f,-.26f,-.59f,.21f,-.45f,.53f,-.17f,.52f,.35f,.72f,.16f,-.31f,.12f,-.03f,-.92f,.02f,.27f,.52f,.34f,-.66f,-.46f,-.14f,.81f,.00f,.92f,-.49f,.80f,.40f,-.90f,.34f,.12f,.47f,.16f,-.66f,-.83f,.30f,.10f,-.99f,.95f,.61f,.92f,.90f,-.46f,.99f,.38f,.38f,.79f,-.33f,.28f,.69f,.44f,-.18f,-.66f,.07f,.65f,-.06f,-.24f,-.70f,.01f,-.31f,-.40f,.19f,.93f,-.52f,-.29f,-.59f,-.55f,-.57f,-.61f,-.51f,.99f,.22f,.51f,.71f,.39f,.59f,.38f,.02f,-.98f,.56f,-.04f,-.02f,.44f,-.85f,-.70f,-.70f,-.02f,-.09f,-.96f,.72f,.25f,-.20f,.15f,.68f,-.90f,-.91f,-.99f,-.21f,-.59f,-.38f,-.02f,-.84f,-.50f,.68f,-.22f,-.88f,.25f,.98f,-.10f,-.37f,-.14f,.70f,.03f,.45f,.17f,-.01f,.33f,-.15f,.03f,-.66f,-.05f,-.10f,.35f,-.00f,-.12f,-.79f,.76f,.29f,-.27f,.79f,-.57f,.62f,.65f,.84f,-.17f,.23f,.74f,.98f,-.24f,-.96f,-.16f,.76f,-.22f,.35f,-.39f,.67f,.59f,.91f,.88f,.69f,-.72f,.43f,.55f,-.55f,.94f,.34f,-.74f,-.44f,-.48f,-.92f,-.62f,.79f,.52f,-.59f,-.14f,-.20f,.14f,.65f,-.27f,.71f,-.80f,.29f,-.15f,.15f,.30f,-.65f,-.17f,-.72f,-.90f,.99f,-.20f,.40f,-.59f,.46f,-.69f,.97f,-.86f,.35f,-.96f,.22f,.26f,-.87f,-.46f,-.54f,.30f,.28f,.63f,.43f,-.87f,-.16f,.40f,.07f,-.04f,.67f,-.30f,-.45f,.88f,.92f,.05f,.24f,.18f,-.29f,.26f,.57f,-.35f,.49f,-.16f,.32f,-.79f,-.29f,-.87f,-.72f,.40f,-.44f,-.69f,-.93f,-.85f,.92f,.00f,.66f,-.32f,.94f,.40f,.81f,.67f,-.72f,-.11f,.82f,-.83f,-.53f,-.01f,.18f,-.95f,-.41f,-.98f,-.08f,.24f,-.33f,.72f,-.81f,.21f,.46f,.25f,-.74f,.78f,.68f,-.20f,-.66f,.73f,-.09f,.98f,.50f,.38f,.95f,-.79f,.47f,.95f,-.28f,-.03f,.15f,.25f,.61f,.10f,-.42f,.98f,-.93f,-.47f,-.03f,.31f,.50f,-.24f,.42f,-.03f,-.75f,.75f,.86f,.07f,.79f,.39f,.58f,-.09f,-.04f,.63f,-.49f,.89f,.64f,.14f,-.70f,.04f,-.24f,.42f,.98f,-.88f,-.86f,.43f,-.17f,-.79f,-.15f,-.43f,.97f,-.88f,-.97f,.87f,-.61f,.56f,.53f,.94f,.26f,-.71f,.68f,-.19f,.87f,.00f,-.02f,.33f,.91f,-.36f,.60f,-.55f,-.04f,-.58f,-.07f,.18f,.42f,-.77f,-.66f,-.90f,-.23f,-.94f,.90f,-.37f,-.63f,.57f,.98f,-.46f,-.47f,.79f,.63f,-.05f,-.39f,-.51f,.22f,.23f,-.89f,-.01f,.98f,-.59f,-.85f,.12f,.24f,.63f,-.69f,.84f,-.15f,.48f,-.16f,.14f,.78f,.84f,.86f,-.80f,-.05f,-.42f,.32f,-.39f,-.27f,.85f,-.58f,.96f,-.53f,-.40f,-.75f,.76f,-.87f,-.39f,.73f,.48f,.87f,-.49f,.94f,-.25f,-.19f,-.30f,.56f,-.35f,.09f,.62f,.02f,.07f,-.32f,.83f,.87f,.96f,.50f,.16f,.12f,-.21f,-.77f,-.69f,.95f,-.84f,-.93f,-.83f,.68f,.06f,.48f,-.95f,.44f,.62f,.21f,-.62f,-.11f,.21f,-.74f,-.04f,.07f,.47f,-.07f,-.27f,.71f,.70f,-.55f,.13f,-.50f,.63f,-.51f,.48f,-.19f,.17f,-.34f,-.87f,-.28f,-.71f,.16f,-.56f,.39f,-.14f,.26f,-.09f,-.48f,-.10f,.50f,-.83f,.02f,-.39f,.19f,.10f,.73f,-.43f,-.47f,.79f,.75f,.89f,-.38f,-.72f,-.93f,-.22f,-.00f,.43f,.07f,-.96f,.08f,.63f,-.55f,-.55f,.83f,-.12f,-.70f,.03f,-.51f,.61f,.49f,.95f,.93f,.69f,.88f,-.10f,.50f,-.05f,-.59f,.36f,-.85f,-.02f,-.02f,.75f,-.77f,-.49f,.16f,.75f,-.08f,-.44f,.17f,-.87f,-.19f,-.60f,.03f,-.30f,-.21f,-.58f,.08f,-.47f,-.03f,.39f,.61f,.71f,.10f,.18f,.53f,-.62f,-.72f,.72f,-.87f,-.45f,-.57f,.14f,.65f,-.14f,-.65f,.17f,-.89f,-.35f,.22f,-.82f,-.23f,-.56f,.25f,.82f,.98f,.24f,.76f,-.27f,-.97f,-.14f,-.99f,.95f,.73f,-.37f,-.26f,.73f,-.51f,.42f,-.25f,.76f,.59f,-.54f,.18f,.12f,.41f,.41f,-.45f,.44f,-.26f,-.10f,-.21f,-.33f,.82f,-.88f,.33f,-.83f,-.47f,.08f,.36f,-.59f,-.77f,.70f,-.23f,.72f,.77f,-.99f,.36f,.16f,.62f,.35f,-.99f,.99f,-.32f,.39f,-.09f,.15f,.97f,-.42f,-.51f,-.98f,-.23f,-.81f,-.62f,.36f,-.63f,.33f,-.67f,-.84f,.17f,.26f,-.57f,.77f,-.86f,.54f,-.11f,.97f,-.29f,.45f,-.59f,-.39f,.21f,-.40f,.03f,.36f,.41f,.56f,.76f,.71f,-.62f,-.49f,.99f,.03f,.55f,-.15f,.03f,.62f,-.62f,-.02f,-.53f,.64f,-.24f,.14f,.67f,-.58f,-.90f,.66f,.44f,-.15f,.16f,.92f,.84f,.61f,.45f,.01f,-.23f,.51f,.65f,-.04f,-.61f,-.66f,.76f,-.52f,.63f,.82f,.67f,.81f,-.08f,.14f,.91f,.20f,-.82f,-.90f,.70f,.65f,.87f,.38f,.77f,.52f,.40f,-.57f,.19f,.58f,.50f,.78f,-.09f,-.93f,.67f,.06f,.11f,.36f,-.40f,.37f,.53f,.01f,-.09f,.89f,-.87f,.99f,-.54f,.92f,-.45f,.69f,-.22f,.92f,-.01f,.04f,.68f,-.96f,-.56f,.26f,.24f,.77f,-.31f,.23f,.26f,-.09f,-.12f,-.57f,.35f,-.80f,.36f,.36f,.55f,.85f,-.60f,.12f,-.96f,-.54f,-.58f,-.21f,.62f,.60f,.64f,.70f,-.14f,-.63f,-.14f,.55f,.25f,.21f,-.19f,.42f,-.61f,-.08f,-.38f,-.86f,.08f,.48f,-.58f,.82f,.14f,.50f,.81f,.07f,.24f,.03f,.54f,.47f,-.97f,-.80f,-.76f,-.86f,.80f,-.14f,-.04f,-.54f,.61f,-.42f,-.90f,.94f,-.84f,.15f,-.72f,.52f,.10f,-.48f,.42f,-.63f,.93f,-.80f,-.17f,-.29f,-.87f,-.24f,.99f,.75f,.78f,-.39f,-.94f,.46f,.34f,-.93f,.87f,-.18f,.20f,.33f,.91f,.12f,.62f,-.19f,-.50f,-.20f,.73f,.81f,-.04f,-.26f,.16f,-.24f,.60f,-.69f,.59f,.18f,.02f,-.00f,-.28f,.48f,-.29f,-.97f,.11f,-.34f,-.22f,.74f,.53f,.11f,-.97f,-.50f,-.46f,-.96f,-.93f,-.02f,-.23f,.04f,-.30f,-.79f,-.46f,.03f,-.18f,.71f,.73f,.27f,.76f,.65f,.52f,.34f,.54f,-.36f,.02f,.90f,-.24f,-.44f,-.81f,-.22f,.50f,-.25f,-.94f,.91f,-.57f,-.38f,.95f,-.05f,.46f,-.74f,-.88f,.59f,.86f,-.52f,-.32f,.30f,-.43f,.68f,.69f,-.88f,.92f,-.29f,-.78f,.89f,-.96f,-.91f,-.80f,.14f,-.36f,-.86f,.79f,-.39f,-.81f,.86f,.43f,-.77f,.44f,-.40f,-.91f,-.43f,-.29f,-.23f,-.07f,-.76f,.14f,.88f,.59f,-.36f,-.80f,-.81f,.74f,.49f,-.41f,.10f,-.45f,.89f,.99f,.95f,-.84f,-.19f,-.50f,-.00f,.17f,-.27f,-.14f,-.27f,-.34f,-.22f,.10f,.78f,-.90f,.64f,.98f,-.78f,.06f,.61f,.51f,-.34f,.70f,-.17f,-.65f,.84f,-.56f,-.81f,.26f,-.08f,.84f,.64f,.96f,-.26f,.73f,.68f,.71f,-.53f,-.04f,.32f,.99f,-.73f,.65f,-.57f,.44f,-.66f,-.20f,-.43f,-.54f,-.97f,.41f,.24f,-.53f,-.55f,-.26f,-.71f,.51f,.83f,.53f,.01f,.95f,.45f,-.40f,.31f,.59f,.34f,-.45f,-.92f,-.27f,.54f,-.14f,-.87f,-.60f,.72f,-.70f,-.04f,.18f,.91f,.68f,-.15f,.21f,.86f,.54f,-.86f,.70f,.40f,.64f,-.82f,-.70f,.68f,.96f,.71f,.84f,-.89f,-.47f,-.63f,-.75f,.79f,-.77f,.14f,.02f,.19f,-.84f,-.19f,.63f,.32f,.65f,-.16f,.44f,-.23f,-.01f,.67f,.85f,.08f,.32f,.14f,.70f,.40f,-.03f,-.29f,.78f,-.17f,-.68f,.36f,.28f,-.48f,.65f,.74f,-.37f,.81f,-.95f,-.21f,.04f,-.23f,-.24f,-.27f,-.93f,.19f,-.27f,-.83f,.19f,-.21f,-.08f,-.19f,.73f,.26f,-.97f,.51f,.92f,-.53f,-.18f,-.54f,.89f,-.49f,-.61f,-.54f,-.63f,-.01f,-.77f,.96f,.44f,-.64f,.63f,.24f,-.51f,.67f,-.16f,-.99f,.61f,.73f,.53f,.84f,-.06f,-.39f,.13f,.40f,-.26f,.90f,.94f,.15f,.71f,.70f,-.71f,-.53f,.74f,.47f,-.45f,.45f,-.24f,.44f,.90f,-.71f,.22f,.32f,-.61f,-.78f,-.66f,-.44f,.88f,-.58f,.57f,.08f,.57f,-.61f,.71f,-.12f,.49f,-.41f,-.69f,-.47f,-.46f,.73f,-.19f,-.01f,.44f,-.41f,.74f,.15f,.15f,.10f,.10f,.59f,.16f,-.80f,-.75f,.79f,-.22f,-.75f,-.05f,.22f,-.92f,-.11f,-.86f,.81f,-.29f,-.08f,-.78f,-.65f,.19f,-.35f,-.76f,.91f,-.55f,-.39f,-.23f,.34f,-.07f,-.95f,.10f,.98f,-.12f,-.49f,-.79f,-.00f,.57f,.83f,.21f,.28f,.33f,-.87f,.60f,-.00f,-.82f,-.00f,.61f,.06f,.45f,.65f,.75f,.04f,-.21f,-.92f,.25f,-.11f,-.52f,.79f,.19f,.48f,-.79f,-.26f,.47f,.05f,.98f,.89f,-.58f,-.03f,.22f,.20f,.17f,-.96f,.13f,.92f,-.31f,-.81f,-.30f,-.60f,.30f,.78f,.86f,-.40f,.23f,-.36f,-.35f,-.69f,.93f,.94f,.52f,.38f,.33f,-.97f,-.28f,.80f,.47f,-.27f,.06f,.25f,-.74f,.57f,.04f,-.93f,.88f,.65f,.68f,.95f,-.69f,.58f,-.67f,.73f,.85f,-.35f,-.10f,-.38f,-.57f,.67f,.79f,-.24f,.60f,.89f,-.85f,.20f,.11f,.14f,-.02f,.53f,-.56f,.39f,-.27f,.43f,.43f,-.41f,-.25f,.00f,-.13f,.26f,.08f,.39f,.07f,.04f,.70f,.49f,-.28f,.19f,-.48f,.29f,.52f,-.99f,.40f,.04f,.47f,.54f,.32f,.48f,.66f,.51f,.29f,-.58f,-.93f,.03f,-.23f,.62f,-.33f,-.51f,-.63f,.22f,.47f,.26f,-.04f,-.50f,.25f,-.61f,.92f,-.52f,.97f,-.12f,.98f,-.12f,.20f,-.04f,-.78f,-.69f,-.06f,-.55f,.38f,-.19f,-.35f,.84f,.46f,.68f,.17f,.27f,.32f,.79f,.37f,-.16f,-.39f,.55f,-.49f,-.70f,-.67f,-.53f,.80f,.35f,-.96f,-.06f,.63f,-.50f,-.16f,.35f,.39f,.51f,.65f,-.12f,.24f,-.88f,.08f,.56f,.92f,-.77f,.93f,.79f,-.06f,-.82f,.00f,-.67f,.49f,.83f,.35f,-.47f,-.67f,-.94f,-.23f,.71f,-.06f,-.06f,.98f,-.94f,-.79f,.03f,-.43f,-.64f,-.27f,-.32f,-.78f,.40f,-.39f,.96f,.81f,.36f,-.64f,.41f,.97f,-.08f,.01f,.31f,-.08f,-.85f,-.09f,-.40f,-.91f,-.11f,-.29f,.79f,-.40f,-.20f,-.92f,.72f,-.25f,.40f,-.20f,.47f,-.52f,.86f,.68f,-.68f,.15f,.99f,.30f,-.99f,-.51f,.44f,-.96f,-.71f,.83f,-.37f,-.12f,-.58f,-.11f,.60f,-.96f,-.35f,-.13f,.52f,.99f,-.94f,-.49f,-.88f,-.50f,.11f,-.79f,-.04f,-.03f,-.60f,-.16f,-.93f,-.85f,-.73f,-.47f,-.22f,-.27f,-.52f,.19f,-.39f,.96f,-.23f,.72f,-.19f,-.71f,.00f,-.78f,.55f,.90f,.47f,-.33f,-.02f,.04f,-.66f,-.34f,-.32f,-.34f,.34f,-.93f,.38f,-.43f,.70f,.90f,-.76f,-.16f,.54f,-.30f,.57f,.20f,-.68f,.62f,.03f,.58f,.41f,-.59f,.54f,.96f,-.04f,-.84f,-.34f,-.76f,.89f,-.86f,-.84f,-.22f,-.61f,-.64f,-.25f,.07f,-.37f,-.85f,-.41f,.59f,.11f,-.97f,.68f,.28f,-.42f,-.19f,.79f,-.46f,.42f,.83f,.97f,-.18f,.16f,-.04f,-.22f,.04f,.14f,.96f,.61f,.72f,-.71f,.23f,-.85f,.06f,.20f,-.72f,.31f,.84f,-.57f,-.23f,-.31f,.17f,.39f,.56f,.31f,.46f,-.15f,-.11f,-.86f,.51f,-.43f,-.63f,.65f,.51f,-.54f,-.58f,.13f,.55f,-.77f,-.64f,-.81f,.18f,-.08f,.21f,-.13f,.18f,.64f,.55f,-.73f,-.35f,.39f,-.79f,.68f,.52f,-.68f,.69f,.42f,-.64f,-.98f,.76f,-.56f,.76f,-.06f,-.64f,-.99f,-.16f,.19f,.90f,-.31f,-.98f,-.71f,-.36f,-.19f,-.03f,.98f,-.34f,-.50f,.91f,-.38f,.20f,-.13f,.62f,-.09f,.66f,-.69f,.25f,-.81f,-.20f,-.16f,-.23f,.26f,.69f,.05f,-.88f,-.71f,-.02f,.96f,.56f,.48f,-.50f,-.68f,-.83f,-.17f,.38f,.48f,.04f,-.05f,-.47f,.46f,.85f,.13f,.08f,.39f,.89f,-.36f,.05f,-.60f,-.95f,-.95f,.08f,.76f,.82f,-.62f,.32f,-.09f,.55f,-.26f,.08f,-.88f,-.93f,-.96f,.26f,.08f,.98f,.40f,.34f,-.34f,-.38f,.35f,.68f,.28f,-.58f,.66f,.30f,-.66f,-.25f,.42f,.05f,-.19f,.33f,.47f,.39f,-.03f,.63f,.09f,.65f,.83f,-.67f,-.86f,.72f,-.20f,.35f,-.88f,-.37f,-.52f,-.29f,-.83f,-.02f,-.08f,.25f,-.75f,.42f,-.23f,-.37f,.48f,-.75f,-.11f,.87f,.87f,.03f,.33f,-.26f,-.43f,.65f,.45f,.77f,.99f,-.52f,.97f,-.72f,.74f,.38f,.86f,-.26f,.81f,.94f,.28f,.16f,-.69f,-.50f,-.78f,-.40f,.11f,-.10f,.89f,.88f,.57f,-.75f,-.55f,-.49f,.06f,.18f,-.56f,.18f,.92f,-.63f,.94f,.01f,.39f,.46f,.44f,.90f,.16f,.77f,-.58f,-.69f,.88f,.12f,.10f,.27f,-.89f,-.94f,-.64f,.55f,-.87f,-.70f,-.47f,.93f,.73f,-.87f,-.20f,-.21f,.61f,-.98f,-.67f,-.45f,.82f,.77f,-.38f,-.29f,-.61f,-.78f,.93f,-.74f,-.48f,.51f,.03f,.73f,-.53f,-.55f,.99f,.08f,.99f,.88f,.57f,.04f,-.98f,.12f,-.39f,.47f,-.54f,-.65f,-.29f,-.51f,-.92f,-.23f,.73f,-.76f,.50f,.11f,.38f,-.44f,.04f,-.40f,-.27f,-.95f,-.83f,.77f,-.20f,-.27f,-.36f,.59f,.65f,-.69f,-.53f,-.86f,-.55f,-.68f,.80f,.55f,.69f,-.41f,-.49f,.95f,.99f,.01f,-.17f,-.72f,-.70f,-.05f,-.56f,.92f,-.37f,.00f,-.38f,-.18f,-.63f,-.46f,-.45f,.82f,.54f,.31f,.41f,-.00f,.08f,-.91f,.16f,.35f,-.62f,-.36f,.22f,.26f,-.31f,-.91f,.48f,.41f,.87f,-.34f,.69f,.24f,-.83f,.90f,.76f,.17f,.11f,.85f,.03f,-.66f,-.29f,.58f,.31f,-.87f,-.90f,-.48f,.21f,-.34f,-.87f,.60f,.91f,-.85f,-.59f,.20f,.64f,-.55f,-.20f,.77f,-.70f,.36f,.44f,-.66f,-.12f,-.56f,-.47f,.37f,-.45f,.43f,.93f,.76f,.03f,-.42f,-.12f,.91f,-.25f,-.93f,-.71f,.00f,.32f,.41f,-.72f,.38f,-.29f,-.83f,.71f,-.73f,-.68f,.81f,.83f,.08f,.74f,-.37f,-.17f,.01f,-.97f,-.35f,-.79f,.80f,-.27f,.28f,-.50f,-.55f,.70f,.36f,.68f,-.25f,-.08f,.84f,-.28f,-.92f,.54f,-.50f,.82f,.23f,-.01f,.09f,.61f,.42f,-.48f,.01f,.56f,.96f,.28f,.67f,-.48f,-.98f,-.56f,-.05f,-.55f,.91f,.77f,-.98f,.55f,.89f,.56f,.58f,.42f,.87f,.40f,.28f,-.51f,.74f,.99f,.77f,.55f,.48f,.58f,-.28f,-.16f,.69f,.19f,-.15f,-.67f,-.89f,-.63f,-.49f,.69f,.14f,.41f,.37f,.96f,.11f,-.89f,-.19f,.83f,.67f,-.51f,.17f,-.79f,.53f,-.27f,-.41f,-.93f,-.01f,.31f,.17f,.20f,.79f,.84f,.90f,-.02f,.59f,-.77f,-.47f,.59f,-.99f,-.88f,.78f,.43f,.90f,.08f,.71f,-.12f,.91f,-.82f,-.86f,-.61f,.28f,.35f,.60f,.55f,-.60f,-.67f,.67f,.99f,-.30f,.24f,.47f,-.17f,-.82f,-.22f,.40f,.01f,-.75f,.45f,.38f,.70f,-.44f,.96f,.72f,.12f,-.58f,.96f,-.41f,-.56f,-.49f,-.95f,-.36f,.62f,-.10f,-.65f,-.51f,-.13f,-.44f,.21f,.75f,-.82f,.75f,-.76f,.20f,-.63f,-.95f,-.10f,-.30f,.85f,.59f,-.76f,-.36f,.75f,-.05f,-.47f,-.67f,-.84f,-.24f,-.22f,.24f,.88f,-.33f,-.60f,-.81f,-.22f,-.55f,-.10f,.84f,-.52f,.31f,.31f,.73f,-.47f,.08f,-.78f,.59f,.27f,.06f,.74f,-.58f,.48f,-.74f,-.60f,-.57f,.16f,-.91f,.48f,-.08f,.42f,.56f,-.31f,-.42f,-.33f,.88f,.84f,.72f,.92f,.36f,.95f,-.40f,.77f,.12f,-.63f,-.67f,-.02f,-.65f,.53f,-.98f,.23f,-.92f,-.88f,-.64f,.84f,.44f,-.08f,-.62f,-.85f,-.16f,.61f,-.11f,.23f,.01f,-.37f,-.36f,.42f,-.73f,.43f,.96f,-.67f,.41f,.94f,-.27f,.04f,-.46f,.26f,.01f,.75f,-.10f,-.41f,-.28f,.57f,-.55f,.18f,-.36f,-.96f,-.15f,-.04f,.80f,.09f,-.84f,.06f,-.63f,.82f,.47f,-.17f,.01f,-.81f,.44f,.19f,-.26f,.88f,.57f,-.14f,-.00f,.34f,.44f,-.20f,-.58f,-.32f,.17f,-.98f,-.29f,-.08f,-.53f,.89f,-.96f,-.43f,.87f,.90f,.25f,-.00f,.55f,.73f,-.28f,.63f,-.73f,.61f,.14f,.51f,-.15f,.90f,.15f,-.24f,.48f,-.74f,.54f,.75f,-.59f,-.22f,.54f,-.99f,.29f,.47f,.87f,-.05f,.65f,.46f,.76f,-.89f,.63f,.65f,-.28f,-.63f,.54f,.37f,.03f,.05f,-.16f,-.38f,.60f,.70f,-.74f,.86f,-.41f,.22f,-.45f,.75f,-.30f,.32f,-.03f,.42f,.48f,.01f,-.79f,-.87f,-.76f,-.51f,-.02f,.00f,.40f,.35f,.78f,-.89f,-.66f,-.15f,-.41f,-.28f,.41f,.37f,.47f,-.56f,.27f,.58f,-.44f,.73f,-.55f,-.54f,.87f,-.67f,-.62f,-.95f,.52f,-.20f,.81f,-.34f,-.48f,.49f,-.23f,.62f,.74f,-.60f,-.39f,-.60f,.34f,-.81f,.17f,.63f,.37f,.11f,-.74f,.60f,-.47f,-.96f,-.42f,-.54f,.20f,.85f,.13f,.83f,.17f,-.56f,-.25f,.69f,-.11f,.11f,.09f,-.83f,-.02f,-.44f,-.50f,-.00f,-.35f,.25f,-.41f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.74480000723600514600519773011910729110240936279296875f,0.0f,0.0f,-3.4616000152021637603638737346045672893524169921875f,0.0f,0.0f,0.57800001569837256010231385516817681491374969482421875f,0.0f,0.0f,2.401799962368607044282953211222775280475616455078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,5.0120000527910892884619897813536226749420166015625f,0.0f,0.0f,-2.6072000677764410880854484275914728641510009765625f,0.0f,0.0f,2.103600078852475352420015042298473417758941650390625f,0.0f,0.0f,2.891499984528869138244999703601934015750885009765625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6195000891871742521033183948020450770854949951171875f,0.0f,0.0f,2.654699988092482865198462604894302785396575927734375f,0.0f,0.0f,-1.245700045715273152069357820437289774417877197265625f,0.0f,0.0f,-0.29900000201910603703225888239103369414806365966796875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-4.87460006688684277520451360032893717288970947265625f,0.0f,0.0f,-2.417099984595178607804655257496051490306854248046875f,0.0f,0.0f,-2.6775000489875662168515191297046840190887451171875f,0.0f,0.0f,-0.27509988536834750760817769332788884639739990234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.008999969162046428383661123007186688482761383056640625f,0.0f,0.0f,-1.4220999077424403278513409532024525105953216552734375f,0.0f,0.0f,1.1591000059261895582807255777879618108272552490234375f,0.0f,0.0f,-1.4597000310823295965434454046771861612796783447265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7447000244140615787813430870301090180873870849609375f,0.0f,0.0f,-1.149800001461804388469545301632024347782135009765625f,0.0f,0.0f,4.483400066940486539124322007410228252410888671875f,0.0f,0.0f,3.957099976548552167088246278581209480762481689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.07550003752857481487126278807409107685089111328125f,0.0f,0.0f,1.433800079402326677069368088268674910068511962890625f,0.0f,0.0f,-0.83460010019093733291128955897875130176544189453125f,0.0f,0.0f,-1.830499976545572504704750826931558549404144287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.9072000223278990915787289850413799285888671875f,0.0f,0.0f,0.3490999449059355441704610711894929409027099609375f,0.0f,0.0f,-0.41279993468224773778274538926780223846435546875f,0.0f,0.0f,0.1874000287823400778819404877140186727046966552734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.15309992777705172528612820315174758434295654296875f,0.0f,0.0f,-0.35900007095560493297625725972466170787811279296875f,0.0f,0.0f,1.132900054054706284745179800665937364101409912109375f,0.0f,0.0f,-1.0078998915947974257534269781899638473987579345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.64930002137720566057765836376347579061985015869140625f,0.0f,0.0f,0.013600060747565034802164518623612821102142333984375f,0.0f,0.0f,1.5095000211633735620608831595745868980884552001953125f,0.0f,0.0f,0.7727999731086196977258850893122144043445587158203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,3.862799977001547091504107811488211154937744140625f,0.0f,0.0f,-1.16960000540092590171070696669630706310272216796875f,0.0f,0.0f,-0.1703001009233287010857793575269170105457305908203125f,0.0f,0.0f,2.1516999686129398838829729356802999973297119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.27430001754015709281020463095046579837799072265625f,0.0f,0.0f,-2.664799979408085217613688655546866357326507568359375f,0.0f,0.0f,1.161000036928801382174469836172647774219512939453125f,0.0f,0.0f,0.93779998977929412973253420204855501651763916015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.09559994495064305652931579970754683017730712890625f,0.0f,0.0f,0.41139992060735774970225975266657769680023193359375f,0.0f,0.0f,1.2544999432303001185573521070182323455810546875f,0.0f,0.0f,0.8087001234956101569650854798965156078338623046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.4467999047756183728097312268801033496856689453125f,0.0f,0.0f,0.18299998123943783667755269561894237995147705078125f,0.0f,0.0f,-1.8758000357642767852439646958373486995697021484375f,0.0f,0.0f,-1.72270000787004651243705666274763643741607666015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0061999196290967173439412363222800195217132568359375f,0.0f,0.0f,-0.90169991988614095479448451442294754087924957275390625f,0.0f,0.0f,0.2774999884702264996150233855587430298328399658203125f,0.0f,0.0f,0.5610999717503781791805295142694376409053802490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0420000529587258863983834089594893157482147216796875f,0.0f,0.0f,-2.4282000636711718044580265996046364307403564453125f,0.0f,0.0f,-2.6976001530118285387516152695752680301666259765625f,0.0f,0.0f,-2.8455998945102098929282874451018869876861572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.33319996834099097071657524793408811092376708984375f,0.0f,0.0f,3.463000037882476345174609377863816916942596435546875f,0.0f,0.0f,0.526300018568337524271782967844046652317047119140625f,0.0f,0.0f,0.40109999153167008589804254370392300188541412353515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0253999589852986762394948527798987925052642822265625f,0.0f,0.0f,2.5893000707000499716059493948705494403839111328125f,0.0f,0.0f,2.36389996059835016950501085375435650348663330078125f,0.0f,0.0f,-1.4803999364845454866923546433099545538425445556640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.05269996346458949432189911021851003170013427734375f,0.0f,0.0f,1.4492998917743558084936239538365043699741363525390625f,0.0f,0.0f,-0.322900004154441422343779777293093502521514892578125f,0.0f,0.0f,-1.44899998935311913328405353240668773651123046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-3.066000056877732138360670433030463755130767822265625f,0.0f,0.0f,-2.95550003227591417953590280376374721527099609375f,0.0f,0.0f,1.145799902547894877358203302719630300998687744140625f,0.0f,0.0f,1.1602999744497231038309337236569263041019439697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.61740002047493991454985007294453680515289306640625f,0.0f,0.0f,-0.26230001286119353220982475249911658465862274169921875f,0.0f,0.0f,-0.1436000861540434048180259196669794619083404541015625f,0.0f,0.0f,-3.2511999930545680825844101491384208202362060546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7321000249028200013157174907973967492580413818359375f,0.0f,0.0f,0.351599893943965735587653398397378623485565185546875f,0.0f,0.0f,-0.3417000870212927310376471723429858684539794921875f,0.0f,0.0f,3.112599975538998808843871302087791264057159423828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0242998404957344149579512304626405239105224609375f,0.0f,0.0f,-1.3328000001355995163265788505668751895427703857421875f,0.0f,0.0f,-0.7884999199360607224917885105242021381855010986328125f,0.0f,0.0f,-2.088800048333404379974354014848358929157257080078125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.692099959393591479539509236929006874561309814453125f,0.0f,0.0f,-0.13609988062977862455227295868098735809326171875f,0.0f,0.0f,-2.689800033573806370412739852326922118663787841796875f,0.0f,0.0f,1.988400114791838202421558889909647405147552490234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.155999961696565936364322624285705387592315673828125f,0.0f,0.0f,-2.18379999200701746531194658018648624420166015625f,0.0f,0.0f,-1.36070010330826018929428755654953420162200927734375f,0.0f,0.0f,-3.19259999470561606216278960346244275569915771484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.6901000172249975861404891475103795528411865234375f,0.0f,0.0f,-1.717600004689394221912834836984984576702117919921875f,0.0f,0.0f,-1.1741000477984531169539650363731198012828826904296875f,0.0f,0.0f,-1.1488999695204196171971489093266427516937255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.854699993233381416501970306853763759136199951171875f,0.0f,0.0f,3.079799990844727997085783499642275273799896240234375f,0.0f,0.0f,-1.3699999982863657610465679681510664522647857666015625f,0.0f,0.0f,-0.35720003778785525128114386461675167083740234375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.566400068724901561978413155884481966495513916015625f,0.0f,0.0f,1.992699959459900416192112970747984945774078369140625f,0.0f,0.0f,-1.09749999430030609204322900041006505489349365234375f,0.0f,0.0f,1.0127999188683929343568479453097097575664520263671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.626100161926448350868668057955801486968994140625f,0.0f,0.0f,0.26299994640797386136199520478839986026287078857421875f,0.0f,0.0f,-0.419900023250281861919575021602213382720947265625f,0.0f,0.0f,2.3448999638691550018165798974223434925079345703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3119999258704491129634561730199493467807769775390625f,0.0f,0.0f,-2.411300049860773953014358994551002979278564453125f,0.0f,0.0f,1.2289000314906253219504606022383086383342742919921875f,0.0f,0.0f,-2.34050004801526512210330110974609851837158203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.227300072503091588060897265677340328693389892578125f,0.0f,0.0f,1.85010001847296745580706556211225688457489013671875f,0.0f,0.0f,-3.644099951536952541886194012477062642574310302734375f,0.0f,0.0f,-1.5583999700829382906164255473413504660129547119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.53550007679313438302415306679904460906982421875f,0.0f,0.0f,-3.786000020816922440047846976085565984249114990234375f,0.0f,0.0f,1.566700057405233881269168705330230295658111572265625f,0.0f,0.0f,-1.8377000847041597975106697049341164529323577880859375f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_4d_4items_strided_dilated_padded_same)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{4,3,8,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{4,3,2,2,3,1};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{4,4,4,7,4,5};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{3,2,2,3},        // move_strides
                                                                  Strides{2,1,3,2},        // filter_dilation
                                                                  CoordinateDiff{3,3,3,3}, // below_pads
                                                                  CoordinateDiff{3,3,3,3}, // above_pads
                                                                  Strides{1,1,1,1}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f,.61f,.17f,-.91f,-.48f,-.48f,.12f,-.18f,.08f,-.16f,.54f,-.20f,-.08f,.19f,-.81f,.56f,-.52f,.32f,-.37f,-.17f,-.46f,-.34f,.85f,.11f,-.10f,.98f,-.22f,.68f,.97f,-.93f,-.92f,.32f,-.19f,-.61f,.93f,.54f,-.75f,.63f,.38f,.69f,-.02f,.19f,-.38f,-.44f,.49f,-.62f,-.41f,-.84f,.93f,-.74f,-.57f,.37f,-.86f,-.01f,.58f,.13f,-.42f,.68f,-.05f,.17f,.09f,-.67f,.11f,-.32f,.08f,-.29f,.60f,.69f,.04f,.85f,.46f,.41f,-.78f,-.98f,.50f,-.96f,-.61f,-.58f,.00f,.81f,-.25f,-.74f,-.46f,-.16f,.32f,.56f,-.05f,-.87f,.59f,.08f,-.39f,.67f,.95f,.70f,.00f,-.58f,.85f,-.14f,-.27f,-.63f,-.75f,.61f,-.12f,-.68f,-.83f,.12f,.94f,-.46f,.27f,-.14f,-.96f,.16f,-.57f,.92f,-.24f,.08f,.60f,.23f,-.14f,-.28f,.14f,-.19f,.00f,-.57f,.18f,.42f,.17f,.71f,-.55f,.89f,.26f,-.67f,.68f,-.75f,-.62f,.62f,-.16f,-.69f,-.96f,-.24f,.00f,.63f,-.95f,-.99f,.06f,.02f,-.04f,.95f,.98f,.13f,.96f,-.22f,-.25f,-.77f,-.02f,-.46f,.28f,-.61f,-.56f,-.65f,-.23f,-.41f,.78f,-.46f,.44f,.15f,.46f,-.90f,-.25f,.81f,.28f,.40f,.56f,.62f,-.63f,-.86f,-.97f,.22f,-.58f,.05f,.53f,-.38f,.79f,.60f,-.63f,-.18f,.62f,-.37f,.48f,.86f,.70f,-.52f,-.39f,-.44f,-.69f,.42f,-.69f,.48f,.70f,-.86f,-.90f,.69f,.08f,-.34f,-.86f,.66f,.61f,.47f,-.22f,-.51f,-.29f,.95f,-.30f,-.41f,.85f,-.13f,.70f,-.51f,-.05f,-.50f,-.28f,-.44f,.33f,-.08f,-.88f,-.64f,.24f,-.51f,.67f,-.40f,.48f,.92f,-.64f,-.42f,-.62f,-.77f,.34f,.29f,.49f,.82f,-.30f,-.40f,.27f,.61f,-.75f,.42f,.69f,.46f,.41f,.48f,.14f,.86f,-.46f,-.88f,-.20f,-.09f,-.10f,.70f,.97f,-.99f,.53f,-.51f,-.70f,-.96f,.86f,-.44f,-.66f,-.28f,.08f,.42f,-.45f,-.73f,-.99f,-.33f,.40f,-.86f,-.84f,-.06f,.77f,.96f,-.06f,.24f,.21f,.27f,-.21f,-.01f,.16f,.40f,.06f,-.61f,-.66f,.76f,.43f,.00f,.37f,.70f,-.76f,.50f,-.38f,-.24f,-.80f,-.66f,.09f,-.70f,.83f,.59f,-.39f,-.21f,.84f,-.17f,.50f,-.48f,-.07f,-.54f,-.16f,.99f,.28f,.29f,-.68f,-.49f,-.18f,-.41f,.95f,.72f,-.26f,-.42f,-.27f,.38f,-.47f,.24f,-.38f,.29f,.34f,-.30f,-.64f,-.27f,-.27f,-.94f,.26f,.52f,.92f,.53f,-.80f,.31f,.29f,.16f,-.47f,.61f,.46f,-.75f,-.88f,.95f,.33f,-.97f,-.03f,.43f,.14f,.73f,.97f,-.01f,.45f,.81f,.77f,.45f,.16f,-.66f,-.29f,-.23f,.32f,.22f,-.09f,.65f,-.46f,.65f,-.01f,.61f,-.12f,.36f,.29f,-.87f,-.01f,-.28f,-.95f,.37f,.73f,-.99f,.47f,-.69f,-.31f,.67f,-.78f,-.77f,-.41f,.95f,.61f,.52f,.85f,-.88f,.97f,.54f,.53f,.87f,-.64f,-.62f,.70f,-.11f,.93f,-.52f,-.08f,.03f,.27f,-.30f,-.62f,.00f,.76f,.12f,-.83f,-.60f,-.95f,.51f,.85f,-.27f,-.83f,-.20f,-.45f,.27f,-.09f,.30f,-.34f,-.38f,-.52f,-.43f,-.73f,-.60f,-.89f,.15f,-.12f,.95f,.97f,-.43f,.85f,.43f,-.35f,-.44f,.84f,.94f,.80f,-.88f,-.73f,-.24f,.93f,.77f,.15f,-.63f,.84f,.70f,-.24f,-.01f,-.63f,.44f,.50f,-.29f,-.27f,.64f,.28f,-.18f,.90f,-.87f,-.86f,-.24f,.66f,-.49f,.86f,.19f,.56f,-.18f,-.25f,-.32f,-.09f,.83f,.25f,.92f,.15f,.71f,-.91f,.86f,-.70f,.05f,-.23f,.19f,.32f,.74f,-.79f,-.82f,.44f,-.91f,-.61f,-.99f,.61f,-.19f,.14f,-.46f,-.33f,.93f,.29f,.68f,-.45f,.47f,.19f,-.48f,.03f,-.10f,.73f,-.99f,.72f,-.58f,.45f,-.83f,.07f,.58f,.02f,.60f,.71f,.75f,-.36f,.19f,.92f,-.90f,-.66f,.57f,.38f,.86f,.61f,.31f,.47f,-.14f,-.15f,.27f,.48f,.44f,-.45f,-.83f,.11f,.96f,-.38f,-.12f,-.72f,.07f,-.10f,-.62f,.21f,-.33f,.18f,.18f,-.50f,.15f,.38f,.29f,-.67f,-.16f,-.61f,.36f,-.50f,-.87f,.41f,.41f,.03f,.71f,.75f,-.02f,.13f,.93f,.51f,-.86f,-.43f,-.97f,.16f,-.95f,-.21f,-.15f,-.43f,.53f,.17f,.18f,.12f,-.76f,.56f,-.71f,.25f,-.03f,-.95f,.65f,.11f,.29f,.14f,.04f,.02f,-.20f,-.39f,.23f,.59f,.07f,.73f,-.75f,.67f,.71f,-.11f,-.30f,.26f,-.93f,-.01f,.18f,-.04f,-.73f,-.61f,-.07f,.00f,-.68f,-.58f,-.95f,-.51f,.68f,-.87f,.07f,.21f,.87f,.65f,-.89f,.57f,-.48f,.53f,-.35f,-.61f,-.55f,.62f,.58f,-.68f,-.52f,.33f,.02f,.74f,.03f,.44f,.84f,-.90f,-.02f,-.54f,.55f,-.24f,.63f,.31f,-.67f,.51f,.67f,.85f,.68f,.61f,.04f,.09f,.70f,.35f,-.06f,-.06f,-.49f,.81f,-.24f,-.70f,.44f,.18f,.12f,.17f,.28f,.26f,.81f,-.24f,-.64f,.32f,.49f,-.06f,.13f,.56f,-.17f,-.28f,-.50f,.89f,-.20f,-.34f,.05f,.71f,-.09f,.02f,-.43f,-.00f,-.07f,.25f,.80f,.18f,-.37f,-.16f,-.42f,.33f,.76f,-.81f,.79f,.88f,-.62f,-.49f,-.79f,.16f,.19f,.37f,-.62f,-.72f,-.37f,.39f,-.84f,-.13f,.21f,.88f,-.80f,-.06f,.28f,-.13f,-.83f,-.91f,-.39f,.19f,-.71f,-.08f,.96f,.89f,-.60f,-.79f,-.13f,-.00f,.03f,.97f,.70f,-.96f,-.50f,-.39f,.13f,.09f,.29f,-.43f,-.40f,-.83f,.51f,-.32f,-.53f,.07f,.35f,.43f,-.65f,-.94f,.40f,-.59f,.80f,-.15f,-.22f,.56f,.90f,-.82f,-.74f,.04f,-.24f,-.97f,.96f,.78f,.29f,-.44f,.62f,.92f,.05f,-.74f,-.49f,-.25f,.34f,-.99f,.78f,-.64f,.09f,-.51f,.99f,-.20f,.95f,.13f,-.44f,-.90f,-.82f,.84f,.26f,-.90f,.97f,-.09f,.45f,-.49f,.29f,.13f,.37f,-.03f,.41f,-.47f,.25f,.44f,.31f,.72f,.09f,.67f,.76f,-.79f,-.99f,.62f,-.71f,-.07f,.63f,-.02f,-.28f,-.02f,.98f,.28f,.48f,-.49f,.32f,-.11f,-.45f,.37f,-.58f,-.82f,-.23f,-.55f,-.61f,-.87f,-.96f,-.34f,-.03f,-.15f,-.15f,-.30f,-.53f,.83f,.20f,-.19f,-.19f,.23f,.45f,.50f,-.67f,.71f,-.97f,.42f,-.33f,-.88f,.56f,.78f,-.09f,.17f,.77f,.95f,-.80f,-.72f,.23f,.39f,-.72f,-.15f,-.79f,-.97f,.97f,-.20f,-.56f,-.95f,-.65f,.71f,-.42f,.10f,-.58f,-.14f,.39f,-.95f,-.71f,-.67f,.62f,-.10f,-.11f,-.76f,-.30f,-.62f,.63f,-.11f,-.48f,.37f,-.08f,.73f,-.84f,-.07f,-.82f,-.93f,.63f,.17f,-.58f,-.17f,.83f,.78f,-.49f,-.41f,.29f,-.90f,.78f,-.97f,-.67f,.47f,.64f,-.57f,.89f,.29f,-.24f,.75f,.61f,.45f,.42f,.16f,-.75f,.31f,.66f,-.04f,.11f,-.22f,-.63f,-.81f,.48f,.40f,-.43f,.37f,.58f,.18f,.27f,-.84f,.31f,.67f,-.23f,.74f,.87f,-.60f,-.90f,-.43f,.07f,-.58f,-.01f,.06f,-.57f,-.48f,-.80f,.30f,-.69f,-.37f,.45f,.85f,-.17f,.84f,-.86f,.36f,.30f,.49f,.19f,-.40f,-.68f,-.49f,.61f,-.07f,.25f,-.84f,.42f,-.23f,-.28f,.86f,-.37f,.13f,-.29f,-.93f,-.28f,.23f,.25f,.86f,.33f,.75f,.75f,-.61f,.43f,.09f,.24f,-.80f,-.34f,-.64f,.25f,-.07f,.78f,.29f,.12f,.28f,-.72f,-.72f,-.70f,.23f,-.30f,-.07f,-.94f,.97f,.08f,-.97f,.20f,.70f,.38f,.92f,-.91f,-.93f,-.13f,.05f,-.56f,.15f,-.45f,-.39f,-.53f,.25f,.74f,.85f,.91f,.25f,.75f,.32f,.86f,.99f,.84f,.17f,.38f,.16f,.26f,.37f,.41f,-.36f,-.00f,-.09f,.70f,-.40f,.15f,.76f,-.17f,.40f,-.06f,.14f,.50f,-.91f,.44f,.06f,.30f,.86f,.45f,-.88f,-.06f,.92f,-.39f,.83f,.04f,-.04f,.70f,.99f,-.96f,.02f,-.45f,.47f,.71f,-.50f,.08f,.98f,.30f,-.60f,-.66f,.66f,-.51f,-.86f,.54f,.51f,-.36f,-.48f,.59f,-.93f,.64f,.49f,.23f,-.72f,.82f,-.81f,-.51f,-.06f,.58f,.45f,-.41f,-.60f,-.08f,-.07f,.47f,-.48f,.85f,-.36f,-.92f,.48f,-.36f,-.09f,.09f,-.57f,-.98f,.67f,-.53f,.50f,-.12f,-.62f,-.85f,.37f,-.98f,.56f,.65f,.51f,.54f,.73f,-.89f,.07f,.52f,.92f,-.26f,-.93f,-.82f,.75f,.96f,.26f,-.21f,-.40f,-.88f,-.38f,-.21f,-.43f,-.77f,.48f,-.95f,.40f,.43f,-.24f,-.72f,.49f,-.78f,.82f,-.21f,-.15f,-.56f,-.94f,-.51f,-.10f,.36f,.68f,-.78f,.09f,-.15f,-.55f,.21f,-.11f,.53f,-.23f,-.85f,-.50f,.66f,.44f,.47f,.61f,.22f,.79f,.71f,-.91f,.70f,-.12f,-.78f,.45f,-.40f,.82f,.27f,.39f,.87f,-.23f,-.37f,-.78f,-.19f,.45f,.29f,-.45f,-.30f,.17f,-.70f,.09f,-.85f,-.67f,.41f,.37f,-.12f,.83f,.92f,.70f,-.23f,.62f,.84f,.13f,-.36f,.75f,-.47f,-.93f,.00f,-.71f,.22f,-.13f,-.19f,-.03f,-.94f,-.56f,-.88f,-.27f,-.48f,.98f,.73f,.87f,.10f,.04f,.34f,.51f,-.95f,.84f,.17f,.13f,-.39f,.37f,.89f,-.04f,-.53f,-.38f,.00f,-.00f,.72f,.08f,.32f,.69f,.80f,.00f,-.12f,-.79f,-.75f,.04f,-.45f,.08f,-.96f,-.95f,-.44f,.98f,.10f,-.28f,.99f,-.07f,.53f,-.86f,-.01f,.29f,-.78f,-.01f,.22f,.18f,-.24f,-.89f,.12f,-.67f,.06f,-.02f,-.41f,-.00f,.91f,.40f,-.20f,.25f,-.08f,.04f,.42f,.40f,.39f,-.22f,-.23f,.28f,.48f,-.72f,-.85f,-.22f,.69f,-.25f,-.85f,-.56f,.80f,-.91f,.69f,-.69f,.86f,.94f,.87f,-.47f,.30f,-.51f,-.52f,.74f,-.43f,.03f,-.79f,.76f,.39f,.00f,.32f,-.43f,-.43f,-.27f,.08f,-.20f,.18f,.01f,-.12f,-.22f,-.64f,-.56f,.72f,-.10f,-.52f,-.23f,.93f,.39f,.14f,.23f,.81f,-.37f,.52f,-.10f,-.16f,-.90f,.35f,.18f,-.89f,.72f,.91f,-.03f,-.67f,.43f,.97f,-.32f,-.40f,.85f,-.25f,.71f,-.60f,.71f,.84f,.20f,-.97f,-.66f,.18f,.71f,-.29f,-.73f,.62f,.76f,-.12f,-.80f,.61f,.38f,.74f,.44f,.60f,.23f,.11f,-.06f,-.09f,-.33f,-.38f,.17f,.57f,.91f,.83f,.89f,.88f,.83f,.71f,.97f,.93f,.23f,-.36f,-.21f,-.77f,.14f,.52f,-.48f,-.21f,-.86f,-.39f,.68f,.46f,.48f,.09f,-.10f,.83f,.63f,.53f,.40f,.05f,-.92f,.61f,.23f,.23f,-.49f,-.12f,-.88f,-.03f,.50f,-.27f,.44f,-.59f,-.50f,.93f,-.52f,-.06f,-.02f,.57f,.75f,.56f,.34f,-.71f,-.04f,-.25f,.81f,.77f,-.40f,.69f,.26f,.86f,.32f,.69f,-.64f,-.75f,.73f,-.40f,-.57f,.95f,.03f,-.51f,.15f,-.25f,.87f,.23f,-.61f,-.37f,-.44f,-.04f,-.84f,-.07f,.27f,-.33f,.68f,-.59f,.32f,.06f,-.80f,-.47f,.09f,-.61f,-.85f,-.77f,.78f,.86f,-.75f,-.45f,-.10f,-.68f,-.30f,-.40f,.83f,.62f,.97f,-.71f,.50f,-.15f,-.06f,.99f,.68f,-.45f,.81f,-.82f,.87f,-.65f,-.68f,-.97f,.68f,.64f,.19f,.23f,.51f,-.96f,.16f,.05f,.82f,-.70f,.08f,-.13f,.80f,.87f,.47f,.27f,.38f,.27f,-.61f,-.63f,.60f,.91f,-.03f,-.65f,-.26f,-.59f,-.08f,-.00f,.55f,-.30f,.28f,-.81f,.32f,-.92f,-.34f,.55f,.25f,-.46f,.93f,.34f,.69f,-.59f,.79f,.95f,.97f,.91f,-.53f,.89f,.29f,-.23f,-.49f,.14f,-.53f,.22f,.87f,.10f,.42f,.31f,.98f,-.18f,-.89f,-.98f,-.20f,.31f,-.54f,.32f,-.14f,.16f,-.24f,-.77f,.15f,.76f,.80f,.70f,.93f,-.55f,.25f,.39f,.16f,-.80f,-.64f,-.51f,-.48f,.75f,.50f,.19f,-.42f,-.42f,.75f,.06f,.61f,-.75f,.65f,.27f,-.72f,-.20f,.81f,.30f,-.34f,-.38f,-.56f,-.15f,-.37f,.38f,.83f,-.76f,-.00f,-.95f,.99f,-.51f,-.06f,-.47f,.92f,.03f,.52f,-.67f,.73f,.59f,-.32f,-.97f,.41f,-.37f,.25f,.77f,-.40f,-.01f,-.95f,.77f,-.31f,-.99f,-.23f,-.38f,-.77f,-.15f,-.75f,-.79f,.85f,-.56f,-.80f,.15f,.75f,.69f,-.70f,-.72f,-.83f,.16f,.83f,.95f,-.33f,-.16f,-.56f,-.36f,-.94f,.37f,.51f,-.95f,.92f,-.86f,.88f,-.63f,.77f,.12f,.54f,.99f,.08f,.38f,-.42f,.86f,-.13f,.45f,.75f,.10f,.56f,.78f,.06f,-.93f,.77f,-.47f,.86f,-.34f,-.34f,.61f,.12f,-.56f,.64f,.58f,-.56f,.32f,.01f,-.35f,.10f,.59f,-.40f,.19f,-.02f,-.53f,.87f,-.31f,-.39f,.80f,-.53f,.84f,.69f,.34f,.46f,.61f,-.38f,.85f,-.38f,-.90f,-.57f,.74f,.21f,.13f,-.87f,-.57f,-.81f,-.75f,-.14f,.99f,-.49f,-.65f,.61f,-.58f,-.04f,-.63f,-.96f,-.75f,-.67f,-.72f,-.33f,.50f,-.01f,.64f,.97f,-.08f,-.34f,-.84f,.38f,-.16f,.42f,.96f,.80f,-.53f,.53f,.51f,.77f,-.04f,-.06f,.65f,.72f,.82f,.77f,-.53f,-.66f,-.43f,.95f,.20f,.14f,-.27f,-.13f,.39f,-.77f,.07f,-.14f,.93f,.69f,-.03f,.89f,.51f,-.40f,-.68f,.42f,-.46f,-.50f,.15f,-.81f,.32f,-.10f,.26f,-.10f,.35f,-.33f,.70f,-.35f,.79f,.97f,.01f,-.77f,-.44f,-.24f,-.93f,.21f,-.74f,-.64f,-.25f,-.90f,-.26f,.64f,-.33f,.82f,-.12f,.56f,-.18f,-.78f,-.82f,-.21f,-.84f,-.81f,-.26f,.20f,.74f,-.56f,.86f,-.17f,-.36f,-.02f,-.16f,.21f,.61f,.65f,-.48f,.04f,-.38f,.32f,-.72f,.22f,-.16f,.44f,.25f,-.51f,-.64f,.43f,.72f,-.88f,.97f,-.27f,.40f,.36f,.49f,.38f,-.53f,-.08f,-.06f,.69f,-.27f,.05f,.72f,-.48f,.74f,.80f,.23f,.00f,-.49f,-.85f,-.30f,-.43f,.14f,-.06f,.03f,-.83f,.91f,-.72f,.27f,.42f,-.63f,-.54f,.92f,-.87f,-.49f,.40f,.60f,-.24f,.66f,-.65f,-.67f,.15f,.70f,.89f,-.13f,.81f,-.52f,-.82f,-.58f,-.20f,.08f,-.51f,.18f,.99f,-.80f,.22f,-.17f,-.72f,-.72f,.39f,.81f,-.36f,.92f,-.63f,.36f,.21f,.00f,-.27f,-.72f,-.47f,-.69f,.93f,-.60f,.98f,-.28f,-.69f,.24f,-.32f,-.13f,-.37f,.66f,-.68f,.82f,.23f,-.45f,.19f,-.02f,.90f,-.77f,-.87f,.94f,-.53f,.95f,.90f,.17f,.80f,-.86f,.67f,.58f,-.27f,-.87f,-.15f,.99f,-.65f,.08f,.34f,-.46f,-.94f,.78f,.10f,-.95f,.54f,.46f,.89f,.01f,-.33f,.43f,-.19f,-.87f,.36f,.84f,.70f,-.93f,-.95f,.86f,-.33f,-.60f,.31f,.70f,-.39f,.01f,-.43f,-.11f,-.42f,-.76f,.72f,.33f,.40f,.43f,-.19f,-.49f,.24f,-.45f,-.02f,-.12f,-.50f,-.74f,-.53f,.09f,.39f,.53f,.02f,.14f,.40f,-.81f,-.28f,-.32f,-.00f,.51f,.54f,-.12f,-.30f,.97f,-.04f,-.42f,-.96f,-.39f,.81f,.93f,-.55f,.46f,-.79f,-.99f,-.53f,.00f,-.28f,.08f,.22f,.34f,.03f,-.89f,.36f,.80f,-.68f,.50f,-.08f,.04f,.98f,-.36f,-.97f,.46f,-.02f,-.20f,.66f,-.67f,.58f,.46f,-.14f,-.74f,-.68f,-.22f,.93f,-.86f,-.58f,.66f,-.03f,.40f,.60f,-.65f,.20f,.96f,-.04f,.91f,.95f,-.19f,.04f,.89f,.70f,-.99f,.27f,-.55f,.79f,.99f,.94f,-.16f,.16f,.74f,-.84f,-.91f,-.15f,.70f,.95f,-.19f,-.72f,.23f,.98f,-.77f,-.58f,-.97f,.20f,-.89f,.94f,-.96f,.78f,.17f,.51f,.91f,-.58f,-.90f,-.48f,.03f,.06f,-.58f,.67f,.70f,-.68f,.13f,-.02f,.04f,-.33f,.64f,-.72f,.01f,.71f,.37f,.31f,-.39f,.91f,.19f,-.34f,.45f,-.68f,-.26f,-.78f,-.74f,.43f,-.05f,-.29f,-.98f,-.07f,.61f,-.93f,.05f,-.44f,.44f,.89f,-.08f,.19f,.95f,.90f,.63f,.25f,.09f,.90f,-.66f,.62f,.72f,-.28f,.76f,-.90f,.37f,-.25f,.60f,.92f,-.75f,.86f,-.79f,.17f,-.11f,-.21f,-.99f,-.41f,.18f,-.59f,-.04f,.92f,-.20f,-.82f,.70f,.18f,-.08f,.04f,.84f,-.48f,-.46f,-.94f,-.55f,.64f,.10f,-.74f,-.66f,-.16f,.52f,.33f,-.41f,.24f,.94f,.22f,-.04f,-.90f,.91f,.13f,.46f,.18f,-.33f,-.48f,-.92f,-.54f,-.90f,-.20f,.58f,-.45f,-.52f,-.50f,-.42f,-.18f,-.49f,-.71f,.61f,-.32f,.82f,-.17f,-.82f,.37f,-.73f,-.22f,.97f,.79f,.43f,.75f,.82f,-.88f,.15f,.04f,.50f,-.21f,-.38f,.63f,-.45f,-.39f,.34f,.20f,-.46f,-.24f,-.44f,-.82f,.63f,-.36f,-.03f,-.20f,-.31f,-.51f,-.44f,.03f,-.28f,.86f,-.98f,.67f,-.01f,.37f,-.58f,.83f,-.71f,.88f,.94f,-.97f,-.02f,.53f,-.47f,.43f,.45f,.66f,-.41f,.01f,.66f,-.73f,.98f,-.51f,-.25f,.10f,.86f,-.61f,-.38f,.24f,-.37f,-.76f,.10f,.41f,-.40f,.45f,-.94f,.06f,-.34f,-.77f,-.49f,.10f,.60f,.59f,-.18f,-.29f,.16f,.66f,-.79f,.31f,-.15f,-.36f,.28f,-.83f,-.40f,.79f,-.79f,.01f,-.10f,-.89f,.89f,-.73f,-.94f,-.25f,.90f,.02f,.87f,-.72f,.14f,.01f,.14f,.33f,.70f,-.23f,.31f,.47f,-.99f,.74f,-.94f,-.18f,.29f,.74f,.41f,-.62f,-.55f,-.23f,-.92f,-.83f,-.04f,.62f,-.21f,.18f,.54f,.10f,.01f,-.56f,.20f,-.27f,.95f,-.72f,-.81f,-.37f,.04f,.35f,-.43f,.74f,.96f,-.39f,-.25f,.84f,-.96f,.63f,-.21f,.38f,.08f,-.30f,-.96f,.62f,.52f,.88f,.24f,.20f,-.98f,-.19f,.73f,-.92f,-.26f,.98f,.23f,.02f,-.56f,-.67f,.84f,.80f,-.99f,.90f,.08f,.54f,-.42f,.12f,.61f,-.37f,-.33f,.96f,.77f,-.97f,.69f,-.03f,.32f,-.05f,.38f,-.13f,-.67f,-.29f,.78f,.49f,.06f,-.80f,-.11f,.67f,.22f,-.76f,-.30f,-.78f,.35f,.64f,-.14f,-.56f,.01f,-.67f,-.73f,-.00f,.49f,.49f,-.48f,-.61f,.93f,-.05f,.14f,.47f,-.07f,.83f,.97f,.07f,-.41f,-.19f,.17f,.62f,.58f,-.37f,.48f,-.62f,-.36f,.94f,.07f,-.83f,.49f,-.81f,-.77f,-.21f,.15f,-.48f,-.65f,.88f,.58f,.74f,-.88f,-.78f,.70f,-.18f,.37f,-.84f,-.49f,-.39f,.98f,.06f,.25f,.97f,.39f,.38f,-.67f,-.52f,.65f,-.66f,-.53f,.38f,-.30f,-.51f,.07f,-.07f,.01f,.73f,.38f,.47f,-.56f,.44f,.78f,-.98f,-.99f,-.31f,-.72f,.91f,.18f,-.84f,-.40f,-.53f,.33f,.13f,.37f,.01f,.81f,.00f,-.19f,-.02f,-.73f,.65f,.84f,-.01f,.16f,-.07f,.64f,.22f,.06f,.49f,-.28f,-.84f,.81f,-.94f,.32f,-.07f,.09f,.95f,.99f,-.56f,.64f,.83f,.51f,-.93f,.18f,-.50f,.38f,-.07f,.93f,.63f,-.29f,.87f,-.04f,.63f,.70f,-.76f,.02f,.02f,-.88f,-.63f,-.18f,-.30f,-.48f,-.42f,.93f,.01f,.36f,.97f,-.25f,.93f,.16f,.46f,-.31f,-.32f,.20f,.43f,-.93f,.09f,.97f,.53f,-.86f,-.33f,.77f,.58f,-.11f,.55f,.94f,-.42f,-.48f,.05f,.15f,.58f,-.59f,-.41f,-.53f,.05f,.96f,.00f,.60f,.56f,-.18f,.49f,-.44f,.47f,-.67f,-.85f,.91f,-.28f,-.17f,.62f,-.96f,.81f,.12f,.54f,-.52f,-.12f,-.28f,.82f,.02f,-.11f,-.51f,-.45f,.20f,.69f,.07f,.48f,-.48f,-.01f,.33f,.64f,.15f,-.79f,-.11f,.28f,.48f,.58f,.95f,-.89f,.01f,-.16f,-.66f,-.92f,.89f,-.34f,.50f,-.28f,.08f,-.74f,.29f,-.02f,.68f,.15f,.95f,.21f,-.58f,-.11f,.64f,.73f,.87f,-.96f,-.64f,-.60f,-.79f,.35f,-.54f,-.65f,.14f,-.48f,.02f,.36f,-.45f,.07f,.58f,.62f,.73f,-.92f,.71f,-.73f,-.58f,-.10f,.95f,-.81f,.05f,-.40f,-.70f,.03f,-.45f,.88f,-.89f,.37f,-.30f,-.37f,.07f,.28f,-.11f,-.38f,.32f,.78f,.21f,.43f,.77f,-.57f,.40f,-.93f,.16f,-.66f,-.66f,-.82f,-.13f,-.69f,.77f,.54f,.69f,-.54f,.29f,-.77f,.14f,.25f,-.14f,-.98f,.36f,.00f,-.17f,-.07f,-.25f,.25f,-.30f,-.53f,.79f,.08f,-.80f,-.05f,-.87f,.22f,.35f,.32f,.09f,-.44f,-.82f,.77f,-.17f,.47f,-.89f,-.83f,.33f,.20f,.93f,-.44f,.89f,-.27f,-.95f,.03f,-.23f,.34f,.35f,-.22f,.56f,-.36f,.20f,.86f,-.18f,-.11f,-.27f,-.87f,-.52f,-.85f,-.71f,-.07f,-.42f,-.73f,-.72f,-.45f,.21f,.22f,.35f,-.67f,.47f,-.17f,-.72f,-.48f,.64f,.68f,-.06f,-.60f,-.82f,.84f,.16f,.49f,-.16f,-.57f,-.16f,-.86f,.05f,-.91f,-.02f,.11f,.91f,.64f,-.68f,.93f,-.25f,-.46f,.70f,.87f,.97f,-.75f,.73f,-.70f,.63f,-.08f,.94f,.84f,-.55f,.29f,-.75f,-.18f,.15f,.41f,.33f,.81f,-.95f,-.12f,-.47f,.19f,-.12f,-.49f,.34f,.23f,.36f,.51f,-.45f,-.97f,.09f,-.47f,.54f,.32f,.46f,.90f,-.58f,-.87f,-.48f,-.91f,-.72f,-.19f,-.93f,.30f,.91f,-.78f,-.46f,.19f,.90f,-.22f,-.43f,.56f,-.42f,.99f,-.46f,.18f,-.75f,-.77f,.85f,-.99f,.10f,-.92f,.68f,-.07f,-.81f,-.57f,-.57f,-.31f,.74f,-.01f,.38f,.00f,.05f,-.85f,-.28f,.91f,-.78f,.53f,.02f,-.12f,-.90f,.21f,-.22f,-.01f,-.87f,.45f,-.37f,.09f,.90f,.15f,-.14f,.66f,.32f,.18f,.87f,.74f,-.35f,.95f,.23f,-.89f,-.81f,.95f,.96f,-.43f,.57f,-.16f,-.82f,-.00f,-.64f,.18f,.01f,.19f,-.90f,-.55f,.31f,.41f,-.41f,-.31f,-.32f,-.65f,.08f,.77f,.24f,-.26f,.39f,-.72f,.75f,-.49f,.19f,.58f,.55f,-.71f,-.12f,.96f,-.78f,.73f,.10f,-.31f,.25f,.78f,-.70f,-.11f,-.87f,-.00f,.66f,.26f,.80f,-.21f,-.94f,-.99f,.94f,.78f,.97f,-.41f,-.27f,.23f,-.20f,-.32f,-.86f,.51f,.67f,.95f,.71f,-.63f,-.26f,-.20f,.79f,.34f,.42f,.27f,-.49f,.99f,.59f,-.09f,-.23f,-.10f,.11f,-.37f,-.99f,.73f,.81f,-.38f,-.57f,.90f,.85f,-.21f,-.73f,.74f,.42f,-.77f,-.04f,.16f,.88f,-.27f,-.34f,-.83f,-.91f,-.47f,.18f,-.06f,.40f,-.10f,-.74f,-.45f,-.24f,-.27f,-.44f,-.49f,-.96f,-.46f,-.12f,.76f,.75f,-.11f,-.96f,.54f,.00f,-.26f,.81f,.79f,.67f,.20f,.69f,.68f,.01f,.17f,-.84f,-.28f,-.05f,.49f,.09f,.12f,-.30f,-.02f,-.02f,-.62f,-.12f,.76f,-.24f,.86f,.36f,.36f,.25f,-.34f,-.57f,.92f,-.72f,.05f,.76f,-.36f,-.54f,-.03f,-.66f,-.95f,.52f,.87f,.11f,.17f,-.88f,-.37f,-.13f,.18f,.03f,.50f,.59f,-.15f,.36f,.72f,.49f,.62f,-.12f,-.69f,.15f,.80f,.63f,-.97f,.19f,.59f,.38f,-.14f,-.72f,-.71f,-.08f,-.36f,-.23f,.56f,.34f,.14f,.68f,-.90f,-.15f,.32f,.08f,-.87f,-.89f,-.93f,.39f,.30f,.54f,-.86f,-.32f,-.45f,-.09f,-.10f,.37f,.18f,-.83f,-.38f,-.53f,-.67f,.98f,-.29f,-.03f,-.30f,.90f,-.58f,.39f,-.29f,.10f,.11f,.04f,.80f,.21f,.08f,.00f,.50f,-.25f,.75f,.26f,.28f,.26f,-.59f,.56f,.68f,.02f,.55f,.81f,-.78f,.14f,.90f,-.15f,.48f,-.74f,-.61f,.31f,.72f,-.82f,.28f,.27f,.82f,-.67f,.90f,-.88f,.56f,-.29f,.05f,-.46f,.67f,-.21f,-.33f,.28f,.43f,-.95f,.88f,.97f,.41f,-.68f,.30f,.04f,-.77f,.14f,-.67f,-.96f,.01f,-.92f,-.55f,.44f,-.70f,-.70f,.37f,-.07f,.39f,-.12f,-.27f,-.71f,.06f,.28f,.92f,-.79f,-.82f,.52f,.26f,-.21f,.82f,-.49f,.88f,.47f,.45f,.31f,.44f,-.98f,-.23f,.44f,.13f,.61f,-.20f,-.23f,-.61f,-.34f,.35f,.56f,-.16f,.57f,-.46f,-.56f,-.92f,-.31f,.97f,-.25f,-.48f,-.94f,.74f,.02f,-.61f,-.71f,.57f,.77f,.33f,-.52f,.38f,.78f,-.79f,.20f,-.52f,.44f,-.45f,-.36f,-.93f,-.66f,-.08f,-.75f,-.84f,-.83f,-.72f,.39f,.32f,-.87f,.58f,-.47f,.08f,.08f,-.55f,.22f,.63f,.21f,-.15f,.08f,-.79f,-.42f,.49f,-.89f,.14f,-.24f,-.42f,.34f,-.20f,-.26f,.90f,.98f,.89f,-.73f,-.21f,-.73f,.36f,-.51f,.56f,-.67f,.65f,.82f,.39f,.18f,.76f,-.26f,-.75f,-.83f,-.94f,-.04f,-.80f,-.89f,.68f,-.48f,-.50f,.75f,.22f,.87f,-.06f,.40f,-.40f,-.13f,.17f,-.01f,-.40f,.75f,.42f,.77f,.28f,-.41f,.12f,.55f,.72f,-.48f,-.71f,.75f,-.18f,-.51f,.55f,.35f,.57f,.03f,.86f,.52f,.71f,.50f,-.25f,.83f,.57f,-.51f,.81f,.03f,-.81f,-.78f,-.70f,.27f,.55f,.62f,.55f,.68f,-.05f,-.73f,-.28f,.24f,-.33f,.59f,-.89f,.78f,-.91f,-.77f,.11f,-.73f,.57f,-.26f,-.25f,.80f,.47f,-.09f,.71f,.15f,.56f,.27f,.11f,-.27f,-.69f,.45f,.23f,.87f,-.35f,-.31f,.73f,-.30f,-.65f,.36f,.52f,-.80f,.62f,.25f,.31f,-.42f,-.32f,.83f,-.59f,-.22f,-.61f,-.35f,-.60f,-.89f,-.46f,-.38f,-.69f,-.99f,.87f,-.04f,.47f,.31f,-.07f,-.64f,.30f,-.09f,-.57f,.28f,.39f,.75f,-.44f,-.50f,.97f,.74f,-.86f,.59f,.28f,-.48f,-.96f,.59f,.17f,.43f,.84f,-.80f,.14f,.10f,-.25f,-.99f,-.89f,.12f,-.67f,-.24f,.65f,-.20f,-.73f,.66f,.80f,-.46f,-.32f,-.60f,.79f,-.99f,.22f,.30f,-.33f,.49f,.71f,-.59f,-.47f,-.36f,.93f,.67f,-.66f,-.48f,.04f,-.11f,.22f,.55f,.31f,.26f,.17f,-.12f,.39f,.40f,-.13f,.44f,-.36f,.81f,.69f,-.47f,-.07f,-.94f,-.06f,.45f,-.47f,.12f,.30f,.81f,-.50f,.64f,.56f,-.01f,.43f,-.24f,.45f,-.30f,.56f,-.13f,.44f,-.17f,.17f,.76f,-.32f,-.49f,-.13f,.07f,-.43f,-.50f,-.80f,-.07f,-.13f,.93f,.13f,-.29f,-.39f,.53f,-.89f,.67f,.25f,.42f,-.29f,.48f,-.37f,.99f,.99f,-.12f,.36f,.64f,-.79f,.85f,-.97f,-.01f,-.51f,-.54f,-.02f,-.79f,-.70f,-.61f,.24f,-.12f,.82f,.18f,-.91f,-.83f,-.90f,-.32f,-.14f,.51f,.05f,-.63f,.49f,.57f,-.31f,-.97f,.77f,-.48f,.05f,-.75f,-.72f,.47f,-.55f,.19f,-.76f,.39f,-.06f,-.66f,.20f,-.85f,-.09f,-.76f,-.84f,.09f,.74f,-.09f,-.75f,-.69f,.18f,.27f,.74f,.73f,.29f,.70f,-.59f,.91f,.93f,.83f,.40f,-.87f,-.29f,-.93f,-.33f,.88f,-.95f,-.81f,-.71f,.96f,-.37f,-.39f,-.23f,.08f,.71f,-.18f,.23f,.61f,.05f,.19f,-.19f,-.81f,-.05f,.63f,.84f,-.12f,-.33f,.27f,.49f,-.29f,-.30f,.70f,-.82f,-.59f,.59f,-.28f,.10f,-.24f,.13f,.85f,-.54f,-.63f,.65f,-.06f,-.59f,-.97f,-.36f,-.40f,-.34f,.84f,-.25f,.77f,.22f,.87f,.91f,-.78f,.74f,-.13f,.13f,.07f,.15f,.92f,.17f,-.51f,.06f,.79f,-.51f,-.76f,-.19f,-.22f,.74f,-.31f,-.31f,-.09f,.74f,-.97f,-.59f,.39f,-.58f,-.70f,-.29f,.44f,-.71f,-.77f,-.83f,.17f,-.55f,-.47f,.98f,.74f,-.04f,.57f,.05f,.64f,-.53f,-.23f,-.15f,.87f,-.86f,.45f,.50f,-.58f,.35f,-.89f,.59f,-.34f,-.12f,-.56f,-.08f,.93f,.93f,.27f,-.60f,.06f,.47f,.22f,.49f,.67f,-.30f,.83f,-.13f,.86f,-.86f,-.79f,-.41f,.30f,.56f,.88f,.17f,.40f,-.84f,.84f,-.18f,.95f,-.17f,-.52f,.14f,.04f,.96f,.21f,-.42f,.92f,.32f,-.24f,-.11f,.26f,-.60f,-.68f,.91f,-.37f,.11f,.76f,.31f,.39f,-.25f,-.10f,.63f,.20f,.83f,-.49f,.71f,.42f,-.01f,.15f,-.47f,-.13f,-.69f,.17f,.90f,-.21f,.14f,.60f,.39f,-.16f,.23f,-.33f,.88f,-.78f,-.87f,.15f,-.68f,.53f,-.11f,.43f,-.54f,-.57f,.87f,.81f,.13f,-.07f,.18f,-.32f,-.85f,.35f,-.32f,.75f,-.80f,.79f,-.12f,.55f,.06f,-.95f,.50f,.99f,-.88f,.75f,-.69f,-.23f,-.53f,.93f,-.57f,.29f,.58f,-.42f,.75f,-.92f,-.13f,-.16f,.28f,-.90f,-.17f,-.74f,-.55f,.99f,-.82f,-.24f,-.43f,-.52f,-.45f,-.02f,-.70f,-.26f,.74f,.21f,.97f,-.10f,-.65f,-.18f,-.06f,-.60f,.54f,-.76f,.23f,.21f,.44f,.06f,-.80f,-.87f,-.13f,.92f,.97f,-.04f,.35f,-.44f,-.00f,-.78f,-.54f,-.91f,.19f,.97f,-.60f,.35f,-.64f,-.84f,.25f,.90f,.17f,-.26f,-.82f,.59f,-.62f,-.14f,.41f,.62f,.91f,-.73f,.62f,.87f,.50f,.55f,-.72f,.00f,-.19f,.90f,.30f,.09f,-.10f,-.90f,.66f,.77f,.63f,-.07f,-.92f,-.63f,-.86f,-.08f,.09f,.38f,.58f,.76f,-.98f,.83f,-.05f,-.37f,.52f,-.57f,-.31f,-.47f,-.49f,.42f,-.13f,-.79f,.60f,-.08f,-.12f,.65f,-.48f,.79f,.22f,.71f,.70f,.78f,-.25f,-.05f,.48f,.14f,-.85f,.50f,-.86f,.31f,.60f,.72f,.82f,-.46f,-.65f,-.54f,.26f,.17f,-.02f,-.48f,.36f,-.63f,.84f,.62f,.59f,-.74f,.00f,.45f,.08f,-.54f,-.34f,.79f,.38f,-.17f,.41f,.30f,.06f,.22f,.59f,.23f,-.36f,.85f,-.00f,-.93f,.74f,.06f,-.99f,-.06f,-.48f,.27f,-.97f,-.44f,-.12f,.46f,-.63f,-.57f,-.98f,.72f,.59f,-.81f,.96f,.64f,-.59f,.13f,-.80f,.43f,.23f,-.66f,-.74f,-.52f,.39f,-.60f,-.92f,.49f,.03f,.84f,-.72f,-.97f,-.81f,.17f,.79f,.69f,-.66f,.11f,.40f,-.37f,.46f,-.74f,-.50f,.81f,-.90f,.66f,-.06f,-.42f,.47f,.12f,.59f,-.26f,.12f,-.93f,.61f,-.19f,-.96f,-.63f,-.86f,-.96f,.33f,-.80f,-.31f,-.22f,-.73f,-.39f,.69f,-.59f,.17f,-.24f,.75f,-.61f,-.90f,.23f,.75f,-.51f,-.08f,-.47f,.37f,-.30f,.36f,.75f,.93f,.97f,.52f,.64f,.98f,-.05f,.34f,-.66f,.60f,.44f,.48f,.53f,-.61f,-.57f,.18f,-.28f,.80f,.14f,.31f,-.54f,-.56f,.48f,.28f,-.86f,-.04f,-.12f,-.97f,.87f,-.82f,-.33f,-.75f,.67f,-.05f,.83f,.14f,.84f,-.47f,.25f,-.52f,-.48f,-.86f,.15f,.19f,-.45f,-.48f,-.33f,-.98f,-.61f,.13f,.83f,-.45f,.78f,.16f,-.52f,.37f,.07f,-.98f,.78f,.91f,-.26f,-.30f,.10f,-.92f,.17f,.10f,.44f,-.31f,-.50f,-.85f,-.29f,.85f,-.47f,.13f,-.59f,-.71f,.34f,.39f,-.72f,-.46f,-.66f,.44f,.95f,.09f,-.10f,.81f,.37f,-.82f,.24f,-.58f,-.79f,.61f,-.73f,.45f,-.76f,-.55f,-.06f,-.59f,-.04f,.42f,.08f,-.51f,-.87f,-.54f,-.25f,.51f,-.14f,.78f,.06f,.44f,-.40f,.11f,-.47f,.36f,.23f,.12f,.49f,-.40f,-.39f,.45f,.74f,-.99f,-.21f,.03f,.85f,.59f,.67f,.15f,.74f,.34f,-.49f,-.59f,-.82f,-.82f,.60f,.12f,-.61f,-.19f,.52f,-.32f,.96f,.46f,.95f,.27f,.97f,-.01f,.18f,.81f,-.55f,.05f,-.76f,-.74f,.08f,.58f,-.77f,.43f,-.55f,-.78f,.37f,.39f,-.37f,-.03f,-.75f,-.46f,.96f,.50f,.55f,-.81f,.39f,-.54f,.42f,-.92f,-.40f,.31f,.16f,-.87f,-.58f,.41f,-.31f,-.74f,-.36f,.18f,.00f,.32f,.32f,.39f,-.91f,.50f,.75f,.38f,.60f,.11f,-.56f,.61f,-.92f,.38f,.10f,-.30f,-.42f,.35f,-.51f,.58f,.84f,-.37f,-.00f,.82f,.15f,.82f,-.89f,.34f,.29f,-.45f,-.31f,-.54f,.02f,-.69f,.15f,-.77f,-.14f,-.89f,-.19f,-.03f,-.41f,.20f,-.28f,-.60f,.25f,-.18f,.82f,-.38f,-.20f,-.25f,-.18f,-.85f,.88f,-.02f,.97f,.62f,-.97f,.63f,.36f,-.13f,-.93f,-.84f,.63f,.87f,-.18f,-.34f,-.72f,.63f,.13f,-.05f,-.56f,.81f,-.19f,-.89f,.61f,.71f,.22f,.45f,-.44f,.89f,.67f,.47f,.85f,-.50f,-.03f,-.73f,.20f,-.86f,.71f,-.12f,-.16f,.50f,.47f,-.95f,-.37f,.81f,.85f,.69f,.46f,-.77f,.25f,.15f,.09f,-.12f,-.32f,.93f,-.04f,-.66f,-.41f,-.91f,-.84f,-.32f,-.51f,.65f,.09f,-.18f,.80f,.55f,-.30f,.56f,.22f,.28f,-.48f,-.61f,-.15f,-.65f,-.38f,.03f,-.83f,.61f,.83f,-.14f,.39f,-.44f,-.49f,-.70f,.35f,-.72f,.51f,.09f,.31f,.98f,.63f,.74f,.25f,-.30f,.20f,.08f,.81f,.69f,.16f,-.85f,.69f,.20f,-.68f,.99f,.14f,-.65f,-.21f,.03f,.06f,-.16f,.80f,.95f,.90f,-.13f,.05f,.84f,.56f,-.42f,-.58f,.02f,-.16f,-.68f,.03f,.94f,-.20f,-.99f,.99f,-.20f,-.69f,.25f,-.00f,.55f,-.11f,.74f,.96f,-.12f,-.50f,.81f,.97f,-.56f,.40f,.94f,.34f,-.98f,-.04f,-.45f,-.89f,-.04f,-.07f,-.35f,.58f,-.52f,.85f,.94f,-.92f,-.46f,-.51f,-.30f,-.98f,.04f,.45f,-.08f,.53f,-.78f,.43f,.90f,-.48f,-.31f,-.84f,-.70f,-.79f,-.33f,.13f,.68f,-.44f,.21f,.62f,.73f,.15f,.72f,.31f,.21f,-.78f,-.89f,-.96f,-.69f,.12f,.45f,-.47f,-.91f,-.33f,.75f,.87f,.35f,-.33f,.08f,.34f,-.93f,.55f,-.30f,.93f,.60f,-.06f,.89f,-.84f,-.34f,.59f,.20f,-.63f,-.40f,.10f,-.47f,-.79f,-.05f,-.28f,-.97f,.68f,.58f,-.96f,-.32f,.09f,-.14f,.20f,.86f,.33f,-.27f,-.62f,.99f,-.56f,.87f,-.75f,-.53f,.86f,-.44f,-.38f,-.30f,-.73f,-.95f,-.19f,.16f,-.16f,-.87f,-.41f,.77f,-.40f,.90f,-.10f,.67f,-.28f,-.81f,.66f,-.97f,-.21f,-.08f,-.57f,-.80f,.07f,-.86f,-.94f,-.33f,-.43f,-.63f,.52f,-.74f,-.07f,-.17f,.75f,.10f,.97f,.70f,-.01f,.13f,.28f,-.59f,-.25f,.39f,.44f,.18f,.38f,-.95f,.93f,-.80f,.37f,.08f,.42f,-.84f,-.05f,.21f,.96f,.93f,-.60f,.96f,-.50f,.17f,-.74f,.47f,-.16f,-.05f,-.36f,-.46f,-.62f,.95f,.44f,-.45f,.42f,-.31f,.79f,.37f,-.21f,-.78f,-.08f,-.70f,-.83f,.02f,.10f,-.45f,-.12f,-.45f,-.90f,.95f,.60f,.34f,-.38f,-.22f,.37f,.04f,-.21f,-.63f,.10f,.61f,-.76f,-.48f,-.45f,.27f,.96f,.75f,.09f,.77f,.99f,-.35f,.81f,.95f,.14f,-.41f,-.54f,-.43f,.25f,.36f,-.61f,-.11f,.83f,.14f,.56f,-.42f,.80f,.35f,.11f,-.51f,-.68f,-.28f,.05f,.54f,-.01f,-.74f,-.19f,-.91f,.86f,-.05f,.19f,-.71f,-.22f,.39f,.97f,.08f,.92f,-.36f,.29f,.08f,.99f,-.31f,-.94f,-.37f,.17f,.41f,.41f,-.57f,-.56f,-.05f,.88f,.19f,.91f,.34f,.07f,.39f,-.56f,.76f,.87f,-.82f,.79f,-.29f,.29f,.79f,-.00f,-.68f,.22f,.75f,-.29f,-.92f,-.57f,.88f,-.78f,-.99f,.88f,.34f,.82f,.86f,-.17f,.79f,.12f,.42f,-.50f,.23f,.83f,.25f,.77f,.62f,-.81f,-.24f,.47f,-.49f,-.45f,.06f,-.38f,-.01f,.15f,-.16f,-.63f,-.25f,-.99f,.24f,.85f,-.69f,-.37f,-.27f,.01f,-.22f,.73f,.63f,-.48f,.87f,.90f,-.64f,.68f,.59f,.11f,-.83f,.17f,.87f,-.88f,.76f,-.85f,-.20f,.94f,.71f,.16f,-.40f,-.73f,.91f,.61f,.89f,-.73f,.79f,-.48f,-.18f,.74f,.64f,-.74f,-.89f,-.84f,-.28f,-.00f,.45f,.50f,.78f,.84f,-.54f,.66f,.67f,-.58f,-.67f,.09f,-.93f,-.50f,-.71f,-.57f,.44f,-.20f,.42f,-.65f,-.81f,.37f,.21f,-.82f,-.16f,.84f,.04f,.51f,.33f,-.43f,.44f,-.82f,.92f,-.33f,-.92f,.76f,-.33f,.83f,-.21f,-.25f,.28f,.78f,.70f,.04f,.65f,-.44f,.28f,-.61f,-.85f,.26f,-.86f,.41f,.02f,-.44f,.93f,.27f,.79f,-.60f,.91f,-.25f,-.92f,.13f,.87f,-.29f,-.89f,-.56f,-.91f,-.95f,-.72f,-.62f,-.99f,-.89f,-.89f,.64f,-.08f,.70f,-.78f,.82f,-.54f,.97f,-.26f,.12f,.46f,-.26f,-.46f,-.65f,.04f,-.50f,.34f,-.78f,-.54f,-.29f,.13f,.19f,.01f,-.77f,-.20f,-.98f,.43f,.37f,.80f,-.97f,-.99f,-.66f,.55f,.85f,.45f,.58f,-.29f,-.54f,-.95f,.41f,.14f,-.84f,.13f,.06f,.37f,.23f,.76f,.88f,.12f,-.84f,-.53f,-.26f,.26f,.67f,-.24f,.76f,.76f,.12f,-.13f,-.05f,-.12f,.92f,.40f,-.68f,.68f,.00f,.19f,-.72f,.06f,-.66f,.55f,.77f,-.71f,.84f,.59f,.15f,.27f,.38f,-.99f,-.03f,.65f,-.94f,.55f,-.51f,.40f,.32f,.33f,-.93f,.49f,.83f,.98f,.32f,-.45f,.32f,-.22f,.37f,-.35f,-.30f,-.63f,-.72f,.86f,-.95f,.70f,.44f,.12f,-.66f,.39f,.67f,-.56f,-.25f,-.64f,-.15f,.33f,.37f,-.26f,.65f,.13f,-.98f,-.27f,-.94f,.90f,-.30f,-.35f,.24f,.20f,-.93f,.24f,-.17f,-.80f,.24f,.92f,-.81f,-.61f,.05f,-.72f,.12f,.95f,-.07f,-.04f,.74f,-.10f,-.07f,-.55f,-.07f,.21f,-.77f,-.79f,.52f,-.76f,-.02f,-.18f,-.88f,-.76f,.78f,-.74f,-.22f,.26f,-.51f,.91f,.33f,.82f,.67f,.89f,-.57f,.47f,-.30f,.06f,.46f,.40f,.33f,.36f,-.12f,.80f,-.46f,.69f,.60f,.96f,.69f,-.85f,-.16f,-.44f,-.13f,-.58f,-.93f,.05f,.51f,-.02f,.79f,-.36f,.27f,.74f,.67f,.61f,-.67f,-.97f,-.01f,.78f,-.94f,.11f,-.35f,-.45f,-.27f,.51f,.07f,-.34f,.46f,.85f,-.25f,.84f,.40f,-.68f,-.66f,-.89f,.72f,-.52f,-.46f,-.19f,.91f,.09f,-.56f,.16f,.01f,-.29f,.86f,-.05f,.50f,-.98f,.98f,-.25f,.21f,-.52f,-.45f,.07f,.97f,.62f,.51f,.91f,-.42f,.80f,-.33f,-.77f,-.98f,.82f,-.35f,.96f,.25f,-.54f,.50f,-.66f,.54f,-.14f,.53f,-.96f,-.05f,-.85f,-.03f,.12f,-.71f,-.06f,.47f,-.80f,.70f,.88f,.57f,-.07f,-.04f,-.13f,.80f,.59f,-.23f,-.66f,-.34f,-.78f,.95f,.14f,.24f,-.37f,-.29f,.51f,-.94f,-.93f,-.62f,-.68f,.58f,-.13f,.16f,.50f,-.23f,-.01f,-.24f,.82f,-.07f,-.01f,-.49f,.23f,-.00f,.83f,-.60f,.72f,.34f,-.12f,-.11f,-.07f,.01f,.15f,-.23f,.58f,-.60f,.74f,.43f,-.15f,-.41f,-.29f,.66f,.60f,.74f,-.13f,.48f,-.13f,-.35f,.29f,-.64f,-.76f,.27f,.56f,.72f,.51f,-.75f,-.30f,.53f,-.62f,.64f,.11f,-.93f,-.12f,-.43f,.96f,.49f,-.66f,.47f,-.96f,-.60f,.72f,.64f,.39f,-.53f,-.61f,-.37f,.99f,-.74f,.50f,-.32f,.82f,-.98f,-.06f,-.68f,.32f,-.28f,-.64f,.30f,-.44f,.60f,.16f,-.29f,-.21f,-.12f,.19f,.26f,-.32f,-.90f,.88f,.96f,-.28f,.51f,-.86f,.28f,.43f,.82f,-.21f,-.15f,.15f,-.70f,-.70f,.05f,-.82f,-.85f,-.57f,.13f,-.22f,.53f,.80f,.43f,.30f,-.77f,-.95f,.95f,.95f,-.21f,.56f,.56f,-.66f,-.70f,-.03f,.51f,.79f,.00f,-.68f,.25f,-.15f,.12f,.07f,-.18f,-.57f,.81f,.22f,-.80f,.14f,.14f,.19f,.65f,-.06f,.72f,-.96f,-.93f,-.96f,-.35f,.06f,.92f,-.01f,-.16f,.13f,.00f,-.24f,.79f,-.15f,-.29f,-.83f,-.38f,-.91f,-.17f,-.21f,-.89f,-.24f,.40f,-.43f,-.31f,.45f,.74f,.77f,-.98f,.53f,.87f,-.42f,-.37f,-.95f,.02f,-.03f,.21f,-.39f,-.27f,-.66f,.62f,-.34f,.34f,.89f,-.64f,-.65f,-.51f,-.98f,.14f,-.49f,.27f,.10f,-.71f,-.20f,-.01f,-.51f,.50f,-.30f,.98f,-.46f,.10f,-.60f,-.71f,-.95f,.49f,.59f,.00f,-.19f,.11f,-.41f,.08f,-.50f,.48f,.19f,-.80f,.55f,-.27f,.14f,-.66f,.04f,-.16f,-.40f,.18f,-.50f,.89f,-.98f,.27f,-.39f,.91f,-.07f,-.88f,-.76f,-.34f,.10f,.80f,-.05f,-.31f,-.87f,.65f,.26f,-.62f,.55f,.57f,-.98f,.11f,.14f,-.63f,-.39f,.13f,-.87f,-.60f,.92f,-.08f,-.88f,.08f,-.82f,-.12f,.71f,.41f,-.81f,.17f,.16f,-.52f,-.44f,-.58f,.24f,-.16f,.42f,-.87f,-.41f,.86f,.51f,-.42f,.57f,.44f,-.90f,-.37f,-.11f,-.13f,.12f,-.13f,-.80f,-.44f,-.12f,.76f,-.65f,-.00f,.94f,-.49f,.64f,-.20f,-.02f,-.30f,-.70f,.81f,.37f,-.12f,.06f,-.73f,-.99f,-.24f,.66f,.32f,-.94f,.29f,.47f,.32f,-.20f,.90f,.37f,.14f,-.65f,-.30f,.03f,-.81f,-.85f,-.45f,.95f,.52f,-.64f,.70f,-.32f,-.50f,.34f,.33f,.44f,.94f,.39f,.78f,-.99f,.35f,-.04f,.56f,-.65f,.33f,-.57f,.23f,-.17f,-.97f,.70f,-.68f,-.23f,-.60f,.20f,-.93f,.00f,.16f,.92f,-.19f,.72f,.28f,.55f,.97f,.44f,-.40f,-.03f,.86f,.27f,.31f,-.50f,.16f,.03f,-.49f,-.09f,-.64f,-.17f,-.01f,.88f,.07f,.45f,-.45f,-.36f,-.40f,.69f,-.54f,-.44f,-.74f,.00f,-.49f,-.40f,-.92f,.16f,-.21f,-.39f,-.26f,-.63f,.14f,-.53f,.31f,-.83f,.33f,.31f,.06f,-.88f,.12f,-.20f,.63f,.35f,.79f,-.25f,.77f,-.79f,.16f,.64f,-.49f,-.14f,.09f,-.00f,.46f,-.96f,-.00f,.67f,.74f,-.23f,-.25f,-.16f,.45f,-.87f,.73f,.89f,-.40f,.85f,.70f,-.98f,.11f,.27f,.64f,.22f,-.38f,.80f,-.38f,.98f,.87f,.96f,.54f,.88f,.46f,.00f,-.08f,.94f,.02f,-.97f,.16f,-.04f,.44f,-.45f,-.58f,.38f,-.49f,.93f,.78f,.84f,-.35f,.28f,.96f,.16f,-.58f,.32f,.34f,-.43f,.34f,.68f,.92f,-.69f,.48f,.42f,-.13f,-.54f,.09f,.70f,-.85f,.74f,-.55f,-.18f,.45f,-.49f,.12f,.33f,-.49f,-.68f,-.67f,.60f,.61f,-.87f,.13f,.68f,-.56f,.83f,-.16f,.26f,-.18f,-.80f,-.47f,.80f,-.25f,.36f,-.12f,-.98f,.78f,-.58f,.76f,-.25f,.34f,.42f,.76f,.09f,-.97f,.57f,.01f,.82f,.02f,-.67f,-.15f,.99f,-.27f,.19f,-.16f,-.28f,-.70f,.17f,.68f,.00f,.81f,-.88f,-.43f,-.71f,-.90f,-.12f,-.31f,-.15f,-.12f,-.44f,.58f,.26f,-.41f,-.23f,-.06f,-.71f,-.21f,.66f,-.27f,.31f,.55f,-.43f,-.23f,-.98f,.17f,-.03f,.66f,.13f,.41f,-.20f,-.93f,-.33f,-.25f,.23f,-.15f,.19f,.73f,.24f,-.49f,.16f,.24f,-.12f,-.21f,-.61f,-.11f,-.23f,-.98f,-.51f,.19f,.09f,.32f,.99f,.28f,.19f,.66f,.74f,.89f,-.10f,.04f,-.12f,.79f,.36f,.17f,.51f,-.66f,-.09f,.75f,-.88f,-.67f,-.74f,-.63f,-.36f,-.93f,.61f,-.12f,-.00f,.06f,.58f,-.27f,.73f,-.97f,-.03f,-.97f,.99f,-.11f,-.55f,-.59f,-.44f,-.90f,-.13f,.20f,.85f,.90f,.38f,-.46f,-.41f,.18f,-.87f,-.82f,.61f,-.75f,.20f,-.74f,-.68f,-.22f,-.21f,.68f,-.00f,.38f,-.50f,-.21f,.11f,.94f,-.98f,.59f,-.69f,-.36f,.92f,-.76f,-.73f,-.29f,.23f,-.52f,.18f,.26f,.37f,.08f,.35f,-.61f,.71f,.36f,.00f,-.67f,.57f,.63f,-.04f,.06f,-.96f,.40f,.37f,.86f,.05f,.49f,.78f,-.47f,-.90f,.53f,-.30f,-.57f,-.72f,.25f,.06f,.98f,.89f,-.66f,-.83f,.55f,.35f,.84f,.27f,-.56f,-.40f,-.56f,-.40f,-.83f,.69f,-.05f,-.39f,.87f,-.81f,-.29f,-.94f,.10f,-.32f,-.48f,.43f,-.40f,-.64f,.31f,.98f,-.96f,-.99f,-.70f,.23f,.95f,-.59f,-.04f,-.06f,.95f,-.52f,.18f,.79f,-.28f,-.30f,-.23f,.66f,.49f,-.21f,-.84f,-.27f,.85f,.12f,.05f,.39f,-.21f,.49f,-.28f,-.07f,.96f,.00f,.78f,-.54f,.75f,.15f,.14f,-.21f,-.19f,.06f,.42f,.34f,-.41f,.11f,.44f,-.02f,.89f,.56f,-.78f,.75f,.06f,-.22f,-.32f,-.78f,-.52f,-.46f,.64f,-.31f,-.53f,-.20f,.66f,.29f,.91f,-.84f,-.82f,.67f,-.92f,-.56f,.26f,-.42f,-.93f,.13f,.90f,-.20f,-.40f,.56f,-.60f,.39f,-.01f,.67f,-.77f,-.70f,.30f,-.21f,.04f,-.05f,-.70f,.87f,-.99f,.23f,-.11f,-.99f,-.72f,.22f,.82f,.59f,-.20f,-.40f,.28f,-.91f,-.44f,-.49f,.67f,.33f,.15f,-.09f,-.77f,-.65f,-.95f,.39f,-.02f,-.08f,-.60f,.10f,-.72f,.51f,-.17f,-.80f,-.07f,.92f,-.81f,.65f,.86f,.51f,-.99f,.32f,-.47f,-.37f,-.95f,.59f,-.46f,-.63f,.84f,.84f,-.61f,.97f,.17f,.48f,.94f,-.02f,.98f,.19f,.16f,.40f,.16f,-.40f,-.94f,-.61f,.49f,-.60f,.52f,-.62f,.91f,.67f,.25f,.68f,-.03f,-.03f,.31f,-.16f,.72f,.28f,-.13f,-.10f,-.47f,-.49f,-.79f,-.24f,-.39f,-.93f,-.93f,-.64f,.79f,.06f,-.73f,-.05f,-.77f,-.42f,-.10f,-.28f,-.26f,.58f,.07f,.80f,.52f,-.68f,.86f,-.35f,.79f,.50f,-.83f,-.52f,.75f,-.54f,.49f,.17f,.29f,.45f,-.50f,-.08f,.81f,.98f,.57f,-.57f,-.73f,-.54f,-.94f,-.91f,.13f,.74f,-.31f,-.23f,-.85f,-.98f,-.88f,-.98f,.70f,-.92f,.85f,-.96f,-.71f,.46f,.07f,-.41f,.80f,.35f,.64f,.50f,-.57f,.86f,.13f,.09f,-.13f,.78f,.51f,.66f,.18f,.63f,.66f,.05f,.06f,-.87f,-.33f,-.19f,-.55f,.94f,-.52f,-.74f,.21f,.33f,.50f,.90f,.53f,.80f,.53f,-.15f,.02f,-.84f,-.14f,.19f,.51f,.26f,.16f,-.88f,.98f,-.73f,.28f,.64f,-.76f,.20f,-.74f,.09f,.21f,-.67f,.63f,-.74f,.18f,-.03f,-.25f,-.42f,.76f,-.79f,.98f,-.48f,.94f,.10f,.36f,.47f,-.56f,-.69f,-.74f,.37f,.49f,.78f,-.57f,.14f,-.83f,.15f,-.88f,.91f,.25f,-.89f,-.98f,-.67f,.74f,.53f,-.50f,-.54f,-.94f,-.93f,.69f,.15f,.76f,.60f,.86f,-.96f,-.52f,.84f,-.73f,.10f,.35f,-.13f,.53f,.43f,.19f,-.59f,-.02f,.43f,.50f,-.18f,.03f,-.13f,-.21f,.46f,-.07f,-.99f,.94f,-.16f,-.43f,.25f,-.55f,-.77f,-.13f,-.99f,.98f,.69f,.92f,-.77f,-.89f,-.03f,.03f,-.10f,.48f,.47f,-.95f,.87f,-.12f,.56f,.36f,.56f,-.04f,-.97f,.57f,-.85f,-.45f,.48f,-.54f,.90f,-.46f,.49f,-.02f,.25f,-.83f,.77f,-.38f,.09f,.35f,.89f,-.79f,-.79f,.13f,-.14f,-.27f,-.49f,.46f,.43f,-.91f,-.79f,-.67f,-.15f,.73f,.88f,.17f,-.03f,.22f,.43f,.47f,-.72f,.38f,.23f,-.28f,-.14f,-.02f,-.23f,.47f,.48f,-.11f,-.42f,-.19f,.20f,-.84f,.91f,-.17f,-.52f,.46f,-.53f,.55f,-.29f,.80f,-.94f,-.24f,.53f,.18f,.52f,-.91f,-.47f,-.98f,.82f,.81f,.89f,-.94f,-.18f,.26f,.98f,-.97f,-.99f,-.82f,.67f,-.88f,.76f,-.73f,.25f,.82f,-.27f,-.16f,.35f,-.61f,.39f,.79f,-.57f,-.41f,-.56f,-.35f,.26f,-.34f,.74f,-.10f,.98f,-.55f,-.38f,-.91f,-.06f,-.85f,.62f,-.34f,.51f,-.83f,.91f,-.09f,-.54f,-.79f,-.37f,.79f,.74f,-.86f,-.52f,.45f,.04f,.65f,-.26f,.70f,.88f,-.13f,.36f,.23f,.06f,.73f,.16f,-.53f,-.91f,-.56f,-.07f,.72f,-.17f,-.13f,.56f,.11f,.19f,.46f,-.31f,.76f,.95f,.68f,.12f,.60f,-.20f,-.89f,.80f,.78f,-.21f,-.04f,.79f,-.18f,.28f,-.37f,-.39f,.92f,-.73f,-.55f,-.32f,-.15f,-.20f,-.59f,.71f,.57f,-.60f,-.58f,-.36f,.05f,.22f,.08f,-.37f,-.73f,.00f,.99f,-.44f,-.83f,.30f,-.39f,-.49f,.40f,-.33f,-.64f,-.07f,.24f,-.80f,.38f,.65f,.18f,.88f,.99f,-.18f,-.08f,.45f,-.76f,-.30f,.87f,.24f,-.21f,-.28f,.62f,-.71f,.57f,.92f,.01f,.70f,.44f,-.77f,-.47f,-.70f,.72f,-.67f,-.60f,.08f,-.26f,.26f,.16f,-.94f,-.06f,.46f,.98f,-.90f,.17f,-.45f,-.43f,-.00f,.71f,-.82f,-.59f,-.44f,.03f,.98f,-.85f,-.33f,.45f,.22f,-.04f,.59f,.87f,.91f,.97f,-.97f,.69f,-.60f,.61f,-.59f,-.56f,-.71f,-.35f,.13f,.06f,.81f,-.19f,.45f,.56f,-.03f,.09f,.28f,-.75f,.97f,.70f,.98f,-.56f,-.62f,.52f,-.57f,.48f,.04f,-.31f,-.60f,-.22f,-.39f,.05f,-.27f,.15f,-.40f,-.47f,.74f,.14f,.96f,-.35f,-.12f,-.95f,.32f,-.99f,.06f,-.22f,-.82f,-.89f,.89f,.00f,.98f,-.90f,.48f,.79f,.87f,.10f,-.29f,.79f,-.88f,-.07f,-.95f,-.70f,.06f,-.44f,.17f,.94f,.54f,.56f,.83f,-.89f,.80f,-.41f,.40f,-.13f,.44f,-.20f,-.29f,.59f,-.58f,-.80f,-.37f,-.33f,-.10f,-.51f,-.90f,-.16f,-.14f,.97f,.06f,.77f,-.70f,-.70f,.30f,-.36f,-.17f,-.17f,.70f,.25f,.24f,-.14f,.22f,.81f,-.16f,.07f,-.89f,.29f,-.60f,-.95f,.26f,-.20f,-.04f,-.90f,.55f,.80f,-.66f,.51f,.59f,.71f,-.51f,.73f,.89f,-.74f,.88f,.70f,.08f,.61f,-.83f,-.89f,-.93f,.34f,-.41f,.85f,-.74f,-.46f,-.48f,.05f,.86f,-.23f,.82f,.92f,-.41f,-.61f,-.76f,.22f,-.80f,-.29f,.18f,-.05f,-.96f,-.47f,.74f,.81f,-.73f,.69f,-.74f,-.05f,-.80f,-.89f,.13f,-.50f,-.40f,.16f,.03f,.85f,-.23f,.63f,.88f,-.77f,-.58f,.47f,-.70f,-.40f,.61f,.62f,-.23f,-.46f,-.19f,.14f,-.18f,.67f,.21f,-.57f,-.75f,-.04f,.20f,.82f,-.18f,-.44f,-.48f,.11f,.50f,-.60f,.67f,-.60f,.48f,.58f,-.92f,.20f,-.86f,.48f,-.87f,.47f,-.43f,.04f,-.90f,-.50f,-.79f,-.28f,-.25f,-.20f,.49f,-.01f,.28f,.83f,-.03f,-.32f,-.72f,.76f,.80f,-.53f,-.64f,-.34f,.59f,-.81f,.20f,.10f,-.13f,-.40f,-.51f,-.57f,.13f,.93f,.89f,-.03f,-.09f,-.96f,-.41f,-.32f,.14f,-.94f,-.21f,.51f,.03f,-.50f,.54f,-.52f,-.23f,.22f,-.73f,-.49f,-.80f,-.84f,-.26f,.53f,-.61f,.81f,-.80f,.83f,.35f,.49f,-.39f,-.10f,.64f,.12f,.65f,.78f,-.84f,-.32f,.01f,-.38f,-.98f,-.75f,.68f,.16f,.56f,-.23f,-.87f,.37f,.19f,.71f,-.73f,.07f,-.17f,.20f,.08f,.33f,-.49f,.67f,-.58f,-.14f,-.60f,-.89f,-.58f,-.06f,.11f,-.14f,.94f,-.81f,-.54f,-.63f,-.89f,-.35f,.90f,.73f,.67f,-.27f,-.36f,.00f,.11f,.48f,.38f,-.78f,.90f,-.36f,-.74f,-.50f,.90f,-.77f,.78f,-.08f,.76f,-.91f,-.97f,-.38f,-.00f,.59f,.79f,-.53f,.21f,.34f,-.79f,.90f,-.42f,-.14f,-.44f,-.25f,-.29f,.34f,-.08f,.76f,.53f,.24f,.07f,.01f,-.29f,-.76f,-.43f,-.40f,.23f,-.66f,.62f,.79f,-.07f,-.26f,.62f,-.18f,-.66f,.90f,.19f,.58f,.59f,-.24f,-.75f,-.30f,.13f,.68f,.08f,.00f,-.00f,.81f,-.56f,.41f,.58f,.50f,-.40f,.59f,-.45f,-.56f,.66f,.02f,.74f,-.27f,.70f,-.50f,-.76f,.24f,.94f,-.61f,.74f,-.95f,.76f,-.24f,.09f,.87f,-.02f,-.73f,-.43f,-.26f,.39f,-.78f,-.50f,-.52f,-.84f,-.81f,-.51f,.86f,-.27f,.27f,-.92f,.39f,.02f,.12f,-.47f,-.75f,-.69f,.00f,-.46f,.19f,.21f,-.75f,.65f,.12f,.74f,-.92f,-.88f,.68f,.85f,-.38f,-.68f,.43f,-.88f,.12f,-.02f,-.77f,.27f,.40f,.53f,.95f,.99f,.56f,.62f,-.68f,-.69f,-.87f,.09f,-.91f,-.25f,-.66f,.19f,-.69f,-.54f,.46f,.21f,-.76f,.84f,-.83f,-.30f,-.30f,.16f,.38f,.39f,-.04f,.32f,-.68f,-.43f,.27f,.08f,.41f,.93f,-.61f,-.54f,.52f,-.99f,.04f,.74f,-.26f,.25f,-.28f,-.60f,.52f,-.68f,.75f,-.62f,-.32f,-.51f,-.69f,-.61f,.90f,-.71f,.81f,-.20f,.24f,.92f,-.25f,-.76f,.88f,.88f,.42f,-.21f,.11f,.33f,-.09f,.01f,-.72f,.96f,.20f,-.70f,-.59f,.48f,-.03f,.42f,.54f,.67f,-.21f,-.13f,.55f,-.11f,.26f,.52f,.83f,.76f,.21f,.51f,-.35f,.49f,.04f,-.62f,.88f,.04f,.00f,.33f,-.68f,.72f,.78f,-.69f,.54f,.04f,-.69f,-.51f,.56f,-.05f,-.68f,.16f,.62f,-.90f,.77f,-.25f,-.78f,.25f,-.86f,.72f,-.82f,-.13f,-.23f,.77f,-.06f,.13f,.29f,-.79f,.13f,-.77f,-.56f,-.22f,-.17f,-.20f,-.35f,.91f,-.92f,.70f,-.07f,.99f,-.44f,.00f,-.91f,.36f,-.69f,-.97f,.97f,-.15f,.76f,.78f,-.36f,-.46f,.43f,.78f,-.08f,-.36f,.91f,-.15f,-.49f,.89f,-.47f,-.88f,.32f,-.35f,.46f,.86f,.19f,.87f,.76f,.41f,.87f,.71f,.11f,.78f,-.87f,.70f,-.29f,-.32f,.01f,-.68f,.19f,-.59f,-.04f,.89f,-.46f,.46f,-.82f,.02f,.35f,-.93f,-.46f,.58f,.03f,-.81f,.27f,-.90f,-.33f,-.93f,-.47f,-.42f,-.27f,-.49f,.82f,.12f,.82f,.29f,-.35f,.74f,.00f,-.70f,.45f,-.19f,.05f,-.39f,-.84f,-.99f,-.22f,-.71f,-.60f,-.54f,.21f,.61f,.18f,.08f,.19f,-.06f,-.98f,.21f,-.78f,.15f,.68f,-.53f,.31f,.89f,-.01f,-.73f,.02f,.40f,-.37f,.23f,-.95f,.62f,.84f,-.35f,-.63f,.96f,-.47f,.14f,.80f,-.68f,.29f,.99f,.49f,-.01f,.16f,.19f,-.38f,.63f,-.93f,-.31f,.83f,-.64f,-.72f,.49f,-.87f,.09f,.89f,-.76f,-.98f,.36f,.71f,.07f,.99f,-.47f,-.41f,.68f,.86f,.77f,.92f,-.79f,-.22f,.24f,.09f,-.45f,.00f,.22f,-.96f,-.50f,.87f,.06f,.93f,-.39f,-.39f,-.39f,-.36f,-.11f,.77f,-.29f,.32f,-.60f,-.69f,-.35f,.12f,-.30f,.66f,.29f,-.45f,-.18f,-.95f,.75f,-.69f,-.71f,-.93f,-.25f,-.36f,.81f,-.79f,.17f,.93f,-.60f,-.98f,.55f,-.87f,-.67f,.32f,.80f,.79f,.62f,.44f,-.19f,-.48f,.34f,.89f,.72f,.37f,-.82f,-.49f,-.71f,-.40f,-.08f,-.05f,-.30f,-.46f,.50f,.24f,.85f,-.95f,-.62f,-.45f,-.09f,.42f,-.95f,-.20f,.21f,-.50f,.39f,-.38f,-.74f,.69f,-.65f,.43f,.60f,-.48f,-.81f,-.14f,-.90f,.56f,.42f,.39f,-.13f,.76f,.06f,-.46f,-.55f,.60f,.72f,-.05f,-.20f,-.17f,-.40f,.23f,-.55f,.90f,-.49f,.58f,.34f,.72f,-.72f,-.54f,-.43f,-.80f,-.49f,-.76f,-.76f,-.11f,.55f,-.68f,.14f,-.98f,.20f,.93f,.19f,.52f,-.26f,.89f,-.82f,-.02f,-.47f,.74f,-.94f,-.01f,.90f,.68f,.20f,.36f,-.50f,-.99f,.20f,.05f,-.18f,-.06f,-.39f,.60f,-.12f,-.94f,.89f,-.31f,.00f,.35f,.74f,-.24f,-.88f,.41f,-.78f,-.04f,.38f,-.17f,.93f,.48f,.31f,.60f,-.20f,-.78f,.85f,-.03f,.26f,.88f,.17f,.82f,.39f,-.03f,.10f,.13f,.72f,.15f,.40f,-.39f,.32f,.44f,-.75f,-.64f,-.29f,.77f,-.50f,.74f,.71f,-.08f,-.30f,-.42f,.17f,-.52f,.84f,-.07f,-.89f,-.60f,-.04f,.64f,-.73f,-.10f,-.82f,.87f,-.78f,.23f,.67f,-.98f,-.68f,.66f,.82f,.64f,.40f,.55f,.91f,-.02f,.01f,.13f,-.82f,-.86f,.79f,-.06f,-.58f,.75f,.99f,-.05f,.32f,-.07f,.97f,.49f,.40f,.42f,.86f,-.62f,.54f,.57f,.90f,-.76f,.54f,-.61f,.32f,.66f,.21f,-.24f,-.70f,.02f,.27f,.78f,-.45f,.29f,.18f,-.06f,.78f,.38f,-.41f,.14f,.50f,-.43f,-.45f,.68f,-.30f,-.79f,-.88f,.57f,-.06f,.36f,.16f,.14f,.48f,-.27f,.46f,-.78f,-.23f,.20f,.34f,.26f,.34f,-.40f,.07f,.00f,.28f,-.92f,-.23f,.46f,-.60f,-.69f,-.98f,.49f,.21f,-.65f,-.01f,.56f,-.42f,.88f,.96f,.40f,.60f,-.24f,-.93f,-.82f,.64f,.23f,-.24f,.28f,-.50f,.48f,.23f,.52f,.75f,.30f,.59f,-.63f,.46f,.47f,-.91f,.70f,.29f,-.47f,.12f,-.82f,.72f,-.20f,-.48f,.07f,.39f,.72f,.24f,.44f,.79f,-.44f,-.59f,.25f,-.95f,-.60f,.24f,-.40f,.50f,-.05f,-.86f,.49f,.95f,.92f,-.87f,.29f,-.71f,-.14f,.37f,.24f,.91f,.82f,-.64f,.12f,-.39f,.53f,-.95f,.32f,.87f,-.90f,.97f,-.85f,.09f,-.54f,-.75f,.74f,-.59f,.51f,-.23f,-.14f,-.64f,.51f,.39f,.60f,.36f,.16f,-.64f,.27f,-.16f,.32f,-.99f,.77f,.48f,-.71f,.55f,.73f,.44f,-.92f,-.29f,.44f,.10f,-.53f,-.71f,-.09f,-.46f,-.66f,-.31f,.05f,.92f,-.52f,.04f,.94f,.91f,.09f,.03f,.23f,-.88f,-.92f,.91f,-.47f,.04f,.37f,-.75f,.74f,.77f,-.69f,-.02f,-.90f,.64f,-.38f,-.21f,.76f,.71f,.38f,-.89f,.36f,-.17f,.86f,.13f,.45f,.36f,-.06f,.40f,.33f,-.53f,-.47f,.71f,.10f,.63f,.84f,-.22f,.07f,.76f,.98f,-.43f,-.54f,.42f,-.23f,-.75f,-.67f,.62f,.55f,.24f,-.77f,-.45f,.77f,.30f,-.80f,-.47f,.70f,.03f,-.26f,-.10f,.16f,.98f,-.33f,-.64f,.67f,.31f,-.61f,-.09f,.54f,-.23f,-.18f,-.41f,-.32f,-.43f,.35f,-.85f,-.45f,.53f,.24f,.86f,-.45f,.37f,.37f,-.87f,-.44f,-.67f,.66f,-.78f,.84f,.10f,.74f,-.33f,-.74f,.28f,.30f,.43f,.57f,.48f,-.36f,-.84f,.82f,.03f,.36f,.04f,.93f,-.42f,.73f,-.33f,.54f,-.85f,.28f,.19f,.31f,-.06f,.59f,.01f,.46f,-.68f,-.79f,-.91f,.95f,-.04f,.59f,.66f,-.03f,-.35f,.77f,.27f,.48f,.04f,-.91f,-.90f,-.38f,-.43f,.19f,.47f,.10f,.01f,-.15f,.38f,.74f,.24f,.68f,.66f,-.06f,-.09f,.79f,.06f,-.98f,-.26f,-.11f,-.01f,-.63f,.17f,-.20f,.94f,.04f,.15f,-.41f,-.88f,.62f,-.03f,-.95f,-.99f,-.35f,-.79f,-.08f,.13f,.45f,-.79f,.30f,-.97f,.53f,-.87f,-.00f,.41f,-.90f,.46f,.14f,-.71f,-.34f,.06f,-.93f,-.31f,-.91f,.88f,.76f,-.69f,.51f,-.97f,.40f,-.95f,.75f,-.95f,-.33f,.74f,-.86f,.17f,.73f,.70f,-.41f,.11f,.91f,.94f,-.67f,.40f,.18f,.01f,.17f,-.10f,.24f,.54f,-.01f,.30f,.21f,.54f,-.37f,-.56f,.76f,-.57f,.23f,-.87f,.35f,.35f,.84f,-.47f,.32f,-.51f,.45f,.03f,.84f,.02f,.74f,.40f,-.56f,.08f,.36f,.97f,.25f,.03f,.48f,-.03f,.65f,.71f,-.04f,.85f,.05f,.13f,-.08f,.81f,.32f,-.34f,-.79f,-.45f,.77f,-.89f,.79f,-.23f,-.65f,-.43f,.55f,.99f,.68f,-.43f,.25f,-.19f,-.32f,-.03f,-.61f,.01f,.10f,.14f,.82f,.50f,.96f,.24f,-.44f,-.21f,.05f,-.28f,.58f,.29f,-.08f,.83f,-.95f,-.21f,-.49f,.61f,-.37f,-.47f,.50f,-.33f,-.83f,.00f,-.68f,.06f,-.96f,-.74f,.57f,.34f,-.54f,-.77f,-.73f,.84f,.81f,-.90f,.95f,.57f,-.79f,.35f,.54f,-.50f,-.86f,.13f,.16f,.75f,.64f,-.74f,-.72f,-.85f,.87f,-.71f,.43f,-.06f,-.69f,.12f,-.06f,.36f,-.34f,.79f,.02f,-.83f,.91f,.40f,.46f,.90f,-.72f,.02f,.01f,-.45f,.30f,-.12f,-.09f,-.14f,.55f,-.33f,.34f,-.82f,.90f,.45f,.95f,-.62f,-.49f,.06f,-.15f,.66f,.10f,-.92f,.71f,.04f,-.28f,-.83f,-.37f,.18f,.37f,.49f,.82f,-.53f,-.30f,.17f,-.65f,-.06f,.29f,-.89f,-.74f,-.21f,-.85f,.93f,-.51f,.29f,-.86f,-.20f,-.19f,.39f,-.51f,.54f,-.67f,.27f,-.23f,.23f,.43f,.65f,.71f,.95f,-.84f,.84f,-.44f,-.78f,.05f,.84f,.05f,-.54f,-.18f,-.41f,-.80f,.05f,-.41f,-.97f,.34f,.64f,-.94f,.19f,.22f,.85f,.58f,.92f,-.07f,-.41f,.30f,-.01f,.72f,-.28f,.91f,.82f,.92f,.17f,-.08f,-.69f,-.92f,.05f,.45f,-.72f,-.34f,.36f,.35f,-.18f,.55f,-.21f,-.69f,-.60f,.86f,.85f,.47f,.48f,.49f,-.31f,.03f,-.42f,-.28f,.38f,-.94f,.82f,.25f,-.40f,-.95f,.81f,-.21f,.50f,-.55f,-.02f,-.91f,-.10f,.45f,-.30f,.31f,.80f,-.57f,.41f,-.83f,.28f,-.02f,.11f,-.45f,.84f,.03f,.14f,.76f,-.21f,-.04f,-.52f,-.55f,-.03f,-.98f,.48f,.91f,-.39f,.57f,.45f,.29f,-.05f,-.33f,-.65f,-.62f,.98f,-.35f,.61f,.81f,.61f,-.51f,.43f,-.36f,.73f,-.20f,-.18f,.05f,.04f,-.83f,.38f,-.98f,-.62f,.07f,.65f,-.23f,.49f,-.33f,-.85f,.33f,-.51f,-.08f,-.21f,.66f,.84f,-.36f,-.01f,.99f,.48f,-.79f,.18f,.82f,.11f,-.38f,-.49f,.24f,.20f,.16f,.60f,-.33f,.58f,.77f,-.43f,-.65f,.81f,.14f,.36f,.00f,-.87f,-.57f,-.01f,-.02f,.30f,.14f,-.17f,.18f,.17f,.64f,.64f,.57f,-.72f,.12f,-.33f,.73f,.54f,-.36f,.16f,.62f,.01f,.09f,.36f,-.65f,-.72f,-.02f,-.40f,.55f,-.54f,-.20f,.24f,.97f,-.31f,.96f,.74f,.71f,.10f,.79f,-.64f,.17f,-.56f,.27f,-.62f,-.65f,.73f,.17f,-.06f,.67f,-.39f,-.46f,.25f,.64f,-.22f,-.13f,.35f,.70f,-.70f,-.60f,-.14f,-.53f,-.09f,.59f,-.09f,-.12f,-.28f,-.65f,.69f,.48f,.11f,-.82f,-.15f,-.01f,.06f,.96f,-.14f,.51f,.39f,.66f,-.27f,-.52f,.54f,-.87f,-.18f,-.69f,-.19f,-.20f,.19f,-.06f,.27f,.18f,-.26f,-.45f,-.09f,.47f,-.15f,-.05f,.54f,.80f,.86f,-.60f,-.02f,.74f,.07f,-.10f,-.73f,.56f,.67f,.51f,.23f,.69f,.85f,.21f,-.92f,.15f,.53f,.12f,.43f,-.26f,-.50f,.29f,.92f,-.29f,-.25f,.94f,-.92f,.53f,.14f,.93f,.88f,.09f,-.29f,.59f,.98f,.49f,.64f,-.18f,-.75f,-.26f,-.47f,.64f,.87f,.46f,-.51f,.04f,-.46f,.00f,-.67f,-.78f,.05f,-.35f,.21f,-.42f,-.85f,.02f,.78f,.86f,-.93f,-.28f,.54f,-.41f,-.83f,.95f,-.72f,.31f,.67f,-.22f,-.07f,.74f,.26f,.27f,-.78f,-.23f,.57f,.55f,-.27f,.17f,.57f,.63f,-.05f,-.76f,-.25f,.37f,-.29f,-.48f,.92f,-.87f,-.08f,-.68f,.61f,.24f,-.48f,-.70f,.61f,-.01f,-.96f,-.54f,.08f,-.75f,.06f,.60f,-.89f,.34f,.76f,-.25f,.34f,-.94f,.00f,.60f,-.24f,-.10f,-.98f,-.56f,-.74f,-.36f,.27f,.74f,.88f,-.94f,.25f,-.85f,-.39f,-.37f,.25f,-.80f,.10f,.52f,-.53f,.20f,.59f,-.17f,-.57f,.45f,.39f,-.63f,.37f,.45f,-.81f,-.16f,.18f,.66f,-.23f,-.98f,-.81f,.47f,.18f,.24f,.38f,-.79f,-.73f,.28f,-.81f,.39f,-.15f,.93f,-.83f,-.42f,-.70f,-.09f,-.95f,.23f,.25f,-.44f,-.79f,-.68f,-.26f,-.04f,.05f,-.39f,-.23f,.22f,-.25f,-.30f,-.00f,-.03f,-.42f,.94f,.27f,.98f,-.32f,.88f,.49f,.75f,.76f,.02f,-.10f,.26f,-.54f,-.00f,-.91f,.30f,.48f,.30f,-.54f,.58f,.13f,.95f,.43f,-.90f,-.79f,.82f,-.03f,.28f,-.69f,.87f,-.35f,-.34f,.97f,-.17f,.03f,.61f,.32f,-.05f,.47f,-.35f,-.48f,-.50f,.37f,.84f,.50f,-.82f,.26f,.47f,-.36f,-.42f,.58f,.71f,.87f,.59f,.92f,.31f,.66f,-.28f,-.13f,-.19f,-.95f,.57f,-.81f,.95f,.16f,.62f,-.60f,-.03f,.75f,.40f,.40f,.78f,.95f,.59f,-.00f,.79f,.49f,-.76f,.91f,-.76f,-.74f,-.68f,.97f,.98f,-.97f,.91f,-.93f,.47f,.86f,.88f,-.56f,.84f,-.00f,.02f,-.23f,.95f,.58f,-.00f,.84f,-.65f,.77f,-.27f,-.42f,-.54f,.19f,-.77f,-.59f,.81f,.45f,.96f,.39f,-.46f,-.74f,-.59f,.93f,-.12f,.26f,-.74f,.28f,-.83f,.58f,.20f,.25f,-.39f,.85f,.29f,.90f,-.42f,.95f,.49f,-.11f,-.66f,.65f,.91f,-.98f,-.61f,-.72f,.20f,.25f,.39f,.76f,-.13f,.56f,.78f,.90f,.10f,.54f,.49f,-.85f,.50f,.91f,-.72f,-.00f,.80f,-.25f,.07f,-.18f,.87f,-.93f,.02f,.03f,-.15f,.37f,.50f,-.21f,-.17f,.78f,-.40f,.70f,-.96f,-.74f,.05f,.22f,-.04f,.78f,.99f,.65f,.63f,.66f,.40f,.53f,-.91f,.46f,.43f,.18f,.65f,-.58f,.73f,.60f,.57f,.72f,.71f,.15f,.70f,-.47f,-.82f,-.38f,.89f,-.91f,.72f,.13f,-.81f,-.79f,-.58f,-.13f,.28f,.79f,.46f,.34f,.27f,.26f,.00f,.39f,.89f,.24f,.83f,.91f,-.87f,-.49f,.30f,-.11f,.77f,-.97f,-.36f,-.41f,-.68f,.85f,.79f,.30f,.81f,-.57f,.91f,-.04f,-.22f,-.16f,.95f,.60f,.27f,-.73f,.26f,-.18f,.50f,-.79f,.10f,-.69f,-.23f,.07f,.76f,-.41f,-.13f,-.64f,-.15f,.47f,-.03f,.93f,.95f,-.67f,.72f,-.80f,-.17f,.98f,.45f,-.91f,.94f,-.67f,.51f,.81f,-.37f,.49f,.43f,.26f,.83f,-.88f,-.10f,.05f,.41f,-.13f,-.32f,.75f,.37f,-.26f,-.42f,-.41f,.79f,.37f,-.06f,.77f,-.61f,.23f,.57f,.28f,.63f,-.43f,-.09f,.22f,-.86f,-.07f,.22f,.48f,-.05f,.71f,-.98f,.47f,.46f,.02f,-.52f,-.24f,.35f,-.02f,-.30f,-.80f,-.18f,-.81f,.19f,-.74f,-.04f,-.58f,-.51f,-.71f,-.28f,.23f,-.51f,-.05f,.88f,-.55f,.54f,.41f,-.06f,-.58f,.32f,.93f,-.14f,-.61f,-.05f,.13f,-.73f,-.19f,-.49f,.59f,.53f,.97f,.64f,.02f,.39f,-.95f,-.87f,-.87f,.96f,-.90f,-.68f,.74f,-.76f,-.08f,-.19f,-.11f,-.32f,.95f,-.13f,.70f,-.81f,.67f,.09f,-.64f,-.24f,.70f,.46f,.28f,.21f,-.10f,.49f,-.22f,.64f,.34f,-.10f,-.78f,-.11f,-.04f,-.31f,-.63f,-.45f,.12f,.71f,.88f,-.41f,.30f,.51f,-.73f,.54f,-.00f,.39f,.62f,.81f,.65f,-.26f,-.35f,.84f,-.12f,.77f,.65f,.77f,-.42f,.54f,.95f,-.82f,.46f,.83f,.15f,-.11f,-.06f,.41f,.87f,-.74f,-.24f,.37f,.94f,-.42f,-.56f,-.01f,-.15f,.03f,.49f,-.75f,.18f,.42f,.54f,.39f,.70f,-.49f,.08f,.68f,-.25f,.36f,.38f,.44f,-.52f,.86f,-.02f,.69f,.90f,.89f,-.77f,.14f,-.78f,-.64f,.40f,-.09f,.78f,-.81f,.94f,.46f,-.53f,.04f,-.46f,.71f,-.85f,.66f,-.51f,-.09f,.91f,.38f,-.26f,.91f,-.04f,-.86f,-.03f,.83f,-.69f,.35f,.65f,-.03f,.65f,.68f,.56f,.30f,-.46f,.73f,.02f,-.21f,.85f,.48f,.91f,.05f,.15f,-.32f,.39f,-.33f,-.84f,-.66f,-.41f,.98f,-.03f,.56f,.30f,.37f,-.86f,.60f,.47f,-.24f,.53f,-.85f,.66f,.53f,-.31f,-.44f,.88f,.34f,-.73f,-.48f,-.33f,-.73f,.56f,.45f,.18f,-.76f,.56f,.23f,-.83f,.97f,-.48f,-.80f,-.66f,.17f,.74f,.39f,-.02f,.49f,-.33f,-.73f,-.81f,.27f,.21f,.38f,-.68f,-.20f,-.09f,-.96f,-.61f,-.93f,-.99f,-.16f,.15f,.62f,.24f,.74f,-.65f,.35f,.26f,.38f,-.64f,-.67f,.80f,.26f,.30f,-.56f,.20f,-.82f,.06f,.81f,.75f,.82f,-.82f,.55f,-.91f,.37f,.16f,-.03f,-.13f,.09f,-.67f,-.33f,-.18f,-.13f,-.98f,-.81f,-.72f,.49f,-.96f,-.22f,.65f,.35f,-.35f,-.60f,-.64f,.19f,-.57f,.73f,-.70f,-.89f,.46f,-.77f,-.32f,.55f,-.81f,-.61f,.97f,-.49f,.30f,.42f,.66f,.19f,.55f,-.72f,-.95f,-.65f,-.28f,-.75f,-.22f,.29f,.56f,-.02f,-.15f,-.80f,.68f,-.05f,-.35f,.68f,-.84f,.95f,-.14f,.04f,.64f,.49f,.02f,.74f,.57f,.58f,.01f,.62f,-.87f,.76f,.37f,.27f,-.51f,.07f,.31f,.39f,.61f,.60f,-.14f,.62f,.01f,-.10f,-.43f,-.23f,-.91f,.99f,.58f,.81f,-.39f,-.98f,.96f,.02f,.38f,-.94f,-.46f,-.02f,.46f,.48f,-.39f,-.46f,.89f,-.62f,.10f,.81f,.53f,-.96f,-.33f,-.58f,-.17f,-.73f,.12f,.05f,.63f,-.83f,.01f,-.02f,-.00f,-.01f,-.79f,.78f,.48f,-.44f,.46f,.48f,-.72f,-.53f,.08f,.00f,-.12f,-.93f,-.87f,-.32f,-.49f,.35f,-.63f,-.20f,.99f,.01f,-.23f,.41f,-.32f,.12f,.85f,.31f,.43f,.21f,.60f,-.33f,-.71f,-.84f,-.23f,-.06f,-.23f,.08f,.89f,.99f,.64f,.65f,-.55f,.60f,-.07f,-.62f,.74f,.57f,.85f,.27f,.56f,-.87f,.00f,-.37f,-.67f,.07f,.49f,.44f,-.92f,.48f,.70f,-.86f,.33f,.39f,.71f,.43f,.80f,-.52f,.01f,-.54f,-.95f,-.39f,.40f,-.75f,.08f,.76f,-.62f,-.39f,.94f,.31f,.09f,-.15f,.83f,-.77f,.05f,.85f,.68f,-.32f,.94f,-.09f,-.57f,.46f,-.01f,-.77f,.68f,.44f,.01f,-.38f,.82f,-.22f,.74f,.75f,-.43f,-.26f,.01f,.87f,.15f,-.93f,-.51f,-.23f,-.57f,.88f,.76f,-.39f,.78f,-.40f,.78f,-.86f,-.15f,.47f,-.75f,-.26f,-.54f,-.00f,-.42f,.40f,.53f,-.34f,.26f,-.20f,-.09f,-.89f,.96f,.44f,-.56f,-.58f,-.40f,-.69f,.21f,-.74f,.42f,.94f,.95f,-.01f,.93f,.22f,-.40f,-.56f,.42f,-.97f,-.72f,.59f,-.69f,.52f,.62f,-.13f,-.47f,.18f,.00f,.37f,.58f,-.61f,.28f,-.85f,.52f,-.69f,-.62f,.54f,-.90f,.89f,.86f,-.83f,-.57f,.32f,.43f,.35f,-.35f,.02f,.93f,.77f,.32f,.52f,.58f,.76f,.35f,-.55f,-.10f,-.07f,-.82f,-.40f,.25f,.91f,.81f,.60f,-.58f,.64f,.20f,.97f,.61f,-.57f,.93f,.69f,-.42f,.99f,-.80f,.59f,.48f,.13f,.42f,-.78f,-.83f,.72f,.49f,.56f,-.69f,-.89f,.95f,-.45f,-.15f,-.39f,.12f,-.85f,-.85f,-.81f,-.63f,.33f,-.00f,.14f,-.46f,-.13f,.44f,.97f,-.03f,.58f,.91f,-.34f,.56f,.79f,.36f,.34f,.00f,-.16f,.98f,-.32f,-.91f,.51f,.65f,-.71f,-.53f,.00f,.41f,.83f,-.42f,-.52f,-.82f,.46f,.20f,-.61f,.50f,.82f,.43f,.05f,-.29f,.92f,.87f,-.57f,-.56f,.10f,.06f,-.95f,-.26f,.35f,-.83f,-.41f,.85f,-.43f,.61f,-.71f,.70f,.70f,-.50f,-.94f,.89f,-.78f,-.64f,.66f,-.37f,-.24f,-.74f,-.40f,.62f,-.01f,.11f,.47f,-.85f,-.10f,-.11f,-.91f,.48f,.43f,.01f,.71f,.66f,.15f,-.33f,-.07f,-.92f,-.77f,-.44f,.33f,-.89f,-.52f,.46f,-.68f,.37f,-.54f,.11f,-.08f,.60f,-.13f,-.01f,-.38f,-.31f,-.32f,.10f,-.58f,-.12f,-.62f,.97f,-.59f,-.26f,-.26f,.08f,-.59f,.91f,.33f,-.12f,-.22f,-.08f,-.86f,.02f,.65f,.56f,-.13f,.34f,.17f,.59f,.62f,.70f,.57f,.73f,-.86f,-.21f,.64f,-.23f,-.68f,.92f,.75f,-.05f,-.63f,-.61f,.52f,.77f,-.92f,-.71f,.73f,-.62f,.32f,-.38f,.99f,-.58f,-.77f,.58f,-.25f,.08f,.87f,.12f,.03f,.60f,-.07f,-.31f,-.70f,.34f,.71f,-.94f,-.72f,.37f,.18f,.71f,.17f,-.75f,-.90f,.24f,-.53f,-.05f,.68f,-.19f,.62f,-.62f,.67f,.02f,.78f,-.85f,.24f,-.57f,.02f,-.31f,-.23f,-.95f,-.78f,-.51f,.46f,.33f,-.06f,-.05f,.75f,-.07f,-.69f,-.55f,-.79f,.39f,-.28f,.92f,-.47f,-.34f,-.04f,.32f,-.35f,.97f,-.45f,.65f,.51f,-.90f,-.87f,.48f,.40f,.46f,.82f,-.02f,-.55f,.85f,.12f,-.10f,.65f,-.23f,.99f,.91f,-.02f,-.89f,-.67f,-.92f,.06f,-.16f,-.97f,-.17f,.11f,.37f,.59f,-.85f,-.43f,-.24f,-.78f,-.35f,.07f,-.26f,.65f,-.86f,-.29f,-.82f,-.03f,.25f,-.23f,.51f,-.61f,.68f,.88f,-.35f,-.17f,.36f,.36f,.80f,-.80f,-.52f,.90f,.65f,.82f,-.19f,-.10f,.62f,.87f,-.11f,-.67f,-.24f,.28f,.80f,.69f,.36f,.06f,.02f,-.15f,-.85f,-.94f,.18f,-.67f,-.03f,.58f,.95f,.70f,.04f,-.66f,-.21f,.77f,-.87f,-.96f,-.86f,-.96f,-.33f,.46f,-.08f,-.15f,.24f,.83f,-.10f,-.90f,-.47f,.56f,-.10f,.96f,-.13f,.12f,.15f,-.14f,-.88f,-.27f,.86f,-.51f,-.00f,.13f,.56f,.00f,.03f,-.64f,.05f,.44f,.85f,.15f,-.70f,.26f,-.57f,-.59f,-.90f,-.42f,.46f,.90f,-.47f,-.05f,-.29f,.77f,-.47f,.63f,-.72f,.24f,-.90f,-.96f,.61f,.74f,-.90f,.12f,-.92f,-.76f,.34f,-.53f,-.50f,.75f,.78f,.13f,.39f,.10f,.57f,-.10f,.00f,.61f,-.29f,.92f,-.89f,.70f,-.25f,.01f,.56f,.17f,-.51f,.78f,.03f,-.45f,.37f,-.65f,.65f,-.18f,.28f,-.04f,-.68f,-.48f,.06f,.94f,.56f,.92f,.96f,-.46f,-.69f,.89f,-.43f,-.68f,.87f,-.73f,.93f,.02f,.99f,.41f,-.78f,.06f,-.56f,.04f,.68f,-.79f,.26f,.39f,.92f,-.71f,-.17f,-.36f,-.48f,-.17f,.97f,-.94f,-.46f,-.84f,-.79f,.40f,.20f,-.71f,-.70f,.28f,-.27f,-.63f,-.39f,-.36f,.35f,-.69f,-.94f,-.17f,-.97f,-.07f,-.78f,.84f,.17f,.54f,.29f,.76f,.73f,-.58f,-.03f,.27f,.44f,-.03f,-.83f,-.04f,-.93f,-.42f,.82f,-.01f,-.62f,-.01f,.61f,-.05f,-.69f,-.63f,-.02f,-.25f,-.27f,.86f,.48f,-.92f,.71f,.69f,-.93f,.23f,-.48f,.22f,.05f,-.00f,-.97f,.82f,-.47f,.53f,.68f,.65f,-.13f,-.29f,-.74f,.66f,-.47f,-.01f,-.62f,.18f,.78f,.35f,.34f,.34f,-.20f,.65f,.40f,-.78f,-.11f,.81f,-.36f,.74f,-.83f,-.66f,-.26f,.90f,.02f,.55f,-.97f,.54f,.50f,.25f,.57f,-.49f,-.15f,-.45f,.50f,-.79f,-.90f,-.91f,.31f,.97f,.77f,.76f,.73f,.40f,-.04f,-.25f,.71f,.44f,.71f,.59f,.88f,.39f,-.23f,.99f,.94f,.65f,.52f,-.55f,-.30f,-.81f,.21f,.37f,.47f,.26f,-.07f,-.08f,.62f,.99f,-.52f,-.04f,.75f,-.67f,-.23f,-.16f,-.25f,.50f,-.17f,-.09f,.16f,-.92f,.62f,.70f,.28f,.00f,.87f,-.73f,.97f,-.31f,.94f,.56f,-.00f,.37f,.05f,-.70f,-.03f,.80f,.95f,.08f,-.34f,.85f,.88f,.95f,.88f,.14f,-.22f,.94f,.47f,-.85f,-.07f,.67f,.55f,.74f,.87f,-.39f,.99f,.49f,-.14f,-.20f,.79f,-.88f,.79f,-.14f,.40f,.57f,-.32f,.47f,-.93f,-.97f,-.18f,.71f,-.96f,-.50f,-.65f,.69f,-.26f,-.94f,-.07f,-.20f,.55f,-.84f,.14f,.88f,-.94f,-.62f,.42f,-.49f,.70f,-.61f,-.36f,-.92f,-.96f,-.42f,-.15f,-.55f,.51f,.56f,-.03f,.16f,.27f,.06f,-.46f,-.54f,-.74f,.48f,.79f,-.30f,-.31f,.37f,-.63f,-.53f,.65f,-.54f,-.68f,.10f,.19f,-.51f,.68f,-.81f,.20f,.38f,.83f,.14f,.81f,.29f,.95f,-.57f,.83f,-.25f,-.26f,.10f,-.23f,.00f,-.77f,-.47f,-.36f,-.33f,-.38f,.48f,.15f,-.61f,-.07f,-.35f,-.52f,.46f,-.51f,-.56f,.32f,-.51f,-.87f,-.76f,.95f,-.88f,-.53f,-.51f,.18f,.95f,-.72f,-.06f,.85f,.07f,-.84f,.90f,-.19f,.14f,.54f,-.84f,-.57f,-.38f,.23f,.45f,.66f,.73f,-.87f,-.29f,.25f,-.74f,-.87f,-.54f,.08f,-.26f,-.15f,.40f,-.62f,-.73f,-.38f,.13f,.37f,.20f,.61f,-.78f,.84f,.99f,.80f,-.99f,.52f,.85f,-.61f,-.74f,-.52f,.03f,-.17f,-.91f,.64f,.61f,.82f,.49f,-.90f,.13f,.92f,.63f,-.38f,.40f,-.12f,-.46f,-.66f,.45f,-.68f,-.32f,-.41f,.97f,-.78f,-.82f,-.93f,-.19f,.33f,-.56f,-.21f,-.37f,.61f,-.37f,-.59f,.50f,.88f,.39f,.11f,.66f,-.67f,.49f,-.07f,.51f,-.20f,.65f,-.63f,-.87f,.96f,-.58f,-.24f,.29f,-.67f,-.39f,-.18f,.48f,.79f,.97f,.24f,.07f,.19f,.54f,-.45f,-.34f,-.13f,.43f,-.55f,-.31f,-.55f,.77f,-.52f,-.20f,.30f,.81f,.74f,.63f,.01f,-.70f,.17f,.50f,.34f,.73f,.01f,-.74f,.37f,.88f,-.40f,.39f,-.07f,-.03f,.14f,.89f,.70f,-.49f,-.41f,-.66f,.20f,.95f,.34f,.69f,-.01f,-.63f,.57f,.60f,.99f,.49f,-.45f,.77f,-.26f,-.83f,.29f,.69f,-.26f,.97f,-.25f,-.96f,.68f,-.17f,.99f,-.61f,.40f,-.56f,.50f,.07f,.74f,-.16f,-.44f,.72f,.50f,.02f,.74f,.95f,-.14f,-.40f,.82f,-.82f,-.58f,.06f,.77f,-.26f,.63f,.17f,-.44f,-.08f,.72f,.16f,-.91f,-.56f,.86f,.34f,.70f,.85f,-.93f,.99f,.33f,.32f,.23f,-.40f,-.96f,.07f,.01f,-.81f,-.39f,-.82f,-.50f,-.15f,.52f,.34f,-.96f,-.28f,.16f,.72f,-.91f,.03f,-.22f,.27f,-.53f,.27f,-.53f,.95f,.48f,.22f,.61f,-.30f,.67f,-.22f,-.06f,-.56f,-.74f,.98f,-.97f,.15f,.97f,-.71f,.53f,-.09f,.58f,-.80f,.85f,-.96f,-.50f,.22f,-.43f,-.73f,.36f,-.27f,.46f,.36f,.92f,.02f,.00f,.55f,-.00f,-.68f,-.39f,-.67f,-.58f,-.50f,.39f,-.18f,.48f,-.66f,.05f,-.38f,-.48f,.42f,-.72f,.30f,.30f,.13f,-.52f,-.47f,-.07f,-.92f,-.28f,.25f,-.19f,.83f,-.30f,.66f,.57f,-.98f,-.47f,.03f,.22f,-.55f,.14f,-.04f,-.81f,.99f,-.48f,-.36f,.24f,.08f,-.82f,-.74f,-.68f,-.12f,-.57f,-.55f,-.43f,-.57f,-.37f,-.84f,-.83f,.44f,.26f,-.04f,.03f,-.58f,-.84f,.78f,-.30f,-.00f,-.82f,.12f,-.00f,.13f,.49f,-.26f,.11f,.50f,-.44f,-.87f,-.15f,-.22f,-.21f,-.76f,-.79f,.80f,.97f,.97f,.06f,.87f,.13f,-.22f,.49f,-.11f,-.36f,-.12f,-.39f,.43f,.17f,.54f,-.17f,.93f,-.21f,.80f,.60f,-.40f,.02f,.85f,.32f,-.52f,-.62f,.54f,.01f,-.93f,.07f,.97f,-.46f,-.94f,-.94f,-.93f,-.41f,.80f,-.54f,-.23f,-.08f,.60f,-.56f,.26f,.12f,-.48f,.53f,.90f,.93f,-.84f,.71f,-.60f,.18f,.52f,-.59f,-.74f,.36f,-.74f,-.25f,-.77f,.13f,-.88f,.88f,-.23f,-.29f,.82f,.26f,.57f,-.29f,.07f,-.40f,.99f,-.89f,.56f,-.19f,.95f,.50f,-.70f,.31f,.75f,.63f,.70f,-.46f,-.34f,.88f,.03f,.56f,.71f,.34f,.17f,-.79f,-.78f,.47f,.14f,.00f,.49f,.10f,.42f,.35f,-.39f,.02f,-.02f,.19f,.36f,-.71f,.30f,-.58f,-.42f,-.23f,.24f,.17f,-.42f,.77f,-.88f,.07f,-.33f,.20f,-.05f,.37f,-.74f,.87f,.00f,-.23f,-.48f,-.95f,-.97f,.12f,-.86f,.58f,-.92f,.62f,.72f,-.53f,.03f,.70f,.26f,.95f,-.44f,.91f,.09f,-.31f,.53f,.90f,.92f,-.68f,.75f,-.85f,-.04f,-.42f,-.77f,-.67f,.94f,-.57f,-.93f,-.43f,-.74f,.95f,-.53f,-.50f,.69f,.93f,.61f,-.62f,.66f,.44f,-.98f,.54f,.33f,-.61f,-.95f,.92f,-.95f,.63f,-.25f,-.36f,-.70f,-.57f,-.36f,-.42f,.34f,-.01f,.27f,-.76f,-.22f,.46f,-.89f,-.02f,.57f,.82f,-.58f,-.52f,.49f,.02f,-.04f,.73f,-.47f,-.08f,.47f,-.98f,.94f,.82f,-.90f,-.88f,-.82f,.04f,.25f,.84f,-.26f,.19f,-.43f,.32f,-.66f,.25f,.44f,.47f,-.69f,-.14f,.64f,-.15f,-.50f,-.12f,.29f,-.08f,.83f,.48f,.07f,-.51f,.24f,-.81f,-.65f,-.54f,.12f,.52f,.15f,-.55f,-.94f,.47f,-.32f,.04f,.17f,-.13f,-.13f,.46f,.13f,-.85f,-.55f,.41f,-.90f,.84f,-.17f,-.73f,.29f,-.70f,.16f,.38f,.39f,-.93f,.96f,-.45f,.54f,-.76f,-.07f,-.02f,.84f,.91f,-.09f,.94f,-.19f,.88f,-.93f,.74f,.19f,-.39f,.11f,-.36f,-.57f,.01f,-.71f,-.67f,-.54f,.54f,-.22f,.06f,-.67f,.68f,-.45f,.64f,.46f,.69f,.75f,.94f,-.40f,-.05f,-.89f,-.07f,-.47f,.03f,-.31f,.54f,-.88f,.90f,-.37f,.01f,-.23f,-.96f,.12f,-.42f,.91f,-.62f,-.06f,.78f,.63f,-.72f,.28f,.11f,.11f,.15f,.73f,-.91f,.36f,.42f,.25f,-.36f,-.27f,.42f,-.51f,-.44f,.87f,.33f,.47f,.84f,.05f,.37f,-.60f,-.66f,.83f,.84f,.43f,.15f,-.90f,-.48f,-.59f,-.98f,.80f,-.43f,-.01f,.43f,.62f,-.52f,.36f,-.55f,.14f,.08f,-.32f,.65f,-.51f,.26f,.69f,-.71f,-.69f,.28f,-.66f,-.91f,-.15f,.47f,.34f,.56f,-.66f,.26f,.56f,.21f,-.67f,.85f,.30f,.83f,.17f,-.30f,.09f,.23f,.64f,.26f,.78f,-.09f,-.37f,-.45f,-.74f,-.96f,-.13f,.16f,-.94f,-.78f,.75f,.40f,.83f,.99f,-.91f,-.26f,.66f,-.55f,.89f,.86f,-.64f,-.06f,.33f,-.09f,-.87f,.63f,-.40f,.91f,-.89f,.76f,-.39f,-.93f,.77f,.61f,-.10f,-.78f,-.79f,-.93f,.44f,.58f,.16f,-.39f,-.97f,-.90f,.01f,-.36f,.70f,.26f,.90f,-.65f,.94f,-.19f,-.00f,.26f,-.39f,.25f,-.27f,.68f,.06f,.11f,.88f,.73f,.17f,.83f,-.39f,-.16f,-.43f,-.16f,.59f,.68f,.45f,.15f,.53f,-.56f,.99f,-.46f,-.50f,-.47f,-.63f,-.07f,.32f,.09f,.40f,-.42f,-.66f,.89f,-.73f,-.62f,.43f,.67f,-.31f,.65f,-.04f,-.37f,.77f,.10f,-.78f,.67f,.29f,-.37f,-.94f,-.00f,-.79f,-.15f,.08f,-.53f,.88f,.21f,.73f,.17f,-.13f,.93f,.86f,-.65f,-.12f,-.69f,.57f,.23f,-.62f,.03f,-.47f,.02f,.20f,-.42f,.97f,.85f,-.16f,-.87f,-.78f,.50f,.34f,-.81f,.87f,.29f,.61f,-.82f,-.50f,-.86f,.56f,-.14f,-.50f,-.35f,-.55f,-.37f,.37f,-.55f,.59f,-.27f,.47f,-.88f,.78f,.09f,-.61f,-.20f,-.81f,.10f,.11f,-.93f,-.80f,.74f,.65f,.27f,-.97f,.97f,-.04f,.49f,-.77f,.36f,.82f,.09f,-.32f,.08f,.67f,.08f,.48f,-.02f,-.07f,.55f,-.47f,.03f,-.17f,-.72f,-.05f,.79f,.60f,-.86f,.68f,.07f,.77f,-.60f,.83f,.47f,.65f,-.95f,-.00f,.71f,.97f,.63f,.75f,.83f,-.08f,-.89f,-.46f,-.46f,-.51f,.69f,.83f,.33f,.25f,.36f,-.47f,-.57f,.36f,-.15f,.03f,.72f,-.91f,-.46f,-.45f,-.08f,.21f,.39f,-.59f,.73f,.34f,-.42f,.35f,-.11f,-.38f,-.51f,.84f,-.07f,-.21f,-.58f,-.96f,-.91f,.00f,-.85f,-.29f,-.73f,-.45f,-.52f,-.13f,-.44f,-.79f,-.90f,.32f,.67f,.66f,.51f,-.57f,-.31f,.85f,-.46f,.91f,-.03f,-.13f,.81f,-.84f,.07f,.37f,-.51f,.73f,-.11f,.59f,-.38f,.64f,-.41f,-.80f,.72f,-.72f,-.70f,.50f,-.19f,-.42f,-.10f,.59f,.79f,-.69f,-.58f,.15f,.98f,.44f,.19f,-.18f,-.06f,.80f,-.27f,-.02f,-.02f,.25f,-.20f,.29f,-.80f,.39f,.89f,.99f,.47f,.43f,.71f,-.43f,-.26f,.62f,.10f,.79f,-.26f,.18f,.27f,-.75f,-.82f,-.09f,-.63f,.66f,-.29f,-.54f,-.21f,-.83f,-.42f,.85f,.69f,-.95f,-.85f,.04f,-.60f,.02f,-.00f,.19f,.01f,.89f,-.69f,.85f,-.70f,.46f,-.47f,-.43f,-.03f,.37f,-.66f,-.44f,.90f,.03f,-.07f,-.09f,.17f,-.42f,.16f,-.81f,-.05f,.56f,-.19f,-.23f,-.89f,-.20f,-.95f,-.47f,-.84f,.84f,-.91f,-.33f,-.69f,.53f,-.99f,.93f,.03f,-.88f,.03f,-.01f,.88f,.84f,-.71f,-.25f,-.04f,-.15f,-.92f,-.12f,-.74f,-.04f,.08f,-.61f,.18f,.93f,-.30f,-.81f,.94f,-.39f,-.41f,.21f,-.08f,-.97f,-.41f,.10f,-.44f,.14f,.30f,-.75f,.68f,-.09f,.43f,.49f,-.58f,.94f,-.01f,.26f,.75f,.77f,-.26f,.56f,.48f,-.50f,-.45f,.56f,-.13f,.53f,-.75f,-.18f,.05f,-.00f,.66f,.23f,-.50f,.57f,.37f,-.23f,.86f,-.05f,-.61f,-.22f,.72f,-.35f,.47f,-.50f,.78f,.81f,.05f,-.04f,-.05f,.41f,-.74f,.52f,.34f,-.02f,.30f,.91f,-.52f,-.66f,-.57f,-.49f,-.76f,-.29f,-.92f,.87f,-.26f,-.88f,-.78f,-.98f,-.61f,-.72f,.09f,.82f,.47f,-.19f,-.71f,.56f,.15f,-.07f,-.94f,.85f,.85f,.40f,-.13f,-.20f,.23f,-.02f,-.74f,.12f,.99f,-.44f,.34f,-.54f,.90f,-.95f,.54f,-.27f,.36f,.78f,-.86f,-.21f,.17f,.28f,-.15f,.66f,-.43f,.64f,.70f,-.95f,.27f,.71f,.27f,-.99f,-.11f,-.56f,.21f,-.84f,-.64f,-.26f,.05f,.31f,.56f,-.41f,-.77f,-.32f,.72f,.25f,.81f,.06f,-.92f,.89f,-.23f,.42f,.53f,-.15f,.11f,.88f,-.36f,-.43f,-.68f,.37f,-.62f,-.77f,-.03f,-.93f,.50f,.57f,-.90f,-.00f,.82f,.61f,-.62f,.38f,-.06f,-.32f,.31f,.66f,-.25f,.90f,.15f,.12f,.60f,.82f,-.86f,.54f,-.99f,.05f,.92f,-.75f,-.26f,-.32f,-.18f,.41f,.38f,-.32f,.27f,.08f,.74f,-.71f,-.26f,.34f,.99f,.46f,.11f,-.41f,-.09f,.19f,-.83f,.43f,.12f,.74f,.31f,-.35f,-.76f,.13f,.05f,.65f,.32f,-.62f,-.34f,-.37f,.94f,-.18f,.03f,-.30f,-.91f,.95f,.76f,.67f,.85f,.15f,.84f,-.21f,.87f,.83f,.56f,-.67f,.72f,.05f,-.70f,-.28f,-.87f,-.25f,.95f,.34f,-.01f,.20f,-.72f,.57f,.06f,.99f,.05f,.36f,.77f,-.89f,-.52f,-.65f,-.95f,-.49f,-.62f,.12f,.17f,-.89f,.27f,-.72f,.09f,.94f,.46f,-.39f,-.27f,-.34f,.05f,-.43f,.76f,.59f,.41f,-.69f,.50f,.30f,.69f,.31f,.31f,.60f,-.81f,-.34f,-.31f,.32f,.72f,-.37f,.08f,-.61f,-.77f,-.86f,.13f,.52f,-.28f,-.43f,-.70f,.34f,.41f,-.85f,-.19f,.09f,.88f,-.40f,-.92f,-.62f,-.97f,.11f,-.54f,-.51f,-.99f,-.03f,.69f,.57f,-.56f,-.87f,.39f,.37f,.72f,.17f,-.28f,-.86f,.46f,-.77f,.49f,-.73f,-.65f,-.81f,-.78f,.02f,.22f,-.58f,.30f,-.95f,-.91f,.94f,-.73f,.67f,.45f,-.49f,.37f,.48f,-.76f,-.59f,.32f,.97f,.01f,.46f,-.42f,.58f,.26f,.62f,.58f,-.26f,-.52f,-.24f,-.72f,-.88f,.89f,.99f,.20f,.48f,.11f,.82f,-.51f,.46f,-.07f,.19f,.66f,-.22f,-.68f,.37f,.81f,.51f,-.81f,-.75f,-.51f,-.26f,-.13f,.47f,.59f,-.71f,-.43f,-.33f,.82f,-.15f,-.18f,.80f,-.09f,.19f,.75f,.20f,.31f,.28f,-.44f,-.43f,.18f,.53f,.76f,.15f,-.63f,-.39f,-.84f,.01f,-.05f,-.33f,.32f,.10f,.00f,.80f,.46f,-.78f,.84f,-.20f,.60f,.62f,.52f,-.79f,.21f,-.97f,-.94f,.45f,-.46f,-.97f,-.53f,-.37f,-.97f,-.68f,.09f,-.80f,.50f,.09f,.56f,.10f,.81f,.33f,.81f,-.32f,-.51f,-.00f,-.15f,.92f,.94f,-.79f,-.72f,.43f,-.04f,.42f,.21f,.99f,.39f,.06f,-.01f,-.59f,.18f,-.94f,-.50f,-.55f,-.13f,.32f,.94f,-.87f,.08f,-.72f,-.17f,-.37f,.21f,.01f,-.28f,.60f,.05f,-.99f,-.50f,-.74f,.32f,.49f,-.29f,.12f,-.52f,-.90f,-.68f,-.45f,.73f,.52f,.03f,-.71f,.39f,.92f,-.95f,.83f,-.60f,-.44f,-.66f,-.59f,-.76f,-.79f,-.10f,-.77f,.35f,.88f,-.49f,.08f,.06f,-.97f,-.84f,-.47f,.40f,.51f,.13f,-.06f,-.35f,.81f,-.99f,-.09f,-.92f,-.38f,-.78f,-.10f,.76f,-.08f,-.31f,-.59f,-.86f,-.34f,-.86f,.54f,.10f,.16f,-.01f,-.68f,-.22f,-.15f,.46f,.92f,-.75f,-.84f,-.62f,-.74f,.40f,.90f,-.41f,-.14f,-.73f,.27f,.86f,.53f,-.41f,.35f,.96f,.34f,.96f,-.83f,.88f,-.35f,.34f,-.71f,-.27f,-.28f,.38f,-.98f,-.30f,-.67f,.52f,-.03f,.63f,-.54f,.26f,.41f,-.01f,-.10f,-.38f,.61f,-.77f,.92f,.58f,-.98f,.93f,-.27f,-.08f,.78f,-.64f,-.93f,.79f,-.36f,.94f,-.10f,.48f,.73f,-.44f,.00f,-.94f,-.90f,.04f,-.72f,.09f,-.57f,.46f,-.62f,-.41f,.09f,-.21f,-.56f,.79f,.49f,-.78f,.43f,.61f,-.29f,-.10f,-.54f,.64f,-.95f,.35f,-.50f,.99f,.08f,-.07f,-.00f,.31f,.69f,.78f,.68f,-.34f,-.39f,.17f,.51f,-.25f,.71f,.83f,.58f,.87f,.79f,.14f,.26f,.54f,.26f,.06f,.85f,-.29f,-.79f,-.57f,-.02f,-.07f,.77f,.16f,.17f,-.35f,.63f,.61f,.89f,.52f,.14f,.77f,.57f,.80f,-.81f,-.93f,-.18f,-.32f,-.88f,.24f,-.25f,-.75f,.88f,.43f,.09f,-.08f,-.15f,.53f,-.93f,.02f,-.48f,.82f,-.81f,.86f,-.84f,.15f,.64f,-.20f,.92f,.10f,.26f,.75f,-.50f,.50f,-.75f,.48f,-.27f,-.39f,-.82f,-.71f,.60f,.99f,-.84f,.42f,-.01f,.39f,-.86f,-.06f,.66f,.70f,.17f,.75f,-.97f,.57f,-.44f,-.68f,.13f,-.20f,-.69f,.99f,.73f,.38f,-.51f,-.23f,.26f,-.85f,.57f,-.62f,.72f,.69f,.88f,-.09f,-.30f,-.30f,.37f,-.62f,-.74f,-.02f,-.33f,-.69f,.00f,-.53f,-.99f,-.39f,-.88f,.56f,.09f,.33f,.56f,.75f,.01f,-.68f,.87f,.80f,-.23f,-.48f,-.20f,.08f,.64f,.10f,-.14f,.13f,.50f,-.92f,.43f,-.61f,-.04f,-.90f,.65f,.76f,.58f,-.21f,.63f,.93f,-.61f,.01f,-.13f,.09f,.89f,.46f,.82f,.95f,.23f,-.86f,-.16f,.38f,.00f,.70f,-.42f,.56f,-.55f,.05f,.88f,-.22f,-.51f,-.84f,.28f,-.03f,.74f,.53f,.40f,-.98f,-.52f,-.50f,-.67f,.84f,-.72f,-.78f,.14f,.08f,-.44f,-.23f,.38f,-.01f,.76f,.26f,.97f,.80f,-.84f,-.20f,-.19f,.51f,.42f,-.51f,-.81f,.69f,.10f,-.98f,-.62f,-.82f,.09f,-.43f,.54f,.47f,.43f,-.85f,.87f,-.86f,.24f,.23f,-.46f,-.33f,.86f,-.90f,.26f,-.42f,-.48f,-.12f,.55f,-.28f,-.49f,-.40f,.45f,.72f,.69f,.10f,-.20f,.02f,.65f,-.55f,.70f,.27f,-.11f,.44f,.87f,-.83f,.09f,.60f,.11f,.05f,-.33f,-.35f,.74f,.23f,-.91f,.35f,-.47f,.52f,-.70f,.56f,.24f,-.60f,-.60f,.42f,-.40f,-.30f,-.87f,.61f,-.53f,.61f,.85f,.00f,-.19f,-.10f,-.72f,-.07f,-.62f,.46f,-.16f,-.75f,-.96f,-.92f,.87f,-.95f,-.25f,-.40f,.63f,.45f,-.22f,-.49f,.27f,.05f,-.96f,-.89f,.43f,.38f,.50f,.09f,.78f,-.73f,.67f,.55f,-.53f,.89f,-.34f,.99f,-.29f,-.04f,-.42f,-.34f,-.74f,-.36f,.38f,.94f,.29f,-.11f,.10f,-.08f,-.21f,-.18f,.88f,-.64f,-.58f,-.18f,.77f,-.47f,-.53f,-.00f,.13f,-.41f,.71f,.52f,-.39f,-.25f,.15f,.09f,.49f,.99f,-.09f,.97f,.21f,-.24f,.76f,-.25f,.81f,-.44f,.96f,-.85f,.48f,-.86f,.12f,.94f,-.09f,-.69f,.98f,.26f,.63f,.65f,.70f,.95f,.96f,.49f,-.25f,.66f,-.97f,-.62f,.85f,.04f,.16f,.22f,.71f,-.52f,-.55f,.08f,-.85f,-.87f,.73f,.08f,.94f,-.34f,-.82f,.22f,-.25f,-.92f,-.96f,.94f,-.47f,.76f,-.83f,.22f,.42f,-.80f,-.22f,-.84f,-.52f,.15f,.20f,.96f,.20f,.63f,-.32f,-.76f,.32f,-.71f,-.68f,.55f,-.45f,.59f,-.41f,-.10f,-.62f,.56f,.07f,-.22f,-.90f,.39f,-.53f,.98f,-.55f,-.31f,-.48f,.22f,-.78f,-.57f,-.88f,-.97f,.26f,-.74f,.20f,-.82f,-.34f,.27f,.96f,-.14f,-.60f,-.20f,-.34f,.68f,-.00f,.57f,.87f,-.48f,.10f,.39f,-.22f,.97f,.45f,-.41f,.85f,-.18f,-.22f,-.40f,.67f,-.09f,-.61f,-.28f,.71f,.45f,-.27f,-.67f,.50f,-.10f,-.09f,-.75f,.96f,.86f,.27f,-.83f,-.35f,-.30f,.37f,.44f,.78f,-.50f,-.53f,-.40f,-.43f,-.04f,.46f,-.57f,-.60f,-.13f,.90f,.32f,.92f,.79f,.48f,-.76f,.87f,.11f,-.17f,-.50f,-.00f,.02f,.61f,.33f,.64f,-.80f,-.48f,.91f,.81f,-.73f,-.02f,.94f,-.63f,.19f,.39f,-.50f,-.91f,-.99f,.95f,-.75f,.67f,.82f,-.54f,-.76f,.50f,.90f,.46f,-.74f,.69f,-.86f,.72f,.52f,.39f,-.46f,-.36f,.34f,.21f,-.44f,.14f,.35f,.13f,-.84f,.95f,.32f,-.82f,-.62f,-.69f,-.50f,.59f,.47f,-.97f,.15f,.59f,.65f,-.26f,.50f,-.03f,-.96f,.97f,-.99f,-.67f,-.73f,-.48f,-.64f,-.84f,-.94f,.62f,.14f,-.32f,-.47f,-.74f,.07f,.05f,.87f,-.83f,.49f,-.32f,-.32f,.03f,.51f,.06f,-.77f,.75f,-.39f,-.78f,.10f,.95f,.38f,-.08f,-.66f,-.07f,.09f,-.09f,-.09f,.83f,-.12f,.43f,-.65f,.41f,.68f,.93f,.88f,-.24f,.70f,-.97f,-.40f,-.96f,-.45f,-.35f,-.29f,.91f,-.28f,.92f,-.57f,-.37f,-.02f,.37f,-.37f,-.01f,-.48f,.74f,-.28f,.21f,.74f,-.99f,.45f,.91f,-.28f,-.11f,-.86f,-.22f,.48f,-.34f,.52f,.17f,.04f,.23f,-.54f,-.80f,-.60f,.04f,.47f,-.14f,.33f,.57f,.22f,-.67f,.02f,.69f,.93f,.69f,.93f,.99f,.75f,.47f,.21f,-.42f,.02f,.20f,.03f,.12f,.76f,.04f,-.43f,-.10f,-.52f,-.26f,-.10f,.78f,-.81f,-.72f,-.76f,.87f,.88f,.83f,-.27f,.82f,-.97f,-.59f,.80f,.66f,.66f,.59f,-.73f,.03f,-.16f,.69f,-.52f,.22f,.82f,-.11f,-.25f,-.39f,.56f,.93f,-.43f,-.73f,-.28f,-.06f,-.49f,-.95f,.39f,.51f,.29f,.40f,-.46f,-.11f,.60f,.03f,-.57f,.24f,-.55f,.88f,-.33f,.64f,-.79f,.59f,-.05f,.94f,.00f,.74f,.44f,-.86f,.23f,-.03f,-.66f,.45f,.08f,-.53f,.09f,.98f,-.28f,-.10f,.89f,.18f,.45f,-.98f,-.89f,.47f,.78f,-.11f,.53f,-.35f,.46f,.00f,.49f,-.13f,-.58f,-.56f,.73f,.21f,.20f,.72f,-.71f,-.68f,-.54f,.61f,-.80f,.53f,.90f,.20f,.25f,-.29f,.91f,-.88f,.04f,.98f,-.18f,-.16f,.28f,.28f,.60f,.35f,-.32f,.48f,-.99f,-.57f,.41f,.93f,-.33f,-.69f,.68f,-.99f,.40f,-.31f,.58f,.70f,-.95f,-.97f,.59f,-.71f,-.43f,.78f,-.92f,-.01f,.79f,-.34f,-.14f,.25f,.86f,.62f,-.12f,-.53f,.77f,-.30f,-.50f,-.82f,.76f,.27f,-.59f,.69f,-.87f,-.03f,.82f,.05f,.58f,.96f,.52f,.04f,.80f,-.66f,-.06f,.88f,-.59f,-.19f,-.47f,.84f,.15f,.90f,-.08f,.69f,-.34f,.12f,.14f,.86f,-.56f,-.59f,.08f,.27f,-.13f,.00f,-.00f,-.47f,.34f,.44f,.17f,-.57f,-.19f,-.09f,-.31f,.03f,-.60f,.40f,.58f,-.43f,-.81f,.08f,.04f,-.28f,.74f,.00f,-.59f,-.57f,-.82f,.99f,-.36f,.44f,-.51f,.26f,.75f,-.56f,-.86f,.85f,-.63f,.49f,.81f,-.53f,.23f,-.35f,-.25f,-.44f,-.03f,.23f,.98f,-.37f,-.25f,-.60f,.81f,.16f,.95f,.77f,-.18f,.45f,-.35f,.47f,.74f,.65f,.81f,-.02f,-.39f,-.75f,.33f,.35f,-.76f,-.12f,-.62f,.06f,.47f,.47f,-.74f,.58f,-.68f,-.77f,-.12f,-.41f,.26f,.39f,-.37f,.47f,.81f,.00f,.80f,-.59f,.74f,-.11f,-.81f,-.10f,-.03f,.88f,-.65f,.58f,-.93f,-.09f,-.30f,.72f,-.75f,-.10f,.01f,.21f,-.17f,.37f,-.96f,-.44f,-.69f,.36f,.89f,.24f,.06f,.20f,-.51f,.20f,-.25f,.64f,.06f,-.91f,.19f,-.71f,-.09f,.85f,.49f,-.39f,.80f,.42f,.61f,-.52f,.61f,-.13f,.48f,.67f,.60f,.93f,-.02f,.58f,.01f,-.17f,-.50f,-.67f,-.74f,-.12f,.92f,-.61f,-.21f,.30f,-.70f,-.56f,-.91f,-.65f,.13f,-.36f,-.91f,.93f,.69f,-.95f,.67f,-.46f,.26f,.59f,.11f,.64f,-.83f,-.00f,-.85f,.50f,.25f,.80f,-.83f,-.54f,-.96f,.68f,-.39f,.84f,.80f,.84f,.86f,-.60f,-.73f,-.90f,.22f,.52f,-.70f,-.70f,.35f,.57f,.55f,-.94f,.51f,.07f,-.57f,-.20f,.16f,.66f,.71f,.46f,-.03f,.11f,-.61f,-.84f,-.57f,.12f,-.37f,-.97f,-.83f,.47f,.40f,.54f,.79f,.95f,.40f,.23f,.45f,.66f,.33f,.61f,.90f,-.66f,-.18f,.95f,.91f,-.89f,-.66f,.37f,-.80f,-.36f,.02f,.23f,-.57f,-.19f,-.11f,-.01f,-.47f,-.67f,-.64f,.80f,-.34f,-.08f,-.54f,.96f,-.41f,-.43f,.52f,.30f,-.02f,.68f,-.82f,-.06f,-.24f,.53f,-.12f,-.64f,-.57f,-.93f,.17f,-.62f,-.57f,-.18f,.44f,-.93f,.27f,.47f,.02f,-.66f,.84f,-.56f,-.86f,-.30f,.45f,-.87f,-.18f,-.78f,-.58f,-.87f,-.01f,-.39f,-.82f,-.88f,.73f,-.96f,-.08f,-.83f,-.78f,-.40f,.55f,-.96f,-.72f,-.48f,-.58f,.56f,.17f,.51f,.81f,-.72f,-.46f,.98f,.59f,-.78f,-.00f,-.97f,.94f,.53f,.29f,.90f,.85f,-.83f,-.66f,-.22f,-.23f,.41f,-.13f,-.10f,.66f,-.18f,.83f,-.50f,-.14f,-.22f,.75f,-.81f,.56f,-.60f,-.11f,-.24f,-.37f,-.44f,.27f,-.84f,.21f,-.47f,.87f,.91f,-.95f,-.09f,.97f,.08f,-.49f,-.27f,.22f,.50f,-.54f,.41f,.84f,.25f,.24f,.12f,.76f,-.50f,.78f,-.35f,.66f,-.78f,.10f,.81f,.32f,-.41f,-.20f,-.99f,.80f,-.91f,-.80f,.53f,.83f,-.43f,-.12f,.26f,-.28f,.33f,-.88f,.39f,.15f,-.11f,-.35f,.46f,-.47f,-.30f,.42f,-.33f,.95f,.59f,-.43f,-.12f,.57f,-.57f,-.47f,.60f,.34f,.80f,.80f,-.64f,-.72f,.01f,.49f,-.03f,.09f,.38f,-.65f,-.22f,.61f,-.82f,.42f,-.33f,.91f,.55f,.45f,.85f,-.06f,-.58f,.64f,.12f,-.57f,.68f,.24f,-.88f,-.67f,-.65f,-.22f,-.11f,-.83f,-.82f,.76f,.33f,-.14f,.97f,-.18f,-.04f,.97f,.00f,.08f,.15f,.52f,-.19f,-.46f,-.50f,-.59f,.69f,.06f,-.14f,.27f,-.75f,.27f,-.14f,.15f,-.57f,-.42f,-.18f,.43f,.44f,-.65f,-.87f,-.49f,-.37f,.82f,-.72f,.01f,-.88f,.14f,-.92f,-.50f,.83f,-.14f,-.16f,-.50f,-.05f,-.63f,-.90f,.83f,.74f,.00f,.44f,-.66f,.93f,.27f,-.28f,.18f,-.80f,-.54f,-.08f,.22f,.43f,.16f,-.25f,-.14f,.25f,.55f,-.85f,.72f,.33f,.12f,-.65f,-.17f,.26f,-.40f,.78f,-.50f,.07f,-.98f,-.60f,-.38f,.95f,.75f,-.10f,-.30f,.57f,-.67f,.12f,.46f,-.23f,-.93f,-.34f,-.63f,.42f,.20f,.25f,-.83f,.09f,-.35f,-.70f,.73f,-.71f,-.65f,.53f,.93f,-.30f,.42f,.10f,.73f,-.96f,-.08f,.64f,.66f,-.20f,.62f,-.90f,-.63f,-.55f,.72f,.64f,-.42f,-.01f,-.62f,-.82f,.68f,-.86f,.48f,-.04f,-.02f,-.91f,-.99f,-.33f,-.46f,-.06f,-.92f,-.23f,.15f,.26f,.37f,.23f,.08f,.47f,-.78f,.86f,.52f,-.27f,-.01f,-.54f,-.42f,-.25f,-.20f,-.13f,.64f,.32f,.45f,.34f,-.13f,.99f,.45f,.81f,-.63f,-.27f,-.59f,-.29f,-.44f,.89f,-.01f,-.17f,.79f,.08f,.59f,-.01f,-.26f,.10f,.27f,.28f,-.12f,-.73f,.19f,.16f,-.12f,.49f,.43f,.74f,.56f,.90f,.28f,-.46f,-.62f,.47f,-.35f,-.36f,-.99f,.89f,-.64f,-.33f,-.43f,.97f,-.34f,.37f,-.16f,-.39f,.18f,-.95f,-.41f,.41f,-.21f,-.82f,-.93f,-.90f,.53f,.25f,.29f,-.84f,-.39f,-.72f,-.21f,-.78f,-.97f,-.92f,-.34f,.81f,-.08f,.34f,.26f,-.74f,-.26f,-.20f,.71f,.67f,-.08f,.35f,-.63f,-.11f,-.50f,-.71f,-.26f,-.46f,.01f,-.24f,.73f,-.22f,-.85f,-.68f,-.21f,-.56f,.16f,-.04f,-.77f,.24f,-.66f,.77f,.25f,-.27f,-.24f,-.36f,.89f,-.65f,.62f,.00f,-.74f,.22f,.82f,.72f,.58f,-.44f,.72f,-.55f,-.20f,.11f,-.62f,.48f,-.24f,-.02f,-.81f,.69f,-.58f,.26f,-.64f,-.48f,.79f,.38f,.68f,.70f,-.19f,.79f,.89f,-.57f,-.24f,-.57f,-.20f,-.80f,.54f,-.12f,-.94f,-.04f,.40f,-.16f,-.99f,.12f,.73f,-.02f,.63f,-.76f,-.40f,-.82f,.60f,.48f,.26f,-.77f,-.68f,.10f,-.81f,.90f,.97f,.10f,-.12f,-.07f,-.64f,.36f,-.37f,.25f,-.38f,-.65f,-.30f,-.71f,.35f,.69f,.29f,-.60f,.66f,.53f,.69f,-.27f,.04f,-.38f,-.47f,-.00f,-.30f,.71f,.44f,-.50f,.52f,.75f,.18f,-.47f,-.15f,.46f,-.42f,.32f,-.97f,.16f,.67f,-.13f,-.59f,.49f,-.02f,-.91f,-.05f,-.89f,.68f,-.21f,.57f,-.17f,.37f,-.36f,-.44f,.40f,.31f,-.51f,-.56f,.09f,.80f,-.61f,-.36f,-.66f,.29f,-.00f,-.72f,.14f,-.95f,-.18f,-.43f,-.25f,.28f,-.64f,-.37f,-.29f,.15f,.48f,.29f,-.86f,.43f,.85f,.56f,-.66f,-.93f,-.00f,.06f,.00f,-.73f,-.29f,-.29f,.59f,-.60f,-.24f,-.88f,.99f,.39f,.71f,-.58f,-.46f,-.86f,-.65f,-.17f,.28f,.85f,.74f,.32f,-.14f,.59f,.37f,-.94f,.28f,-.50f,-.88f,.36f,-.21f,-.22f,.55f,-.49f,-.22f,.33f,.34f,.59f,-.94f,.82f,-.06f,.82f,.71f,.03f,-.11f,-.02f,.85f,-.25f,.86f,-.30f,-.99f,.27f,.17f,.78f,.24f,-.85f,.97f,.35f,.44f,-.86f,.22f,.69f,.89f,-.11f,.94f,-.97f,-.73f,-.46f,-.85f,.33f,-.01f,-.37f,.23f,-.67f,.02f,-.89f,.99f,-.46f,.40f,-.60f,-.35f,-.13f,-.38f,-.18f,-.50f,-.52f,.06f,.01f,.68f,.69f,.17f,-.36f,.68f,-.01f,.22f,.96f,.58f,-.28f,.10f,-.49f,-.21f,-.74f,.06f,-.03f,.36f,.47f,-.94f,-.13f,.66f,-.03f,.81f,-.10f,-.34f,-.74f,-.02f,.36f,.13f,-.09f,-.23f,-.15f,-.84f,.61f,.63f,-.06f,-.59f,.49f,-.83f,.43f,-.51f,-.26f,-.49f,.79f,.92f,.69f,-.51f,.43f,.57f,.59f,.62f,.04f,-.66f,.12f,-.61f,.15f,-.29f,-.66f,-.05f,.27f,.83f,.62f,-.89f,.37f,.08f,-.41f,.60f,.57f,.27f,-.08f,.62f,-.75f,.71f,-.56f,-.08f,.98f,.63f,-.46f,-.76f,.75f,-.99f,-.80f,-.58f,-.16f,-.55f,.09f,.42f,-.11f,.18f,-.87f,-.89f,.04f,.24f,-.86f,-.20f,.09f,-.76f,.77f,-.39f,.60f,-.61f,-.04f,.16f,.57f,-.39f,-.75f,-.99f,-.77f,.91f,.58f,-.29f,.48f,.22f,.73f,-.29f,-.28f,.93f,-.78f,-.19f,.27f,-.76f,-.93f,-.95f,-.98f,.74f,.50f,.09f,-.76f,-.31f,-.95f,.18f,.95f,-.90f,-.87f,.87f,.39f,.91f,.91f,.43f,.21f,-.33f,-.82f,-.85f,.79f,.25f,.62f,.44f,-.19f,.80f,.77f,-.08f,.74f,-.25f,.90f,.93f,.55f,-.46f,-.63f,.73f,.51f,.31f,-.30f,-.46f,-.26f,.82f,-.44f,-.96f,-.05f,.25f,.67f,.37f,-.96f,-.70f,-.50f,-.33f,-.30f,.19f,.45f,.96f,.18f,-.44f,.10f,-.29f,-.83f,.18f,.14f,.91f,-.76f,.33f,-.75f,-.06f,.35f,.35f,-.22f,-.42f,-.34f,.60f,-.56f,.95f,-.35f,-.13f,-.58f,-.84f,.89f,-.67f,-.84f,.78f,-.29f,.83f,-.03f,.74f,-.98f,.54f,-.33f,.40f,.79f,.78f,.47f,-.60f,.66f,.69f,-.25f,-.38f,.90f,.91f,-.22f,.55f,-.30f,-.48f,-.16f,-.09f,-.23f,-.68f,-.62f,.49f,.34f,.18f,.88f,.08f,.95f,-.59f,.40f,.52f,.19f,.94f,.36f,.96f,-.31f,.71f,-.19f,.67f,.87f,-.39f,-.55f,-.01f,-.14f,.28f,-.75f,.70f,-.17f,.18f,.64f,.50f,-.83f,.85f,.85f,.38f,.04f,-.46f,.41f,-.21f,-.78f,.32f,.46f,.95f,-.16f,-.31f,-.24f,.06f,-.25f,-.94f,.99f,.75f,-.00f,-.42f,-.72f,.61f,-.06f,.78f,.92f,.25f,.90f,-.37f,.72f,.21f,-.72f,.36f,.76f,-.53f,-.36f,.45f,-.28f,-.47f,-.46f,.47f,-.30f,.30f,-.72f,.73f,-.61f,-.39f,.06f,.35f,.77f,-.38f,.52f,-.03f,-.85f,-.20f,.02f,.64f,-.72f,.10f,.11f,.81f,-.07f,.72f,.87f,.81f,-.21f,-.03f,-.86f,-.18f,.01f,-.77f,-.60f,.16f,.32f,-.46f,.00f,-.00f,-.77f,-.04f,.92f,.48f,-.84f,-.65f,-.93f,-.77f,-.11f,.77f,.86f,.96f,.31f,-.18f,-.94f,.66f,-.97f,.51f,.39f,-.66f,-.74f,.82f,.91f,.90f,-.22f,-.89f,.72f,.98f,-.14f,.73f,.68f,.68f,-.20f,.44f,-.76f,.40f,-.93f,-.43f,-.89f,-.74f,-.18f,-.79f,-.69f,.32f,.73f,-.85f,-.06f,.72f,-.25f,.37f,.76f,-.20f,-.70f,.27f,-.94f,.75f,-.29f,-.25f,-.82f,.38f,.50f,.56f,.62f,-.41f,-.10f,-.38f,.76f,-.42f,.89f,.16f,-.83f,-.05f,-.08f,-.41f,.55f,.01f,.93f,-.79f,-.46f,.63f,-.93f,-.26f,.51f,.57f,-.76f,-.94f,.21f,-.83f,.51f,-.38f,-.00f,-.35f,-.94f,.32f,-.70f,-.05f,-.73f,.19f,-.52f,.32f,-.48f,.99f,.18f,.36f,-.73f,.48f,.74f,.53f,.41f,.85f,-.42f,.46f,.18f,.66f,.99f,-.55f,-.02f,.54f,-.89f,-.78f,.68f,.34f,.90f,-.34f,.55f,-.50f,-.80f,-.26f,-.91f,-.72f,.75f,.23f,.87f,.15f,.77f,-.64f,-.78f,.03f,-.29f,-.31f,.02f,-.16f,.01f,.50f,-.35f,-.65f,.22f,.53f,-.25f,-.31f,.61f,.59f,-.83f,-.30f,-.53f,-.76f,-.28f,.71f,.52f,-.77f,.86f,-.01f,.92f,.90f,.20f,.20f,.49f,.38f,-.95f,.59f,-.25f,.42f,-.33f,.29f,.17f,-.17f,.78f,-.75f,.99f,-.43f,-.33f,-.61f,.41f,-.22f,-.25f,-.16f,-.74f,-.06f,.24f,.60f,-.96f,.06f,-.13f,-.27f,-.04f,-.04f,.45f,.25f,.83f,-.31f,.27f,-.06f,-.40f,.68f,-.52f,.92f,-.71f,-.78f,.93f,-.84f,-.51f,-.30f,-.15f,.02f,-.59f,-.64f,.06f,-.18f,-.55f,-.96f,.12f,-.89f,.97f,.91f,-.73f,.06f,.57f,-.82f,.59f,-.86f,.08f,.45f,-.95f,.67f,-.65f,-.29f,-.36f,.04f,-.82f,.10f,.55f,.32f,.55f,.32f,.81f,-.92f,.09f,-.35f,.88f,.31f,.06f,.89f,.00f,.37f,-.45f,.58f,.97f,.95f,.34f,-.41f,-.84f,.45f,-.81f,.73f,-.11f,.06f,-.68f,-.92f,.59f,.80f,.60f,-.04f,-.18f,.44f,-.83f,.71f,.68f,.02f,.62f,.83f,.60f,.81f,-.92f,.86f,-.57f,.26f,.30f,-.26f,.29f,-.74f,-.21f,.65f,.40f,.43f,-.95f,.36f,-.28f,.12f,-.89f,-.23f,.86f,-.11f,.70f,.52f,-.22f,-.83f,-.63f,.08f,.58f,-.88f,-.34f,-.06f,.44f,.10f,.36f,.99f,.04f,.41f,-.73f,.41f,.57f,-.41f,.35f,.36f,-.37f,.64f,-.70f,-.33f,-.57f,-.38f,.09f,.50f,-.09f,-.95f,.59f,.07f,.25f,.75f,-.29f,-.46f,-.69f,-.93f,-.49f,-.03f,-.25f,.65f,-.15f,.79f,.76f,-.73f,.57f,-.90f,.74f,.99f,.20f,.09f,.33f,-.55f,.42f,-.98f,-.26f,-.48f,.96f,.83f,-.75f,-.64f,-.09f,-.73f,.96f,-.26f,-.76f,.83f,.54f,.66f,.54f,.64f,-.58f,.45f,-.67f,.81f,.00f,-.77f,.66f,.36f,.93f,.75f,-.62f,.65f,-.52f,.08f,.02f,.12f,.35f,.85f,.83f,-.07f,-.62f,-.78f,-.98f,-.78f,.43f,.53f,-.55f,.04f,.99f,-.57f,-.52f,.78f,.11f,-.00f,.88f,.10f,-.43f,-.44f,-.51f,.73f,-.74f,.53f,-.26f,-.58f,.07f,-.48f,.23f,.94f,-.47f,-.53f,.04f,.18f,-.16f,-.86f,-.90f,-.04f,.05f,.03f,.53f,-.11f,-.86f,.80f,.34f,.61f,-.00f,-.99f,.55f,-.20f,-.84f,.21f,-.66f,-.43f,-.93f,-.83f,.00f,-.62f,-.59f,-.16f,-.56f,-.11f,.55f,-.30f,.96f,-.67f,.05f,.17f,.30f,-.86f,-.44f,.05f,.75f,-.44f,-.03f,-.61f,-.89f,.40f,.35f,-.42f,.35f,-.97f,.58f,.32f,.84f,-.13f,.01f,.89f,.60f,.06f,.80f,.67f,-.54f,.59f,-.52f,.06f,.86f,.19f,.04f,.75f,.86f,.11f,.00f,-.72f,-.85f,.09f,.78f,-.68f,.90f,.06f,.38f,-.14f,.97f,.07f,-.60f,.60f,-.78f,-.38f,.01f,-.99f,.61f,.53f,-.65f,.16f,.32f,.61f,-.94f,.88f,.75f,-.05f,-.24f,-.84f,-.49f,.89f,.39f,.22f,.20f,-.29f,.74f,-.01f,.08f,-.24f,-.80f,.57f,-.16f,.00f,-.48f,.68f,-.06f,.78f,-.66f,-.91f,.30f,-.72f,-.20f,.28f,-.94f,-.44f,-.29f,.27f,-.84f,.86f,-.47f,-.62f,.29f,-.59f,.71f,-.10f,-.47f,-.40f,-.62f,-.45f,.01f,-.88f,-.47f,-.11f,-.27f,.33f,-.12f,.68f,-.96f,.30f,-.73f,.13f,.70f,.78f,.18f,.35f,-.82f,-.65f,-.72f,.44f,-.03f,.12f,-.68f,-.68f,.48f,.89f,-.32f,.84f,.32f,.14f,-.68f,.44f,.17f,.56f,-.76f,.24f,-.88f,.04f,-.46f,-.34f,.28f,-.31f,-.97f,-.21f,-.78f,-.84f,-.22f,-.73f,.07f,-.56f,.01f,-.75f,-.19f,-.41f,.78f,-.23f,.93f,.91f,.96f,-.41f,-.27f,-.20f,-.59f,.25f,.38f,-.63f,.24f,.62f,.44f,.71f,-.55f,.61f,.50f,-.56f,.77f,.44f,.25f,-.17f,-.32f,.83f,-.33f,.16f,.90f,-.58f,-.63f,.28f,-.65f,.02f,.60f,.67f,.30f,.09f,-.48f,.54f,-.07f,.32f,-.51f,-.83f,.67f,.86f,.48f,-.14f,-.03f,-.49f,.17f,-.59f,-.21f,-.26f,-.93f,.80f,-.35f,.52f,-.40f,-.89f,-.77f,-.53f,.56f,-.58f,.45f,.54f,.81f,.05f,.54f,.11f,.78f,.53f,.88f,.13f,-.95f,-.23f,-.40f,.29f,-.37f,-.57f,.39f,.24f,.14f,.86f,.36f,-.72f,.12f,-.54f,.60f,.22f,.32f,-.02f,.69f,-.43f,-.64f,.92f,.37f,.07f,.16f,.05f,.11f,.19f,.21f,-.60f,-.02f,.43f,.84f,-.97f,-.12f,-.06f,-.11f,-.35f,.77f,-.12f,-.44f,-.02f,-.09f,.34f,.37f,.17f,-.63f,.07f,.74f,.57f,-.97f,.57f,.22f,.47f,-.92f,-.97f,-.01f,.47f,.08f,-.31f,.74f,.79f,-.68f,-.89f,-.10f,-.80f,-.91f,.06f,-.40f,.76f,.27f,.79f,.96f,-.95f,.62f,-.09f,.41f,.03f,-.03f,-.85f,-.51f,.92f,-.67f,.44f,.20f,-.29f,-.01f,.48f,.76f,-.74f,-.95f,-.62f,-.46f,.55f,.61f,-.51f,.26f,.53f,-.99f,.28f,.53f,-.41f,-.92f,.38f,-.53f,-.14f,.87f,.80f,.07f,-.76f,.25f,-.75f,-.83f,-.83f,-.01f,-.08f,.92f,-.33f,.67f,.08f,-.11f,.35f,-.17f,.70f,.23f,-.64f,-.91f,.56f,.82f,.42f,-.48f,.54f,-.04f,.00f,.05f,.00f,-.11f,.02f,-.26f,.46f,.04f,.84f,-.54f,-.33f,-.55f,-.00f,-.90f,.73f,-.19f,-.35f,-.40f,-.09f,-.15f,.92f,-.20f,-.95f,-.85f,.92f,.10f,.61f,.02f,-.01f,.54f,.07f,.85f,.50f,-.84f,.14f,-.38f,.40f,.22f,-.49f,.11f,-.49f,-.23f,-.39f,-.25f,-.31f,.65f,-.52f,-.38f,.46f,-.73f,.91f,-.71f,.77f,.65f,-.05f,.31f,-.60f,.08f,-.26f,.25f,.38f,-.68f,.55f,.83f,-.63f,-.75f,-.55f,.08f,-.42f,.35f,-.33f,-.45f,.00f,.67f,.78f,-.59f,.55f,-.90f,-.47f,.91f,.00f,-.15f,.34f,.75f,-.92f,.15f,.21f,.14f,-.14f,.22f,-.19f,.42f,.31f,.75f,-.66f,-.24f,.37f,.40f,.25f,-.15f,.47f,.03f,.14f,-.32f,.56f,-.58f,.88f,-.67f,.96f,.14f,-.78f,.98f,-.77f,-.52f,-.27f,.94f,.55f,.03f,.90f,-.56f,-.43f,-.58f,-.27f,-.79f,-.87f,-.71f,-.22f,-.48f,-.38f,-.96f,.58f,.24f,-.28f,-.47f,.33f,.10f,-.44f,-.94f,.29f,-.16f,.43f,-.79f,.81f,-.89f,-.63f,-.76f,-.15f,-.71f,-.77f,.13f,.67f,-.15f,.86f,.12f,-.37f,.42f,-.82f,-.64f,.42f,.21f,-.37f,-.88f,.14f,-.80f,.22f,.52f,-.84f,-.39f,.89f,.23f,-.21f,-.62f,.49f,.35f,-.13f,-.40f,.94f,-.10f,-.71f,-.54f,.75f,.28f,-.00f,-.78f,-.12f,.42f,.28f,-.99f,.82f,-.99f,-.97f,.61f,.29f,-.76f,.97f,-.57f,-.50f,.20f,.38f,-.95f,.08f,-.57f,-.98f,.24f,-.41f,.40f,.50f,.59f,.44f,-.76f,-.97f,-.65f,-.07f,-.12f,-.03f,.79f,.87f,-.63f,-.09f,.16f,-.47f,.62f,.55f,.17f,.32f,-.44f,.44f,.61f,-.52f,.51f,-.49f,.87f,-.34f,.63f,-.53f,-.30f,-.50f,-.11f,-.82f,-.63f,-.27f,.31f,-.60f,-.83f,.84f,.90f,-.13f,.61f,-.29f,-.62f,.51f,-.32f,.01f,-.35f,-.22f,-.03f,-.65f,.25f,.23f,-.70f,.09f,-.34f,-.82f,-.60f,.27f,.48f,-.04f,.91f,.40f,-.66f,.85f,-.20f,.85f,.61f,-.96f,-.25f,-.55f,-.75f,.00f,.78f,-.92f,.29f,-.00f,.66f,.03f,.47f,-.44f,-.43f,-.63f,.82f,-.00f,-.88f,-.14f,-.55f,.37f,.00f,.95f,.12f,.95f,.22f,-.98f,.82f,-.44f,.37f,-.09f,-.92f,.52f,-.26f,-.37f,.95f,.74f,.84f,-.97f,.09f,-.33f,-.87f,-.06f,.81f,-.85f,-.68f,-.67f,.65f,-.95f,.60f,-.55f,-.45f,.50f,.54f,-.18f,-.00f,-.53f,.68f,.88f,.36f,-.37f,.58f,-.90f,.69f,-.69f,.10f,.40f,-.88f,-.91f,-.90f,-.18f,.47f,-.60f,.17f,-.13f,.64f,-.24f,-.42f,-.89f,-.47f,.24f,-.09f,.13f,.22f,.35f,.49f,.73f,-.97f,-.58f,.49f,.20f,.21f,-.24f,.16f,-.77f,.02f,-.35f,.06f,.83f,-.04f,-.87f,-.61f,.78f,.25f,-.49f,-.73f,-.13f,.95f,-.59f,.41f,-.97f,-.53f,-.10f,-.45f,-.86f,.57f,-.58f,-.78f,.51f,-.05f,-.57f,.11f,.46f,.66f,-.48f,.51f,-.01f,-.53f,-.76f,.19f,-.01f,.62f,-.76f,.49f,.13f,-.44f,.01f,.98f,-.33f,-.18f,-.00f,.27f,.58f,.94f,-.21f,-.79f,.78f,.15f,-.19f,-.60f,.66f,.53f,-.13f,-.91f,-.96f,.16f,-.19f,-.40f,-.79f,-.18f,.66f,-.17f,.80f,.90f,.91f,.43f,-.83f,.74f,-.88f,-.41f,.07f,-.25f,-.53f,-.75f,-.43f,-.62f,-.35f,.90f,-.05f,.75f,-.76f,.87f,-.51f,.16f,.11f,-.79f,.29f,.36f,.56f,-.85f,-.51f,-.04f,.63f,.75f,.66f,-.65f,-.33f,-.58f,.09f,-.50f,-.70f,-.75f,.45f,-.75f,.21f,-.20f,-.39f,.48f,.97f,-.03f,.24f,-.18f,-.48f,.46f,-.04f,-.71f,-.11f,.89f,.56f,-.52f,.76f,.44f,.19f,.83f,.93f,-.51f,.83f,.73f,.12f,.52f,-.97f,.42f,.91f,-.17f,.13f,.44f,-.03f,.54f,.48f,-.25f,-.92f,-.46f,.39f,.82f,-.23f,.05f,.66f,-.08f,.60f,.14f,-.20f,-.32f,-.10f,-.21f,.12f,.56f,-.73f,.63f,.09f,.97f,.10f,-.88f,.71f,.25f,-.62f,-.30f,-.25f,-.64f,.35f,.91f,.57f,.08f,-.97f,-.41f,-.94f,-.91f,-.11f,.57f,-.68f,-.37f,-.39f,.83f,.31f,-.15f,.81f,-.53f,.63f,-.65f,.92f,-.40f,-.35f,.05f,.84f,.63f,.12f,-.97f,.98f,.21f,.91f,-.86f,-.12f,-.77f,.56f,.66f,-.60f,.67f,-.36f,-.33f,.39f,-.21f,.22f,.82f,-.51f,.86f,.28f,-.07f,.13f,-.91f,-.83f,-.60f,-.88f,.42f,.63f,-.70f,.54f,.70f,.38f,-.71f,-.15f,.87f,.07f,.81f,-.33f,-.87f,-.86f,.85f,.37f,-.17f,.71f,.87f,-.56f,.06f,-.66f,.79f,-.17f,.13f,-.22f,-.75f,.65f,.14f,.25f,.56f,.63f,-.68f,-.11f,.91f,.12f,-.01f,-.25f,-.76f,-.93f,.66f,.55f,.43f,-.35f,.60f,-.01f,-.68f,-.94f,-.34f,-.47f,.87f,.78f,.10f,.61f,-.34f,.19f,-.24f,-.20f,.65f,-.04f,-.46f,.08f,.10f,.54f,.28f,-.33f,-.89f,.88f,-.54f,-.04f,.22f,.53f,.70f,.07f,-.65f,-.86f,-.72f,-.61f,.15f,-.33f,-.46f,.96f,.51f,.86f,.84f,.03f,-.62f,.54f,-.40f,-.73f,.04f,-.40f,.23f,-.40f,.28f,-.66f,.71f,.33f,.00f,.59f,.08f,.81f,-.02f,.35f,-.57f,.92f,-.76f,.55f,-.28f,.53f,-.84f,-.76f,.62f,-.86f,.97f,.33f,-.31f,-.70f,.51f,.21f,-.88f,-.14f,.17f,.93f,.34f,-.38f,-.71f,-.68f,-.99f,-.80f,-.26f,.80f,.39f,.65f,.89f,-.11f,.27f,.59f,.72f,.88f,.41f,.27f,-.54f,.61f,-.24f,.64f,.39f,.06f,-.65f,-.23f,-.43f,-.74f,.23f,.06f,-.18f,-.72f,.30f,.07f,-.90f,-.49f,-.50f,.90f,.73f,-.00f,.12f,-.03f,.50f,-.15f,.70f,-.82f,.54f,.73f,-.49f,-.50f,-.22f,-.57f,.58f,-.73f,-.69f,-.58f,-.81f,-.48f,.92f,-.83f,-.67f,-.43f,-.64f,.53f,.00f,-.71f,-.36f,.38f,.92f,.98f,-.60f,-.52f,.11f,.78f,-.40f,.81f,-.50f,-.93f,-.04f,-.65f,.44f,-.66f,-.23f,.09f,-.14f,-.40f,.26f,.82f,.68f,.61f,.93f,.02f,-.86f,-.74f,-.81f,-.84f,-.64f,-.70f,-.36f,.83f,-.57f,-.74f,-.22f,.51f,-.96f,-.07f,-.57f,.18f,.95f,.25f,.76f,-.59f,-.09f,-.74f,.33f,.64f,-.90f,.62f,-.16f,-.69f,.46f,-.86f,.21f,-.25f,.27f,.66f,.63f,-.19f,-.16f,-.31f,-.37f,.99f,.96f,-.84f,-.05f,-.21f,.05f,-.22f,-.09f,.32f,.61f,.94f,-.30f,.73f,-.79f,-.84f,.36f,-.34f,.00f,.19f,.14f,-.94f,-.98f,.93f,.09f,-.75f,.53f,-.39f,-.95f,-.21f,.17f,.87f,.21f,-.29f,.15f,-.96f,-.30f,-.13f,.43f,-.01f,-.08f,.57f,.77f,.98f,-.74f,-.89f,.79f,-.10f,-.15f,.28f,.47f,-.62f,.86f,.77f,.21f,-.22f,.63f,-.42f,-.10f,.42f,.02f,-.15f,-.74f,.66f,-.69f,-.82f,-.39f,.17f,-.16f,-.04f,-.69f,.97f,-.33f,.74f,-.69f,-.01f,-.87f,.43f,.99f,-.18f,-.37f,.86f,-.96f,.97f,-.62f,-.02f,.81f,-.21f,.78f,-.36f,.79f,.14f,.04f,.13f,.77f,.84f,-.21f,-.24f,.50f,-.46f,-.64f,-.43f,-.26f,-.08f,-.14f,-.70f,.77f,-.59f,-.39f,.60f,-.06f,.42f,-.83f,-.43f,.90f,.76f,-.52f,-.92f,-.22f,-.15f,-.54f,.87f,.20f,.80f,.93f,-.14f,-.33f,-.83f,.20f,-.98f,.18f,.32f,-.65f,.00f,-.22f,.89f,-.07f,-.31f,.05f,-.49f,-.58f,.11f,-.23f,.18f,-.58f,-.30f,-.83f,.39f,.56f,-.49f,.45f,-.79f,.79f,-.67f,.67f,-.15f,.32f,-.91f,.46f,.11f,.49f,.78f,.33f,-.66f,-.64f,.63f,-.42f,-.25f,-.80f,-.24f,-.69f,.56f,.84f,.77f,.74f,-.39f,.54f,.83f,.22f,-.98f,.26f,.17f,.63f,.00f,.55f,-.92f,.94f,-.32f,-.87f,-.74f,.11f,.61f,.55f,-.72f,.19f,-.44f,-.97f,-.25f,-.94f,.75f,-.41f,-.40f,-.83f,-.82f,-.51f,-.26f,.49f,.98f,.26f,.91f,.30f,-.37f,.50f,-.77f,.39f,-.02f,.22f,-.91f,.25f,.98f,.92f,.85f,-.23f,-.00f,-.35f,-.63f,.88f,.39f,.74f,-.04f,.34f,.02f,-.82f,.73f,.33f,-.59f,.47f,.56f,.48f,-.34f,.84f,-.68f,-.28f,.96f,-.06f,.30f,-.75f,.91f,.09f,.29f,-.84f,.97f,-.66f,.52f,-.00f,.56f,-.46f,.29f,.49f,.64f,.77f,-.24f,.20f,.34f,.40f,.20f,.85f,-.80f,.28f,.93f,.72f,-.30f,.62f,.61f,-.63f,-.66f,-.46f,.95f,.05f,-.47f,.83f,.02f,-.33f,-.35f,.38f,-.51f,.11f,.52f,.14f,.87f,-.71f,-.87f,-.22f,.52f,.82f,-.77f,.72f,.48f,-.11f,-.77f,.64f,-.61f,.09f,-.16f,-.30f,.56f,.50f,-.66f,-.39f,.81f,.42f,.29f,-.32f,.67f,-.59f,-.60f,.74f,.93f,-.49f,-.34f,-.35f,.32f,-.75f,.32f,-.03f,-.39f,.61f,-.84f,-.46f,-.33f,.99f,-.71f,.71f,-.48f,.27f,.65f,-.08f,.86f,-.69f,.58f,.10f,-.64f,.82f,-.66f,-.65f,.58f,-.42f,.85f,-.34f,.99f,-.02f,-.05f,.51f,-.40f,.24f,.61f,-.13f,-.78f,-.31f,-.89f,.62f,.30f,-.27f,-.68f,-.41f,-.77f,.64f,.94f,.01f,.29f,-.45f,-.15f,-.73f,.78f,-.27f,-.80f,-.70f,-.30f,-.69f,.38f,-.31f,.37f,.08f,.97f,-.32f,-.01f,.70f,.61f,.39f,.58f,.35f,.32f,.17f,.86f,-.04f,-.75f,.98f,.81f,-.58f,.78f,.94f,.65f,.89f,-.36f,-.43f,.27f,.99f,-.99f,.11f,.04f,-.08f,-.16f,.20f,-.66f,.87f,-.05f,-.57f,.27f,.90f,-.41f,-.60f,.52f,-.95f,.30f,.26f,.54f,-.71f,.44f,-.70f,-.29f,-.84f,-.21f,-.59f,.55f,-.98f,-.85f,-.98f,.84f,.99f,.25f,.14f,-.42f,-.91f,-.48f,-.52f,-.49f,.33f,.05f,-.46f,.97f,.89f,-.12f,-.45f,.91f,-.13f,-.22f,-.74f,-.34f,.00f,.34f,.16f,-.31f,-.11f,.77f,-.70f,.81f,.84f,-.30f,.18f,.98f,-.12f,-.36f,.48f,.26f,-.87f,.21f,.64f,.36f,-.85f,-.04f,.44f,-.89f,.94f,-.98f,.19f,-.40f,-.48f,-.36f,.77f,.29f,.18f,-.02f,-.72f,.44f,.35f,.42f,-.27f,.51f,.94f,.14f,.53f,-.56f,-.07f,.37f,-.33f,.83f,-.53f,.33f,.86f,-.13f,-.52f,.11f,-.23f,.25f,-.65f,-.06f,.42f,.71f,.94f,-.78f,.66f,.96f,.54f,.20f,-.36f,-.52f,-.26f,-.59f,.42f,.34f,-.23f,-.35f,.67f,.49f,.22f,-.31f,.17f,-.34f,.13f,.06f,.78f,-.24f,-.50f,.41f,-.18f,-.93f,-.98f,-.02f,-.16f,-.11f,-.37f,-.66f,-.63f,.14f,-.87f,-.60f,.60f,-.45f,.75f,-.41f,.62f,.24f,.71f,.63f,.77f,.11f,-.99f,.03f,.50f,-.72f,-.77f,-.25f,.00f,.51f,-.77f,-.35f,-.91f,.94f,.40f,.89f,.25f,-.42f,-.24f,-.15f,-.85f,.32f,-.42f,.66f,.57f,-.89f,-.08f,.76f,-.57f,.09f,-.26f,-.30f,-.46f,-.44f,-.06f,.79f,.67f,-.60f,-.59f,-.05f,.11f,-.60f,-.22f,-.99f,.51f,-.30f,.70f,.85f,.00f,-.55f,-.26f,-.28f,-.55f,.85f,.49f,-.33f,-.95f,-.34f,-.26f,-.41f,-.37f,-.07f,.84f,-.94f,.36f,-.97f,.04f,.52f,.36f,.36f,-.41f,-.66f,.91f,.99f,-.70f,-.36f,.20f,.97f,-.87f,-.32f,-.86f,-.85f,-.33f,.69f,.44f,-.25f,-.44f,-.44f,-.90f,.93f,.74f,-.52f,.71f,.97f,.13f,.04f,.42f,.08f,-.71f,.08f,.13f,-.42f,-.08f,-.49f,-.13f,.91f,.08f,.02f,.42f,-.44f,-.32f,.34f,.27f,.70f,.49f,-.30f,-.00f,-.87f,.08f,.29f,.64f,.02f,-.29f,.23f,-.53f,-.93f,.42f,-.47f,-.79f,.66f,.75f,-.30f,.16f,.81f,-.81f,.36f,.71f,.90f,.15f,.98f,-.77f,.96f,.10f,.40f,.05f,-.76f,.71f,-.12f,-.30f,.93f,-.00f,-.53f,-.25f,-.10f,.04f,-.07f,.04f,-.89f,.53f,-.76f,.60f,.07f,-.31f,.06f,-.35f,.33f,.01f,-.25f,-.71f,-.38f,-.39f,.15f,-.28f,-.44f,-.64f,-.80f,-.61f,-.57f,.46f,.61f,-.49f,-.07f,-.17f,-.94f,.73f,-.06f,-.02f,.19f,.54f,.38f,.69f,-.67f,-.27f,.05f,.75f,-.48f,-.13f,-.18f,-.02f,.37f,.63f,-.97f,.30f,.82f,.82f,.88f,.25f,-.64f,.60f,-.41f,-.12f,-.73f,-.65f,-.53f,.02f,.58f,-.36f,-.53f,.88f,-.51f,.66f,-.79f,-.23f,.65f,-.38f,.99f,-.51f,-.62f,.72f,-.31f,-.87f,.56f,.18f,.71f,.01f,-.72f,.59f,.15f,-.73f,.43f,-.81f,-.62f,.08f,.82f,.98f,.94f,-.97f,.97f,-.27f,-.17f,-.41f,-.20f,-.61f,.31f,-.25f,-.58f,.95f,-.36f,.28f,-.54f,-.92f,-.87f,-.61f,.12f,-.24f,.71f,.90f,-.29f,-.75f,.78f,.30f,.51f,-.48f,-.39f,.75f,-.90f,.65f,-.07f,.17f,-.55f,.27f,-.66f,-.23f,-.13f,.06f,-.27f,.13f,.53f,-.06f,.88f,.81f,-.56f,-.09f,-.26f,-.84f,-.22f,-.59f,.23f,.60f,-.60f,.90f,-.86f,.00f,-.43f,-.15f,-.33f,-.89f,.20f,-.08f,.37f,.24f,.86f,.67f,-.52f,-.88f,.45f,-.56f,.00f,-.45f,.07f,-.82f,.08f,-.27f,-.64f,-.88f,.92f,-.99f,-.12f,.89f,.35f,-.29f,.48f,-.35f,-.50f,-.60f,.31f,-.46f,.60f,.47f,.54f,.45f,.86f,.54f,-.99f,.59f,-.01f,.86f,-.10f,.54f,-.76f,-.88f,.75f,-.61f,.40f,-.25f,.92f,.14f,-.18f,.04f,-.31f,.29f,-.80f,.67f,-.60f,-.76f,-.01f,-.19f,.18f,.55f,.03f,.23f,-.30f,-.66f,-.12f,-.16f,.97f,.40f,.52f,-.15f,-.89f,.71f,.31f,.96f,.90f,-.19f,-.35f,.28f,-.64f,-.45f,-.78f,-.98f,-.36f,.00f,-.53f,-.91f,-.41f,.21f,.43f,-.24f,-.37f,-.91f,.66f,.19f,.76f,.72f,-.44f,.55f,.51f,.45f,.68f,-.26f,.09f,-.87f,.16f,.52f,.85f,-.61f,-.28f,-.50f,.32f,-.24f,.09f,.18f,.46f,.80f,-.04f,.07f,.29f,-.57f,-.22f,-.92f,-.49f,.98f,.99f,.61f,.26f,.85f,-.24f,.47f,-.40f,-.59f,-.00f,-.89f,-.33f,.37f,.26f,.31f,-.77f,-.89f,-.03f,-.26f,.85f,.71f,-.12f,.64f,.92f,-.54f,.96f,.42f,.85f,-.85f,.22f,.61f,.64f,-.82f,.01f,-.07f,.58f,-.33f,.25f,.26f,-.52f,.44f,-.84f,.50f,-.02f,-.29f,.57f,-.43f,-.62f,.66f,-.01f,-.86f,.83f,.90f,-.16f,.66f,-.81f,-.30f,.15f,-.62f,.38f,.92f,-.11f,.39f,.51f,-.27f,.62f,-.86f,.00f,-.16f,.70f,-.77f,-.92f,-.73f,-.28f,-.47f,-.70f,-.66f,-.78f,-.86f,.56f,.28f,.31f,.96f,-.64f,.18f,.49f,-.81f,-.56f,-.98f,.99f,-.93f,.04f,-.63f,-.22f,-.63f,-.49f,.08f,.47f,.71f,-.35f,-.76f,.01f,-.92f,.80f,.23f,.69f,-.66f,.14f,-.21f,.11f,-.38f,-.11f,-.12f,-.77f,-.90f,-.24f,.78f,.54f,.84f,.18f,-.26f,-.08f,-.66f,-.83f,-.28f,.90f,-.89f,.98f,-.75f,-.92f,-.56f,.29f,.03f,.25f,-.64f,.23f,-.30f,.44f,.98f,-.48f,.66f,.80f,-.31f,.17f,.45f,-.43f,.59f,-.78f,.10f,.27f,.04f,.85f,-.25f,-.22f,-.07f,-.64f,-.86f,-.39f,.73f,.81f,-.71f,.63f,.87f,-.41f,.45f,-.00f,-.01f,.40f,.10f,-.59f,.13f,-.11f,.13f,.18f,-.06f,-.18f,.28f,-.42f,-.62f,-.82f,.34f,-.61f,.61f,-.47f,.34f,-.68f,.92f,-.08f,.90f,.52f,.71f,-.14f,.73f,-.72f,-.43f,.15f,-.10f,.49f,-.33f,.71f,-.13f,-.48f,-.39f,.08f,.50f,.47f,-.79f,-.21f,-.75f,.89f,.15f,.21f,.62f,-.05f,.62f,-.83f,.87f,-.34f,.61f,-.60f,-.42f,.11f,.03f,-.66f,-.05f,.42f,.19f,.82f,.69f,-.33f,-.92f,-.02f,-.00f,.82f,.83f,-.02f,.44f,.11f,.19f,.01f,-.09f,.38f,-.32f,.30f,.84f,-.65f,-.01f,-.08f,-.02f,-.83f,-.47f,.95f,-.99f,.37f,-.38f,.21f,.35f,-.38f,.92f,.60f,.46f,-.14f,.28f,.20f,-.15f,-.03f,.96f,-.33f,.08f,-.48f,-.56f,.96f,-.03f,-.56f,-.25f,.90f,.16f,.21f,-.45f,.77f,.21f,.61f,.74f,-.07f,.93f,-.35f,-.26f,.52f,.74f,-.91f,.33f,.09f,-.45f,.58f,-.53f,.31f,.42f,.70f,-.85f,-.19f,.63f,-.87f,.67f,-.49f,.51f,-.08f,.96f,-.80f,.00f,.79f,.44f,.69f,-.37f,-.86f,-.63f,.90f,.79f,-.56f,-.26f,.66f,.62f,.22f,.27f,.75f,-.86f,.23f,.34f,-.50f,-.40f,-.05f,.12f,-.26f,.40f,-.74f,-.94f,-.65f,-.36f,-.66f,-.96f,-.34f,.57f,-.44f,.09f,-.88f,.66f,-.04f,.46f,-.33f,-.72f,.69f,.58f,-.57f,-.75f,.48f,.31f,-.04f,.34f,-.95f,.49f,-.63f,-.27f,.61f,-.96f,-.00f,-.27f,-.26f,.26f,-.33f,.30f,.61f,.81f,.12f,.24f,.24f,-.07f,-.23f,-.50f,-.55f,-.22f,-.89f,-.97f,.05f,.46f,.11f,.62f,-.02f,-.34f,-.94f,.04f,.97f,.43f,.58f,-.46f,.59f,-.36f,.47f,-.06f,-.57f,-.57f,.10f,.38f,-.43f,.23f,-.91f,.35f,.09f,.91f,.30f,.84f,.09f,.40f,-.75f,.50f,-.36f,-.95f,-.86f,-.78f,-.69f,.28f,.93f,.19f,-.15f,.90f,-.19f,.71f,.98f,.15f,.31f,-.87f,-.61f,.51f,.71f,.67f,-.40f,.70f,.34f,.61f,.61f,.47f,.49f,-.13f,-.43f,.79f,-.45f,-.57f,.40f,-.23f,.53f,.27f,-.58f,.43f,-.10f,-.72f,.27f,-.09f,.56f,.64f,.11f,-.57f,-.27f,.60f,-.29f,.16f,.77f,-.96f,.40f,.13f,.86f,-.42f,.62f,-.21f,-.11f,.92f,.07f,-.08f,.70f,.34f,-.82f,-.46f,.86f,-.01f,-.42f,.41f,.74f,-.03f,-.69f,.38f,.03f,.94f,-.81f,-.44f,.26f,-.48f,-.01f,.81f,.41f,-.87f,-.35f,-.23f,-.32f,.19f,-.31f,-.36f,-.39f,.96f,-.03f,.64f,.65f,-.40f,.80f,.61f,.83f,.37f,.63f,.42f,-.01f,.95f,-.67f,.59f,-.02f,-.06f,.63f,.20f,.27f,-.15f,.13f,.49f,-.43f,.92f,-.76f,.20f,-.35f,.20f,-.82f,-.79f,.09f,-.62f,.60f,-.20f,-.89f,-.22f,-.63f,.31f,.27f,-.67f,.23f,.13f,.18f,-.82f,.95f,.52f,-.26f,.55f,-.77f,-.29f,.43f,-.34f,-.61f,.72f,.21f,-.21f,.21f,.47f,-.02f,-.13f,.10f,.81f,.73f,-.50f,-.59f,-.32f,-.18f,-.02f,.19f,-.58f,-.35f,-.52f,.40f,-.88f,-.87f,.75f,-.08f,-.36f,.49f,.46f,.72f,.47f,-.69f,-.53f,-.67f,.27f,.51f,.73f,-.70f,-.52f,.83f,.03f,-.03f,-.72f,.22f,.29f,-.92f,-.07f,-.58f,.21f,-.62f,.08f,.52f,-.70f,.68f,-.72f,-.66f,.24f,.80f,.48f,.46f,-.41f,-.47f,.06f,-.11f,-.30f,.92f,-.28f,.53f,-.08f,.28f,.55f,.87f,.65f,-.00f,-.44f,-.17f,.35f,-.43f,.01f,-.92f,.01f,-.33f,.21f,.34f,.67f,.48f,-.23f,-.08f,.07f,-.17f,-.02f,-.62f,-.56f,.47f,-.10f,.49f,-.33f,.04f,-.92f,-.68f,.80f,.49f,.42f,.86f,-.23f,-.74f,-.64f,.83f,.39f,-.53f,-.69f,-.86f,-.30f,.22f,-.85f,.78f,.00f,-.86f,.94f,.33f,.85f,.33f,-.34f,.06f,-.25f,.26f,-.88f,.90f,-.47f,-.29f,-.56f,-.59f,.66f,-.06f,-.03f,.75f,-.64f,.03f,-.15f,.69f,-.59f,.66f,.37f,-.21f,.04f,-.24f,.49f,-.30f,-.27f,-.93f,-.16f,.73f,-.22f,-.63f,.98f,-.47f,.60f,-.67f,-.31f,-.24f,-.57f,.10f,.49f,-.52f,-.81f,-.28f,.13f,-.36f,.87f,-.61f,.38f,-.35f,.05f,-.01f,.38f,.92f,-.43f,-.87f,.35f,.84f,.47f,.19f,.71f,-.16f,.32f,.64f,-.27f,.30f,-.88f,.24f,.97f,-.02f,.59f,-.45f,-.13f,.32f,.40f,-.69f,.24f,-.06f,-.29f,-.74f,.01f,.36f,.16f,.18f,-.89f,-.62f,.63f,-.83f,.37f,.88f,.53f,.43f,.16f,.72f,.94f,-.04f,.86f,.64f,.58f,-.03f,-.83f,.78f,-.96f,-.11f,-.30f,.20f,-.60f,-.02f,.52f,.62f,-.99f,-.96f,.17f,-.79f,-.87f,-.32f,.39f,.31f,.02f,-.84f,-.77f,.49f,.65f,.83f,-.30f,.98f,-.13f,-.94f,.14f,.40f,-.47f,.81f,.88f,-.75f,.88f,.54f,.89f,-.52f,.92f,-.10f,.11f,.83f,.11f,-.99f,.22f,-.41f,.00f,.80f,-.53f,.19f,.26f,-.17f,-.70f,-.56f,-.20f,.26f,.91f,-.83f,.67f,.56f,.83f,.43f,-.39f,.28f,.83f,-.43f,.92f,.88f,-.68f,-.37f,.62f,-.68f,.70f,-.16f,-.78f,.88f,-.84f,.32f,-.22f,.72f,.58f,.40f,.24f,.90f,-.08f,-.03f,-.89f,.55f,-.76f,-.34f,-.15f,.19f,.36f,.46f,-.60f,-.43f,.54f,-.38f,-.02f,-.21f,-.60f,-.74f,.92f,.65f,.47f,.75f,.58f,.95f,-.22f,-.64f,-.28f,-.12f,.84f,-.37f,-.18f,-.50f,.08f,.14f,.31f,-.56f,-.18f,.32f,.52f,.46f,.19f,.31f,-.48f,-.95f,-.31f,.83f,-.31f,.22f,.41f,-.02f,.05f,.60f,.36f,.78f,.50f,.50f,.61f,-.59f,-.00f,.50f,.96f,-.80f,-.56f,.92f,-.00f,.25f,.79f,.76f,.14f,-.00f,-.99f,-.85f,.81f,-.26f,-.58f,.21f,-.68f,-.90f,.68f,-.48f,.59f,.08f,.62f,.38f,.64f,-.54f,.65f,.60f,-.45f,-.54f,-.40f,.13f,-.94f,-.86f,-.67f,.09f,-.53f,-.45f,.23f,.19f,.62f,-.18f,-.51f,.56f,.81f,.55f,-.62f,.60f,.44f,.61f,.58f,.13f,.13f,.46f,-.04f,-.01f,-.61f,-.76f,-.87f,-.25f,.69f,-.29f,.53f,-.59f,-.22f,.22f,-.08f,.77f,-.08f,.65f,-.90f,.19f,.82f,-.26f,.00f,.12f,.22f,-.88f,.52f,-.37f,-.97f,.34f,.94f,.84f,.88f,.35f,.31f,-.22f,.41f,-.96f,.93f,-.43f,.30f,-.20f,-.41f,.96f,.36f,-.88f,.72f,.32f,-.23f,.37f,-.80f,.05f,.52f,.50f,-.94f,-.43f,-.05f,-.44f,.87f,-.11f,-.01f,.85f,.41f,.30f,-.13f,-.02f,-.49f,-.68f,-.15f,-.49f,-.04f,-.40f,-.56f,.32f,.70f,-.38f,-.12f,-.64f,-.69f,-.22f,.40f,-.77f,.44f,-.36f,-.68f,-.45f,-.39f,.00f,-.44f,.73f,.14f,-.21f,-.50f,-.28f,.04f,.25f,.24f,-.70f,-.04f,.78f,.90f,-.37f,.14f,-.31f,-.98f,-.23f,.15f,-.04f,-.43f,-.51f,.34f,-.69f,-.33f,-.00f,.33f,.77f,.55f,.88f,.56f,-.06f,.86f,.61f,-.97f,.42f,.96f,-.23f,.56f,-.82f,.41f,.87f,-.93f,-.40f,-.71f,-.88f,.49f,-.39f,-.66f,-.16f,.57f,-.77f,.15f,-.36f,-.13f,.69f,.36f,.75f,-.84f,-.44f,.77f,.69f,-.89f,.90f,-.91f,.09f,.86f,.27f,.83f,.75f,-.17f,.80f,.40f,.20f,-.40f,.87f,-.85f,-.47f,-.40f,-.47f,-.17f,-.42f,-.14f,-.83f,.04f,.80f,.72f,-.07f,-.24f,-.32f,.91f,-.67f,.27f,.65f,.68f,-.94f,-.88f,-.27f,.74f,-.36f,.88f,-.78f,.44f,-.31f,.21f,.57f,.01f,.25f,-.02f,.44f,-.88f,.96f,-.91f,.66f,.88f,.97f,.53f,-.90f,.07f,.70f,-.41f,-.52f,-.00f,-.48f,.27f,-.19f,.96f,-.00f,.23f,.13f,-.56f,-.69f,.66f,.19f,-.79f,.87f,.91f,-.35f,-.67f,.01f,-.64f,-.48f,.77f,.29f,-.32f,-.36f,-.07f,-.36f,-.03f,.51f,-.99f,.42f,-.95f,.31f,.96f,.58f,.91f,.96f,.42f,-.76f,-.73f,-.52f,.64f,.24f,-.28f,-.50f,.56f,.50f,.41f,.54f,.10f,.10f,-.57f,-.73f,.50f,.36f,-.71f,.49f,-.08f,-.33f,-.78f,-.77f,.84f,.40f,.91f,-.39f,.26f,-.76f,-.25f,-.15f,-.07f,-.34f,.82f,-.94f,.21f,-.99f,.33f,.79f,-.41f,.71f,-.40f,.87f,.82f,-.34f,-.99f,.35f,-.41f,.10f,.60f,.84f,-.28f,.46f,.63f,-.47f,-.24f,-.05f,-.92f,.55f,.59f,-.23f,.92f,.86f,-.16f,-.51f,-.63f,-.34f,.56f,-.98f,-.07f,.64f,-.83f,.71f,.40f,-.47f,-.67f,.80f,-.64f,.86f,-.00f,-.64f,-.54f,.53f,-.54f,.13f,-.73f,.71f,.44f,.62f,.17f,.80f,.82f,-.87f,-.54f,-.56f,.35f,.02f,.18f,-.06f,-.00f,.13f,.16f,.18f,-.11f,.35f,.70f,.84f,.48f,-.03f,.07f,.20f,-.84f,-.18f,-.30f,.79f,-.88f,-.28f,.64f,.70f,-.05f,-.18f,.59f,-.87f,-.98f,.57f,.11f,.84f,-.33f,.55f,-.89f,-.07f,-.64f,-.17f,.74f,.70f,-.54f,.42f,-.64f,-.74f,-.90f,-.40f,.69f,.62f,.26f,.50f,-.61f,.60f,.54f,.23f,.30f,.68f,.96f,-.19f,.38f,-.73f,.04f,-.44f,.51f,.87f,.56f,.72f,.26f,-.61f,-.36f,.67f,.64f,-.67f,-.85f,-.30f,.37f,-.34f,-.00f,.30f,.86f,-.19f,.08f,.46f,-.14f,.45f,-.52f,.40f,-.69f,-.60f,-.53f,-.35f,-.15f,-.01f,-.93f,-.97f,.48f,.10f,.64f,.61f,-.02f,-.32f,.67f,.30f,-.73f,-.75f,-.24f,.43f,-.68f,-.09f,.76f,-.50f,-.85f,-.94f,-.73f,.85f,-.73f,-.93f,.52f,-.67f,.67f,.15f,.25f,.24f,-.06f,-.20f,-.43f,.77f,-.09f,.81f,.28f,.32f,-.34f,.15f,.95f,.48f,.37f,-.39f,.92f,.06f,.97f,-.59f,.93f,-.22f,-.16f,-.29f,.00f,.70f,-.78f,.56f,-.05f,.39f,.23f,-.89f,.29f,-.08f,-.67f,-.51f,-.09f,.64f,-.75f,-.89f,.91f,-.05f,-.33f,.84f,-.74f,-.95f,-.12f,.59f,.27f,.03f,.98f,-.16f,.63f,.24f,.65f,-.64f,-.89f,.27f,.55f,-.22f,-.27f,.68f,.89f,-.78f,.36f,-.98f,-.53f,.44f,.72f,-.00f,-.08f,-.58f,.96f,.05f,-.75f,.69f,.14f,-.55f,.83f,-.66f,.30f,-.85f,.63f,.03f,-.10f,.73f,.04f,.70f,-.95f,-.52f,-.45f,.97f,-.16f,-.31f,-.61f,-.55f,.58f,.20f,-.17f,-.80f,.75f,-.31f,-.48f,.81f,-.61f,.59f,-.21f,-.16f,.15f,-.84f,.90f,-.53f,.82f,.17f,-.75f,-.51f,-.00f,-.00f,-.07f,-.25f,-.40f,-.09f,-.39f,.50f,-.63f,.76f,-.80f,.03f,.44f,-.18f,-.75f,.08f,.55f,.22f,-.88f,.47f,.32f,.40f,.58f,-.08f,.50f,.40f,.41f,-.28f,.40f,-.30f,-.39f,.32f,.78f,.56f,-.55f,.08f,-.50f,-.94f,-.69f,.88f,-.97f,-.67f,.98f,.44f,-.78f,.08f,-.66f,.83f,-.84f,.93f,-.55f,.03f,-.25f,-.91f,-.95f,.88f,-.19f,-.23f,.67f,-.21f,-.03f,-.10f,-.77f,-.27f,.19f,-.37f,.95f,.29f,-.43f,.94f,.47f,.96f,-.92f,.26f,-.20f,-.00f,-.55f,.37f,-.76f,-.02f,-.30f,-.47f,.65f,-.14f,.09f,-.26f,-.19f,-.63f,-.28f,.58f,.87f,.87f,.50f,.73f,.96f,-.16f,.55f,.68f,-.62f,-.81f,.32f,-.47f,.29f,.53f,-.19f,-.98f,-.41f,.29f,-.48f,-.16f,.11f,.09f,-.27f,-.97f,.95f,.36f,-.66f,-.68f,.41f,.01f,.07f,-.85f,.13f,-.02f,.31f,-.53f,-.13f,-.29f,-.43f,.68f,.14f,.85f,-.57f,-.63f,-.46f,-.44f,-.70f,-.27f,.68f,.68f,.90f,-.59f,.33f,.63f,.40f,.69f,.26f,.44f,-.91f,-.00f,-.60f,-.45f,-.51f,.42f,.53f,-.19f,-.97f,-.79f,.98f,.99f,-.05f,-.17f,.16f,-.77f,.14f,.34f,.15f,.15f,-.00f,.27f,-.17f,.68f,.37f,.50f,-.14f,-.03f,-.70f,-.06f,-.33f,.50f,.57f,-.37f,-.96f,-.89f,-.43f,-.45f,.11f,-.33f,-.19f,-.62f,.81f,.56f,-.38f,.69f,.61f,-.30f,-.58f,-.67f,-.72f,-.83f,.64f,.94f,-.61f,.83f,-.43f,-.73f,-.67f,.96f,-.73f,.61f,.89f,-.97f,.80f,.30f,-.42f,.92f,.81f,.03f,.48f,.98f,-.04f,-.68f,.13f,.90f,.17f,.59f,.48f,-.40f,-.70f,-.25f,-.46f,.69f,-.02f,-.53f,-.60f,.55f,-.49f,-.47f,-.83f,.59f,-.26f,.21f,.86f,-.51f,-.26f,.59f,-.18f,-.93f,.55f,-.20f,-.92f,.87f,.95f,-.32f,.84f,.78f,.43f,.56f,.43f,.36f,.14f,-.19f,-.84f,-.30f,.48f,.63f,-.62f,-.54f,-.28f,-.08f,.06f,.71f,.64f,.60f,.88f,-.68f,-.91f,-.03f,.79f,.69f,-.51f,-.24f,.95f,-.41f,.01f,.97f,.94f,-.74f,.28f,.55f,-.48f,-.63f,-.85f,-.70f,.56f,.37f,.96f,-.80f,-.08f,.39f,.92f,-.02f,.85f,-.28f,-.14f,.81f,.35f,-.17f,-.39f,.12f,.36f,-.06f,.88f,-.59f,-.25f,-.50f,-.34f,.71f,-.72f,-.75f,.40f,.29f,.29f,.61f,.49f,-.79f,.05f,.03f,.48f,.58f,-.38f,.98f,.60f,-.79f,-.15f,-.24f,.37f,.51f,.95f,-.94f,.30f,-.58f,.62f,.57f,-.86f,.69f,-.18f,-.29f,.33f,.56f,-.37f,.02f,.12f,-.97f,.80f,-.46f,.55f,-.82f,-.60f,-.86f,-.80f,.60f,-.09f,-.51f,-.46f,-.86f,-.46f,-.30f,-.86f,.33f,.31f,-.73f,-.83f,.33f,.35f,-.26f,.58f,.75f,.89f,-.10f,-.55f,-.37f,.51f,.14f,-.37f,.58f,.08f,.54f,.37f,.43f,-.03f,.18f,-.48f,.27f,-.22f,.57f,-.27f,-.89f,.05f,.53f,-.71f,-.12f,-.53f,.57f,-.38f,-.22f,.49f,-.47f,.58f,-.26f,-.15f,.47f,.32f,.97f,-.82f,.70f,.15f,-.08f,.16f,-.76f,.14f,-.75f,-.49f,.68f,-.31f,.02f,.20f,.66f,.16f,-.10f,.13f,-.48f,-.78f,-.81f,-.61f,-.84f,-.23f,-.65f,.18f,.72f,.50f,-.09f,.56f,.97f,-.98f,-.49f,.59f,.37f,.32f,-.61f,.24f,-.76f,-.36f,-.69f,.02f,-.29f,.22f,.70f,.49f,-.99f,-.88f,.28f,-.38f,.17f,.73f,-.86f,-.17f,-.79f,.84f,.47f,.86f,-.23f,-.03f,-.69f,-.38f,-.92f,-.86f,-.46f,-.11f,.32f,.37f,-.29f,-.41f,-.05f,-.11f,.48f,-.16f,-.74f,-.83f,-.07f,-.04f,.14f,.31f,.39f,-.91f,.64f,-.05f,.22f,-.61f,-.70f,-.82f,-.45f,.31f,-.48f,-.31f,.61f,-.31f,.40f,.86f,-.92f,.16f,.02f,.49f,-.60f,-.72f,.71f,.03f,-.67f,.27f,.94f,.06f,.47f,.24f,-.72f,-.41f,.08f,.83f,-.81f,-.61f,-.48f,.08f,-.06f,.35f,-.65f,.91f,-.77f,-.90f,.38f,-.42f,-.48f,.37f,-.71f,.36f,.57f,-.08f,.36f,-.87f,.70f,-.53f,-.00f,.65f,-.73f,.19f,-.09f,.42f,.74f,-.29f,-.31f,-.66f,-.01f,-.40f,.71f,-.36f,.69f,.67f,-.86f,.64f,-.02f,.06f,.49f,.42f,.34f,-.35f,-.58f,-.94f,-.17f,.54f,-.91f,-.31f,.54f,-.19f,.69f,-.39f,.65f,.18f,-.54f,.64f,.43f,-.12f,.44f,-.95f,-.62f,.15f,-.71f,.00f,-.33f,-.30f,-.32f,-.36f,.44f,-.96f,-.65f,.02f,-.21f,.34f,.03f,.10f,-.13f,.58f,.40f,.10f,.75f,-.32f,-.22f,-.48f,.87f,-.03f,-.02f,.82f,.06f,.56f,.41f,.16f,.41f,-.52f,.38f,-.09f,.07f,-.42f,.70f,-.16f,-.13f,.22f,.83f,.56f,-.03f,.20f,-.34f,-.52f,.50f,.73f,.34f,.05f,-.39f,.61f,.67f,-.63f,.27f,-.73f,-.73f,.84f,-.65f,-.51f,-.05f,-.46f,-.38f,-.00f,.38f,-.40f,-.52f,.51f,-.63f,-.40f,-.02f,.39f,.18f,.60f,.30f,.32f,.23f,.77f,.21f,.12f,.49f,.30f,.72f,-.85f,.96f,.23f,-.08f,.57f,.20f,.87f,.99f,.54f,-.76f,.11f,.97f,-.10f,.19f,-.45f,-.12f,.85f,-.21f,.69f,.71f,-.61f,.01f,-.73f,-.61f,.51f,.33f,-.71f,.76f,.67f,.35f,-.51f,-.62f,-.46f,.28f,-.80f,.62f,.65f,-.11f,.87f,.24f,-.97f,.60f,-.03f,-.83f,.32f,-.49f,-.94f,-.42f,-.47f,.50f,.34f,.84f,.11f,-.86f,-.24f,.69f,-.53f,.89f,.00f,.08f,-.81f,-.07f,-.18f,-.52f,-.39f,-.94f,.04f,.89f,-.30f,.44f,-.68f,.49f,.40f,.65f,-.80f,-.72f,.42f,-.49f,.91f,.45f,.26f,.60f,.00f,.20f,.27f,-.00f,.27f,.91f,.44f,.84f,.81f,-.09f,.38f,-.49f,.56f,.04f,.54f,-.79f,.05f,.56f,.34f,-.20f,.12f,.00f,.48f,.18f,.15f,-.86f,.57f,-.03f,-.12f,.56f,-.33f,-.80f,.03f,.00f,.90f,.00f,.50f,.15f,-.09f,-.43f,.17f,-.28f,.21f,-.86f,-.41f,.69f,-.40f,.11f,-.95f,-.08f,-.98f,-.67f,.73f,-.97f,-.67f,-.09f,.52f,-.25f,.26f,.09f,-.66f,-.16f,.01f,.86f,-.11f,-.78f,.15f,-.02f,-.23f,.18f,.91f,.29f,.21f,-.53f,-.65f,.79f,-.68f,.87f,-.22f,-.07f,-.43f,.50f,.26f,.05f,.55f,-.30f,-.16f,-.43f,.56f,-.16f,.03f,-.92f,.32f,-.69f,.05f,-.34f,-.90f,.67f,-.24f,-.24f,.97f,.18f,-.31f,-.45f,-.98f,.94f,.84f,.52f,.80f,.71f,-.26f,.04f,-.78f,.10f,-.21f,-.21f,.42f,-.36f,-.98f,-.29f,-.79f,-.29f,-.89f,-.82f,.17f,-.11f,.59f,-.30f,.43f,.40f,.54f,-.35f,-.60f,-.30f,.61f,.52f,.20f,-.40f,.07f,.81f,.50f,.00f,.26f,.06f,.01f,-.43f,.68f,-.25f,-.28f,.01f,.29f,.87f,.68f,.18f,-.26f,-.16f,-.43f,-.59f,.74f,.91f,.33f,.88f,.07f,-.67f,.26f,-.88f,-.31f,-.25f,.63f,.08f,-.60f,-.08f,-.29f,-.41f,-.12f,-.65f,-.20f,-.73f,.50f,-.50f,-.55f,.80f,.01f,-.47f,-.33f,.52f,-.59f,-.61f,.99f,-.94f,.32f,-.50f,-.41f,.66f,-.06f,-.30f,.79f,.33f,.21f,-.79f,.09f,.66f,-.13f,-.32f,.05f,.32f,.07f,-.11f,-.16f,.72f,.69f,.76f,-.14f,-.96f,-.26f,.93f,.84f,-.00f,.23f,-.68f,.65f,.64f,-.37f,.94f,-.70f,-.04f,-.39f,.90f,-.62f,.02f,.88f,.77f,-.40f,.01f,-.45f,-.92f,-.18f,.79f,.52f,.86f,.57f,-.39f,-.39f,-.47f,.47f,-.70f,-.10f,.27f,.99f,-.62f,-.11f,-.99f,-.68f,.81f,-.94f,.08f,-.01f,.42f,.50f,.05f,.67f,.87f,-.11f,-.55f,-.20f,.12f,-.77f,.50f,-.46f,-.69f,-.76f,-.21f,-.49f,.08f,-.29f,-.44f,.59f,.17f,-.34f,.57f,-.00f,-.21f,.65f,.04f,-.26f,-.73f,.95f,.28f,-.33f,-.95f,-.55f,-.52f,-.93f,.57f,-.56f,-.50f,-.62f,-.73f,.94f,-.37f,.04f,-.09f,-.12f,-.71f,-.38f,-.08f,-.56f,.27f,.45f,-.47f,.84f,.24f,-.76f,-.10f,.96f,.72f,.67f,.04f,.91f,.50f,.29f,-.34f,-.87f,.00f,-.74f,.76f,-.20f,.38f,.54f,-.69f,.54f,-.24f,-.16f,.79f,-.17f,.53f,-.25f,-.54f,.40f,-.78f,-.41f,-.25f,.85f,.70f,-.77f,-.23f,-.96f,.75f,.52f,.59f,-.85f,.70f,-.23f,-.71f,-.22f,.87f,.44f,.57f,.75f,.44f,-.44f,.45f,.41f,-.90f,-.86f,.51f,.56f,-.41f,-.55f,.53f,.78f,-.71f,-.43f,-.20f,.98f,-.39f,-.42f,-.52f,.40f,-.67f,-.32f,.12f,.95f,.99f,-.75f,.97f,-.33f,-.24f,.31f,-.50f,-.54f,-.20f,.92f,-.81f,.38f,-.62f,-.39f,.82f,.91f,-.75f,.03f,-.06f,-.61f,-.17f,-.91f,.27f,.30f,-.75f,-.51f,.50f,.60f,-.96f,-.91f,-.52f,.34f,.92f,.31f,-.02f,.89f,.93f,-.77f,.53f,.98f,.01f,.45f,.56f,-.41f,.38f,-.96f,.72f,-.11f,-.05f,.88f,-.78f,-.27f,-.68f,-.41f,-.16f,-.08f,.70f,.16f,.13f,-.29f,.53f,-.52f,-.75f,.99f,.73f,.11f,.11f,-.05f,.27f,-.87f,-.19f,-.59f,-.73f,.74f,-.18f,.13f,-.32f,.55f,.03f,-.20f,.45f,-.06f,-.10f,-.86f,.94f,-.63f,.88f,.23f,.28f,-.62f,-.59f,.76f,.41f,-.29f,.55f,.01f,.36f,.33f,.06f,.67f,-.63f,-.36f,.77f,-.72f,.76f,-.87f,.95f,-.28f,-.83f,-.67f,-.64f,-.49f,.55f,.06f,.90f,-.01f,-.58f,.94f,-.76f,-.24f,.58f,-.99f,.17f,-.14f,.65f,-.80f,.77f,-.18f,-.37f,.07f,-.71f,-.31f,.43f,.59f,-.52f,-.73f,-.17f,.04f,-.25f,-.62f,.90f,.40f,-.60f,-.31f,.52f,-.60f,.90f,.78f,.13f,-.27f,-.71f,.71f,.24f,.93f,-.39f,.33f,-.02f,.45f,.61f,-.50f,.81f,.95f,-.70f,.72f,.03f,-.16f,.07f,-.28f,-.83f,.84f,-.48f,-.42f,.62f,.62f,.39f,.88f,.03f,-.41f,-.49f,-.80f,-.60f,.55f,.83f,-.02f,-.69f,-.19f,.65f,.49f,-.32f,-.28f,.05f,-.40f,-.03f,-.39f,.48f,-.75f,-.02f,-.91f,-.25f,.33f,.80f,-.30f,-.14f,-.36f,.61f,.82f,.48f,.31f,-.87f,.45f,.06f,.15f,.08f,-.39f,.10f,-.21f,.63f,-.47f,.55f,.47f,.90f,.32f,-.29f,.79f,-.76f,.75f,-.90f,.56f,.92f,-.04f,-.73f,-.75f,-.68f,-.74f,.99f,-.68f,-.49f,.31f,.76f,-.76f,-.03f,-.95f,-.75f,-.89f,-.32f,.91f,-.34f,.58f,-.43f,.33f,-.88f,-.32f,.90f,.23f,-.40f,-.86f,.81f,.26f,-.90f,.78f,-.61f,.65f,-.75f,.69f,-.29f,-.17f,-.92f,.84f,.80f,-.35f,.53f,.73f,-.60f,-.75f,-.46f,-.68f,-.13f,.80f,-.21f,.38f,.89f,-.53f,.34f,-.32f,.71f,.08f,-.43f,.27f,-.17f,.48f,.75f,-.89f,-.47f,-.94f,-.23f,.52f,-.47f,.34f,-.84f,-.81f,-.14f,.36f,-.73f,-.35f,-.06f,-.66f,-.16f,-.84f,-.95f,-.52f,-.02f,-.28f,.85f,.91f,-.61f,-.16f,-.82f,.10f,.19f,-.51f,.04f,-.23f,.74f,-.18f,.31f,-.63f,-.37f,-.27f,-.95f,-.36f,-.32f,-.85f,.52f,.16f,.75f,-.68f,-.99f,-.18f,-.82f,.90f,.61f,-.96f,.55f,.61f,-.82f,.15f,.21f,-.46f,.06f,.35f,.95f,.25f,-.10f,-.08f,.58f,.14f,-.54f,.21f,.48f,.53f,.09f,.75f,.82f,-.12f,-.45f,.65f,.29f,-.04f,.43f,.84f,-.97f,.04f,-.14f,-.02f,.09f,-.46f,.01f,-.90f,-.54f,-.71f,-.03f,-.45f,-.02f,-.49f,-.71f,.06f,-.48f,.90f,.83f,.20f,.42f,-.99f,-.11f,.53f,-.35f,-.01f,.67f,-.68f,-.08f,-.52f,.44f,-.61f,-.58f,-.15f,-.47f,.90f,.37f,-.05f,-.52f,.54f,.88f,.19f,-.36f,.19f,.82f,.04f,-.37f,.36f,.97f,-.93f,.97f,-.82f,.28f,.17f,-.19f,.17f,-.24f,-.69f,.48f,-.01f,.43f,-.70f,.98f,-.49f,-.51f,.97f,.05f,-.87f,.57f,-.93f,-.45f,.78f,-.22f,.70f,.91f,-.54f,.67f,-.53f,-.01f,.38f,.99f,-.56f,.85f,.67f,.88f,-.02f,.33f,-.00f,.74f,.33f,-.37f,-.92f,.79f,-.93f,.24f,.18f,-.56f,-.83f,-.22f,.79f,-.04f,.54f,-.11f,-.40f,.27f,.44f,.50f,.24f,.96f,-.02f,.24f,.18f,-.95f,-.03f,.93f,-.15f,.86f,-.24f,-.30f,.80f,-.05f,.91f,.55f,.05f,.44f,-.43f,.00f,-.96f,-.55f,.52f,-.11f,-.56f,-.08f,.91f,.05f,-.70f,.13f,.43f,.07f,.81f,-.49f,.14f,-.89f,-.38f,.19f,.57f,.94f,.66f,.98f,.75f,.37f,-.12f,.12f,.96f,.40f,-.10f,-.64f,-.88f,-.64f,.95f,-.81f,.61f,-.97f,-.68f,-.73f,-.00f,-.69f,-.99f,.73f,-.09f,-.49f,.22f,.04f,.77f,.79f,-.06f,-.35f,.06f,.81f,-.03f,-.05f,.47f,.27f,-.74f,-.59f,.82f,.47f,.80f,.80f,.58f,-.87f,.05f,.50f,-.26f,-.50f,-.84f,.12f,.62f,.66f,-.77f,-.10f,.74f,.66f,-.07f,.73f,.74f,.52f,.10f,-.78f,.24f,.92f,.02f,-.12f,.80f,.60f,.76f,.96f,.57f,.94f,-.37f,-.01f,-.20f,-.95f,-.37f,-.61f,.74f,.82f,.31f,.32f,-.47f,-.91f,-.60f,-.78f,.19f,-.65f,.44f,-.63f,-.19f,-.62f,.39f,-.58f,-.80f,-.47f,.29f,-.31f,.19f,.13f,.59f,-.52f,.32f,.71f,-.69f,-.21f,-.94f,-.80f,-.82f,-.81f,-.92f,-.08f,-.68f,.19f,.31f,-.02f,-.60f,.97f,-.71f,.21f,-.18f,.19f,.83f,.79f,-.18f,-.91f,-.77f,-.80f,-.00f,.08f,.30f,-.86f,-.88f,-.01f,-.88f,.71f,-.80f,-.54f,.43f,-.68f,-.56f,.32f,.05f,.21f,-.51f,.17f,-.42f,.01f,.00f,.03f,.16f,-.77f,.62f,-.91f,-.70f,-.42f,-.49f,.46f,-.82f,.97f,-.49f,-.49f,.42f,.23f,-.19f,.23f,.99f,-.57f,-.93f,-.46f,.05f,-.86f,.23f,.94f,-.68f,.73f,.08f,.33f,.63f,.65f,.11f,.41f,-.16f,-.05f,.80f,.65f,.07f,-.76f,-.69f,.34f,.18f,.69f,.58f,.67f,.56f,.02f,-.72f,.07f,.38f,-.30f,.59f,.00f,-.67f,.52f,.40f,.60f,-.26f,-.16f,-.99f,-.89f,.48f,.03f,-.59f,-.62f,-.51f,.10f,.79f,.47f,.66f,.23f,-.51f,-.08f,-.84f,-.38f,.27f,.11f,-.46f,.50f,-.75f,-.49f,-.44f,-.86f,-.46f,.70f,-.21f,-.33f,-.69f,.91f,.79f,.29f,-.90f,.70f,-.92f,-.49f,.85f,-.58f,-.71f,.53f,.88f,.29f,.35f,.75f,-.23f,.90f,-.49f,.04f,.07f,.13f,-.12f,-.84f,-.09f,-.19f,-.26f,-.73f,-.66f,-.83f,-.14f,.38f,-.95f,.39f,-.69f,.46f,.44f,.53f,-.38f,.01f,-.41f,.10f,.90f,-.36f,.58f,.41f,.11f,-.73f,.14f,.97f,.89f,-.24f,.49f,.65f,-.52f,.63f,-.82f,.34f,.95f,-.90f,-.19f,-.48f,-.05f,-.92f,-.48f,-.09f,-.72f,.64f,-.92f,.64f,-.85f,-.78f,-.44f,-.67f,-.48f,.91f,-.45f,.23f,-.35f,-.10f,.49f,.65f,.29f,.03f,.59f,-.92f,.24f,.48f,.86f,.29f,.77f,.40f,-.56f,-.27f,-.99f,.95f,.99f,-.65f,.71f,.21f,-.14f,.51f,.25f,-.31f,.73f,.37f,.62f,.58f,-.81f,-.68f,.41f,-.88f,-.20f,.40f,-.71f,-.71f,.97f,-.28f,-.73f,.36f,-.57f,-.56f,-.33f,.98f,-.32f,.25f,-.85f,-.16f,-.14f,-.17f,-.57f,.12f,.03f,-.77f,.12f,-.58f,-.90f,.51f,-.29f,-.47f,.89f,-.81f,.69f,.70f,-.40f,.15f,-.66f,.54f,-.75f,-.81f,-.87f,-.26f,.31f,-.56f,-.15f,.04f,-.86f,.07f,.19f,-.80f,-.94f,.04f,-.01f,.87f,.83f,-.44f,-.77f,-.73f,.24f,-.87f,-.61f,.85f,.40f,-.83f,.18f,-.88f,.49f,-.27f,-.25f,-.59f,-.47f,-.91f,-.81f,-.60f,-.73f,.78f,-.81f,.70f,.20f,.44f,-.60f,-.82f,-.56f,.90f,.08f,-.00f,.06f,.17f,-.42f,.34f,.80f,-.92f,.02f,.44f,-.54f,.45f,.69f,-.19f,-.06f,.32f,-.38f,.72f,.96f,.47f,-.56f,-.50f,.09f,-.60f,-.92f,.10f,-.34f,.16f,.74f,.03f,-.54f,.63f,.60f,.69f,.20f,-.43f,.31f,-.68f,-.64f,-.46f,.61f,-.74f,.56f,.11f,-.44f,-.02f,-.38f,-.96f,.95f,.38f,.24f,-.74f,.91f,-.54f,-.61f,.67f,.38f,.91f,.05f,-.53f,-.08f,.04f,-.86f,-.90f,-.92f,.08f,-.49f,.22f,.92f,.76f,-.30f,.85f,-.76f,.15f,.28f,-.52f,-.16f,.84f,.48f,.70f,.62f,-.52f,.37f,-.43f,.52f,-.87f,.91f,.13f,.15f,.10f,.62f,.72f,-.69f,-.53f,-.09f,-.37f,.76f,-.76f,-.50f,.86f,-.17f,-.73f,-.39f,-.05f,.91f,-.26f,.39f,-.69f,.26f,.97f,-.23f,.61f,.77f,-.28f,.17f,.32f,.85f,-.80f,-.85f,-.03f,-.59f,-.91f,.28f,-.14f,.46f,-.53f,-.75f,-.84f,.52f,-.35f,-.97f,.94f,-.44f,-.26f,.54f,-.51f,.41f,.41f,.89f,-.54f,-.27f,.72f,-.34f,.01f,.19f,.23f,-.30f,-.61f,-.39f,-.11f,.45f,-.22f,-.66f,.06f,.15f,.46f,-.00f,.91f,.25f,.37f,-.13f,.26f,.08f,.78f,-.77f,-.68f,.21f,-.80f,.02f,.33f,.65f,.33f,.02f,-.97f,.01f,-.47f,-.48f,.46f,-.70f,.47f,.01f,-.98f,.49f,-.12f,-.62f,-.45f,.57f,-.05f,.91f,.29f,-.85f,-.14f,-.51f,-.31f,.48f,.81f,-.11f,.10f,.72f,.27f,-.97f,.24f,-.18f,-.09f,-.77f,-.70f,.98f,.50f,.58f,.10f,-.19f,.12f,.05f,.77f,.17f,-.38f,-.17f,.15f,-.18f,.45f,.10f,.86f,.23f,-.74f,-.18f,.41f,.30f,-.75f,-.25f,-.29f,.86f,-.21f,.34f,-.62f,.89f,-.04f,-.34f,-.04f,.38f,-.59f,.02f,.52f,.84f,.14f,.48f,-.89f,-.86f,.12f,-.07f,-.21f,-.94f,-.36f,-.49f,.26f,.70f,.45f,.16f,-.98f,-.85f,-.38f,-.01f,-.31f,-.23f,-.16f,.93f,-.68f,.12f,-.92f,-.55f,-.15f,-.68f,-.59f,-.57f,.79f,-.21f,.17f,-.14f,-.30f,-.35f,.07f,-.15f,-.25f,-.04f,.01f,.28f,-.54f,.85f,-.59f,-.69f,-.18f,.42f,-.93f,-.02f,-.69f,.94f,.11f,-.33f,.34f,.25f,-.77f,-.82f,.33f,-.29f,-.68f,-.52f,-.49f,.57f,.47f,-.60f,-.45f,.69f,.82f,-.15f,-.83f,-.07f,-.79f,-.63f,-.42f,-.62f,-.25f,.78f,.69f,.32f,.36f,.73f,.52f,-.74f,-.11f,.04f,-.96f,-.50f,-.47f,-.88f,-.26f,.67f,-.59f,-.33f,.78f,-.59f,.21f,.13f,-.31f,-.19f,.15f,.26f,-.40f,.43f,-.96f,.75f,-.61f,-.04f,.66f,-.10f,.65f,.12f,-.20f,.98f,.99f,-.75f,.51f,.93f,-.27f,.02f,-.85f,-.62f,-.35f,-.04f,.96f,.85f,.89f,-.38f,-.68f,.02f,-.88f,-.67f,.00f,.92f,-.88f,.56f,.46f,-.72f,.66f,.80f,.48f,.19f,.27f,.15f,-.77f,-.45f,-.94f,.11f,.37f,.22f,-.54f,-.27f,-.67f,-.76f,-.15f,-.20f,.93f,-.05f,.27f,.20f,.72f,-.40f,-.72f,-.49f,.81f,.06f,.83f,.09f,.08f,-.65f,-.26f,.74f,-.22f,-.96f,-.62f,-.45f,.60f,-.10f,.17f,.12f,-.17f,-.74f,.08f,.12f,-.64f,.68f,.92f,.00f,-.04f,.31f,-.83f,-.47f,-.55f,-.89f,-.42f,-.96f,-.62f,.19f,-.47f,.19f,-.46f,-.70f,.39f,-.10f,-.82f,.19f,-.06f,-.47f,-.03f,-.46f,.22f,-.76f,.34f,-.80f,.31f,.41f,.17f,.90f,.47f,.85f,-.94f,.31f,-.81f,.24f,.42f,.20f,.15f,-.24f,-.69f,-.87f,-.65f,.09f,-.25f,.36f,-.38f,.04f,-.80f,.54f,.82f,-.95f,.86f,.40f,.70f,-.34f,-.58f,-.75f,-.37f,.39f,-.62f,.05f,-.76f,-.91f,.65f,.38f,.76f,.36f,-.73f,.06f,-.01f,-.05f,.41f,-.66f,.75f,.56f,.55f,-.09f,.82f,.49f,-.35f,.91f,-.54f,.63f,.00f,-.34f,-.30f,-.08f,.90f,.76f,.41f,-.48f,.95f,.61f,.28f,.41f,-.86f,-.79f,.25f,-.48f,.35f,.22f,-.38f,.33f,.81f,-.85f,-.60f,.87f,.77f,.13f,.90f,.46f,.74f,-.66f,.31f,.00f,.87f,.60f,-.04f,.30f,.61f,.93f,.59f,.19f,.26f,-.93f,.24f,.64f,-.72f,.62f,.31f,-.11f,-.24f,.00f,.79f,.98f,-.43f,-.60f,-.10f,.31f,.38f,-.21f,.81f,-.96f,-.87f,-.66f,.92f,.21f,.53f,.22f,-.25f,-.20f,.02f,.93f,.39f,.38f,.71f,-.38f,.73f,.58f,-.81f,.78f,-.92f,-.26f,.39f,.83f,.85f,.90f,-.26f,.12f,-.40f,-.17f,.54f,.85f,-.38f,-.68f,-.77f,.90f,.08f,.26f,.93f,.52f,-.05f,-.19f,-.64f,.96f,.92f,-.90f,.77f,.05f,-.27f,.76f,.63f,-.15f,.14f,.24f,-.37f,-.39f,.62f,-.68f,-.46f,-.21f,-.72f,.67f,-.75f,-.48f,-.85f,-.06f,-.37f,.07f,.63f,.94f,-.28f,-.88f,-.16f,-.20f,-.23f,.45f,-.02f,-.78f,.76f,-.63f,.09f,-.35f,.91f,.41f,-.89f,.67f,.57f,-.95f,.41f,.62f,-.61f,.17f,-.24f,.34f,.11f,-.25f,-.29f,-.22f,.99f,.73f,-.54f,.39f,-.74f,.32f,.39f,-.40f,.18f,.94f,.94f,.47f,.27f,-.61f,-.62f,.78f,.77f,-.91f,.41f,.67f,-.17f,-.16f,.86f,.50f,.71f,.68f,.54f,.68f,.27f,.35f,-.84f,-.04f,-.06f,.14f,.64f,-.61f,-.51f,-.17f,-.22f,-.55f,.68f,.52f,-.38f,-.18f,.21f,.34f,-.29f,-.85f,-.88f,.92f,.44f,-.02f,.59f,-.06f,-.62f,-.11f,-.72f,-.88f,-.91f,-.59f,-.77f,.67f,.19f,.34f,.05f,.77f,.09f,-.08f,-.60f,.03f,.02f,.00f,.98f,.75f,.96f,.05f,-.09f,.25f,-.53f,.26f,.00f,.26f,.77f,.74f,.38f,.27f,.49f,-.85f,.67f,.59f,.45f,.70f,-.21f,.78f,.74f,-.83f,.85f,-.38f,.78f,-.82f,-.70f,-.36f,.21f,.32f,-.66f,-.33f,.50f,.26f,.59f,-.13f,.86f,-.01f,.52f,-.65f,-.02f,-.82f,.04f,.64f,.62f,.94f,.70f,-.55f,-.03f,.38f,-.45f,.70f,-.17f,.76f,.35f,-.91f,-.85f,.72f,-.18f,-.09f,.54f,-.54f,-.19f,-.97f,-.98f,-.60f,.38f,-.36f,.58f,.56f,.29f,.16f,.27f,-.91f,.53f,-.06f,-.42f,.05f,-.05f,.25f,-.46f,.92f,.37f,-.87f,.14f,.98f,-.33f,.07f,.98f,.16f,-.19f,-.22f,-.45f,.61f,-.77f,.59f,.32f,.34f,.85f,.98f,-.56f,-.46f,-.83f,.64f,-.78f,.33f,.38f,.49f,.90f,-.73f,.50f,.51f,-.25f,-.80f,.17f,-.88f,-.30f,.57f,-.48f,-.83f,-.05f,-.27f,-.28f,.80f,-.26f,-.17f,-.61f,.25f,-.47f,-.65f,-.29f,-.43f,-.88f,-.02f,.97f,.04f,.94f,-.47f,.46f,.19f,-.00f,.70f,.36f,.93f,-.47f,-.85f,-.44f,-.26f,-.44f,.41f,-.50f,-.77f,-.98f,.30f,-.75f,-.53f,-.77f,-.89f,.92f,-.35f,.22f,-.14f,-.50f,.41f,.78f,-.89f,.59f,-.62f,-.82f,.50f,.98f,.85f,.84f,.63f,.40f,.04f,.15f,-.70f,.95f,.10f,.25f,-.02f,-.70f,.88f,-.56f,.56f,-.58f,.52f,-.11f,.02f,-.35f,-.62f,-.15f,-.06f,.15f,.45f,-.99f,.57f,-.19f,.11f,.84f,.02f,-.05f,-.53f,.74f,-.85f,-.07f,.33f,.45f,-.45f,-.74f,.07f,.38f,.60f,.36f,.57f,-.27f,-.54f,-.01f,.68f,-.97f,.67f,-.06f,.67f,.01f,.03f,.65f,.18f,-.81f,.23f,.06f,.45f,.42f,-.17f,.76f,-.27f,-.29f,-.17f,.02f,-.42f,-.46f,-.93f,-.17f,-.78f,-.43f,-.04f,-.69f,.29f,.35f,-.91f,-.97f,.09f,.63f,-.20f,.15f,-.63f,-.96f,-.95f,.60f,.91f,.09f,-.70f,.50f,-.07f,.44f,.01f,.75f,.73f,-.73f,.16f,-.80f,.52f,.64f,-.02f,-.17f,.04f,-.50f,.72f,-.06f,.80f,-.50f,-.78f,-.37f,-.16f,-.06f,-.73f,-.68f,-.45f,.43f,-.41f,-.01f,-.75f,.41f,-.94f,.86f,-.33f,.65f,-.82f,-.53f,-.19f,-.21f,.75f,.39f,-.52f,-.82f,.05f,.29f,-.84f,-.03f,-.05f,-.92f,.66f,.14f,.19f,-.98f,.09f,.48f,.96f,.39f,.67f,-.49f,.42f,.30f,.84f,-.46f,-.41f,.92f,-.93f,.51f,-.90f,-.54f,-.32f,.18f,-.83f,-.05f,.78f,-.76f,.76f,-.50f,-.94f,-.58f,.52f,.45f,-.17f,.62f,.48f,.85f,.11f,.74f,.99f,.94f,-.77f,.44f,-.89f,.97f,-.77f,.16f,-.35f,.16f,-.45f,.50f,-.96f,.74f,-.52f,-.64f,.13f,-.21f,-.03f,-.16f,.35f,.53f,-.82f,.34f,.52f,-.59f,-.17f,-.53f,-.88f,.67f,.13f,-.21f,.93f,-.45f,-.11f,-.60f,.84f,.16f,.76f,.01f,.89f,.87f,.04f,.86f,.20f,.20f,.61f,-.21f,.04f,.26f,.91f,-.40f,.00f,-.26f,-.12f,.35f,.21f,-.87f,.81f,-.03f,-.82f,.19f,-.71f,.93f,.39f,-.70f,-.97f,.20f,.92f,.13f,-.81f,.88f,.34f,-.27f,-.46f,.78f,.94f,-.50f,.57f,.06f,-.52f,.14f,.33f,-.34f,-.31f,-.05f,-.73f,.13f,-.63f,.88f,-.44f,-.15f,.11f,.90f,-.36f,.51f,-.38f,-.64f,-.21f,.13f,.89f,.81f,-.69f,.15f,-.67f,-.89f,-.47f,.73f,-.15f,-.97f,-.24f,-.57f,.91f,-.64f,-.30f,.67f,-.55f,-.68f,.98f,-.98f,.96f,-.36f,-.55f,.11f,-.73f,.57f,.73f,-.46f,-.11f,.52f,.48f,.06f,.43f,-.41f,-.43f,.58f,-.69f,.57f,.71f,-.73f,.19f,-.45f,-.24f,.96f,.61f,.66f,-.50f,-.46f,.32f,.43f,.15f,-.93f,.77f,.60f,-.27f,-.37f,.31f,-.69f,.20f,-.22f,.90f,.51f,-.56f,-.50f,-.55f,.64f,-.03f,.70f,.90f,-.83f,-.11f,-.70f,.82f,.62f,.66f,.26f,.24f,-.55f,.72f,.42f,.58f,.60f,-.02f,-.29f,.73f,-.04f,-.45f,.01f,-.04f,-.59f,.86f,-.29f,.82f,.56f,-.76f,-.58f,.77f,.96f,-.28f,.71f,.19f,-.26f,.98f,.32f,-.75f,-.24f,.85f,-.26f,-.73f,.49f,.26f,-.74f,.08f,-.53f,-.58f,.52f,.41f,.06f,.20f,.47f,-.41f,.14f,.00f,.94f,.76f,-.41f,-.90f,.92f,-.29f,.52f,-.94f,.37f,.87f,-.06f,-.44f,-.33f,-.04f,.78f,-.55f,-.23f,.25f,.08f,.41f,-.48f,.05f,-.37f,.39f,-.74f,-.89f,.51f,-.80f,-.70f,-.94f,.32f,-.43f,-.75f,.63f,-.33f,-.67f,-.33f,.18f,.65f,.34f,.92f,.08f,.92f,.04f,-.33f,-.59f,.62f,.85f,-.49f,.39f,-.37f,-.04f,-.11f,.23f,-.41f,-.84f,.71f,-.35f,.88f,.70f,.94f,.19f,.85f,-.35f,.84f,-.99f,-.70f,.57f,-.78f,-.96f,-.55f,.79f,.67f,.09f,-.28f,-.11f,.22f,.92f,.79f,-.70f,-.62f,.56f,-.70f,-.15f,.18f,.65f,.97f,-.87f,-.04f,-.75f,-.65f,.12f,-.09f,.85f,.98f,.58f,.09f,-.06f,.70f,.19f,-.89f,-.53f,-.26f,-.79f,-.07f,.19f,-.68f,.72f,.87f,-.10f,.68f,-.12f,-.44f,.26f,-.30f,.36f,-.98f,-.76f,-.93f,.93f,-.68f,-.39f,-.06f,.87f,-.19f,.60f,.96f,.91f,.05f,-.95f,.13f,.35f,.87f,.67f,-.48f,.26f,.19f,.74f,-.53f,-.54f,.60f,.08f,.50f,.18f,.89f,.89f,-.19f,.10f,.72f,-.13f,-.38f,-.45f,.19f,.33f,-.01f,-.34f,-.28f,-.42f,-.39f,.48f,.11f,-.66f,.22f,-.08f,.71f,.86f,.02f,-.02f,.39f,-.86f,-.51f,.50f,-.42f,.12f,.44f,-.47f,-.43f,.11f,-.92f,-.07f,.83f,-.69f,-.28f,-.13f,.05f,-.49f,.07f,-.34f,.18f,-.42f,.57f,.44f,.57f,.45f,.97f,-.57f,.30f,-.87f,-.90f,-.11f,-.14f,.89f,.74f,.35f,.30f,-.16f,-.17f,.23f,-.63f,-.95f,.79f,-.39f,.38f,.34f,-.69f,-.45f,-.34f,.96f,-.02f,-.77f,.15f,.37f,-.61f,-.83f,.48f,-.57f,.52f,-.71f,-.02f,-.83f,.34f,.29f,-.05f,.73f,-.15f,.89f,-.17f,-.81f,.59f,.11f,.92f,.31f,.71f,-.89f,-.70f,.31f,.75f,-.59f,-.89f,-.81f,-.81f,.23f,.12f,.26f,.56f,.01f,.24f,-.26f,.61f,-.58f,.57f,-.95f,-.46f,.78f,.06f,-.32f,.88f,.84f,-.25f,-.25f,-.32f,.48f,.28f,-.14f,.48f,.29f,.67f,.89f,.97f,.54f,-.46f,.19f,.67f,-.27f,.19f,-.64f,.68f,.30f,.24f,-.61f,-.28f,-.26f,.32f,-.30f,-.72f,.01f,.19f,-.35f,-.56f,-.02f,.17f,.45f,.07f,-.07f,-.75f,.21f,-.19f,.21f,.89f,-.13f,.51f,.70f,-.00f,-.92f,-.64f,.26f,-.13f,.69f,.28f,-.47f,-.78f,-.32f,.01f,.15f,.36f,-.28f,-.35f,.31f,-.47f,.53f,.51f,.23f,.35f,.97f,.23f,-.09f,.81f,-.26f,.08f,-.17f,-.41f,.78f,-.51f,.16f,-.27f,.05f,-.30f,.32f,.51f,.19f,.94f,.60f,.10f,.21f,.49f,-.69f,.23f,-.87f,.64f,-.36f,-.53f,-.63f,-.05f,-.95f,.23f,.85f,-.32f,-.11f,-.89f,-.13f,.31f,.28f,-.69f,-.37f,-.47f,-.79f,.78f,-.90f,.91f,-.86f,.05f,-.86f,-.74f,.19f,.68f,-.56f,.04f,.10f,-.35f,.47f,-.79f,-.28f,.46f,-.11f,.60f,.72f,-.47f,.03f,.63f,.75f,-.84f,.40f,-.25f,-.72f,.63f,-.22f,-.96f,.43f,-.61f,.73f,-.58f,.78f,-.11f,-.30f,-.59f,-.11f,.01f,.44f,.35f,.42f,-.84f,.13f,.63f,.37f,-.45f,.86f,.90f,.08f,-.50f,.89f,.54f,.50f,-.17f,-.89f,-.72f,.60f,-.75f,.75f,-.79f,.60f,.56f,-.67f,.75f,-.70f,-.15f,-.09f,-.27f,.44f,.74f,-.97f,-.72f,.47f,.83f,.60f,-.97f,.32f,.24f,.33f,.65f,.74f,-.10f,.19f,.30f,-.94f,.07f,-.04f,-.47f,.56f,-.94f,.01f,-.65f,-.18f,.41f,-.34f,-.42f,.56f,.40f,-.17f,-.98f,.62f,.27f,-.27f,.25f,.26f,.41f,-.17f,.18f,-.67f,-.17f,-.78f,.52f,.83f,.04f,-.14f,.35f,.73f,.14f,.72f,-.26f,.83f,-.60f,-.61f,.89f,.30f,.07f,.82f,-.79f,-.68f,-.42f,-.40f,-.99f,-.86f,.48f,.44f,.50f,.51f,.16f,-.77f,-.15f,-.70f,.64f,.97f,-.71f,-.54f,.23f,-.23f,.83f,.53f,-.58f,.62f,-.32f,.84f,-.66f,.95f,.76f,.92f,.48f,-.81f,.43f,.08f,-.67f,.46f,.30f,.23f,.01f,.55f,.02f,.81f,.56f,-.34f,-.69f,.05f,-.99f,.96f,-.84f,.34f,.88f,.01f,.44f,-.19f,.28f,.39f,-.70f,-.24f,-.83f,.44f,.09f,-.08f,.42f,.09f,.24f,-.86f,-.21f,-.90f,-.87f,-.91f,.29f,-.61f,-.84f,.36f,-.40f,-.45f,.46f,.05f,-.55f,.24f,.99f,-.20f,.18f,-.16f,.45f,.90f,.38f,-.37f,.24f,.90f,-.55f,.09f,-.53f,-.14f,.66f,-.59f,-.70f,.24f,.17f,.24f,-.65f,.76f,.32f,.49f,-.65f,.55f,-.59f,-.31f,.78f,-.67f,-.29f,-.60f,.31f,.26f,-.02f,.87f,-.31f,.95f,.14f,.76f,-.89f,.26f,.74f,-.41f,.10f,-.93f,-.42f,-.60f,-.96f,-.31f,.90f,-.20f,.68f,-.57f,.86f,-.71f,.68f,.26f,.75f,.79f,.09f,.94f,.20f,.11f,-.27f,-.06f,.23f,.43f,-.44f,-.87f,.56f,-.34f,.37f,-.49f,-.47f,.10f,.53f,.80f,.53f,-.44f,-.92f,.79f,-.89f,.50f,.82f,-.09f,-.14f,-.88f,.16f,-.15f,.81f,-.96f,.99f,.30f,.09f,-.84f,-.61f,.41f,-.68f,.54f,-.86f,-.87f,-.55f,.69f,.67f,-.43f,.09f,.47f,.85f,-.36f,.27f,-.10f,.93f,.34f,-.77f,.45f,-.37f,.71f,.82f,-.98f,.42f,-.07f,.50f,.66f,.62f,.13f,-.53f,-.83f,-.38f,-.03f,-.49f,-.10f,.21f,.89f,-.90f,.55f,-.62f,.42f,.45f,-.17f,-.04f,-.22f,-.20f,-.10f,.42f,.97f,-.08f,-.58f,-.25f,.02f,-.08f,-.38f,.89f,-.65f,.46f,-.35f,-.54f,.92f,.03f,.58f,-.00f,-.40f,-.55f,.99f,-.46f,-.17f,.17f,-.05f,-.28f,.57f,-.15f,-.41f,.24f,.64f,.55f,-.43f,-.52f,.79f,.59f,.23f,-.41f,-.87f,.82f,-.26f,-.10f,-.14f,-.93f,-.03f,-.51f,.17f,-.53f,-.27f,-.64f,.28f,-.98f,.03f,-.06f,.22f,.65f,.93f,.18f,-.60f,.74f,.13f,.79f,-.20f,-.12f,.08f,-.93f,-.82f,.61f,-.68f,.17f,-.66f,.23f,-.62f,-.23f,.65f,.42f,.29f,-.46f,.97f,-.42f,.85f,-.18f,-.68f,.13f,-.50f,.83f,.59f,-.44f,-.14f,.74f,.46f,-.27f,.38f,-.06f,-.34f,-.52f,.66f,.95f,.69f,-.09f,-.37f,-.35f,.57f,.19f,-.65f,-.70f,.43f,-.01f,.24f,.83f,.28f,.37f,-.18f,-.50f,.10f,.80f,.04f,.11f,.02f,-.44f,.30f,.91f,-.42f,.04f,-.48f,-.25f,-.79f,-.20f,.33f,.22f,-.55f,-.73f,.25f,.53f,.57f,.40f,.53f,-.69f,.86f,.78f,.90f,.79f,-.52f,-.90f,-.16f,.88f,.40f,.75f,.53f,-.91f,-.39f,-.79f,-.26f,.77f,-.56f,-.82f,-.17f,.94f,-.86f,.57f,-.61f,-.86f,.11f,.90f,-.50f,-.38f,.06f,.46f,.60f,-.81f,-.81f,.64f,.13f,.41f,-.12f,-.44f,-.66f,-.86f,-.08f,.16f,.18f,.45f,.84f,-.18f,-.09f,.78f,.67f,.86f,.08f,-.06f,.77f,-.88f,.45f,-.77f,-.06f,-.48f,.81f,-.48f,-.01f,.75f,.43f,-.63f,.65f,-.52f,-.72f,-.56f,-.05f,-.63f,.08f,-.34f,-.72f,-.70f,.84f,-.84f,-.64f,-.80f,.40f,.19f,.07f,.49f,.62f,.17f,-.79f,.62f,-.59f,-.82f,-.40f,-.07f,-.35f,.42f,-.23f,-.92f,.70f,.12f,.93f,.58f,.94f,-.67f,-.13f,-.96f,-.04f,-.57f,.67f,.36f,-.03f,.51f,-.87f,.58f,.85f,.54f,-.10f,-.98f,.64f,-.33f,.70f,.94f,-.53f,-.44f,-.39f,.17f,.34f,.22f,.50f,.63f,-.61f,.15f,-.82f,-.20f,-.79f,-.24f,.58f,-.48f,-.76f,.39f,-.85f,-.52f,-.90f,.00f,.06f,.40f,-.89f,.17f,-.30f,-.79f,-.86f,.44f,.55f,.49f,.54f,.12f,.72f,.17f,.65f,-.74f,-.33f,-.82f,.30f,.24f,-.39f,.47f,.79f,-.31f,.97f,-.17f,.57f,-.63f,.23f,.43f,-.55f,.61f,-.88f,.51f,-.93f,.26f,.34f,-.32f,.22f,-.38f,-.93f,.76f,-.90f,-.02f,.08f,-.07f,-.71f,.61f,-.80f,.81f,-.26f,-.82f,-.18f,.98f,.57f,.17f,-.93f,.67f,-.80f,-.07f,-.43f,.66f,.17f,-.26f,.35f,.15f,.83f,.74f,-.88f,.01f,.81f,-.90f,-.75f,.59f,.00f,-.42f,-.11f,-.54f,-.99f,.49f,.46f,.45f,.27f,.57f,-.88f,.54f,-.59f,.66f,-.71f,-.79f,-.51f,.73f,.53f,-.00f,.52f,-.84f,-.39f,.58f,.17f,-.29f,-.36f,-.57f,-.25f,.36f,.58f,-.06f,-.71f,-.35f,-.87f,-.38f,-.60f,-.75f,.30f,.97f,.55f,.31f,.61f,.59f,-.24f,-.13f,-.91f,-.07f,-.17f,-.82f,-.40f,.68f,.78f,.96f,-.70f,.19f,-.98f,.83f,.77f,-.17f,.26f,-.98f,.62f,-.43f,.74f,.50f,-.36f,-.78f,-.34f,.42f,.62f,.43f,-.17f,.83f,.25f,.51f,.07f,-.40f,.05f,-.69f,.44f,-.88f,.12f,.53f,.24f,.16f,.08f,-.97f,.00f,-.58f,.19f,.59f,.32f,-.46f,.65f,-.27f,-.30f,.63f,-.85f,-.99f,.76f,-.62f,.90f,.68f,-.98f,.40f,.15f,-.34f,-.03f,-.64f,-.03f,.99f,.14f,-.48f,-.91f,-.29f,.43f,-.40f,.36f,.15f,.73f,.00f,-.39f,-.55f,.15f,.12f,.48f,.47f,.47f,-.25f,-.45f,.68f,.48f,-.92f,-.23f,.36f,.78f,-.20f,-.15f,-.14f,.82f,-.49f,-.91f,-.92f,-.92f,-.25f,-.22f,.77f,.44f,-.86f,.16f,.76f,-.06f,.73f,-.94f,-.99f,-.14f,.52f,-.63f,-.18f,.77f,-.70f,-.92f,.36f,.06f,.57f,-.96f,.10f,.12f,-.64f,.38f,-.75f,-.09f,-.81f,-.17f,-.82f,.39f,.16f,-.49f,-.82f,-.95f,.16f,.20f,.12f,.40f,.26f,.94f,.55f,-.83f,.06f,-.04f,.42f,-.97f,.49f,-.15f,-.18f,-.40f,.43f,-.56f,.33f,.29f,-.94f,-.74f,-.20f,.47f,-.08f,-.95f,.08f,-.32f,.75f,-.67f,.36f,-.29f,-.41f,.85f,-.15f,.09f,.26f,-.40f,-.64f,-.43f,-.45f,.87f,-.62f,-.96f,.26f,.87f,-.36f,-.69f,-.93f,-.32f,.09f,.94f,-.85f,.31f,-.59f,-.08f,.10f,.33f,-.97f,-.54f,-.50f,.25f,.26f,.67f,.30f,-.98f,.67f,-.92f,-.56f,-.80f,-.53f,-.34f,-.68f,-.28f,.59f,-.56f,.94f,-.10f,-.26f,-.93f,.54f,.13f,.23f,-.81f,.36f,-.26f,.90f,.53f,-.43f,.68f,.28f,.31f,.25f,.45f,-.21f,.67f,-.56f,-.63f,.05f,.39f,.81f,-.37f,-.92f,.53f,.40f,-.31f,.23f,-.00f,-.83f,.46f,-.54f,.98f,.09f,-.54f,-.89f,.82f,-.62f,-.09f,.02f,-.24f,.42f,-.85f,.15f,-.59f,-.73f,.73f,-.19f,-.58f,-.45f,.98f,-.12f,-.24f,-.49f,-.49f,.46f,.59f,-.74f,.79f,.22f,-.11f,-.64f,-.68f,.31f,.28f,.23f,.77f,-.36f,.30f,.68f,-.51f,-.52f,.59f,.06f,.29f,.69f,-.11f,-.35f,.67f,-.18f,-.59f,-.77f,-.09f,.30f,.78f,-.36f,.06f,-.08f,-.25f,-.69f,.82f,.64f,-.41f,.40f,-.34f,.01f,-.26f,.40f,-.73f,.84f,.39f,-.42f,-.31f,-.28f,-.96f,-.75f,-.64f,.23f,-.76f,-.42f,.92f,.52f,.92f,-.58f,.69f,-.15f,.24f,-.27f,-.86f,-.09f,-.42f,-.74f,-.01f,-.18f,-.41f,.64f,.92f,-.58f,.66f,-.59f,-.64f,.20f,-.80f,-.58f,-.44f,-.34f,-.11f,-.07f,.18f,.31f,-.05f,.90f,-.10f,-.59f,.33f,.89f,-.78f,.66f,.59f,.76f,.52f,.04f,.37f,.79f,.40f,-.97f,-.41f,.04f,.10f,.72f,-.20f,-.95f,.69f,.26f,-.56f,.88f,-.09f,-.73f,-.61f,-.88f,-.53f,-.61f,-.52f,-.29f,.96f,-.61f,-.63f,.18f,.04f,-.31f,.47f,-.72f,.57f,-.69f,-.30f,.32f,.38f,-.31f,-.05f,.83f,-.17f,.40f,.33f,-.27f,.09f,.85f,-.99f,-.96f,-.57f,-.56f,-.73f,.69f,.42f,-.89f,-.24f,.99f,-.18f,.91f,-.66f,-.63f,-.24f,-.24f,-.63f,.80f,.96f,.94f,.59f,-.76f,.26f,.74f,-.89f,.50f,.16f,.21f,-.48f,.32f,-.33f,.13f,-.61f,-.01f,.63f,-.62f,.45f,-.03f,-.25f,.33f,.09f,-.96f,.00f,-.63f,.41f,.48f,.90f,-.61f,.98f,-.02f,-.45f,-.28f,.14f,-.22f,-.68f,-.46f,.64f,-.95f,-.31f,-.24f,.88f,.34f,-.34f,-.42f,.56f,-.40f,-.27f,-.09f,-.78f,.84f,.25f,.34f,-.74f,.46f,-.95f,.05f,-.72f,-.15f,.12f,.49f,-.74f,.91f,.65f,.89f,-.33f,-.57f,-.49f,-.19f,-.08f,-.36f,.74f,-.48f,.07f,.09f,-.48f,-.16f,.65f,-.29f,-.13f,.04f,.37f,-.66f,.06f,-.38f,.10f,.07f,.92f,-.41f,-.17f,.99f,.36f,-.38f,.21f,.66f,-.25f,-.32f,-.31f,.85f,-.47f,-.38f,-.40f,-.10f,.08f,.27f,-.50f,-.31f,.96f,.51f,-.83f,.63f,.65f,-.70f,.09f,-.48f,.48f,.22f,.72f,-.90f,-.06f,.19f,-.16f,.41f,-.69f,-.16f,-.71f,.78f,-.83f,-.76f,-.20f,-.81f,.64f,-.49f,.08f,-.15f,-.33f,.77f,.51f,-.24f,-.08f,.82f,-.58f,.64f,.49f,-.18f,.43f,-.54f,-.85f,-.24f,-.71f,-.90f,.69f,-.73f,.72f,.47f,-.42f,-.75f,-.51f,.80f,.18f,-.08f,-.41f,.61f,-.31f,-.39f,.97f,-.32f,.51f,-.61f,-.35f,.10f,-.08f,-.28f,-.04f,.94f,.71f,.61f,-.13f,.21f,.96f,-.61f,.85f,-.55f,.96f,-.18f,-.05f,-.85f,-.60f,.92f,-.88f,-.71f,-.40f,.14f,.69f,-.21f,.63f,.86f,.44f,-.25f,-.23f,-.05f,-.29f,.89f,-.53f,-.06f,.03f,.36f,-.83f,.79f,-.40f,-.10f,.90f,-.92f,-.49f,-.38f,-.43f,.02f,.98f,-.99f,.13f,-.75f,-.99f,-.37f,.52f,-.20f,-.56f,-.38f,.80f,-.45f,.13f,.71f,-.60f,-.46f,-.90f,.83f,-.01f,-.47f,.08f,-.52f,-.27f,-.83f,-.06f,-.26f,-.43f,.02f,.71f,.26f,-.39f,-.20f,-.12f,.95f,.15f,.71f,-.82f,-.91f,-.98f,.99f,.60f,-.20f,-.89f,-.33f,-.22f,.25f,.04f,.74f,-.92f,-.19f,-.23f,.84f,.34f,.83f,.41f,.77f,.24f,.32f,-.31f,-.36f,-.08f,.58f,-.45f,.63f,-.28f,.15f,-.19f,.58f,.67f,-.21f,-.79f,.23f,-.33f,-.05f,-.60f,-.09f,-.98f,.73f,-.36f,.18f,.70f,.86f,-.18f,-.36f,.06f,-.30f,-.48f,-.08f,.70f,.57f,.00f,.95f,.69f,-.70f,-.98f,-.30f,-.78f,-.63f,-.91f,.83f,.72f,.45f,-.50f,-.30f,-.41f,-.17f,-.39f,-.65f,-.97f,.25f,-.03f,-.38f,-.24f,.42f,.27f,-.18f,.50f,-.04f,-.33f,-.54f,.69f,-.97f,.45f,.03f,.78f,.83f,-.91f,-.37f,-.55f,-.28f,-.15f,-.19f,.38f,.11f,-.39f,-.63f,.84f,-.19f,.61f,-.36f,.34f,-.54f,-.68f,-.68f,.47f,-.08f,.79f,.21f,-.50f,-.18f,-.40f,.80f,.00f,-.40f,-.90f,-.52f,-.21f,-.10f,.99f,.97f,-.71f,-.98f,-.80f,.20f,.73f,.95f,-.47f,.52f,.98f,.01f,.16f,-.50f,.43f,.51f,.81f,.69f,-.34f,-.49f,-.92f,.68f,.79f,-.56f,-.46f,.69f,.51f,-.45f,-.30f,.95f,-.46f,-.88f,-.92f,-.69f,.91f,-.62f,-.58f,.70f,.51f,-.70f,-.87f,-.01f,-.64f,-.22f,-.81f,-.65f,-.55f,-.06f,.84f,.54f,.29f,-.85f,-.99f,.87f,.98f,.61f,.64f,.25f,.87f,.89f,.48f,-.25f,-.78f,-.77f,.49f,.53f,-.06f,-.93f,.21f,.92f,-.51f,.55f,.83f,.14f,.17f,.42f,-.09f,.52f,.95f,.46f,.81f,-.75f,.88f,.99f,-.53f,-.93f,-.46f,-.67f,.61f,-.02f,-.57f,-.58f,.09f,.29f,.27f,.73f,.90f,-.72f,-.68f,-.00f,.64f,.24f,.18f,.73f,.03f,-.06f,-.84f,-.07f,-.25f,-.61f,.71f,-.10f,-.27f,-.83f,.24f,-.11f,-.97f,-.99f,.13f,-.84f,-.51f,.85f,-.11f,-.67f,.45f,.95f,.76f,-.63f,.95f,-.68f,-.15f,.77f,-.44f,.10f,.11f,.76f,.56f,.48f,.35f,-.11f,-.16f,-.00f,.63f,-.01f,.51f,.13f,-.28f,-.01f,.58f,.89f,-.69f,.43f,-.90f,.28f,.86f,.93f,.22f,-.16f,.47f,.33f,-.56f,.12f,-.26f,-.72f,-.51f,.24f,-.19f,-.88f,-.70f,.45f,-.93f,-.61f,-.76f,-.49f,.27f,-.11f,-.06f,.10f,.04f,.62f,.51f,-.27f,-.19f,.90f,-.39f,.18f,-.10f,.84f,.53f,-.71f,.17f,.29f,.72f,-.80f,.27f,.57f,-.53f,-.95f,.88f,.84f,.09f,.11f,-.13f,-.64f,-.36f,-.48f,.82f,.08f,-.35f,.03f,.76f,.14f,.79f,-.94f,-.35f,.89f,-.25f,-.48f,.37f,.82f,-.30f,.57f,.94f,.47f,.17f,-.39f,-.62f,.65f,.37f,.41f,-.85f,.74f,-.59f,.44f,-.63f,.45f,.90f,.99f,-.73f,-.03f,.00f,.49f,-.29f,-.63f,.59f,.24f,.66f,-.05f,.06f,-.95f,.06f,.62f,-.83f,-.46f,-.25f,-.35f,-.91f,.07f,-.77f,.54f,-.82f,.82f,.68f,.49f,-.63f,-.36f,-.31f,-.48f,-.95f,-.55f,-.62f,.52f,.26f,-.39f,.45f,-.09f,-.07f,.56f,.32f,.37f,.57f,-.83f,-.90f,.03f,.47f,.97f,.26f,.37f,-.24f,.72f,.38f,-.73f,.86f,.88f,-.59f,.17f,-.80f,-.86f,.64f,.83f,-.15f,-.31f,-.87f,.92f,-.24f,.45f,.42f,-.61f,-.37f,.02f,.82f,.84f,.06f,.67f,-.12f,.11f,.94f,-.81f,.75f,-.04f,.72f,-.50f,.30f,.90f,.93f,-.05f,-.10f,-.02f,.51f,.03f,-.67f,.18f,.37f,.65f,-.20f,-.45f,.52f,.46f,.95f,-.78f,.20f,.54f,.92f,-.64f,.51f,.63f,-.58f,-.50f,-.40f,.82f,.15f,.15f,.21f,.01f,.89f,.10f,-.05f,-.92f,-.18f,.30f,.00f,-.12f,.71f,-.58f,.15f,.17f,-.31f,-.36f,.95f,-.21f,-.82f,-.31f,-.78f,-.17f,.47f,-.41f,.90f,.75f,.28f,.57f,.16f,-.73f,.66f,-.27f,.76f,-.74f,.35f,-.21f,.39f,-.70f,-.31f,-.14f,-.85f,.80f,.58f,-.43f,.02f,.01f,.31f,-.89f,-.06f,.04f,.28f,-.00f,.74f,.32f,.66f,-.98f,-.53f,-.40f,-.94f,.35f,.53f,-.57f,.60f,.86f,-.94f,-.67f,.95f,-.35f,-.79f,.56f,.10f,-.28f,-.53f,-.27f,.25f,.83f,.77f,.18f,-.33f,.49f,-.64f,.57f,.80f,.98f,.56f,.03f,.58f,.76f,.87f,.61f,.91f,-.30f,-.60f,-.76f,-.30f,.23f,.06f,.03f,.72f,.11f,-.23f,-.21f,-.20f,-.44f,.78f,.15f,.87f,-.31f,.43f,-.05f,-.16f,.04f,-.79f,.87f,.09f,.32f,-.58f,.65f,-.62f,.23f,.20f,-.29f,.88f,-.58f,-.03f,-.33f,-.48f,.27f,.47f,.81f,.91f,-.41f,.67f,-.94f,.31f,.25f,-.00f,.99f,-.06f,.22f,-.19f,.53f,-.62f,-.63f,.28f,-.49f,.37f,-.69f,.59f,-.30f,.57f,.52f,.55f,-.67f,-.15f,.71f,.11f,-.34f,-.06f,.10f,.70f,.98f,.52f,.16f,.87f,-.62f,-.58f,.57f,.65f,-.35f,-.31f,.96f,.21f,.31f,-.82f,.14f,.57f,.24f,-.26f,.77f,-.09f,.20f,.40f,.12f,-.17f,.65f,.64f,-.63f,-.75f,.50f,.31f,-.06f,-.19f,-.63f,.69f,-.54f,.27f,.95f,-.44f,.18f,-.91f,-.02f,.77f,-.66f,-.33f,-.20f,-.22f,.65f,.31f,-.00f,-.26f,.07f,.05f,-.47f,-.52f,-.06f,.49f,.65f,.54f,.10f,-.10f,-.56f,-.87f,.35f,-.51f,.90f,-.04f,-.90f,-.16f,-.75f,-.06f,-.04f,-.11f,.44f,.73f,.72f,.80f,.73f,-.95f,.83f,.72f,.46f,-.94f,-.56f,.00f,-.41f,-.66f,.56f,.92f,-.23f,-.32f,-.16f,-.04f,-.37f,-.81f,-.29f,-.79f,-.45f,-.37f,.65f,-.07f,-.97f,-.93f,.15f,.01f,-.90f,.05f,-.63f,.43f,-.67f,.04f,.86f,-.64f,.80f,.66f,-.02f,.84f,-.46f,.19f,-.42f,-.20f,.86f,.12f,.32f,-.72f,-.40f,-.32f,-.26f,.85f,-.40f,.15f,-.69f,-.77f,-.48f,-.10f,-.16f,.61f,-.12f,-.76f,.21f,-.15f,-.15f,.00f,-.31f,.74f,-.12f,.75f,-.31f,.89f,.75f,-.17f,-.58f,-.33f,-.72f,-.43f,-.36f,-.74f,-.17f,-.61f,-.53f,.62f,-.24f,-.37f,-.16f,-.74f,.39f,-.41f,.57f,-.64f,-.51f,.65f,.40f,.65f,-.04f,-.08f,.30f,-.86f,.37f,-.49f,.24f,.36f,-.66f,.53f,-.42f,.47f,-.10f,-.41f,-.92f,-.88f,.90f,.59f,.00f,.70f,-.55f,.45f,.81f,.42f,-.81f,-.90f,-.66f,.47f,-.49f,-.21f,-.71f,-.62f,.20f,-.90f,-.49f,-.83f,-.94f,.16f,.04f,.84f,-.49f,-.23f,-.75f,-.13f,.91f,.36f,-.51f,.56f,-.26f,-.34f,-.27f,.13f,.45f,-.89f,.69f,-.42f,.49f,-.93f,.46f,.68f,.63f,-.86f,-.26f,.14f,-.29f,.80f,-.74f,.73f,.19f,-.26f,.01f,-.71f,-.54f,-.64f,-.59f,-.37f,.76f,.15f,.90f,-.28f,.59f,.16f,-.55f,.48f,-.49f,-.43f,.56f,-.30f,.12f,.08f,-.28f,.78f,.73f,-.06f,.13f,.31f,-.30f,.90f,.39f,-.77f,-.89f,.91f,-.09f,.76f,.30f,-.80f,.31f,-.94f,.88f,-.38f,-.10f,.02f,-.93f,-.27f,.83f,-.33f,-.55f,.04f,-.31f,-.21f,-.38f,-.04f,-.21f,-.58f,.57f,-.50f,.45f,-.88f,-.95f,.66f,-.15f,-.65f,.04f,-.91f,-.02f,-.40f,.60f,.17f,.07f,-.11f,-.55f,.36f,.47f,.09f,.37f,.75f,-.98f,-.39f,.79f,.61f,-.75f,.51f,-.49f,-.99f,-.72f,-.56f,.97f,-.42f,.06f,-.14f,-.96f,-.28f,-.48f,.99f,.64f,-.12f,.63f,-.95f,-.09f,-.81f,.83f,.99f,-.01f,.17f,.03f,-.52f,.20f,.77f,-.42f,.96f,.79f,-.64f,-.29f,-.84f,-.79f,.83f,-.88f,-.24f,.22f,.55f,.16f,-.85f,.49f,.61f,-.79f,-.75f,.27f,.36f,.36f,-.98f,-.34f,.13f,.38f,-.25f,.74f,.08f,.46f,.13f,.32f,.78f,.44f,.11f,-.57f,-.34f,-.92f,-.03f,-.74f,-.06f,.30f,-.51f,.06f,.74f,.13f,.34f,.05f,.81f,.43f,.80f,-.77f,-.54f,.59f,-.05f,.14f,.29f,-.70f,-.12f,.03f,-.04f,-.07f,-.84f,.86f,.87f,-.24f,.02f,.71f,.31f,-.62f,.62f,-.65f,.26f,.47f,.57f,.84f,.32f,.54f,-.50f,.93f,-.91f,.35f,-.74f,-.56f,-.46f,-.78f,-.60f,-.89f,-.88f,-.16f,-.83f,-.63f,.59f,.85f,.32f,.74f,.84f,-.83f,-.46f,-.77f,-.74f,.81f,-.95f,.99f,-.87f,-.28f,.66f,-.58f,.20f,-.17f,-.95f,.23f,.86f,-.81f,.93f,.38f,.49f,.68f,-.91f,-.58f,-.41f,.31f,.23f,.92f,.71f,.33f,.18f,-.33f,.64f,.75f,-.72f,.70f,-.49f,-.49f,-.18f,-.59f,-.23f,.71f,.42f,-.53f,-.83f,-.04f,-.75f,-.77f,-.59f,.53f,-.30f,.22f,-.78f,.64f,-.64f,-.37f,.79f,.72f,-.06f,.94f,.93f,-.75f,.90f,-.79f,.96f,.28f,-.74f,.70f,-.81f,.18f,-.96f,.84f,-.18f,.05f,-.23f,.26f,-.85f,-.12f,.50f,.12f,-.94f,.64f,.86f,.81f,.89f,-.62f,.74f,-.19f,-.78f,.67f,-.04f,.36f,.62f,-.02f,-.49f,.72f,-.02f,.42f,-.78f,.79f,.26f,-.63f,-.31f,-.38f,-.37f,-.43f,-.97f,.27f,-.34f,-.59f,-.07f,.44f,-.62f,-.64f,-.09f,-.70f,-.11f,-.89f,-.35f,.49f,-.73f,-.89f,.76f,.90f,.04f,.44f,.58f,.18f,-.06f,.68f,.14f,-.48f,-.15f,-.53f,-.31f,.37f,.71f,-.10f,-.83f,-.88f,-.26f,-.88f,.82f,-.04f,.84f,-.92f,-.10f,.00f,.18f,.21f,.74f,-.25f,.88f,-.75f,-.47f,.72f,-.54f,.04f,.10f,.29f,.20f,.08f,-.17f,.71f,-.46f,.96f,-.21f,.10f,.58f,.65f,.06f,.58f,.68f,.27f,-.07f,-.05f,.97f,.25f,-.42f,-.59f,.42f,.87f,-.06f,-.61f,.06f,.84f,.58f,-.68f,.65f,.40f,-.32f,-.38f,.93f,.58f,-.65f,.73f,.96f,-.97f,.08f,.80f,.39f,-.21f,-.47f,.56f,-.17f,-.46f,-.52f,-.27f,.76f,.60f,-.41f,-.73f,.93f,-.06f,.30f,-.64f,-.02f,-.23f,.08f,.72f,-.58f,-.29f,-.49f,.85f,-.42f,.40f,-.37f,-.74f,.51f,-.52f,-.52f,-.63f,-.66f,.81f,-.39f,.33f,-.46f,-.76f,-.06f,.45f,-.05f,-.64f,.42f,.76f,-.55f,-.98f,.78f,-.82f,-.25f,.24f,.35f,.68f,-.68f,.28f,.60f,-.50f,.20f,.15f,-.92f,.09f,-.99f,.49f,.24f,.69f,-.50f,.47f,.94f,-.30f,.96f,-.45f,-.57f,.25f,.77f,.82f,-.88f,-.33f,-.32f,-.65f,-.85f,.83f,.06f,.71f,.53f,-.42f,.25f,-.50f,-.02f,.76f,-.13f,.39f,-.99f,-.20f,.20f,-.80f,-.59f,-.59f,.13f,.57f,.55f,.44f,-.11f,.30f,-.16f,.78f,-.91f,.02f,.68f,.59f,-.28f,-.79f,.48f,-.17f,-.86f,-.68f,.96f,-.61f,-.69f,.95f,.81f,-.89f,.02f,.97f,-.13f,-.10f,.07f,.67f,.30f,-.26f,-.07f,-.55f,-.58f,-.81f,.06f,.65f,.97f,-.29f,.85f,-.49f,-.01f,-.90f,.23f,-.39f,-.71f,.28f,.38f,-.26f,.19f,-.94f,.38f,-.80f,.15f,-.59f,.18f,.87f,-.54f,-.65f,.47f,.74f,.43f,.30f,.99f,-.66f,.89f,-.72f,.25f,.73f,-.26f,-.60f,.10f,.55f,-.86f,-.96f,-.54f,-.81f,.84f,.67f,.34f,.98f,-.50f,-.64f,-.40f,-.61f,-.72f,-.68f,.80f,-.15f,-.99f,-.79f,.71f,-.66f,-.11f,.06f,.24f,-.58f,.60f,-.21f,.24f,-.75f,-.12f,-.91f,-.67f,-.19f,-.47f,-.87f,.40f,.42f,.69f,-.61f,-.98f,-.63f,.97f,.05f,-.19f,-.47f,.41f,-.45f,.94f,.85f,-.10f,.09f,.00f,.20f,.49f,-.81f,.31f,.94f,-.41f,-.71f,.99f,.82f,-.34f,-.08f,.85f,.61f,-.93f,-.54f,.37f,-.90f,-.67f,-.02f,-.43f,-.52f,-.58f,-.61f,.81f,.53f,.86f,-.09f,.26f,.23f,-.76f,-.58f,-.90f,.33f,.82f,-.44f,.18f,-.18f,.72f,.74f,-.84f,.83f,-.26f,-.32f,-.99f,-.07f,.57f,.71f,-.64f,.56f,-.08f,-.78f,.83f,-.60f,-.54f,-.65f,.26f,.67f,.10f,.72f,-.12f,.71f,.90f,-.37f,-.55f,-.42f,-.89f,.87f,.70f,.47f,.07f,.46f,.48f,.53f,-.95f,.82f,.35f,.48f,.99f,-.64f,.45f,-.33f,.94f,.93f,.55f,.95f,.23f,.71f,.42f,.48f,-.45f,.54f,.38f,.58f,.94f,.82f,.65f,-.51f,.53f,-.47f,-.78f,-.51f,.67f,.72f,-.20f,.09f,.41f,.69f,.21f,.95f,-.26f,.23f,-.04f,-.30f,-.18f,-.78f,.06f,.04f,.83f,-.97f,-.55f,-.86f,-.99f,-.12f,.17f,-.50f,-.77f,-.20f,-.47f,-.95f,.05f,-.37f,.88f,.09f,-.54f,-.64f,-.72f,.57f,.16f,-.31f,-.28f,-.36f,.71f,-.82f,.23f,.88f,.69f,.99f,-.99f,.94f,.24f,-.45f,.61f,.85f,-.92f,.96f,.98f,.34f,.03f,-.25f,.15f,-.71f,.48f,-.02f,-.29f,.34f,.48f,-.14f,-.46f,-.32f,.67f,.04f,-.90f,-.82f,.44f,-.46f,-.33f,.63f,-.46f,.16f,.02f,.57f,-.02f,.50f,.26f,.51f,-.15f,.25f,.07f,-.59f,-.87f,.64f,-.92f,.05f,.79f,.14f,-.34f,-.77f,.72f,.53f,-.42f,.95f,-.86f,-.35f,.90f,.36f,.25f,.63f,.59f,-.32f,-.95f,-.98f,-.84f,-.56f,.65f,-.81f,-.06f,-.45f,-.60f,-.84f,.09f,.77f,.17f,.11f,-.57f,-.67f,-.23f,-.98f,.12f,.32f,.35f,.37f,-.81f,-.07f,-.21f,.72f,-.35f,-.02f,.33f,.58f,-.52f,.17f,-.49f,.30f,-.30f,-.78f,-.38f,.25f,.14f,-.20f,.75f,-.51f,.46f,.15f,-.07f,.19f,-.47f,.70f,-.47f,-.03f,.14f,-.25f,.67f,.83f,.22f,-.80f,.15f,.88f,.42f,.76f,-.53f,.85f,.89f,.92f,-.73f,-.88f,.05f,.09f,.19f,.55f,.74f,-.08f,.41f,.06f,-.35f,.66f,.05f,.36f,.28f,.77f,.65f,.94f,-.28f,-.31f,.47f,-.62f,-.99f,.90f,.70f,.88f,-.68f,.69f,-.23f,-.01f,.97f,.87f,.55f,.07f,.59f,-.16f,-.05f,.28f,-.48f,.56f,-.64f,.22f,.55f,-.66f,.29f,.12f,.73f,-.98f,-.13f,-.39f,.33f,.95f,.38f,-.34f,.73f,.25f,.63f,.13f,-.18f,-.23f,-.68f,.39f,.65f,-.08f,-.96f,.35f,-.06f,.81f,-.34f,-.91f,.96f,-.75f,.61f,.81f,-.21f,.21f,-.89f,-.13f,-.92f,-.11f,-.25f,.28f,.31f,.83f,.96f,-.13f,.54f,.95f,.16f,.49f,.12f,.13f,.20f,.37f,.34f,.87f,-.11f,-.28f,-.05f,.72f,-.27f,.44f,.84f,-.03f,.69f,-.79f,.63f,.81f,.88f,.98f,.51f,-.20f,.54f,.73f,-.95f,-.61f,.33f,.26f,.51f,.46f,.43f,-.04f,-.29f,.77f,.34f,-.43f,.48f,.42f,-.14f,.89f,-.93f,-.40f,.77f,-.04f,.85f,.98f,-.26f,.05f,-.24f,-.22f,-.50f,-.82f,-.42f,.63f,.79f,-.84f,-.60f,.20f,-.41f,.86f,.51f,-.17f,-.20f,.94f,-.89f,-.51f,-.35f,.73f,-.53f,.09f,.49f,-.72f,-.47f,-.72f,-.58f,-.28f,.05f,-.08f,-.86f,-.69f,-.22f,-.32f,.17f,-.58f,-.49f,-.62f,.19f,-.28f,-.40f,-.61f,.75f,-.87f,-.96f,-.47f,-.04f,.64f,.80f,.90f,-.57f,.40f,.34f,-.31f,-.94f,.79f,.12f,.55f,.62f,.44f,.58f,-.10f,-.07f,.35f,.73f,.42f,-.05f,-.16f,.89f,-.05f,-.22f,-.25f,-.81f,-.57f,.78f,.49f,.06f,.87f,.61f,-.01f,.77f,.63f,-.15f,-.17f,-.42f,-.59f,.64f,.85f,-.70f,.03f,-.58f,.25f,.31f,.72f,.26f,-.83f,-.37f,-.10f,-.30f,-.75f,-.54f,.13f,.10f,.75f,-.66f,-.73f,-.41f,.15f,.51f,-.72f,.76f,.98f,-.41f,.61f,-.67f,-.82f,-.90f,.57f,.57f,.82f,.50f,.74f,-.06f,-.19f,-.49f,.71f,.82f,.19f,-.04f,.55f,.03f,.59f,-.65f,.99f,.74f,.81f,-.59f,-.60f,.73f,.47f,.73f,.72f,-.33f,.29f,.66f,.72f,-.21f,-.90f,.69f,.40f,.67f,.07f,-.39f,.34f,.94f,.56f,.61f,.24f,-.76f,.53f,-.15f,-.70f,.93f,.42f,-.12f,-.50f,.67f,-.55f,.73f,.46f,-.74f,.90f,-.68f,.57f,-.62f,-.19f,-.57f,.37f,.81f,-.84f,.73f,.20f,-.89f,-.55f,.01f,-.12f,.24f,-.14f,-.35f,.99f,.40f,.11f,-.83f,-.25f,.84f,.89f,-.51f,.75f,.44f,.45f,-.18f,-.70f,.83f,.02f,-.84f,.46f,-.83f,-.02f,.10f,.44f,.53f,-.96f,.70f,.55f,.37f,.84f,-.64f,-.06f,.94f,-.58f,-.83f,.92f,.11f,.34f,-.67f,-.42f,-.27f,-.55f,-.97f,-.92f,-.81f,-.21f,-.40f,.73f,.49f,.60f,-.56f,.95f,-.09f,-.20f,-.88f,-.09f,-.70f,-.30f,-.85f,-.39f,.94f,.06f,.06f,-.09f,.05f,.61f,.51f,.60f,-.78f,-.89f,.92f,-.03f,-.03f,.41f,-.10f,.97f,.67f,.92f,.70f,-.22f,.63f,.77f,-.46f,.25f,.17f,.00f,-.14f,-.87f,.49f,.99f,.42f,-.67f,.61f,.34f,.25f,-.71f,-.18f,.24f,-.27f,.24f,-.06f,-.05f,-.08f,.70f,-.19f,-.29f,.41f,.78f,.08f,-.88f,.39f,.33f,-.50f,.00f,-.35f,.56f,-.85f,.06f,-.02f,-.91f,-.58f,.18f,.66f,.46f,.96f,.90f,.74f,-.17f,-.80f,-.38f,-.01f,.25f,-.53f,.21f,.26f,-.63f,-.78f,.32f,.42f,-.60f,-.31f,-.93f,.36f,.41f,.47f,-.41f,.03f,.37f,-.93f,-.75f,-.10f,-.44f,.74f,.96f,.47f,.69f,.44f,.07f,.28f,-.84f,-.00f,-.65f,.23f,-.17f,.65f,-.05f,-.32f,-.98f,.67f,-.45f,.16f,.26f,.81f,.81f,-.72f,-.23f,.57f,-.87f,.76f,-.57f,.14f,-.44f,.31f,.80f,-.73f,.83f,.00f,-.54f,.10f,.42f,.54f,-.18f,.56f,.35f,.44f,-.52f,.69f,.69f,.94f,.39f,.96f,.58f,-.71f,.98f,-.92f,-.11f,.98f,.90f,.79f,.63f,.84f,-.95f,.61f,-.52f,-.10f,.33f,.10f,.56f,.74f,.11f,-.27f,-.36f,.88f,.33f,-.30f,.67f,.98f,.19f,.56f,.85f,.13f,-.65f,-.67f,.86f,-.03f,.83f,-.50f,-.54f,-.65f,-.64f,-.73f,-.60f,.68f,-.71f,-.36f,-.81f,-.27f,-.06f,-.04f,.47f,-.99f,-.35f,-.89f,.55f,-.39f,-.94f,.32f,-.23f,-.53f,.37f,-.17f,.85f,.27f,-.10f,-.81f,-.31f,-.22f,-.30f,-.29f,.93f,.08f,.98f,.32f,.97f,.86f,.67f,.39f,.50f,.06f,.25f,-.73f,-.88f,-.02f,.88f,.85f,.38f,-.29f,-.99f,.30f,-.06f,.14f,-.02f,.09f,-.62f,.41f,-.46f,.90f,.06f,-.78f,-.10f,.97f,.66f,-.96f,.46f,.12f,.84f,.89f,-.28f,-.65f,-.71f,.04f,.34f,-.98f,-.27f,.21f,.13f,-.56f,.56f,.71f,-.55f,.14f,.81f,.03f,-.19f,.82f,.48f,-.42f,.73f,.04f,.48f,-.10f,-.78f,.75f,-.89f,-.02f,.60f,.15f,-.88f,.01f,.93f,-.59f,.43f,.57f,.14f,.96f,.75f,.58f,.69f,-.92f,.14f,.94f,-.42f,.54f,-.73f,.91f,-.89f,-.95f,.41f,.26f,-.46f,-.64f,-.65f,.53f,-.61f,-.95f,-.41f,-.28f,-.78f,-.01f,-.55f,.69f,.60f,-.29f,.81f,.74f,.39f,.58f,-.65f,.37f,-.88f,-.11f,.08f,-.78f,.01f,-.46f,-.12f,.38f,-.63f,.66f,-.33f,-.51f,-.16f,.32f,.40f,-.03f,-.35f,.49f,-.52f,-.26f,-.99f,-.94f,.18f,.94f,.37f,.94f,.85f,.36f,-.00f,.36f,-.94f,-.47f,.92f,.75f,-.29f,-.82f,.95f,.29f,-.47f,.12f,-.96f,.06f,-.56f,-.20f,.47f,.15f,.08f,-.20f,-.20f,-.31f,.44f,-.83f,-.03f,.75f,-.87f,.90f,-.39f,-.06f,.56f,.03f,-.03f,-.37f,-.08f,-.25f,.15f,-.90f,.29f,-.94f,.20f,.71f,.14f,-.70f,.98f,-.77f,-.97f,-.43f,.25f,-.30f,.28f,-.04f,.19f,.40f,.56f,-.18f,-.66f,.66f,-.21f,-.48f,.76f,.87f,-.63f,-.12f,.22f,-.48f,-.41f,-.87f,-.16f,-.53f,.25f,-.31f,.45f,-.25f,.46f,.35f,.64f,-.45f,.84f,-.70f,.19f,-.96f,-.68f,.92f,-.07f,.10f,.89f,.25f,.70f,.52f,.63f,-.36f,.11f,-.39f,.77f,.59f,-.01f,-.60f,.66f,-.46f,.64f,-.21f,.37f,-.55f,.27f,.09f,-.35f,.07f,.80f,-.51f,-.80f,-.04f,.59f,.59f,-.96f,-.12f,-.28f,.71f,.17f,-.36f,.35f,.99f,-.30f,-.98f,.52f,-.62f,.69f,-.48f,.34f,.30f,-.78f,-.84f,.46f,-.36f,.31f,-.61f,-.71f,.46f,-.97f,-.97f,-.18f,-.42f,-.22f,-.42f,-.08f,.84f,-.48f,-.22f,-.95f,-.42f,-.45f,.17f,-.58f,.82f,-.81f,.23f,.07f,-.16f,.93f,-.82f,-.78f,.13f,-.80f,-.24f,-.00f,.75f,-.59f,.01f,.71f,.53f,.02f,-.97f,.81f,-.35f,.15f,-.32f,-.11f,.79f,-.08f,.82f,-.99f,-.68f,-.48f,-.08f,-.03f,.64f,.48f,-.69f,-.82f,.98f,.86f,-.32f,-.86f,-.08f,.25f,-.37f,-.56f,.04f,-.53f,.52f,.89f,-.26f,.79f,.97f,.46f,.07f,-.99f,-.91f,-.03f,.81f,.06f,-.56f,-.68f,.14f,-.15f,.33f,-.26f,.94f,.76f,.70f,.70f,-.88f,.20f,.02f,-.32f,.75f,.34f,.47f,.63f,.01f,-.80f,.31f,.01f,-.25f,.20f,.40f,-.95f,.52f,.24f,-.46f,.61f,.58f,-.97f,-.90f,-.24f,.18f,.16f,.56f,.21f,-.71f,-.96f,-.86f,.04f,-.11f,-.09f,.07f,-.01f,-.58f,-.12f,-.43f,-.95f,.45f,-.87f,-.07f,.31f,.02f,-.22f,-.62f,.48f,.19f,.42f,-.23f,.70f,-.56f,-.19f,-.54f,-.15f,.78f,-.39f,-.95f,.36f,.12f,-.31f,.93f,.16f,.92f,.58f,-.80f,.76f,-.16f,.15f,.93f,-.18f,.11f,.97f,.36f,-.29f,.56f,.67f,-.46f,-.94f,.64f,-.87f,-.72f,-.86f,.64f,.34f,-.00f,.03f,-.48f,-.11f,.09f,-.89f,-.85f,.71f,-.36f,-.28f,.04f,-.88f,.91f,.19f,.37f,-.58f,.29f,.17f,.33f,-.62f,-.32f,.83f,.00f,-.95f,-.03f,-.80f,.64f,.10f,-.11f,.46f,.54f,.64f,-.17f,-.31f,-.46f,.85f,-.42f,-.27f,.45f,.06f,-.11f,-.20f,-.30f,.29f,.73f,.43f,-.95f,.46f,-.04f,-.51f,.30f,.82f,-.93f,-.86f,.33f,-.39f,-.07f,-.67f,-.86f,-.61f,.48f,-.35f,-.57f,.31f,.05f,-.72f,.46f,-.62f,-.81f,-.74f,.59f,.79f,.77f,-.31f,-.38f,-.85f,.70f,.39f,.57f,.52f,-.08f,.49f,-.49f,-.53f,.61f,-.21f,-.04f,.03f,.54f,-.49f,.95f,-.45f,-.99f,-.39f,.19f,-.74f,-.43f,.97f,-.46f,-.93f,-.56f,.17f,-.74f,-.87f,-.20f,-.87f,-.60f,-.66f,.12f,-.88f,-.25f,-.27f,.36f,-.22f,.97f,-.24f,.35f,-.30f,-.10f,.66f,-.06f,-.21f,-.20f,.87f,-.13f,-.25f,.10f,-.47f,-.51f,.55f,.20f,.14f,.80f,-.25f,.81f,-.19f,-.04f,-.05f,.63f,-.03f,-.60f,.39f,-.25f,.47f,-.51f,-.42f,-.31f,-.08f,.76f,-.64f,.28f,-.15f,.89f,-.39f,.49f,-.31f,.39f,-.48f,-.24f,-.97f,.76f,-.34f,-.35f,-.83f,-.70f,-.92f,.37f,-.05f,.95f,.49f,-.04f,.51f,-.84f,.56f,.50f,.85f,.78f,-.03f,-.32f,.31f,-.13f,-.38f,-.34f,-.15f,.22f,-.91f,.97f,.12f,.71f,.22f,.98f,.45f,-.21f,-.27f,.31f,-.47f,-.90f,-.90f,-.83f,.07f,.20f,.68f,-.42f,-.36f,-.14f,.29f,.57f,.76f,-.18f,-.89f,-.30f,.92f,-.09f,-.05f,.96f,.29f,.07f,-.65f,-.78f,.55f,.17f,-.39f,.17f,.62f,.51f,.00f,-.61f,-.52f,-.57f,-.99f,-.65f,.94f,-.54f,-.00f,.99f,.40f,-.98f,.38f,-.15f,-.91f,-.47f,.61f,-.67f,.79f,-.01f,-.04f,.27f,.27f,.49f,.71f,.56f,-.98f,.42f,-.30f,-.48f,.85f,-.28f,.02f,-.26f,.27f,.40f,-.61f,-.48f,.79f,.08f,-.04f,.96f,.86f,.99f,-.97f,.79f,-.96f,.53f,.61f,-.16f,.39f,.02f,.76f,-.51f,.19f,.79f,.20f,.76f,.26f,-.95f,.03f,.06f,.86f,-.69f,.46f,-.55f,-.71f,-.91f,.69f,.53f,-.27f,-.02f,-.34f,-.78f,.51f,.04f,.92f,-.72f,-.84f,.01f,-.95f,.58f,-.85f,.86f,-.70f,.34f,.14f,-.22f,.89f,.44f,.91f,.51f,.15f,.56f,.75f,-.10f,.34f,.14f,-.34f,-.55f,.77f,.59f,.11f,-.16f,-.12f,-.89f,-.45f,-.56f,-.93f,.51f,-.19f,.82f,.17f,.65f,.18f,.69f,-.72f,.75f,.40f,-.37f,-.55f,.87f,.58f,.33f,-.78f,.29f,-.17f,.62f,-.05f,.39f,.69f,.90f,-.53f,.98f,.74f,-.34f,-.34f,-.25f,.62f,.96f,-.30f,-.54f,.46f,-.99f,.13f,-.40f,.15f,-.59f,-.85f,.18f,.89f,-.85f,.56f,-.23f,.73f,.17f,-.76f,-.23f,.77f,.82f,-.72f,-.94f,-.95f,.28f,.74f,-.97f,.44f,-.02f,.40f,-.53f,-.83f,-.57f,-.07f,.20f,.27f,.10f,-.19f,.99f,.18f,-.24f,.73f,-.30f,-.83f,-.34f,-.16f,-.23f,.18f,-.69f,-.70f,.52f,-.33f,.45f,-.81f,-.17f,.03f,.49f,.80f,.88f,-.30f,.17f,-.69f,.19f,.67f,-.78f,-.84f,-.75f,-.81f,.65f,.50f,.62f,.19f,-.91f,.26f,-.17f,-.33f,.19f,.13f,.32f,.59f,-.31f,.88f,-.33f,-.42f,-.34f,-.05f,-.35f,.20f,.31f,-.21f,-.30f,.70f,.10f,-.28f,-.27f,.48f,.45f,-.67f,-.14f,-.41f,-.80f,.02f,.03f,-.49f,-.73f,-.94f,-.28f,-.11f,.94f,-.02f,.06f,-.64f,-.87f,.59f,-.99f,-.19f,.50f,-.12f,.84f,.98f,-.69f,.64f,.93f,-.93f,-.42f,-.96f,-.26f,.21f,.48f,-.64f,.58f,-.86f,-.85f,-.73f,-.81f,.08f,-.89f,.00f,-.16f,.78f,.25f,-.87f,-.30f,-.19f,.58f,.30f,-.10f,.54f,.06f,-.52f,-.88f,.63f,-.34f,-.19f,-.92f,.60f,-.90f,-.21f,-.90f,-.28f,-.14f,-.71f,.49f,.80f,-.49f,-.38f,.57f,.10f,-.80f,.25f,-.38f,-.12f,-.52f,-.01f,-.72f,.74f,-.35f,.88f,.02f,-.14f,-.40f,-.42f,.12f,.93f,-.98f,.95f,-.34f,.53f,.00f,.22f,.35f,.90f,-.39f,-.04f,-.27f,-.20f,.90f,-.00f,.36f,-.68f,-.95f,.21f,.76f,.29f,.99f,-.28f,.02f,-.79f,-.39f,.63f,-.25f,.64f,.72f,-.93f,-.64f,.34f,.36f,.76f,.77f,.27f,.63f,.28f,.94f,.80f,.45f,.49f,.09f,.22f,.50f,.44f,.94f,-.85f,.32f,.34f,.02f,-.04f,-.50f,-.54f,-.96f,.77f,-.09f,-.20f,-.24f,-.78f,.36f,-.73f,.87f,.23f,-.99f,.72f,-.56f,.79f,.14f,-.48f,.37f,.69f,.91f,-.58f,.37f,-.86f,-.76f,-.15f,-.60f,-.16f,.13f,-.97f,-.08f,.84f,-.34f,-.46f,.23f,.61f,-.71f,.71f,-.07f,-.72f,.17f,-.25f,.58f,.10f,.90f,-.80f,.26f,.21f,.48f,-.24f,-.28f,.27f,-.80f,.52f,-.76f,.87f,-.71f,.20f,-.14f,.17f,.87f,-.37f,-.87f,.62f,.50f,.89f,-.66f,.43f,-.85f,.70f,.73f,-.69f,-.95f,-.85f,-.43f,.94f,-.15f,-.69f,.23f,-.29f,.76f,-.62f,.05f,-.59f,.59f,.47f,.27f,-.13f,.47f,-.18f,-.20f,.02f,-.43f,.60f,.11f,-.26f,-.41f,.82f,.66f,-.31f,-.32f,.58f,.86f,.49f,-.90f,.40f,.35f,-.16f,.48f,-.46f,-.02f,-.07f,.32f,-.98f,-.33f,.33f,.22f,-.26f,.09f,-.79f,.06f,.54f,-.87f,.02f,.83f,-.33f,.10f,.50f,-.44f,-.45f,-.97f,-.51f,-.90f,-.56f,.79f,-.48f,.41f,.72f,-.23f,.91f,-.07f,-.38f,.77f,-.55f,-.81f,-.55f,.75f,.66f,-.47f,.44f,-.90f,-.06f,.68f,-.31f,-.25f,-.00f,.47f,-.12f,.12f,.99f,-.62f,-.84f,.17f,.92f,.38f,-.19f,-.97f,.48f,-.89f,.30f,-.08f,.03f,.42f,.14f,.36f,-.26f,-.09f,.41f,.76f,.66f,-.11f,-.19f,.22f,-.11f,.39f,.68f,-.81f,.87f,.04f,.54f,.02f,.05f,-.24f,.05f,.57f,-.81f,-.77f,-.09f,-.35f,-.00f,-.61f,.03f,-.48f,.40f,-.22f,-.19f,-.55f,-.53f,-.32f,-.17f,-.08f,.57f,.18f,.41f,.54f,.21f,-.25f,.07f,.76f,-.97f,-.79f,-.03f,-.56f,-.50f,-.55f,-.58f,.59f,.82f,-.18f,-.50f,-.32f,-.83f,-.44f,-.94f,-.70f,.90f,-.23f,-.23f,-.83f,.35f,-.45f,.31f,.52f,-.14f,.79f,-.30f,-.51f,.99f,.47f,.00f,-.42f,.43f,-.52f,.68f,-.11f,.54f,.98f,.66f,-.32f,.13f,-.87f,-.20f,.27f,-.74f,.64f,-.08f,-.68f,-.78f,.45f,-.37f,.23f,.79f,-.03f,-.09f,.27f,-.46f,-.22f,-.39f,.92f,.80f,-.83f,.89f,.42f,.75f,-.59f,-.26f,-.62f,-.37f,-.07f,.97f,-.90f,.39f,-.50f,-.20f,-.54f,.34f,-.43f,-.82f,-.35f,.42f,-.46f,.48f,-.69f,.33f,.19f,.99f,.06f,-.30f,-.99f,.13f,.47f,-.23f,-.47f,.07f,.82f,-.58f,.42f,-.83f,-.19f,-.98f,-.80f,.28f,.15f,.44f,-.87f,.99f,-.46f,-.85f,-.07f,-.46f,-.93f,.87f,.49f,-.09f,-.74f,-.43f,-.94f,.44f,.50f,-.84f,-.51f,.50f,-.33f,.58f,.22f,-.89f,-.65f,.39f,.01f,.94f,-.73f,-.30f,.69f,.47f,.11f,-.30f,-.16f,-.58f,.76f,-.78f,-.84f,.83f,.96f,-.44f,-.40f,-.28f,.76f,-.47f,-.00f,.91f,.12f,.58f,.64f,.61f,.30f,-.55f,-.33f,.40f,.17f,.35f,.93f,-.63f,.33f,.21f,-.03f,-.55f,.37f,.55f,-.14f,-.37f,-.32f,-.41f,-.50f,-.30f,.49f,-.16f,-.13f,-.77f,-.82f,-.16f,.67f,-.83f,-.87f,.17f,.39f,-.72f,-.09f,.28f,.43f,.44f,-.28f,.23f,.33f,-.95f,.94f,-.70f,.03f,-.73f,-.06f,.90f,-.13f,.39f,-.57f,.13f,-.00f,.30f,-.83f,.53f,-.61f,.27f,.84f,.98f,.98f,-.34f,-.83f,-.36f,.16f,-.93f,-.45f,.23f,.22f,.18f,.80f,-.56f,-.34f,-.40f,-.84f,-.63f,.98f,-.07f,-.36f,.52f,.06f,-.61f,-.19f,.10f,.60f,.37f,-.74f,.84f,-.86f,.63f,.75f,-.48f,.93f,-.39f,-.01f,.43f,.24f,-.49f,-.36f,-.97f,-.56f,-.08f,.61f,.38f,-.25f,.81f,.42f,.42f,.95f,.18f,-.14f,.12f,-.32f,-.62f,.84f,-.92f,-.57f,.57f,-.18f,-.70f,.39f,.48f,.37f,-.34f,-.65f,-.42f,-.81f,.50f,-.27f,-.63f,.09f,-.97f,.76f,-.86f,-.93f,-.67f,-.86f,-.69f,.15f,-.21f,.20f,-.52f,-.65f,.80f,-.69f,-.78f,.23f,-.69f,-.84f,.09f,.53f,-.24f,.44f,.16f,-.10f,.46f,-.63f,-.50f,-.08f,.33f,.76f,-.37f,-.16f,.81f,.44f,.02f,.33f,-.73f,.72f,-.08f,-.47f,.59f,.76f,-.83f,.09f,-.90f,-.91f,-.53f,.80f,-.51f,.33f,-.78f,.47f,-.71f,-.58f,.22f,.53f,-.10f,-.00f,.10f,-.71f,.94f,.18f,-.67f,-.10f,-.95f,.52f,.87f,-.45f,.68f,-.00f,-.78f,-.01f,-.04f,.39f,.02f,.77f,.82f,-.98f,-.90f,-.63f,-.31f,-.97f,.29f,-.00f,.55f,-.58f,.97f,-.35f,.16f,-.56f,.06f,.97f,.54f,.80f,-.59f,.87f,.74f,-.31f,-.24f,.39f,.43f,.33f,-.12f,-.30f,.61f,.77f,.23f,-.05f,-.13f,.14f,.76f,-.12f,.53f,-.56f,-.75f,-.84f,-.97f,.33f,.65f,.16f,-.46f,.28f,-.39f,.40f,.03f,-.36f,.97f,-.27f,-.31f,-.34f,.68f,-.15f,-.09f,.84f,.29f,.90f,-.42f,-.82f,.89f,.15f,.20f,-.32f,-.03f,.29f,.61f,-.52f,-.62f,-.93f,.79f,.28f,-.03f,.03f,-.54f,-.61f,.31f,-.91f,.92f,-.18f,-.12f,.79f,-.24f,-.40f,.46f,-.63f,-.93f,.82f,.40f,-.09f,.12f,-.77f,-.13f,-.58f,.86f,-.86f,.50f,-.72f,-.46f,-.81f,.94f,-.66f,.24f,-.78f,-.79f,-.48f,.49f,.47f,-.04f,-.83f,-.49f,-.48f,-.16f,-.18f,.03f,-.90f,-.47f,-.26f,.38f,.90f,-.81f,-.64f,-.29f,.69f,.62f,-.99f,.50f,.94f,.68f,-.00f,.51f,.26f,-.78f,-.71f,.63f,-.81f,-.18f,-.30f,.28f,.18f,-.20f,-.44f,.26f,-.13f,-.86f,.77f,.78f,-.19f,-.15f,-.77f,.37f,-.01f,-.21f,-.07f,-.78f,.54f,.78f,.38f,.69f,.32f,.04f,.42f,-.40f,.06f,.79f,-.26f,-.15f,-.27f,.23f,-.37f,.90f,-.22f,.73f,-.87f,.59f,-.53f,-.35f,-.88f,.82f,-.57f,-.86f,-.38f,-.87f,.72f,.51f,.56f,.32f,.27f,.86f,.96f,-.40f,-.54f,-.54f,.64f,.99f,-.50f,.29f,.58f,.66f,-.58f,.23f,.24f,-.67f,-.23f,.31f,.47f,-.91f,.56f,.97f,.06f,-.68f,-.36f,.47f,.98f,.41f,-.71f,-.38f,.94f,-.28f,.10f,.87f,.63f,.19f,-.93f,-.64f,.35f,.85f,-.33f,.62f,-.54f,.37f,.92f,-.22f,-.55f,-.59f,-.06f,-.81f,.81f,-.05f,.70f,.47f,.62f,-.69f,-.19f,.37f,-.57f,.66f,-.14f,-.97f,-.71f,.57f,.66f,-.46f,.66f,-.11f,.91f,.75f,-.02f,.14f,-.25f,-.32f,.92f,-.58f,.48f,.62f,.23f,.06f,.17f,-.28f,.81f,.78f,-.14f,.09f,.12f,.27f,.36f,.92f,-.06f,-.52f,-.17f,.36f,-.22f,-.41f,-.64f,.64f,-.69f,-.68f,-.07f,-.12f,.73f,.88f,-.49f,.00f,-.93f,-.45f,-.39f,.67f,-.02f,-.55f,.61f,.51f,.41f,.27f,.19f,-.49f,.14f,.16f,-.45f,-.31f,.70f,-.17f,.52f,-.34f,.66f,.84f,.47f,-.90f,-.38f,.21f,.55f,.86f,.86f,-.79f,-.44f,-.51f,-.09f,-.30f,-.39f,-.18f,-.81f,.32f,-.90f,-.14f,-.24f,.77f,-.68f,.09f,.64f,.08f,.15f,-.36f,.51f,-.54f,-.34f,-.73f,-.25f,.66f,-.33f,.23f,-.93f,.40f,.62f,.12f,.23f,.20f,.57f,-.52f,.08f,.53f,.10f,.79f,-.63f,.46f,-.12f,.21f,.60f,.84f,.84f,-.54f,-.24f,-.75f,.71f,.94f,.81f,-.07f,-.96f,-.51f,.40f,-.26f,.34f,-.79f,-.95f,-.58f,.54f,-.48f,-.83f,-.71f,-.45f,-.68f,.05f,.28f,.58f,.16f,.48f,.09f,.09f,.13f,.76f,-.54f,.14f,-.47f,.95f,-.40f,.96f,-.26f,-.38f,-.54f,.05f,.77f,-.84f,-.88f,.81f,-.69f,-.93f,-.03f,.32f,.60f,-.59f,-.07f,-.66f,.68f,-.77f,-.55f,.31f,-.22f,-.20f,.13f,.32f,.32f,.32f,-.86f,-.19f,.12f,.82f,.38f,.73f,.07f,.09f,-.04f,-.62f,.13f,-.51f,-.87f,-.48f,.07f,.13f,-.16f,.68f,-.97f,-.59f,.00f,.24f,-.67f,-.57f,.59f,.64f,.36f,-.50f,.01f,.94f,.33f,.69f,.24f,-.96f,.31f,-.21f,.77f,-.78f,-.87f,.73f,.55f,.46f,.95f,.99f,.64f,.03f,.04f,-.19f,-.03f,.54f,.19f,-.09f,.57f,.25f,.75f,-.22f,.45f,.23f,-.52f,-.27f,-.83f,.01f,-.08f,-.51f,.26f,-.36f,-.30f,-.77f,.17f,-.02f,-.57f,-.29f,-.47f,-.53f,.97f,.49f,.03f,.52f,.93f,.71f,.82f,-.35f,.38f,.47f,-.92f,-.36f,.69f,-.88f,.58f,-.02f,-.80f,-.69f,.56f,-.15f,-.74f,.88f,.35f,.24f,.03f,-.43f,-.03f,-.97f,.17f,-.92f,.44f,-.57f,-.90f,.50f,.26f,-.87f,.25f,-.70f,-.19f,.56f,.92f,.09f,.55f,-.38f,.28f,.91f,-.25f,.55f,.46f,-.80f,.24f,-.66f,.32f,-.05f,-.09f,.12f,.86f,.66f,-.93f,.88f,-.31f,-.13f,.68f,.84f,-.58f,.19f,-.61f,-.06f,-.01f,.10f,-.77f,.36f,-.05f,.96f,.83f,-.66f,.81f,.54f,.46f,-.93f,.19f,-.75f,.85f,-.26f,-.03f,-.77f,.54f,-.54f,.72f,-.05f,-.81f,.58f,-.34f,-.53f,.98f,.96f,-.55f,.99f,-.48f,-.29f,-.01f,.09f,-.02f,-.34f,-.15f,.44f,-.54f,-.82f,-.35f,-.87f,-.45f,.80f,.74f,.86f,.75f,.66f,-.69f,-.29f,-.18f,.17f,-.50f,.66f,.37f,.75f,-.44f,-.07f,.06f,-.36f,-.10f,-.63f,.12f,-.25f,-.71f,-.51f,-.75f,-.46f,.52f,-.36f,-.97f,.75f,-.23f,.26f,-.27f,.78f,-.93f,-.54f,.00f,.73f,.05f,.00f,-.95f,.19f,.53f,-.57f,.82f,-.48f,-.46f,.37f,.74f,-.99f,.46f,-.10f,.91f,.83f,-.45f,-.37f,-.05f,-.75f,-.23f,-.88f,-.16f,-.97f,.80f,.64f,-.57f,.53f,-.70f,-.19f,.96f,.26f,-.70f,.59f,-.32f,.49f,-.25f,-.01f,.57f,-.21f,.04f,-.32f,.23f,.19f,-.17f,-.58f,-.27f,-.50f,-.59f,-.77f,.07f,-.37f,.13f,-.83f,.31f,.63f,-.55f,-.32f,-.66f,-.61f,-.95f,.04f,.58f,.19f,.50f,-.79f,.32f,.89f,.58f,-.28f,.21f,-.36f,.84f,.24f,-.18f,.96f,-.43f,.17f,-.38f,-.07f,.05f,.91f,-.41f,-.24f,.93f,-.46f,-.51f,.93f,.49f,.17f,.72f,-.07f,-.94f,-.04f,.87f,.86f,-.73f,-.76f,-.54f,-.03f,.83f,.62f,.76f,-.72f,-.76f,.95f,.32f,.61f,.45f,.63f,.89f,-.73f,.63f,.30f,-.75f,-.03f,.64f,.48f,-.94f,-.40f,.28f,.91f,.29f,-.02f,.62f,-.06f,.07f,.86f,.31f,.93f,.41f,.58f,.67f,-.30f,.64f,.59f,-.33f,-.77f,-.52f,.74f,-.58f,.93f,-.17f,-.49f,-.91f,.82f,-.57f,.05f,.17f,.44f,.72f,-.95f,-.29f,.70f,-.96f,-.11f,.31f,-.52f,-.97f,-.58f,.46f,-.06f,-.30f,.25f,.16f,-.38f,.72f,.47f,.47f,.84f,.64f,-.61f,-.39f,-.78f,.58f,.19f,-.05f,.38f,.35f,.11f,-.16f,.85f,-.63f,.88f,.22f,.79f,.30f,-.50f,-.90f,.41f,.16f,.40f,-.38f,-.94f,-.61f,-.02f,.13f,-.84f,.86f,.61f,.08f,.00f,.13f,-.58f,.09f,-.10f,-.21f,-.89f,-.53f,.41f,.02f,.93f,.94f,-.95f,.14f,-.81f,.51f,.00f,-.23f,.11f,-.50f,-.62f,-.57f,.46f,.65f,.98f,.61f,.67f,-.32f,-.09f,-.70f,.62f,.40f,-.69f,.15f,.73f,.76f,.49f,-.10f,-.13f,.93f,.72f,.08f,.96f,.44f,.63f,-.68f,-.69f,.35f,.50f,-.73f,.58f,.85f,-.21f,.32f,-.87f,.05f,-.05f,.35f,.63f,-.24f,-.96f,-.42f,-.05f,.75f,.07f,-.91f,-.35f,-.80f,-.48f,-.86f,-.35f,.42f,.88f,.38f,.53f,-.74f,-.61f,.14f,-.43f,.68f,-.02f,-.30f,-.81f,-.39f,.32f,-.39f,.23f,-.82f,.17f,-.05f,-.45f,-.37f,.57f,-.44f,.72f,-.00f,.84f,.98f,-.03f,-.34f,-.51f,-.27f,.46f,-.49f,.86f,-.15f,-.70f,.91f,-.87f,-.44f,.68f,.96f,.20f,-.22f,.96f,-.46f,-.18f,.42f,-.02f,-.66f,.85f,.23f,.03f,.28f,-.56f,-.20f,-.94f,.90f,-.11f,-.63f,-.19f,-.37f,.29f,.18f,-.32f,.41f,-.42f,.95f,-.88f,.74f,-.55f,-.36f,-.70f,.09f,.08f,-.33f,.16f,.67f,.46f,-.94f,-.47f,.94f,.61f,-.40f,.97f,-.08f,.29f,.61f,-.88f,.66f,.83f,-.84f,.61f,.66f,.92f,.26f,-.12f,-.25f,-.01f,-.07f,-.98f,-.45f,.92f,-.29f,-.48f,.08f,.16f,.53f,.96f,.55f,.39f,-.61f,.45f,-.45f,-.09f,.10f,-.07f,-.11f,-.87f,-.43f,.32f,.91f,-.72f,-.75f,.55f,-.80f,.33f,.03f,-.19f,-.93f,-.65f,.92f,-.26f,.89f,.50f,.28f,-.98f,-.55f,-.80f,-.50f,.33f,.08f,.85f,-.33f,.57f,-.59f,.97f,.95f,.51f,.60f,-.80f,.92f,-.55f,.10f,.13f,-.89f,.04f,-.75f,.73f,.80f,.87f,.23f,.59f,-.63f,.70f,-.95f,.82f,-.27f,.54f,.19f,.96f,.95f,-.81f,.02f,.45f,.40f,-.52f,-.74f,.27f,.37f,-.27f,-.23f,.24f,.28f,.27f,.38f,.62f,-.84f,.22f,-.88f,-.77f,.61f,-.22f,-.26f,.59f,.63f,.49f,-.77f,.30f,-.57f,.39f,-.21f,-.68f,.43f,.60f,.30f,-.51f,-.28f,.26f,-.06f,-.68f,.50f,-.07f,.16f,.25f,-.39f,-.08f,.81f,.83f,-.72f,-.67f,.35f,-.08f,.37f,-.24f,.64f,.37f,-.17f,.61f,-.35f,.21f,.72f,.98f,.38f,-.14f,.59f,.45f,.30f,.61f,.62f,.56f,-.66f,-.39f,.11f,-.43f,-.08f,.75f,.31f,-.64f,-.53f,.41f,.48f,.54f,.62f,-.64f,-.58f,-.00f,.05f,-.62f,-.65f,.72f,-.90f,.29f,-.56f,-.68f,.98f,.84f,-.43f,.16f,-.43f,.16f,-.86f,.23f,.70f,-.94f,-.74f,.41f,.20f,.02f,-.35f,-.99f,-.87f,-.65f,.77f,.32f,.92f,.14f,.95f,-.80f,-.14f,.10f,.60f,.06f,.19f,.28f,.32f,-.87f,.32f,-.61f,-.94f,-.76f,.91f,-.77f,-.80f,-.33f,.36f,-.66f,.50f,.06f,-.79f,-.14f,.38f,.54f,-.34f,-.51f,-.35f,-.78f,.40f,.78f,-.55f,-.93f,.77f,-.93f,.69f,.86f,-.07f,-.78f,-.16f,.30f,-.95f,-.53f,.73f,-.75f,-.55f,-.16f,-.30f,-.74f,.95f,-.18f,-.65f,.99f,-.87f,.49f,.24f,-.06f,.58f,-.93f,-.93f,-.36f,.74f,.54f,-.62f,.09f,.09f,-.05f,-.06f,.15f,-.69f,-.22f,-.20f,.70f,.56f,-.30f,-.70f,-.69f,-.10f,.44f,-.01f,-.95f,-.85f,.53f,-.05f,.00f,-.66f,.05f,-.52f,.64f,-.21f,.22f,-.25f,-.22f,.28f,-.70f,.57f,-.90f,-.33f,-.69f,-.14f,.08f,-.38f,-.14f,-.97f,.84f,.69f,-.38f,.36f,.42f,-.39f,.29f,.50f,-.36f,-.58f,-.40f,.88f,.70f,.59f,.88f,.90f,-.10f,-.22f,-.68f,-.92f,-.55f,-.46f,.96f,-.50f,.08f,.92f,-.68f,-.27f,.29f,-.95f,.58f,.69f,-.62f,-.25f,.02f,-.50f,.01f,.33f,-.93f,-.85f,-.51f,.04f,.29f,-.32f,-.66f,.08f,.94f,-.81f,-.70f,.44f,-.13f,.72f,-.81f,-.55f,-.53f,-.27f,-.76f,.04f,.91f,.80f,-.91f,-.79f,-.11f,.17f,-.42f,.57f,.72f,-.52f,-.12f,-.00f,-.17f,.98f,-.30f,.20f,.64f,-.29f,-.42f,.45f,.85f,.53f,.47f,-.29f,-.04f,.97f,.86f,-.13f,-.96f,-.34f,.75f,-.04f,-.21f,.59f,.68f,.36f,-.11f,.76f,-.59f,-.28f,.07f,.49f,.23f,-.01f,-.00f,-.33f,-.14f,.17f,.68f,.41f,-.02f,-.40f,.75f,.39f,-.07f,.11f,.66f,.47f,.90f,.17f,-.15f,.48f,-.10f,-.81f,.47f,.72f,.51f,-.64f,.41f,-.21f,.97f,.42f,.81f,.00f,-.99f,.43f,.83f,.78f,-.92f,.32f,.59f,-.96f,-.46f,.32f,.53f,-.28f,-.00f,-.04f,-.99f,.67f,-.13f,-.43f,.50f,-.65f,.96f,.71f,-.71f,-.40f,.44f,.25f,.90f,.45f,.71f,-.66f,.00f,.52f,.93f,.58f,.29f,.62f,.27f,.56f,.42f,.26f,.78f,-.92f,-.29f,.98f,.97f,.80f,.53f,.40f,-.28f,-.01f,-.15f,.00f,.18f,.01f,-.46f,-.81f,-.42f,.58f,-.47f,.40f,-.81f,.67f,.70f,-.64f,.31f,-.75f,-.07f,.73f,.31f,.20f,.07f,-.31f,-.50f,-.91f,-.53f,.80f,.13f,-.31f,.46f,.99f,-.52f,.09f,-.21f,.46f,-.76f,.27f,.88f,.30f,-.83f,-.85f,.98f,-.34f,-.84f,.50f,-.78f,.17f,-.72f,-.06f,-.53f,-.53f,-.74f,.95f,.42f,.25f,-.73f,.10f,-.06f,-.87f,.01f,.54f,.87f,.68f,-.85f,.53f,-.02f,.86f,-.15f,.23f,.18f,-.34f,.67f,-.82f,-.70f,.35f,-.90f,.09f,-.03f,.94f,.69f,.43f,.60f,-.87f,.45f,.61f,-.31f,.54f,-.91f,-.73f,.58f,.27f,-.81f,-.94f,-.63f,-.91f,.10f,.25f,.08f,.47f,-.75f,-.34f,.69f,.37f,.90f,.21f,.81f,-.53f,.80f,-.67f,-.95f,-.55f,.21f,-.04f,-.12f,-.64f,.78f,.15f,-.49f,.26f,-.29f,.09f,-.84f,-.93f,.11f,.58f,-.67f,.60f,-.40f,-.61f,.72f,-.36f,-.24f,-.89f,-.21f,-.82f,-.37f,-.57f,.72f,-.66f,.63f,.74f,.37f,-.51f,.37f,-.71f,.92f,-.34f,.02f,-.95f,-.37f,-.92f,.04f,.12f,-.26f,-.43f,-.81f,-.26f,-.48f,.44f,-.12f,-.75f,.98f,-.31f,-.01f,.29f,.77f,.29f,.87f,.47f,.18f,.07f,-.99f,.81f,.14f,.53f,-.03f,-.22f,.28f,-.56f,-.86f,.52f,.83f,.89f,-.54f,-.48f,-.91f,.80f,-.12f,-.06f,.09f,.75f,-.36f,.55f,.55f,.46f,.74f,.53f,.99f,-.33f,-.45f,-.06f,-.12f,.19f,-.17f,-.38f,-.78f,-.58f,-.74f,-.65f,-.03f,-.77f,-.38f,-.97f,.24f,.57f,.75f,.45f,.39f,.89f,-.81f,-.42f,.93f,-.87f,.52f,.44f,-.34f,-.16f,-.02f,-.85f,.97f,-.41f,-.57f,.96f,-.94f,-.66f,.92f,-.94f,.23f,-.06f,-.68f,.37f,.30f,.75f,-.51f,.12f,.52f,-.17f,.55f,-.17f,.40f,.60f,.97f,.83f,-.13f,-.28f,.73f,-.77f,.36f,-.03f,.96f,.41f,.97f,-.65f,-.78f,-.68f,.71f,-.25f,-.02f,-.14f,.32f,.48f,-.85f,-.12f,-.11f,.90f,-.59f,-.70f,.19f,.53f,-.83f,.34f,-.22f,.34f,-.37f,.01f,-.64f,-.40f,-.64f,-.04f,-.41f,-.77f,-.63f,.16f,-.17f,.29f,-.19f,.87f,.49f,-.09f,.27f,-.24f,.73f,-.55f,.97f,-.62f,.64f,.27f,-.66f,-.34f,.39f,.81f,-.13f,.14f,.88f,-.43f,.79f,-.88f,.91f,-.25f,-.24f,-.40f,-.51f,-.61f,.11f,.45f,.23f,.60f,-.73f,.20f,.65f,-.25f,-.67f,.76f,-.83f,.64f,-.01f,-.51f,-.27f,-.03f,.33f,.27f,-.69f,-.85f,.06f,.53f,-.78f,.85f,-.26f,.46f,-.79f,.86f,-.25f,-.67f,-.88f,-.69f,-.71f,.58f,.09f,-.60f,.83f,.40f,-.88f,-.15f,.46f,.05f,.37f,.25f,-.85f,-.39f,.69f,-.68f,-.21f,.97f,-.03f,-.13f,-.59f,.65f,-.72f,-.84f,-.80f,.42f,-.36f,-.11f,-.84f,-.27f,-.13f,-.03f,.82f,.33f,-.24f,.30f,-.28f,.41f,-.56f,-.47f,-.18f,-.57f,-.56f,-.87f,-.05f,-.17f,-.31f,-.73f,-.89f,.01f,.20f,.06f,-.85f,-.75f,.78f,-.60f,.87f,.91f,-.13f,.75f,-.49f,.96f,-.60f,-.87f,-.49f,.83f,.42f,-.29f,.32f,-.46f,-.84f,.37f,-.36f,.39f,-.72f,.24f,.28f,-.63f,.40f,.92f,-.75f,-.68f,.74f,.07f,.60f,.22f,-.50f,-.10f,-.08f,.50f,-.34f,.24f,-.77f,-.23f,.77f,.89f,-.20f,.89f,-.95f,.35f,.19f,.70f,-.10f,.07f,.71f,.16f,.33f,.37f,.13f,-.98f,-.12f,-.50f,.68f,-.62f,-.38f,.13f,.86f,.22f,-.56f,.35f,-.98f,-.72f,.86f,-.44f,.73f,.57f,.84f,-.93f,-.62f,-.67f,.14f,-.73f,-.85f,-.38f,.01f,-.87f,.05f,-.98f,-.18f,.56f,.90f,-.66f,-.81f,.09f,-.63f,.04f,-.88f,.74f,.73f,.71f,.68f,.27f,-.52f,.16f,-.72f,.10f,-.71f,.97f,-.55f,-.06f,-.68f,-.74f,-.66f,-.93f,.23f,-.35f,.37f,.54f,.60f,.10f,-.76f,.45f,-.66f,-.08f,.98f,-.09f,.96f,-.54f,-.33f,-.04f,.36f,.86f,-.42f,-.50f,.45f,.33f,.78f,.25f,.58f,.15f,-.89f,-.49f,.42f,.78f,-.86f,-.65f,.81f,-.67f,.62f,.14f,.62f,.05f,-.93f,.34f,-.40f,.01f,.46f,-.34f,.51f,-.86f,.94f,-.20f,-.75f,.27f,-.09f,.41f,-.86f,.80f,-.13f,-.31f,.52f,-.12f,.46f,.58f,.60f,-.12f,-.45f,-.61f,-.33f,-.80f,-.25f,-.00f,-.20f,-.48f,-.98f,.96f,-.86f,-.21f,.30f,.26f,.53f,.83f,.82f,-.39f,-.66f,-.69f,-.42f,.35f,-.88f,.27f,-.34f,-.21f,-.89f,.74f,-.98f,-.35f,-.72f,-.37f,-.32f,.39f,-.24f,-.83f,.95f,-.43f,.72f,.24f,-.85f,-.19f,.55f,.02f,.34f,-.41f,-.08f,-.25f,.40f,-.00f,.58f,.43f,.79f,.29f,-.57f,-.02f,-.31f,-.01f,.33f,-.09f,-.07f,.53f,-.58f,.94f,-.64f,.37f,.63f,-.79f,.21f,.72f,.24f,.09f,.34f,.15f,.41f,.94f,.69f,.51f,.52f,-.03f,-.81f,-.99f,-.76f,.33f,.61f,.03f,-.61f,.78f,.69f,.62f,-.22f,-.70f,-.64f,.55f,-.56f,.24f,-.39f,.33f,-.08f,.92f,.88f,-.94f,-.85f,-.47f,-.96f,-.51f,.22f,.87f,-.85f,.51f,.27f,.12f,-.83f,-.13f,-.63f,.72f,.51f,.35f,-.94f,.60f,.79f,-.57f,-.70f,.21f,-.68f,-.84f,.10f,.76f,.92f,.60f,-.16f,.85f,-.28f,-.00f,.25f,.61f,.69f,-.62f,.17f,.05f,.08f,-.87f,.23f,.95f,-.82f,.84f,-.58f,.76f,-.46f,-.35f,.76f,-.57f,.70f,-.14f,.55f,.57f,.57f,-.28f,-.18f,.46f,.89f,.07f,.45f,-.65f,-.46f,.01f,.32f,-.96f,-.71f,.05f,.35f,-.90f,-.56f,.72f,-.33f,-.48f,.42f,.32f,.16f,-.55f,-.83f,.42f,.74f,.41f,.93f,.61f,-.39f,-.03f,-.81f,-.37f,-.55f,-.06f,.62f,.69f,.74f,.21f,.12f,-.18f,.25f,-.66f,.40f,-.33f,.95f,.72f,.29f,.71f,-.46f,-.56f,-.79f,.59f,-.71f,-.02f,.61f,.66f,.23f,-.44f,.99f,.66f,-.47f,.99f,.68f,-.21f,-.96f,-.67f,.39f,.22f,-.72f,.01f,.62f,.68f,-.87f,-.92f,-.64f,-.60f,-.51f,-.45f,.72f,-.86f,.99f,-.77f,.36f,.20f,.17f,-.32f,-.06f,.10f,.63f,-.08f,-.01f,.58f,-.31f,.18f,-.09f,.10f,-.72f,.98f,-.76f,-.21f,-.61f,.19f,-.52f,.13f,.39f,-.53f,-.86f,-.91f,-.57f,.61f,.47f,-.73f,.63f,.07f,.90f,-.58f,.52f,.45f,-.51f,-.87f,.31f,.25f,-.08f,.53f,-.73f,-.08f,-.86f,-.63f,.63f,-.40f,-.66f,-.40f,-.71f,-.76f,.66f,-.22f,-.58f,.49f,-.97f,.84f,.96f,-.09f,-.92f,-.24f,.01f,-.97f,-.80f,.00f,-.99f,.62f,.62f,-.32f,.11f,.30f,.80f,.62f,-.57f,.89f,.57f,-.87f,-.95f,.74f,-.57f,-.31f,.05f,-.78f,-.51f,.84f,-.40f,-.97f,-.31f,.89f,-.41f,.15f,.52f,.19f,-.33f,.07f,.74f,.06f,-.64f,-.70f,.58f,-.31f,-.99f,-.46f,.28f,.08f,-.73f,-.31f,.70f,-.90f,.28f,.88f,-.92f,-.21f,-.63f,.47f,.72f,.86f,.95f,.79f,.34f,.16f,.91f,.65f,-.86f,-.81f,.67f,.18f,-.58f,.25f,-.49f,.52f,-.86f,-.25f,.11f,-.97f,.44f,.43f,.64f,.27f,-.64f,-.44f,.96f,-.44f,-.79f,.17f,-.21f,.86f,-.61f,-.16f,-.68f,.95f,.67f,.58f,.10f,.61f,-.99f,-.01f,.86f,-.80f,.35f,-.44f,.84f,.92f,-.35f,.62f,-.40f,.71f,-.38f,.43f,.02f,.74f,-.66f,.68f,.29f,-.79f,.36f,.07f,-.04f,.43f,-.51f,-.47f,.55f,.15f,.82f,-.94f,-.59f,-.85f,-.74f,.85f,.41f,.59f,.53f,.91f,.12f,.92f,-.52f,.41f,.41f,-.25f,.64f,-.45f,.25f,-.56f,-.53f,.16f,-.75f,-.85f,-.16f,-.84f,-.85f,-.57f,-.57f,-.00f,.41f,-.96f,.34f,.47f,-.53f,-.84f,.00f,.39f,-.38f,.87f,-.84f,.56f,-.49f,.32f,.23f,-.04f,-.16f,-.01f,-.47f,-.02f,.57f,-.69f,-.71f,-.71f,.72f,.82f,.65f,.73f,-.31f,.76f,-.64f,.15f,-.50f,-.59f,-.94f,.81f,.78f,-.19f,-.56f,.87f,-.71f,.65f,.43f,.79f,.33f,.07f,.33f,-.77f,.11f,.88f,.47f,.66f,-.62f,.89f,.22f,-.49f,-.71f,-.87f,.77f,-.86f,.08f,-.53f,-.58f,.89f,.57f,-.65f,-.25f,-.48f,-.28f,-.59f,.60f,-.56f,-.06f,.33f,.29f,-.00f,-.66f,.86f,-.90f,-.59f,-.32f,.00f,-.16f,-.59f,.23f,.45f,-.86f,.59f,.37f,-.79f,-.57f,.77f,.34f,-.08f,-.98f,.01f,.69f,.28f,-.84f,.74f,-.02f,.49f,.29f,.07f,.09f,.47f,.87f,.25f,.38f,-.31f,-.12f,-.23f,.23f,-.94f,-.48f,.38f,-.12f,-.66f,-.72f,-.21f,.83f,.78f,-.27f,.81f,.58f,-.93f,-.63f,-.42f,-.16f,-.96f,-.73f,.76f,.32f,-.60f,.50f,.24f,-.48f,-.20f,.99f,-.24f,.59f,.07f,-.80f,-.01f,.45f,-.48f,.34f,-.60f,.15f,-.40f,.68f,.69f,.13f,-.99f,-.21f,-.17f,.29f,-.20f,.86f,-.80f,.82f,-.36f,.79f,.75f,-.37f,-.54f,.16f,-.86f,.36f,-.46f,.33f,-.31f,.75f,-.62f,.79f,-.45f,.34f,-.72f,-.67f,.06f,.17f,.57f,.89f,-.08f,.58f,-.01f,.27f,.05f,-.42f,-.22f,.86f,.92f,-.66f,-.01f,-.62f,.62f,.57f,.17f,.47f,-.50f,-.72f,-.90f,.05f,-.40f,-.77f,-.71f,-.03f,-.81f,.11f,-.34f,-.70f,.05f,.74f,-.27f,-.50f,-.47f,.87f,-.15f,.14f,.31f,-.67f,.74f,-.55f,.41f,-.80f,.84f,-.23f,.10f,-.17f,.65f,.79f,-.67f,.67f,-.17f,.32f,-.06f,.44f,.13f,.26f,-.67f,.63f,-.17f,.40f,.26f,.55f,.29f,.62f,.83f,-.41f,-.07f,.40f,-.40f,-.21f,.45f,-.88f,-.83f,.85f,-.84f,.43f,.44f,.24f,-.32f,.29f,-.62f,-.24f,.38f,-.84f,-.97f,-.68f,.44f,-.70f,-.95f,-.70f,-.39f,-.30f,-.72f,-.55f,-.09f,-.15f,.77f,.40f,-.16f,-.94f,-.24f,-.79f,-.94f,.46f,-.35f,.93f,.10f,.39f,-.37f,-.63f,.03f,-.13f,.64f,.49f,-.52f,.24f,-.59f,.13f,.42f,-.19f,.33f,-.33f,-.67f,.43f,.22f,-.85f,.83f,-.51f,.60f,.15f,-.90f,-.24f,-.39f,.90f,-.87f,.39f,-.70f,-.05f,-.09f,-.95f,-.05f,-.08f,.92f,.95f,-.15f,-.85f,-.87f,.53f,-.31f,.49f,-.59f,.92f,-.82f,-.97f,.40f,-.60f,.80f,-.48f,-.68f,.33f,.73f,-.37f,-.31f,.26f,.62f,.11f,.65f,-.73f,.26f,-.88f,.33f,-.03f,-.54f,-.92f,-.74f,-.80f,-.82f,-.61f,-.97f,-.09f,-.81f,-.83f,-.84f,.02f,-.26f,.60f,-.30f,.41f,-.59f,-.07f,-.70f,.74f,.25f,-.50f,.88f,.73f,-.40f,.49f,.84f,.27f,.80f,-.06f,-.26f,.16f,-.87f,-.86f,.19f,.88f,.00f,.19f,.89f,.94f,.11f,-.09f,.04f,-.15f,-.07f,.56f,.71f,-.94f,-.13f,.72f,.50f,-.17f,-.58f,-.35f,.20f,-.18f,-.31f,-.80f,-.95f,-.00f,.84f,.91f,.34f,.89f,-.08f,.90f,.80f,-.41f,-.50f,.95f,.41f,.52f,-.21f,-.99f,-.77f,.41f,-.32f,-.99f,.21f,.29f,.82f,.99f,.57f,-.46f,.77f,-.43f,-.71f,.13f,-.61f,-.38f,.89f,-.15f,-.46f,-.81f,-.75f,.54f,.45f,.88f,-.25f,-.74f,.38f,.52f,-.93f,.54f,.50f,.52f,.41f,-.64f,.64f,.04f,.33f,.38f,-.97f,.08f,-.90f,.49f,.54f,-.09f,.90f,.66f,.20f,-.75f,-.42f,.64f,.33f,.21f,-.08f,.02f,.92f,-.18f,-.28f,.75f,.13f,.37f,.17f,.89f,.23f,-.47f,.90f,-.24f,.13f,-.81f,.73f,-.12f,.76f,-.54f,.19f,.98f,.81f,.99f,.76f,-.33f,.12f,.22f,.02f,-.43f,-.39f,-.74f,-.79f,.26f,-.14f,.52f,-.96f,-.60f,-.79f,-.53f,.11f,.65f,-.12f,-.92f,-.85f,-.06f,.31f,-.55f,.27f,-.87f,-.46f,-.95f,.23f,-.52f,-.96f,.06f,-.22f,-.44f,-.11f,.34f,-.89f,.43f,-.24f,.48f,-.07f,.08f,-.85f,-.25f,.92f,-.27f,-.03f,-.06f,-.85f,.12f,-.55f,-.70f,-.11f,.70f,.69f,.76f,-.91f,-.63f,-.61f,-.98f,.73f,-.90f,-.62f,.68f,-.87f,-.56f,-.27f,-.70f,.99f,.79f,-.41f,.54f,-.64f,.61f,-.35f,.90f,-.33f,.51f,-.59f,.95f,-.73f,.15f,-.22f,.26f,.72f,.51f,.61f,-.00f,.37f,.58f,.26f,-.13f,.41f,.41f,-.95f,-.79f,.58f,-.20f,-.86f,-.20f,-.60f,-.44f,.11f,-.23f,.91f,.01f,.87f,-.19f,.24f,.01f,-.88f,.09f,.08f,.83f,.02f,-.57f,-.21f,.93f,.91f,.51f,-.74f,.29f,.31f,.41f,.55f,.72f,-.93f,-.47f,.95f,-.73f,-.76f,-.98f,-.84f,.31f,-.89f,-.27f,-.70f,-.22f,.39f,.81f,-.85f,.69f,-.50f,-.53f,-.14f,-.02f,.31f,-.74f,-.19f,.84f,.99f,-.92f,-.35f,-.30f,.85f,-.64f,-.84f,.69f,.77f,.91f,.04f,.94f,.44f,.28f,.13f,-.65f,-.84f,.61f,.01f,.78f,-.17f,.27f,.72f,-.14f,-.18f,.70f,-.19f,.57f,-.89f,.48f,.36f,.39f,.46f,.67f,.16f,-.86f,-.52f,.20f,-.18f,.70f,-.11f,-.57f,-.96f,-.27f,.41f,.72f,-.33f,-.93f,.99f,-.90f,.79f,.17f,-.40f,-.13f,.92f,.74f,-.58f,-.76f,-.93f,.65f,.97f,.27f,-.84f,-.61f,.83f,-.56f,-.76f,-.69f,-.44f,-.84f,-.95f,.33f,.88f,-.70f,.30f,-.69f,.50f,-.65f,-.16f,-.51f,.81f,.13f,-.45f,.35f,-.10f,.31f,-.21f,.39f,.49f,-.66f,.44f,.71f,.87f,-.61f,.86f,-.65f,.87f,-.89f,.70f,-.20f,-.83f,.61f,.43f,-.98f,-.09f,-.20f,-.09f,.99f,.45f,.28f,-.28f,-.46f,.05f,.96f,.66f,.72f,.27f,-.07f,-.27f,.54f,.16f,.54f,-.60f,-.50f,-.94f,-.28f,-.05f,.52f,.65f,-.99f,.50f,-.51f,.28f,-.93f,-.15f,-.64f,-.62f,.36f,-.83f,.86f,.42f,.60f,.59f,.42f,.89f,.50f,.73f,-.41f,-.61f,-.51f,-.32f,-.53f,.64f,.98f,-.94f,-.13f,.06f,.20f,-.18f,-.05f,.50f,-.39f,-.08f,.94f,-.22f,-.01f,.51f,.43f,-.18f,-.40f,.08f,-.40f,.44f,-.45f,-.39f,.39f,.02f,.81f,-.75f,.91f,-.83f,.06f,-.36f,.03f,.11f,-.19f,-.29f,.21f,-.61f,.44f,-.86f,-.69f,-.18f,.51f,.90f,-.05f,.39f,-.89f,-.61f,.66f,-.06f,.90f,.20f,.91f,-.50f,-.63f,.68f,.40f,-.30f,.28f,.98f,-.56f,-.13f,-.12f,.29f,.66f,.07f,.48f,.60f,.09f,.09f,.38f,.63f,-.27f,.35f,-.26f,-.83f,-.63f,-.84f,.87f,-.94f,.07f,.17f,-.98f,.76f,-.98f,.81f,.18f,-.77f,-.80f,-.26f,.76f,.41f,-.40f,-.80f,.16f,.96f,.81f,.47f,-.29f,.57f,-.06f,-.04f,-.30f,-.92f,-.18f,.32f,-.66f,-.90f,.74f,.61f,.37f,.34f,.67f,.08f,.80f,-.27f,-.49f,-.15f,.84f,-.71f,.56f,.24f,.89f,-.00f,-.44f,.67f,-.27f,.31f,-.86f,.52f,.87f,-.03f,-.34f,-.38f,.82f,.84f,-.30f,-.13f,.86f,-.35f,.67f,.45f,-.24f,.95f,.70f,-.83f,-.17f,-.09f,-.35f,-.47f,.71f,-.75f,.33f,.64f,-.68f,.43f,.38f,.48f,.21f,.85f,-.77f,.11f,-.19f,.05f,-.17f,-.67f,-.84f,-.54f,-.54f,-.70f,-.39f,-.52f,-.42f,-.78f,.39f,.70f,-.06f,-.79f,.41f,.94f,.83f,-.33f,.98f,-.14f,-.40f,-.72f,.66f,.79f,.79f,.29f,-.14f,-.34f,.40f,-.95f,.90f,-.38f,-.29f,.95f,.72f,.17f,-.54f,.29f,-.10f,.93f,.69f,-.19f,.20f,.02f,.96f,-.05f,-.49f,-.93f,.32f,-.31f,.07f,.71f,-.58f,.08f,.95f,-.55f,.95f,.05f,.42f,-.98f,.95f,-.09f,-.77f,.42f,.60f,-.16f,.29f,.25f,.39f,-.81f,-.90f,.46f,-.30f,.10f,.07f,.81f,-.57f,-.31f,-.21f,-.12f,.01f,-.11f,.66f,.01f,.90f,-.38f,.52f,.08f,-.77f,-.66f,-.54f,.00f,-.44f,.17f,.38f,-.33f,.80f,-.80f,.15f,-.51f,-.23f,-.52f,.37f,.58f,-.35f,-.71f,-.79f,-.47f,-.69f,-.88f,-.48f,.28f,.71f,.46f,-.28f,-.13f,.64f,-.43f,-.44f,.44f,-.11f,.22f,.10f,-.94f,.42f,-.15f,.25f,-.25f,-.88f,-.29f,.92f,-.13f,.06f,-.35f,-.66f,-.77f,.02f,-.55f,.51f,-.57f,.50f,.68f,-.22f,.23f,.00f,-.66f,-.36f,.44f,.25f,.28f,.96f,.68f,-.22f,-.33f,-.67f,.11f,-.33f,-.15f,-.16f,-.87f,-.37f,.18f,.62f,-.25f,-.13f,.65f,-.03f,-.00f,.30f,.77f,.63f,.57f,.93f,-.37f,-.06f,-.72f,.39f,.36f,-.28f,.53f,.00f,.71f,-.57f,-.64f,.30f,-.40f,-.30f,.05f,-.90f,.36f,.87f,.77f,.03f,-.41f,-.91f,.81f,.16f,-.12f,-.18f,.30f,-.51f,.26f,-.46f,-.62f,.79f,-.98f,-.60f,-.90f,.01f,-.52f,.66f,-.49f,.75f,-.68f,-.78f,-.70f,.44f,.07f,-.90f,-.50f,.22f,.04f,-.54f,.35f,-.38f,.18f,-.04f,-.09f,-.83f,-.16f,-.99f,-.78f,-.73f,.71f,.27f,.15f,.19f,.29f,-.20f,.00f,.38f,.84f,-.20f,.28f,.19f,-.02f,-.50f,-.66f,-.23f,.79f,.62f,.20f,-.24f,.60f,-.39f,-.61f,-.53f,-.93f,.99f,.78f,-.92f,.92f,.43f,.46f,.87f,.32f,-.53f,.85f,-.73f,-.71f,-.39f,-.90f,-.63f,.01f,-.38f,-.62f,-.25f,-.37f,.60f,-.01f,.18f,.95f,.99f,-.70f,-.76f,-.15f,-.78f,.02f,.08f,.27f,-.61f,.62f,.59f,-.04f,.46f,-.63f,-.05f,.85f,-.05f,-.84f,-.35f,-.35f,.63f,.55f,-.33f,-.53f,.62f,.64f,-.02f,.06f,-.39f,-.55f,-.09f,-.17f,-.09f,-.84f,.94f,-.08f,-.95f,-.25f,.12f,-.26f,-.94f,.32f,-.60f,-.11f,-.27f,-.32f,.31f,.97f,.64f,.99f,-.99f,-.56f,.96f,.11f,-.57f,-.51f,.82f,-.35f,.13f,.70f,-.97f,.92f,.59f,-.03f,.15f,-.09f,.46f,.28f,.73f,-.17f,.26f,-.23f,.45f,-.96f,.21f,-.28f,-.64f,-.09f,.68f,.49f,-.30f,-.75f,.98f,-.71f,-.42f,-.52f,.19f,.52f,.30f,.92f,.61f,-.53f,.62f,-.15f,.19f,.07f,.67f,.92f,.20f,-.69f,-.29f,-.75f,.54f,.78f,.92f,-.70f,-.70f,-.40f,-.41f,.89f,-.29f,.02f,.34f,-.24f,.48f,.97f,.75f,-.66f,.17f,-.32f,-.41f,.24f,.85f,-.70f,.39f,.46f,-.84f,.86f,.45f,-.78f,-.61f,.74f,.73f,.78f,.66f,-.15f,-.85f,.46f,.46f,-.21f,-.78f,-.16f,.12f,.83f,.05f,-.95f,.41f,.45f,-.02f,.56f,-.91f,-.90f,-.33f,-.94f,-.72f,-.90f,-.22f,-.01f,-.59f,.58f,-.37f,.23f,-.96f,.41f,-.65f,.69f,-.79f,.14f,.39f,-.99f,-.21f,-.73f,-.33f,.53f,-.15f,-.04f,-.00f,.33f,.12f,-.44f,-.88f,-.93f,.05f,.73f,-.66f,-.85f,-.17f,-.47f,.15f,.65f,.91f,-.72f,.32f,.64f,-.30f,-.40f,-.79f,-.77f,.27f,.09f,.65f,-.85f,.37f,.57f,-.79f,.28f,.02f,-.92f,.23f,.66f,-.38f,.24f,-.21f,.84f,-.71f,-.12f,.26f,-.29f,.26f,-.08f,.75f,.54f,.46f,.11f,-.81f,-.53f,.85f,-.24f,-.38f,.95f,.36f,.77f,-.37f,.58f,.91f,.03f,.13f,-.98f,.68f,.74f,.52f,.42f,.83f,-.76f,.52f,-.79f,-.57f,.95f,-.58f,-.11f,-.08f,.57f,.75f,-.80f,.33f,.42f,-.85f,-.74f,-.91f,-.99f,-.08f,-.06f,-.42f,.33f,-.45f,-.09f,.56f,-.95f,.98f,-.24f,.48f,-.85f,-.05f,.73f,.75f,.43f,.00f,-.29f,-.75f,-.39f,-.76f,.35f,.29f,-.19f,-.61f,-.65f,-.35f,.74f,-.12f,.62f,.59f,-.11f,-.36f,-.37f,-.59f,.28f,-.28f,-.00f,.60f,-.40f,.10f,-.94f,-.25f,-.50f,-.12f,-.90f,.13f,-.39f,.30f,-.87f,.76f,-.61f,.55f,.17f,.74f,-.36f,.52f,.71f,-.20f,.07f,.40f,.61f,.06f,.68f,.71f,-.36f,-.52f,-.61f,-.70f,-.40f,.00f,.69f,-.44f,-.15f,.45f,-.63f,.88f,.23f,-.98f,.29f,-.86f,-.24f,.65f,-.61f,-.61f,.81f,.10f,.24f,.84f,-.98f,-.26f,-.17f,-.05f,.88f,-.93f,-.34f,.10f,.89f,-.92f,.88f,-.01f,.30f,.01f,.65f,-.87f,-.51f,.42f,.60f,.02f,.56f,-.56f,-.00f,.63f,.40f,.32f,-.12f,.05f,.61f,-.57f,-.84f,.20f,-.91f,.48f,.64f,-.60f,-.24f,.84f,.21f,-.48f,.26f,-.71f,.31f,.21f,.59f,.90f,-.38f,.56f,-.72f,.22f,.01f,.74f,.19f,-.40f,-.63f,.76f,-.67f,.27f,.09f,-.04f,.49f,.33f,.55f,.73f,-.41f,-.49f,.16f,-.93f,.37f,.62f,.68f,-.51f,.08f,.89f,-.35f,.26f,.77f,.33f,.42f,-.66f,.65f,-.34f,.87f,-.97f,-.16f,.43f,-.93f,-.06f,.00f,.40f,.06f,-.16f,.79f,.46f,-.60f,-.68f,.11f,-.19f,.33f,-.37f,-.94f,-.08f,.11f,.20f,.78f,.34f,-.61f,.60f,-.93f,-.36f,.96f,-.37f,-.79f,-.56f,.86f,.36f,-.68f,-.18f,.54f,.14f,.00f,.23f,.14f,.83f,-.45f,.01f,.59f,-.02f,.43f,-.03f,.97f,.62f,-.12f,.90f,-.90f,-.08f,.36f,-.72f,.02f,.22f,-.24f,.96f,-.33f,-.13f,.44f,-.65f,.95f,.68f,-.76f,.18f,-.06f,.26f,.74f,-.38f,-.08f,.15f,.45f,-.97f,.43f,-.82f,-.84f,.99f,-.55f,-.93f,-.07f,-.29f,-.59f,-.91f,.49f,-.60f,-.56f,.63f,.86f,.49f,.76f,.69f,.12f,.67f,.60f,.12f,-.79f,.78f,.35f,.52f,-.80f,-.12f,.25f,.43f,.69f,.12f,-.01f,.47f,.65f,-.38f,.99f,.28f,-.37f,-.18f,.69f,.56f,.80f,.89f,-.13f,.57f,.45f,-.11f,.62f,.83f,.14f,-.80f,-.12f,-.04f,.25f,.59f,-.17f,.04f,-.75f,.75f,-.56f,-.46f,-.27f,.22f,.53f,.58f,.03f,.44f,-.14f,.26f,-.22f,.74f,.28f,.50f,-.21f,-.51f,-.50f,.73f,-.67f,.19f,-.58f,-.29f,.56f,-.77f,.18f,-.46f,.59f,.80f,.95f,-.66f,-.67f,-.57f,.01f,-.20f,.27f,.67f,-.12f,-.34f,.72f,-.05f,.95f,-.26f,.30f,.87f,-.61f,.91f,-.68f,.20f,-.75f,-.04f,.12f,.37f,-.34f,.06f,-.92f,-.76f,-.04f,-.17f,-.73f,.50f,-.32f,-.77f,-.25f,-.72f,-.32f,-.88f,.97f,.63f,-.05f,.05f,.33f,.46f,.65f,-.38f,.36f,.52f,-.33f,-.09f,.57f,-.44f,-.80f,.18f,-.14f,.83f,.53f,.45f,-.82f,.11f,.24f,.56f,-.47f,-.35f,-.33f,.97f,-.77f,-.42f,.61f,-.45f,.57f,.43f,.69f,.46f,.22f,.10f,-.24f,.52f,.55f,-.97f,.09f,-.52f,.78f,.50f,.87f,-.04f,.21f,.92f,.15f,-.85f,-.64f,.25f,.20f,.35f,.85f,-.95f,-.36f,-.75f,.85f,.62f,.04f,-.87f,.41f,-.07f,-.71f,-.30f,.07f,.37f,-.16f,.39f,-.62f,-.02f,.57f,-.37f,.31f,-.55f,-.41f,-.11f,-.32f,-.19f,.31f,.29f,.44f,.09f,-.54f,.38f,.07f,.10f,-.71f,-.97f,-.48f,-.55f,.49f,-.95f,.92f,.69f,.24f,-.82f,.33f,-.03f,-.20f,-.14f,-.77f,-.16f,.57f,.39f,.84f,-.73f,-.48f,.65f,.33f,.24f,-.56f,.31f,.23f,.44f,-.97f,-.87f,.14f,-.73f,-.02f,.18f,-.09f,-.02f,-.27f,.09f,.96f,-.01f,-.79f,.29f,-.34f,.99f,.87f,-.45f,.55f,-.25f,-.69f,-.75f,-.45f,-.27f,-.00f,-.99f,.26f,.07f,-.60f,-.50f,-.11f,.20f,.01f,-.09f,-.56f,.85f,.96f,.02f,.09f,-.63f,.13f,.61f,.46f,-.97f,.29f,.09f,-.41f,-.73f,.43f,-.49f,-.33f,-.35f,-.36f,.63f,.41f,-.57f,-.39f,.65f,-.03f,-.29f,.06f,.04f,.97f,-.50f,-.54f,.86f,-.41f,.57f,-.12f,.51f,-.41f,.77f,-.89f,-.74f,.45f,.99f,.03f,.96f,.48f,.47f,-.71f,-.85f,.62f,-.83f,-.71f,-.87f,.97f,-.41f,-.20f,-.84f,.55f,-.01f,-.76f,.15f,.55f,.26f,.19f,.70f,.07f,-.56f,-.51f,.48f,-.55f,.14f,.53f,-.53f,-.99f,-.22f,-.61f,-.88f,-.26f,.46f,-.19f,-.22f,.20f,-.02f,.71f,-.82f,-.67f,.80f,.31f,-.58f,-.59f,-.63f,.48f,-.73f,.45f,.49f,.74f,.59f,.72f,-.48f,.18f,-.12f,.30f,.81f,.55f,.97f,-.66f,-.52f,-.99f,.14f,-.62f,-.02f,.90f,.78f,.77f,-.05f,-.06f,.51f,-.26f,.64f,.91f,.66f,.52f,.19f,.21f,-.52f,.96f,.71f,.28f,-.89f,-.01f,.83f,.07f,.39f,-.88f,.28f,.68f,-.81f,-.46f,.42f,.47f,-.43f,-.93f,.43f,-.76f,.15f,.88f,.21f,.30f,-.85f,-.26f,-.12f,-.96f,-.84f,-.97f,-.49f,.64f,.42f,-.50f,.87f,.19f,-.13f,-.46f,-.45f,-.95f,.22f,.41f,.83f,-.14f,-.33f,.87f,.76f,-.06f,-.94f,.72f,.73f,-.78f,.90f,.89f,.84f,.90f,.28f,.88f,.86f,-.62f,.02f,-.81f,-.52f,-.82f,-.14f,-.42f,-.30f,-.02f,.79f,.10f,-.58f,-.00f,.95f,.55f,-.75f,-.09f,.36f,.44f,.30f,.03f,-.49f,-.60f,.95f,.86f,-.67f,-.23f,.28f,-.45f,-.58f,-.52f,.77f,-.19f,-.88f,-.16f,.08f,.82f,-.02f,.45f,-.66f,.73f,.94f,-.55f,-.54f,.02f,.62f,.71f,-.99f,.10f,-.06f,.08f,.79f,-.00f,-.77f,-.47f,.42f,.83f,.21f,.08f,-.43f,-.14f,.22f,.32f,-.03f,.11f,.12f,-.64f,-.58f,-.54f,-.08f,-.38f,.41f,.61f,-.65f,.27f,.98f,-.40f,-.13f,.83f,.74f,-.68f,.22f,-.69f,.42f,.77f,-.87f,-.42f,-.77f,-.11f,.25f,-.69f,.84f,-.25f,-.38f,-.56f,-.20f,-.35f,.83f,-.89f,-.51f,.82f,-.77f,.05f,.78f,.71f,.12f,-.82f,-.32f,.09f,.63f,.70f,-.49f,.80f,.25f,-.85f,.04f,-.72f,.56f,.14f,.62f,.17f,-.88f,-.26f,.20f,.72f,-.55f,-.57f,-.33f,.66f,-.12f,.46f,-.14f,-.87f,-.12f,.08f,.92f,.41f,.36f,-.58f,-.17f,.25f,-.97f,-.87f,-.86f,-.43f,.85f,-.38f,-.45f,.47f,.41f,.98f,-.67f,-.13f,-.36f,.08f,-.33f,.92f,-.96f,-.13f,.71f,.77f,.17f,-.03f,-.92f,-.06f,.00f,.29f,-.14f,-.79f,.64f,.53f,.82f,-.02f,-.39f,.94f,-.97f,-.67f,-.98f,-.38f,-.25f,-.76f,.21f,.80f,.24f,-.69f,-.44f,-.80f,.17f,-.87f,.33f,.72f,-.23f,-.39f,.52f,-.39f,-.14f,-.70f,.86f,-.58f,.05f,.82f,.19f,.58f,-.76f,.00f,.09f,.97f,-.60f,.90f,-.73f,-.47f,-.47f,.18f,.93f,-.53f,.71f,.34f,.48f,-.12f,.94f,.12f,-.44f,-.35f,-.93f,-.00f,-.91f,.40f,.37f,-.87f,-.86f,.68f,.55f,.35f,-.30f,.31f,-.37f,.81f,.89f,-.00f,-.72f,-.24f,-.33f,-.09f,-.79f,.41f,.15f,-.30f,-.99f,-.00f,-.07f,-.55f,.22f,.54f,-.15f,-.45f,-.79f,-.75f,-.50f,-.69f,-.59f,-.67f,.70f,.29f,.74f,.83f,-.83f,.41f,-.04f,-.94f,.59f,-.38f,.85f,.34f,-.06f,.08f,.79f,.41f,-.19f,-.15f,.25f,-.66f,.03f,.57f,-.17f,.69f,.32f,.07f,-.95f,.49f,.42f,-.75f,.70f,-.09f,-.98f,-.93f,.40f,-.38f,.36f,-.49f,.65f,-.30f,.14f,-.99f,.66f,-.27f,.94f,.08f,-.70f,.78f,-.93f,-.97f,-.89f,-.94f,-.77f,-.75f,-.87f,.89f,-.25f,.61f,.59f,-.55f,-.73f,.26f,.93f,-.42f,.66f,-.91f,-.72f,.05f,.26f,-.49f,-.00f,-.88f,-.66f,.34f,.93f,.91f,.94f,.25f,.72f,.13f,-.23f,-.67f,-.99f,-.77f,.45f,.33f,.68f,.34f,.43f,.60f,.36f,-.72f,-.56f,-.41f,-.72f,.65f,.34f,.91f,.95f,-.90f,-.85f,.91f,.80f,-.60f,-.96f,.29f,-.36f,-.54f,.81f,-.95f,-.54f,.67f,.13f,.16f,.73f,.92f,.00f,.08f,-.05f,.84f,-.42f,-.82f,-.31f,-.82f,-.14f,-.65f,-.15f,.55f,.56f,-.93f,-.06f,.59f,.48f,.60f,.08f,-.81f,-.72f,-.69f,-.19f,.02f,.25f,-.38f,-.41f,.31f,-.84f,-.09f,.64f,.62f,-.00f,.78f,-.87f,-.50f,-.85f,.61f,-.90f,.96f,.51f,.99f,-.33f,-.44f,-.23f,-.27f,-.03f,-.75f,.12f,-.78f,-.25f,.44f,.49f,.33f,.33f,-.77f,-.35f,-.39f,-.49f,.00f,-.20f,-.06f,-.78f,.41f,.31f,.62f,-.78f,-.36f,.98f,-.03f,-.41f,-.01f,-.81f,.30f,-.81f,.60f,-.05f,.20f,.70f,.81f,.64f,.78f,.84f,.07f,.86f,.72f,.59f,-.58f,-.85f,.23f,.03f,-.40f,-.29f,-.16f,.59f,.79f,.07f,-.29f,.45f,-.94f,-.48f,-.65f,.57f,.43f,-.31f,-.18f,-.83f,-.85f,-.27f,.28f,.58f,-.07f,.74f,.09f,-.80f,.93f,-.84f,.74f,.37f,.85f,.70f,-.95f,.35f,-.81f,-.54f,.81f,.05f,.02f,.47f,-.70f,.45f,.44f,-.12f,.46f,-.76f,.42f,-.38f,-.47f,-.80f,-.96f,-.97f,-.95f,-.91f,.79f,-.41f,-.87f,.60f,.72f,.19f,.45f,.99f,-.02f,-.70f,-.89f,-.75f,-.54f,.11f,-.69f,.94f,-.86f,-.34f,-.79f,-.75f,.84f,-.56f,.63f,.22f,-.69f,.34f,.34f,.38f,.03f,.43f,-.38f,-.70f,-.34f,-.01f,-.47f,-.88f,.97f,-.99f,.33f,-.79f,-.79f,-.78f,-.14f,-.58f,-.93f,.41f,.14f,.40f,-.06f,.31f,-.69f,-.83f,.29f,.08f,.66f,.71f,.11f,.28f,.98f,.94f,.11f,.91f,.05f,-.07f,.33f,-.97f,.48f,-.77f,.93f,.68f,-.00f,.36f,.37f,-.76f,.51f,.55f,-.92f,.15f,.84f,-.82f,.02f,-.15f,.06f,.05f,-.10f,-.57f,.93f,.03f,-.61f,.47f,-.23f,.39f,-.54f,.33f,-.48f,.45f,.65f,.64f,.35f,.64f,-.39f,.15f,.54f,.97f,-.12f,-.88f,.57f,.95f,.91f,.83f,.64f,-.16f,-.85f,.52f,.03f,.81f,.10f,.47f,.39f,-.66f,.86f,.69f,.29f,-.43f,-.76f,.58f,-.76f,-.89f,-.12f,.37f,.83f,-.02f,.36f,.79f,-.29f,.63f,.18f,-.24f,.62f,-.07f,-.25f,.07f,-.17f,-.60f,-.66f,-.21f,.83f,.04f,-.63f,-.35f,.00f,.22f,.05f,-.18f,.26f,.49f,.51f,.54f,-.43f,.10f,.07f,-.70f,.69f,-.19f,-.24f,.94f,.22f,.16f,-.23f,-.81f,-.72f,.58f,.44f,-.58f,-.89f,.59f,.64f,.72f,.43f,.82f,-.57f,.69f,.27f,-.20f,.24f,.07f,.41f,-.49f,.72f,.27f,-.40f,.11f,-.58f,.78f,-.71f,.97f,.12f,.00f,.76f,.77f,-.70f,-.13f,-.41f,.57f,.42f,.48f,.59f,.47f,.16f,-.10f,.80f,.53f,.50f,-.34f,.78f,-.20f,-.70f,.56f,-.23f,-.11f,-.51f,.44f,.21f,-.59f,.08f,-.43f,-.87f,-.60f,-.35f,.79f,-.62f,-.85f,-.39f,-.39f,-.74f,.80f,-.19f,-.93f,.55f,-.26f,-.86f,-.62f,-.00f,-.20f,-.11f,.95f,-.27f,.59f,.61f,-.78f,-.64f,-.09f,-.51f,.10f,.83f,-.22f,.37f,.40f,-.74f,-.68f,-.22f,-.36f,-.82f,.29f,.29f,-.04f,.88f,-.62f,.49f,.89f,-.87f,.30f,.90f,.54f,.07f,-.52f,-.82f,.65f,.28f,.73f,.43f,.34f,.24f,-.61f,-.20f,-.07f,.85f,.34f,-.33f,.14f,.64f,-.58f,.32f,-.46f,.07f,.88f,.64f,.12f,-.26f,-.11f,.72f,-.58f,-.58f,-.95f,.38f,.11f,-.01f,.15f,-.10f,.50f,.66f,.60f,-.81f,.75f,.48f,.65f,-.83f,-.81f,-.92f,-.31f,.81f,-.35f,.49f,.95f,-.99f,.17f,.34f,.60f,.65f,-.49f,-.18f,-.09f,.35f,.06f,.92f,.78f,-.18f,-.66f,.46f,.58f,-.56f,-.45f,.63f,.03f,.28f,-.84f,.39f,.06f,.85f,.04f,-.50f,.53f,.62f,.46f,-.86f,-.90f,-.72f,.12f,.05f,-.39f,-.41f,.59f,-.19f,-.22f,.50f,.32f,.83f,-.33f,-.83f,-.81f,-.46f,-.63f,-.58f,-.78f,-.49f,-.78f,.19f,.69f,-.66f,.27f,-.10f,-.09f,.88f,.08f,.53f,-.64f,-.38f,.46f,-.64f,-.55f,.33f,-.41f,-.80f,.11f,.37f,-.05f,-.73f,-.57f,.93f,.90f,-.60f,.93f,.81f,.94f,.23f,.06f,.68f,-.06f,-.91f,-.05f,.40f,-.11f,-.83f,-.67f,.96f,-.89f,-.56f,.96f,.82f,-.29f,.80f,.42f,.23f,-.05f,-.06f,.16f,-.54f,.08f,.69f,.97f,.19f,-.65f,.87f,.70f,.67f,.44f,.10f,-.12f,.97f,.07f,.92f,-.00f,-.58f,-.44f,.66f,-.63f,-.32f,.08f,.53f,-.11f,.80f,-.35f,.98f,-.66f,-.63f,-.80f,.87f,-.82f,.09f,.95f,-.25f,-.15f,.72f,-.97f,-.45f,.96f,.49f,.92f,-.19f,-.10f,-.12f,-.00f,-.96f,.76f,.99f,-.37f,.84f,.66f,-.63f,-.41f,.37f,-.29f,.69f,.61f,.54f,-.95f,.97f,.76f,.02f,.06f,.83f,-.94f,-.77f,.04f,-.25f,.57f,.75f,.46f,-.96f,-.49f,.76f,.35f,-.80f,-.74f,.52f,-.57f,-.38f,-.75f,-.88f,.76f,-.69f,.22f,-.64f,.65f,.79f,-.79f,.78f,-.13f,.07f,-.98f,.63f,.94f,-.54f,.52f,.02f,-.64f,-.05f,.04f,-.91f,-.05f,-.54f,.67f,.44f,-.43f,-.37f,.03f,.37f,-.51f,-.36f,.21f,.65f,-.62f,.20f,-.90f,-.37f,-.11f,-.40f,-.11f,.50f,.81f,.46f,.03f,-.09f,.05f,-.20f,.98f,.45f,.26f,-.15f,-.40f,.70f,-.17f,-.59f,-.23f,-.80f,.65f,-.47f,.39f,.68f,-.64f,.15f,-.89f,.98f,-.36f,.15f,-.59f,.88f,.47f,-.01f,.26f,-.94f,.39f,-.76f,-.43f,-.29f,-.88f,-.52f,-.42f,.12f,-.56f,.09f,.38f,.32f,-.22f,-.29f,-.46f,.59f,-.96f,.30f,.95f,-.76f,-.90f,-.76f,.56f,.35f,-.76f,.42f,-.72f,-.67f,.74f,-.60f,.90f,.74f,-.87f,-.36f,-.12f,.29f,-.83f,.60f,.35f,.44f,-.50f,.19f,.19f,.42f,-.47f,-.50f,-.35f,.49f,.41f,-.19f,-.08f,-.70f,.76f,-.44f,.69f,.16f,-.01f,.56f,-.85f,-.11f,-.87f,.11f,.17f,-.05f,.50f,.24f,-.47f,.57f,.59f,-.90f,.95f,-.05f,.02f,.51f,.82f,.28f,.91f,-.59f,.65f,-.42f,-.82f,.84f,-.31f,-.41f,.75f,.68f,-.67f,.52f,-.18f,.79f,-.38f,.63f,.33f,-.93f,-.59f,.76f,-.60f,.03f,-.77f,-.34f,.17f,-.35f,.81f,.50f,.82f,.57f,.61f,-.38f,.69f,.69f,.75f,-.61f,-.73f,.53f,.66f,-.83f,-.69f,.67f,-.41f,.84f,-.23f,.70f,.95f,.66f,.81f,.64f,-.55f,-.80f,.45f,-.11f,-.77f,-.47f,-.36f,-.41f,-.82f,-.60f,-.32f,-.58f,.49f,.32f,-.38f,.35f,-.88f,-.11f,.49f,.24f,.97f,.68f,.90f,.62f,-.40f,.94f,-.16f,-.57f,-.00f,-.18f,.55f,-.88f,.95f,-.49f,-.75f,.54f,.45f,.03f,.59f,.17f,-.22f,-.44f,-.63f,.69f,.08f,.06f,-.43f,.00f,-.50f,-.86f,-.48f,.91f,-.19f,-.18f,-.61f,.85f,-.41f,-.73f,.83f,-.86f,.66f,.94f,-.02f,-.07f,.21f,-.68f,-.33f,-.40f,.13f,-.11f,.59f,-.91f,-.28f,.72f,.18f,.46f,-.20f,-.36f,-.93f,-.38f,.91f,-.03f,-.85f,-.05f,.23f,-.03f,-.28f,.68f,.93f,.07f,.69f,-.76f,-.55f,.42f,-.27f,-.28f,.39f,-.02f,.95f,-.43f,.38f,.70f,.93f,-.55f,-.25f,-.72f,-.44f,.97f,-.72f,-.24f,-.66f,-.99f,-.29f,-.93f,.97f,.64f,.65f,.95f,.58f,.13f,.18f,.42f,-.38f,.11f,.60f,.30f,-.01f,.79f,-.31f,-.70f,.23f,-.17f,-.58f,-.82f,.13f,-.75f,-.44f,.99f,.26f,-.55f,.12f,-.40f,.85f,-.93f,.28f,-.78f,-.94f,.99f,-.71f,.55f,-.30f,.20f,.33f,-.84f,-.93f,-.49f,-.83f,-.30f,-.44f,-.14f,.96f,-.71f,-.68f,.60f,.74f,-.86f,-.10f,-.04f,.07f,-.30f,.28f,-.10f,.15f,-.34f,.94f,.71f,.17f,-.48f,-.53f,.93f,.73f,.29f,-.17f,-.34f,-.61f,-.93f,.94f,-.14f,.27f,.49f,.78f,-.69f,.36f,.38f,.49f,-.93f,.83f,-.85f,.51f,.43f,.61f,.40f,-.95f,-.81f,.09f,-.87f,-.42f,-.85f,.34f,-.43f,.72f,-.40f,-.51f,-.39f,-.50f,-.74f,.91f,.49f,.92f,.84f,.68f,-.63f,-.77f,.25f,.63f,-.60f,.32f,-.94f,.69f,-.81f,.55f,.07f,.58f,-.76f,-.82f,.53f,-.87f,-.97f,-.92f,-.19f,.12f,-.70f,.05f,.99f,-.88f,-.76f,-.81f,-.81f,.94f,-.22f,.44f,.10f,-.09f,.34f,-.72f,-.19f,-.44f,.29f,.74f,.45f,.77f,.36f,.37f,-.59f,.46f,.41f,-.86f,-.10f,-.42f,.66f,.11f,.50f,.17f,.78f,.92f,.93f,.64f,.61f,.53f,-.51f,.75f,-.89f,-.01f,.50f,-.27f,-.40f,.06f,.66f,-.02f,-.70f,-.13f,.03f,-.72f,-.64f,.67f,.12f,-.97f,.97f,-.74f,-.56f,-.11f,.81f,-.13f,-.32f,.06f,.10f,.77f,-.80f,-.68f,-.14f,-.53f,-.70f,-.18f,-.37f,-.61f,.12f,.65f,.40f,-.16f,-.92f,.92f,.94f,-.20f,-.24f,.33f,-.75f,.83f,.67f,-.10f,-.18f,.84f,-.23f,-.80f,.77f,-.80f,.60f,-.51f,.64f,-.63f,.89f,-.61f,.00f,.43f,.86f,.23f,-.51f,.05f,-.23f,.05f,.32f,.36f,.78f,.12f,-.52f,.52f,-.86f,-.37f,-.63f,.50f,-.82f,-.93f,.49f,-.20f,-.35f,.20f,-.03f,.09f,.01f,.26f,.14f,.82f,.15f,.26f,-.83f,-.22f,.36f,.26f,-.40f,.90f,.87f,-.96f,.87f,.73f,.67f,.08f,.46f,.59f,.59f,.82f,.99f,-.13f,.18f,-.04f,.60f,-.60f,.84f,-.80f,-.54f,.40f,-.57f,-.43f,-.08f,-.09f,.62f,.58f,.89f,-.38f,.07f,-.72f,-.15f,.20f,.34f,-.81f,-.04f,-.06f,.77f,-.95f,-.99f,-.05f,-.13f,-.37f,-.21f,-.31f,-.77f,-.82f,.67f,.71f,.03f,.98f,.46f,-.87f,.55f,-.14f,.14f,.26f,.05f,.31f,.67f,-.00f,-.49f,-.71f,-.75f,-.66f,.27f,-.29f,.04f,-.91f,.02f,-.68f,-.03f,-.47f,.34f,-.89f,-.98f,-.00f,.11f,-.52f,.28f,-.42f,.60f,.28f,.42f,.91f,-.09f,.23f,-.44f,-.39f,-.20f,.61f,.82f,-.43f,.57f,.15f,-.03f,-.71f,-.19f,.56f,.54f,.99f,-.59f,-.76f,-.90f,.05f,.00f,-.56f,-.85f,-.74f,.31f,-.92f,.38f,.13f,.44f,.94f,.95f,.35f,.18f,.35f,-.16f,.15f,.21f,-.90f,.80f,.50f,-.68f,-.20f,-.99f,.20f,.71f,.65f,.46f,.21f,.19f,-.64f,.79f,-.23f,.69f,-.71f,.29f,-.55f,.61f,-.31f,.18f,-.00f,.40f,-.89f,-.38f,-.08f,-.43f,-.87f,.53f,-.05f,.55f,.80f,-.60f,-.87f,-.15f,-.22f,-.55f,-.41f,-.43f,.69f,.32f,-.10f,-.29f,.26f,.88f,-.86f,-.16f,-.36f,-.94f,.47f,-.75f,-.15f,-.10f,.97f,.93f,.39f,.29f,.12f,.49f,-.38f,.22f,.24f,-.33f,.18f,.37f,-.80f,-.77f,.28f,-.26f,.05f,.60f,-.85f,-.14f,.90f,-.44f,.36f,-.85f,-.89f,.36f,-.93f,.44f,-.95f,-.18f,.62f,-.50f,-.58f,-.08f,.84f,.33f,-.96f,.92f,.41f,-.61f,.90f,.09f,-.70f,-.73f,-.64f,-.09f,-.09f,-.42f,-.90f,.61f,.71f,.82f,-.42f,.56f,.97f,.64f,-.49f,-.22f,-.94f,.24f,-.29f,-.88f,-.54f,.97f,-.76f,.66f,-.71f,-.64f,-.64f,-.53f,.20f,.23f,-.56f,-.62f,-.55f,.18f,.18f,-.77f,.19f,-.68f,.89f,.50f,.42f,-.10f,-.28f,.41f,-.39f,-.51f,.70f,-.98f,.44f,.04f,-.46f,-.63f,-.93f,-.31f,.41f,.35f,.53f,.77f,.60f,.26f,-.21f,.82f,-.92f,.92f,-.70f,-.10f,.64f,-.83f,-.55f,.61f,.77f,-.72f,-.30f,-.85f,.33f,-.60f,.19f,.86f,.70f,.77f,.21f,.86f,.04f,.22f,.95f,.57f,.36f,-.41f,.37f,-.17f,.29f,.70f,-.40f,-.39f,-.83f,-.11f,.84f,-.43f,-.48f,.13f,-.27f,-.75f,-.54f,.79f,.75f,-.85f,.90f,-.41f,.24f,.29f,.61f,.69f,-.03f,-.72f,-.36f,-.33f,-.32f,-.11f,.19f,-.84f,-.24f,-.77f,-.96f,.70f,.04f,.95f,-.97f,-.11f,-.24f,.69f,-.13f,.85f,.43f,-.41f,-.89f,.70f,.09f,-.63f,.57f,.80f,.56f,-.62f,.32f,-.28f,.54f,.14f,-.67f,-.24f,-.79f,.23f,-.95f,-.16f,-.85f,-.18f,.46f,-.60f,.36f,.22f,-.08f,.15f,.62f,-.20f,.88f,-.26f,-.13f,-.11f,.56f,.17f,.67f,-.56f,.49f,.87f,-.35f,-.73f,-.93f,-.80f,-.06f,-.95f,-.36f,-.26f,.34f,.19f,.57f,.56f,-.65f,.60f,.94f,.86f,-.16f,-.95f,.36f,.12f,-.42f,.34f,.75f,-.14f,.15f,.03f,-.17f,.70f,.95f,.95f,.79f,.11f,-.82f,-.30f,.75f,-.72f,-.91f,.31f,-.07f,.42f,.85f,.00f,-.47f,.37f,-.62f,.22f,.10f,.75f,-.15f,-.13f,.18f,-.21f,.45f,-.35f,.77f,-.40f,-.75f,.59f,.87f,-.62f,-.91f,.55f,-.09f,.06f,.30f,.52f,-.51f,-.03f,.89f,-.05f,.88f,.83f,-.66f,-.01f,.34f,-.83f,.60f,.47f,.65f,.51f,.70f,.62f,.99f,.34f,.41f,-.31f,.75f,.61f,.04f,.54f,-.42f,.47f,-.43f,.39f,-.58f,-.44f,-.02f,-.08f,.38f,-.77f,-.55f,-.20f,.63f,-.93f,-.96f,-.04f,-.26f,.83f,.34f,.31f,-.32f,.37f,-.74f,.60f,-.77f,-.04f,.65f,.94f,-.39f,.01f,-.76f,-.25f,.44f,.87f,.06f,-.58f,-.21f,.67f,-.97f,.22f,.24f,-.16f,.82f,-.19f,.66f,.00f,.50f,-.85f,.32f,-.66f,.11f,-.48f,.66f,-.60f,-.50f,.29f,-.93f,.94f,-.02f,.41f,-.13f,-.04f,.77f,.31f,-.23f,.84f,.25f,-.48f,-.51f,.19f,-.42f,-.03f,-.13f,.78f,.76f,.23f,.63f,.23f,-.82f,.41f,.10f,.73f,.49f,-.94f,.80f,-.40f,.78f,-.12f,-.49f,-.80f,.37f,.96f,.22f,-.93f,.78f,-.74f,-.71f,.46f,-.38f,.86f,.03f,.48f,.22f,-.05f,.13f,.91f,-.20f,.36f,.64f,.26f,.38f,-.10f,.57f,-.14f,-.72f,.08f,.55f,-.91f,.62f,.27f,-.49f,-.22f,.22f,-.12f,-.32f,-.28f,.15f,-.92f,.58f,-.51f,-.07f,-.45f,-.24f,-.73f,.63f,-.70f,-.11f,-.46f,-.75f,-.58f,-.15f,.38f,-.22f,.23f,-.26f,.41f,.14f,.87f,.44f,.26f,.79f,.28f,.19f,.06f,.84f,.19f,.91f,-.25f,.24f,.07f,-.46f,-.57f,.86f,.98f,-.33f,.77f,.04f,.55f,-.04f,-.72f,-.44f,.36f,-.73f,.74f,-.86f,.71f,.89f,-.94f,-.03f,.07f,.12f,-.14f,-.20f,-.71f,-.51f,-.23f,-.35f,-.89f,.75f,-.95f,-.48f,-.34f,-.16f,.45f,.34f,-.50f,-.82f,.12f,.18f,.35f,-.26f,-.16f,-.98f,.80f,.58f,-.65f,-.17f,.77f,.80f,-.06f,-.19f,-.43f,-.88f,.08f,.23f,-.70f,-.69f,-.76f,-.11f,.44f,-.87f,.70f,-.73f,-.55f,.05f,.30f,-.02f,.19f,.20f,.14f,.08f,.16f,.44f,.58f,-.51f,-.63f,.32f,.98f,-.48f,-.66f,-.00f,-.83f,.39f,-.95f,-.32f,-.28f,.60f,.59f,-.94f,.34f,-.68f,-.62f,.46f,-.87f,.25f,-.00f,.12f,.38f,.69f,-.54f,.11f,.03f,-.76f,.92f,.52f,-.33f,-.36f,-.33f,.46f,-.60f,.33f,-.56f,-.77f,.13f,.31f,-.65f,-.67f,-.58f,.54f,.41f,.80f,-.92f,.77f,-.44f,.69f,-.60f,.13f,-.82f,.13f,.27f,-.36f,-.88f,-.77f,-.72f,-.94f,.69f,.18f,-.29f,-.12f,-.81f,-.51f,.72f,-.03f,.32f,-.25f,-.83f,.11f,.14f,-.76f,-.62f,.51f,.87f,.76f,-.23f,.52f,.89f,-.02f,-.57f,.81f,-.83f,-.10f,-.88f,.71f,.30f,-.66f,-.38f,-.56f,.37f,.02f,.08f,-.88f,-.86f,.01f,-.61f,-.44f,.60f,.34f,-.90f,-.50f,-.11f,.52f,-.76f,.04f,-.53f,-.10f,.03f,.07f,.32f,.98f,.28f,.13f,.05f,.62f,-.15f,-.39f,.04f,-.81f,.84f,-.48f,.56f,-.74f,.39f,.79f,-.40f,-.67f,-.52f,.67f,-.68f,-.55f,-.65f,.36f,.27f,-.14f,-.14f,-.58f,.34f,.14f,-.33f,-.67f,.92f,-.73f,-.29f,.96f,-.43f,-.85f,.96f,.65f,.21f,.76f,-.74f,.42f,-.78f,-.06f,.21f,-.03f,-.83f,-.42f,.44f,-.79f,-.55f,-.22f,-.54f,-.24f,.68f,.03f,-.90f,.81f,.68f,.70f,-.84f,-.41f,-.26f,-.67f,.21f,-.49f,.00f,-.99f,.73f,.96f,-.50f,.82f,.83f,-.41f,.83f,-.48f,-.20f,.84f,-.79f,-.93f,.89f,-.24f,.59f,-.10f,.95f,.43f,.44f,-.81f,.61f,-.49f,-.78f,-.97f,.83f,-.33f,.26f,.46f,-.74f,-.64f,.49f,.38f,.92f,.70f,.51f,-.64f,.57f,.55f,.82f,-.91f,-.75f,-.50f,-.39f,-.79f,-.96f,-.72f,-.96f,.78f,-.81f,.07f,-.08f,-.34f,-.90f,.26f,.88f,-.69f,-.64f,-.41f,.46f,.93f,-.60f,-.81f,-.11f,-.50f,-.30f,-.27f,.59f,-.02f,-.71f,-.78f,.72f,.11f,-.84f,-.84f,.52f,.45f,-.90f,-.64f,-.38f,.32f,-.20f,.20f,.33f,-.61f,.79f,.33f,-.75f,-.21f,-.04f,.65f,-.27f,.59f,.60f,-.49f,-.38f,.72f,.87f,-.36f,-.97f,.13f,-.62f,-.86f,-.91f,-.52f,.45f,-.41f,-.38f,-.29f,-.07f,-.69f,-.86f,.46f,.60f,.37f,.14f,-.12f,-.75f,.64f,-.74f,.00f,-.91f,-.44f,-.75f,.94f,.91f,-.12f,.76f,.25f,-.75f,-.42f,-.71f,.41f,.55f,-.99f,.12f,.83f,.98f,-.45f,.16f,-.89f,-.38f,-.36f,-.89f,.36f,.27f,.88f,.64f,.31f,-.51f,.48f,-.31f,-.94f,.83f,-.51f,.51f,-.32f,.45f,-.34f,.62f,-.88f,.37f,.48f,-.25f,-.87f,-.40f,-.35f,-.60f,-.48f,-.62f,-.14f,.16f,-.60f,-.95f,.70f,.57f,.38f,-.42f,-.76f,-.37f,-.13f,.67f,.46f,-.54f,.62f,.87f,.93f,-.72f,.56f,.83f,-.24f,-.34f,-.52f,.65f,-.64f,.10f,-.02f,-.63f,-.59f,-.90f,-.98f,.28f,.52f,-.41f,.27f,-.97f,.53f,.29f,-.58f,-.44f,.87f,.32f,-.02f,-.49f,.33f,.61f,.93f,-.95f,.65f,.37f,-.42f,-.82f,.95f,-.96f,.72f,-.14f,-.07f,.03f,.20f,.87f,.52f,-.40f,-.72f,-.95f,.49f,-.29f,.17f,.93f,.04f,-.55f,.37f,.92f,-.99f,.94f,.24f,-.90f,-.60f,-.87f,.48f,.19f,-.27f,.12f,-.50f,-.97f,.85f,-.02f,.68f,.03f,.30f,-.53f,.66f,.38f,.72f,.12f,-.62f,-.46f,-.61f,-.89f,.24f,-.80f,.13f,-.52f,.45f,-.02f,.13f,.03f,.02f,.22f,.50f,.70f,.46f,.40f,-.00f,.49f,-.96f,.75f,-.31f,.60f,.06f,-.57f,.20f,-.79f,-.70f,.08f,.71f,.89f,.75f,-.65f,.50f,-.89f,.15f,.05f,.95f,.77f,-.03f,-.52f,.52f,.60f,.00f,-.92f,.34f,.38f,-.69f,.26f,-.74f,.29f,.39f,.18f,-.06f,.19f,-.67f,-.79f,.96f,-.63f,-.55f,-.86f,-.29f,.68f,-.57f,-.10f,-.87f,.22f,.70f,.09f,.17f,.27f,-.30f,-.53f,.50f,.80f,.07f,.09f,-.85f,.07f,.52f,-.71f,-.71f,.13f,-.57f,.39f,-.45f,.45f,.93f,-.82f,.09f,.94f,-.77f,-.58f,-.46f,-.40f,.47f,-.85f,-.92f,-.01f,-.35f,.33f,-.74f,.79f,.10f,-.24f,.07f,.97f,-.67f,-.69f,-.64f,-.31f,-.19f,.39f,-.73f,.18f,.10f,.14f,-.97f,-.37f,-.36f,-.33f,.64f,-.16f,-.60f,-.78f,.84f,-.77f,-.40f,.68f,-.49f,.13f,.39f,.98f,-.35f,.05f,-.97f,-.12f,.93f,.88f,.33f,.02f,.17f,.80f,-.71f,.98f,.92f,-.97f,.75f,.48f,.22f,.74f,-.51f,.67f,-.87f,.44f,.84f,.28f,-.04f,.41f,.33f,-.68f,.52f,.55f,-.81f,-.63f,.57f,-.48f,-.29f,-.51f,.76f,-.78f,-.39f,-.15f,-.24f,-.01f,-.37f,-.87f,.64f,.07f,.19f,-.96f,.77f,.48f,.95f,.27f,-.55f,-.47f,.00f,-.99f,-.89f,-.31f,.05f,-.08f,-.56f,.30f,.41f,-.63f,.44f,.80f,.11f,.14f,-.62f,-.41f,-.94f,-.64f,.44f,-.49f,-.89f,.42f,-.82f,-.97f,-.83f,-.41f,.68f,.89f,.40f,.09f,.28f,.09f,.01f,.32f,.33f,-.31f,.29f,-.57f,.26f,.70f,.28f,.34f,.12f,-.39f,.77f,.83f,.30f,.52f,-.97f,.80f,-.94f,-.53f,-.33f,-.38f,-.25f,-.20f,-.22f,.37f,-.64f,.95f,-.26f,-.46f,-.97f,-.86f,-.73f,-.19f,.62f,.45f,.33f,-.14f,.87f,-.41f,.01f,-.67f,-.14f,.59f,-.29f,.20f,-.33f,.19f,.50f,.68f,.60f,.21f,.91f,.29f,-.77f,.48f,.54f,-.72f,.26f,.94f,-.82f,.26f,.08f,-.69f,.20f,.66f,.65f,-.77f,.22f,-.17f,-.31f,.56f,.86f,.53f,-.06f,.73f,-.52f,.27f,.99f,-.73f,-.31f,-.09f,.74f,-.15f,-.14f,-.70f,.86f,-.83f,-.02f,-.18f,-.53f,.53f,-.23f,.94f,-.23f,-.32f,.23f,-.73f,-.43f,.06f,-.75f,-.88f,.43f,.50f,-.74f,-.17f,.03f,.74f,-.18f,-.97f,-.09f,.88f,.63f,-.88f,-.42f,.03f,-.19f,.54f,-.26f,.30f,-.13f,.74f,.36f,.05f,-.31f,.77f,-.24f,-.95f,-.52f,.95f,.23f,.87f,-.09f,.08f,.24f,.16f,.31f,.88f,.48f,-.12f,.30f,-.42f,-.10f,.44f,-.91f,-.43f,-.95f,.62f,.36f,.91f,-.74f,.74f,-.08f,.55f,.12f,-.14f,.82f,.22f,.77f,-.11f,.81f,.29f,-.52f,-.47f,.75f,-.08f,.23f,.45f,.87f,.48f,-.41f,.09f,.46f,.71f,.73f,.23f,-.87f,-.90f,-.38f,-.72f,.23f,.36f,.64f,-.87f,-.82f,-.97f,.96f,.97f,-.63f,-.92f,.73f,.29f,-.54f,-.20f,-.96f,-.15f,.98f,-.50f,-.42f,.29f,-.21f,.71f,-.71f,.58f,.12f,-.61f,-.37f,-.28f,-.00f,.66f,-.80f,-.29f,-.83f,-.15f,.70f,.27f,-.37f,.02f,-.28f,.85f,.14f,.26f,.06f,-.72f,.07f,.33f,.18f,-.56f,.66f,.51f,-.21f,.41f,-.69f,-.38f,.42f,-.04f,.71f,.66f,-.26f,.28f,.74f,.61f,.22f,.47f,-.44f,.49f,-.26f,.61f,-.24f,.07f,-.63f,-.24f,-.44f,-.41f,.74f,-.27f,.60f,.47f,-.69f,.13f,-.10f,.36f,-.54f,-.05f,-.26f,.50f,-.62f,.19f,-.05f,.56f,-.12f,.33f,.60f,-.50f,-.40f,.12f,-.06f,.95f,.06f,.60f,.00f,.32f,-.73f,-.05f,.42f,.24f,-.51f,.37f,-.86f,.49f,-.35f,.07f,.21f,-.03f,.13f,-.03f,-.19f,.48f,-.29f,-.56f,-.35f,.57f,-.98f,-.21f,.50f,.95f,.51f,-.40f,.77f,.26f,.77f,.71f,-.28f,.98f,-.78f,.55f,-.96f,-.30f,-.06f,.72f,.79f,.32f,.46f,-.42f,.38f,-.22f,.82f,.65f,.63f,-.92f,-.99f,.07f,-.28f,-.48f,-.22f,.29f,.61f,.04f,.76f,.77f,-.89f,-.84f,-.18f,.27f,-.14f,.13f,.75f,-.53f,.88f,-.46f,.22f,-.57f,-.19f,-.11f,-.96f,.16f,.69f,.37f,.06f,.59f,.21f,-.74f,.53f,.74f,.68f,-.60f,.81f,-.74f,-.82f,-.84f,-.64f,-.09f,.31f,.54f,.34f,.86f,.30f,-.72f,.25f,.51f,.62f,.47f,-.67f,.33f,-.68f,.63f,.84f,.91f,-.85f,-.24f,-.48f,-.40f,.29f,.24f,-.76f,.27f,.60f,-.51f,-.27f,-.99f,.58f,.87f,.58f,-.42f,.18f,.97f,-.42f,-.96f,-.30f,-.98f,-.05f,.98f,.35f,-.82f,-.55f,.87f,.13f,.13f,-.14f,-.61f,-.75f,-.58f,.24f,-.33f,-.79f,.95f,.19f,.01f,.54f,-.40f,-.60f,.08f,-.59f,.71f,-.98f,-.94f,.73f,.46f,-.54f,.23f,.49f,-.26f,.26f,.63f,-.32f,-.34f,-.14f,.01f,.68f,-.03f,.53f,.40f,-.90f,.70f,-.13f,.31f,.12f,-.57f,.52f,-.55f,.98f,-.71f,-.75f,-.71f,-.23f,.54f,.19f,.78f,-.41f,-.81f,-.34f,-.03f,-.78f,-.71f,-.98f,.69f,.08f,.16f,.64f,-.23f,-.71f,.35f,-.53f,-.45f,-.01f,-.99f,.38f,-.20f,-.62f,.56f,.82f,.98f,.26f,.00f,.36f,.29f,-.78f,-.88f,.46f,.00f,.13f,-.22f,.73f,-.42f,.67f,-.75f,.76f,.72f,.95f,.46f,-.81f,-.98f,-.38f,-.05f,-.02f,.77f,-.53f,-.53f,-.68f,-.98f,.07f,-.08f,.40f,-.59f,-.03f,-.13f,.70f,.07f,-.82f,.06f,-.09f,.66f,.15f,.73f,.39f,-.84f,-.33f,-.48f,-.81f,-.01f,-.43f,.42f,.55f,.31f,-.78f,-.15f,-.04f,.81f,.26f,-.38f,.00f,.54f,.21f,-.60f,-.75f,.92f,-.60f,-.51f,.95f,-.37f,-.81f,.44f,.68f,.71f,-.37f,.61f,.29f,-.27f,-.96f,-.14f,-.06f,-.74f,-.09f,.51f,-.89f,.47f,-.75f,.40f,.50f,.45f,-.68f,.77f,.31f,-.50f,-.45f,.52f,.97f,.05f,-.91f,.61f,-.36f,-.26f,-.36f,.47f,-.34f,.97f,-.45f,.19f,-.08f,.39f,.35f,-.28f,.81f,-.73f,.37f,-.39f,.15f,-.79f,-.73f,.37f,-.65f,.72f,-.47f,.57f,-.57f,-.82f,-.87f,-.62f,-.92f,.92f,.85f,-.32f,.04f,-.79f,.96f,.77f,.13f,-.19f,.44f,-.10f,-.65f,-.49f,.33f,.72f,.96f,.13f,-.66f,-.78f,.76f,-.44f,-.69f,.87f,.67f,-.34f,.87f,.92f,.02f,-.28f,-.21f,-.56f,.36f,.96f,.53f,-.19f,-.15f,-.73f,.39f,.53f,-.78f,.26f,-.56f,.17f,.69f,.20f,-.08f,.45f,.19f,.51f,-.96f,-.42f,.59f,.16f,.48f,-.75f,-.71f,.11f,-.24f,-.31f,-.67f,-.96f,-.05f,-.53f,.62f,.81f,.37f,.07f,.80f,.40f,-.38f,-.84f,-.57f,.96f,-.23f,.62f,-.88f,.49f,.49f,.37f,.82f,-.83f,-.00f,.93f,-.25f,.13f,-.69f,.87f,.94f,.71f,.58f,.51f,.97f,-.44f,-.76f,.48f,.01f,.11f,-.57f,.53f,-.85f,-.45f,.31f,-.02f,-.27f,.39f,.86f,-.42f,-.67f,-.34f,.39f,.42f,-.08f,.25f,.01f,-.45f,-.11f,.65f,-.89f,-.44f,.54f,.66f,.53f,-.96f,-.09f,.57f,.20f,-.16f,.90f,-.02f,.92f,.67f,-.06f,.51f,-.54f,-.79f,.08f,.89f,-.96f,.77f,-.83f,-.72f,.95f,.06f,.38f,.80f,.20f,.17f,-.30f,.76f,-.39f,-.81f,-.59f,-.66f,.35f,.01f,-.53f,.07f,.94f,.88f,.97f,-.18f,.14f,.41f,-.83f,-.00f,.59f,-.05f,.03f,.84f,-.17f,-.32f,-.79f,-.86f,-.55f,-.09f,.57f,.01f,.80f,-.11f,.68f,.92f,-.57f,.60f,-.07f,-.10f,.66f,.47f,.62f,.11f,-.50f,.01f,-.03f,.84f,-.15f,-.45f,-.07f,.19f,.94f,-.10f,-.46f,-.47f,-.78f,.05f,.70f,-.96f,.88f,-.84f,.72f,-.40f,.46f,.44f,.31f,.10f,.36f,.42f,.42f,-.29f,-.87f,.16f,-.24f,.62f,.05f,.13f,-.11f,.80f,-.23f,.18f,.43f,-.91f,.75f,.38f,.84f,.61f,-.83f,-.34f,-.01f,.05f,-.39f,.05f,-.64f,.62f,-.29f,.56f,-.06f,.23f,-.39f,.18f,.44f,.04f,.36f,.89f,.93f,.24f,.42f,.25f,-.45f,-.63f,-.22f,.09f,-.27f,.27f,.28f,-.12f,-.97f,-.20f,.55f,-.43f,-.35f,-.64f,.63f,.42f,-.93f,-.76f,-.98f,.90f,.80f,-.64f,-.92f,.34f,-.81f,.79f,-.77f,.56f,.60f,-.05f,-.63f,-.32f,.51f,-.52f,-.20f,.38f,.80f,-.63f,-.52f,-.13f,-.06f,.01f,-.66f,.14f,-.45f,.78f,.07f,.00f,.17f,-.46f,.47f,-.10f,.31f,-.16f,-.62f,-.32f,-.85f,.24f,-.01f,-.63f,.80f,.92f,.54f,-.85f,-.74f,-.93f,-.98f,.54f,-.23f,-.36f,.20f,.75f,.22f,.71f,-.07f,-.13f,.40f,-.82f,.46f,-.39f,.34f,-.35f,.12f,-.42f,.63f,.68f,.04f,.97f,-.04f,-.38f,.87f,.12f,.02f,-.71f,-.81f,-.43f,.56f,.80f,.88f,.51f,-.78f,-.52f,.35f,-.52f,.21f,-.51f,.19f,-.05f,.64f,.72f,-.69f,.45f,-.94f,.41f,-.80f,-.17f,.67f,-.41f,.08f,.15f,-.55f,.13f,-.25f,-.85f,.77f,.47f,-.44f,.39f,-.71f,-.62f,.65f,.35f,.20f,.25f,.62f,-.53f,-.97f,-.36f,-.12f,-.93f,-.46f,.39f,.07f,.06f,-.88f,.07f,.01f,-.95f,-.02f,.52f,.41f,-.88f,.55f,.99f,.11f,.76f,-.90f,.97f,-.25f,-.92f,.65f,.94f,.71f,-.82f,-.46f,.44f,.21f,.84f,-.49f,.55f,-.20f,.45f,.48f,-.58f,.96f,.52f,.01f,.31f,-.67f,.52f,-.66f,-.77f,.18f,.36f,-.32f,.51f,.23f,.58f,-.45f,-.52f,.22f,-.46f,.79f,.49f,.40f,.81f,-.44f,.79f,-.50f,.90f,.13f,.34f,.93f,.31f,.56f,-.83f,.90f,-.18f,.55f,-.05f,-.82f,.16f,.67f,.08f,-.82f,-.04f,-.05f,.69f,.75f,.74f,-.37f,.09f,.82f,-.32f,.16f,.67f,.50f,.37f,.63f,-.65f,.28f,.34f,.09f,.41f,-.64f,.57f,.80f,-.11f,-.43f,-.16f,-.29f,-.79f,-.52f,-.54f,-.22f,-.94f,-.77f,.36f,.75f,-.42f,.71f,-.53f,-.23f,-.26f,-.23f,-.19f,-.13f,.85f,-.15f,.70f,-.27f,-.43f,.63f,.13f,.95f,-.63f,-.28f,-.42f,-.91f,-.42f,.79f,-.05f,-.43f,.24f,.39f,.22f,.63f,-.40f,.12f,.10f,-.23f,-.86f,-.12f,-.42f,.31f,-.87f,-.38f,.04f,.54f,.37f,-.51f,.09f,.58f,.27f,.27f,.17f,-.16f,.36f,.35f,.88f,-.79f,.34f,.34f,-.51f,-.15f,-.84f,.07f,.46f,.46f,.44f,-.70f,-.41f,.98f,.44f,.66f,.31f,.84f,.95f,-.25f,-.52f,-.07f,-.78f,-.72f,-.55f,-.84f,-.38f,-.77f,.90f,-.08f,-.62f,.74f,-.78f,-.46f,-.71f,.80f,.21f,.66f,.19f,.80f,.05f,.83f,.28f,.46f,.75f,.33f,.31f,-.62f,-.20f,.13f,-.79f,-.17f,.32f,-.26f,-.48f,.40f,.93f,-.08f,.49f,.71f,.34f,-.05f,.62f,.30f,.22f,.12f,.60f,-.02f,-.56f,.39f,-.15f,.78f,.80f,-.11f,-.61f,-.66f,-.25f,-.80f,-.44f,-.39f,-.99f,.05f,.57f,-.36f,.82f,-.74f,.37f,-.50f,.68f,-.37f,.56f,.80f,-.50f,.45f,-.50f,.58f,-.01f,.79f,-.48f,.05f,.97f,.32f,.82f,.46f,.49f,-.12f,-.71f,-.04f,-.65f,.60f,-.81f,.78f,-.85f,.90f,-.95f,-.46f,.62f,.36f,-.43f,.10f,.57f,.52f,-.60f,-.05f,-.11f,.12f,.32f,.52f,.91f,.53f,.24f,.04f,-.44f,-.41f,.89f,-.17f,-.45f,-.83f,.28f,-.70f,.46f,.58f,.81f,-.23f,-.97f,-.61f,-.82f,.83f,.45f,.74f,-.55f,-.51f,-.74f,.56f,.00f,.23f,.76f,-.30f,.07f,.91f,.26f,.82f,.32f,-.40f,.82f,.64f,-.83f,-.99f,-.19f,.34f,.09f,.31f,-.47f,.38f,.33f,.68f,.71f,-.20f,-.86f,-.10f,-.53f,-.04f,-.99f,.35f,.49f,.52f,.81f,-.51f,.76f,-.50f,-.31f,-.93f,-.89f,-.88f,.97f,.22f,-.41f,.25f,-.28f,-.29f,-.60f,-.23f,-.75f,-.39f,.81f,.17f,.18f,-.92f,-.94f,.82f,-.83f,-.71f,.77f,-.65f,.15f,.17f,.09f,.23f,.21f,.05f,.14f,-.56f,-.31f,.32f,.45f,-.41f,.40f,-.01f,-.59f,-.07f,.96f,.15f,.98f,-.32f,-.66f,.58f,-.07f,.01f,-.67f,-.17f,-.37f,.76f,-.57f,.18f,-.97f,-.14f,.78f,.13f,.78f,-.58f,.33f,.45f,-.31f,.29f,.50f,-.20f,-.78f,-.82f,-.55f,.46f,.55f,-.73f,-.63f,.01f,.12f,.62f,.58f,.95f,.08f,-.94f,-.90f,-.70f,-.22f,.57f,.69f,.80f,.52f,-.38f,-.38f,.59f,.17f,.53f,.46f,-.55f,.81f,.59f,.54f,.57f,-.38f,.07f,-.23f,.16f,.18f,.57f,-.94f,.95f,-.83f,.75f,-.75f,-.97f,-.57f,-.61f,-.15f,-.12f,-.67f,.72f,-.90f,-.26f,.39f,.65f,-.34f,-.43f,.13f,-.23f,-.22f,-.80f,.60f,-.54f,.31f,-.98f,-.12f,.48f,.94f,-.96f,.42f,-.33f,-.89f,-.03f,.94f,.04f,.36f,-.75f,.28f,.28f,-.77f,.32f,-.33f,-.09f,-.30f,.96f,-.30f,-.17f,.54f,.48f,.61f,-.50f,-.01f,.49f,-.88f,-.28f,.50f,.66f,.03f,-.19f,-.01f,-.44f,-.73f,-.11f,.07f,.71f,.05f,-.38f,-.50f,-.47f,-.92f,.75f,.48f,-.46f,-.08f,-.15f,.59f,.32f,.87f,.87f,.80f,-.79f,.20f,-.08f,-.64f,.45f,-.26f,.42f,-.95f,.03f,.91f,-.11f,.14f,.63f,.55f,.59f,.04f,.79f,.30f,-.13f,.20f,.48f,-.07f,-.69f,.29f,-.02f,.45f,-.61f,-.52f,.46f,-.28f,.81f,-.93f,-.73f,.43f,-.96f,.45f,.09f,.51f,.56f,-.39f,.20f,-.18f,.16f,-.28f,.02f,.05f,-.45f,.31f,-.61f,.82f,-.92f,-.94f,-.20f,.10f,.44f,-.02f,-.84f,.16f,.08f,-.66f,.06f,.72f,.09f,-.91f,.70f,-.96f,.53f,-.30f,.89f,.76f,.11f,.61f,.89f,-.02f,-.65f,-.98f,-.93f,-.66f,-.11f,-.97f,.52f,-.75f,-.15f,.30f,.39f,.32f,.49f,-.72f,.44f,.62f,.64f,-.76f,.63f,-.18f,.29f,-.74f,-.10f,-.35f,-.11f,-.42f,.29f,-.07f,.06f,-.26f,-.91f,.85f,-.31f,-.73f,.84f,.89f,.46f,.96f,.92f,-.76f,.87f,-.83f,-.57f,.42f,-.79f,-.55f,-.32f,-.14f,.36f,-.37f,.14f,.17f,.60f,.48f,-.66f,.24f,-.27f,.30f,-.76f,-.73f,-.14f,-.86f,-.43f,.93f,.61f,-.03f,.70f,.74f,-.91f,.45f,-.78f,-.07f,-.60f,-.57f,.40f,.51f,.53f,-.02f,.27f,.04f,.94f,.49f,-.16f,-.94f,-.16f,.41f,-.45f,.31f,.56f,.25f,-.56f,.87f,.15f,-.83f,-.66f,-.69f,-.16f,.32f,-.61f,.76f,-.59f,-.69f,-.70f,.42f,.65f,.90f,-.45f,.59f,-.25f,-.94f,.74f,-.87f,-.24f,.69f,-.85f,.33f,-.89f,.35f,-.29f,-.67f,-.28f,-.85f,-.91f,-.68f,.38f,.48f,-.21f,-.00f,-.39f,.93f,.00f,-.43f,-.25f,.67f,.82f,.00f,.27f,-.76f,.44f,-.83f,-.61f,-.80f,.98f,-.07f,-.86f,.03f,.69f,-.88f,-.87f,-.65f,.88f,.61f,.32f,-.82f,.76f,-.57f,-.59f,.02f,-.80f,.34f,-.98f,-.25f,-.34f,.83f,.40f,-.73f,-.63f,-.73f,-.59f,-.15f,-.19f,.07f,-.21f,-.87f,-.52f,.03f,.69f,-.73f,.99f,.25f,-.28f,.50f,-.17f,.81f,.70f,-.62f,.19f,-.64f,-.14f,.87f,.37f,-.98f,-.33f,-.41f,-.21f,.51f,.84f,-.93f,-.44f,.40f,.86f,-.39f,-.46f,.01f,-.06f,.74f,.43f,-.97f,.69f,-.14f,.06f,.46f,.72f,.48f,-.33f,.91f,.85f,.07f,.51f,.34f,-.16f,.46f,.88f,-.24f,.17f,.55f,-.61f,.16f,-.32f,.02f,.53f,.91f,.04f,.26f,.33f,-.64f,-.50f,.32f,.55f,-.47f,.37f,.69f,.09f,-.29f,.75f,-.19f,.93f,.92f,.65f,.85f,-.45f,.17f,-.36f,-.08f,-.01f,.08f,.69f,-.98f,-.46f,-.32f,-.66f,.98f,.08f,-.71f,-.98f,-.51f,-.95f,.23f,.26f,-.35f,-.72f,.69f,-.64f,-.13f,-.23f,.95f,-.74f,-.25f,.34f,.36f,.25f,-.51f,.00f,-.60f,-.92f,.81f,-.84f,.85f,-.55f,.26f,.20f,.18f,-.78f,-.27f,.99f,-.36f,-.97f,.76f,.10f,-.80f,.16f,-.90f,.48f,-.38f,-.15f,.97f,.75f,.17f,.60f,.15f,-.76f,-.58f,.72f,.24f,.16f,-.37f,.07f,.45f,-.29f,.17f,-.30f,-.37f,-.15f,.43f,-.90f,-.33f,.90f,-.29f,.53f,.96f,.83f,.11f,.28f,-.70f,.87f,-.26f,.90f,.17f,.89f,-.25f,-.92f,.62f,.10f,.14f,-.91f,-.59f,-.54f,-.34f,.10f,.25f,.69f,-.60f,.57f,.82f,-.12f,-.97f,-.72f,.34f,-.43f,-.02f,-.16f,-.49f,-.10f,-.22f,-.95f,-.29f,.15f,.15f,.55f,-.55f,.89f,-.28f,-.15f,-.22f,-.63f,.28f,.64f,-.82f,-.09f,-.14f,.96f,-.07f,-.58f,-.29f,.77f,.18f,.47f,-.33f,-.14f,-.80f,-.80f,-.52f,.52f,-.37f,-.09f,.13f,.85f,.01f,.25f,.47f,-.67f,.29f,-.52f,.92f,.87f,-.25f,-.87f,.67f,.44f,-.85f,.81f,-.29f,-.51f,-.11f,-.86f,-.92f,.90f,.87f,.57f,.45f,.74f,.02f,.81f,-.50f,-.32f,.24f,-.70f,-.71f,-.30f,.71f,.56f,.60f,-.64f,.56f,-.88f,.75f,-.04f,.90f,.40f,-.39f,-.16f,.99f,-.55f,.55f,.77f,-.76f,.25f,-.18f,-.78f,.71f,.05f,-.03f,.80f,-.55f,-.13f,.63f,-.33f,.18f,-.66f,.27f,-.83f,-.21f,-.79f,-.46f,-.15f,-.61f,-.28f,-.84f,.58f,-.43f,.85f,-.22f,.54f,.73f,-.87f,-.48f,.53f,-.97f,-.93f,.85f,-.96f,.37f,.78f,.61f,.55f,.43f,-.75f,.09f,-.06f,-.23f,.68f,-.42f,-.88f,-.81f,-.36f,.33f,-.50f,.63f,-.74f,-.61f,-.29f,.14f,.78f,-.19f,-.13f,-.73f,.97f,.87f,-.15f,-.62f,-.16f,.83f,-.78f,.87f,.48f,.32f,.77f,.59f,-.02f,.06f,.72f,.84f,.18f,-.24f,-.53f,.26f,.56f,.50f,.62f,-.29f,-.08f,-.39f,-.29f,-.36f,-.98f,.53f,.53f,-.21f,-.08f,.47f,-.77f,-.73f,-.48f,.84f,.12f,.09f,-.22f,.77f,-.10f,-.69f,.10f,.13f,-.42f,.96f,-.67f,-.78f,.10f,.66f,.85f,.52f,.89f,.50f,.38f,-.03f,.75f,.29f,.53f,-.71f,.99f,-.58f,.75f,.74f,-.66f,.50f,.76f,-.73f,-.84f,-.86f,.71f,.07f,-.82f,-.43f,.96f,.94f,-.36f,-.27f,.43f,.03f,-.60f,-.35f,.41f,-.30f,.06f,-.51f,.49f,-.34f,.99f,-.86f,.99f,-.03f,-.43f,-.27f,-.72f,-.92f,.17f,-.78f,.43f,.93f,.12f,-.15f,.73f,.88f,.44f,-.58f,-.20f,.27f,.45f,.40f,.02f,.94f,-.68f,.88f,-.76f,-.16f,.08f,.92f,-.88f,.03f,-.66f,-.59f,-.05f,-.95f,-.54f,.97f,.94f,-.84f,.24f,-.03f,-.14f,.58f,-.75f,.38f,.00f,-.90f,-.34f,-.70f,-.79f,-.33f,.48f,.92f,-.68f,-.88f,.99f,-.58f,.32f,.81f,-.57f,.23f,-.70f,-.26f,.49f,-.13f,.93f,.66f,-.31f,-.58f,.56f,-.57f,-.38f,.59f,.10f,.10f,-.55f,-.41f,.13f,-.22f,-.53f,.30f,.39f,.66f,.93f,-.38f,.93f,.35f,-.47f,.21f,.14f,.86f,.71f,-.51f,-.99f,.72f,-.37f,-.24f,-.04f,.19f,.03f,-.18f,.38f,-.68f,.88f,.98f,-.57f,.26f,-.18f,.34f,.99f,.04f,-.05f,.84f,-.83f,.45f,-.93f,-.94f,-.65f,.91f,-.23f,.34f,-.34f,-.11f,.10f,.96f,.22f,.59f,.67f,.73f,-.05f,.34f,-.69f,-.74f,-.32f,-.47f,.96f,-.38f,-.77f,-.04f,-.30f,-.29f,-.82f,.22f,-.05f,-.56f,.10f,-.78f,-.06f,-.14f,-.46f,-.11f,.66f,.83f,-.46f,-.26f,-.19f,-.55f,.28f,.00f,-.02f,.02f,-.64f,.14f,-.57f,-.49f,.07f,-.38f,.18f,.75f,-.97f,.20f,-.14f,.26f,.58f,-.31f,-.64f,.90f,.52f,-.49f,.98f,.78f,-.84f,-.36f,.64f,-.63f,.46f,-.95f,.05f,.27f,.65f,.69f,-.80f,-.31f,-.57f,.70f,-.34f,-.65f,-.46f,-.06f,-.12f,-.35f,-.03f,-.49f,.38f,-.07f,-.15f,-.46f,.65f,.70f,-.05f,-.98f,-.75f,.07f,-.61f,-.69f,.05f,-.39f,-.93f,-.93f,.66f,-.68f,.52f,.76f,-.24f,-.78f,-.06f,-.77f,.28f,-.65f,-.74f,.42f,-.04f,-.59f,.97f,-.96f,.87f,.30f,-.31f,-.24f,.99f,-.15f,-.06f,.87f,-.86f,.34f,-.68f,-.15f,.45f,-.83f,-.33f,.56f,-.53f,.45f,-.95f,.48f,.89f,.93f,-.74f,-.21f,.15f,-.17f,-.81f,-.07f,.99f,-.32f,-.66f,.64f,-.89f,-.74f,.14f,.04f,-.82f,-.74f,-.30f,.61f,.35f,-.72f,.52f,-.60f,-.44f,.42f,.58f,.29f,.91f,-.44f,.94f,.65f,-.32f,-.26f,-.02f,.79f,.45f,-.40f,.10f,.53f,-.11f,-.15f,-.99f,-.24f,.86f,.85f,-.42f,-.00f,.28f,.07f,-.86f,-.18f,.75f,-.38f,.16f,-.62f,.97f,.28f,.30f,.80f,.14f,.24f,.82f,-.25f,.55f,-.82f,.42f,-.39f,-.24f,-.58f,.54f,.27f,.95f,-.98f,-.30f,-.55f,.51f,-.52f,-.32f,-.50f,.49f,-.90f,-.15f,.38f,-.92f,.38f,-.48f,.39f,.45f,-.56f,-.96f,-.96f,.72f,-.25f,-.77f,-.31f,.59f,-.89f,.42f,.48f,.62f,-.11f,-.78f,.70f,.46f,.88f,-.32f,-.62f,-.87f,-.61f,.14f,.99f,.44f,.81f,-.66f,.24f,-.56f,.69f,-.97f,.76f,-.75f,-.00f,-.94f,.58f,-.32f,.82f,.74f,-.24f,-.88f,.30f,-.07f,.04f,.75f,-.70f,.70f,.23f,-.21f,-.51f,.36f,.63f,-.70f,.10f,.09f,.64f,-.51f,-.12f,-.23f,-.39f,.35f,.74f,-.58f,.39f,-.84f,.81f,-.93f,-.40f,.31f,-.81f,.41f,-.89f,-.27f,-.39f,.13f,-.51f,-.72f,-.55f,.79f,.49f,-.29f,.00f,-.92f,-.10f,-.77f,-.75f,-.33f,-.20f,.21f,.22f,-.31f,.14f,-.58f,.82f,.38f,.33f,-.01f,.54f,-.67f,.24f,.60f,.53f,.52f,.21f,-.73f,-.72f,-.60f,-.37f,-.34f,-.06f,.15f,-.63f,-.99f,-.28f,.04f,-.36f,-.28f,.35f,.88f,.24f,.52f,-.54f,.55f,-.91f,.36f,.59f,.48f,-.43f,-.54f,-.36f,.12f,.56f,-.72f,.54f,-.75f,-.77f,.12f,-.89f,-.30f,.93f,-.11f,-.18f,-.67f,-.15f,.02f,-.76f,-.51f,.45f,.79f,-.49f,.90f,-.98f,-.70f,.10f,.35f,-.22f,.66f,-.62f,.87f,.44f,-.55f,-.63f,.09f,-.42f,-.38f,.81f,.69f,-.56f,-.51f,.25f,.31f,.06f,-.99f,.45f,-.85f,.84f,.32f,.21f,.61f,-.44f,-.63f,.46f,-.87f,-.03f,.40f,-.52f,.89f,-.92f,-.14f,.30f,.39f,.30f,-.63f,-.01f,-.66f,-.02f,-.51f,.14f,.02f,.28f,.85f,.77f,-.55f,.69f,-.13f,-.72f,-.04f,-.35f,.05f,.40f,.49f,.27f,-.64f,-.46f,-.59f,-.33f,.43f,.53f,-.50f,-.60f,.28f,.82f,.51f,.90f,.48f,-.55f,-.08f,-.32f,.68f,-.93f,-.09f,-.85f,-.29f,-.92f,.15f,.84f,-.97f,.12f,.72f,.81f,-.88f,-.98f,-.26f,.85f,.56f,.95f,-.10f,.67f,-.14f,.03f,.25f,.72f,-.22f,.18f,-.72f,-.85f,.83f,.00f,.98f,-.34f,.35f,.62f,-.65f,-.66f,.78f,.09f,-.09f,-.98f,.89f,-.27f,-.54f,-.82f,.06f,-.12f,-.89f,.45f,-.07f,.73f,-.66f,-.00f,-.67f,.56f,-.41f,-.99f,.09f,.30f,-.30f,.76f,-.33f,.02f,.29f,-.36f,.69f,.83f,-.16f,.76f,-.36f,-.47f,-.22f,.18f,-.49f,.57f,-.02f,-.56f,-.93f,-.19f,-.98f,-.98f,-.17f,-.42f,.55f,.49f,-.37f,.02f,-.22f,-.61f,-.36f,.94f,-.65f,-.87f,.56f,-.93f,-.53f,-.23f,-.30f,.34f,.53f,-.55f,-.62f,-.86f,.22f,-.57f,.47f,.12f,-.93f,-.83f,-.90f,.77f,.36f,.73f,-.59f,.87f,.88f,-.54f,.88f,.39f,.42f,.50f,.68f,-.66f,.07f,.22f,-.39f,-.47f,.41f,-.79f,-.38f,.79f,-.09f,.37f,-.07f,.05f,.66f,-.54f,.45f,.39f,.84f,.72f,.12f,.30f,-.55f,.54f,.42f,-.14f,-.85f,-.56f,-.55f,.27f,.70f,.98f,-.28f,-.29f,.48f,-.02f,-.83f,-.86f,-.92f,.88f,-.11f,.53f,.06f,-.45f,.05f,.78f,.69f,.22f,.52f,.11f,.38f,-.52f,.47f,-.02f,-.43f,-.19f,-.39f,-.45f,.81f,.29f,-.97f,-.04f,.25f,-.29f,-.77f,.46f,-.62f,-.92f,.85f,-.89f,-.84f,.03f,.16f,.47f,-.18f,-.60f,.04f,.71f,-.99f,.57f,.12f,-.83f,-.26f,.29f,-.81f,-.48f,.91f,.01f,-.38f,.50f,.69f,.38f,.18f,.03f,-.64f,.19f,.10f,.76f,.27f,-.34f,-.28f,-.74f,-.26f,.28f,-.60f,.95f,.52f,.84f,-.20f,.97f,.93f,-.56f,-.29f,.87f,-.71f,.20f,.83f,-.83f,.69f,-.86f,.40f,.34f,.78f,-.50f,-.19f,.50f,-.78f,-.11f,.77f,-.80f,.95f,-.22f,.03f,-.68f,-.63f,.74f,.01f,-.70f,-.59f,-.64f,-.72f,.83f,.41f,-.65f,-.16f,-.29f,.55f,-.63f,.80f,-.85f,-.49f,.45f,-.54f,-.00f,-.36f,-.04f,.66f,-.14f,.15f,-.44f,.79f,-.44f,.81f,.90f,.65f,.27f,-.08f,.28f,-.88f,.89f,-.80f,-.96f,-.97f,.56f,-.72f,-.68f,-.26f,.91f,.43f,.93f,-.85f,-.97f,-.00f,.04f,-.68f,.76f,.61f,-.66f,-.08f,-.75f,-.95f,.84f,.47f,-.66f,-.92f,-.50f,.11f,-.99f,.22f,.20f,.00f,.33f,.14f,-.35f,.48f,.42f,.30f,.89f,-.93f,.88f,-.44f,.08f,.01f,.03f,-.14f,.71f,.24f,-.17f,-.68f,.17f,.53f,.79f,.97f,.32f,.58f,.40f,-.77f,-.77f,.14f,.38f,.69f,.73f,.88f,.81f,.67f,.13f,-.53f,-.13f,-.79f,.70f,.46f,.71f,.06f,-.42f,.96f,.24f,.02f,-.31f,-.18f,-.87f,-.70f,.05f,-.46f,-.59f,-.84f,.12f,.41f,-.90f,.45f,.11f,.02f,-.49f,-.26f,-.41f,.43f,-.34f,-.19f,-.41f,-.64f,-.20f,.22f,-.20f,-.05f,.33f,-.19f,.69f,-.85f,-.01f,.33f,.07f,.80f,-.45f,-.53f,-.57f,.88f,-.80f,.12f,.60f,-.31f,.11f,.33f,-.10f,-.09f,.21f,-.93f,-.44f,.18f,-.13f,-.33f,.17f,.38f,-.43f,.07f,.30f,-.10f,-.25f,.96f,-.23f,.91f,-.47f,.37f,.52f,-.96f,.33f,.37f,-.75f,-.91f,.33f,.11f,-.92f,-.98f,-.74f,-.26f,.04f,-.06f,.38f,-.73f,.27f,.88f,.25f,-.79f,.48f,.82f,.98f,-.97f,.21f,-.16f,.56f,-.03f,-.53f,-.13f,-.73f,-.31f,.37f,-.68f,-.36f,.42f,-.21f,-.02f,-.57f,.50f,.00f,-.87f,.28f,.27f,.98f,-.96f,.21f,.81f,-.33f,-.94f,-.15f,-.45f,.93f,-.36f,.23f,.64f,-.04f,-.13f,.40f,-.72f,.17f,.87f,-.60f,-.82f,-.20f,.32f,.92f,.99f,.28f,-.47f,.50f,.27f,.58f,-.98f,.36f,.90f,.45f,-.65f,.26f,.54f,-.54f,.35f,.07f,-.94f,.23f,.02f,.87f,.83f,.09f,-.83f,.53f,.44f,-.93f,.35f,-.86f,-.01f,.38f,.56f,-.65f,.72f,-.04f,.09f,.17f,.95f,-.64f,.91f,.34f,-.60f,-.40f,.73f,-.66f,-.65f,-.21f,-.17f,-.12f,.50f,.27f,-.68f,-.74f,-.27f,-.64f,-.75f,-.01f,-.50f,.47f,-.50f,-.08f,-.00f,-.79f,.99f,-.01f,-.69f,-.60f,-.95f,-.11f,.25f,-.52f,-.49f,.31f,.76f,-.83f,-.39f,-.42f,-.18f,.06f,-.04f,-.74f,-.27f,-.23f,.76f,.33f,.59f,-.72f,-.70f,-.04f,.63f,-.21f,.25f,.16f,-.03f,.77f,.03f,-.74f,.94f,-.91f,-.23f,.47f,.90f,.82f,-.58f,.77f,-.37f,.05f,-.51f,.09f,-.80f,.95f,.73f,.28f,.56f,-.74f,-.31f,.37f,.65f,.32f,-.50f,-.52f,-.68f,.78f,.82f,.94f,.27f,-.44f,-.05f,.39f,-.80f,-.88f,.42f,.72f,.02f,.26f,-.58f,.21f,-.90f,.00f,.36f,.77f,-.96f,-.18f,.99f,-.71f,.94f,-.81f,-.08f,-.64f,-.71f,-.84f,-.28f,.44f,.44f,.80f,-.17f,-.50f,.38f,.75f,.29f,.54f,-.56f,-.40f,.35f,.59f,.88f,-.87f,.03f,.88f,-.02f,.26f,.68f,.38f,.24f,.03f,.15f,.28f,.73f,.67f,.94f,-.04f,.18f,-.19f,.68f,-.91f,.41f,-.50f,-.36f,.03f,-.65f,.44f,-.73f,-.18f,.39f,.40f,-.53f,-.94f,.11f,.71f,.82f,-.17f,.65f,.60f,.93f,-.99f,-.11f,-.83f,-.31f,-.09f,-.08f,.91f,-.51f,-.25f,.52f,.54f,.23f,-.00f,.69f,.88f,.49f,-.50f,.99f,.82f,-.56f,-.68f,.18f,-.38f,-.97f,.27f,-.83f,.63f,.98f,-.59f,.43f,.07f,-.68f,-.00f,-.25f,-.69f,-.03f,.86f,.72f,.53f,-.95f,-.88f,.15f,.43f,-.61f,-.03f,-.56f,-.44f,-.35f,-.46f,-.21f,-.80f,-.63f,.39f,-.24f,-.91f,.24f,.32f,-.22f,.73f,-.24f,-.53f,-.22f,-.62f,-.39f,-.40f,.60f,.78f,-.11f,.32f,-.34f,.53f,.60f,.10f,-.59f,.31f,.77f,-.54f,.52f,.68f,.84f,-.55f,-.15f,.16f,.15f,-.64f,.73f,.77f,-.57f,-.05f,.36f,-.03f,.27f,.93f,-.88f,.82f,.40f,-.69f,.23f,.01f,.30f,-.21f,.09f,-.18f,.97f,.33f,.48f,-.15f,-.05f,-.86f,.81f,-.33f,-.92f,.58f,.47f,.22f,.81f,.77f,-.82f,-.15f,.82f,-.53f,.31f,.30f,-.16f,-.24f,-.24f,-.80f,.72f,-.91f,-.53f,-.73f,.85f,.82f,-.97f,-.80f,-.13f,-.20f,.40f,-.70f,-.35f,.88f,.08f,-.48f,.77f,.62f,-.16f,-.47f,.53f,.11f,.43f,-.36f,-.13f,-.01f,-.36f,.88f,-.89f,.03f,.69f,-.36f,-.98f,.52f,.28f,.59f,.27f,-.43f,-.75f,.61f,.67f,.46f,-.50f,-.11f,-.77f,-.43f,-.00f,-.80f,.91f,.22f,-.50f,-.83f,-.52f,.94f,-.92f,-.36f,-.96f,-.40f,.11f,-.37f,.08f,.52f,.53f,.02f,-.83f,.54f,-.48f,.02f,-.05f,.32f,-.78f,.02f,-.79f,.34f,-.54f,.93f,-.80f,.32f,.70f,-.82f,.92f,.04f,.83f,-.26f,-.68f,.76f,.78f,.35f,.71f,.83f,.52f,.57f,-.20f,.12f,.26f,-.99f,.47f,.60f,-.06f,-.17f,.26f,-.60f,.70f,-.22f,-.87f,-.78f,.29f,.47f,.76f,.09f,.73f,.07f,-.91f,-.53f,-.40f,.78f,.81f,-.57f,-.05f,-.54f,-.04f,-.14f,.80f,.11f,-.75f,.56f,.63f,-.73f,-.32f,.86f,-.98f,-.64f,.59f,-.95f,.63f,.10f,.24f,-.83f,-.60f,-.67f,.40f,-.96f,-.30f,-.69f,.70f,-.89f,.40f,-.21f,.83f,.44f,.71f,-.85f,-.05f,-.95f,-.83f,.26f,-.63f,.37f,-.05f,-.76f,-.47f,.37f,.01f,.42f,-.12f,.99f,-.43f,.16f,-.20f,-.17f,-.18f,-.13f,-.98f,-.05f,-.86f,.40f,.70f,-.68f,.93f,-.62f,.08f,-.54f,-.01f,.82f,.05f,-.45f,-.83f,-.22f,-.35f,.54f,.48f,.12f,.27f,-.87f,-.67f,-.15f,.02f,-.67f,.52f,.24f,.08f,-.80f,.09f,.99f,.31f,.01f,.83f,-.99f,-.90f,-.23f,.39f,.75f,-.51f,.75f,.37f,-.28f,.27f,-.72f,.71f,.78f,.95f,-.05f,.08f,-.10f,-.19f,.43f,.38f,-.69f,-.38f,.67f,.58f,.86f,.14f,.86f,-.06f,.41f,-.17f,-.15f,.96f,.10f,.20f,.22f,-.92f,-.51f,-.08f,.84f,.83f,-.90f,.51f,-.07f,.69f,.84f,.30f,.35f,-.01f,-.01f,.85f,.76f,.77f,-.50f,.67f,.65f,-.49f,-.47f,-.64f,.63f,-.12f,-.84f,.76f,.50f,.00f,-.83f,-.71f,.43f,-.10f,.95f,.86f,-.77f,.93f,-.61f,-.67f,-.18f,-.53f,-.12f,-.59f,.95f,-.97f,-.08f,-.20f,-.65f,-.78f,-.02f,-.47f,-.55f,-.77f,.00f,-.01f,-.97f,.29f,-.68f,.66f,-.86f,.70f,-.53f,-.99f,-.29f,-.87f,.34f,.13f,-.12f,-.64f,.65f,-.08f,.80f,-.37f,-.01f,.63f,-.85f,.37f,.23f,.06f,-.57f,-.99f,.28f,-.48f,.70f,.34f,.08f,.96f,-.27f,.36f,.31f,-.39f,-.22f,-.06f,-.67f,-.22f,-.49f,.77f,-.16f,.95f,-.03f,-.40f,.67f,.17f,.73f,.44f,-.69f,.78f,.57f,-.14f,-.61f,-.31f,.73f,-.71f,.03f,.95f,.33f,-.50f,.69f,-.11f,.81f,-.64f,.67f,.66f,-.47f,-.28f,-.11f,.11f,.97f,.67f,.98f,.38f,-.07f,-.38f,.12f,-.59f,.89f,.12f,.05f,-.03f,.58f,-.68f,-.74f,.57f,.81f,.23f,-.35f,-.27f,.29f,-.43f,.03f,-.60f,.18f,.07f,.74f,-.56f,.70f,-.11f,.28f,-.73f,.14f,-.60f,.00f,-.71f,-.38f,.39f,.93f,-.32f,.75f,-.39f,.63f,-.44f,-.10f,.10f,.83f,.56f,-.07f,-.25f,.75f,.96f,.34f,-.78f,-.95f,-.18f,.36f,.57f,-.82f,.44f,-.57f,.65f,.40f,-.02f,.97f,.31f,.06f,-.91f,-.80f,-.74f,-.29f,.46f,-.58f,.47f,.25f,-.06f,.28f,.98f,-.56f,-.13f,.90f,.28f,.99f,.78f,-.46f,.96f,-.34f,-.94f,.22f,.99f,-.48f,.18f,.97f,-.52f,.98f,.74f,-.18f,.04f,.98f,-.85f,-.36f,-.29f,-.18f,-.39f,-.89f,.49f,-.71f,.24f,.59f,-.85f,.58f,.86f,-.68f,.07f,.72f,-.32f,-.76f,.84f,.05f,.16f,.68f,-.76f,.22f,-.65f,-.18f,-.49f,-.17f,.70f,-.42f,.98f,.18f,.18f,.22f,.14f,.35f,.26f,-.36f,.98f,.84f,-.54f,.13f,.90f,.46f,.53f,-.94f,-.15f,-.14f,.77f,.68f,.16f,-.49f,-.43f,-.83f,-.33f,.14f,-.55f,-.39f,-.14f,.08f,-.78f,-.79f,-.65f,-.65f,.73f,.97f,.65f,.50f,-.51f,-.24f,-.88f,.59f,.74f,.03f,-.91f,.33f,.03f,-.66f,-.20f,-.13f,.94f,.46f,.41f,.50f,.45f,.47f,-.10f,-.28f,-.36f,-.02f,-.52f,.11f,-.19f,-.70f,.33f,.20f,-.85f,-.15f,-.89f,.11f,.68f,-.64f,-.70f,.90f,-.02f,-.26f,-.49f,.88f,.46f,.75f,.96f,.46f,.08f,.43f,-.14f,-.17f,-.74f,-.75f,-.19f,.61f,-.87f,.84f,-.04f,.85f,.16f,-.73f,.14f,-.21f,.21f,.73f,.92f,.10f,-.66f,.37f,-.64f,.90f,.32f,-.46f,.04f,.80f,.88f,.13f,-.88f,-.48f,.37f,.42f,-.35f,-.92f,-.16f,.61f,-.15f,.17f,.14f,.89f,-.15f,.18f,.94f,-.13f,.51f,-.67f,.64f,.41f,.45f,.00f,-.81f,.06f,-.06f,-.48f,-.92f,-.42f,.83f,-.87f,.29f,.72f,.70f,.40f,.79f,-.80f,.17f,.32f,.29f,-.30f,-.13f,.82f,.72f,-.96f,-.84f,-.13f,-.36f,.10f,.40f,.59f,.92f,-.30f,-.16f,-.53f,-.47f,.14f,-.82f,.02f,.81f,.27f,-.77f,.63f,.47f,.24f,.19f,-.29f,-.56f,-.49f,-.03f,.60f,-.84f,.10f,-.30f,-.09f,.78f,.31f,-.75f,.19f,-.10f,.02f,.45f,.60f,-.36f,.68f,.49f,-.29f,-.21f,.54f,-.66f,.55f,.03f,.53f,.25f,-.47f,-.13f,.41f,.74f,.02f,-.67f,-.68f,-.61f,-.92f,-.37f,-.65f,-.76f,-.86f,.83f,-.84f,-.79f,-.36f,.09f,.43f,.68f,-.81f,-.25f,.81f,-.66f,.28f,.33f,.74f,.56f,-.29f,-.84f,-.76f,.74f,-.03f,.52f,.30f,.18f,.11f,-.46f,.28f,-.15f,-.27f,.21f,.79f,-.46f,-.41f,-.70f,.60f,-.35f,-.33f,-.56f,-.14f,.17f,.01f,-.81f,-.62f,-.60f,.39f,-.62f,.87f,.83f,-.00f,-.47f,-.39f,-.97f,.11f,.22f,-.32f,-.21f,-.25f,.70f,.70f,.01f,.04f,-.38f,-.12f,.80f,.85f,.40f,.87f,-.59f,-.99f,-.17f,.63f,.30f,.36f,.77f,-.25f,-.50f,.22f,-.26f,.49f,-.97f,.18f,-.29f,-.19f,-.28f,.34f,-.69f,-.15f,-.50f,.18f,.68f,-.31f,.52f,-.18f,-.67f,-.22f,.11f,-.58f,-.17f,-.21f,-.01f,.71f,.93f,.00f,.41f,-.72f,-.66f,-.26f,-.57f,-.06f,-.44f,-.06f,-.79f,-.14f,-.09f,.52f,.67f,-.65f,-.52f,-.85f,-.52f,-.18f,-.70f,.75f,-.51f,-.49f,-.06f,.81f,.06f,-.95f,.53f,-.80f,-.16f,.84f,.97f,-.53f,.00f,-.75f,-.99f,-.38f,.07f,-.44f,.25f,.31f,.16f,-.39f,-.92f,-.17f,-.38f,-.63f,.27f,.53f,.56f,-.82f,.95f,.69f,-.25f,.62f,-.36f,-.21f,.15f,-.82f,-.73f,-.77f,-.78f,.46f,-.15f,.44f,.82f,-.82f,-.50f,-.73f,-.39f,-.85f,.89f,-.30f,-.48f,-.96f,.95f,.66f,-.08f,.82f,-.00f,-.16f,.99f,-.49f,.27f,.88f,-.69f,-.36f,-.81f,.86f,-.53f,.79f,.76f,.56f,.59f,-.70f,.31f,.61f,.80f,.93f,.90f,.05f,.42f,.21f,-.75f,-.87f,.48f,-.83f,-.58f,-.53f,-.20f,-.34f,-.08f,.25f,-.21f,-.18f,-.82f,.59f,.18f,-.73f,.50f,.36f,.01f,-.54f,-.40f,.24f,-.99f,-.30f,-.43f,-.26f,.53f,.98f,.90f,-.68f,-.57f,-.02f,.63f,-.84f,-.96f,-.52f,.32f,.09f,-.98f,-.52f,.64f,.17f,.82f,.07f,.92f,.63f,-.43f,-.52f,.06f,-.85f,.02f,-.71f,-.86f,-.58f,-.32f,.79f,.68f,.02f,.50f,-.76f,-.26f,-.23f,.03f,.42f,-.26f,-.09f,-.40f,.84f,.79f,-.57f,-.07f,-.61f,-.79f,-.97f,.16f,.10f,.45f,-.04f,.66f,-.52f,-.39f,-.77f,.94f,.70f,.61f,-.96f,-.33f,.85f,.53f,-.74f,.30f,.31f,-.33f,.74f,-.56f,-.00f,-.95f,-.91f,-.42f,.85f,-.02f,-.06f,-.73f,.07f,-.52f,.10f,.39f,.90f,.78f,-.11f,.58f,.61f,.30f,.53f,.51f,.06f,.62f,.58f,.39f,-.78f,.47f,-.36f,-.48f,.29f,-.98f,.90f,-.63f,-.40f,.77f,.93f,.32f,.94f,.76f,-.86f,.84f,-.57f,-.35f,-.68f,.58f,-.70f,-.87f,-.13f,-.99f,-.46f,.51f,.28f,-.22f,-.92f,-.02f,.77f,.91f,.41f,-.13f,-.18f,-.80f,-.12f,.99f,-.62f,.71f,-.39f,-.53f,-.75f,-.88f,-.05f,.84f,.32f,.87f,.18f,.71f,.03f,-.48f,-.59f,.43f,-.44f,.23f,-.36f,.79f,-.27f,-.30f,-.35f,.56f,.99f,-.35f,-.89f,-.45f,.45f,-.55f,.27f,-.29f,.90f,-.95f,-.63f,.31f,-.28f,-.56f,.38f,-.08f,-.18f,.51f,.60f,-.93f,-.00f,.89f,-.52f,.08f,-.36f,-.61f,.71f,-.06f,.53f,-.77f,-.46f,-.30f,-.79f,-.36f,.59f,-.32f,-.28f,.88f,-.15f,.99f,.31f,.84f,.72f,.45f,.61f,-.69f,.73f,-.28f,.88f,-.21f,.65f,-.54f,-.76f,-.33f,-.53f,.87f,-.36f,.96f,.60f,-.07f,.43f,.52f,.68f,.72f,-.53f,-.23f,.09f,.47f,-.92f,-.31f,-.63f,-.26f,-.86f,.28f,.77f,-.14f,.63f,.48f,.81f,.31f,.94f,.37f,.93f,-.70f,.64f,-.63f,-.31f,-.10f,-.64f,.61f,.02f,-.86f,-.40f,-.95f,-.80f,.98f,.96f,-.07f,.33f,-.59f,-.13f,-.54f,.67f,-.50f,.14f,-.30f,.30f,-.80f,.71f,.86f,.26f,-.97f,-.57f,-.88f,-.95f,.84f,-.25f,.58f,-.28f,-.61f,-.55f,-.03f,.14f,-.05f,.66f,.98f,-.01f,-.41f,.06f,.74f,.09f,.42f,.34f,-.06f,.27f,-.26f,-.82f,-.07f,.67f,-.12f,-.46f,.16f,-.07f,.05f,-.21f,.35f,-.77f,-.39f,.11f,.15f,.06f,.17f,.69f,.25f,-.89f,.93f,-.29f,-.25f,-.95f,.74f,-.29f,-.22f,.82f,-.71f,.39f,-.54f,.58f,.41f,-.82f,-.65f,-.41f,-.74f,-.09f,-.08f,-.53f,-.06f,-.02f,.51f,.76f,.40f,.23f,-.14f,-.71f,-.59f,.33f,.14f,-.98f,-.01f,-.45f,.25f,-.33f,-.21f,.47f,-.28f,-.43f,.52f,-.04f,.04f,-.63f,.53f,-.15f,-.23f,-.15f,.04f,-.99f,-.69f,.16f,.26f,-.21f,.90f,.99f,-.91f,-.69f,.40f,-.69f,-.52f,.18f,.86f,-.41f,.69f,.44f,-.13f,-.16f,.89f,.60f,-.12f,.06f,.84f,-.58f,.71f,.19f,.81f,.58f,-.64f,-.39f,-.92f,-.70f,-.98f,-.13f,.07f,-.99f,.14f,.69f,-.20f,.18f,-.80f,.34f,-.79f,.94f,.34f,-.81f,-.69f,-.72f,-.07f,.33f,.06f,.87f,-.77f,.72f,.16f,.74f,.87f,-.87f,.57f,.87f,.65f,.63f,-.78f,.71f,.64f,-.94f,.45f,.52f,.61f,.80f,.56f,.89f,-.99f,.31f,.11f,-.63f,.21f,.97f,.04f,-.59f,-.81f,.13f,.46f,.28f,.01f,-.22f,.62f,-.12f,.64f,.82f,-.67f,-.76f,.87f,-.23f,-.43f,.72f,-.64f,.08f,.53f,-.41f,-.03f,-.68f,.62f,.08f,.37f,-.12f,-.52f,.07f,.06f,-.28f,.02f,.35f,.11f,.38f,.90f,-.19f,-.36f,.95f,-.53f,-.66f,-.30f,.46f,.44f,.63f,.15f,.75f,-.88f,-.80f,-.39f,-.33f,.45f,.56f,-.34f,.98f,-.50f,.22f,.15f,.12f,.62f,.06f,.82f,-.40f,.86f,.56f,.18f,.13f,-.30f,-.98f,.57f,-.49f,-.32f,-.74f,.91f,-.07f,.09f,.35f,-.22f,-.42f,.37f,.41f,-.01f,-.45f,.96f,-.97f,-.37f,.58f,-.74f,.61f,.37f,-.25f,.82f,.99f,-.22f,.60f,-.95f,.35f,.63f,-.73f,-.96f,.40f,.17f,-.04f,.01f,-.45f,-.52f,.86f,.32f,-.97f,-.67f,.21f,.60f,-.88f,-.17f,.11f,-.89f,.52f,-.93f,-.17f,-.94f,.72f,-.45f,.81f,-.85f,-.63f,.27f,.72f,-.99f,-.74f,-.70f,-.06f,-.28f,.28f,.89f,-.38f,-.36f,.28f,.51f,-.29f,-.04f,-.22f,-.77f,.62f,.68f,.73f,-.29f,.96f,.26f,.36f,-.79f,.93f,.23f,.06f,-.99f,.93f,.34f,-.40f,.43f,-.80f,-.60f,-.64f,-.17f,-.89f,-.56f,-.81f,.70f,-.67f,.57f,.57f,-.77f,-.28f,-.39f,-.58f,-.35f,-.02f,.21f,-.43f,.60f,-.95f,-.44f,.40f,-.22f,.44f,-.94f,.21f,-.21f,-.38f,-.06f,.43f,-.48f,.49f,-.79f,.31f,-.91f,.34f,.89f,.10f,.50f,.48f,.29f,.00f,.24f,.50f,.56f,-.73f,-.10f,.23f,.29f,.59f,-.02f,-.76f,-.46f,-.97f,-.06f,-.46f,.51f,.14f,.97f,-.37f,.65f,-.74f,-.20f,-.29f,.57f,-.80f,-.34f,-.33f,.17f,-.86f,.58f,-.85f,-.08f,.09f,-.88f,.57f,-.78f,-.55f,-.72f,.18f,-.45f,.37f,-.51f,-.77f,.83f,-.39f,.73f,-.87f,-.18f,-.95f,.94f,-.21f,.70f,.56f,.25f,.99f,.14f,.18f,-.35f,.72f,.09f,.98f,.85f,.10f,.97f,.42f,-.15f,-.85f,.89f,.67f,.99f,-.08f,.32f,.08f,.55f,.96f,-.52f,-.63f,-.21f,-.50f,.79f,.10f,.61f,-.96f,-.47f,.59f,-.02f,-.39f,.89f,.31f,.94f,-.84f,.47f,.79f,.82f,-.18f,-.41f,-.22f,-.99f,-.68f,-.35f,.86f,.16f,.68f,.05f,-.11f,-.65f,.23f,-.81f,-.92f,-.70f,.60f,-.65f,-.37f,-.23f,.10f,.15f,.19f,-.03f,.74f,-.07f,.12f,-.49f,.67f,-.91f,.38f,.53f,.86f,-.41f,-.39f,-.79f,-.48f,-.83f,-.58f,.18f,-.96f,-.77f,-.47f,-.38f,.32f,.23f,.85f,.35f,.40f,-.27f,.66f,.62f,.65f,-.94f,-.69f,-.93f,.69f,-.20f,.05f,-.67f,-.18f,-.36f,-.31f,.15f,-.01f,.86f,-.72f,-.94f,.41f,-.22f,-.86f,-.56f,.69f,.09f,-.00f,-.80f,.05f,.98f,.95f,.46f,-.33f,-.38f,-.04f,-.53f,-.12f,.82f,-.55f,-.79f,.10f,-.14f,-.68f,-.29f,.14f,.93f,.64f,.97f,.37f,.69f,.73f,-.57f,.11f,-.75f,-.07f,.74f,.25f,-.57f,.67f,.60f,-.26f,-.35f,-.19f,.55f,.42f,-.62f,.63f,-.87f,-.82f,.60f,-.10f,-.67f,.89f,-.38f,.29f,.30f,.81f,.95f,.43f,-.64f,.91f,.37f,.96f,-.71f,.55f,.06f,-.95f,-.56f,.34f,.25f,.05f,.26f,.28f,.95f,.68f,-.63f,-.80f,.63f,-.09f,-.06f,.26f,-.34f,-.27f,-.57f,.99f,-.48f,.10f,.96f,-.17f,.70f,-.76f,-.49f,.37f,-.81f,-.05f,-.67f,.70f,-.94f,-.15f,-.98f,.03f,-.77f,.97f,-.61f,-.32f,.63f,-.01f,-.42f,-.89f,.41f,.14f,.11f,-.61f,-.05f,.32f,.21f,.64f,.36f,-.44f,.05f,-.33f,.19f,-.43f,-.40f,-.89f,.90f,.67f,.66f,-.37f,-.67f,-.63f,.93f,.29f,.28f,.41f,.17f,-.76f,-.61f,-.48f,-.86f,-.13f,-.55f,.96f,-.06f,-.14f,.97f,.86f,-.20f,.88f,.40f,.64f,.79f,.47f,-.09f,-.26f,-.31f,.74f,-.09f,-.49f,.64f,-.40f,-.31f,-.87f,-.68f,.40f,-.44f,.19f,.10f,-.69f,.42f,-.59f,.84f,-.39f,-.85f,.33f,.44f,.12f,.69f,.02f,-.71f,.57f,-.16f,-.67f,.53f,-.95f,.37f,.42f,.40f,.76f,-.61f,.02f,.77f,.66f,.54f,-.37f,-.76f,-.56f,.77f,.57f,.99f,-.02f,.00f,.54f,.19f,.89f,-.32f,-.00f,-.74f,-.82f,.43f,.42f,.35f,-.81f,.68f,-.52f,-.17f,.18f,.29f,.76f,-.73f,.01f,-.28f,.04f,.21f,.89f,-.40f,.04f,.13f,-.95f,.33f,.47f,.18f,.61f,-.83f,-.08f,-.08f,-.75f,.30f,.42f,.76f,.25f,.18f,-.84f,.11f,-.94f,-.42f,-.16f,-.97f,.54f,.11f,.37f,.10f,-.39f,-.20f,.09f,.04f,-.92f,.97f,.98f,-.78f,.58f,-.74f,.92f,-.27f,.16f,-.42f,-.31f,.39f,.23f,-.65f,.24f,-.92f,.52f,.98f,.39f,.51f,.98f,-.98f,.34f,.68f,-.84f,.41f,-.87f,.55f,.39f,.16f,.37f,.01f,.82f,.91f,-.82f,-.76f,.65f,-.45f,.44f,.07f,-.46f,.64f,-.63f,-.11f,.54f,-.92f,-.11f,-.94f,-.78f,-.88f,.68f,-.11f,.78f,.28f,-.08f,.99f,.27f,.73f,-.55f,.05f,.69f,-.69f,-.83f,.70f,-.24f,.26f,-.86f,.69f,.91f,.43f,.84f,-.60f,.31f,.74f,.57f,-.93f,-.85f,-.74f,.73f,-.15f,.30f,.70f,-.15f,.09f,-.46f,.59f,.05f,.34f,.33f,.17f,-.08f,.03f,-.60f,.93f,.11f,-.13f,-.65f,-.19f,.49f,.82f,-.77f,.22f,-.02f,.79f,.06f,.16f,-.79f,-.30f,-.48f,.62f,-.12f,-.95f,-.87f,-.94f,-.90f,-.33f,.60f,-.21f,.88f,-.28f,.46f,-.14f,.13f,-.11f,-.17f,-.03f,.19f,-.04f,-.34f,-.14f,.65f,-.35f,-.66f,-.75f,.25f,-.91f,-.45f,-.59f,.32f,.75f,-.24f,-.14f,.10f,-.24f,.03f,-.83f,.90f,.08f,.09f,.65f,-.39f,-.80f,.26f,.36f,-.14f,-.53f,.21f,-.06f,.63f,.94f,-.86f,.30f,.91f,-.01f,-.61f,.67f,-.44f,-.57f,.28f,.83f,-.46f,-.18f,-.27f,-.25f,.43f,.55f,-.74f,-.39f,.51f,.62f,.70f,.17f,.61f,-.87f,-.76f,-.28f,.80f,-.09f,.48f,-.68f,.46f,.76f,.73f,-.06f,.15f,.01f,.84f,-.71f,.43f,-.82f,.77f,-.86f,.30f,.01f,-.03f,-.79f,-.96f,.75f,.36f,-.18f,-.23f,.79f,.36f,-.77f,.96f,.60f,.11f,-.73f,-.44f,-.15f,-.54f,.94f,-.54f,.73f,.43f,.70f,-.02f,-.47f,.14f,-.07f,-.78f,-.12f,.61f,-.65f,.28f,-.66f,-.56f,.76f,.56f,.52f,-.43f,.31f,.11f,-.94f,.04f,-.52f,-.37f,.86f,-.74f,-.38f,-.40f,.41f,-.40f,.09f,-.07f,-.79f,-.08f,-.32f,-.87f,-.31f,.21f,-.58f,-.18f,-.86f,.29f,-.28f,.30f,-.52f,.41f,.66f,.10f,-.40f,.70f,.29f,-.20f,-.25f,.33f,.32f,-.14f,.39f,-.41f,.65f,-.88f,.82f,.34f,-.80f,.06f,.86f,-.69f,-.84f,-.36f,.43f,.25f,-.80f,.84f,.19f,-.80f,.53f,-.53f,.04f,-.54f,-.26f,-.59f,.21f,-.45f,.53f,-.17f,.52f,.35f,.72f,.16f,-.31f,.12f,-.03f,-.92f,.02f,.27f,.52f,.34f,-.66f,-.46f,-.14f,.81f,.00f,.92f,-.49f,.80f,.40f,-.90f,.34f,.12f,.47f,.16f,-.66f,-.83f,.30f,.10f,-.99f,.95f,.61f,.92f,.90f,-.46f,.99f,.38f,.38f,.79f,-.33f,.28f,.69f,.44f,-.18f,-.66f,.07f,.65f,-.06f,-.24f,-.70f,.01f,-.31f,-.40f,.19f,.93f,-.52f,-.29f,-.59f,-.55f,-.57f,-.61f,-.51f,.99f,.22f,.51f,.71f,.39f,.59f,.38f,.02f,-.98f,.56f,-.04f,-.02f,.44f,-.85f,-.70f,-.70f,-.02f,-.09f,-.96f,.72f,.25f,-.20f,.15f,.68f,-.90f,-.91f,-.99f,-.21f,-.59f,-.38f,-.02f,-.84f,-.50f,.68f,-.22f,-.88f,.25f,.98f,-.10f,-.37f,-.14f,.70f,.03f,.45f,.17f,-.01f,.33f,-.15f,.03f,-.66f,-.05f,-.10f,.35f,-.00f,-.12f,-.79f,.76f,.29f,-.27f,.79f,-.57f,.62f,.65f,.84f,-.17f,.23f,.74f,.98f,-.24f,-.96f,-.16f,.76f,-.22f,.35f,-.39f,.67f,.59f,.91f,.88f,.69f,-.72f,.43f,.55f,-.55f,.94f,.34f,-.74f,-.44f,-.48f,-.92f,-.62f,.79f,.52f,-.59f,-.14f,-.20f,.14f,.65f,-.27f,.71f,-.80f,.29f,-.15f,.15f,.30f,-.65f,-.17f,-.72f,-.90f,.99f,-.20f,.40f,-.59f,.46f,-.69f,.97f,-.86f,.35f,-.96f,.22f,.26f,-.87f,-.46f,-.54f,.30f,.28f,.63f,.43f,-.87f,-.16f,.40f,.07f,-.04f,.67f,-.30f,-.45f,.88f,.92f,.05f,.24f,.18f,-.29f,.26f,.57f,-.35f,.49f,-.16f,.32f,-.79f,-.29f,-.87f,-.72f,.40f,-.44f,-.69f,-.93f,-.85f,.92f,.00f,.66f,-.32f,.94f,.40f,.81f,.67f,-.72f,-.11f,.82f,-.83f,-.53f,-.01f,.18f,-.95f,-.41f,-.98f,-.08f,.24f,-.33f,.72f,-.81f,.21f,.46f,.25f,-.74f,.78f,.68f,-.20f,-.66f,.73f,-.09f,.98f,.50f,.38f,.95f,-.79f,.47f,.95f,-.28f,-.03f,.15f,.25f,.61f,.10f,-.42f,.98f,-.93f,-.47f,-.03f,.31f,.50f,-.24f,.42f,-.03f,-.75f,.75f,.86f,.07f,.79f,.39f,.58f,-.09f,-.04f,.63f,-.49f,.89f,.64f,.14f,-.70f,.04f,-.24f,.42f,.98f,-.88f,-.86f,.43f,-.17f,-.79f,-.15f,-.43f,.97f,-.88f,-.97f,.87f,-.61f,.56f,.53f,.94f,.26f,-.71f,.68f,-.19f,.87f,.00f,-.02f,.33f,.91f,-.36f,.60f,-.55f,-.04f,-.58f,-.07f,.18f,.42f,-.77f,-.66f,-.90f,-.23f,-.94f,.90f,-.37f,-.63f,.57f,.98f,-.46f,-.47f,.79f,.63f,-.05f,-.39f,-.51f,.22f,.23f,-.89f,-.01f,.98f,-.59f,-.85f,.12f,.24f,.63f,-.69f,.84f,-.15f,.48f,-.16f,.14f,.78f,.84f,.86f,-.80f,-.05f,-.42f,.32f,-.39f,-.27f,.85f,-.58f,.96f,-.53f,-.40f,-.75f,.76f,-.87f,-.39f,.73f,.48f,.87f,-.49f,.94f,-.25f,-.19f,-.30f,.56f,-.35f,.09f,.62f,.02f,.07f,-.32f,.83f,.87f,.96f,.50f,.16f,.12f,-.21f,-.77f,-.69f,.95f,-.84f,-.93f,-.83f,.68f,.06f,.48f,-.95f,.44f,.62f,.21f,-.62f,-.11f,.21f,-.74f,-.04f,.07f,.47f,-.07f,-.27f,.71f,.70f,-.55f,.13f,-.50f,.63f,-.51f,.48f,-.19f,.17f,-.34f,-.87f,-.28f,-.71f,.16f,-.56f,.39f,-.14f,.26f,-.09f,-.48f,-.10f,.50f,-.83f,.02f,-.39f,.19f,.10f,.73f,-.43f,-.47f,.79f,.75f,.89f,-.38f,-.72f,-.93f,-.22f,-.00f,.43f,.07f,-.96f,.08f,.63f,-.55f,-.55f,.83f,-.12f,-.70f,.03f,-.51f,.61f,.49f,.95f,.93f,.69f,.88f,-.10f,.50f,-.05f,-.59f,.36f,-.85f,-.02f,-.02f,.75f,-.77f,-.49f,.16f,.75f,-.08f,-.44f,.17f,-.87f,-.19f,-.60f,.03f,-.30f,-.21f,-.58f,.08f,-.47f,-.03f,.39f,.61f,.71f,.10f,.18f,.53f,-.62f,-.72f,.72f,-.87f,-.45f,-.57f,.14f,.65f,-.14f,-.65f,.17f,-.89f,-.35f,.22f,-.82f,-.23f,-.56f,.25f,.82f,.98f,.24f,.76f,-.27f,-.97f,-.14f,-.99f,.95f,.73f,-.37f,-.26f,.73f,-.51f,.42f,-.25f,.76f,.59f,-.54f,.18f,.12f,.41f,.41f,-.45f,.44f,-.26f,-.10f,-.21f,-.33f,.82f,-.88f,.33f,-.83f,-.47f,.08f,.36f,-.59f,-.77f,.70f,-.23f,.72f,.77f,-.99f,.36f,.16f,.62f,.35f,-.99f,.99f,-.32f,.39f,-.09f,.15f,.97f,-.42f,-.51f,-.98f,-.23f,-.81f,-.62f,.36f,-.63f,.33f,-.67f,-.84f,.17f,.26f,-.57f,.77f,-.86f,.54f,-.11f,.97f,-.29f,.45f,-.59f,-.39f,.21f,-.40f,.03f,.36f,.41f,.56f,.76f,.71f,-.62f,-.49f,.99f,.03f,.55f,-.15f,.03f,.62f,-.62f,-.02f,-.53f,.64f,-.24f,.14f,.67f,-.58f,-.90f,.66f,.44f,-.15f,.16f,.92f,.84f,.61f,.45f,.01f,-.23f,.51f,.65f,-.04f,-.61f,-.66f,.76f,-.52f,.63f,.82f,.67f,.81f,-.08f,.14f,.91f,.20f,-.82f,-.90f,.70f,.65f,.87f,.38f,.77f,.52f,.40f,-.57f,.19f,.58f,.50f,.78f,-.09f,-.93f,.67f,.06f,.11f,.36f,-.40f,.37f,.53f,.01f,-.09f,.89f,-.87f,.99f,-.54f,.92f,-.45f,.69f,-.22f,.92f,-.01f,.04f,.68f,-.96f,-.56f,.26f,.24f,.77f,-.31f,.23f,.26f,-.09f,-.12f,-.57f,.35f,-.80f,.36f,.36f,.55f,.85f,-.60f,.12f,-.96f,-.54f,-.58f,-.21f,.62f,.60f,.64f,.70f,-.14f,-.63f,-.14f,.55f,.25f,.21f,-.19f,.42f,-.61f,-.08f,-.38f,-.86f,.08f,.48f,-.58f,.82f,.14f,.50f,.81f,.07f,.24f,.03f,.54f,.47f,-.97f,-.80f,-.76f,-.86f,.80f,-.14f,-.04f,-.54f,.61f,-.42f,-.90f,.94f,-.84f,.15f,-.72f,.52f,.10f,-.48f,.42f,-.63f,.93f,-.80f,-.17f,-.29f,-.87f,-.24f,.99f,.75f,.78f,-.39f,-.94f,.46f,.34f,-.93f,.87f,-.18f,.20f,.33f,.91f,.12f,.62f,-.19f,-.50f,-.20f,.73f,.81f,-.04f,-.26f,.16f,-.24f,.60f,-.69f,.59f,.18f,.02f,-.00f,-.28f,.48f,-.29f,-.97f,.11f,-.34f,-.22f,.74f,.53f,.11f,-.97f,-.50f,-.46f,-.96f,-.93f,-.02f,-.23f,.04f,-.30f,-.79f,-.46f,.03f,-.18f,.71f,.73f,.27f,.76f,.65f,.52f,.34f,.54f,-.36f,.02f,.90f,-.24f,-.44f,-.81f,-.22f,.50f,-.25f,-.94f,.91f,-.57f,-.38f,.95f,-.05f,.46f,-.74f,-.88f,.59f,.86f,-.52f,-.32f,.30f,-.43f,.68f,.69f,-.88f,.92f,-.29f,-.78f,.89f,-.96f,-.91f,-.80f,.14f,-.36f,-.86f,.79f,-.39f,-.81f,.86f,.43f,-.77f,.44f,-.40f,-.91f,-.43f,-.29f,-.23f,-.07f,-.76f,.14f,.88f,.59f,-.36f,-.80f,-.81f,.74f,.49f,-.41f,.10f,-.45f,.89f,.99f,.95f,-.84f,-.19f,-.50f,-.00f,.17f,-.27f,-.14f,-.27f,-.34f,-.22f,.10f,.78f,-.90f,.64f,.98f,-.78f,.06f,.61f,.51f,-.34f,.70f,-.17f,-.65f,.84f,-.56f,-.81f,.26f,-.08f,.84f,.64f,.96f,-.26f,.73f,.68f,.71f,-.53f,-.04f,.32f,.99f,-.73f,.65f,-.57f,.44f,-.66f,-.20f,-.43f,-.54f,-.97f,.41f,.24f,-.53f,-.55f,-.26f,-.71f,.51f,.83f,.53f,.01f,.95f,.45f,-.40f,.31f,.59f,.34f,-.45f,-.92f,-.27f,.54f,-.14f,-.87f,-.60f,.72f,-.70f,-.04f,.18f,.91f,.68f,-.15f,.21f,.86f,.54f,-.86f,.70f,.40f,.64f,-.82f,-.70f,.68f,.96f,.71f,.84f,-.89f,-.47f,-.63f,-.75f,.79f,-.77f,.14f,.02f,.19f,-.84f,-.19f,.63f,.32f,.65f,-.16f,.44f,-.23f,-.01f,.67f,.85f,.08f,.32f,.14f,.70f,.40f,-.03f,-.29f,.78f,-.17f,-.68f,.36f,.28f,-.48f,.65f,.74f,-.37f,.81f,-.95f,-.21f,.04f,-.23f,-.24f,-.27f,-.93f,.19f,-.27f,-.83f,.19f,-.21f,-.08f,-.19f,.73f,.26f,-.97f,.51f,.92f,-.53f,-.18f,-.54f,.89f,-.49f,-.61f,-.54f,-.63f,-.01f,-.77f,.96f,.44f,-.64f,.63f,.24f,-.51f,.67f,-.16f,-.99f,.61f,.73f,.53f,.84f,-.06f,-.39f,.13f,.40f,-.26f,.90f,.94f,.15f,.71f,.70f,-.71f,-.53f,.74f,.47f,-.45f,.45f,-.24f,.44f,.90f,-.71f,.22f,.32f,-.61f,-.78f,-.66f,-.44f,.88f,-.58f,.57f,.08f,.57f,-.61f,.71f,-.12f,.49f,-.41f,-.69f,-.47f,-.46f,.73f,-.19f,-.01f,.44f,-.41f,.74f,.15f,.15f,.10f,.10f,.59f,.16f,-.80f,-.75f,.79f,-.22f,-.75f,-.05f,.22f,-.92f,-.11f,-.86f,.81f,-.29f,-.08f,-.78f,-.65f,.19f,-.35f,-.76f,.91f,-.55f,-.39f,-.23f,.34f,-.07f,-.95f,.10f,.98f,-.12f,-.49f,-.79f,-.00f,.57f,.83f,.21f,.28f,.33f,-.87f,.60f,-.00f,-.82f,-.00f,.61f,.06f,.45f,.65f,.75f,.04f,-.21f,-.92f,.25f,-.11f,-.52f,.79f,.19f,.48f,-.79f,-.26f,.47f,.05f,.98f,.89f,-.58f,-.03f,.22f,.20f,.17f,-.96f,.13f,.92f,-.31f,-.81f,-.30f,-.60f,.30f,.78f,.86f,-.40f,.23f,-.36f,-.35f,-.69f,.93f,.94f,.52f,.38f,.33f,-.97f,-.28f,.80f,.47f,-.27f,.06f,.25f,-.74f,.57f,.04f,-.93f,.88f,.65f,.68f,.95f,-.69f,.58f,-.67f,.73f,.85f,-.35f,-.10f,-.38f,-.57f,.67f,.79f,-.24f,.60f,.89f,-.85f,.20f,.11f,.14f,-.02f,.53f,-.56f,.39f,-.27f,.43f,.43f,-.41f,-.25f,.00f,-.13f,.26f,.08f,.39f,.07f,.04f,.70f,.49f,-.28f,.19f,-.48f,.29f,.52f,-.99f,.40f,.04f,.47f,.54f,.32f,.48f,.66f,.51f,.29f,-.58f,-.93f,.03f,-.23f,.62f,-.33f,-.51f,-.63f,.22f,.47f,.26f,-.04f,-.50f,.25f,-.61f,.92f,-.52f,.97f,-.12f,.98f,-.12f,.20f,-.04f,-.78f,-.69f,-.06f,-.55f,.38f,-.19f,-.35f,.84f,.46f,.68f,.17f,.27f,.32f,.79f,.37f,-.16f,-.39f,.55f,-.49f,-.70f,-.67f,-.53f,.80f,.35f,-.96f,-.06f,.63f,-.50f,-.16f,.35f,.39f,.51f,.65f,-.12f,.24f,-.88f,.08f,.56f,.92f,-.77f,.93f,.79f,-.06f,-.82f,.00f,-.67f,.49f,.83f,.35f,-.47f,-.67f,-.94f,-.23f,.71f,-.06f,-.06f,.98f,-.94f,-.79f,.03f,-.43f,-.64f,-.27f,-.32f,-.78f,.40f,-.39f,.96f,.81f,.36f,-.64f,.41f,.97f,-.08f,.01f,.31f,-.08f,-.85f,-.09f,-.40f,-.91f,-.11f,-.29f,.79f,-.40f,-.20f,-.92f,.72f,-.25f,.40f,-.20f,.47f,-.52f,.86f,.68f,-.68f,.15f,.99f,.30f,-.99f,-.51f,.44f,-.96f,-.71f,.83f,-.37f,-.12f,-.58f,-.11f,.60f,-.96f,-.35f,-.13f,.52f,.99f,-.94f,-.49f,-.88f,-.50f,.11f,-.79f,-.04f,-.03f,-.60f,-.16f,-.93f,-.85f,-.73f,-.47f,-.22f,-.27f,-.52f,.19f,-.39f,.96f,-.23f,.72f,-.19f,-.71f,.00f,-.78f,.55f,.90f,.47f,-.33f,-.02f,.04f,-.66f,-.34f,-.32f,-.34f,.34f,-.93f,.38f,-.43f,.70f,.90f,-.76f,-.16f,.54f,-.30f,.57f,.20f,-.68f,.62f,.03f,.58f,.41f,-.59f,.54f,.96f,-.04f,-.84f,-.34f,-.76f,.89f,-.86f,-.84f,-.22f,-.61f,-.64f,-.25f,.07f,-.37f,-.85f,-.41f,.59f,.11f,-.97f,.68f,.28f,-.42f,-.19f,.79f,-.46f,.42f,.83f,.97f,-.18f,.16f,-.04f,-.22f,.04f,.14f,.96f,.61f,.72f,-.71f,.23f,-.85f,.06f,.20f,-.72f,.31f,.84f,-.57f,-.23f,-.31f,.17f,.39f,.56f,.31f,.46f,-.15f,-.11f,-.86f,.51f,-.43f,-.63f,.65f,.51f,-.54f,-.58f,.13f,.55f,-.77f,-.64f,-.81f,.18f,-.08f,.21f,-.13f,.18f,.64f,.55f,-.73f,-.35f,.39f,-.79f,.68f,.52f,-.68f,.69f,.42f,-.64f,-.98f,.76f,-.56f,.76f,-.06f,-.64f,-.99f,-.16f,.19f,.90f,-.31f,-.98f,-.71f,-.36f,-.19f,-.03f,.98f,-.34f,-.50f,.91f,-.38f,.20f,-.13f,.62f,-.09f,.66f,-.69f,.25f,-.81f,-.20f,-.16f,-.23f,.26f,.69f,.05f,-.88f,-.71f,-.02f,.96f,.56f,.48f,-.50f,-.68f,-.83f,-.17f,.38f,.48f,.04f,-.05f,-.47f,.46f,.85f,.13f,.08f,.39f,.89f,-.36f,.05f,-.60f,-.95f,-.95f,.08f,.76f,.82f,-.62f,.32f,-.09f,.55f,-.26f,.08f,-.88f,-.93f,-.96f,.26f,.08f,.98f,.40f,.34f,-.34f,-.38f,.35f,.68f,.28f,-.58f,.66f,.30f,-.66f,-.25f,.42f,.05f,-.19f,.33f,.47f,.39f,-.03f,.63f,.09f,.65f,.83f,-.67f,-.86f,.72f,-.20f,.35f,-.88f,-.37f,-.52f,-.29f,-.83f,-.02f,-.08f,.25f,-.75f,.42f,-.23f,-.37f,.48f,-.75f,-.11f,.87f,.87f,.03f,.33f,-.26f,-.43f,.65f,.45f,.77f,.99f,-.52f,.97f,-.72f,.74f,.38f,.86f,-.26f,.81f,.94f,.28f,.16f,-.69f,-.50f,-.78f,-.40f,.11f,-.10f,.89f,.88f,.57f,-.75f,-.55f,-.49f,.06f,.18f,-.56f,.18f,.92f,-.63f,.94f,.01f,.39f,.46f,.44f,.90f,.16f,.77f,-.58f,-.69f,.88f,.12f,.10f,.27f,-.89f,-.94f,-.64f,.55f,-.87f,-.70f,-.47f,.93f,.73f,-.87f,-.20f,-.21f,.61f,-.98f,-.67f,-.45f,.82f,.77f,-.38f,-.29f,-.61f,-.78f,.93f,-.74f,-.48f,.51f,.03f,.73f,-.53f,-.55f,.99f,.08f,.99f,.88f,.57f,.04f,-.98f,.12f,-.39f,.47f,-.54f,-.65f,-.29f,-.51f,-.92f,-.23f,.73f,-.76f,.50f,.11f,.38f,-.44f,.04f,-.40f,-.27f,-.95f,-.83f,.77f,-.20f,-.27f,-.36f,.59f,.65f,-.69f,-.53f,-.86f,-.55f,-.68f,.80f,.55f,.69f,-.41f,-.49f,.95f,.99f,.01f,-.17f,-.72f,-.70f,-.05f,-.56f,.92f,-.37f,.00f,-.38f,-.18f,-.63f,-.46f,-.45f,.82f,.54f,.31f,.41f,-.00f,.08f,-.91f,.16f,.35f,-.62f,-.36f,.22f,.26f,-.31f,-.91f,.48f,.41f,.87f,-.34f,.69f,.24f,-.83f,.90f,.76f,.17f,.11f,.85f,.03f,-.66f,-.29f,.58f,.31f,-.87f,-.90f,-.48f,.21f,-.34f,-.87f,.60f,.91f,-.85f,-.59f,.20f,.64f,-.55f,-.20f,.77f,-.70f,.36f,.44f,-.66f,-.12f,-.56f,-.47f,.37f,-.45f,.43f,.93f,.76f,.03f,-.42f,-.12f,.91f,-.25f,-.93f,-.71f,.00f,.32f,.41f,-.72f,.38f,-.29f,-.83f,.71f,-.73f,-.68f,.81f,.83f,.08f,.74f,-.37f,-.17f,.01f,-.97f,-.35f,-.79f,.80f,-.27f,.28f,-.50f,-.55f,.70f,.36f,.68f,-.25f,-.08f,.84f,-.28f,-.92f,.54f,-.50f,.82f,.23f,-.01f,.09f,.61f,.42f,-.48f,.01f,.56f,.96f,.28f,.67f,-.48f,-.98f,-.56f,-.05f,-.55f,.91f,.77f,-.98f,.55f,.89f,.56f,.58f,.42f,.87f,.40f,.28f,-.51f,.74f,.99f,.77f,.55f,.48f,.58f,-.28f,-.16f,.69f,.19f,-.15f,-.67f,-.89f,-.63f,-.49f,.69f,.14f,.41f,.37f,.96f,.11f,-.89f,-.19f,.83f,.67f,-.51f,.17f,-.79f,.53f,-.27f,-.41f,-.93f,-.01f,.31f,.17f,.20f,.79f,.84f,.90f,-.02f,.59f,-.77f,-.47f,.59f,-.99f,-.88f,.78f,.43f,.90f,.08f,.71f,-.12f,.91f,-.82f,-.86f,-.61f,.28f,.35f,.60f,.55f,-.60f,-.67f,.67f,.99f,-.30f,.24f,.47f,-.17f,-.82f,-.22f,.40f,.01f,-.75f,.45f,.38f,.70f,-.44f,.96f,.72f,.12f,-.58f,.96f,-.41f,-.56f,-.49f,-.95f,-.36f,.62f,-.10f,-.65f,-.51f,-.13f,-.44f,.21f,.75f,-.82f,.75f,-.76f,.20f,-.63f,-.95f,-.10f,-.30f,.85f,.59f,-.76f,-.36f,.75f,-.05f,-.47f,-.67f,-.84f,-.24f,-.22f,.24f,.88f,-.33f,-.60f,-.81f,-.22f,-.55f,-.10f,.84f,-.52f,.31f,.31f,.73f,-.47f,.08f,-.78f,.59f,.27f,.06f,.74f,-.58f,.48f,-.74f,-.60f,-.57f,.16f,-.91f,.48f,-.08f,.42f,.56f,-.31f,-.42f,-.33f,.88f,.84f,.72f,.92f,.36f,.95f,-.40f,.77f,.12f,-.63f,-.67f,-.02f,-.65f,.53f,-.98f,.23f,-.92f,-.88f,-.64f,.84f,.44f,-.08f,-.62f,-.85f,-.16f,.61f,-.11f,.23f,.01f,-.37f,-.36f,.42f,-.73f,.43f,.96f,-.67f,.41f,.94f,-.27f,.04f,-.46f,.26f,.01f,.75f,-.10f,-.41f,-.28f,.57f,-.55f,.18f,-.36f,-.96f,-.15f,-.04f,.80f,.09f,-.84f,.06f,-.63f,.82f,.47f,-.17f,.01f,-.81f,.44f,.19f,-.26f,.88f,.57f,-.14f,-.00f,.34f,.44f,-.20f,-.58f,-.32f,.17f,-.98f,-.29f,-.08f,-.53f,.89f,-.96f,-.43f,.87f,.90f,.25f,-.00f,.55f,.73f,-.28f,.63f,-.73f,.61f,.14f,.51f,-.15f,.90f,.15f,-.24f,.48f,-.74f,.54f,.75f,-.59f,-.22f,.54f,-.99f,.29f,.47f,.87f,-.05f,.65f,.46f,.76f,-.89f,.63f,.65f,-.28f,-.63f,.54f,.37f,.03f,.05f,-.16f,-.38f,.60f,.70f,-.74f,.86f,-.41f,.22f,-.45f,.75f,-.30f,.32f,-.03f,.42f,.48f,.01f,-.79f,-.87f,-.76f,-.51f,-.02f,.00f,.40f,.35f,.78f,-.89f,-.66f,-.15f,-.41f,-.28f,.41f,.37f,.47f,-.56f,.27f,.58f,-.44f,.73f,-.55f,-.54f,.87f,-.67f,-.62f,-.95f,.52f,-.20f,.81f,-.34f,-.48f,.49f,-.23f,.62f,.74f,-.60f,-.39f,-.60f,.34f,-.81f,.17f,.63f,.37f,.11f,-.74f,.60f,-.47f,-.96f,-.42f,-.54f,.20f,.85f,.13f,.83f,.17f,-.56f,-.25f,.69f,-.11f,.11f,.09f,-.83f,-.02f,-.44f,-.50f,-.00f,-.35f,.25f,-.41f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.690599964918195308172244040179066359996795654296875f,1.071500032305717997616056891274638473987579345703125f,-1.324500101462007251029717735946178436279296875f,0.0f,0.0f,0.59459997982680778960684619960375130176544189453125f,1.7758000159829858688453896320424973964691162109375f,0.6737999885052448068023522864677943289279937744140625f,0.0f,0.0f,1.0509999876245854633083354201517067849636077880859375f,0.5475000176578763699808405363000929355621337890625f,0.274400058327615159470269645680673420429229736328125f,0.0f,0.0f,-0.6859999678805464018438442508340813219547271728515625f,1.548099946925045156120859246584586799144744873046875f,-0.7285999800950282434541804832406342029571533203125f,0.0f,0.0f,-0.62010000323653280407398824536358006298542022705078125f,3.2135999252960090188935282640159130096435546875f,1.4455999228596698724658153878408484160900115966796875f,0.0f,0.0f,0.0497999171957383968134536189609207212924957275390625f,-3.07150002109259379068362250109203159809112548828125f,-0.716099960313737238237763449433259665966033935546875f,0.0f,0.0f,0.45199992411956368965064712028834037482738494873046875f,1.0547999039337037086028203702881000936031341552734375f,2.1131000237405306307891805772669613361358642578125f,0.0f,0.0f,2.070100052037836757534705611760728061199188232421875f,0.0280999645084147875451208165031857788562774658203125f,0.62110000524073871730479368125088512897491455078125f,0.0f,0.0f,1.1483000440157946631103413892560638487339019775390625f,1.003799922101944464003508983296342194080352783203125f,-1.43590013820529005528214838705025613307952880859375f,0.0f,0.0f,0.0019000641390667372121470179990865290164947509765625f,-0.302599999846517864199313407880254089832305908203125f,-1.063999983523040260280367874656803905963897705078125f,0.0f,0.0f,-2.61560003206655355967313880682922899723052978515625f,-1.5433000056639325503482496060314588248729705810546875f,-1.0949999038688833508103925851173698902130126953125f,0.0f,0.0f,-1.2753000150181350580425032603670842945575714111328125f,0.4384000733852368991705361622734926640987396240234375f,-2.027300032940507090728488037711940705776214599609375f,0.0f,0.0f,-0.13580000218078491780460126392426900565624237060546875f,-0.2394000065274533017856128935818560421466827392578125f,0.061999980725348624588377788313664495944976806640625f,0.0f,0.0f,2.2935999279223384661463569500483572483062744140625f,-0.2886000541165467669912914061569608747959136962890625f,1.575900018996001517024296845193020999431610107421875f,0.0f,0.0f,0.518800010428577262899807465146295726299285888671875f,2.92520001021698217158473198651336133480072021484375f,0.2211000177204613947878897306509315967559814453125f,0.0f,0.0f,-1.051699960119278021153377267182804644107818603515625f,-0.0465999763101330177761383311008103191852569580078125f,-0.2159999837875365980011110877967439591884613037109375f,0.0f,0.0f,-0.1704000098168847454616070535848848521709442138671875f,1.686999952394515833731247766991145908832550048828125f,-2.687799959529936444369013770483434200286865234375f,0.0f,0.0f,-1.8978999859377740921928534589824266731739044189453125f,-0.50010001210272359006836495609604753553867340087890625f,-0.1813000306010239537357620065449737012386322021484375f,0.0f,0.0f,1.5977999894067635988648135025869123637676239013671875f,-0.710399987055361226140348662738688290119171142578125f,0.2915000199936326197303060325793921947479248046875f,0.0f,0.0f,-1.25490001829564601365518683451227843761444091796875f,0.558099989859015099824546268791891634464263916015625f,-1.0610000210776922013877765493816696107387542724609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.6828999545335765475329026230610907077789306640625f,-0.439300048199295911643957879277877509593963623046875f,2.40730003554821081479531130753457546234130859375f,0.0f,0.0f,-0.1406000335007906443252068129368126392364501953125f,-1.48570002058818939616458010277710855007171630859375f,1.952900005644559566775342318578623235225677490234375f,0.0f,0.0f,0.8280999464035028179154096505953930318355560302734375f,-0.92360002327114398834595476728281937539577484130859375f,-2.35969997509121842682588976458646357059478759765625f,0.0f,0.0f,0.342299985741078227619027529726736247539520263671875f,-1.2598999671101573927245453887735493481159210205078125f,-0.0541999783918261091031354226288385689258575439453125f,0.0f,0.0f,-0.6384000330775971843166871622088365256786346435546875f,-0.051100029306114347349421223043464124202728271484375f,2.97160002086460650616572820581495761871337890625f,0.0f,0.0f,2.73069997173100720289085074909962713718414306640625f,-2.226299895019084207348214476951397955417633056640625f,4.2592999447479851227171820937655866146087646484375f,0.0f,0.0f,-0.44269998229593010563576171989552676677703857421875f,-0.435100004018842856368110005860216915607452392578125f,1.486999973312019829307928375783376395702362060546875f,0.0f,0.0f,1.64280001838952127712900619371794164180755615234375f,2.5361999956995244787094634375534951686859130859375f,-0.287900019726157641031250022933818399906158447265625f,0.0f,0.0f,1.299999935477973878050761413760483264923095703125f,-2.558400074987113637092761564417742192745208740234375f,-0.66529996167197869016973754696664400398731231689453125f,0.0f,0.0f,-1.206899934461713286282247281633317470550537109375f,-1.27679992947429443717055619345046579837799072265625f,3.9708998740635816915300893015228211879730224609375f,0.0f,0.0f,-1.038900053879620255514737436897121369838714599609375f,0.85740001911521002142535508028231561183929443359375f,1.17659991281256193218496264307759702205657958984375f,0.0f,0.0f,0.3737000734835860260574236235697753727436065673828125f,1.7063000026985994939110469204024411737918853759765625f,-0.29959996321201243230092359226546250283718109130859375f,0.0f,0.0f,0.589999895989894884706927769002504646778106689453125f,-3.85369999677985841657346099964343011379241943359375f,1.3919000496849438519575414829887449741363525390625f,0.0f,0.0f,-3.27139998857528002673689115908928215503692626953125f,0.7734000435829155861711114994250237941741943359375f,0.69460001648366453963490130263380706310272216796875f,0.0f,0.0f,-0.826300026149302535571905536926351487636566162109375f,2.5345999637335534515614199335686862468719482421875f,-2.252299957540631059060842744656838476657867431640625f,0.0f,0.0f,0.7522000241905451023427531254128552973270416259765625f,-1.1541999841287731154437778968713246285915374755859375f,0.2175999631151548374674575825338251888751983642578125f,0.0f,0.0f,-0.703799982097744969422592475893907248973846435546875f,0.1455000409185889242991152059403248131275177001953125f,0.451799997702985844938439186080358922481536865234375f,0.0f,0.0f,0.09770003381669523090380380381247960031032562255859375f,-0.7283000160202377504248261175234802067279815673828125f,0.91790000538378968553132608576561324298381805419921875f,0.0f,0.0f,-0.5652998959392314670679979826672933995723724365234375f,-2.210100048983097753563242804375477135181427001953125f,0.22789998068511396756008480224409140646457672119140625f,0.0f,0.0f,2.269799942937493764105738591752015054225921630859375f,0.33569996845275174024436637409962713718414306640625f,1.5895000169351691798880210626521147787570953369140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.422500061392785397629268118180334568023681640625f,-0.619800003547966493755438932566903531551361083984375f,1.402499963641165692251888685859739780426025390625f,0.0f,0.0f,2.26859999261200329101484385319054126739501953125f,0.57149994945526128020674150320701301097869873046875f,-0.70049995505064732714828323878464289009571075439453125f,0.0f,0.0f,-0.0007999764576549495131985167972743511199951171875f,1.3244000104948867591048156100441701710224151611328125f,0.4790999866664407136340741999447345733642578125f,0.0f,0.0f,-1.0876999902307982193150337479892186820507049560546875f,0.784600022928417129008948904811404645442962646484375f,1.171600033903121840239691664464771747589111328125f,0.0f,0.0f,1.62439998845979527430927191744558513164520263671875f,-0.61240004327371622139253304339945316314697265625f,-2.527699941168725761286850683973170816898345947265625f,0.0f,0.0f,0.92970001100301669083592059905640780925750732421875f,0.3551000240981583289112677448429167270660400390625f,-0.381400023630261930662754821241833269596099853515625f,0.0f,0.0f,1.952900062641500511517733684740960597991943359375f,-0.164399970969558495426099398173391819000244140625f,-0.1034000531196586081250643474049866199493408203125f,0.0f,0.0f,0.3916999859541661077599883356015197932720184326171875f,2.08049998805671965129704403807409107685089111328125f,2.63299991764873286825832110480405390262603759765625f,0.0f,0.0f,-0.88599998557567560819592245024978183209896087646484375f,-0.2912999884307385656256883521564304828643798828125f,1.934100024794786154558323687524534761905670166015625f,0.0f,0.0f,1.2230000848025088799886361812241375446319580078125f,-0.6130000238008792567967475406476296484470367431640625f,0.073100027558952174189244033186696469783782958984375f,0.0f,0.0f,-1.485799983870982554634565531159751117229461669921875f,-0.368599995070695740650990046560764312744140625f,0.996699951234458314530684219789691269397735595703125f,0.0f,0.0f,-1.687399902439117749253227884764783084392547607421875f,0.5654999838471415074536707834340631961822509765625f,1.0655999530158937194102009016205556690692901611328125f,0.0f,0.0f,0.5510000362023708131431476431316696107387542724609375f,1.718100013756752364457724979729391634464263916015625f,-0.538700005944073634367441627546213567256927490234375f,0.0f,0.0f,-1.0547000126421448573665884396177716553211212158203125f,-0.24540002315342468452996627092943526804447174072265625f,-1.5912999701395624807531703481799922883510589599609375f,0.0f,0.0f,-0.4996000413872303536066965534701012074947357177734375f,2.536200008756667845233323532738722860813140869140625f,2.50869999683573752236043219454586505889892578125f,0.0f,0.0f,0.8521999818712473473425461634178645908832550048828125f,-0.2390000296980137051150450133718550205230712890625f,-0.51869995695650505940221819400903768837451934814453125f,0.0f,0.0f,1.0624000134527680216933731571771204471588134765625f,-0.047900004042684962701770245985244400799274444580078125f,-0.45770000502020113941625822917558252811431884765625f,0.0f,0.0f,-0.25639998952522879971382963049109093844890594482421875f,0.8615999793462456590731335381860844790935516357421875f,-1.060300027143954881836407366790808737277984619140625f,0.0f,0.0f,-0.5014999674111606520199302394757978618144989013671875f,0.6291000306993723700799137077410705387592315673828125f,1.742499987781048087498447785037569701671600341796875f,0.0f,0.0f,-0.84219997002482482884033743175677955150604248046875f,1.039400001069903378692060869070701301097869873046875f,-0.35429999757260066672870379989035427570343017578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.713400008621067005520899328985251486301422119140625f,1.205200019809603784182172603323124349117279052734375f,-0.0435999684721231073325498073245398700237274169921875f,0.0f,0.0f,-0.100200003492832312446125797578133642673492431640625f,-1.1406000217288727593967223583604209125041961669921875f,-0.2869999648556114113517878649872727692127227783203125f,0.0f,0.0f,1.48600000362470741066545087960548698902130126953125f,2.027699994812906414409781064023263752460479736328125f,1.349500017195939705061391578055918216705322265625f,0.0f,0.0f,-2.0324999762326481089758090092800557613372802734375f,-2.629000004142522328720588120631873607635498046875f,0.542700106087326883397281562793068587779998779296875f,0.0f,0.0f,0.0927999766290186212103208163171075284481048583984375f,-0.2907999872609978719850687411963008344173431396484375f,-1.492000022187828900399608755833469331264495849609375f,0.0f,0.0f,-3.751199963364005185439964407123625278472900390625f,-0.952100021177529232119240987231023609638214111328125f,1.068000013984740093064829125069081783294677734375f,0.0f,0.0f,0.6993001205630597905127388003165833652019500732421875f,-3.03159998674094754989027933334000408649444580078125f,-1.03610010422170173427502959384582936763763427734375f,0.0f,0.0f,-1.0373000827848912752671139969606883823871612548828125f,-0.461900038546322999621907001710496842861175537109375f,0.914999999701977539956487817107699811458587646484375f,0.0f,0.0f,0.1103999016530823329418353750952519476413726806640625f,-2.379499931663275447135674767196178436279296875f,-1.271099972569942782030238959123380482196807861328125f,0.0f,0.0f,2.30399993220716670094816436176188290119171142578125f,-0.59580002864897274594824239102308638393878936767578125f,2.777499985210597710505453505902551114559173583984375f,0.0f,0.0f,1.3572999733358612761691119885654188692569732666015625f,-1.0920000170096761937799101360724307596683502197265625f,0.9267999245196580915262529742904007434844970703125f,0.0f,0.0f,0.246999974429606583559149157736101187765598297119140625f,-0.026100017981230738683962044888176023960113525390625f,0.9395999889910224833755592044326476752758026123046875f,0.0f,0.0f,-0.94120003990009448724407548070303164422512054443359375f,0.56540005764961265288803815565188415348529815673828125f,0.104900048917530686054533362039364874362945556640625f,0.0f,0.0f,-2.291299954828620588642706934479065239429473876953125f,-1.1778998708263046513167182638426311314105987548828125f,2.1900999947242443255390753620304167270660400390625f,0.0f,0.0f,-1.378900126690416616526135840103961527347564697265625f,-2.987699988889694235894012308563105762004852294921875f,1.732299937833845060453086261986754834651947021484375f,0.0f,0.0f,0.2227999247543512861824410720146261155605316162109375f,0.8903000325083725297048431457369588315486907958984375f,0.03269998937398310712154625434777699410915374755859375f,0.0f,0.0f,-0.5860999812126157504366119610494934022426605224609375f,-1.2870999636776747632183059977251105010509490966796875f,0.60869994764328083647342282347381114959716796875f,0.0f,0.0f,0.30080000488162195182439972995780408382415771484375f,-1.10270003348141809595972517854534089565277099609375f,-0.371399904495476551602450854261405766010284423828125f,0.0f,0.0f,-1.244500032767652530907298569218255579471588134765625f,1.929099978061020070896347533562220633029937744140625f,-0.663199995274840414793970921891741454601287841796875f,0.0f,0.0f,-0.83779994410723535036567000133800320327281951904296875f,-0.520899986306578188788307670620270073413848876953125f,0.27149997370690126530234920210205018520355224609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.641300063234567563341670393128879368305206298828125f,0.3370000192821025475353735600947402417659759521484375f,0.106999998271465113219846898573450744152069091796875f,0.0f,0.0f,0.39589998040199281259532426702207885682582855224609375f,-1.3645999584808947702185832895338535308837890625f,-1.6522999623835088467416198909631930291652679443359375f,0.0f,0.0f,1.89249997422099180965915365959517657756805419921875f,-0.1635999578550462185688729732646606862545013427734375f,0.42299998746067313248886421206407248973846435546875f,0.0f,0.0f,-0.5168999931722872975825566754792816936969757080078125f,-0.2084000173568723024430937584838829934597015380859375f,-0.857200048702955630375299733714200556278228759765625f,0.0f,0.0f,-2.77519995951578035686679868376813828945159912109375f,1.186499997623264857082858725334517657756805419921875f,-3.5873999753616754304630376282148063182830810546875f,0.0f,0.0f,0.4817999769717451474804192912415601313114166259765625f,1.95850000036507854161982322693802416324615478515625f,1.4564999818652870633428619839833118021488189697265625f,0.0f,0.0f,-0.665799963530897986885292993974871933460235595703125f,2.462600008247047167486698526772670447826385498046875f,0.4706001155316830164565544691868126392364501953125f,0.0f,0.0f,0.850999998986721184479620205820538103580474853515625f,-2.06280001593083195388089734478853642940521240234375f,-0.542500034123659435891795510542578995227813720703125f,0.0f,0.0f,0.0680000768303867797470729783526621758937835693359375f,1.8373000284515315083666564532904885709285736083984375f,0.27480000222548894850405076795141212642192840576171875f,0.0f,0.0f,1.2674999936297532965312484520836733281612396240234375f,1.1901999861329797258946427973569370806217193603515625f,-3.451799985279143445637828335748054087162017822265625f,0.0f,0.0f,-2.92209998902827461364495320594869554042816162109375f,-0.571199909198284228040165544371120631694793701171875f,1.1026999698162089913466843427158892154693603515625f,0.0f,0.0f,0.9130000208765263192134398195776157081127166748046875f,1.279300104190409737014988422743044793605804443359375f,-0.3617000674821431172034635892487131059169769287109375f,0.0f,0.0f,-1.9845999451816087599809179664589464664459228515625f,3.3376000361680979722223128192126750946044921875f,-1.010699966031314289693909813649952411651611328125f,0.0f,0.0f,-0.02349998587369750424613812356255948543548583984375f,-0.58559997859001011732971164747141301631927490234375f,-0.1924000212199985515582056905259378254413604736328125f,0.0f,0.0f,-1.6874000052571285213076635045581497251987457275390625f,-2.118199939730762704215294434106908738613128662109375f,-0.24429995409846283482835360700846649706363677978515625f,0.0f,0.0f,0.96990000000447007533210808105650357902050018310546875f,1.59700007110089092776661345851607620716094970703125f,0.4952999724566946593284910704824142158031463623046875f,0.0f,0.0f,-0.01450000795721972934870791505090892314910888671875f,1.700700034725665776846881271922029554843902587890625f,1.356500016249715390159735761699266731739044189453125f,0.0f,0.0f,2.3809000113964078337858154554851353168487548828125f,-2.799199957641958835807827199460007250308990478515625f,-1.4057000035077340527323030983097851276397705078125f,0.0f,0.0f,0.1545999670863149599853159088524989783763885498046875f,0.714100010471045987259230969357304275035858154296875f,-0.7105999824270592402086776928626932203769683837890625f,0.0f,0.0f,-1.744999984167516782207485448452644050121307373046875f,0.4727999772623177054953202969045378267765045166015625f,0.1883000063717370498039826998137868940830230712890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.52779999838471436390818780637346208095550537109375f,0.297300023403763713503167309681884944438934326171875f,-0.47200002315640432470900123007595539093017578125f,0.0f,0.0f,0.077799966496229000512130369315855205059051513671875f,-0.138099988871812850987907950184307992458343505859375f,0.59169999277144658123006593086756765842437744140625f,0.0f,0.0f,-0.246899959346652320135717673110775649547576904296875f,0.198699950549006576494548426126129925251007080078125f,0.3466000057399274414393630650010891258716583251953125f,0.0f,0.0f,0.515399982920289101429034417378716170787811279296875f,-0.445699978734552804127133640577085316181182861328125f,-0.5173000233292572858090352383442223072052001953125f,0.0f,0.0f,-0.696300036375224973284048246569000184535980224609375f,0.2499000362887981996351527413935400545597076416015625f,0.86739994054883684526657816604711115360260009765625f,0.0f,0.0f,0.147099985861778304041536102886311709880828857421875f,0.9265999872624863797909711138345301151275634765625f,-0.96459998481869668740529277783934958279132843017578125f,0.0f,0.0f,-1.0324000342026351528801342283259145915508270263671875f,-1.039500036530196513950841108453460037708282470703125f,0.009400042383371387533230745248147286474704742431640625f,0.0f,0.0f,-1.8921999456614262413722826750017702579498291015625f,0.825199981029331564030826484668068587779998779296875f,-0.85369998612552944194931114907376468181610107421875f,0.0f,0.0f,0.838700035038590474556485787616111338138580322265625f,0.808699973869324129083224761416204273700714111328125f,-0.5673000402048227197582264125230722129344940185546875f,0.0f,0.0f,-0.415799969640375355339756424655206501483917236328125f,0.027899978170543737743258816408342681825160980224609375f,0.72199997761473022617195738348527811467647552490234375f,0.0f,0.0f,0.854099977576732616313393009477294981479644775390625f,0.53760000442862476699446006023208610713481903076171875f,0.0028999759912495814972999141900800168514251708984375f,0.0f,0.0f,0.7826999334573738753562111014616675674915313720703125f,0.05849997771531345591711215092800557613372802734375f,-0.773999998833983848811612915596924722194671630859375f,0.0f,0.0f,1.3684999537616973075415671701193787157535552978515625f,0.4950999472081665242484405098366551101207733154296875f,-0.27900002582371186132803586588124744594097137451171875f,0.0f,0.0f,0.24210000825077326869205762704950757324695587158203125f,0.4446999769538642066635247829253785312175750732421875f,-0.255600023144483490256106961169280111789703369140625f,0.0f,0.0f,0.75379999731555591413467709571705199778079986572265625f,-1.079599967011809713568482038681395351886749267578125f,1.0846999481201178294753617592505179345607757568359375f,0.0f,0.0f,-0.254400008334219662753383772724191658198833465576171875f,0.33409998271763330368600009023793973028659820556640625f,0.862000007770954201902213753783144056797027587890625f,0.0f,0.0f,-0.02850000330805801951328248833306133747100830078125f,0.704999992102384620551447369507513940334320068359375f,-0.2891999734744425953891777680837549269199371337890625f,0.0f,0.0f,-0.14360000274479400861338262984645552933216094970703125f,0.0304000472187991588324251779340556822717189788818359375f,-0.535999976485967533079701752285473048686981201171875f,0.0f,0.0f,-0.594500012688338852484548624488525092601776123046875f,-0.84790000363290285445572180833551101386547088623046875f,-0.18269999652653989841155635076574981212615966796875f,0.0f,0.0f,0.8021999647721653747822756486129947006702423095703125f,0.092999983064830538115330682558123953640460968017578125f,-0.7724999567121273447156681868364103138446807861328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.81510000528544213072024149369099177420139312744140625f,0.198700006093085246305918190046213567256927490234375f,-0.210800014567374793816867395435110665857791900634765625f,0.0f,0.0f,0.0641999738469720693245790243963710963726043701171875f,-0.9609999982044092892152775675640441477298736572265625f,1.197399994304775727727019329904578626155853271484375f,0.0f,0.0f,-0.07150002491101570267773013256373815238475799560546875f,-0.4782999761223789558783892061910592019557952880859375f,-1.3271000342160463869589648311375640332698822021484375f,0.0f,0.0f,0.1320000646188839166228490284993313252925872802734375f,2.83790000512302054147539820405654609203338623046875f,-0.5761000098973523275702746104798279702663421630859375f,0.0f,0.0f,-0.690799985398352855980874664965085685253143310546875f,0.00599994463473552119836540441610850393772125244140625f,0.273399906492232336319148089387454092502593994140625f,0.0f,0.0f,4.18759999530166293624233730952255427837371826171875f,1.6659000653013593673534842309891246259212493896484375f,1.152599949331581274947211568360216915607452392578125f,0.0f,0.0f,2.43169991727843903817074533435516059398651123046875f,0.707700078780949315415682576713152229785919189453125f,2.5347000761955964520666384487412869930267333984375f,0.0f,0.0f,0.21169998312294513453934996505267918109893798828125f,-0.6825000155717131367083538862061686813831329345703125f,0.979900042919814584507776089594699442386627197265625f,0.0f,0.0f,0.70649998303875249927585855402867309749126434326171875f,0.623999917231499612313427860499359667301177978515625f,-0.249599964292347209493527770973742008209228515625f,0.0f,0.0f,-0.38739999858885998396118566233781166374683380126953125f,2.95779996414929602366328253992833197116851806640625f,-1.872699993509053140172682105912826955318450927734375f,0.0f,0.0f,1.8754999943152077523933485281304456293582916259765625f,0.58039999051988200307050647097639739513397216796875f,1.381599984692036375832913108752109110355377197265625f,0.0f,0.0f,-0.3486000315785402481338906000019051134586334228515625f,0.8671999028578394330679657286964356899261474609375f,-2.514199932831526762555540699395351111888885498046875f,0.0f,0.0f,-1.878899941208214574572821220499463379383087158203125f,1.0185000779256221381530167491291649639606475830078125f,0.364200002419948543064265322755090892314910888671875f,0.0f,0.0f,1.377699888772517144985840786830522119998931884765625f,-1.669799979519843713404725349391810595989227294921875f,-0.1289999806545683025404969157534651458263397216796875f,0.0f,0.0f,3.496199976812302434581170018645934760570526123046875f,2.62270001498535254569333119434304535388946533203125f,0.8190999018043270130107202930958010256290435791015625f,0.0f,0.0f,-3.803599927661569068959579453803598880767822265625f,2.272600078487397201598696483415551483631134033203125f,0.6347000676274296981915767901227809488773345947265625f,0.0f,0.0f,1.882599998393654772144145681522786617279052734375f,1.171199912942201937227082453318871557712554931640625f,-1.38159987796247119007375658839009702205657958984375f,0.0f,0.0f,0.925099991315605496566831789095886051654815673828125f,1.907500019706785732154230572632513940334320068359375f,1.35319998406172015137372000026516616344451904296875f,0.0f,0.0f,-1.22389996670782608134686597622931003570556640625f,-0.46319986209571339941248879767954349517822265625f,1.700199944707752042205584075418300926685333251953125f,0.0f,0.0f,0.6406000505626197050190739901154302060604095458984375f,0.0627000403918318571783174775191582739353179931640625f,-1.1676999362885955857649378231144510209560394287109375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,2.305499990999698045612831265316344797611236572265625f,-0.0350999810338026296818725313642062246799468994140625f,0.6914000377118585394242700203903950750827789306640625f,0.0f,0.0f,0.836000029906629915643634376465342938899993896484375f,-0.363500003308057539896935850265435874462127685546875f,0.7550999876275656941260194798815064132213592529296875f,0.0f,0.0f,-0.14150006270408699293739118729718029499053955078125f,1.0274000469431283644183849901310168206691741943359375f,-1.503800120492279734918383837793953716754913330078125f,0.0f,0.0f,-1.1788000202819690631628191113122738897800445556640625f,0.1751000454813242956930707805440761148929595947265625f,-0.1099998717755090726910793819115497171878814697265625f,0.0f,0.0f,2.70829988850355096730027071316726505756378173828125f,-1.4654000582829114218696986426948569715023040771484375f,0.85520002238005454753277945201261900365352630615234375f,0.0f,0.0f,-0.346599935443701046011710786842741072177886962890625f,0.25830002674907304172080557691515423357486724853515625f,-0.73420011265128781730027185403741896152496337890625f,0.0f,0.0f,3.281199942260235591362516061053611338138580322265625f,-0.0758999348416942165584941903944127261638641357421875f,-4.4509000767379998109163352637551724910736083984375f,0.0f,0.0f,-2.346900095915794626222350416355766355991363525390625f,1.08109995606690656444470732822082936763763427734375f,-1.480699999558925838982759159989655017852783203125f,0.0f,0.0f,-0.8472000399857784191937071227584965527057647705078125f,0.2552000580310831434616147816996090114116668701171875f,-0.39429999617561761748874005206744186580181121826171875f,0.0f,0.0f,-1.5246000051215296000606258530751802027225494384765625f,1.7075000318139785804305574856698513031005859375f,4.31159998107850572779398135025985538959503173828125f,0.0f,0.0f,0.1507999054163686736274030408822000026702880859375f,2.2392000386670236622421725769527256488800048828125f,-0.02009993279129396182014488658751361072063446044921875f,0.0f,0.0f,0.2073999066859479878388583529158495366573333740234375f,2.047100024232268111745725036598742008209228515625f,1.4571000345140696641266231381450779736042022705078125f,0.0f,0.0f,1.6097000204280025315028979093767702579498291015625f,-0.47609983153045210002574094687588512897491455078125f,2.7241999260514973002500482834875583648681640625f,0.0f,0.0f,0.6883000293940308988993592720362357795238494873046875f,-1.219200079794225732854329180554486811161041259765625f,0.72249997612088845411193460677168332040309906005859375f,0.0f,0.0f,1.9899000719770791878460158841335214674472808837890625f,0.0232999606810501413978187201792025007307529449462890625f,-0.9081001488357769435566524407477118074893951416015625f,0.0f,0.0f,0.0842000053256739544593756363610737025737762451171875f,1.0538000696420670720954149146564304828643798828125f,-1.0352999612435700438339836182422004640102386474609375f,0.0f,0.0f,-3.02809997099041883217296344810165464878082275390625f,-2.829899992367625571887401747517287731170654296875f,-1.53299996976554364636058380710892379283905029296875f,0.0f,0.0f,0.8633000109910959896097892851685173809528350830078125f,1.75029990674257351912501690094359219074249267578125f,0.37080003855824461567181060672737658023834228515625f,0.0f,0.0f,-0.4596000435292710850632147412397898733615875244140625f,-0.924799995017051568169108577421866357326507568359375f,-0.35929991118311865871959298601723276078701019287109375f,0.0f,0.0f,0.44689994701594148995837940674391575157642364501953125f,0.46429994687140041520478916936554014682769775390625f,0.345499986156821758953583412221632897853851318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.3766000176236035112964373183785937726497650146484375f,0.0729999935328959903557688448927365243434906005859375f,-0.2244999858289957739287956428597681224346160888671875f,0.0f,0.0f,-0.211000013962387822363808709269505925476551055908203125f,-0.239100002536177580481080440222285687923431396484375f,-0.152199995058774961709247008911916054785251617431640625f,0.0f,0.0f,-0.0333999813333158979133941102190874516963958740234375f,0.0518000022292135486878805750166065990924835205078125f,-0.35040001592636116356516140513122081756591796875f,0.0f,0.0f,-0.0929999732673161272344941608025692403316497802734375f,0.05879999211877573461748625049949623644351959228515625f,1.3683999991029498488614990492351353168487548828125f,0.0f,0.0f,1.7735999641880393351556222114595584571361541748046875f,0.471399994871020411313367048933287151157855987548828125f,-0.38259992865622127755642623014864511787891387939453125f,0.0f,0.0f,-0.26580003360360804354201036403537727892398834228515625f,0.7063000306382767323754023891524411737918853759765625f,0.7301999967128034452201745807542465627193450927734375f,0.0f,0.0f,2.0973000058576456439141111331991851329803466796875f,1.3372000532105572379037994323880411684513092041015625f,0.21560000444352578341522530536167323589324951171875f,0.0f,0.0f,-1.0901999684005978519252266778494231402873992919921875f,0.2277999880284067302227413165383040904998779296875f,1.0074999678507448042097394136362709105014801025390625f,0.0f,0.0f,-1.708999967262148089730544597841799259185791015625f,-0.983899959629774389924250499461777508258819580078125f,1.3509000363372261066530199968838132917881011962890625f,0.0f,0.0f,-1.441799989935756087788831791840493679046630859375f,0.93719999243244467290736565701081417500972747802734375f,-0.4716999926038087931345899050938896834850311279296875f,0.0f,0.0f,-2.580999952904881045157026164815761148929595947265625f,0.5726999885544181712049294219468720257282257080078125f,0.121600001344085051346866066523944027721881866455078125f,0.0f,0.0f,0.289900025299191010930144329904578626155853271484375f,0.4381999688997861852612913935445249080657958984375f,1.04769996195584536025080524268560111522674560546875f,0.0f,0.0f,0.817200004297494952965053016669116914272308349609375f,1.3942000101312992388358225070987828075885772705078125f,0.6182000267535452397993367412709631025791168212890625f,0.0f,0.0f,0.2727000298678874390390092230518348515033721923828125f,-0.039199994336067245370713862939737737178802490234375f,1.2173999601811169579690385944559238851070404052734375f,0.0f,0.0f,0.7149999866075820165178811294026672840118408203125f,1.2814999665915962623330415226519107818603515625f,-1.2193000339686868027655464175040833652019500732421875f,0.0f,0.0f,-1.329499944850802695128777486388571560382843017578125f,-0.64139998943209619586269809587975032627582550048828125f,0.3821000052705405547470718374825082719326019287109375f,0.0f,0.0f,1.291300027918814929961399684543721377849578857421875f,-0.59959997289776822970708280990947969257831573486328125f,-1.42299999655038078572033555246889591217041015625f,0.0f,0.0f,0.1165000243335960750101776284282095730304718017578125f,0.4954999923780565307396273055928759276866912841796875f,-0.91490005174875133064915644354186952114105224609375f,0.0f,0.0f,-0.58609997946172998073421922526904381811618804931640625f,0.7116000055909150745492297573946416378021240234375f,0.800100018584727568082826110185123980045318603515625f,0.0f,0.0f,-0.820899950823188451209944105357863008975982666015625f,0.54720000288188430648261828537215478718280792236328125f,-0.2323999797388915933282760306610725820064544677734375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.347100027510524711260586627759039402008056640625f,0.291300006759167562364609693759121000766754150390625f,-0.5050999419927599820567820643191225826740264892578125f,0.0f,0.0f,-0.539399990229308290423659855150617659091949462890625f,-1.9002999896302810167725283463369123637676239013671875f,-1.20899998737871694487466811551712453365325927734375f,0.0f,0.0f,-0.05429997769817707098383152697351761162281036376953125f,0.5652000776633616663247039468842558562755584716796875f,0.4457999926626687869912757378187961876392364501953125f,0.0f,0.0f,0.45390000615865044863994626211933791637420654296875f,-0.583099942305684404431076472974382340908050537109375f,2.118399986921251088034523490932770073413848876953125f,0.0f,0.0f,-0.85420000860392841790513784872018732130527496337890625f,-3.22810001502335097711693379096686840057373046875f,-0.92599989702552643056066017379635013639926910400390625f,0.0f,0.0f,1.46929997697174474780013042618520557880401611328125f,0.2041000928744673981185542288585565984249114990234375f,1.08650000458955720006315459613688290119171142578125f,0.0f,0.0f,1.738700036808103366325894967303611338138580322265625f,0.0558999400198476603662811612593941390514373779296875f,0.496800038290023504572445744997821748256683349609375f,0.0f,0.0f,-1.9225999152451738183344787103123962879180908203125f,-0.921799944226443290773431726847775280475616455078125f,-0.14639991395771534410386038871365599334239959716796875f,0.0f,0.0f,-2.439299959518760996246555805555544793605804443359375f,-0.6935000448822972174411916057579219341278076171875f,-1.6078999709993604394497879184200428426265716552734375f,0.0f,0.0f,-1.1439999807849512425406146576278842985630035400390625f,-0.25369998642355173767981568744289688766002655029296875f,0.104800004702807481038462356082163751125335693359375f,0.0f,0.0f,0.01659997299462412367887509390129707753658294677734375f,1.1756999251276258888765369192697107791900634765625f,-0.5546999569676824837216599917155690491199493408203125f,0.0f,0.0f,0.73529998519718642313591772108338773250579833984375f,-0.977900029337404941287559267948381602764129638671875f,0.180699944312871618734561707242392003536224365234375f,0.0f,0.0f,0.80519992947131324712017885758541524410247802734375f,0.8271000764608389399512589079677127301692962646484375f,0.15659997116923329851800872347666881978511810302734375f,0.0f,0.0f,-2.259500007342548411060079160961322486400604248046875f,-0.32879997747838363153505270020104944705963134765625f,-1.69769996912702847424725405289791524410247802734375f,0.0f,0.0f,-1.40699995787069109809408473665826022624969482421875f,-1.0209999157823614890361341167590580880641937255859375f,-2.192499932125211881128734603407792747020721435546875f,0.0f,0.0f,-0.28100004128739186626262380741536617279052734375f,1.4117000685438523266412857992690987884998321533203125f,2.353700077171623217253682014415971934795379638671875f,0.0f,0.0f,-0.6116000041007987331198592073633335530757904052734375f,0.2609999662041666557144026228343136608600616455078125f,-0.1455999382451189649856360119883902370929718017578125f,0.0f,0.0f,1.327600011990963224661754793487489223480224609375f,-0.280599963130056140325763180953799746930599212646484375f,0.7758999879270793709196141207939945161342620849609375f,0.0f,0.0f,-0.09639995746910534624163346961722709238529205322265625f,-0.7838999586611981040817909160978160798549652099609375f,-0.8544000161014497773948050962644629180431365966796875f,0.0f,0.0f,2.109699963207543049747982877306640148162841796875f,-0.3749999748356633855195241267210803925991058349609375f,0.3025999976485966858064102780190296471118927001953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.877900012312829769456357098533771932125091552734375f,0.15300004948675560711990328854881227016448974609375f,0.00480004933178335146948256806354038417339324951171875f,0.0f,0.0f,-0.01120002460777824726534390720189549028873443603515625f,-0.94829999009221754846521434956230223178863525390625f,-2.14819997020363739892445664736442267894744873046875f,0.0f,0.0f,-1.4181000042572622366066070753731764853000640869140625f,1.37290000739544648666878856602124869823455810546875f,0.4703999178394668856384441824047826230525970458984375f,0.0f,0.0f,-0.4101999905288218339904915410443209111690521240234375f,1.5974999836087224736047573969699442386627197265625f,-0.524100009316205817100353669957257807254791259765625f,0.0f,0.0f,-0.422800006636232961199084456893615424633026123046875f,-0.3072000295922145962634886018349789083003997802734375f,1.155899936853348464183000032790005207061767578125f,0.0f,0.0f,-1.37689998486489084683626060723327100276947021484375f,-0.73139995173216032497975902515463531017303466796875f,-4.15539999446868790755615918897092342376708984375f,0.0f,0.0f,-1.59369987686276548544128672801889479160308837890625f,1.3464999635182330894878077742760069668292999267578125f,-0.531699986699222915120799370924942195415496826171875f,0.0f,0.0f,-0.7112000512436036370189640365424565970897674560546875f,0.07240004124343268898655878729186952114105224609375f,-0.5236000928968198575574888309347443282604217529296875f,0.0f,0.0f,3.20290005995929316640058459597639739513397216796875f,-0.72659997839629764104785181189072318375110626220703125f,1.047799982906134186322333334828726947307586669921875f,0.0f,0.0f,1.72539997226595875901011822861619293689727783203125f,1.7153000715121640151750170844024978578090667724609375f,-0.12459994764030069003268863525590859353542327880859375f,0.0f,0.0f,-4.41829994535148262713164513115771114826202392578125f,0.214399989148975844699407389271073043346405029296875f,-4.7804998831711724704973676125518977642059326171875f,0.0f,0.0f,-0.8097000488519687788624423774308525025844573974609375f,2.67369998295903155138830697978846728801727294921875f,-1.73710001134276392775745989638380706310272216796875f,0.0f,0.0f,-0.4022999699085947344912028711405582726001739501953125f,1.260300081160664387169845213065855205059051513671875f,0.288300004397332276795395955559797585010528564453125f,0.0f,0.0f,1.8282000085368761421733552197110839188098907470703125f,-1.1779000056818127450952715662424452602863311767578125f,-2.621699946391581104165879878564737737178802490234375f,0.0f,0.0f,-1.0748999855130907388200967034208588302135467529296875f,-2.355999904271212752604469642392359673976898193359375f,1.56689994068294691231812976184301078319549560546875f,0.0f,0.0f,0.854799979296327006039746265741996467113494873046875f,2.366500020906328938252727311919443309307098388671875f,-2.435599950426816207027513883076608180999755859375f,0.0f,0.0f,1.287099962988494272764228298910893499851226806640625f,0.95690000021308652122797866468317806720733642578125f,-0.176600018052011531466405358514748513698577880859375f,0.0f,0.0f,-1.3836000126540655141837987684994004666805267333984375f,0.935799987410008693444751770584844052791595458984375f,0.3852999614298335817608176512294448912143707275390625f,0.0f,0.0f,1.455100020670890170748634773190133273601531982421875f,0.0706999816030275951561634428799152374267578125f,1.218800033432245299991336651146411895751953125f,0.0f,0.0f,1.046599940919877180789399062632583081722259521484375f,-1.3928000491231682378412415346247144043445587158203125f,0.0730999955773354859189794296980835497379302978515625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.59569997676014896370588758145458996295928955078125f,0.607500022873282308211173585732467472553253173828125f,-0.39020001243352897546401436557061970233917236328125f,0.0f,0.0f,-1.1944000069558617926901433747843839228153228759765625f,0.1307000231027597436650466988794505596160888671875f,0.7590999725103377837598372934735380113124847412109375f,0.0f,0.0f,-0.0969000198081138020000935284770093858242034912109375f,-0.99260004734396911629801252274774014949798583984375f,0.27660004045963315633116508251987397670745849609375f,0.0f,0.0f,1.04190001164972745328896053251810371875762939453125f,-0.3280999835819005117087954204180277884006500244140625f,-1.55749998375773390790755001944489777088165283203125f,0.0f,0.0f,-2.6319000019639720022723849979229271411895751953125f,0.30550008644163550997063794056884944438934326171875f,0.9458999748885636993378511760965920984745025634765625f,0.0f,0.0f,0.085099983865023265394711415865458548069000244140625f,-1.948100012117624402208093670196831226348876953125f,0.208300039750337617761033470742404460906982421875f,0.0f,0.0f,-0.80169999750256681902982336396235041320323944091796875f,-2.89489998372495183076580360648222267627716064453125f,-0.21780003891587274011953923036344349384307861328125f,0.0f,0.0f,-0.393300006148218361801127684884704649448394775390625f,-2.138599957035481669009868710418231785297393798828125f,-1.7765999513879424664963835311937145888805389404296875f,0.0f,0.0f,0.8992999997705222181565432038041763007640838623046875f,0.082300028246641954154938503052107989788055419921875f,-2.05410007320493459559429538785479962825775146484375f,0.0f,0.0f,0.29839996185004746820368382032029330730438232421875f,-0.8803999805174764770043793760123662650585174560546875f,1.72020000298991870835152440122328698635101318359375f,0.0f,0.0f,1.935599978254735287208632144029252231121063232421875f,-1.4367999866574994172196966246701776981353759765625f,-0.7419000037521124024664231910719536244869232177734375f,0.0f,0.0f,0.1325999817788601209400667357840575277805328369140625f,-2.251199920672178667047091948916204273700714111328125f,0.0949000243864952519601274616434238851070404052734375f,0.0f,0.0f,-0.665000027902424850623219754197634756565093994140625f,-1.9576999882936478858397322255768813192844390869140625f,-0.3966999517559999244298296616761945188045501708984375f,0.0f,0.0f,0.10379997796267304011053056456148624420166015625f,-0.598999974340200136424527954659424722194671630859375f,-0.214499977260828700309502892196178436279296875f,0.0f,0.0f,1.089400017647444318669158747070468962192535400390625f,-0.03639994573444216285906804841943085193634033203125f,-1.974299958457053971727646057843230664730072021484375f,0.0f,0.0f,0.23539997815191748831153972787433303892612457275390625f,1.7775999838650224926794862767565064132213592529296875f,-0.9740999969482422304878355134860612452030181884765625f,0.0f,0.0f,-1.709400005614757134253522963263094425201416015625f,-0.16679999105334297748726157806231640279293060302734375f,1.22589999928921411509463723632507026195526123046875f,0.0f,0.0f,-0.6842000097401437219701847425312735140323638916015625f,-0.4902000188410282621731539620668627321720123291015625f,1.45800000947713837007313486537896096706390380859375f,0.0f,0.0f,0.46380000150650768642179855305585078895092010498046875f,0.0339999965429298445229733260930515825748443603515625f,-0.350200013923644970503801232553087174892425537109375f,0.0f,0.0f,-0.2506999637961389026941105839796364307403564453125f,-0.2858000081598761621393123277812264859676361083984375f,1.7546999926000832470407431173953227698802947998046875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.19149997758120285151761663655634038150310516357421875f,-0.856899936398864081610327048110775649547576904296875f,-2.945399958799035378120834138826467096805572509765625f,0.0f,0.0f,0.00730000567138155975754898463492281734943389892578125f,0.97239995727539108028025793828419409692287445068359375f,-2.40820000034123626875270929303951561450958251953125f,0.0f,0.0f,-2.99940005650222385469305663718841969966888427734375f,0.9674999431893229751011631378787569701671600341796875f,0.40059998700916821245954224650631658732891082763671875f,0.0f,0.0f,0.7462000081419948305239131514099426567554473876953125f,-0.212099996478855690185127969016321003437042236328125f,-0.8252000329412521040239880676381289958953857421875f,0.0f,0.0f,-2.27699992194026545888618784374557435512542724609375f,-1.00039996045678858394012422650121152400970458984375f,-3.1355999944970012194289665785618126392364501953125f,0.0f,0.0f,-2.33299996335804582514583671581931412220001220703125f,-1.417699946514516806672645543585531413555145263671875f,3.5606000001966950918586007901467382907867431640625f,0.0f,0.0f,0.085800049957633017783109607989899814128875732421875f,1.532900024047493925394292091368697583675384521484375f,2.42159998113438401645680642104707658290863037109375f,0.0f,0.0f,-0.6383000107675780299132384243421256542205810546875f,-0.109999971874059021548220016484265215694904327392578125f,-0.4924999737367026231282807202660478651523590087890625f,0.0f,0.0f,-2.4110000010728835917461765347979962825775146484375f,2.45379998431429324767805155715905129909515380859375f,4.0996999567627909755174187012016773223876953125f,0.0f,0.0f,0.3201000136114655614250068538240157067775726318359375f,-0.201300007634610533813201982411555945873260498046875f,-1.0886000909596678098978372872807085514068603515625f,0.0f,0.0f,2.38450005169212975175696556107141077518463134765625f,-1.3629000590279698457152335322462022304534912109375f,-2.01449998303502741947568210889585316181182861328125f,0.0f,0.0f,0.922400082892178563298557492089457809925079345703125f,0.82629999688714683703238961243187077343463897705078125f,-2.870400021179020821904259719303809106349945068359375f,0.0f,0.0f,-0.7438999342791736690827519851154647767543792724609375f,0.3182000704325733675403853339958004653453826904296875f,-1.57270005408227309118274206412024796009063720703125f,0.0f,0.0f,2.65780004526749191740009337081573903560638427734375f,-0.64820001453459175078108955858624540269374847412109375f,0.0213000740565358892553149416926316916942596435546875f,0.0f,0.0f,-3.693799963527172547372856570291332900524139404296875f,2.5492999153368174347633612342178821563720703125f,-1.665499986007808796983908905531279742717742919921875f,0.0f,0.0f,-0.754599979826809263983022901811636984348297119140625f,-2.3929000151813024643843164085410535335540771484375f,1.0782999094024312558559586250339634716510772705078125f,0.0f,0.0f,-0.09660000730454942896585635025985538959503173828125f,1.081100032137334299164876938448287546634674072265625f,-1.32499987933784790783420248772017657756805419921875f,0.0f,0.0f,1.0205999832466245980100438828230835497379302978515625f,-1.34619997647702671628167081507854163646697998046875f,1.0410999897807833303176039407844655215740203857421875f,0.0f,0.0f,-1.557400025299191081984417905914597213268280029296875f,-0.81489995440691753714190781465731561183929443359375f,-0.84869997893571869074236246888176538050174713134765625f,0.0f,0.0f,-0.40949997338652621703403156061540357768535614013671875f,1.76159999739527695084007063996978104114532470703125f,0.71470005518496027008268356439657509326934814453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.2897999431982640761162883791257627308368682861328125f,-0.71029996742755141525549333891831338405609130859375f,2.45250003010034500761094022891484200954437255859375f,0.0f,0.0f,1.046800030094385380863286627572961151599884033203125f,1.9042999827086919406582410374539904296398162841796875f,-1.0677000269249081743083706896868534386157989501953125f,0.0f,0.0f,-0.53159998420774889904549809216405265033245086669921875f,0.988200051191448824994267852162010967731475830078125f,-1.4013999967336643326376588447601534426212310791015625f,0.0f,0.0f,0.107300018560886722962521844237926416099071502685546875f,0.0201999703936285024408192612099810503423213958740234375f,-1.035099986062944399378693560720421373844146728515625f,0.0f,0.0f,1.20750000052154060625753118074499070644378662109375f,-0.1399998956173658104518153777462430298328399658203125f,0.2655999897658833930336186313070356845855712890625f,0.0f,0.0f,1.29719999423921183989705241401679813861846923828125f,0.8223999178059393511119878894533030688762664794921875f,-3.070700049384682106534683043719269335269927978515625f,0.0f,0.0f,1.1327000261425970162321164025343023240566253662109375f,1.04709987730681941542343338369391858577728271484375f,1.552800029639898493627470088540576398372650146484375f,0.0f,0.0f,-0.5389000276908275655074476162553764879703521728515625f,0.4319999420940885404007758552324958145618438720703125f,0.0710999994665375556479602892068214714527130126953125f,0.0f,0.0f,-0.05050001579150542596607920131646096706390380859375f,0.18589998366162252541045063480851240456104278564453125f,0.8953000436842450948660143694723956286907196044921875f,0.0f,0.0f,-1.412800015670061526407152996398508548736572265625f,0.01890002139508706324733111614477820694446563720703125f,2.109199973511696413197569199837744235992431640625f,0.0f,0.0f,0.5628000383570774811659020997467450797557830810546875f,-1.9506000892497576959527805229299701750278472900390625f,-1.557500100620092808867411804385483264923095703125f,0.0f,0.0f,1.2002000413976616766120741885970346629619598388671875f,-1.27490005761608582446342552429996430873870849609375f,-2.57079998074174032041128157288767397403717041015625f,0.0f,0.0f,-3.2192000380337244536121943383477628231048583984375f,-1.4238999699115755248612913419492542743682861328125f,0.73390001176521091341697911047958768904209136962890625f,0.0f,0.0f,-1.5259999616965647106781034381128847599029541015625f,1.098500020220875672549709634040482342243194580078125f,-1.22149998901784417881799527094699442386627197265625f,0.0f,0.0f,-1.0560000649616145818043833060073666274547576904296875f,-0.15479990369155860463479257305152714252471923828125f,-1.9778999860122805642959065153263509273529052734375f,0.0f,0.0f,0.536200077283383702564378836541436612606048583984375f,4.1110000493526452913783941767178475856781005859375f,-0.84379996658191114278224631561897695064544677734375f,0.0f,0.0f,-0.25210005761086951014959822714445181190967559814453125f,0.342299952157586517831333594585885293781757354736328125f,-0.2453000385060908339340812744922004640102386474609375f,0.0f,0.0f,1.737399993000924514063854076084680855274200439453125f,-0.7243999315373603753442921515670605003833770751953125f,0.779599980869889019885476955096237361431121826171875f,0.0f,0.0f,-2.15260006451755803169589853496290743350982666015625f,0.90860000673085483668955930625088512897491455078125f,2.687399926802516603174808551557362079620361328125f,0.0f,0.0f,-1.8990000092461702951851520992931909859180450439453125f,1.6603999188452969182350216215127147734165191650390625f,0.41120001007914563917466921338927932083606719970703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.91499998264014692495038616470992565155029296875f,1.027800021407007324825144678470678627490997314453125f,-1.184099987635017559028938194387592375278472900390625f,0.0f,0.0f,-1.689599990928173145476876015891321003437042236328125f,0.048900002098083206192313809879124164581298828125f,1.2149999727681279892976817791350185871124267578125f,0.0f,0.0f,-0.396199998456239654842647723853588104248046875f,-1.81269997190236953343855930143035948276519775390625f,0.50300002303719537621873314492404460906982421875f,0.0f,0.0f,-0.6875999874413019075092279308591969311237335205078125f,-0.22939998520016657845843610630254261195659637451171875f,-1.648999915406108129900530911982059478759765625f,0.0f,0.0f,-0.198000025719404337820606087916530668735504150390625f,-0.933800000072270375284233523416332900524139404296875f,-0.7113000097766513096786411551875062286853790283203125f,0.0f,0.0f,0.56570000522136698606345817097462713718414306640625f,-0.9071000050842756845526082543074153363704681396484375f,-0.1828999354600899795997293040272779762744903564453125f,0.0f,0.0f,-1.014599967384337109166381196700967848300933837890625f,-0.7161999744281175583893173097749240696430206298828125f,-0.216199995699524638581578983576036989688873291015625f,0.0f,0.0f,2.891099983671308049082426805398426949977874755859375f,0.7671000574432316199136039358563721179962158203125f,-0.8171000612243999317030329621047712862491607666015625f,0.0f,0.0f,-0.43160003425702420098986067387158982455730438232421875f,-0.95090001851171290780939671094529330730438232421875f,0.084600014602393525109391703153960406780242919921875f,0.0f,0.0f,3.06430000476241115592301866854541003704071044921875f,-0.1684999049976469986944493939517997205257415771484375f,0.378500059485434636741274516680277884006500244140625f,0.0f,0.0f,4.33450000229477883095796642010100185871124267578125f,-2.50569997310936543044590507633984088897705078125f,0.886499996300786907710289597162045538425445556640625f,0.0f,0.0f,1.0998999936528506271571359320660121738910675048828125f,1.452499993480742102036629148642532527446746826171875f,-0.303199982497095898992256479687057435512542724609375f,0.0f,0.0f,1.481200005273520137194509516120888292789459228515625f,2.837899951106309881510014747618697583675384521484375f,-0.44610002323389064127212577659520320594310760498046875f,0.0f,0.0f,1.657699965085088766869603205122984945774078369140625f,-3.71669993976056556306275524548254907131195068359375f,0.2625000039488083469763068933389149606227874755859375f,0.0f,0.0f,1.481500014871359294232888714759610593318939208984375f,3.2791999877795579010353321791626513004302978515625f,-0.48459999625533789480158475271309725940227508544921875f,0.0f,0.0f,0.5996000462487340865180840410175733268260955810546875f,2.11659996699690911015068195411004126071929931640625f,1.12439998929798701254867410170845687389373779296875f,0.0f,0.0f,0.231999995142221582256070178118534386157989501953125f,-0.372199987906217444333378807641565799713134765625f,-0.7830999943852423594847778076655231416225433349609375f,0.0f,0.0f,-0.661499972254038226537886657752096652984619140625f,0.84470001875162115112516403314657509326934814453125f,0.08709999971985815658825913487817160785198211669921875f,0.0f,0.0f,0.79229998309612259532741518341936171054840087890625f,0.6137000234901905937334731788723729550838470458984375f,0.3883999821901316096273149014450609683990478515625f,0.0f,0.0f,0.326200005906820411638591394876129925251007080078125f,-0.23109996675848964997612711158581078052520751953125f,-0.572699999320506858424550955533050000667572021484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.268800008659064726401766165508888661861419677734375f,0.958699986833333905877907454851083457469940185546875f,0.42360000024884936209446095745079219341278076171875f,0.0f,0.0f,-0.8769000174984331597016762316343374550342559814453125f,-0.521399989581107892178124529891647398471832275390625f,0.7657000452309847826626310052233748137950897216796875f,0.0f,0.0f,0.6913000333204870617720416703377850353717803955078125f,1.401799980250000743353666621260344982147216796875f,-0.72450001579895673131659350474365055561065673828125f,0.0f,0.0f,-0.71730000656545189752222313472884707152843475341796875f,-0.47420001478791196092288373620249330997467041015625f,-0.381499975364654186904544985736720263957977294921875f,0.0f,0.0f,1.943900053656101523102961436961777508258819580078125f,-0.2137000086635347795294137540622614324092864990234375f,1.442500000894068268308956248802132904529571533203125f,0.0f,0.0f,1.1702000438749788902015325220418162643909454345703125f,-0.4025999936625359598707518671290017664432525634765625f,-0.1755000071674588379977421936928294599056243896484375f,0.0f,0.0f,-1.22959996649026859216746743186376988887786865234375f,0.7015999889388666588274645619094371795654296875f,-0.6608000867076218032281076375511474907398223876953125f,0.0f,0.0f,-3.58590000033229561182679390185512602329254150390625f,2.14320001259744241650651019881479442119598388671875f,1.874899906834961438306663694675080478191375732421875f,0.0f,0.0f,-0.680099969261884762516956470790319144725799560546875f,-0.7625999917998902422056062277988530695438385009765625f,-3.437999986976385091708152685896493494510650634765625f,0.0f,0.0f,0.0951999861516068524025513397646136581897735595703125f,-0.2218999351695176702747858143993653357028961181640625f,2.011899961358308619452373022795654833316802978515625f,0.0f,0.0f,-0.7664000162169346541674030959256924688816070556640625f,0.376899947462975948297980721690692007541656494140625f,1.8162999656692149219594512032927013933658599853515625f,0.0f,0.0f,-1.0282000881463291275252913692384026944637298583984375f,-0.34940002925172375380924449927988462150096893310546875f,0.3582000464975825249069885103381238877773284912109375f,0.0f,0.0f,1.3010999845281234765792532925843261182308197021484375f,1.1399998934566983876948143006302416324615478515625f,-1.42620002155303904345373666728846728801727294921875f,0.0f,0.0f,-1.6741999942429359737872118785162456333637237548828125f,-1.583000009160488108506115167983807623386383056640625f,1.137799983930587899294550879858434200286865234375f,0.0f,0.0f,-0.9642000035934146229266161753912456333637237548828125f,0.539699989619851550060047884471714496612548828125f,-2.2208999991029489962102161371149122714996337890625f,0.0f,0.0f,0.3423000259928403732345714161056093871593475341796875f,3.09449998236447587629527333774603903293609619140625f,-1.699099950531125724495495887822471559047698974609375f,0.0f,0.0f,0.387099955724180322391703157336451113224029541015625f,-0.24430007189214275609145943235489539802074432373046875f,0.371499944090842415533870735089294612407684326171875f,0.0f,0.0f,-1.5764000779449940381482520024292171001434326171875f,2.624100039118527405435088439844548702239990234375f,0.6640000005662438642417555456631816923618316650390625f,0.0f,0.0f,-0.8326999606892464544927179304067976772785186767578125f,-1.23589999053478205581768634147010743618011474609375f,1.003600049310922059930817340500652790069580078125f,0.0f,0.0f,1.32399995157867689243857967085205018520355224609375f,-1.995699945318699430885089896037243306636810302734375f,0.385499997146427464400630924501456320285797119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2985000041276213433860675650066696107387542724609375f,-0.58019997696876568493706827212008647620677947998046875f,-0.351600038373470535901788025512360036373138427734375f,0.0f,0.0f,0.185600035214424252671960857696831226348876953125f,-0.2589999888315797083038205528282560408115386962890625f,0.02350001172721427611378430810873396694660186767578125f,0.0f,0.0f,1.287699977788328897787550886278040707111358642578125f,-1.1697999629236759400185974300256930291652679443359375f,0.05480001565515946193585250512114726006984710693359375f,0.0f,0.0f,-1.255899984686076553686007173382677137851715087890625f,0.50889999185353518296182073754607699811458587646484375f,-0.409099988901614697311970303417183458805084228515625f,0.0f,0.0f,-0.1977000273346902758930809795856475830078125f,1.3228999306917186284948684260598383843898773193359375f,-0.34159998562336024718888438655994832515716552734375f,0.0f,0.0f,-2.388499923363327948067080797045491635799407958984375f,-0.46869997643977379109259118195041082799434661865234375f,1.1700000525638458714894341028411872684955596923828125f,0.0f,0.0f,-0.6074000017926091121722720345132984220981597900390625f,-1.241799931597709960584552391082979738712310791015625f,-0.0718000237986438083481743888114579021930694580078125f,0.0f,0.0f,0.2834000289797773408650982673862017691135406494140625f,2.438199948149918583339967881329357624053955078125f,-0.501699970531463090850365915684960782527923583984375f,0.0f,0.0f,1.76400003902986757253756877616979181766510009765625f,0.2382000268094246475669706342159770429134368896484375f,-1.881199991936982396367739056586287915706634521484375f,0.0f,0.0f,2.56600000092387237060620464035309851169586181640625f,-1.14860001205801953716445495956577360630035400390625f,-0.7440999818235634055696436917060054838657379150390625f,0.0f,0.0f,-1.0273000710129736035014502704143524169921875f,-0.7337999722629804377760365241556428372859954833984375f,0.569000007644294925057693035341799259185791015625f,0.0f,0.0f,-0.354899999725073911349682020954787731170654296875f,-3.12459993059709706386684047174639999866485595703125f,1.34940001556128219561969672213308513164520263671875f,0.0f,0.0f,0.309400037764012925123324748710729181766510009765625f,1.2668999845668675874321706942282617092132568359375f,-0.3571999086134132195269330622977577149868011474609375f,0.0f,0.0f,2.6973000629663470562036309274844825267791748046875f,1.0419000074028954561100590581190772354602813720703125f,0.76230003972053506089423535740934312343597412109375f,0.0f,0.0f,-1.748999980412423216336037512519396841526031494140625f,-0.71320003759414041244468762670294381678104400634765625f,-1.05420001292526688274620028096251189708709716796875f,0.0f,0.0f,-0.73830005629062700478471015230752527713775634765625f,-1.7202000804573300829503068598569370806217193603515625f,0.01279998938813908804235097704804502427577972412109375f,0.0f,0.0f,-0.645400002962350782809153315611183643341064453125f,-1.831000067811459075528546236455440521240234375f,-1.238799984258413200421955480123870074748992919921875f,0.0f,0.0f,-0.180900021380185904718018718995153903961181640625f,0.60139996285214980531463879742659628391265869140625f,-0.333200003284214840704180460306815803050994873046875f,0.0f,0.0f,0.2062000337719922526957816444337368011474609375f,1.180000035390257995260299139772541821002960205078125f,1.80949998158216462940117708058096468448638916015625f,0.0f,0.0f,1.1902000179097054211041495364042930305004119873046875f,-0.73719994966611246400844947856967337429523468017578125f,-0.471700002084672942448406729454291053116321563720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.299299999099969848970204111537896096706390380859375f,-0.029700053769349921140019432641565799713134765625f,1.16799995404481915528549507143907248973846435546875f,0.0f,0.0f,0.34840000036954865691996019450016319751739501953125f,-1.087300031077861550699026338406838476657867431640625f,-0.7812999968498945957406931484001688659191131591796875f,0.0f,0.0f,0.2781000004947180581638122021104209125041961669921875f,1.449199987220764374029613463790155947208404541015625f,-0.2622000403583057970280378867755644023418426513671875f,0.0f,0.0f,1.1575999645680192795538232530816458165645599365234375f,0.3008999954149127464830826284014619886875152587890625f,1.5663999589592216832301119211479090154170989990234375f,0.0f,0.0f,0.132799988251924627746802798355929553508758544921875f,-0.5838999660558996662729214222053997218608856201171875f,-0.5890999605149038753637569243437610566616058349609375f,0.0f,0.0f,0.117499969601631004678665703977458178997039794921875f,0.04510001173019428932775554130785167217254638671875f,0.57469999394118786995022674091160297393798828125f,0.0f,0.0f,-0.510399988768995171994902193546295166015625f,0.567100003370643701572362260776571929454803466796875f,-0.322600049802660659992170621990226209163665771484375f,0.0f,0.0f,-0.5843999808788307159801433954271487891674041748046875f,-0.40499997125938502762210191576741635799407958984375f,-0.760799937640130874427768503664992749691009521484375f,0.0f,0.0f,-1.1008999820038669614774562433012761175632476806640625f,1.1821999815732235905585412183427251875400543212890625f,0.9507000103250142775124231775407679378986358642578125f,0.0f,0.0f,-1.5220999986022707428645617255824618041515350341796875f,0.6575000075250869713272550143301486968994140625f,1.364800013792514032928693268331699073314666748046875f,0.0f,0.0f,-1.240699983111768656129925147979520261287689208984375f,0.55659996442645820469152795340050943195819854736328125f,0.96229996894001923468664472238742746412754058837890625f,0.0f,0.0f,-0.2119999673701828957206316772499121725559234619140625f,-0.5619000151142479193566714457119815051555633544921875f,-0.6947999984443187226901272879331372678279876708984375f,0.0f,0.0f,-1.08839999500513240349164334475062787532806396484375f,-1.639500020027160331892446265555918216705322265625f,-0.9068000328138474674943836362217552959918975830078125f,0.0f,0.0f,1.1187999897904692403471926809288561344146728515625f,0.9731999922573566497163710664608515799045562744140625f,0.580400024103372658146327012218534946441650390625f,0.0f,0.0f,-0.38350000168755749019311451775138266384601593017578125f,0.5037000795185573309709070599637925624847412109375f,-1.492000061918049791387375080375932157039642333984375f,0.0f,0.0f,0.518899960263073278809997646021656692028045654296875f,0.172499976940452881546406160850892774760723114013671875f,0.378000009253620372629001167297246865928173065185546875f,0.0f,0.0f,-0.4815999480724342252102587735862471163272857666015625f,0.4469000121712678463836709852330386638641357421875f,-0.328599999541044240913834073580801486968994140625f,0.0f,0.0f,-0.64469998477697298966404559905640780925750732421875f,-1.44669999660849502021164880716241896152496337890625f,0.6968000090092421050513848967966623604297637939453125f,0.0f,0.0f,-0.027399945056439334312159417095244862139225006103515625f,-0.56330001393407636012256034518941305577754974365234375f,1.3524000509291889127183594609959982335567474365234375f,0.0f,0.0f,0.278999996133148187027472886256873607635498046875f,-0.17809999870657922116379268118180334568023681640625f,-0.595500000536441920218067025416530668735504150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.10869998165518079957791997003369033336639404296875f,0.5888999835833903517823273432441055774688720703125f,-0.7205999986320732464406546569080092012882232666015625f,0.0f,0.0f,0.17380005600750525474040841800160706043243408203125f,-0.063299960662424770152512110144016332924365997314453125f,-1.1822000184535976785582533921115100383758544921875f,0.0f,0.0f,-0.66219999319613032096043525598361156880855560302734375f,-1.4834000563606613010136925367987714707851409912109375f,-0.03970002791583482348869438283145427703857421875f,0.0f,0.0f,1.7092999994352460202406973621691577136516571044921875f,-1.1827999932438133523504575350671075284481048583984375f,-0.2314999644495545627620458617457188665866851806640625f,0.0f,0.0f,-1.774100070187448974223798359162174165248870849609375f,-0.174299945046007653814967852667905390262603759765625f,-1.2198000280976291076484585573780350387096405029296875f,0.0f,0.0f,-1.04879996980428646935479264357127249240875244140625f,2.24549992606788872961942615802399814128875732421875f,1.634299913530050663013071243767626583576202392578125f,0.0f,0.0f,2.750499923460185680568201860296539962291717529296875f,1.338899924388528006602427922189235687255859375f,1.38929993221238046174903502105735242366790771484375f,0.0f,0.0f,-0.9024999578297159263939875017968006432056427001953125f,1.29749998677521904966170041006989777088165283203125f,-0.4681999166339647633350296018761582672595977783203125f,0.0f,0.0f,-0.85279993270784704151310506858862936496734619140625f,-0.368600005352496429278374989735311828553676605224609375f,2.22729997329860918142685477505438029766082763671875f,0.0f,0.0f,-0.519100027029960386926177307032048702239990234375f,0.4377999842286113274525405358872376382350921630859375f,-0.17319991797506872899248264729976654052734375f,0.0f,0.0f,0.9760000426471238110792683073668740689754486083984375f,1.7372998665317893607351606988231651484966278076171875f,-0.47229996972233079244318787459633313119411468505859375f,0.0f,0.0f,0.3852999644473193985305670139496214687824249267578125f,0.6342000134840597436181042212410829961299896240234375f,0.5307000295102586306938974303193390369415283203125f,0.0f,0.0f,1.2810000232011067300419426828739233314990997314453125f,-0.305199965606629319836429203860461711883544921875f,0.5528000107899313508852401355397887527942657470703125f,0.0f,0.0f,0.8448999855317163021339865736081264913082122802734375f,2.597399892771989460271697680582292377948760986328125f,-1.043800006386638390409871135489083826541900634765625f,0.0f,0.0f,0.37179992329031197328959024162031710147857666015625f,0.0194999853819606272509190603159368038177490234375f,4.4775999468356371124855286325328052043914794921875f,0.0f,0.0f,-3.259700002323089762512609013356268405914306640625f,-1.0561999778553836648598007741384208202362060546875f,3.16629997643530369799691470689140260219573974609375f,0.0f,0.0f,0.703700017045437942186936197686009109020233154296875f,1.0007999981015915924587034169235266745090484619140625f,-0.09850003601610790582299159723334014415740966796875f,0.0f,0.0f,3.848199993300436272392062164726667106151580810546875f,-0.994399986839293870133360542240552604198455810546875f,-0.4630999536678201966566348346532322466373443603515625f,0.0f,0.0f,-1.2565000442266460733975463881506584584712982177734375f,-1.26930000714659652061300221248529851436614990234375f,1.624099987411500745082548746722750365734100341796875f,0.0f,0.0f,-1.4560999469399449335327290100394748151302337646484375f,-0.40269998188614852807631905307061970233917236328125f,-1.0830000217705963994063722566352225840091705322265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.44860001131296112220070426701568067073822021484375f,-0.61670001488104586595540013149729929864406585693359375f,1.64490002578347915829226622008718550205230712890625f,0.0f,0.0f,1.95970001793205739204495330341160297393798828125f,0.5238000431925067257310502100153826177120208740234375f,1.2129000026270748691104017780162394046783447265625f,0.0f,0.0f,-1.0580999886482960192779501085169613361358642578125f,0.2198999448329199690732593808206729590892791748046875f,-0.716000078000127881949765651370398700237274169921875f,0.0f,0.0f,2.220299979721009009381305077113211154937744140625f,-0.647499989401549402856517190230078995227813720703125f,1.472200005899369390505171395489014685153961181640625f,0.0f,0.0f,1.1312000649899232040951346789370290935039520263671875f,-1.3233000057011847783172697745612822473049163818359375f,-1.252599981667101847193634966970421373844146728515625f,0.0f,0.0f,-0.04749999504536361882855999283492565155029296875f,1.2443999833747749761414524982683360576629638671875f,-1.9270999788209806080629959978978149592876434326171875f,0.0f,0.0f,0.0219000413775443103503448583069257438182830810546875f,1.631199892546236096535494652925990521907806396484375f,0.4507001372456536092414580707554705440998077392578125f,0.0f,0.0f,-1.93379994309395630835979318362660706043243408203125f,-1.293400011917948955186830062302760779857635498046875f,0.74069999352395488045885940664447844028472900390625f,0.0f,0.0f,-1.151199942353366534320002756430767476558685302734375f,-2.460100115013866162172462281887419521808624267578125f,-0.5291999997377405673404382469016127288341522216796875f,0.0f,0.0f,-1.97360000904053389803038953687064349651336669921875f,-1.1033000574827196427207809392712078988552093505859375f,-0.5454999878332016383097879952401854097843170166015625f,0.0f,0.0f,2.184699976059793602445324722793884575366973876953125f,-0.0436999574035403792748866180772893130779266357421875f,-1.241200069721043686143957529566250741481781005859375f,0.0f,0.0f,1.4348000291407105866170468289055861532688140869140625f,0.42780000644996907244177464235690422356128692626953125f,-0.008999997250735969345214471104554831981658935546875f,0.0f,0.0f,0.8606000050395732348107458165031857788562774658203125f,0.1050000823289156670625743572600185871124267578125f,0.5206999300263819296930023483582772314548492431640625f,0.0f,0.0f,-1.192899951329827512580550319398753345012664794921875f,0.826599949897824703981541460962034761905670166015625f,-0.3550000259652730250081731355749070644378662109375f,0.0f,0.0f,1.393200012895466244344788719899952411651611328125f,2.49940008669570001842430428951047360897064208984375f,-0.2834999655336136736849539374816231429576873779296875f,0.0f,0.0f,2.46760003374666059272612983477301895618438720703125f,2.26450001224130392785127696697600185871124267578125f,-0.30410002270862512840920999224181286990642547607421875f,0.0f,0.0f,1.4906000682204965102073401794768869876861572265625f,2.89470000278875172483594724326394498348236083984375f,1.6625000259280202641321011469699442386627197265625f,0.0f,0.0f,-0.3757000032097088659810424360330216586589813232421875f,1.341400037171690318160699462168850004673004150390625f,1.73859993397593637354248130577616393566131591796875f,0.0f,0.0f,0.6389000368177899513710826795431785285472869873046875f,-0.65470002664923665491869542165659368038177490234375f,-1.5158000224649905707252628417336381971836090087890625f,0.0f,0.0f,-1.3047999937504517475872489740140736103057861328125f,1.22619998262375684561220623436383903026580810546875f,0.138600020179152405574996009818278253078460693359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.8644999962970614237889321884722448885440826416015625f,-0.630499983213841819207345906761474907398223876953125f,-0.997499986328184373718386268592439591884613037109375f,0.0f,0.0f,-0.4518999973446131690479887765832245349884033203125f,0.49780003656893978547515189347905106842517852783203125f,0.6759999901205302474949121460667811334133148193359375f,0.0f,0.0f,0.56579999158233384992655601308797486126422882080078125f,0.55419996505528690278197245788760483264923095703125f,1.6735000169426204053024775930680334568023681640625f,0.0f,0.0f,-0.9047999855920674239229128943406976759433746337890625f,0.320499993234872715408556587135535664856433868408203125f,-0.508499995894730272283368321950547397136688232421875f,0.0f,0.0f,-0.4927999665707349929988367875921539962291717529296875f,0.74949996342137492622015315646422095596790313720703125f,-1.3519999776706097538436779359471984207630157470703125f,0.0f,0.0f,0.6146999477475876272336563488352112472057342529296875f,0.4429000201359392097089084927574731409549713134765625f,-1.5507999407693746629632869371562264859676361083984375f,0.0f,0.0f,-2.026799971011282064381475720438174903392791748046875f,0.1224999637529251561574028528411872684955596923828125f,0.6662999620556833146878261686651967465877532958984375f,0.0f,0.0f,1.081399961157142985967993809026665985584259033203125f,1.53449997508153312963941061752848327159881591796875f,1.17469995847642394437571056187152862548828125f,0.0f,0.0f,0.089100013674796085805240863919607363641262054443359375f,-1.18429998174011696931984261027537286281585693359375f,-0.44330000611096698204249832997447811067104339599609375f,0.0f,0.0f,2.27379995993226824424482401809655129909515380859375f,-1.287900045691429795624571852385997772216796875f,-0.4366999347686775223564836778677999973297119140625f,0.0f,0.0f,-0.6967000165201711059381750601460225880146026611328125f,-1.327699978607892461468509281985461711883544921875f,-0.48530002932324978193179276786395348608493804931640625f,0.0f,0.0f,-0.0930999876238403079042882382054813206195831298828125f,0.72080001914948244046144054664182476699352264404296875f,0.178699954237043545646201891941018402576446533203125f,0.0f,0.0f,-0.07179998691827105261609176523052155971527099609375f,0.867899951069057262742489911033771932125091552734375f,1.1082999536588784561530474093160592019557952880859375f,0.0f,0.0f,-0.2607999472141273589187449033488519489765167236328125f,-0.6426999754041435863882725243456661701202392578125f,0.09839998381063363996901216523838229477405548095703125f,0.0f,0.0f,2.086900025209784015345348962000571191310882568359375f,-0.0965000285431736326557938809855841100215911865234375f,-0.07780000352561533549788919117418117821216583251953125f,0.0f,0.0f,1.7283000210493797421662520719110034406185150146484375f,1.7977999515578151346062441007234156131744384765625f,0.87590000450462091041714529637829400599002838134765625f,0.0f,0.0f,-0.702199976879358533921049456694163382053375244140625f,-1.074900001457333331700283451937139034271240234375f,-0.77569999188482796625976334325969219207763671875f,0.0f,0.0f,-1.438999959215522039812640286982059478759765625f,0.1526999737650154287393888807855546474456787109375f,-0.3917999847948545610876180944615043699741363525390625f,0.0f,0.0f,-0.29739995943605901373985034297220408916473388671875f,1.329399963295459397016884395270608365535736083984375f,-1.74219995609223676780175082967616617679595947265625f,0.0f,0.0f,0.3291999855071308100917804040363989770412445068359375f,0.868299982604384457118840145994909107685089111328125f,0.4230999730020761351312330589280463755130767822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.1982999857336285032261002925224602222442626953125f,1.4031999537378549991473164482158608734607696533203125f,1.467899986608327100867654735338874161243438720703125f,0.0f,0.0f,0.2867000499174003902425056367064826190471649169921875f,-0.6512999741628766603440681137726642191410064697265625f,0.633400006069243293183035348192788660526275634765625f,0.0f,0.0f,4.09630005522519358152067070477642118930816650390625f,0.502200019422172960759098714333958923816680908203125f,-2.0506000321224338023284872178919613361358642578125f,0.0f,0.0f,-1.823299962711334654130723720300011336803436279296875f,-1.1479000113442534569685449241660535335540771484375f,0.65810002623647445574306402704678475856781005859375f,0.0f,0.0f,0.9848000530570739652347356241079978644847869873046875f,1.67689996750503755862382604391314089298248291015625f,3.05249997694045216434233225299976766109466552734375f,0.0f,0.0f,2.270500033393501926326507600606419146060943603515625f,0.70489990487918208206252757008769549429416656494140625f,-1.1654999343752854290556797423050738871097564697265625f,0.0f,0.0f,3.0828999349012971009642569697462022304534912109375f,2.317999861024321983649087997036986052989959716796875f,-2.142200039203463912684810566133819520473480224609375f,0.0f,0.0f,-2.201499956160784421399512211792171001434326171875f,-0.2913000260934237584109496310702525079250335693359375f,-0.9493000666394824094140858505852520465850830078125f,0.0f,0.0f,1.6258999855801457012915989253087900578975677490234375f,2.364600085978954435717014348483644425868988037109375f,-2.42310000492781352221527413348667323589324951171875f,0.0f,0.0f,-1.17029997880831349021946152788586914539337158203125f,1.1626000207081441661927101449691690504550933837890625f,2.97740004113167433530406924546696245670318603515625f,0.0f,0.0f,3.470800043587384919874239130876958370208740234375f,-2.96569994591474728196089927223511040210723876953125f,1.9496000660486512146007953560911118984222412109375f,0.0f,0.0f,0.4638999336272464635300138979800976812839508056640625f,-2.11440004324019081849428403074853122234344482421875f,-0.75870003701299282283798675052821636199951171875f,0.0f,0.0f,0.41810000869035857107292031287215650081634521484375f,-0.55070003394335576007989629943040199577808380126953125f,-0.9744999859780059114910955031518824398517608642578125f,0.0f,0.0f,1.397200027096271224280599199119023978710174560546875f,-1.3952999591574066773347340131294913589954376220703125f,0.82649997206777214842077228240668773651123046875f,0.0f,0.0f,2.403199977188557046048345000599510967731475830078125f,-0.1862000550620248606747964004171080887317657470703125f,-1.437999993532894649206355097703635692596435546875f,0.0f,0.0f,0.2465999759376042987923938198946416378021240234375f,-1.197099915397912273107294822693802416324615478515625f,-1.05279992641210551340691381483338773250579833984375f,0.0f,0.0f,-0.32859996698200755194108069190406240522861480712890625f,-1.13560000082999490444990442483685910701751708984375f,0.41290001838505130393741637817583978176116943359375f,0.0f,0.0f,0.8363000339537858263128100588801316916942596435546875f,0.2906000183016057558660349968704394996166229248046875f,-1.096299976863712277719287158106453716754913330078125f,0.0f,0.0f,1.06430002182424043866149077075533568859100341796875f,1.833900001315771799426102006691507995128631591796875f,-1.5175999938115480869527118557016365230083465576171875f,0.0f,0.0f,-0.91140000448227009854917923803441226482391357421875f,0.24539996421933185644093100563623011112213134765625f,-1.1442000066667803448439144631265662610530853271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7561000017017127650120755788520909845829010009765625f,-0.31009998447969577828331466662348248064517974853515625f,-1.11150007455051014204627790604718029499053955078125f,0.0f,0.0f,0.706500047355891336309241523849777877330780029296875f,-1.097099987202882598325004437356255948543548583984375f,0.31710000016689321444829374740947969257831573486328125f,0.0f,0.0f,-1.50439999082535447172404019511304795742034912109375f,-0.62209997323304444538649704554700292646884918212890625f,1.1027000061005349795806296242517419159412384033203125f,0.0f,0.0f,0.1204000091090799973159164437674917280673980712890625f,-0.2823999977134168393178015321609564125537872314453125f,2.80880004572570296517142196535132825374603271484375f,0.0f,0.0f,-4.1865999431550502407617386779747903347015380859375f,-0.23099999399483228668117362758493982255458831787109375f,-1.326099977487325443092913701548241078853607177734375f,0.0f,0.0f,-1.1673000228449694759547128342092037200927734375f,-1.471499974768608165476280191796831786632537841796875f,0.3914000506572430371221571476780809462070465087890625f,0.0f,0.0f,0.07060003644377044196289716637693345546722412109375f,0.5574999089911560901100529008544981479644775390625f,3.533699888522921828126754917320795357227325439453125f,0.0f,0.0f,-0.40250007446855196047152958271908573806285858154296875f,0.068199999596178617622399542597122490406036376953125f,1.754299974922835847479518633917905390262603759765625f,0.0f,0.0f,1.535500067684799407885520849958993494510650634765625f,0.8919999997243281963932304279296658933162689208984375f,0.20820001840889490463126776376157067716121673583984375f,0.0f,0.0f,2.081399923531710438595609957701526582241058349609375f,0.07690000915378192303251125849783420562744140625f,1.440599986301360768692347846808843314647674560546875f,0.0f,0.0f,0.91669997261762592444256370072253048419952392578125f,-0.059100038019567424640854369499720633029937744140625f,1.708299987298251831902007324970327317714691162109375f,0.0f,0.0f,0.8220000064112247262215760201797820627689361572265625f,-1.3456000450678171631579971290193498134613037109375f,2.1746999604135748285216322983615100383758544921875f,0.0f,0.0f,2.84670008825063813873157414491288363933563232421875f,2.732400037667154446552331137354485690593719482421875f,1.7252000165313485791074299413594417273998260498046875f,0.0f,0.0f,2.735700057151912556463457804056815803050994873046875f,-2.31729990845993238934852342936210334300994873046875f,0.39889991310089778497882662122719921171665191650390625f,0.0f,0.0f,0.4411000218555340968151767810923047363758087158203125f,-0.85839997859522709777735371972084976732730865478515625f,-0.60999992191791696516389720272854901850223541259765625f,0.0f,0.0f,-1.761599947737157112470640640822239220142364501953125f,-0.52009996747374653836004654294811189174652099609375f,-1.519699963486939697787647673976607620716094970703125f,0.0f,0.0f,-0.05830001281648844457805580532294698059558868408203125f,0.0479000099472701723612999558099545538425445556640625f,-0.3053999842986458990168330274173058569431304931640625f,0.0f,0.0f,-0.8045000820532439700372151492047123610973358154296875f,-1.57169996675029377541932262829504907131195068359375f,0.207700046408175520440408945432864129543304443359375f,0.0f,0.0f,-2.79649999274313554309401297359727323055267333984375f,-1.993600026214121800904877090943045914173126220703125f,-1.78490000726878594861091187340207397937774658203125f,0.0f,0.0f,0.36160005878806078527532008592970669269561767578125f,0.91620001708269149798269381790305487811565399169921875f,1.36010002245903027784379446529783308506011962890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.0248000006794926175501814213930629193782806396484375f,-0.034800034505128252249051001854240894317626953125f,1.6816999766409406191769448923878371715545654296875f,0.0f,0.0f,1.16839999515414216801900693099014461040496826171875f,-0.401200024384259723575496536795981228351593017578125f,-1.44459997852295618514517627772875130176544189453125f,0.0f,0.0f,-0.613199995535612440988870730507187545299530029296875f,0.4107000123739243946374699589796364307403564453125f,-2.16260002111792548618041109875775873661041259765625f,0.0f,0.0f,0.45039998210072529349901060413685627281665802001953125f,-0.1568999987229704462521340246894396841526031494140625f,0.979999927058816666658458416350185871124267578125f,0.0f,0.0f,-0.61979997504949491116121862432919442653656005859375f,1.19509999612122808088088277145288884639739990234375f,0.990599973523617283177600256749428808689117431640625f,0.0f,0.0f,-1.5001000012993817023243536823429167270660400390625f,-1.2286999950289725358487658013473264873027801513671875f,1.85649997352063689248780065099708735942840576171875f,0.0f,0.0f,3.4884999791681767788986689993180334568023681640625f,0.78650010818987947658342818613164126873016357421875f,-0.6893999607622631753400810339371673762798309326171875f,0.0f,0.0f,-1.51339997667670189684940851293504238128662109375f,-1.431999959174544034823384208721108734607696533203125f,0.883800014843046710666385479271411895751953125f,0.0f,0.0f,0.452700003828108155090603759163059294223785400390625f,0.467999967381358406015579021186567842960357666015625f,-0.00929998255968100817625554554979316890239715576171875f,0.0f,0.0f,-1.5069999911189080332718503996147774159908294677734375f,1.56329994382411197051396811730228364467620849609375f,0.305899897849560176155137014575302600860595703125f,0.0f,0.0f,-1.5560999776177111275643483168096281588077545166015625f,3.09390000943690512968942130100913345813751220703125f,-0.269700015268475201679621022776700556278228759765625f,0.0f,0.0f,0.16629998638182830195120232019689865410327911376953125f,-0.3989000278025855994457060660351999104022979736328125f,0.432499983757734351996759869507513940334320068359375f,0.0f,0.0f,0.8180999287828800703437082120217382907867431640625f,-0.8391999541401862217071538907475769519805908203125f,0.41800001823157106883854794432409107685089111328125f,0.0f,0.0f,-1.3328000187806774423648903393768705427646636962890625f,2.445599965253472785065014249994419515132904052734375f,-0.5466999824896456861011984074139036238193511962890625f,0.0f,0.0f,-2.11180002022236568137714129989035427570343017578125f,-0.471699939574300675104723268304951488971710205078125f,-0.012499984391032281649813739932142198085784912109375f,0.0f,0.0f,-1.8398000216528771488810889422893524169921875f,-1.1244000061362982023638323880732059478759765625f,-1.965199995669722365931875174283050000667572021484375f,0.0f,0.0f,1.075399946784973526092699103173799812793731689453125f,0.392499975860118865966796875f,0.862599993252754160266704275272786617279052734375f,0.0f,0.0f,2.0721999630212781795535192941315472126007080078125f,0.6299999755620948604928344138897955417633056640625f,-0.881600009092688541301185978227294981479644775390625f,0.0f,0.0f,1.40459994786381781040063287946395576000213623046875f,-0.76910003151893580053410914842970669269561767578125f,-0.098800013911723993942359811626374721527099609375f,0.0f,0.0f,0.160399971148371900397933131898753345012664794921875f,0.89719998969435632574231931357644498348236083984375f,0.366300004523992583926883526146411895751953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.1192999839007848894567587194615043699741363525390625f,0.99370001082420333915479204733856022357940673828125f,-1.035999969072639803613355979905463755130767822265625f,0.0f,0.0f,-0.108899992561340308583339719916693866252899169921875f,-0.648700004416704611998056861921213567256927490234375f,-2.24020002059191458698705901042558252811431884765625f,0.0f,0.0f,-1.854499992050229639772851442103274166584014892578125f,0.35390000790953635156910195291857235133647918701171875f,1.4212999255664644504548732584225945174694061279296875f,0.0f,0.0f,-0.0734000227957967155134610948152840137481689453125f,-1.01700006178766511055755472625605762004852294921875f,2.306499986782668987217448375304229557514190673828125f,0.0f,0.0f,0.0823000151894988096756833328981883823871612548828125f,-0.64959999662786760410426722955889999866485595703125f,2.435499925285577571543171870871447026729583740234375f,0.0f,0.0f,1.5036999401181925239967540619545616209506988525390625f,0.8064999719932666177868441081955097615718841552734375f,0.044599993871151877300462729181163012981414794921875f,0.0f,0.0f,-3.4680999828368417325918926508165895938873291015625f,-0.8477999298021188234741885025869123637676239013671875f,1.4522000617228452146179051851504482328891754150390625f,0.0f,0.0f,-0.661600070112943416944517593947239220142364501953125f,0.36639996199533353404831359512172639369964599609375f,0.942999962277710768177030331571586430072784423828125f,0.0f,0.0f,0.43269996145293188050118260434828698635101318359375f,-1.69560002832263734262596699409186840057373046875f,-1.7673999269515270338359869128908030688762664794921875f,0.0f,0.0f,4.20839996758699452783503147657029330730438232421875f,1.232799924288689563667276161140762269496917724609375f,0.3973000668779018695886406931094825267791748046875f,0.0f,0.0f,2.535399927432089750567456576391123235225677490234375f,-1.292999947506934699248404285754077136516571044921875f,1.297700041695683648157455536420457065105438232421875f,0.0f,0.0f,-0.17280001023113655822527334748883731663227081298828125f,-2.744400001405178368685255918535403907299041748046875f,-1.994500031091272607142173001193441450595855712890625f,0.0f,0.0f,1.443400040602683898072200463502667844295501708984375f,1.16890005538836216913978205411694943904876708984375f,2.02559996413588550012718769721686840057373046875f,0.0f,0.0f,3.23609994816929091854262878769077360630035400390625f,-0.412199932138621161215041865943931043148040771484375f,0.0157999883785835404381714397459290921688079833984375f,0.0f,0.0f,-0.200599962943791343406019223039038479328155517578125f,1.1819001024350523909589583126944489777088165283203125f,-2.131499882325528449911189454724080860614776611328125f,0.0f,0.0f,1.5207000158384438304182140200282447040081024169921875f,1.0972999382063737261461255911854095757007598876953125f,-1.2697000346586111430013943390804342925548553466796875f,0.0f,0.0f,0.82680002864152191133229052866226993501186370849609375f,-0.4870999913007023618405355591676197946071624755859375f,-1.552000017344951299236299746553413569927215576171875f,0.0f,0.0f,-1.5841999547921121749283202007063664495944976806640625f,-0.9450999683678151797039390658028423786163330078125f,-0.0468999380938708299026984605006873607635498046875f,0.0f,0.0f,0.30639995709657774636269778056885115802288055419921875f,1.3526000426128508280498863314278423786163330078125f,0.27469998835325259989303958718664944171905517578125f,0.0f,0.0f,-0.385000004451722499521082454521092586219310760498046875f,-0.10419997190982055457197930081747472286224365234375f,-0.165699984080344808745621776324696838855743408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.94780000479221282461139708175323903560638427734375f,0.8267000220894811235439192387275397777557373046875f,1.387600027748942199679049736005254089832305908203125f,0.0f,0.0f,1.845299983185529679730052521335892379283905029296875f,-1.023399978092312867516966434777714312076568603515625f,-0.10920000409632901750001110485754907131195068359375f,0.0f,0.0f,-1.6058999530211097717113943872391246259212493896484375f,1.17199994634091897438565865741111338138580322265625f,0.99569995131641675367717425615410320460796356201171875f,0.0f,0.0f,0.889199967513978339894720193115063011646270751953125f,0.387099986942111673471345056896097958087921142578125f,0.02780005933046292287968981327139772474765777587890625f,0.0f,0.0f,2.213900036966801820881300955079495906829833984375f,2.503499963298440267323030639090575277805328369140625f,-0.8435001013763236965559144664439372718334197998046875f,0.0f,0.0f,0.6102000326246017625209105972317047417163848876953125f,-0.9408000097990030230477032091584987938404083251953125f,3.054700006420909996762702576234005391597747802734375f,0.0f,0.0f,-2.8860999906189750419116535340435802936553955078125f,-1.267100018122790938690513939945958554744720458984375f,0.4234000970408315911441832213313318789005279541015625f,0.0f,0.0f,-0.224100149163603601376593132954440079629421234130859375f,-1.132200032535196410066191674559377133846282958984375f,1.0126999782726160415080585153191350400447845458984375f,0.0f,0.0f,-0.9153999995723369753619635957875289022922515869140625f,2.635799966883658473904006314114667475223541259765625f,-0.1115000540241606330482682096771895885467529296875f,0.0f,0.0f,-1.702500054873524515386407074402086436748504638671875f,-1.542499941699206456036108647822402417659759521484375f,1.6282999319031825624648490702384151518344879150390625f,0.0f,0.0f,1.1660999607607729355862602460547350347042083740234375f,-2.491499953068793349331144781899638473987579345703125f,2.352000056162478092147694042068906128406524658203125f,0.0f,0.0f,2.0377999930948025308907745056785643100738525390625f,0.585199955976753560804581866250373423099517822265625f,-1.9858999588325605412109098324435763061046600341796875f,0.0f,0.0f,0.346100035247952020345252321931184269487857818603515625f,-3.8223000103652484682470458210445940494537353515625f,-1.31490004724115205902990055619738996028900146484375f,0.0f,0.0f,-1.8529999913722274840210957336239516735076904296875f,0.6010000191777937761372641034540720283985137939453125f,1.066499947220086230714741759584285318851470947265625f,0.0f,0.0f,-2.2521000063881277952759774052537977695465087890625f,-0.248500060658901844590218388475477695465087890625f,-0.08410005770772677191615684932912699878215789794921875f,0.0f,0.0f,-0.07840008128285569366511253974749706685543060302734375f,1.06030001432895648605381211382336914539337158203125f,0.39669991740882382114108395398943684995174407958984375f,0.0f,0.0f,0.030600035735964592475966128404252231121063232421875f,-0.291300022926926249766665932838805019855499267578125f,-0.19859997037202159475555163226090371608734130859375f,0.0f,0.0f,0.3622999572053553496431277380906976759433746337890625f,1.0045999927535655960042504375451244413852691650390625f,-0.459800063078105392122552075306884944438934326171875f,0.0f,0.0f,-1.620699949435145725828988361172378063201904296875f,-2.060499982766806237322043671156279742717742919921875f,0.01390002957209912626268533131224103271961212158203125f,0.0f,0.0f,-0.793400070852041228164353015017695724964141845703125f,-0.07649999017268438450400935835205018520355224609375f,0.01869996820688168970292508674901910126209259033203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.457400028577446438049491916899569332599639892578125f,-2.2961999240994455107056637643836438655853271484375f,-0.638400007149577231047032910282723605632781982421875f,0.0f,0.0f,-0.48030005292296440444488325738348066806793212890625f,-0.47970005809068627655733507708646357059478759765625f,-3.044400005018710775317458683275617659091949462890625f,0.0f,0.0f,-0.42659998757913708544720066129229962825775146484375f,0.64440000464618218956047712708823382854461669921875f,0.8184999944046140729625449239392764866352081298828125f,0.0f,0.0f,0.788100003251432923434549593366682529449462890625f,-0.6770000396966935340969939716160297393798828125f,0.542199972558021325852450900129042565822601318359375f,0.0f,0.0f,0.595200016624480188198731411830522119998931884765625f,0.3293999239563938363772876982693560421466827392578125f,0.87230000758990655640445766039192676544189453125f,0.0f,0.0f,1.9379999782219528015048126690089702606201171875f,-1.217400038635730208369523097644560039043426513671875f,2.08660000745356111195860648876987397670745849609375f,0.0f,0.0f,0.7970000034868718330471892841160297393798828125f,-0.161299980793892405728229277883656322956085205078125f,-0.618999961376190288575571685214526951313018798828125f,0.0f,0.0f,-1.6872999905094501205127244247705675661563873291015625f,0.2566999937772749262876459397375583648681640625f,1.10389995579272603976050959317944943904876708984375f,0.0f,0.0f,0.7421999757058925784036773620755411684513092041015625f,-0.46609996452331614591457764618098735809326171875f,0.7831999463431540409175113381934352219104766845703125f,0.0f,0.0f,-2.019199956822395503053257925785146653652191162109375f,-0.67350007766485209259599287179298698902130126953125f,-0.642600043171643875439258408732712268829345703125f,0.0f,0.0f,1.127399962118266874000482857809402048587799072265625f,-1.210099964214115697558327156002633273601531982421875f,-2.51540000992864332118870152044110000133514404296875f,0.0f,0.0f,-0.9495999608464540386876251432113349437713623046875f,0.4350999856531618803501260117627680301666259765625f,1.0317999476537114134089279104955494403839111328125f,0.0f,0.0f,-0.989700015622377637924955706694163382053375244140625f,1.415000035502016029198557589552365243434906005859375f,0.8379000187575817992780002896324731409549713134765625f,0.0f,0.0f,0.4983999272048473816454361440264619886875152587890625f,0.1338000110059984759658391340053640305995941162109375f,0.7800999423466630222634421443217433989048004150390625f,0.0f,0.0f,0.009700012157857873518196356599219143390655517578125f,-1.769999948032201064762602982227690517902374267578125f,-0.194299967807531448471536350552923977375030517578125f,0.0f,0.0f,0.08249994322657538958765144343487918376922607421875f,0.07049998205900187553396563089336268603801727294921875f,-1.199400028562546349775175258400849997997283935546875f,0.0f,0.0f,-0.652599993550777579542909734300337731838226318359375f,0.57699998337030411477144298260100185871124267578125f,-0.20399999624490749994265570421703159809112548828125f,0.0f,0.0f,-0.72899997061491017547041337820701301097869873046875f,-0.492899989700317231466897283098660409450531005859375f,-0.3812000130593775626408614698448218405246734619140625f,0.0f,0.0f,1.3809999717175964750737193753593601286411285400390625f,0.45260002745091920672848573303781449794769287109375f,0.33409998940452967364223013646551407873630523681640625f,0.0f,0.0f,-0.68769996519684806202121762908063828945159912109375f,-0.133000038832425726553765343851409852504730224609375f,0.95709999640434972878466624024440534412860870361328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.33620000985562814577178869512863457202911376953125f,-0.9525000138580796971154995844699442386627197265625f,1.3138999991551039148163226855103857815265655517578125f,0.0f,0.0f,0.2302000177234411193438745613093487918376922607421875f,-0.1230999911814922853636744548566639423370361328125f,1.6383000339202578032882229308597743511199951171875f,0.0f,0.0f,0.8662999949872489668933894790825434029102325439453125f,-0.3955999661847950932269668555818498134613037109375f,-1.16240000138282706387826692662201821804046630859375f,0.0f,0.0f,0.464299994406104143962465968797914683818817138671875f,-1.147399989610910164827828339184634387493133544921875f,0.883999938689172370942515044589526951313018798828125f,0.0f,0.0f,-0.0749000279814004132905580490842112340033054351806640625f,2.10589998189210891865741359652020037174224853515625f,-0.6385999724209299177601906194468028843402862548828125f,0.0f,0.0f,-1.1159000574409965178546144670690409839153289794921875f,0.821700006145239303378957629320211708545684814453125f,-1.5392000170230872857501935868640430271625518798828125f,0.0f,0.0f,-1.8159999649003142341285865768440999090671539306640625f,-4.91020010258555483773079686216078698635101318359375f,0.1104998887777324245718091333401389420032501220703125f,0.0f,0.0f,-0.1752999544635420736682362985447980463504791259765625f,-1.1456999646529555381135878633358515799045562744140625f,-0.38779998251497593830805499237612821161746978759765625f,0.0f,0.0f,-0.927199922080338634344798265374265611171722412109375f,0.4944999970234935648960572507348842918872833251953125f,1.07269999563246898333090939559042453765869140625f,0.0f,0.0f,0.3521999676406382118187821106403134763240814208984375f,-0.099499974429608339931974114733748137950897216796875f,-0.5564999858140939892336973571218550205230712890625f,0.0f,0.0f,-0.67570008892864041882830861140973865985870361328125f,0.91040002367496430935034368303604423999786376953125f,0.3283000339016322044471962726674973964691162109375f,0.0f,0.0f,-0.1069000599667420114968763300566934049129486083984375f,3.2517999555513252829541670507751405239105224609375f,1.1439000022098426700978279768605716526508331298828125f,0.0f,0.0f,-0.9190999939680100094818726574885658919811248779296875f,3.46780002762824324236134998500347137451171875f,-0.8558000176966178429438514285720884799957275390625f,0.0f,0.0f,-0.51349992313980974412146451868466101586818695068359375f,-0.280800024290382310709901503287255764007568359375f,-0.4660000562816855040892960460041649639606475830078125f,0.0f,0.0f,-4.1873999258339420492802673834376037120819091796875f,-1.679900019003450761800877444329671561717987060546875f,0.297199965852498859675279163639061152935028076171875f,0.0f,0.0f,0.6491000276073821684263975839712657034397125244140625f,0.69130001150891218220095879587461240589618682861328125f,4.5378999775931223581437734537757933139801025390625f,0.0f,0.0f,-0.967399955934285404879346970119513571262359619140625f,-0.3381000089138748165851211524568498134613037109375f,-0.9834000034615397733972486093989573419094085693359375f,0.0f,0.0f,0.28509996738061271770448001916520297527313232421875f,1.167300062221289014274816508986987173557281494140625f,0.92499999046325742657614910058327950537204742431640625f,0.0f,0.0f,1.7441999942988157812351346365176141262054443359375f,-1.721400043411552616134940763004124164581298828125f,-1.3885999638155099855651997131644748151302337646484375f,0.0f,0.0f,0.74629998079389292531260480245691724121570587158203125f,-0.431699994969368450181690377576160244643688201904296875f,-0.5085999518945822917004306873423047363758087158203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.107100041890144215273039662861265242099761962890625f,0.3923000000834464540133694754331372678279876708984375f,0.5710999958157543243686404821346513926982879638671875f,0.0f,0.0f,-1.3468000014096499494797853913041763007640838623046875f,-0.656299984407425096577526346663944423198699951171875f,1.0985999622136357611879020623746328055858612060546875f,0.0f,0.0f,-0.449600016930699286632489020121283829212188720703125f,0.42479999479353447444651692421757616102695465087890625f,-1.154599993684886971578862358001060783863067626953125f,0.0f,0.0f,-0.0692999910533433904902267386205494403839111328125f,-0.448999984249471850716872722841799259185791015625f,-1.137500052079558177098306259722448885440826416015625f,0.0f,0.0f,0.442899972433596855747595100183389149606227874755859375f,-0.8737999676436196683226853565429337322711944580078125f,-1.854799948786198360295429665711708366870880126953125f,0.0f,0.0f,0.48659999214261773214929007735918276011943817138671875f,-0.5798999578714376657018192418036051094532012939453125f,0.0059999926164751027357624479918740689754486083984375f,0.0f,0.0f,0.480399939110875084935514678363688290119171142578125f,1.2395001812577248756497283466160297393798828125f,1.4252000234603894313067939947359263896942138671875f,0.0f,0.0f,1.184699987086653916179557199939154088497161865234375f,-0.75309998849928272424136821427964605391025543212890625f,-1.4723000304445614805359809906804002821445465087890625f,0.0f,0.0f,0.42889997229576071191559094586409628391265869140625f,-1.5936999734595413968918364844284951686859130859375f,1.01889993928968802805457016802392899990081787109375f,0.0f,0.0f,0.5317999845713379247769125868217088282108306884765625f,0.4100999549940229638167465964215807616710662841796875f,-0.943799972523750252406671279459260404109954833984375f,0.0f,0.0f,2.774499988771974567924871735158376395702362060546875f,2.41909994315505105788588480209000408649444580078125f,1.6258999784648420217081365990452468395233154296875f,0.0f,0.0f,0.721700043379514877273095407872460782527923583984375f,1.9567999615862969875479393522255122661590576171875f,1.5886000266239037603810402288218028843402862548828125f,0.0f,0.0f,0.45390005311593417669513428336358629167079925537109375f,-1.059400022322683820874544835533015429973602294921875f,2.071399976319074909270057105459272861480712890625f,0.0f,0.0f,0.55290000445246700877532930462621152400970458984375f,2.426799987979977313301560570835135877132415771484375f,-2.897000008106231039306521779508329927921295166015625f,0.0f,0.0f,2.562799993728100300671712830080650746822357177734375f,-1.9089999352246513897313207053230144083499908447265625f,-2.71089999880492715789159774431027472019195556640625f,0.0f,0.0f,-0.99999995140358777234723675064742565155029296875f,0.5948999281808740047239325576811097562313079833984375f,0.59839998110979752965477018733508884906768798828125f,0.0f,0.0f,1.79759998434931134170255973003804683685302734375f,-1.403199971991778038926668159547261893749237060546875f,-1.101400035774707131253080660826526582241058349609375f,0.0f,0.0f,0.448900097912549700396311891381628811359405517578125f,1.7339000237792732139752160946954973042011260986328125f,-1.52069997154474290113057577400468289852142333984375f,0.0f,0.0f,0.6240000359006219188273689724155701696872711181640625f,0.59499998601153547550524081088951788842678070068359375f,0.09050002123042955037846013510716147720813751220703125f,0.0f,0.0f,0.64530001119971236445138629278517328202724456787109375f,-0.6145999820247289857633177234674803912639617919921875f,-0.8843999373301871802510731868096627295017242431640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.1946000396549700806048122103675268590450286865234375f,1.94049996820092207627794778090901672840118408203125f,0.5582999905765058112905308007611893117427825927734375f,0.0f,0.0f,-0.805800003093480654570157639682292938232421875f,-0.892199992525577467716857427149079740047454833984375f,0.46840001989006996296893703402020037174224853515625f,0.0f,0.0f,-0.76920000379830588688179204837069846689701080322265625f,0.3580000086575747531014712876640260219573974609375f,0.195099977532029011939584961510263383388519287109375f,0.0f,0.0f,-0.5396000337690114445621247796225361526012420654296875f,0.25700002330541593664747779257595539093017578125f,-0.493000005602836299800628694356419146060943603515625f,0.0f,0.0f,0.635699997957051010644136113114655017852783203125f,-1.313299959656596183066312732989899814128875732421875f,-0.13180003031790243905874149277224205434322357177734375f,0.0f,0.0f,-0.651299967196583562412115497863851487636566162109375f,-0.78419996546506975931833949289284646511077880859375f,-0.5918999989837405717452156750368885695934295654296875f,0.0f,0.0f,-2.08430001020133470746031889575533568859100341796875f,1.4946999582715332355320470014703460037708282470703125f,-0.2803000044196843898447468745871447026729583740234375f,0.0f,0.0f,0.19300004082545674233273302888846956193447113037109375f,-0.2000999871805311247641157024190761148929595947265625f,-2.477799905047567197158286944613792002201080322265625f,0.0f,0.0f,-0.54309999084621740639278186790761537849903106689453125f,-0.913000033318996262465816471376456320285797119140625f,-1.48129997878223651497364699025638401508331298828125f,0.0f,0.0f,1.002699990081786562967636200482957065105438232421875f,-0.38550001752376561459101367290713824331760406494140625f,-1.09429998896718050360732377157546579837799072265625f,0.0f,0.0f,-0.1028999853417273957489896929473616182804107666015625f,1.27650004258751881280886664171703159809112548828125f,0.6979000516891480376813206021324731409549713134765625f,0.0f,0.0f,1.609199960864335476884434683597646653652191162109375f,1.2401999445959923651372491804067976772785186767578125f,-0.9912999643839894847729965476901270449161529541015625f,0.0f,0.0f,1.10729994047880264673722194856964051723480224609375f,0.4273999524325133858582148604909889400005340576171875f,-0.4839000038117176050178613877505995333194732666015625f,0.0f,0.0f,-1.837699936139583822836129911593161523342132568359375f,-1.565799980890751452733411497320048511028289794921875f,-1.4867999694839106883392787494813092052936553955078125f,0.0f,0.0f,0.08789995341599043943148217294947244226932525634765625f,0.2938999894693490322339357589953579008579254150390625f,-0.73790002482980465714490492246113717555999755859375f,0.0f,0.0f,1.791699948664010388910128313000313937664031982421875f,-0.9177999973043799997185487882234156131744384765625f,-0.5195000248327847902629628151771612465381622314453125f,0.0f,0.0f,-0.046499963015318002135245478712022304534912109375f,-0.51480001558065335842684362432919442653656005859375f,1.255599978515506531806522616534493863582611083984375f,0.0f,0.0f,0.257099978262185668853589959326200187206268310546875f,-0.078000013500452158865527962916530668735504150390625f,0.002500024139881151796771519002504646778106689453125f,0.0f,0.0f,-0.057099977368116672238329556421376764774322509765625f,0.0967999884456396397780508777941577136516571044921875f,0.78430001578927044381117639204603619873523712158203125f,0.0f,0.0f,0.2186000055015087095711123765795491635799407958984375f,0.761599984580278555057475387002341449260711669921875f,-0.6916000038772818214738435926847159862518310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6550000395625834581636581788188777863979339599609375f,0.15439999171942486722031162571511231362819671630859375f,-0.042100014472008229216015706697362475097179412841796875f,0.0f,0.0f,1.10440001299083245811516462708823382854461669921875f,0.852200092549622922888374887406826019287109375f,-0.67620002714097504625812007361673749983310699462890625f,0.0f,0.0f,-0.6342000488743189112739173651789315044879913330078125f,0.795799996015428501294763918849639594554901123046875f,0.708099916247278304837209361721761524677276611328125f,0.0f,0.0f,0.9755999396979806448371164151467382907867431640625f,0.24939998881369895844528628003899939358234405517578125f,-0.8932000136777755727734984247945249080657958984375f,0.0f,0.0f,-1.0058000169143088697865096037276089191436767578125f,-3.02700006067007532095658461912535130977630615234375f,0.8343999511137620839207329481723718345165252685546875f,0.0f,0.0f,-1.7362000796303156402444756167824380099773406982421875f,-1.740199994514881165486031022737734019756317138671875f,1.5685999532729379968287730662268586456775665283203125f,0.0f,0.0f,-0.411500069409609370296720953774638473987579345703125f,-2.6932000167883956009973189793527126312255859375f,-0.6006000958621522389790925444685854017734527587890625f,0.0f,0.0f,-1.914999982714652215776141019887290894985198974609375f,0.3642999964550153180198321933858096599578857421875f,1.635900042912363350211535362177528440952301025390625f,0.0f,0.0f,1.3947000617042188963523585698567330837249755859375f,-1.5409000135384491869672274333424866199493408203125f,0.4260000171288835790761595490039326250553131103515625f,0.0f,0.0f,2.734399899220466334526236096280626952648162841796875f,1.1645999796405437365365287405438721179962158203125f,-0.1370999517381166210583387510268948972225189208984375f,0.0f,0.0f,0.9556000528909265323562749472330324351787567138671875f,-2.40050003026425695651369096594862639904022216796875f,1.76909997981190958427077930537052452564239501953125f,0.0f,0.0f,-1.101899844197927524902524965000338852405548095703125f,-3.706700020226836755909971543587744235992431640625f,-1.425899972914160596104693468078039586544036865234375f,0.0f,0.0f,1.9073999087348585401713307874160818755626678466796875f,-3.824799952660500768075735322781838476657867431640625f,-0.4053998514920478246637003394425846636295318603515625f,0.0f,0.0f,2.837499973010272125151232103235088288784027099609375f,-0.24290007858574380605887199635617434978485107421875f,0.9902999637767673224431064227246679365634918212890625f,0.0f,0.0f,1.6399000676184891656106401569559238851070404052734375f,3.45830000111907764903662609867751598358154296875f,1.370400034999846194949668642948381602764129638671875f,0.0f,0.0f,-1.33229995813667745352404381264932453632354736328125f,1.71359997640699202037239956553094089031219482421875f,-1.157100031347571889028813529876060783863067626953125f,0.0f,0.0f,1.49439996426403620688461160170845687389373779296875f,0.145399957811832791776396334171295166015625f,1.0054999797865755706283152903779409825801849365234375f,0.0f,0.0f,1.1048000109799200885163372731767594814300537109375f,2.31679994918108089763109092018567025661468505859375f,1.846199943657219844084238502546213567256927490234375f,0.0f,0.0f,1.986900134733317724311518759350292384624481201171875f,0.5158999840155249483331090232240967452526092529296875f,-0.880500104099511293753721474786289036273956298828125f,0.0f,0.0f,2.16199999778717799614469186053611338138580322265625f,-0.922299982611834767709524385281838476657867431640625f,0.18290000407993789455218802686431445181369781494140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.478400002679228553148504943237639963626861572265625f,1.0478999879866837119379852083511650562286376953125f,0.338800035928189746670113891013897955417633056640625f,0.0f,0.0f,1.43440001947283679584188575972802937030792236328125f,-1.5824999859556549974826111792935989797115325927734375f,-2.3285000181347133008102900930680334568023681640625f,0.0f,0.0f,-1.5101998717665654492492421923088841140270233154296875f,-1.670200085076687823715246850042603909969329833984375f,-0.892299951952696179802160258986987173557281494140625f,0.0f,0.0f,-1.1884999987259501352809820673428475856781005859375f,0.4497000434875488839026047571678645908832550048828125f,0.3540000087246306659238825886859558522701263427734375f,0.0f,0.0f,-1.3571000602558263015140482821152545511722564697265625f,-0.9355000067204237712559233841602690517902374267578125f,1.35850000748038279851925835828296840190887451171875f,0.0f,0.0f,1.3650000189244757553552744866465218365192413330078125f,3.91019998892694697900651590316556394100189208984375f,2.72289989884048733159715993679128587245941162109375f,0.0f,0.0f,3.12860001062378234593097658944316208362579345703125f,-0.07730008285939415912935146479867398738861083984375f,-1.71310002463459820631896945997141301631927490234375f,0.0f,0.0f,-1.2608000362858173826197116795810870826244354248046875f,-0.857199967491627301541257111239247024059295654296875f,-1.110399993835390564100862320628948509693145751953125f,0.0f,0.0f,0.0004001126348949224365014742943458259105682373046875f,-1.990299964819847833297217221115715801715850830078125f,-2.4474999688565741706725020776502788066864013671875f,0.0f,0.0f,1.644100088031590356507649630657397210597991943359375f,-1.517599986007064583048986605717800557613372802734375f,0.5133999414727090648824514573789201676845550537109375f,0.0f,0.0f,-1.944200036916136742348726329510100185871124267578125f,-1.4943999810278401074725707076140679419040679931640625f,-0.0372999636873601048137061297893524169921875f,0.0f,0.0f,2.53629992221370326177520837518386542797088623046875f,-0.89239991815984165413055961835198104381561279296875f,0.315299997527895836668676565750502049922943115234375f,0.0f,0.0f,0.1387999953411520603907547410926781594753265380859375f,1.2835999348700062316908088178024627268314361572265625f,-2.76119993255585338687296825810335576534271240234375f,0.0f,0.0f,-2.21100002994388233901190687902271747589111328125f,-1.985499996475874606716160997166298329830169677734375f,-0.919200008939206281155520628090016543865203857421875f,0.0f,0.0f,-0.61230006732493402754613498473190702497959136962890625f,2.3261000477276763120926261763088405132293701171875f,0.58479998700767765740238246507942676544189453125f,0.0f,0.0f,-2.01210007359236175972228011232800781726837158203125f,-0.8389999249801036729223824295331723988056182861328125f,2.52199998039007322603310967679135501384735107421875f,0.0f,0.0f,0.91599995048344151626906750607304275035858154296875f,0.7220000300854447772280764183960855007171630859375f,1.6279999844431873867733884253539144992828369140625f,0.0f,0.0f,-1.0172999746769668671930730852182023227214813232421875f,-2.4638999921143049931515633943490684032440185546875f,0.045600021058322059985812302329577505588531494140625f,0.0f,0.0f,-2.898299961686878756239593712962232530117034912109375f,-0.9036999548330906240067861290299333631992340087890625f,0.62919996438473491107146173817454837262630462646484375f,0.0f,0.0f,-0.49739999799281331416267448730650357902050018310546875f,-0.02799999643862260967352995066903531551361083984375f,0.7789999659955508803932389128021895885467529296875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.63880003238916405194913750165142118930816650390625f,-0.7056999880105256917062206412083469331264495849609375f,-0.72460000642538080484200690989382565021514892578125f,0.0f,0.0f,0.4510000208169222535303788390592671930789947509765625f,0.13549999658763411947148824765463359653949737548828125f,-0.330599990995228409929040935821831226348876953125f,0.0f,0.0f,0.50460002285391103438172422102070413529872894287109375f,-0.22119999372512111346367191799799911677837371826171875f,0.5585999864652755331206890332396142184734344482421875f,0.0f,0.0f,0.9963000384241345042113380259252153337001800537109375f,-0.506800021302699743586117619997821748256683349609375f,0.1512999996438626482131439843215048313140869140625f,0.0f,0.0f,0.974099985660612954774251193157397210597991943359375f,1.1615999364122757686601516979862935841083526611328125f,-0.266099993915855481674270777148194611072540283203125f,0.0f,0.0f,0.31719997671172051045829221038729883730411529541015625f,-0.5988000228151675319310243139625526964664459228515625f,1.93719999595284519955384894274175167083740234375f,0.0f,0.0f,-0.340399993388353916401456444873474538326263427734375f,-1.0380000335425136892553155121277086436748504638671875f,0.023999964319170175031104008667171001434326171875f,0.0f,0.0f,-1.0183999702878294701946515488089062273502349853515625f,-0.0260000128448016898374817174044437706470489501953125f,0.95399996972084044610795672269887290894985198974609375f,0.0f,0.0f,1.7340999450735752329677552552311681210994720458984375f,1.683700010004638869531845557503402233123779296875f,0.31809997739791862070291017516865395009517669677734375f,0.0f,0.0f,-0.8267999576374893688779366129892878234386444091796875f,-0.2529000171557067577765565147274173796176910400390625f,0.01259998855888877500319722457788884639739990234375f,0.0f,0.0f,-0.6256000206112848349704336214927025139331817626953125f,-1.229099987243860159225050665554590523242950439453125f,-0.6690000083520992735230947801028378307819366455078125f,0.0f,0.0f,-0.65619997586235445652391717885620892047882080078125f,0.1426999904915693395679454624769277870655059814453125f,1.12429998300671574185116696753539144992828369140625f,0.0f,0.0f,-0.9641999790996316210822669745539315044879913330078125f,-0.7733000180318949645652537583373486995697021484375f,0.21469997982233846034461066665244288742542266845703125f,0.0f,0.0f,0.65879994257241569766136990438099019229412078857421875f,0.87089993636906160023869460928835906088352203369140625f,-0.529600054425745536690328663098625838756561279296875f,0.0f,0.0f,-0.413900002545119161112552319536916911602020263671875f,0.1935000486448406054051929459092207252979278564453125f,-0.299299981106817991616253493702970445156097412109375f,0.0f,0.0f,-1.5394000378757712876875984875368885695934295654296875f,0.2983000039875509656894791987724602222442626953125f,-0.7347000162556758251497512901551090180873870849609375f,0.0f,0.0f,-1.0278999823987484685261506456299684941768646240234375f,0.744999982267617877340626364457421004772186279296875f,-0.6059999814778560534023199579678475856781005859375f,0.0f,0.0f,-0.2234999690353873802450834773480892181396484375f,0.13599999041855304060533171650604344904422760009765625f,-0.92850001895427614062583643317339010536670684814453125f,0.0f,0.0f,0.7627999975651500452755726655595935881137847900390625f,-0.0978999943941831762828087448724545538425445556640625f,-0.732599973732232623291338313720189034938812255859375f,0.0f,0.0f,-0.5710999815106390098407018740545026957988739013671875f,-0.2712000064283606182158337105647660791873931884765625f,0.20379998232126228518978905412950552999973297119140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.0888000407710674011241280823014676570892333984375f,-1.553400032854079793054324909462593495845794677734375f,1.9429999846667047602721822840976528823375701904296875f,0.0f,0.0f,1.03770002133697314405935685499571263790130615234375f,1.584700071837009005548679851926863193511962890625f,2.6241000555284319517568292212672531604766845703125f,0.0f,0.0f,0.71870001674741512687205613474361598491668701171875f,0.249999970309436658766344407922588288784027099609375f,-1.08129998990222819799100761883892118930816650390625f,0.0f,0.0f,-0.0072999981462960050748733920045197010040283203125f,-1.10279998847544202789094924810342490673065185546875f,-1.6253000148504967992124647935270331799983978271484375f,0.0f,0.0f,-1.8426000340819346234866316081024706363677978515625f,-1.8803000109016902996472708764486014842987060546875f,-0.08579994341433094806603776305564679205417633056640625f,0.0f,0.0f,0.042599850127101657193406936130486428737640380859375f,0.2140000215023765139932265810784883797168731689453125f,1.447499932348727202935378954862244427204132080078125f,0.0f,0.0f,2.05750003304332285125610724207945168018341064453125f,-1.83239998923837976718687059474177658557891845703125f,0.348699951834231303138267321628518402576446533203125f,0.0f,0.0f,-0.85040001018941335786394120077602565288543701171875f,-0.5930999094486235634349213796667754650115966796875f,-0.3196999615311624243219057461828924715518951416015625f,0.0f,0.0f,0.632500061020255799348888103850185871124267578125f,-0.05630001389309757797718702931888401508331298828125f,2.361499917864799957811783315264619886875152587890625f,0.0f,0.0f,-3.22579994278103043825467466376721858978271484375f,0.56499997071921737035182786712539382278919219970703125f,0.55769994969964054032374178859754465520381927490234375f,0.0f,0.0f,-4.33779999456629195009327304433099925518035888671875f,1.4703000839918838327236016993992961943149566650390625f,1.6727000153392548309483345292392186820507049560546875f,0.0f,0.0f,3.5788999991923571286633887211792171001434326171875f,-1.9244999826252457975073184570646844804286956787109375f,1.4264999772459265869883893174119293689727783203125f,0.0f,0.0f,0.6709000131845466174951297944062389433383941650390625f,2.2810000089891264707375739817507565021514892578125f,-1.7738999193221334227388297222205437719821929931640625f,0.0f,0.0f,-0.696099893407524206168091041035950183868408203125f,-0.82959993802905085402699114638380706310272216796875f,-0.7341000287994743356279059298685751855373382568359375f,0.0f,0.0f,2.53330006120800899083178592263720929622650146484375f,2.15979993924945556926786593976430594921112060546875f,2.115200005318225162653789084288291633129119873046875f,0.0f,0.0f,0.6462000211805090099659309998969547450542449951171875f,-0.616500022843480710577068748534657061100006103515625f,-1.810499962165952769765908669796772301197052001953125f,0.0f,0.0f,1.4708999727278937452723539536236785352230072021484375f,0.023599974395334621934949836941086687147617340087890625f,0.27859998594373536207058350555598735809326171875f,0.0f,0.0f,0.56170003944933422079799356652074493467807769775390625f,-1.1636999606072908619580630329437553882598876953125f,0.9656000311911092648387011649901978671550750732421875f,0.0f,0.0f,0.08739992453008949002679628392797894775867462158203125f,-1.120799951311945985565898808999918401241302490234375f,0.2440999874860063645343188909464515745639801025390625f,0.0f,0.0f,1.68329997861832225680700503289699554443359375f,0.274299968180060693700994534083292819559574127197265625f,0.933999986931681736024302153964526951313018798828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.43280006964951756032178309396840631961822509765625f,-0.01599999727308787367263676060247235000133514404296875f,0.73820001811087221366136645883671008050441741943359375f,0.0f,0.0f,-1.172599989117681928973979665897786617279052734375f,-0.473600024276971343084596810513176023960113525390625f,-0.9109000349029889864738152027712203562259674072265625f,0.0f,0.0f,1.1109999844580900063562012292095459997653961181640625f,-1.6288999428287151971517232595942914485931396484375f,-2.389399985498189327159934691735543310642242431640625f,0.0f,0.0f,-0.7979000257238755278166308926302008330821990966796875f,0.88530002047568567835611474947654642164707183837890625f,0.48479998220205278158800865639932453632354736328125f,0.0f,0.0f,0.060200002188979528927603723786887712776660919189453125f,-3.86249994415789910107150717522017657756805419921875f,0.92579999936819101780116625377559103071689605712890625f,0.0f,0.0f,1.176500017218291116449790933984331786632537841796875f,0.2580000291839237291924291639588773250579833984375f,-4.105099880674483614484415738843381404876708984375f,0.0f,0.0f,-1.779999957289545964300714331329800188541412353515625f,2.03070002697706275540667775203473865985870361328125f,-0.490499906584620504190752399154007434844970703125f,0.0f,0.0f,-0.1510000369846815626573288682266138494014739990234375f,-2.20629994287043817990934257977642118930816650390625f,1.246099933640658630196185185923241078853607177734375f,0.0f,0.0f,2.370200035716594211265828562318347394466400146484375f,-0.68009996449351317604481437228969298303127288818359375f,-2.48700003753602505440767345135100185871124267578125f,0.0f,0.0f,-0.6431998814672230668776364836958236992359161376953125f,-2.134999934360385598353104796842671930789947509765625f,-0.5928000728905200578111589493346400558948516845703125f,0.0f,0.0f,-0.93739992181062792298007479985244572162628173828125f,-0.04930003385692838246967539816978387534618377685546875f,0.6127999851971861478006076140445657074451446533203125f,0.0f,0.0f,-0.72370002801641764023088398971594870090484619140625f,-3.00150000881776168881742705707438290119171142578125f,-0.902099960194526762080613480065949261188507080078125f,0.0f,0.0f,0.868599955992401806526004293118603527545928955078125f,1.5326000092901279625579036292037926614284515380859375f,-1.3774999875202775800886456636362709105014801025390625f,0.0f,0.0f,-0.219300046448408725296985721797682344913482666015625f,-0.50690003785714610184953698990284465253353118896484375f,-1.039000067584215702964911542949266731739044189453125f,0.0f,0.0f,-3.17119991903305109559596530743874609470367431640625f,1.577400019413231380127626835019327700138092041015625f,1.914700006271898846677004257799126207828521728515625f,0.0f,0.0f,2.026599974521992475473552985931746661663055419921875f,-1.040699978511034462513862308696843683719635009765625f,-0.4854999906644230645014204128528945147991180419921875f,0.0f,0.0f,-2.613299973663687492120288879959844052791595458984375f,0.7889999405518182129526394419372081756591796875f,0.14070000663697779241800844829413108527660369873046875f,0.0f,0.0f,0.591399983359874870103567445767112076282501220703125f,0.2725999841630455389207554617314599454402923583984375f,0.1348999969124793363306480387109331786632537841796875f,0.0f,0.0f,0.4082000186882901449081373357330448925495147705078125f,-0.82719998060464894340526598170981742441654205322265625f,0.957300044284014717277386807836592197418212890625f,0.0f,0.0f,-1.7038999906614422652495477450429461896419525146484375f,-0.5392999970883127769383236227440647780895233154296875f,0.5041999308198696905236602106015197932720184326171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.2488000352203842968634717180975712835788726806640625f,1.50189991859197657930735658737830817699432373046875f,0.66439999473691013776033287285827100276947021484375f,0.0f,0.0f,-0.50079999102354033624351359321735799312591552734375f,-0.098999962270259533880789604154415428638458251953125f,1.823900000607967086807548184879124164581298828125f,0.0f,0.0f,0.2060000069215892626317554459092207252979278564453125f,-0.00480002228617681936384542495943605899810791015625f,-0.978299963903427549638536220300011336803436279296875f,0.0f,0.0f,-0.83710000933110695342520557460375130176544189453125f,0.9513000171899790302632027305662631988525390625f,-0.1565999738514427530589046000386588275432586669921875f,0.0f,0.0f,-0.9884000115454203427844959151116199791431427001953125f,-0.67509998431205797686516234534792602062225341796875f,-1.327199973489343154398056867648847401142120361328125f,0.0f,0.0f,-0.38679997252002362717604455610853619873523712158203125f,0.02999995738267902112283991300500929355621337890625f,-1.6062999939441677810236797085963189601898193359375f,0.0f,0.0f,-0.1427000233113773219884024001657962799072265625f,-0.58549995367228790943414651337661780416965484619140625f,0.7838999329566949114678209298290312290191650390625f,0.0f,0.0f,1.5282000461809326363749050869955681264400482177734375f,0.49410000067353276875792289501987397670745849609375f,-0.11840000755935931042728270767838694155216217041015625f,0.0f,0.0f,-1.7631999468274421882796332283760420978069305419921875f,-0.131800035160779760445848296512849628925323486328125f,-0.4407000004157428652007411074009723961353302001953125f,0.0f,0.0f,1.58959998362660481774355503148399293422698974609375f,-0.83529996809810480584701508632861077785491943359375f,0.183200014460087601264604018069803714752197265625f,0.0f,0.0f,-1.7819998584985743406150504597462713718414306640625f,0.12610003521069856891045901647885330021381378173828125f,1.8429000168576832319189406916848383843898773193359375f,0.0f,0.0f,0.07560000693946966610070603564963676035404205322265625f,-2.12520002420544518173528558691032230854034423828125f,-0.3055999984830621851727983084856532514095306396484375f,0.0f,0.0f,0.33140004620552065972560740192420780658721923828125f,-0.9487000224843618045866833199397660791873931884765625f,-1.299200013372300954728189026354812085628509521484375f,0.0f,0.0f,-1.433299915474653740687926983810029923915863037109375f,0.52569999009668777478765377964009530842304229736328125f,0.744400071459263568129927080008201301097869873046875f,0.0f,0.0f,1.155099969820678751375453430227935314178466796875f,0.6125999535039066135055918493890203535556793212890625f,0.890899949407577995685869609587825834751129150390625f,0.0f,0.0f,0.64760001560449520496831610216759145259857177734375f,0.247200015436112696676218547509051859378814697265625f,-0.6704999625384797656835189627599902451038360595703125f,0.0f,0.0f,1.51170000301599483094605602673254907131195068359375f,0.271200028705596452027748455293476581573486328125f,-0.4484000024557115438028631615452468395233154296875f,0.0f,0.0f,-0.35010002692937813861817630822770297527313232421875f,0.212199991929530984435814389144070446491241455078125f,1.070699989873170832055393475457094609737396240234375f,0.0f,0.0f,0.712299992260336356508787503116764128208160400390625f,0.4411999684303993252143527570297010242938995361328125f,-0.6228000372767450443234338308684527873992919921875f,0.0f,0.0f,1.090200011241435174014213771442882716655731201171875f,0.0894000208139420493580473703332245349884033203125f,0.33579999117255232565781852827058173716068267822265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.50149998894333858157068561922642402350902557373046875f,1.4667000057727097583892827969975769519805908203125f,-0.01580000419244209641789211673312820494174957275390625f,0.0f,0.0f,-1.65009997172057598646688347798772156238555908203125f,2.330700043144822597440679601277224719524383544921875f,1.762399994193017516153076940099708735942840576171875f,0.0f,0.0f,-1.07509999086856833372394248726777732372283935546875f,-1.35480006829351129482574833673425018787384033203125f,0.9786000084817405575421389585244469344615936279296875f,0.0f,0.0f,-2.405700042921305570331469425582326948642730712890625f,-1.505600011372565827372227431624196469783782958984375f,-0.12409997614249579367395881490665487945079803466796875f,0.0f,0.0f,0.36340003192275605048422448817291297018527984619140625f,-0.2216000723853703835430906110559590160846710205078125f,1.668700037273764369416539921076036989688873291015625f,0.0f,0.0f,-2.979699936087430334197279080399312078952789306640625f,-0.511599982568621935996588945272378623485565185546875f,-1.391899980152399241006833108258433640003204345703125f,0.0f,0.0f,2.032299938914178483173600398004055023193359375f,0.550100083032251330195094851660542190074920654296875f,-0.6655999452672889393767263754853047430515289306640625f,0.0f,0.0f,-2.44949998370558130744711888837628066539764404296875f,-1.407899934118986617903601654688827693462371826171875f,-1.0637000075832017120802674980950541794300079345703125f,0.0f,0.0f,-2.462299972181023033357405438437126576900482177734375f,2.481700021754205121027325731120072305202484130859375f,-0.426199898935110343956012002308852970600128173828125f,0.0f,0.0f,2.04749993557110343544991337694227695465087890625f,0.4112000153318036144156621958245523273944854736328125f,1.297399967557191668987570665194652974605560302734375f,0.0f,0.0f,-1.6716000243477526776558761412161402404308319091796875f,-0.8830999740079032989115148666314780712127685546875f,1.5617000320360059362201354815624654293060302734375f,0.0f,0.0f,1.231600024403632698266619627247564494609832763671875f,-0.44119999122917530343102043843828141689300537109375f,0.207000028520823331490419150213710963726043701171875f,0.0f,0.0f,2.085200058962403346640712697990238666534423828125f,-0.10709993959367292148954220465384423732757568359375f,-2.8932000608958308163209949270822107791900634765625f,0.0f,0.0f,1.0885999975293880748239416789147071540355682373046875f,-0.88589999964311572089314950062544085085391998291015625f,4.00070001878142278428640565834939479827880859375f,0.0f,0.0f,0.03059997674599246320070733418106101453304290771484375f,-1.1832000231958923475161782334907911717891693115234375f,-1.324300010015069251068098310497589409351348876953125f,0.0f,0.0f,-0.71829997064620265678769328587804920971393585205078125f,0.67590001255124843027033421094529330730438232421875f,-0.7945999818757172317162940089474432170391082763671875f,0.0f,0.0f,-0.28300004062056505205191569984890520572662353515625f,0.09299999740719844254499548696912825107574462890625f,0.640799980853498762911613084725104272365570068359375f,0.0f,0.0f,0.254199968296289657843090026290155947208404541015625f,-0.10589997945204310259015301198814995586872100830078125f,-2.40849996292591139734895477886311709880828857421875f,0.0f,0.0f,0.1270000355988731843837058477220125496387481689453125f,1.3122001063331956149937695954577066004276275634765625f,-2.289400020795316237354199984110891819000244140625f,0.0f,0.0f,-1.6103000117585060113611916676745750010013580322265625f,0.86879999515488737227286719644325785338878631591796875f,-0.99979999693557619622197307762689888477325439453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.851200076277553563386391033418476581573486328125f,1.1658999789305026784091978697688318789005279541015625f,-1.964199961516261083716017310507595539093017578125f,0.0f,0.0f,-1.310399987688660505824600477353669703006744384765625f,-3.113000001877546996098544695996679365634918212890625f,0.1236999689146871528322435551672242581844329833984375f,0.0f,0.0f,0.610999969854951086034589025075547397136688232421875f,-3.103800020766259049054269780754111707210540771484375f,2.1482000339061020355302389361895620822906494140625f,0.0f,0.0f,0.7546000280320639408415672733099199831485748291015625f,0.0261000075876711701283738875645212829113006591796875f,-0.81170007260441767726888429024256765842437744140625f,0.0f,0.0f,-2.247500061765312917572146034217439591884613037109375f,1.39610006439834766212015892961062490940093994140625f,-1.128300014045834132048184983432292938232421875f,0.0f,0.0f,-0.601599989981949523354387565632350742816925048828125f,-2.0967000187367208496880266466177999973297119140625f,-0.4886000028938060069094717619009315967559814453125f,0.0f,0.0f,0.294299939420820777513654320500791072845458984375f,-0.7426000151947131922014477822813205420970916748046875f,1.03449999670684267272235956625081598758697509765625f,0.0f,0.0f,-0.5717000774100415849687806257861666381359100341796875f,-1.8172999998599290716327914196881465613842010498046875f,2.42839998714476745789170308853499591350555419921875f,0.0f,0.0f,-3.58119998191595190206726329051889479160308837890625f,2.2707000342786312074849774944595992565155029296875f,-2.014499962843955049862643136293627321720123291015625f,0.0f,0.0f,-0.2354000316098334710801509572775103151798248291015625f,-2.26860001690089685411066966480575501918792724609375f,-4.34709999256730128536219126544892787933349609375f,0.0f,0.0f,-0.214299849455057589420192698526079766452312469482421875f,3.107000048097221966969527784385718405246734619140625f,-1.8496000773921625892626252607442438602447509765625f,0.0f,0.0f,1.197500056773424415013096222537569701671600341796875f,-0.5696000805959100521391746951849199831485748291015625f,-2.55940004197359183280013894545845687389373779296875f,0.0f,0.0f,0.535600067624449227565719411359168589115142822265625f,0.85389999438673225284901491249911487102508544921875f,0.27639999893158695432049398732488043606281280517578125f,0.0f,0.0f,-0.44070000507235562725583122301031835377216339111328125f,0.80390001346021922046247709658928215503692626953125f,1.6632999953076239574301098400610499083995819091796875f,0.0f,0.0f,0.5356999757915728022084067561081610620021820068359375f,-0.8347999933034173647428133335779421031475067138671875f,-2.507299957093596987078853999264538288116455078125f,0.0f,0.0f,-3.6722000376760970397072014748118817806243896484375f,-0.8454999622963377259310391309554688632488250732421875f,-2.72100005939230360496594585129059851169586181640625f,0.0f,0.0f,-0.974700016814470604487041782704181969165802001953125f,0.77939999190568876041851353875244967639446258544921875f,0.2207000504463907386565324486582539975643157958984375f,0.0f,0.0f,-0.34299996522069065019167055652360431849956512451171875f,-1.1906000028043972616131895847502164542675018310546875f,-0.18369996671676691590846530743874609470367431640625f,0.0f,0.0f,-1.85930001279041068329433983308263123035430908203125f,1.3993000297591093872284773169667460024356842041015625f,1.138800044067948658010891449521295726299285888671875f,0.0f,0.0f,-0.025999980192631666486846597763360477983951568603515625f,0.56540001153051822058159814332611858844757080078125f,0.72769996091276478722420506528578698635101318359375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.061800019216537549482382019050419330596923828125f,-1.6782000295102594833451803424395620822906494140625f,-0.2480999621987347580898131127469241619110107421875f,0.0f,0.0f,1.7147999604463570744172784543479792773723602294921875f,1.5329999798238276298434357158839702606201171875f,0.5190000101029870194224713486619293689727783203125f,0.0f,0.0f,-2.480700010548531508902669884264469146728515625f,2.8958999815940860145246915635652840137481689453125f,3.6717999961942435760420266888104379177093505859375f,0.0f,0.0f,0.104200003463029933215011624270118772983551025390625f,0.882900058208405891235770468483678996562957763671875f,-0.5691999916911125723117947927676141262054443359375f,0.0f,0.0f,0.8692999813303352585336369884316809475421905517578125f,-2.03950007672607913633555654087103903293609619140625f,-0.676099986167252087199130983208306133747100830078125f,0.0f,0.0f,0.3235999856829649967693285361747257411479949951171875f,0.23290002829432498998585288063623011112213134765625f,-2.326800034713744746994734668987803161144256591796875f,0.0f,0.0f,-0.84379996779263033435114493840956129133701324462890625f,-1.253500014781952387465935316868126392364501953125f,0.9486999911732969525957059886422939598560333251953125f,0.0f,0.0f,-0.8651999924659723451014770034817047417163848876953125f,1.025300045993924324960744343115948140621185302734375f,2.8500000067055228925028131925500929355621337890625f,0.0f,0.0f,1.430899979731441451491491534397937357425689697265625f,-1.88699992980062969394339233986102044582366943359375f,-0.7954000107109553407980229167151264846324920654296875f,0.0f,0.0f,-1.82119996120333649258782315882854163646697998046875f,-1.001999959174543874951268662698566913604736328125f,-1.85769996614679744340037359506823122501373291015625f,0.0f,0.0f,-0.8681000112608074914533062838017940521240234375f,-2.122700008708239938215456277248449623584747314453125f,0.2650999948173760056846504085115157067775726318359375f,0.0f,0.0f,1.21299994175136038876416932907886803150177001953125f,0.391600041130184539639458307647146284580230712890625f,1.7607999580919744264662085697636939585208892822265625f,0.0f,0.0f,-0.5250000117719164638430129343760199844837188720703125f,-1.486699980068207249672695979825221002101898193359375f,-0.428299947363138588940500994795002043247222900390625f,0.0f,0.0f,0.386200049865245365054988724295981228351593017578125f,2.343399952276052022881458469782955944538116455078125f,-0.288300057091563477484896793612278997898101806640625f,0.0f,0.0f,1.6025000113062566331478819847689010202884674072265625f,0.9482999961458145055104296261561103165149688720703125f,1.5829999687224631799153939937241375446319580078125f,0.0f,0.0f,-1.5727999545939270742422877447097562253475189208984375f,-0.4174999781697985579143050927086733281612396240234375f,0.4583000005602837489959711092524230480194091796875f,0.0f,0.0f,-0.3769000136055049576810915823443792760372161865234375f,0.78860002040266952150204815552569925785064697265625f,-0.304000007569789953976169272209517657756805419921875f,0.0f,0.0f,0.4821000117897986658732634168700315058231353759765625f,-0.028100005263090199747466613189317286014556884765625f,-0.472799977374076974001582129858434200286865234375f,0.0f,0.0f,0.8939999645054343968553212107508443295955657958984375f,0.056099991159140660812454370898194611072540283203125f,0.28869999368786825044708166387863457202911376953125f,0.0f,0.0f,-0.057900017025321659502878901548683643341064453125f,0.9795999918222424440727991168387234210968017578125f,1.49840000260472283599710863200016319751739501953125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.994900029285252340827128136879764497280120849609375f,-0.11430000550746921561540148104541003704071044921875f,0.4847999883487819072769298145431093871593475341796875f,0.0f,0.0f,0.6553000196933747023564365008496679365634918212890625f,-0.620499999940395952080507413484156131744384765625f,0.92320001527965100951433896625530906021595001220703125f,0.0f,0.0f,0.04180004134476267285691619690624065697193145751953125f,0.362200061964988939688936397942597977817058563232421875f,1.6168000025272366571016391390003263950347900390625f,0.0f,0.0f,0.15590003412067954258191093686036765575408935546875f,-1.1634999325275423576186994978343136608600616455078125f,-0.48219999768510479754013431374914944171905517578125f,0.0f,0.0f,-0.5331999501988289313914037848007865250110626220703125f,0.27229997713565889938536201952956616878509521484375f,-2.625199956107140852878956138738431036472320556640625f,0.0f,0.0f,1.23679997458755952521869403426535427570343017578125f,2.41529998407960011519435283844359219074249267578125f,-0.01909995570927847108322339408914558589458465576171875f,0.0f,0.0f,0.45399993623048207069814452552236616611480712890625f,2.124199979099631985235419051605276763439178466796875f,-3.64700004815310219186130780144594609737396240234375f,0.0f,0.0f,-0.23500002693384802743281625225790776312351226806640625f,1.256299935978651749479695354239083826541900634765625f,1.118799998787045613113377839908935129642486572265625f,0.0f,0.0f,1.400699996578693262705428423942066729068756103515625f,-2.4251001042664057649744790978729724884033203125f,1.2699999941512931656006912817247211933135986328125f,0.0f,0.0f,-2.74819998253434771839920358615927398204803466796875f,-0.1722000063464041996752484919852577149868011474609375f,1.101000085264443928423361285240389406681060791015625f,0.0f,0.0f,0.68719993984997296276873157694353722035884857177734375f,-1.480999945305287557317797109135426580905914306640625f,-3.45160003468543319371519828564487397670745849609375f,0.0f,0.0f,1.3501998579829932456419783193268813192844390869140625f,0.093200082148612306554014139692299067974090576171875f,0.646899992986022187579919773270376026630401611328125f,0.0f,0.0f,0.6191999136090282629396597258164547383785247802734375f,2.979400002634525090883244047290645539760589599609375f,-0.00839997153580129829464340218692086637020111083984375f,0.0f,0.0f,-0.003599970334767998991765125538222491741180419921875f,2.4552000474140047714399770484305918216705322265625f,-0.6715999475136402718788986021536402404308319091796875f,0.0f,0.0f,-2.653500049725174836368069009040482342243194580078125f,-0.7085999434940524199788569603697396814823150634765625f,1.4045000566840168954030332315596751868724822998046875f,0.0f,0.0f,0.3283999696917823118980095387087203562259674072265625f,-1.08810007410645415149019754608161747455596923828125f,-0.880799978264420690976521655102260410785675048828125f,0.0f,0.0f,-0.5410999552845954241320214350707828998565673828125f,1.1557999271906904681372907361947000026702880859375f,-0.59740002097785538381913283956237137317657470703125f,0.0f,0.0f,-0.7212999510288238269595240126363933086395263671875f,0.86250002088025168323071056875050999224185943603515625f,-0.187000004343689074648438008807715959846973419189453125f,0.0f,0.0f,-3.01619995043724742345148115418851375579833984375f,-3.40509998323172258238855647505261003971099853515625f,-0.43760002653822216434065239809569902718067169189453125f,0.0f,0.0f,1.66619999916404548656601036782376468181610107421875f,0.56269999499917133789494982920587062835693359375f,-0.5205999750882386312156313579180277884006500244140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.6884000213056802319755433927639387547969818115234375f,-2.702500000111758282628215965814888477325439453125f,1.637699968773126624199676371063105762004852294921875f,0.0f,0.0f,0.6038999883890150410792330148979090154170989990234375f,0.157799969923496430368459186865948140621185302734375f,-1.14240000488460058392092832946218550205230712890625f,0.0f,0.0f,-1.089800045382975923757840064354240894317626953125f,0.29400002790987489476037808344699442386627197265625f,-1.575900091713667183768166069057770073413848876953125f,0.0f,0.0f,0.913099974305927819528960753814317286014556884765625f,-1.278599988830834366382305233855731785297393798828125f,0.4734000316992401646842836271389387547969818115234375f,0.0f,0.0f,-0.7566000859081751794832371160737238824367523193359375f,-0.5641000225782395460782936424948275089263916015625f,-1.77749996058642789620307667064480483531951904296875f,0.0f,0.0f,2.9541999881893392654319541179575026035308837890625f,2.940899922138453081998932248097844421863555908203125f,2.30720004222095109724932626704685389995574951171875f,0.0f,0.0f,-0.27660003407075983705709631976787932217121124267578125f,-1.97329993566572792218494214466772973537445068359375f,0.317299999301134949547531505231745541095733642578125f,0.0f,0.0f,0.05969991455525247570079727665870450437068939208984375f,-2.033099970729649985656806165934540331363677978515625f,-1.216100019713489910344605959835462272167205810546875f,0.0f,0.0f,-0.7664000386059288683071599734830670058727264404296875f,-2.13790002814531288066746128606610000133514404296875f,-0.6091000094465908176744051161222159862518310546875f,0.0f,0.0f,0.6199000369966023438195179551257751882076263427734375f,0.85210001618564223946350466576404869556427001953125f,1.91230000731050875373284725355915725231170654296875f,0.0f,0.0f,0.4488000133156784254850890647503547370433807373046875f,-0.878300022707134697697028968832455575466156005859375f,2.137799911436438105027946221525780856609344482421875f,0.0f,0.0f,-1.8981999703899028464348930356209166347980499267578125f,-1.1174000060021871405524507281370460987091064453125f,1.4082999951213597267241084409761242568492889404296875f,0.0f,0.0f,0.3397999924838550622752109120483510196208953857421875f,2.162599978649616172532432756270281970500946044921875f,-0.513299992960691842114329119795002043247222900390625f,0.0f,0.0f,0.639200021940469209624779978184960782527923583984375f,2.32000002101063618198395488434471189975738525390625f,-0.761300007504224662824299230123870074748992919921875f,0.0f,0.0f,-0.288199882212281099924666705192066729068756103515625f,-2.267000002816320858300969121046364307403564453125f,-0.635999975964427477492790785618126392364501953125f,0.0f,0.0f,-0.12429995517879677269235116909840144217014312744140625f,0.4007999938547615048634042977937497198581695556640625f,0.6275000694394103906148529858910478651523590087890625f,0.0f,0.0f,-0.286800031957029855078644686727784574031829833984375f,1.9772000043347477404864775962778367102146148681640625f,1.790199936139583059002688969485461711883544921875f,0.0f,0.0f,1.58650002679228752100470956065692007541656494140625f,1.159799963501094932638579848571680486202239990234375f,1.323699965077638207588961449800990521907806396484375f,0.0f,0.0f,2.887400011534243748911876537022180855274200439453125f,0.1142000163897869402518381320987828075885772705078125f,2.058200034148245816112421380239538848400115966796875f,0.0f,0.0f,-0.62320003912150834235461616117390803992748260498046875f,0.16649999073147758732460488317883573472499847412109375f,0.28820000253915811772031929649529047310352325439453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.0034000094592572249752038260339759290218353271484375f,0.072700021523237179366105920053087174892425537109375f,0.722399981564283422130756662227213382720947265625f,0.0f,0.0f,-1.0051999983519317449776053763343952596187591552734375f,-1.1357999563038354562394260938162915408611297607421875f,-1.0589000056743620437060826589004136621952056884765625f,0.0f,0.0f,0.7746000270634885165321747990674339234828948974609375f,-0.3890000065267091855503167607821524143218994140625f,-0.9346000252753501325031493252026848495006561279296875f,0.0f,0.0f,-0.0171000153660774145691902958787977695465087890625f,1.3122999603718514816108609011280350387096405029296875f,-0.45139999867081659346013111644424498081207275390625f,0.0f,0.0f,-0.6563999835461380438772494017030112445354461669921875f,1.0392000068157916103928073425777256488800048828125f,-0.551199976626038701255083651631139218807220458984375f,0.0f,0.0f,-0.720699958431720855145385939977131783962249755859375f,-0.9203999720424409503038987168110907077789306640625f,-0.277699979352951320521469824598170816898345947265625f,0.0f,0.0f,0.13529999719262131296915185885154642164707183837890625f,-1.47079997336566403731694663292728364467620849609375f,-1.305200070082396024417903390713036060333251953125f,0.0f,0.0f,-1.147699954393506249772372029838152229785919189453125f,1.48709995378702952706362339085899293422698974609375f,-2.01679996531158689521134874667041003704071044921875f,0.0f,0.0f,-1.36169993365108954463948975899256765842437744140625f,0.8325999848335980946245626910240389406681060791015625f,0.440899983754754476450443689827807247638702392578125f,0.0f,0.0f,1.7179000008761875495366666655172593891620635986328125f,1.3222999559573824779334927370655350387096405029296875f,0.43109999613240346238995925887138582766056060791015625f,0.0f,0.0f,-0.2800000425800686087995927664451301097869873046875f,-0.3984000430241214640858515849686227738857269287109375f,-1.394300032292305591141712284297682344913482666015625f,0.0f,0.0f,-1.036499953962862097256447668769396841526031494140625f,1.3531999550789601816092044828110374510288238525390625f,-0.5396999841623009164237600998603738844394683837890625f,0.0f,0.0f,0.485800018590688775788066777749918401241302490234375f,1.4479999781846999429291145133902318775653839111328125f,2.502699974435568019970332898083142936229705810546875f,0.0f,0.0f,1.138600001925230031929459073580801486968994140625f,-0.05250000838190327367982490613940171897411346435546875f,0.6117999562218783271561051151365973055362701416015625f,0.0f,0.0f,-0.09579996442496929720533671570592559874057769775390625f,-0.101800002024322988614102314386400394141674041748046875f,1.0215999861076474619636655916110612452030181884765625f,0.0f,0.0f,2.69859997376203519792170482105575501918792724609375f,-0.4587999634712935925762167244101874530315399169921875f,-0.27439997685551709150075794241274707019329071044921875f,0.0f,0.0f,0.310400025556235747981048689325689338147640228271484375f,1.00029998989105184392656155978329479694366455078125f,1.10180001936554905483944821753539144992828369140625f,0.0f,0.0f,-0.44810001137256616488002691767178475856781005859375f,0.414199989157915116066988048260100185871124267578125f,-1.380000001788138863645372111932374536991119384765625f,0.0f,0.0f,0.171199979382753164625086128580733202397823333740234375f,1.170900000071525060008070795447565615177154541015625f,0.6638999696880587553238228792906738817691802978515625f,0.0f,0.0f,0.7147999568887062338262694538570940494537353515625f,-0.3422999662205581383744856793782673776149749755859375f,-0.186400013907253470080149782006628811359405517578125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.767200022849440887995342563954181969165802001953125f,0.1321999589979643285886368175852112472057342529296875f,0.4227999464355411962657171898172236979007720947265625f,0.0f,0.0f,-0.463499948434532793584139653830789029598236083984375f,-1.8701000458165994988490865580388344824314117431640625f,-1.351300043155251984217102290131151676177978515625f,0.0f,0.0f,-0.1149000234365467054686860137735493481159210205078125f,-1.8802999965965743189855174932745285332202911376953125f,-1.591599983909725235520227215602062642574310302734375f,0.0f,0.0f,0.50459996593147404819745815984788350760936737060546875f,0.4860999245136967683578177457093261182308197021484375f,-0.3921000426724561283009506951202638447284698486328125f,0.0f,0.0f,-1.3317000167995676207510769017972052097320556640625f,-2.15189998121410486220383972977288067340850830078125f,2.91710001413673136738680113921873271465301513671875f,0.0f,0.0f,-2.797999912992121362975694864871911704540252685546875f,-1.6739000495210285546221484764828346669673919677734375f,1.5691000339776284278059392818249762058258056640625f,0.0f,0.0f,0.5258999885231274706853810130269266664981842041015625f,-2.10450003943592367505743823130615055561065673828125f,4.5603999328896396292520876158960163593292236328125f,0.0f,0.0f,-2.119399961432814283313064152025617659091949462890625f,-2.805700016620755743446125052287243306636810302734375f,1.851100058288873473344438025378622114658355712890625f,0.0f,0.0f,0.242599998915194792203919860185123980045318603515625f,2.0099998970329746583729502162896096706390380859375f,0.0711999683372666059000266614020802080631256103515625f,0.0f,0.0f,0.90470007608383895547632391753722913563251495361328125f,-0.7376000220119944827956715016625821590423583984375f,-2.314499987877904896294012360158376395702362060546875f,0.0f,0.0f,-5.09770004841983226384627414518035948276519775390625f,0.6800001364946350879137071387958712875843048095703125f,3.749299941767751054300106261507607996463775634765625f,0.0f,0.0f,-2.62949994464591174647694060695357620716094970703125f,-1.1408000881418598471128689197939820587635040283203125f,-2.025700022245944698084940682747401297092437744140625f,0.0f,0.0f,-3.5566999718353162762696229037828743457794189453125f,-0.19450003913790059328903225832618772983551025390625f,-0.6102000642523146556328583756112493574619293212890625f,0.0f,0.0f,-2.05939988152533803855703808949328958988189697265625f,-3.09499998463317727015464697615243494510650634765625f,0.514299909394979604115860638557933270931243896484375f,0.0f,0.0f,1.580100051683931638990543433465063571929931640625f,-2.387999990403651739967472167336381971836090087890625f,0.503999921627344971142292706645093858242034912109375f,0.0f,0.0f,1.830999978534877836722216670750640332698822021484375f,0.1239999656602732702737057479680515825748443603515625f,-2.1843999605014925435853001545183360576629638671875f,0.0f,0.0f,-0.0681999495655301046781460172496736049652099609375f,0.81330004988312654123916445314534939825534820556640625f,0.5915999483704557082575092863407917320728302001953125f,0.0f,0.0f,-1.78849998900294249182252315222285687923431396484375f,-0.5533999660409978371689021514612250030040740966796875f,0.54780000024735908681350338156335055828094482421875f,0.0f,0.0f,0.8483999890446660696596836714888922870159149169921875f,-1.3153000088527806443750023390748538076877593994140625f,0.029500044688582693908074361388571560382843017578125f,0.0f,0.0f,1.226000002861022952771463678800500929355621337890625f,-1.3099999980628496132339932955801486968994140625f,-1.031999956510961879985188716091215610504150390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-2.044000007160007736928264421294443309307098388671875f,3.047700015451014099454596362193115055561065673828125f,-1.6076999906033277998318453683168627321720123291015625f,0.0f,0.0f,-1.480400032727420267519846674986183643341064453125f,-1.5044999970793722088302502015721984207630157470703125f,1.898800040063262084544248864403925836086273193359375f,0.0f,0.0f,0.54290002404749537756600830107345245778560638427734375f,-0.5809000638484962397711797166266478598117828369140625f,2.4576000181376951303491296130232512950897216796875f,0.0f,0.0f,-0.38429996084123896338269332773052155971527099609375f,0.20319992993324920149689205572940409183502197265625f,-0.0825000494718552968009817050187848508358001708984375f,0.0f,0.0f,1.433300171793251198693042169907130300998687744140625f,1.37389995594173708326479754759930074214935302734375f,-1.32329986633807461515743852942250669002532958984375f,0.0f,0.0f,-2.429799990993738223465925329946912825107574462890625f,-0.96090004015564811634675379536929540336132049560546875f,-1.8651999903425551252666991786099970340728759765625f,0.0f,0.0f,1.7642000292234130132129621415515430271625518798828125f,-0.621100016155842649112628350849263370037078857421875f,3.35279992361813850010321402805857360363006591796875f,0.0f,0.0f,-0.8909000193685285839961807141662575304508209228515625f,-3.72020002666413684977442244417034089565277099609375f,1.452199940147995693706661768374033272266387939453125f,0.0f,0.0f,-2.143199834472687914654898122535087168216705322265625f,0.3622000023975957549993154316325671970844268798828125f,1.983400092011689874738067373982630670070648193359375f,0.0f,0.0f,-2.21839996132850547638781790737994015216827392578125f,2.269800019231440035838431867887265980243682861328125f,-0.7207999921038752422219886284437961876392364501953125f,0.0f,0.0f,0.849999965727328277154128954862244427204132080078125f,-1.47610002310350463261556797078810632228851318359375f,-2.25490001289397223871446840348653495311737060546875f,0.0f,0.0f,1.120499930016695788737024486181326210498809814453125f,0.61309995998218702606408214705879800021648406982421875f,-2.661499889925121653533324206364341080188751220703125f,0.0f,0.0f,1.664699970360100422794857877306640148162841796875f,-0.533700018610058890544678433798253536224365234375f,0.150699978362023923050827534098061732947826385498046875f,0.0f,0.0f,-1.05490007506907002010620999499224126338958740234375f,-3.62120000937134012275464556296356022357940673828125f,-0.01409995671510622106126220387523062527179718017578125f,0.0f,0.0f,-0.565200042347609343806880133342929184436798095703125f,3.03230000131279009423224124475382268428802490234375f,-4.2239999041184805861348650068975985050201416015625f,0.0f,0.0f,-1.0633000920161601587921040845685638487339019775390625f,-1.2274999536201354199960178448236547410488128662109375f,-0.1118999884411702172570812763296999037265777587890625f,0.0f,0.0f,1.12270003336966084361847606487572193145751953125f,-0.3867000265225761790333081080461852252483367919921875f,0.698700010973215324128204883891157805919647216796875f,0.0f,0.0f,-1.037799932223558929678119966411031782627105712890625f,-0.313199920582771795807275339029729366302490234375f,-0.484600050458312114898262734641321003437042236328125f,0.0f,0.0f,-3.6487998985208580648986753658391535282135009765625f,2.39420003117918867019398021511733531951904296875f,-0.89269992460980984727569875758490525186061859130859375f,0.0f,0.0f,0.276200055266916988383485431768349371850490570068359375f,-0.5632999963134526222319209409761242568492889404296875f,0.20490000481009484900596362422220408916473388671875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.51240000831186771623748654747032560408115386962890625f,0.069599997186660989001438792911358177661895751953125f,-0.38229998909383999805555731654749251902103424072265625f,0.0f,0.0f,0.272299994793534338999307919948478229343891143798828125f,1.0417999733209608592687800410203635692596435546875f,0.367200022737681763818073932270635850727558135986328125f,0.0f,0.0f,-1.2634000060692429645570200591464526951313018798828125f,1.5643999693676826634458620901568792760372161865234375f,1.4356000012025240675228587861056439578533172607421875f,0.0f,0.0f,-0.46069999426901342420848095571273006498813629150390625f,-0.51859999510794896337984027923084795475006103515625f,1.490999998725950348443802795372903347015380859375f,0.0f,0.0f,-0.7561000656276934961397273582406342029571533203125f,-0.91570000354498592987084748529014177620410919189453125f,-0.514999967850744422293018942582421004772186279296875f,0.0f,0.0f,1.263900023369491254499052956816740334033966064453125f,0.362000031240283626221554413859848864376544952392578125f,-0.20670001322329056847593165002763271331787109375f,0.0f,0.0f,-1.0854000255189830692614805229823105037212371826171875f,-1.66869993888884682320394858834333717823028564453125f,1.01719998084679286165510347927920520305633544921875f,0.0f,0.0f,0.78030000430792512133848504163324832916259765625f,1.4272000082463016656930676617776043713092803955078125f,0.2234000264465809681979635570314712822437286376953125f,0.0f,0.0f,0.63249997917562783555922578671015799045562744140625f,-1.17779992358088581028141561546362936496734619140625f,-1.106899977973103244011099377530626952648162841796875f,0.0f,0.0f,-1.2529999797120685567364262169576250016689300537109375f,-0.52519998048916505251071384918759576976299285888671875f,-0.953599978474528153782330264220945537090301513671875f,0.0f,0.0f,0.1742000458568326859420949404011480510234832763671875f,-0.34019997734129392430446614525862969458103179931640625f,0.11349998393654858741541602284996770322322845458984375f,0.0f,0.0f,0.2170000148117543492531922311172820627689361572265625f,-0.88480002575069593984125049246358685195446014404296875f,-0.28049999498575972634029085384099744260311126708984375f,0.0f,0.0f,1.448399965277314294098687241785228252410888671875f,-2.119699907104671599000766946119256317615509033203125f,-1.336499936453998227392503395094536244869232177734375f,0.0f,0.0f,0.9787000319093464018038730500848032534122467041015625f,1.3311999613523486374333515414036810398101806640625f,0.13489999326169321847146420623175799846649169921875f,0.0f,0.0f,2.022799992163479831930317232036031782627105712890625f,0.876299983401596538357125609763897955417633056640625f,0.8314999788105488232048401187057606875896453857421875f,0.0f,0.0f,-1.507799982552230577681484646745957434177398681640625f,-0.52540001842230577011605419102124869823455810546875f,-0.59699996478110506359371356666088104248046875f,0.0f,0.0f,-1.281799939886481443096499788225628435611724853515625f,0.9765000413954254554482758976519107818603515625f,-0.7367999828577038723409486919990740716457366943359375f,0.0f,0.0f,0.67009998416304561441592113624210469424724578857421875f,-0.46389996007680955880658757450873963534832000732421875f,-1.157599976861477752976270494400523602962493896484375f,0.0f,0.0f,1.092199971216917031568982565659098327159881591796875f,-0.2562999595224859650244297881727106869220733642578125f,1.024499959453941233533669219468720257282257080078125f,0.0f,0.0f,-0.21860001371577342244023611783632077276706695556640625f,-0.072900002962350551882764193578623235225677490234375f,0.65999997481703776003314487752504646778106689453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.690800095741451958275547440280206501483917236328125f,-1.749100005851686123747867895872332155704498291015625f,0.0685999674290418415267822638270445168018341064453125f,0.0f,0.0f,1.1156000036239621664435617276467382907867431640625f,-2.541999977521598008678438418428413569927215576171875f,-0.83040000244080969249438339829794131219387054443359375f,0.0f,0.0f,1.5780000216960907710728179154102690517902374267578125f,-0.235199961285293357438064276720979250967502593994140625f,-0.946000039182603114085168272140435874462127685546875f,0.0f,0.0f,0.4222000123068687660321529619977809488773345947265625f,2.68369997178316044283974406425841152667999267578125f,-0.0872000405631958575014550660853274166584014892578125f,0.0f,0.0f,-0.6015000731348998375125347592984326183795928955078125f,-2.29839988712072251786366905434988439083099365234375f,1.3429999315068117393678903681575320661067962646484375f,0.0f,0.0f,1.5457999414399250515117500981432385742664337158203125f,3.29429998863190487412566653802059590816497802734375f,2.31229993775934072886002468294464051723480224609375f,0.0f,0.0f,0.9034000004440532194394108955748379230499267578125f,-2.8586999385163185394276297301985323429107666015625f,1.43259990060478425988321760087274014949798583984375f,0.0f,0.0f,0.268700006502867072555318372906185686588287353515625f,-0.088600063057244771158593721338547766208648681640625f,0.157700009974837573878403418348170816898345947265625f,0.0f,0.0f,2.2230000260733078221164760179817676544189453125f,-2.894600039841235261661722688586451113224029541015625f,2.45509994949922027984712258330546319484710693359375f,0.0f,0.0f,-2.705499993439763439795342492288909852504730224609375f,1.663099922785164164196203273604623973369598388671875f,2.24460006107538756481289965449832379817962646484375f,0.0f,0.0f,-2.486299985618144692267605933011509478092193603515625f,2.50040000903457571013177584973163902759552001953125f,-1.1975999576576068506028605042956769466400146484375f,0.0f,0.0f,-1.6182000174403190495553417349583469331264495849609375f,4.067899980051816299919664743356406688690185546875f,-3.60890007809400525928822389687411487102508544921875f,0.0f,0.0f,-0.1082000687003124017593336247955448925495147705078125f,1.8433000340133904870754122384823858737945556640625f,0.1125000371038906710197124994010664522647857666015625f,0.0f,0.0f,-1.00369999014884303534245191258378326892852783203125f,0.1538000301539901304437307771877385675907135009765625f,-1.0498000148728479086202014514128677546977996826171875f,0.0f,0.0f,-2.8205000055097020350558523205108940601348876953125f,0.0504000501804064970912122589652426540851593017578125f,0.2282000184834005462874984004884026944637298583984375f,0.0f,0.0f,-2.52450000412017061535152606666088104248046875f,-2.00700001689791651671157524106092751026153564453125f,1.515899978706985340437540799030102789402008056640625f,0.0f,0.0f,0.88619997010678108750880710431374609470367431640625f,-1.7716999394625430053196168955764733254909515380859375f,1.011800002638996165416074290988035500049591064453125f,0.0f,0.0f,0.729600056083499648451606844901107251644134521484375f,1.6549999587796617728230330612859688699245452880859375f,-0.7448999653220178540635743047459982335567474365234375f,0.0f,0.0f,0.6635000157505281936920482621644623577594757080078125f,0.5180000402107831636300261379801668226718902587890625f,0.2592000334329902511143473020638339221477508544921875f,0.0f,0.0f,0.465700025039911480462251347489655017852783203125f,-0.9419999661408360314140963964746333658695220947265625f,0.446099990078806740712025202810764312744140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.31950004070252102650329106836579740047454833984375f,-1.99180001241788229293661061092279851436614990234375f,0.8891999621868136305380403427989222109317779541015625f,0.0f,0.0f,0.728700009781122926000307415961287915706634521484375f,1.5748999489307402566140581257059238851070404052734375f,-1.049700021620095125030047711334191262722015380859375f,0.0f,0.0f,-2.827199985745549870586046381504274904727935791015625f,0.34849995484203066808959192712791264057159423828125f,0.437000009298324609829933251603506505489349365234375f,0.0f,0.0f,-1.43520002651512523783594588167034089565277099609375f,0.596100011685489761248391005210578441619873046875f,0.256499990284442203059001030851504765450954437255859375f,0.0f,0.0f,-0.9269000194728367336693963807192631065845489501953125f,-1.910599967898428008794553534244187176227569580078125f,1.587499989829958213505278763477690517902374267578125f,0.0f,0.0f,-0.61369998850971285708766345123876817524433135986328125f,0.41870000117570105313546946490532718598842620849609375f,-0.1157000087417667799627452041022479534149169921875f,0.0f,0.0f,-0.5087999932922435686322160108829848468303680419921875f,0.857699908833204904823332981322892010211944580078125f,0.83979992989897722655001643943251110613346099853515625f,0.0f,0.0f,-1.05069997476711751716038634185679256916046142578125f,1.283300017026066885961199659504927694797515869140625f,-1.7074000306263557025232557862182147800922393798828125f,0.0f,0.0f,1.2036999206349265900684031294076703488826751708984375f,-0.1639000660240659268396257175481878221035003662109375f,2.960500032797455105537665076553821563720703125f,0.0f,0.0f,-0.53840006638169313202268995155463926494121551513671875f,2.670600059205294751762949090334586799144744873046875f,1.454400041489303685438017055275849997997283935546875f,0.0f,0.0f,0.38789999033361766134220260937581770122051239013671875f,1.73939992997273673580593822407536208629608154296875f,-1.207100093969703635821133502759039402008056640625f,0.0f,0.0f,1.153399987144768434887964758672751486301422119140625f,-2.27119991288632139259107134421356022357940673828125f,-0.47390005577951743731546230264939367771148681640625f,0.0f,0.0f,-2.85030002207756094634305554791353642940521240234375f,-1.107899984112381996936846917378716170787811279296875f,-2.935999983303250626676117462920956313610076904296875f,0.0f,0.0f,0.162399993336199255367091609514318406581878662109375f,1.910900054460764163621888656052760779857635498046875f,0.1179999793767929627819057714077644050121307373046875f,0.0f,0.0f,0.866200035224855557913770098821260035037994384765625f,-1.007299985070525760733062270446680486202239990234375f,-1.51350015142559879421924051712267100811004638671875f,0.0f,0.0f,-0.75240001278221679825719547807238996028900146484375f,2.81120002144128111609688858152367174625396728515625f,-1.191099949137866875759073082008399069309234619140625f,0.0f,0.0f,-0.33389998976737278457704860556987114250659942626953125f,-1.18979996849298430561248096637427806854248046875f,-0.257899957104028576537757544429041445255279541015625f,0.0f,0.0f,1.3631999869674444880729424767196178436279296875f,0.645700029100478190002831979654729366302490234375f,-3.354099956417084360538183318567462265491485595703125f,0.0f,0.0f,0.0147000116549433901269594571203924715518951416015625f,1.6970999942064282084430715258349664509296417236328125f,-0.13500006632879324453000435823923908174037933349609375f,0.0f,0.0f,0.243899999266861566837860664236359298229217529296875f,-1.397199963244796006023307199939154088497161865234375f,-1.00580000711679407032761446316726505756378173828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.003000002324581085844101835391484200954437255859375f,1.36310005309581772081628514570184051990509033203125f,1.045499981015920898386184489936567842960357666015625f,0.0f,0.0f,-1.2373999782115225887224596590385772287845611572265625f,-1.9439999719560157043218850958510302007198333740234375f,-0.5036000050544739981006614470970816910266876220703125f,0.0f,0.0f,2.070199982109665715057644774788059294223785400390625f,-2.969799995613097820523762493394315242767333984375f,-3.2540999587267638304410866112448275089263916015625f,0.0f,0.0f,-0.2026000167965886777921014072489924728870391845703125f,-0.7549000522330395757109045007382519543170928955078125f,-1.05080000294446929132163859321735799312591552734375f,0.0f,0.0f,-0.0103999920099984155541505970177240669727325439453125f,1.530400046846268846678640329628251492977142333984375f,-0.02729999769926061414793139192624948918819427490234375f,0.0f,0.0f,-0.696999957442283157860174469533376395702362060546875f,0.0857000212401148875329681686707772314548492431640625f,1.5412000075459477699268973083235323429107666015625f,0.0f,0.0f,1.7346999673239891226472764174104668200016021728515625f,3.32139998600482844182124608778394758701324462890625f,-0.87229993487224011250447119891759939491748809814453125f,0.0f,0.0f,0.996100010716915118536007867078296840190887451171875f,-0.6513000097393992771088733206852339208126068115234375f,-1.688400006926059848666454854537732899188995361328125f,0.0f,0.0f,-1.43719999677240917179688040050677955150604248046875f,0.909399912482500294430565190850757062435150146484375f,0.6904999887645235645550201297737658023834228515625f,0.0f,0.0f,0.9261999586701394893140104613848961889743804931640625f,1.10599995862320010786561397253535687923431396484375f,1.4441999678306285392892505115014500916004180908203125f,0.0f,0.0f,0.560899969188870084479958677547983825206756591796875f,-0.78759993111491233275245349432225339114665985107421875f,-0.863199962157012112839993278612382709980010986328125f,0.0f,0.0f,1.938000012457369525264994081226177513599395751953125f,-0.0955999548226600381894968450069427490234375f,-0.9717999770835044071048969271942041814327239990234375f,0.0f,0.0f,-0.005899952536821739812467058072797954082489013671875f,0.754299927648902635013428152888081967830657958984375f,0.297899998193979609339976377668790519237518310546875f,0.0f,0.0f,-0.14980003405809316774366379831917583942413330078125f,-2.129399928277731390835469937883317470550537109375f,0.2102000083729615997896189583116210997104644775390625f,0.0f,0.0f,-3.016100032547116338577097849338315427303314208984375f,-0.92670000102296423705183769925497472286224365234375f,-1.23109987154006983445242440211586654186248779296875f,0.0f,0.0f,0.6080999973654745982543090576655231416225433349609375f,1.7750999684423209856731773470528423786163330078125f,0.71899996182322478688320188666693866252899169921875f,0.0f,0.0f,0.25479999710321454831074561298009939491748809814453125f,-1.9567000140428536525405434076674282550811767578125f,0.70139999688267717914413879043422639369964599609375f,0.0f,0.0f,-0.702599998021125937697206609300337731838226318359375f,-0.578399982786178323834747061482630670070648193359375f,1.60659997966289491699853897443972527980804443359375f,0.0f,0.0f,-0.77759999623298625692768837325274944305419921875f,-0.7545000188350663439251775344018824398517608642578125f,-0.71709998481869785535991468350403010845184326171875f,0.0f,0.0f,0.05480003176704018041931476545869372785091400146484375f,-0.45160001214742617747788244741968810558319091796875f,-0.163799994987249508682225496158935129642486572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_1o1i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{1,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,1,4,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.688899972295761386931189917959272861480712890625f,0.0440999972462654543647886384860612452030181884765625f,-0.17429999105334292863744849455542862415313720703125f,-0.09449999454617508121145874611102044582366943359375f,0.373499982595443924537903512828052043914794921875f,-0.046199998307228096194876343361102044582366943359375f,0.1825999953389167984596497262828052043914794921875f,0.1889999890923501624229174922220408916473388671875f,-0.4454999924898146446139435283839702606201171875f,0.01319999963343143623006881171022541821002960205078125f,0.0269999986812472503316939764772541821002960205078125f,0.0682000001549720735738446819595992565155029296875f,0.139499997377395601461103069595992565155029296875f,0.1760000016689300395000827847979962825775146484375f,0.359999995827674723614109097979962825775146484375f,-0.1188000040769576770571802626363933086395263671875f,-0.8216999913930891352720209397375583648681640625f,-0.012599999324977406711667526906239800155162811279296875f,0.0497999978855252489751137545681558549404144287109375f,-0.06509999846816061364052075077779591083526611328125f,0.257299996805191000248669297434389591217041015625f,-0.16799999725818626217233031638897955417633056640625f,0.663999996542930404075377737171947956085205078125f,0.11340000096559510467386644450016319751739501953125f,0.39599998736381536446060636080801486968994140625f,-0.028599998795986181221451261080801486968994140625f,-0.05849999630451208076920011080801486968994140625f,-0.054999999701976776123046875f,-0.11249999701976776123046875f,-0.193599997901916509590591886080801486968994140625f,-0.39599998736381536446060636080801486968994140625f,0.054999999701976776123046875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_2o1i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,1,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,1,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,2,4,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.688899972295761386931189917959272861480712890625f,0.0440999972462654543647886384860612452030181884765625f,-0.17429999105334292863744849455542862415313720703125f,-0.09449999454617508121145874611102044582366943359375f,0.373499982595443924537903512828052043914794921875f,-0.046199998307228096194876343361102044582366943359375f,0.1825999953389167984596497262828052043914794921875f,0.1889999890923501624229174922220408916473388671875f,-0.4454999924898146446139435283839702606201171875f,0.01319999963343143623006881171022541821002960205078125f,0.0269999986812472503316939764772541821002960205078125f,0.0682000001549720735738446819595992565155029296875f,0.139499997377395601461103069595992565155029296875f,0.1760000016689300395000827847979962825775146484375f,0.359999995827674723614109097979962825775146484375f,-0.1188000040769576770571802626363933086395263671875f,-0.8216999913930891352720209397375583648681640625f,-0.012599999324977406711667526906239800155162811279296875f,0.0497999978855252489751137545681558549404144287109375f,-0.06509999846816061364052075077779591083526611328125f,0.257299996805191000248669297434389591217041015625f,-0.16799999725818626217233031638897955417633056640625f,0.663999996542930404075377737171947956085205078125f,0.11340000096559510467386644450016319751739501953125f,0.39599998736381536446060636080801486968994140625f,-0.028599998795986181221451261080801486968994140625f,-0.05849999630451208076920011080801486968994140625f,-0.054999999701976776123046875f,-0.11249999701976776123046875f,-0.193599997901916509590591886080801486968994140625f,-0.39599998736381536446060636080801486968994140625f,0.054999999701976776123046875f,-0.74699996519088784907580702565610408782958984375f,0.207899995511770185885325190611183643341064453125f,0.1889999890923501624229174922220408916473388671875f,-0.4454999924898146446139435283839702606201171875f,-0.40499997854232816507646930404007434844970703125f,-0.21780000091791151817233185283839702606201171875f,-0.197999993681907682230303180404007434844970703125f,0.890999984979629289227887056767940521240234375f,-0.05939999924451111468215458444319665431976318359375f,0.0185999997273087469551455797045491635799407958984375f,0.00359999983906746090500661239275359548628330230712890625f,0.096100001478195196114029386080801486968994140625f,0.0185999997273087469551455797045491635799407958984375f,0.248000005602836637308428180404007434844970703125f,0.0479999996423721153604446953977458178997039794921875f,-0.167400007939338735241108224727213382720947265625f,0.890999984979629289227887056767940521240234375f,-0.05939999924451111468215458444319665431976318359375f,-0.053999997362494500663387952954508364200592041015625f,-0.3069000053167343367022112943232059478759765625f,-0.27899999475479120292220613919198513031005859375f,-0.7920000194311143104641814716160297393798828125f,-0.71999999165534944722821819595992565155029296875f,0.5346000263929369111792766489088535308837890625f,0.052799998533725744920275246840901672840118408203125f,-0.04029999883174895103365997783839702606201171875f,-0.007799999539554125504992043715901672840118408203125f,-0.07750000059604644775390625f,-0.014999999664723873138427734375f,-0.27280000061988829429537872783839702606201171875f,-0.052799998533725744920275246840901672840118408203125f,0.07750000059604644775390625f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_2o2i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,2,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{2,2,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,2,4,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{1.237899970626830992159739253111183643341064453125f,1.0242000161290170101580088157788850367069244384765625f,0.71669999392628636059043856221251189708709716796875f,-0.83700000169873245425833374611102044582366943359375f,-0.301499999523162642844908987171947956085205078125f,0.290400008475780513350628098123706877231597900390625f,0.4885999904513358416124901850707828998565673828125f,0.20879998884052053398363568703643977642059326171875f,-0.484499990187585272138903746963478624820709228515625f,0.06590000059306622137711428877082653343677520751953125f,0.037199998560547842263446227661916054785251617431640625f,0.210800003838539140588181908242404460906982421875f,0.16709999726116653295804326262441463768482208251953125f,0.2565999993324279415674027404747903347015380859375f,0.37559999490678297462409318541176617145538330078125f,-0.3513000058650970203188990126363933086395263671875f,-0.23670002834796832758001983165740966796875f,-0.180900002716481711484419747648644261062145233154296875f,-0.1031999996706842726013064748258329927921295166015625f,-0.52050001111626631455919778090901672840118408203125f,-0.1566999997377394038267084397375583648681640625f,-0.42539999029636366145723513909615576267242431640625f,0.430000011324882080998577293939888477325439453125f,0.85590000811815247772074144450016319751739501953125f,0.40139998745769267518568312880233861505985260009765625f,0.00550000128149985556547107989899814128875732421875f,-0.051899996487796362654165704952902160584926605224609375f,-0.287500001490116119384765625f,-0.157499996013939380645751953125f,-0.0634000009655952823095503845252096652984619140625f,-0.37079998871386055103727130699553526937961578369140625f,0.343300004136562364465135033242404460906982421875f,0.816500000119209090598815237171947956085205078125f,0.98460000463724117736319385585375130176544189453125f,0.07950000512599952884329468361102044582366943359375f,-0.902999999642371875552271376363933086395263671875f,0.172499995827674723614109097979962825775146484375f,0.1803999974489212121397940791212022304534912109375f,0.2610000025629997111309421597979962825775146484375f,0.503600005948543039124842835008166730403900146484375f,-1.033699997711181595150264911353588104248046875f,-0.111500003062188657310116468579508364200592041015625f,0.095299998980760580735704934340901672840118408203125f,-0.320900010502338517426323960535228252410888671875f,0.38780000270605086143405060283839702606201171875f,-0.26259999563693980917378212325274944305419921875f,0.76900000429153436698470613919198513031005859375f,0.52770001094341267844356480054557323455810546875f,-0.6295000253915787880032439716160297393798828125f,-0.1820000013262033089489477788447402417659759521484375f,0.0054999991953372795450150078977458178997039794921875f,-0.57220001308918000404446502216160297393798828125f,0.133000003516674070169756305404007434844970703125f,-0.660800013971329036621682462282478809356689453125f,0.575000021457672261249172152020037174224853515625f,0.951600019598007662580130272544920444488525390625f,0.7968999925017357099932269193232059478759765625f,-0.0502000024497508690046743140555918216705322265625f,-0.0868999953329563368242816068232059478759765625f,0.4900000095367431640625f,-0.4074999988079071044921875f,-0.141800002825259952032865840010344982147216796875f,-0.6693999948859215010088519193232059478759765625f,-0.59980001647472391823612269945442676544189453125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_1item_5o3i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,3,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,5,4,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{2.013900005078316013396033667959272861480712890625f,0.5436000047564510939679394141421653330326080322265625f,1.42869999309182116320471322978846728801727294921875f,-0.75599999526143069061845380929298698902130126953125f,-0.4215000060796736391921513131819665431976318359375f,0.3228000090390443777010887060896493494510650634765625f,0.4405999908089637262520454896730370819568634033203125f,0.04679997596591700670387581340037286281585693359375f,-0.132499986849725193138738177367486059665679931640625f,0.00480000298917293566347552769002504646778106689453125f,0.450799995270371456346225613742717541754245758056640625f,0.109400011277198672132726642303168773651123046875f,0.85349996836483488227287352856365032494068145751953125f,0.1994000017404555791245002183131873607635498046875f,0.76279999071061599380527695757336914539337158203125f,-0.3942000059962271762969976407475769519805908203125f,0.0832999811887749075367537443526089191436767578125f,-0.434700012157857418326756260285037569701671600341796875f,0.2728000049784780056683075599721632897853851318359375f,-0.94170001240372602779871158418245613574981689453125f,0.4672999866724012463237158954143524169921875f,-0.66299999845027901557159566436894237995147705078125f,0.78200001466274215999874286353588104248046875f,0.67769999395608859771300558350048959255218505859375f,0.99979999050945045357963181231752969324588775634765625f,-0.01789999879002568405894635361619293689727783203125f,0.106500008948147206755407978562288917601108551025390625f,-0.2771000021040439520447762333787977695465087890625f,-0.2278999940589070405394522822462022304534912109375f,0.0327999967455863128407145268283784389495849609375f,-1.021999993577599408212108755833469331264495849609375f,0.244500009000301332662274944595992565155029296875f,-0.23409997131824411553679965436458587646484375f,-0.406400014764070505179915926419198513031005859375f,0.4541000477612016084094648249447345733642578125f,0.141000011354684762210354165290482342243194580078125f,-0.757500003874302052508937777020037174224853515625f,-0.458800000463426116414922262265463359653949737548828125f,0.320600004778802460947417785064317286014556884765625f,0.82799997762590660244086393504403531551361083984375f,0.15749999791383739733419133699499070644378662109375f,0.082199995207786591411291965414420701563358306884765625f,0.173400001822411997665085436892695724964141845703125f,0.00449997927248502183061873438418842852115631103515625f,0.27989998946189853512578338268212974071502685546875f,-0.37880002322494998878710248391143977642059326171875f,0.3140000078678131245624172152020037174224853515625f,0.4302000207409264476154930889606475830078125f,0.630999990642070684998543583787977695465087890625f,0.1546000034451484594910652958787977695465087890625f,-0.488799991075694606479373760521411895751953125f,0.204800002169609030033825547434389591217041015625f,-1.03989997605085449094985960982739925384521484375f,-0.43420001344680780874796255375258624553680419921875f,-0.74339998397827145026894868351519107818603515625f,-0.031199980872869570447392106871120631694793701171875f,0.50220000623464589040167993516661226749420166015625f,0.123899999155104023618179098775726743042469024658203125f,0.012900005793571533985186761128716170787811279296875f,0.146700004468858224981886451132595539093017578125f,0.155700002837181106229991200962103903293609619140625f,0.33600001470744611520302669305237941443920135498046875f,-0.70320000641345981051699709496460855007171630859375f,0.0264999869689346301271370975882746279239654541015625f,-1.024199995565414411657911841757595539093017578125f,-0.99300001405924565034410989028401672840118408203125f,-0.391799999099969653570951777510344982147216796875f,0.74700000605732197467290234271786175668239593505859375f,8.9406970715799616300500929355621337890625e-10f,-0.292800005239248317678857347345910966396331787109375f,-0.29879999599307771607215045150951482355594635009765625f,-0.117600001113116636730637765140272676944732666015625f,-0.6193999893724910776882097707130014896392822265625f,0.29430000248998400280697751441039144992828369140625f,0.312500011660158671933373852880322374403476715087890625f,0.488899992263316818252860684879124164581298828125f,0.71430002204775799867775276652537286281585693359375f,-0.1926000194787977903843056992627680301666259765625f,0.47220000355243652023773393011651933193206787109375f,-0.13049996596574775509225219138897955417633056640625f,0.40950002241134608738093447755090892314910888671875f,0.135300003674626356797716653090901672840118408203125f,-0.044100003094971196304641125607304275035858154296875f,0.40890001513659957055324412067420780658721923828125f,-0.15630000467300408928394972463138401508331298828125f,0.2873999976575374315501676392159424722194671630859375f,-0.5568000186204908885656550410203635692596435546875f,-0.76590000957101589396103236140334047377109527587890625f,0.3881000152468683239703750587068498134613037109375f,0.267399998253583959240131662227213382720947265625f,0.1464000074625015912488379399292171001434326171875f,-0.511999989032745350669983963598497211933135986328125f,-0.77900002172589299931360073969699442386627197265625f,0.737799970519542824831660254858434200286865234375f,0.0885999886631960720251299790106713771820068359375f,0.9441999937772749262876459397375583648681640625f,0.7124999895691868090352727449499070644378662109375f,0.432299994122981701138996868394315242767333984375f,-0.095500029593706070585312772891484200954437255859375f,0.355499977290630653214975609444081783294677734375f,0.6074999961256981606538829510100185871124267578125f,0.058199990580976024290293935337103903293609619140625f,-0.1734000040948391330175581970252096652984619140625f,-0.4445999439150101206763565642177127301692962646484375f,-0.2316000045478343594140824279747903347015380859375f,-0.45380000620037319603028436176828108727931976318359375f,0.191399996733665478387820257921703159809112548828125f,-0.8152999919772145887009173748083412647247314453125f,0.395999985873699333893682705820538103580474853515625f,-0.2539999834299084824351666611619293689727783203125f,0.49840000156164165900918305851519107818603515625f,0.1325999801397319544093988952226936817169189453125f,-0.60329999102354037177065038122236728668212890625f,0.486299999979138419803348369896411895751953125f,0.50099999634176495266757456192863173782825469970703125f,1.0031999766111372451860006549395620822906494140625f,1.05129998053312334604925126768648624420166015625f,1.05359998657703357594073168002068996429443359375f,0.8871999940156936048651914461515843868255615234375f,-0.269099993669986048416831181384623050689697265625f,0.638399995154142363418259265017695724964141845703125f,-0.25750000774860382080078125f,0.0024000045120715629565211202134378254413604736328125f,0.531700003314018232458693091757595539093017578125f,-0.0803999996840953912169425166212022304534912109375f,-0.053599990153312404572716332040727138519287109375f,-0.701999994099140156578187088598497211933135986328125f,-1.1681000110745429765302105806767940521240234375f,0.83300001871585838131295531638897955417633056640625f,-0.29789999618232254618988008587621152400970458984375f,0.4805999879777431704752643781830556690692901611328125f,0.2354999977797269661294876641477458178997039794921875f,0.09599999511241907867997724679298698902130126953125f,0.0145999986767768874518225175052066333591938018798828125f,-0.04199999905377627129610118572600185871124267578125f,-0.263799994614720312302580396135454066097736358642578125f,0.322400014793873257445966373779810965061187744140625f,-0.29839999854415655278216945589520037174224853515625f,0.0136000053897500361443917427095584571361541748046875f,-0.70669998483657803234336824971251189708709716796875f,-0.0286999857902525690178663353435695171356201171875f,-0.8051999829292295629556974745355546474456787109375f,-0.52220001308918018168014896218664944171905517578125f,0.53399998402595461044484181911684572696685791015625f,-0.0186999989032745617123509873636066913604736328125f,-0.017599998579919340357946566655300557613372802734375f,0.390099999399483199358229512654361315071582794189453125f,0.048300007563829350232254000729881227016448974609375f,0.720699981826543822904795888462103903293609619140625f,0.183600003266334521612179742078296840190887451171875f,0.55040000311136250132904024212621152400970458984375f,-0.2796000064998864775844822361250407993793487548828125f,-0.72729999471902839758286063442938029766082763671875f,-0.033700008440017636957009017351083457469940185546875f,0.159199997204542142981154029257595539093017578125f,0.509499983966350544761780838598497211933135986328125f,-0.2912000121653079975914124588598497211933135986328125f,0.69159999516010284281719577847979962825775146484375f,1.0755999970674512411505929776467382907867431640625f,-0.8570999855637548847653306438587605953216552734375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_2item_5o3i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,3,3,5};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,5,4,8};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{2.013900005078316013396033667959272861480712890625f,0.5436000047564510939679394141421653330326080322265625f,1.42869999309182116320471322978846728801727294921875f,-0.75599999526143069061845380929298698902130126953125f,-0.4215000060796736391921513131819665431976318359375f,0.3228000090390443777010887060896493494510650634765625f,0.4405999908089637262520454896730370819568634033203125f,0.04679997596591700670387581340037286281585693359375f,-0.132499986849725193138738177367486059665679931640625f,0.00480000298917293566347552769002504646778106689453125f,0.450799995270371456346225613742717541754245758056640625f,0.109400011277198672132726642303168773651123046875f,0.85349996836483488227287352856365032494068145751953125f,0.1994000017404555791245002183131873607635498046875f,0.76279999071061599380527695757336914539337158203125f,-0.3942000059962271762969976407475769519805908203125f,0.0832999811887749075367537443526089191436767578125f,-0.434700012157857418326756260285037569701671600341796875f,0.2728000049784780056683075599721632897853851318359375f,-0.94170001240372602779871158418245613574981689453125f,0.4672999866724012463237158954143524169921875f,-0.66299999845027901557159566436894237995147705078125f,0.78200001466274215999874286353588104248046875f,0.67769999395608859771300558350048959255218505859375f,0.99979999050945045357963181231752969324588775634765625f,-0.01789999879002568405894635361619293689727783203125f,0.106500008948147206755407978562288917601108551025390625f,-0.2771000021040439520447762333787977695465087890625f,-0.2278999940589070405394522822462022304534912109375f,0.0327999967455863128407145268283784389495849609375f,-1.021999993577599408212108755833469331264495849609375f,0.244500009000301332662274944595992565155029296875f,-0.23409997131824411553679965436458587646484375f,-0.406400014764070505179915926419198513031005859375f,0.4541000477612016084094648249447345733642578125f,0.141000011354684762210354165290482342243194580078125f,-0.757500003874302052508937777020037174224853515625f,-0.458800000463426116414922262265463359653949737548828125f,0.320600004778802460947417785064317286014556884765625f,0.82799997762590660244086393504403531551361083984375f,0.15749999791383739733419133699499070644378662109375f,0.082199995207786591411291965414420701563358306884765625f,0.173400001822411997665085436892695724964141845703125f,0.00449997927248502183061873438418842852115631103515625f,0.27989998946189853512578338268212974071502685546875f,-0.37880002322494998878710248391143977642059326171875f,0.3140000078678131245624172152020037174224853515625f,0.4302000207409264476154930889606475830078125f,0.630999990642070684998543583787977695465087890625f,0.1546000034451484594910652958787977695465087890625f,-0.488799991075694606479373760521411895751953125f,0.204800002169609030033825547434389591217041015625f,-1.03989997605085449094985960982739925384521484375f,-0.43420001344680780874796255375258624553680419921875f,-0.74339998397827145026894868351519107818603515625f,-0.031199980872869570447392106871120631694793701171875f,0.50220000623464589040167993516661226749420166015625f,0.123899999155104023618179098775726743042469024658203125f,0.012900005793571533985186761128716170787811279296875f,0.146700004468858224981886451132595539093017578125f,0.155700002837181106229991200962103903293609619140625f,0.33600001470744611520302669305237941443920135498046875f,-0.70320000641345981051699709496460855007171630859375f,0.0264999869689346301271370975882746279239654541015625f,-1.024199995565414411657911841757595539093017578125f,-0.99300001405924565034410989028401672840118408203125f,-0.391799999099969653570951777510344982147216796875f,0.74700000605732197467290234271786175668239593505859375f,8.9406970715799616300500929355621337890625e-10f,-0.292800005239248317678857347345910966396331787109375f,-0.29879999599307771607215045150951482355594635009765625f,-0.117600001113116636730637765140272676944732666015625f,-0.6193999893724910776882097707130014896392822265625f,0.29430000248998400280697751441039144992828369140625f,0.312500011660158671933373852880322374403476715087890625f,0.488899992263316818252860684879124164581298828125f,0.71430002204775799867775276652537286281585693359375f,-0.1926000194787977903843056992627680301666259765625f,0.47220000355243652023773393011651933193206787109375f,-0.13049996596574775509225219138897955417633056640625f,0.40950002241134608738093447755090892314910888671875f,0.135300003674626356797716653090901672840118408203125f,-0.044100003094971196304641125607304275035858154296875f,0.40890001513659957055324412067420780658721923828125f,-0.15630000467300408928394972463138401508331298828125f,0.2873999976575374315501676392159424722194671630859375f,-0.5568000186204908885656550410203635692596435546875f,-0.76590000957101589396103236140334047377109527587890625f,0.3881000152468683239703750587068498134613037109375f,0.267399998253583959240131662227213382720947265625f,0.1464000074625015912488379399292171001434326171875f,-0.511999989032745350669983963598497211933135986328125f,-0.77900002172589299931360073969699442386627197265625f,0.737799970519542824831660254858434200286865234375f,0.0885999886631960720251299790106713771820068359375f,0.9441999937772749262876459397375583648681640625f,0.7124999895691868090352727449499070644378662109375f,0.432299994122981701138996868394315242767333984375f,-0.095500029593706070585312772891484200954437255859375f,0.355499977290630653214975609444081783294677734375f,0.6074999961256981606538829510100185871124267578125f,0.058199990580976024290293935337103903293609619140625f,-0.1734000040948391330175581970252096652984619140625f,-0.4445999439150101206763565642177127301692962646484375f,-0.2316000045478343594140824279747903347015380859375f,-0.45380000620037319603028436176828108727931976318359375f,0.191399996733665478387820257921703159809112548828125f,-0.8152999919772145887009173748083412647247314453125f,0.395999985873699333893682705820538103580474853515625f,-0.2539999834299084824351666611619293689727783203125f,0.49840000156164165900918305851519107818603515625f,0.1325999801397319544093988952226936817169189453125f,-0.60329999102354037177065038122236728668212890625f,0.486299999979138419803348369896411895751953125f,0.50099999634176495266757456192863173782825469970703125f,1.0031999766111372451860006549395620822906494140625f,1.05129998053312334604925126768648624420166015625f,1.05359998657703357594073168002068996429443359375f,0.8871999940156936048651914461515843868255615234375f,-0.269099993669986048416831181384623050689697265625f,0.638399995154142363418259265017695724964141845703125f,-0.25750000774860382080078125f,0.0024000045120715629565211202134378254413604736328125f,0.531700003314018232458693091757595539093017578125f,-0.0803999996840953912169425166212022304534912109375f,-0.053599990153312404572716332040727138519287109375f,-0.701999994099140156578187088598497211933135986328125f,-1.1681000110745429765302105806767940521240234375f,0.83300001871585838131295531638897955417633056640625f,-0.29789999618232254618988008587621152400970458984375f,0.4805999879777431704752643781830556690692901611328125f,0.2354999977797269661294876641477458178997039794921875f,0.09599999511241907867997724679298698902130126953125f,0.0145999986767768874518225175052066333591938018798828125f,-0.04199999905377627129610118572600185871124267578125f,-0.263799994614720312302580396135454066097736358642578125f,0.322400014793873257445966373779810965061187744140625f,-0.29839999854415655278216945589520037174224853515625f,0.0136000053897500361443917427095584571361541748046875f,-0.70669998483657803234336824971251189708709716796875f,-0.0286999857902525690178663353435695171356201171875f,-0.8051999829292295629556974745355546474456787109375f,-0.52220001308918018168014896218664944171905517578125f,0.53399998402595461044484181911684572696685791015625f,-0.0186999989032745617123509873636066913604736328125f,-0.017599998579919340357946566655300557613372802734375f,0.390099999399483199358229512654361315071582794189453125f,0.048300007563829350232254000729881227016448974609375f,0.720699981826543822904795888462103903293609619140625f,0.183600003266334521612179742078296840190887451171875f,0.55040000311136250132904024212621152400970458984375f,-0.2796000064998864775844822361250407993793487548828125f,-0.72729999471902839758286063442938029766082763671875f,-0.033700008440017636957009017351083457469940185546875f,0.159199997204542142981154029257595539093017578125f,0.509499983966350544761780838598497211933135986328125f,-0.2912000121653079975914124588598497211933135986328125f,0.69159999516010284281719577847979962825775146484375f,1.0755999970674512411505929776467382907867431640625f,-0.8570999855637548847653306438587605953216552734375f,0.175099972465634134977108260500244796276092529296875f,-0.09959999346137049514027239638380706310272216796875f,0.15539999238252644175872774212621152400970458984375f,0.476700013297796321154464749270118772983551025390625f,-0.451199994730949516252849207376129925251007080078125f,0.5358000133752824911681500452687032520771026611328125f,0.01189996910691260012526981881819665431976318359375f,0.380399980014562544283762690611183643341064453125f,-1.249700002732872849975365170394070446491241455078125f,-0.244699993717670594151059049181640148162841796875f,-0.963900002135336553266142800566740334033966064453125f,-0.104799997401237465766143941436894237995147705078125f,-0.2710000011473894243607674070517532527446746826171875f,-0.057300009024143339075862968456931412220001220703125f,-0.7868000118032096867892732916516251862049102783203125f,0.2993000069975852994730303180404007434844970703125f,-1.114300028753280713544882019050419330596923828125f,1.1490000229179866408912857878021895885467529296875f,-0.54850002014637055935963871888816356658935546875f,0.7283999990284442116461605110089294612407684326171875f,0.36619996239542995652982426690869033336639404296875f,1.1307000524580483880043857425334863364696502685546875f,-0.01680002711415351512869165162555873394012451171875f,-0.92190001768469809206862919381819665431976318359375f,-1.00319996688812995699890961986966431140899658203125f,0.17050000426173195222645517787896096706390380859375f,0.8182999627113343166229242342524230480194091796875f,0.0177000033438206028080230680643580853939056396484375f,0.0360000055432319054915524247917346656322479248046875f,0.019499999575316895317200760473497211933135986328125f,-0.0862999991595745086225832665149937383830547332763671875f,-0.00559999557733537045578486868180334568023681640625f,0.31299998857825972464752339874394237995147705078125f,-0.2930999954283237673990925031830556690692901611328125f,-0.1027000008106231376814321265555918216705322265625f,0.43169999638497824623328824600321240723133087158203125f,0.51730001416504389766259919269941747188568115234375f,-0.51339999932646751545917140902020037174224853515625f,0.587099988096952074556611478328704833984375f,-0.77269998859167099425349078956060111522674560546875f,-0.524100017809868035101317218504846096038818359375f,0.221899993805587225637054871185682713985443115234375f,-0.279500012576580303402806748636066913604736328125f,-0.1622000006467103627727510684053413569927215576171875f,0.07579999206662169086712310672737658023834228515625f,-0.3111999978974460123737344474648125469684600830078125f,-0.16040000713467605919504421763122081756591796875f,-0.340500021919608375498000896186567842960357666015625f,1.215699988532065844992757774889469146728515625f,-0.022900026655197081026926753111183643341064453125f,1.25419999675750659662298858165740966796875f,-0.615999988555908206677713678800500929355621337890625f,0.760199983078241103839900461025536060333251953125f,-0.78210001685619356948109270888380706310272216796875f,1.22630001167654967275666422210633754730224609375f,-0.07190000695586196144404311780817806720733642578125f,-0.7199999934434888615442105219699442386627197265625f,-0.324500023007392612850452451311866752803325653076171875f,0.349699992227554101731357150129042565822601318359375f,0.0930000000894068978851692008902318775653839111328125f,-0.04980000000894069245305217918939888477325439453125f,0.0517000016003847095991119431346305646002292633056640625f,-0.09130000012814998289201184888952411711215972900390625f,0.31920000450611130116129743328201584517955780029296875f,-0.56249999739229672446327867874060757458209991455078125f,0.115199999655783147378684816430904902517795562744140625f,-0.2348999976575374404319518362171947956085205078125f,-0.31170001545846470758505120102199725806713104248046875f,0.4566000070065261429164138462510891258716583251953125f,-0.2159999931380152116133785966667346656322479248046875f,-0.21179999075531963370622179354541003704071044921875f,-0.9173999934107064380128804259584285318851470947265625f,-0.75170002343058595073443939327262341976165771484375f,-0.36120002036094689401579671539366245269775390625f,-0.8986000275552274985102485516108572483062744140625f,-0.75719998075366046208500847569666802883148193359375f,-0.6492000055491919940919842701987363398075103759765625f,-1.0693000178754328288732722285203635692596435546875f,-0.9105000513494021507909792489954270422458648681640625f,0.1732999856591224130397677072323858737945556640625f,0.594900017476081899303608224727213382720947265625f,-0.60119999949932090288484687334857881069183349609375f,0.330900000965595353363823960535228252410888671875f,-0.50639998831450927507802362015354447066783905029296875f,-0.29759998326897640907873210380785167217254638671875f,-0.5553000199913979972876632018596865236759185791015625f,-0.248399990683793703283299691975116729736328125f,0.6183000130400062577251674156286753714084625244140625f,0.160500002026557897494285498396493494510650634765625f,-0.1369000164926053031422270578332245349884033203125f,0.474299998950957757415380910970270633697509765625f,0.28620000374615184313142890459857881069183349609375f,0.18840001460015798340208448280463926494121551513671875f,0.16929999485313895757343516379478387534618377685546875f,0.089300000143051061485266473027877509593963623046875f,0.7840000030994414004226200631819665431976318359375f,-0.1874999998509883081254656644887290894985198974609375f,0.21509998565316212904008352779783308506011962890625f,0.14939999816417692812819950631819665431976318359375f,-0.71279999190270881825881588156335055828094482421875f,-0.607400013005733541149311349727213382720947265625f,-0.3513000168919564458747117896564304828643798828125f,-0.507099998527765105649223187356255948543548583984375f,0.841799990606308057294882019050419330596923828125f,-0.6984000027537344834627219825051724910736083984375f,0.784500029861927572483182302676141262054443359375f,-0.5091999969065188480499273282475769519805908203125f,0.727000001102685811105175162083469331264495849609375f,-0.021200002181529953304561786353588104248046875f,1.222800041896105138761186026385985314846038818359375f,-0.2344000018894671910629767808131873607635498046875f,-0.4670999998688696308590806438587605953216552734375f,-1.3839000047802922921391655108891427516937255859375f,-1.24679999477863390211496152915060520172119140625f,-1.281100004160403926789513207040727138519287109375f,-0.291000009417533789246590458787977695465087890625f,-0.586999992161989059269444624078460037708282470703125f,-0.81180003024339697503819479607045650482177734375f,-1.089000019937753638288313595694489777088165283203125f,0.9039000026941295118376729078590869903564453125f,-0.5903999788224698619387709186412394046783447265625f,-0.322299972367286358121418743394315242767333984375f,0.5121999856710435228279720831778831779956817626953125f,-0.2394000112026930526809564980794675648212432861328125f,-0.032999997854232798744078536401502788066864013671875f,-0.088099999377131421596232030424289405345916748046875f,-0.07699999909102916983982822785037569701671600341796875f,-0.66480001319646842006250153644941747188568115234375f,-0.361900002560019473651919952317257411777973175048828125f,0.11939999823868274742011408307007513940334320068359375f,0.0161000007331371097762939825770445168018341064453125f,-0.210800004136562357803796885491465218365192413330078125f,-0.283000008061528252056859855656512081623077392578125f,0.08290000199377534073619244736619293689727783203125f,-0.4002000060260295644098960110568441450595855712890625f,-0.183299996089935302023832264239899814128875732421875f,0.509300002229213788496053894050419330596923828125f,0.975099974477291908669940312393009662628173828125f,0.183599977636337285957779386080801486968994140625f,0.28859999179244066880301033961586654186248779296875f,-0.464599999123811446821719073341228067874908447265625f,0.5584000133335589310945579200051724910736083984375f,-0.461100021892786315902412752620875835418701171875f,-0.8198999915480609246287713176570832729339599609375f,-0.97450001779198647255952892010100185871124267578125f,-0.164299992245435788618124206550419330596923828125f,-0.77770001839399327536739292554557323455810546875f,-0.0053999983429908748888692571199499070644378662109375f,-0.3039999999701976651778068116982467472553253173828125f,0.051599996948242132788209346472285687923431396484375f,-0.7516000145316124037009331004810519516468048095703125f,0.217100005754828462301020408631302416324615478515625f,0.78749998956918698667095668497495353221893310546875f,-0.800299968433380026766599257825873792171478271484375f,-0.50499999791383753944273848901502788066864013671875f,-0.033600007215142113636829890310764312744140625f,0.20790000557005416936817709938623011112213134765625f,0.04110000028610227840175639357767067849636077880859375f,0.175000000037252856355252106368425302207469940185546875f,-0.03600000196695329890417269780300557613372802734375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(true) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_8item_large_5o3i_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{8,3,16,16};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,2};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{8,5,30,30};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.193899974679947462163909221999347209930419921875f,0.58139999412596221617377523216418921947479248046875f,0.3786999901115895017511547848698683083057403564453125f,-0.64529996142983403473181169829331338405609130859375f,-1.082499991655350157770953956060111522674560546875f,1.065300044131279211256924099870957434177398681640625f,0.5475999905705446479942111182026565074920654296875f,1.07730000067949305275760707445442676544189453125f,-0.1459999814033512421929117408581078052520751953125f,0.7425000236928465113805941655300557613372802734375f,-0.0516999818563466106979831238277256488800048828125f,0.1133999558523288708755671905237250030040740966796875f,1.0937999997928746775954778058803640305995941162109375f,-0.78780002431273477014883610536344349384307861328125f,0.878300022912025468713181908242404460906982421875f,-0.50280000777244548970656978781335055828094482421875f,0.7339999911785124453444950631819665431976318359375f,-0.53730004091262895116187792154960334300994873046875f,-0.265199986505507467882125638425350189208984375f,-1.3071000401020054226819411269389092922210693359375f,0.66539999804496829938216251321136951446533203125f,0.3071999995291232465888242586515843868255615234375f,0.7791000172138211610217695124447345733642578125f,-0.8088000200212004386912667541764676570892333984375f,-0.5644999628067015606802669935859739780426025390625f,0.929100019001960930609129718504846096038818359375f,0.4626000308036797292743358411826193332672119140625f,0.9663000426709658086110721342265605926513671875f,-0.1185000505447391105917631648480892181396484375f,0.291899996972083908275408248300664126873016357421875f,0.356500021800398858573544202954508364200592041015625f,0.06610000311732289191013478557579219341278076171875f,0.5536999821022152712401975804823450744152069091796875f,0.073700000952184208102835327736102044582366943359375f,-0.078999997809529265424544064444489777088165283203125f,0.20150000853836547154429581496515311300754547119140625f,0.583000009886920356194650594261474907398223876953125f,-0.003299985659122484094041283242404460906982421875f,0.078500012509524541570726796635426580905914306640625f,-0.050000005960464388721220529987476766109466552734375f,-0.41109998128712188503186553134582936763763427734375f,0.009800005075335516124823698191903531551361083984375f,0.9767999982431530003879061041516251862049102783203125f,-0.1555999951303004280589448171667754650115966796875f,0.2839999975487590422318362470832653343677520751953125f,-0.270100004540383853424145854660309851169586181640625f,-0.4030999918520450364667340181767940521240234375f,0.1694000012002885302564436642569489777088165283203125f,0.04230000056773419725431040205876342952251434326171875f,-0.120400002068281164468999122618697583675384521484375f,-0.05089999895393848561298000277020037174224853515625f,0.10990000704526892150170169770717620849609375f,-0.881700010950863255487774949870072305202484130859375f,0.315799998697638528710740501992404460906982421875f,0.29559998801499620668664647382684051990509033203125f,-0.0258999920248985660009566345252096652984619140625f,1.0074999796226620230044090931187383830547332763671875f,0.537100012981891694607838871888816356658935546875f,0.2095000094845886895456033016671426594257354736328125f,0.21210000642538062720632296986877918243408203125f,-0.95169991927146924126645899377763271331787109375f,-0.089999999999999857891452847979962825775146484375f,1.0194999720454216429743610206060111522674560546875f,-0.4506000077217817700869773034355603158473968505859375f,-0.6921999841809272080439541241503320634365081787109375f,-1.23600004633516125096548421424813568592071533203125f,-0.3565999736651772611395472267759032547473907470703125f,-0.6726000348269938200473916367627680301666259765625f,-0.7844999669790269081204314716160297393798828125f,0.7401000063657756644630580922239460051059722900390625f,0.17509999578595181191076335380785167217254638671875f,-0.635700020942091992282030332717113196849822998046875f,1.01480001483559600927719657192938029766082763671875f,0.40529997618198354558671780978329479694366455078125f,0.93079995615482413739982803235761821269989013671875f,0.5409000141531230543279207267914898693561553955078125f,-0.3384999843835834187899536118493415415287017822265625f,-0.46800000318139789445837095627211965620517730712890625f,-0.37229998607635506058244345695129595696926116943359375f,0.5151000063657760197344259722740389406681060791015625f,0.4772999862253666236711069359444081783294677734375f,-0.089999996572732499089397606439888477325439453125f,-1.699500014662742586324384319595992565155029296875f,-0.86850000330805787740473533631302416324615478515625f,-0.270399998939037100598170582088641822338104248046875f,-0.12780001149773578816848385031335055828094482421875f,1.7368999677658081282061175443232059478759765625f,-0.93180003128647825860753073357045650482177734375f,-0.269899992442130809422451420687139034271240234375f,-0.8025000043213363909444524324499070644378662109375f,-0.9537000042304395464043409447185695171356201171875f,-0.09430000375658276201562557616853155195713043212890625f,-0.06079999847412109748034936274052597582340240478515625f,-0.1134000090122224690958319115452468395233154296875f,-0.6148000136807560966900609855656512081623077392578125f,0.23329999817609792245320932124741375446319580078125f,-0.19919998975396169260676515477825887501239776611328125f,-0.15569999598264683271509056794457137584686279296875f,0.9941000054791568896206399585935287177562713623046875f,0.167299993601441432389265173696912825107574462890625f,0.5311999794200066293825557295349426567554473876953125f,-0.11280000047087668235690216533839702606201171875f,0.256999993763864065243751610978506505489349365234375f,-0.04309999527931207552455816767178475856781005859375f,-0.190699989277124615227876347489655017852783203125f,0.4496999975919724334971760981716215610504150390625f,-0.387199986788630656775467286934144794940948486328125f,-0.024700002658367026242558495141565799713134765625f,-0.51979997355937979097717516197008080780506134033203125f,-0.242200005042553012657435829169116914272308349609375f,0.3187999969989061543884645288926549255847930908203125f,0.2107000076055527415519463829696178436279296875f,-0.76759999448209992323199912789277732372283935546875f,0.1519999911189078289908138685859739780426025390625f,-0.26740002518743255510713652256526984274387359619140625f,0.1049999985098839516695079510100185871124267578125f,-0.05969999520778668244957998467725701630115509033203125f,0.266399995541572565116439363919198513031005859375f,0.876199995550513310860196725116111338138580322265625f,0.22290000340938576073313015513122081756591796875f,-0.6915000278353691243182765902020037174224853515625f,0.290700012709200483751459387349314056336879730224609375f,0.17720000545233460798044688999652862548828125f,1.26450006264448244763798356871120631694793701171875f,0.43599998021125685454535414464771747589111328125f,-0.118199999260902632869374428992159664630889892578125f,-0.2799999919533728842679920489899814128875732421875f,-0.150000032931566718019666950567625463008880615234375f,1.5310999940276150965701162931509315967559814453125f,-0.34109999574124838517263924586586654186248779296875f,0.7443999694049363569092747638933360576629638671875f,0.4719000026345252507553595933131873607635498046875f,1.02019997220039382312961606658063828945159912109375f,0.634499989777803197199546048068441450595855712890625f,0.46189997640848190485485247336328029632568359375f,-0.30839999887943303491510960157029330730438232421875f,-0.65759999146461467489643837325274944305419921875f,0.78839999470710697693220936344005167484283447265625f,0.06139999315738720042645581997931003570556640625f,0.3138000173389914326804728261777199804782867431640625f,0.77160001259446087118476498289965093135833740234375f,-0.50250002712011365701982867904007434844970703125f,-1.935200006771086833623485290445387363433837890625f,0.52890004650354427440106519497931003570556640625f,0.39179996199607813878174056299030780792236328125f,0.551700003165006336303122225217521190643310546875f,0.7700999711990359486435409053228795528411865234375f,-1.3704000025153160180479972041212022304534912109375f,0.09740004906654320393499801866710186004638671875f,-0.1785000156760219169882475398480892181396484375f,0.1190999982148408520288285217247903347015380859375f,-0.256399997627735132255111238919198513031005859375f,0.1698999941930174895077243490959517657756805419921875f,-0.2912999960303306323794458876363933086395263671875f,0.401100008630752569160904386080801486968994140625f,0.106599999034404646636176039464771747589111328125f,-1.140299986575544100020351834245957434177398681640625f,-0.098000002026557897494285498396493494510650634765625f,0.20100000427663322621896213604486547410488128662109375f,-0.0273999988123774240733609985909424722194671630859375f,0.413800001990795085493601845882949419319629669189453125f,0.306199994730949498489280813373625278472900390625f,-0.14749997366219791405939076867070980370044708251953125f,-0.34659999635219573832500827847979962825775146484375f,-0.2381999875634910335264748937333934009075164794921875f,0.06680000059008595059140134253539144992828369140625f,0.6352999842658639817472021604771725833415985107421875f,-0.1961000013291835930573370205820538103580474853515625f,0.1666000024989247041418138906010426580905914306640625f,0.012700004088878724672895259573124349117279052734375f,0.28350001273304215043680187591235153377056121826171875f,0.07820000268816951205508303246460855007171630859375f,0.033200003209710171603319395217113196849822998046875f,-0.18630000430047510828757140188827179372310638427734375f,-0.1740999943777918890219780223560519516468048095703125f,-0.016300000798702196647127493633888661861419677734375f,-0.187799995218217385950509878966840915381908416748046875f,-0.38169999645948404776163442875258624553680419921875f,-0.0119000020384787408289639643044210970401763916015625f,-0.18100000137090699325881359982304275035858154296875f,-0.04309998559355765479494948522187769412994384765625f,0.81779999868273733909518341533839702606201171875f,1.166499960184097517412737943232059478759765625f,-0.29970002605319034927333632367663085460662841796875f,0.023700025725364781692405813373625278472900390625f,0.2850000275671487059980790945701301097869873046875f,-1.68249996662139977843253291212022304534912109375f,0.2099999998509882725983288764837197959423065185546875f,0.56620000113844870526236263685859739780426025390625f,-0.2256000112608076424436376328230835497379302978515625f,0.454200006815791201830734280520118772983551025390625f,-0.14159999769330067209693879703991115093231201171875f,-0.128899966335296767283580265939235687255859375f,0.899999995231628435732318394002504646778106689453125f,0.3893999919056891911850470933131873607635498046875f,-0.1781999959826467971879537799395620822906494140625f,1.0554999844431875999362091533839702606201171875f,0.29970000317990785898558669941849075257778167724609375f,0.511400004705786503933495623641647398471832275390625f,-0.4386000294923786402279120011371560394763946533203125f,-0.04629998515248257007215215708129107952117919921875f,-0.637800017905235439030775523860938847064971923828125f,-0.72999998301267599032371435896493494510650634765625f,0.694500010676681878152294302708469331264495849609375f,0.4124999919906258138979637806187383830547332763671875f,0.3206999986648557676716109199333004653453826904296875f,0.0633999877035620329479570500552654266357421875f,0.904499985605478062922202298068441450595855712890625f,0.7918999537587172454777828534133732318878173828125f,1.122900040811301014542777920723892748355865478515625f,-0.96969999308437115903558378704474307596683502197265625f,-0.397499997019767903339015902020037174224853515625f,0.637199983175098960686000282294116914272308349609375f,-0.3184999998807906962383640347979962825775146484375f,0.07090001407861701299140122500830329954624176025390625f,-0.10309999798014757299569055248866789042949676513671875f,-0.4915999758630993365926542537636123597621917724609375f,-0.20950000415742398018892345135100185871124267578125f,-0.20490000261217333754615310681401751935482025146484375f,0.316700014638900739782911841757595539093017578125f,0.893199979405105093377414959832094609737396240234375f,0.1406000074982642900067730806767940521240234375f,0.814299974364042444818778676562942564487457275390625f,-0.117100010597706027937192629906348884105682373046875f,-0.576600027309357887617125015822239220142364501953125f,0.06840000128224488218364740532706491649150848388671875f,-0.0492000035375357303024657085188664495944976806640625f,-0.2061000025212764708015811265795491635799407958984375f,0.850799998846650140649217064492404460906982421875f,-0.12870000844001783235626135137863457202911376953125f,-0.187800013248622388317699005710892379283905029296875f,-0.5407000058174133982902276329696178436279296875f,0.390399995847046543673286578268744051456451416015625f,0.185100012922287060490589283290319144725799560546875f,0.4979000013604759633523144657374359667301177978515625f,0.3763999998629092846869070854154415428638458251953125f,-0.86400000403076404609237215481698513031005859375f,-0.10850000822544103584732511080801486968994140625f,-0.91490000395327786275601056331652216613292694091796875f,0.256899997270107149915929767303168773651123046875f,-1.19549999189376876529422588646411895751953125f,0.1653000074744230829537627869285643100738525390625f,1.001600002908705988602378056384623050689697265625f,0.2960999914944171251818261225707828998565673828125f,0.181100021147727829884388484060764312744140625f,0.291899992743879455048272575368173420429229736328125f,-0.6623999732568861542603144698659889400005340576171875f,0.224400005316734318938642900320701301097869873046875f,-0.325100001597404375530686593265272676944732666015625f,-1.3173000346541403615674425964243710041046142578125f,0.261599988603592237268458120524883270263671875f,-0.809100018256902586699652601964771747589111328125f,0.61129997546672854014104814268648624420166015625f,1.1799000653088100865062415323336608707904815673828125f,0.38819996126294054050731574534438550472259521484375f,0.013500006493181171407513829763047397136688232421875f,0.0023999976947901568991028398158960044384002685546875f,-0.17100000487267974502714196205488406121730804443359375f,1.167600016871094492643123885500244796276092529296875f,0.951900048530102349531034633400849997997283935546875f,0.8089999837279311378779311780817806720733642578125f,0.521700012552738456861334270797669887542724609375f,0.85480001230239821552459034137427806854248046875f,-1.1802000487923625104969005406019277870655059814453125f,-0.46229999099373753068675796384923160076141357421875f,-0.30569998696744427046922965018893592059612274169921875f,-1.658000002548098539278953467146493494510650634765625f,0.78240003312230133047933122725225985050201416015625f,-0.80550001394748704797166283242404460906982421875f,0.21570003681182914334613087703473865985870361328125f,-1.2111999857529995860971894217072986066341400146484375f,-0.16580001054406157123821685672737658023834228515625f,-0.7055999906599519366778849871479906141757965087890625f,-0.17440000233650199135126968030817806720733642578125f,-0.57959998366385701018543841200880706310272216796875f,-0.4376000018954275816440713242627680301666259765625f,-0.8939999822378157379176855101832188665866851806640625f,0.2395000119805335003775326185859739780426025390625f,0.9872999712869525712477525303256697952747344970703125f,-0.1145999968886375341980965458787977695465087890625f,-0.6757999705716970506585994371562264859676361083984375f,0.347499988973140716552734375f,0.056899980282783868545948280370794236660003662109375f,-0.30190000174045561465163700631819665431976318359375f,0.340300004754960472297398155205883085727691650390625f,-0.18490000011622897968521783695905469357967376708984375f,0.26970001189708703304148684765095822513103485107421875f,-0.03599999598786240984082951399614103138446807861328125f,0.9289999760538341089244340764707885682582855224609375f,0.11160000067353248454082859097979962825775146484375f,0.7321999928236007093573789461515843868255615234375f,0.357599986174702788588319890550337731838226318359375f,-0.29999999433755863975648026098497211933135986328125f,0.235600002729892565866975928656756877899169921875f,0.8878999739423394377269005417474545538425445556640625f,0.01349999451637273750748136080801486968994140625f,0.68109998642802260793160940011148341000080108642578125f,0.3283999976873399617716131615452468395233154296875f,1.1064000001609326506724073624354787170886993408203125f,-0.0916000002264976131982621154747903347015380859375f,-0.740400020694732319270769949071109294891357421875f,0.5760000080764289265289335162378847599029541015625f,-0.753600008332728776849762653000652790069580078125f,0.819299997162818538498640918987803161144256591796875f,-0.1832000138640399455880469758994877338409423828125f,1.40279998989105170181801440776325762271881103515625f,-0.2836000378370275853967541479505598545074462890625f,-1.47180002085566474789857238647527992725372314453125f,0.140300004196166838710269075818359851837158203125f,0.17009999146461485253212231327779591083526611328125f,-1.1997999562740329082544121774844825267791748046875f,-0.609899977093935063976459787227213382720947265625f,-0.336100007438659531544544734060764312744140625f,0.626400001055002153549366994411684572696685791015625f,1.17769999455213536521114292554557323455810546875f,0.221999984748661172151429354926222003996372222900390625f,0.6155000029578807652086425150628201663494110107421875f,-0.577500007394700975993373504024930298328399658203125f,0.9505999892070888801498540487955324351787567138671875f,-0.6450000070035457611083984375f,0.6470000107884406048697201185859739780426025390625f,0.13320001617371968194447617861442267894744873046875f,0.01679998241066993358572290162555873394012451171875f,-1.18919998796582149225287139415740966796875f,0.335700027275084522671022568829357624053955078125f,-0.10410000007748632100401664501987397670745849609375f,1.185299975585937914956957683898508548736572265625f,-1.26360003366470419194911301019601523876190185546875f,0.760800007414818679762902320362627506256103515625f,0.85800000426173195222645517787896096706390380859375f,-0.861700003835558892006929454510100185871124267578125f,-0.22059999349117287437138656969182193279266357421875f,-0.702600003683567120305042408290319144725799560546875f,0.0552999937653542161797304288484156131744384765625f,0.8466999902755023565958936160313896834850311279296875f,-0.033799997520446822818485088646411895751953125f,-0.7795999864578249116675578989088535308837890625f,0.02349999243021017036880948580801486968994140625f,-0.2202999916419388082289287922321818768978118896484375f,0.1920000104904175941555877216160297393798828125f,0.50650000220537183537317105219699442386627197265625f,0.223399999177456010102105210535228252410888671875f,0.60539999633133423362352232288685627281665802001953125f,0.0275000017881392722074451739899814128875732421875f,0.4451999980539083079378315233043394982814788818359375f,0.3513999982237816510632910649292171001434326171875f,0.643299982716143414762655083904974162578582763671875f,0.0937999981164931995181177626363933086395263671875f,-0.2831999887183311859217837991309352219104766845703125f,-0.03140000641942020109809163841418921947479248046875f,-0.7004000007644293201991558817098848521709442138671875f,-0.412299994719028450873565816436894237995147705078125f,0.2750000048801302110490496488637290894985198974609375f,0.284299994815885970211866151657886803150177001953125f,-0.0269000035285948957408663773094303905963897705078125f,-0.226300002065300986942020244896411895751953125f,-0.175999989897012820705413105315528810024261474609375f,0.1780000017285345847994904033839702606201171875f,-0.7309000015988946774569967601564712822437286376953125f,-0.231499993860721620109188734204508364200592041015625f,-1.615099998021126026515048579312860965728759765625f,1.013400009608269147776127283577807247638702392578125f,-0.2046000338435174370488311978988349437713623046875f,-0.29310000049471884864260573522187769412994384765625f,1.2908999905347826597790117375552654266357421875f,0.444000008463859785479144193232059478759765625f,-0.92759998669624366129937698133289813995361328125f,0.9648000207960603091805751319043338298797607421875f,0.93909997608661655021933256648480892181396484375f,-0.988500047713518625869255629368126392364501953125f,-0.1721000025510779352089230087585747241973876953125f,-0.77160002079010059361507956054992973804473876953125f,0.35539999566078250836653751321136951446533203125f,-0.84720000355243652023773393011651933193206787109375f,-0.283599956178665735251342994160950183868408203125f,-1.18500001415610345389950452954508364200592041015625f,0.002699993062019956369113060645759105682373046875f,0.62730002094507231191755636245943605899810791015625f,0.430800003242492124400087050162255764007568359375f,0.22410000723004319667097661294974386692047119140625f,-1.076800012958049279632177785970270633697509765625f,0.7208999838292597939215511360089294612407684326171875f,1.05579998103976269163695178576745092868804931640625f,-0.9044999862760303432906994203221984207630157470703125f,-0.9786999604582791167928235154249705374240875244140625f,0.41940001053214093218457492184825241565704345703125f,-0.018399977719784299523553272592835128307342529296875f,-0.1205999805271620317626002361066639423370361328125f,-1.372699982929230344552706810645759105682373046875f,0.1118999878823758731272164368419907987117767333984375f,0.83779998337179428791188229297404177486896514892578125f,-0.04739999676346773327395567321218550205230712890625f,-0.0389999940842391534800981389707885682582855224609375f,-0.2901999963589012576647974128718487918376922607421875f,-0.38119998642355212847832035549799911677837371826171875f,-0.0169000006973742955551642808131873607635498046875f,0.000399984484911186655153869651257991790771484375f,-0.06930000222921373076445661354227922856807708740234375f,-0.124300000515580155724393307536956854164600372314453125f,-0.22870001380443572003287044935859739780426025390625f,-0.5357999837964773082177316609886474907398223876953125f,-0.07829999348223204425067933698301203548908233642578125f,0.8022000033661722540045957430265843868255615234375f,-0.3362000127613544719906712998636066913604736328125f,-0.5381000010162588775841641108854673802852630615234375f,0.0916999925851822439426541677676141262054443359375f,0.71380001309216023486214908189140260219573974609375f,0.001700002121925336950880591757595539093017578125f,0.370599990734457929164591405424289405345916748046875f,0.0561999929428100841732884873636066913604736328125f,-0.62949999253451827119221206885413266718387603759765625f,0.43790000326037414879465359263122081756591796875f,0.20159999303668740822104155085980892181396484375f,0.195000005364418083075861432007513940334320068359375f,0.6844999953657389113459430518560111522674560546875f,-0.3743000025272369413187334430404007434844970703125f,0.00260000070333499255781362080597318708896636962890625f,0.2634000078946352463304947377764619886875152587890625f,0.1252000014811753825227924608043394982814788818359375f,0.243300002646446245080369408242404460906982421875f,0.6833999982833862674169722595252096652984619140625f,0.607199992525577325608310275129042565822601318359375f,0.71179997183084520173679266008548438549041748046875f,0.51329999046474694690544993136427365243434906005859375f,-0.35759999414682386298380833977716974914073944091796875f,0.52980000067949273301337598240934312343597412109375f,0.71559993984699321600828625378198921680450439453125f,0.292200009103119440201368206544429995119571685791015625f,0.0854999961033462430037843660102225840091705322265625f,0.935400031423568645294608359108678996562957763671875f,-0.0001999826312069075129329576157033443450927734375f,-0.6678000332906843805602647989871911704540252685546875f,0.5322000167772176215663648690679110586643218994140625f,0.78870002915263182075022996286861598491668701171875f,-0.33150000131130230585085882921703159809112548828125f,-1.19340000990629224730810165056027472019195556640625f,-0.171399929380417148649939917959272861480712890625f,0.1758000017523766445037836092524230480194091796875f,0.9605999779939651261884137056767940521240234375f,0.726900022155046343641515704803168773651123046875f,-0.1425000131130218505859375f,-0.7653000167131427389222153578884899616241455078125f,-0.16840001795291836828027953743003308773040771484375f,-1.1115000296235086896246002652333118021488189697265625f,0.00430001131892243648735529859550297260284423828125f,0.32279999763965605552584747783839702606201171875f,0.0222000029563904632823323481716215610504150390625f,-0.95640002869665641238583475569612346589565277099609375f,-0.7811999892920251653549712500534951686859130859375f,-0.23400000309944157805830400320701301097869873046875f,0.5964999934509396251058888083207421004772186279296875f,0.33159999948143958903301609097979962825775146484375f,-0.5136999939858915009693873798823915421962738037109375f,0.223899998819828027762923738919198513031005859375f,-0.13270000375360246902545213742996565997600555419921875f,-0.064000008389353713056380001944489777088165283203125f,-1.20819998883008938861394199193455278873443603515625f,-0.38290000355839737267160671763122081756591796875f,-0.10629997934103041501430197968147695064544677734375f,0.08290000564455990428314180462621152400970458984375f,-0.020199998016655396071428185678087174892425537109375f,0.0236999980092047479729444603435695171356201171875f,-0.7485999846026303305990268199821002781391143798828125f,0.0770000009536744300930877216160297393798828125f,0.06069999229460965040061637409962713718414306640625f,-0.208999993935227390551290227449499070644378662109375f,-0.09529998147189633339593228811281733214855194091796875f,-0.0801999963775277269206753771868534386157989501953125f,-0.335999982446432365890132132335565984249114990234375f,-0.103700009036064244583030813373625278472900390625f,-0.7151999995440245783129284973256289958953857421875f,-0.09030001075267790611178497783839702606201171875f,-0.006100017310678662596501453663222491741180419921875f,-0.2682999977290629889381534667336381971836090087890625f,0.872700003641843746748918420053087174892425537109375f,0.0407000003039836855123212444595992565155029296875f,0.54199999864399417415228299432783387601375579833984375f,-0.1369999980852008025777877264772541821002960205078125f,0.470399991749227142090461484258412383496761322021484375f,-0.108000004559755335975523848901502788066864013671875f,1.0078999828457835263861852581612765789031982421875f,0.16890001491308215264552927692420780658721923828125f,-0.3759000045418741109415350365452468395233154296875f,-0.35399999490380285038781948969699442386627197265625f,-0.4892999971628189115335771930404007434844970703125f,0.5622000153988597759280310128815472126007080078125f,-1.62540000087618796698052392457611858844757080078125f,0.7125000186264518475809381925500929355621337890625f,0.34230004627704513353592119528912007808685302734375f,-0.595500024676323125305543726426549255847930908203125f,-0.76899999445676758824674834613688290119171142578125f,-0.16649997866153665171395914512686431407928466796875f,-1.6477999380350123459493261179886758327484130859375f,0.526799989861249429168310598470270633697509765625f,0.9356999354839334870348466210998594760894775390625f,-0.0957000204950571031758954632095992565155029296875f,-0.55669996531605736578285359428264200687408447265625f,0.114599994467198929015694375266321003437042236328125f,-0.581599972137809118066797964274883270263671875f,1.3968000514030460834646873991005122661590576171875f,0.436799982857703383842817856930196285247802734375f,0.876000028938054953187020146287977695465087890625f,1.0984999486207964736195208388380706310272216796875f,-0.05430002122819443943768646931857801973819732666015625f,1.300100006589293588632472165045328438282012939453125f,0.0444000023365020979326800443232059478759765625f,1.15399999773502326405605344916693866252899169921875f,-0.252900009407102999148264643736183643341064453125f,0.399600000241398856104524384136311709880828857421875f,0.4707000141620636401995625419658608734607696533203125f,-0.783999978922307771966870859614573419094085693359375f,0.10600000315904622993912198580801486968994140625f,0.7433000005602836690599133362411521375179290771484375f,-0.15909998979568484855917631648480892181396484375f,0.8037999914482234498080970297451131045818328857421875f,-0.0150999979093671132801546264090575277805328369140625f,0.61079998565912252406207016974803991615772247314453125f,-0.298800001990795038864234811626374721527099609375f,-0.719599986793100665494193890481255948543548583984375f,0.28020001302957542321792061557061970233917236328125f,0.37150000157207241358037208556197583675384521484375f,-0.0612000030755996959896947373636066913604736328125f,-0.57169999549090866342027084101573564112186431884765625f,0.114600006127357545437916996888816356658935546875f,1.1219999975636600897388461817172355949878692626953125f,0.176200002889335127775893852231092751026153564453125f,-0.1790999889761211161243181777535937726497650146484375f,0.06249999523162852455016036401502788066864013671875f,-0.26709998604804285893266069251694716513156890869140625f,-0.0305000009834766050431653638952411711215972900390625f,-0.3484999947398901287698436135542578995227813720703125f,0.18209999875128268786994567562942393124103546142578125f,-0.44720000101923940150072667165659368038177490234375f,-0.0999000144958497315883505507372319698333740234375f,-0.56420001340955483470906983711756765842437744140625f,-0.1115000032857060840996155093307606875896453857421875f,-0.6693000111326572465486606233753263950347900390625f,-0.002800000582635409074327981215901672840118408203125f,0.20679999503940338367868889690726064145565032958984375f,0.0891999994210899149749138814513571560382843017578125f,-0.2420000075101853553860564716160297393798828125f,-0.9813000268757345878611886291764676570892333984375f,0.1959000215291979429821367375552654266357421875f,-0.0405000193119047224854512023739516735076904296875f,1.3117999858379363331550848670303821563720703125f,-0.3780000054165719713950011282577179372310638427734375f,0.637199993494153016371228659409098327159881591796875f,0.7484999888539309864654569537378847599029541015625f,-0.6944000184297554056911394582130014896392822265625f,-1.146600045675039591941413164022378623485565185546875f,-0.5502999955534928488987134187482297420501708984375f,0.6663000182330609533210008521564304828643798828125f,-0.24990000734329242959574912674725055694580078125f,-1.06530003429651287660817615687847137451171875f,0.82000001490116147806475055404007434844970703125f,-0.6975000057369469264045847012312151491641998291015625f,-1.1096999552726749538322792432154528796672821044921875f,0.528600001627206328436159310513176023960113525390625f,0.2890999865174297411840598215349018573760986328125f,0.52350000323355183606821583452983759343624114990234375f,0.018099982985854357053767671459354460239410400390625f,-0.017099991337954978387614346502232365310192108154296875f,-0.6772000054523348300250518150278367102146148681640625f,1.29990006456375173371498021879233419895172119140625f,0.59819997422695081468191347084939479827880859375f,0.3108000135615469616823247633874416351318359375f,-0.988700019839405808141918896581046283245086669921875f,0.3621000078409910560850448746350593864917755126953125f,0.8975999801397323007989825782715342938899993896484375f,0.10950000128895032158737876670784316956996917724609375f,0.915099982933699873655086776125244796276092529296875f,0.24029999525547030003735926584340631961822509765625f,-0.35669999966323373552512521200696937739849090576171875f,0.21080001039505003035401387023739516735076904296875f,0.94599998728930945190995771554298698902130126953125f,-0.142300009620189626957653672434389591217041015625f,0.3557000048115852575136841551284305751323699951171875f,0.111399997851252485769890654410119168460369110107421875f,-0.769100007881969194301063907914794981479644775390625f,0.21609999223947529145561929908581078052520751953125f,-0.7732999741107227098524390385136939585208892822265625f,-0.3402999940633772979481364018283784389495849609375f,-0.0364999967291950166980996073107235133647918701171875f,-0.2130999979615211259442730806767940521240234375f,0.567499997578561288236187465372495353221893310546875f,-0.2449999943748115693864519926137290894985198974609375f,0.152100034588575017124867372331209480762481689453125f,0.21730000418126582673039592918939888477325439453125f,0.32949999849498257109559062882908619940280914306640625f,-0.27329999735653398573731465148739516735076904296875f,-0.49559998324662457580558339031995274126529693603515625f,0.3215000007152557515155422152020037174224853515625f,0.0156999956101178117506833586958236992359161376953125f,0.17059999192655095345827476194244809448719024658203125f,-0.585199979725480101677703714813105762004852294921875f,0.0166000011764466659514027924160473048686981201171875f,0.0856000009045004006669188356681843288242816925048828125f,-0.0344999988675116497915951185859739780426025390625f,0.93200000300258400987019058447913266718387603759765625f,0.340699995535612032426797668449580669403076171875f,0.836500003695487492905158433131873607635498046875f,0.036300013464689140363361730123870074748992919921875f,-0.3617000071883200718048101407475769519805908203125f,-1.259400028860568898636529411305673420429229736328125f,0.3220000113844871947321735206060111522674560546875f,-0.844800006192922836589787038974463939666748046875f,-0.6480999481916427384931012056767940521240234375f,0.45090002609267865185671553263091482222080230712890625f,-0.73090001975968499348113027735962532460689544677734375f,-0.097499986439943508997885146527551114559173583984375f,-1.3786999533057215927556171664036810398101806640625f,0.54839997190833056350811602897010743618011474609375f,0.2984999659061440269169906969182193279266357421875f,0.268799989622831247970680124126374721527099609375f,1.222499985098838948260890902020037174224853515625f,-0.0831000330910089957114905701018869876861572265625f,-0.103299949374795030365703496499918401241302490234375f,-0.489900007343292198669360004714690148830413818359375f,0.206500006675720104709625957184471189975738525390625f,0.583499991685151986331447915290482342243194580078125f,-0.461599993371963446264771846472285687923431396484375f,-0.11969999476075177113898462266661226749420166015625f,0.2338999836206436810925879399292171001434326171875f,0.162900008811056207225220759937656112015247344970703125f,-0.7071999825045465204453876140178181231021881103515625f,0.02610000054687267034836395396268926560878753662109375f,0.22040000147223459681100621310179121792316436767578125f,-0.64050002944469497379031963646411895751953125f,0.87080001099109693996069836430251598358154296875f,-0.53369997565746274403863935731351375579833984375f,-0.00230000153630971482243694481439888477325439453125f,-0.046799996268749310956991394050419330596923828125f,0.90159998152554032824212981722666881978511810302734375f,0.3493999927997588628159064683131873607635498046875f,-0.054100003877282087927369502722285687923431396484375f,-0.145399999646842470468754981993697583675384521484375f,-0.452299992670118722326577653802814893424510955810546875f,0.0830000196099280884709514793939888477325439453125f,0.128700004677474222347655086196027696132659912109375f,-0.484500003635883302877118694595992565155029296875f,0.2877000387713308793280475583742372691631317138671875f,0.24500001132488247179708196199499070644378662109375f,0.6375999787986279532248090617940761148929595947265625f,-0.15670000793337823807860331726260483264923095703125f,0.2647999887585641420884030594606883823871612548828125f,0.6333000068187715214662603102624416351318359375f,-0.29279997741133012567615878651849925518035888671875f,-0.3488999931573868451550879399292171001434326171875f,-0.09429998993575594568028463982045650482177734375f,-0.4268999933958053105698127183131873607635498046875f,-0.7461999856412411258332895158673636615276336669921875f,-0.27419999169111264336606836877763271331787109375f,0.327499986626207828521728515625f,0.085600003921985745591882732696831226348876953125f,-0.719399983114004104578498299815692007541656494140625f,-0.2873000036597250783643175964243710041046142578125f,-0.661699989940225830054032485350035130977630615234375f,0.17500000357627865099630071199499070644378662109375f,0.5692999971255658930857634913991205394268035888671875f,0.10429999835491177151425290503539144992828369140625f,0.86910000648498453301726840436458587646484375f,-0.18419999884366955456016512471251189708709716796875f,1.430399988508224140559832449071109294891357421875f,-0.81209997915625553588370166835375130176544189453125f,-0.8282999639034276384563781903125345706939697265625f,0.0539999934881925103269395549432374536991119384765625f,-0.9648999891638754267120248186984099447727203369140625f,-0.92820004172921155571884810342453420162200927734375f,-0.942699976372718850825549452565610408782958984375f,0.65579999683499323737123631872236728668212890625f,0.907100063347815677161634084768593311309814453125f,-1.257600026929378334017428642255254089832305908203125f,-0.2312000051617619789112723083235323429107666015625f,0.631800022792816218242251125047914683818817138671875f,1.20639997557401645877916962490417063236236572265625f,-0.6801000287175185121668619103729724884033203125f,-0.77959999480247432757096248678863048553466796875f,0.48749999389052423026669202954508364200592041015625f,0.0927000145196910096956344204954802989959716796875f,1.19639998332262020852567729889415204524993896484375f,-0.27220003156661931598137016408145427703857421875f,0.003599995964766122824585181660950183868408203125f,0.32169999914169267185570788569748401641845703125f,-0.1794000276684766248536107013933360576629638671875f,-1.5381999998569479970456086448393762111663818359375f,0.71759998952746339995201196870766580104827880859375f,-0.612300024223326744277073885314166545867919921875f,-0.874800008279084995166385851916857063770294189453125f,0.087100029075145357637666165828704833984375f,0.176400014615059053113554909941740334033966064453125f,0.33549997688084876035219394907471723854541778564453125f,-0.0051000044286251267067200387828052043914794921875f,0.2538999933063983416303699414129368960857391357421875f,-0.26519999306201924582637730054557323455810546875f,0.1686000201046464752607789705507457256317138671875f,-0.12249999582767490124979303800500929355621337890625f,-0.51439997849464447199352434836328029632568359375f,-0.10120000128746031720083919935859739780426025390625f,0.73309999527931213325615544817992486059665679931640625f,-0.0806000107765196815989838796667754650115966796875f,-1.1443999885156748153036687654093839228153228759765625f,-0.16489999899864216104106162674725055694580078125f,1.181999971337616361921618590713478624820709228515625f,-0.3606999986648560252433526329696178436279296875f,0.059599998639523921184490973246283829212188720703125f,-0.0032999987721442636257052072323858737945556640625f,-0.5745999944671986714439526622300036251544952392578125f,0.26350000941753393135513761080801486968994140625f,0.715900004355609187456366271362639963626861572265625f,0.085800004285573994167179989744909107685089111328125f,0.14060000295341013032413002292742021381855010986328125f,-0.020099992060661264758891775272786617279052734375f,0.1186999962955714860157740986323915421962738037109375f,-0.1409999939203263608078486868180334568023681640625f,0.7837999702885747321801090947701595723628997802734375f,0.176399991965293878593001863919198513031005859375f,-0.9117999856144189951834277962916530668735504150390625f,-0.05309999963790179577927119680680334568023681640625f,0.05759999761134389217875906297194887883961200714111328125f,-0.27050001305341719870511951739899814128875732421875f,0.804899950718880319300296832807362079620361328125f,0.54990002030730256166179970023222267627716064453125f,1.24609997882842993277563437004573643207550048828125f,0.374999966919421723332561668939888477325439453125f,0.54019999842643784404572215862572193145751953125f,0.0233999942600728871866522240452468395233154296875f,-1.10079995645284700600541327730752527713775634765625f,-0.037500002831220680121759869507513940334320068359375f,1.2460999981999396624132714350707828998565673828125f,0.83970001457929566157645240309648215770721435546875f,-1.13680001921653683893964625895023345947265625f,0.149700027692318826666451059281826019287109375f,2.29839999232292058195525896735489368438720703125f,0.775500008881092384172006859444081783294677734375f,0.7751999912738796183475642465054988861083984375f,-0.23250001549720789029152001603506505489349365234375f,-1.4345999977827066373947673127986490726470947265625f,-0.980400033360719902475466369651257991790771484375f,0.3253000155210497013058557058684527873992919921875f,-0.5340000193417073770518754827207885682582855224609375f,-0.370199989932775253009822336025536060333251953125f,-0.627900034254789574106325744651257991790771484375f,-0.6643000061035149172994351829402148723602294921875f,0.22170000867843686620517473784275352954864501953125f,1.65739996204376183186468551866710186004638671875f,-0.026399978554248804130111238919198513031005859375f,-1.592999966859817817521616234444081783294677734375f,0.00569999974519019847463141559273935854434967041015625f,0.057999999195337270663230810896493494510650634765625f,1.125300031912326748084751670830883085727691650390625f,0.2785999939531087488120419948245398700237274169921875f,-0.0398999937832356099676189842284657061100006103515625f,0.88069998391270643001149664996773935854434967041015625f,0.0912999926403166295330038337851874530315399169921875f,-0.409099975937604998232899333743262104690074920654296875f,0.4118000022292136463875067420303821563720703125f,0.7554999997541305134518552222289144992828369140625f,0.104700000572204743320980924181640148162841796875f,0.95310001260191190564086127778864465653896331787109375f,-0.152999991893768338968584430404007434844970703125f,0.20790001059919582804269566622679121792316436767578125f,0.066399992561340326346908113919198513031005859375f,-0.37899999319016952625815974897705018520355224609375f,0.262300005485117448955634245066903531551361083984375f,0.3069999953657387425920433088322170078754425048828125f,0.0133999990656972212121900156489573419094085693359375f,0.428499994367361136671235044559580273926258087158203125f,0.0141999919891357961887479177676141262054443359375f,0.3892000060334803723804952824139036238193511962890625f,0.13259999265670785462134517729282379150390625f,-0.0665999992206691704410559395910240709781646728515625f,-0.194199999737739492644550409750081598758697509765625f,0.85220000914037219441610204739845357835292816162109375f,-0.1249999943375588173921642010100185871124267578125f,1.1430999930441376744028048051404766738414764404296875f,-0.4282000144600868196675946819595992565155029296875f,-0.215999999694525879334605633630417287349700927734375f,-0.1594000100851058476791877183131873607635498046875f,0.8607999976173044132110589998774230480194091796875f,0.239300000441074356416493174037896096706390380859375f,0.8005999970674511079238300226279534399509429931640625f,-0.74550001446902725010801304961205460131168365478515625f,0.620100007185339752169284111005254089832305908203125f,0.27869999752491647626584381214343011379241943359375f,-0.298099990436434314489133612369187176227569580078125f,-1.133399996048211733068455941975116729736328125f,0.360100048685072948728702613152563571929931640625f,-0.68220003320574829075439993175677955150604248046875f,1.0137000161886220439555472694337368011474609375f,-0.63840002555251107452249925700016319751739501953125f,-0.659499955654144542904759873636066913604736328125f,0.9948000328660011604142709984444081783294677734375f,0.723799984407424545906906132586300373077392578125f,-0.4254000103384256714633693263749592006206512451171875f,0.3041999907970429628534247967763803899288177490234375f,-0.224700005824864046388711358304135501384735107421875f,0.530699994269013597403272797237150371074676513671875f,-0.95130000795125990720180197968147695064544677734375f,-0.551199951219558670345577411353588104248046875f,0.2643000072956080970243419869802892208099365234375f,0.39019997756481217265900340862572193145751953125f,-0.1896000211775306087957915224251337349414825439453125f,1.15820001758933077695701285847462713718414306640625f,0.00689999771714244758413769886828958988189697265625f,2.071300003480910589814811828546226024627685546875f,0.89910002645254127884300032746978104114532470703125f,0.3371999969959258436347226961515843868255615234375f,0.21390000954866383864327872288413345813751220703125f,1.77399999952316278495345613919198513031005859375f,-0.6657000006020068472167849904508329927921295166015625f,0.70799998693913235303654118979466147720813751220703125f,-0.0643000001430511503031084430404007434844970703125f,-0.62339999485611929941342168604023754596710205078125f,-0.1208000095009804653045648592524230480194091796875f,-0.3895000095590948063772884779609739780426025390625f,-0.140399997895956030191655372618697583675384521484375f,-0.1036999923467636641749578529925202019512653350830078125f,-0.362400000786781362194233224727213382720947265625f,-0.030399991935491588179729660623706877231597900390625f,-0.16260000196993351639918046203092671930789947509765625f,-0.57229999997168778946843303856439888477325439453125f,-0.03860000326633450384861134807579219341278076171875f,0.8035999923139811595973469593445770442485809326171875f,0.001400006121397012748275301419198513031005859375f,-0.54199998966604479022635132423602044582366943359375f,0.1745000104904175231013141456060111522674560546875f,1.049900005741417174931484623812139034271240234375f,-0.146599997879564736269486502351355738937854766845703125f,-0.0242999989323318121847705697291530668735504150390625f,-0.34989999512434000195071348571218550205230712890625f,0.1219000008463859696661302223219536244869232177734375f,-0.1564000010550021801947195854154415428638458251953125f,-0.6683999948933720958166304626502096652984619140625f,0.2301999984264373466658071265555918216705322265625f,0.8827999945104121337635660893283784389495849609375f,0.0360000009238718998716421992867253720760345458984375f,-0.45330001657903185030562553947675041854381561279296875f,-0.18950000661611543506523958058096468448638916015625f,-0.7078999654859305934451185748912394046783447265625f,0.20390000010505315852782359797856770455837249755859375f,1.01079999728202807318666600622236728668212890625f,1.12680001593828205130876085604541003704071044921875f,0.28499996662139892578125f,0.94740004325509108440428462927229702472686767578125f,0.422699981141089864422610844485461711883544921875f,0.15089999209940430091592133976519107818603515625f,-0.152299993079900719550323628936894237995147705078125f,0.3036000026702883047846626141108572483062744140625f,-0.054399973726273032070821500383317470550537109375f,1.136100016789138411610338152968324720859527587890625f,0.2834999625906349240977988301892764866352081298828125f,-0.289800004255771792571749756461940705776214599609375f,1.1043999990582467063404692453332245349884033203125f,-0.188700013804435684505733661353588104248046875f,-1.30559998335838312044643316767178475856781005859375f,-0.8454000336587430552981459186412394046783447265625f,1.054500018835067720601728069595992565155029296875f,0.473099994068592710760867703356780111789703369140625f,0.44029997854754354680295591606409288942813873291015625f,0.834299982708692322574961508507840335369110107421875f,0.9892999593138700475947189261205494403839111328125f,1.26510001076161859856483715702779591083526611328125f,0.283999956533313113737904131994582712650299072265625f,-0.99239999899864184129683053470216691493988037109375f,0.592400035357474763486607116647064685821533203125f,0.43530002155602021129965351065038703382015228271484375f,-0.229900031557679351834622138994745910167694091796875f,0.879899997359513630357241709134541451930999755859375f,-0.3640999865174290306413240614347159862518310546875f,0.038100011949986356540875931386835873126983642578125f,0.5020000142529605913210843937122263014316558837890625f,-0.08029999048709868247897247783839702606201171875f,0.87339997343570008325741582666523754596710205078125f,0.21720000429749486414721104665659368038177490234375f,0.9481999956846234312024535029195249080657958984375f,0.02710000299811365920277239638380706310272216796875f,1.10420000327676515183839001110754907131195068359375f,-0.06180000142827635212228187810978852212429046630859375f,-0.51030000844299772122525382656021974980831146240234375f,0.411500013530254449278800166212022304534912109375f,0.8007000124111767203061162945232354104518890380859375f,0.112100001060962739529713871888816356658935546875f,0.59559999878108504933749145493493415415287017822265625f,-0.03870000093355774328784946192172355949878692626953125f,-0.1192999987274407924786601142841391265392303466796875f,-0.15550000072270631079618397052399814128875732421875f,-0.35239999542236322138677451221155934035778045654296875f,-0.00709999599456789809437395888380706310272216796875f,0.2027000006616115346247397610568441450595855712890625f,-0.29619999219775206000804246286861598491668701171875f,-0.038400000779330756284224435148644261062145233154296875f,-0.0082999914705752519950010537286289036273956298828125f,0.9108000031679868158107638009823858737945556640625f,0.04799999994039527706490844138897955417633056640625f,-0.4293000034213065863042402270366437733173370361328125f,-0.27330000853240488112305683898739516735076904296875f,-0.40960000404119456529627996133058331906795501708984375f,0.03639999671503890699142402809229679405689239501953125f,-0.9511000206261870371093891662894748151302337646484375f,0.2830999925971031672133904066868126392364501953125f,-0.7520000128746033851712127216160297393798828125f,-0.234200001190602746792279731380403973162174224853515625f,0.584299997423589179135205995407886803150177001953125f,0.198199993598461077226602355949580669403076171875f,-0.5818999963760376203936175443232059478759765625f,-0.916100016677379613838638761080801486968994140625f,1.111500020086765516680316068232059478759765625f,0.157299973261356473841487968456931412220001220703125f,1.130899997985363114594292710535228252410888671875f,0.413599984192848157960042954073287546634674072265625f,0.757900040662288887460817932151257991790771484375f,-0.320399978077411606847135772113688290119171142578125f,0.031799984981120132943033240735530853271484375f,0.038000000983476667215654742904007434844970703125f,-0.9090999826431271912952070124447345733642578125f,-0.519700002211332279244970777654089033603668212890625f,-0.566699999439716606275396770797669887542724609375f,0.8247000312685965894843320711515843868255615234375f,-0.58170001195669129856469226069748401641845703125f,-0.101099974358081823311295011080801486968994140625f,-1.423700013208389236751827411353588104248046875f,-0.214800011259317358280895859934389591217041015625f,0.2585000213384631706503569148480892181396484375f,0.793599999839067482554355592583306133747100830078125f,-0.835100004875659607250781846232712268829345703125f,0.114899971878528628366211705724708735942840576171875f,0.920300042045116839517504558898508548736572265625f,-0.759900005406141332287006662227213382720947265625f,1.063000000238417896980536170303821563720703125f,0.38880001100599759222831153238075785338878631591796875f,-0.16249997943639726827314007095992565155029296875f,-0.385700015912949945029453147071762941777706146240234375f,0.378499996304511920897084564785473048686981201171875f,0.21040000698566441172943086712621152400970458984375f,-0.23200000192224978778909871834912337362766265869140625f,0.1523999979928134518303295408259145915508270263671875f,-0.0371999971821903585578183992765843868255615234375f,0.483099989877640911828393655014224350452423095703125f,-0.19510000010728845154517330229282379150390625f,-0.2288999938279392409867796231992542743682861328125f,-0.0266999933302400904722162522375583648681640625f,-0.05580001870244755135530567713431082665920257568359375f,0.4914000111877918630653994114254601299762725830078125f,-0.0869999997988342155252894372097216546535491943359375f,0.371399994647502840194874806911684572696685791015625f,0.451700011137127876992281017010100185871124267578125f,-0.08809999460875983512408993192366324365139007568359375f,-0.07210000277459627238130224213819019496440887451171875f,-0.151100000323355188047713681953609921038150787353515625f,-0.0779000002056360330016104853712022304534912109375f,0.160700001794099822660655263462103903293609619140625f,0.273700010190904396978339718771167099475860595703125f,-0.6985000246167185622425677138380706310272216796875f,-0.2529999982640145361756367492489516735076904296875f,-0.1178000052392482732699363623396493494510650634765625f,-0.3763000278815626931105953190126456320285797119140625f,0.707499999701976634014499722979962825775146484375f,-0.6931000265344982036452847751206718385219573974609375f,-0.21410000439882281852987944148480892181396484375f,-0.3955000165551896262883246890851296484470367431640625f,-1.1959999778270713477468234486877918243408203125f,-0.773299992662668245202439720742404460906982421875f,-0.148599992090463928207100252620875835418701171875f,0.682700001741945783351184218190610408782958984375f,-0.442500003166496735929058559122495353221893310546875f,0.811300015550851849610580757143907248973846435546875f,-1.3023000201627610294963233172893524169921875f,1.0940999865174294569669655174948275089263916015625f,-0.6884000137805941221813554875552654266357421875f,-0.6129999984502791932072796043939888477325439453125f,0.79049997952580497440067119896411895751953125f,-0.2044000025600194447861213120631873607635498046875f,-0.7497999854058026159009386901743710041046142578125f,-0.6321999695777893446546613631653599441051483154296875f,0.3695999848186968250729478313587605953216552734375f,0.381499985478818415263191354824812151491641998291015625f,0.576400016887486099648185700061731040477752685546875f,0.26819999644830827545405327327898703515529632568359375f,-0.4683000053472816670563361185486428439617156982421875f,-0.4853999987900257107042989446199499070644378662109375f,0.5195000018477440306696735206060111522674560546875f,1.00530000151395793039910131483338773250579833984375f,0.01259998133182449464584351517260074615478515625f,0.20480001371800893306129864868125878274440765380859375f,-0.89300000239908694510404529864899814128875732421875f,-1.03689999369382856997390263131819665431976318359375f,-0.24199998486042062495471327565610408782958984375f,-0.22550000023841842988758799037896096706390380859375f,-0.93540005407333381981516140513122081756591796875f,-0.45789997763037693090382163063623011112213134765625f,-0.40790001324415214867258328013122081756591796875f,0.03680000044107434309381687853601761162281036376953125f,0.06760000498741869368046764066093601286411285400390625f,-0.3702000094905495419794760891818441450595855712890625f,0.020999998688697729676277958787977695465087890625f,-0.36660000186562557988878552350797690451145172119140625f,-0.20219999818801870361539840814657509326934814453125f,-0.0385000108703972276913418681942857801914215087890625f,0.693700012612342931106468313373625278472900390625f,-0.42519999481290537968192211337736807763576507568359375f,0.2631999955356121301264238354633562266826629638671875f,-0.227200006420910316506933668279089033603668212890625f,0.374800000679492928412628316436894237995147705078125f,-0.2819000084459784005730398348532617092132568359375f,0.09829999847412107527588887023739516735076904296875f,-0.643000001654029151865188396186567842960357666015625f,-0.4226000012993811338901650742627680301666259765625f,-0.26919999496936830585269717630580998957157135009765625f,-0.09079999459981902276695109321735799312591552734375f,0.1739999954253435010542716554482467472553253173828125f,0.36600001224875455818619229830801486968994140625f,0.28640002098530548124699635081924498081207275390625f,-0.7724000213503838807582724257372319698333740234375f,-0.63120000288933486576326004069414921104907989501953125f,-0.00600001001358041463618064881302416324615478515625f,-0.64590000841617600801924936604336835443973541259765625f,0.159900000190734914440326974727213382720947265625f,-0.057400006225705180185059361974708735942840576171875f,0.129400014257431195119352196343243122100830078125f,0.190000025890767698655281492392532527446746826171875f,0.928499986469745408612652681767940521240234375f,-0.153100009472668163734709878553985618054866790771484375f,0.295300012093782537903052798355929553508758544921875f,-1.08080003378987310469483418273739516735076904296875f,1.33940002677440617162574199028313159942626953125f,-0.21850000122189516815751630929298698902130126953125f,-0.08470002381801577939768321812152862548828125f,-0.59469998328685758082201573415659368038177490234375f,-0.26999999612569780538251507095992565155029296875f,-0.5998999700158835768348808414884842932224273681640625f,-0.3942999909788369450325262732803821563720703125f,-0.8409999921321869020829353758017532527446746826171875f,0.4422000048935412763739805086515843868255615234375f,-0.739899995273351596125621654209680855274200439453125f,0.659799970579147299076794297434389591217041015625f,-0.27449999812245362562634909409098327159881591796875f,-0.24430004305839503331299056299030780792236328125f,-0.6288999944984912104928298504091799259185791015625f,0.85269996117353485942658153362572193145751953125f,-0.25980001766681670716252483543939888477325439453125f,0.26860003798604026314933435060083866119384765625f,1.28350002491474146637528974679298698902130126953125f,-0.41130004699230227060979814268648624420166015625f,-1.03020000185370452783217842807061970233917236328125f,0.57220001338720294370432384312152862548828125f,-1.207199991035461383859228590154089033603668212890625f,0.569699974346161042149105924181640148162841796875f,0.466399991071224206962142488919198513031005859375f,-1.46699997234344436947139911353588104248046875f,1.12890002146959300688422445091418921947479248046875f,-0.03089999664425846503945649601519107818603515625f,-0.0997000067189335315021025962778367102146148681640625f,0.4452999982357024322254801518283784389495849609375f,0.72030000598430632408053497783839702606201171875f,0.197700008857250253413440077565610408782958984375f,0.131300002177059393915214968728832900524139404296875f,-0.750500003218650846292803180404007434844970703125f,-0.03180000230371959890618427380104549229145050048828125f,0.23340000737309463119117936003021895885467529296875f,0.01899999535083764234943970450331107713282108306884765625f,0.22580000614821915672081331649678759276866912841796875f,-0.55230000697523407016120700063765980303287506103515625f,-0.17489998528957340084843963268212974071502685546875f,0.2725999974995849672865233515040017664432525634765625f,0.1738000037789344531802271376363933086395263671875f,-0.363100015656650010331674138797097839415073394775390625f,0.406300001394748644401033743633888661861419677734375f,0.1545999976709484524661775139975361526012420654296875f,0.24500000752508643842020319425500929355621337890625f,0.15530000140219912818650982444523833692073822021484375f,0.04630000512003906720792656415142118930816650390625f,0.283600008407235204543894724338315427303314208984375f,-0.218800002288818351559029906638897955417633056640625f,0.0091000034675001728601273498497903347015380859375f,0.13850000476092105827774503268301486968994140625f,-0.106999996595084689854360249228193424642086029052734375f,-0.01900000146031377568078823969699442386627197265625f,0.2333000071167947719885660262661986052989959716796875f,0.346299993944167994186500436626374721527099609375f,0.0032999983251096320913120507611893117427825927734375f,-0.31489999408125868285424076020717620849609375f,-0.9103999904453754510313956416212022304534912109375f,0.793500003457069169598980806767940521240234375f,0.8607000006020069093892743694595992565155029296875f,-0.3629999817609785850436310283839702606201171875f,0.696199961799383260085960500873625278472900390625f,0.41989998707771292174584232270717620849609375f,-0.34760000100135801659462231327779591083526611328125f,0.1846000062763692994849407114088535308837890625f,-0.070199997793138013690850129933096468448638916015625f,-0.27519999473839984460710184066556394100189208984375f,-0.462200013834237921628300682641565799713134765625f,-0.11010002990960998658920289017260074615478515625f,-0.245400008177757200655833003111183643341064453125f,0.9167999974608420643562567420303821563720703125f,-0.727999998152256022621031661401502788066864013671875f,-0.24669998438954365838071680627763271331787109375f,-0.169300007593631729463368174037896096706390380859375f,0.273700016412139035537620657123625278472900390625f,0.360500010073184995462725055404007434844970703125f,-0.47380001406073546377228922210633754730224609375f,0.8510000077784061200958376502967439591884613037109375f,-0.6438000063121318561343287001363933086395263671875f,-0.443100002580881113800614912179298698902130126953125f,0.7108000046208502453737310133874416351318359375f,-0.2686999926447868336509827713598497211933135986328125f,0.3438999867498875318005957524292171001434326171875f,-0.45009997581839567004635682678781449794769287109375f,0.89439999667406056005347636528313159942626953125f,-0.37720002024173737709134002216160297393798828125f,-0.48750000715255747962828536401502788066864013671875f,0.493599987806379925103783534723334014415740966796875f,0.5540000063776968630691044381819665431976318359375f,0.50410000380277619758118134996038861572742462158203125f,0.17630001152753838056241875165142118930816650390625f,0.286800003048777230052479580990620888769626617431640625f,-0.254399993731081408565586343684117309749126434326171875f,0.44290001797527100979579017803189344704151153564453125f,-0.0613000018417835246253844161401502788066864013671875f,-0.228800021697580735491328596253879368305206298828125f,0.1603999888062475065453327260911464691162109375f,-0.151900011426210124199087658780626952648162841796875f,0.29949999022483808630568091757595539093017578125f,-0.316600000636279244048409964307211339473724365234375f,0.017799990636110152308901888318359851837158203125f,-0.1494000070303679039707134279524325393140316009521484375f,-0.034200002084672498359196879391674883663654327392578125f,0.175799993035197221757925944984890520572662353515625f,0.5802000165313483392992566223256289958953857421875f,-0.320600009547173936397257421049289405345916748046875f,0.22340000305175777839394868351519107818603515625f,0.697200006867945365485184083809144794940948486328125f,0.508800013911724136050906963646411895751953125f,0.1944999963715672297581704697222448885440826416015625f,-0.079300006252527310834921081550419330596923828125f,-0.213799990405142192884113683248870074748992919921875f,-0.776200017753243542983909719623625278472900390625f,0.14850001981109406390402227771119214594364166259765625f,-0.38610000892877582856499429908581078052520751953125f,-0.72760001243799898862363306761835701763629913330078125f,0.72309999617338149846546002663671970367431640625f,0.04009998133182524071571606327779591083526611328125f,0.07490005562305412922796676866710186004638671875f,0.32749999031424525952616022550500929355621337890625f,0.274000014126301039141253568232059478759765625f,0.443699984207004405334373586811125278472900390625f,0.343699983052164537156158985453657805919647216796875f,0.49750000678002825882373372223810292780399322509765625f,0.23800001610815524344388904864899814128875732421875f,0.2007000153541564912984540569595992565155029296875f,-1.4125000083446508369888761080801486968994140625f,-0.0699999797344208474214610760100185871124267578125f,-0.8998999906539921767034684307873249053955078125f,-0.9711000181674958042776779620908200740814208984375f,1.24950003448128654781612567603588104248046875f,-0.161400007220357633030971555854193866252899169921875f,0.022799998887628181165609930758364498615264892578125f,-0.2175000049173831939697265625f,-0.27809996421039073766223737038671970367431640625f,-1.0507000242948532076070478069595992565155029296875f,0.9755000309348105247408966533839702606201171875f,0.1632999877452849801784395822323858737945556640625f,0.4545000581741334144680877216160297393798828125f,0.87450002002716065163667735760100185871124267578125f,-1.23740002298951168313578818924725055694580078125f,0.57220000101923940150072667165659368038177490234375f,-0.19280003004968193636159412562847137451171875f,0.155699996876716539873086730949580669403076171875f,0.88580000473260867011049413122236728668212890625f,-0.9967000012278557363742947927676141262054443359375f,-0.3468000046133994374031317420303821563720703125f,0.334000035136938377977600111989886499941349029541015625f,-0.305200002896785616002262031543068587779998779296875f,0.10060001401752227678798590204678475856781005859375f,-0.1186000005841254534288964350707828998565673828125f,0.4556000298500062495321571987005881965160369873046875f,-0.12440000710487364443679325631819665431976318359375f,0.0262999796018006382070097970427013933658599853515625f,0.1752999913811681409470111248083412647247314453125f,0.5064000057116146980007442834903486073017120361328125f,-0.43799999308586112789498656638897955417633056640625f,-0.401099987173080041458206324023194611072540283203125f,-0.200100006663799234729594900272786617279052734375f,0.066499993972480186954499004059471189975738525390625f,0.50230001170635230067773591144941747188568115234375f,0.13230000120103346805677801967249251902103424072265625f,0.29250000964850197959776778589002788066864013671875f,0.0355999876052143504612246260876418091356754302978515625f,0.479199999178945901245896266118506900966167449951171875f,-0.164200011658668543201855527513544075191020965576171875f,0.301000006735324898698991091805510222911834716796875f,-0.80639998570680615497252574641606770455837249755859375f,-0.1871999998271463017118776406277902424335479736328125f,-0.1285000244304537186934567216667346656322479248046875f,0.20770000319480885764278355054557323455810546875f,-0.31569999940991377851418064892641268670558929443359375f,0.47050000292062765794298684340901672840118408203125f,-0.4119999999105929955334204350947402417659759521484375f,0.3400000089406969294714144780300557613372802734375f,-0.0544999919757247486273854519822634756565093994140625f,1.57989999065399189248637412674725055694580078125f,0.412200020986795312438744076644070446491241455078125f,0.6509999934732917381552397273480892181396484375f,-0.15979999009966849854436077293939888477325439453125f,0.88239997783899326577738975174725055694580078125f,0.008000027358531891508164335391484200954437255859375f,1.492399973666667989391498849727213382720947265625f,0.5777000023007392570661977515555918216705322265625f,-1.58319997959137026555254124104976654052734375f,0.937999975651502726492481087916530668735504150390625f,0.240399982845783455331911682151257991790771484375f,-0.028899983471632140208384953439235687255859375f,-0.603599990749358994435169734060764312744140625f,-0.663700004565715762083755180356092751026153564453125f,0.578800017786026188559844740666449069976806640625f,-0.202299993824958779242706441436894237995147705078125f,0.18439999984055788218029192648828029632568359375f,-0.1077999935232103467797060147859156131744384765625f,-0.688199967316538430139871707069687545299530029296875f,-0.16480000023245811036076702293939888477325439453125f,-0.726599985176324736357855726964771747589111328125f,-0.89759997559487825213864198303781449794769287109375f,0.1871999698013073754054858000017702579498291015625f,0.22199997547268868203218517010100185871124267578125f,-1.28689995703697235285289934836328029632568359375f,-0.76169997664093980205279876827262341976165771484375f,-0.1797999851822851269389502704143524169921875f,-0.212399965023994496959858224727213382720947265625f,-1.3812000096321099817942013032734394073486328125f,-0.28759999489188192711708325077779591083526611328125f,-0.689100016915798363470457843504846096038818359375f,0.128599997007846855723300905083306133747100830078125f,-0.1358000136733057416904557612724602222442626953125f,-0.3597000037759541246629169108928181231021881103515625f,0.52170000897645962822934961877763271331787109375f,0.156000000476836930829449556767940521240234375f,-0.415200002598762551997424452565610408782958984375f,-0.6809000058457257420485575494240038096904754638671875f,0.1985000061392785397629268118180334568023681640625f,0.02589999858140945576678859652020037174224853515625f,-0.00070000581741336276309084496460855007171630859375f,-0.26939999779164780502327403155504725873470306396484375f,0.15420000092983254802447845577262341976165771484375f,0.3473999935597180677149253824609331786632537841796875f,0.0126000084519386490455872262828052043914794921875f,-0.281099994437396194513212321908213198184967041015625f,0.0218999939918518293779925443232059478759765625f,-0.5578000182405118412276578965247608721256256103515625f,0.0214000074625015912488379399292171001434326171875f,0.277000030271709329898754958776407875120639801025390625f,-0.439000011146068569445333196199499070644378662109375f,0.2953999947294592143265390404849313199520111083984375f,0.4930000048577785065617717918939888477325439453125f,0.0227000046104193220486422433168627321720123291015625f,-0.37189999779164784055041081956005655229091644287109375f,0.318199999856948689824776010937057435512542724609375f,0.0346000095546246111410937373875640332698822021484375f,0.0522999993011354913363675223081372678279876708984375f,-0.5995000092983246986477752216160297393798828125f,0.0869999851956964054267018582322634756565093994140625f,0.003199982571601367453695274889469146728515625f,-0.20220000742673871485521885915659368038177490234375f,1.08959998973607952166275936178863048553466796875f,-0.77189998371005064115024651982821524143218994140625f,-0.38679998166561091466064681299030780792236328125f,0.3521999788165093292491292231716215610504150390625f,0.5329999835491179283053497783839702606201171875f,-1.4819000005483626836166877183131873607635498046875f,1.000399984335899716825224459171295166015625f,0.503800028514862052730904906638897955417633056640625f,-1.044300028455257489667928894050419330596923828125f,-0.1041999732136726919407010427676141262054443359375f,-0.83440000919103596288550761528313159942626953125f,0.8236999799787998455258275498636066913604736328125f,-0.903699972677230789486202411353588104248046875f,0.1962000309407709863762647728435695171356201171875f,-1.2493000007390975270027411170303821563720703125f,-1.0104000256121157264033172396011650562286376953125f,0.6701999865055086047505028545856475830078125f,0.628400020635127898316341088502667844295501708984375f,0.30219999118447304908841033466160297393798828125f,-0.4162999960303306323794458876363933086395263671875f,0.6755000078380106742770294658839702606201171875f,0.790699986408650801195108215324580669403076171875f,-0.895399987725913337754946041968651115894317626953125f,0.16429997771680360241219887029728852212429046630859375f,0.43090001638829722452328496729023754596710205078125f,0.65679997615218166373551866854541003704071044921875f,-0.03130000705719027109807939268648624420166015625f,-0.4742999965667724637796709430404007434844970703125f,0.276400000905990594901595613919198513031005859375f,-0.3314000085800887784870383256929926574230194091796875f,0.201799998950958237031727549037896096706390380859375f,0.50980000693798055433347826692624948918819427490234375f,-0.081399999006092571551818082298268564045429229736328125f,-0.3665999953091141350114412489347159862518310546875f,0.200999986767768756834584564785473048686981201171875f,0.009100001716613792535071070233243517577648162841796875f,0.0197000005349516893460304345353506505489349365234375f,0.1192000147134064658160923499963246285915374755859375f,-0.0399999862909314884973355219699442386627197265625f,0.2747999929681419484239768280531279742717742919921875f,0.35160000831037763191488920710980892181396484375f,0.4929000280708077941227429619175381958484649658203125f,-0.093299998176097798108230563229881227016448974609375f,0.31520000595152364297746316879056394100189208984375f,0.080800023210048976807229337282478809356689453125f,-0.336800014522671720129665118292905390262603759765625f,0.34399999982118600883040926419198513031005859375f,-0.35889999397695071525760113217984326183795928955078125f,-0.19910000112056724930198470246978104114532470703125f,-0.5772000037387012838507871492765843868255615234375f,-0.153600010418891930186191530083306133747100830078125f,0.01829999646246438516072885249741375446319580078125f,0.086099999839067464790787198580801486968994140625f,0.65480001617670080538147203697008080780506134033203125f,0.13720000325441361610501189716160297393798828125f,0.0319999995380639834507974228472448885440826416015625f,-0.2768000072211027617896661467966623604297637939453125f,-0.41040001577734985804823963917442597448825836181640625f,-0.990799990129470842248338158242404460906982421875f,-0.8805999921500682336983345521730370819568634033203125f,0.6151999777138232872175649390555918216705322265625f,0.4607999942459166664576741823111660778522491455078125f,0.5449999960325657699655721444287337362766265869140625f,-0.8866999675512314826875126527738757431507110595703125f,0.5342999880731102990694125765003263950347900390625f,-0.14550000314414501811910440665087662637233734130859375f,0.30520000509470701643977008643560111522674560546875f,-0.2621000164091586981385262333787977695465087890625f,0.991900014555454845321946777403354644775390625f,0.345900006777048130146567928022705018520355224609375f,-0.76769999160915602942623081617057323455810546875f,0.3192000004827975079280122372438199818134307861328125f,0.8381000294029714581256484962068498134613037109375f,0.8878999895513057794005362666212022304534912109375f,-1.2796999725580207041275571100413799285888671875f,-0.8084999982416629649151218472979962825775146484375f,0.1367000039100645381040521897375583648681640625f,-0.61440000174045561465163700631819665431976318359375f,0.8070999825835230012671672739088535308837890625f,-0.2148999750077724701924353212234564125537872314453125f,-0.7717000332653523031467557302676141262054443359375f,0.4756000158429145585614605806767940521240234375f,-1.1881000031769275437909527681767940521240234375f,0.536700004655122686614276972250081598758697509765625f,0.305500025272369413187334430404007434844970703125f,0.74220001197159291006499870491097681224346160888671875f,-0.96580002410411847080240477225743234157562255859375f,-0.314500014513730885568065787083469331264495849609375f,0.399100002610683457504592297482304275035858154296875f,-0.6571000101506712098853313364088535308837890625f,-0.332900004303455343546147560118697583675384521484375f,-0.19809999866932626044757626004866324365139007568359375f,-0.241100004756450658760513761080801486968994140625f,0.36660003539323826604601208600797690451145172119140625f,-0.6906000094354152185616158021730370819568634033203125f,0.43409999158382372996811682241968810558319091796875f,0.203100022697448867603498001699335873126983642578125f,0.268200008164346304084801886347122490406036376953125f,-0.243000007092952774456762199406512081623077392578125f,0.4698000036969778303586053880280815064907073974609375f,-0.62459999390840525990142850787378847599029541015625f,-0.62329999992698414956748820259235799312591552734375f,0.24169998841285700308390005375258624553680419921875f,0.629300001707672862494291621260344982147216796875f,-0.1000999928057193510966271787765435874462127685546875f,0.284799997699260398764664614645880647003650665283203125f,-0.20559999475777146482613488842616789042949676513671875f,-0.4418000069230796356123391888104379177093505859375f,-0.0045999974846839819520027958787977695465087890625f,-0.451899992687999851881386348395608365535736083984375f,0.32949999588727951760347423260100185871124267578125f,0.22469999163150777832242965814657509326934814453125f,0.64290001451075085014963406138122081756591796875f,-0.42140001990497133466107015919988043606281280517578125f,0.421400009250640916746988295926712453365325927734375f,0.2745999996826054001530792447738349437713623046875f,0.26180000170767303568908346278476528823375701904296875f,0.0717000027924774574472621679888106882572174072265625f,-0.190099997574090906482524587772786617279052734375f,-0.59940001053214064796748061780817806720733642578125f,0.245499967008829145242998492904007434844970703125f,0.137700001406669603198906770558096468448638916015625f,-0.330800008606910722619431908242404460906982421875f,0.8028000093370675216419840580783784389495849609375f,0.6981999821990729770959660527296364307403564453125f,0.0182999537706376447232514692586846649646759033203125f,0.7108000314056877044777138507924973964691162109375f,0.8333000144183635438110968607361428439617156982421875f,-0.5949000155389310151576864882372319698333740234375f,1.219299955141544433701028538052923977375030517578125f,-0.084099990689754378081488539464771747589111328125f,-1.240199961322546062802985034068115055561065673828125f,0.529699968087673056515995995141565799713134765625f,0.915999985501170232993217723560519516468048095703125f,-0.0975000002235173468534412677399814128875732421875f,0.48219997997134933509499887804850004613399505615234375f,0.1485999913826585583365158527158200740814208984375f,-1.20990002805590624035403379821218550205230712890625f,1.48470000712871552650540252216160297393798828125f,-1.272099980199337121433700303896330296993255615234375f,0.880800013077259080773728783242404460906982421875f,-0.26829999653697012007569355773739516735076904296875f,-0.165999977901577722150250338017940521240234375f,-0.9199000082373618436548667887109331786632537841796875f,-0.0128999934256075476923797396011650562286376953125f,0.2280000051930546334233440575189888477325439453125f,-0.31499997887760411430235762964002788066864013671875f,-0.0810000152289866992560973812942393124103546142578125f,-0.09260000100135812317603267729282379150390625f,0.1018999966740607998616496843169443309307098388671875f,0.1825000065565109963472423260100185871124267578125f,-0.0130000240430236946309605627902783453464508056640625f,0.5926000030875204771518838242627680301666259765625f,0.00139999256134049065991575844236649572849273681640625f,0.31540000020563596638822900786180980503559112548828125f,0.530200030128658017503084920463152229785919189453125f,-0.24580000681877134383057637023739516735076904296875f,0.00629999405592673866749464650638401508331298828125f,-0.1647000086188317169444417231716215610504150390625f,-0.05949999640882019935617108785663731396198272705078125f,-0.17450000214576721901948985760100185871124267578125f,-0.0562000140652061386248306007473729550838470458984375f,0.4342000086784363332981229177676141262054443359375f,0.2058999982833860098452305464888922870159149169921875f,-0.40389999680817123550724545566481538116931915283203125f,-0.41489999635368601804685795286786742508411407470703125f,-0.00269999663829789682267801254056394100189208984375f,-0.1428999966293573731235255763749592006206512451171875f,-0.06230000298917293122258342918939888477325439453125f,-0.1883999952286481427421449552639387547969818115234375f,-0.348000008024275331308672321029007434844970703125f,-0.45700000017136321872612825245596468448638916015625f,0.0718000004410743741800615680404007434844970703125f,0.3873000241115693853544144076295197010040283203125f,-0.38010001538097870632526564804720692336559295654296875f,-0.38180001172870403625125845792354084551334381103515625f,0.29180000170767306233443605378852225840091705322265625f,-0.49180000781714905411234894927474670112133026123046875f,1.3465999826431271912952070124447345733642578125f,0.431400009548664087333236238919198513031005859375f,-1.0678999950647352079613483510911464691162109375f,-0.525100002193450876575298025272786617279052734375f,-0.1384999867677692009237944148480892181396484375f,-0.09929999466687444709123155917041003704071044921875f,-0.45029998016804473337515446473844349384307861328125f,0.520600023740529938010013211169280111789703369140625f,0.819599981987476411404713871888816356658935546875f,0.70100001969933511958288363530300557613372802734375f,-1.18430003262758276605381979607045650482177734375f,-0.23580000890493391096924824523739516735076904296875f,0.7330999937891959916669293306767940521240234375f,0.050800011140108125573533470742404460906982421875f,-1.1941999937772749262876459397375583648681640625f,0.9479999803081156084516578630427829921245574951171875f,-0.675300006617605586001218398450873792171478271484375f,-0.764499989926815004537274944595992565155029296875f,0.56699997293949166987658827565610408782958984375f,-0.371299994689226142696725219138897955417633056640625f,0.565799983796477334863084251992404460906982421875f,0.084500001847743977378968338598497211933135986328125f,0.04209998321682206778859836049377918243408203125f,-1.25070002071857455661074709496460855007171630859375f,1.37030002684593199546725372783839702606201171875f,0.6715000125989316615005009225569665431976318359375f,0.38200000486522878162531924317590892314910888671875f,-0.90660000056177381111410795710980892181396484375f,0.342800001402199239208812286960892379283905029296875f,-0.9373999835573136873989597006584517657756805419921875f,0.3535000001788137780067700077779591083526611328125f,-0.41829999631345271549065500948927365243434906005859375f,-0.26210000433921809559478788287378847599029541015625f,-0.04560001889765263882736689993180334568023681640625f,0.2255999991536139503978120046667754650115966796875f,0.8035000276342036951149339074618183076381683349609375f,-0.0474999865889547123742886469699442386627197265625f,-0.20569999547973261133648748000268824398517608642578125f,-0.40990001350864779805505122567410580813884735107421875f,-0.0348999880462887990262288440135307610034942626953125f,-0.6238999919652936654301811358891427516937255859375f,0.4201000119909645835747369346790947020053863525390625f,0.19189999309778205116572280530817806720733642578125f,0.005699991326034048455539959832094609737396240234375f,0.5343000064015388517191240680404007434844970703125f,0.5746999974429602975334319125977344810962677001953125f,0.09130001949965971252964891391457058489322662353515625f,-0.32410001564919965755251496375421993434429168701171875f,0.0541000012695789234129506439785473048686981201171875f,0.47600000893324601047851274415734224021434783935546875f,-0.1411999987542629231285218338598497211933135986328125f,-0.6931000129371882589879305669455789029598236083984375f,-0.04490000466108323706748706172220408916473388671875f,-0.225399987465143392117994380896561779081821441650390625f,-0.382999994911253249796345698996447026729583740234375f,-0.1058000071167946865013931301291449926793575286865234375f,0.073400002120435237085160906644887290894985198974609375f,0.07879999763220546515896103301201947033405303955078125f,0.45760001143217099439652884029783308506011962890625f,-0.47560002094656272930706109036691486835479736328125f,-0.95579998953342482082007336430251598358154296875f,-0.3028999961078167046935050166212022304534912109375f,0.08919998185634625542661524377763271331787109375f,0.34240000406503678931358081172220408916473388671875f,-1.36559999378919627588402363471686840057373046875f,1.4423000048518179738721300964243710041046142578125f,-0.900599961304664020644850097596645355224609375f,-0.1048000050380826220219887545681558549404144287109375f,0.54669998887851800617454500752501189708709716796875f,0.559299980622530057416952331550419330596923828125f,-0.0012000218510621607492794282734394073486328125f,0.089100028240680639868287471472285687923431396484375f,0.539599988543987052480588317848742008209228515625f,-0.59729999382495879700627483543939888477325439453125f,0.19720001308918000404446502216160297393798828125f,0.67159998335093273880147535237483680248260498046875f,-0.11410000719279000946926316828466951847076416015625f,-0.21710000500977033599525611862190999090671539306640625f,-0.51960000828802588301869036513380706310272216796875f,0.43869999450743202284996868911548517644405364990234375f,0.62949999104440212960298595135100185871124267578125f,-0.763299997133016550066031413734890520572662353515625f,-0.105200022566318551753283827565610408782958984375f,-0.016000001072883573982608140795491635799407958984375f,0.238599973948300458914673072285950183868408203125f,-0.2092000056795775553286631520677474327385425567626953125f,0.0177999995581805681155174170271493494510650634765625f,-0.00489998320341111792686206172220408916473388671875f,-0.752099990928172701387666165828704833984375f,-0.2241000409066675747027375109610147774219512939453125f,0.357100013279914918484791996888816356658935546875f,-0.22600002394616602874322097704862244427204132080078125f,0.60760000457763663206378623726777732372283935546875f,-0.191099997976422297796261773328296840190887451171875f,-0.35969999699592580810758590814657509326934814453125f,0.550900017990171964044066044152714312076568603515625f,-0.31330000376403333017805152849177829921245574951171875f,0.444299984906613598667490805382840335369110107421875f,-0.2571000079154970308081828989088535308837890625f,0.5130999907717102015425325589603744447231292724609375f,0.480500036895275428605600609444081783294677734375f,0.0347999900996687205889656979707069694995880126953125f,-0.006000012695789536110169137828052043914794921875f,-0.2482000016450880419682789579383097589015960693359375f,0.442600004875659980285718120285309851169586181640625f,-0.85120001101046849356634993455372750759124755859375f,-0.498800004971027277633766061626374721527099609375f,0.518499991908669244367047213017940521240234375f,0.12590000573396675775939002051018178462982177734375f,0.5096000134289264682507791803800500929355621337890625f,-0.0920000090003014747708220966160297393798828125f,0.48509998896867045914405025541782379150390625f,0.256499997735023299583190237171947956085205078125f,0.314600023561716124476106415386311709880828857421875f,-0.60980001170635222962346233543939888477325439453125f,0.495300033514201931694742597755976021289825439453125f,-0.214500007808208437154462444595992565155029296875f,-0.0539000148013235591548664160654880106449127197265625f,0.08990000644922258032920581172220408916473388671875f,0.0548000054478643416899785734131000936031341552734375f,0.9083000425815583867006353102624416351318359375f,-0.73440000650882719668288700631819665431976318359375f,-0.2883999705672266600231523625552654266357421875f,0.470399984931945791544194435118697583675384521484375f,-0.795499990403651935366724501363933086395263671875f,0.93740000298470249617821536958217620849609375f,0.096699997688829864017634463380090892314910888671875f,1.2229999902546406786996158189140260219573974609375f,-0.9475000196695333443130948580801486968994140625f,-0.10390004173517208840848979889415204524993896484375f,0.60440002617836086074021295644342899322509765625f,0.64780001462697978098503881483338773250579833984375f,-1.32779999972581919109870796091854572296142578125f,-1.0036999971151352450959848283673636615276336669921875f,0.6005000251233578723031314439140260219573974609375f,-0.2878999882102011298457000521011650562286376953125f,-0.6203000792980191135939094237983226776123046875f,0.299099976831674663202420560992322862148284912109375f,0.484000011742114821799987112171947956085205078125f,0.14450001212954521889741954510100185871124267578125f,1.26809997560977905095569440163671970367431640625f,0.3717999862849712400247881305404007434844970703125f,-0.0474999690055852852310636080801486968994140625f,1.019600023710727665360309401876293122768402099609375f,-0.097000045359134645650556194595992565155029296875f,0.464100030624866377593207289464771747589111328125f,0.78259998371601113831275142729282379150390625f,0.269799987119436313065534704946912825107574462890625f,-0.937299989056587179447888047434389591217041015625f,0.482099979752302232327565434388816356658935546875f,0.29089998993873589228087439551018178462982177734375f,-0.5457000143110752876651758924708701670169830322265625f,0.398100005412101776158806387684307992458343505859375f,0.2625999975740906933197038597427308559417724609375f,0.277400007343292287487201974727213382720947265625f,0.5522000020623203386804789261077530682086944580078125f,-0.37219999431371686426928135915659368038177490234375f,-0.11750001072883609065655718950438313186168670654296875f,0.53870001052618032844065965036861598491668701171875f,0.1594000374659898522367029727320186793804168701171875f,-0.53740000913143148864037357270717620849609375f,-0.346800040338933523997866359422914683818817138671875f,0.9480000124573706443698029033839702606201171875f,0.243599992649257135468587875948287546634674072265625f,0.350400004899501738009348628111183643341064453125f,0.4930000257566573562684197895578108727931976318359375f,-0.51820001147985461642520021996460855007171630859375f,-0.264700019682944098775578822824172675609588623046875f,0.15670000420808793961668925476260483264923095703125f,0.12820000127255914978974260520772077143192291259765625f,-0.099400001591444020476728837820701301097869873046875f,0.4679000124245884517648619294050149619579315185546875f,-0.14750001102685939002867598901502788066864013671875f,-0.2476000268921254576781620926340110599994659423828125f,0.675999996304511796552105806767940521240234375f,0.0802000174254178244126478602993302047252655029296875f,-0.699399999952316164808507892303168773651123046875f,0.09320000049024824984034154340406530536711215972900390625f,0.04719999982714651043380627015721984207630157470703125f,0.1350000168010592194178798308712430298328399658203125f,0.07010000785589198812886024825274944305419921875f,-1.2943000055074691534429121020366437733173370361328125f,0.5272000093638897766368245356716215610504150390625f,-0.0999999922513961081449451739899814128875732421875f,0.3457999829769136113100103102624416351318359375f,1.0060999724209309658107258655945770442485809326171875f,0.0738999946475029645398535649292171001434326171875f,-0.6081999992609024019429853069595992565155029296875f,-0.12449997830390913122755591757595539093017578125f,0.27680002204775799867775276652537286281585693359375f,0.97059997560977961938988300971686840057373046875f,-1.3469000324368476384506720933131873607635498046875f,0.0115000638961788581582368351519107818603515625f,-0.379500014364719362447431194595992565155029296875f,0.7573000230312345593119971454143524169921875f,1.184700019049644392765685552149079740047454833984375f,-0.171500018894672479063956416212022304534912109375f,-0.001099996709823525264937416068278253078460693359375f,-1.055400018012523588595286128111183643341064453125f,0.5022000069797039056851417626603506505489349365234375f,-0.5504000096678733910948722041212022304534912109375f,1.011500025898218080300239307689480483531951904296875f,-0.648600010716915420516670565120875835418701171875f,-1.01560000466704369870285518118180334568023681640625f,0.12960003354549343157486873678863048553466796875f,0.73559998484849931088547236868180334568023681640625f,0.08379998679161104746526689268648624420166015625f,0.07090000003427265884425878539332188665866851806640625f,-0.00349999380856758079971768893301486968994140625f,-0.322100011789798745809321189881302416324615478515625f,0.26620000627934936598961712661548517644405364990234375f,0.19989998677968989770903363023535348474979400634765625f,0.37970000213682641998502731439657509326934814453125f,-0.21669998647570631877812274979078210890293121337890625f,-0.20449999253451800473868615881656296551227569580078125f,-0.5265000329390172595367403118871152400970458984375f,0.1233000145673752712127679842524230480194091796875f,-0.23670000789612555269769700316828675568103790283203125f,0.45270001600980780409599901759065687656402587890625f,0.74949999935179933974183086320408619940280914306640625f,-0.07989999363422395362022143672220408916473388671875f,-0.72300001148879555756820991518907248973846435546875f,0.0894000059127808555103911203332245349884033203125f,-0.4704000162243844318510355151374824345111846923828125f,0.05999999649822707059598769774311222136020660400390625f,-0.08810000138878815167942093466990627348423004150390625f,0.23570000231564047776799952771398238837718963623046875f,0.4894000195100904893052984334644861519336700439453125f,-0.06819998733997323370203957892954349517822265625f,-0.4879999931231140397613899040152318775653839111328125f,0.0451999996185303842821667785756289958953857421875f,0.16059999311864359139434554890613071620464324951171875f,0.57380001793503776497118451516143977642059326171875f,-0.3127000388085845461461076411069370806217193603515625f,0.8635000165700912333477390347979962825775146484375f,0.458600025655329535112514349748380482196807861328125f,0.19300000858306898265936979441903531551361083984375f,-0.2382000047370791406819989788345992565155029296875f,0.7314000193834306884355100919492542743682861328125f,-0.134600004301965281428010712261311709880828857421875f,0.129000014238059623750132232089526951313018798828125f,0.2648999988496303359397643362171947956085205078125f,-0.85279997059404877290944568812847137451171875f,-0.219499990932643385121991741470992565155029296875f,0.33179996970743008688486952451057732105255126953125f,-0.22340003389716134307718675700016319751739501953125f,-1.20629998977184271780060953460633754730224609375f,-0.372299983990192373539684922434389591217041015625f,-0.78900000816583570895090815611183643341064453125f,1.30579998491406445992879525874741375446319580078125f,-0.676299988281726882632938213646411895751953125f,-1.45120001589059821611726874834857881069183349609375f,1.05059998872280146997582050971686840057373046875f,-0.434699996139109090531604806528775952756404876708984375f,-0.449500017754733693919888537493534386157989501953125f,-0.212399989722669157998780065099708735942840576171875f,-0.27369999238401643282259101397357881069183349609375f,1.12160000424981110000999251496978104114532470703125f,-1.00339997503757416552616632543504238128662109375f,-0.7719999808371067473444782081060111522674560546875f,0.281599984431267102991114370524883270263671875f,-0.22179999805688854763729978003539144992828369140625f,-0.2963999822795386762663838453590869903564453125f,-1.1612000233411787775139600853435695171356201171875f,-0.126299968910217330630985088646411895751953125f,-0.031199996668100471453044519876129925251007080078125f,0.909000037968159091406050720252096652984619140625f,-1.04959999644160273390980364638380706310272216796875f,0.11220003395080624386537238024175167083740234375f,0.2964999947547912739764797152020037174224853515625f,0.47420000689029695450926737976260483264923095703125f,-0.37840000498890891122982793604023754596710205078125f,-0.0392000054001808706516385427676141262054443359375f,-0.2286999990150331285576612572185695171356201171875f,0.07030000002384184654147247783839702606201171875f,0.27850000301003452474191135479486547410488128662109375f,-0.0156999980688095064351728069595992565155029296875f,0.658400004988908715830575602012686431407928466796875f,0.1048000099182129218888803734444081783294677734375f,-0.1752999970063566781419694962096400558948516845703125f,0.0060999933198093714281640131957828998565673828125f,-0.1124000048846005572755757384584285318851470947265625f,0.494000009357929315001456416212022304534912109375f,0.43060000850409263506435308954678475856781005859375f,-0.477400007045269081373817243729718029499053955078125f,-0.33150003137439532086006011013523675501346588134765625f,0.46110001190900806733452554908581078052520751953125f,0.00349999956414104307356183198862709105014801025390625f,0.16569999801293011643110730801708996295928955078125f,0.1391000002264977108978882824885658919811248779296875f,0.4188999973297118373238845379091799259185791015625f,-0.238199995870888248106211904087103903293609619140625f,-0.017400007566809705394916818477213382720947265625f,-0.2640000181496144904258471797220408916473388671875f,0.278400013780593980072808335535228252410888671875f,-0.17990000175535658666348126644152216613292694091796875f,-0.17650001494586486927573787397705018520355224609375f,0.158900018265843101517020841129124164581298828125f,-0.19129998917579627004670328460633754730224609375f,-0.5396000140994787752646288936375640332698822021484375f,-0.246199966418743088070186786353588104248046875f,-0.97170000376105303274698599125258624553680419921875f,1.20389997706413254263679846189916133880615234375f,-0.798000013798475293924639117904007434844970703125f,0.993700020956993057552608661353588104248046875f,0.28779999734163286273513904234278015792369842529296875f,0.15629999901056290667611392564140260219573974609375f,0.57919998126029970109129862976260483264923095703125f,0.29040002844333656639719265513122081756591796875f,-0.764399996972084050383955400320701301097869873046875f,1.2022999888703225224162451922893524169921875f,-0.44179999120235446952165148104541003704071044921875f,-0.38379997695684409109162515960633754730224609375f,1.0499999958276748923680088410037569701671600341796875f,-0.1306000186741353985553359962068498134613037109375f,-0.419700005936622488889042870141565799713134765625f,-0.96080000771284090888002538122236728668212890625f,-0.3687999917089939128089781661401502788066864013671875f,0.47269999401196827193416538648307323455810546875f,-0.598299980741739290124314720742404460906982421875f,0.8103999947667119840843952260911464691162109375f,-0.88299999144673346762601795489899814128875732421875f,1.343599976071715218495228327810764312744140625f,-0.1030000154376029541936077293939888477325439453125f,-1.0877999761819836521681281737983226776123046875f,-0.2734000018596649539404097595252096652984619140625f,0.4918999989837402608827687799930572509765625f,-0.05149999937415117301497957669198513031005859375f,0.956999973237515177970635704696178436279296875f,-0.716500005912035664579207150381989777088165283203125f,0.524600008139014306607350590638816356658935546875f,-0.28660001568645210312524795881472527980804443359375f,0.664599988543987052480588317848742008209228515625f,-0.4570000301599506098426672906498424708843231201171875f,0.421000011503696480730241091805510222911834716796875f,-0.3486000278905037053078785902471281588077545166015625f,0.676200018572807426409099207376129925251007080078125f,0.212900012573600083687352935157832689583301544189453125f,-0.277700011129677404309035182450315915048122406005859375f,-0.5139000311180954572165546778705902397632598876953125f,0.1395000066161156127009235206060111522674560546875f,-0.07460000232756136551159897862817160785198211669921875f,0.18980000425577170375390778644941747188568115234375f,0.1345000061690807824366089562317938543856143951416015625f,-0.076900000883638852666734919694135896861553192138671875f,0.4026000143378973827879008240415714681148529052734375f,-0.17940000270903111445619515507132746279239654541015625f,0.3478999990880489168176836756174452602863311767578125f,-0.06349999809265138850378207280300557613372802734375f,0.60579999970346687376832051086239516735076904296875f,0.045799996984004831546144487219862639904022216796875f,0.225799992886185574292312594479881227016448974609375f,0.343000006273388891031572711654007434844970703125f,0.1657000037312508222697715609683655202388763427734375f,-0.349300005805492475019491394050419330596923828125f,0.42900002828240435537310304425773210823535919189453125f,-0.0762000044167041590270628148573450744152069091796875f,-0.0346999854475255331553995574722648598253726959228515625f,-0.490900023056566936485722862926195375621318817138671875f,-0.2813000053808092371099291995051316916942596435546875f,-0.3914999896883966101768237422220408916473388671875f,-0.551999997563660027566356802708469331264495849609375f,-0.08939999484866867351939845320885069668292999267578125f,0.97769998702704885573666615528054535388946533203125f,0.1766999931812289759136547218076884746551513671875f,-0.74070002824366110161236065323464572429656982421875f,-0.446999994993209526228383765555918216705322265625f,-0.82470000951290123314407765064970590174198150634765625f,0.35879999215602875750619205064140260219573974609375f,-0.71610003314316295064401174386148341000080108642578125f,0.43230000365972554021709584048949182033538818359375f,-0.52139998663812858747945711002103053033351898193359375f,-0.37139999274760526759564527310431003570556640625f,0.14520000122040510603937946143560111522674560546875f,-0.3012000126123428600521947373636066913604736328125f,0.22200001276284464513111061023664660751819610595703125f,-0.56790000877380375499114961712621152400970458984375f,0.222300010067224551590925329946912825107574462890625f,0.45360001012086836880143891903571784496307373046875f,0.71370002609789384706573400762863457202911376953125f,-0.46440000382661850153453997336328029632568359375f,-0.55260001497119670688107362366281449794769287109375f,-0.2070000057220457989615169935859739780426025390625f,0.59940000147968530708197931744507513940334320068359375f,0.42779998601675028879753881483338773250579833984375f,-1.005300036084652237633463300880976021289825439453125f,0.19259999384880099881911519332788884639739990234375f,-0.38430000226646665595353624667041003704071044921875f,-0.25199998337030393713575904257595539093017578125f,-0.26550000388920313998397659815964289009571075439453125f,1.019100019448995197990370797924697399139404296875f,-0.6453999934256078319094740436412394046783447265625f,-0.043199999856948778642617980949580669403076171875f,0.71999999612569809404050147350062616169452667236328125f,0.4626000149339437594875334980315528810024261474609375f,1.077800010715424861018618685193359851837158203125f,1.0732000541344290933665206466685049235820770263671875f,1.182399986481666331883388920687139034271240234375f,0.6342000374376777216411937843076884746551513671875f,-0.7804999896585940177828888408839702606201171875f,-0.6223000146865842463483886604080908000469207763671875f,0.06530000009834768803784754709340631961822509765625f,0.379500014737248481111464570858515799045562744140625f,-0.628099958026409499467490604729391634464263916015625f,-0.47469998865127482901016264804638922214508056640625f,0.22209999182224293701182205040822736918926239013671875f,-0.4467000245481733688990289010689593851566314697265625f,0.338799989622831365654320734392968006432056427001953125f,0.45680001247376200712579930041101761162281036376953125f,-0.559499992161989201377991776098497211933135986328125f,-0.4969000148534774563557903093169443309307098388671875f,0.8413999895811077323060089838691055774688720703125f,0.311200016039610005691429250873625278472900390625f,0.3804999996423721331240130894002504646778106689453125f,0.79470002068877221290676970966160297393798828125f,-0.8179999889135363133618739084340631961822509765625f,-0.12419999631047229371461071423254907131195068359375f,-0.0456000101804736601707190857268869876861572265625f,0.89340005237460218268097378313541412353515625f,-0.1644999899268153598086428246460855007171630859375f,0.99299997669458406335252220742404460906982421875f,-0.14339999180138107703896821476519107818603515625f,-0.689699994611740141436939666164107620716094970703125f,0.431400002060830567440774530041380785405635833740234375f,0.45749999571591615232790672962437383830547332763671875f,0.81720002202987684913892962867976166307926177978515625f,0.3098999888285991488601212040521204471588134765625f,0.486000018283724966039471837575547397136688232421875f,0.6866999785780905085630365647375583648681640625f,-0.418500004649162260506045640795491635799407958984375f,-0.2594999945461751167385955341160297393798828125f,0.05430000405460599921525499667041003704071044921875f,-0.4461000129520893420220772895845584571361541748046875f,-0.53909997970014789103743169107474386692047119140625f,-0.477299976092577349362500171991996467113494873046875f,-0.399300018583238214286978973177610896527767181396484375f,0.41879999912232179770654738604207523167133331298828125f,0.4470000028163194638608501918497495353221893310546875f,0.134399990732222807121587493384140543639659881591796875f,-0.450300003451108932228663661589962430298328399658203125f,-0.280499992787837992835875411401502788066864013671875f,0.5394000156730414108352533730794675648212432861328125f,0.9177000058770179435896352515555918216705322265625f,0.7311000057622789594091727849445305764675140380859375f,-0.0105000004619359987856341831502504646778106689453125f,-0.3767999865084885602328768072766251862049102783203125f,-0.991499996840953912169425166212022304534912109375f,0.9528000360846522465152474978822283446788787841796875f,-0.2973000215411190083614201284945011138916015625f,0.3237000174924731243919495682348497211933135986328125f,-0.556000022381544223293303730315528810024261474609375f,-0.20330000663250669123982561359298415482044219970703125f,-0.274500018052757088771187454767641611397266387939453125f,-1.3509000325560567290494873304851353168487548828125f,-1.08670006619691950078276931890286505222320556640625f,-0.2308000053286551889186739572323858737945556640625f,0.180600007796287709282978539704345166683197021484375f,-0.3849999698996544594820079510100185871124267578125f,-0.108799985748528893481079649063758552074432373046875f,-0.442699976894259439319512239308096468448638916015625f,0.21370000195801264197825730661861598491668701171875f,-0.846699984538555217028488186770118772983551025390625f,-0.529700012791156638058964745141565799713134765625f,-0.85029998512268090138377374387346208095550537109375f,-0.573099997514485703931086391094140708446502685546875f,-0.2902000082612037346052602515555918216705322265625f,0.42720001785755190581994611420668661594390869140625f,-0.82219998328685761634915252216160297393798828125f,-0.64080000324249208887295026215724647045135498046875f,-0.2446000110447403841362756793387234210968017578125f,-0.4006000314891344782353144182707183063030242919921875f,1.1943000144481654700712169869802892208099365234375f,0.6915000475049026817941921763122081756591796875f,-1.1317000104665755344512945157475769519805908203125f,-0.4375000366568571052994229830801486968994140625f,-1.27169998573064813029986908077262341976165771484375f,-0.5139000058233733625456807203590869903564453125f,0.81199997770786325190783827565610408782958984375f,1.1368000135540956563318104599602520465850830078125f,0.9355000115633007595761228003539144992828369140625f,0.15600001507997518501724698580801486968994140625f,-0.255300012056529579407282426473102532327175140380859375f,-0.06540000191926942374465170360053889453411102294921875f,-0.7719000285252930471102672527194954454898834228515625f,-0.488700000095366959840248455293476581573486328125f,0.3234000037595630150377701284014619886875152587890625f,-0.11490000674724587526043251273222267627716064453125f,-0.4313999903261660495701335094054229557514190673828125f,-0.679499996185302990170384873636066913604736328125f,0.123300001081824206483616990226437337696552276611328125f,-0.5855999809741974093668659406830556690692901611328125f,-0.552899998529255309875907187233678996562957763671875f,-0.611099988514185010757273630588315427303314208984375f,-0.43289998444765787066756956846802495419979095458984375f,-0.41339999381303815795263290056027472019195556640625f,0.6876000098302961216489848084165714681148529052734375f,-0.1251000090479852389080406283028423786163330078125f,-0.391199994470178946670557706966064870357513427734375f,-0.23909999426603345540343070751987397670745849609375f,-0.5358000232100488258168979882611893117427825927734375f,-0.2328000055372712040480109862983226776123046875f,0.8691000294327737574207048965035937726497650146484375f,1.0464000101447101798157746088691055774688720703125f,-0.1983000125929714752004429101361893117427825927734375f,-0.60809998753070804156095618964172899723052978515625f,-0.33929999120235432297221223052474670112133026123046875f,-0.7820999915242197175757610239088535308837890625f,0.82679998561441880866595965926535427570343017578125f,-0.018000024080276233462427626363933086395263671875f,0.7803000233441592659033858581096865236759185791015625f,0.2774999985098836052799242679611779749393463134765625f,-0.980999976933002670875794137828052043914794921875f,-0.85500001564621896932294475845992565155029296875f,0.955500008881092099954912555404007434844970703125f,0.17190001202225690946079339482821524143218994140625f,0.43859995409250274178702966310083866119384765625f,0.0097999856293196074830120778642594814300537109375f,-0.3355999948322772041819916921667754650115966796875f,-0.2604000089228153402842735886224545538425445556640625f,0.037200005005300074145679900539107620716094970703125f,0.1160000064000488162463398111867718398571014404296875f,-0.456400015807151504532157559879124164581298828125f,0.19600000390410432515864158631302416324615478515625f,-0.41479999173879633644901332445442676544189453125f,-0.841600014829635423296849694452248513698577880859375f,-0.57379999945759774249154361314140260219573974609375f,0.0225999953389166563511025742627680301666259765625f,-0.18070000008046616102319603669457137584686279296875f,-0.34630001517832298407739699541707523167133331298828125f,0.4668000131070613178962958045303821563720703125f,0.3578000268459324839653845629072748124599456787109375f,0.931800002229213664151075136032886803150177001953125f,0.617400025224685844449368232744745910167694091796875f,-0.566299993683397673294166452251374721527099609375f,-0.6459000225350262081036589734139852225780487060546875f,-0.342899989104270996875811761128716170787811279296875f,-0.2669000011444089270895574372843839228153228759765625f,-0.5758999589443209998762540635652840137481689453125f,-0.903900006121396426550518299336545169353485107421875f,-0.6757000186324120249992120079696178436279296875f,-0.06330001054406153571108006872236728668212890625f,-0.868799994353949944780879377503879368305206298828125f,-0.6059999825954438534836299368180334568023681640625f,-0.125100002230703832850622347905300557613372802734375f,0.476999995887279482076337444595992565155029296875f,0.36359998855143782492405080120079219341278076171875f,0.799799978327751404094669851474463939666748046875f,-0.3144000053912401337896653785719536244869232177734375f,-0.27479999963641166260686077293939888477325439453125f,-0.02999999899417160609260690762312151491641998291015625f,-0.17340000327527527179682920177583582699298858642578125f,0.36179999578446153662980577792041003704071044921875f,-0.30120002095699316413401902536861598491668701171875f,-0.80039999953508367269705559010617434978485107421875f,-0.0293999960780144675709379953332245349884033203125f,-0.11159999936968090228361916160793043673038482666015625f,-0.686399999713897646103077931911684572696685791015625f,-0.42750000961124900999976716775563545525074005126953125f,-0.13260000122487536344806358101777732372283935546875f,0.21960001119375238687325690989382565021514892578125f,0.14189999622702575976518346578814089298248291015625f,0.5286000069916247934287412135745398700237274169921875f,0.530399996703863063629569296608678996562957763671875f,-0.57330000957548621354220585999428294599056243896484375f,-0.20339999325424429343911469914019107818603515625f,-0.173699995923042249668100112103275023400783538818359375f,-0.11219999252557766311610976117663085460662841796875f,-0.9284999884441493822606616959092207252979278564453125f,-0.229199971127510337964849895797669887542724609375f,-0.6153000110507012010430116788484156131744384765625f,-0.37480001572966603617942382697947323322296142578125f,0.1260999895572663120901779620908200740814208984375f,-0.3455000255703932765527497394941747188568115234375f,0.2885000022649766293625361868180334568023681640625f,-0.3283000090539456294891351717524230480194091796875f,0.3042999907739460940803155608591623604297637939453125f,-0.170899998264759656052547143190167844295501708984375f,0.4170000090003014303619011116097681224346160888671875f,-0.183200010883808328543409515987150371074676513671875f,0.547500003576278260197796043939888477325439453125f,1.0847000509381299337974269292317330837249755859375f,0.193900012826919265762626309879124164581298828125f,0.597300026160478836345646413974463939666748046875f,-1.22450002077221853369337623007595539093017578125f,-1.0141000653445730872448393711238168179988861083984375f,-0.1683000096686184665628616130561567842960357666015625f,0.003899997311085368068006573594175279140472412109375f,-0.0383999971657988936613037367351353168487548828125f,-0.05700001130998166853913744489545933902263641357421875f,-1.200500021398067218569849501363933086395263671875f,-0.888500055611134431643449715920723974704742431640625f,0.150500001430511787248178734444081783294677734375f,-0.641900038993359345340650179423391819000244140625f,1.1164000141680237021546417963691055774688720703125f,1.0386000589966781770812076501897536218166351318359375f,0.45329998320043074500063084997236728668212890625f,0.53590001474916937329595612027333118021488189697265625f,-0.123800004971027277633766061626374721527099609375f,-0.54440003079772036898020814987830817699432373046875f,-1.02510001322031030213111080229282379150390625f,0.76619999651908887727813635137863457202911376953125f,-0.6591000160217286296671090894960798323154449462890625f,-0.1152000040888783161108221975155174732208251953125f,-0.4545000100806355636251510077272541821002960205078125f,0.23999999552965167737283991300500929355621337890625f,-0.06450000108405939958089447827660478651523590087890625f,0.461399988277256423163663612285745330154895782470703125f,-0.2070000101923943625070734242399339564144611358642578125f,0.4215000116676093089296273319632746279239654541015625f,0.807900032615661789492378375143744051456451416015625f,-0.204000006377697218340472318232059478759765625f,0.342300018560886432084089392446912825107574462890625f,-0.32729999322891245583377894945442676544189453125f,-0.725700027126074065364491616492159664630889892578125f,-0.428999986261128896103400620631873607635498046875f,0.05310000103488565770248897024430334568023681640625f,-0.099900004251301244284633185088750906288623809814453125f,-0.37980001487284908989749965257942676544189453125f,-0.3807000058919189910966451861895620822906494140625f,-0.732900025910139252260933062643744051456451416015625f,-0.60299999904632528568981797434389591217041015625f,-0.909900030291080685884708145749755203723907470703125f,0.0660000037550929619101225398480892181396484375f,0.8028000311672689459641105713672004640102386474609375f,0.380699994939565300455797114409506320953369140625f,0.8595000094100833099020064764772541821002960205078125f,0.588899999043345445670638582669198513031005859375f,-0.25620001536905778749542150762863457202911376953125f,0.41010001381635685646642741630785167217254638671875f,0.036899999803304606160736511810682713985443115234375f,-1.139500034630298586080243694595992565155029296875f,0.19400002440810215631472601671703159809112548828125f,-0.3992000124037264896514898282475769519805908203125f,-0.3788999794483185468152441899292171001434326171875f,-0.667100007319449961329382858821190893650054931640625f,-0.35639995322227502327905312995426356792449951171875f,-1.17640001044273301289422306581400334835052490234375f,0.955199987101554626178767648525536060333251953125f,1.20620003719925872331941718584857881069183349609375f,0.77309998067617424766240219469182193279266357421875f,-0.00829999489784238875245137023739516735076904296875f,0.0720999980807306428687297739088535308837890625f,0.604099997991323078849745797924697399139404296875f,-0.68179998971223820802833870402537286281585693359375f,-0.714000022619962759762302084709517657756805419921875f,-0.18339998252540823386880219914019107818603515625f,-0.295199999730288709098857680146466009318828582763671875f,0.1427000039212404924882093837368302047252655029296875f,0.331300005548447185077520771301351487636566162109375f,0.06259999608397492920630611479282379150390625f,0.45580000875592219244936131872236728668212890625f,-0.94919998252689818940552868298254907131195068359375f,-0.03000000424683069155662451521493494510650634765625f,0.5483999592423440816446600365452468395233154296875f,0.95560000228285701950881048105657100677490234375f,-0.6096999755382537244940976961515843868255615234375f,-0.06450000467896455091931784409098327159881591796875f,0.1732000284671784129386651329696178436279296875f,0.98120004062652643739284030743874609470367431640625f,-0.5339999879002572669151049922220408916473388671875f,0.51780002774000166709811310283839702606201171875f,-0.22560001070201407546989003094495274126529693603515625f,0.519000023365020712873274533194489777088165283203125f,-0.4778999976351856471268320092349313199520111083984375f,0.1083000142693517631187205552123486995697021484375f,-0.94679999299049366623393098052474670112133026123046875f,0.35880003536939586439302729559130966663360595703125f,0.838800017264485386903061225893907248973846435546875f,0.0230999872326851374282341566868126392364501953125f,0.10769999675005681449846406394499354064464569091796875f,0.84329998089075086653565449523739516735076904296875f,0.6578999902591107229454792104661464691162109375f,-0.091199991452693751625702134333550930023193359375f,-0.82500000983476640126212942050187848508358001708984375f,-0.49289999655485150498179791611619293689727783203125f,-0.40740000022798772416621204683906398713588714599609375f,-0.18869999685436500413970861700363457202911376953125f,0.00330000214353198817462953229551203548908233642578125f,-0.360899998469650651333751056881737895309925079345703125f,0.2345999991238116866298923923750407993793487548828125f,-0.358200009095668736591733249952085316181182861328125f,0.2483999971657990801787718737614341080188751220703125f,-0.5435999855339528341602317595970816910266876220703125f,0.6617999899730084845117517033941112458705902099609375f,-0.175800026488303728910977952182292938232421875f,-0.2223000023558736604201158115756697952747344970703125f,-0.71459998437166216689320208388380706310272216796875f,0.67560001226663590756515986868180334568023681640625f,-0.5423999984025957843414289527572691440582275390625f,-0.3941999936655162795062778968713246285915374755859375f,0.31970000906586637512418747064657509326934814453125f,-0.4705999997496608244773597107268869876861572265625f,-0.829000015616416785491082919179461896419525146484375f,-0.5626999832272527868326505995355546474456787109375f,0.08169999571442598806925161625258624553680419921875f,0.141999980092048616597821819595992565155029296875f,-0.2432000064134598460441338829696178436279296875f,-1.629399992203712344007726642303168773651123046875f,-0.8820000288188456494253841810859739780426025390625f,0.72730002481937372493803195538930594921112060546875f,0.85070005573630425033115898258984088897705078125f,0.11840002123117443488808930851519107818603515625f,0.61800002765655559500146409845910966396331787109375f,1.008199973332882137810884159989655017852783203125f,0.71640000746250098728751254384405910968780517578125f,0.479800015580654104496716172434389591217041015625f,1.01980003212094327835757212596945464611053466796875f,-1.129700021731853354367558495141565799713134765625f,-0.53350002282857911239943859982304275035858154296875f,0.45830001650452611983155293273739516735076904296875f,-0.04870000113844863420808906084857881069183349609375f,-0.46549997684359567529099876992404460906982421875f,-0.7979000089228149850129057085723616182804107666015625f,0.90189996925592452736708537486265413463115692138671875f,0.9063000054925673953931664073024876415729522705078125f,0.0675999849081041492837584883091039955615997314453125f,-0.3810000217854980775200601783581078052520751953125f,0.5317999640822410611917803180404007434844970703125f,0.3089999981820579222357991966418921947479248046875f,-0.3532999870002271336488774977624416351318359375f,0.797399995720386556286030099727213382720947265625f,-0.6005999910324810731054867574130184948444366455078125f,0.1272000110030173658515195711515843868255615234375f,-0.1257000064507127756741056145983748137950897216796875f,-0.7805999931931495439130230806767940521240234375f,-0.009600000688433585338543707621283829212188720703125f,0.482999990105628995706865680404007434844970703125f,-0.636600006186962108500893009477294981479644775390625f,-0.8966999922871590200657010427676141262054443359375f,0.65370003190934677927970142263802699744701385498046875f,0.13590000215768771596458464046008884906768798828125f,0.458400011247396488300864803022705018520355224609375f,-0.29579999430179615416136584826745092868804931640625f,0.472799989257752795168698867200873792171478271484375f,0.3890999769806864350130126695148646831512451171875f,0.840000010617077297325749896117486059665679931640625f,-0.13259999265670785462134517729282379150390625f,-0.302700006137788246718400841928087174892425537109375f,-0.5811000139951705278917870600707828998565673828125f,0.12689999745637170480705435693380422890186309814453125f,0.636900020813941925013068612315692007541656494140625f,-0.935099995413422480083909249515272676944732666015625f,-0.2939999895393849982383471797220408916473388671875f,0.88769998624473789305966420215554535388946533203125f,0.47819998149126796693764163137529976665973663330078125f,-0.3822000165537000615501028732978738844394683837890625f,0.1982999902039768169714761825161986052989959716796875f,0.5351999974206089749628745266818441450595855712890625f,0.63239998528957386270121787674725055694580078125f,-0.557699994850158642378801232553087174892425537109375f,0.693700016486644699398311786353588104248046875f,-0.957199982392788140117545481189154088497161865234375f,-0.584000001013278424721875126124359667301177978515625f,0.307500004917383107372330641737789846956729888916015625f,-0.42670000290423615307844329436193220317363739013671875f,-0.944799964320659935168578158481977880001068115234375f,-0.51620000277757593920568979228846728801727294921875f,-0.2072000010564922778399932212778367102146148681640625f,-0.258600012318790095644516213724273256957530975341796875f,-0.488400007820128934099557227455079555511474609375f,-0.794600044870377164585306672961451113224029541015625f,0.5772000111892818807746152742765843868255615234375f,0.447000025503337672372339284265763126313686370849609375f,0.100900005435943906917373169562779366970062255859375f,-0.65930004022717536571462915162555873394012451171875f,1.169199983346462801137022324837744235992431640625f,0.99180001116990990084332224796526134014129638671875f,-0.751199995028972722366233938373625278472900390625f,-0.2614000059723853297555251629091799259185791015625f,-0.8181000003457068459056245046667754650115966796875f,-0.548300020974874513512986595742404460906982421875f,-0.061299987387657406401331172673963010311126708984375f,0.7263000368595129430104861967265605926513671875f,0.749100008422136198760199476964771747589111328125f,0.9269000383973124979064550643670372664928436279296875f,0.47700001347064979739798218361102044582366943359375f,-0.341600013041496453070067218504846096038818359375f,0.8888000022143123235451867003575898706912994384765625f,0.91800004427135040163676649171975441277027130126953125f,-0.27180001742839809963925290503539144992828369140625f,-0.5331000057101249467450543306767940521240234375f,-0.483599986875057064050764665807946585118770599365234375f,-0.552599991017580283170218535815365612506866455078125f,-0.399899999222159296952128215707489289343357086181640625f,0.466500001702457589569661422501667402684688568115234375f,-0.422999985896050867406614770516171120107173919677734375f,-0.573899968868494259055523798451758921146392822265625f,-0.2124000058159232484644718397248652763664722442626953125f,-0.0440999973580240567372356963460333645343780517578125f,-0.621000032030046167363934728200547397136688232421875f,0.0363000005006792747508370666764676570892333984375f,0.10620000866353518365059471761924214661121368408203125f,-0.018900006978213923503062687814235687255859375f,-0.570300029155612175912892780615948140621185302734375f,0.431700016874075220840722977300174534320831298828125f,0.658199977393448154572297426057048141956329345703125f,0.335099974477291340235751704312860965728759765625f,-0.305399995176494076787321318988688290119171142578125f,-0.64409999665021899772909819148480892181396484375f,-0.2445000063925981681478560858522541821002960205078125f,-0.20729999322891234925236858543939888477325439453125f,0.767100024604797425809010746888816356658935546875f,-0.272399995571375175273942659259773790836334228515625f,0.669900014868378690380268380977213382720947265625f,0.060299994808435286586245638318359851837158203125f,-0.47760001206547031404880954141844995319843292236328125f,0.406500010848045434386222041212022304534912109375f,0.84600002208352098609367430981365032494068145751953125f,0.4013999934554097759331625638878904283046722412109375f,0.32640000608414421190417442630860023200511932373046875f,-0.0233999947071072966764404554851353168487548828125f,-0.700500017374753980448076617904007434844970703125f,-0.01430000044107426759865120402537286281585693359375f,0.16379999185800553362923892564140260219573974609375f,0.39280000821948046763054662733338773250579833984375f,0.48980000686347491267724763019941747188568115234375f,-0.2554000097423791970641104853712022304534912109375f,-0.106300031793117089051747825578786432743072509765625f,-0.69430004112124521498117246665060520172119140625f,0.898900009548663891933983904891647398471832275390625f,0.589700032758713188485444334219209849834442138671875f,1.08509994735717807401442769332788884639739990234375f,0.34669998781680977373298446764238178730010986328125f,-1.27919995592832602682165088481269776821136474609375f,-0.52119999189972787689839606173336505889892578125f,0.89249999262392520904541015625f,0.1011000115364790730154709308408200740814208984375f,0.3953999840006232613376369044999592006206512451171875f,-0.034999997802078741671039097127504646778106689453125f,-1.5142000220894811235439192387275397777557373046875f,-1.1900000584125525193712746840901672840118408203125f,-1.37379997606277459709644972463138401508331298828125f,-0.856400036221742766429088078439235687255859375f,-0.10149998335540288962874910794198513031005859375f,-0.18029999965131271455476280607399530708789825439453125f,-0.810600004220008774069583523669280111789703369140625f,-0.1728000083684921861504335538484156131744384765625f,0.342500007264316064237164027872495353221893310546875f,0.120700003246962950242959777824580669403076171875f,-0.205500010520219600351765620871447026729583740234375f,-0.616199993836879844621989832376129925251007080078125f,0.3027000066965819247144509063218720257282257080078125f,-0.277500008642673545722345807007513940334320068359375f,0.47100000241398809475157349879737012088298797607421875f,0.0968999969720840681475237943232059478759765625f,0.07619999167621127611482734209857881069183349609375f,0.9174000161349773474483981772209517657756805419921875f,-0.73650003144890074935347001883201301097869873046875f,0.105599975758791320146201542229391634464263916015625f,0.522900015851855304305217941873706877231597900390625f,0.535200001555681037501699393033050000667572021484375f,0.48629997814893710650352431912324391305446624755859375f,1.02779996843338050638294589589349925518035888671875f,-0.39179997611492856091075509539223276078701019287109375f,-0.83759996911287348808627939433790743350982666015625f,-0.00749999955296509757118883499060757458209991455078125f,0.6584999860227107593146911312942393124103546142578125f,0.0161999987170100834266150968687725253403186798095703125f,0.43319998465776443641317428046022541821002960205078125f,-0.8232000229910017008450040520983748137950897216796875f,-0.5420999992728230409966272418387234210968017578125f,-0.73380001078248036083806482565705664455890655517578125f,-0.81089997086524956415587439551018178462982177734375f,-0.5181000025436282019342115745530463755130767822265625f,-0.41100000099837785683121182955801486968994140625f,-0.258000001765787578822397563271806575357913970947265625f,-0.777300007981061913397979878936894237995147705078125f,-0.1005000012069940940051537836552597582340240478515625f,0.01950000266730789189040251585538499057292938232421875f,-0.283500014036893899227464999057701788842678070068359375f,-1.0733000142693516210101734031923115253448486328125f,0.6969000059127807844561175443232059478759765625f,0.7795000294148923813963847351260483264923095703125f,-0.366499992966652143877581693232059478759765625f,-0.1688999937534330086918998858891427516937255859375f,0.065800003875791990282095866859890520572662353515625f,0.2168000040546058659884920416516251862049102783203125f,0.27190003422498687513098047929815948009490966796875f,-0.5730999997496599718260767986066639423370361328125f,0.9198000117063518388249576673842966556549072265625f,1.043800060254336248277695631259121000766754150390625f,-0.4291000061869620907373246154747903347015380859375f,-0.5121000199854375267705108853988349437713623046875f,0.403700009930133774105343036353588104248046875f,0.769100033605099042688379995524883270263671875f,0.99829997135400783125902535175555385649204254150390625f,0.7345000180155036861862072328221984207630157470703125f,-0.871999984264373750875165569595992565155029296875f,-0.443800005120038587591579926083795726299285888671875f,-0.50129999029338367932950859540142118930816650390625f,-0.42570000604093050089460348317516036331653594970703125f,0.0068999944016336911545295151881873607635498046875f,0.139300003607571165620271358420723117887973785400390625f,-1.5133000193357464269183765281923115253448486328125f,-1.13250006914138889868581827613525092601776123046875f,0.493000008173286818902170125511474907398223876953125f,-0.146400008319318430949351750314235687255859375f,-0.90329999638795854810524588174303062260150909423828125f,-0.4027000129550695195490561673068441450595855712890625f,-0.869099983090162453436278156004846096038818359375f,0.3567000027179716425962396897375583648681640625f,0.48810000667870045987228877493180334568023681640625f,0.0266999913930892063262945157475769519805908203125f,-0.43169998796582209621419679024256765842437744140625f,-0.5730000029206276934701236314140260219573974609375f,0.006000001296400991890322984545491635799407958984375f,-0.25709999875128264346102469062316231429576873779296875f,-0.43349999559670682192091817341861315071582794189453125f,0.61470002590417838206349188112653791904449462890625f,0.8916000344619157136349940628861077129840850830078125f,0.2933999944090839306909401784650981426239013671875f,-0.2943000064015388606009082650416530668735504150390625f,0.03870000307559973151683152536861598491668701171875f,0.38550001227110630708239114028401672840118408203125f,-0.3429000099658967570803724811412394046783447265625f,0.731099998944997775396359429578296840190887451171875f,0.64589997567087420460296698365709744393825531005859375f,-0.25199999722838395399548971909098327159881591796875f,-0.42149999177455921284263240522705018520355224609375f,-0.26669999612420791645917006462696008384227752685546875f,-0.12539999111592781133595053688623011112213134765625f,0.32730000392049551916073824031627736985683441162109375f,0.169200001637637598150831763632595539093017578125f,-0.8145000325441362765133135326323099434375762939453125f,-0.6173999909520144768748650676570832729339599609375f,-0.0030000033304096174191499812877736985683441162109375f,0.66750002004206188122026333076064474880695343017578125f,-0.378299999628961047815067786359577439725399017333984375f,-0.65489999650269747899500316634657792747020721435546875f,-0.1964999919980764120364113978212117217481136322021484375f,0.6264000192344187922799392254091799259185791015625f,-0.363700003373622848812374286353588104248046875f,0.069499998420476938321144189103506505489349365234375f,0.6865999954581258890584649634547531604766845703125f,1.007800038468837922067677936865948140621185302734375f,1.596600000524521334455130272544920444488525390625f,0.723600003415345582880036090500652790069580078125f,-0.303700012034922917081303239683620631694793701171875f,-0.2419000143133105684700723259084043093025684356689453125f,0.42569997363090550379638443700969219207763671875f,0.299699998039006931804806299624033272266387939453125f,-0.00059995713233984560019962373189628124237060546875f,-0.54879998872876090132422177703119814395904541015625f,-0.5241999824523926321262479177676141262054443359375f,-0.415600006157159640451936866156756877899169921875f,0.695099997760355581277735836920328438282012939453125f,0.0257000152423980665616909391246736049652099609375f,-0.0819000100851059453788138853269629180431365966796875f,0.411300015103817084849424645653925836086273193359375f,0.2555000053048133157318488883902318775653839111328125f,-0.465700003582238952759553285432048141956329345703125f,-0.72530000389814386352327346685342490673065185546875f,0.13590000707507154942277338705025613307952880859375f,-0.142100003482401149312863708473742008209228515625f,-0.002700002561509540210948898675269447267055511474609375f,-0.193300004658102986265788558739586733281612396240234375f,-0.04510000156015159245015411215717904269695281982421875f,0.2531000212073326594008904066868126392364501953125f,0.39190002081394226252086809836328029632568359375f,-0.119300051999091660803742342977784574031829833984375f,-0.5691000216841697323388871154747903347015380859375f,0.279300000478327259401112314662896096706390380859375f,-0.12059999766349793759445674368180334568023681640625f,0.6750000176578760946455304292612709105014801025390625f,-0.1122000139832497467295979731716215610504150390625f,0.409799974080919948704604394151829183101654052734375f,0.8462999954342844688426339416764676570892333984375f,0.067499999385327080769769736434682272374629974365234375f,0.15990000631883749537820449404534883797168731689453125f,0.55949998616427176756360495346598327159881591796875f,0.5765999808549882033048561424948275089263916015625f,-0.61499998256564125664880293697933666408061981201171875f,0.12270002584457362360126353451050817966461181640625f,-0.5969999985694884880871313725947402417659759521484375f,-0.5360999934315682224905685870908200740814208984375f,-0.15210000035315740518626625998876988887786865234375f,0.42629997881948933358131625936948694288730621337890625f,0.3615000088736415317924866030807606875896453857421875f,-0.2802000123590231428494234933168627321720123291015625f,-0.39209999957084651356353788287378847599029541015625f,0.4860000020414589894102164180367253720760345458984375f,0.27930001120716341223015888317604549229145050048828125f,-0.55140000209808359699081847793422639369964599609375f,0.24329999325871465298831708423676900565624237060546875f,0.149699996511638122598242262029089033603668212890625f,-0.049499999463558173873334311565486132167279720306396484375f,-0.165900004597753281121441659706761129200458526611328125f,0.0519000042736530442510911598219536244869232177734375f,-0.29520000051260009144016294158063828945159912109375f,0.5666999812602995234556146897375583648681640625f,-0.9162000547826298912923448369838297367095947265625f,-0.5326000042796135147682434762828052043914794921875f,-0.04500000804662729336769189103506505489349365234375f,0.57049996685981785304875302244909107685089111328125f,0.82789999998211794007829666952602565288543701171875f,0.82880000392794606067781160163576714694499969482421875f,0.0476000055462122428906468485365621745586395263671875f,1.354999985992908051457561668939888477325439453125f,0.87660005103945781712582174804992973804473876953125f,-0.1484000173568720271077836514450609683990478515625f,-0.74300003227591560062137432396411895751953125f,0.9308000011563297704242359031923115253448486328125f,1.084000043928623480127271250239573419094085693359375f,-1.027999990701675248061519596376456320285797119140625f,-0.690200034785271032689024650608189404010772705078125f,-0.231699998545646934644537395797669887542724609375f,0.778700053441525597008876502513885498046875f,0.20349998944997782501786787179298698902130126953125f,-0.47610001056790363094251006259582936763763427734375f,-0.1665999692320827563207785715349018573760986328125f,-1.01120000218152927828896281425841152667999267578125f,0.419999990761279917705905972979962825775146484375f,-0.103600005203485778793037752620875835418701171875f,0.89700002121925326292739555356092751026153564453125f,0.3566000371813782265917325275950133800506591796875f,0.23140004024505600455086096189916133880615234375f,-0.55639999986290877842520785634405910968780517578125f,0.49279997975826272949007034185342490673065185546875f,0.723600015634298099342913701548241078853607177734375f,0.29979997475147257546268519945442676544189453125f,-0.329400002932548119360944838263094425201416015625f,-0.2970000010281801383626998358522541821002960205078125f,-0.75780000479221332199131211382336914539337158203125f,0.86729998436272126571822127516497857868671417236328125f,0.26849998110532791173454825184307992458343505859375f,0.07739999888688324503061011228055576793849468231201171875f,0.77640000429600475317926111529232002794742584228515625f,0.714000029884279019398718446609564125537872314453125f,0.740999979019164811688824556767940521240234375f,-0.9336000309079885983720714648370631039142608642578125f,-0.083700028407573512367889634333550930023193359375f,0.8352000241309405215162087188218720257282257080078125f,0.179399996674060702162023517303168773651123046875f,-0.707400015874207088728553571854718029499053955078125f,-0.712499987035989601480423516477458178997039794921875f,1.044300036129355735425860984832979738712310791015625f,-0.192600013220310728456752258352935314178466796875f,-0.54630000329464667441214942300575785338878631591796875f,0.27899998998641972747236650320701301097869873046875f,-0.8471999850377438523452156005077995359897613525390625f,0.39870002244710889272027998231351375579833984375f,-0.317400000451505182041955777094699442386627197265625f,0.179999995827675007831203402020037174224853515625f,0.52020002167224899114472691508126445114612579345703125f,0.87900001055002174865649067214690148830413818359375f,-0.42899999602138993992639370844699442386627197265625f,0.557700030910968536090877023525536060333251953125f,0.52620000005811451426751546023297123610973358154296875f,-0.03360002114772786541152527206577360630035400390625f,0.30479999430924653580632366356439888477325439453125f,-0.184000003546476165183776174671947956085205078125f,-1.301700000631809128748272996745072305202484130859375f,-0.59690002751946469317090304684825241565704345703125f,-0.123799974870681950278594740666449069976806640625f,-0.424999986588954214994373614899814128875732421875f,0.889599975281953891936836953391321003437042236328125f,0.0942000034630297022886225022375583648681640625f,-0.4604999930858610923678497783839702606201171875f,-0.150300008815527075256568423355929553508758544921875f,-0.0540999763846397030420121154747903347015380859375f,-0.54190001336336113268998815328814089298248291015625f,-1.131500027537345687278502737171947956085205078125f,-0.4467000423550615550993825308978557586669921875f,-0.47530000121593474204928497783839702606201171875f,-0.794500028073787945004369248636066913604736328125f,1.09150001442432387221970202517695724964141845703125f,0.36630002374649084373459118069149553775787353515625f,0.26939999935626968152746485429815948009490966796875f,0.796800040525198483010171912610530853271484375f,0.594400003677606481033990348805673420429229736328125f,0.4908000245511534576081658087787218391895294189453125f,1.10210002311468091562574045383371412754058837890625f,0.57970003886818943783509894274175167083740234375f,-0.8881000003457071301227188087068498134613037109375f,-0.41750001788139368130714501603506505489349365234375f,0.5631999748229983282499233609996736049652099609375f,0.261599993073939884880019235424697399139404296875f,0.0785000008121132797356267474242486059665679931640625f,-0.02350000163167720312884512168238870799541473388671875f,-0.59510000189542733295411380822770297527313232421875f,-0.53879997129440315717374687665142118930816650390625f,-0.547500008158385842449433766887523233890533447265625f,-0.921599997097253709199549120967276394367218017578125f,-0.528599986428022194928644239553250372409820556640625f,-0.077399996614456512133983778767287731170654296875f,0.143399995042383709975553074400522746145725250244140625f,0.83729997907280928615136872394941747188568115234375f,-0.366900006285309832065877344575710594654083251953125f,-0.61350000077486033234208662179298698902130126953125f,-0.1791000058516860615753785168635658919811248779296875f,0.55770002703666676779903355054557323455810546875f,-0.756300020393729344192479402408935129642486572265625f,-1.22700001496076538387569598853588104248046875f,-0.84810000980794431058029658743180334568023681640625f,-0.209399997866153597669836017303168773651123046875f,0.44190000986158850526663854907383210957050323486328125f,0.95400001144409163345017077517695724964141845703125f,0.59460002125203625933380635615321807563304901123046875f,-0.27030001671314263234080499387346208095550537109375f,0.403200010585784973660139485218678601086139678955078125f,0.268199996280670038828475298942066729068756103515625f,0.42270002111345539308473462369875051081180572509765625f,0.62280001075267765742182746180333197116851806640625f,-0.6158999972775578424233344776439480483531951904296875f,-0.88829999489784228217104100622236728668212890625f,0.5435999853104351853261277938145212829113006591796875f,0.741599989199638542913817218504846096038818359375f,-0.0627000012204051160313866830620099790394306182861328125f,0.03600000114738943768344370255363173782825469970703125f,-0.74190002886056916064916322284261696040630340576171875f,-0.2168000198870900785408366573392413556575775146484375f,0.5232999964624642785793184884823858737945556640625f,0.50610001287758332733091037880512885749340057373046875f,-0.327299987454712226764286242541857063770294189453125f,-0.16409999847561085317693141405470669269561767578125f,0.0817999533534052858385621220804750919342041015625f,0.94660001050829833957322989590466022491455078125f,-0.095999979317188177674324833787977695465087890625f,0.43740002656579068940345678129233419895172119140625f,1.4067999864339828519632646930404007434844970703125f,0.55840002167820923517638220801018178462982177734375f,-1.6316000106573103067830743384547531604766845703125f,-0.8692000369906427437172169447876513004302978515625f,-0.30800000143051164958052368092467077076435089111328125f,0.3574000199347737716237816130160354077816009521484375f,-0.099299989600479587892323252162896096706390380859375f,0.114500003896653712587294648983515799045562744140625f,1.3048999975085262548191167297773063182830810546875f,0.83030001045465429143632718478329479694366455078125f,-0.805099981033801981311626150272786617279052734375f,-0.2309000054359433562467529554851353168487548828125f,-0.0395999866068361683346665813587605953216552734375f,-0.0383999972403048506208733670064248144626617431640625f,-0.935700012969970629228555480949580669403076171875f,-0.29550001499056843812240913393907248973846435546875f,-0.20149998611211739074633442214690148830413818359375f,-0.85210004636049330173364069196395576000213623046875f,-0.9066999869227405639549033367075026035308837890625f,-0.43010002812147174466872456832788884639739990234375f,0.4106999845087528910880791954696178436279296875f,-0.59400000470131619767499842055258341133594512939453125f,0.153900002731382823384365110541693866252899169921875f,-0.10260000748336306486407920601777732372283935546875f,0.0248999978810549027485876649734564125537872314453125f,-0.02639999710619467254701930869487114250659942626953125f,0.7847999940857290912532562288106419146060943603515625f,-0.4506000417709348226935617276467382907867431640625f,0.15840000811815269088356217253021895885467529296875f,-0.56250000625848795010597314103506505489349365234375f,0.3102000009968877147770172086893580853939056396484375f,0.80639998064041140679591990192420780658721923828125f,-0.5616000113278627470236870067310519516468048095703125f,-0.8328000012159346709950114018283784389495849609375f,0.348600016714632643388682708973647095263004302978515625f,-0.42150000566989198613754297184641472995281219482421875f,-0.01289999919980766573956998399808071553707122802734375f,-0.27689999585449698304984167407383210957050323486328125f,0.5714999811574814625458884620456956326961517333984375f,0.6026999912738801867817528545856475830078125f,-0.095699999521672740598887685337103903293609619140625f,-0.5012999855995179387946336646564304828643798828125f,-0.3588000049710273753333922286401502788066864013671875f,-0.38580000547766690743856088374741375446319580078125f,-0.603300004434585535051382976234890520572662353515625f,-1.1001000182867048948764931992627680301666259765625f,-0.85890003312975193860978606608114205300807952880859375f,0.11550001901388196046127632143907248973846435546875f,-0.6561000216320158617833158132270909845829010009765625f,-0.9320999959945677204586900188587605953216552734375f,0.603900001055002189076503782416693866252899169921875f,-0.02630001182556185312932939268648624420166015625f,-1.2253999953627587871096693561412394046783447265625f,-0.94720002173185324778614813112653791904449462890625f,-0.98820001147985436773524270392954349517822265625f,-0.83420004846453732483269050135277211666107177734375f,0.2407000026136636439133553722058422863483428955078125f,-0.13149999855458727182622169493697583675384521484375f,0.5155999905109407421832656837068498134613037109375f,-0.3140000185966496104583711712621152400970458984375f,-1.015299988065660130587275489233434200286865234375f,-0.9759000252917406914576758936163969337940216064453125f,-0.2481999888300894241410787799395620822906494140625f,0.082199996697902566467064389144070446491241455078125f,0.959099993073940115806408357457257807254791259765625f,0.3161000205516817374018501141108572483062744140625f,-0.101200003969669438674827688373625278472900390625f,0.5774000319302086126072026672773063182830810546875f,-0.434699983417242918903866666369140148162841796875f,-0.46450000335648622407092034336528740823268890380859375f,-0.68689996299743683039196184836328029632568359375f,-0.866100004011391977343237158493138849735260009765625f,-1.1670999801248314042823039926588535308837890625f,-1.077300020423531012880857815616764128208160400390625f,0.200799975824356380371682462282478809356689453125f,0.76520000557899425785990388249047100543975830078125f,-0.4109000065535308721109686302952468395233154296875f,-0.30950001399219051467781582687166519463062286376953125f,-0.30449998545646650427443091757595539093017578125f,-0.695700011628865855328740508412010967731475830078125f,-0.6999000102676451451344519227859564125537872314453125f,-0.41189999756813033826574610429815948009490966796875f,-0.6311999953642486449467696729698218405246734619140625f,-0.4226999897837639963427136535756289958953857421875f,-0.63240002257972971477784085436724126338958740234375f,-0.37559999229907958806506940163671970367431640625f,-0.1544999994635581630486598214702098630368709564208984375f,0.24209999919831748371734647662378847599029541015625f,-0.4404000178635121276471409146324731409549713134765625f,0.4427999928712846866574182058684527873992919921875f,-0.72089999951273198153245402863831259310245513916015625f,-0.2888999823912978826001562993042171001434326171875f,-0.1650000067055226171675030855112709105014801025390625f,-0.537599997723102518421001150272786617279052734375f,0.395700010026991455536204966847435571253299713134765625f,0.855299994212388892123044570325873792171478271484375f,0.28320001751482504204204815323464572429656982421875f,-0.63510002275705357277502116630785167217254638671875f,-0.445499993439763708469314451576792635023593902587890625f,-0.20909998903200044395589429768733680248260498046875f,-0.91589998721927390334940355387516319751739501953125f,-0.3731999772071841192655483609996736049652099609375f,-0.7736999935761093016850509229698218405246734619140625f,-0.336299978968501278586700209416449069976806640625f,0.507599989601969614483323312015272676944732666015625f,-0.384600024604797141591916442848742008209228515625f,-0.110099996978044512019323519780300557613372802734375f,-0.03869999018609515673006171709857881069183349609375f,-0.4755000075772404244389690575189888477325439453125f,0.22200000452995283239943091757595539093017578125f,0.0777000033996999572849517790018580853939056396484375f,-0.49869998660981573745942796449526213109493255615234375f,-0.785699961709976690826806589029729366302490234375f,-0.238499985575675310656151850707828998565673828125f,-0.29740001822113981688744388520717620849609375f,0.508600010567903382252552546560764312744140625f,-0.574700003403425085934941307641565799713134765625f,0.11810000377297402707199580618180334568023681640625f,0.2443999995984136308635470413719303905963897705078125f,-0.143200008555501856477576438919641077518463134765625f,0.119599994206428306142697692848742008209228515625f,0.981000054568053059256271808408200740814208984375f,0.258600010418891912422623136080801486968994140625f,0.51200002092123053643035746063105762004852294921875f,0.19120000216290355954384949654922820627689361572265625f,-0.00220000021830202285855193622410297393798828125f,0.45440000613629816239580350156757049262523651123046875f,-0.1286000014781951694686767950770445168018341064453125f,0.86309998991489411679367549368180334568023681640625f,0.504700014728307611022728451644070446491241455078125f,0.5841000000774858591512384009547531604766845703125f,-0.17689999801516531618972294381819665431976318359375f,0.68640001796782001974861486814916133880615234375f,0.65060003871619753734734104000381194055080413818359375f,0.4722999798178673103166147484444081783294677734375f,0.175299995702504762817852679290808737277984619140625f,0.1560000213384629130786152018117718398571014404296875f,-0.488200020867586292894202415482141077518463134765625f,-0.1432000050537289848051614171708934009075164794921875f,-0.33840002016946701957067489274777472019195556640625f,0.3773999954223636876804448547773063182830810546875f,0.7641000032067299940763405174948275089263916015625f,-0.222899990780651702237946665263734757900238037109375f,-0.448299995791912042619742351234890520572662353515625f,-0.120000030994415141094577847979962825775146484375f,0.3018999859452247136459845933131873607635498046875f,-0.416400019979477065135142765939235687255859375f,-0.85460001834630983097440548590384423732757568359375f,-0.986999959528446879630791954696178436279296875f,-1.09539999089241035790109890513122081756591796875f,-0.69959998854398719458913546986877918243408203125f,-0.746900028860569165090055321343243122100830078125f,0.57960002527534992822211279417388141155242919921875f,0.22180002342611537802241628014598973095417022705078125f,1.137300006937980612065075547434389591217041015625f,1.0272999982953070485791613464243710041046142578125f,0.90869998310804334096246748231351375579833984375f,0.7188000016927720281501024146564304828643798828125f,0.1602000270366676915045900386758148670196533203125f,0.426600014233589064360785414464771747589111328125f,1.461599991583824476037989370524883270263671875f,1.472400015091896108287983224727213382720947265625f,0.225900011992454352593995281495153903961181640625f,-0.0627000070691110522602684795856475830078125f,0.216300024640560195621219463646411895751953125f,0.602500010430812693584812222979962825775146484375f,-0.512999976098537757707163109444081783294677734375f,-0.769200022387504844800787395797669887542724609375f,-0.738900040245056288767955265939235687255859375f,-0.95970001636743518247385509312152862548828125f,-0.42839999329149680562522917171008884906768798828125f,-0.1845999933123589897832061978988349437713623046875f,0.210500011563301114847490680404007434844970703125f,0.4455999924480915641566980411880649626255035400390625f,-0.5192000114351511541599393240176141262054443359375f,-0.170599999824166292938798505929298698902130126953125f,-1.1388000376418236925246674218215048313140869140625f,0.1494000030815600954525734778144396841526031494140625f,-0.947100017154216544668088317848742008209228515625f,-0.1903999916374683465392081416212022304534912109375f,0.810899998432397683956196487997658550739288330078125f,-0.08279999078512201293733596685342490673065185546875f,-0.54380000392794602959156691213138401508331298828125f,0.52740000033974643312006946871406398713588714599609375f,0.352199981200694622884839191101491451263427734375f,0.24899999251961713042646806570701301097869873046875f,0.05590000521242632913754277979023754596710205078125f,-0.320199997755885146233367777313105762004852294921875f,-0.295900001449882932291757242637686431407928466796875f,-0.0093999973073601861273118629469536244869232177734375f,0.465500005453824972079246435896493494510650634765625f,0.05799999964237212424222889239899814128875732421875f,-0.2105000005364416892916779033839702606201171875f,-0.6296000050842760487057603313587605953216552734375f,-0.46820000686049478844097393448464572429656982421875f,0.15139999434947970513576365192420780658721923828125f,0.1219000121712685569264067453332245349884033203125f,0.71119999130368238837718308786861598491668701171875f,-0.11650000816583627738509676419198513031005859375f,0.23240001404881471813723692321218550205230712890625f,-0.364899990952014974254780099727213382720947265625f,0.7926999969363208720096736215054988861083984375f,0.77339997116327285908710109652020037174224853515625f,0.40729999233484281973005636245943605899810791015625f,-0.131699988599121464716290574870072305202484130859375f,0.2182000023901462526509931194595992565155029296875f,0.68070002585947531059673565323464572429656982421875f,1.079800023217499127525798030546866357326507568359375f,-0.06659995820522368603633367456495761871337890625f,0.398100018823146939439538982696831226348876953125f,-0.409799992930889089848278672434389591217041015625f,-0.655299983483553116769826374365948140621185302734375f,1.128300002199411178338550598709844052791595458984375f,0.924999999403953498955388567992486059665679931640625f,0.2966999903500084911911471863277256488800048828125f,-0.1041999902009962397642084397375583648681640625f,-0.693299984578788386357928175129927694797515869140625f,-0.63530001149773607238557815435342490673065185546875f,0.1287000038020311709630050245323218405246734619140625f,0.339100004920363407023842228227294981479644775390625f,-0.0738000060141086322573755751363933086395263671875f,-0.349100001865625397812209484982304275035858154296875f,-1.0493999975681305159014300443232059478759765625f,-0.43929999865293467564697493799030780792236328125f,0.52349998371303119171216167160309851169586181640625f,0.76059999825954438534836299368180334568023681640625f,1.253399970865249457574464031495153903961181640625f,0.6630999914050104138141250587068498134613037109375f,0.642000010490416883612851961515843868255615234375f,0.831500045120716180235831416212022304534912109375f,1.261499947667122256689253845252096652984619140625f,-0.4222000016152858137274961336515843868255615234375f,0.19480001002997171521968766683130525052547454833984375f,-0.0123999986261129624409704774734564125537872314453125f,1.3570000593662268073558152536861598491668701171875f,-0.09539999938607213181285260361619293689727783203125f,0.25779999912977213938347631483338773250579833984375f,-0.017399991250038215895301618729718029499053955078125f,-0.19270001988410978555066321860067546367645263671875f,0.59239999855160707653567442321218550205230712890625f,0.0575999957859512079494379577226936817169189453125f,0.41359998583197599142380340708768926560878753662109375f,0.5300000068545340781156482989899814128875732421875f,0.285799995642900483971970970742404460906982421875f,0.75269999842643731113867033855058252811431884765625f,0.0538000064611434680728052626363933086395263671875f,0.3797999974012373769483019714243710041046142578125f,-0.0657999874472618984100336092524230480194091796875f,0.8955999899148938681037179776467382907867431640625f,-0.129399986839294545148959514335729181766510009765625f,0.047600018137693655972952910815365612506866455078125f,0.1751999987244605705427602515555918216705322265625f,-0.54760003616809882487359573133289813995361328125f,-0.616000001221895132630379521287977695465087890625f,0.9908000384092332524232915602624416351318359375f,0.106199991303682406140751481871120631694793701171875f,0.997500006258487559307468472979962825775146484375f,0.238800004673004107047518118633888661861419677734375f,-1.088200001645088121904336730949580669403076171875f,0.3764000006079672999703689129091799259185791015625f,-0.17760003139972724284234573133289813995361328125f,-0.99290000221729268758963371510617434978485107421875f,-0.112800000806152933563453188980929553508758544921875f,-0.22700000799447306309986061023664660751819610595703125f,-0.617100032949448262797886854968965053558349609375f,-1.0782000278711318941304853069595992565155029296875f,-0.0290999963521958449064186424948275089263916015625f,0.03640001848936069706041962490417063236236572265625f,1.086000003457069595924622262828052043914794921875f,0.61410000767707817459495345246978104114532470703125f,0.794999959468842387622089518117718398571014404296875f,0.566199986684322364993704468361102044582366943359375f,0.3254999819099904101449283189140260219573974609375f,-0.1380000044107436707463421043939888477325439453125f,-0.140699995982647152459321659989655017852783203125f,-0.46449997711181634230115378159098327159881591796875f,-0.08009999802112588440650142729282379150390625f,0.1104000308275221442499969271011650562286376953125f,-0.5114999834299087666522609652020037174224853515625f,-0.7357999686717988652162603102624416351318359375f,0.22319999598264672613368020392954349517822265625f,-0.070400023525953425718171274638734757900238037109375f,-0.88979997892379714130584034137427806854248046875f,-0.11379997367858862844514078460633754730224609375f,-0.0495000516176222760122982435859739780426025390625f,-0.3764000229597090907418532879091799259185791015625f,0.1745999843716621313660652958787977695465087890625f,-0.2822999822020531013322397484444081783294677734375f,1.10219999938011170570462127216160297393798828125f,1.352999987721442920474146376363933086395263671875f,-1.24199997830390884701046161353588104248046875f,0.08040000251531598252086041611619293689727783203125f,0.6893000058054923329109442420303821563720703125f,0.23539999431967739695892305462621152400970458984375f,-0.9111000232338906101858810870908200740814208984375f,-0.025800001454353349572556908242404460906982421875f,0.229500022709370199436307302676141262054443359375f,-0.6933999850213528048925581970252096652984619140625f,0.164600000911951038773395339376293122768402099609375f,0.157200001466274219552587965154089033603668212890625f,-0.235000006332993560675959088257513940334320068359375f,0.215200000214576692147971925805904902517795562744140625f,0.3987000015854835766049291123636066913604736328125f,0.070200020331144141749746268033050000667572021484375f,0.46820000134706507566306754597462713718414306640625f,-0.11919999526739122330809550476260483264923095703125f,0.114300013852119519697225769050419330596923828125f,0.4765999941170215858932124319835565984249114990234375f,0.0714000051528216328478038121829740703105926513671875f,0.088000000387430166171043310896493494510650634765625f,-0.502500025779009096282834434532560408115386962890625f,0.066800003868341395474317323532886803150177001953125f,-0.73880002076625839180223920266143977642059326171875f,-0.1496000007629394445984871708787977695465087890625f,0.53850000625103711104912918017362244427204132080078125f,-0.0217999985039234456252188465441577136516571044921875f,0.359399995929002624706072310800664126873016357421875f,-0.0391999986946582890112722452613525092601776123046875f,0.48069998614788023161281671491451561450958251953125f,0.03019999067783363244643624057061970233917236328125f,0.94030003876686141239815697190351784229278564453125f,0.284899997061490939387340404209680855274200439453125f,0.137099984669685426297291996888816356658935546875f,-0.429700003850459921750370995141565799713134765625f,0.12510002961158761536353267729282379150390625f,0.2804999951720237305607952293939888477325439453125f,-1.29809997560978018782407161779701709747314453125f,-0.792699992763996164057971327565610408782958984375f,0.2046000004649162207215340458787977695465087890625f,-0.028600001031160449116441668593324720859527587890625f,0.4536000034898519661652471768320538103580474853515625f,0.3562000009894370489149650893523357808589935302734375f,0.1137000167846675680038970313034951686859130859375f,0.10930003024339640660400618799030780792236328125f,-0.475199982333183612581706256605684757232666015625f,-0.75499999433755871081075383699499070644378662109375f,0.85889997348785396269477132591418921947479248046875f,0.50289998880624775523529024212621152400970458984375f,0.0726000035345553129673135117627680301666259765625f,-0.1360000082999468506983475890592671930789947509765625f,0.2760000216364861813644893118180334568023681640625f,0.405500017076730667753281522891484200954437255859375f,-0.0263999843657014565678764483891427516937255859375f,0.38560000571012498227219111868180334568023681640625f,-0.26370000091493128735464779310859739780426025390625f,-0.54390000310391195537107478230609558522701263427734375f,-0.18420000048279749904622804024256765842437744140625f,-0.287599991613626482234167269780300557613372802734375f,-0.218100004816054848788553499616682529449462890625f,-0.63530000061988811665969478781335055828094482421875f,-0.975000019669533202204547706060111522674560546875f,-0.642599994891881909353514856775291264057159423828125f,-0.384399980282783104712507338263094425201416015625f,0.2463999903261662183240332524292171001434326171875f,-0.230100007855892130237407400272786617279052734375f,-0.081799995970726069316469875047914683818817138671875f,0.023199993114173533825805861852131783962249755859375f,-0.35159998651742940811715243398793973028659820556640625f,-0.198900007611513274241588078439235687255859375f,-0.237200007128715473214697340154089033603668212890625f,-0.898000011563301114847490680404007434844970703125f,0.4479999917447567980843814439140260219573974609375f,-0.57170000599622738945981836877763271331787109375f,0.4387999855995179387946336646564304828643798828125f,1.241000055164099169502378572360612452030181884765625f,-0.10000000998377789329651932348497211933135986328125f,0.148200010772049584062415306107141077518463134765625f,0.0198000012755394083630022805664339102804660797119140625f,-0.3821999994546174672649385684053413569927215576171875f,0.434999997317790931816716692992486059665679931640625f,1.02740004698038145392047226778231561183929443359375f,0.10719999833703042213528533466160297393798828125f,-0.26639999196529373648445471189916133880615234375f,0.0767999919474124936868975055404007434844970703125f,-1.124100036436319616228729501017369329929351806640625f,0.09159999828934672905234037898480892181396484375f,0.0558000028699636718698684489936567842960357666015625f,-0.489799998146295489931389965931884944438934326171875f,0.5659000134825706851415816345252096652984619140625f,-0.501600003653764670019654658972285687923431396484375f,0.8208000384092333234775651362724602222442626953125f,-0.971300001394748591110328561626374721527099609375f,0.4289999935626980231972993351519107818603515625f,0.136199963140487767532249563373625278472900390625f,-0.1892999739170075912397805950604379177093505859375f,-0.2413000025868416997809617896564304828643798828125f,-0.68399996342510027691474761013523675501346588134765625f,-0.49059998149573813552848378094495274126529693603515625f,-0.44519999775588470214415792725048959255218505859375f,-0.3257000153541564912984540569595992565155029296875f,1.19339998695850368903847993351519107818603515625f,1.355800006818771663574807462282478809356689453125f,1.01939997372627288996227434836328029632568359375f,0.9708999902367594359020586125552654266357421875f,-0.592200049746036683018246549181640148162841796875f,-1.012400039976835142141453616204671561717987060546875f,0.01529999462217068728620006368146277964115142822265625f,-0.00900000019371505199927696594386361539363861083984375f,0.8378999999821186150938956416212022304534912109375f,0.5183999828606842275036115097464062273502349853515625f,-0.154200031328201703217928297817707061767578125f,-0.66360002740621570893608804908581078052520751953125f,0.03540000340938576073313015513122081756591796875f,-0.2570000379085541908352752216160297393798828125f,0.5652000195860864550922997295856475830078125f,0.992700021225214168651973523083142936229705810546875f,-0.7827000042378902122663930640555918216705322265625f,-0.2077999889224766416617740105721168220043182373046875f,-1.1214000092506406502934623858891427516937255859375f,-1.004500014364719362447431194595992565155029296875f,0.1949999463558196310941639239899814128875732421875f,-0.67800000068545340781156482989899814128875732421875f,0.221199979531764512330482830293476581573486328125f,-0.4604000040054321285509786321199499070644378662109375f,0.59819998763799642205185591592453420162200927734375f,-0.2797999957621097877336069359444081783294677734375f,0.7619999723434442984171255375258624553680419921875f,-0.51040000221729275864390729111619293689727783203125f,-1.2519000002503393176311874412931501865386962890625f,0.40399998804926884332644476671703159809112548828125f,-0.2428000089645383496872454998083412647247314453125f,-0.54779998318552980407503127935342490673065185546875f,-0.11369999562501931222868734039366245269775390625f,0.0869999896287918517145953956060111522674560546875f,0.37750000596046451306619928800500929355621337890625f,0.261200000542402221981319598853588104248046875f,0.02049998634308558553840384774957783520221710205078125f,0.1476000024914740915704669532715342938899993896484375f,-0.575199995855987022963518029428087174892425537109375f,0.47659998651742940811715243398793973028659820556640625f,-0.426999998867511720845868694595992565155029296875f,0.4135999979019164829452392950770445168018341064453125f,0.849599994951486081617986201308667659759521484375f,0.09360000267028800724489201456890441477298736572265625f,-0.88909997221231407138475333340466022491455078125f,0.42859999447464947053276773658581078052520751953125f,0.1232999963879581883929859031923115253448486328125f,0.498199989128112807890147450962103903293609619140625f,-0.7192000343084341551502802758477628231048583984375f,0.64159999635219566727073470246978104114532470703125f,0.7808999985814093491853782325051724910736083984375f,-1.634600001955032411160573246888816356658935546875f,-0.96299999427795324180578973027877509593963623046875f,-0.79360000207424175044934600009582936763763427734375f,-0.748499986767768632489605806767940521240234375f,-0.84859998508691791840874429908581078052520751953125f,-1.307999982655047688240301795303821563720703125f,-1.4499999862909316306058826739899814128875732421875f,1.192199984478950369748417870141565799713134765625f,1.4620999855637553110909720999188721179962158203125f,-0.966899948990345592392259277403354644775390625f,-0.5423999799251557618617880507372319698333740234375f,0.31109996333122325040676514618098735809326171875f,0.4942999831557273182625067420303821563720703125f,0.261600000226497542143988539464771747589111328125f,-0.24530000717639932616975784185342490673065185546875f,0.20040001272261154241505209938623011112213134765625f,-0.02869999335259210138104890575050376355648040771484375f,1.029299977158010026556667071417905390262603759765625f,0.84719997665584101387281634742976166307926177978515625f,0.92039999119043347519664166611619293689727783203125f,0.82379999394416802971363722463138401508331298828125f,0.039599974983931307548346012481488287448883056640625f,-0.105599984252453094057955240714363753795623779296875f,1.127699979352950521160892094485461711883544921875f,1.252699971306324044917346327565610408782958984375f,0.8774999690055853562853371840901672840118408203125f,0.470099987590312906604594900272786617279052734375f,1.497599990272522774148455937393009662628173828125f,1.4608000133752820914878611802123486995697021484375f,-0.829499994993209810445478069595992565155029296875f,-0.5777999985337256561024332768283784389495849609375f,0.73460001536607766325914781191386282444000244140625f,-0.338800001096725367233375436626374721527099609375f,9.99983191487530120866722427308559417724609375e-05f,0.58859999283552166815525197307579219341278076171875f,0.2872000089168551539842155762016773223876953125f,-0.4643999892234802473467425443232059478759765625f,1.360100017988681742053813650272786617279052734375f,0.20440000725388518532099624280817806720733642578125f,-0.830300038468838152994067058898508548736572265625f,0.1743999975681305159014300443232059478759765625f,-0.4154000224828724441294980351813137531280517578125f,0.355199995148181901782891145558096468448638916015625f,-1.0009999974966046210056447307579219341278076171875f,0.05220000146627423731615635915659368038177490234375f,-0.7229000198006634292369199101813137531280517578125f,0.3305999882757664209975700941868126392364501953125f,1.0022000291824344486713016522116959095001220703125f,0.08540001458525647848318840260617434978485107421875f,0.0041999833464618774314658367075026035308837890625f,-0.4826000071108340616632403907715342938899993896484375f,0.253799994093179659415682181133888661861419677734375f,0.1603999978959560479552237666212022304534912109375f,-0.703099994608759715220003272406756877899169921875f,-0.135399997970461871688030441873706877231597900390625f,0.05620001037716892966500381589867174625396728515625f,-0.15579999385476117623738900874741375446319580078125f,-0.054699981796741070638745441101491451263427734375f,-0.47179999433159824917538571753539144992828369140625f,-0.074599996441602645091961676371283829212188720703125f,-0.4010999958157537292890992830507457256317138671875f,-0.91260000100135840739312698133289813995361328125f,-1.04299999934434861614818146335892379283905029296875f,1.093499973654747492446404066868126392364501953125f,0.6856999888300894241410787799395620822906494140625f,-0.9449999773502355537857511080801486968994140625f,-0.7155999881267547380048199556767940521240234375f,-0.024000018000602807433097041212022304534912109375f,-0.6282999892354013127260259352624416351318359375f,0.631200012314319991446609492413699626922607421875f,0.900300024163723122683222754858434200286865234375f,0.8483999893426901195425671176053583621978759765625f,0.85900000786781305350814363919198513031005859375f,0.402900014734267841731707449071109294891357421875f,0.7227999869108199249012614018283784389495849609375f,0.93869997565746388090701657347381114959716796875f,1.1500999965310096939674622262828052043914794921875f,-0.110700001645088974555619643069803714752197265625f,-0.4609999867677689877609736868180334568023681640625f,-0.89310000123977584252088490757159888744354248046875f,-0.55180000371932980129940915503539144992828369140625f,0.0768000087857245716804754920303821563720703125f,-0.378800000649690549181514143128879368305206298828125f,0.059100001902878229742555049597285687923431396484375f,0.572499990612268305767429410479962825775146484375f,-0.37829998245537321821529985754750669002532958984375f,-0.41720000712871563308681288617663085460662841796875f,-0.784499992609024587864041677676141262054443359375f,-0.30489999989271154845482669770717620849609375f,0.71459998914003364234304171986877918243408203125f,0.528199994045496001859874013462103903293609619140625f,0.75860000028610219402480652206577360630035400390625f,0.15760000338554380761024731327779591083526611328125f,-0.019000009395182093641096798819489777088165283203125f,-0.3289999961331486755256037213257513940334320068359375f,0.726599996650218571403456735424697399139404296875f,0.1829999874234200518685611314140260219573974609375f,0.22770000643581156385408803544123657047748565673828125f,-0.04160000037550926066387546597979962825775146484375f,0.66480001319646842006250153644941747188568115234375f,-0.24499999254941950965758223901502788066864013671875f,-0.84120002151578676663490341525175608694553375244140625f,0.3125999987661838286356896787765435874462127685546875f,0.22270000379085530539668980054557323455810546875f,-0.38400000727176664128137417719699442386627197265625f,-1.2656000167369843012465935316868126392364501953125f,0.18140000597238525870125158689916133880615234375f,0.389600012087822022976979496888816356658935546875f,0.3525999971270561417213684762828052043914794921875f,0.94090001169443127082558930851519107818603515625f,-0.174399994289875071018514063325710594654083251953125f,-0.0920000143647197177187990746460855007171630859375f,0.2111999978601932781430150498636066913604736328125f,-0.937100022965669854357884105411358177661895751953125f,0.26879999603033066790658267564140260219573974609375f,-0.335600011372566342515710857696831226348876953125f,-0.167800007176399201824779083835892379283905029296875f,-0.7680000234097243971831403541727922856807708740234375f,-0.053199999111890772240940350457094609737396240234375f,0.1856000072002412792926406837068498134613037109375f,1.0903999938726425256163565791212022304534912109375f,0.0455999957263468758128510671667754650115966796875f,-0.36059998306035989656948004267178475856781005859375f,-0.6938999725379051408680197710054926574230194091796875f,-0.615799994376301729204215007484890520572662353515625f,0.1118999652326113647404781659133732318878173828125f,-0.284000001311301986106627737171947956085205078125f,-0.16740000331997872962119799922220408916473388671875f,-0.25590000495165587945933793889707885682582855224609375f,-0.723299991917610185510056908242404460906982421875f,-0.90500000923872025992977796704508364200592041015625f,0.79470001599490647237189477891661226749420166015625f,0.80100000341981647533629029567237012088298797607421875f,-0.8820000031888479696817739750258624553680419921875f,-0.873100027763843655748132732696831226348876953125f,0.7395000247955323402493377216160297393798828125f,1.0917999929904933509305919869802892208099365234375f,0.52829998059272753607729100622236728668212890625f,0.131799997162818982587850769050419330596923828125f,-0.5796000079154968176453621708787977695465087890625f,-0.7368999919056893332935942453332245349884033203125f,0.5621999831378463596820438397116959095001220703125f,0.6994000202178956016041411203332245349884033203125f,0.836400002992153446257361792959272861480712890625f,1.073100006157159835851189200184307992458343505859375f,-0.3338999892830845084290558588691055774688720703125f,-0.3190000180006027363788234652020037174224853515625f,0.22140001319944868640732238418422639369964599609375f,0.6828000185757876483449990701046772301197052001953125f,0.2193000093817705220544667099602520465850830078125f,0.4345999955475330356335916803800500929355621337890625f,0.7687000173807145841919918893836438655853271484375f,-0.0783999887466431122362564565264619886875152587890625f,-0.0778999983429908837706534541212022304534912109375f,-0.060399999684095373453374122618697583675384521484375f,0.2389999865591523331431744736619293689727783203125f,-0.77519999961853025549629592205747030675411224365234375f,0.1679000236749654817458576872013509273529052734375f,-0.1145999855637551689824249478988349437713623046875f,-0.68700002539157889458465433563105762004852294921875f,-0.1716000021636485683984574279747903347015380859375f,-0.4573999835014337378424897906370460987091064453125f,-0.62159998547434813787049279198981821537017822265625f,0.944899989759921510312778991647064685821533203125f,0.30099999228119855843033292330801486968994140625f,-0.6193000148952005901037409785203635692596435546875f,-0.27039999178647999844571359062683768570423126220703125f,-0.12720000094473338236866766237653791904449462890625f,-0.279199988561868728265125128018553368747234344482421875f,1.532200047063827952342762728221714496612548828125f,-0.11659999307394031120566069148480892181396484375f,1.065500016033649188784693251363933086395263671875f,0.2491999876677990499729276052676141262054443359375f,-0.3395000107139349854179499743622727692127227783203125f,0.446999997973442031451440925593487918376922607421875f,0.3454000150322915629885756061412394046783447265625f,0.466600003504753058081178096472285687923431396484375f,-0.48860000617206100770317789283581078052520751953125f,0.164199992436170616461055260515422560274600982666015625f,0.330300009411573558537611461360938847064971923828125f,0.43450000005960465188081798260100185871124267578125f,-0.25920000659227415695795571082271635532379150390625f,-0.26349997872114183650182894780300557613372802734375f,-0.0108000065207480844264864572323858737945556640625f,0.196100002968311315498795011080801486968994140625f,-1.508399986511468338079566819942556321620941162109375f,-1.047799996745586437185693284845910966396331787109375f,-0.3896999682366857342685761977918446063995361328125f,-0.5921000105977061167550345999188721179962158203125f,-0.0803999850809571370291450875811278820037841796875f,0.33320001088380823972556754597462713718414306640625f,-0.99059996339082800886899349279701709747314453125f,-0.3540000030994416846397143672220408916473388671875f,0.294299968254566834957586252130568027496337890625f,-0.208099984252452685495882178656756877899169921875f,-0.380399957662821197601488165673799812793731689453125f,-0.091899991011619608372029688325710594654083251953125f,-0.4991999682962899864691053153364919126033782958984375f,-0.2947999877154827697012251519481651484966278076171875f,-0.70470002024173794552552863024175167083740234375f,-1.204000010251998986632315791212022304534912109375f,0.106799953353405641109930002130568027496337890625f,-0.498900019681453699149642488919198513031005859375f,0.8259000076711178195409956970252096652984619140625f,0.4452999942868947069740670485771261155605316162109375f,0.762899988061189304744402761571109294891357421875f,0.31960000076293937354421359486877918243408203125f,0.3986999979346992351025846801348961889743804931640625f,0.3601999863564968595852633370668627321720123291015625f,-0.4265999896466734497835204820148646831512451171875f,-0.4653999901473522271544425166212022304534912109375f,-0.919100021088123497747801593504846096038818359375f,0.29359999849796292181736134807579219341278076171875f,-0.110599995279311968943147803656756877899169921875f,0.48319999642968181063906740746460855007171630859375f,-0.357399998626112935795617886469699442386627197265625f,0.320599991703033493362084982436499558389186859130859375f,0.17979997057914687275115284137427806854248046875f,-0.5102000033438205406355336890555918216705322265625f,-0.91750003278255487515480126603506505489349365234375f,0.0798000002324581458879038109444081783294677734375f,0.623100012862682461900476482696831226348876953125f,-0.242799996149539953904650246840901672840118408203125f,-0.8858000193357469242982915602624416351318359375f,0.56179999790787693569882321753539144992828369140625f,-0.69110000334084009665502890129573643207550048828125f,-0.259999989941716247443537213257513940334320068359375f,0.79889999554157231642648184788413345813751220703125f,0.037200006979703825749083989649079740047454833984375f,0.5633000000387429206938350034761242568492889404296875f,-0.099399997866153722014814775320701301097869873046875f,0.20480000272840259700757314931252039968967437744140625f,-0.2111999986052512934264768773573450744152069091796875f,1.4550000518560413098612116300500929355621337890625f,-0.03280000225901602561862091533839702606201171875f,-0.03130000444948688453905560891143977642059326171875f,-0.488400011247396348412763700252980925142765045166015625f,0.58940001433193689450718011357821524143218994140625f,0.291399999639391893424544832669198513031005859375f,0.477800005052983589592940916190855205059051513671875f,-1.309399984455108523206945392303168773651123046875f,0.846000008821487625709778512828052043914794921875f,1.0043000034213065418953192420303821563720703125f,0.860999995112418758935746154747903347015380859375f,0.4407999966859819096498540602624416351318359375f,0.6866999855071307390375068280263803899288177490234375f,0.5595999861598015545638418188900686800479888916015625f,-1.086899981176852492126272409223020076751708984375f,-0.946199991750717117611202411353588104248046875f,0.5145000158548356239407439716160297393798828125f,0.93750002682209032656146519002504646778106689453125f,-0.6498000003814698999349275254644453525543212890625f,-0.7178999989390373315245597041212022304534912109375f,1.328700003075599767043968313373625278472900390625f,1.279800005149841268803356797434389591217041015625f,-0.208799972225726104824161666329018771648406982421875f,0.32870000978112212663972968584857881069183349609375f,-0.1488000016927717439330081106163561344146728515625f,-0.41229998011589064077497823745943605899810791015625f,-0.3597000016152858137274961336515843868255615234375f,-0.4962999907404184174453121158876456320285797119140625f,-0.38460000486075873737945585162378847599029541015625f,-0.18659999333471055660282900134916417300701141357421875f,-0.52740004072189350381449912674725055694580078125f,-1.0608000294685364650604242342524230480194091796875f,-0.9204000078052276823115107617923058569431304931640625f,-0.6109000157177446599376935409964062273502349853515625f,0.355499981947243082203158337506465613842010498046875f,-0.06620000113844870526236263685859739780426025390625f,0.5960999702416363010826216850546188652515411376953125f,0.5647999996364116981339975609444081783294677734375f,0.43870000545978538042390937334857881069183349609375f,-0.076599997246265427719436047482304275035858154296875f,-1.02240000942945474804446348571218550205230712890625f,0.4283999956011772525243941345252096652984619140625f,-1.39310002716779735010277363471686840057373046875f,-0.14720000578761105458625024766661226749420166015625f,0.6117000187553467416279318058514036238193511962890625f,-0.341600000897049838055608006470720283687114715576171875f,0.04489999899864205445965126273222267627716064453125f,-0.46659998234510435821675855549983680248260498046875f,0.0478999715209003085192307480610907077789306640625f,-0.08840000796914093683653845801018178462982177734375f,0.1342999986529349598640692420303821563720703125f,0.3731999961316585512349774944595992565155029296875f,-0.58950002367794507307507956284098327159881591796875f,-0.102399983501434477251024190991302020847797393798828125f,-0.121899995630979418592687579803168773651123046875f,0.241400004035234427846035032416693866252899169921875f,0.07719998969435692970364470966160297393798828125f,-0.378399994409084339253013240522705018520355224609375f,0.5215000111460685872089015902020037174224853515625f,0.21620000337362288433951107435859739780426025390625f,0.38979999408572896246738537229248322546482086181640625f,-0.2601999882936478769579480285756289958953857421875f,0.103800005678832629740782067528925836086273193359375f,0.08860000248402355449428569045267067849636077880859375f,-0.661600024366379102502833120524883270263671875f,0.456799997013807246304395448532886803150177001953125f,0.0032000317454343729650645400397479534149169921875f,1.091899997866153881886930321343243122100830078125f,-0.1887000120162962701897413353435695171356201171875f,-0.13649999177455907073408525320701301097869873046875f,1.158599997901916367482044734060764312744140625f,1.31440000084638608512932478333823382854461669921875f,0.097800021481514676224833237938582897186279296875f,0.598899976766109176651298184879124164581298828125f,-0.7680000151023269605587984187877736985683441162109375f,-0.6541000024802980927773887742660008370876312255859375f,-0.791699971127510337964849895797669887542724609375f,-0.32709998109340698846381201292388141155242919921875f,-0.29159999068975395175584708340466022491455078125f,-0.4468999930977819445843124412931501865386962890625f,0.568799993944168136295047588646411895751953125f,0.11469999490976334755032439716160297393798828125f,-0.0947999418944125427088920332607813179492950439453125f,0.0359000225722787558169102339888922870159149169921875f,0.55109999723136393612321626278571784496307373046875f,0.5429000093698501672179190791212022304534912109375f,-0.80699997062981143614024404087103903293609619140625f,-0.7145999909281730566590340458787977695465087890625f,0.34889998034238800528328283689916133880615234375f,0.2023000137925148322892709984444081783294677734375f,-0.5330999871581791893504487234167754650115966796875f,-0.36859998057931686599175691299024038016796112060546875f,0.1364999997280536003785300636081956326961517333984375f,0.0446000021785497580140145146287977695465087890625f,1.0194000053167346919735791743732988834381103515625f,0.8826000021934508055210244492627680301666259765625f,0.4014000037372102980270938132889568805694580078125f,0.069800000530481298710583359934389591217041015625f,-0.0419999806880948511889073415659368038177490234375f,0.592399993336200747506836705724708735942840576171875f,-0.4404999880194662154053730773739516735076904296875f,-0.0719999964833260008845172706060111522674560546875f,-0.39570001647174368741843863972462713718414306640625f,-0.4418000008136033773809003832866437733173370361328125f,-1.1123000179648396823495204444043338298797607421875f,-0.1793999947369098180161017808131873607635498046875f,0.01010000487566031779351760633289813995361328125f,0.09040002114176726166761000058613717555999755859375f,-1.05720001338720326344855493516661226749420166015625f,0.21719998924732220046962538617663085460662841796875f,0.66900001859664914860559292719699442386627197265625f,0.31699999290704727883394298260100185871124267578125f,0.2715999796628949525256757624447345733642578125f,0.011600018852949034453558851964771747589111328125f,-0.0788999940514560904603058588691055774688720703125f,-0.16219999431371689979641814716160297393798828125f,0.5407999817848203605308299302123486995697021484375f,-0.46419999973773951040811880375258624553680419921875f,-0.46429998971223795933838118799030780792236328125f,0.04259999280571946655982173979282379150390625f,-0.33300003957748458560672588646411895751953125f,-0.5559999969005584574688327847979962825775146484375f,0.1806999679684633264287185738794505596160888671875f,-0.4680000063478946259465374168939888477325439453125f,-0.68049999666213967230987691436894237995147705078125f,0.25180000297427174160702634253539144992828369140625f,0.17960000821352029021227281191386282444000244140625f,-0.621900028860569165090055321343243122100830078125f,1.0781999760150906553235472529195249080657958984375f,0.62039998195171364159250515513122081756591796875f,0.118799994838237665817359811626374721527099609375f,0.5340999918818472025350274634547531604766845703125f,-0.738299978320300365197681458084844052791595458984375f,-0.359499999910593004415204632095992565155029296875f,0.03210003712177211809830623678863048553466796875f,0.5971000207304957285714408499188721179962158203125f,-0.002999947786332057830804842524230480194091796875f,-0.366299996626377577513267169706523418426513671875f,0.794999990463256978046047152020037174224853515625f,1.14159999784231214192686820751987397670745849609375f,0.3260999700367452902582954266108572483062744140625f,-0.23480002094507224086328278644941747188568115234375f,0.156000015377998124677105806767940521240234375f,0.436200060892105057064327411353588104248046875f,-0.391499972403050122693457524292171001434326171875f,-0.4533000017523765023952364572323858737945556640625f,-1.154099976086616408110785414464771747589111328125f,-1.2485999847888944458418336580507457256317138671875f,0.1227000025987621256717829965054988861083984375f,0.08429997898340246820225729607045650482177734375f,-0.886799970638751489104834035970270633697509765625f,-0.30459997036457053809499484486877918243408203125f,-1.00619998757839113068257574923336505889892578125f,-0.8904999945759772828068889793939888477325439453125f,0.722100004637240999727509915828704833984375f,0.87169999854564661490030630375258624553680419921875f,-0.922499980330467650446735206060111522674560546875f,0.305799982529878722203875440754927694797515869140625f,0.87170002536773694146177149377763271331787109375f,0.39540000013113019150523541611619293689727783203125f,0.03259997656345348104878212325274944305419921875f,0.06400000607967371024642488919198513031005859375f,-0.4130000032186504910214353003539144992828369140625f,-0.4833999866604805628611529755289666354656219482421875f,-0.00769999544620514342341266456060111522674560546875f,0.474399996972084014856818612315692007541656494140625f,0.6257999829769129718215481261722743511199951171875f,-0.63620000143647192913931576185859739780426025390625f,0.2864000539541251555419876240193843841552734375f,0.8411999930918216961117650498636066913604736328125f,0.36560001614093806665550800971686840057373046875f,0.0539999987781047963153469027020037174224853515625f,-0.5478000304222110372620591078884899616241455078125f,-0.53240000361800188244387754821218550205230712890625f,-0.590500014245510129740068805404007434844970703125f,0.3944000057637691014633674058131873607635498046875f,-0.534600016707182046360458116396330296993255615234375f,-0.022399999594688413395715542719699442386627197265625f,-0.904300035011768699177991948090493679046630859375f,-0.15580000890493383991497466922737658023834228515625f,0.2984000298738482825910978135652840137481689453125f,0.57859998732209216854016631259582936763763427734375f,0.01260000278949746643775142729282379150390625f,-0.57799998980760580735704934340901672840118408203125f,-0.0847999999344348776020297009381465613842010498046875f,0.005599997961521163691855917932116426527500152587890625f,0.80780001283884050877759364084340631961822509765625f,0.1506999846577645740808293339796364307403564453125f,0.4604999726712701857422871398739516735076904296875f,-0.137100007021427217068776371888816356658935546875f,0.0168000287532805714363348670303821563720703125f,-0.20219997851848603431790252216160297393798828125f,-0.80129995505213802431399017223156988620758056640625f,-0.37159998919963843633240685448981821537017822265625f,0.8339999887943267964374172152020037174224853515625f,0.42009998967647543821612998726777732372283935546875f,-1.375499996662139068348551518283784389495849609375f,-1.1613999931573868451550879399292171001434326171875f,1.39439996001720345475405338220298290252685546875f,0.56659995880126956535605131648480892181396484375f,-0.13319999926090275721435318700969219207763671875f,-0.516200014400482132259639911353588104248046875f,-0.8099999806284898795638582669198513031005859375f,-0.2255999917030333534739838796667754650115966796875f,0.985800010097026557787103229202330112457275390625f,1.051100022041797643623795011080801486968994140625f,0.14760001187920579468482173979282379150390625f,0.399000009208917649772274671704508364200592041015625f,-0.0218999871373171117738820612430572509765625f,0.3921000058293344636695110239088535308837890625f,0.919499961614608452009633765555918216705322265625f,0.2987999736785891258250558166764676570892333984375f,-0.998999982237815942198722041212022304534912109375f,-0.463799993348121830649688490666449069976806640625f,0.0173999994456768281025915712234564125537872314453125f,0.010199995371699355217742777313105762004852294921875f,-0.8717999956727027210945379920303821563720703125f,0.304400000771880119287970956065692007541656494140625f,-0.8427000004380940101356145532918162643909454345703125f,0.3741999913930893484348416677676141262054443359375f,0.44989998387396301637863871292211115360260009765625f,-0.142599989080429201404598416047520004212856292724609375f,-0.47909997578859275790819083340466022491455078125f,0.4830000126063822563082794658839702606201171875f,-0.4161000256180766854186003911308944225311279296875f,0.565200004535913347325504219043068587779998779296875f,-1.1719000169396398547405624412931501865386962890625f,-0.193399995750188846699302303022705018520355224609375f,1.438200032639503689324556034989655017852783203125f,0.126000010311603460877449833787977695465087890625f,0.05920000219643117844725566101260483264923095703125f,0.282600001969933511958288363530300557613372802734375f,-0.14110000497996821877677575685083866119384765625f,0.258399995303153995340750270770513452589511871337890625f,-1.1994000237941744302361257723532617092132568359375f,-0.03980000202059752467675934894941747188568115234375f,0.640399996852874409114519949071109294891357421875f,0.14859999894499775763279103557579219341278076171875f,-0.383000013202428934988574837916530668735504150390625f,0.438400001561641605718477876507677137851715087890625f,0.1523000275015835569547562045045197010040283203125f,0.78920000271797174917765005375258624553680419921875f,0.3002000025987623388346037245355546474456787109375f,-0.1654000080287456597716300166212022304534912109375f,0.2754000075817106818476531771011650562286376953125f,0.620999996453523550599129521287977695465087890625f,-0.400199998873472129190531632048077881336212158203125f,0.10259999071955672178546592476777732372283935546875f,0.920699993747472689165078918449580669403076171875f,0.9044999826997519587479246183647774159908294677734375f,-0.3632999851375815492104948134510777890682220458984375f,-0.3552999754369261520281497723772190511226654052734375f,1.16490000019073391968049691058695316314697265625f,1.21510000368356685385151649825274944305419921875f,1.1916000088691713898469970445148646831512451171875f,0.973500018060207139569683931767940521240234375f,-0.0560999591588977608580535161308944225311279296875f,0.397500000894069671630859375f,-0.17580001814365431300757336430251598358154296875f,-0.722199992823600922520199674181640148162841796875f,0.587699989672004985408193533658050000667572021484375f,0.51380001376271255963956718915142118930816650390625f,0.5183999898642304149376514033065177500247955322265625f,0.3810999953687190799911377325770445168018341064453125f,0.289500017642975393528104177676141262054443359375f,0.7381999873399731626477660029195249080657958984375f,0.125699980187416571197900339029729366302490234375f,-0.12479999263286600807987269945442676544189453125f,0.8484000122904777896337691345252096652984619140625f,0.533799999159574323215338154113851487636566162109375f,1.352999988317488799793864018283784389495849609375f,0.6340999894976615536279496154747903347015380859375f,-0.537300006043910372000027564354240894317626953125f,-0.7742000307321550423012013197876513004302978515625f,0.99749997973442106058428180404007434844970703125f,0.3417999816656109857149203890003263950347900390625f,0.269400000250339388685461017303168773651123046875f,0.398199991363286986967295888462103903293609619140625f,1.2608000175476075099822992342524230480194091796875f,-0.11499999463558197021484375f,0.9655000345110895665357020334340631961822509765625f,-0.044200002568960172766310279257595539093017578125f,-0.12050000709295272116605701739899814128875732421875f,-0.13279999778866768966878453284152783453464508056640625f,-0.349599998378753440420041442848742008209228515625f,-0.1911999983072281139584447373636066913604736328125f,1.08210001287013302118111823801882565021514892578125f,-0.1480000006854533811662122388952411711215972900390625f,-0.21160000201463713409566480549983680248260498046875f,0.48399999386072156681848355219699442386627197265625f,-0.4630000148415565064397014793939888477325439453125f,-0.4873999952733516494163268362171947956085205078125f,-0.466900005614757418470617267303168773651123046875f,0.633600008183717644527632728568278253078460693359375f,0.358499996695667400015139492097659967839717864990234375f,0.04279999816119672839676013609278015792369842529296875f,0.49569998853206609368271529092453420162200927734375f,0.13219999401569371144660181016661226749420166015625f,1.2466000068575142645244113737135194242000579833984375f,-0.169799998742341973212433003936894237995147705078125f,-0.6191999890089032732021223637275397777557373046875f,0.493200002837181017412149230949580669403076171875f,0.5411000209242107228391205353545956313610076904296875f,-0.161600006186962019683051039464771747589111328125f,0.60709998288154576329134215484373271465301513671875f,-0.3681999858498574695886418339796364307403564453125f,0.503200018669665016801673118607141077518463134765625f,0.4629999775886535218205608543939888477325439453125f,-0.60750000894069700052568805404007434844970703125f,-0.9913999877929686732613845379091799259185791015625f,-0.4095000265836716835110564716160297393798828125f,-0.7795000221133232543024860206060111522674560546875f,-0.254399984082579688760006320080719888210296630859375f,-0.198299996685981749777738514239899814128875732421875f,-0.3683999762296679136852617375552654266357421875f,-0.324000015020370568663565791212022304534912109375f,-0.64800001253187655692045154864899814128875732421875f,-1.005299998421966822803597096935845911502838134765625f,0.933599984788894943221748690120875835418701171875f,0.61999998718500126670960526098497211933135986328125f,-0.7514999704658986701133471797220408916473388671875f,-0.2269999866485594708365169935859739780426025390625f,1.3554000099658960465376367210410535335540771484375f,1.1455000196099280884709514793939888477325439453125f,-0.078600001962482846096236244193278253078460693359375f,-0.321699996254593056921322613561642356216907501220703125f,-0.025500002026557666567896376363933086395263671875f,-0.50509999623298629245482516125775873661041259765625f,-0.7347000090658664106513242586515843868255615234375f,-1.1281999889045952745192380461958236992359161376953125f,1.163399993515014330114354379475116729736328125f,1.1443999910116193774456405662931501865386962890625f,-0.444000023514032449156729853712022304534912109375f,-0.4863000098139045324074913878575898706912994384765625f,-0.91769996683597554465450230054557323455810546875f,-0.96369997565746334799996475339867174625396728515625f,0.27959998150318832177418926221434958279132843017578125f,0.267399999743699989807055317214690148830413818359375f,0.16939999518394444066871074028313159942626953125f,0.619999982267618321429836214520037174224853515625f,-0.268599986279010494172325707040727138519287109375f,0.6248000051498412421580042064306326210498809814453125f,-0.1181999935984610061723287799395620822906494140625f,0.7272000057876109035959188986453227698802947998046875f,0.075100003348290922389196566655300557613372802734375f,-0.3418000064015387362559295070241205394268035888671875f,-0.53090001527070995734902680851519107818603515625f,0.50040001369118680685232902760617434978485107421875f,-0.5086000157833101553706001141108572483062744140625f,0.299799997699260689643097066436894237995147705078125f,-0.08330000432282691491536752437241375446319580078125f,-0.1112000009149312906853168669840670190751552581787109375f,-0.15780001052916059478548049810342490673065185546875f,-0.31500000171363351508801997624686919152736663818359375f,-0.76430000789165486452247932902537286281585693359375f,-0.05979999978542327454533733543939888477325439453125f,-0.4038000043749807588255862356163561344146728515625f,-0.1958000030934810009597413227311335504055023193359375f,-0.9858000302881004817123766770237125456333160400390625f,0.364999997913837415097759730997495353221893310546875f,-0.15009999027252174386148908524774014949798583984375f,-0.31999999940395351671895696199499070644378662109375f,0.1101999902307984058325018850155174732208251953125f,-0.311400010889768541488820119411684572696685791015625f,0.586700019221007895708908108645118772983551025390625f,-0.48860001071691494534121602555387653410434722900390625f,-0.9323000066399576013509431504644453525543212890625f,0.267900001621246275362864253111183643341064453125f,-0.1382999976545572218267210473641171120107173919677734375f,-0.0905999981477856319855845868005417287349700927734375f,0.33569999374747272469221570645458996295928955078125f,-0.317100010448694291653737309388816356658935546875f,-0.0040000054836273335467922152020037174224853515625f,-0.443400003200769443623130428022705018520355224609375f,-0.3191999957144260324781726012588478624820709228515625f,-0.44039999640583982198904777760617434978485107421875f,-0.3663000057160854527893434351426549255847930908203125f,-0.15510001128315931140377870178781449794769287109375f,-0.1441999994769691539886480313725769519805908203125f,0.481000023759901529363247618675814010202884674072265625f,-0.01469999811351296870043370290659368038177490234375f,0.8404000087738037194640128291212022304534912109375f,0.1904000058680772600183672693674452602863311767578125f,0.4267000009298325124973416677676141262054443359375f,-0.160800004583597111462722750729881227016448974609375f,0.372000012725591577833483825088478624820709228515625f,0.2334000027537346255712691345252096652984619140625f,0.8596000128328800116150887333787977695465087890625f,-0.16830000063478944838379902648739516735076904296875f,0.26060000571012498227219111868180334568023681640625f,-0.0055000006109475751969739576452411711215972900390625f,0.28660000886917114115703952847979962825775146484375f,-0.4038000030338764201331969161401502788066864013671875f,-0.0382000026881694765279462444595992565155029296875f,0.1089000005334614940011306316591799259185791015625f,-0.7400000207126140594482421875f,-0.195300003451108927787771563089336268603801727294921875f,1.3114000070154663291077667963691055774688720703125f,-0.503299985361099100344972612219862639904022216796875f,-0.77679999969601620790626839152537286281585693359375f,-0.00899999982118615537984851471264846622943878173828125f,0.6049000119999050628649683858384378254413604736328125f,-0.617400005070865010026182062574662268161773681640625f,0.5655000250115993498667421590653248131275177001953125f,0.01429997063875276808175840415060520172119140625f,0.8934000027537347676798162865452468395233154296875f,0.3074999918043612723295154864899814128875732421875f,-0.41550001260638236288968982989899814128875732421875f,-0.6764000007569788142092193083954043686389923095703125f,-0.421400006493926060358035101671703159809112548828125f,-0.013200009393691924941549586947076022624969482421875f,-0.6311999854922289987513295272947289049625396728515625f,0.65869998400211315914276610783417709171772003173828125f,0.371799986247718095011549621631274931132793426513671875f,-0.243899993604421549964200721660745330154895782470703125f,0.2453000038422643858115890225235489197075366973876953125f,0.170499996215105031893699560896493494510650634765625f,-0.4153000083684921239779441748396493494510650634765625f,0.4952999985337260824280747328884899616241455078125f,1.08880002404451392550299715367145836353302001953125f,-0.6159999721646312043077386988443322479724884033203125f,0.15460002214610579773790277613443322479724884033203125f,-0.69549999278783758427380234934389591217041015625f,-1.13180000580549222632953387801535427570343017578125f,-0.884699995505808800544400583021342754364013671875f,-0.14079999370574913797327099018730223178863525390625f,-0.940499963581562070658037555404007434844970703125f,0.5358000087559222635036348947323858737945556640625f,0.1269000026345252774007121843169443309307098388671875f,0.3380999967694282570818131716805510222911834716796875f,-0.0210000005513429621739618369247182272374629974365234375f,0.029299998615682110170155283412896096706390380859375f,0.0506000029161572939528923598118126392364501953125f,0.5817000136330723325528424538788385689258575439453125f,-0.12269999008178711363825641456060111522674560546875f,0.220900001561641801117730210535228252410888671875f,0.034899996465444615978412912227213382720947265625f,-0.4208999848723411485451606495189480483531951904296875f,0.02040000393092633590441664637182839214801788330078125f,0.76810000749826434773837036118493415415287017822265625f,-0.04519999894797799289136719380621798336505889892578125f,0.14970000534057614771654698415659368038177490234375f,-0.2319000022619962797509884921964840032160282135009765625f,-0.2604000059053301896483390009962022304534912109375f,0.09190000084638595689856543913265340961515903472900390625f,0.0640000018142163895618068636395037174224853515625f,-0.03290000087618827340207872111932374536991119384765625f,-0.1046999986350536371304542626603506505489349365234375f,-0.022899996256828369922686761128716170787811279296875f,-0.5821000204324722204773934208787977695465087890625f,0.201599997320771218145551983980112709105014801025390625f,0.254099999406933729773072627722285687923431396484375f,0.085500006496906255648582373396493494510650634765625f,0.6758999979853630435400191345252096652984619140625f,0.408300009649992023952336239744909107685089111328125f,0.11050002020597471386054166941903531551361083984375f,0.193700003001093801202614486101083457469940185546875f,0.0629000014722347344786612666212022304534912109375f,0.16039999860376108475890077897929586470127105712890625f,-0.130100012698769751384730852805660106241703033447265625f,0.5032000117778778758292901329696178436279296875f,-0.76580004124045419899857733980752527713775634765625f,-0.19759999325275412473956748726777732372283935546875f,-0.08690000188946722659011356881819665431976318359375f,-0.484500011086463899800946819595992565155029296875f,-0.78340000096559503361959286849014461040496826171875f,-0.6163999643981463716357893645181320607662200927734375f,-0.6068999775260686124767062210594303905963897705078125f,-0.065399998641014089884038185118697583675384521484375f,-0.7906000063061713323264712016680277884006500244140625f,0.14389998421669059069927243399433791637420654296875f,-0.577200008171796508094075761619023978710174560546875f,-0.3703999840378759955683563021011650562286376953125f,-0.04409999620318405533225813996978104114532470703125f,0.34459998317956941349393673590384423732757568359375f,-0.395900010055303397393799968995153903961181640625f,0.1027999971926210065475970623083412647247314453125f,-0.345400024122000104398466646671295166015625f,0.2975999935507775973064781283028423786163330078125f,1.31090004477500965407443800359033048152923583984375f,-0.08739996383190185014200324076227843761444091796875f,-0.76300000694394132239040118292905390262603759765625f,-0.1718999945878980639690780662931501865386962890625f,-0.973500008225440804920935988775454461574554443359375f,-0.920999974250793940200310316868126392364501953125f,0.213800001394748306893234257586300373077392578125f,0.4999999943375588173921642010100185871124267578125f,-0.812000009298324609829933251603506505489349365234375f,-0.0504000014349818281633730521207326091825962066650390625f,-0.066600001269578934515180890230112709105014801025390625f,0.042399999222159312495250560459680855274200439453125f,-0.65090001631379124091836274601519107818603515625f,0.19319999918639663150088381371460855007171630859375f,-0.2062999876856803638247583876363933086395263671875f,-0.055899998432397790537606852012686431407928466796875f,0.735100015604496004328893832280300557613372802734375f,0.1807999931097030721360852112411521375179290771484375f,0.33919999899268138410235451374319382011890411376953125f,0.00860000006258487825760994383017532527446746826171875f,0.09550000113248824362699451739899814128875732421875f,0.0517000039845705305907586080138571560382843017578125f,-0.259399988776445322713470886810682713985443115234375f,0.356399998596310663145914077176712453365325927734375f,-0.37929998196363445828183103003539144992828369140625f,0.055799998399615358124492558999918401241302490234375f,-0.513299978506565057756461101234890520572662353515625f,-0.148400001412630100361411678022705018520355224609375f,0.2359999949634075111504216692992486059665679931640625f,0.018000000983476649452086348901502788066864013671875f,-0.4544000024855137098711566068232059478759765625f,0.22419999578893179403848989750258624553680419921875f,-0.373900013571977662962808608426712453365325927734375f,0.2078999998331070298007716701249592006206512451171875f,-0.227099986010789844925739089376293122768402099609375f,0.13219999684691430275051970966160297393798828125f,0.76840000483989712165566743351519107818603515625f,0.046799998652935048681911212042905390262603759765625f,-0.01210000597834604008085079840384423732757568359375f,0.1844999896287919938231425476260483264923095703125f,-0.8533000078618524497642283677123486995697021484375f,0.178499975144863043396981083787977695465087890625f,0.5604000038564205254942862666212022304534912109375f,0.654699989843368967967762728221714496612548828125f,0.896699979621171650023825350217521190643310546875f,-0.02020000393986709497085030307061970233917236328125f,-0.36100000956654554329361417330801486968994140625f,-0.24260000226795674915791778403217904269695281982421875f,-0.1708000012680888080041086141136474907398223876953125f,-0.34529999585151660568271836382336914539337158203125f,-0.2901000193297864626629234408028423786163330078125f,0.6173999677062038671238042297773063182830810546875f,-0.2092000185132025791290288907475769519805908203125f,-0.556099983298778521856320367078296840190887451171875f,-0.759900004959106478708008580724708735942840576171875f,0.1465999906152486431665238342247903347015380859375f,-0.22170001381933701622983790002763271331787109375f,-0.18590001184344284723692908301018178462982177734375f,0.23040001011490840454598583164624869823455810546875f,-0.080400001919269659111932924133725464344024658203125f,0.7174000191152096750357713972334749996662139892578125f,0.344899992032349000936619631829671561717987060546875f,-0.3606000134214759622608426070655696094036102294921875f,0.13219999684691430275051970966160297393798828125f,-0.1790999985873698019389621549635194242000579833984375f,0.502699978756904641841174452565610408782958984375f,-0.44539999193549117961765659856610000133514404296875f,0.7620999974846842661690970999188721179962158203125f,0.0893000007390976069387988900416530668735504150390625f,-0.086099997976422315559830167330801486968994140625f,0.02199999856948853249605235760100185871124267578125f,-0.30689999883472918185134403756819665431976318359375f,0.46619999964833258587759701185859739780426025390625f,0.01090000267922874854775727726519107818603515625f,-0.83740000615119924987084232270717620849609375f,-0.034200000929832441443068091757595539093017578125f,0.1196000053822994235730448053800500929355621337890625f,-0.015799999442696555096432575737708248198032379150390625f,0.329400005242228510748958569820388220250606536865234375f,0.271699998322129321337570218020118772983551025390625f,-0.202799972903728420448032920830883085727691650390625f,-0.21979999620914458802189983543939888477325439453125f,-0.211399998223781526718312306911684572696685791015625f,0.121700001302361471289259498007595539093017578125f,0.4111999983072280873130921463598497211933135986328125f,-0.1237999999791383753944273848901502788066864013671875f,0.12219999975264073110992057991097681224346160888671875f,-0.032899998939037278233854522113688290119171142578125f,0.2825000120699405758983857595012523233890533447265625f,-0.03720000027418135513102015465847216546535491943359375f,0.144800000232458092597198628936894237995147705078125f,-0.08370000214427708584707943373359739780426025390625f,-0.195799997840821748962270021365839056670665740966796875f,0.01939999905824663084530357082257978618144989013671875f,-0.190999995261430743909158991300500929355621337890625f,-0.224699999454617493910291159409098327159881591796875f,-0.054000002950429948356259046704508364200592041015625f,-0.08129999781847008222257500165142118930816650390625f,0.776299989920854471847633249126374721527099609375f,0.1191999887108803335422635427676141262054443359375f,-0.20650001978874232833049973123706877231597900390625f,0.40549997371435164694730701739899814128875732421875f,0.19499998643994320701722244848497211933135986328125f,0.493599972383678309295618191754329018294811248779296875f,0.47829998660907147023380048267426900565624237060546875f,0.4504999957680697608708442203351296484470367431640625f,0.42900000898540013860582575944135896861553192138671875f,-1.01599998527765222888774587772786617279052734375f,-0.0030999732255931888857958256267011165618896484375f,-0.738699981021881058040889911353588104248046875f,-0.2422999759435651156991298194043338298797607421875f,0.4890000238120553177623150986619293689727783203125f,-0.68470002366900484247480562771670520305633544921875f,-0.069599996404349606660133531477185897529125213623046875f,-0.10920000560507181486258332370198331773281097412109375f,-0.29880000251233607144740744843147695064544677734375f,-0.440700013938546408809315835242159664630889892578125f,0.241000012397766028016121708787977695465087890625f,-0.85390003458261531932294019497931003570556640625f,0.48599998408555933337993337772786617279052734375f,-0.0669000196218494380673291743732988834381103515625f,-0.5347000063836571115416518296115100383758544921875f,0.637200032609701594310536165721714496612548828125f,0.09560001770555981437382797594182193279266357421875f,0.772800019618868905268982416600920259952545166015625f,0.76690000382661782651894100126810371875762939453125f,0.3459999998807905541298168827779591083526611328125f,-0.33319996260404582955061414395458996295928955078125f,-0.688600012505054426270589829073287546634674072265625f,-0.295599996024370259561919738189317286014556884765625f,0.54719998105168343727200408466160297393798828125f,-0.26509999958574770317909496952779591083526611328125f,0.121600002759695069443068859982304275035858154296875f,-0.1179999975934624412143847393963369540870189666748046875f,-0.3325999913528561169329123003990389406681060791015625f,-0.202900004452466975468638565871515311300754547119140625f,-0.085100009569525791874866627040319144725799560546875f,0.19720000846981999842455479665659368038177490234375f,0.74159999679923060966757475398480892181396484375f,0.10110000326633450384861134807579219341278076171875f,0.6476999914228915855574086890555918216705322265625f,0.02739999631643286193138919770717620849609375f,-0.6038000255346298583702946416451595723628997802734375f,0.0672000020250677898570046409076894633471965789794921875f,-0.0669000021502375308291021838158485479652881622314453125f,-0.132000001251697529625062088598497211933135986328125f,0.67470000303089616711105236390721984207630157470703125f,0.02419999839663500296182974125258624553680419921875f,-0.297000011533498753379944901098497211933135986328125f,-0.40960000329613688307972552138380706310272216796875f,0.3707999872982501443630098947323858737945556640625f,0.032700005578994773003387308563105762004852294921875f,0.5221000056803226652135663243825547397136688232421875f,0.2155999993026256333905621431767940521240234375f,-0.64370000672638416627791002611047588288784027099609375f,-0.086500005483627351310360609204508364200592041015625f,-0.719300013852119501933657375047914683818817138671875f,0.294400000101327830037689636810682713985443115234375f,0.34130000139474869769173892564140260219573974609375f,0.719999997317790185746844144887290894985198974609375f,0.495400017863512065474651535623706877231597900390625f,0.6151999814391135856794790015555918216705322265625f,-0.01679999999403936072894794051535427570343017578125f,1.19319996588230115008855136693455278873443603515625f,0.154800012600422309816394772497005760669708251953125f,-0.9520999807953831606255334918387234210968017578125f,0.2730000208020211260873111314140260219573974609375f,0.66019996504783673429983537062071263790130615234375f,0.936799985390901479576086785527877509593963623046875f,-0.513299952876568710280480445362627506256103515625f,0.02700002285838110083204810507595539093017578125f,0.14689998564720152529616825631819665431976318359375f,-0.699800021243095482503804305451922118663787841796875f,0.060699984546005669727719578077085316181182861328125f,-0.2738000050827860132329760745051316916942596435546875f,-0.5765999866291882103297439243760891258716583251953125f,-0.33699999281391501693150303253787569701671600341796875f,-0.649799986970424736654194930451922118663787841796875f,-0.3058000007092953609344476717524230480194091796875f,-0.3383999659478671429013729721191339194774627685546875f,-0.604799988088011719611358785186894237995147705078125f,-0.900699971842765734209024230949580669403076171875f,0.00069999136328657840522282640449702739715576171875f,-0.51589997771978435281425845460034906864166259765625f,-0.80999999031424518847188664949499070644378662109375f,-1.1975999729871755761223539593629539012908935546875f,-0.401199971485137751869842759333550930023193359375f,0.588499986171722611061341012828052043914794921875f,-0.95380000392794617170011406415142118930816650390625f,-0.157600010389089550955077356775291264057159423828125f,-0.5086000002861024160694114470970816910266876220703125f,-0.102400002500414810668871723464690148830413818359375f,-0.487099992269277581868891502381302416324615478515625f,-0.3068000043153762401715312080341391265392303466796875f,-0.70599999719858175240005948580801486968994140625f,0.0997000074267386793280820711515843868255615234375f,0.8764999855160713337909328402020037174224853515625f,-0.1785999966353177814681885138270445168018341064453125f,-0.4116999930322169376495367032475769519805908203125f,0.2520999905556440268128426396287977695465087890625f,0.0239999940097330721755497506819665431976318359375f,-0.1472999999344348776020297009381465613842010498046875f,0.198900004482269299188601507921703159809112548828125f,-0.113100000680983026057901952299289405345916748046875f,0.201100010605156420684380691454862244427204132080078125f,-0.0225999977603555113336852855354663915932178497314453125f,0.7452999888852238630221336279646493494510650634765625f,0.0914000006079673799064266859204508364200592041015625f,0.5686000032663345304939639390795491635799407958984375f,0.35759998893141753395497062228969298303127288818359375f,-0.39239998156428346209878554873284883797168731689453125f,0.136700004282593656768085566000081598758697509765625f,0.74459998556375506240101458388380706310272216796875f,0.08669999459683896958495097351260483264923095703125f,0.446600003057718186738611620967276394367218017578125f,0.24879999454021461957609062665142118930816650390625f,0.856000021934509192078621708787977695465087890625f,-0.027700000140070901721855989308096468448638916015625f,0.957400010770559362072162912227213382720947265625f,0.7577999788641935907662627869285643100738525390625f,-0.02920000867843608460816540173254907131195068359375f,-0.683899976766110029302581096999347209930419921875f,-0.85339999724030501937477311003021895885467529296875f,0.585999980807304865493279066868126392364501953125f,0.4893999859452247136459845933131873607635498046875f,0.0307000061154365511129071819595992565155029296875f,-1.1770000230073929259333453956060111522674560546875f,-0.57090000245571115300435849349014461040496826171875f,0.326500034093857305350638853269629180431365966796875f,-0.71419998066425360860876025981269776821136474609375f,-0.201199975955486110024139634333550930023193359375f,-0.2875999995112419327369934762828052043914794921875f,0.5784999943673607702976369182579219341278076171875f,-0.896899963593483562362962402403354644775390625f,0.0643000372469426650923196575604379177093505859375f,0.0035999979019170069705069181509315967559814453125f,-0.7534000356853010060831365990452468395233154296875f,0.545600005114077912793391078594140708446502685546875f,0.700200026440620515444379634573124349117279052734375f,0.35499997034668950846025836654007434844970703125f,-0.49710001134276371459463916835375130176544189453125f,-0.32469997762441671351751892871106974780559539794921875f,0.7216000048086044227346746993134729564189910888671875f,0.4351999823331833550099645435693673789501190185546875f,0.13989997687041733076540594993275590240955352783203125f,0.282100002551079143131573800928890705108642578125f,0.77149999549984915603317858767695724964141845703125f,-0.194600008362531440297971130348742008209228515625f,-0.59180001400113102505429196753539144992828369140625f,-0.129799993973970462235456579946912825107574462890625f,-0.591200013208389218988259017351083457469940185546875f,0.116299994763731984193100288393907248973846435546875f,0.578200007456540987504922668449580669403076171875f,-0.05559999632239343014816768118180334568023681640625f,-0.58460000135898582129811984486877918243408203125f,0.1808999969422817599706831970252096652984619140625f,-0.39329998536109922468995137023739516735076904296875f,0.0781000034749508298403952721855603158473968505859375f,0.4757000101387500734517743694595992565155029296875f,0.166199996519088788460294381366111338138580322265625f,0.467800010752677852821079795830883085727691650390625f,-0.0743999979406595457476214505732059478759765625f,0.4800999947428703507057434762828052043914794921875f,0.22489999504983426703574878047220408916473388671875f,0.53500000119209278892640213598497211933135986328125f,0.1804000030368566598326651728712022304534912109375f,-0.3808999888956545731844016700051724910736083984375f,-0.08480000760853290131535686668939888477325439453125f,-0.4805000075399876724446812659152783453464508056640625f,-0.25299999621510504965726795489899814128875732421875f,0.1868000013351440724562735340441577136516571044921875f,0.1178999966293573231634894682429148815572261810302734375f,0.076199995811283560698257133481092751026153564453125f,-0.178799999383091934301859282641089521348476409912109375f,-0.108300001528859102251090007484890520572662353515625f,0.062700003120303104964250451303087174892425537109375f,-0.51240001151561731518313536071218550205230712890625f,-0.12099999541044237361120394780300557613372802734375f,-0.449599990779161462484125877381302416324615478515625f,0.040599990212917713705564892734400928020477294921875f,-0.7348000070869920019589471849030815064907073974609375f,0.67599996873736412084099356434307992458343505859375f,0.42859999434426432340927703990018926560878753662109375f,-0.027599986994266689777077772305347025394439697265625f,-0.7442999813675876819019094909890554845333099365234375f,0.1690999992206692059681927275960333645343780517578125f,-0.10260000580698258598744132541469298303127288818359375f,0.6541999931812281232623718096874654293060302734375f,-0.12390002668499988658368238247931003570556640625f,-0.408600011350214398220259681693278253078460693359375f,0.0669000066205860388635073832119815051555633544921875f,0.835299998235701668392039209720678627490997314453125f,0.3420999979317185246685539823374710977077484130859375f,-0.550599992001057358947946340776979923248291015625f,0.5748999979555602379832635051570832729339599609375f,-0.2953999941706655363304889760911464691162109375f,-0.7711000078856944384142479975707828998565673828125f,0.30830000503063192951458404422737658023834228515625f,-0.34010000994205480395748963928781449794769287109375f,-0.743599974954128217774496079073287546634674072265625f,-0.1486999719321724366949410978122614324092864990234375f,-0.6952999906361103654717226163484156131744384765625f,0.25750002339482325197650425252504646778106689453125f,0.5389999837279315642035726341418921947479248046875f,0.376900003379583292684174011810682713985443115234375f,-0.4003999976724383902393356038373894989490509033203125f,0.63780002580285100055590419287909753620624542236328125f,-0.3525999995112414353570784442126750946044921875f,0.620700001794099787133518475457094609737396240234375f,0.06260000278949739982436994978343136608600616455078125f,-0.155399990147352173863737334613688290119171142578125f,-0.255199997532367717223422687311540357768535614013671875f,-0.230100001318007674200316614587791264057159423828125f,0.08479999747574329393984271519002504646778106689453125f,-0.1257000029861927714591729454696178436279296875f,-0.032700000996887690352110666935914196074008941650390625f,-0.118700002218782907359440059735788963735103607177734375f,-0.1306000077217817079144879244267940521240234375f,-0.463099995577335388219353262684307992458343505859375f,-0.101399996919929957783779173041693866252899169921875f,0.7089000065311790166333594243042171001434326171875f,-0.27130000959038735874884196164202876389026641845703125f,-0.37610001280307781001965850009582936763763427734375f,-0.03780000590980048258415990858338773250579833984375f,0.70530001194477076609246068983338773250579833984375f,0.08620000255405901867788998060859739780426025390625f,0.1869000005483627546709612943232059478759765625f,0.113299996238946931725877220742404460906982421875f,-0.605099994295835497126745394780300557613372802734375f,0.33600000002980234814486948380363173782825469970703125f,0.11080000890493391096924824523739516735076904296875f,0.08170000122785570084715800476260483264923095703125f,0.6169000047206878445393840593169443309307098388671875f,-0.3274000032454729147701755209709517657756805419921875f,0.098699991154670652093727767351083457469940185546875f,0.106800003197789206144108220541966147720813751220703125f,0.19460000404119492944943203838192857801914215087890625f,0.2090000039190054170745725059532560408115386962890625f,-0.69169999705553042446126710274256765842437744140625f,-0.143499988853931004229025347740389406681060791015625f,-0.30530001060366629417330841533839702606201171875f,-0.2088999851107598004773535649292171001434326171875f,0.191800008487701401094227549037896096706390380859375f,0.9385999950706953587342695755069144070148468017578125f,0.971000016942620280957498835050500929355621337890625f,0.593899968421459423240094110951758921146392822265625f,-0.0298000513434415470470639775157906115055084228515625f,-0.052200004744529682199072340154089033603668212890625f,0.7173000255644323974735243609757162630558013916015625f,0.53619998668432256039295680238865315914154052734375f,1.216899977600574001002087243250571191310882568359375f,-0.21589998874664306782733547152020037174224853515625f,-1.037499981075524768669993136427365243434906005859375f,0.4004999667108062322284922629478387534618377685546875f,0.75049999301135539298002186114899814128875732421875f,0.36079997619688553012196052804938517510890960693359375f,0.47459998552650206615766137474565766751766204833984375f,0.5496000028491021538457061978988349437713623046875f,-0.84590004134774243738092991407029330730438232421875f,0.05490002373456892570402487763203680515289306640625f,-1.078599999988079094492832155083306133747100830078125f,-0.23150001479685311522871415945701301097869873046875f,-0.500600009211897845062821943429298698902130126953125f,-0.469199994671344544627800132730044424533843994140625f,-0.9011000199556351564211809090920723974704742431640625f,-0.11400000183284297872177148747141472995281219482421875f,0.0430000057891011255861712925252504646778106689453125f,0.358499991089105574104678453295491635799407958984375f,0.3930000059008598167764603203977458178997039794921875f,0.3083000019013881765062023987411521375179290771484375f,-0.5228999910414218188492441186099313199520111083984375f,0.1468999989092349967023665158194489777088165283203125f,-0.100500000536441813636656661401502788066864013671875f,-0.12260000800490378214391284927842207252979278564453125f,-0.86860000520348557007110912309144623577594757080078125f,-0.27589999776184559454605960127082653343677520751953125f,-0.06399999773502340616460060118697583675384521484375f,-0.03139999881982802332203164041857235133647918701171875f,0.098399999773502333511032702517695724964141845703125f,-0.1093999983131885844755970538244582712650299072265625f,-0.438200006860494539751016418449580669403076171875f,0.1902999984592199655963895565946586430072784423828125f,-0.1319999914169311949763141456060111522674560546875f,-0.24929999567270277882613527253852225840091705322265625f,0.06430000252723699905033072354854084551334381103515625f,-0.107199996027350419325330221909098327159881591796875f,-0.201199995923042218581855422598891891539096832275390625f,0.064399998685717552149299081065692007541656494140625f,-0.7519000038266181462631720933131873607635498046875f,0.0742999922454357086820664335391484200954437255859375f,-0.18230000485181818703495082445442676544189453125f,-0.1712999987125396472720240126363933086395263671875f,0.68860000721514225130448494383017532527446746826171875f,0.12480000239312649679135347469127736985683441162109375f,0.308900008505582857054605483426712453365325927734375f,-0.127599999660253538014131891031865961849689483642578125f,0.424099993893504112474346356975729577243328094482421875f,-0.06570000149607657657924164595897309482097625732421875f,0.1733999778687955739542303490452468395233154296875f,-0.6156999956846238575280949589796364307403564453125f,0.18850001388788228950943448580801486968994140625f,-0.3490000123381609142825254821218550205230712890625f,-0.6767000104665754633970209397375583648681640625f,-0.39239999087750920825357070498284883797168731689453125f,-0.2436999950662255276512269119848497211933135986328125f,0.91679997630119292040262735099531710147857666015625f,0.564400019770860961187963766860775649547576904296875f,-0.5899000022768969841990838176570832729339599609375f,0.54290003216862725565761138568632304668426513671875f,0.45129999513626106732999687665142118930816650390625f,-0.06090001169443137740699967253021895885467529296875f,-0.9015999951601028072900589904747903347015380859375f,-0.22579998968243586432436131872236728668212890625f,-0.06690000501871085081262435778626240789890289306640625f,0.871899998052418023775089750415645539760589599609375f,0.04950000035762780470349753159098327159881591796875f,-0.5580000044107435996920685283839702606201171875f,0.25289999528825291008615749888122081756591796875f,-0.22600000159442412694943413953296840190887451171875f,0.054800009657442583232977995066903531551361083984375f,0.2207000076428056178912129325908608734607696533203125f,0.4414000208735462393860871088691055774688720703125f,-0.91830003364086199013627265230752527713775634765625f,0.63650000332295852967234850439126603305339813232421875f,0.7130000219568610742015835057827644050121307373046875f,-0.18559999438524232839853311816114000976085662841796875f,-0.8116000123336910565541302275960333645343780517578125f,-0.415199983078241408041009208318428136408329010009765625f,-0.609100000375509154082465101964771747589111328125f,0.02210000031590464431019427138380706310272216796875f,0.66330000726580617964600605773739516735076904296875f,-0.06419999184012414872313456726260483264923095703125f,0.58979999904036528590722809894941747188568115234375f,-0.005599999004602451702083953932742588222026824951171875f,0.485499994352459884883188578896806575357913970947265625f,-0.2648000047773122478389495881856419146060943603515625f,-0.4975000004470349068697032635100185871124267578125f,0.121100006023049378001132936333306133747100830078125f,0.389000007569789918449032484204508364200592041015625f,-0.0129000007271766747862784541212022304534912109375f,-0.497500001937150937436626918497495353221893310546875f,0.06640000276863577965968943317420780658721923828125f,0.91770001049637794920954547706060111522674560546875f,0.0245000027418136590018349352249060757458209991455078125f,-0.01989999784380191982791075133718550205230712890625f,0.130899996644258553857298466027714312076568603515625f,-0.33089998338222503804217922152020037174224853515625f,0.0218999993562698502813645973219536244869232177734375f,-0.33619999614655970976428989160922355949878692626953125f,0.125599998967349524292558271554298698902130126953125f,-0.362800001327693466546264744465588591992855072021484375f,0.070199994105100582686418420053087174892425537109375f,-0.63460001223683359938831927138380706310272216796875f,-0.147300004479289092795823989945347420871257781982421875f,-0.444700018826127141391424402172560803592205047607421875f,0.0952000010341405855651686351848184131085872650146484375f,0.039300003011524697882350665167905390262603759765625f,0.163199999260902395281647159208660013973712921142578125f,-0.50440000978708265932937138131819665431976318359375f,-0.10689998266696942863518415833823382854461669921875f,-0.1153999961078167046935050166212022304534912109375f,-0.8987999906659123183771953335963189601898193359375f,0.08160001423358931305074293049983680248260498046875f,0.0387000033736217829982706462033092975616455078125f,1.0864000167012211051087433588691055774688720703125f,0.35510001014694581034092379923094995319843292236328125f,0.13230000062361357837659170400002039968967437744140625f,0.2528999992370604132929656771011650562286376953125f,0.7081999898731705656018675654195249080657958984375f,0.49329997373819356454305307124741375446319580078125f,0.019400018727779855254311769385822117328643798828125f,-0.1219000085949897282944220933131873607635498046875f,-0.68620000784397117143953437334857881069183349609375f,0.27069995597302953793672486426657997071743011474609375f,0.4754999821707606155740677422727458178997039794921875f,-0.5282999922156330629974263501935638487339019775390625f,-0.22540000288784500792615972386556677520275115966796875f,0.7279999683499338569703240864328108727931976318359375f,0.44289999394714840708076053488184697926044464111328125f,-0.49209998079538319615267027984373271465301513671875f,-0.581000000625848667112904877285473048686981201171875f,0.15680000345855926724425444263033568859100341796875f,0.1701999972715972919701243881718255579471588134765625f,-0.091600001120567153822804584706318564713001251220703125f,-0.1938000075601041538586599699556245468556880950927734375f,-0.57480000008344678263938476447947323322296142578125f,-0.25149999028444280924077247618697583675384521484375f,-0.555599983060359736697364496649242937564849853515625f,0.68229999740123759011112269945442676544189453125f,0.208899996808171284357058539171703159809112548828125f,-0.34739999557137490882041674922220408916473388671875f,0.105200006324052797168633333058096468448638916015625f,0.810599999451637298619743887684307992458343505859375f,-0.2631000098824500810223980806767940521240234375f,0.50449999678134904712578645558096468448638916015625f,0.098499999526888093459575657107052393257617950439453125f,-0.6491000107504427685256587210460565984249114990234375f,0.10109999574124817200981851783581078052520751953125f,-0.562999986976385091708152685896493494510650634765625f,-0.2536999990522861292419065648573450744152069091796875f,0.00089999917745586799355805851519107818603515625f,-0.092899998492002477945561622618697583675384521484375f,0.3893000031232833890726396930404007434844970703125f,-0.250699995610117853939158294451772235333919525146484375f,0.233300018218159765925889814752736128866672515869140625f,0.176700003835558894227375503760413266718387603759765625f,0.226600005963444761647451741737313568592071533203125f,-0.239599998229742039423939559128484688699245452880859375f,-0.327599999138712849600096888025291264057159423828125f,0.3141000014930963590842338817310519516468048095703125f,-0.11339999247193333076211274601519107818603515625f,0.1295999943926930964011035030125640332698822021484375f,-0.489299984757602224316741512666339986026287078857421875f,0.03400000175833702009686732026239042170345783233642578125f,0.03910000214502218351952933517168276011943817138671875f,-0.03280000136494631846062475233338773250579833984375f,0.762400011962652168762133442214690148830413818359375f,0.26800000157952308388331630339962430298328399658203125f,-0.5589000383079056888391278334893286228179931640625f,-0.57919999079704265199097790173254907131195068359375f,-0.85560001375675209800419906969182193279266357421875f,-0.41349997127056159484936870285309851169586181640625f,0.473300000262260045502671346184797585010528564453125f,0.55309998455047593157729579615988768637180328369140625f,0.8727000120982528752477946909493766725063323974609375f,-0.1295000265836705466426792554557323455810546875f,1.0045000121295455386416506371460855007171630859375f,0.4626999534249307544087059795856475830078125f,-0.25300005716085482987409704946912825107574462890625f,-0.722399985587596660252529545687139034271240234375f,0.42300003123283413941635444643907248973846435546875f,-0.019999987483025005730041812057606875896453857421875f,-0.9369000136613847562472301433444954454898834228515625f,-0.684399992501734999450491159223020076751708984375f,-0.0448999855875964470897088176570832729339599609375f,0.55549996417760905131899562547914683818817138671875f,0.208199977058172169819272312452085316181182861328125f,1.087799956810474810708910808898508548736572265625f,0.23270000393986745024221818312071263790130615234375f,0.176499984025955569677535095252096652984619140625f,0.1740999923288821804590043029747903347015380859375f,0.42099998497962953791784457280300557613372802734375f,1.054200034010410735874074816820211708545684814453125f,0.859299981963633996429052785970270633697509765625f,0.50440002528429062067516497336328029632568359375f,-0.60289999238252622859590701409615576267242431640625f,0.111499995797872397673700106679461896419525146484375f,0.4385999931335451407221626141108572483062744140625f,-0.0682000076055526704976728069595992565155029296875f,0.03840000439286228584023774601519107818603515625f,0.633799994838237790162338569643907248973846435546875f,0.1998999958693981060520172832184471189975738525390625f,0.00469999833703038660814854665659368038177490234375f,-0.2027000014811754236010443719351314939558506011962890625f,-0.2276000057324767400501741576590575277805328369140625f,-0.07529998683631422551343348459340631961822509765625f,0.27579999668598187412271727225743234157562255859375f,-0.339099998438358252172974971472285687923431396484375f,0.24570001982450495603416129597462713718414306640625f,0.099200005549192393772273135255090892314910888671875f,0.600599990212917322907060224679298698902130126953125f,-0.00520000446140767014213679431122727692127227783203125f,0.080899996942281671152841227012686431407928466796875f,0.48880000407695778363859062665142118930816650390625f,-0.313799969059228800460914499126374721527099609375f,-0.2822999931544065699284828951931558549404144287109375f,-0.01580000458359709369915435672737658023834228515625f,-0.315399996778368940653081153868697583675384521484375f,-0.5663000019907951099185083876363933086395263671875f,-0.24049999316036707597277199965901672840118408203125f,0.335199982035160104487658827565610408782958984375f,-0.0434000012636184262504457365139387547969818115234375f,-0.4461999935388565319271947373636066913604736328125f,-0.25090000662803646491738618351519107818603515625f,-0.45890000090599070148300597793422639369964599609375f,0.0909000034242868348854926807689480483531951904296875f,0.494400002038478891819295313325710594654083251953125f,0.02980000045597552826848186668939888477325439453125f,-0.6378999767363069395287311635911464691162109375f,-0.235899987852573378432907702517695724964141845703125f,-1.1482000222086907736951388869783841073513031005859375f,0.243399964904785548469590139575302600860595703125f,-0.1774000043630596934463028446771204471588134765625f,0.567199968981743296581043978221714496612548828125f,0.9504999851882456329832393748802132904529571533203125f,-0.38549999636411680370429166941903531551361083984375f,-0.7169000109791756614185942453332245349884033203125f,0.888799998712539007783561828546226024627685546875f,0.51180001936554919694799536955542862415313720703125f,-0.63999998599290819356610882095992565155029296875f,-1.4623000447869305418180374545045197010040283203125f,0.430399980461597664316286682151257991790771484375f,-0.400900023615360368012261460535228252410888671875f,0.45499999195337270663230810896493494510650634765625f,1.137900032615661860546651951153762638568878173828125f,-0.83669999884366941245161797269247472286224365234375f,-0.135999993920326023300049200770445168018341064453125f,-0.182200002807378691471740239649079740047454833984375f,0.3996000138759614461747560199000872671604156494140625f,0.00569999181032176949202039395458996295928955078125f,0.810800037068128975903391619795002043247222900390625f,-0.388199991512298225870836176909506320953369140625f,-1.07800001409649848227445545489899814128875732421875f,0.40940000472068760473121074028313159942626953125f,0.88379999841451617470511337160132825374603271484375f,0.03799999859929081846843246239586733281612396240234375f,0.03430000047832726384200441316352225840091705322265625f,0.78379999155998181237237076857127249240875244140625f,0.178799993050098304792072667623870074748992919921875f,0.13549999941885471077540614714962430298328399658203125f,0.02240000376701356543662768672220408916473388671875f,-0.18099999600648875031083662179298698902130126953125f,0.137400014197826436657123849727213382720947265625f,-0.19319999561011791389120162421022541821002960205078125f,-0.2696999992310999960665185426478274166584014892578125f,-0.012900000280141821207280372618697583675384521484375f,0.5171000041902065191834481083787977695465087890625f,-0.077600010314583744985839075525291264057159423828125f,-0.890899999624490845917534898035228252410888671875f,0.01800000619888297848092406638897955417633056640625f,0.7813999867498875318005957524292171001434326171875f,-0.217999996960163144876787555404007434844970703125f,0.0133999927699566256933394470252096652984619140625f,-0.1266000029087066547361928314785473048686981201171875f,-0.299600004786253037991627934388816356658935546875f,0.1827000061750412296390777555643580853939056396484375f,0.5707000144600868907218682579696178436279296875f,0.00540000147223473891955336512182839214801788330078125f,0.1822000029563903833462745751603506505489349365234375f,-0.1260999987214803663704287828295491635799407958984375f,0.24279999480843539316765600233338773250579833984375f,0.002600006440281799058311662520281970500946044921875f,0.5022999847352505042863413109444081783294677734375f,0.054599994951486596761469627381302416324615478515625f,-0.64879999871253968279916080064140260219573974609375f,-0.04639999986290932465493597192107699811458587646484375f,0.059999997094273560538368883499060757458209991455078125f,-0.1530000075399875480997025078977458178997039794921875f,-0.7054000182732942381136354015325196087360382080078125f,-0.517899996033310827669993159361183643341064453125f,-0.013799995657801655823959663393907248973846435546875f,0.13589999746382208645201217223075218498706817626953125f,-0.0603999999448656677003555159899406135082244873046875f,-1.0778999801635738009508713730610907077789306640625f,-0.30390002549290695554873309447430074214935302734375f,-0.79949999827146545072764638462103903293609619140625f,-0.5375999959349631041050088242627680301666259765625f,0.14409997310638456013975883251987397670745849609375f,1.00850000032782549652665693429298698902130126953125f,0.0798000075340272729818025254644453525543212890625f,-1.10220003365278262918991458718664944171905517578125f,-0.58699998903274497319415559104527346789836883544921875f,-0.392899996443092647258055194470216520130634307861328125f,-0.31609999156892321092726660936023108661174774169921875f,-0.2856999941572546664048104503308422863483428955078125f,-0.215199991273880897324488614685833454132080078125f,0.83940001470446556908200363977812230587005615234375f,-0.1801999823331836836359798326157033443450927734375f,-0.6027999933183192382557535893283784389495849609375f,-0.31620001216530813081817541387863457202911376953125f,-0.442400010174512914318256662227213382720947265625f,-0.6579000046014780167524804710410535335540771484375f,-1.2895000385046007096434550476260483264923095703125f,0.72599999690055749823613950866274535655975341796875f,-0.051400007909536693517793537466786801815032958984375f,-0.4163999877929693838041202980093657970428466796875f,-1.12980001841187505107200195197947323322296142578125f,-0.3945999869048595343201668583787977695465087890625f,0.11560000150054694501022822805680334568023681640625f,-0.06569999598264697482363771996460855007171630859375f,0.75939999138414859825019220807007513940334320068359375f,0.093299994003772812600772113000857643783092498779296875f,-0.370399979045987148840168856622767634689807891845703125f,0.313100005486607546600907880929298698902130126953125f,0.563400010204315293549370835535228252410888671875f,0.107099998825788578216133828391321003437042236328125f,0.719700024712085717482068503159098327159881591796875f,-0.25449999476969242806490001385100185871124267578125f,0.364799997103214312943464392446912825107574462890625f,0.209799998742342008739569791941903531551361083984375f,-0.5207999895334243234401583322323858737945556640625f,0.2330000041872263005782173195257200859487056732177734375f,0.17470000656992201726325220079161226749420166015625f,0.028899998447299013459854677421390078961849212646484375f,0.318900002023577655574371192415128462016582489013671875f,-0.116000008150935141060244859545491635799407958984375f,0.47770000274777402182735386304557323455810546875f,0.180199993881583253596545546315610408782958984375f,-0.17189998967051511868930901982821524143218994140625f,-0.1230000000894069245305217918939888477325439453125f,0.6746000125348567966199198053800500929355621337890625f,0.026700008082389814489943091757595539093017578125f,0.7741000048458577253995827049948275089263916015625f,-0.3020000107139348966001080043497495353221893310546875f,-0.15830000815987599338541258475743234157562255859375f,-0.007600006291270222646971888025291264057159423828125f,0.56130000959038728325367628713138401508331298828125f,0.1382000004529952974507978069595992565155029296875f,-0.46740000391602531948365140124224126338958740234375f,0.4481999912142757835908923880197107791900634765625f,-0.66310001688599573554938615416176617145538330078125f,0.3791000112533566834827070124447345733642578125f,-0.589100022727251371179590933024883270263671875f,0.276899984008073918317904826835729181766510009765625f,0.24069999777078632252624856846523471176624298095703125f,0.5471999755382537244940976961515843868255615234375f,0.42319998316764806389755904092453420162200927734375f,0.559899992553889891411245116614736616611480712890625f,-0.5217000094607471272212251278688199818134307861328125f,-0.508399987852573786994980764575302600860595703125f,-0.58420000063180932414752533077262341976165771484375f,0.419199981260300003071961327805183827877044677734375f,1.0178000110507012809790694518596865236759185791015625f,-0.96190000442266399005575294722802937030792236328125f,-0.5890000102519987734694950631819665431976318359375f,0.1950999895833434027991870607365854084491729736328125f,-0.32970000066533666771562138819717802107334136962890625f,0.3616999770879747444496388197876513004302978515625f,-0.5977999983847137599468624102883040904998779296875f,0.60879998820722125429938387242145836353302001953125f,-0.594800007161497834573538057156838476657867431640625f,-0.908399979209900010346245835535228252410888671875f,-0.2706000116705897795554847107268869876861572265625f,0.2377000096768142611836083233356475830078125f,-0.104499998643994285174585456843487918376922607421875f,0.727499973177909708965671597979962825775146484375f,0.1039999883472919606219875277020037174224853515625f,-0.340799985323846088558497058329521678388118743896484375f,0.539499995440244628497339363093487918376922607421875f,0.065800001975893973593656482989899814128875732421875f,-0.64489999691247934521243223571218550205230712890625f,0.0040999961286782848901566467247903347015380859375f,-0.42600001001358034358190707280300557613372802734375f,-0.131299998712539667256038455889211036264896392822265625f,-0.03609999946653841274013529982767067849636077880859375f,-0.323199999558925643583506825962103903293609619140625f,0.07719999341964722816555877216160297393798828125f,-0.0279000003919005479247061884962022304534912109375f,-0.5683000026836990681289307758561335504055023193359375f,0.013199997174739852567881825962103903293609619140625f,0.595200003790855358687394982553087174892425537109375f,-0.10879999670088291718883510839077644050121307373046875f,-0.29250000327825542711934758699499070644378662109375f,0.161500008463859590079891859204508364200592041015625f,0.788300017845630751622820753254927694797515869140625f,-0.064699998839944594664075339096598327159881591796875f,-0.0648999987937509958957349454067298211157321929931640625f,-0.18689999779164789828200810006819665431976318359375f,0.032400001233816144718957730219699442386627197265625f,-0.0096000001668929968445809208787977695465087890625f,-0.66409999776780603841785932672792114317417144775390625f,0.16840000051856041096698390902020037174224853515625f,0.69420000569820405900145487976260483264923095703125f,0.05990000160634512127444395446218550205230712890625f,-0.41130001532733440061662122388952411711215972900390625f,-0.1239000048547982402169509441591799259185791015625f,-0.575599980527162546906083662179298698902130126953125f,0.2244000019267201329054017833186662755906581878662109375f,-0.468399997463821993193278103717602789402008056640625f,-0.527399992740154033299404545687139034271240234375f,-1.0410999818086619228552081040106713771820068359375f,-0.961799987924098598313094043987803161144256591796875f,-0.6946000173032285118779327603988349437713623046875f,-0.821399999713897432940257203881628811359405517578125f,-0.98730002318024645546756801195442676544189453125f,0.446300001953542080368464439743547700345516204833984375f,0.43990001015588642285791820540907792747020721435546875f,-1.1063000019907942572672254755161702632904052734375f,-0.2959999992847439642673634807579219341278076171875f,-0.54679999150037783550715175806544721126556396484375f,-0.0852999876558779845936442143283784389495849609375f,0.144899997843801864316759520079358480870723724365234375f,0.208600003918260334945244949267362244427204132080078125f,0.47879998813271507085431721861823461949825286865234375f,0.5295000073984266197868464587372727692127227783203125f,-0.08230000634193412878403250942938029766082763671875f,0.581800004464387843228223573532886803150177001953125f,0.465199973690510137913634025608189404010772705078125f,0.48059999885559090415654281969182193279266357421875f,-0.57320000946819771314721947419457137584686279296875f,0.099900015762448557410380090004764497280120849609375f,0.252100001657009187283620121888816356658935546875f,0.57949999976158128589531770558096468448638916015625f,0.6639999992251388594155514510930515825748443603515625f,0.32290001421272751525037847386556677520275115966796875f,0.5911000158950685534620106409420259296894073486328125f,0.34730001369938257038683104838128201663494110107421875f,-1.205699963200093094428666518069803714752197265625f,-0.897499944567680785212360206060111522674560546875f,-1.311299988821893247603611598606221377849578857421875f,-0.06089996365681382695100865021231584250926971435546875f,0.0494999899268151466458220966160297393798828125f,-0.068899986302852767039439640939235687255859375f,-0.096000004164874308543176084640435874462127685546875f,0.107799994584917957496372764580883085727691650390625f,-0.82529999480843496684201454627327620983123779296875f,0.94789999804496716251378529705107212066650390625f,0.3164999537765975645697835716418921947479248046875f,0.566500025749206770342425443232059478759765625f,1.1565000533163551921234102337621152400970458984375f,0.506899952864646508032819838263094425201416015625f,-0.4215000133812427662860500277020037174224853515625f,0.12009999623298650561764588928781449794769287109375f,-1.0617000000357623701319198517012409865856170654296875f,-0.02669997872710272446283852332271635532379150390625f,0.5294999950677148348177070147357881069183349609375f,-1.19309999274611389097344726906158030033111572265625f,-0.9252000118374821280298192505142651498317718505859375f,-0.51329997865557697167560036177746951580047607421875f,-0.32579999955445515524843358434736728668212890625f,1.021299988654255752607014073873870074748992919921875f,1.4142000428020953251007085782475769519805908203125f,-0.24050000143051164513963158242404460906982421875f,-0.91800002206861963660600167713710106909275054931640625f,0.256400026163458694128394199651665985584259033203125f,0.1535999958902596329579637313145212829113006591796875f,-0.6737999963283536342117940876050852239131927490234375f,-1.55400005853176192971432101330719888210296630859375f,0.395300008778274314380496434750966727733612060546875f,-0.1406999906182289095113446819595992565155029296875f,0.5896999956920740970645056222565472126007080078125f,0.128400004541874057650829854537732899188995361328125f,0.7834000080063938664665101896389387547969818115234375f,0.3749999907612799887601795489899814128875732421875f,-0.3569000054657458509410616898094303905963897705078125f,-0.01520000513195984837011565105058252811431884765625f,-0.8103999789714813051233477381174452602863311767578125f,0.4499000085353852540492880507372319698333740234375f,0.89049998887628323274157082778401672840118408203125f,-0.01079999430179601205281869624741375446319580078125f,0.41289998075962086598877931464812718331813812255859375f,-0.139899994863569798742020111603778786957263946533203125f,-0.84670001625940194411867878443445079028606414794921875f,-0.20499999731779094958028508699499070644378662109375f,-0.04649998778849850911143448684015311300754547119140625f,-0.16620000814199453742503465036861598491668701171875f,0.3597999965816736089863070446881465613842010498046875f,0.0425000098347663168851795489899814128875732421875f,0.7282999747067691043156401065061800181865692138671875f,-0.449800006937980612065075547434389591217041015625f,-0.013799983587861053280221312888897955417633056640625f,0.15710000672340385108327609486877918243408203125f,1.0885999771893024590241338955820538103580474853515625f,0.0746000164091586981385262333787977695465087890625f,-0.0602999880656602815776068382547236979007720947265625f,-0.079099993968009929545814884477294981479644775390625f,0.67440000501871100180295570680755190551280975341796875f,-0.10769999336004243417619363754056394100189208984375f,-0.402099978709220096106946584768593311309814453125f,0.41639998376965525750392771442420780658721923828125f,1.553099967861175656480554607696831226348876953125f,-0.9891000403106218197279986270586960017681121826171875f,0.28820001788735449821388101554475724697113037109375f,-0.56909996699690790222803116193972527980804443359375f,-1.0462999701023107945729861967265605926513671875f,0.38400000384449928247931893565692007541656494140625f,-1.0487999853014944307005862356163561344146728515625f,-1.40820004321932845670062306453473865985870361328125f,0.2834999837875376016427253489382565021514892578125f,-0.652500012665986783844118690467439591884613037109375f,-0.1201000066637991636753213242627680301666259765625f,1.320900015848130326645559762255288660526275634765625f,0.03629992418810756849012477687210775911808013916015625f,0.338999987453222129119012606679461896419525146484375f,0.1445000036358834449856658466160297393798828125f,0.430800027084350833916914780274964869022369384765625f,1.10020000945329599062461056746542453765869140625f,-0.72030001656413045196813982329331338405609130859375f,0.4556999825716019358878838829696178436279296875f,0.7872000038504598506960974191315472126007080078125f,0.54740001449584951842552982270717620849609375f,-1.0464000101447101798157746088691055774688720703125f,0.83060000541210143865100690163671970367431640625f,-0.75900003901123991312260841368697583675384521484375f,-1.08249997258186336779317571199499070644378662109375f,-0.6966000277936459372796207389910705387592315673828125f,0.24480002824664115479436077293939888477325439453125f,-0.819000051379204396795330467284657061100006103515625f,0.24059998078793309872480676858685910701751708984375f,0.04759999653100965844032543827779591083526611328125f,0.1540000018328429032266058129607699811458587646484375f,0.15270001094341267844356480054557323455810546875f,0.7989999858513472386079001807956956326961517333984375f,-0.305700002837181017412149230949580669403076171875f,0.14850002532452333259271881615859456360340118408203125f,0.2051999885916711008349011535756289958953857421875f,-0.319099996612965952635931898839771747589111328125f,-0.061800004315376355634725769050419330596923828125f,-0.0798000145375727942820276439306326210498809814453125f,0.249400006508827321027865764335729181766510009765625f,-0.6708999800294639381803563082939945161342620849609375f,-0.21749999662861230209642826594063080847263336181640625f,-0.003699997040629343647566429353901185095310211181640625f,-0.182700000810623208735705702565610408782958984375f,0.353799982134998192151442708563990890979766845703125f,-0.128400007820129502533745835535228252410888671875f,-0.88660001717656822251001358381472527980804443359375f,0.257299996805191000248669297434389591217041015625f,-0.201799995039403601282401723437942564487457275390625f,0.24589999738931656025897609652020037174224853515625f,-0.305899985207617408633495870162732899188995361328125f,0.3981999944925309620202824589796364307403564453125f,-0.25729999624639743327492169555625878274440765380859375f,-0.09129999960660928337574659963138401508331298828125f,0.592900000354647627176518653868697583675384521484375f,0.04479999546408652832951702293939888477325439453125f,0.487099972264469049942903211558586917817592620849609375f,0.12400000742077832427412431570701301097869873046875f,-0.0177999881029133888432625099085271358489990234375f,-0.673799999606609301139314993633888661861419677734375f,-0.581999962508678692074681748636066913604736328125f,-0.7461000098228449672888018540106713771820068359375f,0.683000008583068307643770822323858737945556640625f,0.12059998499751056755258105113171041011810302734375f,0.102300040316581686283825547434389591217041015625f,0.48900001755356736765634195762686431407928466796875f,0.297899972414970903855646611191332340240478515625f,0.730200029718876209017253131605684757232666015625f,0.786999968767165825056508765555918216705322265625f,-0.2313000264286999563267954727052710950374603271484375f,-1.32829999772906237609504387364722788333892822265625f,-0.037800010622292656847776015638373792171478271484375f,-0.3094999872073531665961354519822634756565093994140625f,-0.3807000040292738418656881549395620822906494140625f,-0.055000003576278544414890347979962825775146484375f,0.9873000432968144224332718295045197010040283203125f,-0.489800035548210388469669851474463939666748046875f,-0.2240999820470808145955743384547531604766845703125f,-0.404199975299835756459287949837744235992431640625f,-0.679799997550249113231757291941903531551361083984375f,-1.2496999496102336735248172772116959095001220703125f,-0.0978000010669237695992705994285643100738525390625f,-0.174300024580954726616255356930196285247802734375f,0.025499986678361619141242044861428439617156982421875f,1.0742999903082850465807496220804750919342041015625f,-0.1589999917745590352069484652020037174224853515625f,0.770899961030483638069199514575302600860595703125f,-0.6831000282108785626178359962068498134613037109375f,0.50240000227689740608383317521656863391399383544921875f,-0.1267999926924705533792803180404007434844970703125f,0.095300015446543540065249544568359851837158203125f,0.066500001311302181505880071199499070644378662109375f,-0.173599999094009416200634632332366891205310821533203125f,0.349599987351894458953438515891321003437042236328125f,-0.695399972340464689324335267883725464344024658203125f,-0.17899999326467508353744051419198513031005859375f,0.25810002891868322194568463601171970367431640625f,-0.270500006496906308939287555404007434844970703125f,-0.644400013586878372962019057013094425201416015625f,0.167999999940395383646318805404007434844970703125f,0.017200002844631523402085804264061152935028076171875f,-0.2580999997496604692059918306767940521240234375f,0.4813999908477069045176222061854787170886993408203125f,0.1821999919295311798350667231716215610504150390625f,-0.42899997836351422275669165173894725739955902099609375f,0.25790000460147854965953229111619293689727783203125f,-0.1777000015184284631430955414543859660625457763671875f,0.1059000015616415879549094825051724910736083984375f,-0.24690002725869408806147475843317806720733642578125f,-0.462099995100498261990651371888816356658935546875f,-0.21669999157935393441221094690263271331787109375f,0.16240000168085089171654544770717620849609375f,0.7206999909162521422700820039608515799045562744140625f,-0.063599997566640364965451226453296840190887451171875f,0.52309999878108504045570725793368183076381683349609375f,0.18049999498575919343323903376585803925991058349609375f,0.0647999975502491221135414889431558549404144287109375f,-0.3676999976813792869734243140555918216705322265625f,0.6892000165387985877174514826037921011447906494140625f,-0.59790002680420872849254010361619293689727783203125f,-0.79049995374679582482713158242404460906982421875f,-0.0894000011444091580159465593169443309307098388671875f,-0.3823999985516071120628112112171947956085205078125f,-0.4403999844849113554090536126750521361827850341796875f,-1.55019994165301344679619433009065687656402587890625f,-0.447600034826994175318759516812860965728759765625f,-0.328099944019317746324304607696831226348876953125f,0.8244000202178956016041411203332245349884033203125f,-0.4116000380754467613542146864347159862518310546875f,0.1623000036597250783643175964243710041046142578125f,0.4511999887704849498959447373636066913604736328125f,0.5033999950051306626619407325051724910736083984375f,1.3297999664068225200708184274844825267791748046875f,0.7014000187873836722474152338691055774688720703125f,0.353800001692771814987281686626374721527099609375f,-0.278999999970197531951043856679461896419525146484375f,-0.433100000345707059068445232696831226348876953125f,0.69120002855658579932196516892872750759124755859375f,0.60329993886947619330385350622236728668212890625f,0.7889999978840354089015818317420780658721923828125f,0.14430000431537592930908431299030780792236328125f,-1.3659000034987922589380104909650981426239013671875f,-0.315899947619438847823403193615376949310302734375f,-0.29730001315921550375520610032253898680210113525390625f,0.52410000678300860954550444148480892181396484375f,-0.663299983833729900339903906569816172122955322265625f,-0.58679996128827394130667016725055873394012451171875f,1.06349999451637255987179742078296840190887451171875f,1.069600014397501741569840305601246654987335205078125f,0.02789998821020134300852078013122081756591796875f,-1.066399977436661483665147898136638104915618896484375f,-0.374199997055530531042677466757595539093017578125f,-0.9192999963060020096605740036466158926486968994140625f,-0.1976000036276877036556243183440528810024261474609375f,-0.191699996496736968509821963380090892314910888671875f,0.101399996435642236747298738919198513031005859375f,-0.46820001974999880811623143017641268670558929443359375f,-0.121900000660121410334113534190692007541656494140625f,0.3647999875292182725416978428256697952747344970703125f,0.0589000013530254218352411044179461896419525146484375f,0.21189999592900277125551156132132746279239654541015625f,0.213400003089010692480798070391756482422351837158203125f,0.43549999455735088904617668958962894976139068603515625f,-0.2363000002026556245482424856163561344146728515625f,-0.711499969944358046092247604974545538425445556640625f,0.1216000012695790388761452049948275089263916015625f,-0.410999996639787923413678072392940521240234375f,0.0652000009596347407381244920543394982814788818359375f,0.08139999699443578595747794679482467472553253173828125f,0.0529000019378960129046163274324499070644378662109375f,-0.06399999654293064832444315470638684928417205810546875f,-0.104199986326694471472364966757595539093017578125f,0.221900018280744237841872745775617659091949462890625f,0.190100000703334881535511158290319144725799560546875f,-0.4424999867752195115144786541350185871124267578125f,-0.09969999699592602127040663617663085460662841796875f,-0.75930001225024457145451606265851296484470367431640625f,0.284200000408291852949105305015109479427337646484375f,1.85040001592636116356516140513122081756591796875f,0.19320001267194708560737126390449702739715576171875f,-1.75759997060298900350971962325274944305419921875f,1.175399996703863081393137690611183643341064453125f,-0.57470004110336248004387016408145427703857421875f,0.50700000993162397389113493773038499057292938232421875f,0.10569999955892550591585177244269289076328277587890625f,0.72600002193450929866003207280300557613372802734375f,0.219399935579300375820821500383317470550537109375f,0.7359000035732983224789904852514155209064483642578125f,1.5835999466419223846713748571346513926982879638671875f,-0.33510000331103795279119594852090813219547271728515625f,0.012100005754828391246746832621283829212188720703125f,-1.106700011938065220107318964437581598758697509765625f,-0.44829997103735819141689944444806315004825592041015625f,1.28189998430609630730714343371801078319549560546875f,0.132299973559380390497608459554612636566162109375f,-0.48330002380609560219681952730752527713775634765625f,-1.32300000679492857358354740426875650882720947265625f,-0.39149999900162202326470151092507876455783843994140625f,-0.310599984326958900027193521964363753795623779296875f,-0.23820000555664300190272797408397309482097625732421875f,-0.39279998869895937918528261434403248131275177001953125f,-0.5223000401675701453996225609444081783294677734375f,-0.415499965071678190042803180404007434844970703125f,-0.2028000104546550108608471418847329914569854736328125f,-0.91629999170899356641939448309130966663360595703125f,0.776400022214651652774364265496842563152313232421875f,-0.5035000317692759352894427138380706310272216796875f,0.1815000058561562212844364694319665431976318359375f,0.523399979880452281832958760787732899188995361328125f,-0.13110000073313710089450978557579219341278076171875f,-0.61739997161775850287313005537725985050201416015625f,0.1946999949097634186045979731716215610504150390625f,-0.9430999818682670454705885276780463755130767822265625f,-0.06899999460577976861941351671703159809112548828125f,-0.914499994285404849136966731748543679714202880859375f,0.21349999094009408651118064881302416324615478515625f,-0.7099999754875898627659580597537569701671600341796875f,0.0052000118374824211286977515555918216705322265625f,0.0557000116288659530283666754257865250110626220703125f,0.20239999333620062316185794770717620849609375f,-0.0966000068947672207997356963460333645343780517578125f,0.1961000122070313267386154620908200740814208984375f,-0.33909998566061272384786207112483680248260498046875f,0.070800010693073289758103783242404460906982421875f,0.333200004141032568583113970817066729068756103515625f,-0.39390000150203707818263865192420780658721923828125f,0.5871999951705337839058529425528831779956817626953125f,-0.18299999412894241146432250388897955417633056640625f,0.467800029714405207759142513168626464903354644775390625f,0.4360999844908715061819748370908200740814208984375f,-0.426500001907348380569828805164434015750885009765625f,-0.01190000248551381645256697083823382854461669921875f,-1.055500018492340874587398502626456320285797119140625f,-0.312099992492794964249469558126293122768402099609375f,-0.00799997252225903565658882143907248973846435546875f,0.0838999925613403974011816899292171001434326171875f,-0.9459999932870266636797396131441928446292877197265625f,0.39010000040531167542212642729282379150390625f,-0.4881999983668325882035787799395620822906494140625f,0.5637000008404253748039991478435695171356201171875f,-0.56059997173547770898949238471686840057373046875f,0.01650000235438309204027973464690148830413818359375f,-1.572899986720084797298113699071109294891357421875f,0.62700001704692898130133471568115055561065673828125f,-0.9746999955058104347926928312517702579498291015625f,-0.257999995172024210177141867461614310741424560546875f,-1.4819999595284460980337826185859739780426025390625f,-0.72930002830624562903949481551535427570343017578125f,-0.897299959850311523723576101474463939666748046875f,-0.5030999732255931888857958256267011165618896484375f,-0.6509999688863761235779747948981821537017822265625f,-0.75420004131197959651444762130267918109893798828125f,-1.4740999862194055225472766323946416378021240234375f,-0.8031000331282616144790154066868126392364501953125f,-0.3887999814271925203001956106163561344146728515625f,0.133499996900558581813811542815528810024261474609375f,0.8652000076651571447428068495355546474456787109375f,-0.42630003641247782297796220518648624420166015625f,0.1014000542521475978219314129091799259185791015625f,-0.30929997376799622799126154859550297260284423828125f,-0.7503000000238415623243781737983226776123046875f,0.75270003687143383075408564764074981212615966796875f,-1.0069000360131266091912038973532617092132568359375f,0.05579997202455988514202545047737658023834228515625f,-0.29679995797276514934992519556544721126556396484375f,0.24930000938177077074442422599531710147857666015625f,-1.383200007605552173117757774889469146728515625f,-0.4752000101983551161310970201157033443450927734375f,0.7011999814689160448466509478748776018619537353515625f,-0.1507999959409236367946505197323858737945556640625f,0.179000003956258257886702267569489777088165283203125f,-0.3358000020503997262721895822323858737945556640625f,-0.7606999840244650368248358063283376395702362060546875f,0.4645000065974891345632613592897541821002960205078125f,0.502299991887807717461100764921866357326507568359375f,-0.0103000104546548953976525808684527873992919921875f,-0.7537000238627193926532754630898125469684600830078125f,-0.26220000027418155497116458718664944171905517578125f,0.770699973183870401527428839472122490406036376953125f,0.2755999997496605402602654066868126392364501953125f,0.7462999782979491403267502391827292740345001220703125f,-0.086599995458126244329832843504846096038818359375f,-1.1107000010862948524703597286134026944637298583984375f,-0.315299995069205640785270361448056064546108245849609375f,0.24990001194402566842001078839530237019062042236328125f,0.198199997696280494352549794712103903293609619140625f,0.22219999014139180104621118516661226749420166015625f,0.1832000053703783937208982024458236992359161376953125f,0.46539997979104530401173178688623011112213134765625f,-0.2971000091075897131531746708787977695465087890625f,-1.0251999732807279830382185537018813192844390869140625f,-0.3464000104427338300183691899292171001434326171875f,-0.558600011983513500268827556283213198184967041015625f,-0.0824999976158142800386485760100185871124267578125f,-0.5468999999523163069170550443232059478759765625f,-0.16499999642372120689515213598497211933135986328125f,-0.1531999904692173952724942864733748137950897216796875f,0.0056999944925308909660088829696178436279296875f,0.564399994587898135023351642303168773651123046875f,0.26189998236894584948686315328814089298248291015625f,0.53499998390674630144303591805510222911834716796875f,0.94859999805092787283911093254573643207550048828125f,-0.5389000098466869559388214838691055774688720703125f,-1.005300015036761696052280967705883085727691650390625f,0.10969999967813504060387685967725701630115509033203125f,1.11210007005333988416850843350403010845184326171875f,0.075299970221518464086329913698136806488037109375f,0.108900025194884175760989819536916911602020263671875f,1.3364999986290921896170402760617434978485107421875f,-1.06710001491904282744371812441386282444000244140625f,0.277699984121323240060519310645759105682373046875f,0.942000032097102035777425044216215610504150390625f,-0.93690000710487453261521295644342899322509765625f,0.70649997115507667810874181668623350560665130615234375f,1.098899970451743257626731065101921558380126953125f,0.04020000922977917667822111980058252811431884765625f,0.71219997538924229019130507367663085460662841796875f,-1.13100000487267937643309778650291264057159423828125f,-0.458999983355403262663685381994582712650299072265625f,1.21350002461671824249833662179298698902130126953125f,-0.63759998013973273600640823133289813995361328125f,1.12680003620386148810439408407546579837799072265625f,0.08579997045993792426088475622236728668212890625f,0.29790000892281565114672048366628587245941162109375f,-0.7370999989748003855538627249188721179962158203125f,0.913199984061717540129166081896983087062835693359375f,0.7803999572157866992938579642213881015777587890625f,0.13500000104308096382510484545491635799407958984375f,-0.79069999218285058617539107217453420162200927734375f,-0.2458000060737133285471145427436567842960357666015625f,-0.2955999949067831256144245344330556690692901611328125f,0.0208000023484229412673585102311335504055023193359375f,0.5063999879792333569383799840579740703105926513671875f,0.1461999965190887706967259873636066913604736328125f,-0.99049997472018025401752083780593238770961761474609375f,0.288999988794326867491690791212022304534912109375f,-0.4807999864414334023621222513611428439617156982421875f,0.1952999983847141241000144873396493494510650634765625f,-0.44329999761730431995232493136427365243434906005859375f,0.0369999869465829078762908466160297393798828125f,-0.7028999870181082965103769311099313199520111083984375f,0.0292999993048608242673935819766484200954437255859375f,-0.060400002180039859267512980522951693274080753326416015625f,-0.1175999921351671417824036325328052043914794921875f,-0.452799978528917124176444986005662940442562103271484375f,-0.2210999958157540135061935870908200740814208984375f,0.2526999830409886182991385794593952596187591552734375f,0.000499995768070249368975055404007434844970703125f,0.2811000243142245391680944521795026957988739013671875f,-0.0329000019751488625985302860499359667301177978515625f,0.3344999832212927071140029511298052966594696044921875f,-0.317500003874302105799642959027551114559173583984375f,0.3367999726504089519352191928192041814327239990234375f,-0.1102999934673309923027773038484156131744384765625f,-0.1915999820470812853301367795211263000965118408203125f,0.08209999890029429092663804112817160785198211669921875f,0.299200001525878889196974341757595539093017578125f,0.2867000077843666616672635427676141262054443359375f,-0.357300051641463056739667081274092197418212890625f,0.93600002215802680094469678806490264832973480224609375f,0.444399994215368909777907902025617659091949462890625f,-0.50430000096261473618852733125095255672931671142578125f,0.302000008925795260239510753308422863483428955078125f,0.2484000109493731400789329200051724910736083984375f,-1.39869996969699883493376546539366245269775390625f,-0.116699985134602002290193922817707061767578125f,-0.757199982094764578732792870141565799713134765625f,-0.18059999296963213932798453242867253720760345458984375f,-0.881099992761015915476718873833306133747100830078125f,0.9909000222742552210775102139450609683990478515625f,0.059699976134300669627918978221714496612548828125f,0.031500003751367355864232422391069121658802032470703125f,-0.039200004059076309914644298260100185871124267578125f,-0.000900015568733092408137963502667844295501708984375f,-1.009999964535236660623240823042578995227813720703125f,-0.00240000272393192659592386917211115360260009765625f,0.230099985504150339465923025272786617279052734375f,-0.76860000818371787545402185060083866119384765625f,-0.522299922299385599444576655514538288116455078125f,0.45570000071376537409406637380016036331653594970703125f,1.239099943006039428183839845587499439716339111328125f,0.4092000356495388846411742633790709078311920166015625f,0.85199999216198829543600368197076022624969482421875f,-0.24300000977516145184154083835892379283905029296875f,-0.83319994978904787785722874104976654052734375f,-0.6165000023916362703602089823107235133647918701171875f,-0.1827999695137145774737064130022190511226654052734375f,-1.26630003358125708956549715367145836353302001953125f,-0.5435000076293945436844978758017532527446746826171875f,-0.2142999946668744382094473621691577136516571044921875f,-0.283699987243116014123955892500816844403743743896484375f,-0.11899999592825782901428510740515775978565216064453125f,0.3466999706059696872983977300464175641536712646484375f,0.00970000116825087133065608213655650615692138671875f,0.5747999724045398384220106891007162630558013916015625f,-0.20230001289844512513127483543939888477325439453125f,-0.9859999714940788084760470155742950737476348876953125f,-0.4998000057458877876115366234444081783294677734375f,0.0959999860599640708613833339768461883068084716796875f,-0.1483999999970198047094527282752096652984619140625f,-0.3147999904721976438537467402056790888309478759765625f,-0.0173999967634677066286030822084285318851470947265625f,0.57730000913590184818957595780375413596630096435546875f,0.273599994921684075421808302053250372409820556640625f,-0.575700015428662226213418762199580669403076171875f,-0.5400000122189521079008045489899814128875732421875f,0.3919000170513988745568667582119815051555633544921875f,-0.2847999959297478067554720837506465613842010498046875f,-0.40459998671859509489223682976444251835346221923828125f,0.2670999930143356237977059208787977695465087890625f,-0.39360000185072419043308400432579219341278076171875f,0.25769999172091484496149860206060111522674560546875f,-0.166400012715160638965272710265708155930042266845703125f,0.0601000026032327827607559811440296471118927001953125f,1.1942999587550762186083375127054750919342041015625f,0.12759999369978913819068111479282379150390625f,-0.558199982161819985293504942092113196849822998046875f,0.0541000136375426876611527404747903347015380859375f,-0.3125000241398812050874767010100185871124267578125f,0.06359998855143800255973474122583866119384765625f,-0.705499974682927355473793795681558549404144287109375f,-0.25919999506249968845850162324495613574981689453125f,0.1728999803498385556821403952199034392833709716796875f,-1.059899990653990986544386032619513571262359619140625f,-0.3487999739766127760276503977365791797637939453125f,1.202400035655498466979906879714690148830413818359375f,-0.4683999717593199108023327426053583621978759765625f,0.78390001133680353717636535293422639369964599609375f,0.697199971961975251133480924181640148162841796875f,0.33300000634789483910935814492404460906982421875f,-0.316399983322620759196297512971796095371246337890625f,-0.49260002774894273702699365458101965487003326416015625f,0.411100016006827662096156927873380482196807861328125f,-0.4040999561190596889304060823633335530757904052734375f,-1.35629997293353188325681912829168140888214111328125f,0.58260000264048539264649662072770297527313232421875f,0.942900028443336424288645503111183643341064453125f,0.46649998763948674618262657531886361539363861083984375f,-0.46139999583959567797109002640354447066783905029296875f,0.02280002386569979222485926584340631961822509765625f,-0.4208000008583070439271978102624416351318359375f,-0.05969997747540478627570337266661226749420166015625f,-0.1076000185847281187534463242627680301666259765625f,-0.72960000117272105768506662570871412754058837890625f,1.362099966117739757720528359641321003437042236328125f,0.465900008267163823205692096962593495845794677734375f,-0.284600000762939231435666442848742008209228515625f,-1.24140003726482373025419292389415204524993896484375f,-0.22810000690221787778000361868180334568023681640625f,-0.064200005847215635412794654257595539093017578125f,-0.1250999964192509583682522134040482342243194580078125f,0.2669999967813492247614703956060111522674560546875f,-0.003899996137619154978892765939235687255859375f,0.229399996078014201117412085295654833316802978515625f,-0.6888000224798915827051359883625991642475128173828125f,0.3343999939918518293779925443232059478759765625f,0.4883999897152186964177644767914898693561553955078125f,-0.3122000022113322614814023836515843868255615234375f,-0.5348000041812654981043806401430629193782806396484375f,0.153100009882450205367376838694326579570770263671875f,0.81080000544041386234539459110237658023834228515625f,-0.2031000000476836930829449556767940521240234375f,-0.54129998798370360102438780813827179372310638427734375f,-0.2171000064253807426695175308850593864917755126953125f,0.1009999988749623600625682229292578995227813720703125f,0.404099998438358198882269789464771747589111328125f,-0.3313000247895716565693646771251223981380462646484375f,-0.19789999670386304586600090260617434978485107421875f,0.5002000318050381366674628225155174732208251953125f,-0.15920000450611127007505274377763271331787109375f,-1.156000013999640696482629209640435874462127685546875f,-0.0848999991476535598167174612171947956085205078125f,-0.069800007534027042055413403431884944438934326171875f,-0.2614000058233738599255957524292171001434326171875f,-0.4135999960020184662567999112070538103580474853515625f,0.286200002777576401058468036353588104248046875f,-0.05629999811649355478948564268648624420166015625f,0.31739999631643289745852598571218550205230712890625f,0.6608999964952460004496970213949680328369140625f,0.4992000138938426090362554532475769519805908203125f,0.37669999377727503286905630375258624553680419921875f,0.06390000910162907388212261139415204524993896484375f,0.3878999932765960778624503291212022304534912109375f,-0.1433999587953083132418896639137528836727142333984375f,-1.26400000175833771010047712479718029499053955078125f,-0.9245999953985215569218780728988349437713623046875f,0.045200012731552163813830702565610408782958984375f,0.83339999619722338053406929248012602329254150390625f,-0.23560004534721290525567383156158030033111572265625f,-1.1841000486552719639377073690411634743213653564453125f,0.09920001747012197057529192534275352954864501953125f,0.43109999596476544780898620956577360630035400390625f,-0.680299995851516570155581575818359851837158203125f,0.57930002003610148175738459030981175601482391357421875f,0.748199999782442670692717001657001674175262451171875f,-0.166499968379735463486213120631873607635498046875f,-0.405500026464462592912241234444081783294677734375f,-0.58200003775954289864102975116111338138580322265625f,-0.0689999360442161702167140902020037174224853515625f,1.0131000390887265893979929387569427490234375f,-1.009200031924248008863287395797669887542724609375f,0.67800000262260429195748656638897955417633056640625f,0.70739995548725165264158931677229702472686767578125f,0.86730002660751370768821288947947323322296142578125f,0.10360000326633400646869631600566208362579345703125f,0.09870001365542435678435140289366245269775390625f,0.357099998974799603956853388808667659759521484375f,0.072000027477741479486894604633562266826629638671875f,0.15060000004768359094242669016239233314990997314453125f,0.043299999666214006310838158242404460906982421875f,-0.4737000056087967703177810108172707259654998779296875f,-0.02460000158250341684151862864382565021514892578125f,0.3972000007957217082577017208677716553211212158203125f,-0.32430000640153888724626085604541003704071044921875f,-0.3072000002741812618722860861453227698802947998046875f,-0.4096999874591826795722226961515843868255615234375f,0.1424000155016778013106204525684006512165069580078125f,-0.2175000068545340781156482989899814128875732421875f,-1.177699973914027253840686171315610408782958984375f,0.3203999973930418132539443831774406135082244873046875f,0.1787999940559268363671208135201595723628997802734375f,0.13030000240802763755709747783839702606201171875f,1.11419999459683882747640382149256765842437744140625f,0.14940000561475752505202763131819665431976318359375f,0.506000006325542539542539088870398700237274169921875f,0.24260001441240319763892330229282379150390625f,0.86210000225305538634756885585375130176544189453125f,-0.1465999999284743893213089904747903347015380859375f,0.8867999831929804788188675956916995346546173095703125f,-0.20159999985396870325615736874169670045375823974609375f,0.33299998701363808795150589503464289009571075439453125f,0.299900007753074238081580915604718029499053955078125f,0.05669999806135894715453105163760483264923095703125f,0.027399992442131093639545724727213382720947265625f,-0.0983000020876525493207509498461149632930755615234375f,-0.28309999915361405697922236868180334568023681640625f,-0.916099996672570693334591851453296840190887451171875f,0.041399997776746744193587801419198513031005859375f,-0.00979998190403019719951771548949182033538818359375f,-0.413700007694959737136741750873625278472900390625f,-1.4375000023841852936357099679298698902130126953125f,0.318600023978949042913200173643417656421661376953125f,1.179600017452238969184463712736032903194427490234375f,0.924600021922588499734274591901339590549468994140625f,0.315999989449977647382183931767940521240234375f,0.42869995926618553738762784632854163646697998046875f,0.39799999248981521304813213646411895751953125f,1.14570001133084264921535577741451561450958251953125f,-0.934299999547004489386381465010344982147216796875f,-0.94349999291449770932871388140483759343624114990234375f,-0.59439997074604067339720359086641110479831695556640625f,-0.882600011432170816760844900272786617279052734375f,1.07620002453327145985895185731351375579833984375f,-0.3717000088274478031280523282475769519805908203125f,0.63280001999139745549882718478329479694366455078125f,-1.182900052434206372709013521671295166015625f,0.302700004982948911447238060645759105682373046875f,0.03749998956918698667095668497495353221893310546875f,1.594399965381622763516134000383317470550537109375f,0.232800010491907960474833316766307689249515533447265625f,0.7333999917641278454283337850938551127910614013671875f,-0.8364000292927029622802592712105251848697662353515625f,-0.7051000069618218457634384321863763034343719482421875f,0.3816000125944611909289960749447345733642578125f,0.3706999795913699102811733609996736049652099609375f,1.379400012022256927224361788830719888210296630859375f,-0.1811000417113302063398805330507457256317138671875f,0.400500012904405622293779742904007434844970703125f,-0.76399998473376040664106767508201301097869873046875f,-0.139100000040233151565871594357304275035858154296875f,-0.175399992419779382490929720006533898413181304931640625f,0.0666999978005886617893338552676141262054443359375f,0.02789999674111609540005929375183768570423126220703125f,0.08539999297857292503977078013122081756591796875f,-0.2770999860480429788367473520338535308837890625f,0.05730000574588789419294698745943605899810791015625f,0.5632000168442725662742986969533376395702362060546875f,0.1775999890804289549350869492627680301666259765625f,-0.117600039595365135625115726725198328495025634765625f,0.2092000110626219822052007657475769519805908203125f,0.8397999869704244613188848234131000936031341552734375f,0.0646000073194503698914559208787977695465087890625f,-0.475899980327487259756935600307770073413848876953125f,-0.19250000551342960619649602449499070644378662109375f,-0.9148999997064468647778312515583820641040802001953125f,0.0971999984860421051280354731716215610504150390625f,-0.66689998806864014824213882093317806720733642578125f,0.028800002810358993343697875388897955417633056640625f,0.3051999998047947304513627386768348515033721923828125f,-0.08509999697804460083716548979282379150390625f,0.5193999773398043640071364279720000922679901123046875f,-0.2294000047206878889483050443232059478759765625f,-0.3996999962508678283512608686578460037708282470703125f,0.132000001251697529625062088598497211933135986328125f,0.3156999989628790359574850299395620822906494140625f,0.26949999722838402504976329510100185871124267578125f,-0.290799992774427196184916510901530273258686065673828125f,0.1172000020809471632077247704728506505489349365234375f,0.06119999875426263002964333281852304935455322265625f,0.3057000029116869743717188612208701670169830322265625f,-0.0745999903321268309497327209101058542728424072265625f,0.2970000019967553583910557790659368038177490234375f,0.54209997155666389545558558893389999866485595703125f,0.950400017863511692439715261571109294891357421875f,0.8806999795913696971183526329696178436279296875f,-1.134600049340725558266740335966460406780242919921875f,-0.4199999633431428236463034409098327159881591796875f,0.47880005414485982129235708271153271198272705078125f,0.640799944233894080980462604202330112457275390625f,-1.01340001467466311879661589046008884906768798828125f,0.456200016188621049195717205293476581573486328125f,-0.34590001571774475763731970801018178462982177734375f,-1.3232999615192415632236588862724602222442626953125f,0.70140000835657101418973979889415204524993896484375f,-1.0143000260710710591638417099602520465850830078125f,-0.09420000942349471273473682231269776821136474609375f,-1.305499996066092904811739572323858737945556640625f,-0.24210000664889808064117460162378847599029541015625f,0.28640000731348980167467743740417063236236572265625f,-0.146099995666741921951370386523194611072540283203125f,0.701799982857703952277006465010344982147216796875f,0.8769000185787678702808989328332245349884033203125f,0.1804999901056287825440449523739516735076904296875f,-0.275700000229477826252377781202085316181182861328125f,0.378700007396936211279125927831046283245086669921875f,0.40530001954436301048190216533839702606201171875f,0.3103999929785725697684029000811278820037841796875f,-0.32040000538378954342277893374557606875896453857421875f,0.73189998531192534958478290718630887567996978759765625f,-0.493600002372264867744888761080801486968994140625f,0.045599985444546131674314892734400928020477294921875f,-0.10780000419616708740022659185342490673065185546875f,-0.66130000135749578138444348951452411711215972900390625f,-0.0081000164389610773696404066868126392364501953125f,-0.0214000253811474916432189274928532540798187255859375f,-0.2162999966263772222418992896564304828643798828125f,0.85929998535364848777362567489035427570343017578125f,0.058300007712841050988572533242404460906982421875f,0.7957000064134593930731398359057493507862091064453125f,-0.126500008463859447971344707184471189975738525390625f,-0.6820999740898610053108086503925733268260955810546875f,0.29239999959468843115928393672220408916473388671875f,-0.5081999993726611375421953198383562266826629638671875f,0.06130000392794610064584048814140260219573974609375f,-0.0012999933481216974229255356476642191410064697265625f,0.0905000012814998200383342918939888477325439453125f,-0.0906999923318625445034513177233748137950897216796875f,0.02709999774545435169414986376068554818630218505859375f,-0.265399990780651628963227040003403089940547943115234375f,-0.23739999795556077089031532523222267627716064453125f,0.52879997781366139264491721405647695064544677734375f,0.4081999962806701631734540569595992565155029296875f,-0.3635999919041992267665364124695770442485809326171875f,0.45220000459551812355130095966160297393798828125f,0.620299992014467616030515273450873792171478271484375f,0.21829998819231999362955320975743234157562255859375f,-0.286699995341896884948340584742254577577114105224609375f,-0.3530000079870223572697796043939888477325439453125f,0.9708999887466429612459251075051724910736083984375f,0.7632000004529952974507978069595992565155029296875f,0.5783999601364140374926137155853211879730224609375f,0.697200030821562233285249021719209849834442138671875f,-0.4335000013709073840573182678781449794769287109375f,0.82170005159378067816078328178264200687408447265625f,1.0881999551534651260453756549395620822906494140625f,0.46860001354813629603768276865594089031219482421875f,2.058699985790251929529404151253402233123779296875f,-0.11910001676678660942343412898480892181396484375f,1.437100004339217917959103942848742008209228515625f,0.9411000080347056151452989070094190537929534912109375f,-0.15709999852776501683138121734373271465301513671875f,-0.079499993503093779878554414608515799045562744140625f,-0.7546000088095663471676743938587605953216552734375f,-0.560100015753507651794507182785309851169586181640625f,-0.70829997552633283675049824523739516735076904296875f,-0.64530000225901584798293697531335055828094482421875f,-0.96529995532035872685128197190351784229278564453125f,0.180600001239776430939087958904565311968326568603515625f,-0.2305999969556926654234985107905231416225433349609375f,-0.12509999936222993000001224572770297527313232421875f,0.5739999873042105349441044381819665431976318359375f,-0.67049998921155928854886951739899814128875732421875f,-1.161599964761734327112208120524883270263671875f,-1.1484000130355358493261519470252096652984619140625f,0.12490001032352449072959643672220408916473388671875f,0.36030001462697924807798699475824832916259765625f,0.1970999772191053267533789039589464664459228515625f,0.2405999998986718679816476651467382907867431640625f,1.27009997619092462883827465702779591083526611328125f,-0.146000000998377732486233071540482342243194580078125f,-0.94029996575117136270449691437534056603908538818359375f,0.1314999956488609456073390902020037174224853515625f,0.16039998943954714150095242075622081756591796875f,0.388800001692771957095828838646411895751953125f,-0.036499994419515235932749419589526951313018798828125f,0.15999999832361933016500188386999070644378662109375f,-0.1088999947965145764783301274292171001434326171875f,-0.08790000266432773656788413063623011112213134765625f,-1.0390000115931032897975683226832188665866851806640625f,0.04640000060796722891609533689916133880615234375f,0.92599996154755370714184437019866891205310821533203125f,-0.135100001597404428821391775272786617279052734375f,-0.0361999933898446979441132498322986066341400146484375f,-0.14050000470876700120470559340901672840118408203125f,-0.6078999984174966453309707503649406135082244873046875f,0.0851999912738801157274792785756289958953857421875f,-0.97789998012632117774245443797553889453411102294921875f,0.02780000047087671788403895334340631961822509765625f,0.011300002288818278284310281378566287457942962646484375f,0.100200006473064373579973107553087174892425537109375f,0.45659998487830166791212604948668740689754486083984375f,-0.347800000023841704432925325818359851837158203125f,0.8080000115260481674539505547727458178997039794921875f,-0.03379999483823770134449659963138401508331298828125f,0.26860000732690092473120557770016603171825408935546875f,-0.08870000754594808967112840036861598491668701171875f,-0.82230000090301036408391155418939888477325439453125f,0.0420999840736389074891121708787977695465087890625f,1.29790000698566387882237904705107212066650390625f,0.55229999419748754263537193764932453632354736328125f,-1.21649996957182882084680386469699442386627197265625f,0.19410001333951942825706282746978104114532470703125f,0.7295999727487565422734405728988349437713623046875f,-0.78240000987648983254985068924725055694580078125f,-0.7078999825477598761835906771011650562286376953125f,-0.5570999941319225712277329876087605953216552734375f,-0.7982999625623230510740313548012636601924896240234375f,0.7872000329077248892417628667317330837249755859375f,-1.0146000249028208628487845999188721179962158203125f,-0.24869997863769466306393951526843011379241943359375f,1.495199971604346700360110844485461711883544921875f,-0.462000017344951885434056748636066913604736328125f,-0.865399975991248737727801199071109294891357421875f,0.627900024420023239457577801658771932125091552734375f,-0.4640999984383586962621848215349018573760986328125f,0.1038000021398062244770699180662631988525390625f,-1.644899985289573152158482116647064685821533203125f,0.246299999904632471725562936626374721527099609375f,0.39509998818635949646704830229282379150390625f,-0.6522000108540055851591432656277902424335479736328125f,0.09219999684691426722338292165659368038177490234375f,0.426899969851969984802053659223020076751708984375f,1.75079998357296062749810516834259033203125f,-0.440700027871132160584011216997168958187103271484375f,-0.1313999886870380606751496088691055774688720703125f,1.215900040155649453055275444057770073413848876953125f,-0.0033000244021419433693154132924973964691162109375f,0.75870004479885100323599544935859739780426025390625f,-0.2151999974578618635145943471798091195523738861083984375f,-0.323700011718273117367061786353588104248046875f,-0.56959999442994568852327574859373271465301513671875f,0.01039999282956127757415742962621152400970458984375f,-1.1878999770715832351442031722399406135082244873046875f,0.0582999926626683873109868727624416351318359375f,-0.350299978864193006788951834096224047243595123291015625f,-0.055500005304813360140769873396493494510650634765625f,-0.3424999902397394269115693532512523233890533447265625f,0.0142000001847744083960378702613525092601776123046875f,0.212300000753998741043204745437833480536937713623046875f,-0.12969999537542464818073995047598145902156829833984375f,-0.225199986580014444026431874590343795716762542724609375f,-0.157600002491474100452251150272786617279052734375f,-0.292999990142881916455053215031512081623077392578125f,0.21149999862909307779545997618697583675384521484375f,-0.4501000096440315179080471352790482342243194580078125f,0.1874000057041644851096862112171947956085205078125f,0.331900002820789463697792598395608365535736083984375f,-0.128499996751546774476082646287977695465087890625f,0.923300013189017665382607447099871933460235595703125f,-0.1746999990820885528819417231716215610504150390625f,-1.176199996183812768180132479756139218807220458984375f,0.105599996583163768804070059559307992458343505859375f,0.376699982750415995891302145537338219583034515380859375f,0.2012999885797501775641649146564304828643798828125f,-0.8725999831944701856656365634989924728870391845703125f,0.410499993681907593412461210391484200954437255859375f,-0.450500010557472574390658337506465613842010498046875f,0.172100009070337012229146012032288126647472381591796875f,-0.3291999988064169269108560911263339221477508544921875f,1.015200019288062804889705148525536060333251953125f,-0.0640000156164166611461041611619293689727783203125f,0.47520001362562158675473256153054535388946533203125f,-1.6212999793410300952700708876363933086395263671875f,0.462899996554850812202630550018511712551116943359375f,0.032699994254112851876925560645759105682373046875f,0.3705000097751616205954405813827179372310638427734375f,-0.21109999328255657502495523658581078052520751953125f,-0.20190000449717049324505069307633675634860992431640625f,0.146100007364153849920285210828296840190887451171875f,0.090299992442876231280024512670934200286865234375f,-0.4583999762669208610788018631865270435810089111328125f,0.7803000030040738721481829998083412647247314453125f,0.3349999839067461238073519780300557613372802734375f,-0.018899975425004544860030364361591637134552001953125f,-0.6515000022053722972259492962621152400970458984375f,-0.01560000109076487007087052916176617145538330078125f,0.83409999933242762182317164842970669269561767578125f,-0.451800031137466806541169717092998325824737548828125f,1.007500026226043843280422152020037174224853515625f,1.0623000221371654561153263784945011138916015625f,-0.99780001671314266786794178187847137451171875f,0.0525000057369469086410163072287105023860931396484375f,0.9208999616265300947048899615765549242496490478515625f,-0.28979999591112193257913531851954758167266845703125f,-1.8537999760627741352436714805662631988525390625f,-0.5579999731183049593852274483651854097843170166015625f,-1.17069998868107827405538046150468289852142333984375f,-1.232400062496215920049280612147413194179534912109375f,0.97289998660832655019703452126123011112213134765625f,0.15920000182390214860106425476260483264923095703125f,-0.282399995645880697026086636469699442386627197265625f,0.030100001634657314042442521895281970500946044921875f,-0.403399980588257545122132796677760779857635498046875f,0.1081999923691154830152072463533841073513031005859375f,-0.706799974513053985702981663052923977375030517578125f,-0.1069000023365020979326800443232059478759765625f,-0.61410000920444696159705699756159447133541107177734375f,0.182099996292591104207758689881302416324615478515625f,-0.01180000219196075672556389690726064145565032958984375f,0.0629000051975250329405753291212022304534912109375f,-0.0896999871611596777398744961828924715518951416015625f,-0.5359000012636183640779563575051724910736083984375f,0.0736000146657229681323997283470816910266876220703125f,-0.306800009083747937665975769050419330596923828125f,-0.6265000026524065734889745726832188665866851806640625f,0.142800003115832685463004736448056064546108245849609375f,-0.63630002089664328135398818631074391305446624755859375f,-0.0061000000253319530685303107020445168018341064453125f,0.377199995505809848594935829169116914272308349609375f,0.359400003826618075208898517303168773651123046875f,-0.04440002006888399410655665633385069668292999267578125f,0.07179999567270289873022193205542862415313720703125f,-0.5458999861761928418246725414064712822437286376953125f,-0.300200000214576601109683906543068587779998779296875f,0.7561000164166091774831102156895212829113006591796875f,0.192699996452033450733409836175269447267055511474609375f,-0.11070000564977522383713903764146380126476287841796875f,-0.25740000883340830029055723571218550205230712890625f,-0.448499989748000871259137056767940521240234375f,0.8093999771535396181576516028144396841526031494140625f,-1.409099945259839614664088003337383270263671875f,0.519099992477893845688186047482304275035858154296875f,0.02960001283287994056081515736877918243408203125f,0.325600001053512055904093358549289405345916748046875f,-0.12859998132437500117930540000088512897491455078125f,-0.47980000902414321473088421043939888477325439453125f,-0.92629997308254274912542314268648624420166015625f,0.26109999298453345772941247560083866119384765625f,0.2621999880552294825974968262016773223876953125f,-1.31080000622272496713094369624741375446319580078125f,1.23250000953674287984540569595992565155029296875f,-0.3434000011146067787848323860089294612407684326171875f,-0.4329000247180463389895521686412394046783447265625f,0.536999990820884676168134319595992565155029296875f,-1.136499970167875517290667630732059478759765625f,0.733500025138258848755867802537977695465087890625f,0.6265999754160649359846502193249762058258056640625f,0.818699991750717170901907593361102044582366943359375f,-0.962999990254640891862436546944081783294677734375f,-0.6856999989256262750814130413345992565155029296875f,-0.39999998722225438285704512964002788066864013671875f,-0.066200003820657826736351125873625278472900390625f,1.504300025177002453347085975110530853271484375f,0.5953999965548515405089347041212022304534912109375f,-1.01659997854530814720419584773480892181396484375f,0.76530000870376824195773224346339702606201171875f,0.182500004284083860994769565877504646778106689453125f,0.50250002458691600537576960050500929355621337890625f,0.0549000052571297914028036757372319698333740234375f,-0.2945000253543259560728984070010483264923095703125f,0.604100001418590437651801039464771747589111328125f,0.0323999978438018754189897663309238851070404052734375f,0.2088000075042248937506883521564304828643798828125f,-0.2782999956429004217994815917336381971836090087890625f,-0.495700000452995226396524230949580669403076171875f,0.173600019359588519929360472815460525453090667724609375f,-0.461599999034404628872607645462267100811004638671875f,-0.47930000867396582719948128215037286281585693359375f,0.11929999150037762234433103003539144992828369140625f,0.30750000059604676305724524354445748031139373779296875f,0.06329999102354033624351359321735799312591552734375f,-0.225899999047070476621001944295130670070648193359375f,-0.130500025626272275758310570381581783294677734375f,0.25950000121444449785457209145533852279186248779296875f,0.100500004708766965677568805404007434844970703125f,-0.1214000091761351374231026056804694235324859619140625f,0.43820001505613337400291129597462713718414306640625f,0.0626999862819912490152773898444138467311859130859375f,0.272099990332126395742307067848742008209228515625f,0.489000011965632364052680713939480483531951904296875f,0.30300001633167283898728783242404460906982421875f,-0.1556000237405299468917974081705324351787567138671875f,0.4148000012755392873486925964243710041046142578125f,0.4352999909713861637072795929270796477794647216796875f,-0.334500005424022806010952990618534386157989501953125f,0.36699999320507037392502525108284316956996917724609375f,0.24500000938773158765116022550500929355621337890625f,0.53699999856948810172951880304026417434215545654296875f,-0.72179999001026118321533431299030780792236328125f,-1.1024999973177909140531482989899814128875732421875f,0.3477999964475628758009406737983226776123046875f,0.3254000193536281670958487666212022304534912109375f,-1.0790000106990333961221040226519107818603515625f,0.36759998192787168846962231327779591083526611328125f,-0.51509999325275401815815712325274944305419921875f,0.4439999993741512440692531527020037174224853515625f,0.476799977046251655110609135590493679046630859375f,-0.030299961280822795828271409845910966396331787109375f,-1.496800029051303937421835144050419330596923828125f,0.63620001082420341020906562334857881069183349609375f,-0.70549998235702560123172588646411895751953125f,-0.7934999861717224600710096638067625463008880615234375f,1.42059997745379718026015325449407100677490234375f,0.070999998837709465959733279305510222911834716796875f,0.336499995648861016661612666212022304534912109375f,-0.746600020492076854594643009477294981479644775390625f,0.386400037264824192106971167959272861480712890625f,0.17850000956654543671220380929298698902130126953125f,-0.80599998319149079861745121888816356658935546875f,-0.0382000064134597749898603069595992565155029296875f,0.7724000228405003554144059307873249053955078125f,-0.0643000038683414487650225055404007434844970703125f,-1.169099972510338147912989370524883270263671875f,1.2001999912738801068456950815743766725063323974609375f,-0.7561000254690650734801238286308944225311279296875f,0.56909999739527705742148100398480892181396484375f,-0.78269998754858960410274448804557323455810546875f,0.88729999933838854531131801195442676544189453125f,-0.06159999814033501053245345246978104114532470703125f,0.3208000076383352716646868429961614310741424560546875f,-0.14079999340772619831341216922737658023834228515625f,-0.2214000329062344452069055478204973042011260986328125f,0.31140000120401367667000158689916133880615234375f,0.56670000893920635665068630260066129267215728759765625f,0.16770001958608649061943651759065687656402587890625f,-0.70149998950213188209090731106698513031005859375f,-0.0617000009298322993345209397375583648681640625f,-0.3694000075519083825525967768044210970401763916015625f,0.218199996876716539873086730949580669403076171875f,-0.0670999914497139027247385456576012074947357177734375f,-0.598900009995698923148665926419198513031005859375f,0.5565000058561564433290413944632746279239654541015625f,-0.009299999062717001496736202170723117887973785400390625f,0.545400013504922842599853538558818399906158447265625f,0.1277999955534934173329020268283784389495849609375f,0.02400001580268185108479883638210594654083251953125f,-0.2868000150442124862593118450604379177093505859375f,-0.341800008599460358738042486947961151599884033203125f,-0.2449999955296515707914295489899814128875732421875f,0.091500004701316139943401140044443309307098388671875f,-0.494699994015693533810917870141565799713134765625f,-0.74969999181777235985890683878096751868724822998046875f,-0.24390000686645496585924774990417063236236572265625f,-0.115100006142258681762768901535309851169586181640625f,0.457100007319450440945729496888816356658935546875f,-0.245699995125829684372575911766034550964832305908203125f,0.303299991619586872815261813229881227016448974609375f,-0.212000025205314635012854296292061917483806610107421875f,-0.626199972975253871254608384333550930023193359375f,0.844799992036819435980987691436894237995147705078125f,-0.695199981737136596393611398525536060333251953125f,-0.19690001813173285682978530530817806720733642578125f,-0.83389998660087627513348706997931003570556640625f,0.49649999117851262298017900320701301097869873046875f,0.0888000115275389134694705717265605926513671875f,-1.048799979490041778262821026146411895751953125f,0.54049997580051467593875713646411895751953125f,-1.00100000554323198542761019780300557613372802734375f,0.74320001416206338262782082892954349517822265625f,0.56080001367330556405477182124741375446319580078125f,-0.14590005833506580756875337101519107818603515625f,0.7170999964602291498749764286912977695465087890625f,-0.049599987966567393726791124208830296993255615234375f,0.74450001376867280811211458058096468448638916015625f,-0.43369996612072014841032796539366245269775390625f,-0.18500000134110461402769942651502788066864013671875f,1.08860001280307727711260668002068996429443359375f,-0.09910002198219292068870345246978104114532470703125f,-0.19010000517368297323628212325274944305419921875f,0.87849998170137411079849698580801486968994140625f,-0.639600008809566134004853665828704833984375f,-0.66659997146725658012655912898480892181396484375f,-0.04870004330873456410699873231351375579833984375f,-0.5624999888241291046142578125f,-0.0447999947190282910014502704143524169921875f,-0.499599972748756560037008966901339590549468994140625f,-0.2120999800503255983130657114088535308837890625f,-0.2518000114679335155187800410203635692596435546875f,0.36200001231580980487478882423602044582366943359375f,0.71510000182092170462055946700274944305419921875f,-0.1498999930381774703391783987171947956085205078125f,0.000600001239776604133879800428985618054866790771484375f,-0.1662000035226345318051244248636066913604736328125f,-0.135399981877208031022519207908771932125091552734375f,-0.699799988609552059415364055894315242767333984375f,0.6388999974414701910063740797340869903564453125f,0.0307000171422959766687199589796364307403564453125f,0.36340002726614528683057869784533977508544921875f,-0.16180001415014277910131568205542862415313720703125f,-0.539800011669099522038095528841949999332427978515625f,0.06820000104904178073184084496460855007171630859375f,-0.0200000032037495056869147447287105023860931396484375f,0.56000000149011608385762883699499070644378662109375f,-0.75060000630617185191084672624128870666027069091796875f,-0.06299998950958229926300191436894237995147705078125f,-0.3139000104054809892062394283129833638668060302734375f,-0.24770000677108772180190499057061970233917236328125f,-0.65550001398473956015777730499394237995147705078125f,0.0722999816060063693612391944043338298797607421875f,0.6254999987110494430453400127589702606201171875f,0.15210000404119483619069796986877918243408203125f,0.2732999901294708156029855672386474907398223876953125f,0.018300008606910722619431908242404460906982421875f,0.10329999407827851476415759179872111417353153228759765625f,0.28190000576376916807674888332257978618144989013671875f,0.039199999886751157873732154257595539093017578125f,-0.20839999399930231316346862513455562293529510498046875f,0.2300000030919908677873308988637290894985198974609375f,-0.12069997545629718160853371955454349517822265625f,1.30619998638629919440745652536861598491668701171875f,-0.62149999594688409843001863919198513031005859375f,0.1914999996721744590644220807007513940334320068359375f,-0.0675000081956387276704845135100185871124267578125f,0.6655999854445457941665154066868126392364501953125f,-0.3490000220239153350121341645717620849609375f,0.97899998357892048517214789171703159809112548828125f,-0.49050000202655752445934922434389591217041015625f,0.202500018924474556314407891477458178997039794921875f,0.889500012576579734968618140555918216705322265625f,-0.808200000900030079975522312452085316181182861328125f,0.171199981915950871780296438373625278472900390625f,-0.763899984365701723021402358426712453365325927734375f,0.441300015103816889450172311626374721527099609375f,-1.2098000105142592275342394714243710041046142578125f,0.76599996918439927640065434388816356658935546875f,-0.035900006330013223276864664512686431407928466796875f,-0.248200020122528286492524784989655017852783203125f,-1.20749999672174457288065241300500929355621337890625f,0.73139999613761830232760985381901264190673828125f,0.264899982011318257946186349727213382720947265625f,0.796000015974044572431012056767940521240234375f,0.97379997934103013079720767564140260219573974609375f,-1.45089996490478512214394868351519107818603515625f,0.018600001813471322975601651705801486968994140625f,-0.36169998852461571203775747562758624553680419921875f,0.5504999805316328487236887667677365243434906005859375f,-0.667499980628490252598794540972448885440826416015625f,-0.3916999969065189102224167072563432157039642333984375f,0.7994000252842905496208913973532617092132568359375f,0.35260003609359269471923425953718833625316619873046875f,-0.72699999886751154321018475457094609737396240234375f,0.4907000214263799175995472978684119880199432373046875f,-0.2315000156164170874717456172220408916473388671875f,0.1899000063002109772725134462234564125537872314453125f,0.054900003431737454118177765849395655095577239990234375f,-0.55339999403655493193099346171948127448558807373046875f,-0.08020001928806319568820981658063828945159912109375f,-0.4656000089511274087072933980380184948444366455078125f,0.55559999743998045307336042242241092026233673095703125f,0.00229999650716783410331345294252969324588775634765625f,0.03010000018179415093300121952779591083526611328125f,0.1028999956794083292521690964349545538425445556640625f,-0.0992999988950789003183672321029007434844970703125f,-0.035099987217784178739066192065365612506866455078125f,-0.0404999987483023460299591533839702606201171875f,0.2968000267788770685228882939554750919342041015625f,-0.494800013792514814525702604441903531551361083984375f,0.09999999787658453431760108287562616169452667236328125f,-0.08919999966323377549315409851260483264923095703125f,0.08940000036209816425269991668756119906902313232421875f,-0.139800000493228437914439155065338127315044403076171875f,0.534899989908933282123371100169606506824493408203125f,-0.0272999944210052802873178734444081783294677734375f,-0.06749999139457951713438887964002788066864013671875f,-0.41490000391602510632083067321218550205230712890625f,0.04890000057071464123481518981861881911754608154296875f,-0.27210001685619378264391343691386282444000244140625f,-0.7573000215411191948788882655207999050617218017578125f,-0.14769998650550775209921994246542453765869140625f,0.993700002330541565243038348853588104248046875f,0.31469998239278851315248175524175167083740234375f,0.255400017267465528902903315611183643341064453125f,1.2688999824285502882048604078590869903564453125f,-0.0548000046841800314556536477539339102804660797119140625f,0.5227000106267631895917702422593720257282257080078125f,-0.96139997617006311969589660293422639369964599609375f,0.790799983870982714506681077182292938232421875f,-1.4238999809756875958299815465579740703105926513671875f,0.696500001348554942381952059804461896419525146484375f,0.1356999991118907900045087444595992565155029296875f,-0.358799994912743613895145244896411895751953125f,0.7435999965608119932625186265795491635799407958984375f,-1.15469999168738723938076873309910297393798828125f,-0.6182999822318553384548067697323858737945556640625f,1.3609999533891681267050444148480892181396484375f,1.0398000331640242510644611684256233274936676025390625f,-0.4362000291526317852230931748636066913604736328125f,0.39019999201893806883845172706060111522674560546875f,-0.40099999190866952858414151705801486968994140625f,0.3490999970041215316740590424160473048686981201171875f,-0.2307000051282346309022841523983515799045562744140625f,0.93719998805522930496181288617663085460662841796875f,-0.55790001175403602928781765513122081756591796875f,0.382700007069110892388152933563105762004852294921875f,-0.145600032681226565500764991156756877899169921875f,-0.06670000808238985001707987976260483264923095703125f,0.862999997258185658211004920303821563720703125f,-0.9924000185936687756793617154471576213836669921875f,-0.0508000127792360700595963862724602222442626953125f,0.212600003199278908549985089848632924258708953857421875f,-0.244999990463256711592521241982467472553253173828125f,0.0197000229239463475749971621553413569927215576171875f,-0.7073000054478644216260363464243710041046142578125f,0.172299994383752153037647758537787012755870819091796875f,-0.445900008118152779701404142542742192745208740234375f,0.0610000223070381597523237360292114317417144775390625f,-0.6369999943971631495287510915659368038177490234375f,0.5182999923646447459901764887035824358463287353515625f,-0.2338999985218048749402441899292171001434326171875f,-0.0507999993309378172767765136086381971836090087890625f,-0.26959999718666072254791288287378847599029541015625f,0.28670000610798584972371827461756765842437744140625f,-0.54590001109838492965309342253021895885467529296875f,0.268599993096291900229743987438268959522247314453125f,-0.05620000128746038825511277536861598491668701171875f,0.5360000020787121233212246806942857801914215087890625f,0.5044000106811523664873675443232059478759765625f,0.5203999939098948868121397026698105037212371826171875f,0.196000021040439786901288243825547397136688232421875f,-0.588499979615211277206299200770445168018341064453125f,-0.4519000032305715564007186912931501865386962890625f,0.012900012275576966391810174172860570251941680908203125f,-0.506100022339821098427137258113361895084381103515625f,0.71539999834299050629482508156797848641872406005859375f,0.0262000133574009641534985348698683083057403564453125f,0.1100000305846335901804877721588127315044403076171875f,-0.617600010836124369006938650272786617279052734375f,-0.5777000033065677886412458974518813192844390869140625f,-1.22789999059438770245833438821136951446533203125f,0.213699979084730262712810144876129925251007080078125f,0.63069998301863705592040787450969219207763671875f,0.6471000068724155340760262333787977695465087890625f,0.131599963569641431604395620524883270263671875f,0.31259997000694283997290767729282379150390625f,0.730899988895654217913033789955079555511474609375f,0.7664999972879886769305812777020037174224853515625f,-0.170500024378299031013739295303821563720703125f,1.1993999829649926169850004953332245349884033203125f,-0.7427000067710878283833153545856475830078125f,0.416999979048967350792054276098497211933135986328125f,-0.486599979364871870757269789464771747589111328125f,1.201900012767314951389607813325710594654083251953125f,-0.713800045502186009116485365666449069976806640625f,0.766200001734495206306974068866111338138580322265625f,-0.844400010383129284718961571343243122100830078125f,0.15209999748468394642486600787378847599029541015625f,0.04850004309415822945084073580801486968994140625f,0.67439998534917844352776228333823382854461669921875f,-0.492400000339745957944614929147064685821533203125f,-0.18029998348355302795198440435342490673065185546875f,-0.24160001602172798129686270840466022491455078125f,0.108499997198581610291512333787977695465087890625f,0.876599999183415690140463993884623050689697265625f,0.838099981048703313035730388946831226348876953125f,0.0402999986082317462887658621184527873992919921875f,0.56830001784563055622356841922737658023834228515625f,0.36109997584819808480460778810083866119384765625f,-0.318899986004829383290370969916693866252899169921875f,0.296399997329711784033179355901665985584259033203125f,0.067599998430907692892333216150291264057159423828125f,0.238400000369548781264938952517695724964141845703125f,0.4877000221565366278042574776918627321720123291015625f,-0.19970000236034390894701573415659368038177490234375f,-0.6173000200137497284202936498331837356090545654296875f,-0.019900004549324556979428280101274140179157257080078125f,-0.4281000038474795932330607683979906141757965087890625f,-0.1443000093817712325972024700604379177093505859375f,0.138899979597330247571562722441740334033966064453125f,0.6279000066876410102167938021011650562286376953125f,-0.2434999970495697052541572702466510236263275146484375f,0.1462999927520751697329615126363933086395263671875f,0.0772999918505548055946974272956140339374542236328125f,-0.4733000121831896223056901362724602222442626953125f,0.0854999993629752541579591706977225840091705322265625f,0.46530000373050572903821375803090631961822509765625f,-0.62820000969171518878653159845271147787570953369140625f,0.1745999976336956027722635553800500929355621337890625f,0.178599992910027705050879376358352601528167724609375f,-0.085000006332993649493801058270037174224853515625f,0.3316000171765682846825029628234915435314178466796875f,-0.2991999904990194636411615647375583648681640625f,0.268600020477176126831864166888408362865447998046875f,0.0193999945878982060776252183131873607635498046875f,0.2619000035285949934404925443232059478759765625f,-0.29430000699758540605444068205542862415313720703125f,-0.2216000006735325822404547579935751855373382568359375f,0.2359999966025352335918796597979962825775146484375f,0.39270002774447199112017869992996565997600555419921875f,-0.683999980598688495092574157752096652984619140625f,-0.2533999881505966111916450245189480483531951904296875f,0.2365999923288821804590043029747903347015380859375f,0.331400007611513114369472532416693866252899169921875f,1.024299987924099042402303894050419330596923828125f,-0.2694999947510658966365326705272309482097625732421875f,-1.0460000214315952060672998413792811334133148193359375f,-0.932700026887655297969104140065610408782958984375f,1.2064000271320338697478291578590869903564453125f,-0.43920002536773683488036112976260483264923095703125f,0.01050006490945776249645859934389591217041015625f,0.07580003215074526679018163122236728668212890625f,-1.1456999950885773387199151329696178436279296875f,0.068999963909387673766104853712022304534912109375f,1.0698999936342232786046224646270275115966796875f,-0.6452999785661697185190632808371447026729583740234375f,0.668999984491616572768180049024522304534912109375f,-0.954599991598725328145746971131302416324615478515625f,0.0267999930649995832254717242904007434844970703125f,0.83639999494552608183539632591418921947479248046875f,-1.184199978503585271027986891567707061767578125f,0.10009997805356984912350526428781449794769287109375f,1.32329999668598219386694836430251598358154296875f,-0.22079999698400509799967039725743234157562255859375f,1.18140002832412704947273596189916133880615234375f,0.445499988168478022743101973901502788066864013671875f,0.356999998867511436628774390555918216705322265625f,-0.9205999788880347978192730806767940521240234375f,0.9305999693512916337567730806767940521240234375f,1.0171000004649162207215340458787977695465087890625f,-0.137100014472007813992604496888816356658935546875f,0.0320000023692845747547153223422355949878692626953125f,0.14080000629723077310018197749741375446319580078125f,0.067399990542233056967091897604404948651790618896484375f,0.188200001719593945637143406202085316181182861328125f,-0.01609999071210654886243673900025896728038787841796875f,-0.65349999809265124639523492078296840190887451171875f,-0.3521999851122499247679797917953692376613616943359375f,-0.414599995994567649404416442848742008209228515625f,-0.09669999381452798470348852788447402417659759521484375f,-0.40730000552237033417668499168939888477325439453125f,-0.4762999823212623606849547286401502788066864013671875f,-0.153699998903274348549530259333550930023193359375f,-0.062799999465048272906386728209326975047588348388671875f,-0.034400001814961445045781829321640543639659881591796875f,0.5261999989405270472531128689297474920749664306640625f,-0.3353999970018863852061485886224545538425445556640625f,0.4013000004634263984115705170552246272563934326171875f,0.1962999936461446992552737356163561344146728515625f,0.4397000026956197071825727107352577149868011474609375f,-0.07969997643232318296213634312152862548828125f,-0.44910000041276187943140030256472527980804443359375f,0.423899992598593178261268121786997653543949127197265625f,0.2729999924153092560885625061928294599056243896484375f,0.4182000017940996627885397174395620822906494140625f,0.5553999938726421614632045020698569715023040771484375f,-0.231399991667270654716048738919198513031005859375f,0.1159999981671571145369625810417346656322479248046875f,-0.01159999273866407776267806184478104114532470703125f,-0.01000000890344399095965854940004646778106689453125f,1.2466999735116957026548334397375583648681640625f,-0.531800012063980087617665049037896096706390380859375f,0.965000012740492962848293245770037174224853515625f,0.15619999353885649640005794935859739780426025390625f,-0.564399977526068852284879540093243122100830078125f,0.44419998140931138408404876827262341976165771484375f,0.3684999685883525444296537898480892181396484375f,-0.01199999693036080117281017010100185871124267578125f,-0.04900002098083433565989253111183643341064453125f,0.32909999814033508158672702847979962825775146484375f,-0.12280001514852045829684357158839702606201171875f,-0.92639998540878298882716990192420780658721923828125f,1.08089997444152885464063729159533977508544921875f,-0.06229999913349748563717867000377736985683441162109375f,0.040099999045580592138549036462791264057159423828125f,0.95259997768104087700891113854595459997653961181640625f,0.04279998624026770670525365858338773250579833984375f,0.3243000037193297657722723670303821563720703125f,-0.0423999739646916395940934307873249053955078125f,0.98309997411966332236943344469182193279266357421875f,-0.80759998311996383790756226517260074615478515625f,-1.1524999688751995652324922048137523233890533447265625f,0.4215999978609381582117521247710101306438446044921875f,-0.2289000219166279492810645024292171001434326171875f,0.3589000642359252424284932203590869903564453125f,1.01149996748566639581667914171703159809112548828125f,-0.23029999644756316001803497783839702606201171875f,0.44809999080896378842453486868180334568023681640625f,-0.651499987043440142997496877796947956085205078125f,0.662300003957748462113386267446912825107574462890625f,-0.250700010883808221961999151972122490406036376953125f,0.6201000038698314398288857773877680301666259765625f,-0.18089999776184562119141219227458350360393524169921875f,0.30189999794065991434166562612517736852169036865234375f,0.166099988700449230361044783421675674617290496826171875f,0.4847999975875023626059601156157441437244415283203125f,0.029999997615814066875827847979962825775146484375f,0.0902000128060576056299169067642651498317718505859375f,-0.277399985289573436375576420687139034271240234375f,0.5538000178232793668797739883302710950374603271484375f,0.3857999990701674875026583322323858737945556640625f,0.2704000009506939417036619488499127328395843505859375f,-0.1047999957621097433246859509381465613842010498046875f,0.11289999666661010291335287547553889453411102294921875f,0.24710000918209562570382331614382565021514892578125f,-0.04350000055134284338009820203296840190887451171875f,-0.593700007396936513259788625873625278472900390625f,0.6479000179752707477831563664949499070644378662109375f,0.5329000260591509885443883831612765789031982421875f,0.54520000922977918111911321830120868980884552001953125f,-0.116800000627338917968955911419470794498920440673828125f,-0.45299999733269213919584217364899814128875732421875f,-0.2958000029444693979030489572323858737945556640625f,-0.487999993123113873227936210241750814020633697509765625f,-0.156400006866455054677089719916693866252899169921875f,-0.2211000141069288105200030258856713771820068359375f,0.61709999293982964463367579810437746345996856689453125f,-0.368799992491305073105678502543014474213123321533203125f,-0.246799997162818751661461647017858922481536865234375f,0.5389999972879888634480494147283025085926055908203125f,0.497699987548589461994197336025536060333251953125f,0.900699985216558030032274473342113196849822998046875f,0.08400000194460144342656349181197583675384521484375f,0.35020000373497595091976108960807323455810546875f,-0.306699965707213362264837996917776763439178466796875f,1.08009998610019675169269248726777732372283935546875f,-1.13389994636774105174481519497931003570556640625f,-0.21370003004670135027254218584857881069183349609375f,1.31289999893903797101302188821136951446533203125f,-0.029999986141920231830226839520037174224853515625f,0.7542000408649442988462396897375583648681640625f,0.27039998247325425229092843437683768570423126220703125f,0.36320000469982627322451662621460855007171630859375f,0.15460000806450846511097552138380706310272216796875f,-0.56250000216066808889081585220992565155029296875f,0.66779996664524077232272247783839702606201171875f,-0.3296999795615676021043327637016773223876953125f,0.0515999973952769863672074279747903347015380859375f,0.51590004358291707831085659563541412353515625f,0.5585999978460371995225841601495631039142608642578125f,0.5005999936960636720328921001055277884006500244140625f,-0.30950000378489495034273204510100185871124267578125f,0.08109998955726638314445153810083866119384765625f,-0.43789998835921295494699734263122081756591796875f,-0.027900003260374006686106440611183643341064453125f,-0.47689998091608298835808454896323382854461669921875f,-0.86599995961040310277212483924813568592071533203125f,-0.459799997401237448002575547434389591217041015625f,0.539399969404936285855001187883317470550537109375f,1.32390001059174533537543538841418921947479248046875f,0.1545000072121620604548297706060111522674560546875f,-0.1618999976053834277678333819494582712650299072265625f,0.097100004637241443816719765891321003437042236328125f,-0.6921000102996828218238078989088535308837890625f,0.01170000271797189128619720577262341976165771484375f,-0.0767999953001734514401732667465694248676300048828125f,-0.5784000163137914540811834740452468395233154296875f,-0.3575999997347591996543769710115157067775726318359375f,0.00480000276565561989627894945442676544189453125f,0.3988000282913450877941841099527664482593536376953125f,-0.074800013792514885579976180451922118663787841796875f,0.1079999930486081893832306377589702606201171875f,0.1548000032126903846574350609444081783294677734375f,0.44790001533031464919787367762182839214801788330078125f,-0.23550000172853469138090076739899814128875732421875f,0.0253999988272786136889180852449499070644378662109375f,0.302200009289383952904017860419116914272308349609375f,-0.6717000085294244637879046422312967479228973388671875f,-0.32550001722574251061814720742404460906982421875f,0.6588000010967249853166549655725248157978057861328125f,0.162000023603439569086503979633562266826629638671875f,0.222200018192828085972223561839200556278228759765625f,-0.4778000190973283878292932058684527873992919921875f,-0.278800001990795021100666417623870074748992919921875f,0.206799992394447240684485223027877509593963623046875f,0.1854000036701559661622695784899406135082244873046875f,0.027000005424022699429542626603506505489349365234375f,-0.7373000266075138142696232534945011138916015625f,-0.0150999986171722611061341012828052043914794921875f,-0.50219999349415267442253707486088387668132781982421875f,0.88480002779960642556034144945442676544189453125f,-0.5254000016212463375353536321199499070644378662109375f,0.5147000025093557695754498126916587352752685546875f,-0.900000000894069618340154192992486059665679931640625f,0.079499979794025676937962998636066913604736328125f,0.4885999830007552446886620600707828998565673828125f,-0.0561999868333344920756644569337368011474609375f,-0.00759999176263814746334901428781449794769287109375f,-0.96999999165534944722821819595992565155029296875f,0.1646000333964823258980914033600129187107086181640625f,0.8841999933302400194179426762275397777557373046875f,0.56030001790523531468579676584340631961822509765625f,-1.27770001466870297690547886304557323455810546875f,0.56050000098347663168851795489899814128875732421875f,0.4843999925017357099932269193232059478759765625f,-0.5139000111877918275382626234204508364200592041015625f,0.5623000260859729593221345567144453525543212890625f,-0.46430000327229503653825304354541003704071044921875f,-0.10960000672340441951746470294892787933349609375f,0.95069998823404322507713004597462713718414306640625f,-0.654099995309113779740073368884623050689697265625f,0.21899999520182600321049903868697583675384521484375f,1.14280000598430575564634636975824832916259765625f,-0.8534999843835830990457225198042578995227813720703125f,0.6880999903619287039191476651467382907867431640625f,-0.2072000196456909026920811811578460037708282470703125f,0.9078000189483166337822694913484156131744384765625f,-0.94880002195835100309295739862136542797088623046875f,0.12149998104572290458236238919198513031005859375f,-0.27099998825788507161860252381302416324615478515625f,-0.088599995219707405880171791068278253078460693359375f,0.4659000267446045118191477740765549242496490478515625f,-0.51270001481771476647963936557061970233917236328125f,-0.3972000294059518932243690869654528796672821044921875f,0.4188000161468983240098395981476642191410064697265625f,0.2366000091671944804971872144960798323154449462890625f,0.13060000302791596737961299368180334568023681640625f,0.5843999972328541847588212476694025099277496337890625f,0.283200001943111345781289855949580669403076171875f,0.27270002502501000662959995679557323455810546875f,-0.441899999654292940931554767303168773651123046875f,-0.144200002196431142920118873007595539093017578125f,-0.233799998675286724303390428758575581014156341552734375f,-0.23040002315342444916268505039624869823455810546875f,0.496800014150142743574178894050419330596923828125f,-0.345000026486814437287620194183546118438243865966796875f,0.2334000102043152224950972595252096652984619140625f,-0.13650001770257969013755428022705018520355224609375f,0.1904999990463256409611858543939888477325439453125f,-0.1474000056669113600804621455608867108821868896484375f,0.6838000016927718860415552626363933086395263671875f,0.13059999255985033084215274357120506465435028076171875f,0.34460000206679097800588351674377918243408203125f,-0.111899998126923971852875183685682713985443115234375f,-0.28410001222193248526792785924044437706470489501953125f,-0.3070999920085071277497945629875175654888153076171875f,0.06949999976158149905813843361102044582366943359375f,0.112100009889900764648018594016321003437042236328125f,-0.155300011944770943728144629858434200286865234375f,-0.520500012084841756632158649154007434844970703125f,-0.5042000055491921983730208012275397777557373046875f,1.232500021010637158980216554482467472553253173828125f,-0.363200007307529659783540409989655017852783203125f,-1.018200038078427294152561444207094609737396240234375f,0.27660002384483828308248121174983680248260498046875f,-0.263300006371736561305851864744909107685089111328125f,0.950500021100044278909990680404007434844970703125f,0.26870000724792486579417527536861598491668701171875f,0.39509998848438243612690712325274944305419921875f,0.1864999990761280201923000277020037174224853515625f,1.2759999748468402458456694148480892181396484375f,0.47899999130889769904939612388261593878269195556640625f,-0.9555999928764997708441342183505184948444366455078125f,-0.3083000096499919351344942697323858737945556640625f,-1.0078999777793882230980671010911464691162109375f,-0.4592000192582605944124907182413153350353240966796875f,-0.4234000042438506028474876075051724910736083984375f,0.22440001783370977506137933232821524143218994140625f,-1.27730002601146708229862269945442676544189453125f,-0.81039997584223755211496609263122081756591796875f,-0.0681999795913696971183526329696178436279296875f,-0.25600000822544100032018832280300557613372802734375f,0.187400031073391204472500248812139034271240234375f,0.5129000228181481713107814357499592006206512451171875f,-0.835000032819807724848715224652551114559173583984375f,-0.624999986439943366889337994507513940334320068359375f,0.397899996554851753671755432151257991790771484375f,-0.37969999550580979530423064716160297393798828125f,1.479599982583522432832978665828704833984375f,-1.1115999985873699085203725189785473048686981201171875f,-0.126199991601705363564178696833550930023193359375f,0.194600000502169023786080970239709131419658660888671875f,-0.01099999593198297542784303004737012088298797607421875f,-0.40430000040382141346384514690726064145565032958984375f,0.130699996876716628690928700962103903293609619140625f,-0.789100008012354692255030386149883270263671875f,0.1483000074148179692201665602624416351318359375f,0.4334000075221060566121877855039201676845550537109375f,-0.008599991941451889942982234060764312744140625f,-0.734400000995397039815770767745561897754669189453125f,0.08639998481273636343757971189916133880615234375f,-0.1742000304341317917788956037838943302631378173828125f,0.2390000013113020571609013131819665431976318359375f,0.363499999657272976349986493005417287349700927734375f,-0.48949999976158142800386485760100185871124267578125f,0.41370003351122175416776372003369033336639404296875f,-0.413700013804435773323575631366111338138580322265625f,0.19310002251118429938969711656682193279266357421875f,-0.538700007843971206966671161353588104248046875f,-0.0462000092968345710264799208744079805910587310791015625f,0.1362000035226345051597718338598497211933135986328125f,0.040999989934265845814564954707748256623744964599609375f,0.336999995887279357731358686578460037708282470703125f,0.0933000057756900869065930237411521375179290771484375f,0.03990000197887422217490893672220408916473388671875f,-0.27530001328587550002424677586532197892665863037109375f,0.127700004386901877495574808563105762004852294921875f,-0.807600016014278221376798683195374906063079833984375f,-0.074399997568130427083588074310682713985443115234375f,-0.0637000032246113345735238908673636615276336669921875f,1.2661999951779847606303519569337368011474609375f,0.105699989090859924889542753589921630918979644775390625f,0.320700003768503716372606504592113196849822998046875f,-0.7598999842464924103779821962234564125537872314453125f,0.3005999870836735254897575941868126392364501953125f,-1.56460000180602065711354953236877918243408203125f,0.99549997401237533267703838646411895751953125f,1.08630001733899117510873111314140260219573974609375f,-1.20550001305341680790661484934389591217041015625f,-1.2167999923944474716108743450604379177093505859375f,0.5064000241518016309782979078590869903564453125f,0.0535999841928482823050217120908200740814208984375f,-1.105399988508224851102568209171295166015625f,1.052399981117248639606032156734727323055267333984375f,-0.293300022017955797082322533242404460906982421875f,0.54810002225041376533454240416176617145538330078125f,0.799399990266561388807531329803168773651123046875f,0.764800015729665716435192734934389591217041015625f,-0.016100029492378098439075984060764312744140625f,-0.06340000476539131568642915226519107818603515625f,-0.2767999967902898816873857867904007434844970703125f,-0.179899999743700078624897287227213382720947265625f,-0.209699970918894251781239290721714496612548828125f,-0.337600007259845735774206332280300557613372802734375f,0.917700007665157357905627577565610408782958984375f,-0.32420000763535494758826871475321240723133087158203125f,-0.30629999945759767143727003713138401508331298828125f,-1.12930001370310773012306526652537286281585693359375f,0.456299974423647114463165053166449069976806640625f,0.1515999990902841265683065330449608154594898223876953125f,0.362900000429153379855051753111183643341064453125f,0.581600019896030939747788579552434384822845458984375f,0.33999999344348896812562088598497211933135986328125f,0.07710000363141300550040568850818090140819549560546875f,-0.22830000294446950448445932124741375446319580078125f,-0.6005999936774368830327830437454394996166229248046875f,0.29459999748468401747913958388380706310272216796875f,-0.29570003446489590004375713760964572429656982421875f,0.828500010907649908631356083787977695465087890625f,-0.4735000338554387067091511198668740689754486083984375f,0.54910001274347308708456694148480892181396484375f,-0.0540999842822553755894432470086030662059783935546875f,-0.1510999920904634308271852205507457256317138671875f,-0.3743999975308776040350267066969536244869232177734375f,-0.4212000099301338451596166123636066913604736328125f,0.2831000027298926635666020956705324351787567138671875f,-0.214699999678134911818006003159098327159881591796875f,0.3257000008255241940702262581908144056797027587890625f,-0.314899995422363243591235004714690148830413818359375f,-0.06279999506920584673164142941459431312978267669677734375f,-0.13159999855011694780415609784540720283985137939453125f,0.387799999241531245797176552514429204165935516357421875f,0.357399996912479256394590265699662268161773681640625f,-0.2769999989792706340807626474997960031032562255859375f,-0.54019999902248372336543980054557323455810546875f,-0.540500018529594061789111947291530668735504150390625f,0.014899999296665189518762417719699442386627197265625f,-0.6514999870806932324995841554482467472553253173828125f,-0.086500000119209108362383631174452602863311767578125f,0.65250002462416911708231737065943889319896697998046875f,-0.549099993968010124945067218504846096038818359375f,0.055200024354457699615750243538059294223785400390625f,0.739200029242038425536520662717521190643310546875f,-0.03300001975893973593656482989899814128875732421875f,0.7717000092744825678892084397375583648681640625f,-1.399699990737438337617959405179135501384735107421875f,0.832100029671192231717213871888816356658935546875f,-1.3289000235557555384957595379091799259185791015625f,0.3666000380754468324084882624447345733642578125f,-1.02050001320242866853504892787896096706390380859375f,-0.20520000557899464865840855054557323455810546875f,-0.38089999813437458442422212101519107818603515625f,1.019399977153540248764329589903354644775390625f,-0.051399995690584177054915926419198513031005859375f,-0.0082000293612480845695245079696178436279296875f,0.8096999972939491474477335941628552973270416259765625f,-0.5624000057041644851096862112171947956085205078125f,0.9924999803304672685300147350062616169452667236328125f,-0.6369999942481516796988216810859739780426025390625f,-0.046699999253451807135206763632595539093017578125f,0.21329998729825039305296741076745092868804931640625f,0.22240000406503668273217044770717620849609375f,-0.197799959939718661416918621398508548736572265625f,0.45719998582005505483039087266661226749420166015625f,-0.61500001728534670064618694595992565155029296875f,-0.1369999976754188164562719975947402417659759521484375f,-1.2043000119149684934427568805404007434844970703125f,-0.90329997761249547494344369624741375446319580078125f,0.407599975818396131899135070852935314178466796875f,0.583800009590387247726539499126374721527099609375f,0.5811000065445899309679589350707828998565673828125f,0.4295000161528583770831346555496565997600555419921875f,-0.4906999943435190747464957894408144056797027587890625f,-0.55240000507086506331688724458217620849609375f,0.18879999781847001116830142564140260219573974609375f,-0.318500002972781803833868252695538103580474853515625f,-0.33670000212192530142374380375258624553680419921875f,0.099700000497698670898216732894070446491241455078125f,-0.29769999566972249471774603080120868980884552001953125f,0.161200013655424412295502634151489473879337310791015625f,-0.478000017523765752258668726426549255847930908203125f,-0.41460002434402676296798517796560190618038177490234375f,0.59819999687671643329167636693455278873443603515625f,0.7008000236570837149230328577687032520771026611328125f,-0.1967999986529349598640692420303821563720703125f,0.1809000083416700821459244252764619886875152587890625f,-0.213300001454353349572556908242404460906982421875f,0.23570003461390742760528382859774865210056304931640625f,-0.69650000369548781264938952517695724964141845703125f,-0.262500005960464577459134716264088638126850128173828125f,0.100500000536441813636656661401502788066864013671875f,0.01579999027848255632733298625680617988109588623046875f,0.092199994164705145749394432641565799713134765625f,0.0813999995276331045346296377829276025295257568359375f,0.7790000150203706397178393672220408916473388671875f,0.27720000239759678972717438227846287190914154052734375f,0.03600000196695329890417269780300557613372802734375f,-0.28189999659955500277419560006819665431976318359375f,-0.1537000042676925914975072373636066913604736328125f,-0.6853999957725402936148384469561278820037841796875f,-1.10739996442794819131449912674725055694580078125f,0.70739998745024212922771766898222267627716064453125f,0.6540999718397859652441184152849018573760986328125f,-0.946499989986419709708798109204508364200592041015625f,0.185499998152256040384600055404007434844970703125f,0.277600002044439353454663432785309851169586181640625f,-0.765300026547908629481753450818359851837158203125f,0.6545999826207757887885918535175733268260955810546875f,-0.542099995882808993741264202981255948543548583984375f,0.2510999934315680803820214350707828998565673828125f,0.905500001281499322658419259823858737945556640625f,-0.92940000486969953641391839482821524143218994140625f,-0.3511999550938611491801566444337368011474609375f,1.26550001230835906795846312888897955417633056640625f,-0.47009999146461467489643837325274944305419921875f,0.11549998578429221396390857989899814128875732421875f,0.69400000876188272513900301419198513031005859375f,0.86400001427531236686263582669198513031005859375f,0.21769997071027802348908153362572193145751953125f,-0.532799998086690873577708771335892379283905029296875f,0.24909998964667323662069975398480892181396484375f,0.248200003135204294579807537957094609737396240234375f,-0.703999991327524554662886657752096652984619140625f,-0.7967999758541584043314287555404007434844970703125f,0.32300000441074416812625713646411895751953125f,0.52200000974535942788179454510100185871124267578125f,-0.4768000072956084522957098670303821563720703125f,-0.79230001274943351319279827293939888477325439453125f,1.31459999808073035865163546986877918243408203125f,-1.46709999122619638711739753489382565021514892578125f,-0.113800001655519011922734762265463359653949737548828125f,0.30540002792179610224820862640626728534698486328125f,-0.03179999865293492433693245402537286281585693359375f,0.242900025425851262550480669233365915715694427490234375f,-0.710900001561641570191341088502667844295501708984375f,-0.309899986779689939897508565991302020847797393798828125f,-0.089699994313716668870029025129042565822601318359375f,0.0319000085204839134433996150619350373744964599609375f,-0.11469999595284452453824997064657509326934814453125f,0.007699997755885090722216546055278740823268890380859375f,0.0823000034362078025651499046944081783294677734375f,0.363999997511505857250568851668504066765308380126953125f,-0.129999995864927775368613538375939242541790008544921875f,-0.062799995441734868251160150975920259952545166015625f,0.08080000205039983285359994624741375446319580078125f,-0.2565000033974647042356309611932374536991119384765625f,-0.3519000088930130942799223703332245349884033203125f,-0.617700030277669842604382210993207991123199462890625f,0.236100005352497088750851617078296840190887451171875f,0.165499989770352573525968864487367682158946990966796875f,0.525700018632412113817053977982141077518463134765625f,0.23260000923424961616348127790843136608600616455078125f,-0.47380001271963134712450482766143977642059326171875f,-0.4963000022143122524909131243475712835788726806640625f,0.29829999482333657834232099048676900565624237060546875f,0.2016000382989646677600603652535937726497650146484375f,-0.7704000031113622526390827260911464691162109375f,-0.322699993695318543274197509163059294223785400390625f,-0.5629000089228153314024893916212022304534912109375f,0.1641000004686412605536816045059822499752044677734375f,0.140399999292939892114873146056197583675384521484375f,-0.076299988132715323985166833153925836086273193359375f,1.071100010120868972762764315120875835418701171875f,0.70879999364614487689095767564140260219573974609375f,0.612099974685907710636456613428890705108642578125f,-0.52529998333454130943209747783839702606201171875f,0.509599967831373845683629042468965053558349609375f,-0.034700003701448434156873190659098327159881591796875f,0.409199996906519203321295208297669887542724609375f,0.0323999995946884222774997397209517657756805419921875f,-0.2266999962359665943267827969975769519805908203125f,0.52469998229965575387012677310849539935588836669921875f,0.109099997488409261592323673539794981479644775390625f,-0.521799995523691162446766611537896096706390380859375f,0.812199988949299012119809049181640148162841796875f,0.0042999865829944639017412555404007434844970703125f,0.04219999729394885434885509312152862548828125f,-0.9698000170707701261818556304206140339374542236328125f,-0.0423000136435032203507944359444081783294677734375f,-0.047899999088048872408762690611183643341064453125f,-0.565599990063905266879373812116682529449462890625f,0.20489998618364335669639331172220408916473388671875f,1.1938999868988986463591572828590869903564453125f,-0.906999977372586752011329735978506505489349365234375f,0.017100001917779383830975348246283829212188720703125f,1.07510001694560042295734092476777732372283935546875f,-0.18530002505779208377134636975824832916259765625f,0.36709999361038203602447538287378847599029541015625f,-0.49160000424981120659140287898480892181396484375f,0.7511000242769718671098644335870631039142608642578125f,0.3267000068902969900364041677676141262054443359375f,0.101200007322430618472708374611102044582366943359375f,-0.38200000542402268166597423260100185871124267578125f,0.56500000167638031456363023607991635799407958984375f,-0.51099999086558811622893472303985618054866790771484375f,-0.265999985277652950532711884079617448151111602783203125f,-0.34599999563395955615163757101981900632381439208984375f,0.40210002412051004672122189731453545391559600830078125f,-0.36610001488924037715833037509582936763763427734375f,-0.19499999742954965409325041036936454474925994873046875f,-0.132600000256299921375102712772786617279052734375f,0.2866999994397161355408343297312967479228973388671875f,-0.26149999743700025334192105219699442386627197265625f,0.7362000207334757906352251666248776018619537353515625f,0.3294000079989434226490629953332245349884033203125f,0.1245000103414060088624637501197867095470428466796875f,0.00870000009536742169302669935859739780426025390625f,-0.172500000614672632792689910274930298328399658203125f,-0.3999000220768156310668928199447691440582275390625f,-0.064600002551078727908162591120344586670398712158203125f,0.22300000210106374343155266615212894976139068603515625f,-0.70720002366900447388076145216473378241062164306640625f,-0.11360001161098498556611957610584795475006103515625f,-0.4934999941810966905819668681942857801914215087890625f,-0.111300001990795038864234811626374721527099609375f,0.1676999965265391079327628176542930305004119873046875f,0.6531000195682050701861953712068498134613037109375f,-0.032299999506026466633556992746889591217041015625f,-0.27370000022575258213919369154609739780426025390625f,0.0932999949723478660956743624410592019557952880859375f,0.8459999772310258236984736868180334568023681640625f,0.7244999842084942631714739036397077143192291259765625f,0.2402999756790702434727791114710271358489990234375f,-0.21150000261515378152665789457387290894985198974609375f,0.352199996101856260821705291164107620716094970703125f,-0.11159999936968090228361916160793043673038482666015625f,0.16680000349581247665042837979854084551334381103515625f,0.3465000013858079430661973674432374536991119384765625f,-0.65010001351833324179096962325274944305419921875f,-0.6416999889716503702175032231025397777557373046875f,0.1343999895215031159523277892731130123138427734375f,-0.6321000152170659358574766883975826203823089599609375f,-0.670499982953071338442896376363933086395263671875f,0.403500019624829453501746456822729669511318206787109375f,-0.330300010305643265695607624365948140621185302734375f,0.62249999597668637107972244848497211933135986328125f,0.1154999845921995671460535959340631961822509765625f,-0.092700007925927696561529955943115055561065673828125f,0.7902000212252138222623898400343023240566253662109375f,0.65340000655353069891617678877082653343677520751953125f,0.40949998471140869327200562111102044582366943359375f,0.0036000028565525976631533922045491635799407958984375f,-0.68550000299513336177170685914461500942707061767578125f,-0.94800003998726634080185249331407248973846435546875f,0.306600020194053968225489370524883270263671875f,0.342000002145767201255921463598497211933135986328125f,0.133799985972046897586551494896411895751953125f,-0.08040000434070830592769851818957249633967876434326171875f,0.63750001031905412229860985462437383830547332763671875f,0.8964000257536770543964621538179926574230194091796875f,0.52820002861619030909423599950969219207763671875f,-1.035799984169006648926369962282478809356689453125f,-0.55919999899268102439009453519247472286224365234375f,0.60150001874566072501693270169198513031005859375f,0.7789000409901147836677637315005995333194732666015625f,0.3810999621391299996275847661308944225311279296875f,0.66769999082684439173362989095039665699005126953125f,0.2352000009596348917284558410756289958953857421875f,-0.17600000271201121648800835828296840190887451171875f,0.33480002660751306819975070538930594921112060546875f,1.18340006161332222944793102215044200420379638671875f,0.7905000044107435286377949523739516735076904296875f,0.532300018113851525214386128936894237995147705078125f,-1.1190999803893273423227583407424390316009521484375f,-1.086700017339735868660000051022507250308990478515625f,0.01100000911951060089677412179298698902130126953125f,-0.332999999791383505254316332866437733173370361328125f,-0.73040001443624458232761753606610000133514404296875f,-0.518800040435791753878902454744093120098114013671875f,0.4085000037550923224216603557579219341278076171875f,0.52490002179741868104656532523222267627716064453125f,-0.156399988985061355606376309879124164581298828125f,-0.7944000300526621316521413973532617092132568359375f,0.282599987590312906604594900272786617279052734375f,0.7620000167489049403002354665659368038177490234375f,1.274599988543986928135609559831209480762481689453125f,0.75780004696250014006864148541353642940521240234375f,0.77720000012516976539700408466160297393798828125f,0.5062000217020512504717544288723729550838470458984375f,1.104000008761882867247550166212022304534912109375f,0.7449000106215475369708656216971576213836669921875f,-0.678599983075260926312921583303250372409820556640625f,-0.882899995958805305917849182151257991790771484375f,0.489300015230476947891702366177923977375030517578125f,-0.0585000053942206221790911513380706310272216796875f,0.8318999814748762045013563692918978631496429443359375f,0.294299983453750968465101323090493679046630859375f,0.062399994975328443302942105219699442386627197265625f,0.549600001657009062938641363871283829212188720703125f,0.96000003598630445633688168527442030608654022216796875f,-0.37409999575614971689674348453991115093231201171875f,0.3129000082522630510339922693674452602863311767578125f,0.28350000435113908991979769780300557613372802734375f,-0.7256999812491236045985942837432958185672760009765625f,-0.223799968444556185431792982853949069976806640625f,-0.36599999783188097435271401991485618054866790771484375f,0.0930000008344649131686310283839702606201171875f,-0.60000002726912515793600277902442030608654022216796875f,-0.57420000689029659923789949971251189708709716796875f,0.2535000159367919447817030231817625463008880615234375f,-0.122999998152256040384600055404007434844970703125f,-0.877200022476911644986330429674126207828521728515625f,0.04980000455379496315799769945442676544189453125f,0.411600005628168563731605900102294981479644775390625f,-0.4137000135064123895745069603435695171356201171875f,0.6636000261023642377011810822295956313610076904296875f,0.787799980503320451674653668305836617946624755859375f,0.18660000361651185585287748835980892181396484375f,0.16709998690485949879303007037378847599029541015625f,0.270600007237494111222986248321831226348876953125f,0.55749998331069861023934208787977695465087890625f,0.95219998477697398442387566319666802883148193359375f,0.620600005114078090429075018619187176227569580078125f,0.0578999765872953986445281771011650562286376953125f,0.5407000153541563491899069049395620822906494140625f,0.4932000043272974920682827359996736049652099609375f,-0.17580000458359723580770150874741375446319580078125f,-1.1529999808669089134127716533839702606201171875f,-0.38220001294016814341603094362653791904449462890625f,-1.1002000031948089286970571265555918216705322265625f,-0.694600033843517650211651925928890705108642578125f,0.4649000095784661112929825321771204471588134765625f,0.4071000404000291172934566930052824318408966064453125f,0.7056000006996094864319957196130417287349700927734375f,0.021800003477185914135816346970386803150177001953125f,0.8457000111818313570211103069595992565155029296875f,0.2701000015974042156585710472427308559417724609375f,-0.4821000040411949072449715458787977695465087890625f,-0.7519000404834752515625950763933360576629638671875f,-0.0989000307083127694340873858891427516937255859375f,0.2858999941110607068139870534650981426239013671875f,0.90559997054338481348167988471686840057373046875f,0.753400012439489064064446210977621376514434814453125f,-0.681799979281425549970663269050419330596923828125f,0.1870000165998941810130418161861598491668701171875f,-0.44969998090267182533352752216160297393798828125f,-0.176499998182058082107914742664434015750885009765625f,-0.37939997804760938748813714482821524143218994140625f,0.0385999949216841997667870600707828998565673828125f,-0.0013000081002711993960474501363933086395263671875f,-0.24390000716447790551910657086409628391265869140625f,0.48299998690187917471661194213083945214748382568359375f,0.49259998729228993141759929130785167217254638671875f,0.3075000105053187215986554292612709105014801025390625f,-0.4068000218987464222664129920303821563720703125f,-0.3456000147253275311953757409355603158473968505859375f,0.32309999080896378842453486868180334568023681640625f,-0.11160000249892476631430326960980892181396484375f,-0.6119999824762345497219939716160297393798828125f,-0.5694000094145537094192377480794675648212432861328125f,-0.615599990510940386911897803656756877899169921875f,0.6369000272959473019085407941020093858242034912109375f,0.539699999827146115194409503601491451263427734375f,-0.09960000175014138701357069294317625463008880615234375f,0.50129999867528685086881523602642118930816650390625f,0.0248999938577412205376049314509145915508270263671875f,0.3996000117897988701542999478988349437713623046875f,-0.4407000147208572915502600153558887541294097900390625f,0.1827000210762026455313389305956661701202392578125f,0.4214999913275241372190293986932374536991119384765625f,-0.0402000195860860998209318495355546474456787109375f,0.8171999918550252317572812899015843868255615234375f,0.630299976181984078493769629858434200286865234375f,0.4332000142738223757987725548446178436279296875f,-0.41339998636245756102880477556027472019195556640625f,-0.34589999705404039787026704289019107818603515625f,-0.540599994683265805406335857696831226348876953125f,-0.07200000002235185103671710749040357768535614013671875f,-0.487199977326393085519384840154089033603668212890625f,0.231300009776651993576024324283935129642486572265625f,-0.082500009126961482142093018410378135740756988525390625f,1.3814999881982803486835109652020037174224853515625f,0.54850002357363702998327426030300557613372802734375f,0.169999996721744484062810442992486059665679931640625f,0.1342000062942505511642821147688664495944976806640625f,0.59399997314810804027729318477213382720947265625f,0.6236000019252296855398753905319608747959136962890625f,1.0679999927878380816537173814140260219573974609375f,0.358800022703409382529571303166449069976806640625f,-0.03699997860193260379446655861102044582366943359375f,-0.6632000248908997974695012089796364307403564453125f,-0.81070000075101855685488771996460855007171630859375f,-0.1497000008702276119265661691315472126007080078125f,-0.73679997242689143632787818205542862415313720703125f,-0.6258000124812124198570018052123486995697021484375f,-1.361000006139278184491558931767940521240234375f,-0.5634000197410582444490501075051724910736083984375f,-0.02480001438856138662458761245943605899810791015625f,0.333400013929605609774853292037732899188995361328125f,-1.3254000004291537351264196331612765789031982421875f,-0.60360002129673961945854898658581078052520751953125f,0.10399999773502344169173738919198513031005859375f,-0.7474000191152097016811239882372319698333740234375f,1.2798999656200411578765852027572691440582275390625f,1.1629000148832790983988161315210163593292236328125f,0.153700004751980312533987671486102044582366943359375f,0.15630000724345449736318869327078573405742645263671875f,0.64749996829777989315601871567196212708950042724609375f,0.6418999959662550214289922223542816936969757080078125f,-0.592499979138374488485396796022541821002960205078125f,-0.16140000638216722705919892177917063236236572265625f,0.3459000018596649628221939565264619886875152587890625f,0.8384999808073043681133640347979962825775146484375f,0.189000003956258322279637695828569121658802032470703125f,0.169499998569488496968915569595992565155029296875f,0.871199980947375163253809660091064870357513427734375f,0.599999978989362858783351839520037174224853515625f,0.1230000006482006025265718562877736985683441162109375f,0.5819999759197234112662044935859739780426025390625f,-0.51000000838190329144339330014190636575222015380859375f,0.37950003194808967776907593361102044582366943359375f,-0.03329999381750814446689901160425506532192230224609375f,-0.55710000225305567056466315989382565021514892578125f,-0.76739999843984840754984588784282095730304718017578125f,-0.60989998171329506959637001273222267627716064453125f,-0.240600006678700406581583592924289405345916748046875f,-0.67800000619888312058947121840901672840118408203125f,0.45360001190900811618433863259269855916500091552734375f,-0.00690000919103628262973870732821524143218994140625f,-0.3617999909788369716778788642841391265392303466796875f,-0.7226999832272529289411977515555918216705322265625f,-0.79320000633895393349348523770458996295928955078125f,0.31349999898672109566177823580801486968994140625f,0.8318999884039162129312217075494118034839630126953125f,0.377699971008301105257487506605684757232666015625f,-0.0458999988421797677773383838939480483531951904296875f,-0.14490000346899034600056666022283025085926055908203125f,0.59549998254328950775260409500333480536937713623046875f,0.30899998030066522236580794924520887434482574462890625f,-0.9128999968528745956319880860974080860614776611328125f,-0.25360000326633436174006419605575501918792724609375f,0.755799997878074947266213712282478809356689453125f,0.07519999455213532968400613754056394100189208984375f,-0.085399970328808194608427584171295166015625f,-0.944600008660554379957829951308667659759521484375f,-0.162199998337030304451644724395009689033031463623046875f,-0.46020001947432775946111860321252606809139251708984375f,-1.1287999793410303794871651916764676570892333984375f,-0.58360001459121679801000937004573643207550048828125f,-1.436499970614910193233981772209517657756805419921875f,-0.8133000165790316149383443189435638487339019775390625f,0.2192999957472085981180498492904007434844970703125f,0.27810000667870038437712310042115859687328338623046875f,0.9666999846689403597110867849551141262054443359375f,0.98450002511218170919704562038532458245754241943359375f,-0.918499962031841477028137887828052043914794921875f,-1.08890000284314059086909765028394758701324462890625f,1.015700025784969096065424309927038848400115966796875f,0.57910003658533160120214233756996691226959228515625f,0.45349998654425149879898526705801486968994140625f,0.36530000471770740499977137005771510303020477294921875f,0.365199994403123839870062283807783387601375579833984375f,0.25100000960379842407377282142988406121730804443359375f,1.0318999939918516162151718162931501865386962890625f,0.4457000345766548576875720755197107791900634765625f,0.32660000246167175674827376496978104114532470703125f,0.3192000208973888586427847258164547383785247802734375f,-0.3130000085830690892407801584340631961822509765625f,-0.576000019103288796173956143320538103580474853515625f,-1.01910001818239681625755110871978104114532470703125f,-0.9069000085949898704029692453332245349884033203125f,0.3539999838396905573745243600569665431976318359375f,0.98190000323057180509067620732821524143218994140625f,-0.71999998960643996070984940160997211933135986328125f,0.5178000298261640210739642498083412647247314453125f,-0.4260000111311674775294022765592671930789947509765625f,0.056400004500895828396522801995160989463329315185546875f,-0.2903999905943870363245196131174452602863311767578125f,-0.49139996991157541827988097793422639369964599609375f,-0.813299993258714604138504000729881227016448974609375f,-1.022999977327883502908889568061567842960357666015625f,0.199500001139938820671204666723497211933135986328125f,0.01139999561607840661281443317420780658721923828125f,0.7911000025771557364606678675045259296894073486328125f,0.3110999840252102188742355792783200740814208984375f,-0.815099987031519201963192244875244796276092529296875f,-0.13799997967481658633914776146411895751953125f,0.6375000179931522570342394828912802040576934814453125f,0.788100016587972351800317483139224350452423095703125f,0.3068999972328543268673683996894396841526031494140625f,0.21749999083578597236510177026502788066864013671875f,0.2466000028342008876247604121090262196958065032958984375f,0.239099994172900898181666207165108062326908111572265625f,0.2427000163078310013275995515868999063968658447265625f,0.5546999797105787166628942941315472126007080078125f,0.47460001074671753773515092689194716513156890869140625f,0.36029999838471393758254635031335055828094482421875f,-0.3173999969869851778270231079659424722194671630859375f,0.21450001049041755862845093361102044582366943359375f,0.0885000048726798382858760305680334568023681640625f,1.009800040316581881683077881461940705776214599609375f,0.01729998801350607351423604995943605899810791015625f,-0.4063000092923640949038599501363933086395263671875f,0.464899999892711690563373849727213382720947265625f,0.22890001148581529122338906745426356792449951171875f,0.0525999704539774626255166367627680301666259765625f,-0.39060001375675224011274622171185910701751708984375f,0.662999989509582832170053734444081783294677734375f,0.440000007450580543633122942992486059665679931640625f,1.289699980902671683224980370141565799713134765625f,0.69550003227591528087714323191903531551361083984375f,0.46509996494054828275466206832788884639739990234375f,0.5272999878644935023430662113241851329803466796875f,1.22680000669956168479757252498529851436614990234375f,0.83020005579590883115770338918082416057586669921875f,0.92589999917745569035787411849014461040496826171875f,0.7041000377774242480199973215349018573760986328125f,0.2315000081658364905479174922220408916473388671875f,-0.302100017452240354742798444931395351886749267578125f,0.803899996882677214671275578439235687255859375f,0.286500020831823576372698880732059478759765625f,-0.2049000004887577830459122196771204471588134765625f,0.43669999631047229371461071423254907131195068359375f,-0.1901000103890897463543296908028423786163330078125f,-0.47410002526044880966082928352989256381988525390625f,0.80919999474585058152342753601260483264923095703125f,-0.07499998919665795682476527872495353221893310546875f,0.3529000117540359582335440791212022304534912109375f,-0.00469999848604185643807795713655650615692138671875f,-0.24099999451637277303461814881302416324615478515625f,0.5096999963998793958808164461515843868255615234375f,-0.23550000701844686634700565264211036264896392822265625f,0.356399987494945502675136594916693866252899169921875f,0.535500001989305030036803145776502788066864013671875f,0.6912000021815298822502882103435695171356201171875f,-0.11159999981671564484031478059478104114532470703125f,0.505799989235401259435320753254927694797515869140625f,0.653999989651143476265815479564480483531951904296875f,0.6914999905824661396991359652020037174224853515625f,0.545700011703372123150757033727131783962249755859375f,0.7610999764442443193956933100707828998565673828125f,0.55769997998625020496632487265742383897304534912109375f,0.27029998303651847635364902089349925518035888671875f,0.86040003172159218447490047765313647687435150146484375f,0.85889999434947927881012219586409628391265869140625f,0.51570001829713596475102121985401026904582977294921875f,0.38069998823404294086003574193455278873443603515625f,-0.6093000160127879372140569103066809475421905517578125f,-0.02699999678134901159864966757595539093017578125f,0.0080999998614192936230438135680742561817169189453125f,0.317699976074695467076480781543068587779998779296875f,0.6980999889090655496914905597805045545101165771484375f,-0.0491999922871588779571538907475769519805908203125f,-0.1107000033214688983207452110946178436279296875f,0.409200017023086726197789175785146653652191162109375f,-0.227399999780952821737400881829671561717987060546875f,0.5960999822556971583509266565670259296894073486328125f,0.2888999990433454012617175976629368960857391357421875f,0.658200016844272539628946105949580669403076171875f,0.60420001635253439697947897002450190484523773193359375f,0.416800011914968404624914910527877509593963623046875f,-0.255099978649616421222390272305347025394439697265625f,-0.30929999328851653928040832397527992725372314453125f,0.05020002018213265415624846355058252811431884765625f,-0.76020001198649378437721679802052676677703857421875f,-0.026500008016824894152563274474232457578182220458984375f,0.88790003496259484183639187904191203415393829345703125f,-1.2079000329136846403343952260911464691162109375f,-0.89710006439089884366921978653408586978912353515625f,-0.3105000225901601851319355773739516735076904296875f,-0.313500037580729529196332805440761148929595947265625f,0.36270001928806294699825230054557323455810546875f,0.87450002911686919304656839813105762004852294921875f,-0.266400036370754378367564640939235687255859375f,-0.650000027865171858820758643560111522674560546875f,-0.69149995819106724326275070779956877231597900390625f,-0.7706999948464325900232552157831378281116485595703125f,-0.8997999949425459220719858421944081783294677734375f,-0.07900000391900530161137794493697583675384521484375f,1.066199979755282356563839130103588104248046875f,0.99860002092421018549117661677883006632328033447265625f,-0.2613000100374218703791484585963189601898193359375f,-0.95010003885030780423903706832788884639739990234375f,-0.4803999908924101447382781771011650562286376953125f,-0.98360004767179542994881558115594089031219482421875f,0.29549998742341987423287719138897955417633056640625f,-0.16170000674128548467933796928264200687408447265625f,-1.061599963271618207727442495524883270263671875f,-0.861999998867510885958154176478274166584014892578125f,0.806399998223781722117564640939235687255859375f,-0.222000003933906508990503425593487918376922607421875f,-0.376499989427626025051409897059784270823001861572265625f,-0.198299990874529097339973304769955575466156005859375f,-0.59280000222176310042954128221026621758937835693359375f,0.51060001703500734748786271666176617145538330078125f,0.83250002123415478560986002776189707219600677490234375f,-0.408000012662261879459180136109353043138980865478515625f,-0.52770002737194321440483690821565687656402587890625f,-0.3641999940752977948932311846874654293060302734375f,-0.5949000226914884503770508672459982335567474365234375f,-0.4905000076889987070671850233338773250579833984375f,0.6555000119358300736394085106439888477325439453125f,-0.2318999892234803184010161203332245349884033203125f,-0.27780000333935017664543920545838773250579833984375f,0.4127999916791917911496057058684527873992919921875f,-0.85409998456165181490717941414914093911647796630859375f,-0.44999998161569276788895876961760222911834716796875f,-0.01379999558329580988669249563827179372310638427734375f,-0.7223999943792820399579568402259610593318939208984375f,0.7782000032097100561401248342008329927921295166015625f,0.366599987336993393682860187254846096038818359375f,-0.6549000242561102691496444094809703528881072998046875f,0.43859999879598632332999841310083866119384765625f,-0.8304000230044128461059926848975010216236114501953125f,0.0888000249385836326609933166764676570892333984375f,-0.03150000105053185617265398832387290894985198974609375f,0.47699999856948860355032593361102044582366943359375f,-0.7403999814301727155907428823411464691162109375f,-0.5237999787449840738418060936965048313140869140625f,0.335999993920326200935733140795491635799407958984375f,-0.97070000000595957345694841933436691761016845703125f,-0.71000000201165658353374965372495353221893310546875f,-0.85520003634989294472035226135631091892719268798828125f,0.3158999892681837451391402282752096652984619140625f,0.36770000297129123989492427426739595830440521240234375f,0.2239999780654908789756518672220408916473388671875f,0.0115999982893466579980668029747903347015380859375f,-0.0785000055432316656833791057579219341278076171875f,0.25209999808073035865163546986877918243408203125f,0.270799989458918588525193626992404460906982421875f,0.29820000633895393793437733620521612465381622314453125f,-1.1748999805212019253985999966971576213836669921875f,-0.7883000251471994346275096177123486995697021484375f,-0.076100000528246170006241300143301486968994140625f,-0.013700001753121637815269195925793610513210296630859375f,0.8516999820053576808476236692513339221477508544921875f,0.103100012862682444136908088694326579570770263671875f,0.36140000567436203482429846189916133880615234375f,-0.07560000317692772142663670820184051990509033203125f,1.1235999728679662013064444181509315967559814453125f,0.678200005668401217917562462389469146728515625f,-1.1350999638974668126678579938015900552272796630859375f,-0.52710000683516222697022612919681705534458160400390625f,-0.2254000199496744016869342885911464691162109375f,-0.5032000467956077027764649756136350333690643310546875f,0.9907999695658684657928461092524230480194091796875f,0.27500000715255712435691748396493494510650634765625f,0.58149998540431280247275935835205018520355224609375f,0.531900006620585674710355306160636246204376220703125f,0.93009999504089346800128623726777732372283935546875f,0.411900043463706566626569838263094425201416015625f,-0.74160001736283309714536926549044437706470489501953125f,-0.21299999539554104188709970912896096706390380859375f,0.1736999971523880414991936049773357808589935302734375f,-0.063300008234381532901124955969862639904022216796875f,0.3473999948635697609944372743484564125537872314453125f,0.58199998605251312966402110760100185871124267578125f,0.4844999899640678453494047062122263014316558837890625f,0.100499997258186368753740680404007434844970703125f,0.4608000013053417109887277547386474907398223876953125f,0.32489999653995038642051440547220408916473388671875f,-0.4281000086903572476870749596855603158473968505859375f,-0.383699980425834752395530813373625278472900390625f,0.0206999999128281986227051447713165543973445892333984375f,-0.025799998399615275967988736738334409892559051513671875f,0.1371000043764711573412995448961737565696239471435546875f,0.7820999813169240422183747796225361526012420654296875f,-0.24659999948143962456015287898480892181396484375f,0.1647000086188317169444417231716215610504150390625f,0.4703999783009289448187928428524173796176910400390625f,0.54929996765851996087803854607045650482177734375f,-0.5276999901562928751985737108043394982814788818359375f,-0.8507999713726344115372057785862125456333160400390625f,-0.663600027666986225227674367488361895084381103515625f,-0.2066999989926809888629577471874654293060302734375f,0.2105999953910708821780417565605603158473968505859375f,0.735899972355365861176323960535228252410888671875f,0.3926999922797083097947279384243302047252655029296875f,0.18299998552352203518012174754403531551361083984375f,0.90930001735389243133766967730480246245861053466796875f,-0.272100012385845246853932621888816356658935546875f,0.820999990530312029335391343920491635799407958984375f,-0.007999995134770898630449664779007434844970703125f,0.4411999987356363117640967175248079001903533935546875f,0.15839999692365491767986895865760743618011474609375f,1.3128999715209008769534193561412394046783447265625f,0.890100001001357110652634219150058925151824951171875f,-0.5594000202178950331699525122530758380889892578125f,-0.942000047743321022863938196678645908832550048828125f,-0.128699974167346908870968036353588104248046875f,-0.82170003281831771602128355880267918109893798828125f,0.1597999869704247455359791274531744420528411865234375f,-0.25740001226067565909261247725225985050201416015625f,0.32500001229345798492431640625f,0.33620002162754569763336576215806417167186737060546875f,0.611699945348501472608404583297669887542724609375f,0.5534999811053264995308609286439605057239532470703125f,0.032600016796589414980189758352935314178466796875f,-0.70140003562569663841941292048431932926177978515625f,0.405900007969140885766279325252980925142765045166015625f,-0.37509999910145974677533331487211398780345916748046875f,-0.3746000135779379291989243938587605953216552734375f,0.09359999015331244009985312004573643207550048828125f,-0.4480999993026255623362885671667754650115966796875f,0.11749999150633794187115199747495353221893310546875f,-0.148999988608062405859300270094536244869232177734375f,0.42120000460296846966912198695354163646697998046875f,-0.6158999957501887223543235450051724910736083984375f,-0.306900006061792129941068196785636246204376220703125f,1.5708000017523762181781421531923115253448486328125f,-0.110099992358684506399413294275291264057159423828125f,-0.04860000084489579652569091194891370832920074462890625f,0.70949999029934407435149523735162802040576934814453125f,-0.02819999991282828855077013940899632871150970458984375f,0.135600008504092872652080359330284409224987030029296875f,0.5684999854266641339251009412691928446292877197265625f,0.517199986863136729198231478221714496612548828125f,-0.6072000319764019327095638800528831779956817626953125f,0.207299992036819702434513601474463939666748046875f,-0.95490001676827684917725491686724126338958740234375f,0.0279000272512437419436537311412394046783447265625f,-0.2046000087350607066216667817570851184427738189697265625f,0.2993999921292067600830932860844768583774566650390625f,0.1092000068530440959335692241438664495944976806640625f,-0.08100000546872632156691906857304275035858154296875f,0.7424999794363973393274136469699442386627197265625f,0.575099979990721266176478820852935314178466796875f,-0.957600035087764478447525107185356318950653076171875f,-0.0071999952077863582644567941315472126007080078125f,-0.349499997004866502781084136586287058889865875244140625f,0.53670000370517367827005728031508624553680419921875f,0.3227999932810663441529186457046307623386383056640625f,-0.1173000018715857351025988464243710041046142578125f,0.27749999530613411735657791723497211933135986328125f,-0.271799990010261449668860223027877509593963623046875f,0.058200004662573245894918727572076022624969482421875f,-0.70049998854100692380342252363334409892559051513671875f,-0.03090000130087133811684907414019107818603515625f,-0.140699997472763183026245314977131783962249755859375f,0.7554000219985843589398655240074731409549713134765625f,-0.78010005509257407396717098890803754329681396484375f,-0.609199997055530406697698708740063011646270751953125f,-0.45920002015233052361509180627763271331787109375f,-0.88950000393390649122693503159098327159881591796875f,-0.04170000227093684230794679024256765842437744140625f,0.3781999633491039958244073204696178436279296875f,0.3245999830305568156774143062648363411426544189453125f,0.2149999940395357356237582280300557613372802734375f,0.8034000108003613860319092054851353168487548828125f,-0.056199957776069897619208859396167099475860595703125f,-0.70180000640153838986634582397527992725372314453125f,0.8647000156223771938357458566315472126007080078125f,0.9635000524818904121815421603969298303127288818359375f,0.3891000070810317623681839904747903347015380859375f,-0.30130000243782983915252771112136542797088623046875f,-0.570300003525614496169282574555836617946624755859375f,-0.60350003363192128436054417761624790728092193603515625f,-0.4869000217080117209889067453332245349884033203125f,0.272299993973970089200520305894315242767333984375f,1.102599998766184086207431391812860965728759765625f,0.42880002210736289924852826516143977642059326171875f,-0.1383000077128411220428461092524230480194091796875f,-0.71330003453493162623999523930251598358154296875f,-0.946599977576732865003350525512360036373138427734375f,-0.64800000917911493303336101234890520572662353515625f,-0.403300011885165954339527161209844052791595458984375f,-0.76070003859996848660784962703473865985870361328125f,-0.9515000257492065571796047152020037174224853515625f,-0.061300004822015807803836651146411895751953125f,-0.3804000040054322795413099811412394046783447265625f,-0.5604000043034549349840744980610907077789306640625f,-0.370200009937584451069625401942175813019275665283203125f,-0.291599995309114401464967158972285687923431396484375f,0.11250000268220905486060701150563545525074005126953125f,-0.62550000411272055345079934340901672840118408203125f,0.564599988394975493832816937356255948543548583984375f,0.5015999962031845171850363840349018573760986328125f,0.6977999947339295072623599480721168220043182373046875f,-0.2130000056028364951998810283839702606201171875f,-0.5921999938666819840449306866503320634365081787109375f,0.326400037711858548306054217391647398471832275390625f,0.6435000238716603870869903403217904269695281982421875f,0.0815999914348122246110506239347159862518310546875f,-0.2103000013276934421213582027121447026729583740234375f,0.56880000616312020866871534963138401508331298828125f,-0.6279000200986863955421313221449963748455047607421875f,-0.332399998477101110694320595939643681049346923828125f,0.5241000038400291938245345590985380113124847412109375f,-0.2591999988436697321958490647375583648681640625f,0.1055999949440360463626120690605603158473968505859375f,0.489299976301193151329016473027877509593963623046875f,-0.3561000102326274951991535999695770442485809326171875f,0.5361000193595888418940376141108572483062744140625f,-0.54719999040216205088427159353159368038177490234375f,-0.519299980026483698480888051562942564487457275390625f,-0.64350002130121009003005383419804275035858154296875f,0.038100000196695571474947428214363753795623779296875f,0.1209000054359437026363366385339759290218353271484375f,-0.6435000177621841288555515347979962825775146484375f,0.4092000058472156087674420632538385689258575439453125f,0.47500000268220876620262060896493494510650634765625f,1.18270001124143586679338113754056394100189208984375f,0.52110002070069327828605310060083866119384765625f,-0.9726000331878659199702497062389738857746124267578125f,-0.408600032919645972384614651673473417758941650390625f,-0.44799999666213974336415049037896096706390380859375f,-0.847000037014484785657941756653599441051483154296875f,0.2239000271320339408021027338691055774688720703125f,-0.48129998232126158796972958953119814395904541015625f,-0.193499990642070684998543583787977695465087890625f,-0.84110002040266973466486888355575501918792724609375f,0.6390999753415587125715546790161170065402984619140625f,0.89650001148134428330394030126626603305339813232421875f,0.024399987733364270070524071343243122100830078125f,0.594600010597705619375119567848742008209228515625f,-0.305100018733739819509764856775291264057159423828125f,0.248300009053945558434861595742404460906982421875f,0.5453000149250026851177608477883040904998779296875f,0.95630005936026662993754143826663494110107421875f,-0.64229996849298487404666957445442676544189453125f,-0.26410000261068322657820317544974386692047119140625f,-0.22640000392347570112860921653918921947479248046875f,-0.322400020121038355380704842900740914046764373779296875f,0.63500001281499829364207698745303787291049957275390625f,0.8208000465303666448591002335888333618640899658203125f,-0.68439998534917823036494155530817806720733642578125f,-0.344800013941526373173473984934389591217041015625f,-0.73559998365640666406761738471686840057373046875f,-1.137000022858380976487069347058422863483428955078125f,-1.108499995708465579724588678800500929355621337890625f,-0.317400017327070127493016116204671561717987060546875f,0.56490000648647542558222767183906398713588714599609375f,0.94560001375675195589565191767178475856781005859375f,-0.44460001834630968886585833388380706310272216796875f,-0.7848000237017866975719471156480722129344940185546875f,-0.77460001868158590010438047102070413529872894287109375f,-0.02249998897314053891705043497495353221893310546875f,-0.6452999806523320724949144278070889413356781005859375f,0.193200006115436195841539301909506320953369140625f,-0.484500008739531029533509354223497211933135986328125f,0.531000011503696356385262333787977695465087890625f,0.8456999938219784684889646086958236992359161376953125f,0.204299983137101126562384934004512615501880645751953125f,0.256199997375905486052261039731092751026153564453125f,-0.576300020468234919235328561626374721527099609375f,0.147900005011260604614875546758412383496761322021484375f,-0.483300023359060304528611595742404460906982421875f,0.6729000348508360485055845856550149619579315185546875f,-0.1671000097036365872327223769389092922210693359375f,-0.5228999958470463838011710322462022304534912109375f,-0.7868999814748764976002348703332245349884033203125f,-0.4548000097319484513747056553256697952747344970703125f,-0.241199998344480803780243149958550930023193359375f,0.805800028127432010904840353759936988353729248046875f,0.26850000260025230769400650387979112565517425537109375f,-0.23580000521689659098711899787303991615772247314453125f,-0.37919998573064805924559550476260483264923095703125f,-0.8255999950557946220897065359167754650115966796875f,0.089700022625923025998417870141565799713134765625f,-0.383700000616908065698140717358910478651523590087890625f,-0.9597000457227233827239842867129482328891754150390625f,0.02729999427199369943508600044879131019115447998046875f,-0.2643000097543002357980412853066809475421905517578125f,-0.781599982345104304926053373492322862148284912109375f,-0.29060000258088081181995221413671970367431640625f,-1.7179999933838843162448029033839702606201171875f,-0.849600024455785529653439880348742008209228515625f,1.3280000169277190735783733543939888477325439453125f,0.969000049889088277410564842284657061100006103515625f,-1.280400004601478514132395503111183643341064453125f,-0.42360004141330787064134710817597806453704833984375f,0.349399976110458254652257892303168773651123046875f,0.79460002371072757654246743186376988887786865234375f,0.99629997844696038811207472463138401508331298828125f,0.45730002288222326711775167495943605899810791015625f,0.271600024217367064238715101964771747589111328125f,-0.47180000938177091285297137801535427570343017578125f,0.64460001119375220923757296986877918243408203125f,0.26180001906752625728813654859550297260284423828125f,0.02729999538958072236027874168939888477325439453125f,0.16270000595152378508601032081060111522674560546875f,-0.0633000010073185848114007967524230480194091796875f,0.008699993389844618008055476821027696132659912109375f,-0.52969999878406515136930465814657509326934814453125f,-0.7711000299394132895258735516108572483062744140625f,-0.2207000100642442230736151032033376395702362060546875f,0.200300003227591449928013389580883085727691650390625f,-1.2143000154912471089119208045303821563720703125f,-0.3031000127136707078534527681767940521240234375f,0.204399995034933057436177250565378926694393157958984375f,-0.278099995726346804758577491156756877899169921875f,-0.241500006079673756875791923448559828102588653564453125f,0.14639999595135455567884719130233861505985260009765625f,-0.203999993562698322957515983944176696240901947021484375f,-0.513299986702203892008355978759936988353729248046875f,-0.5622000031426548360968808992765843868255615234375f,-0.9419999955892564003079314716160297393798828125f,0.6606000146135688755322235010680742561817169189453125f,0.51749999105930299947431194595992565155029296875f,-0.212400013750791760713809708249755203723907470703125f,-0.78119997804164853505426435731351375579833984375f,0.531000011801719740134331004810519516468048095703125f,-0.2658000241041182931667208322323858737945556640625f,0.52590000789463531294387621528585441410541534423828125f,0.82589997891187660883360877051018178462982177734375f,-0.2436000016272068524614269335870631039142608642578125f,0.678300028872489857434402438229881227016448974609375f,0.43680000807791952155412218417041003704071044921875f,0.6948000081300733654643408954143524169921875f,0.066300002400577096040024116518907248973846435546875f,-0.1431000076472759285906022341805510222911834716796875f,-0.174900000786781362194233224727213382720947265625f,-0.269399996078014236644548873300664126873016357421875f,-0.63810001271367078334861844268743880093097686767578125f,-0.05609999223947514934707214706577360630035400390625f,0.1497000010192394425789785827873856760561466217041015625f,-0.332100010672211620743610183126293122768402099609375f,-0.0206999991863965338012576467008329927921295166015625f,-0.683400009012222309223716365522705018520355224609375f,0.297000005051493654040228875601314939558506011962890625f,0.29460000866055491286488177138380706310272216796875f,-0.05119996314048780305938635137863457202911376953125f,-0.7888000207662582141665552626363933086395263671875f,-0.2512000194668768671135694603435695171356201171875f,-0.523600031429529710891301874653436243534088134765625f,-1.49309999289512607134611243964172899723052978515625f,-0.79390005335807867226094458601437509059906005859375f,-1.217600012326240488391704275272786617279052734375f,-0.68140003234744117577292854548431932926177978515625f,-0.8403000050902367235039491788484156131744384765625f,-0.07469999237656566037912853062152862548828125f,-0.630899987107515158868409343995153903961181640625f,-0.7569000190258023952338817252893932163715362548828125f,-0.12730000514984141091190394945442676544189453125f,0.222900010263920034248030788148753345012664794921875f,0.873899993157386756337245969916693866252899169921875f,0.549100021535158244745389310992322862148284912109375f,1.049499972641468215073246028623543679714202880859375f,0.65590001094937289138897540397010743618011474609375f,-0.118799998041987375785311087383888661861419677734375f,-0.108600001031160242614959088314208202064037322998046875f,0.3283000054776668008571505197323858737945556640625f,-0.02510000830292735685134175582788884639739990234375f,0.4824999698996546015905551030300557613372802734375f,0.77030001164674732905268683680333197116851806640625f,0.144799988162517490053460278431884944438934326171875f,0.997200032311678086216488736681640148162841796875f,-0.986099979871511322926380671560764312744140625f,-0.2641000091671941163440351374447345733642578125f,0.539800017666816955852482351474463939666748046875f,-0.71220000623464585487454314716160297393798828125f,-0.91200000572204598103809303211164660751819610595703125f,0.09840003374814987324725734652020037174224853515625f,-0.3054000152930618217084202115074731409549713134765625f,0.20639999941587472420678750495426356792449951171875f,-0.63510002514123942152224344681599177420139312744140625f,-0.90179997570514647264872110099531710147857666015625f,-0.888000009477138529945250411401502788066864013671875f,-1.104300008785724429571928340010344982147216796875f,-0.2360999894455073633281472211820073425769805908203125f,-0.926400017893314497996470890939235687255859375f,-0.4971000092938542280762703740037977695465087890625f,-0.0101999920189381754198620910756289958953857421875f,0.47010000983029609500363221741281449794769287109375f,0.0831000009417532936595307546667754650115966796875f,0.473700005199015172774323900739545933902263641357421875f,0.517199989694356876412939527654089033603668212890625f,0.598499993138015273785867975675500929355621337890625f,0.659999978095293204916060858522541821002960205078125f,-0.0030000006482004959451614922727458178997039794921875f,0.05010000059157604379578287989716045558452606201171875f,-0.28710000549405823022652839426882565021514892578125f,0.012900008475780655459175250143744051456451416015625f,0.9338999958023428415998523632879368960857391357421875f,0.3494999794960023109524627216160297393798828125f,0.8826000147476793511458481589215807616710662841796875f,-0.357600016051530911198597095790319144725799560546875f,-0.0183000027582049806795794211211614310741424560546875f,-0.529799983096122861780941093456931412220001220703125f,-0.41280001965612189707144352723844349384307861328125f,0.63950000616908031503271558904089033603668212890625f,0.44489999564588078584392860648222267627716064453125f,-0.31730000030696370316718457615934312343597412109375f,-0.92549999189376830344144764239899814128875732421875f,-0.21830001069307325423096699523739516735076904296875f,0.38079998983144758284424824523739516735076904296875f,0.797600028866529697779697016812860965728759765625f,0.70309997337460539146292148870998062193393707275390625f,0.579700004968046744835419303853996098041534423828125f,-0.0530000108182431262093814439140260219573974609375f,-0.5172000260531905979632938397116959095001220703125f,-0.7093000108718872098734209430404007434844970703125f,0.01849998021125731639813238871283829212188720703125f,1.2986000113129616551077560870908200740814208984375f,0.448000016033649473001787555404007434844970703125f,-0.0865000126361845644851200631819665431976318359375f,-0.471700027006864974765676379320211708545684814453125f,0.657800009560585152712519629858434200286865234375f,0.1446000071704389711158000864088535308837890625f,-0.55509999489188199817135682678781449794769287109375f,-0.24010000219345073446675087325274944305419921875f,0.41899999639391882766403796267695724964141845703125f,0.5294000182807445664678880348219536244869232177734375f,-0.700499954044818906595537555404007434844970703125f,-0.637899999535083139790003770031034946441650390625f,0.6223000120043753469190050964243710041046142578125f,0.360100025439262783066851625335402786731719970703125f,-1.0981000089883803383372651296667754650115966796875f,-0.986900043314695718521534217870794236660003662109375f,-1.5456999921083447446790160029195249080657958984375f,-0.7308000166535375541343455552123486995697021484375f,-0.0879000035583972216812753686099313199520111083984375f,0.7826999929130076250061165410443209111690521240234375f,-0.14549999986588957323618842565338127315044403076171875f,-0.68699999022483826394136485760100185871124267578125f,0.85980001152008778131374810982379131019115447998046875f,0.13139998749494541385729462490417063236236572265625f,0.57989999095201483214623294770717620849609375f,0.43529998083859700042097529149032197892665863037109375f,-0.190800006558001111756084355874918401241302490234375f,0.508500013589859189977460118825547397136688232421875f,-0.2019000063970686209557925394619815051555633544921875f,-0.90689999876022309166501145227812230587005615234375f,0.2951999958559869963181654384243302047252655029296875f,0.85530000389814375694186310283839702606201171875f,-0.2955000149533153486203218562877736985683441162109375f,0.291000001668930252662903512828052043914794921875f,0.417599995748698660236186697147786617279052734375f,0.86040000936985006063650871510617434978485107421875f,-0.18029999540448182759888595683150924742221832275390625f,-0.354299997758865448105325413052923977375030517578125f,0.35940001030862334108206823657383210957050323486328125f,-3.7252902984619140625e-09f,-0.911099995927512562587935462943278253078460693359375f,-0.7214999840259554986232615192420780658721923828125f,0.2001000109478832111875590271665714681148529052734375f,0.25889999822378140237333354889415204524993896484375f,-0.6243000143364072496154904001741670072078704833984375f,-0.2387999903678892366087893606163561344146728515625f,-0.5661000221908094598433081046096049249172210693359375f,-0.068100001183897251877397138741798698902130126953125f,-0.32459997841119747619131885585375130176544189453125f,-0.8708000336408616703920415602624416351318359375f,0.414399992054701016286344383843243122100830078125f,-0.17040000787973408336029024212621152400970458984375f,-0.6430999772489069954417573171667754650115966796875f,-0.359900000488757587646659885649569332599639892578125f,-0.14870000084042533927686235983856022357940673828125f,-0.2803000131368638125906045388546772301197052001953125f,0.098699999424815221260587350116111338138580322265625f,0.14530000382363794653173272308777086436748504638671875f,0.459899987021833656086755581782199442386627197265625f,-0.04209999754056337195606829482130706310272216796875f,-0.67669998513460161149168925476260483264923095703125f,-0.6981000143527982260138742276467382907867431640625f,-0.071700024771690351599318091757595539093017578125f,0.143899991220235445865682777366600930690765380859375f,-0.887200015771389072227748329169116914272308349609375f,-0.05239999765753733385054147220216691493988037109375f,0.1816000111043454268155983299948275089263916015625f,0.203800015401840450834924922673963010311126708984375f,-0.1629000102639199809573256061412394046783447265625f,-0.75730002273321161965213832445442676544189453125f,-0.82809997933506973577522103369119577109813690185546875f,-0.1355000038892029134984795746277086436748504638671875f,0.9408000059247019208896745112724602222442626953125f,0.50980001200437552455468903644941747188568115234375f,0.3625999725401403139812828158028423786163330078125f,0.6824000004887575965284440826508216559886932373046875f,1.03700002496317011946302955038845539093017578125f,0.261300003369152544774323132514837197959423065185546875f,-0.72480001934319748801982541408506222069263458251953125f,0.18270002405643470666518624057061970233917236328125f,0.1793999903038143717282082434394396841526031494140625f,0.838200000154972091337413075962103903293609619140625f,-0.16529999886900181849114233045838773250579833984375f,-0.238799989473819973540003047673963010311126708984375f,-0.16530000959783797132018889897153712809085845947265625f,0.09540000117421154612884492962621152400970458984375f,0.068699999275803558251851654858910478651523590087890625f,-0.048900003662705415763412020169198513031005859375f,-0.0453000006012617972839251478944788686931133270263671875f,0.433199987153708943043994850086164660751819610595703125f,-0.56850000096112485170607442341861315071582794189453125f,-0.22109998925924312374036162509582936763763427734375f,0.347699996750056694594377404428087174892425537109375f,0.1133999941110612752481756615452468395233154296875f,-0.026999990634620052443182203205651603639125823974609375f,-0.765600017184019154825591613189317286014556884765625f,-0.147599998989701219898051931522786617279052734375f,-0.305700011479854705243042189977131783962249755859375f,-0.3945000016987323387951391850947402417659759521484375f,0.53580000771284108651570932124741375446319580078125f,-0.149699995207786595852184063915046863257884979248046875f,-0.72629998213499791237524050302454270422458648681640625f,0.7457999922528861791448662188486196100711822509765625f,0.94590000692605968879433930851519107818603515625f,0.8873999927401541309990307127009145915508270263671875f,0.3182999923646452344883073237724602222442626953125f,0.748200027517229582230129381059668958187103271484375f,0.79380001435875868764924234710633754730224609375f,0.67379999543428414909840284963138401508331298828125f,0.504600020134449156472555841901339590549468994140625f,1.3972999559760097554317326284945011138916015625f,0.6353000063568348121378903670120052993297576904296875f,-0.449399991123378139690203170175664126873016357421875f,-0.3014000028058883007275881027453579008579254150390625f,0.5526000114321707457065713242627680301666259765625f,-0.08960000299811365920277239638380706310272216796875f,0.06749999418854724098082442651502788066864013671875f,0.3179000026643274967597108116024173796176910400390625f,0.87189998713731764468093388131819665431976318359375f,0.52070002205967913511130973347462713718414306640625f,0.204200039970874769323927466757595539093017578125f,-0.71360000624656638734677471802569925785064697265625f,-0.72530000419616680318313228781335055828094482421875f,-1.071100050801039316183960181660950183868408203125f,-0.10530001244768527612905018031597137451171875f,-0.0447000049866737303450037188667920418083667755126953125f,-0.109399991123378281798750322195701301097869873046875f,0.0612000032618642553217114254948683083057403564453125f,-0.6568000272631646652143899700604379177093505859375f,0.21079999758005119048220876720733940601348876953125f,-1.0849999803304672951753673260100185871124267578125f,-0.633800006610154564867798399063758552074432373046875f,-0.356599972957372557402777601964771747589111328125f,-0.4022000002741810131823285701102577149868011474609375f,0.47869997306838651951466090395115315914154052734375f,0.5142999969206745181082851559040136635303497314453125f,-0.4359000039458269526448930264450609683990478515625f,0.1407000019431112747270162799395620822906494140625f,0.910799986217915691355528906569816172122955322265625f,1.241099961822480057804796160780824720859527587890625f,-0.15989998052120224514283108874224126338958740234375f,-0.07970000504255292383959385915659368038177490234375f,0.0893999973446131246390677915769629180431365966796875f,0.1185999852359296280468470285995863378047943115234375f,1.287599968218803070385547471232712268829345703125f,1.0954999814629555743294986314140260219573974609375f,0.023700056123733048707435955293476581573486328125f,-0.14509997328519830261939205229282379150390625f,-0.453000048518180875589678180404007434844970703125f,-0.94710002073049537330007296986877918243408203125f,0.4655999978125091587344286381267011165618896484375f,0.4533000202298165248748773592524230480194091796875f,0.768599989855289322804310359060764312744140625f,1.014099979513883642567861897987313568592071533203125f,-1.224899972325562469421811329084448516368865966796875f,-0.885899985170364612230287093552760779857635498046875f,0.3366000070810315492053632624447345733642578125f,0.730499994724989054617481087916530668735504150390625f,0.9533999758571385374494866482564248144626617431640625f,0.6354999920427799597888451899052597582340240478515625f,-1.2813000005006784931538277305662631988525390625f,-1.446700024473667411939459270797669887542724609375f,0.8034000106513499162019797950051724910736083984375f,0.9571999921530485178067237939103506505489349365234375f,-0.627299985666572634812609976506792008876800537109375f,-0.36620000560879706341665951185859739780426025390625f,1.31130002315044436045354814268648624420166015625f,-0.1068000080406665830423662555404007434844970703125f,0.5097000074267388214366292231716215610504150390625f,0.500799999219179170495408470742404460906982421875f,-0.868000033766031631188297978951595723628997802734375f,0.33900000086426729239974520169198513031005859375f,-0.1444999797940251795580479665659368038177490234375f,-0.19639999613761904839748240192420780658721923828125f,0.2295999879479406757809556438587605953216552734375f,-0.4959999909400940598658280578092671930789947509765625f,-0.7447000324606900534263331792317330837249755859375f,0.515199992913007775996447890065610408782958984375f,-0.81430000848770145438493273104541003704071044921875f,0.069599987053871270603622178896330296993255615234375f,1.29830001674667006028585092280991375446319580078125f,-0.27420000934898836053577042548567987978458404541015625f,0.03249999046325680041036321199499070644378662109375f,-0.068999996095895799186337171704508364200592041015625f,0.39340002778768567992528915056027472019195556640625f,0.30880000079870217888355909963138401508331298828125f,-0.6847000008702277540351133211515843868255615234375f,0.32939998654127133903557478333823382854461669921875f,0.165399995064735350069895503111183643341064453125f,-0.0671999972939492096202229731716215610504150390625f,-0.779599990630149619619260192848742008209228515625f,0.57579999191761022103719369624741375446319580078125f,-0.95450002121925336950880591757595539093017578125f,-0.26939999175667761477370731881819665431976318359375f,-0.930200016754865632861992708058096468448638916015625f,0.202600001001357998831053919275291264057159423828125f,-1.17220004140138644999069583718664944171905517578125f,0.49489999750852575743920169770717620849609375f,0.714599989438056582002900540828704833984375f,0.0801000072598458956463218783028423786163330078125f,0.92130001510381731577581376768648624420166015625f,1.041600016766786485078455370967276394367218017578125f,-0.187800010752678048220332129858434200286865234375f,0.2348999903559683133380531216971576213836669921875f,-0.97919997619390475307454835274256765842437744140625f,-0.69519998412132277820774106658063828945159912109375f,1.315799985063076604774323641322553157806396484375f,1.4557000216126443348230168339796364307403564453125f,0.3659999915361407119007708388380706310272216796875f,0.58589998397827169895890619955025613307952880859375f,-0.935100028903782298783653459395281970500946044921875f,-1.2580999995447690320560241161729209125041961669921875f,-0.2054999808669091265755923814140260219573974609375f,-0.28849999213218691096471957280300557613372802734375f,0.38999998599290819356610882095992565155029296875f,-0.07620002304315587338123805238865315914154052734375f,0.819899981713295034069233224727213382720947265625f,0.8328999858260157651557165081612765789031982421875f,-0.32519998054504384299434605054557323455810546875f,-0.602600009942054981593173579312860965728759765625f,1.337099980497360007802853942848742008209228515625f,1.2669999836683274452298064716160297393798828125f,-0.1820999579966073866899023414589464664459228515625f,0.391300025236606785483672865666449069976806640625f,0.6201000215649603575229775742627680301666259765625f,0.738800000947713808585604056133888661861419677734375f,0.5415000555515288027663700631819665431976318359375f,0.2049999944865703582763671875f,-0.81280000628232951243035131483338773250579833984375f,-0.09319999955892566134707521996460855007171630859375f,-0.460599982166289834140115999616682529449462890625f,0.45960000031590464431019427138380706310272216796875f,-0.4275000187754631753023204510100185871124267578125f,-0.076799991351366170277970013557933270931243896484375f,1.0287000009894367025253814063034951686859130859375f,0.092600001150369593005962087772786617279052734375f,0.8902000300168992907856591045856475830078125f,0.16039999506473545665130586712621152400970458984375f,-0.039700028437376300161076869699172675609588623046875f,-0.4111999896645546215268041123636066913604736328125f,-0.521300010000169233848055228008888661861419677734375f,-0.1848000018715858505657934074406512081623077392578125f,-0.812200011301040802891293424181640148162841796875f,-0.0084000163137911698640891700051724910736083984375f,0.828400025701522935150933335535228252410888671875f,-0.419400009936094164686437579803168773651123046875f,0.155800018739700618652932462282478809356689453125f,-0.0399999897181988472993907635100185871124267578125f,-0.56190000352859481580480860429815948009490966796875f,-0.29759998803734788452857173979282379150390625f,0.619299987328052026214209035970270633697509765625f,0.072600002044439282400389856775291264057159423828125f,0.05389999256134014871122417389415204524993896484375f,0.396999997675418825338056194595992565155029296875f,0.041700001525878604979880037717521190643310546875f,0.3521999813497067588485833766753785312175750732421875f,-0.423600005948543412159779109060764312744140625f,0.592799974989890898768862825818359851837158203125f,0.0882000126719475474601495079696178436279296875f,0.46479998995065674449733705841936171054840087890625f,1.028399992322921008280900423415005207061767578125f,0.93359999611377730843742028810083866119384765625f,-0.1169999610185623595270953956060111522674560546875f,-0.09749999612569837381670367904007434844970703125f,-0.0675000256299966849837801419198513031005859375f,-0.3650999887824060152752281283028423786163330078125f,0.006599967741966139556097914464771747589111328125f,-0.455400016522407469210520503111183643341064453125f,-0.65399997240304941215072176419198513031005859375f,-0.181899958378076487264252136810682713985443115234375f,-0.25619998091012241570041396698798052966594696044921875f,-0.07050000627338881553640703714336268603801727294921875f,0.164399991607666606796556152403354644775390625f,0.338399971461296189545464585535228252410888671875f,-1.05630002851486182180451578460633754730224609375f,-1.102800025057792510096987825818359851837158203125f,-0.031799998950958752175210975110530853271484375f,0.100000005364417887676609097979962825775146484375f,-0.2921999743461611842576530762016773223876953125f,0.27130001063346842471446507261134684085845947265625f,0.0836999810218816975293520954437553882598876953125f,0.0575000253319739584867420489899814128875732421875f,0.6776999942541124255512841045856475830078125f,0.30670000092983240591593130375258624553680419921875f,0.64589995596408922295950105763040482997894287109375f,0.39469997643232357376064101117663085460662841796875f,1.04010001918077410465457433019764721393585205078125f,0.2307999999642371680153019042336381971836090087890625f,-1.0979000203967093085566375521011650562286376953125f,-0.245199993360042611811877577565610408782958984375f,-0.048600001627206790288937554578296840190887451171875f,-0.1157999987721443080346261922386474907398223876953125f,-0.135000006705522590522150494507513940334320068359375f,-0.42999998182058352114154331502504646778106689453125f,-0.93490002492666235411888919770717620849609375f,-0.10419998617768300164243555627763271331787109375f,0.3765999987363812806506757624447345733642578125f,-0.402200015175342429074589745141565799713134765625f,0.5994000094890594709795550443232059478759765625f,0.2030000057518481781926311668939888477325439453125f,0.3447999991893766491557471454143524169921875f,0.35859999045133594819390054908581078052520751953125f,1.245800015163421647912400658242404460906982421875f,0.068400017356872400142719925497658550739288330078125f,0.08670000748634354437172078178264200687408447265625f,-0.04119999517798422772330013685859739780426025390625f,1.1331000349164010998492813087068498134613037109375f,0.146399990624189513255259953439235687255859375f,0.068900004780292789519080542959272861480712890625f,-0.23279999778866766746432404033839702606201171875f,-1.360099995934962890942188096232712268829345703125f,0.17759999921321867333290356327779591083526611328125f,-0.3765000062286854021209592247032560408115386962890625f,0.245199998128414142772868444808409549295902252197265625f,-0.54519998847991202328699955614865757524967193603515625f,-0.0555999969929457105166648034355603158473968505859375f,0.6367999962687491688484442420303821563720703125f,0.25939998918622719781268415317754261195659637451171875f,-0.16379995520114931650823564268648624420166015625f,0.3191000067830085384912308654747903347015380859375f,-0.17429999671578411124528429354541003704071044921875f,-0.030799995344877206804312663734890520572662353515625f,-0.625799973142147081262010033242404460906982421875f,-0.046599980705976928874179066042415797710418701171875f,0.02730005640983534931365284137427806854248046875f,0.3173000179648397534037940204143524169921875f,-0.938700018274783332117294776253402233123779296875f,-0.9292000229835508662290521897375583648681640625f,0.249299989712238101446928340010344982147216796875f,-0.00659998621940616203573881648480892181396484375f,0.464099981749058088098536245524883270263671875f,-0.078200011181831285966836730949580669403076171875f,-0.1842000009298327967144359718076884746551513671875f,-0.5517999745130541811022339970804750919342041015625f,-0.01320000194311177210693131200969219207763671875f,0.14350001299381265340571189881302416324615478515625f,-0.03420004920959485161802149377763271331787109375f,-0.4677000106453892414037909475155174732208251953125f,-0.60299997580051467593875713646411895751953125f,-0.730700007307529375566446105949580669403076171875f,0.784800008130073223355793743394315242767333984375f,1.21489998141527166808373294770717620849609375f,0.5382000029116869033174452852108515799045562744140625f,0.4448999966889619628318541799671947956085205078125f,0.122400002239644312140853799064643681049346923828125f,0.45939998754709943096230517767253331840038299560546875f,-0.5369999888837337920222125831060111522674560546875f,0.6946000064253805561520493938587605953216552734375f,-0.0446000186443331614327689749188721179962158203125f,-0.730999992430210188132377879810519516468048095703125f,0.56129998649358725515412515960633754730224609375f,-0.56500000610947598289612869848497211933135986328125f,0.238100004331767589604851309559307992458343505859375f,-0.1134000001087784870179575591464526951313018798828125f,1.0942000128507611833583723637275397777557373046875f,-0.015200011241435884556949531543068587779998779296875f,0.92920000905096511445435680798254907131195068359375f,0.459599991449713751734407196636311709880828857421875f,-0.2803000002473592378038347305846400558948516845703125f,0.0853999986410141076476065791212022304534912109375f,-0.98010000735297797547218578984029591083526611328125f,0.09239999907314777605193967247032560408115386962890625f,1.0842999777913089332059826119802892208099365234375f,-0.2585999898552895359671310870908200740814208984375f,-0.58280003966093119061042671091854572296142578125f,-0.4022000057876109480048398836515843868255615234375f,-0.3947999973267315443337110991706140339374542236328125f,-0.031399999862909311332259676419198513031005859375f,-0.252600004540383782369872278650291264057159423828125f,-0.0965999975070357397299858348560519516468048095703125f,-0.895300021779537047450503450818359851837158203125f,-0.1019999902248382994685016456060111522674560546875f,-0.043700016337633318386224345886148512363433837890625f,-0.26999999299645427441873835050500929355621337890625f,0.63190002200603512250154381035827100276947021484375f,-0.376800004464387772173949997522868216037750244140625f,0.8233000173985960401523698237724602222442626953125f,0.5896000061273571901665491168387234210968017578125f,-1.3118999731302256606113587622530758380889892578125f,-0.731999990820884960385228623636066913604736328125f,-1.326299990963935471199874882586300373077392578125f,-1.320499996364116412905787001363933086395263671875f,-0.3452999969132244384439900386496447026729583740234375f,-0.460900007894635255212278934777714312076568603515625f,-0.3192000444412226300983093096874654293060302734375f,-0.621000004053115617352887056767940521240234375f,0.520799965430796429899373833904974162578582763671875f,-0.21500000461935986351136307348497211933135986328125f,0.259499998420476885030439007095992565155029296875f,0.3268999971956014771734544410719536244869232177734375f,0.533400005920231290446054117637686431407928466796875f,0.9280999971605836673660405722330324351787567138671875f,-0.87569997720718362188563332892954349517822265625f,-1.1952999645590782762383241788484156131744384765625f,-0.5192999880731097306352239684201776981353759765625f,0.05480001468658457497440394945442676544189453125f,0.082500003427266932476413785479962825775146484375f,0.2803999949902296062731466008699499070644378662109375f,-0.0830999902687967750836151026305742561817169189453125f,0.107600004950165750727819613530300557613372802734375f,0.0450000470876688041244051419198513031005859375f,0.364900020754337361950092599727213382720947265625f,-0.41519998412132252951778355054557323455810546875f,-0.083299976867437397487492489744909107685089111328125f,-0.9102000127315523769766514305956661701202392578125f,-0.90410000886917085693994522443972527980804443359375f,0.192600003832577915119372846675105392932891845703125f,0.051399988090992110301158390939235687255859375f,0.309399999058246777394742821343243122100830078125f,-0.3699999834597111902922961235162802040576934814453125f,0.1022999827980992648690516944043338298797607421875f,-0.57219999163150792043097681016661226749420166015625f,0.4269000226020818189454075763933360576629638671875f,-0.5281999910652637186814217784558422863483428955078125f,-0.16900001651048679462974178022705018520355224609375f,-0.47899998730421078363406195421703159809112548828125f,-1.0187000141024586952198660583235323429107666015625f,-0.22239999184608461035850268672220408916473388671875f,-0.32559998037815063298694440163671970367431640625f,-0.10419999839663507401610331726260483264923095703125f,-0.7875000366568567500280551030300557613372802734375f,-0.389199993926286680334669654257595539093017578125f,-0.8850000202655792236328125f,0.0154000020682811822325675166212022304534912109375f,-0.44040000191926953476695416611619293689727783203125f,0.19879999200701714556771548814140260219573974609375f,-0.837600024545192667346782400272786617279052734375f,0.0626000142633913458922734207590110599994659423828125f,0.3352999991297718196392452227883040904998779296875f,-0.1070000003576279112849078956060111522674560546875f,0.6795000241994859635497050476260483264923095703125f,-0.5466000085711477662897550544585101306438446044921875f,-0.590600018078088595530061866156756877899169921875f,-0.22659998741149911527514859699294902384281158447265625f,0.1991999836444851013084189617075026035308837890625f,-0.57400000116229055180383511469699442386627197265625f,0.15129999006986594167756265960633754730224609375f,0.96459999152421982415717138792388141155242919921875f,-0.81239996323585561555091771879233419895172119140625f,-0.71999998092651384951068394002504646778106689453125f,-1.008899980640411087051688809879124164581298828125f,-0.4976999778628350412645886535756289958953857421875f,-1.11509998341798866050567085039801895618438720703125f,-0.90689999488592132337316797929815948009490966796875f,-0.751499972701072493919127737171947956085205078125f,-0.20599998110532791173454825184307992458343505859375f,-0.100799959135056127479401766322553157806396484375f,0.4175000134110451455171642010100185871124267578125f,-0.02430000640153906488194479607045650482177734375f,0.287199983286857474240605370141565799713134765625f,-0.409199963676929456823927466757595539093017578125f,0.2701000346779824923260093783028423786163330078125f,0.315900023019313636041260906495153903961181640625f,0.64980001349449167946659144945442676544189453125f,0.360000002980232380878078402020037174224853515625f,0.12419997380971903311319692875258624553680419921875f,0.30150005286931946812956084613688290119171142578125f,0.43080001873970008574588064220733940601348876953125f,-0.14670002477168964105658233165740966796875f,0.0817000024199483476650129887275397777557373046875f,-1.20450002390146249098279440659098327159881591796875f,-1.03450001674890490477309867856092751026153564453125f,-0.344099945539236617975120680057443678379058837890625f,-0.160399985229969122002557924133725464344024658203125f,-1.078799996328353216767936828546226024627685546875f,-0.6661999923467636364193822373636066913604736328125f,0.31859999402761474129874841310083866119384765625f,0.3503999914884567967732209581299684941768646240234375f,0.118399987852573218560792156495153903961181640625f,0.118599998646974569282974698580801486968994140625f,0.3738999845147130685063530108891427516937255859375f,-0.49659999858736991740215671597979962825775146484375f,-0.272700000251829610675713411183096468448638916015625f,0.3754000021800398823046407414949499070644378662109375f,1.3085000609755521594479432678781449794769287109375f,-0.17940000501871100624384780530817806720733642578125f,-0.12809996130466405617198688560165464878082275390625f,0.4331999891281128611808526329696178436279296875f,-0.6785000174641613313042398658581078052520751953125f,0.39519998531341560266127999057061970233917236328125f,0.77230003763437338193398318253457546234130859375f,-0.57699999275803559584119284409098327159881591796875f,0.4316999782614405756220321563887409865856170654296875f,0.21059999952614305573916908542742021381855010986328125f,0.5618000119149684223884833045303821563720703125f,0.33959999778270721915163221638067625463008880615234375f,-1.0689999975115058727936911964206956326961517333984375f,0.083199991810321893836999151972122490406036376953125f,0.7432000082015992603601262089796364307403564453125f,-0.57319998689293871763084098347462713718414306640625f,0.6802000163078307792829946265555918216705322265625f,-0.2828000116467475066883707768283784389495849609375f,0.0561000077366831106928657391108572483062744140625f,-0.376199998903274490658077411353588104248046875f,0.90639998809099164844838014687411487102508544921875f,0.09160000112056732035625827847979962825775146484375f,-0.1367000203013422066078419447876513004302978515625f,0.7539999867081643714072924922220408916473388671875f,0.1299000045120717317104208632372319698333740234375f,-0.090399992084503111300364253111183643341064453125f,-1.13249997824430437276532757095992565155029296875f,-1.0906999935984611482808759319595992565155029296875f,0.9248999868355689546461917416309006512165069580078125f,1.016900016865134315224850070080719888210296630859375f,-0.720900045967102443000840139575302600860595703125f,-1.0403000342965125213368082768283784389495849609375f,0.73979997296333266376677784137427806854248046875f,0.30089995417594916915504654753021895885467529296875f,1.017599971199036446023455937393009662628173828125f,1.1196999919295311798350667231716215610504150390625f,-1.313099994683266658057618769817054271697998046875f,-1.197100002253055350820432067848742008209228515625f,0.148500009454787029739009085460565984249114990234375f,-0.347099983756989249883417869568802416324615478515625f,0.520199970859289262392621822073124349117279052734375f,0.193000001728534709144469161401502788066864013671875f,0.5327999930202960143788004643283784389495849609375f,0.9465999830156566208216872837510891258716583251953125f,-1.369199971723556785718756145797669887542724609375f,-1.3729999960660936864087489084340631961822509765625f,-0.83700002181529953304561786353588104248046875f,-1.0666000315189361202783402404747903347015380859375f,-0.731699991095066337720709270797669887542724609375f,-0.5187000027775763300041944603435695171356201171875f,-0.159000001311301986106627737171947956085205078125f,-0.63719997822046270385953903314657509326934814453125f,-1.2083999672889706999967529554851353168487548828125f,-0.48240000942945471251732669770717620849609375f,0.7152000169783832017600389008293859660625457763671875f,-0.072399997508525881784180455724708735942840576171875f,-0.4997999931544063745292305611656047403812408447265625f,0.2209999952614307705545115823042578995227813720703125f,0.315300000917911660280879004858434200286865234375f,-0.560599982613325220626165901194326579570770263671875f,0.27819999568462350225672707892954349517822265625f,-0.19139999151825914935898254043422639369964599609375f,-0.0008999998480081483620551807689480483531951904296875f,-0.25839999664425850056659328402020037174224853515625f,1.253500000774860012597855529747903347015380859375f,-0.14839999530315406417457779753021895885467529296875f,0.08640000247061252469649161866982467472553253173828125f,-0.0124000007867813011319668703436036594212055206298828125f,-0.4154000083267685994314888375811278820037841796875f,-0.4409999870657921317018690388067625463008880615234375f,-0.3860999987959861101671776850707828998565673828125f,0.02359998762011539241711943759582936763763427734375f,-1.00450001078844053381544654257595539093017578125f,-0.06639998585581796675114674144424498081207275390625f,0.705499999530613042253435196471400558948516845703125f,0.4013999870851636675439522150554694235324859619140625f,0.030200038212538249382532740128226578235626220703125f,0.166199986684322453811546438373625278472900390625f,-0.6546000001668925705189394648186862468719482421875f,-0.32299998712539679246447121840901672840118408203125f,-0.60440000170320262729006799418129958212375640869140625f,0.0668000022292136730328593330341391265392303466796875f,-1.0136000223398209385550217120908200740814208984375f,-1.2468999901175497058147811912931501865386962890625f,-0.4290000023543836249473315547220408916473388671875f,-0.7676000094205142065817426555440761148929595947265625f,0.55949999275803552478691926808096468448638916015625f,0.5773999855130911296186013714759610593318939208984375f,-1.0676999704122547285578548326157033443450927734375f,-0.6672999762415887659017244004644453525543212890625f,-0.333899998521804519668876309879124164581298828125f,-0.111399986600876133024939917959272861480712890625f,-0.4049999974668025970458984375f,-0.0932999927371740422898938049911521375179290771484375f,-0.631800012660026055755224660970270633697509765625f,-0.9270999873518945832984172739088535308837890625f,-0.03840000450462099923498726639081723988056182861328125f,-0.0410000009611248827923191129229962825775146484375f,-0.721499953851104436353125493042171001434326171875f,-0.31979998279809951355900921043939888477325439453125f,0.082799991679191720095332129858434200286865234375f,0.0638999779582025695390257169492542743682861328125f,0.178200042772292732706773676909506320953369140625f,0.571900002634524895483991713263094425201416015625f,0.5114999606497592221643344601034186780452728271484375f,-0.050500005528330671467074353131465613842010498046875f,0.18629997159242606130646890960633754730224609375f,-0.140000050514936358325712717487476766109466552734375f,-0.432299955677986957880420959554612636566162109375f,-0.0327999928712845445488710538484156131744384765625f,0.33000000856816757988099197973497211933135986328125f,0.53619999528974282565485509621794335544109344482421875f,0.718800018680095575973609811626374721527099609375f,-0.1648000009775161256442288504331372678279876708984375f,-0.46890000843107682015897808014415204524993896484375f,-0.361199994581937799953408330111415125429630279541015625f,-0.503499990753829340661695823655463755130767822265625f,0.0801999826312066732558747617076733149588108062744140625f,-1.29879998828172649183443354559130966663360595703125f,0.05999998643994342018004317651502788066864013671875f,0.94420001851320289887325998279266059398651123046875f,-0.47379998291730895942919232766143977642059326171875f,0.0929999990463254988526387023739516735076904296875f,-0.02220001711249341980192184564657509326934814453125f,0.04880000631213210482428621617145836353302001953125f,-0.2347999939739704444718881859444081783294677734375f,-0.5393000212281944261150101738166995346546173095703125f,0.25060000228285783663295660517178475856781005859375f,-0.202499963492154488875485185417346656322479248046875f,-0.38279999912977213938347631483338773250579833984375f,-0.1767000119566919380531544447876513004302978515625f,0.10099999973177897771847710828296840190887451171875f,-0.083200013305246844907969716587103903293609619140625f,-0.3601999971225857688494897956843487918376922607421875f,0.4634000143766403567724410095252096652984619140625f,-0.0893999966740608442705706693232059478759765625f,0.4060000097155569420692700077779591083526611328125f,0.0483999945580959600732740000239573419094085693359375f,-0.49329999050199990762166635249741375446319580078125f,0.6995999770700933595435344614088535308837890625f,0.74359999790191633195490794605575501918792724609375f,-0.1605999894678593609143035791930742561817169189453125f,-1.4798000248193741157365366234444081783294677734375f,-0.49929999984502781984474495402537286281585693359375f,-0.5927999653786424349988237736397422850131988525390625f,-0.2908999966442584739212406930164434015750885009765625f,0.25589997708648459795455210041836835443973541259765625f,0.29629996823966531227512177792959846556186676025390625f,0.499799989056587179447888047434389591217041015625f,0.90499995917081843543883223901502788066864013671875f,-1.215299976778030810464770183898508548736572265625f,-1.30519999634027517032563991961069405078887939453125f,-0.238500015377997698351464350707828998565673828125f,-0.5550000351667403464261951739899814128875732421875f,-0.49379998224675691886886852444149553775787353515625f,-0.42400000071525578704267900320701301097869873046875f,0.61230001572966585854373988695442676544189453125f,0.5985000102370976460264273555367253720760345458984375f,-0.545400012052059679490412236191332340240478515625f,-0.024300022941827759126454111537896096706390380859375f,0.053100019717216895287492661736905574798583984375f,-0.2877000213742260115168392076157033443450927734375f,-0.7298999718226493182982039797934703528881072998046875f,-0.603199993374943677082455906202085316181182861328125f,-0.21030001552104948814303497783839702606201171875f,-0.1407999978780747341033929842524230480194091796875f,0.05519996832609219694631974562071263790130615234375f,0.03539999327659604233531354111619293689727783203125f,1.4447999494940046982804915387532673776149749755859375f,1.1254999655187132301392693989328108727931976318359375f,-0.47129999483823770134449659963138401508331298828125f,-0.5283999783158304097696600365452468395233154296875f,0.450900013184547532318902085535228252410888671875f,0.08340000245571133064004243351519107818603515625f,0.5363000082492828113345240126363933086395263671875f,0.03820000179409976936995008145458996295928955078125f,0.6621000111937522802918465458787977695465087890625f,0.2268000072956084522957098670303821563720703125f,0.037600031250715648667437562835402786731719970703125f,-0.4080000059008597190768341533839702606201171875f,-0.440000001192093037616359652020037174224853515625f,0.29399999624490735783410855219699442386627197265625f,0.339599978411197156447087763808667659759521484375f,-0.34640001342296589115221649990417063236236572265625f,-1.086000035494566251514925170340575277805328369140625f,0.30720000042319295374682042165659368038177490234375f,-0.0083000124812127040740961092524230480194091796875f,-0.415799997878074645285551014239899814128875732421875f,0.4054000171929599272147015653899870812892913818359375f,0.124399999207258193933967049815692007541656494140625f,0.506500030815601576250628568232059478759765625f,-0.00540000519752492635916496510617434978485107421875f,-1.0820000316500664183649860206060111522674560546875f,0.0186000133991239824382546430570073425769805908203125f,0.70300002229213731652635033242404460906982421875f,-0.6418000083386896648107722285203635692596435546875f,0.7692000042080877619810053147375583648681640625f,0.13099999421834951363052823580801486968994140625f,0.54060002061128642480980488471686840057373046875f,-0.20099999734759332881139926030300557613372802734375f,0.73450002926588098262072890065610408782958984375f,0.21400001204013818778548738919198513031005859375f,0.56780001641511912424675756483338773250579833984375f,-0.6143000230908395309370462200604379177093505859375f,-0.1119000069558619969711799058131873607635498046875f,-0.3627000023007393991747449035756289958953857421875f,0.307799986016750182216128450818359851837158203125f,0.0723000149846075856885363464243710041046142578125f,-0.67800002378225343591111595742404460906982421875f,-0.246800023239850840894860084517858922481536865234375f,0.781799980473518729695570073090493679046630859375f,0.913599993133544785450794734060764312744140625f,-0.8400000064074983896489356993697583675384521484375f,-0.8848000057458875744487158954143524169921875f,0.936600019299984154486082843504846096038818359375f,1.170200019735097907158660746063105762004852294921875f,-0.8258999780178069016756126075051724910736083984375f,-0.633899995243549341239486238919198513031005859375f,0.03239999154806127990013919770717620849609375f,-0.3304000174909831333280862963874824345111846923828125f,-0.023100041174888730211023357696831226348876953125f,0.051299991858005711264922865666449069976806640625f,0.28890005723237965895577872288413345813751220703125f,0.52580001575946777592207581619732081890106201171875f,-1.4355000169277190025240997783839702606201171875f,-1.291000020742416154462262056767940521240234375f,-0.0189000213205812173100639483891427516937255859375f,-0.43139999852180466177742346189916133880615234375f,-0.6170999979317191019845267874188721179962158203125f,-0.809000014424324120909659541212022304534912109375f,0.26939998981952584244936588220298290252685546875f,0.03030001581907271201998810283839702606201171875f,-0.4116000380754467613542146864347159862518310546875f,0.196000007331371239871486977790482342243194580078125f,-0.672700024950504626986003131605684757232666015625f,-0.48540000936985006063650871510617434978485107421875f,0.5484000407516962383169811801053583621978759765625f,0.4056000100314616219066010671667754650115966796875f,0.52660001274347312261170372948981821537017822265625f,-0.14620000307559966046255794935859739780426025390625f,-0.2088000261306763860602586646564304828643798828125f,-0.04799999800324439291898670489899814128875732421875f,0.7612999912619589082396487356163561344146728515625f,-0.654599990332126679959401371888816356658935546875f,-0.608199995200335852274520220817066729068756103515625f,0.0370000012889504237278970322222448885440826416015625f,-0.59199999439716322058302466757595539093017578125f,0.635400001323223051485911128111183643341064453125f,-0.0337999885797497512385234585963189601898193359375f,0.3690000106990336536938457356882281601428985595703125f,-0.87430003024339697503819479607045650482177734375f,0.416400001502037042655501863919198513031005859375f,0.0581999950885769834485472529195249080657958984375f,0.58579998610615735543660775874741375446319580078125f,0.02300001614540836403222101580468006432056427001953125f,0.1875999931037426460278538797865621745586395263671875f,-0.54170002603828937282059996505267918109893798828125f,-0.033600001999735784607992172823287546634674072265625f,0.572700001704692596149470773525536060333251953125f,0.08619999696612357098501888685859739780426025390625f,1.05110000624656674261814259807579219341278076171875f,-0.4168000011861323628181708045303821563720703125f,0.7409000179529190432958785095252096652984619140625f,0.4918000061035154857336237910203635692596435546875f,-0.6710999940276138886474655009806156158447265625f,-0.051100003564357621144154109060764312744140625f,0.56879999223053356871560026775114238262176513671875f,0.0623999909520147610919593716971576213836669921875f,-0.5343000046133994374031317420303821563720703125f,-0.80040001801252369517669649212621152400970458984375f,-0.164999965429306172381984652020037174224853515625f,-0.3045999834775923176266587688587605953216552734375f,-1.496099974060058457325794734060764312744140625f,-1.3970999843716622734746124478988349437713623046875f,0.3578999961264430762497568139224313199520111083984375f,0.731399995020031834513929425156675279140472412109375f,1.380899994707107225622166879475116729736328125f,1.18579999430179583441713475622236728668212890625f,0.73470000697970316849705341155640780925750732421875f,0.55980001289844505407700125942938029766082763671875f,1.10340001318454739021035493351519107818603515625f,1.23670002745389950860044336877763271331787109375f,1.002299971175194315264889155514538288116455078125f,0.4549999850988388772066173260100185871124267578125f,0.26699998672306524127861848683096468448638916015625f,-0.00300002270191901398987965876585803925991058349609375f,0.202200003813207462854961704579181969165802001953125f,0.59350002148747460495314953732304275035858154296875f,-0.0039000190854068250700947828590869903564453125f,-0.24790000191926964134836453013122081756591796875f,-1.1706000012397765885907574556767940521240234375f,-1.38849999570846538432533634477294981479644775390625f,0.4199999739229678397123279864899814128875732421875f,-0.21819999017119418027732535847462713718414306640625f,0.07940000330507768211418806458823382854461669921875f,-0.1299999959021806983372471222537569701671600341796875f,0.5803000000238416333786517498083412647247314453125f,0.182799997490644461350939309340901672840118408203125f,1.45150001651048654593978426419198513031005859375f,0.2030000114142893608004669658839702606201171875f,-1.30860004707574884008636217913590371608734130859375f,0.06020000319480889316992033855058252811431884765625f,1.0272000428915024627940510981716215610504150390625f,0.2231999825716020069421574589796364307403564453125f,-0.7097999884605403764226139173842966556549072265625f,0.43600000047683717951940707280300557613372802734375f,-0.5303000104546544690720111248083412647247314453125f,-0.4455999866366386896743279066868126392364501953125f,0.2694999836683269478498914395458996295928955078125f,-0.5964000085055827238278425284079276025295257568359375f,-0.1667000250697139307476390968076884746551513671875f,-0.4749999977648258209228515625f,-0.17739999855160704100853763520717620849609375f,0.16860000162720678584804545607767067849636077880859375f,-0.20619998698234542899854204733856022357940673828125f,0.2815999881565571794084235079935751855373382568359375f,0.62790001205205925316477078013122081756591796875f,-0.16199999916553498024995860760100185871124267578125f,-0.001099991643428666066029109060764312744140625f,0.24400000354647632949678381919511593878269195556640625f,1.056000024616718491188294137828052043914794921875f,0.1332000125229357845313415964483283460140228271484375f,-0.192300002430379368689727925811894237995147705078125f,-1.106999998718500410888054830138571560382843017578125f,-0.308399987368285888322816390427760779857635498046875f,-0.3220000002086162993464313331060111522674560546875f,0.183599981063604644759834627620875835418701171875f,-0.111699994075298292273146216757595539093017578125f,-0.0327000133275987536762841045856475830078125f,-0.62989998022317905679301475174725055694580078125f,0.519300037097931266316663823090493679046630859375f,0.95640002027750004032213837490417063236236572265625f,0.16199992704391519282580702565610408782958984375f,-0.2496000359296797199704087688587605953216552734375f,1.078799996328353216767936828546226024627685546875f,1.1209999980926514950851924368180334568023681640625f,-0.633599957668781854636108619160950183868408203125f,-0.0792999783873558072855303180404007434844970703125f,-1.23479999948739926907137487432919442653656005859375f,-0.99610000475645055217910339706577360630035400390625f,-0.78449998247623398128780536353588104248046875f,-0.3128999828457832421690909541212022304534912109375f,0.370500002175569331797078120871447026729583740234375f,0.322200001466274255079724753159098327159881591796875f,0.54659997698068618632305515347979962825775146484375f,0.3547999762415887659017244004644453525543212890625f,-0.56100000137090688667740323580801486968994140625f,-0.792700007665157357905627577565610408782958984375f,0.490799995568394376022069991449825465679168701171875f,0.387900003111362412511198272113688290119171142578125f,0.02039998761415429129328913404606282711029052734375f,-0.27819998137950907590720817097462713718414306640625f,0.1173000030077993027788352264906279742717742919921875f,0.534999996870756167055560581502504646778106689453125f,0.058399989342689195837010629475116729736328125f,-0.0640000191926954897780888131819665431976318359375f,0.53570002429485352735127889900468289852142333984375f,-0.336599997991323451884682071977294981479644775390625f,1.1578000339984892974598551518283784389495849609375f,0.28079998640418057931356088374741375446319580078125f,0.5197000384211545309653956792317330837249755859375f,0.6699999980628490447998046875f,0.23870001052618050607634359039366245269775390625f,0.61580000979900351154583404422737658023834228515625f,0.839299992841481934391367758507840335369110107421875f,-0.2769999890327454750149627216160297393798828125f,0.32920000182390207754679067875258624553680419921875f,-0.19619999637007712323111263685859739780426025390625f,-0.67550001424551009421293201739899814128875732421875f,-0.16059999676942826596359736868180334568023681640625f,-0.767700002002715820026423898525536060333251953125f,-0.23859999313354496308647867408581078052520751953125f,0.22279999633580438445079607845400460064411163330078125f,-0.117399995273351709368370165975647978484630584716796875f,-0.4985999958157538713976464350707828998565673828125f,0.1789999844729901479212230697157792747020721435546875f,-0.2217999953746794261633112910203635692596435546875f,-0.21339998919367797469703873503021895885467529296875f,-0.4627000002145766899275258765555918216705322265625f,0.404200001972913725012404029257595539093017578125f,0.883099996471404580233865999616682529449462890625f,0.0802000011086463349130326605518348515033721923828125f,-0.398900017893314640105018042959272861480712890625f,0.764299997460842206464803894050419330596923828125f,-0.302400014197825584005840937606990337371826171875f,-0.57200002837181074255568091757595539093017578125f,-0.8109000030517581336653165635652840137481689453125f,-0.80690001187324522646804325631819665431976318359375f,0.1649999389052396736587979830801486968994140625f,-0.4464000342845917401746191899292171001434326171875f,0.9947999759435646183192147873342037200927734375f,0.1844999786019325682673297706060111522674560546875f,1.073100001835822325801927945576608180999755859375f,0.565100016200542398792094900272786617279052734375f,-0.79889998332262024405281408689916133880615234375f,-0.954699981945753339829252581694163382053375244140625f,-0.356700006294250471228224341757595539093017578125f,-0.14599998468160624298661787179298698902130126953125f,-0.0710999786794186405813888995908200740814208984375f,0.31730000753402709534611858543939888477325439453125f,-0.1760999751031402382750457036308944225311279296875f,0.31990000048875799620873294770717620849609375f,-0.2714999932646751101827931051957421004772186279296875f,-0.24339999049752958359960075540584512054920196533203125f,0.39449998396635077568816996063105762004852294921875f,0.28779996843338029322012516786344349384307861328125f,-0.184499991416931408139134873636066913604736328125f,0.2974000127077101041095374966971576213836669921875f,1.0298999884188173581378578091971576213836669921875f,1.1579000114560127343565909541212022304534912109375f,-0.04230002273321087358226577634923160076141357421875f,-0.27289998821020144958993114414624869823455810546875f,-0.2960999883651727060396297019906342029571533203125f,0.7493999930977821577471331693232059478759765625f,0.14529998437762259300143341533839702606201171875f,-0.63519998352527629048580593007500283420085906982421875f,0.6184000119924544236482688575051724910736083984375f,0.29719999595284463111966033466160297393798828125f,-0.307100008809566560330495121888816356658935546875f,-0.0169999930560589262995563331060111522674560546875f,-0.498499996528029232223389044520445168018341064453125f,-0.166399993529915857237710952176712453365325927734375f,0.57240002015829105630473350174725055694580078125f,-0.5798000057458876366212052744231186807155609130859375f,0.0734000358343127601301603135652840137481689453125f,0.659399984306097142194857951835729181766510009765625f,-0.9672000309705737208787468262016773223876953125f,-0.30300000441074370627347889239899814128875732421875f,0.38510001530647297585119304130785167217254638671875f,-0.34619999860525130230826107435859739780426025390625f,-0.01890002623796505076825269497931003570556640625f,-0.651399994796514558714761733426712453365325927734375f,0.4181000036239623796063824556767940521240234375f,0.1212000024795532038268675023573450744152069091796875f,-0.560600000494718475607669461169280111789703369140625f,0.189199993181228709460128811770118772983551025390625f,0.15979997475147200702849659137427806854248046875f,0.51339999753832810114317908301018178462982177734375f,-0.61040002397298831482430614414624869823455810546875f,0.018600000435113894781125054578296840190887451171875f,1.26550003063678762060817462042905390262603759765625f,-0.2545999974846839819520027958787977695465087890625f,1.1826000290155409544468056992627680301666259765625f,1.3358999795079231631689253845252096652984619140625f,-1.252799951893091456867068700375966727733612060546875f,-0.914499974727630782211917903623543679714202880859375f,0.403199969160557003533540409989655017852783203125f,0.0481999968767166109273603069595992565155029296875f,0.192900006091594633517161128111183643341064453125f,0.545100020372867533069438650272786617279052734375f,-0.100799985994398877409139458904974162578582763671875f,0.290899999773502226929622338502667844295501708984375f,-1.25910001274347305155743015347979962825775146484375f,-1.07340001497268655583638974349014461040496826171875f,1.1978999473810194587031219271011650562286376953125f,0.706599962973594841741942218504846096038818359375f,-0.311099975848197374261872028000652790069580078125f,0.165399995064735350069895503111183643341064453125f,-0.794699998188018952305355924181640148162841796875f,-0.75170001046657564103270487976260483264923095703125f,-1.1546999779224389470755340880714356899261474609375f,-0.6086999837040902860962887643836438655853271484375f,0.10619999473094932085359687334857881069183349609375f,-0.11089999202489853047381984652020037174224853515625f,0.5465999909877776730127152404747903347015380859375f,0.656999993056059050644535091123543679714202880859375f,0.771300005269051069944907794706523418426513671875f,0.122200006532668936642949120141565799713134765625f,0.2043000192165376205366555950604379177093505859375f,0.37880001033544541400033267564140260219573974609375f,-0.8721000174522401948706828989088535308837890625f,-1.16770000677108765074763141456060111522674560546875f,0.23459994711875964412683970294892787933349609375f,-0.1754000005781650439740104729935410432517528533935546875f,0.564199994969367679686911287717521190643310546875f,-0.315000005066394805908203125f,0.2802999916791915069325114018283784389495849609375f,-0.7341999891579151782394774272688664495944976806640625f,0.66549998325109438468416556133888661861419677734375f,-0.0545999918222428437530879818950779736042022705078125f,0.291700000780820811741023135255090892314910888671875f,-0.1705999955773354059829216566868126392364501953125f,-0.1841000025361776426535698192310519516468048095703125f,0.1004000004291534242639727381174452602863311767578125f,-0.228400001747905978533026427612639963626861572265625f,-0.2407999917685985646897961487411521375179290771484375f,0.68079999907016741644838475622236728668212890625f,-0.0597999969542026832414194359444081783294677734375f,0.26410002257823972371397758251987397670745849609375f,-0.19139999911785121611274007591418921947479248046875f,0.497700020480156268831706256605684757232666015625f,0.395400010412931379732981440611183643341064453125f,-0.59430001891851436113256568205542862415313720703125f,0.4472000045955180080881063986453227698802947998046875f,0.7334000155687334654430742375552654266357421875f,-0.651199996966123517694313704851083457469940185546875f,0.43670000383555862555340354447253048419952392578125f,0.3919999895542860546271413113572634756565093994140625f,-0.2992000289440159832565768738277256488800048828125f,-0.62579999638795857919149057124741375446319580078125f,-0.086099985533952594352058440563268959522247314453125f,-0.23720000012516972986986729665659368038177490234375f,-1.15850005372986242235811005230061709880828857421875f,-0.21980000075399874770454289318877272307872772216796875f,-0.853500002562999071642479975707828998565673828125f,-1.005600013160705685777429607696831226348876953125f,-1.42049997252225868038522094138897955417633056640625f,-0.95989998499155060329712796374224126338958740234375f,-0.27809999319314915311451841262169182300567626953125f,-0.4296999755382540087111920001916587352752685546875f,-0.417299993228912313725231797434389591217041015625f,-0.422699997681379446845539860078133642673492431640625f,0.24300000202655791525785389239899814128875732421875f,0.2392999997705220760479960517841391265392303466796875f,-0.41579997094422627146315107893315143883228302001953125f,-0.23539999260604382858019789637182839214801788330078125f,-0.365699997770786211503946105949580669403076171875f,-0.648799992156028793033328838646411895751953125f,-0.356100013995170883163154940120875835418701171875f,-0.17550000500679008297311156638897955417633056640625f,0.6374999944865695766793578513897955417633056640625f,0.283900016105175012626204988919198513031005859375f,0.94980002094507209875473563442938029766082763671875f,0.7405000073909757674073262023739516735076904296875f,-1.4669999920129779269473146996460855007171630859375f,-1.3351999990224836523111662245355546474456787109375f,0.621599962191284216572739751427434384822845458984375f,0.242099992567300859036549809388816356658935546875f,-0.984899981713294181417950312606990337371826171875f,-0.2921999874591829637893170001916587352752685546875f,-0.267600016796588846546001150272786617279052734375f,0.187099994355440113480426589376293122768402099609375f,0.787200038234889820643047642079181969165802001953125f,0.491399993306398386039290926419198513031005859375f,-0.38840001229047782516090592253021895885467529296875f,-0.256399997627735132255111238919198513031005859375f,-0.87179999887645198697327941772527992725372314453125f,-0.491599985101819125787869779742322862148284912109375f,0.692399988642334651700593894929625093936920166015625f,-0.279199987220764278550433346026693470776081085205078125f,-0.1053000092625619998898400808684527873992919921875f,-0.468800010782480125470783605123870074748992919921875f,-0.061399998670816469115152358426712453365325927734375f,0.016800000292062744478016611537896096706390380859375f,-0.6295000125765799481314388685859739780426025390625f,-0.20859999194145206757866617408581078052520751953125f,-0.1482000314474110069795642630197107791900634765625f,-0.10740000466108323706748706172220408916473388671875f,0.899100025260448632025145343504846096038818359375f,-0.26400000250339505925012417719699442386627197265625f,0.389700020856410478398146324252593331038951873779296875f,-0.30120000732243046304148492708918638527393341064453125f,-0.105800003764033423436785597004927694797515869140625f,0.228199996280670169834792204710538499057292938232421875f,0.4960000350475315400444742408581078052520751953125f,0.177199999976158206749232704169116914272308349609375f,1.10229999680519075155871178139932453632354736328125f,-0.07259999310374265490963807678781449794769287109375f,-0.082100019687414427238536518416367471218109130859375f,0.426600003057718168975043226964771747589111328125f,-0.328099987512081636698013653585803695023059844970703125f,-0.11659999881088733975076365823042578995227813720703125f,0.04950000393390663333548218361102044582366943359375f,0.1220000009536743590388141456060111522674560546875f,0.00210000227168194442128879018127918243408203125f,0.785699978976696744581431630649603903293609619140625f,-0.22629999744594098132211001939140260219573974609375f,0.1429999956190586285487142959027551114559173583984375f,-0.143799998041987453500922811144846491515636444091796875f,0.228399988076090798561068595518008805811405181884765625f,0.207700004535913418379777795053087174892425537109375f,0.6936999921977520244809056748636066913604736328125f,-0.33129999237954610435963331838138401508331298828125f,0.31680001906752597307104224455542862415313720703125f,0.155899996197223611460458414512686431407928466796875f,-0.65710001313090327101917864638380706310272216796875f,0.23470000608563423849517448616097681224346160888671875f,0.09950000840425499148977905861102044582366943359375f,0.048700002032518341366085223853588104248046875f,0.6032999883413314723412668172386474907398223876953125f,-0.18450001190602771661275482983910478651523590087890625f,-0.62619998906850826703163193087675608694553375244140625f,0.0021000047862529402920017673750407993793487548828125f,0.3716999912440777098510125142638571560382843017578125f,0.149900002053379999278348577718134038150310516357421875f,0.464300000217556962933684872041339986026287078857421875f,-0.2871000130563974295228035771287977695465087890625f,-0.7138000064611436101813524146564304828643798828125f,-0.081199996593594558902395874611102044582366943359375f,0.73899999825656415008978683545137755572795867919921875f,-0.2210000035315752009434930869247182272374629974365234375f,-0.1603000063195825219963808194734156131744384765625f,0.138000001952052198106457581161521375179290771484375f,0.93240003356933609524048733874224126338958740234375f,-0.264100013339518824295737431384623050689697265625f,-1.1004000144362446889090279000811278820037841796875f,-0.679000000566244210631339228712022304534912109375f,0.09380001927614234347174715367145836353302001953125f,-0.2522999675989154866329045034945011138916015625f,0.47049999785423235465486868633888661861419677734375f,0.586399988090991808320495692896656692028045654296875f,0.57760000144839285241005200077779591083526611328125f,-1.20889997497797008207953695091418921947479248046875f,-0.170199957895278419783835488487966358661651611328125f,-0.22449999350309379764212280861102044582366943359375f,0.42290002218484890050831381813623011112213134765625f,0.7102999942682688416795144803472794592380523681640625f,-0.47079999394789295852348232074291445314884185791015625f,0.3144999774098398148680644226260483264923095703125f,0.032499993592500775463349782512523233890533447265625f,-0.05579999400377300133868629927746951580047607421875f,-0.748400032258034197951701571582816541194915771484375f,-0.533499990642070542889996431767940521240234375f,-0.03099998274445514567787540727294981479644775390625f,0.6201999751806255289920954965054988861083984375f,-0.1136000199556352896479438641108572483062744140625f,-0.9437999811291692964232424856163561344146728515625f,-0.3052999974906445146416444913484156131744384765625f,0.006699971723557407443649935885332524776458740234375f,1.0327000106453898542469005406019277870655059814453125f,-0.30960001313090312891063149436376988887786865234375f,0.28630000169277192156869205064140260219573974609375f,-0.24400003290176375259079577517695724964141845703125f,0.5187000027775763300041944603435695171356201171875f,0.01410000797510146952618015347979962825775146484375f,0.33130000526905067914640312665142118930816650390625f,0.04940000032484537229038323857821524143218994140625f,0.6797000160694122161686436811578460037708282470703125f,0.232299995017051674750518941436894237995147705078125f,-0.2587000047147274273129369248636066913604736328125f,-0.0520000042319297417492407475947402417659759521484375f,-0.5975999923586845508083342792815528810024261474609375f,0.3181000018358230985171530846855603158473968505859375f,0.69930001206398006985409665503539144992828369140625f,-0.08109999805092815705620523658581078052520751953125f,0.42889998496174808195746663841418921947479248046875f,-0.005899996178597266549559208215214312076568603515625f,-0.79780001650825138614919751489651389420032501220703125f,-0.1544999981969594760045083603472448885440826416015625f,-0.016999994993209810445478069595992565155029296875f,-0.0410000025629997377762947508017532527446746826171875f,0.195799999070167540793363514239899814128875732421875f,0.004300005730986544705274354782886803150177001953125f,0.6181999859988689394185712444595992565155029296875f,-0.31860000445842739935642384807579219341278076171875f,0.007000004529952974507978069595992565155029296875f,0.109800005224347074772595078684389591217041015625f,0.87169999288320543229247050476260483264923095703125f,-0.07709998921453953268922987263067625463008880615234375f,0.11860000132918369075696318759582936763763427734375f,-0.125199996563792215198418489308096468448638916015625f,0.6365000041425228349822873497032560408115386962890625f,-0.180399997076392093475760702858678996562957763671875f,-0.153399999922514407302287509082816541194915771484375f,-0.11959999450445213398097621393389999866485595703125f,0.675200004088878369401527379523031413555145263671875f,-0.752199994909762637007588637061417102813720703125f,-0.34210001178979876357288958388380706310272216796875f,0.34589997116327264592428036849014461040496826171875f,0.31859998598098737687678294605575501918792724609375f,-0.12649997568130544323139474727213382720947265625f,-0.755099995189905115466899587772786617279052734375f,0.0806000125646590959149762056767940521240234375f,-0.82760001620054257642777884029783308506011962890625f,0.1392999877750877857351952116005122661590576171875f,0.6893000220477585315848045866005122661590576171875f,0.339699981535971196056067356039420701563358306884765625f,0.55900000252202153294689424001262523233890533447265625f,0.0554000054955476883833398460410535335540771484375f,0.49610002144575116034275197307579219341278076171875f,0.7558000115871426061175952781923115253448486328125f,0.05019999976754174753068582504056394100189208984375f,-0.2217999879240988292394831660203635692596435546875f,0.01809998395442935503751868964172899723052978515625f,-0.081299996924400375064578838646411895751953125f,0.8597999979972836825936610694043338298797607421875f,-0.40909997370243100789366508251987397670745849609375f,-0.4150999787986275890716569847427308559417724609375f,-0.30290000162124641747141140513122081756591796875f,-0.6406000019848345772288666921667754650115966796875f,-0.414299968850613264947924108128063380718231201171875f,-0.534899978435039447077770091709680855274200439453125f,-0.730600009286403118125008404604159295558929443359375f,0.1538999930083750911080642254091799259185791015625f,-0.0596000029981136325574198053800500929355621337890625f,0.244199997651576961033015322755090892314910888671875f,0.123800010037422136832674368633888661861419677734375f,0.61639999598860750751327941543422639369964599609375f,-0.275100002193450876575298025272786617279052734375f,0.208500012099742892957010553800500929355621337890625f,0.264899991324544004100971505977213382720947265625f,-0.42159998621940619756287560448981821537017822265625f,0.065799999219179117204703288734890520572662353515625f,-0.20520000826120377013239703956060111522674560546875f,0.123900002396106767577066420926712453365325927734375f,-0.4808999863624572324027894865139387547969818115234375f,-0.2344999983832240231240717776017845608294010162353515625f,0.1110999933198094369313224660800187848508358001708984375f,-0.215800003242492710597844052244909107685089111328125f,0.40519998248219479819454136304557323455810546875f,-0.063800004747510019598166763898916542530059814453125f,-0.7439000225126743970349707524292171001434326171875f,0.197400000339746473088098355219699442386627197265625f,-0.192599995189905115466899587772786617279052734375f,0.083299998772144334679978783242404460906982421875f,-0.137099993610382053788043776876293122768402099609375f,0.361099994102120458450144724338315427303314208984375f,-0.326599983239173941029775960487313568592071533203125f,-0.009500000283122034261396038345992565155029296875f,0.408600008481740939458859429578296840190887451171875f,0.082299994644522644904327535186894237995147705078125f,0.3239999893903731109645605101832188665866851806640625f,0.1090000066012144497307190249557606875896453857421875f,-0.2005000127926470643302536700502969324588775634765625f,0.1834999718666079360218645888380706310272216796875f,1.04290000102519986313609479111619293689727783203125f,0.03600000002980241475825096131302416324615478515625f,0.2175000041723251786862647350062616169452667236328125f,0.0250000120699400696366865304298698902130126953125f,0.80219999625086746419810879160650074481964111328125f,0.1274999618530276990213678800500929355621337890625f,0.71749998912215229296407414949499070644378662109375f,0.81000000208616196317734647891484200954437255859375f,0.284500017791986525850234102108515799045562744140625f,-0.28639999256134007765695059788413345813751220703125f,-0.648000009328126846952500272891484200954437255859375f,-0.01260000368356717359574759029783308506011962890625f,-0.811700001376867152913519021240063011646270751953125f,-0.03099999570846545537960992078296840190887451171875f,-0.865600027465820431871179607696831226348876953125f,-0.23509999325275376946819960721768438816070556640625f,0.064800000977516258870991805451922118663787841796875f,-0.0482999785065647557757984031923115253448486328125f,-0.92520000349283204599259988754056394100189208984375f,0.78949996191263238642932265065610408782958984375f,0.189799979221820791508434922434389591217041015625f,-0.64669998662471783745786524377763271331787109375f,0.65829999936819039163538036518730223178863525390625f,-0.26110000326633453493485603758017532527446746826171875f,-0.1317000003710389055555651793838478624820709228515625f,-0.25799998358637121764758148856344632804393768310546875f,0.477499994114040926529440866943332366645336151123046875f,0.677999970138073226877395427436567842960357666015625f,0.413400001971423625146684344144887290894985198974609375f,-0.234699995505809777540662253159098327159881591796875f,0.27150000086426739898115556570701301097869873046875f,0.02840000074207782443380665426957421004772186279296875f,-0.117900000727176657022710060118697583675384521484375f,0.184599991524219575467213871888816356658935546875f,-0.4911999820649624037827152278623543679714202880859375f,-0.237899995288252785741178740863688290119171142578125f,0.36040001384019859642648953013122081756591796875f,-0.2104000067621469849399318263749592006206512451171875f,-0.483100019717216611070398357696831226348876953125f,0.2088000013202429006042848413926549255847930908203125f,-0.1104999896585940888371624168939888477325439453125f,-0.1062000004678964604210023026098497211933135986328125f,0.29749999850988384508809758699499070644378662109375f,0.266999997228384078340468477108515799045562744140625f,-0.533199976313114110126889499952085316181182861328125f,0.1907000041782855870309276724583469331264495849609375f,-0.163899998819827974472218556911684572696685791015625f,0.210300003004074032020298545830883085727691650390625f,-0.38010001471042631493446606327779591083526611328125f,-0.3586999950289726513119603623636066913604736328125f,-0.1147000080227851270819883211515843868255615234375f,0.003500001817941633674990953295491635799407958984375f,0.71949999275803566689546642010100185871124267578125f,-0.052499998658895485892372789749060757458209991455078125f,0.433300002199411393721817375990212894976139068603515625f,0.0687999977067112744588683881374890916049480438232421875f,0.122199996101856223074122453908785246312618255615234375f,-0.21839999813437460662868261351832188665866851806640625f,-1.06630002329945572370206718915142118930816650390625f,0.72959999688863685918249757378362119197845458984375f,1.1113000113785265199339846731163561344146728515625f,1.1376999710083008920946667785756289958953857421875f,0.38010001336932219828668166883289813995361328125f,0.394099991434812391144504317708197049796581268310546875f,-0.0337000093899668395902580186884733848273754119873046875f,0.11940002319812759168371485429815948009490966796875f,-0.66319998525381063103623091592453420162200927734375f,-0.15799999308586121227193643790087662637233734130859375f,-1.307300003845989433415297753526829183101654052734375f,-0.213899995318055147208724520169198513031005859375f,0.069800002840161301520538472686894237995147705078125f,-0.5390999885104597044716001619235612452030181884765625f,0.54930001185908905458887829809100367128849029541015625f,0.761499966442585218828753568232059478759765625f,-0.3776000035345550287502192077226936817169189453125f,0.167400001382827401386066412669606506824493408203125f,0.9982000448584560370335339030134491622447967529296875f,-0.1209999963790178156841648160479962825775146484375f,0.317400000563263784414402834954671561717987060546875f,-0.009600002998113532637347589115961454808712005615234375f,0.31830000292584303966947345543303526937961578369140625f,0.0472999634265907076269286335445940494537353515625f,0.70609999790191668722627582610584795475006103515625f,0.05769999797940261743178780307061970233917236328125f,0.470200012880563900097286023083142936229705810546875f,0.63849999511241950500561870285309851169586181640625f,0.12159999665021903325623497948981821537017822265625f,-0.3576999982774251574113577589741908013820648193359375f,0.75210002311468127089710833388380706310272216796875f,-0.078800010111927942801912649883888661861419677734375f,-0.730299994957447129451111322850920259952545166015625f,-0.28809999997317792264084346243180334568023681640625f,-0.69460001089572909194203020888380706310272216796875f,-0.134100002014637009750686047482304275035858154296875f,-0.1540000025965273522388088167645037174224853515625f,0.18219999669790265528490635915659368038177490234375f,-0.525400008773803772754718011128716170787811279296875f,0.0647999996364116703784219453154946677386760711669921875f,0.1006000018730759393292828463017940521240234375f,0.02550000083446501975004139239899814128875732421875f,0.18939999853670597129706720807007513940334320068359375f,0.067900001379102459253545021056197583675384521484375f,0.45089999807849523438818550857831723988056182861328125f,-0.09550000120699397854195922263897955417633056640625f,-0.65719997956156728236010167165659368038177490234375f,-0.02260000420510765994919211152591742575168609619140625f,-0.1932999999642373012420648592524230480194091796875f,0.003400000965595229018845202517695724964141845703125f,0.11889999591410160828530706567107699811458587646484375f,0.00040000117421150338525848155768471769988536834716796875f,-0.00509999934360383377907055546529591083526611328125f,-0.179299990606307968477040049037896096706390380859375f,0.320900006777048218964409898035228252410888671875f,0.0986999992012977944710883093648590147495269775390625f,-0.3166999906480312265699694762588478624820709228515625f,-0.05729999650716788295312653644941747188568115234375f,-0.624500019133090944478681194595992565155029296875f,0.31370000382065776900475384536548517644405364990234375f,0.77350003042817139231601686333306133747100830078125f,0.602599971199035966407109299325384199619293212890625f,0.313299977463483703132851587724871933460235595703125f,0.4003000003218648572556048748083412647247314453125f,0.8214000156581402478650488774292171001434326171875f,0.721699979174137862258930908865295350551605224609375f,0.48449998039007180494763815659098327159881591796875f,0.2260000028610225086822538287378847599029541015625f,0.86450000646710378759962623007595539093017578125f,0.0238999746799475332181827980093657970428466796875f,0.9889000012040138898328223149292171001434326171875f,-0.18779997797012359939117231988348066806793212890625f,0.411799992841481721228547030477784574031829833984375f,0.02709998198747687325749211595393717288970947265625f,1.129200030434132084877774104825221002101898193359375f,-0.27520001273155170196105245850048959255218505859375f,0.567600013071298814537612997810356318950653076171875f,0.1539999876022335456582368351519107818603515625f,-0.09930000848770159649347988306544721126556396484375f,-0.024200036394595780819827268715016543865203857421875f,0.4079999853372573426213421043939888477325439453125f,-0.3202999562144288603349195909686386585235595703125f,0.04220001860260946813241389463655650615692138671875f,0.726300010037422172359811156638897955417633056640625f,0.36120000665187834698599544935859739780426025390625f,0.4783999537289147507834741190890781581401824951171875f,0.643299983945488929037992420489899814128875732421875f,0.5488000061631197468159371055662631988525390625f,0.73530002416372308715608596685342490673065185546875f,-0.55049998265504829220162719138897955417633056640625f,0.574699984180927270216443503159098327159881591796875f,-0.11179999865293499539120603003539144992828369140625f,-0.4667999886691569511043553575291298329830169677734375f,0.077699994775652925227404921315610408782958984375f,-0.687999992042779950907060992904007434844970703125f,-0.0752999930948019535748017005971632897853851318359375f,-0.6986000107169150208363817000645212829113006591796875f,0.07599999265372758827652432955801486968994140625f,-0.481499986410141023185360609204508364200592041015625f,-0.1278999916374683465392081416212022304534912109375f,0.2168999998033047660328520578332245349884033203125f,0.1005999969929456394623912274255417287349700927734375f,-0.02990000779032708777549487422220408916473388671875f,0.0125000055134296683689854035037569701671600341796875f,-0.11529999346733088572136693983338773250579833984375f,-0.0359999938458204216118474505492486059665679931640625f,0.3781000025808811670913200941868126392364501953125f,-0.304800001275539411693671354441903531551361083984375f,0.522199992376565891305517652654089033603668212890625f,-0.2093999957054852867344152400619350373744964599609375f,0.4883000182926655607928978497511707246303558349609375f,0.336699987593293226240120930015109479427337646484375f,-0.39719999267458927505458632367663085460662841796875f,-0.0107000011235476311100001112208701670169830322265625f,-0.8460000250637531582498240823042578995227813720703125f,-0.31559999416172501707222863842616789042949676513671875f,0.12990000838637361102456679873284883797168731689453125f,0.006799992096424123388942462042905390262603759765625f,-0.6958000013053418086883539217524230480194091796875f,0.3148999992966652339276834027259610593318939208984375f,-0.16319999434351917244612195645458996295928955078125f,0.071799985837936564081473989062942564487457275390625f,-0.2865999972462651701476943344459868967533111572265625f,0.98389997169971454837877899990417063236236572265625f,0.409900002425909182335317382239736616611480712890625f,-0.9834999770820136433258085162378847599029541015625f,-0.26699998871609553763306621476658619940280914306640625f,0.4637000254273413446526319603435695171356201171875f,-0.586600028687715990827200585044920444488525390625f,-0.1814999926686287068378078402020037174224853515625f,-0.61590002778768582203383630258031189441680908203125f,-0.8690999659538274357828413485549390316009521484375f,-0.0772999637246130077983252704143524169921875f,0.8626999829292305577155275386758148670196533203125f,0.32829997388720499884584569372236728668212890625f,0.2264999721236529151013883165433071553707122802734375f,-0.66359999982044060029551246771006844937801361083984375f,-0.4725999779045582105396761107840575277805328369140625f,-0.817600000777840563159770681522786617279052734375f,-0.51259998334348200188514965702779591083526611328125f,0.613400015345215710027559907757677137851715087890625f,1.094399973428249239759679767303168773651123046875f,0.29229997206926316977160240639932453632354736328125f,0.8322000023603433049856903380714356899261474609375f,-0.10580000413656254210081897326745092868804931640625f,0.50249998897314096524269189103506505489349365234375f,0.497999996811151479647605810896493494510650634765625f,0.2747999816060069377954278024844825267791748046875f,-0.7609999992847438221588163287378847599029541015625f,0.68829998744726150761152894119732081890106201171875f,0.5723999912500381537228122397209517657756805419921875f,-0.08219999811351297314132580140721984207630157470703125f,0.1179000048995018090636222041212022304534912109375f,-0.2703000027805566585215046870871447026729583740234375f,-0.5562000003933906810971166123636066913604736328125f,0.358300004583597175855658178988960571587085723876953125f,0.34720000830218200604804223985411226749420166015625f,0.10149999736249437631840919493697583675384521484375f,-0.748600023680925374947037198580801486968994140625f,-0.147299996805191124593648055451922118663787841796875f,0.580499978631734858680601973901502788066864013671875f,0.18269999619126320311579547706060111522674560546875f,0.6058999978363512894929954200051724910736083984375f,-0.05949999253451832004202515236102044582366943359375f,-0.89280001760721194870029648882336914539337158203125f,-0.1580999980546533567604683412355370819568634033203125f,0.1284000126071274394679022634591092355549335479736328125f,0.24459999897480011465944471638067625463008880615234375f,0.03440000457167624592358379231882281601428985595703125f,0.0358000031679868158107638009823858737945556640625f,0.49019998479187487649966215030872263014316558837890625f,-0.214800004255771614936065816436894237995147705078125f,-0.8073999955713748732932799612171947956085205078125f,-0.219400007253885309665975000825710594654083251953125f,-0.46960001864433298379708503489382565021514892578125f,-0.1028999975234270447543849513749592006206512451171875f,-0.37860001056790348883396291057579219341278076171875f,-0.01639999837279315642035726341418921947479248046875f,-0.248199988234043100732151287957094609737396240234375f,-0.098300005477666818620718913734890520572662353515625f,0.2563000288128858272784782457165420055389404296875f,0.5019999907463787991446224623359739780426025390625f,-0.4146000234872104783789836801588535308837890625f,-0.357199992302060120863416159409098327159881591796875f,0.003599999019503474784187346813268959522247314453125f,0.48649998939037342182700740522705018520355224609375f,0.65489998931288706529585397220216691493988037109375f,-0.07399998104572347301655099727213382720947265625f,0.16900000384449942458786608767695724964141845703125f,0.0481000081688165437299176119267940521240234375f,0.426600007155537586100990665727294981479644775390625f,0.365700009393691516379476524889469146728515625f,-0.58550000321865081076566639239899814128875732421875f,-0.003199996280670192039252697213669307529926300048828125f,-0.03089999893531203412067043245770037174224853515625f,0.5163999648451811363969454760081134736537933349609375f,0.89079998945891858408430152849177829921245574951171875f,0.1512999954342841846255396376363933086395263671875f,0.163800007653236434634891338646411895751953125f,-0.132300011408329254436466726474463939666748046875f,0.74899998372793152867643584613688290119171142578125f,-0.243699983555078769636992319647106342017650604248046875f,-1.0557999873913821442528160332585684955120086669921875f,0.195200007814168685627009836025536060333251953125f,-0.33850002223253294886262665386311709880828857421875f,0.3355999699473386499448679387569427490234375f,0.87389998168349247720243511139415204524993896484375f,-0.285999994851648953186895596445538103580474853515625f,0.3126999983891842749272882429067976772785186767578125f,-0.71679999746084188672057280200533568859100341796875f,-0.60350000271201142965082908631302416324615478515625f,-0.1156000022083520928362077029305510222911834716796875f,-0.30460000113546847888557067562942393124103546142578125f,-0.0067999979078769978713125965441577136516571044921875f,0.43489999281466007463592404747032560408115386962890625f,0.065499998971819906046221149154007434844970703125f,-0.753199990022182408466733249952085316181182861328125f,0.213199991735816052340624082717113196849822998046875f,-0.40969998641610150258429712266661226749420166015625f,0.108299998697638510947172107989899814128875732421875f,-0.32530000024735927777186361709027551114559173583984375f,0.134299990606308039531313625047914683818817138671875f,-0.70869998623728758246897996286861598491668701171875f,0.0283999995499849278157711296444176696240901947021484375f,-0.0598000012010335979528719008158077485859394073486328125f,-0.184399993470311163168418033819762058556079864501953125f,-0.2272999977737664156762775746756233274936676025390625f,-0.19649999788403516021162431570701301097869873046875f,0.264499982476234407613446819595992565155029296875f,-0.109700002062320667306494215154089033603668212890625f,0.37030001283884050877759364084340631961822509765625f,0.09929999701380730348088121672844863496720790863037109375f,0.1099999999441205744687977130524814128875732421875f,-0.20599999995529660878901268006302416324615478515625f,0.2590999729573726373388353749760426580905914306640625f,-0.187999993458390246559019942651502788066864013671875f,-0.0059000000715254685701438575051724910736083984375f,0.0071999994173645998074562157853506505489349365234375f,0.304200001339614389817000983384787105023860931396484375f,0.1284000039458275121972974375239573419094085693359375f,-0.17470000012516972986986729665659368038177490234375f,0.53249997042119534551574133729445748031139373779296875f,0.7853999944314360437402910974924452602863311767578125f,-0.049099990503490065218983318118262104690074920654296875f,0.18050001284852623673060634246212430298328399658203125f,-0.335199981737137164827800006605684757232666015625f,0.75590000871419871231182696647010743618011474609375f,0.921199982213973811440155259333550930023193359375f,0.03789996153712227311416427255608141422271728515625f,0.6155999928951256805476077715866267681121826171875f,-0.144599996143579545559987309388816356658935546875f,0.4275999834179879943718560753040947020053863525390625f,0.27919998476207241733249020398943684995174407958984375f,-0.35250000841915607452392578125f,-0.023299995121359717842324243974871933460235595703125f,0.0489000260889529414498611004091799259185791015625f,0.75869999130368181994299447978846728801727294921875f,0.4776999793529501658895242144353687763214111328125f,-0.1605000197589401267350694979541003704071044921875f,0.68979996864199666362083007697947323322296142578125f,0.51739999901726851927463712854660116136074066162109375f,-0.1191999773859979683265919447876513004302978515625f,-0.10630000184178367561571576516143977642059326171875f,-0.265199966835976130852259302628226578235626220703125f,-0.27409998905062682439393029198981821537017822265625f,-0.90549996604025395630088723919470794498920440673828125f,-0.6957999821200966383827335448586381971836090087890625f,0.172200001168250960148498052149079740047454833984375f,-0.21580000562667844832276387023739516735076904296875f,-0.35440000501871082860816386528313159942626953125f,-0.5087000059068202961753968338598497211933135986328125f,-0.249699996027350434868452566661289893090724945068359375f,-0.091500003769993731861376318192924372851848602294921875f,-0.1268999979034066971639305165808764286339282989501953125f,0.3395999748721718614064002395025454461574554443359375f,-0.116399999266862952396195396431721746921539306640625f,0.623599973911046934205160141573287546634674072265625f,-0.112800005611777276470775177585892379283905029296875f,-0.825299991083145112469310333835892379283905029296875f,-0.35640000559985640649074412067420780658721923828125f,0.1004999825060366447360138408839702606201171875f,-0.133199998515844353352832740711164660751819610595703125f,-0.210600001761317212611857030424289405345916748046875f,-0.034699999082088428536962965154089033603668212890625f,0.49460001201331615927614393513067625463008880615234375f,0.12869999935030929094637031084857881069183349609375f,-0.3891000207901000873533803314785473048686981201171875f,-0.3969000089675187581406134995631873607635498046875f,0.356000007033347998230965458787977695465087890625f,-0.261799997758865365948821590791339986026287078857421875f,-0.235500001747161125109641943708993494510650634765625f,0.229199996086955071206148204510100185871124267578125f,-0.37989999810457231177451831172220408916473388671875f,0.2343999936938285788556868283194489777088165283203125f,-0.2122000022113323947081653386703692376613616943359375f,0.084300000515580120197256519531947560608386993408203125f,0.9040999828666447246661164172110147774219512939453125f,0.1469999959617853679816334988572634756565093994140625f,-0.5284999842345714515801091692992486059665679931640625f,-0.110799993482232128627629208494909107685089111328125f,-0.78340003747344066908908644109033048152923583984375f,0.141100002074241448468683302053250372409820556640625f,-0.4189000113368035460581495499354787170886993408203125f,-0.41249998390674580406312088598497211933135986328125f,-0.700500009477138529945250411401502788066864013671875f,0.174400027519464373426671954803168773651123046875f,0.62980000410675973654406334389932453632354736328125f,-0.781999965190887991184354177676141262054443359375f,-0.06919998692274109686195515678264200687408447265625f,0.791599989199637921188923428417183458805084228515625f,0.2166000266015533082253341490286402404308319091796875f,-0.71760000502467136129780556075274944305419921875f,0.287700023311376895662760944105684757232666015625f,0.66969997881650922266771885915659368038177490234375f,0.56980000321269042018457184894941747188568115234375f,0.199399998685717338986478353035636246204376220703125f,-0.4967000217169525999594270615489222109317779541015625f,-0.356299962055683039352516061626374721527099609375f,-0.22579999415278440011434213374741375446319580078125f,-0.009800034952163638735100903431884944438934326171875f,0.62769999008178700705684605054557323455810546875f,1.0004000016212462043085906771011650562286376953125f,0.45160000246167175674827376496978104114532470703125f,0.113800000798702338755674645653925836086273193359375f,-0.7112999978184697535965597126050852239131927490234375f,0.620399989104270854767264609108678996562957763671875f,-0.1158000249981883111871638902812264859676361083984375f,-0.4064999911785118769103064551018178462982177734375f,-0.72410002555847174932068810448981821537017822265625f,-0.04779996634721772608145329286344349384307861328125f,-0.32440000635981558474441044381819665431976318359375f,0.01249999679625035664543020175187848508358001708984375f,-0.1739999960958957814227687777020037174224853515625f,0.2900999992132187177418245482840575277805328369140625f,-0.14609998642802235480075978557579219341278076171875f,0.11759999876618376646320029976777732372283935546875f,-0.5048000249683857187932289889431558549404144287109375f,0.236399994796514523187624945421703159809112548828125f,0.38020000349283211704687346355058252811431884765625f,-0.23720000534057616992100747665972448885440826416015625f,-0.39680000982880603288549536955542862415313720703125f,0.041200003969669385384122506366111338138580322265625f,0.7344000127673149247442552223219536244869232177734375f,-0.2021000009864568625062020146287977695465087890625f,-0.351200001734495170779837280861102044582366943359375f,-0.10060000265538693309252948893117718398571014404296875f,0.02039999767243860784304843036807142198085784912109375f,0.343900000384449899826222463161684572696685791015625f,-0.35840001303553581379901515902020037174224853515625f,-0.2505999983340501113815435019205324351787567138671875f,0.5551000188827515113842991922865621745586395263671875f,-0.127600002565980030766468189540319144725799560546875f,-0.9045000258386135527644000831060111522674560546875f,0.03300000031292438240626552214962430298328399658203125f,-0.17470000101923943702786345966160297393798828125f,-0.16120000165998937813327529511298052966594696044921875f,-0.35370000635385512310904232435859739780426025390625f,0.309700005936622613234021628159098327159881591796875f,-0.196899991160631060438390704803168773651123046875f,0.2629999969601630738225139793939888477325439453125f,-0.2514000093996526796757962074480019509792327880859375f,0.32690000442266420321857367525808513164520263671875f,1.160600038045644932793720727204345166683197021484375f,-0.2891999991416926985010604767012409865856170654296875f,-0.95820004791319413062211651777033694088459014892578125f,0.63199999082088442747817680356092751026153564453125f,-0.202100015217065998029966067406348884105682373046875f,0.4787999525189405858327518217265605926513671875f,0.11009999980926554741245126933790743350982666015625f,1.092499975264072276104343472979962825775146484375f,0.4189999934136867665301906527020037174224853515625f,-0.5287999762117865376609415761777199804782867431640625f,0.4148000058017669022802920153480954468250274658203125f,-0.938199988234043047441446105949580669403076171875f,-0.5209000115454198720499334740452468395233154296875f,-0.56720000265836656438978025107644498348236083984375f,-0.4862000217020512327081860348698683083057403564453125f,-0.8929000040054315689985742210410535335540771484375f,0.01070002191066787844420105102472007274627685546875f,-0.4375999902725222767685409053228795528411865234375f,-0.9341999921381471949644037522375583648681640625f,0.038199999447166899102512616082094609737396240234375f,-0.3382000118896366647192053278558887541294097900390625f,-0.41749999284744243599476476447307504713535308837890625f,0.470800033529103212348587703672819770872592926025390625f,-0.00729998607635540253113504149951040744781494140625f,-0.491699997204542160744722423260100185871124267578125f,0.97529997856617001161794178187847137451171875f,-0.1342000045061109148036848637275397777557373046875f,-0.260499987721443115873398710391484200954437255859375f,0.057400008088350329416016393224708735942840576171875f,-0.0993000042408704874929981087916530668735504150390625f,-0.21280001209378252013948440435342490673065185546875f,0.090000004470348338170282431747182272374629974365234375f,0.18000000424683104682799239526502788066864013671875f,-0.2012000034481287169541019466123543679714202880859375f,-0.271500007867813142325985609204508364200592041015625f,-0.324799992483854271796417378936894237995147705078125f,0.1764000049293041882947363774292171001434326171875f,-0.16570000499784942604719617520458996295928955078125f,-0.92379999349713326495248111314140260219573974609375f,0.175999998390674594617166803800500929355621337890625f,0.197699998668581233829399934620596468448638916015625f,0.11700000296533108468111095135100185871124267578125f,0.85910000767707828117636381648480892181396484375f,0.15280000665485860000814000159152783453464508056640625f,0.3397000118970872062362786891753785312175750732421875f,0.141700007784366643903695148765109479427337646484375f,0.72350001403689390144791104830801486968994140625f,-0.02480000149905681183781780418939888477325439453125f,0.6160999992430209459826073725707828998565673828125f,-0.1197999986305833264310649610706605017185211181640625f,0.245999988742172714939471234174561686813831329345703125f,0.1587000023305416285257507524875109083950519561767578125f,0.103900004296004766501937410794198513031005859375f,0.0888999941259622783462646111729554831981658935546875f,-0.17119999592304235846995652536861598491668701171875f,-0.153299999070167558556931908242404460906982421875f,-0.787700007367134080737969270558096468448638916015625f,0.14929999999403953392373978203977458178997039794921875f,-0.50460002788305313714545263792388141155242919921875f,0.31259998639822017540979004479595459997653961181640625f,0.107299988274276170141519060052814893424510955810546875f,-0.152699989783763978579145259573124349117279052734375f,-0.6295999955475328757614761343575082719326019287109375f,-0.0204999954700468833834747783839702606201171875f,-1.21970002515912057106106658466160297393798828125f,-0.3920000134706498329251189716160297393798828125f,0.77310002523064635937544153421185910701751708984375f,-0.23279994876384790813972358591854572296142578125f,-0.9527999982357027164425744558684527873992919921875f,-0.8421999928236001409231903380714356899261474609375f,-0.1326000057697296341530091012828052043914794921875f,0.28909997608661708312638438655994832515716552734375f,1.014699997591972380206470916164107620716094970703125f,0.903699994134902429010480773285962641239166259765625f,0.65800002422928827172654564492404460906982421875f,0.35569999628066995001063332892954349517822265625f,0.82990002418160457864360068924725055694580078125f,-0.2464000010550020380861724333954043686389923095703125f,-0.126400004705786717096316351671703159809112548828125f,-0.240599990808963770660966474679298698902130126953125f,-0.30069999911189082553164553246460855007171630859375f,0.56009999027252188597003623726777732372283935546875f,-0.222300015580654264368831718456931412220001220703125f,-0.40889999300837498452665386139415204524993896484375f,-0.32750000879168517098349866500939242541790008544921875f,-0.237599988484382240727654789225198328495025634765625f,-0.8175000230967999215181407635100185871124267578125f,-0.184699997144937400062048027393757365643978118896484375f,-0.7374999916553497403270966970012523233890533447265625f,-0.1097999990403652204040696460651815868914127349853515625f,-0.1208000000759958947327277201111428439617156982421875f,0.13069999791979791670115673696273006498813629150390625f,-0.089499994248151804043800439103506505489349365234375f,0.234899997583031705516987130977213382720947265625f,-0.454799979817867239262341172434389591217041015625f,-0.082799999800324375343052452080883085727691650390625f,0.612500015199184399961040980997495353221893310546875f,0.26159999366998665237815657746978104114532470703125f,-0.27960002102851877481271003489382565021514892578125f,0.1366000094652175533838089904747903347015380859375f,0.68259999712705621277564205229282379150390625f,-0.0642999943315982758207383085391484200954437255859375f,-0.24149999609589567484135841368697583675384521484375f,-0.1897000069797039056851417626603506505489349365234375f,-0.6587000108242035967265337603748776018619537353515625f,-0.0077000038653611824202016578055918216705322265625f,-0.44039999625682835215911836712621152400970458984375f,0.02480000254213809984804584019002504646778106689453125f,0.23870000247955325267668058586423285305500030517578125f,-0.0461999975621700809114145158673636615276336669921875f,0.4029999851882457573282181328977458178997039794921875f,-0.1241000020146370008689018504810519516468048095703125f,-0.36270000334382057616267047706060111522674560546875f,0.127300002169609072222300483190338127315044403076171875f,0.20450000505149368290602751585538499057292938232421875f,0.3196000013589859189977460118825547397136688232421875f,-0.4133999865114688088141292610089294612407684326171875f,0.06710000135898590289951215481778490357100963592529296875f,-0.731500003248453101178938595694489777088165283203125f,0.64559998931884710060558063560165464878082275390625f,-0.059199985134601451619573708740063011646270751953125f,0.58609999700784687348686929908581078052520751953125f,0.12289998269677138154065687558613717555999755859375f,-0.05419995950460521072500341688282787799835205078125f,-1.165700007903576196355288630002178251743316650390625f,-0.353700002181529526978920330293476581573486328125f,-1.3736000402212145132807563641108572483062744140625f,-0.668100008094310027217943570576608180999755859375f,-1.04740002656579012096926817321218550205230712890625f,0.60569998510479905462489114142954349517822265625f,-0.16600001209974291072057894780300557613372802734375f,-0.0625999869942663877964150742627680301666259765625f,0.1438000098884106847663133521564304828643798828125f,-0.036700007486343277918194871745072305202484130859375f,0.6886000089287758196832101020845584571361541748046875f,-0.006500004589557573098090870189480483531951904296875f,0.8699999973177907630628169499686919152736663818359375f,0.13739999717324968830922671259031631052494049072265625f,0.03469999856054784004300017841160297393798828125f,-0.016399997627735363181500360951758921146392822265625f,0.00420000629425043570108755375258624553680419921875f,-0.287999982357025174906084430404007434844970703125f,0.55340000051856019780416318099014461040496826171875f,-1.037899974799155966564967457088641822338104248046875f,-0.1890999903917311986134563994710333645343780517578125f,-0.157699975180626505988357166643254458904266357421875f,-0.6393999919056891911850470933131873607635498046875f,0.49509998967647472767339422716759145259857177734375f,0.489000001609325440909969984204508364200592041015625f,-0.3608000044345855972238723552436567842960357666015625f,0.0711999825119971063713819603435695171356201171875f,-0.067700000587105790827990858815610408782958984375f,-0.539200011062622053259474341757595539093017578125f,0.1473999896109103868724332642159424722194671630859375f,-0.2174000104725362092494833632372319698333740234375f,-0.01789999312758450145111055462621152400970458984375f,0.52309999557733544151005844469182193279266357421875f,0.146100008258223557078281373833306133747100830078125f,0.50080001680254948581705320975743234157562255859375f,-0.042900005719065603670969721861183643341064453125f,-0.6049999839067459195263154470012523233890533447265625f,0.22019999954402447173151813331060111522674560546875f,-0.437899997299909582437749122618697583675384521484375f,-0.04449999819695948932718465584912337362766265869140625f,0.111600001120567338119826672482304275035858154296875f,-0.04269999850094319260307429431122727692127227783203125f,0.06319999836683276583926271996460855007171630859375f,0.026999998383224033116078999228193424642086029052734375f,-0.224899993671476838841272183344699442386627197265625f,-0.1995999993473291933554492061375640332698822021484375f,0.4762999809801577999479604841326363384723663330078125f,0.246399998298287403741824164171703159809112548828125f,-0.2530999933421611558514996431767940521240234375f,0.33390000321567059327065862817107699811458587646484375f,0.46320000805258754184023928246460855007171630859375f,0.2580999905854464149257410099380649626255035400390625f,-0.3749999852478504980268780855112709105014801025390625f,-0.317500007823109609006451137247495353221893310546875f,-0.68360000818371791098115863860584795475006103515625f,0.8662999654829504070363555001677013933658599853515625f,0.806499990358948704027852727449499070644378662109375f,-0.33999997824430483461810581502504646778106689453125f,-0.807500000596046429990337855997495353221893310546875f,-0.48989998648166643846479928470216691493988037109375f,0.315100011879205776921253345790319144725799560546875f,-0.10829999504685405842252521324553526937961578369140625f,0.605499996103346038722747834981419146060943603515625f,0.827000001549720753502015213598497211933135986328125f,0.46170000212192530142374380375258624553680419921875f,-0.73359996809959415742241617408581078052520751953125f,-1.00720000876784343546432864968664944171905517578125f,0.280199990379809804608157719485461711883544921875f,1.04130002300143242877084048814140260219573974609375f,0.603499994516372595398934208787977695465087890625f,0.22699998769164064782444256707094609737396240234375f,0.58509999861717165714480870519764721393585205078125f,0.9909000216782093417577925720252096652984619140625f,-0.075700000154971913701729135937057435512542724609375f,-0.442900005644559779938163046608678996562957763671875f,-0.42699999156594259375196998007595539093017578125f,0.108400011545419783232091504032723605632781982421875f,-0.1100000196695329890417269780300557613372802734375f,-0.9406000028789041067511789151467382907867431640625f,-0.10340001229047768305235877051018178462982177734375f,0.36040001279711741943856395664624869823455810546875f,0.6243000025272369413187334430404007434844970703125f,-0.46640002161860483198552174144424498081207275390625f,-0.1413999630570419441255580750294029712677001953125f,0.96279999540448191197583582834340631961822509765625f,-0.17350000047683711290602559529361315071582794189453125f,-0.65909998860359186867441394497291184961795806884765625f,0.1963999973297119172599423109204508364200592041015625f,-0.01139999747872355584377146442420780658721923828125f,0.2491000005990267052169429007335565984249114990234375f,-0.0105999899148940812665387056767940521240234375f,0.0442999996960163067161175831643049605190753936767578125f,-0.0036999999836087038573850804823450744152069091796875f,-0.0810000011473894776514725890592671930789947509765625f,-0.8081000221014023043863971906830556690692901611328125f,0.1264000024706124492013259441591799259185791015625f,0.620899982041120512832321765017695724964141845703125f,-0.235000004544854146359966762247495353221893310546875f,0.15869999681711188799226874834857881069183349609375f,-0.107900001844763782088421066873706877231597900390625f,-0.472600008898973467097448519780300557613372802734375f,-0.024300009009242007351758729782886803150177001953125f,-0.679599993014335712615547890891321003437042236328125f,0.07810000094175340024094111868180334568023681640625f,-0.0684999940693378217559939002967439591884613037109375f,0.0406000039219855946015513836755417287349700927734375f,0.4031999919593334258394179414608515799045562744140625f,-0.1700000018626450781766834552399814128875732421875f,0.56430001787543293545468259253539144992828369140625f,-0.0837999983400106174258326063863933086395263671875f,0.29590000558495521687518703402020037174224853515625f,0.030499997481703733370750342146493494510650634765625f,-0.780400005942583074869389747618697583675384521484375f,0.190599990063905710968583662179298698902130126953125f,0.276400003681331885019289984484203159809112548828125f,0.81679999388456270281722026993520557880401611328125f,0.022100000911950967719121763366274535655975341796875f,0.818299988639354491937183411209844052791595458984375f,0.901300001096725367233375436626374721527099609375f,0.124299998652934728937680119997821748256683349609375f,-0.33739999780654894578901803470216691493988037109375f,0.30209999614357929687002979335375130176544189453125f,0.048999992966651806369782207184471189975738525390625f,-0.15909999910891059471396147273480892181396484375f,-0.0251000004798174192188753295340575277805328369140625f,0.356799975667894286868175868221442215144634246826171875f,0.47209998800382013683929471881128847599029541015625f,0.38059998812675477353195674368180334568023681640625f,-0.390200008410215293253031632048077881336212158203125f,0.0121000162601472283085968228988349437713623046875f,0.159999996572732783306491910479962825775146484375f,-0.531899998760222647575801602215506136417388916015625f,-0.8607000245928764226022167349583469331264495849609375f,-0.443300015461444818498648601234890520572662353515625f,-0.30300000813603400473539295489899814128875732421875f,1.03419998096227683248571338481269776821136474609375f,0.44209999092817309218617083388380706310272216796875f,-0.44669997470378908221988467630580998957157135009765625f,-0.8352999888107179859986217707046307623386383056640625f,0.368600010716914283648293348960578441619873046875f,1.16210002892613406544342069537378847599029541015625f,-0.238299966138601604370705899782478809356689453125f,0.501000010460614930707379244267940521240234375f,-0.6515000142194329324496493427432142198085784912109375f,-0.1321000024393201022210320161320851184427738189697265625f,-0.2188000095158815216933589908876456320285797119140625f,-0.4591000035047532179532936424948275089263916015625f,-0.0591000053673982339574877187260426580905914306640625f,-0.8693999957799911637579270973219536244869232177734375f,0.1024999936670065725508038667612709105014801025390625f,-0.3643999786436558085966908038244582712650299072265625f,-0.0329000033348798570642657068674452602863311767578125f,-0.2817999959707260249075488900416530668735504150390625f,0.00470000005066395498687370491097681224346160888671875f,0.17730000239312648790956927769002504646778106689453125f,-0.1483999956384301210920995117703569121658802032470703125f,-0.1019999992772936125984273303402005694806575775146484375f,-0.0534000014871358619217289742664434015750885009765625f,-0.3056999934494495363423993694595992565155029296875f,0.1555000007599591782536663231439888477325439453125f,-0.378200009542703607934299725457094609737396240234375f,0.23750000737607479539548194225062616169452667236328125f,0.1224000116646291047572958632372319698333740234375f,-0.0954999981522559604485422823927365243434906005859375f,0.7541000163197517647262202444835565984249114990234375f,-0.1369999978244305083308063331060111522674560546875f,-0.922200013238191562692236402654089033603668212890625f,0.1714999965429306050257451943252817727625370025634765625f,0.1722999998971819213267053783056326210498809814453125f,0.034999988451600128058771588257513940334320068359375f,-0.5694999932050706092923064716160297393798828125f,0.258699995923042269652114555356092751026153564453125f,-0.3371000074684620262388534683850593864917755126953125f,0.03510000228658327614539302885532379150390625f,-0.16049999114871038585761198191903531551361083984375f,0.00960000106096270400257708388380706310272216796875f,0.666400014913082205936234458931721746921539306640625f,0.30859998121261622561206650061649270355701446533203125f,1.2956999849185344420021692712907679378986358642578125f,0.30919999623596650106804872848442755639553070068359375f,-0.0126000022307037884417013628990389406681060791015625f,0.59570000134706457828315251390449702739715576171875f,0.73320002429485331418845817097462713718414306640625f,-0.238999984174967039507464505732059478759765625f,0.057700005131959830606547257048077881336212158203125f,0.02129998589754134030727072968147695064544677734375f,0.729800002020597471386054166941903531551361083984375f,0.6983999657988544385034401784650981426239013671875f,0.1287000056087972410523434518836438655853271484375f,0.81089999530315370890320991748012602329254150390625f,-0.266100021147728238446461546118371188640594482421875f,0.2319000139407816352132840620470233261585235595703125f,0.22410001004263768731306072368170134723186492919921875f,-0.25479999494254601533071991070755757391452789306640625f,-0.2686999964073299995703791864798404276371002197265625f,-0.49959997394084876276565410080365836620330810546875f,-0.49820000820159915377871584496460855007171630859375f,0.215499999344348935892412555404007434844970703125f,-0.59090000200867631718892880599014461040496826171875f,-0.12170002164244664299985743127763271331787109375f,-0.64490001181364053906008848571218550205230712890625f,-0.1640999832578005079852800918160937726497650146484375f,0.42939999859258504155690161496750079095363616943359375f,-0.084300012362003684529554448090493679046630859375f,0.987099993610382142605885746888816356658935546875f,-1.01190000814795411088198306970298290252685546875f,-0.6990999582052239702534279786050319671630859375f,1.478100037598609528544102431624196469783782958984375f,-0.1327999830365182987179650808684527873992919921875f,-0.634200020003318432060268605710007250308990478515625f,0.1153999923825264062315909541212022304534912109375f,0.5445000259876255910285181016661226749420166015625f,1.4366000112533559018856976763345301151275634765625f,1.25099999660253491384764856775291264057159423828125f,0.26299994957447125898397644050419330596923828125f,0.6707999993681905692710643052123486995697021484375f,-0.94719998328685761634915252216160297393798828125f,-0.2072999644696709165003767338930629193782806396484375f,-1.19299997267127100286643326398916542530059814453125f,0.09240003520846418183509740629233419895172119140625f,-1.96389995143413642608720692805945873260498046875f,-0.386099995666742135114191114553250372409820556640625f,0.12790002099275543656631271005608141422271728515625f,-0.78180001310706170869480047258548438549041748046875f,-0.02790002725124285376523403101600706577301025390625f,0.1533000272333620017661814927123486995697021484375f,-0.41050003361701925541638047434389591217041015625f,0.7875000052154064178466796875f,0.6663000225543971311026325565762817859649658203125f,1.16460000046491618519439725787378847599029541015625f,-0.035000067949294333402576739899814128875732421875f,-0.79680002323985110734838599455542862415313720703125f,-0.04389999077320094755805257591418921947479248046875f,-0.99000002101063699910810100845992565155029296875f,-1.0720000156313178596434454448171891272068023681640625f,0.205599994310736722269439269439317286014556884765625f,-0.31269999514818191954645953956060111522674560546875f,0.0673999933362008363246786757372319698333740234375f,-0.67039996716231120554851941051310859620571136474609375f,-0.14369999607205397040843308786861598491668701171875f,0.576799979057908362989337547332979738712310791015625f,-0.3820999913752078924744637333787977695465087890625f,0.0102000031948088487609993535443209111690521240234375f,-0.15649999237060541190658113919198513031005859375f,0.373899999751150513560560284531675279140472412109375f,-0.0173000060439107983256690204143524169921875f,-0.332299992856383230588335209176875650882720947265625f,-0.057500008642673350323093472979962825775146484375f,-1.1204999639913439946070639052777551114559173583984375f,0.3171000049352645788758309208787977695465087890625f,0.6135999940276144481998699120595119893550872802734375f,-0.1692000101871788952934139160788618028163909912109375f,0.11889999267309914188217589980922639369964599609375f,-0.134099997320771269215811116737313568592071533203125f,0.2183999893426896432568895534132025204598903656005859375f,0.0797000099599360911639678306528367102146148681640625f,0.5118999832257629112319818887044675648212432861328125f,-0.292200000572204743320980924181640148162841796875f,-0.7001999892622234877848086398444138467311859130859375f,-0.24799999554455276484787873414461500942707061767578125f,0.1810000027492641994086852719192393124103546142578125f,-0.102600002789497324329204275272786617279052734375f,-0.5652999983102080516772502960520796477794647216796875f,0.08919999407529832780028300476260483264923095703125f,-1.814399997568131084335618652403354644775390625f,0.35190001209676236015866379602812230587005615234375f,0.12069998242259050158509126049466431140899658203125f,0.04469999520778600299308891408145427703857421875f,-1.153999961376189986594908987171947956085205078125f,-0.5112000100791451728809988708235323429107666015625f,0.2391999934792519155735135427676141262054443359375f,0.89159998323917388773907077847979962825775146484375f,0.81619996731281307944527725339867174625396728515625f,0.414599977368116157094846130348742008209228515625f,1.1160999651193623805056631681509315967559814453125f,1.112699997681378505376414977945387363433837890625f,0.881099937105179975560531602241098880767822265625f,0.440999997347592653795800288207828998565673828125f,-1.61669997112750962742211413569748401641845703125f,-0.85740001032352441967532286071218550205230712890625f,0.360000010728835917461765347979962825775146484375f,0.70320003222972171652571660160901956260204315185546875f,1.0094999896287912832804067875258624553680419921875f,-0.198600004085898174110980107798241078853607177734375f,-0.00800000068545347886583840590901672840118408203125f,-0.84300002087652659010785782811581157147884368896484375f,-0.089100003132223992707849902217276394367218017578125f,0.892800030720234616410380112938582897186279296875f,-0.4493999725341808471057447604835033416748046875f,0.4949999846145509696526687548612244427204132080078125f,0.7501999782726171961400041254819370806217193603515625f,0.1190999947130677583118085749447345733642578125f,-1.0435000094175332918666754267178475856781005859375f,0.388200016844272521865377711947076022624969482421875f,0.62159998368620872355450046597979962825775146484375f,-0.0435000011287629551048894427367486059665679931640625f,-0.18089999954998481346279959325329400599002838134765625f,-0.04249999940395365882750411401502788066864013671875f,-0.92290001980066282971648661259678192436695098876953125f,-0.00140000150203700712836507591418921947479248046875f,-0.42709998459517961588716161713819019496440887451171875f,0.25720000057220460121243377216160297393798828125f,1.0463999931946397214943544895504601299762725830078125f,-0.16320001118183125043969994294457137584686279296875f,-0.72859997193664316483818765846081078052520751953125f,0.1119000032305716985092658433131873607635498046875f,-0.4865000177398324243682736778282560408115386962890625f,-0.15660000708103183342245756648480892181396484375f,0.3051000027522443591720957556390203535556793212890625f,0.1301999969363212272810415015555918216705322265625f,-0.12960000504702318124827797873876988887786865234375f,0.28599998795986181221451261080801486968994140625f,-0.16590000890046330273008834410575218498706817626953125f,0.31460000500977047810380327064194716513156890869140625f,-0.182599990160763336888294361415319144725799560546875f,-0.0484999991729855306488161659217439591884613037109375f,-0.305399985677003993345124399638734757900238037109375f,0.247199992823600833702357704169116914272308349609375f,-0.7726999813273549566616793526918627321720123291015625f,0.1892999964177608518411943805404007434844970703125f,-0.18059999539107085553268916555680334568023681640625f,-0.157100000762939284726371624856255948543548583984375f,-0.5956999650627377018707875322434119880199432373046875f,-0.0495999992728233252137215458787977695465087890625f,-0.3623999647259719125713672838173806667327880859375f,-0.13110000240951780181575259121018461883068084716796875f,-0.58069999359846102837678927244269289076328277587890625f,1.017900030380487930159461029688827693462371826171875f,-0.4187000570178032177182103623636066913604736328125f,0.021299996179342084445806904113851487636566162109375f,-0.7960999803185462297960839350707828998565673828125f,-1.5225000175833702797945079510100185871124267578125f,0.204400039887428164320226642303168773651123046875f,0.887700011241435493758444863487966358661651611328125f,-0.3629999874234197676514668273739516735076904296875f,1.03680002145171190619521439657546579837799072265625f,0.626899927234649823049039696343243122100830078125f,0.5982000235497952900232121464796364307403564453125f,1.2377000067710870467863060184754431247711181640625f,-0.41849999123811709722531304578296840190887451171875f,-0.6803999861240388469241224811412394046783447265625f,-0.13559997933506995337893386022187769412994384765625f,-0.17930001564025843663330306299030780792236328125f,-0.76620002509206575069811151479370892047882080078125f,-1.0525999878510827745259348375839181244373321533203125f,0.22739999650269737685448490083217620849609375f,-0.329799987640976954850202673696912825107574462890625f,-0.0803999902963641321917975801625289022922515869140625f,-1.31809997516274446383022223017178475856781005859375f,-0.37529999316930773289868739084340631961822509765625f,-0.65099997782707230697951672482304275035858154296875f,0.9044999821782104820755421314970590174198150634765625f,-0.07630000333189901340347205405123531818389892578125f,0.46860000296831127997165822307579219341278076171875f,-0.68850001187622533738164065653108991682529449462890625f,0.127700000736117313948625451303087174892425537109375f,-0.25450000274181372450499338810914196074008941650390625f,0.188600010120868688545670011080801486968994140625f,-0.8582999771282080470058417631662450730800628662109375f,0.2440999963521957027978714904747903347015380859375f,-0.033700013096630510034401595476083457469940185546875f,0.06730000365972532705427511245943605899810791015625f,0.7213000155508517696745229841326363384723663330078125f,0.011000000253319708320987047045491635799407958984375f,-0.66499999374151241848807103451690636575222015380859375f,-0.33340000245571133064004243351519107818603515625f,-0.02839999224841582847744803075329400599002838134765625f,0.2593999984622001875322894193232059478759765625f,-0.34620000005811457644000483924173749983310699462890625f,-0.0730999917030334955825310316868126392364501953125f,-0.287599984386563534144443110562860965728759765625f,-0.094299995337426611996534120407886803150177001953125f,0.1917999971255659463764686734066344797611236572265625f,0.110800005626678466086332264239899814128875732421875f,-0.186300003480911247066842406638897955417633056640625f,0.20019999544620503684200230054557323455810546875f,-0.513400011061131866796358735882677137851715087890625f,-0.0877999965220690459233310320996679365634918212890625f,0.3074999863281846490536963756312616169452667236328125f,-0.134199999625980836981398169882595539093017578125f,-0.1250999961584806641212708200328052043914794921875f,-0.102099991971254411282643559388816356658935546875f,0.948799981725215957339969463646411895751953125f,0.007000019431114168355634319595992565155029296875f,-0.8795000084042552401797365746460855007171630859375f,0.135900013110041850694642562302760779857635498046875f,-0.15590000469088582946142196306027472019195556640625f,-0.147000016301870317647626507095992565155029296875f,-1.64289997539520271629953640513122081756591796875f,-0.05759997969269736728392672375775873661041259765625f,0.1356999843597410659867819049395620822906494140625f,-1.100700044411421441026277534547261893749237060546875f,-0.0564999634623521984622129821218550205230712890625f,0.4290000116676093711021167109720408916473388671875f,-0.67580000070929546751585803576745092868804931640625f,0.28859999223947507829279857105575501918792724609375f,-0.0379999918937681258057637023739516735076904296875f,0.10650000935792913736577247618697583675384521484375f,-0.222200015473365652951542870141565799713134765625f,-0.36360001578330969351782187004573643207550048828125f,-1.142799956512451586831957683898508548736572265625f,0.5960999888867137830317233238019980490207672119140625f,1.0877999565124516490044470629072748124599456787109375f,-0.2361000107169151096542236700770445168018341064453125f,-0.55930000982880567761412748950533568859100341796875f,-0.05669996874332383640648913569748401641845703125f,-0.83899999654293111461811349727213382720947265625f,-0.12660000007748573036536754443659447133541107177734375f,0.316899992725252932501689429045654833316802978515625f,0.431699998620152403105976191000081598758697509765625f,0.2026999893367291694090681630768813192844390869140625f,-0.327000013023614588547616222058422863483428955078125f,1.259999986886978007305515347979962825775146484375f,-1.18950005042552930945021216757595539093017578125f,0.5074999826401473956849486057762987911701202392578125f,0.28920000264346601426268534851260483264923095703125f,0.47649999207258220135230430969386361539363861083984375f,-0.0802000034928322946825574035756289958953857421875f,-0.9772999980717897994253462456981651484966278076171875f,0.056900003156065803722185592050664126873016357421875f,-0.98380000787675381257457729589077644050121307373046875f,-0.232000000059604527535839224583469331264495849609375f,-0.6567999800264836540719670665566809475421905517578125f,0.03499999731779102063455866300500929355621337890625f,0.620900003349781126615880566532723605632781982421875f,-0.501199987578392125442405813373625278472900390625f,0.287600003944337156980282088625244796276092529296875f,0.08229999322891234925236858543939888477325439453125f,-0.238899999080598224310278965276665985584259033203125f,-0.0391999991603195763190115030738525092601776123046875f,-0.30409999206811189065291500810417346656322479248046875f,-0.0140999884545802700586136779747903347015380859375f,0.573500005282461255973203151370398700237274169921875f,-0.18980000067949287512192313442938029766082763671875f,-0.41059998663663865858808321718242950737476348876953125f,0.1726999975323675329264005995355546474456787109375f,-1.125700012969970575937850298942066729068756103515625f,0.0135000041648745128242126156692393124103546142578125f,0.8697999816432595565629526390694081783294677734375f,0.12220000176429746119310948415659368038177490234375f,-0.398199989388883235363891799352131783962249755859375f,-0.384199991691112519021089610760100185871124267578125f,0.212099989959597667876778359641321003437042236328125f,0.304300004017353131757772644050419330596923828125f,0.63889997229576156456687385798431932926177978515625f,-0.620100007855892254582386158290319144725799560546875f,0.392299995017051816859066093456931412220001220703125f,0.85590003538727810195041456609033048152923583984375f,-0.791500000715256391004004399292171001434326171875f,0.480600025752187409722182565019465982913970947265625f,-1.16660001289248516087582174804992973804473876953125f,1.3127999930202955969349432052695192396640777587890625f,0.28559996443390911480264549027197062969207763671875f,-1.008900015062093924456121385446749627590179443359375f,-0.1600999571919441422096497262828052043914794921875f,0.3644999815821652333625024766661226749420166015625f,0.484000012040137761459845933131873607635498046875f,-0.242999987572431574989195723901502788066864013671875f,-0.703099981272220730943445232696831226348876953125f,-0.25380000538080926819617388900951482355594635009765625f,-0.98349997454881676883786667531239800155162811279296875f,0.33179997704625119325783089152537286281585693359375f,1.55009997537136090528520071529783308506011962890625f,0.686999992907046941326143496553413569927215576171875f,-0.0582000138640399455880469758994877338409423828125f,0.03510003929734306638010821188800036907196044921875f,-1.924699992823601490954388282261788845062255859375f,-0.5523000089868901341105811297893524169921875f,0.928199992629885617390073093702085316181182861328125f,-0.387900001323222998195205946103669703006744384765625f,-1.1653999667525294370307165081612765789031982421875f,0.7124999824166298179051182160037569701671600341796875f,0.95509995197653818621574828284792602062225341796875f,-0.6948000206470492656762871774844825267791748046875f,-0.5714999925568700067657346153282560408115386962890625f,-0.1615000026896595830550040773232467472553253173828125f,-0.444999996274709541665970391477458178997039794921875f,-0.0843999993748962840101057736319489777088165283203125f,-0.2471999933823943174093784591605071909725666046142578125f,-0.024400010867416777049498932683491148054599761962890625f,-0.6052999756045640555868203591671772301197052001953125f,-0.290400000131130209268803810118697583675384521484375f,0.7530000037029385140385784325189888477325439453125f,0.2767000045061109858579584397375583648681640625f,-0.982000005424022770483816202613525092601776123046875f,0.0435000013709068156231296597979962825775146484375f,0.4840999920681119395027280916110612452030181884765625f,0.082199985074996817502324120141565799713134765625f,-0.40400002187490446914353015017695724964141845703125f,0.252700008559227029536486952565610408782958984375f,-0.7901999901562930528342576508293859660625457763671875f,0.455900000965595353363823960535228252410888671875f,-0.6154999863803387594174409969127736985683441162109375f,0.225100005173683115344829275272786617279052734375f,-1.0762000132828948295582449645735323429107666015625f,0.287100009256601396145924809388816356658935546875f,0.3343999933212995490094954220694489777088165283203125f,-0.31879999662637725776903607766143977642059326171875f,0.30529998068958530410554885747842490673065185546875f,0.08299999070167540793363514239899814128875732421875f,-0.21380000754147754360445787824573926627635955810546875f,-0.104700001168251066729908416164107620716094970703125f,0.05529999715536837445739593022153712809085845947265625f,-0.035100000107288309436626150272786617279052734375f,0.283099978217482561859696943429298698902130126953125f,0.158999999783933088082221729564480483531951904296875f,-0.82379999595582464877452366636134684085845947265625f,0.05189999604076145356401639219257049262523651123046875f,-0.50199999007582662979842780259787105023860931396484375f,0.68400001686438882675389550058753229677677154541015625f,-0.24789998368397359040926630768808536231517791748046875f,-0.0003000106036663563457977943471632897853851318359375f,1.11860001652836782426447825855575501918792724609375f,-0.270599977397917967891771695576608180999755859375f,-1.951599982047081738301130826584994792938232421875f,-0.46560000585913652315639410517178475856781005859375f,0.338700004863738968197139911353588104248046875f,0.9924000681400304557655545067973434925079345703125f,0.690399968242644490601378493010997772216796875f,-0.161400017297268050242564640939235687255859375f,-1.457800007176398793262706021778285503387451171875f,-0.48270001228451775432404247112572193145751953125f,-1.3324999964237207450423738919198513031005859375f,-0.3305999997496602560431711026467382907867431640625f,-2.275800020527839251371915452182292938232421875f,-0.8052000309109690334707920555956661701202392578125f,-0.14080001635551386840461418614722788333892822265625f,0.252600012177229604759531866875477135181427001953125f,0.5726999927639955245695091434754431247711181640625f,0.420300028038024908738634621840901672840118408203125f,0.265399981653690275607004878111183643341064453125f,0.0978000050902367856764385578571818768978118896484375f,0.12190000218749030835851954179815948009490966796875f,-0.15390001223087335091577187995426356792449951171875f,-0.1778000115349887000348871879396028816699981689453125f,-0.1842999974608421354105303180404007434844970703125f,0.16750001426786165215077062384807504713535308837890625f,-0.1532000017940997604881658844533376395702362060546875f,0.1740000089481472667074513083207421004772186279296875f,-0.388899996100366240892043379062670283019542694091796875f,0.73529997582808148681721149841905571520328521728515625f,0.171000004649162207215340458787977695465087890625f,0.02909998979568462207367929295287467539310455322265625f,-0.08089999925196167396279633976519107818603515625f,0.2418999955937266577166155911982059478759765625f,0.0626999998420476600813344703055918216705322265625f,-0.0154999932721258115719820125377736985683441162109375f,0.2366999997377394748809820157475769519805908203125f,0.7925999936997889516732129777665250003337860107421875f,-0.33330000525414948953084603999741375446319580078125f,-0.1595999970749020224758396580000407993793487548828125f,0.1758000011749565327789923685486428439617156982421875f,0.14720000120550380540151991226593963801860809326171875f,0.147100004935264649930104496888816356658935546875f,-0.3687999923422935477645978608052246272563934326171875f,0.31389999583959582452052927692420780658721923828125f,-0.647500000484287685509343646117486059665679931640625f,0.321300002884864710495094186626374721527099609375f,-0.04770002401918171930361722843372263014316558837890625f,-0.12870000113844870526236263685859739780426025390625f,-0.704299986322224125245838877162896096706390380859375f,0.084800000604987157970526823191903531551361083984375f,0.0590000025033949604402749855580623261630535125732421875f,-0.25490000376701349438235411071218550205230712890625f,0.5181999676704407420402276329696178436279296875f,0.0143999899685383070391253568232059478759765625f,0.1313000297069546462580547085963189601898193359375f,0.18209999338686444492196869759936816990375518798828125f,0.412000006988644695837820108863525092601776123046875f,0.377400006467849546964998808107338845729827880859375f,1.494699975556879945060018144431523978710174560546875f,0.18690001410841983187083314987830817699432373046875f,0.51169998185634568699242663569748401641845703125f,0.36479999591112122203639955841936171054840087890625f,0.879299971830845183973224266082979738712310791015625f,-0.438599999094009262989857234060764312744140625f,-0.565699964392185616901542744017206132411956787109375f,-1.26719999610185585225963222910650074481964111328125f,0.04500005483627234070809208787977695465087890625f,0.703500018507242419474323469330556690692901611328125f,0.284000006675719784965394865139387547969818115234375f,-0.40409999787956463190852218758664093911647796630859375f,-0.07399998134374652369871228074771352112293243408203125f,0.853499998390674363690777681767940521240234375f,0.640399955129623776883818209171295166015625f,0.067800014179944856351767157320864498615264892578125f,-0.7945000155568120447924229665659368038177490234375f,-0.05219997956156685603446021559648215770721435546875f,0.204899965023994212742763920687139034271240234375f,0.49140000358819957426703695091418921947479248046875f,-0.8256999930024147005269696819595992565155029296875f,0.108600001254677724915609360323287546634674072265625f,0.36179999492764469692929196753539144992828369140625f,1.042800005537271257338716168305836617946624755859375f,-0.277999992340803192547582511906512081623077392578125f,-0.31669999884366983877725942875258624553680419921875f,-0.1712999994948505300129681927501223981380462646484375f,-0.07799999904632581859686979441903531551361083984375f,-1.035700012932717850588915098342113196849822998046875f,-0.0859000057339667222322532325051724910736083984375f,-1.1666999668434263615068857689038850367069244384765625f,0.1118999969162046692616740983794443309307098388671875f,-0.01230000291466704087639527642750181257724761962890625f,-0.25810000436902047482590205618180334568023681640625f,-0.2921000016197561333086696322425268590450286865234375f,0.25579999713301670993814695975743234157562255859375f,-0.284899975715577846724357868879451416432857513427734375f,-0.15279999674558641942212489084340631961822509765625f,-0.444199986512959166251590659157955087721347808837890625f,-0.1907000117778778758292901329696178436279296875f,0.5677999795719983477937375937472097575664520263671875f,0.140100000554323145252055837772786617279052734375f,0.21819999456703642604082915568142198026180267333984375f,0.23979999538958074456473923419252969324588775634765625f,-0.770499992080032836128111739526502788066864013671875f,0.3957000025391579356437432579696178436279296875f,-0.5148999919578434170119862756109796464443206787109375f,-0.227500012554228359107355572632513940334320068359375f,0.0799999917671085913895012708962894976139068603515625f,0.141299996104836456112252562888897955417633056640625f,-0.3125000058487057419398524871212430298328399658203125f,-0.029100000636279599319777844357304275035858154296875f,0.049600001619756139970007779993466101586818695068359375f,-0.4418999902665614598618049058131873607635498046875f,-0.309999995455145782585759661742486059665679931640625f,0.878399991875886687608954161987639963626861572265625f,0.5228999607920652437087483122013509273529052734375f,1.003200033086538933702058784547261893749237060546875f,-0.6975000366568568921366022550500929355621337890625f,0.85140000343918753600291893235407769680023193359375f,-1.16969998745918246640940196812152862548828125f,-0.45989999396949998722305963383405469357967376708984375f,-0.53249998152256028394191389452316798269748687744140625f,0.941400016254186677855386733426712453365325927734375f,0.44449997562170029397066173260100185871124267578125f,-0.012000006616116110080838552676141262054443359375f,-0.2722999845862386081307704444043338298797607421875f,0.33509999787211430799516165279783308506011962890625f,-0.56819998406171823290833344799466431140899658203125f,0.5613000362634661399852120666764676570892333984375f,1.65419998006820634373070788569748401641845703125f,0.25500001639127756192237939103506505489349365234375f,0.93539998850822403397842208505608141422271728515625f,0.050100014300644313269827989643090404570102691650390625f,-1.1414999938234686727156486085732467472553253173828125f,-0.2982000077545645666532436734996736049652099609375f,-0.951900005316733910376569838263094425201416015625f,0.8355000381618740146194568296778015792369842529296875f,1.0507999895334239059963010731735266745090484619140625f,0.285600017705559761083122793934307992458343505859375f,-0.04470001979470250574877354665659368038177490234375f,0.2007000061899424370182032362208701670169830322265625f,0.3404000013232229004955797790898941457271575927734375f,0.93359997316598875016779857105575501918792724609375f,-0.3943999843806029748094488240894861519336700439453125f,0.2206000039219855324290620046667754650115966796875f,1.1610999900043008636885133455507457256317138671875f,0.0708000014543532785182833322323858737945556640625f,-0.524800030146538976083547822781838476657867431640625f,0.01470000742673871485521885915659368038177490234375f,0.7282999825298788199035016077687032520771026611328125f,-0.2482999972261488164093634622986428439617156982421875f,-0.140399994096159996814776604878716170787811279296875f,0.3032999993681908534881586092524230480194091796875f,1.0549000042885541272852378824609331786632537841796875f,-0.1149000010289251694217682597809471189975738525390625f,-0.016800000292062744478016611537896096706390380859375f,-0.02460000180602062158641274436376988887786865234375f,0.6144000094890593732799288773094303905963897705078125f,-0.0498000021323561359309906038106419146060943603515625f,-0.313799990405142337213106884519220329821109771728515625f,0.024199997428059560888868873007595539093017578125f,0.05539999711364507195554551799432374536991119384765625f,-0.074100000599026660808021915727294981479644775390625f,0.1290999980285764958676253399971756152808666229248046875f,-0.1846000046372413549988777958787977695465087890625f,-0.43449999611079681560710241683409549295902252197265625f,0.22069998972415927340762209496460855007171630859375f,-0.87829998722374436059823210598551668226718902587890625f,-0.4021999984860418209109411691315472126007080078125f,0.3009000240623951327734175720252096652984619140625f,0.17190000092089174899001591256819665431976318359375f,0.07040000128597012629683149498305283486843109130859375f,0.3371999967537819831164824790903367102146148681640625f,-0.96479998250007614757350893341936171054840087890625f,-1.15590000171065288014915495296008884906768798828125f,0.891700040864943588303503929637372493743896484375f,0.98790002337694193812467347015626728534698486328125f,0.43799990963935897525516338646411895751953125f,-1.116900026923417943436334098805673420429229736328125f,-0.1258999938130376250455810804851353168487548828125f,0.74279999335557211903591223745024763047695159912109375f,0.47319997839927696592354777749278582632541656494140625f,-1.38330002425313036695797563879750669002532958984375f,0.46670003907680523980161524377763271331787109375f,0.43500000301748509290433730711811222136020660400390625f,0.43839998472332963874720235253334976732730865478515625f,-0.27960002058148436532292180345393717288970947265625f,0.7696000088095669156018630019389092922210693359375f,0.221700000594556190236517068115063011646270751953125f,-0.355399994096159854706229452858678996562957763671875f,0.1275000105798245186861095135100185871124267578125f,0.311399997031688524629089442896656692028045654296875f,-0.163800007131695846140928551903925836086273193359375f,-0.06309999870657900800097195315174758434295654296875f,0.5583000096499919351344942697323858737945556640625f,-0.2767000062942504001739507657475769519805908203125f,0.128700023192167112284778340836055576801300048828125f,-1.2624999824166298623140392010100185871124267578125f,0.26669995428919701652148432913236320018768310546875f,0.487899999237061621215616469271481037139892578125f,-0.422100000315904555492352301371283829212188720703125f,-0.251599983835220530892229362507350742816925048828125f,-0.83639998686164596097825096876476891338825225830078125f,-0.01599999488890180288080955506302416324615478515625f,-0.092100005233287873807057621888816356658935546875f,-0.6401000087127088900018634376465342938899993896484375f,0.0363000005751847876211968468851409852504730224609375f,0.2459999902695418072529065511844237335026264190673828125f,-0.1638999982237816510632910649292171001434326171875f,1.035099980996549096090575403650291264057159423828125f,-0.4155999928951263910903435316868126392364501953125f,0.600199992428720019432830667938105762004852294921875f,-0.3754999826550484698373111314140260219573974609375f,0.555899996756016978594061583862639963626861572265625f,-0.18000000156462192535400390625f,-0.05839999612271784545924901976832188665866851806640625f,-0.27910000946521762443808256648480892181396484375f,-1.0226000093087550535386753836064599454402923583984375f,0.358500004649162207215340458787977695465087890625f,1.0587999814644455387480093122576363384723663330078125f,-0.11830001486539831745403716922737658023834228515625f,-1.0262999784842132822149096682551316916942596435546875f,-0.191000000476837072937996708787977695465087890625f,0.7333999780178073191194698665640316903591156005859375f,-0.0792999962314964168541564504266716539859771728515625f,-0.574799996060133100428402030956931412220001220703125f,-0.19990000600218760240522897220216691493988037109375f,-1.0846999695777892469550351961515843868255615234375f,-0.1519999872446060606989703956060111522674560546875f,0.6467999934747812318391879671253263950347900390625f,-0.2281000060081481706220074556767940521240234375f,-0.41490000104755131449252303355024196207523345947265625f,-0.31349999898672109566177823580801486968994140625f,-1.040799972248077409631150658242404460906982421875f,1.181400035327673236906775855459272861480712890625f,0.296499959588050643333190237171947956085205078125f,-0.25739999810457214746151066719903610646724700927734375f,0.159599998304247758795781919616274535655975341796875f,0.25560000645518332618166823522187769412994384765625f,2.162899975395202289973894949071109294891357421875f,0.20309998395442985241743372171185910701751708984375f,0.9617999917984008106941473670303821563720703125f,-0.00840002883076618189761575195007026195526123046875f,0.62930000371932948155517806299030780792236328125f,0.341400002247095102347884676419198513031005859375f,0.153600001180171918946371079073287546634674072265625f,1.3599000347614289552211630507372319698333740234375f,-0.409000046014785567649596487171947956085205078125f,-1.506300005567073441170578007586300373077392578125f,0.36720003037452642047355766408145427703857421875f,1.132200026649236246356622359598986804485321044921875f,-0.5469000020384786608929061912931501865386962890625f,0.516599988305568302848769235424697399139404296875f,1.91139992699623206817705067805945873260498046875f,0.25320000771731177025003489688970148563385009765625f,-0.849199989232421348361867785570211708545684814453125f,0.828899996137618444436157005839049816131591796875f,-1.1736999932408327396160530042834579944610595703125f,0.38069998406171823290833344799466431140899658203125f,1.60079997105598437201479100622236728668212890625f,0.28020000393986688180802957504056394100189208984375f,-0.67430000550746882481689681299030780792236328125f,0.23850000047683739268222780083306133747100830078125f,0.8451999937325715972491479988093487918376922607421875f,0.24790000936985023827219265513122081756591796875f,1.056400018861889389398811545106582343578338623046875f,-0.0819000075519085157793597318232059478759765625f,0.1200000028312204758407233384787105023860931396484375f,0.082599994053691727646793196981889195740222930908203125f,-0.6388999770455063842433673926279880106449127197265625f,0.1336999974131583002190382103435695171356201171875f,-0.878899987234175394945623338571749627590179443359375f,-0.3224999961256980185453357989899814128875732421875f,0.55530000397264966949251174810342490673065185546875f,0.01680000006854542871082003330229781568050384521484375f,0.73870001402795304468185122459544800221920013427734375f,-0.312299996507167776371716172434389591217041015625f,0.6795000055730342491955298100947402417659759521484375f,0.02880000899434076444549646112136542797088623046875f,0.941099970856309031574937762343324720859527587890625f,0.3044999909698964390969422311172820627689361572265625f,-0.5981999749347568506863126458483748137950897216796875f,0.3729999999403954547005923814140260219573974609375f,0.945300008070468766874228094820864498615264892578125f,0.36689999376833437594314091256819665431976318359375f,0.2683999843508007021597450147964991629123687744140625f,0.11830000368952742206829498172737658023834228515625f,1.0552999714136124254082460538484156131744384765625f,0.273600004048645484289181695203296840190887451171875f,0.3068999963760374871668545893044210970401763916015625f,-0.4480999955773352638743745046667754650115966796875f,0.1843000206321476763804412257741205394268035888671875f,0.118700002479553035072967759333550930023193359375f,0.482000005722046154232884873636066913604736328125f,-1.191600040459633191858301870524883270263671875f,0.671600048804283034087347914464771747589111328125f,0.856200029003620244338890188373625278472900390625f,1.32399999237060495005380289512686431407928466796875f,0.39960000318437771671398195394431240856647491455078125f,-0.58389999068006848137457609482225961983203887939453125f,-0.12719997330307908356417101458646357059478759765625f,-1.71829994721412759872691822238266468048095703125f,-0.12930003188252481294284734758548438549041748046875f,0.52150000131130269664936349727213382720947265625f,-0.783600025804341182578127700253389775753021240234375f,0.3585000400766731321056113301892764866352081298828125f,-0.014100023621320900701903156004846096038818359375f,0.932500008344650410663234652020037174224853515625f,-0.68250000447034775419297147891484200954437255859375f,0.9028999905943866366442307480610907077789306640625f,-0.16829999571740683617093736756942234933376312255859375f,-1.037399970611929678199203408439643681049346923828125f,-1.36530002461075827824288353440351784229278564453125f,0.360300039958953988161738379858434200286865234375f,-0.789599986010789844925739089376293122768402099609375f,-0.154799996209144641312605017446912825107574462890625f,-0.83070000075101813052924626390449702739715576171875f,1.0206999935984608640637816279195249080657958984375f,-0.6897000070542096406001064679003320634365081787109375f,-0.0492999894142152061959905040566809475421905517578125f,0.50849996903538663417521092924289405345916748046875f,0.545300004792213854898363933898508548736572265625f,-1.127999986827372680409098393283784389495849609375f,-0.664500011570751336620332949678413569927215576171875f,-0.050200000512599984858752577565610408782958984375f,-0.4455000382363791811002329268376342952251434326171875f,0.0965999991834163296289261779747903347015380859375f,-0.3646999918550253294569074569153599441051483154296875f,0.0538999988198280988171973149292171001434326171875f,-0.1484999957457182606646028943941928446292877197265625f,0.28210000761747355824127225787378847599029541015625f,-0.615700002762675335787889707717113196849822998046875f,-0.2603000068783760667656679288484156131744384765625f,0.7058999923974276224925006317789666354656219482421875f,-0.246900005316734283411506112315692007541656494140625f,0.1545000103041529460057290634722448885440826416015625f,-0.231699998843669874304396216757595539093017578125f,0.161900009004771749943074610200710594654083251953125f,0.3244000072538852919024066068232059478759765625f,0.48509998338073501145117916166782379150390625f,-0.1000000088289378918915417671087197959423065185546875f,-0.8118999810278413331587898937868885695934295654296875f,-0.1442999965667723927253973670303821563720703125f,0.16350001277029502233517632703296840190887451171875f,-0.12520000259876251647028766456060111522674560546875f,-0.9257999960526823723938605326111428439617156982421875f,0.1307999942831694628608829589211381971836090087890625f,-0.12860000039786090297866394394077360630035400390625f,0.133800003257393829159127562888897955417633056640625f,-0.1623999961674214009832439842284657061100006103515625f,0.11660000424981120659140287898480892181396484375f,-0.30059998686015620972256101595121435821056365966796875f,0.3344999994635582396540485206060111522674560546875f,-1.060700012969970629228555480949580669403076171875f,1.109100022429228005194090656004846096038818359375f,0.61989989230632858152603148482739925384521484375f,-0.48509999668002112827025484875775873661041259765625f,-1.26489996025562323467283931677229702472686767578125f,-0.73320001848340066175069296150468289852142333984375f,-1.240699979591369128684164024889469146728515625f,-0.461100005203485263649554326548241078853607177734375f,-1.5026999993205070182966665015555918216705322265625f,0.20850001791119598948398561333306133747100830078125f,1.351700011658667932579191983677446842193603515625f,0.4182000034332273852299977079383097589015960693359375f,0.59390001208186138370592743740417063236236572265625f,-0.2808000171005726741668695467524230480194091796875f,-0.3430999708414077531415387056767940521240234375f,-0.9558000215709210323211664217524230480194091796875f,-0.01590001795291851038882668945007026195526123046875f,0.4269000004366034772829152643680572509765625f,-1.089699994686245343444852551328949630260467529296875f,0.138899979597329803482352872379124164581298828125f,0.3421000031471255198312064749188721179962158203125f,1.092900018012523677413128098123706877231597900390625f,-0.4154000305294989203730438021011650562286376953125f,-0.62429998699277622353775996089098043739795684814453125f,-0.96299996209144633763088449995848350226879119873046875f,-0.4500000074505805525149071399937383830547332763671875f,-0.783199982720613530062792051467113196849822998046875f,-0.27120001410245908601837072637863457202911376953125f,-0.82179998285770405885841682902537286281585693359375f,0.0774000064492224026935218716971576213836669921875f,0.69679998200088755222481040618731640279293060302734375f,-0.132699999432265680621156889174017123878002166748046875f,0.5767000118635594585470016681938432157039642333984375f,-0.29789999819397916525076652760617434978485107421875f,0.376300024566054214236743291621678508818149566650390625f,0.076600007602572350862146777217276394367218017578125f,1.0675999767497181824893459634040482342243194580078125f,-0.461299992156028793033328838646411895751953125f,0.00840000230669968317442908301018178462982177734375f,-0.16260000606775282250282543827779591083526611328125f,-0.43519998926222325241752741931122727692127227783203125f,-0.2802999918282031988070457373396493494510650634765625f,0.1321999952450394477665440717828460037708282470703125f,0.142999993979930906107256305404007434844970703125f,-0.2568999948859216342356148743419907987117767333984375f,-0.0654999945946037609445511407102458178997039794921875f,0.11860000289380534521654908530763350427150726318359375f,0.1937000033736229198666478623636066913604736328125f,0.60780001362115110286055141841643489897251129150390625f,0.10080000175237646686809966922737658023834228515625f,0.66399997955560696727417280271765775978565216064453125f,0.2929999981522559693303264793939888477325439453125f,-0.105700015689432547105752746574580669403076171875f,0.04619999748766423497414734811172820627689361572265625f,0.1515000047758221501936759523232467472553253173828125f,-0.02580001158714306797037352225743234157562255859375f,-0.48740000734329225196006518672220408916473388671875f,0.212699997830390952202606058563105762004852294921875f,-0.3964999941959976847982716208207421004772186279296875f,-0.16119999800920492560862840036861598491668701171875f,0.643599997305869919728138484060764312744140625f,-0.2211000215761367015954164116919855587184429168701171875f,0.65370001888945716839174338019802235066890716552734375f,-0.14310003163814588589275444974191486835479736328125f,0.1853000372767450443234338308684527873992919921875f,-0.6729000069111581439074143418110907077789306640625f,1.21519999350905383295184947201050817966461181640625f,0.668999987602234114092425443232059478759765625f,0.413199989724159166826211730949580669403076171875f,0.687000014066696085279772887588478624820709228515625f,-0.0743999978661538108326567453332245349884033203125f,0.58769998374879361957567880381247960031032562255859375f,0.747399963459372873586517016519792377948760986328125f,0.756900023347139239149328204803168773651123046875f,0.71769997875690449973262730054557323455810546875f,0.34739999039322133622675892183906398713588714599609375f,0.63499998301267635003597433751565404236316680908203125f,-0.6669000139594079001881254953332245349884033203125f,0.471000039219855892724808654747903347015380859375f,-0.854399998164176555093263232265599071979522705078125f,0.2128000116467472224712764727883040904998779296875f,-0.12779998169541340047317135031335055828094482421875f,-0.044300021004676892744100769050419330596923828125f,-0.344100000934302874266990102114505134522914886474609375f,-0.1314999771341682777148207605932839214801788330078125f,1.015800057482719953583227834315039217472076416015625f,0.4377999913811674304042753647081553936004638671875f,-0.4295999992728234406769161068950779736042022705078125f,-1.11589998174309723566466345801018178462982177734375f,0.656700010913610743301660477300174534320831298828125f,0.6151999988734722091265894050593487918376922607421875f,-0.3329999898076057007756389793939888477325439453125f,-0.777799970780313376650383361265994608402252197265625f,0.1105999914050102006513043306767940521240234375f,-0.2487000028148294017427133439923636615276336669921875f,-0.12700000460445881600435313885100185871124267578125f,0.319899991510808501260498815099708735942840576171875f,-0.199000001609325405382833196199499070644378662109375f,0.21220000310540199084385903915972448885440826416015625f,0.121399992263317102469954988919198513031005859375f,1.07849999015778319488845227169804275035858154296875f,-0.3669999982714653441462360206060111522674560546875f,-0.3602999813973906118036438783747144043445587158203125f,0.1298999936342237759845374966971576213836669921875f,-1.0756000156939027334601632901467382907867431640625f,-0.2894000030815602197975522358319722115993499755859375f,0.6576999899327755638722692310693673789501190185546875f,0.11160000529289249016073881648480892181396484375f,-0.2309999951124190875617614437942393124103546142578125f,0.023999999001622196459493352449499070644378662109375f,-0.1401999992832541064302631639293394982814788818359375f,0.33030000538825987632662872783839702606201171875f,0.26270000371634927294195449576363898813724517822265625f,-0.3298999966144560147540687466971576213836669921875f,0.7816000106573104400098372934735380113124847412109375f,-0.09339999664425846503945649601519107818603515625f,0.65719999908208848182766814716160297393798828125f,0.244900008535385182995014474727213382720947265625f,-0.76639999557882543257392171653918921947479248046875f,0.0093000135540961537117254920303821563720703125f,0.05840001959204688120053106104023754596710205078125f,1.04129997844696031705780114862136542797088623046875f,-0.4402000031948087865885099745355546474456787109375f,0.7344000352680684073902739328332245349884033203125f,0.665099980735778473217578721232712268829345703125f,0.678600016379356407725254030083306133747100830078125f,1.411699971127510000457050409750081598758697509765625f,0.2417999996960162878423261645366437733173370361328125f,0.51580000458359709369915435672737658023834228515625f,-1.1411999975621700542660619248636066913604736328125f,0.63870003943443265370660810731351375579833984375f,0.82230000440478345780093150096945464611053466796875f,0.022700019884108968426517094485461711883544921875f,0.11940004465580056347562276641838252544403076171875f,-1.76039999685287540387435001321136951446533203125f,0.10650001226365601869616739350021816790103912353515625f,1.083900011411308650366436268086545169353485107421875f,-0.6618000224947930831831399700604379177093505859375f,-1.234599985265731447725556790828704833984375f,-0.1082999984741210841576730672386474907398223876953125f,-0.392999993532896052528258223901502788066864013671875f,-0.42510000353455534849445029976777732372283935546875f,0.297300011110305462125324993394315242767333984375f,0.45659996297359395356352251837961375713348388671875f,1.7615999731063851641010842286050319671630859375f,-0.156000017315149452912237393320538103580474853515625f,0.944399996972084210256070946343243122100830078125f,-0.4953000247597696414914025808684527873992919921875f,-1.8836000074386589631103561259806156158447265625f,-0.7758000260412689641498218406923115253448486328125f,0.57619998772740366188571670136298052966594696044921875f,-0.34160000298321246958721530972979962825775146484375f,-0.228199993374943677082455906202085316181182861328125f,0.26949999529123314090384155861102044582366943359375f,-0.56059998410344136221539201869745738804340362548828125f,-0.1963000007987023565192430396564304828643798828125f,-1.1917000094234941887094691992388106882572174072265625f,0.08619999881014239750953720431425608694553375244140625f,1.144600004264712200807707631611265242099761962890625f,0.13840000960826870368691743351519107818603515625f,1.09039997893422846431121797650121152400970458984375f,0.092599982523918100696391775272786617279052734375f,-0.4572000234827395814818373764865100383758544921875f,0.175299993132054854338974791971850208938121795654296875f,-0.5412999854691327161759772934601642191410064697265625f,0.40400000220537179984603426419198513031005859375f,0.06379999159723503954211309974198229610919952392578125f,-0.1026999859094619438337758765555918216705322265625f,0.46519998918771765517021776759065687656402587890625f,-0.07719999341964722816555877216160297393798828125f,0.13390002225190367202145580449723638594150543212890625f,-0.161600003728270547043166516232304275035858154296875f,-0.11229999732673168200136615269002504646778106689453125f,-0.1617000019729137871848934082663618028163909912109375f,0.3112999952852726348595524541451595723628997802734375f,0.1896999998271464704657773836515843868255615234375f,-0.2488999908849598430293781348154880106449127197265625f,0.31099999392032628975357511080801486968994140625f,0.08019998415857576556931007871753536164760589599609375f,0.2784000119924545657568160095252096652984619140625f,0.0966000124454495789905195124447345733642578125f,0.690000004619359952329205043497495353221893310546875f,0.123399998283386214126267077517695724964141845703125f,0.099600006425380360752797059831209480762481689453125f,-0.6337999936461446992552737356163561344146728515625f,1.09710003071427397713932805345393717288970947265625f,-1.00430003292560598993077292107045650482177734375f,-0.653700024216622654904540468123741447925567626953125f,1.4014000110015276678865348003455437719821929931640625f,-1.103100024783611132761507178656756877899169921875f,0.733999996542930688292472041212022304534912109375f,1.0359000605702401998087225365452468395233154296875f,0.659999948143958903301609097979962825775146484375f,0.056400011616200007580346209579147398471832275390625f,-0.76469998211339118565632588797598145902156829833984375f,-0.42629998768568011513480087160132825374603271484375f,-0.003400000071525965950058889575302600860595703125f,-1.03260005166530621778520071529783308506011962890625f,-0.65160000529289163750945590436458587646484375f,-0.45480001990199081518539969692938029766082763671875f,-0.460699943828583258209619089029729366302490234375f,0.5001000121772289919164222737890668213367462158203125f,0.290699999111890594605256410432048141956329345703125f,0.09030000739991683733620675411657430231571197509765625f,0.539900006523728048790644606924615800380706787109375f,-0.66419999362826320776775901322253048419952392578125f,-1.18269994895458285810718734865076839923858642578125f,-0.12869998966455487021676162839867174625396728515625f,0.3337999623537069027179313707165420055389404296875f,-1.27080003870725644077310789725743234157562255859375f,0.7971999840319157204504563196678645908832550048828125f,0.24489998907074339573597399066784419119358062744140625f,-0.375700006487965365575831810929230414330959320068359375f,0.223099999749660327097444678656756877899169921875f,0.7306999601453545434281977577484212815761566162109375f,-0.20420000361204149186278300476260483264923095703125f,-0.056300006572902017154547138488851487636566162109375f,0.2199999988079071044921875f,-0.7367999890044332200744747751741670072078704833984375f,0.4180000017285347979623111314140260219573974609375f,-0.1484999990239739275921238004229962825775146484375f,0.263799998116493128463844186626374721527099609375f,0.5099999760463835851709291091538034379482269287109375f,0.46479998875856409767948207445442676544189453125f,-0.3965000009015200443940329932956956326961517333984375f,-0.178299992513656579973257976234890520572662353515625f,-0.266700001786649210089308326132595539093017578125f,0.2755000084340573085484038529102690517902374267578125f,-0.388399995079636628503294559777714312076568603515625f,-0.37289999446868904442453640513122081756591796875f,0.343199978287518359110208621132187545299530029296875f,-0.179799996209144552494763047434389591217041015625f,-0.3900999948918819626442200387828052043914794921875f,0.26659999530911449028280912898480892181396484375f,0.08649997895956074156487147774896584451198577880859375f,0.20880001033544548505460625165142118930816650390625f,0.15669999761432418239337494014762341976165771484375f,0.0340999915838242628751686424948275089263916015625f,-0.6169999977499245336076683088322170078754425048828125f,0.37860000863075260468804117408581078052520751953125f,-0.69099999660253530464615323580801486968994140625f,1.380199968922138253901721327565610408782958984375f,-1.066899965381622905624681152403354644775390625f,-0.45530001283884047325045685283839702606201171875f,1.582300002467633248670608736574649810791015625f,0.53930000446438786099179196753539144992828369140625f,-0.70349998408556047024831059388816356658935546875f,-1.198900038158893455175757480901665985584259033203125f,0.498100027763843655748132732696831226348876953125f,-0.53079997522830968392781869624741375446319580078125f,1.31139997885227188589851721189916133880615234375f,0.32419998781681069743854095577262341976165771484375f,0.7733999699711802122692461125552654266357421875f,0.5130999800801276933270855806767940521240234375f,-0.131099990600347382496693171560764312744140625f,1.01479995880723006251855622394941747188568115234375f,0.22789999864101417870188015513122081756591796875f,-0.0767000170230864863896158567513339221477508544921875f,-0.4113999933063983149850173504091799259185791015625f,-0.1886999668657780571123794288723729550838470458984375f,-0.8038000212132931920905321021564304828643798828125f,-0.56679997808933268998998755705542862415313720703125f,0.2306999810814858165031182579696178436279296875f,-0.255900022870302112920626314007677137851715087890625f,0.773400018250942622444199514575302600860595703125f,-0.4246999730050564636485432856716215610504150390625f,1.2436999768495553553293575532734394073486328125f,-0.3469000057637690037637412387994118034839630126953125f,-0.8092000422060490194553494802676141262054443359375f,1.3762999969244003040103052626363933086395263671875f,-0.7858000178456308049135259352624416351318359375f,-0.60049999874830251034296679790713824331760406494140625f,-0.1087000014364718492032579888473264873027801513671875f,0.16020001992136212454198584964615292847156524658203125f,-0.48779999048709843378901496180333197116851806640625f,0.432600001671910650014041266331332735717296600341796875f,0.244199994969367839559026833740063011646270751953125f,0.2323999933362006498072105387109331786632537841796875f,0.357999997258186208881625134381465613842010498046875f,-0.139900011329352924605018415604718029499053955078125f,0.44229999829530708410629813442938029766082763671875f,-0.636299987909197728441768049378879368305206298828125f,0.1718999942898751243092192453332245349884033203125f,0.3973000349894164617836622710456140339374542236328125f,-0.6708999956011769683072998304851353168487548828125f,-0.4342000208973886277163956037838943302631378173828125f,0.1246000022530555639832527958787977695465087890625f,-0.1951000014483927902375626217690296471118927001953125f,0.360700008611381128798001327595557086169719696044921875f,0.3602000020027162907609863395919091999530792236328125f,0.0933999969422816267439202420064248144626617431640625f,0.1824999913200737733376399773987941443920135498046875f,0.0622999910682437985087744891643524169921875f,0.3929999936074015653986180041101761162281036376953125f,-0.1769999932050705382380328956060111522674560546875f,0.12219999710768458811571690603159368038177490234375f,0.33140000064522023848212484153918921947479248046875f,0.4577000290483239641758927973569370806217193603515625f,-0.37490001032352449072959643672220408916473388671875f,-0.02459998317956924029914489437942393124103546142578125f,0.18730000813007308124724659137427806854248046875f,-0.81619999182522295910757748060859739780426025390625f,0.4012999746471646034251534729264676570892333984375f,0.612399997061491063732319162227213382720947265625f,0.12299997133016660200155456550419330596923828125f,0.56290000996589650839041496510617434978485107421875f,-0.588099967712164328759172349236905574798583984375f,-0.4554999942779540678117200513952411711215972900390625f,0.8788999947965141501526886713691055774688720703125f,-0.765499983847141241000144873396493494510650634765625f,0.368999987304210463889830862171947956085205078125f,-1.47329995928406720651082650874741375446319580078125f,1.15999995887279538919756305404007434844970703125f,0.881100005352497106514420011080801486968994140625f,0.5663999657392508879638626240193843841552734375f,-0.24150000444054597892318270169198513031005859375f,-0.9169000047206878889483050443232059478759765625f,-0.8988000091992318640876646895776502788066864013671875f,0.6881000306509437169921739041456021368503570556640625f,0.45850000144541258606523115304298698902130126953125f,-0.2320999795287850275826713186688721179962158203125f,0.5727000070691108390974477515555918216705322265625f,-0.90399999229610017437153146602213382720947265625f,0.2201999519348145639696667785756289958953857421875f,0.963600019955634934376575984060764312744140625f,-0.3795999900341033850281746708787977695465087890625f,0.466599997507035624266791273839771747589111328125f,0.845600022101402171159634235664270818233489990234375f,0.177299991142749746586559922434389591217041015625f,0.2695999980807304297059090458787977695465087890625f,0.035599999749660327097444678656756877899169921875f,0.30230001248866333884990353908506222069263458251953125f,-0.183500002823770047566398488925187848508358001708984375f,-0.18769999514818158647955215201363898813724517822265625f,-0.30550002408027676636947944643907248973846435546875f,0.21470001107752312297094476889469660818576812744140625f,-0.31189999786615363319697280530817806720733642578125f,-0.0090000088736414962653498150757513940334320068359375f,0.19980001051425944069706019945442676544189453125f,0.10040002177506668346040896722115576267242431640625f,-0.2047999893546101901620204444043338298797607421875f,-0.8079000140264629425956854902324266731739044189453125f,0.0230999800801274801642648526467382907867431640625f,-0.2441000126317145468846092626336030662059783935546875f,0.46010001649856580030473196529783308506011962890625f,0.07620000728517783983306799200363457202911376953125f,-0.25980000276565551331486858543939888477325439453125f,-0.49829998480230541790803044932545162737369537353515625f,-0.18570000462532032514673119294457137584686279296875f,-0.0737999949499965612886853705276735126972198486328125f,-0.4374000066354870597251647268421947956085205078125f,0.105200001555681044163037540783989243209362030029296875f,-0.14119999659359450117079859410296194255352020263671875f,-0.1234999838247896608578457744442857801914215087890625f,-0.618099999302625491282014991156756877899169921875f,-0.07259999589771037875607362366281449794769287109375f,-0.283799998116493146227412580628879368305206298828125f,0.0019000109046694024783619170193560421466827392578125f,-0.0990999932229517099813165259547531604766845703125f,-0.51660001497864715958030501496978104114532470703125f,-1.2173999635338788039007340557873249053955078125f,0.60130000510141246028439354631700552999973297119140625f,-0.1147999973826110409191869621281512081623077392578125f,-0.4456999958336353984122979454696178436279296875f,1.12819999032020490403738222084939479827880859375f,0.5343999969720840681475237943232059478759765625f,0.06869998638629937204314046539366245269775390625f,0.53619999994635580975455013685859739780426025390625f,-1.63999997854232759664228069595992565155029296875f,-0.11580000041127203047608418273739516735076904296875f,0.96859998449087214567043702118098735809326171875f,-1.517499990761280059814453125f,1.1073999727725976072179037146270275115966796875f,-0.998200020867586079731381687452085316181182861328125f,0.562100026094913829410870675928890705108642578125f,0.525599997514486272365274999174289405345916748046875f,-0.401399993455409997977767488919198513031005859375f,-0.388399979656934846161675523035228252410888671875f,-0.10310000630617111028186627663671970367431640625f,0.663400013184547443501060115522705018520355224609375f,-0.7368000378057359966987860389053821563720703125f,0.1480999921128154372951257755630649626255035400390625f,0.26089998759180321741268926416523754596710205078125f,0.43599999675154688105749301030300557613372802734375f,0.01769997145533608318146434612572193145751953125f,0.3416999933302402592261159952613525092601776123046875f,-0.3489000046312806802006889483891427516937255859375f,-0.3402999903380869994862223393283784389495849609375f,0.9711999561369424327494925819337368011474609375f,-1.369999986588954943300677769002504646778106689453125f,-0.51070001118183139254824709496460855007171630859375f,-0.2773000081300736052725142144481651484966278076171875f,-0.1439000040352345077820928054279647767543792724609375f,0.1148999972477551256844208182883448898792266845703125f,-0.7053000071763992906426210538484156131744384765625f,-0.527500005997717291705839670612476766109466552734375f,-0.05570000551939013888613771996460855007171630859375f,0.2989000040352343123828404714004136621952056884765625f,0.1223000117063524072591462754644453525543212890625f,-0.0017999974608422231181492634277674369513988494873046875f,-0.3366000051438808871040464509860612452030181884765625f,0.59880001637041579964915172240580432116985321044921875f,0.099600005531311097684010746888816356658935546875f,-0.52400000231713061982219414858263917267322540283203125f,-0.22840000662803650044452297152020037174224853515625f,0.60939999838769409734595683403313159942626953125f,-0.350199997532367746089221327565610408782958984375f,-0.3326000099048018743275179076590575277805328369140625f,0.37219999587833874077347218189970590174198150634765625f,0.013100000941753453531646300689317286014556884765625f,-0.2327000107944012796679089660756289958953857421875f,-0.166399997441470492987036777776665985584259033203125f,-0.4132000082015991893058526329696178436279296875f,0.1454999937191606973474478081698180176317691802978515625f,0.1856999969512223014334040271933190524578094482421875f,0.059100001865625362285072696977294981479644775390625f,0.07410000000298022637679196122917346656322479248046875f,-0.007800021779537147370575667082448489964008331298828125f,0.591000001370906691278150901780463755130767822265625f,0.6611999926820397721627386999898590147495269775390625f,0.14019999306201924582637730054557323455810546875f,-0.28070001080930229164778211270458996295928955078125f,0.1723999922186134625690101529471576213836669921875f,0.841399983471632140208384953439235687255859375f,-0.0433000479459764164857915602624416351318359375f,-0.577699999916553519341277933563105762004852294921875f,-0.041200001585483647659202688373625278472900390625f,0.7865000154674053334247219027020037174224853515625f,-1.188200007307528949240804649889469146728515625f,0.0933999891191721332006636657752096652984619140625f,0.5075999854296444624424111680127680301666259765625f,0.517000005722045852252222175593487918376922607421875f,0.276600015425682244085692218504846096038818359375f,-0.49759999578595159874794262577779591083526611328125f,0.159999974071979522705078125f,1.2418999759614468558766020578332245349884033203125f,-0.3395000039339066688626189716160297393798828125f,-1.028699983294308228920499459491111338138580322265625f,0.575899984164535805319928840617649257183074951171875f,0.33620000814199446637076107435859739780426025390625f,-0.21190001738667518793590716086328029632568359375f,0.1887999857485294086245630751363933086395263671875f,0.0104000051975248197777546010911464691162109375f,0.07539999797046181839732525986619293689727783203125f,-0.1666999803662303492046703468076884746551513671875f,-0.198399997352063650613018808144261129200458526611328125f,0.442699996452033506244561067433096468448638916015625f,-0.3807000001549720735738446819595992565155029296875f,-0.440899973994493876716660452075302600860595703125f,1.49240000630021096839072924922220408916473388671875f,0.210699994194507578271213787957094609737396240234375f,-0.47610001354813580309865983508643694221973419189453125f,0.1124999986588954836719267404987476766109466552734375f,0.0569000037521123491757180090644396841526031494140625f,-0.4061000056505204014456467120908200740814208984375f,0.01759999988377103363745845854282379150390625f,-0.5576000172436239576967409448116086423397064208984375f,-0.0983999909818173978948152580414898693561553955078125f,0.0119999978244305083308063331060111522674560546875f,0.359900006449222598092774205724708735942840576171875f,0.0829000120520593242190443561412394046783447265625f,0.777399985699355422497092149569652974605560302734375f,0.3710000058412550316688793827779591083526611328125f,0.115300013323128336395484438980929553508758544921875f,-0.27369999890327445513094062334857881069183349609375f,0.45190001083016395622138361432007513940334320068359375f,-0.322300000269711006684048015813459642231464385986328125f,-0.526700033228099773197072863695211708545684814453125f,0.555500004410743652982773710391484200954437255859375f,0.14940001157522198038662963881506584584712982177734375f,-0.04140000060796733549750570091418921947479248046875f,0.155900010800361865648255843552760779857635498046875f,-0.836300018084049412436797865666449069976806640625f,-0.0718000145599244632421687128953635692596435546875f,0.44260000092685214401200255451840348541736602783203125f,0.215200003083050095398220946663059294223785400390625f,-0.449200000035762769812208716757595539093017578125f,0.267499997541308598414389052777551114559173583984375f,0.44169999154210071168336071423254907131195068359375f,-0.092599994966387821904163502040319144725799560546875f,-0.4767000016748905721897244802676141262054443359375f,-0.39579999333322046783933956248802132904529571533203125f,-0.66350000889599325404333285405300557613372802734375f,0.0247999744534492805314584984444081783294677734375f,0.968500000476836930829449556767940521240234375f,0.28989998148977758063438159297220408916473388671875f,0.5974999896436923307874167221598327159881591796875f,-0.75839998457431789802285493351519107818603515625f,1.386399962908029692698619328439235687255859375f,0.85360000088214871283298634807579219341278076171875f,-1.08609997793436008350909105502068996429443359375f,0.46059997695088394920048813219182193279266357421875f,0.30920002596378282078148913569748401641845703125f,0.502899997746944382726042022113688290119171142578125f,-0.214599993014335410634885192848742008209228515625f,0.9691999937213957583281853658263571560382843017578125f,-0.2248999999858438680888639282784424722194671630859375f,-1.3071000018060205949410601533600129187107086181640625f,0.2709999782741068230507153202779591083526611328125f,-0.1049999892711639404296875f,0.72919998245239270318052149377763271331787109375f,0.922299977284669836308239609934389591217041015625f,0.1694000029325479772523976862430572509765625f,-0.220499995507299928476641071029007434844970703125f,-0.65349997629970335566440553520806133747100830078125f,0.852399984246492437023334787227213382720947265625f,-0.341200009930133774105343036353588104248046875f,-0.401199981021881146858731881366111338138580322265625f,0.675500004559754785304903634823858737945556640625f,0.595700011628866210600108388462103903293609619140625f,0.0031000007182360178603630629368126392364501953125f,0.50520002167224908884435308209504000842571258544921875f,-0.26760000405609607465606814002967439591884613037109375f,0.29580000763833524946022635049303062260150909423828125f,-0.162600000293552870989088887654361315071582794189453125f,-0.14409998746737862074240865695173852145671844482421875f,-0.19429998865053033085814604419283568859100341796875f,0.30039999301582565038870598073117434978485107421875f,0.5744000164926053031422270578332245349884033203125f,0.062999998152255987093894873396493494510650634765625f,-0.8478000164151193729367150808684527873992919921875f,-0.0053000059470533766869948522071354091167449951171875f,0.18530000211000441368014435283839702606201171875f,-0.74600000494718532451088321977294981479644775390625f,0.069199997353553754919630591757595539093017578125f,-0.0651999952226878232153239878243766725063323974609375f,-0.6620000167489052955716033466160297393798828125f,-0.4022999912917615272789362279581837356090545654296875f,-0.6597000098109244703437070711515843868255615234375f,0.2840000229924921626434297650121152400970458984375f,-0.939200031626224784986334270797669887542724609375f,-0.2530000062733888110955149386427365243434906005859375f,-0.063800011229515174449034020653925836086273193359375f,0.41329999120980520643797717639245092868804931640625f,0.3354999987483022749756855773739516735076904296875f,-0.40959999543577418901207920498563908040523529052734375f,-0.0044000014424323552475470933131873607635498046875f,0.14830000111907726267901352912303991615772247314453125f,0.0821000018060207281678231083787977695465087890625f,0.32400002072006495179579133036895655095577239990234375f,1.193899991890788214732310734689235687255859375f,0.78990000730603926459849617458530701696872711181640625f,0.201100002856552606544937589205801486968994140625f,0.48739999789968135868178933378658257424831390380859375f,0.0751999978862702977977505724993534386157989501953125f,-0.3316000076770782012403060434735380113124847412109375f,0.75669998496696422307650209404528141021728515625f,-0.13900000622868535771203823969699442386627197265625f,-0.09939996880888912755835917778313159942626953125f,0.96289997107386593455657930462621152400970458984375f,-0.14970001308918057247865363024175167083740234375f,0.05010000159740446434852856327779591083526611328125f,-0.52159998681545260978964506648480892181396484375f,-1.4606000170350075251235466566868126392364501953125f,1.0518000251770018849128973670303821563720703125f,0.58610000818371776887261148658581078052520751953125f,-0.06190005240440399347789934836328029632568359375f,0.3187000145494938152523900498636066913604736328125f,-0.3948000558137891857768408954143524169921875f,1.3473000172197817647656847839243710041046142578125f,-0.2027000294208534114659414626657962799072265625f,0.1265000239610671872725333741982467472553253173828125f,-0.8349000286519530078521711402572691440582275390625f,0.09359998434185978766208791057579219341278076171875f,0.20410004433393424960740958340466022491455078125f,-0.704799996954202701004987829946912825107574462890625f,0.452800003898143899050410254858434200286865234375f,0.064299995970725998262196299037896096706390380859375f,0.087000013917684526632001507095992565155029296875f,0.67770002435445775290645542554557323455810546875f,0.0761999962210655468197728623636066913604736328125f,0.34489999464005205442873602805775590240955352783203125f,0.1375000122189522500093517010100185871124267578125f,0.1374000024631618632753315978334285318851470947265625f,0.20460000873506078988839362864382565021514892578125f,0.2574999849312009647661625422188080847263336181640625f,0.7073000013314185707713477313518524169921875f,-0.57230002481937425340419167696381919085979461669921875f,0.08629999513626085416717614862136542797088623046875f,-0.21390000466853376082099202903918921947479248046875f,0.3218999992072582028157512468169443309307098388671875f,0.1938999989315866034900182057754136621952056884765625f,-0.1794999939501285002307895410922355949878692626953125f,-0.0204000178635121987014144906424917280673980712890625f,0.106800013554096295820272644050419330596923828125f,0.3496000053450465383519940587575547397136688232421875f,0.1580000078380108252673608149052597582340240478515625f,-0.19260001113414781936938879880472086369991302490234375f,-0.048599996225535846416931917701731435954570770263671875f,-0.8148000168100002493787314961082302033901214599609375f,0.0407999928116797860866427072323858737945556640625f,-0.465299991716444427680698936455883085727691650390625f,-0.4707000111818313570211103069595992565155029296875f,-0.71070001814812411300437133832019753754138946533203125f,-0.06690001515150090227734835934825241565704345703125f,0.308700013171136322664978024477022700011730194091796875f,-0.337500001788139325498150355997495353221893310546875f,-0.31980000932216656739370819195755757391452789306640625f,0.0822000019133091175405070316628552973270416259765625f,0.119300008450448302710356074385344982147216796875f,0.5907000028371811595206963829696178436279296875f,0.298699982810020525647587419371120631694793701171875f,-0.013999990582465926536315237171947956085205078125f,-0.098599998646974551519406304578296840190887451171875f,0.156400005152821819365271949209272861480712890625f,-0.4907000062644482074603047294658608734607696533203125f,0.2788000499166543022511177696287631988525390625f,-0.8764000155091284938180251629091799259185791015625f,0.19489999333620033894476364366710186004638671875f,-0.72629998731315137394659586789202876389026641845703125f,0.33739999773204321087405332946218550205230712890625f,0.60969998276531700565072924291598610579967498779296875f,-0.43799999316036686280995127162896096706390380859375f,0.48329997209906583321981088374741375446319580078125f,-1.35509996702671031698628212325274944305419921875f,-0.04210001514256000998415174763067625463008880615234375f,0.71030002751648435577180862310342490673065185546875f,-0.0032000135101377391588783893894287757575511932373046875f,-0.4150000008009373519968221444287337362766265869140625f,-1.4544999932050703961294857435859739780426025390625f,0.911099950814246994923450984060764312744140625f,-0.19029999078512194188306239084340631961822509765625f,0.15719996927976609413235564716160297393798828125f,-0.79770000200271606871638141456060111522674560546875f,-0.0300000080466276131119229830801486968994140625f,0.44989999467730523718955737422220408916473388671875f,0.567399987822771123546772287227213382720947265625f,-0.522000002816319419451929206843487918376922607421875f,0.1077999902635813356255312100984156131744384765625f,-0.325900005435943551646005289512686431407928466796875f,-0.16759999913871281851385219852090813219547271728515625f,0.1813000097021462142521386340376920998096466064453125f,0.2146999946117400526190976961515843868255615234375f,-0.03049999707192163622693215074832551181316375732421875f,-0.39430001951456095099501908407546579837799072265625f,0.1751000230178234584599294976214878261089324951171875f,-0.54949999767541868322950904257595539093017578125f,0.08970000575035819156965999354724772274494171142578125f,-0.186899999617040124544331547440378926694393157958984375f,0.0557000065997244053761505711008794605731964111328125f,0.145899999177455885757126452517695724964141845703125f,-0.456900000362098601680571618999238125979900360107421875f,-0.227099992716312204521500461851246654987335205078125f,0.348400001524388602813786519618588499724864959716796875f,-0.200799998474121110803025658242404460906982421875f,-0.2734000051751730442362031681113876402378082275390625f,0.6442000095725060049289822927676141262054443359375f,-0.65080002462565922360937520352308638393878936767578125f,0.254800003361701943305206441436894237995147705078125f,-0.2070999889537692428831405777600593864917755126953125f,-0.5669000060990452283249396714381873607635498046875f,-0.4522999985560776092796686498331837356090545654296875f,-0.512500010430812835693359375f,-0.1351999980911611221046086939168162643909454345703125f,0.38360000974833974307642847634269855916500091552734375f,-0.39429999898821110893010200015851296484470367431640625f,-0.1313000063121319005432496851426549255847930908203125f,-0.110400006650388349260794029760290868580341339111328125f,0.11400000406801702457215696995262987911701202392578125f,0.452599998393654612272030135500244796276092529296875f,0.14859998847693223211763324798084795475006103515625f,-0.384899979329109243053608224727213382720947265625f,0.8891999940752981501645990647375583648681640625f,-0.1644000108301639784258441068232059478759765625f,1.12389999434947895906589110381901264190673828125f,0.30149999833106999602705400320701301097869873046875f,0.98459996201992083797449595294892787933349609375f,0.49709999642297619804054420455940999090671539306640625f,-0.4571999913148581118349511598353274166584014892578125f,-0.5094999981224537233259752611047588288784027099609375f,0.9694000087440013402328986558131873607635498046875f,-0.35899999952316274942631935118697583675384521484375f,0.033299960178137411048737703822553157806396484375f,0.4517999841988087350586056345491670072078704833984375f,0.3793999939918517583237189683131873607635498046875f,-1.1811000041604042820608810870908200740814208984375f,0.493900050377845900584361515939235687255859375f,-1.1565000159144400715405254231882281601428985595703125f,0.2410999992430209459826073725707828998565673828125f,0.224399997866153722014814775320701301097869873046875f,0.148899974270165103007457219064235687255859375f,0.03750000983476642346658991300500929355621337890625f,-0.22280004830360411460787872783839702606201171875f,-0.907900010077655306162114356993697583675384521484375f,0.822900037793815375408712498028762638568878173828125f,-0.50849999652802946314977816655300557613372802734375f,0.3307999900549649652248263009823858737945556640625f,-0.334300008524954284094832246410078369081020355224609375f,0.195700003768503716372606504592113196849822998046875f,-0.0828999971508979083267831811099313199520111083984375f,-0.328800002586841610963119819643907248973846435546875f,-0.2635000334084036666126849013380706310272216796875f,0.425500015139579801370928180404007434844970703125f,-0.650600001649558112859494940494187176227569580078125f,-0.0226000284194949330185409053228795528411865234375f,0.4347000007212165062497888357029296457767486572265625f,0.12329999430179583441713475622236728668212890625f,0.1038000000163912250883413435076363384723663330078125f,0.184199999402463399089668882879777811467647552490234375f,-0.2319000098615886518160777995944954454898834228515625f,0.2571000153660776277320110239088535308837890625f,-0.0477999998375773227365925777121447026729583740234375f,0.155800000374019143034587386864586733281612396240234375f,-0.053199999856948954057855871724314056336879730224609375f,0.355600012266636067437275414704345166683197021484375f,0.1172000084325670599127988680265843868255615234375f,-0.14959999860227102264076393112190999090671539306640625f,-0.18600000409036876458657161492737941443920135498046875f,0.0888000021398067662659059351426549255847930908203125f,0.2361000080719591609490493056000559590756893157958984375f,0.0374999998509883969433076345012523233890533447265625f,0.24850001515448105759986674456740729510784149169921875f,-0.1081000012397765885907574556767940521240234375f,-0.16389998160898688883690965667483396828174591064453125f,-0.61730000097751602794460268341936171054840087890625f,0.6063000197231769305972193251363933086395263671875f,0.34050001037120836144822533242404460906982421875f,-0.12840000606924306669753832466085441410541534423828125f,-0.1127999983847141063364460933371447026729583740234375f,-0.285500009328127024588184212916530668735504150390625f,-0.0550000201165676827486095135100185871124267578125f,-0.1556000286579131142161713796667754650115966796875f,-1.2842999673604964527839911170303821563720703125f,-1.25819997437596331479880973347462713718414306640625f,1.05999998092651281922371708787977695465087890625f,0.96190001500248900612177749280817806720733642578125f,-1.20819997988939320521240006200969219207763671875f,-0.38929999490901823211430610172101296484470367431640625f,0.7524999928288160777611892626737244427204132080078125f,0.14210000508427622634144427138380706310272216796875f,-1.49779999525546969607603386975824832916259765625f,-0.40549999619647858484228208908461965620517730712890625f,0.4351000011317431859225735024665482342243194580078125f,-0.24979998622536658814397014793939888477325439453125f,-0.351800000292062264861669973470270633697509765625f,0.1731999998942017615632948945858515799045562744140625f,0.25599999224394576202712414669804275035858154296875f,-0.37080000436007976016838938448927365243434906005859375f,0.12400000555813317504316728445701301097869873046875f,0.337700009080767582503312951303087174892425537109375f,-0.1837999923795461398867701063863933086395263671875f,0.138800003182887987662752493633888661861419677734375f,0.602100003147125306668385746888816356658935546875f,0.28549999502301215414945545489899814128875732421875f,0.239199980068207196381990797817707061767578125f,0.322000031948089571187665569595992565155029296875f,0.214299988818169140358804725110530853271484375f,0.14229999196231368774334669069503434002399444580078125f,-0.42490000369250768841311582946218550205230712890625f,0.1939999815113842840386126908924779854714870452880859375f,-0.2944000041246415122486723703332245349884033203125f,-0.3442999993607400721629119288991205394268035888671875f,-0.03010000845193872009986080229282379150390625f,-0.0210000036805867151823434824109426699578762054443359375f,0.0857999985486268545997745604836381971836090087890625f,-0.352300027687847627788642057566903531551361083984375f,0.877900008475780424532786128111183643341064453125f,0.527999992303550325090100159286521375179290771484375f,0.5412000036716460016350538353435695171356201171875f,0.617299977247416808978641711291857063770294189453125f,0.4267000012278554521572004887275397777557373046875f,0.20360000408589840059647713133017532527446746826171875f,0.0916000036537647499557124319835565984249114990234375f,0.1742000273421411282726012359489686787128448486328125f,-0.24260001441240319763892330229282379150390625f,-0.2728000254675746472088349037221632897853851318359375f,0.2116000049948691952295121154747903347015380859375f,-0.0372999725535512194340981295681558549404144287109375f,-0.337099995398521201650510192848742008209228515625f,-0.3265999993324275596506822694209404289722442626953125f,0.7513999931573867030465407879091799259185791015625f,0.303400000220537091610850666256737895309925079345703125f,-0.33220000496804724665622643442475236952304840087890625f,0.4241000321149825946775990814785473048686981201171875f,-0.492499999403953392373978203977458178997039794921875f,-0.2702000335559251453787510399706661701202392578125f,0.6338000010967252961791018606163561344146728515625f,0.55690002904683388873507965399767272174358367919921875f,-0.19690000501871107729812138131819665431976318359375f,0.47280001064091969897873468653415329754352569580078125f,-0.75060000660419490259300800971686840057373046875f,-0.96350000405311586604284457280300557613372802734375f,1.25939999876022312719214824028313159942626953125f,0.0201000001817941420512170225265435874462127685546875f,-0.285299988549947869387324317358434200286865234375f,-1.341700005698204023474318091757595539093017578125f,0.1348000355482099621440283954143524169921875f,0.07619999905228613812369076185859739780426025390625f,0.11620003108978238515192060731351375579833984375f,0.38989998290538796510418251273222267627716064453125f,-0.09039998016357486676497501321136951446533203125f,0.00839999746382236178732227926957421004772186279296875f,0.34160000849664218236512169823981821537017822265625f,-0.21549998712539686351874479441903531551361083984375f,1.47190000472068760473121074028313159942626953125f,0.190499985933303861429521930404007434844970703125f,-1.6194999753236771766751189716160297393798828125f,-0.28529999704360964329907801584340631961822509765625f,1.25099998348951402249440434388816356658935546875f,-1.321399954116344588328502140939235687255859375f,0.4304000069856641630394733510911464691162109375f,0.552099980534613177240998993511311709880828857421875f,0.31509999783486097868490105611272156238555908203125f,0.57050001007318495993558826739899814128875732421875f,0.946599968487024767682669335044920444488525390625f,-1.08180000684857358095314339152537286281585693359375f,0.30480000038146926044646534137427806854248046875f,0.73330001605749117743471288122236728668212890625f,0.31660001095533374382284819148480892181396484375f,0.256100016677379471730091609060764312744140625f,0.330400001621246275362864253111183643341064453125f,-0.246200029525161401267041583196260035037994384765625f,0.3326000197768212984783531283028423786163330078125f,-0.5242000150859358331700832422939129173755645751953125f,0.416200013208389396623942957376129925251007080078125f,-0.166099987638741952711285421173670329153537750244140625f,-0.330699995926767409404334330247365869581699371337890625f,0.243300002460181463703747795079834759235382080078125f,-0.7112999978184699756411646376363933086395263671875f,0.58250000782310973335142989526502788066864013671875f,0.3787000057578087108822728623636066913604736328125f,0.16789999715089776177734393058926798403263092041015625f,0.25690001742392798433911593747325241565704345703125f,0.4030999963223933502121099081705324351787567138671875f,0.514900011813640645641498849727213382720947265625f,-0.046700025293230862377669154739123769104480743408203125f,0.446299996030330436980193553608842194080352783203125f,-0.2873999848052861683100900336285121738910675048828125f,-0.5225999936252829769500749534927308559417724609375f,-0.355700012075901295105495591997168958187103271484375f,0.17930001504421255731358542107045650482177734375f,-0.4225999992877242927846737075014971196651458740234375f,-0.1102000027477740839998432420543394982814788818359375f,-0.209900020456313896044520106443087570369243621826171875f,0.472699999618530153355777656543068587779998779296875f,-0.2945000135451556477761414498672820627689361572265625f,-0.04030000054538251941238513609278015792369842529296875f,0.5935000057667492168178569045267067849636077880859375f,0.3461000107169152073538498370908200740814208984375f,0.3107999864041806059589134747511707246303558349609375f,-0.90799999070167558556931908242404460906982421875f,-0.03430001310706121131488544051535427570343017578125f,-1.30730001289844466327849659137427806854248046875f,-1.4297000184535979538935634991503320634365081787109375f,0.8473000130474570568139824899844825267791748046875f,-0.1017999981872736992016825752216391265392303466796875f,0.478899968831241640287998961866833269596099853515625f,0.99669995741844186198932220577262341976165771484375f,-0.02399999952316278495345613919198513031005859375f,0.44710001789927478199615507037378847599029541015625f,-0.20959998645782462745046359486877918243408203125f,0.352800000172853511770654222345910966396331787109375f,-0.8724999927356833495650789700448513031005859375f,0.04810000347495080319504268118180334568023681640625f,-0.10639998064041122916023596189916133880615234375f,-0.13060000675320626584152705618180334568023681640625f,-0.792099966639280950175816542468965053558349609375f,0.18930000103712085746110460604541003704071044921875f,-0.0873000348776571399866952560842037200927734375f,0.0492000033140183035129666677676141262054443359375f,-1.4637000054597848475168575532734394073486328125f,-0.05009999347627169807850577853969298303127288818359375f,-0.80749999277293671440247635473497211933135986328125f,-0.28609999998807911225640054908581078052520751953125f,-0.949599965745210994327862863428890705108642578125f,-0.001000004388391928511481410168926231563091278076171875f,-0.715900007782876546258421512902714312076568603515625f,0.1859000070750713717870894470252096652984619140625f,0.468999999523163069170550443232059478759765625f,0.9692999678075313596536943805404007434844970703125f,-0.40730001379251490334354457445442676544189453125f,-0.4936999916017048573024794677621684968471527099609375f,0.0868999677658077729347496642731130123138427734375f,0.3339000078350304878682663911604322493076324462890625f,-0.459899998700618706237719379714690148830413818359375f,0.5125000248476865305491401159088127315044403076171875f,-0.390100004279613443713969900272786617279052734375f,0.001100001254677795969882936333306133747100830078125f,-0.64550001752376573449510033242404460906982421875f,0.0922999918133020624821938326931558549404144287109375f,0.6169000169396401389576567453332245349884033203125f,0.1927000037536024112938548569218255579471588134765625f,0.282500013560056828509914339520037174224853515625f,0.689300010834634324652370196417905390262603759765625f,0.02350000733137136421646573580801486968994140625f,-0.3077000075533984357889494276605546474456787109375f,-0.009100012147426728148502661497332155704498291015625f,0.34630003432631506488093009465956129133701324462890625f,-0.44350000084936602728902244052733294665813446044921875f,0.15469999885857088628426936338655650615692138671875f,0.18010000606775289355709901428781449794769287109375f,-0.0439999886080622015782637390657328069210052490234375f,-0.2764000101447106061414160649292171001434326171875f,0.2324000099509955230558944094809703528881072998046875f,-0.336799996753036900987154922404442913830280303955078125f,0.1092000009298323970341471067513339221477508544921875f,-0.2964000029921531886856200799229554831981658935546875f,0.08700000162422638627646165332407690584659576416015625f,-0.331800001633167251924305674037896096706390380859375f,-0.60300001081824350368520981646724976599216461181640625f,0.3028000095605847263868781737983226776123046875f,-1.218799958777427860923125990666449069976806640625f,1.1670999906301489090765244327485561370849609375f,1.0555999835073948123209675031830556690692901611328125f,-0.4444999990165232617300716810859739780426025390625f,1.33330001844167700397747466922737658023834228515625f,-0.72140000507831558707039221189916133880615234375f,0.7964999984800815724383937777020037174224853515625f,-0.3824000296950345045843278057873249053955078125f,-0.53470001532435418312161345966160297393798828125f,0.1184000507354738829235429875552654266357421875f,-0.21190000561475752505202763131819665431976318359375f,0.39200001689791719172717421315610408782958984375f,0.9684999983906745768535984097979962825775146484375f,-0.3141999836444853144712396897375583648681640625f,0.0816000225782393950879622934735380113124847412109375f,-1.0017000147879127069927562843076884746551513671875f,0.6218000170931219372505438514053821563720703125f,0.51569998082071588640928894164972007274627685546875f,-0.034699995356798130075048902654089033603668212890625f,0.114399991309642956593961571343243122100830078125f,-0.3289999972879886769305812777020037174224853515625f,1.1997999810099599926616065204143524169921875f,0.8861999872244894493888978104223497211933135986328125f,-0.6093999836541710291015760958543978631496429443359375f,0.61419999549090864565670244701323099434375762939453125f,-0.42940001045763498410678948857821524143218994140625f,0.5329999971091747834606167089077644050121307373046875f,-0.2344000242412089818344611558131873607635498046875f,-0.4693000163853167094885066035203635692596435546875f,0.39500000059604634117249588598497211933135986328125f,0.183099997905641631579243266969569958746433258056640625f,0.353300014027208231226495627197436988353729248046875f,0.6291000064104792510732977461884729564189910888671875f,0.238500017762184324254803868825547397136688232421875f,-0.23700002863258118335210156146786175668239593505859375f,0.5430000012814997223387081248802132904529571533203125f,0.59579999713301656782959980773739516735076904296875f,0.2754000001311300849238250521011650562286376953125f,0.09680000960528845066477288128226064145565032958984375f,-0.06079999519586554157513091922737658023834228515625f,0.1549999943748117114949991446337662637233734130859375f,0.3309999936968086586830395390279591083526611328125f,-0.7903000104546549220430051718722097575664520263671875f,0.0883000032424927638885492342524230480194091796875f,-0.17200001216679827731326213324791751801967620849609375f,0.222400000004470299597159055338124744594097137451171875f,-0.27940002256482887599275954926270060241222381591796875f,0.19660001631975188018941480549983680248260498046875f,0.05919999049901980558985314928577281534671783447265625f,0.17119999979734412676179999834857881069183349609375f,-0.55330000611096608054140233434736728668212890625f,-0.13790000996589668602609890513122081756591796875f,0.072100001619756159954022223246283829212188720703125f,-0.0180999930813907639048920827917754650115966796875f,-0.45860000684261326142632242408581078052520751953125f,0.0194000038266182173174456693232059478759765625f,0.0277000110551715028606167834368534386157989501953125f,-0.4669000036776065343246955308131873607635498046875f,0.164100003132224170343533842242322862148284912109375f,-0.44439996329546005426891497336328029632568359375f,0.0177000033438206028080230680643580853939056396484375f,-0.294399993004649473959943861700594425201416015625f,0.63649999922513966765791337820701301097869873046875f,-0.202399993783235032651646179147064685821533203125f,-0.45760000599324701653358715702779591083526611328125f,-0.7937999727100137903335053124465048313140869140625f,0.17529999391436579259107020334340631961822509765625f,0.64990001389980278645452926866710186004638671875f,-0.15300000590085982565824451739899814128875732421875f,0.7334000006675722715954179875552654266357421875f,-0.38269998799562454649958453956060111522674560546875f,0.5638999972552056760832783766090869903564453125f,-1.420400005197524961886301753111183643341064453125f,0.8008999809980394957165117375552654266357421875f,-0.5772999992452561013323020233656279742717742919921875f,0.3342999884270130284136257614591158926486968994140625f,-0.2043000195145605601965144160203635692596435546875f,-0.73050000411272009159802109934389591217041015625f,0.3279999945759772828068889793939888477325439453125f,-0.93119996850490593942595296539366245269775390625f,-0.502999997109174756815264117904007434844970703125f,-0.101500007271766889971331693232059478759765625f,0.269599994802474984822993064881302416324615478515625f,-0.36189999320954058248389628715813159942626953125f,-1.0706000295519828569013043306767940521240234375f,1.078800021660327956851688213646411895751953125f,0.65530000091791151817233185283839702606201171875f,-0.378600015336274964283802546560764312744140625f,0.0145999953985214148133309208787977695465087890625f,0.1148000041067600562882944359444081783294677734375f,0.418800000090896684667995941708795726299285888671875f,-0.17760000100135808764889588928781449794769287109375f,-0.6991000081613660821489020236185751855373382568359375f,0.087100002551078858914479496888816356658935546875f,-0.385300009299814671948070099460892379283905029296875f,0.5005000048577785687342611709027551114559173583984375f,0.18179999999403950727838719103601761162281036376953125f,0.257400007790327123302631662227213382720947265625f,0.00889999699443588809799621230922639369964599609375f,0.3727000169038774401997216045856475830078125f,0.4466999969065186260053224032162688672542572021484375f,0.0537000102281570690365697373636066913604736328125f,0.1678000165641311269837387953884899616241455078125f,-0.7654000168204309062502943561412394046783447265625f,0.2666999916911125811935789897688664495944976806640625f,0.543300012704730050927537376992404460906982421875f,0.3622000118970871707091419011703692376613616943359375f,-0.4270000081062317320856891456060111522674560546875f,0.1000000054016709771786963756312616169452667236328125f,-0.1468000013351440369291367460391484200954437255859375f,-0.61130002650320569568265227644587866961956024169921875f,0.02450000363588333840425548260100185871124267578125f,0.0293999969720839526843292333069257438182830810546875f,0.76980001081228266457401332445442676544189453125f,-0.004800009620189726877725888698478229343891143798828125f,0.436800006699562093359645587042905390262603759765625f,0.19760001631230128538163626217283308506011962890625f,-0.770000021755695485126125277020037174224853515625f,0.3776999869897961215059467576793394982814788818359375f,-0.97739999557137480223900638520717620849609375f,0.181999990671873224101773303118534386157989501953125f,1.119999963939189768780124722979962825775146484375f,-1.29540000981688496750621197861619293689727783203125f,0.77549999785423295861619408242404460906982421875f,-1.247799998012184996554196914075873792171478271484375f,0.64330001508891587747029916499741375446319580078125f,-0.119000003397464748644551946199499070644378662109375f,0.210000015646219395648586214520037174224853515625f,0.222599996381998010974712087772786617279052734375f,-1.265699955749511218527914024889469146728515625f,0.11479997236728678444706019945442676544189453125f,0.8507000082015991893058526329696178436279296875f,0.83369998266100875383699531084857881069183349609375f,0.24670001106262162693383288569748401641845703125f,-0.2040000159144401692401515902020037174224853515625f,0.01910005066692832542685209773480892181396484375f,1.1456000101804733048993512056767940521240234375f,-0.6370000182390214149563689716160297393798828125f,0.30659999970495699361805463922792114317417144775390625f,-0.09260000018179415093300121952779591083526611328125f,-0.57810001599192606391852677916176617145538330078125f,-0.4406000194191932450848980806767940521240234375f,-0.86729997102618217041936077293939888477325439453125f,0.35689999458789856134899309836328029632568359375f,-0.3209999937713146067608249722979962825775146484375f,-0.235399991637468275484934565611183643341064453125f,1.1677000207781791374372915015555918216705322265625f,-0.4023000480651859334102482534945011138916015625f,0.85429999269247058890641710604541003704071044921875f,0.19159999903440461110903925145976245403289794921875f,0.416600011029839567555654866737313568592071533203125f,0.1018000041663646992873282215441577136516571044921875f,-0.42879999468922636030043804566957987844944000244140625f,-0.39199999976158128589531770558096468448638916015625f,0.2663000146567824355514630951802246272563934326171875f,-0.471500019490719068926409818232059478759765625f,-0.3160000115782024554533791160793043673038482666015625f,0.607600016908347750899110906175337731838226318359375f,-0.06780001928359258300815781694836914539337158203125f,0.3953999974489210700312469271011650562286376953125f,-0.7761999912664290235397857031784951686859130859375f,0.0453999920845031823546378291212022304534912109375f,-0.17829998988732709275240040369681082665920257568359375f,-0.39049999701604232171803232631646096706390380859375f,-0.68240002086609619080803668111911974847316741943359375f,-0.12760000398159032641842713928781449794769287109375f,0.79380001093149221702560680569149553775787353515625f,-0.0378000140309335819210900808684527873992919921875f,0.46469999047666743496876051722210831940174102783203125f,-0.06869998579025260454500312334857881069183349609375f,0.06590000178515909023957419776706956326961517333984375f,0.1105000047087669745593530024052597582340240478515625f,0.1552999964475632044269559628446586430072784423828125f,0.27310000682771207181076533743180334568023681640625f,0.1848999987378714404684387773158960044384002685546875f,-0.444099995160102789526490596472285687923431396484375f,-0.69519999585598679647802100589615292847156524658203125f,0.0003999902963638390929190791212022304534912109375f,-0.22500002987682865551732902531512081623077392578125f,-0.721699974405765942719881422817707061767578125f,0.09759999809563157935343724602716974914073944091796875f,0.70380001324117191785489922040142118930816650390625f,-0.23029999927878375132195287733338773250579833984375f,0.1772999646186832478633732534945011138916015625f,0.22809999393820756807826910517178475856781005859375f,1.22959999778270656634049373678863048553466796875f,-0.6223999862022697726615660940296947956085205078125f,-0.782399993913620139807107989327050745487213134765625f,0.08900000920891759648156948969699442386627197265625f,-1.22759998371601142252984573133289813995361328125f,-1.49759999191164983045609915279783308506011962890625f,1.10220001428127289955227752216160297393798828125f,0.0998999952733516938252478212234564125537872314453125f,0.12639998099431437594830640591681003570556640625f,-0.53020001317858689304785002605058252811431884765625f,-0.4158000130772592228822759352624416351318359375f,1.51110003142952908916640808456577360630035400390625f,-1.10030000061988886272956733591854572296142578125f,0.85479997162222876028181417495943605899810791015625f,-0.478499998390674363690777681767940521240234375f,-0.25500000722706317457522118274937383830547332763671875f,0.50890001096427450733017394668422639369964599609375f,-0.093700008887052543826712280861102044582366943359375f,0.048000026836991338541338336654007434844970703125f,0.9418999726831913665847650918294675648212432861328125f,-0.6331999979197977523881490924395620822906494140625f,-0.801999972194433308203542765113525092601776123046875f,-0.122000014066696138570478069595992565155029296875f,0.099999990165233754169094027020037174224853515625f,0.41520000192821016060662486779619939625263214111328125f,-0.5942999850183723342667008182615973055362701416015625f,-0.190500007875263610568339345263666473329067230224609375f,-0.1849000133782621180245087089133448898792266845703125f,0.1740999924778936502889337134547531604766845703125f,0.242500010170042745727414512657560408115386962890625f,0.1030999985575675736981793306767940521240234375f,0.0386000100836156878614247034420259296894073486328125f,-0.647000005125999422261884319595992565155029296875f,-0.4948999965026974479087584768421947956085205078125f,-0.34030001075267790611178497783839702606201171875f,-0.0738000046357509820182940529775805771350860595703125f,-0.0378000030040741563652773038484156131744384765625f,-0.6260999821439383605792272646795026957988739013671875f,-0.45390000686645493033211096189916133880615234375f,0.06750000257045041252013106714002788066864013671875f,0.0548999869287012387530921841971576213836669921875f,-0.11219999543130398933499236591160297393798828125f,-0.47100001545250425039057518006302416324615478515625f,0.834400010718405749088333323015831410884857177734375f,0.21319999061822869634852395392954349517822265625f,0.0901000053226947983375794137828052043914794921875f,-0.01450000751018531985891968361102044582366943359375f,-0.38329998893737748932863951267790980637073516845703125f,-0.095500013649463699749730949406512081623077392578125f,0.21030000788420455393179508973844349384307861328125f,-0.28230001379251490334354457445442676544189453125f,-0.5867999865084885247057400192716158926486968994140625f,-0.05400000816583627738509676419198513031005859375f,-0.38300000262260436301176014239899814128875732421875f,-0.66450000602006920047415405861102044582366943359375f,0.59069999829232688881575086270458996295928955078125f,0.7796999713659289454881218262016773223876953125f,-0.8967000387042762810096974135376513004302978515625f,0.0546000073194505830542766489088535308837890625f,0.2166000147923827778839722668635658919811248779296875f,0.184499999165534944722821819595992565155029296875f,-0.67950002002716070048649044110788963735103607177734375f,-0.89580002052783935795332581619732081890106201171875f,-0.8933999897897242359334768480039201676845550537109375f,-0.1121999671936038112107780762016773223876953125f,-0.1644000004738568332385284520569257438182830810546875f,0.51779999525547015792881211382336914539337158203125f,0.5678999783009288648827350698411464691162109375f,0.4907999878942970184425576007924973964691162109375f,0.5297999775454400595009474272956140339374542236328125f,1.0097999753475193074336857534945011138916015625f,0.264899995534121968088214771341881714761257171630859375f,-0.154800017815828194756022639921866357326507568359375f,0.6972000211358071464218255641753785312175750732421875f,-0.284099987262487641004327088012360036373138427734375f,0.3194999992400406885195707218372263014316558837890625f,-0.25409998025894164896953952847979962825775146484375f,-0.85110001984387639861751040371018461883068084716796875f,-0.10920000867843615566243897774256765842437744140625f,-0.715799981077015257113771440344862639904022216796875f,-0.011099963033199600204170565120875835418701171875f,0.7806000283598901745563125587068498134613037109375f,-0.384900022393465324199723909259773790836334228515625f,0.647400007045269010319543667719699442386627197265625f,0.21479998130798261257723424932919442653656005859375f,-0.90429999351203438351376462378539144992828369140625f,-0.25050000627338864234161519561894237995147705078125f,0.474499997973442333432103623636066913604736328125f,0.12550000679492967492478783242404460906982421875f,0.726600007528066527129340101964771747589111328125f,0.3452000081121919805582365370355546474456787109375f,-0.575399975693226206630015440168790519237518310546875f,-0.89680000491142219942730662296526134014129638671875f,-0.74399997091293368356446080724708735942840576171875f,-0.50579999147057463915189146064221858978271484375f,-1.6624999472498895869421176030300557613372802734375f,-1.001300007206200604059631587006151676177978515625f,0.5996000091075899263159953989088535308837890625f,-0.158599998050927837311974144540727138519287109375f,-0.00160001333951953483847319148480892181396484375f,0.70620002214908605964183152536861598491668701171875f,-0.9616000033557412507434491999447345733642578125f,-0.72480004422813715247997379265143536031246185302734375f,0.51560000101625913515590582392178475856781005859375f,0.129800001648068263904178820666857063770294189453125f,0.7739999997466799896983502549119293689727783203125f,0.69980002921521677894389767971006222069263458251953125f,-0.039600040698051230947385192848742008209228515625f,-0.717600041234494057107440312393009662628173828125f,-0.41259997697323580467809733818285167217254638671875f,-0.543400000555812834335256411577574908733367919921875f,0.70970002083778371826383590814657509326934814453125f,0.015100007706880802516025141812860965728759765625f,0.04539999387264259667063015513122081756591796875f,0.89010000010728873576226760633289813995361328125f,-0.0074999977648257942774989714962430298328399658203125f,-0.4628999931275845636235999336349777877330780029296875f,0.31229999274462461045231975731439888477325439453125f,0.60149999177455892862553810118697583675384521484375f,0.0480000010952352784698859977652318775653839111328125f,0.204300006699562164413919163052923977375030517578125f,-0.9287999863073228556942240174976177513599395751953125f,-0.239699985820055250229643206694163382053375244140625f,-0.58799998041987400210217629137332551181316375732421875f,-0.59249998688697846915829359204508364200592041015625f,-0.7250999854668973565452461116365157067775726318359375f,-0.8489999628663067454681367962621152400970458984375f,0.13439999198019503268142216256819665431976318359375f,0.9393000040173529185949519160203635692596435546875f,0.5778000104919076562737245694734156131744384765625f,-0.4023000167727470710588022484444081783294677734375f,-0.70380002486407777784194195191957987844944000244140625f,-0.5957999942086633637927661766298115253448486328125f,-0.0258000011935830553255755148711614310741424560546875f,0.2493000026762486331932677785516716539859771728515625f,0.470400016522407593555499261128716170787811279296875f,0.19649999706447118796859285794198513031005859375f,-0.550800023657083581696269902749918401241302490234375f,0.4121999809026721806048954022116959095001220703125f,-0.617399991063773523336521975579671561717987060546875f,-0.26579998845309038557616077014245092868804931640625f,0.10410000238716599074706437022541649639606475830078125f,0.74610001965761174602675964706577360630035400390625f,0.170999994814395872566592515795491635799407958984375f,0.954400017535685929459532417240552604198455810546875f,0.5753000028550625533085849383496679365634918212890625f,0.16850000453740354711129612041986547410488128662109375f,-0.747699998128414478060221881605684757232666015625f,-0.766500023812055619742977796704508364200592041015625f,0.444299997162818982587850769050419330596923828125f,0.083700005310773928357548356871120631694793701171875f,0.9324999877810480342077426030300557613372802734375f,1.2423000280976292941659266944043338298797607421875f,-0.3720999900341031008110803668387234210968017578125f,-0.696300022852420763541658743633888661861419677734375f,-1.72139998958110851390301831997931003570556640625f,-0.88040001667141876140476597356610000133514404296875f,-1.01060001167058910453988573863171041011810302734375f,-0.6674000390827661277626248192973434925079345703125f,0.55289998761415493078175131813623011112213134765625f,0.44790000251531569830376611207611858844757080078125f,-0.388399991130828681207276531495153903961181640625f,0.19319999374747265363794213044457137584686279296875f,0.7298000036224721043254248797893524169921875f,0.8122000423327093709957580358604900538921356201171875f,0.0575999962702393730751282419078052043914794921875f,-0.1497999997481702649793078307993710041046142578125f,0.3027999885499479404415978933684527873992919921875f,0.2852000043869018508502222175593487918376922607421875f,0.357899984931945969179878375143744051456451416015625f,0.42270000408887842269223256153054535388946533203125f,-0.5706999734818936076408135704696178436279296875f,-0.7446999972939483125600190760451368987560272216796875f,-1.31739997932910934963501858874224126338958740234375f,0.544199976193905143873053020797669887542724609375f,0.1586999979346990219397639521048404276371002197265625f,0.455400003688782473165730380060267634689807891845703125f,-0.3962999936833977443484400282613933086395263671875f,0.01140002355575564507716990192420780658721923828125f,0.1778999979704618539244620478712022304534912109375f,0.48389999583959575346625570091418921947479248046875f,0.8618999974563716914843780614319257438182830810546875f,-0.0255000205039976890475372783839702606201171875f,-0.4563000167056918687791267075226642191410064697265625f,0.1827000064730643913435415015555918216705322265625f,-0.517799981322884406154116732068359851837158203125f,-0.8486999783396722563111325143836438655853271484375f,-0.69960002315193425825867734602070413529872894287109375f,-0.72180000759363149853697905200533568859100341796875f,0.46349999358505000746077939766109921038150787353515625f,0.36269999574422850940891294158063828945159912109375f,0.35339999280720935104227464762516319751739501953125f,-0.24419998602867121206827505375258624553680419921875f,0.86940002114921821618764852246385999023914337158203125f,0.4403999947294590100455025094561278820037841796875f,-0.066599999928474373778186645722598768770694732666015625f,0.20129999446570867593919729188201017677783966064453125f,0.5645999945417046195217380955000407993793487548828125f,0.491399993306398386039290926419198513031005859375f,0.488699994134902893083705066601396538317203521728515625f,0.250199989932775590517621822073124349117279052734375f,-0.526499991215765295038409021799452602863311767578125f,-0.0065999914348129351537863840349018573760986328125f,-0.63179999299049371952463616253226064145565032958984375f,0.148899983769654742360444288351573050022125244140625f,-0.33890001483857634667629099567420780658721923828125f,-0.060500005975365755972461556666530668735504150390625f,0.69379997934102988210725015960633754730224609375f,0.366600029879808886335013085044920444488525390625f,-0.56180000908374783108456540503539144992828369140625f,0.08199999186396578210178631707094609737396240234375f,1.057700015711784402583361952565610408782958984375f,0.898100043112040058446154944249428808689117431640625f,-0.08860001138746742554985758033581078052520751953125f,-0.2534000078946352374487105407752096652984619140625f,0.45740001538991936769207313773222267627716064453125f,-0.29780000747442247899243739084340631961822509765625f,-0.5625000017881394143159923260100185871124267578125f,0.00010000249147434914220866630785167217254638671875f,1.216999975621700258443524944595992565155029296875f,0.41840001735687248896056189551018178462982177734375f,-1.05989997247457523599223350174725055694580078125f,-0.6330999996751540681572123503428883850574493408203125f,0.3309000032007691771696045179851353168487548828125f,0.2831000200152400392283880137256346642971038818359375f,0.103099972331524014634851482696831226348876953125f,0.19529998571872653201353386975824832916259765625f,0.14619999756216994768465156084857881069183349609375f,0.04139999807476979487574908489477820694446563720703125f,-0.27969999058842665018431716816849075257778167724609375f,-0.39870000642836078696973345358856022357940673828125f,-0.1575999966800214480144859408028423786163330078125f,0.078200005072355249780002850457094609737396240234375f,1.7119000014424319289219056372530758380889892578125f,0.06240000078678153982991716475225985050201416015625f,0.0873000012382865175908364108181558549404144287109375f,-0.1212000053107737951307854018523357808589935302734375f,0.620400019986927730997194885276257991790771484375f,0.7913999812364576058598686358891427516937255859375f,0.20699999856948847476445507709286175668239593505859375f,-0.40289999864101400106619621510617434978485107421875f,0.567900010152161183185626214253716170787811279296875f,0.2705999899148938681037179776467382907867431640625f,-0.02340000007152553962441743351519107818603515625f,0.2711999981582164931381839778623543679714202880859375f,-0.41220000951290136637084060566849075257778167724609375f,0.39720000861883164589016814716160297393798828125f,0.2313000077649951524705329575226642191410064697265625f,-0.25349999749660501180414939881302416324615478515625f,0.370800002534687589417217168374918401241302490234375f,0.910499977290630368997881305404007434844970703125f,-0.60869998295903193774591954934294335544109344482421875f,-0.71519998341351775916763244822504930198192596435546875f,0.3357000157266856188442716302233748137950897216796875f,0.2607000029861925582963522174395620822906494140625f,0.399299989525973675341674606897868216037750244140625f,0.25529999465942410097341053187847137451171875f,-0.0947999996736645922368325045681558549404144287109375f,-0.0353999991625546517326483808574266731739044189453125f,-0.35969999878406522242357823415659368038177490234375f,-0.06809999330490834612561457106494344770908355712890625f,-0.240599992708861787349405858549289405345916748046875f,-0.534300002229213699678211924037896096706390380859375f,0.7281000311538579072845323025831021368503570556640625f,0.307900011605024559457888244651257991790771484375f,-0.2115000010132790375649847192107699811458587646484375f,0.507100018644333072614927004906348884105682373046875f,-0.273500026106834326355965458787977695465087890625f,-0.59850003311038069142568929237313568592071533203125f,0.03999997414648515103863246622495353221893310546875f,-0.222200012716651240651799525949172675609588623046875f,-1.050399966603517754037966369651257991790771484375f,-1.1320000097453586374030010119895450770854949951171875f,1.0920999956965449229073783499188721179962158203125f,0.831100016528367735446636288543231785297393798828125f,0.4462999936461446992552737356163561344146728515625f,-0.4611000053524971775686935870908200740814208984375f,0.48559999021291755383344934671185910701751708984375f,0.300599995428323385482372032129205763339996337890625f,0.931399990922212595023665926419198513031005859375f,0.31820000876039256088034790082019753754138946533203125f,-1.225999988555908526421944770845584571361541748046875f,-0.46379998962283064400935472804121673107147216796875f,-0.276199978637695409133812063373625278472900390625f,-0.581000003755092198076681597740389406681060791015625f,0.69909995909929278923300444148480892181396484375f,0.25389998496174737141473087831400334835052490234375f,0.018500002451240948886379555915482342243194580078125f,0.32490000782757988417870365083217620849609375f,0.8206999772071839061027276329696178436279296875f,0.483300011885166025393800737219862639904022216796875f,-0.46049996551871341665673753595910966396331787109375f,-0.7723000018715853176587415873655118048191070556640625f,0.64940000203847869642004297929815948009490966796875f,-0.567899987763166524956659486633725464344024658203125f,0.43890001156032087958891452217358164489269256591796875f,-0.4545000053495169556327937243622727692127227783203125f,-0.267900014063715996570635979878716170787811279296875f,0.3404999972581865819165614084340631961822509765625f,0.1098000021323561892216957858181558549404144287109375f,0.4934999988377095636593594463192857801914215087890625f,-0.8615999887526033518270196509547531604766845703125f,-0.2471999780714515537738407147116959095001220703125f,0.5228999897003172581122498741024173796176910400390625f,0.3512999870896340581793992896564304828643798828125f,-0.7166999990299343092914341468713246285915374755859375f,0.2858999899387359988622847595252096652984619140625f,0.34799998712539659262432678588083945214748382568359375f,0.39029999734163300928457829286344349384307861328125f,0.313199996951222414676152538959286175668239593505859375f,0.73319999309554706456992789753712713718414306640625f,-0.5291999763429162673133987482287921011447906494140625f,-1.030500004559755655719754940946586430072784423828125f,-0.4613999959141015239083571941591799259185791015625f,0.09240001360177974021326008369214832782745361328125f,0.59129998943656669752044763299636542797088623046875f,0.931799993884563804158460698090493679046630859375f,0.0405000040009617645608841485227458178997039794921875f,-0.42300000165402884544363359964336268603801727294921875f,0.55769999898523092696223102393560111522674560546875f,0.662099983179569306912526371888816356658935546875f,-0.88289998411238179709670248485053889453411102294921875f,-0.26849997559189819895664186333306133747100830078125f,0.81720001856535684492399695955100469291210174560546875f,-0.96700003157556058130950304985162802040576934814453125f,0.6704000081405043776072716354974545538425445556640625f,-0.057400001569092307107666783849708735942840576171875f,0.425299999353289603920558192839962430298328399658203125f,0.00390000243335963947544087204732932150363922119140625f,-0.53120000179037418774896650575101375579833984375f,-0.5184000226654112974955523895914666354656219482421875f,-0.0040999944150446054891290259547531604766845703125f,-0.2051000097930433252457760318065993487834930419921875f,0.8091999529480933261993413907475769519805908203125f,0.523400004243850247576119727455079555511474609375f,0.2025999986171722611061341012828052043914794921875f,0.33720000727772703186246872064657509326934814453125f,-1.644800022435187969449543743394315242767333984375f,-0.8596000639438639012723797350190579891204833984375f,0.4584000009655948559839089284650981426239013671875f,0.36860003515482020475246827118098735809326171875f,0.275300002110004271571597200818359851837158203125f,0.65290003604292934369368595071136951446533203125f,1.177199999082088055502026691101491451263427734375f,0.6010000281929972487660052138380706310272216796875f,0.301400016105174639591268714866600930690765380859375f,0.721600044929981976338240201584994792938232421875f,0.1073999888658523360618346487171947956085205078125f,-0.343400023466349235690131536102853715419769287109375f,-0.78829999996423705255210734321735799312591552734375f,-0.350100024843216228731535011320374906063079833984375f,0.05799999770522124009630715590901672840118408203125f,-0.1162000093340875839231784993899054825305938720703125f,0.645300021779537047450503450818359851837158203125f,-0.2981999875634908647725751507095992565155029296875f,-0.0174000052571297303405373213536222465336322784423828125f,0.68070000984072687177928173696273006498813629150390625f,0.021899998909235003641260419726677355356514453887939453125f,0.408899999285489312850216947481385432183742523193359375f,-0.270600016904622375779609910750878043472766876220703125f,-0.01860000138506290312534474651329219341278076171875f,-0.5187000116810203120820688127423636615276336669921875f,-0.2853000055372714172108317143283784389495849609375f,0.8262000020697712887596253494848497211933135986328125f,0.8960999815106394095209907391108572483062744140625f,0.1674000003397464464427457642159424722194671630859375f,-0.128700003969669296566280536353588104248046875f,-0.538800029334426167082483516423963010311126708984375f,-0.833399991726874844744088477455079555511474609375f,0.6390000280961396494916471056058071553707122802734375f,0.59370000039339032582574873231351375579833984375f,0.931500020943582196508714332594536244869232177734375f,0.3323999981045719920302872196771204471588134765625f,0.885000020675361209754328228882513940334320068359375f,1.184100014531612288237738539464771747589111328125f,0.646800030653178925632573736947961151599884033203125f,-0.069299996417761189348993866587989032268524169921875f,-0.5280000111535192441891695125377736985683441162109375f,0.0132000001549723577909389859996736049652099609375f,-0.17970001185983430769255164705100469291210174560546875f,-0.4028999902963636969843719271011650562286376953125f,-0.16440000695586209911169817132758907973766326904296875f,0.0405000003874302905160220689140260219573974609375f,0.202500008046627044677734375f,-0.4967000271558763557777638197876513004302978515625f,0.31899999088048947015749945421703159809112548828125f,-0.068400001710653413056206773035228252410888671875f,-0.09289999066889276235770012135617434978485107421875f,-0.23070000484883779190425912020145915448665618896484375f,-0.406799996138364150510824401862919330596923828125f,-0.57500002590939447433271425325074233114719390869140625f,-0.878900018489360945750377140939235687255859375f,-0.17630000780224808210050468915142118930816650390625f,-0.7213999803423882806185929439379833638668060302734375f,-0.4196000043392178469048303668387234210968017578125f,0.6579999778866769677421189044252969324588775634765625f,0.43460000404119458750074045383371412754058837890625f,0.7985999588608745369811003911308944225311279296875f,1.05480000723600308987215612432919442653656005859375f,-0.1733000056266782440417273392085917294025421142578125f,-0.668900034433603796202305602491833269596099853515625f,0.082499980777502146533919358262210153043270111083984375f,0.36950000531226379063554077220032922923564910888671875f,-1.612099985861778606022198800928890705108642578125f,-0.7284999989867202430104953236877918243408203125f,-0.3092000012278557363742947927676141262054443359375f,0.115400003856420685366401812643744051456451416015625f,-0.7800000140070917353796176030300557613372802734375f,0.08579999743103972065227935672737658023834228515625f,0.243599998646974569282974698580801486968994140625f,-0.31460000642538066273345975787378847599029541015625f,-0.515800001677870767480271751992404460906982421875f,-0.113800001022219543500568761373870074748992919921875f,-0.57299999099969856075631469138897955417633056640625f,-0.5855999836564063087962495046667754650115966796875f,-0.19860000654458997981777201857767067849636077880859375f,0.1694999854564667174372516456060111522674560546875f,-0.31350000520795584524336163667612709105014801025390625f,-0.09569999903738501956240725121460855007171630859375f,-0.86400000803545129457461371202953159809112548828125f,-0.4928999925501642564995563589036464691162109375f,-0.056099992686510002926070228568278253078460693359375f,-0.60570000820159908272444226895458996295928955078125f,-0.44339999358952042474157906326581723988056182861328125f,-0.5366999874442817120012705345288850367069244384765625f,0.38639999189078799712859790815855376422405242919921875f,0.3914999820142985864634965764707885682582855224609375f,0.7667999848321078726343102971441112458705902099609375f,0.024299965274334311970960698090493679046630859375f,-0.67530002177953729614046096685342490673065185546875f,0.0756000014632942640702140124631114304065704345703125f,0.353099999079108217703293348677107132971286773681640625f,-0.076500016007572335663411422501667402684688568115234375f,-0.407699976074695324967933629523031413555145263671875f,-0.8507999910235408691505654132924973964691162109375f,0.4320000063925981681478560858522541821002960205078125f,0.0339000068664550013863845379091799259185791015625f,0.2556000111490488446719382409355603158473968505859375f,-0.55349999839067454132646162179298698902130126953125f,-0.1380000032186508018838821953977458178997039794921875f,0.537300002169609047797393941436894237995147705078125f,-0.06059999639689916950402448492241092026233673095703125f,-0.3714000031411648361512334304279647767543792724609375f,-0.70380000742971893235022662338451482355594635009765625f,-0.03780000125318760950676733045838773250579833984375f,-0.53619996522665047677946859039366245269775390625f,-0.834800008577107899299107884871773421764373779296875f,-0.490400012648105843027224182151257991790771484375f,-0.727200027543306593003080706694163382053375244140625f,-0.0355999940872191444896088796667754650115966796875f,-0.54820000745654073881496515241451561450958251953125f,0.5516999879658224248402120792889036238193511962890625f,0.45810000291615693424063238126109354197978973388671875f,-0.688399988448619826186813952517695724964141845703125f,-0.862200029331445794156252304674126207828521728515625f,-0.398799999755620859787086374126374721527099609375f,0.100000012218952605280719581060111522674560546875f,0.314099989646673183329994571977294981479644775390625f,-0.22810000317692757931808955618180334568023681640625f,-1.137899999237060200130144949071109294891357421875f,-0.6983000503301628469898787443526089191436767578125f,-1.129600015664100620682575026876293122768402099609375f,-0.29100001567602173935256359982304275035858154296875f,0.077099988804757657590016606263816356658935546875f,0.157300001163780678314907390813459642231464385986328125f,-0.0092000003337862068519825697876513004302978515625f,0.4446000280320649977738867164589464664459228515625f,-0.99690000323057137432414265276747755706310272216796875f,-0.8481000525742776741111583760357461869716644287109375f,-0.59380000087320805590707095689140260219573974609375f,-0.190800010320544277675480770994909107685089111328125f,-0.34770000572800630056491399955120868980884552001953125f,-0.2093000100523234596749944103066809475421905517578125f,-0.252999974906444879962919003446586430072784423828125f,0.07319999948441979764623965820646844804286956787109375f,-0.79619999234676352983797187334857881069183349609375f,-0.1460999740600588125971626141108572483062744140625f,-0.3545999999061226759522469365037977695465087890625f,0.23580001248121273960123289725743234157562255859375f,-0.1812000039696692876844963393523357808589935302734375f,0.5817000042080877619810053147375583648681640625f,0.4310999924629925761365711878170259296894073486328125f,0.309599993591755773270079998837900348007678985595703125f,-0.7548000091731548177875765759381465613842010498046875f,-0.197699984270334194746965295053087174892425537109375f,0.2934000057712198294979089041589759290218353271484375f,-0.182400008833408566744083145749755203723907470703125f,-0.149700001354515499496500297027523629367351531982421875f,0.4592999922454357175638506305404007434844970703125f,-0.811500029101968056721716493484564125537872314453125f,-0.9191999899029728027244345867075026035308837890625f,-0.214200000594556294597481382879777811467647552490234375f,-0.843300009351968693493972750729881227016448974609375f,0.45749999921768902400032175137312151491641998291015625f,0.3542999983176589040567705524154007434844970703125f,0.7056000205367805033773720424505881965160369873046875f,0.12689999846219990331519511528313159942626953125f,-0.82410003077387827818967025450547225773334503173828125f,-0.58019999198168481502335680488613434135913848876953125f,0.0053999997586011705408282068674452602863311767578125f,-0.2528999928295612154016680506174452602863311767578125f,-0.196500004291534413614073173448559828102588653564453125f,-0.22860000404864544432115280869766138494014739990234375f,-0.9047999851450321262547049627755768597126007080078125f,0.18340002123117482568659397657029330730438232421875f,0.222299962830543762493107351474463939666748046875f,0.85530000732541022756549864425323903560638427734375f,-1.4674999764561658111006181570701301097869873046875f,-0.82650001591443977844164692214690148830413818359375f,1.2327000073671339208658537245355546474456787109375f,0.908300032895803521881816777749918401241302490234375f,-0.5149999819695951774889408625313080847263336181640625f,-0.70560000813156331656017528075608424842357635498046875f,0.5282999969840052045810807612724602222442626953125f,1.10830003170371060861043588374741375446319580078125f,-0.463599990153312713214717177834245376288890838623046875f,-0.42700001067668191812032318921410478651523590087890625f,-0.09519998009800900717891636304557323455810546875f,0.11960001134276421197455420042388141155242919921875f,0.08150000231713061538130205008201301097869873046875f,-0.142300002653896751070305981556884944438934326171875f,-0.3715000045895575642163066731882281601428985595703125f,-0.13570001103281992271831768448464572429656982421875f,0.3768000094562768964578935992904007434844970703125f,0.122200004968047171161060759914107620716094970703125f,-0.02309999200105661287807379267178475856781005859375f,-0.4497000162184239258067464106716215610504150390625f,0.1062999858975413047801339416764676570892333984375f,0.105899990981817015978094786987639963626861572265625f,0.32770005266666402121700230054557323455810546875f,-0.36209998064637094472573153325356543064117431640625f,0.2138999857068062393494756179279647767543792724609375f,0.405500008732080363671457234886474907398223876953125f,0.350799966734648027699705608029034920036792755126953125f,0.707999994128942322646480533876456320285797119140625f,0.501899992203712397298431824310682713985443115234375f,-0.506100035452842433869591332040727138519287109375f,-0.4868999806925653484057647801819257438182830810546875f,-0.6614999631643296851279956172220408916473388671875f,0.5457000111445784451547069693333469331264495849609375f,0.3906000000476836930829449556767940521240234375f,-0.6635999942511319193982899378170259296894073486328125f,-0.169499988418072622398113935560104437172412872314453125f,0.7773000012382864643001312288106419146060943603515625f,-0.29160001989603046013144194148480892181396484375f,-0.4002000008851289702960229988093487918376922607421875f,-0.240299991809576785062319004282471723854541778564453125f,-0.0779999989718198616373001641477458178997039794921875f,-0.354599999272823485085837091901339590549468994140625f,-0.059100002834200859869184796480112709105014801025390625f,0.22050000150501725126872543114586733281612396240234375f,-0.1101000055834651381037048167854663915932178497314453125f,-0.2625000009685753976640398832387290894985198974609375f,0.02399999915063377731172522544511593878269195556640625f,0.1899000054061412701145172832184471189975738525390625f,-0.34710000873506074992036474213819019496440887451171875f,0.257700011241436044429065077565610408782958984375f,0.4340999877467750511783606270910240709781646728515625f,0.43139999732971201495956847793422639369964599609375f,-0.58709999063014972620067055686376988887786865234375f,0.17610001876354175465166918002068996429443359375f,0.39990000078678133998977273222408257424831390380859375f,0.0431999878615141330584492607158608734607696533203125f,0.7565999898701905035380832487135194242000579833984375f,0.7302000335931779773090966045856475830078125f,-1.2284000000715256106786910095252096652984619140625f,-0.998200038748979778802095097489655017852783203125f,0.100799996536970137839261951739899814128875732421875f,0.19260000100135787892696725975838489830493927001953125f,-1.16779999644756316001803497783839702606201171875f,-0.51360002308487917588308846461586654186248779296875f,0.1493000123620031871496394160203635692596435546875f,-0.37450000512599945778902110760100185871124267578125f,0.4793999957799908173683434142731130123138427734375f,-0.15519998903870568796037332504056394100189208984375f,-0.0525999962329863901544513282715342938899993896484375f,-0.33060001152157791892705063219182193279266357421875f,-0.5908999821901321780615035095252096652984619140625f,-1.0917000408649446541176075697876513004302978515625f,0.5944999693632126991360564716160297393798828125f,1.230900022423266904070260352455079555511474609375f,-0.6259999900460240240818166057579219341278076171875f,-0.857800033551454621516541010350920259952545166015625f,-1.23939994273185760675914934836328029632568359375f,-0.695400004303454721821253770031034946441650390625f,0.401399980007111967239552541286684572696685791015625f,-0.11120000549703856762562281801365315914154052734375f,0.244900019711256078380756662227213382720947265625f,-0.554700004148482861410229816101491451263427734375f,-1.0002999937653545003968247328884899616241455078125f,-0.53929999924898108787374439998529851436614990234375f,0.602400017774105123180561349727213382720947265625f,-0.18740000838637360658367470023222267627716064453125f,0.248100020611285998484163428656756877899169921875f,0.8327999863147734771473551518283784389495849609375f,-0.6828000074744224878742215878446586430072784423828125f,-0.40769998382329930564083042554557323455810546875f,0.109799999226629696469359487309702672064304351806640625f,-0.0711000009566544743933036443195305764675140380859375f,-0.77100000509619714961218051030300557613372802734375f,-1.176899999058246493177648517303168773651123046875f,-0.6824999994412064285853602996212430298328399658203125f,-0.0770999953985214148133309208787977695465087890625f,-0.4733999866604803319347638534964062273502349853515625f,0.1346999889492987989569883211515843868255615234375f,-0.34740000682175165014342610447783954441547393798828125f,0.0522000030308962248426496444153599441051483154296875f,-0.7593000126600266685983342540566809475421905517578125f,0.22980003405809412697635707445442676544189453125f,0.90450000080466264051892721909098327159881591796875f,-0.2700000277161596073938198969699442386627197265625f,-0.4836000199556351741847493030945770442485809326171875f,0.160500014543533353617021930404007434844970703125f,-0.852599986025690981250590994022786617279052734375f,-1.0631999599218371344022671109996736049652099609375f,0.01799999852478506578989936315338127315044403076171875f,0.55469999185502538718850473742350004613399505615234375f,-0.2697000033661722806499483340303413569927215576171875f,0.748200013566016952637482972932048141956329345703125f,-0.6836999817296860459947538402047939598560333251953125f,-0.8859000024557115438028631615452468395233154296875f,-0.165600009584426999253992107696831226348876953125f,0.64470001934766774098761743516661226749420166015625f,-0.4611000016272068791067795245908200740814208984375f,0.573000030636787727189584984444081783294677734375f,0.3277999872088432908867616788484156131744384765625f,0.91760004093647040690484573133289813995361328125f,-1.502800009262561520273493442800827324390411376953125f,-0.8674000422120098363620854797773063182830810546875f,-0.2703999958097933475897889366024173796176910400390625f,-0.2260000020042059187819205590130877681076526641845703125f,0.8389999464750292901271677692420780658721923828125f,0.34839998800158422653794332290999591350555419921875f,0.5531000295519827858470307546667754650115966796875f,-0.03369998593926393226638538180850446224212646484375f,0.5900000024214389071630648686550557613372802734375f,0.5820000201389194938172977344947867095470428466796875f,0.1679000168204307641417472041212022304534912109375f,-0.175299996894597853724917513318359851837158203125f,0.138099987679719760080843116156756877899169921875f,0.43510000725984543379354363423772156238555908203125f,0.10529999428689507112721912562847137451171875f,0.33210000828802599404099282764946110546588897705078125f,0.415699994492531033074556034989655017852783203125f,1.12270003463625922535129575408063828945159912109375f,0.1149999895691873330605403680237941443920135498046875f,0.728400005584954879367387547972612082958221435546875f,0.0280999918520450364667340181767940521240234375f,0.55770000587701762384540415951050817966461181640625f,0.15489998677968974671870228121406398713588714599609375f,0.6151000194042921531689671610365621745586395263671875f,0.138500028192996893494637333787977695465087890625f,-0.57189999622702547554808916174806654453277587890625f,1.202599951827526325587314204312860965728759765625f,-0.015000004172325276385890902020037174224853515625f,0.5940000137910248501071919235982932150363922119140625f,-0.3981000355124475476031875587068498134613037109375f,-0.7710000188425183420548592039267532527446746826171875f,-0.954900003618001758098898790194652974605560302734375f,0.030599997849762472501566890059621073305606842041015625f,0.124800000735372174087700614109053276479244232177734375f,0.5813999813854693332615397594054229557514190673828125f,0.9074999704957011914530085050500929355621337890625f,-0.3470999952122568732448826267500407993793487548828125f,0.1638000097393987886107424856163561344146728515625f,0.28259999697804449869664722427842207252979278564453125f,-0.00900001557916407790571611258201301097869873046875f,-0.4875000010058283539393642058712430298328399658203125f,-0.136799999845027997480428894050419330596923828125f,0.1185000064373016304131169817992486059665679931640625f,-0.389700008022785038264146351139061152935028076171875f,0.5876999948501586690241538235568441450595855712890625f,0.2663999918907880015694900066591799259185791015625f,0.855300004345178610520861184340901672840118408203125f,-0.32790002248287208885813015513122081756591796875f,0.7127999948456882339797857639496214687824249267578125f,-0.1755000029951332418676201996277086436748504638671875f,0.2355000010952352784698859977652318775653839111328125f,-0.541200006055831739359973653336055576801300048828125f,0.561300008994340959844748795148916542530059814453125f,-0.14130001365095379295411248676828108727931976318359375f,-0.749699996213614827667015561019070446491241455078125f,0.1320000066161153284838292165659368038177490234375f,0.718799983848631196536871357238851487636566162109375f,-0.06880001137852698178676291718147695064544677734375f,-1.4330999589204793664976023137569427490234375f,-0.972500019222497513737835106439888477325439453125f,1.039499974727630782211917903623543679714202880859375f,0.56770000751614535516864634701050817966461181640625f,1.3854000105619430627257315791212022304534912109375f,0.75280002788901345667227360536344349384307861328125f,0.738899984812736221329032559879124164581298828125f,0.619300028008222813724614752572961151599884033203125f,-0.42990000764131508503851364366710186004638671875f,-0.39430003575682714966887942864559590816497802734375f,-0.2419999948441979853441807790659368038177490234375f,-0.4622000202417375636088081591879017651081085205078125f,1.016200003075599767043968313373625278472900390625f,0.2164000079095362849557204754091799259185791015625f,0.306200010526179955405723376316018402576446533203125f,0.833800038796663756102134357206523418426513671875f,0.402300019939243913569271171581931412220001220703125f,-0.223500002227723371106549166142940521240234375f,-0.02589998546838767623512467253021895885467529296875f,-0.18549999114870985295056016184389591217041015625f,-0.645299991679191720095332129858434200286865234375f,-0.844700021135806888850083851139061152935028076171875f,0.9358999751865866301159258000552654266357421875f,0.64769999913424192161670589484856463968753814697265625f,0.6383999888956546353568910490139387547969818115234375f,0.4868000174283981795753106780466623604297637939453125f,0.508399994409084232671602876507677137851715087890625f,0.34560002120733290809084792272187769412994384765625f,-0.5568000204086303028816473670303821563720703125f,0.70110001667737975594718591310083866119384765625f,-0.69029998154640193064324193983338773250579833984375f,-0.6422999479293824975911775254644453525543212890625f,0.602699991944432245105645051808096468448638916015625f,0.787499983161688010824263983522541821002960205078125f,0.6714000030666589236005847851629368960857391357421875f,0.905700004327297136796914855949580669403076171875f,0.810300019209087007965308657730929553508758544921875f,0.67019999425411214133418980054557323455810546875f,-0.6417000276401640945778126479126513004302978515625f,-0.5322000050425526040953627671115100383758544921875f,-0.5448000161021948617445787022006697952747344970703125f,-0.148500000029802237122567021287977695465087890625f,0.005399993500113442479459990863688290119171142578125f,0.61409999545812610222128569148480892181396484375f,0.7176000250294805038464573954115621745586395263671875f,-0.152399996465444775850528458249755203723907470703125f,-0.0213000068709254808396735825226642191410064697265625f,0.67890001588165749257797187965479679405689239501953125f,-0.26969999330788840818939888777094893157482147216796875f,-0.05219999938011188334030521218664944171905517578125f,-0.503099999414384324580851171049289405345916748046875f,0.1011000172734259905382714350707828998565673828125f,0.62369998318254926683579242308042012155055999755859375f,0.5966999861218038603283275733701884746551513671875f,0.50520000580251223087913103881874121725559234619140625f,0.4301999904543161168390952298068441450595855712890625f,0.418800010335445505038620694904238916933536529541015625f,0.424199990946054317220159646240063011646270751953125f,0.367800005351007097686988345230929553508758544921875f,0.15989999959468814694218963268212974071502685546875f,0.150900011973828185318780015222728252410888671875f,0.0641000084407629400207184744431287981569766998291015625f,0.708900012975931304026744328439235687255859375f,0.02530001194477105030955499387346208095550537109375f,-0.16070000961720953824851676472462713718414306640625f,0.3983000077873466437949900864623486995697021484375f,0.0736000101208684753828492830507457256317138671875f,-0.68580000920295702826479100622236728668212890625f,-0.3273999989986418057696937466971576213836669921875f,-0.573000022888183746516688188421539962291717529296875f,-0.437899975022673970670439302921295166015625f,-0.62389999889433334079313908659969456493854522705078125f,-1.54490000197887411559349857270717620849609375f,-0.662300024074315984989880234934389591217041015625f,-0.594799991291761642742130788974463939666748046875f,-0.37139999811202251134290008849347941577434539794921875f,0.00549997967481630212205345742404460906982421875f,0.5135000150799751139629734097979962825775146484375f,0.5631999843597412791496026329696178436279296875f,0.675200006175040723377378526492975652217864990234375f,-0.49100000852346425972427823580801486968994140625f,0.17139999777674663761217743740417063236236572265625f,0.3408999922856690290018377709202468395233154296875f,0.300700002427398971338590172308613546192646026611328125f,-1.28180002964734995885009993799030780792236328125f,-0.869800059390068991405087217572145164012908935546875f,0.634599990779161515774831059388816356658935546875f,0.5300000140070915133350126779987476766109466552734375f,-0.174700000870227967197934049181640148162841796875f,-0.03629999811649309293670739862136542797088623046875f,-0.171300001674145441210583840074832551181316375732421875f,-0.151800005079060884582986545865423977375030517578125f,-0.2397000040367245343730928652803413569927215576171875f,0.340199991869926332554996406543068587779998779296875f,0.0819000063970685143743821754469536244869232177734375f,-0.6501000103890894887825879777665250003337860107421875f,-0.8021999971076845259432275270228274166584014892578125f,0.1833000059247016366725802072323858737945556640625f,-0.4338000130176544733018317856476642191410064697265625f,0.06480000425577170375390778644941747188568115234375f,-0.6866999843895433830454066992388106882572174072265625f,-0.2471999838083984712966412189416587352752685546875f,-0.38610000341534611578708791057579219341278076171875f,-0.8636999977111816662045384873636066913604736328125f,-0.364199988636374316630650582737871445715427398681640625f,-0.402899997951835564524714072831557132303714752197265625f,0.357900000876188340015460198628716170787811279296875f,-0.35700002926588059182222423260100185871124267578125f,0.422399995422363172536961428704671561717987060546875f,-0.01440000978708244616655065328814089298248291015625f,0.32400000395625827565027066157199442386627197265625f,-0.33899999713897699393783113919198513031005859375f,0.22649999255686992238878474381635896861553192138671875f,0.1196999909982086052195882075466215610504150390625f,-0.583800029781460949607208021916449069976806640625f,-0.5364000029921527357146260328590869903564453125f,0.6438000007241964084414576063863933086395263671875f,0.5192999946296215085794756305404007434844970703125f,-0.368699999052286120360122367856092751026153564453125f,0.7380000269114972422102027849177829921245574951171875f,-0.11249999016523393180477796704508364200592041015625f,-0.84589999589919973033147471142001450061798095703125f,-1.4180000094771383345459980773739516735076904296875f,-0.6448000348031523998315606149844825267791748046875f,-1.2637999894738196626775561526301316916942596435546875f,-0.747000029265880716167202990618534386157989501953125f,-0.11499999567866314720276932348497211933135986328125f,-0.3014000120818616235141007564379833638668060302734375f,0.57519998739957856059845653362572193145751953125f,0.8540000002682202051573767676018178462982177734375f,-0.07270001273155202170528355054557323455810546875f,-0.73770002360940001295830370509065687656402587890625f,0.6087999650359154912848680396564304828643798828125f,0.15659997727870855754872536635957658290863037109375f,-0.1001000091224906185516374534927308559417724609375f,-0.28550002862513135237776396024855785071849822998046875f,1.17700000274181348913771216757595539093017578125f,0.7078000342965129476624497328884899616241455078125f,0.2910999968588352526666085395845584571361541748046875f,0.1413000019907950655095874026301316916942596435546875f,-0.4559000215291977298193160095252096652984619140625f,0.37290001398921024389210288063623011112213134765625f,-0.7783999485135080220743475365452468395233154296875f,-0.65899999177455814702852876507677137851715087890625f,-0.1755999870836735254897575941868126392364501953125f,-0.015999992877244739730713263270445168018341064453125f,1.080300009560584584278331021778285503387451171875f,0.6867000465273864762139055528677999973297119140625f,1.0139999887943265122203229111619293689727783203125f,0.0821999905884265302802305086515843868255615234375f,-0.65549998388439389973569859648705460131168365478515625f,0.41850000077486004812499231775291264057159423828125f,-0.3918000125482679063537716501741670072078704833984375f,-0.779999997615814066875827847979962825775146484375f,-0.7512000083282590612299145504948683083057403564453125f,-0.8933999892681836474395140612614341080188751220703125f,-0.40500000871717933836890779275563545525074005126953125f,-0.11399999937415117301497957669198513031005859375f,0.46859997804611897009863241692073643207550048828125f,-0.218400015568732897008885629475116729736328125f,-0.6963000121235847217349146376363933086395263671875f,0.2468999898195267661549223703332245349884033203125f,0.4769999832585452015365490296971984207630157470703125f,0.8648999998927120458347417297773063182830810546875f,-0.557700022566318676098262585583142936229705810546875f,-0.297300005522370014432453899644315242767333984375f,0.6882000141620636668449151329696178436279296875f,0.8009999980926512108680981327779591083526611328125f,0.1628999980822205262409596571160363964736461639404296875f,0.2402999992042780252887723690946586430072784423828125f,0.4095000109747053418374207467422820627689361572265625f,-0.51120002185106283576487840036861598491668701171875f,-0.9113999866381286008021334055229090154170989990234375f,-0.761699980962276868012850172817707061767578125f,-0.235199995073676060286516076303087174892425537109375f,-0.403799999904632667124815270653925836086273193359375f,0.883500030353665533056073400075547397136688232421875f,0.965400007283687244807879324071109294891357421875f,0.4416000166922808833902536207460798323154449462890625f,0.4788000148057935945189456106163561344146728515625f,-0.0467999906063079063045506700291298329830169677734375f,-0.639600020879507180637801866396330296993255615234375f,-0.3378000038981436858875895268283784389495849609375f,0.060000004470348589080685997032560408115386962890625f,-0.1481000057101251599078750587068498134613037109375f,-0.78890002877116227608667031745426356792449951171875f,-0.3110999821625648475986736229970119893550872802734375f,0.361500016305595917032178476802073419094085693359375f,0.4411000023722646545820680330507457256317138671875f,0.815300030720234047976191504858434200286865234375f,-1.6966999970555303178798567387275397777557373046875f,-0.8881000557780271975616415147669613361358642578125f,-0.0054000095002351145012653432786464691162109375f,0.09039999683424826404376517530181445181369781494140625f,-0.4437000257253647106381322373636066913604736328125f,0.42810000630617128791755021666176617145538330078125f,1.7108000285744662249953762511722743511199951171875f,0.88220005481243202183350149425677955150604248046875f,0.9259999754428864804367549368180334568023681640625f,0.40360001474022855205703308456577360630035400390625f,-0.3429000011742113773749451866024173796176910400390625f,-0.4703000213325025935517942343722097575664520263671875f,-0.05050000508129581788807627162896096706390380859375f,-0.17570001632273235525616428276407532393932342529296875f,0.975199961471557852377145536593161523342132568359375f,0.72020000751614521306009919499047100543975830078125f,-0.75049998176097876267931496840901672840118408203125f,0.12810000332593940441938684671185910701751708984375f,0.45959998645782462745046359486877918243408203125f,0.27809999080896385947880844469182193279266357421875f,-0.63840001147091396394017692728084512054920196533203125f,0.1896000088840723574179492061375640332698822021484375f,0.3408000025346875627718645773711614310741424560546875f,-0.01290000340938579626026694313623011112213134765625f,-0.4275000036880374221226475128787569701671600341796875f,0.55920001791715634453794336877763271331787109375f,0.03750000229105354687675344393937848508358001708984375f,-0.78330000529140253462401233264245092868804931640625f,0.4437000161513686702363656877423636615276336669921875f,-0.3117000062942505422824979177676141262054443359375f,-0.5421000227048995423473343180376105010509490966796875f,-0.8420999739408490114556116168387234210968017578125f,0.308999992612749296849727898006676696240901947021484375f,0.21029999452903869183728602365590631961822509765625f,0.5457000068977475581988301200908608734607696533203125f,-0.42810001554489129915737066767178475856781005859375f,0.5298000289544464447288873998331837356090545654296875f,0.8499000112175938426162247196771204471588134765625f,0.2465999973580241810822144543635658919811248779296875f,0.52499996960163120007791803800500929355621337890625f,-0.43530001149773600577219667684403248131275177001953125f,-0.1046999978154896648874228048953227698802947998046875f,-0.30990001274496326022500625185784883797168731689453125f,-0.13500000275671464322613246622495353221893310546875f,0.745199990156292901843926301808096468448638916015625f,0.662099972301721795275852855411358177661895751953125f,0.25050000735372301985393050927086733281612396240234375f,-0.5954999814629555743294986314140260219573974609375f,0.70320001930296438530376690323464572429656982421875f,0.273400005324184736110737503622658550739288330078125f,-0.7825999831758438940454425392090342938899993896484375f,-0.146800009139627096743652145960368216037750244140625f,0.3456999757170677867179620079696178436279296875f,0.777699996787308833745555602945387363433837890625f,-0.029799990248680074955700547434389591217041015625f,-0.356200016337633318386224345886148512363433837890625f,0.6230999812722206598891716566868126392364501953125f,0.4205000106692313721623577293939888477325439453125f,0.01820001267194726324305520392954349517822265625f,0.61780003504157132709906363743357360363006591796875f,0.699699977028370057041684049181640148162841796875f,0.950700006711482359378351247869431972503662109375f,-0.2053999923825262641230438021011650562286376953125f,0.4918000095307828445356790325604379177093505859375f,-0.4161999695479874361581096309237182140350341796875f,-0.5867999917984001445603325919364579021930694580078125f,0.48280001216828782872880765353329479694366455078125f,0.55300004322826945557523004026734270155429840087890625f,0.979300016236305026495756465010344982147216796875f,0.036100002968311173390247859060764312744140625f,-0.38729997773170499186790038947947323322296142578125f,-0.6158999999225134303060258389450609683990478515625f,-0.121499981343746288331431060214526951313018798828125f,0.38630001048445716804735639016143977642059326171875f,1.165800021123885699125821702182292938232421875f,0.986200057464838852894217779976315796375274658203125f,-1.4379999677538872759896548814140260219573974609375f,-0.90020001839399288456888825749047100543975830078125f,0.137800006878375569385752896778285503387451171875f,0.8909000015616417300634566345252096652984619140625f,0.2148000194549553043543710373342037200927734375f,0.7476999796509744555805809795856475830078125f,-1.21199998426437360876661841757595539093017578125f,-1.46860000058412598633594825514592230319976806640625f,0.500099996232986399036235525272786617279052734375f,0.64019998799562483071667884360067546367645263671875f,0.55859998985528847015302744694054126739501953125f,-0.077200011897087250645199674181640148162841796875f,-0.789600003743171185988103388808667659759521484375f,-1.1281999915122984390336569049395620822906494140625f,-1.1369999639987948825137209496460855007171630859375f,-0.9415999754905701735196998924948275089263916015625f,-0.58920000420808804619809961877763271331787109375f,-0.164000012487173219000169410719536244869232177734375f,-1.350299974393845303666239487938582897186279296875f,-0.6957000013470651111902043339796364307403564453125f,0.3999000009357924767527947551570832729339599609375f,0.41549999964237205318795531638897955417633056640625f,0.6515999720633036673689275630749762058258056640625f,0.75670000838041318047544336877763271331787109375f,-0.2712000382423394029274277272634208202362060546875f,-0.2593000004410743741800615680404007434844970703125f,-0.224099975788593752668020897544920444488525390625f,-0.562900003111362678964724182151257991790771484375f,-0.9051000164985651963434065692126750946044921875f,-1.1349999964237209582051946199499070644378662109375f,0.6792000067412846675551918451674282550811767578125f,0.769900036847591451305561349727213382720947265625f,0.117600018286704965930766775272786617279052734375f,-0.69140000478029239872057587490417063236236572265625f,0.83450000527501089209181373007595539093017578125f,0.04720000474452967775818024165346287190914154052734375f,-0.03180002160072348260655417107045650482177734375f,-0.449399984306097177721994739840738475322723388671875f,-0.83660000678300860954550444148480892181396484375f,0.1457999844670296862858549502561800181865692138671875f,0.469199995863437635534864966757595539093017578125f,0.04219999788999562184699243516661226749420166015625f,0.395899991428851905084229656495153903961181640625f,0.3211999972641468303891087998636066913604736328125f,1.445499985635279927009833045303821563720703125f,0.1409999977946281290996921597979962825775146484375f,0.10989997009038887654241989366710186004638671875f,-0.74719998671412479751552382367663085460662841796875f,-0.3353999950647352790156219271011650562286376953125f,0.40220000295639035670092198415659368038177490234375f,0.692700026105344424109944156953133642673492431640625f,0.2209999967142939336639528846717439591884613037109375f,-0.49460000240206714039459257037378847599029541015625f,-0.00820000820159905163819757945020683109760284423828125f,-0.8593000082641839565411601142841391265392303466796875f,0.145599990957975489180853401194326579570770263671875f,0.40860003426075053312160889618098735809326171875f,-0.43679998852014545462907335604541003704071044921875f,0.24659999162107693049250656258664093911647796630859375f,0.1311999975621700453842777278623543679714202880859375f,-0.250900024807453547737168264575302600860595703125f,-0.47200000721216195387341940659098327159881591796875f,0.42060000034570688143276129267178475856781005859375f,-0.6168000210046766795812800410203635692596435546875f,-0.092400014942884300950254328199662268161773681640625f,-0.289299985241890045273294163052923977375030517578125f,-0.7880999619007109657786713796667754650115966796875f,-0.3997999741554263408715996774844825267791748046875f,0.46649998968839678781250768224708735942840576171875f,0.55679997451305407452082363306544721126556396484375f,-0.1928999928295613841555677936412394046783447265625f,-0.6153999944686887602074421010911464691162109375f,0.415199995148181955073596327565610408782958984375f,-0.055299988400935973231753450818359851837158203125f,-0.17910002377033151788054965436458587646484375f,-0.78969997702836991493313689716160297393798828125f,-1.416599957013129795768691110424697399139404296875f,-0.928899974381923954069861792959272861480712890625f,0.942299992036819134000324993394315242767333984375f,0.9468000078916549711038896930404007434844970703125f,0.1394999756775796573293746405397541821002960205078125f,-0.365100023725628997084413640550337731838226318359375f,0.086699998694658386710898412275128066539764404296875f,0.181199984002113456682536707376129925251007080078125f,0.5489999960958957814227687777020037174224853515625f,0.7818999937683346335148826256045140326023101806640625f,-1.018799977254868593945502652786672115325927734375f,-1.007400011515617421764545724727213382720947265625f,0.08160000313222415257996544823981821537017822265625f,-0.25059999591261139517683886879240162670612335205078125f,-0.82199999171495363725625793449580669403076171875f,-0.4443000010371207508796942420303821563720703125f,-1.0019999926090239483755794935859739780426025390625f,0.0699999946355820412691173260100185871124267578125f,-0.346700010578334438804404271650128066539764404296875f,-0.2218000029370188475041913989116437733173370361328125f,1.04330002470016491855631102225743234157562255859375f,-0.369800009322166278735721789416857063770294189453125f,-0.12550001156330115037462746840901672840118408203125f,-0.323199993002414753817674863967113196849822998046875f,-1.265800010991096513635056908242404460906982421875f,0.44459999793171878224029569537378847599029541015625f,0.66719999073743796458302313112653791904449462890625f,-0.3609999890029431668381221243180334568023681640625f,1.5575000289082527871187267010100185871124267578125f,0.09359999536871921321790068759582936763763427734375f,0.63530002714395550356130115687847137451171875f,0.33580000488162031757610748172737658023834228515625f,-0.3701999990224837944197133765555918216705322265625f,-0.154200001972913725012404029257595539093017578125f,0.3575000002980228686055852449499070644378662109375f,0.042599994295835497126745394780300557613372802734375f,-0.49500002581626201259012987065943889319896697998046875f,-0.1811999945819378066147464778623543679714202880859375f,0.11399999646842484679609697195701301097869873046875f,-0.189399991905689291105119309577275998890399932861328125f,0.136299992603063468976642980123870074748992919921875f,-0.4357999884903431819793695467524230480194091796875f,-0.199799999487400015141247422434389591217041015625f,-0.1319999951422214934382282081060111522674560546875f,0.752099976474046361119007997331209480762481689453125f,-0.2473999855875970155238974257372319698333740234375f,0.7542000128507613254669195157475769519805908203125f,0.97519997786283507679172544158063828945159912109375f,-0.3164999925009903680717116003506816923618316650390625f,-0.057900003930926313699956153868697583675384521484375f,-1.0005000312328338196721233543939888477325439453125f,-1.10590001020431483169659259147010743618011474609375f,-0.5531999772071838350484540569595992565155029296875f,-0.26889998838901529865097472793422639369964599609375f,1.276500000715255822569815791212022304534912109375f,1.53139999256134018423836096189916133880615234375f,-0.913499978423118363934918306767940521240234375f,-0.96959998675584824212592138792388141155242919921875f,-0.255000043511390828143703402020037174224853515625f,-0.7904999933838841030819821753539144992828369140625f,0.37049998980760534550427109934389591217041015625f,-0.192900013244152290781130432151257991790771484375f,-0.126900002038478731947179767303168773651123046875f,0.1937999921560287219790552626363933086395263671875f,0.0686999691009528845597742474637925624847412109375f,0.086899995481967806654211017303168773651123046875f,-0.07619998243749126487500689108856022357940673828125f,0.41860002789050343441346058170893229544162750244140625f,-0.3962999806076290543188633819227106869220733642578125f,-0.3225999877393246340062660237890668213367462158203125f,-0.75209998317956916480397921986877918243408203125f,-0.323099980825185983945857515209354460239410400390625f,-0.1169999945461750456843219581060111522674560546875f,0.164400003528594851331945392303168773651123046875f,-0.7292999676585196766609442420303821563720703125f,-0.895899962669611138466052580042742192745208740234375f,0.735599965178966641587976482696831226348876953125f,-0.255800000560283624650992351234890520572662353515625f,0.37770001675486586378838183009065687656402587890625f,-0.04779999644756316445892707633902318775653839111328125f,-0.11570000909566857671961770392954349517822265625f,-0.57179998270869258902848741854541003704071044921875f,0.4275000187754631753023204510100185871124267578125f,0.1411999975621700542660619248636066913604736328125f,-1.18170004146099127950719775981269776821136474609375f,0.20260000338554373655597373726777732372283935546875f,0.4026000088989736269695640658028423786163330078125f,-0.34679999418258666832315384453977458178997039794921875f,-0.3084000167608262898966131615452468395233154296875f,-0.172800003302097326951525246840901672840118408203125f,0.5844000071048736089096564683131873607635498046875f,-0.027799996298551565843126809340901672840118408203125f,-0.7830000079870220730526853003539144992828369140625f,-0.412999989807605771829912555404007434844970703125f,0.72499999888241273282574184122495353221893310546875f,0.295599996992945701634880606434307992458343505859375f,-0.169500008255243361787734102108515799045562744140625f,-0.14560000183582300081752691767178475856781005859375f,0.16280002326965359316091053187847137451171875f,-0.2630000006854533722844280418939888477325439453125f,-0.2598999932616947727836986814509145915508270263671875f,0.12299999263882643862899612940964289009571075439453125f,0.289299998429417559719922792282886803150177001953125f,-0.040799999740719794516508045489899814128875732421875f,-0.34899999237060530532517077517695724964141845703125f,0.5333999911308289210154498505289666354656219482421875f,-1.53840003046989437507363618351519107818603515625f,-0.30230000693798064759221233543939888477325439453125f,-0.156300007504225124677077474188990890979766845703125f,-0.172699996936321209517473107553087174892425537109375f,-0.945299958300591214310770737938582897186279296875f,-0.363499962627887640564949833787977695465087890625f,0.213599971377849584541763761080801486968994140625f,0.1086999950289726513119603623636066913604736328125f,0.744900030589104034106640028767287731170654296875f,1.03790001711845381038301638909615576267242431640625f,-0.749399992352724364508276266860775649547576904296875f,-0.620599992597103078395548436674289405345916748046875f,-0.3371999823927875894469252671115100383758544921875f,-0.3032000087976456370597588829696178436279296875f,-0.1365000132322311543475734652020037174224853515625f,-0.159199986028671247595411841757595539093017578125f,-0.54569995574951235539629124104976654052734375f,-0.0123999900579452315696471487171947956085205078125f,0.291599984915554610864774076617322862148284912109375f,-0.23479998950362190868190737091936171054840087890625f,-0.177599998319148966174907400272786617279052734375f,0.056300008994341066426159159163944423198699951171875f,-0.46050001513957994347947533242404460906982421875f,-0.19840001199245449470254243351519107818603515625f,0.272099990630149779491375738871283829212188720703125f,0.2158999846488238727459929577889852225780487060546875f,-0.206399997366964793599208860541693866252899169921875f,-0.355699997472763040917698162957094609737396240234375f,1.04309998425245264996874539065174758434295654296875f,0.7011999848961831816041012643836438655853271484375f,0.293100029850005938669710303656756877899169921875f,0.40399998998641972747236650320701301097869873046875f,-0.102699998873472253535510390065610408782958984375f,0.362399999445676801457238980219699442386627197265625f,0.7099000285029415380222417297773063182830810546875f,-0.50079999459981916487549824523739516735076904296875f,0.46080002484917681471188188879750669002532958984375f,-0.53859999879598607908093299556639976799488067626953125f,1.157599988335370699843451802735216915607452392578125f,-0.20159999396800998283652006648480892181396484375f,-1.15470001904964458816493788617663085460662841796875f,0.13399999892711633719954988919198513031005859375f,-0.444999997019767334904827293939888477325439453125f,-0.0301999991714954063581899390555918216705322265625f,-0.25930000133514408133805773104541003704071044921875f,-0.210800000113248842126267845742404460906982421875f,-0.558300011028349363328970866859890520572662353515625f,-0.3589999973252415710334162213257513940334320068359375f,0.653100006604194760484460857696831226348876953125f,0.554800007385015447880505234934389591217041015625f,0.43539998642206168000257093808613717555999755859375f,-0.209999997615814226747943394002504646778106689453125f,0.0165000266432766551361055462621152400970458984375f,-0.7234000012636183640779563575051724910736083984375f,-0.45419999623596662985391958500258624553680419921875f,0.478399990683794074097789916777401231229305267333984375f,-0.43960000284910183410147510585375130176544189453125f,-0.35659999143481257988241850398480892181396484375f,0.279299992245435557691735084517858922481536865234375f,0.13039998821020137853565756813623011112213134765625f,-0.42180002219676993036046042107045650482177734375f,0.6188999964356423078015723149292171001434326171875f,0.79079998439252374708985371398739516735076904296875f,0.7287000072479248302670384873636066913604736328125f,-1.153199998664856451568994089029729366302490234375f,-1.0691000067830085384912308654747903347015380859375f,-1.101300003927946580262187126209028065204620361328125f,-0.821000002861022704081506162765435874462127685546875f,-0.79199998992681486242872779257595539093017578125f,-1.176799974364042267183094736537896096706390380859375f,0.60060001554489161890160175971686840057373046875f,0.91629999155998209658946507261134684085845947265625f,-0.09329996986389232915826141834259033203125f,-0.220600009584426715036897803656756877899169921875f,-0.279599998080730216543088317848742008209228515625f,0.01669999258518206630697022774256765842437744140625f,-0.5024999764002859858891270050662569701671600341796875f,-0.066099998870492004954257936333306133747100830078125f,0.8135999977529042581636531394906342029571533203125f,0.165900024211406815766167710535228252410888671875f,-0.5777999875068662305466204998083412647247314453125f,-0.68379998768568039935189517564140260219573974609375f,-1.26690000591278106867321184836328029632568359375f,-0.6263000156998632661498049856163561344146728515625f,1.0133999813705683035180982187739573419094085693359375f,0.8183999847233296431880944510339759290218353271484375f,-0.4784999754428866935995756648480892181396484375f,0.01280000926256175119988256483338773250579833984375f,0.023999991774559248369769193232059478759765625f,-0.400900005882978138771477460977621376514434814453125f,-0.097499985396861887920749722979962825775146484375f,-0.143799996477365521485580757143907248973846435546875f,-0.27640001468956487684636158519424498081207275390625f,-0.409800003808736768018405882685328833758831024169921875f,-0.194600007654726514516596580506302416324615478515625f,-0.23699999830871820716282627472537569701671600341796875f,0.7153000012345611668962419571471400558948516845703125f,-0.23379998701512827441462150090956129133701324462890625f,-0.352000002890825225421167488093487918376922607421875f,0.35739999914765352428958067321218550205230712890625f,-0.1493999773025507238344289362430572509765625f,-0.6449999940395354514066639239899814128875732421875f,-0.4035000007748603678692234097979962825775146484375f,0.23559999602437020627121455618180334568023681640625f,1.459600056791305888737042550928890705108642578125f,0.117999995470047025492021930404007434844970703125f,0.02519998352527608176387730054557323455810546875f,-0.47499999314546581530294133699499070644378662109375f,0.0602999800562855625685187987983226776123046875f,-0.30179999060630802176774523104541003704071044921875f,-0.380000013709068440448390902020037174224853515625f,-0.7798000090241430370952002704143524169921875f,-0.4246000213265421763253470999188721179962158203125f,0.18259999623298650561764588928781449794769287109375f,-0.20929997868537864036397877498529851436614990234375f,0.0829999896585942309457095689140260219573974609375f,0.70600001686811442169755537179298698902130126953125f,0.054199997204542160744722423260100185871124267578125f,-0.063499995857477120608791665290482342243194580078125f,-0.002000002294778813194398026098497211933135986328125f,-0.3780000246763233207047960604541003704071044921875f,-0.9966999977260828647018797710188664495944976806640625f,-0.7208999869585033248853278564638458192348480224609375f,-0.4166000057399272815672475189785473048686981201171875f,-0.4034999847002327388878484271117486059665679931640625f,-0.21019999790489674040827594581060111522674560546875f,-0.648299985696375369315092029864899814128875732421875f,-0.732499986607581643482944855350069701671600341796875f,0.658500009119510565369637333787977695465087890625f,0.335999987512826780999830589280463755130767822265625f,-1.0388999919652945891357376240193843841552734375f,-0.33759999861717204794331337325274944305419921875f,0.565199993658065835688830702565610408782958984375f,0.5528999923825264062315909541212022304534912109375f,-0.191400055742263930369517765939235687255859375f,-0.729600030565261903348073246888816356658935546875f,-0.788699981617927647903343313373625278472900390625f,-0.288999966740608016380065237171947956085205078125f,-0.413699989366531184487030259333550930023193359375f,0.066700036990642530554396216757595539093017578125f,-1.2339000009059901685759541578590869903564453125f,-0.383799995732307053231124882586300373077392578125f,0.5576999949991705562979404930956661701202392578125f,0.7392000194072725349769825697876513004302978515625f,0.1052999892950055027540656737983226776123046875f,-0.07750003337860089658306605997495353221893310546875f,-0.0831000271677968527228586026467382907867431640625f,-0.31520000945329673669448311557061970233917236328125f,-0.0204000010251998986632315791212022304534912109375f,-0.060300010007619864182970559340901672840118408203125f,-0.3725999974250786550555858411826193332672119140625f,0.12599999436736109004186801030300557613372802734375f,-0.313700002777576258949920884333550930023193359375f,-0.007599992209673001042347095790319144725799560546875f,-0.0034000075966120757442467947839759290218353271484375f,0.083400002008676477061044352012686431407928466796875f,-0.02549997917190172103119039093144237995147705078125f,0.40719998559653769465427330942475236952304840087890625f,0.6455000303387645743669054354541003704071044921875f,0.223600003266334557139316530083306133747100830078125f,0.43869999994635566764600298483856022357940673828125f,0.25919999586343767106200175476260483264923095703125f,-0.50269999857544878096859974903054535388946533203125f,-0.1255999937891960627212029066868126392364501953125f,-1.0331999855518336772775001008994877338409423828125f,0.32700000393390649122693503159098327159881591796875f,0.2420000130236150681639628601260483264923095703125f,-0.120800000262260454064744408242404460906982421875f,-0.152599991129338530271297713625244796276092529296875f,0.1002000057652592257539936326793394982814788818359375f,1.411200010228156997982296161353588104248046875f,0.14400000891089437260461636469699442386627197265625f,0.53730000276565537120632143341936171054840087890625f,-0.23279999778866766746432404033839702606201171875f,0.5955000297427179845044520334340631961822509765625f,0.191999995142221546728933390113525092601776123046875f,0.1792999947786331205179521930404007434844970703125f,-0.4619999945461750190389693671022541821002960205078125f,0.35900000682473187652021806570701301097869873046875f,0.15120000426769253376590995685546658933162689208984375f,0.75029999883174891550652318983338773250579833984375f,-0.37160001065731051994589506648480892181396484375f,0.0168000272631640967802013619802892208099365234375f,0.0187000027775763300041944603435695171356201171875f,0.1011000096738339237845138995908200740814208984375f,-0.0525999988406897767134751120465807616710662841796875f,0.618599976928531969377900168183259665966033935546875f,0.0876999639488758031546922211418859660625457763671875f,0.2741999794721596828139809076674282550811767578125f,-0.011099997007846695851185359060764312744140625f,0.31499998532235640169574253377504646778106689453125f,-0.07809999915361398592494879267178475856781005859375f,-0.05819998234808476667012655525468289852142333984375f,0.24569999955892551923852806794457137584686279296875f,0.985500003218650277858614572323858737945556640625f,1.2365999850273130533651055884547531604766845703125f,-0.40619999525249017580108557012863457202911376953125f,-0.5994000187277794822193754953332245349884033203125f,0.2883000022925432670461987072485499083995819091796875f,0.371400003960728586349659963161684572696685791015625f,-0.107400012707710601489452528767287731170654296875f,-0.609399966573715090589757892303168773651123046875f,-0.47820001356601693487391457892954349517822265625f,-0.29829998655319212019776387023739516735076904296875f,0.2825999598741528728851335472427308559417724609375f,0.12869999622106575998259359039366245269775390625f,-0.94079997910261159432820932124741375446319580078125f,-0.73349999243021013484167269780300557613372802734375f,0.197999995321035182627156245871447026729583740234375f,0.013000008583068822787254248396493494510650634765625f,-0.773399981296062577484917710535228252410888671875f,-0.126399995541572607304914299675147049129009246826171875f,0.52509998848438232954549675923772156238555908203125f,-0.05240000689625734509036192321218550205230712890625f,0.90570002906322510938252889900468289852142333984375f,-0.489800003808736672539225764921866357326507568359375f,0.6232999850630758231773143052123486995697021484375f,-0.65319998510479937436912223347462713718414306640625f,-0.426900000362098186457160409190692007541656494140625f,-0.0930000009462237375856830112752504646778106689453125f,0.69790000966787335556773541611619293689727783203125f,-0.07720000370144841639330479665659368038177490234375f,0.428099990510940386911897803656756877899169921875f,-0.0269999841153623076905887501197867095470428466796875f,-0.004000000119209090598815237171947956085205078125f,-0.3373999922931194550557165712234564125537872314453125f,0.603400031661987412689995835535228252410888671875f,0.49879998902678490679818423814140260219573974609375f,0.745200030314922656771159381605684757232666015625f,0.371000005245208708259951890795491635799407958984375f,0.30910000153034922210082413585041649639606475830078125f,-0.3915999932229519142623530569835565984249114990234375f,0.233099990212917163034944678656756877899169921875f,-0.20239999229311944617393237422220408916473388671875f,0.74440002848804009971672712708823382854461669921875f,0.194799995389580704596710347686894237995147705078125f,0.60150001218914983525110073969699442386627197265625f,-0.02680000088810918779103076303726993501186370849609375f,0.24440000986158860740715681458823382854461669921875f,0.0900000029057264061549403777462430298328399658203125f,0.32899998432397836722884676419198513031005859375f,-0.2023999919950962844694686282309703528881072998046875f,-0.360000003576278260197796043939888477325439453125f,-0.69129999930858598844451989862136542797088623046875f,-1.1907000171422961187772671109996736049652099609375f,-1.18050000977516145184154083835892379283905029296875f,-1.407599965238571115833110525272786617279052734375f,-1.1830999723315240856891250587068498134613037109375f,-0.0044999992959200429964994327747263014316558837890625f,0.278699992719292577447731673601083457469940185546875f,-0.4368000101268290080724909785203635692596435546875f,-0.77330001039505003035401387023739516735076904296875f,-0.08969998418092739456142226117663085460662841796875f,-0.06169996606111549652950998279266059398651123046875f,-0.66689997357726138460520814987830817699432373046875f,-0.50479999263286590149846233543939888477325439453125f,0.665999956965446671119934762828052043914794921875f,-0.02920003907680523980161524377763271331787109375f,0.503099979037045841323561035096645355224609375f,0.0546000019550323401062996708787977695465087890625f,-0.72539999648034569901255963486619293689727783203125f,-0.39309998674839741195086162406369112432003021240234375f,-0.304199996757507307165724341757595539093017578125f,0.006300031197071120914188213646411895751953125f,0.056399973599612795993607505806721746921539306640625f,-0.48140002966523187666325611644424498081207275390625f,-0.180000019818544298999540842487476766109466552734375f,-0.280499999344348882601707373396493494510650634765625f,0.081599998326599365583433609572239220142364501953125f,-0.0870000008046627471003375831060111522674560546875f,-0.5132999703109266675937760737724602222442626953125f,-0.38879999334812165301400455064140260219573974609375f,-0.7752999782681460061439793207682669162750244140625f,0.640400004005432066378489253111183643341064453125f,1.3854000218868254279414031771011650562286376953125f,0.0141999843895437294349903822876513004302978515625f,-1.31510002067089093458207571529783308506011962890625f,0.16139998660087595538925597793422639369964599609375f,0.487399996726214812525768138584680855274200439453125f,-0.0257999998524784945885812703636474907398223876953125f,-0.9637000176787378080689450143836438655853271484375f,0.53820000373125065351587181794457137584686279296875f,0.37420000268071884619303091312758624553680419921875f,0.0589999984428286605719904400757513940334320068359375f,-0.2348000197529795940454278024844825267791748046875f,0.3524000013828276767213765197084285318851470947265625f,0.0999999941140412573759022052399814128875732421875f,-0.198999996393919020842844247454195283353328704833984375f,0.25520001079440124414077217807061970233917236328125f,0.108600000286102282842648492078296840190887451171875f,-0.34810001152157798998132420820184051990509033203125f,-0.01800000485777852876623228439711965620517730712890625f,0.25269999753236760398067417554557323455810546875f,-0.272000001847743977378968338598497211933135986328125f,0.34270000200271599766210783855058252811431884765625f,-0.460599991405010289469146300689317286014556884765625f,-0.3691000514864928305769353755749762058258056640625f,0.020399999237060484347239253111183643341064453125f,-0.214399996823072314100500079803168773651123046875f,0.031200004267692538206802055356092751026153564453125f,-0.405699985440075128195758225047029554843902587890625f,-0.21399998223781580009017488919198513031005859375f,1.475399994468688191773253493010997772216796875f,1.3936999878764151361565382103435695171356201171875f,-0.3627000568389888712772517465054988861083984375f,-0.8046000061273570480580019648186862468719482421875f,0.691499999523163211279097595252096652984619140625f,1.02929998583793658184504238306544721126556396484375f,-0.2768999950535595555578538551344536244869232177734375f,-0.573999994233250543373969776439480483531951904296875f,1.373100005412101864976648357696831226348876953125f,1.4823000078320500705331141944043338298797607421875f,-0.0429000059612094641892099389224313199520111083984375f,-0.28440000166594980868239872506819665431976318359375f,0.70500000625848802116024671704508364200592041015625f,0.50640000805854779031278667389415204524993896484375f,-0.4100999887078999872613849220215342938899993896484375f,-0.3251999941051006981496129810693673789501190185546875f,0.13049999018013469509469359763897955417633056640625f,-0.02360000445842747041069742408581078052520751953125f,0.045600002282858209667892879224382340908050537109375f,0.133499994218349460339823053800500929355621337890625f,-0.6347999965071675632088954444043338298797607421875f,-0.6287000057578087108822728623636066913604736328125f,-0.8678999938726423835078094271011650562286376953125f,-0.50689999041557332049023898434825241565704345703125f,0.0102000341892249934971914626657962799072265625f,-0.1212999829173089239020555396564304828643798828125f,0.17940000330507732684282018453814089298248291015625f,0.3330000047087668946232952293939888477325439453125f,0.4478999957911666829346586382598616182804107666015625f,-0.271999997675418825338056194595992565155029296875f,1.273900029218197005320689640939235687255859375f,-0.12920000256896013723917349125258624553680419921875f,-0.2045999956220387883121247796225361526012420654296875f,0.12299999547004702993291402890463359653949737548828125f,0.3883000261902811445224870112724602222442626953125f,0.7675999925076961716285950387828052043914794921875f,-0.3820000027418135601919857435859739780426025390625f,0.2177999915301800371025819913484156131744384765625f,-0.6164999992251392058051351341418921947479248046875f,0.1363999983727932630017676274292171001434326171875f,0.08519999976754188963923297706060111522674560546875f,-0.058000000089406977821226973901502788066864013671875f,0.993400010204315009332276531495153903961181640625f,-0.50320001103281963850122338044457137584686279296875f,-1.02119998757839169911676435731351375579833984375f,0.53839999783635139607440578402020037174224853515625f,1.00849999868869755204059401876293122768402099609375f,-0.46599999228119859395746971131302416324615478515625f,0.59779998929500521853697136975824832916259765625f,0.60999997973442088294859786401502788066864013671875f,-0.01139999040067163349476686562411487102508544921875f,-0.411799995374679539406059802786330692470073699951171875f,0.376299973082542038582687382586300373077392578125f,-0.6861999911546707853204907223698683083057403564453125f,0.4546000150680544749093314749188721179962158203125f,0.523000000387430219461748492904007434844970703125f,-0.2337000170826915024235859164036810398101806640625f,-0.393200009244680348530209812452085316181182861328125f,-0.2596000037431718254765655728988349437713623046875f,0.38920000867843640435239649377763271331787109375f,-0.646800027561187818037069519050419330596923828125f,-1.041300009143352411911109811626374721527099609375f,0.297899990147352244918010910623706877231597900390625f,0.2975999906450510490429905985365621745586395263671875f,1.22999996721744508931806194595992565155029296875f,0.4230999782919884211196404066868126392364501953125f,0.36990000495910635436302982270717620849609375f,0.0867999822616578597944680950604379177093505859375f,0.3048000230312339908778085373342037200927734375f,0.1870999945044520273995658499188721179962158203125f,-0.19259999422132967339393871952779591083526611328125f,-0.28680000491142276786149523104541003704071044921875f,-1.309800021839141237478543189354240894317626953125f,-1.44080001933574664008119725622236728668212890625f,1.409399986243247582251569838263094425201416015625f,1.571399987792968744315658113919198513031005859375f,-1.191599992775916661003066110424697399139404296875f,-1.25869999234676388510933975339867174625396728515625f,0.870599954748154658545900019817054271697998046875f,0.093799988579750248618438490666449069976806640625f,-0.7754999833256011054771761337178759276866912841796875f,-0.5107999998152255027861201597261242568492889404296875f,-1.43309996458291966092701841262169182300567626953125f,-1.16809997829198852770105077070184051990509033203125f,0.76769999127387933413046994246542453765869140625f,0.1286000056505205435541938641108572483062744140625f,-0.7259999957084648514182845246978104114532470703125f,-0.488900014317035669364486238919198513031005859375f,-0.0426000176906580207969454932026565074920654296875f,0.33319999419450763156191896996460855007171630859375f,-0.80580001397132861029604100622236728668212890625f,0.5674000159859655667560218716971576213836669921875f,1.1002000326991083767325108055956661701202392578125f,0.34580000652074804889934966922737658023834228515625f,0.5228999906502662664564695660374127328395843505859375f,-0.28219998812973512425372746292850933969020843505859375f,-0.2120999790072435331467204377986490726470947265625f,-0.6227999852716923356865663663484156131744384765625f,-0.6773000298857692058618340524844825267791748046875f,0.13219999312162400428860564716160297393798828125f,-0.82710002143830074761154946827446110546588897705078125f,0.2942000056982039257746919247438199818134307861328125f,-0.44510001500844964539282955229282379150390625f,0.27859999790191647406345509807579219341278076171875f,-0.5832999823808666661761890281923115253448486328125f,0.4971999883532525199569818141753785312175750732421875f,0.1949999868124721036366508997161872684955596923828125f,-0.289199985283613347775144575280137360095977783203125f,-0.8279000168204309062502943561412394046783447265625f,0.5140000072717665346999638131819665431976318359375f,-0.2549999932944775338228282635100185871124267578125f,0.1989999946057796620380031527020037174224853515625f,-0.5640999856233595011190118384547531604766845703125f,0.5997999878644943994032701084506697952747344970703125f,-0.29709999711215484552440102561376988887786865234375f,0.193600003042817103704464898328296840190887451171875f,-0.1611000268101694388178657391108572483062744140625f,-0.01579999698400502694539682124741375446319580078125f,-1.18839997742175995654179132543504238128662109375f,0.918399999475479233979058335535228252410888671875f,1.360500029444693836921942420303821563720703125f,1.3604000290393827299340046010911464691162109375f,0.2756999815285201549386329133994877338409423828125f,-0.4092000062942506843910450697876513004302978515625f,-0.64289998336732379069502485435805283486843109130859375f,-0.560499976489693185754958903999067842960357666015625f,-1.02839997205734334784210659563541412353515625f,-0.4084999995827676144699580618180334568023681640625f,0.33330001695156141749976086430251598358154296875f,0.2714999962449073933612453402020037174224853515625f,0.78930002301633361838639757479541003704071044921875f,0.86190000751465534190032258266001008450984954833984375f,0.534900015091896108287983224727213382720947265625f,0.290099995338916727405376150272786617279052734375f,1.09139997557401624561634889687411487102508544921875f,0.93569997631311441210755219799466431140899658203125f,-0.4823999815642832089679359341971576213836669921875f,-0.1487999730080369342744006644352339208126068115234375f,1.31220000772476197425930877216160297393798828125f,1.4351000156044957378753679222427308559417724609375f,0.56429997279942067933689031633548438549041748046875f,0.7235999898552893938585839350707828998565673828125f,1.30349996858835215363114912179298698902130126953125f,1.11829997552633297885904539725743234157562255859375f,0.535199996973573899339271520148031413555145263671875f,0.65670000793337823807860331726260483264923095703125f,-0.133499965906143103211434208787977695465087890625f,-0.3366999899029732290500760427676141262054443359375f,0.513899998521804235451782005839049816131591796875f,-0.4948000106632708394727160339243710041046142578125f,1.32930001415014231724853743799030780792236328125f,0.02679997510910059332900345907546579837799072265625f,-0.6223000027656553356791846454143524169921875f,-0.38639999375343325738185740192420780658721923828125f,-0.9933000339388851074318154132924973964691162109375f,-0.34400000444054601445031948969699442386627197265625f,-0.28070000343322742963891869294457137584686279296875f,-0.428199997770786211503946105949580669403076171875f,-0.0038999797463414864751030108891427516937255859375f,0.42139999583959575346625570091418921947479248046875f,0.128899999564886069691738157416693866252899169921875f,0.083200002688169405473672668449580669403076171875f,-0.842100020134449067654713871888816356658935546875f,-0.1576000005543232163063294137828052043914794921875f,-0.5075000131130220637487582280300557613372802734375f,0.2721999917805195678965901606716215610504150390625f,0.10929997354447795743226379272527992725372314453125f,-0.546799997758865341523915049037896096706390380859375f,-0.050300015223026406374629004858434200286865234375f,0.066200003969669296566280536353588104248046875f,0.97320000790357585884748914395458996295928955078125f,-0.40819999911189075447737195645458996295928955078125f,-0.664799998034536709923258968046866357326507568359375f,-0.2289999988153577081817502403282560408115386962890625f,-0.419600006425380644969891363871283829212188720703125f,-0.18839999634623527668964015902020037174224853515625f,-0.248600012505054479561295011080801486968994140625f,-0.242399991846084628122071080724708735942840576171875f,0.649400010085106060842008446343243122100830078125f,-0.5565000188946722659011356881819665431976318359375f,-0.364200067090987289475378929637372493743896484375f,-0.869300017130374413909521535970270633697509765625f,-0.4871999765813352922805279376916587352752685546875f,0.073500000774860296814949833787977695465087890625f,-0.313799980831145575166374328546226024627685546875f,0.3163000019907951099185083876363933086395263671875f,-0.6020999918222429414527141489088535308837890625f,-0.01509998490810371407633283524774014949798583984375f,0.681899986839294314222570392303168773651123046875f,0.2104999689459798872803730773739516735076904296875f,0.0294000091910362471026019193232059478759765625f,-0.225900006479025083905298743047751486301422119140625f,-0.11579997165203081976869725622236728668212890625f,0.150699994194507524980508605949580669403076171875f,0.7586999820649626968815937289036810398101806640625f,0.9239000071644785094804319669492542743682861328125f,-1.0649999799579374926139507806510664522647857666015625f,-0.7528999898493291542678207406424917280673980712890625f,0.086700017917156202429396216757595539093017578125f,-0.032699987399578134272815077565610408782958984375f,-1.077900004005432066378489253111183643341064453125f,-1.18039999476671209066580559010617434978485107421875f,-0.1754999921359121639596878594602458178997039794921875f,0.302399987897276734116758234449662268161773681640625f,-0.17339998628795161295101934229023754596710205078125f,0.192000008255243326260597314103506505489349365234375f,-0.328799976807832905478790053166449069976806640625f,0.0082000201225280733297040569595992565155029296875f,-0.2558999881505972240347546176053583621978759765625f,0.266199987727403719617313981871120631694793701171875f,-0.395500017020851524218727490733726881444454193115234375f,0.26140000306665889251434009565855376422405242919921875f,-0.7087000298976899870240231393836438655853271484375f,0.0078000068783758980117681858246214687824249267578125f,-0.4217999836027619409151157015003263950347900390625f,0.63539999223351484314292747512809000909328460693359375f,-0.02140000209808334830086096189916133880615234375f,-0.102600002789497324329204275272786617279052734375f,0.48480000425577163269963421043939888477325439453125f,-0.207999995768070267132543449406512081623077392578125f,0.2933999947816132713995784797589294612407684326171875f,0.102799993616342622004822260350920259952545166015625f,1.30350002342462545357193448580801486968994140625f,0.19160000827312462234885970246978104114532470703125f,0.402400002239644560830811315099708735942840576171875f,0.1694000025972723921796614376944489777088165283203125f,-0.327399997210502391453701420687139034271240234375f,0.372800013881921632474814032320864498615264892578125f,-0.77599999123811702617103946977294981479644775390625f,0.275199991273880062436774096568115055561065673828125f,0.4053000176072121263359804288484156131744384765625f,0.097399996465444615978412912227213382720947265625f,-0.34210000333338974609631577550317160785198211669921875f,0.0256000028789042932686470521730370819568634033203125f,1.23220005093812989827029014122672379016876220703125f,-0.0211999965190887706967259873636066913604736328125f,-0.305300009113550263606384760350920259952545166015625f,-0.300799996238946931725877220742404460906982421875f,0.41230000991821302847029073745943605899810791015625f,0.530599974417686581773523357696831226348876953125f,0.549900017066299984236366071854718029499053955078125f,0.409300005004554823262452600829419679939746856689453125f,0.12390004292726519707912302692420780658721923828125f,0.18090000931024530217428036849014461040496826171875f,1.311899976259469635664345332770608365535736083984375f,1.026199983704090268332720370381139218807220458984375f,-0.33809998276233699243675800971686840057373046875f,-0.53200000691413862341505591757595539093017578125f,-0.5795999930143356237977059208787977695465087890625f,-0.68760000070333493482621634029783308506011962890625f,0.0047999972522259071183725609444081783294677734375f,-0.3415999988108870955016982406959868967533111572265625f,0.010199979352950805377986398525536060333251953125f,-0.493099987828731656236413982696831226348876953125f,0.1670999999247488876363831877824850380420684814453125f,-0.136999989107251085584948668838478624820709228515625f,0.8442000170230858913100746576674282550811767578125f,0.7936000163793561767988649080507457256317138671875f,0.7679999847412108948674358543939888477325439453125f,0.88799997729063040452501809340901672840118408203125f,0.1568999695539474714678362943232059478759765625f,0.1218999972701073630787504953332245349884033203125f,0.16440000807344912203689091256819665431976318359375f,0.29009999880343662059800635688588954508304595947265625f,-0.41460002877712298641199595294892787933349609375f,-0.83580002142190945590982664725743234157562255859375f,-0.38579998983144747626283788122236728668212890625f,0.072300008875131549501702465931884944438934326171875f,-0.701400001502037184764049015939235687255859375f,0.2259999986886978007305515347979962825775146484375f,0.5071999874591828216807698481716215610504150390625f,-0.4733999862134457004003706970252096652984619140625f,1.204700031864643250401059049181640148162841796875f,0.16460000419020648365631132037378847599029541015625f,0.81970001919865609352200408466160297393798828125f,0.39979999591112136414494671043939888477325439453125f,-0.001799999696016296724110361537896096706390380859375f,0.099599999570846531327106276876293122768402099609375f,-0.996099999988079076729263761080801486968994140625f,0.505399994915723738131418940611183643341064453125f,0.2689000095486644426046041189692914485931396484375f,-0.25799998950958258348009621840901672840118408203125f,0.0642000259637836023784984718076884746551513671875f,-0.686600004696845900298285414464771747589111328125f,-0.2302999825149771861987346710520796477794647216796875f,0.323999994158744808459005071199499070644378662109375f,-0.796600026154518303656004718504846096038818359375f,-0.33680000014305111477597165503539144992828369140625f,-0.154800002765655531078436979441903531551361083984375f,-0.12980000008344649842229046043939888477325439453125f,0.1893000147461894044909058720804750919342041015625f,0.301000011503696374148830727790482342243194580078125f,0.2313999716997141575802743318490684032440185546875f,0.51779999704360957224480443983338773250579833984375f,-0.18880000705719002240812187665142118930816650390625f,0.3779999978542327720987259453977458178997039794921875f,-0.463600030684471420272529940120875835418701171875f,-0.586000000029802237122567021287977695465087890625f,-0.964900007641315227147060795687139034271240234375f,0.889600002402067158158160964376293122768402099609375f,-1.128899954712391462408049846999347209930419921875f,-1.1483999684810637376131126075051724910736083984375f,-0.020700028467178555047212284989655017852783203125f,-0.4895000024437905494778533466160297393798828125f,0.44249997191131118512430475675500929355621337890625f,-0.21410001184940341545370756648480892181396484375f,0.1146000043392181311219246708787977695465087890625f,-0.079400007551908569070064913830719888210296630859375f,1.2608999893426897642711992375552654266357421875f,1.296099974954127986848106957040727138519287109375f,-0.676199969100952813505500671453773975372314453125f,-0.7960999987959862522757248370908200740814208984375f,-1.22850001299381261787857511080801486968994140625f,-0.6576000230550764769077431992627680301666259765625f,0.5615999917328355905965509009547531604766845703125f,0.2230999682337044820457094829180277884006500244140625f,-0.339599978113174216787228942848742008209228515625f,0.25260001232624063050025142729282379150390625f,-0.18269999626576893803076018230058252811431884765625f,-0.018000000983476649452086348901502788066864013671875f,0.6056999986648552436463432968594133853912353515625f,0.514500019729137392232587444595992565155029296875f,0.764100004696846468732474022544920444488525390625f,0.0980000127553939393010296043939888477325439453125f,0.71459999927282336074085833388380706310272216796875f,0.4395999998688697729676277958787977695465087890625f,-0.848399980998038927282323129475116729736328125f,-0.102999972820281726626490126363933086395263671875f,0.299400016641616417700788588263094425201416015625f,0.125799993109703134308574590249918401241302490234375f,0.1801999976813792869734243140555918216705322265625f,-0.18520000125765800902399860206060111522674560546875f,0.51919999765157687221517335274256765842437744140625f,-0.188199988532066431190514776972122490406036376953125f,0.7317000191092493111000294447876513004302978515625f,-0.66700000572204576343438020558096468448638916015625f,-0.346000015713274766682161498465575277805328369140625f,0.7094000008463858808482882523094303905963897705078125f,-0.9006000054121017228681012056767940521240234375f,0.541799989861249997602499206550419330596923828125f,1.525400039172172483858958003111183643341064453125f,0.10099998930096631966080167330801486968994140625f,0.262299998984485804864874580744071863591670989990234375f,-0.26199999044835575734424537586164660751819610595703125f,0.2740000233650210503810740192420780658721923828125f,0.203800005418062202267037719138897955417633056640625f,-1.5394000458478931392392041743732988834381103515625f,-0.10140001521110519888679846189916133880615234375f,-0.7719000074028965485695152892731130123138427734375f,-0.124199985283613312248007787275128066539764404296875f,0.333700009036064226819462419371120631694793701171875f,-0.026799999099969884497340899542905390262603759765625f,-0.1044999905973671427972249148297123610973358154296875f,0.148999997138977047228536321199499070644378662109375f,-0.63929999418258631749267806299030780792236328125f,-0.29119998951554304511546433786861598491668701171875f,0.5092000009298320861717002117075026035308837890625f,0.254599991375207945765168915386311709880828857421875f,-0.8774000109195707608478187466971576213836669921875f,0.6425999849081041048748375033028423786163330078125f,-0.50819998950064171339136009919457137584686279296875f,-0.6370000090003014037165485206060111522674560546875f,-0.4064999941587448262225734652020037174224853515625f,-0.28059997531175628893151952070184051990509033203125f,-1.502700008559227029536486952565610408782958984375f,-1.3715000129342076462535260361619293689727783203125f,1.42769999593049323749482937273569405078887939453125f,1.06640000664293754795153290615417063236236572265625f,1.30469998507499695961087127216160297393798828125f,1.2901999912738801867817528545856475830078125f,-0.23850006365776010852641775272786617279052734375f,-0.780200027036666909907580702565610408782958984375f,-0.496799990494549259523182627162896096706390380859375f,-0.284499986741692023617389395440113730728626251220703125f,0.4280999815702433153319361736066639423370361328125f,0.425200007963180581782580702565610408782958984375f,0.23850001776218476834401371888816356658935546875f,0.7791999997377399012066234718076884746551513671875f,-0.0230999529600151021213605417869985103607177734375f,0.287500012814998484600437222979962825775146484375f,-0.20340000059306628799049576628021895885467529296875f,-0.394900007492303917189246931229718029499053955078125f,0.2480999963223933235667573171667754650115966796875f,0.0746999830633401362689482994028367102146148681640625f,-0.2765999759376054356607710360549390316009521484375f,0.274700002658367026242558495141565799713134765625f,0.4190999590992934287214666255749762058258056640625f,0.24160000052452090812948881648480892181396484375f,1.29180002040862973444745875895023345947265625f,0.440399993872642558923047317875898443162441253662109375f,0.769500000916421011964985154918394982814788818359375f,-0.0036000047564506143515927760745398700237274169921875f,-0.65309997352361648381702252663671970367431640625f,0.37939998356103910026604353333823382854461669921875f,0.06870000069141379839265937334857881069183349609375f,-0.066200007694959595028194598853588104248046875f,-0.125400003409385618624583003111183643341064453125f,-0.48459999137520792800160052138380706310272216796875f,0.0252999818444248347759639727883040904998779296875f,-0.001399998670816415824447176419198513031005859375f,-0.81699998843669874304396216757595539093017578125f,0.19619999070763594062327683786861598491668701171875f,0.318099991405009774325662874616682529449462890625f,-0.08460000031590464431019427138380706310272216796875f,0.469299988073110352360117758507840335369110107421875f,-0.10080000547766676533001373172737658023834228515625f,-0.1982000246673825127885493202484212815761566162109375f,-0.253799997818469957877596243633888661861419677734375f,-0.8932000123739243235831963829696178436279296875f,-0.0788000200212000123656252981163561344146728515625f,0.50949999111890775793654029257595539093017578125f,-0.174800000977516134526013047434389591217041015625f,0.026899993395804955298444838263094425201416015625f,0.125199986207485292055707759573124349117279052734375f,-0.99190003988742869722727846237830817699432373046875f,-0.1014000058233737178170486004091799259185791015625f,1.4124000001907344170604119426570832729339599609375f,-0.04980000187158584168400921043939888477325439453125f,0.03819998704195004535222324193455278873443603515625f,0.596100005650520348154941530083306133747100830078125f,-0.1796999905884266723887776606716215610504150390625f,0.514299978834390714155233581550419330596923828125f,-0.1572000121951102613593320711515843868255615234375f,-0.826599986964464239491690022987313568592071533203125f,-0.098199993374943783663866270217113196849822998046875f,0.523899985110759747186648382921703159809112548828125f,0.0502000045359134450251303860568441450595855712890625f,0.17500000342726718116637130151502788066864013671875f,-0.156600002014636974223549259477294981479644775390625f,-0.5333999997735023868017378845252096652984619140625f,-0.09839999932646747993203462101519107818603515625f,-0.694599990034103331737469488871283829212188720703125f,0.106999994844198198507001507095992565155029296875f,-0.4284000084161758703515943125239573419094085693359375f,-0.013300000783801113612980770994909107685089111328125f,-0.8058000083088874276882052072323858737945556640625f,0.10570000246465205417933930220897309482097625732421875f,0.194800002765655566605573767446912825107574462890625f,0.29589999753832818552012895452207885682582855224609375f,0.21090000290274613092833533301018178462982177734375f,0.306099991196393972359146573580801486968994140625f,-0.524700001019239525845705429674126207828521728515625f,-0.2502999995023011958750203120871447026729583740234375f,-0.007599992805719324451274587772786617279052734375f,-0.11099999935925008998083285405300557613372802734375f,-0.63570001118183139254824709496460855007171630859375f,0.37350000897049906445346323380363173782825469970703125f,0.145300017756223809328730567358434200286865234375f,-0.1508999957501888644628706970252096652984619140625f,1.03950000825524302427993461606092751026153564453125f,-0.1180999852955342266369598291930742561817169189453125f,-0.562300001648068370485589184681884944438934326171875f,0.38999998688697790072410498396493494510650634765625f,0.6977999995768069396717692143283784389495849609375f,-0.156799995970726246952153815072961151599884033203125f,0.29410001751184466911581694148480892181396484375f,0.488199967372417997779621146037243306636810302734375f,-0.4557000079035755657486106429132632911205291748046875f,-0.07950001496076541940283277654089033603668212890625f,-0.765999999582767099326474635745398700237274169921875f,0.1615000045895573776988385361619293689727783203125f,-1.14859998821616127173683707951568067073822021484375f,0.8756999792933459758614844758994877338409423828125f,1.015000008046627044677734375f,-0.8551999838232990924780096975155174732208251953125f,-0.30030000047087668235690216533839702606201171875f,0.100300010901689107267742429030477069318294525146484375f,-0.81390002353712953553355191615992225706577301025390625f,0.0631999973237513668067322214483283460140228271484375f,0.301300007429718963436471312888897955417633056640625f,-0.41749999605119203494041357771493494510650634765625f,0.361400019755959700518133104196749627590179443359375f,0.8777999726057057472417000099085271358489990234375f,0.3095999566555018844837832148186862468719482421875f,0.195799981151521418354377601644955575466156005859375f,-0.4110000020042059443170501253916881978511810302734375f,0.56489999542236279950202515465207397937774658203125f,0.833300028872490106124359954264946281909942626953125f,0.50580000115632994805991984321735799312591552734375f,-0.757700026142597238276721327565610408782958984375f,0.23969999639987948913955051466473378241062164306640625f,-0.38309998842477799296801777018117718398571014404296875f,-0.012400005480647013911266185459680855274200439453125f,-0.463499981254339221692362116300500929355621337890625f,-0.1699999991804361787472998912562616169452667236328125f,0.5592999828577041032673378140316344797611236572265625f,-0.22459999383389950278200331013067625463008880615234375f,-0.03670000137686724173136099125258624553680419921875f,-0.0320999952495097939930701613775454461574554443359375f,0.20410000648498538566855131648480892181396484375f,0.151699993553757739306320218020118772983551025390625f,-0.4813999822795391736462988774292171001434326171875f,-0.12730000813007347204575125942938029766082763671875f,-0.78609998508691791840874429908581078052520751953125f,0.24840000431835651539813625277020037174224853515625f,0.449900006747245839733295724727213382720947265625f,-0.02430000491142274288147717697938787750899791717529296875f,-0.022600003553181924331738628097809851169586181640625f,-0.14839999895393851669922469227458350360393524169921875f,0.25109998791813847862641750907641835510730743408203125f,-0.010099994519352950561597026535309851169586181640625f,0.49589998911917210211441897627082653343677520751953125f,-0.227399995422363332409076974727213382720947265625f,-0.52500001072883595298890213598497211933135986328125f,-0.1381999974355101468148632193333469331264495849609375f,0.106600002573430552299527107606991194188594818115234375f,-0.1609000055849552079934028370189480483531951904296875f,-0.33530000702738771423128127935342490673065185546875f,0.042999995768070231605406661401502788066864013671875f,0.799000001907348433860533987171947956085205078125f,0.214099997991323232060523196196299977600574493408203125f,0.5410000118575990502023387307417578995227813720703125f,0.64810000660419486706587122171185910701751708984375f,-0.14199999514222128027540748007595539093017578125f,0.31709998914003367787017850787378847599029541015625f,0.556700001823902113073927466757595539093017578125f,-1.017999978780746772599741234444081783294677734375f,0.22640001059174519326688823639415204524993896484375f,0.7017999849438663062528576119802892208099365234375f,0.06059998797774301948493302916176617145538330078125f,0.0857000126719480448400645400397479534149169921875f,-1.12120000396966901234918623231351375579833984375f,-0.04669999914169320476275970577262341976165771484375f,-0.942700037020445247293309876113198697566986083984375f,-0.06119998698234585532418350339867174625396728515625f,0.497400008684396599534238703199662268161773681640625f,-0.30129996235370715140788888675160706043243408203125f,-0.2274999822676182503755626385100185871124267578125f,-0.269399997977912253332988257170654833316802978515625f,0.6612000250175598115021102785249240696430206298828125f,0.27819998488128205860192565523902885615825653076171875f,0.193499991945922378278055475675500929355621337890625f,0.2031000058591363455207101651467382907867431640625f,0.6109000042438506028474876075051724910736083984375f,-0.118199990022182621629553977982141077518463134765625f,0.3681000013887880673024710631580092012882232666015625f,0.5920999678313734193579875864088535308837890625f,-0.16020000677108736653053711052052676677703857421875f,-0.35579998953342428791302154422737658023834228515625f,0.670999981701374093034928591805510222911834716796875f,-0.09530000225901603949640872315285378135740756988525390625f,-0.059400004776567261188802149263210594654083251953125f,0.026700001153349806060077753500081598758697509765625f,-0.8154000214397907342345206416212022304534912109375f,-0.055500001579523061678855810896493494510650634765625f,-0.27229999457001685669865764793939888477325439453125f,0.116399999341368687311160101671703159809112548828125f,0.91790000475049016159800885361619293689727783203125f,-0.097800007846951420020786827080883085727691650390625f,-0.603099984997510940587517325184307992458343505859375f,0.25070000402927394844709851895458996295928955078125f,-0.6205000061988830140080608543939888477325439453125f,-0.0061000014036893812630069078295491635799407958984375f,0.11460000523328783827992083388380706310272216796875f,0.0371999973684549178898350874078460037708282470703125f,-0.03750000610947612500467585050500929355621337890625f,0.249299989935755750281032305792905390262603759765625f,-0.206199997413158531145427332376129925251007080078125f,0.15500000115484002805033014738000929355621337890625f,-0.070799989943206298459443814863334409892559051513671875f,-0.048199998813867550584433274707407690584659576416015625f,-0.225699993821978528441007938454276882112026214599609375f,0.136899994662404100864705469575710594654083251953125f,-0.5817999892652034876761035775416530668735504150390625f,0.10019999768137932694145320056122727692127227783203125f,-0.108599997007846837959732511080801486968994140625f,-0.07010000122487536344806358101777732372283935546875f,-0.5269999759197234734386938725947402417659759521484375f,0.1094000011444091757795149533194489777088165283203125f,0.106800002676248340094389277510344982147216796875f,-0.0202999942123887056055764332995750010013580322265625f,-0.144600008735060736597688446636311709880828857421875f,0.3630999750137338111244389438070356845855712890625f,0.960700000005960674798188847489655017852783203125f,-0.34309997352361687461552719469182193279266357421875f,-0.427800000321865159236267572850920259952545166015625f,-0.5281000083923341748004531837068498134613037109375f,0.48360001921057715890128747560083866119384765625f,0.4337999923050406270164103261777199804782867431640625f,0.251999992981553067039612869848497211933135986328125f,0.5027999809503551631451045977883040904998779296875f,0.376200003075599642698989555356092751026153564453125f,-0.15099998557567584356320367078296840190887451171875f,-0.27719999744296064392301559564657509326934814453125f,0.361999970853329244846463552676141262054443359375f,1.0982999938547610696559786447323858737945556640625f,-0.05780000658035244764931803729268722236156463623046875f,-0.9271000008001919479028174464474432170391082763671875f,-0.007700000363588088703181711025536060333251953125f,0.350100016051531071070712641812860965728759765625f,0.08020002048015584250606480054557323455810546875f,0.3283999981343743712614013929851353168487548828125f,-0.0810999988704919072546317693195305764675140380859375f,-0.03309999915361405697922236868180334568023681640625f,0.25509998982548720558582999728969298303127288818359375f,-0.158900003550946633534834973033866845071315765380859375f,-0.515199999320506751843140591518022119998931884765625f,-0.58270000408887856480077971355058252811431884765625f,-0.22020002167224816957968869246542453765869140625f,-0.60320000864863398959414553246460855007171630859375f,0.12470000228583810741866955140721984207630157470703125f,-0.25480000187158580171598032393376342952251434326171875f,0.05130000787675381701546939439140260219573974609375f,-0.59059999289512621345465959166176617145538330078125f,0.2386999984562396814880003148573450744152069091796875f,-0.12310000272989274350265986868180334568023681640625f,-0.0443000006645917210335028357803821563720703125f,0.698200017291307428735080975457094609737396240234375f,0.00380000139474867548727843313827179372310638427734375f,-0.53040000236630435725970755811431445181369781494140625f,-0.2956000033259391646112135276780463755130767822265625f,0.068399997687339730845224039512686431407928466796875f,0.24379999938309193652230533189140260219573974609375f,-0.367699994403123842090508333058096468448638916015625f,-0.18559999341666699734787471243180334568023681640625f,-0.05330000234842291462200591922737658023834228515625f,0.0430000030696392199214272977769724093377590179443359375f,0.0144000058010220755022601224482059478759765625f,0.044500002741813649009827713598497211933135986328125f,-0.1099000034689903149143219707184471189975738525390625f,0.12779999831020827372185522108338773250579833984375f,-0.4029000124990940179259268916212022304534912109375f,-0.07409999739527706186237310248543508350849151611328125f,0.26659998882710933543194187222979962825775146484375f,-0.0734999997317790920714486446740920655429363250732421875f,-0.1234999988749623245354314349242486059665679931640625f,-0.0403999937981367462924708888749592006206512451171875f,0.7258999929189682287500318125239573419094085693359375f,-0.162199989172816305682545134914107620716094970703125f,-0.5670999806463716819138198843575082719326019287109375f,-0.7344999806880949666521019025822170078754425048828125f,-0.39909999329745760032750467871665023267269134521484375f,0.7624999934434892878698519780300557613372802734375f,0.2580999794840810324103586026467382907867431640625f,0.5949999980628495332979355225688777863979339599609375f,0.507299995389580704596710347686894237995147705078125f,0.71119997342228913339567952789366245269775390625f,-0.50800000485777818681754069984890520572662353515625f,-0.4132999945998196977825500653125345706939697265625f,0.55950000959634760278049725457094609737396240234375f,0.5175999545097358378598073613829910755157470703125f,0.21559997784495354977707393118180334568023681640625f,0.07570001118183089516833206289447844028472900390625f,0.497600005769729403226619979250244796276092529296875f,0.311299989026784851287033006883575581014156341552734375f,0.88110000626519313637885488788015209138393402099609375f,-0.437500016391276336236160204862244427204132080078125f,-1.2432000205695625805191184554132632911205291748046875f,0.52659998115301132060039890347979962825775146484375f,-0.01759999831914882406636024825274944305419921875f,0.567700013625621835444690077565610408782958984375f,1.09440000337958309728492167778313159942626953125f,-0.61099998833239077544732253954862244427204132080078125f,-0.4064999962821602697005118898232467472553253173828125f,0.14739999333620090737895225174725055694580078125f,0.8383999996244906327547141700051724910736083984375f,0.35949997815489798114185759914107620716094970703125f,-0.462899993425607281238853829563595354557037353515625f,-0.24519999634027467294572488754056394100189208984375f,0.3211000011801719011828026850707828998565673828125f,0.246500001758337028423540004951064474880695343017578125f,0.317300006118416799694159635691903531551361083984375f,-0.051299999457597778018680401146411895751953125f,-0.7905000117123126557316936668939888477325439453125f,0.01820000425279133526856867320020683109760284423828125f,-0.76510001813769334955139811427216045558452606201171875f,-0.077600000554323145252055837772786617279052734375f,-0.6327999867618083751352742183371447026729583740234375f,-0.073900003588199592030605344916693866252899169921875f,0.627500003725290245171208880492486059665679931640625f,-0.412799989518523258169580003595910966396331787109375f,0.32729998950362215737186488695442676544189453125f,0.0062999946892261515785094161401502788066864013671875f,-0.127500002682209068183283307007513940334320068359375f,-0.131300000202655808845264573392341844737529754638671875f,-0.107600003553181888804601840092800557613372802734375f,0.12750001028180124595934330500313080847263336181640625f,0.2821000158131123924931671353988349437713623046875f,-0.121000001519918409798037828295491635799407958984375f,-0.3403999938726425256163565791212022304534912109375f,0.033499999806284819214852177537977695465087890625f,-0.79660002540946006632793796597979962825775146484375f,0.025800002720952008861132753736455924808979034423828125f,0.6781999942690133398315310842008329927921295166015625f,-0.0017999982804060010721514117904007434844970703125f,-0.209699999827146488229345777654089033603668212890625f,-0.2338999955415725917617919549229554831981658935546875f,0.1360999910473823515388858140795491635799407958984375f,0.1552000007361173938846832243143580853939056396484375f,-0.530100016424059905517651714035309851169586181640625f,0.56179999068081343249758674573968164622783660888671875f,0.74450001373142005167693469047662802040576934814453125f,0.307599988782405853182666533029987476766109466552734375f,0.4552000024683773293787680813693441450595855712890625f,0.4180999943666157125932159033254720270633697509765625f,-0.12530001777485033631620581218157894909381866455078125f,-0.11420001389384282219907618127763271331787109375f,-0.3658000160574914616518071852624416351318359375f,0.3324000046610837699745388817973434925079345703125f,1.11089999962449059722757738200016319751739501953125f,-0.379299992096424176679647644050419330596923828125f,-0.07499999642372134900369928800500929355621337890625f,0.074400037503242089087507338263094425201416015625f,-1.1006000283598904587734068627469241619110107421875f,0.215799991023541082313386141322553157806396484375f,0.74190002126097720491770814987830817699432373046875f,-0.1928999894022940253535125521011650562286376953125f,0.5118000238358977327379761845804750919342041015625f,0.487400008237361959118061349727213382720947265625f,1.4467000448882583185650219093076884746551513671875f,-0.607199990737437911292317949119023978710174560546875f,0.0737000295996670384823801214224658906459808349609375f,0.2362999924540520879645555396564304828643798828125f,-0.06110001325011271688936176360584795475006103515625f,-0.010599978291988776391008286736905574798583984375f,-0.518000004559755478084071000921539962291717529296875f,0.1094999981224535900992123060859739780426025390625f,0.019599995845556161810918638366274535655975341796875f,0.23760000010728798969239505822770297527313232421875f,-0.56979999687969684618593646519002504646778106689453125f,-0.19820000388026237647665084296022541821002960205078125f,-0.241900007253885329649989444078528322279453277587890625f,-0.11160000007748603623181082866722135804593563079833984375f,-0.12770000149980187398313091762247495353221893310546875f,0.024299993530660923823916164110414683818817138671875f,-0.5405999841220676671582623384892940521240234375f,-0.200199999544024453967949739308096468448638916015625f,0.6105000038146972674013568394002504646778106689453125f,0.102400005331635401972789622959680855274200439453125f,-0.6789000175952910609566970379091799259185791015625f,0.021300000649690620235787719138897955417633056640625f,0.40029999823570250327975372783839702606201171875f,0.22239999378323549450442442321218550205230712890625f,-0.5434000125885010135107222595252096652984619140625f,0.124600004562735566793207908631302416324615478515625f,-0.57489999661445612133547911071218550205230712890625f,0.2980999987810850626601677504368126392364501953125f,-0.48119998534321783978384701185859739780426025390625f,0.02420000025928015219278677250258624553680419921875f,-0.6981000215053558832778435316868126392364501953125f,0.18920000383555890532960575001197867095470428466796875f,0.276100004905462270698990323580801486968994140625f,-0.241899996003508643838131320080719888210296630859375f,0.279599977815151223836664939881302416324615478515625f,0.1385999942511320082161319078295491635799407958984375f,-0.279900001233816198009662912227213382720947265625f,-0.0827000005125999582133999865618534386157989501953125f,0.0598999952733516582981110332184471189975738525390625f,-0.097200003701448434156873190659098327159881591796875f,-0.751500002205371941954581416212022304534912109375f,0.156499977171421278399066068232059478759765625f,0.171399982428550590185523105901665985584259033203125f,0.087199989619850981625859276391565799713134765625f,-0.15640000634491446618312693317420780658721923828125f,0.0118999943830073728889828998944722115993499755859375f,-0.67380001471266159907003157059079967439174652099609375f,-0.30229999084472591874828140134923160076141357421875f,-0.707500018179416656494140625f,-0.0922999997854232478999847444356419146060943603515625f,-0.68480000030696375201699765966623090207576751708984375f,-0.0405000026226043363664075513952411711215972900390625f,0.52539999335110165734619158683926798403263092041015625f,-0.823799981725215957339969463646411895751953125f,0.24389998674988699889354393235407769680023193359375f,0.5583999866604802964076270654913969337940216064453125f,-0.05710002080500153898157122966949827969074249267578125f,-0.337999993532895837144991446621133945882320404052734375f,-0.0074000085540117110927127441755146719515323638916015625f,-0.0372000044465057300158150610513985157012939453125f,-1.0923000111103053910710514173842966556549072265625f,-0.0068999802827836020924223703332245349884033203125f,0.06780001254081735595491409185342490673065185546875f,-0.45149997121095619689867817214690148830413818359375f,-0.59339999947547905634337439551018178462982177734375f,0.670699981975555470370409238967113196849822998046875f,0.533599997752904897652115323580801486968994140625f,-0.1845999976336953896094428273499943315982818603515625f,-0.43540000855028626602916119736619293689727783203125f,0.71049997520446783738634621840901672840118408203125f,-0.28800000187754637437365090590901672840118408203125f,-0.1664999969899654264082755616982467472553253173828125f,0.189300005209446009502016750047914683818817138671875f,-0.1008000014543533051636359232361428439617156982421875f,0.13610000676810740660727105932892300188541412353515625f,-0.23229999585524208072229157551191747188568115234375f,0.55089998094215986146338082107831723988056182861328125f,0.218300006520748102190054851234890520572662353515625f,-0.110700002092122939956198024447076022624969482421875f,0.019100000226497659827629149731365032494068145751953125f,0.097900003036856642069096778868697583675384521484375f,-0.021499999225139621028546343950438313186168670654296875f,0.071300000426173237855209663393907248973846435546875f,0.109800001946091629889679097686894237995147705078125f,0.70409999903440478874472319148480892181396484375f,-0.233200002092123048758054437712416984140872955322265625f,-0.11880000683367253344613345689140260219573974609375f,0.125200002300739299254672687311540357768535614013671875f,0.110500003535300539425634269719012081623077392578125f,0.268000005155801801492998492904007434844970703125f,-0.51719998462796201721403122064657509326934814453125f,0.252099996590614328084711814881302416324615478515625f,-0.572699997234344504448699808563105762004852294921875f,0.3183000038385390695339083322323858737945556640625f,-0.170100009644031491262694544275291264057159423828125f,-0.1351000008523464135379299477790482342243194580078125f,-0.5041000005245208637205678314785473048686981201171875f,0.11740000153183938191858715072157792747020721435546875f,-0.03099999317526813680245822979486547410488128662109375f,-0.1421000016570090895839939548750407993793487548828125f,0.0753999991253018198023028162424452602863311767578125f,0.5240999826431274044580277404747903347015380859375f,-0.3373999933362004099990372196771204471588134765625f,0.78769999812841451358735866961069405078887939453125f,0.115499994128942518045732867904007434844970703125f,0.912299975645542104984997422434389591217041015625f,0.53719999520778660695441431016661226749420166015625f,-0.116099988812208454458385631369310431182384490966796875f,0.4398000060997901972115187163581140339374542236328125f,0.517699993062019192535672118538059294223785400390625f,-0.3992000097215175902221062642638571560382843017578125f,-0.179100000226497524380420145462267100811004638671875f,-0.177900005495548185763254878111183643341064453125f,0.53019997547864949893892116961069405078887939453125f,0.4633999885976314292435063180164434015750885009765625f,-0.1857999767184264072739097173325717449188232421875f,-1.1699000219464306127292729797773063182830810546875f,-0.402199988949298425922052047098986804485321044921875f,-0.9963000255346299294245682176551781594753265380859375f,0.199700009401142519749328130274079740047454833984375f,0.4321000054195522910305271580000407993793487548828125f,-0.193899990475177474991141934879124164581298828125f,0.23940001634359386883943443535827100276947021484375f,0.20520001153945865990380070797982625663280487060546875f,-0.8225000255927448211679120504413731396198272705078125f,-0.0130999844014645372425320601905696094036102294921875f,-0.317399999520182607426477261469699442386627197265625f,-0.01240000168085086951208495520404540002346038818359375f,-0.297700012917816725366293439947185106575489044189453125f,0.725999951899052309300941487890668213367462158203125f,-0.231599994787573815191450421480112709105014801025390625f,-0.18370000084042548138540951185859739780426025390625f,-0.178300001752376591213078427244909107685089111328125f,-0.028499997422099188071342723560519516468048095703125f,-0.86540002129077908676890729111619293689727783203125f,-0.083900004333257616195851369411684572696685791015625f,-0.90529998601675032432467560283839702606201171875f,0.02609999745488165989737439076634473167359828948974609375f,0.0551999962843954250502065406180918216705322265625f,-0.12690000248551369654848031132132746279239654541015625f,-0.297900005048513438765667160623706877231597900390625f,0.2254999983757735027012358841602690517902374267578125f,-0.2975999753713607365313009722740389406681060791015625f,-0.15499999664723873582516944225062616169452667236328125f,-0.2901000007033347483087482032715342938899993896484375f,-0.013000006198883085062334430404007434844970703125f,0.30479999158978454687485282192938029766082763671875f,0.22500000394880770304695261074812151491641998291015625f,0.005300004643201905452087885350920259952545166015625f,0.15469999680966139976590056903660297393798828125f,-0.6098999968752265576910076561034657061100006103515625f,0.284700000870227842852955291164107620716094970703125f,-0.43609998925924298163181447307579219341278076171875f,-0.0648000064909458328710201158173731528222560882568359375f,-0.05260000565797096072628846741281449794769287109375f,0.15989999780654906569310469421907328069210052490234375f,-0.33550000098347665389297844740212894976139068603515625f,0.0152000009596347795959303539348184131085872650146484375f,-0.0057999979898333275940558451111428439617156982421875f,-0.30639999338090417868585291216732002794742584228515625f,0.8254000068366529330177172596449963748455047607421875f,-1.083899983620643325821220059879124164581298828125f,-0.3883000223159793762306435382924973964691162109375f,0.1820000218153001725340800476260483264923095703125f,-0.90239998454451519904750966816209256649017333984375f,-0.45109999432563796517570153810083866119384765625f,0.61840002942681326913998418604023754596710205078125f,0.409999979585409402016438207283499650657176971435546875f,-0.3361000063024459638683083539945073425769805908203125f,-1.108899981534481327116736792959272861480712890625f,-0.09639999271035204486679504043422639369964599609375f,0.144799996209144576919669589187833480536937713623046875f,-0.3563000045798718762313228580751456320285797119140625f,-0.390400001323223388993710614158771932125091552734375f,-0.3721999971449374555731992586515843868255615234375f,0.28699999238550655267232514233910478651523590087890625f,0.216399999453127378501449129544198513031005859375f,-0.0909999930262566092409315388067625463008880615234375f,-0.31960000813901434657537947714445181190967559814453125f,0.0340999991834161075843212529434822499752044677734375f,0.2348000107377767875505014671944081783294677734375f,0.5492999923944470452852328890003263950347900390625f,0.193300000560283624650992351234890520572662353515625f,0.30110001220703086488583721802569925785064697265625f,0.50740000153183917319665852119214832782745361328125f,0.365899964308738390172948129475116729736328125f,0.11510001649856604899468948133289813995361328125f,-0.2731999935984610328176813709433190524578094482421875f,0.13329999810159198769810018347925506532192230224609375f,-0.5908999763786793035791333750239573419094085693359375f,-0.19279999778866763193718725233338773250579833984375f,0.16470000593662259547045323415659368038177490234375f,0.91450000229477890201223999611102044582366943359375f,0.193200000676512662067807468702085316181182861328125f,-0.61250001654028896069803522550500929355621337890625f,-0.101299997892975834901108100893907248973846435546875f,0.731799988073110529995801698532886803150177001953125f,-0.134399998015165333953291337820701301097869873046875f,-0.157299998090416170981598042999394237995147705078125f,0.192499998435378127936701275757513940334320068359375f,0.8660000179111957407940280972979962825775146484375f,-0.03800000053644179975886885358704603277146816253662109375f,-0.0656999995030462802247939180233515799045562744140625f,0.0039999972879887213395022627082653343677520751953125f,0.4674000159859656999827848267159424722194671630859375f,-0.051800001782178861642336187287583015859127044677734375f,-0.228899996659159665757243828920763917267322540283203125f,0.057399999222159381329078087219386361539363861083984375f,-0.00909999970495700694073093472979962825775146484375f,-0.09130000199079513212296888013952411711215972900390625f,0.15769999738335604977379489355371333658695220947265625f,-0.1437000038206577290367249588598497211933135986328125f,-0.325300004047155422171044847345910966396331787109375f,0.13529999294579031499097254709340631961822509765625f,-0.68819999464154246737734865746460855007171630859375f,-0.3267000033140181614044195157475769519805908203125f,0.31310001718401903048061285517178475856781005859375f,0.107200001317262649802586338410037569701671600341796875f,0.068800002810359028870834663393907248973846435546875f,0.218899999415874457753261594916693866252899169921875f,1.07620001380443586214141760137863457202911376953125f,0.497700005877018014643908827565610408782958984375f,-0.65720001487731938283332056016661226749420166015625f,-0.2123999913245440129827557029784657061100006103515625f,-0.0398999970614910548505349652259610593318939208984375f,-0.543099992597102954050569678656756877899169921875f,-1.41980002422332773903690394945442676544189453125f,0.1705999768018724438434219337068498134613037109375f,-0.17020000632405274387792815105058252811431884765625f,0.1632999606251726021355352713726460933685302734375f,0.11099998617172257553420422482304275035858154296875f,0.2847999899506568066698264374281279742717742919921875f,-0.01200000654160982005436153485788963735103607177734375f,1.050200001108645864178470219485461711883544921875f,0.03130000646114350359994205064140260219573974609375f,-1.1839999718070028933425419381819665431976318359375f,-0.046500000119209516924456693232059478759765625f,0.829499995589255689765195711515843868255615234375f,0.00439999130964263684973047929815948009490966796875f,-0.3037999873876575662734467186965048313140869140625f,-1.329999980032443573918499168939888477325439453125f,0.52279998400509397615820716964663006365299224853515625f,0.5948999912872909856531578043359331786632537841796875f,1.028299972844123555404394210199825465679168701171875f,0.71920000689029706109067774377763271331787109375f,-0.2856000173330297542406697175465524196624755859375f,-1.10000002130866025851219092146493494510650634765625f,0.62929999567270211713321259594522416591644287109375f,0.6768000195145607023050615680404007434844970703125f,0.31519999872446025079852915951050817966461181640625f,0.168799995434284255679813213646411895751953125f,0.045099998468160595876952356775291264057159423828125f,-0.65590000871419906758319484652020037174224853515625f,0.021600000375509242900307071977294981479644775390625f,0.20189999466240404757400028756819665431976318359375f,0.019500003784894914815595257095992565155029296875f,0.66209999718666079360218645888380706310272216796875f,-0.312199995133280783221607634914107620716094970703125f,0.52309999006390572873215205618180334568023681640625f,-0.317299986523389865311628454946912825107574462890625f,0.52969998924732220046962538617663085460662841796875f,-0.129600000911951063198301881129737012088298797607421875f,-0.03700000102818012948091563885100185871124267578125f,-0.167400007492303881662110143224708735942840576171875f,-0.85140001774430285053085754043422639369964599609375f,0.22680000446438786099179196753539144992828369140625f,0.87319999702572825839297365746460855007171630859375f,-0.044300010424852320767286073532886803150177001953125f,-0.872899990296363892383624261128716170787811279296875f,0.00059999662041665402512080618180334568023681640625f,0.41999999910593022178773026098497211933135986328125f,-0.111999995887279546469272872855071909725666046142578125f,-0.386300009366869867566407492631697095930576324462890625f,-0.1959000062555074084258421862614341080188751220703125f,-0.7937999896228313811974430791451595723628997802734375f,0.003200011405348757165256756707094609737396240234375f,0.38150000205636036554324164171703159809112548828125f,-0.240100007110834123835729769780300557613372802734375f,-0.221900010383129231428256389335729181766510009765625f,-0.254800001424551059159284704946912825107574462890625f,0.09760001381635685646642741630785167217254638671875f,-1.0444000085949891598602334852330386638641357421875f,-0.25470001323819158045580479665659368038177490234375f,-0.049599995100498173172809401876293122768402099609375f,-1.2413000389456751992867111766827292740345001220703125f,0.307699994272738586165161223107133992016315460205078125f,0.095699996727705016752452138462103903293609619140625f,0.4604999945759775670239832834340631961822509765625f,1.08539997956752731766982833505608141422271728515625f,0.0946999788165090450320349191315472126007080078125f,0.16900001278519649616782771772705018520355224609375f,-0.495500005491078088226686304551549255847930908203125f,0.1070999992355703422930446322425268590450286865234375f,-0.02470002113580704872219939716160297393798828125f,-0.19460000538229937916412382037378847599029541015625f,-0.674299987328052186086324581992812454700469970703125f,-0.3469000006973743666094378568232059478759765625f,-0.00539999171197447225267751491628587245941162109375f,0.370500002995133304040109578636474907398223876953125f,-1.13609998180866256234367028810083866119384765625f,-0.1324999946355820412691173260100185871124267578125f,-0.6819999607205395886211363176698796451091766357421875f,-0.04169998625218884757970272403326816856861114501953125f,-0.881299974870681790406479194643907248973846435546875f,-0.4893999969720841392017973703332245349884033203125f,-0.57769998799562427560516653102240525186061859130859375f,-0.028299997766315920788571247612708248198032379150390625f,0.4984999584555627194504268118180334568023681640625f,0.0227999948084356418576135183684527873992919921875f,-0.4437999847054481250552271376363933086395263671875f,0.74700000005960465188081798260100185871124267578125f,0.172000007733702720003066133358515799045562744140625f,0.8449000267148019105434286757372319698333740234375f,0.09639999874532223511636175317107699811458587646484375f,-0.102299991887807806278942734934389591217041015625f,0.0762999957136810402769100392106338404119014739990234375f,-0.54079998385235672486714975093491375446319580078125f,-0.004899998551607165353516393224708735942840576171875f,-0.58230000634193412878403250942938029766082763671875f,-0.285500000163912748263328467146493494510650634765625f,0.53460000150799746876373319537378847599029541015625f,-0.0333000008955597337489962228573858737945556640625f,0.6530000158473849847240444432827644050121307373046875f,-0.235699998292326906579319256707094609737396240234375f,0.57840000439286232136737453402020037174224853515625f,0.0263000073552130952947436526301316916942596435546875f,0.7484999839365482632302928323042578995227813720703125f,0.20339999314248569106666764128021895885467529296875f,-0.47759997932016851063252715903217904269695281982421875f,0.25369999920129782111644090036861598491668701171875f,0.75950002166628838295991954510100185871124267578125f,0.266399995020031921111325345918885432183742523193359375f,0.19269999790489666935400236980058252811431884765625f,0.1065000030249356743894395549432374536991119384765625f,0.8147999892055988180317172009381465613842010498046875f,0.1937000033736228921110722467346931807696819305419921875f,0.23610000389963392564141031471081078052520751953125f,-0.343699998232722236934932880103588104248046875f,0.200800009649992006188767845742404460906982421875f,-0.02539999618232258171701687388122081756591796875f,0.7666000195980071652002152404747903347015380859375f,0.2963000249385832063353518606163561344146728515625f,-0.91289997181892346134191029705107212066650390625f,0.171499997138977011701399533194489777088165283203125f,0.9947000013172624388602116596302948892116546630859375f,0.1013000007987017170307808555662631988525390625f,1.16280003370046625121858596685342490673065185546875f,0.071300002288818831175376544706523418426513671875f,0.878200019079447002923188847489655017852783203125f,-0.1462999960303310587050873436965048313140869140625f,-0.6786000266611580400422099046409130096435546875f,0.2214999883472916764048932236619293689727783203125f,-0.263800016444921681113555678166449069976806640625f,0.2628999804615972379906452260911464691162109375f,0.7236000047564505877062401850707828998565673828125f,-0.761099978828430057120613128063268959522247314453125f,0.0043000302433970905013893570867367088794708251953125f,0.71309999103248056240289542984100989997386932373046875f,0.7105000168904662682933803807827644050121307373046875f,0.08569998048543947533062237198464572429656982421875f,-0.098600002521276319811249777558259665966033935546875f,0.7693999907135964377857817453332245349884033203125f,-0.059400003081560015516515704803168773651123046875f,-0.042199987310171549470538820969522930681705474853515625f,0.79860000328496061339222933384007774293422698974609375f,-0.0392000016748903501451195552363060414791107177734375f,0.58060000727474692094887132043368183076381683349609375f,0.071799989414215392713458641082979738712310791015625f,0.5452000054299832587645369130768813192844390869140625f,-0.15159999843835780808376512140966951847076416015625f,-0.43350001955032357869868064881302416324615478515625f,0.10129999752044671623707472463138401508331298828125f,-0.53570002265572558286521598347462713718414306640625f,-0.04270000036358834183403132556122727692127227783203125f,-0.15190000323057173403640263131819665431976318359375f,-0.105700004923343637841526287957094609737396240234375f,0.0666999979496001316192632657475769519805908203125f,0.12110000468194481726413869182579219341278076171875f,-0.403800007504224733878572806133888661861419677734375f,-0.13490000264942647589805346797220408916473388671875f,0.5137999961793422443179224501363933086395263671875f,-0.155500003069639181063621435896493494510650634765625f,0.1089000058233737799895379794179461896419525146484375f,-0.27370000069141386944693294935859739780426025390625f,0.2830999982595443498212262056767940521240234375f,0.21950000266730784748148153084912337362766265869140625f,0.39079999892115591109131855773739516735076904296875f,-0.132300009098648974070755457432824186980724334716796875f,-0.5688999939769507907527668066904880106449127197265625f,-0.1012999981909989966055718468851409852504730224609375f,0.1362000097811222332211400498636066913604736328125f,-0.0454000013977289285094229853712022304534912109375f,-0.796700004357099533791597423260100185871124267578125f,-0.00180000401735307125061780197938787750899791717529296875f,0.014899994211643896591112934402190148830413818359375f,0.030400001919269559191860707869636826217174530029296875f,-0.0518000002175569296269941332866437733173370361328125f,0.02820000253915788679393017446273006498813629150390625f,-0.175399993425607636510221709613688290119171142578125f,0.296000002190470734575455935555510222911834716796875f,-0.012399995571374944347553537227213382720947265625f,-0.2027000126384199685247722300118766725063323974609375f,-0.15940000432953238540534357525757513940334320068359375f,0.191299967122077863024287580628879368305206298828125f,0.3781999950885772676656415569595992565155029296875f,-0.8423999860346313983683330661733634769916534423828125f,-0.643800005790591267640365913393907248973846435546875f,0.6331999637961391869112048880197107791900634765625f,0.05689998937249196586662947083823382854461669921875f,0.50319999061822873187566074193455278873443603515625f,-0.04620001261234296663360510137863457202911376953125f,0.27219997933805029077802828396670520305633544921875f,-0.407099998155235898167347841081209480762481689453125f,-0.1042999882221220531164362910203635692596435546875f,-1.09350002253055578194107511080801486968994140625f,-0.022600011581182022357694449965492822229862213134765625f,-0.52550000856444223362728962456458248198032379150390625f,-0.68100000196695287257853124174289405345916748046875f,-0.29110002025365844247062341310083866119384765625f,-0.56159998115301146270894605549983680248260498046875f,0.1216000052928922769979180884547531604766845703125f,-0.35749997198581695556640625f,-0.3374999947845935821533203125f,-0.148699997226894087276605205261148512363433837890625f,0.1948999979928134340667611468234099447727203369140625f,0.299400016343593478040929767303168773651123046875f,-0.7736000384330754542361319181509315967559814453125f,0.0117000076353548365659662522375583648681640625f,0.69040001309514043015269635361619293689727783203125f,0.570099988782406086329501704312860965728759765625f,0.613499986320733992073428453295491635799407958984375f,-0.1025000003166496309603417103062383830547332763671875f,0.48060001235976812505867883373866789042949676513671875f,-0.2869000000268220329502355525619350373744964599609375f,0.4142000135958194828589284952613525092601776123046875f,0.09520000669658179592858004980371333658695220947265625f,0.80249999284744266692115388650563545525074005126953125f,-0.350399994096159961287639816873706877231597900390625f,0.056299990963935897525516338646411895751953125f,-0.0960000034570693827618015347979962825775146484375f,-0.36879999752044678729134830064140260219573974609375f,-0.16059999490678311673264033743180334568023681640625f,0.06799999554455282702036811315338127315044403076171875f,0.26029999853372576268384364084340631961822509765625f,-0.42069998719096179939924695645458996295928955078125f,0.01500000461935997730922309756351751275360584259033203125f,0.017700005411356745099737963755615055561065673828125f,0.155800004583597218044133114744909107685089111328125f,0.457000018090009785254324015113525092601776123046875f,0.030400001323223069249479522113688290119171142578125f,0.58399998790025708927942105219699442386627197265625f,0.2662999993830919009951685438863933086395263671875f,-0.17380000467300416033822330064140260219573974609375f,0.002299998258054269939520963816903531551361083984375f,0.159900003804266443996340285593760199844837188720703125f,0.1029999968111514618840374168939888477325439453125f,-0.550000007748603803037212855997495353221893310546875f,0.08469999744296075050442595966160297393798828125f,-0.2412999982655048558655153101426549255847930908203125f,-0.11249999791383746838846491300500929355621337890625f,0.24220001204609875600226587266661226749420166015625f,0.98299994540214630234231663052923977375030517578125f,0.289999985247850755598619798547588288784027099609375f,-0.05849998140335088692154386080801486968994140625f,-0.99730002377629300980288462596945464611053466796875f,-0.1110999919414521475147239470970816910266876220703125f,-1.1495000174194576203490214538760483264923095703125f,0.1229999908506869132907013408839702606201171875f,-0.192000010043382740576589640113525092601776123046875f,-0.872099972152710734007996506989002227783203125f,-0.069700000125170191722645540721714496612548828125f,0.7470999632120136624280348769389092922210693359375f,0.11949996817111951941114966757595539093017578125f,0.5876000191807744243988054222427308559417724609375f,0.99930000565648047228251016349531710147857666015625f,-0.05570000157058263567932954174466431140899658203125f,-0.395300021705031756624748595640994608402252197265625f,0.0669999938011169859919391456060111522674560546875f,1.038200026679039211785493534989655017852783203125f,0.0782000011980532594435544524458236992359161376953125f,0.29700000445544716409784768984536640346050262451171875f,-0.65829999221563273437141106114722788333892822265625f,-0.48840001571774482869159328402020037174224853515625f,-0.120800022614002244836228783242404460906982421875f,-0.98929999850392302818136158748529851436614990234375f,-0.329200005102157522429706659750081598758697509765625f,-0.45580000219941130268352935672737658023834228515625f,0.243599994623661331161201815120875835418701171875f,1.2203000408530240150639656349085271358489990234375f,-0.209500013172625898505430086515843868255615234375f,0.5618000041663646637601914335391484200954437255859375f,-0.2517999920219182730107831957866437733173370361328125f,-0.59169999184012400661458741524256765842437744140625f,0.223299997058510779623929920489899814128875732421875f,-0.39439999548196791323562138131819665431976318359375f,0.042299998369812963350256040939711965620517730712890625f,0.08940000136792658480544560006819665431976318359375f,-0.1318000007390975891752304960391484200954437255859375f,0.1668000008881092011137070585391484200954437255859375f,0.0528999942451715554625479853712022304534912109375f,0.9064000018000601954781814129091799259185791015625f,-0.27269999604225159117731891456060111522674560546875f,-0.258499999433755789368660771287977695465087890625f,0.011999996557831682508776793838478624820709228515625f,-0.76610002651214603730522867408581078052520751953125f,-0.191500000342726739432919202954508364200592041015625f,0.52329999094903467682371456248802132904529571533203125f,-0.04299999904632567648832264239899814128875732421875f,-0.03060000153779984799484736868180334568023681640625f,-0.015000001117587091759020268000313080847263336181640625f,-0.0719000019639730592047044410719536244869232177734375f,0.2979000064641237344176261103712022304534912109375f,0.114400012320280186628451701835729181766510009765625f,-0.149199999812245298613788691000081598758697509765625f,0.531700018215179426306349341757595539093017578125f,-0.0379999980032443840372025078977458178997039794921875f,0.4958000056266784749681164612411521375179290771484375f,0.05520000230073929259333453956060111522674560546875f,-0.4718000026762485532572100055404007434844970703125f,-0.090099990868568458068921245285309851169586181640625f,0.40410000484585761881817234097979962825775146484375f,0.6233999809980390605090860844939015805721282958984375f,0.07579999169409268322539219298050738871097564697265625f,-0.00659999277591705180157077847979962825775146484375f,0.00349999943375589595007113530300557613372802734375f,1.075299991381168496218379004858434200286865234375f,0.46140000045895579461330271442420780658721923828125f,-0.91619999029785414990101344301365315914154052734375f,-0.79960000033453104695269075818941928446292877197265625f,-0.9107999865531919425620799302123486995697021484375f,-0.18119998474717124992139360983856022357940673828125f,0.0936000437974922050443637999705970287322998046875f,-1.11580000756382968774005348677746951580047607421875f,0.1277000057838855173741876569692976772785186767578125f,0.25659999745115624758540207039914093911647796630859375f,-0.62339997682571368642356901546008884906768798828125f,-0.38300000813603407578966653090901672840118408203125f,-0.0844000145554542058334845933131873607635498046875f,1.12310004877448132987183271325193345546722412109375f,0.06449997055530598544237363967113196849822998046875f,0.66509998148083671054564547375775873661041259765625f,0.2806999943435191102736325774458236992359161376953125f,-0.21470001361072077461500384742976166307926177978515625f,0.007499998584389544475925504229962825775146484375f,-0.18940001194775124560010226559825241565704345703125f,-0.039099999630451076626513895462267100811004638671875f,0.8873999919950958936709639601758681237697601318359375f,-0.52299996268749282535281963646411895751953125f,-0.631999979048967208683507124078460037708282470703125f,-1.03580000383853843004544614814221858978271484375f,0.574299987328052541357692462042905390262603759765625f,-0.253800002139806746281891491889837197959423065185546875f,-0.1544000036031007994097308255732059478759765625f,0.223799996998906181033817119896411895751953125f,-0.50509998490810392723915356327779591083526611328125f,-0.154899999818205902357703962479718029499053955078125f,-0.9304000235259533013731925166212022304534912109375f,0.1193999982014299077182073460789979435503482818603515625f,0.84130002043098206510052250450826250016689300537109375f,0.079600005605816814835407058126293122768402099609375f,0.896099994772672658882584073580801486968994140625f,0.234399989297986000025275643565692007541656494140625f,-0.5927000101983548319140027160756289958953857421875f,0.119899995515495572107056432287208735942840576171875f,-0.43709999027624724110552278943941928446292877197265625f,0.3474000039905309478172057424671947956085205078125f,-0.045099997574090888718956193770281970500946044921875f,-0.243199995759129539152354482212103903293609619140625f,0.60169998781681055532999380375258624553680419921875f,-0.15009999377429483757850903202779591083526611328125f,0.235800008308887587560320753254927694797515869140625f,-0.087100001507997515393100229630363173782825469970703125f,-0.1197000010937452341153175439103506505489349365234375f,-0.1087000003933906722153324153623543679714202880859375f,0.24939999510943888783032207356882281601428985595703125f,0.03550000143051146306305554389837197959423065185546875f,-0.07850000062584872040361005929298698902130126953125f,0.3101999940305948388896695178118534386157989501953125f,-0.065699997323751535560631964472122490406036376953125f,0.1798000091731548621964975609444081783294677734375f,-0.3876000040188432027576936889090575277805328369140625f,-0.122699968065322073673684144523576833307743072509765625f,-0.43619998372271651110310131116420961916446685791015625f,-0.7870999587416651621651908499188721179962158203125f,-0.08279999153018025026540271937847137451171875f,0.317899998939036976253191824071109294891357421875f,0.005100002640485712390727712772786617279052734375f,0.2332000010490418162589776329696178436279296875f,0.84600000628828997406571943429298698902130126953125f,-0.4730999809741973649579449556767940521240234375f,0.0835000280439857078818022273480892181396484375f,-0.6735999663114551339049285161308944225311279296875f,0.3026000115811822155365007347427308559417724609375f,-0.380299960684777005326395737938582897186279296875f,0.02710002534985544997425677138380706310272216796875f,0.40509998207688369120660354383289813995361328125f,-0.1975999871432776444635237567126750946044921875f,-0.0884000017851587216455300222150981426239013671875f,0.56650003014504957121744155301712453365325927734375f,0.3315999894976613404651288874447345733642578125f,0.6077000219702721750536511535756289958953857421875f,0.49289998135566737147428284515626728534698486328125f,-0.115099995338916460951850240235216915607452392578125f,-0.432499963641167273209475752082653343677520751953125f,-0.1392999672114847431458883875166065990924835205078125f,-0.331900000250339388685461017303168773651123046875f,0.8411000448405747675906241056509315967559814453125f,0.297000011384486839460805640555918216705322265625f,-0.83429999343752836438170561450533568859100341796875f,-0.27539999119043301334386342205107212066650390625f,1.09859995334744464656751006259582936763763427734375f,-0.532800016713142809976488933898508548736572265625f,-1.387999991893767770534395822323858737945556640625f,-0.07169999943971649969398640678264200687408447265625f,-0.384200017619132694335348787717521190643310546875f,0.654600022369623335549704279401339590549468994140625f,1.418399989342689071492031871457584202289581298828125f,-0.097500012814998537891142404987476766109466552734375f,0.18330001158714281928041600622236728668212890625f,-1.071899991756677383847318196785636246204376220703125f,-0.25989994505643920774673460982739925384521484375f,-0.3930000032186509173470767564140260219573974609375f,0.22759999980926526319535696529783308506011962890625f,-0.006600016915798700978257329552434384822845458984375f,1.0706000200152399060016250587068498134613037109375f,-0.44490003610253392452023035730235278606414794921875f,-2.090899954771995084001900977455079555511474609375f,-0.5040000090599061621787768672220408916473388671875f,-0.74799999487400015141247422434389591217041015625f,-0.4791000219076873012369333082460798323154449462890625f,-0.8201999871015546172969834515242837369441986083984375f,0.98969999941736386261936786468140780925750732421875f,-0.1276000057324762071431223375839181244373321533203125f,1.3601999944031231137842041789554059505462646484375f,-0.225300024461745351800345815718173980712890625f,-0.2034000163137914540811834740452468395233154296875f,-0.918699973273277237240108661353588104248046875f,0.0152999992229044534663984222788712941110134124755859375f,0.00910000005885958085372067216667346656322479248046875f,-1.125900024956464573477887824992649257183074951171875f,0.2260999898552895626124836780945770442485809326171875f,0.245400006352365085415812018254655413329601287841796875f,-0.34669999539777640595872298945323564112186431884765625f,0.4475999959349632462135559762828052043914794921875f,-0.28850002230703797323485559900291264057159423828125f,-0.0999000071942806044944518362171947956085205078125f,-0.0545999994218349105068455173750407993793487548828125f,-0.1148000117063521230420519714243710041046142578125f,-1.0133999777197837399711488615139387547969818115234375f,0.0158999975755811373545611786539666354656219482421875f,0.353400000816583681473304068276775069534778594970703125f,0.379999998472630995394183628377504646778106689453125f,0.4786999822139741400661705483798868954181671142578125f,0.3195000137686729857477985206060111522674560546875f,0.27999999757856119941834549535997211933135986328125f,-0.100500007092952703402488623396493494510650634765625f,-0.502799992200732237535021340590901672840118408203125f,-0.19510000755786904846900142729282379150390625f,0.976999957665801499473445801413618028163909912109375f,0.13089999440908428596230805851519107818603515625f,0.3874000009730458327084079428459517657756805419921875f,-0.15840000543594356940957368351519107818603515625f,0.0322999949425459842444752212031744420528411865234375f,-0.190000001192093037616359652020037174224853515625f,0.7181999931141731519090853907982818782329559326171875f,0.00950000602006895178419654257595539093017578125f,0.310899999438225993486639708862639963626861572265625f,-0.45729999591112147072635707445442676544189453125f,0.3490999795883895640002947402535937726497650146484375f,0.2500999980211258133522278512828052043914794921875f,0.78909998890161492379746732694911770522594451904296875f,-0.358500021059066309447871390148065984249114990234375f,-0.88370001084282934211699966908781789243221282958984375f,-0.48929998866915713762182349455542862415313720703125f,-0.72030003638267459109556511975824832916259765625f,0.70680001802444447633888557902537286281585693359375f,0.75939998266696928652663700631819665431976318359375f,1.13580001724958368214402071316726505756378173828125f,-0.51300001037120779301403672434389591217041015625f,-0.18060000813156396048952956334687769412994384765625f,0.449799998667836486987425814731977880001068115234375f,-0.7646999991565943677329642014228738844394683837890625f,0.3004999853372576357202206054353155195713043212890625f,-1.08450003939867034574717763462103903293609619140625f,-0.5956000006437296434569361736066639423370361328125f,0.9048000199019907707764787119231186807155609130859375f,0.16839998561739921711932765902020037174224853515625f,0.271800027263164878377210698090493679046630859375f,2.03739994952678671324974857270717620849609375f,-0.8000999914646147459507119492627680301666259765625f,-0.2109999527931218921139588928781449794769287109375f,0.9099000134527681638019203091971576213836669921875f,1.1412999599695208274852120666764676570892333984375f,0.474300027561188386471258127130568027496337890625f,1.82460001208782074400005512870848178863525390625f,-0.845399995511769475342589430510997772216796875f,-0.560899936890603356687279301695525646209716796875f,0.6573000041067604826139358920045197010040283203125f,1.05889996633529648306648596189916133880615234375f,-0.1607999937057495998260492342524230480194091796875f,0.187699994738399933424943810678087174892425537109375f,-0.35979999948740015724979457445442676544189453125f,0.40459998589903134469381029703072272241115570068359375f,-0.17780000926256178672701935283839702606201171875f,0.7087999753549698578325433118152432143688201904296875f,-0.230599997588992078334513280424289405345916748046875f,-0.150099996083974840388464144780300557613372802734375f,-0.089800000530481316474151753936894237995147705078125f,0.085800006408989382133967183108325116336345672607421875f,-0.11859999849796309945304528810083866119384765625f,0.8007999873727558171054852209636010229587554931640625f,0.0188999988198279567086501629091799259185791015625f,-0.1959000132218002843131898771389387547969818115234375f,0.21610000885427005368200070734019391238689422607421875f,0.40660000533014528656394759309478104114532470703125f,-0.4804999981522559693303264793939888477325439453125f,0.032299998332560253544443185091949999332427978515625f,0.26650000756978986515832730219699442386627197265625f,0.5557999908745288042410948037286289036273956298828125f,0.41159998741149905754355131648480892181396484375f,-0.17280001622885432510656755766831338405609130859375f,0.1536999932408331659416944603435695171356201171875f,-0.5460000077784059602237221042742021381855010986328125f,0.06610000118017200776421304908581078052520751953125f,-0.0143999939918517672055031653144396841526031494140625f,0.492700003194808999751330702565610408782958984375f,0.1243999857962132526978393798344768583774566650390625f,0.2442999980568885121101629920303821563720703125f,-0.8420999787092211530392660279176197946071624755859375f,0.041500001311302270323722041212022304534912109375f,1.119099987411498631217909860424697399139404296875f,0.02130000437498136278691163170151412487030029296875f,0.396600005888938511589003610424697399139404296875f,0.5031000176310538307689057546667754650115966796875f,1.8603999521493914670600133831612765789031982421875f,0.6908999915778635259044904159964062273502349853515625f,0.402899973458052063080003790673799812793731689453125f,0.1040999960541724167484289864660240709781646728515625f,0.20660000693202018595684421597979962825775146484375f,0.0630000193119055751367341144941747188568115234375f,1.49600001239776503325629164464771747589111328125f,0.43620002229809795579740239190869033336639404296875f,0.248299975824355811937493854202330112457275390625f,-1.1304000390604140857675474762800149619579315185546875f,-0.5337999801233406227396471877000294625759124755859375f,0.78419998528361301026734508923254907131195068359375f,0.596899966573715801132493652403354644775390625f,-0.958800016593932991071369542623870074748992919921875f,0.049800008428096731449841172434389591217041015625f,-0.328499973356723984352356637828052043914794921875f,-0.31560001971721618474475690163671970367431640625f,-0.51059997084140729128876046161167323589324951171875f,-1.5843999653816229766789547284133732318878173828125f,-0.551400015509128760271551072946749627590179443359375f,-0.715999980509280931073590181767940521240234375f,-0.90300001037120836144822533242404460906982421875f,-0.445200019288062520672610844485461711883544921875f,-0.4412999836623665572687968960963189601898193359375f,-1.955499933481217311737054842524230480194091796875f,-0.67530002520680465494251620839349925518035888671875f,0.9548000078320499905970564213930629193782806396484375f,0.0643999963760375493393439683131873607635498046875f,0.2912000023305416629426645158673636615276336669921875f,-0.2457999888256192644320208273711614310741424560546875f,0.5629000122010705542408004475873894989490509033203125f,-0.12840001169443127082558930851519107818603515625f,0.4028999873161318578951295421575196087360382080078125f,-0.12079999564290044844483418273739516735076904296875f,0.130900013557076311254689926499850116670131683349609375f,-0.30449999916553505130423218361102044582366943359375f,0.357899987874925162856243332498706877231597900390625f,0.178100001016259223973747793934307992458343505859375f,-0.12839998368024863051317652207217179238796234130859375f,0.04799999606609350877306496840901672840118408203125f,-0.485799987819790946019793409504927694797515869140625f,-0.19529999540448184092156225233338773250579833984375f,0.5003000070646403951712954949471168220043182373046875f,-0.311399995057284717514534122528857551515102386474609375f,0.78470001135691991311915671758470125496387481689453125f,0.2239000162541866512100341424229554831981658935546875f,0.771999987877905358146790604223497211933135986328125f,0.0284999971985815392372387577779591083526611328125f,-0.921900012730061835242167944670654833316802978515625f,0.35599999570846563301529386080801486968994140625f,-0.09170001527219984183147971634753048419952392578125f,0.135800005999207495932523670489899814128875732421875f,0.98810000332593883598519823863171041011810302734375f,0.14040000333487989703229459337308071553707122802734375f,-0.205899999661743660084312068647705018520355224609375f,0.246399999713897699393783113919198513031005859375f,0.586000037431716691571637056767940521240234375f,-1.01639999852180462625028667389415204524993896484375f,-0.77279994041919763958503608591854572296142578125f,-0.403500028006732236462994478642940521240234375f,0.244300041791796207490961023722775280475616455078125f,0.5583000209748743003501658677123486995697021484375f,1.403499968886375626198059762828052043914794921875f,-0.2475000190734863991792735760100185871124267578125f,-0.229299971234798505292928894050419330596923828125f,0.615600010925531737626670292229391634464263916015625f,1.19369997953176465443902998231351375579833984375f,0.156600001642107855559515883214771747589111328125f,0.17400001218914962208828001166693866252899169921875f,0.682800008964538296396540317800827324390411376953125f,0.0641999851346017891273731947876513004302978515625f,-0.11940002468824406633984835934825241565704345703125f,0.5753000080704691043820275808684527873992919921875f,0.13049996352568216106959653188823722302913665771484375f,1.3368999951280660098973385174758732318878173828125f,-1.39170004839003080832071646000258624553680419921875f,-0.182000001624226026564201674773357808589935302734375f,0.14040003395676681208215086371637880802154541015625f,-1.5512999951362616002370486967265605926513671875f,-0.36779999540448216066579334437847137451171875f,-0.2775000321865075392224753159098327159881591796875f,-0.664500016376376123616864788345992565155029296875f,1.14150002470612488281176410964690148830413818359375f,-0.156300007839500987305569879026734270155429840087890625f,-0.4525000080838796900906118025886826217174530029296875f,-0.8349000028729438582786315237171947956085205078125f,0.68619997874945415361480627325363457202911376953125f,-0.2109999951496719372556754024117253720760345458984375f,0.290400033062696127927893030573613941669464111328125f,-0.00459999092817309218617083388380706310272216796875f,-0.47290000709742319173756186501123011112213134765625f,-0.16990001688003530944115482270717620849609375f,-0.9941999769762157601604712908738292753696441650390625f,0.092199999156594270033338034409098327159881591796875f,0.1196999966979026830404819747855071909725666046142578125f,-0.0300000029802323098238048260100185871124267578125f,-0.51459997651129985030138414003886282444000244140625f,0.374400004124641583302945946343243122100830078125f,0.9252000045359134450251303860568441450595855712890625f,-0.0842999959707260160257646930404007434844970703125f,-0.1828000083684921950322177508496679365634918212890625f,-0.02189999676719300847338445237255655229091644287109375f,1.003700008365511653352086796076036989688873291015625f,-0.15250000268220897936544133699499070644378662109375f,-0.91299997673183697077803344654967077076435089111328125f,-0.2018000013351439747566473670303821563720703125f,-1.20859997037202138159273090423084795475006103515625f,-0.095899990050494654525436999392695724964141845703125f,0.2802000043496486458849403788917697966098785400390625f,-0.23840001109838482307168305851519107818603515625f,-0.6820999930888411721952024890924803912639617919921875f,0.22919999839663507401610331726260483264923095703125f,0.370699994231760587837243292597122490406036376953125f,0.167000005982816279725966523983515799045562744140625f,-0.198099999824166150830251353909261524677276611328125f,0.0751000010013580521217591012828052043914794921875f,0.48619999368786803728426093584857881069183349609375f,-0.15540003391951362488043741905130445957183837890625f,0.102400048097968099369836636469699442386627197265625f,0.817199999529123477515213380684144794940948486328125f,0.2148999456524851581207258277572691440582275390625f,1.2249000299930568047557244426570832729339599609375f,-0.41579999847412096869447850622236728668212890625f,-0.1712999968498944980410669813863933086395263671875f,0.09950000110268586439588034409098327159881591796875f,-0.25289998731613128057915673707611858844757080078125f,-1.377799925518036872063021291978657245635986328125f,-1.394400044059754151248853304423391819000244140625f,0.2828000125408181020247866399586200714111328125f,1.07220000653266911427863306016661226749420166015625f,1.036399938917160312712439917959272861480712890625f,-0.597000027310103487110382047831080853939056396484375f,1.0839000235185027865014717463054694235324859619140625f,0.976500009059905860198114169179461896419525146484375f,-0.5833999967932701480322066345252096652984619140625f,1.17750000238418550679853069595992565155029296875f,-0.931800001931190280402006465010344982147216796875f,0.5444999838918447299107583603472448885440826416015625f,1.1849999731779099487738449170137755572795867919921875f,0.6483000171005723899497752427123486995697021484375f,-0.691500000119209090598815237171947956085205078125f,-0.808800004822015861094541833153925836086273193359375f,-0.14229998637437812902817313442938029766082763671875f,-0.8529000294119122305147584484075196087360382080078125f,-1.3722999956130974563706104163429699838161468505859375f,-0.61740001181364068116863563773222267627716064453125f,-0.26160002518594260845219423572416417300701141357421875f,-0.276299997222423598941531963646411895751953125f,-0.4037999992340803867563181484001688659191131591796875f,-0.232799999129772228201318284845910966396331787109375f,0.2119999945461750190389693671022541821002960205078125f,0.074999999403953410137546597979962825775146484375f,-0.964000012524425908821967823314480483531951904296875f,0.1021999977409839832631632816628552973270416259765625f,-0.35269999365806581348437021006247960031032562255859375f,-0.3814999920725821169753544381819665431976318359375f,0.04560001904666410865729631041176617145538330078125f,-0.403700013506412602737327688373625278472900390625f,-0.6471000092938539172138234789599664509296417236328125f,0.23110000222325322027927541057579219341278076171875f,0.6729999955818055656919796092552132904529571533203125f,-0.354399992240965300283050964935682713985443115234375f,-0.254199991765618360517464680015109479427337646484375f,-0.351299996626377009079078561626374721527099609375f,-0.09929998010098939342782387029728852212429046630859375f,0.413000004708766965677568805404007434844970703125f,-0.221900000175833778115475070080719888210296630859375f,0.0409000000715256106786910095252096652984619140625f,-0.98829996982663892435283514714683406054973602294921875f,-0.011700000035762769812208716757595539093017578125f,0.319999999925494160724070979995303787291049957275390625f,0.05940000010132795438266839482821524143218994140625f,1.02130000403970466749115075799636542797088623046875f,-0.127700000736117313948625451303087174892425537109375f,0.1056000110000371583485190285500721074640750885009765625f,0.18839999917745586799355805851519107818603515625f,-0.161600027942657931134817772544920444488525390625f,1.0122000086188318590529888751916587352752685546875f,0.3061999744653700616936475853435695171356201171875f,0.3984000037968160601309364210464991629123687744140625f,0.69359998970627767533869700855575501918792724609375f,0.4425000336766249375841653090901672840118408203125f,-1.160400014138222246629084111191332340240478515625f,0.00210000426471224077573651811690069735050201416015625f,-0.560099993476271595937987513025291264057159423828125f,0.68639996156692451023673129384405910968780517578125f,0.6007999794006355642750349943526089191436767578125f,1.216200046736002349234695429913699626922607421875f,-0.026900014853477927090352750383317470550537109375f,-0.55229999963641152049831362091936171054840087890625f,0.79860001369714694874346605502068996429443359375f,0.8663999756485221137580765571328811347484588623046875f,0.3029999638795859429052370614954270422458648681640625f,0.40769997205138164275695089600048959255218505859375f,-0.0302999958515162148842136957682669162750244140625f,-0.6591000102102757551847389549948275089263916015625f,-0.86690000054836247045386699028313159942626953125f,0.0995999964416025562741197063587605953216552734375f,-1.72169995980262768853208399377763271331787109375f,-0.044400003677606658669674288830719888210296630859375f,0.54239999959468843115928393672220408916473388671875f,-0.342000012576580303402806748636066913604736328125f,1.5647000110030173658515195711515843868255615234375f,0.011999989256262555414878079318441450595855712890625f,0.07360001444220554134290068759582936763763427734375f,-0.6542999964177607097326472285203635692596435546875f,0.335499980084598359297842762316577136516571044921875f,-0.154500005424022646138837444595992565155029296875f,-0.1445000015497204248759999245521612465381622314453125f,-0.3359999945163725243446606327779591083526611328125f,-1.1675999484002594908105265858466736972332000732421875f,0.35290000154674050492076275986619293689727783203125f,0.5091999958261845815599144771113060414791107177734375f,0.071399995541572724988554909941740334033966064453125f,-0.54739997109621774118437542711035348474979400634765625f,0.043299994078278558617967064492404460906982421875f,0.283599985235929719085135047862422652542591094970703125f,-0.497499998509884022723781527020037174224853515625f,0.020400010152160863441395122208632528781890869140625f,-0.0401000002190470272722677691490389406681060791015625f,0.473800001655518998600058466763584874570369720458984375f,0.26789999901354299982614293185179121792316436767578125f,-0.4668000206321476497350886347703635692596435546875f,-0.137800002408027477684981931815855205059051513671875f,-0.4126999710455538927789120862144045531749725341796875f,-0.43729999352693571523786886245943605899810791015625f,0.2524999805539847042012979727587662637233734130859375f,-0.01479999322891245583377894945442676544189453125f,0.92139998011887058737556799314916133880615234375f,0.1664000032901764569714941899292171001434326171875f,-0.63449997867643848081797841587103903293609619140625f,-0.018100002804398496181192967924289405345916748046875f,-0.64439998296499256813518741182633675634860992431640625f,0.19390000776052485065292785293422639369964599609375f,0.7051000120282171490515565892565064132213592529296875f,0.34180000453889369804727493828977458178997039794921875f,0.03449998933076880991421830913168378174304962158203125f,-0.28050000426173227197068626992404460906982421875f,-0.845099987888335846264453721232712268829345703125f,1.46159997489452297969592109438963234424591064453125f,-0.613199996876716113547445274889469146728515625f,-0.798899998670816291479468418401665985584259033203125f,0.280800019931792821381577596184797585010528564453125f,0.1796999879062168847809743965626694262027740478515625f,-0.72499998435378021355290911742486059665679931640625f,0.051000010609626666990834564785473048686981201171875f,0.667599974477291357999320098315365612506866455078125f,0.7461000160813338055731946951709687709808349609375f,0.5016000350952136699334005243144929409027099609375f,-0.232500011362135605708090224652551114559173583984375f,0.545800005999207638041070822509936988353729248046875f,-0.2528999719306821436504151279223151504993438720703125f,-0.9432000083133582624128621318959631025791168212890625f,1.0595999879479400807014144447748549282550811767578125f,0.51389998138547010597676489851437509059906005859375f,0.717900007134676609865664431708864867687225341796875f,0.9794999696612354256330945645458996295928955078125f,-0.023999996244906895981330308131873607635498046875f,1.727399981117248017881138366647064685821533203125f,-0.35370001723170307883492569089867174625396728515625f,-1.5230999726295468832404367276467382907867431640625f,0.32820000291168671679997714818455278873443603515625f,-0.80919998736977571951456411625258624553680419921875f,-1.185300043088198140850408890401013195514678955078125f,-0.0594999678730960823713758145458996295928955078125f,-0.4716000045850872179897805835935287177562713623046875f,-0.185799994152784364587205345742404460906982421875f,0.18860000376924868081829345101141370832920074462890625f,0.024800003287195837575751511394628323614597320556640625f,0.2838000031828880054263208876363933086395263671875f,-0.15319999434351949663124514700029976665973663330078125f,0.026299993497133300479617901146411895751953125f,-0.02610000401139245251869169806013815104961395263671875f,0.307299986374378164555309922434389591217041015625f,-0.17990000931769589698205891181714832782745361328125f,-0.542200000870227682980839745141565799713134765625f,0.1869000302761791854067041640519164502620697021484375f,-0.06980000358521942782630276269628666341304779052734375f,-0.7037000039324163846998771987273357808589935302734375f,0.2154999996237456638681351250852458178997039794921875f,-0.018500000551342932197940172045491635799407958984375f,-0.3029999921917916338998111314140260219573974609375f,-1.3237000016227364884713324499898590147495269775390625f,0.367299996209144552494763047434389591217041015625f,-0.52670002994984355115803964508813805878162384033203125f,0.436099997603893285713638761080801486968994140625f,-0.1857000035077334132438409142196178436279296875f,0.3106999998569488496968915569595992565155029296875f,0.268399999848007997371723831747658550739288330078125f,0.0032000025391578645894696819595992565155029296875f,0.430100001969933476431151575525291264057159423828125f,0.3082000040292740550285088829696178436279296875f,-0.2751999804705385077596702103619463741779327392578125f,-0.361699994075298292273146216757595539093017578125f,0.1430999918520452496295547462068498134613037109375f,-0.20849999585747713837236005929298698902130126953125f,0.11699997830390973518888131366111338138580322265625f,0.018900006885081754859356806264258921146392822265625f,0.2908000055894250923671506825485266745090484619140625f,-0.1331999941945074539262350299395620822906494140625f,-0.153999990582466494970503845252096652984619140625f,0.30870001037716843228508878382854163646697998046875f,0.40769997875690489053113196860067546367645263671875f,-0.3689999630153177889724247506819665431976318359375f,-0.505299983334541735757738933898508548736572265625f,0.731699973362683664390715421177446842193603515625f,0.751500016808510196142378845252096652984619140625f,0.905100025625527049299989812425337731838226318359375f,-0.1835000281557441770274863301892764866352081298828125f,-0.78150000157207244466661677506635896861553192138671875f,-0.87109997271895422965570787710021249949932098388671875f,1.42620001305937815772040266892872750759124755859375f,-0.925500044941902189066240680404007434844970703125f,0.11280002729296700891836735536344349384307861328125f,-0.4377000598192211100467829965054988861083984375f,-0.3050999865472316940895325387828052043914794921875f,-0.279900004959106496471576974727213382720947265625f,-0.47399999594688413395715542719699442386627197265625f,0.218000012159347278384302626363933086395263671875f,-0.63870001395344733197134701185859739780426025390625f,-0.05169997291564953911802149377763271331787109375f,-0.0774000097274782916656477027572691440582275390625f,-0.28399998998641962089095613919198513031005859375f,0.3674999792873858694974842364899814128875732421875f,0.36969996719360409542787238024175167083740234375f,-0.11730002481937429337222056346945464611053466796875f,-0.103699991489946796718868426978588104248046875f,0.45210000009238715534820585162378847599029541015625f,0.276300000761449116026824412983842194080352783203125f,0.005599998855590815338700849679298698902130126953125f,-0.13149999933689837661177080008201301097869873046875f,0.3288000049710273486880396376363933086395263671875f,0.0302000028967854827754990765242837369441986083984375f,0.190300015521049470379466583835892379283905029296875f,0.4626999826684594641079684151918627321720123291015625f,0.2880000011324881370455841533839702606201171875f,-0.02970002381801606361477752216160297393798828125f,-0.061400004780292505301986238919198513031005859375f,-0.45819999248087395216089134919457137584686279296875f,-0.17200001198053360695894298260100185871124267578125f,-0.2591000009715556107181555489660240709781646728515625f,0.460700004029273912919961730949580669403076171875f,-0.33620000229328894647551351226866245269775390625f,0.367699995744228402827502577565610408782958984375f,-0.451500018149614046336637329659424722194671630859375f,-0.09529999585151660568271836382336914539337158203125f,0.4006000135332346001604264529305510222911834716796875f,0.30790001920461662621164578013122081756591796875f,1.35090000573396640248802214046008884906768798828125f,-0.21199999171495420569044654257595539093017578125f,0.982300019492208686955336816026829183101654052734375f,0.32829999996423708807924413122236728668212890625f,0.7451999796882271542841635891818441450595855712890625f,-0.02760000070333479271766918827779591083526611328125f,0.367099996665120276162497248151339590549468994140625f,-0.18480000246763239601932582445442676544189453125f,-0.699099977874755751372504164464771747589111328125f,-0.969599992418288980644547336851246654987335205078125f,-0.291099961543083818327204426168464124202728271484375f,0.3342000083804130827758172017638571560382843017578125f,0.21589999068379395197325720801018178462982177734375f,-0.51660000201463684987857050145976245403289794921875f,-0.2797999842882159526880059274844825267791748046875f,-1.1586000363469122209636452680570073425769805908203125f,-0.49430000863671263999776783748529851436614990234375f,0.0837000276625161632182425819337368011474609375f,0.389699967789649548421948566101491451263427734375f,-0.0155999943852425104751091566868126392364501953125f,-0.9761999798297882335873509873636066913604736328125f,1.055100034528970720515417269780300557613372802734375f,0.7646999806046483882937536691315472126007080078125f,-0.4688999871969219412903839838691055774688720703125f,-0.76899998879432729381733224727213382720947265625f,-0.0230999870836736675983047462068498134613037109375f,-0.480500049114226612800848670303821563720703125f,-0.562200039315224220359823448234237730503082275390625f,-0.1157999814867967103282353491522371768951416015625f,-1.451100061231851867660225252620875835418701171875f,0.949100018405914624963770620524883270263671875f,0.04919995011687205987982451915740966796875f,1.719699994611740834216107032261788845062255859375f,-1.0850999846100801704551486182026565074920654296875f,0.3112000263214103057407555752433836460113525390625f,-0.609300011616945358383645725552923977375030517578125f,-0.1678999834418295478144500521011650562286376953125f,0.4962000236392025698251018184237182140350341796875f,-0.13020000449866053759961914693121798336505889892578125f,0.16780000017285345847994904033839702606201171875f,0.13610000341534611578708791057579219341278076171875f,-0.197099993163347253499750877381302416324615478515625f,0.7257999940410255756972901508561335504055023193359375f,0.098899987494945662547252140939235687255859375f,-0.17539998582601545873416171161807142198085784912109375f,0.2640999921798705685205277404747903347015380859375f,-0.0448000076830386007031847839243710041046142578125f,-0.11180000411048529596058642709976993501186370849609375f,-0.772999992564320503873886991641484200954437255859375f,0.3095999936103819294430650188587605953216552734375f,-1.006400008431076908976820050156675279140472412109375f,-0.32279999287128458007600784185342490673065185546875f,0.944999975562095695380548932007513940334320068359375f,0.203000003814697294046709430404007434844970703125f,0.3171999895825983184494134548003785312175750732421875f,0.163199993821978583952159169712103903293609619140625f,-0.26210000880956629831786131035187281668186187744140625f,0.2203000010669231567561610063421539962291717529296875f,-0.6497999984443187937444008639431558549404144287109375f,0.26369999338984495551585496286861598491668701171875f,0.275599982911348628800141113970312289893627166748046875f,-0.18410001155138022710389122948981821537017822265625f,-0.960299992722272843792552521335892379283905029296875f,-0.1966000013440847293821889252285473048686981201171875f,0.0741999983593821799132683736388571560382843017578125f,0.31649999713897702946496792719699442386627197265625f,-0.14959999878853569299508308176882565021514892578125f,0.21559999647140504208664424368180334568023681640625f,-0.6017999833419922506294597042142413556575775146484375f,-0.8205000054538249543156780418939888477325439453125f,-0.68169996337890648163693185779266059398651123046875f,-0.2505000216960906467278391573927365243434906005859375f,0.82659999858736998845643029198981821537017822265625f,0.53520001273155148879823173047043383121490478515625f,0.51699999082088510249377577565610408782958984375f,-0.44699997711181627124688020558096468448638916015625f,-0.402399984693527557055858778767287731170654296875f,0.411900006825476783234307731618173420429229736328125f,-1.0490000007525084324555564307956956326961517333984375f,-0.26939995703697139362020607222802937030792236328125f,-1.959199974703789592922476003877818584442138671875f,0.281399996137619456959555463981814682483673095703125f,1.8421999662995336422000036691315472126007080078125f,-0.70170001612901700127622461877763271331787109375f,-0.127200009512900891195386066101491451263427734375f,0.148500016793608580201180302537977695465087890625f,-0.11230002258419968796943066990934312343597412109375f,0.11940000434815878360694796356256119906902313232421875f,-0.835000015571713216644411659217439591884613037109375f,-0.30419999362826377620194762130267918109893798828125f,0.4131999685645109110510020400397479534149169921875f,1.130100044661760616548917823820374906063079833984375f,-0.4980999955773359744171102647669613361358642578125f,0.62010000390708441830867059252341277897357940673828125f,0.7765999782472849943815162987448275089263916015625f,-0.363899986004829312236097393906675279140472412109375f,-0.4291999833464625879742015968076884746551513671875f,-0.222899991786480011768389886128716170787811279296875f,-0.3571000000551343145360760900075547397136688232421875f,0.30130000050067895500660597463138401508331298828125f,-0.581600011141598205455238712602294981479644775390625f,-0.54250000417232513427734375f,-0.1688999728173019576615843106992542743682861328125f,0.11709999767094847467063800650066696107387542724609375f,-0.5684000043928623124855903370189480483531951904296875f,-0.362300005149841286566925191436894237995147705078125f,0.66350000990182156357377607491798698902130126953125f,0.22300000806152830978845713616465218365192413330078125f,0.31200000706315034193494284409098327159881591796875f,0.309199996161460877175386485760100185871124267578125f,-0.267800003115832740974155967705883085727691650390625f,0.05639999367892745141261912067420780658721923828125f,0.28749998055398495733214758729445748031139373779296875f,-0.059799998891353567387341172434389591217041015625f,0.124200004990398849002986025880090892314910888671875f,-0.40269999693632119175390471355058252811431884765625f,0.2944999995753170285439637154922820627689361572265625f,-0.13030000240802763755709747783839702606201171875f,-0.47239998581111442010893597398535348474979400634765625f,0.061300001618266097835885375388897955417633056640625f,0.53779998817741880845488822160405106842517852783203125f,-0.100000005364417887676609097979962825775146484375f,-0.880199963632226189957918904838152229785919189453125f,-0.3809999967515469432299823893117718398571014404296875f,0.3561999826610089403544634478748776018619537353515625f,0.15590000036954876350137055851519107818603515625f,-0.1056000052630901020478404461755417287349700927734375f,0.0167999932885170011331865680404007434844970703125f,0.5554999486431484445603246058453805744647979736328125f,0.026700019407272623794824539800174534320831298828125f,-0.7388000198721886846442430396564304828643798828125f,0.978900005227327341117415926419198513031005859375f,0.4073000349521631591187542653642594814300537109375f,0.305100017914176124822489555299398489296436309814453125f,-0.5392000154212117646324031738913618028163909912109375f,0.2813999988198279122997291779029183089733123779296875f,1.25370002378821343569370583281852304935455322265625f,-0.9405000396892433389695042933453805744647979736328125f,-0.4451999907895916042122053113416768610477447509765625f,-0.4202999891459942016780360063421539962291717529296875f,-0.74379999379515648882943423814140260219573974609375f,0.072300009918212726489628039416857063770294189453125f,0.719699961829185941297737372224219143390655517578125f,0.0476999900817869360025724745355546474456787109375f,0.2369999991655351578856425476260483264923095703125f,0.588599978679418267546452625538222491741180419921875f,0.9210999683976179852606946951709687709808349609375f,0.59280000747442240793816381483338773250579833984375f,-0.2395999930143357659062530728988349437713623046875f,-0.4002000058025121376203969703055918216705322265625f,0.55339999276995666122047623503021895885467529296875f,0.8771999966979020513235809630714356899261474609375f,-0.59739998946189842854437301866710186004638671875f,0.67080000540316142565444579304312355816364288330078125f,1.178999966815113875640008700429461896419525146484375f,-0.3545999953985212727047837688587605953216552734375f,-0.5159999939203263608078486868180334568023681640625f,1.00110002785325047369724416057579219341278076171875f,-1.013899988873303126268865526071749627590179443359375f,-0.353500007033348051521670640795491635799407958984375f,0.887900010599195876892508749733678996562957763671875f,0.15769999946951873681655342807061970233917236328125f,0.5859000002205372314989517690264619886875152587890625f,0.135500001132488279154131305404007434844970703125f,0.76989999601840963805443607270717620849609375f,0.0779999983012675812688030418939888477325439453125f,-0.235600006566941744257093205305864103138446807861328125f,0.2645999937221408249143905777600593864917755126953125f,0.094299989265203609800636286308872513473033905029296875f,-0.1059999900460244504074580618180334568023681640625f,0.160899998059868654110005081747658550739288330078125f,0.205599996247887606415361005929298698902130126953125f,0.16839998487234131285816829404211603105068206787109375f,0.1317000104665755344512945157475769519805908203125f,1.1053999874651430079808278605923987925052642822265625f,-0.21850000494718546661943037179298698902130126953125f,0.5737000011384487674348520158673636615276336669921875f,-0.08780000568628310020358185283839702606201171875f,-0.2129000058680772244912304813624359667301177978515625f,-0.0421999987840653290049885981716215610504150390625f,0.81379995173662944552006592857651412487030029296875f,0.21240001240968719997681546374224126338958740234375f,0.63990001866817447506718963268212974071502685546875f,0.0782000019431112747270162799395620822906494140625f,-0.814500010713934852191187019343487918376922607421875f,0.195799999070167540793363514239899814128875732421875f,-0.178899996547400963464724554796703159809112548828125f,0.1618999927997588628159064683131873607635498046875f,-1.828099988722801327867273357696831226348876953125f,0.2766000011205671516023585354560054838657379150390625f,1.69010002678036652667969974572770297527313232421875f,-0.5085000138878825737265287898480892181396484375f,0.60690000203847915827282122336328029632568359375f,-0.4149000083863736421108114882372319698333740234375f,1.066100000286102300606216886080801486968994140625f,0.329100017958879664803362175007350742816925048828125f,0.091199984002113154701874009333550930023193359375f,-0.014099985437095341467284015379846096038818359375f,0.466499974414706475300818055984564125537872314453125f,-0.50790000892281472744116399553604423999786376953125f,-0.515299966049195035111551987938582897186279296875f,-0.09119998660981654126089779310859739780426025390625f,0.43479997639060030678592738695442676544189453125f,-1.06200001674890476266455152654089033603668212890625f,0.81220002322196904742668266408145427703857421875f,0.315299998980760332045747418305836617946624755859375f,1.2926999945521355783739636535756289958953857421875f,1.0683000224649905618434786447323858737945556640625f,0.968899955010414259959361515939235687255859375f,0.04230001051425968938701771548949182033538818359375f,1.580199926900863971468425006605684757232666015625f,-1.061100058847666449679536526673473417758941650390625f,0.0081000194191940266819074167869985103607177734375f,0.09990002418160504049637893331237137317657470703125f,-1.3431000000476842615171335637569427490234375f,-0.4979999993443489092470599644002504646778106689453125f,-0.81029997871518144592073440435342490673065185546875f,0.4464000341355802703446897794492542743682861328125f,-0.37629999588131901600007722663576714694499969482421875f,0.0366999963477254720345399618963710963726043701171875f,0.7167999931395054868943361725541763007640838623046875f,0.03600000584125506719601617078296840190887451171875f,0.73739997739195839443482327624224126338958740234375f,-0.004399999654292940931554767303168773651123046875f,-0.2546999971821903852031709902803413569927215576171875f,-0.01559999559596192408861270450870506465435028076171875f,0.6613999671548607128812591327005065977573394775390625f,-0.3617999995470047025492021930404007434844970703125f,0.25169999433606882188740883066202513873577117919921875f,0.31649998976290227847840696995262987911701202392578125f,-0.493599987918138527476230592583306133747100830078125f,-0.197099994504451814236745121888816356658935546875f,0.1006999731093649597113426352734677493572235107421875f,0.2370000069141386944693294935859739780426025390625f,-1.0114000141307710745053327627829276025295257568359375f,0.06490000793933869971397143672220408916473388671875f,0.217999996475875423840307121281512081623077392578125f,-0.087900007879734065596721848123706877231597900390625f,-0.84389999725520603579553835515980608761310577392578125f,-0.300500000238418607523271930404007434844970703125f,-0.602199996101856260821705291164107620716094970703125f,-0.236199996817112012337247506366111338138580322265625f,0.54849998915195474769035399731365032494068145751953125f,-0.09210000255107875233306913287378847599029541015625f,-0.3427999868735671640251894132234156131744384765625f,0.278199989761412080913061117826146073639392852783203125f,-0.62909999856874343482360245616291649639606475830078125f,-0.191300003778934524234500713646411895751953125f,0.165099980139732593897861079312860965728759765625f,-0.38640001107603350050112567259930074214935302734375f,0.889599990704656118367665840196423232555389404296875f,-1.199400008296966024801122330245561897754669189453125f,-0.1981999763131145897432361380197107791900634765625f,0.752399990803002793882114929147064685821533203125f,0.54629994745254606414164300076663494110107421875f,-0.356099995275586334031459045945666730403900146484375f,0.7645999734565613881187573497300036251544952392578125f,0.2531999913632869692037274944595992565155029296875f,0.420699989724159451043306034989655017852783203125f,-0.3698999788448216197167539576184935867786407470703125f,-1.1011999620601538030228994102799333631992340087890625f,-0.41190001351237270910132792778313159942626953125f,-0.603100025081634072421365999616682529449462890625f,-0.04709999003410292317539642681367695331573486328125f,-1.744200024175643903845411841757595539093017578125f,-0.737700027036666927671149096568115055561065673828125f,-0.5271999949097629922789565171115100383758544921875f,0.9411000306844712337550618030945770442485809326171875f,-0.42490001464486137905396390124224126338958740234375f,1.1832000132679940662683293339796364307403564453125f,0.132599974179267832141704275272786617279052734375f,-0.0849000079393384954329349056934006512165069580078125f,0.66909999620318405533225813996978104114532470703125f,0.434100001120567124957005944452248513698577880859375f,-0.1655999967694281593821870046667754650115966796875f,-0.176999963682144656473838040255941450595855712890625f,-1.139299970918149274012876048800535500049591064453125f,0.4620000155568124711180644226260483264923095703125f,0.317400003580748713005732497549615800380706787109375f,-0.13759999565556657596943068710970692336559295654296875f,0.3430999707669022402711789254681207239627838134765625f,-0.3057000009000301332662274944595992565155029296875f,-0.23529999041259286318705790108651854097843170166015625f,-0.13489999333620072974326831172220408916473388671875f,0.54959998772442342218624844463192857801914215087890625f,-0.2551999986127018171799818446743302047252655029296875f,0.452000013321638138830138586854445748031139373779296875f,0.2466999937772751394504666677676141262054443359375f,1.02540001514404988114392835996113717555999755859375f,-0.131799997460841922247709590010344982147216796875f,0.4573000193431975279878543005906976759433746337890625f,-0.213300001975893938066519694984890520572662353515625f,-0.20699999089539045105112791134160943329334259033203125f,0.174400010122358839481648828950710594654083251953125f,-0.1540999933347106942704840548685751855373382568359375f,0.3215999934837221463368450713460333645343780517578125f,-0.207400034426152346522798097794293425977230072021484375f,-0.0397999962650239513806127433781512081623077392578125f,0.15390000206083043199356552577228285372257232666015625f,-0.617900003111362394747629878111183643341064453125f,0.3415000359565016907481549424119293689727783203125f,0.00590000469088547419005408301018178462982177734375f,1.0321999671936037401565045001916587352752685546875f,-0.10690001462996001624361497306381352245807647705078125f,-0.6877999770388008027310888792271725833415985107421875f,0.108700003001093836729751274106092751026153564453125f,1.1217999770835043182870549571816809475421905517578125f,-0.31779999197721497949942204286344349384307861328125f,1.5365999963521950189004883213783614337444305419921875f,-0.1997999958179885737763470388017594814300537109375f,0.2268999777123330563455283481744118034839630126953125f,0.4148999973595142165549987112171947956085205078125f,-0.23029998989105227025220301584340631961822509765625f,0.081599986815452663080350248492322862148284912109375f,1.0183999729514123799845037865452468395233154296875f,-0.3414000288829210916219381033442914485931396484375f,0.105600027987361233527963122469373047351837158203125f,-1.08150000488758113448284348123706877231597900390625f,0.763000066399573739772677072323858737945556640625f,-0.983400059527159697836395935155451297760009765625f,-0.63209997215270874448833637870848178863525390625f,0.173399993142485442376710125245153903961181640625f,-0.40419999005198459229859508923254907131195068359375f,-0.6816000308483840530726638462510891258716583251953125f,-1.0414999908804889461322318311431445181369781494140625f,0.14310003554970052164208027534186840057373046875f,0.167799942728877216069349742610938847064971923828125f,0.26099999656528238833885779968113638460636138916015625f,0.61289999148845664134199751060805283486843109130859375f,0.6656999857008454313245238154195249080657958984375f,0.871300052356719589852218632586300373077392578125f,-0.35040000236630408636528954957611858844757080078125f,1.5125999637484550675026184762828052043914794921875f,0.9900000322982667189108951788512058556079864501953125f,-0.0777999820306900513600112390122376382350921630859375f,-0.696900007924437403517003986053168773651123046875f,1.32339998680949211262714015902020037174224853515625f,0.063000002920628350722154209506697952747344970703125f,-0.6511999788239599684658287515048868954181671142578125f,0.09290000162124645299854819313623011112213134765625f,0.9802000155627725863638488590368069708347320556640625f,0.0833000044345855172878145822323858737945556640625f,0.249599993312358936492501015891321003437042236328125f,0.3639000050783156581246657879091799259185791015625f,0.0446999889492987190209305481403134763240814208984375f,0.2114000048920512409011962517979554831981658935546875f,0.03049999979138373618070545489899814128875732421875f,-0.13380000556707383196908267564140260219573974609375f,-0.23319999236911537199290478383773006498813629150390625f,0.48159999516010287834433256648480892181396484375f,-0.5113999811992051558462435423280112445354461669921875f,0.07809999915361398592494879267178475856781005859375f,0.566399975089729057486920282826758921146392822265625f,-0.02600000040233130249589521554298698902130126953125f,0.34800000806152808774385221113334409892559051513671875f,-0.10649999065697191014123745844699442386627197265625f,-0.344299983006715937250419301562942564487457275390625f,-0.0657999966859819096498540602624416351318359375f,-1.10260000543445357834571041166782379150390625f,-0.14670000271797167812337647774256765842437744140625f,-0.814999974668026094803963133017532527446746826171875f,-0.0619000005483627546709612943232059478759765625f,-0.6252999929830431735666707027121447026729583740234375f,-0.03389999045655145426536591912736184895038604736328125f,0.8759999746233224637848024940467439591884613037109375f,-0.23030000851750376256177332834340631961822509765625f,-0.3269999907836318442377887549810111522674560546875f,-0.4721999984860421051280354731716215610504150390625f,-0.01799997460842206464803894050419330596923828125f,-0.998400034046173612267693897592835128307342529296875f,0.6142000417590143257484669447876513004302978515625f,-1.02900001263618445790370969916693866252899169921875f,-0.848799968910217472739532240666449069976806640625f,-0.51689999265074693113319881376810371875762939453125f,-0.2119000005483631099423291743732988834381103515625f,-0.72870001708269127593808889287174679338932037353515625f,-0.7080999901384112948932170183979906141757965087890625f,0.443100014650821716344353262684307992458343505859375f,0.0104000108599660023855904000811278820037841796875f,-0.5678999891042710856936537311412394046783447265625f,-1.220199957299232806917643756605684757232666015625f,-0.443699992942809817719762577326036989688873291015625f,0.567599984014034220081157400272786617279052734375f,-0.1050000097602608040148197687813080847263336181640625f,0.49600001120567327461685636080801486968994140625f,-0.194700027990341251182826454169116914272308349609375f,-1.056499977469444129241082919179461896419525146484375f,0.88050003123283460126913269050419330596923828125f,-0.9826000159025198854578775353729724884033203125f,1.095599996471404491416024029604159295558929443359375f,-0.2226000173926347969199923682026565074920654296875f,0.1530000048577786486703189439140260219573974609375f,-1.0654999877214432757455142564140260219573974609375f,-0.546300004021077700855357761611230671405792236328125f,0.8939999952390784709876925262506119906902313232421875f,0.885900026297569365141271191532723605632781982421875f,0.33220000683069184077567115309648215770721435546875f,0.79740000749230421916990962927229702472686767578125f,0.413999978661537426649630333486129529774188995361328125f,0.1176000118792057680394691487890668213367462158203125f,0.7151999989852309003168784329318441450595855712890625f,0.03809999460876001275977387194870971143245697021484375f,-0.50629997393935932681330314153456129133701324462890625f,-0.110299999651312763404575889580883085727691650390625f,0.53080000972449770113570366447675041854381561279296875f,0.27450000542402275272024780861102044582366943359375f,-0.654299985018372609602010925300419330596923828125f,-0.4096999835848809112803792231716215610504150390625f,0.21709998578727240925445585162378847599029541015625f,0.147900011754035887179270503111183643341064453125f,0.5019999909326433584766391504672355949878692626953125f,0.0391000055909156429834183654747903347015380859375f,0.75079999165683963813222590033547021448612213134765625f,-0.34689999428987494667353530530817806720733642578125f,0.8213000120863316322328273599850945174694061279296875f,-0.2918999983131886910570074178394861519336700439453125f,0.572599975818395723337062008795328438282012939453125f,0.007299993079900701786755234934389591217041015625f,-0.02480000868886700349236207330250181257724761962890625f,-0.142999993979930906107256305404007434844970703125f,0.3616000011950728509901864526909776031970977783203125f,-0.1118999959476292271887132301344536244869232177734375f,-0.171499988943338566027563274474232457578182220458984375f,0.40220000385046006385891814716160297393798828125f,0.4607999972447751613202626685961149632930755615234375f,-0.10189999898374091369390725958510302007198333740234375f,-0.637500018477439756026114991982467472553253173828125f,0.13110000147819533822257653810083866119384765625f,0.39880001912712970124630373902618885040283203125f,0.57100001329183580622839144780300557613372802734375f,-0.45580004973411547553041600622236728668212890625f,-0.36259999042153356896278637577779591083526611328125f,-0.0182000347256661143546807579696178436279296875f,-1.22640001334846004965584143064916133880615234375f,0.61640002199113386183171314769424498081207275390625f,0.26999999299645427441873835050500929355621337890625f,-0.134299991053342893110311706550419330596923828125f,0.450199973392486629819586596568115055561065673828125f,-1.120099978947639129955859971232712268829345703125f,-0.0825999922096729566334261107840575277805328369140625f,-0.4277999962985514770252848393283784389495849609375f,-0.38139999673366542509711507591418921947479248046875f,-0.087999991297721180671942420303821563720703125f,1.3548999985516072541713583632372319698333740234375f,-0.3402000633954997965702204965054988861083984375f,0.9455000185668467338473419658839702606201171875f,-0.507499992847442626953125f,0.8082000049605966296439873985946178436279296875f,-0.467000016935169792731130655738525092601776123046875f,-0.542499991655349678154607317992486059665679931640625f,1.0762999610498553693105350248515605926513671875f,-0.2411999767005443828793431748636066913604736328125f,1.4560999669075016527131083421409130096435546875f,0.621499992072582330138175166212022304534912109375f,-0.1592000271558760005063959397375583648681640625f,0.014799999561905863298960639440338127315044403076171875f,0.014300000347942098954945322475396096706390380859375f,0.31399999386072163787275712820701301097869873046875f,0.260300004382431560134847359222476370632648468017578125f,-0.1324999953992664902813203298137523233890533447265625f,-0.4039000128827989755109229008667171001434326171875f,-0.55149998767673924504606475238688290119171142578125f,-0.4169000214099884971119536203332245349884033203125f,-0.2727999989807605718539207373396493494510650634765625f,0.229600004935264667693672890891321003437042236328125f,-0.0553999739050865525058497951249592006206512451171875f,-0.326199998307227900795624009333550930023193359375f,-0.08799999770522115571935728439711965620517730712890625f,0.19600000472366818637937058156239800155162811279296875f,-0.5855000064224002986890127431252039968967437744140625f,0.05629999416768549647116515188827179372310638427734375f,-0.08760000953227287112667909241281449794769287109375f,-0.21120001171827329500274572637863457202911376953125f,-0.184599992939829871119172821636311709880828857421875f,-0.0313999980747698970162673504091799259185791015625f,-0.258400006106495272462098000687547028064727783203125f,0.841599998438358198882269789464771747589111328125f,0.0921999985232948571223232647753320634365081787109375f,-0.01659998711347565603091425145976245403289794921875f,-0.3338999994903801837864421031554229557514190673828125f,0.37350000047683717951940707280300557613372802734375f,-0.2884000349774960003657042761915363371372222900390625f,0.7240000191926956318866359652020037174224853515625f,0.461099981026351191104595272918231785297393798828125f,-0.0866999899029732290500760427676141262054443359375f,0.4134999953359368429772757735918276011943817138671875f,0.5260999958157537292890992830507457256317138671875f,-0.41720000221133235474013645216473378241062164306640625f,0.2204000189997257219687298857024870812892913818359375f,0.3664000177443027528312313734204508364200592041015625f,-0.31050004130974429017442162148654460906982421875f,-0.07979996938705458120466573745943605899810791015625f,-0.43810002150535609644066425971686840057373046875f,-0.8800999986171722522243499042815528810024261474609375f,0.7147000054895880083449810626916587352752685546875f,-0.31919999079704286515379862976260483264923095703125f,1.25339997384548240688673104159533977508544921875f,-0.197899987725913550917766769998706877231597900390625f,-0.22069999840408538460678755654953420162200927734375f,-0.412599983231723366206011860413127578794956207275390625f,-0.804900003953278098123291783849708735942840576171875f,0.596300015699863461549057319643907248973846435546875f,-1.109300033223629355916273198090493679046630859375f,-0.63230000142455100586857952293939888477325439453125f,0.96429999284148237848057760857045650482177734375f,-1.20049998369812982446092064492404460906982421875f,0.155900060272216478551854379475116729736328125f,0.510499982059001933265562911401502788066864013671875f,-0.844099974000453556755019235424697399139404296875f,-1.180999987065792122820084841805510222911834716796875f,0.907100002849102082791432621888816356658935546875f,-1.229400042122602343397375079803168773651123046875f,0.3894000458478927839678362943232059478759765625f,1.05249995261430751014586348901502788066864013671875f,-0.88799996507167833215135033242404460906982421875f,-0.2688999986708164868787207524292171001434326171875f,0.600600038790702228652662597596645355224609375f,-0.299699996846914284986951315659098327159881591796875f,0.3375000077486038918550548260100185871124267578125f,0.63720000195056225589240739282104186713695526123046875f,0.32400000309944143594975685118697583675384521484375f,-0.3349999987706537840637111003161408007144927978515625f,0.72740000197887422217490893672220408916473388671875f,0.106100003564357781016269655083306133747100830078125f,0.1566999970555305043973248757538385689258575439453125f,0.091200000132620251402926214723265729844570159912109375f,0.106800005656480845317446437547914683818817138671875f,-0.1888000327616932150220918629202060401439666748046875f,0.6280000139474868348088421043939888477325439453125f,-0.2765000062659381807605996073107235133647918701171875f,0.0172999929308890187940050964243710041046142578125f,0.1299999978765844499406512113637290894985198974609375f,-0.130000001750886440277099609375f,0.540900014264881878744972709682770073413848876953125f,0.3374999967217444662992420489899814128875732421875f,-0.2032000163972378370402793734683655202388763427734375f,0.04479999740123741247543875942938029766082763671875f,-0.64979998603910171794950656476430594921112060546875f,-0.268200008797645494951211730949580669403076171875f,0.3122000147283079396487437406904064118862152099609375f,-0.60740000466108323706748706172220408916473388671875f,0.2718000096797943410109610340441577136516571044921875f,-0.22140000030994411162055257591418921947479248046875f,-0.5626000026404855969275331517565064132213592529296875f,-0.26540001503229149193430203013122081756591796875f,0.1979000028133388600082298580673523247241973876953125f,-0.806299994838237665817359811626374721527099609375f,-0.06010001232624084810396425382350571453571319580078125f,0.50990001270771045938090537674725055694580078125f,0.49199999186396592421033346909098327159881591796875f,-0.04279995144605663881520740687847137451171875f,-1.303999975830316682134935035719536244869232177734375f,0.420200035083293954585315077565610408782958984375f,-0.33719998842775822378570182991097681224346160888671875f,0.70209999085366714410838540061376988887786865234375f,-0.031600002014636974223549259477294981479644775390625f,0.0908000026464463871889165602624416351318359375f,-0.8242000355005263401153570157475769519805908203125f,-0.031399955010414259959361515939235687255859375f,-0.5665000054836273335467922152020037174224853515625f,0.463400005435943285192479379475116729736328125f,0.815900013035535831562583553022705018520355224609375f,-1.17530002002865074928195099346339702606201171875f,-0.00309997739791878501591781969182193279266357421875f,0.76260001053810100302143837325274944305419921875f,0.164699999827146559283619353664107620716094970703125f,-1.015700003433227749383149784989655017852783203125f,-0.4109999785721303311447627493180334568023681640625f,-0.2904000063896177152855671010911464691162109375f,1.172299980562925281191155590931884944438934326171875f,-0.447099979603290620389088871888816356658935546875f,0.797500006407499295590923793497495353221893310546875f,-0.551900006508827090101476642303168773651123046875f,-0.03149997642636304107099931570701301097869873046875f,-0.89930002875328085565342917107045650482177734375f,1.17469995899796497695888319867663085460662841796875f,-0.33940001187324497777808574028313159942626953125f,-0.12410000186562530899436751496978104114532470703125f,0.3000000137090683693941173260100185871124267578125f,0.4704000056445598598742208196199499070644378662109375f,-0.1679999889135359580905060283839702606201171875f,0.60389998287558521727902416387223638594150543212890625f,0.27810001338422296601748939792742021381855010986328125f,-0.3336999947309491343361287363222800195217132568359375f,0.51730000574588785866581019945442676544189453125f,0.412299999263882444022755180412787012755870819091796875f,0.008900007760524797362222670926712453365325927734375f,0.071699988822639060259689358645118772983551025390625f,0.51150000309944143594975685118697583675384521484375f,-0.4822000157713894896716055882279761135578155517578125f,-0.0433999924719332685896233670064248144626617431640625f,-0.381799997535348101251884145312942564487457275390625f,-0.1077999997258185516102457768283784389495849609375f,0.296899996413290401253703976180986501276493072509765625f,0.336700008082389867780648273765109479427337646484375f,0.2336999949730931280811319084023125469684600830078125f,0.6483000126115978645202631014399230480194091796875f,0.1171999877944589485423421137966215610504150390625f,0.0447999891310928433085791766643524169921875f,0.314700012977421561544133510324172675609588623046875f,-0.6387000146985055692994137643836438655853271484375f,-0.52849998993426527960082239587791264057159423828125f,-0.22030001432895662816235926584340631961822509765625f,-0.34410003122091337157684165504178963601589202880859375f,0.4845000154078007437163932991097681224346160888671875f,-0.1092999989137053429288926054141484200954437255859375f,-0.21830000782459979546956674312241375446319580078125f,-0.29220001364797365539516249555163085460662841796875f,-1.1854000022172925810082233510911464691162109375f,1.21799998116493224387113514239899814128875732421875f,-1.05559996964931457341663190163671970367431640625f,0.69399997504800570169436468859203159809112548828125f,-0.48329998234361415399007455562241375446319580078125f,-1.14139998332262049274277160293422639369964599609375f,0.51250002503395108988115680404007434844970703125f,0.621799991351366099223696437547914683818817138671875f,-0.271099998646974427174427546560764312744140625f,-0.660300007772445685105822121840901672840118408203125f,0.565800030660629005296868854202330112457275390625f,-0.63450001622736440509697786183096468448638916015625f,0.1820999750584366694283744436688721179962158203125f,-0.70319999717473979927717664395458996295928955078125f,0.74619997357130074533415609039366245269775390625f,0.12530000434517862828442957834340631961822509765625f,-0.186599993520975004912543226964771747589111328125f,-0.3499999840557574959376552214962430298328399658203125f,0.02810000319555427950035664252936840057373046875f,0.76870002036094664532583919935859739780426025390625f,-1.479300013628602528115152381360530853271484375f,0.89309998186826700106166754267178475856781005859375f,0.24540001235008190860753529705107212066650390625f,-0.284399969404935948347201701835729181766510009765625f,-0.342100020134449067654713871888816356658935546875f,-0.54240001650750624406782662845216691493988037109375f,-0.7732999928861854499473338364623486995697021484375f,0.13340000782012939595233547152020037174224853515625f,-0.127500020749867104541408480145037174224853515625f,-0.1002000069200992271589711890555918216705322265625f,0.269399996525049090223546954803168773651123046875f,0.474399996227025499972995703501510433852672576904296875f,0.16640002050101776465140801519737578928470611572265625f,-0.36490001147836448325278979609720408916473388671875f,-0.0455000178217890294263270334340631961822509765625f,-0.0009999713078141514444041604292578995227813720703125f,-0.2761999920487401283253348083235323429107666015625f,-0.1549000034689903271267752415951690636575222015380859375f,0.0217000004827976322729909952613525092601776123046875f,0.54120000393241607383743030368350446224212646484375f,-0.479999985694885111797702847979962825775146484375f,-0.3260000045001507640307636393117718398571014404296875f,0.1532000046253205738366887089796364307403564453125f,-0.5230000127553939837099505894002504646778106689453125f,0.2709999894499777184364575077779591083526611328125f,-0.023300014939904467592413084275904111564159393310546875f,0.527300017182529057180317977326922118663787841796875f,0.10460000981539463449365712222061119973659515380859375f,-0.30979999799728386022934500942938029766082763671875f,0.22680001679509853573790678638033568859100341796875f,-0.4536000014781951250597558100707828998565673828125f,-0.3362000247940424291215322227799333631992340087890625f,0.41900000108778467922121535593760199844837188720703125f,0.4413000317186119847434611074277199804782867431640625f,-0.25770000021457661887325230054557323455810546875f,-0.129699999789893549717589849024079740047454833984375f,-0.035899998879432626353036539512686431407928466796875f,0.35770001720190069960381151759065687656402587890625f,-0.47650001360476024192536215196014381945133209228515625f,-0.04520001340210466622693274985067546367645263671875f,-0.7234999764859679771689116023480892181396484375f,0.5145999868676067734440948697738349437713623046875f,-0.20060000671595279442271930747665464878082275390625f,-0.80179999254345890591366696753539144992828369140625f,0.87769996832609109560507931746542453765869140625f,-0.3386999992012977855893041123636066913604736328125f,1.33829998953342510503716766834259033203125f,-0.13000000081956386566162109375f,-0.1812999984890222293643091688863933086395263671875f,1.2201999556601048624315808410756289958953857421875f,-0.2001999882936473795780329965054988861083984375f,0.1232000296592712373922040569595992565155029296875f,-1.490800024104117937895352952182292938232421875f,-1.401699989455938322180372779257595539093017578125f,1.0988999806404109449431416578590869903564453125f,-0.21449999992921942038037741440348327159881591796875f,-0.715399990575760202915489571751095354557037353515625f,-0.0593000105738639149421942420303821563720703125f,1.07929997630119345330967917107045650482177734375f,0.12479999546408659938379059894941747188568115234375f,1.31239996263980884805278037674725055694580078125f,-1.0503000021472572722558425084571354091167449951171875f,0.513899981869756938834825632511638104915618896484375f,0.46860000863075246257949402206577360630035400390625f,0.71750000834465055277178180404007434844970703125f,0.2564000062704085980413992729154415428638458251953125f,-0.8456999964296816330033834674395620822906494140625f,1.275200023944675908893486848683096468448638916015625f,-0.830700023885071470175489594112150371074676513671875f,0.02449998679757126041067749611102044582366943359375f,-0.20840001616477987766984369955025613307952880859375f,0.1047999971777200389766449006856419146060943603515625f,0.08959999569654453210887368186376988887786865234375f,0.1431999949395657001360859794658608734607696533203125f,0.317600002640485712390727712772786617279052734375f,-0.007399995459616004467306993319652974605560302734375f,-0.5578000173091890445675744558684527873992919921875f,-0.06049999383091930749145603840588591992855072021484375f,-0.26710000143349166279449491412378847599029541015625f,0.36980000418126568462184877716936171054840087890625f,0.30700000244379044289644298260100185871124267578125f,0.3855000219568613939458145978278480470180511474609375f,-0.011100000584125524483170011080801486968994140625f,-0.49260002547651549065221843193285167217254638671875f,0.287400008535385165231446080724708735942840576171875f,0.325800005924701707726853783242404460906982421875f,0.14939999302327622299202403155504725873470306396484375f,0.58830001285374156072549567397800274193286895751953125f,0.092100005233287873807057621888816356658935546875f,-0.372900003334879937000323479878716170787811279296875f,-0.429900010919570974010639474727213382720947265625f,0.35150001401454178395766803078004159033298492431640625f,-0.70429999805688847658302620402537286281585693359375f,-0.108200005891919193157235667968052439391613006591796875f,0.2234000048398971927099410095252096652984619140625f,-0.4251000220492484604761784794391132891178131103515625f,0.633900017893314515760039284941740334033966064453125f,0.2430000049322842692323121127628837712109088897705078125f,0.0882000077545643801357755364733748137950897216796875f,-0.2919000106811522332606045893044210970401763916015625f,0.0411999986052506983469356782734394073486328125f,-0.432100006574392292435504714376293122768402099609375f,1.052299998891353283170246868394315242767333984375f,-0.29520000438690185973200641456060111522674560546875f,0.36640001953244194510261877439916133880615234375f,0.27719998432993886439135167165659368038177490234375f,0.55779999763965548709165886975824832916259765625f,0.16199999916553498024995860760100185871124267578125f,0.04669998699724686730405665002763271331787109375f,-0.141699979919195140354304385255090892314910888671875f,0.667699977862834970210315077565610408782958984375f,-0.09110000892877589961926787509582936763763427734375f,1.27840004000663753913613618351519107818603515625f,-0.604500014513730921095202575088478624820709228515625f,-0.61989999378323545897728763520717620849609375f,-0.141599980966746807897749249605112709105014801025390625f,0.884199979881941988679727728595025837421417236328125f,0.453900010889768612543093695421703159809112548828125f,0.403799990963936039634063490666449069976806640625f,0.2344000074028969038408831693232059478759765625f,-0.618900035476684706736705265939235687255859375f,1.01669998304843911540729095577262341976165771484375f,0.2121999686837199305955437012016773223876953125f,-0.3122999990403652059711703259381465613842010498046875f,-0.0808999956011771104158469825051724910736083984375f,-0.92980000276565544226059500942938029766082763671875f,-0.455299990934133091968760709278285503387451171875f,0.217300000903010381847479948191903531551361083984375f,-0.0064999978095290344981549424119293689727783203125f,-0.2497999982953071906877084984444081783294677734375f,-0.026700005586445307859122522131656296551227569580078125f,0.6659000292778018970096809425740502774715423583984375f,-0.208700002181529953304561786353588104248046875f,0.3108000099480145994590429836534895002841949462890625f,-0.31079999430179583441713475622236728668212890625f,-0.5100000211969020202928959406563080847263336181640625f,0.0672000054895878218275129256653599441051483154296875f,-0.01879998288005613193973886154708452522754669189453125f,-0.34479999457001664353583691990934312343597412109375f,-0.294999996423721044802590540712117217481136322021484375f,0.2301999976813793313823452990618534386157989501953125f,0.617599986435472647627875630860216915607452392578125f,0.31480001736879348328557171043939888477325439453125f,0.02299999766796824440806545908344560302793979644775390625f,0.26860000721514232235875851984019391238689422607421875f,-0.2040000001192090461898942521656863391399383544921875f,-0.4548000159159304445211091660894453525543212890625f,-0.2614999940469864281311629383708350360393524169921875f,0.0959000070750715138956365990452468395233154296875f,0.3700999884098770120743893130565993487834930419921875f,0.4758999953031537444303467054851353168487548828125f,-0.4370000357478858177273650653660297393798828125f,0.6818000037193296947179987910203635692596435546875f,0.20810001129806021680224148440174758434295654296875f,-0.61490000317096704662844786071218550205230712890625f,0.1350000122189520812554519579862244427204132080078125f,-0.3581999991118907100684509714483283460140228271484375f,0.0666999922871587269668225417262874543666839599609375f,0.073700007843971349075218313373625278472900390625f,-0.15739999523609871090457090758718550205230712890625f,-0.7332999920107423630355469867936335504055023193359375f,1.14630001614689813749237146112136542797088623046875f,-0.27579998297691332709291600622236728668212890625f,-0.03360000445842725724787669605575501918792724609375f,1.58759993841648139323297073133289813995361328125f,-0.31230001341998570030256132668000645935535430908203125f,-0.84430000051558007356788948527537286281585693359375f,0.1984000100553036105566206970252096652984619140625f,0.24599996218085351529225590638816356658935546875f,-0.59569998279511937600005921922274865210056304931640625f,0.02250001110136512494364069425500929355621337890625f,0.130499952256679652151660775416530668735504150390625f,0.724500042974948854634931194595992565155029296875f,-0.100099994407594206080602816655300557613372802734375f,-0.28949999234825352090183514519594609737396240234375f,0.272699990975856731978410607553087174892425537109375f,-0.189299996827543282051919959485530853271484375f,-0.9149999861419200186674061114899814128875732421875f,1.1086999591171746715190238319337368011474609375f,-0.3069000090420246351641253568232059478759765625f,0.6736000363469116791748092509806156158447265625f,-1.13880001391172402946949659963138401508331298828125f,-0.12699997532367746089221327565610408782958984375f,0.974999999105930381659845807007513940334320068359375f,-0.278100036704540087839632178656756877899169921875f,0.330499995842576055338213336654007434844970703125f,0.4018999815493824456780203036032617092132568359375f,0.58190000844597822293735589482821524143218994140625f,-1.26500001579523058126142132095992565155029296875f,0.582200006830692284864881003159098327159881591796875f,0.088599995219707405880171791068278253078460693359375f,-0.483400023093819963815320761568727903068065643310546875f,0.019000002913176994301380773322307504713535308837890625f,-0.50160001825690303522975455052801407873630523681640625f,-0.17519999961853027770075641456060111522674560546875f,-0.28689999324679360537260208730003796517848968505859375f,0.096099999094009458389109568088315427303314208984375f,-0.36289999122768623607271365472115576267242431640625f,-0.2850999983191488951206338242627680301666259765625f,0.6479000107854602230617047098348848521709442138671875f,0.435700020122528286492524784989655017852783203125f,-0.151899994476139443833062614430673420429229736328125f,-0.18290000896006830544138210825622081756591796875f,0.0899000038787722832722693055984564125537872314453125f,-0.316700000967085337766349084631656296551227569580078125f,0.3415000073835254834619945540907792747020721435546875f,-0.4423000230312350566919121774844825267791748046875f,-0.69329999277442644345370581504539586603641510009765625f,-0.36510002692937870705236491630785167217254638671875f,-0.72010000793039807831519283354282379150390625f,-0.2699000052571296492942565237171947956085205078125f,-0.529400015971064785702537847100757062435150146484375f,0.01100000435113912544693448580801486968994140625f,0.253899996808171213302784963161684572696685791015625f,0.0809000066280365359716597595252096652984619140625f,-0.5384000134080653143797690063365735113620758056640625f,-0.2463999979257582850777907879091799259185791015625f,0.5151000094950202168320174678228795528411865234375f,0.25889999166727051260750158689916133880615234375f,0.6625000066310164914540337122161872684955596923828125f,0.5877999891459939618698626873083412647247314453125f,-0.62600002105906593641293511609546840190887451171875f,0.026599987728148644094972041784785687923431396484375f,-0.4030000181198118980319122783839702606201171875f,-0.10220002173185349647610564716160297393798828125f,-0.5888999857068062393494756179279647767543792724609375f,0.3161999981582166441285153268836438655853271484375f,-0.02050000739097601609728371840901672840118408203125f,-0.33619998519420590810113935731351375579833984375f,0.06720000385046009938605493516661226749420166015625f,0.68910000529289305859492742456495761871337890625f,-0.456800002527236959082301837042905390262603759765625f,0.988499993510544072705670259892940521240234375f,0.7662999953784048745575319117051549255847930908203125f,-0.7757999999083577957748047992936335504055023193359375f,0.138599994474649435005630948580801486968994140625f,1.5708999780178061200786032713949680328369140625f,-0.588899974679947035838267765939235687255859375f,0.19219998120069448077629203908145427703857421875f,-0.50790000087618825119761822861619293689727783203125f,-0.26400002396106714286361238919198513031005859375f,-0.4411000096738337816759667475707828998565673828125f,-0.4917000080823896723813959397375583648681640625f,0.54739999482631684912803393672220408916473388671875f,-0.69269998769760121604122105054557323455810546875f,-0.4118000078916548289953425410203635692596435546875f,-0.031000045180320512372418306767940521240234375f,0.233500023275613699524910771287977695465087890625f,0.335800008010863848539884202182292938232421875f,0.997099988245964130584297890891321003437042236328125f,-0.1598999984025955001243346487171947956085205078125f,-0.494300019551813907270343406707979738712310791015625f,-0.1752999933183192471375377863296307623386383056640625f,-0.2003000035628677011345644132234156131744384765625f,0.0274999998509883880615234375f,-0.4134000182509425247445733475615270435810089111328125f,-0.18420000033378602921629862976260483264923095703125f,0.1840999911740424099804158686310984194278717041015625f,-0.0977000164568426754385654930956661701202392578125f,-0.7443000204086303028816473670303821563720703125f,-0.0009000119924547078653631615452468395233154296875f,0.40460001916587362558175300364382565021514892578125f,-0.34699999976158135694959128159098327159881591796875f,-0.2572999942719935706492151439306326210498809814453125f,0.2105000059008599322396548814140260219573974609375f,-0.5734000182136895773510332219302654266357421875f,-0.44540000728368767113352078013122081756591796875f,-0.629899987822770679457562437164597213268280029296875f,-0.36010002156496057068579830229282379150390625f,0.4488000095158815039297905968851409852504730224609375f,0.091200006055832005813499563373625278472900390625f,-0.34870003854036379919989485642872750759124755859375f,0.4531000074982642900067730806767940521240234375f,-0.049300015453994650016511513967998325824737548828125f,0.7621000162601472283085968228988349437713623046875f,-0.3564000230714678973953368767979554831981658935546875f,0.13680000461339947293026853003539144992828369140625f,0.339300008264184160822196645312942564487457275390625f,0.0459000027537346255712691345252096652984619140625f,-0.3082000238478184162005391044658608734607696533203125f,-0.23149999535083765067611238919198513031005859375f,0.00469997620880613364846567492350004613399505615234375f,-0.98560000369846800793993679690174758434295654296875f,-0.4324000030219554702171080862171947956085205078125f,0.3524999953806400299072265625f,-0.102800015223026175448239882825873792171478271484375f,-0.507400018966198018688373849727213382720947265625f,0.8677000163078307792829946265555918216705322265625f,-1.2114000099956996336914016865193843841552734375f,-0.96300000828504561667386951739899814128875732421875f,0.10349999868869730335063650272786617279052734375f,0.818999998033046683332258908194489777088165283203125f,0.031600003206730065130614093504846096038818359375f,-1.0098000036597252204728647484444081783294677734375f,1.08530001522302654848317615687847137451171875f,-0.091300021064281366989234811626374721527099609375f,-0.411500024557113874834612943232059478759765625f,-0.4498999675571919709682333632372319698333740234375f,0.0465999707221982362170820124447345733642578125f,0.74370001812577246624869076185859739780426025390625f,-0.61750000745058031270673382095992565155029296875f,-0.145299995702504958217105013318359851837158203125f,-1.598800024938583419498172588646411895751953125f,-0.76399998208880415262456153868697583675384521484375f,-0.07050001513957937504528672434389591217041015625f,-0.047300022435188271430206441436894237995147705078125f,-1.160899969375133622406792710535228252410888671875f,0.67060000779628747835658941767178475856781005859375f,-0.65769998382329930564083042554557323455810546875f,-1.259600018346309635575153151876293122768402099609375f,-0.24720000172704459373562713153660297393798828125f,0.15270000066161149021581877605058252811431884765625f,-0.213899995839595735702687306911684572696685791015625f,0.281799990196526073393812339418218471109867095947265625f,0.456200004118680890741188704851083457469940185546875f,-0.55679999030828486450417358355480246245861053466796875f,0.06720000921487834233403191319666802883148193359375f,-0.2705999972164629951976166921667754650115966796875f,-0.2117999992489814076179754920303821563720703125f,0.365600023517012762130917735703405924141407012939453125f,-0.419600006313621987086293074753484688699245452880859375f,-0.004399989894032341197771529550664126873016357421875f,-0.866800011467933728681600769050419330596923828125f,0.059799977731704867522921631461940705776214599609375f,0.8258000005602834647788768052123486995697021484375f,-0.1202000003263352656546203434118069708347320556640625f,-0.034600007915496888699635746888816356658935546875f,-0.3686999920487401549706873993272893130779266357421875f,-0.1461000075876712767097842515795491635799407958984375f,-0.2775999953389167718142971352790482342243194580078125f,-0.449600012609362664761647465638816356658935546875f,-0.5501999935090537974247126840054988861083984375f,0.082199992823600798175220916164107620716094970703125f,0.088100013309717173370927412179298698902130126953125f,-0.2860999973058699907824120600707828998565673828125f,-0.0412000010266899696631526239798404276371002197265625f,0.2968000017076730667753281522891484200954437255859375f,-0.40520000643581166599460630095563828945159912109375f,-0.2607999990701674875026583322323858737945556640625f,-0.0619999921992422375893738717422820627689361572265625f,-0.362699993993341962550402968190610408782958984375f,0.7711999907076358962143558528623543679714202880859375f,-0.8348999955713747311847328091971576213836669921875f,0.0625000037252902984619140625f,-0.344099980854988274359129718504846096038818359375f,-1.006099986726045614204849698580801486968994140625f,0.56549998146295621381796081550419330596923828125f,0.06589999306797977851601899601519107818603515625f,-0.4322999902486799328471533954143524169921875f,0.631600005870312397604493526159785687923431396484375f,0.495599991125613481557365957996807992458343505859375f,0.8955999729275703202802105806767940521240234375f,-0.143999996542930830401019193232059478759765625f,-0.79599999988079073176550082280300557613372802734375f,0.16220003961324636065910453908145427703857421875f,0.21600001850724215302079755929298698902130126953125f,-0.73300000858306901818650658242404460906982421875f,-0.3981999871164561000114190392196178436279296875f,0.1878999921590089883238761103712022304534912109375f,-0.0006999937474727602193524944595992565155029296875f,0.20449996720254404181105201132595539093017578125f,-0.6112999701023102971930711646564304828643798828125f,-0.323500004500150595276863896287977695465087890625f,-0.17410002302527427531231296597979962825775146484375f,1.23800001126527803307908470742404460906982421875f,-0.61119999510347844395852234811172820627689361572265625f,0.60760000636577604637977856327779591083526611328125f,-0.09710001268386836414947538287378847599029541015625f,-0.33390000954866394522468908689916133880615234375f,0.47139999092221263055080271442420780658721923828125f,0.14829999001771199118593358434736728668212890625f,-0.438400002492964180333956392132677137851715087890625f,-0.436400015807151930857799015939235687255859375f,0.6483000042110678151630054344423115253448486328125f,0.26250002235174196840716831502504646778106689453125f,-0.214399995407462018448541130055673420429229736328125f,-0.31480000973194843805202935982379131019115447998046875f,0.305299996782839144771060091443359851837158203125f,0.616300023001432606406524428166449069976806640625f,0.0178000000983475903382213800796307623386383056640625f,-0.12940000326782463702102177194319665431976318359375f,-0.281199994991719659509499251726083457469940185546875f,-0.37040000534653660935191510361619293689727783203125f,-0.34509999317824802123055860647582449018955230712890625f,0.2406999937474727513375682974583469331264495849609375f,0.041599999965727274542359737097285687923431396484375f,0.11320000283718112399355959496460855007171630859375f,0.3635000072941185766950411561992950737476348876953125f,0.457299999785423239018200547434389591217041015625f,0.11100000502169116156636619052733294665813446044921875f,-0.125399995958805021700754878111183643341064453125f,-0.1272000073522328300601458295204793103039264678955078125f,0.2495999997198581343837986423750407993793487548828125f,0.036600017139315266234689261182211339473724365234375f,-0.324599992418288962880978942848742008209228515625f,0.2149999919533731596033021560288034379482269287109375f,0.5337999977439640275633792043663561344146728515625f,-0.0477999998748301901940749303321354091167449951171875f,-0.21860000475645069428765054908581078052520751953125f,-0.5538999884635205095406718101003207266330718994140625f,1.1189999790713187621804536320269107818603515625f,-0.1679000019192695702940909541212022304534912109375f,0.1076999796509738871463923715054988861083984375f,0.104299971681833358871926975552923977375030517578125f,0.978200023996830481110009714029729366302490234375f,-0.130900000406801719776694881147705018520355224609375f,0.37749998755753022550152309122495353221893310546875f,-0.304500011384487123677899944595992565155029296875f,0.1912000369012361034037894569337368011474609375f,0.58560001986622811642746455618180334568023681640625f,-0.972600028978288122516460134647786617279052734375f,0.181400001651048636830410032416693866252899169921875f,-0.378200011926889345659219543449580669403076171875f,-0.67739997329413903504047311798785813152790069580078125f,0.04430000900924202511532712378539144992828369140625f,-0.071999995142221440147523026098497211933135986328125f,0.031199996221065617874046438373625278472900390625f,-0.23689997611045843228794183232821524143218994140625f,0.54020000647306432028926792554557323455810546875f,-0.032900010114908173619596709613688290119171142578125f,0.644099992775917229437254718504846096038818359375f,-0.189699992749094992205982634914107620716094970703125f,-0.4604999891370535891610416001640260219573974609375f,-0.083199998962879107011758605949580669403076171875f,0.97819996051788393742754124104976654052734375f,-0.52859999969005588837944742408581078052520751953125f,0.6176000317722554200372542254626750946044921875f,0.304499996930360783409241776098497211933135986328125f,-0.338100001537799954576257732696831226348876953125f,-1.18019996862411513660617856658063828945159912109375f,-0.7679000144362451152346693561412394046783447265625f,0.168099992187320967929053949774242937564849853515625f,0.782300028544664627361271413974463939666748046875f,-0.302500005811452921111737168757827021181583404541015625f,0.241300005865097144663877770653925836086273193359375f,-0.418900017595291274119517765939235687255859375f,0.4153000080704689622734804288484156131744384765625f,-0.315800004844367487866207966362708248198032379150390625f,-0.055000004768371635321955182007513940334320068359375f,-0.67870000143647180035344490534043870866298675537109375f,0.05589999612271789874995420177583582699298858642578125f,0.560699987414479128489119830192066729068756103515625f,-0.067499998807907246600734652020037174224853515625f,-0.5164000038862226471536587268928997218608856201171875f,0.0843999947369098446614543718169443309307098388671875f,-0.0912000327289106405004304178874008357524871826171875f,0.42240000704526892150170169770717620849609375f,-0.0203000013649463628695457373396493494510650634765625f,0.55670001285076153862974024377763271331787109375f,-0.4955000018030404351776496696402318775653839111328125f,0.2470999971866607580750496708787977695465087890625f,-0.36440000893026525119466896285302937030792236328125f,0.62719998328685733213205821812152862548828125f,-0.03870001022815738878080082940869033336639404296875f,0.0711000047564507298147873370908200740814208984375f,0.168500003531575348603155362070538103580474853515625f,-0.5753000155210497013058557058684527873992919921875f,0.005100004018843140585204309900291264057159423828125f,-0.3291000009715556728906449279747903347015380859375f,-0.66970001051872973807377320554223842918872833251953125f,0.10859996392726856129229418002068996429443359375f,-0.6205000136494636109318889793939888477325439453125f,0.162000042229891505485284142196178436279296875f,-0.348299987745285033469144764239899814128875732421875f,-0.57000000432133646199872600845992565155029296875f,-0.69249999806284900927266789949499070644378662109375f,-0.49699999558925611609083716757595539093017578125f,-0.49980000485181808045354046043939888477325439453125f,0.362799996894597853724917513318359851837158203125f,-0.8855999851837754732741814223118126392364501953125f,-0.01030000892728555328403672319836914539337158203125f,0.9981999718427658763175713829696178436279296875f,-0.536699979472160748628084547817707061767578125f,-0.6727999895185232670513642005971632897853851318359375f,-0.099599999943375649991139653138816356658935546875f,0.01629998157620438092862968915142118930816650390625f,-1.18809997680187251489769550971686840057373046875f,-0.67169999362826349198485331726260483264923095703125f,0.24320002101659810023193131200969219207763671875f,-1.3443999919056892622393206693232059478759765625f,1.0980999958753585588056012056767940521240234375f,-1.031599969679117378973387531004846096038818359375f,-0.049599966788292704222840256989002227783203125f,0.39420001791715630901080658077262341976165771484375f,-1.13130003894567465749787515960633754730224609375f,0.67439998177289961489577763131819665431976318359375f,0.19530000061988772586119011975824832916259765625f,0.53699999544024468178804454510100185871124267578125f,-0.47270000751614560385860386304557323455810546875f,-1.10830001128911970198487324523739516735076904296875f,-0.08430000073909749147560432902537286281585693359375f,-0.17379999796748146767555454061948694288730621337890625f,0.38980000522434721244025013220380060374736785888671875f,0.44179999284148241400771439657546579837799072265625f,0.0981999977707861404496725299395620822906494140625f,-0.4318999949231743240574132869369350373744964599609375f,0.0934999909400939799297702847979962825775146484375f,-0.0412000126868482807740434736842871643602848052978515625f,0.361599990950524585731074012073804624378681182861328125f,0.54530000997036720544741683625034056603908538818359375f,0.29710000076293940907135038287378847599029541015625f,-0.2171999894708396272591244269278831779956817626953125f,-0.5207999943390486663474803208373486995697021484375f,0.7391000187784440011995457098237238824367523193359375f,-0.07310001941919352930199238471686840057373046875f,0.0028000066176059323908020814997144043445587158203125f,-0.7840000287294390801662302692420780658721923828125f,0.294000002168119056733530669589526951313018798828125f,-0.11760000696182260071509517729282379150390625f,-0.06109998709857455967409123331890441477298736572265625f,-0.27370000441670416790884701185859739780426025390625f,0.137600004279613497004675082280300557613372802734375f,0.0063999958395957179391189129091799259185791015625f,0.34349999600648895015098105432116426527500152587890625f,0.38730000231862060644516532192938029766082763671875f,0.071600007267296206237006117589771747589111328125f,-0.08959999659061423926686984486877918243408203125f,-0.200699992052465692182749990024603903293609619140625f,-0.5480999993957578197978364187292754650115966796875f,-0.0696000029981134193945990773499943315982818603515625f,0.5896999812006953334275749512016773223876953125f,-0.37909997739046819020813927636481821537017822265625f,-0.46769999011903973240578125114552676677703857421875f,1.03239999244213098705813536071218550205230712890625f,-1.29139996007680935008465894497931003570556640625f,-0.977999968796968488504717242904007434844970703125f,0.7880999636888503800946637056767940521240234375f,-0.20379999508038171285306816571392118930816650390625f,-0.433799967475981151920905176666565239429473876953125f,0.2914000108897685237252517254091799259185791015625f,0.20830002499818789374330663122236728668212890625f,0.370899986660480518452231990522705018520355224609375f,-0.3019000097498292234377004206180572509765625f,1.17890002608895283486845073639415204524993896484375f,-0.43799997639656140790975769050419330596923828125f,0.724800001722574194218395859934389591217041015625f,0.0771999719619742563736508600413799285888671875f,0.8213000121235847217349146376363933086395263671875f,-0.7659000069260599730114336125552654266357421875f,-0.28030000553727152379224207834340631961822509765625f,1.0393999959290027845781878568232059478759765625f,-0.39870000441670416790884701185859739780426025390625f,1.24569999687671639776453957892954349517822265625f,0.1216000014185905087060746154747903347015380859375f,-0.15929997078776381158604635857045650482177734375f,-0.0381000009417533647138043306767940521240234375f,0.471499989688396681231097318232059478759765625f,0.3237999787449836741615172286401502788066864013671875f,-0.0940999976001677396197919733822345733642578125f,-0.689499996632337630586562227108515799045562744140625f,0.54420001381933691408931963451323099434375762939453125f,0.30309999788701580403227353599504567682743072509765625f,0.186499988459050414224549285790999419987201690673828125f,0.54510001180469991322041778403217904269695281982421875f,-0.008699995923042269652114555356092751026153564453125f,-0.02980001252591601978991775467875413596630096435546875f,0.44739999721050249803511178470216691493988037109375f,0.63800000962615011079748228439711965620517730712890625f,0.21880001204907906231511560690705664455890655517578125f,-0.31140001733452127297141487360931932926177978515625f,0.3510000299811366630819975398480892181396484375f,0.8305000294819475659124918820452876389026641845703125f,-0.0240999939680097696736993384547531604766845703125f,0.492400016507506421703510568477213382720947265625f,-0.0819999975264071867542270410922355949878692626953125f,0.1847000015780329018610927960253320634365081787109375f,-0.37910000939071186731865736874169670045375823974609375f,-0.7763999986335630154599130037240684032440185546875f,-0.11280002066195027321526822561281733214855194091796875f,-0.18650001345574873656829595347517170011997222900390625f,0.215300002445280524998594273711205460131168365478515625f,0.7991000020891423361035776906646788120269775390625f,0.5293000332236292848619996220804750919342041015625f,-0.295200019884109210455136462769587524235248565673828125f,0.640799994897842228880335824214853346347808837890625f,-0.13599998740106844508090944145806133747100830078125f,-0.1375999854668972233184831566177308559417724609375f,-0.32500001933425648470432633985183201730251312255859375f,0.5734000020086764681792601550114341080188751220703125f,0.46699998307228096638965553211164660751819610595703125f,0.761100014032423555221384958713315427303314208984375f,0.30520000474080433150447788648307323455810546875f,-0.247299966462702247582683412474580109119415283203125f,0.4866999961614608682936022887588478624820709228515625f,0.4286000102698803715384201495908200740814208984375f,-0.47469999058842660133450408466160297393798828125f,0.01430000908374751134033431299030780792236328125f,0.7596000066861510191529305302537977695465087890625f,-0.40879998675435746946504877996630966663360595703125f,-0.184100021982192885161566664464771747589111328125f,-1.192699981141089438096969388425350189208984375f,1.521000039666891012757332646287977695465087890625f,-1.0513000112295145527241402305662631988525390625f,0.601700005027651751987605166505090892314910888671875f,0.1865000045150519980552417109720408916473388671875f,0.892000008963048429677655803970992565155029296875f,-0.662100030975043996050999339786358177661895751953125f,-0.931399980528652804423472844064235687255859375f,0.18499999914318276950098152155987918376922607421875f,-0.54780000555589791755295436814776621758937835693359375f,0.2444999953098594414058197799022309482097625732421875f,0.20289997494816791601124350563623011112213134765625f,0.60120002930164417875857907347381114959716796875f,-0.7706999876379967417960870079696178436279296875f,-0.467899970477819948655451298691332340240478515625f,-0.517900016969442322789518584613688290119171142578125f,1.07049999811500384794271667487919330596923828125f,-0.58249999113380912518778131925500929355621337890625f,-0.8222999755710365121785798692144453525543212890625f,0.20219997851848603431790252216160297393798828125f,-0.012799984228610838954409700818359851837158203125f,0.0203999913394448117998081215773709118366241455078125f,0.354000019788742292803362943232059478759765625f,0.456600002610683564086002661497332155704498291015625f,-0.204600002253055635037526371888816356658935546875f,-0.5057000198245049649159454929758794605731964111328125f,-0.17470000742673885696376601117663085460662841796875f,0.03310000101625920621017939993180334568023681640625f,-0.25990000600218776671823661672533489763736724853515625f,0.10700000501424067333999801121535710990428924560546875f,-0.00259999295473090086261436226777732372283935546875f,-0.4544000009581450338913555242470465600490570068359375f,-0.6147999950170515148784033954143524169921875f,-0.149600005270540403756740488461218774318695068359375f,0.25399999535083761514897560118697583675384521484375f,-0.0420000044554473539459849007471348159015178680419921875f,0.2220000095963479136429441496147774159908294677734375f,0.590500005602836441909175846376456320285797119140625f,-0.3277000024497509489407320870668627321720123291015625f,0.069999997504055500030517578125f,-0.49120001559257531198454671539366245269775390625f,-0.053199985849857522879347015987150371074676513671875f,-0.191599999928474318267035414464771747589111328125f,0.3653000131741166445209501034696586430072784423828125f,-0.40850000435113908991979769780300557613372802734375f,0.0347999736338854559480893158252001740038394927978515625f,0.451199998046457551037491384704480879008769989013671875f,0.0498000038459895932874132995493710041046142578125f,0.097800010454654806579810610855929553508758544921875f,0.4832999961271882138902356018661521375179290771484375f,1.28390001938343090159833081997931003570556640625f,0.18550000277161604600451028090901672840118408203125f,-1.11209999808072979021744686178863048553466796875f,1.239499995142221422383954632095992565155029296875f,-1.0626999847173692614887841045856475830078125f,-0.225700012522935811176694187452085316181182861328125f,-0.507400011217594482104686903767287731170654296875f,0.620000007040798717383722760132513940334320068359375f,-0.728100015656650167983343635569326579570770263671875f,-0.0760000113546848155010593472979962825775146484375f,0.46620000963211083444548421539366245269775390625f,0.25459998154044161111642097239382565021514892578125f,-0.49100003147125192981548025272786617279052734375f,-0.200699991512298669960046026972122490406036376953125f,-0.506699974703789024488287395797669887542724609375f,-0.20000000022351738238057805574499070644378662109375f,-0.1459999989122149344211720745079219341278076171875f,1.117899995884299446657905718893744051456451416015625f,-0.16440001910030854759270368958823382854461669921875f,0.105199974137544671748401015065610408782958984375f,1.007999993085860523933661170303821563720703125f,-0.417299989503622015263317734934389591217041015625f,1.189099960887432416711817495524883270263671875f,0.744699993419647210401990378159098327159881591796875f,0.220399994766712126192942378111183643341064453125f,-0.09870001251921056706351009779609739780426025390625f,-0.650699965528399371095247261109761893749237060546875f,-0.4475000177323817940333583464962430298328399658203125f,0.9187000157415869949772968539036810398101806640625f,-0.11940001559853552492995731881819665431976318359375f,0.25379999364614480583668409963138401508331298828125f,0.2557999951586127362901379456161521375179290771484375f,0.07899999736249441184554598294198513031005859375f,-0.18069998886734273124687888412154279649257659912109375f,-0.20449999331682899850193280144594609737396240234375f,0.1605999978870152056220632630356703884899616241455078125f,0.3146000125348569209648985633975826203823089599609375f,0.1189000133484601207101150066591799259185791015625f,-0.7129000111579895104796378291212022304534912109375f,0.42789998575150978155079428688623011112213134765625f,0.3964999873042103217812837101519107818603515625f,0.3063999883517622979667294202954508364200592041015625f,-0.0868000046133996505659524700604379177093505859375f,-0.49080003077238820718974920964683406054973602294921875f,0.55920000599622721182413442875258624553680419921875f,0.0778999844476577774088354999548755586147308349609375f,0.7861000157833100132620529620908200740814208984375f,0.1227999851226808658566369558684527873992919921875f,0.575600000718235804697542334906756877899169921875f,-0.27720000128000965577967917852220125496387481689453125f,0.115200001406669638726043558563105762004852294921875f,-0.3080000260919335541842656311928294599056243896484375f,0.50240000615119928539797911071218550205230712890625f,0.242099996627867020126956276726559735834598541259765625f,-0.065699996764957913075733131336164660751819610595703125f,-0.4442000160172583367312881819088943302631378173828125f,-0.04899999892711637272668667719699442386627197265625f,-0.1296999997526404602155025713727809488773345947265625f,-0.122300018151104694652531179599463939666748046875f,0.414400035379827169634836536715738475322723388671875f,-0.3518999828159810050465239328332245349884033203125f,0.8976000204473735255561450685490854084491729736328125f,0.491399997329711624161063809879124164581298828125f,0.3987000157043338877116411822498776018619537353515625f,-0.17789999685287494202157176914624869823455810546875f,-0.74460001633465300319159041464445181190967559814453125f,-0.8862000009149311186007480500848032534122467041015625f,-0.1268999984622001253598000403144396841526031494140625f,0.117899993276596060098881935118697583675384521484375f,0.78509998379051670891470848800963722169399261474609375f,0.09929996736049684358249578508548438549041748046875f,0.221400001055002182415165634665754623711109161376953125f,-0.177599999660253526911901644780300557613372802734375f,-0.3918000084131956217703418587916530668735504150390625f,-0.43200000423192985721243530861102044582366943359375f,0.875700020979345072902333413367159664630889892578125f,1.131599982941150273063612985424697399139404296875f,0.395999999620020304291756474412977695465087890625f,0.622500009834766387939453125f,0.468900013683736405223356769056408666074275970458984375f,0.55349999559670670645772361240233294665813446044921875f,-0.48510000021904697842245468564215116202831268310546875f,-0.1073999894991519710174543433822691440582275390625f,-0.843599998833238995388228431693278253078460693359375f,0.18120002870559659413629560731351375579833984375f,0.38040000869929802007618491188623011112213134765625f,0.5120999867558477802731431438587605953216552734375f,-0.02249999972060322261935283449929556809365749359130859375f,0.0110999998949468103859317125170491635799407958984375f,0.5571000127583742855819082251400686800479888916015625f,-0.28950002535432617367661123353173024952411651611328125f,0.3505000111348923752530026831664144992828369140625f,0.45110003573224022321852544337161816656589508056640625f,-0.0558999893426896932169256615452468395233154296875f,0.57870001693367978390369898988865315914154052734375f,-0.974199991691112376912542458740063011646270751953125f,-0.68080002768039715732584227225743234157562255859375f,-0.6573999823093412686603187466971576213836669921875f,-0.86420002447366695008668102673254907131195068359375f,-0.104799986262619437837884106556884944438934326171875f,0.09020000509470715854831723845563828945159912109375f,-0.20609999700784686904597720058518461883068084716796875f,0.6537000165611506208307446286198683083057403564453125f,0.79230000574588732575875837937928736209869384765625f,1.00870005701780396378808291046880185604095458984375f,-0.8397999945700167501172472839243710041046142578125f,-0.7636000272572041058793956835870631039142608642578125f,-1.04239998201131811583763919770717620849609375f,-0.517400038635731363001468707807362079620361328125f,0.68509997298717539848667001933790743350982666015625f,0.6922999979972832562680196133442223072052001953125f,-1.28309997680187226620773799368180334568023681640625f,-0.9037000267684458520989210228435695171356201171875f,-1.145900032556056657995213754475116729736328125f,-0.650500045537949489471429842524230480194091796875f,1.2079999448060991795728114084340631961822509765625f,0.74620000084042459320698981173336505889892578125f,-0.299099986517429528021239093504846096038818359375f,-0.7575000242888929591345004155300557613372802734375f,-0.28980000395774840882268108543939888477325439453125f,-0.577199995896965134534184471704065799713134765625f,0.5853000257469715972291623984347097575664520263671875f,0.3105000045783814499600339331664144992828369140625f,0.7821000134289264327236423923750407993793487548828125f,-0.0296999809026718963878010981716215610504150390625f,-0.5856000112608075180986588748055510222911834716796875f,-0.539099990242719595556764033972285687923431396484375f,-0.4902000109434128116703277555643580853939056396484375f,0.133800017786025904342750436626374721527099609375f,-0.01139999993741513950595845017232932150363922119140625f,-0.24389999561607844658084331967984326183795928955078125f,0.60510000860095025654317169028217904269695281982421875f,-0.48029999745339158057078066121903248131275177001953125f,0.90870003526210807454077666989178396761417388916015625f,0.24839999351501429458721759147010743618011474609375f,-0.5352000099375844310856109586893580853939056396484375f,-0.2348999902069568435081237112171947956085205078125f,-0.7758000170633199132907975581474602222442626953125f,-1.061699975001811679931051912717521190643310546875f,0.52469998000860196274430791163467802107334136962890625f,0.18779997797012359939117231988348066806793212890625f,-0.81870000396966935429787781686172820627689361572265625f,-0.75329997910261159432820932124741375446319580078125f,-0.81210002449303875238229011301882565021514892578125f,-0.9987000221490855977890532813034951686859130859375f,0.5219999849349259424258207218372263014316558837890625f,0.5834999590516094514214273658581078052520751953125f,-0.946500003956258328940975843579508364200592041015625f,-0.2147999833941458547315050964243710041046142578125f,0.33000000413507224461540090487687848508358001708984375f,-0.5530000328719619773210069979541003704071044921875f,0.5482999966859818385955804842524230480194091796875f,-0.14590001765489635232597720460034906864166259765625f,-1.22469996927976598755094528314657509326934814453125f,-0.6753000319123270145382775808684527873992919921875f,-0.4580999766528608407867295682081021368503570556640625f,-0.6515000047385688386469837496406398713588714599609375f,-0.03129999722242349236012159963138401508331298828125f,-0.1287000027775764277038206273573450744152069091796875f,-0.674600031161308066884885192848742008209228515625f,-0.2658000306606300711109724943526089191436767578125f,-0.652400007343292287487201974727213382720947265625f,-0.37460001968741440947496812441386282444000244140625f,1.0358000061109660094871287583373486995697021484375f,1.014800049965084216552213547402061522006988525390625f,-0.1135999603509905142573188641108572483062744140625f,-0.80500000849366148969465939444489777088165283203125f,0.40019999008178697152970926254056394100189208984375f,0.815200029420852789741047672578133642673492431640625f,-0.383099996173381640574007178656756877899169921875f,0.391499995648860732444518362171947956085205078125f,1.12559996279478102820803542272187769412994384765625f,0.64099999034404664399744433467276394367218017578125f,0.85840000185966491841327297152020037174224853515625f,0.544200021493435048824949262780137360095977783203125f,0.1248000060439107272713954444043338298797607421875f,0.88300003659725234683719463646411895751953125f,1.0658999461293223731672696885652840137481689453125f,0.6527000021517270766935325809754431247711181640625f,0.1359000024557115438028631615452468395233154296875f,-0.8802000008106229955728849745355546474456787109375f,-0.4179000155165792929068402372649870812892913818359375f,0.24690000174045589886873131035827100276947021484375f,-0.82290001238733545552150872026686556637287139892578125f,-1.04609997197389592571425964706577360630035400390625f,-0.60809999215006815820316887766239233314990997314453125f,-0.1352999841541053793747551026172004640102386474609375f,-0.17430000316053628761636673516477458178997039794921875f,-0.033900001651048672357546820421703159809112548828125f,-0.486600017809867946283475248492322862148284912109375f,-0.7728000214815136104107295977883040904998779296875f,-0.238800002735853222901596382143907248973846435546875f,-0.3086999944329260614495069603435695171356201171875f,0.82440002215504659677236531933885999023914337158203125f,0.36689999030381414968360331840813159942626953125f,-0.9143999962270259462826516028144396841526031494140625f,0.0627000279307363683756193495355546474456787109375f,0.65280001473873860984298289622529409825801849365234375f,-0.11190001336336141690708245732821524143218994140625f,0.582299993862211540118778430041857063770294189453125f,-0.276299986195564173385719186626374721527099609375f,0.73979997888654447368850242128246463835239410400390625f,0.91229998726844829803894754149951040744781494140625f,0.47580000409930944815783959711552597582340240478515625f,0.51269999634027474399999846355058252811431884765625f,0.9426000236883760319273051209165714681148529052734375f,-0.1370999956965448518531047739088535308837890625f,0.8756999856635927503845095998258329927921295166015625f,0.994199964272976188794928020797669887542724609375f,0.1400999997720122625111116576590575277805328369140625f,0.8312000322818757780396481393836438655853271484375f,1.3915999707221988757055441965349018573760986328125f,0.92120000605583118868935343925841152667999267578125f,0.836299984295666209987984984763897955417633056640625f,0.23650001386553043403182527981698513031005859375f,-1.1274999773502347721887417719699442386627197265625f,-0.6505000331699850590894129709340631961822509765625f,0.66709999614357951003285052138380706310272216796875f,0.1997000124931336273448323481716215610504150390625f,-0.67379999722242356341439517564140260219573974609375f,-0.708000028103590306471915027941577136516571044921875f,-0.66560001442730420961879644892178475856781005859375f,-0.107799999800324286525210482068359851837158203125f,-0.8731999924063682527730634319595992565155029296875f,-0.541200010824203214809813289321027696132659912109375f,0.227600013965368219714946462772786617279052734375f,-0.02059998902082416094572181464172899723052978515625f,-0.2240999654509128191648414940573275089263916015625f,-0.3414999885521823319578516020555980503559112548828125f,1.09230001118481112598601612262427806854248046875f,1.179900061807037214833826510584913194179534912109375f,0.6905999690532684809340935316868126392364501953125f,0.09119998176693844271767375175841152667999267578125f,-0.23939998296499265251213728333823382854461669921875f,0.405800015014410320191018399782478809356689453125f,-0.25870001507103435045564765459857881069183349609375f,0.413900013199448579825912020169198513031005859375f,0.11750000361353141187237270059995353221893310546875f,0.212500016503036348591848536671022884547710418701171875f,0.136899984753132031300992821343243122100830078125f,-0.3258000300645829128143304842524230480194091796875f,0.6971999800086019494216316161327995359897613525390625f,0.68879997606277498789495439268648624420166015625f,-0.090899998618662269933565767132677137851715087890625f,0.30989997463524343590535181647283025085926055908203125f,-0.6963000168174506843143944934126920998096466064453125f,-0.8504999787807463462740997783839702606201171875f,0.06030000076889996396545257084653712809085845947265625f,0.381299987983703569938143118633888661861419677734375f,-0.82620000866353515700524212661548517644405364990234375f,-0.49589999381303773162699144450016319751739501953125f,0.0438000035181642655146561082801781594753265380859375f,-0.3993000160872937076561584035516716539859771728515625f,-0.2807999983996152248977296039811335504055023193359375f,-0.3095999933123589897832061978988349437713623046875f,-0.2531999953866004293701053029508329927921295166015625f,-0.047700001406669745307453922578133642673492431640625f,-0.64769999099448316570715178386308252811431884765625f,-0.4292999930836263189348755986429750919342041015625f,0.8655000356286766116653552671778015792369842529296875f,0.218399995079636255468358285725116729736328125f,0.3293999870628117054849326450494118034839630126953125f,0.8702999985337260824280747328884899616241455078125f,0.54240001185089348201273651284282095730304718017578125f,-0.21119998191595090730743322637863457202911376953125f,0.1473000040322542947279771397006697952747344970703125f,-0.6891000257074832990866752879810519516468048095703125f,0.3003000131741166978116552854771725833415985107421875f,0.13050000236183390001087900600396096706390380859375f,0.507899997635185673772184600238688290119171142578125f,0.4609000097572801823986310409964062273502349853515625f,0.576600000859797123808903052122332155704498291015625f,0.06460001629740030892889990354888141155242919921875f,-0.9871999874591832480064113042317330837249755859375f,-0.678799999159573896889696698053739964962005615234375f,-0.686299990367889023445968632586300373077392578125f,-0.9595000402927400529051737976260483264923095703125f,-0.08070000410377975441633680020458996295928955078125f,0.1439000006824731059396071941591799259185791015625f,1.139699951994419535594715853221714496612548828125f,0.36949999022483748234435552149079740047454833984375f,0.488800021958351038620094186626374721527099609375f,1.156000055313111118948654620908200740814208984375f,-1.560399979269504910917021334171295166015625f,-1.00340001497268627161929543945007026195526123046875f,0.0544000064529479576691528563969768583774566650390625f,0.333000016573816726150880640489049255847930908203125f,-0.362300000977516134526013047434389591217041015625f,-0.736700017023086406453558083740063011646270751953125f,-0.2638999922633171735242285649292171001434326171875f,-0.863300011587142535063321702182292938232421875f,-1.057899988806247915107405788148753345012664794921875f,-0.6046999969214195136402167918276973068714141845703125f,0.2425000125169756159948519780300557613372802734375f,-0.4805000239312651189038660959340631961822509765625f,0.435199996042251679995160884573124349117279052734375f,0.712000016301870264356921325088478624820709228515625f,1.3377000136256214091190486215054988861083984375f,0.8895000454336410289357672809273935854434967041015625f,0.2263999868988992858476194669492542743682861328125f,-0.171000000923871908753426396287977695465087890625f,-0.1385999999508260582814500594395212829113006591796875f,0.27479997989237292532749279416748322546482086181640625f,-0.4229999747201798054874188892426900565624237060546875f,-0.3695999843716624155831595999188721179962158203125f,-0.601500025525689263616868629469536244869232177734375f,0.1236000142931938938772873370908200740814208984375f,0.134099999257922153361732853227294981479644775390625f,-0.1371000027000903731533298923750407993793487548828125f,0.414299975220858929247924606897868216037750244140625f,0.962099965000152934635480050928890705108642578125f,0.888000023335218546804981087916530668735504150390625f,-0.2736000050544742379088347661308944225311279296875f,-0.80219998392015678945199397276155650615692138671875f,-0.8396999794125559191115826251916587352752685546875f,-0.00719999772235746515747223384096287190914154052734375f,-0.456300013576448060259593830778612755239009857177734375f,-0.4347000045955180524970273836515843868255615234375f,0.27660000171661369705589095246978104114532470703125f,-0.4856999986276029357412653553183190524578094482421875f,0.4983000074148176139487986802123486995697021484375f,-0.609899980670213448519234589184634387493133544921875f,-0.75719999971538809457882734932354651391506195068359375f,-0.3363000192761422812992577746626920998096466064453125f,0.5595000176429749672024627216160297393798828125f,0.5982000004157423944661786663345992565155029296875f,0.012599993997812308776929057785309851169586181640625f,0.85530002133250249141127596885780803859233856201171875f,0.86610000583678459218361922467011027038097381591796875f,0.2357999951586127185265695516136474907398223876953125f,0.09019998978376353448993540951050817966461181640625f,-0.64279999495744721826895329286344349384307861328125f,-0.9050000217556952719633045489899814128875732421875f,-0.3103999964058399285704581416212022304534912109375f,-0.4560000164210797901631622153217904269695281982421875f,0.0366999744057654453399663907475769519805908203125f,-0.1887000188708309877938518184237182140350341796875f,0.096299995657801673587528057396411895751953125f,0.09170000204741957094967119701323099434375762939453125f,-0.23219995795488390655236798920668661594390869140625f,-0.70599998617172143866582700866274535655975341796875f,-0.3824000025749203501845840946771204471588134765625f,-1.053000060290099071380609530024230480194091796875f,-0.39110001921057691021132995956577360630035400390625f,0.38890000463128071572782573639415204524993896484375f,-0.31119996343851130671254168191808275878429412841796875f,-0.8055999941617242310343272038153372704982757568359375f,0.36570002667903889204126244294457137584686279296875f,-0.39749999120831436272283099242486059665679931640625f,0.2878999938726423124535358510911464691162109375f,0.7249000328242782842380620422773063182830810546875f,0.8057999820828438686248773592524230480194091796875f,0.1480000096261502307015689439140260219573974609375f,-0.242599996381998028738280481775291264057159423828125f,-0.036000000625848738167178453295491635799407958984375f,-0.6791999969065187769956537522375583648681640625f,0.086800002825260236249960144050419330596923828125f,0.229000006899237718016593134962022304534912109375f,-0.046799997833371076438879754277877509593963623046875f,-0.035700007009506151689492980949580669403076171875f,-0.31860000088214857072443919605575501918792724609375f,-0.76979999900311224525495390480500645935535430908203125f,-0.10889999017715457085841990192420780658721923828125f,-0.5472000054895878040639445316628552973270416259765625f,-0.223199995833635256303750793449580669403076171875f,0.1412999959185718967802358747576363384723663330078125f,0.4761000020742418570307563641108572483062744140625f,0.2210999993920326200935733140795491635799407958984375f,0.20189999838769434603591435006819665431976318359375f,-0.78479998067468381695022117128246463835239410400390625f,-0.04709998347759292158798416494391858577728271484375f,-0.8940000349134209489676550219883210957050323486328125f,0.210300018799305377115160808898508548736572265625f,0.26130000050067903050177164914202876389026641845703125f,-0.643800019872188489244990705628879368305206298828125f,-0.77159998070597624053590379844536073505878448486328125f,0.04440002219229899349528523089247755706310272216796875f,-0.48329999426454295363697610810049809515476226806640625f,0.3816000088691708924670820124447345733642578125f,0.863100020536780476732019451446831226348876953125f,0.158699994134902766518280259333550930023193359375f,0.3989999987035990347550296064582653343677520751953125f,0.96989998499155039013430723571218550205230712890625f,-0.118800000388920301563899784014211036264896392822265625f,-0.303300001305341737634080345742404460906982421875f,-0.1650000012293457718470790496212430298328399658203125f,-0.939600012385845229090364227886311709880828857421875f,-0.13440000349134206825141291119507513940334320068359375f,0.1274999983608722242678368274937383830547332763671875f,0.331499998256564065712836963939480483531951904296875f,0.62740000641196946862265804156777448952198028564453125f,1.1363000133156777593512742896564304828643798828125f,0.2967000040590761500425287522375583648681640625f,-0.4535999788284303946284126141108572483062744140625f,-1.15919999735355272463266373961232602596282958984375f,-0.0977000240564345201477181035443209111690521240234375f,0.677500012069940371617349228472448885440826416015625f,0.071500013679266061217276728712022304534912109375f,-0.0354999878704542748408812258276157081127166748046875f,-0.5399999849498271498049462024937383830547332763671875f,-0.115800010097026895294902715249918401241302490234375f,0.02049996805191067750229194643907248973846435546875f,-0.7663000413298615143276037997566163539886474609375f,0.019500008814036906557021211483515799045562744140625f,0.107100005345046600524483437766321003437042236328125f,0.222299973373115022923229844309389591217041015625f,0.379499994881450586348137221648357808589935302734375f,-1.1231999681174755778556573204696178436279296875f,-0.9388000049710265582092461045249365270137786865234375f,-0.3620999971866609712378703989088535308837890625f,-0.791300025981664933993897648178972303867340087890625f,-0.9480000103712082903939517564140260219573974609375f,-0.203200010883808346306977909989655017852783203125f,0.89409999396800987625510970246978104114532470703125f,0.51110002904534379553069811663590371608734130859375f,0.16159999761879451085633263573981821537017822265625f,-0.16260000241696825895587608101777732372283935546875f,0.8675000053644179587308826739899814128875732421875f,0.993500048011541903036913936375640332698822021484375f,0.5383999925091860205839111586101353168487548828125f,-0.249599992623180055861809023554087616503238677978515625f,0.1916999898657201217844203711138106882572174072265625f,0.81540001205205925316477078013122081756591796875f,-0.8063999849617478066221565313753671944141387939453125f,0.403200003433227038840414024889469146728515625f,0.60150000183284280108608754744636826217174530029296875f,-0.4206000220268963918357485454180277884006500244140625f,0.153300003205239843140361699624918401241302490234375f,0.29250000163912748263328467146493494510650634765625f,-0.1434000039830804762441829325325670652091503143310546875f,-0.4844999847859143837780493413447402417659759521484375f,-0.88950002717971810017871803211164660751819610595703125f,0.15599997842311896789624370285309851169586181640625f,-0.05970000068396330572539909553597681224346160888671875f,-0.21000000096857551756812654275563545525074005126953125f,0.57149999814480534343630324656260199844837188720703125f,0.2843999976798894380181081942282617092132568359375f,-0.719399991644918745947734350920654833316802978515625f,-0.4415999863684181292455832590349018573760986328125f,-0.937500002793967723846435546875f,-0.2117999900102613963781550410203635692596435546875f,-0.4115999965757131118238021372235380113124847412109375f,-1.0320000087022780377310482435859739780426025390625f,0.66570001147985469192036589447525329887866973876953125f,0.81029999018907528096633541281335055828094482421875f,0.028199996839463725706309560337103903293609619140625f,0.4232999989211558844459659667336381971836090087890625f,0.7173000177785755493431452123331837356090545654296875f,0.122999983549117786196802626363933086395263671875f,0.83520001887828121400758618619875051081180572509765625f,-0.53539999566078133597102350904606282711029052734375f,-0.6101000228129327407344817402190528810024261474609375f,-0.010700001849979212664720762404613196849822998046875f,-0.400800023806095140344041283242404460906982421875f,0.1876000054717064102760559762828052043914794921875f,-0.625699984955787602558530124952085316181182861328125f,-0.29170001150965685354776724125258624553680419921875f,0.0031999760150913658662830130197107791900634765625f,0.4249999810755245022164672263897955417633056640625f,-0.0100999700069426268100869492627680301666259765625f,-0.796300008398294068001632695086300373077392578125f,-0.70350000051409011803116300143301486968994140625f,-0.7125000260397793550026790398987941443920135498046875f,0.861299983215332076724735088646411895751953125f,0.77970001647919395804109399250592105090618133544921875f,-0.3073999969124798781194840557873249053955078125f,-1.08420001776814434180096213822253048419952392578125f,-0.684199991691112785474615520797669887542724609375f,0.04599999794363984761957908631302416324615478515625f,-0.4835000138878822184551609097979962825775146484375f,0.3737000068008899944516087998636066913604736328125f,-0.313800017488002680465797311626374721527099609375f,0.41760000696182242307941123726777732372283935546875f,0.7046999928236008514659260981716215610504150390625f,0.51570001401305187727075463044457137584686279296875f,-0.439000009655952538878409541212022304534912109375f,0.1650000190734868255049150320701301097869873046875f,0.19870003436803784779840498231351375579833984375f,-0.4184999943673606281890897662378847599029541015625f,0.96909999679923064519471154198981821537017822265625f,0.144600014621019123950418361346237361431121826171875f,0.08730000431165108043529698989004828035831451416015625f,-0.444300017689168436074709234162583015859127044677734375f,0.3444000067323447211720122140832245349884033203125f,-0.25740001807212831153037768672220408916473388671875f,-0.212700003679096749653609776942175813019275665283203125f,-0.378599985831975960337558717583306133747100830078125f,0.5543999765574929572409246247843839228153228759765625f,-0.0039000059723850455384308588691055774688720703125f,-0.9813000078767537548429800153826363384723663330078125f,0.068700005459785273842499009333550930023193359375f,-0.42510000424236060734273223715717904269695281982421875f,-0.06149998704344028510604402981698513031005859375f,0.74669999847114076896303913599695079028606414794921875f,0.470399987558275556320808163945912383496761322021484375f,-0.68759998703151925525389742688275873661041259765625f,0.5598000194549559438428332214243710041046142578125f,0.3738000123471021396426294813863933086395263671875f,-0.27209997423887255507679583388380706310272216796875f,0.5847000060483813488332316410378552973270416259765625f,-0.3390000063776970051776515902020037174224853515625f,0.433200001756846919676036122837103903293609619140625f,-0.42090001586675640510293305851519107818603515625f,0.3026999984264373555475913235568441450595855712890625f,0.2180999875307083613051872816868126392364501953125f,0.371400015807151984148504197946749627590179443359375f,-0.2280000151395800145337489084340631961822509765625f,-0.56909998819380991363914290559478104114532470703125f,0.16260002394914607748432899825274944305419921875f,0.02489999888688332330133334835409186780452728271484375f,0.2398000115573406532121225609444081783294677734375f,0.1965999605894090951352382035111077129840850830078125f,0.907600012624263374760857914225198328495025634765625f,-0.4950000010430812835693359375f,-0.2922000098109245325161964501603506505489349365234375f,-0.01860002412796024628960367408581078052520751953125f,0.51860001667737964936577554908581078052520751953125f,1.10040000534653614749913685955107212066650390625f,1.0266000520825391273405102765536867082118988037109375f,-0.9585000189542771664719111868180334568023681640625f,-0.05310001241564776819359394721686840057373046875f,0.72239999542236343899048733874224126338958740234375f,0.1227999967455861707321673748083412647247314453125f,-1.188299995791911811693353229202330112457275390625f,-0.974500046700239597186055107158608734607696533203125f,-0.07850003683567052803482511080801486968994140625f,0.5715000120401381167312138131819665431976318359375f,-0.515099981629848713282626704312860965728759765625f,0.1597000093638900608539188397116959095001220703125f,0.8800000232458113913480701739899814128875732421875f,0.441400032496452876529247077996842563152313232421875f,0.30930000342130625767822493799030780792236328125f,1.102100065582991561541348346509039402008056640625f,-0.5819999607205392777586894226260483264923095703125f,-0.30359997599720855987470713444054126739501953125f,0.24489996085166954031819841475225985050201416015625f,0.8961000011801711906400669249705970287322998046875f,0.8097000044465065826670979731716215610504150390625f,0.500300012391805548617185195325873792171478271484375f,-1.18180001027584058448383075301535427570343017578125f,0.7109999980926513529766452847979962825775146484375f,0.90059999706745141878627691767178475856781005859375f,-0.1557000316709278298077379076858051121234893798828125f,-0.196800000925362095216542002162896096706390380859375f,-0.2399999983608722686767578125f,0.58620000978112229983452152737299911677837371826171875f,-0.12780002058744432957837489084340631961822509765625f,0.805200032401085064037715710583142936229705810546875f,0.3866999983966348253261458012275397777557373046875f,0.1215000049620866828803400494507513940334320068359375f,-0.6615000007152556094069950631819665431976318359375f,0.18479998905658712171629076692624948918819427490234375f,0.68820000015497218015525504597462713718414306640625f,-0.806700022611022138363523481530137360095977783203125f,-0.5468999871373174670452499412931501865386962890625f,0.79590000923573978042213639128021895885467529296875f,-0.02400002187490457572494051419198513031005859375f,0.445500015176832686591978927026502788066864013671875f,-0.2057999749302865666322759352624416351318359375f,0.1974000107333064857328963626059703528881072998046875f,0.370500000834464771060083876363933086395263671875f,0.69750003855675490882504163892008364200592041015625f,-0.5736000262141232042267802171409130096435546875f,-0.6149999802187082753590630090911872684955596923828125f,-0.7769999979734425465949243516661226749420166015625f,0.70169998997747884406095408849068917334079742431640625f,-0.31980002601146662044584445538930594921112060546875f,0.2900999965310095962678360592690296471118927001953125f,0.3230999991536140925063591566868126392364501953125f,-0.81780001730918894242705619035405106842517852783203125f,0.413999999932944395819589544771588407456874847412109375f,0.898899981236457978894804909941740334033966064453125f,0.77630001614689803091096109710633754730224609375f,0.2251000108361242979526650742627680301666259765625f,0.0515000096559525832873305262182839214801788330078125f,-1.092999982357024890688990126363933086395263671875f,-0.47000001668930035947369105997495353221893310546875f,0.12279997946023968324880115687847137451171875f,0.46459999241828864313674785080365836620330810546875f,0.4209000062741339309724253325839526951313018798828125f,-0.219300006010383352617054697475396096706390380859375f,0.738999950647354353350237943232059478759765625f,0.53059998574256805881077525555156171321868896484375f,-0.6541999887108804756508106947876513004302978515625f,-0.528200015354156615643432814977131783962249755859375f,0.3705000100731847822999043273739516735076904296875f,0.619100027644634565149317495524883270263671875f,-0.45689999361932276400466435006819665431976318359375f,-0.068300007489323633080857689492404460906982421875f,-0.1320000005811454713011698913760483264923095703125f,0.05900000987947018327162140849395655095577239990234375f,-0.5039999794065952443133937777020037174224853515625f,0.24740000510811821499146390124224126338958740234375f,-0.5055000193119045803769040503539144992828369140625f,-0.876700050252676721385114433360286056995391845703125f,-0.6760999854594469393731515083345584571361541748046875f,-0.63510001687109463031077893901965580880641937255859375f,-0.0671000237107275410153306438587605953216552734375f,0.413300001454353083119030998204834759235382080078125f,0.3992999822616578597944680950604379177093505859375f,0.269099989981949505590819171629846096038818359375f,0.6776999961912630876526009160443209111690521240234375f,0.431699982601404297355429662275128066539764404296875f,-0.25529999406337733347527318983338773250579833984375f,-0.160200000363588390683844409068115055561065673828125f,-0.27720000940114264409430688829161226749420166015625f,-0.61349998944997796712641502381302416324615478515625f,0.5273999861463902760760902310721576213836669921875f,0.0158999920248987791637773625552654266357421875f,-0.0345000047720969149622760596685111522674560546875f,0.6747000113382934927130918367765843868255615234375f,0.856199982735514453224823228083550930023193359375f,0.854999985098839232477985206060111522674560546875f,-0.8453999977469444093713946131174452602863311767578125f,-0.7685999916434287371203026850707828998565673828125f,0.5235000154897571089662733356817625463008880615234375f,0.007500004172324992168796597979962825775146484375f,0.102299998891353549623772778431884944438934326171875f,-0.2006999871909618260445995474583469331264495849609375f,0.336600012780726043359891264117322862148284912109375f,0.172500005140900469768894254229962825775146484375f,0.269400004385411673268890808685682713985443115234375f,-0.526499981641769476681247397209517657756805419921875f,-0.5517000289812685753787491194088943302631378173828125f,0.2220000021457675387637209496460855007171630859375f,-0.6351000024542212685219055856578052043914794921875f,-0.3902999884009361597492215878446586430072784423828125f,0.5420999964416026717373142673750407993793487548828125f,-0.0648000146865843618115832214243710041046142578125f,0.5498999959811567972423063110909424722194671630859375f,-0.9785000084117048135823324628290720283985137939453125f,-0.269700014579296265537777799181640148162841796875f,0.138300000858306848527945476234890520572662353515625f,-0.07270000647306407159931040951050817966461181640625f,-0.94950003656744996760608046315610408782958984375f,-0.272500005960464530829767682007513940334320068359375f,-0.148900008282065454690012984428904019296169281005859375f,-0.223199996131658195963609614409506320953369140625f,-0.4622000247120860993987889742129482328891754150390625f,0.76610001470297550696386679192073643207550048828125f,0.853900049670041294547218058141879737377166748046875f,0.19269998844265945336928780307061970233917236328125f,0.4965000074207781555202245726832188665866851806640625f,-0.6222999762415886149113930514431558549404144287109375f,-0.136500008165836295148665158194489777088165283203125f,-0.05569999061822894503848146996460855007171630859375f,-0.0810999955177305054121461580507457256317138671875f,-0.2671999591469766954787701251916587352752685546875f,-0.673400001412629567454359857947565615177154541015625f,-0.20260000338554373655597373726777732372283935546875f,-0.47000001460313800549783991300500929355621337890625f,-0.0319999955147505232844196143560111522674560546875f,0.2618000073701143293192217242904007434844970703125f,-0.36519998710155476828731480054557323455810546875f,-0.6444000062108035109531556372530758380889892578125f,-0.55359998784363284585197106935083866119384765625f,-0.76680002420842663379829673431231640279293060302734375f,0.33159998830556869364727390347979962825775146484375f,0.3818000066995619334875300410203635692596435546875f,-1.3854999692440035374829676584340631961822509765625f,-0.35999997582286624719927203841507434844970703125f,0.426300008323788670594467475893907248973846435546875f,0.0294000017404556501787737943232059478759765625f,-1.011300026987492994834383353008888661861419677734375f,0.20639999017715471296696705394424498081207275390625f,0.09270000066161154794741605655872263014316558837890625f,0.07800000578910115667241598202963359653949737548828125f,-0.74040002300441276616993491188623011112213134765625f,-0.353100006157159640451936866156756877899169921875f,0.7239000254184009275348898881929926574230194091796875f,0.24629999644011224546602534246630966663360595703125f,0.63569999974519009189322105157771147787570953369140625f,0.15509999638199811755612245178781449794769287109375f,-0.1539000006079674631731535328071913681924343109130859375f,-0.5408999770492315395387095122714526951313018798828125f,-0.12989999665170903764277454683906398713588714599609375f,-0.07709999808073053628731940989382565021514892578125f,-0.8351999921679496008408705165493302047252655029296875f,-0.185099975967407459620517329312860965728759765625f,-0.32400000708550213968095476957387290894985198974609375f,0.14969999878406525795071502216160297393798828125f,0.10260000379532574488194995865342207252979278564453125f,-0.3044999990165233594296978480997495353221893310546875f,-0.34080000197589388477581451297737658023834228515625f,0.242400003767013316746670170687139034271240234375f,-0.9144000031560659547125169410719536244869232177734375f,-0.3977999857932328353626871830783784389495849609375f,0.41820000108629462598486270508146844804286956787109375f,0.20729999501705176356836091144941747188568115234375f,-0.5498999945282936341328650087234564125537872314453125f,0.267000020772218960019017686136066913604736328125f,-0.49199999693036033932003192603588104248046875f,-0.5118000299453739909694149901042692363262176513671875f,-0.16249998942017551684102727449499070644378662109375f,0.362300015580654388713810476474463939666748046875f,-0.4804999981522559693303264793939888477325439453125f,0.2247000072777272094981526606716215610504150390625f,-0.578300007265806215173142845742404460906982421875f,-0.256500013679266114507981910719536244869232177734375f,-0.846699992100894416324763369630090892314910888671875f,0.009899995161593011516742990352213382720947265625f,1.078699969696998550716671161353588104248046875f,0.44650000950694046508715473464690148830413818359375f,-0.614399996599555020537763994070701301097869873046875f,0.06239999549686903179690489196218550205230712890625f,0.3665999763846397030420121154747903347015380859375f,0.77120002006292320828606534632854163646697998046875f,-0.6026999823331831152017912245355546474456787109375f,-0.462700017350912151670172534068115055561065673828125f,-1.50909998383522037102011381648480892181396484375f,-0.9925000314414500479642811114899814128875732421875f,-0.87189997283220321833141497336328029632568359375f,-0.24330001099109654916219369624741375446319580078125f,0.6543000189185139703340610140003263950347900390625f,0.890100062245131606886161534930579364299774169921875f,0.5016999705314635349395757657475769519805908203125f,0.10789999163746788468642989755608141422271728515625f,0.5671999861180783142344807856716215610504150390625f,0.5420000119805334914957484215847216546535491943359375f,-1.1702000148177145177896818495355546474456787109375f,0.9536999902606009271721632103435695171356201171875f,-0.834000027313828606878587379469536244869232177734375f,-0.631500008910894194968932424671947956085205078125f,0.22290000568181278506330045274808071553707122802734375f,-0.438000001728534815725879525416530668735504150390625f,0.0525000036880374221226475128787569701671600341796875f,-0.7170000072121620604548297706060111522674560546875f,-0.1017000014886260128577077921363525092601776123046875f,-0.261899997568130427083588074310682713985443115234375f,0.1269000012561678492062355871894396841526031494140625f,-0.64709998995959761014518107913318090140819549560546875f,0.258299999517202394372361595742404460906982421875f,0.614099976682663140081785968504846096038818359375f,0.117600001224875405636538516773725859820842742919921875f,-0.5231999921083450022507577159558422863483428955078125f,0.4008000067815185030184466086211614310741424560546875f,-0.36270002673864354392208042554557323455810546875f,-0.6393000112071633989074825876741670072078704833984375f,-0.5855999928951263200360699556767940521240234375f,-0.78750000335276126861572265625f,-0.7922999801158905341935678734444081783294677734375f,-0.43049999170750385957262551528401672840118408203125f,-0.923699968504905655208858661353588104248046875f,0.6615000331252816057059362719883210957050323486328125f,0.0188999880909914708126962068490684032440185546875f,0.3416999975770711461819928445038385689258575439453125f,0.6860999955177309317377876141108572483062744140625f,0.5873999993339180658580289673409424722194671630859375f,0.3755999900639057642592888441868126392364501953125f,-0.1950000116229058111372296480112709105014801025390625f,-0.41890001506209362247545868740417063236236572265625f,-0.187799976964294845771519248955883085727691650390625f,0.20280000758618133005484196473844349384307861328125f,1.3153999804615974511534659541212022304534912109375f,0.987800017309188316261270301765762269496917724609375f,-1.111599960291385968957911245524883270263671875f,-0.669199996012448394822058617137372493743896484375f,-0.0307000005833806266508645421708934009075164794921875f,0.183099997980147310983056740951724350452423095703125f,0.291200020360946609798702411353588104248046875f,-0.3340000074207782887469875277020037174224853515625f,0.330899973659217749855088186450302600860595703125f,0.503299995978176095690059810294769704341888427734375f,1.2305000208020206997616696753539144992828369140625f,0.361300016742944762881961651146411895751953125f,0.538299986255168505522306077182292938232421875f,0.31050001573562635570624479441903531551361083984375f,0.949900007045268779393154545687139034271240234375f,0.668300034236908313545200144289992749691009521484375f,-0.5609000034987925431551047950051724910736083984375f,-0.7145000319480898642865440706373192369937896728515625f,-0.6701999921679497873583386535756289958953857421875f,-1.0244000270724296086655158433131873607635498046875f,0.227300006788969000126598984934389591217041015625f,-0.0689000032901765369075519629404880106449127197265625f,-0.16149999505281442679915926419198513031005859375f,-0.344700007128715402160423764144070446491241455078125f,0.23259996358081735934320022352039813995361328125f,0.34619998253062289617076885406277142465114593505859375f,-0.68419999109506601797647817875258624553680419921875f,-0.44009998908042913257077088928781449794769287109375f,-0.004799999561905909928327673696912825107574462890625f,-0.48089999031126506867650505228084512054920196533203125f,0.6185999976038931702504442000645212829113006591796875f,0.4301999862074854519278233055956661701202392578125f,-0.47219998012036068502084162901155650615692138671875f,-0.5537999724864963724257904686965048313140869140625f,-0.03449999672546988360721798017038963735103607177734375f,-0.341099993543326707179375034684198908507823944091796875f,-0.529800004591047812851911658071912825107574462890625f,0.16560001435279847470383174368180334568023681640625f,0.70409998349994407096374970933538861572742462158203125f,0.3491999843522908264503712416626513004302978515625f,0.1563000095531343891508413435076363384723663330078125f,0.765000022947788238525390625f,0.64170001937001952541095306514762341976165771484375f,0.7569000065088271611557502183131873607635498046875f,0.513900016924738967105668052681721746921539306640625f,0.4541999991416929560728021897375583648681640625f,-0.3999000131919980827177596438559703528881072998046875f,0.10080001084208500827799070975743234157562255859375f,-0.8268000029370188297406230049091391265392303466796875f,-0.0200999863982200821510559762828052043914794921875f,-0.326700002717971837995492023765109479427337646484375f,-0.05909999679923050308616438996978104114532470703125f,-0.236700003388524038427931373007595539093017578125f,0.09900000250339502372298738919198513031005859375f,0.592799978808313365874482769868336617946624755859375f,0.3677999855510893922883042250759899616241455078125f,-0.62640001055449257005847130130860023200511932373046875f,-0.8631000345438721854662844634731300175189971923828125f,0.3878000007875261889722651176271028816699981689453125f,0.1001999965079125587408270803280174732208251953125f,0.372900005197525086231280511128716170787811279296875f,-0.36790001041293152184152859263122081756591796875f,-0.379799983990192657756779226474463939666748046875f,-0.223599996113776899875347226043231785297393798828125f,0.871800015528500171768655491177923977375030517578125f,0.2026000143006445597393394564278423786163330078125f,0.15969997375011502072084113024175167083740234375f,0.81810000541210126101532296161167323589324951171875f,1.7118000439524649891609442420303821563720703125f,0.834200052934885860622671316377818584442138671875f,0.52600001053512102089371182955801486968994140625f,-0.13340000133812424110146821476519107818603515625f,0.94560000272989241931753667813609354197978973388671875f,0.7128000434607273394505000396748073399066925048828125f,-0.967499995939433699021492429892532527446746826171875f,-0.06490001520365495935038779862225055694580078125f,-0.5469999996125698427107408861047588288784027099609375f,-0.29420000282973035599098921011318452656269073486328125f,0.1593000022292141437674217741005122661590576171875f,-0.7755000257194044621655848459340631961822509765625f,-0.5205999894678594586139297462068498134613037109375f,0.08840000051856033991271033301018178462982177734375f,-0.7730000092908735354058080702088773250579833984375f,-0.801600042136013879456868380657397210597991943359375f,-0.26969999513328080098517602891661226749420166015625f,0.410700008872151300920450012199580669403076171875f,0.270999983787536535828621708787977695465087890625f,-0.96480001263767467545306999454624019563198089599609375f,-0.08399999909475452941620687852264381945133209228515625f,0.110400009854138059228745305517804808914661407470703125f,-0.399300008748471768615928567669470794498920440673828125f,0.439200003463029897687874836265109479427337646484375f,-0.395999988444149408906014286912977695465087890625f,-0.46289999148845684118214194313623011112213134765625f,-0.0942000030532479382117116983863525092601776123046875f,0.38939999764263621973015006005880422890186309814453125f,0.52169998036622988735189210274256765842437744140625f,-0.4854000364899633268578327260911464691162109375f,0.47760001385360961734249940491281449794769287109375f,0.8634000096082683484155495534650981426239013671875f,-0.16860000576227907043147524746018461883068084716796875f,0.4905000060498714287149368828977458178997039794921875f,0.70620002486854804857330236700363457202911376953125f,0.60179999459236832581865428437595255672931671142578125f,0.1767000027179717047687290687463246285915374755859375f,-0.58949997703731049636388661383534781634807586669921875f,-0.2975999945193528173348340715165250003337860107421875f,-0.399000002820044830098566990272956900298595428466796875f,-1.0461000269964337849870616992120631039142608642578125f,0.1193999847531319602467192453332245349884033203125f,-0.19559999226182700482468135305680334568023681640625f,-0.831299990963935808707674368633888661861419677734375f,-0.52080002879798426018709278650931082665920257568359375f,-0.1005000021383164465760273742489516735076904296875f,0.0951000050246715300517053037765435874462127685546875f,-0.7536000010311603158896787135745398700237274169921875f,-0.45660000596344474388388334773480892181396484375f,-1.05570005141496725542538115405477583408355712890625f,0.5006000048160554882770156837068498134613037109375f,0.737200029182434324326322894194163382053375244140625f,1.40719998418092728798001189716160297393798828125f,0.94500002384185766146629248396493494510650634765625f,-0.13610002561807643672864287509582936763763427734375f,0.51790001190304746359061027760617434978485107421875f,0.71909999806582913794983369371038861572742462158203125f,0.71610003053545978612959288511774502694606781005859375f,-0.18070000671148278570399270392954349517822265625f,-0.383300022017956099062985231284983456134796142578125f,0.2434999668002131301136614638380706310272216796875f,0.7121000120878218098141587688587605953216552734375f,-0.03129999901056290667611392564140260219573974609375f,0.298299998474120808822362960199825465679168701171875f,-0.11899999557435503305669044493697583675384521484375f,0.0150000030547381424383956982637755572795867919921875f,1.0608999991774556992396583154913969337940216064453125f,0.265700026977062631061698994017206132411956787109375f,-0.22050002795457857018845970742404460906982421875f,-0.59070002489090001063232193700969219207763671875f,-0.71259997596740731751197017729282379150390625f,-0.876000007629393895314251494710333645343780517578125f,0.59559999170303346005539424368180334568023681640625f,-0.0002000015556813394823620910756289958953857421875f,0.1376999845497308694319826827268116176128387451171875f,0.314100003486126322371774222119711339473724365234375f,-0.625700008499622040147869483917020261287689208984375f,-0.797700040149689204582728052628226578235626220703125f,-0.2202000005125999138044790015555918216705322265625f,0.190799992811680141358010587282478809356689453125f,0.3852000021144748043155914274393580853939056396484375f,-0.20670001881122601616880274377763271331787109375f,0.73740000089853996456668028258718550205230712890625f,0.713099986636638760728601482696831226348876953125f,0.60090001046508556559189173640334047377109527587890625f,-0.21780001492500300486199193983338773250579833984375f,0.68070001393854628890522917572525329887866973876953125f,0.36029999447315919081091806219774298369884490966796875f,-0.318900015695393168613236412056721746921539306640625f,0.04589999798536315012142949854023754596710205078125f,0.9656999961689114453378124380833469331264495849609375f,0.25349997425079351387466886080801486968994140625f,0.13349999816715707456893369453609921038150787353515625f,-0.32339999440908417938089769450016319751739501953125f,-0.0930000014305114586221634453977458178997039794921875f,-0.2370000051259995021979420926072634756565093994140625f,0.2451000136300922893628495558004942722618579864501953125f,0.8531999918848274244709273261833004653453826904296875f,-0.22110000151544806357151173870079219341278076171875f,0.43049999904632585412400658242404460906982421875f,-0.56879999413043247358245935174636542797088623046875f,0.011700015234946903319723787717521190643310546875f,0.12959999621808526715227571912691928446292877197265625f,0.69329999907016759408406869624741375446319580078125f,0.00450000443682074813267490753787569701671600341796875f,-0.338400010688602759234555605871719308197498321533203125f,-0.67890002385377901106267017894424498081207275390625f,-0.150600000494718333499122309149242937564849853515625f,-0.9432000133797526775225605888408608734607696533203125f,0.04259999757409094200966137577779591083526611328125f,1.1021000146955248766289514605887234210968017578125f,0.99830005368292396017437795308069325983524322509765625f,-0.4158999881133436105073997168801724910736083984375f,-0.326700000222026887275461604076554067432880401611328125f,0.15500000946223746467467208276502788066864013671875f,0.0860000072568656293015010305680334568023681640625f,0.9967000015258786760341536137275397777557373046875f,0.727900042301417027346133181708864867687225341796875f,-0.14469998060464916989076300524175167083740234375f,-0.64709999524950934102207611431367695331573486328125f,1.278699984002113154701874009333550930023193359375f,1.065500038087368039896318805404007434844970703125f,-0.94790000489950188011789578013122081756591796875f,-0.17590000364780422614785493351519107818603515625f,-0.703999972999096002013175166212022304534912109375f,-0.633800000053643675101966437068767845630645751953125f,-0.473600004681944852791275479830801486968994140625f,-0.51180002972185667520221841186867095530033111572265625f,-0.658700005757808515483020528336055576801300048828125f,-0.33830002619028132215817095129750669002532958984375f,-1.11580000026226056064615477225743234157562255859375f,-0.68140001342296585562507971189916133880615234375f,0.225299991977214786320615758086205460131168365478515625f,-0.11749999869614835557030119161936454474925994873046875f,0.20189998534917830141921513131819665431976318359375f,0.8941000282406807997404030174948275089263916015625f,-0.7337999993458392822276437073014676570892333984375f,-0.7564000251576306066425559038179926574230194091796875f,1.235500025272369128970240126363933086395263671875f,-0.112800017160176846431340891285799443721771240234375f,-0.443399983382224860406495281495153903961181640625f,0.093600033664703374824966886080801486968994140625f,-0.1041000106573104488916214904747903347015380859375f,-0.0367999798774718556160223670303821563720703125f,0.46259997917115658339071160298772156238555908203125f,-0.18920001314878476250669336877763271331787109375f,0.14220001457929587473927313112653791904449462890625f,0.1436999989032745617123509873636066913604736328125f,0.6576000066637988084039534442126750946044921875f,0.95649999237060523427089719916693866252899169921875f,0.463199990171194286858735722489655017852783203125f,0.451999997377395601461103069595992565155029296875f,0.6738000148057938787360399146564304828643798828125f,0.33300000590085954144115021335892379283905029296875f,-1.036499935150146711748675443232059478759765625f,-0.2176999468684197580614636535756289958953857421875f,-0.160499996662139210457098670303821563720703125f,0.247999990701675443460771930404007434844970703125f,-0.200699978585541227715793866082094609737396240234375f,0.20520001377463348291030342807061970233917236328125f,-0.79679998069703561469623309676535427570343017578125f,-0.98979996979683659841242615584633313119411468505859375f,-1.20149997985362944064036128111183643341064453125f,-1.385899978315830338715386460535228252410888671875f,-0.452999966263771369767709984444081783294677734375f,-0.08849997514486318550552823580801486968994140625f,-0.0095999989748001279821210118825547397136688232421875f,-0.012500000018626444830971422561560757458209991455078125f,1.08240000376701317463812301866710186004638671875f,0.2757999961271881961266672078636474907398223876953125f,-0.115100020875782027562905796003178693354129791259765625f,-0.21079999795258042016854460598551668226718902587890625f,0.1502999967455860286236202227883040904998779296875f,-0.0580000118613241966158966533839702606201171875f,0.78400001233816141166244051419198513031005859375f,0.14179999984502789089901853003539144992828369140625f,1.0257999925136562779925952781923115253448486328125f,-0.45199999260902412601126343361102044582366943359375f,-0.09920001039207004822628732654266059398651123046875f,0.218599997901916476283901147326105274260044097900390625f,-0.11569999844133871391704815323464572429656982421875f,0.380799993035197292812199520994909107685089111328125f,-0.7768000245809556503218118450604379177093505859375f,0.062400000041723302501850412227213382720947265625f,0.907000010043382598468042488093487918376922607421875f,-0.13179999626874927542985460604541003704071044921875f,0.32400002634525293387923738919198513031005859375f,0.7045999742388726616582061978988349437713623046875f,-0.73259999444484691366596962325274944305419921875f,0.10480000351071350195297782192938029766082763671875f,0.9823000096201894848491065204143524169921875f,0.23879999483823777239877017564140260219573974609375f,0.53360004916191172696926514618098735809326171875f,0.58340000528693192194396033301018178462982177734375f,-1.08509997924566192750717164017260074615478515625f,-0.0647999921858311012101694359444081783294677734375f,0.092500012218952321063625277020037174224853515625f,0.15939998475313199577385603333823382854461669921875f,0.261200004267692520443233661353588104248046875f,0.015499983921646975915109578636474907398223876953125f,-0.266999991901218702849973851698450744152069091796875f,0.078300026395172028781388462448376230895519256591796875f,0.00179996020793993238839902915060520172119140625f,0.2636999997973443754517575143836438655853271484375f,-0.0087000160396097925286085228435695171356201171875f,-0.443400005435943711518120835535228252410888671875f,-1.169999991059302857365764793939888477325439453125f,-1.2513999854087831664628538419492542743682861328125f,0.4340999956816438132278790362761355936527252197265625f,0.3138000009477137641766830711276270449161529541015625f,0.859199967662990449213111787685193121433258056640625f,0.84249999836087230420389460050500929355621337890625f,0.454200010168552381628614966757595539093017578125f,0.8658000252962114728916276362724602222442626953125f,-0.545100005471706339221782400272786617279052734375f,-0.817199997740984063199221054674126207828521728515625f,1.12349993371963563504323246888816356658935546875f,0.3669999479055405799954314716160297393798828125f,0.4542000065922735529966303147375583648681640625f,0.705199987101554626178767648525536060333251953125f,0.08939997402429611383922747336328029632568359375f,-0.521100005948543554268326261080801486968994140625f,0.841499984622000596345969825051724910736083984375f,0.107199976134300101193730370141565799713134765625f,0.2082000260829914850546629168093204498291015625f,0.633699984300136520687374286353588104248046875f,0.107699982929229776118518202565610408782958984375f,-0.3191000312209126832385663874447345733642578125f,0.0836999890685081737728978623636066913604736328125f,0.3594000018894671910629767808131873607635498046875f,-0.5943999939918516162151718162931501865386962890625f,0.042599988633394314518909595790319144725799560546875f,0.5978000113487240696485969237983226776123046875f,0.59619998176693922431468308786861598491668701171875f,0.447099992418289016171684124856255948543548583984375f,-0.0388000021398067218569849501363933086395263671875f,-0.011600003355741517196975109982304275035858154296875f,0.0354000023663043617005996566149406135082244873046875f,0.124000041246414838269629399292171001434326171875f,0.55040000132322308701304791611619293689727783203125f,0.5981000235915185925250625587068498134613037109375f,0.02439999622702604398227776982821524143218994140625f,-1.018800030861795047343321130028925836086273193359375f,0.0634000017106532975930122120189480483531951904296875f,0.1118999811768528473976402892731130123138427734375f,-0.0564000114858149714791579754091799259185791015625f,-0.61040000191926946371268059010617434978485107421875f,0.28099999928474428401159457280300557613372802734375f,0.34620000545978513173395185731351375579833984375f,0.062999992042779950907060992904007434844970703125f,-0.6017999870300290155000766390003263950347900390625f,-0.49180000133514401028378415503539144992828369140625f,-0.65740002015829102077759671374224126338958740234375f,-0.16379999930858613055306705064140260219573974609375f,-0.20060000958442714136253925971686840057373046875f,0.14299999487400061326525246840901672840118408203125f,-0.414299989116191369475927785970270633697509765625f,-0.62499998033046733070250411401502788066864013671875f,-0.5611000142931938938772873370908200740814208984375f,-0.150300009262561928835566504858434200286865234375f,0.1322999985933304145646616234444081783294677734375f,0.086799965572357251630819519050419330596923828125f,0.84989994863271789427017210982739925384521484375f,0.091399968719482416190658113919198513031005859375f,-0.28259999794661982974730563000775873661041259765625f,-0.545499994426965617577707234886474907398223876953125f,0.03750000804662700915059758699499070644378662109375f,-0.039199998396635127306808499270118772983551025390625f,0.927599984312056591306827613152563571929931640625f,0.401199975061416580501827411353588104248046875f,-0.127200023818016205723324674181640148162841796875f,-0.3387000105261801508049757103435695171356201171875f,0.49290002062022697515430991188623011112213134765625f,0.92420001996606593319683042864198796451091766357421875f,-0.2999999976158136405501863919198513031005859375f,-0.5857000096917150955277975299395620822906494140625f,0.78569999598264672613368020392954349517822265625f,0.94460000523328790933419440989382565021514892578125f,0.152099958443642435668152756989002227783203125f,0.23459999241828910498952609486877918243408203125f,-0.666299997818470046695438213646411895751953125f,0.0007999788045882638698458322323858737945556640625f,-0.08069998286962487554774270392954349517822265625f,0.30680000640153881619198728003539144992828369140625f,0.488699976849556350089187617413699626922607421875f,0.747200013685226593906918424181640148162841796875f,-0.934499961018563141124104731716215610504150390625f,-0.1770999873518945832984172739088535308837890625f,1.0515000162124632510085575631819665431976318359375f,0.50040000996589650839041496510617434978485107421875f,-1.2208000038385389274253611802123486995697021484375f,-0.10079999787807469857625619624741375446319580078125f,-0.90510000614225827320069583947770297527313232421875f,0.096000007629394479291562447542673908174037933349609375f,0.673300010097026557787103229202330112457275390625f,0.41839998934268951558124172152020037174224853515625f,-0.567900013542175230440989253111183643341064453125f,-0.0835999940276146435991222460870631039142608642578125f,0.36270001988410971449638964259065687656402587890625f,0.268799993199110076602664776146411895751953125f,0.5606000109255315777545547462068498134613037109375f,0.09620001559257496115407093384419567883014678955078125f,0.91140000358819950321276337490417063236236572265625f,-0.08599999243021017036880948580801486968994140625f,-0.4105000163316727679330142564140260219573974609375f,0.1916000000774860101415697499760426580905914306640625f,-0.197300025434047288097616501545417122542858123779296875f,0.41399999929964537148663339394261129200458526611328125f,-1.2472000288099052145440737149328924715518951416015625f,0.255199992614984605410199947073124349117279052734375f,-0.0674999788403507494649602449499070644378662109375f,-0.6384000040948389909090110450051724910736083984375f,0.199699992525577130209057941101491451263427734375f,0.0681999907672405925040948204696178436279296875f,-0.3114999919235703629283307236619293689727783203125f,0.62280000836849203071921010632649995386600494384765625f,0.00169999277144661231631062037195079028606414794921875f,-0.1083999993264674888138188180164434015750885009765625f,-0.446599995160102736235785414464771747589111328125f,1.22520001630783070822872105054557323455810546875f,0.201600020194053541899847914464771747589111328125f,0.7335999841928479980879274080507457256317138671875f,0.3612000398069612483453738605021499097347259521484375f,0.4507000020921230198922557974583469331264495849609375f,0.519599962019920980083043104968965053558349609375f,-0.10170002208948147881528711877763271331787109375f,-0.02459996976852441008531968691386282444000244140625f,0.15950000721216195387341940659098327159881591796875f,0.278999983727931777366393362171947956085205078125f,-0.23010002186298361692706748726777732372283935546875f,0.069300009530782080702238090452738106250762939453125f,-0.087199975836277143770303155179135501384735107421875f,-0.41220000176429749672024627216160297393798828125f,-0.63059998246431359092412094469182193279266357421875f,0.40560001643896104184250361868180334568023681640625f,0.28470000325441358057787510915659368038177490234375f,0.708600015522539816714697735733352601528167724609375f,0.278100008708983603700204412234597839415073394775390625f,0.9138000057160855504889696021564304828643798828125f,1.2842000124782326775374485805514268577098846435546875f,-1.1358000044345857304506353102624416351318359375f,-0.6171000207304953022457993938587605953216552734375f,0.186299966824055474035048973746597766876220703125f,0.281500003397464837462393916212022304534912109375f,1.276200004118680286779863308765925467014312744140625f,0.997900008177757147365127821103669703006744384765625f,-0.15059999893009656801723394892178475856781005859375f,0.01470001230686912574441294054850004613399505615234375f,0.8972999938249586193705908954143524169921875f,0.32879998902678497785245781415142118930816650390625f,-0.57120002430975436169546810560859739780426025390625f,0.004000015616416774388852672927896492183208465576171875f,0.99690001127719884976841058232821524143218994140625f,-0.0670000014007090527456966810859739780426025390625f,1.043000000238418323306177626363933086395263671875f,-0.44679999194741260026830786955542862415313720703125f,-0.0062999958813190204409693251363933086395263671875f,0.10039999983310698983274278361932374536991119384765625f,-0.6127999916791910806068699457682669162750244140625f,-0.51419998260140431511899805627763271331787109375f,-0.87400003767013600963764474727213382720947265625f,0.48880000050067895500660597463138401508331298828125f,1.2974000162839889327415221487171947956085205078125f,0.18979999680519110683007966144941747188568115234375f,-0.546300013054907385168235123273916542530059814453125f,0.5380000041127204202240363883902318775653839111328125f,0.75180000103712085746110460604541003704071044921875f,-0.462399987822771141310340681229718029499053955078125f,0.82959999122619620948171359486877918243408203125f,-0.654599986606836381497487309388816356658935546875f,0.67900000704824936548220648546703159809112548828125f,0.343600005277991282781613335828296840190887451171875f,0.178299990427493781908196979202330112457275390625f,-0.47160000380277633524883640347979962825775146484375f,-0.54070000671148310544822379597462713718414306640625f,0.1645999976336955938904793583787977695465087890625f,-0.94130003603994882865890758694149553775787353515625f,-0.339200004878640104521991816000081598758697509765625f,-0.5004000016212462043085906771011650562286376953125f,0.769199982005357885128660200280137360095977783203125f,0.1056000531703229494695506218704394996166229248046875f,0.1708000224649903753260105077060870826244354248046875f,-0.4209000155687334654430742375552654266357421875f,-0.71449998486041987888484072755090892314910888671875f,-1.184399992203712059790632338263094425201416015625f,-1.30879999573230776377386064268648624420166015625f,0.216599993222951869853432071977294981479644775390625f,0.199099998736381511577064884477294981479644775390625f,-0.7664999536275871605539578013122081756591796875f,-0.1895999995708468333077689749188721179962158203125f,1.280400004005432634812677861191332340240478515625f,1.438800024938583277389625436626374721527099609375f,-0.068700027513504124954124563373625278472900390625f,-0.7055999922990796591193429776467382907867431640625f,1.13760001221448181496498364140279591083526611328125f,0.90810000667870027779571273640613071620464324951171875f,-1.1201999763727190639883701805956661701202392578125f,-1.12279998244047174438264846685342490673065185546875f,-1.51109996690750136849601403810083866119384765625f,-1.42599997365474706612076261080801486968994140625f,0.392399999333917559596329738269560039043426513671875f,-0.15519998158812509103654520004056394100189208984375f,-0.995399993872641886127894395031034946441650390625f,-0.84670001195669186699888086877763271331787109375f,0.5396999935686590532668560626916587352752685546875f,0.73949999797344201368787253159098327159881591796875f,-0.281999983556568079023918471648357808589935302734375f,0.39490001345276848354615140124224126338958740234375f,0.75930001385211909337158431299030780792236328125f,0.030799995195865736974383253254927694797515869140625f,0.7208000127792359990053228102624416351318359375f,-0.15140000001192088774359945091418921947479248046875f,0.151200008887052650408122644876129925251007080078125f,0.1311999959230423229428197373636066913604736328125f,0.451800026965141654500257573090493679046630859375f,-0.5234000020086764237703391700051724910736083984375f,0.0952999978631734467882097305846400558948516845703125f,-0.19279999585151674779126551584340631961822509765625f,0.1877999684333797603130733477883040904998779296875f,-0.005799998176097886926072533242404460906982421875f,0.6545000239014626686184783466160297393798828125f,-0.36240000928044313610598692321218550205230712890625f,-0.0589999831318852585582135361619293689727783203125f,0.4944000051677226448276769588119350373744964599609375f,0.39739997985064956509404510143212974071502685546875f,-0.150000002756714767571111224242486059665679931640625f,-0.2007000299572947454862514859996736049652099609375f,-0.197399997508525881784180455724708735942840576171875f,-0.144300007593631818281210144050419330596923828125f,-0.0959999960064887858379734097979962825775146484375f,-0.123500012695789251893074833787977695465087890625f,-0.695199982780218217470746822073124349117279052734375f,0.02580000056028364241456074523739516735076904296875f,0.2218000001430511236577558520366437733173370361328125f,0.001100002670288091621841886080801486968994140625f,0.7086000049054621552357957625645212829113006591796875f,-0.21820001252293597104880973347462713718414306640625f,-0.01259999712705617280761316578718833625316619873046875f,-0.310099991166591593128032400272786617279052734375f,-0.032400010323524242039638920687139034271240234375f,-0.58530000032186535463551990687847137451171875f,-0.885000001788139201153171597979962825775146484375f,0.11519999455213536521114292554557323455810546875f,-0.175200013178586910811418420053087174892425537109375f,-1.0647000124931338405076530762016773223876953125f,-0.781800010871886996710600215010344982147216796875f,-0.3455999945342540513593121431767940521240234375f,-0.1720999920457602083700976436375640332698822021484375f,-0.196499972999095717796080862171947956085205078125f,-0.4863999776601790614449782879091799259185791015625f,-0.994200017023086246581442537717521190643310546875f,-1.189900038635730794567280099727213382720947265625f,0.99899998939037271128427164512686431407928466796875f,0.78800000202655784420358031638897955417633056640625f,-0.512999986417591369303181636496447026729583740234375f,-0.75019998605847337813656849903054535388946533203125f,-0.4202999663472173352829486248083412647247314453125f,-0.415499987423419980814287555404007434844970703125f,0.00839999947547909187051118351519107818603515625f,0.378700022447109319045921438373625278472900390625f,-1.2185999531984332833189910161308944225311279296875f,-0.6306999739289285145105168339796364307403564453125f,0.39329999653697012007569355773739516735076904296875f,0.211199999797344162288936786353588104248046875f,1.31369999443292595486809659632854163646697998046875f,0.81810000511407832135546414065174758434295654296875f,-0.001499983280897065895942432689480483531951904296875f,0.005600003623962290788540485664270818233489990234375f,0.879599976921081605496510746888816356658935546875f,0.2441999894186854813682430176413618028163909912109375f,-0.78630002464056047983831376768648624420166015625f,-0.33460000970363612537994413287378847599029541015625f,0.9659999748468397484657543827779591083526611328125f,-0.62159997608661665680074293049983680248260498046875f,-0.2119999862015244929125401540659368038177490234375f,0.3708000059247016366725802072323858737945556640625f,0.215199974435567487063281078008003532886505126953125f,-0.301399987643957256722160309436731040477752685546875f,0.2678000037491319318405658123083412647247314453125f,0.2713999900281429678017275364254601299762725830078125f,0.0164000283241279021240188740193843841552734375f,-0.0927999915301800371025819913484156131744384765625f,-0.250600008615851521653894451446831226348876953125f,0.249199998992681470699750434505403973162174224853515625f,0.04200002356618670606991372551419772207736968994140625f,-0.2472000053405760677804892111453227698802947998046875f,1.082299983841180424093408873886801302433013916015625f,-0.0337999881327153417487352271564304828643798828125f,0.18790002069473299428636892116628587245941162109375f,0.12259998818635953199418509029783308506011962890625f,0.264200030732155255464022047817707061767578125f,0.69159999605417254997519194148480892181396484375f,-0.3805000225901604693490298814140260219573974609375f,-0.492399988567829183239155099727213382720947265625f,0.202599993103742548328227712772786617279052734375f,-0.39059999170303350002342313018743880093097686767578125f,-1.03830003125667591490355334826745092868804931640625f,0.25660000588893883133323470246978104114532470703125f,-0.15300000735372309978998828228213824331760406494140625f,0.700699986967444399255100506707094609737396240234375f,-0.429299990010260756889692856930196285247802734375f,-0.002700016009807626460315077565610408782958984375f,-1.537199937987328013377918978221714496612548828125f,-1.6043999469041825278736723703332245349884033203125f,0.86039999111592724290176192880608141422271728515625f,0.8678000050902365813954020268283784389495849609375f,-0.5900999720931050518402116722427308559417724609375f,-0.394799969238043058084031144971959292888641357421875f,0.4304999688714747918538705562241375446319580078125f,0.16159998651742935038555515347979962825775146484375f,-0.343799972784520235791205777786672115325927734375f,-0.57980002064704905251346644945442676544189453125f,0.5160000015944243845211758525692857801914215087890625f,0.3907999992191790727957823037286289036273956298828125f,-0.37260002357661736738236868404783308506011962890625f,-0.04020001388639193873331123540992848575115203857421875f,-0.431099983894824845265247859060764312744140625f,-0.859099966102838674686381637002341449260711669921875f,0.018899986004829116836845059879124164581298828125f,-0.40090002868175478312195991748012602329254150390625f,1.17899997150897917208567378111183643341064453125f,0.561399986004829543162486515939235687255859375f,-0.728999964058399285704581416212022304534912109375f,-0.1294999714493751952204547706060111522674560546875f,-0.779699979561567335650806853664107620716094970703125f,-0.567199986565113167813478867174126207828521728515625f,0.843900025939941400565658113919198513031005859375f,1.12370002274513236528719062334857881069183349609375f,-0.48359999797642228003269337932579219341278076171875f,-0.1480000023245811036076702293939888477325439453125f,0.439800022099912535367138843866996467113494873046875f,0.1718000105738639593511152270366437733173370361328125f,0.360800016951561275391213712282478809356689453125f,0.051600011849403326635865596472285687923431396484375f,0.49130000109672522512482828460633754730224609375f,0.114199995863437653298433360760100185871124267578125f,0.062100018942356172146901371888816356658935546875f,-0.038000000983476667215654742904007434844970703125f,-0.04030002833604839906911365687847137451171875f,-0.00399999326467526117312445421703159809112548828125f,0.90110001798123129379547435746644623577594757080078125f,-0.2632000011980533127342596344533376395702362060546875f,-0.67030000527650113184563451795838773250579833984375f,-0.1365999984756112173300834911060519516468048095703125f,0.9297000127911569933303326251916587352752685546875f,-0.72980000172257408763698549591936171054840087890625f,0.7597000151753423580203161691315472126007080078125f,-0.081600014084577399131603669957257807254791259765625f,0.427700013029575387690783827565610408782958984375f,0.0806000051140784989911480806767940521240234375f,0.0790000120401384009483081172220408916473388671875f,0.2722000068306922315741758211515843868255615234375f,-0.7583000089049338754421114572323858737945556640625f,0.0988000007987022144106958876363933086395263671875f,0.449599999570846620144948246888816356658935546875f,-0.00319998286962519529197379597462713718414306640625f,-0.193900019681453983366736792959272861480712890625f,-0.0330000133514403160006622783839702606201171875f,-0.6613000231504440051821802626363933086395263671875f,-0.527799992275237883632144075818359851837158203125f,0.24689999805241746866357743783737532794475555419921875f,0.08140001260340212763111367166857235133647918701171875f,0.0834000012636177956437677494250237941741943359375f,0.0924000180721282760032408987171947956085205078125f,0.0710999801695351152375224046409130096435546875f,-0.1667000030159950796360135427676141262054443359375f,0.01679997630119345330967917107045650482177734375f,0.209299983155727176153959590010344982147216796875f,-0.179999958872794962871921597979962825775146484375f,-0.490099990570545429591220454312860965728759765625f,-0.77280001455247404607007410959340631961822509765625f,-0.92590000405758610124706819988205097615718841552734375f,0.0237000078998506058525208572973497211933135986328125f,0.48530000009834761698357397108338773250579833984375f,-1.712699994254112567659831256605684757232666015625f,-1.6664999962449069670356038841418921947479248046875f,-0.2502000544548028670988060184754431247711181640625f,-0.22750000685453386495282757095992565155029296875f,0.16829998118877398383119725622236728668212890625f,0.224500013172626466939618694595992565155029296875f,0.57929999179840052647705306299030780792236328125f,0.5356000089883803383372651296667754650115966796875f,0.40740001389980307067162357270717620849609375f,0.5946999972939490675116758211515843868255615234375f,-0.034799984884262613604732905514538288116455078125f,-0.00039996228218086571359890513122081756591796875f,-0.125699999856948352316976524889469146728515625f,-0.252700008857249969196345773525536060333251953125f,-0.42869994943141964682808975339867174625396728515625f,-0.2850000001490116119384765625f,-0.3288999828755851950745636713691055774688720703125f,0.20440001097917548378291030530817806720733642578125f,0.445900009161233956689329716027714312076568603515625f,0.02620000203251837689322201185859739780426025390625f,-0.0289999893903729599742291611619293689727783203125f,0.08640000984668727568305257591418921947479248046875f,-1.094500020176172316865859102108515799045562744140625f,0.076399990326166289378306828439235687255859375f,0.72630003715753588267034501768648624420166015625f,0.23039999625682838768625515513122081756591796875f,-0.3182000093936920848136651329696178436279296875f,-0.40719999908208848182766814716160297393798828125f,1.05770002226829529234919391456060111522674560546875f,0.05679999895095821926815915503539144992828369140625f,0.190600022697449134057023911736905574798583984375f,-0.07719998969435692970364470966160297393798828125f,0.591899998462199761206647963263094425201416015625f,-0.0734000102043150803865501075051724910736083984375f,-0.86960003920793571552394496393389999866485595703125f,0.02820000015497203804670789395458996295928955078125f,-0.94530002237558363731295685283839702606201171875f,0.745400008177757200655833003111183643341064453125f,-0.03070002995729481654052506200969219207763671875f,0.6134000055849551102937766700051724910736083984375f,-0.59199997234344436947139911353588104248046875f,0.42839999843239784382831203402020037174224853515625f,-0.80450001585483565946788075962103903293609619140625f,0.03719999431371689979641814716160297393798828125f,0.5698000373363498738399357534945011138916015625f,-0.003300000560283677941697533242404460906982421875f,0.62519999954402383224305594922043383121490478515625f,0.8571000079154966755368150188587605953216552734375f,-0.083100006306171536607507732696831226348876953125f,-0.251899997717142110076338212820701301097869873046875f,0.27840000334978043383671320043504238128662109375f,0.4218000168323516874124834430404007434844970703125f,0.53040000072717674584055203013122081756591796875f,0.962300005298853289303906421991996467113494873046875f,0.30239995787143669758734176866710186004638671875f,0.05489999214410801187113975174725055694580078125f,-0.669599983775615470449338317848742008209228515625f,-0.29260000398159036194556392729282379150390625f,-0.270000022053718424785984097979962825775146484375f,-0.77810001494884506456628514570184051990509033203125f,-0.04080000503063274663873016834259033203125f,0.229600015664100709500416996888816356658935546875f,-0.18510004034042282228256226517260074615478515625f,-0.110899995005130591607667156495153903961181640625f,0.278100028657913611596086411736905574798583984375f,0.49700001317262643141248190659098327159881591796875f,1.7475000178813928641829988919198513031005859375f,1.69790003470182426781320828013122081756591796875f,1.0569000202178955305498675443232059478759765625f,0.47990002671480169738060794770717620849609375f,1.07039997807741116275792592205107212066650390625f,1.149799980115890463139294297434389591217041015625f,0.29820000581741368250732193700969219207763671875f,0.52889999017715449980414632591418921947479248046875f,0.82919996963739350803734851069748401641845703125f,-0.1217999990999698578519883085391484200954437255859375f,-0.7731000060081480995677338796667754650115966796875f,-0.0683999961972237002783003845252096652984619140625f,-0.522200008171796792311170065659098327159881591796875f,0.3159999940693378750466990823042578995227813720703125f,0.9191999973535533996482627117075026035308837890625f,0.138600006246566653800300628063268959522247314453125f,-0.0941999866247176242950445157475769519805908203125f,0.004199999737739545935255591757595539093017578125f,0.076399989245831800843689052271656692028045654296875f,-0.5314000006079673266157215039129368960857391357421875f,-0.0859999724626536732330350787378847599029541015625f,-0.6379999969601630738225139793939888477325439453125f,0.05300001722574254614528399542905390262603759765625f,0.58479998607635508278690394945442676544189453125f,-0.46180001385211966180577292107045650482177734375f,0.14259999340176587878659120178781449794769287109375f,0.4691000054419038889363946509547531604766845703125f,-0.02160000469684597579345108897541649639606475830078125f,0.4103999996095895053116464623599313199520111083984375f,-0.287799997043609590008372833835892379283905029296875f,0.2253999959588046664293869980610907077789306640625f,0.3041999890089035485374324707663618028163909912109375f,0.88830002708435085168048317427746951580047607421875f,-0.46659999456405643059042631648480892181396484375f,0.4984000046163796771025999987614341080188751220703125f,0.134599996888637551961664939881302416324615478515625f,0.10000001341104525209857456502504646778106689453125f,-0.0055999927461147791518669691868126392364501953125f,-0.05280000091791148264519506483338773250579833984375f,0.18419999374002216097068185263196937739849090576171875f,0.1626000145614145964145791367627680301666259765625f,0.5795000006556509930533138685859739780426025390625f,0.66540000340938565415171979111619293689727783203125f,0.488399991875887007353185254032723605632781982421875f,0.0128999891042713699107480351813137531280517578125f,-0.345799986553192439941994962282478809356689453125f,0.1256999843597412791496026329696178436279296875f,0.3139999893903731020827763131819665431976318359375f,-1.03109998758286192099831168889068067073822021484375f,-0.72639999934136867398848380616982467472553253173828125f,-1.03380000467300448008245439268648624420166015625f,-0.3396000132799148474305184208787977695465087890625f,0.94199997353553754919630591757595539093017578125f,0.29099997276067757212558717583306133747100830078125f,0.467099990034103740299542550928890705108642578125f,0.640600003623962521714929607696831226348876953125f,-0.123600024574994638015823511523194611072540283203125f,-0.169500002741813649009827713598497211933135986328125f,-0.5727000033438205406355336890555918216705322265625f,-0.3645999832540749441278649101150222122669219970703125f,0.57809996100664218232623170479200780391693115234375f,0.43199999499320984597261485760100185871124267578125f,-1.17930000014305136346592917107045650482177734375f,-1.24450001347064986845225575962103903293609619140625f,0.03029999056160459502962112310342490673065185546875f,-0.3596000051587819879017615676275454461574554443359375f,0.0677999952554699802931281737983226776123046875f,0.22860000654458989544082214706577360630035400390625f,-0.518099986338615536851648357696831226348876953125f,-0.0466000033557414372609173369710333645343780517578125f,0.47480001736879362539411886245943605899810791015625f,-0.2068000002920626911873114295303821563720703125f,0.187300003659725433635685476474463939666748046875f,-0.187799988400936257448847754858434200286865234375f,0.48600001109391455234032264343113638460636138916015625f,-0.236000000178813895690410618044552393257617950439453125f,-0.132499998956918663139958880492486059665679931640625f,0.33680000208020199892189339152537286281585693359375f,-0.76460003358274730711485744905075989663600921630859375f,0.0658000017523765468041574422386474907398223876953125f,-0.039199997502565420148812336265109479427337646484375f,-0.12339999873042106770526515902020037174224853515625f,0.327900000578164707576434011571109294891357421875f,0.292399986034631798048621931229718029499053955078125f,-0.01480000813007364968143519945442676544189453125f,0.0674000009357929030784362112171947956085205078125f,0.09089997980594599624737384147010743618011474609375f,0.13399999043345456328779619070701301097869873046875f,0.4050000029802323098238048260100185871124267578125f,-0.246599992477893881215322835487313568592071533203125f,-0.2519999999105930754694782081060111522674560546875f,0.31499999374151232967022906450438313186168670654296875f,0.732999982953071338442896376363933086395263671875f,-0.4331999834656716785730168339796364307403564453125f,0.2778000145524741615332686706096865236759185791015625f,0.228799987983703712046690270653925836086273193359375f,-0.21459999659061423926686984486877918243408203125f,-0.09220000057220456568529698415659368038177490234375f,1.3047000133872028726500502671115100383758544921875f,-0.86969999002963283007971995175466872751712799072265625f,-0.416400019979477065135142765939235687255859375f,-0.24219999550580961766854670713655650615692138671875f,-0.583499960839748865737419691868126392364501953125f,-0.833900005376339237272986792959272861480712890625f,-0.5358000117361545466820871297386474907398223876953125f,-0.45459999878853551980029124024440534412860870361328125f,0.531000011503696356385262333787977695465087890625f,0.105499984890222275879523294861428439617156982421875f,0.42390001737177396279321328620426356792449951171875f,0.77060002511888747722679227081243880093097686767578125f,-0.10830000860691058051088475622236728668212890625f,0.184000003844499548932844845694489777088165283203125f,0.45539996071755961537519397097639739513397216796875f,0.1580999851465225702895622816868126392364501953125f,0.107100005829334321560963871888816356658935546875f,0.0256000054121017228681012056767940521240234375f,0.080400001025200396043146611191332340240478515625f,-0.29390000388622272708971649990417063236236572265625f,-0.6152999848246576419796838308684527873992919921875f,-0.64759999325275430237525142729282379150390625f,0.671099985235929619165062831598334014415740966796875f,0.558199992406368306063768613967113196849822998046875f,-1.04339996609687801765176118351519107818603515625f,-1.029799967598915344524357351474463939666748046875f,0.2051999812901019737410024390555918216705322265625f,-0.2910000251382587332926732415216974914073944091796875f,-0.051300003182888076480594463646411895751953125f,0.185199998426437417720080702565610408782958984375f,-0.361800054383277114311567856930196285247802734375f,-0.6915999905407428371972855529747903347015380859375f,-0.040099994891881873826378068770281970500946044921875f,0.51520000334382043405412332504056394100189208984375f,-0.18430001027584097528233542107045650482177734375f,0.3865999999284743804395247934735380113124847412109375f,-0.031099996411800390205826261080801486968994140625f,0.546200000691413833919796161353588104248046875f,0.55620001797676099641876135137863457202911376953125f,-0.0002000000655650868708335110568441450595855712890625f,0.57990000875890235221277180244214832782745361328125f,0.271999996185302739259981308350688777863979339599609375f,-0.9002999931693071999916355707682669162750244140625f,-0.1451999975323676750349477515555918216705322265625f,0.3692000112116335941436773282475769519805908203125f,0.25519999395608905512489172906498424708843231201171875f,-0.8503999885082240695055588730610907077789306640625f,0.56340000007152557515155422152020037174224853515625f,0.19189999876022323377355860429815948009490966796875f,0.39099999690055842194169599679298698902130126953125f,1.273500018656253729432137333787977695465087890625f,0.16239999691247941626670581172220408916473388671875f,0.2624000162839887906329749966971576213836669921875f,0.61179997138381025933995260857045650482177734375f,-0.84330004198551211658241300028748810291290283203125f,0.2664000086545943446481032879091799259185791015625f,0.0191000204920772631567160715349018573760986328125f,-0.5320000012516974408072201185859739780426025390625f,-0.361900003677606552088263924815692007541656494140625f,-0.17099999615550043330358676030300557613372802734375f,0.945700032937526913201509159989655017852783203125f,-0.55509997745752315267964149825274944305419921875f,0.832200007426738608273808495141565799713134765625f,0.246699983495473507133510793210007250308990478515625f,0.81449999320507071587371683563105762004852294921875f,0.68290000728368749349783684010617434978485107421875f,1.05269998710155476828731480054557323455810546875f,0.735700002539157793535196105949580669403076171875f,-0.13950001674890533109874013462103903293609619140625f,-0.282600002491474100452251150272786617279052734375f,0.4139999659210447102708485545008443295955657958984375f,0.1713000003516673697134820031351409852504730224609375f,-0.03299997520446851240194519050419330596923828125f,0.3198999909520150453090536757372319698333740234375f,0.425999967396259560103999319835565984249114990234375f,0.232399993932247195260742955724708735942840576171875f,1.326899998164176253112600534223020076751708984375f,1.2749999928474426980073985760100185871124267578125f,0.582299992036819702434513601474463939666748046875f,0.09189998981952651746496485429815948009490966796875f,-0.112800030422210539882144075818359851837158203125f,-0.7231000033259391557294293306767940521240234375f,1.024499931216240611320245079696178436279296875f,0.4507999627113346008400185382924973964691162109375f,0.8100000321865081787109375f,1.02690004048347471865554325631819665431976318359375f,-0.950400002217293149442411959171295166015625f,-0.5111000101208684753828492830507457256317138671875f,-0.137699991571903268550158827565610408782958984375f,0.232300005298852862978264965931884944438934326171875f,0.1319999693632123438646885915659368038177490234375f,0.007200002956390338937353590154089033603668212890625f,-0.22060001368224657625205509248189628124237060546875f,0.439399992203712452809583055568509735167026519775390625f,-1.38499999910593007967918310896493494510650634765625f,0.1543999886274338706471098703332245349884033203125f,0.98229998756647063373748096637427806854248046875f,0.07779999749064447911450770334340631961822509765625f,-0.308799984593689291756390957743860781192779541015625f,0.36739998320341127335808550924411974847316741943359375f,-0.319700011599064026768246549181640148162841796875f,0.008599989408254682388133005588315427303314208984375f,-0.03199999503046269122563671771786175668239593505859375f,-0.2735999892592431148585774280945770442485809326171875f,-0.97870002006292367013884359039366245269775390625f,-0.22660002347230889796492192544974386692047119140625f,0.0768000066995622177046243450604379177093505859375f,-0.61580000428557379876792765571735799312591552734375f,-0.84480002213716520742536886245943605899810791015625f,-0.05400000444054597892318270169198513031005859375f,0.863900015658140230101480483426712453365325927734375f,-0.372199998486042016310193503159098327159881591796875f,0.844000010848045434386222041212022304534912109375f,-0.2601999996185302421736196265555918216705322265625f,-0.431100001031160307007894516573287546634674072265625f,0.209999991059303336982111432007513940334320068359375f,0.30479999531507484533676688442938029766082763671875f,-0.1755999945342541224135857191868126392364501953125f,0.115100020875782027562905796003178693354129791259765625f,-0.33659999314844613049757526823668740689754486083984375f,0.3744000154495239485186175443232059478759765625f,-0.4544999911189080421536345966160297393798828125f,0.8804999831765896178836783292354084551334381103515625f,0.6487999943912028388837143211276270449161529541015625f,0.7067999965667723927253973670303821563720703125f,1.08659997221231474640035230549983680248260498046875f,-0.152100015664100141066228388808667659759521484375f,-0.549099979364871870757269789464771747589111328125f,0.75749996516853579375805338713689707219600677490234375f,0.57839997093975570319202006430714391171932220458984375f,-0.0122999979972835404851139173842966556549072265625f,-0.1275000181794165854398670489899814128875732421875f,-0.38219999215304856221564477891661226749420166015625f,0.02460000676065654534596660596434958279132843017578125f,-0.189300010275839980522505356930196285247802734375f,0.196499972403049838476363220252096652984619140625f,-1.066500007271766747862784541212022304534912109375f,-0.4883999947071071545678933034650981426239013671875f,0.1752000115394594104145653545856475830078125f,0.54380000556707397407762982766143977642059326171875f,-0.9669000074028968327866095933131873607635498046875f,-1.036500004738569291617977796704508364200592041015625f,-0.828899999117851393748424015939235687255859375f,-1.103800001692771814987281686626374721527099609375f,0.4313999964356423078015723149292171001434326171875f,0.279099981003999797479764310992322862148284912109375f,-0.44279998750686644370944122783839702606201171875f,-0.47099999123811731038813377381302416324615478515625f,-0.554100003616512015724993034382350742816925048828125f,-0.173500006344169566130375415013986639678478240966796875f,0.3326999730944635302876122295856475830078125f,0.418800009068846623705439924378879368305206298828125f,-0.459099991174042099117968973587267100811004638671875f,0.0849999824166299544625502448980114422738552093505859375f,-0.1415000084638595723163234652020037174224853515625f,-0.264599999570846566854243064881302416324615478515625f,0.02580000056028364241456074523739516735076904296875f,0.350599992001056737223052550689317286014556884765625f,-0.87560002992451213543745325296185910701751708984375f,0.025999999880790658490781197542673908174037933349609375f,-0.6609999974966047631141918827779591083526611328125f,0.58320001401305177068934426642954349517822265625f,-1.53370007132291874540896969847381114959716796875f,-0.11180000416636470816911241854541003704071044921875f,-0.2812000166356565244996090768836438655853271484375f,-0.26060000094175339580004902018117718398571014404296875f,-0.663100025007128834886316326446831226348876953125f,-0.236199996891617747252212211606092751026153564453125f,0.82000002555549134086732010473497211933135986328125f,0.140999984532594824226947594070225022733211517333984375f,0.345700007791817309765747268102131783962249755859375f,0.1846000034824013535939002395025454461574554443359375f,-0.241900011575221896009679767303168773651123046875f,0.03260001381635646566792274825274944305419921875f,-0.1215999865174293148584183654747903347015380859375f,0.6725999831199647172041977682965807616710662841796875f,0.94960001391321424701885689501068554818630218505859375f,-0.2481999868929387620397619684808887541294097900390625f,-0.3681999934494495363423993694595992565155029296875f,0.69319998927712445091486870296648703515529632568359375f,-0.41979997922182032965565667836926877498626708984375f,-0.28920000256896027934772064327262341976165771484375f,0.24509997689872997650439856442972086369991302490234375f,0.25549996872246294454811277319095097482204437255859375f,-0.545099979394674249988383962772786617279052734375f,-0.482700005280971566890002577565610408782958984375f,0.5864999864101410054217922152020037174224853515625f,0.23359999402761477682588520110584795475006103515625f,0.2214000304847958400245033772080205380916595458984375f,0.3475999999582766175620918147615157067775726318359375f,1.345800013375281878325040452182292938232421875f,1.2836000038623804897497393540106713771820068359375f,0.207900042748452307250772719271481037139892578125f,0.736900016343593478040929767303168773651123046875f,-0.464699981349706359168294511619023978710174560546875f,-0.301800002229213770732485500047914683818817138671875f,-0.174299976710975190741237383917905390262603759765625f,0.334700023370981369907894986681640148162841796875f,0.1226999362140899219042466938844881951808929443359375f,-0.0696000178992746132422553273499943315982818603515625f,0.2204999989159401874161403611651621758937835693359375f,-0.060099993476271595937987513025291264057159423828125f,-0.07889994666576427562176831997931003570556640625f,-0.393100000941753791039445786736905574798583984375f,1.26629995728731170601122357766143977642059326171875f,0.80739997575283073416585466475225985050201416015625f,-0.77249999053776274848814864526502788066864013671875f,-0.97579999672323503023818602741812355816364288330078125f,1.412399969345331296466383719234727323055267333984375f,1.084399984157085494729244601330719888210296630859375f,0.2789999971389764965579161071218550205230712890625f,-0.35679998837113391374487036955542862415313720703125f,-0.88680002487897890972590175806544721126556396484375f,0.46020000736713395639299051254056394100189208984375f,-1.2134000108003615281404563575051724910736083984375f,-0.1230000038146972229924358543939888477325439453125f,0.0509000200390818946516446885652840137481689453125f,0.12460000314712527114124895888380706310272216796875f,-0.5855000149905682516049409969127736985683441162109375f,-0.0823999996691942016013854299671947956085205078125f,0.26780000807046899780061721685342490673065185546875f,-0.119799996060133029374128454946912825107574462890625f,0.030000004768371724139797152020037174224853515625f,-0.23299998161196722179511198191903531551361083984375f,-0.26839999083280563496600734652020037174224853515625f,0.333999986857175912291495478712022304534912109375f,-0.082000006020069182710585664608515799045562744140625f,0.21000000387430184378700914749060757458209991455078125f,-0.631700023430585844153029029257595539093017578125f,-0.42019999321103096434626422706060111522674560546875f,0.7312000322818761333110160194337368011474609375f,-0.57739999706149092162377201020717620849609375f,0.97079998625516861210371644119732081890106201171875f,-0.3381999903202057566886651329696178436279296875f,-0.13650000399351114310775301419198513031005859375f,-0.47439999741911886843581669381819665431976318359375f,-0.544799980525672022935168570256792008876800537109375f,0.44879999066591269585302370614954270422458648681640625f,0.66500001847744005800677769002504646778106689453125f,-0.1031999946415425029044854454696178436279296875f,0.1671000135779383555245658499188721179962158203125f,-1.1910000094175341445179583388380706310272216796875f,1.136100022935867315254654386080801486968994140625f,1.15060001405477496660978431464172899723052978515625f,0.16530001373291014488131622783839702606201171875f,0.7227999904870987535332460538484156131744384765625f,0.3362999853014940754292183555662631988525390625f,0.4449000058531762391567099257372319698333740234375f,0.096000008486211374503227489185519516468048095703125f,0.4855000111907721294102202591602690517902374267578125f,-0.3440999939680100538907936424948275089263916015625f,-0.42660000559091582061910230549983680248260498046875f,-0.270299983930588183511645183898508548736572265625f,0.18440002558231327611792949028313159942626953125f,0.716399968421459476530799292959272861480712890625f,0.60499997705221186805601973901502788066864013671875f,0.399000007420778235456282345694489777088165283203125f,0.2540000056326388033767216256819665431976318359375f,-0.61529995777905011067332452512346208095550537109375f,-0.148499981403350744812996708787977695465087890625f,-1.294500001549721446281182579696178436279296875f,-1.1516000046968457581897382624447345733642578125f,-0.9566999818563459712095209397375583648681640625f,-1.12379997218847282880460625165142118930816650390625f,-0.84659998234510425163534819148480892181396484375f,-0.481699998843669874304396216757595539093017578125f,0.97019998199790669080044835936860181391239166259765625f,0.79979997706115268929494277472258545458316802978515625f,-0.4649999977648260340856722905300557613372802734375f,-0.75400000995397586933677303022705018520355224609375f,-0.220499994277954414201303734444081783294677734375f,0.3149999865889550054731671480112709105014801025390625f,-1.2170000347048046673847920828848145902156829833984375f,0.200599996322393447911736075184307992458343505859375f,0.57149998600035878393299526578630320727825164794921875f,-0.1893999875843526137231265238369815051555633544921875f,-0.3774000143468381196498739882372319698333740234375f,0.23980000246763222282453398293000645935535430908203125f,-0.45850002908706688486972780083306133747100830078125f,-0.5017999926924705533792803180404007434844970703125f,0.0365000004172326697471362422220408916473388671875f,0.0657999892354013127260259352624416351318359375f,-1.24760000994205455526753212325274944305419921875f,0.05439999741911893949009026982821524143218994140625f,0.34560002031326320093285175971686840057373046875f,0.608400001412630064834274890017695724964141845703125f,0.15569998853206623579126244294457137584686279296875f,0.52160000275969498062522688996978104114532470703125f,0.0996000273615125220061372601776383817195892333984375f,0.350199987697601411440473384573124349117279052734375f,0.51760002037286767517798580229282379150390625f,0.117599997276067735896276644780300557613372802734375f,0.686400019383430759489783667959272861480712890625f,0.39040000489950177353648541611619293689727783203125f,-0.076900000883638686133281225920654833316802978515625f,-0.16139999319463971261257029254920780658721923828125f,-0.4218000001430510792488348670303821563720703125f,0.29500000461935993456563664949499070644378662109375f,0.89370002494305389451056953475927002727985382080078125f,-0.2272000086188314948998367981403134763240814208984375f,-0.4788000493764883458425174467265605926513671875f,0.3162999826192856023254762476426549255847930908203125f,0.212799999576807064016747972345910966396331787109375f,-0.39619998817443846661490169935859739780426025390625f,0.2690000004917383336078273714520037174224853515625f,-0.342499993294477445004986293497495353221893310546875f,0.037599999362230296373610372029361315071582794189453125f,0.11100000308454038844274691655300557613372802734375f,-0.1364999962449073844794611432007513940334320068359375f,0.2520999991238117576841659683850593864917755126953125f,0.1442999999940395294828476835391484200954437255859375f,0.434700003105402021930103728664107620716094970703125f,0.08679999835491181148228179154102690517902374267578125f,0.254400001889467208826545174815692007541656494140625f,-0.08020000274777405735449065105058252811431884765625f,0.4896000192403793249695809208787977695465087890625f,0.00770000282228000543227608432061970233917236328125f,-0.54119997848868361955965156084857881069183349609375f,-0.11000000499188899993896484375f,0.1194999923110007244986263685859739780426025390625f,-0.0389999973624944595851360418237163685262203216552734375f,0.013300001268088834649461205117404460906982421875f,0.00879999524801977628385429852642118930816650390625f,-0.6693999778613448636832572447019629180431365966796875f,-0.19910000670850269699485579621978104114532470703125f,-0.752399986481666616100483224727213382720947265625f,-0.00609999700784680243259572307579219341278076171875f,-0.22449998992681496901013815659098327159881591796875f,-0.0126999996744096288470160516226314939558506011962890625f,0.0052999997444450885897282432779320515692234039306640625f,0.070900007373094542373337390017695724964141845703125f,-0.70180001118853707620104387387982569634914398193359375f,-0.0825000000931319643626693505211733281612396240234375f,0.44230002562031189849989232243387959897518157958984375f,-0.4222999562740330503629593295045197010040283203125f,0.09270003895759604262138964259065687656402587890625f,0.122600004875659696068623816245235502719879150390625f,-0.7403000140309334842214639138546772301197052001953125f,0.81419999139308885105492663569748401641845703125f,-0.02660000350475311137188327847979962825775146484375f,-0.26859999395310907299716518537024967372417449951171875f,-0.2536999939486384025855159052298404276371002197265625f,-0.86069996320009256596250679649529047310352325439453125f,-0.396499981604516438249419252315419726073741912841796875f,-0.58839999917745533508650623844005167484283447265625f,0.486500029623508911669205190264619886875152587890625f,0.43639999747872337820808752439916133880615234375f,-0.4175000134110451455171642010100185871124267578125f,-0.453799965035916130773330223746597766876220703125f,-1.2927000000655652911518700420856475830078125f,-0.40779998601675071512318027089349925518035888671875f,0.351299991858005089540029075578786432743072509765625f,0.12730000455379464341376660740934312343597412109375f,-1.025300008070468837928501670830883085727691650390625f,-0.2898000015735622270085514173842966556549072265625f,-1.24900005629658750194721505977213382720947265625f,-0.1605000094771380503289037733338773250579833984375f,0.80099997514486265259847641573287546634674072265625f,0.3752999866127968431328554288484156131744384765625f,-0.4016000063359737026758011779747903347015380859375f,-0.278599983894824987373795011080801486968994140625f,0.07319999868348237015425183926708996295928955078125f,0.139300002173334380728420001105405390262603759765625f,-0.2436999945260584221617961020456277765333652496337890625f,0.332699994254112230152031770558096468448638916015625f,-0.27070000715851794126365348347462713718414306640625f,0.02499999605119229695304738925187848508358001708984375f,-0.16529999600052835972974207834340631961822509765625f,-0.038200008797645512714780124952085316181182861328125f,-0.867399988567829183239155099727213382720947265625f,0.029799997103214292959449949194095097482204437255859375f,0.259500006988644560390611104594427160918712615966796875f,0.31349999779462811577701586429611779749393463134765625f,0.3119000015541910642014045151881873607635498046875f,0.1616000072300434187155815379810519516468048095703125f,0.29720000429749493520148462266661226749420166015625f,0.001599996128678338180861828732304275035858154296875f,-0.4958999957501888378175181060214526951313018798828125f,-0.0178000053137541414116640225984156131744384765625f,0.6364999798536299380202763131819665431976318359375f,0.0345999970376491550183573053800500929355621337890625f,0.38260000115036962853309887577779591083526611328125f,0.0150999965310096850856780292815528810024261474609375f,-0.13429999760985378287614366854541003704071044921875f,-0.00859999454915527650200601783581078052520751953125f,0.42069999821782122495505973347462713718414306640625f,-0.111099992239475309219187693088315427303314208984375f,0.4040999889016152479825905174948275089263916015625f,-0.302099996814131799283131840638816356658935546875f,0.27079997925162313521241230773739516735076904296875f,0.2133999988049268825562876372714526951313018798828125f,-1.09250002607703233792335595353506505489349365234375f,0.24859999134540533560766562004573643207550048828125f,0.23839999545216539189596005599014461040496826171875f,-0.30439997432231979956895884242840111255645751953125f,-1.306599998587369970692861897987313568592071533203125f,0.3952999825894834007300460143596865236759185791015625f,-0.28669999936521028960356716197566129267215728759765625f,0.063399992173910124648728015017695724964141845703125f,-0.065600005859136611974236075184307992458343505859375f,-0.4191999973535533996482627117075026035308837890625f,-0.9182000421762470665498767630197107791900634765625f,0.0727000106453896677294324035756289958953857421875f,-0.4192000080823898855442166677676141262054443359375f,-0.519400002671777993867863187915645539760589599609375f,0.6177000250622632915309395684744231402873992919921875f,0.6332999755263326591148143052123486995697021484375f,-0.10199998962879153197036430356092751026153564453125f,-0.72459998824596372202222482883371412754058837890625f,0.044500009149313068945730265113525092601776123046875f,-0.469199950265885235012319753877818584442138671875f,-0.2207999774634838985321039217524230480194091796875f,0.22120001767873720410761961829848587512969970703125f,1.149900005108117628793706899159587919712066650390625f,-0.03520000259876265857883481658063828945159912109375f,0.682700016158819256162360034068115055561065673828125f,-0.756399967825412744559798738919198513031005859375f,0.03490003252625495377969855326227843761444091796875f,0.29529999376535442934255115687847137451171875f,1.24559998320937115323658872512169182300567626953125f,-0.66270000408887863585505328956060111522674560546875f,-0.02779999585151671226412872783839702606201171875f,-0.31579999959170823586873666499741375446319580078125f,0.419899980968236885559008442214690148830413818359375f,0.0083999926954507753151801807689480483531951904296875f,0.40169999422430979763021241524256765842437744140625f,-0.127299999040365208191616375188459642231464385986328125f,-0.1586999988287687290977601151098497211933135986328125f,-0.061800000217556938508778330287896096706390380859375f,0.070300004196166998582384621840901672840118408203125f,-0.002399993932247213024311349727213382720947265625f,0.54239999393224724855144813773222267627716064453125f,0.077300000828504522587536484934389591217041015625f,-0.2421000094056129370301277958787977695465087890625f,0.062200003515183954760914275539107620716094970703125f,0.437500008307397381113190704127191565930843353271484375f,-0.344900001233816144718957730219699442386627197265625f,0.05129999305009835808277784963138401508331298828125f,0.1682000051468610646310253287083469331264495849609375f,0.463500000774860421159928591805510222911834716796875f,0.34199998955428601021822032635100185871124267578125f,-0.22170000078082097161313868127763271331787109375f,-0.0128000042706728489605438880971632897853851318359375f,-0.286300017488002822574344463646411895751953125f,-0.0396000011354684655628943801275454461574554443359375f,0.098900002098083472645839719916693866252899169921875f,0.355999999955296519971170710050500929355621337890625f,0.07110000207424160834079884807579219341278076171875f,0.054700001391768449110486471909098327159881591796875f,-0.53259999787211409483234092476777732372283935546875f,0.03510000092685228167965760803781449794769287109375f,-0.19220001040697098915188689716160297393798828125f,-0.212799994063377795328051433898508548736572265625f,0.95479999978542284821969587937928736209869384765625f,-0.00090003531277116355369116718065924942493438720703125f,0.3878999810203909159866952904849313199520111083984375f,-0.158499978423119358694748370908200740814208984375f,-1.08230000425577177480818136245943605899810791015625f,0.112699978458881577836336873588152229785919189453125f,0.462899995809793463052983497618697583675384521484375f,0.13889999315738688068222472793422639369964599609375f,-0.702000019133091068823659952613525092601776123046875f,-0.1932000041782850896510126403882168233394622802734375f,-0.463300022390484844692082333494909107685089111328125f,0.2185999981999398045218185870908200740814208984375f,-0.45300000858306876949654906638897955417633056640625f,-0.060300018501281638094724257825873792171478271484375f,-0.061600000971555601836371351964771747589111328125f,-0.1187000254087149375692433750373311340808868408203125f,-0.55430000191256378894166800819220952689647674560546875f,-0.77639999859631014800243065110407769680023193359375f,0.485300035414100161546002709656022489070892333984375f,0.6267000077843665195587163907475769519805908203125f,0.99000000327825521395652685896493494510650634765625f,-0.44029996426105544315987572190351784229278564453125f,-0.146499968081712506062785905669443309307098388671875f,-0.8748999994456763129591081451508216559886932373046875f,-0.682900021514296629021600892883725464344024658203125f,-0.046499998219311056146807459299452602863311767578125f,0.1913000122353436527333769845427013933658599853515625f,-0.765199981737136880610705702565610408782958984375f,0.77140001580715189533066222793422639369964599609375f,-0.089600002551078805623774314881302416324615478515625f,0.4087999957323074085024927626363933086395263671875f,-0.257499990686774260506552991500939242541790008544921875f,0.5721000017315149843710742061375640332698822021484375f,0.028899991518259060541140570421703159809112548828125f,0.17250000044703472923401932348497211933135986328125f,-0.155899997091293318618454577517695724964141845703125f,0.2023000047028065129239848829456605017185211181640625f,-0.1496999975919723890882551131653599441051483154296875f,0.213899990177154553094851507921703159809112548828125f,0.19840000342428687485352156727458350360393524169921875f,-0.20509998297095288766200837926589883863925933837890625f,0.115799998027086292751164364744909107685089111328125f,-0.499999988377094251035259730997495353221893310546875f,-0.1584999976456165082794314002967439591884613037109375f,0.4369000059127807755743333473219536244869232177734375f,-0.192499998081475443001409075804986059665679931640625f,0.607300014109164454367828511749394237995147705078125f,0.0696000081390142355530770146287977695465087890625f,0.72879999819099905433716912739328108727931976318359375f,-0.068700000989437182141728044371120631694793701171875f,-0.62670002536773683488036112976260483264923095703125f,0.2874999942630529492504365407512523233890533447265625f,-0.1344000002503396018482817453332245349884033203125f,0.14770000662207605213183114756247960031032562255859375f,0.72230001409053812277960560095380060374736785888671875f,0.0881000014260411301592057498055510222911834716796875f,-0.156499998666346062936582939073559828102588653564453125f,0.18200000058114529366548595135100185871124267578125f,-0.09729999009966849854436077293939888477325439453125f,0.12319996074140104891370128825656138360500335693359375f,0.3149999834224581629626982248737476766109466552734375f,0.2760999910473831420176793471910059452056884765625f,-0.54869999964833215955195555579848587512969970703125f,0.8753999947667114867044801940210163593292236328125f,-0.0583000130772592939365495112724602222442626953125f,-0.2128999913394451493076076076249592006206512451171875f,-0.122100000390410468043000946636311709880828857421875f,0.453399980998039353607964585535228252410888671875f,1.1469999739825720297403677250258624553680419921875f,-1.1203999837398530559084974811412394046783447265625f,-0.0651999698162075702612128225155174732208251953125f,0.1571999966979031881919581792317330837249755859375f,-1.140100008302926681835742783732712268829345703125f,-0.65170000633150326763143311836756765842437744140625f,-0.44280000808909536491597691565402783453464508056640625f,0.81339997563362143040421869955025613307952880859375f,0.15749998390674591064453125f,1.0881999658823016119413296109996736049652099609375f,0.37839999008774771738217168604023754596710205078125f,-0.11580001933574590733400100361905060708522796630859375f,-0.968300018031894804693138212314806878566741943359375f,0.806499995350836940133376629091799259185791015625f,0.52270000736713395639299051254056394100189208984375f,-0.545099980437755871065519386320374906063079833984375f,0.1619000130653380775669347713119350373744964599609375f,-0.02310000034570702798220054319244809448719024658203125f,1.1485000422373417006127738204668276011943817138671875f,-0.5242000012278555942657476407475769519805908203125f,0.5701999891877174153620444485568441450595855712890625f,-0.210399996331334049326500235110870562493801116943359375f,0.322000020250678142819111826611333526670932769775390625f,0.088700007173418971007095024106092751026153564453125f,-0.500800004434585499524246188229881227016448974609375f,-0.0758000115871428903346895822323858737945556640625f,-0.85159998964667327214783654198981821537017822265625f,0.07929999969601631004678665703977458178997039794921875f,0.07430000073909759361612259453977458178997039794921875f,-0.15019999924600124785456500831060111522674560546875f,-0.24409999918341629410178938996978104114532470703125f,0.246799999248981549726522644050419330596923828125f,0.75360002174377438421970509807579219341278076171875f,0.09440000405013559969802372506819665431976318359375f,-0.3455000002384185364689983543939888477325439453125f,-0.0014999977722763568888098006937070749700069427490234375f,0.79030001593083143429652182021527551114559173583984375f,-0.094600000613927814896442214376293122768402099609375f,-0.746199992495775177303585223853588104248046875f,-0.147400000563263855468676410964690148830413818359375f,-0.958699991005659057918819598853588104248046875f,0.0434000088632106872932325813962961547076702117919921875f,0.08360000956207525035779326572082936763763427734375f,-0.216100009524822223028195367078296840190887451171875f,-0.47750000551342974830504317651502788066864013671875f,0.13309999773800373912280292643117718398571014404296875f,0.327400002127885780822680317214690148830413818359375f,-0.0177000002145767110217633444335660897195339202880859375f,0.0118999970838427060471076401881873607635498046875f,0.045400001844763782088421066873706877231597900390625f,-0.33080000205039983285359994624741375446319580078125f,0.62139997974634209043642840697430074214935302734375f,-0.309100014978647141816736620967276394367218017578125f,0.1591999915421009603733182302676141262054443359375f,-0.3456000112235546595229607191868126392364501953125f,0.5564000074625017333573850919492542743682861328125f,0.49499999836087216209534744848497211933135986328125f,0.105100002267956682544536306522786617279052734375f,0.245900002530217154372849108767695724964141845703125f,0.48159995850324666122332928352989256381988525390625f,-0.202499995976685998044786174432374536991119384765625f,0.97670000063180850702337920665740966796875f,0.01459998556375463607537312782369554042816162109375f,-0.798199985551833801622478858917020261287689208984375f,-0.635300016564130931584486461360938847064971923828125f,0.64459996589422263735258411543327383697032928466796875f,-0.119799992595612525558834704497712664306163787841796875f,0.5708999640107155215673628845252096652984619140625f,0.338400004392862552293763656052760779857635498046875f,-0.401699986028670963378317537717521190643310546875f,0.331500018894672621172503568232059478759765625f,0.66109997555017496750906502711586654186248779296875f,1.11279999585151667673699193983338773250579833984375f,-0.21519999872446060606989703956060111522674560546875f,-0.3459000046908855541261118560214526951313018798828125f,-0.814899994528293536433238841709680855274200439453125f,-1.0297000181555748810069417231716215610504150390625f,0.1159999875128268076451831802842207252979278564453125f,0.13009999794661997185585278202779591083526611328125f,-0.6958999750375749471231756615452468395233154296875f,-0.307800015223026246502513458835892379283905029296875f,-0.1473999965399503953022986024734564125537872314453125f,-0.37710000642538066273345975787378847599029541015625f,-0.136799998503923436743434649542905390262603759765625f,0.1431999940454959929780898164608515799045562744140625f,0.032400001308321879633922435459680855274200439453125f,-0.751700020748376829260450904257595539093017578125f,0.058899998298287403741824164171703159809112548828125f,-0.27029999682009220318690267959027551114559173583984375f,-0.2587999965518712475187612653826363384723663330078125f,0.03710001208782198744984270888380706310272216796875f,-0.273900008356571245116128920926712453365325927734375f,-0.52030002058744440063264846685342490673065185546875f,0.225800003987550734763090076739899814128875732421875f,0.4510000077784062089136796203092671930789947509765625f,-0.2365999963521957238921089583527646027505397796630859375f,-0.209199998880922777289015357382595539093017578125f,-0.3008999996989965008964418302639387547969818115234375f,0.002400007492303846134973355219699442386627197265625f,0.2729000023663044061095206416212022304534912109375f,-0.169499994844198198507001507095992565155029296875f,-0.0726999996185302421736196265555918216705322265625f,-0.663899991369247377548390431911684572696685791015625f,0.02280000024735928665364781409152783453464508056640625f,0.2173000044047832535198949699406512081623077392578125f,0.1349000014573336070355935589759610593318939208984375f,0.6980000181198120490222436274052597582340240478515625f,-0.11640000112950799060484996516606770455837249755859375f,0.12410000633597373376204586747917346656322479248046875f,0.156699999588727933996779029257595539093017578125f,-0.1456999774865806729362560645313351415097713470458984375f,0.372099999868868991370618459768593311309814453125f,0.96600002595782274994462568429298698902130126953125f,1.234799935412407734247608459554612636566162109375f,0.20999998092651406267350466805510222911834716796875f,-0.8383999795079228878336152774863876402378082275390625f,-0.32570000902116313934442359823151491582393646240234375f,0.27899998834729178298630358767695724964141845703125f,0.32080000592470181430826414725743234157562255859375f,-0.2819999771118169018535581926698796451091766357421875f,-0.56289999111592770031364807437057606875896453857421875f,0.67199996310472531746427193866111338138580322265625f,0.009499949991702560936346344533376395702362060546875f,-0.26700000028312220745618787987041287124156951904296875f,-0.2136999994620680798362855057348497211933135986328125f,-0.038399974106252177108444811892695724964141845703125f,0.3993000079289077408617458786466158926486968994140625f,0.41109997659325614449699060060083866119384765625f,-0.68410001006126375244775772443972527980804443359375f,0.4102999827384952169495591078884899616241455078125f,-0.4046000146210193548768074833787977695465087890625f,-0.6589999923706049145266661071218550205230712890625f,-1.19260002394914632617428651428781449794769287109375f,0.25169998602867149628536935779266059398651123046875f,1.010300012838840633122572398860938847064971923828125f,0.4731999880850314976754589224583469331264495849609375f,0.439299997833371147493153330287896096706390380859375f,-0.701500005483627120383971487171947956085205078125f,0.34080001471638698973265491076745092868804931640625f,-0.050199990119039750169349645148031413555145263671875f,0.23679999639913429376747444621287286281585693359375f,-0.234900007417798040165735073969699442386627197265625f,0.05419999035000788722982179024256765842437744140625f,-0.226799995672702703330969598027877509593963623046875f,-0.94079997463226305853822850622236728668212890625f,0.282800003004074096413233974089962430298328399658203125f,0.35260000733435159503414979553781449794769287109375f,0.034599995324015697661934609641321003437042236328125f,-0.4224999791383743374950654470012523233890533447265625f,0.093599994847178458190484207079862244427204132080078125f,0.1440999974697827479275247242185287177562713623046875f,-0.3715999941170216036567808259860612452030181884765625f,0.0615999914348126509366920799948275089263916015625f,-0.022200000795721985813457877156906761229038238525390625f,0.374800005969405158889884432937833480536937713623046875f,0.1668000002548097882026922889053821563720703125f,-0.356400018601119528138809755546390078961849212646484375f,0.00489999803006657685955360648222267627716064453125f,-0.4607999747812747681763312357361428439617156982421875f,-0.28269999388158328912368233432061970233917236328125f,0.185399978226423289839885910623706877231597900390625f,0.105400008028745606480924834613688290119171142578125f,0.59039999566078193993234890513122081756591796875f,0.0187000009894371377328070593648590147495269775390625f,-0.38749999091029163622579289949499070644378662109375f,-0.04320000201761720060034122070646844804286956787109375f,-0.47809999423623084080503531367867253720760345458984375f,0.069100003430247358693350179237313568592071533203125f,0.649000016659498246696102796704508364200592041015625f,0.187000001884996880363587479223497211933135986328125f,0.0478000055372718168911205793847329914569854736328125f,-0.309199998284875710030661366545245982706546783447265625f,-0.49380002272203571411779421396204270422458648681640625f,-0.30759999504089297062137120519764721393585205078125f,-0.2434000155687332522802535095252096652984619140625f,-0.051899985945225157735194443375803530216217041015625f,-0.4803999956607818422327227381174452602863311767578125f,-0.3000999607682235392758229863829910755157470703125f,0.08759999980926469476116835721768438816070556640625f,0.61829995436668383490541600622236728668212890625f,-0.09490001479387277782961973571218550205230712890625f,0.538500000961124936083024294930510222911834716796875f,-0.2131000070139765778520768435555510222911834716796875f,-0.237899989104271070150531386389047838747501373291015625f,0.68600001077726469045359181109233759343624114990234375f,1.3036999685049064368058679974637925624847412109375f,0.400499997705221488786264671944081783294677734375f,-0.18289995661973978968717347015626728534698486328125f,-0.2863999711036679940434623858891427516937255859375f,-0.4994999726414679486197201185859739780426025390625f,-0.29379999826550484698373111314140260219573974609375f,-0.62759998282194118246479774825274944305419921875f,-0.3791000090181828596769264549948275089263916015625f,-0.2601000033855438431373841012828052043914794921875f,0.34690000174045554359736343030817806720733642578125f,-0.2600000041723244947888815659098327159881591796875f,-0.2146000097036360187985337688587605953216552734375f,0.41089998070001598762246430851519107818603515625f,0.089600018495321620548565988428890705108642578125f,0.426699963974952911627269713790155947208404541015625f,-0.12780000106692313011080841533839702606201171875f,0.2036000059857964450404921308290795423090457916259765625f,-0.0792999930277465681083270965245901606976985931396484375f,0.249200004953145981545503673260100185871124267578125f,-0.197799991679191489168943007825873792171478271484375f,0.08349999473989011544716731805237941443920135498046875f,-0.106999997079372466401991914608515799045562744140625f,0.224499991267919529747132401098497211933135986328125f,-0.1661000035643578343069748370908200740814208984375f,-0.398300001603364872693191500729881227016448974609375f,0.19090001631379127644549953402020037174224853515625f,0.0062999986752867442874048720113933086395263671875f,-0.6365000078305602659867190595832653343677520751953125f,0.07109999909400939455128565214181435294449329376220703125f,0.0831999981991946579995556021458469331264495849609375f,-0.22049999360740191178820168715901672840118408203125f,-1.045400020098686155733958003111183643341064453125f,0.332899995958805039464323272113688290119171142578125f,-0.533900014615058982059281333931721746921539306640625f,0.370299999353289610581896340590901672840118408203125f,-0.2473999931871890822776549612171947956085205078125f,0.26880000161826611559945376939140260219573974609375f,0.13970000921487812917121118516661226749420166015625f,-0.061499998629093166613301946199499070644378662109375f,0.42950000110268593545015392010100185871124267578125f,0.268800004374980971988406963646411895751953125f,-0.29999997943639744590882401098497211933135986328125f,-0.2864999987035989903461086214520037174224853515625f,0.1693999893724917882309455308131873607635498046875f,-0.25749999754130836748799993074499070644378662109375f,0.83810001003146172848801143118180334568023681640625f,-0.8002999776720998159618147838045842945575714111328125f,0.01819998771250219693484950767015106976032257080078125f,0.0555000009834767382699283189140260219573974609375f,-0.32630000824928284686166080064140260219573974609375f,-0.47419999258518163998132877168245613574981689453125f,-0.031400007760524761835085882921703159809112548828125f,-0.518699997562169556886146892793476581573486328125f,0.64160003494620365671607942203991115093231201171875f,-0.4274999707937237047872258699499070644378662109375f,-0.74250000804662708020487116300500929355621337890625f,0.43579999549388848123498974018730223178863525390625f,0.791200006949901446517969816341064870357513427734375f,0.3499000076413150139842400676570832729339599609375f,-0.809500021219253795834447373636066913604736328125f,-0.40899998313188490328684565611183643341064453125f,0.097899992382526335177317378111183643341064453125f,-0.22569996320009266810302506200969219207763671875f,-0.1895999666392800264702600543387234210968017578125f,-0.095000017285347126971828402020037174224853515625f,0.49990001941323303213948747725225985050201416015625f,-1.32890000776052374931168742477893829345703125f,-0.412199994760751309286206378601491451263427734375f,-0.4095000289678569771467664395458996295928955078125f,-0.93250000610947569867903439444489777088165283203125f,-0.94719996749162671534350010915659368038177490234375f,-0.169900010323524863764532710774801671504974365234375f,-0.1690999966502193530004660715349018573760986328125f,0.46030001462697978098503881483338773250579833984375f,-0.28659999217987053299339095246978104114532470703125f,0.0673999972105026046165221487171947956085205078125f,0.321500002652406691172615182949812151491641998291015625f,0.2025000079721213097627696697600185871124267578125f,0.05250000022351741790771484375f,-0.1695999982297420327626014113775454461574554443359375f,0.2470000054240226727841900355997495353221893310546875f,-0.00739999303817739928490482270717620849609375f,0.03750000804662700915059758699499070644378662109375f,0.4920999895870685758580975743825547397136688232421875f,0.312300003510713519716546215931884944438934326171875f,-0.177300009769201238896130234934389591217041015625f,-0.1294000044226646384259993283194489777088165283203125f,-0.253800003778934524234500713646411895751953125f,-0.0107000065624713869283368694595992565155029296875f,-0.347100000017881438196809540386311709880828857421875f,0.3438000055670737964419458876363933086395263671875f,-0.3119999987185000378531185560859739780426025390625f,0.31729999449551105072941936668939888477325439453125f,-0.45260000413060197388404048979282379150390625f,-0.153400000444054551707040445762686431407928466796875f,0.438700010526180239622817680356092751026153564453125f,0.2245000130981207320246539893560111522674560546875f,1.066400022214651244212291203439235687255859375f,-0.077699995371699248636332413298077881336212158203125f,0.7042000268578529897922635427676141262054443359375f,0.2340000025033950326047715861932374536991119384765625f,0.58459999286532404738636614638380706310272216796875f,-0.096500002503395077013692571199499070644378662109375f,0.397299996954202594423577465931884944438934326171875f,-0.0013999979257584005409853489254601299762725830078125f,0.3867000004455445338891195206088013947010040283203125f,-0.275699994492530908729577276972122490406036376953125f,0.6411000046074388780681374555570073425769805908203125f,-0.211800011914968333570641334517858922481536865234375f,-0.17269999932050694724239292554557323455810546875f,-0.07499998658895545844416119507513940334320068359375f,-0.7767000061452389303440213552676141262054443359375f,-0.32639997140169185740887769497931003570556640625f,0.116700000780820545287497225217521190643310546875f,0.70379999122470593420075601898133754730224609375f,0.70640001116916539825041354561108164489269256591796875f,0.300000023245811320293796597979962825775146484375f,1.067199999678134503255932941101491451263427734375f,-0.2354000048995015248465279000811278820037841796875f,-0.97060001226663583651088629267178475856781005859375f,-0.47599998557567602119888761080801486968994140625f,0.13700002464652083489227152313105762004852294921875f,-0.0702999775230888079846636173897422850131988525390625f,-0.234099989572167377360756290727294981479644775390625f,0.13300000612437745672878008917905390262603759765625f,0.209100011327862933541155143757350742816925048828125f,-0.577999962687492985224935182486660778522491455078125f,-0.53209997602701175622996743186376988887786865234375f,0.8814999959468838852671979111619293689727783203125f,0.06779997394680936650956937228329479694366455078125f,0.1837999940186738623282280968851409852504730224609375f,-0.56720000802278514040466461665346287190914154052734375f,-0.3371999820947646497870664461515843868255615234375f,0.01749999374151212094830043497495353221893310546875f,0.092600004875659891467876150272786617279052734375f,-0.059100005628168583715620343355112709105014801025390625f,0.03489999989271164171356076622032560408115386962890625f,0.21270000110864639708552203956060111522674560546875f,-0.1549999958276748746044404470012523233890533447265625f,0.61219999729394913856594939716160297393798828125f,0.182599991092085911503772877040319144725799560546875f,-0.292499979436397605780939557007513940334320068359375f,0.18279999518096445854098419658839702606201171875f,-0.04110000267028812714897867408581078052520751953125f,-0.1422000044092535941953059364095679484307765960693359375f,-0.53510000521093614711531927241594530642032623291015625f,0.1335999974548816027208886225707828998565673828125f,-0.71020001839399338194880328956060111522674560546875f,-0.153399993813037927026243778527714312076568603515625f,0.674999986290931719423724644002504646778106689453125f,0.119200000408291817421968517010100185871124267578125f,0.27929999909996983120663571753539144992828369140625f,0.16309999498128890937920232317992486059665679931640625f,-0.27970000176429754557005935566849075257778167724609375f,0.164200000333786011452730235760100185871124267578125f,-0.54249999918043612101570261074812151491641998291015625f,0.2535999930590391304718878018320538103580474853515625f,0.1237000015854834433781661573448218405246734619140625f,-0.104300005283951779944118243292905390262603759765625f,-0.799600007170438775716547752381302416324615478515625f,-0.075000000596046445533460200749686919152736663818359375f,-0.01649999997019768738226730420137755572795867919921875f,0.239199999588727951760347423260100185871124267578125f,-0.152799995404481858685130646335892379283905029296875f,0.108199997845292106291026357212103903293609619140625f,-0.6027999994650479198554648974095471203327178955078125f,-0.032399980521201854344326420687139034271240234375f,0.067100006723403993191823246888816356658935546875f,0.85809995743036271420578486868180334568023681640625f,0.027999960154294623038140343851409852504730224609375f,0.21200000955909492628137513747788034379482269287109375f,0.05970000265836716835110564716160297393798828125f,0.0340999845802780754411287489347159862518310546875f,-0.49460002624392540582221045042388141155242919921875f,-0.478800002251565270938726825988851487636566162109375f,0.4510000274851921897578677089768461883068084716796875f,-0.23259997820258160317052897880785167217254638671875f,0.279400010681152277669525574310682713985443115234375f,-0.3070999721527105652540967639652080833911895751953125f,-0.62969998291432849324422704739845357835292816162109375f,-0.00540000698566434067515729111619293689727783203125f,-0.103599999541044152095992103568278253078460693359375f,0.339099978470802199126410414464771747589111328125f,-0.35039999119043319097954736207611858844757080078125f,0.4907999844670296596405023592524230480194091796875f,0.069899986034631655940074779209680855274200439453125f,-0.4529999881684780849155913529102690517902374267578125f,-0.28589999373853203223916352726519107818603515625f,0.718599974060058599434341886080801486968994140625f,0.138999990284442986876456416212022304534912109375f,0.25179998978674422716039771330542862415313720703125f,-0.606000010237097708198916734545491635799407958984375f,-0.12479998995065688660588421043939888477325439453125f,0.311300006312131838370760306133888661861419677734375f,0.23050002646446188236950547434389591217041015625f,-0.392000000394880754317483706472557969391345977783203125f,0.24000000119209285998067571199499070644378662109375f,-0.515000007599592191098736293497495353221893310546875f,-0.3993000003665685415654706957866437733173370361328125f,-0.0943999938428401463852424058131873607635498046875f,0.103599998833239059781163859952357597649097442626953125f,-0.48920000528842211284796803738572634756565093994140625f,-0.23290000236630437058238385361619293689727783203125f,0.5175000073015689761035673654987476766109466552734375f,0.0932000025764108119830098075908608734607696533203125f,0.326900011463463313621247152696014381945133209228515625f,0.2045999969631433490491190241300500929355621337890625f,-0.2090999996304512276168452444835565984249114990234375f,0.111899993693828581076132877569762058556079864501953125f,0.134899995794892202383152834954671561717987060546875f,-0.0361999997228383829650510961073450744152069091796875f,0.0943000050604343442728350055404007434844970703125f,-0.265900000220537169326462390017695724964141845703125f,0.227899999237060502110807647113688290119171142578125f,-0.0856000005692243437493971214280463755130767822265625f,-0.383699995923042269652114555356092751026153564453125f,0.05980000060498713576606633068877272307872772216796875f,0.40889999807476995474786463091732002794742584228515625f,-0.05460000389218322425222140736877918243408203125f,-0.7255999785900115739423199556767940521240234375f,-0.21899999691545968261152665945701301097869873046875f,0.23539998470842837807737168986932374536991119384765625f,0.0796000002413987939320350051275454461574554443359375f,-0.05090000454187393330585109652020037174224853515625f,0.145499993979930852816551123396493494510650634765625f,1.12040000698566455383797801914624869823455810546875f,-0.1455000117123130820573351229541003704071044921875f,-0.81500003352761307695573123055510222911834716796875f,-0.625499987721443329036219438421539962291717529296875f,-0.412599993550777366380089006270281970500946044921875f,-0.7374999877810477499906482989899814128875732421875f,-0.75200000646710396523531017010100185871124267578125f,0.022400009727478131793532156734727323055267333984375f,-0.3420000072121620604548297706060111522674560546875f,-0.47389996879398854634501958571490831673145294189453125f,-0.7142999933257698774724531176616437733173370361328125f,0.093899980640411939702971721999347209930419921875f,0.772499989867210246075046597979962825775146484375f,-0.4307999723970892258506637517712078988552093505859375f,-0.5667999938100576340360703397891484200954437255859375f,-0.915799995195865079722352675162255764007568359375f,-0.24710001104474077493478034739382565021514892578125f,-0.122900001621246701688505709171295166015625f,-0.75100001105666169820551658631302416324615478515625f,0.1853000077724455962879801518283784389495849609375f,-1.085100012028217264514751150272786617279052734375f,-0.54379996533393892832464189268648624420166015625f,-1.0803999794185159544213092885911464691162109375f,-0.6939000149130816197384774568490684032440185546875f,0.20430001981258438803479293710552155971527099609375f,0.4749000031709673663726789527572691440582275390625f,0.78890000120401371219713837490417063236236572265625f,-0.117099992567300859036549809388816356658935546875f,0.53210000672340385108327609486877918243408203125f,-0.20289997688531880015716524212621152400970458984375f,-0.6917000054001807285430913907475769519805908203125f,-0.1941000030577182400293168029747903347015380859375f,0.6551000115811824731082424477790482342243194580078125f,0.1458999986559152972631636657752096652984619140625f,0.4204000119030475435266680506174452602863311767578125f,0.165900001487135906330649959272705018520355224609375f,0.5226000100910663803688294137828052043914794921875f,0.1128000002473592555674031245871447026729583740234375f,-0.2681000034749507765496900901780463755130767822265625f,0.285899995377659810191772749021765775978565216064453125f,-0.063699995252490093644581747867050580680370330810546875f,-0.1866999913930893484348416677676141262054443359375f,0.27989998722672471132000282523222267627716064453125f,0.2195999973356723700135262333787977695465087890625f,0.03039999864101405879779349561431445181369781494140625f,0.082600009271502461416503138025291264057159423828125f,0.8982999981760979579803461092524230480194091796875f,-0.088700003448128672545180961606092751026153564453125f,0.384000006824731787702376095694489777088165283203125f,0.08589999724030494832049953402020037174224853515625f,-0.36169999869465829789305644226260483264923095703125f,0.0749999993294476752225818927399814128875732421875f,0.51929997704625119325783089152537286281585693359375f,0.103500005990266874533745067310519516468048095703125f,0.567900024121999802417803948628716170787811279296875f,-0.010099997276067806950550220790319144725799560546875f,-0.57290002084374425095347760361619293689727783203125f,0.083499999061226848340311335050500929355621337890625f,-0.0795999988257884982800760553800500929355621337890625f,0.2384999979436397410381687222979962825775146484375f,-0.57230000276565551331486858543939888477325439453125f,-0.54959998370111040610908048620331101119518280029296875f,-0.4950999868080019350458087501465342938899993896484375f,-0.4821999889492989410655354731716215610504150390625f,0.67900001293420775283493640017695724964141845703125f,0.1202999988317490220879335538484156131744384765625f,-0.75229998711943579792205127887427806854248046875f,-0.431399981422722345048015313295763917267322540283203125f,-0.34419999178424476315996116682072170078754425048828125f,0.34189998624324768883298020227812230587005615234375f,0.109900006896257451671772287227213382720947265625f,-0.081199981282651823022433745791204273700714111328125f,0.4974999964609738700715979575761593878269195556640625f,0.268500001370906460351761779747903347015380859375f,0.92180004171729112982802689657546579837799072265625f,0.3926000069618229559864630573429167270660400390625f,0.912000024050474422665502061136066913604736328125f,-0.187200004148483145627324120141565799713134765625f,0.642300024223327881145451101474463939666748046875f,0.66670000048279742799195446423254907131195068359375f,-0.032800009560585152712519629858434200286865234375f,0.7885999797225000662592719891108572483062744140625f,-0.23450001391768449110486471909098327159881591796875f,-0.03690000665783887967563714482821524143218994140625f,-0.08490000004172326697471362422220408916473388671875f,0.351999988436698885152509319595992565155029296875f,0.0392999941825866727640459430404007434844970703125f,0.0587000228382643651769967618747614324092864990234375f,0.48589999167099551247162025902071036398410797119140625f,0.00060000809431048907072181464172899723052978515625f,-0.433099990659952194249626700184307992458343505859375f,0.061199995326995881850251635114545933902263641357421875f,0.5293000067740678549199628832866437733173370361328125f,-0.093799996626377168951194107648916542530059814453125f,0.748199980783462503808323162957094609737396240234375f,0.10609999842345718690239664283581078052520751953125f,-0.34779998959302904637524989084340631961822509765625f,0.0050000030174850718100998392401379533112049102783203125f,0.5118999800965189361789953181869350373744964599609375f,-0.293900001427531254449831976671703159809112548828125f,0.266399990475177705917531056911684572696685791015625f,0.1911999936506152408810521592386066913604736328125f,-0.366799991910159606245400709667592309415340423583984375f,-0.271300000575184885320823013898916542530059814453125f,0.26139997155070293644030243740417063236236572265625f,0.10020000416338437077001799480058252811431884765625f,-0.7349000200092792312034362112171947956085205078125f,-0.04519999693632126280817828956060111522674560546875f,0.292199996548891061109998190659098327159881591796875f,-0.02470000437200070564358611591160297393798828125f,-0.7248000051498413309758461764431558549404144287109375f,-0.166599998959898965011916516232304275035858154296875f,-0.5331000057101249467450543306767940521240234375f,-0.187899998193979289595745285623706877231597900390625f,0.4778999905943870363245196131174452602863311767578125f,-0.0588000016927718860415552626363933086395263671875f,-0.280599993640184397492021162179298698902130126953125f,0.17009999472424386368629711796529591083526611328125f,-0.48220000331029300166818529760348610579967498779296875f,-0.0650000020116567878147861847537569701671600341796875f,-1.253100047619640999840839867829345166683197021484375f,-0.023899991853535251795648264305782504379749298095703125f,0.14030001274570824687515369078028015792369842529296875f,0.601299977552890663190510167623870074748992919921875f,0.4095999956965445942813630608725361526012420654296875f,-0.20960000433921788243196715484373271465301513671875f,-0.564299998503923205817045527510344982147216796875f,0.08959997803866837084996177509310655295848846435546875f,0.4696000063881278219213299962575547397136688232421875f,-1.03059998544454600732933613471686840057373046875f,-0.37210002460479785213465220294892787933349609375f,-0.0389000199794775625150577980093657970428466796875f,1.127900045281648999662138521671295166015625f,0.4629999845921990431207859728601761162281036376953125f,0.51590000625550747059833156527020037174224853515625f,-0.08430001027584033135298113847966305911540985107421875f,0.8183000236198310073376660511712543666362762451171875f,-0.35299995084106960963055144020472653210163116455078125f,-0.6637999819114803745634389997576363384723663330078125f,-0.079500007659196458842387755794334225356578826904296875f,-0.5051000131644307611367139543290250003337860107421875f,0.6260999487280844988390526850707828998565673828125f,-0.03580004094243083301307706278748810291290283203125f,-0.740199978160858140796563020558096468448638916015625f,-0.90969999401569356933805465814657509326934814453125f,0.574999995492398330299010922317393124103546142578125f,-0.2753000297144059249632164210197515785694122314453125f,-0.9293999771535397247390619668294675648212432861328125f,-0.756099995592236506780636773328296840190887451171875f,0.12899998551607172458943750825710594654083251953125f,0.0720000066533685867398162372410297393798828125f,-0.1352999977514148521873238451007637195289134979248046875f,0.3311999750241636508718556797248311340808868408203125f,-0.27050000105798244209864833464962430298328399658203125f,-0.106400003290176403680789007921703159809112548828125f,-0.049999995604157465578509800252504646778106689453125f,0.38809999587535859433273799368180334568023681640625f,-0.270400001248717269941579388614627532660961151123046875f,0.491400006195902794292607040915754623711109161376953125f,0.1877999977886677385185976163484156131744384765625f,0.79820002384781840731875490746460855007171630859375f,-0.26510000524818888578693076851777732372283935546875f,0.59820001356601704145532494294457137584686279296875f,-0.21700000296533106247665045884787105023860931396484375f,-0.07170000107884411111314193476573564112186431884765625f,0.0291000049203634369998638931065215729176998138427734375f,-0.007399997173249683868334614089690148830413818359375f,0.334599995100498148747902860122849233448505401611328125f,-0.319700015100836842929510339672560803592205047607421875f,0.02990000406503677543579300390774733386933803558349609375f,0.04940000451579695806003655889071524143218994140625f,-0.468900000384449899826222463161684572696685791015625f,0.3406000159919262415542107191868126392364501953125f,0.0790000031739472863279161174432374536991119384765625f,0.7324999880790710182765224089962430298328399658203125f,-0.013800007839500860740145071758888661861419677734375f,-0.63029998754411931116692358045838773250579833984375f,0.13140000194907186514825525591732002794742584228515625f,0.82719999729394910747970470765721984207630157470703125f,-0.254599992269277652923165078391321003437042236328125f,-0.00750004172325180462621574406512081623077392578125f,-0.76980000336170206765018519945442676544189453125f,-0.08070000030398372103945803246460855007171630859375f,-0.209599997484684053006276371888816356658935546875f,0.98710001611113540320729953236877918243408203125f,-0.53869998131990382006506479228846728801727294921875f,-0.131200002926588066287649780861102044582366943359375f,-0.27329999571740615227355419847299344837665557861328125f,0.5503000192090869990835244607296772301197052001953125f,0.33359999402761442155451732105575501918792724609375f,-0.0286000153362753195551704266108572483062744140625f,-0.300999984979629431336434208787977695465087890625f,0.43619999309182144742180753382854163646697998046875f,-0.49559997501373320716311354772187769412994384765625f,-0.299299989712238367900454250047914683818817138671875f,-0.2080000044107437329188314834027551114559173583984375f,-0.262000006690621400906593407853506505489349365234375f,0.4382999677777295044478478303062729537487030029296875f,1.0030000082105399172860416001640260219573974609375f,0.83099998915195527615651371888816356658935546875f,0.360399984186887678561106440611183643341064453125f,0.71979997653961191872440394945442676544189453125f,-0.1659000040948388488004638929851353168487548828125f,0.54369998906850813824576107435859739780426025390625f,0.772200003552436786691259840154089033603668212890625f,-0.5435999944373961501042913369019515812397003173828125f,-0.31530000693425541680170454128528945147991180419921875f,0.506599991583823960894505944452248513698577880859375f,-0.3331000282108786958445989512256346642971038818359375f,0.58109997465610518929679528810083866119384765625f,-0.52839999902844425605508149601519107818603515625f,0.037400000116229126234657087479718029499053955078125f,0.822300021839141859203436979441903531551361083984375f,-0.077999998226761846353838336654007434844970703125f,0.37689998981952665957351200631819665431976318359375f,0.2855000055283307691667005201452411711215972900390625f,-0.0150000025331973318998279864899814128875732421875f,0.0837000018090009734184064882356324233114719390869140625f,0.10250000108033419099484717662562616169452667236328125f,-0.085600002655386919769853193429298698902130126953125f,-0.1942999990999698667337725055404007434844970703125f,0.322599998244643249023511089035309851169586181640625f,-0.40669998379349703299112661625258624553680419921875f,0.0727999988317489243883073868346400558948516845703125f,0.429799988311529101991936840931884944438934326171875f,-1.4901158085223187299561686813831329345703125e-10f,0.25700001019239426813811633110162802040576934814453125f,-0.195499994650483122082817999398685060441493988037109375f,-0.11559999878108495607875738642178475856781005859375f,-0.0485999945491553120291428058408200740814208984375f,-0.88210001938939086585378390736877918243408203125f,-0.0522000024348496793891172274015843868255615234375f,-0.717699984717369066089531770558096468448638916015625f,-0.1201000007033348193630217792815528810024261474609375f,-0.40090000603199005269061672152020037174224853515625f,-0.0215999916955829374121123009899747557938098907470703125f,0.7031999842479825790775294080958701670169830322265625f,-0.11340000349879264529562306051957421004772186279296875f,-0.318999998927116390490255071199499070644378662109375f,-0.326699997130036390302620930015109479427337646484375f,-0.4978999850809573413101816186099313199520111083984375f,-0.59080000585019565306765798595733940601348876953125f,0.521200029227137928700130942161194980144500732421875f,0.270699990133941081893453883822076022624969482421875f,-0.0828000042334198216309459894546307623386383056640625f,-0.2020000100433827494583738371147774159908294677734375f,-0.134100003877282158981643078732304275035858154296875f,0.1218999933958055947869070223532617092132568359375f,1.10000002816319497611630140454508364200592041015625f,0.40809996756315314314633724279701709747314453125f,-0.2163999989688392133757588453590869903564453125f,-0.4776000012993808496730707702226936817169189453125f,0.7633000151634217189666742342524230480194091796875f,-0.61679999626874870699566599796526134014129638671875f,-1.11030002148151396568209747783839702606201171875f,-0.1205000288486477444394040503539144992828369140625f,-0.9498000182628629772807471454143524169921875f,-0.01419999862015242086954458500258624553680419921875f,-0.648300017175078568953949798014946281909942626953125f,-0.028399984276294976126564506557770073413848876953125f,-0.5069000111281873888202653688495047390460968017578125f,-0.10960001447200706792273194878362119197845458984375f,-1.053000029146671234769883085391484200954437255859375f,0.283099979633093246089714511981583200395107269287109375f,0.26119998923614617947208671466796658933162689208984375f,-0.85719998090267157664357000612653791904449462890625f,-0.066599994713067989238197696977294981479644775390625f,0.535900003238022559770570296677760779857635498046875f,-0.2688000015065072911824017865001223981380462646484375f,0.535999988257884751874371431767940521240234375f,-1.028199975419044509550303700962103903293609619140625f,-0.84390003182590056507450526623870246112346649169921875f,0.228000042632222399419106295681558549404144287109375f,-0.956400025343894899521046681911684572696685791015625f,0.1194000258803367131577033433131873607635498046875f,-0.5931000063806772804042566349380649626255035400390625f,0.3522999938249589124694693964556790888309478759765625f,0.562200004912167816684132048976607620716094970703125f,-1.3494999801293017238634774912497960031032562255859375f,-0.61470000385045953095186632708646357059478759765625f,0.7412000090360635340402950532734394073486328125f,0.0050999896764754026889931992627680301666259765625f,0.32720002322196961586087127216160297393798828125f,-0.89099999652802919225536015801480971276760101318359375f,-0.274399978718161907664807586115784943103790283203125f,-0.5153999947667120551386688021011650562286376953125f,-0.790600017333030535837679053656756877899169921875f,-0.567000004529953027798683251603506505489349365234375f,0.1423999849915507098785383277572691440582275390625f,-0.0932999993868171839661584954228601418435573577880859375f,-0.0562999963097274847001472153351642191410064697265625f,-0.1119000072538849366310387267731130123138427734375f,-0.674699970769882639842762728221714496612548828125f,0.35460001447200806268256201292388141155242919921875f,-1.780799966287613500526276766322553157806396484375f,0.207899994915723862476397698628716170787811279296875f,-0.241299985301494768208385721663944423198699951171875f,-0.236700001525878889196974341757595539093017578125f,-1.093000012159347278384302626363933086395263671875f,-0.441899978047608943398927294765599071979522705078125f,-0.205799999703466574008103862070129252970218658447265625f,0.1175999993622303119167327167815528810024261474609375f,0.440500002287328218386619482771493494510650634765625f,-0.05020000438690175315059605054557323455810546875f,-0.84209998984783862585601355021935887634754180908203125f,-0.0495999992728233252137215458787977695465087890625f,-0.536899983263015911916227196343243122100830078125f,0.12729999769926081398807582445442676544189453125f,0.85020001343935713844501833591493777930736541748046875f,0.02549999107420453103856061716214753687381744384765625f,-0.65809997765868910590825180406682193279266357421875f,0.45269999306201924582637730054557323455810546875f,0.0713999735996129203385862638242542743682861328125f,-0.016600001418590526469643009477294981479644775390625f,0.57290000437796118060163053087308071553707122802734375f,-0.5964000029921532330945410649292171001434326171875f,0.36829998368471894565345792216248810291290283203125f,-0.078100003400444872880825641914270818233489990234375f,-0.96199996332824244671400037987041287124156951904296875f,-0.36280000419616698081881622783839702606201171875f,-0.5404999800845982083075114132952876389026641845703125f,-0.049099997432529907115394962602294981479644775390625f,0.123600000993907332969001799938268959522247314453125f,-0.31780000240802763755709747783839702606201171875f,-0.4947000032171604555486510435002855956554412841796875f,0.268700000840425445858272723853588104248046875f,-0.430400006650388300410980946253403089940547943115234375f,-0.186299993646144912418094463646411895751953125f,-0.9706999850675466223748344418709166347980499267578125f,-0.16509999995827673302528637577779591083526611328125f,-0.270600006902217415927225374616682529449462890625f,-0.75180000811517244674320181729854084551334381103515625f,-0.056899969330430177905100208590738475322723388671875f,0.19919999213814687522017266019247472286224365234375f,-1.40630000824928202973751467652618885040283203125f,0.29970000012516972986986729665659368038177490234375f,-0.677499983310699604999172152020037174224853515625f,-0.76110001965761231446094825514592230319976806640625f,0.731800035905838086591757019050419330596923828125f,0.572100012274086022756591773941181600093841552734375f,-0.4474999889358877158684890673612244427204132080078125f,-0.727499966025352051701702293939888477325439453125f,-0.35020000081062363506134715862572193145751953125f,-0.7671000126838685151398067318950779736042022705078125f,0.02310003864169107856696427916176617145538330078125f,0.66540000683665301295377503265626728534698486328125f,0.947299979519844015385388047434389591217041015625f,0.436799991872906190337744192220270633697509765625f,-1.34039996898770308320081312558613717555999755859375f,1.1844000123202800267563361558131873607635498046875f,0.1817000039100644670497786137275397777557373046875f,0.4406999972492455786010623342008329927921295166015625f,0.7927999788641930667409951638546772301197052001953125f,0.7986000040113925280138573725707828998565673828125f,-0.2129000302314754122789963730610907077789306640625f,0.075300015968084554884853787370957434177398681640625f,-0.417200015473365937168637174181640148162841796875f,0.9816000042498114197542236070148646831512451171875f,-0.726500007271766889971331693232059478759765625f,0.648000001877546250028672147891484200954437255859375f,0.98230001531988353491442467202432453632354736328125f,-0.3869999940991402098688922706060111522674560546875f,0.2748999861091376839539179854909889400005340576171875f,0.0900999866962433060280091012828052043914794921875f,0.19039998139292035883585185729316435754299163818359375f,-0.04769999932050694724239292554557323455810546875f,-0.241499995835125158549772095284424722194671630859375f,-0.19720000191330910865872283466160297393798828125f,-0.01329999698400496921379954073927365243434906005859375f,0.0435000052452085839149731327779591083526611328125f,-1.00590000539869084406063848291523754596710205078125f,-0.182600007855892254582386158290319144725799560546875f,0.154299993884563502177798000047914683818817138671875f,-0.15860000559464093594641553863766603171825408935546875f,-0.3874000041395422311296670159208588302135467529296875f,-0.0509000121414662221042135570314712822437286376953125f,-0.7566000011950730907983597717247903347015380859375f,-0.1285000004023313380230320035479962825775146484375f,-0.262999995693564303511635671384283341467380523681640625f,-0.325100006663799234729594900272786617279052734375f,0.653500019997358183587721214280463755130767822265625f,0.133500004462897781110086725675500929355621337890625f,-0.2606999894633890146877774895983748137950897216796875f,-0.2227000055789947197126821265555918216705322265625f,-0.63829999303519713294008397497236728668212890625f,0.06319999715611335222575917214271612465381622314453125f,-0.9191999986201524475148971760063432157039642333984375f,-0.140600003623962521714929607696831226348876953125f,-0.82560002321898906529895612038671970367431640625f,-0.22760001046359545906483390353969298303127288818359375f,0.8797000306725504259475201251916587352752685546875f,0.47009999012351055824865397880785167217254638671875f,0.7370999673843385835425578989088535308837890625f,0.2493000287532805003820612910203635692596435546875f,0.8163999681234361815995725919492542743682861328125f,-0.4974000280559067732610856182873249053955078125f,-1.210099998021124889646671363152563571929931640625f,0.37290000475049023265228242962621152400970458984375f,0.26450000214576707691094270558096468448638916015625f,0.1104000187575824298846782767213881015777587890625f,-1.737699992465973508615206810645759105682373046875f,0.4845000286698344371671964836423285305500030517578125f,0.75930000386834084480369710945524275302886962890625f,0.218700002572685292534515610896050930023193359375f,-0.6266000141218299734902075215359218418598175048828125f,1.04280005091130778538399681565351784229278564453125f,-0.073000007465482408264279001741670072078704833984375f,-0.021000006437301710349174754810519516468048095703125f,-0.684199991691112341385405670735053718090057373046875f,0.1622999963581559512704188819043338298797607421875f,1.112100042486190432100556790828704833984375f,-0.6498000147610902832440160636906512081623077392578125f,-1.2500999670267105567944554422865621745586395263671875f,0.83190001053214057691320704179815948009490966796875f,-0.3345000146627423731615635915659368038177490234375f,0.57810002174749985925927830976434051990509033203125f,-0.8996000134661794600532402910175733268260955810546875f,0.9594000363111501172852513263933360576629638671875f,-0.42480004925727854470096644945442676544189453125f,1.30920004388242983139889474841766059398651123046875f,0.57779999864548425847488033468835055828094482421875f,0.2595000039339065978083453956060111522674560546875f,-0.94280001019388404159826677641831338405609130859375f,0.171199997115135005287811509333550930023193359375f,0.6187999625399711600692853608052246272563934326171875f,0.05429999954700459596779182902537286281585693359375f,-0.66790001421272748860502588286180980503559112548828125f,-0.0721999963998793958808164461515843868255615234375f,0.1724000042140485522423887232434935867786407470703125f,0.1631000083923339616376324556767940521240234375f,-0.6754000024408102209605431198724545538425445556640625f,-0.209800002467632307201483854441903531551361083984375f,-0.2791000013440847471457573192310519516468048095703125f,0.3369000102341175306719378568232059478759765625f,0.568100001947581478844995217514224350452423095703125f,-0.28320000462532046725527834496460855007171630859375f,-0.0458000008955596893400752378511242568492889404296875f,0.1327999982357024322254801518283784389495849609375f,-0.24350000233948254635407693058368749916553497314453125f,-0.0596999951332808365123128169216215610504150390625f,0.5826999814018607448673492399393580853939056396484375f,0.1599000058531760970481627737171947956085205078125f,0.6532999943390487285199696998461149632930755615234375f,-0.2008999929189680955232688575051724910736083984375f,0.19050003603845799826643769847578369081020355224609375f,-0.1767999953746794972175848670303821563720703125f,0.69260001769065837606831337325274944305419921875f,0.1079999993443490069466861314140260219573974609375f,-0.29989997623711861507445064489729702472686767578125f,-0.380200000773370294648856315689045004546642303466796875f,0.4066000434398642227051823283545672893524169921875f,-0.38970001279115695780319583718664944171905517578125f,-2.0608000324487676380158518441021442413330078125f,-0.924899971878527793478497187606990337371826171875f,0.10880000884532847749142092652618885040283203125f,0.625500029295683379615411467966623604297637939453125f,-0.3593000314354899416002808720804750919342041015625f,-0.81480003197193173747336913947947323322296142578125f,-0.879599977517127484816228388808667659759521484375f,-0.1254000090718268012324188021011650562286376953125f,-1.28940001485347721654761699028313159942626953125f,1.0785000165700913132837968078092671930789947509765625f,0.71509996717572210656044262577779591083526611328125f,-0.8778000252068043351982851163484156131744384765625f,0.26670001106262208878661112976260483264923095703125f,0.525900012886524326205517354537732899188995361328125f,0.3149999997019765629602261469699442386627197265625f,0.6441000316679481585424582590349018573760986328125f,0.575499988317488941902411170303821563720703125f,-0.2796000017315145580454327500774525105953216552734375f,0.6680999908089635397345773526467382907867431640625f,-0.66000001236796368431214432348497211933135986328125f,0.52950000929832441443068091757595539093017578125f,-0.31080004213452383510229992680251598358154296875f,-0.294099946880340468169379164464771747589111328125f,-0.4422000324606898402635124512016773223876953125f,0.520200036573410073970080702565610408782958984375f,0.72239999974369961677211904316209256649017333984375f,0.54669999258518231499692774377763271331787109375f,0.550200002319365832903486079885624349117279052734375f,0.14089999867454172655101274358457885682582855224609375f,0.1574999970197676901761951739899814128875732421875f,-0.9389999855905772907505024704732932150363922119140625f,-0.03589999187588688300820649601519107818603515625f,-0.03369998761564507727683803750551305711269378662109375f,0.356900002038478714183611373300664126873016357421875f,-0.8524000148683785749170738199609331786632537841796875f,0.21210000642538062720632296986877918243408203125f,0.874299983491003462887647401657886803150177001953125f,0.30300000068545340781156482989899814128875732421875f,0.3781999976962804321800604157033376395702362060546875f,0.24049999696016310934965076739899814128875732421875f,-0.21319999907463793586970268734148703515529632568359375f,-0.35599999570846563301529386080801486968994140625f,-0.469499985046684997769261826761066913604736328125f,0.0685000073313712931621921597979962825775146484375f,0.4427000217095014011903231221367605030536651611328125f,-0.102700006324052850459338515065610408782958984375f,0.409699990662932389540173971909098327159881591796875f,-0.130999998539686135501369790290482342243194580078125f,0.494200011621415524754041825872263871133327484130859375f,-0.03710000091195109206410052138380706310272216796875f,-0.6199999919161200967465674693812616169452667236328125f,-0.2933999980971215837399768133764155209064483642578125f,0.5401000298723577230930459336377680301666259765625f,0.0569999958872795531306110206060111522674560546875f,-0.70319998469501732163422502708272077143192291259765625f,0.09609999492168430634819742408581078052520751953125f,0.208299983572960145661312481024651788175106048583984375f,0.0149999988079070334379139239899814128875732421875f,-0.38089996723309205162166790614719502627849578857421875f,-0.01499997764825788948428453295491635799407958984375f,-1.6513999591827399626708938740193843841552734375f,-0.32760002737641347181352102779783308506011962890625f,-0.56029998273849468404250728781335055828094482421875f,-0.3887999756157392017286156260524876415729522705078125f,-1.77319999881386802798033386352472007274627685546875f,-1.104600008660554522066377103328704833984375f,0.399700017857551159750073566101491451263427734375f,-0.991200005161762209837661430356092751026153564453125f,-0.29579997195005436338988147326745092868804931640625f,-0.180299987804889649822825958835892379283905029296875f,-0.3847999923348428552571931504644453525543212890625f,0.96000000432133703043291461654007434844970703125f,0.0155000061988825876824193983338773250579833984375f,-1.21620002766251555925691718584857881069183349609375f,-0.752699918556213987130831810645759105682373046875f,0.621900025135278866628141258843243122100830078125f,1.53429998166561087913351002498529851436614990234375f,-0.2037000212550166278191454694024287164211273193359375f,0.52060001837611213915124608320184051990509033203125f,0.525300012987852227297480567358434200286865234375f,-0.5107999987721445478427995112724602222442626953125f,-0.329400040073693389786058105528354644775390625f,0.248300047424435899046102349529974162578582763671875f,-0.0002999976396562686886682058684527873992919921875f,-1.291599979066848646880316664464771747589111328125f,0.1479000081777570585472858510911464691162109375f,0.71729995895624210078267424250952899456024169921875f,-0.01679998524188963671122110099531710147857666015625f,-0.7197999898016453901306022089556790888309478759765625f,-0.2115000013113021992694484652020037174224853515625f,0.46730000459104770182960919555625878274440765380859375f,-0.10050000187754637437365090590901672840118408203125f,-0.5205000088438389571621200957451947033405303955078125f,0.563800002586841486618141061626374721527099609375f,-0.3719000100106002459909859680919907987117767333984375f,-0.0708000014543532785182833322323858737945556640625f,-0.6550999836787581021013693316490389406681060791015625f,-0.4388000005006791326422899146564304828643798828125f,-0.0660000015571713394280095599242486059665679931640625f,0.42529999674558638389498810283839702606201171875f,-0.34530000784695147331149200908839702606201171875f,0.40279998929500582249829676584340631961822509765625f,-0.637999988503754167368242633529007434844970703125f,0.340799995791912113674015927244909107685089111328125f,-0.3236999913781881677010687781148590147495269775390625f,-0.2444000035285949223862189683131873607635498046875f,0.7330000048950313651374699475127272307872772216796875f,0.29069999747276309420840334496460855007171630859375f,-0.19079999076277032177273440538556315004825592041015625f,0.5474999827146529440824451739899814128875732421875f,-0.39360002643763980501034893677569925785064697265625f,0.0817999928414820942634833045303821563720703125f,-0.469399999989569050029558638925664126873016357421875f,-0.03410000395178780507876581395976245403289794921875f,-0.5392999679565431137007180950604379177093505859375f,0.24820000075101855685488771996460855007171630859375f,-0.06010000610500565443317100289277732372283935546875f,-0.21830000056028353583315038122236728668212890625f,-0.529000001907348860186175443232059478759765625f,0.0191999983966351095432401052676141262054443359375f,0.685100021266936920483203721232712268829345703125f,0.45300001752376584107651069643907248973846435546875f,-0.4919000169396401389576567453332245349884033203125f,-0.61949997964501335445675067603588104248046875f,-0.9280999934911733362241648137569427490234375f,0.04499999180436109469383154646493494510650634765625f,-1.2828999819517132863211372750811278820037841796875f,0.62490001538991979401771459379233419895172119140625f,0.234400011873244551452444284223020076751708984375f,-0.215099981182813593250102712772786617279052734375f,-0.475500012159347562601396930404007434844970703125f,-0.327899975395202769590241587138734757900238037109375f,-1.2223999683022501727691633277572691440582275390625f,-0.5756999953866006602964944249833934009075164794921875f,-1.03069997348189357211367678246460855007171630859375f,0.47369999830722786526848722132854163646697998046875f,1.789799995613098104740856797434389591217041015625f,-0.243299994599819324747613791259936988353729248046875f,-0.371199984896183110549827688373625278472900390625f,-0.44639994428157780959054434788413345813751220703125f,-0.74880000824928316660589189268648624420166015625f,-0.3764999783635136765269635361619293689727783203125f,-1.33339996371269275954318800359033048152923583984375f,0.5510999910473817209322078269906342029571533203125f,-0.3158999878525730053979714284650981426239013671875f,-0.411000005096197273957159268320538103580474853515625f,-0.4003000116467472224712764727883040904998779296875f,1.1297999903976911895142620778642594814300537109375f,0.306300004337728026815312887265463359653949737548828125f,-0.01190000442266470059848870732821524143218994140625f,-0.3331000137567519114867309326655231416225433349609375f,-0.3549000025749204922931312466971576213836669921875f,0.12810001841336504657675732232746668159961700439453125f,-0.444399997270107149915929767303168773651123046875f,0.65030001596808428843132787733338773250579833984375f,-0.23999999463558197021484375f,-1.1489999855533243877658833298482932150363922119140625f,0.3146999922275544037120198481716215610504150390625f,-0.37669997947216049549723493328201584517955780029296875f,0.055000003576278544414890347979962825775146484375f,0.329199995788931776274921503500081598758697509765625f,-0.0642000039100647512668729177676141262054443359375f,0.688699986274540432162893921486102044582366943359375f,-0.09010000636577597532550498726777732372283935546875f,0.58950000881403685770720812797662802040576934814453125f,0.08530000061988829429537872783839702606201171875f,0.93029999607503410796738307908526621758937835693359375f,0.29749999940395355224609375f,-0.4230000032931566789073940526577644050121307373046875f,0.1880999975889920960980816744267940521240234375f,0.26409999199360612021081351485918276011943817138671875f,0.218899984812736647654674015939235687255859375f,-0.1249999957531688909995182257262058556079864501953125f,-0.15300000768899923997423684340901672840118408203125f,-0.81030000021010639699170496896840631961822509765625f,-0.293599999392032628975357511080801486968994140625f,0.454100015574693482989232506952248513698577880859375f,-0.291099998199939591358997859060764312744140625f,0.1709000081181528685192461125552654266357421875f,0.765300029379129664874881200375966727733612060546875f,0.4042999628901480235754206660203635692596435546875f,0.1415999681890003358830654178746044635772705078125f,0.035500006198883937713617342524230480194091796875f,0.3008999650537962367025102139450609683990478515625f,1.16090000245571189907423104159533977508544921875f,1.3395000021457672545466266456060111522674560546875f,-0.636200030493736079506561509333550930023193359375f,-0.2018999914586547816952588618732988834381103515625f,-0.644899969792366078991108224727213382720947265625f,-1.1021999981880181707083465880714356899261474609375f,-0.8291999383449564220427419058978557586669921875f,0.21240001792311691275472185225225985050201416015625f,1.5492999888181682166532482369802892208099365234375f,-0.085200006324053223494274789118207991123199462890625f,0.9157000150561334095300480839796364307403564453125f,-0.95070001073479648567854383145458996295928955078125f,0.5753000328063961887892219237983226776123046875f,0.1290000129342079304706203402020037174224853515625f,-0.295400020098686155733958003111183643341064453125f,-0.40739999154806127990013919770717620849609375f,0.161100005352496911115167677053250372409820556640625f,0.3078000182032578635471509187482297420501708984375f,0.436999981284142080539822927676141262054443359375f,1.450800042432546899107137505779974162578582763671875f,0.2714999706149097136176351341418921947479248046875f,0.0386999910056591289730931748636066913604736328125f,0.59230003912448836445037159137427806854248046875f,-0.070200036722422254342745873145759105682373046875f,-1.1778999832928180868663048386224545538425445556640625f,0.2095999994218349371521981083787977695465087890625f,0.4741999955654143672489908567513339221477508544921875f,0.2656999969139695050301952505833469331264495849609375f,-0.350400001472234823296503236633725464344024658203125f,0.2833000074148177560573458322323858737945556640625f,0.160799998287856293899267257074825465679168701171875f,-0.0909999972358346287393260354292578995227813720703125f,0.214999993741512351874689557007513940334320068359375f,0.0739999908804893635760890902020037174224853515625f,-0.6229999899938700735901875304989516735076904296875f,0.1294000136800110833945609556394629180431365966796875f,-0.269199985693395316133091910160146653652191162109375f,-0.009699999603629105848767721909098327159881591796875f,0.5932999908372760256014544211211614310741424560546875f,0.1103999935746191596308563021011650562286376953125f,-0.5701000286057589239163689853739924728870391845703125f,-0.1340999985873698729932357309735380113124847412109375f,0.10440000136792659812812189557007513940334320068359375f,-0.2773000114083290501554301954456605017185211181640625f,-0.3197999918505549654668129733181558549404144287109375f,0.28889998987913134698146677692420780658721923828125f,-0.808099975684285265487005744944326579570770263671875f,-0.424200008976459486120802466757595539093017578125f,-0.9696999653309585909255474689416587352752685546875f,0.066299992454052159018829115666449069976806640625f,-1.03949997178465114444634309620596468448638916015625f,-0.3454000049367547120482413447462022304534912109375f,-0.1152999979004263320092604772071354091167449951171875f,-0.00520000259876240988887730054557323455810546875f,-1.2460999981999396624132714350707828998565673828125f,-0.55499999880790706896505071199499070644378662109375f,0.38660001244544961451765630044974386692047119140625f,0.2028000198051334024285097257234156131744384765625f,-0.089600011715293081948630060651339590549468994140625f,-0.40770000170469256062233398552052676677703857421875f,0.070800007414817400785977952182292938232421875f,-0.0840000045523046079409823505557142198085784912109375f,0.193799997296929316092928274883888661861419677734375f,-0.41249998927116404701109786401502788066864013671875f,-1.6885999597549439243948654620908200740814208984375f,-0.74850004164129513295478091094992123544216156005859375f,-1.407999980270862039333223947323858737945556640625f,-0.274200006219744718549691242515109479427337646484375f,0.75349999377131471334223533631302416324615478515625f,0.79740005010962544673702723230235278606414794921875f,0.132099948906898134737275540828704833984375f,0.40139999144375304584997365964227356016635894775390625f,0.603599978604913101065676528378389775753021240234375f,1.055700035545230175415554185747168958187103271484375f,0.527400007566809048142886240384541451930999755859375f,-0.389099982196092764041850387002341449260711669921875f,-1.7662999411940578653457123436965048313140869140625f,1.4703000228226184020741129643283784389495849609375f,-0.320699973034859198151025339029729366302490234375f,0.1145999866068354577919308212585747241973876953125f,-1.77049998891353510543922311626374721527099609375f,0.942000013843178773953468407853506505489349365234375f,0.6860999858319758448743641565670259296894073486328125f,0.41399999400973275243131865863688290119171142578125f,-0.39999999959021781403833983858930878341197967529296875f,0.22359999453052881168702015202143229544162750244140625f,0.233899985930323739413694283939548768103122711181640625f,-0.380000014007091380108249722979962825775146484375f,-1.0570999762877819616591068552224896848201751708984375f,0.2336000026702880205675683100707828998565673828125f,-0.581999999277293955657341939513571560382843017578125f,-0.581699992883205396765333716757595539093017578125f,0.312300018560886183394131876411847770214080810546875f,0.51090000454187389777871430851519107818603515625f,-0.3079000332862136257716656473348848521709442138671875f,0.1381999947905541148429620079696178436279296875f,0.63189998471587927042492083273828029632568359375f,0.106999998569488496968915569595992565155029296875f,-0.82430000319778906625600711777224205434322357177734375f,-0.1808000011563299924688408282236196100711822509765625f,0.805000017918646282311101458617486059665679931640625f,0.314300001931190564619100769050419330596923828125f,0.6360999932453037519763938689720816910266876220703125f,-0.0882999989584087874305851073586381971836090087890625f,0.369400007402896857211516135066631250083446502685546875f,-0.0680000003874301484074749168939888477325439453125f,-0.31150000302493541237680574340629391372203826904296875f,0.24659999486207961894024265347979962825775146484375f,0.458299979996681428051630291520268656313419342041015625f,0.159199997204542142981154029257595539093017578125f,0.1127000030830502819156890836893580853939056396484375f,0.3405000011324883502084048814140260219573974609375f,1.182499990202486639390144773642532527446746826171875f,0.32080000979900358260010762023739516735076904296875f,-0.3332000046253202896195944049395620822906494140625f,-0.094199988245219135052366254967637360095977783203125f,0.5776999733552339311160039869719184935092926025390625f,1.356600010806321421341635868884623050689697265625f,-0.5961000133991234406494186259806156158447265625f,0.52470002173185381622033673920668661594390869140625f,-0.127200016069412669139637728221714496612548828125f,0.5882999735891818460231661447323858737945556640625f,0.7493000156402587208503973670303821563720703125f,-0.3812999923050401918089846731163561344146728515625f,-0.53530006380081118777525261975824832916259765625f,-0.5910000059902671409872709773480892181396484375f,0.558899984216690626226409221999347209930419921875f,0.15330001292824757541666258475743234157562255859375f,-1.25490001032352438414818607270717620849609375f,-0.27210002691447787714906780820456333458423614501953125f,0.968200028690695990718495522742159664630889892578125f,-1.2510000247657302452353178523480892181396484375f,-0.1452999904870981850990574457682669162750244140625f,-0.2550000162050130025903627029038034379482269287109375f,0.257800016191601866211158267105929553508758544921875f,-0.3075000138580801234411410405300557613372802734375f,-1.10530000360011992910358458175323903560638427734375f,-0.4550999887824058731666809762828052043914794921875f,0.53689998236894620475823103333823382854461669921875f,-0.8808000001132487710719942697323858737945556640625f,-0.82029995591640503249664106988348066806793212890625f,-1.50000003024935768536352043156512081623077392578125f,0.593900010442734327398284221999347209930419921875f,-0.633600014740228534293464690563268959522247314453125f,0.578099990957975595762263765209354460239410400390625f,-0.3999999967217444662992420489899814128875732421875f,0.21940002554506043974669182716752402484416961669921875f,-0.1712000024795532482357884873636066913604736328125f,0.366999992273747910331849197973497211933135986328125f,-0.274700007724761885441466802149079740047454833984375f,0.894100018703937404751513895462267100811004638671875f,-0.0565999963521957027978714904747903347015380859375f,0.5439999868944287442218410433270037174224853515625f,-0.06429999358654026053727648104541003704071044921875f,0.1604000045642255400935027864761650562286376953125f,0.0935000021159648753155124722979962825775146484375f,-0.2158999982461333733141373159014619886875152587890625f,0.09060000019669534054855830618180334568023681640625f,1.182099975505471523007372525171376764774322509765625f,0.13309999885559076204799566767178475856781005859375f,0.9417999876633287303917541066766716539859771728515625f,0.05019999762549998134630868662497960031032562255859375f,-0.361399994871020313613740881919511593878269195556640625f,-0.2379000010624527927660665227449499070644378662109375f,0.200999987214804054502792496350593864917755126953125f,0.25559999930262566891769893118180334568023681640625f,0.087100008511543036693325348096550442278385162353515625f,-0.3841000118494033443994339904747903347015380859375f,-1.12259994970411103309970712871290743350982666015625f,-0.1360000100135803080547702847979962825775146484375f,-0.4833999823391439409903114210464991629123687744140625f,0.1191999955281615175550768981338478624820709228515625f,0.071999979943037306640007955138571560382843017578125f,0.01340001065134988067484300700016319751739501953125f,0.455499978184700893280023592524230480194091796875f,0.26309997292757003606311627663671970367431640625f,0.3282000296592713084464776329696178436279296875f,0.64110001339912425777356475009582936763763427734375f,1.3997999765396116345073096454143524169921875f,0.0344999938011165685480818865471519529819488525390625f,1.45710004025101635960481871734373271465301513671875f,0.46229999233484253551296205841936171054840087890625f,1.62519994716644333720978465862572193145751953125f,-0.364500013470649530944456273573450744152069091796875f,-0.2695999754309656992745658499188721179962158203125f,-0.17160000126957886124046126496978104114532470703125f,-0.54689999339580541715122308232821524143218994140625f,-0.80790001175403602928781765513122081756591796875f,1.2650999834179881275986190303228795528411865234375f,-1.244100002610683208814634781447239220142364501953125f,0.2458000398993487323195950011722743511199951171875f,0.231300006740540187166743635316379368305206298828125f,-0.3190000024661421296201524455682374536991119384765625f,-0.1098000079438090637040659203194081783294677734375f,-0.037800012317299458430852610035799443721771240234375f,-0.23789999566078190440521211712621152400970458984375f,0.1321000177502629213677209918387234210968017578125f,1.55790002784728986995332888909615576267242431640625f,-0.448199948000908676704057143069803714752197265625f,1.19340003076791756342345252051018178462982177734375f,0.565000001788138916936077293939888477325439453125f,0.04740001438409091605308276484720408916473388671875f,0.341599964985251691729217782267369329929351806640625f,-0.3681000016868107849177249590866267681121826171875f,1.074799999822676266347798446076922118663787841796875f,0.230700002092123046537608388462103903293609619140625f,-0.0386000070288776697680077631957828998565673828125f,-0.4343000084877015609663430950604379177093505859375f,0.33859997376203576635589342913590371608734130859375f,0.2675999950967729024142727212165482342243194580078125f,0.2201999889269472543418260102043859660625457763671875f,-0.09150000650808209901043710488011129200458526611328125f,-0.4793999977916476584738347810343839228153228759765625f,0.181300005865097091373172588646411895751953125f,-0.52980001293569767284452609601430594921112060546875f,-0.22610001250505451508843179908581078052520751953125f,-0.45019999067783356139216266456060111522674560546875f,-0.030300004792213286464175325818359851837158203125f,1.10710000098645711119615953066386282444000244140625f,-0.0609999921321868754375827847979962825775146484375f,0.1153000225991007710035773925483226776123046875f,-0.18270000736713409850153766456060111522674560546875f,-0.59659999363273374939353743684478104114532470703125f,0.08259999993965017051067434294964186847209930419921875f,-0.8566000160217284165042883614660240709781646728515625f,-0.04120000069141394050120652536861598491668701171875f,-0.9548000066399573437792014374281279742717742919921875f,-0.1716000020146370985685280174948275089263916015625f,-1.032100015701353346031510227476246654987335205078125f,0.2997000016152859824813958766753785312175750732421875f,-0.5049999851360917890730206636362709105014801025390625f,0.4198000039577483022412707214243710041046142578125f,0.7144999901130792974157657226896844804286956787109375f,0.11540000907182701439523953013122081756591796875f,0.7816000312209130385099342674948275089263916015625f,-0.4275000055134297038961221915087662637233734130859375f,-0.4018000101268288659639438265003263950347900390625f,0.44280003250837385309068849892355501651763916015625f,0.76779999108314456179869011975824832916259765625f,-0.58949998974055051181863973397412337362766265869140625f,-0.08459999599456791141705025438568554818630218505859375f,0.225900003703683349698394522420130670070648193359375f,-0.7170000070258970570336032324121333658695220947265625f,-0.6471000234127046724097453989088535308837890625f,-1.710600027465819295002802391536533832550048828125f,0.948000037789344940364344438421539962291717529296875f,0.185400013840197885883753770031034946441650390625f,-0.42270002360940051033821873716078698635101318359375f,1.515299995851517422806864487938582897186279296875f,-0.59430002830624584220231554354541003704071044921875f,-0.6712999462604525291453683166764676570892333984375f,0.806700020897388458962495860760100185871124267578125f,-0.2430000047087670367318423814140260219573974609375f,0.54630003196075616056504031803342513740062713623046875f,-0.8006000289186838703159310171031393110752105712890625f,0.872400023138523383892106721759773790836334228515625f,-0.6804000347018241967589347041212022304534912109375f,1.20540003365874337504237701068632304668426513671875f,-0.51790004990100868553781765513122081756591796875f,-0.32610001645386266222459425989654846489429473876953125f,-1.081799996641277683551152222207747399806976318359375f,-1.16939999265074678902465166174806654453277587890625f,0.2403000381708135790859159897081553936004638671875f,-0.9078000410020354848938950453884899616241455078125f,0.4608000177338718028607900123461149632930755615234375f,0.507699988889694253657580702565610408782958984375f,-0.348799997520446769527779906638897955417633056640625f,0.01339999947547898528910081950016319751739501953125f,0.536599979327619269753313346882350742816925048828125f,-0.0020999981924891830686874527600593864917755126953125f,0.5915999821960926663422242199885658919811248779296875f,-0.0041000055909155008748712134547531604766845703125f,-0.37459997833669189670757759813568554818630218505859375f,-0.0753999970018863763243643916212022304534912109375f,0.372800000917911489306533212584326975047588348388671875f,-0.15829999899566171706055683898739516735076904296875f,-0.42129999070316548337444828575826250016689300537109375f,-0.0892000098705292288059354177676141262054443359375f,0.104999981150031285181967177777551114559173583984375f,0.0335000034570693827618015347979962825775146484375f,-0.768400002604722853760677026002667844295501708984375f,0.06949999704211951012666759197600185871124267578125f,-0.65999999202787884566845377776189707219600677490234375f,-0.2940999935209751559028745759860612452030181884765625f,0.2471999898433685505239054691628552973270416259765625f,-0.07769999784901748807186550038750283420085906982421875f,-0.7605000065714120349724680636427365243434906005859375f,0.12219999520778657142727752216160297393798828125f,-0.546599993260204808365188000607304275035858154296875f,-0.2926999985754490385403414620668627321720123291015625f,0.129799994011223329692938932566903531551361083984375f,0.00340000057443978764837311246083118021488189697265625f,0.92939998765885845077860949459136463701725006103515625f,0.5532999957919120248561739572323858737945556640625f,0.732400020456314138073139474727213382720947265625f,-0.3377999737977983585324182058684527873992919921875f,-0.55759998967647561585181392729282379150390625f,-0.975299990189074872404262350755743682384490966796875f,-0.3009999673962600041932091698981821537017822265625f,-0.322800002445280398433169466443359851837158203125f,0.691599986293911950241408703732304275035858154296875f,0.647399990653990897726544062606990337371826171875f,0.169799982500077106806202209554612636566162109375f,-0.315600011521577794582071874174289405345916748046875f,0.19520001511573781272090855054557323455810546875f,0.115199991273879920328226944548077881336212158203125f,-0.835099990272521797152194267255254089832305908203125f,0.80160003718137762263040713150985538959503173828125f,1.210999962925910722333355806767940521240234375f,0.08339999768733985519020279753021895885467529296875f,-1.328400014972686449254979379475116729736328125f,0.37800001242011782576213363427086733281612396240234375f,-0.66960000120997431594105364638380706310272216796875f,0.50759999288022539243314668055973015725612640380859375f,0.842399970015883514662391462479718029499053955078125f,0.453600017664582022547392625710926949977874755859375f,-0.9279000104501846646343210522900335490703582763671875f,0.21150001442432397880111238919198513031005859375f,-0.62099999630451208076920011080801486968994140625f,0.62730000537335894872370545272133313119411468505859375f,0.783399981817602952816059769247658550739288330078125f,-0.548100013142079234285120037384331226348876953125f,1.0373999950870873565378360581235028803348541259765625f,-0.77399997642636275685390501166693866252899169921875f,-0.8978999895513057882823204636224545538425445556640625f,0.3144999905228615943997283466160297393798828125f,-0.4799000014573333583456360429408960044384002685546875f,0.12970000385046009938605493516661226749420166015625f,0.2989000010550022512489931614254601299762725830078125f,0.0350999927684665369298500081640668213367462158203125f,-1.0282999831259249656767451597261242568492889404296875f,0.3089999977350235127460109652020037174224853515625f,-0.25649999330192818636220408734516240656375885009765625f,0.1833000059247016366725802072323858737945556640625f,0.7184999771565199200296092385542578995227813720703125f,-0.0551000051736831863991028512828052043914794921875f,-0.112000006392598105975366706843487918376922607421875f,0.031000000700354579663553522550500929355621337890625f,1.14069997452497506884583344799466431140899658203125f,-0.4160000074803829051006687222979962825775146484375f,-0.3001000002562996815669293937389738857746124267578125f,0.325100006663799234729594900272786617279052734375f,0.52689999510943863469947245903313159942626953125f,0.2230999958753585588056012056767940521240234375f,-0.16270001533925515513345771978492848575115203857421875f,-0.498800001096725509341922588646411895751953125f,0.01790000530928370636729596299119293689727783203125f,-0.18029999853372569162957006483338773250579833984375f,0.1362000012129545023498167211073450744152069091796875f,0.1420000003203749638913677699747495353221893310546875f,0.370599995428323669699466336169280111789703369140625f,0.32540000169575211685923932236619293689727783203125f,0.223699997748434409317042081966064870357513427734375f,-0.2385000165700912333477390347979962825775146484375f,-0.5941000127434730160302933654747903347015380859375f,0.1659000118434423853841508389450609683990478515625f,-0.33450003194808886064492980949580669403076171875f,-0.841500023812055797378661736729554831981658935546875f,-0.4404999844431873867733884253539144992828369140625f,0.5835000128820535536533498088829219341278076171875f,-1.134899999669193970674996307934634387493133544921875f,-0.20099999079108243904556729830801486968994140625f,-0.4182999883413316410951665602624416351318359375f,-0.55890000463128064467355216038413345813751220703125f,0.822299979519844015385388047434389591217041015625f,0.68880002195835121625577812665142118930816650390625f,0.67519997458457936545528355054557323455810546875f,-0.53070000887215140750186037621460855007171630859375f,1.50089997369647054625829696306027472019195556640625f,0.77610000967383374614882995956577360630035400390625f,0.01849998676776909434238405083306133747100830078125f,-0.7437000044167039192188894958235323429107666015625f,0.35950001525878860775264911353588104248046875f,-0.08009998237490645323077842476777732372283935546875f,-0.168500021934509192078621708787977695465087890625f,0.891000006884336670509583200328052043914794921875f,0.6961000014781948408426615060307085514068603515625f,0.598199992704391281250764222932048141956329345703125f,0.884599971854687083805401925928890705108642578125f,-0.243599999131262290319455132703296840190887451171875f,0.5405999986320733086131440359167754650115966796875f,-0.701399998521804679540991855901665985584259033203125f,-0.166499981045723277617298663244582712650299072265625f,1.28250004082918156456116776098497211933135986328125f,-1.0627999684706332939043704755022190511226654052734375f,-0.003900000011920923270736238919198513031005859375f,0.055900009422003904546727426350116729736328125f,0.04239999586939813269736987422220408916473388671875f,0.619699976246059325291071218089200556278228759765625f,-0.107999996252357954862333144774311222136020660400390625f,-0.44950000264868139598917196053662337362766265869140625f,-0.2389999938607214602370731881819665431976318359375f,0.8639000088036059565865798504091799259185791015625f,0.04929999671578411124528429354541003704071044921875f,0.7771999755382541508197391522116959095001220703125f,0.07179999954700466702206540503539144992828369140625f,-0.0594000085949897282944220933131873607635498046875f,-0.4199999886751175637300548260100185871124267578125f,0.3277999844148755670403261319734156131744384765625f,0.0585000143349170276252380062942393124103546142578125f,0.6919999809861183681647389676072634756565093994140625f,0.304499990522861363473339224583469331264495849609375f,-0.5294000083342194074020881089381873607635498046875f,0.242200011003017579014340299181640148162841796875f,0.838500017911195438813365399255417287349700927734375f,0.160099997723102571711706332280300557613372802734375f,-0.05679999831765891737944684791727922856807708740234375f,-0.367100003743171754422291996888816356658935546875f,0.1605999784037475119902182996156625449657440185546875f,0.13860000281929973908745523658581078052520751953125f,0.850600003512203439726135911769233644008636474609375f,-0.06240000533163547746795529747032560408115386962890625f,-0.40219999606460332675084146103472448885440826416015625f,-0.10230000291466723183475551195442676544189453125f,-0.914399980878830120900602196343243122100830078125f,0.7368000107228755979349443805404007434844970703125f,1.124499989628791496443227515555918216705322265625f,-0.23639999703168879108261535293422639369964599609375f,0.916699967253208569673006422817707061767578125f,0.76950000011548402856220718604163266718387603759765625f,0.07509995933398638801037350276601500809192657470703125f,-0.02190004093051012290516155189834535121917724609375f,1.36169998960495064466158510185778141021728515625f,-0.19260000144839306557287272880785167217254638671875f,1.289199958312511995472959824837744235992431640625f,0.051000009417534020172979580820538103580474853515625f,0.0385999874711036028429589350707828998565673828125f,0.50369998400211368760892582940869033336639404296875f,0.847799962472915780153925879858434200286865234375f,-0.94200003306567647864966374982031993567943572998046875f,0.096799988892674715401653884327970445156097412109375f,-0.1670999550163740909880516483099199831485748291015625f,-0.90299997892975891744526961701922118663787841796875f,-1.0461000540792948498847181326709687709808349609375f,0.4463000207662588536550174467265605926513671875f,-0.2894999951422214667928756171022541821002960205078125f,-0.32679998717904101823705786955542862415313720703125f,0.26370000903606438669157796539366245269775390625f,0.253399978017807114838433335535228252410888671875f,-0.57390001595616357832341236644424498081207275390625f,1.0050000157952307944242420489899814128875732421875f,0.62940001809448009595371331670321524143218994140625f,0.0245999917849897276056481132400222122669219970703125f,0.486600027197599871442434960044920444488525390625f,0.5403000092998144765488177654333412647247314453125f,0.4168999916076661094166411203332245349884033203125f,-0.37949998378008620303347697699791751801967620849609375f,-0.1809999994710087545257692909217439591884613037109375f,-0.610799981337785791168926152749918401241302490234375f,0.1648999975085256863849281216971576213836669921875f,-0.44140002243816855997948778167483396828174591064453125f,0.04680000282526020072282335604541003704071044921875f,-0.0194999890327454128424733426072634756565093994140625f,0.170899982190132249115777085535228252410888671875f,-0.46919999791234712205323376110754907131195068359375f,-0.083299991321563737756150658242404460906982421875f,0.850199988889694058258328368538059294223785400390625f,-0.3666999949693678928497320157475769519805908203125f,0.38470003499388660817004392811213620007038116455078125f,-0.14969999563619496019128973784972913563251495361328125f,-0.4150999803632500206873601200641132891178131103515625f,0.00659999366998675895956694148480892181396484375f,-0.58849999343603831558624506214982829988002777099609375f,0.4357999996662140773651117342524230480194091796875f,0.030999993100762512909795987070538103580474853515625f,0.2781999879360199656730401329696178436279296875f,-0.1647000083208081111507681271177716553211212158203125f,0.106399991518259184886119328439235687255859375f,-0.292099978671968074639408996517886407673358917236328125f,-0.0374999958276749367769298260100185871124267578125f,1.10389997453093524626410726341418921947479248046875f,-0.235000006854534149169921875f,0.3967000004455447648155086426413618028163909912109375f,-0.100199996154010317894744730438105762004852294921875f,0.649700015994906454608326384914107620716094970703125f,-0.0431999862968925896211658255197107791900634765625f,-0.312899988508224424776926753111183643341064453125f,0.373799994726478956863502389751374721527099609375f,-0.868399982562661154616989733767695724964141845703125f,0.7662000489711768835832117474637925624847412109375f,0.3997999622344963199793710373342037200927734375f,-0.552900018757581879214058062643744051456451416015625f,-0.7500999720931051939487588242627680301666259765625f,0.8145000340342516409464224125258624553680419921875f,0.419999967813492247614703956060111522674560546875f,0.1451999862074853098192761535756289958953857421875f,1.745799970459938066369431908242404460906982421875f,-0.02010003631711043681207229383289813995361328125f,0.292900052285194334444895503111183643341064453125f,0.175199992074817512133222408010624349117279052734375f,-0.7300999695226553232174637741991318762302398681640625f,-0.01679999403357523846125332056544721126556396484375f,-1.1510999883651731323652711580507457256317138671875f,-0.80040001205205957290900187217630445957183837890625f,-0.5486000139951698884033248759806156158447265625f,-0.1688999819815162339864400564692914485931396484375f,-0.2321000156641002121205019648186862468719482421875f,0.368400002753733968319238556432537734508514404296875f,0.069300004315376639851820073090493679046630859375f,-0.360900002604722658361424691975116729736328125f,1.586499981045722762473815237171947956085205078125f,0.4038000116765498859194849501363933086395263671875f,1.054900003767013316746670170687139034271240234375f,0.0533999938875437951679714387864805757999420166015625f,-0.2575999957486987401722444701590575277805328369140625f,0.20030000360012056859204676584340631961822509765625f,0.6895999970749020491211922490037977695465087890625f,-0.1267999888181687850874368450604379177093505859375f,0.29629998347610231324011920150951482355594635009765625f,0.20109999536871914216362711158581078052520751953125f,0.8178999874278904957947133880225010216236114501953125f,-0.352299988646805228853509106556884944438934326171875f,-0.485099979059398389580337607185356318950653076171875f,-0.10209999904930600056474077064194716513156890869140625f,-0.50040001555383195608328605885617434978485107421875f,-0.228799994242191218063453561626374721527099609375f,0.4269000100851056966888563692918978631496429443359375f,-0.010300000582635415735666128966840915381908416748046875f,-0.187100001265853632670399520065984688699245452880859375f,0.423400002157688248871636460535228252410888671875f,-0.55570000194311131025415306794457137584686279296875f,0.5756999965786935291589543339796364307403564453125f,-0.215699979852140177882802163367159664630889892578125f,0.0045999974846839819520027958787977695465087890625f,-0.25930001552849990531512958114035427570343017578125f,0.0795000006556509930533138685859739780426025390625f,-0.69700000452995303223957535010413266718387603759765625f,-0.1832000096917152376363446819595992565155029296875f,0.760699981603026653687038560747168958187103271484375f,-0.22989999586939813269736987422220408916473388671875f,0.022400009131431530828848508463124744594097137451171875f,0.0172000016152857870821435426478274166584014892578125f,0.59619998534321794192436527737299911677837371826171875f,-0.30299999874830252366564309340901672840118408203125f,-0.27960001916587362558175300364382565021514892578125f,0.54000000372529033398905085050500929355621337890625f,-0.93339999522864758318974054418504238128662109375f,0.72839999917745590352069484652020037174224853515625f,-1.05099998110532766304459073580801486968994140625f,-0.404299993549287306482398207663209177553653717041015625f,-0.631000009603798428514664919930510222911834716796875f,0.72619998398348695189952195505611598491668701171875f,0.676099979145079910125559763400815427303314208984375f,-0.23410001989603035355003157746978104114532470703125f,-0.70009998013973273600640823133289813995361328125f,0.66740000153183931530520567321218550205230712890625f,-0.0627999627709385777052375487983226776123046875f,0.4737999933481216174868677626363933086395263671875f,-0.93099998192489152870621182955801486968994140625f,0.12710001015067096119537382037378847599029541015625f,-0.47010002067089118327203323133289813995361328125f,-0.1002999843776226640557069913484156131744384765625f,-0.60050000083446519738572533242404460906982421875f,0.0738999983541667460062996042324812151491641998291015625f,-0.0961999978974461544822815994848497211933135986328125f,0.947399979329109243053608224727213382720947265625f,-0.10489998916387577310160850174725055694580078125f,0.9368999688088894828297270578332245349884033203125f,0.488699981021881058040889911353588104248046875f,0.35569998890459542106867729671648703515529632568359375f,0.22060000101625920621017939993180334568023681640625f,0.34190000829696653994460575631819665431976318359375f,-0.16190003154277832209118059836328029632568359375f,0.164199978876113927839242023765109479427337646484375f,-0.032800000917911464881626670830883085727691650390625f,0.0588999969944357104623122722841799259185791015625f,0.0419000083714724524952544015832245349884033203125f,0.41710003626495628648029878604575060307979583740234375f,-0.55390001103878017119086507591418921947479248046875f,0.17570000402927377081141457892954349517822265625f,-0.287299997997283895756481797434389591217041015625f,-0.17580000864416389649846905740560032427310943603515625f,0.34500001996755624844581689103506505489349365234375f,-0.1504999864175917689834705015528015792369842529296875f,-0.2778999931648371557457721792161464691162109375f,-0.5726000046148893574127214378677308559417724609375f,-0.237400008237361959118061349727213382720947265625f,0.32430000170767303568908346278476528823375701904296875f,0.14010001247525227796586477779783308506011962890625f,0.73310001193136020702922905911691486835479736328125f,0.548500002264976416199715458787977695465087890625f,0.365800022092461318834466510452330112457275390625f,-0.5529999941289422960011279428726993501186370849609375f,0.2543000061407683087821851586340926587581634521484375f,0.05530000956058511718538284185342490673065185546875f,-0.2860000111684204426865107961930334568023681640625f,0.26080000078380105588138349048676900565624237060546875f,0.4177000215604904642674455317319370806217193603515625f,-0.05770000915527351281752999057061970233917236328125f,-0.477600007073581245276017170908744446933269500732421875f,-0.303600010716915225117418231093324720859527587890625f,0.2130999988928434785151466712704859673976898193359375f,-0.3499000070452691346645224257372319698333740234375f,-0.063100002618134087839507628814317286014556884765625f,0.3102000131785871417378075420856475830078125f,0.3939999949038028859149562777020037174224853515625f,-0.80189998989403221685279277153313159942626953125f,0.916600027495622526885199476964771747589111328125f,0.29169997500181210625669336877763271331787109375f,0.3313999874949455914929785649292171001434326171875f,0.3594999848604203407376189716160297393798828125f,-0.16310000004768365755580816767178475856781005859375f,-0.8560999964117996796630905009806156158447265625f,-0.211700002568960155002741885255090892314910888671875f,0.65219997810870466992128058336675167083740234375f,-0.1525000053644181008394298260100185871124267578125f,-0.7182999853610994023256353102624416351318359375f,0.6431999956846237154195478069595992565155029296875f,-0.83929997853636706395263900049030780792236328125f,0.1040000071227549227614872506819665431976318359375f,0.5991000547647473695178632624447345733642578125f,0.7388999956160784421399512211792171001434326171875f,0.5434999653846031009152284241281449794769287109375f,-0.298000015288591324491562772891484200954437255859375f,1.236700001227855949537115520797669887542724609375f,-0.7974999978765844321770828173612244427204132080078125f,0.428899991033971250686818166286684572696685791015625f,0.189400017386674779373834098805673420429229736328125f,0.844300003421306399786772090010344982147216796875f,-0.3581000049650668870526715181767940521240234375f,0.138299980145692558153314166702330112457275390625f,0.108699981915950871780296438373625278472900390625f,1.0889999750852581428262055851519107818603515625f,-0.5652999989807605540903523433371447026729583740234375f,0.435300018203258698434865436865948140621185302734375f,0.4271000000923874662106527466676197946071624755859375f,0.3972999938249586193705908954143524169921875f,-0.51079999113529883647544238556292839348316192626953125f,0.2335999987959862522757248370908200740814208984375f,0.626300031457841743787184896063990890979766845703125f,-0.3671000094056129370301277958787977695465087890625f,0.19690000233650206684643535481882281601428985595703125f,0.137900004899501826827190598123706877231597900390625f,0.34630000366717583748510378427454270422458648681640625f,-0.71350001686811470591464967583306133747100830078125f,-0.013500003457069364998233140795491635799407958984375f,0.30510000979304319201901307678781449794769287109375f,0.421600028315186892147181652035214938223361968994140625f,-0.202000007994472985384248886475688777863979339599609375f,-0.275199995073676095813652864308096468448638916015625f,-0.171199998828768684688839130103588104248046875f,0.466800015863776340818702692558872513473033905029296875f,-0.2004000023663043972277364446199499070644378662109375f,0.312299997252225569610573074896819889545440673828125f,0.56250002950429944803545367904007434844970703125f,0.3159000050261615566427053636289201676845550537109375f,-0.44550000180304050179103114714962430298328399658203125f,0.238900019755959647227427922189235687255859375f,-0.15610000296831127997165822307579219341278076171875f,-0.1069999905973672282843978109667659737169742584228515625f,-0.472600010873377385234306302663753740489482879638671875f,0.0424000130802397734441910870373249053955078125f,-0.25120002095699334176970296539366245269775390625f,-0.129399995370209186518195565440692007541656494140625f,-0.902700003939866491009524906985461711883544921875f,-0.04799998861551291184923684340901672840118408203125f,0.42350002133846231799907400272786617279052734375f,-0.961099980616569649072289394098334014415740966796875f,0.235700017142296047722993534989655017852783203125f,1.34170003281831728969564210274256765842437744140625f,-0.47840001318454739021035493351519107818603515625f,-0.0562000053107738484214905838598497211933135986328125f,0.3675000126659870858247813885100185871124267578125f,1.013299976122379320031541283242404460906982421875f,0.22519999455213479677695431746542453765869140625f,-0.519600017154216775594477439881302416324615478515625f,0.462500035613775395404445589520037174224853515625f,0.59920002207085476531034373692818917334079742431640625f,0.255200005299597965091606965870596468448638916015625f,-0.68590002108216285847674953402020037174224853515625f,1.1345000113099814598172088153660297393798828125f,0.76810000548660750663287899442366324365139007568359375f,0.00090000193417072438251125277020037174224853515625f,-0.1911000171244143785997948725707828998565673828125f,0.0740000329017647118234890513122081756591796875f,1.0988999928966165509081065465579740703105926513671875f,-0.620400014175474634470219825743697583675384521484375f,0.039300007742643305874707948532886803150177001953125f,0.898999995648861016661612666212022304534912109375f,0.01299999552592634444181385333649814128875732421875f,0.689199981800466598969023834797553718090057373046875f,-0.180599995279312253160242107696831226348876953125f,1.053000009775161061043036170303821563720703125f,-0.822800016713142401414415871840901672840118408203125f,0.11880001599788680977098920266143977642059326171875f,0.17520002983063509560679449350573122501373291015625f,-0.8628000319123270145382775808684527873992919921875f,-0.0326000096067782241249233265989460051059722900390625f,0.1081999956846235733110006549395620822906494140625f,-0.25590000163614734507433468024828471243381500244140625f,-0.26610001518726367208955707610584795475006103515625f,0.7088000188291074810109648751677013933658599853515625f,-0.1760000016689300395000827847979962825775146484375f,0.0624000044375658813322615969809703528881072998046875f,-0.5592000197052957588539356947876513004302978515625f,-0.1733000016778707408349191609886474907398223876953125f,0.20569999851584430672346570645458996295928955078125f,-0.442100025536120266878015172551386058330535888671875f,0.082100001358985874588825026876293122768402099609375f,0.3044000069186092449768921142094768583774566650390625f,0.1744000050187111128252581693232059478759765625f,-0.68610002211630372048745130086899735033512115478515625f,0.05610000147819516058689259807579219341278076171875f,0.0484999865815044783889931068188161589205265045166015625f,0.3330999979615210104810785196605138480663299560546875f,-0.2273000198647381342453854813356883823871612548828125f,0.234500003635883302877118694595992565155029296875f,0.62269999980479429080304498711484484374523162841796875f,0.03250001817941683412982456502504646778106689453125f,0.3977999934673306370314094237983226776123046875f,0.3582000177383424244226262089796364307403564453125f,-0.65630001245886138150353872333653271198272705078125f,0.0443000078916551132124368450604379177093505859375f,0.204099992533773200165114758419804275035858154296875f,-0.852199981200694622884839191101491451263427734375f,1.17230003793239578868679018341936171054840087890625f,-0.27530004234314020550300483591854572296142578125f,0.303799986046552561447242624126374721527099609375f,-0.9935999460458759813263895921409130096435546875f,-0.37850000688433649287389926030300557613372802734375f,0.7087999951362604633686714805662631988525390625f,1.4250000175833701376859607989899814128875732421875f,-0.846199997711181595150264911353588104248046875f,0.6357000065624713869283368694595992565155029296875f,-0.0431000328302388879819773137569427490234375f,-0.912400000190734861149621792719699442386627197265625f,1.109100000375509154082465101964771747589111328125f,-0.2037000035226343985783614698448218405246734619140625f,-0.9314000225126743970349707524292171001434326171875f,-0.02820000566840175082461428246460855007171630859375f,0.52100002461671834907974698580801486968994140625f,-1.20860002561807622356582214706577360630035400390625f,0.68489999512433996642357669770717620849609375f,-0.10350000278651716456579379155300557613372802734375f,-0.3501999748080972807429134263657033443450927734375f,-0.17670000569820398794718130375258624553680419921875f,-0.73039999625682838768625515513122081756591796875f,0.8920999891400338555058624478988349437713623046875f,-0.34600001329183527332133962772786617279052734375f,0.38519999485015876672377999057061970233917236328125f,-0.53299999129772146488903672434389591217041015625f,-1.19840000558495507476663988200016319751739501953125f,0.76889995918273967845379956997931003570556640625f,-0.5714000047802925141837704359204508364200592041015625f,-0.119899993764609025248546458897180855274200439453125f,0.1316000058144328743736650721984915435314178466796875f,-0.714800000083446462895153672434389591217041015625f,0.35650000023096783952070154555258341133594512939453125f,-0.147700000214576743218231058563105762004852294921875f,-0.1089999975115059083208279844257049262523651123046875f,-0.798200020867586346184907597489655017852783203125f,-0.129500022225081945492775048478506505489349365234375f,0.2303000021100043426258707768283784389495849609375f,-0.1718000076681376331322326223016716539859771728515625f,-0.101799996566772410488965761032886803150177001953125f,-0.32880000195354230907440751252579502761363983154296875f,-0.23640000090599055937445882591418921947479248046875f,0.37249999430030555913617718033492565155029296875f,0.02350000464916224274247724679298698902130126953125f,0.5332999885275955254115842762985266745090484619140625f,-0.1300999822258948057651650742627680301666259765625f,-0.448300014343857800014347958494909107685089111328125f,0.545500009179115341595434074406512081623077392578125f,0.209599997894465761572035944482195191085338592529296875f,0.298000012308359263357715462916530668735504150390625f,0.016400002582371120407600528778857551515102386474609375f,-0.264800002169609083324530729441903531551361083984375f,0.6741000065967437393510408583097159862518310546875f,0.29070002228319669068667963074403814971446990966796875f,0.23740001937597987602401872209156863391399383544921875f,-0.52900000131130209268803810118697583675384521484375f,-0.456899996078014070111095179527183063328266143798828125f,0.2084999930262565470684421597979962825775146484375f,-0.382000023454427850566617053118534386157989501953125f,-0.5982999832935629314079051255248486995697021484375f,0.80769996817708022973647530307061970233917236328125f,0.09729998905658732155643519945442676544189453125f,0.7302999998748302967754852943471632897853851318359375f,-0.3292000156819820944065213552676141262054443359375f,0.76149998626112935795617886469699442386627197265625f,-0.27910001259446204358027898706495761871337890625f,0.16029998959302904637524989084340631961822509765625f,-1.20119997923374199899626546539366245269775390625f,0.3497999939739704355901039889431558549404144287109375f,-1.0267999988019465007482722285203635692596435546875f,-0.11150000637769696965051480219699442386627197265625f,-0.1455000103712080772311310283839702606201171875f,-0.016000025361776248900014252285473048686981201171875f,1.002500001490115977276218472979962825775146484375f,1.357499968111515187274562777020037174224853515625f,-1.2739999923706051276894868351519107818603515625f,-1.3391000088691711322752553314785473048686981201171875f,0.5774000247776509553432333632372319698333740234375f,0.09160000201463702751425444148480892181396484375f,-0.267599994742869995434375596232712268829345703125f,0.00609998762011532136284586158581078052520751953125f,0.597299989205598791386364609934389591217041015625f,0.734600000651180806698903325013816356658935546875f,-0.40849999485164811829918107832781970500946044921875f,0.766399999266862863578353426419198513031005859375f,0.08059998246431376855980488471686840057373046875f,-0.8234999738037587224681601583142764866352081298828125f,0.1315999993324279415674027404747903347015380859375f,-0.9094000175356864446030158433131873607635498046875f,-0.2232000031352043833976495079696178436279296875f,0.24709999439269292320631166148814372718334197998046875f,0.36490001419782647218426063773222267627716064453125f,0.172100010187924146176641215788549743592739105224609375f,-0.215300010156631582702857485855929553508758544921875f,-0.6877000171273948136985154633293859660625457763671875f,-0.5327000115394593393602917785756289958953857421875f,0.4857000226929784059137773510883562266826629638671875f,-0.47130000228881829826832472463138401508331298828125f,0.6149999999627471236607334503787569701671600341796875f,0.2022000065326690076972226961515843868255615234375f,-0.6781000034004449616986676119267940521240234375f,-0.3551000096440315445533997262828052043914794921875f,-0.3809999894872309056381709524430334568023681640625f,-0.616199997413158229164764634333550930023193359375f,-0.2082999933332205788616420250036753714084625244140625f,-0.47209999971985805444774086936376988887786865234375f,-0.1644000118359925100008922527194954454898834228515625f,0.7368000180244447250288430950604379177093505859375f,-0.41360000404864571965646291573648341000080108642578125f,-0.24519999723434438010372105054557323455810546875f,-0.76979998808801131104928572312928736209869384765625f,-0.4830000106692313721623577293939888477325439453125f,0.342000015966594350658169787493534386157989501953125f,-0.496799999845027873135450136032886803150177001953125f,-0.053699988770485262978837681657751090824604034423828125f,-0.194699991035461206223544650129042565822601318359375f,-0.25169999664574849163045655586756765842437744140625f,-0.20910000916719440056112944148480892181396484375f,-0.0392999918356540245412134026992134749889373779296875f,0.89009999325275401815815712325274944305419921875f,-0.0277000127315520927595571265555918216705322265625f,-0.04379997338056540456818765960633754730224609375f,0.1896999998271464704657773836515843868255615234375f,0.509800005894899044278645305894315242767333984375f,-0.0836000190615653337999901850707828998565673828125f,-0.554100022578239759241114370524883270263671875f,1.001600004547834377177650821977294981479644775390625f,0.114099984133243737005614093504846096038818359375f,0.2853999755442142571837393916212022304534912109375f,0.6148000397205350964213721454143524169921875f,-0.42269999827742577025446735206060111522674560546875f,-0.15600002253055578194107511080801486968994140625f,0.2325999886333942612282044137828052043914794921875f,-0.2388000186800951496479683555662631988525390625f,0.436599996650218979965529797482304275035858154296875f,-0.527300019007920894864582805894315242767333984375f,-1.0866000012695788967675980529747903347015380859375f,0.384200020599365643647615797817707061767578125f,-0.178900006866455019149952931911684572696685791015625f,-0.208900020873546310440360684879124164581298828125f,-0.31229999725222601369978292495943605899810791015625f,-0.37769999008178700705684605054557323455810546875f,1.0467999840497970165387187080341391265392303466796875f,-0.3239999835789202364821903756819665431976318359375f,-0.155699996876716539873086730949580669403076171875f,0.614599964553118383037144667468965053558349609375f,0.069800015431642492558239609934389591217041015625f,-0.39510001798868188416236080229282379150390625f,-0.59649998998641962089095613919198513031005859375f,0.098900004780292594119828208931721746921539306640625f,-0.3819000017404554636613056572969071567058563232421875f,0.03989999184608450377709232270717620849609375f,0.68030002688318502723774372498155571520328521728515625f,0.16210000821352021915799923590384423732757568359375f,0.49570000328421581770044213044457137584686279296875f,0.61670001523494732964536524377763271331787109375f,0.1628000066921113653251040886971168220043182373046875f,-0.346400012230873244334361515939235687255859375f,-0.40429999537467986581162904258235357701778411865234375f,-0.36969999133348441233692938112653791904449462890625f,0.538799991485476414965205549378879368305206298828125f,-0.17159999277591708732870756648480892181396484375f,-0.375300017867982393937609231215901672840118408203125f,0.59850000941753389582800082280300557613372802734375f,-0.03109999853521594470606714821769855916500091552734375f,0.41270001690387747572685839259065687656402587890625f,0.02869998750388613739659149359795264899730682373046875f,0.33490001300573357667644813773222267627716064453125f,-0.5924000047355890696820779339759610593318939208984375f,-0.278800009739399001773563213646411895751953125f,-0.28210000631362197598406282850191928446292877197265625f,0.0373000014245510325139321139431558549404144287109375f,-0.6987999858230351524213119773776270449161529541015625f,0.014800004255771881389591726474463939666748046875f,-0.31349999902397396311926058842800557613372802734375f,-0.0537000027775764721127416123636066913604736328125f,0.437499992400407489157032614457421004772186279296875f,0.365300020289421212282832129858434200286865234375f,0.631700011099874725317704360350035130977630615234375f,-0.37630000273585295644807047210633754730224609375f,-0.87429999969601635001481554354541003704071044921875f,0.80940000323057148534644511528313159942626953125f,0.6325000283122061972562732989899814128875732421875f,0.10959999927282382259363657794892787933349609375f,-0.07779998542070387657076935283839702606201171875f,0.207900009965896970243193209171295166015625f,-0.12740000242590898693606504821218550205230712890625f,1.46739996442794762288031051866710186004638671875f,-0.0761000075876712145372948725707828998565673828125f,-0.14520002614259652773398556746542453765869140625f,1.3469999651908874938044391456060111522674560546875f,-1.1469999639987946693509002216160297393798828125f,-1.100600008839368815216630537179298698902130126953125f,0.134900031483173421520405099727213382720947265625f,-0.36589999470710754536639797152020037174224853515625f,-0.1561999857902520716379513032734394073486328125f,0.1900999919116497238746887887828052043914794921875f,-1.05879997397661185232209390960633754730224609375f,-0.54829999772906301558350605773739516735076904296875f,0.19329997224807726752260350622236728668212890625f,-0.18930000304877758754429351029102690517902374267578125f,-0.43059999267160886660121832392178475856781005859375f,-1.02729997668862349513574372394941747188568115234375f,0.33669998364448616712252260185778141021728515625f,-1.03260001456141470299598950077779591083526611328125f,1.54709999986886970191335421986877918243408203125f,0.198999977767467584044425166212022304534912109375f,-0.027099974536895388155244290828704833984375f,0.67470000563859944264777368516661226749420166015625f,-0.554300004613399455166700136032886803150177001953125f,-0.3326000126242638632589887492940761148929595947265625f,0.138200005519390156649706113967113196849822998046875f,-0.6110000068843366438642306093242950737476348876953125f,-0.19540000422894954201780137736932374536991119384765625f,-0.456800025512278384809405906707979738712310791015625f,-0.0436000011801720432913498370908200740814208984375f,0.169799997401237579008892453202861361205577850341796875f,0.13259999898970120657537563602090813219547271728515625f,0.4704000290021301822207533405162394046783447265625f,-0.62519999902248368783830301254056394100189208984375f,0.4062000027775762855952734753373079001903533935546875f,-0.489000009171664806739698860837961547076702117919921875f,-0.0821000061646103007628738623679964803159236907958984375f,0.3413000030338764201331969161401502788066864013671875f,-0.5757000019803639734305988895357586443424224853515625f,-0.12990001747608204141215537674725055694580078125f,-0.04020000043809413003970121280872263014316558837890625f,0.2345999997943639669983895146287977695465087890625f,0.0290000038817523897449746073107235133647918701171875f,0.169000013306737084661790504469536244869232177734375f,0.031900016865133995480618978035636246204376220703125f,-0.762699991869926208210017648525536060333251953125f,0.4117000138193365188499228679575026035308837890625f,-0.1560999835968015503340211580507457256317138671875f,0.30590004160851247405616959440521895885467529296875f,-0.7379000043034551481468952260911464691162109375f,0.1719000052794814603629447447019629180431365966796875f,0.28170000547468665441641633151448331773281097412109375f,0.16130001610964506131296047897194512188434600830078125f,0.987499981969595097552883089520037174224853515625f,-0.2716000057399272638036791249760426580905914306640625f,-0.6017999943315981425939753535203635692596435546875f,-0.68090000539869066642495454289019107818603515625f,0.25809998108595610943893916555680334568023681640625f,-0.33390001789331424930651337490417063236236572265625f,-0.41430001444816610955967917107045650482177734375f,0.13260000021904705391762036015279591083526611328125f,-0.11359998832792062017915668548084795475006103515625f,1.337799996894597942542759483330883085727691650390625f,-0.343799996626376724861984257586300373077392578125f,1.2968000058613717850874991199816577136516571044921875f,-0.7194000380061569277501121177920140326023101806640625f,-0.275499992415308980753252399154007434844970703125f,-0.3431999856263399095723798382095992565155029296875f,-0.97880000243783005231534843915142118930816650390625f,0.87470000832080785357902641408145427703857421875f,-0.450299988251924521165392434340901672840118408203125f,0.386699994447827322119337623007595539093017578125f,-0.6177000221192836537653647610568441450595855712890625f,1.0869000247627500499447705806232988834381103515625f,0.904899978882074407238178537227213382720947265625f,-0.288000016629695210212958045303821563720703125f,-0.05040002933740606039236809010617434978485107421875f,1.56969999997615872189271613024175167083740234375f,0.982900027549266752657786128111183643341064453125f,0.234399954652786846054368652403354644775390625f,-0.486600002573430556740419206107617355883121490478515625f,0.945700017701089468147301886347122490406036376953125f,0.44839999604821212386696061003021895885467529296875f,-0.1168000004410743031257879920303821563720703125f,-0.5925000015646217921272409512312151491641998291015625f,0.131699995975196382236305225887917913496494293212890625f,0.4389000434115530868695032040704973042011260986328125f,-0.2660999979019162964277711580507457256317138671875f,-0.657400018630922122753190706134773790836334228515625f,-0.246200007247924901321312063373625278472900390625f,0.786699996459483852362382094725035130977630615234375f,0.4769000145554542768877581693232059478759765625f,-0.7771000023275609347450654240674339234828948974609375f,-0.3929000329136851377143102581612765789031982421875f,-0.2164999922588464187356294132769107818603515625f,0.2452999988317490220879335538484156131744384765625f,0.06310001517236252244202887595747597515583038330078125f,-0.56350001478195199666743064881302416324615478515625f,0.216799998094141244120436340381274931132793426513671875f,0.506800021675229306339360846322961151599884033203125f,-0.1368999987974761634035303359269164502620697021484375f,-0.00710000165700908070220975787378847599029541015625f,0.21240000019073479453624031521030701696872711181640625f,0.18360000840723522674835521684144623577594757080078125f,0.56670002782642880934105278356582857668399810791015625f,-0.347100011193752333582551727886311709880828857421875f,-0.4903000027060506749165824658120982348918914794921875f,0.16269999663829803893122516456060111522674560546875f,0.2323000054851174223102816540631465613842010498046875f,-0.253899994051456356913831768906675279140472412109375f,-0.300100005881488218761887765140272676944732666015625f,0.30730000485181818703495082445442676544189453125f,-0.2495000106766820380244098487310111522674560546875f,0.506300012421608158774688490666449069976806640625f,-0.80799998326972133355639016372151672840118408203125f,-0.108900002377480209503346486599184572696685791015625f,0.1356000216543673531077729421667754650115966796875f,1.466899986243248577011399902403354644775390625f,-0.846600019747018706084418226964771747589111328125f,0.60719997255802127256174571812152862548828125f,0.2710999891102314762747482745908200740814208984375f,0.5282000177383423533683526329696178436279296875f,-0.4326999690711499368944714660756289958953857421875f,-0.32340003345012746649445034563541412353515625f,-0.20579997031092656101236570975743234157562255859375f,-0.65949998962879163855177466757595539093017578125f,0.507600003981590219837016775272786617279052734375f,0.25189998356103870946753886528313159942626953125f,-0.0529999960660934021916546043939888477325439453125f,-0.373799988803266813874870422296226024627685546875f,0.432500022053718513603826067992486059665679931640625f,-1.317200006830692160519902245141565799713134765625f,0.2215000020191073559772121370770037174224853515625f,-0.30329999411553121291262868908233940601348876953125f,1.1594000248372553496523096328019164502620697021484375f,-0.2799000030219556123256552382372319698333740234375f,-0.5367999855399132602684630910516716539859771728515625f,-0.4970999897360801611512215458787977695465087890625f,0.9343999892234802207013899533194489777088165283203125f,-0.8936999880254266059864676208235323429107666015625f,0.0683000261902808603053927072323858737945556640625f,-1.6277999976396557713087531737983226776123046875f,0.264000003844499619987118421704508364200592041015625f,0.1780999958753586298598747816868126392364501953125f,0.64720000277012557088340827249339781701564788818359375f,0.27800001692771925121405729441903531551361083984375f,-0.274800012190639986187079557566903531551361083984375f,0.5232000058174133272359540569595992565155029296875f,0.160499994874000240230316194356419146060943603515625f,0.692700031954050388094401569105684757232666015625f,-0.5086000151500105204149804194457828998565673828125f,0.5841999961614607883575445157475769519805908203125f,0.349999993257224417675388394854962825775146484375f,-0.015199996638298074458361952565610408782958984375f,0.0211000001743435472434384791995398700237274169921875f,-0.229900003767013583200196080724708735942840576171875f,-0.228100008913874330307436366638285107910633087158203125f,0.577300001275539376166534566436894237995147705078125f,0.1325999910548329996373695394140668213367462158203125f,0.23100001060962682686295011080801486968994140625f,-0.106399995169043581899614991925773210823535919189453125f,-0.185200001816451520486594972680904902517795562744140625f,0.5520000180900102026981812741723842918872833251953125f,0.11759999466836446035955532352090813219547271728515625f,-0.481100014591217373105536125876824371516704559326171875f,-0.033699992644786735951356604346074163913726806640625f,0.32900001103430920323944519623182713985443115234375f,-0.2389999735951420234414399601519107818603515625f,-0.3310000018179419800645746363443322479724884033203125f,0.0934000131845476033731756615452468395233154296875f,-0.362799996000527980033467656539869494736194610595703125f,0.06759999019801610220525844852090813219547271728515625f,-0.194800035585463160447972086330992169678211212158203125f,-0.71390001223087296011726721189916133880615234375f,0.43019999291300781152358467807061970233917236328125f,0.2198000060439113667598576284945011138916015625f,-1.11300000306963919882718982989899814128875732421875f,0.59780001850128172691256622783839702606201171875f,-0.4347000120460986494208555086515843868255615234375f,-0.0789999665915956938988529145717620849609375f,-1.37239997918009759558799487422220408916473388671875f,0.40379999513626074758576578460633754730224609375f,0.6216999863266945425266385427676141262054443359375f,-0.3894999935030938331692595966160297393798828125f,0.3128999961078167135752892136224545538425445556640625f,-0.1444000093400479300953520578332245349884033203125f,-0.41279997782111177428987502935342490673065185546875f,-0.942099973195791307034596684388816356658935546875f,0.55149998909234998478723355219699442386627197265625f,-1.35389996067285522940437658689916133880615234375f,-0.077199999957531684202649557846598327159881591796875f,0.276399987774342381641190513619221746921539306640625f,0.63270002074092612787126199691556394100189208984375f,-0.14609997595995682928560199798084795475006103515625f,-0.453800014358758829757789499126374721527099609375f,-0.236100013548135478913536644540727138519287109375f,-0.120900045073032202935792156495153903961181640625f,1.677299993824959756238968111574649810791015625f,-1.0628000202894209991200114018283784389495849609375f,1.251799997758865856667398475110530853271484375f,-0.53319998126775036695335074909962713718414306640625f,0.04640000437051039483549175201915204524993896484375f,-0.5000000205636023764554920489899814128875732421875f,0.423200017142296047722993534989655017852783203125f,-0.20819999900013197446924095856957137584686279296875f,-0.198600009375810682143992380588315427303314208984375f,0.52840000871419956496310987859033048152923583984375f,0.4364000029921530909859939129091799259185791015625f,-0.310799999442696428531007768469862639904022216796875f,-0.067200001205503900880700030029402114450931549072265625f,0.303000026836991620537986591443768702447414398193359375f,-0.270600005374848795458575523298350162804126739501953125f,0.34180002737492376496675205999054014682769775390625f,-0.6478000283360483280148400808684527873992919921875f,0.05760000755786887083331748726777732372283935546875f,0.02880001242160812324755170266143977642059326171875f,-0.19209999904930585845619361862190999090671539306640625f,0.67090001467466375828507807455025613307952880859375f,0.5426999950736757227787165902554988861083984375f,-0.13589998874664299677306189551018178462982177734375f,0.13940001414567237958408441045321524143218994140625f,-0.1357999990701674875026583322323858737945556640625f,-0.1627999980866906837295715604341239668428897857666015625f,-0.438400019107759220116093956676195375621318817138671875f,-0.0619999911189077490547560955747030675411224365234375f,-0.384400016939640210011930321343243122100830078125f,0.038300011996925231727573191164992749691009521484375f,-0.3119000208139421914665945223532617092132568359375f,-0.2041999902382494180841376874013803899288177490234375f,-0.5122000070542096494818906649015843868255615234375f,-0.4963000332459810426399826610577292740345001220703125f,0.07870000784397124249380794935859739780426025390625f,0.4189000149503352421476165545755065977573394775390625f,-1.128099975311755542861646972596645355224609375f,0.1223000121533870387935394319356419146060943603515625f,-0.4151000156044961642010093783028423786163330078125f,0.05589999247193322418070238200016319751739501953125f,0.38630007143020606008576578460633754730224609375f,-0.0449999953620136283749531003195443190634250640869140625f,-0.6053999938912688616454715884174220263957977294921875f,0.5097000180996953400125448752078227698802947998046875f,0.276299997129291430297826082096435129642486572265625f,1.31010004063844665012084078625775873661041259765625f,-0.58530002982616480267097358591854572296142578125f,-0.405900023168325407851853015017695724964141845703125f,0.995300022673607287515551433898508548736572265625f,-0.63509997075200086413815370178781449794769287109375f,-0.5511999890685075342844356782734394073486328125f,1.077999979972839472708301400416530668735504150390625f,-0.616500004589557448753112112171947956085205078125f,-0.09030000419616701634595301584340631961822509765625f,0.87110000356435790536124841310083866119384765625f,-0.07160000851526859833029448054730892181396484375f,0.651199994302540385859856542083434760570526123046875f,-0.14999999940395358777323053800500929355621337890625f,0.98479998518228484272185596637427806854248046875f,-0.24879999826550491803800468915142118930816650390625f,1.32269999842643670717734494246542453765869140625f,0.35850001478195192561315707280300557613372802734375f,-0.2573000472456214993144385516643524169921875f,-0.129000032156705746189118144684471189975738525390625f,-1.237699985611438791011096327565610408782958984375f,1.2618000297963618550056708045303821563720703125f,0.4850000160932541604097423260100185871124267578125f,-0.780999996900558546286674754810519516468048095703125f,-0.40699999946355802649122779257595539093017578125f,0.431199997822940606084785031271167099475860595703125f,0.033200001943111345781289855949580669403076171875f,-0.06220000757575017136247197413467802107334136962890625f,0.32139999800026408660613697065855376422405242919921875f,-0.28799998585879815760080191466840915381908416748046875f,-0.03599999660253505595619571977294981479644775390625f,0.225999995410442300336484322542673908174037933349609375f,0.162800005388259894090197121840901672840118408203125f,0.51850001593679195810437931868364103138446807861328125f,-0.2665000026524066978339533307007513940334320068359375f,-0.456600002983212238660826187697239220142364501953125f,0.3846000001668929968445809208787977695465087890625f,0.264600016111135705187962230411358177661895751953125f,-0.5454000180125238017581068561412394046783447265625f,-0.10659999270141130855993338855114416219294071197509765625f,-0.35420000391006467577170724325696937739849090576171875f,0.1885999920904637416896321155945770442485809326171875f,0.3873999957948921490924476529471576213836669921875f,0.244500005647540208375545489616342820227146148681640625f,-0.395700013305247011441423410360584966838359832763671875f,-0.166399992598593282622232436551712453365325927734375f,-0.33400000190734857596908113919198513031005859375f,0.15579999478608375085286752437241375446319580078125f,0.34460000113546840339040500111877918243408203125f,-0.082400007417797904718526069700601510703563690185546875f,0.489200005213916322421852100887917913496494293212890625f,-0.730700012262165632392907355097122490406036376953125f,0.2446000198364259858863078989088535308837890625f,-0.4457000014960765810201337444595992565155029296875f,-0.2751000212669367783746565692126750946044921875f,0.970399983590841230807200190611183643341064453125f,-1.043999950349331129473284818232059478759765625f,-0.209899992553889802593403146602213382720947265625f,-0.39169998010545992173092599841766059398651123046875f,-0.633899976617097848929915926419198513031005859375f,0.69529996217489298260261421091854572296142578125f,0.285799995642900483971970970742404460906982421875f,-0.364199987965822202795607154257595539093017578125f,0.82630000281035898002102157988701947033405303955078125f,0.15439999525845049532790653756819665431976318359375f,-1.29159999843835837651795372948981821537017822265625f,0.79340002570152279304238618351519107818603515625f,0.77490001077055925549075254821218550205230712890625f,0.17109997882842975513995043002068996429443359375f,0.003299995903670804864304955117404460906982421875f,0.45869998158067470939158738474361598491668701171875f,-0.35309999572634698239426143118180334568023681640625f,0.46950001280009701076778583228588104248046875f,0.44009998868927358017799633671529591083526611328125f,0.538199999503045756199526294949464499950408935546875f,-0.0096000038921832953064949833787977695465087890625f,0.2807999853610994023256353102624416351318359375f,-0.41270000408887863585505328956060111522674560546875f,0.603500018879771005231305025517940521240234375f,-0.27719998658373956601508325547911226749420166015625f,-0.656399979429691970977955861599184572696685791015625f,-0.24610001399517056341892384807579219341278076171875f,-0.3129000051975250329405753291212022304534912109375f,-0.6314999826103446789460349464206956326961517333984375f,-0.3081000098824500099681245046667754650115966796875f,0.1741000036537647677192808259860612452030181884765625f,-0.0985000010728835917461765347979962825775146484375f,0.01490002269148826830047482872032560408115386962890625f,-0.55130000415146340753125286937574855983257293701171875f,-0.45530000527650138497648413249407894909381866455078125f,-0.27909999933242790604026595246978104114532470703125f,-0.3963000205427407163227826458751223981380462646484375f,0.3206999941945074539262350299395620822906494140625f,-0.336600002685189192419557002722285687923431396484375f,0.1781999995589256258199384319595992565155029296875f,-0.27510001143217055474821108873584307730197906494140625f,0.6531000004947185022530220521730370819568634033203125f,0.5685000253617766841074399053468368947505950927734375f,0.083100000345706970250603262684307992458343505859375f,-0.4947000289589169597093132324516773223876953125f,0.102300003063678701664684922434389591217041015625f,-0.39069999680221056959084080517641268670558929443359375f,-0.16010001024007802783444276428781449794769287109375f,0.51070000101178880669294812832958996295928955078125f,0.36770000975131988951716266456060111522674560546875f,-0.0968000028997659800467090462916530668735504150390625f,0.33440000054836271914382450631819665431976318359375f,-0.353400012290477849585812464283662848174571990966796875f,0.18060000295341016585126681093242950737476348876953125f,-0.366300014172494581288219706038944423198699951171875f,-0.1088999983727931830657098544179461896419525146484375f,0.018000021435320423535131340031512081623077392578125f,1.0405999836564063798505230806767940521240234375f,-0.5882999697148800777313226717524230480194091796875f,0.23779996664524105653981678187847137451171875f,0.7049000113666057387717955862171947956085205078125f,-0.876200006353855087581905536353588104248046875f,0.23190000423640011462111942819319665431976318359375f,0.70519997164160042046887610922567546367645263671875f,-0.22060000854134553804897223017178475856781005859375f,-0.1579000209927556852562702260911464691162109375f,-0.5114999946057796620380031527020037174224853515625f,-0.634199990797043255952303297817707061767578125f,-0.90490000033974649085166674922220408916473388671875f,0.70530000538825987632662872783839702606201171875f,-0.6707999785810709880706781405024230480194091796875f,-0.6606999732583762607873723027296364307403564453125f,0.2618000119149683779795623195241205394268035888671875f,0.7947000224769116272227620356716215610504150390625f,-0.3509000175058840653719016700051724910736083984375f,-0.7920000045299531166165252216160297393798828125f,-0.30409999024271971990174279198981821537017822265625f,-0.0508000035405160588197759352624416351318359375f,-0.118400014227628691543259265017695724964141845703125f,0.871100029790401464424576261080801486968994140625f,-0.703299988192319869284574451739899814128875732421875f,0.579499994993209810445478069595992565155029296875f,-0.48360000181347129188935696220141835510730743408203125f,-0.2793999990209936612473029526881873607635498046875f,-0.02340001161992544265189053476206026971340179443359375f,-0.71570000574290748573957898770458996295928955078125f,-0.49250000804662708020487116300500929355621337890625f,-0.4489999947547911318679325631819665431976318359375f,-0.586700022126734666017000563442707061767578125f,0.3203000092625618577812929288484156131744384765625f,-0.251099997343122549597893566897255368530750274658203125f,0.373499995410442320320498765795491635799407958984375f,-0.0908999983765184094153255500714294612407684326171875f,-0.04230001213476075605512960464693605899810791015625f,0.1336999936878680017571241478435695171356201171875f,0.6475000101327896828706798260100185871124267578125f,-0.3854000010251994456922375320573337376117706298828125f,0.507799995851516694500560333835892379283905029296875f,-0.181000002562999640076668583787977695465087890625f,-0.0278000050902367235039491788484156131744384765625f,0.4527999917164446941342248464934527873992919921875f,0.450799991321563453539056354202330112457275390625f,0.2137999875366689561673183561651967465877532958984375f,0.1605999886482951666266671963967382907867431640625f,-0.32499999929219480776509954011999070644378662109375f,-0.4706000019848346482831402681767940521240234375f,-0.228100017184019510096959493239410221576690673828125f,0.2245000092983246986477752216160297393798828125f,-0.13399999963492159604783182658138684928417205810546875f,-0.157599998766183801990337087772786617279052734375f,0.5986000178322201659142365315346978604793548583984375f,0.2329999939799307639987091533839702606201171875f,-0.347200020800531194975491189325111918151378631591796875f,0.415600012862682444136908088694326579570770263671875f,-0.11859999525696041100530919720767997205257415771484375f,-0.06139999922960992506659749778918921947479248046875f,-0.4037000279977920325080731345224194228649139404296875f,1.030399970626831418485380709171295166015625f,-1.36490001628398882616011178470216691493988037109375f,0.73799998891353624230760033242404460906982421875f,-0.531999973535537851176968615618534386157989501953125f,-0.30509998088479051148169673979282379150390625f,-0.4417999886691570399221973275416530668735504150390625f,0.821299980365484572075729374773800373077392578125f,-0.40079997820854185164307637023739516735076904296875f,-0.13070000224113442754969582892954349517822265625f,-0.839299968254566319814102826057933270931243896484375f,-0.27789998448491104454660671763122081756591796875f,-0.22419999943971635758543925476260483264923095703125f,0.06200000065565092199904029257595539093017578125f,-0.0797999927818775489640756859444081783294677734375f,0.4498000188589088566004647873342037200927734375f,0.57110001325011250372654103557579219341278076171875f,-1.0250999958604580797327798791229724884033203125f,0.0956999739289283724019696819595992565155029296875f,-0.093699988919496490780147723853588104248046875f,0.0152000133275986826220105285756289958953857421875f,-1.1365000340938564704629243351519107818603515625f,0.112099995994567880330805564881302416324615478515625f,-0.277500003725290156353366910479962825775146484375f,0.40970001457929594579354670713655650615692138671875f,0.22870004301071134023004560731351375579833984375f,-0.52800000411272041134225219138897955417633056640625f,-0.666399998074769683853446622379124164581298828125f,-0.392400004363059995426965542719699442386627197265625f,0.6769999977871776764004607684910297393798828125f,-1.2469000316917895343493682958069257438182830810546875f,0.27110001153647911298349981734645552933216094970703125f,-0.77170000029653340600788169467705301940441131591796875f,-0.31549999457597710517120503936894237995147705078125f,0.47560001330971701793970396465738303959369659423828125f,-0.30999999858438964661644376974436454474925994873046875f,-0.75200001920759673712524318034411408007144927978515625f,-0.04720001159906406229538333718664944171905517578125f,0.3223000063419341199022483124281279742717742919921875f,-0.225099997723102518421001150272786617279052734375f,-0.78879998362511383191986169549636542797088623046875f,-0.0499999991059301152063198969699442386627197265625f,-0.363600028896331917138695644098334014415740966796875f,0.6948000024676321828565050964243710041046142578125f,0.69629999737143466465028041056939400732517242431640625f,0.29370002423524876888905055238865315914154052734375f,0.448300000634789197473395461202017031610012054443359375f,-0.239499994955956918563089175222557969391345977783203125f,0.31070002492815274042214923611027188599109649658203125f,-0.4475000035762786154691639239899814128875732421875f,-0.454199993106722654800933014485053718090057373046875f,-0.2946000114917755041687996708787977695465087890625f,-0.4544999824762341322781367125571705400943756103515625f,-0.19710000493526447229442055686376988887786865234375f,-0.384499991230666859909348431756370700895786285400390625f,-0.083499990940093749003381162765435874462127685546875f,-0.22050002303719506979717834838083945214748382568359375f,0.69210000225305545740184243186376988887786865234375f,0.0458999988421797677773383838939480483531951904296875f,0.458100013756752133531335857696831226348876953125f,0.346999998494982475616410511065623722970485687255859375f,-0.40600000431388583166381067712791264057159423828125f,-0.6561000020742415728136620600707828998565673828125f,0.02099996441602769436940434388816356658935546875f,0.6885999930962920512200753364595584571361541748046875f,0.43539998884350072927418295876123011112213134765625f,-1.25650002083182332768274136469699442386627197265625f,0.8319000044226640966371633112430572509765625f,0.86420000003576280533934550476260483264923095703125f,-0.55390000835657104971687658689916133880615234375f,-1.32269997994899757287612374057061970233917236328125f,0.88409998323917449170039617456495761871337890625f,-1.13989999795556062878176817321218550205230712890625f,0.04810000660419433415881940163671970367431640625f,0.6120999842226506615361358853988349437713623046875f,-0.08279999614953936770689324475824832916259765625f,0.6864999792017043578340462772757746279239654541015625f,0.214799992539733697327619665884412825107574462890625f,0.792600007408857276658409318770281970500946044921875f,0.048499994516372879616028512828052043914794921875f,0.08020002882480614658788908855058252811431884765625f,-0.78840003046989437507363618351519107818603515625f,-0.307999976694584010061817025416530668735504150390625f,-0.137300007236003551724934368394315242767333984375f,-0.57839999232292171882363618351519107818603515625f,0.410299973052740796219950425438582897186279296875f,-0.7186999981582165020199681748636066913604736328125f,-0.48879996563196215220159501768648624420166015625f,-0.54380000571608544390755923814140260219573974609375f,0.35130002970695528574651689268648624420166015625f,0.566999981246888662411720360978506505489349365234375f,-0.428999997138976851829283987171947956085205078125f,0.011999993242323370168378460220992565155029296875f,0.085200004386901895259143202565610408782958984375f,0.37389998432844884224124371030484326183795928955078125f,0.11569999330043767571396529092453420162200927734375f,-0.42750000599771720288799770059995353221893310546875f,0.362700005578994844057660884573124349117279052734375f,0.5379000041171906332237995229661464691162109375f,-0.06990000413954267077798476748284883797168731689453125f,-0.1412000008404252771043729808297939598560333251953125f,-0.0496000148817898889319621957838535308837890625f,-0.2017000134468081018468410547939129173755645751953125f,0.5869000065088272322100237943232059478759765625f,-0.043699998921900962134401424918905831873416900634765625f,-0.05710000279322269278736712294630706310272216796875f,-0.48109998747110349626154857105575501918792724609375f,-0.552100021028518739285573246888816356658935546875f,-0.787500002495944873004418695927597582340240478515625f,-0.422099996292590873281369567848742008209228515625f,-0.39169999739080640921429221634753048419952392578125f,0.0604999871253966148287872783839702606201171875f,-0.01879999990463254722072861113701947033405303955078125f,-0.42760001024007809888871634029783308506011962890625f,-0.270300001886486729318903599050827324390411376953125f,0.7275000083446503396089610760100185871124267578125f,0.657400004959106443180871792719699442386627197265625f,-0.030999995112419131970682428800500929355621337890625f,0.23459999301433576146536097439820878207683563232421875f,0.15420000294148916708536489750258624553680419921875f,0.5058999964207411270677994252764619886875152587890625f,0.6962999873131512362789408143726177513599395751953125f,0.3212999988242983828712340255151502788066864013671875f,0.11969998559653749481412887689657509326934814453125f,0.3888000043004751216102476973901502788066864013671875f,0.2339999873042106770526515902020037174224853515625f,0.441900011537969150676730123450397513806819915771484375f,-0.44700000493973501836109107898664660751819610595703125f,-0.03480000352933998453153208174626342952251434326171875f,0.82019999458938830372289885417558252811431884765625f,0.1905000047832727805285912836552597582340240478515625f,-0.412200020539760458859745995141565799713134765625f,-0.4401000181004406375251392091740854084491729736328125f,0.36690000293254865226799665833823382854461669921875f,0.642599994966387644268479562015272676944732666015625f,0.155999986991286476722962106578052043914794921875f,0.7554000150695443505100001857499592006206512451171875f,0.1083000029444693979030489572323858737945556640625f,0.407400009615719316258264370844699442386627197265625f,-0.21149999475479130950361650320701301097869873046875f,0.071699998974055045852793455196660943329334259033203125f,0.036599997954070595529429965608869679272174835205078125f,0.05309999684393407193283564993180334568023681640625f,0.69569998555183421018455192097462713718414306640625f,0.263399993254244346729819881147705018520355224609375f,0.9848999826073647767543661757372319698333740234375f,-0.1803000024452805216679251998357358388602733612060546875f,0.3131999939709901870088515352108515799045562744140625f,0.6081000171840191814709442041930742561817169189453125f,0.3582000031352041702348287799395620822906494140625f,0.64709999308884136315356272461940534412860870361328125f,-0.2573000097692013099504038109444081783294677734375f,0.5733999894171954991861639427952468395233154296875f,0.63140001260340194999542973164352588355541229248046875f,0.718800026130676172897437936626374721527099609375f,0.00880001257062001940312256920151412487030029296875f,0.185499990701675443460771930404007434844970703125f,-0.16350000151991839203446943429298698902130126953125f,0.1021000010609629526925345999188721179962158203125f,0.54090001586675651168434342253021895885467529296875f,-0.4066999988809225641261946293525397777557373046875f,-0.393100014166533895387800612297724001109600067138671875f,-0.028700028705596736244842759333550930023193359375f,0.722499993443488364164295489899814128875732421875f,0.842299993675947433757755788974463939666748046875f,0.6041000106573102268470165654434822499752044677734375f,0.067400021350383809703998849727213382720947265625f,-0.78020003314614339018362443312071263790130615234375f,0.48900000198185455957400336046703159809112548828125f,-0.194799995240568790677571087144315242767333984375f,-0.56119999860525116019971392233856022357940673828125f,-1.0452000331461430704393933410756289958953857421875f,-0.82289999029636395899700573863810859620571136474609375f,-0.4621000050097701095097590950899757444858551025390625f,-0.01979997445344938711286886245943605899810791015625f,-0.6782000146090982894975240924395620822906494140625f,-0.4363000140607358190436571021564304828643798828125f,0.051699999588727951760347423260100185871124267578125f,-0.209000009059906233233050443232059478759765625f,-0.7300000099837777867151089594699442386627197265625f,-0.660199989187717495298102221568115055561065673828125f,0.420900020933151264301841365522705018520355224609375f,0.454799995203316154146477856556884944438934326171875f,0.0971999854475260605113362544216215610504150390625f,0.19980000503808259537663616356439888477325439453125f,0.87240002224445323264490070869214832782745361328125f,-0.017700001332163761702531701303087174892425537109375f,0.394499992609024019429853069595992565155029296875f,0.2804999958425760109292923516477458178997039794921875f,-0.39720001696348194997199243516661226749420166015625f,-0.12930001042485239626245174804353155195713043212890625f,0.06840000092834230827065766789019107818603515625f,0.806099990004300881452081739553250372409820556640625f,-0.192600006663798950512500596232712268829345703125f,0.325499990031123065392648641136474907398223876953125f,0.23789998627305042333546225563623011112213134765625f,-1.045200018654763862713252819958142936229705810546875f,0.043200021910667629754243534989655017852783203125f,0.0527999970808625818108339444734156131744384765625f,0.7994999960362909607880510520772077143192291259765625f,-0.89340001515895128836319827314582653343677520751953125f,0.03540000251531605357513399212621152400970458984375f,-0.429299992543458852622961785527877509593963623046875f,-0.55019999540895236922466438045375980436801910400390625f,-0.58680000159591438890771541991853155195713043212890625f,0.33060002657175058260463629267178475856781005859375f,0.29970000750124459187873071641661226749420166015625f,-0.137400001680850980534387417719699442386627197265625f,-0.43259999273121341190062594250775873661041259765625f,0.4094999988675116497915951185859739780426025390625f,-0.5729999996423720265426027253852225840091705322265625f,0.53200002866983453486682265065610408782958984375f,-0.0684999772310259658070208388380706310272216796875f,-0.55230000708699211742214174591936171054840087890625f,-0.94269998501539209456723256153054535388946533203125f,-0.2823000266075137432153496774844825267791748046875f,1.3424000293970106412189124966971576213836669921875f,0.53900003305077603954487130977213382720947265625f,-0.112499998509883791797392404987476766109466552734375f,-0.37350001522898690353713391232304275035858154296875f,0.797799965751170958583315950818359851837158203125f,0.1419999972283836342512586270458996295928955078125f,-0.55250001266598669502627672045491635799407958984375f,-0.5219000395894057842127722324221394956111907958984375f,0.48530002194717525298983673565089702606201171875f,-0.12930000384971496174557614722289144992828369140625f,-0.957000020846724197554067359305918216705322265625f,-0.83200005199015214518709626645431853830814361572265625f,0.7076000062167644433230861977790482342243194580078125f,0.075400004899501826827190598123706877231597900390625f,-0.096500001609324925766486558131873607635498046875f,-0.3413000224053863718154389061965048313140869140625f,1.124399985349178177074236373300664126873016357421875f,0.7050000263005495870771710542612709105014801025390625f,-0.19649998492002485050988980219699442386627197265625f,-0.66850001910328860077470380929298698902130126953125f,-0.2337000082723796889894174455548636615276336669921875f,-0.33230001317784196857019196613691747188568115234375f,-0.46299999785423295861619408242404460906982421875f,-0.7406000328302386748191565857268869876861572265625f,-1.105200008894502872180964914150536060333251953125f,-0.398400017058849531537134680547751486301422119140625f,-0.7244999943226575478405493413447402417659759521484375f,-0.05849998140335088692154386080801486968994140625f,-0.22710001477003116665542847840697504580020904541015625f,-0.6995999793052671833493150188587605953216552734375f,0.4692000114351511097510183390113525092601776123046875f,0.9567000173211095415126692387275397777557373046875f,-0.402900009146332782616894974125898443162441253662109375f,-0.001499998182058259743598682689480483531951904296875f,0.40020000110864639708552203956060111522674560546875f,0.97289999119043368835946239414624869823455810546875f,-0.55770002569556254012894669358502142131328582763671875f,-0.35850000211596455557128138025291264057159423828125f,-0.0885000048168004538329256547513068653643131256103515625f,0.537900022594630655703440424986183643341064453125f,-0.4968000268533827590289320141891948878765106201171875f,-0.1964999973624941276284516789019107818603515625f,0.0593999981269240084902349963158485479652881622314453125f,0.6033000051051378154198800984886474907398223876953125f,-0.620700026790797831921508986852131783962249755859375f,-0.28200001436471922033888404257595539093017578125f,0.705600006455183059728142325184307992458343505859375f,0.77669998517185445674471111487946473062038421630859375f,-0.48990000738054517714914481985033489763736724853515625f,0.26400001740455625309778042719699442386627197265625f,0.00809999891147017425652165911742486059665679931640625f,0.291900006397068534358396618699771352112293243408203125f,-0.45420000540018090617877533077262341976165771484375f,0.16800002586841600304978783242404460906982421875f,-0.87600002532452359016446052919491194188594818115234375f,0.66029999227523727967081867973320186138153076171875f,1.0383000217199320758254543761722743511199951171875f,0.62830003989934990471510900533758103847503662109375f,0.5420999670863153596656047739088535308837890625f,0.759899987971781598616871633566915988922119140625f,-0.5793000117659570236128274700604379177093505859375f,-0.440500020653009638493813326931558549404144287109375f,1.543200014758109972490274230949580669403076171875f,0.74440003392696407757966881035827100276947021484375f,0.521599998438357914665175485424697399139404296875f,0.2990000254511837596282930462621152400970458984375f,-1.10009998922944074450924745178781449794769287109375f,-0.9951000296115874999003381162765435874462127685546875f,0.030399990296363643693666745093651115894317626953125f,0.7386000364959244279816630296409130096435546875f,-0.1134999980926512108680981327779591083526611328125f,-0.520500026166439422325993291451595723628997802734375f,-1.3189000256419181056344314129091799259185791015625f,-0.5567000293910506769634594093076884746551513671875f,0.04219999617636194244596481439657509326934814453125f,0.1247999997109172642950625231605954468250274658203125f,0.0393999904155730718002814683131873607635498046875f,0.49320001669526103427187990746460855007171630859375f,0.95100000196695333443130948580801486968994140625f,0.24080002693533941737769055180251598358154296875f,0.5630000050067902606087955064140260219573974609375f,0.24740001717805881753520225174725055694580078125f,-1.3303999959588050927550284541212022304534912109375f,-0.62080000547766633900437227566726505756378173828125f,-0.5441999900706111503723150235600769519805908203125f,-0.48990003327131237398361918167211115360260009765625f,0.8907000284671784395840177239733748137950897216796875f,1.0224000255227085887099747196771204471588134765625f,0.54029998232871268015742316492833197116851806640625f,-0.0267000104665751081256530596874654293060302734375f,-0.1490999995931983068686577098560519516468048095703125f,-0.04139999136924732425768524990417063236236572265625f,0.512400007268786428227258511469699442386627197265625f,0.8826000021934508055210244492627680301666259765625f,0.504600022928416880318991388776339590549468994140625f,0.6033000041365621513023143052123486995697021484375f,-0.83850000397861002010557740504737012088298797607421875f,-0.47129998545050622027474673814140260219573974609375f,0.6264000225126744592074601314379833638668060302734375f,-0.366300019723177161523608447168953716754913330078125f,-0.5589000165894628668894483780604787170886993408203125f,0.04050000590086000329392845742404460906982421875f,-0.3105000063478946525918900078977458178997039794921875f,-0.72870001022815689140088579733856022357940673828125f,-0.0845999982297420682897381993825547397136688232421875f,-0.2439000015020369449558756969054229557514190673828125f,0.3114000082075596420594365554279647767543792724609375f,-0.348300014269351976281541283242404460906982421875f,0.034200004953146123654050825280137360095977783203125f,0.55049997907876946356964253936894237995147705078125f,-0.0785999998390674026182978195720352232456207275390625f,0.0569999848604201275747982435859739780426025390625f,-0.3605999922618268183072132160305045545101165771484375f,-0.7002000043869021084219639305956661701202392578125f,-0.9072000122509897845901605251128785312175750732421875f,0.5430999983154232158000240815454162657260894775390625f,0.6271999585509302477248638751916587352752685546875f,0.25579999221563287647995821316726505756378173828125f,0.756200002181529828959583028336055576801300048828125f,0.32400002679228823154744532075710594654083251953125f,0.6525999644935129850864541367627680301666259765625f,0.6124000065982337925873935091658495366573333740234375f,0.4731999789953231783101728069595992565155029296875f,0.87500001892447443196942913345992565155029296875f,0.554799981606006742396175468456931412220001220703125f,0.892800019991397686425216306815855205059051513671875f,-0.1195000146627425152701107435859739780426025390625f,0.244899999892711495164121515699662268161773681640625f,-0.24000001043081287122049616300500929355621337890625f,-0.844000022023916329771964228712022304534912109375f,1.583999964356422651690081693232059478759765625f,1.07860003053545927542700155754573643207550048828125f,-1.328900001800060071133202654891647398471832275390625f,-0.7121000375688080197278395644389092922210693359375f,0.2061000101208687595999435870908200740814208984375f,0.0675000092387201267030150120262987911701202392578125f,-0.18030001060366629417330841533839702606201171875f,-0.361500012785196389586417353712022304534912109375f,0.861000015489757242193036290700547397136688232421875f,0.166600018666684945856104604899883270263671875f,0.625899995303154099701714585535228252410888671875f,0.1765000048875808857928859652020037174224853515625f,-0.790099976116418911686878345790319144725799560546875f,-0.1877000055789945776041349745355546474456787109375f,-0.768400016164779486871339031495153903961181640625f,0.219899983743578442130228722817264497280120849609375f,0.5357999871119856205581299946061335504055023193359375f,0.800099978053570026759189204312860965728759765625f,0.249000013679266107846643762968597002327442169189453125f,0.491099992388486672467706739553250372409820556640625f,0.9072000044837593701885225527803413569927215576171875f,0.711299992007017323203399428166449069976806640625f,0.5880000043362378914224564141477458178997039794921875f,-0.2079000198006628608027313021011650562286376953125f,0.7812000008404254014493517388473264873027801513671875f,0.059399981921911315652096163830719888210296630859375f,0.3824999977648257942774989714962430298328399658203125f,-0.04200000751018517775037253159098327159881591796875f,-0.74700000538676969430440522046410478651523590087890625f,0.19499999374151233411112116300500929355621337890625f,0.78599999637901785121130160405300557613372802734375f,0.7304999570250512164193423814140260219573974609375f,-0.733500016160309353807633669930510222911834716796875f,-0.985199997085332679347402518033050000667572021484375f,-0.14610000039786097403293751995079219341278076171875f,-0.077700005429983232119184322073124349117279052734375f,-0.1479000035583972749719805506174452602863311767578125f,0.207599998021125831115796245285309851169586181640625f,-0.1668000037193297924176249580341391265392303466796875f,0.35099998848140223639546775302733294665813446044921875f,0.35609999570399519797092580120079219341278076171875f,0.71369999830722807843130794935859739780426025390625f,-0.152699996303021945376343637690297327935695648193359375f,-0.616799977940321042524374206550419330596923828125f,-0.285600011149048871317290831939317286014556884765625f,-0.63280001581907274754712489084340631961822509765625f,0.05179999507665655755772604607045650482177734375f,-0.1484000127375129096662931260652840137481689453125f,0.0271999983370303510810117586515843868255615234375f,-0.32020001571178458021904589259065687656402587890625f,-0.054900040721893361705951974727213382720947265625f,0.7341000117003915192981366999447345733642578125f,0.85740000466108323706748706172220408916473388671875f,0.11140000999569910078434986644424498081207275390625f,0.174699985969066773350277799181640148162841796875f,-0.62310003372430866619424705277197062969207763671875f,-0.4415000129342079304706203402020037174224853515625f,0.324500003486871690938642132095992565155029296875f,0.164299973320961356648695073090493679046630859375f,0.504099996948241813043978254427202045917510986328125f,0.45529998378157632288321110536344349384307861328125f,0.6635000138878821562826715307892300188541412353515625f,-1.003699985492229274086639634333550930023193359375f,-0.65750002205371860242166803800500929355621337890625f,-0.17210001387596118860301430686376988887786865234375f,0.304300006401538869482692462042905390262603759765625f,-0.3078000253558155208111202227883040904998779296875f,0.56999997794628054492704904987476766109466552734375f,0.996499971807003337431751788244582712650299072265625f,0.48669999109506534296087920665740966796875f,-0.3280999870836733833812104421667754650115966796875f,-0.4060999987959856838415362290106713771820068359375f,0.118600006246566636036732234060764312744140625f,0.432200019198656359975529994699172675609588623046875f,-0.84919997231960309136411524377763271331787109375f,0.40470000146627427284329314716160297393798828125f,-0.355800014045834611664531621499918401241302490234375f,-0.13410000916719422292544550145976245403289794921875f,-0.175200004684925136899664721568115055561065673828125f,0.305100011134147752756007321295328438282012939453125f,0.991500008538365396049130140454508364200592041015625f,-0.0237000203609465387444288353435695171356201171875f,0.22199999957531690863987705597537569701671600341796875f,0.87120000277757636553133124834857881069183349609375f,-0.720300021220743769134742251480929553508758544921875f,0.2750999962329867543076034053228795528411865234375f,0.5811000053152441946480166734545491635799407958984375f,-0.2310000013709068156231296597979962825775146484375f,0.789299986396729824633666794397868216037750244140625f,0.291899985945224926808805321343243122100830078125f,0.80129999826550479813391802963451482355594635009765625f,0.33389998794198039178127146442420780658721923828125f,-0.851100011126697086893955201958306133747100830078125f,-0.9020999983787536535828621708787977695465087890625f,0.45150000071525575595643431370262987911701202392578125f,-0.100800011885166185265916283242404460906982421875f,0.8567999787971374292538939698715694248676300048828125f,-0.1628999959588046664293869980610907077789306640625f,0.55709997934251997975252379546873271465301513671875f,0.817499988824129442122057298547588288784027099609375f,-0.2030999991908668533824311452917754650115966796875f,0.076200000691413638520543827326036989688873291015625f,0.49680001232475046180070421542041003704071044921875f,0.02580000532865511786440038122236728668212890625f,-0.7523999922931192685382484341971576213836669921875f,0.2602000124335290820454247295856475830078125f,-1.0258999994754791629247847595252096652984619140625f,-0.663100029104948252012263765209354460239410400390625f,0.6194000425696373213213519193232059478759765625f,-0.1947999880135053984986370778642594814300537109375f,-0.0068999635934829939287737943232059478759765625f,-0.4838999953925604557980477693490684032440185546875f,-0.5238999898791316667256978689692914485931396484375f,-1.0381000107765192552733424236066639423370361328125f,-0.1103000143289563084181281737983226776123046875f,0.3113000030338763934878443251363933086395263671875f,1.536999953269958751889134873636066913604736328125f,0.9946000079154959649940792587585747241973876953125f,-0.98640000478029232766630229889415204524993896484375f,-0.37960002773404166731552322744391858577728271484375f,-0.2417999876260754632539828890003263950347900390625f,-0.068000006347894714764379386906512081623077392578125f,0.48929998762607596063389792107045650482177734375f,0.70250001117587057564151109545491635799407958984375f,-0.6288000055670739385504930396564304828643798828125f,-0.0001999981284139806803068495355546474456787109375f,-0.0784000108003615192586721605039201676845550537109375f,0.34499999999999975131004248396493494510650634765625f,-1.0711999789357182777393973083235323429107666015625f,-0.2526000144124029844761025742627680301666259765625f,-1.3656000072002409950755463796667754650115966796875f,-1.222800050240755442843010314390994608402252197265625f,0.26809998991489436548363300971686840057373046875f,-0.1701000106871131123398299678228795528411865234375f,0.7108000205278397487518304842524230480194091796875f,-0.066600001120567409174100248492322862148284912109375f,-0.4509000051006675224840591909014619886875152587890625f,-0.46019998590946187277950230054557323455810546875f,-0.382200000125169747633435690659098327159881591796875f,0.42570001803636525750107466592453420162200927734375f,-0.7988999981492757029855056316591799259185791015625f,-0.18869999115467095407439046539366245269775390625f,-0.6620999869421122507873178619774989783763885498046875f,0.32130001420974707571076578460633754730224609375f,0.518699992905557127897964164731092751026153564453125f,0.02159998592138290263164890347979962825775146484375f,0.740999984979629378045729026780463755130767822265625f,0.76199996578693429682971327565610408782958984375f,-0.51900001379102478349381044608890078961849212646484375f,-0.92520000170469263167660756153054535388946533203125f,-0.2574000121861697021330428469809703528881072998046875f,-0.41190000114440916689773075631819665431976318359375f,0.366299993012845437334590315003879368305206298828125f,-0.18660001870393738698794550145976245403289794921875f,0.2718000096425414735534786814241670072078704833984375f,-0.26789999774694450707102078013122081756591796875f,0.30959999599456777819028729936690069735050201416015625f,-0.2367000065177677914363130184938199818134307861328125f,-0.0510000092312694608409628926892764866352081298828125f,-0.662099983179569306912526371888816356658935546875f,-0.8514000156953931419678838210529647767543792724609375f,-0.3887999888777731172240237356163561344146728515625f,-0.41430001556575302146256944979541003704071044921875f,0.030299982142448556032832129858434200286865234375f,-0.2279999998658894799774543571402318775653839111328125f,-0.5537000176787376659603978623636066913604736328125f,-0.100200012135505556187808906543068587779998779296875f,0.43860000565052015275568919605575501918792724609375f,-0.756600011886656265147621525102294981479644775390625f,-0.0920000081434846350703082862310111522674560546875f,-0.2377000246524811899462292785756289958953857421875f,0.37750001341104510999002741300500929355621337890625f,0.179600001545250353984783941996283829212188720703125f,0.0224000012710690032946558858384378254413604736328125f,1.25929998613595994783054266008548438549041748046875f,0.55870000128745989087519774329848587512969970703125f,1.330900001412629762853612191975116729736328125f,0.80270005315095260822744194229017011821269989013671875f,-0.12479999181330203583684124168939888477325439453125f,0.120600002804398531708329755929298698902130126953125f,-1.1742000107645989004367947927676141262054443359375f,-0.62740003908276609223548803129233419895172119140625f,-0.48119997960627092226104650762863457202911376953125f,-0.42540000154674018073563956932048313319683074951171875f,-0.815900011619925091821414753212593495845794677734375f,-0.96790005235970089092489843096700496971607208251953125f,0.80049996626377151187625713646411895751953125f,0.596899999803304215362231843755580484867095947265625f,-0.4367000104665752502342002117075026035308837890625f,-1.2225000463426116681375788175500929355621337890625f,0.76160002615451816154745756648480892181396484375f,0.1478000204384333216012237244285643100738525390625f,-0.63519998933672894292357113954494707286357879638671875f,-0.91240003304779548898295615799725055694580078125f,0.2236000008821488194143967120908200740814208984375f,0.700500009477138529945250411401502788066864013671875f,0.341399998372793167522587509665754623711109161376953125f,-0.360600012415647430685794461169280111789703369140625f,0.1620000012516975562704146796022541821002960205078125f,-0.37230000224411485820752432118752039968967437744140625f,0.4143000119894744148751897228066809475421905517578125f,-0.3126000203728676041237122262828052043914794921875f,-0.087599998877942564234899691655300557613372802734375f,0.03180000096261503816919002929353155195713043212890625f,0.65069998208731394395698544030892662703990936279296875f,1.05689999488592167864453585934825241565704345703125f,0.78390002944171455201427534120739437639713287353515625f,0.9086999901674683854935210547409951686859130859375f,-0.063599999392032646738925905083306133747100830078125f,-0.3668999977171421011945540158194489777088165283203125f,-0.2940000079795719312159008040907792747020721435546875f,-0.4715999826431271912952070124447345733642578125f,-0.44099999336153261619841714491485618054866790771484375f,-0.3050999882608653734905601595528423786163330078125f,-0.8427000317305328724870605583419091999530792236328125f,-0.256200000467896149558555407566018402576446533203125f,0.84629998094290481702728357049636542797088623046875f,0.800999977231025894752747262828052043914794921875f,-0.9567000284224749240280516460188664495944976806640625f,0.3428999950647355632327162311412394046783447265625f,0.42600000442564489588903597905300557613372802734375f,0.9603000158190724278028937987983226776123046875f,-0.888600014479458355509677858208306133747100830078125f,-0.22559998808950176840681933754240162670612335205078125f,0.0750000003352762067976300386362709105014801025390625f,-0.36930002100467707037978470907546579837799072265625f,-0.7345999897919595067463660598150454461574554443359375f,0.050599996415525705373283926746807992458343505859375f,-0.253399963414668860650635906495153903961181640625f,-1.087400028651952510472256108187139034271240234375f,-1.00670002328157437432309961877763271331787109375f,-0.35050001126527785544340076739899814128875732421875f,0.232300013196468313481091172434389591217041015625f,-0.6905000066459177077149433898739516735076904296875f,-0.420099989974499266054408508352935314178466796875f,0.4831000249326233841884459252469241619110107421875f,-0.0029999766945838501897014793939888477325439453125f,0.42980001096129427651248988695442676544189453125f,0.329099995458125960112738539464771747589111328125f,0.06050000098347663168851795489899814128875732421875f,0.12930001616179964685215963982045650482177734375f,0.05430000796616096803148820981732569634914398193359375f,0.71920000987052912222452505375258624553680419921875f,1.083000039726496055436655296944081783294677734375f,0.318800006498396459875266373273916542530059814453125f,0.1510000088587404309947714864392764866352081298828125f,1.123900020277500022558569980901665985584259033203125f,0.370100019329786533717197016812860965728759765625f,-0.376899995929002695760345886810682713985443115234375f,0.348099999749660327097444678656756877899169921875f,1.063199978101253773132839341997168958187103271484375f,0.840000011473893692937053856439888477325439453125f,0.495800015163421647912400658242404460906982421875f,1.19320004172921212415303671150468289852142333984375f,0.33919999258518185314414949971251189708709716796875f,0.315600015544891476793054607696831226348876953125f,0.10080000136122102549762757917051203548908233642578125f,-0.63479998872131115916772614582441747188568115234375f,-0.801600016357004729883328764117322862148284912109375f,0.4848000313758848989209582214243710041046142578125f,-0.00209999253004777841624672873876988887786865234375f,-0.401100008630752569160904386080801486968994140625f,-0.950700002874433902633199977572076022624969482421875f,0.16739999691247930968529544770717620849609375f,0.744900031296908959888014578609727323055267333984375f,-0.2627999782681467166867150808684527873992919921875f,0.2544000012561677959155304051819257438182830810546875f,-0.3569999854564667174372516456060111522674560546875f,0.32190000315606592806716435006819665431976318359375f,0.57630000824928284686166080064140260219573974609375f,-0.264900005815923211827112027094699442386627197265625f,-0.263100007423758608382513557444326579570770263671875f,0.87240001352727414296595043197157792747020721435546875f,0.02909999754428849172427362645976245403289794921875f,-0.0234000007420778199929145557689480483531951904296875f,0.03989999564588064817627355296281166374683380126953125f,0.341100005650520288202898200324852950870990753173828125f,0.863700016933679481923036291846074163913726806640625f,0.293099998408555884044091044415836222469806671142578125f,-0.53369999309182158953035468584857881069183349609375f,0.73019998915046446796850432292558252811431884765625f,0.546599981898069575692034050007350742816925048828125f,0.832200012418627732557752096909098327159881591796875f,-0.38610000416040435311515466310083866119384765625f,0.71940002517253176517186830096761696040630340576171875f,0.5107000283181672983801036025397479534149169921875f,0.4197000113010407318370198481716215610504150390625f,-0.338499997794627915936871431767940521240234375f,-1.1182999934077262338405489572323858737945556640625f,-0.72530002803802506861075016786344349384307861328125f,-0.27349999675154634815044119022786617279052734375f,-0.2723000200510028445677335184882394969463348388671875f,-1.3142999700427058229479371220804750919342041015625f,-0.58650000280141778574716227012686431407928466796875f,0.689499986946582765767743694595992565155029296875f,0.310300011944770748328892295830883085727691650390625f,0.293399991726875253306161539512686431407928466796875f,0.22640000850558283929103708942420780658721923828125f,-0.0768999825179577101152972318232059478759765625f,0.50250001251697540283203125f,0.8878999780774119443549352581612765789031982421875f,0.988300009500979736998260705149732530117034912109375f,-0.1729000006712972936639971521799452602863311767578125f,-0.132700003883987616148942834115587174892425537109375f,0.669600022257864413433026129496283829212188720703125f,0.0402000017419457122969106421805918216705322265625f,-0.3929000184595583533564422396011650562286376953125f,0.22850000286102289948075849679298698902130126953125f,-0.54530002446174563601744011975824832916259765625f,-1.26730005283355762202290861750952899456024169921875f,-1.299800008726119671109699993394315242767333984375f,-1.05140004471540482455793608096428215503692626953125f,-0.515399987576901974506426995503716170787811279296875f,-0.05500000711530450558939264738000929355621337890625f,-0.2879000214397908763430677936412394046783447265625f,-0.3812999844074251853953683166764676570892333984375f,-0.5457000042155384367248416310758329927921295166015625f,0.27209999301433551721629555686376988887786865234375f,-0.7701000080421567250965608764090575277805328369140625f,-0.8138999931573867030465407879091799259185791015625f,-0.600900026111304885745312276412732899188995361328125f,-0.525000013560056544292820035479962825775146484375f,-0.6428999842241405193732362022274173796176910400390625f,-1.0568999808788301919548757723532617092132568359375f,0.171900002261996309727010157075710594654083251953125f,0.362999985337257413675615680404007434844970703125f,0.288600004309415847369990615334245376288890838623046875f,0.257699994999170289844414583058096468448638916015625f,0.1431000039964913650436528769205324351787567138671875f,-0.659699992525577538771131003159098327159881591796875f,0.6182999867767093871151473649661056697368621826171875f,0.0371999791145327662889030762016773223876953125f,0.015299999371916062074205910903401672840118408203125f,0.057300002132356142592328751561581157147884368896484375f,-0.1494000027090311988331450265832245349884033203125f,0.38430001869499685884790096679353155195713043212890625f,0.29909999970495704246786772273480892181396484375f,-0.362700017797946916431328645558096468448638916015625f,-0.93090003695189960097877701628021895885467529296875f,0.3395999813914301057593547739088535308837890625f,-0.790200021672248897885992846568115055561065673828125f,-0.5154000022172926520624969271011650562286376953125f,0.0018000011488794598335516639053821563720703125f,-0.38819998219907281278295840820646844804286956787109375f,-0.087299993638694228792473950306884944438934326171875f,0.718500012993812386952185988775454461574554443359375f,0.176100003117322767565156027558259665966033935546875f,0.4417000192582609674474269922939129173755645751953125f,0.0481999965786936712675014859996736049652099609375f,-0.54480004836320983230280035058967769145965576171875f,0.081899992352724215738390967089799232780933380126953125f,0.53850000371783945940507010163855738937854766845703125f,0.3807000175707042632922139091533608734607696533203125f,-0.108900003308802784118825002224184572696685791015625f,1.2833000264883036578567043761722743511199951171875f,0.77170004563331673352877260185778141021728515625f,-0.61580001575946763381352866417728364467620849609375f,-0.827600050473213944002282005385495722293853759765625f,-0.3642999637842176952062800410203635692596435546875f,0.13770001079440152835786648211069405078887939453125f,1.16149998283386235442549150320701301097869873046875f,0.54490002507567414369304970023222267627716064453125f,-0.3120999986767767353512681438587605953216552734375f,-0.651300028216839077543909297673963010311126708984375f,-0.3007000169001521339140481359208934009075164794921875f,-0.31530001855716138781104973531910218298435211181640625f,-0.4428000003218652835812463308684527873992919921875f,-0.617200018155574792189099753159098327159881591796875f,-0.6021999972939493517287701251916587352752685546875f,-0.93140003264546411543278736644424498081207275390625f,0.310300007846951331202944857068359851837158203125f,0.46230003189742630009817503378144465386867523193359375f,0.17219995199441928690475833718664944171905517578125f,0.99220001085400522100599118857644498348236083984375f,1.36740002254247627888616989366710186004638671875f,-0.298200009989738390459024230949580669403076171875f,0.489300012660026595323614628796349279582500457763671875f,0.055800003242492568489296900224871933460235595703125f,-0.7608000333800915004900389249087311327457427978515625f,-0.0038999874949450230587899568490684032440185546875f,0.5192999954491852587779021632741205394268035888671875f,-0.141300002568215010700924949560430832207202911376953125f,-0.011700005344301545040508472084184177219867706298828125f,0.498600017832219577496033480201731435954570770263671875f,0.8553000287085772423978369261021725833415985107421875f,0.9924000243306156932021622196771204471588134765625f,-0.66840003326386232540556875392212532460689544677734375f,-0.08490000227689709078049418167211115360260009765625f,-0.1953000002101064058734891659696586430072784423828125f,-0.76139998987913148909001392894424498081207275390625f,0.37350000308454045505612839406239800155162811279296875f,0.67349997335672373566239912179298698902130126953125f,-0.4653000150740147500272314573521725833415985107421875f,0.1602000091552735483446667785756289958953857421875f,0.01590000057443974323945212745456956326961517333984375f,0.213900008021295218174628871565801091492176055908203125f,-0.2675999969780444853739709287765435874462127685546875f,0.213600016081333166084732511080801486968994140625f,-0.5700000018998980788609287628787569701671600341796875f,0.1944000249862671836353911203332245349884033203125f,0.6651000186592341822944263185490854084491729736328125f,0.3474000005632635890151505009271204471588134765625f,0.83639999691992994446110287753981538116931915283203125f,-0.37740003803968402706914275768212974071502685546875f,0.776400027765333788920543156564235687255859375f,-0.46250000149011594174908168497495353221893310546875f,-0.446400017893314071670829434879124164581298828125f,0.4505999970674512411505929776467382907867431640625f,1.186499970912933576983050443232059478759765625f,0.817499998509882974673246280872263014316558837890625f,-0.053799994726479116735617935773916542530059814453125f,0.168800002102553747818092233501374721527099609375f,-0.780799966883659379845994408242404460906982421875f,-0.537799998235701792737017967738211154937744140625f,0.3911999975621700542660619248636066913604736328125f,0.210000007003545707817693255492486059665679931640625f,0.7166000055909156341016341684735380113124847412109375f,-0.020399994170665625148330946103669703006744384765625f,-1.3623999954223631192462562466971576213836669921875f,-0.81120004450082827673895735642872750759124755859375f,0.6104000048995015248465279000811278820037841796875f,0.10940001366138485394685631035827100276947021484375f,-0.168200005854666212457004803582094609737396240234375f,-0.20080000700503586319456417186302132904529571533203125f,-0.376999984785914676876927842386066913604736328125f,-0.57520000773966295515293722928618080914020538330078125f,0.2297999856479464586556105132331140339374542236328125f,-0.260400012182444573483053318341262638568878173828125f,-0.136300007951259516403297311626374721527099609375f,-0.06290000698566444725656765513122081756591796875f,-0.47349999354779725990738370455801486968994140625f,-0.66270002085268508995596903332625515758991241455078125f,-0.0270999865137040440998816848150454461574554443359375f,0.305500008974224357416460406966507434844970703125f,-0.1294000351190565112347030662931501865386962890625f,0.1766999973535536838653570157475769519805908203125f,0.727799991939961810061276992200873792171478271484375f,-0.25770000766515721579708042554557323455810546875f,0.52229998268634059144233106053434312343597412109375f,0.457499984204769560847125831060111522674560546875f,-0.0203999979704618805698146388749592006206512451171875f,-0.32489999579489226011475011546281166374683380126953125f,-0.33539999316483726232718254323117434978485107421875f,-0.29189998296499286567495801136828958988189697265625f,0.02999999932944774627685546875f,0.078599992537498497569004030083306133747100830078125f,0.0078000003218650915126630707163712941110134124755859375f,0.6963000040024518444425893903826363384723663330078125f,-0.7260000178366901035786895590717904269695281982421875f,-0.8591999836444852434169661137275397777557373046875f,0.3066000127807260167145386731135658919811248779296875f,0.7437000117182730463127882103435695171356201171875f,0.0491999984711408711035574015113525092601776123046875f,0.20190000129342078327709941731882281601428985595703125f,-0.6851999927267431456101576259243302047252655029296875f,-0.2624999880045653100069102947600185871124267578125f,-0.05220000085160136915618522834847681224346160888671875f,0.5409000035360456148936236786539666354656219482421875f,0.158699995364248724882827445981092751026153564453125f,0.14909999933242801262167631648480892181396484375f,-0.7239000046312809022452938734204508364200592041015625f,-0.26399999229609960593734285794198513031005859375f,0.00390000155791636604618588535231538116931915283203125f,-0.477299997363984596088215539566590450704097747802734375f,0.9449999964237212335405047269887290894985198974609375f,-0.703500018805265359134182290290482342243194580078125f,-0.683299977910518663293260033242404460906982421875f,-0.0208999972403045575219948659650981426239013671875f,0.865700005221366808427774230949580669403076171875f,0.754100031667948034197479501017369329929351806640625f,-0.039699996213614863194152349024079740047454833984375f,-0.3129000081405042266169402864761650562286376953125f,-0.2306000167369841591380463796667754650115966796875f,0.2806000043690204392987652681767940521240234375f,-0.291300001394748875327422865666449069976806640625f,0.38070001222491267611758303246460855007171630859375f,-1.0057999963879584726100802072323858737945556640625f,-0.64600002729892747055373547482304275035858154296875f,-0.37729998004138476375146638019941747188568115234375f,0.23890000589787963036769724567420780658721923828125f,0.088300022315979109777117628254927694797515869140625f,-0.71110002666115779135225238860584795475006103515625f,-0.140500026166439084818193805404007434844970703125f,0.6175000165402888541166248614899814128875732421875f,-0.3137000006914139049740697373636066913604736328125f,0.13389999628663051822741181240417063236236572265625f,-0.24459999688863742761668618186376988887786865234375f,-0.906600032860040982995997183024883270263671875f,-0.75199997651577010771006825962103903293609619140625f,-0.627800006580352398799504953785799443721771240234375f,-0.3716000044360756593420092031010426580905914306640625f,0.156400002209842181599697141791693866252899169921875f,0.1201999772667885491017614185693673789501190185546875f,0.650600013458728643200856822659261524677276611328125f,-0.8114999995231624296820882591418921947479248046875f,0.2802000103473663017439321265555918216705322265625f,0.279300006066262707093983408412896096706390380859375f,-0.2783999783158304097696600365452468395233154296875f,0.596700010652840173719368976890109479427337646484375f,0.3425999956369398802280556992627680301666259765625f,0.03689999555647371920485966256819665431976318359375f,0.47340000878870480693905165026080794632434844970703125f,0.4596000010237097921361737462575547397136688232421875f,-0.10980001170635222962346233543939888477325439453125f,0.216900012432038824528035547700710594654083251953125f,-0.5904000013232231225401847041212022304534912109375f,-0.5184000077828765373766373159014619886875152587890625f,-0.5438999898791312404000564129091799259185791015625f,-0.08969999543130402486212915391661226749420166015625f,-0.8048999893128871985226169272209517657756805419921875f,-0.717300016549229813023202950716949999332427978515625f,0.330900022870302290556310254032723605632781982421875f,0.533700009222328741742558122496120631694793701171875f,-0.447000024497508974263837444595992565155029296875f,0.2709000008538364756560667956364341080188751220703125f,-0.18059999080896371737026129267178475856781005859375f,-1.002000017009675669754642512998543679714202880859375f,-0.00479999650716766979030580841936171054840087890625f,-0.63659999333471051219390801634290255606174468994140625f,-0.47249998599290865541888706502504646778106689453125f,0.1056000018730759715257505604313337244093418121337890625f,-0.45480000574588774764350773693877272307872772216796875f,0.6396000030353665710691757340100593864917755126953125f,-0.1374000187426804853174644449609331786632537841796875f,-0.7755000320523979251419177671778015792369842529296875f,-0.63150001531839361490483497618697583675384521484375f,-1.4330000061988830140080608543939888477325439453125f,-0.7300000210106372122709217364899814128875732421875f,-0.37639997721314433221095896442420780658721923828125f,0.08119999800920485455435482435859739780426025390625f,-0.62429998216852577996860418352298438549041748046875f,-0.6457000042714173826396972799557261168956756591796875f,-0.2372999616384507959310212754644453525543212890625f,-0.20749998077750131386665088939480483531951904296875f,-0.65559996979832657615361313219182193279266357421875f,0.00060000019669548265710545820184051990509033203125f,-0.2370000030398369261774860206060111522674560546875f,-0.029800004553794945394429305451922118663787841796875f,-0.095299979460239825357348308898508548736572265625f,-0.60030000240802738886713996180333197116851806640625f,0.81860000766217666523516527377068996429443359375f,0.75040003984272518611220448292442597448825836181640625f,0.02569995649457013087157974950969219207763671875f,0.3140999759376039701663785308483056724071502685546875f,0.316500015020370284446471487171947956085205078125f,0.8291000558078298610098499921150505542755126953125f,0.14249998971819888282652755151502788066864013671875f,0.3069000033795832305116846328019164502620697021484375f,0.3893000180244445829202959430404007434844970703125f,-0.3223000200510028889766545034945011138916015625f,-0.28210000299811355262136203236877918243408203125f,0.35970001353621494644130507367663085460662841796875f,-0.5470000009164213139456478529609739780426025390625f,-0.5178000218167901902432959104771725833415985107421875f,-1.199600024604797088301211260841228067874908447265625f,0.4382999937057494577175020822323858737945556640625f,-0.6018000010371207242343416510266251862049102783203125f,-0.9045000072121620604548297706060111522674560546875f,-0.07379999971836814776082746902829967439174652099609375f,-0.550199980545043931812188020558096468448638916015625f,-0.458699986144155136269517925029504112899303436279296875f,-0.12689998167976757059705050778575241565704345703125f,-0.5216999852836130546762660742388106882572174072265625f,-0.4988999913692477861104634939692914485931396484375f,-0.17579999538213020304766587287303991615772247314453125f,-0.801299977552890840826194107648916542530059814453125f,0.0306000008672475676263502464280463755130767822265625f,-0.1379999969601630738225139793939888477325439453125f,-0.794699986825883275542992123519070446491241455078125f,-0.090899983978271592377495835535228252410888671875f,0.4524000259324909567482109196134842932224273681640625f,0.1412999995321033708250979543663561344146728515625f,0.555299977746605666339974050060845911502838134765625f,0.1835999846398834733918192796409130096435546875f,0.576900029307604089723326978855766355991363525390625f,-0.29550001573562667545047588646411895751953125f,0.34769999708533283477862596555496565997600555419921875f,0.06599999287724500618423917330801486968994140625f,-0.4995000130981208652514169443747960031032562255859375f,0.27240002313852329507426475174725055694580078125f,0.1413000041887164659470954575226642191410064697265625f,-0.636300015699863497076194107648916542530059814453125f,-0.34380000910609964659414572452078573405742645263671875f,-0.11699999465793364805676901596598327159881591796875f,-0.678000020205974607279131305404007434844970703125f,0.21040001149326581497689403477124869823455810546875f,-0.701000009715556871014996431767940521240234375f,0.1652000006616116678515027160756289958953857421875f,0.51059999919086695996384150930680334568023681640625f,-0.2270000006183980900686947279609739780426025390625f,-1.4184000274896622073583785095252096652984619140625f,-0.6246000389099126692826757789589464664459228515625f,0.9228999971508979882628409541212022304534912109375f,0.545500023037195358455164750921539962291717529296875f,-1.4950999807357785442718522972427308559417724609375f,-0.6643000314354896573831865680404007434844970703125f,-0.9865999883055689423372314195148646831512451171875f,-0.3551999987244602863256659475155174732208251953125f,0.6683000032424928349428228102624416351318359375f,-0.10129998932480788198517984710633754730224609375f,0.5721999849073589050618693363503552973270416259765625f,-0.079599997652322063146357322693802416324615478515625f,0.67400000250339520135867132921703159809112548828125f,0.16080000041127195942181060672737658023834228515625f,0.2024000138998029996173499966971576213836669921875f,0.80200003463029911898729551467113196849822998046875f,-0.30469998805522902074471858213655650615692138671875f,0.2246999962508677839423398836515843868255615234375f,-0.6707999881923196738853221177123486995697021484375f,-0.278800004971027082234513727598823606967926025390625f,-0.4556999974727631297355401329696178436279296875f,0.0778999967038629392845905385911464691162109375f,-0.5217999962687489556856235140003263950347900390625f,-0.5066000286877159197729270090349018573760986328125f,0.49819998480677607499700343396398238837718963623046875f,-0.40050001178681859936858700166339986026287078857421875f,0.457799997863173491197130715590901672840118408203125f,-0.37050000888109213548204934340901672840118408203125f,-0.120600002543628181950197131300228647887706756591796875f,0.67079999839961523822040589948301203548908233642578125f,-0.2958000063344836672030169211211614310741424560546875f,-0.70169999884366962561443870072253048419952392578125f,0.4677000044614076923465972868143580853939056396484375f,0.560699990618228838457071105949580669403076171875f,-0.3171000138759614284111876258975826203823089599609375f,-0.7367999771952629117777178180404007434844970703125f,-0.5519999857172368518121174929547123610973358154296875f,-0.9992999986529351730268899700604379177093505859375f,-0.380700004439055827987203883822076022624969482421875f,0.33239998022317873704878365970216691493988037109375f,-0.036600001660734349240300389283220283687114715576171875f,0.6020999852284789621847949092625640332698822021484375f,0.28619999223500480756143815597170032560825347900390625f,0.7020000066161156127009235206060111522674560546875f,0.8712000191688538119905160783673636615276336669921875f,-0.0257999919176103986728776362724602222442626953125f,0.37409999366998658576477509996038861572742462158203125f,-0.19109998449087139960056447307579219341278076171875f,-0.08940000416189430865188114694319665431976318359375f,-0.299399996972084192492502552340738475322723388671875f,-0.246299998377263573701156929018907248973846435546875f,-0.81359999998807897014785339706577360630035400390625f,-0.6372000201299787836006771613028831779956817626953125f,-0.443399998283386054254151531495153903961181640625f,0.02339999828338627796409099346419679932296276092529296875f,0.65950000184774371092544242856092751026153564453125f,0.750499995768070249368975055404007434844970703125f,0.937700030016899166440680346568115055561065673828125f,0.584599982881545798818478942848742008209228515625f,0.2034000055849549681852295179851353168487548828125f,-0.29659999203085885000064081395976245403289794921875f,0.43960000553131095557546359486877918243408203125f,0.17180002249479253695341185448342002928256988525390625f,-0.6299999856203786219310813976335339248180389404296875f,-0.7382999969467523015964616206474602222442626953125f,-0.7127000223055481864520288581843487918376922607421875f,-1.1018999677658083413689382723532617092132568359375f,-0.7872999987423412449061288498342037200927734375f,-0.170400000447779920165203293436206877231597900390625f,-0.1678000058166682906257705099051236175000667572021484375f,-0.003699977445602797843093867413699626922607421875f,0.4773000374853619831583273480646312236785888671875f,-0.23520000677108754416622105054557323455810546875f,0.71220002903342294331423545372672379016876220703125f,-0.98739998201131840005473350174725055694580078125f,-0.59220001562237722936288264463655650615692138671875f,0.0693999767065047734604377183131873607635498046875f,-0.07440001068115265070446184836328029632568359375f,-1.0738999800443647103520561358891427516937255859375f,-0.58810003387331999391562931123189628124237060546875f,0.91919999601244928300047831726260483264923095703125f,0.1218000068485736164802801795303821563720703125f,0.7287999828428031179328172584064304828643798828125f,0.60640000396072812893777381759718991816043853759765625f,0.4134999948143960324387080618180334568023681640625f,-0.0207999820828437265163302072323858737945556640625f,0.735300028783083092775996192358434200286865234375f,0.8778000099331138006419905650545842945575714111328125f,0.736200013208389236751827411353588104248046875f,0.95099999362230303034948519780300557613372802734375f,0.747299983505904652503204488311894237995147705078125f,0.69070000715851787020937990746460855007171630859375f,-1.33949996470659993264007425750605762004852294921875f,-0.96409998495280746499958013373543508350849151611328125f,0.959999990463256125394764239899814128875732421875f,0.8237999909639359685797899146564304828643798828125f,0.0789000193834308305440572439692914485931396484375f,0.0781999795913694839555319049395620822906494140625f,0.4979999926388263276066936668939888477325439453125f,0.7755999870091676573480299339280463755130767822265625f,-0.04890002057552322867195471189916133880615234375f,0.263800005269050785727813490666449069976806640625f,0.5594999783039096286074709496460855007171630859375f,0.59639999643564234332870910293422639369964599609375f,0.0327000002704560532862387844943441450595855712890625f,0.072499998677521937384682360061560757458209991455078125f,-0.4391999699354176556198581238277256488800048828125f,-0.1119000032305716985092658433131873607635498046875f,-1.448099963092804642883493215776979923248291015625f,-0.8947999866724014594865366234444081783294677734375f,-0.3620999784857037440133353811688721179962158203125f,-0.28169999675750734269286112976260483264923095703125f,-0.4818000025272368702644598670303821563720703125f,-0.10199998486042005652052466757595539093017578125f,-0.074400006806850882412618375383317470550537109375f,-0.2414000093996524487494070854154415428638458251953125f,-0.6725000061839818688014247527462430298328399658203125f,0.153400002306699700937997477012686431407928466796875f,-0.1227000332951551087035113596357405185699462890625f,-0.615800003689527475359000163734890520572662353515625f,0.1326999981284142648974011535756289958953857421875f,-0.342399990057945302623920724727213382720947265625f,-0.5182000141620637378991887089796364307403564453125f,0.474000007718801352751825106679461896419525146484375f,0.64209999700039610548429891423438675701618194580078125f,-0.2773999884188176334731679162359796464443206787109375f,-0.05259997239112834677143837325274944305419921875f,0.13599999317526823006119229830801486968994140625f,-0.934300012809038182837184649542905390262603759765625f,0.146400000607967317733937306911684572696685791015625f,-0.1866999988436699453586697927676141262054443359375f,0.06979998175501833657108363695442676544189453125f,0.1165999765336511728719415259547531604766845703125f,-0.64219998358488095124840810967725701630115509033203125f,0.7345000072121621315091033466160297393798828125f,-0.26019999112486846826186592807061970233917236328125f,0.573600006172060972176041104830801486968994140625f,0.220400001695752134622807716368697583675384521484375f,0.29189999011754963476050761528313159942626953125f,-0.33260000979304304991046592476777732372283935546875f,0.52920001761913315618812703178264200687408447265625f,-0.0971999974429607060955049746553413569927215576171875f,0.666100008928776077254951815120875835418701171875f,-0.53119998981356619793814388685859739780426025390625f,0.55510000427961347924110668827779591083526611328125f,-0.38080001397132878793172494624741375446319580078125f,0.5205000084340571930852092918939888477325439453125f,0.7064999985545872274173007099307142198085784912109375f,-1.1048999846935263491332079865969717502593994140625f,-0.631699989902973157995802466757595539093017578125f,-0.681599975490570386682520620524883270263671875f,-0.5002999866127968431328554288484156131744384765625f,1.0632000123739242525289228069595992565155029296875f,0.962000010490417167829946265555918216705322265625f,-0.68909998644292347336204329621978104114532470703125f,-0.68549998336285378197629825081094168126583099365234375f,0.4181999587297440257316338829696178436279296875f,0.60719997971057892982571502216160297393798828125f,0.5400000157952309365327892010100185871124267578125f,0.739899992591142474651633165194652974605560302734375f,-0.0201000111341471665582503192126750946044921875f,-0.3681000420689581886790620046667754650115966796875f,-1.233899952477216732660281195421703159809112548828125f,-0.839599970662594063952610667911358177661895751953125f,-0.81749998003244428446123492904007434844970703125f,-1.092499997317791127215969027020037174224853515625f,0.214799992297589614764774523791857063770294189453125f,-0.176199992197751953426632098853588104248046875f,-0.8025000026822084464583895169198513031005859375f,-0.844700010406970847043339745141565799713134765625f,-0.24360001608133341477469002711586654186248779296875f,-0.1897999923348425710400988464243710041046142578125f,-1.231799975705147431881414377130568027496337890625f,-1.17389998332262024405281408689916133880615234375f,-0.1128000022590160966728944913484156131744384765625f,0.5202000081121920249671575220418162643909454345703125f,-0.0531000033259392267837029066868126392364501953125f,0.08719998656511318557704726117663085460662841796875f,0.62000001251697511861493694595992565155029296875f,0.308799988132715252930893257143907248973846435546875f,-0.90470004557371197506654425524175167083740234375f,-0.4072000084698199628974180086515843868255615234375f,0.109900003170967153209858224727213382720947265625f,-0.024800000083446516185858854441903531551361083984375f,-0.108699982511996751100014080293476581573486328125f,-0.702799998086690802523435195325873792171478271484375f,0.361900025731325403199889478855766355991363525390625f,0.15719999818801877466967198415659368038177490234375f,-0.163600018204749009242959800758399069309234619140625f,-0.34100001075863828337020322578609921038150787353515625f,1.1208000369936230722345271715312264859676361083984375f,-0.238599996411800407969394655083306133747100830078125f,-0.3996000155150891686162140103988349437713623046875f,-0.3102000024497508778864585110568441450595855712890625f,-0.227900002515315946993723628111183643341064453125f,0.3138000082492826692259768606163561344146728515625f,-0.74310001502335065737270269892178475856781005859375f,-0.338399993440508861652915584272705018520355224609375f,0.4918999948859212878460311912931501865386962890625f,-0.35780000151395796592623810283839702606201171875f,0.637800011460483151637390619725920259952545166015625f,-0.44779999734163278279908126933150924742221832275390625f,0.784000023365020837218253291212022304534912109375f,-0.39200001376867277258497779257595539093017578125f,1.1774000327870250970363485976122319698333740234375f,1.03270001839399316878598256153054535388946533203125f,0.043199992108345242058931034989655017852783203125f,-0.2373000179648396823495204444043338298797607421875f,0.4097999529957778719335692585445940494537353515625f,0.0205999785900115028880463796667754650115966796875f,-0.490199980545042990343063138425350189208984375f,0.0957000055193901744132745079696178436279296875f,-0.145199995595216790889026015065610408782958984375f,-0.2831000074982643610610466566868126392364501953125f,-1.2359999885559087573483338928781449794769287109375f,-0.6459000045418736846158935804851353168487548828125f,0.11699998545646650427443091757595539093017578125f,-0.234700025159120695406045342679135501384735107421875f,-0.62790000240355647864731736262910999357700347900390625f,-0.41200001190602775213989161784411408007144927978515625f,-0.7869000185787680123894460848532617092132568359375f,-1.0222000152498484926155697394278831779956817626953125f,-0.491099984565376956879845238290727138519287109375f,-0.19940000263452528628249638131819665431976318359375f,0.53010002365112285360737587325274944305419921875f,0.176899991905688835913679213263094425201416015625f,-0.350999971009791078557782384450547397136688232421875f,0.23730001304745673706975139793939888477325439453125f,-0.875699995982646584025133051909506320953369140625f,-0.9101000018954277237526184762828052043914794921875f,-0.98610000352710491799967940096394158899784088134765625f,-0.83059999550282948010959671591990627348423004150390625f,-0.99720003275871249570627696812152862548828125f,-1.056400024747848220840751309879124164581298828125f,-0.73140001662671583204655689769424498081207275390625f,0.419400009936094164686437579803168773651123046875f,-0.3436000312805180811892569181509315967559814453125f,-0.67200001123547536963087623007595539093017578125f,-0.74299996984004934574841172434389591217041015625f,0.26859998493790637752454131259582936763763427734375f,0.77710002698898339446031968691386282444000244140625f,-0.243000003516673945824777547386474907398223876953125f,-1.2656000387907031523582190857268869876861572265625f,-0.22360000565052029486423634807579219341278076171875f,-0.073600008034706121406998136080801486968994140625f,-0.44220000027418127075407028314657509326934814453125f,1.003300015908479725368351864744909107685089111328125f,0.020799998772144334679978783242404460906982421875f,-0.3387000030755995538811475853435695171356201171875f,0.374800007683038671757458359934389591217041015625f,0.13440000680685049161411370732821524143218994140625f,-0.05980000351071357300725139793939888477325439453125f,1.124100041353703005597708397544920444488525390625f,0.152400008833408318054125629714690148830413818359375f,-0.321299989473819547214361591613851487636566162109375f,0.30079999117255218354927137625054456293582916259765625f,-0.3734999960064886437294262577779591083526611328125f,0.38440000129342077883620731881819665431976318359375f,-0.80910003300905231071737944148480892181396484375f,-0.122799983781576305119642711360938847064971923828125f,-0.76660002138614657951620756648480892181396484375f,0.23520000677108754416622105054557323455810546875f,1.1635000061392783976543796597979962825775146484375f,0.118600017422437531422474421560764312744140625f,0.17430000928863886855424425448291003704071044921875f,0.9067999941825863885469516390003263950347900390625f,-1.03620000367164522003804449923336505889892578125f,-0.2585999865770336469950052560307085514068603515625f,0.770099962258338877063579275272786617279052734375f,0.926199954795837498977562063373625278472900390625f,-0.6567000235795976692543263197876513004302978515625f,-0.70770000796318033309262318653054535388946533203125f,-0.055799973440169736704774550162255764007568359375f,0.5012000006914139049740697373636066913604736328125f,-0.729899993038177541393451974727213382720947265625f,-0.27499998390674562642743694595992565155029296875f,-0.332400014495849660534076974727213382720947265625f,-0.817700003939866970625871545053087174892425537109375f,0.9000000046193594727128584054298698902130126953125f,0.9381000235915184504165154066868126392364501953125f,-0.245700000452995226396524230949580669403076171875f,-0.41760001798868184863522401428781449794769287109375f,-0.0114000199794777046236049500294029712677001953125f,-0.4300999977231025894752747262828052043914794921875f,0.624599984222650395082609975361265242099761962890625f,0.47309998171925560228601170820184051990509033203125f,0.867599996232986114819141221232712268829345703125f,0.8037000027775764721127416123636066913604736328125f,-0.0515999373435978014867941965349018573760986328125f,0.1961000274062154602461305330507457256317138671875f,0.59760003378391246542378212325274944305419921875f,0.57700000870227796667677466757595539093017578125f,-0.09329999251365705958960461430251598358154296875f,-0.5150999649405481051189781283028423786163330078125f,0.425099993737041881303184709395281970500946044921875f,-0.00159999761879447977008794623543508350849151611328125f,-0.0211999857902522848007720313034951686859130859375f,-0.60939998594522482022739495732821524143218994140625f,-0.560500020503997831156084430404007434844970703125f,-0.174799994868040098339179166941903531551361083984375f,-0.101399980491399421822507065371610224246978759765625f,-0.526600000822543989897894789464771747589111328125f,-0.82249999076127977559735882095992565155029296875f,0.4311999966681003826352025498636066913604736328125f,-0.64260000100135794554034873726777732372283935546875f,0.16000000298023220324239446199499070644378662109375f,0.17880000914335258954679375165142118930816650390625f,-0.045599995130300552403923575184307992458343505859375f,0.451500012338161837988081970252096652984619140625f,-0.282999990850687055399248492904007434844970703125f,-1.4453000134348865657329952227883040904998779296875f,-0.05439998430609715995842634583823382854461669921875f,0.8223000295877458398763337754644453525543212890625f,0.466999997973442049215009319595992565155029296875f,-0.396400016254186748909660309436731040477752685546875f,0.2054000032603739978043222436099313199520111083984375f,0.454400009787082836965055321343243122100830078125f,-0.3135999935805797722565557705820538103580474853515625f,-0.90850003765523457133213014458306133747100830078125f,0.069000008910894250480083655929774977266788482666015625f,-0.130600013160705685777429607696831226348876953125f,-0.499399993544817011326131250825710594654083251953125f,0.4933000011563297704242359031923115253448486328125f,0.2897999887585640532705610894481651484966278076171875f,0.2492000283479693933941234718076884746551513671875f,0.445899990218132591390798324937350116670131683349609375f,-1.052699975180626523751925560645759105682373046875f,-0.4937999927520753118415086646564304828643798828125f,-0.164399969702959225514860008843243122100830078125f,0.1367000167250633779758572927676141262054443359375f,-0.8097000098109248256150749512016773223876953125f,-0.165200003343820345236281355028040707111358642578125f,1.104599990034103029756806790828704833984375f,1.1906999894261360850578057579696178436279296875f,0.5868000011861322917638972285203635692596435546875f,0.86940000263452521522822280530817806720733642578125f,-0.0546000055313111687382843228988349437713623046875f,0.06340000573396675775939002051018178462982177734375f,-0.772499974071980233247813885100185871124267578125f,-0.929499996483325929830243694595992565155029296875f,0.354600053811072513099134084768593311309814453125f,0.9311000119090078186445680330507457256317138671875f,0.5726999699652193243082365370355546474456787109375f,-0.1205000172257424395638736314140260219573974609375f,0.44850001835823061213659457280300557613372802734375f,0.351200005608796939071680753841064870357513427734375f,-0.72059999140501052039553542272187769412994384765625f,-0.66339999411106109761249172152020037174224853515625f,0.2967000506252051028610594585188664495944976806640625f,0.3789000135719773343367933193803764879703521728515625f,-0.8534999778270719872352856327779591083526611328125f,-0.4001999888896943247118542785756289958953857421875f,0.41489996219277447409012893331237137317657470703125f,0.08689999250173574552036370732821524143218994140625f,0.88979997892379714130584034137427806854248046875f,-0.43619999473094939190787044935859739780426025390625f,-0.257499996721744395244968472979962825775146484375f,0.204800001722574176454827465931884944438934326171875f,0.2503000089645386339043398038484156131744384765625f,-0.3124000060021876468141499572084285318851470947265625f,0.181700027453899792817537672817707061767578125f,-0.08079999281167982161377949523739516735076904296875f,-0.3217000226855279976234669447876513004302978515625f,-0.54419999601244928300047831726260483264923095703125f,-0.0633999827861781994897683034650981426239013671875f,-0.15900000205636022343469448969699442386627197265625f,0.4769000145554542768877581693232059478759765625f,-0.01779999793767927940280060283839702606201171875f,0.16140000716447833184474802692420780658721923828125f,-0.3029999884665013354378970689140260219573974609375f,-0.215300004643201869924951097345910966396331787109375f,-0.199599992120266023221120121888816356658935546875f,0.4216000058889388668603714904747903347015380859375f,0.55260000115036955747882529976777732372283935546875f,0.20360000416040424653374429908581078052520751953125f,-0.012799990040063935481384760350920259952545166015625f,0.3882000302553176851461103069595992565155029296875f,-0.046200008141994430843624286353588104248046875f,-0.5351999945521352941568693495355546474456787109375f,-0.438599998944997793159927823580801486968994140625f,0.52689998207092259008277324028313159942626953125f,-0.2393999850511552285325933553394861519336700439453125f,-0.08190000829696675310742648434825241565704345703125f,-0.0066000030577182400293168029747903347015380859375f,0.956099986279010494172325707040727138519287109375f,-0.9549999964237212424222889239899814128875732421875f,0.3786000098228452515058961580507457256317138671875f,0.1856999888300894241410787799395620822906494140625f,-0.67920000897645937953939210274256765842437744140625f,-0.59130001271963106290741052362136542797088623046875f,-0.009600016260147725688511854968965053558349609375f,0.328700010526180363967796438373625278472900390625f,-0.92189997044801685888160136528313159942626953125f,-0.4466999893069267812961697927676141262054443359375f,-0.420899989342689906379746389575302600860595703125f,-0.5446000263929366980164559208787977695465087890625f,-0.02970002381801606361477752216160297393798828125f,0.0769000085949897993486956693232059478759765625f,-0.46409999694824222160605131648480892181396484375f,-0.04939998892545727215974693535827100276947021484375f,-0.184199991989135725134474341757595539093017578125f,0.244500013321637954533116499078460037708282470703125f,0.800099993252754160266704275272786617279052734375f,0.0943999939918516162151718162931501865386962890625f,0.0150000026822088017297573969699442386627197265625f,0.13020001660585389657853738754056394100189208984375f,-0.01350005471706339221782400272786617279052734375f,0.2183999762296675584138938575051724910736083984375f,-0.6134999904930591441143405972979962825775146484375f,-0.0524000150918961793422568007372319698333740234375f,-0.60569997094869609810530164395458996295928955078125f,-0.62709996887445473845446031191386282444000244140625f,0.09689998922348053156383684836328029632568359375f,0.27950000810623176761282593361102044582366943359375f,-0.703199981379508898271524230949580669403076171875f,0.1462000005424022308631037958548404276371002197265625f,0.95670002536773690593463470577262341976165771484375f,0.01619999473094946296214402536861598491668701171875f,-0.509900045788288736048343707807362079620361328125f,-0.122800003451108974417138597345910966396331787109375f,-0.284900029695034362475780653767287731170654296875f,0.270399999237060484347239253111183643341064453125f,1.002400001382827809948139474727213382720947265625f,-0.57819999344944950081526258145458996295928955078125f,0.247899998342990812716379878111183643341064453125f,-0.122599984908104087111269109300337731838226318359375f,-1.350799990129470273814149550162255764007568359375f,-0.1103999971508979882628409541212022304534912109375f,0.4153000265479089847531213308684527873992919921875f,0.5609999958574771738994968472979962825775146484375f,-0.10410000946521780207376650650985538959503173828125f,0.354400001591443913895318473805673420429229736328125f,-0.8523000012755392873486925964243710041046142578125f,0.433399999773502297983895914512686431407928466796875f,0.670800012183189409142869408242404460906982421875f,-0.04939999667406080874343388131819665431976318359375f,-0.08909999054074280167014876496978104114532470703125f,0.198799999010562888912545531638897955417633056640625f,0.866799997162818414153662160970270633697509765625f,0.18620000039339057451570624834857881069183349609375f,1.47030003459453606495799249387346208095550537109375f,-0.2081999970257282228658368694595992565155029296875f,-0.409900000488757854100185795687139034271240234375f,0.1384000078201292893709251075051724910736083984375f,-0.6397000378251078700486687012016773223876953125f,0.38190000561475745399775405530817806720733642578125f,-0.259500031948089571187665569595992565155029296875f,-0.60610001161098470134902527206577360630035400390625f,-0.1931999712467185048581086448393762111663818359375f,-0.1406999941945077381433293339796364307403564453125f,0.4272000244140627955857780762016773223876953125f,0.054100004398822676421332289464771747589111328125f,-1.436099982404709152206123690120875835418701171875f,-1.4903999795675275663597858510911464691162109375f,-0.1904999877214432757455142564140260219573974609375f,0.0063000222563740493342265835963189601898193359375f,0.21960002222061092425064998678863048553466796875f,0.797599980735778757434673025272786617279052734375f,0.878699970591068080238983384333550930023193359375f,0.2774999794363974814359607989899814128875732421875f,0.6474000025749209186187727027572691440582275390625f,0.36309999766349765337736243964172899723052978515625f,1.0743000019311903514562800410203635692596435546875f,1.092499990165233469951999722979962825775146484375f,-0.188700017380714513137718313373625278472900390625f,-0.2017999864339827809089911170303821563720703125f,0.45359998560845848913913869182579219341278076171875f,0.44849999421834940704911787179298698902130126953125f,0.156299973082543175451064598746597766876220703125f,-0.14779999287128475771169178187847137451171875f,-0.87030001909732845888356678187847137451171875f,-1.30950000751018524880464610760100185871124267578125f,0.27480002899169875263396534137427806854248046875f,0.134299995076656131232084590010344982147216796875f,-0.0341999526500700312681146897375583648681640625f,-0.7011999941349029707993167903623543679714202880859375f,-0.18019999216794957419551792554557323455810546875f,0.32320000194311138130842664395458996295928955078125f,0.73500001884996901679869552026502788066864013671875f,0.009400005316734294513736358567257411777973175048828125f,0.0772000172615054935931766522116959095001220703125f,0.1924000083863735000022643362171947956085205078125f,-0.238600000211596441346273422823287546634674072265625f,0.064799995613098182456468521195347420871257781982421875f,-0.6588000142097474309821336646564304828643798828125f,-0.57659999769330028129843412898480892181396484375f,-0.897000033400952911932790811988525092601776123046875f,-0.3845999934241175477467322707525454461574554443359375f,-0.20270000125765808007827217807061970233917236328125f,0.34959999405741692957150235088192857801914215087890625f,1.1797000425934793810256451251916587352752685546875f,-0.06060001032948481025641740416176617145538330078125f,0.3281999938219783974346910326858051121234893798828125f,0.1253999977469444360167472041212022304534912109375f,0.8114000257164242224661165892030112445354461669921875f,-0.071199994879961092664188981871120631694793701171875f,-0.23150000190734854044194435118697583675384521484375f,-0.544999983012676381122219027020037174224853515625f,0.860700009691715450799165409989655017852783203125f,-0.53359998270273223397452966310083866119384765625f,-0.065900044476986607833168818615376949310302734375f,-0.709599993759393754544362309388816356658935546875f,0.575300008741021162705919778090901672840118408203125f,-0.007800002333521849351427590590901672840118408203125f,0.2706999822735784988481100299395620822906494140625f,-1.21369999264478689582347215036861598491668701171875f,0.7151999905288217718180021620355546474456787109375f,0.660999999284744177430184208787977695465087890625f,-0.1113000172644855734205293629202060401439666748046875f,-0.2095999907791614713659100743825547397136688232421875f,0.4247999979972831852137460373342037200927734375f,0.4250000137090683693941173260100185871124267578125f,0.1583999986559152528542426807689480483531951904296875f,0.1365999918818474601067691764910705387592315673828125f,-0.831299978744983292244796757586300373077392578125f,-0.1292000051021577888832325697876513004302978515625f,-0.3908999592982234094762361564789898693561553955078125f,0.282200031045079402503006349434144794940948486328125f,0.695399987167120148257026812643744051456451416015625f,0.496899991309642796721846025320701301097869873046875f,-0.424500053405761690328290569595992565155029296875f,-0.7161000232338903259687867830507457256317138671875f,0.08549999651312845116990501992404460906982421875f,-0.2015999956816434401929427622235380113124847412109375f,-0.46620000218153023752165609039366245269775390625f,-0.8576000165730717217371648075641132891178131103515625f,-0.8276999704122545153950341045856475830078125f,-0.16589998576641118432917210157029330730438232421875f,-1.37519996206760453105744090862572193145751953125f,-1.525499993979931190324350609444081783294677734375f,-1.2533999678850165082621970213949680328369140625f,-0.659099971020221886419676593504846096038818359375f,-0.303300003726780342816482516354881227016448974609375f,-0.702500013113021903876642682007513940334320068359375f,-1.0460999615430832676565842120908200740814208984375f,-0.276800005656480774263172861537896096706390380859375f,0.426900007067620546052921781665645539760589599609375f,0.3185999934315681958452159960870631039142608642578125f,0.76029997588396014407408074475824832916259765625f,-0.60899999937415127959638994070701301097869873046875f,1.0466000315189365466039816965349018573760986328125f,-0.0753999895513057794005362666212022304534912109375f,-0.286300013911723993942359811626374721527099609375f,0.0094000041246413701401252183131873607635498046875f,0.434300006997585086310209590010344982147216796875f,0.01579998565912266172972522326745092868804931640625f,-0.32830001426935240260718273930251598358154296875f,0.37659998726248744560507475398480892181396484375f,0.1541000002264976131982621154747903347015380859375f,-0.497600001895427634934776506270281970500946044921875f,-0.4900000225752594307238041437813080847263336181640625f,0.3818000050604342110460720505216158926486968994140625f,-0.899900024330616332690624403767287731170654296875f,0.27199999484419823403413829510100185871124267578125f,-0.423600013622641657917711199843324720859527587890625f,0.123000001430511429756364805143675766885280609130859375f,-0.6999000305891041051609136047773063182830810546875f,-0.406800010275840673301672723027877509593963623046875f,0.0371000076174734516598618938587605953216552734375f,0.3763999904751778036171572239254601299762725830078125f,-0.8867000056982039524200445157475769519805908203125f,-0.11539999700188641185150117962621152400970458984375f,-1.0063000252366069986464935936965048313140869140625f,0.6223999931871890822776549612171947956085205078125f,-0.25130000348091119377613722463138401508331298828125f,-0.559000014424324120909659541212022304534912109375f,0.53069996457845014514731474264408461749553680419921875f,0.28069999575912951694789398970897309482097625732421875f,-1.467899983143805542340487590990960597991943359375f,-1.500699995386600704705415409989655017852783203125f,-0.36990001151561813230728148482739925384521484375f,-0.5305999839544295326732026296667754650115966796875f,-0.09569996051788365321044693700969219207763671875f,-0.355499999940395383646318805404007434844970703125f,0.073499948024750239028435316868126392364501953125f,0.2211000047564510850861552171409130096435546875f,0.83249997258186425597159541212022304534912109375f,0.74589998904466625617715180851519107818603515625f,-0.9180000020265577376221699523739516735076904296875f,-0.53329999817609774481752538122236728668212890625f,0.78150002291798603692996039171703159809112548828125f,0.5540000063031909061095348079106770455837249755859375f,0.871799995076656841774820350110530853271484375f,1.16530000628232954795748810283839702606201171875f,0.3207000142365694639323692172183655202388763427734375f,0.3202000023007391948937083725468255579471588134765625f,-0.557399990803002509665020625106990337371826171875f,-0.054100004398822676421332289464771747589111328125f,0.76019996281266255522268693312071263790130615234375f,0.61409998711347579813946140347979962825775146484375f,0.1077000146687030479597524390555918216705322265625f,0.5934999948143957482216137577779591083526611328125f,1.55219998686313687130677863024175167083740234375f,1.6353000033020972381336832768283784389495849609375f,0.107099998676776664296994567848742008209228515625f,0.3711999905586242931576634873636066913604736328125f,-0.398800021958351180728641338646411895751953125f,-0.042199991780519585660158554674126207828521728515625f,0.6880000199079514544564517564140260219573974609375f,0.3881999967277049989888837444595992565155029296875f,-0.217700004237890265557098246063105762004852294921875f,0.49140000746250134255888042389415204524993896484375f,0.7394000047206876757854843162931501865386962890625f,0.547399991995096257824116037227213382720947265625f,-0.60480000365972497178290723240934312343597412109375f,-0.0603999964058399285704581416212022304534912109375f,-0.1243000007390975270027411170303821563720703125f,-0.15799999740719794516508045489899814128875732421875f,-0.545100005173683399561923579312860965728759765625f,0.6847999845862389189932173394481651484966278076171875f,-1.16819999717473965716862949193455278873443603515625f,0.37079999564290044844483418273739516735076904296875f,0.310800001193582975389517741859890520572662353515625f,-0.1549999967217445540068609943773481063544750213623046875f,-0.59520001787245302438122962485067546367645263671875f,-0.069000015616416832120449953436036594212055206298828125f,0.2359000113964082601114569115452468395233154296875f,0.1768000114679335599277010260266251862049102783203125f,1.0178000143289569479065903578884899616241455078125f,-0.5883999713122847197155351750552654266357421875f,1.2534000245094301106973944115452468395233154296875f,-0.03759998729228986036332571529783308506011962890625f,0.34920000711381415925416149548254907131195068359375f,0.165199988293647848092149388321558944880962371826171875f,-0.20159997221231407138475333340466022491455078125f,0.778300006520748155480760033242404460906982421875f,-0.0944999562501909196043925476260483264923095703125f,-0.1675999908685685824139000033028423786163330078125f,0.4454999775886534507662872783839702606201171875f,-0.1840000126361847065936672152020037174224853515625f,0.868500017464160833924324833787977695465087890625f,0.409099992626904995773884365917183458805084228515625f,0.75989996800422687783793662674725055694580078125f,0.0626000015974046419842125033028423786163330078125f,0.04800002169609030033825547434389591217041015625f,0.26140000120401385430568552692420780658721923828125f,-0.2195999934613706017216827603988349437713623046875f,-0.002299994122981985356091172434389591217041015625f,1.44359996660947853541756558115594089031219482421875f,1.17149998074769978728681962820701301097869873046875f,1.070399996256828245577708003111183643341064453125f,1.27239997783899294603315865970216691493988037109375f,-0.35849999887496231121275513942237012088298797607421875f,-0.31839999329149726303711531727458350360393524169921875f,-0.0297000052660696400863571398076601326465606689453125f,0.0821999780714512962020990016753785312175750732421875f,0.33960000165700865437656830181367695331573486328125f,0.28050001484155639985829111537896096706390380859375f,-1.50839994642734609442413784563541412353515625f,-1.6471999838829045614829738042317330837249755859375f,-0.488399999475479518196152639575302600860595703125f,-0.9709999957084658461781145888380706310272216796875f,0.2381999629765754189492099612834863364696502685546875f,0.0667999871790410093552736725541763007640838623046875f,1.01369998310804287910968923824839293956756591796875f,0.525999997645616446106942021287977695465087890625f,-0.135400006240606440854890024638734757900238037109375f,-0.00960000299811358814849882037378847599029541015625f,-0.1707999770164487784995799302123486995697021484375f,0.08979998130798350075565394945442676544189453125f,-0.196799995262920912608706203172914683818817138671875f,0.1589999962076544814948420025757513940334320068359375f,-0.042600017206371187938884759205393493175506591796875f,-0.35760000576972961194854860877967439591884613037109375f,-0.73340003762245231655469979159533977508544921875f,-0.508400014525651755548096843995153903961181640625f,0.71920002447366737641232248279266059398651123046875f,-0.1721999940156937469737385981716215610504150390625f,-0.2763000246405606930011344957165420055389404296875f,0.675799994003772663830886813229881227016448974609375f,-0.9447000175595281490359411691315472126007080078125f,-0.17759998699426660095923580229282379150390625f,0.55620000694990157086294857435859739780426025390625f,-0.3039999987781047963153469027020037174224853515625f,0.6560000271871688770630726139643229544162750244140625f,-0.39620000426769241830271539583918638527393341064453125f,0.82520001481771476647963936557061970233917236328125f,-0.44720000295639028564664840814657509326934814453125f,0.9977000225663186228075574035756289958953857421875f,-0.4846000092566011829831040813587605953216552734375f,-0.0090000229924920294166668099933303892612457275390625f,-0.279199992138147390363656086265109479427337646484375f,-0.5284999715685838594936285517178475856781005859375f,0.44420000763535494314737661625258624553680419921875f,-1.059700043189526041942372103221714496612548828125f,1.20739999154806110226445525768212974071502685546875f,0.1013999906241895843095335294492542743682861328125f,0.294000006824731929810923247714526951313018798828125f,0.06689997313022644220836809836328029632568359375f,-0.2038000571250915271548365126363933086395263671875f,0.4400999802701177632258122685016132891178131103515625f,0.549599992343783316783856207621283829212188720703125f,-0.67139999781399894995814747744589112699031829833984375f,-0.44400000436604025733089429195388220250606536865234375f,-0.64800000143051050827125436626374721527099609375f,0.09320000760555302576904068700969219207763671875f,-0.605699999558925838982759159989655017852783203125f,-0.85900001591444041793010910623706877231597900390625f,1.3058999887466438138972080196253955364227294921875f,0.87569999955892541265711770392954349517822265625f,-0.063599949920178033835327369160950183868408203125f,0.3559000164628027818025657325051724910736083984375f,-0.7829999984502791221530060283839702606201171875f,-0.85560000630617150108037094469182193279266357421875f,-0.89070002172440310150847153636277653276920318603515625f,-0.76840000960826848608320460698450915515422821044921875f,-1.076400012826919549979720613919198513031005859375f,-1.04840000156164148137349911849014461040496826171875f,-1.2405000240802763755709747783839702606201171875f,-1.3221000153660771303520959918387234210968017578125f,-0.4049999846518037571740933344699442386627197265625f,-0.0119999673515558402669967108522541821002960205078125f,1.1114999914765348876244388520717620849609375f,1.2124999967217444662992420489899814128875732421875f,-0.094499981284140943671445711515843868255615234375f,0.07060000258088106050990973017178475856781005859375f,0.475400012350082334933176753111183643341064453125f,-0.0021999922275544037120198481716215610504150390625f,-1.18279998899698224334997576079331338405609130859375f,0.064599990481138291897877934388816356658935546875f,-0.25699999611079693551118907635100185871124267578125f,0.336199989962577883151340074618929065763950347900390625f,0.7887999784469599262592964805662631988525390625f,-0.053399993216991514799474316532723605632781982421875f,-0.48200000512599938673474753159098327159881591796875f,0.1131999998569488408151073599583469331264495849609375f,-0.3241000167667866804777077049948275089263916015625f,-0.405199999618530259937188020558096468448638916015625f,1.0006000310420990473403435316868126392364501953125f,0.3083999908328056704931441345252096652984619140625f,-0.0838000112295151922126024146564304828643798828125f,-0.5370000039339064556997982435859739780426025390625f,0.4562000042676925826157230403623543679714202880859375f,-0.316599994266033191170350846732617355883121490478515625f,0.1898000040322542769644087456981651484966278076171875f,0.151399991965293967410843833931721746921539306640625f,0.213900014130771420894916445831768214702606201171875f,-0.48000000208616244723458521548309363424777984619140625f,0.37000000476837158203125f,-0.2497999945700168922257944359444081783294677734375f,0.3230000110417605707624488786677829921245574951171875f,0.105999995261430779436295779305510222911834716796875f,-0.448400006665289563301257658167742192745208740234375f,0.5181999912142754016741719169658608734607696533203125f,0.0627000237584116604239170555956661701202392578125f,-0.1022000011682511200206135981716215610504150390625f,0.02729998130798350075565394945442676544189453125f,0.172400010919570689793545170687139034271240234375f,0.47489999423027029479271732270717620849609375f,0.83919996427297594010497050476260483264923095703125f,0.6896999793380500509698549649328924715518951416015625f,0.5243999843060971333130737548344768583774566650390625f,-0.3338999931573862767208993318490684032440185546875f,-0.5661999720811845548951168893836438655853271484375f,0.32460001000165927820262368186376988887786865234375f,0.359199996459484083288771216757595539093017578125f,-0.69719997695386393132821467588655650615692138671875f,-0.3780999970674514543134137056767940521240234375f,0.2345999548673631807105266489088535308837890625f,-0.3926000260353088577858216012828052043914794921875f,-0.9440999939680096986194257624447345733642578125f,-0.4927999940633771558395892498083412647247314453125f,-0.6902999956279993742924716570996679365634918212890625f,-0.567199990364909201190357634914107620716094970703125f,0.1196999920785426496649961336515843868255615234375f,-0.2360000100880859097429720350191928446292877197265625f,-0.96659999959319853335415473338798619806766510009765625f,-0.72690000792443731914005411454127170145511627197265625f,-0.52469999878406525795071502216160297393798828125f,-0.39039999268054970116281765513122081756591796875f,-0.0024000068962575227260458632372319698333740234375f,-0.3688000136882065849164291648776270449161529541015625f,1.08089998386651320316786950570531189441680908203125f,0.84799998839199541844635632514837197959423065185546875f,0.398399978017806688512791879475116729736328125f,-0.45659999575614929057110202847979962825775146484375f,0.739899991548060853574497741647064685821533203125f,-0.06660000261068343974102390347979962825775146484375f,-0.82650002276897449604575740522705018520355224609375f,0.02019999842643738219294391456060111522674560546875f,0.54629999237954596225108616636134684085845947265625f,-0.551799992990493759492665049037896096706390380859375f,0.28010000845193872009986080229282379150390625f,-0.033799990516901079473655045148916542530059814453125f,-0.1544999887347220379751888685859739780426025390625f,0.49019999157190330407729561557061970233917236328125f,0.8334000191450119388036910095252096652984619140625f,0.1253999977469444360167472041212022304534912109375f,-0.288500000327825745216614450328052043914794921875f,0.70919998424053198871064296326949261128902435302734375f,0.1190999909877774598498945124447345733642578125f,-0.26640000954866405180609945091418921947479248046875f,-0.185699983465671181193101801909506320953369140625f,0.38340000692605968879433930851519107818603515625f,0.3039000101447104640328689129091799259185791015625f,0.016999993503093779878554414608515799045562744140625f,0.221300008547306248374297865666449069976806640625f,-0.04220000072121621315091033466160297393798828125f,0.52399999684095366347946765017695724964141845703125f,0.18699999767541886086519298260100185871124267578125f,0.08620000806748873145579636911861598491668701171875f,0.306000000923871973146361824547057040035724639892578125f,-0.2328999904453752378685749135911464691162109375f,0.16500000566244121813497258699499070644378662109375f,1.0480000157356261780705608543939888477325439453125f,-1.09009999325275419579384106327779591083526611328125f,-0.234900026416777762960919062606990337371826171875f,-0.24849998646974569282974698580801486968994140625f,0.327300011706352478313419851474463939666748046875f,0.67310000720024110165695674368180334568023681640625f,-1.1582999873727557460512116449535824358463287353515625f,-0.9104999848902226577962437659152783453464508056640625f,-0.0399000007867815753570539527572691440582275390625f,0.0759000152707098862947532325051724910736083984375f,1.003499970674514685242684208787977695465087890625f,0.7994999917149545609618144226260483264923095703125f,-0.0381000232934951554852887056767940521240234375f,-0.4504000081777572717101065791212022304534912109375f,1.391399962908030030206418814486823976039886474609375f,0.977299980115890587484273055451922118663787841796875f,-0.6635999992430203775484187644906342029571533203125f,-0.755500020802021055033037555404007434844970703125f,0.8660999943256371125244186259806156158447265625f,0.838300008904933946496385033242404460906982421875f,0.0164999709129336480373240192420780658721923828125f,0.034299993288517072187460144050419330596923828125f,-0.312899988210201485117067932151257991790771484375f,-0.657700017201900521968127577565610408782958984375f,0.123899993455410140086314640939235687255859375f,-0.30419999586343760000772817875258624553680419921875f,0.5813999879419802230273717214004136621952056884765625f,0.4082000011980533304978280284558422863483428955078125f,0.48149999661743603240893207839690148830413818359375f,0.633700006651878311458858661353588104248046875f,-0.996000004053115617352887056767940521240234375f,-0.56859998195767413875501006259582936763763427734375f,1.072300023925304657268497976474463939666748046875f,0.31240001136660566771752201020717620849609375f,-0.0273000015735629375512871774844825267791748046875f,0.4241999834954739423409364462713710963726043701171875f,0.7153000012345611668962419571471400558948516845703125f,-0.14640000306665890139612429265980608761310577392578125f,-0.2333000285744670776466591632924973964691162109375f,0.474799997997283895756481797434389591217041015625f,0.1069999970793720223127820645458996295928955078125f,0.5673999829053879562223983157309703528881072998046875f,0.20580000950098042977742807124741375446319580078125f,0.0283999970912933719091597595252096652984619140625f,-0.0564000003099440760934157879091799259185791015625f,0.114999987035989903461086214520037174224853515625f,-0.9826000144869089236721038105315528810024261474609375f,0.231799988818168767323868451057933270931243896484375f,0.221400033837557241866988988476805388927459716796875f,-0.23559999602437020627121455618180334568023681640625f,-0.686700034010410576001959270797669887542724609375f,0.4484000102043150803865501075051724910736083984375f,-0.094399998015165298426154549815692007541656494140625f,-0.027199996846914320514088103664107620716094970703125f,-0.34860000267028823373038903810083866119384765625f,-0.06200002017617212146660676808096468448638916015625f,-0.2244000002503394597397345933131873607635498046875f,0.5522000054895876974825341676478274166584014892578125f,0.62820001077204945527654444958898238837718963623046875f,-0.1263999974787235469619872674229554831981658935546875f,0.6677000425338750488890582346357405185699462890625f,-1.1597999759435655420247712754644453525543212890625f,0.24959999361038143206314998678863048553466796875f,-0.353699986684322453811546438373625278472900390625f,0.79709996082782819115664096898399293422698974609375f,0.51409997996091849614685997948981821537017822265625f,-0.517200008358061236179992192774079740047454833984375f,-0.717299992353468862660292870714329183101654052734375f,0.85500001132488279154131305404007434844970703125f,0.4349000118136405745872252737171947956085205078125f,1.02089995596408922295950105763040482997894287109375f,0.58879998321533211225187187665142118930816650390625f,0.01499998569488525390625f,-0.03340000454187386225157752051018178462982177734375f,0.083699994730949356380733661353588104248046875f,-0.2364000152111049857239777338691055774688720703125f,0.7358999966442585360937300720252096652984619140625f,0.9335999970823529725549860813771374523639678955078125f,-0.392400016582012511889843153767287731170654296875f,-0.108000014692545054373340462916530668735504150390625f,1.10580002589225756537416600622236728668212890625f,1.150700038897991106523477355949580669403076171875f,0.0330000002384185364689983543939888477325439453125f,0.182200002807378691471740239649079740047454833984375f,-0.134400017833709028991506784223020076751708984375f,-0.190000038743018961895359097979962825775146484375f,1.12230000008344621420519615639932453632354736328125f,0.86880000884532915250701989862136542797088623046875f,-0.43920000487864019333983378601260483264923095703125f,-0.59700000113993889616637034123414196074008941650390625f,0.6827999810993663487579397042281925678253173828125f,0.348400006777048021344711514757364057004451751708984375f,0.694100004994869124175238539464771747589111328125f,0.012800014477968080228720282320864498615264892578125f,-0.06840001206696033619891750277020037174224853515625f,-0.473599992835521732548187401334871537983417510986328125f,1.297600054645538847353236633352935314178466796875f,0.069399998462200240822994601330719888210296630859375f,-0.70270001988410957238784249057061970233917236328125f,-0.18319999568462375094668459496460855007171630859375f,1.370000008940696289982952293939888477325439453125f,0.070399999982118544039622065611183643341064453125f,0.335600011372566342515710857696831226348876953125f,0.6461999965190887706967259873636066913604736328125f,-0.67590002599954601691933930851519107818603515625f,0.006600014829635458823986482457257807254791259765625f,-0.217100004152953385272439845721237361431121826171875f,-0.37159998860359200190117689999169670045375823974609375f,-0.2211000142931940359858344891108572483062744140625f,-0.455800000709295272116605701739899814128875732421875f,-0.2082000189304356041475330130197107791900634765625f,0.0647999959111213996720834984444081783294677734375f,0.293299996387958117338712327182292938232421875f,0.02519999365806580016169391456060111522674560546875f,0.65329999102354019413496644119732081890106201171875f,-0.0073999893128873228675956852384842932224273681640625f,-0.20389998451471313956062658689916133880615234375f,0.690599988126754826822661925689317286014556884765625f,0.220500009477138547708818805404007434844970703125f,0.27419999839663500296182974125258624553680419921875f,-0.23580000167787051879031423595733940601348876953125f,0.5379000122010706430586424175999127328395843505859375f,-0.0414000032901764569714941899292171001434326171875f,-0.0370999736428263560128470999188721179962158203125f,-0.9059999677687884389598593770642764866352081298828125f,-0.6367999898612499709571466155466623604297637939453125f,-0.2037000453948980549512270954437553882598876953125f,-0.6002000145196912939127287245355546474456787109375f,-0.109199974107742292517286841757595539093017578125f,0.29670001046657556997843130375258624553680419921875f,-0.22140003547668385408542235381901264190673828125f,-0.639799996805191284465763601474463939666748046875f,1.0514999810457226203652680851519107818603515625f,0.402199999976158295567074674181640148162841796875f,0.0885000594854350453033475787378847599029541015625f,0.0937000170826909339893973083235323429107666015625f,-0.66479996182471567944816115414141677320003509521484375f,-0.40229998823702339816321682519628666341304779052734375f,-0.7640999889016146795484019094146788120269775390625f,-0.34290000072717674584055203013122081756591796875f,0.3287999838113790929128299467265605926513671875f,0.6142000158309937063449979177676141262054443359375f,0.05339997056722722845734097063541412353515625f,0.101899996078014254408117267303168773651123046875f,-0.17189999369382835681108190328814089298248291015625f,-0.3256999778032305670194546109996736049652099609375f,1.3088999681234358973824782879091799259185791015625f,0.8384999769330025998215205618180334568023681640625f,0.354899993038177541393451974727213382720947265625f,-0.0677000172019006640766747295856475830078125f,-0.455399969844520580153357514063827693462371826171875f,0.03180000982880581972267464152537286281585693359375f,-0.94120001410245901496409715036861598491668701171875f,0.2723999991476535598167174612171947956085205078125f,-0.587499987781047394719280418939888477325439453125f,0.022599993699789155954249508795328438282012939453125f,-0.2021000007629396577613078989088535308837890625f,0.5495999934613704507313514113775454461574554443359375f,0.14289997651278962820242668385617434978485107421875f,-0.371999994024634350608948807348497211933135986328125f,0.82800002128630889597360464904340915381908416748046875f,-0.1508000074148176938848564532236196100711822509765625f,0.65009999116659145101948524825274944305419921875f,0.40640000269412990263617757591418921947479248046875f,0.226100004663318421282980352771119214594364166259765625f,-0.07160000030100344137196088922792114317417144775390625f,0.247099977219104260939275263808667659759521484375f,-0.220400002217292723116770503111183643341064453125f,0.2402999889969823499313861248083412647247314453125f,0.0432000166952608566361959674395620822906494140625f,0.8885000115036962853309887577779591083526611328125f,0.05259999071955689942114986479282379150390625f,0.291600019598007520471583120524883270263671875f,-0.381799997758865305996778261032886803150177001953125f,0.4563000139117239228880862356163561344146728515625f,0.5939999867081642292987453402020037174224853515625f,-0.69740001300573339904076419770717620849609375f,-0.2149999982118606656200654470012523233890533447265625f,-0.8617999939590690328117261742590926587581634521484375f,0.1883999935150146853857222595252096652984619140625f,-0.53989999661445597922693195869214832782745361328125f,-0.1310999979764222445055565913207828998565673828125f,-0.069600001209974227123211676371283829212188720703125f,0.67190001686513423084790019856882281601428985595703125f,-0.097799996596574789720079934340901672840118408203125f,0.724399997866153722014814775320701301097869873046875f,0.25250000171363351508801997624686919152736663818359375f,0.279800005559623254924872526316903531551361083984375f,-0.1532999991819262441961058129891171120107173919677734375f,-0.73459999945908782681414095350191928446292877197265625f,0.092000008702278091021753425593487918376922607421875f,0.5857999944508076595184320467524230480194091796875f,-0.37280000397264956291110138408839702606201171875f,0.453499996453523568362697915290482342243194580078125f,0.089599999794363949234821120626293122768402099609375f,0.36559999326765535432315346042742021381855010986328125f,0.264600000539422008927203933126293122768402099609375f,-0.2215000066757202290546047152020037174224853515625f,0.160399999311566343607182716368697583675384521484375f,0.26389999703168864897406820091418921947479248046875f,0.00510000001415610026977986990459612570703029632568359375f,0.0313999993041157721140876901699812151491641998291015625f,-0.231299994987249402100815132143907248973846435546875f,0.012699992018938122129156909068115055561065673828125f,-0.090799997207522409325974876992404460906982421875f,-0.828500003457069311707527958787977695465087890625f,-0.155499997630715369734133446399937383830547332763671875f,-0.0843999998778104387753273840644396841526031494140625f,0.167000001400709141563538651098497211933135986328125f,-0.4246000029981136236756356083787977695465087890625f,0.2123000008285045314693206819356419146060943603515625f,0.412600017690658571467565707280300557613372802734375f,-0.43169999325573460424720906303264200687408447265625f,0.237700001183152043182644774788059294223785400390625f,0.68209999361038153864456035080365836620330810546875f,0.9741000337541105835725829820148646831512451171875f,0.444499981880188244076634873636066913604736328125f,0.4062999906659126025942896376363933086395263671875f,-0.74979999531507512955386118846945464611053466796875f,-0.33190000650882733879143415833823382854461669921875f,0.392099995584785698810037501971237361431121826171875f,-0.0020000098571182345352781339897774159908294677734375f,-0.673399949705601130744980764575302600860595703125f,-0.0600999926567080677841659053228795528411865234375f,-0.32289999998211893483812673366628587245941162109375f,0.38630000154376009646739476011134684085845947265625f,0.573299986553191587290712050162255764007568359375f,-0.1213000015437604162116258521564304828643798828125f,0.7899999709427356631152861154987476766109466552734375f,0.851299997892975834901108100893907248973846435546875f,0.83809997084140785972294906969182193279266357421875f,-0.178100027912855285450177689199335873126983642578125f,-0.06080000532865492690604014569544233381748199462890625f,-0.707700010161101733530131241423077881336212158203125f,0.77389998779296842457142702187411487102508544921875f,0.225900013035535973671130705042742192745208740234375f,-0.099599984967708277139308847836218774318695068359375f,-0.13710000061392779713287382037378847599029541015625f,0.9074999663233764835013062111102044582366943359375f,0.306399983322620528269908390939235687255859375f,0.238099989318847793384748001699335873126983642578125f,-0.0853000058352948453688213703571818768978118896484375f,0.037299999338388456493476041941903531551361083984375f,0.41000000469386577162111962024937383830547332763671875f,-0.128500007405877081367862047045491635799407958984375f,-0.554000001013279064210337310214526951313018798828125f,-0.06689999861121176394362919381819665431976318359375f,-0.38849999555945391449540693429298698902130126953125f,0.1013999982237816510632910649292171001434326171875f,0.664400021559000020232588212820701301097869873046875f,0.047499994225800090674738385132513940334320068359375f,-0.5691999853953719767929442241438664495944976806640625f,0.34099999615550036224931318429298698902130126953125f,0.012599988037347742420024587772786617279052734375f,-0.087500001788139325498150355997495353221893310546875f,0.5609000036478042350296391305164434015750885009765625f,-0.44290000318586830729827852337621152400970458984375f,0.34719997911453237549039840814657509326934814453125f,-0.12800000314414494706483083064085803925991058349609375f,-0.67589998442530629940705466651706956326961517333984375f,-0.2229000009506940660486407068674452602863311767578125f,-0.46499999523162838244161321199499070644378662109375f,0.0508000027954578492472847983663086779415607452392578125f,-0.010599994124472100764933202299289405345916748046875f,-0.2443000037938356516775684212916530668735504150390625f,-0.36020001124143607995620186557061970233917236328125f,0.2360000019669532544952517127967439591884613037109375f,-0.4170000025928020104259985600947402417659759521484375f,-0.13669999347925188004637675476260483264923095703125f,-0.76750000298023213218812088598497211933135986328125f,-0.05609999924302100371420465307892300188541412353515625f,-0.33630000437498086540699659963138401508331298828125f,0.341499983429908837706534541212022304534912109375f,-0.18449999469518640893284100457094609737396240234375f,-0.30439996210336772719529108144342899322509765625f,-0.835299993467331081120619273860938847064971923828125f,0.3022999935269350402222698903642594814300537109375f,1.17460004457235367425482763792388141155242919921875f,0.274499990224838175123522887588478624820709228515625f,-0.0809000100553036727291100760339759290218353271484375f,0.66909999188184787755062643554992973804473876953125f,1.1150000002980231528226795489899814128875732421875f,0.11650000563263862574103768565692007541656494140625f,-0.508700025278330247857638823916204273700714111328125f,0.508600001291930059466039892868138849735260009765625f,0.5708000248305500345935570294386707246303558349609375f,0.53500001065433000491111670271493494510650634765625f,-0.387300027427077697694812741247005760669708251953125f,0.3941999896049497298378128107287921011447906494140625f,0.68550001223385337301152731015463359653949737548828125f,-0.0035000186562541557577787898480892181396484375f,-0.3902000316560272352717220201157033443450927734375f,0.05429999120235462495287492856732569634914398193359375f,1.011000010497868029091250718920491635799407958984375f,0.73149998909234970057013924815692007541656494140625f,0.155600001984834701573845450184307992458343505859375f,0.5276000017091633242927173341740854084491729736328125f,0.26039999973997474125297912905807606875896453857421875f,0.73200000989437086218458716757595539093017578125f,0.035800006818771379357713158242404460906982421875f,0.672599999995529440610653182375244796276092529296875f,0.7824000228404999202069802777259610593318939208984375f,-0.275099996978044547546460307785309851169586181640625f,0.2380999852955341111737652681767940521240234375f,0.1679999899566173571230365269002504646778106689453125f,0.013199997323751322397811236442066729068756103515625f,-0.178700005161762209837661430356092751026153564453125f,-0.0027000028967858469286511535756289958953857421875f,-0.13530000076889991955653158584027551114559173583984375f,-0.008500001668930057263651178800500929355621337890625f,-0.070299994808435517512634760350920259952545166015625f,-0.678900021767616212997609181911684572696685791015625f,-0.0757000029861927270502519604633562266826629638671875f,0.0612999939441680741225582096376456320285797119140625f,-0.1758000072285532955351783357400563545525074005126953125f,-0.2223000097319484946734036157067748717963695526123046875f,0.04539999491572377365855572861619293689727783203125f,-0.71350000651180744970503155855112709105014801025390625f,-0.173800002065300940312653210639837197959423065185546875f,-0.0981000048905611432559226159355603158473968505859375f,-0.21850000308454031738847334054298698902130126953125f,0.523600014740228658638443448580801486968994140625f,-0.0218999977171421000843309911942924372851848602294921875f,-0.0626999989107250854658559546805918216705322265625f,-0.1583000048071145915429269734886474907398223876953125f,-0.50400000265240674224287431570701301097869873046875f,0.0523999987378716280961299389673513360321521759033203125f,-0.7523000059321522936528481295681558549404144287109375f,-0.0736000023722649387991623370908200740814208984375f,-0.6928000265479088426445741788484156131744384765625f,-0.06780000423341991933057215646840631961822509765625f,-0.406600026601553476979233892052434384822845458984375f,0.3416000115513799784139337134547531604766845703125f,1.27250005900859886054377057007513940334320068359375f,-0.6287999605655674173476654686965048313140869140625f,0.1264000028431411237761494703590869903564453125f,0.34990000972747825613851091475225985050201416015625f,-0.13340000409483909749042140902020037174224853515625f,0.0643999892234798920753746642731130123138427734375f,1.1326000316977502535564781283028423786163330078125f,0.2641999952673916851608737488277256488800048828125f,0.7367000216424468561626781593076884746551513671875f,0.42029999302029619201448440435342490673065185546875f,-0.730100015753507580740233606775291264057159423828125f,-0.849699995505809102525063281063921749591827392578125f,-0.2531999997079370512409468574333004653453826904296875f,0.41499999195337267110517132095992565155029296875f,-0.06540001100897807617684520664624869823455810546875f,-0.08729999084472606085682855336926877498626708984375f,-0.92190003422498723040234835934825241565704345703125f,-0.3130999946832655478345941446605138480663299560546875f,-0.248400001040101070515220271772705018520355224609375f,-0.66369999324083295277887373231351375579833984375f,-0.28620000367164610821646419935859739780426025390625f,0.19749995559454003313248904305510222911834716796875f,0.6581999906182289805656182579696178436279296875f,-0.1990000245571135195632450631819665431976318359375f,0.10939999160766600283523075631819665431976318359375f,-0.0153000101566309609779636957682669162750244140625f,-0.90350002416968333562863335828296840190887451171875f,0.1210999872662126275457694646320305764675140380859375f,0.39830000845789914620809213374741375446319580078125f,0.066799997311830505708485361537896096706390380859375f,-0.61600001314282426534418846131302416324615478515625f,0.08399999915063371958012794493697583675384521484375f,0.53989997411370271862551817321218550205230712890625f,0.08650000138580793418441317044198513031005859375f,-0.60260001485943792687294262577779591083526611328125f,-0.209200001674890501135450904257595539093017578125f,0.3485999973058699907824120600707828998565673828125f,0.0538000045984983188418482313863933086395263671875f,-0.46930000789165493557675290503539144992828369140625f,-0.0483000002622604451829602112411521375179290771484375f,-0.3475000017881393343799345529987476766109466552734375f,0.25890000790953637821445454392232932150363922119140625f,0.41710001283288011819649909739382565021514892578125f,-0.199800002691149725109198698191903531551361083984375f,-0.026500006824731858756649671704508364200592041015625f,0.22970000355243680445482823415659368038177490234375f,-0.378899997776746655375745831406675279140472412109375f,-0.03689999607801441872112491182633675634860992431640625f,0.46429998852014542354282866654102690517902374267578125f,0.128100004965066904816239912179298698902130126953125f,0.499900003767013600963764474727213382720947265625f,-0.243499995559453896731838540290482342243194580078125f,0.293900019979477011844437583931721746921539306640625f,-0.175399997150897934972135772113688290119171142578125f,0.630200013625621835444690077565610408782958984375f,0.211300001916289392767112076398916542530059814453125f,-0.42219997464418401733610153314657509326934814453125f,-0.215499999362975369621153731714002788066864013671875f,0.35729999544545976153386845908244140446186065673828125f,0.45739999572038669839457725174725055694580078125f,0.94749998614191976997744859545491635799407958984375f,0.112899978077412033172777228173799812793731689453125f,0.6122000075757505488383003466879017651081085205078125f,0.1252000200331213619620029930956661701202392578125f,0.9243000126600264820808661170303821563720703125f,-0.8630999836564061666877023526467382907867431640625f,-0.05179999984502803300756568205542862415313720703125f,-0.645399982845783259932659348123706877231597900390625f,0.2520000049769877126237815900822170078754425048828125f,-0.19199998545646668191011485760100185871124267578125f,0.00749999761581410240296463598497211933135986328125f,0.79749996244907439546523164608515799045562744140625f,0.040499970287084519071640897891484200954437255859375f,-0.3115000218749042204535726341418921947479248046875f,1.032999987423419696597193251363933086395263671875f,-0.209299987924099095693009076057933270931243896484375f,-1.2444000276684763495183005943545140326023101806640625f,-0.145800014120340382106633114744909107685089111328125f,-0.0760000039041042185772312222979962825775146484375f,0.464399990415573338253807378350757062435150146484375f,0.1582999887883662637477755197323858737945556640625f,0.08149996443092810682884419293259270489215850830078125f,0.4186999958112835873436097244848497211933135986328125f,0.43479999442100503159736035740934312343597412109375f,0.8330000084340571930852092918939888477325439453125f,-0.31669997410774275437006508582271635532379150390625f,-0.7237999848544596215305091391201131045818328857421875f,-0.52079998774528490912416600622236728668212890625f,0.1708999981157481584670421170812915079295635223388671875f,0.033400000295042975295700671267695724964141845703125f,-0.6597000023603438734198789461515843868255615234375f,-0.10529999540448187200780694183777086436748504638671875f,0.078900004333257722777261733426712453365325927734375f,0.180800002273917126416336031979881227016448974609375f,-0.6061000192105769901473877325770445168018341064453125f,0.126700005400180781833796572755090892314910888671875f,0.7258999950051308047704878845252096652984619140625f,0.17580000056028366461902123774052597582340240478515625f,0.343000003963708888221617598901502788066864013671875f,0.1867999988019466428568193805404007434844970703125f,-0.20279999659657477195651154033839702606201171875f,-0.2494999941736459891927779608522541821002960205078125f,-0.3670000021457671124380794935859739780426025390625f,-0.10329999370574949324463887023739516735076904296875f,0.55230000962018976906620082445442676544189453125f,0.06889999844729899347584023416857235133647918701171875f,0.14840000096559524678241359652020037174224853515625f,-0.1085999996155500024741513698245398700237274169921875f,0.423700010824203499026907593361102044582366943359375f,-0.0215999991834163740378471629810519516468048095703125f,-0.5036000010311603158896787135745398700237274169921875f,-0.295900000369548721312895622759242542088031768798828125f,0.5618000189930201226928829782991670072078704833984375f,-0.036400003960728621876796751166693866252899169921875f,-0.467699996787309668633270121063105762004852294921875f,0.15709999487698078635133924763067625463008880615234375f,0.0471999990820883841280419801478274166584014892578125f,0.10400000235438344731164761469699442386627197265625f,0.31949998679757118935640392010100185871124267578125f,-0.02490001062154778566082313773222267627716064453125f,-0.184000018000602949541644193232059478759765625f,0.494900000786781202322117678704671561717987060546875f,0.27400000414252279057336636469699442386627197265625f,-0.5252999708175654092201511957682669162750244140625f,0.1330999979615210548899995046667754650115966796875f,0.59029998393058757955031978781335055828094482421875f,0.992900013691186966724444573628716170787811279296875f,0.6583999774217605960302535095252096652984619140625f,0.1723999756038185893203262821771204471588134765625f,-0.3734999697804450846660984097979962825775146484375f,-0.18339999366402626179706203402020037174224853515625f,-0.110399976289272672147490084171295166015625f,0.43559999840855567754260846413671970367431640625f,-0.22389998466372507124333424144424498081207275390625f,0.513100007647275813127407673164270818233489990234375f,-0.2174000067472459107875693007372319698333740234375f,-1.1438000247895718786139696021564304828643798828125f,-0.2791999890089034153106695157475769519805908203125f,-0.045100004875660015812854908290319144725799560546875f,-0.5015999322772035640127796796150505542755126953125f,-0.05549998667836231192040941095910966396331787109375f,0.261000010013579863965560434735380113124847412109375f,1.0116000026106830578243034324259497225284576416015625f,0.397299980115890516429999479441903531551361083984375f,0.00279999257326135619905471685342490673065185546875f,-0.294299983751773908124960144050419330596923828125f,0.1209000204861165883585272240452468395233154296875f,0.6410999713778497977045844891108572483062744140625f,-0.545100005471706339221782400272786617279052734375f,-0.153099999749660486969560224679298698902130126953125f,0.388900002098083508172976507921703159809112548828125f,-0.0881000013887882627017233971855603158473968505859375f,-0.3927000145196914981937652555643580853939056396484375f,0.417300004404783209110973984934389591217041015625f,-0.34550000306963912777291625388897955417633056640625f,-0.1543000030487775564580488207866437733173370361328125f,-0.387999998748302488138506305404007434844970703125f,-0.3455999981850386149062615004368126392364501953125f,0.0100999847590923508278137887828052043914794921875f,0.354699998411536210340955221909098327159881591796875f,-0.3649999983608722686767578125f,0.26959999338686468917103411513380706310272216796875f,-0.509099990838766114364943859982304275035858154296875f,0.1998999967634678132100134462234564125537872314453125f,-0.2198999921441078253536716147209517657756805419921875f,-0.06340000104010101722451508976519107818603515625f,0.456100012058019643745865323580801486968994140625f,0.2299999992549419669529697785037569701671600341796875f,-0.19679998837113377163632321753539144992828369140625f,0.430799988266825639726675944984890520572662353515625f,-0.39510001143217099439652884029783308506011962890625f,-0.05140000485479834235746920967358164489269256591796875f,-0.23790001056194309825286836712621152400970458984375f,-0.0025000030547379648027117582387290894985198974609375f,-0.4609999773800372846466189002967439591884613037109375f,0.171099998870491987190689542330801486968994140625f,-0.052199999082088499591236541164107620716094970703125f,-0.08630000064969056694508253713138401508331298828125f,-0.021099993431568098145589829073287546634674072265625f,0.188700013804435684505733661353588104248046875f,-0.64680000804066661856950304354541003704071044921875f,0.43429997481107651680076742195524275302886962890625f,0.428200019824505062615571659989655017852783203125f,0.16769996683597554465450230054557323455810546875f,-0.3173000030636785595561377704143524169921875f,1.086199965524674126982063171453773975372314453125f,0.14499999269843133475887952954508364200592041015625f,-0.18209998914003389103299923590384423732757568359375f,0.093399993067979636407471843995153903961181640625f,-0.2180000008344649131686310283839702606201171875f,1.0509999909400935536041288287378847599029541015625f,-0.43429998524189006303686255705542862415313720703125f,-1.109900022989511736426493371254764497280120849609375f,-0.29109999849796341919727638014592230319976806640625f,-0.47520000766515746448703794158063828945159912109375f,-0.72589999023675932932064824854023754596710205078125f,-0.065400004452467186411013244651257991790771484375f,-0.154599983179569022695432067848742008209228515625f,-0.29249999776482571434144119848497211933135986328125f,-0.4432999805927277936490327192586846649646759033203125f,-0.19080000078380121575349903650931082665920257568359375f,-0.27339996937513433294952847063541412353515625f,-0.78469999386668209950812524766661226749420166015625f,0.6902999991297722459648866788484156131744384765625f,-0.68250002786517161013080112752504646778106689453125f,0.108699974465370274856468313373625278472900390625f,-0.006700026857853202955084270797669887542724609375f,0.2780999639868735329173432546667754650115966796875f,0.2562000078439712336120237523573450744152069091796875f,0.090099998393654789907714075525291264057159423828125f,-0.396600008720159546982131359982304275035858154296875f,-0.325100006216764381150596818770281970500946044921875f,0.213900009846687222392347393906675279140472412109375f,-0.31649999997019762076888582669198513031005859375f,0.5435000125467777110088718472979962825775146484375f,-0.147799995478987700181505715590901672840118408203125f,-0.94410000216364853287132063996978104114532470703125f,0.237699995371699390744879565318115055561065673828125f,-0.335199980694055543750664583058096468448638916015625f,-0.11079999579191213143758432124741375446319580078125f,0.458799987089633987125125713646411895751953125f,0.0678999997586011705408282068674452602863311767578125f,0.40839999887943267964374172152020037174224853515625f,-0.030100004875659891467876150272786617279052734375f,0.43390001372098918608344320091418921947479248046875f,0.031900000771880154815107744070701301097869873046875f,0.782600003832578661189245394780300557613372802734375f,0.277400000414252279057336636469699442386627197265625f,-0.43969999699592587916185948415659368038177490234375f,0.153299998921155922193548803988960571587085723876953125f,0.17870000002086161572378841810859739780426025390625f,0.270599988201260632791900206939317286014556884765625f,-0.26069998316764841916892692097462713718414306640625f,0.010499997928738569186180029646493494510650634765625f,-0.801600005292892436870033634477294981479644775390625f,-0.236699997800588590735060279257595539093017578125f,0.414900006598234227794819162227213382720947265625f,-0.284900000786781237849254466709680855274200439453125f,0.66749999955296512865743352449499070644378662109375f,-0.617399983501434324040246792719699442386627197265625f,-0.3098000050008298256187799779581837356090545654296875f,-0.16719998261630519831300034638843499124050140380859375f,-0.4281000077590346730715964440605603158473968505859375f,-0.5123999945282928791812082636170089244842529296875f,-0.26690001276731489809890263131819665431976318359375f,-0.01180000334680075813054145328351296484470367431640625f,0.0380000016912818150416342177777551114559173583984375f,0.39060000541210104785250223358161747455596923828125f,1.293000014692544663574835794861428439617156982421875f,0.076799976450205476030674844878376461565494537353515625f,1.177800024741143314344071768573485314846038818359375f,-0.3980999915540219813436806362005881965160369873046875f,-0.385399997076392164530034278868697583675384521484375f,0.174100031518936049224066664464771747589111328125f,-0.64970000519156467788661757367663085460662841796875f,0.08759999183714395343258729553781449794769287109375f,-0.428699999275803378395721665583550930023193359375f,0.556199993762373612327110095066018402576446533203125f,-0.4861000353783373473248730078921653330326080322265625f,0.1965000066757198737832368351519107818603515625f,1.0197999945700164659001529798842966556549072265625f,1.2220999614238738928406746708787977695465087890625f,0.12809995921850170219613573863171041011810302734375f,0.6586999908566468775461544282734394073486328125f,1.102600032293796772364657954312860965728759765625f,0.52009998744130114811667908725212328135967254638671875f,-0.442300016437471021912841706580366007983684539794921875f,0.61669997708797463786822845577262341976165771484375f,-0.8881000016868114688151081281830556690692901611328125f,0.17580000056028366461902123774052597582340240478515625f,0.33740000510811807288291674922220408916473388671875f,0.268399999624490737115678484769887290894985198974609375f,-0.397699995669722528024436769555904902517795562744140625f,0.238400007820129378188767077517695724964141845703125f,0.06860000430941592952649443759582936763763427734375f,-0.081799998280406016615273756542592309415340423583984375f,0.199399994066357610922324283819762058556079864501953125f,-0.0874000081628560909763336894684471189975738525390625f,-0.32130000362992294782316093915142118930816650390625f,-0.04639999285936353967674250498021137900650501251220703125f,-0.0410999957971274199053368647582828998565673828125f,0.01769999976754188519834087855997495353221893310546875f,0.44570000000596043943090762695646844804286956787109375f,0.191499997660517617958930713939480483531951904296875f,-0.61160002019405368400839506648480892181396484375f,-0.0479999996423721153604446953977458178997039794921875f,0.02790000378191470620237168986932374536991119384765625f,-0.12860000483095646028885994383017532527446746826171875f,-0.33500000007450581041013037975062616169452667236328125f,0.1158999937385321032934371032752096652984619140625f,-0.545799989384412764792386951739899814128875732421875f,-0.290100003236532177908202356775291264057159423828125f,-0.7768999911606311314926642808131873607635498046875f,-0.0503000098586081634266520268283784389495849609375f,-0.7118999859452248557545317453332245349884033203125f,-0.1937000025540590308903432514853193424642086029052734375f,-0.1456000029161572673075397688080556690692901611328125f,-0.0447000021368264555121641024015843868255615234375f,0.274100006484985225796435770462267100811004638671875f,-0.50669996907860070933793394942767918109893798828125f,-0.69329999089315530458321745754801668226718902587890625f,1.2306999801874152211667023948393762111663818359375f,0.3145000073611736723933063331060111522674560546875f,0.021600004696846752949568326584994792938232421875f,-0.51280002133250235374362091533839702606201171875f,0.60009994884729422892633010633289813995361328125f,0.0355999790370464808120232191868126392364501953125f,-0.5156999572396276931840475299395620822906494140625f,-0.1851999598324294282747359829954802989959716796875f,-0.6214999709129340743629654753021895885467529296875f,-0.30909997444748871231468001496978104114532470703125f,0.43270000557899468418554533855058252811431884765625f,0.6575000154972077126558360760100185871124267578125f,-0.293400015643239253648744124802760779857635498046875f,-0.286600008347630552663076741737313568592071533203125f,-0.82809997829198866980959792272187769412994384765625f,0.256600030326843420169780074502341449260711669921875f,-0.11340000923573978486302848978084512054920196533203125f,0.0729000000938773151659688664949499070644378662109375f,0.366299998116492719901771124568767845630645751953125f,1.0365000369250776390828150397283025085926055908203125f,-0.6722999637246136472867874545045197010040283203125f,-0.5202999878048897297588837318471632897853851318359375f,-0.239099990987778010520514726522378623485565185546875f,0.7819000050187108197263796682818792760372161865234375f,-1.24889997080564540965497144497931003570556640625f,-0.1839999718070028933425419381819665431976318359375f,-0.37409999307393970724433529539965093135833740234375f,-0.282000006169080830176199015113525092601776123046875f,0.2466999956026673379572144995108828879892826080322265625f,0.0636000021860002873186346050715656019747257232666015625f,-0.277900012573599752840891596861183643341064453125f,-0.82399999356269848505007757921703159809112548828125f,0.2656000056356191407758160494267940521240234375f,-0.60889999777674663761217743740417063236236572265625f,-0.45009999451935289727089184452779591083526611328125f,0.324600003296136918606862309388816356658935546875f,0.4169000010699033254013556870631873607635498046875f,-0.34040001339316372508392305462621152400970458984375f,0.115899993291497249714439021772705018520355224609375f,0.479400001144409060316320392303168773651123046875f,0.028899998074769950306972532416693866252899169921875f,-0.606300009888410595948471382143907248973846435546875f,-0.1314000016510485924214890474104322493076324462890625f,0.65950001890957354877542684334912337362766265869140625f,0.1724999982118606833836338410037569701671600341796875f,0.56490000615119928539797911071218550205230712890625f,-0.091000000029802297074610351046430878341197967529296875f,0.338100006380677220452213305179611779749393463134765625f,-0.17270000468492507916806744105997495353221893310546875f,-0.084900009727478131793532156734727323055267333984375f,0.23149999564886092340287859769887290894985198974609375f,0.2844999976754187809291352095897309482097625732421875f,0.01109999767839908724198494383017532527446746826171875f,0.2173999994456767836936705862171947956085205078125f,0.22609999760389332124077554908581078052520751953125f,0.95770001153945916172460783855058252811431884765625f,0.2021000063508748834095740676275454461574554443359375f,-0.31829996777772873173262269119732081890106201171875f,0.42339995834827437448666387354023754596710205078125f,0.22699998560547829384859142010100185871124267578125f,-0.0745999977827072058289559208787977695465087890625f,-1.05740002060532578553875282523222267627716064453125f,-0.2465000121891502971038789837621152400970458984375f,-0.64640003041029014951845965697430074214935302734375f,-0.3635999943256376099043336580507457256317138671875f,-1.29820000000596014189113702741451561450958251953125f,0.0159999843835834099081694148480892181396484375f,0.46589999530315395759316743351519107818603515625f,0.034299999248981638544364614062942564487457275390625f,0.3350000034272671012303135285037569701671600341796875f,-0.935299972307682470074041702901013195514678955078125f,-0.6332999828279017862087130197323858737945556640625f,-0.77819998018741642908935318700969219207763671875f,0.24270000081062281793720103451050817966461181640625f,0.168200000788271408769247727832407690584659576416015625f,0.03839999934509397139237307783332653343677520751953125f,0.22619999942481461285836985553032718598842620849609375f,0.4317000238403678746834657431463710963726043701171875f,-0.432299991738795963414077050401829183101654052734375f,-0.35330001590847981418619383475743234157562255859375f,1.26539995542764671654367703013122081756591796875f,0.1162999417901033893940621055662631988525390625f,0.4473999924421310225852721487171947956085205078125f,-0.7890000388622286919826365192420780658721923828125f,-0.24079997262060681695317043704562820494174957275390625f,-0.4415999852135776837513958525960333645343780517578125f,-0.603799997222422835108091021538712084293365478515625f,0.42490000838637342894799076020717620849609375f,-0.336199997934699013057979755103588104248046875f,0.261900011426210443943318750825710594654083251953125f,-0.004599999794363984761957908631302416324615478515625f,0.14949999991059303994234142010100185871124267578125f,-0.157900004079937872347727534361183643341064453125f,0.6826000188827514580935940102790482342243194580078125f,0.104400002411007886138349931570701301097869873046875f,0.25090000379681587361346828402020037174224853515625f,-0.11559999498128892270187861868180334568023681640625f,0.223399997836351449365110966027714312076568603515625f,0.03490000153183936415501875671907328069210052490234375f,-0.13650000116229055180383511469699442386627197265625f,0.0992999982059001862211289335391484200954437255859375f,0.9016999972045420808086646502488292753696441650390625f,0.026599999332427959330971134477294981479644775390625f,0.841199995476007433836684867856092751026153564453125f,0.0477999984964728452663251800913712941110134124755859375f,-0.30889999729245898318907848079106770455837249755859375f,-0.241800004017353076246621412792592309415340423583984375f,0.267299986001849065875290989424684084951877593994140625f,0.2323000017225742563908852389431558549404144287109375f,-0.0072999868214129737253870189306326210498809814453125f,-0.2522000035524367689276914461515843868255615234375f,-0.91399998059868803323979591368697583675384521484375f,0.022999996662139920999834430404007434844970703125f,-0.539799988162517507817028672434389591217041015625f,0.143199995759129505845663743457407690584659576416015625f,-0.023800007429719050033867233651108108460903167724609375f,0.0566000105828046162770306182210333645343780517578125f,-0.1866999949693681770668263197876513004302978515625f,-0.281799985986947998384266611537896096706390380859375f,0.1559000185489656242765477145439945161342620849609375f,0.3694000079989425699977800832130014896392822265625f,-0.12310000973343893093669976224191486835479736328125f,-0.5141999709784987881988627123064361512660980224609375f,-0.0437999904237688919561577449712785892188549041748046875f,0.473400001151859151082845755809103138744831085205078125f,0.53250002006068830606722031006938777863979339599609375f,0.1806000086903569723517648526467382907867431640625f,1.26610006138086372828865933115594089031219482421875f,0.58539999625682792583347691106610000133514404296875f,-0.265700032490492343839605382527224719524383544921875f,-0.73309998842477863689737205277197062969207763671875f,-0.82259998952746382627765342476777732372283935546875f,0.06749996960163162640355949406512081623077392578125f,0.859499992012977553912378425593487918376922607421875f,0.6417999935865399407930453890003263950347900390625f,0.04409999068975434255435175145976245403289794921875f,0.454600010634958973110286706287297420203685760498046875f,0.17720000040456651557718714684597216546535491943359375f,0.684399999058246777394742821343243122100830078125f,0.11360000237226497432629912509582936763763427734375f,0.9081000000476837641372185316868126392364501953125f,0.02630000184178360456144218915142118930816650390625f,-0.05139999859631050327379853115417063236236572265625f,0.486900021037459662665014548110775649547576904296875f,-1.05609997584819748084328239201568067073822021484375f,-0.212900019651651728480601377668790519237518310546875f,-0.0857000058174133272359540569595992565155029296875f,0.886200011271238263788063704851083457469940185546875f,0.15260000010728835828643923377967439591884613037109375f,-0.0262000008404255080307621028623543679714202880859375f,-0.321000005319714620810600536060519516468048095703125f,0.30749997228384007286194901098497211933135986328125f,0.194999996125698071836040980997495353221893310546875f,0.159299999006837555981519471970386803150177001953125f,-0.1170000073239207127873129365980275906622409820556640625f,-0.3138000040397048717721872890251688659191131591796875f,-0.008200001943111434599131825962103903293609619140625f,-0.25480001394152651528202113695442676544189453125f,-0.1138000057905912409950133223901502788066864013671875f,-0.413900000607967388788210882921703159809112548828125f,0.0374999939650297875459727947600185871124267578125f,0.815000017732381731860868967487476766109466552734375f,-0.1626999941051006093317710110568441450595855712890625f,0.25170000852942475688678314327262341976165771484375f,-0.115800004583597182516996326739899814128875732421875f,-0.490600002878904373204704825184307992458343505859375f,0.0750000014528631464560959329901379533112049102783203125f,-0.71430001947730781264311872291727922856807708740234375f,-0.004399999952316324680623438325710594654083251953125f,-0.799200013148784638161714610760100185871124267578125f,-0.086500000789761610775485678459517657756805419921875f,-0.86960002281665804702015520888380706310272216796875f,0.1805999990791082865371208754368126392364501953125f,-0.37699998657405375812601278084912337362766265869140625f,0.3089000064939260159491141166654415428638458251953125f,0.5430000017285347979623111314140260219573974609375f,-0.05509999869167803154823559452779591083526611328125f,0.092999985784292249491045367904007434844970703125f,-0.4847999619364742329707951284945011138916015625f,-0.1860999992430212301997016766108572483062744140625f,-0.3209999840855601860312162898480892181396484375f,0.6748000032126899583317936048842966556549072265625f,-0.40359998739659797006851249534520320594310760498046875f,-0.3286999936506151964721311742323450744152069091796875f,0.21939999309778190905717565328814089298248291015625f,-0.4336999945819375312794363708235323429107666015625f,-0.129600007319450316600750738871283829212188720703125f,0.12819999791979785896955945645458996295928955078125f,0.5436999819159507030263966953498311340808868408203125f,0.76940000971257693773708297158009372651576995849609375f,-0.0217999768972401142264061491005122661590576171875f,-1.12340001527071020603898432455025613307952880859375f,0.5107999951958657192108148592524230480194091796875f,0.85250002399086977078468407853506505489349365234375f,0.34559999900460247612699049568618647754192352294921875f,0.1864999974742531652083243898232467472553253173828125f,0.21599998460710079228874747059307992458343505859375f,-0.414700003775954062490427531884051859378814697265625f,0.6380999930068853576159426665981300175189971923828125f,0.54959999962672589912671128331567160785198211669921875f,0.19740000555515280211693607270717620849609375f,0.1563999980747698970162673504091799259185791015625f,0.28899998782575142541872992296703159809112548828125f,-0.433900013199448597589480414171703159809112548828125f,-0.64019998713880799101616503321565687656402587890625f,-0.47119999221637842712340216166921891272068023681640625f,-0.7542999605059623746683428180404007434844970703125f,0.4573000052988529517961069359444081783294677734375f,0.41639999308288100365871287067420780658721923828125f,-0.375499988317489652445146930404007434844970703125f,-0.1014000000864268002942480961792171001434326171875f,0.576499979704618414899641720694489777088165283203125f,0.016000000774860356766993163546430878341197967529296875f,0.453299992588162392603834405235829763114452362060546875f,0.069799994346499527608784774201922118663787841796875f,-0.396499978065490754630673109204508364200592041015625f,-0.0887999979674816142249937911401502788066864013671875f,0.3493999999523162980352708473219536244869232177734375f,-0.200100000852346415758375997029361315071582794189453125f,-0.217500003501772898317767612752504646778106689453125f,0.076199994879960986082778617856092751026153564453125f,-0.09440000770092016324497308232821524143218994140625f,-0.053599999615550064646640748833306133747100830078125f,-0.51850001120567323908971957280300557613372802734375f,0.055599998483061796594739689680864103138446807861328125f,-0.5422999981462954810496057689306326210498809814453125f,-0.1747999953150749519181772484444081783294677734375f,0.16689999063909055632137778957257978618144989013671875f,-0.0968999976053834533029629483280587010085582733154296875f,-0.5541000175490975454550834911060519516468048095703125f,0.09379999714791775744515689439140260219573974609375f,-0.4542999970138072995951006305404007434844970703125f,-0.1661999979346990841122533311136066913604736328125f,0.06239999251663685964075511947157792747020721435546875f,0.0241999991789460244806519995108828879892826080322265625f,0.71930000129789106733113612790475599467754364013671875f,0.424299999099969848970204111537896096706390380859375f,0.1683999939620495212011519470252096652984619140625f,-0.1420999700665479537065039039589464664459228515625f,-0.3079999765455720961426777648739516735076904296875f,-0.3360999979019165806448654620908200740814208984375f,0.536400021022558348704478703439235687255859375f,0.65109999574124788335183211529511027038097381591796875f,0.4662000049754977393234867122373543679714202880859375f,-0.26269998710155473276017801254056394100189208984375f,-0.05950000557303436465872437111102044582366943359375f,-0.7580999794840810324103586026467382907867431640625f,-0.5783999942600726029695579200051724910736083984375f,0.0984000087141989610017844825051724910736083984375f,-0.73770001258254058740249092807061970233917236328125f,-0.8410999754011634355066462376271374523639678955078125f,-0.849499987617134966200183043838478624820709228515625f,0.782099988245963384514425342786125838756561279296875f,0.2175000052154063556741903084912337362766265869140625f,-0.814499987542628645087461336515843868255615234375f,-0.3678000123918057084893007413484156131744384765625f,-0.21289997271299387904264222015626728534698486328125f,-0.16979998801350593140568889793939888477325439453125f,0.66129997189044953387337955064140260219573974609375f,-0.198100024634599858330830102204345166683197021484375f,0.11499998986721049476500411401502788066864013671875f,-0.657100003594159876030289524351246654987335205078125f,-0.46819998815953722814953152919770218431949615478515625f,-0.47300000534206632085698629452963359653949737548828125f,-0.5940999838352201134483721034484915435314178466796875f,-0.01570000365674506515034636322525329887866973876953125f,0.8319999988675110813574065105058252811431884765625f,-0.732200001615285867018201315659098327159881591796875f,0.2944999910444021651301227393560111522674560546875f,-0.49279999093413362487581252935342490673065185546875f,0.013300000336766260033982689492404460906982421875f,0.338200002986192682641330975457094609737396240234375f,0.009299994182586701629844583294470794498920440673828125f,-0.8085999945491553209109270028420723974704742431640625f,0.248299999740719812280076439492404460906982421875f,-0.257399990653991661559985004714690148830413818359375f,0.18090000446736809180947602726519107818603515625f,0.5039999943077564381610500277020037174224853515625f,0.060399997821450224222417091368697583675384521484375f,-0.21810000153779984799484736868180334568023681640625f,0.08509999906420706583531909927842207252979278564453125f,0.83409999635219567171162680097040720283985137939453125f,-0.32450000706315040854832432160037569701671600341796875f,-0.18570000998973856809470817097462713718414306640625f,0.26690000717937945040603153756819665431976318359375f,0.3630000057518483203011783189140260219573974609375f,0.1948999966889619628318541799671947956085205078125f,-0.188200006264448216342088926467113196849822998046875f,-0.3452999992042780075252039750921539962291717529296875f,0.0239999938607216023456203402020037174224853515625f,-0.0537999992340802979384761783876456320285797119140625f,0.020100004577636720881628207280300557613372802734375f,0.1514000011295080216910946546704508364200592041015625f,0.226000004649162311576304773552692495286464691162109375f,0.234100002536177631551339572979486547410488128662109375f,0.16220000452101235310919946641661226749420166015625f,-0.110000010132789594052837855997495353221893310546875f,0.3599999789893626456205311114899814128875732421875f,-0.1095000060200685965128286625258624553680419921875f,-1.15580003483295445931844369624741375446319580078125f,-0.501299971592426896194183427724055945873260498046875f,-0.596699980962276388396503534750081598758697509765625f,0.4218999900989237605841708500520326197147369384765625f,-0.27349999572709171591355925556854344904422760009765625f,-0.2731000209093092934153901296667754650115966796875f,-0.60349999570846524221678919275291264057159423828125f,-0.633199970650673460426105521037243306636810302734375f,-0.892899980759620515158303533098660409450531005859375f,-0.0709999900460243082989109097979962825775146484375f,-0.15839999887943267964374172152020037174224853515625f,0.349399975514412375332540250383317470550537109375f,-0.2309999979436394568210744182579219341278076171875f,-0.50880000042617279376599981333129107952117919921875f,0.309500025317073212960394812398590147495269775390625f,-0.0565999699771406739046142320148646831512451171875f,0.27449998739361713973039513803087174892425537109375f,-0.88850001001357981067485525272786617279052734375f,-0.1580999893188472782412645756267011165618896484375f,-0.173999990433454598814932978712022304534912109375f,0.138700000542402168690614416846074163913726806640625f,0.416600002014636316971518681384623050689697265625f,0.23130001510381692497730909963138401508331298828125f,-0.786199995028972420385571240331046283245086669921875f,-0.6348000111103058173966928734444081783294677734375f,0.330999999694526092497426361660473048686981201171875f,-0.2358000108793378846172572593786753714084625244140625f,-0.277699995744228100846839879523031413555145263671875f,-0.8442999883711337361091864295303821563720703125f,0.145900003796815891377036678022705018520355224609375f,-0.1494999860525130230826107435859739780426025390625f,0.082899994990229597391362403868697583675384521484375f,0.4277999937653540474258306858246214687824249267578125f,-0.0250999977044761291011809589690528810024261474609375f,-0.424700003869086462060522535466589033603668212890625f,-0.135699997770786229267514499952085316181182861328125f,0.66090001512169838093768703402020037174224853515625f,0.11999999545514583587646484375f,0.5120999971866606603754235038650222122669219970703125f,0.076600000077486019023353946977294981479644775390625f,-0.084400004720687871184736650320701301097869873046875f,-0.29579999258816247476033822749741375446319580078125f,0.27849998304247858271764926030300557613372802734375f,-0.02679999209642414115251085604541003704071044921875f,0.64389998905956746799716938767232932150363922119140625f,0.2129999962151050141301311668939888477325439453125f,-0.4348000093221664474896215324406512081623077392578125f,0.16450000602006920047415405861102044582366943359375f,0.674300027412176206098592956550419330596923828125f,0.1031999986648559630708632539608515799045562744140625f,-0.0387999972224235545326109786401502788066864013671875f,-0.309800004851818133744245642446912825107574462890625f,0.20919997664093958888997804024256765842437744140625f,0.1494000030070543605376087725744582712650299072265625f,0.6119000150024891393485404478269629180431365966796875f,0.0017999973490834264566728961654007434844970703125f,-0.381399999304115777665202813295763917267322540283203125f,0.05810000317692753935006066967616789042949676513671875f,-0.4030000089183449762941791050252504646778106689453125f,-0.35249997943639765907164473901502788066864013671875f,-0.371200007247924901321312063373625278472900390625f,0.69319997385442266857324966622400097548961639404296875f,0.6989999973997473858844386995770037174224853515625f,-0.00179997540712317771749439998529851436614990234375f,-0.90470002456307430094284427468664944171905517578125f,-0.001700009572505933874708716757595539093017578125f,0.73610001041889194794975992408581078052520751953125f,0.0048000316739074122551755863241851329803466796875f,-0.95170000033378610027057220577262341976165771484375f,-0.5182000037312501916630935738794505596160888671875f,-1.18810001360774020184862820315174758434295654296875f,0.27809995146989852088381667272187769412994384765625f,0.342599978798627802234477712772786617279052734375f,0.5198999696806076986632660918985493481159210205078125f,0.65259998619332904024048502833466045558452606201171875f,0.42850000166892954212016775272786617279052734375f,0.815700016099214497700131687452085316181182861328125f,-0.6101999710083010342032139305956661701202392578125f,0.14160003509521512654600883251987397670745849609375f,-0.28689996299743736329901366843841969966888427734375f,-0.162099979007244154871614227886311709880828857421875f,0.016900002336501795952017346280626952648162841796875f,-0.38770000557899475523981891456060111522674560546875f,-0.72589998487234108637267127051018178462982177734375f,-0.893000005155801801492998492904007434844970703125f,0.02489999899864159260687301866710186004638671875f,-0.60150001934170749251507004373706877231597900390625f,0.35139997706413306222117398647242225706577301025390625f,0.389700015100836794079697256165673024952411651611328125f,0.374499995291233123140273164608515799045562744140625f,-0.425899978017806990493454577517695724964141845703125f,-0.199599999943375572275527929377858527004718780517578125f,-0.390799998399615267086204539737082086503505706787109375f,0.25980000284016124822983329067938029766082763671875f,-0.549100014233589117651490596472285687923431396484375f,-0.060699999409913996117893475457094609737396240234375f,0.105300004345178610520861184340901672840118408203125f,0.271599987262487463368643147987313568592071533203125f,-0.5815999872624875166593483299948275089263916015625f,0.057000007584691037010315994848497211933135986328125f,0.5357000031352043833976495079696178436279296875f,-0.333699996146559652032692611101083457469940185546875f,0.421200018721819002820438981871120631694793701171875f,-0.1786999957367777780437023693593800999224185943603515625f,-0.2328999975606798333860325556088355369865894317626953125f,-0.0765000064522027845015372804482467472553253173828125f,-0.367600002789497448674183033290319144725799560546875f,0.303799996626377133424057319643907248973846435546875f,0.01380000646114343254566847463138401508331298828125f,0.2348999894618988282246618837234564125537872314453125f,-0.19329999713301670993814695975743234157562255859375f,0.1422999938994646829115708897006697952747344970703125f,-0.3269999780058860938680709296022541821002960205078125f,0.041500003769993742963606564444489777088165283203125f,0.800799992066621779684965076739899814128875732421875f,-0.112900003632903089823003028868697583675384521484375f,0.25860000207424160834079884807579219341278076171875f,-0.1313999961376190184214607370449812151491641998291015625f,-0.11399999997019794051311691873706877231597900390625f,-0.70599998974800115547623136080801486968994140625f,0.256100012803077703438248136080801486968994140625f,0.00369998936653193055690280743874609470367431640625f,0.3178999967038631524474112666212022304534912109375f,-0.850699970054626941617925695027224719524383544921875f,-0.630799992066621850739238652749918401241302490234375f,0.82509995033741068137800311887986026704311370849609375f,0.372400000675023046259326520157628692686557769775390625f,0.442400002090633126528018692624755203723907470703125f,-0.3387000023677944060551681104698218405246734619140625f,-0.035300022673606434864268521778285503387451171875f,-1.07310001167058910453988573863171041011810302734375f,0.1327000003077090095615631071268580853939056396484375f,-0.04360000138506292532980523901642300188541412353515625f,-0.203899980640411371268783113919198513031005859375f,0.2633000407934193987102844403125345706939697265625f,-0.657899979865550932345286128111183643341064453125f,-0.08459999390840522437429171986877918243408203125f,0.11770001451969136496700230054557323455810546875f,-0.6522999927818773358012549579143524169921875f,0.36980000187158612590110351447947323322296142578125f,0.39149999460577955545659278868697583675384521484375f,-0.3242999786853797417052192031405866146087646484375f,-1.21290000817775744934579051914624869823455810546875f,0.350399975395202734063104799133725464344024658203125f,0.714699998784065204660009840154089033603668212890625f,-0.391399989804625647593638859689235687255859375f,0.285400001397728697583033863338641822338104248046875f,0.103499987065791998475106083787977695465087890625f,-0.882199989843368115316479816101491451263427734375f,0.4614000192344189787974073624354787170886993408203125f,-0.04619999756217030295601944089867174625396728515625f,0.4553999755069615407165883880225010216236114501953125f,1.4241999557420614852532025906839407980442047119140625f,0.236100001776218704208076815120875835418701171875f,1.95119997982978787831598310731351375579833984375f,-0.57210001760125184233629624941386282444000244140625f,-1.126899987435340477759382338263094425201416015625f,-0.093299978804588068470593498204834759235382080078125f,-1.3871999937176706652053326251916587352752685546875f,-1.20990004724115163270425910013727843761444091796875f,-0.5135999972686164927182517203618772327899932861328125f,1.00050004091858912858015173696912825107574462890625f,1.1224999827146522335397094138897955417633056640625f,-0.13979999136626730660282191820442676544189453125f,0.36479997877478620438296275096945464611053466796875f,-0.348900011336803483885660170926712453365325927734375f,-0.701499994754791078577227381174452602863311767578125f,-0.273000012755394205754555514431558549404144287109375f,-0.1914000059723850455384308588691055774688720703125f,0.4475999716460703492515449397615157067775726318359375f,1.22439995390772882188912262790836393833160400390625f,0.86129999468922591177033609710633754730224609375f,-0.795700016546248889426351524889469146728515625f,0.25470001308918011062587538617663085460662841796875f,0.57300001573562564516350903431884944438934326171875f,0.866100038135052319177020763163454830646514892578125f,-0.4730000309348110221208116854541003704071044921875f,0.7985999892592428039961305330507457256317138671875f,-0.0595000234916802650531053586746565997600555419921875f,0.1189999966174364320892209434532560408115386962890625f,-0.5186999935761094082664612869848497211933135986328125f,-0.0182999986231326960961496297386474907398223876953125f,-0.5240000003799796246539699495770037174224853515625f,0.0536999917507170465569288353435695171356201171875f,-1.105300018873810241615274208015762269496917724609375f,0.0515000041425228705094241377082653343677520751953125f,0.3856000055238604229401744305505417287349700927734375f,0.560999990344047461121590458787977695465087890625f,-0.148400018325447913269954369752667844295501708984375f,-0.17559999170303353110966781969182193279266357421875f,0.3741999952673911167266851407475769519805908203125f,0.161400006270408624686751863919198513031005859375f,-0.03120000311285264782412696149549447000026702880859375f,0.032200000497698777479627096909098327159881591796875f,-0.01459999994337557449597397862817160785198211669921875f,-0.0612000069499014642815382103435695171356201171875f,-0.2502999892205000076472742875921539962291717529296875f,-0.1934999973475932666389098812942393124103546142578125f,0.61880000329464668329393362000701017677783966064453125f,0.2340000022053718709003078402020037174224853515625f,0.24899999386072158014115984769887290894985198974609375f,-0.023700006651878435803837419371120631694793701171875f,-0.70700000344961877463134669596911408007144927978515625f,-0.14609999760389325018650197307579219341278076171875f,0.269100008533894818896214928827248513698577880859375f,0.263899999341368651784023313666693866252899169921875f,0.1090000006034970159163322023232467472553253173828125f,0.1882999990701677006654790602624416351318359375f,-0.2353000196933745513661051518283784389495849609375f,0.25620001037716855218917544334544800221920013427734375f,-0.508900003215670526657277150661684572696685791015625f,0.69990000577867028663803239396656863391399383544921875f,-0.062000033810734489492233478813432157039642333984375f,0.2217000395238404308884128113277256488800048828125f,-1.763500012993812759987122262828052043914794921875f,-0.65310002590119864418483075496624223887920379638671875f,-0.101999983593821230698495128308422863483428955078125f,-0.7085999597549434980692240060307085514068603515625f,-0.623599989557266809470092994160950183868408203125f,-0.257400011515616977675335874664597213268280029296875f,0.1634000140786167065698464284650981426239013671875f,-0.28350001001358027252763349679298698902130126953125f,-0.2676000075578688353061806992627680301666259765625f,-0.2100000020861625404933192839962430298328399658203125f,-0.282199992078542738482838103664107620716094970703125f,0.97290000683665223135676569654606282711029052734375f,0.795099967622757475282924133352935314178466796875f,0.01769998225867730479166084478492848575115203857421875f,0.996000000402331497895147549570538103580474853515625f,-0.50100000643730169258560636080801486968994140625f,0.054999995827675007831203402020037174224853515625f,0.7431000362575057582859017202281393110752105712890625f,-0.36140000969767616112449104548431932926177978515625f,0.457199992972612268005150326644070446491241455078125f,0.9848999939322471419700377737171947956085205078125f,-0.9777000011086462993858958725468255579471588134765625f,-0.841399961417913733185969249461777508258819580078125f,-0.045300019395352109086161362938582897186279296875f,-0.6941000037655233878552962778485380113124847412109375f,0.26059998902082437410854254267178475856781005859375f,-0.5909000040948388932093848779913969337940216064453125f,0.18009999489188199817135682678781449794769287109375f,0.6603999792322519102327760265325196087360382080078125f,-0.224600005531311097684010746888816356658935546875f,0.045199987846613165487497099093161523342132568359375f,0.22270001213550560947851408855058252811431884765625f,0.8785999767422676409722726020845584571361541748046875f,0.01340000677704811238299953402020037174224853515625f,1.0073999929264185748678528398158960044384002685546875f,-0.090399990594387080733440598123706877231597900390625f,0.74869998612552890460136723049799911677837371826171875f,-0.202999996885657285616844092146493494510650634765625f,-0.162399997359514269845703893224708735942840576171875f,0.06989999959468828905073678470216691493988037109375f,-0.84280000367462637456128504709340631961822509765625f,-0.1385000055432319410186892127967439591884613037109375f,-0.16799999532103537802640857989899814128875732421875f,0.06860000065863136597954508033581078052520751953125f,0.33559998969137672109042114243493415415287017822265625f,0.065400001025199827608958003111183643341064453125f,-0.15790000583082453022853997026686556637287139892578125f,-0.303999996840953912169425166212022304534912109375f,-1.010700009989738301641182260937057435512542724609375f,-0.270699998962879107011758605949580669403076171875f,0.09420001795440902103706548587069846689701080322265625f,0.1123000025235116883237651563831605017185211181640625f,-0.2294999984949825933000511213322170078754425048828125f,-0.182200005638599282775658139144070446491241455078125f,-0.479100022205710462941397054237313568592071533203125f,-0.459899968153238081214340127189643681049346923828125f,-1.4778999703288082656627011601813137531280517578125f,-0.5157000014960768652372280484996736049652099609375f,0.766399972593784895025237346999347209930419921875f,0.2916000178098681061555907945148646831512451171875f,0.23319999389648415899500832892954349517822265625f,-1.279200024175643601864749143715016543865203857421875f,0.1781999995589256258199384319595992565155029296875f,-0.36510001783967016564247387577779591083526611328125f,1.4529999945759772828068889793939888477325439453125f,-0.7320000277757643392106956525822170078754425048828125f,0.32609999894499797079561176360584795475006103515625f,0.7043999939918517139147979833069257438182830810546875f,0.429599972301721866330126431421376764774322509765625f,0.237600027525425527841207440360449254512786865234375f,-1.2163999922633177419584171730093657970428466796875f,0.70200001294910896465495397933409549295902252197265625f,0.489599988022446641622309471131302416324615478515625f,0.252599998170137229891452079755254089832305908203125f,1.076399959778786108444137425976805388927459716796875f,0.58920001985430747737382262130267918109893798828125f,0.587099976921081179170869290828704833984375f,1.1727000155627731459162532701157033443450927734375f,-0.674100046420097243071722914464771747589111328125f,0.30389998153448072315541139687411487102508544921875f,0.29180001206398031854405417107045650482177734375f,-0.60090001731961983910679236942087300121784210205078125f,-1.146499991476535473822195854154415428638458251953125f,-0.2412000173807149483451439664349891245365142822265625f,-0.373500002227723282288707196130417287349700927734375f,-0.025099997946619989619421176030300557613372802734375f,0.73919999258518220841551737976260483264923095703125f,0.4834000027537346255712691345252096652984619140625f,-0.1811000096738337727941825505695305764675140380859375f,0.2812999981164931995181177626363933086395263671875f,-0.0441999975770712705269716025213710963726043701171875f,0.438699995028972722366233938373625278472900390625f,-0.335499996326863669793283406761474907398223876953125f,-0.2517000039100647512668729177676141262054443359375f,0.42019998661726720712294991244561970233917236328125f,-0.35939998974502085360427372506819665431976318359375f,-0.4479999754279855306293711691978387534618377685546875f,0.08569999553561213900820803246460855007171630859375f,-0.5979999867901206744846831497852690517902374267578125f,-0.08289998791217811913156765513122081756591796875f,0.64919998241513976466876556514762341976165771484375f,0.40270000349283208151973667554557323455810546875f,1.0116999741449952399108269673888571560382843017578125f,0.1166999921754003910479013939038850367069244384765625f,-0.425899979507923187593831926278653554618358612060546875f,-0.08909998204708102775839506648480892181396484375f,0.407200012008845479982710457989014685153961181640625f,-0.12610000177621838446384572307579219341278076171875f,-0.55039999681562168376558474847115576267242431640625f,0.109999997094273549436138637247495353221893310546875f,0.5974999735876920237132026159088127315044403076171875f,0.2272999899506569221330209984444081783294677734375f,-0.0849000078648327605179702004534192383289337158203125f,-0.0801000113762915244564055683440528810024261474609375f,-0.0005000061988831294712554154102690517902374267578125f,-0.557400012782216069950891323969699442386627197265625f,0.62480000857710837891545452293939888477325439453125f,-0.72810000675320640795007420820184051990509033203125f,-0.80290001711845349063878529705107212066650390625f,-0.62819999240636814619165306794457137584686279296875f,-0.63259997000694312419000198133289813995361328125f,-0.5976000014483933142628302448429167270660400390625f,-0.9712999981164927021382027305662631988525390625f,0.71490001926422142020101091475225985050201416015625f,1.51219997375011416806955821812152862548828125f,0.772199978369474404615857565659098327159881591796875f,-0.207199992823600798175220916164107620716094970703125f,0.54090001422762856719828050700016319751739501953125f,-0.3056000009417534357680779066868126392364501953125f,-0.898500026404857266015824279747903347015380859375f,-0.087999994277954129984209430404007434844970703125f,-1.4687999985635276090079059940762817859649658203125f,0.37910001900196022006639395840466022491455078125f,0.2943000009998673061062390843289904296398162841796875f,-0.279699995878338381061212203348986804485321044921875f,-0.8829000448346135954125202260911464691162109375f,-0.540399957513809425790896057151257991790771484375f,0.05369999458193763786084673483856022357940673828125f,-1.13239999989271122871059560566209256649017333984375f,-0.24899999170005315818343660794198513031005859375f,0.93409996669888517573099306900985538959503173828125f,0.28650002589821799148239733767695724964141845703125f,0.4944999753236771766751189716160297393798828125f,0.69330003159195185968854957536677829921245574951171875f,0.5893999857962131105892922278144396841526031494140625f,-0.1749000138998031417258971487171947956085205078125f,-0.872999997109174419307464631856419146060943603515625f,-0.122299991440772970463513047434389591217041015625f,0.9077999930202960143788004643283784389495849609375f,-0.093300000411272066003220970742404460906982421875f,0.537799999092519076526741628185845911502838134765625f,-0.00859999641180042573296304908581078052520751953125f,0.50389999677091823482300014802603982388973236083984375f,-0.05920001098811622508577556800446473062038421630859375f,-0.825199982705712375974371752818115055561065673828125f,0.129900002276897463815430455724708735942840576171875f,0.3967000056609510938443463601288385689258575439453125f,-0.3463999907545745049475272026029415428638458251953125f,-0.31239998309165262213582536787725985050201416015625f,0.456800000143051221357382019050419330596923828125f,0.78999998021870876385719384416006505489349365234375f,-0.056300000239908609689365448502940125763416290283203125f,-0.44129998710826046082189577646204270422458648681640625f,-0.0892999949276447324564287555404007434844970703125f,-0.50389998250305667593096359269111417233943939208984375f,-0.325700021016597673906289855949580669403076171875f,-1.1750999749615786971190800613840110599994659423828125f,-0.183299995903670909225269269882119260728359222412109375f,0.56509997550174606573136770748533308506011962890625f,0.123800007280707280443721174378879368305206298828125f,1.032399982607364652409387417719699442386627197265625f,0.3700999953389167984596497262828052043914794921875f,-0.19440000755190844916597825431381352245807647705078125f,-0.242300005894899417313581579946912825107574462890625f,-0.0814000026941297250004936358891427516937255859375f,0.83520002971887574716447488754056394100189208984375f,-0.727800010156631316249331575818359851837158203125f,-0.0204000269532205180667006061412394046783447265625f,1.58259997298717536295953323133289813995361328125f,0.56010000964403161560767330229282379150390625f,1.70659998681545221899114039842970669269561767578125f,-0.201600003057718080157201256952248513698577880859375f,0.682900008475780140315691824071109294891357421875f,1.02930002819448684903136381763033568859100341796875f,-0.2336000008448959608386985564720816910266876220703125f,-0.9084000194430352781438386955414898693561553955078125f,-0.34429996691346165249569821753539144992828369140625f,0.83009997831434001458461580114089883863925933837890625f,0.14959998735189461438466196341323666274547576904296875f,-1.3689000184893611589131978689692914485931396484375f,0.151999988436699595695245079696178436279296875f,0.643199994175880807034673125599510967731475830078125f,-0.02470001966431689677250460590585134923458099365234375f,-0.0777000134766101524519399390555918216705322265625f,-1.18249997019767771888609786401502788066864013671875f,1.5402000402986999461063533090054988861083984375f,-0.5068999933958053816240862943232059478759765625f,-0.02069999395236321593216644032509066164493560791015625f,0.85089998273029887609908428203198127448558807373046875f,-0.61080000555217228708215770893730223178863525390625f,1.28759999906420663506878554471768438816070556640625f,-0.0893999835610390647389067453332245349884033203125f,-0.073600019359588486622669734060764312744140625f,0.9393000257730486080021137240692041814327239990234375f,-0.088100000792741772759342211429611779749393463134765625f,0.105000007748603962909328402020037174224853515625f,1.17130002035647606817292398773133754730224609375f,-0.0313999979257584271863379399292171001434326171875f,0.956199972751736737563987844623625278472900390625f,0.017000006541609824495253633358515799045562744140625f,1.134800015580653909097463838406838476657867431640625f,-0.353799997818470046695438213646411895751953125f,0.3659999896734953406252088825567625463008880615234375f,-0.3220000057220460121243377216160297393798828125f,-1.0300000090524552565085514288512058556079864501953125f,0.13209999733567234780906574087566696107387542724609375f,0.10929999831765890849766265091602690517902374267578125f,-0.310399989402294185225628098123706877231597900390625f,0.7237999818369746929391794765251688659191131591796875f,0.20979998607635508278690394945442676544189453125f,-0.36709999781995994450056741698062978684902191162109375f,0.00379999915957451861459048814140260219573974609375f,-0.98369998754113918693064988474361598491668701171875f,-0.332099999868869844021901371888816356658935546875f,-0.7527000127688048269902765241567976772785186767578125f,-0.0162000012874603527279759873636066913604736328125f,0.04660000432431699035618066773167811334133148193359375f,-0.05119999666810048921661291387863457202911376953125f,0.9038999608218671877324368324480019509792327880859375f,-0.4681999930024147715812432579696178436279296875f,0.028800001990795021100666417623870074748992919921875f,0.4541000045478345015226295799948275089263916015625f,0.28029998389333510289134210324846208095550537109375f,0.21770000855922688742793980054557323455810546875f,-0.2594999914541842311876962412497960031032562255859375f,-0.93180004857182563426931665162555873394012451171875f,1.089100033903122266565333120524883270263671875f,-0.04799999457597703411693146335892379283905029296875f,1.742199966597556937131230370141565799713134765625f,-0.694800031897425895976994070224463939666748046875f,1.24600003847479801066810978227294981479644775390625f,0.6714000020980837035722288419492542743682861328125f,1.2299999725818633322660389239899814128875732421875f,1.4775000414252286162763994070701301097869873046875f,-0.309600035929680217350323800928890705108642578125f,-0.464099992515146608784704085337580181658267974853515625f,-0.2804999773278835650813789470703341066837310791015625f,0.2378999776303769575491742216399870812892913818359375f,1.36939995748400722419546582386828958988189697265625f,0.41520002271532963078470856999047100543975830078125f,0.10829997820854231349585461430251598358154296875f,0.334800004851818044926403672434389591217041015625f,-1.4294999946951865155142513685859739780426025390625f,1.33500001922249822428057086654007434844970703125f,0.016299943280220219321563490666449069976806640625f,-0.4620000009536742169302669935859739780426025390625f,-0.5689999655485156182521677692420780658721923828125f,-0.33029999108314456179869011975824832916259765625f,1.2272999678969380710213954444043338298797607421875f,0.77879999454021486826604814268648624420166015625f,0.5967999917984005975313266390003263950347900390625f,-1.05360002755522774720020606764592230319976806640625f,-0.39750003278255352512360332184471189975738525390625f,-1.3677000130295748903108687954954802989959716796875f,-0.47989998834431180085857704398222267627716064453125f,0.116599994415044871942654935992322862148284912109375f,-0.8101999804332853383215251596993766725063323974609375f,-0.2806000098824501520766716566868126392364501953125f,0.9282999711304906753639443195424973964691162109375f,0.0268999966740608442705706693232059478759765625f,-0.6845999744251372032266544920275919139385223388671875f,-0.16230000082850448706039969692938029766082763671875f,0.321100011014938402365004321836750023066997528076171875f,-0.1459999967515468455303562222979962825775146484375f,0.326600005851685804003636803827248513698577880859375f,0.360399995362758573946848628111183643341064453125f,0.4440999833509327032743385643698275089263916015625f,0.18070000000596042610823133145458996295928955078125f,1.0053999631389978208773072765325196087360382080078125f,0.298799998116493270572391338646411895751953125f,0.9313999773621561839576088459580205380916595458984375f,0.065300008666515196864565950818359851837158203125f,0.44209998541474337940826444537378847599029541015625f,-0.2367999920964241056253740680404007434844970703125f,-0.6441999870717527443275685072876513004302978515625f,-0.132800005164742440655345490085892379283905029296875f,-0.52360000296831099575456391903571784496307373046875f,-0.16989999721050264014365893672220408916473388671875f,0.42109998672604564973198648658581078052520751953125f,-0.06349999816715712341874677804298698902130126953125f,-0.27350000051408984713674499289481900632381439208984375f,-0.370600008094310595652132178656756877899169921875f,-1.1638999787405130970086020170128904283046722412109375f,0.3346999919295310377265195711515843868255615234375f,0.04309996666908322282552035176195204257965087890625f,0.3219000047206874715044477852643467485904693603515625f,-0.98299999323487252667064240085892379283905029296875f,0.40350002387166039596877453732304275035858154296875f,2.02769996623992909690059605054557323455810546875f,-0.0590999975442887404142311424948275089263916015625f,-1.3717999524593356142077027470804750919342041015625f,-0.30570002101659810023193131200969219207763671875f,0.0093000224947932252916871220804750919342041015625f,0.42389999345540996245063070091418921947479248046875f,1.04979999263286583044418875942938029766082763671875f,-0.9647999835431573245614345069043338298797607421875f,-0.0660999812126163988068583421409130096435546875f,-0.71549999859929069856434580287896096706390380859375f,1.1557999647974970258701432612724602222442626953125f,-1.0458000100970270551670182612724602222442626953125f,0.690999984979629999770622816868126392364501953125f,-1.0215000044405453394347205176018178462982177734375f,-0.4876999662399299495518789626657962799072265625f,0.85259999057054525195553651428781449794769287109375f,-0.3334000155687331101717063575051724910736083984375f,0.3960000062137840615150707890279591083526611328125f,-0.61400001934170678197233428363688290119171142578125f,0.497099999421835025970040078391321003437042236328125f,1.30609996124506011483390466310083866119384765625f,-0.195000017285347215789670372032560408115386962890625f,-0.870900007820128774227441681432537734508514404296875f,1.441199997264146048792099463753402233123779296875f,-0.67410003628730663649548660032451152801513671875f,-0.13829996613860107146365407970733940601348876953125f,0.3531999899849296298270928673446178436279296875f,-0.12460000791549674659108859486877918243408203125f,0.8802000159353016162100402652868069708347320556640625f,0.208200009691715148818502711947076022624969482421875f,0.739999975450337021953828298137523233890533447265625f,0.1946999949097634186045979731716215610504150390625f,0.7016999930694700626787607689038850367069244384765625f,-0.06719999550580979530423064716160297393798828125f,0.377499987557530669590732941287569701671600341796875f,0.116099997305870061836685636080801486968994140625f,-0.9379999990835783396647684639901854097843170166015625f,0.007599996009469034419225863530300557613372802734375f,-0.8036999937251210202049378494848497211933135986328125f,-0.341000013887882147400887333787977695465087890625f,-1.217899982883035914227320972713641822338104248046875f,-0.09130000558570017243908978343824855983257293701171875f,-1.2695999641805888469292540321475826203823089599609375f,-0.2155999899148941523208122816868126392364501953125f,1.2101999668359757489355388315743766725063323974609375f,-0.21530000240802760202996068983338773250579833984375f,0.7693999935075641616322172922082245349884033203125f,0.09280000002384181101433568983338773250579833984375f,-0.59530001041740165934612605269649066030979156494140625f,-0.06610000118017200776421304908581078052520751953125f,-0.621500002838671239402401624829508364200592041015625f,0.05099999518692488464921552804298698902130126953125f,0.196399993902444780502491994411684572696685791015625f,-0.11220000385046002833178135915659368038177490234375f,-0.7616999961242079120182779661263339221477508544921875f,0.4016999949693680349582791677676141262054443359375f,-0.12799998950958269006150658242404460906982421875f,0.0155999985575672184268114506267011165618896484375f,1.5766000315189359071155195124447345733642578125f,-1.2741000158727164848215807069209404289722442626953125f,-0.09879999409317985481493451516143977642059326171875f,-0.55680000595450440670219904859550297260284423828125f,0.805899994707107936164902639575302600860595703125f,-0.87090002257227894233437837101519107818603515625f,-0.555899991726874986852635629475116729736328125f,-0.261299996626377151187625713646411895751953125f,-2.038399993515014330114354379475116729736328125f,0.53940001552402971896071903756819665431976318359375f,-0.79430000714659687588437009253539144992828369140625f,1.5204000238239760989245041855610907077789306640625f,-0.63450002121925308529171161353588104248046875f,0.745500001970678116691715331398881971836090087890625f,-1.5211999779298899238000331024522893130779266357421875f,0.23459998556375527556383531191386282444000244140625f,2.4121999344110491847459343262016773223876953125f,0.274499994099139943415366360568441450595855712890625f,1.6173999790310862323394758277572691440582275390625f,0.212400013750791760713809708249755203723907470703125f,-0.7475000229477881674711170489899814128875732421875f,0.991800022047758300658415464567951858043670654296875f,0.0994999666810034710806576185859739780426025390625f,0.267600023502111650230972372810356318950653076171875f,0.808399996495246409011770083452574908733367919921875f,1.000800014716386687751992212724871933460235595703125f,-0.2115000314116475266246197861619293689727783203125f,-1.27980000574588803630149413947947323322296142578125f,0.2701000025287267902740495628677308559417724609375f,-0.3085999981999396624132714350707828998565673828125f,0.38280000505298394930520089474157430231571197509765625f,0.0190999985873698818750199279747903347015380859375f,0.0969999935030936288882230655872263014316558837890625f,0.228099999935925001892655927804298698902130126953125f,0.380499992080032767294284212766797281801700592041015625f,-0.32129999245405205243741875165142118930816650390625f,0.62659999612867822715855936621665023267269134521484375f,0.03649999773502354827314775320701301097869873046875f,-0.4699999907240270946573446053662337362766265869140625f,-0.084800004851818044926403672434389591217041015625f,-0.002000003561377194927217715303413569927215576171875f,-0.06269999499917044971653012908063828945159912109375f,-1.1481999956846238308827423679758794605731964111328125f,-0.2226000093460083206764466012828052043914794921875f,-0.6893999841198326095081938547082245349884033203125f,-0.0396000013589858923523934208787977695465087890625f,-0.28640000343918803338283396442420780658721923828125f,0.2001000008523463602472247657715342938899993896484375f,-0.78860000606030222769504689495079219341278076171875f,0.45540000519752510399484890513122081756591796875f,0.0483999922484159572633188872714526951313018798828125f,0.48020000408887852927364292554557323455810546875f,-0.3763000269874929859525991560076363384723663330078125f,0.0743999904155732139088286203332245349884033203125f,-0.225399995586275991854563471861183643341064453125f,-0.0969000044226646650713519193232059478759765625f,-0.50169999977499235566114066386944614350795745849609375f,-0.394700007128715668613949674181640148162841796875f,0.81669999914169277843711824971251189708709716796875f,-0.385500030636787727189584984444081783294677734375f,-0.15039998373985241642003529705107212066650390625f,0.52320001773834245994976299698464572429656982421875f,0.93419997828006717810467307572253048419952392578125f,-0.2807999955683946335938117044861428439617156982421875f,0.6070999927163123199846950228675268590450286865234375f,-0.088200012671947103370939657907001674175262451171875f,0.7174000147938723870311150676570832729339599609375f,-0.037500003576278917449826622032560408115386962890625f,-0.9167999920964238214082797640003263950347900390625f,-0.4086000068426134390620063641108572483062744140625f,-0.6675999914646144617336176452226936817169189453125f,0.5688000212132937605247207102365791797637939453125f,-1.4989999908804900741188248503021895885467529296875f,1.09770003061294563195815499057061970233917236328125f,-0.06459999003410388240808970294892787933349609375f,0.93720001428127286402514073415659368038177490234375f,0.7007999573469163578920415602624416351318359375f,0.12330001657903222334056181352934800088405609130859375f,-1.120600012937188250106146369944326579570770263671875f,-0.6006000173330310332175940857268869876861572265625f,-0.215600029850005370235521695576608180999755859375f,-0.2357999786555762256057278136722743511199951171875f,-0.500900040006637681244683335535228252410888671875f,0.66120001529455141309199461829848587512969970703125f,0.56699997293949166987658827565610408782958984375f,0.42240001524090775575359657523222267627716064453125f,-0.4939000038862229047254004399292171001434326171875f,1.2891000458240515769148260005749762058258056640625f,-0.118699999480694540210379273048602044582366943359375f,-0.0872000044465064405585508211515843868255615234375f,-0.42419998666197067382910290689324028789997100830078125f,0.21739999288916589392783862422220408916473388671875f,-0.431700017134845015487343289350974373519420623779296875f,0.33459998914003374892445208388380706310272216796875f,-0.777499985881149768829345703125f,-0.078599988365173345528091886080801486968994140625f,0.853799999569356149464738336973823606967926025390625f,-0.318199998664856043006921026972122490406036376953125f,0.2207999922156334005052258362411521375179290771484375f,0.3692000000357626987579351407475769519805908203125f,0.98349996426701569163242311333306133747100830078125f,0.17169999556541437613077505375258624553680419921875f,0.1956999951258302505863184705958701670169830322265625f,0.040700002241134569658242980949580669403076171875f,0.6967999892279503892922321028891019523143768310546875f,-0.1936000071406365208304123370908200740814208984375f,-1.1233000157967205634435003958060406148433685302734375f,0.051199995923042251888546161353588104248046875f,-0.5594000126183031884607999018044210970401763916015625f,0.28969999938011170570462127216160297393798828125f,-0.49649999922513965433523708270513452589511871337890625f,-0.04150000518560403861556551419198513031005859375f,-0.1656000004947182357994961421354673802852630615234375f,-0.173299994897842424279588158242404460906982421875f,0.689299998056888352238047446007840335369110107421875f,-0.1314999995231627138991825631819665431976318359375f,0.699600023151933925191769958473742008209228515625f,-0.05180000371932980129940915503539144992828369140625f,-0.25530000358149396166851374800899066030979156494140625f,0.708300006520747871263665729202330112457275390625f,0.06299999189376848107713158242404460906982421875f,0.516000025808811102479012333787977695465087890625f,0.1320999462246899014417067519389092922210693359375f,-0.262799976927042155949720836360938847064971923828125f,-1.4796999651074411730178326251916587352752685546875f,-0.1125000250339507346097889239899814128875732421875f,1.400999993920326147645027958787977695465087890625f,0.4968000089347366365899461015942506492137908935546875f,0.74879999319910961474988653208129107952117919921875f,-1.26509999250769578083009037072770297527313232421875f,0.568499994516372453290387056767940521240234375f,-0.802799989444017203510384206310845911502838134765625f,-0.53769996355772065044220653362572193145751953125f,0.047100010597706187809308175928890705108642578125f,1.456399993753432653420532005839049816131591796875f,0.92460003339648277886908545042388141155242919921875f,-1.10830003840923296820619725622236728668212890625f,0.7878000289320947757687463308684527873992919921875f,0.071399957990646356620345613919198513031005859375f,-0.05609999209046367951714273658581078052520751953125f,-0.70270000796318043967403355054557323455810546875f,-0.3654000069856646604193883831612765789031982421875f,-0.8735999925374979824255206040106713771820068359375f,0.4640999950110913374601295799948275089263916015625f,1.7738999830245969491215873858891427516937255859375f,-0.17910002883672770934708751155994832515716552734375f,0.950400028443336708505739807151257991790771484375f,-0.882900013244152237490425250143744051456451416015625f,-0.3751999972343444955669156115618534386157989501953125f,0.2998000027656555488420053734444081783294677734375f,1.2656000051885840651522130428929813206195831298828125f,-0.1465999991461635065803648103610612452030181884765625f,0.244499985419213972104302001753239892423152923583984375f,-0.4681999876379965286332662799395620822906494140625f,0.2863000348851081344747626644675619900226593017578125f,-0.182500006072223219799610660629696212708950042724609375f,-0.02310000069960949087288781811366789042949676513671875f,-0.49610000088214878388725992408581078052520751953125f,0.4587999923422934056560507087851874530315399169921875f,-0.511300000202655979819610365666449069976806640625f,0.18109999402761456366306447307579219341278076171875f,0.341599998140335259222410968504846096038818359375f,1.2075999957486989178079284101841039955615997314453125f,-0.12399999356269830741439363919198513031005859375f,0.164200021009146990280669342610053718090057373046875f,-0.18050000023841850094186156638897955417633056640625f,-0.46039998783767221990359530536807142198085784912109375f,-0.05579999802708623946045918273739516735076904296875f,0.20690000263452523743268329781130887567996978759765625f,-0.3747000020623207916514729731716215610504150390625f,0.4890999905034900674394293673685751855373382568359375f,-0.318400012886524308441948960535228252410888671875f,-1.16009999261945484505531567265279591083526611328125f,0.04459999569654460316314725787378847599029541015625f,-0.189100010173022692328004268347285687923431396484375f,0.071800008413195559597852479782886803150177001953125f,1.132299991738796141049760990426875650882720947265625f,-0.0374999958276749367769298260100185871124267578125f,-1.255199978682398853635504565318115055561065673828125f,-1.38300002110004438549140104441903531551361083984375f,0.869600050234794252901338040828704833984375f,-0.124200001414120020371001373860053718090057373046875f,0.131099989929795324172800974338315427303314208984375f,0.33179994217753350227440023445524275302886962890625f,0.47410001661777556591914617456495761871337890625f,0.59760002031698833402373338685720227658748626708984375f,0.57389999727383222971610621243598870933055877685546875f,0.574500000953674483383792903623543679714202880859375f,1.1111999890685080316643507103435695171356201171875f,0.5163000227034100220180334872566163539886474609375f,0.477300026607512251075604581274092197418212890625f,-1.57590003851652227240265347063541412353515625f,0.538400010800362593954559997655451297760009765625f,-0.50370003347396874460173421539366245269775390625f,-0.523799958777427576706031686626374721527099609375f,0.656399991369247093331296127871610224246978759765625f,-0.18600001686811395984477712772786617279052734375f,0.031499991923570558327583057689480483531951904296875f,0.37869999920129782111644090036861598491668701171875f,0.31800001022219692714543270994909107685089111328125f,0.953700003671645646363685955293476581573486328125f,1.5948000562608246610807327670045197010040283203125f,-0.393899999713898552045066026039421558380126953125f,0.499800024670362663670175606966949999332427978515625f,0.3790999757885931131795587134547531604766845703125f,-0.68070001513063882470078169717453420162200927734375f,1.32370001305937723401484618079848587512969970703125f,0.804300014895201087483656010590493679046630859375f,0.21419999467134476223151295926072634756565093994140625f,0.190800002050399708508621188229881227016448974609375f,-0.6931000122293828891173461670405231416225433349609375f,-0.073600000584125524483170011080801486968994140625f,-0.368699992570280965509255111101083457469940185546875f,0.1931999964296817751119306194595992565155029296875f,0.005299994957447096144420584096224047243595123291015625f,-0.2901999912738801867817528545856475830078125f,0.23749999266117793883523745535057969391345977783203125f,0.1283000023484230922576898592524230480194091796875f,1.1215000174045559599989019261556677520275115966796875f,0.08040000445246686666678215260617434978485107421875f,0.9949999750778080187529894828912802040576934814453125f,0.00840000394582762766049199854023754596710205078125f,0.50940002297461017821689210904878564178943634033203125f,0.137199998150765889448621237534098327159881591796875f,0.0106999983295799072635645643458701670169830322265625f,0.313100003325939013620882178656756877899169921875f,-0.47110001995563521859367028810083866119384765625f,-0.27710000523328781607546034138067625463008880615234375f,-0.1703999958097934808165518916212022304534912109375f,0.147500005066394823671771519002504646778106689453125f,0.7590000000447034711470450929482467472553253173828125f,-0.02089999962449073933612453402020037174224853515625f,0.0371000082135199971133943108725361526012420654296875f,-0.12670000666677960765582611202262341976165771484375f,0.1602000068083402339880194631405174732208251953125f,-0.13170000391006464468546255375258624553680419921875f,0.97430000345855949372975146616226993501186370849609375f,0.46139999107122431354355285293422639369964599609375f,0.97219998090267178980639073415659368038177490234375f,0.0042000140428546384185892748064361512660980224609375f,-1.11210001879334452468128802138380706310272216796875f,-0.1935000036060809947002780972979962825775146484375f,-1.06409997608661655021933256648480892181396484375f,-0.204899989461898801579309292719699442386627197265625f,-0.0517000009298325124973416677676141262054443359375f,0.07260000532269561546172553789801895618438720703125f,0.225300020885466523168361163698136806488037109375f,-0.94890002877116241819521746947430074214935302734375f,0.996100048565864426564075984060764312744140625f,-0.77370000113844827893672118079848587512969970703125f,0.9398000000834461076237857923842966556549072265625f,-0.765600042665005364739272408769465982913970947265625f,-0.0685999630331988186071612290106713771820068359375f,-0.5597999995619058477558382946881465613842010498046875f,-0.5974999812245369579244425040087662637233734130859375f,0.3453000368297107769421927514486014842987060546875f,-0.07050002884864881025350769050419330596923828125f,0.989700016851723152200293043279089033603668212890625f,0.7243999856844542950540244419244118034839630126953125f,-1.3710000286400318980639667643117718398571014404296875f,-0.26619995941519736248892513685859739780426025390625f,-0.354900006598234174504113980219699442386627197265625f,-0.44429997004270571636652675806544721126556396484375f,0.873900037413835928390426488476805388927459716796875f,1.374600015068053071587428348721005022525787353515625f,-0.256200009185076016393622921896167099475860595703125f,1.3733000184416770395046114572323858737945556640625f,-0.411899990862608422759194581885822117328643798828125f,-0.8358999902367594270202744155540131032466888427734375f,-0.2068999956309796051101557168294675648212432861328125f,0.57999998454004531023286972413188777863979339599609375f,0.122100008809566507039789939881302416324615478515625f,0.334999995790421944974468715372495353221893310546875f,-0.067600004875659980285718120285309851169586181640625f,-1.0029000021055338720543659292161464691162109375f,-0.107999998971819888282652755151502788066864013671875f,-0.50749999277293678101585783224436454474925994873046875f,0.1411000125050545506155685870908200740814208984375f,-0.6004999744221570789903807963128201663494110107421875f,0.0022000110030173658515195711515843868255615234375f,0.4216999891206623107819950746488757431507110595703125f,0.3654999896585942043003569779102690517902374267578125f,-0.7601999759629369801672282846993766725063323974609375f,-0.15109999656081196661716603557579219341278076171875f,0.0070000001713632631350492374622263014316558837890625f,-0.0082000014960765810201337444595992565155029296875f,0.22969999733120205487324483328848145902156829833984375f,-0.00369998765289791808896779912174679338932037353515625f,0.83429997481107720513904268955229781568050384521484375f,-0.31960000918209552356330505062942393124103546142578125f,0.2124000109940766822802515889634378254413604736328125f,0.0145000030398367840689388685859739780426025390625f,-0.74400001200288523595105516506009735167026519775390625f,0.1323999987006185818927406216971576213836669921875f,-0.82980001707077011285917933491873554885387420654296875f,-0.0381000009417533647138043306767940521240234375f,0.07630000195354236236511269453330896794795989990234375f,-0.0953000034511088944810808243346400558948516845703125f,-0.473699998307228753446906921453773975372314453125f,0.09869999763667614001150241165305487811565399169921875f,1.017999983474612069045406315126456320285797119140625f,-0.7923000264585018381779946139431558549404144287109375f,-0.31139999479651425673409903538413345813751220703125f,1.0482000290632249583921975499833934009075164794921875f,-0.55990003133416177405479174922220408916473388671875f,0.16649999855458741393476884695701301097869873046875f,-0.8724999912083148956298828125f,-0.55380004058480292172816916718147695064544677734375f,-1.7638999722957606763884541578590869903564453125f,-1.19880002955794306984671493410132825374603271484375f,-0.84529997618198393638522247783839702606201171875f,-0.32729998540580307331282483573886565864086151123046875f,-1.500199955436587462287434391328133642673492431640625f,0.813299996089935195442421900224871933460235595703125f,0.96699996131658583209400603664107620716094970703125f,-0.0425999984681606491676575387828052043914794921875f,0.37469999938011167017748448415659368038177490234375f,-0.823200022394954711302261785021983087062835693359375f,0.4644999875053763549459517889772541821002960205078125f,0.6759000168725848300965708403964526951313018798828125f,0.9792000177308912700624432545737363398075103759765625f,0.28050001513958022769656963646411895751953125f,-1.030000019669532917987453402020037174224853515625f,0.01860003366470408536770264618098735809326171875f,-1.423000002026558519219179288484156131744384765625f,-0.47400001069903385797488226671703159809112548828125f,-0.5534999808073042260048168827779591083526611328125f,0.63060000682771188973418929890613071620464324951171875f,0.00590001448839883035901721086702309548854827880859375f,0.05489999504983422706771989396656863391399383544921875f,0.6560000037550925977569704627967439591884613037109375f,0.3198000100672244716548675569356419146060943603515625f,-0.378000018194317721764718953636474907398223876953125f,-0.285700007009506151689492980949580669403076171875f,-0.88719997810870410148709197528660297393798828125f,0.05719999759197236244290252216160297393798828125f,0.11159998193532250088111368313548155128955841064453125f,0.0214000007942318770659539950429461896419525146484375f,0.290199994216859324946966580682783387601375579833984375f,0.5184999989867211667160518118180334568023681640625f,-0.332699972461164339421202384983189404010772705078125f,-0.3211000011801719011828026850707828998565673828125f,-0.1232999921783803909391963316011242568492889404296875f,0.102300011035799887082475834176875650882720947265625f,1.274899986183643196824277765699662268161773681640625f,0.0374000034689903060325377737171947956085205078125f,-0.1117000135585666242832303396426141262054443359375f,-0.34959998794794078236236600787378847599029541015625f,0.1622000096246598577209852010128088295459747314453125f,-0.23479999203681956032596644945442676544189453125f,0.61979998250007639626346644945442676544189453125f,0.1663999932691455629907295588054694235324859619140625f,-0.203999998293816930949873267309158109128475189208984375f,-0.0513999976277350612008376629091799259185791015625f,1.1957999912098049577480196603573858737945556640625f,0.249999988675117634784328402020037174224853515625f,-0.35389999647289493989177344701602123677730560302734375f,0.341199991303682281795772723853588104248046875f,0.17530001075267787058464818983338773250579833984375f,-0.9116999984338878260103911088663153350353240966796875f,0.0652000569507473759500726373516954481601715087890625f,-0.44640001066625101255880281314603053033351898193359375f,-0.980400036415457254435068534803576767444610595703125f,1.2183000100970264867328296531923115253448486328125f,-0.3516000076770779969592695124447345733642578125f,0.24630000824928277580738722463138401508331298828125f,0.6223999554872516881687261047773063182830810546875f,-0.111899999989569121083832214935682713985443115234375f,0.429399996152520113668060730560682713985443115234375f,-0.58080001084208543460363216581754386425018310546875f,-0.850899967288970771051026531495153903961181640625f,0.58259998952746361311483269673772156238555908203125f,0.20049998712539718326297588646411895751953125f,-0.7563000130921597730093708378262817859649658203125f,1.52910001110434468074572578188963234424591064453125f,0.86400001129508030572878851671703159809112548828125f,1.02489993879795093789653037674725055694580078125f,0.431099964821338499376679465058259665966033935546875f,0.5085999838948254136994364671409130096435546875f,-0.413400012290477292253854102455079555511474609375f,0.353000002622604114321802626363933086395263671875f,0.105300007511675008942120257415808737277984619140625f,-0.056400013199448206790975746116600930690765380859375f,-0.407100024902820933903058175928890705108642578125f,1.683799984407425398558189044706523418426513671875f,0.17040000862479143250993729452602565288543701171875f,-0.159300023984908278862349106930196285247802734375f,-0.8459999775290487633583325077779591083526611328125f,1.218700008067488571583680823096074163913726806640625f,0.2412000102281570690365697373636066913604736328125f,1.1057999940410254691158797868411056697368621826171875f,0.349400005912780642347570392303168773651123046875f,1.240799960625171838302094329264946281909942626953125f,-0.2062000063538551586361791123636066913604736328125f,0.701899999803304641687873299815692007541656494140625f,-0.01769999708533287474665485206060111522674560546875f,0.015099998654425184074767685160622932016849517822265625f,0.284300008189678266035116394050419330596923828125f,0.7240000000074504615810155883082188665866851806640625f,0.39009999772310255394813793827779591083526611328125f,-0.079699992264807395514480958809144794940948486328125f,-0.0766999884128569675567632657475769519805908203125f,0.3752000197723506236258117496618069708347320556640625f,0.1334000029399990960854438526439480483531951904296875f,0.101000007182359408108851539509487338364124298095703125f,-0.0391000039517879205419603749760426580905914306640625f,-0.29179998788684613497679265492479316890239715576171875f,0.1761999941349028375725538353435695171356201171875f,-0.7462000063166021046612286227173171937465667724609375f,0.23090000707507130073281587101519107818603515625f,0.356999994471669079842968130833469331264495849609375f,0.1077999922752379546864176518283784389495849609375f,-0.58690003716796590449433779212995432317256927490234375f,-0.1774000004887579251544593716971576213836669921875f,0.3643000158265234933452347831916995346546173095703125f,-0.23850000345706945381607511080801486968994140625f,-0.693399999549984880076181070762686431407928466796875f,0.1285999871730805210745529620908200740814208984375f,1.123700042414665034584686509333550930023193359375f,-1.14990003297328957643230751273222267627716064453125f,0.8170000104904175941555877216160297393798828125f,-1.5420000033378595816202505375258624553680419921875f,0.658399998283385912145604379475116729736328125f,0.16170001240372666728717376827262341976165771484375f,1.3413000177860254780171089805662631988525390625f,-0.24900000935792920842004605219699442386627197265625f,-0.336299996030330561325172311626374721527099609375f,-1.04190003422498733698375872336328029632568359375f,0.2796000067234043484631911269389092922210693359375f,-0.777599991911649635056846818770281970500946044921875f,-0.7830999586224560715663756127469241619110107421875f,-0.92310005339384115785605899873189628124237060546875f,-0.64359996035098987476885668002068996429443359375f,-0.0071999997153877615119199617765843868255615234375f,0.338200008872151069994060890167020261287689208984375f,0.31410000641047941538630539071164093911647796630859375f,-0.172599989452958180180530689540319144725799560546875f,-0.25589996788501689906070168945007026195526123046875f,-1.591799975109100984127508127130568027496337890625f,-0.7902000121355055028971037245355546474456787109375f,-0.14099998736381547104201672482304275035858154296875f,0.7860000435411933494833647273480892181396484375f,0.0808999225854876158336992375552654266357421875f,-0.69360004275441244914190974668599665164947509765625f,-0.462299973559378685195042635314166545867919921875f,0.9834000230193140623669023625552654266357421875f,-0.249000015616417158526019193232059478759765625f,-0.17999998718500176408952029305510222911834716796875f,0.4751999885171651616388999173068441450595855712890625f,-0.24100000852346425972427823580801486968994140625f,-0.61290000873655048341248630094924010336399078369140625f,-0.186500004589557732970206416212022304534912109375f,-0.33480000954568378546127860317938029766082763671875f,-0.067800000023841899832177659845910966396331787109375f,-0.56800000564008945591609744951711036264896392822265625f,0.52520000587701787253536167554557323455810546875f,-0.46810001923292876568893916555680334568023681640625f,-0.09910000328123569346416843472979962825775146484375f,-0.127799996857345110612413918715901672840118408203125f,0.1437000026285648601742650498636066913604736328125f,-0.045099999473988905407395577640272676944732666015625f,-0.1425000113248824362699451739899814128875732421875f,0.5861999880626800329963543845224194228649139404296875f,-0.20800000523030759413956047865212894976139068603515625f,0.1883000022739171885888254109886474907398223876953125f,-0.075799998474121110803025658242404460906982421875f,-0.0178999914884567790096525641274638473987579345703125f,-0.2314000047802924342477126629091799259185791015625f,-1.114299989414214930860680397017858922481536865234375f,0.26599999503791327271073896554298698902130126953125f,0.29139999122023585442775583942420780658721923828125f,0.2374000025749207765102255507372319698333740234375f,1.1440000026896595120007305013132281601428985595703125f,0.0412999996066094610114305396564304828643798828125f,-0.0638999930083752332166113774292171001434326171875f,-0.09800000023841859420059563490212894976139068603515625f,0.09779999693185092990432849546778015792369842529296875f,0.077899998790025737349651535623706877231597900390625f,-0.50180006570816004796142806299030780792236328125f,-0.1443000095307827024271318805404007434844970703125f,0.4945000088512898628323455341160297393798828125f,-0.481099984937906288706699342583306133747100830078125f,0.36650001528114017901316401548683643341064453125f,-1.129500006914138765523603069595992565155029296875f,0.1229000320196149687035358510911464691162109375f,0.9113000119745731097964380751363933086395263671875f,-0.5406000209093093644696637056767940521240234375f,0.60089999321699139045449555851519107818603515625f,0.0016999910950659113950678147375583648681640625f,0.8250000216066837310791015625f,-1.260800025705993476776711759157478809356689453125f,-1.264900024479627571594164692214690148830413818359375f,1.008200025784969255937539855949580669403076171875f,-0.631199987056851430367032662616111338138580322265625f,-0.1460000060647725916851413785479962825775146484375f,0.54800000463426112862208583464962430298328399658203125f,-0.32790001406371604986134116188623011112213134765625f,0.0093000116169452695658037555404007434844970703125f,-0.267500020563602447509765625f,-0.55849997678399088130163363530300557613372802734375f,0.38169999780058816440941882319748401641845703125f,0.48140001312494273832953695091418921947479248046875f,0.9492999813675879750007879920303821563720703125f,-0.7662000254273413357708477633423171937465667724609375f,0.2537999979674818717967355041764676570892333984375f,-0.08270000766515739343276436557061970233917236328125f,0.95470001398324910724113578908145427703857421875f,-0.839299970191717203960024562547914683818817138671875f,-0.18950001019239426369722423260100185871124267578125f,-0.2078999939098954696792276308769942261278629302978515625f,-0.2961000033408403009360654323245398700237274169921875f,-0.214399998126924007380011971690692007541656494140625f,-0.12760001046359548126929439604282379150390625f,0.28910002906024512281391025680932216346263885498046875f,-0.74630002106428161567919232766143977642059326171875f,0.1720000001713632986621860254672355949878692626953125f,0.04040000333487991923675508587621152400970458984375f,-0.731600004175305240750049051712267100811004638671875f,-0.3988000052690505725649927626363933086395263671875f,0.422599988186359354358501150272786617279052734375f,0.189400002038478731947179767303168773651123046875f,-0.15540000717192892221163447175058536231517791748046875f,-0.136200007843971349075218313373625278472900390625f,0.09140000187456609470615376267232932150363922119140625f,-0.0914000006079673799064266859204508364200592041015625f,-0.4664999943822623063027776879607699811458587646484375f,0.1821000031471253777226593228988349437713623046875f,0.12399999278038731365114699656260199844837188720703125f,0.47720000958740715901740259141661226749420166015625f,-0.31300000709295272560694911589962430298328399658203125f,-0.01100000345706941828893832280300557613372802734375f,-0.13699999525398010025156736446660943329334259033203125f,-0.24820000447630885531680178246460855007171630859375f,-0.141900012953579679475524244480766355991363525390625f,0.3795000087022781798395953956060111522674560546875f,0.0558000001505016829383976073586381971836090087890625f,-0.2969999973773955748157504785922355949878692626953125f,0.074400021968781704373441243660636246204376220703125f,-0.332999991595745115091631305404007434844970703125f,-0.11620000002086161572378841810859739780426025390625f,0.3226999808430672800341199035756289958953857421875f,-0.48589999202489853047381984652020037174224853515625f,0.7640999775022265794177656061947345733642578125f,0.9185000015199182854530590702779591083526611328125f,0.34920001016855195530297351069748401641845703125f,0.488700008440017708011282593361102044582366943359375f,-0.699700005266070235165898338891565799713134765625f,-0.21500001192092899060526178800500929355621337890625f,-0.66899999535083765067611238919198513031005859375f,0.4783999851703644168310347595252096652984619140625f,-0.3109999787211421562460600398480892181396484375f,0.063600003117322945200839967583306133747100830078125f,-0.274200017619133262769537395797669887542724609375f,0.29499999888241290602053368274937383830547332763671875f,-0.20879999923408032458382876939140260219573974609375f,-1.204999983310699462890625f,1.0072999777317050984493107534945011138916015625f,-0.2798999957203864852317565237171947956085205078125f,-0.061099988439679009388783015310764312744140625f,-0.139999993145465850830078125f,-0.52100000971555715523209073580801486968994140625f,-0.3029000147342681970030753291212022304534912109375f,0.82770000423789014121211948804557323455810546875f,-0.73230000485181800939926688442938029766082763671875f,0.42419999765157712090513086877763271331787109375f,0.79989998953640462087122386947157792747020721435546875f,-0.98549999628961071351795908412896096706390380859375f,0.75470001130104069630988306016661226749420166015625f,-0.2933000107675791667816156405024230480194091796875f,0.0389000183403493959843899574480019509792327880859375f,-0.625700003135204241289102355949580669403076171875f,-0.390799995978176217814592519061989150941371917724609375f,0.293599996113777184092441530083306133747100830078125f,0.294400009042024901617651266860775649547576904296875f,0.152000000357627840230634319595992565155029296875f,-0.0232000192284582684720817269408144056797027587890625f,0.1671999916315076717410192941315472126007080078125f,-0.288800026093423534145898656788631342351436614990234375f,0.616400007760524726307949094916693866252899169921875f,0.38189999358207005197840544497012160718441009521484375f,0.28769999678730950876115457504056394100189208984375f,0.035600000047683710846513349679298698902130126953125f,0.1515999963521956761525188994710333645343780517578125f,0.09780000263154509099194910959340631961822509765625f,-0.54420000987052929986020899377763271331787109375f,-0.115399996554851558272503098123706877231597900390625f,0.144200004431605410815109280520118772983551025390625f,-0.53880000951588147284354590738075785338878631591796875f,0.3443999959290027224056984778144396841526031494140625f,-0.52670001780539765778854643940576352179050445556640625f,0.10189999697208396156611343030817806720733642578125f,0.371900014257431354991467742365784943103790283203125f,-0.3035000213384630995960833388380706310272216796875f,0.37020000487118942533726340116118080914020538330078125f,0.19500001043081294227476973901502788066864013671875f,0.312900015740096737459907672018744051456451416015625f,-0.402900008811056642432646412999019958078861236572265625f,0.13069999929815512285102840905892662703990936279296875f,0.6865999880820510270496015436947345733642578125f,0.82519998888969414707617033855058252811431884765625f,-0.38509999057054500326557899825274944305419921875f,-0.501099968546629082055687831598334014415740966796875f,-0.5842999904572963032478583045303821563720703125f,0.090999998986720953553231083787977695465087890625f,0.27740003356933584655052982270717620849609375f,0.431700005549192411535841529257595539093017578125f,-1.3774999895691877327408292330801486968994140625f,-0.73400000041723245658431551419198513031005859375f,-0.479599989587068620267018559388816356658935546875f,0.56640001029372211149848226341418921947479248046875f,-0.8272999817550186207881779409945011138916015625f,-0.42659999165832995782210446122917346656322479248046875f,0.71859999007880670518488841480575501918792724609375f,0.493299976569414155846970970742404460906982421875f,0.337099999272822969942353665828704833984375f,-0.6062999999046325694251891036401502788066864013671875f,0.7203000020354988208737267996184527873992919921875f,-1.09619998169243348939971838262863457202911376953125f,0.22179999068081368562843636027537286281585693359375f,-1.025400013095140394625559565611183643341064453125f,0.61629999364614462820100015960633754730224609375f,0.17340000513792031000548377051018178462982177734375f,1.28009999444484634523178101517260074615478515625f,0.1409000015616417300634566345252096652984619140625f,0.28189999905824691950328997336328029632568359375f,1.0339000150807200473224156667129136621952056884765625f,-0.5756000146694482122455838180030696094036102294921875f,-0.2673999964654445449241393362171947956085205078125f,0.06330000301897531489458970099803991615772247314453125f,0.0949999928474426769131611081320443190634250640869140625f,0.3442000058472156620581472452613525092601776123046875f,-0.5116999989926813707796782182413153350353240966796875f,-0.42790001652240761131906765513122081756591796875f,-0.1732000039547683112317599807283841073513031005859375f,-0.280399997150897917208567378111183643341064453125f,-0.421699986736476351012470331625081598758697509765625f,-0.49270000706911076804317417554557323455810546875f,-0.16540001000314941137503410573117434978485107421875f,0.5938000055670737964419458876363933086395263671875f,0.5135999960392709784429143837769515812397003173828125f,-0.0311999969661236331575082658673636615276336669921875f,-0.303999997250735953802092126352363266050815582275390625f,-0.246799998950958165977453973027877509593963623046875f,0.54479999244660159973818736034445464611053466796875f,0.1499999931454656376672573969699442386627197265625f,-0.3833000245139002171157471821061335504055023193359375f,0.1889000006079672999703689129091799259185791015625f,-0.28109998657703416657938078060396946966648101806640625f,-0.21209999457895734042978119759936816990375518798828125f,0.676799984012543820455221066367812454700469970703125f,-0.0251999963402749216356824035756289958953857421875f,0.5568000268533828123196371961967088282108306884765625f,-0.3876000084519386490455872262828052043914794921875f,-0.39290000434070793122742770719924010336399078369140625f,0.3496999941647052079218838116503320634365081787109375f,-0.7240999989598988495487219552160240709781646728515625f,0.036499998629093255431143916212022304534912109375f,-0.49690000133067346865800573141314089298248291015625f,0.896200034107268006522417636006139218807220458984375f,0.076700003239512426489454810507595539093017578125f,-0.6462999797135591251162622938863933086395263671875f,0.09320002012252803780256726895458996295928955078125f,-0.68930003530979178094639792107045650482177734375f,0.33429998568892482779801866854541003704071044921875f,-0.61450000274181348913771216757595539093017578125f,0.2005000193119048645939983543939888477325439453125f,-0.5428000339984890842970344237983226776123046875f,-1.07530000360012056859204676584340631961822509765625f,0.66210003175735465674733859486877918243408203125f,0.29739998551309121399555124298785813152790069580078125f,0.81019998091757283731340066879056394100189208984375f,-0.415100006514787622791118337772786617279052734375f,0.617299977284670120525333913974463939666748046875f,0.97070000283718105293928601895458996295928955078125f,-0.98409997727871001416133367456495761871337890625f,0.0556999897241592378804853069595992565155029296875f,-1.573999973893165815752581693232059478759765625f,-0.30009998949021099434730786015279591083526611328125f,0.35079997027367415540766160120256245136260986328125f,1.28329998208284390415201414725743234157562255859375f,-0.94190000293254882990368059836328029632568359375f,1.0111000148892401728772938440670259296894073486328125f,0.1082999954938888009792208322323858737945556640625f,-0.680799980667233572972918409504927694797515869140625f,-0.3116999819308521324501271010376513004302978515625f,-1.06789999014735226268157930462621152400970458984375f,0.31119999353885674509001546539366245269775390625f,-1.3804999920241536326415143776102922856807708740234375f,-0.02440000919103635368401228333823382854461669921875f,0.246000032812357272149483833345584571361541748046875f,-0.33240000346899023497826419770717620849609375f,-0.1728000195443630815361757413484156131744384765625f,0.6768000028252600941414129920303821563720703125f,-0.491800024767220289589886306202970445156097412109375f,0.617800011944770854910302659845910966396331787109375f,-0.09290001752823606739895012651686556637287139892578125f,0.301700006592273695105177466757595539093017578125f,-0.2081999877870084336706213434808887541294097900390625f,-0.2273999947145579625384925748221576213836669921875f,0.1728999993488191666557440839824266731739044189453125f,-0.05049999383091918758736937888897955417633056640625f,0.3653999975234267783008590413373894989490509033203125f,0.091799999435245915879733047404442913830280303955078125f,-0.47260000036805838163900261861272156238555908203125f,0.058599992537498479805435636080801486968994140625f,-0.1230999931000174019146697901305742561817169189453125f,-0.08570000259503718353926160489208996295928955078125f,0.54410000760257226648519690570537932217121124267578125f,-0.403700000095367439456595093361102044582366943359375f,0.2295000369772315362837389329797588288784027099609375f,-0.2870999891400334291802209918387234210968017578125f,0.15749998750165128402755954084568656980991363525390625f,0.42209999672099929313162647304125130176544189453125f,-0.3800000330805779480414230420137755572795867919921875f,0.5384000046908854475447014920064248144626617431640625f,-0.72180000237822528053044379703351296484470367431640625f,-0.19260000725984571801063793827779591083526611328125f,0.0233000021994114181467239177436567842960357666015625f,-1.254300014150143027791273198090493679046630859375f,0.2037000088870524194817335228435695171356201171875f,0.9309000060319903013805742375552654266357421875f,-0.8080999855935573350507183931767940521240234375f,-0.1276999972343446643208153545856475830078125f,-1.4281000130116938695579165141680277884006500244140625f,0.5013000116765500280280321021564304828643798828125f,-0.288200012522935811176694187452085316181182861328125f,-0.259899986779689839977436349727213382720947265625f,-0.546200008589029284422622367856092751026153564453125f,1.12799998641759202655521221458911895751953125f,0.692499999403953570009662144002504646778106689453125f,-0.954599998825788276235471130348742008209228515625f,-0.0238000013388692532867452200662228278815746307373046875f,0.8543999809719620142089979708543978631496429443359375f,0.0134000338971613786043235450051724910736083984375f,-1.4456000072002410661298199556767940521240234375f,-0.3828999880611896333704180506174452602863311767578125f,0.695599971195310473603967693634331226348876953125f,1.0722999875664711577627485894481651484966278076171875f,-0.0276000127732753952614075387828052043914794921875f,-0.1931000039219856745376091566868126392364501953125f,1.66840000245571218329132534563541412353515625f,-0.1709000048398972015917252065264619886875152587890625f,-0.09339996205493861225477303378283977508544921875f,-0.66060001174509519472621832392178475856781005859375f,-0.7245999792307618037057181936688721179962158203125f,-0.68169999154210092484618144226260483264923095703125f,-0.048700020658969833675655536353588104248046875f,-0.5423000111103057907513402824406512081623077392578125f,-0.12329999977797267973755879211239516735076904296875f,-0.16190002167075923100725276526645757257938385009765625f,0.52190002081394215593945773434825241565704345703125f,-0.39700002203881723517042701132595539093017578125f,0.699399999952316164808507892303168773651123046875f,-0.1120000274777417370586363176698796451091766357421875f,0.5872000230729581549127260586828924715518951416015625f,0.1435999921649695476588703968445770442485809326171875f,0.54040000162124623983572746510617434978485107421875f,0.1914000134602191760535561115830205380916595458984375f,-0.202200011897087250645199674181640148162841796875f,-0.02070000130981208830149853383773006498813629150390625f,-0.0980999961361288530525825990480370819568634033203125f,0.1811999869450928724035065897624008357524871826171875f,0.64319999404549577093348489142954349517822265625f,-0.63029998471289860884070321844774298369884490966796875f,-0.12570000045299511981511386693455278873443603515625f,0.257900017900764944034364134495262987911701202392578125f,-0.610700008499622359892100575962103903293609619140625f,0.1775000117346647865446129799238406121730804443359375f,-0.047900017416477425058474182151257991790771484375f,0.36940000524222849076494412656757049262523651123046875f,-0.13899999415874475516829988919198513031005859375f,-0.104300016385316440814534644232480786740779876708984375f,0.554299992692470322452891196007840335369110107421875f,0.517199992525577467716857427149079740047454833984375f,0.33240000257492052782026803470216691493988037109375f,-0.43669999366551603969810457783751189708709716796875f,-0.204100019001960841791287748492322862148284912109375f,-0.0140000169947740449316597732831723988056182861328125f,-0.1293999986484646314011115464381873607635498046875f,-0.222199996846914160641972557641565799713134765625f,-1.06530001283883990481626824475824832916259765625f,-1.0933999947071075808935347595252096652984619140625f,-0.1559999700784686638144194148480892181396484375f,-0.33140001379549488547127111814916133880615234375f,-0.8265999944895501272412730031646788120269775390625f,-0.623399999177455921284263240522705018520355224609375f,0.618600025022029598176231957040727138519287109375f,-0.28620000918507582099437058786861598491668701171875f,1.58920001225471452244164538569748401641845703125f,0.28289998992383480480583557437057606875896453857421875f,-0.47379998973458992139740075799636542797088623046875f,-0.46179998330473903678239366854541003704071044921875f,0.14340001661181389636112726293504238128662109375f,-0.8681999803364277568107354454696178436279296875f,0.47149998193979314464741037227213382720947265625f,0.70499999821186065673828125f,0.447299975794553716923473984934389591217041015625f,-0.37309998961687096397099594469182193279266357421875f,1.41400000071525511202708003111183643341064453125f,0.802599988484382631526159457280300557613372802734375f,-0.472799985122680510585269075818359851837158203125f,-0.466199994879961110427757375873625278472900390625f,-0.436499955564737973645605961792171001434326171875f,-0.025400013542175248204557647113688290119171142578125f,0.758200018632411598673570551909506320953369140625f,0.1419000456988810388025967768044210970401763916015625f,-1.0708000346839430250156510737724602222442626953125f,0.4261000053524970354601464350707828998565673828125f,0.029499992012977482858104849583469331264495849609375f,-0.097199983026087399817782852551317773759365081787109375f,-0.489600001209974156068938100361265242099761962890625f,-0.1271000021785493316883730585686862468719482421875f,0.8579000069856643762022940791212022304534912109375f,0.3454000129088756754214273314573802053928375244140625f,-0.53619999264478668266065142233856022357940673828125f,0.455900010800361632501420672269887290894985198974609375f,0.130900009608268863559032979537732899188995361328125f,-0.147200015361607317032621722319163382053375244140625f,0.410000008046627062441302769002504646778106689453125f,-0.3659000078573821923555442481301724910736083984375f,-0.044499998569488496968915569595992565155029296875f,-0.422600010426342098668328617350198328495025634765625f,0.476599992179870479702685770462267100811004638671875f,-0.4146000028491017008747121508349664509296417236328125f,0.29219999699592591468899627216160297393798828125f,0.39169998498558999955321269226260483264923095703125f,-0.056900001442432568410367821343243122100830078125f,0.2601999997675417120035490370355546474456787109375f,-0.15220000653266918533290663617663085460662841796875f,0.29560002537965812674514154423377476632595062255859375f,-0.2380000093281269268885580459027551114559173583984375f,-0.19610001146197308941054870956577360630035400390625f,0.50929999775886525270607307902537286281585693359375f,0.44910002134889392966243804039550013840198516845703125f,-0.2763000104844570703477302231476642191410064697265625f,0.4081000429257752859513175280881114304065704345703125f,-0.3145000075101851422232357435859739780426025390625f,-0.541699990126490060760033884434960782527923583984375f,1.0401999750316139881078925100155174732208251953125f,0.16240000436306001319053393672220408916473388671875f,0.421199966716766738272781367413699626922607421875f,-1.11479998816251768545271261245943605899810791015625f,0.28970005898475648109524627216160297393798828125f,0.934899980521202156324989118729718029499053955078125f,0.017499986588955351862750831060111522674560546875f,0.5949000082373618880637877737171947956085205078125f,-0.353199991363286613932359614409506320953369140625f,-0.7502000061750411230576673915493302047252655029296875f,0.3842000004827976766819119802676141262054443359375f,0.0451999923169612571882680640555918216705322265625f,-1.0042999810695647511238348670303821563720703125f,-0.6084999764859677640060908743180334568023681640625f,-0.824299973469972968587171635590493679046630859375f,-0.27679997153282176469701880705542862415313720703125f,-1.14339998993873592780801118351519107818603515625f,1.0236999755084514873715306748636066913604736328125f,-1.0773999703884129530706559307873249053955078125f,0.052399995124340126295692243729718029499053955078125f,0.919799976241588268521809368394315242767333984375f,0.390400019428133848720108289853669703006744384765625f,0.4507999985486268457179903634823858737945556640625f,-0.934299990457296392065700274542905390262603759765625f,0.446400010442733474747001309879124164581298828125f,0.89670002305805673881167194849695079028606414794921875f,-0.17020000982582583759494809783063828945159912109375f,0.10020003493428220053829136304557323455810546875f,1.5646999677896502589646843262016773223876953125f,-0.34809998171925560228601170820184051990509033203125f,0.084399997866153597669836017303168773651123046875f,-0.1802000110179189373837971288594417273998260498046875f,0.6698000295877459819848809274844825267791748046875f,0.081799985577166278716276792692951858043670654296875f,0.0657999914705751365318064927123486995697021484375f,-0.4165000012740491808216347635607235133647918701171875f,0.3085000055432320920090205618180334568023681640625f,0.630500012531876929955387822701595723628997802734375f,-0.043700006651878453567405813373625278472900390625f,0.4204000306412580822978952710400335490703582763671875f,-0.8056000158429146296157341566868126392364501953125f,-0.0044999869838356632012477120952098630368709564208984375f,-0.3987000065028667439293030838598497211933135986328125f,0.304500038616359436360880863503552973270416259765625f,-0.326100002074241501759388484060764312744140625f,0.32440002669990042338810098954127170145511627197265625f,-0.6663999976649879197765358185279183089733123779296875f,-0.3046000275850293537160951018449850380420684814453125f,1.002999993681907398013208876363933086395263671875f,-0.101000005282461557953865849412977695465087890625f,0.6878000089645386339043398038484156131744384765625f,-0.07009999217241980051795735562336631119251251220703125f,-0.36910001572370543243550855549983680248260498046875f,-0.271300001581013194851266234763897955417633056640625f,-0.088700008440017796829124563373625278472900390625f,-0.433600013399124240009996356093324720859527587890625f,0.2464000064194202810341494114254601299762725830078125f,-0.0920999874636529991533961947425268590450286865234375f,-0.1887000081419945018978978623636066913604736328125f,-0.418199997770785980577556983917020261287689208984375f,-0.5384999778270724846152006648480892181396484375f,-0.828200011181831285966836730949580669403076171875f,-0.0883999762296667768168845213949680328369140625f,0.65000000134110447191915227449499070644378662109375f,-1.35779998229146059429695014841854572296142578125f,-0.5266999800682068411106229177676141262054443359375f,-0.629599995994567507295869290828704833984375f,1.236600016617774855376410414464771747589111328125f,-0.92529998154640225038747303187847137451171875f,1.3645000119805335003775326185859739780426025390625f,-0.153900016999244826365611515939235687255859375f,0.09279999905526636894137482158839702606201171875f,0.6342999836772680311014482867904007434844970703125f,-0.0557999858081341670867914217524230480194091796875f,1.6522999884605411580196232534945011138916015625f,0.64149999585375194754988115164451301097869873046875f,0.888699962190539149986534539493732154369354248046875f,-1.38399997925758366790205400320701301097869873046875f,0.09140002534389424226901610381901264190673828125f,-0.86929998583793643973649523104541003704071044921875f,0.18050002139806764489549095742404460906982421875f,0.1408000054776668008571505197323858737945556640625f,0.28549998891353567387341172434389591217041015625f,-0.73030000389814375694186310283839702606201171875f,1.06719998835325213804026134312152862548828125f,-0.88540001421272751525037847386556677520275115966796875f,0.25140001439154158191513488418422639369964599609375f,-0.422999991446733503153154742904007434844970703125f,1.0892999819636344227546942420303821563720703125f,-0.00609997450709354183118193759582936763763427734375f,0.245700006413459792753428700962103903293609619140625f,0.83780001861304065347013647624407894909381866455078125f,0.15220000474452977101691431016661226749420166015625f,-0.5000000154972077393011886670137755572795867919921875f,0.269600001358985874588825026876293122768402099609375f,-0.5381000149860979409055516953230835497379302978515625f,0.17090000104010105719254397627082653343677520751953125f,0.50999998576939098438032260673935525119304656982421875f,0.44760000815391531858722373726777732372283935546875f,0.1912000166729093120210336564923636615276336669921875f,-0.367600003385543772083110525272786617279052734375f,0.520000025704503432422143305302597582340240478515625f,-0.15280000240802760202996068983338773250579833984375f,0.326400000309944093856984181911684572696685791015625f,-0.643200011777878000174268890987150371074676513671875f,0.0616000075653193590596856665797531604766845703125f,-0.0939999905824659975905888131819665431976318359375f,-0.47200000527501106972749767010100185871124267578125f,0.1407999943017959054714083322323858737945556640625f,-0.157799995367228884646237929700873792171478271484375f,-0.551400016030669348765513859689235687255859375f,-0.6644000050932168388584386775619350373744964599609375f,-0.206800015044212415205038269050419330596923828125f,-0.7783000134870408093235027990886010229587554931640625f,-0.38810002448558833521019550971686840057373046875f,-0.56779999309480189140231232158839702606201171875f,0.0782000016450883350671574589796364307403564453125f,0.17690001287907375360219930371385999023914337158203125f,-0.2129000037074089135558097041212022304534912109375f,0.31620000028163186556184882647357881069183349609375f,0.1019999939501285979304157081060111522674560546875f,1.08800001767277709774361937888897955417633056640625f,-0.448599982106685502003529109060764312744140625f,-1.1103000077724456406969011368346400558948516845703125f,0.5189000025451182551705642254091799259185791015625f,-0.686500000454485448386776624829508364200592041015625f,-0.313400002418458445418991686892695724964141845703125f,0.27609998344779018708550211158581078052520751953125f,-0.1758999660968783018688554875552654266357421875f,0.58290000758171078842906354111619293689727783203125f,0.01609998449087157723624841310083866119384765625f,0.976400019383430350927710605901665985584259033203125f,-0.417199985969066489133183495141565799713134765625f,0.7074999678134918923433360760100185871124267578125f,0.698299990278481885752626112662255764007568359375f,-0.36280002192854876597039037733338773250579833984375f,1.17320000581741368250732193700969219207763671875f,-1.15799999368190764670316639239899814128875732421875f,0.97639998540878281119148596189916133880615234375f,0.23689999667406080874343388131819665431976318359375f,0.2226999798744913050541072152554988861083984375f,-0.35419997231960298478270487976260483264923095703125f,-0.58740004310607929483012412674725055694580078125f,-0.50779999719262125523755457834340631961822509765625f,-0.17050002288818433271444519050419330596923828125f,-1.08959998437166216689320208388380706310272216796875f,0.69699998307228128169299452565610408782958984375f,0.208099998557567555934610936674289405345916748046875f,0.477000003337860079000165569595992565155029296875f,-0.2255000401735305359807171043939888477325439453125f,-0.117900003949552800719402512186206877231597900390625f,-0.10759999284297239174890137292095459997653961181640625f,-0.04719999699592580810758590814657509326934814453125f,-0.69169999254792902121380393509753048419952392578125f,-0.136300015699863497076194107648916542530059814453125f,-0.1918999835982919677235258859582245349884033203125f,-0.693699998009204676918670884333550930023193359375f,-0.178800030787289454536903576808981597423553466796875f,0.6000000071525573019926014239899814128875732421875f,0.2691999925851822350608699707663618028163909912109375f,0.36440000367760649879755874280817806720733642578125f,-0.44390001565813985706654420937411487102508544921875f,0.2458999947071074387849876075051724910736083984375f,0.0979000018075109057491545172524638473987579345703125f,-0.16269999485015862461523283855058252811431884765625f,-0.5060000306144358805937599754543043673038482666015625f,0.53840000811815258430215180851519107818603515625f,0.3559000308796769207475563234766013920307159423828125f,-0.48550002110004442101853783242404460906982421875f,-0.4309999982416627961612221042742021381855010986328125f,-0.123400012588501084564995835535228252410888671875f,-0.39209999867677691742784418238443322479724884033203125f,-0.40350000911951067195104769780300557613372802734375f,0.4694000281155112919151406458695419132709503173828125f,-0.26060000392198556795619879267178475856781005859375f,-0.305500027358532211252395427436567842960357666015625f,0.69790000966787335556773541611619293689727783203125f,0.08239999352246485386785934679210186004638671875f,0.46120002185106301340056234039366245269775390625f,0.50319999922364910815986149827949702739715576171875f,-0.0550000103376806492860850994475185871124267578125f,-0.42899999788403508915735073969699442386627197265625f,0.75779998482465771303395740687847137451171875f,-0.9829999769926072161752017564140260219573974609375f,0.580899982637166800714112468995153903961181640625f,0.5053999892532825555235831416212022304534912109375f,-0.1613000180840487018940621055662631988525390625f,-0.70320000656247128034692650544457137584686279296875f,-0.21239999840259571328715537674725055694580078125f,-0.227400012707710263981653042719699442386627197265625f,0.460200030761956924152400461025536060333251953125f,-0.16739999393224724855144813773222267627716064453125f,-1.36689997014999420343883684836328029632568359375f,0.649099986815452556498939884477294981479644775390625f,-0.823399978613853278375245281495153903961181640625f,-1.13050001811981193355904906638897955417633056640625f,-0.02729998130798350075565394945442676544189453125f,0.43400001189112646926560046267695724964141845703125f,1.04790000698566387882237904705107212066650390625f,-0.80039999566078190440521211712621152400970458984375f,0.86059999170303314031116315163671970367431640625f,-0.10029999749064444358737091533839702606201171875f,0.01399997180700296439681551419198513031005859375f,0.989100021237134807705615457962267100811004638671875f,-0.353099994981288745066194678656756877899169921875f,-1.222500009089708239429228342487476766109466552734375f,0.383200010883808062089883605949580669403076171875f,-0.223499999284744177430184208787977695465087890625f,-0.265500000834464344734442420303821563720703125f,0.9643999723851681693531645578332245349884033203125f,-0.54580000570118425429200215148739516735076904296875f,-0.3360000272244219043926705126068554818630218505859375f,0.4152000210762025744770653545856475830078125f,0.3193000053212049493822632939554750919342041015625f,0.14149999542534341667732178393634967505931854248046875f,0.7234999989494677041790282601141370832920074462890625f,0.356500013828277673155753291212022304534912109375f,-0.0061000000067055193397891343920491635799407958984375f,0.214900006877631000179462716914713382720947265625f,0.5124000017181040700364746953709982335567474365234375f,0.57840000722408291267129243351519107818603515625f,0.74189999190568922671218388131819665431976318359375f,0.3489000119328498072945876629091799259185791015625f,-0.2577999952927230165045102694421075284481048583984375f,0.29740000674724598184184287674725055694580078125f,0.599200002978741874670731704100035130977630615234375f,-0.0627999904870986114246989018283784389495849609375f,0.17700001384317876063079211235162802040576934814453125f,-0.0906000030279159318524762056767940521240234375f,0.00029999525547019789684100032900460064411163330078125f,0.168900003290176403680789007921703159809112548828125f,0.4000999961957336648055161276715807616710662841796875f,0.4999000094294547835716002737171947956085205078125f,0.16190001325160263689895145944319665431976318359375f,-0.2554999966621398499455608543939888477325439453125f,-0.391100003825127995327193275443278253078460693359375f,0.0454999957680701783147014793939888477325439453125f,-0.2539000339865686850515658079530112445354461669921875f,0.5887000106751918426795100458548404276371002197265625f,0.06909999694824187077557553493534214794635772705078125f,-0.615100018808245607715434744022786617279052734375f,-0.12820001475810993696313744294457137584686279296875f,-1.52099998795986124378032400272786617279052734375f,0.32940000818520775993647475843317806720733642578125f,-0.16049997702986029679550483706407248973846435546875f,0.42220000608563434951747694867663085460662841796875f,0.27809998037815120142113300971686840057373046875f,-0.4839000099956989320304501234204508364200592041015625f,0.596400020147115128565928898751735687255859375f,-0.15329999996423726571492807124741375446319580078125f,0.50270004223585118552364292554557323455810546875f,0.36389995933175089959377146442420780658721923828125f,0.47380006248950934377717203460633754730224609375f,0.16580004034638395893352935672737658023834228515625f,-1.70720000414848271930168266408145427703857421875f,1.03979999457001692775293122394941747188568115234375f,-0.53380000526905035940217203460633754730224609375f,-0.012299985927343382030585416941903531551361083984375f,0.704299980175495221601522644050419330596923828125f,-0.1651999991714954152399741360568441450595855712890625f,0.0058999999225139987402144470252096652984619140625f,-0.100500012159347562601396930404007434844970703125f,0.2468000478267668995613348670303821563720703125f,-0.33120003153681754071158138685859739780426025390625f,1.72230001885890970925174769945442676544189453125f,-0.76420000227093698441649394226260483264923095703125f,0.52830000294446932684877538122236728668212890625f,-0.5156000150233506218455659109167754650115966796875f,-0.8090999807804823973356178612448275089263916015625f,0.178099971586465954942468670196831226348876953125f,0.274400004124641494485103976330719888210296630859375f,-0.0946999944254753867056706440052948892116546630859375f,-0.5245000192821027695799784851260483264923095703125f,0.523100015619397229471587706939317286014556884765625f,-0.346700000929832441443068091757595539093017578125f,-0.312100001060962639609641655624727718532085418701171875f,-0.080300001215934724285716583835892379283905029296875f,0.51869998009055839904846152421669103205204010009765625f,0.21570000402927380633855136693455278873443603515625f,-0.1785000213384633216406882638693787157535552978515625f,0.477300025117397552776310476474463939666748046875f,-0.130000021494924900000711431857780553400516510009765625f,0.450400001323222998195205946103669703006744384765625f,0.29660000298321220757458149819285608828067779541015625f,0.09220001338720340555710208718664944171905517578125f,0.0931000058963895593677762008155696094036102294921875f,-0.20830000033676621118416960598551668226718902587890625f,-0.7707000248908997264152276329696178436279296875f,-0.23370001529455208810759359039366245269775390625f,-0.0003999984174967163852443263749592006206512451171875f,0.2560000061020255568422498981817625463008880615234375f,0.2927999957770108796495378555846400558948516845703125f,0.0456000067532063013686638441868126392364501953125f,0.3035000028610228550718375117867253720760345458984375f,-0.105499993979930817289414335391484200954437255859375f,-0.45100002577155873240144501323811709880828857421875f,0.4546000205814839656426329383975826203823089599609375f,-0.00430000800341379108004957743105478584766387939453125f,0.63790002009868640442391551914624869823455810546875f,-0.5004999883919956094047165606752969324588775634765625f,0.2975000010430814967321566655300557613372802734375f,0.3480000015795230439152874168939888477325439453125f,0.0912999842584127918598824180662631988525390625f,1.0505999964714050065595074556767940521240234375f,-0.158699997711181595150264911353588104248046875f,-0.218199999111890807768077138462103903293609619140625f,-0.19710000210404388099050265736877918243408203125f,0.41699996265769012637747437111102044582366943359375f,0.0313000377535814777729683555662631988525390625f,-0.19490001077055918443647897220216691493988037109375f,-1.07579999549388816149075864814221858978271484375f,-0.19420000569820405900145487976260483264923095703125f,-0.887399965471029616992382216267287731170654296875f,0.65080000979900365365438119624741375446319580078125f,-0.8294000142574304845766164362430572509765625f,0.567999999381601838877031696029007434844970703125f,-0.559299996156990442131018426152877509593963623046875f,-0.912900023525953230318918940611183643341064453125f,0.417999989509581837410223670303821563720703125f,-0.70030000561177729867523567008902318775653839111328125f,1.00370001123398555620269689825363457202911376953125f,0.976100000435113912544693448580801486968994140625f,-1.461099988812208039234974421560764312744140625f,0.6398999906539917237324743837234564125537872314453125f,-0.3637999940931795350707034231163561344146728515625f,-1.381500042960047469620121773914434015750885009765625f,0.85830002581775222125770596903748810291290283203125f,-0.45190000025033949526687138131819665431976318359375f,-0.3737999718904490009663277305662631988525390625f,0.1012000021815300243588353623636066913604736328125f,-0.2377000023007393991747449035756289958953857421875f,0.11479998760372411847896501058130525052547454833984375f,0.47920000040829169307698975899256765842437744140625f,0.135399995176494147841594894998706877231597900390625f,-0.05260000562071809326880611479282379150390625f,-0.10769998743683084629907398266368545591831207275390625f,-0.35310001241564759055791000719182193279266357421875f,0.40350001187622552834000089205801486968994140625f,-0.3135000039041042629861522073042578995227813720703125f,0.4049000064864751724513780573033727705478668212890625f,-0.66410000737905505729941069148480892181396484375f,0.6913000010594727573476347970427013933658599853515625f,-0.136900007104873822072477196343243122100830078125f,-0.2261999844864013287093484905199147760868072509765625f,-0.7061999936133620536793387145735323429107666015625f,-0.2638000088825822597726755702751688659191131591796875f,0.317799997341632778358189170830883085727691650390625f,-0.06990000238865612391947479409282095730304718017578125f,0.156300003182888058717026069643907248973846435546875f,0.19689998393356822337096900810138322412967681884765625f,0.31429999109059547635069975513033568859100341796875f,0.13940000039935096509680079179815948009490966796875f,0.42580002026706953444090686389245092868804931640625f,0.14600000662356638070349390545743517577648162841796875f,-0.45200001853704474541473246063105762004852294921875f,0.10820000700950649363818456549779511988162994384765625f,-0.61940001723766346941602023434825241565704345703125f,0.41570001308172954690434153235401026904582977294921875f,-0.1204999987483024170842327293939888477325439453125f,-0.12639999762773523883652160293422639369964599609375f,1.098399995899200121129979379475116729736328125f,-0.33170000122785570084715800476260483264923095703125f,0.02430002748668158574218978174030780792236328125f,0.592500008642673492431640625f,-0.8309000082671644804577226750552654266357421875f,-0.3183999999225139987402144470252096652984619140625f,1.16629998694062209096955484710633754730224609375f,0.7009999966770410484429021380492486059665679931640625f,0.218499994739890013306649052537977695465087890625f,1.35850000166893014608149314881302416324615478515625f,-0.48540005109310158104563015513122081756591796875f,1.55140000224709506682074788841418921947479248046875f,-1.25769998471736954570587840862572193145751953125f,0.50469998894929890553839868516661226749420166015625f,-0.22670002857148574548773467540740966796875f,-0.9982999911725520814087531107361428439617156982421875f,0.8126999869525430852945646620355546474456787109375f,-0.2057999990701675496751477112411521375179290771484375f,-0.0688999921143054194772048504091799259185791015625f,0.356500006973743399640852658194489777088165283203125f,-0.92419997936040232389132142998278141021728515625f,-0.52860001652836796637302541057579219341278076171875f,0.642000030718744341129422537051141262054443359375f,0.4504999969601630738225139793939888477325439453125f,0.36649999386072096285715815611183643341064453125f,0.677599983417987772327251150272786617279052734375f,0.11780000747442187503111199475824832916259765625f,0.849300006401538798428418886032886803150177001953125f,-0.489399991607665896253820392303168773651123046875f,-0.512299999934434868720245503936894237995147705078125f,0.127300006639957441478827604441903531551361083984375f,0.218399996271729346375423119752667844295501708984375f,0.1452000187292696864460594952106475830078125f,-0.228000000759958965090845595113933086395263671875f,-0.4632000273495915365629116422496736049652099609375f,0.2268000212654470715278876014053821563720703125f,-0.194399997270107149915929767303168773651123046875f,-0.38699999853223532308987842043279670178890228271484375f,0.17459999003410331397390109486877918243408203125f,-0.1204000071719288911253897822462022304534912109375f,0.17440000121891496842607693906757049262523651123046875f,-0.628900004594028505522373961866833269596099853515625f,-0.487099993908405082265744567848742008209228515625f,0.3937000148847700220500200884998776018619537353515625f,0.13910000141859057976034819148480892181396484375f,-0.3455000434517863094896483744378201663494110107421875f,0.6659000097572802534529046170064248144626617431640625f,-0.6949000162467360297569030080921947956085205078125f,0.2412999879837034455931643606163561344146728515625f,0.4267999947413800843065700973966158926486968994140625f,0.29679999924898137209083870402537286281585693359375f,0.5545999924182893892066203989088535308837890625f,0.3165999981403349039510430884547531604766845703125f,-0.3741999906107782436492925626225769519805908203125f,-0.11900000198185434197029053393634967505931854248046875f,-0.20040001231282944527123390798806212842464447021484375f,0.74400000846385960784346025320701301097869873046875f,-0.54629998012334091139763359024072997272014617919921875f,-0.20970000414848311010018733213655650615692138671875f,-0.5336000037133690199198099435307085514068603515625f,0.2944999966323376128229938331060111522674560546875f,0.576099980020523094736972780083306133747100830078125f,-0.990699965621530509451986290514469146728515625f,0.342600019776821085315532400272786617279052734375f,-0.39180003344714720014962949790060520172119140625f,-0.1256999974727630586812665569595992565155029296875f,1.3134999837875369621542631648480892181396484375f,-0.753399977570772261259435254032723605632781982421875f,0.239599998974799888173947692848742008209228515625f,-0.25160000033825635767925632535479962825775146484375f,-0.14399999300390486922651689383201301097869873046875f,-0.069700004446506369504277245141565799713134765625f,-0.52180005438327725642011500895023345947265625f,0.08710001477003093128814725787378847599029541015625f,0.58480000097751627663456019945442676544189453125f,-0.66210002155005920343455727561376988887786865234375f,-0.8995999762505295649361869436688721179962158203125f,-1.48939999056458471926589481881819665431976318359375f,0.88729998130798293232146534137427806854248046875f,0.059800008577108432206159704946912825107574462890625f,0.398599988961219509064903832040727138519287109375f,0.60319999568462367989241101895458996295928955078125f,-0.49899996316433004039936349727213382720947265625f,-0.40859998467713598557793375221081078052520751953125f,0.13799998358637122208847358706407248973846435546875f,-0.717999978333711652567217242904007434844970703125f,-0.54639998570680603506843908689916133880615234375f,-0.57239996904730805482586220023222267627716064453125f,0.2248999712824826247015153057873249053955078125f,-0.4139999913275242970911449447157792747020721435546875f,0.49570002340078378466614594799466431140899658203125f,-0.2884000258132817240408485304214991629123687744140625f,0.3856000048160552751141949556767940521240234375f,-0.320400023972987835207959506078623235225677490234375f,0.3347999938249586193705908954143524169921875f,-0.000300010305643194641334048355929553508758544921875f,0.5943000198125840682905618450604379177093505859375f,0.226100003452599118691779267464880831539630889892578125f,-0.082100003147125288904817352886311709880828857421875f,-0.2720000140294434043397586719947867095470428466796875f,0.13880000079870224993783267564140260219573974609375f,-0.302300007124245428968833948601968586444854736328125f,-0.3672999944210051381787707214243710041046142578125f,0.6948000028029082120184511950355954468250274658203125f,-0.064799994122981985356091172434389591217041015625f,-0.37060001837611233899139051573001779615879058837890625f,0.06820000559389594041448390271398238837718963623046875f,0.00860000509172664795443097318639047443866729736328125f,-0.09859999454915513439345886581577360630035400390625f,0.1688000123098494231754784777876920998096466064453125f,-0.6620000075101852843317828956060111522674560546875f,-0.140800015088915930761004347004927694797515869140625f,-0.039200002121925425768722561770118772983551025390625f,-0.55169998852461510363553998104180209338665008544921875f,-0.14670002655982994355099435779266059398651123046875f,0.7675000271573664267776848646462894976139068603515625f,0.0173000075340272729818025254644453525543212890625f,0.190000009164214223034150563762523233890533447265625f,-0.139600004637241426053151371888816356658935546875f,0.1826000313252211793013657370465807616710662841796875f,-1.21579998863935401232083677314221858978271484375f,-0.15379999588131898491383253713138401508331298828125f,-1.27100000226497655830826261080801486968994140625f,0.0386000023722647966906151850707828998565673828125f,-1.6761999655246739848735160194337368011474609375f,-0.60720001606941220728685948415659368038177490234375f,0.07450004029274026606799452565610408782958984375f,0.472500003427267056821392543497495353221893310546875f,-0.25419999958872807610532618127763271331787109375f,0.035100013518333472717358745285309851169586181640625f,-1.117300018560886343266247422434389591217041015625f,0.32269997994899757287612374057061970233917236328125f,-0.75560000988244979680530377663671970367431640625f,1.36010000576972966968014588928781449794769287109375f,-0.97830001337528216254213475622236728668212890625f,-0.517400015650689493185154788079671561717987060546875f,-0.01120001175552598482454413897357881069183349609375f,-0.05110000624656674261814259807579219341278076171875f,0.34749999426305322458574664779007434844970703125f,0.9215999798119067776269730529747903347015380859375f,-0.166399990177154677439830265939235687255859375f,0.2366999937772751305686824707663618028163909912109375f,-0.8253000043451788059201135183684527873992919921875f,-0.92859997569918650839326801360584795475006103515625f,0.8649000028729432187901693396270275115966796875f,1.2678000296771525512440348393283784389495849609375f,-0.750699999856948352316976524889469146728515625f,-0.108300008457899110680955345742404460906982421875f,1.0573000051498411266948096454143524169921875f,0.8218000061035155567878973670303821563720703125f,0.249199999402463234776661238356609828770160675048828125f,0.41830002429038326550170268092188052833080291748046875f,-0.2095000069141388365778766456060111522674560546875f,-0.07100000040233123144162163953296840190887451171875f,0.09979999710321418859848563442938029766082763671875f,-0.05059999576359974771122551828739233314990997314453125f,-0.16900001099705708185183539171703159809112548828125f,-0.53970001066774120346281051752157509326934814453125f,-0.605100022459030384425204829312860965728759765625f,-0.023799998228251961762680366518907248973846435546875f,0.0706000031769276059634421471855603158473968505859375f,0.317900011940300597501618540263734757900238037109375f,-0.353900005674362194696414007921703159809112548828125f,-0.4157999960526821414674714105785824358463287353515625f,0.66420000718832028496763086877763271331787109375f,0.04149999773502344169173738919198513031005859375f,0.325700011070072736885094855097122490406036376953125f,0.4507000040292739040381775339483283460140228271484375f,-0.13749999463558193468770696199499070644378662109375f,0.192700028824806413041414998588152229785919189453125f,-0.35470001189708710853665252216160297393798828125f,-0.305000007152557373046875f,-0.0333999954521655428862914050114341080188751220703125f,-0.3766000138983132128345232558785937726497650146484375f,0.452200011897087250645199674181640148162841796875f,-0.5119000015914441092945708078332245349884033203125f,0.169899998104572347301655099727213382720947265625f,0.02459999804347756668931879175943322479724884033203125f,0.1554000010624527750024981287424452602863311767578125f,-0.1949000021278859406947958632372319698333740234375f,-0.114600013279915202701886300928890705108642578125f,-0.276600014121830550806180326617322862148284912109375f,0.0165000008642675055625659297220408916473388671875f,-0.89219998224377616846680894013843499124050140380859375f,-0.4574999816343190417455843999050557613372802734375f,-0.73230000291466712525334514793939888477325439453125f,-1.0092000006318091465118413907475769519805908203125f,0.657300011520087767991071814321912825107574462890625f,0.652799999725818480555972200818359851837158203125f,0.5438999954670666880929275066591799259185791015625f,0.6053999980449678020022474811412394046783447265625f,0.8721000342160465379492961801588535308837890625f,0.35189999633878432661049373564310371875762939453125f,-0.84570002142637978881367644135025329887866973876953125f,-0.78360000088214842861589204403571784496307373046875f,0.17460000359416005810686556287691928446292877197265625f,-0.4949999924004078177830479035037569701671600341796875f,0.41130001011192807158778350640204735100269317626953125f,0.3956999982923268266432614836958236992359161376953125f,0.306600013451278297083035795367322862148284912109375f,-0.0194999979734422623778300476260483264923095703125f,-0.7523999776527283689375735775684006512165069580078125f,-0.4730999787390235411521643982268869876861572265625f,-0.4532999959409236279128663227311335504055023193359375f,0.63210002043247204284170948085375130176544189453125f,-0.184500008292496187056741518972557969391345977783203125f,-0.531900018578767674881646598805673420429229736328125f,-0.3963000110059977654231033739051781594753265380859375f,0.2184000149726870176891679875552654266357421875f,-0.69299997874349339443966755425208248198032379150390625f,0.38139999151825865197906750836409628391265869140625f,-0.24220000280737874476244542165659368038177490234375f,-0.11340000379681580700008680651080794632434844970703125f,-0.670099988409877056483310298062860965728759765625f,-0.55330000599920714510204788894043304026126861572265625f,0.671299985599517867740360088646411895751953125f,0.0588999798953526720879381173290312290191650390625f,0.64170000741183752523966177250258624553680419921875f,0.54410002712309391004197323127300478518009185791015625f,-0.1310000374317166205173634807579219341278076171875f,0.7657999913215629561591413221322000026702880859375f,0.570999988257884893982918583787977695465087890625f,0.145800005030631840696742074214853346347808837890625f,0.09869999830722786526848722132854163646697998046875f,0.29490001926422149125528449076227843761444091796875f,0.3167999961197376368460254525416530668735504150390625f,0.2100000053644179853762352649937383830547332763671875f,-1.36529997200965880210787872783839702606201171875f,-0.88870001812577203992304930579848587512969970703125f,-0.1678999851554632272154776728712022304534912109375f,-0.18269999790489654944991571028367616236209869384765625f,-0.019000000417232598692862666212022304534912109375f,0.44300001692771928674119408242404460906982421875f,-0.529700016516446936520878807641565799713134765625f,-0.5509000335991387498069116190890781581401824951171875f,-0.7361999926447868602963353623636066913604736328125f,-0.520800007861852432000659973709844052791595458984375f,0.9454999756515027886649704669252969324588775634765625f,0.9427000174999233816919286255142651498317718505859375f,0.5819000094890593999252814683131873607635498046875f,-0.19349998795986156352455509477294981479644775390625f,0.1091999999240040875037038858863525092601776123046875f,0.0714000020235776577948172416654415428638458251953125f,-0.3164999895393847406666054666857235133647918701171875f,-0.1457999855846168202333501540124416351318359375f,0.365099985913932112424618026125244796276092529296875f,0.94140001550912888461652983096428215503692626953125f,0.36330000935196882227984360724803991615772247314453125f,0.18659999448955044698550409520976245403289794921875f,0.9569999864995477611984142640721984207630157470703125f,-0.1800000131130214953145696199499070644378662109375f,-0.076199998605251284544692680356092751026153564453125f,0.218999988496303643614737666212022304534912109375f,0.33210001599937688521180234602070413529872894287109375f,0.026099997603893143605091609060764312744140625f,0.1896000000178813815754352845033281482756137847900390625f,0.1913999969571829051773192986729554831981658935546875f,-0.680099999660253473621196462772786617279052734375f,-0.6938999806404115844316038419492542743682861328125f,-0.413099998632073361903849217924289405345916748046875f,-0.34919999936521062267047454952262341976165771484375f,0.4170000101923942992243610206060111522674560546875f,-0.20700000274181373782766968361102044582366943359375f,-0.269700015100836854031740585924126207828521728515625f,0.03240000331997894278401872725225985050201416015625f,-0.5669999978989361810732816593372263014316558837890625f,-0.5391000017166138746915748924948275089263916015625f,0.87029999562799920109767981557524763047695159912109375f,0.4439999803751708551402543889707885682582855224609375f,0.5727000131785870973288865570793859660625457763671875f,-0.41400001479685311078782206095638684928417205810546875f,0.7710999701857570443053191411308944225311279296875f,0.610499989956616584407811387791298329830169677734375f,-0.29769997592568397948298297706060111522674560546875f,0.3527000081121922647753308410756289958953857421875f,0.05360000088214889046867028810083866119384765625f,-0.3108000252962117571087219403125345706939697265625f,0.76499999344348879048993694595992565155029296875f,1.044400038099289051984897014335729181766510009765625f,-0.4868999956309796317555083078332245349884033203125f,0.10930001012682932781672207056544721126556396484375f,0.7928000097095964093796283123083412647247314453125f,0.5216000255584718470203142715035937726497650146484375f,-0.5495999806463720549487561584101058542728424072265625f,-0.6596000058293338685899698248249478638172149658203125f,0.2993999677658081282061175443232059478759765625f,-0.019600014770031481958767471951432526111602783203125f,-0.725799991992115867134316431474871933460235595703125f,-0.65120002326667314473951364561798982322216033935546875f,-1.0497999786257745657991335974656976759433746337890625f,-0.322600000554322807744256351725198328495025634765625f,-1.116800011467933728681600769050419330596923828125f,-0.52920001955628404033404876827262341976165771484375f,-0.1810999889612201485533660161308944225311279296875f,-0.9145000201761721569937435560859739780426025390625f,0.10910001512765887810019194148480892181396484375f,-0.33809999915361377276212806464172899723052978515625f,1.0591000069320199994393760789535008370876312255859375f,0.64830002928227214997747296365560032427310943603515625f,-0.063199984806775688639390864409506320953369140625f,0.0837000095576048153134252061136066913604736328125f,0.7850999786496160037785330132464878261089324951171875f,0.684299991500378013142835698090493679046630859375f,0.20790000333487979045088422935805283486843109130859375f,-0.564599982583522841395051727886311709880828857421875f,-0.38280001753270642694815251161344349384307861328125f,0.079800009322166687297794851474463939666748046875f,0.72840002052336938476173600065521895885467529296875f,-0.0345000098943710753474078956060111522674560546875f,-0.16829999046474686252850005985237658023834228515625f,-0.800700007903575983192467901972122490406036376953125f,0.1734000127002597313463638784014619886875152587890625f,0.14340000022053711603575720801018178462982177734375f,-0.59639999144375310802246303865103982388973236083984375f,-0.1937999861210586427517910124151967465877532958984375f,0.24239999240487808407351622008718550205230712890625f,0.57869999175071740182829671539366245269775390625f,-0.553200009356439093011204022332094609737396240234375f,-0.318899991592764830983242063666693866252899169921875f,-0.30899998819828022877942430568509735167026519775390625f,-0.8303999853044750079078539783949963748455047607421875f,-0.34739999843984836758181700133718550205230712890625f,-0.62640000150203700712836507591418921947479248046875f,-0.6338999941259622072919910351629368960857391357421875f,0.4827000258445739433454946265555918216705322265625f,-0.442500001341104454155583880492486059665679931640625f,0.10380000079870210782928552362136542797088623046875f,0.6405000110417604641810385146527551114559173583984375f,0.7278000033207236985077770441421307623386383056640625f,-0.346200014810264189435429216246120631694793701171875f,-0.7836000411897903195068693094071932137012481689453125f,0.28680000304877761863053819979541003704071044921875f,0.359000009283423349160102588939480483531951904296875f,0.1673000027656552646249110694043338298797607421875f,0.78710003399252936873153885244391858577728271484375f,0.4119999738335611283446269226260483264923095703125f,0.64180000759363142748270547599531710147857666015625f,0.225800004434585588342088158242404460906982421875f,0.6948000243723395641382012399844825267791748046875f,-1.0830999934911726967357026296667754650115966796875f,-0.81590003643035924341120335157029330730438232421875f,0.0978000036001203110203050528070889413356781005859375f,-0.666999997228383545433416657033376395702362060546875f,-0.5903000052392481933338785893283784389495849609375f,-0.37310001211762422457240973017178475856781005859375f,1.1950999986171719768890397972427308559417724609375f,0.708300028574466722375291283242404460906982421875f,0.448999970316886987120597041212022304534912109375f,1.2092000189602369886188171221874654293060302734375f,-0.411299987424909918587445645243860781192779541015625f,-0.1733000001505013987213033033185638487339019775390625f,1.4558999845743176848600342054851353168487548828125f,0.75190004256963760553844622336328029632568359375f,0.883700013655424054803688704851083457469940185546875f,0.07330000726580632175455320975743234157562255859375f,-0.523799986568093345340457744896411895751953125f,0.103799996104836367294410592876374721527099609375f,-1.1006999953866003494340475299395620822906494140625f,-0.24110001861453067562024443759582936763763427734375f,-1.3998999878227709192657357561984099447727203369140625f,0.32190001928657319130167024923139251768589019775390625f,0.080999999620020357582461656420491635799407958984375f,-0.1821000044882297164150486423750407993793487548828125f,0.9405000228881836843441988094127736985683441162109375f,0.0429000040054319242699421010911464691162109375f,0.6785999948099254996947138351970352232456207275390625f,0.187499981224537037860500277020037174224853515625f,0.882600009047985079035925082280300557613372802734375f,0.16890000299215301993172033689916133880615234375f,-0.9003000137329102425809423948521725833415985107421875f,-0.7937999900698660127318362356163561344146728515625f,-0.61259998937845205890795341474586166441440582275390625f,0.3923999950498340272275754614383913576602935791015625f,-0.0963000031828880054263208876363933086395263671875f,-0.092400000786781344430664830724708735942840576171875f,0.3531000131607056058413718346855603158473968505859375f,0.4367999950766563443949053180404007434844970703125f,0.923400000965595157964571626507677137851715087890625f,-0.3495000227093694178392979665659368038177490234375f,0.035699998068809524198741200962103903293609619140625f,-0.07739999504983441358518803099286742508411407470703125f,0.4479000189810993237671254973975010216236114501953125f,0.7181999727368353347856100299395620822906494140625f,0.134100000151991805008577784974477253854274749755859375f,0.852600013965368219714946462772786617279052734375f,-0.01319999885112055348912463159649632871150970458984375f,-0.6503999853789805829507031376124359667301177978515625f,-0.0477000068083406336683083281968720257282257080078125f,-0.702599989378452249866313650272786617279052734375f,-0.7233000180691482317030249760136939585208892822265625f,1.04760004361867942179742385633289813995361328125f,-0.077399989759921794529873295687139034271240234375f,-0.61080003274679217639686612528748810291290283203125f,-0.62779997469186810121755115687847137451171875f,-0.2111999872803682620769905042834579944610595703125f,-1.384300001931190404746985223027877509593963623046875f,-0.6827000249505044138231824035756289958953857421875f,-0.16360001071691510077243947307579219341278076171875f,0.071999999016523208439366499078460037708282470703125f,-0.830500002659857017306421766988933086395263671875f,-0.8027000335559251187333984489669091999530792236328125f,0.88059999364018448630986313219182193279266357421875f,0.7952000255465507816410308805643580853939056396484375f,-0.200899983233213730304811406313092447817325592041015625f,-0.75549999736994444266002801668946631252765655517578125f,0.354100016319751631499457289464771747589111328125f,1.1703000402569774252015122328884899616241455078125f,-0.54659998009130372764730054768733680248260498046875f,-0.52119999981597009419687083209282718598842620849609375f,0.971099988961219739991292954073287546634674072265625f,0.19230001394152651528202113695442676544189453125f,-0.622399991846084077451450866647064685821533203125f,-1.23300005671381995853153057396411895751953125f,-0.06510000347867617165320552885532379150390625f,-0.1315000126920644607508847911958582699298858642578125f,-0.37440001448094850644565667607821524143218994140625f,0.35000000633299332974956996622495353221893310546875f,-0.6868000025272369413187334430404007434844970703125f,0.1614000006079674420789160649292171001434326171875f,-0.721500002652406546843621981679461896419525146484375f,-0.12210000910758989078885861090384423732757568359375f,-0.356400021730363558702947557321749627590179443359375f,0.4383000214219094914369634352624416351318359375f,-0.472799982328712786738833528943359851837158203125f,-0.7958999932169916746715898625552654266357421875f,-0.7383000057756901046701614177436567842960357666015625f,-1.085400004750490143834440459613688290119171142578125f,-0.07860000084489582317104350295267067849636077880859375f,-0.3468000120639800343269598670303821563720703125f,-0.45990001710355288278009311397909186780452728271484375f,-0.0824999949708580260221424396149814128875732421875f,0.60420000249445438011974829350947402417659759521484375f,0.30329998551011083662842793273739516735076904296875f,-0.71189998572170709589812531703501008450984954833984375f,0.138300005831569150505089282887638546526432037353515625f,0.979500017233192910026673416723497211933135986328125f,-0.314699994015693818028012174181640148162841796875f,-0.332999990757554653608707440071157179772853851318359375f,-0.09929998745843793273024857626296579837799072265625f,0.237300003249943325389637038824730552732944488525390625f,0.871799988669156977749707948532886803150177001953125f,-0.858000037863850817387856295681558549404144287109375f,0.3003000170111658206906213308684527873992919921875f,-0.352500003408640683044694696945953182876110076904296875f,-0.26279999683871846372085201437585055828094482421875f,0.0780000059008598700671655024052597582340240478515625f,-0.7782000158011912471778259714483283460140228271484375f,0.3810000047609209961052556536742486059665679931640625f,-0.43889999583959582452052927692420780658721923828125f,-0.740100017206370974776064031175337731838226318359375f,0.167199997852742476833753926257486455142498016357421875f,0.125800010395050065881150658242404460906982421875f,0.62660003971457545191015015006996691226959228515625f,-0.893999992966652001769034541212022304534912109375f,-0.18960001178979890568143673590384423732757568359375f,-0.02240000577867018449751412845216691493988037109375f,0.398800019946694561667754896916449069976806640625f,-0.64459998586177835733224128489382565021514892578125f,-0.7678000220775604844902773038484156131744384765625f,-0.6787000027775764721127416123636066913604736328125f,-1.2073000495553021238492874545045197010040283203125f,0.3642999822989108071880082206916995346546173095703125f,0.433900000272690877789472096992540173232555389404296875f,-0.3598999711334709417087651672773063182830810546875f,-0.4308999948561187398610172749613411724567413330078125f,-0.973999984025955001243346487171947956085205078125f,-0.29840001631379120539122595801018178462982177734375f,0.4671999959528445600653867586515843868255615234375f,-0.084399997866153597669836017303168773651123046875f,-0.6939999804496768120998240192420780658721923828125f,-1.13740002924799910033470951020717620849609375f,0.86079998446702976622191272326745092868804931640625f,0.4364000032901760306458527338691055774688720703125f,-0.2484999994337560025314814993180334568023681640625f,0.09609999074935871021807543002068996429443359375f,-0.13519999663829818103977231658063828945159912109375f,-0.798000017523765592386553180404007434844970703125f,0.343800029706954557440212738583795726299285888671875f,0.98840005207657899433115744614042341709136962890625f,0.795899989938735785699464031495153903961181640625f,0.07169999485760936153155853389762341976165771484375f,0.25320001539140957191875713760964572429656982421875f,-0.53460002162456543572943701292388141155242919921875f,-0.418799998712539700562729194643907248973846435546875f,-1.010999993324279699891121708787977695465087890625f,0.032400003916025266192946219234727323055267333984375f,-0.5802000274837018078954997690743766725063323974609375f,-0.90300000321865081520655849089962430298328399658203125f,-0.46679998792409893582089353003539144992828369140625f,-0.8733000157222152726177455406286753714084625244140625f,0.192900020694732887704958557151257991790771484375f,0.39089999113082873893887381200329400599002838134765625f,0.1283999879643322739042332614189945161342620849609375f,-0.7028999828830360119269471397274173796176910400390625f,-0.4826999827802183062885887920856475830078125f,-0.0294000081852079375721586984582245349884033203125f,-0.47999998271465305066385553800500929355621337890625f,0.19140000183731320504421091754920780658721923828125f,0.771600003206729834204224971472285687923431396484375f,-0.8951999977558851018244467923068441450595855712890625f,0.049500031054019899556806194595992565155029296875f,0.32879998932480802853461909762700088322162628173828125f,0.509399987137317822316617821343243122100830078125f,-0.1869000004738570197559965890832245349884033203125f,-0.583499991834163456161377325770445168018341064453125f,-0.90120000151097767737695676260045729577541351318359375f,0.04740000227689744605186206172220408916473388671875f,0.9654000335097314700050219471449963748455047607421875f,-0.068099982613325504843260205234400928020477294921875f,0.733500011578202215645205797045491635799407958984375f,-0.87689999622702519133099485770799219608306884765625f,-0.030699996876716539873086730949580669403076171875f,-0.1060999984979626997727564230444841086864471435546875f,-0.9866999812602994524013411137275397777557373046875f,-0.63970004214644493600872010574676096439361572265625f,0.80249997854232812954933251603506505489349365234375f,0.2269000017404554370159530662931501865386962890625f,0.6827000137746335184374402160756289958953857421875f,0.20670001136064541924497461877763271331787109375f,-0.738699994432926221321622506366111338138580322265625f,-0.50250000804662686704205043497495353221893310546875f,0.264799974453449493694279226474463939666748046875f,0.8552000027477735244474388309754431247711181640625f,-0.3126999866545201456347058410756289958953857421875f,0.46250000700354565452698807348497211933135986328125f,0.0474000162839889327415221487171947956085205078125f,0.8102000225663186228075574035756289958953857421875f,1.228899964249134058036361238919198513031005859375f,0.8893000148952001637780995224602520465850830078125f,-0.18779999287128479323882856988348066806793212890625f,-0.68820000075101805947497268789447844028472900390625f,0.231200019839405968014034442603588104248046875f,-0.2664000067919491954171462566591799259185791015625f,-0.91289998135566730042000926914624869823455810546875f,-0.598500011950731103382850051275454461574554443359375f,0.8572000211358068444411628661327995359897613525390625f,0.251800020706653970847810342092998325824737548828125f,-0.31919994877576840508481836877763271331787109375f,-1.142800013583897822400103905238211154937744140625f,-0.3723000006794929817033334984444081783294677734375f,0.170400019949674241814818742568604648113250732421875f,0.1898999997809529549641638368484564125537872314453125f,0.345900003200769301514583276002667844295501708984375f,-0.9075000234693291201182319127838127315044403076171875f,-0.9770999793052670412407678668387234210968017578125f,0.37529998892247673492050807908526621758937835693359375f,0.82199998247623451419485718361102044582366943359375f,-0.006300003741681603486313179018907248973846435546875f,0.309599999570846495799969488871283829212188720703125f,-0.56729999743849024440578432404436171054840087890625f,-0.569100002461671916620389310992322862148284912109375f,0.764999990053474743234573907102458178997039794921875f,-0.168600014591216762482872582040727138519287109375f,0.206100006097555077388960853568278253078460693359375f,-0.744599987053871092967938238871283829212188720703125f,0.5682000079035758322021365529508329927921295166015625f,-0.523799992156028793033328838646411895751953125f,0.632699998761713455763810998178087174892425537109375f,0.514199975001812248365240520797669887542724609375f,-0.51179998673200588488185758251347579061985015869140625f,0.27660000559091546534773442544974386692047119140625f,-0.14880000295937068077734011239954270422458648681640625f,0.4416000224292278009130541249760426580905914306640625f,-0.68189999410361046283668429168756119906902313232421875f,-0.71519998561143882653823311557061970233917236328125f,0.2388000103354454284332319957684376277029514312744140625f,0.6801000153809784176672792455065064132213592529296875f,-0.8129999986365437081303753075189888477325439453125f,0.4944000410795208466652184142731130123138427734375f,0.4466999981358645843698695898638106882572174072265625f,0.37950001078844053381544654257595539093017578125f,-0.6575999992132182114801253192126750946044921875f,-0.2396000189423563853097220999188721179962158203125f,0.9532999870002267783775096177123486995697021484375f,1.0723000315248965019776505869231186807155609130859375f,-0.309399984455108523206945392303168773651123046875f,0.38360000937581073543469756259582936763763427734375f,1.40330001039504992377260350622236728668212890625f,0.74910002689957622123984037898480892181396484375f,1.207899999237060484347239253111183643341064453125f,0.4919000151515007246416644193232059478759765625f,-0.12980000194609164765324749168939888477325439453125f,-0.32700001130998135323579845135100185871124267578125f,-0.492799973499774779384097200818359851837158203125f,-1.2044000379502772801743049058131873607635498046875f,0.2943000020988286724588078868691809475421905517578125f,-0.420500004392116988327643412048928439617156982421875f,-1.1753999682426456985240292851813137531280517578125f,-0.53980000246763193416654758038930594921112060546875f,-0.7554999743103980591740764793939888477325439453125f,-0.470700012671947387588033961947076022624969482421875f,0.00199999827146513098341529257595539093017578125f,-0.054000002056360241198262883699499070644378662109375f,-0.96919999943971646416684961877763271331787109375f,-0.816600024515390376933510196977294981479644775390625f,-0.8704000141382215449681325480923987925052642822265625f,-0.304400025433302356958620293880812823772430419921875f,-0.69479998548030863503299769945442676544189453125f,-0.794400018876790792177189359790645539760589599609375f,0.3252000082612038767138074035756289958953857421875f,0.383999995052814568907706416212022304534912109375f,-0.48960002192258844644356940989382565021514892578125f,-0.7998000140905379140576769714243710041046142578125f,0.7791000134512782171469780223560519516468048095703125f,0.1538999891340733228162207524292171001434326171875f,0.229800000232458057070061840931884944438934326171875f,-0.5966999952673912588352322927676141262054443359375f,0.46530000613331790049187475233338773250579833984375f,0.693900005078315729178939363919198513031005859375f,0.702300002244114818239495434681884944438934326171875f,1.1682000090956687898824384319595992565155029296875f,-0.048000003106892119575377364526502788066864013671875f,0.3156000070512294808366959841805510222911834716796875f,-0.832200017894804577878176132799126207828521728515625f,0.4056000304460525285321637056767940521240234375f,-0.05490000531300898156672474215156398713588714599609375f,0.8364000013902781471841763050178997218608856201171875f,-0.8657999811887739927129814532236196100711822509765625f,-1.252199979114532624180355924181640148162841796875f,-0.6951000041306019117115511107840575277805328369140625f,-0.7769999872446060606989703956060111522674560546875f,0.162600005583465101466345004155300557613372802734375f,0.26700001078844071145113048260100185871124267578125f,-0.5279999994188546708073772606439888477325439453125f,-0.266099988365173345528091886080801486968994140625f,-0.26760001284778123231689050953718833625316619873046875f,-0.6462000058323142948069062185822986066341400146484375f,-0.49859999950379130240207814495079219341278076171875f,-0.0143999805808068259693754953332245349884033203125f,0.8682000026509164580801325428183190524578094482421875f,-0.1796000056803226385682137333787977695465087890625f,1.1871000246047973547547371708787977695465087890625f,0.258500017762184342018372262828052043914794921875f,-1.145400004303455343546147560118697583675384521484375f,-0.53080002171993267978677977225743234157562255859375f,-0.56260000562071799112828784927842207252979278564453125f,0.1964000024706125113738153231679461896419525146484375f,0.3923999927401542464622252737171947956085205078125f,-0.091000003755092873092280569835565984249114990234375f,0.4929000048995018090636222041212022304534912109375f,0.154100008869171301029155074502341449260711669921875f,1.01420001523494729411822845577262341976165771484375f,0.3914000086545943446481032879091799259185791015625f,0.4061999647796152856926710228435695171356201171875f,-0.2764000135719779649434713064692914485931396484375f,-0.6377000124335285846655096975155174732208251953125f,-0.91430004305839585043713668710552155971527099609375f,-1.3203999867200852946780287311412394046783447265625f,-0.84360001995563482779516562004573643207550048828125f,0.075299986091255988185366732068359851837158203125f,0.08770000103414055114825487180496565997600555419921875f,-0.2485999841928485665221160161308944225311279296875f,0.6050000387430198856009155861102044582366943359375f,-0.5270000167489055087344240746460855007171630859375f,0.3562000085890293377133275498636066913604736328125f,-1.2019999806880949932974544935859739780426025390625f,-0.58200001794099787133518475457094609737396240234375f,0.05299999874830252366564309340901672840118408203125f,-0.29480001572966596512515025096945464611053466796875f,-0.527300030183791790250324993394315242767333984375f,-0.685500012606382380653258223901502788066864013671875f,-0.0603000004336238237812040097196586430072784423828125f,0.63330000801086416828411529422737658023834228515625f,-0.440400002776086318956316745243384502828121185302734375f,-0.755699998813867512836850437452085316181182861328125f,0.181800003719329861251452484793844632804393768310546875f,-0.60330000521689652881462961886427365243434906005859375f,-0.4110000040903687423821111224242486059665679931640625f,0.02969998924732220046962538617663085460662841796875f,0.2661000011801719633552920640795491635799407958984375f,0.523200004029273912919961730949580669403076171875f,0.26819999337494360158729023169144056737422943115234375f,0.64860001190900806733452554908581078052520751953125f,0.044399998126924078434285547700710594654083251953125f,0.836099988961219953154113682103343307971954345703125f,-0.6735000246539712964732871114392764866352081298828125f,0.0188999985218050170487913419492542743682861328125f,-0.6227999949574469784607799738296307623386383056640625f,-0.6551999841213227426806042785756289958953857421875f,0.3849000065982342011494665712234564125537872314453125f,0.3744000024110079039019183255732059478759765625f,0.7194000310957433530489879558444954454898834228515625f,-0.316799991798401237019788823090493679046630859375f,0.6444000189140439704971186074544675648212432861328125f,-0.2849999964237213134765625f,-0.3672000039249658787099406254128552973270416259765625f,-0.6479999835491181414681705064140260219573974609375f,-0.15780000903904445319625438060029409825801849365234375f,0.3105000073909760516244205064140260219573974609375f,-0.938100034171343022393330102204345166683197021484375f,0.1024000082559885227428964071805239655077457427978515625f,-0.5982999901294707711940645822323858737945556640625f,-0.590900014078616919732667156495153903961181640625f,-1.1123999839484692842006552382372319698333740234375f,-0.380000014007091380108249722979962825775146484375f,0.396799975109100699910413823090493679046630859375f,0.469999996870755776257055913447402417659759521484375f,-0.5210999964118006033686469891108572483062744140625f,-0.1052999875068660884380733477883040904998779296875f,-0.25720000340342519251635167165659368038177490234375f,-0.5600000245869163340017848895513452589511871337890625f,0.1958999863624573123388472595252096652984619140625f,1.01570001252293540261462112539447844028472900390625f,0.78739997426271468583536261576227843761444091796875f,0.744800001424550384143685732851736247539520263671875f,-0.9897000154733657240058164461515843868255615234375f,-0.213300016057491603760354337282478809356689453125f,0.0906000086903571144603120046667754650115966796875f,-0.63780002937912971816558638238348066806793212890625f,-0.9965999975442887404142311424948275089263916015625f,-0.6250000241398812050874767010100185871124267578125f,-0.176500006675720300108878291212022304534912109375f,0.19710000836253183109647579840384423732757568359375f,0.99160001631975180913514122948981821537017822265625f,0.3926000111341476639381653512828052043914794921875f,-1.14269999514818199060073311557061970233917236328125f,-0.6323000136435030782422472839243710041046142578125f,-0.06949998605251295202833716757595539093017578125f,-0.00669998930692639049766512471251189708709716796875f,1.171699973511696413197569199837744235992431640625f,0.091500001292675658959296924876980483531951904296875f,-0.4101000035718083136515588194015435874462127685546875f,-0.12989999244213112916668251273222267627716064453125f,-0.073799995248019722993149116518907248973846435546875f,-0.535199974286556301450445971568115055561065673828125f,0.761399985073506702093482090276665985584259033203125f,0.524399987733364270070524071343243122100830078125f,-0.3500999841257927958082518671290017664432525634765625f,-0.7173000105142595117513337754644453525543212890625f,-0.67920001054108147808818785051698796451091766357421875f,-0.16859999864697439164729075855575501918792724609375f,0.7575000006705521204963815762312151491641998291015625f,-0.4803000059843061109177142498083412647247314453125f,0.63899998484551889799121227042633108794689178466796875f,0.33269998322725324868542884360067546367645263671875f,-0.3309000036478042527932075245189480483531951904296875f,-0.963600012505054337452747859060764312744140625f,-0.3228000098213554824866378112346865236759185791015625f,0.6708000306606294316225103102624416351318359375f,-0.3270000003576278846395553046022541821002960205078125f,-0.349799983096122701908825547434389591217041015625f,0.444900008721649697918110177852213382720947265625f,0.04799999994039527706490844138897955417633056640625f,0.33539999562352884598936952897929586470127105712890625f,0.70230001289844512513127483543939888477325439453125f,-0.78089999683052291334917072163079865276813507080078125f,-0.98760000338554387866452088928781449794769287109375f,-0.25950000125169747633435690659098327159881591796875f,-0.34800001066923158532517845742404460906982421875f,0.398699976141750678237940519466064870357513427734375f,0.76640001945793623239211456166231073439121246337890625f,0.3009999790191653090687395888380706310272216796875f,1.05620002572536453300244829733856022357940673828125f,0.383000007800757824583115507266484200954437255859375f,0.050199998389184319336209227913059294223785400390625f,0.4134000325560567290494873304851353168487548828125f,-0.42659997325539489310131102683953940868377685546875f,-0.5654000035025177339775837026536464691162109375f,-0.5884000314198438719159867105190642178058624267578125f,-0.1546999847769736646796445711515843868255615234375f,-0.706700026559829996841699539800174534320831298828125f,0.3106999697566035223417202359996736049652099609375f,-0.5833000442206870417294339858926832675933837890625f,0.6067000304341316763157010427676141262054443359375f,1.26090004864931160000196541659533977508544921875f,1.1192999998450279264261553180404007434844970703125f,0.44030002684593227968434803187847137451171875f,-0.18979997624158873037458761245943605899810791015625f,-0.538400000667571543289113833452574908733367919921875f,-0.129899994826316866891602330724708735942840576171875f,-0.085299997490644319242392157320864498615264892578125f,-0.0598000027656553356791846454143524169921875f,-0.45600002923607885207957224338315427303314208984375f,-0.7628000235676761775494014727883040904998779296875f,-1.2892000678360471255246011423878371715545654296875f,-0.8538000034809112293032740126363933086395263671875f,-0.436200022894144279206329883891157805919647216796875f,-0.20490001412332059516074878047220408916473388671875f,0.38970001122653474823209762689657509326934814453125f,-0.140900025701522935150933335535228252410888671875f,0.49529998862445356877515223459340631961822509765625f,0.6785999989449977842781436265795491635799407958984375f,-0.53400003468990320243392488919198513031005859375f,-0.1032000020921230998283135704696178436279296875f,0.15900000928342350459132603646139614284038543701171875f,-0.6707999815240499597024381728260777890682220458984375f,0.256199998307227616578529705293476581573486328125f,-0.5646000176198782138925480467150919139385223388671875f,-0.278699997506290486626312485896050930023193359375f,-0.9555000141710043859433199031627736985683441162109375f,-0.168599988365173203419544734060764312744140625f,-0.7923000314131380950044558630906976759433746337890625f,0.2585999856829648280154287931509315967559814453125f,0.8805000131651758987771927422727458178997039794921875f,-0.3308999970912935850719804875552654266357421875f,0.2427000052437187083143044219468720257282257080078125f,0.6428999849319456671992156771011650562286376953125f,-0.40919999366551629282895419237320311367511749267578125f,0.167100015962123649160275817848742008209228515625f,-0.152699997085332828117287817804026417434215545654296875f,-0.164700000721216266441615516669116914272308349609375f,-0.69660002004504217865132886799983680248260498046875f,-0.1628999995350834950613716500811278820037841796875f,-0.9030000361874701564346423765528015792369842529296875f,0.2046000037431721096936598769389092922210693359375f,-0.2760000056549907210268202106817625463008880615234375f,-0.4433999926209448716463157325051724910736083984375f,0.05550000567734247880480324965901672840118408203125f,-0.7077000215232371882478901170543394982814788818359375f,-0.13709999789446580820140297873876988887786865234375f,-0.34760001858472833191626705229282379150390625f,0.21819998957514741277918801642954349517822265625f,0.1794000047206878445393840593169443309307098388671875f,0.9906999938964844432121026329696178436279296875f,0.2729000062406061744013641146011650562286376953125f,-0.18190000635981551369013686780817806720733642578125f,0.211899998760223251537126998300664126873016357421875f,0.4381999676704406709859540569595992565155029296875f,-0.14980001572966639145079170702956616878509521484375f,0.1462999924540522300731026916764676570892333984375f,0.65710002236962328225899909739382565021514892578125f,0.098899987643957132377181551419198513031005859375f,0.51430000446438750572042408748529851436614990234375f,0.8384000143766403567724410095252096652984619140625f,0.620000037252903535289760839077644050121307373046875f,0.63979999054670344538209292295505292713642120361328125f,0.5514000096231697067850063831428997218608856201171875f,-1.02550003510713594323533470742404460906982421875f,-0.1975000180304053998270319425500929355621337890625f,-0.8606999930396674880483942615683190524578094482421875f,-0.9403000341102483172761594687472097575664520263671875f,1.289599991971254411282643559388816356658935546875f,1.1574000364005565533176422832184471189975738525390625f,0.684299989414215215077774701057933270931243896484375f,0.334300006848573527662438209517858922481536865234375f,-1.4513000326871867695643913975800387561321258544921875f,-0.897700053411722986851373207173310220241546630859375f,-0.2331999921083447446790160029195249080657958984375f,0.000999993175267999134803176275454461574554443359375f,0.062299988758564239788029226474463939666748046875f,-0.841499995201826056501204220694489777088165283203125f,0.428400009123981018177573787397705018520355224609375f,0.4176000059187412460914856637828052043914794921875f,0.275099994742870279651469900272786617279052734375f,0.812099996590614381375416996888816356658935546875f,0.27329999750554556658954652448301203548908233642578125f,-0.173700001585483487787087142351083457469940185546875f,-0.3396000092938542547216229650075547397136688232421875f,0.24059997978210478919436354772187769412994384765625f,0.3032999991819262941561419211211614310741424560546875f,-0.504600029075145783963307621888816356658935546875f,0.1958999997362493861174925768864341080188751220703125f,-0.456300009739398770847174091613851487636566162109375f,0.3684000096827746428829186697839759290218353271484375f,0.25919998871088001379803245072253048419952392578125f,0.51359999637454745169407033245079219341278076171875f,0.34439999358206996316056347495759837329387664794921875f,0.13470000560134653966315454454161226749420166015625f,-0.49200000780820829504591529257595539093017578125f,-0.85950001365691430788018578823539428412914276123046875f,-0.3575999908313154396211075436440296471118927001953125f,0.80940000483244656237502567819319665431976318359375f,0.3692999812185764341165850055404007434844970703125f,0.246899994550645318636128422440378926694393157958984375f,0.407399988120794365187293806229718029499053955078125f,-0.83010002871751797925981009029783308506011962890625f,-0.9294000204414125665408619170193560421466827392578125f,-0.3252000131785870440381813750718720257282257080078125f,-0.58410000648498527908714095246978104114532470703125f,0.853499997720122305366885484545491635799407958984375f,-0.79110002666115786240652596461586654186248779296875f,-0.23729999524056921700321254320442676544189453125f,-0.25290000884830965421912196688936091959476470947265625f,0.754900003021955257054287358187139034271240234375f,0.6969000346720222172081093958695419132709503173828125f,-0.641799998801946713911092956550419330596923828125f,-0.7782000272750855263126368299708701670169830322265625f,0.571499997511506041547590939444489777088165283203125f,-0.04549999852478503470365467364899814128875732421875f,1.3043999987602230561378746642731130123138427734375f,0.69540005213618361068483864073641598224639892578125f,1.7671999931216237911257849191315472126007080078125f,1.06200003865361214394624767010100185871124267578125f,0.408699976775050544119949336163699626922607421875f,0.53070000410377959898511335268267430365085601806640625f,-1.094699978816509489121244769194163382053375244140625f,-0.803100008988379965302328855614177882671356201171875f,-0.154599998378753600292156988871283829212188720703125f,-0.01540000385642059654855984263122081756591796875f,0.61620000221878257207208662293851375579833984375f,0.5956000257894398242086708705755881965160369873046875f,-0.57950000285357194940161207341589033603668212890625f,-0.732100039394200319264882637071423232555389404296875f,0.268499990046024095136090181767940521240234375f,-0.0895000039339066688626189716160297393798828125f,0.514199967849254591101271216757595539093017578125f,0.19459998645782405901627498678863048553466796875f,0.91340000334978110885231217253021895885467529296875f,0.43920001240372652517862661625258624553680419921875f,-0.63599999235570425781816084054298698902130126953125f,0.2987999924540520879645555396564304828643798828125f,-0.4665000000447034889106134869507513940334320068359375f,-0.36839999612271778772765173926018178462982177734375f,0.53730001833736906746707973070442676544189453125f,0.2591999951183794337339350022375583648681640625f,-0.398400004206597824207847224897705018520355224609375f,0.12360001712441448518120523658581078052520751953125f,0.037499999329447752938193616500939242541790008544921875f,0.6345000009536743146298931605997495353221893310546875f,0.7908000285744669621834646022762171924114227294921875f,1.0526999873995777079471736215054988861083984375f,0.7422000123068690502492472660378552973270416259765625f,0.873599990153312688789810636080801486968994140625f,0.8222999877154826275926779999281279742717742919921875f,0.5135999832242728047049240558408200740814208984375f,-0.727499988600611491307290634722448885440826416015625f,-0.627599985653162217857925497810356318950653076171875f,-0.076800003384053738830772317669470794498920440673828125f,-0.205799999815225564958609538734890520572662353515625f,0.25740001743882900964166537960409186780452728271484375f,-0.03390000116676095132106638629920780658721923828125f,-0.8157000286906959107824377497308887541294097900390625f,-0.36150000380724645054897337104193866252899169921875f,0.130500005453825007606383223901502788066864013671875f,0.52800000768899923997423684340901672840118408203125f,0.44459999241828906946238930686376988887786865234375f,0.68459999778270752557318701292388141155242919921875f,0.31139999680817120886189286466105841100215911865234375f,0.5336999994993210094662572373636066913604736328125f,-0.43860000397413967387905131545267067849636077880859375f,-0.31339999843239763066549130599014461040496826171875f,0.81269997417479800105866161175072193145751953125f,0.729499999277293031951785451383329927921295166015625f,0.3348000053733581893311566091142594814300537109375f,0.54960003511011656929241553370957262814044952392578125f,-0.458499983787536535828621708787977695465087890625f,-0.9877000249505041296060880995355546474456787109375f,-0.75309997918605819933191014570184051990509033203125f,-0.26250000596046429990337855997495353221893310546875f,-0.178500000663101676678934381925500929355621337890625f,0.039299999360740134335401307907886803150177001953125f,-0.04020001392364491721309605054557323455810546875f,0.68360003023743676209278419264592230319976806640625f,-0.0227999726057053209160585538484156131744384765625f,-0.55940000606179207665036301477812230587005615234375f,-0.31190002193152910336948480107821524143218994140625f,0.4305000105947254240845722961239516735076904296875f,-1.6027999854207042318421372328884899616241455078125f,-0.80960001342892606857049031532369554042816162109375f,0.02290002605915031352878941106610000133514404296875f,-0.484099984729289278817532249377109110355377197265625f,0.7589999971389769228835575631819665431976318359375f,0.233000005304813129214380751363933086395263671875f,-0.450499988131225048704209257266484200954437255859375f,-0.040300001327693291131026853690855205059051513671875f,-0.432099971556663575711354496888816356658935546875f,0.099300012958050132283460698090493679046630859375f,-0.6463999740839003749215407879091799259185791015625f,-0.06079999921917877969690380268730223178863525390625f,-0.161200009036063907075231327326036989688873291015625f,0.110999994367360965696889252285473048686981201171875f,0.44309997750967722307535723302862606942653656005859375f,0.156599974186718782220850698649883270263671875f,0.71280003250837342676504704286344349384307861328125f,0.2934000110238788039396240492351353168487548828125f,-0.714900009727478025212121792719699442386627197265625f,0.246000015079975042908699833787977695465087890625f,-0.20849999358505011404218976167612709105014801025390625f,-0.534299978387355878339803894050419330596923828125f,-0.03269999954402447173151813331060111522674560546875f,-0.25950000132620332227162407434661872684955596923828125f,0.91620001179277910097198400762863457202911376953125f,-0.020100021564960712794345454312860965728759765625f,-0.5904000006526708421716875818674452602863311767578125f,0.146100017273425919483997859060764312744140625f,0.0675000074878335798445050386362709105014801025390625f,-0.8532000262320040828711853464483283460140228271484375f,-0.5993999838218091458230674106744118034839630126953125f,-0.9383999821901323201700506615452468395233154296875f,-0.62009997987896181559364094937336631119251251220703125f,0.04170000808238949474571199971251189708709716796875f,-0.1025999992132187177418245482840575277805328369140625f,0.2309999974966050473312861868180334568023681640625f,0.1113000000536441547183130751363933086395263671875f,-0.22649998961389072871241978646139614284038543701171875f,-0.26069999169856306053816297207958996295928955078125f,-0.84029999287128465113028141786344349384307861328125f,0.1794000014796852671139504309394396841526031494140625f,-0.29819997899532335594585674698464572429656982421875f,0.555599988126754595896272803656756877899169921875f,0.771300017040967844650367624126374721527099609375f,0.2134999995827673302528637577779591083526611328125f,0.86450003850460088727913898765109479427337646484375f,0.483199982571601793779336730949580669403076171875f,1.2204000230789180392321213730610907077789306640625f,-0.4945000075101848580061414395458996295928955078125f,-0.34390002996325552686585069750435650348663330078125f,0.50120000173449508196199531084857881069183349609375f,0.23700001138448723025931030861102044582366943359375f,0.31530000777244548970656978781335055828094482421875f,0.856100043499470331198608619160950183868408203125f,0.4165999656558039276887939195148646831512451171875f,0.803600011163949634607206462533213198184967041015625f,1.5313000094413755647337893606163561344146728515625f,0.79970004974603714487102479324676096439361572265625f,-0.486800014932453617433338877162896096706390380859375f,-0.0079999989345669320073284325189888477325439453125f,-0.147500002905726290691745816729962825775146484375f,-0.23670000994503503921606579751824028789997100830078125f,0.807899963176250679453005432151257991790771484375f,0.44409998800754468817331144236959517002105712890625f,0.410799985063075911995156275224871933460235595703125f,0.697400024032592824596576974727213382720947265625f,-1.1263999854087831664628538419492542743682861328125f,-0.61240003044009228005961631424725055694580078125f,1.405200034487247506831408827565610408782958984375f,0.57200003418326428317186582717113196849822998046875f,-0.37280000211000441368014435283839702606201171875f,-0.8034000307679178831676836125552654266357421875f,0.71319999747276341395263443700969219207763671875f,-0.69360002681016919012790822307579219341278076171875f,0.5349000198975206732399101383634842932224273681640625f,-0.49050001633167283898728783242404460906982421875f,0.843600007215142166927535072318278253078460693359375f,-0.4254000117540357450707233510911464691162109375f,-0.5709000246584416959905183830414898693561553955078125f,-0.5865000090599057358531354111619293689727783203125f,0.17640000463128091556797016892232932150363922119140625f,0.305100000107288327200194544275291264057159423828125f,0.65130002307593837240773382291081361472606658935546875f,-0.25260000934600856936640411731787025928497314453125f,0.8765999945640563506543685434735380113124847412109375f,0.1742999733209611434858743450604379177093505859375f,0.5013000183820726096683983996626920998096466064453125f,0.776399986302852340713798184879124164581298828125f,0.03960000370591887364213334876694716513156890869140625f,-0.38940001522600653505179479907383210957050323486328125f,-0.1347000077247619831410929691628552973270416259765625f,0.06599999757111074671911410405300557613372802734375f,0.646499982945620832452959803049452602863311767578125f,0.8096999892473224491595829022116959095001220703125f,0.5910000116899609690079842039267532527446746826171875f,0.003899984663724875844081907416693866252899169921875f,-0.2789999999329448865381664290907792747020721435546875f,-0.43319996230602253461938744294457137584686279296875f,0.267000003822147835563782791723497211933135986328125f,0.7356000083923337484748117276467382907867431640625f,-0.59220001074224715154059595079161226749420166015625f,0.188100010478496670884851482696831226348876953125f,0.52199998292326910132032935507595539093017578125f,0.163900005096941914217723024194128811359405517578125f,0.19080000324249279941568602225743234157562255859375f,-0.48800003048777629288679236196912825107574462890625f,-0.79060000511407846346401129267178475856781005859375f,-0.8168000382900242328787498991005122661590576171875f,-0.4264999972879888190391284297220408916473388671875f,-0.5199000143468379686595426392159424722194671630859375f,0.173599972569942195832481957040727138519287109375f,0.759200021046400053137404029257595539093017578125f,-0.3772999985933302991014670624281279742717742919921875f,-0.394900015836954221271071219234727323055267333984375f,1.197800000023841793250767295830883085727691650390625f,0.83200003060698524137706044712103903293609619140625f,-1.3376999668359754736002287245355546474456787109375f,-0.71970002977848057668097681016661226749420166015625f,-0.488400001524388560625311583862639963626861572265625f,-0.5172000272825363342832361013279296457767486572265625f,0.86929998762607585405248755705542862415313720703125f,0.32510000681281070455952431075274944305419921875f,-0.419099968934059319281004718504846096038818359375f,-0.95850001493096304017171860323287546634674072265625f,0.1037999754667284779685587636777199804782867431640625f,0.571399996286630074138201962341554462909698486328125f,-0.0062999847054481250552271376363933086395263671875f,0.75750003427267120770238761906512081623077392578125f,-1.136299970400333592124297865666449069976806640625f,-0.54549999740719723462234469479881227016448974609375f,-0.2161999970778822888206605057348497211933135986328125f,-0.233400008602440423022272852904279716312885284423828125f,-0.256500001758336981794172970694489777088165283203125f,0.30020004909038533469356480054557323455810546875f,-0.3807000019431114878898370079696178436279296875f,-0.459800007981062019979390242951922118663787841796875f,0.51329999005496507180623666499741375446319580078125f,-0.00940001012235847088760465339873917400836944580078125f,1.092599975371360443432422471232712268829345703125f,0.374599981689453187527760746888816356658935546875f,-0.3152999827384945774610969237983226776123046875f,0.2023000137925148322892709984444081783294677734375f,-0.806400002098083490409408113919198513031005859375f,-0.33460000225305552845611600787378847599029541015625f,0.56790001421272773285409130039624869823455810546875f,1.007400021238625154040846609859727323055267333984375f,0.017699972498416371990970219485461711883544921875f,-0.6131000206112862116469841566868126392364501953125f,0.419399971789121916998510641860775649547576904296875f,0.257999999344348918128844161401502788066864013671875f,-0.19919998982846731649942739750258624553680419921875f,0.119300009083747937665975769050419330596923828125f,0.112500000596046589862453402020037174224853515625f,0.20860001429319385835015054908581078052520751953125f,0.417899992382526619394411682151257991790771484375f,-0.059599997931718551313906573341228067874908447265625f,-1.2305999797821041141787645756267011165618896484375f,-1.0852999913811682830555582768283784389495849609375f,0.23670000227093623834662139415740966796875f,-0.0611999917507173307740231393836438655853271484375f,-0.963600023531913763008560636080801486968994140625f,-0.98200002390146234887424725457094609737396240234375f,-0.068100021207332162020975374616682529449462890625f,0.07159998845458037664002404198981821537017822265625f,0.4375000026077031645144188587437383830547332763671875f,-0.21419999511837961581051104076323099434375762939453125f,0.7804000043779610873428964623599313199520111083984375f,-0.160400000280141785680143584613688290119171142578125f,0.037700012433528939936877577565610408782958984375f,-0.730600011670589299939138072659261524677276611328125f,-0.6916000217586757425891619277535937726497650146484375f,0.096400000756978965199550657416693866252899169921875f,0.127600030803680652979892329312860965728759765625f,0.0567999998450279264261553180404007434844970703125f,-0.61219999669790237106781205511651933193206787109375f,0.0583999982833862674169722595252096652984619140625f,-0.0738000022888183337954615126363933086395263671875f,-0.004199999290704692356257510255090892314910888671875f,-0.238600002968311297735226617078296840190887451171875f,-0.063999999970197674059591008699499070644378662109375f,1.1796999984860416788023940171115100383758544921875f,0.11880000318288796989918409963138401508331298828125f,-0.1668000087112189167015685598016716539859771728515625f,0.3351999988734721824812368140555918216705322265625f,-0.13100000166893011055435636080801486968994140625f,0.187999999046325694251891036401502788066864013671875f,0.765400026506185771069112888653762638568878173828125f,-0.289999998658895474790142543497495353221893310546875f,0.44190000412464147672153558232821524143218994140625f,0.26400000533461565055404207669198513031005859375f,-0.803400002753734465699153588502667844295501708984375f,-0.07319999628067019870059084496460855007171630859375f,0.0051999656438823649295954965054988861083984375f,0.2792999813675880460550615680404007434844970703125f,-0.480900000071525557387985827517695724964141845703125f,-0.3982999896079302271090227804961614310741424560546875f,-0.5316000090181827175683793029747903347015380859375f,-0.6893999901920555561929404575494118034839630126953125f,-1.3269000184297556899082337622530758380889892578125f,-0.7599000261187551785724281216971576213836669921875f,0.3996000155150891686162140103988349437713623046875f,0.5950000093132257195094325652462430298328399658203125f,0.2567999637842177662605536170303821563720703125f,0.508799987983703516647437936626374721527099609375f,0.22920001285076097019555163569748401641845703125f,0.28899998968839657464968695421703159809112548828125f,0.0692999992489813365637019160203635692596435546875f,0.195000011324882649432765902020037174224853515625f,-0.03479999568760394623723186668939888477325439453125f,0.114199999588727951760347423260100185871124267578125f,-0.155400012350082050716082449071109294891357421875f,-0.6848999855875970155238974257372319698333740234375f,0.595800005477666871911424095742404460906982421875f,0.2849999990314244779909813587437383830547332763671875f,0.4619999896287918517145953956060111522674560546875f,0.5070000063180923888239703956060111522674560546875f,-0.7656000128626825329547500587068498134613037109375f,-0.824400010532140736785322587820701301097869873046875f,0.30060000422000854314319440163671970367431640625f,-0.13649999177455907073408525320701301097869873046875f,0.19560001010596739234870256041176617145538330078125f,0.684399999803304570633599723805673420429229736328125f,-1.010999959945678483563824556767940521240234375f,-0.296800000590085932827832948532886803150177001953125f,-0.31609999015331258220840027206577360630035400390625f,-0.42519999961853027770075641456060111522674560546875f,-0.120000008046627471003375831060111522674560546875f,0.4587999854505062646836677231476642191410064697265625f,-0.044799990844726522709606797434389591217041015625f,-0.0308000097990035470729708322323858737945556640625f,-1.04960000404119480066356118186376988887786865234375f,0.37819999032020579221580192097462713718414306640625f,-0.1355999955773352638743745046667754650115966796875f,0.6537999988615512680922847721376456320285797119140625f,-0.978700015145540280769864693866111338138580322265625f,0.238199989277124490882897589472122490406036376953125f,0.521199997115135094105653479346074163913726806640625f,-0.022000001400709123799970257095992565155029296875f,0.18940002290010493624095033737830817699432373046875f,-0.5093999966740607732162970933131873607635498046875f,0.6538000072807073070890737653826363384723663330078125f,0.022600001299381222708007044275291264057159423828125f,0.6308000039875505393638377427123486995697021484375f,0.405599994534254104650017325184307992458343505859375f,0.9315000296235087517970896442420780658721923828125f,0.146200003522634514041556030861102044582366943359375f,0.674900008237361959118061349727213382720947265625f,-0.5824000091314314175860999966971576213836669921875f,-0.1082000177383424244226262089796364307403564453125f,-0.023999995201826163082614584709517657756805419921875f,-0.706900025470555082307555494480766355991363525390625f,-0.3126000029757618481340841753990389406681060791015625f,-0.404300016981363530277349127572961151599884033203125f,-0.7244999999850987304483851403347216546535491943359375f,-0.593400004243850531793214031495153903961181640625f,-0.00950001406669631620616200962103903293609619140625f,0.93479996163845147094662024755962193012237548828125f,0.74059998753070832577805049368180334568023681640625f,-0.11940001306533787328589824028313159942626953125f,-0.216400028324127191581283113919198513031005859375f,1.079099988007545363188910414464771747589111328125f,1.28899999326467540328167160623706877231597900390625f,1.23869998966455430178257302031852304935455322265625f,0.80320000045299533297793459496460855007171630859375f,0.6957000013470651111902043339796364307403564453125f,0.797499989420175747767416396527551114559173583984375f,-0.26460000009238715534820585162378847599029541015625f,-0.548799994093179588361408605123870074748992919921875f,-0.96269999783039139629181590862572193145751953125f,-0.60360000654458989544082214706577360630035400390625f,-0.200700006115436480058633605949580669403076171875f,-0.5248000020951033572913502212031744420528411865234375f,0.57599997104704403483310670708306133747100830078125f,0.09760000368356713806861080229282379150390625f,0.0116999800682062726764343096874654293060302734375f,-0.37709999808073035865163546986877918243408203125f,-1.3680000067949293907076935283839702606201171875f,-1.345700009691714882364976801909506320953369140625f,-0.1037999736785888416079615126363933086395263671875f,-0.2119999914169312660305877216160297393798828125f,-0.3218999861128624129236186490743421018123626708984375f,0.221900007626414375039303195080719888210296630859375f,0.9483000136733057416904557612724602222442626953125f,-0.292800009039044351055736115085892379283905029296875f,-0.59100000539422037348913363530300557613372802734375f,0.34579999489784241095691186274052597582340240478515625f,-0.0783000089049341596592057612724602222442626953125f,-0.03560000243186944857143316767178475856781005859375f,-0.3179999930858610213135762023739516735076904296875f,-0.04899999326467519011885087820701301097869873046875f,-0.041500015914440524511519470252096652984619140625f,-0.1411999975621700542660619248636066913604736328125f,0.6410000237226487485031611868180334568023681640625f,0.3943999926507473219317034818232059478759765625f,0.181599990540743050360106281004846096038818359375f,-0.3383999882996082675390425720252096652984619140625f,0.7476000078558922012916809762828052043914794921875f,-0.1959999901950360001734452453092671930789947509765625f,-1.2380000115633009727389435283839702606201171875f,0.105599988573789715928796795196831226348876953125f,-0.889499982774257347273305640555918216705322265625f,0.54999999418854716992655085050500929355621337890625f,0.520699986669420855633916289662010967731475830078125f,-0.130199990677833665753126979325315915048122406005859375f,-1.31560001971721618474475690163671970367431640625f,-0.0803999922335147942931143916212022304534912109375f,-0.39000002264976529886553180404007434844970703125f,-0.5028000055372714438561843053321354091167449951171875f,0.0913000081002710572875002981163561344146728515625f,0.45459998511672028431718217689194716513156890869140625f,0.8685000156760214196083325077779591083526611328125f,0.21579999817609785139893574523739516735076904296875f,0.932500023953616530292265451862476766109466552734375f,-0.85030002997517595275667190435342490673065185546875f,0.798899996584653937503617271431721746921539306640625f,0.726799990010261520723133799037896096706390380859375f,0.122999982953071906877084984444081783294677734375f,0.4679000174164773540042006061412394046783447265625f,0.101100004458427150666466332040727138519287109375f,0.4162000036716460016350538353435695171356201171875f,-0.09600000584125467639751150272786617279052734375f,0.288500019252300177186043583787977695465087890625f,0.44339997205734249519082368351519107818603515625f,-0.2232000236988067598531415569595992565155029296875f,-0.79439995606839719499703278415836393833160400390625f,-0.81959997841119758277272921986877918243408203125f,-0.55139999643564241438298267894424498081207275390625f,-0.60859998478889476558606475009582936763763427734375f,0.53340000543594356940957368351519107818603515625f,0.82779998512268093691091053187847137451171875f,1.406399974083900161758720059879124164581298828125f,1.541299982917308852847781963646411895751953125f,-0.3530999911814924896447109858854673802852630615234375f,-0.3632999776124956614609118332737125456333160400390625f,0.2148000492572776920496835373342037200927734375f,0.671400015211105483103892765939235687255859375f,-0.84479998562931957195587528985925018787384033203125f,-0.41060000243186944857143316767178475856781005859375f,0.840299956661463287588276216411031782627105712890625f,0.534699980306625466397463242174126207828521728515625f,0.212699964451790179964518756605684757232666015625f,-0.1091000020146370985685280174948275089263916015625f,0.4664999549500647990640800344408489763736724853515625f,0.30939999637603765592075433232821524143218994140625f,0.5147999991893765781014735694043338298797607421875f,-0.5010000046491622782696140347979962825775146484375f,0.034399993395805239515539142303168773651123046875f,0.5993999945878982771318987943232059478759765625f,0.7994000252842905496208913973532617092132568359375f,0.550400004154443678316965815611183643341064453125f,-0.12789998910427069489514906308613717555999755859375f,0.308200001794099787133518475457094609737396240234375f,1.0590000118538736018081181100569665431976318359375f,-0.3029999901056290578793550594127736985683441162109375f,-0.887300016623735476883894079946912825107574462890625f,0.071200002926588012996944598853588104248046875f,0.341399988724291336694705023546703159809112548828125f,-0.2021999941274523759915382470353506505489349365234375f,-0.709400019621849509121602750383317470550537109375f,0.457399988418817571300678537227213382720947265625f,0.64709998982921224097708545741625130176544189453125f,-0.14679999559819700838403377929353155195713043212890625f,-0.4837000138044356134514600853435695171356201171875f,-0.486999991416931177212745751603506505489349365234375f,1.098500003159045945722027681767940521240234375f,-0.59039999282956134862843100563623011112213134765625f,-0.219499982494860634840705415626871399581432342529296875f,0.28739998715221892755522503648535348474979400634765625f,-0.2439999800026413367959321476519107818603515625f,0.66759999742507936115742950278217904269695281982421875f,0.593800011229514979049781686626374721527099609375f,0.102600000405311586604284457280300557613372802734375f,0.725900020486116570594958830042742192745208740234375f,1.13740001032352466836528037674725055694580078125f,-1.12770000557899408022421994246542453765869140625f,-1.0338000142097474309821336646564304828643798828125f,1.0424999892711639404296875f,0.507800003600120675173457129858434200286865234375f,0.7349999831616873535722334054298698902130126953125f,-0.0072000023603440155284260981716215610504150390625f,0.5978999983429904574450119980610907077789306640625f,0.40549999606609343771879139239899814128875732421875f,-0.88889999695718291405910349567420780658721923828125f,-1.059799992446601724083166118361987173557281494140625f,0.438600017571449285469498136080801486968994140625f,0.767100005680322549750371763366274535655975341796875f,-0.5888999714575711141861802389030344784259796142578125f,-0.762999981686472761310824353131465613842010498046875f,1.202099970364571390746277756989002227783203125f,1.377400001382827809948139474727213382720947265625f,-0.479999993629753429758011407102458178997039794921875f,-0.6252999804474412837151930943946354091167449951171875f,-0.775499960452318504167124046944081783294677734375f,-0.298899986898899072684798738919198513031005859375f,-1.516199988770484452516029705293476581573486328125f,-1.645900005733967219612168264575302600860595703125f,0.527699977900087713322818672168068587779998779296875f,0.276899970466643774447135228911065496504306793212890625f,1.333799979788064415942017149063758552074432373046875f,0.991599990689754573480740873492322862148284912109375f,0.1364999721050264014365893672220408916473388671875f,0.0796000079154968176453621708787977695465087890625f,-0.4299000089824200898647177382372319698333740234375f,-0.047999998450279246497984786401502788066864013671875f,-0.6965000266432763709190112422220408916473388671875f,0.641800011616945109693688209517858922481536865234375f,0.2066000207901002028165748924948275089263916015625f,0.696799987328052594648397644050419330596923828125f,-0.5568000074446199931799128535203635692596435546875f,0.63320001177787765822557730643893592059612274169921875f,0.34040001056194313378000515513122081756591796875f,0.25759999176263814746334901428781449794769287109375f,1.068200027871132107293306034989655017852783203125f,-0.49860000609755505962539245956577360630035400390625f,-0.34609999428838478241488019193639047443866729736328125f,0.0150000017881393166163661589962430298328399658203125f,-0.062400004214048454542762556229718029499053955078125f,0.4073999857366086274623739882372319698333740234375f,0.9151000012993808496730707702226936817169189453125f,0.031799999994039485073926698532886803150177001953125f,0.137299992036819418217419297434389591217041015625f,-0.6333999945580959245461372120189480483531951904296875f,0.94090001169443127082558930851519107818603515625f,-0.34480001125931725169948549591936171054840087890625f,-0.6830000044107435996920685283839702606201171875f,-0.147199997889995604083424041164107620716094970703125f,-0.260899978613853278375245281495153903961181640625f,0.5367999798774720776606272920616902410984039306640625f,0.129200006890297203199224895797669887542724609375f,-0.0516000011205672848291214904747903347015380859375f,-0.44410003032684386425898992456495761871337890625f,0.1779999884665013354378970689140260219573974609375f,-1.139999997019767619121921597979962825775146484375f,0.05989999821633096299677845308906398713588714599609375f,1.4151000272274014690765397972427308559417724609375f,1.23570002876520135259852395392954349517822265625f,1.1999999582767486572265625f,0.5881999772071839771570012089796364307403564453125f,1.327800022822617886930629538255743682384490966796875f,1.054000015318393490559856218169443309307098388671875f,0.24839998666048046516152680851519107818603515625f,-0.274400017833709597425695392303168773651123046875f,-1.338300016355514543420213158242404460906982421875f,-1.5242000217914579707212396897375583648681640625f,0.1755000003874300773532013408839702606201171875f,0.36229999393671741270139818880124948918819427490234375f,0.65669998453855527031919336877763271331787109375f,0.185599987977743463574142879224382340908050537109375f,-0.1431000262737267547663577715866267681121826171875f,-0.353499994218349655739075387828052043914794921875f,-1.19609997972249981756931447307579219341278076171875f,-0.76609998747110363837009572307579219341278076171875f,-1.023000019311904651431177626363933086395263671875f,-1.286700013744830783934958162717521190643310546875f,-0.0707999826788903163787836092524230480194091796875f,0.2633999929189680955232688575051724910736083984375f,1.04039995483160030431690756813623011112213134765625f,0.69649996227026012007854660623706877231597900390625f,-0.292199985373020609813465853221714496612548828125f,-0.580000009536743021953952847979962825775146484375f,0.6119999760687360179645111202262341976165771484375f,0.90430001653432867669835104607045650482177734375f,1.225199975180625955317736952565610408782958984375f,-0.2793999999523162358627814683131873607635498046875f,0.422399989014863752601058877189643681049346923828125f,-0.4169999903738499380523307991097681224346160888671875f,0.3131000200152396217845307546667754650115966796875f,0.649599980199337068142995121888816356658935546875f,-0.2730000079870222862155060283839702606201171875f,-0.52999998524785052467223067651502788066864013671875f,-0.672300025415420776653263601474463939666748046875f,0.0097999997854232301364163504331372678279876708984375f,0.41480000663995753029666957445442676544189453125f,0.296200004416704132381710223853588104248046875f,-0.458099984252452685495882178656756877899169921875f,0.2649999968707561492919921875f,-0.2825999929547311495525718783028423786163330078125f,0.6519999799430371556496766061172820627689361572265625f,-0.531800010275841117390882573090493679046630859375f,0.56399998641014104094892900320701301097869873046875f,-1.172099989140033216017400263808667659759521484375f,0.014599997186660829129323246888816356658935546875f,0.4893999969720841392017973703332245349884033203125f,-0.3655999940872192155438824556767940521240234375f,0.0979999869763847897274899878539144992828369140625f,-0.36080000860691063824248203673050738871097564697265625f,0.50290000936985013169078229111619293689727783203125f,0.3777999916791916490410585538484156131744384765625f,-0.5147000284373763889789188397116959095001220703125f,-0.332400007641315387019176341709680855274200439453125f,0.86389996663331913850925047881901264190673828125f,-0.65300000217556952719633045489899814128875732421875f,0.4093000177264212169347956660203635692596435546875f,-0.95569998286962487554774270392954349517822265625f,-0.84749998450279218076275355997495353221893310546875f,-0.609199978131055974728269575280137360095977783203125f,0.99719994990825711056459113024175167083740234375f,0.2340999537348746883935746154747903347015380859375f,-0.8771999603390696620408562012016773223876953125f,-0.370399985527992470224489807151257991790771484375f,0.16290002218484911367113454616628587245941162109375f,0.28770000051259980722306863754056394100189208984375f,0.3675000019371505999288274324499070644378662109375f,-0.0843999939918518293779925443232059478759765625f,0.7430999768018722306806012056767940521240234375f,0.918499984681606207459481083787977695465087890625f,1.33619994794130381166041843243874609470367431640625f,1.05319997422695177391460674698464572429656982421875f,1.289999964833260293062267010100185871124267578125f,1.2293999898195266951006487943232059478759765625f,0.398100008392333393203443847596645355224609375f,0.827499981820583485614406527020037174224853515625f,-0.893999977767467868261519470252096652984619140625f,-0.9301999829292295629556974745355546474456787109375f,-0.733200004178284903133544503361918032169342041015625f,-0.576400007760524690780812306911684572696685791015625f,0.482099976027011933865651371888816356658935546875f,-0.07550000917911514619618174037896096706390380859375f,-0.47519998985826905624207938672043383121490478515625f,-0.08829999787807452094057225622236728668212890625f,-1.575899984574316903263024869374930858612060546875f,-1.605499985039234189798662555404007434844970703125f,-0.05430005319118436091230250895023345947265625f,0.0031999931514263835197198204696178436279296875f,0.071400002694129938163314363919198513031005859375f,0.58960000880956631164053760585375130176544189453125f,-1.123000000685453247939449283876456320285797119140625f,0.26179998956322680925268286955542862415313720703125f,-0.1331000086903575407859534607268869876861572265625f,0.48679999492764469692929196753539144992828369140625f,-1.2621000254988672395484172739088535308837890625f,-0.0820000131726263958853451185859739780426025390625f,-0.464000019192695845049456693232059478759765625f,-0.73160000365376465225608626496978104114532470703125f,0.504500003188848467061689007095992565155029296875f,-0.41579999683499335727532297823927365243434906005859375f,1.021899990713595940405866713263094425201416015625f,-0.6425999999582767685524231637828052043914794921875f,0.422399981527030288219748399569652974605560302734375f,-0.77599998408555992401858247831114567816257476806640625f,0.3102000115394591972517446265555918216705322265625f,0.85519998054504409168430356658063828945159912109375f,0.21019999812841394515317006153054535388946533203125f,0.5295999922692775641053231083787977695465087890625f,0.29090000662803650044452297152020037174224853515625f,-0.309800005000829603574175052926875650882720947265625f,1.06330002052783978427896727225743234157562255859375f,-0.164799999338388403202770859934389591217041015625f,0.564800027352571731853458913974463939666748046875f,0.29000000193715091967305852449499070644378662109375f,0.92730000336170181896022768341936171054840087890625f,-0.30540000028014180344371197861619293689727783203125f,-0.6922000118970874638080204022116959095001220703125f,0.421399985408783095408580265939235687255859375f,1.012800006580352629725894075818359851837158203125f,0.4713999979257579298064229078590869903564453125f,0.883499988257884893982918583787977695465087890625f,1.196999986499548196405839917133562266826629638671875f,0.994799978327751688311764155514538288116455078125f,0.7928999983429907416621063021011650562286376953125f,0.20340000662803703335157479159533977508544921875f,0.65609998508691802499015466310083866119384765625f,-1.157999987721442636257052072323858737945556640625f,-0.380999998092651281922371708787977695465087890625f,-0.906899997121095591268158386810682713985443115234375f,-0.763299994301795958762113514239899814128875732421875f,-1.59389999315738606355807860381901264190673828125f,-1.667500000298023365985500277020037174224853515625f,-1.573799957250058501045941738993860781192779541015625f,-1.191799972054362388718118381802923977375030517578125f,1.395599954152107358140710857696831226348876953125f,0.5185999800205234322447722661308944225311279296875f,0.8309999819993976188925444148480892181396484375f,0.23239998797178262890383848571218550205230712890625f,-0.6207000135660170059281881549395620822906494140625f,-0.43210000255107861022452198085375130176544189453125f,-0.6402000151157380969380028545856475830078125f,-0.92029999883174884445224961382336914539337158203125f,0.37859997920095889156755220028571784496307373046875f,-0.001200009930133916213890188373625278472900390625f,-0.872100006723403708974728942848742008209228515625f,-1.02809999468326562777065191767178475856781005859375f,1.1876999692201621883214102126657962799072265625f,0.3372000081717967390204648836515843868255615234375f,-1.07820003681182896571044693700969219207763671875f,-0.085000007897615414975689418497495353221893310546875f,0.828800025236606785483672865666449069976806640625f,0.28300000113248824362699451739899814128875732421875f,0.22050001223385340409777199965901672840118408203125f,0.3664000005334615561736200106679461896419525146484375f,-0.58999999552965132210147203295491635799407958984375f,0.203399997389316578022544490522705018520355224609375f,-0.14440001574754735003125460934825241565704345703125f,-0.34879999796748162310677798814140260219573974609375f,-0.770400024867058164090849459171295166015625f,-0.2180000100731849244084514793939888477325439453125f,0.27280002237558420574714546091854572296142578125f,-0.7253999920845031201821484501124359667301177978515625f,0.8564000193834306884355100919492542743682861328125f,-0.298999990433454598814932978712022304534912109375f,1.1616000154256820309228714904747903347015380859375f,0.0943999993562698591631487943232059478759765625f,0.3230000110417605707624488786677829921245574951171875f,-0.409199999588727880706073847250081598758697509765625f,0.127599986696242950756641221232712268829345703125f,0.16719999788999562184699243516661226749420166015625f,0.553500027000904282203919137828052043914794921875f,-0.00700000557303415149590364308096468448638916015625f,1.0302000031948086444799628225155174732208251953125f,0.127600001299381204944438650272786617279052734375f,0.2207000037312507600972821819595992565155029296875f,-0.306400001651048636830410032416693866252899169921875f,0.773000040769577623223085538484156131744384765625f,0.1405000094771384766545452293939888477325439453125f,0.0609000251054769847769421176053583621978759765625f,0.3073999918460845748313658987171947956085205078125f,0.2762999771058556319758281460963189601898193359375f,-0.20780000300407408531100372783839702606201171875f,0.680999979801475863183668479905463755130767822265625f,0.4685000021159648753155124722979962825775146484375f,0.4377000133275981141878219204954802989959716796875f,0.29339998368024833297340592253021895885467529296875f,-0.581399986004829116836845059879124164581298828125f,-0.24859999104738239594780679908581078052520751953125f,-0.197399995124339255880840937606990337371826171875f,0.17839998636245724128457368351519107818603515625f,-1.42709998019933781421286766999401152133941650390625f,-1.01459999837875347594717823085375130176544189453125f,-0.87689999041557342707164934836328029632568359375f,-1.0800000035762788996862582280300557613372802734375f,-0.18720002262592316810696502216160297393798828125f,-0.6791999949693678928497320157475769519805908203125f,-0.7752000075161458170214245910756289958953857421875f,-0.4583999953776596303356427597464062273502349853515625f,0.422399980521202422778515028767287731170654296875f,0.52800002527236955529588158242404460906982421875f,-0.014100042247772393011473468504846096038818359375f,0.09130000199079546518987626768648624420166015625f,-0.04860002114772754566729418002068996429443359375f,-0.4517999917984010238569680950604379177093505859375f,-0.6582000001549719314652975299395620822906494140625f,-0.562500007450580596923828125f,-0.986999996781349864249932579696178436279296875f,-0.06959999964535235061902085362817160785198211669921875f,0.68079999907016741644838475622236728668212890625f,-0.133399994260072762841673466027714312076568603515625f,1.053300012779235572679681354202330112457275390625f,0.0249999912083149755659405855112709105014801025390625f,0.05910000231266021586407077847979962825775146484375f,-0.4393999887764454825855864328332245349884033203125f,-0.023999995946883956321471487171947956085205078125f,0.5574000030219554702171080862171947956085205078125f,0.113200016099214817444362779497168958187103271484375f,0.1083999953031540286474410095252096652984619140625f,-0.6780999848484992043040620046667754650115966796875f,0.57759998550415048157447017729282379150390625f,-0.71789999446868879573457888909615576267242431640625f,-0.01279999987483027013013270334340631961822509765625f,0.4048000307798389485469670034945011138916015625f,0.58720000161528584925463292165659368038177490234375f,0.5137000075459479120354444603435695171356201171875f,-0.67580001412034018670738078071735799312591552734375f,1.043000022292137174417803180404007434844970703125f,-0.10160000469684593582542220246978104114532470703125f,0.3613000037789344531802271376363933086395263671875f,-0.180599995875358576569169599679298698902130126953125f,-0.674200027155876568940584547817707061767578125f,-0.29500000834465023302755071199499070644378662109375f,0.54430001713037512445225729607045650482177734375f,0.5702000020027160331892446265555918216705322265625f,-0.2884000209331514241739569115452468395233154296875f,0.372800009709596480433901888318359851837158203125f,-1.17960001238584499816397510585375130176544189453125f,-0.009099993874877643218468392660724930465221405029296875f,-0.4760999949216841997667870600707828998565673828125f,-0.6615999891996384718595436424948275089263916015625f,-0.19410004552602710958808529539965093135833740234375f,-0.44210000225305545740184243186376988887786865234375f,-0.70589998993873592780801118351519107818603515625f,-0.19509998967647579348749786731787025928497314453125f,0.998100002431868915664381347596645355224609375f,0.541200009036064244583030813373625278472900390625f,0.055799995493888587816400104202330112457275390625f,-0.256000020891427926272854165290482342243194580078125f,1.38239995548725147500590537674725055694580078125f,1.4040999740004540541349342674948275089263916015625f,0.24900000101327890433822176419198513031005859375f,0.60769998590946183725236551254056394100189208984375f,0.965399977779387796772425645031034946441650390625f,0.411399991369247430839095613919198513031005859375f,-1.452900019502639139545863145031034946441650390625f,-1.2389000238537786202641655108891427516937255859375f,-0.478800031495094202682594186626374721527099609375f,-0.728600005650520188282825984060764312744140625f,-0.3387000066518783825131322373636066913604736328125f,-0.152299985182285269047497422434389591217041015625f,-0.35189999309778130509585025720298290252685546875f,0.074799992036819418217419297434389591217041015625f,0.819899989759921510312778991647064685821533203125f,0.1018999996542930830401019193232059478759765625f,0.7218000269651412281746161170303821563720703125f,0.4618000162363049554414828890003263950347900390625f,0.46050004553794821049450547434389591217041015625f,-0.271399994945526135126101507921703159809112548828125f,-0.8952000014066694433267912245355546474456787109375f,0.69840000931024537322855394450016319751739501953125f,-0.404900003096461347240619943477213382720947265625f,0.029799988311529246320930042202235199511051177978515625f,-0.4219000425696375344841726473532617092132568359375f,-0.01779998944401750549104690435342490673065185546875f,0.461300016575306781607679340595495887100696563720703125f,0.06199999968707559094838188684661872684955596923828125f,-0.07039999715089795273570416611619293689727783203125f,0.18359998970627788850151773658581078052520751953125f,-0.286299969804286291719108703546226024627685546875f,-0.064200002121925336950880591757595539093017578125f,-1.10420004146099159925142885185778141021728515625f,0.61459999226927752857818632037378847599029541015625f,-0.93160002794265750480917631648480892181396484375f,-0.1549999918043614144380626385100185871124267578125f,0.39339998368024797770203804248012602329254150390625f,-0.252599998915195467219518832280300557613372802734375f,9.999712705610619423168827779591083526611328125e-05f,0.128400000965595229018845202517695724964141845703125f,-0.1688999863028524117680717608891427516937255859375f,0.20599999257922174678014926030300557613372802734375f,1.198100037896633551781633286736905574798583984375f,-0.552999994128942518045732867904007434844970703125f,0.74710002222061167032052253489382565021514892578125f,0.072999997407197980692217242904007434844970703125f,-0.420099987143277786572070908732712268829345703125f,0.67880000169277188160066316413576714694499969482421875f,0.6761000178694729356720927171409130096435546875f,0.2632000109583139124680428722058422863483428955078125f,1.644000008463859074936408433131873607635498046875f,1.6076000075578686931976335472427308559417724609375f,0.1250999904960397035580399460741318762302398681640625f,0.1528999800145627308012308276374824345111846923828125f,-0.066299940299987980552032240666449069976806640625f,-0.1908999687790871035986128845252096652984619140625f,-0.086400008319318377658646568306721746921539306640625f,-0.400700014441460472536249426411814056336879730224609375f,0.187199996399879609043637174181640148162841796875f,-0.22540002814531323593882916611619293689727783203125f,-0.20609998896122050382473389618098735809326171875f,-0.368300039899349229699510033242404460906982421875f,1.567499990463257830697330064140260219573974609375f,1.690600008392333819529085303656756877899169921875f,-0.0476999543190004260395653545856475830078125f,0.31240001509189596617943607270717620849609375f,-0.6398999879717823802138809696771204471588134765625f,-0.69399998730421064152551480219699442386627197265625f,0.215400003260374006686106440611183643341064453125f,0.08550000321865081076566639239899814128875732421875f,0.298799994540214441940406686626374721527099609375f,-0.0237000313878059643002416123636066913604736328125f,-1.47510000100135840739312698133289813995361328125f,-1.662500019967556141864406527020037174224853515625f,-0.07410002615451816154745756648480892181396484375f,-0.35750000834465023302755071199499070644378662109375f,1.388999994307755780909019449609331786632537841796875f,1.067299996805191053539374479441903531551361083984375f,-1.3034999760389336387333969469182193279266357421875f,0.365799999964237176897086101234890520572662353515625f,0.165900008863210768339513379032723605632781982421875f,-0.386600003951787840605902601964771747589111328125f,-0.568300013077259080773728783242404460906982421875f,-0.4131999989628791780660321819595992565155029296875f,0.12050000709295272116605701739899814128875732421875f,0.0775999998092651299685940102790482342243194580078125f,-0.4314999747872345636778845801018178462982177734375f,-0.013799999457597689200838431133888661861419677734375f,-0.6891000115513801205224808654747903347015380859375f,0.61500001251697522519634730997495353221893310546875f,-0.5847999845862386081307704444043338298797607421875f,0.5449999903142452861715128165087662637233734130859375f,-0.90140003875494034701887358096428215503692626953125f,0.117200008022785073791283139144070446491241455078125f,-0.508300005105137842065232689492404460906982421875f,-0.092600000628829004511999301030300557613372802734375f,0.998300049436092962196198641322553157806396484375f,0.00540000221729286522531765513122081756591796875f,0.77040001320689921460171945000183768570423126220703125f,0.0200000007450580330470302214962430298328399658203125f,-1.385600021058321029698845450184307992458343505859375f,0.1937999996066093189028833876363933086395263671875f,-0.54230000723600380041489188442938029766082763671875f,-0.11859999581575397797905679908581078052520751953125f,1.0442000484645372893055537133477628231048583984375f,0.34880001301765428678436364862136542797088623046875f,-0.30130000407695778363859062665142118930816650390625f,0.556399999862909222514417706406675279140472412109375f,0.19449999260902384179416912957094609737396240234375f,0.01019999783039082785762730054557323455810546875f,-0.69030001224279402549655060283839702606201171875f,-0.342300000232457879434377900906838476657867431640625f,-0.61019997696876515647090855054557323455810546875f,-0.150500001728534726908037555404007434844970703125f,0.17159998621940619756287560448981821537017822265625f,0.188699998903274490658077411353588104248046875f,0.0045000134706487671110153314657509326934814453125f,-0.012500021755695200909030972979962825775146484375f,1.371900022006034447485944838263094425201416015625f,1.2241000157237049705827303114347159862518310546875f,1.19669997559785823426636852673254907131195068359375f,1.03679997689723979448217505705542862415313720703125f,0.4854000379800798015139662311412394046783447265625f,0.71100002342462520488197696977294981479644775390625f,0.029100003318488720793766333372332155704498291015625f,0.355099996680021234851665212772786617279052734375f,-0.18090003523826680975616909563541412353515625f,-0.337700002300739043903377023525536060333251953125f,-0.287400005257129720348530099727213382720947265625f,-0.73300000809878118612772368578589521348476409912109375f,0.8169000181317329634111956693232059478759765625f,1.234600000315904555492352301371283829212188720703125f,-0.0620999797523023033818390103988349437713623046875f,0.19970000057220449463102340814657509326934814453125f,0.28259999139010805180305396788753569126129150390625f,-0.4124000106215479632965070777572691440582275390625f,1.0607999996662140773651117342524230480194091796875f,0.66749999046325658724754248396493494510650634765625f,-0.315599993491172625681429053656756877899169921875f,-0.44659999039173126078594577847979962825775146484375f,-0.0927999901145695194060181165696121752262115478515625f,0.32479999099373824122949372394941747188568115234375f,-0.759000016957521506100192709709517657756805419921875f,0.06039999729990963572845430462621152400970458984375f,1.004500015705823923184425439103506505489349365234375f,-0.439800001126527728700921215931884944438934326171875f,-0.1735000077784061289776218472979962825775146484375f,-0.41399999833106992941367252569762058556079864501953125f,-0.7105000357031823199349673814140260219573974609375f,-0.55959998690485956984730364638380706310272216796875f,-1.5697000211358069776679258211515843868255615234375f,-0.11540000832676877706717277760617434978485107421875f,0.0385999951452016265562861008220352232456207275390625f,-0.425999983340501930939581143320538103580474853515625f,0.84980000455379478552231375942938029766082763671875f,0.18919999929070474564696269226260483264923095703125f,0.0035000022649764872539890347979962825775146484375f,0.155399998492002477945561622618697583675384521484375f,-0.89370000288933504339894398071919567883014678955078125f,0.3375999863982201976142505372990854084491729736328125f,0.36500001255422842572073705014190636575222015380859375f,0.1670000045299528945719202965847216546535491943359375f,0.16800001096725480920213158242404460906982421875f,-0.460000005215406293501700929482467472553253173828125f,0.01260002484321631754937698133289813995361328125f,-0.532000004529952885690136099583469331264495849609375f,-0.7365000182986261023643237422220408916473388671875f,-0.145600003176927561554521162179298698902130126953125f,0.6176000031620263452936114845215342938899993896484375f,-1.1216999976515769077423101407475769519805908203125f,0.55019998576045026084102573804557323455810546875f,0.2137999810546638013164510994101874530315399169921875f,0.378000008285045652201006305404007434844970703125f,0.666000007778406288849737393320538103580474853515625f,-1.1253000128388404021961832768283784389495849609375f,-1.180100002044439211346116280765272676944732666015625f,-0.740999981105327609753885553800500929355621337890625f,-0.436499997735023459455305783194489777088165283203125f,-0.761999947309494274350072373636066913604736328125f,-0.0144999583363532025259701185859739780426025390625f,0.257700018692016641352893202565610408782958984375f,0.8824999907612802729772738530300557613372802734375f,-0.6647999787747860267472788109444081783294677734375f,-0.1383999782413247725543214983190409839153289794921875f,0.0407999890863894876247286447323858737945556640625f,-0.48279999138116824752842148882336914539337158203125f,0.279899993783235601085834787227213382720947265625f,0.157299995315074880863903672434389591217041015625f,0.88139998742043967894232991966418921947479248046875f,0.93189998486489089568607369074015878140926361083984375f,0.08730000537335858012966127716936171054840087890625f,-0.35540001477152138864568087228690274059772491455078125f,-0.8850000071525574441011485760100185871124267578125f,-0.5890000066757199448375104111619293689727783203125f,-0.9300000125169756159948519780300557613372802734375f,-0.4558000181436536735191111802123486995697021484375f,-0.0593999865412708771827965392731130123138427734375f,0.278699995028972580257686786353588104248046875f,-0.2642999993979930906107256305404007434844970703125f,-0.056199993389844937752286568866111338138580322265625f,-0.95959999834150055963988279472687281668186187744140625f,0.2124000040650366738503862507059238851070404052734375f,-0.0936000063210726818141438343445770442485809326171875f,-0.210000008046626884805618828977458178997039794921875f,0.823999992966651717551940237171947956085205078125f,-0.45599999347329145393814542330801486968994140625f,0.5170000045299527613451573415659368038177490234375f,0.221399988538026892825882896431721746921539306640625f,-0.011199994954466774288448505103588104248046875f,0.199399998015165336173737387071014381945133209228515625f,0.1544999887347220379751888685859739780426025390625f,-0.0711999784886838682496090768836438655853271484375f,0.1070999819874760561333459918387234210968017578125f,-0.12800000366568564658109607989899814128875732421875f,-0.4124999777972693237870771554298698902130126953125f,0.78460000135898588791150132237817160785198211669921875f,1.2837000218510628002377416123636066913604736328125f,0.261799996119737699018514831550419330596923828125f,-0.06960002013444910318185065989382565021514892578125f,0.01460000001788142043324114638380706310272216796875f,-0.8753999998331067899925983510911464691162109375f,0.14479998846054087380252894945442676544189453125f,0.4345999956220385485039514605887234210968017578125f,0.009799999934434866499799454686581157147884368896484375f,-0.2672000130891802882615593262016773223876953125f,0.733799991708993903927193969138897955417633056640625f,0.6534999864697450533412848017178475856781005859375f,-0.017400001531839404123047643224708735942840576171875f,-0.2455999836564064509047966566868126392364501953125f,-0.2464999902844429158221828402020037174224853515625f,0.69240001322925071036706867744214832782745361328125f,0.89459998522847861135431912771309725940227508544921875f,-0.2466000164687631723836602759547531604766845703125f,0.1356000120431188893377338899881578981876373291015625f,-1.1753999819517133573754108510911464691162109375f,-1.287799989891052376833613379858434200286865234375f,0.2696999639153485617271144292317330837249755859375f,-0.038600008034705979298450984060764312744140625f,0.3746999912589787928851592369028367102146148681640625f,0.2419999957382679145467818671022541821002960205078125f,0.00060002210140197576038190163671970367431640625f,0.3137000456929204261768973083235323429107666015625f,-0.374999989271163514104046043939888477325439453125f,-0.5093000004410743741800615680404007434844970703125f,1.590299988847970080740878984215669333934783935546875f,1.205099993848800732365589283290319144725799560546875f,0.10829996390342699896791600622236728668212890625f,-0.5044999940991399256517979665659368038177490234375f,-0.07169997619390500176450586877763271331787109375f,-0.26959998615980129699210010585375130176544189453125f,0.44520000021457661887325230054557323455810546875f,0.4945999751329424043433391489088535308837890625f,-0.0549000135272736944358484834083355963230133056640625f,-0.1105999924480915996838348291930742561817169189453125f,1.4018999832630161250790479243732988834381103515625f,0.9123999960184097091087096487171947956085205078125f,-0.1413000207662573615152723505161702632904052734375f,-0.20109998687505736825187341310083866119384765625f,1.0511999525606636751717815059237182140350341796875f,0.725000008642673332559525078977458178997039794921875f,-0.7424000091314315596946471487171947956085205078125f,0.61580000041127203047608418273739516735076904296875f,-1.026799985837936191046537715010344982147216796875f,0.6587999844074250432868211646564304828643798828125f,0.0431000164389612194781875587068498134613037109375f,0.4428000006198882232411051518283784389495849609375f,-0.3909000090122223269872847595252096652984619140625f,-0.0940000023543834384298634176957421004772186279296875f,-0.631600019598007378363035968504846096038818359375f,0.39400000146031377568078823969699442386627197265625f,-0.31289999148845648591077406308613717555999755859375f,-0.04759998803734788452857173979282379150390625f,-1.3584000435829164388223944115452468395233154296875f,-0.11039999431967739695892305462621152400970458984375f,0.31140001290142560463891641120426356792449951171875f,0.178400010204315118134132944760494865477085113525390625f,0.2638000005751848231483336348901502788066864013671875f,-0.139599993461370530667409184388816356658935546875f,-0.2587999826192852736994609585963189601898193359375f,-0.53499999925494190478048039949499070644378662109375f,-0.5151999987244604284342130995355546474456787109375f,0.16220000176429749672024627216160297393798828125f,1.1517000268578527766294428147375583648681640625f,-0.079800020498037138594327188911847770214080810546875f,-1.2475000557303435044786965590901672840118408203125f,-0.11180000416636470816911241854541003704071044921875f,0.656400014615059035349986515939235687255859375f,-0.3566000008225440609521683654747903347015380859375f,-0.2226000164985659779404159053228795528411865234375f,1.3969000074028965485695152892731130123138427734375f,1.427699998426437133502986398525536060333251953125f,1.3656000107765198237075310316868126392364501953125f,1.62779996008872984702975372783839702606201171875f,1.695999973058700760475403512828052043914794921875f,0.7253999938726423124535358510911464691162109375f,0.2544999791979787318041417165659368038177490234375f,-0.06119999294280997759187812334857881069183349609375f,0.13229999948740012172265778644941747188568115234375f,1.0044000032305717695635394193232059478759765625f,1.09660001959800723625448881648480892181396484375f,0.13920000927448228367211413569748401641845703125f,0.51710001089572887877920948085375130176544189453125f,0.177000051021575899312665569595992565155029296875f,0.6782000219106674165914228069595992565155029296875f,0.2784000035732978606262122411862947046756744384765625f,0.1186000138461589248350946945720352232456207275390625f,-0.346799989563226773725546081550419330596923828125f,-0.3559999939948321756588711650692857801914215087890625f,-0.839699999976158295567074674181640148162841796875f,-0.2440000150203704976092922152020037174224853515625f,0.5283000013053413823627124656923115253448486328125f,0.7222000059366227020518635981716215610504150390625f,-0.4521000669240944347393451607786118984222412109375f,-0.7213000258326527358576640835963189601898193359375f,0.107100034737587890276699909009039402008056640625f,0.5107000067114828567582662799395620822906494140625f,-0.92790000162124641747141140513122081756591796875f,-1.02900000965595239676986238919198513031005859375f,-0.769199986028670679161223233677446842193603515625f,0.03639999345541011788185414843610487878322601318359375f,0.1833000059247016366725802072323858737945556640625f,-0.423800009888410489367061018128879368305206298828125f,0.79180002696514151239171042107045650482177734375f,-0.066200003969669296566280536353588104248046875f,0.619800010961294223221784704946912825107574462890625f,-0.242600002938508918504112443770281970500946044921875f,0.05810001882314708154808613471686840057373046875f,-0.186600000971555601836371351964771747589111328125f,0.3626000062167644699684387887828052043914794921875f,-0.01599999868869783625768832280300557613372802734375f,-0.909100023174286064886473468504846096038818359375f,-0.232000002294778795430829632095992565155029296875f,0.7434000048398967663842995534650981426239013671875f,0.55799998846650122885648670489899814128875732421875f,0.29770001202374707371944850820000283420085906982421875f,0.1582000007510184769188299469533376395702362060546875f,-0.651900007998943209486242267303168773651123046875f,-0.19639999613761904839748240192420780658721923828125f,0.9036999860882755086777251563034951686859130859375f,-0.5780000020265578797307171043939888477325439453125f,-0.2493999840080736163372421287931501865386962890625f,0.19360000073313710089450978557579219341278076171875f,-0.5926000212669375599716659053228795528411865234375f,0.67940000129342070778193374280817806720733642578125f,0.9010999922394749006571146310307085514068603515625f,0.197000002145767183492353069595992565155029296875f,0.12500000450760129222516070512938313186168670654296875f,0.0767999977588653681692676400416530668735504150390625f,0.231300001543760291866647094138897955417633056640625f,-0.39339997414350502680235877051018178462982177734375f,-0.04700000266730786080415782635100185871124267578125f,-0.2617000025689601994116628702613525092601776123046875f,-0.240599992634356052434441153309307992458343505859375f,0.482699990342557450073712743687792681157588958740234375f,-0.01959999524950983840199114638380706310272216796875f,0.62679999731183055899919054354541003704071044921875f,0.01169999906718732773924784851260483264923095703125f,-0.09279999808669092686841395334340631961822509765625f,0.08359999760389325018650197307579219341278076171875f,-0.553400010055303592793052303022705018520355224609375f,0.0744000060990452904974290504469536244869232177734375f,0.483700003708899040066881980237667448818683624267578125f,0.0327000002890824870149799608043394982814788818359375f,-0.21320000626444812752424695645458996295928955078125f,0.23039999715089798382194885562057606875896453857421875f,0.02380000213980670853430865463451482355594635009765625f,0.013900001353025437378363449170137755572795867919921875f,-0.068100000718235964569657880929298698902130126953125f,0.1126000006288290222755676950328052043914794921875f,-0.004899993932247159733606167719699442386627197265625f,-0.157599994519352915034460238530300557613372802734375f,0.3366999980986118412573659952613525092601776123046875f,-0.2535000022649764872539890347979962825775146484375f,-0.616500002950429948356259046704508364200592041015625f,0.13740000294744969533411449447157792747020721435546875f,-0.057299988758563902280229740426875650882720947265625f,-0.081000001370906904440971629810519516468048095703125f,-0.647200023967027693316822478664107620716094970703125f,-0.0261999985307455052208069901098497211933135986328125f,-0.132399980223179003502309569739736616611480712890625f,0.1771000041157007043324256301275454461574554443359375f,0.05950000005960465188081798260100185871124267578125f,0.34679999589622045874648392782546579837799072265625f,-0.274099998214840656629576187697239220142364501953125f,0.71610002084970414415465711499564349651336669921875f,1.063400021678209128594971843995153903961181640625f,-0.2996000048607587729065926396287977695465087890625f,0.3185000127702952710251338430680334568023681640625f,-0.6779999471902851126969835604541003704071044921875f,-0.03499999880790749529069216805510222911834716796875f,0.01699998337030450556994765065610408782958984375f,0.269799992334842642094372422434389591217041015625f,-0.1955999982595439945498583256267011165618896484375f,0.091800008636713226195524839567951858043670654296875f,-0.0173999994456768281025915712234564125537872314453125f,0.25640000381171701437921228716732002794742584228515625f,0.1818999999523160937542343162931501865386962890625f,-0.95390000686645493033211096189916133880615234375f,-0.164800012376904447819470078684389591217041015625f,-0.430500006273388802213730741641484200954437255859375f,-0.49299998533725730709420531638897955417633056640625f,-0.1369999905228613812369076185859739780426025390625f,0.48660000633597366714866438996978104114532470703125f,-0.07490001494288467398519060225225985050201416015625f,-0.0219000148534771454933434142731130123138427734375f,-0.70920001940727228628702505375258624553680419921875f,-0.3869999867975713048195984811172820627689361572265625f,0.69030000650584677490684271106147207319736480712890625f,0.4047999845862388923478647484444081783294677734375f,-0.1013000094413758489508836646564304828643798828125f,0.10289999789595605239611586512182839214801788330078125f,-0.44659999597966670847881687222979962825775146484375f,0.04480000105202197602238811668939888477325439453125f,-0.49660000104606150106434370172792114317417144775390625f,-0.051700006964802813769210843020118772983551025390625f,-0.774200028496980774406210912275128066539764404296875f,-0.0199999988824128838160731902462430298328399658203125f,0.38220000675618648333653482040972448885440826416015625f,0.427599995040893521291991419275291264057159423828125f,-0.18160000559091571403769194148480892181396484375f,-0.18179999328851703666032335604541003704071044921875f,0.383399990534782464379759403527714312076568603515625f,0.1092000041335821070020983825088478624820709228515625f,-0.024900000041723213684008442214690148830413818359375f,-0.008999999970197680720929156450438313186168670654296875f,0.0285999985724687544319522203295491635799407958984375f,0.09009999504089361010983338928781449794769287109375f,-0.3736999844491482036801244248636066913604736328125f,-0.11399999840557573094201870844699442386627197265625f,0.47560000638067723155444355143117718398571014404296875f,0.181000001072883609509744928800500929355621337890625f,0.171000003159046176648416803800500929355621337890625f,0.022599997574090924246092981775291264057159423828125f,-0.6196000091075897220349588678800500929355621337890625f,-0.027299996805191018012237691436894237995147705078125f,0.122700010049343166684820971568115055561065673828125f,0.100999999433755871525164593549561686813831329345703125f,0.1896000000923871997571268366300500929355621337890625f,0.061199996966123659802860856871120631694793701171875f,0.010799999442696606166691708494909107685089111328125f,0.085700026977061583011163747869431972503662109375f,0.89170000778436619981448529870249330997467041015625f,-0.714399964785576546688616872415877878665924072265625f,-0.555299974840879340121091445325873792171478271484375f,0.281200004267692094117592205293476581573486328125f,0.021799998056888370001615840010344982147216796875f,-0.8649999856948848986348821199499070644378662109375f,0.1759000201880933644815740990452468395233154296875f,-0.731799998652934657883406543987803161144256591796875f,-0.87120001022815696245515937334857881069183349609375f,-0.34390000463128078678209931240417063236236572265625f,0.2627000174999238879536278545856475830078125f,0.3607999837219717509384508957737125456333160400390625f,-0.35300000225007510668007171261706389486789703369140625f,0.494800000977516418743107351474463939666748046875f,0.6432999920666215842857127427123486995697021484375f,0.266399997106194330598327724146656692028045654296875f,-0.4895000125020743109161003303597681224346160888671875f,-0.3795999870538711018497224358725361526012420654296875f,-0.98979999658167339138259421815746463835239410400390625f,-0.04209999480247450520664642681367695331573486328125f,-0.8037000214040279644223119248636066913604736328125f,1.0982999823808672346103776362724602222442626953125f,0.34850000733137154185214967583306133747100830078125f,0.31419997321367265641356425476260483264923095703125f,0.042099993312358918728932621888816356658935546875f,0.058700000244378880420725863586994819343090057373046875f,0.9446000296153129749399113279650919139385223388671875f,-0.38590000677704860976291456609033048152923583984375f,-0.59770000729262828809140728480997495353221893310546875f,0.102299992334842659857940816436894237995147705078125f,-0.378300009351968835602519902749918401241302490234375f,0.177999993681907664466734786401502788066864013671875f,0.4581999997079371222952204334433190524578094482421875f,-0.17160000395178798271444975398480892181396484375f,0.0617999858379363331550848670303821563720703125f,0.10460000739395614033355741412378847599029541015625f,0.767699990081787131401824808563105762004852294921875f,0.0806000057846307793596452029305510222911834716796875f,0.7171000068724155962485156123875640332698822021484375f,-0.1336999954015016811581517686136066913604736328125f,0.6973999908030033001438141582184471189975738525390625f,-0.097599998021125788927321309529361315071582794189453125f,-0.1829999994188546974527298516477458178997039794921875f,0.00400000183284276999984285794198513031005859375f,-0.621500015020370444318587033194489777088165283203125f,-0.03160000305771826223377729547792114317417144775390625f,-0.21669999623596669646730106251197867095470428466796875f,0.166000000178813944540223701551440171897411346435546875f,0.113600004830956446966183648328296840190887451171875f,0.160500003889203046725242529646493494510650634765625f,-0.279200000631809164275409784750081598758697509765625f,-0.227899998193979325122882073628716170787811279296875f,-0.78470002277493478004544158466160297393798828125f,-0.210599999898672063380899999174289405345916748046875f,0.111400009697675717035281195421703159809112548828125f,-0.03030000136494635787354212652644491754472255706787109375f,-0.04540000275745992297515840618871152400970458984375f,-0.110100004503130843858116350020281970500946044921875f,0.192799996559321618061488834428018890321254730224609375f,-0.4543999956309796584008608988369815051555633544921875f,-0.131699996832013166425667805015109479427337646484375f,-0.532899972414970335421458003111183643341064453125f,0.21630003089904814572719260468147695064544677734375f,-0.334999985992908477783203125f,0.34409999709725358485457036294974386692047119140625f,-0.362899973905086881131865084171295166015625f,0.3374000264167786866664755507372319698333740234375f,0.0018000040173529185949519160203635692596435546875f,-1.05490002105236069240845608874224126338958740234375f,0.8055999481916436000261683147982694208621978759765625f,0.29619998213946840959209794164053164422512054443359375f,0.258399998879432768461583691532723605632781982421875f,-0.17610001175999645539604898658581078052520751953125f,-0.2648000066399571750253016944043338298797607421875f,0.62770001869201674793430356658063828945159912109375f,-1.2204999629855155518498577293939888477325439453125f,-0.1385999961137773794916938641108572483062744140625f,0.10790000184476344902151367932674475014209747314453125f,-0.1185999984607100099509580104495398700237274169921875f,-0.0772000372290602143721116590313613414764404296875f,0.95029999972581880030020329286344349384307861328125f,0.59639999136924704004059094586409628391265869140625f,0.9633000244021416857975737002561800181865692138671875f,-0.5870999684274200713929303674376569688320159912109375f,-0.70249998666346069597921086824499070644378662109375f,-0.31389996842145961863934644497931003570556640625f,-0.82740000153183945741375282523222267627716064453125f,-0.166299974721670740240853092473116703331470489501953125f,-0.2527000131413341676989148254506289958953857421875f,-0.03319999918639660041463912421022541821002960205078125f,0.61609999954104421870937358107767067849636077880859375f,0.338499999880790714001932428800500929355621337890625f,-0.1600999981701374252907044137828052043914794921875f,0.165799999740719794516508045489899814128875732421875f,-0.0030999970674514543134137056767940521240234375f,0.29609999417662624665581461158581078052520751953125f,-0.26849998974800115547623136080801486968994140625f,-0.067000000804662729336769189103506505489349365234375f,0.20259999399781225548622387577779591083526611328125f,-0.286399991667270648054710591168259270489215850830078125f,-0.304399994811415552931066486053168773651123046875f,0.1123999982535839325947790712234564125537872314453125f,-0.5523999908030032823802457642159424722194671630859375f,-0.171599993222951940907705647987313568592071533203125f,0.673599983447790151558365323580801486968994140625f,0.273700003522634460750850848853588104248046875f,0.81309999289512635556320674368180334568023681640625f,0.120499993681907724418778116159955970942974090576171875f,-0.396999980911612537770594144603819586336612701416015625f,-0.207999987423419963050719161401502788066864013671875f,0.5210000003278256741623408743180334568023681640625f,-0.19600000435113906771533720529987476766109466552734375f,-0.2982000082015991981876368299708701670169830322265625f,0.14789999692738053926888142086681909859180450439453125f,0.38479999248385421406482009842875413596630096435546875f,0.280699993523955360075206044712103903293609619140625f,-0.2341999945223331991428494802676141262054443359375f,0.09069999315142630857966565827155136503279209136962890625f,0.082300022286176943708824182976968586444854736328125f,0.8322000044465056589615414850413799285888671875f,0.3627000063240526372965177870355546474456787109375f,-0.484600007617473238497041165828704833984375f,-0.93310000079274146855823346413671970367431640625f,-0.312600000703334490737006490235216915607452392578125f,-1.3501000336349011643477524557965807616710662841796875f,-0.3407000013470646848645628779195249080657958984375f,-0.372700014668703172304731197073124349117279052734375f,0.599999996609985597473269081092439591884613037109375f,-0.2544000195845963485652418967220000922679901123046875f,-0.40229999904036528590722809894941747188568115234375f,0.500800009053945505144156413734890520572662353515625f,0.6791999569714075590809443383477628231048583984375f,-0.0048000096388157997839840618325979448854923248291015625f,-1.148799958181381697386314044706523418426513671875f,-0.1063999602258203225346733233891427516937255859375f,0.3492999878309668293496770274941809475421905517578125f,-0.21659999823346709479920946250786073505878448486328125f,0.518699972975254386398091810406185686588287353515625f,1.0094000005483627635527454913244582712650299072265625f,1.2166999899029722342902459786273539066314697265625f,0.0805999812722202335635302006267011165618896484375f,-0.1366999961428343457470191424363292753696441650390625f,-0.32360000458881266194310910577769391238689422607421875f,-0.8947999874174590306807886008755303919315338134765625f,-0.795400013914704384632159417378716170787811279296875f,-0.41359996958971034786145537509582936763763427734375f,-0.44929999343752857754452634253539144992828369140625f,0.499499994546174885812206412083469331264495849609375f,0.48890000030994418267482615192420780658721923828125f,-0.13980001103579997590031780418939888477325439453125f,-0.67470000742673885696376601117663085460662841796875f,-0.00779999443590639884860138408839702606201171875f,0.6323000060439110114884897484444081783294677734375f,0.08420000316500664272467702176072634756565093994140625f,0.23980000872612006190820466144941747188568115234375f,0.0207000042527913930001659537083469331264495849609375f,0.37080000115633016122274057124741375446319580078125f,0.0247999940112233474565073265694081783294677734375f,-0.7490999912485480916046753918635658919811248779296875f,0.0139000004589557857315185174229554831981658935546875f,0.4163000060141086589027281661401502788066864013671875f,-0.243199993300437899979016265206155367195606231689453125f,-0.2402999958328901897175455815158784389495849609375f,0.31919999638497831284666972351260483264923095703125f,0.6231000037729739204905854421667754650115966796875f,0.0054999991394579783587914789677597582340240478515625f,-0.41229999019280072491966393499751575291156768798828125f,-0.17669999668300151451916235600947402417659759521484375f,-0.253400000071525521860849039512686431407928466796875f,-0.2019000142574309819565314683131873607635498046875f,-0.97029999376535425170686721685342490673065185546875f,-0.118099996266514128961944152251817286014556884765625f,0.44679998138621446202023435034789144992828369140625f,0.15160000737905499068602921397541649639606475830078125f,0.74219999863505370374383574016974307596683502197265625f,0.34199999700486660714204845135100185871124267578125f,-0.275899995303154010883872615522705018520355224609375f,-0.11190000211298467558407310207257978618144989013671875f,0.115599998371302969957241657539270818233489990234375f,-0.93690001306533776670448787626810371875762939453125f,-0.4685000059902666436073559452779591083526611328125f,-0.656999980688095064351728069595992565155029296875f,-1.1668000066995620755960771930404007434844970703125f,-0.7988000137627120356142995660775341093540191650390625f,-0.567000022187829078035292695858515799045562744140625f,0.20279998810291299804475784185342490673065185546875f,-0.682600012624263730032225794275291264057159423828125f,1.10959999241828910498952609486877918243408203125f,-0.03400001308321964899050726671703159809112548828125f,-0.2368999888136984477426949524669907987117767333984375f,0.23319999751001574406217287105391733348369598388671875f,-0.0858000135242935613177905906923115253448486328125f,-0.63179999388456309361572493799030780792236328125f,-0.09049997162818979035137090249918401241302490234375f,-0.5908999933660030734472456970252096652984619140625f,0.690899987852573449487181278527714312076568603515625f,0.84410000797510154058045372948981821537017822265625f,0.93349998885393148384537198580801486968994140625f,-0.18019999693632104964535756153054535388946533203125f,0.0365999969482420084432305884547531604766845703125f,0.655599999451637049929786371649242937564849853515625f,-0.559499976217746830542409952613525092601776123046875f,-0.63289999521374706858978242962621152400970458984375f,0.61759996464252520098625609534792602062225341796875f,-0.117400019860267601501391254714690148830413818359375f,-0.797899968540668691474593288148753345012664794921875f,0.1322000528752809156429748327354900538921356201171875f,-0.9471999740481376051093320711515843868255615234375f,-0.064100000859797001684370343355112709105014801025390625f,0.088900005003810012027543052681721746921539306640625f,0.919400029754638747903072726330719888210296630859375f,0.08820000253915782906233289395458996295928955078125f,0.62619999234676360089224544935859739780426025390625f,0.03800000619888310726679492290713824331760406494140625f,0.87860002219080934882100564209395088255405426025390625f,-0.1891999976515770232055047017638571560382843017578125f,0.22839999172687530659686672152020037174224853515625f,-0.190400002291798653430987542378716170787811279296875f,-0.86530002133250238927075770334340631961822509765625f,0.062899998752772801058341656244010664522647857666015625f,0.122799999018013472618093828714336268603801727294921875f,-0.178999993637204202201473890454508364200592041015625f,0.5414999899864196830634455182007513940334320068359375f,0.248899989357590722960367202176712453365325927734375f,-0.435199988591671083071332759573124349117279052734375f,-0.0653000012904405569003074560896493494510650634765625f,-0.7028000024080276464388816748396493494510650634765625f,-0.190700000154972126864549863967113196849822998046875f,-0.65070001922845843722598146996460855007171630859375f,-0.0998000009775161789349340324406512081623077392578125f,0.153799997222423545650826781638897955417633056640625f,0.0059000026047228981695980110089294612407684326171875f,0.6742999779403209270611796455341391265392303466796875f,-0.3335999935805797900201241645845584571361541748046875f,0.044599990928173127713307621888816356658935546875f,0.30789999819397928515485318712308071553707122802734375f,0.2253000031530856261952067143283784389495849609375f,0.100400004899501738009348628111183643341064453125f,0.0064000038862233044056893049855716526508331298828125f,0.390599996918439718029958385159261524677276611328125f,0.36720000459551815907843774766661226749420166015625f,-0.2998999757528313381271800608374178409576416015625f,-1.22470000802278544682621941319666802883148193359375f,0.47299998384714125876371326739899814128875732421875f,0.963299991917609954583667786209844052791595458984375f,0.040899986958503387057817235472612082958221435546875f,0.38340000782012939595233547152020037174224853515625f,-0.061400014019012072452596839866600930690765380859375f,-0.7390000200867652413450059611932374536991119384765625f,-0.784899962639808990161327528767287731170654296875f,-0.0086999969661238907292499789036810398101806640625f,-0.951399959778786108444137425976805388927459716796875f,-0.73409997891783707046897688996978104114532470703125f,-1.044199960994721010365537949837744235992431640625f,-0.41539997181892385214041496510617434978485107421875f,-0.325699998962878822794664301909506320953369140625f,0.7535999986469743561201539705507457256317138671875f,0.75539996526241370844445555121637880802154541015625f,0.165399992382526672685116864158771932125091552734375f,0.5524000006377691729397838571458123624324798583984375f,0.43640002050101767139267394668422639369964599609375f,-0.10230000038146958019069643341936171054840087890625f,-0.921800005358457408277672584517858922481536865234375f,0.3198999939322468843982960606808774173259735107421875f,0.80800003160536315593986955718719400465488433837890625f,1.2870999745368951749924235627986490726470947265625f,0.314300020259619117268812260590493679046630859375f,-0.2750999512314802331047758343629539012908935546875f,-0.3815999993324279415674027404747903347015380859375f,0.0733999953776598434984634877764619886875152587890625f,-0.64469998865127564613430877216160297393798828125f,-0.088100006753206283605095450184307992458343505859375f,0.61349998781085002264035210828296840190887451171875f,-0.0766000017166137414648119374760426580905914306640625f,-0.42679999254345890591366696753539144992828369140625f,-0.1845000026673078163952368413447402417659759521484375f,0.3560000062882899829475036312942393124103546142578125f,-0.02579999601542948273191768748802132904529571533203125f,0.167100008213520112576588871888816356658935546875f,0.24969999751746652094652745290659368038177490234375f,0.3501999957442283317732290015555918216705322265625f,0.1869999984204768761486548100947402417659759521484375f,0.7245999886929987976458278353675268590450286865234375f,0.236199995923042305179251343361102044582366943359375f,0.70540000072717656820486809010617434978485107421875f,-0.06909999270141131688660607323981821537017822265625f,0.5041999828994274679416776052676141262054443359375f,-0.1697999938249588058880590324406512081623077392578125f,-0.50620000069141379839265937334857881069183349609375f,-0.13320000626444816749227584296022541821002960205078125f,-0.36480000857710848105597278845380060374736785888671875f,-0.0236999976366758513535160091123543679714202880859375f,0.219199996310472489113863048260100185871124267578125f,-0.141100002521276468581135077329236082732677459716796875f,-0.091700006070733142138351468020118772983551025390625f,-0.26630000966489308922291456838138401508331298828125f,-0.91679999448061000322240943205542862415313720703125f,0.281099993208050680237874985323287546634674072265625f,0.5021000073194503698914559208787977695465087890625f,-0.4642000021219256922222484718076884746551513671875f,-0.94320003144741093592529068700969219207763671875f,-0.7403999832928178648216999135911464691162109375f,0.3914000196814537702039160649292171001434326171875f,-0.7632999737381940263958313153125345706939697265625f,-0.5845999882459640417664559208787977695465087890625f,-0.089899999594688750903515028767287731170654296875f,0.923100032234192013902429607696831226348876953125f,0.5448000042557712419011295423842966556549072265625f,1.4047000330567360748545979731716215610504150390625f,0.5308999978363513339019164050114341080188751220703125f,0.286700000855326653237398204510100185871124267578125f,1.27389998540878224275729735381901264190673828125f,0.2035000015199184275616062222979962825775146484375f,1.0075999725028872422427639321540482342243194580078125f,0.84689999732598664206051353176007978618144989013671875f,-0.59179999477863276524658431299030780792236328125f,-1.51019999902248347467548228451050817966461181640625f,-0.2689999983310702447170115192420780658721923828125f,-0.96610000535249707098728322307579219341278076171875f,0.27850000911951067195104769780300557613372802734375f,0.2891000128924847700773170799948275089263916015625f,0.46309999766349818628441425971686840057373046875f,-0.4622000118970870374823789461515843868255615234375f,-0.2507999859571456990892102112411521375179290771484375f,-0.78100002290308501162741094958619214594364166259765625f,0.6368999942898749822006720933131873607635498046875f,-0.1778999999821184729853484896011650562286376953125f,-1.0875999548077590617367604863829910755157470703125f,0.3999999873340129852294921875f,-0.00700000363588326734998190659098327159881591796875f,0.60700002181529999489839610760100185871124267578125f,0.071600005814433043127564815222285687923431396484375f,0.6864999858140945487861017682007513940334320068359375f,0.1908999948561191928320113220252096652984619140625f,0.4875000101327895407621326739899814128875732421875f,-0.1869000005483627546709612943232059478759765625f,0.4888999864518641658150954754091799259185791015625f,-0.05720000504255295936673064716160297393798828125f,-0.57700001123547561832083374611102044582366943359375f,0.00269999730885028821347759730997495353221893310546875f,-0.64360000192523003637035117208142764866352081298828125f,-0.22750001125037666582784368074499070644378662109375f,-0.98479999948740015724979457445442676544189453125f,-0.1240000044777988807975788176918285898864269256591796875f,-0.9403999872043729002513146042474545538425445556640625f,-0.014599993014335677088411102886311709880828857421875f,0.8017999897122383146097490680404007434844970703125f,-0.066600002163648586162025821977294981479644775390625f,0.515400004452466919957487334613688290119171142578125f,0.056699999514222110263972354005090892314910888671875f,-0.47100001180171968684362582280300557613372802734375f,0.0381000009417533647138043306767940521240234375f,-0.608700004863738985960708305356092751026153564453125f,0.131599999481439577930785844728234224021434783935546875f,0.03100000114738943324255160405300557613372802734375f,-0.02730000284016131928410686668939888477325439453125f,-0.676800001335144063574489337042905390262603759765625f,0.281599993595480935226760266232304275035858154296875f,-0.7986000236809254193559581835870631039142608642578125f,0.22719998030662491572684302809648215770721435546875f,0.75180002145171176408666724455542862415313720703125f,-0.153799991858005302702849803608842194080352783203125f,-0.9493000201106072122314571970491670072078704833984375f,-0.60369998072385777643233950584544800221920013427734375f,-0.6288000045612454069754448937601409852504730224609375f,0.03579997761249575916053800028748810291290283203125f,0.00789998418688764303396965260617434978485107421875f,0.296799996566772250616850215010344982147216796875f,0.61550001364946371751329934340901672840118408203125f,0.17240000495910567934743085061199963092803955078125f,0.800400027847290029825444435118697583675384521484375f,0.891399979448319168540137980016879737377166748046875f,0.87609998508691777630019714706577360630035400390625f,0.7425999866962431639194619492627680301666259765625f,-0.16720003111958536834436017670668661594390869140625f,0.1792000021219255501137013197876513004302978515625f,-0.90120000963211044364697954733856022357940673828125f,0.26660000909268877222757510025985538959503173828125f,0.439700008544325893211635047919116914272308349609375f,-0.67319997661113717413172707892954349517822265625f,-0.22219997017383530391043677809648215770721435546875f,-0.432499964237212708439983543939888477325439453125f,-0.23609998642802221269221263355575501918792724609375f,-0.0129999853372577689469835604541003704071044921875f,-0.83820000253915782906233289395458996295928955078125f,0.478499998390674363690777681767940521240234375f,0.285200000512599860513773819548077881336212158203125f,0.967399982607364705700092599727213382720947265625f,0.08670000942349442851764251827262341976165771484375f,-0.34400000257790086521936245844699442386627197265625f,0.484699997442960661686583989649079740047454833984375f,0.13420000160038469960710472150822170078754425048828125f,-0.08069999851584430672346570645458996295928955078125f,0.2353000010669231423232616862151189707219600677490234375f,0.2005000050440430836573568740277551114559173583984375f,-0.26659999344646934105185209773480892181396484375f,0.56829999161958699716024057124741375446319580078125f,-0.033200003284215906518284100457094609737396240234375f,-0.302399997955560717599610143224708735942840576171875f,-0.1676000071108341149539455727790482342243194580078125f,0.1425999926567076414585244492627680301666259765625f,-0.1117999936610460931518673532991670072078704833984375f,-0.8333000163555144279570185972261242568492889404296875f,-0.113800003704428664974557250388897955417633056640625f,-0.60329999743103979170655293273739516735076904296875f,0.100800000262260436301176014239899814128875732421875f,-0.39759999802112577782509106327779591083526611328125f,0.12500000096857544207296086824499070644378662109375f,-0.62080000972449778107176143748802132904529571533203125f,0.365200001406669638726043558563105762004852294921875f,-0.03519999157190323302302203956060111522674560546875f,0.403700003895163472833473861101083457469940185546875f,-0.40540001294612881821421979111619293689727783203125f,0.174599993163347289026887665386311709880828857421875f,-0.34399998760223393645674150320701301097869873046875f,-0.0877000031948089731059781115618534386157989501953125f,-0.374900007045269045846680455724708735942840576171875f,-0.25260000025630002795651307678781449794769287109375f,0.12400000632181773507767275077640078961849212646484375f,0.37739999125003809155032286071218550205230712890625f,-0.26540000862479207199839947861619293689727783203125f,-0.064299967658520085223017304087989032268524169921875f,-0.66869998549222930961377642233856022357940673828125f,0.09110001786947208302080980502068996429443359375f,0.686700001078843769164450350217521190643310546875f,-0.5022000154733650134630806860513985157012939453125f,-0.82320001699328404498601230443455278873443603515625f,0.27359998762011539241711943759582936763763427734375f,-0.307400018221140047813833007239736616611480712890625f,-1.1686999524116519211247577914036810398101806640625f,-0.32409998413324370147847730549983680248260498046875f,-0.32769998233318364810884304461069405078887939453125f,0.524400000846385605512978145270608365535736083984375f,-0.584399990713595940405866713263094425201416015625f,-1.11710002877712266666776486090384423732757568359375f,1.037700005578994222332767094485461711883544921875f,0.61630001853108407061654361314140260219573974609375f,0.25780001492500304038912872783839702606201171875f,-0.51460000195503230457916288287378847599029541015625f,-0.06949998694658265918633333058096468448638916015625f,0.0994000075519085868336333078332245349884033203125f,0.22320000492334290953522213385440409183502197265625f,0.861500030070543321158993421704508364200592041015625f,-0.29290001085996575369563288404606282711029052734375f,-1.232600028270482983572264856775291264057159423828125f,-0.300000019371509552001953125f,-0.3925000131130218505859375f,-0.25060001405477549951683613471686840057373046875f,-0.0862999995134770825355730039518675766885280609130859375f,-0.01180000275075432369931149878539144992828369140625f,-0.403299991172552108054105701739899814128875732421875f,0.2674999942630529314868681467487476766109466552734375f,-0.5048000039577484887587388584506697952747344970703125f,0.19209999279081824141712786513380706310272216796875f,-0.580399992680549647872112473123706877231597900390625f,0.012600009867548944697546176030300557613372802734375f,0.600900007969141114472222398035228252410888671875f,-0.2055999973654747403628562096855603158473968505859375f,0.1645999889910221281041913243825547397136688232421875f,0.274299999696016261196973573532886803150177001953125f,0.75989998782277101696536192321218550205230712890625f,0.0504999972581863243448196953977458178997039794921875f,0.243699995326995821898208305356092751026153564453125f,0.146400005227327323353847532416693866252899169921875f,0.40520000185370452783217842807061970233917236328125f,-0.176100006619095861282175974338315427303314208984375f,-0.8452000264406205332079480285756289958953857421875f,0.129399998164176910364631112315692007541656494140625f,-0.57420000838041307389403300476260483264923095703125f,0.223499999657273296094217585050500929355621337890625f,-0.435199997085332856983086458058096468448638916015625f,-0.1364000075370073172820184481679461896419525146484375f,0.00849999481439578374875054578296840190887451171875f,0.0029000063151121135973653508699499070644378662109375f,0.3972000066816807617442464106716215610504150390625f,-0.0763999998629092402779861004091799259185791015625f,0.54640002370476725701564646442420780658721923828125f,-0.14780000158846373636833959608338773250579833984375f,0.71390001305043693236029866966418921947479248046875f,-1.259999986886978007305515347979962825775146484375f,-0.37740001330375694266194841475225985050201416015625f,0.05550000239908670085497988111455924808979034423828125f,0.1843000116541981814322070931666530668735504150390625f,0.4712999373197561681081424467265605926513671875f,0.19009999429583590568881845683790743350982666015625f,0.2366999991603193631561907750437967479228973388671875f,-0.43300001929327869287789098962093703448772430419921875f,0.3615999832391736390491132624447345733642578125f,-0.34070001118183146360252067097462713718414306640625f,0.5968999764084816916920317453332245349884033203125f,0.16039996213316864981379694654606282711029052734375f,-1.261599969530106335469099576584994792938232421875f,-0.10879996071457842532481663511134684085845947265625f,0.1185999704837801260737251141108572483062744140625f,0.80300000247359282212755715590901672840118408203125f,0.5523999817132949630149596487171947956085205078125f,0.074500006467104196161699292133562266826629638671875f,-0.071300006163120155378010167623870074748992919921875f,-0.2021000035941600270206208733725361526012420654296875f,0.1788999916672704415532280108891427516937255859375f,-0.281400018191337863981971167959272861480712890625f,1.1888999895811078744145561358891427516937255859375f,-0.03730003748536159235982268000952899456024169921875f,0.0219000145554542058334845933131873607635498046875f,-0.58130000899434097760831718915142118930816650390625f,-0.8822999975502485714429212748655118048191070556640625f,-0.709900018146634170790321149979718029499053955078125f,0.80769997488260347751065637567080557346343994140625f,-0.23009999884068965680938845252967439591884613037109375f,0.217600003385543860900952495285309851169586181640625f,0.999600018942356172146901371888816356658935546875f,-0.143500000774860414498590444054570980370044708251953125f,0.254599986532330457844608417872223071753978729248046875f,-0.384899992963671611079234935459680855274200439453125f,0.319000019788742150694815791212022304534912109375f,-0.0779000025339424695403067744337022304534912109375f,-0.0785000023953616732352855933640967123210430145263671875f,-0.343600000584125542246738405083306133747100830078125f,0.378999989241361578962141720694489777088165283203125f,-0.408499996379017904502006786060519516468048095703125f,0.2269999790489674040827594581060111522674560546875f,0.215999993696808889609428661060519516468048095703125f,0.99210001789927471094188149436376988887786865234375f,-0.204199996683001483432917666505090892314910888671875f,0.2895000088512897917780719581060111522674560546875f,-0.1240000016838311847067188864457421004772186279296875f,-0.368299995195865648156541283242404460906982421875f,-0.02389999852180479944507851541857235133647918701171875f,0.148300003838539140588181908242404460906982421875f,-0.26250000014901164746561335050500929355621337890625f,0.4048999882698058883079283987171947956085205078125f,-0.181700005027651823041878742515109479427337646484375f,-0.9782000114798545808980634319595992565155029296875f,0.117299998518824555304718160186894237995147705078125f,-0.266200004565715797610891968361102044582366943359375f,0.09750000819563864329353464199812151491641998291015625f,0.84730000455379494983532140395254828035831451416015625f,-0.036099993953108810984531373833306133747100830078125f,-1.0063000118255616133211560736526735126972198486328125f,0.221900011724233792165250633843243122100830078125f,0.50610001548528682491223662509582936763763427734375f,0.405899989342689337945557781495153903961181640625f,0.9593000090837477955574286170303821563720703125f,-0.279299981665611429804130239062942564487457275390625f,-0.147099997931718906585274453391321003437042236328125f,0.26089997682571475223767265561036765575408935546875f,0.23209997170567486790559996734373271465301513671875f,-0.93390000358819946768562658689916133880615234375f,-0.42840001690983786630795293604023754596710205078125f,-0.784199980962276388396503534750081598758697509765625f,-0.40420000182390225518247461877763271331787109375f,-0.31420001791715623795653300476260483264923095703125f,0.41380001033544555610887982766143977642059326171875f,-0.16189998981952691270436162085388787090778350830078125f,0.497900009854137792775219395480235107243061065673828125f,-0.16449997800588533891641418449580669403076171875f,-0.581800018769502713666952331550419330596923828125f,0.4515999896839260063785559395910240709781646728515625f,-0.59370002099424612485023544650175608694553375244140625f,-0.651799996715784146772421081550419330596923828125f,0.64750001490116115832051946199499070644378662109375f,0.040099992060661282522460169275291264057159423828125f,0.5211999983072279629681133883423171937465667724609375f,-0.0126000024914738606440778312389738857746124267578125f,-1.21990006061494415945389846456237137317657470703125f,-0.49589999887943303491510960157029330730438232421875f,-0.65440001366138478289258273434825241565704345703125f,-0.2508999783158305518782071885652840137481689453125f,0.002700007218122468799492708058096468448638916015625f,0.10530000114142890055290990858338773250579833984375f,-0.5264000141680240663077938734204508364200592041015625f,-0.1643000016331672696878740680404007434844970703125f,-0.1469000051677227247637347318232059478759765625f,0.1624999977648258209228515625f,-0.0370999988257885160436444493825547397136688232421875f,-0.2560999903768301777517990558408200740814208984375f,0.268999982237815959962290435214526951313018798828125f,0.103100002655386990824126769439317286014556884765625f,0.8814000254929066358045020024292171001434326171875f,0.0698000040322541703829983816831372678279876708984375f,0.7806999895751476348237929414608515799045562744140625f,-0.06979999963641159155258719692938029766082763671875f,0.50880002046823502581673892564140260219573974609375f,0.0362999978929758049250864360146806575357913970947265625f,0.082799997900426358654613068210892379283905029296875f,0.3289000067919491954171462566591799259185791015625f,-0.5320000124275683361929623060859739780426025390625f,-0.120800002087652680327067855614586733281612396240234375f,-0.224799999524652960314341498815338127315044403076171875f,-0.00779999816119669731051544658839702606201171875f,0.7519000030815601309797102658194489777088165283203125f,-0.08529999980032443307464973258902318775653839111328125f,0.123000001430511485267516036401502788066864013671875f,0.058600001925230016386336728828609921038150787353515625f,-0.06019999298751343985713901929557323455810546875f,-0.0541000024616717922754105529747903347015380859375f,0.7381000113725662004071637056767940521240234375f,0.3252999933183193803643007413484156131744384765625f,0.1697999704301356160840441589243710041046142578125f,-0.123600003340840203236439265310764312744140625f,-0.466300006535649291805611937888897955417633056640625f,-0.3562000014364716804493582458235323429107666015625f,0.43170001925826095856564279529266059398651123046875f,0.74290000028014180344371197861619293689727783203125f,-0.01970000265836713282396885915659368038177490234375f,0.5254999851882458106189233149052597582340240478515625f,0.6965000046640634767669553184532560408115386962890625f,0.285299969327450497758036362938582897186279296875f,1.31720002024173776788984469021670520305633544921875f,-0.196100006842613083790638484060764312744140625f,1.18790002650618564672413413063623011112213134765625f,0.03740001218616928468918558792211115360260009765625f,0.687499998882412644007899871212430298328399658203125f,0.144599988842010862555298444931395351886749267578125f,-0.8647000084698197586163814776227809488773345947265625f,-0.153699995326995964006755457376129925251007080078125f,-0.2329000025153160624569181891274638473987579345703125f,-0.531099996300041343744169353158213198184967041015625f,0.0816000167295338751927147313836030662059783935546875f,0.263999989353119968171768050524406135082244873046875f,-0.5572000323489312290092811963404528796672821044921875f,0.50200000065565131279754496063105762004852294921875f,0.571500009953975762755362666212022304534912109375f,0.41140001461505892876857615192420780658721923828125f,0.8058000047326085990562205552123486995697021484375f,0.04929999805688822789306868799030780792236328125f,0.682800015223026246502513458835892379283905029296875f,0.2609999932497739649761570035479962825775146484375f,-0.671600007826089751006293226964771747589111328125f,-0.117599996307492293823315776535309851169586181640625f,0.43719998991787434317046745491097681224346160888671875f,0.01980000440478324463811077293939888477325439453125f,0.3519999988675117652547896796022541821002960205078125f,-0.004500002667307878567726220353506505489349365234375f,-0.8612000091850757765854496028623543679714202880859375f,-0.14949999991059303994234142010100185871124267578125f,-0.30850000509619712740772001779987476766109466552734375f,-0.041699993330240214817195010255090892314910888671875f,-0.303499990046024237244637333787977695465087890625f,-0.171899994811415712803182032075710594654083251953125f,0.566399985110759729423079988919198513031005859375f,0.2141999931067228857273221365176141262054443359375f,-0.56799998392164707805562784415087662637233734130859375f,0.00060000302791596293872089518117718398571014404296875f,-0.129199995565414393894343447755090892314910888671875f,0.01449999909102916983982822785037569701671600341796875f,0.158600001031160342535031304578296840190887451171875f,-0.049299992506206091746889796922914683818817138671875f,0.7326999845311046311024938404443673789501190185546875f,-0.1736000041976570873458740607020445168018341064453125f,0.114700006048381319967433000783785246312618255615234375f,-0.031999997824430526094374727108515799045562744140625f,-0.544000019043684002184591008699499070644378662109375f,0.028600000286102211788374916068278253078460693359375f,-0.587800024759769446092150246840901672840118408203125f,-0.1209000027537345811623481495189480483531951904296875f,0.185899997389316506968270914512686431407928466796875f,-0.0102000017046928181940756985568441450595855712890625f,0.0560999871730805121927687650895677506923675537109375f,-0.479599991561472815959632498561404645442962646484375f,0.3222999881252641696249838787480257451534271240234375f,-0.16439998616874174075519476900808513164520263671875f,0.444600035110116476033681465196423232555389404296875f,0.973999976575374404319518362171947956085205078125f,0.0700999908685684403053528512828052043914794921875f,-0.1951999793529513027579014305956661701202392578125f,-0.647599986547231498690280204755254089832305908203125f,-0.23699999462068077615839456484536640346050262451171875f,-0.2812000025913119483078617122373543679714202880859375f,-0.488899992263316818252860684879124164581298828125f,0.12599999898672109566177823580801486968994140625f,0.524899980223178630467373295687139034271240234375f,0.04380000556707397407762982766143977642059326171875f,-1.0272999954640857911414286718354560434818267822265625f,-0.863400023540854544279454785282723605632781982421875f,-0.0610999925374979824255206040106713771820068359375f,-1.20870000709891289858433083281852304935455322265625f,0.37179996050596297862966821412555873394012451171875f,-0.058199999409913605319388807401992380619049072265625f,-0.0590000090599058779616825631819665431976318359375f,0.29940000412464140566726200631819665431976318359375f,-0.11399998320639170845680610000272281467914581298828125f,-0.2571999906256794421466338462778367102146148681640625f,-0.76819999270439165428570049698464572429656982421875f,-0.9187999937951563111937502981163561344146728515625f,-0.13729997355938028391619809553958475589752197265625f,-0.33949997904896722644707551808096468448638916015625f,-0.938399947619438901114108375622890889644622802734375f,0.033800005418062273321311295148916542530059814453125f,-0.0296000025138258848755867802537977695465087890625f,0.61540000352114443415985078900121152400970458984375f,0.195600004890561063319864842924289405345916748046875f,-0.27660001177489756951644039872917346656322479248046875f,-0.18320000507235523201643445645458996295928955078125f,-0.7275999934017658432594544137828052043914794921875f,0.1223999971359968075290680644684471189975738525390625f,-0.019700004893541400718959266669116914272308349609375f,0.040200000959634774044815230809035710990428924560546875f,0.199899999520182614087815409220638684928417205810546875f,0.38289999983310707420969265513122081756591796875f,-0.30909997072219841385276595246978104114532470703125f,-0.238900002694129920399745969916693866252899169921875f,-0.071299999010562942203250713646411895751953125f,0.13410001102983948317870499522541649639606475830078125f,0.94120000158548366986366318087675608694553375244140625f,0.190700003954768160241428631707094609737396240234375f,-0.30640000120401378325141195091418921947479248046875f,-0.2589999925941228742232169679482467472553253173828125f,0.159500002890825332002577852108515799045562744140625f,-0.240199994850158748960211596568115055561065673828125f,0.604999981820583343505859375f,0.165999994218349489205621694054570980370044708251953125f,-0.233899992188811356452760037427651695907115936279296875f,0.030600000122189552342888418934307992458343505859375f,0.876700009125471080295710635255090892314910888671875f,0.2452999895185232759331483975984156131744384765625f,-0.384199986922741043571249974775128066539764404296875f,0.28399999356269833850063832869636826217174530029296875f,-0.614000013977289427202777005732059478759765625f,-1.074999992251395752873577293939888477325439453125f,-0.36859999417662603349299388355575501918792724609375f,-1.30159995820522311760214506648480892181396484375f,-0.2221999835848809112803792231716215610504150390625f,-0.2214999989271166924709177692420780658721923828125f,-0.72790002903938333389532999717630445957183837890625f,0.040899995899200458637778865522705018520355224609375f,0.387600009793043209782581470790319144725799560546875f,-0.86839999262094469401063179248012602329254150390625f,-0.103899983620643343584788453881628811359405517578125f,-0.4602999839305876861317301518283784389495849609375f,0.36419999288320514807537620072253048419952392578125f,-0.068400031960010210241307504475116729736328125f,1.02510001769065883792109161731787025928497314453125f,-0.27860000199973533607789022426004521548748016357421875f,-0.4332000191912055431231465263408608734607696533203125f,0.2449999936670064215604725177399814128875732421875f,0.025299990412592787691892226575873792171478271484375f,0.3098000158786771152108485694043338298797607421875f,0.9964000019490715231995636713691055774688720703125f,-0.539399993097781749185060107265599071979522705078125f,0.1493000040173531051124200530466623604297637939453125f,0.19170004026889753134810234769247472286224365234375f,-0.60389998019337642887194306240417063236236572265625f,0.0708999833822243630265802494250237941741943359375f,0.9317000332653524452553028822876513004302978515625f,0.781399988687038415946517488919198513031005859375f,0.043499988406896505921395146287977695465087890625f,0.3154000111579886578283549170009791851043701171875f,0.924000022619962724235165296704508364200592041015625f,0.179800006863474859386542448191903531551361083984375f,0.86850000986456876717056729830801486968994140625f,0.23520000393986695286230315105058252811431884765625f,1.0005999845504760514813824556767940521240234375f,-0.088100002133846277985185224679298698902130126953125f,0.49730000351071357300725139793939888477325439453125f,-0.06449999901652336831148204510100185871124267578125f,0.0808999961972236558693793995189480483531951904296875f,0.1936000041604042376519601020845584571361541748046875f,0.58140001267790797356838083942420780658721923828125f,0.245500000089406977821226973901502788066864013671875f,-0.039499991714954330035425300593487918376922607421875f,-0.207199994760751682321142652654089033603668212890625f,0.505200009900331536982776015065610408782958984375f,0.151400004556775102937393739921390078961849212646484375f,0.001400009921193212658607762932660989463329315185546875f,-0.019800002095103241828155660186894237995147705078125f,-0.24359999365508555602133355932892300188541412353515625f,0.032599996903538652759380056522786617279052734375f,-0.4869000161945820082110003568232059478759765625f,0.13660000536739824728016401422792114317417144775390625f,0.31330000011324887765340463374741375446319580078125f,0.1842999938100575718635809607803821563720703125f,-0.618000023782253382620410775416530668735504150390625f,-0.2693000058799981388801825232803821563720703125f,0.4884000093102454087556907325051724910736083984375f,-0.153000001206994085123369586654007434844970703125f,-0.5693000102758407621195146930404007434844970703125f,0.00859998896121982880913492408581078052520751953125f,0.44789999512061473563306890355306677520275115966796875f,0.760100002193450308141109417192637920379638671875f,0.448500005990266625843787551275454461574554443359375f,0.44280000151395793039910131483338773250579833984375f,-0.459500017493963408554691341123543679714202880859375f,0.45799999800324453502753385691903531551361083984375f,-0.0606000000476836220286713796667754650115966796875f,-0.394299970936774712981787160970270633697509765625f,0.3599000161349774629115927382372319698333740234375f,0.2033999967932700325690120735089294612407684326171875f,-0.23470000973641880204212384342099539935588836669921875f,-0.102899997150898148134956500143744051456451416015625f,0.9644000266253949238404175048344768583774566650390625f,-0.275399978673458889488756540231406688690185546875f,-1.33650000220537190642744462820701301097869873046875f,0.13169999567717294297608532360754907131195068359375f,-0.474500020809471845240068432758562266826629638671875f,0.16309998693466187802414424368180334568023681640625f,0.693899998223781455664038730901665985584259033203125f,1.05629998381137824026154703460633754730224609375f,0.2107000129699709844999233609996736049652099609375f,-0.5593999731302263800358787193545140326023101806640625f,-0.06100000517070325312118939109495840966701507568359375f,-1.10759998669624337708228267729282379150390625f,-0.56019999023079858346818582504056394100189208984375f,-0.074100000226497542143988539464771747589111328125f,-0.8643999974191185486915856017731130123138427734375f,0.0565999979540705577818471283535473048686981201171875f,-0.21700000479072334425012513747788034379482269287109375f,-0.065499998599290787382187772891484200954437255859375f,0.195199989187717193317439523525536060333251953125f,0.245100005173683133108397669275291264057159423828125f,0.130600013458728625437288428656756877899169921875f,-0.4215000049620865052446561094257049262523651123046875f,0.6233000083088870990621899181860499083995819091796875f,-0.9552000346362594651594690731144510209560394287109375f,-0.72399997970461793528329508262686431407928466796875f,-0.2736000071033830582933887853869237005710601806640625f,0.422799997118115111760516811045818030834197998046875f,0.48510000591874113951007529976777732372283935546875f,-1.076699990200996381872755591757595539093017578125f,1.25460000582933428603382708388380706310272216796875f,0.568999948859215010088519193232059478759765625f,1.479600030267238963688214425928890705108642578125f,-0.41939997909069148818161920644342899322509765625f,0.889199997204542125217585635255090892314910888671875f,0.8842999628901484499010621220804750919342041015625f,-0.59730003606975123897626644975389353930950164794921875f,0.620000026300549844648912767297588288784027099609375f,-0.21960000694692138889507759813568554818630218505859375f,-1.032400027832388378357109104399569332599639892578125f,-0.17759999355077749072506776428781449794769287109375f,-1.14599997246265417061295011080801486968994140625f,0.97979998995065642475310596637427806854248046875f,-0.445400023078917683960753493010997772216796875f,-0.21780004770755834186957144993357360363006591796875f,-1.81719995884895269000480766408145427703857421875f,0.5918999665737147974908793912618421018123626708984375f,0.88089996654391367059133699513040482997894287109375f,-0.2859000045418740310054772635339759290218353271484375f,0.1196999954685569189649640975403599441051483154296875f,-0.1677000078886747846951266183168627321720123291015625f,0.0471999984860420607191144881653599441051483154296875f,-0.4931999935984610061723287799395620822906494140625f,-0.14849997849762441859411410405300557613372802734375f,-0.332700008633732835505725233815610408782958984375f,-0.03770000043809407230810393230058252811431884765625f,-0.3075000065565109963472423260100185871124267578125f,-0.7312000000953673417569689263473264873027801513671875f,0.0165000045895578040244799922220408916473388671875f,0.88120002043545231362742242708918638527393341064453125f,0.06719999550580979530423064716160297393798828125f,0.3601000078931451131580843139090575277805328369140625f,0.15140000567436207035143524990417063236236572265625f,0.8647999759063125235769575738231651484966278076171875f,0.06899999460577976861941351671703159809112548828125f,1.040799992029369125390303452149964869022369384765625f,0.0877000040888784582193693495355546474456787109375f,-0.86970000306814920776332655805163085460662841796875f,-0.2768000061035156278421709430404007434844970703125f,-1.04660000901818239782414821092970669269561767578125f,-0.00369998981356634004669103887863457202911376953125f,0.960599976690113432908901813789270818233489990234375f,0.2150000017881392722074451739899814128875732421875f,0.6281999798521400446560392083483748137950897216796875f,0.5990000081658362063308231881819665431976318359375f,-0.2827000167921187578912167737144045531749725341796875f,-0.1041000032067298519677933654747903347015380859375f,-0.56729999635815619996037639793939888477325439453125f,-0.2769999946951866576227985206060111522674560546875f,-0.72569997065067326502685318700969219207763671875f,0.48840002517998237774321523829712532460689544677734375f,0.57520000610053489964457185124047100543975830078125f,0.77309995906948980604056487209163606166839599609375f,0.218099980974198359717775019817054271697998046875f,0.95130002672672286934130170266143977642059326171875f,0.863299993407725896332749471184797585010528564453125f,1.17420003192424804439042418380267918109893798828125f,-0.136800014448166251668226323090493679046630859375f,-0.97920004891157219617525697685778141021728515625f,0.362000037908554617160916677676141262054443359375f,-0.9222000051915646423594807856716215610504150390625f,-0.506099945449829391463936190120875835418701171875f,-1.051500003099441027387683789129368960857391357421875f,0.3852000112414355470491500454954802989959716796875f,-0.4743000145971780767695236136205494403839111328125f,1.4299999859929091172716653090901672840118408203125f,0.00060002374053080842486451729200780391693115234375f,-1.57629997844696134734476800076663494110107421875f,1.32300002259016036276761951739899814128875732421875f,-0.5038000565290445820210152305662631988525390625f,-0.6705000075399869530201613088138401508331298828125f,0.9198000006794924132691448903642594814300537109375f,-0.89969999818801849045257768011651933193206787109375f,0.21220000653266879453440196812152862548828125f,-0.65279998437762198904010801925323903560638427734375f,-0.761700006890297487416319199837744235992431640625f,1.08510001307129844150267672375775873661041259765625f,0.2896999618291857814256218262016773223876953125f,0.49769999544620535658623339259065687656402587890625f,0.15029997636824876661165717450785450637340545654296875f,-0.24170000331401819693155630375258624553680419921875f,0.8756000071629881364998482240480370819568634033203125f,0.5330999890208243385814057546667754650115966796875f,-0.1597000303745266247545941951102577149868011474609375f,0.286400013720989221610579988919198513031005859375f,-1.10549998477846411759628608706407248973846435546875f,-0.0645000110864639708552203956060111522674560546875f,-0.566400004258751810226613088161684572696685791015625f,-0.35270000349283225915542061557061970233917236328125f,0.104599994727969214380891571636311709880828857421875f,-0.26659999068975448466289890347979962825775146484375f,0.11990000123381605590111576020717620849609375f,-0.24720000168979172627814477891661226749420166015625f,-0.1958999935895204824731763437739573419094085693359375f,0.3643999989092350233477191068232059478759765625f,0.1542999893769623209749397574341855943202972412109375f,0.21430000699758533500016710604541003704071044921875f,-0.553699990595877267196556203998625278472900390625f,-0.03670000331401812587728272774256765842437744140625f,-0.4867999781638385758952836113166995346546173095703125f,-0.353200002539157953407311651972122490406036376953125f,0.069199995937943459267671642010100185871124267578125f,-0.1967999912023543629402411170303821563720703125f,0.57690001325160256140378578493255190551280975341796875f,-0.05660001214742660380352390347979962825775146484375f,-0.6078999861985445729573029893799684941768646240234375f,-0.12699999707937248416556030861102044582366943359375f,0.6456999865204096078485918042133562266826629638671875f,0.11149999520182607426477261469699442386627197265625f,0.1524999582767492256607511080801486968994140625f,0.001499989986419425491703805164434015750885009765625f,1.4136000172734259905382714350707828998565673828125f,-0.77009995793700181110352787072770297527313232421875f,-0.762700006175041522737956256605684757232666015625f,-0.2892000204503535343292242032475769519805908203125f,-2.20449999141693098181349341757595539093017578125f,1.19640004441738145857243580394424498081207275390625f,0.49119997148513760976129560731351375579833984375f,0.741000018954277361871163520845584571361541748046875f,0.823399993813037411882760352455079555511474609375f,0.402899979418516185347698410623706877231597900390625f,0.4832999802947046674717057612724602222442626953125f,0.364500008404255115834757816628552973270416259765625f,-0.156899982666969695088710068375803530216217041015625f,-0.8370000025928021614163299091160297393798828125f,-0.44350000107288334305621901876293122768402099609375f,-0.379200018662214421993894575280137360095977783203125f,-1.3474999949336048388204289949499070644378662109375f,0.992399999892711104365616847644560039043426513671875f,0.3939999795556072825775117962621152400970458984375f,0.7614000192344192452509332724730484187602996826171875f,0.78279999629855101517250659526325762271881103515625f,-0.65700004461407690570240447414107620716094970703125f,-0.02369997029304471425348310731351375579833984375f,0.893700034815072985594497367856092751026153564453125f,0.0007000090956683635567969758994877338409423828125f,-0.218699999946355472246750650811009109020233154296875f,0.7967000024199482055564658367075026035308837890625f,-0.327899988657236018951834921608678996562957763671875f,-0.6278999711856247945007680755225010216236114501953125f,-0.098799998414516476685776069643907248973846435546875f,-0.218700008477270557705196551978588104248046875f,0.200800002646446262843937802244909107685089111328125f,0.2955000039637087905219914318877272307872772216796875f,0.005200000773370294648856315689045004546642303466796875f,0.67490000378563996807912417352781631052494049072265625f,0.087300009918212850834606797434389591217041015625f,0.3607999721735719589332802570424973964691162109375f,0.198199997323751375688516418449580669403076171875f,-0.300199997718632261012317030690610408782958984375f,-0.35659999620318405533225813996978104114532470703125f,0.5256000088766217270830338748055510222911834716796875f,0.008699995326995946243187063373625278472900390625f,0.00720000340342519251635167165659368038177490234375f,0.1134999938830734134143085611867718398571014404296875f,-0.684499982774257720308241914608515799045562744140625f,-0.4600999989151954849830872262828052043914794921875f,-0.2239999881610275078713812035857699811458587646484375f,0.43279998959302901084811310283839702606201171875f,-0.33570001725405429482407271279953420162200927734375f,0.2579999942779540589299358543939888477325439453125f,-0.36590002510547614544833550098701380193233489990234375f,-0.11590000603198991058206956950016319751739501953125f,-0.5917999684035779583979319795616902410984039306640625f,-0.3872000050425530304210042231716215610504150390625f,-0.9571000002786516080988121757400222122669219970703125f,0.437900003744661869831134026753716170787811279296875f,0.275799984988570390243012298014946281909942626953125f,-0.33210000642538073378773333388380706310272216796875f,-0.29489998707771292174584232270717620849609375f,0.6276000078558923167548755372990854084491729736328125f,0.33429996512532245134252661955542862415313720703125f,-0.6999000147938729821106562667409889400005340576171875f,-0.16909998130202286148460188996978104114532470703125f,-0.718500016886741033061980488128028810024261474609375f,0.31230003016516560965243343162001110613346099853515625f,-1.1952000282287595922525724745355546474456787109375f,-0.8600000149011606254134676419198513031005859375f,-0.570899984872340837682713754475116729736328125f,-1.14259995927810731330964699736796319484710693359375f,-0.17130003373026880808538408018648624420166015625f,0.3668000129580502033377342741005122661590576171875f,0.7782000052213664531564063508994877338409423828125f,1.1023999614477162367620621807873249053955078125f,0.4845000088885422773188338396721519529819488525390625f,-0.518700001510977504182164921076036989688873291015625f,0.85529999912977228149202346685342490673065185546875f,0.2244000041246412280315780662931501865386962890625f,-0.229199967700242979162794654257595539093017578125f,-0.467900019502639707980051753111183643341064453125f,0.4359000330030919911905584740452468395233154296875f,0.194699944543838654453793424181640148162841796875f,0.767999996066092815993897602311335504055023193359375f,-0.2105999914050098453799364506267011165618896484375f,1.42020003195405042362153835711069405078887939453125f,-0.323300022912025752930276212282478809356689453125f,-1.0650000194460156333065015132888220250606536865234375f,-0.4406000185251228717930871425778605043888092041015625f,0.599700015175342660000978867174126207828521728515625f,-1.1117999790951607064215522768790833652019500732421875f,0.331499985516071404845206416212022304534912109375f,-0.141700000594556119182243492105044424533843994140625f,0.25080000031813975969186003567301668226718902587890625f,0.243699989888071899546417853343882597982883453369140625f,0.2549999932013451431345174569287337362766265869140625f,0.148499995745718482709207819425500929355621337890625f,-0.006799990010261325323881465010344982147216796875f,0.47200002297013998742158946697600185871124267578125f,-0.06529999920427798087985138408839702606201171875f,-0.13939999879747622113512761643505655229091644287109375f,0.158799999010562853385408743633888661861419677734375f,0.5647999947935342657245882946881465613842010498046875f,0.14389999971389766386664632591418921947479248046875f,0.31119999275654552928216389773297123610973358154296875f,0.25689999801516538724399651982821524143218994140625f,-0.6091999811112882579067218102863989770412445068359375f,-0.4871000029981136236756356083787977695465087890625f,-0.250500005900859523677581819356419146060943603515625f,0.2263999907732010541394629399292171001434326171875f,-0.930199976707994924396416536183096468448638916015625f,-0.2231999957591295213887860882095992565155029296875f,-0.17539998869448913954016688876436091959476470947265625f,0.122999992936849711355762337916530668735504150390625f,-0.7337999759137632249661464811651967465877532958984375f,0.1702000073671341429104586495668627321720123291015625f,0.1583000015661120141174933451111428439617156982421875f,0.28609999998807911225640054908581078052520751953125f,-0.70449999115616090961111694923602044582366943359375f,0.39350000368058690725092674256302416324615478515625f,-1.38060000198483390221326772007159888744354248046875f,-0.2417999794304375171805077116005122661590576171875f,-0.22749999940395326802899944595992565155029296875f,-0.093599988197535122225190207245759665966033935546875f,0.6286999857529995683336210277047939598560333251953125f,-0.63299997784942363399096620923955924808979034423828125f,-0.29769996534586018466228551915264688432216644287109375f,-1.2711000738978395219191952492110431194305419921875f,-0.858799968314169603900154470466077327728271484375f,-0.0267000061452389303440213552676141262054443359375f,-0.380999999582767312489295363775454461574554443359375f,-0.8861999990522859693697910188348032534122467041015625f,0.04400003394484475194303740863688290119171142578125f,-0.444899994528293429851828477694652974605560302734375f,0.149200010764598545165426912717521190643310546875f,0.2070000058710572687914464040659368038177490234375f,-0.5931999912142753572652509319595992565155029296875f,0.93630000213980668632984816213138401508331298828125f,0.25449996638298078011075631366111338138580322265625f,-0.32249999105930360343563734204508364200592041015625f,-1.981399954414367670096908113919198513031005859375f,0.40949999037384987587984142010100185871124267578125f,-0.027399995720386538522461705724708735942840576171875f,0.5358000062227243898149708911660127341747283935546875f,-0.53179999701380697985086953849531710147857666015625f,-0.74040002792179604451661134589812718331813812255859375f,-0.525099994668364100647295344970189034938812255859375f,-0.028799997967481782978893534163944423198699951171875f,-1.0703000021100042005173236248083412647247314453125f,-1.032000025391579089983906669658608734607696533203125f,1.135699966590106857466935252887196838855743408203125f,0.2361999904468656819034322325023822486400604248046875f,-0.232999989248812211517503101276815868914127349853515625f,-0.02389999680817123106635335716418921947479248046875f,0.7589000039234756744832566255354322493076324462890625f,-0.20489999840259542907006107270717620849609375f,1.0601999793529512938761172335944138467311859130859375f,0.18969999788999558631985564716160297393798828125f,-0.25969999911934138481228728778660297393798828125f,-0.1355999973654746781903668306767940521240234375f,-0.304999992512166695490805068402551114559173583984375f,-0.3698999843955039779075377737171947956085205078125f,-0.01689998546093729459016685723327100276947021484375f,0.3395000095963478514704547706060111522674560546875f,0.6993999974563716026665360914194025099277496337890625f,0.4883999944090842149080344825051724910736083984375f,0.16159999225437626790835565770976245403289794921875f,-0.184499999165534944722821819595992565155029296875f,-0.07969999006688588405467044140095822513103485107421875f,-0.09590000633001327656756984652020037174224853515625f,0.182400000637769732492188268224708735942840576171875f,0.127499990165233612060546875f,-0.7370999818384647017666111423750407993793487548828125f,0.119700012008845835254078338039107620716094970703125f,-0.242999983586371204324905193061567842960357666015625f,0.012699991571903268550158827565610408782958984375f,-0.0821000124976036804724799367249943315982818603515625f,0.21219999878406525795071502216160297393798828125f,-0.993599994586408019614509612438268959522247314453125f,0.0311999908193947295131920327548868954181671142578125f,1.7158999577522280333141679875552654266357421875f,0.24090000927299204391829334781505167484283447265625f,0.155799990203977056779649501550011336803436279296875f,-0.5172000213593246353838139839353971183300018310546875f,0.719900017476081899303608224727213382720947265625f,0.3239999801516528776801351341418921947479248046875f,2.258999934554100974537504953332245349884033203125f,-0.062399992591142705578022287227213382720947265625f,-0.369500001549720735738446819595992565155029296875f,1.0593000031232833180183661170303821563720703125f,0.7237999671220780584235399146564304828643798828125f,0.46859996199011799689060353557579219341278076171875f,0.191699991989136009351568645797669887542724609375f,-1.01490002939701096096314358874224126338958740234375f,0.3330000044107439549634364084340631961822509765625f,-0.78179996736049606198548644897527992725372314453125f,-0.2103999640703211326808741432614624500274658203125f,-0.22230001602768911794782979995943605899810791015625f,-0.67389997587203964712898596189916133880615234375f,0.241800012958049759248524424037896096706390380859375f,0.576800005209445743048490840010344982147216796875f,-0.4058999835312366855077925720252096652984619140625f,-1.838099941635132239525773911736905574798583984375f,-0.5811000326961279771609270028420723974704742431640625f,-1.1365999748945234681940519294585101306438446044921875f,0.759000021427869597800963674671947956085205078125f,0.908099949383736060326555161736905574798583984375f,-0.26849999719858175240005948580801486968994140625f,-2.045199990677833312702205148525536060333251953125f,0.569700003589689085714553584693931043148040771484375f,0.7251999987989663498666459418018348515033721923828125f,-0.139300004911422803388632019050419330596923828125f,0.7641999696001413600043861151789315044879913330078125f,-0.050500007688999204447100055404007434844970703125f,-1.025900005920231006228959813597612082958221435546875f,-0.163399995265901043506318046638625673949718475341796875f,-0.03569999793842437707525050427648238837718963623046875f,-0.25100000181794168696569613530300557613372802734375f,-0.031900007253885032110218844536575488746166229248046875f,0.3263999922633171735242285649292171001434326171875f,0.6782999910980465063659039515187032520771026611328125f,-0.017199999864399462268238494289107620716094970703125f,-0.253199988904595496563842971227131783962249755859375f,-0.066400003737211221732650301419198513031005859375f,-0.46279999596327525690497850519022904336452484130859375f,-0.167699992763996164057971327565610408782958984375f,1.16469999502152177228708751499652862548828125f,0.2706000027298927079755230806767940521240234375f,0.1971999946862459385243937504128552973270416259765625f,0.16010000100135801659462231327779591083526611328125f,0.5048999843582511193318396180984564125537872314453125f,0.23830000286996366742897635049303062260150909423828125f,0.25159999940693378306377780972979962825775146484375f,0.26900001919269556083236238919198513031005859375f,1.2713999734133480945530436656554229557514190673828125f,0.27159999158382408523948470246978104114532470703125f,-0.46519999686181545683894000831060111522674560546875f,-0.0444000005483626836166877183131873607635498046875f,-0.54899998044967668331395316272391937673091888427734375f,-0.20420000361204149186278300476260483264923095703125f,-0.1435999520063404588654520921409130096435546875f,0.0897000184535983180467155762016773223876953125f,1.438599969291686875294544734060764312744140625f,0.4494000239431859000660551828332245349884033203125f,-1.41450001466274244421583716757595539093017578125f,0.516899985963851005266178617603145539760589599609375f,0.45969997078850910465774859403609298169612884521484375f,0.16590000096559531783668717253021895885467529296875f,-0.1201000066637991636753213242627680301666259765625f,-1.03650000116229090707520299474708735942840576171875f,0.247300013494491821575138601474463939666748046875f,-0.1241999978750942812411039994913153350353240966796875f,-0.6803999679446224302381551751750521361827850341796875f,-0.10649999803304677215010087820701301097869873046875f,-1.11679998464584340212013557902537286281585693359375f,0.0563999888360502410478147794492542743682861328125f,2.132499997019767334904827293939888477325439453125f,-1.0170000081062315899771419935859739780426025390625f,-0.7179999763965607684212955064140260219573974609375f,-0.43020000051259987827734221355058252811431884765625f,0.56189998654127126798130120732821524143218994140625f,-0.9225000153109432776687981458962894976139068603515625f,-0.5191999801799653813105805966188199818134307861328125f,-1.3524000386357304392959122196771204471588134765625f,0.887799983930587899294550879858434200286865234375f,-0.468599970185756387053288563038222491741180419921875f,-1.2347999559760101107031005085445940494537353515625f,0.4721999987840650447878942941315472126007080078125f,-0.4557999919176101144557833322323858737945556640625f,0.514200011062622142077316311770118772983551025390625f,0.20450000117719202563648650539107620716094970703125f,0.2335999987959862522757248370908200740814208984375f,-0.74369999126642927222974321921356022357940673828125f,-0.24129999623522169205358522958704270422458648681640625f,0.176799987104535316628783903070143423974514007568359375f,-0.2511999959230422074796251763473264873027801513671875f,-0.3065999909132719469795347322360612452030181884765625f,-0.1882999898314476894256586092524230480194091796875f,0.1014999989643692313023848328157328069210052490234375f,0.07020000021457661887325230054557323455810546875f,0.1139999886825682207103227483457885682582855224609375f,0.5156999998569489207511651329696178436279296875f,0.028400015158950964178075082600116729736328125f,-0.12009999817013738976356762577779591083526611328125f,-1.1334999731704595848924554957193322479724884033203125f,-0.170099999511241772864877930260263383388519287109375f,0.88990001933872686645798921745154075324535369873046875f,0.0859999994896352148998630582354962825775146484375f,-0.05939999745637181138846472094883210957050323486328125f,0.0288000005006789905337427626363933086395263671875f,-1.035899980141222531671019169152714312076568603515625f,0.2083000080108643459197992342524230480194091796875f,-0.38289999629408111303519035573117434978485107421875f,-0.0532000032842159242818524944595992565155029296875f,-0.36109999909400924966718093855888582766056060791015625f,-0.053700000989437057796749286353588104248046875f,0.8989000121563670564484027636353857815265655517578125f,-0.1499999983981250561981823921087197959423065185546875f,0.6385000198855994568702953984029591083526611328125f,-0.0736000098228455357229904620908200740814208984375f,-0.4452999779701229954298469237983226776123046875f,-0.53160001140236889938250897102989256381988525390625f,-1.903099988722800617324537597596645355224609375f,-0.147900006408244077960034701391123235225677490234375f,-0.1412000024423003541329535437398590147495269775390625f,0.98219998395740948371468448385712690651416778564453125f,0.480999958232045887740468970150686800479888916015625f,-0.42060001748204189908619810012169182300567626953125f,-0.6034999754428866935995756648480892181396484375f,-0.9020999951004977646107363398186862468719482421875f,-0.94959996559619952449793345294892787933349609375f,-0.66750000387430219461748492904007434844970703125f,-0.27759997358322152649634517729282379150390625f,0.844800002765655477787731797434389591217041015625f,-1.1540999889016152479825905174948275089263916015625f,-0.3291000344991690251816862655687145888805389404296875f,1.06530002580285110269642245839349925518035888671875f,-0.3050999985054134722162189063965342938899993896484375f,-0.43859998188316839939915325885522179305553436279296875f,0.1860000068843365994553096243180334568023681640625f,-1.707699969220161761995768756605684757232666015625f,-0.72600003460049666870190776535309851169586181640625f,-1.5055000050067892658489654422737658023834228515625f,0.0327000042378902122663930640555918216705322265625f,-0.71249999582767475914124588598497211933135986328125f,-0.2253000103056432834591760183684527873992919921875f,1.3603000292301175022657844237983226776123046875f,-0.45600003396719746007192952674813568592071533203125f,0.427500033900142017984080666792578995227813720703125f,0.87000004261732133414852796704508364200592041015625f,0.5534999929517507855081248635542578995227813720703125f,-0.0047000013172626697866007816628552973270416259765625f,-0.6109999982044100885758552976767532527446746826171875f,-0.16869999279379843670767513685859739780426025390625f,0.451100024835765001096632431654143147170543670654296875f,0.27809998902082444516281611868180334568023681640625f,-0.744099991770088564635443617589771747589111328125f,0.23029999927878375132195287733338773250579833984375f,-0.141800004576146776447131969689507968723773956298828125f,-0.35189999130964277895827763131819665431976318359375f,0.15770000338107337256587925367057323455810546875f,-0.06069999710023399330793836270458996295928955078125f,-0.3765999950855969391483313302160240709781646728515625f,-0.3277000172019004509138540015555918216705322265625f,-0.663599989967047942940325810923241078853607177734375f,-0.45069999397099014259993055020458996295928955078125f,-0.3307999840572477534550444033811800181865692138671875f,-0.42650000213086602940393277094699442386627197265625f,-0.5220999904438852201593590507400222122669219970703125f,-0.0574000027611851759701266928459517657756805419921875f,-1.0742999896749854116251299274154007434844970703125f,-0.3141999948203564319015868022688664495944976806640625f,0.7732999835357070583796712526236660778522491455078125f,-0.308700003671646072689327411353588104248046875f,0.02799999900907301775276891930843703448772430419921875f,-0.012799998496472841935656106215901672840118408203125f,-0.35329999217838048419793040011427365243434906005859375f,-0.0462999912619590503481958876363933086395263671875f,0.2538999854460362026742359375930391252040863037109375f,0.428000012755394010355303180404007434844970703125f,-0.1647000056385987676321747130714356899261474609375f,0.54810001398026952923459020894370041787624359130859375f,-0.424600013652443930567415009136311709880828857421875f,-0.51600002849102022395300082280300557613372802734375f,-0.0386999384045605410165080684237182140350341796875f,-0.295199976819753739931684322073124349117279052734375f,-1.5250999703049661349041343783028423786163330078125f,0.30990002179741882315511247725225985050201416015625f,0.4205999848484989200869677006267011165618896484375f,0.617999979227781270907371435896493494510650634765625f,0.7562999769568445884715401916764676570892333984375f,-0.103800015848875215596081034163944423198699951171875f,-0.940200000214576281365452814497984945774078369140625f,1.18170003460645656190308727673254907131195068359375f,-0.0192000021219254080051541677676141262054443359375f,0.939899981564283226731504328199662268161773681640625f,0.146399987196922598542414561961777508258819580078125f,0.912600000107288256145920968265272676944732666015625f,-0.232700009751319658590773542528040707111358642578125f,0.558000012643635301401445758529007434844970703125f,-1.365299999502301186993236115085892379283905029296875f,0.18779999324381380088055948363034985959529876708984375f,1.391099979647993922782234221813268959522247314453125f,0.39509999504089332589273908524774014949798583984375f,0.2467999858379368305349998991005122661590576171875f,-0.2808000067442655289795538919861428439617156982421875f,-1.0854999763965607062488061274052597582340240478515625f,-0.96900003781914723077761664171703159809112548828125f,-0.90630000824928202973751467652618885040283203125f,-0.8307000272750855174308526329696178436279296875f,-0.18769998099207907404917250460130162537097930908203125f,-0.02539999357461919515799309010617434978485107421875f,0.715100018063187459205209961510263383388519287109375f,0.11710000851154322987213163287378847599029541015625f,0.49109998814165589653413235282641835510730743408203125f,0.442799983632564675417597754858434200286865234375f,-0.33239999601840952703213361019152216613292694091796875f,-0.489799998593330343510388047434389591217041015625f,0.2998000270918008691722889125230722129344940185546875f,0.10609999842345718690239664283581078052520751953125f,0.4464999838024379386069995234720408916473388671875f,-0.538699996519088841750999563373625278472900390625f,0.2567999770835045492134440792142413556575775146484375f,-0.12310000094175332918666754267178475856781005859375f,0.21240001393705598697891900883405469357967376708984375f,-0.01519999395608895298437346355058252811431884765625f,0.65600001001358021479603621628484688699245452880859375f,-0.114299996678531134985945527660078369081020355224609375f,0.55130000104084608825161240019951947033405303955078125f,-0.2992000054001806574888178147375583648681640625f,-1.0522999772474168622693468932993710041046142578125f,0.135999998725950643763127345664543099701404571533203125f,0.87309999093934897995694655037368647754192352294921875f,0.210300006021559238167384364714962430298328399658203125f,0.15429999498352409137424956497852690517902374267578125f,-0.3722999987423418755128068369231186807155609130859375f,0.6145000173449515212809046715847216546535491943359375f,0.257900001323223104776616310118697583675384521484375f,0.308799995583295849854721382143907248973846435546875f,0.129499999463558168599774944595992565155029296875f,-0.711499961376190537265529201249592006206512451171875f,-1.056900045400858356714479668880812823772430419921875f,-0.1142999527573582696504672640003263950347900390625f,-0.729900016134977125403793252189643681049346923828125f,0.0705000017285346558537639793939888477325439453125f,-0.19799997907877031622092545148916542530059814453125f,-0.406800003719329339446630910970270633697509765625f,0.668099985146522357126741553656756877899169921875f,0.941400030112266250625907559879124164581298828125f,0.032700000214576530055410330533050000667572021484375f,1.028699959564209454043748337426222860813140869140625f,0.435299997937679705728442058898508548736572265625f,0.479599970960617127957448246888816356658935546875f,-0.059100012743473318010956063517369329929351806640625f,0.1864000214695931134656348149292171001434326171875f,-0.794400035119056990851049704360775649547576904296875f,0.1472000288844110826858013751916587352752685546875f,-0.2124000190220774741955978015539585612714290618896484375f,0.63790000275746006064281345970812253654003143310546875f,1.33139999390244412325046141631901264190673828125f,-0.52890003309249777885270304977893829345703125f,-1.337100001042335950529604815528728067874908447265625f,-0.01229994671866363464829419172019697725772857666015625f,-0.800700016825646176954478505649603903293609619140625f,-0.57029999285265786657106446000398136675357818603515625f,0.39989997180402225840367691489518620073795318603515625f,1.311900001516938640833132012630812823772430419921875f,-1.0230000081360339780900403638952411711215972900390625f,-0.52559997456669815818486313219182193279266357421875f,-0.1904999935328959281832794658839702606201171875f,-0.6023999924421310492306247397209517657756805419921875f,0.14300000053644179587308826739899814128875732421875f,-0.538999985478818555151292457594536244869232177734375f,0.257600007259845664719932756270281970500946044921875f,-0.7630000097379088241922318047727458178997039794921875f,-0.0435999976038932146593651850707828998565673828125f,-0.580899990162253576642115149297751486301422119140625f,0.0980999980174004360122808066080324351787567138671875f,-0.767600007967650821427696428145281970500946044921875f,0.00839999202489849494668305851519107818603515625f,0.659299975854158670784954665577970445156097412109375f,-0.2747999968051910713029428734444081783294677734375f,0.43509998989999310037291024855221621692180633544921875f,0.066600000226497702016104085487313568592071533203125f,0.17580000849515198257932979686302132904529571533203125f,-0.125500003218650846292803180404007434844970703125f,0.3057000022411344719586168139358051121234893798828125f,-0.024499987840652437398603069595992565155029296875f,-0.0966999998122453074955728880013339221477508544921875f,-0.40460000076293933801707680686376988887786865234375f,0.5770000207349657017630306654609739780426025390625f,0.125599995577335477037195232696831226348876953125f,0.8756999992609024729972588829696178436279296875f,0.11789999640584003515186850563623011112213134765625f,0.8940999929249289213117890540161170065402984619140625f,-0.356799992506206142817148929680115543305873870849609375f,0.42289997362568987337994030895060859620571136474609375f,-0.00839999664425850056659328402020037174224853515625f,-0.332499986030161565064844353400985710322856903076171875f,0.0281999970257282850383262484683655202388763427734375f,0.215999987959861527997418306767940521240234375f,-0.301499999821186026593977658194489777088165283203125f,-1.2933999657988550779919023625552654266357421875f,-0.1862999970734116050863349300925619900226593017578125f,-1.43420001717209810720987661625258624553680419921875f,1.25070000239014600396103560342453420162200927734375f,0.562499951124191710505328956060111522674560546875f,0.498300025575608318550990816220291890203952789306640625f,-0.6776000223100187014324546908028423786163330078125f,-0.0104999789297579582125763408839702606201171875f,1.26709997632503501563405734486877918243408203125f,-0.089700011450052130612675682641565799713134765625f,0.399299995374679639326132019050419330596923828125f,-0.5691000155746939181966581600136123597621917724609375f,-0.37299997505545601228504892787896096706390380859375f,0.615300016564130913820918067358434200286865234375f,1.2982999972820277179152981261722743511199951171875f,-0.62730003867745409706913051195442676544189453125f,-1.0841000005245202686410266323946416378021240234375f,0.342299974453448729860838284366764128208160400390625f,1.1226000150084498585556502803228795528411865234375f,-0.4785000143349176227047792053781449794769287109375f,1.170699996876717108307275339029729366302490234375f,-0.6963000189781194393390251207165420055389404296875f,0.8954999927878386500879059894941747188568115234375f,0.64800000167265581207942659602849744260311126708984375f,1.3038999489940703568890967289917171001434326171875f,0.0161999938368797558041478623636066913604736328125f,-0.6286999795317651518189450143836438655853271484375f,0.25200000058114502277106794281280599534511566162109375f,-0.071199997711181683968106881366111338138580322265625f,-0.22470000176429749672024627216160297393798828125f,-0.59659997854530832928077188626048155128955841064453125f,-0.259800001871585806156872422434389591217041015625f,0.54620000396966938982501460486673749983310699462890625f,0.36019999559521664878047886304557323455810546875f,0.1344999945834277621514729617047123610973358154296875f,-0.158399992918968113286837251507677137851715087890625f,0.08340000338703390525552094914019107818603515625f,0.18729999887272719139019727663253434002399444580078125f,-0.1496999890238048802615367094404064118862152099609375f,-0.0943999991700052998311321061919443309307098388671875f,0.208000002622604374113990388650563545525074005126953125f,0.152400006039440649718841314097517170011997222900390625f,0.77489999663308262167760176453157328069210052490234375f,0.0142000089764593440122553147375583648681640625f,0.75679997723251568597646610214724205434322357177734375f,0.502799993467331063357050879858434200286865234375f,-0.4619999903738498669980572230997495353221893310546875f,0.181400000049173837357585625795763917267322540283203125f,0.11250000180676578143135202481062151491641998291015625f,-0.03909999709725386907166466698981821537017822265625f,0.8713999639511111894307759939692914485931396484375f,-0.0534999992847442484844577847979962825775146484375f,0.25120001093596211472203094672295264899730682373046875f,0.06009999951124189720985668827779591083526611328125f,0.683699995625019152356571794371120631694793701171875f,-0.18709999390840525990142850787378847599029541015625f,0.173799994875490693146957710268907248973846435546875f,-0.219599995994567809276531988871283829212188720703125f,-0.8215999877840278742269219947047531604766845703125f,0.76620000963211065680980027536861598491668701171875f,-0.3473999793291095983249761047773063182830810546875f,0.03419998900890330872925915173254907131195068359375f,0.80260000934600839173072017729282379150390625f,-0.600599971586465386508280062116682529449462890625f,-0.15889996573925113665382014005444943904876708984375f,0.4592999737679954730396048034890554845333099365234375f,0.61539996272921637654462756472639739513397216796875f,0.160800005123764133685426713782362639904022216796875f,0.1493000032350420003268709479016251862049102783203125f,-0.1218000101640821508652834381791763007640838623046875f,0.124000006899237735780161528964526951313018798828125f,-1.245600028415769333633988935616798698902130126953125f,-0.05609997232779850850903358150389976799488067626953125f,-0.34289999506473467505429653101600706577301025390625f,1.001199995923041541345810401253402233123779296875f,-0.314399997568131084335618652403354644775390625f,-0.749200005102157007286223233677446842193603515625f,-0.670200005262345133161261401255615055561065673828125f,-0.50329997640177615014778211843804456293582916259765625f,-0.26759999936223000105428582173772156238555908203125f,1.2653999572157861308596693561412394046783447265625f,-0.4557000123739243235831963829696178436279296875f,-0.2083999631166459920450506615452468395233154296875f,-1.039500010341405822344995613093487918376922607421875f,-0.0524999627470972285436801030300557613372802734375f,0.523499982893466775379920363775454461574554443359375f,0.845799956452847112586823641322553157806396484375f,0.0911999764025208659035115488222800195217132568359375f,-0.02839999541491255996561449137516319751739501953125f,0.140799996089935319787400658242404460906982421875f,-0.16409999996572743885536738162045367062091827392578125f,-0.1765000013113020571609013131819665431976318359375f,0.7515999868154528140706815975136123597621917724609375f,0.162500014342367660180599386876565404236316680908203125f,0.71529998849406839500630894690402783453464508056640625f,-0.429599996292591157498463871888816356658935546875f,-0.0528999877631666226562856536475010216236114501953125f,-0.0481999946787954325344571770983748137950897216796875f,-0.8262999912247064315806710510514676570892333984375f,0.0364999858140944155593388131819665431976318359375f,-0.0906000376358624404105057692504487931728363037109375f,-0.13260001500844964539282955229282379150390625f,-0.20710001503080111007193409022875130176544189453125f,-0.0430999979615211969985466566868126392364501953125f,0.48449999521672726388032970135100185871124267578125f,-0.023300003838539140588181908242404460906982421875f,0.358400005994737202996702762902714312076568603515625f,-0.000400003409385618624583003111183643341064453125f,0.6009999898597599976568517377017997205257415771484375f,-0.139599993014335677088411102886311709880828857421875f,-0.4221000144720077340565467238775454461574554443359375f,-0.1423999927401542464622252737171947956085205078125f,0.9930999721825124471052959052030928432941436767578125f,0.1186000040113925901863467515795491635799407958984375f,-0.3880000003874302105799642959027551114559173583984375f,0.280100004875659891467876150272786617279052734375f,0.720899987256526930679001452517695724964141845703125f,-0.2581999948278068668372497995733283460140228271484375f,-1.3327999755859369912514011957682669162750244140625f,0.61020001884102814670995940105058252811431884765625f,0.69089997444152828620644868351519107818603515625f,0.50939998326301516584635464823804795742034912109375f,1.953199935483933558089120197109878063201904296875f,-1.249800042980164338501936072134412825107574462890625f,-0.16529999940916850764693890596390701830387115478515625f,0.6296999989330771541062858887016773223876953125f,0.2769000196218485143617726862430572509765625f,0.41640001033097551186301643610931932926177978515625f,-1.072499987557530953807827245327644050121307373046875f,0.87210006275177054391178899095393717288970947265625f,1.075799943041801043364102952182292938232421875f,1.085400056010485059232451021671295166015625f,0.952299975347518312673855689354240894317626953125f,-0.55770000572800615401547474903054535388946533203125f,0.112300023627280864957356243394315242767333984375f,-0.8805000041127204468693889793939888477325439453125f,-0.555599942827225135033586411736905574798583984375f,-0.153300006222725215820901212282478809356689453125f,0.946599979662895663068411522544920444488525390625f,1.0749000058531763546199044867535121738910675048828125f,0.51489993239045173112344855326227843761444091796875f,0.25409998219609253311546126496978104114532470703125f,2.104499929428101268058526329696178436279296875f,-0.05580000421106812158456023098551668226718902587890625f,-0.689600008884072135373344281106255948543548583984375f,-1.2567000106155870042812239262275397777557373046875f,-0.028899975872040073454627417959272861480712890625f,-0.36270003743022716236055202898569405078887939453125f,0.300799996425211713102498833904974162578582763671875f,-0.16870000396966933209341732435859739780426025390625f,0.113999999746680291679012952954508364200592041015625f,0.05040001145601280541086453013122081756591796875f,0.8529000074699519817755799522274173796176910400390625f,0.0504000022172927941710440791212022304534912109375f,-1.10639998604208233956569529254920780658721923828125f,-0.27420000212192530142374380375258624553680419921875f,-0.4852999938026070214647234024596400558948516845703125f,-0.1995999985277652211124177483725361526012420654296875f,0.66920000785887234773241516450070776045322418212890625f,0.073699992942810155227562063373625278472900390625f,-0.17529998933225876545094479297404177486896514892578125f,-0.175799999964237230187791283242404460906982421875f,-0.7486999883234501940165728228748776018619537353515625f,-0.083299994599819182639066639239899814128875732421875f,-0.2442000137820838912006138343713246285915374755859375f,0.0187000113829969283330001417198218405246734619140625f,1.023799985450505989348357616108842194080352783203125f,-0.41209999115169038130801482111564837396144866943359375f,0.8082000067487358219153747995733283460140228271484375f,0.28370000039339071662425340036861598491668701171875f,-0.26819999173581610119043716622400097548961639404296875f,-0.393600003154575939223747127471142448484897613525390625f,0.77149998534843333697352818489889614284038543701171875f,-0.17599999146163458618730146554298698902130126953125f,0.0764000058233735845902856453903950750827789306640625f,-0.0751000010013580521217591012828052043914794921875f,-0.70389999457299701202117603315855376422405242919921875f,0.463899993455409997977767488919198513031005859375f,-0.834799983096122133474636939354240894317626953125f,0.531600008571147863989381221472285687923431396484375f,0.76279998780488966758639435283839702606201171875f,-1.1706000511586669432517737732268869876861572265625f,0.0811000265121464281037333421409130096435546875f,0.14969999788999555079271885915659368038177490234375f,-1.8691999907970426875181146897375583648681640625f,1.193400004392862090440985411987639963626861572265625f,0.402499962151050993952594581060111522674560546875f,-0.14640002020299458518337587520363740622997283935546875f,0.858800014880299666941709801903925836086273193359375f,0.60120000918507532361445555579848587512969970703125f,0.5796999832868579005662468262016773223876953125f,0.84510000979304322754614986479282379150390625f,-0.475300004792213570681269629858434200286865234375f,0.7280999978125095584147175031830556690692901611328125f,0.44219993813633973189780590473674237728118896484375f,-0.527400010733305890653355163522064685821533203125f,1.265800004658102384524909211904741823673248291015625f,0.36059995836168479144134835223667323589324951171875f,1.5853999772205955576254154948401264846324920654296875f,-0.251400005376339219509418398956768214702606201171875f,-0.5397999976992604587167079444043338298797607421875f,0.33480001742467335734687594595015980303287506103515625f,1.5001000040374687927169361501000821590423583984375f,0.777599981778859916659030204755254089832305908203125f,1.022799962472916046607451789896003901958465576171875f,0.1647000048935414184825276606716215610504150390625f,-1.216099991345405584297623136080801486968994140625f,-0.806399983471631998099837801419198513031005859375f,0.72059997777044781930300132444244809448719024658203125f,0.1082999961271882138902356018661521375179290771484375f,0.908999970465898865512599513749592006206512451171875f,-0.2540999998539685833520707092247903347015380859375f,-0.0202000005498528256708823391818441450595855712890625f,0.3662000176787376659603978623636066913604736328125f,0.7652000009968873417420809346367605030536651611328125f,0.11470000236034394447415252216160297393798828125f,0.02320000235289310097641646279953420162200927734375f,-0.2443000111699102916418269160203635692596435546875f,-0.957499975487589694012058316729962825775146484375f,-0.246800010424852445112264831550419330596923828125f,0.17809999591261149731735713430680334568023681640625f,0.463799998675286762050973266013897955417633056640625f,0.4663999854087830243543066899292171001434326171875f,-0.275299998757243091773716514580883085727691650390625f,-0.1103999767735601711393655932624824345111846923828125f,-0.2932000090956687898824384319595992565155029296875f,-1.2693000017449256322521478068665601313114166259765625f,-0.05760000755786887083331748726777732372283935546875f,0.6732000054448843417986836357158608734607696533203125f,-0.15830000264644628060750619624741375446319580078125f,0.04899999926239251291093523832387290894985198974609375f,0.0985999973058699907824120600707828998565673828125f,-1.116500011518597457182977450429461896419525146484375f,-0.3160999910473822893663964350707828998565673828125f,-0.34229998827427632335229645832441747188568115234375f,-0.18319999389648433663069226895458996295928955078125f,0.63930000845044843149622693090350367128849029541015625f,0.273499994814395908093729303800500929355621337890625f,0.1233999997735022446931907325051724910736083984375f,-0.56339999906569726562111100065521895885467529296875f,1.097399969568848998591192867024801671504974365234375f,0.9284999994337557183143871952779591083526611328125f,0.955799961221218463691684519289992749691009521484375f,-1.29330004526376729501180307124741375446319580078125f,0.1048000134944917505208650254644453525543212890625f,0.536700005996227247351271216757595539093017578125f,0.920399975395202574190989253111183643341064453125f,1.01190000516772204974813575972802937030792236328125f,-0.725700005221366239993585622869431972503662109375f,0.835800037068128887085549649782478809356689453125f,1.21830000026225970799487186013720929622650146484375f,-1.0667999970883130789189863207866437733173370361328125f,-0.0455999934911728299624655846855603158473968505859375f,0.07229997884929151297228600014932453632354736328125f,-0.42919996799826609645833741524256765842437744140625f,1.37640002757906909636176351341418921947479248046875f,-0.878300041985511370512540452182292938232421875f,-0.095400003111362874363976516178809106349945068359375f,1.1004000102639199809573256061412394046783447265625f,0.273900012826919780906109735951758921146392822265625f,0.292200008618831219564526691101491451263427734375f,0.3207000235497954321317592984996736049652099609375f,-1.596300023746490381881812936626374721527099609375f,0.66959997170567486790559996734373271465301513671875f,-0.095700024890899015872491872869431972503662109375f,-0.505200031656027004345332898083142936229705810546875f,0.3227000255465508615770886535756289958953857421875f,-1.1048999984025955622968240277259610593318939208984375f,-0.335700011070072523722274127067066729068756103515625f,-0.2319000097870826948565081693232059478759765625f,-0.5938999936416744507283738130354322493076324462890625f,0.1302000004008412314959741706843487918376922607421875f,0.5814999902471900128375636995770037174224853515625f,-0.4454000019192694281855438021011650562286376953125f,0.412900028629600956975309600238688290119171142578125f,-0.1744999973773957435696502216160297393798828125f,0.8985999914571640712068756329244934022426605224609375f,0.03189999831318868217522322083823382854461669921875f,0.884700010816752868691992262029089033603668212890625f,0.4325999878883362015358216012828052043914794921875f,-0.3190000034719704391505956664332188665866851806640625f,-0.06089999783635136054726899601519107818603515625f,0.133000003553926993138389889281825162470340728759765625f,-0.17530000419616698081881622783839702606201171875f,0.4086000031173231406000923016108572483062744140625f,-0.4124999997019767050687732989899814128875732421875f,0.5261000215575097893605516219395212829113006591796875f,-0.2142000031650066471655691202613525092601776123046875f,-0.08219999893307683436205479665659368038177490234375f,0.0228000037491320473037603733246214687824249267578125f,0.4587999807566405241487927924026735126972198486328125f,-0.1060000050961971140850437222979962825775146484375f,0.546199990819394631813565865741111338138580322265625f,-0.18100000524520876155065707280300557613372802734375f,-0.5242999876633287481553225006791763007640838623046875f,0.34360000334084050965799406185396946966648101806640625f,-0.5299999751895667632339836927712894976139068603515625f,-0.37790001018941410393381374888122081756591796875f,-0.033900008505582945872447453439235687255859375f,0.069399992948770528045088212820701301097869873046875f,0.239800013196468597698185476474463939666748046875f,-0.6406000166252254768295415487955324351787567138671875f,-0.464500009112060130433974336483515799045562744140625f,0.866900014704465871062666337820701301097869873046875f,-0.976900022155046343641515704803168773651123046875f,0.28689999276250599535842411569319665431976318359375f,-0.34209997122138791070256047532893717288970947265625f,0.5000999942958355148903137887828052043914794921875f,0.5819999812841416542141814716160297393798828125f,-1.09959999539852137928619413287378847599029541015625f,1.31739997336864433918890426866710186004638671875f,-0.131100042754411560963490046560764312744140625f,1.5933000002622605961732915602624416351318359375f,-0.6111999889194965618344212998636066913604736328125f,0.878500004351138841229840181767940521240234375f,0.32210000851154330092640520888380706310272216796875f,-0.70109999305903869526446214877068996429443359375f,0.21860001786947247381931447307579219341278076171875f,-0.14300004009902522739139385521411895751953125f,0.9968999974191188329086799058131873607635498046875f,-0.13129999424219107595490640960633754730224609375f,0.338200023996829912675821105949580669403076171875f,1.04759998431205758606665767729282379150390625f,1.17889999777674692182927174144424498081207275390625f,-0.11930007286071742100830306299030780792236328125f,-0.367399976050853727116418667719699442386627197265625f,0.554699985224008429440800682641565799713134765625f,0.078200002241134658476084950962103903293609619140625f,-0.159300000739097669111288269050419330596923828125f,0.0045999995708465579724588678800500929355621337890625f,0.2258000097244978743304955060011707246303558349609375f,0.67650001703202722325158902094699442386627197265625f,0.198300000858306901818650658242404460906982421875f,0.092700002822279969905139296315610408782958984375f,0.3429000098168850652058381456299684941768646240234375f,0.2302000115394593482420759755768813192844390869140625f,-0.0934000021576881778173628845252096652984619140625f,0.24299999822676154881406773711205460131168365478515625f,0.28980001677274724869448618846945464611053466796875f,0.3545000068023800299243930567172355949878692626953125f,-0.080899986362457099176026531495153903961181640625f,-0.0186000189498065626736433841870166361331939697265625f,0.2453999989390371894160125521011650562286376953125f,-0.3570000003948805566977853231946937739849090576171875f,0.598200009095668949754553977982141077518463134765625f,0.25129999696165317146778761525638401508331298828125f,-0.6545000107884408890868144226260483264923095703125f,0.20980002943873454768208830500952899456024169921875f,-0.2746000192403794670781280728988349437713623046875f,0.012199971403181575357876909038168378174304962158203125f,0.466599999928474229449193444452248513698577880859375f,-0.1628000118330119594389771009446121752262115478515625f,0.123199998068809435380899230949580669403076171875f,-0.7947000268355013385956908678053878247737884521484375f,-0.4797000172615053514846295001916587352752685546875f,-0.28009999459385870324013012577779591083526611328125f,-0.0115000057816505574237453402020037174224853515625f,0.56720000046044560804148204624652862548828125f,-0.68429998047351858758702292107045650482177734375f,-0.42830001493990421668200951899052597582340240478515625f,0.47910002809166929438333681900985538959503173828125f,0.2946000263929366980164559208787977695465087890625f,0.7706999801874161448722588829696178436279296875f,-0.64390001141130925876865376267232932150363922119140625f,0.94720003059804458445114505593664944171905517578125f,-0.190500012010335950662920367904007434844970703125f,1.2505000145435332115084747783839702606201171875f,0.509900012856721929210834787227213382720947265625f,-1.08590000811815201586796320043504238128662109375f,0.95159998934864997721660984097979962825775146484375f,-0.96789997897148083438878529705107212066650390625f,0.259199991244077665442091529257595539093017578125f,-1.1497999684929851582637638784945011138916015625f,-0.75039996973276146263742703013122081756591796875f,-0.590599988573789147494608187116682529449462890625f,0.89509996494054799853756776428781449794769287109375f,0.10160000052452033969530020840466022491455078125f,-0.12519997637271895740695981658063828945159912109375f,1.43739999303817711506781051866710186004638671875f,-0.0813000137627124530581568251363933086395263671875f,-0.783899967080355253301604534499347209930419921875f,0.11679999760985371182187009253539144992828369140625f,-0.96679997928142569207921042107045650482177734375f,-0.2478000189483164916737223393283784389495849609375f,-0.59570002727508608586504124104976654052734375f,-0.85449999067187309975679454510100185871124267578125f,1.15339998040199276374551118351519107818603515625f,0.641999975770711994726980265113525092601776123046875f,-0.01210001536607752115060065989382565021514892578125f,0.1159999931380151227955366266542114317417144775390625f,0.6652000210762025744770653545856475830078125f,-0.6108999941110606624050660684588365256786346435546875f,-0.4295000131726265379938922706060111522674560546875f,0.11970000157803273310719305300153791904449462890625f,-0.9477000198841096789692528545856475830078125f,-0.461799997833371111966016542282886803150177001953125f,0.0297999994874000861955209984444081783294677734375f,0.226799997386336382731997218797914683818817138671875f,0.37080000681877134383057637023739516735076904296875f,0.2704999935328959992375530418939888477325439453125f,0.2514999968409536990066044381819665431976318359375f,0.4215000043660401818357286174432374536991119384765625f,-0.0002999939143657481821492183371447026729583740234375f,-0.3192999976471064815797262781416065990924835205078125f,-0.213500007331371310925760553800500929355621337890625f,0.0689000060841440387093825847841799259185791015625f,-0.5657000117778778758292901329696178436279296875f,-0.4145999975591900810201195781701244413852691650390625f,0.011400003886222975779674015939235687255859375f,0.231199995103478439517630249611102044582366943359375f,0.352000007063150377462079632095992565155029296875f,0.67290000136047589673893298822804354131221771240234375f,0.19110000863075260468804117408581078052520751953125f,-0.2260999935060741261594330353545956313610076904296875f,-0.115899990236759009576417156495153903961181640625f,0.1306999868184328672526817172183655202388763427734375f,0.405700004327297136796914855949580669403076171875f,0.1485000056922434197304028202779591083526611328125f,-0.31280000687837627992848865687847137451171875f,-0.44970000236034390894701573415659368038177490234375f,-0.10849997156858393054790212772786617279052734375f,-0.0084999973475934353928096243180334568023681640625f,-0.731399987792968886424205265939235687255859375f,1.131199993985891438796898000873625278472900390625f,-0.154300060939789318581460975110530853271484375f,-1.16990000838637353552940112422220408916473388671875f,1.26790000519752510399484890513122081756591796875f,-0.402100013726949701009516502381302416324615478515625f,0.717900034403800901827708003111183643341064453125f,-0.08039999506473538559703229111619293689727783203125f,0.37929999626874888463134993799030780792236328125f,0.34979998361766340142509079669252969324588775634765625f,0.37650001807510857787519853445701301097869873046875f,0.1928000143289565482263014928321354091167449951171875f,-0.8429000101149082269103018916212022304534912109375f,0.770500006496906308939287555404007434844970703125f,-0.3102000392556192309712059795856475830078125f,-0.99769999201893799778417815105058252811431884765625f,1.05449996250867883418322890065610408782958984375f,-0.3896000177502632055848152958787977695465087890625f,0.74240003342032423461205326020717620849609375f,0.978000006049871473123857867904007434844970703125f,-0.73829999638795840155580663122236728668212890625f,-0.60320001803636547066389539395458996295928955078125f,0.96710000106096316585535532794892787933349609375f,-0.09520000885725021788630328956060111522674560546875f,-0.29929996765851996087803854607045650482177734375f,0.6622999795198440953214458204456605017185211181640625f,-0.261099991047382129494280889048241078853607177734375f,-0.12710000184327352457103188498876988887786865234375f,0.07669999124407755886068116524256765842437744140625f,-0.1047000054895879106453548956778831779956817626953125f,-0.050099996978044458728618337772786617279052734375f,0.17989999570175996268517337739467620849609375f,0.2341000117935241320310524315573275089263916015625f,0.5154000107854610490676350309513509273529052734375f,-0.184200021493435173169928020797669887542724609375f,0.1200000020489095930997791583649814128875732421875f,-0.447599998319148983938475794275291264057159423828125f,0.71300000921636819128934803302399814128875732421875f,0.341800008785724696025454250047914683818817138671875f,-0.62300000277161604600451028090901672840118408203125f,0.299000001609325494200675166212022304534912109375f,-0.281699989008903639575720490029198117554187774658203125f,-0.33389999963939176463867397615103982388973236083984375f,0.5736000046446918521070301721920259296894073486328125f,0.17160000678300857401836765347979962825775146484375f,-0.69049999185651511623973419773392379283905029296875f,-0.3535000112056732035625827847979962825775146484375f,-0.76329999966621364659857817969168536365032196044921875f,-0.11870001797676099641876135137863457202911376953125f,0.0052000146687027903880107260192744433879852294921875f,-0.1491999902009961687099348637275397777557373046875f,0.3541000148668886904346209121285937726497650146484375f,-0.1669000065088273032642973703332245349884033203125f,-0.38639999777674649550363028538413345813751220703125f,-0.21120001492202289394839453962049447000026702880859375f,0.2173999979183076636246596535784192383289337158203125f,0.646799963635206864864812814630568027496337890625f,-0.525199994105100653740691996063105762004852294921875f,0.652899996107816349422137136571109294891357421875f,0.24820000075101855685488771996460855007171630859375f,-0.72890000910162910940925939939916133880615234375f,0.3569999978244304816854537421022541821002960205078125f,-0.8002999844707545662458869628608226776123046875f,1.3735000295341013298866528202779591083526611328125f,-1.2426999912738807552159414626657962799072265625f,0.8199999767541885642430088410037569701671600341796875f,-0.5327999930202960143788004643283784389495849609375f,0.662000013172626466939618694595992565155029296875f,-0.250599984699487521311311866156756877899169921875f,-1.4106999827206134767720868694595992565155029296875f,0.7858999804019930479626054875552654266357421875f,-0.28289999659210440796641705674119293689727783203125f,0.57139996875673570997378192259930074214935302734375f,0.151499988198280366447079359204508364200592041015625f,0.863900014317035669364486238919198513031005859375f,-0.43279998854994783386018752935342490673065185546875f,-0.17090000841617580817910493351519107818603515625f,-1.0493999844551087363697661203332245349884033203125f,0.49489999035596810017523239366710186004638671875f,-0.22649999639391893424544832669198513031005859375f,0.8350999703049666322840494103729724884033203125f,-0.0884000106513500583105269470252096652984619140625f,1.51970001130104037656565196812152862548828125f,0.216100031541287818104279949693591333925724029541015625f,-1.082700024764239987717928670463152229785919189453125f,-0.383700018274784060423598930356092751026153564453125f,-0.6178000040471551823628715283120982348918914794921875f,-0.539499984376132335484044233453460037708282470703125f,-0.209299998056888370001615840010344982147216796875f,-0.6639000188618899844783527441904880106449127197265625f,0.138299998585879768686623947360203601419925689697265625f,-0.2087000046402216479890512346173636615276336669921875f,-0.136899992538988490853313351180986501276493072509765625f,0.74180001903027292797787595191039144992828369140625f,-0.1261999971151350763420850853435695171356201171875f,0.25260000871270904543308688516844995319843292236328125f,-0.1122000031054020130483195316628552973270416259765625f,-0.0036000088542700314775402148370631039142608642578125f,0.0828000085175037980889101163484156131744384765625f,-0.17160000920444729022307228660793043673038482666015625f,0.060000002384185791015625f,-0.455000001639128071051487722797901369631290435791015625f,-0.387399998998641859060398928704671561717987060546875f,0.590700022506714272907402119017206132411956787109375f,0.183299994301795887707839938229881227016448974609375f,0.196800027076899919364905144902877509593963623046875f,-0.826799997758865146124662715010344982147216796875f,0.31799999945610746276969393875333480536937713623046875f,0.0312000026285648157653440648573450744152069091796875f,-0.317799987655878302117429257123149000108242034912109375f,-0.351799996715783880318895171512849628925323486328125f,0.0734999999925494140740056536742486059665679931640625f,-0.167100005903840109766633759136311709880828857421875f,-0.29349999380856761632685447693802416324615478515625f,-0.548900009250640863456283113919198513031005859375f,-0.387499995417892595384046217077411711215972900390625f,0.6956999692350631647741465712897479534149169921875f,-0.36789998433589943260813015513122081756591796875f,-0.210600008392333393203443847596645355224609375f,-0.9028000049598514653581560196471400558948516845703125f,-0.107900005029887058327631166321225464344024658203125f,0.1645999797709286338598388965692720375955104827880859375f,-0.6420999814100560865171019031549803912639617919921875f,1.63900003603100774540735073969699442386627197265625f,-1.33680002934932673497314681299030780792236328125f,0.42630000593960282184724519538576714694499969482421875f,-0.01510000137388711749508729553781449794769287109375f,0.3439999867081642292987453402020037174224853515625f,-0.9491999778330324488706537522375583648681640625f,-0.054500008106231678794983963598497211933135986328125f,-0.476099989259243017158951261080801486968994140625f,-0.3181000077962874428294526296667754650115966796875f,0.290499961048364951921030296944081783294677734375f,0.1237000197649000821087383883423171937465667724609375f,0.9472000189006326564822302316315472126007080078125f,1.1349999941885471343994140625f,-0.210600023293494587051100097596645355224609375f,0.17179999291598802013680824529728852212429046630859375f,0.42099999623000616821855146554298698902130126953125f,-0.336199996072053863827022723853588104248046875f,0.629199961888790681996397324837744235992431640625f,0.5930000130534172075869037144002504646778106689453125f,-0.7563000199466944906134813209064304828643798828125f,0.1387000036716461437436009873636066913604736328125f,0.0643999609112739790361956693232059478759765625f,0.08250004239380348547427956873434595763683319091796875f,0.73589999589920029876566331950016319751739501953125f,-0.6315999926641584494291237206198275089263916015625f,-0.09199999640881995066621357182157225906848907470703125f,0.0506999896124003812669656099387793801724910736083984375f,0.3741000112533570121087223014910705387592315673828125f,-0.3192000062942501603657774467137642204761505126953125f,0.92399999803304666556869051419198513031005859375f,-0.2718999986484647024553851224482059478759765625f,-0.2249000034689903060325377737171947956085205078125f,-0.324000009767711372177245721104554831981658935546875f,0.17639999852180476835883382591418921947479248046875f,0.54249999139457916186302099958993494510650634765625f,0.177499998509883738506687222979962825775146484375f,-0.4130000210627915446792712828028015792369842529296875f,0.12859999939203259344822072307579219341278076171875f,-0.66520002010762713240410448634065687656402587890625f,-0.19159999337196342850120345246978104114532470703125f,0.65910001550018793015084384023793973028659820556640625f,-0.13709999688863749867095975787378847599029541015625f,-0.051399999862909329095828070421703159809112548828125f,0.23140000925064097003769347793422639369964599609375f,0.3969000236451625251987707088119350373744964599609375f,-0.71009999355077724203511024825274944305419921875f,0.28699999666959019606338188168592751026153564453125f,-0.40580000421106821040240220099803991615772247314453125f,-0.5702999949201938978404768931795842945575714111328125f,0.22469999431371689979641814716160297393798828125f,0.223400023801624936226062345667742192745208740234375f,-0.8534000149726868045263472595252096652984619140625f,-0.052499974817037664553964759761583991348743438720703125f,-0.43949996787309686396838515065610408782958984375f,-0.8167999856516718892862627399154007434844970703125f,0.28199997260421572065069995005615055561065673828125f,-0.0240999994069337475366410217247903347015380859375f,-0.6077999905616042752853900310583412647247314453125f,-1.37429995678067218278783911955542862415313720703125f,0.19890001163482651236336096189916133880615234375f,-0.10729999918937682679143108543939888477325439453125f,-0.02400001442432397880111238919198513031005859375f,-1.064200000780820776213886347250081598758697509765625f,1.096599978768825423003363539464771747589111328125f,0.2918999990582467063404692453332245349884033203125f,0.4551999835252757975467829965054988861083984375f,-0.143699992942809995355446517351083457469940185546875f,-1.084300020110607221113241394050419330596923828125f,-0.282400032973289416560191966709680855274200439453125f,-0.781199989068507960610077134333550930023193359375f,1.0463000024378299457339380751363933086395263671875f,-0.22579999340772616278627538122236728668212890625f,-0.385300005537271506028673684340901672840118408203125f,0.218900018191337863981971167959272861480712890625f,1.31909998160004615641582859097979962825775146484375f,-0.32159999933242744418748770840466022491455078125f,0.9948999951615931092163691573659889400005340576171875f,-0.553700032915175111014605136006139218807220458984375f,-1.26989997813701638307293251273222267627716064453125f,0.77659999694824222160605131648480892181396484375f,1.16600000658631319794267255929298698902130126953125f,-0.14980002839565287331424769945442676544189453125f,0.04980001211613416245427288231439888477325439453125f,0.1144000100851059187334612943232059478759765625f,-0.1398000056341286434502535485080443322658538818359375f,0.6114000000119208522164626629091799259185791015625f,0.323500031433999968299985994235612452030181884765625f,-0.597100015366077485623463871888816356658935546875f,0.0164999993927776866797785260132513940334320068359375f,0.1742999954305588516945135779678821563720703125f,0.495900014823675461261842656313092447817325592041015625f,0.047699996936321209517473107553087174892425537109375f,-0.3534999962300060527553569045267067849636077880859375f,0.1051000057697297762615562533028423786163330078125f,0.27200000154972059362989966757595539093017578125f,-0.23599999488890166521315450154361315071582794189453125f,0.48900002250820417959431551935267634689807891845703125f,-0.384600003445148441727496901876293122768402099609375f,-0.259100036622584184442530386149883270263671875f,0.8999000153899192611106627737171947956085205078125f,0.0227999980121851297809598690946586430072784423828125f,-0.24600000286102297053503207280300557613372802734375f,-0.325300009746849416725211767698056064546108245849609375f,0.210099997127056070667094900272786617279052734375f,-0.0008000008583068929368664612411521375179290771484375f,-0.168400001002848132003464343142695724964141845703125f,-0.2882000359922647358956737662083469331264495849609375f,0.43219999043941470517893321812152862548828125f,-0.04799999420344835954210793715901672840118408203125f,-0.53519999634027470847286167554557323455810546875f,0.03700000512599921353995569006656296551227569580078125f,-0.2313999973297118373238845379091799259185791015625f,0.18000000469386578938468801425187848508358001708984375f,-1.22399997270107263602767488919198513031005859375f,-0.7612999909639359685797899146564304828643798828125f,-0.001499953627586592119769193232059478759765625f,0.719700010406970847043339745141565799713134765625f,0.5647999979972837536479346454143524169921875f,-0.3811999859392642964195374588598497211933135986328125f,0.5172999860204754440928809344768524169921875f,0.49400000861287107767338966368697583675384521484375f,0.157500021308660365093601285479962825775146484375f,-0.0385999835968018345511154620908200740814208984375f,-1.10449999022483780208858661353588104248046875f,0.6071999812379480498947259548003785312175750732421875f,-0.098399998022615786652522729127667844295501708984375f,1.0006000039219855590744145956705324351787567138671875f,-0.0599999968707560782377186114899814128875732421875f,-1.15070001565217960859399681794457137584686279296875f,-0.08099997484683996162857511080801486968994140625f,0.732199997889995568556287253159098327159881591796875f,-1.034600000464916291775807621888816356658935546875f,-0.3860999919414520586968819770845584571361541748046875f,-0.4817999929904939193647805950604379177093505859375f,0.5887000041186809529136780838598497211933135986328125f,-0.9469000069186090939865607651881873607635498046875f,-0.09859999074935910101658009807579219341278076171875f,-1.49239999899864272947525023482739925384521484375f,0.640799973738193529015916283242404460906982421875f,-0.40719999192953082456369884312152862548828125f,0.027199994015693729210170204169116914272308349609375f,0.535399977183342201669802307151257991790771484375f,-0.098600006544590002022232511080801486968994140625f,-0.0030999952793120399974213796667754650115966796875f,0.24070002731233852699688213760964572429656982421875f,-0.7663000124216079456118677626363933086395263671875f,0.6024000065237287149244593820185400545597076416015625f,0.085199994738399842386655791415250860154628753662109375f,0.0154000051230192003259844568674452602863311767578125f,0.150199994179606388655656701303087174892425537109375f,0.7240999965384602443663197846035473048686981201171875f,-0.07610000177621856209952966310083866119384765625f,0.4630000050440432612930408140528015792369842529296875f,-0.2578000018119812608574648038484156131744384765625f,-0.74800001540035054858890362083911895751953125f,-0.2491999934792517024106928147375583648681640625f,0.245100008787214440619806055110529996454715728759765625f,-0.525900001859664900649704577517695724964141845703125f,0.18949999685585472430915388031280599534511566162109375f,0.54850002148747467600742311333306133747100830078125f,0.05840000081658358599412395051331259310245513916015625f,-0.162799998496472808628965367461205460131168365478515625f,0.33970001141279915213289086750592105090618133544921875f,-0.70690000293254851015944950631819665431976318359375f,0.25890001740902679472355885081924498081207275390625f,-0.56850001746416101156000877381302416324615478515625f,0.29590001646280306157876793804462067782878875732421875f,-0.256300003778934470943795531638897955417633056640625f,-0.1763000193506483181948851779452525079250335693359375f,0.5903000235676767459835900808684527873992919921875f,0.3725000016763804211450406000949442386627197265625f,0.31150001762807388860920809747767634689807891845703125f,-0.6006000121921299950145112234167754650115966796875f,-0.47189997491836521703589824028313159942626953125f,1.23520001884102814670995940105058252811431884765625f,-0.44210003294944755225515109486877918243408203125f,0.74230000323131672956833426724188029766082763671875f,-0.182099973065406484096229178248904645442962646484375f,-0.5827999790132045720980613623396493494510650634765625f,1.0125999732106922834873330430127680301666259765625f,1.16709998496770861464710833388380706310272216796875f,-0.44639998362064403636395581997931003570556640625f,1.2343999869883059972153205308131873607635498046875f,-0.9201999716043474109028466045856475830078125f,-0.38830000979900347601869725622236728668212890625f,-0.64950003671646072689327411353588104248046875f,0.3704999836981297534066470689140260219573974609375f,0.972399966663122228283100412227213382720947265625f,-0.0968999914586543553696174058131873607635498046875f,-0.44019999440312318483847775496542453765869140625f,0.36299999265745286347595310871838591992855072021484375f,-0.2969999992586671577754486861522309482097625732421875f,0.9631999791443348613029229454696178436279296875f,0.30409997817277911735800444148480892181396484375f,1.174100029134750311499146846472285687923431396484375f,-0.679800040614605194377872976474463939666748046875f,0.5802000048339366333749467230518348515033721923828125f,0.0701000020444393356910950387828052043914794921875f,-0.63770001407265652915157261304557323455810546875f,-0.3299999839067453422103426419198513031005859375f,0.4020000049397349783930621924810111522674560546875f,-0.53949999644607249393857273389585316181182861328125f,-1.59280000926256182225415614084340631961822509765625f,0.0428999908924101447382781771011650562286376953125f,-0.0841999953791499056166003356338478624820709228515625f,-0.2506000068277121073379021254368126392364501953125f,0.56369998813718513730464110267348587512969970703125f,0.149100020194053772826237036497332155704498291015625f,-0.0482999791398643907314180978573858737945556640625f,-0.67890000090599045279304846189916133880615234375f,-0.719000018708408017431565895094536244869232177734375f,0.002599998021125760061522669275291264057159423828125f,0.2910999934688209833666405756957828998565673828125f,0.364099999332427870513129164464771747589111328125f,0.4272000168889762417023803209303878247737884521484375f,-0.3192000082314014886009090332663618028163909912109375f,0.237200028325617484625809083809144794940948486328125f,-0.03919999169111232362183727673254907131195068359375f,0.5428000060960649353347662326996214687824249267578125f,0.1232000006020068649803533844533376395702362060546875f,0.6073000202372671463280084935831837356090545654296875f,-0.0313000006496906291175719161401502788066864013671875f,0.2825000201538205857332286541350185871124267578125f,-0.59210000888407232633170451663318090140819549560546875f,0.2024999862536789319023000643937848508358001708984375f,0.6615000025779007586379520944319665431976318359375f,0.42720001826733378091915938057354651391506195068359375f,0.1812000003933906810971166123636066913604736328125f,0.43510001124590635956934647765592671930789947509765625f,-0.37030000203549862103358236709027551114559173583984375f,0.7213000191643839098531998388352803885936737060546875f,-0.174100016319751915716551593504846096038818359375f,-0.5044000253960494450922169562545605003833770751953125f,-1.040400013244152432889677584171295166015625f,-0.14230000306367873719182171043939888477325439453125f,0.621099987098574501942493952810764312744140625f,0.81269997264742865894504575408063828945159912109375f,-0.580399990743398319636980886571109294891357421875f,0.68589999411106106208535493351519107818603515625f,-0.200800010097026415678556077182292938232421875f,-1.0425000135600566153470936114899814128875732421875f,0.341299987536668680832008249126374721527099609375f,-0.2576999945521354362654165015555918216705322265625f,0.5877999985337254429396125487983226776123046875f,0.86430001362860198188542426578351296484470367431640625f,-0.06650001375377190271365179796703159809112548828125f,-0.233700000095367510510868669371120631694793701171875f,1.249500018537045065158963552676141262054443359375f,0.11610000274479392867732485683518461883068084716796875f,0.96169999102056014095296632149256765842437744140625f,0.34690000554025168799654466056381352245807647705078125f,0.95660000506937503672588718472979962825775146484375f,0.61259999694079159127113598515279591083526611328125f,0.67949998161941760344006979721598327159881591796875f,-0.42779999898076059849927332834340631961822509765625f,0.087100034663080805330537259578704833984375f,0.180500018417835139672433797386474907398223876953125f,0.379300014448165967451132019050419330596923828125f,1.00399999531358474769149324856698513031005859375f,-0.245300004009902483659288918715901672840118408203125f,1.51230003405809387828639955841936171054840087890625f,-1.00959999778270770320887095294892787933349609375f,-0.07340000558495507476663988200016319751739501953125f,-0.2116999939262866892164538512588478624820709228515625f,0.492499999478459127288942909217439591884613037109375f,0.137500014007091664325344027020037174224853515625f,0.01629998686611655589473457439453341066837310791015625f,0.08809999527931200447028459166176617145538330078125f,-0.74039998075962054624454822260304354131221771240234375f,-0.33959999957084630040071715484373271465301513671875f,0.5099999934807415424842247375636361539363861083984375f,0.494400023794174359181852196343243122100830078125f,-0.51950000490248182671848553582094609737396240234375f,0.3430999948322772663544810711755417287349700927734375f,0.8300000104680662627032461386988870799541473388671875f,0.375999991834163438397808931767940521240234375f,0.22960000273734315623386237348313443362712860107421875f,0.14120001142025007112579260137863457202911376953125f,0.29759999563693984026002681275713257491588592529296875f,0.19200000959634788699759155861102044582366943359375f,0.1454999951533972879502698560827411711215972900390625f,0.3369000060431659449022845365107059478759765625f,0.2605000336542726557809146470390260219573974609375f,-0.27490000615119924987084232270717620849609375f,0.233699991322308786578787476173602044582366943359375f,0.144300012715905978666341979987919330596923828125f,0.0250999976672232616436986063490621745586395263671875f,-0.194300006941706016050375183112919330596923828125f,0.2644999987184999401534923890721984207630157470703125f,0.624700016888976250584164517931640148162841796875f,-0.001400001949071860707363157416693866252899169921875f,-0.164199999886751157873732154257595539093017578125f,0.2731000243365768742620502962381578981876373291015625f,-0.17970000839531419245531651540659368038177490234375f,0.992400003468990377086811349727213382720947265625f,-1.148300000560283251616056077182292938232421875f,0.28950000050663948769624767010100185871124267578125f,-0.785399995660782224149443209171295166015625f,-0.52199997800588615604056030861102044582366943359375f,-0.13920002447366641717962920665740966796875f,-0.12600002059340464910519585828296840190887451171875f,0.61520001630783127666290965862572193145751953125f,-0.96629997583925730797460573739954270422458648681640625f,-0.01060000191032894889531235094182193279266357421875f,0.48770001094341264291642801254056394100189208984375f,0.38210004606246883440689998678863048553466796875f,0.1872999982953071906877084984444081783294677734375f,-0.08989999661445580159124801866710186004638671875f,0.57680000401735309623063585604541003704071044921875f,-0.875999999284744035321637056767940521240234375f,-0.0184999937713146156426091692992486059665679931640625f,-0.2817999916307627472633612342178821563720703125f,-0.06599997052550321541275479830801486968994140625f,1.44529996247291592226247303187847137451171875f,0.7985999813616275755379092515795491635799407958984375f,-1.44009995814189295515461708419024944305419921875f,0.5859000063300132676857856495189480483531951904296875f,-0.8146000103555621763007366098463535308837890625f,-0.4190999896466731655664261779747903347015380859375f,0.31580000570118471614478039555251598358154296875f,0.6320999988257884893982918583787977695465087890625f,-1.053699997264146759334835223853588104248046875f,1.0487000151455401208977491478435695171356201171875f,-0.0177999961495398650868082768283784389495849609375f,0.173600005538761426038263380178250372409820556640625f,-0.52279999972581858713738256483338773250579833984375f,-0.0723999979183076458610912595759145915508270263671875f,-0.65120002319216752084685140289366245269775390625f,-0.546300006386637893029956103418953716754913330078125f,0.038699992942810013119014911353588104248046875f,-0.2107999955311416206971131259706453420221805572509765625f,-0.390400013765692832645726184637169353663921356201171875f,-0.363700004118680642051231188816018402576446533203125f,0.48970000236034394447415252216160297393798828125f,0.50090000297725223443734421380213461816310882568359375f,0.30189999995231620033564468030817806720733642578125f,0.18280000512748950658448165995650924742221832275390625f,-0.0855999948322772041819916921667754650115966796875f,-0.3327000174999239501261172335944138467311859130859375f,0.46590001288652427291481217253021895885467529296875f,0.5929000101894141838698715218924917280673980712890625f,-0.30850001388788239609084484982304275035858154296875f,0.406800002489983825171293574385344982147216796875f,0.576000020444393356910950387828052043914794921875f,-0.4025000035762786865234375f,0.4709000108003618123575506615452468395233154296875f,-0.226299996775388534420159203364164568483829498291015625f,0.402700005578994879584797672578133642673492431640625f,0.138499986823648502110017943778075277805328369140625f,0.5742999904386696385927280061878263950347900390625f,0.185099999324977171344386306373053230345249176025390625f,-0.242699998128414140552422395558096468448638916015625f,-0.03599999250471608291945813107304275035858154296875f,1.201500001311302412432269193232059478759765625f,0.825199984568357525205328784068115055561065673828125f,-0.237200016069412100705449120141565799713134765625f,0.57940000159144400271316044381819665431976318359375f,-0.08980002914071150144081912003457546234130859375f,-1.3085999907493590654894433100707828998565673828125f,1.3254999579191206748873810283839702606201171875f,-0.151100003955513262354770631645806133747100830078125f,0.593699988304823289553269205498509109020233154296875f,-0.8829999904036522906380923814140260219573974609375f,-0.0830999654769897233563824556767940521240234375f,0.395400006687641081271067378111183643341064453125f,-0.15369997237920784982634359039366245269775390625f,0.574400003081560139861494462820701301097869873046875f,-0.590800003093480796678704791702330112457275390625f,-1.121500017851591035622504932689480483531951904296875f,0.581000015676021774879700387828052043914794921875f,1.309900018817186317932055317214690148830413818359375f,-0.624400018727779837490743375383317470550537109375f,-0.1715999947130679714746293029747903347015380859375f,0.2600000122189527473892667330801486968994140625f,-0.69139997855424883965724802692420780658721923828125f,-0.57349999749660440784282400272786617279052734375f,-0.30809996964931496421513656969182193279266357421875f,-0.795999993771314251489457092247903347015380859375f,-0.6487999942421913690537849106476642191410064697265625f,0.5885000286586574702596408315002918243408203125f,0.43839998695850379561989029753021895885467529296875f,0.052699986952543298457385390065610408782958984375f,0.034199998545646703718148273765109479427337646484375f,-0.3138000059396028884606266728951595723628997802734375f,0.27200000296533077825955615480779670178890228271484375f,-0.14959999152421943335866671986877918243408203125f,0.31020000230073929703422663806122727692127227783203125f,0.43140001223087320880722472793422639369964599609375f,-0.4803000178679825982186457622447051107883453369140625f,-0.138899992859363496933156056911684572696685791015625f,-0.01480000079125165513005413231439888477325439453125f,0.24159999411702148819358626496978104114532470703125f,-0.496300010111928313616402874686173163354396820068359375f,-0.058099995614588173520331793042714707553386688232421875f,0.21080000234842299899895579073927365243434906005859375f,0.11320000365674498521428859021398238837718963623046875f,0.176099993189424264272702203015796840190887451171875f,0.10470000398084522430508513934910297393798828125f,-0.17420003322809940460302868814324028789997100830078125f,0.43699999976158121484104412957094609737396240234375f,-0.7267999901592732125976681345491670072078704833984375f,-0.482800005388259734218081575818359851837158203125f,0.0169000004552304350369240637519396841526031494140625f,-0.17529999817982311327568822889588773250579833984375f,-0.164400010681151897973251152507145889103412628173828125f,0.553199990916251937989045472932048141956329345703125f,0.34560000060647710906636120853363536298274993896484375f,0.0180000106692315142709048814140260219573974609375f,0.289299999248981531962954250047914683818817138671875f,0.1175000089406967873628673260100185871124267578125f,-0.043000003330409430901681844261474907398223876953125f,0.33459999301433551721629555686376988887786865234375f,0.13759999489188190491262275827466510236263275146484375f,-0.65710001864433298379708503489382565021514892578125f,0.105899976676702589628575879032723605632781982421875f,0.827899996256828529794802307151257991790771484375f,-0.015799996089935319787400658242404460906982421875f,-0.03999997705221147725751507095992565155029296875f,-0.1862000322818755382314748203498311340808868408203125f,-0.6020999866068361683346665813587605953216552734375f,-0.452599975818395616755651644780300557613372802734375f,0.446199976998567393593475571833550930023193359375f,-0.640900008211284788472994478070177137851715087890625f,0.189199977628141891017321540857665240764617919921875f,0.218800003220140926174508422263897955417633056640625f,-0.15119999372512094026888007647357881069183349609375f,0.7188000043749809275794859786401502788066864013671875f,-1.33589999215528365539285005070269107818603515625f,-0.44240002239346498669192442321218550205230712890625f,-0.429399969851971263778978027403354644775390625f,-0.32199998068809509987886485760100185871124267578125f,-0.23920004801750138767602038569748401641845703125f,0.4965999969482421949606987254810519516468048095703125f,-0.679700001522153485211674706079065799713134765625f,-0.56969998090267193191493788617663085460662841796875f,-0.368899967974424924932463909499347209930419921875f,0.630099989974498786438061870285309851169586181640625f,-0.491099990153312404572716332040727138519287109375f,0.74969999565482137171557042165659368038177490234375f,-1.12259997268915157064839149825274944305419921875f,-0.601099978530406975352207155083306133747100830078125f,0.499499989628791496443227515555918216705322265625f,-0.31479998756647109559025921043939888477325439453125f,-0.39520000587701797911677203956060111522674560546875f,-0.72170001065284028474167143940576352179050445556640625f,0.10250000000000003552713678800500929355621337890625f,-0.3657999990701672476944850131985731422901153564453125f,0.7617999980568885831644365680404007434844970703125f,0.1866999901823700458436405824613757431507110595703125f,0.058099990864842965265779639594256877899169921875f,0.5820999943554399091993900583474896848201751708984375f,0.2135000106096267558086765347979962825775146484375f,0.2368999968603251460308456444181501865386962890625f,-0.46010000614225876613971877304720692336559295654296875f,-0.7102000009968871818699653886142186820507049560546875f,0.26019999037981023093379917554557323455810546875f,-0.40349999909847955592567814164794981479644775390625f,0.2595000039339065978083453956060111522674560546875f,0.27449999432265759224947032635100185871124267578125f,0.153900005972385400809798738919198513031005859375f,0.542000007919967163871888260473497211933135986328125f,-0.0631999864459036331254537799395620822906494140625f,-0.2015999981775877802903096380759961903095245361328125f,0.392400001978874257702045724727213382720947265625f,-0.2639000071644781453272798898979090154170989990234375f,0.09469998075365992917795665562152862548828125f,-0.3807000169932839295228177434182725846767425537109375f,0.84509999146461467489643837325274944305419921875f,-0.0367999977588653326421308520366437733173370361328125f,-0.31600001053512105642084861756302416324615478515625f,-0.095100015008449556574987582280300557613372802734375f,0.0555000011324882080998577293939888477325439453125f,0.5670999970376489063283997893449850380420684814453125f,-0.986100009524822240791763761080801486968994140625f,-1.2780000012814998200383342918939888477325439453125f,0.64229998339414606789432582445442676544189453125f,-1.375899954324960816620659898035228252410888671875f,0.4218999999523163069170550443232059478759765625f,0.7755000173747539360391556328977458178997039794921875f,-1.33300000744685576137271709740161895751953125f,0.2321999740481377472178792231716215610504150390625f,0.618700020956993057552608661353588104248046875f,0.5057999729558826373931879061274230480194091796875f,0.50949998757988179676203799317590892314910888671875f,-1.52909999724626555206441480549983680248260498046875f,0.89310003521442382634631940163671970367431640625f,-1.24940001261830335721469964482821524143218994140625f,1.11230003435611735085331019945442676544189453125f,0.4692999901592731504251787555404007434844970703125f,-0.6193999755144119490068987943232059478759765625f,1.14869998579025267559927669935859739780426025390625f,-0.93049996984004934574841172434389591217041015625f,-0.52699997696280487247122437111102044582366943359375f,-0.22139999017715439322273596189916133880615234375f,-0.92419998751878740250731425476260483264923095703125f,1.12509998207688255433822632767260074615478515625f,-1.34699996608495720096243530861102044582366943359375f,0.13910000946521705600389395840466022491455078125f,0.262600004279613497004675082280300557613372802734375f,-0.007000020995736377926732529886066913604736328125f,0.43129999379515648882943423814140260219573974609375f,-1.33419998662471783745786524377763271331787109375f,0.57039999517649420113230007700622081756591796875f,-0.1756000039219856034833355806767940521240234375f,-0.061800000217556938508778330287896096706390380859375f,0.29220000534057621877082056016661226749420166015625f,0.3232999919548629197407763058436103165149688720703125f,0.1951000057697296341530091012828052043914794921875f,0.35300001912564038519803943927399814128875732421875f,-0.7742000122547150198215604177676141262054443359375f,0.0039000099584460823365361648029647767543792724609375f,0.09689999607801436098952763131819665431976318359375f,0.23799999539554106409156020163209177553653717041015625f,0.45320001378953467252586051472462713718414306640625f,-0.54669999873191132966354643940576352179050445556640625f,0.09310000511407867662683202070184051990509033203125f,0.1743000059545039004404998195241205394268035888671875f,-0.235500002622604398538896930404007434844970703125f,-0.2282000012353060380831948350532911717891693115234375f,0.0949999822676179661584683344699442386627197265625f,-0.185600037747621737782566242458415217697620391845703125f,0.56720000556409344572017516838968731462955474853515625f,0.28759999217241993818561240914277732372283935546875f,0.756400007872283453025374910794198513031005859375f,-0.34919999966323389539724075802951119840145111083984375f,-0.30600000345706934723466474679298698902130126953125f,0.2822999879576266746283863540156744420528411865234375f,0.7185000137574970580089939176104962825775146484375f,-0.2132000141620638000716780879884026944637298583984375f,0.3571999977409838766817529176478274166584014892578125f,0.43010001776516448845910645104595459997653961181640625f,-0.46250000685453418469705866300500929355621337890625f,-0.5573000051125884812819322178256697952747344970703125f,-1.0992999947786330494636786170303821563720703125f,-0.590499998897314082313414473901502788066864013671875f,0.515300011348724495974238379858434200286865234375f,0.785200011092424432490588515065610408782958984375f,-1.2743000066995620045418036170303821563720703125f,1.05699999871850014443452892010100185871124267578125f,0.27639996752738937857429846189916133880615234375f,-0.650199986654520056816863871063105762004852294921875f,1.244499986648559541890790569595992565155029296875f,-0.0387999984145164233950708876363933086395263671875f,-0.2327999850481745625074836425483226776123046875f,-1.0480999914050102006513043306767940521240234375f,0.63079997910261198512671398930251598358154296875f,0.02359998955726627656304117408581078052520751953125f,0.92709998764991752295827609486877918243408203125f,-0.714899977540969899791889474727213382720947265625f,0.758099981719254856216139160096645355224609375f,-0.48470002027899017349454879877157509326934814453125f,-0.64509997667521279396396494121290743350982666015625f,-0.489099979215860347636635196977294981479644775390625f,0.252500002272427082061767578125f,-0.04089999455809589790078462101519107818603515625f,-0.20890003190040573599617346189916133880615234375f,-0.4496000189423561277379803868825547397136688232421875f,0.23150006194040173568282625637948513031005859375f,-0.95359999112188809089474261782015673816204071044921875f,0.76759997693598247536783674149774014949798583984375f,0.8676999976813792869734243140555918216705322265625f,0.24019999455213536521114292554557323455810546875f,-0.09190000829696653994460575631819665431976318359375f,0.20259999235868431100016096024774014949798583984375f,-0.313200000900029695838355792147922329604625701904296875f,0.46439999615252014475430542006506584584712982177734375f,-0.0890000001564621445737657268182374536991119384765625f,0.3445999985277652388759861423750407993793487548828125f,-0.2986000303491953555834470535046420991420745849609375f,0.07179999775886525270607307902537286281585693359375f,-0.70200002125650673434620330226607620716094970703125f,0.2160000007748603678692234097979962825775146484375f,0.32290003656446941704416531138122081756591796875f,-0.326499998033046523460143362171947956085205078125f,-0.14110000695437197038017984596081078052520751953125f,0.457900014436245061943964174133725464344024658203125f,-0.549200009498000074614765253500081598758697509765625f,-0.07000000275671480753914011074812151491641998291015625f,0.66840001225322465128186877336702309548854827880859375f,-0.0455999894678591477514828511630184948444366455078125f,0.3271000356689099941576159835676662623882293701171875f,-0.4207000159502030101066338829696178436279296875f,-0.1463999997884035675355107741779647767543792724609375f,0.470400016820430977304567932151257991790771484375f,0.14579999776631591412723309986176900565624237060546875f,0.145800006222724931603806908242404460906982421875f,0.11050002191960850428387175270472653210163116455078125f,-0.71890002295970933943181080394424498081207275390625f,0.43150000760704276814294644282199442386627197265625f,0.00769999276399602194942417554557323455810546875f,0.49009999664276826525366459463839419186115264892578125f,0.29470001040697102467902368516661226749420166015625f,-0.035900002381205720070056486292742192745208740234375f,-0.98539997599124973248763126321136951446533203125f,-0.5342999717935921211164895794354379177093505859375f,-0.66389997397214184360336730605922639369964599609375f,-0.86009998706877233143330840903217904269695281982421875f,0.92649999661743631662602638243697583675384521484375f,0.181600000375509385008854223997332155704498291015625f,-1.371400029218197147429236792959272861480712890625f,-1.31970000534057607666227340814657509326934814453125f,0.54589997861385342048379243351519107818603515625f,0.2097000180065631269599180086515843868255615234375f,1.1149999791383748970474698580801486968994140625f,-0.9816000219821929828611928314785473048686981201171875f,0.8113000399887564384471261291764676570892333984375f,-0.110999995224177860908554293928318656980991363525390625f,-1.104200001712143386356501650880090892314910888671875f,0.8358999962717295062475386657752096652984619140625f,0.0702999996513131719666489516384899616241455078125f,0.12520001764893518014787332504056394100189208984375f,1.51509999325275401815815712325274944305419921875f,-0.55559999259710313168625361868180334568023681640625f,-0.17399998372793135104075190611183643341064453125f,-0.079500014066696156334046463598497211933135986328125f,0.262700021374225656245471327565610408782958984375f,0.62520000736713399192012730054557323455810546875f,0.45289998582601498355870717205107212066650390625f,0.20970001606941224281399627216160297393798828125f,0.69719997613429995908518321812152862548828125f,0.58640000045895579461330271442420780658721923828125f,-0.591099987769126755665638484060764312744140625f,0.581499984174966844108212171704508364200592041015625f,-0.31510001322031033765824759029783308506011962890625f,0.1247000130519271277051984725403599441051483154296875f,-0.06709999569654456763601046986877918243408203125f,-0.4281000154331326967849236098118126392364501953125f,0.33450000311434269217869541535037569701671600341796875f,0.826900015784799879980937475920654833316802978515625f,0.339500022411346691342259873636066913604736328125f,-0.1550000328198077870212046036613173782825469970703125f,0.7274000159859657088645690237171947956085205078125f,-0.247100007952749944895032285785418935120105743408203125f,0.513500017166137912028034406830556690692901611328125f,-0.6361999924212693446889943515998311340808868408203125f,-0.3718000013351439037023737910203635692596435546875f,0.327800007064640575027425484222476370632648468017578125f,-0.021799999248981460908680674037896096706390380859375f,0.8292000192582609230385060072876513004302978515625f,0.0060000109076501217941768118180334568023681640625f,0.6843000039800999356742750023840926587581634521484375f,0.4245000215172769486571269226260483264923095703125f,0.073400002157688160053794490522705018520355224609375f,0.185800003466010110741990501992404460906982421875f,0.1531999903947116603575295812333934009075164794921875f,0.1203999970763920401850555208511650562286376953125f,-0.459600011081993553574420730001293122768402099609375f,0.621600007379055075062979085487313568592071533203125f,0.0813000038906930289073216044926084578037261962890625f,-0.07409999366998665237815657746978104114532470703125f,-0.17379998768568061251471590367145836353302001953125f,-0.5930000054910777862460236065089702606201171875f,0.24350000438839192185014326241798698902130126953125f,-0.448800005567073778678377493633888661861419677734375f,-0.28530001317411668448897898997529409825801849365234375f,0.0740999981403351881681373924948275089263916015625f,0.264300009977817662587540326057933270931243896484375f,-0.60750002119690182933453570512938313186168670654296875f,0.76560001818984757537833729656995274126529693603515625f,0.47069999374747251152939497842453420162200927734375f,-0.119999997317790985107421875f,0.04829999631345283095384957050555385649204254150390625f,-0.024300003384053692201405283412896096706390380859375f,0.612000001251697511861493694595992565155029296875f,-0.8249999852478502315733521754737012088298797607421875f,-0.52619998221397423776579671539366245269775390625f,-0.880800023321807401543992455117404460906982421875f,0.3110999880671503348139594891108572483062744140625f,-0.899399995891749792775726746185682713985443115234375f,-0.3509999799132348385910518118180334568023681640625f,0.0159000036478043060839127065264619886875152587890625f,-0.03420000726282612646400593803264200687408447265625f,0.6132000262320043137975744684808887541294097900390625f,0.273000009998678905276392470113933086395263671875f,0.326399986414611209539771152776665985584259033203125f,0.79589999798536315012142949854023754596710205078125f,-0.66779999741613860209099584608338773250579833984375f,0.505200028824805968952205148525536060333251953125f,0.6294000287488106248900976424920372664928436279296875f,1.0208999824881548335042680264450609683990478515625f,-0.76469997956156710028352563313092105090618133544921875f,-0.26009997969269793571811533183790743350982666015625f,0.34110000609755519729304751308518461883068084716796875f,0.61200001183152163974909854005090892314910888671875f,-0.4826000101655718577120524059864692389965057373046875f,-0.49840003546178401894195530985598452389240264892578125f,0.147300049853324566129231243394315242767333984375f,-0.75089998621344467011340384487994015216827392578125f,-0.7766999997377392883635138787212781608104705810546875f,-0.939500045955181661838651052676141262054443359375f,-0.4880000041127203758151154033839702606201171875f,-0.98460004308223769697860916494391858577728271484375f,0.797400018072128347057514474727213382720947265625f,0.73720004140138684078920050524175167083740234375f,1.1293999823689464534481885493732988834381103515625f,0.8130000093281264383904272108338773250579833984375f,0.56029997975826262290865997783839702606201171875f,0.821300013017653984803700950578786432743072509765625f,-0.4808999680340289017976829200051724910736083984375f,0.2113000086963179313670480041764676570892333984375f,0.7223999536991119185813658987171947956085205078125f,0.21859999238848626390563367749564349651336669921875f,-0.5013999705076219726151975919492542743682861328125f,-1.04980002809762940074733705841936171054840087890625f,0.1878999829947947119990203646011650562286376953125f,0.4163000075042244674250468960963189601898193359375f,0.3572999851822853401017709984444081783294677734375f,0.73630001063346828260591792059130966663360595703125f,-0.194800041210651642131779226474463939666748046875f,0.74720001860260953918668747064657509326934814453125f,-1.0948999820113183290004599257372319698333740234375f,-0.92330002037882774601484925369732081890106201171875f,0.46149998551607129826379605219699442386627197265625f,0.689399986839294598439664696343243122100830078125f,-0.502800023679435437173879108740948140621185302734375f,-0.2673000017225739544102225409005768597126007080078125f,-0.830699983316659729126740785432048141956329345703125f,0.315900023317336575701119727455079555511474609375f,-0.9237000253155829287976530395098961889743804931640625f,-0.3545999929398295780202943205949850380420684814453125f,-0.76500001911073933769102950464002788066864013671875f,0.1575000116229059443639926030300557613372802734375f,0.36960001063495884210396980051882565021514892578125f,0.04769998739957781452858398552052676677703857421875f,0.5939999833926556949137420815532095730304718017578125f,0.493199980187416286980806034989655017852783203125f,0.51450000113993876738049948471598327159881591796875f,-0.1311000148892400574140992830507457256317138671875f,-0.06509999932497745334103456116281449794769287109375f,-0.830699987637996795086792189977131783962249755859375f,0.4499999948590993614772060027462430298328399658203125f,0.8456999810814860296659389859996736049652099609375f,-0.7182000016450882373675312919658608734607696533203125f,0.2958000401973723825221895822323858737945556640625f,0.080700005444884315153331044712103903293609619140625f,-0.358800014209747164528607754618860781192779541015625f,0.5427000001028179365647474696743302047252655029296875f,-0.158100009584426715036897803656756877899169921875f,0.997800019209087007965308657730929553508758544921875f,-0.1644000202178954594955939683131873607635498046875f,-0.6482999971702694264052979633561335504055023193359375f,-0.3581999804854394398034855839796364307403564453125f,-0.31350000319629900413787026991485618054866790771484375f,0.2794000108301641915886648348532617092132568359375f,-0.2864999896883961838511822861619293689727783203125f,-0.233300011587143085733941916259936988353729248046875f,0.077899973905086739023317932151257991790771484375f,0.82629999230503958784765927703119814395904541015625f,0.95479998399019194721404346637427806854248046875f,0.5724000438511378519024219713173806667327880859375f,-1.4194000044226644519085311912931501865386962890625f,-1.02880004937648816820683350670151412487030029296875f,-0.48440000293254836805090235429815948009490966796875f,-0.771400037264824423033360289991833269596099853515625f,-0.1172999827980998333032403024844825267791748046875f,-0.440299993616342089097770440275780856609344482421875f,0.2534999897480012531758575278217904269695281982421875f,-0.324300018620491403709138467092998325824737548828125f,0.339399999058246581995490487315692007541656494140625f,0.8022000227749348510997151606716215610504150390625f,0.71219999908208819761057384312152862548828125f,0.519600034290552681426333947456441819667816162109375f,-1.2069999794960022398981891456060111522674560546875f,-0.848000010073184373737831265316344797611236572265625f,-0.48140000671744331839363439939916133880615234375f,-0.7826000375092034833102161428541876375675201416015625f,1.153800013613700770065406686626374721527099609375f,0.48300003346800846060205003595910966396331787109375f,-0.8281000134587284122744677006267011165618896484375f,-0.723900040692091462091184439486823976039886474609375f,0.0522999929308891609025522484444081783294677734375f,0.045299993020295747925274554290808737277984619140625f,0.641299972486496283607948498683981597423553466796875f,-0.34709997900724420816231940989382565021514892578125f,-0.5469000141084194854812494668294675648212432861328125f,-0.53400000488758081473861238919198513031005859375f,0.993299987559020269856091545079834759235382080078125f,-0.0377999997258182673931514727883040904998779296875f,0.90300003261119154984726264956407248973846435546875f,1.043399988746642748083104379475116729736328125f,-0.7116000190392137714212594801210798323154449462890625f,-0.5696999877572057613406286691315472126007080078125f,-0.83700001924484979110019366999040357768535614013671875f,-0.23969999610185599436817938112653791904449462890625f,-0.5288999854087828023097017648979090154170989990234375f,-0.0488999922633173156327757169492542743682861328125f,-0.34290001473426823253021211712621152400970458984375f,0.3284999950379134947553438905742950737476348876953125f,0.800400008364021697815360312233678996562957763671875f,0.019199999886751140110163760255090892314910888671875f,0.67740002157390133419312405749224126338958740234375f,0.6440999969482419373889570124447345733642578125f,-0.5723999941930173473991771970759145915508270263671875f,-0.48749998927116422464678180404007434844970703125f,-0.837000018909573650915945108863525092601776123046875f,-0.220199998873471969318416086025536060333251953125f,0.12960001041144142419625495676882565021514892578125f,0.48239999184608439719568195869214832782745361328125f,-0.4521000248283149058892149696475826203823089599609375f,-0.189300002229213504278959590010344982147216796875f,-0.2013000056415796823472419418976642191410064697265625f,-0.248100005710124804636507178656756877899169921875f,0.464699984590709103127181833770009689033031463623046875f,-0.387200007724761707805782862124033272266387939453125f,-0.64439998967051526079785617184825241565704345703125f,-0.5636000119090078808170574120595119893550872802734375f,0.3978999949157238091856925166212022304534912109375f,0.6225000229477883895157219740212894976139068603515625f,0.5732999974496664208345464430749416351318359375f,0.53350001386925571811303825597860850393772125244140625f,1.582200015175342144857495441101491451263427734375f,1.0796000448703768626046439749188721179962158203125f,0.833299963605404148125899155274964869022369384765625f,0.6427000005125993453702903934754431247711181640625f,0.7641000257074832546777543029747903347015380859375f,0.01190001440644294916637591086328029632568359375f,-1.492399973666667989391498849727213382720947265625f,-0.7574000138998027154002556926570832729339599609375f,-0.467899995772540489014090780983678996562957763671875f,-0.2951000054344532497196951226214878261089324951171875f,-0.01789999506473538559703229111619293689727783203125f,-0.80990002194643029298504188773222267627716064453125f,-0.4476000072598456114292275742627680301666259765625f,0.336399992412328430191337247379124164581298828125f,-1.153899994647503035594127140939235687255859375f,-0.3153000177562237382744569913484156131744384765625f,-0.438599984192848069142200984060764312744140625f,-0.610000007748603412238708187942393124103546142578125f,-0.464200000631809217566114966757595539093017578125f,-1.17420004488825835409215869731269776821136474609375f,0.41720001487731905864819736962090246379375457763671875f,0.9994000435382133229467171986470930278301239013671875f,-0.42310000571012462700082323863171041011810302734375f,-0.147900005495548381162507212138734757900238037109375f,-0.44819999415725464420034995782771147787570953369140625f,-0.25619998564124113471507371286861598491668701171875f,0.5307000072330236850604023857158608734607696533203125f,0.035699988383054659379922668449580669403076171875f,0.254099996072798928192781886536977253854274749755859375f,0.0158999871261418235235396423377096652984619140625f,0.818400022758543510548179256147705018520355224609375f,0.764100012743472944976019789464771747589111328125f,0.70109998721033317536921458668075501918792724609375f,0.597899978524446762406796551658771932125091552734375f,-0.3212999981537461025027369032613933086395263671875f,0.318300005328655100100831987219862639904022216796875f,-0.604199995230138231505634394125081598758697509765625f,-0.920699982273578854119477909989655017852783203125f,0.00389999904334548119777537067420780658721923828125f,0.01680000424087035870712725227349437773227691650390625f,-0.84030000993311404489105598258902318775653839111328125f,0.24180000312328342459977648104541003704071044921875f,0.5921999948725102935753739075153134763240814208984375f,-0.2417999912023542918859675410203635692596435546875f,-0.017099996218085333765657196636311709880828857421875f,-0.60569997839927669502912976895458996295928955078125f,-0.3972000040367245077277402742765843868255615234375f,0.0597000047445295223269567941315472126007080078125f,-0.901200016374886114789433122496120631694793701171875f,0.3084000167608262898966131615452468395233154296875f,0.96660002529770139290121733210980892181396484375f,-0.01589999191314006576902784217963926494121551513671875f,-0.77310002567768110193213715319870971143245697021484375f,-0.2813000040024515868708476773463189601898193359375f,-0.348399995899200121129979379475116729736328125f,0.2937999937951563111937502981163561344146728515625f,-0.8256000105343759276621540266205556690692901611328125f,0.049599996534734902553509527933783829212188720703125f,0.280599966520071408826453307483461685478687286376953125f,0.59739999136179611216590501499013043940067291259765625f,1.8723000382304189059823329444043338298797607421875f,1.1193000731587421370250012842006981372833251953125f,0.3961000062465667159727900070720352232456207275390625f,0.05530000583529481872346877935342490673065185546875f,0.56619997759461426767302327789366245269775390625f,0.7346000065743918394645106673124246299266815185546875f,0.06709998526573190957833503489382565021514892578125f,0.366700001227855398866495306720025837421417236328125f,-0.4480000105202197602238811668939888477325439453125f,-0.0333999963462350279996826429851353168487548828125f,-0.061699966657161819938437474775128066539764404296875f,-0.88410002213120453262718001496978104114532470703125f,0.994099982941150983606348745524883270263671875f,0.55750000774860364316509730997495353221893310546875f,0.0765000028014182209545879231882281601428985595703125f,-0.3713000018417833558714846731163561344146728515625f,-0.5361999935388563898186475853435695171356201171875f,-0.3322000041484829413462875891127623617649078369140625f,0.716800007221102486454356039757840335369110107421875f,0.69320003792941620179846040628035552799701690673828125f,0.016599995458125960112738539464771747589111328125f,0.221200014549494117233052747906185686588287353515625f,0.3090000281333920018056460321531631052494049072265625f,0.7044000224530695408731162387994118034839630126953125f,0.4523999878227709103839515591971576213836669921875f,-0.245699993002414629472696105949580669403076171875f,0.112200004204362657755922327851294539868831634521484375f,-0.703800005380809334809555366518907248973846435546875f,0.5783999810725448664783243657439015805721282958984375f,0.019799979538470846573972039550426416099071502685546875f,0.7701000288292767503861568911815993487834930419921875f,0.9159000006675714899984086514450609683990478515625f,0.0453000021100044281130436729654320515692234039306640625f,0.3318000046879052700177226142841391265392303466796875f,0.527399990281462560659520022454671561717987060546875f,0.0185999801695349020747016766108572483062744140625f,0.29069999628067011432364097345271147787570953369140625f,-0.157200010406970847043339745141565799713134765625f,0.271800004725158306229104709927923977375030517578125f,-0.052500008940696840653572508017532527446746826171875f,-0.9075000063702465258330676078912802040576934814453125f,0.236400031155347800648769407416693866252899169921875f,0.81269998780935992499507847242057323455810546875f,0.999299989414215161787069519050419330596923828125f,-0.3728999976351856648904004032374359667301177978515625f,0.2024999985843896066484148832387290894985198974609375f,-0.00780000099241728861443334608338773250579833984375f,-0.003599996858835385893371494603343307971954345703125f,0.606600020156800923132323077879846096038818359375f,0.308399997165798911424872130737639963626861572265625f,0.516600011551380244867459623492322862148284912109375f,0.270299999427795256679019075818359851837158203125f,0.84300002847611910095082521365839056670665740966796875f,0.01879999975562096636849673814140260219573974609375f,-0.874499987132847156345860639703460037708282470703125f,-0.15990000425130102001958221080712974071502685546875f,0.18580000436007981789998666499741375446319580078125f,0.3152000142961741691038923818268813192844390869140625f,-1.2053999435067179746283727581612765789031982421875f,-0.5891999994397156825698402826674282550811767578125f,-0.1410000052452087260235202847979962825775146484375f,0.138000002473592786600420367904007434844970703125f,-1.25009998759031315529455241630785167217254638671875f,-0.964700015473365368734448566101491451263427734375f,0.212800017309188405079112271778285503387451171875f,-0.47819998152851983519440182135440409183502197265625f,0.13229999978542306138251660740934312343597412109375f,0.83990004131794027131263646879233419895172119140625f,-0.13180005259513816184835377498529851436614990234375f,0.75579999072551640182382470811717212200164794921875f,1.041300005269050643619266338646411895751953125f,0.22210001074671748000355364638380706310272216796875f,-0.38040000162124609772718031308613717555999755859375f,-0.286600021088123657619917139527387917041778564453125f,1.2200999703049664191212286823429167270660400390625f,0.5840999967992299701791125698946416378021240234375f,1.015699992406367879738127157907001674175262451171875f,0.6329000414818530817484543149475939571857452392578125f,-1.3675999753713607987037903512828052043914794921875f,-0.716600024515390288115668226964771747589111328125f,1.0052999931693076263172770268283784389495849609375f,0.5195000185370446388333220966160297393798828125f,-0.186899984417855602458757857675664126873016357421875f,-0.90299998354911803488676014239899814128875732421875f,0.0608999985069036409157661182689480483531951904296875f,-0.49349999220669282795626031656865961849689483642578125f,0.04140000239610674981349802692420780658721923828125f,-0.2541000097632408749603882824885658919811248779296875f,-0.8603999848946928263870859154849313199520111083984375f,-1.1996999657034876918260124512016773223876953125f,0.2814000019490718873527157484204508364200592041015625f,-0.0165000010132789753924953402020037174224853515625f,-0.76709999118894323100192877973313443362712860107421875f,-0.57329999042749424376097522326745092868804931640625f,-0.579599976995587073957949542091228067874908447265625f,0.252900001323222767268816824071109294891357421875f,0.67470002448856847276914550093351863324642181396484375f,-0.36960001327991509612047593691386282444000244140625f,0.82859998352229580209638015730888582766056060791015625f,-0.31140003219842871118316907086409628391265869140625f,0.098699996705353232329116508481092751026153564453125f,0.73439999905824659975905888131819665431976318359375f,-0.32940001578480015975713968146010302007198333740234375f,-0.29370000039339050346143267233856022357940673828125f,0.7298999767586586973067142025684006512165069580078125f,1.07279998244047192201833240687847137451171875f,0.635700022655725671683057953487150371074676513671875f,0.70559998428970549522176725076860748231410980224609375f,-0.5460000024139881613649549763067625463008880615234375f,-0.801299973380565688785281963646411895751953125f,0.7911000062093138662788760484545491635799407958984375f,1.0419000029325484746323127183131873607635498046875f,-0.029700003813207143110730612534098327159881591796875f,0.92580001993179283914514599018730223178863525390625f,-0.623499990344047461121590458787977695465087890625f,-0.284300026369095348854898475110530853271484375f,0.45720000355243683998196502216160297393798828125f,-0.1894000147044661019890554598532617092132568359375f,-0.45529997575357583627919666469097137451171875f,-0.50549999634548969940084361951448954641819000244140625f,0.4525000117719173431396484375f,-0.17170000838041321600258015678264200687408447265625f,0.234500003337860363217259873636066913604736328125f,0.86510001426339133701048922375775873661041259765625f,0.56739997992515578406624854324036277830600738525390625f,0.1813999993413684119758499946328811347484588623046875f,0.912500000000000088817841970012523233890533447265625f,0.21190000203847869642004297929815948009490966796875f,-0.9290000025033950947772609652020037174224853515625f,-0.3520000088512897917780719581060111522674560546875f,0.90059998395442963925461299368180334568023681640625f,0.9598000254154204213818957214243710041046142578125f,-0.225199997979402599668219409068115055561065673828125f,0.30620000694990157086294857435859739780426025390625f,0.755899993924796564925827624392695724964141845703125f,0.852300028581917334946638220571912825107574462890625f,0.3264000003099436497677743318490684032440185546875f,1.0228000503897671791264656349085271358489990234375f,0.614199957716465405610506422817707061767578125f,0.588999989092349185426655822084285318851470947265625f,-0.13940000025033949526687138131819665431976318359375f,-0.3278000024080274243942767498083412647247314453125f,-0.199399999952316164808507892303168773651123046875f,0.39299997520446794396775658242404460906982421875f,-0.503100013160705739068134789704345166683197021484375f,-0.7169999851584432093432042165659368038177490234375f,0.08879999088942991392059411737136542797088623046875f,0.803100020015239834947351482696831226348876953125f,-0.484499985996633630858099195393151603639125823974609375f,-0.2108999853380026934246416203677654266357421875f,-0.2655000072419644308041597469127736985683441162109375f,0.370500013053417287522961487411521375179290771484375f,0.693299994152784204715089799719862639904022216796875f,-0.2925000023841857199613514239899814128875732421875f,0.202199992078542667428564527654089033603668212890625f,0.47819998469501745486098798210150562226772308349609375f,0.254699992413818743219877660521888174116611480712890625f,0.808500004202127531272026317310519516468048095703125f,-0.6773999987751244677980366759584285318851470947265625f,-1.0935000132918357707012546597979962825775146484375f,0.86460000493526456555315462537691928446292877197265625f,0.37079998625516896737508432124741375446319580078125f,0.200400005346536624895037448368384502828121185302734375f,-0.4370999985277652655213387333787977695465087890625f,0.746700008194148612261642483645118772983551025390625f,0.265199989746510933485978966928087174892425537109375f,0.6336000360488893701216284171096049249172210693359375f,-0.51180001087188742303624167107045650482177734375f,0.74099997805058936961586368852294981479644775390625f,0.52259997805357016886773635633289813995361328125f,-0.14339999917745582802552917200955562293529510498046875f,0.1997999976992606008252550964243710041046142578125f,-0.45300001215934759812853371840901672840118408203125f,0.35610000609755498857111888355575501918792724609375f,1.1556000063061713234446870046667754650115966796875f,0.71560003148913420290000431123189628124237060546875f,0.7781000042758881019011596436030231416225433349609375f,0.082900001230090669679384518531151115894317626953125f,-0.768599962809682235587160903378389775753021240234375f,-0.89779999994933523677076436797506175935268402099609375f,1.2203999855279921149531219271011650562286376953125f,0.37020001496672616525529520004056394100189208984375f,1.359699973452091370518246549181640148162841796875f,0.8539000077605241045830553048290312290191650390625f,-0.2755000205039976890475372783839702606201171875f,0.67930002249479315423741354607045650482177734375f,0.0385999911963939013048729975707828998565673828125f,-0.552400006151199107762295170687139034271240234375f,0.109700021880865250523129361681640148162841796875f,0.0897000148773197114593358492129482328891754150390625f,0.389499987766146749113005398612585850059986114501953125f,0.317300004442035987750614367541857063770294189453125f,0.732399975752830556530170724727213382720947265625f,0.0748000014245508992871691589243710041046142578125f,1.18780001969337423162187405978329479694366455078125f,0.81440005225539291444647460593841969966888427734375f,0.498900002396106767577066420926712453365325927734375f,0.05269999827742566367305698804557323455810546875f,-0.43250000506639452169110882095992565155029296875f,-0.0049000045120717317104208632372319698333740234375f,0.572000014550984037242642443743534386157989501953125f,0.27600001958757669484612051746807992458343505859375f,-1.6492000051021573625575911137275397777557373046875f,0.6897000071287155975596760981716215610504150390625f,0.8946000108584761800756268712575547397136688232421875f,1.0038000058650968782103518606163561344146728515625f,-0.11070000158920889621327887653023935854434967041015625f,0.46380000642389074272387006203643977642059326171875f,-0.7757999827533959003034169654711149632930755615234375f,-0.241799976226687807212556435842998325824737548828125f,0.1680000038892031088977319086552597582340240478515625f,0.7556999870419502940421807579696178436279296875f,0.692699989150464379150662352913059294223785400390625f,0.7550999855041506947372909053228795528411865234375f,0.8169000087440014823414458078332245349884033203125f,-0.3420000239014626686184783466160297393798828125f,-0.211799997907876846880981247522868216037750244140625f,0.621299995732307319684650792623870074748992919921875f,-0.252900000540912106572477568988688290119171142578125f,-0.32040000624060649414559520664624869823455810546875f,0.316499996691942175885969845694489777088165283203125f,0.241499990973621680989680271522956900298595428466796875f,0.3461999967053532856198216904886066913604736328125f,0.95219998686313633839972681016661226749420166015625f,0.86580003013908879427873444001306779682636260986328125f,0.85290000936984977641941441106610000133514404296875f,0.107699994626641204487071945550269447267055511474609375f,0.500700006264448216342088926467113196849822998046875f,-0.2679000099286437119872061884962022304534912109375f,-0.6930000190138816407170452293939888477325439453125f,-0.0270000001341104134411352788447402417659759521484375f,0.08099999101459964823135351252858527004718780517578125f,-0.8160000214129687901021270590717904269695281982421875f,0.819300050061941842471924246638081967830657958984375f,-0.327100002774596276822194340638816356658935546875f,-0.38350001157820223784966628954862244427204132080078125f,0.949399981027841732839078758843243122100830078125f,0.37880001599788659660816847463138401508331298828125f,0.58379997934103045054143876768648624420166015625f,0.489199996310472062788221592199988663196563720703125f,-1.0893000172793865232279131305404007434844970703125f,-0.24970001413226139419521132367663085460662841796875f,-0.315799977910518947510354337282478809356689453125f,-0.664799999338387959113561009871773421764373779296875f,0.7940000129342077439531522031757049262523651123046875f,0.188600018614530906546633559628389775753021240234375f,-0.51949999022483783761572340154089033603668212890625f,-1.00990005055666021149818334379233419895172119140625f,-0.0836999774456026468527625183924101293087005615234375f,-0.865300004643201337017899277270771563053131103515625f,0.1124000192642211626292692017159424722194671630859375f,-0.790200018095970069254008194548077881336212158203125f,0.327999995954334711001365576521493494510650634765625f,-0.283600007997453218422378995455801486968994140625f,-0.310599988648295521898035076446831226348876953125f,-0.39460001111924658534491072714445181190967559814453125f,0.12270002733469009825739703956060111522674560546875f,-0.41330000920295706379192779422737658023834228515625f,0.96079999161958695719221168474177829921245574951171875f,0.3584000119179486798515199552639387547969818115234375f,1.6656000268697734867373583256267011165618896484375f,0.87620004584193278418524641892872750759124755859375f,0.0870999861598011904106897418387234210968017578125f,0.2502000020027157489721503225155174732208251953125f,-0.1502999998748302257212117183371447026729583740234375f,0.1032000058919191332051923382095992565155029296875f,0.0941999960124493274093993022688664495944976806640625f,0.410699968266487136503428700962103903293609619140625f,0.7373999842464922238605140591971576213836669921875f,0.63989999303817768350199912674725055694580078125f,-0.038099998632073361903849217924289405345916748046875f,-0.668400006628036447153817789512686431407928466796875f,-0.764999987259507019388138360227458178997039794921875f,-0.162899989700317604501833557151257991790771484375f,0.2166000096514821560145236389871570281684398651123046875f,0.6915000086873768214701385659282095730304718017578125f,-0.81390003346502803882600574070238508284091949462890625f,0.11100001776218437754550905083306133747100830078125f,-0.85169999265968787671710060749319382011890411376953125f,0.2514000117093322383965414701378904283046722412109375f,-0.72840000722408293487575292601832188665866851806640625f,0.4587000173062085739417170771048404276371002197265625f,0.022199996995925896925427878159098327159881591796875f,0.75000000663101673570309912975062616169452667236328125f,-0.6888000003889203082252379317651502788066864013671875f,-0.4757999933332204278713106759823858737945556640625f,-0.50610000587403780603068526033894158899784088134765625f,0.1566000201940536129541214904747903347015380859375f,0.372599999548494820178490272155613638460636138916015625f,0.76739999374598266701497095709783025085926055908203125f,0.6108000216454267938814837179961614310741424560546875f,0.909300015938281802618803340010344982147216796875f,0.8109000303581359592186572626815177500247955322265625f,0.230100013816356696594311870285309851169586181640625f,1.251300006163120315250125713646411895751953125f,0.850300040405989054903557189390994608402252197265625f,0.4768999942898748400921249412931501865386962890625f,0.597700023609399888613324947073124349117279052734375f,-0.5387999912619587661311015835963189601898193359375f,0.30599999779462816462682894780300557613372802734375f,-0.1696999966979024776492224191315472126007080078125f,-0.758300018143653886681931908242404460906982421875f,-0.8708999765276910220990203015389852225780487060546875f,-0.133299999368190480453222335199825465679168701171875f,0.4569000208139417651409530662931501865386962890625f,-0.525100016796589130763095454312860965728759765625f,0.228900005078315871287486515939235687255859375f,-0.003499997198581628055080727790482342243194580078125f,0.7960000064373016215313327847979962825775146484375f,0.610400030827522588339206777163781225681304931640625f,0.109800012209266384388683945871889591217041015625f,0.0632000096358360796688913296748069114983081817626953125f,-0.4457999546647073429994634352624416351318359375f,-1.05820000864863317246999940834939479827880859375f,1.1310999691240493092436736333183944225311279296875f,1.095700009114294548595580636174418032169342041015625f,0.718099996601790024897127295844256877899169921875f,0.75570003057196755147373323779902420938014984130859375f,-0.3612999683886766177920435438863933086395263671875f,-0.56610000065863086415873794976505450904369354248046875f,0.8231999859988690104728448204696178436279296875f,0.9374000232875345783867260252009145915508270263671875f,0.908700001585483363442108384333550930023193359375f,0.5900999781280755751566857725265435874462127685546875f,0.48090000789463527297584732878021895885467529296875f,0.420899982488155188775635906495153903961181640625f,0.4143000134423375779846310251741670072078704833984375f,0.01319999210834499336897351895458996295928955078125f,0.5723999913990496235527416502009145915508270263671875f,-0.30239999005794526709678393672220408916473388671875f,-0.9519000135496258341305519934394396841526031494140625f,-0.101100011610984807930435636080801486968994140625f,-0.1658999910190701310153116310175391845405101776123046875f,-0.7871999823182822542122494269278831779956817626953125f,-0.7803000052392483620877783323521725833415985107421875f,0.31920002328157437432309961877763271331787109375f,-0.125100005024671556697057894780300557613372802734375f,0.07319999449253078438459851895458996295928955078125f,0.3096000085115433453353261938900686800479888916015625f,0.16799998801946625093250986537896096706390380859375f,-0.155399995902925691648732708927127532660961151123046875f,-0.17010000020042070900672115385532379150390625f,-0.7355999942734836594127045827917754650115966796875f,0.339600034141540163545869290828704833984375f,0.7238999830059704709839252245728857815265655517578125f,0.219299972557276845464002690277993679046630859375f,0.67950001460686326737459239666350185871124267578125f,0.298799994074553154632667428813874721527099609375f,-0.816299999718367974566035627503879368305206298828125f,-0.410399993947148544748415588401257991790771484375f,0.810600002320110757381144139799289405345916748046875f,0.272099986308813157620534184388816356658935546875f,0.603300007228553258897818523109890520572662353515625f,-0.96500004053115873148271930404007434844970703125f,0.75669997768402108562213470577262341976165771484375f,0.435300011050701041170896132825873792171478271484375f,0.4602999879539009242535030352883040904998779296875f,0.3865000202357771197370084337308071553707122802734375f,-1.565499978184700324845834984444081783294677734375f,-1.07650001069903300532359935459680855274200439453125f,-0.3557000101573766226437101067858748137950897216796875f,-0.289300014690309914566768156873877160251140594482421875f,-0.745899992918968024468995281495153903961181640625f,-0.15050002363324210818973369896411895751953125f,0.475700012075900957597696105949580669403076171875f,-0.039299999696016385541952331550419330596923828125f,0.67630000035166748517667656415142118930816650390625f,0.5149000185191632272818651472334749996662139892578125f,-1.13350000792741756328041446977294981479644775390625f,-0.68190003109574348627575091086328029632568359375f,1.35590001586675601430442839046008884906768798828125f,0.631500045269728360608496586792171001434326171875f,-0.117499973773956156719577847979962825775146484375f,-0.503300004136562062484472335199825465679168701171875f,-0.4514999774694441470046513131819665431976318359375f,0.2599000107705595752349836402572691440582275390625f,-0.04149997165799135245833895169198513031005859375f,0.4665000185966494683498240192420780658721923828125f,-0.6559999835826459957388578914105892181396484375f,-0.7592000179730356013152459127013571560382843017578125f,0.36779998825192450340182404033839702606201171875f,0.0650000068545342202241954510100185871124267578125f,-0.07799999926984302334176391013897955417633056640625f,-0.2414999914765356692214481881819665431976318359375f,0.56729999855607748937558199031627736985683441162109375f,0.68609998359680179902397867408581078052520751953125f,0.6489000201657415534128858780604787170886993408203125f,0.57990000212788572753197513520717620849609375f,-0.734699982690810937668857150129042565822601318359375f,-0.6584999825954440666464506648480892181396484375f,0.02190000008270143183608524850569665431976318359375f,0.131400007611513214289544748680782504379749298095703125f,-0.288300014045834718245941985514946281909942626953125f,-0.78569998853206612920985207892954349517822265625f,-0.29430000148415569327653429354541003704071044921875f,0.16110000743865970918022867408581078052520751953125f,0.40290000098794698235593614299432374536991119384765625f,0.3068999923527240270004767808131873607635498046875f,-0.69450001343339684556354995947913266718387603759765625f,-0.80700000631809221118828645558096468448638916015625f,0.51810002645999209125449169732746668159961700439453125f,0.88620000545978516726108864531852304935455322265625f,-0.773700006763637038176284477231092751026153564453125f,-0.232499997913837574969875277020037174224853515625f,0.0477000013321637883478842923068441450595855712890625f,-0.7484999957084657040695674368180334568023681640625f,0.2043000042781233371869120674091391265392303466796875f,-0.503999989986419816290208473219536244869232177734375f,-0.9125999932341277709468840839690528810024261474609375f,-0.5009999809004366255749118863604962825775146484375f,0.1344000023737550453262201699544675648212432861328125f,0.392399988567829094421313129714690148830413818359375f,0.01439999846220019197318151782383210957050323486328125f,0.68360002986490753240644835386774502694606781005859375f,-0.11440001306533797986730860429815948009490966796875f,-0.61500002145767229677630894002504646778106689453125f,0.075000014007091664325344027020037174224853515625f,-0.18580000622272496713094369624741375446319580078125f,-0.069400047338008530317665645270608365535736083984375f,0.5769999888837329393709296709857881069183349609375f,-0.49569996364712753944559153751470148563385009765625f,-0.4782999855101099928589292176184244453907012939453125f,-0.6762000082723795824080070815398357808589935302734375f,-0.106000001501291851724317893967963755130767822265625f,0.268200006376206889768809560337103903293609619140625f,0.0606000047788024520656335880630649626255035400390625f,1.0223999968193471232780211721546947956085205078125f,1.0320000357665122248107536506722681224346160888671875f,-0.2491000249624253370939186424948275089263916015625f,0.14909999516010241649155432241968810558319091796875f,-0.3487999933481216174868677626363933086395263671875f,0.45520001988410996318634715862572193145751953125f,0.60419998854324230563861419796012341976165771484375f,0.63700001557543861618881919639534316956996917724609375f,-0.3645999836266042848365032114088535308837890625f,0.04399999818205824198003028868697583675384521484375f,0.72709999092817323429471798590384423732757568359375f,0.377500008940696574200046597979962825775146484375f,1.0060999883651735586909126141108572483062744140625f,0.847300014239549259542627623886801302433013916015625f,-0.63449996727705038068734211265109479427337646484375f,-0.564899995124339415752956483629532158374786376953125f,-0.2151999753296376383104870910756289958953857421875f,0.4112999927520750720333353456226177513599395751953125f,-0.45900001244992016946611101957387290894985198974609375f,0.294299991202354505048788269050419330596923828125f,-0.43260000927150255023434510803781449794769287109375f,-0.14399999773502347721887417719699442386627197265625f,0.62279998366981725865798580343835055828094482421875f,-0.220200029271840680422656078008003532886505126953125f,-0.49109997715204933843580192842637188732624053955078125f,-0.29609998076558152746429186663590371608734130859375f,0.055800002590566943894145879312418401241302490234375f,-0.398100008802115989947623120315256528556346893310546875f,-0.05700000046938658027073643097537569701671600341796875f,0.09060000012218949461129113842616789042949676513671875f,0.707400009001046381484911762527190148830413818359375f,0.196199985697120382610592059791088104248046875f,-0.0704999977424740631448685235227458178997039794921875f,-0.53130001599788645449962132261134684085845947265625f,0.7584000108003616791307877065264619886875152587890625f,-0.140399996256828529794802307151257991790771484375f,0.57959999996200195493400997293065302073955535888671875f,0.25859998849555854150139566627331078052520751953125f,-0.2135999939158559346452648242120631039142608642578125f,-0.642299983096122684145257153431884944438934326171875f,0.20789999350011345580213628636556677520275115966796875f,0.3377999851226807237480898038484156131744384765625f,0.5330999911442397820593441792880184948444366455078125f,0.2597999848842622583333650254644453525543212890625f,-0.59969998179674133265137925263843499124050140380859375f,-0.41699997919797926471119353664107620716094970703125f,-0.4799999928474425470170672269887290894985198974609375f,0.967700041639805164095378131605684757232666015625f,-1.36779999942779539878756622783839702606201171875f,-0.54260001813769331846515342476777732372283935546875f,-1.2883999407649042723278398625552654266357421875f,-0.69180000133514329974104839493520557880401611328125f,1.06800000960752328182934434153139591217041015625f,1.051400053674728329866638887324370443820953369140625f,0.124299989414215161787069519050419330596923828125f,-0.63030002073645619020680896937847137451171875f,0.287999972708523177544748250511474907398223876953125f,-0.20600000640004889618239758419804275035858154296875f,-1.6391000035047529337361993384547531604766845703125f,-0.82270005922317590574266432668082416057586669921875f,-1.4178000098586078792095577227883040904998779296875f,-1.01380006412863821196879143826663494110107421875f,0.625899987852573502777886460535228252410888671875f,0.42030000464320149689001482329331338405609130859375f,1.3154999799728397391618273104541003704071044921875f,0.7663000055670732280077572795562446117401123046875f,-0.3870999754309654150574715458787977695465087890625f,0.001900001740455792287320946343243122100830078125f,-1.1360999795734887385378897306509315967559814453125f,-0.9515000111460680809472023611306212842464447021484375f,-1.1800999562978748969044318073429167270660400390625f,-0.50029999570250449636432676925323903560638427734375f,0.19699999953806379693332928582094609737396240234375f,0.15780001395642784256523327712784521281719207763671875f,0.7336999801278114574643041123636066913604736328125f,1.0653000255048272748581439373083412647247314453125f,0.721100008369982337086412371718324720859527587890625f,1.00979999918937668468288393341936171054840087890625f,-0.315599999414384324580851171049289405345916748046875f,-0.80039999566078190440521211712621152400970458984375f,-0.84659998618215315246970931184478104114532470703125f,-1.1066999630808833643413890968076884746551513671875f,0.744000029828399522813242583652026951313018798828125f,0.250499995600431901721094618551433086395263671875f,-0.724500011757016171287659744848497211933135986328125f,0.2348999900579453736781943007372319698333740234375f,0.0246000009119511364730215063900686800479888916015625f,0.59999998755753047863237270576064474880695343017578125f,-0.6159000235408547130333545283065177500247955322265625f,-0.970799976122378893705899827182292938232421875f,-0.8304000327274205783822935700300149619579315185546875f,-0.7253999885082240695055588730610907077789306640625f,0.186899993656575613698578308685682713985443115234375f,0.157199987757206116611996549181640148162841796875f,0.5078999803125856526975212545949034392833709716796875f,0.6359999799132349806995989638380706310272216796875f,-0.1556999988511205135210957450908608734607696533203125f,-0.534299985837936475263632019050419330596923828125f,-0.7280999798566101244290393879055045545101165771484375f,-0.44429997466206572198643698357045650482177734375f,-0.7376999776393173835487004907918162643909454345703125f,-1.17029997141361263857106678187847137451171875f,0.29220001286566266163191585292224772274494171142578125f,0.27600000233948218664181695203296840190887451171875f,0.8175000048056244583705165496212430298328399658203125f,0.0065999805569649794279030174948275089263916015625f,-0.1604999999776481889313117790152318775653839111328125f,0.95730003584623357681948618846945464611053466796875f,-0.63579999072551718342083404422737658023834228515625f,-0.556800020706653686630716038052923977375030517578125f,1.0898000114083288281108252704143524169921875f,0.9326000506222253960686430218629539012908935546875f,0.8170999939084051533200181438587605953216552734375f,0.12890001372098947030053750495426356792449951171875f,-0.89699997234344497343272450962103903293609619140625f,-1.061000019997357934897763698245398700237274169921875f,0.1008000118106604503509515780024230480194091796875f,-0.03599999593198288661000106003484688699245452880859375f,-1.143799985897540949508766061626374721527099609375f,-0.53600001344084713394977370626293122768402099609375f,-0.2331000030279160029067497816868126392364501953125f,-0.65090001631379124091836274601519107818603515625f,-0.849099972063303454206106835044920444488525390625f,-0.6539000007569784056471462463377974927425384521484375f,-0.216400019346177696633048981311731040477752685546875f,0.263400006963312449670411297120153903961181640625f,-0.38999995652586250116655719466507434844970703125f,-0.5781999878987662544460590652306564152240753173828125f,-0.049600001060962739529713871888816356658935546875f,0.330600018227100722612021854729391634464263916015625f,-0.3540000086687502545146344345994293689727783203125f,-0.5428000396050520759416713190148584544658660888671875f,-1.0335999752521518590953064631321467459201812744140625f,-0.787199997442959986670985017553903162479400634765625f,0.70029999525547026451022247783839702606201171875f,0.0066999992907046390655523282475769519805908203125f,0.0481999742269518804960171109996736049652099609375f,0.76740000585317602599388919770717620849609375f,-0.56760000871270899214238170316093601286411285400390625f,-0.39389999226331706694281820091418921947479248046875f,0.55200002103298917432994130649603903293609619140625f,0.2303999873161313161062935250811278820037841796875f,0.4197000082835555812010852605453692376613616943359375f,1.0323000009775160634717394714243710041046142578125f,-0.87239999754577868262828133083530701696872711181640625f,-0.22499997615814226747943394002504646778106689453125f,-0.312000002816319454979065994848497211933135986328125f,-0.20760000405609613238766542053781449794769287109375f,-0.3438000018417834979800318251363933086395263671875f,-0.63689999130964292106682478333823382854461669921875f,-0.4215000025033950326047715861932374536991119384765625f,0.281700003314018232458693091757595539093017578125f,-0.50549997907876953462391611537896096706390380859375f,-0.3737999729335310661326730041764676570892333984375f,-0.04619999234676352983797187334857881069183349609375f,-0.64470002076327792561727392239845357835292816162109375f,-0.89819998354017716035713192468392662703990936279296875f,-0.511499985851347815923872985877096652984619140625f,0.489000011332333173186270869337022304534912109375f,0.01469999639987928929940608213655650615692138671875f,-0.850200026310235390525349430390633642673492431640625f,-0.4770000032447274662672498379833996295928955078125f,-0.74639997915029498987138367738225497305393218994140625f,-0.6167999885946515714607585323392413556575775146484375f,0.148499996714293924782168687670491635799407958984375f,0.80040000311136250132904024212621152400970458984375f,0.8585999922394751404652879500645212829113006591796875f,0.692000025093555404254175300593487918376922607421875f,-0.279599978746473798452143455506302416324615478515625f,0.33300000288337461284982055076397955417633056640625f,-0.9814999729990960819492329392232932150363922119140625f,-0.91590001631379092117413165397010743618011474609375f,0.50289999297857246318699253606610000133514404296875f,1.09630005921125484036338093574158847332000732421875f,-1.3649000001907349854946005507372319698333740234375f,-0.518900008356570907608329434879124164581298828125f,-0.112499971091747141826999722979962825775146484375f,-0.76690001679062813622067551477812230587005615234375f,-1.051800012958049368450019755982793867588043212890625f,-0.8738000542938717529750647372566163539886474609375f,0.13119998668432242272530174886924214661121368408203125f,0.9564000196069477599536412526504136621952056884765625f,0.78670000681579121959430267452262341976165771484375f,-0.06329999288618548547447062446735799312591552734375f,-0.3035999767422676853811935870908200740814208984375f,-1.04840003300309181355487453402020037174224853515625f,-0.4109999864697453375583791057579219341278076171875f,-0.076200008738041002942509294371120631694793701171875f,-0.10970001010894758763924983213655650615692138671875f,-0.289700020837784233407319334219209849834442138671875f,0.368099995577335192820100928656756877899169921875f,-0.0313000054180623266120164771564304828643798828125f,0.045800031256675310942227952182292938232421875f,-0.58999998673796572035143981338478624820709228515625f,0.7596000076174735937684090458787977695465087890625f,0.321200019913912004909661845886148512363433837890625f,0.80549997177720111807275316095910966396331787109375f,0.22319999479055407931582521996460855007171630859375f,0.0882000005275009879568415271933190524578094482421875f,-0.65429998293220992255925239078351296484470367431640625f,-0.86669999556541432728096197024569846689701080322265625f,-0.5129999797493220992095075416727922856807708740234375f,0.87750003632158068977986431491444818675518035888671875f,-0.18330002321004901233436612528748810291290283203125f,-0.3596999902904032264672196106403134763240814208984375f,-0.8832000013470651111902043339796364307403564453125f,-0.497100010634958788813264618511311709880828857421875f,0.480000022947788096416843472979962825775146484375f,-0.880800032038986824289850119384936988353729248046875f,-0.656400001651048281559042152366600930690765380859375f,0.89640000463128088892261757791857235133647918701171875f,-0.29340001001805060543148329088580794632434844970703125f,-0.1713000043004750949648951063863933086395263671875f,0.6215999947130679270657083179685287177562713623046875f,-0.6678000091508031754727880979771725833415985107421875f,0.53220003633499146644680877216160297393798828125f,-0.2682000130444766039516935052233748137950897216796875f,-0.566999999463558168599774944595992565155029296875f,-0.33090001437664040562225409303209744393825531005859375f,-0.04260000427961330160542274825274944305419921875f,0.320100002118945070606059744022786617279052734375f,0.74730001647472388270898591144941747188568115234375f,-0.5549999794363973393274136469699442386627197265625f,0.29490001717805824910101364366710186004638671875f,-0.0089999963566660401426133830682374536991119384765625f,0.20579999102354040729778716922737658023834228515625f,0.83249998468905672677209395260433666408061981201171875f,0.243400006777048094619431140017695724964141845703125f,-0.4948000033617017123788173194043338298797607421875f,-0.8226000401914124182667364948429167270660400390625f,-0.552000000245869149040345291723497211933135986328125f,0.1156000006064772378522320650517940521240234375f,0.75620002363920235666228109039366245269775390625f,-0.17019999529719331832211537403054535388946533203125f,-0.7423000224351881115580908954143524169921875f,-0.3559000268936163280386608676053583621978759765625f,-0.35139998168349251272957189939916133880615234375f,0.16720001249313387603478986420668661594390869140625f,-0.3128000149250027561720344237983226776123046875f,0.41159999247789347265324977342970669269561767578125f,0.64070000835061069466291883145458996295928955078125f,0.281100015336275266264465244603343307971954345703125f,1.5697999982953074749048028024844825267791748046875f,0.675600008094310311435037874616682529449462890625f,0.52680001146793387079014792107045650482177734375f,-0.19900000116229055180383511469699442386627197265625f,-0.4767999942570924076790106482803821563720703125f,-0.61240002246797098361952293998911045491695404052734375f,0.52279998385608195121676544658839702606201171875f,0.46200000281631903309431663728901185095310211181640625f,-1.342299985182285215756792240426875650882720947265625f,-0.67810002657175072471318344469182193279266357421875f,-0.43600001001358013041908634477294981479644775390625f,-0.688400030022859876766005982062779366970062255859375f,0.511500014387070933707946096546947956085205078125f,0.6273000469103464649123225171933881938457489013671875f,-0.5958000154614444543454965241835452616214752197265625f,0.11860000326633457490288492408581078052520751953125f,-0.1364999908804893635760890902020037174224853515625f,-0.20120000635385526521758947637863457202911376953125f,0.797999995507299164643200128921307623386383056640625f,0.6093000131070613889505693805404007434844970703125f,0.23370002274513268503142171539366245269775390625f,0.696000020295381549573221491300500929355621337890625f,0.4196999955803153437017272153752855956554412841796875f,0.3859999802112581424040627098293043673038482666015625f,-1.086299977254867599185672588646411895751953125f,-0.8030999884247780329360466566868126392364501953125f,-0.530400011754036171396364807151257991790771484375f,-1.0285999859809873413496461580507457256317138671875f,-1.426499968528748496510161203332245349884033203125f,-1.562199996399879609043637174181640148162841796875f,-0.128100008392333819529085303656756877899169921875f,-0.58950000512599931568047395558096468448638916015625f,0.7908000242531301182680181227624416351318359375f,0.7590000076442955379008026284282095730304718017578125f,-0.4422000087678430446658239816315472126007080078125f,-0.0979999810904261803301551481126807630062103271484375f,-0.6614999822378155869273541611619293689727783203125f,-0.18920000569820416558286524377763271331787109375f,-1.10579998178481986315091489814221858978271484375f,-1.09339999291896816657754243351519107818603515625f,-1.03889992520809215648114331997931003570556640625f,-0.3535999365091324619925217120908200740814208984375f,0.07500001013278989603350055404007434844970703125f,-0.306299986046552508156537442118860781192779541015625f,0.0546000019550323401062996708787977695465087890625f,-0.17820000239014621712385633145458996295928955078125f,0.3610000203698875420599279095767997205257415771484375f,0.087400000041723213684008442214690148830413818359375f,0.0038999576926226353634774568490684032440185546875f,-0.210000000894069671630859375f,0.64370002021193517549590978887863457202911376953125f,0.06539999819397923630504010361619293689727783203125f,0.7698000200510026758138337754644453525543212890625f,-0.36860000147819516058689259807579219341278076171875f,-1.05820004098415409998779068700969219207763671875f,0.331800006401538727374145310022868216037750244140625f,-1.054500011384487123677899944595992565155029296875f,-0.01059999632239350120244125719182193279266357421875f,-0.87939998773336380821774582727812230587005615234375f,0.39659999143481261540955529198981821537017822265625f,-0.161800016534328960915445350110530853271484375f,0.6821999908864497808025362246553413569927215576171875f,-0.1232999743342393372813603491522371768951416015625f,-0.60399999400973314322982332669198513031005859375f,0.9365999993324276573503084364347159862518310546875f,-0.533600012803077561329700984060764312744140625f,-0.62859998210668521778643480502068996429443359375f,0.4915999923288822959221988639910705387592315673828125f,-0.6095999906301496906735337688587605953216552734375f,0.31899999281764035430342119070701301097869873046875f,0.2411000321745877528201162931509315967559814453125f,0.00300000545382506089708840590901672840118408203125f,1.1357000067114828567582662799395620822906494140625f,-0.11779999942779539878756622783839702606201171875f,-0.0651999996185299579565253225155174732208251953125f,0.4052999878048897386406679288484156131744384765625f,-0.00510000897347939741166555904783308506011962890625f,-0.2952000193625690105392322948318906128406524658203125f,-0.5525999652385706895074690692126750946044921875f,-0.6938999809384345240914626629091799259185791015625f,-0.18570001006424430300967287621460855007171630859375f,-0.65550002199411405712226041941903531551361083984375f,-1.007100004339218202176198246888816356658935546875f,-1.1796000168561935339539559208787977695465087890625f,0.9267000343084337288246388197876513004302978515625f,1.0500000214576719059778042719699442386627197265625f,0.317700021076202432368518202565610408782958984375f,0.72689999250173542577613261528313159942626953125f,1.037699982929229491901423898525536060333251953125f,1.13259997864961636793168509029783308506011962890625f,1.31729997504949647435523729654960334300994873046875f,0.90299999487400040010243174037896096706390380859375f,-1.03619998698234638823123532347381114959716796875f,-0.4885000055432318077919262577779591083526611328125f,-1.34760001947879715089584351517260074615478515625f,-1.4350000119209287419153042719699442386627197265625f,1.08359998508691734997455569100566208362579345703125f,0.92799997997283956152614337042905390262603759765625f,0.837299980115890463139294297434389591217041015625f,0.9363999842166901288464941899292171001434326171875f,0.137699977564810893682079040445387363433837890625f,0.4141000127434733002473876695148646831512451171875f,-0.58890001223087296011726721189916133880615234375f,-0.9518999919056891911850470933131873607635498046875f,-0.872099961125851308452183729968965053558349609375f,0.195999985724687686428069355315528810024261474609375f,-0.20120000456571585090159715036861598491668701171875f,0.49040000087618818014334465260617434978485107421875f,-0.0246999746441840528632383211515843868255615234375f,0.00379998977184303754484062665142118930816650390625f,-0.199700014579295981320683495141565799713134765625f,-0.739799987417459448124645859934389591217041015625f,1.3810000332593919125656611868180334568023681640625f,-0.0453999995350837792784659541212022304534912109375f,0.34920001970529579438107248279266059398651123046875f,0.08219999610185624305813689716160297393798828125f,0.05229999203681945374455608543939888477325439453125f,0.0355999976634979731215935316868126392364501953125f,-0.0746999941647050746951208566315472126007080078125f,-0.2075999975740909775367981637828052043914794921875f,-0.3695000069141389786864237976260483264923095703125f,0.09139999569058421258205271442420780658721923828125f,-0.28570001893043528440330192097462713718414306640625f,-0.4023999943792819777854674612171947956085205078125f,1.17539999595880484406507093808613717555999755859375f,-0.03779998899698266967561721685342490673065185546875f,0.364800021988153755359007845981977880001068115234375f,0.06199999767541886086519298260100185871124267578125f,-1.1601000352740289400799156283028423786163330078125f,0.051000001221895185921084703295491635799407958984375f,0.94730002124309553579450948745943605899810791015625f,-0.18319999002218256833884879597462713718414306640625f,-0.35509998848438240059977033524774014949798583984375f,0.31999999374151233411112116300500929355621337890625f,-0.34489999631643275534997883369214832782745361328125f,0.3522999935269357507650056504644453525543212890625f,0.873000005602836637308428180404007434844970703125f,0.42859999104738211173071249504573643207550048828125f,0.195899950599670802375840139575302600860595703125f,0.5228999762892723168761222041212022304534912109375f,-1.1936999637365346416117972694337368011474609375f,-0.398299959433078498705071979202330112457275390625f,-0.516300029110908553775516338646411895751953125f,-0.9936000116109848789847092120908200740814208984375f,-0.074100007379055199407957843504846096038818359375f,-0.46780002416372301610181239084340631961822509765625f,-0.041399990177154677439830265939235687255859375f,-0.24669999496936778626832165173254907131195068359375f,-0.44099998430907749735752076958306133747100830078125f,-0.4089000104427338300183691899292171001434326171875f,0.4010999840438369545836394536308944225311279296875f,0.6787000027775764721127416123636066913604736328125f,-0.57659997906684878898886381648480892181396484375f,-0.04809997382163988532965959166176617145538330078125f,-0.431699994373321516150099341757595539093017578125f,-0.82759997149705899488481009029783308506011962890625f,-0.115800004732609096436135587282478809356689453125f,-0.500000022798776644350482456502504646778106689453125f,0.3717000384807587209934354177676141262054443359375f,0.611700008976459486120802466757595539093017578125f,-0.636000000476837357155091012828052043914794921875f,-0.85690000114440945111482506035827100276947021484375f,0.646799990606307773077787715010344982147216796875f,0.45189997968673711881137933232821524143218994140625f,-0.3620999803483488932442924124188721179962158203125f,-0.1867999821126462567377757295616902410984039306640625f,0.613500009566545490002908991300500929355621337890625f,-0.01340000498890869806700720801018178462982177734375f,-0.471700009721517599103890461265109479427337646484375f,0.124800003957748373295544297434389591217041015625f,-0.78110001181587573881870412151329219341278076171875f,0.26000000111758703180697693824185989797115325927734375f,-1.404200027751922874585943645797669887542724609375f,-0.0954000209927556852562702260911464691162109375f,-0.5051999897837635700170721975155174732208251953125f,-0.2897999936759472205949350609444081783294677734375f,-0.821400032198429386198768042959272861480712890625f,0.0673999955713748821750641582184471189975738525390625f,1.141400003290176101700126309879124164581298828125f,0.299800000977516134526013047434389591217041015625f,0.694899996390938579082785508944652974605560302734375f,-0.262199990439415053788962950420682318508625030517578125f,0.1659999998807908383469111868180334568023681640625f,-0.21239999765753747595908862422220408916473388671875f,0.476100015485286576222279109060764312744140625f,-0.009800005596876104618786484934389591217041015625f,1.048500024616718206971199833787977695465087890625f,0.077599989676475633615382321295328438282012939453125f,0.64519998382329912800514648552052676677703857421875f,-0.26299998652935041576483854441903531551361083984375f,0.8723000191569330041829744004644453525543212890625f,-0.51380000273585313408375441213138401508331298828125f,-0.49680002033412495876518732984550297260284423828125f,0.157799991455674248896912104100920259952545166015625f,0.23580001426935215391722522326745092868804931640625f,-0.6270999697685244456124564749188721179962158203125f,-0.24090001616477962897988618351519107818603515625f,-0.500300003153085537377364744315855205059051513671875f,0.4368000081896781239265692420303821563720703125f,0.625700011032819691791928562452085316181182861328125f,0.709200012366473675484712657635100185871124267578125f,0.7846999974988400072817285035853274166584014892578125f,0.2738999913692481413818313740193843841552734375f,0.878399985170364772102402639575302600860595703125f,-0.3110999879181388649840300786308944225311279296875f,0.1983999953031538865388938575051724910736083984375f,0.24630002642869985862716930569149553775787353515625f,0.26179999716281887600644040503539144992828369140625f,0.21029997797012356386403553187847137451171875f,-0.399599987649917665066823246888816356658935546875f,-0.6470999932378529440057945976150222122669219970703125f,-0.6198999793291093407532343917409889400005340576171875f,-0.593999977171421278399066068232059478759765625f,-0.7690000060796737813006984652020037174224853515625f,-0.01230004091262770771209034137427806854248046875f,0.0930999931931495439130230806767940521240234375f,-0.0828000536680217891216670977883040904998779296875f,-0.3441000091671941873983087134547531604766845703125f,-0.711899973130226015882726642303168773651123046875f,-0.80539997390508677455045472015626728534698486328125f,-1.329900001978874257702045724727213382720947265625f,-1.4767000167250632358673101407475769519805908203125f,0.585299979944527759556649471051059663295745849609375f,0.90100001135468499313674328732304275035858154296875f,0.081000004947185288983746431767940521240234375f,-0.66439999741911892616741397432633675634860992431640625f,0.310699998366832375040758051909506320953369140625f,0.0571999995291232465888242586515843868255615234375f,0.489500021480024027908939387998543679714202880859375f,0.3474000013828277833027868837234564125537872314453125f,-0.312099981577694141066103838966228067874908447265625f,0.0778000038608908095483229772071354091167449951171875f,-1.675000074505806679781017010100185871124267578125f,-0.110199999171495477412463515065610408782958984375f,-0.22820001043677340391013785847462713718414306640625f,-0.168000002324581121371238623396493494510650634765625f,-0.733199994343519012574006410432048141956329345703125f,0.2316000008225440609521683654747903347015380859375f,-0.22079998983144744073570109321735799312591552734375f,0.187199999827146523756482565659098327159881591796875f,0.6294000032305717695635394193232059478759765625f,-0.1957999847650528923992396812536753714084625244140625f,0.2152999886989590550001594237983226776123046875f,-0.223800013911723993942359811626374721527099609375f,-0.3132000159502030811609074589796364307403564453125f,-0.6629999878704548876839908189140260219573974609375f,0.04879999528527267926847343915142118930816650390625f,-0.156599995458126084457717297482304275035858154296875f,0.771399992412328483482042429386638104915618896484375f,-0.2739999870061875508753246322157792747020721435546875f,-0.6447000227004291428301030464353971183300018310546875f,-0.019800000232458092597198628936894237995147705078125f,0.31829999549388876545208404422737658023834228515625f,-0.3339999925196170948993312777020037174224853515625f,-0.4885000271499162494137635803781449794769287109375f,-0.974799985182285499973886544466949999332427978515625f,0.12479997713565893491249880753457546234130859375f,0.175500000536441547183130751363933086395263671875f,0.57389997441917628417940022700349800288677215576171875f,0.29730000224411490261644530619378201663494110107421875f,0.3218999929301438189810369294718839228153228759765625f,0.524099987933039468401830163202248513698577880859375f,0.29550005269050672040975769050419330596923828125f,0.947000031948089571187665569595992565155029296875f,-0.26099999250471572764809025102294981479644775390625f,-0.09640000030994411162055257591418921947479248046875f,0.6954000001311300138695514760911464691162109375f,0.868399989193677779297786401002667844295501708984375f,0.4565999912858007547811212134547531604766845703125f,0.47859999343156811590915822307579219341278076171875f,-0.45749998897314103629696546704508364200592041015625f,-0.37119997088909162386016760137863457202911376953125f,-0.6387000044167034928932480397634208202362060546875f,-0.83830002022981631171205663122236728668212890625f,-1.04009996941089610800190712325274944305419921875f,-0.2995999840736391917062064749188721179962158203125f,-0.37649997873604323928020676248706877231597900390625f,-0.41039999327659604233531354111619293689727783203125f,-0.68819998733997334028344994294457137584686279296875f,-0.672599972540140367271987997810356318950653076171875f,0.213900011932850020457408390939235687255859375f,0.5519000158220530583008667235844768583774566650390625f,-0.60719999461174012367337127216160297393798828125f,-0.29329997642040250838135762023739516735076904296875f,0.74699997495114889289880011347122490406036376953125f,0.731399979895353435921379059436731040477752685546875f,0.76879999640285934248140620184130966663360595703125f,0.091400006121396926150879380656988359987735748291015625f,-0.523500010609626809099381716805510222911834716796875f,0.37680000148415560001780022503226064145565032958984375f,0.407999987721442636257052072323858737945556640625f,0.78059998097419747153935531969182193279266357421875f,0.2231000033259391557294293306767940521240234375f,-0.07719999714493752662747283466160297393798828125f,1.125200010943412820552111952565610408782958984375f,0.0510000053942203379619968472979962825775146484375f,-0.1233000184416770395046114572323858737945556640625f,-0.1255999957263469468671246431767940521240234375f,-0.4834000102043152224950972595252096652984619140625f,0.42760000293850897179481762577779591083526611328125f,-0.0219999628067011343546255375258624553680419921875f,0.213200009095668718828164855949580669403076171875f,-0.7435000228285790768723018118180334568023681640625f,-0.286200000393390663333548218361102044582366943359375f,0.20820000909566882540957521996460855007171630859375f,0.156200000095367386165889911353588104248046875f,-0.575700006115436480058633605949580669403076171875f,-0.62099998974800119100336814881302416324615478515625f,-0.66740002202093595684573301696218550205230712890625f,-0.309999989792704599977923862752504646778106689453125f,1.0903000110507008457716437987983226776123046875f,0.22679999135136608146012804354541003704071044921875f,-0.28860001518726363656242028810083866119384765625f,-0.7029999983012675812688030418939888477325439453125f,0.576899981102346970374128432013094425201416015625f,0.828499973356723984352356637828052043914794921875f,0.0182999879688028332935800790437497198581695556640625f,-0.1707999819338323899131637517712078988552093505859375f,0.835800009053945469617019625729881227016448974609375f,0.7181000025808810249827729421667754650115966796875f,1.233899958586693657025534776039421558380126953125f,0.3857999895334245366029790602624416351318359375f,-0.14280000538825987632662872783839702606201171875f,-0.0468999925017357099932269193232059478759765625f,-0.288300004136562648682229337282478809356689453125f,-0.793199984955787584794961730949580669403076171875f,-0.3185999639272694139435770921409130096435546875f,-0.4052999806523320813766986248083412647247314453125f,1.0403999989390371183617389760911464691162109375f,1.0900000202655792946870860760100185871124267578125f,0.562199982392787234175557387061417102813720703125f,0.7043999933958051684612655662931501865386962890625f,-0.3638999716997144417973686358891427516937255859375f,-0.006300001394748733218875713646411895751953125f,0.190499993383884458353350055404007434844970703125f,-0.05560001107454315416589452070184051990509033203125f,-0.918899969315528863944564363919198513031005859375f,-0.1796999996781352137986687012016773223876953125f,-0.326399960710108683059615941601805388927459716796875f,0.20640002936720858173202941543422639369964599609375f,0.093599958562851526266967994160950183868408203125f,-0.441700007188320142859083716757595539093017578125f,-1.1252999851226803684767219237983226776123046875f,-0.3717999882221221241707098670303821563720703125f,-0.9521999820202584619011076938477344810962677001953125f,0.16040000072717663925914166611619293689727783203125f,0.105000019073486328125f,0.51599998185038575826411033631302416324615478515625f,0.170899997389316382623292156495153903961181640625f,-0.6656000079452990547679291921667754650115966796875f,0.381499990787356912935734953862265683710575103759765625f,0.03719999766647819061660129591473378241062164306640625f,-0.3099000079393388062953818007372319698333740234375f,-0.1572000101089476853388759991503320634365081787109375f,-0.5501000096440318287704940303228795528411865234375f,0.39319999479055400826155164395458996295928955078125f,-0.3063999985963103966923881671391427516937255859375f,-0.254799992111325313004499548696912825107574462890625f,-0.461400014317035811473033390939235687255859375f,-0.448200002390146234887424725457094609737396240234375f,0.139900018966198302905468153767287731170654296875f,0.791200001734495117489132098853588104248046875f,-0.81400000607967371024642488919198513031005859375f,-0.01440000025033949526687138131819665431976318359375f,-0.095499994575977353861162555404007434844970703125f,0.3547999933779240055997661329456605017185211181640625f,-0.72760001325756318291126945041469298303127288818359375f,0.0376000007033348015994533852790482342243194580078125f,-0.915000002980232096660984097979962825775146484375f,0.69039999059438716955128256813623011112213134765625f,-0.25999998241662947151553453295491635799407958984375f,-0.3227999919772148729180116788484156131744384765625f,0.38779999525547026451022247783839702606201171875f,-0.283999989390373297482028647209517657756805419921875f,0.24800000649690634446642434340901672840118408203125f,1.0399999898672103171293201739899814128875732421875f,0.863399954771995936653183889575302600860595703125f,0.356799961400032117353475769050419330596923828125f,-1.28340000603198944872929132543504238128662109375f,-0.873500012695789251893074833787977695465087890625f,-0.101399999825656284002661777776665985584259033203125f,-0.358299990632384879329919158408301882445812225341796875f,-0.28020000051259952300597433350048959255218505859375f,-0.20770001899003975864843596355058252811431884765625f,0.992999979972839952324648038484156131744384765625f,1.0832999934077260917320018052123486995697021484375f,-0.379799976874888312039502125116996467113494873046875f,-0.0943999974191189750172270578332245349884033203125f,-0.703499985128640759057816467247903347015380859375f,-0.2498000057458877876115366234444081783294677734375f,1.34579999132156302721341489814221858978271484375f,0.603899994051456445731673738919198513031005859375f,0.30840000394582745002480805851519107818603515625f,0.760800000858306901818650658242404460906982421875f,0.73079997627139103855142820975743234157562255859375f,0.59409999307394034673279747948981821537017822265625f,0.36660000700652606298035607323981821537017822265625f,0.73100000707060097671075027392362244427204132080078125f,1.624799981904029522183918743394315242767333984375f,1.58260000278949775065484573133289813995361328125f,-0.41939999458789856134899309836328029632568359375f,0.07519999276399591536801381153054535388946533203125f,-0.6491999785780908638344044447876513004302978515625f,-0.5957999811887741969940179842524230480194091796875f,-0.3011999977111816662045384873636066913604736328125f,-0.16280000866651533897311310283839702606201171875f,-0.5319000306487087215145947993732988834381103515625f,-0.5905999993026256333905621431767940521240234375f,-0.666000006102025476906192125170491635799407958984375f,-0.1044000115752219404186007523094303905963897705078125f,0.7661999851197001110136852730647660791873931884765625f,-0.068000000834465001986472998396493494510650634765625f,-0.9721000064253804140435022418387234210968017578125f,-0.22240000123381609142825254821218550205230712890625f,-1.107399998402595286961513920687139034271240234375f,-0.19100000509619707855790693429298698902130126953125f,0.1532000028371811595206963829696178436279296875f,0.173600017124414307545521296560764312744140625f,0.552499998211860798846828402020037174224853515625f,-0.6563999846637249557801396804279647767543792724609375f,-0.518700027960539156168806584901176393032073974609375f,0.434600005233287678407805287861265242099761962890625f,-0.2763999998256563284115827627829276025295257568359375f,-0.0921999970331788265553996097878552973270416259765625f,-0.042100001060962455312619567848742008209228515625f,-0.709399986988306086033162500825710594654083251953125f,-0.7365000403523449534759492962621152400970458984375f,-0.403200002986192629350625793449580669403076171875f,-0.1963000082492829534430711646564304828643798828125f,-0.308000002920627569125144873396493494510650634765625f,-0.116899996674060702162023517303168773651123046875f,0.56980000902414307262233705841936171054840087890625f,-0.74850003005564236246982545708306133747100830078125f,0.209000005632638818919843970434158109128475189208984375f,1.1686000420093538565424751141108572483062744140625f,0.2658999830842017075838157325051724910736083984375f,-0.84029998154640139773619011975824832916259765625f,-0.0748999826073646346458190237171947956085205078125f,-0.07439999805241803709776604591752402484416961669921875f,0.08619997826516645478278633163427002727985382080078125f,-0.437999996066093189028833876363933086395263671875f,-0.8005999823898075629102777384105138480663299560546875f,-0.180599972629547522728898911736905574798583984375f,0.2050999923586847017986656283028423786163330078125f,0.005100006365776010852641775272786617279052734375f,0.5693999784946441877764300443232059478759765625f,0.44700000631809189144405536353588104248046875f,0.5754000326156614164574421010911464691162109375f,-1.40399996346235322874917983426712453365325927734375f,-1.17349997514486314997839144780300557613372802734375f,0.8901000245451928805096031283028423786163330078125f,0.639300011318921779235324720502831041812896728515625f,-0.0566999926038087576074531170888803899288177490234375f,0.1588000006496905758268667341326363384723663330078125f,-1.2728999646663670120005917851813137531280517578125f,-0.7094999845623970458063922706060111522674560546875f,-0.445799981486796781382508925162255764007568359375f,0.2339000244498254943437132169492542743682861328125f,-0.521999994248151466536000953055918216705322265625f,-0.256100005352497106514420011080801486968994140625f,1.056900009489059044653913588263094425201416015625f,0.63750000327825517842939007095992565155029296875f,0.5535000324398280380222558960667811334133148193359375f,0.5625000111758706733411372624686919152736663818359375f,0.40019995372295369406856480054557323455810546875f,0.14699999290704734988821655861102044582366943359375f,0.5598000088006258589956587456981651484966278076171875f,-0.273199997919797876733127850457094609737396240234375f,-0.9421000126838681154595178668387234210968017578125f,0.0276000145614145875327949397615157067775726318359375f,-0.09650000011920933928877275320701301097869873046875f,-0.4691999939262867513889432302676141262054443359375f,0.8882000299572947454862514859996736049652099609375f,0.1828000061333179271372273433371447026729583740234375f,0.19289999208450314682750104111619293689727783203125f,0.0783999978363514316015425720252096652984619140625f,-0.4144000236451628182976492098532617092132568359375f,-0.376200002628564789119991473853588104248046875f,0.7616000058889387247518243384547531604766845703125f,-0.3292000007808209005588651052676141262054443359375f,0.26959998511672012000417453236877918243408203125f,-0.256399997627735132255111238919198513031005859375f,0.2134999884068964348671215702779591083526611328125f,-0.3956000027298927079755230806767940521240234375f,0.29509999705255030022499340702779591083526611328125f,-0.668199996131658480180703918449580669403076171875f,-0.1248999920696018950394545754534192383289337158203125f,0.424199988859891963244308499270118772983551025390625f,-0.0820000164508822848574709496460855007171630859375f,-0.070800009351968729021109538734890520572662353515625f,-0.55830001240670679152344746398739516735076904296875f,-0.38659999732077121592510593472979962825775146484375f,-1.4029000418543819961314511601813137531280517578125f,-0.19760001754164679965697359875775873661041259765625f,-0.20640000448226913931648596189916133880615234375f,0.88530001134872460255564874387346208095550537109375f,-0.6672000025093556274669026606716215610504150390625f,-0.6567999960452317598225135952816344797611236572265625f,0.27360005184411928524923496297560632228851318359375f,0.47350001150369624980385196977294981479644775390625f,-0.73319999389648415899500832892954349517822265625f,-0.18089999768733999729874994955025613307952880859375f,0.14759997805356928068931665620766580104827880859375f,-0.15439999339580534609694950631819665431976318359375f,-0.03419998960495007622739649377763271331787109375f,-0.37109999551773054093928294605575501918792724609375f,-0.58409998256862127874455836717970669269561767578125f,-0.1969999929070471722525326185859739780426025390625f,-0.939600002253055066603337763808667659759521484375f,-1.1510000177621844130726458388380706310272216796875f,-0.70199997211992748447073608986102044582366943359375f,-0.873799990069866083786109811626374721527099609375f,-0.93119998303353757052036598906852304935455322265625f,-0.9649000055551528731712096487171947956085205078125f,-1.3274999841302632663797567147412337362766265869140625f,-0.9622999933779239345454925569356419146060943603515625f,0.7046999855220317243720273836515843868255615234375f,0.2400999773830176930999869000515900552272796630859375f,-0.2099999997019761366345846909098327159881591796875f,-0.31500001251697540283203125f,-0.543899976281821739831912054796703159809112548828125f,-0.07089999664425850056659328402020037174224853515625f,0.041400008058548820599753526039421558380126953125f,0.6414999932646754388088083942420780658721923828125f,0.538500012397765459581933100707828998565673828125f,-0.270399987465143265552569573628716170787811279296875f,-1.31930003709793108868097988306544721126556396484375f,0.14240000585317602599388919770717620849609375f,-0.62390000209808338382799774990417063236236572265625f,0.200199995744228420591070971568115055561065673828125f,0.5435999943256373256872393540106713771820068359375f,0.012000002145767130201647887588478624820709228515625f,0.0207999948978425663881353102624416351318359375f,0.10020001168847070260881082504056394100189208984375f,0.4343000014841553735323032015003263950347900390625f,0.43899999013543133941084306570701301097869873046875f,-0.4344000029325485456865862943232059478759765625f,-0.05680002145171147986957294051535427570343017578125f,-0.27240001300573357667644813773222267627716064453125f,0.047200004893541258610412114649079740047454833984375f,-0.94080002559423459018717039725743234157562255859375f,0.201600002908706610327271846472285687923431396484375f,-0.352700011558085779750371102636563591659069061279296875f,0.25919999787509440114519065900822170078754425048828125f,0.8986999705910676539133419282734394073486328125f,-0.55219999863505364601223845966160297393798828125f,-1.32049999586120225814056539093144237995147705078125f,0.28559999833405014246778819142491556704044342041015625f,-0.64710001188293109208871101145632565021514892578125f,-0.01420000174939628490022869300446473062038421630859375f,-0.0009000219017269994736807348090223968029022216796875f,0.335000002384185702197783029987476766109466552734375f,-0.809800006788969017890167378936894237995147705078125f,0.05979999978542327454533733543939888477325439453125f,-0.2927000112414361865376122295856475830078125f,-0.03909998502731326652792631648480892181396484375f,0.6444000410795212019365862943232059478759765625f,0.97370001499652847343213579733856022357940673828125f,0.5912999969244001619017581106163561344146728515625f,0.72529999823570268091543766786344349384307861328125f,0.0107999758243568777515974943526089191436767578125f,0.0828000107526776218946906737983226776123046875f,0.009900039529799897763950866647064685821533203125f,-0.367599995636940235499423579312860965728759765625f,0.70829996382892179696000312105752527713775634765625f,0.2946999963998795379893635981716215610504150390625f,-0.157800010156631032032237271778285503387451171875f,-0.291400039052962966934501309879124164581298828125f,0.1254000200986862267882315791212022304534912109375f,0.10900000518560393203415515017695724964141845703125f,0.6432000254869461031148603069595992565155029296875f,0.80480001021623603918442313442938029766082763671875f,0.539400005652010516854488741955719888210296630859375f,0.396899999672919501225720750881009735167026519775390625f,-1.377899983143805684449034743010997772216796875f,-1.4518999787926674116533831693232059478759765625f,0.961200004155933651617260693456046283245086669921875f,1.2808999775148925959200596480513922870159149169921875f,0.238500003941357174852555544930510222911834716796875f,0.5973000070311129672262495660106651484966278076171875f,0.4851000196278096865398765658028423786163330078125f,0.0099000077158211130523568499484099447727203369140625f,0.438300002944469468957322533242404460906982421875f,0.825999999731779066536319078295491635799407958984375f,-0.977700003194808431317142094485461711883544921875f,-0.17539999566078190440521211712621152400970458984375f,-0.32120000396966918998487017233856022357940673828125f,-0.417399989312887242931537912227213382720947265625f,0.009600003296136971897567491396330296993255615234375f,-0.459400001591443896131750079803168773651123046875f,1.4987000069499014642815382103435695171356201171875f,-0.0203999935746193017394034541212022304534912109375f,0.65200001846253885506854430786916054785251617431640625f,-0.33320000119805326388444655094644986093044281005859375f,0.2393000177264212879890692420303821563720703125f,0.5119999866485596129450641456060111522674560546875f,-0.561300008547306106265750713646411895751953125f,0.090799989235401223908183965249918401241302490234375f,-0.613600017720461021752953456598334014415740966796875f,-0.022999997109174774578832511906512081623077392578125f,0.726700025665760307447271770797669887542724609375f,0.36660000529289238357932845246978104114532470703125f,-0.990100035274029011134189204312860965728759765625f,-0.32540000549554815023611809010617434978485107421875f,-0.16640002266168618660913125495426356792449951171875f,0.2598000038087366903027941589243710041046142578125f,-0.105900014078617488166855764575302600860595703125f,0.5967999971628188404793036170303821563720703125f,-0.429300022196770214577554725110530853271484375f,0.5245999912261962716542029738775454461574554443359375f,0.3207000165276230330135831536608748137950897216796875f,0.29059998358190075595786083795246668159961700439453125f,-0.11390000329017624380867346189916133880615234375f,-0.2573999908030033534345193402259610593318939208984375f,0.2711999936133622224332384575973264873027801513671875f,-1.022800004196167122927363379858434200286865234375f,-0.617699973094463672396159381605684757232666015625f,-0.10879998649358757489835625165142118930816650390625f,-0.753900003588199751902720890939235687255859375f,-0.257599989825486819228217427735216915607452392578125f,-0.49860000863075271126945153810083866119384765625f,-1.0222999732613562429150988464243710041046142578125f,-0.76920001571923501959560098839574493467807769775390625f,-0.68340000219494090316629808512516319751739501953125f,0.86639995560646099193036206997931003570556640625f,0.40589997056722637580605805851519107818603515625f,0.2394000005483629678337820223532617092132568359375f,0.1976999808430672800341199035756289958953857421875f,0.1635000171661378232101924368180334568023681640625f,0.430800006967782866951210962724871933460235595703125f,0.4100999893784518235406721942126750946044921875f,-0.188900005972385542918345890939235687255859375f,-0.30749997526407213399579632095992565155029296875f,0.26030001224279430971364490687847137451171875f,0.38610002651214614388663903810083866119384765625f,0.00440000442266441638139440328814089298248291015625f,1.16399998462200215953998849727213382720947265625f,0.828700003373622706703827134333550930023193359375f,1.12589998248815614800832918263040482997894287109375f,0.95069999777078617597680931794457137584686279296875f,0.313499971605837313148867906420491635799407958984375f,-0.229100025837868148670395385124720633029937744140625f,-0.43889997497797050840517840697430074214935302734375f,-0.21209998675584795790882708388380706310272216796875f,-0.4589999850690364979755031527020037174224853515625f,-0.138599997752904879888546929578296840190887451171875f,0.4080000073909761937329676584340631961822509765625f,-0.351399987494945609256546958931721746921539306640625f,-0.3110000106096268979172236868180334568023681640625f,0.233999997138977011701399533194489777088165283203125f,-0.0452999666452402749428074457682669162750244140625f,0.16020000841021531101660002605058252811431884765625f,0.351999980837106374309541934053413569927215576171875f,0.11179999671578411124528429354541003704071044921875f,0.578700009147822935773319841246120631694793701171875f,0.08840001109838480086722256601206026971340179443359375f,-0.302800009635090905391052729100920259952545166015625f,0.042799998831748953254106027088710106909275054931640625f,-1.1889000188060105944742872452479787170886993408203125f,0.25619999793469905302600864160922355949878692626953125f,-0.1917999685525888509118885849602520465850830078125f,0.468999995499849386959567709709517657756805419921875f,0.45799998831748922611950547434389591217041015625f,-0.08479999263286597255273591144941747188568115234375f,-0.53610000604167584725701090064831078052520751953125f,-0.02059999743998053300941819543368183076381683349609375f,-0.1563999904751778302625098149292171001434326171875f,0.5441999788761140433024365847813896834850311279296875f,-0.680400010561942991671458003111183643341064453125f,-0.015599994832277364054107238189317286014556884765625f,-1.12250000894069668078145696199499070644378662109375f,0.1905999970674514543134137056767940521240234375f,0.41539999059438681427991468808613717555999755859375f,0.1831999928534031596427666954696178436279296875f,-0.121600014680624202156877800007350742816925048828125f,0.3936000136226416312723586088395677506923675537109375f,-0.8396999716639523825278956792317330837249755859375f,-0.851399992561340468455455265939235687255859375f,0.4314000074625017333573850919492542743682861328125f,0.20579999012947070013979100622236728668212890625f,0.4286999886214726274147324147634208202362060546875f,0.5412999924540518037474612356163561344146728515625f,0.04530000121593502626637928187847137451171875f,-0.254399983412027186346904272795654833316802978515625f,0.05280000065714085533130628391518257558345794677734375f,-0.09849998103082191480694973506615497171878814697265625f,0.1362000080674887758647173541248776018619537353515625f,0.1507999984741210663941046732361428439617156982421875f,0.873900010778009939116373061551712453365325927734375f,1.180500001151114641828598905703984200954437255859375f,0.9182999799966804488349225721322000026702880859375f,0.640999980807304581276184762828052043914794921875f,-0.120000008940696289982952293939888477325439453125f,0.090000029802322245586765347979962825775146484375f,0.183900007648766017354091673041693866252899169921875f,0.492100003240257433123616692682844586670398712158203125f,1.02689995435476344454173158737830817699432373046875f,0.64899997031688716475628098123706877231597900390625f,0.1560000246167181359169262577779591083526611328125f,0.3682000010490416030961569049395620822906494140625f,0.70290000758171089501047390513122081756591796875f,0.98029998631477344162021836382336914539337158203125f,0.146999989926815288754369248636066913604736328125f,-0.24519999455213525862973256153054535388946533203125f,0.5676000042796136568767906283028423786163330078125f,-0.32840000349879261420937837101519107818603515625f,1.1209000206351280581884566345252096652984619140625f,0.207400004810094795715258442214690148830413818359375f,0.6249999839067452711560690659098327159881591796875f,0.63259998431205755053952088928781449794769287109375f,-1.19470002599731106585068118874914944171905517578125f,0.21639999248683461363640390118234790861606597900390625f,-0.032099992120265596895478665828704833984375f,-0.13999999403953555798807428800500929355621337890625f,0.19990001471936746924029648653231561183929443359375f,-0.5217999904572963032478583045303821563720703125f,1.3010000499486924496750361868180334568023681640625f,0.28039998299479496068897788063623011112213134765625f,1.20430004335641882562413229607045650482177734375f,0.138599994474649435005630948580801486968994140625f,-0.7212999996066091767943362356163561344146728515625f,0.134999997913837432861328125f,-1.188800010037422083541969186626374721527099609375f,-0.0663999980747700391248145024292171001434326171875f,-0.0060000088214877678183256648480892181396484375f,0.40739999050498010291221362422220408916473388671875f,1.097100014024972924886469627381302416324615478515625f,-0.0368000078916548289953425410203635692596435546875f,0.4170999992728230409966272418387234210968017578125f,0.75119998102188123567657385137863457202911376953125f,0.0343999958544969342000285905669443309307098388671875f,-0.228799999308586077262361868633888661861419677734375f,-0.950899998581409278131104656495153903961181640625f,0.3089999977350235127460109652020037174224853515625f,-0.84910002742111689855164513573981821537017822265625f,0.527400007343292287487201974727213382720947265625f,0.0926999769687650854166349745355546474456787109375f,-0.361299990665912673648563213646411895751953125f,0.908699963140488620183532475493848323822021484375f,0.1143999969720841392017973703332245349884033203125f,0.806400006158650040077873200061731040477752685546875f,1.1509000079132618576949198541115038096904754638671875f,-0.392399971580505990687015582807362079620361328125f,-0.535500007092952756693193805404007434844970703125f,-1.0286999776691205799039607882150448858737945556640625f,-0.7350999922096728145248789587640203535556793212890625f,0.136799925041199088582288823090493679046630859375f,-0.4964000426292418666207595379091799259185791015625f,-0.15090004239082333015176118351519107818603515625f,-0.74670003281831753838559961877763271331787109375f,0.4467000062942503291196771897375583648681640625f,0.5662999870896339160708521376363933086395263671875f,0.22500002384185791015625f,0.667399985885619884129482670687139034271240234375f,0.745499977588654161309023038484156131744384765625f,0.441999992012977571675946819595992565155029296875f,-0.44160002585649493767050444148480892181396484375f,-0.873499997645616144126279323245398700237274169921875f,1.198799955201149458616782794706523418426513671875f,0.4072999893546107585962090524844825267791748046875f,-0.3875999972760677536598450387828052043914794921875f,-0.1563999890595673125659459401504136621952056884765625f,0.9428999950647352079613483510911464691162109375f,1.205499993979930906107256305404007434844970703125f,0.4440000431090591614946561094257049262523651123046875f,-0.1244000099360942357407111558131873607635498046875f,0.44500000685453411364278508699499070644378662109375f,0.154199998247623426550489966757595539093017578125f,-1.271300035369396397300079115666449069976806640625f,0.26540000117421147507457135361619293689727783203125f,-0.0734000137805939090185347595252096652984619140625f,-0.7605999936401843797284527681767940521240234375f,0.95689999756813026721147252828814089298248291015625f,-0.14580000160336492598389668273739516735076904296875f,-0.3980000131279231023739839656627736985683441162109375f,0.30880000079870217888355909963138401508331298828125f,0.9603000027060506482712298748083412647247314453125f,0.143600006097555077388960853568278253078460693359375f,0.57170000599622738945981836877763271331787109375f,-0.40199999380111695046480235760100185871124267578125f,0.782200001615285689382517375634051859378814697265625f,0.0159999901950360623459346243180334568023681640625f,-0.35199997867643784132951623178087174892425537109375f,0.6105999994516373430286648726905696094036102294921875f,-0.07320002567321093334129500362905673682689666748046875f,0.4393999956309795340558821408194489777088165283203125f,0.14919999288320529018392335274256765842437744140625f,-0.080999993026256600359147341805510222911834716796875f,-0.0838999746613200425571221785503439605236053466796875f,0.41579999169409276316144996599177829921245574951171875f,0.7699999977648257498685779864899814128875732421875f,0.2104000023663044061095206416212022304534912109375f,-0.176100007140636449776138761080801486968994140625f,-0.540999999582767454597842515795491635799407958984375f,-0.12989998797178259337670169770717620849609375f,0.7525999956369400223366028512828052043914794921875f,0.091499990880489434630362666212022304534912109375f,-0.27800000768899923997423684340901672840118408203125f,0.880500028252601651956865680404007434844970703125f,1.1469000163435936201494769193232059478759765625f,-1.34969997196197510902493377216160297393798828125f,-0.7241999818563460422637945157475769519805908203125f,-0.64470000385045977964182384312152862548828125f,-1.02809999468326562777065191767178475856781005859375f,0.6144000138103962171953753568232059478759765625f,0.3987999990850685794185892518726177513599395751953125f,0.0011999893665315397583981393836438655853271484375f,-0.39049998503923433190720970742404460906982421875f,-0.964199985432625084058599895797669887542724609375f,-0.964499992907047243306806194595992565155029296875f,-0.23220002471208545102854259312152862548828125f,-0.561999996632337239788057559053413569927215576171875f,1.2260999908238641165070248462143354117870330810546875f,0.9001999914228917543113084320793859660625457763671875f,0.68370001179277917202625758363865315914154052734375f,0.13100000781565912522097505643614567816257476806640625f,-0.0956999962806701631734540569595992565155029296875f,0.08500002235174175524434758699499070644378662109375f,0.650099985168874017205098425620235502719879150390625f,0.129899959938973186268640347407199442386627197265625f,0.09480000015795209122870801365934312343597412109375f,-0.4323999808192251492755531216971576213836669921875f,-0.9608999833822249314607688575051724910736083984375f,-0.5405000005364417603459514793939888477325439453125f,0.455999986171722326844246708787977695465087890625f,0.208399987852573520541454854537732899188995361328125f,-0.25320000127255948285664999275468289852142333984375f,0.5777999809503555628253934628446586430072784423828125f,0.78540000087618810908907107659615576267242431640625f,0.1311999959230423229428197373636066913604736328125f,-0.8034000143766402146638938575051724910736083984375f,0.39620000680088995892447201185859739780426025390625f,1.026400019383430617381236515939235687255859375f,0.304800007385015447880505234934389591217041015625f,0.5902999758839602151283543207682669162750244140625f,-0.554599996292591157498463871888816356658935546875f,0.7387000346660617111638202914036810398101806640625f,0.2558000001132487710719942697323858737945556640625f,-0.37559999475777150479416377493180334568023681640625f,0.324399997493624692168623369070701301097869873046875f,-0.52480002467036257485233363695442676544189453125f,-0.27119999309182174496157813337049447000026702880859375f,0.8984999983906742926365041057579219341278076171875f,-0.69860000907778729839492370956577360630035400390625f,-0.01920000659227394379513498279266059398651123046875f,0.44040000013113012045096184010617434978485107421875f,0.07300001305341741186794024542905390262603759765625f,0.016800000292062744478016611537896096706390380859375f,0.272300004851818044926403672434389591217041015625f,-0.668000007092952596821078259381465613842010498046875f,0.1644999756217000452807042165659368038177490234375f,-0.261200000542402221981319598853588104248046875f,-0.89010002186298375903561463928781449794769287109375f,0.165800000709295236589468913734890520572662353515625f,-0.854100000971555584072802957962267100811004638671875f,0.89339998308420209838232040056027472019195556640625f,1.1672999544113882297580175873008556663990020751953125f,0.8861999755084515317804516598698683083057403564453125f,-0.08100001142919044383461368852294981479644775390625f,-0.60119999949932090288484687334857881069183349609375f,1.0875000140070909537826082669198513031005859375f,1.2862000304937364347779293893836438655853271484375f,0.28979998488426161884490284137427806854248046875f,-0.2205999833583831559735699556767940521240234375f,-1.27169998006820605951361358165740966796875f,-1.2052999884009363285031213308684527873992919921875f,0.1628999809086320027518013375811278820037841796875f,-0.4303000271439554325070275808684527873992919921875f,0.862199976246059041073976914049126207828521728515625f,1.02719999923110005823900792165659368038177490234375f,-0.432599957638978960261511019780300557613372802734375f,-0.215799991023540638224176291259936988353729248046875f,-1.643100009584426146602709195576608180999755859375f,-1.6037000111222265985588819603435695171356201171875f,0.790199994850159015413737506605684757232666015625f,0.43199999529123278563247367856092751026153564453125f,-0.0449999997019769892858676030300557613372802734375f,-0.17800001364946371751329934340901672840118408203125f,-1.2672000118970867532652846421115100383758544921875f,-0.8003000062823293347946673748083412647247314453125f,-0.633599977040290696095325984060764312744140625f,-0.6846999823927877315554724191315472126007080078125f,0.5055000264644622376408733543939888477325439453125f,0.57530000181198115427605443983338773250579833984375f,0.5817000116586683589048334397375583648681640625f,-0.221000005394220266907723271287977695465087890625f,0.699300014746189191328085144050419330596923828125f,-0.27639999807477000359767771442420780658721923828125f,0.11930001005232337973893663729541003704071044921875f,0.455399997895956032412101421869010664522647857666015625f,-0.7509000379204753272688321885652840137481689453125f,-0.0109999949634076443771846243180334568023681640625f,0.131400034284592237554534221999347209930419921875f,0.60640000194907184294379476341418921947479248046875f,-0.1793000168323519716295777470804750919342041015625f,0.5632000038802622565725641834433190524578094482421875f,0.3642000122547148777130132657475769519805908203125f,-0.02719999729394917409308618516661226749420166015625f,-0.55380001838207260078661420266143977642059326171875f,-0.0652000018537044478961206550593487918376922607421875f,-1.396400024151802199412486515939235687255859375f,-0.1194000053167343367022112943232059478759765625f,-0.6347000223279000152842854731716215610504150390625f,0.10320000507235516096216088044457137584686279296875f,0.4258000076383352539011184489936567842960357666015625f,0.005599998855590815338700849679298698902130126953125f,-0.6347999888330695394955682786530815064907073974609375f,0.154399991905689315530025851330719888210296630859375f,0.76400001651048654593978426419198513031005859375f,0.544199995118379575842482154257595539093017578125f,0.61470001428127307718796146218664944171905517578125f,-0.23939999235272413358188714482821524143218994140625f,-0.15460003730803772992175026956829242408275604248046875f,-0.3237999879837036854013376796501688659191131591796875f,0.65650002828240427987793736974708735942840576171875f,-0.1143999992072581850521828528144396841526031494140625f,-0.1258000022739171885888254109886474907398223876953125f,0.01030000136494635398776154033839702606201171875f,0.2483000074148178082378279896147432737052440643310546875f,0.2823000093594194570556510370806790888309478759765625f,0.0380000012069940940051537836552597582340240478515625f,0.2769000090420246085187727658194489777088165283203125f,-0.153599994027614650260460393838002346456050872802734375f,0.468899984291195892627257535423268564045429229736328125f,-0.09690000166594980868239872506819665431976318359375f,-0.63119999875426291424673763685859739780426025390625f,0.022200004446506493849256003159098327159881591796875f,-0.575399998939037260470286128111183643341064453125f,-0.23379999774396420519906314439140260219573974609375f,-0.87069999955892551923852806794457137584686279296875f,-0.171099998870491987190689542330801486968994140625f,-0.056200003671646125980032593361102044582366943359375f,-0.129999999478459304924626849242486059665679931640625f,0.72460001409947871753303161312942393124103546142578125f,0.20280000218451021964938263408839702606201171875f,-0.44340000357329845126486134176957421004772186279296875f,-0.125600005486607546600907880929298698902130126953125f,-0.222700004088878689145758471568115055561065673828125f,-0.30340000498890873359414399601519107818603515625f,-0.5099999997019768471773204510100185871124267578125f,-0.038399993738531978948458345257677137851715087890625f,-0.53489997336864458787886178470216691493988037109375f,-0.1996999965488910344646455996553413569927215576171875f,0.1517000068902969456274831827613525092601776123046875f,0.1093000002920627711233692025416530668735504150390625f,0.89130000601410852567596521112136542797088623046875f,0.1728000029295679862428869455470703542232513427734375f,-0.40190002275109326657087649437016807496547698974609375f,0.788099952363968014878992107696831226348876953125f,0.1911000128030782008181631681509315967559814453125f,0.4420999948024746384334093818324618041515350341796875f,-0.59140002601444752183823538871365599334239959716796875f,0.8959999894499777184364575077779591083526611328125f,-0.028500017315149506202942575328052043914794921875f,-0.58120001469850546271800340036861598491668701171875f,0.2015000106990338935020190547220408916473388671875f,-0.27769999544620560527619090862572193145751953125f,0.71490000123381580721115824417211115360260009765625f,-0.7683999807000159165681907325051724910736083984375f,0.0218000035703180827795222285203635692596435546875f,-0.830799976718426425037478111335076391696929931640625f,-0.88299998593330375484811156638897955417633056640625f,0.5112999942421918575519157457165420055389404296875f,0.98939998892545677477983190328814089298248291015625f,1.0681999968767161846017188508994877338409423828125f,0.11340001735687277317765619955025613307952880859375f,-0.640199994850158216053159776492975652217864990234375f,-0.31690000784993177518344964482821524143218994140625f,-0.70539997539520271629953640513122081756591796875f,-0.02399999281764042535769476671703159809112548828125f,-0.6420999715566626520057980087585747241973876953125f,-0.0205000009834765961613811668939888477325439453125f,0.4716999952673912588352322927676141262054443359375f,-0.58720000548958761754647639463655650615692138671875f,0.76499996185302787665705182007513940334320068359375f,0.258599988067150121651138761080801486968994140625f,-0.074000003397464764187674290951690636575222015380859375f,-0.01390000314116479618320454392232932150363922119140625f,-0.38489999899864191235110411071218550205230712890625f,-0.05309999125599862423996455618180334568023681640625f,-0.168100003996491442759264600681490264832973480224609375f,-0.105400004228949573104046066873706877231597900390625f,-0.19320000291168692996279787621460855007171630859375f,-0.608300012183189409142869408242404460906982421875f,-0.051999999389052309339831481338478624820709228515625f,0.791900022900104527678877275320701301097869873046875f,-0.059100003728270511516029728227294981479644775390625f,0.42680000371932980129940915503539144992828369140625f,0.07240000458657736004397520446218550205230712890625f,0.735499987274408351112242598901502788066864013671875f,0.1219999937266111889044850613572634756565093994140625f,0.7391000125944613419193274239660240709781646728515625f,-0.026899994289875106545650851330719888210296630859375f,-0.58690001872777930458369155530817806720733642578125f,-0.175400006389617946211956223123706877231597900390625f,-0.85820001684427271726463004597462713718414306640625f,-0.0025999911665917085912269612890668213367462158203125f,0.7731999868929386732219199984683655202388763427734375f,0.135800001379847490312613444984890520572662353515625f,0.5184999916851520396221530972979962825775146484375f,0.4597000086933373808051328524015843868255615234375f,-0.2980000064969061668307404033839702606201171875f,0.023899998149275791803347601671703159809112548828125f,-0.58019999708533287474665485206060111522674560546875f,-0.2641999945968389607031667765113525092601776123046875f,-0.32579996986389136992556814220733940601348876953125f,-0.28000000834465055277178180404007434844970703125f,-0.64790001801252383728524364414624869823455810546875f,-0.6206999381661422177103304420597851276397705078125f,0.1352000185430046741430487600155174732208251953125f,0.41109997942447762397932820022106170654296875f,1.285300018054247317422777996398508548736572265625f,0.3992000125527379594814192387275397777557373046875f,-0.84350002327561401926914186333306133747100830078125f,0.38819999032020557905298119294457137584686279296875f,-0.429900023287534960303446496254764497280120849609375f,0.2788999737858777194787762709893286228179931640625f,-0.14870000098943680910679177031852304935455322265625f,0.4874999833106994362452724089962430298328399658203125f,0.29609998501241185930865640330011956393718719482421875f,-0.588699971485138195959052609396167099475860595703125f,0.1903000235676766127568271258496679365634918212890625f,0.120899992322922145149277639575302600860595703125f,0.8276000206708911122177596553228795528411865234375f,0.31799998712539689904588158242404460906982421875f,-0.71490001226067523276697102119214832782745361328125f,0.4131999877870082826802899944595992565155029296875f,-0.41040002650618578883268128265626728534698486328125f,-0.06090002868175492523050706950016319751739501953125f,0.6309000109493734242960272240452468395233154296875f,0.448300001156329841478509479202330112457275390625f,-0.35770002822876012515962429461069405078887939453125f,-0.262899998939037260470286128111183643341064453125f,-0.27050000917911543041327604441903531551361083984375f,0.093100001239776464245778697659261524677276611328125f,0.052299993526935484311479740426875650882720947265625f,-0.132400001829862556945727192214690148830413818359375f,0.663600011163949954351437554578296840190887451171875f,0.385099992284178682666606619022786617279052734375f,-0.15980001513361941078983363695442676544189453125f,0.0921000090330839071839363896287977695465087890625f,-0.7561999997973440912346632103435695171356201171875f,0.086799995002150520662098642787896096706390380859375f,-0.62670000301599504410887675476260483264923095703125f,-0.2102000004380941700077300993143580853939056396484375f,0.046499990582466121935567571199499070644378662109375f,-0.183699993836879738040579468361102044582366943359375f,0.100499997705221222332738761906512081623077392578125f,-0.213900000011920943254750682172016240656375885009765625f,-0.09690000539124010714431278756819665431976318359375f,0.23909999746978283230447459573042578995227813720703125f,0.1344000003993510716782111558131873607635498046875f,0.07470000325441361610501189716160297393798828125f,-0.353299996984004938127554851234890520572662353515625f,0.087199998262524669456752235419116914272308349609375f,-0.538499979615211454841983140795491635799407958984375f,-0.21159999933242801262167631648480892181396484375f,0.0193999905645847459112474098219536244869232177734375f,-0.253499996230006185982119859545491635799407958984375f,0.621600006484985367904982922482304275035858154296875f,0.030899994260072727314536678022705018520355224609375f,-0.579899993187189100041223355219699442386627197265625f,-0.086699996906518972394906086265109479427337646484375f,0.520499991148710261512633223901502788066864013671875f,0.01049999777972698833394815665087662637233734130859375f,-0.240200016754865686152697890065610408782958984375f,0.25069999821782129600933330948464572429656982421875f,-0.401100000882148588488007590058259665966033935546875f,-0.4566999948203562809112554532475769519805908203125f,0.16910000633597377373007475398480892181396484375f,-0.4369000006414951453592721009044907987117767333984375f,0.12490000009760249266577147864154540002346038818359375f,-0.281599988305568871282957843504846096038818359375f,1.0711000528872016701598113286308944225311279296875f,-0.023399991130828912133665653527714312076568603515625f,0.8003999953627582986115385210723616182804107666015625f,0.173199973332881729248811097932048141956329345703125f,0.34980000991821302847029073745943605899810791015625f,-0.096799993288517072187460144050419330596923828125f,-1.1791000063359735605672540259547531604766845703125f,0.28569999993145456240739576969644986093044281005859375f,-0.0380000022128224035355970045202411711215972900390625f,0.78519996534585967395969419158063828945159912109375f,0.097299981307983340883538403431884944438934326171875f,-0.3906999527692800455724864150397479534149169921875f,-0.19289998448491108007374350563623011112213134765625f,-0.16729996223449727921206431346945464611053466796875f,-0.733499989748001013367684208787977695465087890625f,0.5397999789237974965772082214243710041046142578125f,-0.0583000096499919351344942697323858737945556640625f,1.1610999824047087969347558100707828998565673828125f,0.1000999847590922087192666367627680301666259765625f,-0.76859996988773360282465318960021249949932098388671875f,0.192300041322410464328385160115431062877178192138671875f,0.27579998774528480254275564220733940601348876953125f,-0.5916999770879745046414655007538385689258575439453125f,-0.0114999985545873872894162559532560408115386962890625f,-0.2486000066936016050789248765795491635799407958984375f,0.114300001335144063574489337042905390262603759765625f,0.2663000079512596318664918726426549255847930908203125f,-0.04150000002607701077295132563449442386627197265625f,0.60210000633224847188529338382068090140819549560546875f,-0.09079999810159211648397103999741375446319580078125f,0.4866999773859976841094976407475769519805908203125f,0.05539999946057794222298298336681909859180450439453125f,-0.13830000503063200056885762023739516735076904296875f,-0.3332000000804662409592538097058422863483428955078125f,0.5429000026643275855775527816149406135082244873046875f,0.16039999692738060588226289837621152400970458984375f,-0.1964999858140945576678859652020037174224853515625f,0.118699995774030730988357618116424418985843658447265625f,-0.6055999870091677284023035099380649626255035400390625f,-0.3447999982953071640423559074406512081623077392578125f,-0.137000003486871690938642132095992565155029296875f,0.361099992313981044134152398328296840190887451171875f,-0.359000003993511285216300166212022304534912109375f,0.303299996238946878435172038734890520572662353515625f,-0.462700009900331554746344409068115055561065673828125f,-0.05300000448524944118844359763897955417633056640625f,-0.509699979114532464308240378159098327159881591796875f,-0.2595000020712614485773883643560111522674560546875f,-0.7740000162124633931171047152020037174224853515625f,0.28819999903738502400329934971523471176624298095703125f,0.23310000127702945604823980829678475856781005859375f,-0.1986000022232532469246280015795491635799407958984375f,0.729100023099780170099393217242322862148284912109375f,-0.378299962413311874342980445362627506256103515625f,-0.20799998772144334679978783242404460906982421875f,-0.555699979330599003191082374542020261287689208984375f,-0.76990000974610450779067605253658257424831390380859375f,-0.437299975496530823892982198231038637459278106689453125f,0.1474999895133073668507250886250403709709644317626953125f,-0.2124000264167786866664755507372319698333740234375f,1.24640004665255599292095212149433791637420654296875f,0.2061999941349028642179064263473264873027801513671875f,0.37990000980198390667652574848034419119358062744140625f,-0.57359998553395286080558435060083866119384765625f,0.17159999590516061829248428693972527980804443359375f,-0.4067999864339828519632646930404007434844970703125f,-0.098700003969669491965532870381139218807220458984375f,0.038000000238418873976797840441577136516571044921875f,-0.05970000921487805811693760915659368038177490234375f,0.838099983060359488007406980614177882671356201171875f,0.0758000106930734052212983442586846649646759033203125f,0.376200003075599198609779705293476581573486328125f,1.231800010126828937018217402510344982147216796875f,0.4309999739527705653330258428468368947505950927734375f,0.14659999061524875418882629674044437706470489501953125f,0.302599996978044405437913155765272676944732666015625f,-0.0680000041127204468693889793939888477325439453125f,-0.303900001427531041287011248641647398471832275390625f,0.4869000226765873851064725386095233261585235595703125f,0.0877000055789949328755028545856475830078125f,0.352200010406971131260434049181640148162841796875f,-0.8194999690651896262494346956373192369937896728515625f,-0.83079999161958706377362204875680617988109588623046875f,0.2794999885857105681452594581060111522674560546875f,-0.185899990236759293793511460535228252410888671875f,0.276500002242624720594221798819489777088165283203125f,0.0566000031694770255885629239855916239321231842041015625f,0.163699997115135165159927055356092751026153564453125f,0.131399999024718994178329012356698513031005859375f,-0.188399999698996456487520845257677137851715087890625f,0.6205000173747539093938030418939888477325439453125f,-0.092200000721216202048680088410037569701671600341796875f,-0.0494000031560659635943011380732059478759765625f,0.07120000009536742169302669935859739780426025390625f,0.5021999995291233087613136376603506505489349365234375f,0.1064000007569789740813348544179461896419525146484375f,0.23879999856352807086068423814140260219573974609375f,0.2248000000089407368619731641956605017185211181640625f,-0.5566999839425086360478189817513339221477508544921875f,-0.36090000446736814065928911077207885682582855224609375f,-0.161200013804435826614280813373625278472900390625f,0.0469999923855066725764118018560111522674560546875f,-0.60929999164938930533708116854541003704071044921875f,-0.177399996539950366436499962219386361539363861083984375f,-0.1068999986857175343857306870631873607635498046875f,0.1317999950766564065673946970491670072078704833984375f,-0.653999981194734569811544133699499070644378662109375f,0.05590000290274621530528520452207885682582855224609375f,0.20520000371634961044975398181122727692127227783203125f,0.19669999958872796952391581726260483264923095703125f,-0.570699993598461130517307537957094609737396240234375f,0.269499997973442095844376353852567262947559356689453125f,-1.004699988204240668210331932641565799713134765625f,-0.21299998168647327201341568070347420871257781982421875f,-0.5775999984309077017741174131515435874462127685546875f,-0.4749000046610831748949976827134378254413604736328125f,-0.220800003167986869101468982989899814128875732421875f,-0.505799990427494350342385587282478809356689453125f,-1.46129998425841289844129278208129107952117919921875f,-0.0914999890923500203143703402020037174224853515625f,0.01340000305175781392108547152020037174224853515625f,0.32779998393058829009305554791353642940521240234375f,-0.836200018125772270849438427831046283245086669921875f,0.532099949800969085345059284009039402008056640625f,0.1781999953866009178682361380197107791900634765625f,-0.93739999035596799359382202965207397937774658203125f,-0.236499989390372977737797555164434015750885009765625f,-0.78959996321201320057525663287378847599029541015625f,-0.185200021672249359738771090633235871791839599609375f,0.36409998174905755519148442544974386692047119140625f,0.8600000105798244476318359375f,-0.01279999585151703200835981988348066806793212890625f,-0.387300013643503415750046769971959292888641357421875f,0.392700002300738759686282719485461711883544921875f,1.31700000080466228524755933904089033603668212890625f,0.05539997658729622909135059671825729310512542724609375f,0.916900010569393852932762456475757062435150146484375f,-0.034699977624416789012684603221714496612548828125f,-1.03759999682903281126300498726777732372283935546875f,0.4396000037431715412594712688587605953216552734375f,1.27140002698302279071640441543422639369964599609375f,0.53669999370276888495112643795437179505825042724609375f,0.809399989521503382405853699310682713985443115234375f,0.260999992877245012845577321058954112231731414794921875f,-0.319699981275200884045517568665673024952411651611328125f,-0.02149999803304669665493520369636826217174530029296875f,0.61770000915527345508593271006247960031032562255859375f,-0.006600001642107944377357853227294981479644775390625f,0.68010000249147406492511436226777732372283935546875f,0.150299998682737356858751809340901672840118408203125f,-0.23749999701976776123046875f,0.0218000036448240397390918587916530668735504150390625f,-0.3942999928414820942634833045303821563720703125f,-0.29729998935461043885197796043939888477325439453125f,0.04750000208616267372008223901502788066864013671875f,0.2530000061988830761805502334027551114559173583984375f,0.5328000114977360368584413663484156131744384765625f,0.3800000001490115852931239714962430298328399658203125f,0.06580000145435338509969369624741375446319580078125f,-0.246100001403689372381222710828296840190887451171875f,0.094399998015165298426154549815692007541656494140625f,-0.0284000034242868348854926807689480483531951904296875f,0.098900002098083472645839719916693866252899169921875f,-0.045900007000565512527145983767695724964141845703125f,-0.41999999895691875195780085050500929355621337890625f,-0.0205999924480914919922014405528898350894451141357421875f,-0.06239999367147675002343021333217620849609375f,0.148699993687868126102102905861102044582366943359375f,-0.24949999946355827518118530861102044582366943359375f,0.033999996915459629320821477449499070644378662109375f,-0.6559000058829784762792769470252096652984619140625f,0.024899992069602083777368761730031110346317291259765625f,0.62479999948739983750556348240934312343597412109375f,-0.3629999755024915231160775874741375446319580078125f,-0.8601000010013581942303062533028423786163330078125f,0.793900007164477727883422630839049816131591796875f,0.857000020921230287740399944595992565155029296875f,0.2248999847866597345813488573185168206691741943359375f,-0.34029999605640737225797920473269186913967132568359375f,0.391499994456767641537453528144396841526031494140625f,0.3735000107586383677471530972979962825775146484375f,-0.8994999628067024133315499057061970233917236328125f,-0.1456999819755555591882512089796364307403564453125f,0.22679998881816876288297635255730710923671722412109375f,0.569599990220367669024881251971237361431121826171875f,0.455999996304511601152853472740389406681060791015625f,0.9432000153541564291259646779508329927921295166015625f,-0.6135000097155565157436285517178475856781005859375f,-1.29440003303289419278598870732821524143218994140625f,-0.45879998619556427996712955064140260219573974609375f,0.612200014579296070138525465154089033603668212890625f,-0.57689997968673711881137933232821524143218994140625f,-0.4227999928712844468492448868346400558948516845703125f,-0.45919999046176673829222636413760483264923095703125f,0.472100015850365206659944306011311709880828857421875f,-1.220199988293646953252391540445387363433837890625f,-0.35999998211860617658430783194489777088165283203125f,-0.023599980914593032821358065120875835418701171875f,0.7071999910354609397700187400914728641510009765625f,0.428199982571602077996431034989655017852783203125f,0.156199991154670758675138131366111338138580322265625f,0.37519999157190309091447488754056394100189208984375f,0.7293999972701072920244769193232059478759765625f,-0.023800003331899688419071026146411895751953125f,0.52259998505711546812335654976777732372283935546875f,-0.105500007390975980570146930404007434844970703125f,-0.73230001677274714211307582445442676544189453125f,-0.07529999758377670104891876690089702606201171875f,-0.0750999976299702998172591605907655321061611175537109375f,-0.2339000029921531886856200799229554831981658935546875f,0.0578999874651431323258066186099313199520111083984375f,0.2322999926328659370255991234444081783294677734375f,0.531300007504224680587867624126374721527099609375f,-0.0763999988198280355344849112952942959964275360107421875f,-0.118999999709427317640120236319489777088165283203125f,-0.15280000456869602398768392959027551114559173583984375f,-0.231300007057190004644553482648916542530059814453125f,-0.005299991679191595750353371840901672840118408203125f,0.7955000067949296038705142564140260219573974609375f,0.1124000010848045238986969707184471189975738525390625f,0.25159999754428863383282077847979962825775146484375f,0.15250000044703482249275339199812151491641998291015625f,0.3486999978601932337340940648573450744152069091796875f,0.10440000066012145030214242069632746279239654541015625f,0.277200002993643279669555568034411408007144927978515625f,0.192200012791156726876806715154089033603668212890625f,1.01019999485015876672377999057061970233917236328125f,0.129199996012449247473341529257595539093017578125f,-0.302600003385543825373815707280300557613372802734375f,-0.0297999997854232478999847444356419146060943603515625f,-0.442199990439414936105322340154089033603668212890625f,-0.134400001740455632415205400320701301097869873046875f,0.83650002098083486856694435118697583675384521484375f,0.2477000043869015399877753225155174732208251953125f,1.26870003690123578365955836488865315914154052734375f,0.253199991549551362002290488817379809916019439697265625f,0.52890001165345312816867817673482932150363922119140625f,0.513799975839257516696534366928972303867340087890625f,-0.4553999957352872218763195633073337376117706298828125f,0.255299975883961138833910808898508548736572265625f,0.91340000156164169453631984652020037174224853515625f,-0.1015999903917315094759032945148646831512451171875f,1.0154000098168847188162544625811278820037841796875f,-0.7439999837279316352578462101519107818603515625f,-0.23410000767707828117636381648480892181396484375f,0.940699967074394738375531233032234013080596923828125f,0.57979998622536665919824372394941747188568115234375f,-0.36710001760125177128202267340384423732757568359375f,-0.34320000522136684395491101895458996295928955078125f,-0.056999997600913065998184947602567262947559356689453125f,0.3349999998696147063270700527937151491641998291015625f,0.70629998083114653439906760468147695064544677734375f,1.0780999945342539803050385671667754650115966796875f,0.03809999617338188926396469469182193279266357421875f,1.119600046062469989038845596951432526111602783203125f,0.346299999308585793045267564593814313411712646484375f,0.5681000112235545795869029461755417287349700927734375f,-0.5625000056624411826078357989899814128875732421875f,-0.775400027698278648813356994651257991790771484375f,-0.19850002022087587238985406656865961849689483642578125f,0.1424999998137355117222568878787569701671600341796875f,-0.1273999769449236652008039527572691440582275390625f,0.33709999435544002466258461936376988887786865234375f,0.039499995365738893582374657853506505489349365234375f,-0.44299999934434897141954934340901672840118408203125f,-0.25599999880045654077065364617737941443920135498046875f,0.26339998222738503219630956664332188665866851806640625f,-0.11379999848902222492341707038576714694499969482421875f,-0.32059999356567858708189078242867253720760345458984375f,-0.263699992942810101936856881366111338138580322265625f,0.259899986779689839977436349727213382720947265625f,-0.0856000002712011820449333754368126392364501953125f,0.26609998374581333990818166057579219341278076171875f,0.412400001531839421886616037227213382720947265625f,-0.05959997677206985144948703236877918243408203125f,-0.1162999973714351487075191471376456320285797119140625f,-0.8664999944567679523999004231882281601428985595703125f,-0.119700000944733542240783208399079740047454833984375f,0.7220000218152999860166119105997495353221893310546875f,0.02259999988377094093383590234225266613066196441650390625f,-0.00110000000670551489889703589142300188541412353515625f,-0.075099998691678049311803988530300557613372802734375f,-0.709800000679492892885491528431884944438934326171875f,0.0247000008702278339711710941628552973270416259765625f,-0.15569999955892566134707521996460855007171630859375f,-0.09830000368952751532702905024052597582340240478515625f,-0.209100005888938955678213460487313568592071533203125f,0.02290000139772896403655977337621152400970458984375f,0.64580001829266553414754525874741375446319580078125f,-0.157500000447034771422494259240920655429363250732421875f,0.5870000169724225447254184473422355949878692626953125f,0.119399996525049179041388924815692007541656494140625f,0.44580003259778067103979992680251598358154296875f,0.40579999750554573978433836600743234157562255859375f,0.038099996694922477757927481434307992458343505859375f,-0.024700035142898091322649634093977510929107666015625f,0.530599978440999819895296241156756877899169921875f,0.156100019359588060297028278000652790069580078125f,0.8182000079035756101575316279195249080657958984375f,-0.278599984788894250442581324023194611072540283203125f,-0.79090001959204681014625748503021895885467529296875f,0.3598999691963200575628434307873249053955078125f,-0.3228000053882595921095344237983226776123046875f,0.3829999880194663308685676383902318775653839111328125f,0.80810002247393153407273302946123294532299041748046875f,0.87599999719858079316736620967276394367218017578125f,-0.085900013184547763245291207567788660526275634765625f,0.8415999567151073446069631245336495339870452880859375f,0.09709999338686492453831533566699363291263580322265625f,0.9492999724268911254654312870115973055362701416015625f,0.34400000004470354664221076745889149606227874755859375f,0.82949998836219307474237893984536640346050262451171875f,0.7787000049754977393234867122373543679714202880859375f,-0.11170000815689551387777100899256765842437744140625f,-0.616900008222460716211799081065692007541656494140625f,0.412399990952014405820591491647064685821533203125f,0.085100011432170941105823658290319144725799560546875f,0.31009998937845228983434253677842207252979278564453125f,0.9279000088855624550632228419999592006206512451171875f,-0.02030000270605114565114490687847137451171875f,1.18940005270242732393626283737830817699432373046875f,-0.723799999904631619074280024506151676177978515625f,0.544199999290704727883394298260100185871124267578125f,0.01779999980032442863375763408839702606201171875f,-0.51920000323951243093034690900822170078754425048828125f,-0.2140999935209750848486009999760426580905914306640625f,0.4937000117182732683573931353748776018619537353515625f,0.1314999919235706471454250277020037174224853515625f,-0.524299998652935084209048000047914683818817138671875f,0.2806000027298927168573072776780463755130767822265625f,-0.2771999962508677750605556866503320634365081787109375f,-0.22589999411106109761249172152020037174224853515625f,0.110499999344348953655980949406512081623077392578125f,-0.134600000688433640849694938879110850393772125244140625f,-0.18070000365674498965518068871460855007171630859375f,-0.2036000119090080051620361700770445168018341064453125f,-0.55820000119805346372459098347462713718414306640625f,-0.333199996280670152071223810708033852279186248779296875f,-0.229799998816847761418102891184389591217041015625f,-0.339300002676248546595871857789461500942707061767578125f,-0.35430000573098663352311632479541003704071044921875f,-0.098300002348423010101186036990839056670665740966796875f,-0.7868000055819750482299923532991670072078704833984375f,-0.1858999958992004764013472595252096652984619140625f,0.58699998955428600577732822785037569701671600341796875f,-0.2416000049948692218748647064785473048686981201171875f,0.064999995380639941089384592487476766109466552734375f,-0.1156999999314546612172449613353819586336612701416015625f,-0.144200003574788571114595470135100185871124267578125f,-0.200899999773502369038169490522705018520355224609375f,0.426299984705448054000953561626374721527099609375f,0.313400009831786174885337459272705018520355224609375f,0.70309998760521408289747569142491556704044342041015625f,-0.38590001944303509162637055851519107818603515625f,0.6397000101089478363292073481716215610504150390625f,-0.47069999568462339567531671491451561450958251953125f,0.13720000966191303604091444867663085460662841796875f,-0.41159995760917755802665851661004126071929931640625f,-0.26639998332262049274277160293422639369964599609375f,0.46059995534420039575707050971686840057373046875f,-0.27560003595948234789148045820184051990509033203125f,-0.4972999774336817058184578854707069694995880126953125f,-0.90279998847544173923296284556272439658641815185546875f,0.590799984467029304369134479202330112457275390625f,0.23910001125335700322693810448981821537017822265625f,0.03679998643398274538185432902537286281585693359375f,0.0782999943017959054714083322323858737945556640625f,-0.38400001293420782388920997618697583675384521484375f,0.30960000731945047647286628489382565021514892578125f,-0.2129000081591307935724444178049452602863311767578125f,-0.17959999865815035047234005105565302073955535888671875f,1.105899973845481554235448129475116729736328125f,0.16010001456141509379449416883289813995361328125f,-0.7222999830029908086004297729232348501682281494140625f,0.46749999711289957904369885000050999224185943603515625f,-0.3562999935530124506755100810551084578037261962890625f,0.48960001918450013480565985446446575224399566650390625f,0.07389997147619720152533773216418921947479248046875f,-0.579600010374188290285246694111265242099761962890625f,-0.5215999848783017256437233299948275089263916015625f,0.49730001289844505407700125942938029766082763671875f,0.4329000102639195546316841500811278820037841796875f,-0.035299995777010817477048476575873792171478271484375f,-0.1332999977290629800563692697323858737945556640625f,0.727500014454126375795794956502504646778106689453125f,0.04790000557005402725962994736619293689727783203125f,0.43639999449849131707424021442420780658721923828125f,0.37799998682737356858751809340901672840118408203125f,-0.369599984073639031834090928896330296993255615234375f,-0.340599996992945630580607030424289405345916748046875f,0.25270001317858703515639717807061970233917236328125f,0.177699997755885130690245432560914196074008941650390625f,0.2208000018268823083644747384823858737945556640625f,-0.442399999445676872511512556229718029499053955078125f,0.30799997177720062069283812888897955417633056640625f,-0.110800001007318460466422038734890520572662353515625f,0.2068000095307827024271318805404007434844970703125f,-0.07999999731779094958028508699499070644378662109375f,0.61920001016855241715575175476260483264923095703125f,-0.08509999848678706513283032109029591083526611328125f,0.453700004137307377760635063168592751026153564453125f,-0.197900006911158499178782221861183643341064453125f,-0.8531999909162522044425713829696178436279296875f,0.002299999692290999320221089874394237995147705078125f,0.82029999505057926700857251489651389420032501220703125f,0.08550000275298952345792713458649814128875732421875f,0.1987999982096254891761333283284329809248447418212890625f,-0.227100000017881331615399176371283829212188720703125f,0.46160001624524593655252147073042578995227813720703125f,0.1132000000804662676046064007095992565155029296875f,0.3290999997794628040281850189785473048686981201171875f,-0.01410000328123572899130522273480892181396484375f,-0.4667000222384931618080372572876513004302978515625f,0.21139999285936372785954517894424498081207275390625f,0.855999999135732547728139252285473048686981201171875f,0.1906000058591370560634459252469241619110107421875f,0.7302000204801561977774326805956661701202392578125f,0.3724999868869787178482511080801486968994140625f,-0.9517000110626216979881064617075026035308837890625f,0.369600009405613050272876307644764892756938934326171875f,0.09070000069513917129171431952272541821002960205078125f,-0.5012999683141714513112674467265605926513671875f,-0.9067000012278558784828419447876513004302978515625f,0.10689999041557296521887110429815948009490966796875f,0.1659000075221062076025191345252096652984619140625f,-0.1754000020682813243411146686412394046783447265625f,0.447200008916854852003552878159098327159881591796875f,-0.09650000220537169326462390017695724964141845703125f,-1.040000034719705812591428184532560408115386962890625f,0.17229998160600690226829101447947323322296142578125f,1.115000035613775697385108287562616169452667236328125f,0.15699996966123563879591529257595539093017578125f,-0.369200012999773008459669654257595539093017578125f,-0.7944999812841420094855493516661226749420166015625f,-0.78029999257326121409050756483338773250579833984375f,-0.774499978303909930588133647688664495944976806640625f,-0.4661999783396719720940382103435695171356201171875f,0.1674999919719997176770220903563313186168670654296875f,-0.71910000393316131361842735714162699878215789794921875f,0.258099983060360305131553104729391634464263916015625f,0.4506999930024147005269696819595992565155029296875f,0.2459999926537275172222507535479962825775146484375f,-0.58449999856948853249605235760100185871124267578125f,0.0174000008612871237545505209709517657756805419921875f,-0.329699997144937473336767652654089033603668212890625f,0.128600004532933187562093735323287546634674072265625f,-0.5539000131249427472113211479154415428638458251953125f,0.11710000284910204726429583388380706310272216796875f,-0.65949998858571046156384909409098327159881591796875f,0.0939999995976686097964858390696463175117969512939453125f,-0.65380001063346859790925691413576714694499969482421875f,0.09649999460577962651086636469699442386627197265625f,0.4116999930322169376495367032475769519805908203125f,-0.244499999165534998013527001603506505489349365234375f,0.426499988645315131208235470694489777088165283203125f,-0.0166000011950730996801439687260426580905914306640625f,0.2223000066399574148334750134381465613842010498046875f,0.0297000012427568460537941064103506505489349365234375f,0.09580000637173657906942025874741375446319580078125f,-0.1598999953478574820309177084709517657756805419921875f,0.10979999307990073731389202293939888477325439453125f,-0.27700000132620328230359518784098327159881591796875f,0.468100015991926188263505537179298698902130126953125f,0.157699995297193584775641284068115055561065673828125f,0.6097000165164470075751523836515843868255615234375f,0.1100999943703413475049046610365621745586395263671875f,0.6800000102818011260552566454862244427204132080078125f,-0.18909999421015388776368126855231821537017822265625f,0.271399979615956521428188352729193866252899169921875f,-0.0508999968677759095925239307689480483531951904296875f,-0.2083999961972236025786742175114341080188751220703125f,0.0203999979704618805698146388749592006206512451171875f,0.60070001684427276611444312948151491582393646240234375f,0.7101999724984171535879795555956661701202392578125f,0.200099973732232871981295829755254089832305908203125f,-0.01100001060962707555290762684307992458343505859375f,-0.2036000050544739536917404620908200740814208984375f,-0.591299973082542340563350080628879368305206298828125f,-0.1221000225186352761141961309476755559444427490234375f,0.13459998213648827203314795042388141155242919921875f,-0.429799988758563511481725072371773421764373779296875f,-0.191899996860325050551665526654687710106372833251953125f,-0.46140001455917956096897114548482932150363922119140625f,0.00239999326169482163351176495780237019062042236328125f,0.1112000026658177542770999934873543679714202880859375f,-0.6836999950103461731742982010473497211933135986328125f,0.42930001580789689530348596235853619873523712158203125f,-0.54919999377727446443486769567243754863739013671875f,-0.56260001932978642713578665279783308506011962890625f,-0.39559997262954738062035175971686840057373046875f,-0.05439998087882980115637110429815948009490966796875f,-0.2940999920867384265221744499285705387592315673828125f,0.42030000682249657462108416439150460064411163330078125f,-0.560199976372719010697664998588152229785919189453125f,-0.7086999854922293451409132103435695171356201171875f,-0.058800010931491897281375713646411895751953125f,0.451499995648860785735223544179461896419525146484375f,-0.4952999910831454855042466078884899616241455078125f,0.46120000650286652188469815882854163646697998046875f,0.10499998807907129361183251603506505489349365234375f,-0.582699991869926048337902102502994239330291748046875f,-0.12240001464486116589114317321218550205230712890625f,0.02350000166893007058632747430237941443920135498046875f,-0.1673999989241361507907868144684471189975738525390625f,-0.460299995553493435096470420830883085727691650390625f,-0.189800002020597435858917378936894237995147705078125f,0.457600003683567013723632044275291264057159423828125f,0.27770000073611733615308594380621798336505889892578125f,0.0642000001847744528049588552676141262054443359375f,-0.0649999969452619286158778777462430298328399658203125f,0.0117000062942504978735769327613525092601776123046875f,0.2059000010773539834918466340241138823330402374267578125f,-0.2238999860420822496376302979115280322730541229248046875f,-0.093600000211596479093856260078609921038150787353515625f,0.206200001212954509011154868858284316956996917724609375f,0.023700002535432584949148804298602044582366943359375f,0.727800002575665683224315216648392379283905029296875f,0.05160000953972332382591048371978104114532470703125f,0.554099986964464275018826810992322862148284912109375f,0.400399993872642578907061761128716170787811279296875f,-0.44999998152256015515604303800500929355621337890625f,0.0748999995760619707851901694084517657756805419921875f,0.1534000023998320083595814367072307504713535308837890625f,0.0262000009149312429457268081023357808589935302734375f,0.6322999830961226752634729564306326210498809814453125f,-0.1095999995708465402088904738775454461574554443359375f,0.298200004774332061430186513462103903293609619140625f,-0.045100001224875452265905551030300557613372802734375f,0.662599998170137371999999231775291264057159423828125f,-0.0701999970853328658648706550593487918376922607421875f,0.065800000113248824362699451739899814128875732421875f,-0.1287999982655048114565943251363933086395263671875f,1.322500036358833508387533584027551114559173583984375f,-0.15509998729228957614623141125775873661041259765625f,-0.998900015956163400687728426419198513031005859375f,-0.3387999912619594766738373436965048313140869140625f,-1.377499983161687424626506981439888477325439453125f,-0.6544999999664720125025496599846519529819488525390625f,0.49400003150478044577909031431772746145725250244140625f,0.6340999629735950549047629465349018573760986328125f,0.1262999709963796846068362356163561344146728515625f,0.64549997870624131746097873474354855716228485107421875f,0.6270999847814440197879548577475361526012420654296875f,-0.076199952709675500273078796453773975372314453125f,-1.24750001043081315543759046704508364200592041015625f,0.2479000230789178971235742210410535335540771484375f,-1.01570002831816719179869323852472007274627685546875f,-0.237700000512599984858752577565610408782958984375f,0.2587999973714349977171877981163561344146728515625f,-0.1420000000596051137335962266661226749420166015625f,0.86289999983310661235691441106610000133514404296875f,-0.43019998412132309795197215862572193145751953125f,-0.59579998789429655658977935672737658023834228515625f,0.4192999925434595098749923636205494403839111328125f,-0.689699998337029995809643878601491451263427734375f,-0.544699991035461295041386620141565799713134765625f,-1.40929998732805206174134582397527992725372314453125f,0.111999999091029422970677842386066913604736328125f,0.34220001152455825632614505593664944171905517578125f,-0.8380999812722205177806245046667754650115966796875f,0.22720000787377347961637497064657509326934814453125f,-0.03530003317892538294842097457149066030979156494140625f,0.1957999977290629800563692697323858737945556640625f,0.242299999412894262462714323191903531551361083984375f,-0.324399991011619537317756112315692007541656494140625f,0.01679999604523185752213976229541003704071044921875f,0.426300007057189844772437936626374721527099609375f,0.02530000708326696212679962627589702606201171875f,0.68669999670162795712968772932072170078754425048828125f,-0.333299995046854036218064720742404460906982421875f,0.01269999619126327417006905307061970233917236328125f,-0.082299993824958839194749771195347420871257781982421875f,-0.6000000078231095823610985462437383830547332763671875f,0.194199992287158895720722284750081598758697509765625f,-0.29480002049803744057498988695442676544189453125f,0.043299992737173997880972819984890520572662353515625f,-0.34100001001358037910904386080801486968994140625f,-0.08669999832212926804686503601260483264923095703125f,0.46519999499917030760798297706060111522674560546875f,-0.145600003176927561554521162179298698902130126953125f,0.45839999873042103217812837101519107818603515625f,0.051799995747208615881618243292905390262603759765625f,0.41550000336766235164986937888897955417633056640625f,0.004700005340576129952978590154089033603668212890625f,-0.4697000110030173924968721621553413569927215576171875f,0.035900003498792631972946765017695724964141845703125f,0.62589999545216556953164399601519107818603515625f,0.0600000021606683642261259592487476766109466552734375f,-0.28560000250637529450870033542742021381855010986328125f,0.147300003436207749274444722686894237995147705078125f,0.635799997133016603356736595742404460906982421875f,-0.253499997124075837628964791292673908174037933349609375f,1.0493000403761865158003274700604379177093505859375f,0.12039999804496748225801638909615576267242431640625f,-0.554900011366605827589637556229718029499053955078125f,-0.5929000141382214650320747750811278820037841796875f,0.44530001835227039919118396937847137451171875f,0.7291999842405321174965138197876513004302978515625f,1.17880001465678230232470014016143977642059326171875f,0.66049998384714125876371326739899814128875732421875f,-0.469900003319978498694808877189643681049346923828125f,-0.1764000142425299344495215336792171001434326171875f,-0.243200006935000434538096669712103903293609619140625f,-0.04309999319314972154870702070184051990509033203125f,-0.8166000091671943295068558654747903347015380859375f,0.74029997916221645937184803187847137451171875f,0.17989998767375947608115893672220408916473388671875f,0.235799998325109783081643399782478809356689453125f,-0.55009998714327767999066054471768438816070556640625f,-0.73580000093281194839534009588533081114292144775390625f,-0.6661000205889344716325695117120631039142608642578125f,-0.005000032298266621211269011837430298328399658203125f,-0.6956999998196955470319835512782447040081024169921875f,-0.21019999246597276254533426254056394100189208984375f,0.088900008207559721995494328439235687255859375f,-0.0035000031776728501853312991443090140819549560546875f,-0.64970002875402610431621042152983136475086212158203125f,0.1347999771356589437942830045358277857303619384765625f,-0.85000000901520211815665106769301928579807281494140625f,0.62109998985528935833144714706577360630035400390625f,0.89280000911355017478854279033839702606201171875f,-0.70809996517896678369652363471686840057373046875f,0.43499999597668637107972244848497211933135986328125f,-0.0576000021189451150149807290290482342243194580078125f,0.017500001490116101621197230997495353221893310546875f,-0.05749999456107612871846868074499070644378662109375f,0.80640001044273379449123240192420780658721923828125f,-0.069199999663233757729585704510100185871124267578125f,-0.75510000353455541954872387577779591083526611328125f,-0.1213000019162893128310543033876456320285797119140625f,-0.474999998211860674501849644002504646778106689453125f,-0.1451999993950128242659047828055918216705322265625f,0.55160000864565372769021678323042578995227813720703125f,0.169099995607137731923330647987313568592071533203125f,-0.297099983477592477498774314881302416324615478515625f,-0.121899998760223393645674150320701301097869873046875f,-0.59890000165104861906684163841418921947479248046875f,0.0206000027298927079755230806767940521240234375f,-0.2974000076413154669552341147209517657756805419921875f,0.057000010937452272319347912343800999224185943603515625f,0.765899996420741135949583622277714312076568603515625f,-0.249799995575845201756237656809389591217041015625f,0.6119000099733471476071144934394396841526031494140625f,0.210599999824166328465935293934307992458343505859375f,-0.2398999891638755599387877737171947956085205078125f,-0.240499999698251532009862785344012081623077392578125f,0.586599987243860976349196789669804275035858154296875f,-0.022799993467330914587165580087457783520221710205078125f,-0.06479999136626712896713797817938029766082763671875f,-0.142300001648068441539862760691903531551361083984375f,-0.446800008338689824682887774542905390262603759765625f,0.342099996441602716146235252381302416324615478515625f,0.3874000159859658509731161757372319698333740234375f,-0.78419996330440111531601132810465060174465179443359375f,-0.6608999776825308369865297208889387547969818115234375f,0.3109999885559082688502030578092671930789947509765625f,-0.76170000547468641460824301248067058622837066650390625f,-0.99920000271797082547209356562234461307525634765625f,0.023400010353327171941373308072797954082489013671875f,-0.2467999980568880147302479599602520465850830078125f,-1.08710002043247211389598305686376988887786865234375f,0.983499982893466295763573725707828998565673828125f,0.3764000079095364270642676274292171001434326171875f,0.161900004720687551440505558275617659091949462890625f,-0.9131000378966336317176910597481764853000640869140625f,-0.94959996231794374654811008440447039902210235595703125f,-0.125399986682832309536905768254655413329601287841796875f,0.4963999673783778820990164604154415428638458251953125f,0.674899997583031652226281948969699442386627197265625f,1.253299993705748960337587050162255764007568359375f,0.37710001372694978982735847239382565021514892578125f,-0.401999992311001363987088552676141262054443359375f,-0.64920000748634354437172078178264200687408447265625f,0.1785999955177306475206933100707828998565673828125f,-0.109100016319751969007256775512360036373138427734375f,0.64570001118183117938542636693455278873443603515625f,0.8297000226259232391612385981716215610504150390625f,0.6710999630331997423127177171409130096435546875f,0.203800010037422651976157794706523418426513671875f,-0.14330002082586279499309966922737658023834228515625f,0.29010000174641614734127870178781449794769287109375f,-0.62049998250603710658879208494909107685089111328125f,0.68879998291730881732064517564140260219573974609375f,0.127799994957447038412823303588083945214748382568359375f,0.660299992945790226173130577080883085727691650390625f,-0.088200000005960454974029971708660013973712921142578125f,-0.126199998381733902164114624611102044582366943359375f,0.2360000121742487078080330320517532527446746826171875f,0.63170001210570347893735743127763271331787109375f,0.226400004333257687250124945421703159809112548828125f,-0.175999988704919729798348271287977695465087890625f,-0.18950001026689999861218893784098327159881591796875f,-0.73919999064803132426959564327262341976165771484375f,-0.07150000362098218875672728245262987911701202392578125f,0.01799999681115149741117420489899814128875732421875f,0.34079999877214428583016569973551668226718902587890625f,0.3406000020965933572369976900517940521240234375f,-0.28800000023841854090989045289461500942707061767578125f,0.04400000727176667236761886670137755572795867919921875f,-0.212500008270144480349017612752504646778106689453125f,-1.003400015270710099457573960535228252410888671875f,0.013199994343519261263963926467113196849822998046875f,0.473400013780593820200692789512686431407928466796875f,-0.1054000002801418478526329636224545538425445556640625f,0.03699999544024468178804454510100185871124267578125f,0.012699996787309597578996545053087174892425537109375f,-0.827700019437074718808844409068115055561065673828125f,-0.246699995714426023596388404257595539093017578125f,-0.23289999864101407212046979111619293689727783203125f,-0.20149999691545961155725308344699442386627197265625f,0.615800005030632036095994408242404460906982421875f,0.1288999969571829051773192986729554831981658935546875f,0.405700015801191415931725714472122490406036376953125f,0.697899994170665394221941824071109294891357421875f,-0.10080001814365413537188942427746951580047607421875f,-0.60549998407065863670339922464336268603801727294921875f,-0.7171000050470233144750409337575547397136688232421875f,0.371899962401389672095319838263094425201416015625f,0.398699996072053863827022723853588104248046875f,-0.4126000036835666406886957702226936817169189453125f,-1.0185000416040423942831694148480892181396484375f,-0.666299998116493430444506884668953716754913330078125f,-0.7052000036418437645124868140555918216705322265625f,-0.3945999602317815657670507789589464664459228515625f,-0.094799995017051941204044851474463939666748046875f,0.027399992144107709890477053704671561717987060546875f,0.440700012224912729408288214472122490406036376953125f,0.0465999954581253206242763553746044635772705078125f,1.03990001345276805722050994518212974071502685546875f,0.2370999658942221088864243938587605953216552734375f,0.1629999862313269431979279033839702606201171875f,0.32539999245703210561941887135617434978485107421875f,-0.334400012096762733193600070080719888210296630859375f,-0.31639998980462546995795491966418921947479248046875f,0.189600002327561245607512319111265242099761962890625f,-0.298599988365173540927344220108352601528167724609375f,-1.3210000162720680894068436828092671930789947509765625f,0.58349998646974565730261019780300557613372802734375f,-0.088500023871660449259479719330556690692901611328125f,-0.084000005297362623224444178049452602863311767578125f,0.6572000268355016050492167778429575264453887939453125f,0.7138999883890149167342542568803764879703521728515625f,-0.085299966645241198648363933898508548736572265625f,0.2801999955952165777262052870355546474456787109375f,0.587500009536743306171047152020037174224853515625f,0.340799992644041704892288180417381227016448974609375f,-0.9118999695166947194735485027194954454898834228515625f,-1.1922000598788269343231149832718074321746826171875f,0.40140001103878120147783192805945873260498046875f,0.03299999506026463080843313946388661861419677734375f,0.336100024649500728202156096813268959522247314453125f,-0.1772999944210056355586857534945011138916015625f,1.0223999781370167738714371807873249053955078125f,0.5511000281512739018552338166045956313610076904296875f,0.580900006777047561712379319942556321620941162109375f,-0.72660001944899565984314904198981821537017822265625f,-0.0992999947786330494636786170303821563720703125f,0.69780000255703900080561652430333197116851806640625f,1.281499935746193585828223149292171001434326171875f,-0.090899991875886598791112191975116729736328125f,-1.313299987745285335449807462282478809356689453125f,0.55950000169873259636688089813105762004852294921875f,-0.4548999811172489415866948547773063182830810546875f,-0.204299986061453608954252558760344982147216796875f,0.47009998684525466927652814774774014949798583984375f,0.33149999583512534950813233081134967505931854248046875f,-0.11510000815391518980135288074961863458156585693359375f,0.80100004000216740163153872344992123544216156005859375f,1.200499984443187173610567697323858737945556640625f,0.9033000270843505319362520822323858737945556640625f,0.751399953818321364451549015939235687255859375f,-0.858900004482268997207938809879124164581298828125f,-0.69129999483823778572144647114328108727931976318359375f,-0.25809999878108502713303096243180334568023681640625f,0.0057000018313526634727850250783376395702362060546875f,-0.231600006484985243560004164464771747589111328125f,0.8848000113710761116436742668156512081623077392578125f,0.3004999991953372084907414318877272307872772216796875f,-0.199800012078881206178948559681884944438934326171875f,0.073699996817111923519405536353588104248046875f,-1.115900010390579399910393476602621376514434814453125f,0.0343999996542929675769073583069257438182830810546875f,-0.7030000104829667861849884502589702606201171875f,0.0290999853253364193506058654747903347015380859375f,-0.2663000194624063343695752337225712835788726806640625f,-0.112699993062019387934924452565610408782958984375f,0.79019998695254312082170145004056394100189208984375f,0.4423000009775162055802866234444081783294677734375f,-0.087200012381374758518859380274079740047454833984375f,0.157599999213218655569335169275291264057159423828125f,-0.3886000026702880472129209010745398700237274169921875f,0.449499993801116826119823599583469331264495849609375f,-0.714500019617378523406614476698450744152069091796875f,-0.25269999663829789682267801254056394100189208984375f,0.4290000229552386468157010313007049262523651123046875f,0.06919999457821257582423868370824493467807769775390625f,0.697299987193942261143320138216949999332427978515625f,-0.225699997174739763750039855949580669403076171875f,-0.535699979330599429516723830602131783962249755859375f,-0.2410999955177306475206933100707828998565673828125f,0.12030002278536533477648617918021045625209808349609375f,0.052900001323223033722342734108678996562957763671875f,-0.51030001358389842636142930132336914539337158203125f,0.645600001239776677408599425689317286014556884765625f,0.609099986815452520971803096472285687923431396484375f,-0.00150001293420842785053537227213382720947265625f,1.438800017189979740805938490666449069976806640625f,-0.59519998330175838407996025125612504780292510986328125f,-0.864999988153577259453186343307606875896453857421875f,0.283800019723177143760040053166449069976806640625f,-1.674900023436546092625576420687139034271240234375f,0.52830002507269424594227302804938517510890960693359375f,-0.584600022891164083915782612166367471218109130859375f,0.84900004795193684259402289171703159809112548828125f,0.696299958181381128952125436626374721527099609375f,0.2963999946475031066484007169492542743682861328125f,1.80579996926784502875307225622236728668212890625f,-0.47369999979734433992462072637863457202911376953125f,-0.3164000265359874219939229078590869903564453125f,-0.1901999908268450223403078780393116176128387451171875f,-0.83219999505877506607021132367663085460662841796875f,-0.3959999602437014321054675747291184961795806884765625f,-1.41070000179410026674986511352472007274627685546875f,0.155099964940547341285537186195142567157745361328125f,0.7522000104069714865318019292317330837249755859375f,-0.398400006162375586171719987760297954082489013671875f,0.8726999822959309316416920410119928419589996337890625f,0.6587999848544594527766093960963189601898193359375f,-0.37990000317096672688421676866710186004638671875f,0.2339999764263627213267682236619293689727783203125f,0.279300020408630444990194519050419330596923828125f,-0.5306999955356117570914875614107586443424224853515625f,0.2599999885261059517915782635100185871124267578125f,-0.150799994003772752648728783242404460906982421875f,0.8900999854668973920723828996415250003337860107421875f,-0.055800002646446245080369408242404460906982421875f,-0.084499994136393308252763745258562266826629638671875f,0.2283000077128409799342989572323858737945556640625f,-1.05790001860856985871350843808613717555999755859375f,0.0032000025391578645894696819595992565155029296875f,-0.2161000149264931113890497726970352232456207275390625f,-0.35289999864101417870188015513122081756591796875f,0.79159998539984233190125451073981821537017822265625f,0.4934000006675720584325972595252096652984619140625f,-0.0401999942913653374176874422118999063968658447265625f,-0.0573999992966651717551940237171947956085205078125f,-0.16270000140666962540336726306122727692127227783203125f,0.5773999987006188661098349257372319698333740234375f,-0.3782999847650529989806500452687032520771026611328125f,0.3350999914646148880592591012828052043914794921875f,0.798999990805983717478966354974545538425445556640625f,0.00319999352395539116145073421648703515529632568359375f,-0.371399987085163640898599624051712453365325927734375f,0.12529999516233802925313511877902783453464508056640625f,-0.82659999344646928332025481722666881978511810302734375f,-0.363399992322921860932183335535228252410888671875f,0.363199983726441910647508848342113196849822998046875f,-0.19159999516010284281719577847979962825775146484375f,0.4168999872490763980437122881994582712650299072265625f,0.2306999954052268986259832672658376395702362060546875f,-0.090699999707937184467709812452085316181182861328125f,0.4356000018358230363446637056767940521240234375f,0.649599977889657065333040009136311709880828857421875f,0.42479999337792406777225551195442676544189453125f,2.132699959087371865962268202565610408782958984375f,0.9294000206649304374195708078332245349884033203125f,1.0608999851703639905053933034650981426239013671875f,-0.07350000002980205948688308126293122768402099609375f,-1.880500043749808725124239572323858737945556640625f,0.85500000879168513989725397550500929355621337890625f,0.771699934172630719331209547817707061767578125f,0.219600002551079143131573800928890705108642578125f,1.425999992728232967920121154747903347015380859375f,-0.69450000154972091337413075962103903293609619140625f,-0.472999984741210965921709430404007434844970703125f,0.5739000022470950312936111004091799259185791015625f,0.4154999704360964329907801584340631961822509765625f,-0.46890000582337432177837399649433791637420654296875f,-0.71609999551773029224932543002068996429443359375f,-1.30979999591112150625349386245943605899810791015625f,0.11140003547668442251961096189916133880615234375f,0.400800010953843377503602596334530971944332122802734375f,-0.1573999930754303999691501303459517657756805419921875f,0.497700019008666227904313927865587174892425537109375f,-0.7610000088587403066497927284217439591884613037109375f,0.3524999967217450347334306570701301097869873046875f,0.7715999796628949525256757624447345733642578125f,0.566399993008375179925906195421703159809112548828125f,1.369099953138828595911036245524883270263671875f,-0.87989998935013991054887583231902681291103363037109375f,-1.129299954545498252400648198090493679046630859375f,-0.43079999161958681952455663122236728668212890625f,0.0653000145152209388044184379396028816699981689453125f,-0.29059999587535845222419084166176617145538330078125f,-0.7767999692976477188466333245742134749889373779296875f,0.33730000514984137538476716144941747188568115234375f,0.0128000010669231389925926123396493494510650634765625f,0.16440000054836279019809808232821524143218994140625f,-0.0649999926239253067450363232637755572795867919921875f,-0.3223999998927116195091002737171947956085205078125f,0.269300002005696370588339050300419330596923828125f,0.1818999978661537397783831693232059478759765625f,-0.028999998331070031554190791212022304534912109375f,0.0491999997377394748809820157475769519805908203125f,-0.39240000294744969977500659297220408916473388671875f,0.069400002261996274199873369070701301097869873046875f,0.50859999969005575959357656756765209138393402099609375f,0.08490000108480444396263919770717620849609375f,-0.2383000091284515065126470290124416351318359375f,-0.12800000193342564447362974533461965620517730712890625f,-0.977999980494379972384422217146493494510650634765625f,-0.1444999943971634337458453956060111522674560546875f,0.40869999797195200041954876724048517644405364990234375f,0.20240000525712975587566688773222267627716064453125f,0.8887000160768625534046805114485323429107666015625f,0.28769999857544892307714690105058252811431884765625f,-0.494600014211237393180198296249727718532085418701171875f,-0.03829999608993528426026387023739516735076904296875f,0.68909999739527705298058890548418276011943817138671875f,-0.2528999990880489434630362666212022304534912109375f,-0.579899976162612684760233605629764497280120849609375f,-0.11550000143051164513963158242404460906982421875f,-0.7256999634981156077628838829696178436279296875f,1.245599980080127178183602154604159295558929443359375f,-0.1064000202774995074150865548290312290191650390625f,-0.874200023281574534195215164800174534320831298828125f,-0.7991999964594835859088561846874654293060302734375f,-0.109200006890297185435656501795165240764617919921875f,-0.0878000003218648572556048748083412647247314453125f,0.28289999014735212057303215260617434978485107421875f,0.54109999790191665169913903810083866119384765625f,0.058800007355213068649391061626374721527099609375f,0.2345999942064285193055184208787977695465087890625f,-0.02819998376369436954291813890449702739715576171875f,-0.73959998705387164363855845294892787933349609375f,-0.319800007459521307140448698191903531551361083984375f,0.61780000792145717269931992632336914539337158203125f,0.6711000263631348161652567796409130096435546875f,0.56789997301101635684972279705107212066650390625f,0.547500004153698593967192209674976766109466552734375f,-1.2751999864682554974848471829318441450595855712890625f,-0.691200019019841960243866196833550930023193359375f,-0.33549997043609636193650658242404460906982421875f,-1.238400036281347649236295183072797954082489013671875f,0.2309000498414039981298628845252096652984619140625f,0.17850002595782310521599356434307992458343505859375f,-0.3762000266194345243775387643836438655853271484375f,-0.695700022357702341224694464472122490406036376953125f,0.3213000156998635503668992896564304828643798828125f,0.65760000144839292346432557678781449794769287109375f,-0.49809997441768683046348087373189628124237060546875f,0.0570000286698348901381905307061970233917236328125f,0.14180000181943175352472508166101761162281036376953125f,0.00089999897256498595510265658958815038204193115234375f,0.78679999217092999597156222080229781568050384521484375f,-0.028000000648200573660773216033703647553920745849609375f,-0.29190001028999679189013249924755655229091644287109375f,-0.1643999939918519004322661203332245349884033203125f,0.4939999966546892995467032960732467472553253173828125f,0.201999996855855012967140282853506505489349365234375f,0.10389999963939178240224237015354447066783905029296875f,-0.0690999993324279415674027404747903347015380859375f,0.5012999981909989077877298768726177513599395751953125f,-0.03710000651851286246341032892814837396144866943359375f,-0.67749997921288007685092225074185989797115325927734375f,0.10250000387430180381898026098497211933135986328125f,-0.8677000095278024627276636238093487918376922607421875f,-0.201600012743472944976019789464771747589111328125f,-0.8468999677658082259057437113369815051555633544921875f,-0.2973000015735625112256457214243710041046142578125f,-0.761799974177777894368546185432933270931243896484375f,-0.113199992704391405595742980949580669403076171875f,0.0314000012785198290288235511980019509792327880859375f,-0.2182000004529953685050713829696178436279296875f,-1.081900004608928878013784924405626952648162841796875f,-0.048800005567073867496219463646411895751953125f,0.6626999690338972737180256444844417273998260498046875f,0.13399999609589574589563198969699442386627197265625f,-0.459200009870529057831589625493506900966167449951171875f,-0.067400004139542613046387486974708735942840576171875f,-0.60780001485049683918049368003266863524913787841796875f,-0.0586000056505202593370995600707828998565673828125f,-0.3533999814450741183691206970252096652984619140625f,-0.4455000130720438988873866037465631961822509765625f,0.9015999954953792805412149391486309468746185302734375f,0.41940001688376093991195148191764019429683685302734375f,-0.01410002144202604501543873993796296417713165283203125f,-0.78630001331567722644422246958129107952117919921875f,-0.3255999571323400232358835637569427490234375f,0.089100010135769736052679945714771747589111328125f,0.55609999030232426520115041057579219341278076171875f,0.53549999889731392244129892787896096706390380859375f,1.6298999709844590455531943007372319698333740234375f,0.373799999774246938244459670386277139186859130859375f,-0.8316999902382489207042226553312502801418304443359375f,0.341700018662214777265262455330230295658111572265625f,-1.0827000195860865261465733055956661701202392578125f,1.265700019079446292380453087389469146728515625f,-0.11149999058246606864486238919198513031005859375f,1.28669999437332105429732109769247472286224365234375f,-0.028200018334388232688070274889469146728515625f,-0.59880003954172167368597001768648624420166015625f,-0.89939999995231545426577213220298290252685546875f,0.93600001880526573216911856434307992458343505859375f,-1.04090002808570858405801118351519107818603515625f,0.0462000172317029722535153268836438655853271484375f,1.286499961376190270812003291212022304534912109375f,0.37920002104640015971881439327262341976165771484375f,-0.2064000286221503444039626629091799259185791015625f,0.23400001420080673852908148546703159809112548828125f,-0.92580002544522255192305237869732081890106201171875f,-0.95790001652240786000902517116628587245941162109375f,-0.065900006255507292962647625245153903961181640625f,-0.42210000731945029883718234486877918243408203125f,-0.942599984610080543490084892255254089832305908203125f,-0.226999995887279482076337444595992565155029296875f,-0.6757999795123936781493512171437032520771026611328125f,0.27189999682307242068191044381819665431976318359375f,-0.0809000009283422638617366828839294612407684326171875f,0.1078999987900257639950041266274638473987579345703125f,-0.5357999902784824630685989177436567842960357666015625f,-0.00770000133216386384304996681748889386653900146484375f,-0.80030000739991657976446504108025692403316497802734375f,-0.352599995823204448441856584395281970500946044921875f,-0.27329999363124368727540058898739516735076904296875f,0.02089999500513073371621430851519107818603515625f,-0.455300004009902448132152130710892379283905029296875f,-0.0066999991416931692356229177676141262054443359375f,-0.11770000244975076242326395004056394100189208984375f,0.1573000084280966603955675964243710041046142578125f,1.1578999825477598317746696920949034392833709716796875f,-0.32910001021027568413046537898480892181396484375f,-0.5348999846190214402241736024734564125537872314453125f,0.069199982452392561071974341757595539093017578125f,-0.522400006300210772991476915194652974605560302734375f,-0.14339999738931652473183930851519107818603515625f,-9.9996493756915327821843675337731838226318359375e-05f,0.0363999914810062552561475968104787170886993408203125f,-1.0370999746859073109561677483725361526012420654296875f,0.006100009077787404976334073580801486968994140625f,-0.05679998457133794698137307932483963668346405029296875f,-0.367100003743171754422291996888816356658935546875f,0.7618999871373173249367027892731130123138427734375f,1.35240000704526863728460739366710186004638671875f,-0.4294000231981272008852101862430572509765625f,0.801299992156028650924781686626374721527099609375f,-0.4542000009298323703887945157475769519805908203125f,-0.865799996238946878435172038734890520572662353515625f,-1.04349996858835236679396984982304275035858154296875f,0.15059999922811961869939523239736445248126983642578125f,-0.701000002786516862585131093510426580905914306640625f,0.821400023443997095995428026071749627590179443359375f,-0.4145000351890921752584517889772541821002960205078125f,1.0010999841183421121826313537894748151302337646484375f,0.4548999596595770800178115678136236965656280517578125f,0.64860002576708808419425622560083866119384765625f,0.04449998009204847448927466757595539093017578125f,-0.71370000411868073086907315882854163646697998046875f,-1.2238999669313432860917600919492542743682861328125f,-0.96330000666975923451218477566726505756378173828125f,1.079700017857550875532979262061417102813720703125f,0.91170002819895756829282618127763271331787109375f,-0.14669997589588223974033098784275352954864501953125f,0.9348000528335571601701303734444081783294677734375f,0.36719997792243930234690196812152862548828125f,0.88079999832510935675600194372236728668212890625f,1.055699965286255093133149784989655017852783203125f,0.5889000079467887616146981599740684032440185546875f,-1.142899992159008615288939836318604648113250732421875f,-0.43800002467632292990629139239899814128875732421875f,-0.70959996857643137246896003489382565021514892578125f,1.0032000208675864172391811734996736049652099609375f,-0.12179999366104621305595401281607337296009063720703125f,-0.0636000046074389757677636225707828998565673828125f,0.1515999791786077910415997394011355936527252197265625f,0.16109999060034763118665068759582936763763427734375f,-0.283300002162158282015269605835783295333385467529296875f,0.219100003877282123454506290727294981479644775390625f,0.26470000124275661068651288587716408073902130126953125f,-0.0348000021696091010880991234444081783294677734375f,0.495599991442263299035175805329345166683197021484375f,0.0217999922454357175638506305404007434844970703125f,-0.162400006151198983417316412669606506824493408203125f,0.1595000080317259261164508643560111522674560546875f,-0.1958999897524715816388152234139852225780487060546875f,0.0700999908685684403053528512828052043914794921875f,0.0004999817237261172664375408203341066837310791015625f,-0.33599999526143076167272738530300557613372802734375f,0.017699997718632343168820852952194400131702423095703125f,0.07329999728202807318666600622236728668212890625f,-0.566999998010695005490333642228506505489349365234375f,-0.09539999089241035790109890513122081756591796875f,-0.1497999889075758428447215919732116162776947021484375f,-0.4409999910891055918682468472979962825775146484375f,-0.0758999929189683175678737825364805757999420166015625f,0.0465999984383582699365433654747903347015380859375f,-0.1792000100195407785719226012588478624820709228515625f,0.14209999890029434421734322313568554818630218505859375f,-0.253999992892146142509091077954508364200592041015625f,-0.1438999912202358899548926274292171001434326171875f,-0.0536999875783923386052265414036810398101806640625f,-0.06329999936819064032533788122236728668212890625f,0.29619999607205382829988593584857881069183349609375f,0.74970000712871520676117143011651933193206787109375f,1.25839992496967401081064963364042341709136962890625f,0.4170000107884401785440786625258624553680419921875f,0.2164999622702605819313248503021895885467529296875f,-0.2054999997168778236300568096339702606201171875f,0.4777000048339363758032050100155174732208251953125f,-0.69570000253915775800805931794457137584686279296875f,-0.04799997639656083947556908242404460906982421875f,-0.55799999397993094163439309340901672840118408203125f,-1.06159998145103440236880487645976245403289794921875f,-0.5376000254392625521404625033028423786163330078125f,-0.939699986565112777014974199119023978710174560546875f,0.33420002134442317043294679024256765842437744140625f,0.5446999588489536137103641522116959095001220703125f,0.12779998512268075927522659185342490673065185546875f,-0.1749000028729437161700843716971576213836669921875f,-0.3218999767065042760805226862430572509765625f,-1.455799955558777725173058570362627506256103515625f,-0.3870000131726261116682508145458996295928955078125f,-0.9349999570846563301529386080801486968994140625f,0.92789998031258580368785260361619293689727783203125f,0.63269997279644041299206946860067546367645263671875f,0.56640001938343065290837330394424498081207275390625f,0.4927999678373335967762614018283784389495849609375f,-0.138300002087652862403643894140259362757205963134765625f,-0.5183999935522674018528732631239108741283416748046875f,-0.00210001044869390085523264133371412754058837890625f,-0.27189997014999445212879436439834535121917724609375f,0.47429998315572685640972849796526134014129638671875f,-0.44509998282194163099489969681599177420139312744140625f,-0.0612999954342843267340867896564304828643798828125f,0.98659997519254682973866010797792114317417144775390625f,0.436000005245208654969246708787977695465087890625f,-0.2550000244379042602105300829862244427204132080078125f,0.4098000133454799964738413109444081783294677734375f,0.5509999956339595161836086845141835510730743408203125f,-0.3145000036358833739313922706060111522674560546875f,-0.0700000030547378582213013942237012088298797607421875f,-0.1072999965071677053174425964243710041046142578125f,0.6351000044658778875827920273877680301666259765625f,0.231299999308586023971656686626374721527099609375f,1.24599998717755067190182671765796840190887451171875f,-0.296100004458427434883560636080801486968994140625f,-0.6850999847590923952367347737890668213367462158203125f,-0.0907000010490417452047040569595992565155029296875f,0.106399990437925140440711402334272861480712890625f,0.03499998807907100939473821199499070644378662109375f,-0.6267999942570925409057736032991670072078704833984375f,0.3160000038109720410517411437467671930789947509765625f,0.1928999898120762335196332060149870812892913818359375f,-0.09599999349191790098956289511988870799541473388671875f,-0.4426999833017590813000197158544324338436126708984375f,-0.388899997925758356132064363919198513031005859375f,-0.0671999669328335880180702588404528796672821044921875f,-0.1656999927043916187585637089796364307403564453125f,-1.020599998371303218647199173574335873126983642578125f,-0.0737000060558319347592259873636066913604736328125f,0.8755999870464206136233542565605603158473968505859375f,0.01770000557899464865840855054557323455810546875f,-1.08529996962547325978221124387346208095550537109375f,-0.175199993658065267254642094485461711883544921875f,1.590199985313414998699954594485461711883544921875f,-0.57299999040365179325817734934389591217041015625f,-0.785000026226043701171875f,-0.90240003058910378541668251273222267627716064453125f,0.28339999113082914306005477556027472019195556640625f,0.27600000762939469467482922482304275035858154296875f,-0.1139999935626985205772143672220408916473388671875f,0.39779997782111120585568642127327620983123779296875f,1.653299968671799291541901766322553157806396484375f,-1.2128999929785724276598557480610907077789306640625f,0.988800045204161648371155024506151676177978515625f,1.0269000153005123365801409818232059478759765625f,-0.2388999973297121215409788419492542743682861328125f,-0.33930000699758533500016710604541003704071044921875f,-0.369200002121925052733786287717521190643310546875f,0.925800044071674488321832541259936988353729248046875f,0.1139999980330461681887754821218550205230712890625f,-0.78840002000182896058078085843590088188648223876953125f,-0.35730001796483923381941849584109149873256683349609375f,0.265199997327477010600205176160670816898345947265625f,-0.5968999748066072807972659575170837342739105224609375f,0.513899990475177759208236238919198513031005859375f,0.135600050115584735976881347596645355224609375f,0.68670001404285496704460456385277211666107177734375f,-1.20780000240802820599128608591854572296142578125f,0.16350001627206811605219627381302416324615478515625f,1.83119998996257749013238935731351375579833984375f,-0.188699995625018601685951580293476581573486328125f,-0.8746999743461609799766165451728738844394683837890625f,0.124400004757940774169355790945701301097869873046875f,0.1282000024646519076299000516883097589015960693359375f,-0.2130999979615211259442730806767940521240234375f,0.8771000049725172775794135304749943315982818603515625f,0.30989999959468850221355751273222267627716064453125f,0.8448000162139532864813418200355954468250274658203125f,0.2353000009179115892266054288484156131744384765625f,-0.59669998878538610398436503601260483264923095703125f,0.0478000009179115892266054288484156131744384765625f,-0.93399997299909609527190923472517170011997222900390625f,-0.3819000008463859785479144193232059478759765625f,-1.255699998925626115209297495312057435512542724609375f,0.058799981129169509586063213646411895751953125f,-0.199900000935792743206320665194652974605560302734375f,-0.02590000055581331839249514814582653343677520751953125f,-0.029900000526010994672532206095638684928417205810546875f,0.1485999941766262821829513995908200740814208984375f,-0.480799981151521393929471059891511686146259307861328125f,0.2481999941945076670890557579696178436279296875f,0.8967000073373316837432867032475769519805908203125f,-0.11950001734495163674409923260100185871124267578125f,-0.48969999308437106577684971853159368038177490234375f,-0.1285999977901577384642450851970352232456207275390625f,0.64220000802278509599574363164720125496387481689453125f,0.168500005543231967664041803800500929355621337890625f,0.1719999982714652819737466415972448885440826416015625f,-0.1646999977037311158056809290428645908832550048828125f,0.425199982593953806908615433712839148938655853271484375f,-0.2679999921917914917912639793939888477325439453125f,-0.009300032925604995170942856930196285247802734375f,-0.2304000091463327404284200383699499070644378662109375f,0.0710000028610229261261110877967439591884613037109375f,0.2741999900519846988800054532475769519805908203125f,1.8273000009775159924174658954143524169921875f,-1.393500024318695462710593346855603158473968505859375f,0.0548000614762305104932238464243710041046142578125f,-0.35970000966191317814946160069666802883148193359375f,-1.37929998881816828770752181299030780792236328125f,0.1415999918818471314807538874447345733642578125f,-1.55079997403621661078432225622236728668212890625f,1.553400016015768159149956773035228252410888671875f,-0.70870004718303647450738935731351375579833984375f,0.745500041812658054141138563863933086395263671875f,0.715099984908103891712016775272786617279052734375f,0.155100003571808364721817952158744446933269500732421875f,0.12749999579042192721090032136999070644378662109375f,0.487500010877847334000989576452411711215972900390625f,-0.081199997115135147396358661353588104248046875f,-0.99630000690817865915960283018648624420166015625f,0.63450004327297193640333716757595539093017578125f,1.252200049597024911207654440659098327159881591796875f,0.6532999940037722552688137511722743511199951171875f,-0.4224000241443517378314709276310168206691741943359375f,0.505800021794438503519586447509936988353729248046875f,-0.7590000172555446678046564557007513940334320068359375f,-0.52639998704791057804186493740417063236236572265625f,-0.2331000042572615171820871182717382907867431640625f,0.307999990776181231666441817651502788066864013671875f,0.71909998025894150686099237645976245403289794921875f,-0.4939999967291950344616680013132281601428985595703125f,0.4684999870657922116379268118180334568023681640625f,-0.3314999915137886166149883138132281601428985595703125f,0.141700004133582080356745791505090892314910888671875f,0.22120000538527928579668468955787830054759979248046875f,-0.031199996221065617874046438373625278472900390625f,1.01959996615499282057726304628886282444000244140625f,-0.30989999959468850221355751273222267627716064453125f,0.6333999823763967196299518036539666354656219482421875f,-0.09960000195503226905202609486877918243408203125f,-0.007900015553831796211170512833632528781890869140625f,0.097200000050663870609923833399079740047454833984375f,-0.1563000130921602115474655647631152532994747161865234375f,0.40970000787377358619778533466160297393798828125f,0.149499997749924506962315717828460037708282470703125f,0.3425000104308129067476329510100185871124267578125f,0.9574999928846954500016863676137290894985198974609375f,-0.17100000077486043892349698580801486968994140625f,-0.85959999387115237912126985975191928446292877197265625f,0.1602000130295753166365102515555918216705322265625f,-0.5039999953135849697360981735982932150363922119140625f,0.1489999947920441769610988558270037174224853515625f,0.258799984407425132104663134668953716754913330078125f,-0.26239999661445612133547911071218550205230712890625f,-0.3870999881714582979697070186375640332698822021484375f,-0.141400006568431990672252140939235687255859375f,0.6800999769732356536877659891615621745586395263671875f,0.16329999966621411289224852225743234157562255859375f,-0.754999976009130602250252195517532527446746826171875f,-0.1180000011324882080998577293939888477325439453125f,-0.1144000100851059187334612943232059478759765625f,-0.28499998554587424592909883358515799045562744140625f,-0.49609999670982318775713793002068996429443359375f,0.00720000317990798777145755593664944171905517578125f,0.599600009852647275465642451308667659759521484375f,-0.11549999368190722037752493633888661861419677734375f,1.771199961352348140053436509333550930023193359375f,0.516300011527538682543081449694000184535980224609375f,1.5813999767661091055970246088691055774688720703125f,0.905700000900029777994859614409506320953369140625f,1.1254999221563348754671096685342490673065185546875f,0.21570001930296456293945084325969219207763671875f,0.10909998100399942444482803693972527980804443359375f,-0.8751000166475773056617981637828052043914794921875f,-0.45349999511241900762570367078296840190887451171875f,-1.28700003984570532367115447414107620716094970703125f,0.457299999189377359698482905514538288116455078125f,0.55110001339912439988211190211586654186248779296875f,-1.09119999473094964059782796539366245269775390625f,-0.48210002192258816222647510585375130176544189453125f,-1.4563999913692473597848220379091799259185791015625f,-0.2387999912619591658113904486526735126972198486328125f,0.3395999806463720904758929464151151478290557861328125f,0.759299987179040680729258383507840335369110107421875f,0.00389998093843502147137769497931003570556640625f,0.5076000387012964409905180218629539012908935546875f,1.85309998455047519883009954355657100677490234375f,0.105599986934661327353524029604159295558929443359375f,-1.1225999935507768867637423682026565074920654296875f,-0.305700013715028973138032597489655017852783203125f,0.1997999991521239859793013238231651484966278076171875f,0.0103999920845030402460906771011650562286376953125f,-0.533800021474063246529340176493860781192779541015625f,-0.13380000335052610882513590695452876389026641845703125f,-0.0611999986797571171592835526098497211933135986328125f,-0.01269999812841415831599078956060111522674560546875f,-0.082999999046325767526610661661834456026554107666015625f,-0.276700006592273783923019436770118772983551025390625f,0.426199994470179088779104858986102044582366943359375f,-0.0620000060200691649470172706060111522674560546875f,0.9162999688357118532877620964427478611469268798828125f,-0.422899999572336593445243124733678996562957763671875f,-0.4344999992027875901356992471846751868724822998046875f,0.295099989005923379892237790045328438282012939453125f,-0.40489999952018240758633282894152216613292694091796875f,-0.363899991965293878593001863919198513031005859375f,0.58530002274811232521045667454018257558345794677734375f,0.1420999967396259222596199833787977695465087890625f,-0.082000001735985261763772768972557969391345977783203125f,-0.010499999865889453332101766136474907398223876953125f,1.1417000025317072253727701536263339221477508544921875f,0.09930000669956218217748755705542862415313720703125f,0.68900001163035629048181363032199442386627197265625f,-0.431400005823373788871322176419198513031005859375f,-0.508999993897974345458123934804461896419525146484375f,-0.27259999429583547936317700077779591083526611328125f,0.6057000127092002639273005115683190524578094482421875f,0.299700000572204583448865378159098327159881591796875f,-0.2130999986320734063127702029305510222911834716796875f,0.165799999740719794516508045489899814128875732421875f,0.92219999714493727793751531862653791904449462890625f,0.93810005190372525163411410176195204257965087890625f,0.28219996242523137652824516408145427703857421875f,0.446100011238455829154503362588002346456050872802734375f,0.34639999706894142317281648502103053033351898193359375f,0.29160000126957896782187162898480892181396484375f,0.22939998892545698794265263131819665431976318359375f,0.5619000242412093282240448388620279729366302490234375f,1.28300000366568500709263389580883085727691650390625f,-0.23729998711943611766628237091936171054840087890625f,1.367699973392486612056018202565610408782958984375f,0.8831999848812814857268449486582539975643157958984375f,-0.1003000429391851522353817927069030702114105224609375f,0.20010001552998968321617212495766580104827880859375f,-0.16470002515911996709974118857644498348236083984375f,-0.07410002898573875285137546597979962825775146484375f,0.608300046157836504789884202182292938232421875f,-0.135299989742040605023021271335892379283905029296875f,-0.1823999954223634034633505507372319698333740234375f,-0.592200025233626803355946321971714496612548828125f,1.36990001106858283463907355326227843761444091796875f,-1.392900049006939422469031342188827693462371826171875f,-0.4869999589323992239542349125258624553680419921875f,1.088700012314319565120968036353588104248046875f,0.03099998617172250447993064881302416324615478515625f,-0.23940002588033681973911370732821524143218994140625f,0.5235000317692755089638012577779591083526611328125f,-0.7725000020861625404933192839962430298328399658203125f,-1.13729997609257704738183747394941747188568115234375f,-0.13529999577701090629489044658839702606201171875f,-0.56669998543262478651882929625571705400943756103515625f,0.180199996340274726236430069548077881336212158203125f,-0.75440001738667472608312891679815948009490966796875f,-0.0805999976634979020673199556767940521240234375f,0.5632999858453870967167631533811800181865692138671875f,0.1890000004172325276385890902020037174224853515625f,0.67839999817162743855902817813330329954624176025390625f,-0.404599990332126679959401371888816356658935546875f,0.152899975432455814683407879783771932125091552734375f,0.0868999972701072209702033433131873607635498046875f,-1.031899998126923900798601607675664126873016357421875f,0.09509999489188203369849361479282379150390625f,-0.618799983513355300601688213646411895751953125f,0.26159999346509577033970117554417811334133148193359375f,-0.318600009748339518811377502061077393591403961181640625f,0.219900002574920705455951974727213382720947265625f,1.056400018787383654483846839866600930690765380859375f,-0.279999989569187146543072230997495353221893310546875f,0.3503999929413199598826622604974545538425445556640625f,0.14590001094937310455179613200016319751739501953125f,0.962899976363777998500381727353669703006744384765625f,-0.0987999970734119159487818251363933086395263671875f,0.124300008450448473684701866659452207386493682861328125f,0.0422999988913534963330675964243710041046142578125f,-0.605900011582672703980279038660228252410888671875f,-0.1137999974086880694557066817651502788066864013671875f,-0.427599982076883489146013062054407782852649688720703125f,-0.001300002586841486618141061626374721527099609375f,0.76510000040531167542212642729282379150390625f,-0.59509999116659173523657955229282379150390625f,-1.03409997966289513016135970246978104114532470703125f,0.4788000340282920763712581901927478611469268798828125f,-0.58480003539919866994978292495943605899810791015625f,0.1169999909698962170523373060859739780426025390625f,1.1562999641418461038711029686965048313140869140625f,-0.7089000074625015912488379399292171001434326171875f,0.543600018167495591114857234060764312744140625f,0.4853999856770041532172399456612765789031982421875f,0.50269994478225754619415965862572193145751953125f,-0.02879998485445955935801976011134684085845947265625f,-1.930899987554550278900933335535228252410888671875f,0.627000007361173228304096483043394982814788818359375f,-0.21560001077651946843616315163671970367431640625f,0.255300022953003580283848350518383085727691650390625f,0.0397999700203539585885437190881930291652679443359375f,-1.2186000241279604239252876141108572483062744140625f,0.60200006711482956234249286353588104248046875f,0.3380999730765819055733345521730370819568634033203125f,0.82279996262192778289090711041353642940521240234375f,-0.6681000024318688446101077715866267681121826171875f,1.0994999970793717380956877605058252811431884765625f,-0.269400017237663380598178264335729181766510009765625f,-0.24709998049736014991140109486877918243408203125f,1.13100003936886839284170491737313568592071533203125f,0.447099957847594708937322138808667659759521484375f,0.20099999455362560496496371342800557613372802734375f,-0.677199975612759796916861887439154088497161865234375f,-0.4437000178277497042245158809237182140350341796875f,0.859700002956390818553700228221714496612548828125f,0.6692999889671806812430077116005122661590576171875f,0.4130999919265507802634829204180277884006500244140625f,-0.15239999721050256908938536071218550205230712890625f,0.324400009675323897084808777435682713985443115234375f,0.05599999973177904877275068429298698902130126953125f,0.8481999599218372765108142630197107791900634765625f,-0.01809999798014760852282734049367718398571014404296875f,0.9885999811381103175023099538520909845829010009765625f,0.213699995923042340706388131366111338138580322265625f,-0.09619999640733001289305548198171891272068023681640625f,0.1015999981403350460595902404747903347015380859375f,0.3155999857798221785998293853481300175189971923828125f,-0.14609999760389325018650197307579219341278076171875f,-0.03509998818635950978972459779470227658748626708984375f,-0.263000001989305065563939933781512081623077392578125f,-0.35419998617768300164243555627763271331787109375f,-0.2318999966740609153248442453332245349884033203125f,-0.6803999872416259808716176848975010216236114501953125f,-0.126700002531707323072396320640109479427337646484375f,-0.03539999823123218813947232774808071553707122802734375f,0.0034999985396861887920749722979962825775146484375f,-0.5488000017300247446172534182551316916942596435546875f,-0.06450000572204572790724341757595539093017578125f,0.92850000438839208616315090694115497171878814697265625f,-0.21689999712109564455886356881819665431976318359375f,-0.7756999772071839771570012089796364307403564453125f,-0.45930000811517235348446774878539144992828369140625f,-0.5202999993905421671769317981670610606670379638671875f,0.21300001022219650081979125388897955417633056640625f,0.8708999894917008877115449649863876402378082275390625f,0.274600011491775486405231276876293122768402099609375f,0.5669000267744053189744590781629085540771484375f,-0.089700020837784055771635394194163382053375244140625f,0.3264000107407571960038694669492542743682861328125f,-0.2321999929726121791873083566315472126007080078125f,1.3789999455809596184963083942420780658721923828125f,-0.5463000112853944045809839735738933086395263671875f,1.1293999884411694001329351522144861519336700439453125f,-0.3158999972403051526015360650490038096904754638671875f,-0.37779999093413341171299180132336914539337158203125f,-1.13490000466108309495893990970216691493988037109375f,-0.84719997434616090004055877216160297393798828125f,0.150899988001584883789973901002667844295501708984375f,-0.02649998968839639701400301419198513031005859375f,0.5142000106900932454578878605389036238193511962890625f,-0.2729999740123754126130961594753898680210113525390625f,0.52560000958442731899822319974191486835479736328125f,-0.8507999758243567356430503423325717449188232421875f,0.2685000108331443424702911215717904269695281982421875f,0.1582000084996220135025168929132632911205291748046875f,0.144300007146596964702212062547914683818817138671875f,-0.8548000063419340932568957214243710041046142578125f,-0.13200002136826594067997575621120631694793701171875f,1.53429998881816942457589902915060520172119140625f,0.64589999500513073371621430851519107818603515625f,-0.8506999858498573985343682579696178436279296875f,1.027200010108947569875681438134051859378814697265625f,-0.108100024485587642431028143619187176227569580078125f,-0.6609000090122219006616433034650981426239013671875f,1.00910000678300804111131583340466022491455078125f,-1.3275000289082525828376901699812151491641998291015625f,-1.30910000354200573013940811506472527980804443359375f,0.13710000433921809559478788287378847599029541015625f,0.166000003680586483145731335753225721418857574462890625f,0.17070000402927387739282494294457137584686279296875f,0.8983999769002199986545065257814712822437286376953125f,0.152700004759430907341766214813105762004852294921875f,0.62160001393556552073960119741968810558319091796875f,-0.17460000210404391651763944537378847599029541015625f,0.5879999979659917830332460653153248131275177001953125f,0.2576000045962631990192903685965575277805328369140625f,-0.0528999996468424438234023909899406135082244873046875f,-0.26639999651014800718940023216418921947479248046875f,0.0118000020429491758733320239116437733173370361328125f,-0.4592999979078769001716864295303821563720703125f,-0.2622999979972837625297188424156047403812408447265625f,0.36680000312328342459977648104541003704071044921875f,-0.4937999991223216422753239385201595723628997802734375f,-0.2114000089526175685250564129091799259185791015625f,-1.0678999708130956225460295172524638473987579345703125f,-0.30730000194609152774916083217249251902103424072265625f,0.8389000208362935584460728932754136621952056884765625f,0.144200003612041438572077822755090892314910888671875f,0.0477999993905424691575944962096400558948516845703125f,-0.06219999558031563680060571641661226749420166015625f,-0.2650999837160112893030827763141132891178131103515625f,0.075999990940093908875496708787977695465087890625f,-0.63470002091228938656541913587716408073902130126953125f,0.22029999942779543431470301584340631961822509765625f,-0.2374999932199717278535899822600185871124267578125f,0.086900002634525463918180321343243122100830078125f,-1.13290002963542946190500515513122081756591796875f,0.07920000525116943634884592029266059398651123046875f,0.58760000547170587736900415620766580104827880859375f,-1.200000002086162087522325236932374536991119384765625f,0.2664000202774996495236337068490684032440185546875f,-0.4482000139385462489372002892196178436279296875f,0.75360003142952880494931378052569925785064697265625f,-0.59670001836419128693478342029266059398651123046875f,0.1068000189185145387682496220804750919342041015625f,-0.92640001942068350704317936106235720217227935791015625f,-1.08989998454451519904750966816209256649017333984375f,0.783899989730119539643737880396656692028045654296875f,0.802999966263771458585551954456605017185211181640625f,1.031699984985589679808981600217521190643310546875f,0.39589995596408922295950105763040482997894287109375f,-0.317100005233287962624899591901339590549468994140625f,-0.9706000116705890690127489506267011165618896484375f,1.01160001259446108434758571092970669269561767578125f,-0.901299986195564173385719186626374721527099609375f,0.079499974541365592273223228403367102146148681640625f,1.3755000237450005773354178018053062260150909423828125f,-0.7029000071346758193868708985974080860614776611328125f,-0.71619997521042844113026148988865315914154052734375f,-0.189300005507468949161875571007840335369110107421875f,-0.845299956512451711176936441916041076183319091796875f,0.85770004685521161746919460711069405078887939453125f,0.051499954819679061301940237171947956085205078125f,-0.324900000637769803546461844234727323055267333984375f,-0.65529998601675032432467560283839702606201171875f,0.8825999841630451925311717786826193332672119140625f,0.0256000278756018939674277135054580867290496826171875f,-0.171599993260204808365188000607304275035858154296875f,-0.32789997926950487094899244766565971076488494873046875f,-0.2931000036239623796063824556767940521240234375f,-0.55329998707473271313261875548050738871097564697265625f,-0.1186999893665312555413038353435695171356201171875f,0.23220004166215613139456763747148215770721435546875f,-0.092199997591972504551449674181640148162841796875f,0.8719999714121222655904830389772541821002960205078125f,0.0759000039458275210790816345252096652984619140625f,-0.4867999875515700569650334728066809475421905517578125f,-0.1673000024676323249650522484444081783294677734375f,-0.31890000202357760006321996115730144083499908447265625f,0.0261999975621700631478461218648590147495269775390625f,-0.313099991479516059911247793934307992458343505859375f,0.523500003755092535584481083787977695465087890625f,-0.276500007085502375048235990107059478759765625f,0.03289999372363094920501680462621152400970458984375f,0.5544999828115109163917395562748424708843231201171875f,-0.1195999974846839730702185988775454461574554443359375f,0.162400000004470357328756335846264846622943878173828125f,-0.00949999968707559983016608384787105023860931396484375f,0.31270000226348637628603910343372263014316558837890625f,0.262099992120266023221120121888816356658935546875f,-0.04830000622272478949525975622236728668212890625f,-0.22780000538825984079949193983338773250579833984375f,-0.4423999876737594316722379517159424722194671630859375f,0.2877000031948089286970571265555918216705322265625f,0.84010000420510733132317682247958146035671234130859375f,-0.2531000027298926369212495046667754650115966796875f,0.56360005214214226043623057194054126739501953125f,0.3146999891728163856186029079253785312175750732421875f,-0.3936999750614169624185478824074380099773406982421875f,0.60210000240206706934031899436376988887786865234375f,-0.56989999303817739928490482270717620849609375f,-0.66510005345344591631828734534792602062225341796875f,-0.61019993226528157492793980054557323455810546875f,0.038100018823147507873727590776979923248291015625f,1.62919998185634540277533233165740966796875f,-0.00840001139640822458432012354023754596710205078125f,-0.8635999943256376099043336580507457256317138671875f,0.495300011795759331789668067358434200286865234375f,-0.1211000092267990879690842120908200740814208984375f,0.18750000260770305349211639622808434069156646728515625f,-0.314099997171759515168787402217276394367218017578125f,-0.4889999829828735045111898216418921947479248046875f,-0.609499995589256826633572927676141262054443359375f,-0.13919999645948433197872873279266059398651123046875f,0.914699972856045206981434603221714496612548828125f,0.09389998980462543887171023015980608761310577392578125f,0.305999992504716100683026525075547397136688232421875f,-0.347700008670985771797035113195306621491909027099609375f,0.1105000154748558838235794610227458178997039794921875f,0.0885000146329399939304494182579219341278076171875f,0.33069997482299839930419693700969219207763671875f,0.778800006759166496550506053608842194080352783203125f,-0.0360000049471853600380200077779591083526611328125f,-0.8517000037610529261655756272375583648681640625f,0.7711000402212135895751998759806156158447265625f,0.13979998354315714692575056687928736209869384765625f,-0.1571999927118421513938528732978738844394683837890625f,-0.235400003111362554619745424133725464344024658203125f,-0.749800005336105801490020894561894237995147705078125f,-0.031200000095367386165889911353588104248046875f,-0.0802000142589209819021789371618069708347320556640625f,-0.0044000088930129521713752183131873607635498046875f,-0.3691999728038910522087689969339407980442047119140625f,0.3598999976575374404319518362171947956085205078125f,-0.032600011432170727943002930260263383388519287109375f,0.23490000674724598184184287674725055694580078125f,1.0756000035494566180460651594330556690692901611328125f,-0.48660000529289249016073881648480892181396484375f,0.2090000055581331395160304964520037174224853515625f,0.23440000829696661099887933232821524143218994140625f,-0.4438999874576927684444171973154880106449127197265625f,0.026700008082389814489943091757595539093017578125f,0.511999977484345780709418249898590147495269775390625f,0.136400000309944147147689363919198513031005859375f,-0.013400003685057171320949009896139614284038543701171875f,0.20929999880194660732968259253539144992828369140625f,-0.299300012771785250986766868663835339248180389404296875f,-0.1297999974012373769483019714243710041046142578125f,0.94390001614242768912532710601226426661014556884765625f,-0.11340000364780422614785493351519107818603515625f,0.370499995879828958322832477279007434844970703125f,-0.2231999882340431895499932579696178436279296875f,0.35680001124441595994341014375095255672931671142578125f,0.0175000000000000710542735760100185871124267578125f,-0.640699994045495824224190073437057435512542724609375f,0.09960000195503226905202609486877918243408203125f,0.456500003993511427324847318232059478759765625f,0.44860000907778729839492370956577360630035400390625f,0.21190004346370727716930559836328029632568359375f,0.673599982423335408299180926405824720859527587890625f,0.405899983363598781949121985235251486301422119140625f,0.3051000312507152756325012887828052043914794921875f,-1.300900026297568956579198129475116729736328125f,-0.906200016076862624458954087458550930023193359375f,0.05249996040016480236545248772017657756805419921875f,-0.74789997301101696081104819313623011112213134765625f,-0.2414999861121174262734712101519107818603515625f,-0.62780001842677590762065165108651854097843170166015625f,0.55510002074539688265986114856787025928497314453125f,-0.22500000253319729637269119848497211933135986328125f,-0.73730003435611735085331019945442676544189453125f,-1.45599997112154966316666104830801486968994140625f,0.6872999854803083508159033954143524169921875f,0.73029998616576197179028895334340631961822509765625f,-0.01000001251697568704912555404007434844970703125f,0.460099977904558254948597095790319144725799560546875f,0.602600002789497324329204275272786617279052734375f,-0.70620000359714030224722591810859739780426025390625f,-0.2192999993234874267500345013104379177093505859375f,0.1405000065155327104715610175844631157815456390380859375f,0.3495999970190226679989109470625407993793487548828125f,-1.0768000070534646805953116199816577136516571044921875f,0.7819000380061569277501121177920140326023101806640625f,-1.4193999911606312025469378568232059478759765625f,0.9505999875307082902509137056767940521240234375f,-0.05939997953772557792717634583823382854461669921875f,-0.3437000059068202606482600458548404276371002197265625f,0.1242999967530369342938456611591391265392303466796875f,0.2573000026166438747310394319356419146060943603515625f,0.0890000002309678794887304320582188665866851806640625f,0.67060002329349543970238300971686840057373046875f,-0.1620000020712613064688412123359739780426025390625f,-0.38520000788867481134047920932061970233917236328125f,0.198700026582181887846445533796213567256927490234375f,-0.7315000230669976843955737422220408916473388671875f,-0.114399997977912282198786897424724884331226348876953125f,-0.328400014004111451271228361292742192745208740234375f,-0.5808999923601743109458084290963597595691680908203125f,0.1200999953389167984596497262828052043914794921875f,-0.37680002890229236101049536955542862415313720703125f,0.7080000029206274803073029033839702606201171875f,-0.658000002659857141651400525006465613842010498046875f,-0.2600000146031380410249767010100185871124267578125f,-0.0273999951243399930689292887109331786632537841796875f,-0.36860000699162487336479898658581078052520751953125f,-0.400699993933737008688922287547029554843902587890625f,-0.700900016909837830780816148035228252410888671875f,0.2626000001445409903766403658664785325527191162109375f,0.3890000144243241919639331172220408916473388671875f,-0.232699994626640982442467020518961362540721893310546875f,0.369500001959502777371113779736333526670932769775390625f,0.324500010602176036655919233453460037708282470703125f,-0.144500000059604616353681194595992565155029296875f,0.33849999686330523029909045362728647887706756591796875f,0.18710001238584528238106940989382565021514892578125f,0.05459999282807126430583366527571342885494232177734375f,0.5640999760121108153043678612448275089263916015625f,-0.38270000326931474798897170330747030675411224365234375f,0.63760000763237467680255576851777732372283935546875f,-0.48659999322295188761700046597979962825775146484375f,-0.11199997591972277177774230949580669403076171875f,0.425899989193677885879196765017695724964141845703125f,-0.562899997523427231271853088401257991790771484375f,0.76350001239776599248898492078296840190887451171875f,0.41419998543262437351586413569748401641845703125f,-0.10100000703334810481237582280300557613372802734375f,0.613399994781612889482858008705079555511474609375f,-1.2326999921679497873583386535756289958953857421875f,0.888800016593933150943485088646411895751953125f,-1.240100002044439264636821462772786617279052734375f,0.205900012588500658239354379475116729736328125f,-0.35929998047351840995133898104541003704071044921875f,-0.44490002999305744424418662674725055694580078125f,0.385199996340274797290703645558096468448638916015625f,-0.140600007349252820176843670196831226348876953125f,0.3160999910473822893663964350707828998565673828125f,-0.296600003653765043054590933024883270263671875f,-0.0946999898433684705878476961515843868255615234375f,0.09549999040365264590946026146411895751953125f,-0.428399971777945864914727280847728252410888671875f,-0.475899987833946713777777404175139963626861572265625f,0.217700004684925119136096327565610408782958984375f,0.71259997596740731751197017729282379150390625f,0.1418999930977822288014067453332245349884033203125f,0.20999999314546613504717242904007434844970703125f,0.218499994218349424812686265795491635799407958984375f,0.278799996849894426986793405376374721527099609375f,-0.4367000279381871852280028178938664495944976806640625f,0.83270000319480885764278355054557323455810546875f,-0.534500017531216453647857633768580853939056396484375f,0.28970000772476200978644556016661226749420166015625f,0.06890001670122192223288948298431932926177978515625f,-0.8285000365376475883749662898480892181396484375f,-0.5497000070169566487976453572628088295459747314453125f,0.160899984574317755914307781495153903961181640625f,0.26179998770058166002172583830542862415313720703125f,0.6886000083327292742296776850707828998565673828125f,-0.6368000110581519823682583592017181217670440673828125f,-0.3100000005960463766996326739899814128875732421875f,-0.21570000164508817963593401145772077143192291259765625f,0.0968999951839446538315314683131873607635498046875f,-0.8329000028133395350238288301625289022922515869140625f,-0.481100003266334397267200984060764312744140625f,-0.228799999383091812177326573873870074748992919921875f,0.0920000107884408890868144226260483264923095703125f,-0.1177999925360084798597881672321818768978118896484375f,-0.12699999450892196506401887745596468448638916015625f,-0.244599990406632439299272618882241658866405487060546875f,-0.435800006707012632656272899112082086503505706787109375f,0.413399989752471608284167814417742192745208740234375f,0.41819999628066995001063332892954349517822265625f,-0.1947000116363166721811239767703227698802947998046875f,0.53309999557733522834723771666176617145538330078125f,0.1673000069007277712529457858181558549404144287109375f,-0.408499996714293878152801653413916938006877899169921875f,-0.73780002066195038423757068812847137451171875f,0.17720001197159263028879649937152862548828125f,-1.48679999492764469692929196753539144992828369140625f,0.32250001311302156636884319595992565155029296875f,-1.364600017005205057074590513366274535655975341796875f,0.945299996149540078249629004858434200286865234375f,0.859100022727250944853949476964771747589111328125f,0.055199963259696005479781888425350189208984375f,-1.23319998152852061679141115746460855007171630859375f,0.88789998075962017765050404705107212066650390625f,-0.39350000748038294062780551030300557613372802734375f,0.11510004689693431600971962325274944305419921875f,-0.183200004178285524858438293449580669403076171875f,-0.6704000314235685209496296010911464691162109375f,-0.627599991464614870295690707280300557613372802734375f,0.594699988055229056271855370141565799713134765625f,-0.38159999098777763748557845246978104114532470703125f,-0.40050005805492361332653672434389591217041015625f,0.25810000258088106050990973017178475856781005859375f,-1.39349997544288584094829275272786617279052734375f,-0.22400000339746473088098355219699442386627197265625f,0.446199990372360133505935664288699626922607421875f,-0.15230000144317745736088909325189888477325439453125f,0.601099976164847671356028513400815427303314208984375f,-0.0276000053226947983375794137828052043914794921875f,0.2962000310897821009348263032734394073486328125f,-0.986799985092878362280544024542905390262603759765625f,0.515700004029273628702867426909506320953369140625f,0.9910999845467507451957089870120398700237274169921875f,-0.8663999876998362825730737313278950750827789306640625f,-0.57240001300573339904076419770717620849609375f,-0.1386999962210655468197728623636066913604736328125f,0.1588000092923638195685498430975712835788726806640625f,-0.101200000393390610042843036353588104248046875f,-0.1497999975502490865864047009381465613842010498046875f,-0.3326000087499618729225403512828052043914794921875f,-0.2786000021487475830639368723495863378047943115234375f,-0.0957999999642371591335177072323858737945556640625f,0.41790000557005413384104031138122081756591796875f,0.3201000057697296341530091012828052043914794921875f,-0.46920000689029706109067774377763271331787109375f,-0.00599999898672098908036787179298698902130126953125f,0.11200000028312206978853282635100185871124267578125f,-0.3064000038862229047254004399292171001434326171875f,-0.1930000139474870035627418474177829921245574951171875f,0.2578000067293644281818387753446586430072784423828125f,-0.593900020202994483042857609689235687255859375f,0.071899994885921358900304767303168773651123046875f,0.4098000192314386058711761506856419146060943603515625f,-0.521400002507865334422376690781675279140472412109375f,0.66250000130385167107505139938439242541790008544921875f,0.05030000330209727366082006483338773250579833984375f,-0.010900014339387809059189748950302600860595703125f,0.1909000170588495137735662865452468395233154296875f,-0.568600004942715120392904282198287546634674072265625f,-0.313400014376640445590282979537732899188995361328125f,0.25989999646544459377395241972408257424831390380859375f,0.25490000555515290869834643672220408916473388671875f,0.414700002993642957704878426739014685153961181640625f,-0.177099991822242674999188238871283829212188720703125f,-0.3112000331014395104745062781148590147495269775390625f,-0.5282000028371811595206963829696178436279296875f,-0.30619999026060096269929999834857881069183349609375f,1.32859995468854918954093591310083866119384765625f,0.5271000170052051458924324833787977695465087890625f,-0.866300030601024673160281963646411895751953125f,-0.20839999500513073371621430851519107818603515625f,-0.0942000220894811945981928147375583648681640625f,0.09860000565052029486423634807579219341278076171875f,0.23850001835823064766373136080801486968994140625f,-0.605100000852346386892577356775291264057159423828125f,0.072499986588955067645656527020037174224853515625f,0.42559998246431351986984736868180334568023681640625f,0.01760000576972942099018837325274944305419921875f,-0.317000004604458762713647956843487918376922607421875f,-0.3755999958008526817820893484167754650115966796875f,-1.1211000168263911547228417475707828998565673828125f,0.7089999947547909187051118351519107818603515625f,0.7125999932713806828132874215953052043914794921875f,0.654399981158226839994540569023229181766510009765625f,1.10689999399185179385085575631819665431976318359375f,-0.751399976170063155223033390939235687255859375f,0.40639999509453783588242004043422639369964599609375f,-1.33850000554323145252055837772786617279052734375f,-0.5115000020563602589618312777020037174224853515625f,0.250699984806775688639390864409506320953369140625f,0.42690000546574591311355106881819665431976318359375f,-0.781199985194206192318233661353588104248046875f,0.3592999764502049497849611725541763007640838623046875f,0.7147999942719938104573884629644453525543212890625f,-1.12090003002285953925820649601519107818603515625f,-0.101599998587369899638588321977294981479644775390625f,-0.079200002643465883256368442744133062660694122314453125f,0.421200005646050035235106179243302904069423675537109375f,-0.0991000003941355789294220812735147774219512939453125f,-0.05930000805929314111608618986792862415313720703125f,0.7381999956473708213167128633358515799045562744140625f,0.07900000667572015800033113919198513031005859375f,-0.611900010457635090688199852593243122100830078125f,0.1043000026762486154296993845491670072078704833984375f,-0.5119000180572272906687203430919907987117767333984375f,0.58030000628232958348462489084340631961822509765625f,0.163500022754073104369609836794552393257617950439453125f,-0.365099997909366924542240440132445655763149261474609375f,-0.0476999990969895204528938847943209111690521240234375f,-0.5247000190496446947463482501916587352752685546875f,-0.0342999816656115452673248000792227685451507568359375f,-0.1492999864339825677461703890003263950347900390625f,0.135400014026462844896059323218651115894317626953125f,-0.0741999982476233554962163907475769519805908203125f,0.7170000206977131806951319958898238837718963623046875f,-0.1986000098228455357229904620908200740814208984375f,0.38450001939386158067435417251545004546642303466796875f,-0.44930001742839831280207363306544721126556396484375f,-0.2709000021949405923038511900813318789005279541015625f,0.515699996280670092119180480949580669403076171875f,-0.370399995660781689021945339845842681825160980224609375f,-0.205600011074543065348052550689317286014556884765625f,0.307000025168061796421170583926141262054443359375f,-0.3646000170052052791191954383975826203823089599609375f,0.5664000083938238727654379545128904283046722412109375f,-0.5558000008583068307643770822323858737945556640625f,-0.26149998385831718650251787039451301097869873046875f,-0.537099985284358094617118695168755948543548583984375f,-0.2700000020861625937840244660037569701671600341796875f,0.454500003207474634336904273368418216705322265625f,1.22059998052716256466965205618180334568023681640625f,-0.85799996387958543664353783242404460906982421875f,-0.89500000052154060625753118074499070644378662109375f,0.0894999889582397400999980163760483264923095703125f,-1.407899995213746979771940459613688290119171142578125f,0.481799998652935101972616394050419330596923828125f,0.35030000820085405166537384502589702606201171875f,0.450599977714568833420116789056919515132904052734375f,0.2456999910652637453267743694595992565155029296875f,0.44679998434781964533613063395023345947265625f,-0.512600006663799234729594900272786617279052734375f,1.3614999822378166527414578013122081756591796875f,-0.37889999166727061918891195091418921947479248046875f,1.36809996905326869409691425971686840057373046875f,1.35240002909898748839623294770717620849609375f,-0.6146000097036363740699016489088535308837890625f,0.4318000017821788105720770545303821563720703125f,1.05349999421834894519633962772786617279052734375f,-0.81799998250603689342597135691903531551361083984375f,-0.02929996438026449823155417107045650482177734375f,-0.45719999327063565175421899766661226749420166015625f,0.441199989217519572548553696833550930023193359375f,0.5946000179737804813839829876087605953216552734375f,0.3000000048428770327291204011999070644378662109375f,1.08999998584389690137186335050500929355621337890625f,0.325500006198883085062334430404007434844970703125f,0.482800032508373444528615436865948140621185302734375f,-0.180400003111362394747629878111183643341064453125f,0.2928000048294661095127366934320889413356781005859375f,-0.19560000064373017636398799368180334568023681640625f,0.0517000076726079615951903178938664495944976806640625f,-0.397300000679492892885491528431884944438934326171875f,-0.0907999904274940927706438742461614310741424560546875f,-0.34840000491440292762490571476519107818603515625f,-0.18609999462366089151288406355888582766056060791015625f,-0.28910001471787705185079175862483680248260498046875f,0.228100008243322438516997863189317286014556884765625f,0.18589999403655521614808776575955562293529510498046875f,-0.306299998973309950400789603008888661861419677734375f,-0.0969000044226646650713519193232059478759765625f,0.62750001326203364016009800252504646778106689453125f,-0.35750000268220905041971491300500929355621337890625f,-0.2569000363111497708956676433444954454898834228515625f,0.4981000074982642189524995046667754650115966796875f,0.3795000123530625213419398278347216546535491943359375f,-0.044699986863136143000474476139061152935028076171875f,-0.607199990588426441462388538639061152935028076171875f,-0.04079999847412096869447850622236728668212890625f,-0.330500011079013500392420610296539962291717529296875f,0.3377000002145766899275258765555918216705322265625f,0.01230002526640883164787965142750181257724761962890625f,-0.55589999984800797516726333924452774226665496826171875f,0.35219999468624563210283895386965014040470123291015625f,-0.208199997472763076444834950962103903293609619140625f,0.4271000018432733469353479449637234210968017578125f,0.9532000371098519053703057579696178436279296875f,0.115700016397237703813516418449580669403076171875f,1.4476999900817872912739403545856475830078125f,0.155999993175268247824760692310519516468048095703125f,0.871999976813793153951337444595992565155029296875f,0.8308999957501888022903813180164434015750885009765625f,-0.8509999988377092705604809452779591083526611328125f,0.126100006395578390083755948580801486968994140625f,0.21729997333586226204715785570442676544189453125f,-0.364699999082088499591236541164107620716094970703125f,0.91799998641759117390392930246889591217041015625f,-0.359599979566037675215994795507867820560932159423828125f,1.022499983869492812260659775347448885440826416015625f,-0.6029000043034553613097159541212022304534912109375f,0.7067999965667723927253973670303821563720703125f,1.437699993807077358809465295053087174892425537109375f,-0.708399983680248368500542710535228252410888671875f,-0.3039000213205813594186111004091799259185791015625f,-1.09279997081756619081716053187847137451171875f,0.0764999756813049547332639122032560408115386962890625f,0.9624000223934654485447026672773063182830810546875f,-1.1628000030040741563652773038484156131744384765625f,1.0060000088214877678183256648480892181396484375f,-1.13099999526143069061845380929298698902130126953125f,0.88209998586177817969655734486877918243408203125f,0.23020000583976507613215289893560111522674560546875f,0.71169996520429901210036405245773494243621826171875f,0.8314999876022339808656624882132746279239654541015625f,-0.4359000145256519687109175720252096652984619140625f,0.092699965047836396792035884573124349117279052734375f,-0.0155999966204165563254946391680277884006500244140625f,-0.5033999932542438937588258340838365256786346435546875f,0.398999987900257035988715870189480483531951904296875f,-0.55869998515695318275220415671356022357940673828125f,-0.060500006198882960717355672386474907398223876953125f,-0.51250002317130594065019977279007434844970703125f,0.1129000038564205166125020696199499070644378662109375f,0.354600004823506065321225833031348884105682373046875f,0.1098000051498413398576303734444081783294677734375f,0.462800027143955627906279914896003901958465576171875f,-0.333200003135204259052670749952085316181182861328125f,0.1323000008657573278725294585456140339374542236328125f,-0.3555000057518482581286889399052597582340240478515625f,-0.4264999901726838071880365532706491649150848388671875f,0.12409999247789382792461765347979962825775146484375f,0.70410001009851708175801832112483680248260498046875f,0.073499996602535144774037689785473048686981201171875f,0.372600005583465065939208216150291264057159423828125f,-0.5489999998211860798846828402020037174224853515625f,0.6077999936163422933788069713045842945575714111328125f,-0.2549999970197678322847423260100185871124267578125f,0.3926999969363211828721205165493302047252655029296875f,0.309299998056888458819457810022868216037750244140625f,-0.4524000074923038017260523702134378254413604736328125f,0.07799999368190757564889281638897955417633056640625f,-0.1015999950110912930512085949885658919811248779296875f,-0.247600001411139913898296072147786617279052734375f,0.377899986422061573421160574071109294891357421875f,-0.035899994111061150903196903527714312076568603515625f,0.37980001546889485819491483198362402617931365966796875f,0.2454999816864731343457606271840631961822509765625f,-1.3377999591946603263892257018596865236759185791015625f,0.7371999956548211940798864816315472126007080078125f,-0.8382999697148800777313226717524230480194091796875f,0.461399976915121357023963355459272861480712890625f,-0.6389000140935181537571452281554229557514190673828125f,-0.22500001080334186553955078125f,0.67379999915957444756031691213138401508331298828125f,-0.75959996857643119483327609486877918243408203125f,1.1374000113666056233086010252009145915508270263671875f,-0.5445999818384645863034165813587605953216552734375f,0.73730000403225426364173245019628666341304779052734375f,-0.51280002960264692291048049810342490673065185546875f,-0.799299976599216588368790326057933270931243896484375f,0.341400005972385400809798738919198513031005859375f,0.747600006067752786975688650272786617279052734375f,0.108900015658139892593680997379124164581298828125f,1.10529996753931047948071864084340631961822509765625f,-0.25849998438358312569107511080801486968994140625f,1.166199971783161259963890188373625278472900390625f,-0.3795999953985216279761516489088535308837890625f,-0.2303999952137469886537246566149406135082244873046875f,0.9218999947369095337990074767731130123138427734375f,-0.900999999880790714001932428800500929355621337890625f,0.594599995696544425527463317848742008209228515625f,0.160500002920627604652281661401502788066864013671875f,-0.10390001748353228094856604002416133880615234375f,0.5020999715566636378838438758975826203823089599609375f,-0.0039999978840352667930346797220408916473388671875f,0.824699984478950653965512174181640148162841796875f,-0.391700000333786046979867023765109479427337646484375f,-0.24320002876520152579331579545396380126476287841796875f,0.67520000349283204599259988754056394100189208984375f,-0.4770000075101850089964727885671891272068023681640625f,-0.4230000228881838353345301584340631961822509765625f,-0.5902000271111729201578555148444138467311859130859375f,0.057399993336200605398289553704671561717987060546875f,0.5850000139325859382921635187813080847263336181640625f,0.06300000232458113913480701739899814128875732421875f,-0.3851000134810806940777183626778423786163330078125f,0.629900006449222615856342599727213382720947265625f,-0.29080002980381269850340686389245092868804931640625f,0.4672000181555748810069417231716215610504150390625f,0.2913000107824801343525678021251223981380462646484375f,-0.11489999572038644970461973571218550205230712890625f,0.42320001606196200327758560888469219207763671875f,-0.045200005280971566890002577565610408782958984375f,-0.4619999995380641433229129688697867095470428466796875f,-0.1211999944329260614495069603435695171356201171875f,-0.260299996447562964618782643810845911502838134765625f,-0.164500012613833013208619604483828879892826080322265625f,0.264099991211294737869508253425010479986667633056640625f,-0.228099996806681193373123051060247235000133514404296875f,0.5803999939844006750178095899173058569431304931640625f,0.1576000227570535372478843783028423786163330078125f,0.416599999965727274542359737097285687923431396484375f,-0.4994000133633615945427663973532617092132568359375f,-0.4144000166788697203656965939444489777088165283203125f,0.72040001324415214867258328013122081756591796875f,-0.311200026433169796291622333228588104248046875f,-0.1080000179708004992562564439140260219573974609375f,-0.82030001119971274192721466533839702606201171875f,-0.28949996191263238642932265065610408782958984375f,0.0580000061988830140080608543939888477325439453125f,-0.527000018239021983390557579696178436279296875f,-0.3537999926030634956219955711276270449161529541015625f,-0.9504000339567664212836461956612765789031982421875f,0.538499997347593239993557290290482342243194580078125f,0.269500029861927004048993694595992565155029296875f,-1.2290999808549880611963089904747903347015380859375f,0.3257999871492387455873540602624416351318359375f,-0.19950002866983407301404440659098327159881591796875f,-1.35069995634555795049891457892954349517822265625f,-0.025300012689828843548411896335892379283905029296875f,1.097200000274181519444027799181640148162841796875f,0.7273000161767004811963488464243710041046142578125f,-0.3594999848604203407376189716160297393798828125f,-0.9066000201940536129541214904747903347015380859375f,1.006600000822544416223536245524883270263671875f,0.1992000288136302754704587414380512200295925140380859375f,-0.8007000234194101562223977452958934009075164794921875f,0.4047999716781081058769586888956837356090545654296875f,0.311699989698082102762555223307572305202484130859375f,0.3350999652385713289959312533028423786163330078125f,0.50550000917911575015750713646411895751953125f,0.63509997447729116260006776428781449794769287109375f,0.79769999246597222963828244246542453765869140625f,-1.23450000452995301003511485760100185871124267578125f,0.06430003530979178094639792107045650482177734375f,-0.75030000717639921958834747783839702606201171875f,-0.1053000021100043426258707768283784389495849609375f,-0.1918000113189222144427503735641948878765106201171875f,0.00459999994337556561418978162691928446292877197265625f,-0.1635000223442915068261527267168276011943817138671875f,0.7683000178456307338592523592524230480194091796875f,-0.1280000104829672746831192853278480470180511474609375f,0.09560002031326320093285175971686840057373046875f,0.02610001332461830969577931682579219341278076171875f,-0.5913000079512595874575708876363933086395263671875f,0.27360001488924001744607039654511027038097381591796875f,-0.64800000143051139644967406638897955417633056640625f,0.398100023777783196266000231844373047351837158203125f,-0.34050001782178895837205345742404460906982421875f,-0.608499989300966159788686127285473048686981201171875f,0.1297000083208086351760357501916587352752685546875f,-0.040700000937283042912184782835538499057292938232421875f,0.03710000012964009830085387875442393124103546142578125f,-0.477399999408424202673728586887591518461704254150390625f,-0.181399994349479509736511317896656692028045654296875f,-0.3119000096380712960808523348532617092132568359375f,0.286700020599365501539068645797669887542724609375f,-0.41269999376982458016982491244561970233917236328125f,0.1103000119447710147824182058684527873992919921875f,0.29299999785423269660356027088710106909275054931640625f,0.32620001209080229376269244312425144016742706298828125f,0.0845999977827072147107401178800500929355621337890625f,-0.163800003927946136172977276146411895751953125f,0.3504000017330053395880895550362765789031982421875f,0.23639999621212470781728143265354447066783905029296875f,0.444800000493227987163891157251782715320587158203125f,1.5283999577522280333141679875552654266357421875f,-0.132400003655254783208050639586872421205043792724609375f,-0.240200012545287666654303393443115055561065673828125f,-1.018500008523464117615731083787977695465087890625f,0.1684000179529192564586992375552654266357421875f,0.41989999154806145753582313773222267627716064453125f,-1.48450000065565035356485168449580669403076171875f,0.73090000796914100789081203402020037174224853515625f,-0.28460003414154044776296359486877918243408203125f,0.84979999322891242030664216144941747188568115234375f,0.248999963462352980059222318232059478759765625f,0.0575000104308127646390857989899814128875732421875f,1.6875999855041499131402815692126750946044921875f,-1.22619999562501913459300340036861598491668701171875f,0.77850000941753449978932621888816356658935546875f,-0.157500001080334184333509028874686919152736663818359375f,0.158600001031160342535031304578296840190887451171875f,-0.663200004625320360673867980949580669403076171875f,0.553799970549345488279868732206523418426513671875f,-0.1680000056028365662541546043939888477325439453125f,-1.0922999872684471256434335373342037200927734375f,-1.20730001558065414002385296043939888477325439453125f,1.31280001403093393719245796091854572296142578125f,0.27420000297874214112425761413760483264923095703125f,-0.50479998827427596808092857827432453632354736328125f,0.61270000334382057616267047706060111522674560546875f,-0.77670000793337834466001368127763271331787109375f,0.3353000072136520515186930424533784389495849609375f,-0.29149997124820981042603307287208735942840576171875f,0.397399984544515749718129882239736616611480712890625f,-0.101200000393390610042843036353588104248046875f,-0.7360999864652753021942999112070538103580474853515625f,-0.372699998724460357379939523525536060333251953125f,-0.46580002656281038753149914555251598358154296875f,0.1634000073730945690186899810214526951313018798828125f,-0.37940002457648491773767318591126240789890289306640625f,0.72139999673366528298856792389415204524993896484375f,0.0141999851718546121759345624013803899288177490234375f,0.6806000048160552040599213796667754650115966796875f,-0.451299993273615474009829995338805019855499267578125f,0.33969998626708974853727340814657509326934814453125f,-0.317300008502602537419079453684389591217041015625f,-0.0391000075280667491739450269960798323154449462890625f,-0.396900013884902147509592396090738475322723388671875f,-0.233100007200241154947661925689317286014556884765625f,-0.3487000177904964193231762692448683083057403564453125f,0.2190999978423118221826371154747903347015380859375f,0.44780001421719795029474653347278945147991180419921875f,-0.42980000634193427089257966144941747188568115234375f,0.2521999977782367619028036642703227698802947998046875f,-0.56540001473426837463875926914624869823455810546875f,-0.31439999600350831965300812953501008450984954833984375f,0.09839999567568302740738772627082653343677520751953125f,0.2567000078588723699368756570038385689258575439453125f,0.00609999700784680243259572307579219341278076171875f,-0.4111000226750969321898310226970352232456207275390625f,0.7063000100374221545962427626363933086395263671875f,-0.073199985402822687063917328487150371074676513671875f,-0.531600003504753004790472914464771747589111328125f,0.47170001530945315781906401753076352179050445556640625f,0.75670000092983258355161524377763271331787109375f,-0.44569998376369479586855959496460855007171630859375f,-0.22580003125667502672513364814221858978271484375f,-0.7273000229567287977516798491706140339374542236328125f,-0.009500007733702631185224163345992565155029296875f,-1.05899999490380292144209306570701301097869873046875f,-0.2311999652266507609965628944337368011474609375f,-0.8351000063657760819069153512828052043914794921875f,0.43470004020929309263010509312152862548828125f,-1.294599958741665002293075303896330296993255615234375f,0.910499985039233905581568251363933086395263671875f,-0.4618000066250562696268389117904007434844970703125f,0.2405000019520517895443845191039144992828369140625f,0.1321000022530556261557421748875640332698822021484375f,-0.8784000282347204091593084740452468395233154296875f,-0.0148999802231788436301940237171947956085205078125f,-1.38760001769065866028540767729282379150390625f,0.676399988240003580131087801419198513031005859375f,0.64259999831914882406636024825274944305419921875f,1.046799995821714457377993312547914683818817138671875f,-0.422600032293797056581752258352935314178466796875f,-0.364199982787668796735403020647936500608921051025390625f,-0.261799992953240856508045908412896096706390380859375f,-0.048599990451335894903195367078296840190887451171875f,0.795699982571601793779336730949580669403076171875f,-1.3818000214517116575052568805404007434844970703125f,0.42990005204677572692162357270717620849609375f,0.332700010496377984736682265065610408782958984375f,0.199099951350689252649317495524883270263671875f,0.769800010514259280824944653431884944438934326171875f,0.35680001072287570451635474455542862415313720703125f,-0.461600002871453529706968765822239220142364501953125f,-0.0460000100135804501633174368180334568023681640625f,0.0130000004246830780374466485227458178997039794921875f,0.127400004139542610825941437724395655095577239990234375f,-0.149200005996227347271343433021684177219867706298828125f,0.04839999858140942023965180851519107818603515625f,0.009399990601837604486945565440692007541656494140625f,0.5414000144660473523572363774292171001434326171875f,-0.1563000020653004806803210158250294625759124755859375f,0.5991000005245208370752152404747903347015380859375f,0.5745000279620293692772747817798517644405364990234375f,0.02309998864829532205789064391865395009517669677734375f,-0.650899984797835173822022625245153903961181640625f,-0.2383000054776667209210927467211149632930755615234375f,0.62409998539984179455331059216405265033245086669921875f,0.42350002163648614583735252381302416324615478515625f,-0.195599999786913392174625414554611779749393463134765625f,-0.117600000107288327200194544275291264057159423828125f,-0.12810001051574959607393111582496203482151031494140625f,0.6321000161111356430154728514025919139385223388671875f,0.46340000517517321299010291113518178462982177734375f,-0.0133999929189680955232688575051724910736083984375f,0.476800019626319571131034535937942564487457275390625f,0.045200001108646414849090433563105762004852294921875f,0.519699992115795428304636516259051859378814697265625f,0.373100015246868199625396300689317286014556884765625f,-0.0166999959379434681494558390113525092601776123046875f,-0.4621000044882297430604012333787977695465087890625f,-0.34290001026392002980713868964812718331813812255859375f,0.505800023955107036499612149782478809356689453125f,-0.97900001159310345855146806570701301097869873046875f,1.00740001568794212971624801866710186004638671875f,-0.31000000579282638302203167768311686813831329345703125f,0.4502000099189580328840065703843720257282257080078125f,-0.3673000020205974269771331819356419146060943603515625f,0.3030999922990798012278901296667754650115966796875f,-0.73870001916885374981802669935859739780426025390625f,0.50550004300475137597459251992404460906982421875f,-0.614999983310699604999172152020037174224853515625f,-0.3444999544620515052884002216160297393798828125f,0.271700034792721145660010506617254577577114105224609375f,0.917399992851912760016830361564643681049346923828125f,-0.69089997526109225844948014128021895885467529296875f,0.2629999722987417243302843417041003704071044921875f,0.432499974817037724506008089520037174224853515625f,-0.15939998236894581395972636528313159942626953125f,-0.073500003010034564709940241300500929355621337890625f,-0.11990000123381605590111576020717620849609375f,-0.44029998802840708549410919658839702606201171875f,-0.7251999868780369951082320767454802989959716796875f,1.25090001497268676899921047152020037174224853515625f,-1.47810000630617111028186627663671970367431640625f,0.0328000030040740409020827428321354091167449951171875f,1.1328000129878521562432069913484156131744384765625f,0.42659998890161521245545372948981821537017822265625f,-0.323699982064962199501678696833550930023193359375f,0.7592999967157840757181475055404007434844970703125f,-0.7406000127136707078534527681767940521240234375f,-0.368400007745623536692392008262686431407928466796875f,-0.368499999284744195193752602790482342243194580078125f,-0.4011999957740304267872488708235323429107666015625f,-0.3908000182926656407289556227624416351318359375f,-0.1955000067576764255505850087502039968967437744140625f,0.46909999694824211502464095246978104114532470703125f,-0.27340001374334110817443388441461138427257537841796875f,0.23380000884532936566984062665142118930816650390625f,0.522900004191697131972205170313827693462371826171875f,0.312299981904029522183918743394315242767333984375f,0.362300022621452999516122872591949999332427978515625f,-0.787100012981891694607838871888816356658935546875f,-0.381799992692470724353626110314507968723773956298828125f,-0.230199999916553377232730781543068587779998779296875f,-0.6583999979108570155261759282439015805721282958984375f,-0.151600008682906628454389874605112709105014801025390625f,-0.157800024461746790649385729921050369739532470703125f,0.3234000265955927488903398625552654266357421875f,0.2297999942347408453002799433306790888309478759765625f,0.4001999929130075628336271620355546474456787109375f,-0.3523000390499830825064009331981651484966278076171875f,0.463899999713897503994530779891647398471832275390625f,0.379500003747641978524285377716296352446079254150390625f,-0.005099995636939969045897669275291264057159423828125f,-0.64850001571327453575577237643301486968994140625f,-0.0283000100970269841127446852624416351318359375f,0.34840000290274597549711188548826612532138824462890625f,-0.24039999603331085875623784886556677520275115966796875f,-0.063699998903274490658077411353588104248046875f,0.4057000096917153797448918339796364307403564453125f,0.7774999912828206571902001087437383830547332763671875f,0.1955999939382078167682266212068498134613037109375f,-0.3777999991297722459648866788484156131744384765625f,0.57699999037384941402706317603588104248046875f,-0.716599980109930090321768147987313568592071533203125f,0.051500006973743239768737112171947956085205078125f,-0.21000000372529026293477727449499070644378662109375f,-0.780999993026256333905621431767940521240234375f,0.139400023645162463026281329803168773651123046875f,0.4515000174045553649193607270717620849609375f,1.165099995338916727405376150272786617279052734375f,0.0897999890565870373393408954143524169921875f,-0.56749999433755871081075383699499070644378662109375f,0.707099966043234218204815988428890705108642578125f,-0.803799989157170102771488018333911895751953125f,0.307399984376877544178796597407199442386627197265625f,0.0964999767243863715293628047220408916473388671875f,-1.3360999931335442170166061259806156158447265625f,-0.2325999811828136643043762887828052043914794921875f,0.2875999981701369279107893817126750946044921875f,-0.5325000193715094809476795489899814128875732421875f,-0.773599976891279794699585181660950183868408203125f,0.6039999995231628560077297152020037174224853515625f,-0.291299997967481516525367624126374721527099609375f,-1.318700016486644699398311786353588104248046875f,1.21199999201297714535030536353588104248046875f,0.550999984346330240470024364185519516468048095703125f,0.24419999053627261531573822139762341976165771484375f,-0.22929998151659969352067491854541003704071044921875f,-0.53299999129772146488903672434389591217041015625f,0.21929998360276226065934679354541003704071044921875f,0.4051000105381010740757119492627680301666259765625f,0.264200000073015583978985887370072305202484130859375f,0.214600007617473664822682621888816356658935546875f,-0.4090000084638589772367822661180980503559112548828125f,0.545799988639354527464320199214853346347808837890625f,-0.063400009980797589204115638494840823113918304443359375f,0.5350000023469327903313796923612244427204132080078125f,-0.22219999494701625497583563628722913563251495361328125f,-0.19899999937415113748784278868697583675384521484375f,0.4586000141814354780223084162571467459201812744140625f,-0.195800003242492692834275658242404460906982421875f,0.25570000134706483141400212844018824398517608642578125f,0.0359000048398971927099410095252096652984619140625f,0.429000003173947153101153162424452602863311767578125f,-0.06179999082982534641672600628226064145565032958984375f,0.5190000004917381115632224464206956326961517333984375f,-0.30299999874830252366564309340901672840118408203125f,0.11859999928027387117168700569891370832920074462890625f,0.07940000539124014711234167407383210957050323486328125f,0.173200013975799260723675843109958805143833160400390625f,-0.356800009381771143779360500047914683818817138671875f,-0.23769999675005670791705369992996565997600555419921875f,0.64810001405477546398969934671185910701751708984375f,0.4411000086307523826434362490545026957988739013671875f,-0.329499999165534962486390213598497211933135986328125f,0.5975000316277150336219392556813545525074005126953125f,0.0144999966323375861776412421022541821002960205078125f,-0.4709000386655333159069414250552654266357421875f,0.395300004494190115877927382825873792171478271484375f,0.00309999118149306696068379096686840057373046875f,0.142100058579444521456025540828704833984375f,0.1794000039756298292559222318232059478759765625f,-0.1347999982953069775248877704143524169921875f,-0.88359997301697745797355310060083866119384765625f,-0.325399972265959291917170048691332340240478515625f,-0.2952999870039523244713564054109156131744384765625f,-0.662299973615259052195369804394431412220001220703125f,0.0521000043392181311219246708787977695465087890625f,-0.29690001142621014196265605278313159942626953125f,1.28460000717043865137156899436376988887786865234375f,-1.17599997365474706612076261080801486968994140625f,0.213499997049570122698014529305510222911834716796875f,0.148399996197223771332573960535228252410888671875f,0.342599975483119545405230610413127578794956207275390625f,0.544400013847649244524973255465738475322723388671875f,0.663899968868494116946976646431721746921539306640625f,0.593700003969669154457733384333550930023193359375f,-0.051500012077391132958581465572933666408061981201171875f,0.268300014604628156433818730874918401241302490234375f,0.464300004762411067105176698532886803150177001953125f,0.2066999930322168665952631272375583648681640625f,-0.809899968898296407360248849727213382720947265625f,-0.23959999569654399920182186178863048553466796875f,0.533499987065792158347221629810519516468048095703125f,0.72659998130202296806601225398480892181396484375f,0.23930001884400831091426198327098973095417022705078125f,1.07040001570284371013030977337621152400970458984375f,-0.622000019282102467599315787083469331264495849609375f,-0.75249999359250097086260211654007434844970703125f,0.64880001927614205925465284963138401508331298828125f,-0.52350001746416108261428234982304275035858154296875f,-0.480600026646256839324422571735340170562267303466796875f,0.14940000561475752505202763131819665431976318359375f,0.0309999835640193399655117900692857801914215087890625f,0.213800001096725367233375436626374721527099609375f,-0.35580002738237437309720689881942234933376312255859375f,0.2910000164210797546360254273167811334133148193359375f,0.56700001402944355533009002101607620716094970703125f,0.221400007164478385135453208931721746921539306640625f,0.117900006240606369800616448628716170787811279296875f,-0.398700007806718381697663744489545933902263641357421875f,0.0347999987050890968731664543156512081623077392578125f,0.3143999980151652717808019588119350373744964599609375f,0.4565000177025797523100436592358164489269256591796875f,0.209499992609023966139147887588478624820709228515625f,-0.3791999976143241024573171671363525092601776123046875f,-0.5028000140309334398125429288484156131744384765625f,0.3025000252947209755660651353537105023860931396484375f,-0.4104999942779539168213887023739516735076904296875f,0.23409999683648319290796280256472527980804443359375f,0.669500025279820221157933701761066913604736328125f,0.1663999980375170384405691947904415428638458251953125f,-0.2348000014245510413957163109444081783294677734375f,0.4370999967396256291607414823374710977077484130859375f,-0.2678999934256076631555743006174452602863311767578125f,-0.50449999476969209499799262630403973162174224853515625f,-0.15430001385211955522436255705542862415313720703125f,-0.155899998357892144440484116785228252410888671875f,-0.30850000107288355621903974679298698902130126953125f,-0.59550000347942155798364183283410966396331787109375f,1.33479999412298155903044971637427806854248046875f,-0.7675000227987764933601511074812151491641998291015625f,0.0690999918818473446435746154747903347015380859375f,0.41969998999238011805346104665659368038177490234375f,-1.30249996453523664285967242904007434844970703125f,-0.487500020936131317483841485227458178997039794921875f,-0.5133000062972303823016773094423115253448486328125f,0.79080001426935186970013091922737658023834228515625f,-0.677399972623586421605068608187139034271240234375f,0.9432000097475945477043524078908376395702362060546875f,-0.9161000182233751676363908700295723974704742431640625f,-0.61059998730719089454765935442992486059665679931640625f,0.77619999286830410056836626608856022357940673828125f,-0.1116999723196030469551942587713710963726043701171875f,1.0463999950945375161381889483891427516937255859375f,0.128600011461973195991959073580801486968994140625f,-0.2458000410914422673158696852624416351318359375f,0.2593999947369098890703753568232059478759765625f,1.1286999810218816264750785194337368011474609375f,-0.344099997395276968603639033972285687923431396484375f,-0.02919998502284215646795928478240966796875f,0.750799991768598573571580345742404460906982421875f,-0.70759999936223039185279048979282379150390625f,0.87159997228682051240156170024420134723186492919921875f,-0.16529999980032439310662084608338773250579833984375f,-0.89739999810457238282879188773222267627716064453125f,0.940900004541873613561620004475116729736328125f,0.236299997669458416993393257143907248973846435546875f,-0.290000017881393290508640347979962825775146484375f,-1.0872999857783316457471300964243710041046142578125f,0.3398000277996064966146150254644453525543212890625f,0.34749999321997127044170383669552393257617950439453125f,-0.12069999397099018256795943671022541821002960205078125f,0.54030002006590371887995161159778945147991180419921875f,-0.17669999824762339102335317875258624553680419921875f,0.67419999657124218828130324254743754863739013671875f,-0.0729999808669088423584980773739516735076904296875f,-0.2714000288084148504452741690329276025295257568359375f,0.6710000028610227928993481327779591083526611328125f,0.0826000049501656175010566585115157067775726318359375f,-0.37060000511407853451828486868180334568023681640625f,0.19350000800192368455299174456740729510784149169921875f,-0.0459000027537346255712691345252096652984619140625f,-0.04049999390542513566515481215901672840118408203125f,-0.1646999994546174406195859774015843868255615234375f,-0.741500021837651956957415677607059478759765625f,-0.4105000088810921710091861314140260219573974609375f,-0.268099994795024054727861084757023490965366363525390625f,0.361699999141693151472054523765109479427337646484375f,0.130199999730289006638628279688418842852115631103515625f,0.161400000235438323414882688666693866252899169921875f,0.12349999902397390538766330791986547410488128662109375f,0.10330000573843722833089486812241375446319580078125f,-0.6754999966621397788912872783839702606201171875f,-0.01210000344514838843679171986877918243408203125f,0.1876000141143798760623440102790482342243194580078125f,-0.0291999964594840122344976407475769519805908203125f,-0.48830003311932135279249678205815143883228302001953125f,0.29750001698732386756773848901502788066864013671875f,0.336900006248056715918437475920654833316802978515625f,0.7774000329732899672308121807873249053955078125f,0.388999976910650857586659867592970840632915496826171875f,0.344899989126622674717737027094699442386627197265625f,0.5412999997556209308413599501363933086395263671875f,0.65360000237226500985343591310083866119384765625f,1.21889998570680635481267017894424498081207275390625f,-0.70410002198219245883592520840466022491455078125f,-0.9532000166952609987447431194595992565155029296875f,-0.06299995434284255679813213646411895751953125f,0.436900000995397608249959375825710594654083251953125f,0.049499971449375124166181194595992565155029296875f,0.045099998468160595876952356775291264057159423828125f,0.526600008273124586821722914464771747589111328125f,0.00720000057220460121243377216160297393798828125f,0.22549998779594915276902611367404460906982421875f,-0.32150002038478842081303810118697583675384521484375f,-0.43500002443790464212725055404007434844970703125f,-0.588899974232912182259269684436731040477752685546875f,-0.20019998427033414145626011304557323455810546875f,-0.02699999380111695046480235760100185871124267578125f,0.070400001844763693270579096861183643341064453125f,0.226899999207258229461103837820701301097869873046875f,-0.38639999207705244543831213377416133880615234375f,-0.869899982458353093761616037227213382720947265625f,0.108999985218048323076800443232059478759765625f,-0.11219999729394913856594939716160297393798828125f,0.8233999967932703611950273625552654266357421875f,-0.46550002840161308625965830287896096706390380859375f,-0.9377999925732609654005500487983226776123046875f,0.08710000165700915175648333388380706310272216796875f,0.1758000109910966557436040602624416351318359375f,-0.010399998082220474060477499733678996562957763671875f,-0.0724000071942807466029989882372319698333740234375f,-0.4723999988868829991162101578083820641040802001953125f,0.2311999863862990167717725853435695171356201171875f,-0.4850000083819037133281426577013917267322540283203125f,0.0638000189781191551219308166764676570892333984375f,-0.235999996304511849842810988775454461574554443359375f,-0.340000007748603838564349644002504646778106689453125f,-0.1826000025659799685939788105315528810024261474609375f,0.3266000032067299940763405174948275089263916015625f,0.4702000023752450630354360328055918216705322265625f,0.4658000157594681667205804842524230480194091796875f,0.0545000038221477023370198367047123610973358154296875f,-0.52970000981092457692511743516661226749420166015625f,0.564600010970235377527615128201432526111602783203125f,-0.0606000146508218762164688087068498134613037109375f,-0.218400003536045550500688250394887290894985198974609375f,-0.06600000315904619441198519780300557613372802734375f,-0.68230000868886708786931194481439888477325439453125f,-0.066500011444091899903696685214526951313018798828125f,-0.06970001498907840709051697558606974780559539794921875f,0.65290002069473285217782176914624869823455810546875f,-0.33030000631958245094210724346339702606201171875f,0.48150000548362736907392900320701301097869873046875f,0.72119998646080463178265063106664456427097320556640625f,0.09240000972747797192141661071218550205230712890625f,0.294900024479627820284122208249755203723907470703125f,-0.47850001209974291072057894780300557613372802734375f,0.23000000976026058197021484375f,0.33749998450279239392557428800500929355621337890625f,-0.6996000220343472353334846047800965607166290283203125f,0.2418000132560731429975930950604379177093505859375f,-0.06660000221952788734824935090728104114532470703125f,0.6617999881848692922403643024154007434844970703125f,0.597000029750168526021525394753552973270416259765625f,-0.1353000027060513588139656349085271358489990234375f,0.1656000046670438319296181362005881965160369873046875f,-0.62700002293288725763176216787542216479778289794921875f,-0.0873000067144633629112604467081837356090545654296875f,-0.69299998265504836325590076739899814128875732421875f,-0.474300022941827936762138051562942564487457275390625f,-0.3813000047475096909721514748525805771350860595703125f,0.7056000246718527879608018338331021368503570556640625f,-0.282600017094612354640048579312860965728759765625f,-0.6155999857798217789195405202917754650115966796875f,-0.9983999714612963316540117375552654266357421875f,0.4521000025883317174901776525075547397136688232421875f,0.6749999913573265786226329510100185871124267578125f,-0.38550000713020560194621566552086733281612396240234375f,0.4931999915122986521964776329696178436279296875f,0.2007000020921230198922557974583469331264495849609375f,-0.5751000041306018051301407467690296471118927001953125f,-0.2546999985605478133976475874078460037708282470703125f,0.181200009054690525100994591412018053233623504638671875f,-0.80640003033578422364513471620739437639713287353515625f,-0.7211999901674683854935210547409951686859130859375f,-0.517500001005828380584716796875f,-0.8372999819040298774552866234444081783294677734375f,0.3483000094637274113296143696061335504055023193359375f,-0.28190000971257678230585952405817806720733642578125f,-0.4685999919414520764604503710870631039142608642578125f,-0.647600016200542416555663294275291264057159423828125f,-0.3204999796748158047421384253539144992828369140625f,-0.2319000097870826948565081693232059478759765625f,0.46939997231066232785678948857821524143218994140625f,0.64920000472962813287125527494936250150203704833984375f,0.5088000010967252961791018606163561344146728515625f,0.002599997574090906482524587772786617279052734375f,-0.2999000118881465226650107069872319698333740234375f,-0.33890001662671587201458578419988043606281280517578125f,-1.376799994778632907355131465010344982147216796875f,-0.75780004696250014006864148541353642940521240234375f,-1.11739999214410801187113975174725055694580078125f,-0.4896000082135198994137681438587605953216552734375f,-0.2960999949216844839838813641108572483062744140625f,0.51390002266168632871767840697430074214935302734375f,0.33659998726248741007793796597979962825775146484375f,0.2914000037372110885058873464004136621952056884765625f,0.24259996598362931763404048979282379150390625f,0.5923999933362000813730219306307844817638397216796875f,-0.040499972522258786966631305404007434844970703125f,-0.274099986517428728660661363392136991024017333984375f,-0.2375999739371239538598956642090342938899993896484375f,0.221600005535036448378605200559832155704498291015625f,0.02140001699924454214851721189916133880615234375f,-0.5255999966204161211180689861066639423370361328125f,0.125000007450580596923828125f,-0.2743999938428398621681481017731130123138427734375f,0.36609998702406887360893961158581078052520751953125f,-0.0272999927818775578458598829456605017185211181640625f,-0.66060000243186944857143316767178475856781005859375f,-0.2102999938398599777400477250921539962291717529296875f,-0.5427000153020025141614723906968720257282257080078125f,-0.5625f,0.73559999740272752344338869079365395009517669677734375f,0.29369998884499093350086695863865315914154052734375f,0.3336000025212764175108759445720352232456207275390625f,0.8409000194430351626806441345252096652984619140625f,-0.0693000002548098681387500619166530668735504150390625f,0.153300006893277274144793409504927694797515869140625f,-0.522000020250678264943644535378552973270416259765625f,-0.7208999777197835356901123304851353168487548828125f,-0.65999999094754446815613846410997211933135986328125f,-1.0406999983668328013663995079696178436279296875f,0.6747000193476677676329700261703692376613616943359375f,-0.2747999929308893030110994004644453525543212890625f,0.42060000019669530058052941967616789042949676513671875f,0.34979999695420271876855622394941747188568115234375f,0.905399993053078500082619939348660409450531005859375f,0.363299995940923992066018399782478809356689453125f,-0.468899993008375037817359043401665985584259033203125f,-0.1589999989271166924709177692420780658721923828125f,0.03299999969825140322399192882585339248180389404296875f,-0.51179998490661393617529029143042862415313720703125f,-0.40379999223053442136688317987136542797088623046875f,0.347700007665157073688533273525536060333251953125f,-0.382199997442960626159447201644070446491241455078125f,0.09149999207258208144821765017695724964141845703125f,0.25169999578893176295224520799820311367511749267578125f,-0.27740000637471673439193864396656863391399383544921875f,-1.3599999824166300044225863530300557613372802734375f,-0.6416000105082986237903241999447345733642578125f,-1.48580000711679449665325591922737658023834228515625f,-0.8866000316679478743253639549948275089263916015625f,0.66530000687837542727720574475824832916259765625f,0.34690002722144175351104422588832676410675048828125f,-1.3693999770045284236630322993732988834381103515625f,-0.77780001030564260844357704627327620983123779296875f,-0.2785999970078467669054589350707828998565673828125f,-0.432800022673607287515551433898508548736572265625f,-0.0831000176310539018231793306767940521240234375f,0.72070002310276048973491924698464572429656982421875f,-0.72669998573064820135414265678264200687408447265625f,-0.5221000071704384737358850543387234210968017578125f,-0.3695000033378601500544391456060111522674560546875f,0.59030002967715322625963381142355501651763916015625f,0.5859999864697460481011148658581078052520751953125f,1.10180001057386345308941599796526134014129638671875f,-0.34320000071376544070744785130955278873443603515625f,-0.298400013221800375351477896401775069534778594970703125f,-0.3671000049911439244709754348150454461574554443359375f,-0.272300010421127058890533589874394237995147705078125f,-0.015699995982647152459321659989655017852783203125f,-0.4579000120520593242190443561412394046783447265625f,-0.97519997398853330849988196860067546367645263671875f,-0.666400007015466311344198402366600930690765380859375f,1.082899968987703687162138521671295166015625f,0.89410000652223764472381617451901547610759735107421875f,-0.49000002443790435791015625f,-0.288599991121888166389908292330801486968994140625f,-0.8285999901160596703419969344395212829113006591796875f,-1.2269999920129777137844939716160297393798828125f,-0.71880000709444280548865435775951482355594635009765625f,-0.85140000299215312651313070091418921947479248046875f,0.67830002913326037372598875663243234157562255859375f,0.8604000215888021330101764760911464691162109375f,-0.5651999828919767043799993189168162643909454345703125f,-0.733199974822998257195649784989655017852783203125f,-0.7452000121727586634534645781968720257282257080078125f,-0.4532999981760976737632518052123486995697021484375f,0.85590001068860299682228287565521895885467529296875f,-0.1833000169515610622283929842524230480194091796875f,-0.40709999286532394524584788086940534412860870361328125f,-0.3470999873518945122441436978988349437713623046875f,0.8853000182032586540259444518596865236759185791015625f,-0.21750000834465055277178180404007434844970703125f,0.8561999848589298967027616527047939598560333251953125f,-0.1395000143647191492846104665659368038177490234375f,-0.1404000092580914849094142482499592006206512451171875f,-0.02759999977201221810219067265279591083526611328125f,0.06569999845996499221456588202272541821002960205078125f,0.145200005020201194927409460433409549295902252197265625f,-0.659099998699128608592445743852294981479644775390625f,-0.07709999361038200049733859486877918243408203125f,-0.74699998795241107085729481696034781634807586669921875f,-0.74459998168945329410917111090384423732757568359375f,0.86549998556077467615210707663209177553653717041015625f,0.6371999802134933332808941486291587352752685546875f,0.5832000081270933833366143517196178436279296875f,0.4692000273048879677162403822876513004302978515625f,-0.6195999694705012217355033499188721179962158203125f,-1.041599999034403811748461521347053349018096923828125f,0.646800005507468966925443965010344982147216796875f,0.871000041902065813559374873875640332698822021484375f,-0.198799996626377151187625713646411895751953125f,0.134000012040138560820423663244582712650299072265625f,0.1015000168085098408710109652020037174224853515625f,-0.36430000744462009976132321753539144992828369140625f,-0.60500000327825542711934758699499070644378662109375f,-0.0368000022292136463875067420303821563720703125f,0.3513999758720398602918066899292171001434326171875f,0.12699999201297718087744215154089033603668212890625f,-0.17340000416934486793252290226519107818603515625f,0.191400006046891224542605414171703159809112548828125f,-1.236500016510486688048331416212022304534912109375f,-0.5877000242054464251850731670856475830078125f,0.4440999967060983433242427054210565984249114990234375f,-0.302100005326420006923626715433783829212188720703125f,1.2553999929785728539854972041212022304534912109375f,0.561200016337632945351288071833550930023193359375f,0.7359999927282334653000361868180334568023681640625f,1.000800038558244953179610092774964869022369384765625f,-0.6127000134766102945604870910756289958953857421875f,-0.03150000801682484308230414171703159809112548828125f,0.6407000055193901033590009319595992565155029296875f,0.504700019198656146812709266669116914272308349609375f,0.152199987459182839444338242174126207828521728515625f,-0.51560001390576370994267563219182193279266357421875f,-0.92440002766847584325660136528313159942626953125f,0.7730999836564063087962495046667754650115966796875f,-0.7895999841481444736501771330949850380420684814453125f,0.09420001493692353733422351069748401641845703125f,0.877800021891296022857886782730929553508758544921875f,0.2858999976873395354459717054851353168487548828125f,0.2136000085562468342459396808408200740814208984375f,-0.1404000019192697124026381061412394046783447265625f,-0.5373000051498413309758461764431558549404144287109375f,0.0285000082254409647930515347979962825775146484375f,0.07680000181943180681543026366853155195713043212890625f,-0.41250000238418582654276178800500929355621337890625f,0.2309999928027390847518063310417346656322479248046875f,0.3941999907970430427894825697876513004302978515625f,0.05820000209212305986028468396398238837718963623046875f,-0.3792000097215175724585378702613525092601776123046875f,-0.362699999208748291579240685678087174892425537109375f,-0.666000001966953192322762333787977695465087890625f,-0.05610000414177773730983744826517067849636077880859375f,0.79530000147670509846875575021840631961822509765625f,0.2645999984532594329067478611250407993793487548828125f,0.61169998930692681682330658077262341976165771484375f,0.6798000074967742722975572178256697952747344970703125f,-0.0510000275969506589035518118180334568023681640625f,0.260100004801154138789343051030300557613372802734375f,-0.219599995994567809276531988871283829212188720703125f,0.2037000034108757962059144119848497211933135986328125f,0.06359999432563778754001759807579219341278076171875f,-0.399000008761882740682125358944176696240901947021484375f,0.458099989169836074864861075184307992458343505859375f,-0.5826000225707890223247886751778423786163330078125f,0.4981000150978565077508619651780463755130767822265625f,-0.0518999829091131292813088293769396841526031494140625f,0.2405000084899366896706851548515260219573974609375f,-0.37700000313296921916617066017352044582366943359375f,-0.3382000151120128084158977799233980476856231689453125f,1.3954999784827233355599673814140260219573974609375f,0.63130001033544491662041764357127249240875244140625f,-0.86950000251829617781140768784098327159881591796875f,-0.097300005969405134464977891184389591217041015625f,-1.36729998399019248012109528644941747188568115234375f,-0.64810001271367045916349525214172899723052978515625f,0.1684000113405288434620388215989805757999420166015625f,-0.200200001164525520636061628465540707111358642578125f,0.037699982929229491901423898525536060333251953125f,-0.112300004106760109578999617951922118663787841796875f,-0.8364999923706051276894868351519107818603515625f,-1.0375000350177288055419921875f,-0.6940999731063843825040748924948275089263916015625f,-1.08570001296997009632150366087444126605987548828125f,1.41560002865791290105335065163671970367431640625f,0.56600004070997300686940434388816356658935546875f,0.094000003695487777122252737171947956085205078125f,-0.66080001888871198190145150874741375446319580078125f,-0.6901999844193458244490102515555918216705322265625f,-0.2106000204622748839256018982268869876861572265625f,-0.5831999988138676371818291954696178436279296875f,-0.24640001178383830193752146442420780658721923828125f,0.44000001169741143058899979223497211933135986328125f,-0.08360000676810752651135771884582936763763427734375f,1.3090999859213834355387007235549390316009521484375f,0.231899990713596348967939775320701301097869873046875f,-0.01229999905899170631329297975753434002399444580078125f,-0.421199995066225574280593946241424418985843658447265625f,-0.210000001396983881907232216690317727625370025634765625f,-0.07799999228492371372567504295147955417633056640625f,0.2876999918326734739792982509243302047252655029296875f,0.661499979853630293291644193232059478759765625f,0.0045000005811453025472701483522541821002960205078125f,-0.6465000020563602678436154747032560408115386962890625f,-0.68789999540001145472700727623305283486843109130859375f,-1.06439999339580548820549665833823382854461669921875f,-0.00360000458881259977061972676892764866352081298828125f,0.449700007165968307365488954019383527338504791259765625f,0.192300007943809081467634314321912825107574462890625f,0.4206000030279160029067497816868126392364501953125f,-0.71970002046525483052619165391661226749420166015625f,-0.0314999941587448262225734652020037174224853515625f,-0.8324999962002037623420847012312151491641998291015625f,0.0432000168442723264661253779195249080657958984375f,0.384000019267201508910147822462022304534912109375f,0.8868000132560727166719516390003263950347900390625f,-0.3654000007644295777708975947462022304534912109375f,0.66000001966953281140604303800500929355621337890625f,-0.3684000082299114797734773674164898693561553955078125f,-0.735599980378150775095491553656756877899169921875f,-0.0047999988913536295598305514431558549404144287109375f,-0.171599987709522228129799259477294981479644775390625f,0.05519999775588502188838901929557323455810546875f,0.5910000232756138505152421203092671930789947509765625f,0.4436999772593377411311621472123079001903533935546875f,-0.941300047439337372878753740224055945873260498046875f,-0.25219996808767319862454314716160297393798828125f,-1.11440002528429005224097636528313159942626953125f,-0.08449999469518676420420888462103903293609619140625f,-0.62990000317096672688421676866710186004638671875f,0.925999980509281339635663243825547397136688232421875f,0.881000014632939976166881024255417287349700927734375f,0.0072000043720006345893125399015843868255615234375f,-0.00739999266564816959856898392899893224239349365234375f,-0.61799999997764842873948509804904460906982421875f,-0.6030000188276172234935756932827644050121307373046875f,-0.54749996215105090069386051254696212708950042724609375f,-0.9317000032395117425920716414111666381359100341796875f,-0.764800004553794821049450547434389591217041015625f,-0.51740002418160457864360068924725055694580078125f,1.48989998439550408448894813773222267627716064453125f,0.72190001321434937864296443876810371875762939453125f,0.048299972546100633508103783242404460906982421875f,0.66430001042485198325948658748529851436614990234375f,-0.153900014317035260802413176861591637134552001953125f,-0.7791000441849238455915838130749762058258056640625f,-1.366000005245208370752152404747903347015380859375f,-0.7756000513076788394073446397669613361358642578125f,-1.2355999818682672497516250587068498134613037109375f,-0.8664000123798842878386494703590869903564453125f,-0.043699992458522185501124113216064870357513427734375f,-0.3159000053986904532621338148601353168487548828125f,0.867399988269805799490086428704671561717987060546875f,0.44000002622604394986183251603506505489349365234375f,-0.16020002167224856037819336052052676677703857421875f,-0.228599987173080165803185082040727138519287109375f,-0.87600000833719970927404574467800557613372802734375f,0.441900029456615328626867267303168773651123046875f,-0.45149999389797434989901603330508805811405181884765625f,0.35250000208616238950298793497495353221893310546875f,0.8747999944210052003512601004331372678279876708984375f,0.523499987214803841339971768320538103580474853515625f,0.19260000625401740848019471741281449794769287109375f,0.236100004085898262928822077810764312744140625f,-0.27839999750107524700837302589206956326961517333984375f,0.01950001492351294274385509197600185871124267578125f,-0.866699986177682735188909646240063011646270751953125f,-0.09149997445195945378060287112020887434482574462890625f,-0.284400006732344667881307032075710594654083251953125f,-0.24989999274015417540795169770717620849609375f,0.60569999348670233718650024457019753754138946533203125f,0.97319999091625231102398174698464572429656982421875f,0.315300002445280558305285012465901672840118408203125f,-0.5916000264525411722615899634547531604766845703125f,-0.6633000320389867976444975283811800181865692138671875f,0.271799996119737929944903953582979738712310791015625f,-0.447600026929378724815933310310356318950653076171875f,-0.59999998688697786519696819595992565155029296875f,-0.7721999908491967712365067200153134763240814208984375f,-0.7127999886989595523800744558684527873992919921875f,0.037499994970858097076416015625f,0.47490000741779792026164841445279307663440704345703125f,0.370800011028349529862424560633371584117412567138671875f,0.557699981886148332677066719043068587779998779296875f,-0.73500002436339861855429944625939242541790008544921875f,-0.1686000069171189608141503413207828998565673828125f,-1.391699953544140111461047126795165240764617919921875f,-0.77030001373290968302853798377327620983123779296875f,-0.9303999713718891229063956416212022304534912109375f,-0.3242000059962268920799033367075026035308837890625f,-0.547500016614794748903705112752504646778106689453125f,0.1509000050634146106176558532752096652984619140625f,0.8480999937891962048297500587068498134613037109375f,0.55230000365972475862008650437928736209869384765625f,1.165600000643730371763240327709354460239410400390625f,0.58360000356435737245419659302569925785064697265625f,0.7657999993681905426257117142085917294025421142578125f,0.58200003582239201449510801467113196849822998046875f,-0.83039998016357419174937604111619293689727783203125f,-0.31980001811385161403222809894941747188568115234375f,0.7146000134289263172604478313587605953216552734375f,-0.15299999666213981441842406638897955417633056640625f,1.0284999557733538466663958388380706310272216796875f,0.45689998981952584244936588220298290252685546875f,1.1705999714374542008954449556767940521240234375f,0.4058000136733053153648143052123486995697021484375f,-0.375899975037575106995291207567788660526275634765625f,-0.91530000404715483597328784526325762271881103515625f,-0.99559999945163735191044906969182193279266357421875f,-0.50320001833438876559512209496460855007171630859375f,0.1328999989762902345091788447462022304534912109375f,0.206900009451806699001252809466677717864513397216796875f,0.481499977171421011945540158194489777088165283203125f,0.00470000399947156921598434564657509326934814453125f,0.541499993860721673399893916212022304534912109375f,-0.5205000048577785864978295649052597582340240478515625f,-0.6914999903216958454521545718307606875896453857421875f,-0.908099959367513864805232515209354460239410400390625f,-0.098399996942281742207114803022705018520355224609375f,-0.479699974942207507666580568184144794940948486328125f,0.170700006674230186920482310597435571253299713134765625f,-0.5340000162124634019988889122032560408115386962890625f,0.2942999884083865591577477971441112458705902099609375f,0.2852999940633775821652307058684527873992919921875f,0.521999986276030281118210041313432157039642333984375f,0.787800006133318131418263874365948140621185302734375f,0.60420001970529568779966211877763271331787109375f,0.5168999922782180345137703625368885695934295654296875f,-0.21840000498890888014358324653585441410541534423828125f,-0.53069997541904445625959851895458996295928955078125f,-0.3095999981179833326905281865037977695465087890625f,0.532800009262561768963450958835892379283905029296875f,0.5540999809294937517023527107085101306438446044921875f,0.8879999778866771720231554354541003704071044921875f,0.31260000077784066974118104553781449794769287109375f,0.8207999752283097194549554842524230480194091796875f,-0.9530999872326848532111398526467382907867431640625f,-0.056699986177683125987414314295165240764617919921875f,-0.33119999882876871577508381960797123610973358154296875f,-0.53579999102354047835206074523739516735076904296875f,0.30930000360757115007714901366853155195713043212890625f,0.163199998106062338365518371574580669403076171875f,-0.060299998645484353954060452451813034713268280029296875f,0.365999978274107018449967654305510222911834716796875f,0.09990000235140339412964749499224126338958740234375f,0.258900017446279662181041203439235687255859375f,-0.603000009775161771585771930404007434844970703125f,-0.075600013160705969994523911736905574798583984375f,0.096999986052512809919790015555918216705322265625f,0.65100001984834676704849698580801486968994140625f,-0.115600000643730549398924267734400928020477294921875f,0.77200004625320506335128811770118772983551025390625f,0.47300000232458128124335416941903531551361083984375f,-0.27400001859664957493123438325710594654083251953125f,-1.148799965929985233970000990666449069976806640625f,-0.54979999516606259390982813783921301364898681640625f,0.2026999608755115644953548326157033443450927734375f,0.8906999989628783254147492698393762111663818359375f,-0.29670000122785555873861085274256765842437744140625f,-0.851900029307604000905485008843243122100830078125f,0.779500012874603243062665569595992565155029296875f,0.29710000821352000599517850787378847599029541015625f,-1.132600019180774353344531846232712268829345703125f,-0.743800044906139934397515389719046652317047119140625f,0.345200017350911825264603294272092171013355255126953125f,0.74200003742426690056532834205427207052707672119140625f,0.2787999765284359110495415734476409852504730224609375f,-0.156400000738352584761514663114212453365325927734375f,0.239499983668327587338353623636066913604736328125f,-0.511300015103817173667266615666449069976806640625f,0.3402999755859372754684954998083412647247314453125f,-0.42970001189708728617233646218664944171905517578125f,-1.0772999991893765781014735694043338298797607421875f,-0.37850002878904387415559540386311709880828857421875f,-0.5695000081062318031399627216160297393798828125f,0.725999982446432046145901040290482342243194580078125f,-0.35160000272244218422201811335980892181396484375f,-0.83850000613927822001869571977294981479644775390625f,0.825000019222497993354181744507513940334320068359375f,0.0510000016689300395000827847979962825775146484375f,0.6972000345468523097025581591879017651081085205078125f,-0.469800012153387402946691508986987173557281494140625f,-0.37200001633912338494525329224416054785251617431640625f,0.42150000667572040669028865522705018520355224609375f,-0.664799985033273532764042101916857063770294189453125f,-0.984899987971783019702343153767287731170654296875f,0.5138999829500912053248384836479090154170989990234375f,-0.5634000349402423779565651784650981426239013671875f,-0.66450001410394909928669449072913266718387603759765625f,0.24840000424385078048317154753021895885467529296875f,0.161099998721480286434371009818278253078460693359375f,0.453000011265277890970537555404007434844970703125f,-0.413400023317337161898876729537732899188995361328125f,-0.3948000018715855929940516944043338298797607421875f,0.7158000229865313368549095685011707246303558349609375f,0.018600009245425208614932444106671027839183807373046875f,-0.061799998969584601926641198588185943663120269775390625f,0.419399981214106098992289162197266705334186553955078125f,-0.603300015610456430437125163734890520572662353515625f,0.2999999764561653847749767010100185871124267578125f,-0.1676999973088502127183119227993302047252655029296875f,0.7643999954819680198170317453332245349884033203125f,-0.5721000165581704433037657508975826203823089599609375f,-1.0704000001311300138695514760911464691162109375f,-0.1970999964416026983826668583787977695465087890625f,-1.19630001018643294941057320102117955684661865234375f,-0.07280000509023676347197806535405106842517852783203125f,0.2016000131160022856846580907586030662059783935546875f,-0.88239999303817739928490482270717620849609375f,-0.4966000125944614040918168029747903347015380859375f,0.7644999839663508822695803246460855007171630859375f,1.170100031995773282034178919275291264057159423828125f,0.6630999991536139503978120046667754650115966796875f,0.51270002569556272220552273211069405078887939453125f,0.6459999930262565470684421597979962825775146484375f,0.09440001395940811335094622336328029632568359375f,-0.393599978232384017928779940120875835418701171875f,-1.230200026589631789875056711025536060333251953125f,-1.3434999951124186878814725787378847599029541015625f,-0.671300044757128233641196857206523418426513671875f,-0.18209999968260526248542419125442393124103546142578125f,-0.145300005351007011089592424468719400465488433837890625f,-0.8043000023782251872717097285203635692596435546875f,-0.345300009411573238793380369315855205059051513671875f,0.140299988698959765542895183898508548736572265625f,0.76690001023411724645484355278313159942626953125f,-1.4508000083088870013625637511722743511199951171875f,-1.105800060909987170276735923835076391696929931640625f,0.442800009299814778529480463475920259952545166015625f,0.2440000132694841727953871668432839214801788330078125f,0.7411999954760073450188428978435695171356201171875f,0.7078000281870366894310109273646958172321319580078125f,0.436500007085502073067573292064480483531951904296875f,0.1105000000521538794373554992489516735076904296875f,0.16690000025033935315832422929815948009490966796875f,-0.0551999748826030867121517076157033443450927734375f,0.195600010031461823967191548945265822112560272216796875f,-0.14640000824481258501208458255860023200511932373046875f,-0.74280000412166113932244115858338773250579833984375f,-0.9087000069499016063900853623636066913604736328125f,0.8852999974161386287363484370871447026729583740234375f,-0.04500003159046173095703125f,0.6945000116452575422698600959847681224346160888671875f,0.6287999984145162812865237356163561344146728515625f,0.3456000048905611965466277979430742561817169189453125f,0.84299999248981460908680674037896096706390380859375f,-0.865199997755885075179094201303087174892425537109375f,0.49770003210306157370723667554557323455810546875f,-0.4539000272065403240873138202005065977573394775390625f,-0.7424999868869779362512417719699442386627197265625f,-0.12270000133216381332790234637286630459129810333251953125f,-0.08879999856352804865622374563827179372310638427734375f,-0.0705000026598572304692424950189888477325439453125f,-0.300900003051757902738927441532723605632781982421875f,0.515099988037347689129319405765272676944732666015625f,-0.434100021982192885161566664464771747589111328125f,-0.8232000340551139938582991817384026944637298583984375f,-0.6098999942302700816298965946771204471588134765625f,-0.042000000469386566948060135473497211933135986328125f,-0.00660000655949122272403428723919205367565155029296875f,0.6138000117883086748094001450226642191410064697265625f,0.31709999197125426917409640736877918243408203125f,-0.1119000014796853736953607949544675648212432861328125f,0.10800000761449346509124325166339986026287078857421875f,-0.42089999101906994116717442011577077209949493408203125f,-0.56890002444982545881657642894424498081207275390625f,-0.4511999965190884864796316833235323429107666015625f,0.39220000489354145400966444867663085460662841796875f,-0.673000021770596479342430029646493494510650634765625f,-0.05080000272095208657674447749741375446319580078125f,-0.8392999947786332626264993450604379177093505859375f,-0.13950001108646414849090433563105762004852294921875f,-0.781699995863437635534864966757595539093017578125f,-0.679300030243397134910310342092998325824737548828125f,-1.40049995017051731593937802244909107685089111328125f,-0.885500009328126225227606482803821563720703125f,-0.51050001133978373246691262465901672840118408203125f,-0.16430001258552096032872213982045650482177734375f,0.284899989759921812293441689689643681049346923828125f,0.843700036901235961295242304913699626922607421875f,0.343600011610984523713341332040727138519287109375f,1.04140005201697416481465552351437509059906005859375f,0.428899989581108087577376863919198513031005859375f,-0.33770001392364523695732714259065687656402587890625f,1.00609998806714973085263409302569925785064697265625f,0.60350003772974059046418915386311709880828857421875f,-0.27519998985826965576251268430496565997600555419921875f,0.1795999985277652033488493543700315058231353759765625f,-0.22769997726678870009209276759065687656402587890625f,-0.662300000828504042971189846866764128208160400390625f,-1.306400017297267623916923184879124164581298828125f,-0.6700000511109838186030174256302416324615478515625f,0.17740000972747793639427982270717620849609375f,0.094000020980835596873248505289666354656219482421875f,0.81330001128911977303914682124741375446319580078125f,0.02040001026391990990305203013122081756591796875f,0.6551999961912631231797377040493302047252655029296875f,-0.257099996888637605252370121888816356658935546875f,-0.021599997134506609963722212341963313519954681396484375f,-0.5655000202059745628702103203977458178997039794921875f,-0.3950999968662858297108186889090575277805328369140625f,-1.00889999256134021976549774990417063236236572265625f,-0.878100010478496617594146300689317286014556884765625f,-0.6965999933719633219197930884547531604766845703125f,-0.76169998438954344965878817674820311367511749267578125f,-0.82349995845556289708611075184307992458343505859375f,-0.03809999997317792264084346243180334568023681640625f,-0.2654999985992907429732667878852225840091705322265625f,0.8595000223740936196037409899872727692127227783203125f,-0.025800012928247628707367766764946281909942626953125f,0.770400029262900520876655718893744051456451416015625f,-0.3687000075459483383610859164036810398101806640625f,-0.13890000720173123482936716754920780658721923828125f,0.74070000224113474729392692097462713718414306640625f,0.70230002537816782481883137734257616102695465087890625f,0.81629999006986597720469944761134684085845947265625f,0.157199998560547726800251666645635850727558135986328125f,-0.34259998844712968857351143014966510236263275146484375f,-0.57209999063014971287799426136189140379428863525390625f,0.13290001413822150055921156308613717555999755859375f,-0.791400025977194498949529588571749627590179443359375f,-1.0932000028371806621407813508994877338409423828125f,0.36120001302212489946441564825363457202911376953125f,0.431700002121925052733786287717521190643310546875f,0.12569999717473978595450034845271147787570953369140625f,-0.5785000325143341282085884813568554818630218505859375f,-1.1889000155091284938180251629091799259185791015625f,-0.78030004770755834186957144993357360363006591796875f,-0.379700000460445774574935740020009689033031463623046875f,-0.428300019335746962045874397517764009535312652587890625f,-0.416599998885393230096951810992322862148284912109375f,-0.2640000035464762362380497506819665431976318359375f,-0.7305000096261498043759274878539144992828369140625f,-0.6705000427067286938864754120004363358020782470703125f,-0.4054999859333037193209747783839702606201171875f,0.00189998281598047213947211275808513164520263671875f,0.05860004886388747724623726753634400665760040283203125f,-0.7828000017374747709908433535019867122173309326171875f,-0.77549998228251926235543578513897955417633056640625f,-0.07390000336468194319650137913413345813751220703125f,0.535899991726875413178277085535228252410888671875f,-0.09769999529719353148493610206060111522674560546875f,-0.06610000877976407451797058456577360630035400390625f,0.161899997121095484686748022795654833316802978515625f,-0.14279998221695411331211289507336914539337158203125f,0.295400014212727768381228088401257991790771484375f,1.5491000109553336727685746154747903347015380859375f,1.192300053131580828136293348507024347782135009765625f,-0.218299983274936604260574313229881227016448974609375f,-0.9649000314831734925746786757372319698333740234375f,0.5582000003039837565665948204696178436279296875f,0.06460000821352007704945208388380706310272216796875f,0.81909998249411586357382475398480892181396484375f,0.8247000217318534165400478741503320634365081787109375f,-0.3247000107794999479438047274015843868255615234375f,-0.48389999598860722329618511139415204524993896484375f,-0.4803000160798431839026534362346865236759185791015625f,-0.4676999882936474506323065725155174732208251953125f,-0.39990000983923679189757649510283954441547393798828125f,-0.161099997622519641726768213629839010536670684814453125f,-0.196799995560944018802018717906321398913860321044921875f,-0.2102999986827374101494569913484156131744384765625f,-0.7989000265359880170734641069429926574230194091796875f,-0.5730000033676621029599118628539144992828369140625f,-0.27150001189112682453696834272705018520355224609375f,-0.6824999845027921452356167719699442386627197265625f,-0.74339999068379392088701251850579865276813507080078125f,0.38130003793984623694512947622570209205150604248046875f,0.2067000005945562879361432351288385689258575439453125f,-0.3659999821484090087864160523167811334133148193359375f,-0.42870000475198022371614570147357881069183349609375f,0.15119997595548628765982357435859739780426025390625f,0.22829999571740622332782777448301203548908233642578125f,-0.043800003480911176012568830628879368305206298828125f,-0.067199998784065240187146628159098327159881591796875f,-0.6489000043332577849497511124354787170886993408203125f,0.8295000142157078482085807991097681224346160888671875f,0.5777999875068662305466204998083412647247314453125f,-0.874500014774501455150357287493534386157989501953125f,0.251100018912553846206492380588315427303314208984375f,-0.2544000025972723566525246496894396841526031494140625f,0.126599985921382884868080509477294981479644775390625f,0.879300003831088528016834970912896096706390380859375f,0.5147999877154827430558725609444081783294677734375f,0.16530000054538251941238513609278015792369842529296875f,-0.00349999943375589595007113530300557613372802734375f,-0.5720000094473363105862517841160297393798828125f,-0.2676000181376936293276003198116086423397064208984375f,-0.61899997478723545185630428022705018520355224609375f,-0.2573999988496300517226700321771204471588134765625f,0.0184999915361405697922236868180334568023681640625f,0.5367000116586684299591070157475769519805908203125f,0.09100003296136804920024587772786617279052734375f,-0.5385999989449974378885599435307085514068603515625f,0.96219998388290406410305877216160297393798828125f,0.28680000148415540905943998950533568859100341796875f,-0.7555999909579753648358746431767940521240234375f,-0.451000006586312895962009861250407993793487548828125f,-0.8788999969758093389060604749829508364200592041015625f,-0.151100010512024152120602593640796840190887451171875f,0.459599974834919322574933175928890705108642578125f,0.9418000239849089183508112910203635692596435546875f,-0.2160999685466293840363505296409130096435546875f,-0.4346999882042401619486327035701833665370941162109375f,-0.2529000250160695628665052936412394046783447265625f,0.4243000072956082391328891390003263950347900390625f,0.6518000023782253293802568805404007434844970703125f,0.224200012850761520866171849775128066539764404296875f,-1.5620000152587891761868377216160297393798828125f,-0.94780003742575669178904718137346208095550537109375f,0.433799989064037827546371772768907248973846435546875f,-0.09899999874085185780359097407199442386627197265625f,-0.0244999786019324261587826185859739780426025390625f,0.2439000111877920318192991544492542743682861328125f,0.1629000031113623236933563021011650562286376953125f,0.751200000244379051395071655861102044582366943359375f,0.080400000205635979710905303363688290119171142578125f,-0.0329999872744082267672638408839702606201171875f,-0.39719999040216202867981110102846287190914154052734375f,-0.63299998384714140087226041941903531551361083984375f,0.3554999997541308243143021172727458178997039794921875f,-0.38250001430511471056661321199499070644378662109375f,-0.69119998351782552248323554522357881069183349609375f,0.105000030398368693340671597979962825775146484375f,0.54659999568164341354759017121978104114532470703125f,1.073100000941753506822351482696831226348876953125f,-0.1481999977335333174011111623258329927921295166015625f,-0.186299993050098589009166971663944423198699951171875f,0.10410000099018212882384659678791649639606475830078125f,-0.46169998815208668219156606937758624553680419921875f,0.598199995274841800352305654087103903293609619140625f,-0.308700040924549057308468036353588104248046875f,-0.591899979127943343115703100920654833316802978515625f,-0.2192999824106696138414918095804750919342041015625f,0.167700009043514786100104174693115055561065673828125f,-0.676200019764900073226954191341064870357513427734375f,0.093600001515448116862216920708306133747100830078125f,0.368399994707107492075692789512686431407928466796875f,-0.57840000163614746497842133976519107818603515625f,-0.6704999939799307639987091533839702606201171875f,-0.02940000170320267169898897918756119906902313232421875f,0.51239998923838137034891815346782095730304718017578125f,-0.0050999999955296804188265014090575277805328369140625f,-0.3929999957680703204232486314140260219573974609375f,-1.0041000042870640296399642465985380113124847412109375f,-0.3385000301301488434546627104282379150390625f,0.24530001194477080161959747783839702606201171875f,-0.003299990129470575794812248204834759235382080078125f,-0.6805999784410001751666641212068498134613037109375f,0.0221999956548213361884336336515843868255615234375f,-0.0346999875895679377180158553528599441051483154296875f,0.2877000073857605144667104468680918216705322265625f,0.080899997240305054901909898035228252410888671875f,0.3461000072896480705963995205820538103580474853515625f,1.4784999924302102414230830618180334568023681640625f,1.03310001822709995877858091262169182300567626953125f,0.18010000427961347924110668827779591083526611328125f,-0.148699996668100187235950215836055576801300048828125f,0.19659998353719732033795253300922922790050506591796875f,-0.4322000225514177618180156059679575264453887939453125f,0.45179997421503070853532335604541003704071044921875f,-0.36120001559257541856595707940869033336639404296875f,-0.09630000854730591530739047811948694288730621337890625f,-0.2615000173300507935181258289958350360393524169921875f,0.33630000064969056694508253713138401508331298828125f,-0.005700000602006927152842763462103903293609619140625f,-0.609399959123134493665929767303168773651123046875f,-0.10359999387860252539894645451568067073822021484375f,0.28989999646544450939700254821218550205230712890625f,-0.453700003373622706703827134333550930023193359375f,-0.0093999689579009615414406653144396841526031494140625f,-0.904400028264522592991170313325710594654083251953125f,-0.35750002309679995704527755151502788066864013671875f,0.4447000136852263807440976961515843868255615234375f,0.9467000046551223846336142742075026035308837890625f,0.2156999819755558434053455130197107791900634765625f,-0.170099997089803167682475759647786617279052734375f,0.036899998462200045423742267303168773651123046875f,-0.17339999381303794478981217253021895885467529296875f,-0.853799976956844286490877493633888661861419677734375f,-0.03749999737367037955237947244313545525074005126953125f,-0.5045999958083033920530624527600593864917755126953125f,0.418499999061226812813174547045491635799407958984375f,0.03119999726414679486197201185859739780426025390625f,0.8006999930396674347576890795608051121234893798828125f,0.718499996304512222877747262828052043914794921875f,-0.18929999869018787617136467815726064145565032958984375f,0.1727999973416327605946207768283784389495849609375f,-0.396600017362833068279570625236374326050281524658203125f,0.37679998982399720564018252844107337296009063720703125f,-0.23760000915974377466710620865342207252979278564453125f,0.68489998648166672268189358874224126338958740234375f,-0.269100011998415045155752522987313568592071533203125f,-0.00240000380426630410823918282403610646724700927734375f,0.14070000030398366330786075195646844804286956787109375f,0.476400008058547985712039007921703159809112548828125f,-0.36059999974966050473312861868180334568023681640625f,-0.81269998561143896864678026759065687656402587890625f,-0.27869999938756218060831315597170032560825347900390625f,0.628199996578693298232565211947076022624969482421875f,-0.83820001416206368904937562547274865210056304931640625f,0.3933000311821699401804153239936567842960357666015625f,0.24450001153349887328403156061540357768535614013671875f,-0.715500024080276464388816748396493494510650634765625f,0.8955000266879797887753511531627736985683441162109375f,-0.85220002992749233072800052468664944171905517578125f,-0.732600021713972093806432894780300557613372802734375f,-0.10760000442862516223385682678781449794769287109375f,0.7449999767541886086519298260100185871124267578125f,0.958800013315677102099243711563758552074432373046875f,-0.3012000183492897775749952415935695171356201171875f,0.294600011417269769253834965638816356658935546875f,0.963900015658140318919322453439235687255859375f,0.42330001084208479511516998172737658023834228515625f,1.04959999748468391089772921986877918243408203125f,0.93200003720074919844051919426419772207736968994140625f,-0.7106999727368357167023305009934119880199432373046875f,-0.784100006633996482463544452912174165248870849609375f,-0.307899954235554051962253652163781225681304931640625f,-0.9719000069558614285369912977330386638641357421875f,0.08629999990463232961701578460633754730224609375f,0.455700026381016254362066320027224719524383544921875f,-0.08319999538660027837977395392954349517822265625f,-0.73720002247691152064135167165659368038177490234375f,-0.20129997870773053136872476898133754730224609375f,-0.3049000021651384617626945328083820641040802001953125f,0.846099982851743703804459073580801486968994140625f,0.6819000182807444243593408828019164502620697021484375f,0.8332999728441239373211146812536753714084625244140625f,0.244300005209445725284922446007840335369110107421875f,-1.1149000040650367537864440237171947956085205078125f,-0.67270003880858464384573380812071263790130615234375f,0.23749999046325687146463678800500929355621337890625f,0.389900012260675499220496931229718029499053955078125f,-1.289199973511696128980474895797669887542724609375f,0.64710002073049555093575690989382565021514892578125f,-0.055199996526539230057295526421512477099895477294921875f,-0.57330001859068868697022480773739516735076904296875f,0.6263999971434472957554362437804229557514190673828125f,-0.040500017523765308169458876363933086395263671875f,0.15660000316947708665082927836920134723186492919921875f,-0.5391000280916691256294370759860612452030181884765625f,0.0908999923601745418721975511289201676845550537109375f,0.353700003969669385384122506366111338138580322265625f,0.90060001472532758004518882444244809448719024658203125f,0.5897999941788611977244727313518524169921875f,-0.7760999903395771060132801721920259296894073486328125f,-0.3590999825686218560605311722611077129840850830078125f,-0.959699990625679344447007679264061152935028076171875f,0.082800033551454266245173130300827324390411376953125f,0.69330002041608107532510985038243234157562255859375f,0.1839000018000600533696342608891427516937255859375f,-0.42660001501590016914633451960980892181396484375f,0.5424000061511993209251158987171947956085205078125f,-0.63030000903904437326019660758902318775653839111328125f,-0.443400004504621136902642319910228252410888671875f,0.61230000280290841629948772606439888477325439453125f,0.4502999858677387834404726163484156131744384765625f,0.2312999951362610107086226207684376277029514312744140625f,0.6608999766021967925411217947839759290218353271484375f,-0.3321000107467175777031798133975826203823089599609375f,-0.39269998054504373641293568653054535388946533203125f,0.49410000339299442462248634910793043673038482666015625f,0.19499999195337291979512883699499070644378662109375f,-0.383999976351856897593961548409424722194671630859375f,-0.821500039160252271130957524292171001434326171875f,0.27609998598098750566265380257391370832920074462890625f,-0.2380999981850384639159301514155231416225433349609375f,0.34290000817775734276438015513122081756591796875f,-0.47110001354813579865776773658581078052520751953125f,1.3300999962329864700905091012828052043914794921875f,0.58090004239082393411308657960034906864166259765625f,-0.784500015556812257955243694595992565155029296875f,-0.257100026094914113627964979968965053558349609375f,0.34299999994039520601063486537896096706390380859375f,0.13280001730918922220325839589349925518035888671875f,-0.08509999802112577782509106327779591083526611328125f,-0.41550001707673089867967064492404460906982421875f,-0.0754999988228082230534710106439888477325439453125f,-0.10370000165998927155186493109795264899730682373046875f,-0.2900000381469727273042735760100185871124267578125f,0.5810000104606150017616528202779591083526611328125f,-0.45779997603297228891960912733338773250579833984375f,0.00199999529123306984956798260100185871124267578125f,-0.0118999885529280646778715890832245349884033203125f,-0.13969999677240829694113699588342569768428802490234375f,0.32789999919980772347116726450622081756591796875f,0.260700006897747471601434199328650720417499542236328125f,-0.87149998849630350150619051419198513031005859375f,-0.0551000053226946562290322617627680301666259765625f,-0.30239998737573614562279544770717620849609375f,-0.665200018990039776412004357553087174892425537109375f,-0.1817000337123868547450911137275397777557373046875f,0.64310000913739173711292096413671970367431640625f,0.1192000146389009529457325697876513004302978515625f,0.1292999751091006288561402470804750919342041015625f,-0.198299997729062926765664087724871933460235595703125f,0.38429998408704990620066155315726064145565032958984375f,-0.392100010187923952997834931011311709880828857421875f,0.57420000838041307389403300476260483264923095703125f,0.3405000110417606418167224546778015792369842529296875f,0.7367999700427052545137485140003263950347900390625f,-0.483900009213388049289505943306721746921539306640625f,-0.8835000052452084418064259807579219341278076171875f,0.27480001196712267041988297933130525052547454833984375f,0.42749999761581403134869105997495353221893310546875f,-0.34050000735372309978998828228213824331760406494140625f,0.1572000083208084930674885981716215610504150390625f,0.0128999988645315255553214228712022304534912109375f,0.1005000010579824021306194481439888477325439453125f,0.8220000101923943258697136116097681224346160888671875f,-0.2025000211596488242093982989899814128875732421875f,-0.12540000288784514115292267888435162603855133056640625f,-0.564899981415271756901574917719699442386627197265625f,-0.2192999955609440387860331611591391265392303466796875f,-0.050699994865059938575768683222122490406036376953125f,0.12209999778270719250627962537691928446292877197265625f,0.04919999403804542481566386413760483264923095703125f,0.11070000063925977684675672207958996295928955078125f,-0.59099998766183858833755948580801486968994140625f,-0.5412000091105699794979955186136066913604736328125f,0.1038000109314918262271021376363933086395263671875f,0.4520999970003962697973065587575547397136688232421875f,-0.61560003610849367561286271666176617145538330078125f,-0.4044000089675188203131028785719536244869232177734375f,-0.7167000369906428858257640968076884746551513671875f,-0.8623000064909458473039194359444081783294677734375f,-1.0713000417768958616449026521877385675907135009765625f,-1.259899964129925109546093153767287731170654296875f,-0.7293000072956079549157948349602520465850830078125f,-1.261300015699863052986984257586300373077392578125f,-0.78550003391504308325465899542905390262603759765625f,0.496199987280368848274747506366111338138580322265625f,0.81320001535415631366277011693455278873443603515625f,0.447400039231776958104092045687139034271240234375f,0.9656000596523295342876735958270728588104248046875f,-0.2731999986648556699719847529195249080657958984375f,-0.837600038850307981874721008352935314178466796875f,0.75660000052452058838525772443972527980804443359375f,0.611400040692091639726868379511870443820953369140625f,1.5474000165820118724013809696771204471588134765625f,1.0172000411033632616408795001916587352752685546875f,-0.126700003463029897687874836265109479427337646484375f,0.144700003999471693560963103664107620716094970703125f,-1.0904000011742116527102552936412394046783447265625f,-0.33660001155138008499534407746978104114532470703125f,-0.39239998350143423522240482270717620849609375f,-0.088799990218877411507492070086300373077392578125f,-1.07999998241662975573262883699499070644378662109375f,-0.613200018781423494829141418449580669403076171875f,1.1537999781489371997622583876363933086395263671875f,0.36700001615285859912773958058096468448638916015625f,0.65890001908540707376005229889415204524993896484375f,0.15250001892447517803930168156512081623077392578125f,0.670999974846839819520027958787977695465087890625f,-0.468899992263317244578502140939235687255859375f,0.0363000094413763463307986967265605926513671875f,-0.107500027716159962665187777020037174224853515625f,-0.90149995739013000228823102588648907840251922607421875f,-0.61729998197853575003790638220380060374736785888671875f,1.154700019645691355663075228221714496612548828125f,1.27340002987384792731972993351519107818603515625f,0.3045000174194567588159543447545729577541351318359375f,0.0871000399529930913189446073374710977077484130859375f,0.8423999650239952785568675608374178409576416015625f,0.4925999866962431639194619492627680301666259765625f,-0.02220000928938414830327019444666802883148193359375f,-0.350300013583898728342091999365948140621185302734375f,0.571500003099441045151252183131873607635498046875f,0.6825000289082527871187267010100185871124267578125f,0.3848999602556233412542496807873249053955078125f,-0.2670999968886373920895493938587605953216552734375f,-0.78629998738765749521917314268648624420166015625f,-0.31769999514818181296504917554557323455810546875f,-0.7868999698519711927247044513933360576629638671875f,-0.68969999788999558631985564716160297393798828125f,0.543599972122907448834894239553250372409820556640625f,0.353699996519088788460294381366111338138580322265625f,-0.3578999934442339547757683249074034392833709716796875f,-0.359899999073147736083910785964690148830413818359375f,0.1958999664507807825231111564789898693561553955078125f,-0.399700030747056178626053224434144794940948486328125f,-0.090300036978721465175112825818359851837158203125f,-0.6255999994516372453290387056767940521240234375f,1.0898999584674839979925309307873249053955078125f,-0.24759999563694001789571075278217904269695281982421875f,0.3789000055253506360486426274292171001434326171875f,0.043599999392032628975357511080801486968994140625f,-0.18060001375675227563988300971686840057373046875f,0.50420000078082072292318116524256765842437744140625f,-0.2851999855369327097065479392767883837223052978515625f,-0.142200000870227771798681715154089033603668212890625f,0.166099998199939591358997859060764312744140625f,-0.7004000083267687415400359896011650562286376953125f,0.5820000209957363335178115448798052966594696044921875f,-0.3214000025451182462887800284079276025295257568359375f,1.1889000304102896876656814129091799259185791015625f,0.13039999014735226268157930462621152400970458984375f,0.468800018382072636313750990666449069976806640625f,0.632999996066093473245928180404007434844970703125f,0.2335999990940091919355836580507457256317138671875f,0.0991999966084957662815213552676141262054443359375f,-0.075799996238946842908035250729881227016448974609375f,-0.2487999993085860950259302626363933086395263671875f,0.135900026148557895311341781052760779857635498046875f,-0.35120000322461120134676093584857881069183349609375f,-0.16380002881288557858852072968147695064544677734375f,0.19360000639557828350234558456577360630035400390625f,-0.175300012578070596447332718526013195514678955078125f,0.4449999883770943132077491100062616169452667236328125f,0.273099981868266894480257178656756877899169921875f,-0.3447999933779239967179819359444081783294677734375f,-0.136100015485286718330826261080801486968994140625f,-0.006799990159273239243020725552923977375030517578125f,-0.498999990433454332361407068674452602863311767578125f,-0.606699989008903539655648273765109479427337646484375f,-0.12119999882876886232452307012863457202911376953125f,-0.4391000041007995235986527404747903347015380859375f,0.895500006198883369279428734444081783294677734375f,0.439799994421004925015949993394315242767333984375f,-0.0723000109612943475667634629644453525543212890625f,0.3129999927133322312755581151577644050121307373046875f,-1.303200009393691516379476524889469146728515625f,-0.79070000611543633795008645392954349517822265625f,-0.74850001522898690353713391232304275035858154296875f,-0.6860000065118072587466713230242021381855010986328125f,-0.106200051951407914430092205293476581573486328125f,-0.5931000247836113459243279066868126392364501953125f,0.9776999769687648722538142465054988861083984375f,0.274999994635582112323390902020037174224853515625f,0.2621999731540682887498405762016773223876953125f,0.3746000132799149895390655728988349437713623046875f,-0.378000000834465055277178180404007434844970703125f,-0.070599999302625615626993749174289405345916748046875f,-0.528600028598308568916763761080801486968994140625f,-0.040900011843442829473360689007677137851715087890625f,0.316200029897689915969749563373625278472900390625f,0.07480001915693268443874330841936171054840087890625f,0.88829997332841237334122297397698275744915008544921875f,0.65649998946487897466539607194135896861553192138671875f,-0.78779997141361235435397247783839702606201171875f,-0.758899980342388147391829988919198513031005859375f,-0.0539999649524691704982615192420780658721923828125f,-0.1476999838232995188036511535756289958953857421875f,0.4799999874830245261136951739899814128875732421875f,0.181999992609024108247695039608515799045562744140625f,0.60790001622438438744211453013122081756591796875f,0.72819999374747279574648928246460855007171630859375f,1.18810002895593669336449238471686840057373046875f,0.2350000087916850333158436114899814128875732421875f,0.0034999912381170616981762577779591083526611328125f,-0.66820001118183114385828957892954349517822265625f,1.1601000132203100889682900742627680301666259765625f,0.15139999241232882098984191543422639369964599609375f,-0.1553999861240384916527546010911464691162109375f,0.420199990379810373042346327565610408782958984375f,0.0420000027418137023005328956060111522674560546875f,0.09340001065134995172911658301018178462982177734375f,0.64490000346899023497826419770717620849609375f,0.041999998569488550259620751603506505489349365234375f,0.4252999913811681409470111248083412647247314453125f,-0.01959998586177835733224128489382565021514892578125f,-0.8816000029087067701993873924948275089263916015625f,0.3921999966979026197577695711515843868255615234375f,0.45060000142604106354582427229615859687328338623046875f,-0.1363999930083752420983955744304694235324859619140625f,0.688200007270276525872532147332094609737396240234375f,-0.3579999945759773094522415703977458178997039794921875f,-0.140199989485740417194392648525536060333251953125f,0.1301999893367291605272839660756289958953857421875f,0.55470000474452962890836715814657509326934814453125f,0.38339999098181731795875748503021895885467529296875f,-0.805299998198449440423019041190855205059051513671875f,-0.2169999967440963128950670579797588288784027099609375f,0.3234000191450121519665117375552654266357421875f,-0.0044000106066464095277979140519164502620697021484375f,1.097099970960616843740353942848742008209228515625f,0.252499993741512440692531527020037174224853515625f,0.056099988067149553216950153000652790069580078125f,-0.56009999474287042176001705229282379150390625f,-1.143300012779235430571134202182292938232421875f,-0.5060999928355212063024737290106713771820068359375f,-0.068100035810470416208772803656756877899169921875f,-0.5764999962449071091441510361619293689727783203125f,0.69329998714923846137025975622236728668212890625f,0.218799970102310226138797588646411895751953125f,0.326099998497962673127403832040727138519287109375f,0.5392000241756438327911382657475769519805908203125f,-0.1350000110268592123929920489899814128875732421875f,-0.4216999946713446689727788907475769519805908203125f,0.034200000929832441443068091757595539093017578125f,0.2515000415444372805495731881819665431976318359375f,1.092899985527992612333036959171295166015625f,1.3025999855041501263031022972427308559417724609375f,-0.38219999409019944636156651540659368038177490234375f,-0.43219999241381901189384961980977095663547515869140625f,-0.81600000237673520064873855517362244427204132080078125f,-0.70759998930394651939224104353343136608600616455078125f,0.15569999330043771124110207892954349517822265625f,-0.113000022888183337954615126363933086395263671875f,0.4694999768137932960598845966160297393798828125f,-0.12310000481605509747851101565174758434295654296875f,-0.0776999898396433241742897735093720257282257080078125f,0.499499991789460029423253217828460037708282470703125f,0.4193999880313867834047414362430572509765625f,-0.23599999377131464228796176030300557613372802734375f,0.89239997903108569943242400768212974071502685546875f,-0.36439998862743383511997308232821524143218994140625f,-0.53180002249479318976455033407546579837799072265625f,-0.05259999921321867333290356327779591083526611328125f,0.17139999405145633915026337490417063236236572265625f,0.03640000537633891752875570091418921947479248046875f,-0.1863000103354455205817430396564304828643798828125f,0.063599995219707494698013761080801486968994140625f,0.463600008630752569160904386080801486968994140625f,0.139800004553794821049450547434389591217041015625f,-0.0759999855756756659275197307579219341278076171875f,-0.2755999979615211259442730806767940521240234375f,-0.12939999682307234962763686780817806720733642578125f,0.32840000245571132619915033501456491649150848388671875f,1.0623000370383266499629826284945011138916015625f,0.1294000032305717695635394193232059478759765625f,0.163399990870058520187058093142695724964141845703125f,-0.6521999919295311531897141321678645908832550048828125f,-1.15770001362562169333614292554557323455810546875f,-0.051200003820657702391372367856092751026153564453125f,-1.03070002191066745211855959496460855007171630859375f,0.3854000107109545325556609896011650562286376953125f,0.6954000239729882792971693561412394046783447265625f,-0.0940000014603137312718672546907328069210052490234375f,-0.83630001540184029096280937665142118930816650390625f,0.245799996089935302023832264239899814128875732421875f,0.197300001573562866497013601474463939666748046875f,-0.40039998686909683556223171763122081756591796875f,0.202400048470497750940921832807362079620361328125f,0.25410001095533374382284819148480892181396484375f,-1.030199967432022134516955702565610408782958984375f,-1.23759996434450147972938793827779591083526611328125f,0.177300005745888000774357351474463939666748046875f,0.60200002092123039432181030861102044582366943359375f,0.071400002694129938163314363919198513031005859375f,0.08460001715421672230377225787378847599029541015625f,0.06180000789165518426671042107045650482177734375f,-0.1165000099539756917010890902020037174224853515625f,0.179099989497661482573676039464771747589111328125f,0.025400009369850096163645503111183643341064453125f,-0.46379999692440065928167314268648624420166015625f,-0.2016000070810317623681839904747903347015380859375f,0.661799998205899964176524008507840335369110107421875f,0.501800002527236888028028261032886803150177001953125f,-0.0513000249385839879323611967265605926513671875f,-0.4605000005364416892916779033839702606201171875f,-1.28309997013360277406945897382684051990509033203125f,-0.92739998804628853701359503247658722102642059326171875f,0.2022000214338302015448789461515843868255615234375f,0.5494999881386757323298297706060111522674560546875f,1.07970002858638736142893321812152862548828125f,1.26550001990795113471222066436894237995147705078125f,-0.28080002917051327671060789725743234157562255859375f,-0.27479999978542313243679018341936171054840087890625f,0.6849000079393388062953818007372319698333740234375f,0.76369999532699583966177669935859739780426025390625f,-0.8759999630749231158688417053781449794769287109375f,-0.795899989640712846039605210535228252410888671875f,1.221899964189528731139944284223020076751708984375f,-0.008199998217821136137217763462103903293609619140625f,-0.357200010667741540970610003569163382053375244140625f,0.4439999926686286624288868551957421004772186279296875f,0.439100015407055732641339318433892913162708282470703125f,-0.09220000422298901820994387890095822513103485107421875f,-1.3066000041007992393815584364347159862518310546875f,-0.03580000219941137373780293273739516735076904296875f,0.5979000127971172418028800166212022304534912109375f,0.26920000420808787300330777725321240723133087158203125f,0.8027000112414359733747915015555918216705322265625f,0.53180000044107433865292478003539144992828369140625f,0.215699981342255764360515968292020261287689208984375f,-0.4139999932646751812370666812057606875896453857421875f,0.40010001769065883792109161731787025928497314453125f,-0.46500000089406956504944901098497211933135986328125f,1.0866999994397161799497553147375583648681640625f,-0.34239998528957382717408108874224126338958740234375f,0.9425999861001965740570085472427308559417724609375f,-0.3427999915301800371025819913484156131744384765625f,-1.09300004494190261539188213646411895751953125f,-0.28440000919103614052119155530817806720733642578125f,0.4923000054478645637345834984444081783294677734375f,-0.658600005799531817984870940563268959522247314453125f,0.199700014579295981320683495141565799713134765625f,0.49139998108744631366562316543422639369964599609375f,0.6680000130534171631779827293939888477325439453125f,-0.1112000036716460638075432143523357808589935302734375f,-0.025400005346536858041872619651257991790771484375f,-0.43520001362562166224989823604119010269641876220703125f,-1.2948000224351883247209116234444081783294677734375f,0.411200003224611254637466117856092751026153564453125f,0.91739998718947213252050687515293247997760772705078125f,0.70739999236762518552978917796281166374683380126953125f,-0.308700016151368661354581490741111338138580322265625f,-0.406700008957832925826636483179754577577114105224609375f,0.267900024271011005794207449071109294891357421875f,0.6425999813318252762428528512828052043914794921875f,0.382199982541799432311790951644070446491241455078125f,0.097700007963180457437601944548077881336212158203125f,0.70829998029470431220033788122236728668212890625f,-0.0064999956488609456073390902020037174224853515625f,-0.83279997927397453327813536816393025219440460205078125f,-0.61869998452365415619880195663427002727985382080078125f,-0.933300007414818111328713712282478809356689453125f,-0.65989999959468814694218963268212974071502685546875f,-1.017599979543685861926860525272786617279052734375f,-1.254299984347820640095960698090493679046630859375f,-1.0079999775886534507662872783839702606201171875f,-1.2527999767780304551934023038484156131744384765625f,-0.232199976730345980513448012061417102813720703125f,0.286600005888939080023192218504846096038818359375f,-1.4235000004768370018837231327779591083526611328125f,-1.229900008237361674900967045687139034271240234375f,0.821099950218201257712280494160950183868408203125f,0.35509996672868737732642330229282379150390625f,-0.35640002444982510354520854889415204524993896484375f,-0.42080000205039969074505279422737658023834228515625f,-0.810000014752148889130012321402318775653839111328125f,-0.515800017249584019651820199214853346347808837890625f,0.461100040817260747871841886080801486968994140625f,0.0089999932646751545917140902020037174224853515625f,0.7051999835252757975467829965054988861083984375f,-0.5546000019550323401062996708787977695465087890625f,0.425199993062019387934924452565610408782958984375f,-0.40139999062418940667384958942420780658721923828125f,-0.69330000473260877669190449523739516735076904296875f,-0.17239999855160714758994799922220408916473388671875f,0.2555999913305042614553030944080092012882232666015625f,-0.274999995231628435732318394002504646778106689453125f,0.87890001681298024482913433530484326183795928955078125f,-0.3208000026464461473807432412286289036273956298828125f,0.58829998409450023899580628494732081890106201171875f,-0.113600003936886739808187485323287546634674072265625f,0.81510001977682122742407955229282379150390625f,-0.0959999997317790842998874722979962825775146484375f,-1.0839000113368033595406814129091799259185791015625f,-0.359800003361701925541638047434389591217041015625f,-0.6346999782204623130610343650914728641510009765625f,0.66019999902248382994685016456060111522674560546875f,0.4161000145912172598627876141108572483062744140625f,-0.33059999289512642661748031969182193279266357421875f,1.306400031900405878104720613919198513031005859375f,-0.0048000009775162055802866234444081783294677734375f,0.96090000752210613654824555851519107818603515625f,0.2146000021040439520447762333787977695465087890625f,0.55369998541772336153599098906852304935455322265625f,-0.555599989318847686803337637684307992458343505859375f,-0.637800014030933226649722200818359851837158203125f,-0.200999990344047585466569216805510222911834716796875f,0.44890001848936122996747144497931003570556640625f,-0.69450002956390388675345093361102044582366943359375f,-1.36649998641014036593333003111183643341064453125f,-1.44659999873638156486777006648480892181396484375f,-0.93689997044801742731578997336328029632568359375f,-0.91839998099803921149941743351519107818603515625f,-0.0155999945342539803050385671667754650115966796875f,0.51469999818801870361539840814657509326934814453125f,-0.5414999922215937289138310006819665431976318359375f,-0.3577999835580587539851649125921539962291717529296875f,-0.85020001712739468047175250831060111522674560546875f,-0.91510000141113978511242521562962792813777923583984375f,-0.502799994398653193883319545420818030834197998046875f,-0.83149999043345435012497546267695724964141845703125f,-0.41100001776218419990982511080801486968994140625f,-0.60530000389814375694186310283839702606201171875f,-0.336299981129169367477516061626374721527099609375f,0.3000999875903129776588684762828052043914794921875f,1.448099986934661131954271695576608180999755859375f,1.2652999872088432908867616788484156131744384765625f,-0.8396999831378462175734966876916587352752685546875f,-0.9327000136256220486075108055956661701202392578125f,-0.37440003809928867894996074028313159942626953125f,-0.776000015974044998756653512828052043914794921875f,0.0566999907970426875181146897375583648681640625f,-0.5192999891161917958015692420303821563720703125f,-1.1669999791234728636624140563071705400943756103515625f,-0.9180999775469305124175889432081021368503570556640625f,-0.1748999630868439680853043682873249053955078125f,0.1976000182867050369850403512828052043914794921875f,-0.923399971759319981856606318615376949310302734375f,0.13980000842809670036359648293000645935535430908203125f,0.997399999296664674375278991647064685821533203125f,0.34499998286366473365660567651502788066864013671875f,0.84349999332427927356548025272786617279052734375f,0.0595999955475330356335916803800500929355621337890625f,0.26810001599192645471703144721686840057373046875f,0.29780000836849207512813109133276157081127166748046875f,-0.8982000058174133272359540569595992565155029296875f,0.0989999931156635426532375277020037174224853515625f,-0.814800016176700836467716726474463939666748046875f,-0.314800009024143179203747422434389591217041015625f,-0.50270002062916763208022530307061970233917236328125f,-0.2267999941825866727640459430404007434844970703125f,0.6313000073552128554865703335963189601898193359375f,0.17799998757243162827990090590901672840118408203125f,-0.5289000152111054120496191899292171001434326171875f,-0.1986000078856944295324638005695305764675140380859375f,-0.50979998369216872333709034137427806854248046875f,-0.5003999940216541375548331416212022304534912109375f,-0.815000001788138916936077293939888477325439453125f,-0.33239999125003816260459643672220408916473388671875f,0.364499991416931123922040569595992565155029296875f,-0.05640000120401378325141195091418921947479248046875f,0.8508000211238861965057367342524230480194091796875f,0.10679999775886539481462023104541003704071044921875f,0.01399999315291633461555420581134967505931854248046875f,-0.1375999956369400312183870482840575277805328369140625f,-0.330299994659423390430674771778285503387451171875f,-0.1533999896407127749853316345252096652984619140625f,0.1493999749183654301987189683131873607635498046875f,-0.107799986016750448669654360855929553508758544921875f,0.3110999500691900010451718117110431194305419921875f,-0.32700000870227796667677466757595539093017578125f,-0.08760002812147060780034735216759145259857177734375f,-0.32369999204874044806956590036861598491668701171875f,0.538499995112418528009357032715342938899993896484375f,0.355700012373924234765354412957094609737396240234375f,0.436500000715255964678362943232059478759765625f,0.6485999794244765581652245600707828998565673828125f,-0.3245999985277645549786029732786118984222412109375f,0.203199981379508898271524230949580669403076171875f,-0.20999997563660155464049239526502788066864013671875f,0.22810002359151848594365219469182193279266357421875f,0.16079995764494015020318329334259033203125f,-0.1463000151038169605044458876363933086395263671875f,-0.31949998515844324487034100457094609737396240234375f,-0.19230001140832886363796205841936171054840087890625f,-0.736799982261658215065835975110530853271484375f,-0.1432000123739243235831963829696178436279296875f,-0.396599959993363171406599576584994792938232421875f,0.073800004076957748111453838646411895751953125f,-0.3617999844968320388716165325604379177093505859375f,-0.71009999355077724203511024825274944305419921875f,-0.04260002633333215271704830229282379150390625f,-0.3856000030279158607982026296667754650115966796875f,-0.20699999357759946150281393784098327159881591796875f,-0.02299998790770774181879687603213824331760406494140625f,-0.2306999672234066878218072815798223018646240234375f,-0.089499992907047243306806194595992565155029296875f,-1.4003999324798588332896542851813137531280517578125f,-0.408999988496303590324032484204508364200592041015625f,-0.029599972152709597139619290828704833984375f,0.650199991273880062436774096568115055561065673828125f,0.0170000134706498329251189716160297393798828125f,-0.07200000691413865894219270558096468448638916015625f,-0.224200005996227247351271216757595539093017578125f,0.40880000407695771258431705064140260219573974609375f,-0.3010000016689300395000827847979962825775146484375f,-0.19500000685453411364278508699499070644378662109375f,0.5706999906182286252942503779195249080657958984375f,0.554799999934434850956677109934389591217041015625f,-0.803999975085258000717658433131873607635498046875f,0.693200002092122957719766418449580669403076171875f,0.598700006353855229690452688373625278472900390625f,-0.37859999372959141084038492408581078052520751953125f,-0.6444000166416170571892507723532617092132568359375f,-0.113200011330842897905313293449580669403076171875f,1.18280003310441994557322686887346208095550537109375f,-0.11519999157190330407729561557061970233917236328125f,-0.389300020892918485770906045217998325824737548828125f,0.1017999929532409364441036814241670072078704833984375f,0.017500007562339714439758608932606875896453857421875f,-0.31819998674392702131541454946273006498813629150390625f,-0.11680000073909724278564681299030780792236328125f,-0.16239998186230675258912015124224126338958740234375f,0.2678000179052357765385750099085271358489990234375f,-0.6627999957025050292713785893283784389495849609375f,0.3916000231742859938321998924948275089263916015625f,0.67999999508261677050313664949499070644378662109375f,0.04440002439022094904430559836328029632568359375f,-0.19089998189210888313027680851519107818603515625f,1.1750999694108958948390863952226936817169189453125f,0.6609999748468400326828486868180334568023681640625f,0.011999969959259004781415569595992565155029296875f,0.323000000834465339494272484444081783294677734375f,0.9527999951064582973003780352883040904998779296875f,0.9702000282287599475239403545856475830078125f,-0.373499994218349229413433931767940521240234375f,-0.3140000227689743184100734652020037174224853515625f,0.806399990773201125193736515939235687255859375f,0.1307000150561332674215009319595992565155029296875f,1.58399998760223326144114253111183643341064453125f,1.478999979257583419212096487171947956085205078125f,-0.97049998325109498864549095742404460906982421875f,-1.07010000010728845154517330229282379150390625f,-0.045600001835822467910475097596645355224609375f,0.2049999850988388772066173260100185871124267578125f,-0.550800014567375484375588712282478809356689453125f,-0.84800000411272069555934649542905390262603759765625f,0.4094999867789452796529303668648935854434967041015625f,0.656900011798739580370920521090738475322723388671875f,-0.61889996734112584508835652741254307329654693603515625f,-0.44429998665750025654830324128852225840091705322265625f,-0.394199947285652996242788503877818584442138671875f,-0.466999990224838512631322373636066913604736328125f,-1.3463999818325049773193313740193843841552734375f,-1.0412000000953671730030691833235323429107666015625f,1.099799968194961508061169297434389591217041015625f,0.4117999780893324413000300410203635692596435546875f,0.984599970364570253877900540828704833984375f,-0.35219999282360081593878931016661226749420166015625f,-0.01939999928176394661250014905817806720733642578125f,0.110000004544854146359966762247495353221893310546875f,0.253100008094310879869226482696831226348876953125f,0.61440000367760649879755874280817806720733642578125f,-1.03960001715421679335804583388380706310272216796875f,0.37720001204609854283944514463655650615692138671875f,-0.2281000188231470104938125587068498134613037109375f,-0.41219999431371689979641814716160297393798828125f,-0.028800017189979598697391338646411895751953125f,-0.632399985885620186110145368729718029499053955078125f,0.96830000056028353583315038122236728668212890625f,-0.6912000059068201807122022728435695171356201171875f,1.1273000203490255444194190204143524169921875f,0.1663999997138976283395095379091799259185791015625f,0.15710000370591881146964396975818090140819549560546875f,0.014199999961256981606538829510100185871124267578125f,0.84740000495910638989016661071218550205230712890625f,-0.0765999866664410777872262769960798323154449462890625f,-0.581300004374980971988406963646411895751953125f,0.51640000686645493033211096189916133880615234375f,1.37110003247261058589856475009582936763763427734375f,-0.0079999941289425891000064439140260219573974609375f,-0.653000018343329546866016244166530668735504150390625f,0.240800000858306828543931032982072792947292327880859375f,-0.670200012284517310234832621063105762004852294921875f,-0.0174000001907348433860533987171947956085205078125f,-0.53560000235736371365646846243180334568023681640625f,0.115199986952543353968536621323437429964542388916015625f,0.2259999873638154355148799368180334568023681640625f,-1.3836999711871147411557103623636066913604736328125f,0.249000003583728979350553345284424722194671630859375f,0.24820001192688945224062990746460855007171630859375f,0.973799999308585739754562382586300373077392578125f,0.29829999877214419257143163122236728668212890625f,1.104000023961067000755065237171947956085205078125f,1.36380001569986308851412104559130966663360595703125f,-0.60449998247623426550489966757595539093017578125f,-0.0756999781012535066793134319595992565155029296875f,-1.1297999598503114526693025254644453525543212890625f,-0.6171999719619751800792073481716215610504150390625f,-1.68059999945163696111194440163671970367431640625f,-1.7241000026106831910510663874447345733642578125f,-0.01200003373622848812374286353588104248046875f,-0.48690001217126877008922747336328029632568359375f,-0.029700002844631701037769744289107620716094970703125f,-0.10970000120550393862828286728472448885440826416015625f,-0.34169999526739136541664265678264200687408447265625f,-0.4845999772191049714820110239088535308837890625f,1.2006000024318694840985699556767940521240234375f,1.162100002253055208711884915828704833984375f,-0.463800019574165389713016338646411895751953125f,-0.99890001282691986972395170596428215503692626953125f,0.5904000177890063039143342393799684941768646240234375f,0.5558000011563299924688408282236196100711822509765625f,0.22530000866651533897311310283839702606201171875f,0.569200007039308619738449124270118772983551025390625f,0.3137999893993141942161173574277199804782867431640625f,0.3113999769151212237972004004404880106449127197265625f,-0.977699949848652494210909935645759105682373046875f,-0.21199999991059303994234142010100185871124267578125f,0.4736999938368793294785064063034951686859130859375f,0.00040000624060620992850090260617434978485107421875f,0.3930000215470794699967882479541003704071044921875f,0.28300001156330101270697241488960571587085723876953125f,0.1326000242471696566326500033028423786163330078125f,0.720999984234571567043303730315528810024261474609375f,0.30700002270936987969207621063105762004852294921875f,0.44079999027848248971395150874741375446319580078125f,1.087699995148181386639407719485461711883544921875f,0.2515999853253365614591530174948275089263916015625f,0.4625000225007536158727816655300557613372802734375f,0.03399999743700032883708672670763917267322540283203125f,-0.3291000046968459713525589904747903347015380859375f,0.108600004458427434883560636080801486968994140625f,-0.7761000214457514090327094891108572483062744140625f,0.52879999766945839922982486314140260219573974609375f,0.1190000036954881323936206172220408916473388671875f,-0.592599997872114148123046106775291264057159423828125f,-0.5318000126600264110265925410203635692596435546875f,-0.44379999498724931328297316213138401508331298828125f,0.06800000150501706031036519561894237995147705078125f,0.22679999053180222023939904829603619873523712158203125f,0.39249998629093152402447230997495353221893310546875f,-0.22119999860525130230826107435859739780426025390625f,0.671900047934055777432149625383317470550537109375f,0.60379999319911004107552798814140260219573974609375f,0.23079997671842544804121644119732081890106201171875f,-0.4181999885320664134269463829696178436279296875f,-0.660100022161007160548251704312860965728759765625f,-0.7114000050783158002332129399292171001434326171875f,0.019199984836578920521787949837744235992431640625f,0.1393000183224675225801547640003263950347900390625f,0.447900000727175839898563936003483831882476806640625f,0.186800012362003275967481386032886803150177001953125f,1.2662999528169631702212427626363933086395263671875f,0.66179997212886831903233542107045650482177734375f,0.607799975585937346522769075818359851837158203125f,-0.01440002826452246864619155530817806720733642578125f,0.101399958288670433148581651039421558380126953125f,-0.519300000441074161017240840010344982147216796875f,-0.042600022011995530846206747810356318950653076171875f,-0.167700007367133974156558906543068587779998779296875f,0.4358999999225137145231201429851353168487548828125f,0.7141000210881234266935280174948275089263916015625f,1.313399995303154099701714585535228252410888671875f,1.3891000181078911879239967674948275089263916015625f,-1.1756999831676484546960637089796364307403564453125f,-0.875400003409385618624583003111183643341064453125f,-0.5972999738574031880489201284945011138916015625f,0.0138000219583513938914620666764676570892333984375f,0.44459997032731823907170110032893717288970947265625f,0.341199988919496544070852905861102044582366943359375f,-0.61019998441934575339473667554557323455810546875f,-0.73689998743534079750361343030817806720733642578125f,0.854099957013129795768691110424697399139404296875f,0.08649996227025980033431551419198513031005859375f,-0.7766999747037885981626459397375583648681640625f,-0.44379996503591545575773125165142118930816650390625f,-0.0881999852538104534005469758994877338409423828125f,0.216199998754262878719600848853588104248046875f,1.1812000447988513229802265414036810398101806640625f,-0.0578000064313410888416910893283784389495849609375f,0.33380000906884677025487917489954270422458648681640625f,0.014200000445544702643019263632595539093017578125f,0.3628000059843063951348085538484156131744384765625f,0.0338000019907951365638609786401502788066864013671875f,0.369400026923418334234838766860775649547576904296875f,0.32319999821782108284651258145458996295928955078125f,-0.1456999894261361561120793339796364307403564453125f,0.57239998022317895021160438773222267627716064453125f,0.21029997044503634384682300151325762271881103515625f,-0.336000012621283428160268158535473048686981201171875f,0.77449999573826744381221942603588104248046875f,-0.01100000077486040783725229630363173782825469970703125f,-0.6757000096917149534192503779195249080657958984375f,0.143800011527538185163166417623870074748992919921875f,0.08000000655651096082010553800500929355621337890625f,-0.0035999986469745781647588955820538103580474853515625f,-0.4655000158846380742261317209340631961822509765625f,0.65519999887347213363142373054870404303073883056640625f,-1.54540004483461412831957204616628587245941162109375f,0.0944000012934207433090705308131873607635498046875f,0.4611999980092047479729444603435695171356201171875f,-0.324200003761053068274122779257595539093017578125f,-0.7109000131845473191560813575051724910736083984375f,0.1690000044405459700413985046907328069210052490234375f,-0.546900006061792343103888924815692007541656494140625f,-0.218599996411800390205826261080801486968994140625f,0.1913000092923642370124071021564304828643798828125f,-0.228800015848875215596081034163944423198699951171875f,-0.453900043821334975291392765939235687255859375f,-0.8084000245094298264803001075051724910736083984375f,-0.1191000030018389832520142590510658919811248779296875f,-0.32460000690966850367402685151319019496440887451171875f,-0.05070000268816965416363018448464572429656982421875f,-0.20779999465942378122917943983338773250579833984375f,0.385799986255168647630853229202330112457275390625f,-0.149600027137994917580954279401339590549468994140625f,1.067399953699112558069828082807362079620361328125f,0.6508999622225761783056441345252096652984619140625f,-0.820199995930491976281473398557864129543304443359375f,-0.89530000225901584798293697531335055828094482421875f,-0.150300025504826795241797299240715801715850830078125f,-0.229099988603592041869205786497332155704498291015625f,0.3399000472784035054019113886170089244842529296875f,0.24390000597238525870125158689916133880615234375f,0.01799999308586119894926014239899814128875732421875f,0.07899999922513956107650301419198513031005859375f,1.331399998968839870627789423451758921146392822265625f,0.993300001454353154173304574214853346347808837890625f,0.666600034499168714319239370524883270263671875f,1.18440001515150061806025405530817806720733642578125f,-0.8762999934971329452082500210963189601898193359375f,-1.0512000126123428600521947373636066913604736328125f,0.42750003129243818733584703295491635799407958984375f,0.37970000027418127075407028314657509326934814453125f,-0.13409999128580096794394194148480892181396484375f,0.397500007897615414975689418497495353221893310546875f,0.264599996143579208052187823341228067874908447265625f,-0.475399995958805110518596848123706877231597900390625f,0.804400024837255589460482951835729181766510009765625f,-0.26680000550746907350685432902537286281585693359375f,0.1978999977469442228539264760911464691162109375f,0.41619998951554304511546433786861598491668701171875f,-0.372599998617172190051860525272786617279052734375f,0.408200002390146199360287937452085316181182861328125f,-0.112500008046627186786281527020037174224853515625f,-0.0150000213086602940393277094699442386627197265625f,-0.2255000029206275513615764793939888477325439453125f,-0.7407999995172023233180880197323858737945556640625f,0.88419999794960002503785290173254907131195068359375f,-0.1729999907016754878696929154102690517902374267578125f,0.813200013901293150553328814567066729068756103515625f,0.0501999975323677016803003425593487918376922607421875f,-0.904600005531310813466916442848742008209228515625f,0.5358000163555143302573924302123486995697021484375f,0.012999986380338501845699283876456320285797119140625f,0.1793999908626080497242583078332245349884033203125f,-0.30409997131824439975389395840466022491455078125f,0.6097999965071678740713423394481651484966278076171875f,-0.55470001577138905446417993516661226749420166015625f,-0.0797999981462955698674477389431558549404144287109375f,1.5013000449061397745253998436965048313140869140625f,-0.017599998170137354236430837772786617279052734375f,-0.10760000465214236697875094250775873661041259765625f,-0.343999989986419729692812552457326091825962066650390625f,-0.367100018346310008610089425928890705108642578125f,0.426199995923042251888546161353588104248046875f,-0.212299995315074596646809368394315242767333984375f,-0.4596999955058098663585042231716215610504150390625f,-0.665400032317638334689036128111183643341064453125f,-0.632000007957219889220823461073450744152069091796875f,0.6734999757409099174765287898480892181396484375f,0.245299993169307839480097754858434200286865234375f,0.9206999956846235733110006549395620822906494140625f,0.8562000012874602106194288353435695171356201171875f,-0.13170001642703965671898913569748401641845703125f,-0.325900021231174008562447852455079555511474609375f,-1.25579998714923846137025975622236728668212890625f,-0.5711000011801719011828026850707828998565673828125f,-0.54689999727010718544306655530817806720733642578125f,-0.6623999772429467469692099257372319698333740234375f,-0.06150003279000493261463589078630320727825164794921875f,-0.21959999815523623123425522862817160785198211669921875f,1.32270002822875909487265744246542453765869140625f,1.363200015354156136027086176909506320953369140625f,0.2282999981760980290346196852624416351318359375f,-0.079399998611211497490103283780626952648162841796875f,1.25369997774362484932453298824839293956756591796875f,0.98869998728036900814686305238865315914154052734375f,-0.01409997817277908183086765347979962825775146484375f,0.17590000364780422614785493351519107818603515625f,-0.4650000327825551948990323580801486968994140625f,-0.9399000034689901639239906216971576213836669921875f,-0.6347999681204561372993566692457534372806549072265625f,-0.4087999884307385034531989731476642191410064697265625f,0.88709999539852191219324595294892787933349609375f,0.692499999105930186260593472979962825775146484375f,-0.141299980235100264280845294706523418426513671875f,0.04639999762773516778224802692420780658721923828125f,-0.33900001651048672357546820421703159809112548828125f,0.099800001722574194218395859934389591217041015625f,0.0898000114083288281108252704143524169921875f,0.6127999767780305528930284708621911704540252685546875f,-0.470600002320110899489691291819326579570770263671875f,0.54799998801946647741800688891089521348476409912109375f,-0.031800004464388020863907513557933270931243896484375f,-0.025999997049570122698014529305510222911834716796875f,-1.3030000145435334246712955064140260219573974609375f,-0.09700001093745225233533346909098327159881591796875f,-0.10810000958442689267258174368180334568023681640625f,-0.084199995565414464948617023765109479427337646484375f,0.05020000863373308419568274985067546367645263671875f,-0.286799992319941576823794093797914683818817138671875f,0.0162000120162968386239299434237182140350341796875f,-0.51059999080896378842453486868180334568023681640625f,0.1026000100165607165081382845528423786163330078125f,-0.02599999846518041834997347905300557613372802734375f,0.01839999649524681757384314551018178462982177734375f,-0.3680000027716159305413157198927365243434906005859375f,0.0048999754548066931647554156370460987091064453125f,0.618799995285272519396357893128879368305206298828125f,0.18729999263286600807987269945442676544189453125f,-0.5281999891281128345355000419658608734607696533203125f,0.3088999870479103293519074213691055774688720703125f,-0.3772000110030173658515195711515843868255615234375f,-0.183099978590011147616678499616682529449462890625f,0.5932000040292739750924511099583469331264495849609375f,-1.085100016051530946725733883795328438282012939453125f,-0.045100035274028726917094900272786617279052734375f,0.231300017786026046451297588646411895751953125f,0.18060000571012491121791754267178475856781005859375f,1.11119994078874650966781700844876468181610107421875f,0.65499997019767786099464501603506505489349365234375f,1.134599977852404517619788748561404645442962646484375f,0.87399998261034494984045295495889149606227874755859375f,0.0347999982953073327962556504644453525543212890625f,0.194000007420778164402008769684471189975738525390625f,0.245699999856949347076806589029729366302490234375f,0.8253999783754348840147940791212022304534912109375f,-0.1712999799370766851325242896564304828643798828125f,-0.15929999239444736502946398104541003704071044921875f,-0.628199975530803644829802578897215425968170166015625f,-0.58900000533461582818972601671703159809112548828125f,-1.01609996899366450406887452118098735809326171875f,-0.7788000005006789905337427626363933086395263671875f,-0.121199997375905699215081767761148512363433837890625f,-0.2132000089466572489982354454696178436279296875f,-0.67619999890327431302239347132854163646697998046875f,-0.40460000165700904517507296986877918243408203125f,1.1063999889850624214204799500294029712677001953125f,0.602100003445148246328244567848742008209228515625f,-1.09409996297359501937762615852989256381988525390625f,-0.89309998365640641537765986868180334568023681640625f,-0.93720000019967475424209624179638922214508056640625f,-1.0371000186443328772156746708787977695465087890625f,1.21019998948573981323306725244037806987762451171875f,0.95670000301599511516315033077262341976165771484375f,0.844799987566471344280216726474463939666748046875f,-0.744000004440545925632477519684471189975738525390625f,0.46700002360343972895861952565610408782958984375f,0.2634000108003615725493773425114341080188751220703125f,-1.03979998935461015463488365639932453632354736328125f,0.38259999176263814746334901428781449794769287109375f,-0.0953999931275841817068794625811278820037841796875f,0.43260001471042619503037940376088954508304595947265625f,-1.0218000254750254640612183720804750919342041015625f,0.0989999910295009666327814557007513940334320068359375f,-0.80110002096146359917838708497583866119384765625f,-0.1842000001475214698842819416313432157039642333984375f,0.5623999946773050595538734341971576213836669921875f,0.092999996960163144876787555404007434844970703125f,0.4078999783754344576891526230610907077789306640625f,-0.17620000993013373857820624834857881069183349609375f,0.15159999247789368581607050145976245403289794921875f,-0.21539999670386311692027447861619293689727783203125f,0.6206999912142752151567037799395620822906494140625f,-0.61459998839497576028634284739382565021514892578125f,-0.24250002007931492631342962340568192303180694580078125f,0.4402000057280062161879641280393116176128387451171875f,-0.686000003904104094232252464280463755130767822265625f,-0.111399998074769968070540926419198513031005859375f,-0.500699999707936882487047114409506320953369140625f,-0.3227999882519245744560976163484156131744384765625f,1.1697000336527825226085042231716215610504150390625f,-0.114800008428096678159135990426875650882720947265625f,-0.08130000348091126483041080064140260219573974609375f,-0.1333999938130379092626753845252096652984619140625f,1.259499987542628929304555640555918216705322265625f,-1.501500009953975478538268362171947956085205078125f,0.38849999824166214779097572318278253078460693359375f,0.11400001144409177555871792719699442386627197265625f,1.066499985218047896751158987171947956085205078125f,1.23839997652769095992653092253021895885467529296875f,0.870300015074013888494164348230697214603424072265625f,0.670800018441676915159632699214853346347808837890625f,0.42570000969171584159767007804475724697113037109375f,0.59579998416900625812786529422737658023834228515625f,-0.0098999877296386262770511166309006512165069580078125f,0.5575000121444464440401134197600185871124267578125f,-0.08850000233948218664181695203296840190887451171875f,-0.5111999950289725092034132103435695171356201171875f,-0.5996999968469136632620575255714356899261474609375f,-0.8841000008225439188436212134547531604766845703125f,-0.313500002861022863953621708787977695465087890625f,0.01720002351999294631923476117663085460662841796875f,-1.379099970126151930571722914464771747589111328125f,-1.2582999862551691450107682612724602222442626953125f,0.77130002337396152967130547040142118930816650390625f,0.33910000510662763328895152881159447133541107177734375f,0.05010001202821712240620399825274944305419921875f,0.455200001257658026787566996063105762004852294921875f,-0.46139996253550119575947974226437509059906005859375f,-0.0856999927043915477042901329696178436279296875f,-0.5322000458717344173464880441315472126007080078125f,-0.8005000190138815696627716533839702606201171875f,-0.144299992692470624433553894050419330596923828125f,0.11630001301765435783863722463138401508331298828125f,-0.40259998848438272034400142729282379150390625f,-0.08419998379349724615394734428264200687408447265625f,-0.03190000062286824089596848352812230587005615234375f,-0.269599986830353799405202153138816356658935546875f,0.0408999890446661851228782325051724910736083984375f,-0.431399998819828045526492132921703159809112548828125f,-1.1282000397920608492086103069595992565155029296875f,-0.15799997952580469018357689492404460906982421875f,0.17020003836154984355744090862572193145751953125f,0.61659999024271971990174279198981821537017822265625f,-0.0242000122547150198215604177676141262054443359375f,-0.32059999319314957944015986868180334568023681640625f,0.240500005304813413431475055404007434844970703125f,-0.192600003087520565969725794275291264057159423828125f,0.2136999967426060909048146640998311340808868408203125f,-0.152399995720386538522461705724708735942840576171875f,0.317500028312206694636188331060111522674560546875f,0.0100000090897083282470703125f,0.0688999961376186575989777338691055774688720703125f,0.4023999866306782191571755902259610593318939208984375f,-0.0626000070363283978025492615415714681148529052734375f,0.0677999974906444702327235063421539962291717529296875f,-0.40870000683814289299533584198798052966594696044921875f,0.1058000004112720215942999857361428439617156982421875f,0.671100004756450374543419457040727138519287109375f,0.19779999763965605552584747783839702606201171875f,0.47479999531507477428249330841936171054840087890625f,-0.22999999761581424451151178800500929355621337890625f,-0.3088999815344806165740010328590869903564453125f,0.556600012743472927212451395462267100811004638671875f,-0.2281000188231470104938125587068498134613037109375f,-0.613000007390976264787241234444081783294677734375f,-0.541199962283671620610903119086287915706634521484375f,-0.43169998409152032792235331726260483264923095703125f,-0.8996999785184858211550817941315472126007080078125f,-0.767100002253055635037526371888816356658935546875f,0.00720007567405644977043266408145427703857421875f,0.4400000369548795475793667719699442386627197265625f,1.043699962544440751344154705293476581573486328125f,0.467799967241287362185175879858434200286865234375f,-0.55229997981786738137088832445442676544189453125f,-0.2579999793767928650822796043939888477325439453125f,-0.4803000021100043426258707768283784389495849609375f,-0.5191000097632407772607621154747903347015380859375f,-0.3329999935328959992375530418939888477325439453125f,-0.2760999915689230643778273588395677506923675537109375f,0.10199998545646593584024230949580669403076171875f,0.301500014126300897032706416212022304534912109375f,0.72539996585846022725263537722639739513397216796875f,0.42009998401403425560829418827779591083526611328125f,0.10170000256896027934772064327262341976165771484375f,0.0001999999910593519558688058168627321720123291015625f,0.31170000726282598435545878601260483264923095703125f,0.36649999851733439104606304681510664522647857666015625f,0.25049996924400375064578838646411895751953125f,0.030700004327297136796914855949580669403076171875f,-0.638999990582465926536315237171947956085205078125f,-0.76639999822378168659042785293422639369964599609375f,1.2063000034809103766519911005161702632904052734375f,1.0683000077128408378257518052123486995697021484375f,0.161400024747848647166392765939235687255859375f,0.15540000743269910543631340260617434978485107421875f,1.060400029188394821488827801658771932125091552734375f,-0.29060000138878816500209723017178475856781005859375f,1.010400001621245991145769949071109294891357421875f,0.19779998914599428161409377935342490673065185546875f,1.1550000214576723323034457280300557613372802734375f,0.04420002045035342774781383923254907131195068359375f,-0.395199999320507089350940077565610408782958984375f,-0.04340000334978100227090180851519107818603515625f,-0.906200047779083917021125671453773975372314453125f,0.55179999626874920437558103003539144992828369140625f,0.1458000097990037602357915602624416351318359375f,0.071199999201297714535030536353588104248046875f,-0.37690002319812787590080915833823382854461669921875f,-0.34659999262690543986309421597979962825775146484375f,-1.448600030088424972518623690120875835418701171875f,-0.0174000189661978055255531216971576213836669921875f,0.09240000406503678931358081172220408916473388671875f,0.0450000001490116208202607595012523233890533447265625f,0.9703000226736069322441835538484156131744384765625f,0.0919999981224537410895436551072634756565093994140625f,-0.163900014913082259226939640939235687255859375f,0.571400007015466560034155918401665985584259033203125f,0.651900007998943209486242267303168773651123046875f,0.394199996608495695227247779257595539093017578125f,-1.0468000061035152015165294869802892208099365234375f,-0.075599992597103149449822012684307992458343505859375f,-0.1841000270485881884496848215349018573760986328125f,-0.4794000029325484746323127183131873607635498046875f,0.01560000720024135034691425971686840057373046875f,-0.084299993735551748130774285527877509593963623046875f,-0.409500006690621365379456619848497211933135986328125f,0.416599993968009840727972914464771747589111328125f,-0.1740999975815415157232024512268253602087497711181640625f,-0.40979999334067096494749193880124948918819427490234375f,0.08540000229179867119455593638122081756591796875f,0.81730002243518828919377483543939888477325439453125f,0.313100002431869584018642171940882690250873565673828125f,-0.167399967780709146847328838703106157481670379638671875f,0.137999991297721891214678180404007434844970703125f,0.383400004690885420899348901002667844295501708984375f,-0.003700002926588064067203731610788963735103607177734375f,-0.1126999993950128509112573738093487918376922607421875f,0.314100007304549233566604016232304275035858154296875f,0.1330000127553940814095767564140260219573974609375f,0.15969999856054784004300017841160297393798828125f,-0.07679999656677249930680773104541003704071044921875f,0.008500002637505499336612047045491635799407958984375f,-0.459800008577108343388317734934389591217041015625f,-0.2442999963432550547537402962916530668735504150390625f,-0.306000003010034493655666665290482342243194580078125f,0.25590000126361844845490622901706956326961517333984375f,0.08189999905824663084530357082257978618144989013671875f,-0.12250000074505805469637920168679556809365749359130859375f,-0.1412000015296042132462162044248543679714202880859375f,-0.01610000714063646032325749501978862099349498748779296875f,-0.0164000052459538636639990727417171001434326171875f,0.21339999671876430653583156527020037174224853515625f,-0.45669999750256540238524394226260483264923095703125f,0.096600000599026625280885127722285687923431396484375f,-0.083899999639391875660976438666693866252899169921875f,0.3262999841094018638187890246626920998096466064453125f,-0.37020001310408112704664063130621798336505889892578125f,-0.302000005722046438449979177676141262054443359375f,-0.67920001240372673834144734428264200687408447265625f,-0.25219997740089894477932830341160297393798828125f,0.46450000609457475775343482382595539093017578125f,0.6871000100016591005669397418387234210968017578125f,0.94040002784729015417042319313623011112213134765625f,0.416200005683303064785150127136148512363433837890625f,0.103799998340010635189401000388897955417633056640625f,0.0679000347018234862161989440210163593292236328125f,-0.9558000065207483686435807612724602222442626953125f,-0.4438000019907946125385933555662631988525390625f,-1.261900018578767657118078204803168773651123046875f,-0.5651999695181846306013540015555918216705322265625f,-0.19389997646808598830148184788413345813751220703125f,0.06620000858902930218619076185859739780426025390625f,0.452200006979703861276220777654089033603668212890625f,-0.1182999595820906080234635737724602222442626953125f,0.53400001159310317433437376166693866252899169921875f,0.0529999698400498431283267564140260219573974609375f,-0.2379000043034551481468952260911464691162109375f,-0.58369997539669338237899864907376468181610107421875f,-0.52509998090341680754278286258340813219547271728515625f,0.677299968492985016155216726474463939666748046875f,0.277599999511242145899814204312860965728759765625f,0.25649996644258532541016393224708735942840576171875f,0.041799986135959699140585144050419330596923828125f,-0.341799986881017758832967956550419330596923828125f,-0.62189999607801438319398812382132746279239654541015625f,-0.143499998241662984899136290550814010202884674072265625f,-0.03650000153481958165002652094699442386627197265625f,-0.121200005087256368341286361101083457469940185546875f,0.66530001656413073618523412733338773250579833984375f,0.1710000001788138934699645687942393124103546142578125f,-0.11810000995695590919609685442992486059665679931640625f,-0.020300004345178646047997972345910966396331787109375f,-0.8061000190615654759085373370908200740814208984375f,0.0422000010192393748553740806528367102146148681640625f,-0.59050001417100428380280163764837197959423065185546875f,0.1580999908089637528973980806767940521240234375f,-0.3955000082850457232552798814140260219573974609375f,0.0474000069707631865867369924671947956085205078125f,0.4739000006079674420789160649292171001434326171875f,0.357599999362230303034948519780300557613372802734375f,-0.14599999764561655268835238530300557613372802734375f,0.0745999994218349282704139113775454461574554443359375f,-0.2706000052630901375749772341805510222911834716796875f,0.2946999965488910078192930086515843868255615234375f,-0.5621000164091587425474472183850593864917755126953125f,-0.1749999989569186453763904864899814128875732421875f,0.350200019437074683281707621063105762004852294921875f,0.0900999935880303637336652400335879065096378326416015625f,0.50460000355690703965905186123563908040523529052734375f,-0.18089999887943264411660493351519107818603515625f,-0.394499992609024019429853069595992565155029296875f,-0.189699996548891025582861402654089033603668212890625f,0.131300013017654482183615982648916542530059814453125f,0.0102000012576579646150776170543394982814788818359375f,-0.380400000056624332245291952858678996562957763671875f,-0.472500000596046021428264793939888477325439453125f,-1.50110001682639104814143138355575501918792724609375f,0.05489999661445654766112056677229702472686767578125f,-1.129100035542249802489322973997332155704498291015625f,0.44310000928640391748558613471686840057373046875f,1.03460004472136546382898814044892787933349609375f,0.0697000077247622584764030762016773223876953125f,-0.97409999620318377111516383592970669269561767578125f,-0.0692000065922737661594510427676141262054443359375f,-0.60030001686215417322500798036344349384307861328125f,-0.6561999798297879493702566833235323429107666015625f,-0.049199997800588590735060279257595539093017578125f,0.15109999730587020394523278810083866119384765625f,-0.493300001454353154173304574214853346347808837890625f,-0.556499979853629866966002737171947956085205078125f,-0.11169998960494975648316540173254907131195068359375f,-0.9749999615550049014700562111102044582366943359375f,0.07270001019835437006122447201050817966461181640625f,0.230599998669326122779921206529252231121063232421875f,-0.0906000091746450575413973638205789029598236083984375f,0.371800004874169809365724859162583015859127044677734375f,0.11009999919459223516327028846717439591884613037109375f,0.251599983239174207483301870524883270263671875f,-0.1858000072658061441188692697323858737945556640625f,-0.07290000430345511261975843808613717555999755859375f,-0.968700001734494886562742976821027696132659912109375f,-0.197699984270334583325023913857876323163509368896484375f,0.89540000428482857675049899626174010336399078369140625f,-0.64699997949600174251827411353588104248046875f,0.16939999317228793263012676106882281601428985595703125f,0.046100005874037730535519585828296840190887451171875f,0.52220000206232075612433618516661226749420166015625f,0.1099000020533800192623630209709517657756805419921875f,-0.2613999910712241359078689129091799259185791015625f,0.0797000022858380674506406649015843868255615234375f,-0.75320002205967906405703615746460855007171630859375f,0.139100001940131168254310978227294981479644775390625f,-0.3532000040292739839742353069595992565155029296875f,-0.217599997797608413208081401535309851169586181640625f,0.608899989283084863700423738919198513031005859375f,0.37460000128448012191029192763380706310272216796875f,-0.085099985951185175281352712772786617279052734375f,0.0289000006079673799064266859204508364200592041015625f,-0.219699998635053628248670065659098327159881591796875f,0.459699998784065311241420204169116914272308349609375f,-0.3940999762356280911035355529747903347015380859375f,0.204999993070960062624408237752504646778106689453125f,0.67190000472068778236689468030817806720733642578125f,0.0301999960795045208072906461893580853939056396484375f,-0.335599991293251520563245549055864103138446807861328125f,0.1141999971672892633112184057608828879892826080322265625f,-0.70359999823719265865662464420893229544162750244140625f,-0.26119999391138559730052293161861598491668701171875f,0.3118999819219112623613909818232059478759765625f,-0.0334999966770410662064705320517532527446746826171875f,0.209399997270107274260908525320701301097869873046875f,0.05809999691843985292205587711578118614852428436279296875f,0.0555999958939850102979107759892940521240234375f,0.3750000037252902984619140625f,0.954000002801418389708487666212022304534912109375f,0.896799956333637737770914100110530853271484375f,-0.11020000617504077666808370850048959255218505859375f,-0.4383999899387358567537376075051724910736083984375f,0.501100028449297241195381502620875835418701171875f,-0.1925999971270559996128213242627680301666259765625f,-0.1261999971151350763420850853435695171356201171875f,0.2654999835491178572510762023739516735076904296875f,-0.04830000145435331404542012023739516735076904296875f,-0.26299998861551276974068969138897955417633056640625f,-0.463700006204843528934134155861102044582366943359375f,0.2188000016927720281501024146564304828643798828125f,0.46639999405145626809598979889415204524993896484375f,-0.4517999970138071308412008875166065990924835205078125f,-0.38320000797808173587100100121460855007171630859375f,-0.020499992191790994411348947323858737945556640625f,-0.8592000243246555868381619802676141262054443359375f,0.86719997766166923014452549978159368038177490234375f,0.82399999939277768223888642751262523233890533447265625f,-0.01270001302957535216364703956060111522674560546875f,0.79200000751018517775037253159098327159881591796875f,-0.8455000041127203047608418273739516735076904296875f,0.14829999921917913496827168273739516735076904296875f,-0.091399975872039629365417567896656692028045654296875f,-0.24509999817013738976356762577779591083526611328125f,-0.38310000243186959067998031969182193279266357421875f,0.17140000880360606316799021442420780658721923828125f,0.751899965679645898575245155370794236660003662109375f,0.4021999762833117220139911296428181231021881103515625f,-0.5897999833941458547315050964243710041046142578125f,0.23050000083446509080431496840901672840118408203125f,-0.167999997705221115751328397891484200954437255859375f,-0.662999983847141205473008085391484200954437255859375f,0.1844000010699033964556292630732059478759765625f,0.068100006306171412262528974679298698902130126953125f,0.14140000016093257073634958942420780658721923828125f,-0.091599989497661571391518009477294981479644775390625f,-0.244700000870227807325818503159098327159881591796875f,0.25109999805092808600193166057579219341278076171875f,0.1969999995380640189779342108522541821002960205078125f,-0.1198999918460845748313658987171947956085205078125f,-0.007599998766183890808179057785309851169586181640625f,-0.26220000981092450587084385915659368038177490234375f,0.07980000246763230276059175594127736985683441162109375f,0.36640000641942027659325731292483396828174591064453125f,0.18570000455081459023176648770458996295928955078125f,-0.35960000359416000037526828236877918243408203125f,-0.1608000016406178345196309464881778694689273834228515625f,-0.69009999977201219589773018014966510236263275146484375f,-0.22849999719858171687292269780300557613372802734375f,0.499899991250038144841028042719699442386627197265625f,0.092700003269314823484137377818115055561065673828125f,0.773400020933151299828978153527714312076568603515625f,0.2670000003576278313488501225947402417659759521484375f,-0.4924000074923038372531891582184471189975738525390625f,-0.08049999867379664664213123614899814128875732421875f,0.6260000009238719798076999722979962825775146484375f,-0.21549999740719794072418835639837197959423065185546875f,-0.4088999956905841060006423504091799259185791015625f,0.03020000386536114689306486980058252811431884765625f,0.3610000056922435529571657752967439591884613037109375f,-0.54079998860210221067745806067250669002532958984375f,-0.42169999077841635592989177894196473062038421630859375f,0.2175000050850213195730020743212662637233734130859375f,-0.15550000245496631290365030508837662637233734130859375f,-0.036000014185905371277840458787977695465087890625f,0.86119999756216980557610440882854163646697998046875f,-0.3954999839961528262932688448927365243434906005859375f,-0.736500009581446679618466077954508364200592041015625f,-0.32009999951124168404703596024774014949798583984375f,-1.319200017470121277796124559245072305202484130859375f,0.52229999021142692328112389077432453632354736328125f,0.46080000650212171731112675843178294599056243896484375f,0.4529000016212467727427792851813137531280517578125f,0.449000003844499673277823603712022304534912109375f,0.8104999817609783718808103003539144992828369140625f,-0.2477000243544580371235497295856475830078125f,0.8917999670624734420698587200604379177093505859375f,-0.17610000833272909659399374504573643207550048828125f,0.2101999841213224584635099745355546474456787109375f,1.100600041472912238305070786736905574798583984375f,1.04349998736381532893346957280300557613372802734375f,0.5991000078260899641691139549948275089263916015625f,-0.421899969553948039902024902403354644775390625f,-0.86409999262690551091736779198981821537017822265625f,0.097600017392635241009202218265272676944732666015625f,-0.2379999947249888947453655418939888477325439453125f,0.5310000098645681898545944932266138494014739990234375f,0.641200026842951853467411638121120631694793701171875f,-0.24780000896453913128425483591854572296142578125f,0.1749000015318393774776950522209517657756805419921875f,0.0219999977126717760622653941027238033711910247802734375f,0.60520000412613150775342774068121798336505889892578125f,0.0121000003717839366146336033125407993793487548828125f,-0.275600010012835294759270254871807992458343505859375f,-0.255999995633959809282487185555510222911834716796875f,0.5868999878823757399004534818232059478759765625f,0.24189999920725824278378013332257978618144989013671875f,-0.05389999017715452200860681841732002794742584228515625f,0.0997000019878149235097453129128552973270416259765625f,0.21160001050829890800741850398480892181396484375f,-0.0587000060930847611384564288528054021298885345458984375f,-0.50129998995810753914526003427454270422458648681640625f,0.0547000037759541868354062899015843868255615234375f,-0.679000016063451727887922970694489777088165283203125f,-0.08090000767111771295958533301018178462982177734375f,-0.75449998173117638344820079510100185871124267578125f,-0.165000001490116066094060442992486059665679931640625f,-0.66079998729825017989014668273739516735076904296875f,-0.22940000010132788332839481881819665431976318359375f,0.22519999485015862461523283855058252811431884765625f,-0.193900000831484842223062514676712453365325927734375f,-0.810400019204616572920940598123706877231597900390625f,0.048299995419383012773550944984890520572662353515625f,0.42679998703002919313576057902537286281585693359375f,0.1503999974489211854944414881174452602863311767578125f,-0.4476000054717064191578401732840575277805328369140625f,-0.10780000553727153711491837384528480470180511474609375f,-0.40780001924633990650903569985530339181423187255859375f,-0.166600004622340147619752315222285687923431396484375f,-0.4522000191986563777390983887016773223876953125f,1.210599974715709237216287874616682529449462890625f,0.260300004941225182619746192358434200286865234375f,0.76899996554851579588785170926712453365325927734375f,0.261999989777803587998050716123543679714202880859375f,-0.269499982476234745121246305643580853939056396484375f,0.7668000128090379785561481185141019523143768310546875f,0.21739999847710134162070971797220408916473388671875f,0.264300008115172513356583294807933270931243896484375f,0.523500003047287609803106533945538103580474853515625f,-0.0994000000640749004077179051819257438182830810546875f,0.64249997228384037040171961052692495286464691162109375f,-0.296699994485079832085006046327180229127407073974609375f,0.24090000871419903205605805851519107818603515625f,-0.3822000098109246124522542231716215610504150390625f,0.16929999805688833447447905200533568859100341796875f,1.230600015395879598401052135159261524677276611328125f,-1.0038999874949450230587899568490684032440185546875f,-0.5234000166118146779581365990452468395233154296875f,0.809699982988834054964399911114014685153961181640625f,0.1397999702811236755195523073780350387096405029296875f,0.188000030040740284675848670303821563720703125f,-0.8236000169754031929869597661308944225311279296875f,0.1312999879837040140273529686965048313140869140625f,-0.97230001453757264329169629490934312343597412109375f,0.65509999198615520565880387948709540069103240966796875f,0.4673000039204955324834145358181558549404144287109375f,0.076799976003170566940525532118044793605804443359375f,0.7106999989628792757656583489733748137950897216796875f,0.89509996851682682716955241630785167217254638671875f,-0.101800010424852427348696437547914683818817138671875f,-0.307000007137656183431317913345992565155029296875f,-0.7311000011801720432913498370908200740814208984375f,-0.1737999965518712830458980533876456320285797119140625f,-0.51769999529719346043066252605058252811431884765625f,0.10509999697804450757843142127967439591884613037109375f,0.038499998241662947151553453295491635799407958984375f,0.068599998348951363169589967583306133747100830078125f,-0.42529999428689480023280111709027551114559173583984375f,0.041699999849498237125544619630090892314910888671875f,-0.7055000104829669549388881932827644050121307373046875f,-0.2112999987125396550435851850124890916049480438232421875f,-0.256099998833239084206070401705801486968994140625f,0.08509999802112577782509106327779591083526611328125f,-0.4593000034213066129495928180404007434844970703125f,-0.1609000008910894674585279062739573419094085693359375f,0.10999998733401294970235539949499070644378662109375f,0.1355000086575746109929241356439888477325439453125f,0.893999996691942300230948603712022304534912109375f,-0.23970000481903552813633950790972448885440826416015625f,-0.408600001478195196114029386080801486968994140625f,0.18159998979568481303203952847979962825775146484375f,-0.596800000739097669111288269050419330596923828125f,0.020700003060698524137706044712103903293609619140625f,-0.1543999925017356389389533433131873607635498046875f,0.010499993011355457372957289408077485859394073486328125f,-0.8160999873965979478640520028420723974704742431640625f,-0.08599999354779719329400222704862244427204132080078125f,0.072300005000829781209858992951922118663787841796875f,-0.2682999990701675496751477112411521375179290771484375f,-0.610300013509392780264306566095910966396331787109375f,-0.090399978971481775857910179183818399906158447265625f,-1.1691999812602993369381465527112595736980438232421875f,-0.078299976420403094579114622320048511028289794921875f,-0.59679998330473882361957294051535427570343017578125f,-0.5114999940097326724952608856256119906902313232421875f,-0.5608000167280435821481887614936567842960357666015625f,-0.2447999914407734678434280795045197010040283203125f,0.42840000796914079472799130599014461040496826171875f,0.143200014758109173129696500836871564388275146484375f,0.9966000260055063453279444729560054838657379150390625f,-0.0449000109195705210396454276633448898792266845703125f,0.69700001592934157645942150338669307529926300048828125f,-0.14129996980428760622316985973156988620758056640625f,-0.654799986970424630072784566436894237995147705078125f,0.488699977743625613157973930356092751026153564453125f,0.55650001025199902215945257921703159809112548828125f,0.3410000007748603678692234097979962825775146484375f,1.08409999009370761058335119741968810558319091796875f,0.2961999720811849812207583454437553882598876953125f,0.99519998978376378317989292554557323455810546875f,0.638699960756302420605834413436241447925567626953125f,-0.2565000074207779423574038446531631052494049072265625f,0.00480001111030592397810323745943605899810791015625f,-0.697800008368492097332591583835892379283905029296875f,-0.015199997122585795494842386688105762004852294921875f,0.2267000061079860184776180176413618028163909912109375f,0.260499970138073688730173671501688659191131591796875f,0.4061999789357184642568654453498311340808868408203125f,0.87299995583295864065576097345910966396331787109375f,-0.22629998783469196244055865463451482355594635009765625f,0.09919999444782734432379811551072634756565093994140625f,-0.06430000282526027177709693205542862415313720703125f,0.2098999913990498011884255902259610593318939208984375f,-0.3598999912500382425406542097334749996662139892578125f,0.2247000052660703683926612939103506505489349365234375f,0.11320000894665727120269593797274865210056304931640625f,-0.103200002613663688322276357212103903293609619140625f,0.50309999244809144869350348017178475856781005859375f,-0.10720000846982002951079948616097681224346160888671875f,0.0289999892413614901442997506819665431976318359375f,0.035700004178285615896726312712416984140872955322265625f,-0.062899994543194726048795928363688290119171142578125f,0.1376999923169612838336206550593487918376922607421875f,-0.1177000049829484140673230285756289958953857421875f,-0.3230999984830618121378620344330556690692901611328125f,0.1391999881148338058522995197563432157039642333984375f,-0.062999999940395401409887199406512081623077392578125f,-0.3088000128686427814272974501363933086395263671875f,-0.19459999226927759963245989638380706310272216796875f,0.04959999450445184976388190989382565021514892578125f,-0.29069999389648437659872115546022541821002960205078125f,-0.07170000152587885366983755375258624553680419921875f,0.1336000004351138858993408575770445168018341064453125f,-0.27780000330209730918795685283839702606201171875f,0.086799998690187951666530352667905390262603759765625f,-0.191999994136393070665036475475062616169452667236328125f,-0.15769999149739744037646005381247960031032562255859375f,0.035599999451637387437585857696831226348876953125f,-0.125399999237060466583670859108678996562957763671875f,0.8617999950766563888038263030466623604297637939453125f,-0.62099998825788471634723464376293122768402099609375f,-0.9521000176012517357548858853988349437713623046875f,-0.4209999694824215765720509807579219341278076171875f,0.1970000224113466202879862976260483264923095703125f,-0.946199991154669906023855219245888292789459228515625f,-0.3603999889552589319663411515648476779460906982421875f,0.50599999004602391750040624174289405345916748046875f,0.408499999433755700550818801275454461574554443359375f,-0.347100008213519828359494567848742008209228515625f,-1.2116000052928921348893709364347159862518310546875f,-1.1575999744772911270729309762828052043914794921875f,-0.44490002313852361481849584379233419895172119140625f,0.85909997698068618632305515347979962825775146484375f,0.099799980860948434013835139921866357326507568359375f,0.094000003993510716782111558131873607635498046875f,0.5631000235915184504165154066868126392364501953125f,0.3188000109314916841185549856163561344146728515625f,-0.589000030219555270605269470252096652984619140625f,-0.5480999799311161435610983971855603158473968505859375f,0.1765000355280939292601516399372485466301441192626953125f,0.449099971131981134941923983205924741923809051513671875f,0.43449997835978859672678709102910943329334259033203125f,0.5939999527335171336517305462621152400970458984375f,0.2236999601602551734913504333235323429107666015625f,0.9425999738812453898617604863829910755157470703125f,0.76309998842477799740890986868180334568023681640625f,-0.553599982106685928329170565120875835418701171875f,-1.27190002260208157025545006035827100276947021484375f,-0.5839999873042103217812837101519107818603515625f,-0.234799998295307066342729740426875650882720947265625f,0.036600005665421431189088252722285687923431396484375f,0.691699989902973211286507648765109479427337646484375f,0.30900000339746469535384676419198513031005859375f,-0.22300001364946364645902576739899814128875732421875f,0.32780000926256180893147984534152783453464508056640625f,0.37780001119971284850862502935342490673065185546875f,-0.2840000031739473573821896934532560408115386962890625f,0.03649998760223382987533113919198513031005859375f,0.05150000205636029448896806570701301097869873046875f,0.34850001686811449275182894780300557613372802734375f,0.176400002694129920399745969916693866252899169921875f,0.9770000014007091948542438331060111522674560546875f,-0.2035000017434358543511052630492486059665679931640625f,-0.54760000025629995690223950077779591083526611328125f,-0.14560000362396241513351924368180334568023681640625f,0.19579998938441267597454498172737658023834228515625f,0.12999999523162841796875f,-0.643599996709823596319210992078296840190887451171875f,0.203299999368190764670316639239899814128875732421875f,0.1694000019453465455399054917506873607635498046875f,-0.1090999928876757729678814712315215729176998138427734375f,-0.2985999986842273745679676721920259296894073486328125f,-0.3151999955952167198347524390555918216705322265625f,0.01360001205801975032727568759582936763763427734375f,-0.178999992668628760128513022209517657756805419921875f,-0.73400001814961424173588966368697583675384521484375f,0.073999997884035106920919133699499070644378662109375f,0.5424000024259090224632018362171947956085205078125f,0.087600005918741175037212087772786617279052734375f,-0.30740000153183899556097458116710186004638671875f,-0.26260000129938088075931545972707681357860565185546875f,-0.120200002636015323975726687422138638794422149658203125f,-0.3523000126004216525643641944043338298797607421875f,-1.1182000287652016368156182579696178436279296875f,-0.9619999914169312660305877216160297393798828125f,0.19129999752044657412852757261134684085845947265625f,0.1187000003933906810971166123636066913604736328125f,0.80960002028346078617460079840384423732757568359375f,0.60599997782707237803379030083306133747100830078125f,0.96060000302791603843388656969182193279266357421875f,1.361299975764751479800906963646411895751953125f,0.3002000058770182278067295555956661701202392578125f,-0.0164999703168877687176063773222267627716064453125f,-0.96190001455545459663198926136828958988189697265625f,0.04629999971836806782476969601702876389026641845703125f,-0.128800003406405461081618568641715683043003082275390625f,0.080400001025199951953936761128716170787811279296875f,-0.40180001444816593192399523104541003704071044921875f,-0.945899980401993190071152639575302600860595703125f,-0.3251000081539157093857284053228795528411865234375f,0.4209000108003611018148149014450609683990478515625f,-0.90500003516674087933324699406512081623077392578125f,-0.23740001203715810351724257998284883797168731689453125f,0.0388000000163912783790465255151502788066864013671875f,-0.3095999973356722279049790813587605953216552734375f,0.4995000152587891761868377216160297393798828125f,-0.03999999783933183561401847327942959964275360107421875f,0.1391000109925866201621147411060519516468048095703125f,0.82579995019435958880649195634759962558746337890625f,-0.7831999851047992677877118694595992565155029296875f,0.1026000039815902764583910311557701788842678070068359375f,0.0792000054374337736362576833926141262054443359375f,-0.04969999639987943140795323415659368038177490234375f,0.58190001217126852139926995732821524143218994140625f,0.168799999606609407720725357648916542530059814453125f,0.733800022405385998780502632143907248973846435546875f,0.094499998793005968167335595353506505489349365234375f,-0.39519999470114708373102985206060111522674560546875f,-0.04999999985098835253438664949499070644378662109375f,-0.643799990963935808707674368633888661861419677734375f,-0.26890000150203707818263865192420780658721923828125f,-0.9973000155806541755509897484444081783294677734375f,0.178099987977743179357048575184307992458343505859375f,-0.3418999923527241691090239328332245349884033203125f,-0.0011000000253319763832138278303318656980991363525390625f,-0.045600000345706936943912523929611779749393463134765625f,0.1835999973803759122148449023370631039142608642578125f,-0.4954999822080135007951184888952411711215972900390625f,0.18829999638795857919149057124741375446319580078125f,0.69530001865029333885104279033839702606201171875f,0.050699991214275375028819325962103903293609619140625f,-0.566499994754791291740048109204508364200592041015625f,-0.128199999484419790984901510455529205501079559326171875f,0.57060000816881650820278082392178475856781005859375f,0.052400002649426458134485073969699442386627197265625f,0.2192999996960163233694629525416530668735504150390625f,-0.158799999681115189265057097145472653210163116455078125f,0.403999985143542239551806005692924372851848602294921875f,-0.2297999934524297405147308381856419146060943603515625f,-0.09390000627040873126816222793422639369964599609375f,-0.44289996436834400128645938821136951446533203125f,-0.19799998712539679246447121840901672840118408203125f,-0.3820000016987317170702453950070776045322418212890625f,-0.6381000276893378231335418604430742561817169189453125f,-0.69759997835159293089191123726777732372283935546875f,-1.15440000457167624148269169381819665431976318359375f,-0.0474000013828277388938658987171947956085205078125f,-0.920300019991397988405879004858434200286865234375f,0.06750001132488225863426123396493494510650634765625f,-1.098699981319903873355769974295981228351593017578125f,-0.0544999870955942267158889080747030675411224365234375f,-0.327100006052851721705110321636311709880828857421875f,-0.650699987637996191125466793891973793506622314453125f,0.1494000130653383440204606813495047390460968017578125f,-1.1062999864935871840998515835963189601898193359375f,-0.1949999764561649584493352449499070644378662109375f,0.84479998309612280849023591144941747188568115234375f,0.7549999971687793021146717364899814128875732421875f,0.17719998388290481017293132026679813861846923828125f,1.035400016671419454183933339663781225681304931640625f,-0.39639997244477320759159510998870246112346649169921875f,-0.342999985523521899732912743274937383830547332763671875f,0.61309997292757056897016809671185910701751708984375f,0.009700001168251315419865932199172675609588623046875f,-0.3392999798774720687788430950604379177093505859375f,-1.3573000368893151090787796420045197010040283203125f,0.29979998965263376931034144945442676544189453125f,0.5041000125944616883089111070148646831512451171875f,0.144099993371963108756972360424697399139404296875f,-0.43160000186562541557577787898480892181396484375f,0.388099989765882558145904113189317286014556884765625f,-0.359299981367588117109335144050419330596923828125f,0.179200006517767906899507579510100185871124267578125f,0.06910000976324093269198556299670599400997161865234375f,0.08400000376999372520003817044198513031005859375f,0.682899987167119970621342872618697583675384521484375f,-0.1369999978244305083308063331060111522674560546875f,0.414199989157915116066988048260100185871124267578125f,0.0700999962329864612087249042815528810024261474609375f,-0.186500000864267434508292353712022304534912109375f,0.12170000167489047893099041175446473062038421630859375f,-0.19890000835657117850274744341732002794742584228515625f,0.27830000525414944068103295649052597582340240478515625f,0.11940000621080404386020745732821524143218994140625f,0.2387000051617622631283666123636066913604736328125f,0.7619000109791755903643206693232059478759765625f,-0.188800000128150013978256538393907248973846435546875f,-0.596300011080503455929147094138897955417633056640625f,0.0052000083357095494562827298068441450595855712890625f,-0.2703000049412249694569254643283784389495849609375f,0.1443999950349330874121989154446055181324481964111328125f,0.15139999714344731795989673628355376422405242919921875f,-0.1794999985694885058506997665972448885440826416015625f,-0.309299997013807281831532236537896096706390380859375f,0.0432999997407197412258028634823858737945556640625f,0.36519998978376388976130328956060111522674560546875f,0.0877999981611967683647890225984156131744384765625f,-0.577699984121322618335625520558096468448638916015625f,-0.1697000032544135894596593061578460037708282470703125f,-0.659800019156933093000816370476968586444854736328125f,0.2862000233411787775139600853435695171356201171875f,-0.67820001699328447131165376049466431140899658203125f,0.205599996508657845151191168042714707553386688232421875f,-0.26860001237466946122367517091333866119384765625f,0.05179999597072626471572220907546579837799072265625f,-0.2879000065386294604508066186099313199520111083984375f,0.051000000029802095014019869267940521240234375f,-0.7799000325262550603611089172773063182830810546875f,-0.5614999685287482833473404753021895885467529296875f,-0.762499991655349873553859652020037174224853515625f,0.88169997708797420710169490121188573539257049560546875f,0.259200026001036587519621434694272466003894805908203125f,-0.1714999701678758814438197077834047377109527587890625f,-0.416799983975291166160559441777877509593963623046875f,0.13029995621442846953641492291353642940521240234375f,0.1411999716341494348625928978435695171356201171875f,-0.143199987637996795086792189977131783962249755859375f,-0.013300000709295378698016065754927694797515869140625f,-0.7720999961435797143138870524126105010509490966796875f,-0.665000000521540624021099574747495353221893310546875f,-0.5238000088453294011969774146564304828643798828125f,0.88610002740621585104463520110584795475006103515625f,0.929399979686736710249306270270608365535736083984375f,0.0771999937176703898700225181528367102146148681640625f,-0.008100028359889765994239496649242937564849853515625f,0.1482999871492385324245333322323858737945556640625f,-0.223299988192319887048142845742404460906982421875f,0.74640001372098918608344320091418921947479248046875f,-0.3048999922931192596564642371959052979946136474609375f,0.0739000015020370160101492729154415428638458251953125f,0.119399997493624621114349793060682713985443115234375f,-0.57670001717209817826415019226260483264923095703125f,-0.06160000130683183916513456779284751974046230316162109375f,-0.087300000735372307314463569127838127315044403076171875f,0.03180000215470790703164993828977458178997039794921875f,-0.1196999983370303777263643496553413569927215576171875f,-0.121800001633167287451442462042905390262603759765625f,0.2569999958872795087216900355997495353221893310546875f,0.01429999477863308499081540503539144992828369140625f,0.6636999874293803003411085228435695171356201171875f,-0.32930000386834146208769880104227922856807708740234375f,-0.293700007583201028182884328998625278472900390625f,0.29109999067485337054250749133643694221973419189453125f,-0.44519998918771752638434691107249818742275238037109375f,-0.313899992635846114552578001166693866252899169921875f,0.559200010019540894035117162275128066539764404296875f,0.110799998176097869162504139239899814128875732421875f,-0.0849999992549419491894013845012523233890533447265625f,0.03009999906420712800780847828718833625316619873046875f,0.87010001575350759406290990227716974914073944091796875f,-0.071799999099969813443067323532886803150177001953125f,0.736600011402368526347572696977294981479644775390625f,-0.30860000453293323641190681883017532527446746826171875f,-0.38830000681877141488484994624741375446319580078125f,-0.2478999957352876482019610193674452602863311767578125f,0.5702000058770180235256930245668627321720123291015625f,0.13409999970495700694073093472979962825775146484375f,-0.0812000000953674305748108963598497211933135986328125f,0.15850000122189522588911358980112709105014801025390625f,0.8350000053644179853762352649937383830547332763671875f,0.2360000226795673228252780972979962825775146484375f,-0.08659999471306800700176609097979962825775146484375f,-0.30649999356269841399580400320701301097869873046875f,-0.759699992376565713669833712629042565822601318359375f,-0.72199998664855957741792735760100185871124267578125f,-0.31500000402331362892027755151502788066864013671875f,0.457499984502792500506984652020037174224853515625f,0.000300021630526003946215496398508548736572265625f,0.635900000965595069146729656495153903961181640625f,1.2568000159382819447273504920303821563720703125f,0.250799998772144316916410389239899814128875732421875f,-0.296300004374980829879859811626374721527099609375f,-0.189799971435964331956114392596646212041378021240234375f,-0.53979999455139038300188758512376807630062103271484375f,-0.99880000139474844900178140960633754730224609375f,-0.1507000217616560888700405484996736049652099609375f,0.1347999790728096058955998159945011138916015625f,-0.33799999010562853385408743633888661861419677734375f,-0.871299987983702894922544146538712084293365478515625f,-0.66620001559257513434886277536861598491668701171875f,0.086099985384941124522129030083306133747100830078125f,0.45390000388622286919826365192420780658721923828125f,0.27150000876188329357319162227213382720947265625f,-0.937400026118755391735248849727213382720947265625f,0.47159997541606457627239024077425710856914520263671875f,0.5493999894842505060665871496894396841526031494140625f,-0.5214999920725826854095430462621152400970458984375f,-0.38959999077916140919342069537378847599029541015625f,0.641299957883358473509360919706523418426513671875f,-0.335200000214576743218231058563105762004852294921875f,0.177599998319148966174907400272786617279052734375f,-0.6798000145375728830998696139431558549404144287109375f,0.0153000005453824972079246435896493494510650634765625f,0.36179999865293499539120603003539144992828369140625f,0.15240000086128713263633471797220408916473388671875f,0.514100008422136323105178234982304275035858154296875f,-0.307099995771050515713795903138816356658935546875f,0.166199975657463028255733661353588104248046875f,-0.055500002920627622415850055404007434844970703125f,-0.68070001296997073580996584496460855007171630859375f,0.1359999966770411017336073200567625463008880615234375f,-0.5916999854326248087232897887588478624820709228515625f,0.27859999562948944973328480045893229544162750244140625f,-0.38909999899715186355564355835667811334133148193359375f,0.13760000375807290851071229553781449794769287109375f,0.86510002424716958557837642729282379150390625f,-0.17879999431669713061410220689140260219573974609375f,0.2676999954462051523051968615618534386157989501953125f,0.1541000107318162282155071807210333645343780517578125f,0.7025999913156033560568403117940761148929595947265625f,-0.1380999989300966124261549339280463755130767822265625f,0.1932000022411346495943007539608515799045562744140625f,0.180100003385543772083110525272786617279052734375f,-0.68590000811815254877501502051018178462982177734375f,-0.137999997332692136975396124398685060441493988037109375f,-0.264399996748566568083305128311621956527233123779296875f,0.017099996218085333765657196636311709880828857421875f,0.5922000108540057539130430086515843868255615234375f,-0.453699992421269460152188912616111338138580322265625f,0.1628999661564822787340744980610907077789306640625f,0.00159998353719714714316069148480892181396484375f,0.0371999998271466125743245356716215610504150390625f,-0.692199963915348437382135671214200556278228759765625f,-0.94699998083710656970879426808096468448638916015625f,-0.6513999855950476369770285600679926574230194091796875f,-0.50569999145641919557903065651771612465381622314453125f,-0.22419998617768310822384592029266059398651123046875f,0.10890000701546664885199788841418921947479248046875f,-0.27599998199939790310963871888816356658935546875f,1.01120002766251548820264360983856022357940673828125f,0.234699980902671967442074674181640148162841796875f,0.159799995166063357743269079946912825107574462890625f,0.61419997678995141843216742927324958145618438720703125f,0.327999976359307499063078239487367682158946990966796875f,0.80759996970892000689445922034792602062225341796875f,0.686699978131055654984038483235053718090057373046875f,0.19339999500513072039353801301331259310245513916015625f,-0.057900014026463220151441646521561779081821441650390625f,0.387600000703334668372690430260263383388519287109375f,0.530500012308359192303441886906512081623077392578125f,-0.57729999173879686935606514452956616878509521484375f,-0.9289999914765356692214481881819665431976318359375f,0.843299965095520409619211932295002043247222900390625f,0.605199989485740719175055346568115055561065673828125f,0.9866999857306473220575071536586619913578033447265625f,0.24740001024901869808303445097408257424831390380859375f,-0.906499992668627907477230110089294612407684326171875f,-0.704500018686056161953956689103506505489349365234375f,-0.78009998267292957052632118575274944305419921875f,0.42229997996687895778222809894941747188568115234375f,-0.131999998942017526815106975845992565155029296875f,0.298700007098913200564993530861102044582366943359375f,0.1289999983310698983274278361932374536991119384765625f,0.5633999845743178358503655545064248144626617431640625f,0.0102000003263354177551747170582530088722705841064453125f,0.7661999960348009341970509922248311340808868408203125f,0.148599996560812019907871217583306133747100830078125f,-0.0823999933362007386250525087234564125537872314453125f,-0.070400004079937961165569504373706877231597900390625f,0.436299986344575785324195749126374721527099609375f,-0.1278999984174966630945391443674452602863311767578125f,0.00960000478625300246449114638380706310272216796875f,-0.2229999993443489147981750875260331667959690093994140625f,-0.22640000206083055189765218528918921947479248046875f,-0.2327999951809645029499051815946586430072784423828125f,-0.448500008225440893738777958787977695465087890625f,-0.0850000004470348458074369091264088638126850128173828125f,-0.0296000025138258848755867802537977695465087890625f,-0.043200002092123046537608388462103903293609619140625f,-0.3820000079572201112654283861047588288784027099609375f,0.033699993389844973279423356871120631694793701171875f,0.645900019741058262212618501507677137851715087890625f,-0.20689999734461306246657841256819665431976318359375f,-0.5457999971330165234206788227311335504055023193359375f,-0.351100008183717726684136550829862244427204132080078125f,-0.36650001077353966394412054796703159809112548828125f,0.21290001026391980332164166611619293689727783203125f,0.6111000031173230429004661345970816910266876220703125f,0.120100005173683133108397669275291264057159423828125f,0.5462000024795532482357884873636066913604736328125f,-0.3412000012874596421852402272634208202362060546875f,-0.6583000275313855009784447247511707246303558349609375f,-0.807999978184700040628740680404007434844970703125f,0.15090000439286210820455380599014461040496826171875f,-0.6504000059425825153169853365398012101650238037109375f,-0.5391000259310008146940162987448275089263916015625f,-0.109200001823902770325958044850267469882965087890625f,0.4758000175476073678737520822323858737945556640625f,-0.262299993824958777022260392186581157147884368896484375f,0.85070002680942435890898423167527653276920318603515625f,0.3205999815702440525200245247106067836284637451171875f,-0.56380000266134711051080330435070209205150604248046875f,0.794699972856044656310814389144070446491241455078125f,-0.1125999863982196647071987172239460051059722900390625f,-0.1633999878525731475065185804851353168487548828125f,0.3046000202834608927560111624188721179962158203125f,1.0150999765634534099945085472427308559417724609375f,0.45269998933672894736446323804557323455810546875f,-0.160100022645294881584732138435356318950653076171875f,-0.5626000168338418649938148519140668213367462158203125f,-0.18579999802708613287904881872236728668212890625f,0.64240000674724573315188536071218550205230712890625f,0.3110999764442448078938241451396606862545013427734375f,0.78509998498260957777716839700588025152683258056640625f,0.26870002810955018190952614531852304935455322265625f,-0.57150000339746487298953070421703159809112548828125f,-0.12849999392032618317216474679298698902130126953125f,0.300600004369020457062333662179298698902130126953125f,0.186099982106686212546264869160950183868408203125f,-1.014700019943714170977955291164107620716094970703125f,0.18080000674426555118401438448927365243434906005859375f,0.0185000080764294860813379273167811334133148193359375f,0.068400003275275178538095133262686431407928466796875f,0.784899987673759458317590542719699442386627197265625f,0.15720000616014007110976535841473378241062164306640625f,0.42990001911520969724023188973660580813884735107421875f,-0.2172000045210122909367100874078460037708282470703125f,0.6026999973833560009239818100468255579471588134765625f,0.08230000082850456866179200687838601879775524139404296875f,0.0782000001735985161843700552708469331264495849609375f,-0.124599998006224621516224715378484688699245452880859375f,-0.06369999793469904858511654310859739780426025390625f,-0.31510000137388705088170581802842207252979278564453125f,-0.20600000464916234932388761080801486968994140625f,0.25380000102221966784554751939140260219573974609375f,-0.404299995821714386323719736537896096706390380859375f,-0.17030000620782370646111303358338773250579833984375f,-0.8239999908804893635760890902020037174224853515625f,-0.190700001682341024888955871574580669403076171875f,0.6522000208005309662695481165428645908832550048828125f,0.0210000023394822932232273160479962825775146484375f,0.1388999967336654872696044549229554831981658935546875f,-0.137499995231628424630088147750939242541790008544921875f,-0.088499999508261595337899052537977695465087890625f,0.15849999511241907867997724679298698902130126953125f,-0.650900014823675210351439091027714312076568603515625f,0.1344999995380640189779342108522541821002960205078125f,-0.171799994033575043062000986537896096706390380859375f,0.18630000199079521649991875165142118930816650390625f,-0.031800054830313229103921912610530853271484375f,0.47469996510744161266615037675364874303340911865234375f,0.377299979706108568056066587814711965620517730712890625f,0.79929997838735555859557280200533568859100341796875f,0.539799999338388403202770859934389591217041015625f,0.0761999509215360859570864704437553882598876953125f,0.93609999283552181026379912509582936763763427734375f,-0.5099999850988385929895230219699442386627197265625f,-1.0394000236451628182976492098532617092132568359375f,0.24599998766183883702751700184307992458343505859375f,0.505300007325411026926076374365948140621185302734375f,0.44329999459981905829408788122236728668212890625f,0.088399997240304895029794352012686431407928466796875f,0.17349999757111067566484052804298698902130126953125f,0.091899999430775647368818681570701301097869873046875f,-0.5474999758601182264783346909098327159881591796875f,-0.09800000798702246385118996840901672840118408203125f,-0.448099973821640684690237321774475276470184326171875f,-0.629199984389543498508601260255090892314910888671875f,0.053199991586804440402147520217113196849822998046875f,-0.087199997517466432128685482894070446491241455078125f,-0.182700001220405194857221431448124349117279052734375f,0.1070000006929039404468539942172355949878692626953125f,-0.37449997025728265498401015065610408782958984375f,-0.664799991887807806278942734934389591217041015625f,0.5991999857306478105556379887275397777557373046875f,-0.0226000063657760819069153512828052043914794921875f,-0.97889998987913084960155174485407769680023193359375f,-0.55160002779364614156065727001987397670745849609375f,0.1408999795079228789518310804851353168487548828125f,0.06070000626444826963279410847462713718414306640625f,-0.1626999933600425662927335679341922514140605926513671875f,-0.203099995018541701341519001289270818233489990234375f,-0.264500003412365902732972244848497211933135986328125f,-0.359000002652406724479305921704508364200592041015625f,-0.220599993193149490622317898669280111789703369140625f,0.37040002382397663183155600563623011112213134765625f,0.0300000049918889288846912677399814128875732421875f,0.58069998540282252719180178246460855007171630859375f,0.0112000021070242095078128841123543679714202880859375f,-0.3390999951601028072900589904747903347015380859375f,-0.1221000015825033369054608556325547397136688232421875f,-0.245200007814168952080535746063105762004852294921875f,0.020899998208880443684165584272705018520355224609375f,-0.25609999499619007234940681883017532527446746826171875f,0.413500005692243544075381578295491635799407958984375f,-0.2994999994635580975455013685859739780426025390625f,0.0828999935746193017394034541212022304534912109375f,0.3668999987602232781824795893044210970401763916015625f,-0.0843999989092349967023665158194489777088165283203125f,0.13510000003278255231720095252967439591884613037109375f,-0.034500000171363343071107010473497211933135986328125f,0.288400002790987486367413339394261129200458526611328125f,0.2836999939113855617733861436136066913604736328125f,-0.1777999924242497087334413663484156131744384765625f,-0.14720000399947164027025792165659368038177490234375f,-0.365399996554851558272503098123706877231597900390625f,0.24970000586211682502835174091160297393798828125f,0.604400014406442753767123576835729181766510009765625f,-0.194100002610683386450318721472285687923431396484375f,-0.90160002928376226094542289501987397670745849609375f,0.789399992054701016286344383843243122100830078125f,-0.31480002005100260475956019945442676544189453125f,0.0302000103473663017439321265555918216705322265625f,-0.9533999871075149457055886159650981426239013671875f,0.1903999914884568767092787311412394046783447265625f,-0.8148000440418723400171074899844825267791748046875f,-0.446599971020221975237518563517369329929351806640625f,0.4043000189185141923786659390316344797611236572265625f,-1.0331999924063683948816105839796364307403564453125f,-0.1696999634683127311518546775914728641510009765625f,0.5125999789476392010101335472427308559417724609375f,-0.31320002787113221387471639900468289852142333984375f,0.02069999002218292361021667602472007274627685546875f,0.809300020855665547259150116587989032268524169921875f,-0.3031999918103220892362514859996736049652099609375f,0.97090004701018361998876571306027472019195556640625f,-0.185399990892410215792551753111183643341064453125f,-0.0941999975025653579763229572563432157039642333984375f,-0.142299979519843855513272501411847770214080810546875f,-0.71540000236630429952811027760617434978485107421875f,-0.48630001599788652555389489862136542797088623046875f,-0.765100015455484339099712087772786617279052734375f,-0.15449999499320998808116200962103903293609619140625f,-1.029300010722875580171375986537896096706390380859375f,0.12559996428489750286416892777197062969207763671875f,0.73759998490810385618487998726777732372283935546875f,0.44829999459981895171267751720733940601348876953125f,0.829000030070543569848950937739573419094085693359375f,0.627999993085861518693491234444081783294677734375f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_8item_large_5o3i_uneven_filter_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{8,3,16,16};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{8,5,30,29};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.1838999768406155421729408772080205380916595458984375f,-0.1766000023126601536915813994710333645343780517578125f,0.5644999911189076957640509135671891272068023681640625f,0.85430000595450383826801044051535427570343017578125f,-0.0899000499606132930097146527259610593318939208984375f,0.130299989891052181434361045830883085727691650390625f,-0.836399994163215421139057070831768214702606201171875f,0.0786999825119973905884762643836438655853271484375f,-1.6740999867782002663574303369387052953243255615234375f,-0.190300012093782555666621192358434200286865234375f,-1.125299976889788666767344693653285503387451171875f,-0.8074000163957475439957534035784192383289337158203125f,0.3775999926939606066156329688965342938899993896484375f,-0.87740000242590898693606504821218550205230712890625f,0.5976999956324695606468822006718255579471588134765625f,-0.36739998588562006176516661071218550205230712890625f,0.7733000159829857000914898890187032520771026611328125f,-0.31230001230239867737736858543939888477325439453125f,0.58659999929517570027570627644308842718601226806640625f,-0.6096999904394149183417539461515843868255615234375f,0.2131999691233037452775533893145620822906494140625f,-0.5744000129163264745102424058131873607635498046875f,0.2586000163421033892774403284420259296894073486328125f,-0.02400001069903368033919832669198513031005859375f,-1.3206000204995278490827104178606532514095306396484375f,-0.4679000212907791222960440791212022304534912109375f,-1.41859999935477976151787515846081078052520751953125f,0.646100020998716217945911921560764312744140625f,-0.0054999968111513197754902648739516735076904296875f,-1.7425000502169130101037808344699442386627197265625f,-1.1171999683856963514472226961515843868255615234375f,0.42890001189559701089137888629920780658721923828125f,0.69649999151378860773320411681197583675384521484375f,-0.017599999585747649888389787520281970500946044921875f,0.223699989441036972692700146581046283245086669921875f,-0.02410006601512504342821330283186398446559906005859375f,0.7455999738216405603452585637569427490234375f,-1.19750000879168538858721149154007434844970703125f,-0.0782000090956689319909855839796364307403564453125f,0.812599997946619634348053295980207622051239013671875f,-1.21370000963211044364697954733856022357940673828125f,-0.475500011861324178852328259381465613842010498046875f,-0.980899965649843608161972952075302600860595703125f,0.9872999969914548756833028164692223072052001953125f,0.40769997246116407296767647494561970233917236328125f,-0.112799983986466845209406528738327324390411376953125f,0.362299995259195650731953719514422118663787841796875f,-0.3196000102438032453022742629400454461574554443359375f,-0.4597999954640865638566538109444081783294677734375f,0.511199999052286191414395943866111338138580322265625f,1.8652000240564348132465966045856475830078125f,-0.55259998170435409292622352950274944305419921875f,0.2127000172764059726659979787655174732208251953125f,-0.69250004924833774566650390625f,-1.925999966502189408856793306767940521240234375f,-1.2980000303387644322583582834340631961822509765625f,0.24460001208782244930262095294892787933349609375f,-0.560400042450428514939630986191332340240478515625f,-0.745700008574127881644244553172029554843902587890625f,-0.45139998391866686944240427692420780658721923828125f,0.8043999991327523968465129655669443309307098388671875f,0.1823999992966651717551940237171947956085205078125f,0.6148000301837923675662977984757162630558013916015625f,-0.247400002537667640378771238829358480870723724365234375f,0.17660003289729342412783807958476245403289794921875f,-0.0087999973714349977171877981163561344146728515625f,-1.051399970544875106526205854606814682483673095703125f,0.2668999802827836109742065673344768583774566650390625f,0.721599995085596912502978739212267100811004638671875f,-0.84049999852478496364938109763897955417633056640625f,-0.1289999944195148184888921605306677520275115966796875f,-0.4398999921441077987083190237171947956085205078125f,-0.5151000179141760337842015360365621745586395263671875f,0.089899990206956825744555317214690148830413818359375f,-0.222599982319027223098828471847809851169586181640625f,0.1531999998569488763422441479633562266826629638671875f,-0.26010000649616127343932703297468833625316619873046875f,-0.1077000023007392837115503425593487918376922607421875f,0.06680001899302023815607753931544721126556396484375f,0.92740002090334883178002201020717620849609375f,0.385600017221272128864484329824335873126983642578125f,0.082700007367134009683695694548077881336212158203125f,0.6095999743506315127206107717938721179962158203125f,-0.95079997895359991133545918273739516735076904296875f,1.6544000296428797991410419854219071567058563232421875f,0.36000000283122091104814899154007434844970703125f,1.7276999956324694540654718366567976772785186767578125f,0.78540002389848240227365749888122081756591796875f,-0.163800011043250481890254377503879368305206298828125f,-0.62640001971870662433872212204732932150363922119140625f,-0.2886999935388558924387325532734394073486328125f,0.2694000607490547594125018804334104061126708984375f,0.3318999963760376203936175443232059478759765625f,0.49449999886751161426445833058096468448638916015625f,-1.729800000381469970989201101474463939666748046875f,-0.8460999975293876929072212078608572483062744140625f,-0.83839996058344912199800091912038624286651611328125f,-0.259400019472837417566779549815692007541656494140625f,-1.059299980622530057416952331550419330596923828125f,-0.15170000793337834466001368127763271331787109375f,-0.408099987232684924265413428656756877899169921875f,-0.80939999399185236228504436439834535121917724609375f,0.75530000240802763755709747783839702606201171875f,-0.2822000080227855534076297772116959095001220703125f,0.059099992179870497466254164464771747589111328125f,1.174699998039006487715596449561417102813720703125f,-0.8388000216603277436888674856163561344146728515625f,0.192100034439564471000494449981488287448883056640625f,2.0740000191926952766152680851519107818603515625f,-1.557000038206577841037869802676141262054443359375f,-0.311099960350990301094498136080801486968994140625f,-0.4047999777317041747437542653642594814300537109375f,-0.730199977862834970210315077565610408782958984375f,0.8052000204801554872346969204954802989959716796875f,-0.28260002841949471985572017729282379150390625f,-0.936700005102157007286223233677446842193603515625f,-0.1400000100955367532407080943812616169452667236328125f,-0.053500005133450045935461503177066333591938018798828125f,-0.85150003368407534143358361689024604856967926025390625f,0.35010000427961340818683311226777732372283935546875f,0.6834000150844452559084629683638922870159149169921875f,0.398799989771843055308409020653925836086273193359375f,0.54279999719262128632379926784778945147991180419921875f,-1.047800004345178503939450820325873792171478271484375f,0.5900999679580334333905966559541411697864532470703125f,-0.318099990287423306511982445954345166683197021484375f,0.3230999522522097766596971268882043659687042236328125f,-0.38449999558925629372652110760100185871124267578125f,-0.055099995525181189037766671390272676944732666015625f,0.077499997168779533041060858522541821002960205078125f,0.92470001621842357053537853062152862548828125f,0.77979998726844801382185323745943605899810791015625f,0.343599999764561570003706947318278253078460693359375f,0.38539997777938861389657176914624869823455810546875f,-0.07490002418160479624731351577793247997760772705078125f,-0.5378000046432018788067352943471632897853851318359375f,0.0942000412374739415355406890739686787128448486328125f,0.948900007760524744071517488919198513031005859375f,-0.50550002650171510509835570701397955417633056640625f,0.3277000136256216222818693495355546474456787109375f,0.665999980546534420255966324475593864917755126953125f,0.090899995154142487763238023035228252410888671875f,1.889299975407123355353178340010344982147216796875f,-0.4520000066161156127009235206060111522674560546875f,0.296400019010901960569981383741833269596099853515625f,-0.116300011825561266931572390603832900524139404296875f,-1.18969997553825379554837127216160297393798828125f,1.943100028359889819284944678656756877899169921875f,-0.13230003018379221657596644945442676544189453125f,-0.8586000357508662972350066411308944225311279296875f,1.90829996747970653814263641834259033203125f,0.290900027638674174568222952075302600860595703125f,-0.6034000075221062076025191345252096652984619140625f,0.26860000464469191427951955120079219341278076171875f,-0.539600006425380751551301727886311709880828857421875f,-1.3009000081926587188974053788115270435810089111328125f,0.176899968957901165822477196343243122100830078125f,0.8364999774694439338418305851519107818603515625f,-0.343600011610984523713341332040727138519287109375f,-0.10749999508261698366595737752504646778106689453125f,-1.1710999803185462297960839350707828998565673828125f,0.42800000061094767289660012465901672840118408203125f,-0.54760001336932173643390342476777732372283935546875f,0.3788000201702119706936855436651967465877532958984375f,-0.0198000126004220788900056504644453525543212890625f,0.075799989086389185644065946689806878566741943359375f,0.70759999220967273458882118575274944305419921875f,-0.512200018937885914738217252306640148162841796875f,-0.371900005242228548496541407075710594654083251953125f,0.17120001790225514159970998662174679338932037353515625f,-0.01919999273419392693540430627763271331787109375f,0.812000004678964604210023026098497211933135986328125f,-0.789899972772598601977733778767287731170654296875f,0.17859997644424385754291506600566208362579345703125f,-0.408500007256865416138680302537977695465087890625f,-0.461399992114305490531478426419198513031005859375f,-0.0289999912157654637923087648232467472553253173828125f,-0.59550000485777854208890857989899814128875732421875f,-1.117799980354309496988207683898508548736572265625f,1.1285999908983708195364670245908200740814208984375f,-0.432599974887073468465814585215412080287933349609375f,-0.2932000065624713602829842784558422863483428955078125f,0.19779999682009219430511848258902318775653839111328125f,-0.387600002454221215231200403650291264057159423828125f,0.4160000001788135559621650827466510236263275146484375f,0.41559997128844283764692590921185910701751708984375f,-0.13899999851733479960813610887271352112293243408203125f,-0.166000005245208637205678314785473048686981201171875f,-0.1087999944657089823607520884252153337001800537109375f,-0.51819998912811282565371584496460855007171630859375f,0.1967999811813234689594764859066344797611236572265625f,-0.35450000430643557880472371834912337362766265869140625f,0.0611000210359693429751359872170723974704742431640625f,-0.2100000077486037231011550829862244427204132080078125f,0.20090001307278881892415256515960209071636199951171875f,0.100199999916553483814141145558096468448638916015625f,-0.8958000093147158171547062011086381971836090087890625f,-0.067300007943809025956483083064085803925991058349609375f,-0.2381999981060622939565973865683190524578094482421875f,0.1292999965667724904250235340441577136516571044921875f,-0.7281999999687072122611652957857586443424224853515625f,-0.408099992150068313634392325184307992458343505859375f,-1.1837000149965286599496039343648590147495269775390625f,1.387200006830692444736996549181640148162841796875f,-1.1452000109434123942264704965054988861083984375f,0.65840002719163948086134041659533977508544921875f,-0.210700033831596300615274230949580669403076171875f,0.1981999990187583993161979378783144056797027587890625f,0.7529999649226668534396367249428294599056243896484375f,0.82000002350658174332664884786936454474925994873046875f,0.3521999960273500818175307358615100383758544921875f,-0.38390001692473862959786856663413345813751220703125f,-0.443599964523315293263294734060764312744140625f,-0.784700037527084504063168424181640148162841796875f,-0.7757999603271485966615728102624416351318359375f,-1.139700037974119339878598111681640148162841796875f,-0.2511999711871144569386160583235323429107666015625f,0.6564000443615027879928902621031738817691802978515625f,0.0129000044524671952927974416525103151798248291015625f,0.74040001387745146832486398125183768570423126220703125f,-1.34550002154707915025255715590901672840118408203125f,-0.9020000243484973534435766850947402417659759521484375f,-0.751599995160102452018691110424697399139404296875f,2.26970005779266426060303274425677955150604248046875f,-0.9532000296592713084464776329696178436279296875f,0.170000023394823784883556072600185871124267578125f,0.3454000078797339057246063021011650562286376953125f,-1.613800044086576068735894295969046652317047119140625f,1.83570001788735392977969240746460855007171630859375f,0.1367000085294247657685673402738757431507110595703125f,0.990000005960464335430515347979962825775146484375f,-0.8926999984264369913944392465054988861083984375f,-1.060399988172948670950290761538781225681304931640625f,-0.9540999859213830092130592674948275089263916015625f,-1.087999982468784043732057398301549255847930908203125f,-0.370100012177228876453227712772786617279052734375f,-1.0043999701499941590299158633570186793804168701171875f,0.34129998319670573625472798084956593811511993408203125f,-0.8096999855220319286530639146803878247737884521484375f,-0.011400000160932677317759953439235687255859375f,0.52390001822859100233387152911745943129062652587890625f,-0.4298999718785285750755065237171947956085205078125f,1.083099999265372925805195336579345166683197021484375f,-0.558699973720312215164085500873625278472900390625f,-0.467100054109096607390938515891321003437042236328125f,0.3197000150263307194364870156277902424335479736328125f,0.39569999780803921662908351208898238837718963623046875f,0.10350000407174231309426204461487941443920135498046875f,0.1871999877572056991681392901227809488773345947265625f,-0.95520000580251218647021005381247960031032562255859375f,-0.74450004126131563708668181789107620716094970703125f,-0.030699997174739923622155401972122490406036376953125f,-0.422000024311244725794267651508562266826629638671875f,-0.882500003874302052508937777020037174224853515625f,-0.066699960659443835453430438064970076084136962890625f,-0.1875999899744986709748673092690296471118927001953125f,0.2268000144109130200575918934191577136516571044921875f,1.16190000688135608442053126054815948009490966796875f,-0.43180001105815168926937985816039144992828369140625f,0.68299999979138359407215830287896096706390380859375f,-0.08670002078562955016849400635692290961742401123046875f,1.1458000674664976514804948237724602222442626953125f,0.8877999979376784978057912667281925678253173828125f,0.27120001052618025738638607435859739780426025390625f,0.302699998426437133502986398525536060333251953125f,1.15070003055334080244165306794457137584686279296875f,0.45840001050233780688358820043504238128662109375f,0.8559999870657915010951910517178475856781005859375f,-0.3515999778747556092639570124447345733642578125f,-0.2748000266075134589982553734444081783294677734375f,1.31829994930028959743140148930251598358154296875f,-0.9322999696850775563916613464243710041046142578125f,0.3337000251293176233957638032734394073486328125f,0.24749998718500165750810992904007434844970703125f,-1.242600010836124369006938650272786617279052734375f,0.6184000220879912745886031189002096652984619140625f,-0.672200013014674357947342286934144794940948486328125f,0.33759998641684629827608432606211863458156585693359375f,-1.1426999809920788120365386930643580853939056396484375f,-0.59490000738054504836327396333217620849609375f,-0.7932000016450881929586103069595992565155029296875f,-1.95719997963607283963938243687152862548828125f,0.07320002131462022276764400885440409183502197265625f,0.613899964994191638112397413351573050022125244140625f,-0.51910000469684547397264395840466022491455078125f,-1.052699980545042990343063138425350189208984375f,-1.311099974656105615622436744160950183868408203125f,-0.453699973571300319008514634333550930023193359375f,-0.97839998993873678045929409563541412353515625f,-0.8661000545263295435916006681509315967559814453125f,-1.2460000061020248818266509260865859687328338623046875f,0.45879998634457574979705896112136542797088623046875f,-1.1781000079452985662697983570978976786136627197265625f,0.309299987626075800761782375047914683818817138671875f,-1.50229998715668866537953363149426877498626708984375f,0.31799999010562896017972889239899814128875732421875f,0.4073000096201899733472373554832302033901214599609375f,-0.56879996876120575421964531415142118930816650390625f,-0.2643999745085838792846288924920372664928436279296875f,0.51569998018741625145366924698464572429656982421875f,-0.0248999916598202641893067266209982335567474365234375f,0.307100005382299201528439880348742008209228515625f,-0.014700046467780669701141960103996098041534423828125f,-0.640600007051229436427774999174289405345916748046875f,-0.3379999973326921480776263706502504646778106689453125f,-0.433400010912120270401004518134868703782558441162109375f,0.2343999866157771005958920795819722115993499755859375f,-0.84109997970387351262644415328395552933216094970703125f,0.6018999854236841695609427915769629180431365966796875f,-0.52339999724030494832049953402020037174224853515625f,0.607399975156784233121243232744745910167694091796875f,0.574799983170628525641632222686894237995147705078125f,1.97999994367361065172872258699499070644378662109375f,-0.50759998148083695923560298979282379150390625f,0.4747999822393060842529166620806790888309478759765625f,-0.61429999611973773454565161955542862415313720703125f,1.62099996873736440505808786838315427303314208984375f,-0.62440000710487364443679325631819665431976318359375f,0.0395999666392807814219167994451709091663360595703125f,0.0790999784708024122892311424948275089263916015625f,0.349400020217895956875509000383317470550537109375f,0.19680004782676707719701880705542862415313720703125f,-1.4498999876737599379339371807873249053955078125f,-0.0059000295758249166055975365452468395233154296875f,1.081799981069565319558023475110530853271484375f,-0.97219998924732209388821502216160297393798828125f,-0.86219998984336854164212127216160297393798828125f,1.1419000285625457991045550443232059478759765625f,0.06120001648664441518121748231351375579833984375f,-1.004800020945072702716061030514538288116455078125f,-0.473599983596802331931030494160950183868408203125f,0.37349996471404978137798025272786617279052734375f,0.1746999671936038112107780762016773223876953125f,-1.1875999801397316701923045911826193332672119140625f,-0.13169997053146431653658510185778141021728515625f,-1.6357000657200817528291736380197107791900634765625f,-0.3487000108242028773020138032734394073486328125f,1.040500057905912711930795921944081783294677734375f,1.2089000119328492388603990548290312290191650390625f,0.78579998729825017989014668273739516735076904296875f,-1.1150000002980231528226795489899814128875732421875f,0.082000019468366769359590762178413569927215576171875f,0.97529997882694008382031825021840631961822509765625f,0.066799985800683803205402000457979738712310791015625f,0.0458999670654538505232267198152840137481689453125f,-0.144400002112984537916418048553168773651123046875f,1.51589999172687583950391854159533977508544921875f,0.461199999350309752799148554913699626922607421875f,-1.089399970969557873701205608085729181766510009765625f,0.365000013709068316103412144002504646778106689453125f,0.115000002682208890547599366982467472553253173828125f,-0.71410000156760222722596154198981821537017822265625f,0.1515999827176333081268921887385658919811248779296875f,0.65479999382495890358768519945442676544189453125f,-0.693099990510940955346086411736905574798583984375f,0.1293999863922596915699614328332245349884033203125f,1.5830000166669491168391914470703341066837310791015625f,-0.2088999927103518672311111004091799259185791015625f,0.75509999384880099881911519332788884639739990234375f,-0.29599999988079073176550082280300557613372802734375f,0.89649997217953203421103580694762058556079864501953125f,-0.30740000019073487891319018672220408916473388671875f,0.6657000072330239159867915077484212815761566162109375f,-0.17979998950362219289900167495943605899810791015625f,0.244899960590899024026612096349708735942840576171875f,0.28249998837709444643451206502504646778106689453125f,0.4010000095292924893186636836617253720760345458984375f,0.6048999988496301938312171841971576213836669921875f,-1.1631000076845283697224431307404302060604095458984375f,-0.6840999988853930791066204619710333645343780517578125f,0.3338000040024515779890634803450666368007659912109375f,0.36049999621510497860299437888897955417633056640625f,-0.4911000089660289003035131827346049249172210693359375f,-0.072400010621547661315844379714690148830413818359375f,-0.324499975547194896563496513408608734607696533203125f,0.8482000050723552675435712444595992565155029296875f,-0.2979999831393362885734177325502969324588775634765625f,-0.9069000310957431310043830308131873607635498046875f,-0.6867999821126460346931708045303821563720703125f,1.42560000300928901850738839129917323589324951171875f,0.420099981052428628203188054612837731838226318359375f,-0.6171999873287974391899979309528134763240814208984375f,-0.697899944847822695237482548691332340240478515625f,0.53990000142008032213425394729711115360260009765625f,-0.058400000667572005141892077517695724964141845703125f,0.30410002719759943179411720848293043673038482666015625f,0.111299964737892054245094186626374721527099609375f,1.2530000293329361937821886385791003704071044921875f,-0.708300013151764940033672246499918401241302490234375f,0.12900000645220266459745062093134038150310516357421875f,0.428399986809491650774361914955079555511474609375f,0.9043000554263596058035545866005122661590576171875f,0.00689994153976414281714824028313159942626953125f,-1.4462000459909436500538504333235323429107666015625f,-1.021099980020522934864857234060764312744140625f,-0.55599999123811727486099698580801486968994140625f,0.287600005173683115344829275272786617279052734375f,-0.3478999794185160254755828646011650562286376953125f,0.142000036269426033186391578055918216705322265625f,0.093499981999396908349808654747903347015380859375f,-0.1515999948620798676302001695148646831512451171875f,0.524299985837936244337242897017858922481536865234375f,-0.043300014269352260498635587282478809356689453125f,-0.264899993485212092991787358187139034271240234375f,0.74660000611245624924094954621978104114532470703125f,-1.3551000498026615215252377311117015779018402099609375f,0.32409998700171727126218002013047225773334503173828125f,-0.065200000810623048863590156543068587779998779296875f,-0.2394999867789449066179940928122960031032562255859375f,0.0548999943792820577215252342284657061100006103515625f,-0.94359998626038399383020305322133935987949371337890625f,-0.0679999853372574847298892564140260219573974609375f,0.2263999721840026513319799050805158913135528564453125f,0.027199998821318127628643424031906761229038238525390625f,-0.8088000221818687496266875314177013933658599853515625f,0.209999974370002728818462855997495353221893310546875f,0.125200009416043922527705944958142936229705810546875f,-0.7499999998882411755829480171087197959423065185546875f,-1.121000023834407333112039850675500929355621337890625f,0.134299991351365832770170527510344982147216796875f,0.0525999852806331436028131065540947020053863525390625f,-0.46840002361536026143085109652020037174224853515625f,-0.30760001880824550113402438000775873661041259765625f,-0.3623999915480613509544127737171947956085205078125f,0.094799987491965165276042171171866357326507568359375f,0.5636999895155430095883275498636066913604736328125f,1.285100006589293908376703257090412080287933349609375f,0.26530000866651537450024989084340631961822509765625f,1.62589999139159946395238875993527472019195556640625f,-0.3602999937653541540072410498396493494510650634765625f,-0.5258000165790317481651072739623486995697021484375f,-0.3570000021457673256009002216160297393798828125f,-0.11229997147321657546825690587866120040416717529296875f,0.20560000608265405208641141143743880093097686767578125f,0.3915000104010102344176402766606770455837249755859375f,-1.36800002959370647914738583494909107685089111328125f,0.5036000092267993721861785161308944225311279296875f,0.15360001101493825359511902206577360630035400390625f,0.519800007236003835942028672434389591217041015625f,0.25340002726614496708634760580025613307952880859375f,1.85879999454021405114190201857127249240875244140625f,1.106599988603591544489290754427202045917510986328125f,-0.330300065737962000866900780238211154937744140625f,0.54040004274845188092513126321136951446533203125f,0.75820000358223893499598489142954349517822265625f,0.07089996564984257787500609993003308773040771484375f,1.77919994162321160047213197685778141021728515625f,-1.3604999793767920124309966922737658023834228515625f,-0.92089994076490455654493416659533977508544921875f,1.81859999037682928957337935571558773517608642578125f,0.55239995995759993974161261576227843761444091796875f,-0.04020001723915367364270423422567546367645263671875f,0.639599967458844620438185302191413938999176025390625f,-1.0001000208199026797473152328166179358959197998046875f,-0.265700000452994800070882774889469146728515625f,0.435300064694882138383036362938582897186279296875f,-1.07339995924234443691602791659533977508544921875f,1.5218000237613917846601907513104379177093505859375f,-1.48310001509785660545048813219182193279266357421875f,-0.8938000267267229848044962636777199804782867431640625f,-1.24689999950528118688453105278313159942626953125f,0.876500010140240259914889975334517657756805419921875f,-0.505699999558925750164917189977131783962249755859375f,-0.3300000063329935340306064972537569701671600341796875f,0.6442999898985026785425134221441112458705902099609375f,0.7484999957084657040695674368180334568023681640625f,1.1946000029981134193945990773499943315982818603515625f,0.35220000221133229700853917165659368038177490234375f,-0.30910000197738385363521729232161305844783782958984375f,1.017399988642334829336277834954671561717987060546875f,-1.37020001258254087161958523211069405078887939453125f,-0.30170002253651606594075929024256765842437744140625f,-0.24739994922876373806275296374224126338958740234375f,0.149900004661083219303918667719699442386627197265625f,-0.390399992717803012709509857813827693462371826171875f,0.73649998581409459319502275320701301097869873046875f,-0.7723999660298226377364017025684006512165069580078125f,-0.0431999904692172975728681194595992565155029296875f,0.558899994535744237822427749051712453365325927734375f,-0.081300009664893035932209386373870074748992919921875f,-1.03369995911717449388333989190869033336639404296875f,0.2333999893799425517926238171639852225780487060546875f,-1.252500002905727161106597122852690517902374267578125f,0.4389999929666519307147609652020037174224853515625f,-0.0010000251382589198101413785479962825775146484375f,-0.1597999945700170343343415879644453525543212890625f,0.4362999662652615739943939843215048313140869140625f,-1.040900004467367967464497269247658550739288330078125f,-0.073900012677908133440496385446749627590179443359375f,-0.47580000175237646686809966922737658023834228515625f,0.62259999228417861605322514151339419186115264892578125f,-0.5320999928280711888106679907650686800479888916015625f,-0.70440000803619629010654534795321524143218994140625f,0.861800038886070041144193965010344982147216796875f,-0.35680000967979452752842917107045650482177734375f,-0.45310005935430552881371113471686840057373046875f,-1.47239998946189842854437301866710186004638671875f,0.045899998693168075902804048382677137851715087890625f,-0.762699988293647823667242846568115055561065673828125f,0.4977000253230337012411155228619463741779327392578125f,0.82590000275373398608280695043504238128662109375f,0.189199994969367679686911287717521190643310546875f,0.465700014162063524736367980949580669403076171875f,-0.94020004789829325630989842466078698635101318359375f,0.372199998784065400059262174181640148162841796875f,0.6882000234007836780847355839796364307403564453125f,-1.05170000271797192681333399377763271331787109375f,-0.491600038261712057874319725669920444488525390625f,1.13199996686726844785653156577609479427337646484375f,-1.301800001521408400861901100142858922481536865234375f,-0.21619999473094964059782796539366245269775390625f,0.27439999853670560270302303251810371875762939453125f,0.06130000959038728325367628713138401508331298828125f,-0.0813999931946398636029016415704973042011260986328125f,0.773200010958313921349827069207094609737396240234375f,-0.72210001052320016423635706814820878207683563232421875f,-0.45759998848438243612690712325274944305419921875f,1.614200072120131057573644284275360405445098876953125f,1.111400018489360430606893714866600930690765380859375f,-0.5714000049665568514711821990204043686389923095703125f,-0.92209998500496137552318032248876988887786865234375f,-0.345899992453306825979097993695177137851715087890625f,-0.016399954675138328497041584341786801815032958984375f,-0.5049000009357929030784362112171947956085205078125f,-0.0515000051856040474973497111932374536991119384765625f,-0.8838999860048293299996657879091799259185791015625f,0.3483000023856757110252146958373486995697021484375f,-0.545799988378584455261943730874918401241302490234375f,-0.7675999947428697733897706712014041841030120849609375f,0.340099992358684488635844900272786617279052734375f,0.129700035403669478029087258619256317615509033203125f,0.009000010401010616334360747714526951313018798828125f,-0.41510000252872725212682780693285167217254638671875f,0.41469999565482140724270720966160297393798828125f,0.82820001423656941508255613371147774159908294677734375f,-0.842899997150897917208567378111183643341064453125f,0.6588000045984980790336749123525805771350860595703125f,0.388899992710352027103226646431721746921539306640625f,-0.4579999842569233425848551632952876389026641845703125f,0.250399985080957510064081361633725464344024658203125f,0.17209998932629833046092926451819948852062225341796875f,0.24689999674856666356248524607508443295955657958984375f,0.27640000273138287667507029254920780658721923828125f,0.5763000015810131326787768557551316916942596435546875f,-0.569600045540929134091356900171376764774322509765625f,0.31089999887943253753519456950016319751739501953125f,0.1014999906197193713097703948733396828174591064453125f,0.4982000105485318020015483853057958185672760009765625f,-0.67220001115202876462717540562152862548828125f,-0.231899997270107238733771737315692007541656494140625f,0.63839998466745029315205783859710209071636199951171875f,-1.273599999988078934620716609060764312744140625f,0.44830001218318926703432225622236728668212890625f,0.47849998795986170563310224679298698902130126953125f,-0.520599991703033282419710303656756877899169921875f,0.71680002696514133475602648104541003704071044921875f,0.54079995436668326647122739814221858978271484375f,-1.6745000356175001599012830411083996295928955078125f,0.8960000205561520214558868246967904269695281982421875f,0.1414000347688795233835890030604787170886993408203125f,1.533699956583976842239280813373625278472900390625f,0.92390000984668763095442045596428215503692626953125f,-0.30139998034238857371747144497931003570556640625f,0.2811999875783914859539436292834579944610595703125f,-1.333799994540214584048953838646411895751953125f,1.169300002340972088887838253867812454700469970703125f,0.051599938833713832053717851522378623485565185546875f,-0.9894000303506855775736994473845697939395904541015625f,-0.2650999970525500515350358909927308559417724609375f,0.66199999186396585315605989308096468448638916015625f,0.5487999794155360433478563209064304828643798828125f,-1.21480000820457956223208384471945464611053466796875f,-0.21329997552633273016908788122236728668212890625f,-0.22509998256117125237096843193285167217254638671875f,0.833699981394409928014965771581046283245086669921875f,0.17129998647496080810270768779446370899677276611328125f,-0.233500001948326652012610793462954461574554443359375f,0.4922000224582852023758050563628785312175750732421875f,-1.065600007796287940209367661736905574798583984375f,-1.4549000515997416727032032213173806667327880859375f,0.785700005929171663154875204782001674175262451171875f,0.536499996393918987536153508699499070644378662109375f,1.48800000120699404959623279864899814128875732421875f,-0.576199983704090090697036430356092751026153564453125f,0.42189998795688143928828139905817806720733642578125f,-0.3702000155627729327534325420856475830078125f,-1.197200016535072908396841739886440336704254150390625f,0.76930000805929299456664693934726528823375701904296875f,0.3323000101231035063875651758280582726001739501953125f,0.918299972099065886510516065754927694797515869140625f,-0.6628999793440100152253080523223616182804107666015625f,-0.31379999409317971270638736314140260219573974609375f,-0.8134000047281382261843418746138922870159149169921875f,-0.7405999978125095140057965181767940521240234375f,-0.2027999797955157834650208314997144043445587158203125f,-0.3447000251963732164739440122502855956554412841796875f,-0.190899969747662989760783602832816541194915771484375f,-0.0716999969065188480499273282475769519805908203125f,-0.21959998534023783545165997566073201596736907958984375f,0.169299991202354505048788269050419330596923828125f,-0.22659997992366565089383811937295831739902496337890625f,0.27410000395178801824158654198981821537017822265625f,0.917699991609155940608388846158050000667572021484375f,0.6772999768748879656499184420681558549404144287109375f,0.267399991790205182606854350524372421205043792724609375f,-0.0683000003367661978614933104836381971836090087890625f,0.5617000037796795464117849405738525092601776123046875f,-0.8363000052690505725649927626363933086395263671875f,0.65219997237175653115315299146459437906742095947265625f,0.311900014853478069198899902403354644775390625f,-1.604199986624717411132223787717521190643310546875f,-0.56389998391866669180672033689916133880615234375f,0.838799982321262405093875713646411895751953125f,0.551100011201203443533813697285950183868408203125f,1.03609998624175769776911693043075501918792724609375f,0.500199965979158722717556884163059294223785400390625f,0.89289998970031803082747501321136951446533203125f,0.3112999739766113549421788775362074375152587890625f,-0.97720008938312474811027641408145427703857421875f,0.129200006890297203199224895797669887542724609375f,0.091400028324127191581283113919198513031005859375f,-1.10410002451539046575135216698981821537017822265625f,-1.24489998603463192239360068924725055694580078125f,-1.30079998998045898161990407970733940601348876953125f,0.86260001173019418274634517729282379150390625f,1.22619994198083759329165332019329071044921875f,-0.088100028359889392959303222596645355224609375f,0.7442000146389009529457325697876513004302978515625f,0.413700007843971206966671161353588104248046875f,1.4607999877452844117442509741522371768951416015625f,-0.40450000214576675716671161353588104248046875f,-0.07909996237754768344530020840466022491455078125f,1.66920000301599458225609851069748401641845703125f,0.562600011432170532543750596232712268829345703125f,0.73320000939369123216238222084939479827880859375f,0.1800999834179872749473361182026565074920654296875f,-0.206700015532970127196676912717521190643310546875f,-0.72340001884698867939960109652020037174224853515625f,0.01039999212175590770357302972115576267242431640625f,-0.86419998513460161149168925476260483264923095703125f,1.119499986425041893056686603813432157039642333984375f,0.43269999440312378879980315105058252811431884765625f,-0.028299996946751893034388558589853346347808837890625f,0.291999996781349135943628425593487918376922607421875f,-0.2371999458849429753826143496553413569927215576171875f,0.04420001448988930548011921928264200687408447265625f,-1.337699970598519971787254689843393862247467041015625f,-0.7516000397145745637317304499447345733642578125f,0.2935000423118470092020970696466974914073944091796875f,-0.258799973380565706548850357648916542530059814453125f,-0.3535000314712524183136110877967439591884613037109375f,-0.445599999153614145797064338694326579570770263671875f,1.3149000209033492314603108752635307610034942626953125f,0.88689998832941085993297747336328029632568359375f,0.44960000400394140029902700916863977909088134765625f,-0.26110000162720670147109558456577360630035400390625f,-1.864299944934248998151815612800419330596923828125f,0.20980000008344656947656403644941747188568115234375f,-1.0200999776065351465348385318065993487834930419921875f,-0.542399981862306646007709787227213382720947265625f,-0.57379994383901333776520914398133754730224609375f,0.763799994242191360172000713646411895751953125f,-1.244699989396333084101797794573940336704254150390625f,0.29899999341368665994878028868697583675384521484375f,-0.018700005199014935186596630956046283245086669921875f,-0.293599998050928068238363266573287546634674072265625f,0.0203000125035639467085957221570424735546112060546875f,-0.6615000190436841620567065547220408916473388671875f,-1.272499986141919947613132535479962825775146484375f,1.2207000184834004841150090214796364307403564453125f,-0.5800000131130218505859375f,0.8406999795913687734127961448393762111663818359375f,1.1840999523937707493814741610549390316009521484375f,-0.22199999603629105848767721909098327159881591796875f,-1.388800001692771957095828838646411895751953125f,-0.09319996081590620207180108991451561450958251953125f,1.3673000081300727970301522873342037200927734375f,0.3521999570608134177973624900914728641510009765625f,-2.5209999957084647803640109486877918243408203125f,0.69110001816749644376614014618098735809326171875f,-0.7890000078678127692910493351519107818603515625f,0.92830005301237150661108898930251598358154296875f,1.52980000187158537983123096637427806854248046875f,-0.7034000310659411780989103135652840137481689453125f,-0.46760000129938106283589149825274944305419921875f,-0.24700002479553262446643202565610408782958984375f,0.3327999974906443725330973393283784389495849609375f,0.439999998062849062563373081502504646778106689453125f,0.618300014269351549955899827182292938232421875f,-0.816900066113472433926290250383317470550537109375f,-1.803299964499473162504727952182292938232421875f,-0.0330999630928037191779367276467382907867431640625f,1.77219997136592866127102752216160297393798828125f,0.272500004805624751469395050662569701671600341796875f,-0.072799999949336058335802590590901672840118408203125f,0.070500017970800410438414473901502788066864013671875f,0.20999994758516582482599233117070980370044708251953125f,-0.296700000184774381750685279257595539093017578125f,0.2658000097990036447725969992461614310741424560546875f,-0.423400009906291785455323406495153903961181640625f,-0.60449996966123631381151426467113196849822998046875f,0.372799987059831750002558692358434200286865234375f,-0.7083999814450745446947621530853211879730224609375f,-0.768300000709295272116605701739899814128875732421875f,-0.47540001115798913300380945656797848641872406005859375f,0.91569998644590366865259056794457137584686279296875f,-0.3357000516012311752689356580958701670169830322265625f,-1.261299981725215957339969463646411895751953125f,-0.210600004033744792053539640619419515132904052734375f,-0.443899997627735132255111238919198513031005859375f,-0.361999964632093718108762914198450744152069091796875f,0.55730000038146965124497000942938029766082763671875f,0.2019000211492183449735193789820186793804168701171875f,-0.3437999982655046693480471731163561344146728515625f,0.782900013951957074453957829973660409450531005859375f,-0.0039999987781047519064259176957421004772186279296875f,0.12140002452433151791666432472993619740009307861328125f,-0.0652999879539013505791444913484156131744384765625f,-0.6749000179976223368072396624484099447727203369140625f,-0.87589997205734260177223404753021895885467529296875f,0.4095000035241243008243827716796658933162689208984375f,1.006399982726573938407454988919198513031005859375f,-0.2301999965637921974348500953055918216705322265625f,-0.10009999776035549690078596540843136608600616455078125f,-1.048000020690262434897022103541530668735504150390625f,0.900800004471838722253096420899964869022369384765625f,-0.80989999795556055772749459720216691493988037109375f,-1.2064999856278297851730485490406863391399383544921875f,0.390099984610080330327264164225198328495025634765625f,-0.4652000405222185275277979599195532500743865966796875f,-0.5023000283956520206629647873342037200927734375f,-0.554099989795684422233534860424697399139404296875f,-1.201800012063980460652601323090493679046630859375f,1.028499962329863848253808100707828998565673828125f,0.585499959111213996720834984444081783294677734375f,-2.253900033390522139598033390939235687255859375f,-0.613399996495246568883885629475116729736328125f,0.279900013341009668010883615352213382720947265625f,-0.35249998066574317334698207559995353221893310546875f,0.00179999753534776374408465926535427570343017578125f,-0.615199991869926687826364286593161523342132568359375f,0.8120000360459089794318288113572634756565093994140625f,0.4474999612569803275619051419198513031005859375f,-1.8123000099182124955632389173842966556549072265625f,-0.35709997930526782283777720294892787933349609375f,1.07690000516772244054664042778313159942626953125f,-1.3360000240206719723801143118180334568023681640625f,-0.499300054681301563874740168103016912937164306640625f,-2.2886000086307518586181686259806156158447265625f,0.9070000492334369113223146996460855007171630859375f,-0.3025000163912778816666104830801486968994140625f,0.1267000369906430279343112488277256488800048828125f,-1.93570000879764592127685318700969219207763671875f,-0.1403999984920023536005828646011650562286376953125f,0.9365999913975595614346048023435287177562713623046875f,-0.78249998442828661016079649925814010202884674072265625f,-0.25619999473094934305805736585170961916446685791015625f,0.4328999742403627504216956367599777877330780029296875f,1.36919998260140385326621981221251189708709716796875f,-0.237800004345178450648745638318359851837158203125f,1.38430000919550622029419173486530780792236328125f,-0.73200001451373086780449739308096468448638916015625f,0.52270000025182949965341094866744242608547210693359375f,-0.26200000259280198378064596909098327159881591796875f,-1.34279999283403217447130373329855501651763916015625f,0.3115999877095223524747780174948275089263916015625f,1.43490000454932431495080891181714832782745361328125f,-0.018399992322921665532931001507677137851715087890625f,0.844199989865720201720478144125081598758697509765625f,-0.3662999977812171348290348760201595723628997802734375f,0.64699997964501398950432076162542216479778289794921875f,-0.25610001548528682491223662509582936763763427734375f,-0.5346000260576599938389108501723967492580413818359375f,0.1498999930381774703391783987171947956085205078125f,0.6382999701246621082617593856411986052989959716796875f,-0.9444000114262103284801241898094303905963897705078125f,-0.04870001983940641654413639116683043539524078369140625f,-1.17629999498724924222869958612136542797088623046875f,-0.4450000233948234740211091775563545525074005126953125f,-0.25690001276731511126172335934825241565704345703125f,-0.737800046031176659511174875660799443721771240234375f,-0.96290001205205921763763399212621152400970458984375f,0.96369997673779739244537267950363457202911376953125f,-1.0236000205516813110762086580507457256317138671875f,-0.13689998147487614232886699028313159942626953125f,0.5098000188589093539803798194043338298797607421875f,-0.32829999251365649115541600622236728668212890625f,1.23140003860592894824321774649433791637420654296875f,0.16439998750984674558139886357821524143218994140625f,1.161600002089142602557103600702248513698577880859375f,0.0395999620199205537574016489088535308837890625f,-0.67549999692290985109366374672390520572662353515625f,-0.152799978640675515606517365085892379283905029296875f,0.89020003709495065802315139080747030675411224365234375f,-1.263599997901916793807686190120875835418701171875f,0.36359998940825466462456461158581078052520751953125f,1.396899986392259762624235008843243122100830078125f,-0.62630003194212946482366533018648624420166015625f,-1.260000009536742737736858543939888477325439453125f,-0.357700027316063984272886955295689404010772705078125f,-0.4313999878302217094727666335529647767543792724609375f,0.6628000014767050362962663712096400558948516845703125f,-1.011299977850914189048125990666449069976806640625f,-0.19970000377595464868818453396670520305633544921875f,-0.13069997497200969149844240746460855007171630859375f,0.6843000172793860524933506894740276038646697998046875f,-0.76820002012252786016688332892954349517822265625f,-0.962900030156969677364031667821109294891357421875f,0.33080003170371075071898303576745092868804931640625f,0.5640000446736814776471646837308071553707122802734375f,0.507699966835975402545955148525536060333251953125f,-0.4964000175766651867803602726780809462070465087890625f,-0.64840001217871889860333567412453703582286834716796875f,0.41419999884366998088580658077262341976165771484375f,-0.21510002260804217399936533183790743350982666015625f,0.16500000014901150535706619848497211933135986328125f,-0.019099995346367304449586299597285687923431396484375f,-0.0391000037282704937524613342247903347015380859375f,-0.94649999393522765700481613748706877231597900390625f,-0.3140000078678131245624172152020037174224853515625f,-1.6504999866411089737283646172727458178997039794921875f,0.280900001673400401269731219144887290894985198974609375f,0.389299985875189324957545977667905390262603759765625f,-0.755000000298023277167658307007513940334320068359375f,0.1831000065296890522148487434606067836284637451171875f,0.52649999982118611541181962820701301097869873046875f,0.18980003536194534063952232827432453632354736328125f,-0.746000000327825318890972994267940521240234375f,0.1572000061415133043141167945577763020992279052734375f,-0.1508999977059662100931092254540999419987201690673828125f,-0.766499993879347751857267212471924722194671630859375f,-0.498699994283914538328872367856092751026153564453125f,-1.0492999985784290384316363997641019523143768310546875f,0.36309999907910828209622877693618647754192352294921875f,1.09409998886436188314519313280470669269561767578125f,-0.52720000295639035670092198415659368038177490234375f,0.209099986442923491125611690222285687923431396484375f,0.41090001743137827272533968425705097615718841552734375f,-0.56049998779594900621958686315338127315044403076171875f,0.410099967771768714186464421800337731838226318359375f,-0.04760000301301492431349515754845924675464630126953125f,-0.2508000311076645783003868928062729537487030029296875f,-1.783999937534332502764300443232059478759765625f,-0.5184000321090227458853405551053583621978759765625f,-1.2252000073671336366487594204954802989959716796875f,-0.53650002560019505182253851671703159809112548828125f,-1.8224000150918957530166153446771204471588134765625f,0.0748999954037368187442780254059471189975738525390625f,0.7765000127851962030689492166857235133647918701171875f,-0.704900014756619786027158625074662268161773681640625f,-0.53420000510215714939477038569748401641845703125f,-0.4263000249385839879323611967265605926513671875f,-0.473899998521805088103064917959272861480712890625f,-0.2033000171191992055952368900761939585208892822265625f,0.2531999972864984460585446868208236992359161376953125f,0.58630001178830870589564483452704735100269317626953125f,0.6262999924167989007628420949913561344146728515625f,0.558700000877678348842891864478588104248046875f,0.350299982440471779909785254858434200286865234375f,0.55369997759461409003733933786861598491668701171875f,0.1670999912261965647530814749188721179962158203125f,0.35120001417696489198760900762863457202911376953125f,-0.69930000625252741741633144556544721126556396484375f,-0.6502000417888169625513228311319835484027862548828125f,0.947599993848800892237704829312860965728759765625f,0.63230004366934355886087359976954758167266845703125f,0.3653000062078231024997876374982297420501708984375f,-0.2515999962218102847799627852509729564189910888671875f,1.1715000393092631636449141296907328069210052490234375f,0.27470005410462661199488820784608833491802215576171875f,-0.62840001478642204535418613886577077209949493408203125f,-0.002600001671910268097320795277482829988002777099609375f,0.471500016696750900990764421294443309307098388671875f,-0.5811999959602951459913811049773357808589935302734375f,-1.752300005000829941081974538974463939666748046875f,-0.52589999791085728642059393678209744393825531005859375f,0.59759996687769945111767810885794460773468017578125f,-0.197299998518824626358991736196912825107574462890625f,1.296199974763393214516327134333550930023193359375f,0.15619999081939450746858710772357881069183349609375f,0.840800027717649722802661926834844052791595458984375f,0.59280001507401436366961888779769651591777801513671875f,0.73890003242194735033621100228629074990749359130859375f,0.8919999948814509860284260867047123610973358154296875f,0.1687000489711767414746645954437553882598876953125f,0.2582999898314477515981479882611893117427825927734375f,0.171599983090161334331469333847053349018096923828125f,0.5745000151842829527737421813071705400943756103515625f,-0.0845999404132360410812907502986490726470947265625f,0.706700002717971731414081659750081598758697509765625f,-0.17839999694228136917217852897010743618011474609375f,0.389199998284876169662993561360053718090057373046875f,1.1209000204861165883585272240452468395233154296875f,0.3608000090911982482566600083373486995697021484375f,0.29289999491572427103847076068632304668426513671875f,0.286299993050098233737799091613851487636566162109375f,1.342700056540966357943034381605684757232666015625f,-0.913300013263523169371183030307292938232421875f,-0.2936000622004282245569584119948558509349822998046875f,-0.790599990622699255737870771554298698902130126953125f,0.334800012004375258101163126411847770214080810546875f,0.66600000382959845257602182755363173782825469970703125f,0.0685999897807836322982666388270445168018341064453125f,0.658600030572712658027967336238361895084381103515625f,0.56020000654757018399010348730371333658695220947265625f,-0.17969998209476489847702396218664944171905517578125f,0.071200006651878311458858661353588104248046875f,-1.219600007244944439577238881611265242099761962890625f,-0.287799991679191347060395855805836617946624755859375f,0.9678000044196843898447468745871447026729583740234375f,0.5020000186860562596535828561172820627689361572265625f,-0.415599978850781592853991242009215056896209716796875f,-0.05919997947216071310094775981269776821136474609375f,-0.076900030089914916953119927711668424308300018310546875f,-0.5112000083655117155245761750848032534122467041015625f,0.5940000117793677869570956318057142198085784912109375f,0.22979999557584529501497172532253898680210113525390625f,-0.8038000029593705964003902408876456320285797119140625f,-0.183399993216991408218063952517695724964141845703125f,0.5068000102758407621195146930404007434844970703125f,-0.286800023984909557839273475110530853271484375f,0.289299987104535194504251194302923977375030517578125f,0.529199988561868561731671434245072305202484130859375f,-0.3920000011399389361343992277397774159908294677734375f,0.44880002672672247854279703460633754730224609375f,1.4241000256329774398267318247235380113124847412109375f,0.7324000186681747237571471487171947956085205078125f,0.2479999801218510935285621599177829921245574951171875f,-0.0634999092817298826929572896915487945079803466796875f,0.197499987967312495840133124147541821002960205078125f,0.42319998409897052749073509403388015925884246826171875f,0.09630000165551899637961241751327179372310638427734375f,-0.188200000676512768649217832717113196849822998046875f,0.62610000877976401678637330405763350427150726318359375f,0.34109998739659885824693219547043554484844207763671875f,0.406099993021786342950463222223334014415740966796875f,0.0445999931633469515190881793387234210968017578125f,-0.511199977333844035598531263531185686588287353515625f,-0.60899999795854020678831375334993936121463775634765625f,0.77139999211430543279988114591105841100215911865234375f,0.366600038447975951072521638707257807254791259765625f,0.14640000098049643639797068317420780658721923828125f,1.03259997444003825961544862366281449794769287109375f,-0.061900002858042757480916407075710594654083251953125f,0.474200010745972455605823370206053368747234344482421875f,0.01380000256821511950278136282577179372310638427734375f,-0.52669999733492733895445780945010483264923095703125f,-0.118399998991191413022505685148644261062145233154296875f,0.913599990451336108066016095108352601528167724609375f,-0.517500010766088536229290184564888477325439453125f,-1.0437000347405682720847153177601285278797149658203125f,0.12140000239610670984546914041857235133647918701171875f,-1.396399978032707878128348966129124164581298828125f,0.5638999791130425709440032733255065977573394775390625f,0.21790000147223498316861878265626728534698486328125f,-0.22229998358041047623601116356439888477325439453125f,-1.95670000301599511516315033077262341976165771484375f,-0.05330001296550046507860542988055385649204254150390625f,-0.14700000095367415919866971307783387601375579833984375f,-0.89080003583878319961542047167313285171985626220703125f,-0.481200006949901393227264634333550930023193359375f,0.9653000253930688057124598344671539962291717529296875f,0.166600001120567497991942218504846096038818359375f,-0.0432999852493403114550574173335917294025421142578125f,0.30320003025531772067324709496460855007171630859375f,0.7100999802142384798031571335741318762302398681640625f,0.4481999958336351230769878384307958185672760009765625f,-0.359400008073449128698229060319135896861553192138671875f,-0.02919998602867135417682220577262341976165771484375f,0.002900011567771476617139114750898443162441253662109375f,-0.17159997876882560063904747948981821537017822265625f,0.8884999936595558978069675504229962825775146484375f,0.3241999979496004158363575697876513004302978515625f,-0.9591999909088013698266195206088013947010040283203125f,-0.31379998500347117129649632261134684085845947265625f,0.082599970267712929938852539635263383388519287109375f,-0.131599997246265143502341743442229926586151123046875f,0.8507000508189204168729702359996736049652099609375f,-0.101000018656254297866325941868126392364501953125f,-1.1970000599622727666115906686172820627689361572265625f,0.00999998778104771446351151098497211933135986328125f,0.0047000105932355484839035852928645908832550048828125f,0.05460001775026324111195208388380706310272216796875f,1.7684999806582926140663403202779591083526611328125f,0.7747999908447269490352482534945011138916015625f,-0.43269995364844771046364257927052676677703857421875f,-0.70950003828108332282198489338043145835399627685546875f,-0.04090000130087141638757231021372717805206775665283203125f,0.02690001265555663767514715800643898546695709228515625f,-0.7910000135153529843279329725191928446292877197265625f,-0.347500022351742376969241377082653343677520751953125f,-0.38329998178482072024308990876306779682636260986328125f,-1.2755000000894067824219746398739516735076904296875f,0.76890000139027814274328420651727356016635894775390625f,0.54080006783902623279658428145921789109706878662109375f,0.1687000009894370489149650893523357808589935302734375f,-0.495499972298741209186800915631465613842010498046875f,0.0323999963164329773945837587234564125537872314453125f,0.21040000117421175929166565765626728534698486328125f,-0.36489998994618655370203441634657792747020721435546875f,-0.31100001001358013041908634477294981479644775390625f,-0.6964999704658987322858365587308071553707122802734375f,-1.573600014591216567083620248013176023960113525390625f,-0.6252999702960255756778451541322283446788787841796875f,-0.40510001396536754469934749067761003971099853515625f,0.344599987649917560705858932124101556837558746337890625f,1.530700020867585831041424171417020261287689208984375f,-0.39629998820722089902801599237136542797088623046875f,-1.448900004333257829358672097441740334033966064453125f,-0.641200007098913005165741196833550930023193359375f,-1.207100004637241763560950857936404645442962646484375f,-0.487799984042346590484839907730929553508758544921875f,-0.6604000129461287116328094271011650562286376953125f,0.731999999240040555292807766818441450595855712890625f,0.62910004448294642998007475398480892181396484375f,-0.9223999909520148587915855387109331786632537841796875f,-0.2449999803304674372839144780300557613372802734375f,0.884600006164610430658967743511311709880828857421875f,-0.1115999960541724789209183654747903347015380859375f,-0.392200008805096533848200124339200556278228759765625f,-0.43220002024173709287424571812152862548828125f,-0.55139999013990159681952718528918921947479248046875f,-0.024399991309642654613298873300664126873016357421875f,0.4458000067070126970492083273711614310741424560546875f,0.17600000844895846707771625005989335477352142333984375f,0.24670000074356790431551189612946473062038421630859375f,0.27100000822544156875437693088315427303314208984375f,-0.943599983932077890358414151705801486968994140625f,-0.53759998371601103173134106327779591083526611328125f,0.720499985784292196200340185896493494510650634765625f,0.378600010120868635254964829073287546634674072265625f,-0.08510000223070390834578802241594530642032623291015625f,-0.1517999934375284798449001755216158926486968994140625f,0.651800026890635653131766957812942564487457275390625f,0.170800008904933964259953427244909107685089111328125f,0.3098000050008298256187799779581837356090545654296875f,0.1363999910712241359078689129091799259185791015625f,-1.1866000126689673077606812512385658919811248779296875f,-0.32239999475181091437292479895404540002346038818359375f,0.193800006982684236422898038654238916933536529541015625f,-0.1233999958992004764013472595252096652984619140625f,-0.7014999926686285025567713091732002794742584228515625f,-0.47039998403787652847540812217630445957183837890625f,-0.3331000403180721658458196543506346642971038818359375f,-0.559699987310171120924451315659098327159881591796875f,0.0251999916836620485582898254506289958953857421875f,1.149599965596198813955197692848742008209228515625f,0.7904999914839863084381477165152318775653839111328125f,1.733200007754565064033158705569803714752197265625f,-0.832999994985759162346994344261474907398223876953125f,-1.187200032013655093265924733714200556278228759765625f,0.126800002303719405727377989023807458579540252685546875f,0.228800013799965562544258546040509827435016632080078125f,0.351399999602138912724313968283240683376789093017578125f,0.0313000190898780061843353905715048313140869140625f,-0.4589999667033557440021240836358629167079925537109375f,-0.77660000648498428432731088832952082157135009765625f,-0.06540000273883339598768316136556677520275115966796875f,0.1991000082731246845213490814785473048686981201171875f,0.281199989254772741986698747496120631694793701171875f,0.20069999590814102674585228669457137584686279296875f,0.2128999947667121472871798459891579113900661468505859375f,0.6821000077664851612979646233725361526012420654296875f,0.346900009451806379257021717421594075858592987060546875f,0.482300009173155075359318288974463939666748046875f,0.24180000237822529829401219103601761162281036376953125f,-0.151199992309510644616921126726083457469940185546875f,-0.1704999965503812831002505845390260219573974609375f,0.31680000945627695418949087979854084551334381103515625f,-0.234099991881847435681862634737626649439334869384765625f,-0.061100003564357852070543231093324720859527587890625f,0.08809999375194321746818104656995274126529693603515625f,1.2177999875068667989808091078884899616241455078125f,-0.339500012613833224150994283263571560382843017578125f,-0.142999993979930906107256305404007434844970703125f,-0.0284999822974205674341874328092671930789947509765625f,-0.32879999305009821597423069761134684085845947265625f,-0.11559998315349229169868294775369577109813690185546875f,-0.357399996726214641551422346310573630034923553466796875f,-0.3262000045470891951282510490273125469684600830078125f,-0.335200013178587052919965572073124349117279052734375f,1.19910002242922786308554350398480892181396484375f,0.83420000957250550754906726069748401641845703125f,0.152899992382526495049432924133725464344024658203125f,0.577800004494189778370127896778285503387451171875f,-1.237800057393312780362748526385985314846038818359375f,-0.462600016498565747014026783290319144725799560546875f,0.73620001460537298765274272227543406188488006591796875f,0.050400000540912148760952504744636826217174530029296875f,0.08549999288097021032939437645836733281612396240234375f,0.1830000117123129488305721679353155195713043212890625f,-0.59590002707988054542198597118840552866458892822265625f,-0.28100000137090663798744571977294981479644775390625f,0.0961999996483324792961866478435695171356201171875f,-0.433400000369548621392823406495153903961181640625f,1.500600052797794514702900414704345166683197021484375f,0.499600005829333948526027597836218774318695068359375f,-0.34770001764893532225642047706060111522674560546875f,0.0223999744117258092757083431934006512165069580078125f,-1.30020002275258317325778989470563828945159912109375f,-0.58840002272129066085426529753021895885467529296875f,0.0740999779492614862874688697047531604766845703125f,-0.1653000451743604770626916433684527873992919921875f,0.807099963435530920463634174666367471218109130859375f,1.818899994945525566691912899841554462909698486328125f,0.359299997870623943896362106897868216037750244140625f,1.3439999998770659761504475682158954441547393798828125f,-0.222499988432973749841181643205345608294010162353515625f,0.3684999917037783401241313185892067849636077880859375f,0.1066999948576092815955007608863525092601776123046875f,0.0512000255018474348389645456336438655853271484375f,0.633699983480572992533552678651176393032073974609375f,-0.6153999992370611238357014372013509273529052734375f,0.59179998110681797385268509970046579837799072265625f,0.035900010800361314977635629475116729736328125f,-0.7221000201344487390286985828424803912639617919921875f,-0.2337000281654304512102271473850123584270477294921875f,-0.134400003174692306284754295120364986360073089599609375f,-0.48630001601651340337184592499397695064544677734375f,0.79289999059438687201151196859427727758884429931640625f,0.8318000525951390056178524901042692363262176513671875f,-0.40490001270771014407756638320279307663440704345703125f,0.40779997215866981008502989425323903560638427734375f,0.72149997784197328343225308344699442386627197265625f,1.83030003995895373947178086382336914539337158203125f,0.5824999996647239886016222953912802040576934814453125f,0.207300004777312363302144149201922118663787841796875f,-0.80079999080002295386293553747236728668212890625f,-1.1400000408291821596407089600688777863979339599609375f,-0.6718999998405574825000030614319257438182830810546875f,-0.859199985581636571652097700280137360095977783203125f,-0.0511000067681071090675004597869701683521270751953125f,-0.541200018870830579231778756366111338138580322265625f,-0.68989999564588055935843158295028842985630035400390625f,-0.50559999557733537045578486868180334568023681640625f,-0.9722999907329674673661656925105489790439605712890625f,-0.9703999962568286008490758831612765789031982421875f,1.89789998493194556061780531308613717555999755859375f,0.803199998068809151163804926909506320953369140625f,-0.887699986281991204606356404838152229785919189453125f,-0.4488000025868412734553203335963189601898193359375f,0.34469998012036107137845419856603257358074188232421875f,0.437599973881244608264751150272786617279052734375f,-1.00059999933987864295659164781682193279266357421875f,-0.20399998164176924575485827517695724964141845703125f,0.39350001306831849934297906656865961849689483642578125f,-0.08719998529851424873271525939344428479671478271484375f,0.7444999909885228195349782254197634756565093994140625f,0.426800005023181217023164890633779577910900115966796875f,0.555699998161941710606015476514585316181182861328125f,0.498800006163120368540830895653925836086273193359375f,-0.1034000053614374525778885072213597595691680908203125f,0.2399999932944776315224544305237941443920135498046875f,1.1952999759584661543243555570370517671108245849609375f,0.7365999921798707106290748924948275089263916015625f,-0.3955999892070887202777385027729906141757965087890625f,0.33800000649690620235787719138897955417633056640625f,1.4749000111803416412925571421510539948940277099609375f,0.9432000093936920848136651329696178436279296875f,-1.1339000009059905238473220379091799259185791015625f,0.6542999970138068110969697954715229570865631103515625f,-0.377399983054399701387637833249755203723907470703125f,0.2198000117063525493676934274844825267791748046875f,-0.43959998399913324806931314014946110546588897705078125f,0.62599998706579196294796929578296840190887451171875f,-0.4951999851644039996045876250718720257282257080078125f,0.33299999383091982707583156297914683818817138671875f,0.8784999699667099815059145839768461883068084716796875f,0.758499958604573976117535494267940521240234375f,-0.2217999888181687584420842540566809475421905517578125f,-0.33399997150898030895405099727213382720947265625f,-0.17549999461323029237291848403401672840118408203125f,-1.11689999205470069654211329179815948009490966796875f,0.27589999601095904768754962788079865276813507080078125f,0.94330001490265136254720346187241375446319580078125f,0.28450000642985084908787030144594609737396240234375f,0.33550003095343738923617138425470329821109771728515625f,0.78839997486993684816525274072773754596710205078125f,0.3494000208698209153368452462018467485904693603515625f,0.509999994710087722893376849242486059665679931640625f,-0.29159994188845217433225798231433145701885223388671875f,-0.755999963596463597781394128105603158473968505859375f,-1.57809999103247999840959892026148736476898193359375f,0.68599997138232016169467897270806133747100830078125f,-0.567199962127209911244563045329414308071136474609375f,0.3432999998524783880071709063486196100711822509765625f,-0.27149994647502939670857813325710594654083251953125f,0.6828000190973277927497520067845471203327178955078125f,0.03950004610419366457563228323124349117279052734375f,-1.43849999887496249328933117794804275035858154296875f,-0.62399999594688448922852330724708735942840576171875f,0.7093999791651965924899059245944954454898834228515625f,0.471200008738041020706077688373625278472900390625f,-0.1400999916508796072633913354366086423397064208984375f,-0.42520000438690175315059605054557323455810546875f,-0.834300011393427798367383729782886803150177001953125f,-0.1319999842643735377123448415659368038177490234375f,1.681200037795305579635396497906185686588287353515625f,0.5002000094532963458959784475155174732208251953125f,0.17709998712837726753122069567325524985790252685546875f,0.56800000321865073971139281638897955417633056640625f,0.440199986952543032003859480028040707111358642578125f,0.285399992084503395517458557151257991790771484375f,0.583300016616285166293209840659983456134796142578125f,0.7477999892950055738083392498083412647247314453125f,-1.054700017820298985071758579579181969165802001953125f,-0.065999985873698818750199279747903347015380859375f,0.271000009901821936608712348970584571361541748046875f,-0.373200024294853438533436928992159664630889892578125f,-0.9573999967262150079250204726122319698333740234375f,-0.359399976110458485578647014335729181766510009765625f,1.491099978455900920693011357798241078853607177734375f,0.3467999751836063904164575433242134749889373779296875f,-1.1638999926730992928725072488305158913135528564453125f,-0.350999999433755593969408437260426580905914306640625f,0.3449000059649345306667100885533727705478668212890625f,-0.1410000201463699198711765347979962825775146484375f,-0.6276999906405806850528961149393580853939056396484375f,-0.6957000535011292896570012089796364307403564453125f,-0.52980001282393895944977657563867978751659393310546875f,-0.54280002505779378907391219399869441986083984375f,0.4567000021591780534180315953562967479228973388671875f,0.8173000149846085804483664105646312236785888671875f,-0.5413999959886073298775954754091799259185791015625f,-0.98320000745654123619488018448464572429656982421875f,-0.659899980930984231264346817624755203723907470703125f,-0.38340000245571115300435849349014461040496826171875f,0.471600006149709205516273868852294981479644775390625f,-0.518399978017807239183412093552760779857635498046875f,0.3502000105708835686613156212843023240566253662109375f,-0.01569999598264626428090195986442267894744873046875f,0.6192999870300293085989551400416530668735504150390625f,-0.05839994717240415411652065813541412353515625f,0.3958000032797455158828370258561335504055023193359375f,-0.87599998036026871517378822318278253078460693359375f,-0.6125999891549349651853617615415714681148529052734375f,0.0051999847173684887735589654766954481601715087890625f,-0.3897999912172556147282875826931558549404144287109375f,-0.38250001266598676608055029646493494510650634765625f,0.364299990308284804552130253796349279582500457763671875f,1.450700002054869930390168519807048141956329345703125f,0.04370000158548348334619504385045729577541351318359375f,-0.4760000047609207474152981376391835510730743408203125f,0.03949999625980848971806835834286175668239593505859375f,0.917799997863173899759203777648508548736572265625f,-0.537200004334747571732577853254042565822601318359375f,-1.25150003260374109714803125825710594654083251953125f,-0.74689998777061694656964618843630887567996978759765625f,-0.103999978959560479552237666212022304534912109375f,-0.00709999822974205496706190388067625463008880615234375f,-0.52119999640733005730197646698798052966594696044921875f,-0.31790000363290282781036921733175404369831085205078125f,-0.04819999359846161013365417602472007274627685546875f,0.225299976889789033140942819954943843185901641845703125f,-0.14960000113546845224021808462566696107387542724609375f,-0.68820002053231010830103286934900097548961639404296875f,-0.55060000720024060427704171161167323589324951171875f,0.9844000343367458327747954172082245349884033203125f,0.32700001399219036368748447785037569701671600341796875f,-0.98020002740919576211808816879056394100189208984375f,-0.65980002183914177038559500942938029766082763671875f,1.287500003799796122194720737752504646778106689453125f,0.4997999902486807144441627315245568752288818359375f,-0.806799984459579011542018633917905390262603759765625f,0.16460001447200767188405734486877918243408203125f,-0.196600011365115623362953556352294981479644775390625f,-0.27380000258684145109100427362136542797088623046875f,1.1428000155955555516840149721247144043445587158203125f,0.61180000073909734936705717700533568859100341796875f,0.59710000135898599893380378489382565021514892578125f,0.593400001859664794068294213502667844295501708984375f,-0.9641000089436768671902200367185287177562713623046875f,-0.42260000576972966968014588928781449794769287109375f,1.01080002510994670927857441711239516735076904296875f,0.3979999966621396989552295053726993501186370849609375f,-0.41800000724196451074021751992404460906982421875f,-0.522599979990720608924448242760263383388519287109375f,-0.27499999232590199849113332675187848508358001708984375f,0.538799957976490606625930013251490890979766845703125f,-0.006200001566857071821914360043592751026153564453125f,0.19030002529993605975278114783577620983123779296875f,-0.261899999952316164808507892303168773651123046875f,0.012299979482591094637200512806884944438934326171875f,-0.10870000110119580616352408242164528928697109222412109375f,0.29939999116063098494322503029252402484416961669921875f,-0.39969998649060756168438501845230348408222198486328125f,0.151600016580522467535274699912406504154205322265625f,0.84509999429583537722265873526339419186115264892578125f,1.3016000220566983358594370656646788120269775390625f,-0.156200002293288786603397966246120631694793701171875f,0.633500010758638154584332369267940521240234375f,0.8303000093370671574888319810270331799983978271484375f,-0.163599915647505866900246473960578441619873046875f,0.069099993148446170465604154742322862148284912109375f,0.123400001263618275260114387492649257183074951171875f,-0.76539998333007108755055014626123011112213134765625f,-1.042099988245964059530024314881302416324615478515625f,-0.1642999869555235026297168587916530668735504150390625f,-1.20720001100301743690579314716160297393798828125f,0.61419999910444023072386698913760483264923095703125f,0.41950000244379026526075904257595539093017578125f,0.399499990150332440208558182348497211933135986328125f,1.1605000236332416729823080459027551114559173583984375f,0.1726000046521425634882263011604663915932178497314453125f,-0.8561000090032810927453965632594190537929534912109375f,0.226599993111193054318164286087267100811004638671875f,0.110200001406669745307453922578133642673492431640625f,0.3947999980717897816617778516956605017185211181640625f,0.20079999012947080672120137023739516735076904296875f,-0.79860000006258469174014180680387653410434722900390625f,-0.6644000287115574554519525918294675648212432861328125f,-0.9348000207215552137540726107545197010040283203125f,-0.50479999144077236650218765134923160076141357421875f,0.945200045923888954035874121473170816898345947265625f,0.1141999937772748552333723637275397777557373046875f,-0.394800022509694537031776917501701973378658294677734375f,-0.3138000234484668027334919315762817859649658203125f,-0.98319996767044026686477309340261854231357574462890625f,0.0818000174283977088407482369802892208099365234375f,0.56739999464005219653728318007779307663440704345703125f,-0.2653999964058397775801267925999127328395843505859375f,-0.37409999057799592225848073212546296417713165283203125f,-0.24999999754130819429320808922057040035724639892578125f,-1.31600002986937791860100332996807992458343505859375f,-0.52499999880790682027509319595992565155029296875f,0.0998999913617970580759219956235028803348541259765625f,-0.1503999998331071452639662311412394046783447265625f,0.72900000593066216492132980420137755572795867919921875f,0.1192000219404698579950263592763803899288177490234375f,0.31129999368339766885327435375074855983257293701171875f,0.2942000191092493111000294447876513004302978515625f,0.5359000068515540338154323762864805757999420166015625f,0.08520000163018692784788754579494707286357879638671875f,-0.46050000865757478862860807566903531551361083984375f,0.0297000202417379011166076452354900538921356201171875f,-0.8319999834820628326070846014772541821002960205078125f,-1.319400027370452921360310938325710594654083251953125f,-0.2106999953493475796761913443333469331264495849609375f,0.09349998505413481542092313247849233448505401611328125f,-0.7527999830365181832547705198521725833415985107421875f,-0.340600044378637445419144569314084947109222412109375f,0.1516999944850800918771938086138106882572174072265625f,1.269800019901990761894694514921866357326507568359375f,0.18750000193715099516822419900563545525074005126953125f,0.56319996051788390190040445304475724697113037109375f,-0.2837000006914138783287171463598497211933135986328125f,-1.08839996624589030460583671811036765575408935546875f,-0.376999988324940193962220291723497211933135986328125f,0.29389994301497990303317919824621640145778656005859375f,0.3293999981269239984982277746894396841526031494140625f,0.9199999857321385920982947936863638460636138916015625f,-0.09099999641627076751859704017988406121730804443359375f,-0.186900000064074589545271010138094425201416015625f,-0.923099998408555943996134374174289405345916748046875f,-0.76569999255538068183568611857481300830841064453125f,-0.340499999828636656928892989526502788066864013671875f,-0.08910000536739826060284030972979962825775146484375f,0.83469999256283033961523187826969660818576812744140625f,0.7196000463604932395611513129551894962787628173828125f,0.1580999990046023651046880331705324351787567138671875f,0.223100003661215240402526660545845516026020050048828125f,0.55979998656064278161892389107379131019115447998046875f,0.849300007258355638128932696417905390262603759765625f,1.387300003473460829894747803336940705776214599609375f,0.5170000042319298216852985206060111522674560546875f,-0.46319999307692061307761832722462713718414306640625f,0.2046000266909597797848618938587605953216552734375f,0.6620999923065304937352948400075547397136688232421875f,0.28980000373423087101087958217249251902103424072265625f,-0.741899982443451566638259464525617659091949462890625f,-0.7016000067830088227083251695148646831512451171875f,1.19280001980513361559133045375347137451171875f,0.54180000610351530809793985099531710147857666015625f,-1.3048000279858709138380845615756697952747344970703125f,-0.37060000332593912020229254267178475856781005859375f,1.1924000173270703495376210412359796464443206787109375f,0.7076000162005424698463684762828052043914794921875f,0.34030000311583263883363770219148136675357818603515625f,0.1469999892562630083858721263823099434375762939453125f,-1.088999990284442720422930506174452602863311767578125f,-0.0867999900102613963781550410203635692596435546875f,-0.069800002914667036435503177926875650882720947265625f,-0.2251999958932400236477633370668627321720123291015625f,0.3563999932691455097000243767979554831981658935546875f,-0.01020001057088382179216523581999354064464569091796875f,-1.29640005041509898120466459658928215503692626953125f,-0.4450000098347661747766323969699442386627197265625f,0.547400016619265183948073172359727323055267333984375f,-0.45040002538859880143462532942066900432109832763671875f,-0.44610001276582489371236306396895088255405426025390625f,0.0728000132113695297420008500921539962291717529296875f,0.40839999697953477397760480016586370766162872314453125f,0.48069996945857962344916813890449702739715576171875f,0.706400000272691119818091465276665985584259033203125f,-0.14389997244477203963697320432402193546295166015625f,0.628699979978800005397943095886148512363433837890625f,0.7130000042244788449607995062251575291156768798828125f,0.5042999857261778462458323701866902410984039306640625f,1.093900001986324088676383325946517288684844970703125f,-0.2735999885886908344900803058408200740814208984375f,0.3460999992430213723082488286308944225311279296875f,0.36140000369995839424319683530484326183795928955078125f,-0.326300014507770352878424091613851487636566162109375f,-0.526399995802342868245204954291693866252899169921875f,-0.67020001168847098682590512908063828945159912109375f,0.9097999967306851232251574401743710041046142578125f,0.82050001070648492795811534961103461682796478271484375f,0.02820000388026233650862195645458996295928955078125f,-0.806599985437095323703715621377341449260711669921875f,-0.50369997606724525862631480777054093778133392333984375f,-0.4208999977618450571981156826950609683990478515625f,-0.39509999142736196287017946815467439591884613037109375f,-0.1024000079020856990297261290834285318851470947265625f,-0.471699992734193773724626908006030134856700897216796875f,-1.068800003853440205858760236878879368305206298828125f,-0.8081000049650668426437505331705324351787567138671875f,0.3748999856993548984718245264957658946514129638671875f,-0.30010000725984553593406189975212328135967254638671875f,-0.56530000211000464016564137637033127248287200927734375f,-0.099200001339614374273878638632595539093017578125f,-0.473199981621653054109088998302468098700046539306640625f,-0.1086000024467705937780692693195305764675140380859375f,0.0919999923110008666071735206060111522674560546875f,1.0627000138118862793135122046805918216705322265625f,0.74080000801086409722984171821735799312591552734375f,0.1042999885946513938250745923141948878765106201171875f,0.0595999745368960276437064749188721179962158203125f,-0.9921999972939492540291439581778831779956817626953125f,-0.263400014935434023666260827667429111897945404052734375f,0.18749999670311801036604038017685525119304656982421875f,-0.12080001665353723439011446316726505756378173828125f,-1.017799964298307546783917132415808737277984619140625f,-0.4167999876260761737967186491005122661590576171875f,0.03529997946023943899973573934403248131275177001953125f,0.01840001974105831550332368351519107818603515625f,0.45700002244859927458264792221598327159881591796875f,-0.18979998182952362295594639363116584718227386474609375f,0.379100004957616310008461368852294981479644775390625f,0.416800008040666636333071437547914683818817138671875f,-0.75239998651891948355796557734720408916473388671875f,-0.560199997085332856983086458058096468448638916015625f,0.35149999356269834294153042719699442386627197265625f,0.3624000064492225448020690237171947956085205078125f,-0.02180000435262907654276887114974670112133026123046875f,-0.09720000765025582634137890636338852345943450927734375f,-0.11899999810755257367844706095638684928417205810546875f,0.05640000563710934056160795080359093844890594482421875f,0.60180001295804996797045305356732569634914398193359375f,0.43340001407861716842262467253021895885467529296875f,0.4899999748170371649536036784411408007144927978515625f,-0.151699999439717014837469832855276763439178466796875f,-0.54909998580962426917295715611544437706470489501953125f,-1.21710001029968228891675607883371412754058837890625f,0.78839998546838774284850615003961138427257537841796875f,0.6707999908745279071808909066021442413330078125f,0.83790000031739442221123681520111858844757080078125f,0.872200077406317664241441889316774904727935791015625f,-0.763000000424682855992841723491437733173370361328125f,-0.4712000510573390865687315454124473035335540771484375f,-0.62729997173398766019403183236136101186275482177734375f,-0.83269999678730943770688099903054535388946533203125f,0.2070999972984195469649648657650686800479888916015625f,0.5852999986827374101494569913484156131744384765625f,0.436699992398917657965284888632595539093017578125f,1.304599976734817534662624893826432526111602783203125f,0.48940001053214043924555198827874846756458282470703125f,0.4796000355944045612233139763702638447284698486328125f,0.0819000008091330666815110816969536244869232177734375f,0.1383999705672265267963894075364805757999420166015625f,-0.1218999971210956712042161598219536244869232177734375f,-0.089299983975291041815580683760344982147216796875f,-0.4206999906554818036141796255833469331264495849609375f,-0.8165000209435822053904985295957885682582855224609375f,-0.61419997053146391241540413830080069601535797119140625f,-0.992799994715302336345530420658178627490997314453125f,-0.0010000006817281192894597552367486059665679931640625f,-0.2960999980323020253081267583183944225311279296875f,0.8217000026434657655727278324775397777557373046875f,0.401800037992001701780964140198193490505218505859375f,0.838200011517107324010567026562057435512542724609375f,0.38000001668930050158223821199499070644378662109375f,1.1269999877661465159661702273297123610973358154296875f,0.39939997640848190485485247336328029632568359375f,-0.733199973258376047624551574699580669403076171875f,-0.3714000133484602894640147496829740703105926513671875f,0.51050003305822588828277730499394237995147705078125f,0.2026000063657757976898210472427308559417724609375f,-1.481300017078221120669923038803972303867340087890625f,-0.4279999746084204303997466922737658023834228515625f,1.6820000393241645308961551563697867095470428466796875f,0.63399999654293015538542022113688290119171142578125f,-1.2728000348180532608211024125921539962291717529296875f,-0.0821999943137168287421445711515843868255615234375f,1.47190004148930331240308078122325241565704345703125f,0.5882000126719475474601495079696178436279296875f,-1.040500011935830304565797632676549255847930908203125f,-0.44959998854398719458913546986877918243408203125f,-0.91269997156709414820596748540992848575115203857421875f,-0.87819998912811314539794693700969219207763671875f,0.3836999786749479834924159149522893130779266357421875f,-0.1445999959945680757300578989088535308837890625f,0.235400035484135461416599355288781225681304931640625f,-0.2324000109195711871734602027572691440582275390625f,-1.2503000074744221592482062987983226776123046875f,-0.6604000087738040036811071331612765789031982421875f,0.94169998222887496552857555798254907131195068359375f,0.50460000195503251774198361090384423732757568359375f,-0.52139999304562822945996458656736649572849273681640625f,0.313400028383731932279943066532723605632781982421875f,0.5787999779999257743412499621626920998096466064453125f,0.71130001033544498767469121958129107952117919921875f,-0.1185999945864082416591145374695770442485809326171875f,-0.32230000414401305253164764508255757391452789306640625f,-0.0105999994516372542108229026780463755130767822265625f,0.55869997308701346838688550633378326892852783203125f,0.4353999860867860949298346895375289022922515869140625f,0.08679993115067485831559679354541003704071044921875f,0.51610000747591211478493278264068067073822021484375f,1.41310006263256138225870017777197062969207763671875f,0.59579998178482085346985286378185264766216278076171875f,-0.598299994450808281243325836840085685253143310546875f,0.01629999439120288773352740463451482355594635009765625f,0.14820002876520188550557577400468289852142333984375f,-0.7332999598056080259311784175224602222442626953125f,-1.57070006512403370635411192779429256916046142578125f,0.2272999858900905945091608373331837356090545654296875f,1.46279997707605335932612433680333197116851806640625f,-0.3326999987989662788123723657918162643909454345703125f,0.8553999531924727506293493206612765789031982421875f,0.56619996816962991914579106378369033336639404296875f,1.61039998612403767452860847697593271732330322265625f,-0.35839997943341739272682389128021895885467529296875f,-0.0764000180423263230977681814692914485931396484375f,-0.2482999877825380341533900718786753714084625244140625f,0.1663999989688402791898624855093657970428466796875f,0.46419999351650476082653540288447402417659759521484375f,0.357299997997284179973576101474463939666748046875f,-0.1504000287041069583438002155162394046783447265625f,0.08160001423358931305074293049983680248260498046875f,0.257300034393370291585512177334749139845371246337890625f,-0.22999997437000274658203125f,0.25379999752044657412852757261134684085845947265625f,-0.373800004673003893884697390603832900524139404296875f,-0.338400001822412033192222224897705018520355224609375f,0.23420001493692410576841211877763271331787109375f,-0.38559999490678276146127245738171041011810302734375f,-0.668400021827221024750542710535228252410888671875f,-0.0884000462278728971199370789690874516963958740234375f,0.363800021660328098960235365666449069976806640625f,-0.022599982449412525653542616055347025394439697265625f,-0.406999988138675661275556194595992565155029296875f,0.777000024273991840573216904886066913604736328125f,-0.2582000198245051336698452359996736049652099609375f,0.35420000785887240102312034650822170078754425048828125f,0.6748000176668167426896616234444081783294677734375f,0.09000000752508652279715306576690636575222015380859375f,0.188799999308586041735225080628879368305206298828125f,0.126800020408630642609892902328283526003360748291015625f,0.06919999407529786594750476069748401641845703125f,-0.76220003316849493923967884256853722035884857177734375f,0.20000001817941637227704632095992565155029296875f,0.6790999943777917824405676583410240709781646728515625f,-0.2324000264167782603408340946771204471588134765625f,-0.4952999907478690122530906592146493494510650634765625f,-0.03100000055134299881132164955488406121730804443359375f,-1.0174000214621425897121298476122319698333740234375f,-0.460399981504678645904959921608678996562957763671875f,-0.176500000678002866294491468579508364200592041015625f,0.165299983781576287356074317358434200286865234375f,0.27880000713169561521453942987136542797088623046875f,1.4944999705553063673590941107249818742275238037109375f,-0.0570999983787536802282147618825547397136688232421875f,-0.402799995255470388855201235855929553508758544921875f,0.50639999364167442852391332053230144083499908447265625f,1.29220003111958448016594047658145427703857421875f,-0.8493999760359527417818981120944954454898834228515625f,-1.420099999064206031107460148632526397705078125f,0.7048999607771637698760969215072691440582275390625f,1.2115999979913230077954722219146788120269775390625f,0.15829998394548905338297117850743234157562255859375f,1.6845999904811375103008685982786118984222412109375f,-0.1278999928295612154016680506174452602863311767578125f,-0.71150000265240631591723285964690148830413818359375f,0.452100004301965174846600348246283829212188720703125f,-0.1071000004649160786129868938587605953216552734375f,0.222200002136826502141531136658159084618091583251953125f,0.290999997943639510111779600265435874462127685546875f,0.425699994827807071118286330602131783962249755859375f,-0.40629998813271495095023055910132825374603271484375f,0.4493999661266807610360274338745512068271636962890625f,1.37039997450113215649025733000598847866058349609375f,-0.6675999815553428362591148470528423786163330078125f,-1.018900003997981418280005527776665985584259033203125f,0.1042999956354499213606601415449404157698154449462890625f,0.4329999959543344711931922574876807630062103271484375f,0.93200001168251034311396097109536640346050262451171875f,0.4096000050842760753511129223625175654888153076171875f,-0.40689999086260775218448770829127170145511627197265625f,-0.10759999839365452789508026398834772408008575439453125f,1.16119998541772329048171741305850446224212646484375f,0.862600007557869474794642883352935314178466796875f,0.1757000363275411647379087298759259283542633056640625f,0.59640002236366296273217812995426356792449951171875f,0.42770001317858685752071323804557323455810546875f,-0.03260000159740439329425498726777732372283935546875f,-1.3480000183805824764959879757952876389026641845703125f,-0.16119998817443814687067060731351375579833984375f,1.201800017651915464256262566777877509593963623046875f,0.4514000108152627088742292471579276025295257568359375f,0.09659999277591717337099197493444080464541912078857421875f,0.22700000415742394022089456484536640346050262451171875f,0.83179999436885132535479669968481175601482391357421875f,0.250799980294704738525979337282478809356689453125f,-0.8307999941900370277636511673335917294025421142578125f,0.0976000018954277237526184762828052043914794921875f,0.362900000764429631061602776753716170787811279296875f,0.197600020372867835050101348315365612506866455078125f,0.179799993228912047271705887396819889545440673828125f,0.397000034630298870297337998636066913604736328125f,-0.290200012582540356476101806038059294223785400390625f,-0.615600012862682177683382178656756877899169921875f,0.28739999002069238631662528860033489763736724853515625f,0.2164000149130824723897603689692914485931396484375f,1.2642000123664736133122232786263339221477508544921875f,-0.130199997420608948317521935678087174892425537109375f,0.86389997650533933981620293707237578928470611572265625f,0.725500021286308083290350623428821563720703125f,-0.47609996683299582453940956838778220117092132568359375f,-1.1355999868229016414034049375914037227630615234375f,0.32579999858587971317547271610237658023834228515625f,-1.146599977725744512468963876017369329929351806640625f,0.6405000165924426003272174057201482355594635009765625f,0.917000007659197979847931492258794605731964111328125f,0.6753999889180064553073634669999592006206512451171875f,0.12240003207922001138285850174725055694580078125f,-0.813599985310435425134301112848334014415740966796875f,-0.97330000432282648414883396981167607009410858154296875f,-0.032299997662007751131341137806884944438934326171875f,-0.7668000071465972400375221695867367088794708251953125f,0.294999998584389633293767474242486059665679931640625f,0.57480000645369244693938526324927806854248046875f,0.6710000060647722808226944835041649639606475830078125f,-0.07739991421103464830366647220216691493988037109375f,-0.376999981582164966908976566628552973270416259765625f,-0.719800001722573856710596373886801302433013916015625f,0.7854000002056358287205739543423987925052642822265625f,0.301500044226647556655507287359796464443206787109375f,0.7037999799743295259446540512726642191410064697265625f,1.36720001115202816066585000953637063503265380859375f,0.0701999982029197777677609337843023240566253662109375f,0.371600055509805571318793226964771747589111328125f,0.5166000014930960393400027896859683096408843994140625f,0.8987000531435018757520083454437553882598876953125f,-0.7474999941512943113508526948862709105014801025390625f,-0.4241999884128571096653104177676141262054443359375f,-0.1559000376597048376225984611664898693561553955078125f,-0.3662000066518782404045850853435695171356201171875f,0.4310000218227505452972536659217439591884613037109375f,-0.2089999974370002622237052491982467472553253173828125f,-0.37330000666975993173224424026557244360446929931640625f,-0.424000001907348433860533987171947956085205078125f,-0.9035999944001436023910400763270445168018341064453125f,-0.50739999430477633612923682449036277830600738525390625f,0.890099985913932467695985906175337731838226318359375f,0.34920000867843636882525970577262341976165771484375f,0.3263000238955020559927788781351409852504730224609375f,-0.24040000996589672155323569313623011112213134765625f,0.275000010766088875957535719862789846956729888916015625f,0.72640003070831316023259205394424498081207275390625f,-0.546199995662271842178370206966064870357513427734375f,-0.196999989219010018803857064995099790394306182861328125f,-0.4544999935589728590201730185071937739849090576171875f,-0.36959997781515152581732763792388141155242919921875f,-0.412000014625489630049059996963478624820709228515625f,-0.5527999897420408093040578023646958172321319580078125f,1.637899975655972895083323237486183643341064453125f,0.7172000029563907474994266522116959095001220703125f,-0.658199992927908983375573370722122490406036376953125f,-0.1970000085532668254728605461423285305500030517578125f,-0.63489999244213091156296968620154075324535369873046875f,-0.58519999455213511652118540951050817966461181640625f,0.37399998903647058146049175775260664522647857666015625f,0.6263000338792803489695870666764676570892333984375f,-0.7952999924987553814759166925796307623386383056640625f,-0.4480000289976597827035220689140260219573974609375f,-0.5056000056728719993515142050455324351787567138671875f,0.24589997861385326505256898599327541887760162353515625f,0.08639999945312737406055703104357235133647918701171875f,0.79279997484087960657461735536344349384307861328125f,0.376799988520145401338368174037896096706390380859375f,0.966300028924643950034578665508888661861419677734375f,-0.626599997656047236205267836339771747589111328125f,-0.43720001554787202291407766097108833491802215576171875f,0.57939999577999101720848784680129028856754302978515625f,1.252800012242794469585760452901013195514678955078125f,-0.1441999983221291525836704749963246285915374755859375f,-0.945600017333030340438426719629205763339996337890625f,0.665899999736249359472139985882677137851715087890625f,0.1670000130794944137591073740622960031032562255859375f,-0.043400000742077789184225622420854051597416400909423828125f,0.271999993614852331180742339711287058889865875244140625f,0.13789999495297677878369313475559465587139129638671875f,0.942700000587105790827990858815610408782958984375f,-0.74959999115169040351247531361877918243408203125f,-0.5718000283062456556848474065191112458705902099609375f,0.5849999956786631205574167324812151491641998291015625f,-0.001899959793686623044095540535636246204376220703125f,-0.49640001159757340953859738874598406255245208740234375f,-0.0976000521123416842073083898867480456829071044921875f,-0.50729996897727291926827319912263192236423492431640625f,-0.41039997209831968749682573616155423223972320556640625f,0.43599999455362581368689234295743517577648162841796875f,0.31900002425908979830637690611183643341064453125f,1.190299976629018541274263043305836617946624755859375f,0.719600018942356367546153705916367471218109130859375f,0.2983000092029570726737119912286289036273956298828125f,0.566600029730797150051557764527387917041778564453125f,0.03080000281408434847918442756053991615772247314453125f,-0.320200016568601364408408471717848442494869232177734375f,1.059900029676407395839987657382152974605560302734375f,0.8860000284910203305344111868180334568023681640625f,0.146199997711181861603790821391157805919647216796875f,0.43700000512599945778902110760100185871124267578125f,-0.269900002481788359176562153152190148830413818359375f,-0.099200004617869874667945850887917913496494293212890625f,-0.294399996096640748088901773371617309749126434326171875f,-0.344600008884072384063301797141321003437042236328125f,0.384000003658235000703058403814793564379215240478515625f,0.11919999526739122330809550476260483264923095703125f,-0.67679998669475327499611694292980246245861053466796875f,-0.348399997687339979535181555547751486301422119140625f,0.735900021193921727302722501917742192745208740234375f,0.4666000158727168223293801929685287177562713623046875f,-0.25160001613348736970721120087546296417713165283203125f,-0.14420001612901689469481425476260483264923095703125f,-0.8789000039607284264775444171391427516937255859375f,-0.520200017350912258251582898083142936229705810546875f,-0.29590001752451067229543468783958815038204193115234375f,-0.437400026826560928139286943405750207602977752685546875f,1.2650999820582569110882786844740621745586395263671875f,-0.2199999952688813653622901256312616169452667236328125f,0.270500011451542121676538954488933086395263671875f,1.4853999798282984112773874585400335490703582763671875f,-0.5444000178337091711000539362430572509765625f,-0.17970001785755140844003108213655650615692138671875f,0.67560001852512385767113300971686840057373046875f,-0.82559998648762711326298813219182193279266357421875f,-0.1425999452710158266199869103729724884033203125f,-1.510400003856420259040760356583632528781890869140625f,-0.2287999537110323444721871055662631988525390625f,-1.3139000079095366491088725524605251848697662353515625f,0.61159999523460850667788690770976245403289794921875f,-0.28570000697225317320970816581393592059612274169921875f,0.348699995326995804134639911353588104248046875f,0.61210002073049540882720975787378847599029541015625f,0.563299977314471789213712327182292938232421875f,-0.044500018388033080185550716123543679714202880859375f,-0.51540000817775677433019154705107212066650390625f,0.200399991935491517125456084613688290119171142578125f,0.331199952411651565853389911353588104248046875f,-0.3467000085294245081968256272375583648681640625f,0.4281000238895416032391949556767940521240234375f,0.24269998739957809874567828956060111522674560546875f,-0.6230999943852424394208355806767940521240234375f,-1.545200007963180244274781216518022119998931884765625f,0.131400060808658736277720890939235687255859375f,0.271700036543607748029671711265109479427337646484375f,0.3224999904632568359375f,-0.582200015100836854031740585924126207828521728515625f,0.23059997352361705225121113471686840057373046875f,0.4666000076770782101220902404747903347015380859375f,-0.5283999807745221044541494848090223968029022216796875f,-0.034200007113814379078320371263544075191020965576171875f,-0.3087000213667752124280241332598961889743804931640625f,0.4214000044822692192525437349104322493076324462890625f,-0.11789999316483690261492256468045525252819061279296875f,-0.1569999863505362469595638685859739780426025390625f,0.39700003552436857745533416164107620716094970703125f,-0.14719999282360074488451573415659368038177490234375f,0.111899980729818881997061907895840704441070556640625f,0.46120000993013388068675340036861598491668701171875f,0.1628000141054390947914498610771261155605316162109375f,-0.0151999911248685837250604890868999063968658447265625f,-0.452599968442320588213334531246800906956195831298828125f,-0.6538000092178581912349955018726177513599395751953125f,-0.8640000269412995148599065942107699811458587646484375f,0.2356000004574656803146837091844645328819751739501953125f,0.8367000080637634340519070974551141262054443359375f,-0.1263999970316886933829891859204508364200592041015625f,-1.025500030040741084036426400416530668735504150390625f,-0.37060000690221794883427719469182193279266357421875f,0.24260000040531093379314597768825478851795196533203125f,0.58039999059438696082935393860680051147937774658203125f,1.34400000838935351765712766791693866252899169921875f,0.55940001693964003237624638131819665431976318359375f,-1.08319996796846407249859112198464572429656982421875f,0.96060003283023842612919906969182193279266357421875f,1.34060002582669302029216851224191486835479736328125f,-0.381999978452920885274579632095992565155029296875f,0.805399973309040007052317378111183643341064453125f,0.61899999084323642506433316157199442386627197265625f,-0.62269999764412631293453159742057323455810546875f,0.007999991111457216419466931256465613842010498046875f,-0.54390001662671583204655689769424498081207275390625f,-0.07289997356981048337587481000809930264949798583984375f,-0.86339998040199361639679409563541412353515625f,-0.15089999664425857162086686003021895885467529296875f,0.372600006067752786975688650272786617279052734375f,-0.448800001841783480216463431133888661861419677734375f,0.5244999936521050898363682790659368038177490234375f,-0.10959997498393081460932307891198433935642242431640625f,0.6968999723851680982988909818232059478759765625f,-0.340800010953843379724048645584844052791595458984375f,-0.331099964709579808186390437185764312744140625f,0.085899996513873311254627651578630320727825164794921875f,-0.28510000321790585076087154448032379150390625f,-0.26899999447539446606469937250949442386627197265625f,0.435800001603364961511033470742404460906982421875f,0.7415000284314154921361250671907328069210052490234375f,-1.201400001502037184764049015939235687255859375f,0.144999977722763961907048724242486059665679931640625f,-0.20990002932250462208685348741710186004638671875f,-0.34629998269379147490809600640204735100269317626953125f,1.233399971461295763219823129475116729736328125f,2.0117000171720977874656455242075026035308837890625f,0.0031999682664860529257566668093204498291015625f,0.4512000297486782329769994248636066913604736328125f,0.56269998784661279245256082504056394100189208984375f,-0.146200002256035743730677722851396538317203521728515625f,0.37510001716911822722266833807225339114665985107421875f,-0.25239999884963015830408039619214832782745361328125f,-0.37740002239346548407183945528231561183929443359375f,0.31540000743269924754486055462621152400970458984375f,1.214499992609024303646947373636066913604736328125f,0.27740001211166376293704161071218550205230712890625f,-0.7177999748408792068943284903070889413356781005859375f,0.5849999864399431093175962814711965620517730712890625f,0.7491000093907114187885554201784543693065643310546875f,0.0752000063240527705232807420543394982814788818359375f,-0.079199994671344864372031224775128066539764404296875f,-0.022199978369474404615857565659098327159881591796875f,-0.56859999089837032215655199252068996429443359375f,0.58180001042485240958512804354541003704071044921875f,1.5998999899089341170110856182873249053955078125f,-0.1509999845325946665752780972979962825775146484375f,0.5111000014781961198195858742110431194305419921875f,-0.1980000050067900474459747783839702606201171875f,-1.711600047761202336715768979047425091266632080078125f,-0.21300000619888326269801837042905390262603759765625f,1.25090001869797706746112453402020037174224853515625f,0.29759999280571935997841137577779591083526611328125f,0.843799993646144219638927097548730671405792236328125f,0.342000017046928395103577713598497211933135986328125f,-0.3124999856948846854720613919198513031005859375f,0.703999987900257195860831416212022304534912109375f,-0.46640003458261514168725625495426356792449951171875f,-0.4936000145539640726610741694457828998565673828125f,0.14800001338869339662096535903401672840118408203125f,0.0691000136375430340507364235236309468746185302734375f,0.70990002343654623473412357270717620849609375f,0.7853999888062475065453327260911464691162109375f,-0.0077000073671342761372216045856475830078125f,-0.818300000411272154821062940754927694797515869140625f,0.92959998526573173194265109486877918243408203125f,0.7028999870181085185549818561412394046783447265625f,0.6065999552249916604296231525950133800506591796875f,0.03269999499917031204887507556122727692127227783203125f,0.8368999878823757399004534818232059478759765625f,0.3160000176131723126360384412691928446292877197265625f,0.55469999312162343585441703908145427703857421875f,1.558899990922212541732960744411684572696685791015625f,-0.1359000191450121519665117375552654266357421875f,0.32000000134110440086487869848497211933135986328125f,0.33310000302791564763538190163671970367431640625f,-0.828500005096196812104381024255417287349700927734375f,0.447200024116039429600277799181640148162841796875f,0.972000016301870051194100597058422863483428955078125f,-1.43500001728534698486328125f,0.07209999524951005156481187441386282444000244140625f,0.66949997949600259516955702565610408782958984375f,0.636299993646144645964568553608842194080352783203125f,0.91379997814893698659943765960633754730224609375f,0.58869996075630215415230850339867174625396728515625f,-0.184600003147124880342744290828704833984375f,1.3586000139951703857832399080507457256317138671875f,1.068299998772144210335000025224871933460235595703125f,-0.1727999897420406938408632413484156131744384765625f,-0.8524999897181988472993907635100185871124267578125f,-0.4290000062286853932391750277020037174224853515625f,-1.3769999891817565895735242520458996295928955078125f,-0.432399997657537227269131108187139034271240234375f,1.0702999794602394700859804288484156131744384765625f,-0.0087999972224235278872583876363933086395263671875f,-0.270600001016259028574495459906756877899169921875f,0.143600002744793897591080167330801486968994140625f,-0.39939997711628638654701717314310371875762939453125f,0.491600015276670632147215656004846096038818359375f,1.3490999790668496416401467286050319671630859375f,-0.5971999874591826795722226961515843868255615234375f,-0.542199991184472818162021212629042565822601318359375f,0.494400004869699483123213212820701301097869873046875f,0.207900029039382872042551753111183643341064453125f,-0.2057999993681907113796114572323858737945556640625f,-0.7649000120371580724309978904784657061100006103515625f,0.07820000462532039620100476895458996295928955078125f,-0.5382000041782852850502649744157679378986358642578125f,-0.0084000075221062342478717255289666354656219482421875f,0.7090000122636557211563967939582653343677520751953125f,-0.2877999991297721660288289058371447026729583740234375f,-0.33520001619607209253359769718372263014316558837890625f,-0.0589999965429306438835510562057606875896453857421875f,0.688099985146522374890309947659261524677276611328125f,-0.52179999671578425335383144556544721126556396484375f,-0.69759998013973234520790356327779591083526611328125f,-0.629999976009130602250252195517532527446746826171875f,0.9304999918937681968600372783839702606201171875f,-0.83770000855922699400935016456060111522674560546875f,-0.4142999774932860646003973670303821563720703125f,0.63170001210570347893735743127763271331787109375f,-1.0430999583244329187436960637569427490234375f,-0.90699999827146537967337280861102044582366943359375f,0.405800004136562364465135033242404460906982421875f,-0.1432000031352043123433759319595992565155029296875f,0.22399999780952928318811245844699442386627197265625f,1.103899977809190691147023244411684572696685791015625f,0.17550000441074331547497422434389591217041015625f,-0.638199967372417464872569325962103903293609619140625f,0.27920002477168992527367663569748401641845703125f,-0.214499992907047243306806194595992565155029296875f,0.798299979996681230431931908242404460906982421875f,-0.1598000024676322627925628694356419146060943603515625f,0.2946000152170658026307137333787977695465087890625f,-0.0924000036925078926941523604909889400005340576171875f,-0.195600003325939297837976482696831226348876953125f,0.142699997979402581904651015065610408782958984375f,-0.72080000190138804327943944372236728668212890625f,-0.57950000154972070021131003159098327159881591796875f,0.38730001148283488277002106769941747188568115234375f,0.06620000177174800715107494397670961916446685791015625f,0.15779999480843542869479279033839702606201171875f,-1.003099978739023345752912064199335873126983642578125f,0.54729999203681956032596644945442676544189453125f,-0.06759997760653480014525484875775873661041259765625f,0.52209999480247493153228788287378847599029541015625f,-0.35580000443458548176067779422737658023834228515625f,-1.03729997609257740265320535399951040744781494140625f,-0.52020000036358826633886565105058252811431884765625f,-0.954000008352100525854666557279415428638458251953125f,-0.4421999912217258188462665202678181231021881103515625f,-0.04130001631453594779230797939817421138286590576171875f,-0.49580001799464235023862102025304920971393585205078125f,-0.85740003766715577881996068754233419895172119140625f,0.8342000110626219822052007657475769519805908203125f,0.4431000453472133671084520756267011165618896484375f,0.5731999992609022598344381549395620822906494140625f,0.63690006432533241564897252828814089298248291015625f,-0.26060001137256616488002691767178475856781005859375f,-0.203100020126999236680376270669512450695037841796875f,0.1128000048294663937298309974721632897853851318359375f,0.6012000205285847886926831051823683083057403564453125f,0.074999996870756202582697369507513940334320068359375f,-0.2274999684840437463861917422036640346050262451171875f,-0.10140000030994400503914221189916133880615234375f,0.24550000172853447821808003936894237995147705078125f,-0.7030000169277190735783733543939888477325439453125f,-2.170500041663647294853944913484156131744384765625f,0.349400005912780642347570392303168773651123046875f,-0.4089999770224093111892216256819665431976318359375f,0.123999994009733160993391720694489777088165283203125f,1.3645999917477367180396186085999943315982818603515625f,-0.580200016605854074214221327565610408782958984375f,0.2163999728918071241423604078590869903564453125f,-1.987999985933303737084543172386474907398223876953125f,0.39500002562999725341796875f,-0.83850000211596498189692283631302416324615478515625f,-0.08789996824264534325266140513122081756591796875f,-0.23259999139010911761715760803781449794769287109375f,-0.495199968214333541283167505753226578235626220703125f,-0.83599999742209896158584570002858527004718780517578125f,-0.38159998867809719058641348965466022491455078125f,0.12510001374185064637600817150087095797061920166015625f,0.014599956655501955538056790828704833984375f,0.35020001258254040976680698804557323455810546875f,0.291399960374832289744517765939235687255859375f,0.40090000573396711303075790056027472019195556640625f,0.636200007545948409415359492413699626922607421875f,0.23869999510347827964551470358856022357940673828125f,0.080699999149143497589875551057048141956329345703125f,-0.92640001565814011907917802091105841100215911865234375f,0.5925000102818014369177035405300557613372802734375f,0.1207000087231400176079887387459166347980499267578125f,0.85100001537799840889420011080801486968994140625f,-1.51120001276135429435498735983856022357940673828125f,0.271800038290024303933023475110530853271484375f,0.11220000787377326645355424261651933193206787109375f,-0.619200023877620964185553020797669887542724609375f,1.4234000078201294314794722595252096652984619140625f,-0.9693000125110149411966631305404007434844970703125f,-0.1921000057548283290742574536125175654888153076171875f,-0.41419998543262437351586413569748401641845703125f,0.424100005739927343739736897987313568592071533203125f,-0.535500053882599136301223552436567842960357666015625f,-0.624400023198127929191514340345747768878936767578125f,0.11459995516538601378897510585375130176544189453125f,-0.48799999740719801621935403090901672840118408203125f,0.214199966657161677829890322755090892314910888671875f,-1.057600003981590486290542685310356318950653076171875f,-1.333200036215782091630899230949580669403076171875f,0.6901999993205070182966665015555918216705322265625f,-0.027599949592352235328007736825384199619293212890625f,-0.621199993091821500712512715836055576801300048828125f,-0.3762000160396095083115142188034951686859130859375f,0.568599975699186188649036921560764312744140625f,1.610199996340274442019335765508003532886505126953125f,-0.213399993515014596567880289512686431407928466796875f,-0.6423999972850082951225658689509145915508270263671875f,-0.1393999953329563279424974098219536244869232177734375f,-0.44720000072121590672935553811839781701564788818359375f,0.3535999955549835593870966476970352232456207275390625f,-0.4559999621622268239917730170418508350849151611328125f,0.50960000106096270400257708388380706310272216796875f,-0.02379997814893730634366875165142118930816650390625f,0.58079999385476099860170506872236728668212890625f,0.629399978420138950241380371153354644775390625f,0.70239999586939827480591702624224126338958740234375f,0.71310003119110998426322112209163606166839599609375f,0.43659999262690529775454706395976245403289794921875f,-0.5183999988794329993879728135652840137481689453125f,1.01199999618530256384474341757595539093017578125f,0.64260000800490413297438863082788884639739990234375f,-0.7130000316798685844332794658839702606201171875f,-0.518599994325636970415871473960578441619873046875f,-0.5917000086784363066527703267638571560382843017578125f,-0.00029999227523802574069122783839702606201171875f,-0.968099989318847775621179607696831226348876953125f,-0.12719999073743792905588634312152862548828125f,-0.351799995672702703330969598027877509593963623046875f,-0.221400027132034438182017765939235687255859375f,0.23520000215172753854631082504056394100189208984375f,-0.961899953758717174423509277403354644775390625f,0.388999982237816066543700799229554831981658935546875f,-0.122700032997130392686813138425350189208984375f,-0.8052000125825404808210805640555918216705322265625f,0.81400001353025430717025301419198513031005859375f,-0.4213000025868416376084724106476642191410064697265625f,0.35440000568926333102126591256819665431976318359375f,-0.46519999175816761916024688616744242608547210693359375f,0.449899981899559708864444473874755203723907470703125f,0.30149998596310612075654944419511593878269195556640625f,0.352399987971782735485248849727213382720947265625f,1.5385999689936638024079229580820538103580474853515625f,0.4499999631941324906847512465901672840118408203125f,0.428499987363815559859858694835565984249114990234375f,0.0342999971628188404793036170303821563720703125f,0.0880999887228011147044526296667754650115966796875f,0.86849996829032960476979496888816356658935546875f,0.810999954432249037239444078295491635799407958984375f,0.4255999511718755456968210637569427490234375f,0.69570001356601718356387209496460855007171630859375f,1.14819998853206595157416813890449702739715576171875f,-0.60439998892545698794265263131819665431976318359375f,0.0152000212252136890356268850155174732208251953125f,0.57299999472498885921822875388897955417633056640625f,-0.420599997663497759958772803656756877899169921875f,-0.4403999917864798163691375521011650562286376953125f,0.8978000059843065372433557058684527873992919921875f,0.1942000140428543630832791677676141262054443359375f,-0.446600044035911469819666308467276394367218017578125f,0.426400006568431688691589442896656692028045654296875f,0.1847000267982483734385823481716215610504150390625f,0.6195999959945677204586900188587605953216552734375f,-0.040999983638524639673050842247903347015380859375f,0.0909999931752683011154658743180334568023681640625f,-0.49630000690817865915960283018648624420166015625f,0.7835999849379060577803102205507457256317138671875f,1.059900032228231481212787912227213382720947265625f,0.1972000221788885454543560626916587352752685546875f,-0.057200021582841653611239962629042565822601318359375f,-0.450000022798776821986166396527551114559173583984375f,0.96539997286200485149265659856610000133514404296875f,-0.45419998871088029801512675476260483264923095703125f,-1.9642999783128496371631399597390554845333099365234375f,0.296599984058737831116303595990757457911968231201171875f,1.027799995441734726142612998955883085727691650390625f,-0.45339999977350220472516184599953703582286834716796875f,-1.10290001853406405274427015683613717555999755859375f,-0.14240000123381602037397897220216691493988037109375f,1.36689999220371216637204270227812230587005615234375f,-0.7191999602496625954017872572876513004302978515625f,0.01100001239776560169048025272786617279052734375f,-0.48750002682209014892578125f,0.9180999732255941125913523137569427490234375f,0.8044000114262104261797503568232059478759765625f,-0.5605999714374547693296335637569427490234375f,-0.1013999982237816510632910649292171001434326171875f,1.1179999918937681968600372783839702606201171875f,0.55830000517964339934451345470733940601348876953125f,-0.316400033390522139598033390939235687255859375f,0.422499981224536913515521519002504646778106689453125f,0.20199996340274850581408827565610408782958984375f,0.108299984467029819512617905274964869022369384765625f,-0.521299979639053390201297588646411895751953125f,0.56919998528361315237589224125258624553680419921875f,-0.0911000479698174103759811259806156158447265625f,0.7666999970555306020969510427676141262054443359375f,0.675500012159346852058661170303821563720703125f,0.41919999973773958146239237976260483264923095703125f,-0.758100004220008116817552945576608180999755859375f,-1.6644999932050705826469538806122727692127227783203125f,0.640800011289119453294915729202330112457275390625f,0.8086999761417510423910925965174101293087005615234375f,-0.79309999367743699849597760476171970367431640625f,-0.6721000005021691148243689895025454461574554443359375f,-0.09049996380507963067429955117404460906982421875f,0.62360001794397834640193423183518461883068084716796875f,-0.8934999819993976188925444148480892181396484375f,-1.34379998023509994453661420266143977642059326171875f,0.8722000217318532921950691161327995359897613525390625f,0.0588000186800956559096675846376456320285797119140625f,-0.0839999943450090158503229531561373732984066009521484375f,-0.68919999709278345623175709988572634756565093994140625f,-0.135400014082342590171492702211253345012664794921875f,0.1237999997556207265603234191075898706912994384765625f,0.01590001638829685592924079173826612532138824462890625f,-0.13540000162124632421267733661807142198085784912109375f,0.22100001735240237810131702644866891205310821533203125f,-0.536600009912252318144965101964771747589111328125f,-0.4467000558406113430720552059938199818134307861328125f,0.1578000038981437480600789058371447026729583740234375f,-0.24069996524900172385486030179890803992748260498046875f,-0.8028000280380249709111240008496679365634918212890625f,-1.14990002820491810098246787674725055694580078125f,0.2929999831020833056527408189140260219573974609375f,0.477699983674288120027995319105684757232666015625f,0.2954000107109546746642081416212022304534912109375f,-0.02869997357130049664419857435859739780426025390625f,-0.07399999743700025334192105219699442386627197265625f,-0.1065000415444374848306097192107699811458587646484375f,0.8014000053763388198291295339004136621952056884765625f,0.63870000650286673504751888685859739780426025390625f,0.51079999817609778034466216922737658023834228515625f,0.51710002371072771865101458388380706310272216796875f,-0.671000016123056486350151317310519516468048095703125f,-1.61130003209114125439782583271153271198272705078125f,0.33640000254511825961145632390980608761310577392578125f,0.5716000252604487297247715105186216533184051513671875f,0.44010001634955386862202431075274944305419921875f,0.72799998787045439030407578684389591217041015625f,-0.42749998563900593051556597856688313186168670654296875f,-0.37809997208938028734337422065436840057373046875f,-0.33519998302236209131166333463625051081180572509765625f,0.732899960047006970853544771671295166015625f,0.117299986299872482931050399201922118663787841796875f,0.10620000575780874640940965036861598491668701171875f,-0.36340002618581035420675107161514461040496826171875f,0.12570003621578251795654068700969219207763671875f,-0.8396000022530556439193105688900686800479888916015625f,0.05249999754130829643372635473497211933135986328125f,-0.49970001282840936784879204424214549362659454345703125f,-0.2983999733984461499858298338949680328369140625f,-0.5634000634014604269594883589888922870159149169921875f,-0.5050999807357783311090315692126750946044921875f,-0.07279997901320456321627716533839702606201171875f,0.7972999848842619030619971454143524169921875f,0.8674999959766864776611328125f,0.24160000052452090812948881648480892181396484375f,1.066099996262788618395234152558259665966033935546875f,0.0393999512255192030352191068232059478759765625f,0.4243999871373176357991496843169443309307098388671875f,-0.228800024044513605758766061626374721527099609375f,-0.881000004202127318109205589280463755130767822265625f,-0.25729997862577391742888721637427806854248046875f,0.33649999870359881271042468142695724964141845703125f,-0.6870000189095737397337870788760483264923095703125f,0.61590000185966486956345988801331259310245513916015625f,-0.37429997943043691321918231551535427570343017578125f,0.40000001609325419593687911401502788066864013671875f,0.653299990874529168394246880779974162578582763671875f,0.268599996709823596319210992078296840190887451171875f,0.2651999875485895330484709120355546474456787109375f,-0.74400002142786991754519476671703159809112548828125f,0.8117999842733147897178014318342320621013641357421875f,-0.6425999931037422729929176057339645922183990478515625f,-1.4744000448048113494081690078019164502620697021484375f,-0.0206000042200089605870516606955789029598236083984375f,-0.357699979203939033567394290002994239330291748046875f,-0.484600006127357207930117510841228067874908447265625f,0.959899957126378655658527350169606506824493408203125f,0.354800010663270715127737275906838476657867431640625f,-0.126499988049268541345782068674452602863311767578125f,-0.519599995845556161810918638366274535655975341796875f,-0.58249999895691828566413050793926231563091278076171875f,-0.34759999124109730583853661300963722169399261474609375f,0.618400021044910541689887395477853715419769287109375f,-0.25779999182820301228957760031335055828094482421875f,0.592299982500076094282803751411847770214080810546875f,0.072199990439414829523911976139061152935028076171875f,0.160799993705749155736839384189806878566741943359375f,-0.003799994987249366573678344138897955417633056640625f,-0.627599969485401754099029858480207622051239013671875f,0.4470000148117543314896238371147774159908294677734375f,-0.08849998937547187694008243852294981479644775390625f,-0.053200006487965634249803770217113196849822998046875f,-0.3617999755188823218787774749216623604297637939453125f,1.6062000051617622009558772333548404276371002197265625f,0.139599990332126111525212763808667659759521484375f,0.7502999985337257538020594438421539962291717529296875f,-0.26960001119375220923757296986877918243408203125f,0.36009998714327817737057557678781449794769287109375f,-1.0731999848067754754765701363794505596160888671875f,-1.665700015801191202768904986442066729068756103515625f,0.1301000664174551957330550067126750946044921875f,0.0074000185191629430647708431934006512165069580078125f,-0.72840000945925709174844087101519107818603515625f,0.1453999902963640789010923981550149619579315185546875f,-1.285399943506718045682646334171295166015625f,-0.318099974268675200761435917229391634464263916015625f,0.998899948751926558543345890939235687255859375f,-0.210800006222724878313101726234890520572662353515625f,-0.6831999645411972466035877005197107791900634765625f,-0.43409998376071456949176763373543508350849151611328125f,-0.47999996915459686164240338257513940334320068359375f,-0.10989998178780069348903225545655004680156707763671875f,0.773500022530555497723980806767940521240234375f,-0.11470000519156453577807042165659368038177490234375f,1.08079997433424068731255829334259033203125f,-0.8404000199496746148497550166212022304534912109375f,0.6721000053822994146912606083787977695465087890625f,0.2079000054210424774936427638749592006206512451171875f,0.93309999334216087163440533913671970367431640625f,-0.1713999942004680310247977104154415428638458251953125f,0.05409999940693399622659853775985538959503173828125f,-0.061700008231401370917268422999768517911434173583984375f,-1.11820000671148278570399270392954349517822265625f,0.404799997699260671879528672434389591217041015625f,0.5533000160574914616518071852624416351318359375f,0.180200018095970193598986952565610408782958984375f,-0.65849995778501035914587191655300557613372802734375f,0.250599997141957242519083592924289405345916748046875f,-0.202299996991456065842385214637033641338348388671875f,-0.855200011688471040116610311088152229785919189453125f,-1.13240004951357864371175310225225985050201416015625f,0.47320000984072674299341088044457137584686279296875f,0.82190004286766082941539934836328029632568359375f,-0.29599999809265131744950849679298698902130126953125f,-0.064300020557642056928671081550419330596923828125f,0.642000006169080705831220257095992565155029296875f,-0.789799970355629543661279967636801302433013916015625f,0.038000000461936134232843187419348396360874176025390625f,0.894499999426305425487271349993534386157989501953125f,0.1138000088453292590884302626363933086395263671875f,0.380599983060360358422258286736905574798583984375f,-0.13139999479651454095119333942420780658721923828125f,-0.28580001158714285480755279422737658023834228515625f,0.0801999982774257436091147610568441450595855712890625f,1.338500023089349344473930614185519516468048095703125f,-0.18140000686645496585924774990417063236236572265625f,-0.40830004861652913206171433557756245136260986328125f,-0.568600022414326833342101963353343307971954345703125f,0.469199992920457720213534003050881437957286834716796875f,0.236000010237097768150960064303944818675518035888671875f,0.1918000089347361714064987836536602117121219635009765625f,-0.55350000852346425972427823580801486968994140625f,-0.1280000121593474204928497783839702606201171875f,-0.61269997160434730432143624057061970233917236328125f,0.8037999939441675678608589805662631988525390625f,0.03910000648498535014141452847979962825775146484375f,0.31489998730123058834351468249224126338958740234375f,-0.25039999953508385033273953013122081756591796875f,-0.548299991619586535307462327182292938232421875f,0.029600002253055368584000461851246654987335205078125f,-0.57010002961158789958062698133289813995361328125f,0.174000009059906091124503291212022304534912109375f,0.01020001273155202170528355054557323455810546875f,-0.417800010752678030456763735855929553508758544921875f,0.3192999786853789601082098670303821563720703125f,0.9284999989494682193225116861867718398571014404296875f,-0.9259999798759821487692533992230892181396484375f,0.099300001297891016260876995147555135190486907958984375f,0.4955999893188476335126324556767940521240234375f,0.2490000065267086171161281527020037174224853515625f,0.1888999968826770015084548504091799259185791015625f,0.679799999412894262462714323191903531551361083984375f,-0.32260000807940958367225903202779591083526611328125f,-0.02100000625103681795025067913229577243328094482421875f,0.8610000129938129020956694148480892181396484375f,0.0823999978065490523704283987171947956085205078125f,-0.74319999873936115619699194212444126605987548828125f,-0.5020000071749091130612896449747495353221893310546875f,0.81889998939484343054573400877416133880615234375f,0.449800004348903825768246633742819540202617645263671875f,0.048200050967931673540078918449580669403076171875f,0.274199999290704710119825904257595539093017578125f,0.3753999753952026452452628291212022304534912109375f,0.66800000649690627341215076739899814128875732421875f,-0.24799995255470275168363514239899814128875732421875f,-0.3150000366568566079195079510100185871124267578125f,-0.50640003342777506478711302406736649572849273681640625f,-0.13320000253915786903036178046022541821002960205078125f,-0.022700003213435515636575701137189753353595733642578125f,-0.0846000032961367054440415813587605953216552734375f,1.286199936914444119651079745381139218807220458984375f,-0.558999998927116603653075799229554831981658935546875f,-0.28719998030662541310675806016661226749420166015625f,0.00900000727176664128137417719699442386627197265625f,-0.837399982085823779698330326937139034271240234375f,-0.429999986365437347757278985227458178997039794921875f,-0.78030000214725647733615687684505246579647064208984375f,0.51300001186132437869247269190964289009571075439453125f,0.52640001200735586639467555869487114250659942626953125f,-0.70739999676346776436020036271656863391399383544921875f,-0.528700008663534948283313497086055576801300048828125f,0.635000013262033480287982456502504646778106689453125f,0.5250000152736904457384525812813080847263336181640625f,0.0211999986052513467171820593648590147495269775390625f,0.74959996820390273342127329669892787933349609375f,0.0972000063464046049066524801673949696123600006103515625f,0.1010000040344893879051824114867486059665679931640625f,0.33979999918937675573715750942938029766082763671875f,-0.7408999589443199251803662264137528836727142333984375f,1.36120000113844863420808906084857881069183349609375f,-0.0005000145435332115084747783839702606201171875f,0.104400002485513621053314636810682713985443115234375f,-0.03060002269744899194847675971686840057373046875f,-0.3576000448107716778167741722427308559417724609375f,-0.9781999772071827692343504168093204498291015625f,0.39330001121461399815615322950179688632488250732421875f,-0.253200002054870143552989247837103903293609619140625f,0.72339996607825163277283309071208350360393524169921875f,-0.84789997360706337303781765513122081756591796875f,-1.2769999721944333970213847351260483264923095703125f,0.07759999295473107849829830229282379150390625f,-0.969300000888109192231922861537896096706390380859375f,0.801199995923042251888546161353588104248046875f,-0.498700004863739110305687063373625278472900390625f,-0.37439995800554815019722809665836393833160400390625f,0.239000020906329158076886187700438313186168670654296875f,0.19489998774826489125189254991710186004638671875f,-0.28219998328685769184431819667224772274494171142578125f,-0.41059997136294867914330097846686840057373046875f,0.54170000815689556272758409249945543706417083740234375f,0.47469997225999804868479259312152862548828125f,1.0756999769091606378168535229633562266826629638671875f,-0.28609999454915513439345886581577360630035400390625f,-0.1339999945126473246403975281282328069210052490234375f,0.1947000012613830932650671456940472126007080078125f,-0.469900015632063183801392369787208735942840576171875f,0.36619998579025292428923421539366245269775390625f,1.214199982154369639175683914800174534320831298828125f,-0.49930001266002665971655005705542862415313720703125f,0.43120000873804098517894090036861598491668701171875f,-1.188999958992004390978536321199499070644378662109375f,0.440399986422062017510370424133725464344024658203125f,0.6884999682158230438488999425317160785198211669921875f,-0.5748000117808580444744848136906512081623077392578125f,-0.807100039915740641305319513776339590549468994140625f,-0.0235999831497668566271386225707828998565673828125f,0.96170007781982480565829973784275352954864501953125f,-0.61379998589754070081880854559130966663360595703125f,-1.96640004858970574019849664182402193546295166015625f,0.53959999867677677087840493186376988887786865234375f,1.1700000624358650735956643984536640346050262451171875f,0.0377999954044818675669148433371447026729583740234375f,-1.27650002560019526498535924474708735942840576171875f,0.9090000290274620198260890902020037174224853515625f,2.035100014263392598223845197935588657855987548828125f,-0.632199989098310322077622913639061152935028076171875f,-0.44880001048445716804735639016143977642059326171875f,-1.00179999164938937639135474455542862415313720703125f,-0.655300001066922988002261263318359851837158203125f,-0.3507999989211558755641817697323858737945556640625f,-1.452700002896786468653544943663291633129119873046875f,-0.342400004363060173062649482744745910167694091796875f,0.678199975419044420732461730949580669403076171875f,-0.8034000158667566893200273625552654266357421875f,-0.778000018566846751610910359886474907398223876953125f,0.466000003904104342922209980315528810024261474609375f,-0.13789998329281782929456312558613717555999755859375f,-0.4797999875664711311173959984444081783294677734375f,0.95699998903274519079786841757595539093017578125f,1.453599973762035535429504307103343307971954345703125f,-0.5727999979376789951857062987983226776123046875f,-0.2727999989435077043964383847196586430072784423828125f,-0.863699982102215102486297837458550930023193359375f,-0.391699968855083235919067874419852159917354583740234375f,-0.9916999919891367198943044058978557586669921875f,-1.7150000092387198691312732989899814128875732421875f,0.4250000721216196097884676419198513031005859375f,0.2753000374257563720448160893283784389495849609375f,-0.41100003266334539375748136080801486968994140625f,-0.3509000100553034684480735450051724910736083984375f,0.99389999420046848399579175747931003570556640625f,2.212299995613097536306668189354240894317626953125f,-0.17840003762245260077179409563541412353515625f,-0.751299969059228800460914499126374721527099609375f,-0.1112999602675444066335330717265605926513671875f,-1.027899956768751454916355214663781225681304931640625f,-0.189099983239174207483301870524883270263671875f,-1.3867999776422976765388739295303821563720703125f,-0.05029998154640225038747303187847137451171875f,0.4490000042915340827676118351519107818603515625f,-1.141200006949901535335811786353588104248046875f,-0.992400005853176114811731167719699442386627197265625f,-0.4861999989032739222238888032734394073486328125f,-0.285000018924474574077976285479962825775146484375f,-0.09369999979734444650603109039366245269775390625f,0.855300008070468908982775246840901672840118408203125f,0.4825000284612175338594397544511593878269195556640625f,0.3178000100076199263554599383496679365634918212890625f,0.155399993425607618746653315611183643341064453125f,-0.3239999834299087666522609652020037174224853515625f,-1.1379999881684772500278768347925506532192230224609375f,-0.5769999954402444952705764080747030675411224365234375f,0.4309000003695491187727384385652840137481689453125f,0.2544000032305717695635394193232059478759765625f,-0.7070999979317189598759796353988349437713623046875f,-0.65400001963973064533774959272705018520355224609375f,1.165799986851215663818948087282478809356689453125f,0.4694000121712686990349538973532617092132568359375f,-0.367699958193302478548503131605684757232666015625f,-0.470999994218349371521981083787977695465087890625f,-1.615100019776821493877605462330393493175506591796875f,-0.447400015985965904263821357744745910167694091796875f,0.040699981379508809453682260937057435512542724609375f,0.711000018060207406023209841805510222911834716796875f,0.0954000224828723819570086561725474894046783447265625f,0.1076000001072883183184103472740389406681060791015625f,-0.22800001797080060583766680792905390262603759765625f,-0.152200008767843453227897043689154088497161865234375f,0.71760002827048285922728609875775873661041259765625f,0.28280001060366632970044520334340631961822509765625f,-1.39339995507001912500300022657029330730438232421875f,-0.23440001217126837929072280530817806720733642578125f,0.90839995723068722721649237428209744393825531005859375f,-0.07020000207722187912651179431122727692127227783203125f,0.2909000226095319607821920726564712822437286376953125f,0.257299985033273781453999617951922118663787841796875f,1.1542999907553195271248114295303821563720703125f,-0.3631999879360201521905082699959166347980499267578125f,0.27200000929832413021358661353588104248046875f,-0.14869998772740355974519843584857881069183349609375f,-0.953899953967333402715667034499347209930419921875f,-1.09509999593496321068641918827779591083526611328125f,0.81759999504089364563697017729282379150390625f,0.730199988591671012017059183563105762004852294921875f,-0.61169999854564593988470733165740966796875f,-1.457300000083446178678059368394315242767333984375f,1.618400006628036180700291879475116729736328125f,-0.250599976950883984727624920196831226348876953125f,0.506000024318694840985699556767940521240234375f,0.198600000733136994313099421560764312744140625f,-1.179600000464915865450166165828704833984375f,-0.167600005322694922682558171800337731838226318359375f,0.1393999793887132909731008112430572509765625f,0.469700008916854816476416090154089033603668212890625f,-0.399600006276369157376393559388816356658935546875f,-0.2466999902009965328630869407788850367069244384765625f,-0.7729000156283376554711139760911464691162109375f,-0.938699976104497846307594954851083457469940185546875f,1.188499984383582841473980806767940521240234375f,1.395499976694583921243975055404007434844970703125f,-1.0241999770879743891782709397375583648681640625f,-0.649199997428059560888868873007595539093017578125f,-0.01099999794363970551103193429298698902130126953125f,-0.40300001871585833246314223288209177553653717041015625f,-0.08660000309497117854107273160479962825775146484375f,0.264199998098611832375581798260100185871124267578125f,-0.4389999649524691793800457162433303892612457275390625f,0.006600001642107944377357853227294981479644775390625f,0.018199988569320257880690405727364122867584228515625f,0.99560001599192649024416823522187769412994384765625f,0.90250001817941694071123492904007434844970703125f,0.627000009298324556539228069595992565155029296875f,-0.53199999544024390019103520899079740047454833984375f,-0.37080000130534163105266998172737658023834228515625f,0.23450003403425245807056853664107620716094970703125f,0.18880001629591003364794232766143977642059326171875f,0.077199975165725021053475529697607271373271942138671875f,0.6102000147432089516286168873193673789501190185546875f,0.65230002508014450768314418382942676544189453125f,0.2371999973684548734809141024015843868255615234375f,-0.671499988757073840162092892569489777088165283203125f,-0.00360002725720409699761148658581078052520751953125f,0.20519997950196255942501011304557323455810546875f,0.3083999984323977372469016700051724910736083984375f,0.7049999961256983027624301030300557613372802734375f,0.0948000045537948921037241234444081783294677734375f,-0.057399968749284102642604921129532158374786376953125f,0.4836000247240068716791938641108572483062744140625f,-0.332099947565793751635965236346237361431121826171875f,-0.750800026637315820465801152749918401241302490234375f,-0.29480001572966596512515025096945464611053466796875f,0.29239999586939813269736987422220408916473388671875f,-1.26980001021623589707587598240934312343597412109375f,-0.2514999937489628134557051453157328069210052490234375f,0.1292999940335750608255693805404007434844970703125f,0.7050999779790638744003672400140203535556793212890625f,-0.01630000042617307798309411737136542797088623046875f,0.701699987667799174317906363285146653652191162109375f,0.26430000193118985407636500895023345947265625f,0.29999999254941922544048793497495353221893310546875f,0.60719997971057892982571502216160297393798828125f,-0.141799990159273026080199997522868216037750244140625f,-0.91629996771812471934026689268648624420166015625f,0.154800010216236128002265104441903531551361083984375f,1.013400002455711046422948129475116729736328125f,0.8689000127151607077991002370254136621952056884765625f,0.3426999743983145663150935433804988861083984375f,0.149299995262920870420231267416966147720813751220703125f,0.32599998594820522868076295708306133747100830078125f,-0.02460004844665508727530323085375130176544189453125f,-0.77909998800754465264617465436458587646484375f,0.29659998532533649040487944148480892181396484375f,0.5422999741554264119258732534945011138916015625f,-0.715700021612644121660196105949580669403076171875f,0.590800006520748155480760033242404460906982421875f,-0.52459998034834853797292453236877918243408203125f,1.433000005006789479011786170303821563720703125f,-0.37340001199245431706685849349014461040496826171875f,0.1517000256657610179900075308978557586669921875f,-0.997200038719177062063181438134051859378814697265625f,1.2536999959230428203227347694337368011474609375f,1.1082999916195870326873773592524230480194091796875f,-0.09459998869299912627184312441386282444000244140625f,-0.19819998331665988899885633145458996295928955078125f,0.4701000151574614704941268428228795528411865234375f,-0.2168000056564807209724676795303821563720703125f,-0.22490002671480213702892569926916621625423431396484375f,-0.31039999864101408544314608661807142198085784912109375f,-0.1350999925076958874115007347427308559417724609375f,-0.67360000684261311931777527206577360630035400390625f,-0.3606000101059676499204442734480835497379302978515625f,-0.3643999935448170024443470538244582712650299072265625f,0.02090000144988291008729675013455562293529510498046875f,0.35319999568462367989241101895458996295928955078125f,-0.9826000097930431831372288797865621745586395263671875f,-0.4036000004351139036629092515795491635799407958984375f,0.3620000011026860420315642841160297393798828125f,0.78420002685785306084653711877763271331787109375f,0.39140003774911213962894862561370246112346649169921875f,-0.17559999371469026119285672393743880093097686767578125f,0.314899999240785655185703717506839893758296966552734375f,-0.3909999901950360623459346243180334568023681640625f,-0.455799983870983194123027715249918401241302490234375f,-0.375199989038705883359625659068115055561065673828125f,-0.4293999915331598860035455800243653357028961181640625f,0.7468000010371209640425149700604379177093505859375f,0.329200004953146052599777249270118772983551025390625f,-0.088800000500679043824447944643907248973846435546875f,0.4461000097483391879649161637644283473491668701171875f,-0.5837999964773654681948755751363933086395263671875f,-0.15260003315061254625106812454760074615478515625f,0.513000020802020895160922009381465613842010498046875f,0.5939000003099437208220479078590869903564453125f,-0.05880000094771364871348851011134684085845947265625f,0.5659000113964083311657304875552654266357421875f,-0.317600001075863946908839352545328438282012939453125f,-0.2059999859482051220993525930680334568023681640625f,-1.1772999998599289472878126616706140339374542236328125f,-0.30999995768070220947265625f,-0.3825999761164189383322309367940761148929595947265625f,0.49080000048577776539104888797737658023834228515625f,-0.52990002243071787635386726833530701696872711181640625f,0.67759998669624366129937698133289813995361328125f,0.5943000107228755268806708045303821563720703125f,-1.0596999837338927363816765137016773223876953125f,-0.4976999929130079269867792390868999063968658447265625f,0.64819998614787976976003847084939479827880859375f,0.18770002770721905260842277129995636641979217529296875f,0.362999996177852057854806844261474907398223876953125f,-0.9285999783255161066364280486595816910266876220703125f,0.70269999306201924582637730054557323455810546875f,0.067100017601251504828496763366274535655975341796875f,0.5490999962031839487508477759547531604766845703125f,0.1378999714463950976295336658949963748455047607421875f,0.3005999964714050065595074556767940521240234375f,0.767200031566619866651990378159098327159881591796875f,0.0379999917447566559758342918939888477325439453125f,-0.2360000232011081333638458090717904269695281982421875f,-0.12909997370243075920370756648480892181396484375f,0.290600011000037294905951057444326579570770263671875f,-0.4675999954134224534385566585115157067775726318359375f,0.4050000107288358464074917719699442386627197265625f,-0.54909993346929564239644605549983680248260498046875f,0.83600001746416108261428234982304275035858154296875f,0.61600004443526312769563446636311709880828857421875f,0.65020001958608641956516294158063828945159912109375f,-0.558299983014165945860440842807292938232421875f,-0.3756000121548772607837918258155696094036102294921875f,-0.2981000181712214480711509168031625449657440185546875f,1.0102000329971314585009167785756289958953857421875f,0.64800004062056615339315612800419330596923828125f,0.415399998790025648531809565611183643341064453125f,0.016500009916722901959218461342970840632915496826171875f,-0.147800004829466369304924455718719400465488433837890625f,-0.136600012166053164097689887057640589773654937744140625f,-0.484600009331107195453824942887877114117145538330078125f,-0.56870001637488609702586472849361598491668701171875f,-0.054200003761053050510554385255090892314910888671875f,0.04339999873042099665099158301018178462982177734375f,-0.58279999927878378684908966533839702606201171875f,-0.827399997061491365712981860269792377948760986328125f,0.302600005322694709519737443770281970500946044921875f,-0.6696999700993295778772562698577530682086944580078125f,-0.19920000659227365957804067875258624553680419921875f,0.49469998954534488699863459260086528956890106201171875f,-0.65140002504587191101137477744487114250659942626953125f,-0.704700036595762302482626182609237730503082275390625f,-0.3632000150933862858693146336008794605731964111328125f,0.2093000020615749523500426221289671957492828369140625f,0.49150000943243465911791645339690148830413818359375f,0.0135999964863062761111223153420723974704742431640625f,0.4304000060170893871003272579400800168514251708984375f,0.31700000065565081541762992856092751026153564453125f,1.4492000253677368437621453267638571560382843017578125f,0.12959999465346339064808489638380706310272216796875f,0.155800029803812467577017741859890520572662353515625f,0.40599997469782866943432964035309851169586181640625f,-1.1733999972030513259113604362937621772289276123046875f,0.23539999446868886678885246510617434978485107421875f,-1.0659000510707505871010880582616664469242095947265625f,-0.48440000077187972404857418950996361672878265380859375f,-0.8695000375732779840376451829797588288784027099609375f,-0.68919998647570590577515758923254907131195068359375f,-0.0683999887093900138523849818739108741283416748046875f,-0.48979998458623885682072796043939888477325439453125f,-0.5518000216752294573296921953442506492137908935546875f,-0.094800001424550917050737552926875650882720947265625f,-0.78920007174760165025162450547213666141033172607421875f,-0.740399979418516096529856440611183643341064453125f,-0.19039999435693033547067898325622081756591796875f,-0.27360000103116011160864218254573643207550048828125f,-0.9632000226929784414409141390933655202388763427734375f,-0.03120000680088974576165128382854163646697998046875f,-1.0760000002160665655281945873866789042949676513671875f,0.0599999891221525416540316655300557613372802734375f,0.17930002953559220912893579225055873394012451171875f,0.4176000045776362412652815692126750946044921875f,0.338500006586313073597693801275454461574554443359375f,0.227000020101666422078778850845992565155029296875f,0.24959998437166230900174923590384423732757568359375f,0.090099999436736133429093342783744446933269500732421875f,-0.412099993945658216176752830506302416324615478515625f,-0.26839999746382237066910647627082653343677520751953125f,-0.5607000034704803237417536365683190524578094482421875f,0.23320000313520450330173616748652420938014984130859375f,-0.3142000153839585863124739262275397777557373046875f,0.133899998447299051207437514676712453365325927734375f,0.1911000086307523826434362490545026957988739013671875f,-0.19960000396668908795305696912691928446292877197265625f,0.3265999940425157177514847717247903347015380859375f,0.6392999970883133098453754428192041814327239990234375f,0.649199969190359382764654583297669887542724609375f,0.7073000012755394916297291274531744420528411865234375f,0.39900000894814713348068835330195724964141845703125f,-0.0584000005185605630675382826666464097797870635986328125f,-0.47169999297633768975401835632510483264923095703125f,-0.0388999975904822381522762952954508364200592041015625f,0.4220999958455562417469764113775454461574554443359375f,-0.00800000388920285576688229411956854164600372314453125f,-0.83360001384615856068194261752068996429443359375f,-0.93230001446306698387189726418000645935535430908203125f,-0.722999989882111560035582442651502788066864013671875f,-0.2575000122189521345461571399937383830547332763671875f,0.68219997032284762639164910069666802883148193359375f,-0.357700029048323653313445902313105762004852294921875f,-1.13440000993609402257789042778313159942626953125f,-0.90710001007616458679194693104363977909088134765625f,-0.79880003082454198448658644338138401508331298828125f,-0.4825999793946744720329888878040947020053863525390625f,-0.074299989786744180531030679048853926360607147216796875f,0.1537999965518712652823296593851409852504730224609375f,-0.956400042480230805352903189486823976039886474609375f,-0.27179999887645223122234483525971882045269012451171875f,-1.350500003330409537483092208276502788066864013671875f,0.1013999906241895843095335294492542743682861328125f,-0.094499988622963382312036628718487918376922607421875f,0.190199982631206854222227775608189404010772705078125f,0.2096999776244163893323957381653599441051483154296875f,-0.69219998582005481946310965213342569768428802490234375f,-0.04320000019222514087147146710776723921298980712890625f,-0.28979998801350603798709926195442676544189453125f,-0.37299999539554118399564686114899814128875732421875f,0.3151999892622232568584195178118534386157989501953125f,-0.49230001328960060202888371350127272307872772216796875f,0.01719999785274273218504959004349075257778167724609375f,0.32680000809654596860553965598228387534618377685546875f,-0.039799999785423256781768941436894237995147705078125f,-0.13589998151957960459412788623012602329254150390625f,0.7465999944150443212720347219146788120269775390625f,-0.05160001956075443985128003987483680248260498046875f,-0.156999989330768752182621028623543679714202880859375f,0.54980000108927473689846010529436171054840087890625f,-0.83259997045397771131547415279783308506011962890625f,0.9008000360622998581305864718160592019557952880859375f,-0.14900000831484749852506865863688290119171142578125f,0.9972000178948052795391276958980597555637359619140625f,-0.401500009581446770656754097217344678938388824462890625f,0.252900007209181654221907820101478137075901031494140625f,0.109299998876452253426805327762849628925323486328125f,0.42860000952482213421035339706577360630035400390625f,-0.636300016295910264574331449694000184535980224609375f,-0.539499987840652561743581827613525092601776123046875f,-0.2044000023365020179966222713119350373744964599609375f,0.844999983459711057065533168497495353221893310546875f,0.353900015583634708349336506216786801815032958984375f,-0.050299996224045795401025316095910966396331787109375f,0.0999000016063451568015807424671947956085205078125f,0.60619998713135725409983933786861598491668701171875f,0.17969998134970666114895720966160297393798828125f,0.279199980813265025147984488285146653652191162109375f,-0.8126000058442353068954844275140203535556793212890625f,-1.08709998109340677530099128489382565021514892578125f,-0.821599989944696407206947696977294981479644775390625f,0.10470000176429739013883590814657509326934814453125f,0.0574000065982342988490927382372319698333740234375f,0.7462000082910058562646327118272893130779266357421875f,0.23200000631809236661950990310288034379482269287109375f,-1.125700010883807777872789301909506320953369140625f,-0.87910000290870637940088272443972527980804443359375f,-0.1117999920964241056253740680404007434844970703125f,-0.691700017619133245005969001795165240764617919921875f,0.115499994128942518045732867904007434844970703125f,-0.579199989679455740088087623007595539093017578125f,-0.5193999680638317073544385493732988834381103515625f,-0.1003999657094478692442862666212022304534912109375f,0.5770000018104910477489966069697402417659759521484375f,0.01519999768137936246858998856623657047748565673828125f,0.98590005360394783462396617323975078761577606201171875f,0.25939999771714195020422266679815948009490966796875f,1.2701000254020093915841016496415250003337860107421875f,0.115399987912178314530819989158771932125091552734375f,0.395099974924325803016245117760263383388519287109375f,-0.7669999859035012690355870290659368038177490234375f,0.189899973033368585362268277094699442386627197265625f,-0.43799997624754938296831596744596026837825775146484375f,0.819499995104968181891536005423404276371002197265625f,-0.29899998953938489165693681570701301097869873046875f,0.879400018988549359022499629645608365535736083984375f,0.02279999197721505055369561887346208095550537109375f,1.366300008398294352218726999126374721527099609375f,0.245399981653690701932646334171295166015625f,1.647600003460049311598822896485216915607452392578125f,0.262999976396561141456231780466623604297637939453125f,0.34509999668002178108139332834980450570583343505859375f,-0.264799993824958779242706441436894237995147705078125f,-0.7455000094026329460206170551828108727931976318359375f,0.71319998689293839788660989142954349517822265625f,0.916500000230968225878314115107059478759765625f,0.043000012010335542100847305846400558948516845703125f,2.0515000211670990637458089622668921947479248046875f,-0.125400003111362234875514332088641822338104248046875f,0.45549998295307181361835091593093238770961761474609375f,-0.5167999897122381725012019160203635692596435546875f,-0.411300007429719283180702404933981597423553466796875f,0.18919999265968800994386356251197867095470428466796875f,0.941299973827600666709258803166449069976806640625f,0.8588999892085791287854590336792171001434326171875f,0.53279999391436572153679662733338773250579833984375f,0.510800009873509264224367143469862639904022216796875f,-0.8038999801933766065076270024292171001434326171875f,-0.532500008419156234396041327272541821002960205078125f,0.3994000012934206811365811518044210970401763916015625f,0.24100000174343583214664477054611779749393463134765625f,0.207599998579919287067241384647786617279052734375f,-0.2683999907955529895531299189315177500247955322265625f,-0.61949997040629423139534992515109479427337646484375f,-0.013499989524483613223537759040482342243194580078125f,0.825199981737136933901410884573124349117279052734375f,0.1367999804735184898873967540566809475421905517578125f,0.25749999374151233411112116300500929355621337890625f,0.4548999981790780022805620319559238851070404052734375f,0.56449999924004068407867862333660013973712921142578125f,0.340499999046325607654495115639292635023593902587890625f,-0.0608000032424926839524914612411521375179290771484375f,0.13050001491606255665800517817842774093151092529296875f,-0.442999997109174703524558935896493494510650634765625f,0.21660000752806662926985836747917346656322479248046875f,0.5282000008627771858726873688283376395702362060546875f,0.0953999978214502275530861652441672049462795257568359375f,0.0682999965369701644846145427436567842960357666015625f,0.42089999053478266421990383605589158833026885986328125f,1.038599966162443433148609983618371188640594482421875f,0.6301999806195499598260312268394045531749725341796875f,0.498000014916062383463213336654007434844970703125f,-0.25639998913407335834335754043422639369964599609375f,-1.0573000115945936361327994745806790888309478759765625f,-0.1643999978661536687241095933131873607635498046875f,-1.0202999734997746372755500487983226776123046875f,0.8783999686300756337686834740452468395233154296875f,0.244900004102289958751725862384773790836334228515625f,-0.1903999992370604132929656771011650562286376953125f,-0.160200010608136711454108080943115055561065673828125f,-0.292399995124339950880454352954984642565250396728515625f,0.622100002215802305727265775203704833984375f,0.049199975150824304392926933360286056995391845703125f,0.67470001871436868778886264408356510102748870849609375f,0.1107999910235406559877446852624416351318359375f,0.1335000024884942515512875615968368947505950927734375f,-0.5315999807059765824845953829935751855373382568359375f,0.129000021241605367094962275587022304534912109375f,-0.1463999985218047417134812349104322493076324462890625f,-0.6319000198826196790236053857370279729366302490234375f,-0.141000003606080781537457369267940521240234375f,-0.91540002721399071905494793099933303892612457275390625f,0.1015999981403350460595902404747903347015380859375f,-0.208000005938112852987842416041530668735504150390625f,0.04679999544918544973626239880104549229145050048828125f,0.3568999952957032650857627231744118034839630126953125f,0.0887999955832958765000739731476642191410064697265625f,0.144699990327656069499795421506860293447971343994140625f,-0.0735999910473825735834907391108572483062744140625f,0.1959000370264056822833254045690409839153289794921875f,0.376099977114796590882406235323287546634674072265625f,1.1108999854683876407079878845252096652984619140625f,1.3248999903559681712295059696771204471588134765625f,0.668599999541044098805286921560764312744140625f,0.6392000026807187484934047461138106882572174072265625f,-0.04029999583289034514876902903779409825801849365234375f,0.270900011601299228747308234233059920370578765869140625f,0.27740000279843801678225645446218550205230712890625f,-0.219100017884373610144166377722285687923431396484375f,-0.5493000260710712012723888619802892208099365234375f,-0.11130001435875858106783198309130966663360595703125f,-0.071700021344422992797262850217521190643310546875f,0.285399998268485166619257142883725464344024658203125f,0.4139000055253505561125848544179461896419525146484375f,-0.34810002422481811645837979085627011954784393310546875f,-0.12480000129416579657259944724501110613346099853515625f,0.188699996463209285213480370657634921371936798095703125f,0.80310000444352602766429072289611212909221649169921875f,0.74320000939369190717798119294457137584686279296875f,0.579900008237361763718809015699662268161773681640625f,0.8054999969601628340143406603601761162281036376953125f,1.337700002300739043903377023525536060333251953125f,0.713499992579221586908033714280463755130767822265625f,-0.5668000183224675136983705669990740716457366943359375f,-0.43169998967945544254831702346564270555973052978515625f,-1.320399989476799706977772075333632528781890869140625f,-0.7090999923288821005229465299635194242000579833984375f,-0.06039998984932903880462617962621152400970458984375f,-0.800300014850497287710595628595910966396331787109375f,-1.011199989180266101129745948128402233123779296875f,-0.4459999785721304732533099013380706310272216796875f,-1.481499986372888155727878256584517657756805419921875f,-0.016800006997585104073777984012849628925323486328125f,-0.92569999857172380330894156941212713718414306640625f,0.39199997562170063591935331714921630918979644775390625f,-0.765700015261023825274833143339492380619049072265625f,0.1998000015735623691170985694043338298797607421875f,-0.71030003276914388532503608075785450637340545654296875f,-0.5635999745070936484125923016108572483062744140625f,-0.173799992193281571672969221253879368305206298828125f,-0.5565999698281292040746848215349018573760986328125f,0.812400055921078223519771199789829552173614501953125f,0.248800009590386839164466437068767845630645751953125f,0.884300028548389516203087623580358922481536865234375f,0.0138000002026557877510271055143675766885280609130859375f,-0.22230000375285730029872865998186171054840087890625f,-0.6185999864280222748647020125645212829113006591796875f,0.283200036737323301849755807779729366302490234375f,-0.02719999535679828994716444867663085460662841796875f,-0.377799983297288033412542063160799443721771240234375f,-0.2675999886333944033367515658028423786163330078125f,-1.711100037054718026041655321023426949977874755859375f,-0.205799993258714675192777576739899814128875732421875f,-0.1185999910846351479420945906895212829113006591796875f,0.6857999871492383991977703772136010229587554931640625f,0.66940001332610865603101046872325241565704345703125f,0.638599990600347222624577625538222491741180419921875f,1.242500012181699364788300954387523233890533447265625f,0.107300015431642581376081579946912825107574462890625f,0.05500001385807973264263637247495353221893310546875f,0.105800009500980340959586101234890520572662353515625f,0.292399995422363279118371792719699442386627197265625f,0.5273000023186209528347490049782209098339080810546875f,0.76839997950792326975033574854023754596710205078125f,-0.170200021970271730964441303513012826442718505859375f,-0.45860001652836768215593110653571784496307373046875f,0.49329999825060344420535329845733940601348876953125f,-0.22279999138116846069124221685342490673065185546875f,-0.634899980968236743450461290194652974605560302734375f,-0.793799972933530995078399428166449069976806640625f,-0.013599991866946492535817014868371188640594482421875f,1.020599993044137843156704548164270818233489990234375f,0.75169999832212919255169936150196008384227752685546875f,0.5785999956294893831199033229495398700237274169921875f,0.42229999922662997402511564359883777797222137451171875f,0.39019998618885887164964287876500748097896575927734375f,0.347899995306879150636092390413978137075901031494140625f,0.00260000413060179624835654976777732372283935546875f,-0.413599975699186384048289255588315427303314208984375f,-0.647999971702695187758536121691577136516571044921875f,-0.9661999849706888188194398026098497211933135986328125f,-0.366299989622831390079227276146411895751953125f,0.10110000527799112290949778980575501918792724609375f,0.40720000012516965881559372064657509326934814453125f,-0.206999984636902834012062157853506505489349365234375f,-0.38439998803138752947461398434825241565704345703125f,0.215400027698278595522651812643744051456451416015625f,-0.3041000571116809769733890789211727678775787353515625f,0.5299999880790708939315436509787105023860931396484375f,-0.023600041413307959459189078188501298427581787109375f,0.36299998429417623668769010691903531551361083984375f,-0.02800002501159948309350511408410966396331787109375f,0.58799998414516441158639281638897955417633056640625f,-0.8876000451087950882111954342690296471118927001953125f,-0.57779996917843856607532870839349925518035888671875f,-0.9827999610200526259262687744922004640102386474609375f,0.58659996818900150827857942203991115093231201171875f,-0.0778000285968178939555173201370052993297576904296875f,-0.0263999874949458757100728689692914485931396484375f,0.514999991357325992424875948927365243434906005859375f,-0.33939999592900260694250391679815948009490966796875f,-0.316200009781122393093255595886148512363433837890625f,-0.22060000161230541859680442939861677587032318115234375f,-0.604200019276886823860195363522507250308990478515625f,-0.64819996975660354454618072850280441343784332275390625f,-0.412799998254329114644178844173438847064971923828125f,-0.5155999863386153680977486146730370819568634033203125f,1.220599989393353013156229280866682529449462890625f,0.09359999194145218748275283360271714627742767333984375f,0.8664999890178446406707735150121152400970458984375f,-0.56079997627139110960570178576745092868804931640625f,0.1655000072792172094438001295202411711215972900390625f,-0.569999978244304816854537421022541821002960205078125f,0.272799938780068362831343620200641453266143798828125f,-0.76639998362064343240263042389415204524993896484375f,0.5968999995052810536577680977643467485904693603515625f,-0.282400027385353968867320872959680855274200439453125f,0.378799971294403459154409574694000184535980224609375f,-0.2304000245690345227700390751124359667301177978515625f,0.36020000781416872115414662403054535388946533203125f,0.352800012466311319059286688570864498615264892578125f,-0.25899999892711633719954988919198513031005859375f,-0.5158000127792359279510492342524230480194091796875f,0.3474000060021875668780921841971576213836669921875f,-0.481500005260109720239825037424452602863311767578125f,-0.4637000181257722175587332458235323429107666015625f,-0.25959999025762048319165842258371412754058837890625f,-0.31719999639987950246222681016661226749420166015625f,0.079000019192695614123067571199499070644378662109375f,-0.19449998664855971952647450962103903293609619140625f,-0.216999983519315708946351151098497211933135986328125f,-0.70269999201893806883845172706060111522674560546875f,-0.345499993905425295537270358181558549404144287109375f,0.01460000761747348718699868186376988887786865234375f,-0.25849998557567577250893009477294981479644775390625f,-0.328599992090463643990005948580801486968994140625f,0.315499988317489599154441748396493494510650634765625f,1.1838999718487264178889972754404880106449127197265625f,0.273500004500150717401396605055197142064571380615234375f,-0.927599967734515917783255645190365612506866455078125f,0.04700001570582357235394965755403973162174224853515625f,0.430500007316469979201656315126456320285797119140625f,-0.0159999992847443817112207398167811334133148193359375f,-0.8402999903380869994862223393283784389495849609375f,-0.214900023660063776986817174474708735942840576171875f,0.266600039342046013501885681762360036373138427734375f,0.43839999470710733220357724349014461040496826171875f,0.393399986660480482925095202517695724964141845703125f,-0.68199998009204865212495860760100185871124267578125f,0.2440000261217354360354647724307142198085784912109375f,0.57799997758865373498338158242404460906982421875f,1.3422000181555748810069417231716215610504150390625f,-0.05900000831484764063361581065692007541656494140625f,0.6214999740794298066504097732831723988056182861328125f,-0.2637999924540519458560083876363933086395263671875f,-0.1691000413537029345434348215349018573760986328125f,-0.415599987828731531891435224679298698902130126953125f,-0.67990001322925086579829212496406398713588714599609375f,-0.18459999629259105091705350787378847599029541015625f,-0.6806000337988138415568073469330556690692901611328125f,-0.39279997901320484743337146937847137451171875f,1.151900023682415064030237772385589778423309326171875f,0.05379998455643697496952881920151412487030029296875f,0.6346000083252791856835983708151616156101226806640625f,0.5633999936640259331710467449738644063472747802734375f,-0.4131000291422015635589559678919613361358642578125f,-0.265199994403123806563371545053087174892425537109375f,-0.721500003769993458746512260404415428638458251953125f,0.1435999964863061695297119513270445168018341064453125f,-0.3864999889060853899280800760607235133647918701171875f,0.132199989694357089575760255684144794940948486328125f,-0.345199970784782816934921356732957065105438232421875f,0.59799998578429214290963500388897955417633056640625f,-0.0729999895468351756022684639901854097843170166015625f,0.0282999960899352753784796732361428439617156982421875f,0.377999981611967239558680375921539962291717529296875f,0.42759999071955701044345232730847783386707305908203125f,0.427799988680332976542786127538420259952545166015625f,0.140499987237155732344717762316577136516571044921875f,0.319499978154897945614720811136066913604736328125f,0.0291999913930893750801942587713710963726043701171875f,0.173000001467764341622768142769928090274333953857421875f,0.30390001364648322468298147214227356016635894775390625f,0.465500028252601172340519042336381971836090087890625f,0.16920000331401807702746964423567987978458404541015625f,0.309899989946186560363372564097517170011997222900390625f,1.0432000287652016812245392429758794605731964111328125f,0.5351000170946118572601335472427308559417724609375f,0.15140002094805193877391502610407769680023193359375f,-0.28769995878934917499236689764074981212615966796875f,0.250000022947788114180411866982467472553253173828125f,0.3590999871134759047208717674948275089263916015625f,-0.0512000014364719646664525498636066913604736328125f,-0.1226999890387059366503308410756289958953857421875f,-1.1028000038236378532729986545746214687824249267578125f,-0.90610000311732274980158763355575501918792724609375f,-0.6752999951064584394089251873083412647247314453125f,-0.4099000028729438138697105387109331786632537841796875f,0.75340001907050624385675519079086370766162872314453125f,0.7142999968647956166023504920303821563720703125f,0.20099999704957005608463305179611779749393463134765625f,-0.81860000520348530361758321305387653410434722900390625f,-0.5920000033378600701183813725947402417659759521484375f,0.932400022579729093052947064279578626155853271484375f,-0.10040000221729261653536013909615576267242431640625f,-0.03879998416528096516486812106450088322162628173828125f,0.31059998764246710134528939306619577109813690185546875f,0.14629998401626942428066513457451947033405303955078125f,-0.12659998413324391464129803352989256381988525390625f,0.61090000256746979090394233935512602329254150390625f,0.06059999893009659910347863842616789042949676513671875f,0.225500007912516620134368849903694353997707366943359375f,0.31619998131990456613493734039366245269775390625f,-0.6658000403091313135206519291386939585208892822265625f,-0.49979998972713934879408270717249251902103424072265625f,-1.03300000422447890713328888523392379283905029296875f,0.39499999612569780538251507095992565155029296875f,-1.3694000651448969829715451851370744407176971435546875f,-0.3550000026822086596212102449499070644378662109375f,-0.21460001052319999104156522662378847599029541015625f,-0.3661999878764152782650853623636066913604736328125f,1.181200030717253657286391899106092751026153564453125f,0.343399985766410953402782979537732899188995361328125f,1.27979997825324520732692690216936171054840087890625f,-0.14679999313950542472184679354541003704071044921875f,0.206399973897635646835624356754124164581298828125f,-0.388599987471103691660800905083306133747100830078125f,-1.13880001383721829455453189439140260219573974609375f,0.056600003504752915972630944452248513698577880859375f,-1.197700036722422201052040691138245165348052978515625f,0.0323999995946884222774997397209517657756805419921875f,0.3879000099658964639814939800999127328395843505859375f,-0.354900001904368433969239049474708735942840576171875f,-0.7430999917030334245282574556767940521240234375f,-1.042999998673796557824289266136474907398223876953125f,-0.61019999082684517333063922706060111522674560546875f,-0.197499976083636230583806536742486059665679931640625f,-0.454999983683228581554658376262523233890533447265625f,0.102000000432133308692073114798404276371002197265625f,0.86620001529455148414626819430850446224212646484375f,0.3840000028759240624509629924432374536991119384765625f,-0.464100006932020026084728669957257807254791259765625f,0.0217000122547148510676606747438199818134307861328125f,-0.65889997020959878426538125495426356792449951171875f,-0.5425000078976154327392578125f,0.2270999749839308634591361624188721179962158203125f,0.229499991938471925578824084368534386157989501953125f,0.176299996253848068050729125388897955417633056640625f,0.44499999769031994834023180374060757458209991455078125f,-0.0609999979063868824624705666792578995227813720703125f,-0.02420000372380026743002190414699725806713104248046875f,0.4171000102996824665524400188587605953216552734375f,-0.332200007724761992022877166164107620716094970703125f,0.6121999992311000227118711336515843868255615234375f,0.4002999907106159493963559725671075284481048583984375f,0.97219998947083940965541160039720125496387481689453125f,0.6147999965816735024048966806731186807155609130859375f,0.35229999844431869604477469692938029766082763671875f,0.38740000205338009919842079398222267627716064453125f,0.3885999923512339915276925239595584571361541748046875f,0.56710000452548248173201272948062978684902191162109375f,1.27780002956539373570876705343835055828094482421875f,0.2797999845862391143924696734757162630558013916015625f,1.234300015044212717185700967092998325824737548828125f,0.120199998575448940840715295053087174892425537109375f,0.10239999732226123807521389608155004680156707763671875f,0.93159997727870924588700063395663164556026458740234375f,-0.51040000780522820633677838486619293689727783203125f,0.05679998166561173178479293710552155971527099609375f,-1.0627000094532963458959784475155174732208251953125f,0.1355999993026255623362885671667754650115966796875f,-0.997999984033405507233283060486428439617156982421875f,0.68239997202754043570394060225225985050201416015625f,0.6243999901548027420261632869369350373744964599609375f,-0.2131999883830548725427433964796364307403564453125f,-0.0790000000074508879066570443683303892612457275390625f,0.18459998586177850388168053541448898613452911376953125f,-1.141299991820752257609683510963805019855499267578125f,0.292799981397390662873903011131915263831615447998046875f,0.88190004856735537686063253204338252544403076171875f,0.30979998756647120217166957445442676544189453125f,1.786100015559792808517158846370875835418701171875f,-0.180199984270334567781901569105684757232666015625f,0.179599977852403558387095472426153719425201416015625f,-0.6435999873220918932048562055570073425769805908203125f,-0.014099992626905422099525821977294981479644775390625f,-0.496599992775916820875181656447239220142364501953125f,-0.135499991707503486537689241231419146060943603515625f,-0.293799986046552830121214583414257504045963287353515625f,-0.43069998994767677569228681022650562226772308349609375f,0.04949997964501395841807607212103903293609619140625f,-0.1688999971807003674939551274292171001434326171875f,0.49019999343454845330825264682061970233917236328125f,0.7754999698400499852368739084340631961822509765625f,0.59169998595416561926185750053264200687408447265625f,0.2452999923124909997795839444734156131744384765625f,0.7894000096008182421059018452069722115993499755859375f,0.33039999029636391014719265513122081756591796875f,-0.279300013256072787726225215010344982147216796875f,-0.773300017696618713358702734694816172122955322265625f,-0.0530999963968989963092326433979906141757965087890625f,0.051700002866983396643263404257595539093017578125f,-0.358800013539195106204715557396411895751953125f,0.041399994051456445731673738919198513031005859375f,0.259100017362833001666189147726981900632381439208984375f,-0.76669998397976169002276947139762341976165771484375f,-0.407999992191791616136242737411521375179290771484375f,0.015600005263090022111782673164270818233489990234375f,-0.2820999956220386906124986126087605953216552734375f,0.08719999513328080542606812741723842918872833251953125f,-0.446000001147389302236234698284533806145191192626953125f,-0.586699994261562718378399949870072305202484130859375f,-0.4436999959230423229428197373636066913604736328125f,0.46510001769065834054117658524774014949798583984375f,-0.0568999977916480048634184640832245349884033203125f,-0.1442000020846723185030668901163153350353240966796875f,-0.224300018136203427321362369184498675167560577392578125f,0.35279999320656063588330653146840631961822509765625f,-0.1845000105649231836313362009605043567717075347900390625f,-0.874900026826560317516623399569652974605560302734375f,-0.43899999028444280924077247618697583675384521484375f,-0.5247999979972839401654027824406512081623077392578125f,-0.62799998131394385580961170489899814128875732421875f,-0.42239998387396304746488340242649428546428680419921875f,-0.429799981009960363476096745216636918485164642333984375f,-0.63470000709146301431928804959170520305633544921875f,0.548599987322091919850208796560764312744140625f,-0.856500058420003096415484833414666354656219482421875f,-0.13999999597668644213399602449499070644378662109375f,0.1860000208541752186874873586930334568023681640625f,0.352199988651275663897877166164107620716094970703125f,-0.12170002916753308586095272403326816856861114501953125f,-0.74979998264908775951198549591936171054840087890625f,-0.783899989655613804728773175156675279140472412109375f,0.2641999850600960098034875045414082705974578857421875f,0.5308999988421796434323596258764155209064483642578125f,0.076199988025427156657087834901176393032073974609375f,1.07359999868422750779473062721081078052520751953125f,0.1827999930202961476055634193471632897853851318359375f,0.6577000030085444759464508024393580853939056396484375f,0.32439999302327626740094501656130887567996978759765625f,1.50540003626644658396571685443632304668426513671875f,0.2015999997794626352742852759547531604766845703125f,0.48250001035630718515534454127191565930843353271484375f,0.5446000009119504881027751252986490726470947265625f,0.32039998929053503662345292468671686947345733642578125f,-0.2689999933391808983884629924432374536991119384765625f,1.157000009540468443702820877660997211933135986328125f,-0.2485000014454126215923679410479962825775146484375f,-0.575399987167120041675616448628716170787811279296875f,-0.7480000073164700946648508761427365243434906005859375f,-0.328800012123584561862799091613851487636566162109375f,0.6160000320672989193582225198042578995227813720703125f,-0.01519996638894127727326122112572193145751953125f,0.276500027686357341405454235427896492183208465576171875f,-0.45429999852254976389076546183787286281585693359375f,-0.82459999662786753749088575204950757324695587158203125f,-0.911799961251020807395661904592998325824737548828125f,-0.0993999898940323145524189385469071567058563232421875f,0.76379996697306662412074729218147695064544677734375f,0.684499980762601101247355472878552973270416259765625f,0.85659998398423198295859037898480892181396484375f,0.936299995285272412814947529113851487636566162109375f,0.4105000010207293659192373525002039968967437744140625f,0.1050000024214388094634387016412802040576934814453125f,-0.34780000084340567667595678358338773250579833984375f,0.31100000174343589431913414955488406121730804443359375f,0.36149999140203004088789384695701301097869873046875f,-0.42770001407265656467870940105058252811431884765625f,-0.51299999979138366512643187888897955417633056640625f,-0.72939998780786996501745989007758907973766326904296875f,-0.568199973370135058559071694617159664630889892578125f,-0.55129999036788934763109182313201017677783966064453125f,0.0517999985598027734567949664779007434844970703125f,0.087499999441206455230712890625f,0.3550999986171721189975869492627680301666259765625f,-0.0098999821603300564021310492535121738910675048828125f,0.855800045524537811303389389649964869022369384765625f,0.210799993109703098781437802244909107685089111328125f,0.21139998339712617880792322466732002794742584228515625f,-0.58439997998476034268833245732821524143218994140625f,-1.74860002658665170116591980331577360630035400390625f,-0.04419999780058869731647064327262341976165771484375f,-0.67290001505091812106229554046876728534698486328125f,0.5129999931976196858585126392426900565624237060546875f,0.66250000948086362750899525053682737052440643310546875f,0.6147999664068226621793655795045197010040283203125f,-0.1790000296980143179581546064582653343677520751953125f,0.02140000492930393960477886139415204524993896484375f,-0.5330000229254368004916386780678294599056243896484375f,-0.4801999872505664512800649390555918216705322265625f,-1.031900001442432657228209791355766355991363525390625f,-0.019800021541117984735791424100170843303203582763671875f,-0.539100005479156596521761457552202045917510986328125f,-0.24439999712109550245031641679815948009490966796875f,-0.13269997540414324443958093979745171964168548583984375f,0.3859999837875367489914424368180334568023681640625f,0.50749999735504391917828570512938313186168670654296875f,-0.10359999909400929851699402206577360630035400390625f,1.3742000128135083425462426021113060414791107177734375f,0.40339997615516220985654172181966714560985565185546875f,0.62419998954907074395492827534326352179050445556640625f,-0.0870000002458690691042875187122263014316558837890625f,-0.32790002756789349280808210096438415348529815673828125f,-0.635599986785650106213552135159261524677276611328125f,0.10280001369565738844613633773406036198139190673828125f,0.402699983152746909809138742275536060333251953125f,0.60519998620748527429213936557061970233917236328125f,-0.0197999902486802881185212754644453525543212890625f,-0.98809996353983908790041823522187769412994384765625f,-0.124799976465106154943640603960375301539897918701171875f,-0.08469999934285876719286534353159368038177490234375f,0.037899981057643916670940598123706877231597900390625f,-0.54370002676844553235468993079848587512969970703125f,0.782299988535046342263967744656838476657867431640625f,1.2594000176846975591615773737430572509765625f,0.4623000059694053476277986192144453525543212890625f,-0.595800018441676737523948759189806878566741943359375f,0.211999997526407302217421602108515799045562744140625f,0.763300001901388025515871049719862639904022216796875f,-0.7340000096559526898687408902333118021488189697265625f,-1.48829999638795840155580663122236728668212890625f,-0.413299999666213668803038672194816172122955322265625f,0.70099998423457154927973533631302416324615478515625f,1.030799979326129101053766135009936988353729248046875f,0.74329996598959002795936612528748810291290283203125f,0.619199968445301340835840164800174534320831298828125f,0.625099977308511967066806391812860965728759765625f,0.489499990895390535428077782853506505489349365234375f,-0.733400004839896979547120281495153903961181640625f,0.06700001391768495295764296315610408782958984375f,0.33799999144673353868029153090901672840118408203125f,-0.1507000052958726854512860882095992565155029296875f,-0.331499992519617148190036459709517657756805419921875f,-0.0398999867051841317078242354909889400005340576171875f,0.0123000130847099597986016306094825267791748046875f,-0.718799975168705085337705895653925836086273193359375f,0.07850002528727084705195693459245376288890838623046875f,0.1197999956130981757951303734444081783294677734375f,-0.4890000060424208871978635215782560408115386962890625f,0.43319999024569977308374291169457137584686279296875f,-1.533400013110041815167505774297751486301422119140625f,0.089599996441602769436940434388816356658935546875f,-1.277499973513185782536538681597448885440826416015625f,-0.23300002363324079368567254277877509593963623046875f,-1.2424000166192643579421428512432612478733062744140625f,-0.3299999773502353406229303800500929355621337890625f,0.1981000143527984480584791526780463755130767822265625f,-0.32939998654127133903557478333823382854461669921875f,1.3683999981343737406547234058962203562259674072265625f,0.187999976545572877739687100984156131744384765625f,0.58330002987831885974401302519254386425018310546875f,0.089999992251396321307765902020037174224853515625f,-0.9038999799698590464913650066591799259185791015625f,0.50739998648166650951907286071218550205230712890625f,-1.1429999997913833365004165898426435887813568115234375f,-0.18279998556971577272634021937847137451171875f,-1.1341000030204657633703391184099018573760986328125f,0.63719998090267182533352752216160297393798828125f,-0.626400050824881038380453901481814682483673095703125f,0.501799991053342608893217402510344982147216796875f,-0.692700043614209004516624190728180110454559326171875f,-0.3403999940216539954462859896011650562286376953125f,-0.0425999645307657548443103223689831793308258056640625f,-0.0149000092059374811270799909834749996662139892578125f,0.4942000000357626987579351407475769519805908203125f,0.41259998751580706510821983101777732372283935546875f,0.706199978339672185256858938373625278472900390625f,0.6715999999284745225480719454935751855373382568359375f,-0.17849999362230306587662198580801486968994140625f,0.0901999941051006004499868140555918216705322265625f,0.644499997824430348458690787083469331264495849609375f,0.5522000262767077227721301824203692376613616943359375f,-0.23139998689889917926620910293422639369964599609375f,-0.307700017052888963320356197073124349117279052734375f,1.158699990260600998226436786353588104248046875f,1.183899981385469146744071622379124164581298828125f,0.987699985611438791011096327565610408782958984375f,0.550899999326467604277013379032723605632781982421875f,-0.53350000077486026128781304578296840190887451171875f,-0.55090001199245453022967922152020037174224853515625f,-0.4120000076591967541617123060859739780426025390625f,-0.0356999920338390008822671006782911717891693115234375f,-0.3196000028491019495646696668700315058231353759765625f,0.14460000463724165253864839542075060307979583740234375f,-0.2368000184714791345186313265003263950347900390625f,0.302599977532029384974521235562860965728759765625f,0.88159997012615232137022758251987397670745849609375f,0.267099999272823129814469211851246654987335205078125f,-0.914199966061115532056646770797669887542724609375f,-0.52879999118745335540126006890204735100269317626953125f,0.11969999770373107583765204253722913563251495361328125f,0.4459000129237769005641212061163969337940216064453125f,1.197500007338821337299350489047355949878692626953125f,-0.355999992132186804383309208787977695465087890625f,0.3904000096678730269417201270698569715023040771484375f,-0.14120000724792447499567060731351375579833984375f,-1.086099989408254362643901913543231785297393798828125f,0.4861999753594401507683642194024287164211273193359375f,-0.5408000109910962027726100131985731422901153564453125f,-0.588999985516071244973090870189480483531951904296875f,0.13709999718666054935312104134936816990375518798828125f,0.81299998012185081819325205287896096706390380859375f,0.5272999902114274828335283018532209098339080810546875f,-1.0015999666988850691495827049948275089263916015625f,-0.002199929903447195300714156473986804485321044921875f,-0.1111999963700771587582494248636066913604736328125f,-0.985600008578598529851433340809307992458343505859375f,0.55799999055266358283233785186894237995147705078125f,-0.5207000422880059264940655339160002768039703369140625f,-0.474199995118379291625387850217521190643310546875f,-0.274799997438490262169352718046866357326507568359375f,-0.02040000013113019150523541611619293689727783203125f,-1.120800031182170197752157037029974162578582763671875f,0.06160000767707796143213272443972527980804443359375f,-0.3929999971836803940306026561302132904529571533203125f,-0.68299996760487591274113583494909107685089111328125f,0.1362000430479646251313852189923636615276336669921875f,0.73199997323751464506358388462103903293609619140625f,0.01069998305588970577417740059900097548961639404296875f,-0.031599997097253862410326519238878972828388214111328125f,0.09480001029074192064882709019002504646778106689453125f,0.1413999948710202847479422416654415428638458251953125f,0.26779998862445364427031790910405106842517852783203125f,0.345200010049342920215309504783363081514835357666015625f,-0.2655999804154041132875363473431207239627838134765625f,-0.944599991561472673851085346541367471218109130859375f,-0.680099977010488743189853266812860965728759765625f,-0.3076999876230956942890770733356475830078125f,0.14880000452399233523692601011134684085845947265625f,0.687100022146105882114852647646330296993255615234375f,0.00989999706149102820518237422220408916473388671875f,0.145899997463822206356098831747658550739288330078125f,0.1902000106453893835123380995355546474456787109375f,-0.8190000135302544226334475752082653343677520751953125f,-0.39190000825971349485143946367315948009490966796875f,-0.233600000435113752672577902558259665966033935546875f,0.1604999990090727468583509107702411711215972900390625f,0.28880002907365553976859473550575785338878631591796875f,-0.27339996699094815113539880258031189441680908203125f,-0.003099967563152450367169876699335873126983642578125f,-0.0887999915599824163336961646564304828643798828125f,0.030500005900859772367539335391484200954437255859375f,0.7767999990999696624527359745115973055362701416015625f,0.75779999093413341615388389982399530708789825439453125f,0.99469998552203175989916417165659368038177490234375f,1.25660001699030399180401218472979962825775146484375f,0.95660001661777460668645289842970669269561767578125f,0.576900010383128769575478145270608365535736083984375f,0.95820001326799353336127751390449702739715576171875f,0.22370001872181877189404985983856022357940673828125f,-0.2255999949440357088548125830129720270633697509765625f,-0.5627999769270422003586418213672004640102386474609375f,-0.1046000005021696654949892035801894962787628173828125f,0.251199978339672502780643981168395839631557464599609375f,0.7338999899908902335710081388242542743682861328125f,-0.525599998408555535434061312116682529449462890625f,0.41069999952167268730818250332958996295928955078125f,-0.695399994319676917342576416558586061000823974609375f,-1.300500010780989867953394423238933086395263671875f,0.036399994498491405892082184436731040477752685546875f,0.29950002676993625616574945524916984140872955322265625f,0.055199987995624866243815631605684757232666015625f,1.6373000185981394505319030940881930291652679443359375f,-0.2577999942868947069740670485771261155605316162109375f,0.50740000391602524398848572673159651458263397216796875f,-0.320599989020824482910398955937125720083713531494140625f,-1.0645000416710974633360820007510483264923095703125f,-0.10499999955296512865743352449499070644378662109375f,-0.0268000425741081738095772379892878234386444091796875f,-0.05779999987483019907585912733338773250579833984375f,0.2643999687716360735834086881368421018123626708984375f,-0.6195999924182888918267053668387234210968017578125f,-0.19229998555481409017176019915495999157428741455078125f,-0.94699998098611803953872367856092751026153564453125f,-0.2766999697864049867490621181786991655826568603515625f,0.1222000010192392238650427316315472126007080078125f,-0.221000048123300540936497782240621745586395263671875f,-0.7409999845325945244667309452779591083526611328125f,0.0632999946370720323329805978573858737945556640625f,0.4492999944806097545324519160203635692596435546875f,0.27589998725652709055111699854023754596710205078125f,-0.234000011146068498391059620189480483531951904296875f,0.42270000960230813547013895004056394100189208984375f,0.11869999122917629374995840407791547477245330810546875f,0.2100999951899051865211731637828052043914794921875f,0.740400002217292740880338897113688290119171142578125f,0.6632000028371809463578756549395620822906494140625f,0.62559999378919595169890044417115859687328338623046875f,0.4572999868586660188185533115756697952747344970703125f,-0.11300001342594623299220302214962430298328399658203125f,0.465000003278255302774368828977458178997039794921875f,0.502800011795759171917552521335892379283905029296875f,-0.49260000442862494907103609875775873661041259765625f,-0.451300024491548601446311295148916542530059814453125f,0.027399988716840795177631662227213382720947265625f,0.456900001665950072915478585855453275144100189208984375f,0.45489998411610732542698087854660116136074066162109375f,0.5987999808311463834087362556601874530315399169921875f,1.049399966126680627809264478855766355991363525390625f,0.56239998029768489828938982100225985050201416015625f,0.45199998284876363729978265837416984140872955322265625f,-0.5397000144302845914268118576728738844394683837890625f,-0.25259997954368618167109161731787025928497314453125f,0.0552000118374824655376187365618534386157989501953125f,-0.13110001108944391301491805279511027038097381591796875f,-0.046000008448958629170277845332748256623744964599609375f,0.256400013273954119341624391381628811359405517578125f,-0.500999997757375137297231049160473048686981201171875f,0.72590003311485029602323493236326612532138824462890625f,0.2829999889135361712533267564140260219573974609375f,1.41720003648400361129233715473674237728118896484375f,0.1460000006258486138221996952779591083526611328125f,0.1304000073209403698371033897274173796176910400390625f,0.10459999435544020673916065788944251835346221923828125f,-1.18929999939799291297504169051535427570343017578125f,0.0855999946832657343520622816868126392364501953125f,-0.174499985940754331892321715713478624820709228515625f,0.27299999479949466074657493663835339248180389404296875f,0.41809998130947345629238043329678475856781005859375f,-0.6103999822497367944151847041212022304534912109375f,-0.6909999957829715544477267030742950737476348876953125f,0.51699998471140862221773204510100185871124267578125f,-0.3602000148549677760456688702106475830078125f,-0.764599991673230672262207008316181600093841552734375f,0.22690002902820738395206490167765878140926361083984375f,-0.042599998058378718557293041158118285238742828369140625f,0.0548999974898991549565607783733867108821868896484375f,-0.183999992072582330138175166212022304534912109375f,0.6480000162571666333377606861176900565624237060546875f,0.3217999909043312012357773710391484200954437255859375f,0.5381999904319645366967961308546364307403564453125f,-0.6035999889612195801191774080507457256317138671875f,0.49650004102289724272623061551712453365325927734375f,0.2710000047981735438185069142491556704044342041015625f,0.53270000818669738240629385472857393324375152587890625f,0.46479996879100848872212736750952899456024169921875f,0.8294999880455435903314764800597913563251495361328125f,0.6842999803245073398016984356218017637729644775390625f,0.746299954605103010862876544706523418426513671875f,0.11049997192621230368558826739899814128875732421875f,-0.05019999768137939355483467807061970233917236328125f,0.071399994945525957490417567896656692028045654296875f,0.525299998384713973109683138318359851837158203125f,0.5653000164523722670395500244922004640102386474609375f,-0.12390000245198595774098748734104447066783905029296875f,-0.7505000298358501797935105059877969324588775634765625f,-0.673600009673833710621693171560764312744140625f,-0.2618999889999624741676598205231130123138427734375f,-0.10829999817609792245320932124741375446319580078125f,0.242600015418231562680517754415632225573062896728515625f,-0.01339999826475979372020219670957885682582855224609375f,0.281700006983429229734383625327609479427337646484375f,0.087299999152123941570380338816903531551361083984375f,0.1573999987006187151195035767159424722194671630859375f,-0.163799998861551276974068969138897955417633056640625f,0.6771999955803156279188215194153599441051483154296875f,0.4885999789774420065668891766108572483062744140625f,0.4858999796569347662256177500239573419094085693359375f,0.15849998624622829712649263456114567816257476806640625f,0.4750000137090684138030383110162802040576934814453125f,-0.411899993246793716394904549815692007541656494140625f,0.19360001079142097335505923183518461883068084716796875f,0.423500004127621565430672490037977695465087890625f,0.0840000047758222567750863163382746279239654541015625f,-0.39930000444576119189576957069220952689647674560546875f,-0.8969000080175699718409987326595000922679901123046875f,0.347999994017183400529802383971400558948516845703125f,-0.60899999020993711429383665745262987911701202392578125f,-0.8617999803617590881543719660839997231960296630859375f,1.666799968180060265154907028772868216037750244140625f,-0.02039994857311189235815618303604423999786376953125f,-0.2926000318467616434503497657715342938899993896484375f,0.7011000518441203865904753911308944225311279296875f,-0.71580000190138814986084980773739516735076904296875f,0.6013000311970717604026503977365791797637939453125f,-0.950400015702843603548899409361183643341064453125f,1.168199999260901567055270788841880857944488525390625f,-1.17279999316930716446449878276325762271881103515625f,0.9824999847263093943894318726961500942707061767578125f,-0.734599991151690279167496555601246654987335205078125f,0.5554999933838846715161707834340631961822509765625f,-0.013199980187416304744374428992159664630889892578125f,-0.08650003558397401093316148035228252410888671875f,-0.330599999600648342124031842104159295558929443359375f,-0.0639000554442414880895739770494401454925537109375f,0.08140001997947710066227955394424498081207275390625f,0.733000005304813129214380751363933086395263671875f,-1.00300000761449314978790425811894237995147705078125f,-0.170299997639656197634394629858434200286865234375f,0.3017999912768605241808472783304750919342041015625f,0.4884000310659413202074574655853211879730224609375f,-1.40140001878738384988309917389415204524993896484375f,0.5750000482797634759890570421703159809112548828125f,0.4912000071734183848093380220234394073486328125f,0.0438999881654975609990287921391427516937255859375f,0.36250000454485498124768128036521375179290771484375f,-0.024100000748038308273635266232304275035858154296875f,-0.790400013393163902719606994651257991790771484375f,-0.13510000167191027475865894302842207252979278564453125f,-0.75310003867894448337239055035752244293689727783203125f,-0.341100010195374558907843720589880831539630889892578125f,-0.96730006400942869504433474503457546234130859375f,-0.154900010100007090585449986974708735942840576171875f,0.35259997946917931432864179441821761429309844970703125f,0.1773000011265277731098422009381465613842010498046875f,0.3554000035583972927355489446199499070644378662109375f,-0.1179000018820166584276876164949499070644378662109375f,0.27789997673630661978450007154606282711029052734375f,0.160499994017183678085558540260535664856433868408203125f,0.6433000241786237527463754304335452616214752197265625f,0.20740000577867034881052177297533489763736724853515625f,0.35870002069622319851305292104370892047882080078125f,-0.1682000016264617314742935150206903927028179168701171875f,0.015699993244558563265655948271160013973712921142578125f,0.161900001889467237692343815069762058556079864501953125f,-0.426000012323260346391862185555510222911834716796875f,-0.0919000056892633754301868975744582712650299072265625f,-0.6532000375568871586295927045284770429134368896484375f,-0.30470000105649230892623791078221984207630157470703125f,-0.2995000298619270306943462855997495353221893310546875f,0.0156999973982572260666756847058422863483428955078125f,-1.337800047484041243706087698228657245635986328125f,-0.4001000148594380245725687927915714681148529052734375f,-1.57800010432303050578184411278925836086273193359375f,0.23389999256134075267254956997931003570556640625f,-0.35699998061358950707244730438105762004852294921875f,-0.26840001210421338129208379541523754596710205078125f,0.3811999996483326214047337998636066913604736328125f,-0.9070000523999339758773885478149168193340301513671875f,0.27680001295805001237937403857358731329441070556640625f,-0.9769000477105380664255562805919907987117767333984375f,0.13060002932846526135790554690174758434295654296875f,-0.052299979817867825460098174517042934894561767578125f,0.04959997148215811169080780018703080713748931884765625f,0.72089999426007267402383149601519107818603515625f,-0.7141999860286711854229224627488292753696441650390625f,1.118099986264109002576105922344140708446502685546875f,-0.74419997485280031668253286625258624553680419921875f,0.4689000120446085162484450847841799259185791015625f,-0.3804000109344720659265703943674452602863311767578125f,-0.3929999981708822698323047006851993501186370849609375f,0.48139999665915966797768987817107699811458587646484375f,0.24000000966712831118599069668562151491641998291015625f,-0.38000000260770316895531095724436454474925994873046875f,-0.9776000181376929276666487567126750946044921875f,1.117000025019049047614316805265843868255615234375f,-1.20500004909932645347225843579508364200592041015625f,0.6881999967277050433978047294658608734607696533203125f,0.868799970921874109563987076398916542530059814453125f,-0.983399962893128698482314575812779366970062255859375f,-0.041500008165836987927832524292171001434326171875f,1.2790000095814466618548976839520037174224853515625f,-0.050400060331821094905535574071109294891357421875f,0.44990002321302913212974772250163368880748748779296875f,0.09940000442266468116958577638797578401863574981689453125f,0.791400051458180708863210384151898324489593505859375f,0.3159000127002598024006374544114805757999420166015625f,-0.0794999714493747067223239355371333658695220947265625f,-0.1167000020846724606116140421363525092601776123046875f,0.1232999770909544157149184684385545551776885986328125f,0.0552999964475631156091139928321354091167449951171875f,-0.019399994736910119996764478855766355991363525390625f,-0.119599997037649175002371748632867820560932159423828125f,-0.03879998276829699221934788511134684085845947265625f,0.160300001104176026434089408212457783520221710205078125f,0.4713000062376255794305279778200201690196990966796875f,0.15799999409168952180237965876585803925991058349609375f,-0.805300017607212925696558158961124718189239501953125f,-0.2441000044360757126327143851085565984249114990234375f,-0.808700024980307130562096062931232154369354248046875f,0.175899996681511350260507242637686431407928466796875f,0.79950002360343941365528053211164660751819610595703125f,0.17040000877380379051828640513122081756591796875f,-0.510000005513429499615085660479962825775146484375f,-0.2204000120520592798101233711349777877330780029296875f,-0.1880000196099285147965929354541003704071044921875f,0.0621000084742904245871386592625640332698822021484375f,-0.18409998197257504415347284520976245403289794921875f,0.09149999788403506695289024719386361539363861083984375f,-0.9706000019848346482831402681767940521240234375f,-0.38479999538958065130600516567938029766082763671875f,-1.8714000317513939108948761713691055774688720703125f,0.03360001216977848148559360197396017611026763916015625f,-0.2418000099778174760700721890316344797611236572265625f,0.724600069867075813334622580441646277904510498046875f,-0.061000018209219408760191072360612452030181884765625f,-0.04329996151924131453370137023739516735076904296875f,0.7431999789953234181183461259934119880199432373046875f,0.89509998431205683999678512918762862682342529296875f,-1.2497999854058023938563337651430629193782806396484375f,0.9666999967575069518943564617075026035308837890625f,-0.032600003981590131019174805260263383388519287109375f,1.015600006380676934014672951889224350452423095703125f,-0.644999992549419420839740269002504646778106689453125f,0.9159000192940229823079789639450609683990478515625f,-0.1131999933749436859642401032033376395702362060546875f,-0.5385999755501753583075696951709687709808349609375f,1.461199968206882804366841810406185686588287353515625f,-0.2779999668598183149015312665142118930816650390625f,0.194399974769354333403725831885822117328643798828125f,0.825300025951862803452740990906022489070892333984375f,-0.93960000672340393546022596638067625463008880615234375f,-1.12440002170801189862459068535827100276947021484375f,1.3606000156939028755687104421667754650115966796875f,-0.568000006198882800845240126363933086395263671875f,0.424199986922740635009176912717521190643310546875f,0.6746000332474704208607363398186862468719482421875f,0.11399998886883278004233943647705018520355224609375f,-0.066000000476837072937996708787977695465087890625f,0.18119998057484654196969131589867174625396728515625f,-1.82440001068115265070446184836328029632568359375f,0.5331999962061644282584893517196178436279296875f,0.288999999634921511670881955069489777088165283203125f,0.870100012996792404607049320475198328495025634765625f,0.045599998892843718323319990304298698902130126953125f,0.06710002229511768945258154417388141155242919921875f,-0.01159999832659952545554915559478104114532470703125f,0.46749999299645439432282501002191565930843353271484375f,0.091200001958012588687552124611102044582366943359375f,0.28980001051425940961081550995004363358020782470703125f,-0.028600000658631330452408292330801486968994140625f,-0.556599986442923633234158842242322862148284912109375f,-0.14580000067204235136841816711239516735076904296875f,0.0440999860703945589790464509860612452030181884765625f,0.324999998807907086728619105997495353221893310546875f,1.09460001156628106144808043609373271465301513671875f,-0.04219999829977749694620570153347216546535491943359375f,0.0961000117599966063863803356071002781391143798828125f,0.145400003334879901473186691873706877231597900390625f,0.30140000545084466576639670165604911744594573974609375f,-0.0977000057280064115872164620668627321720123291015625f,-0.5470000262856486994422766656498424708843231201171875f,-0.176800003719329856810560386293218471109867095947265625f,0.4014000053763389086469715039129368960857391357421875f,0.222800003749132169428293082091840915381908416748046875f,0.58010002312958242054463653403217904269695281982421875f,0.074599998676776857475800852625980041921138763427734375f,1.0413999971061940197358808291028253734111785888671875f,0.3465999971345066210659524585935287177562713623046875f,1.3323000365167854130987734606605954468250274658203125f,0.99830000622272496713094369624741375446319580078125f,-0.987799979832768570986445411108434200286865234375f,0.7448999772429463206435684696771204471588134765625f,-0.932399984022974859243504397454671561717987060546875f,-1.042999967157840046638739295303821563720703125f,1.23899997813999629414638548041693866252899169921875f,-0.153799992752075009860845966613851487636566162109375f,-0.47560000585913642101587583965738303959369659423828125f,0.555600002506375201249966266914270818233489990234375f,-0.38599999697506415241576860353234224021434783935546875f,-0.1368999524548655255529183705220930278301239013671875f,0.861799952831864946034556851373054087162017822265625f,0.43240002790689491263265153975225985050201416015625f,-0.8211999886214733646028207658673636615276336669921875f,1.0275000014901163325475863530300557613372802734375f,-0.40579999005496514286051024100743234157562255859375f,0.8588000155508519473102069241576828062534332275390625f,-0.68740000019073477233177982270717620849609375f,0.1838999909222120976437508943490684032440185546875f,-0.10499999620020361579264545071055181324481964111328125f,-0.718300029915571514038674649782478809356689453125f,0.3428000047922135085087802508496679365634918212890625f,0.16770000960230813102924685153993777930736541748046875f,-0.46400001032650461940676223093760199844837188720703125f,0.4023000177413222910871581916580908000469207763671875f,0.04259999593496333059050584779470227658748626708984375f,0.988999984920024388657111558131873607635498046875f,-1.1171999717384577532897083074203692376613616943359375f,0.70710002251863546263166426797397434711456298828125f,1.61930001161694558931003484758548438549041748046875f,0.2012000066518784269220532223698683083057403564453125f,1.37540004118382963582689626491628587245941162109375f,0.1838000018790364453735719507676549255847930908203125f,0.79530002438724045621398772709653712809085845947265625f,0.086999997414648533311520850475062616169452667236328125f,0.670800002422928809409086170489899814128875732421875f,0.1187000040441751613773391227368847467005252838134765625f,-0.0246000191658735101185584426275454461574554443359375f,-0.389200011062622142077316311770118772983551025390625f,-1.1906000780552634665809819125570356845855712890625f,-0.210500004559755371502660636906512081623077392578125f,-0.05139998511075949405579876838601194322109222412109375f,0.2986000138461590847072102405945770442485809326171875f,0.27600003631413050353415883364505134522914886474609375f,-0.022500000149011600836246316248434595763683319091796875f,0.448500006437301756978541789067094214260578155517578125f,0.06270000546723608625399037919123657047748565673828125f,0.8809000583350667934467992381541989743709564208984375f,0.2630000117123127978402408189140260219573974609375f,1.4212000610411177348169076140038669109344482421875f,0.326400011336803519412796958931721746921539306640625f,0.591800015863776618374458848848007619380950927734375f,-0.32530001283884057983186721685342490673065185546875f,-1.630600056672097064591753223794512450695037841796875f,-0.23530000341385604034627476721652783453464508056640625f,-0.574900012037157903677098147454671561717987060546875f,0.18930000662505630515397569979541003704071044921875f,0.331600026303530359683691131067462265491485595703125f,0.6439999789595614032577941543422639369964599609375f,-1.54059996920228048367107476224191486835479736328125f,0.93260001202821740662329830229282379150390625f,-0.86440000002682193525060938554815948009490966796875f,-0.10049997673183685975573098403401672840118408203125f,0.15179998952597395511787681243731640279293060302734375f,-0.22590000564083434486661872142576612532138824462890625f,-0.33660000030100345469463718472979962825775146484375f,-0.246700063142180514574874905520118772983551025390625f,0.25900004675984344970629535964690148830413818359375f,0.0328999315857885221703327260911464691162109375f,-0.2147999624580148037011895212344825267791748046875f,0.639600031161309257043967591016553342342376708984375f,-0.07900000965595299629029568677651695907115936279296875f,0.250600018562376902764299302361905574798583984375f,0.19380000217705950493751743124448694288730621337890625f,0.86309999809190640629452673238120041787624359130859375f,-1.2168000001430510081945612910203635692596435546875f,1.310100064033270950147880284930579364299774169921875f,-0.4054000147342684545748170421575196087360382080078125f,1.4313000255346306488490881747566163539886474609375f,-1.7325999960839748581520325387828052043914794921875f,-0.031900041377544763321338905370794236660003662109375f,0.30460003257691858902234116612817160785198211669921875f,-1.7640000230669976577502211512182839214801788330078125f,1.7561999904096123881203084238222800195217132568359375f,-1.150999995559453648041881024255417287349700927734375f,0.390599983879923673413259166409261524677276611328125f,-0.033699987280368048914169776253402233123779296875f,1.35620006327629116782418350339867174625396728515625f,0.16820000410377977662079729270772077143192291259765625f,0.9982000432193280925474709874833934009075164794921875f,0.2332999985113739516151554198586381971836090087890625f,1.399900019264220585313296396634541451930999755859375f,0.45159999724626531669713358496665023267269134521484375f,0.730199995520710576357714671758003532886505126953125f,-0.395300005313754088120958840590901672840118408203125f,-0.989600032353401548590454694931395351886749267578125f,0.05549999699741603453873040052712894976139068603515625f,-0.17430000438988191291400653426535427570343017578125f,-0.26339999377578493522378266789019107818603515625f,0.0439000246733434185131272897706367075443267822265625f,0.26590000379681588693614457952207885682582855224609375f,1.0932000324904918020507693654508329927921295166015625f,0.126899999319016909549162619441631250083446502685546875f,0.58980000216960914993791220695129595696926116943359375f,-0.089899998849630347041994582468760199844837188720703125f,-0.269899999296665082937352053704671561717987060546875f,-0.1645999992728233163319373488775454461574554443359375f,-0.967599989974498697620219900272786617279052734375f,-0.1043999972701072920244769193232059478759765625f,-1.3474000011593094683348681428469717502593994140625f,-0.3307999954566358535856807066011242568492889404296875f,-0.485700009915232211454849675646983087062835693359375f,-0.0043999964877963204656907691969536244869232177734375f,-0.776299993050098446900619819643907248973846435546875f,-0.3728000038981438279961366788484156131744384765625f,-0.923000010445714114126758431666530668735504150390625f,-0.29969998924732266232240363024175167083740234375f,0.12879997457265890403732555569149553775787353515625f,-0.024499987095595088248956017196178436279296875f,-0.0882000104367730575205541754257865250110626220703125f,0.398200004923342731899538193829357624053955078125f,-0.551999997675418629938803860568441450595855712890625f,-0.096800068092345981085600215010344982147216796875f,0.047400046160816611262589503894560039043426513671875f,-0.9006000033259393688922500587068498134613037109375f,0.341799988222122319569962201057933270931243896484375f,-0.9149000013828274546767715946771204471588134765625f,0.9051999748080969965258191223256289958953857421875f,0.76650001323223104776616310118697583675384521484375f,-1.1909999993592499389905015050317160785198211669921875f,1.19130001808404895058401962160132825374603271484375f,-0.73700000162422629745861968331155367195606231689453125f,0.8983999738082293351482121579465456306934356689453125f,-0.78779996958822040564740518675534985959529876708984375f,0.57859998760148878549358641976141370832920074462890625f,-0.25199999722838395399548971909098327159881591796875f,-0.04719995884895400450886882026679813861846923828125f,1.03439994884133401598091950290836393833160400390625f,-0.277100008288024834968155118986032903194427490234375f,0.0446000138014550628895449335686862468719482421875f,0.7040999972462653744287308654747903347015380859375f,-0.590800002571940208184742004959844052791595458984375f,0.5142999801754939426245982758700847625732421875f,0.063199987936020551870797135052271187305450439453125f,0.0213000192761416684561481815762817859649658203125f,0.3676999684751030006424343810067512094974517822265625f,0.2907999972075223649170538919861428439617156982421875f,0.981600008347629948701751345652155578136444091796875f,-0.0575999955996870927066311196540482342243194580078125f,-0.5814999804496761015570882591418921947479248046875f,0.088199998292326942106456044712103903293609619140625f,0.78240001129209968411259978893212974071502685546875f,0.211700003463029862160738048260100185871124267578125f,-0.64640002661049411614158088923431932926177978515625f,-0.28210001246035110167298398664570413529872894287109375f,-0.939200049060584074567259449395351111888885498046875f,-0.238200003284215977572557676467113196849822998046875f,-1.028000010371207917359015482361428439617156982421875f,-0.193499997608363560885891274665482342243194580078125f,-0.865100023278593699416205708985216915607452392578125f,-0.379200002010166681287728351890109479427337646484375f,-0.901699996608495535355132233235053718090057373046875f,0.105100001709163226593091167160309851169586181640625f,-0.476300007206201581055893257143907248973846435546875f,0.03830000156611201855838544361176900565624237060546875f,1.381700019109249222282187474775128066539764404296875f,0.3243999982386827074520851965644396841526031494140625f,0.016400023034214950001796751166693866252899169921875f,-0.31399999572336667608141169694135896861553192138671875f,-0.22749999035149726100968337050289846956729888916015625f,0.1717000046178699113053056635180837474763393402099609375f,-0.0790999868154520502372406554059125483036041259765625f,-0.1174000000789761077868433858384378254413604736328125f,0.56549997192621226371755938089336268603801727294921875f,-0.49690001544952355772011287626810371875762939453125f,-0.1335999733150010637672266966546885669231414794921875f,0.9088999904751773328825947828590869903564453125f,-0.597600010165571848830268208985216915607452392578125f,-0.117699966239928954792048898525536060333251953125f,0.532799994957446898524722200818359851837158203125f,0.516800047528743533575834590010344982147216796875f,-0.04260002953708141859578972798772156238555908203125f,-0.00610003068447184659817139618098735809326171875f,0.25080002611577523197183836600743234157562255859375f,-0.2580000434517870644413051195442676544189453125f,0.156399997180700633947481037466786801815032958984375f,-0.399500017940997764753774390555918216705322265625f,-0.097999997258186422044445862411521375179290771484375f,-0.6018000350117684860151712200604379177093505859375f,0.533400001040101212623767423792742192745208740234375f,0.0939000390529631800973220379091799259185791015625f,0.279799977582693149003034704946912825107574462890625f,-0.5196999868631362318183164461515843868255615234375f,0.531799988669157119858255100552923977375030517578125f,0.741999979645013407747455858043394982814788818359375f,-1.4017999745130538258308661170303821563720703125f,-0.36899999300390451395514901378192007541656494140625f,0.939599976995587393702180634136311709880828857421875f,-0.3617999808087939417333700475865043699741363525390625f,-0.4632000087976455571237011099583469331264495849609375f,-0.936399979224800116384130888036452233791351318359375f,1.13219999632537327016734707285650074481964111328125f,0.449999969601631466531443948042578995227813720703125f,-0.5365000196397307075102389717358164489269256591796875f,0.158599995666742266120508020321722142398357391357421875f,0.46279999436139995783179301724885590374469757080078125f,0.20869999795332550007742611342109739780426025390625f,1.212500000838189873064720813999883830547332763671875f,0.131099998162686748326422048194217495620250701904296875f,0.1535000171661373702391983897541649639606475830078125f,0.0887000024423003352591621251121978275477886199951171875f,0.71000005349516914776586418156512081623077392578125f,0.28130001104325064176236992352642118930816650390625f,0.82970002955496380270261624900740571320056915283203125f,-0.101100004309415847369990615334245376288890838623046875f,0.53550002646446248633083087042905390262603759765625f,0.284100011476874414473314800488878972828388214111328125f,0.5776000719308864717049800674431025981903076171875f,-0.287899994207918563660086874733678996562957763671875f,-0.02269999142289069737898898893035948276519775390625f,0.054499999724328507255677322973497211933135986328125f,-0.1480999852210285183673477149568498134613037109375f,0.1291000028342008665305229442310519516468048095703125f,-0.844400019174814442379783940850757062435150146484375f,-0.326400007909536438166497873680782504379749298095703125f,-1.657400061210990482862825956544838845729827880859375f,-0.3057000057429075656756367607158608734607696533203125f,-0.02800001141428953843615090590901672840118408203125f,0.20710000422745944437252774150692857801914215087890625f,0.156300005120038942862947806133888661861419677734375f,-0.31340000923573974045410750477458350360393524169921875f,-1.0282000421017414293345382247935049235820770263671875f,0.67080001173615411147466147667728364467620849609375f,-0.2595999934613704152042146233725361526012420654296875f,0.347700002021342324809438650845550000667572021484375f,-0.43699999007582668308913298460538499057292938232421875f,0.64369998733624778086692685974412597715854644775390625f,-0.20559998544454582969365219469182193279266357421875f,0.46070000410377953681262397367390803992748260498046875f,-0.1262000040441751680386772704878239892423152923583984375f,0.3906000574171544226231844731955789029598236083984375f,-0.2868000446975227379908801594865508377552032470703125f,0.47339998964071272613551855101832188665866851806640625f,-0.73079998029470427667320109321735799312591552734375f,0.2935000185817482343253459475818090140819549560546875f,-0.47520002189576615592159214429557323455810546875f,-0.28529999510645875915315627935342490673065185546875f,-0.104400033777951595226340941735543310642242431640625f,0.49140001521110576732098706997931003570556640625f,-0.40759998833537114393266165279783308506011962890625f,0.35950004476308805578810279257595539093017578125f,0.4975999723166228960735679720528423786163330078125f,-0.536800012809038662453531287610530853271484375f,1.015000002831220715648896657512523233890533447265625f,-0.640900053566694882789533949107863008975982666015625f,0.12560001457631597698849645894370041787624359130859375f,0.12009997775554648313800498726777732372283935546875f,-0.93039999282956120651988385361619293689727783203125f,-0.6537000357836488806384522831649519503116607666015625f,0.7640000144243239699193281921907328069210052490234375f,-0.69430000155866178346286687883548438549041748046875f,-0.833500000700354792826374250580556690692901611328125f,-0.0947000025838613535000831689103506505489349365234375f,-1.265400034925341810065901881898753345012664794921875f,-0.16670000063180923088879126225947402417659759521484375f,-0.798200002092122939956198024447076022624969482421875f,0.1110000024139881080742497942992486059665679931640625f,1.354600055524707524767791255726478993892669677734375f,0.298200008685886974735268495351192541420459747314453125f,0.4993000056564811384163249385892413556575775146484375f,-0.170400006762147004923946269627776928246021270751953125f,-0.70540000445246686666678215260617434978485107421875f,-0.07499999552965153082340066248434595763683319091796875f,0.5361999803513288753720189561136066913604736328125f,0.09979999423474061881478291979874484241008758544921875f,-0.202700008484721916346416037413291633129119873046875f,0.04689999659955501609687189557007513940334320068359375f,0.5755999963223932258671311501529999077320098876953125f,0.0427999977886676929994536067169974558055400848388671875f,0.8988000262051822897291231129202060401439666748046875f,0.3401000096812845097105082459165714681148529052734375f,1.00130004572570374676843130146153271198272705078125f,0.23830000801086426154284936274052597582340240478515625f,0.8900000428408387787015954017988406121730804443359375f,0.11280000058263539575165168571402318775653839111328125f,0.703700024086236997078458443866111338138580322265625f,0.016100001217424886323925647957366891205310821533203125f,-0.0609000064790248263335570300114341080188751220703125f,0.00530000088065862851038900771527551114559173583984375f,0.44610002986490748799752736886148341000080108642578125f,-0.08259996300339711439164602779783308506011962890625f,1.170799980071187018637601795489899814128875732421875f,-0.81379999796748148099823083612136542797088623046875f,0.733399992099404318679489733767695724964141845703125f,-1.2870999904811384340064250864088535308837890625f,0.839999974071979682577193671022541821002960205078125f,0.076300033357739938111308219959028065204620361328125f,-0.968400019517540844304903657757677137851715087890625f,-0.3225000074505803837610073969699442386627197265625f,0.3910000147819518989678044817992486059665679931640625f,-1.53829999296069086511806744965724647045135498046875f,0.823399976229667540650325463502667844295501708984375f,0.0685999964118000349344583810307085514068603515625f,-0.05539998917877664297293449635617434978485107421875f,0.0693999846041197798740540747530758380889892578125f,-0.339199990871548617832331729005090892314910888671875f,-0.440199994328617894012722899788059294223785400390625f,0.1091999971300363636572683390113525092601776123046875f,0.4477000683501370925654327947995625436305999755859375f,0.01479997206926331188014955841936171054840087890625f,1.2797000458717349147264030762016773223876953125f,-0.47100001187622542175859052804298698902130126953125f,1.5513999991178508697231563928653486073017120361328125f,-1.411999986872076728872116291313432157039642333984375f,1.369600026988983199061067352886311709880828857421875f,-0.444400002336502009114838074310682713985443115234375f,0.71319999292790914324768891674466431140899658203125f,-0.6733999828606844761935690257814712822437286376953125f,0.2732000269398093816874961703433655202388763427734375f,0.98209999085366650461992321652360260486602783203125f,-0.24170000323951246201659159851260483264923095703125f,0.18839997414350495574808519450016319751739501953125f,0.07539999610781666916636822861619293689727783203125f,0.26599998602271035519351016773725859820842742919921875f,-0.06169999925345182045788305913447402417659759521484375f,0.22330000428557372771365407970733940601348876953125f,0.25489999784380190650523445583530701696872711181640625f,0.1776999795764677703147071952116675674915313720703125f,-0.3552000138863922185095134409493766725063323974609375f,-0.59050003860890942775085932225920259952545166015625f,0.1579000023663044149913048386224545538425445556640625f,0.1617999892652033366857722285203635692596435546875f,-0.25070000488609078814761232933960855007171630859375f,-1.3251000249177218304197367615415714681148529052734375f,-0.232900002329051447613750269738375209271907806396484375f,-0.19860001567155138868514541172771714627742767333984375f,0.093299999070167505266226726234890520572662353515625f,0.013600004681944610762656111546675674617290496826171875f,0.1215999988481402116491381093510426580905914306640625f,-0.32870000210702443599330990764428861439228057861328125f,-0.084000001199543483654252895576064474880695343017578125f,0.274400031617284323459671213640831410884857177734375f,0.3214000149130822325815870499354787170886993408203125f,0.2186000417858371402957118334597907960414886474609375f,0.08790000244081019875608262736932374536991119384765625f,0.8706000243365765722813875981955789029598236083984375f,0.09410000119507312688060807204237789846956729888916015625f,0.0630999994143841558269514280254952609539031982421875f,0.108199967968463539591539301909506320953369140625f,-0.16839999329149701878804989974014461040496826171875f,0.93259997477531442200415767729282379150390625f,-1.10939996150732067548005943535827100276947021484375f,0.926399982056021809029289215686731040477752685546875f,-0.49959998265802874062302407764946110546588897705078125f,0.0453999925687902372573034881497733294963836669921875f,-0.2057999995172021812095408677123486995697021484375f,-0.80210006394386379469096937100403010845184326171875f,0.6250000291317701073268153777462430298328399658203125f,-0.4367999986529351730268899700604379177093505859375f,0.20239998864233488262698301696218550205230712890625f,0.2932999683737751439593921531923115253448486328125f,-0.486199983555078318886444321833550930023193359375f,-0.83100000297278153738034234265796840190887451171875f,0.7815999915093183592063041942310519516468048095703125f,-0.2843999776378278721011838570120744407176971435546875f,0.2265999755650762725878166747861541807651519775390625f,0.2402000219702717931369306825217790901660919189453125f,0.101199995103478490587889382368302904069423675537109375f,-0.51509999310374265935053017528844065964221954345703125f,0.83619999145269385820711249834857881069183349609375f,0.3693000578850516024687067329068668186664581298828125f,-0.091000031769275846471600743825547397136688232421875f,-0.191099987843632135309235309250652790069580078125f,0.2978000061333176962108382213045842945575714111328125f,0.4007999986231325362240340837161056697368621826171875f,-0.3193999992072582561064564288244582712650299072265625f,0.9721000088654456749992505137925036251544952392578125f,-1.2882000359922651799848836162709631025791168212890625f,-0.08839999951273203038226711214520037174224853515625f,-0.9769000389933586436796986163244582712650299072265625f,-0.3366000065594911827560054007335565984249114990234375f,-0.9256000169605016481000347994267940521240234375f,-0.136899991980194979390716980560682713985443115234375f,0.0798000384166848153455475767259486019611358642578125f,0.0395000013448298104012934572892845608294010162353515625f,-0.8205999879404901609092348735430277884006500244140625f,-0.126799997870624014950635682907886803150177001953125f,0.415099977084994353759839214035309851169586181640625f,0.24709999390840520216983122736564837396144866943359375f,1.59080001568496154362719607888720929622650146484375f,0.1587000002443790247497190648573450744152069091796875f,0.6180000161081549148178737596026621758937835693359375f,0.07149999549984931146440203519887290894985198974609375f,0.27179998412430261822692045825533568859100341796875f,-0.1796000034824013769085837566308327950537204742431640625f,-0.2799000235110524759107875070185400545597076416015625f,0.2151999979048967170935924286823137663304805755615234375f,0.4390000001192089218449154941481538116931915283203125f,-0.14000000301748516395861088312813080847263336181640625f,-1.286500011518597830217913724482059478759765625f,-0.042499998882412903800087633499060757458209991455078125f,-0.20990000115931006785530144043150357902050018310546875f,0.003499999675899749529417448457024875096976757049560546875f,0.103699989757686628077948398640728555619716644287109375f,-0.1052000018537044834232574430643580853939056396484375f,-0.804799978476762323253979047876782715320587158203125f,-0.25619998191595083625315965036861598491668701171875f,0.8607999787300826266545072940061800181865692138671875f,-0.37310004564523691072963629267178475856781005859375f,-0.0435999686956407561666537731071002781391143798828125f,-0.5912000301957132109009762643836438655853271484375f,-0.3874000221699471779857049114070832729339599609375f,-0.7198999810427422207936842823983170092105865478515625f,0.8407999949723479193863795444485731422901153564453125f,-1.375499967530370071244760765694081783294677734375f,1.2495999498754744028161667301901616156101226806640625f,-0.103300024104118648438088712282478809356689453125f,-0.889399969032406811919599931570701301097869873046875f,1.3264999816417688549563536071218550205230712890625f,-1.099799984958767851139782578684389591217041015625f,0.626100008854269418634430621750652790069580078125f,-0.6600000031292434510277189474436454474925994873046875f,-0.0034999970868228036380287448992021381855010986328125f,0.31840000372231003211709321476519107818603515625f,-0.114899996018409833453688406734727323055267333984375f,-0.31219999565482137171557042165659368038177490234375f,-0.045799991098046444193414572509936988353729248046875f,0.731999993428587902855042557348497211933135986328125f,-0.63539996496439032913627897869446314871311187744140625f,0.8861999571800236452645549434237182140350341796875f,-0.1788999885939059897310698943329043686389923095703125f,-0.05379999930858614387574334614328108727931976318359375f,0.57739997856542413234848254433018155395984649658203125f,-0.76739999497532807026800583116710186004638671875f,-0.1000999534666533463678206317126750946044921875f,1.275100044512749608571766657405532896518707275390625f,0.0058000021249056121774856364936567842960357666015625f,-1.112799989220499607966985422535799443721771240234375f,-0.31589999463260165502020981875830329954624176025390625f,-0.8878000115722410878760229024919681251049041748046875f,0.04699999909102915707226344466107548214495182037353515625f,-0.03690003746598990996830025324015878140926361083984375f,-0.243600013957917749252146677463315427303314208984375f,0.9610999857574693550077427062205970287322998046875f,0.3347000069051981885337454514228738844394683837890625f,0.4299999934434894921508885090588591992855072021484375f,-0.3594000080734491842093802915769629180431365966796875f,-0.0054000129461291290766666861600242555141448974609375f,0.213700008328259027923223811740172095596790313720703125f,-1.047699996638298269857614286593161523342132568359375f,-0.392600015231967081064112790045328438282012939453125f,-0.880800070372224741532818370615132153034210205078125f,0.23359999920576812737493810345767997205257415771484375f,1.679099997991322812396219887887127697467803955078125f,0.4044999936148523378420804874622263014316558837890625f,1.76150000272691187319651362486183643341064453125f,0.108600000248849415385166139458306133747100830078125f,-0.01570001788735453374101780354976654052734375f,-0.09860000982284555792745095459395088255405426025390625f,0.438999981269239558656636290834285318851470947265625f,0.245699998590350077165567199699580669403076171875f,0.5298000207215547430195101696881465613842010498046875f,-0.24700000091642138055902933047036640346050262451171875f,-1.0603000141799447941792777783120982348918914794921875f,1.5266000166177757790819669025950133800506591796875f,-0.909999977052212027928135285037569701671600341796875f,-0.396199976551532273560951580293476581573486328125f,1.079999968782067387707002126262523233890533447265625f,-1.09410000920444705485579106607474386692047119140625f,0.0962000025540590275596741776098497211933135986328125f,-1.1387000626429915062232112177298404276371002197265625f,0.353800047215818924684072044328786432743072509765625f,0.576800020110606936896147090010344982147216796875f,-1.5526000286430114982749728369526565074920654296875f,0.07949997651577067614425686770118772983551025390625f,0.3272000411033626487977699071052484214305877685546875f,0.73689999861121169288935561780817806720733642578125f,-0.8000000089406966718996727649937383830547332763671875f,-0.47509997701048956031399939092807471752166748046875f,1.893399982041120921394394827075302600860595703125f,-0.2764000104427335458012748858891427516937255859375f,-0.833599980095029113869031789363361895084381103515625f,0.51999999612569780538251507095992565155029296875f,-0.604799974900484205164730155956931412220001220703125f,0.582199962127209147411122103221714496612548828125f,-0.9201999600559478409422808908857405185699462890625f,-0.89120002840757450712771969847381114959716796875f,0.936000001221895416847473825328052043914794921875f,-0.731600035095215872615881380625069141387939453125f,-0.226200001063942668366735233576036989688873291015625f,-0.23140002415180216388534972793422639369964599609375f,0.1170000037103890999645727788447402417659759521484375f,-0.99829997314214669046350536518730223178863525390625f,0.33170003557503235924741602502763271331787109375f,0.141000004909932641350422954928944818675518035888671875f,0.455500005006789887573859232361428439617156982421875f,0.187399993187189028986949779209680855274200439453125f,0.439200000333785478545678415684960782527923583984375f,0.029499998085200818120910071229445748031139373779296875f,1.0470999986022710981359296056325547397136688232421875f,0.053300003167986886865037376992404460906982421875f,-0.067199970024824029479759701644070446491241455078125f,0.1847000024721026090190889590303413569927215576171875f,1.5117000457823284165215227403677999973297119140625f,0.136200010153651351885173426126129925251007080078125f,0.8774000325262552024696560692973434925079345703125f,0.31140000381171706322902537067420780658721923828125f,0.357800008666515179100997556815855205059051513671875f,-0.07480000462830072027742289719753898680210113525390625f,-0.2100000301748519149214189383201301097869873046875f,-0.29750000901520257112764511475688777863979339599609375f,-1.194400057321787667063972548930905759334564208984375f,-0.151000004798173936837457631554570980370044708251953125f,-0.514200046899915275133707837085239589214324951171875f,-0.13700000814348467503833717273664660751819610595703125f,0.855099981629848127084869702230207622051239013671875f,0.1303000021472573433101160844671539962291717529296875f,-0.6992999794304370908548662555404007434844970703125f,-0.1011999964445828847914299331023357808589935302734375f,-0.027000001996755063071731228774297051131725311279296875f,0.0220000005811452903348168774755322374403476715087890625f,0.734000045195222217131458819494582712650299072265625f,1.2513000268757341615355471731163561344146728515625f,-0.429200009796023351782423560507595539093017578125f,1.09290000981688439907202337053604423999786376953125f,-0.8655999849975106741339914151467382907867431640625f,-0.3266999921381472660186773282475769519805908203125f,0.215999996602535215828311265795491635799407958984375f,0.56969999759197254007858646218664944171905517578125f,-0.9147999973267315620972794931731186807155609130859375f,-0.0844999654889111440070337266661226749420166015625f,0.654199969488382659932312890305183827877044677734375f,1.393200007605553736311776447109878063201904296875f,-1.545399978524446993333185673691332340240478515625f,1.80930003948211748365793027915060520172119140625f,-1.0985999858319761557368110516108572483062744140625f,-0.5113000168919565879832589416764676570892333984375f,0.617800004494190257986474534845910966396331787109375f,-0.6463000478863722975120253977365791797637939453125f,0.20500001393258582282896895776502788066864013671875f,-0.483900026684999762238703624461777508258819580078125f,0.22360000848174099719045671008643694221973419189453125f,-0.77210006409287501583094126544892787933349609375f,0.04080003103315821277874420047737658023834228515625f,0.981199961650371932364578242413699626922607421875f,-1.133399959242344490206733098602853715419769287109375f,-0.518199964094161913408242980949580669403076171875f,1.27239995980858822122172568924725055694580078125f,-0.5906000054121014475327910986379720270633697509765625f,-0.174199995341897062584024524767301045358180999755859375f,0.4065000502243638624833010908332653343677520751953125f,-0.146400005562603074960037474738783203065395355224609375f,-0.13000000048428772547737253262312151491641998291015625f,-0.2888999908477070110990325702005065977573394775390625f,0.01899999948590991305508168807136826217174530029296875f,-0.83569999784529169772895329515449702739715576171875f,-0.3839999991878866314465312825632281601428985595703125f,-1.220900042167305965534751521772705018520355224609375f,-0.16290000385642056102142305462621152400970458984375f,-0.670400020471215274397991379373706877231597900390625f,-0.0898000035479664671100863415631465613842010498046875f,0.75910001646876335001934421597979962825775146484375f,0.1980000050812960044055444086552597582340240478515625f,-0.5434000109493732910692642690264619886875152587890625f,-0.18090000565946107169423839877708815038204193115234375f,-0.66990003248900220622630286015919409692287445068359375f,-0.042099999421834954915766502381302416324615478515625f,-0.554399983412027008711220332770608365535736083984375f,-0.2154999945387243709404856417677365243434906005859375f,-0.06529997111558838440714680473320186138153076171875f,0.013700001399219008391128227231092751026153564453125f,0.22130001555085188069682544664829038083553314208984375f,0.054200000929832459206636485760100185871124267578125f,1.040200021448731693141098730848170816898345947265625f,0.087100002104044005335481415386311709880828857421875f,0.791400039798021648351777912466786801815032958984375f,0.35410001322776085697086045911419205367565155029296875f,1.929900098687411524878143609385006129741668701171875f,0.1377000094905496130337496651918627321720123291015625f,-0.53169996606111447068343522914801724255084991455078125f,0.598699979345501009930785585311241447925567626953125f,-0.696799969968199928160856870817951858043670654296875f,-0.023199973705411824909106144332326948642730712890625f,0.5055999607086187896953788367682136595249176025390625f,-0.3321999787047500429792989962152205407619476318359375f,0.5425999854296446045509583200328052043914794921875f,0.3578000038981432595619480707682669162750244140625f,-0.467400008907913555589175302884541451930999755859375f,-0.210000019669532633770359097979962825775146484375f,-0.334599981167912563506661172141321003437042236328125f,0.4190000528693200720908862422220408916473388671875f,0.19479996774792684988142354995943605899810791015625f,0.28310001491159209052739242906682193279266357421875f,0.4188000092923640504949389651301316916942596435546875f,0.2684999870285389445001555941416881978511810302734375f,-0.31039999744892110555838371510617434978485107421875f,-0.3881000121548774384194757658406160771846771240234375f,-0.040600024634599218842367918114177882671356201171875f,-0.04449997949600259516955702565610408782958984375f,0.27259997715950046170974019332788884639739990234375f,0.975600000047683213466598317609168589115142822265625f,-1.18599999913573217469320297823287546634674072265625f,1.99420002268552831736769803683273494243621826171875f,-1.374799980935454524200167725211940705776214599609375f,1.4591000479102138598364035715349018573760986328125f,-1.022200024265050632976681299624033272266387939453125f,1.745700037705898211015664855949580669403076171875f,-1.245400021141767066268357666558586061000823974609375f,0.197800023865700058678385175880976021289825439453125f,0.474100029581785431531670838012360036373138427734375f,0.2658999990656971679214848336414434015750885009765625f,0.748100030222535128388017255929298698902130126953125f,0.251800010313093736158407409675419330596923828125f,0.5243000315099959340869872903567738831043243408203125f,0.0808999987304210577132579373937915079295635223388671875f,0.9588000193506480695049276619101874530315399169921875f,0.1972000048935413918371750696678645908832550048828125f,1.5010000366866591559045218673418276011943817138671875f,0.304300001148879506462918698161956854164600372314453125f,0.8328000220030544742400024915696121752262115478515625f,-0.0255999963223934035028150901780463755130767822265625f,-0.606799999397992895211473296512849628925323486328125f,-0.064900005890429046662148948598769493401050567626953125f,0.136699960175155954544834457919932901859283447265625f,-0.2223000077575445487809702171944081783294677734375f,-0.316200039546192357420295593328773975372314453125f,0.1620999980993568512221969513120711781084537506103515625f,1.3104000010624521355140359446522779762744903564453125f,0.323799996291100899981074690003879368305206298828125f,1.349100023174285123417348586372099816799163818359375f,0.32909999877363438347543933559791184961795806884765625f,0.151300014656782000344037442118860781192779541015625f,-0.282599996456503799180381975020281970500946044921875f,-0.8544000140339129689692754254792816936969757080078125f,0.07370000084042549470808580736047588288784027099609375f,0.3522000191241498878724769383552484214305877685546875f,0.248200000341236570733371991082094609737396240234375f,0.48650001777708540284805849296390078961849212646484375f,0.84470004378557206337063689716160297393798828125f,0.099199996310472382532452684245072305202484130859375f,0.7526000471949583214836820843629539012908935546875f,-0.193000018343329582393153032171539962291717529296875f,0.29040001257359993047657553688623011112213134765625f,-0.2579999949485063392984329766477458178997039794921875f,0.2572000020623210758685672772116959095001220703125f,-0.86639999792575839165920115192420780658721923828125f,0.67100003147125264479910811132867820560932159423828125f,-0.16960000009238729301586090514319948852062225341796875f,0.9965999967247245461265947596984915435314178466796875f,-0.7644000011444089803802626192918978631496429443359375f,-0.533900004929304117240462801419198513031005859375f,0.583000009924173223652132946881465613842010498046875f,-0.02080003468394320265133501379750669002532958984375f,-0.35459998325407493524608071311376988887786865234375f,0.7083000045835969871177439927123486995697021484375f,-0.449999988824129115716488058751565404236316680908203125f,1.303099988014995602725321077741682529449462890625f,-1.1531999723643064914568867607158608734607696533203125f,1.1040000245571131021193878041231073439121246337890625f,-0.1046000006139278237782264113775454461574554443359375f,0.366000002413988223537444355315528810024261474609375f,-0.024800011408328881401530452421866357326507568359375f,-0.0727999824404708117953077817219309508800506591796875f,0.376400012603401667998781476853764615952968597412109375f,-0.045699969905615756005090588587336242198944091796875f,-0.0176000362426036449647881454438902437686920166015625f,0.15170002644807123459713693591766059398651123046875f,1.2618999840080729057945063686929643154144287109375f,0.064599993908405206610723325866274535655975341796875f,-0.26280001030564381636622783844359219074249267578125f,-0.2174000010848045061351285767159424722194671630859375f,-0.472000021517276824312148164608515799045562744140625f,-0.0630000002384185631143509453977458178997039794921875f,-0.50900000771880138383806979618384502828121185302734375f,0.067000001698732380983614120850688777863979339599609375f,-0.382900025984645342447265647933818399906158447265625f,-0.393300013673305581818340215249918401241302490234375f,-1.442200075078011511919839904294349253177642822265625f,-0.17530000229626896413037684396840631961822509765625f,-0.4485000062882900095928562222979962825775146484375f,0.01610000043511390643846681314244051463901996612548828125f,0.23560000656694179976824443656369112432003021240234375f,0.09090000029504298473259638058152631856501102447509765625f,0.29490000305920849310581388635910116136074066162109375f,-0.12500000152736900904670847012312151491641998291015625f,0.423499982148408893323221491300500929355621337890625f,0.1420999976709484968750984990037977695465087890625f,0.77469998075365964496086235158145427703857421875f,-0.17630000724345451512675708727329038083553314208984375f,-0.5491000160962344889270525527535937726497650146484375f,-0.0486999985307454696936702021048404276371002197265625f,0.463600011610985074383961546118371188640594482421875f,0.2187000062420964285880842226106324233114719390869140625f,0.58390004329979483532753192775999195873737335205078125f,0.092000003114342698840033563101314939558506011962890625f,-0.2578999853044745549368599313311278820037841796875f,0.02950000697001764482507724096649326384067535400390625f,-0.52529998119249954324772033942281268537044525146484375f,-0.49249998906627301398231111306813545525074005126953125f,-0.27190000054836271914382450631819665431976318359375f,-0.7297999849587679666029771397006697952747344970703125f,-0.3167999876633287303917541066766716539859771728515625f,0.428699992197751900135926916846074163913726806640625f,-0.60649997940659527984053056570701301097869873046875f,0.0721000234127042727294565338524989783763885498046875f,-0.6857000187814232816663206904195249080657958984375f,-1.3433000052541499425018400870612822473049163818359375f,0.4293000098288057841955378535203635692596435546875f,1.2594000051677227691726557168294675648212432861328125f,0.09410000603795065643453199299983680248260498046875f,1.382199996846913858661309859598986804485321044921875f,-0.3692999855399132780320314850541763007640838623046875f,-0.6276999943286176719681179747567512094974517822265625f,0.657699972125887821761125451303087174892425537109375f,-0.06450000348687134898995054754777811467647552490234375f,-0.0929999874234199719325033584027551114559173583984375f,0.1852000002518294774489504561643116176128387451171875f,-0.5199999895319342702038056813762523233890533447265625f,-0.7117000429883599821323514333926141262054443359375f,0.3265999774277208445028009009547531604766845703125f,-0.103200003023445230354582236032001674175262451171875f,-0.66559999445974826759453435442992486059665679931640625f,-0.6826999919071792266578313501668162643909454345703125f,0.136599996948242097261072558467276394367218017578125f,0.483200029473007219849023385904729366302490234375f,-1.25260007163286246623101760633289813995361328125f,-1.65679997749328578038330306299030780792236328125f,0.5698000249683854434579188819043338298797607421875f,-0.4547000074267391056537235272116959095001220703125f,-1.406800035607815413385424108128063380718231201171875f,1.12259998729228982483618892729282379150390625f,-0.0061999761044972956369747407734394073486328125f,1.2079999787807462752198262023739516735076904296875f,-0.49759999027252188597003623726777732372283935546875f,-0.456799958717823528786539100110530853271484375f,0.0548999501228326636237397906370460987091064453125f,0.02080001754760729681947850622236728668212890625f,-1.212400038915127264971260956372134387493133544921875f,0.1527000621661545043394880849518813192844390869140625f,1.5154000322803853340047908204724080860614776611328125f,-0.14150001412630075492415926419198513031005859375f,-0.02030000538826026712513339589349925518035888671875f,-1.164000018894672194846862112171947956085205078125f,0.56580000830888810270380417932756245136260986328125f,-0.60989996979236593688256107270717620849609375f,0.4423000084280968025041147484444081783294677734375f,-0.5552000365734102160786278545856475830078125f,0.104899993038177541393451974727213382720947265625f,-1.05189998803138706762183574028313159942626953125f,0.05419996680617344964048243127763271331787109375f,1.072299982649088434527584468014538288116455078125f,0.1559999802112574940338163287378847599029541015625f,-0.3989000238537787623727126629091799259185791015625f,0.415999996006488625965857863775454461574554443359375f,-1.232400007529556251739677463774569332599639892578125f,-0.71379998902678476468963708612136542797088623046875f,1.14540002162605514257620598073117434978485107421875f,-0.5223000055968760602098654999281279742717742919921875f,0.6671999718502161780264714252552948892116546630859375f,0.71329999504685392963665435672737658023834228515625f,0.0001000014483929501096781677915714681148529052734375f,0.7055999808251858240737419691868126392364501953125f,1.0185000234246259775972021088819019496440887451171875f,-0.30359998523592945929294728557579219341278076171875f,1.07519997633546626758516140398569405078887939453125f,-0.329699979263544218355264092679135501384735107421875f,-1.2400999962143595123365003018989227712154388427734375f,0.538300024979561531068839030922390520572662353515625f,-0.21459998215511444374925531519693322479724884033203125f,-0.167000007063150324171374450088478624820709228515625f,-0.65400002086907671472459924189024604856967926025390625f,-0.522400007939338717477539830724708735942840576171875f,0.6756000007182361155599892299505881965160369873046875f,-0.57169997157454499614459564327262341976165771484375f,-0.8788000123471022551058240424026735126972198486328125f,-0.4532000204205512972066571819595992565155029296875f,0.197299997624754919200995573191903531551361083984375f,-0.7769999797940254637751422706060111522674560546875f,0.7105000398755072499312746003852225840091705322265625f,0.196600006037950691961668781004846096038818359375f,-0.2110999567374591823210039365221746265888214111328125f,-0.6284000017106532443023070300114341080188751220703125f,-0.05339996579885497585138409704086370766162872314453125f,0.4851000209689146913660806603729724884033203125f,0.533799971592426203415016061626374721527099609375f,-0.80790005198717107504080559010617434978485107421875f,-0.84139999554157185457370360381901264190673828125f,0.734000018894672479063956416212022304534912109375f,-0.151200053143501378372093313373625278472900390625f,-1.70209998615980140357351046986877918243408203125f,-0.20469997553825436398255988024175167083740234375f,0.1556999840617177000012816279195249080657958984375f,-0.75139997646808609488289221189916133880615234375f,0.2573000336110595753780216909945011138916015625f,1.4628000038981436858875895268283784389495849609375f,1.0531999796099953403683002761681564152240753173828125f,0.2899999834597111192380225475062616169452667236328125f,0.71070002008527544123950292487279511988162994384765625f,-0.0385999925374989061310770921409130096435546875f,-0.801600007081031407096816110424697399139404296875f,0.67490002701282492125756107270717620849609375f,-0.069799974751472149137043743394315242767333984375f,0.4547999830961231282344670034945011138916015625f,0.113400014674663651703667710535228252410888671875f,1.301599961781501946234129718504846096038818359375f,-1.88729999144077265071928195538930594921112060546875f,0.251700032520293603965910733677446842193603515625f,0.4982999853610987628371731261722743511199951171875f,-1.188499995112419327369934762828052043914794921875f,-0.29639999211430545500434163841418921947479248046875f,-0.8608999550700193736929577426053583621978759765625f,-0.16730002198815352443261872394941747188568115234375f,0.95449997245520368238658193149603903293609619140625f,-0.035400006687641205616046136128716170787811279296875f,0.2808000040620564075055654029711149632930755615234375f,-0.5402999821424483428700114018283784389495849609375f,-0.0198999834641819806080320631735958158969879150390625f,-0.41400002530217161478276466368697583675384521484375f,-0.74290000776797515680271999372052960097789764404296875f,0.42579998789429662764405293273739516735076904296875f,0.47239994509369154229716514237225055694580078125f,-0.051799991500377728925741394050419330596923828125f,0.7077000324010851439737734835944138467311859130859375f,0.8646999931216241552789369961828924715518951416015625f,-0.21019998553693308718237631183001212775707244873046875f,-0.17999999886378648117357670344063080847263336181640625f,-0.70909999508559684588959726170287467539310455322265625f,-0.4496999862670900682815045001916587352752685546875f,-1.416700028236209885079688319819979369640350341796875f,0.6189000116348264413090873858891427516937255859375f,0.23329999553114166843670318485237658023834228515625f,0.4324999916553498024995860760100185871124267578125f,0.948700012649595247893330451915971934795379638671875f,0.565599984401464528360747863189317286014556884765625f,0.349700009475648609935660715564154088497161865234375f,0.592999993234872846414873492904007434844970703125f,0.95399996402114661808724349612020887434482574462890625f,-0.635100002044439282400389856775291264057159423828125f,0.417999966487288876493266798206605017185211181640625f,-0.40059998186826728527876184671185910701751708984375f,0.967299982909858524493529330356977880001068115234375f,1.04990000160634533443726468249224126338958740234375f,0.74059995653629329126488300971686840057373046875f,-0.894900018221139958995991037227213382720947265625f,0.416300001543760345157352276146411895751953125f,-0.585299971115589290349134898860938847064971923828125f,1.6901999510407446081217130995355546474456787109375f,1.191299989473819653795771955628879368305206298828125f,0.2496999403715136622849968262016773223876953125f,-0.18279999704360960777194122783839702606201171875f,0.53470002709627095782707328908145427703857421875f,-0.665299995255470122401675325818359851837158203125f,-0.43389998868703827383797033689916133880615234375f,0.648200004029273912919961730949580669403076171875f,-1.4208999783158304808239336125552654266357421875f,-1.261100000286102584823311190120875835418701171875f,-0.235000010728835917461765347979962825775146484375f,0.2715999927759167320573396864347159862518310546875f,0.477400010025501586596874403767287731170654296875f,-1.14620003466606146247386277536861598491668701171875f,-0.52439993975162479955542949028313159942626953125f,2.0298000143885612800431772484444081783294677734375f,-0.1164000235557551832243916578590869903564453125f,0.669500020921230287740399944595992565155029296875f,0.14999996840953855326006305404007434844970703125f,-0.693900014503299633616961727966554462909698486328125f,-0.635000005811452883364154331502504646778106689453125f,-0.29550000485777871972459251992404460906982421875f,0.566599974037707010410258590127341449260711669921875f,0.03889999341815764122287646387121640145778656005859375f,-0.10849997611343875636435996057116426527500152587890625f,-0.13830000696778288471477935672737658023834228515625f,-0.8169999694377185761595683288760483264923095703125f,0.2284000012636184795411509185214526951313018798828125f,0.4501000033855435678020739942439831793308258056640625f,0.9947999650657177728163560459506697952747344970703125f,0.153800016966461949863287372863851487636566162109375f,-0.3504000199496744016869342885911464691162109375f,-1.5980999893561005364972515963017940521240234375f,0.2866000100612637879748945124447345733642578125f,-0.40360000855624633686602464877068996429443359375f,0.11229999710321436623416957445442676544189453125f,0.272399982793629202859619908849708735942840576171875f,-0.722399992740154317516498849727213382720947265625f,-0.1089000178188086476183116246829740703105926513671875f,0.48739997456073780313090537674725055694580078125f,1.307300003063678328629748648381792008876800537109375f,0.416000005990266874533745067310519516468048095703125f,0.1911999931290744303424844474648125469684600830078125f,0.25260002648234358701984092476777732372283935546875f,-0.12879999800473473925421785679645836353302001953125f,-0.2744000009953975194321174058131873607635498046875f,0.065800026302039071879335097037255764007568359375f,-0.3966999960124493185276151052676141262054443359375f,-0.099799994644522715958601111196912825107574462890625f,-0.51369999894052742472894124148297123610973358154296875f,0.42489998819529983808962469993275590240955352783203125f,0.161299994949996416959692169257323257625102996826171875f,-0.9767999945178627019259920416516251862049102783203125f,0.2080000309348104536866230773739516735076904296875f,1.959899968004226167295200866647064685821533203125f,0.745400000429153664072146057151257991790771484375f,0.75130002136230444875764078460633754730224609375f,0.804800020255893144849324016831815242767333984375f,-0.069700009866803913638477752101607620716094970703125f,-0.9372999887026842724679909224505536258220672607421875f,-0.099599938178062785709698800928890705108642578125f,0.19910000845938924385336576960980892181396484375f,-1.63339995823651573658708002767525613307952880859375f,0.701400009809434177299181101261638104915618896484375f,-0.05750000014901157641133977449499070644378662109375f,0.0061999975435436116555365515523590147495269775390625f,0.3477999893322587521282684974721632897853851318359375f,-0.9282000111445782852825914233108051121234893798828125f,0.02250000119209261129071819595992565155029296875f,1.1823000371873373381959027028642594814300537109375f,1.390300015968083613415728905238211154937744140625f,-0.227300034132600359271236811764538288116455078125f,0.28729999129176153616072042495943605899810791015625f,-0.1802000031761823439779846012243069708347320556640625f,0.400299992554634886943176752538420259952545166015625f,0.8363999785728750691049526722053997218608856201171875f,0.3473999662160878187933121807873249053955078125f,-0.99380000929236445017522783018648624420166015625f,1.0053999968528746222773406771011650562286376953125f,-0.099000039756298452431337864254601299762725830078125f,0.660400025463104611844755709171295166015625f,-0.920499995693564443399736774154007434844970703125f,0.029199991318583418120624628500081598758697509765625f,1.00099997395277018341630537179298698902130126953125f,-1.681899990788101373340168720460496842861175537109375f,0.41900000652670854606185457669198513031005859375f,-1.2327999948270618180146129816421307623386383056640625f,-0.0418000055074692067336172840441577136516571044921875f,-0.5506000087462361580747938205604441463947296142578125f,0.45200001704692827075859895558096468448638916015625f,0.28019996568113614276995804175385273993015289306640625f,-0.519499990224838281704933251603506505489349365234375f,0.8635999830752612016482316903420723974704742431640625f,-0.11369999644458295140481141061172820627689361572265625f,0.8333000056080519524215333149186335504055023193359375f,-0.112299995296448518189436072134412825107574462890625f,-1.033499982874840128488358459435403347015380859375f,0.33289997941851634521981395664624869823455810546875f,0.212300023850799046698512029252015054225921630859375f,0.53710000195503226905202609486877918243408203125f,-0.143200010399520127890582443797029554843902587890625f,0.01930000245273111314503466928726993501186370849609375f,0.03840001245811575092403700182330794632434844970703125f,0.1291999989926815306517937642638571560382843017578125f,0.02200004008784872677750854563782922923564910888671875f,-0.2447000034034250148806677316315472126007080078125f,-0.435399951851368260946628652163781225681304931640625f,0.5973999915480614486540389407309703528881072998046875f,-0.5443999995425343740862444974482059478759765625f,0.545800017398595738171707125729881227016448974609375f,0.09800002329796508870884963471326045691967010498046875f,-0.74260002827048321449865397880785167217254638671875f,0.6813999588847163835225728689692914485931396484375f,-0.04859996571540836640679117408581078052520751953125f,1.760899989938735643590916879475116729736328125f,0.196799954396485787810888723470270633697509765625f,1.1568999892234810289437518804334104061126708984375f,0.19240006307363533011312028975225985050201416015625f,1.619499964594840690779165015555918216705322265625f,0.82999996960163091586082373396493494510650634765625f,0.8611999679088597758891410194337368011474609375f,-0.9444000136613848184197195223532617092132568359375f,0.6676999814391137988422997295856475830078125f,0.09110000922679972745754639618098735809326171875f,1.524999998509883880615234375f,-0.0210000407099730779236779198981821537017822265625f,0.30549999159574525720017845742404460906982421875f,0.558599985384940822541466332040727138519287109375f,-1.000600017929077267808679607696831226348876953125f,0.71540000773072254247608725563623011112213134765625f,-0.2173000593900678012460048194043338298797607421875f,-1.94549998384714140087226041941903531551361083984375f,0.85390001342296528719089110381901264190673828125f,0.006899978345632717946500633843243122100830078125f,1.2241000339031220534025123924948275089263916015625f,1.90620002773702168497038655914366245269775390625f,0.27329994661808054701168657629750669002532958984375f,-0.565999972909688953137674616300500929355621337890625f,1.570600007796286945449537597596645355224609375f,-0.9150000008940697426851329510100185871124267578125f,-0.931099980244040725807508351863361895084381103515625f,0.40749997720122355104876987752504646778106689453125f,-0.30839996464401497888729863916523754596710205078125f,1.04709998392462733107777239638380706310272216796875f,-0.6367999897122385011272172050666995346546173095703125f,0.7322000038504601349131917231716215610504150390625f,-0.096299956579506851284122603829018771648406982421875f,0.872999972522258804730199699406512081623077392578125f,0.1920000248700379774646762598422355949878692626953125f,0.02550000068545354992011198191903531551361083984375f,-0.2089999850690371641093179277959279716014862060546875f,0.31489999780654898131615482270717620849609375f,0.5524000444844368029606584968860261142253875732421875f,0.56120000203251851900176916387863457202911376953125f,0.11360004078000773830581238144077360630035400390625f,-0.16989999452829351866967044770717620849609375f,-0.708700002367794734681183399516157805919647216796875f,-0.901099999690055941670152606093324720859527587890625f,-0.5982999776497484045734154278761707246303558349609375f,-0.55190002245306946093705846578814089298248291015625f,-0.052499999515712436615189062649733386933803558349609375f,0.80489998782277094591108834720216691493988037109375f,0.187599955105782090214461277355439960956573486328125f,0.9005000187158582747315449523739516735076904296875f,-0.5282999672561885518007329665124416351318359375f,-0.266800016310810850228563140262849628925323486328125f,-1.12649996405839925017744462820701301097869873046875f,0.93109999223947514934707214706577360630035400390625f,0.4478999995723371707612159298150800168514251708984375f,-0.2136000069916246246748414705507457256317138671875f,-0.570899991875887025116753648035228252410888671875f,0.8026000115811822155365007347427308559417724609375f,0.68189999071359608251441386528313159942626953125f,-0.7599000075109305640808088355697691440582275390625f,-0.196899978177994317007915014983154833316802978515625f,-1.1689000492043797319041686932905577123165130615234375f,0.10050002288818404849735088646411895751953125f,1.5758000330448158621265974943526089191436767578125f,-1.494600001955031842726384638808667659759521484375f,-0.45289994648694875678529569995589554309844970703125f,-0.4275999640464789308680337853729724884033203125f,-0.7049000779747967015964604797773063182830810546875f,1.16449999856948949172874563373625278472900390625f,0.977300049015135119390151885454542934894561767578125f,-1.1456999895378949805291313168709166347980499267578125f,-0.788700013096631291631410931586287915706634521484375f,-0.73379997159242638105070000165142118930816650390625f,0.780399994841217914398612265358678996562957763671875f,0.34960000448822992069608517340384423732757568359375f,-0.4405000036656854245364911548676900565624237060546875f,0.841399957394600050974986515939235687255859375f,1.0298000438928607280786309274844825267791748046875f,0.02620001261234250478082685731351375579833984375f,0.8454000221848492202525449101813137531280517578125f,0.8396999961018565272752312012016773223876953125f,-0.2870999891400334291802209918387234210968017578125f,-0.733799974274635502524688490666449069976806640625f,1.06120003079175884153073639026843011379241943359375f,-0.170099992358684393156664782509324140846729278564453125f,-0.3232999990701674875026583322323858737945556640625f,-1.042899991376697865774758611223660409450531005859375f,0.32659998934864997721660984097979962825775146484375f,0.3694000207953156422746587850269861519336700439453125f,0.0313000103354454939363904486526735126972198486328125f,0.2361999524675304495957561812247149646282196044921875f,0.58390001237988453652860698639415204524993896484375f,0.2576999713808294512062957437592558562755584716796875f,-0.956699982005357885128660200280137360095977783203125f,0.813000003367662760211942440946586430072784423828125f,-0.35309998440146461717858983320184051990509033203125f,-0.3235000256597995171858883622917346656322479248046875f,0.868800009590387389835086651146411895751953125f,-0.8399999490194023366029796306975185871124267578125f,-0.57570000438317647795116727138520218431949615478515625f,0.16039998703673485902498896393808536231517791748046875f,-0.040999994888901714062967585050500929355621337890625f,1.616999984785914445950538720353506505489349365234375f,-0.16379998045861732247630015990580432116985321044921875f,-1.2402000016301872875601475243456661701202392578125f,0.58049997252225882249376809340901672840118408203125f,-1.5518000113561749575552539681666530668735504150390625f,0.11080000354051566802127126720733940601348876953125f,-0.763799998787045186787736383848823606967926025390625f,0.41010000472664831505653637577779591083526611328125f,-1.0015999707221983072713555884547531604766845703125f,-0.090400001919269445949112196103669703006744384765625f,-0.0034999898970127230057869383017532527446746826171875f,0.510599996247886878109056851826608180999755859375f,-0.37820001103281963850122338044457137584686279296875f,0.7960000190287830346136388470767997205257415771484375f,-0.41010000115036948642455172375775873661041259765625f,-0.7744000012934206811365811518044210970401763916015625f,1.599499968171119945736791123636066913604736328125f,-0.5436999676108360546322728623636066913604736328125f,0.86489998767375908528265426866710186004638671875f,-0.1195000052005052992853961768560111522674560546875f,0.97160000007748603678692234097979962825775146484375f,-0.9974999780952928940536139634787105023860931396484375f,0.10660001065730995151170645840466022491455078125f,0.8381000100500879401721476824604906141757965087890625f,0.0538000048220157456313472721376456320285797119140625f,0.877399987189471541881857774569652974605560302734375f,1.081699960622191980519346543587744235992431640625f,0.56089999597370621131631196476519107818603515625f,-0.2949999868869781494140625f,0.5240000236630439900409328402020037174224853515625f,0.44359992697238936898429528810083866119384765625f,-1.416800010480731231155004934407770633697509765625f,-1.285899948457629005105218311655335128307342529296875f,1.1779999845363195998260152919101528823375701904296875f,-0.9254999987483021328671384253539144992828369140625f,0.921600020492077565137378769577480852603912353515625f,0.849499949097634043937432579696178436279296875f,-0.21759999187439671430865928414277732372283935546875f,0.113399977943301255578489872277714312076568603515625f,-0.42550000895559769720222220712457783520221710205078125f,-1.456100021147727741066546514048241078853607177734375f,-0.02920000905096531429450124051072634756565093994140625f,0.2478999986410139744208436241024173796176910400390625f,-0.40469999036490922339481812741723842918872833251953125f,-0.164899985252321024109534164381329901516437530517578125f,1.026399978405237334300181828439235687255859375f,0.23430000837594244789130470962845720350742340087890625f,0.70969997881650925819485564716160297393798828125f,0.6397999864116312718209655940881930291652679443359375f,0.60039999797046184060178575236932374536991119384765625f,-0.41740001408606752786312199532403610646724700927734375f,0.5892999800264835386087725055404007434844970703125f,0.54009997511059071317873758744099177420139312744140625f,0.078500001464039070864231462110183201730251312255859375f,-0.163099974305927986062414447587798349559307098388671875f,0.321899990862608120778531883843243122100830078125f,-0.93119996839314733705350590753369033336639404296875f,-0.42619998176693929536895666387863457202911376953125f,-0.1207999918058517696550779874087311327457427978515625f,-0.18520002837777127524532261304557323455810546875f,-0.6809000054173168781090907941688783466815948486328125f,-0.3864999873042107569887093632132746279239654541015625f,0.107699938244372628304290628875605762004852294921875f,-0.6289999826848509112409146837308071553707122802734375f,-0.4125999733597043306332352585741318762302398681640625f,0.10839999053478255319760137354023754596710205078125f,-0.307499966435134997055911298957653343677520751953125f,-0.147100004823505881024203745255363173782825469970703125f,0.880200022491812728020477152313105762004852294921875f,1.603600011573731887182248101453296840190887451171875f,0.740599969053268747387619441724382340908050537109375f,0.327299994141608596720516288769431412220001220703125f,-0.2552999742448329723032429683371447026729583740234375f,0.59420000670403283482556844319333322346210479736328125f,0.207199992227554474766293424181640148162841796875f,-1.05849999421834883861492926371283829212188720703125f,0.672899950361252052744021057151257991790771484375f,1.91390005455017142566020993399433791637420654296875f,-0.729299989414215588112710975110530853271484375f,0.4695000102669002473021464538760483264923095703125f,0.3725999758929017247055526240728795528411865234375f,0.421300022703409382529571303166449069976806640625f,-0.53490001404881493130005765124224126338958740234375f,-0.7812000388383870674857689664349891245365142822265625f,1.4693999895215039686036107013933360576629638671875f,1.317700063395501164364986834698356688022613525390625f,-1.039300051403046154518960975110530853271484375f,0.7419000051490964864342458895407617092132568359375f,0.526899983579665498467647921643219888210296630859375f,-1.0299999937601389543573304763413034379482269287109375f,0.51590000722408291267129243351519107818603515625f,-0.599199980217218097777731600217521190643310546875f,0.1578000268459316401958858477883040904998779296875f,0.270399997411668646662974424543790519237518310546875f,-1.599399951970577493653991041355766355991363525390625f,-1.2835999973803755569434770222869701683521270751953125f,0.4134999951124189720985668827779591083526611328125f,1.6479999987483022749756855773739516735076904296875f,0.040200015413761003291170936790877021849155426025390625f,0.0741999955281615886093504741438664495944976806640625f,-0.79049998231977236518019935829215683043003082275390625f,-0.40039997194930931900813675383687950670719146728515625f,0.2161999891430139708603519466123543679714202880859375f,-0.422499979138374559539670372032560408115386962890625f,-1.5007000090584161622331293983734212815761566162109375f,0.626599966251850215570584623492322862148284912109375f,-1.1440000110715626835400371419382281601428985595703125f,-0.59849999243021034800449342583306133747100830078125f,-1.1161000052407381577523892701719887554645538330078125f,0.132999992787838028363012199406512081623077392578125f,-0.0801999734669924801977458628243766725063323974609375f,-0.50500000707805148270068684723810292780399322509765625f,0.29409998156279282266467589579406194388866424560546875f,0.8139000083565710585986607839004136621952056884765625f,-0.7945000132471324860716777038760483264923095703125f,-0.88940002811551099881626214482821524143218994140625f,-1.4486999931849535716565924303722567856311798095703125f,0.0860999953687191510454113085870631039142608642578125f,-0.51679999066218751391232899550232104957103729248046875f,0.59539999566078183335093854111619293689727783203125f,0.8572999719947578256551423692144453525543212890625f,0.38330000339150416266420506872236728668212890625f,1.245499953858554942343062066356651484966278076171875f,-0.9909999671354892125663127444568090140819549560546875f,0.24759997779279974139399200794287025928497314453125f,0.584699981051683526089846054674126207828521728515625f,1.109800017480552458692955042351968586444854736328125f,0.4560000468194491674012169823981821537017822265625f,-0.341500002056360330016104853712022304534912109375f,-0.878599996560812002144302823580801486968994140625f,-0.3617999903082846913093817420303821563720703125f,0.0799999891221521153283902094699442386627197265625f,1.4032000156521799993925014859996736049652099609375f,-0.507599957787989719548704670160077512264251708984375f,-0.13819999449253117518310318700969219207763671875f,0.2912000085890298350932425819337368011474609375f,0.335600030893086653804857633076608180999755859375f,0.6433000106930730765952830552123486995697021484375f,-0.2756000033259393688922500587068498134613037109375f,-0.171200034070015494336303163436241447925567626953125f,0.776299986195564173385719186626374721527099609375f,0.1766000031322239038900079322047531604766845703125f,-0.76420001165866846548624380375258624553680419921875f,-1.1070000000596043054912342995521612465381622314453125f,0.49430004246234915399327292107045650482177734375f,1.7536000317275526327875923016108572483062744140625f,-0.0597000679254531263495664461515843868255615234375f,-0.74050003212690373999294024542905390262603759765625f,1.241300023150444076236453838646411895751953125f,0.12660003136992514782832586206495761871337890625f,-0.677499964684248112689601839520037174224853515625f,0.8603000050902362971783077227883040904998779296875f,-0.019900018519162898655849858187139034271240234375f,-0.9618000086367128886877253535203635692596435546875f,-0.3185999949216835602783248759806156158447265625f,-0.594999987781047678936374722979962825775146484375f,0.11019998725056634469865457504056394100189208984375f,0.01159999173283576823223484097979962825775146484375f,0.4828999881729483245607070784899406135082244873046875f,0.22709999286532411844063972239382565021514892578125f,1.08589997518658609720887397998012602329254150390625f,0.8554000172674653956761403605923987925052642822265625f,0.9417999927669764570481447663041763007640838623046875f,-0.0758000002622605251190179842524230480194091796875f,-0.3098000245958558718228914585779421031475067138671875f,0.1474000067472456265704749966971576213836669921875f,0.2646000163346535760666711212252266705036163330078125f,-0.5070999982297419439447594413650222122669219970703125f,-0.57999999452382322573384954011999070644378662109375f,0.28769999291300774046931110206060111522674560546875f,-1.1174000106960537692657453590072691440582275390625f,-0.33930000617802147377943811079603619873523712158203125f,0.4499999873712658970958955251262523233890533447265625f,0.674100020343065153838324476964771747589111328125f,0.9509000109493730423793067529913969337940216064453125f,-0.5794000269234178546184921287931501865386962890625f,-1.6505999958753587719684219337068498134613037109375f,0.841500013530254165061705862171947956085205078125f,-0.807399992702900970442669859039597213268280029296875f,-0.075799991023540513879197533242404460906982421875f,-0.430000023245811657801596084027551114559173583984375f,0.074499992311000795552899944595992565155029296875f,-0.0015999880447987724352287841611541807651519775390625f,0.289099999779462990545653156004846096038818359375f,1.176099999988078792512169457040727138519287109375f,0.690600026422739432518937974236905574798583984375f,1.5119000115752214696840383112430572509765625f,-1.015300053891539988626391277648508548736572265625f,-1.2420000233799211031282538897357881069183349609375f,0.80930003329813526846692184335552155971527099609375f,-0.254900006449222615856342599727213382720947265625f,1.58860001667737993358286985312588512897491455078125f,-0.443600009226798874806263484060764312744140625f,0.0224000109195703345221772906370460987091064453125f,1.17009997954368571981831337325274944305419921875f,0.3237000198580322063435232848860323429107666015625f,0.555499989863485144070409660344012081623077392578125f,-0.5644000223599372478844315992319025099277496337890625f,-1.298500013589858781415387056767940521240234375f,-1.0035000430941583005051143118180334568023681640625f,-0.7249000152409070807379976031370460987091064453125f,-0.12320001192688945224062990746460855007171630859375f,-0.4805999887228011857587262056767940521240234375f,-0.63340003613233619716993416659533977508544921875f,-1.7660999692916874437287333421409130096435546875f,0.37249998208135348676250941934995353221893310546875f,-0.802599997052550140352877861005254089832305908203125f,-0.29639997806996054574568688622093759477138519287109375f,0.69040002356320595566785414121113717555999755859375f,-0.957100016744435233562171561061404645442962646484375f,-0.336699983942509106782381422817707061767578125f,0.456099993431568151436295011080801486968994140625f,0.372400021052360585827045724727213382720947265625f,-0.01329998431801815428343616076745092868804931640625f,0.82700000467896483957730424663168378174304962158203125f,0.5213000078022478334105471731163561344146728515625f,-0.7052000245779754816766171643394045531749725341796875f,-0.42420000629425036464681397774256765842437744140625f,-0.3073000060439114999866205835132859647274017333984375f,-0.084800004255771721517476180451922118663787841796875f,-1.0044999687671658517018613565596751868724822998046875f,-0.52869999979734405570752642233856022357940673828125f,-1.7433999704554679066603739556740038096904754638671875f,0.79829997716546063912801400874741375446319580078125f,0.3387000044725834158043653587810695171356201171875f,0.2033000001132487799537784667336381971836090087890625f,0.986999985586851646957029515760950744152069091796875f,-0.840399999535083708224192378111183643341064453125f,0.85479999926388217090789112262427806854248046875f,-0.3069000036776063922161483787931501865386962890625f,1.463400041235983106702178702107630670070648193359375f,-0.478099991255998446604280616156756877899169921875f,0.27239998320341118898113563773222267627716064453125f,-0.95849998825788507161860252381302416324615478515625f,-0.012300037336349323169315539416857063770294189453125f,-0.50799999036639942318061002879403531551361083984375f,0.57430002994537387994711252758861519396305084228515625f,0.289000009804964097526180921704508364200592041015625f,0.028899979224801253252508104196749627590179443359375f,0.0771999953567981123114805086515843868255615234375f,-0.7309000156059857200574469970888458192348480224609375f,0.474000003844499584459981633699499070644378662109375f,-0.329000001423060961514011069084517657756805419921875f,1.299600025685131665653671007021330296993255615234375f,0.10539998310655374069710887852124869823455810546875f,-0.640599987642466839332655581529252231121063232421875f,-0.53059997426867511194359394721686840057373046875f,-1.0317000085294250055767406593076884746551513671875f,-0.788699992942810013119014911353588104248046875f,1.97700004446506483191114966757595539093017578125f,0.281899973726272179419538588263094425201416015625f,-2.35550005447864574392724534845910966396331787109375f,-0.58679993934631369256749167107045650482177734375f,0.6441999979496007000534518738277256488800048828125f,-0.62679999895095761530683375895023345947265625f,-0.1167000208973885122532010427676141262054443359375f,1.4023999659180645949163590557873249053955078125f,0.151400023108720915843150578439235687255859375f,1.1886000163793557504732234519906342029571533203125f,0.35310003104209908286748031969182193279266357421875f,1.43109999909400897877276293002068996429443359375f,-0.07430002465546170498100764234550297260284423828125f,-0.34790000460147840755098513909615576267242431640625f,0.102999983251094828773375411401502788066864013671875f,-0.8083999798059462449373313575051724910736083984375f,-0.0642999929904934930391391390003263950347900390625f,-0.096200005161762192074093036353588104248046875f,0.05730001140832907680078278644941747188568115234375f,-0.43440000226199604327348424703814089298248291015625f,-1.581100025394559072111633213353343307971954345703125f,-0.2197999938249584062077701673842966556549072265625f,0.67070001194551576162439232575707137584686279296875f,-0.038100004518032193345788982696831226348876953125f,0.013499994069337883928483279305510222911834716796875f,-0.309899994230270314776731765959993936121463775634765625f,0.00160000126957893229473484097979962825775146484375f,0.16869997241646050856189731348422355949878692626953125f,0.237199982243776474888363736681640148162841796875f,1.6267000141546130720371365896426141262054443359375f,-0.363200010883808044326315211947076022624969482421875f,-0.4875000467896459799277408819762058556079864501953125f,0.21480002362728090048449303139932453632354736328125f,1.776199979978799969870806307881139218807220458984375f,-0.50299998325109473995553344138897955417633056640625f,0.4433000413894647717683028531610034406185150146484375f,0.53489999169707314052857327624224126338958740234375f,-1.214599969992041561539508620626293122768402099609375f,0.6188000005006788484251956106163561344146728515625f,-0.6108999882623546984206086563062854111194610595703125f,0.71619999219775198895376888685859739780426025390625f,-0.100100001038610830761399483890272676944732666015625f,-0.236099998870491933899984360323287546634674072265625f,0.1570999911144374816540647543661179952323436737060546875f,-0.543899979895353435921379059436731040477752685546875f,-0.7339000118955969487188895072904415428638458251953125f,0.0880999942362308274823590181767940521240234375f,0.1632000030606983731473746956908144056797027587890625f,-0.153700000169873174371559798601083457469940185546875f,0.4888999942749734373137471266090869903564453125f,0.46309999125599876634851170820184051990509033203125f,1.2942000040031966801024054802837781608104705810546875f,1.1297000175595286464158562012016773223876953125f,-0.60159997698068679028438054956495761871337890625f,0.00089999917745586799355805851519107818603515625f,0.569099993371963819299708120524883270263671875f,-0.59219995646476686346204587607644498348236083984375f,-1.05519996176958130718048778362572193145751953125f,0.6819999881386760165469240746460855007171630859375f,-2.21579999608993460924466489814221858978271484375f,-0.2782000049233435134965475299395620822906494140625f,-1.58030000598430575564634636975824832916259765625f,-0.12249998345971135904619586654007434844970703125f,0.3036999807238576210011160583235323429107666015625f,-0.234099994862079885393768563517369329929351806640625f,0.868900016999244684257064363919198513031005859375f,0.74750000163912755368755824747495353221893310546875f,0.69349998304247861824478604830801486968994140625f,9.998431205726632242658524774014949798583984375e-05f,0.966099964970350555404365877620875835418701171875f,-0.752400001233815896029000214184634387493133544921875f,0.2874999941140412573759022052399814128875732421875f,1.08769999917149551293960030307061970233917236328125f,-0.6946999874591828216807698481716215610504150390625f,-1.14069999181032155632919966592453420162200927734375f,1.2382999811887742680482915602624416351318359375f,-1.0320000198781489331167904310859739780426025390625f,-0.239699987608194220456425682641565799713134765625f,-1.6454999972581862976994671043939888477325439453125f,-0.1132999788045889744125815923325717449188232421875f,1.93000003427267063926819901098497211933135986328125f,-0.21820002164989682480467081404640339314937591552734375f,-0.543999987602234114092425443232059478759765625f,-1.329400018131732696957669759285636246204376220703125f,0.455199999469518612471574670053087174892425537109375f,-0.52680003646463158251123104491853155195713043212890625f,-0.06929999418258692145400345907546579837799072265625f,0.4732999427065254227642299156286753714084625244140625f,-1.03599998900294298920243818429298698902130126953125f,0.72430001396387755363548421883024275302886962890625f,-0.68130000243782973257111734710633754730224609375f,-0.0085999947726731473807149086496792733669281005859375f,0.4340999731063843736222906954935751855373382568359375f,0.2703000040471554843435342263546772301197052001953125f,0.734500001251697565152198876603506505489349365234375f,0.7517000116959213773526471413788385689258575439453125f,0.110099997872114219177319682785309851169586181640625f,0.0088000177860256822981455115950666368007659912109375f,0.2272999936759472205949350609444081783294677734375f,-0.5062999824330212117473593025351874530315399169921875f,0.253199991139769597925379684966173954308032989501953125f,-0.071800013963878084322089989655069075524806976318359375f,-0.6858999808490278127237615990452468395233154296875f,0.24320000458806756427065920433960855007171630859375f,0.75829998923540120614461557124741375446319580078125f,1.5059000059202312105099963446264155209064483642578125f,-0.07519999261498444553808440105058252811431884765625f,0.838599974544346427052232684218324720859527587890625f,0.41969998626708981959154698415659368038177490234375f,0.284199953469634503111507228823029436171054840087890625f,-0.00459999681413147953890074859373271465301513671875f,1.39399995660781872430789007921703159809112548828125f,-0.777899998790025914985335475648753345012664794921875f,-0.7401999719023706347797997295856475830078125f,-0.2472999932289123847795053734444081783294677734375f,0.71969999908208848182766814716160297393798828125f,-0.131800020147860408314954838715493679046630859375f,0.81159997269660255625467470963485538959503173828125f,-0.35290001208931176535088525270111858844757080078125f,1.2235000195503236142258174368180334568023681640625f,0.03710000612735786518214808893389999866485595703125f,-1.667699963259696716022517648525536060333251953125f,1.228200053501129929145463393069803714752197265625f,0.84349997842311807971782400272786617279052734375f,-0.533700023639202214553733938373625278472900390625f,0.584699989843368683750668424181640148162841796875f,0.40500006675720268134455182007513940334320068359375f,1.832899984931945169819300645031034946441650390625f,-0.399800020796060806560490163974463939666748046875f,-0.3857999916195868905788302072323858737945556640625f,0.43299999234080299714833017787896096706390380859375f,-0.1897999848842619741162707214243710041046142578125f,0.062200008916855065166373606189154088497161865234375f,-1.91150000190734914440326974727213382720947265625f,0.6162999939441675678608589805662631988525390625f,0.06389999136924728873054846189916133880615234375f,-1.16010003408193629326206064433790743350982666015625f,0.188500010013580521217591012828052043914794921875f,-0.19879994775056797351453496958129107952117919921875f,-0.054499986425042390436601635883562266826629638671875f,0.697499973997473876607955389772541821002960205078125f,-0.53189999399185194928207920384011231362819671630859375f,-0.05609999581575397797905679908581078052520751953125f,1.510599994422495484514001873321831226348876953125f,0.4471999962508679260508870356716215610504150390625f,0.834299996790289544179586300742812454700469970703125f,0.2952999928712845001399500688421539962291717529296875f,-0.772399998588860103865272321854718029499053955078125f,0.81620001246333107047803423483856022357940673828125f,0.040399981802701934174848474867758341133594512939453125f,-0.80029997141361253198965641786344349384307861328125f,0.63499999683350349943822266141069121658802032470703125f,-0.036200006353855229690452688373625278472900390625f,-1.4535000107586382167568217482767067849636077880859375f,0.384099996501207296972779658972285687923431396484375f,-0.4455999801546333127788557249004952609539031982421875f,0.9765999795138835537500199279747903347015380859375f,-0.45070000905841556271269610078888945281505584716796875f,-0.01130000139474862663746534963138401508331298828125f,0.82070000239014628817812990746460855007171630859375f,-0.2749999828636646714841162975062616169452667236328125f,-0.39530000196099257525617076680646277964115142822265625f,-1.09710001178979865699147921986877918243408203125f,0.1438000001654031745346173920552246272563934326171875f,-0.235900001859664865122567789512686431407928466796875f,-1.107700015525519887660266249440610408782958984375f,0.570099996978044476492186731775291264057159423828125f,0.00079995920956132948731465148739516735076904296875f,0.44600003419071454491273698295117355883121490478515625f,0.17589999679326950854374445043504238128662109375f,0.5066000364363194563566139549948275089263916015625f,1.8542999733209608592687800410203635692596435546875f,-0.36549996909499160580026000388897955417633056640625f,0.0524999970197672638505537179298698902130126953125f,0.4908999999225134303060258389450609683990478515625f,0.2816999866247176242950445157475769519805908203125f,0.307000008478760744168312157853506505489349365234375f,-0.19060000541210175839523799368180334568023681640625f,0.453099982967227532526521827094256877899169921875f,0.4911999714106325409801456771674565970897674560546875f,-0.29299999312311408861120298752211965620517730712890625f,-0.26729999829530726174198207445442676544189453125f,-0.130500005602836921525522484444081783294677734375f,0.750500010371208503556772484444081783294677734375f,-0.79360002278685559673476745956577360630035400390625f,-0.878499997347592653795800288207828998565673828125f,0.951500012934207717307799612171947956085205078125f,-1.2020000289678574034724078956060111522674560546875f,-0.3897000396132472843646610272116959095001220703125f,1.234000003695488345556441345252096652984619140625f,-0.160099960358440540630908799357712268829345703125f,-0.97799995907396120031762620783410966396331787109375f,0.314300050769746430745499310432933270931243896484375f,2.0184999831914893775319796986877918243408203125f,-0.9885000164210797635178096243180334568023681640625f,1.2674000076413154403098815237171947956085205078125f,0.40000002356246167067865826538763940334320068359375f,-1.199800015133619002227760574896819889545440673828125f,0.1517999876260756053625300410203635692596435546875f,-0.9039000026196237769227082026191055774688720703125f,1.0711999786376953380795384873636066913604736328125f,-0.6161999907448885149818806894472800195217132568359375f,0.331099977040290927021715106093324720859527587890625f,0.065799977165460654671136353499605320394039154052734375f,0.2350999878883361926540374042815528810024261474609375f,0.0660999986469745781647588955820538103580474853515625f,-0.16590000163614748718288183226832188665866851806640625f,-0.230099987739324773894367126558790914714336395263671875f,0.08969999362453828783969811411225236952304840087890625f,-1.067299977880716621569945345981977880001068115234375f,0.059899991250038198131733224727213382720947265625f,0.205699994455277757055000620312057435512542724609375f,0.604500007957220031329370613093487918376922607421875f,0.6012000088870521619099918098072521388530731201171875f,-0.1471999995291231044802771066315472126007080078125f,1.049300012697278905449138619587756693363189697265625f,-0.928000012457370626606234509381465613842010498046875f,-1.027099994988739606327499132021330296993255615234375f,0.8421000034451484594910652958787977695465087890625f,-0.23889999054968347191874045165604911744594573974609375f,-0.266299989473819831431455895653925836086273193359375f,0.8509999785348778544857850647531449794769287109375f,1.01759997864961615476886436226777732372283935546875f,0.3153999742776159020962722934200428426265716552734375f,0.8302000034928320726379524785443209111690521240234375f,1.262200026928633977973959190421737730503082275390625f,-0.62570001446008660650477395392954349517822265625f,1.015399996554851469454661128111183643341064453125f,0.57309998101144987714405942824669182300567626953125f,1.56139993709176838621033311937935650348663330078125f,-1.3161999801278110311386626563034951686859130859375f,0.706799987030028997736508244997821748256683349609375f,1.1078000158935783492353266410646028816699981689453125f,1.087200017857551159750073566101491451263427734375f,-0.442499987930059290874851285479962825775146484375f,0.2504000016212462043085906771011650562286376953125f,0.401599991881847362407143009477294981479644775390625f,0.80869997357130074533415609039366245269775390625f,1.0186000175714493565237717120908200740814208984375f,-0.4975000166893011055435636080801486968994140625f,0.0091999934792514892478720867075026035308837890625f,-0.021200002181529953304561786353588104248046875f,0.162300017908961091706032675574533641338348388671875f,0.8982000393077733679803031918709166347980499267578125f,0.338100024560094192249692923724069260060787200927734375f,-0.575699993300438084276038352982141077518463134765625f,-1.4561000470012428475996557608596049249172210693359375f,-0.046999966979026197577695711515843868255615234375f,-0.64869997774362531117731123231351375579833984375f,0.002700016307830566120173898525536060333251953125f,1.47100001791119527894124985323287546634674072265625f,-1.5477000231623652126700108055956661701202392578125f,-0.2799000133596365458288346417248249053955078125f,0.6858999885603787038945711174164898693561553955078125f,0.204000015132129453032661103861755691468715667724609375f,0.3213000171713533692496866933652199804782867431640625f,0.87719996652305154416495724944979883730411529541015625f,0.585800002162157884555426790029741823673248291015625f,0.364899990020692399639301584102213382720947265625f,0.868300003205239701031814547604881227016448974609375f,0.3751999821469189644318475984618999063968658447265625f,-0.0882999836847186969635004061274230480194091796875f,0.5263999776229264515592376483255065977573394775390625f,0.25559996398687356844448004267178475856781005859375f,0.610500005193054473551228511496447026729583740234375f,1.00460004636049315962509353994391858577728271484375f,-0.903400005249678894614362434367649257183074951171875f,-0.881500026792287894039645834709517657756805419921875f,-0.1055999959871172233505376425455324351787567138671875f,-0.72410002377033233500469577847979962825775146484375f,0.6303999796047807802068518867599777877330780029296875f,0.76400000764429443211867010177229531109333038330078125f,-0.4558999668791894332997571837040595710277557373046875f,0.00170001486241933008614068967290222644805908203125f,0.445499984778464030998890166301862336695194244384765625f,-0.6726999860212210524679221634869463741779327392578125f,0.31749999038875109658164319625939242541790008544921875f,0.6529999973699442961105887661688029766082763671875f,-0.524199990163743745341662361170165240764617919921875f,0.507200016888976534801258821971714496612548828125f,0.87859998568296437948532684458768926560878753662109375f,1.17410001427084154101976309902966022491455078125f,0.061699999849498254889113013632595539093017578125f,0.02620001428872331672437212546356022357940673828125f,-0.8874999994039536321821515230112709105014801025390625f,0.0988000344753263703978518606163561344146728515625f,1.534500013843179022643425923888571560382843017578125f,0.560400011754035975997112473123706877231597900390625f,0.1910999653056262292949440961820073425769805908203125f,0.2721999710679057216111687012016773223876953125f,-0.8912999754667281226971908836276270449161529541015625f,-0.416000010907649819813514113775454461574554443359375f,1.216300025683641816698354887194000184535980224609375f,0.98240000972747765217718551866710186004638671875f,-0.05980001669824119847618248968501575291156768798828125f,0.2177999919772144465923702227883040904998779296875f,-1.5233000026836991391832043518661521375179290771484375f,-0.6211999967053538629357944955700077116489410400390625f,0.329799991869181408077338346629403531551361083984375f,-0.2629999895095824768986858543939888477325439453125f,-0.21940003351718206925369258897262625396251678466796875f,-0.03679999418258628196554127498529851436614990234375f,1.148400027638673659424739526002667844295501708984375f,0.4264000035881991834685322828590869903564453125f,-1.0523000136807560966900609855656512081623077392578125f,-0.538399993515014330114354379475116729736328125f,1.444400004273652893260759810800664126873016357421875f,0.7370000069141386944693294935859739780426025390625f,0.37170004239231335674276124336756765842437744140625f,0.09580000473260863458335734321735799312591552734375f,-0.1695999986395240188841171402600593864917755126953125f,0.268200009095668878700280401972122490406036376953125f,0.3322000334665180520943295050528831779956817626953125f,0.367500058263541262704166001640260219573974609375f,0.2554999892488123425238200070452876389026641845703125f,0.51329998685121491774907553917728364467620849609375f,0.6944000097870823839940612742793746292591094970703125f,0.266900037801266254433585345395840704441070556640625f,-0.4265999933347106587433472668635658919811248779296875f,-0.7690000089108945946492212897283025085926055908203125f,-0.5511999781906606887815769368899054825305938720703125f,-1.07019997130632393833593596355058252811431884765625f,0.4777000075533985867792807766818441450595855712890625f,-0.471799999547004578204223435022868216037750244140625f,0.55759997891038681760988993119099177420139312744140625f,0.3920000136755403818966669859946705400943756103515625f,-0.8857000109583135216695382041507400572299957275390625f,-0.0896000842466957436727170716039836406707763671875f,0.10900000216811889242052302506635896861553192138671875f,0.19140000627040887337670937995426356792449951171875f,0.18299999241530884308559734563459642231464385986328125f,0.454000032603740777403800166212022304534912109375f,0.6550999730244282392987997809541411697864532470703125f,0.6534999736547462134694796986877918243408203125f,0.255400003036856393379139262833632528781890869140625f,0.71690004674196305956002106540836393833160400390625f,0.819299976077676017638395933317951858043670654296875f,0.800399980163574387148628375143744051456451416015625f,0.270999995149672212590985509450547397136688232421875f,-0.111500037372112448252892136224545538425445556640625f,0.8042999926924705444974961210391484200954437255859375f,0.925999999582767685524231637828052043914794921875f,0.28810001461356871121921585654490627348423004150390625f,0.16159998443126699640970400650985538959503173828125f,0.15290001272261111164851854482549242675304412841796875f,0.5802000148177146598982290015555918216705322265625f,-0.3797999877899884690890530691831372678279876708984375f,-0.26879999275207477893445684458129107952117919921875f,-0.2719000043481585304760983490268699824810028076171875f,0.0457999990701676296112054842524230480194091796875f,-0.24760002637058509122880423092283308506011962890625f,-0.1455999970674515253676872816868126392364501953125f,0.8922000399857761276933842964353971183300018310546875f,-0.029400001144409326769846302340738475322723388671875f,-0.54100001293793320567004911936237476766109466552734375f,-0.235200002934038698843011161443428136408329010009765625f,0.102099994709342833942855577333830296993255615234375f,-0.011400014913082401335486792959272861480712890625f,-1.1068000258475543251535100353066809475421905517578125f,-0.6084000230193140623669023625552654266357421875f,0.6339999924078580928465953547856770455837249755859375f,0.197399993932247497241405653767287731170654296875f,0.4434000133335589399763421170064248144626617431640625f,0.16239998350143469707518306677229702472686767578125f,-0.470899983903765306791910916217602789402008056640625f,0.31699999856948846144177878159098327159881591796875f,-0.10080000924020993124941014684736728668212890625f,0.17600001746416094050573519780300557613372802734375f,0.476799982298910851596929205697961151599884033203125f,0.284599993908404957920765809831209480762481689453125f,0.76910001188665610971639807758037932217121124267578125f,-0.4295999924182893892066203989088535308837890625f,0.36279999883174884889314171232399530708789825439453125f,-0.412099972897767230506360647268593311309814453125f,0.532099977964163084465099018416367471218109130859375f,0.463899962908028928865178386331535875797271728515625f,0.4701000090852376356309605398564599454402923583984375f,0.5094000784814358695484770578332245349884033203125f,-0.7243999721989038409475369917345233261585235595703125f,-1.19639998570680550216138726682402193546295166015625f,-0.3382000091701744537431295611895620822906494140625f,0.36309997903704616106779212714172899723052978515625f,-0.55939997488111270484978376771323382854461669921875f,-0.67320004901215402259850861810264177620410919189453125f,0.352299989969283411372913406012230552732944488525390625f,0.930400006967037285221522324718534946441650390625f,0.646699979621172094113035200280137360095977783203125f,1.0841000301778311865064097219146788120269775390625f,-0.740000010281800957301356902462430298328399658203125f,-0.401500030368566740435198880732059478759765625f,-0.363699991601705630017704606871120631694793701171875f,-0.5163000200212000123656252981163561344146728515625f,-0.14549998619407433775307936230092309415340423583984375f,-1.247799955767393331740322537370957434177398681640625f,-0.673399968965351813920960921677760779857635498046875f,-1.6788000031828875791006794315762817859649658203125f,0.39370000080317246737848790871794335544109344482421875f,0.2208000503301617012397173311910592019557952880859375f,0.237599988894164670938380368170328438282012939453125f,-0.2725999990642067327684117117314599454402923583984375f,0.5971000287398695594021091892500407993793487548828125f,-0.0590000016093252810378544381819665431976318359375f,-0.4728999953627586183557696131174452602863311767578125f,-0.003800003778934524234500713646411895751953125f,0.335199990044534157362932091928087174892425537109375f,0.079599976027012520063408373971469700336456298828125f,-0.3516999604731798623191707520163618028163909912109375f,-0.048199987936019539347398676909506320953369140625f,-1.1428000095978376737804182994295842945575714111328125f,-0.6402000204801563398859798326157033443450927734375f,0.20389999341815701061619847678230144083499908447265625f,0.1808000118851662563201898592524230480194091796875f,-0.26270000330656750886504369191243313252925872802734375f,-0.083799983215332218833282240666449069976806640625f,0.12259998546689743204041178614716045558452606201171875f,-0.04239998707771253094733765465207397937774658203125f,0.825900011023878999338876383262686431407928466796875f,0.21239999959468836010501036071218550205230712890625f,-0.77010001903176317217258883829344995319843292236328125f,-0.0395999971866611844006911269389092922210693359375f,-0.2651000000327827788026979760616086423397064208984375f,-0.646399990773200983085189363919198513031005859375f,1.58259999317824817666178205399774014949798583984375f,0.5893999871373178933708913973532617092132568359375f,-0.567699977825582013934990754933096468448638916015625f,0.19180000856220724703149471679353155195713043212890625f,0.606199983666837027840301743708550930023193359375f,0.26339998189210922507896839306340552866458892822265625f,-1.5155999724432824837805355855380184948444366455078125f,1.412799995777011208275553144630976021289825439453125f,0.5623999952360987375499234985909424722194671630859375f,0.5907000160992146309268946424708701670169830322265625f,-0.131699998247623406566475523504777811467647552490234375f,0.019299985092878113590586508507840335369110107421875f,-0.6103999623566871424173996274475939571857452392578125f,-0.956500013381241576126967629534192383289337158203125f,0.6342000054374334894191633793525397777557373046875f,1.18670003728866557679566540173254907131195068359375f,-0.32430000848770124122211200301535427570343017578125f,-0.14309999662041672507939438219182193279266357421875f,-0.33799998958408838944933449965901672840118408203125f,-0.8233000026464463161346429842524230480194091796875f,0.33929999023377899192155382479541003704071044921875f,0.514500021815300190297648441628552973270416259765625f,-0.854199967774749158166969209560193121433258056640625f,-0.9726000029385080125621243496425449848175048828125f,-0.32389999751597631227895135452854447066783905029296875f,-1.30630000124573708575326236314140260219573974609375f,-0.6495000145509834954538064266671426594257354736328125f,-0.0761000853717330727477019536308944225311279296875f,0.181699984426796579128904340905137360095977783203125f,1.18439998356103881604894922929815948009490966796875f,0.818799983252584961945785835268907248973846435546875f,-0.0120999706998470291097191875451244413852691650390625f,0.4951999941051006270953394050593487918376922607421875f,0.63590001217871916505686158416210673749446868896484375f,0.11309999807327979215454405448326724581420421600341796875f,-0.25439999201744789569801241668756119906902313232421875f,0.3694000039383771305523396222270093858242034912109375f,-0.47780002982616398554682746180333197116851806640625f,-1.53889998172074538018705425201915204524993896484375f,-0.9006000030279164292323912377469241619110107421875f,-0.098900026871264035133179959302651695907115936279296875f,-0.33220000120550385869222509427345357835292816162109375f,-0.2882000058919189644512925951858051121234893798828125f,-0.14960000493526504072860916494391858577728271484375f,-0.18470001755952825117645943464594893157482147216796875f,-0.0037999786704780991186680694227106869220733642578125f,1.147099991524219486649371901876293122768402099609375f,0.1815999988853931323973256439785473048686981201171875f,0.82730000514984125548068050193251110613346099853515625f,0.496999991007149255484165450980071909725666046142578125f,-1.834499971840530374578293049125932157039642333984375f,-0.6603999792695045556456534541212022304534912109375f,1.12060000373572155041301812161691486835479736328125f,-0.03240001807212866680174556677229702472686767578125f,-0.290600022436678540049825869573396630585193634033203125f,0.1307999932587147196016985617461614310741424560546875f,-0.075899989230930831052290841398644261062145233154296875f,0.04099999708682300347817317742737941443920135498046875f,0.1532000319689511280696336825712933205068111419677734375f,0.020200014221668283198596327565610408782958984375f,-0.1789999922215936845049100156757049262523651123046875f,0.011199994432926185794485718361102044582366943359375f,-1.03790001503229145640716524212621152400970458984375f,-0.5060000088214877678183256648480892181396484375f,0.2015999972462652056748311224509961903095245361328125f,1.876200039136409980500275196391157805919647216796875f,-0.6290999717652800971023907550261355936527252197265625f,-0.11019999499916988128234152100048959255218505859375f,-0.48020000233799198241513295215554535388946533203125f,0.5524999611265954957417534387786872684955596923828125f,-0.059200001246482170103035969077609479427337646484375f,0.09560004000142185986987897194921970367431640625f,-0.7613999988943336294511254891403950750827789306640625f,-1.304700014765561189022946564364247024059295654296875f,-0.0485000038668512711836910966667346656322479248046875f,0.442099991412460813222651268006302416324615478515625f,0.1874999958649277986832970555042265914380550384521484375f,0.03279999996796245653740697889588773250579833984375f,0.470999994292855272970399482801440171897411346435546875f,-0.107799988810718172516089907730929553508758544921875f,-0.6556999706879260703118461606209166347980499267578125f,0.610899989640712792748900028527714312076568603515625f,-0.109199986624717748640023273765109479427337646484375f,-0.450200009229778874697558421758003532886505126953125f,0.1495999945044517998038458017617813311517238616943359375f,0.040699996932595905452245688138646073639392852783203125f,0.01950000130757689742466709503787569701671600341796875f,0.212799965061992946147029215353541076183319091796875f,0.543799995918571976716293647768907248973846435546875f,0.69160000082254402542503157746978104114532470703125f,-0.380899981035292267694813972411793656647205352783203125f,-0.5077000137746328078947044559754431247711181640625f,-0.61190001489072998186458107738872058689594268798828125f,-0.79760004324615019211108801755472086369991302490234375f,-1.2625000025704504746926204461487941443920135498046875f,-0.461799995374679195236922168987803161144256591796875f,0.16230001297295115758601014022133313119411468505859375f,-0.22960001953840247779226046986877918243408203125f,-0.987400005592406149190765063394792377948760986328125f,-0.536600005590915696274123547482304275035858154296875f,0.31300000496953739315131315379403531551361083984375f,-0.08500002086162528058821408194489777088165283203125f,0.04580002913326008950889445259235799312591552734375f,0.073000001132488279154131305404007434844970703125f,-0.164800025638938141270273263216949999332427978515625f,0.20279997767210033998708240687847137451171875f,0.25640006114393487646196945206611417233943939208984375f,0.09719999699592563047190196812152862548828125f,-0.12709999673962590893694368787691928446292877197265625f,0.2776000054717062681675088242627680301666259765625f,-0.78500001702457655738953690160997211933135986328125f,-0.16059999498128885164760504267178475856781005859375f,0.5610000315457581709921441870392300188541412353515625f,0.044000018596649592694802777259610593318939208984375f,0.1982999830886724712541990811587311327457427978515625f,0.2911999735713006742798825143836438655853271484375f,-0.9599000113293529867775077946134842932224273681640625f,-0.547600025439262783066851625335402786731719970703125f,-0.075099978016317070483864881680347025394439697265625f,-0.3449999867379662799038442244636826217174530029296875f,-0.2826999980166551296179022756405174732208251953125f,-0.36420002506971371758481836877763271331787109375f,0.2634000026792289173016570202889852225780487060546875f,0.5349999666213987037366450749686919152736663818359375f,-0.384099984133243754769182487507350742816925048828125f,0.41649999669194226470381181570701301097869873046875f,-0.7330999723687769975555283963331021368503570556640625f,-0.6562000331759441085210937671945430338382720947265625f,-0.6478000076606866830530861989245750010013580322265625f,0.00949994626641181838522243197076022624969482421875f,-0.50749996811151543152362819455447606742382049560546875f,-0.54020001511573667585253133438527584075927734375f,0.4135999926492572864589192249695770442485809326171875f,0.1873999946773050595538734341971576213836669921875f,-0.0804000034093856896788565791212022304534912109375f,-0.08680000565648082755387804354541003704071044921875f,-0.0436999888822435789137443862273357808589935302734375f,-0.52330003677010505924727112869732081890106201171875f,0.47839999537765987014381607878021895885467529296875f,0.18429998747706388684264311450533568859100341796875f,0.79629998135268686976218077688827179372310638427734375f,1.7123000051498415974293720864807255566120147705078125f,0.723400007522105870094719648477621376514434814453125f,0.67200007084012014502150123007595539093017578125f,-0.6928999714836481249591315645375289022922515869140625f,-1.0760999855339521413810643934994004666805267333984375f,-0.8673000121906395687432222985080443322658538818359375f,-0.15240009190738235123063759601791389286518096923828125f,0.444500000432133557382030630833469331264495849609375f,1.39980002236068212795316867413930594921112060546875f,-0.6910999803930522045192219593445770442485809326171875f,-1.2082999896824357932700877427123486995697021484375f,-0.411499986261129269138336894684471189975738525390625f,-0.09479998488426222280622823745943605899810791015625f,-0.63320000078827121114954934455454349517822265625f,-0.7722000029563904632823323481716215610504150390625f,1.295800000746547997465540902339853346347808837890625f,0.79440000721663228677726920068380422890186309814453125f,-1.353500054195523549793733764090575277805328369140625f,-0.4256000188231467973309918306767940521240234375f,0.4228999892160292262133225449360907077789306640625f,0.06900000399351124968916337820701301097869873046875f,1.4109000239133837606431143285590223968029022216796875f,0.706999992907046959089711890555918216705322265625f,-1.25380001726448586651940786396153271198272705078125f,-0.6650000083446503396089610760100185871124267578125f,-0.496599971392750472176658149692229926586151123046875f,-0.265199970375001592426400520707829855382442474365234375f,0.705099972931295670974805034347809851169586181640625f,0.3160000071823596545783630062942393124103546142578125f,1.2633999967932698638151123304851353168487548828125f,0.4795999854147432461815014903550036251544952392578125f,-1.824500005051493900509740342386066913604736328125f,-0.86860000654458957569659105502068996429443359375f,-0.6551000158280133600641192970215342938899993896484375f,-0.6996000162601472283085968228988349437713623046875f,0.2910000065118072409831029290216974914073944091796875f,-0.361200009334087468459983938373625278472900390625f,-0.88949998117238260331163246519281528890132904052734375f,-0.30199997532367728325652933563105762004852294921875f,0.576699988636374616390867231530137360095977783203125f,0.298200000602006465300064519396983087062835693359375f,0.1696000003159046087830574833787977695465087890625f,1.10989998469352713073021732270717620849609375f,-0.3441999913558364543320067241438664495944976806640625f,-0.152799985625594436644547613468603231012821197509765625f,0.0611000038437545800462658007745631039142608642578125f,-1.055500002231448686274006831808947026729583740234375f,-0.9289000162169334107176155157503671944141387939453125f,-0.8408000413894658464641906903125345706939697265625f,0.69059996029883652379766090234625153243541717529296875f,0.8301000424265847499327719560824334621429443359375f,0.4943999967113137383734056129469536244869232177734375f,0.23029999451041316405053294147364795207977294921875f,-0.88060001371949869763966489699669182300567626953125f,-0.6962000611715029041448588031926192343235015869140625f,0.2121000059038399554456333362395525909960269927978515625f,0.39370002986043728387954843128682114183902740478515625f,-0.2253999955117702569395987666212022304534912109375f,-0.35920001240372645412435304024256765842437744140625f,0.543999978289008367937640286982059478759765625f,0.4426999775648112755987995114992372691631317138671875f,-0.6668999837845566158733845441020093858242034912109375f,0.4037000081419943597893507103435695171356201171875f,-0.3336000104933976029286668563145212829113006591796875f,0.95759998550415037499305981327779591083526611328125f,-0.2823000073477623939055547452881000936031341552734375f,-0.36170001314878508225092446082271635532379150390625f,-0.31259998833537105955571178128593601286411285400390625f,1.028199964243173614164561513462103903293609619140625f,-0.410100044140219832655702703050337731838226318359375f,-0.4571999965488908745925300536327995359897613525390625f,1.018100002878904231096157673164270818233489990234375f,0.2871999992311000671207921186578460037708282470703125f,-0.32699998988956224810209505449165590107440948486328125f,-0.31740001151561703096604105667211115360260009765625f,-0.0697000042602418101722605570103041827678680419921875f,0.0955999848484996306297034607268869876861572265625f,-0.0317999945923685967130722929141484200954437255859375f,0.0151999866545200479350796740618534386157989501953125f,-1.137800010752677781766806219820864498615264892578125f,-0.3438000019907949678099612356163561344146728515625f,0.52879999522939324929637905370327644050121307373046875f,0.001799998019635651314018787161330692470073699951171875f,-0.0393999802269043575275730972862220369279384613037109375f,-0.33839999873042092559671800700016319751739501953125f,-0.4018000021919607700482401924091391265392303466796875f,-0.121600002908706983362208120524883270263671875f,0.67029999704360943013625728781335055828094482421875f,0.1751999942541129229311991366557776927947998046875f,-0.944599983850121116546461053076200187206268310546875f,0.09440001220852101138092393739498220384120941162109375f,0.1552999512225385903008145760395564138889312744140625f,-0.210200007665157340142059183563105762004852294921875f,0.5844000318780537828189380888943560421466827392578125f,-0.19500000059604616353681194595992565155029296875f,0.1447000052288174298809053652803413569927215576171875f,0.26339999433457872424213519479962997138500213623046875f,0.78340003147572290220779223091085441410541534423828125f,0.54879998016059428067592307343147695064544677734375f,-0.26539999983310702535987957162433303892612457275390625f,0.30900000227987778345095648546703159809112548828125f,0.470799984392523906961969260009936988353729248046875f,0.7376999938070774032183862800593487918376922607421875f,-0.806199975545704550228265361511148512363433837890625f,-1.06440000278353696927524651982821524143218994140625f,-0.59659996990263497451678631477989256381988525390625f,-0.942399987152218177044460389879532158374786376953125f,-0.40129999129921201106441230876953341066837310791015625f,-0.9847999918878083125406419640057720243930816650390625f,-0.912599968926608884345341721200384199619293212890625f,-0.66669999830350212821628019810304976999759674072265625f,-0.08929999982640145217427374291219166480004787445068359375f,0.49589995993152313769058991965721361339092254638671875f,-0.02359999894499775763279103557579219341278076171875f,0.24619998422563060813672564108856022357940673828125f,0.4542999750718477169897369094542227685451507568359375f,1.10849997052550275355997655424289405345916748046875f,0.54390001070350368905792493023909628391265869140625f,-0.320699912852793811435248017005505971610546112060546875f,-0.0463000013016163780577016950701363384723663330078125f,0.225199989020079538448726452770642936229705810546875f,0.416299962204695062073511735434294678270816802978515625f,1.629799969983100726977909289416857063770294189453125f,0.2146999947607517444936320316628552973270416259765625f,0.267399996130168293717588312574662268161773681640625f,0.344299998801946560700315558278816752135753631591796875f,-0.06439998199641683296334804254001937806606292724609375f,0.0104000114187596803816404644749127328395843505859375f,-0.36800000195205206932058672464336268603801727294921875f,-0.0389000087104738323517949538654647767543792724609375f,0.047399989871680314745816531285527162253856658935546875f,0.776499982479959438563810181221924722194671630859375f,0.33859997614622105999160339706577360630035400390625f,-1.7317000034630301019689113672939129173755645751953125f,-0.74700000631809171380837142351083457469940185546875f,-0.3692999960824845384621539778891019523143768310546875f,-0.54120000516176247629118734039366245269775390625f,0.2224999992176888152783931218436919152736663818359375f,-0.326400001353025270844909755396656692028045654296875f,0.46110002166926877809061124935396946966648101806640625f,0.2782000128209591860439786614733748137950897216796875f,0.29359996523112019684020879140007309615612030029296875f,0.050999964714050438630010830820538103580474853515625f,-1.2651999914228915233849193100468255579471588134765625f,-0.43539999506473492374425404705107212066650390625f,0.023100030632317081202842246057116426527500152587890625f,-0.52359999779015786725011594171519391238689422607421875f,0.147499978020787236854260981999686919152736663818359375f,0.1045999876499177361210968228988349437713623046875f,0.0672999890193344274536002558306790888309478759765625f,0.204999985694885200615544817992486059665679931640625f,0.6017999883338809308241934559191577136516571044921875f,0.66320000231564002479700548065011389553546905517578125f,-0.3016999994024633924283307351288385689258575439453125f,-0.148399998879432448717352599487639963626861572265625f,1.218300023172795842896221074624918401241302490234375f,-0.1438000634208320693829818992526270449161529541015625f,0.16589999411106115534408900202834047377109527587890625f,0.5400000014714894103207143416511826217174530029296875f,0.479799967096001311883668449809192679822444915771484375f,0.8344999831281594282472724444232881069183349609375f,0.82069996521174937154086137525155209004878997802734375f,0.8026000007033342598106173682026565074920654296875f,-0.78849997842311891904643061934621073305606842041015625f,0.10330000964999186408022069372236728668212890625f,0.3833999804764987029415124197839759290218353271484375f,1.7238000037044287626741834174026735126972198486328125f,-0.09560000161230564064140935442992486059665679931640625f,0.080699995386600331670479135937057435512542724609375f,0.55500000510364755346159881810308434069156646728515625f,0.7677000207781798923889482466620393097400665283203125f,-0.459600012646615263545157858970924280583858489990234375f,-0.887700036424399652190686538233421742916107177734375f,0.6880000034794206964505747237126342952251434326171875f,2.273400052169711305083410479710437357425689697265625f,0.0003999988831580036929835841874592006206512451171875f,0.372500009927898556583159006549976766109466552734375f,-0.5994999876916409231597526741097681224346160888671875f,-1.2927999842286113096889721418847329914569854736328125f,-0.4376999920934439014530425993143580853939056396484375f,-1.4786999774828555320738132650149054825305938720703125f,0.84349996814131766864619521584245376288890838623046875f,0.5533000175848596935423984177759848535060882568359375f,-0.70079997224807766276200027277809567749500274658203125f,-0.418900018340348623269164818339049816131591796875f,0.2044000125065446038519212379469536244869232177734375f,-0.15619999741315826469190142233856022357940673828125f,0.4171999990448356232519699915428645908832550048828125f,0.283800008547306248374297865666449069976806640625f,0.38629997561573958808622819560696370899677276611328125f,-0.064400034821033624865549427340738475322723388671875f,0.4636000004351139569536144335870631039142608642578125f,0.6423999888658524781703818007372319698333740234375f,-1.2335999841183420411283577777794562280178070068359375f,-0.63480000783205081660298674250952899456024169921875f,1.3599000125586988563242130112485028803348541259765625f,0.611000012993812458006459564785473048686981201171875f,-1.00710000448822967200612765736877918243408203125f,-0.49680000580549243949235460604541003704071044921875f,-0.5630000247880817543233433752902783453464508056640625f,-0.2209999964535234173723665662691928446292877197265625f,0.789099993073940186860681933467276394367218017578125f,0.2927999836325643201462298748083412647247314453125f,0.2501000257745384924845666319015435874462127685546875f,0.051800012958050256628439456108026206493377685546875f,-1.2541000139728188944587827791110612452030181884765625f,-0.76880002642869982310003251768648624420166015625f,-0.3820999981924890764872770887450315058231353759765625f,-0.16299998861551312501205757143907248973846435546875f,-0.255500029407441819895296930553740821778774261474609375f,-0.52520000319480875106137318653054535388946533203125f,0.47990000648647512804245707229711115360260009765625f,0.278200013864040585076509159989655017852783203125f,0.2558999921739102401119225760339759290218353271484375f,-0.162599995934963548194218674325384199619293212890625f,-0.2293999932095409644006167582119815051555633544921875f,-0.47829997537732094059492737869732081890106201171875f,-0.4457999833121897292897983788861893117427825927734375f,-0.48000000551342925092512814444489777088165283203125f,-0.5846000146210190706597131793387234210968017578125f,-1.431400013422967187892709262087009847164154052734375f,0.210400005793571320822366033098660409450531005859375f,-0.133799971145391882743069800199009478092193603515625f,-0.21580000011324873554485748172737658023834228515625f,0.355099989229440637927837087772786617279052734375f,0.7749000035434960853564234639634378254413604736328125f,0.71830000294446971764728004927746951580047607421875f,-0.1672999957248568758672035983181558549404144287109375f,0.090399997076392235584307854878716170787811279296875f,0.0634999946653842239907561406653258018195629119873046875f,0.3041000425457955902430740025010891258716583251953125f,-0.6785000061020252104526662151329219341278076171875f,-1.0064000788390641272229686364880762994289398193359375f,0.8455000101476903839881060775951482355594635009765625f,1.5175000515580183702013528090901672840118408203125f,-0.82840001221597159286602618522010743618011474609375f,-0.636500025898218968478659007814712822437286376953125f,-0.269900001159310232168309084954671561717987060546875f,0.335499967306852386883520011906512081623077392578125f,-0.257400004586577413334680386469699442386627197265625f,-0.1378999901473516587202539085410535335540771484375f,-0.51779999424964184839836889295838773250579833984375f,-0.970600004816055683676268017734400928020477294921875f,0.131600020492077141032183362767682410776615142822265625f,-0.150400027549267178983427584171295166015625f,-0.91250003438442994774248973044450394809246063232421875f,0.140400017863512527327429779688827693462371826171875f,-0.3637999939441680652407740126363933086395263671875f,-0.54700000274181359571912253159098327159881591796875f,0.03070004154294736498087559084524400532245635986328125f,-0.40179997421503088617100729607045650482177734375f,-0.7417000143408774004427641557413153350353240966796875f,-0.8496000076174734516598618938587605953216552734375f,1.1875999866589903586344689756515435874462127685546875f,0.48399998376518504894505667834891937673091888427734375f,0.1503000065058468504020083855721168220043182373046875f,0.71060001494884517114769550971686840057373046875f,0.24450001477450111764255780144594609737396240234375f,0.418800019872188844516358585678972303867340087890625f,1.0849000426217916270843488746322691440582275390625f,0.731800021302699832403959590010344982147216796875f,-0.5768000213026998057586069990065880119800567626953125f,0.252400016582011943455654545687139034271240234375f,0.21609998035579902619929271168075501918792724609375f,-0.07639999636113647962787354117608629167079925537109375f,1.116700033600628305663349237875081598758697509765625f,0.3987999974459406349325263363425619900226593017578125f,-0.914700028884410709650865101139061152935028076171875f,-0.03780000121593474204928497783839702606201171875f,-0.92969999733120201046432384828221984207630157470703125f,-0.81199999737739592120533416164107620716094970703125f,0.3215999764591455090112503967247903347015380859375f,-0.1946000021040443783704176894389092922210693359375f,-0.055299999986588854739011367200873792171478271484375f,-0.9119000080734485091937813194817863404750823974609375f,0.0437999917834996921328638563863933086395263671875f,0.4620999991983172350273889605887234210968017578125f,-0.14410000588893889794661617997917346656322479248046875f,0.9547999814569951837484040879644453525543212890625f,0.4592999938100577050903439157991670072078704833984375f,1.8286999828100203302483350853435695171356201171875f,-0.8208999766394498021071512994240038096904754638671875f,-0.262500004041939571930441843505832366645336151123046875f,0.10240000099167223535090442965156398713588714599609375f,-1.296099985105544138974664747365750372409820556640625f,0.78289999200999726980398918385617434978485107421875f,-0.0540999726593487384462832778808660805225372314453125f,0.2014000005334615206464832226629368960857391357421875f,-0.42509999712705592855854774825274944305419921875f,0.66020000181645155379328571143560111522674560546875f,-0.59919997872710251130001779529266059398651123046875f,0.6045999884694814863195233556325547397136688232421875f,0.857200017745793019230404752306640148162841796875f,0.33279997990727439027835998786031268537044525146484375f,0.7772000139459962486654376334627158939838409423828125f,0.02050000739097601609728371840901672840118408203125f,-0.98399999814480576532105260412208735942840576171875f,-0.293799989287555352035496980533935129642486572265625f,0.1372000010192397922992313397116959095001220703125f,-0.72930000003129225039089078563847579061985015869140625f,-0.451000020891428210489948469330556690692901611328125f,0.31960001205056898232470530274440534412860870361328125f,-0.2497999982208012337281388681731186807155609130859375f,-0.09819999717473966438507915199807030148804187774658203125f,0.0684000075221060654939719825051724910736083984375f,-0.419300016385316831613039312287583015859127044677734375f,-0.09059998097419708074085065163671970367431640625f,1.733000033579767062974497093819081783294677734375f,0.38900000429153447356611650320701301097869873046875f,-1.0406000546231866099589069563080556690692901611328125f,0.1103999825477597340750435250811278820037841796875f,1.386400014168023719918210190371610224246978759765625f,0.6756000113725662004071637056767940521240234375f,-0.062599975073337699171815984300337731838226318359375f,-0.132200004446506369504277245141565799713134765625f,-1.22839999940097310826558896224014461040496826171875f,-0.674800021541118955070714946486987173557281494140625f,0.1451000036835672357682369693065993487834930419921875f,-0.2002000157117844736376355285756289958953857421875f,-0.218500009007752016287895457935519516468048095703125f,0.1872000069797039589758469446678645908832550048828125f,0.05459999178498975425100070424377918243408203125f,0.158200001645087962032221184927038848400115966796875f,-0.685900008416175932524083691532723605632781982421875f,-0.48459999986886970191335421986877918243408203125f,0.76330000197589387145313821747549809515476226806640625f,0.3058000074148179425748139692586846649646759033203125f,-0.2440000047385693926482730375937535427510738372802734375f,0.133400010800361901175392631557770073413848876953125f,0.6510000093430281520312519205617718398571014404296875f,-0.17770001053363027043729971410357393324375152587890625f,-0.0321000008001923742284589025075547397136688232421875f,-0.10449997834116242501067972625605762004852294921875f,-0.2510999916806817555681163867120631039142608642578125f,-0.8195999879479405336724084918387234210968017578125f,-0.77259997444003847277826935169287025928497314453125f,-0.29150002306699729359706907416693866252899169921875f,0.78310001167058918003505141314235515892505645751953125f,-0.27129991943836184731253524660132825374603271484375f,-0.5856000143527981816049532426404766738414764404296875f,-1.02380002851486207049447330064140260219573974609375f,0.6091999801427128158337609420414082705974578857421875f,0.57989996383190156592490893672220408916473388671875f,-0.1257999938920140170495187703636474907398223876953125f,0.2153999864220612625587136790272779762744903564453125f,-0.4990999941542744178235579965985380113124847412109375f,0.4115999859213825828874178114347159862518310546875f,-0.3987999816134573105586014207801781594753265380859375f,-0.82200000832974862152724426778149791061878204345703125f,0.22519999663829803893122516456060111522674560546875f,-0.0718999903410673102399641720694489777088165283203125f,0.44429997496008899471320319207734428346157073974609375f,0.70719999386668153107393663958646357059478759765625f,-0.3054000002428889359862296259962022304534912109375f,0.00310000138878840925116264770622365176677703857421875f,0.0900000001117586823085048308712430298328399658203125f,-0.6922999839901919916229644513805396854877471923828125f,-0.764699982616305451443849960924126207828521728515625f,-1.02400002524629218925156237673945724964141845703125f,-0.74400000283867095962619941929006017744541168212890625f,-0.5568000031232838153982811491005122661590576171875f,-0.10910002615451841467830718102050013840198516845703125f,-0.46460001149177543311452609486877918243408203125f,-0.4553000350043177046899245397071354091167449951171875f,-0.0654000102639198388487784541212022304534912109375f,0.29999999314546543782711296444176696240901947021484375f,0.1396000287771226311406280728988349437713623046875f,0.965000021904706795083939141477458178997039794921875f,0.4408000357270243085849870112724602222442626953125f,-0.4458999980226157067164649561163969337940216064453125f,-0.508399996495246586647454023477621376514434814453125f,0.6028000017747283667546298602246679365634918212890625f,0.08439999130964270790400405530817806720733642578125f,0.25230000712424516251530803856439888477325439453125f,0.0975999944448471268287903512828052043914794921875f,0.062499999888241120071796785850892774760723114013671875f,0.09239998707771324149007341475225985050201416015625f,-0.7580999879777430283667172261630184948444366455078125f,-0.104600000464916131903692075866274535655975341796875f,0.045099990011751633911529779652482829988002777099609375f,-0.003600003266334805829274046118371188640594482421875f,0.77759999805837853870116305188275873661041259765625f,0.310799982678890529541604337282478809356689453125f,0.4712000146612524975608948807348497211933135986328125f,0.84420000420808793961668925476260483264923095703125f,-0.7186000100836154036443303994019515812397003173828125f,0.082399997210502728961500906734727323055267333984375f,-0.0534000095710156497119669438689015805721282958984375f,0.25499999701976694410632262588478624820709228515625f,0.36239998111724869289673733874224126338958740234375f,1.644100003653764741073928234982304275035858154296875f,-0.51220000537782894856064785926719196140766143798828125f,-0.73989997977614496704745761235244572162628173828125f,-0.4439000034391880067374813734204508364200592041015625f,-0.7994999924600119101114614750258624553680419921875f,0.4646999736383559120866948433103971183300018310546875f,0.36450000333786025663584950962103903293609619140625f,0.33449999957531684202649557846598327159881591796875f,0.47369999815821639543855781084857881069183349609375f,-0.625499971739948090743155262316577136516571044921875f,-0.174899981564282658297315720119513571262359619140625f,-0.63439998698830601941978102331631816923618316650390625f,-0.842600041681528022508018693770281970500946044921875f,0.20990000112205742244242401284282095730304718017578125f,0.276299984407425203158936710678972303867340087890625f,0.17109999890774485464817189495079219341278076171875f,-0.190599990138411612417002061192761175334453582763671875f,-0.259499988473951948009244006243534386157989501953125f,-0.20530000207275145296392793170525692403316497802734375f,0.6705999709159138344460870939656160771846771240234375f,1.09440001187324487119667537626810371875762939453125f,-0.4253999832183124052420453153899870812892913818359375f,-1.860899987256526610934770360472612082958221435546875f,0.2110000029727816528435369036742486059665679931640625f,-1.2131999709486966931848428430384956300258636474609375f,-0.659899970574677308121636087889783084392547607421875f,-0.402800005686283046912876670830883085727691650390625f,-0.6100999720558524241909026386565528810024261474609375f,-0.6278000034511086457911233082995750010013580322265625f,-0.2040000144243239166286230101832188665866851806640625f,0.206200004565715744320186786353588104248046875f,1.12940000986158839424433608655817806720733642578125f,0.4485999949216843418753342120908200740814208984375f,-0.055600007833540576740460892324335873126983642578125f,0.1541999865502121647153899175464175641536712646484375f,-1.1993000028252602806588811290566809475421905517578125f,-0.624200023281574534195215164800174534320831298828125f,0.324899972139298665041451386059634387493133544921875f,0.49600000494718532451088321977294981479644775390625f,0.6533000224277378809034644291386939585208892822265625f,-0.1370000164508820006403766456060111522674560546875f,-1.0956000169232489316328837958280928432941436767578125f,-0.47120001171827308183992499834857881069183349609375f,0.1102000154510143214992012872244231402873992919921875f,-0.36940002498626700599970718030817806720733642578125f,-0.621400004370510572471175692044198513031005859375f,0.021400009846687328973757757921703159809112548828125f,0.82890000150203702045104137141606770455837249755859375f,0.358399999773502120348211974487639963626861572265625f,-0.777299988833069388505236929631792008876800537109375f,-0.0243999674677848332748908433131873607635498046875f,1.056000009305775311219122158945538103580474853515625f,0.1136000053524970354601464350707828998565673828125f,-1.3127000310972334329306931977043859660625457763671875f,-0.54840000513792031000548377051018178462982177734375f,0.0207999677404762106647240216261707246303558349609375f,1.110399987688660328188916537328623235225677490234375f,-0.5007999775752427495945084956474602222442626953125f,-0.73709997625052903202913512359373271465301513671875f,-0.2346999961763620579091593754128552973270416259765625f,-0.40599999943375575384152398328296840190887451171875f,-0.187199984814226727536379257799126207828521728515625f,-0.84370001779049597079307432068162597715854644775390625f,0.008400002828240327179543101010494865477085113525390625f,0.0417000055864453766929500488913618028163909912109375f,-0.76210001022517648294751779758371412754058837890625f,-1.192900018012524210320179918198846280574798583984375f,0.4819999667555097122573215528973378241062164306640625f,1.07609998180866117678533555590547621250152587890625f,0.441900001516938090162511798553168773651123046875f,0.75730002154111897283428334048949182033538818359375f,-0.40030000259429210363038009745650924742221832275390625f,0.609399987286328492785969501710496842861175537109375f,-0.62839997038096218062719344743527472019195556640625f,-1.0940999866664402606630801528808660805225372314453125f,-0.17139999483376744393581248004920780658721923828125f,-0.211000005245208566151404738775454461574554443359375f,0.406999984338879794432131120629492215812206268310546875f,-0.341200000244379797464944203966297209262847900390625f,0.66580000704228847752119690994732081890106201171875f,1.595800032299757642562099135830067098140716552734375f,0.41260000204443925131414516727090813219547271728515625f,0.30940001842975650703237988636828958988189697265625f,-0.89609999659806482963375628969515673816204071044921875f,-1.01599999049305989018421314540319144725799560546875f,-1.0063000144891438569771935362950898706912994384765625f,-0.6738000112295150501040552626363933086395263671875f,0.07510000275224448795796661215717904269695281982421875f,-0.569400021111964971254337797290645539760589599609375f,-0.5979999923035499431733796882326714694499969482421875f,-0.126399993753433026455468279891647398471832275390625f,-0.066500012524425944349104611319489777088165283203125f,-0.240800003838538945188929574214853346347808837890625f,0.29610000457018614827831015645642764866352081298828125f,0.1258000021994114536738607057486660778522491455078125f,-0.185599988778680724532677004390279762446880340576171875f,-0.250599997179210054465414714286453090608119964599609375f,-0.81039999638721338381941450279555283486843109130859375f,-0.3515999906897544491357621154747903347015380859375f,0.58249998193234187038314075834932737052440643310546875f,0.02039998522996899765757916611619293689727783203125f,-0.090599992038309817843355631339363753795623779296875f,0.35860002382993716452119770110584795475006103515625f,0.597700016196072159146979174693115055561065673828125f,0.316800023090839655282024978077970445156097412109375f,-0.897700001034140715461262516328133642673492431640625f,-0.79780001105070130762442204286344349384307861328125f,0.23409998830556888460563413900672458112239837646484375f,0.2870000047534702058982247763196937739849090576171875f,0.62689999771714244314324560036766342818737030029296875f,-0.172200033354759085568730370141565799713134765625f,-0.0743000200733543447739748444291763007640838623046875f,0.266399970805645036620035170926712453365325927734375f,0.490100053341687047492314377450384199619293212890625f,0.407300001033394920568753150291740894317626953125f,-0.312699991720915004833614148083142936229705810546875f,1.1698000024676324937189519914682023227214813232421875f,-0.8103999759539963765320180755225010216236114501953125f,-0.9389999950528145067352170372032560408115386962890625f,-0.49909996703416148466914137316052801907062530517578125f,-0.29849999272823257712161648669280111789703369140625f,-0.270899989752471592741045469665550626814365386962890625f,-0.1551999940305945901997120017767883837223052978515625f,0.1837999999418854224497721361331059597432613372802734375f,0.48229998479113034282050875845015980303287506103515625f,0.048099998054653425594295867995242588222026824951171875f,0.20269999822154627366899148910306394100189208984375f,-0.31479999024868021706424769945442676544189453125f,0.22040000072717713663905669818632304668426513671875f,-0.51710000355690699525013087622937746345996856689453125f,-1.511000022828578703837365537765435874462127685546875f,-0.12899999807029960408044644282199442386627197265625f,-0.5956000282108784293910730411880649626255035400390625f,0.85930000256448957340893457512720488011837005615234375f,1.569000027835369959205991108319722115993499755859375f,-0.840500003367662174014185438863933086395263671875f,-0.89540001439899297164259905912331305444240570068359375f,-0.0204000003546476182947344568674452602863311767578125f,0.5428999998703591955973024596460163593292236328125f,-0.5448999682277444467359828195185400545597076416015625f,-1.01589997056722580737186945043504238128662109375f,-0.52929999451786269304420784465037286281585693359375f,-1.4195000202693048851898538487148471176624298095703125f,0.0714000177815556913429873020504601299762725830078125f,0.06559998872280115023158941767178475856781005859375f,-0.040900002083182673828787301317788660526275634765625f,-0.1242000163525338596315350514487363398075103759765625f,-1.11049997874349326565379669773392379283905029296875f,-0.24040000095069380403600689533050172030925750732421875f,-0.132700004498660373286611502408050000667572021484375f,-0.41560002329349554628379337373189628124237060546875f,0.1400999863237142317728967100265435874462127685546875f,0.209399991607666091653072726330719888210296630859375f,0.22770001913905135726423623054870404303073883056640625f,0.0551999975323678171434949035756289958953857421875f,-1.259699986080824896106378218973986804485321044921875f,-0.584599990034103456082448246888816356658935546875f,-0.713800035779178276840184480533935129642486572265625f,-0.69460000821351997046804171986877918243408203125f,0.52630000359266981835304477499448694288730621337890625f,-0.154200016464293432338905631695524789392948150634765625f,0.0336999953828752418782954691778286360204219818115234375f,0.17820000529587265436504139870521612465381622314453125f,0.350799985659122182113378585199825465679168701171875f,0.37879999930858598844451989862136542797088623046875f,-0.6494999910444019253219494203221984207630157470703125f,-0.22179999060630795071347165503539144992828369140625f,0.1941000002637507382274861811310984194278717041015625f,0.015800027978420505547774155274964869022369384765625f,0.9567999838262795098131618942716158926486968994140625f,0.184799981866777096595200191586627624928951263427734375f,-0.74190000183358773000463770586065948009490966796875f,-0.50119999927580327625520340006914921104907989501953125f,-0.419999993182718522888308143592439591884613037109375f,-0.589999991375953047878510915325023233890533447265625f,-0.285299977374076974001582129858434200286865234375f,-0.7259999858736991829033513567992486059665679931640625f,-0.11579999676048746692913482547737658023834228515625f,0.1242000018239022285371220277738757431507110595703125f,0.633599977040290696095325984060764312744140625f,-0.77319998495578801112060318700969219207763671875f,0.1047000351428977182877133600413799285888671875f,0.32320002772212053088196626049466431140899658203125f,0.8533999905347826597790117375552654266357421875f,0.83030000285506222468256964930333197116851806640625f,0.2211999796807759821604122407734394073486328125f,0.3158999996244908903264558830414898693561553955078125f,-0.272200005042552817258183495141565799713134765625f,0.114999970048665911548368967487476766109466552734375f,-0.8594999876916400438631171709857881069183349609375f,-0.43720000221133215045909992113593034446239471435546875f,-0.613799999308586308188750990666449069976806640625f,-0.05149999460577969756513994070701301097869873046875f,0.76379997852146619408131300588138401508331298828125f,-0.226000026851892243939801119267940521240234375f,-0.072699961471556662218063138425350189208984375f,-0.919000014424323996564680783194489777088165283203125f,-0.1841999975025654379123807302676141262054443359375f,-0.294299999211728557924061533412896096706390380859375f,-0.658299986366927214476163499057292938232421875f,-0.1914000256791712484272238725679926574230194091796875f,0.1473999954223632613548033987171947956085205078125f,0.2162000266194343822689916123636066913604736328125f,-0.95739997083544725597903379821218550205230712890625f,0.47440000308156005104365249280817806720733642578125f,-0.582599968963860792570130797685123980045318603515625f,0.36999998509883891273375411401502788066864013671875f,0.9078999688386915067894733510911464691162109375f,-0.440000005066394805908203125f,-0.181600058192014746083486897987313568592071533203125f,1.079000006377696774251262468169443309307098388671875f,0.59500002950429919934549616300500929355621337890625f,0.04959998914003360681590493186376988887786865234375f,0.37790000910907906028768366013537161052227020263671875f,-0.43979999032318606300151486721006222069263458251953125f,0.081000012639909524381209848797880113124847412109375f,-0.349999993592500668881939418497495353221893310546875f,0.071300029706955481145769226714037358760833740234375f,0.019599998378753813454977716901339590549468994140625f,-0.73240002775788326516703818924725055694580078125f,0.34260000025629988584796592476777732372283935546875f,0.43740006009340248738226364366710186004638671875f,-0.6980000067949294617619671043939888477325439453125f,-1.2762000255763528144825613708235323429107666015625f,0.7134000032007694613866988220252096652984619140625f,-0.6632999843180182875101991157862357795238494873046875f,-0.1123999848425387693140464762109331786632537841796875f,0.20770002793073683022839759360067546367645263671875f,0.06500000223517421460428522550500929355621337890625f,-0.647699986505508196188429792528040707111358642578125f,-0.87209997379183779031563972239382565021514892578125f,1.1873999858856194578038412146270275115966796875f,0.39619998832344993644483110983856022357940673828125f,-0.037300020349025686527966172434389591217041015625f,1.412299978476763140378125171991996467113494873046875f,-0.613399996495246568883885629475116729736328125f,0.2260999874711036028429589350707828998565673828125f,-0.655999981701373968689949833787977695465087890625f,0.3657999975800512171275613582110963761806488037109375f,0.1381999351859093394523370079696178436279296875f,0.190100005322694887155421383795328438282012939453125f,-0.3616000130414960267444257624447345733642578125f,0.07760004533231235068768683049711398780345916748046875f,0.401899981586634869046292806160636246204376220703125f,-0.6789000074066222634172618199954740703105926513671875f,0.023500015079974900800152681767940521240234375f,-0.680800010395049781664056354202330112457275390625f,0.82360001906156554696281091310083866119384765625f,0.013500024020671741453725189785473048686981201171875f,0.103699975657463028255733661353588104248046875f,-0.9894000194728373998032111558131873607635498046875f,0.24280003638267544374684803187847137451171875f,-0.253499993026256476014168583787977695465087890625f,0.3804999799728392417819122783839702606201171875f,0.6052000285267826740209784475155174732208251953125f,-1.0390999878585347460102639161050319671630859375f,-0.9178999937236309580868010016274638473987579345703125f,-0.176899991309642956593961571343243122100830078125f,-0.62800001096725477367499479441903531551361083984375f,0.277199985074996657630208574119023978710174560546875f,-0.004000015318393668195540158194489777088165283203125f,0.1113999886870384870007910649292171001434326171875f,0.60700001794099822660655263462103903293609619140625f,0.24190005329847341641880120732821524143218994140625f,-0.49620000277757636553133124834857881069183349609375f,-0.5870000124275680519758680020458996295928955078125f,0.361599997991323363066840101964771747589111328125f,1.10809998693466216224123854772187769412994384765625f,0.03960000225305559951038958388380706310272216796875f,-0.8558000270843511003704406903125345706939697265625f,0.0100000110268592123929920489899814128875732421875f,0.88549999506026555451398962759412825107574462890625f,-0.4606000056728720704057877810555510222911834716796875f,-0.6094000050745906893467918052920140326023101806640625f,-0.26060002418756500475183202070184051990509033203125f,-0.35269999172091548444996078615076839923858642578125f,-0.54160001706480986882752404198981821537017822265625f,-0.31700001287460377596971738967113196849822998046875f,0.33140000507831590681462330394424498081207275390625f,-0.0031000104784961735049364506267011165618896484375f,0.1045999951004983330449249478988349437713623046875f,0.65579997150301938546590463374741375446319580078125f,0.622800002259015883510073763318359851837158203125f,1.5048999931871893664947492652572691440582275390625f,0.2030000057518481781926311668939888477325439453125f,-0.296399989730119717279421820421703159809112548828125f,0.359199984091520096995964195230044424533843994140625f,-0.0860999844908718614533427171409130096435546875f,0.227099970960617181248153428896330296993255615234375f,-0.65619998549222913197809248231351375579833984375f,-0.126399977660179185789957045926712453365325927734375f,0.3835999925374977692626998759806156158447265625f,-0.5556000121176243311538200941868126392364501953125f,-0.20339996252059933112832368351519107818603515625f,0.620399988955259384937335198628716170787811279296875f,0.6398999703884129530706559307873249053955078125f,0.1792000021219255501137013197876513004302978515625f,0.8181999944925308909660088829696178436279296875f,1.19660000380277597997746852342970669269561767578125f,-0.827200011599064310985340853221714496612548828125f,-0.95789999275505532949637199635617434978485107421875f,-0.482599979804456236109899691655300557613372802734375f,-0.9965000045895575642163066731882281601428985595703125f,-0.38920000212192462640814483165740966796875f,-0.02229999844431862499050112091936171054840087890625f,-0.115699994492530322531820274889469146728515625f,0.49949997532367707009370860760100185871124267578125f,-0.11779999167919186220387928187847137451171875f,0.79619998474717146308421433786861598491668701171875f,-0.94979998071193705300174769945442676544189453125f,0.99879999290108667509002771112136542797088623046875f,0.25749995529651670267412555404007434844970703125f,-0.290500011265277802152695585391484200954437255859375f,0.72349998795986181221451261080801486968994140625f,0.25049999472498907238104948191903531551361083984375f,0.567299961042404987665577209554612636566162109375f,0.2692999819636343605822048630216158926486968994140625f,-1.120300008033215721781061802175827324390411376953125f,-0.37760000040531149778644248726777732372283935546875f,-0.4286999990522859516062226248322986066341400146484375f,-0.0021999995291233087613136376603506505489349365234375f,-0.542299988311528924356252900906838476657867431640625f,0.13759998878240597974809134029783308506011962890625f,1.013999942600727788288850206299684941768646240234375f,-0.33479999099373802806667299591936171054840087890625f,-0.277900005048512976912888916558586061000823974609375f,-0.725800024253130171558723304769955575466156005859375f,-0.277700015264750188492826055153273046016693115234375f,0.404800014835596133622175329946912825107574462890625f,1.078000006943941269099696000921539962291717529296875f,-0.068999997884035213502329497714526951313018798828125f,-0.6082999656915664132839083322323858737945556640625f,0.3068000183224679489057962200604379177093505859375f,0.205999966204167161976101851905696094036102294921875f,0.331800010871887263164126125047914683818817138671875f,1.1738000112295150501040552626363933086395263671875f,0.26460000940561290150299100787378847599029541015625f,0.6838000207662577878409138065762817859649658203125f,-0.853799994242191218063453561626374721527099609375f,-1.73419999198913554749879040173254907131195068359375f,0.39119998906850828035430822637863457202911376953125f,0.365100011059641627042537948000244796276092529296875f,0.11840000163614750050555812777020037174224853515625f,-0.841699975784122589317348683835007250308990478515625f,0.214599988171458366803534545397269539535045623779296875f,1.1300999828964470861336621965165250003337860107421875f,-0.35559999576359990758334106430993415415287017822265625f,-0.9709999676942828727987944148480892181396484375f,-0.10319998629689219882266115746460855007171630859375f,-0.25290000087618835777902859263122081756591796875f,1.1614999519884587897422534297220408916473388671875f,-0.970199980247020477008845773525536060333251953125f,-1.008700000542402275272024780861102044582366943359375f,-0.6180999613046651575132273137569427490234375f,-0.65729997773170456554225893341936171054840087890625f,-0.27410000022649683160125277936458587646484375f,0.135999985426664249388295502285473048686981201171875f,0.585899985468387285436620004475116729736328125f,-0.611799988818168660742458087042905390262603759765625f,0.9270999873518945832984172739088535308837890625f,1.099299981218576416353016611537896096706390380859375f,0.74180000073909724278564681299030780792236328125f,1.025200019288063035816094270558096468448638916015625f,-0.16080001009702726832983898930251598358154296875f,0.532100036972761092357586676371283829212188720703125f,0.8441999699354170161313959397375583648681640625f,-0.90249998763203631568785567651502788066864013671875f,-0.074199958610535077241365797817707061767578125f,-0.290300013881921614711245638318359851837158203125f,-0.6067999876260756764168036170303821563720703125f,-0.387299995166063393270405867951922118663787841796875f,0.20939999868571756991286747506819665431976318359375f,0.5822999802276493941377566443406976759433746337890625f,-0.51279998612850885564284908468835055828094482421875f,-0.89079997642040265048990477225743234157562255859375f,-1.4906999928534023780457573593594133853912353515625f,-0.573800022703409240421024151146411895751953125f,0.8408999227344988725008079200051724910736083984375f,-1.011600003653764900946043781004846096038818359375f,-0.3676999995067717907915039177169091999530792236328125f,-0.3784000044301152332337778716464526951313018798828125f,-0.1537000209756198554345019147149287164211273193359375f,0.12279999763965587789016353781335055828094482421875f,1.205000004842876837329868067172355949878692626953125f,0.205000009909272085106834992984659038484096527099609375f,-0.71849998434633022270645597018301486968994140625f,0.157799998980760580735704934340901672840118408203125f,-0.50039999785870314941149672449682839214801788330078125f,0.38919999400079252627193682201323099434375762939453125f,1.01710000499114361360852853977121412754058837890625f,-0.5621999834358692993419026606716215610504150390625f,-1.1008000451147552656294692496885545551776885986328125f,-0.1600000159442427349887339005363173782825469970703125f,0.311199995177984245486868530861102044582366943359375f,0.069599996441602751673372040386311709880828857421875f,-0.0570000020340085955528053318630554713308811187744140625f,-0.0037999995693564214693793701371760107576847076416015625f,-0.1077000126384197731255198959843255579471588134765625f,-0.22679998404979695436622932902537286281585693359375f,-0.714899985885619759784503912669606506824493408203125f,0.044200002121925319187312197755090892314910888671875f,0.805599984401464741523568591219373047351837158203125f,-0.46339999858140945576678859652020037174224853515625f,0.12360001563429801052507173153571784496307373046875f,1.11659997891783735468607119400985538959503173828125f,-1.22089997622966706103397882543504238128662109375f,-1.221399973636865698978226646431721746921539306640625f,-0.3692999906063079151863348670303821563720703125f,-0.96010001264289013267472228108090348541736602783203125f,0.068900016570836442042491398751735687255859375f,0.465800015889853202821768718422390520572662353515625f,0.56319996170997654871825943700969219207763671875f,-0.2876000024542213484579633586690761148929595947265625f,1.35759997086375960861914791166782379150390625f,0.519299991388618931154042002162896096706390380859375f,-0.0607000035822391481588056194595992565155029296875f,0.36699998977780345921217985960538499057292938232421875f,-0.549000004999339541456038205069489777088165283203125f,-0.2149999861977998083517604754888452589511871337890625f,0.3553000041961666966017219237983226776123046875f,0.29409999426603317118633640347979962825775146484375f,-1.060700020867585635642171837389469146728515625f,-0.2808000129282473000813524777186103165149688720703125f,-0.3200999945938587387672669137828052043914794921875f,0.09600000198557967490131659360486082732677459716796875f,-0.3344999961294234935849090106785297393798828125f,-0.56589998150095366380440964348963461816310882568359375f,-0.63339997622966848211945034563541412353515625f,0.5073000099182127797803332214243710041046142578125f,-0.568400005435943711518120835535228252410888671875f,-0.08669996486902231680460317875258624553680419921875f,-0.178999992966652143877581693232059478759765625f,0.626499994605779431111614030669443309307098388671875f,0.192300048363208464508034012396819889545440673828125f,-0.499799989652633502856815539416857063770294189453125f,-0.7151000337839121812066878192126750946044921875f,-0.2042000082314014974826932302676141262054443359375f,0.7967999642312530994558983366005122661590576171875f,-0.5451999887406824285562834120355546474456787109375f,-0.4969000029325485456865862943232059478759765625f,-0.097000011831521959493329632095992565155029296875f,0.32169994132518819895949491183273494243621826171875f,-0.1711999914526938226799757103435695171356201171875f,-0.347299967747926263683666547876782715320587158203125f,0.203199986296892287640503127477131783962249755859375f,-0.232000052511692533840914620668627321720123291015625f,-0.041399992561340415164750083931721746921539306640625f,0.327400001829862841162821496254764497280120849609375f,0.120800007265806197409574451739899814128875732421875f,0.59520003657341025160576464259065687656402587890625f,-0.39020001064538956114802203956060111522674560546875f,-1.3056999952375889506583916954696178436279296875f,-0.20119998802542671256787798483856022357940673828125f,-1.07299998980760502576004000729881227016448974609375f,0.52819997974038113142114525544457137584686279296875f,1.1813999518811701960885329754091799259185791015625f,-0.4814000120818615613416113774292171001434326171875f,0.3513999840676778063652818673290312290191650390625f,-0.62979998905658707286647768341936171054840087890625f,-1.1811000101953741392435404122807085514068603515625f,-0.36420001851320282781898640678264200687408447265625f,0.293699958074092659643383740331046283245086669921875f,-0.128700003820657826736351125873625278472900390625f,-0.340399961686134844285334111191332340240478515625f,0.6346999956548213805973546186578460037708282470703125f,-0.9722999968051908581401221454143524169921875f,-0.20349997082352633270829755929298698902130126953125f,-0.561299966824055474035048973746597766876220703125f,0.4460999701857566446250302760745398700237274169921875f,-0.99249998509883852193524944595992565155029296875f,-0.4968999843060970533770159818232059478759765625f,-0.961999975919724192863213829696178436279296875f,0.26919999169111274994747873279266059398651123046875f,-0.4740999877095219972034101374447345733642578125f,0.534300015640258418869734668987803161144256591796875f,-1.2024000076413159376897965557873249053955078125f,-0.413799986642599204600401208153925836086273193359375f,-0.536700001823902539399568922817707061767578125f,-1.444700000274181217463365101139061152935028076171875f,0.28320002012252842860107193700969219207763671875f,-0.74110001116395007869641631259582936763763427734375f,-0.289499965488911215061307302676141262054443359375f,1.3467999765992166860684164930717088282108306884765625f,-0.2209000286817541791606345213949680328369140625f,0.52530001388192193445547673036344349384307861328125f,-0.474299998652934817755522090010344982147216796875f,-1.3245000076591970650241592011298052966594696044921875f,-0.5374999542534357743761574965901672840118408203125f,0.33330000190138819871066289124428294599056243896484375f,-0.855600004220008258926100097596645355224609375f,0.620999981552362356751473271287977695465087890625f,0.1712000260233874637805229212972335517406463623046875f,-0.1159999919831753434351639953092671930789947509765625f,-0.283399993813037820444833414512686431407928466796875f,-0.898000005453825078660656799911521375179290771484375f,-0.98770003094822167444277738468372263014316558837890625f,0.97860001425594089763393412795267067849636077880859375f,1.4246000544629995321344040348776616156101226806640625f,-0.17580000756382929694154881872236728668212890625f,0.74719999669790215790499132708646357059478759765625f,-0.00840000424385056732035081950016319751739501953125f,-1.34549999457597824203958225552923977375030517578125f,0.729599986159801261464963317848742008209228515625f,0.4595000219643115002554623060859739780426025390625f,-0.613999988347291747459166799671947956085205078125f,0.90349998747557347389403048509848304092884063720703125f,-0.27539998485744010547904281338560394942760467529296875f,-0.7603999785430726188195649228873662650585174560546875f,0.562800006729364366009349396335892379283905029296875f,-0.247600000926852192861815638025291264057159423828125f,0.339199989902973175759370860760100185871124267578125f,0.6448000384539362972446951971505768597126007080078125f,-0.91020000289678559823869363754056394100189208984375f,0.09869995867013958701363662839867174625396728515625f,-0.73600002163648614583735252381302416324615478515625f,-0.880200034934282893317458729143254458904266357421875f,-0.433800001245737032462557181133888661861419677734375f,0.66199998754262878719600848853588104248046875f,-0.1651999820351601755419324035756289958953857421875f,-1.02979999710321390438139133038930594921112060546875f,-0.2198999896109106177988223862485028803348541259765625f,0.346099994176626069020130671560764312744140625f,-0.4747999939739704355901039889431558549404144287109375f,-0.391099977338313919972279109060764312744140625f,0.56349999948963513940469738372485153377056121826171875f,0.1826999626077707716831355355679988861083984375f,0.82410001592859660757284245846676640212535858154296875f,0.4877000115394594104145653545856475830078125f,-1.09499999985098828148011307348497211933135986328125f,0.77100001716613775215591886080801486968994140625f,0.683099985742568804880647803656756877899169921875f,0.081999950885773387199151329696178436279296875f,0.65480003018379218104882966144941747188568115234375f,-0.4264999649524696678781765513122081756591796875f,-1.3854999819844966424398080562241375446319580078125f,0.771899992837011605928410062915645539760589599609375f,0.6625999999769032200447327340953052043914794921875f,0.7669999707043171355280719581060111522674560546875f,0.604399998462200382931541753350757062435150146484375f,-0.5992000079333781314971929532475769519805908203125f,-0.3008000100225209916260382669861428439617156982421875f,-0.318799949240684554752078838646411895751953125f,-1.140999996155500184613629244267940521240234375f,0.1402000145196922176182852126657962799072265625f,-0.23749999791383746838846491300500929355621337890625f,-0.5270999813914301057593547739088535308837890625f,-0.648399979656935077088064645067788660526275634765625f,0.77499998390674562642743694595992565155029296875f,0.55979998011589060524784144945442676544189453125f,-0.192299995464086492802380234934389591217041015625f,-0.4163999997138976283395095379091799259185791015625f,-0.6586000180929898473181083318195305764675140380859375f,0.219199997204542196271859211265109479427337646484375f,-0.5779999825805426372227202591602690517902374267578125f,-0.26519999589323983713029520004056394100189208984375f,0.40769997294545223809336675913073122501373291015625f,0.24839999127984047078143703402020037174224853515625f,0.508399956709146838562674020067788660526275634765625f,0.060999996304512027478494928800500929355621337890625f,1.0609999980181454848349176245392300188541412353515625f,-0.08200000274181373782766968361102044582366943359375f,0.25489995880275995165931135488790459930896759033203125f,0.03059999911636113179014273555367253720760345458984375f,-0.2485000087283550929129205542267300188541412353515625f,-0.53439998981952652190585695279878564178943634033203125f,0.08459999264180684264147203066386282444000244140625f,-0.291800008487701489912069519050419330596923828125f,-1.09799996685981771094020587042905390262603759765625f,-0.04700000661611536401096600457094609737396240234375f,0.6103999728992584028475221202825196087360382080078125f,0.2846000009492037907676831309800036251544952392578125f,0.2044000343926251073156663551344536244869232177734375f,-0.31260001396536818418780967476777732372283935546875f,-1.346099997603893871911395763163454830646514892578125f,-0.42299999263882614997100972686894237995147705078125f,-0.4184999959319826157155830514966510236263275146484375f,0.1777999945104122847538974383496679365634918212890625f,-0.48510001526921986414464527115342207252979278564453125f,-0.935300015819072516620735768810845911502838134765625f,0.4434000222742557895116988220252096652984619140625f,-0.2524999997764825199197957772412337362766265869140625f,0.0500999987661838730446106637828052043914794921875f,1.011399993380904138717824025661684572696685791015625f,-0.80069997363090550379638443700969219207763671875f,0.601699973213672745231406224775128066539764404296875f,-0.3077000049829479166874079965054988861083984375f,0.654299994778633209335794163052923977375030517578125f,-0.5274000018298625747092955862171947956085205078125f,0.3380999800056219140031998904305510222911834716796875f,0.4785999901533131151154520921409130096435546875f,0.1222999933779240766540397089556790888309478759765625f,0.014399996934831182926473047700710594654083251953125f,-0.160099987404048516026477955165319144725799560546875f,-0.889699961009622342089642188511788845062255859375f,-1.212299972292780747551432796171866357326507568359375f,-0.0647999979972837536479346454143524169921875f,-0.28070004441142071271997338044457137584686279296875f,-0.6993999588251114118975237943232059478759765625f,-0.33459997734948998537873876557569019496440887451171875f,1.081699958442897013810579665005207061767578125f,-0.598900004761665716301877182559110224246978759765625f,0.464900021946430541674999403767287731170654296875f,-0.199599998080730589578024591901339590549468994140625f,-0.844499953866005625968682579696178436279296875f,-0.3365000016093253609739122111932374536991119384765625f,-0.29479999181330196478256766567938029766082763671875f,0.47299999736994491339459045775583945214748382568359375f,0.308299986925721025698976518469862639904022216796875f,-0.60459999904930583625173312611877918243408203125f,-0.37060000883936883298019893118180334568023681640625f,-0.0980000133141876172970796687877736985683441162109375f,-0.8980999751441187672895694049657322466373443603515625f,0.07459997632503512221546770888380706310272216796875f,0.205700039047002292136312462389469146728515625f,-0.69340000111460664555806943099014461040496826171875f,0.44769994433522253274304603110067546367645263671875f,-0.7308000218689443272523931227624416351318359375f,-1.325600004369020812333701542229391634464263916015625f,-0.40879999826550472707964445362449623644351959228515625f,-0.263000014171004270480125342146493494510650634765625f,0.190000004619359952329205043497495353221893310546875f,-0.9418999781221144562692870749742724001407623291015625f,0.06959998466968553287870236090384423732757568359375f,1.671100003564357283636354623013176023960113525390625f,-0.6824000179231166640647643362171947956085205078125f,-1.507700045551359391282630895148031413555145263671875f,-0.70719999852329495926284153028973378241062164306640625f,-0.7589999964870510762438016172382049262523651123046875f,0.30139999658465388421291208942420780658721923828125f,0.6988999836206435389840407879091799259185791015625f,0.4041999890089034153106695157475769519805908203125f,0.9350999840885396707790278014726936817169189453125f,0.676999990150332298100011030328460037708282470703125f,-0.302399953438342361522472856449894607067108154296875f,0.040800005775690095788377220742404460906982421875f,-0.1682000540971753110852660029195249080657958984375f,-0.5558999957501888911082232880289666354656219482421875f,-0.73799996544420753963322567869909107685089111328125f,-0.9389999763518570574660770944319665431976318359375f,-0.100300007586180850438495326670818030834197998046875f,-0.35060001539587959396016003665863536298274993896484375f,-0.67419999079704240330102038569748401641845703125f,-0.12030002222657198984734350233338773250579833984375f,-0.09349994802474981270279386080801486968994140625f,-1.77399999311566336501755358767695724964141845703125f,0.30090001020431600409210659563541412353515625f,-0.385500009104609464571922217146493494510650634765625f,-0.26619997126162076028776937164366245269775390625f,-0.58350000487267983384498393206740729510784149169921875f,0.0960999959647654833361229975707828998565673828125f,1.1476000006288293864287197720841504633426666259765625f,-0.890999981105327520936043583787977695465087890625f,-1.974000039309263154763129932689480483531951904296875f,0.36330004019737316411919891834259033203125f,-0.66229998408332468873283005450502969324588775634765625f,-0.46929997456893346452488913200795650482177734375f,0.56199998998269451444542710305540822446346282958984375f,-0.0217000092744825678892084397375583648681640625f,0.84709998273253450662423347239382565021514892578125f,0.171999973237515035862088552676141262054443359375f,-0.2600000047683714843316238329862244427204132080078125f,0.80359995670020634150887417490594089031219482421875f,1.24199997696280473036267721909098327159881591796875f,0.107299998295307119633434922434389591217041015625f,-0.2561999831080437051156195593648590147495269775390625f,0.1179999851882456152196709808777086436748504638671875f,-0.0636000001370906620223877325770445168018341064453125f,-0.074700006681680530817857288639061152935028076171875f,0.565200012731552181577399096568115055561065673828125f,0.05289999655485200236171294818632304668426513671875f,-0.01640001699924464872992757591418921947479248046875f,0.528999991327523400030941047589294612407684326171875f,0.72639998809099193266547445091418921947479248046875f,0.9747999786257739440742398073780350387096405029296875f,-0.7380000160336497305735292684403248131275177001953125f,-1.0152000277817254669798785471357405185699462890625f,0.48460000731945029883718234486877918243408203125f,0.107300026309490537101964946486987173557281494140625f,-0.63820000507235530307070803246460855007171630859375f,0.2396000038921834995875315144076012074947357177734375f,-0.2078000033020972470154674738296307623386383056640625f,-0.9619000150769949630813471230794675648212432861328125f,0.56659999888539314127910984097979962825775146484375f,0.96760000800490342243165287072770297527313232421875f,-0.25739999586939799058882272220216691493988037109375f,-0.60050000828504490613113375729881227016448974609375f,-0.77120001603960997016429246286861598491668701171875f,0.5351999732434740142394957729266025125980377197265625f,-0.0169999927580357645950925871147774159908294677734375f,0.1133000261902810112957240562536753714084625244140625f,-0.5452000033438204606994759160443209111690521240234375f,-0.46260001247525206480304404976777732372283935546875f,0.58120002572536488827381617738865315914154052734375f,0.37930000222921389507746425806544721126556396484375f,0.0876999927639960930036977515555918216705322265625f,0.413900003737211363841197453439235687255859375f,0.8733999924719333396438969430164434015750885009765625f,0.6053999816536901334984577260911464691162109375f,1.262700008559227260462876074598170816898345947265625f,-0.765199996936321014118220773525536060333251953125f,0.361799961847066864351063486537896096706390380859375f,0.0261999744653707011821097694337368011474609375f,-0.30939999265074735745884026982821524143218994140625f,-0.1824000092804425321446615271270275115966796875f,-0.3696000205814840011697697264025919139385223388671875f,-0.231000021636486252418762887828052043914794921875f,-0.132699990379810284224504357553087174892425537109375f,-0.56089997920989986823769868351519107818603515625f,-1.0166000085711477396444024634547531604766845703125f,0.5620000170469285905028300476260483264923095703125f,1.408900011858344125670328139676712453365325927734375f,0.0618999707460403669756487943232059478759765625f,-0.7271000310122968102177765103988349437713623046875f,-0.496099962437153152450264315120875835418701171875f,-0.0085999619901178103731353985494934022426605224609375f,-0.530899985766410509313573129475116729736328125f,-0.660399995809793693979372619651257991790771484375f,0.7124999798834323883056640625f,-0.3132000062644479942974840014358051121234893798828125f,0.1419000322878360975664691068232059478759765625f,0.7025000144541262425690320014837197959423065185546875f,0.61659999396800913018523715436458587646484375f,1.51550001782178878073636951739899814128875732421875f,0.140599973523616750270548436674289405345916748046875f,-0.340600023740530222227107515209354460239410400390625f,0.95460002177357672348279038487817160785198211669921875f,0.30120001872181922930593600540305487811565399169921875f,0.197200014206767360036565150949172675609588623046875f,-0.588600008183717715581906304578296840190887451171875f,-0.3805000371932987235368273104541003704071044921875f,-0.62179998345375064872087023104541003704071044921875f,-1.00869997342228856496149091981351375579833984375f,-0.8654000035583973016173331416212022304534912109375f,-0.7386000409290189860911368668894283473491668701171875f,0.4919999910071493065544245837372727692127227783203125f,0.8900000082142651702810098868212662637233734130859375f,0.7238000081002713415045946021564304828643798828125f,-0.0254999670088287277991412338451482355594635009765625f,0.5228000225245954535324699463672004640102386474609375f,-0.18540000132322287385022718808613717555999755859375f,0.70200001883506768507459128159098327159881591796875f,0.977300051194429642009708913974463939666748046875f,0.282799998682737196986636263318359851837158203125f,-0.3963999676018954421152784561854787170886993408203125f,-0.1296000024020671492763767673750407993793487548828125f,-1.293800016556680976265170102124102413654327392578125f,0.403000002846121641031373883379274047911167144775390625f,1.0804000285550952042967765009962022304534912109375f,0.106799994629621419761633660527877509593963623046875f,0.5178999774813650702753875521011650562286376953125f,-0.7613999946475029645398535649292171001434326171875f,0.0478000018119810743399966668221168220043182373046875f,0.5185000012218952125664372942992486059665679931640625f,-1.2037000075459474146555294282734394073486328125f,-0.6792000099450348216123529709875583648681640625f,0.9912000220745795786569942720234394073486328125f,-0.30189998676478857486671358856256119906902313232421875f,0.25750002235174207498857867904007434844970703125f,-1.354699974793196037836651157704181969165802001953125f,0.030600016140937214004225097596645355224609375f,-0.1684999976456165171612155972979962825775146484375f,-0.4842999703407286915535223670303821563720703125f,0.20349997859075685813223799414117820560932159423828125f,-0.47950000529363734358412330038845539093017578125f,-0.142400011347979305043054409907199442386627197265625f,0.6022999920368192761088721454143524169921875f,0.71480001409053794958481375942938029766082763671875f,0.490900002008675784281876985915005207061767578125f,0.566300024193524986770853502093814313411712646484375f,0.0193999671697611120180226862430572509765625f,-0.45639999226331706694281820091418921947479248046875f,1.061699970531464032319490797817707061767578125f,-0.2078999990880487924727049175999127328395843505859375f,0.42980000820457942012353669269941747188568115234375f,0.87580000547766656548986929919919930398464202880859375f,-0.54260003285259017502539791166782379150390625f,0.150199985127151103281306632197811268270015716552734375f,0.4610999844908718614533427171409130096435546875f,-0.142399976348877022047645368729718029499053955078125f,0.10700000989437086218458716757595539093017578125f,-0.2301000021934509476295716012828052043914794921875f,0.099399955770373693564323502869228832423686981201171875f,-0.307799997863173413481518991829943843185901641845703125f,-0.0091000073418021631965757478610612452030181884765625f,0.2438000043749808387616440086276270449161529541015625f,0.717200019943714295322934049181640148162841796875f,0.28040001935362823815012234263122081756591796875f,-0.37870003511309580090937743079848587512969970703125f,0.14259999727606764707843467476777732372283935546875f,0.40570002518892334109068542602472007274627685546875f,0.4319999848604201275747982435859739780426025390625f,0.1659999915361396460866671986877918243408203125f,0.6696000150680543328007843228988349437713623046875f,-0.367999967157840668363633085391484200954437255859375f,-0.277999991595744955219515759381465613842010498046875f,0.36129998410940178388273125165142118930816650390625f,-0.77700002151727698418426371063105762004852294921875f,-0.652000044465065542453885427676141262054443359375f,-0.25920000972151768792173243127763271331787109375f,0.591899982964992688039274071343243122100830078125f,0.18400000607967381682783525320701301097869873046875f,0.18520001235902316949477608432061970233917236328125f,0.116999996483325929830243694595992565155029296875f,-0.014099966996908630534335316042415797710418701171875f,-0.467600001597404446584960169275291264057159423828125f,-0.47870001231431957844364433185546658933162689208984375f,0.39280000866651543223184717135154642164707183837890625f,0.38850002312660247838493887684307992458343505859375f,0.48700002062320724149913075962103903293609619140625f,0.49389998917132660327666826560744084417819976806640625f,-0.3798999974712728988635035420884378254413604736328125f,-0.084399979873001740315885399468243122100830078125f,0.088200004513561636176888214322389103472232818603515625f,0.59919997604489338982602930627763271331787109375f,0.7251999903798103108698569485568441450595855712890625f,1.13469999103546115293283946812152862548828125f,0.1402999836325644622547770268283784389495849609375f,-0.70650000429153436698470613919198513031005859375f,0.56450000795722043989144367515109479427337646484375f,0.8651999638557441585362539626657962799072265625f,0.4867999959707258739172175410203635692596435546875f,0.16679999060630734675214625895023345947265625f,0.69320001803636532855534824193455278873443603515625f,-1.0675999720931059755457681603729724884033203125f,-0.540299988102912909226915871840901672840118408203125f,-0.694399997866153029235647409223020076751708984375f,0.0037999771058558895475698591326363384723663330078125f,-0.9191000029087064149280195124447345733642578125f,-0.485100003087520548206157400272786617279052734375f,0.1510999995410440277510133455507457256317138671875f,-0.34349998319149011027917595129110850393772125244140625f,0.380599990808963895005945232696831226348876953125f,-0.1916000009715557172995659129810519516468048095703125f,0.2382000186324120249992120079696178436279296875f,0.09829999661147592604493183898739516735076904296875f,0.3183999946326014907072021742351353168487548828125f,1.20990000756680959881350645446218550205230712890625f,0.685199991571902700115970219485461711883544921875f,0.56890000591650602412840953547856770455837249755859375f,0.746500023663043688060270142159424722194671630859375f,-0.756800019812584157108403815072961151599884033203125f,-1.21099999108910605372102509136311709880828857421875f,-0.44040000385642041891287590260617434978485107421875f,0.634700002658366901897579737124033272266387939453125f,0.214999982714652926318876779987476766109466552734375f,0.81429999865293467564697493799030780792236328125f,0.2506000197172166821246719337068498134613037109375f,-0.54499995991587635302266789949499070644378662109375f,0.59820003189444559410503643448464572429656982421875f,0.05500004097819388704238008358515799045562744140625f,-0.41939999771714209231276981881819665431976318359375f,-0.493600046181678298040651498013176023960113525390625f,0.2488000030338763934878443251363933086395263671875f,0.4895999854147434771078906123875640332698822021484375f,-0.0422000019133090820133702436578460037708282470703125f,-0.3529999963641163862604344103601761162281036376953125f,-0.0654000050485132877753358116024173796176910400390625f,0.3397999932289124114248579644481651484966278076171875f,-0.0617999985784292160673203397891484200954437255859375f,0.38849998661875739802695761682116426527500152587890625f,-0.1836000129520893864309982745908200740814208984375f,-1.54999997884035156658910636906512081623077392578125f,0.40359999775290500423352568759582936763763427734375f,0.8037999771058561560010957691702060401439666748046875f,0.9868000095307827290724844715441577136516571044921875f,1.09610003217458729096733804908581078052520751953125f,0.37099999555945384344113335828296840190887451171875f,0.7535999758481981558588813641108572483062744140625f,-0.86399998909234998478723355219699442386627197265625f,0.209600000464917002318543381989002227783203125f,-0.116900007402896743968767623300664126873016357421875f,-0.166299975466728255923953838646411895751953125f,-0.171400017893314160488671404891647398471832275390625f,1.139799977433682443006546236574649810791015625f,-0.557199996250867801705908277654089033603668212890625f,1.5088000070571894184467964805662631988525390625f,0.433000019460916263369654188863933086395263671875f,1.1008999941110602094340720213949680328369140625f,0.60649999997019765629602261469699442386627197265625f,0.119300016534328534589803894050419330596923828125f,0.872900004303455379073284348123706877231597900390625f,-0.28730001647472391823612269945442676544189453125f,0.03219999446272847620775792165659368038177490234375f,-0.69169999213814747918149805627763271331787109375f,0.0200999939978123709494184367940761148929595947265625f,-0.8882999759733678502016118727624416351318359375f,-0.059499997675418914155898164608515799045562744140625f,-0.073299997133016603356736595742404460906982421875f,-1.0049999910220501764257505783461965620517730712890625f,0.128699999797344588614578242413699626922607421875f,1.3173999736666679449825778647209517657756805419921875f,-0.70509999802112588440650142729282379150390625f,0.81019999946951859470800627605058252811431884765625f,0.1240999675929543855090741999447345733642578125f,1.2140999795138835981589409129810519516468048095703125f,0.526699970233440950551084824837744235992431640625f,-0.8119000322878358044675906057818792760372161865234375f,-0.30699996504187598844737294712103903293609619140625f,-0.74239999616742124999291263520717620849609375f,0.330499974384904415813934974721632897853851318359375f,0.431800002229213664151075136032886803150177001953125f,-0.421700006145238948107589749270118772983551025390625f,0.541600004547834412704787609982304275035858154296875f,0.547700006696582253340466195368207991123199462890625f,0.091999994620680924928279864616342820227146148681640625f,0.44389998268932107094997263629920780658721923828125f,-0.61660001170039180351523100398480892181396484375f,0.62549999547004642153069653431884944438934326171875f,0.61919999899268152177000956726260483264923095703125f,-0.74999997958540998155285706161521375179290771484375f,-0.43900002172589314142214789171703159809112548828125f,-0.14290000624060628098277447861619293689727783203125f,-0.49380000690817826836109816213138401508331298828125f,-0.72360003321766885875376829062588512897491455078125f,-0.425600018674135327501062420196831226348876953125f,0.890299956810474579782521686865948140621185302734375f,0.138000009477138529945250411401502788066864013671875f,-0.194700004893541223083275326644070446491241455078125f,0.306000000178813902351748765795491635799407958984375f,0.990000031888484510744774524937383830547332763671875f,-0.0651999917209149515429089660756289958953857421875f,-1.48320000060200651859076970140449702739715576171875f,-0.02759999876618390857174745178781449794769287109375f,0.296600026005506833826075308024883270263671875f,-0.35019999857544892307714690105058252811431884765625f,0.6417999971628187694250300410203635692596435546875f,0.65620000113844856315381548483856022357940673828125f,-0.8563999596297744432149556814692914485931396484375f,-0.22769999514818184405129386504995636641979217529296875f,0.7695999736428262849585735239088535308837890625f,1.413699985045194562616188704851083457469940185546875f,-0.4277000166058542163227684795856475830078125f,0.4801999891504646900131092479568906128406524658203125f,-0.652399984060227922100239084102213382720947265625f,0.100000026039779310593758054892532527446746826171875f,-0.548099987828731372019319678656756877899169921875f,-0.7403999996840953112808847436099313199520111083984375f,1.08049997520446794396775658242404460906982421875f,0.153999975681304857033637745189480483531951904296875f,-0.9824999874830240287337801419198513031005859375f,-0.483400001114606681085206218995153903961181640625f,-0.25209997572898856788015109486877918243408203125f,0.36389997244477267912543538841418921947479248046875f,-1.10969999282360021197746391408145427703857421875f,-0.447800000917911500408763458835892379283905029296875f,0.426199992346763423256561509333550930023193359375f,0.1817000113606452860182116637588478624820709228515625f,-0.081800015044212415205038269050419330596923828125f,-1.165700008946657373343214203487150371074676513671875f,1.1182999913215638798646978102624416351318359375f,0.12740000048875810279014331172220408916473388671875f,-0.3107000016450882640128838829696178436279296875f,0.176200012016296536643267245381139218807220458984375f,0.38530001254081724937350372783839702606201171875f,0.111599975341558632635496906004846096038818359375f,0.3235999995596705591793806888745166361331939697265625f,-0.729800019603967786707698905956931412220001220703125f,-1.08750004217028628517027755151502788066864013671875f,-0.60400000980496404423547573969699442386627197265625f,0.6884999560713769284348018118180334568023681640625f,0.0318000032722949299568426795303821563720703125f,0.293399979805947008770772299612872302532196044921875f,-0.224600003296136829789020339376293122768402099609375f,-0.0428000205874441430609067538171075284481048583984375f,0.1060000001788139467606697508017532527446746826171875f,-0.14349999850243311261266399014857597649097442626953125f,-0.38499999303370702197213404360809363424777984619140625f,0.2169000027649106154825631165294907987117767333984375f,-0.279999994188547152162982456502504646778106689453125f,-0.654800009173154506925129680894315242767333984375f,0.1381999985158444133048760704696178436279296875f,0.9798000038087366636574415679206140339374542236328125f,0.5948000288426877890657351599656976759433746337890625f,0.524100028687715990827200585044920444488525390625f,0.197400009131431630748920724727213382720947265625f,-0.6453999904453748825972070335410535335540771484375f,-0.80139999852180476835883382591418921947479248046875f,0.320299984452127983747260486779850907623767852783203125f,0.475999997720122192124136972779524512588977813720703125f,-0.772899999721348596182224355288781225681304931640625f,-0.23660002660155310394429761799983680248260498046875f,0.2410999505162241263178657391108572483062744140625f,0.385799985957145707970994408242404460906982421875f,0.75919995954185670594682733280933462083339691162109375f,-0.76660001699030377864119145669974386692047119140625f,0.007100014770031748412293381989002227783203125f,0.283199976164102640296960089472122490406036376953125f,-1.024699972856044638547245995141565799713134765625f,-0.2415000203847885718033694502082653343677520751953125f,0.271299990665912815757110365666449069976806640625f,0.0568999890744688041621657248470000922679901123046875f,-0.071899987435340761976476642303168773651123046875f,-0.202600004726648297292967981775291264057159423828125f,0.0453999995350837792784659541212022304534912109375f,-0.2074999890476465491673252472537569701671600341796875f,-0.449799970541894911235658582882024347782135009765625f,-0.50809998790323740447405498343869112432003021240234375f,0.393700004565715744320186786353588104248046875f,0.787000006020069253764859240618534386157989501953125f,-0.27230000097751627663456019945442676544189453125f,0.32170001285076121888550915173254907131195068359375f,0.825299992424248785027884878218173980712890625f,-0.3704000026643277099225315396324731409549713134765625f,0.443400005733966651177979656495153903961181640625f,0.4039000392019749252625615554279647767543792724609375f,-0.51939997372627288996227434836328029632568359375f,-0.3694000059500337496132260639569722115993499755859375f,0.829999961443245837244830909185111522674560546875f,1.321799951379001303308768910937942564487457275390625f,-1.29909999873638071221648715436458587646484375f,0.78049997714161900574936225893907248973846435546875f,-0.664800010812282238248371868394315242767333984375f,-0.06170002007782493524956635155831463634967803955078125f,-0.454499996185302901352542903623543679714202880859375f,0.014399996078014343225959237315692007541656494140625f,-0.155399999162554369736000126067665405571460723876953125f,-0.363000003144144767208700841365498490631580352783203125f,0.307299983803928089542978341341949999332427978515625f,-0.085999996677041057324686335050500929355621337890625f,0.26939994940012734758738588425330817699432373046875f,-0.54500002115964907289935581502504646778106689453125f,-0.225600011074543527200830794754438102245330810546875f,0.273799994540214530758248656638897955417633056640625f,0.004899984097480825084858224727213382720947265625f,-0.080599996322393341330325711169280111789703369140625f,0.806799997907877042280233581550419330596923828125f,-0.8518000089347363967817727825604379177093505859375f,-0.863699993836879453823485164321027696132659912109375f,-0.665600009286403615504923436674289405345916748046875f,-0.862900042413175416555759511538781225681304931640625f,-0.05280000233352188931945647709653712809085845947265625f,0.7457000091888004167373082964331842958927154541015625f,0.183799999383091883231600149883888661861419677734375f,-0.2916999891951679302337652188725769519805908203125f,0.65640001372098932819199035293422639369964599609375f,1.19960002147555311324822469032369554042816162109375f,0.03240001106858247936770567321218550205230712890625f,0.952499992698431441340289893560111522674560546875f,-0.06839999157786369465839015902020037174224853515625f,-1.572099989400803199401934762136079370975494384765625f,0.17899999020993717646632603646139614284038543701171875f,1.2000000202842058261154534193337894976139068603515625f,-1.7444000093214204749614282263792119920253753662109375f,-0.442199975426495417440975188583252020180225372314453125f,-1.3384999792613090363602168508805334568023681640625f,0.0155999988555908242204850466805510222911834716796875f,-1.19360000360161055965591003769077360630035400390625f,-0.105599988126755028883252407467807643115520477294921875f,-0.266899992464482538334635819410323165357112884521484375f,-0.70139997259378450422673267894424498081207275390625f,-0.5782999716520313970846700613037683069705963134765625f,-0.19680001504421174018943929695524275302886962890625f,0.4385000309497126824709312131744809448719024658203125f,0.20500000506639448616397203295491635799407958984375f,1.5408000221669670981583522006985731422901153564453125f,-0.17139999971389752175809917389415204524993896484375f,-0.016199984747170770305046971770934760570526123046875f,-0.361399981534481273826031610951758921146392822265625f,-0.726100018018484671955548037658445537090301513671875f,0.71159997400045404969404216899420134723186492919921875f,-0.9554000572770835475466810748912394046783447265625f,-0.095399992233514918638093149638734757900238037109375f,-0.07639999002814279460693569490103982388973236083984375f,-0.5645999812424182806580574833787977695465087890625f,0.27190002237856403244364855709136463701725006103515625f,0.57519998427033414145626011304557323455810546875f,-1.03830005260258939614459450240246951580047607421875f,-0.4595999866068363193249979303800500929355621337890625f,-1.3731000064924359094220562838017940521240234375f,0.3321999951332805789405711038853041827678680419921875f,-1.152000074379146976610854835598729550838470458984375f,0.6952999924987555147026796475984156131744384765625f,0.85159996520876912740050102001987397670745849609375f,0.2486999861627818830811520456336438655853271484375f,-0.1766000012695789767036558259860612452030181884765625f,-0.26529996604919425795543475032900460064411163330078125f,-1.10449996057152777240162322414107620716094970703125f,-0.573599976518750498399867865373380482196807861328125f,-0.3407999840199948948793462477624416351318359375f,-0.991000001817941456039307013270445168018341064453125f,-0.79140001387000058485909903538413345813751220703125f,-0.014899990430473852853765492909587919712066650390625f,-0.054500013917684331232749173068441450595855712890625f,0.3727999792367222564593021161272190511226654052734375f,0.33279996040538006862874453872791491448879241943359375f,-0.0128000387668609771907313188421539962291717529296875f,0.447499990910291689516498081502504646778106689453125f,0.4185000002533196816756344560417346656322479248046875f,0.732400014793872511376093825674615800380706787109375f,0.3045999924182891671620154738775454461574554443359375f,0.05619997491240535936185551690869033336639404296875f,0.991599998363852375149463114212267100811004638671875f,1.0688000073552128554865703335963189601898193359375f,0.34539999566078183335093854111619293689727783203125f,-0.0323000084280966603955675964243710041046142578125f,0.3599999880790709649858172269887290894985198974609375f,-0.5719000193238255036476402892731130123138427734375f,0.177499989345669906271041327272541821002960205078125f,0.2040999956071376519872728749760426580905914306640625f,0.75980000768303856961694009441998787224292755126953125f,-0.792200029517710735404989463859237730503082275390625f,-0.5633999862134455582918235450051724910736083984375f,-0.2096000113055112024085246957838535308837890625f,-0.480199993509053957296828230028040707111358642578125f,-0.0357000019058582029884973962907679378986358642578125f,0.31219999192953107325365635915659368038177490234375f,0.4253000000238416067332991588045842945575714111328125f,0.6127999743938448151681086528697051107883453369140625f,0.166399936681985438635678065111278556287288665771484375f,-0.5881999791443348613029229454696178436279296875f,-0.40090000979453321861001313664019107818603515625f,-0.190999979913235140571714509860612452030181884765625f,0.27770003551170219235899594423244707286357879638671875f,0.456400003848969448849715035976259969174861907958984375f,0.35350001163408129034593230244354344904422760009765625f,0.031999993056059050644535091123543679714202880859375f,-0.06589999355226716648559204259072430431842803955078125f,-0.3579999939799307639987091533839702606201171875f,-0.3297999952405687995593552841455675661563873291015625f,-0.46759997179508250297885751933790743350982666015625f,-0.8089999884217979886358307339833118021488189697265625f,-0.0172000080227849849734411691315472126007080078125f,-0.849300025251507495482883314252831041812896728515625f,-0.74139997169971483259587330394424498081207275390625f,-1.0087999689102173928034744676551781594753265380859375f,0.214399989074468777516813133843243122100830078125f,-1.4458000084206459323610260980785824358463287353515625f,-0.38139999405145630362312658689916133880615234375f,-1.840800032746792158633297731284983456134796142578125f,-0.265099973583221792949871087330393493175506591796875f,-0.333599978232383964638074758113361895084381103515625f,-0.1280000064969064599296189044252969324588775634765625f,-0.0647000148773191341433630441315472126007080078125f,0.705900001263618293023682781495153903961181640625f,0.60780001209378209381384294829331338405609130859375f,0.074100023844838158737502453732304275035858154296875f,-0.24649998357892055622642146772705018520355224609375f,-0.518099994459748192099368679919280111789703369140625f,0.437999995768070249368975055404007434844970703125f,-0.383600012654066180317613543593324720859527587890625f,-0.9609000051379201767787208154913969337940216064453125f,-0.20519999824017254308472502088989131152629852294921875f,0.28579999316558246658104280868428759276866912841796875f,0.6871000056244433995544795834575779736042022705078125f,0.2994999955892563292536578956060111522674560546875f,0.608900007909536356009994051419198513031005859375f,0.1022000011682511200206135981716215610504150390625f,-0.7023000152826313069454045034945011138916015625f,-0.6344999777078630387450175476260483264923095703125f,-0.71369998221397423776579671539366245269775390625f,-0.946199969547987240758857296896167099475860595703125f,-1.0506999798148870528535780977108515799045562744140625f,-0.80409998934865001274374662898480892181396484375f,-0.58189999376833423383459376054815948009490966796875f,0.565799991768598520280875163734890520572662353515625f,0.2642000091999772148909642055514268577098846435546875f,0.19409999456405646611756310448981821537017822265625f,-0.032900013467669797506687245913781225681304931640625f,-0.22540001030117284841480795876123011112213134765625f,0.01719999848604203407376189716160297393798828125f,-0.5334999954849479752994056980242021381855010986328125f,-0.56359998314976689215427541057579219341278076171875f,-0.7852999768897890309204967707046307623386383056640625f,-0.045000013858079501716247250442393124103546142578125f,0.41689997011274082527876316817128099501132965087890625f,0.0779999944269658129769595689140260219573974609375f,1.8133000093892217563507074373774230480194091796875f,-0.09039999089241046448250926914624869823455810546875f,0.6913000097393984244575904085650108754634857177734375f,0.546999976813793420404863354633562266826629638671875f,-0.547100014863163064404716351418755948543548583984375f,0.0937999993085860683805776716326363384723663330078125f,-1.69550002938881494429779195343144237995147705078125f,-0.1435999798715119624148428556509315967559814453125f,-0.457899990333616724313969825743697583675384521484375f,0.570200000661611028363040531985461711883544921875f,0.74050001115351893332672261749394237995147705078125f,0.1351999881446364071280186180956661701202392578125f,0.215300010938942687488406591000966727733612060546875f,0.34379998008608847470668479218147695064544677734375f,1.019299975816905234893283704877831041812896728515625f,0.099199994224310028556601537275128066539764404296875f,1.14669997421950053961836601956747472286224365234375f,-0.48679998941421498415138557902537286281585693359375f,0.21789998113214981145802084938623011112213134765625f,-0.3995999711096291928669188564526848495006561279296875f,0.1570999908909194164863265541498549282550811767578125f,0.79430000729560834571429950301535427570343017578125f,-0.08490000004172326697471362422220408916473388671875f,0.2314000044077635376282842116779647767543792724609375f,-0.2678999999821185529214062626124359667301177978515625f,-0.62699997606873536515337264063418842852115631103515625f,-1.2373999446094041676502683912985958158969879150390625f,-0.35399997329711940796670432973769493401050567626953125f,0.18890000254511818411629064939916133880615234375f,0.834100023248792066254964083782397210597991943359375f,0.385899993216991532563042710535228252410888671875f,-0.303800019723177161523608447168953716754913330078125f,-0.136400000309944147147689363919198513031005859375f,0.052299993228912544651620919466949999332427978515625f,0.99009997954368600403540767729282379150390625f,0.0899999944865701451135464594699442386627197265625f,-0.173599979424476913436592440120875835418701171875f,-0.936599997693300156953455370967276394367218017578125f,-0.648800002139806597512006192118860781192779541015625f,-0.7009000193685290813760957462363876402378082275390625f,0.05879997799992597862228649319149553775787353515625f,0.344899967780709193476695872959680855274200439453125f,0.9429999816119671862679751939140260219573974609375f,0.3694999904483555752676693373359739780426025390625f,-0.4794999802410604416991191101260483264923095703125f,0.3596000095546243446875678273499943315982818603515625f,0.29689999522119758790239529844257049262523651123046875f,0.13600001534074535047835752266109921038150787353515625f,-0.580299971041083662015580557635985314846038818359375f,0.3497000036269424239065983783802948892116546630859375f,-1.640000006780028218855704835732467472553253173828125f,0.1141999896049501472816700697876513004302978515625f,-1.014400004646181852052677641040645539760589599609375f,0.1481999962806701542916698599583469331264495849609375f,0.218099990063905513348885278901434503495693206787109375f,0.583199965286255395113812483032234013080596923828125f,-0.741300034698844090286229402408935129642486572265625f,-0.045200017052888341595462406985461711883544921875f,-1.1344000093773001225372354383580386638641357421875f,0.4538000055670732280077572795562446117401123046875f,0.5750999849826090581927928724326193332672119140625f,-0.11820000298619248724207864142954349517822265625f,0.824300005842745431294815716682933270931243896484375f,-0.44739998201131836452759671374224126338958740234375f,1.2751000168710941995442453844589181244373321533203125f,0.128799976658821702102386552724055945873260498046875f,1.56260001918077495730585724231787025928497314453125f,0.1277999954044819475029726163484156131744384765625f,1.6390000487342482049513137098983861505985260009765625f,0.0018000154912467536405529244802892208099365234375f,0.34480004922002560618921052082441747188568115234375f,0.162999993532896070291826617904007434844970703125f,-1.6287000072851771648174690199084579944610595703125f,-0.34419998930692674576903300476260483264923095703125f,-0.96900000082701442938315494757262058556079864501953125f,-0.355799991172552065865630765983951278030872344970703125f,-0.5655999971047040997262911332654766738414764404296875f,0.04239999735951427428659599172533489763736724853515625f,-0.0853999875769020366789163745124824345111846923828125f,0.686000045627355614641373904305510222911834716796875f,-0.69049996417760883815617489744909107685089111328125f,0.059400013512373117663400989840738475322723388671875f,0.5334999714195731712607084773480892181396484375f,0.30109998188316833278577178134582936763763427734375f,0.427300002374499943158525638864375650882720947265625f,-0.1211000041231511392680886274320073425769805908203125f,-0.07280001373290989619135871180333197116851806640625f,-0.396400026684999351456184513153857551515102386474609375f,0.8428999773696066455386244342662394046783447265625f,0.1482999905019999342670189435011707246303558349609375f,-0.11979999926388262831977726818877272307872772216796875f,-0.17919999519288543066153351901448331773281097412109375f,-0.61049999042227864887166788321337662637233734130859375f,0.280800005328655177816443710980820469558238983154296875f,0.54439998460412031278110589482821524143218994140625f,-0.2069999914914364413931480157771147787570953369140625f,-0.7346000018060205860592759563587605953216552734375f,-0.3236999933898447867619552198448218405246734619140625f,-0.25299999420344831957407905065338127315044403076171875f,-0.0512999978184700833327980262765777297317981719970703125f,-0.2534999980740249014843357144854962825775146484375f,0.189399989409744673451996277435682713985443115234375f,0.0150999835669995974285484408028423786163330078125f,0.0634000026792289617105780052952468395233154296875f,-0.6981999933004379155221386099583469331264495849609375f,-0.31190000166594955555154911053250543773174285888671875f,0.034399982219934788219006804865784943103790283203125f,-0.7921000366002324266645473471726290881633758544921875f,-0.374200008752942014922382441000081598758697509765625f,0.866599969232082489867252661497332155704498291015625f,-0.37480003264248384908796651870943605899810791015625f,0.64319999136328664945949640241451561450958251953125f,-0.41540001497641221295253899370436556637287139892578125f,0.11319999624341725574794281783397309482097625732421875f,0.91710000312849881964893938857130706310272216796875f,0.186800009977817094153351717977784574031829833984375f,1.7449000033572310552898443347658030688762664794921875f,-0.4551999818116427842795701508293859660625457763671875f,0.4249000067472457065065327697084285318851470947265625f,-0.1219999951422214845564440111047588288784027099609375f,-0.62370001006051889458348114203545264899730682373046875f,-0.180599993453919915875616197808994911611080169677734375f,0.20810000304654174829721569039975292980670928955078125f,0.3847999753475193074336857534945011138916015625f,-0.179200051966309725770543082035146653652191162109375f,0.43219999505877471079884344362653791904449462890625f,-0.57780003172606253514231866574846208095550537109375f,0.0030000011324882169816419263952411711215972900390625f,-0.36869999717101453740042416029609739780426025390625f,0.09539999569803481183072335625183768570423126220703125f,-1.074800001517683512020084890536963939666748046875f,-0.0230000100731846401913571753539144992828369140625f,-0.699500001251697423043651724583469331264495849609375f,0.363799983960390260762096659163944423198699951171875f,0.47840002387613100864882653695531189441680908203125f,0.501799998205899822067976856487803161144256591796875f,1.3659000342324372212488015065900981426239013671875f,0.4261999939858911456980194998322986066341400146484375f,0.06150000950694067824997546267695724964141845703125f,-0.385699982869625035419858249952085316181182861328125f,-0.93859997927546512386243193759582936763763427734375f,-0.560000000074505788205669887247495353221893310546875f,-0.2029000078797338346703327260911464691162109375f,-0.611899999281764195302457665093243122100830078125f,-0.952499961555004492907983149052597582340240478515625f,-0.4411999863862992032892407223698683083057403564453125f,-0.37410000752806649160220331395976245403289794921875f,-0.13659998987019061900127780972979962825775146484375f,-0.636999969661236065121556748636066913604736328125f,-0.443099995875358754204853539704345166683197021484375f,-0.94040001339316336981255517457611858844757080078125f,-0.3271000087350603990898889605887234210968017578125f,-0.311200012612342646889374009333550930023193359375f,0.7643000029742719192427102825604379177093505859375f,1.052499995231628293623771241982467472553253173828125f,1.1532999987721439527632583121885545551776885986328125f,0.35560000287890414227831570315174758434295654296875f,-0.158099961602687688611013072659261524677276611328125f,-1.09619995911717449388333989190869033336639404296875f,-0.778599976071715271785933509818278253078460693359375f,-0.31440000591278050023902324028313159942626953125f,0.0379999715536829540951657691039144992828369140625f,0.469499993279576255389429206843487918376922607421875f,0.3552000198096039174089355583419091999530792236328125f,-0.65329998506307607186727182124741375446319580078125f,-1.143299997431039827233689720742404460906982421875f,-0.764599976399541247928937082178890705108642578125f,0.4435999706327919955839433896471746265888214111328125f,0.0502999874696137272422902242396958172321319580078125f,0.74159997340440764190816480549983680248260498046875f,0.3209000198900702205406787470565177500247955322265625f,0.6623999803721904999775915712234564125537872314453125f,0.0166000024244190580446911553735844790935516357421875f,0.6009999703764918876913725398480892181396484375f,-0.8642000249952077606252487385063432157039642333984375f,0.13219999014139194315475833718664944171905517578125f,-0.8078999862730495973295319345197640359401702880859375f,0.13419999780058855520792349125258624553680419921875f,-0.5172000085815791070587010835879482328891754150390625f,0.40979998533129702309452113695442676544189453125f,-1.00420002018958332001830058288760483264923095703125f,-0.11459999882578841834401828236877918243408203125f,-1.327100007356702970895412363461218774318695068359375f,-0.3959999854266644803146846243180334568023681640625f,-1.8219000205531721281460022510145790874958038330078125f,-0.2310000143349164591910493982140906155109405517578125f,-0.0258000257059929349878757420810870826244354248046875f,0.284799992930889089848278672434389591217041015625f,1.251400009697675841380259953439235687255859375f,0.7319999979734415074261733025196008384227752685546875f,-0.68589999634623577406955519109033048152923583984375f,0.10939998147487683954892645488143898546695709228515625f,-0.8852000165313480550821623182855546474456787109375f,0.72719998373389227452889826963655650615692138671875f,0.8346999569490545223260369311901740729808807373046875f,0.508599988067150121651138761080801486968994140625f,0.487799981695413720217402442358434200286865234375f,0.760300002557039444894826374365948140621185302734375f,0.46529999302029612096021082834340631961822509765625f,-0.63770001697838296639275768029619939625263214111328125f,-0.863800002456456272881268887431360781192779541015625f,-0.4505999930068849135267328165355138480663299560546875f,0.09820001162886615730940320645458996295928955078125f,-0.077600027599931120647624993580393493175506591796875f,0.981899993544816940271857674815692007541656494140625f,0.3023999941557644621781264504534192383289337158203125f,-0.45590000022053711603575720801018178462982177734375f,0.139000000193715056440169064444489777088165283203125f,-0.14190001023411724645484355278313159942626953125f,-0.038700040999054940993318041364545933902263641357421875f,1.0199999702535571532990843479637987911701202392578125f,0.07879999066591292233852072968147695064544677734375f,-0.101600000450015048869545353227294981479644775390625f,-0.383999999448656925693512675934471189975738525390625f,-0.55979999956190573673353583217249251902103424072265625f,0.4938000110805031983574053811025805771350860595703125f,0.36970001934766738571624955511651933193206787109375f,0.329500012129545272188124727108515799045562744140625f,0.72340000215768807123595252051018178462982177734375f,0.27699998843669870751682537957094609737396240234375f,-0.10560000466704355659430802916176617145538330078125f,0.008599975475668930613437623833306133747100830078125f,0.634599972450733407214329417911358177661895751953125f,0.1114000092506410855008880389505065977573394775390625f,-0.274399989074468886318669547108584083616733551025390625f,-0.16459999673962588673248319537378847599029541015625f,-0.434500001288950610245365169248543679714202880859375f,0.54799998310208319907133045489899814128875732421875f,0.14569996966347076305936525386641733348369598388671875f,-0.35759999120384444726283845739089883863925933837890625f,1.67760007321611137598438290297053754329681396484375f,0.37960000255107839706170125282369554042816162109375f,0.5635000428706413710955303031369112432003021240234375f,-0.59819997214078934888448202400468289852142333984375f,-0.474800008577108467733296492951922118663787841796875f,-0.48339997354745900537409397657029330730438232421875f,0.3824000545799732808660564842284657061100006103515625f,0.76039998180270185201834465260617434978485107421875f,0.3159000124953685872952746649389155209064483642578125f,-0.275999997906386684842772183401393704116344451904296875f,0.74199999449029563347579596666037105023860931396484375f,-0.27679999060630799956328473854227922856807708740234375f,0.11340000707507169597221263757091946899890899658203125f,-0.1891999828994277432769877123064361512660980224609375f,-0.40369996783435357912139807012863457202911376953125f,0.69679996750950845552097234758548438549041748046875f,-0.0467000084176654173262477343087084591388702392578125f,0.33200000423192932430538348853588104248046875f,-0.336199996295571512661126689636148512363433837890625f,0.43079999161958681952455663122236728668212890625f,0.4370000027418132759748914395458996295928955078125f,-0.0340000030994414004226200631819665431976318359375f,0.0139999898374079112528534096782095730304718017578125f,0.3989999888688327001062816634657792747020721435546875f,0.66100000412762149881729101252858527004718780517578125f,0.112599993103742523903321171019342727959156036376953125f,0.04959999890029431757199063213192857801914215087890625f,0.038300008755922043679476018951390869915485382080078125f,-0.774099976831674752020262531004846096038818359375f,-1.08909997735321528722352013573981821537017822265625f,-0.81819996439218556361083756200969219207763671875f,-0.8724999751150608329197666535037569701671600341796875f,-0.69379998850524426945440836789202876389026641845703125f,-0.611499986335635181688985539949499070644378662109375f,-0.0004999825060370000073817209340631961822509765625f,-0.531399992002546728286915822536684572696685791015625f,-0.067099999291449774485585066940984688699245452880859375f,0.183699994712322844936380761282634921371936798095703125f,-0.15520000386536114689306486980058252811431884765625f,0.366899991160631433473326978855766355991363525390625f,0.49299998131394406897243243292905390262603759765625f,0.538500002339481920188291041995398700237274169921875f,-0.22899996361136487621479318477213382720947265625f,0.1673000201255084029572373083283309824764728546142578125f,0.538599980709701942060974033665843307971954345703125f,0.5697999940857292333618033808306790888309478759765625f,0.9192999976098537029400858955341391265392303466796875f,0.80399998387694304735617834012373350560665130615234375f,-0.11469999580383305470832056016661226749420166015625f,0.06040000430345526805098188560805283486843109130859375f,-0.247499984912574466644485937649733386933803558349609375f,-0.250299992573261409489759898860938847064971923828125f,-0.05790002173781427785570485866628587245941162109375f,0.1139999934136868287026800317107699811458587646484375f,-0.2252000023007389994944560385192744433879852294921875f,-0.3209999861717225400070674368180334568023681640625f,-0.01529997227042922958162307622842490673065185546875f,0.742799965006113449561553352396003901958465576171875f,0.86889997978359467811770855405484326183795928955078125f,0.33859998136162783310965096461586654186248779296875f,0.7545999847069382315822849705000407993793487548828125f,0.3569999899268150311826275355997495353221893310546875f,1.33149999352544501363126983051188290119171142578125f,0.34819998346567171410015362198464572429656982421875f,1.003399980979412742243539469200186431407928466796875f,0.0336000005096196707743416709490702487528324127197265625f,-0.5874999837763608123708536368212662637233734130859375f,0.4369999771118167064543058586423285305500030517578125f,-1.1899999994412058246240349035360850393772125244140625f,-0.13619998236298602733995721791870892047882080078125f,-1.5098000129356983212147724771057255566120147705078125f,-0.05180001727938599032086131046526134014129638671875f,0.01879997645393027827509513372206129133701324462890625f,-0.39199997871369152147025261001545004546642303466796875f,0.2021000226490203832696579411276616156101226806640625f,-0.276199976849556438907029587426222860813140869140625f,-1.2785999663859606290117199023370631039142608642578125f,0.26819998137950928907002889900468289852142333984375f,-1.0928000025942916639820623458945192396640777587890625f,-0.13599999950826158201522275703609921038150787353515625f,-0.71740002157390136972026084549725055694580078125f,-0.066200010824203570081181169371120631694793701171875f,0.1385999946609139943376476367120631039142608642578125f,0.42249999135732674293564059553318656980991363525390625f,0.27149998236820127583968087492394261062145233154296875f,0.468099992615729487699383071230840869247913360595703125f,-0.195499991893768321205016036401502788066864013671875f,0.6137000135064119010763761252746917307376861572265625f,0.256500032007693778979273702134378254413604736328125f,0.4742000135958195361496336772688664495944976806640625f,1.21619998847246169049185482435859739780426025390625f,0.90149999281764003900008219716255553066730499267578125f,0.172800003376603061866489952080883085727691650390625f,0.1091999946713446689727788907475769519805908203125f,0.16089999761283391155330946276080794632434844970703125f,-0.030099977830052304650365613269968889653682708740234375f,-1.0238999708056450987925245499354787170886993408203125f,0.24480001327097367092022750512114726006984710693359375f,1.37170001955628340084558658418245613574981689453125f,1.3359000177294013678164219527388922870159149169921875f,0.377399987357109889529738211422227323055267333984375f,-0.2445000003948807343334692632197402417659759521484375f,-0.64729997951984419302107198745943605899810791015625f,-0.987799986016750342088243996840901672840118408203125f,-0.5182999843180180477020257967524230480194091796875f,-0.537100002476572857545988881611265242099761962890625f,0.3573999823465945357980899643735028803348541259765625f,0.23330000164061759360123460282920859754085540771484375f,-0.318999984771013433970665573724545538425445556640625f,-0.4054999984502789533991062853601761162281036376953125f,-0.19549998544901614483393359478213824331760406494140625f,0.310199987250566577845489746323437429964542388916015625f,-1.305500002790241698136242121108807623386383056640625f,-0.1915999737024313420707954946919926442205905914306640625f,-1.61579999566152654466577587299980223178863525390625f,-0.2371999762833123526206691167317330837249755859375f,-0.4985999673172833990264507519896142184734344482421875f,0.63019996474981354595001903362572193145751953125f,-0.3513000233739609345917642713175155222415924072265625f,-0.09299998250603724869733923696912825107574462890625f,-0.9614000045195225663263727255980484187602996826171875f,0.245199988144636282783039860078133642673492431640625f,-0.6463000226289035143878436429076828062534332275390625f,-0.3711999971151349608788905243272893130779266357421875f,-0.31099997488409203416637183181592263281345367431640625f,0.498599994623660780490581601043231785297393798828125f,-0.6280000119358302157479556626640260219573974609375f,-0.28440000188946701342729284078814089298248291015625f,-1.527300026924163223185360038769431412220001220703125f,0.34059998794049028969510573006118647754192352294921875f,0.14069999641105546128727610266651026904582977294921875f,0.2361999926447868602963353623636066913604736328125f,1.429799996581673671158796423696912825107574462890625f,0.0632000053703782871394878384307958185672760009765625f,0.61389997959733033638940469245426356792449951171875f,-0.876199953454733559965461608953773975372314453125f,0.66420002007782386055367851440678350627422332763671875f,0.3201999795019629946324357661069370806217193603515625f,-0.5463000289991495339592120217275805771350860595703125f,-0.0724000007122758137967366565135307610034942626953125f,0.28050000739097580293446299037896096706390380859375f,-0.51550000090897096782782682566903531551361083984375f,-0.34709999018311510354806159739382565021514892578125f,-0.7223999855130909253375648404471576213836669921875f,-0.8961999847471713298574513828498311340808868408203125f,-1.069400001144409362296983090345747768878936767578125f,-0.6435999764442446036127876141108572483062744140625f,0.2922000250101091101129213711828924715518951416015625f,0.528199980336427898919282597489655017852783203125f,-0.010800017249584126233230563229881227016448974609375f,-0.3009000109493731311971487230039201676845550537109375f,0.60670001732111000780633958129328675568103790283203125f,0.14419999750256540238524394226260483264923095703125f,0.2979999989718198349919475731439888477325439453125f,0.70350000695884207235764051802107132971286773681640625f,-0.380500001877546345507852265654946677386760711669921875f,-0.903399995452165427423096843995153903961181640625f,-0.30020000669658220004976101336069405078887939453125f,0.31310001092553108037463971413671970367431640625f,0.7968000277101996431383668095804750919342041015625f,-0.031200003969669154457733384333550930023193359375f,-0.3321000317573548077376699438900686800479888916015625f,0.4488999795973303008622679044492542743682861328125f,0.3487999921560287486244078536401502788066864013671875f,0.5610000071823595391151684452779591083526611328125f,-0.4251000208199027241562362178228795528411865234375f,-0.392299998444318731571911484934389591217041015625f,-0.9816000010460612656970624811947345733642578125f,0.994300020259618833051717956550419330596923828125f,0.00759999638199815308325923979282379150390625f,1.3002000033065674866605831994093023240566253662109375f,-0.11220000504255267514963634312152862548828125f,1.1013000384986408874965491122566163539886474609375f,0.548000000834464540133694754331372678279876708984375f,0.4609999868422745006313334670267067849636077880859375f,-0.31719998894929890553839868516661226749420166015625f,0.0309999831169840422973038585041649639606475830078125f,0.37200000333785965267452411353588104248046875f,-0.9555000645369300382725441522779874503612518310546875f,-0.464399991756677454901591772795654833316802978515625f,-0.8837000019207597034665013779886066913604736328125f,0.437599986398220064387487582280300557613372802734375f,-0.29790000929534421469924154735053889453411102294921875f,-0.1515999972462653833105150624760426580905914306640625f,0.9451000251784924532927334439591504633426666259765625f,0.17840000811815226455792071647010743618011474609375f,-0.2213000028848652878110669917077757418155670166015625f,-0.2146000150680535956126959717948921024799346923828125f,-0.8996999849259845749571695705526508390903472900390625f,0.8089999946057790936038145446218550205230712890625f,0.707900000019371145043578508193604648113250732421875f,-0.07739999214410797634400296374224126338958740234375f,0.481500026345253129278489723219536244869232177734375f,0.2393999914586544264238909818232059478759765625f,1.0265000321194530652491039290907792747020721435546875f,-0.082600007259845398266406846232712268829345703125f,1.84230004627704690989276059553958475589752197265625f,-0.331199975359439680033801778336055576801300048828125f,-0.6289999981820582064528935006819665431976318359375f,-0.24220001167356997040513988395105116069316864013671875f,0.4950000087171790852380581782199442386627197265625f,0.587000004231929661813182974583469331264495849609375f,0.758100001239776499772915485664270818233489990234375f,0.8627999779701231997108834548271261155605316162109375f,0.92309996219873458045412917272187769412994384765625f,0.902699999916553696976961873588152229785919189453125f,0.11020000960230813547013895004056394100189208984375f,-0.728599992910027527415195436333306133747100830078125f,-0.854399976166337893346280907280743122100830078125f,-0.451899994848668828950621900730766355991363525390625f,0.115300005686282958095034700818359851837158203125f,0.1245999989747998970557318898499943315982818603515625f,-0.0598000006794929817033334984444081783294677734375f,-0.27490001270771013963667428470216691493988037109375f,-0.391100022190808971345177269540727138519287109375f,0.3185999993175272493317606858909130096435546875f,0.86019998143911369226088936557061970233917236328125f,0.84429999582171444405531701704603619873523712158203125f,0.6533999916896224835483053539064712822437286376953125f,0.0067999891161918679660658426655572839081287384033203125f,-0.85240001065880033337407439830712974071502685546875f,-0.407099991971254127065549255348742008209228515625f,0.1037000073969363000969678978435695171356201171875f,0.2118999989837409003712309640832245349884033203125f,0.519799977582693362165855432976968586444854736328125f,-0.311900029978156023702240418060682713985443115234375f,-0.09760000621676423460115756824961863458156585693359375f,0.49839999917745547719505339046008884906768798828125f,0.0964000048547981602808931711479090154170989990234375f,-0.415199999767541516604296703008003532886505126953125f,0.4101000461518771178504039198742248117923736572265625f,0.1561999983072279718498975853435695171356201171875f,-0.9122000104069714065957441562204621732234954833984375f,-0.07800000306963905671864267787896096706390380859375f,-0.6684999945536256316103163044317625463008880615234375f,0.8163999681234361815995725919492542743682861328125f,-0.05240003463104347947165706500527448952198028564453125f,-0.03859999846071004991898689695517532527446746826171875f,-0.0141000072859227554289418549160473048686981201171875f,-0.794599983775615470449338317848742008209228515625f,0.245900032183528072238232198287732899188995361328125f,-0.4018000011861320164285871214815415441989898681640625f,0.0093000025644898176579999926616437733173370361328125f,-0.54259998937845210775776649825274944305419921875f,-0.04970000951290121093961715814657509326934814453125f,-0.72419997127652191437618967029266059398651123046875f,-0.2275999885588884108500451475265435874462127685546875f,-0.27199998567998429077618993687792681157588958740234375f,-0.34680000804066690722748944608611054718494415283203125f,0.0482000023156403667456970651983283460140228271484375f,0.6528999885082242826683796010911464691162109375f,-0.02039999819397930735931367962621152400970458984375f,0.8431000159546730987614182595279999077320098876953125f,0.55539998925328237788789920159615576267242431640625f,1.032100005270540687973834792501293122768402099609375f,-0.146299995881319200297099314411752857267856597900390625f,0.24350000066310162338822919991798698902130126953125f,-0.0827000047594308451692768358043394982814788818359375f,0.09039999476671223277435274212621152400970458984375f,-0.428900004854798400089066490181721746921539306640625f,0.210900007671117606378174968995153903961181640625f,-0.3330999916285275475047455984167754650115966796875f,-0.313099997961521214762115050689317286014556884765625f,-0.0853999759912487110824486080673523247241973876953125f,-0.06959999420642848377838163287378847599029541015625f,-0.658000028252601953937528378446586430072784423828125f,-0.26500001102685910581158168497495353221893310546875f,0.16519999529719342490352573804557323455810546875f,-0.62309999721646303072475348017178475856781005859375f,0.078400013855099981441298950812779366970062255859375f,0.013999997735023583800284541212022304534912109375f,-0.26569999531209465004621961270458996295928955078125f,-0.66509999250769613610145825077779591083526611328125f,-0.430900000295042939768563883262686431407928466796875f,0.041199997487664230533255249611102044582366943359375f,0.2384999942928553995358242900692857801914215087890625f,0.409099985474348226688334762002341449260711669921875f,0.6436999927937983034809121818398125469684600830078125f,0.530700000602006838335000793449580669403076171875f,-0.16400001166760935777944041547016240656375885009765625f,-0.116899996972084085911092188325710594654083251953125f,-0.5558000041365624976918979882611893117427825927734375f,-0.36449999037384994693411499611102044582366943359375f,-0.73290000132322308257215581761556677520275115966796875f,0.5264999999701978072863539637182839214801788330078125f,0.127599992284178842538722165045328438282012939453125f,0.476299995359778349790502716132323257625102996826171875f,-0.123399995452165622822349178022705018520355224609375f,2.008500015266239113742585686850361526012420654296875f,-0.02920001359581902988793444819748401641845703125f,0.1405999639496204878241769620217382907867431640625f,-0.2262000024795529640186941833235323429107666015625f,0.4134999960064893453903778208768926560878753662109375f,-0.083999975085259137586035649292171001434326171875f,0.864200023132562389349686782225035130977630615234375f,-0.55319998376369472481428601895458996295928955078125f,-0.6218999540567395900581004752893932163715362548828125f,0.4753999765872958249701696331612765789031982421875f,-0.61870000821649995259576826356351375579833984375f,0.6901999920189376691581628620042465627193450927734375f,-0.449500048749149616611475721583701670169830322265625f,0.5659999836385249949444187222979962825775146484375f,-0.1231000260874630658491923895780928432941436767578125f,-0.21499999657273283215630499398685060441493988037109375f,-0.22289998735338401036898403617669828236103057861328125f,-0.3823999764978889270850004322710447013378143310546875f,-0.028199970315396838405064272592426277697086334228515625f,0.21299999338388442282621326739899814128875732421875f,0.311000000029802325940408991300500929355621337890625f,-0.25419999720454200531349897573818452656269073486328125f,1.589600039654970320412985529401339590549468994140625f,0.0503999871671203525380633436725474894046783447265625f,1.172200019664317860446089980541728436946868896484375f,0.81690000390112427197664146660827100276947021484375f,1.159999977648258351337062777020037174224853515625f,0.7725999916136263578891885117627680301666259765625f,0.071500006824731787702376095694489777088165283203125f,-0.3351000000327826189305824300390668213367462158203125f,0.45410001512765862941023442544974386692047119140625f,0.094099978172778708795931379427202045917510986328125f,1.17999998196959499097147272550500929355621337890625f,0.425899966171383592694610342732630670070648193359375f,0.41659998636841777397421537898480892181396484375f,0.367800010082125705679345628595910966396331787109375f,0.2192000140428540522208322727237828075885772705078125f,-0.67559999133050430142333198091364465653896331787109375f,-0.821799994480609807823157098027877509593963623046875f,-0.631299999159574465323885306133888661861419677734375f,-0.400199996936321245044609895558096468448638916015625f,-0.04339998926818378066627701628021895885467529296875f,-0.552799983036518227663691504858434200286865234375f,-0.12759998982548725887653517929720692336559295654296875f,-0.126699993702769353465242829770431853830814361572265625f,0.2206000017240645483251881842079455964267253875732421875f,0.64419998528361333001157618127763271331787109375f,0.0103000016629692936476203612983226776123046875f,-1.21299996894598027807887774542905390262603759765625f,-1.3122999938249588769423326084506697952747344970703125f,-0.84209999510049815540924100787378847599029541015625f,-0.683699979010224279107887923601083457469940185546875f,-0.31379998083114646334479402867145836353302001953125f,0.091000030949711874228569286060519516468048095703125f,-0.22589998863488414126976522311451844871044158935546875f,-0.046399981236458387456877971999347209930419921875f,-0.545699993002414895926222015987150371074676513671875f,0.439599990183114908148809263366274535655975341796875f,1.23980000921040822703389494563452899456024169921875f,-0.18399998149275820225057032075710594654083251953125f,1.2853000281497835022292974826996214687824249267578125f,-0.7821999786674975752021055086515843868255615234375f,0.70140005183070908145737121230922639369964599609375f,-0.68239999512433957562507202965207397937774658203125f,0.8542000329673291592058603782788850367069244384765625f,0.403599974656105420223184410133399069309234619140625f,0.9199999852478504269726045095012523233890533447265625f,0.3517999904572963743021318805404007434844970703125f,0.0659999932125210353461852719192393124103546142578125f,0.11839999411106116866676529753021895885467529296875f,-1.0550000038743017061193540939711965620517730712890625f,0.206399989283085005808970890939235687255859375f,-0.26130000146925447257473251738701947033405303955078125f,0.174999989941716338481825232520350255072116851806640625f,-0.5085000120997424932767216887441463768482208251953125f,-0.3389999749362473391300909497658722102642059326171875f,-0.4141999938145282111889855514164082705974578857421875f,0.33099998304247879588046998833306133747100830078125f,0.7808999911680818417636373851564712822437286376953125f,-0.41959999018311489038524086936376988887786865234375f,1.563500003941356908399029634892940521240234375f,0.0771999940156937736190911891753785312175750732421875f,0.093499957934021715733052815267001278698444366455078125f,0.658000011190771783020636576111428439617156982421875f,-0.153299989160895488993219260009936988353729248046875f,-0.5197999987423418399856700489181093871593475341796875f,0.03510000100135801659462231327779591083526611328125f,-0.71049999755620962815783059340901672840118408203125f,-1.0150999874413013657203919137828052043914794921875f,-0.650099982151389088613768763025291264057159423828125f,-0.6154999783709647065421677325502969324588775634765625f,-0.07759998669624323941462762377341277897357940673828125f,-0.17530000345110874349074947531335055828094482421875f,-0.526400032049417543333902358426712453365325927734375f,0.45399998879432690301882757921703159809112548828125f,0.659300006103515467970055397017858922481536865234375f,-0.090899991428852189301323960535228252410888671875f,0.455400016522407469210520503111183643341064453125f,0.117400007790326998957652904209680855274200439453125f,-0.1945999835520983989312071571475826203823089599609375f,-0.7588999745309354949540647794492542743682861328125f,-0.621299992826580993465768187888897955417633056640625f,0.10569999508857730319277834496460855007171630859375f,-0.181200008514523336344836934586055576801300048828125f,-0.1812000115692613544382538748322986066341400146484375f,0.58859999015331265770356594657641835510730743408203125f,1.30739998260736456359154544770717620849609375f,0.8839000080585481811112913419492542743682861328125f,-0.006400007462501466903859181911684572696685791015625f,0.2439000064194203343248545934329740703105926513671875f,0.218800005120038942862947806133888661861419677734375f,-0.5850000029057265127363507417612709105014801025390625f,-0.3014999597370626727155240587308071553707122802734375f,0.73219996331930203847804250472108833491802215576171875f,0.5607999790281059659946549800224602222442626953125f,-0.236799990159273221479452331550419330596923828125f,0.9373999994829294024611954228021204471588134765625f,0.08239999199509639993266318924725055694580078125f,0.0970000038594009961201436453848145902156829833984375f,0.1741999904245139507708017845288850367069244384765625f,0.023600029529631694202862490783445537090301513671875f,0.7373999936342234828856589956558309495449066162109375f,0.512700009453296967620872237603180110454559326171875f,-0.758599961990118476506950173643417656421661376953125f,-0.87029999618679310113833480500034056603908538818359375f,0.19739999721050249803511178470216691493988037109375f,-0.9329000124990940445712794826249592006206512451171875f,0.4227999898910521636707926518283784389495849609375f,-0.1875000227242711314801226762938313186168670654296875f,0.790799973589181970368144902749918401241302490234375f,0.42199996403604711492363321667653508484363555908203125f,-0.0818000010371207064707732570241205394268035888671875f,0.076300002586841830787278695424902252852916717529296875f,-0.270399984037876350839724182151257991790771484375f,-0.30880001793503819573771806972217746078968048095703125f,-0.67819998957514737725205122842453420162200927734375f,-0.8848000315621498135243427896057255566120147705078125f,-0.110199999767541800821391007048077881336212158203125f,0.41840002484470628640877976067713461816310882568359375f,0.44239998886585230053469786071218550205230712890625f,1.89510000763237496101965007255785167217254638671875f,0.45190001561716197642226688913069665431976318359375f,0.6211000095248220276289430330507457256317138671875f,0.1230000188648698866700215148739516735076904296875f,-0.59489997634887714639262412674725055694580078125f,-0.7046999872356654037730550044216215610504150390625f,-0.074199987220764374029613463790155947208404541015625f,-0.2269999794214963007021879093372263014316558837890625f,0.0418000117659567127503805750166065990924835205078125f,-0.11759999571144585939208582203718833625316619873046875f,0.01830000227391714862079652448301203548908233642578125f,0.330200000624358558365400995171512477099895477294921875f,0.07469999731257559449915106597472913563251495361328125f,0.261299994968622939506275315579841844737529754638671875f,0.4505999949812888871747418306767940521240234375f,-0.32510000204443922910968467476777732372283935546875f,-0.631599991434812491064576533972285687923431396484375f,-0.85400002627074744765423019998706877231597900390625f,-0.0393000072956084522957098670303821563720703125f,0.07009999183714388237831371952779591083526611328125f,0.587799997937679119530685056815855205059051513671875f,0.8605000134259463973052106666727922856807708740234375f,0.0038999904751779723710569669492542743682861328125f,-0.5963000240445135435862766826176084578037261962890625f,0.14689999395459896192051019170321524143218994140625f,-0.235599969500303263458818037179298698902130126953125f,-1.06849996322393447911736075184307992458343505859375f,-1.18419996680617334305907206726260483264923095703125f,-1.2153999721169472802984046211349777877330780029296875f,-0.859099996724725034624725594767369329929351806640625f,-0.2391000163383782961545875878073275089263916015625f,0.3609999930262564049598950077779591083526611328125f,-0.210000026002526762880506794317625463008880615234375f,0.9079999667108060723563767169252969324588775634765625f,0.6826999575600023462129684048704802989959716796875f,0.1663999827265743025606070659705437719821929931640625f,0.6555999803781509260858229026780463755130767822265625f,0.225199988591671118598469547578133642673492431640625f,-0.0253000156700611267268641313421539962291717529296875f,-0.13899999788403505363021395169198513031005859375f,-0.54759998712465163261953193796216510236263275146484375f,0.2211999865353109495647032645138096995651721954345703125f,-0.04989999301955094779259525239467620849609375f,0.117799991828203332033808692358434200286865234375f,0.649000016324222439578761623124592006206512451171875f,0.31639999643564209463875158689916133880615234375f,1.3051000357210635893778771787765435874462127685546875f,-0.35400000086426697265551410964690148830413818359375f,0.21130001067072168297045209328643977642059326171875f,-0.6401999949991699079276941120042465627193450927734375f,-0.3542999636724586398628389360965229570865631103515625f,0.819999981373548347818314141477458178997039794921875f,0.76479998853504660427660155619378201663494110107421875f,-0.3793999806553127740471609286032617092132568359375f,0.314799973894655138462184140735189430415630340576171875f,0.75459997602701189833851458388380706310272216796875f,0.3368999597936868095615636775619350373744964599609375f,0.377399995571374713421164415194652974605560302734375f,-0.179800046481938036180281414999626576900482177734375f,0.116700019630789686431171503500081598758697509765625f,-0.670399994468688920079557647113688290119171142578125f,-0.195999988853931605969904694575234316289424896240234375f,-0.6360999752148988051203559734858572483062744140625f,-0.33129998276829741854498934117145836353302001953125f,-0.380199980358779665579049833468161523342132568359375f,-0.303199977095424788586797149037010967731475830078125f,-0.15160000365376458120181268895976245403289794921875f,-0.384700002285838227322756210924126207828521728515625f,-0.4404999860823155533040562659152783453464508056640625f,-0.24069999255537999349741085097775794565677642822265625f,-0.43659999620318412638653171597979962825775146484375f,0.773400009682774669528271260787732899188995361328125f,1.19129998619556420891285597463138401508331298828125f,1.1136000089287758640921310870908200740814208984375f,0.6801000053226946562290322617627680301666259765625f,-0.203200015577673920308399146961164660751819610595703125f,-0.5349000124655660659556133396108634769916534423828125f,-0.493800023075938288030783951398916542530059814453125f,0.18869999700337647396963802748359739780426025390625f,-0.4232000101760029497910409190808422863483428955078125f,-0.64260000487565971383219221024774014949798583984375f,-0.742600013965368344059925220790319144725799560546875f,-0.047499999850988405825091831502504646778106689453125f,-0.0183000028699635830520264789811335504055023193359375f,1.0768999936938283834564344942918978631496429443359375f,0.469500021144747758938819970353506505489349365234375f,-0.6241999789319934155429336897213943302631378173828125f,0.091600011402368064494794452912174165248870849609375f,-0.6620000297129140509611033849068917334079742431640625f,-0.077999984294176538668352804961614310741424560546875f,-0.08590001314729411863169161733821965754032135009765625f,-0.6007999857336283611175531405024230480194091796875f,0.2360999756246808800597136723808944225311279296875f,-0.500399976512790001237362957908771932125091552734375f,0.4677999777093535538341484425473026931285858154296875f,-0.144399980655312898392139686620794236660003662109375f,0.86649999386072096285715815611183643341064453125f,-1.32399998097121684992316659190692007541656494140625f,0.97670005129575798719088197685778141021728515625f,0.20679998472034899492655313224531710147857666015625f,0.1092000058472155643585210782475769519805908203125f,0.553200010362267402541647243197076022624969482421875f,-0.423600032025575057303967696498148143291473388671875f,0.05680001422464819871294139375095255672931671142578125f,-0.79649999102950186369298535282723605632781982421875f,0.354799961265922281228313295287080109119415283203125f,-0.913300036397576064928216510452330112457275390625f,0.27660002116262882854158533518784679472446441650390625f,0.25429998174011725353693691431544721126556396484375f,-0.3145999897360802766144161068950779736042022705078125f,0.6314000582382084392207843848154880106449127197265625f,-0.52420004526078667339561434346251189708709716796875f,0.2413999848127368341721421529655344784259796142578125f,-0.5031999801874160738179853069595992565155029296875f,-0.289700014691054885673793251044116914272308349609375f,-0.1511999990522860937147697768523357808589935302734375f,-0.4414000562265527616290228252182714641094207763671875f,1.297200009587406999145287045394070446491241455078125f,0.169099994787573759680299190222285687923431396484375f,-0.20360002859830961252640690872794948518276214599609375f,-0.2499000067844987515996990623534657061100006103515625f,-1.478900005003809692283311960636638104915618896484375f,-0.2784999931752680790708609492867253720760345458984375f,-1.018199996876715918148192940861918032169342041015625f,0.066400000011920923270736238919198513031005859375f,-0.541000038251281178958151940605603158473968505859375f,-0.45730001207888137937374040120630525052547454833984375f,-1.5154000663667923642918822224601171910762786865234375f,-0.1353000020354986343562586625921539962291717529296875f,0.444599983365833395509980618953704833984375f,0.32999999610707153596678153917309828102588653564453125f,0.9205000007227053249181381033849902451038360595703125f,0.14999999795109031364148677312186919152736663818359375f,1.1873000383794309886553719479707069694995880126953125f,0.17300001081824312176848934541339986026287078857421875f,0.181000005021692000894972807145677506923675537109375f,-0.1584000042065978330896314218989573419094085693359375f,0.67570001431107495903916060342453420162200927734375f,0.342600007110834159362866557785309851169586181640625f,0.92840002942681376651989921811036765575408935546875f,-0.262300000865757443335724019561894237995147705078125f,-1.29010005635917224253716995008289813995361328125f,-0.207600012959539947932086079163127578794956207275390625f,-0.02410005998015496420094905261066742241382598876953125f,-0.1123000030264258430889867668156512081623077392578125f,0.0446999826908107689149574071052484214305877685546875f,1.1310999815106388410868021310307085514068603515625f,-1.008199967297911836539014984737150371074676513671875f,0.605799972248076468162025776109658181667327880859375f,-0.15499999403953512722154073344427160918712615966796875f,-1.07699998009204822579931715154089033603668212890625f,1.349999962821602839113666050252504646778106689453125f,-1.2429999749064446490365298814140260219573974609375f,0.622399971878528912583306009764783084392547607421875f,-0.67000005587935529405285706161521375179290771484375f,0.584200022462010526425046919030137360095977783203125f,-0.41250006377697001624937911401502788066864013671875f,-0.2131999674469235994678228962584398686885833740234375f,0.256099986726045614204849698580801486968994140625f,0.08520003225207262165241672846605069935321807861328125f,0.46419998532086548248543067529681138694286346435546875f,-0.5149999944120644546075027392362244427204132080078125f,1.10720003603696870442263389122672379016876220703125f,-0.8484000186979769875250667610089294612407684326171875f,0.6692999774932868461974067031405866146087646484375f,-0.376799954470992570776388674858026206493377685546875f,0.87490000793933830891546676866710186004638671875f,-1.22840001631379092117413165397010743618011474609375f,0.761499984622002301648535649292171001434326171875f,-0.36639997155070336276594389346428215503692626953125f,-0.777400040870904973644428537227213382720947265625f,0.463000033691525203494165907613933086395263671875f,-0.395700051265955199397694741492159664630889892578125f,-0.60079998416900626256875739272800274193286895751953125f,0.0025999741792670505446949391625821590423583984375f,-1.2480999851465213179579905045102350413799285888671875f,-0.17019999641478056329191304030246101319789886474609375f,-0.250600014948844318496412597596645355224609375f,-0.2004000037819146928796953943674452602863311767578125f,-0.124800015208125447685461040236987173557281494140625f,0.1407000024646520852655839917133562266826629638671875f,0.654200040343404065623644783045165240764617919921875f,0.03220000284463164774706456228159368038177490234375f,-0.0238999739348884077116963453590869903564453125f,0.1912000059813261376717719031148590147495269775390625f,-0.71760000897347964166073097658227197825908660888671875f,-0.166300012198090652049131676903925836086273193359375f,-0.17700004521012380198641267270431853830814361572265625f,0.066899999468028548132991772945388220250606536865234375f,0.760800004881620584029633391764946281909942626953125f,-0.0131999965041875721993847037083469331264495849609375f,0.64740004221201008505204299581237137317657470703125f,0.2457000080153346477374043388408608734607696533203125f,0.01250003874301963691095807007513940334320068359375f,-0.1501999976813792603280717230518348515033721923828125f,-1.4598000134199853317085171511280350387096405029296875f,-0.257699999693036030379289513803087174892425537109375f,-1.08860002450048920508152150432579219341278076171875f,-0.17160000391453511525696740136481821537017822265625f,-0.5001000165730717927914383835741318762302398681640625f,0.059599999533593628342487136251293122768402099609375f,0.15530000739991667302319910959340631961822509765625f,-0.03539999614506961211901625574682839214801788330078125f,-0.36329999161958692610596699523739516735076904296875f,-0.75239999721050221381801748066209256649017333984375f,0.1707999871492387189420014692586846649646759033203125f,-0.04200002032518312944375793449580669403076171875f,-0.19339996877908749439711755258031189441680908203125f,0.3921000082135197573052209918387234210968017578125f,-0.855000016316771027646836955682374536991119384765625f,0.360600047731399087069803499616682529449462890625f,0.603799961534142948238468306954018771648406982421875f,0.755600035214424536889055161736905574798583984375f,-0.29120000143647217782927327789366245269775390625f,-0.730799999368191510740189187345094978809356689453125f,1.267199997964501445579799110419116914272308349609375f,-0.7066000384852291205106666893698275089263916015625f,-0.46419998502284298691478170439950190484523773193359375f,-0.00810004320517154763336975520360283553600311279296875f,-0.62179997957944888042902675806544721126556396484375f,-0.0449999874830240287337801419198513031005859375f,0.25459999510049780013787312782369554042816162109375f,-0.5115999742984769227405195124447345733642578125f,0.83219997464418415944464868516661226749420166015625f,-1.41209998079538312509839670383371412754058837890625f,1.0513999714761974590970794452005065977573394775390625f,-0.9977999884009367548287627869285643100738525390625f,1.15619997260272544536974237416870892047882080078125f,0.53609999134540498033629773999564349651336669921875f,-0.7783999947816131470545997217413969337940216064453125f,0.9081999888300895662496259319595992565155029296875f,-0.31819998979866515043113395222462713718414306640625f,0.58369999249577464439653340377844870090484619140625f,0.108800006312132158114991398178972303867340087890625f,-0.0935000042021274513359685442992486059665679931640625f,0.16299998265504822558824571387958712875843048095703125f,-0.050700001756846913014697975086164660751819610595703125f,-1.1636999872803688305111791123636066913604736328125f,-0.2515999959796666463063274932210333645343780517578125f,-0.553200011405348135440362966619431972503662109375f,-0.0338000047847628604102965255151502788066864013671875f,0.8765000004172325276385890902020037174224853515625f,0.26570000540763138996425141158397309482097625732421875f,0.79670004011988648784381439327262341976165771484375f,-0.01970000180155040414575751128722913563251495361328125f,0.39329998938441246281172425369732081890106201171875f,0.22670000141412016692044062438071705400943756103515625f,0.434300047975778369391264277510344982147216796875f,0.086400001874566090265261664171703159809112548828125f,-0.9860000187307595975738649940467439591884613037109375f,-0.16250000461935998341544973300187848508358001708984375f,-0.412800037649274198514603995135985314846038818359375f,0.1173000006049871313251742321881465613842010498046875f,0.57060000101625885093881151988171041011810302734375f,0.33990000086128713263633471797220408916473388671875f,0.61420001471340679444210763904266059398651123046875f,-0.3530999969929455861716860454180277884006500244140625f,-0.330600015470384978044648960349150002002716064453125f,-0.0289000006079673799064266859204508364200592041015625f,-0.55979998823702292742865438412991352379322052001953125f,-0.217399994379281924494762279209680855274200439453125f,0.46820000436455055936590952114784158766269683837890625f,-0.2433000133007763299275438839686103165149688720703125f,-0.181599987486004810222084415727294981479644775390625f,-0.590500011116266154687082234886474907398223876953125f,0.34160000082254404762949206997291184961795806884765625f,-1.528499962627888120181296471855603158473968505859375f,1.57799993154406603679262843797914683818817138671875f,-0.95990000376701356543662768672220408916473388671875f,-0.50059999788701503575794049538671970367431640625f,-0.041300001990794754647140507586300373077392578125f,-0.2395999948024751802222453989088535308837890625f,0.2810000100135798817291288287378847599029541015625f,0.30259999071955689942114986479282379150390625f,1.2528999992370604132929656771011650562286376953125f,-1.190799981933832185632127220742404460906982421875f,0.9025000694394105238416159409098327159881591796875f,0.59119995256066371069891829392872750759124755859375f,-0.40559997769594158256722948863171041011810302734375f,0.8278000035256145583417719535646028816699981689453125f,0.1729000040054327058669514372013509273529052734375f,0.251400019308924083105694080586545169353485107421875f,0.738499999582767685524231637828052043914794921875f,-1.06139997937381291848168984870426356792449951171875f,-0.225300020587443583508502342738211154937744140625f,0.09960000292360771112498696311376988887786865234375f,0.0471999857455493998514839404379017651081085205078125f,-0.5479999899566172505416261628852225840091705322265625f,-0.43199998866021582788476962377899326384067535400390625f,0.5461999789357185886018442033673636615276336669921875f,0.386900014220178167789754297700710594654083251953125f,-0.515699965286254613516803146922029554843902587890625f,-0.0030999961361288796979351900517940521240234375f,0.9045000125020745684878420433960855007171630859375f,0.37479999654442064382919852505438029766082763671875f,1.61670003363788072903162174043245613574981689453125f,0.1818000040359795399513842539818142540752887725830078125f,0.6087000197835268355817106566973961889743804931640625f,0.1190000004172324654660997111932374536991119384765625f,0.6124000089824195303123133271583355963230133056640625f,0.229800000642239987680426338556571863591670989990234375f,-0.06599998866766709770814713920117355883121490478515625f,-0.09200000039488076541971395272412337362766265869140625f,-0.8609000200763343713106223731301724910736083984375f,-0.32010000176504249669307000658591277897357940673828125f,0.0523999849542970963511834270320832729339599609375f,0.359399994029104608017632926930673420429229736328125f,0.051299985599516872980530024506151676177978515625f,-0.171700007002055732296952328397310338914394378662109375f,-0.368900014317035840338832031193305738270282745361328125f,-0.11229999945014712547930457731126807630062103271484375f,-0.39630001227259625817822552562574855983257293701171875f,-0.077100001899153003392939353943802416324615478515625f,-0.171100033832341791306674849693081341683864593505859375f,-0.0789000096976757436806337864254601299762725830078125f,-0.32120003980696287815277401023195125162601470947265625f,-0.13050000530481342675415135090588591992855072021484375f,-0.1688000244170428931766991809126920998096466064453125f,0.1905000001639127749086810581502504646778106689453125f,0.186600004622339721294110859162174165248870849609375f,-0.4421999487161638597854107501916587352752685546875f,1.023599952304363736033110399148426949977874755859375f,-0.5952000115394593393602917785756289958953857421875f,-0.3157999779850246824253190425224602222442626953125f,0.341899997363239283032498860848136246204376220703125f,-0.41260000644028183014455635202466510236263275146484375f,0.39100001506134873796582951399614103138446807861328125f,0.442800004047155137953950543305836617946624755859375f,1.37819999937266057798979090875945985317230224609375f,-0.8521999870121479414564191756653599441051483154296875f,0.60179999988228072282936409465037286281585693359375f,0.032800004568695972917424796833074651658535003662109375f,-0.682700027632713091207961042528040707111358642578125f,0.40360000520348549013505135008017532527446746826171875f,0.11210001726597440541155492610414512455463409423828125f,-0.244800011482834367626537641626782715320587158203125f,-0.807400033718348453248836449347436428070068359375f,0.8158000041365625065736821852624416351318359375f,-0.8967000329673291414422919842763803899288177490234375f,0.189800001573562526768768066176562570035457611083984375f,-0.5175000080652534251157703693024814128875732421875f,0.25360000088214873503744684057892300188541412353515625f,-0.50190003442987818349507733728387393057346343994140625f,-0.27719997307956223409064477891661226749420166015625f,-0.920000010132790091432752888067625463008880615234375f,0.90819998756349107349450378023902885615825653076171875f,-0.5849000034689897375983491656370460987091064453125f,0.2626000191062682898035518519463948905467987060546875f,0.012900032764672886287371511571109294891357421875f,-0.17450001466274311923143613967113196849822998046875f,0.15639999949038030369052876267232932150363922119140625f,-0.244399999132752565600412708590738475322723388671875f,-0.098600002335011982523838014458306133747100830078125f,-0.238500011578202109063795433030463755130767822265625f,0.138799998451769379670395210268907248973846435546875f,-0.3619000026345255971449432763620279729366302490234375f,-0.16149999874085185780359097407199442386627197265625f,-0.699500026658177009863948114798404276371002197265625f,-0.172100007691979417501215721131302416324615478515625f,-0.5208000193357467111354708322323858737945556640625f,-0.19309999278336753558704685929114930331707000732421875f,-1.324900039157271169898422158439643681049346923828125f,-0.25910001438260088502119060649420134723186492919921875f,-0.67580007610917203209055514889769256114959716796875f,-0.20230000853985552478064846582128666341304779052734375f,0.6482999834239480119180143447010777890682220458984375f,0.1906000060826540387637351159355603158473968505859375f,1.02460003682374978239977281191386282444000244140625f,-0.0197000042602419878079444970353506505489349365234375f,-0.536699991616606641997577753500081598758697509765625f,-0.2409999944046139486175661659217439591884613037109375f,-1.110699993672966012780989331076852977275848388671875f,0.143400003387033958546226131147705018520355224609375f,0.6949000061511991788165687466971576213836669921875f,0.1218999957427382430097395626944489777088165283203125f,0.347300001126527480010963699896819889545440673828125f,-0.0049999996274709967991611847537569701671600341796875f,-0.612800005909800660219843848608434200286865234375f,-0.5192999941825870990896873991005122661590576171875f,0.12459997870922112639391343691386282444000244140625f,-1.28979997758269337992942382697947323322296142578125f,1.329199968370795570393738671555183827877044677734375f,-1.1763999907732003435967271798290312290191650390625f,0.526600008198618407817548359162174165248870849609375f,-1.4624999813735488629617975675500929355621337890625f,1.2339999506473546819762532322783954441547393798828125f,-1.40750002890825243184735882095992565155029296875f,0.189200022312998594742339264485053718090057373046875f,-1.04169999824762271600775420665740966796875f,0.6985999764442443193956933100707828998565673828125f,-1.63440003541708023249157122336328029632568359375f,0.997000020101666439842347244848497211933135986328125f,-1.1384000668287281854418324655853211879730224609375f,0.091000025063753042786629521287977695465087890625f,0.61659997340440764190816480549983680248260498046875f,-1.4745999872028827670789041803800500929355621337890625f,0.6140000016093249968207601341418921947479248046875f,-0.71740000078678090034145498066209256649017333984375f,-0.74029996530413644251211735536344349384307861328125f,1.469199946466088757546231136075221002101898193359375f,-1.0519999866485587602937812334857881069183349609375f,0.748000006720423105122108609066344797611236572265625f,-0.30780001999139816604156294488348066806793212890625f,-0.69839999068379388091898363200016319751739501953125f,-0.909299991723895306705571783822961151599884033203125f,1.013799975913763251611499072168953716754913330078125f,-0.983500011205673985159592120908200740814208984375f,0.13879998261928516711805059458129107952117919921875f,0.121099996151030009361448946947348304092884063720703125f,0.8069000115007156637148000299930572509765625f,0.31359999980181452183813917145016603171825408935546875f,0.23529999771416154619174676554393954575061798095703125f,-0.38140000736936929826015330036170780658721923828125f,-1.04190001572892043668616679497063159942626953125f,0.2388000133529306623358934302814304828643798828125f,0.9643000437289488413483695694594644010066986083984375f,0.1455000103712082992757359534152783453464508056640625f,-0.127999990627169157875187011086381971836090087890625f,-0.32110000199973587342583414283581078052520751953125f,-0.8534000059574839980314209242351353168487548828125f,0.21220000060945753972418970079161226749420166015625f,1.458699983368813235529160010628402233123779296875f,0.28619999430254094985315305166295729577541351318359375f,0.601500011481344021291306489729322493076324462890625f,-0.05159999955594540832493066773167811334133148193359375f,-1.262200026202201730285423764144070446491241455078125f,-0.31420000085532662215115351500571705400943756103515625f,-0.038399983680247995465606436482630670070648193359375f,0.3530000067204237534923549901577644050121307373046875f,1.775400061076880309229863996733911335468292236328125f,0.3668000107973814483131036467966623604297637939453125f,0.943200052085519491384957291302271187305450439453125f,0.07919999977499248000611942188697867095470428466796875f,1.0536999920487397641721827312721870839595794677734375f,0.267799993839859862276853164075873792171478271484375f,-0.114799990770221516100946246297098696231842041015625f,0.64289998433589889970107833505608141422271728515625f,-0.58739998469352716625735411071218550205230712890625f,0.28690000725388475899535478674806654453277587890625f,-0.311400003513693679479956699651665985584259033203125f,-0.345100021695345926531217628507874906063079833984375f,0.86120000821650022349018627210170961916446685791015625f,-0.00199995366111271888343026148504577577114105224609375f,0.347999983027576842431471959571354091167449951171875f,1.21130001391172559266351527185179293155670166015625f,-1.3523999774664645645572136345435865223407745361328125f,0.451999975025653810689618694595992565155029296875f,0.230000011101365142707209088257513940334320068359375f,-0.53509998222589416627670289017260074615478515625f,0.5762000100046389405861191335134208202362060546875f,0.8432999864041799131797461086534895002841949462890625f,-1.159199981111287858226432945230044424533843994140625f,0.979700013796984681135882055968977510929107666015625f,0.2111999941349029796811009873636066913604736328125f,-0.4598000321954482938480168741079978644847869873046875f,0.3160000229775901114948055692366324365139007568359375f,-0.383199960070848444360080975457094609737396240234375f,-0.056200037497281751797117976821027696132659912109375f,0.4717000355005271927666399278677999973297119140625f,-0.585200007516145870312129773083142936229705810546875f,-0.1200999920606609094875238952226936817169189453125f,0.248400007894634899940911054727621376514434814453125f,0.59249999731779023903754932689480483531951904296875f,-0.5063999886870382827197545339004136621952056884765625f,-0.681199986684323022245735046453773975372314453125f,1.8784000550568100695869588889763690531253814697265625f,0.296200006651878400276700631366111338138580322265625f,0.605000025480986192150112401577644050121307373046875f,-0.10789999852925535872572027074056677520275115966796875f,-0.76629999297559248105926599237136542797088623046875f,-0.0286000009194016246993896857020445168018341064453125f,-0.716099998944997651051380671560764312744140625f,-0.1460999968960881023605224982020445168018341064453125f,-1.1657000033587212595165283346432261168956756591796875f,-0.12710000091195106097785583187942393124103546142578125f,-0.16259998736679559527829042053781449794769287109375f,0.1955000042617320854532181328977458178997039794921875f,-0.0189999765753743332652447861619293689727783203125f,-0.0049999996274709690435855691248434595763683319091796875f,-0.2046000085115433630988945878925733268260955810546875f,0.0242999946668744082334256972899311222136020660400390625f,0.8782999735891816595056980077060870826244354248046875f,0.08590000096559524678241359652020037174224853515625f,0.1828000303477050891842736746184527873992919921875f,-0.1714999960213898777539043294382281601428985595703125f,0.52260000152289920460901839760481379926204681396484375f,0.1313999999932944084957142649727757088840007781982421875f,0.690500037621707196677789397654123604297637939453125f,0.2199000125214458645217519006109796464443206787109375f,0.521300038945675670021273617749102413654327392578125f,-0.0259000021204351948966859708889387547969818115234375f,0.10909998718798175065813893525046296417713165283203125f,-0.163399998171627480747503113889251835644245147705078125f,-1.030600030073523409868130329414270818233489990234375f,0.7668000171303746004269896729965694248676300048828125f,-0.55520001451969136496700230054557323455810546875f,0.4546999995291238771955022457404993474483489990234375f,-0.280199986654520227791209663337212987244129180908203125f,-0.941899963593483047219478976330719888210296630859375f,1.19419996449649357117550607654266059398651123046875f,-1.2704999671578409703442957834340631961822509765625f,1.1357999500453477281780578778125345706939697265625f,-1.1743999877333639592080771762994118034839630126953125f,0.913399987182021089182626383262686431407928466796875f,-0.230099958235026047503879453870467841625213623046875f,0.44199995513260414981004942092113196849822998046875f,0.04180001448541881270415387916727922856807708740234375f,0.1125999987289309223204014642760739661753177642822265625f,-0.64820000296756619118099251863895915448665618896484375f,0.175400007432699123199881796608678996562957763671875f,-0.020400036564468759792134733288548886775970458984375f,-0.729399967542291083333338974625803530216217041015625f,-0.3076999969363214404438622295856475830078125f,-0.0250000325590367111772138741798698902130126953125f,0.702200007333606546211512977606616914272308349609375f,-0.53799998809397220345118739714962430298328399658203125f,1.347600007278473288607756330748088657855987548828125f,-1.153199984061718641470406510052271187305450439453125f,-0.05759998759031237369754308019764721393585205078125f,0.017600004279613390423264718265272676944732666015625f,-0.55310000600814779314617908312357030808925628662109375f,0.08939999190568936882073103333823382854461669921875f,-0.8073000301092865260699227292207069694995880126953125f,-0.0938000084728003447054334174026735126972198486328125f,0.07949999774992466683443126385100185871124267578125f,0.85039998716712006388007694113184697926044464111328125f,-0.006999997824430448378763003347557969391345977783203125f,-0.1124999955296510645297303199185989797115325927734375f,-0.2327999939143656771278756423271261155605316162109375f,0.2944000524044045885574405474471859633922576904296875f,0.3187000095948577804705337257473729550838470458984375f,1.840500051945448856116627212031744420528411865234375f,0.363300008346140401727097923867404460906982421875f,1.3855000678896909338533305344753898680210113525390625f,0.1190000017583370539586695713296649046242237091064453125f,0.52599999853968615326493818429298698902130126953125f,-0.08920000372380021413931672213948331773281097412109375f,-0.99349999637901731386335768547724001109600067138671875f,-0.2120999922692774486421285473625175654888153076171875f,0.716000034675003060868903048685751855373382568359375f,0.340600011782348222055816222564317286014556884765625f,1.58550006669759824262655456550419330596923828125f,0.1948999971546232501395934377796947956085205078125f,0.35690001688376060684504409437067806720733642578125f,-0.1022999967679381061458343538106419146060943603515625f,-0.986099998348950901316811723518185317516326904296875f,-0.110199995967745767444512239308096468448638916015625f,-0.4412000107496971690324016890372149646282196044921875f,-0.14969999751746654315098794540972448885440826416015625f,-0.6925999813318253206517738362890668213367462158203125f,0.04509999984651802407142895390279591083526611328125f,-0.722100042784214135593856553896330296993255615234375f,-0.48460001201331603937205727561376988887786865234375f,-0.2205999885737897070470125981955789029598236083984375f,-0.0054000425249329797594555202522315084934234619140625f,-0.51299996242672263324635650860727764666080474853515625f,-0.33540004798248312933850456829532049596309661865234375f,-0.1941999784290795449948063833289779722690582275390625f,-0.19799993526935555365753316436894237995147705078125f,0.1187999456644062146182250216952525079250335693359375f,0.937000014662743296867120079696178436279296875f,-1.442999985784292338308887337916530668735504150390625f,0.48220001778304588668788710492663085460662841796875f,-0.1308000044345856149874407492461614310741424560546875f,0.1858999858409162708738904257188551127910614013671875f,-0.436199995550512975572843288318836130201816558837890625f,-1.04719997956156607443745087948627769947052001953125f,1.283999982610344314792882869369350373744964599609375f,0.42150001800060277190596025320701301097869873046875f,-1.751800027337669707350187309202738106250762939453125f,0.580900020504742986560131612350232899188995361328125f,-0.38779998810291294031316056134528480470180511474609375f,-0.44029997955337163428879421189776621758937835693359375f,0.959599971854686817351876015891321003437042236328125f,-0.5164999825358389529128544381819665431976318359375f,0.746999979048967421846327852108515799045562744140625f,0.60169995749294802322992836707271635532379150390625f,-0.7821999625742439565811991997179575264453887939453125f,0.2204000325039021657147486621397547423839569091796875f,0.6693999653816227191072130153770558536052703857421875f,-0.81350006350874881633217228227294981479644775390625f,0.470900046935678329163010857882909476757049560546875f,0.135800004173815269670200223117717541754245758056640625f,-0.09649997799098475326218249392695724964141845703125f,-0.07830000041127205268054467524052597582340240478515625f,-1.475400012648105274593035574071109294891357421875f,-0.1391999942243098420391334002488292753696441650390625f,-0.98480000485181751201935185235925018787384033203125f,-0.3173999969869851778270231079659424722194671630859375f,-0.41980000693798036337511803139932453632354736328125f,0.286399999415874406683002462159493006765842437744140625f,0.50279999972581801426230185825261287391185760498046875f,-0.3036000102698803715384201495908200740814208984375f,-0.389800029513240442469168556272052228450775146484375f,0.1529999991580843765603958672727458178997039794921875f,1.35470003678202655095219597569666802883148193359375f,0.216400008058548032341406042178277857601642608642578125f,-0.267199975463747296799965624813921749591827392578125f,-0.1925999986544251196818322569015435874462127685546875f,-0.802500020116567736039314695517532527446746826171875f,-0.0501000027522444657535061196540482342243194580078125f,0.550300011199713168252856121398508548736572265625f,0.2527000055417418789005523649393580853939056396484375f,1.2093000202596186909431708045303821563720703125f,0.187299998556077429423538660557824186980724334716796875f,0.739700033652782362736388677149079740047454833984375f,0.281100007699430054497469200214254669845104217529296875f,0.466700029018521700407973185065202414989471435546875f,-0.0787000020325183680114378148573450744152069091796875f,-1.4174000396788122291269473862485028803348541259765625f,0.7262000503122809647038593539036810398101806640625f,-0.179800017294287517444217883166857063770294189453125f,0.32320003934502583575749667943455278873443603515625f,0.572399977764487477571719864499755203723907470703125f,-1.011399981534479852740560090751387178897857666015625f,1.212399995124339380225819695624522864818572998046875f,-0.91280001522302622873894506483338773250579833984375f,0.56199998068809531304168558563105762004852294921875f,-0.039600014024973262394269113428890705108642578125f,-0.43959998586177828627796770888380706310272216796875f,-0.108000062078238645568717402056790888309478759765625f,-0.06499997839331606019896980797057040035724639892578125f,-0.39620001201629673204251957940869033336639404296875f,-0.065200000736117313948625451303087174892425537109375f,0.6301000193297865425989812138141132891178131103515625f,-0.7594000118732451287684170893044210970401763916015625f,-0.130799972546099763093252477119676768779754638671875f,1.26199999834597065273555926978588104248046875f,-0.3848000179648405350008033565245568752288818359375f,-0.732000007957219533949455581023357808589935302734375f,-0.39159997683167357962474852683953940868377685546875f,0.4370000028908247458048208500258624553680419921875f,0.445499992638826114443872938863933086395263671875f,-0.3351999875485898172655652160756289958953857421875f,0.73180003292560602545790970907546579837799072265625f,-0.454600013428926530423268559388816356658935546875f,0.32000004544854210308812980656512081623077392578125f,0.6539999911040066393752567819319665431976318359375f,-0.65179998524188942354840037296526134014129638671875f,-1.503400035610794827078962043742649257183074951171875f,-0.13020000233799222666419836968998424708843231201171875f,0.589100023174286224758589014527387917041778564453125f,0.0984000065535307888442417834085063077509403228759765625f,0.614000028058887092896611648029647767543792724609375f,0.32050000757724050881591892903088591992855072021484375f,1.655900021380185993535860689007677137851715087890625f,0.2495999909281729767229762728675268590450286865234375f,1.249400018802284240138078530435450375080108642578125f,0.303600001515448081335080132703296840190887451171875f,0.04810001956820519897206622772500850260257720947265625f,-0.328299998045712637662063571042381227016448974609375f,-0.8357000186510379347026855612057261168956756591796875f,-0.213299999181926214220084148109890520572662353515625f,-0.750100016349553921912729492760263383388519287109375f,-0.121500003062188610680749434322933666408061981201171875f,-1.040700069817901418645078592817299067974090576171875f,-0.3327000139236451214941325815743766725063323974609375f,-0.113400051480532670922229954157955944538116455078125f,0.09250000122934576296529485261999070644378662109375f,0.6083000287979842823915532790124416351318359375f,0.137600004763901273552306747660622932016849517822265625f,-0.4304000156283380729149712351500056684017181396484375f,-0.300300010566413477786085195475607179105281829833984375f,-0.56020002632886234383846613127388991415500640869140625f,0.07700000352412461612772176522412337362766265869140625f,0.5375000216811895992208292227587662637233734130859375f,0.391299998936057047416170462383888661861419677734375f,1.256200022968649410159969193045981228351593017578125f,-1.29110002517104138775039245956577360630035400390625f,0.68120000702440730577791327959857881069183349609375f,0.3073000175923112919917912222445011138916015625f,-0.536400012230873191043656333931721746921539306640625f,1.040100050845742973848473411635495722293853759765625f,-0.7136000074386597002984444770845584571361541748046875f,0.9350999822258945215480707702226936817169189453125f,-1.176199960905313535164395943866111338138580322265625f,0.0664000286221497759697740548290312290191650390625f,0.287199970248341873713116001454181969165802001953125f,-0.915000001285224850988697653519921004772186279296875f,0.83419998498559000399410479076323099434375762939453125f,0.06169998872950699375650174260954372584819793701171875f,-0.5019999963343142468374935560859739780426025390625f,0.5380000212490561040112879709340631961822509765625f,-0.038199998515844324487034100457094609737396240234375f,-0.01960004040002871761316782794892787933349609375f,0.0828000327318907380913515225984156131744384765625f,1.0910999639272684902380206040106713771820068359375f,-1.186199979382753344481216117856092751026153564453125f,1.31580000383853956691382336430251598358154296875f,-0.83799998503923445181129636694095097482204437255859375f,-0.4332000313356523246710594321484677493572235107421875f,0.8722000168889760818302647749078460037708282470703125f,-0.1566000053301459526977623681887052953243255615234375f,0.03379998150169871706793855992145836353302001953125f,0.44280002386569972117058568983338773250579833984375f,-0.1825999897509815728113835575641132891178131103515625f,0.915300013881921614711245638318359851837158203125f,0.5791000386714941772225984095712192356586456298828125f,0.11440000125616790749294438001015805639326572418212890625f,0.53360000777393568061057749218889512121677398681640625f,0.047899999237060508772145794864627532660961151123046875f,0.110000023469328578329395895707421004772186279296875f,0.1846000018432736311524422490037977695465087890625f,-0.7051000179886821594976709093316458165645599365234375f,-0.2675000081583858158040811758837662637233734130859375f,-0.165700001943111185909174309927038848400115966796875f,0.046800010722875651225649562547914683818817138671875f,-0.869600000613927281989390394301153719425201416015625f,-0.281800004613399490693836924037896096706390380859375f,-1.315000051632523980771338756312616169452667236328125f,-0.12170000651776791134039967801072634756565093994140625f,0.164199973735212889636159161454997956752777099609375f,0.1995000014007091149181860600947402417659759521484375f,0.215200018915533863861355712288059294223785400390625f,-0.0806000041082501894607048598118126392364501953125f,0.00589998800158486602640550700016319751739501953125f,-0.0561000012174248663399112047045491635799407958984375f,0.1456999864459038729336270989733748137950897216796875f,0.01420000227093698441649394226260483264923095703125f,0.6118000273376704711836282513104379177093505859375f,0.27270000498294832969037315706373192369937896728515625f,0.52200002442300341698455667938105762004852294921875f,-0.0997000004231929637388276432830025441944599151611328125f,-0.6440000034719706167862796064582653343677520751953125f,-0.0090000000447034711470450929482467472553253173828125f,-0.97400002731382862020126367497141472995281219482421875f,0.23760003285259045924249221570789813995361328125f,-0.2354000025153160091662130071199499070644378662109375f,0.4477000117257234901302354046492837369441986083984375f,0.026199995848536605791423426126129925251007080078125f,0.91060005890726980481986174709163606166839599609375f,0.1529999654442075751603624667041003704071044921875f,-0.17330002112388687152133570634759962558746337890625f,-0.0543999939918513586434301032568328082561492919921875f,0.15170001106262276380221010185778141021728515625f,0.352799992275237617178618165780790150165557861328125f,0.1911999792337422121590861934237182140350341796875f,-0.023199970501661670851945018512196838855743408203125f,-1.0206000083923338905833588796667754650115966796875f,0.708800001618266062308748587383888661861419677734375f,-0.9920000149607659523098845966160297393798828125f,0.230999996230006221509256647550500929355621337890625f,-1.19240002149939616771234796033240854740142822265625f,0.884199987742305015814281432540155947208404541015625f,-0.3907000202715398273767277714796364307403564453125f,-0.1525999941468239029518372262828052043914794921875f,0.577599981704354892286801259615458548069000244140625f,-0.6397999726653103014228918254957534372806549072265625f,0.70150002470612538019167914171703159809112548828125f,-0.4004000092953443612486807978712022304534912109375f,0.37360001421868815896232263185083866119384765625f,0.07559999773800374356369502493180334568023681640625f,0.17720005168318753163703149766661226749420166015625f,0.74499997392296801734801192651502788066864013671875f,-0.257399992162734125855649836012162268161773681640625f,0.31369998877048477226026079733856022357940673828125f,0.35440000442266461622153883581631816923618316650390625f,1.3722000681489709972282753369654528796672821044921875f,0.1782000057801604864238242953433655202388763427734375f,0.961900050020218611024347410420887172222137451171875f,0.19620001410245901940498924886924214661121368408203125f,0.947800060299039959232914043241180479526519775390625f,0.217700005131959972715094409068115055561065673828125f,0.5208000064462421363487010239623486995697021484375f,-0.0194000045344233651434251441969536244869232177734375f,-0.12449999998509941878666040793177671730518341064453125f,0.2416000022009014980284291596035473048686981201171875f,-0.1930999884247779352364204896730370819568634033203125f,-0.139500003375113035275489892228506505489349365234375f,-1.0041000356167557594488926042686216533184051513671875f,-0.153800003667175777533060454516089521348476409912109375f,-0.642300017592310812375444584176875650882720947265625f,-0.187399999855458687658682492838124744594097137451171875f,-0.1891000086456535722589933357085101306438446044921875f,0.023499999657273284991987338798935525119304656982421875f,0.491099997529387710670789601863361895084381103515625f,0.072700003753602526757049417938105762004852294921875f,-0.617499977201223071432423239457421004772186279296875f,-0.3142999999567865909710917549091391265392303466796875f,-1.707500048652291102513345322222448885440826416015625f,-0.406600005181133761222866951356991194188594818115234375f,-1.474700026127696350641826938954181969165802001953125f,-0.0015999972835183395858393851085565984249114990234375f,0.01250003136694477490209465031512081623077392578125f,1.055199980843067208979846327565610408782958984375f,-1.374399974322319195607633446343243122100830078125f,0.39510002424716983426833394332788884639739990234375f,0.1757999897569415548304050389560870826244354248046875f,0.8462000436067587116895083454437553882598876953125f,-0.257800026547908789353868996840901672840118408203125f,2.087100035333634195922059006989002227783203125f,-1.450199978682399137852598869358189404010772705078125f,1.814600031459331574978932621888816356658935546875f,-0.5971999893218278288031797274015843868255615234375f,0.620200057733058418563132363487966358661651611328125f,0.07719996249973826163426338098361156880855560302734375f,-0.63099997931718831978287198580801486968994140625f,1.1577999889969825542124226558371447026729583740234375f,-1.04130002657771125740282514016143977642059326171875f,0.3140000043660402528900021934532560408115386962890625f,-1.16080001367330520878340394119732081890106201171875f,0.49119999257028101879996029310859739780426025390625f,-0.58479999091923240417401075319503434002399444580078125f,0.25719998634159579875557710693101398646831512451171875f,0.430999971605837917110193302505649626255035400390625f,-0.991399969166517625041024075471796095371246337890625f,-0.84989998573660852088096362422220408916473388671875f,1.461799967807531519525809926562942564487457275390625f,-1.0573000181138514363965441589243710041046142578125f,0.766200002703070648379934937111102044582366943359375f,-0.18949997219443392992843655520118772983551025390625f,0.64239995958507112305824193754233419895172119140625f,0.23860000460743879813207968254573643207550048828125f,-0.84370000285208224255484310560859739780426025390625f,0.1470999975591897879212410771287977695465087890625f,0.757200005787610930241271489649079740047454833984375f,0.00580000227391719302971750948927365243434906005859375f,-1.376800011244416310773885925300419330596923828125f,-0.32200000523775818006555482497788034379482269287109375f,-1.5133000326722860773287493429961614310741424560546875f,-0.191499998293816975358794252315419726073741912841796875f,-0.33680000502318085953135096133337356150150299072265625f,0.1787000037088990467282201279886066913604736328125f,0.869500012725592075213398857158608734607696533203125f,-0.0410999936923384101561396164470352232456207275390625f,-0.6172999945700166080087001319043338298797607421875f,-0.0632999999269843183213879456161521375179290771484375f,0.6325000321120028967669668418238870799541473388671875f,0.1752000053927302847256441964418627321720123291015625f,-0.08069997355639912939295754767954349517822265625f,-0.04099999999254944071935824467800557613372802734375f,0.2651000077813865374309898470528423786163330078125f,0.04889999867081640250177088091732002794742584228515625f,-0.36300002393126507005405301242717541754245758056640625f,-0.17240000309646130727259105697157792747020721435546875f,0.76619998981356562950395527877844870090484619140625f,0.251899998722970419606781433685682713985443115234375f,0.4853000137329097629645957567845471203327178955078125f,-0.0825000053271651712094580943812616169452667236328125f,0.4257000152796506231567263967008329927921295166015625f,0.2882000065997243343218769950908608734607696533203125f,0.817300054919720242452285674517042934894561767578125f,-0.3292999669134619722399293095804750919342041015625f,0.405199976447224941011882037855684757232666015625f,0.127600012847781218994214214035309851169586181640625f,0.074599991673231169642122040386311709880828857421875f,-0.6413999970316890397725728689692914485931396484375f,1.14059998775422588579431248945184051990509033203125f,-1.244800001983344284184340722276829183101654052734375f,0.63399998380243782758469706095638684928417205810546875f,-0.9108999869957556949628951770137064158916473388671875f,0.5536000034153456539343096665106713771820068359375f,0.68819997884631245455011594458483159542083740234375f,-0.707399974411726528700228300294838845729827880859375f,-0.0947000193476679186233013751916587352752685546875f,-0.116800000590085772955717402510344982147216796875f,-0.40699998754262889377741885255090892314910888671875f,0.0743999885529278426332666640519164502620697021484375f,-1.2135999838948254847537100431509315967559814453125f,1.0933999625951056344774769968353211879730224609375f,-0.3301999996185305263907139305956661701202392578125f,-0.027600001671910234790630056522786617279052734375f,0.035599972778558974795259928214363753795623779296875f,-0.01039997405409842645696016916190274059772491455078125f,1.42089998248815430059721620636992156505584716796875f,-1.6777999930202955880531590082682669162750244140625f,0.68409998800754490133613217039965093135833740234375f,-0.1925999875903128266685371272615157067775726318359375f,0.20270004014968900918347571860067546367645263671875f,0.2079999798983334091673214061302132904529571533203125f,1.001100054526329330428779940120875835418701171875f,0.9005000348836185164458356666727922856807708740234375f,0.33210000795274974283444180400692857801914215087890625f,0.534500065699220261450363977928645908832550048828125f,0.0652000013694167268596402209368534386157989501953125f,0.3849000182956456850291715454659424722194671630859375f,0.076199999089539005581173114478588104248046875f,0.00030002185404309766880714960279874503612518310546875f,0.09440000296980144423031333644757978618144989013671875f,0.212700012657046577668751297096605412662029266357421875f,-0.044899999967217441021460899719386361539363861083984375f,-0.02980000224411483156217173018376342952251434326171875f,0.0583999978736043090510321462716092355549335479736328125f,0.9124000039905306724818956354283727705478668212890625f,0.22870000043064353523192266948171891272068023681640625f,-0.0770999854892491232050133476150222122669219970703125f,-0.094799998109042604710339219309389591217041015625f,-0.40890001886188986901515818317420780658721923828125f,-0.057000003673136234727536475475062616169452667236328125f,-0.2343000309139495751509230103692971169948577880859375f,-0.02810000205934048977951533743180334568023681640625f,0.215200007069110910151721327565610408782958984375f,0.2690000002682209068183283307007513940334320068359375f,0.9190000037699934676282964574056677520275115966796875f,-0.01069999862760308284581611815156065858900547027587890625f,-1.355199984084069608769596015918068587779998779296875f,-0.1706999985530972541170058320858515799045562744140625f,-1.22329999199211503224660191335715353488922119140625f,-0.29169999660849565970011099125258624553680419921875f,-1.410000047497451447497951448895037174224853515625f,0.25410001972839213468802199713536538183689117431640625f,-0.48920002268552753577068870072253048419952392578125f,-0.824199962484836845533209270797669887542724609375f,1.012199963393807689016057338449172675609588623046875f,-0.485499975651503490325922030024230480194091796875f,0.3179999649971729791531060982379131019115447998046875f,-0.025399970477819611147651812643744051456451416015625f,0.08879997971355935160175931741832755506038665771484375f,0.0341000030577183199653745759860612452030181884765625f,-0.108400000593066148102394663510494865477085113525390625f,-0.189999996386468250619827813352458178997039794921875f,-0.09859999495893723153727705721394158899784088134765625f,0.36620001874044516565476214964292012155055999755859375f,-0.3138000082492826692259768606163561344146728515625f,-0.32029997916221564224770190776325762271881103515625f,0.9043999934703108589673092865268699824810028076171875f,0.1190000148713590277793628047220408916473388671875f,0.09980000269114963629135672817938029766082763671875f,0.8991999978005882354636923992075026035308837890625f,-1.0261999961465593234066773220547474920749664306640625f,0.00150003245472962731810184777714312076568603515625f,0.406600003802775944450331735424697399139404296875f,0.38129998861700276080455296323634684085845947265625f,-0.1451999977558851018244467923068441450595855712890625f,-1.1562999886170037822097356183803640305995941162109375f,1.373199954110384535255207083537243306636810302734375f,-1.938499998986720651572568385745398700237274169921875f,1.514599975356459626851801658631302416324615478515625f,-1.10990005130171898173330191639252007007598876953125f,-1.2845000146627423287526426065596751868724822998046875f,-0.16900000376999380069520384495262987911701202392578125f,-0.8778000140309332177679380038171075284481048583984375f,-0.15519999920874827381567229167558252811431884765625f,-0.0674000008240340786613842283259145915508270263671875f,0.05359999931752679191987454032641835510730743408203125f,-0.191400002470612395910620762151665985584259033203125f,0.0713000011712312531386714908876456320285797119140625f,0.09560001822710051389009322519996203482151031494140625f,-0.13739999855160711650370330971782095730304718017578125f,-0.885400015851855126669534001848660409450531005859375f,-0.14540000463873148373039612124557606875896453857421875f,0.979499988734721771521662958548404276371002197265625f,0.349500000618398143359399909968487918376922607421875f,2.06230007525980507665508412173949182033538818359375f,0.37650001002848154652014045495889149606227874755859375f,0.50180002938658052613618565374054014682769775390625f,-0.0219000012002885380280048366330447606742382049560546875f,-0.04180001716762815622274729321361519396305084228515625f,-0.02149999985843897842840988232637755572795867919921875f,-0.628900008058547843603491855901665985584259033203125f,-0.1694000012189149639851848405669443309307098388671875f,-0.8296000266164547110037119637127034366130828857421875f,0.1427999977514147200707839147071354091167449951171875f,0.79730000999271821715552732712239958345890045166015625f,0.1799999999627470703700282683712430298328399658203125f,-0.086699972580372930419656540834694169461727142333984375f,-0.1863999960817395995515965978484018705785274505615234375f,0.3945000321343548055352812298224307596683502197265625f,1.062600001634657065352485005860216915607452392578125f,-0.28909998130202296806601225398480892181396484375f,-1.96649999132752473229857059777714312076568603515625f,0.482299958658218397289374479441903531551361083984375f,0.07350000409036905324455801746807992458343505859375f,-0.485599975013732976236724425689317286014556884765625f,-0.6101000128477818140737554131192155182361602783203125f,-0.536900005912780642347570392303168773651123046875f,-0.5027999830737710507122528724721632897853851318359375f,0.15260000070333479271766918827779591083526611328125f,-0.7875999849081039005938009722740389406681060791015625f,0.7007999796986581486635259352624416351318359375f,-0.13879999409318022340897869071341119706630706787109375f,0.7704999938309191609420167878852225840091705322265625f,-0.1095999986395244096826218083151616156101226806640625f,1.17559998618960381833176143118180334568023681640625f,-0.0369999755844478972477418210473842918872833251953125f,-0.085500001877546250028672147891484200954437255859375f,1.12450000862777255150604105438105762004852294921875f,0.1164000138700007624947829754091799259185791015625f,0.9327999547615652975451894235447980463504791259765625f,0.75890000284314140799324377439916133880615234375f,-0.00830003952682012435815295248175971210002899169921875f,-0.4223000202000140035352160339243710041046142578125f,-1.354299975779652331908664564252831041812896728515625f,0.35800001722574226192818969138897955417633056640625f,0.81720002806484703938849634141661226749420166015625f,0.2338000021398070060740792541764676570892333984375f,1.3794000233098859808933411841280758380889892578125f,-1.10019997138082903376243848470039665699005126953125f,-0.03359998359680194113252582610584795475006103515625f,0.695600031191110357298157396144233644008636474609375f,1.282699957895278686237361398525536060333251953125f,0.984999992102384425152195035479962825775146484375f,-0.3744999872446062028075175476260483264923095703125f,-0.12140002698302243544503653538413345813751220703125f,-0.818699986833333781532928696833550930023193359375f,0.2612999988615509749934062710963189601898193359375f,-1.18219997524023057167141814716160297393798828125f,-0.91619998281002068551970296539366245269775390625f,-0.782099953675270853636902756989002227783203125f,1.45890003994703221223971922881901264190673828125f,1.83039996109008740177159779705107212066650390625f,-0.521900027072430106045430875383317470550537109375f,-0.47469999461173983945627696812152862548828125f,-0.5152000137932599699297497863881289958953857421875f,-1.020100001280754842269971049972809851169586181640625f,1.2258000278852885234215364107512868940830230712890625f,-0.05029999763965609105298426584340631961822509765625f,0.048699989143014210668525265646167099475860595703125f,-0.02719998254179945007535934564657509326934814453125f,-0.0938000477373603924746703341952525079250335693359375f,0.58319995694160553512119804508984088897705078125f,0.69220003950148889515503469738177955150604248046875f,-0.786399990847706842345132827176712453365325927734375f,0.81470003417432312886603540391661226749420166015625f,1.1412000075459474146555294282734394073486328125f,-0.49419999794960034478208399377763271331787109375f,-0.2625999954134223823842830825014971196651458740234375f,0.433899991741776513975992202176712453365325927734375f,0.66299999636411666159574451739899814128875732421875f,0.6798999680042268067836630507372319698333740234375f,-0.10400000380724649939878645454882644116878509521484375f,-0.61139998019337671308903736644424498081207275390625f,-1.330399990370869200972947510308586061000823974609375f,-0.4277999925732611785633707768283784389495849609375f,-0.6093999741733073793881203528144396841526031494140625f,-0.5255999924480914131663666921667754650115966796875f,-0.352900004266202049763023751438595354557037353515625f,0.18409999203085902763632475398480892181396484375f,0.274099983611703290620198458782397210597991943359375f,0.9531999663293362345939385704696178436279296875f,0.2552000054672360906948824776918627321720123291015625f,-0.21439999399185172279658218030817806720733642578125f,0.1630999624408777659567704176879487931728363037109375f,-0.3272000047445298154258352951728738844394683837890625f,0.2050999879069628217820309146190993487834930419921875f,-0.29959998891651640207101081614382565021514892578125f,0.11140001722276238194098141320864669978618621826171875f,-0.32519997793734078950222965431748889386653900146484375f,-1.0430999999359256147357655208907090127468109130859375f,0.3131999918103218760734307579696178436279296875f,-1.05879995479136734815028830780647695064544677734375f,-0.432199997480213593537001770528149791061878204345703125f,-0.4516999888971444665486387748387642204761505126953125f,0.4278999999821184729853484896011650562286376953125f,-0.8847000030308957985170081883552484214305877685546875f,-0.58599999174103079013775641215033829212188720703125f,0.604899995180219196555526650627143681049346923828125f,-0.8209000263534484975735949774389155209064483642578125f,0.618900050377845900584361515939235687255859375f,1.07280004681348817285879704286344349384307861328125f,0.87399997866153711356673738919198513031005859375f,0.13420000718832003627767335274256765842437744140625f,-0.42670001761913312066099024377763271331787109375f,-0.97950000482797605627638404257595539093017578125f,0.512899969434737812434832449071109294891357421875f,-1.02690002915859235343987165833823382854461669921875f,-0.4915999337673184754748945124447345733642578125f,0.9159000191450115124780495534650981426239013671875f,-1.28470001934766742124338634312152862548828125f,0.4873000340580944111934513784945011138916015625f,0.69989999572038641417748294770717620849609375f,-1.2643999737262721083652650122530758380889892578125f,0.229100024366378107743003056384623050689697265625f,-0.01049999759346231797962900600396096706390380859375f,1.085000002458691437112747735227458178997039794921875f,-0.23490003323406039026366443067672662436962127685546875f,0.389099982196092764041850387002341449260711669921875f,-0.4249999784678222258804680677712894976139068603515625f,1.4762999799370764009154299856163561344146728515625f,-0.085199999171495566230305485078133642673492431640625f,0.686399986153841012992415926419198513031005859375f,-0.2431999935984610061723287799395620822906494140625f,0.2032999832749364799155955552123486995697021484375f,-0.121500018894673100788850206299684941768646240234375f,-0.61479997566416866305871735676191747188568115234375f,0.1446999996781348496455166241503320634365081787109375f,-1.06800000476464607146454000030644237995147705078125f,0.723700022894143923934962003841064870357513427734375f,0.03850001716613771218788997430237941443920135498046875f,0.395699987190961888217088926467113196849822998046875f,0.85160000298321236744669704421539790928363800048828125f,-0.6036999997973442333432103623636066913604736328125f,-0.5796000189051031536990876702475361526012420654296875f,0.451299968612194124517600357648916542530059814453125f,-0.60840000867694588038148140185512602329254150390625f,0.44700002166628838295991954510100185871124267578125f,0.6002999707803124973537478581420145928859710693359375f,-0.479800011855363806034802109934389591217041015625f,0.6262999945029614767832981669926084578037261962890625f,0.27510000129938116941730186226777732372283935546875f,0.27369997703582082948514653253369033336639404296875f,0.41479999665617928172878237091936171054840087890625f,1.2965999922543769429239546298049390316009521484375f,0.51819999751001599719302248558960855007171630859375f,-0.1138999925240875565890519283129833638668060302734375f,0.2207999897941947953228236656286753714084625244140625f,0.005100017243623522489315291750244796276092529296875f,0.9427999797582626850811493568471632897853851318359375f,0.4485999957785009595312430974445305764675140380859375f,0.3692999923944473295023271930404007434844970703125f,-0.6003999835535882567683074739761650562286376953125f,0.3306999733328821466926683569909073412418365478515625f,-0.2216000110298395053831654877285473048686981201171875f,-0.140900001934170404638280160725116729736328125f,0.21790000698566469594652517116628587245941162109375f,0.175199989932775412881937882048077881336212158203125f,1.80909998025894225293086492456495761871337890625f,-1.020900013184547816535996389575302600860595703125f,-0.1105999765038490068036480806767940521240234375f,1.0232000353217127752714077359996736049652099609375f,-0.108100026869774268334367661736905574798583984375f,-1.121100057134032823569214087910950183868408203125f,0.81259999682903316653437286731787025928497314453125f,1.478800037306547299209569246158935129642486572265625f,0.024799980413913402799153118394315242767333984375f,-1.32780000777244566734225372783839702606201171875f,0.311100019955635076485123136080801486968994140625f,1.445999988555907833642777404747903347015380859375f,1.17529995889663751995612983591854572296142578125f,-1.0376000173553823202610146836377680301666259765625f,-1.10899996677786116805464189383201301097869873046875f,0.445899998469650338250858112587593495845794677734375f,0.5940000165104866169940578402020037174224853515625f,-0.617000015258789336058953267638571560382843017578125f,0.9471000055313112397925578989088535308837890625f,0.53240000674724630158607396879233419895172119140625f,-0.384699990886449683102910057641565799713134765625f,-0.1021999961391089062345827187527902424335479736328125f,-0.17599999790638687358068636967800557613372802734375f,0.520299983595311488215884310193359851837158203125f,-1.4522999842882153842538173194043338298797607421875f,0.9312999853014947149176805396564304828643798828125f,0.60919998982846734758567208700696937739849090576171875f,0.33410000357925895286825834773480892181396484375f,0.44400004661083192214476866865879856050014495849609375f,0.978999993711710203569964505732059478759765625f,0.1023000196412200413220716654905118048191070556640625f,0.24740000719428056896731504821218550205230712890625f,1.369500006429851257649943363503552973270416259765625f,-0.505700012820958999526510524447076022624969482421875f,-0.6002000024497509134135952990618534386157989501953125f,0.587799997714161914785790941095910966396331787109375f,-0.59519998769760140699958128607249818742275238037109375f,-0.28940000799894338712192620732821524143218994140625f,-0.58489999624192690053092746893526054918766021728515625f,0.53609999998807911225640054908581078052520751953125f,0.64680001806169717948336028712219558656215667724609375f,0.24219999520778667800868788617663085460662841796875f,-1.120299962435662877169306739233434200286865234375f,-0.320099992358684470872276506270281970500946044921875f,0.6207000102877617830898770989733748137950897216796875f,0.2931000098824501076677506716805510222911834716796875f,0.134799994122981825483975626411847770214080810546875f,0.65770000617504109641231480054557323455810546875f,0.185899998246133346668784724897705018520355224609375f,-0.35019998620748526985124726706999354064464569091796875f,-0.5282999942272901261475226419861428439617156982421875f,-0.0419000017404556057698528093169443309307098388671875f,0.111399998559057689107021360541693866252899169921875f,-0.941599980705976502548537609982304275035858154296875f,0.68860004349946990487296716310083866119384765625f,-0.778900022587180274058482609689235687255859375f,-0.4759999819248914576519382535479962825775146484375f,-0.25930003270208867860446844133548438549041748046875f,0.9923999918460850722112809307873249053955078125f,0.30790000869929823323900563991628587245941162109375f,1.36690001366138513816395061439834535121917724609375f,0.132700031805038864973766976618207991123199462890625f,-0.1391999881148340278969044447876513004302978515625f,1.675300036978720985558766187750734388828277587890625f,0.014199968147277530761130037717521190643310546875f,0.70239994207620615185305723571218550205230712890625f,-0.5661000271081917389892623759806156158447265625f,-0.267499964833259440410984097979962825775146484375f,0.744599986755847709218869567848742008209228515625f,0.52430000682994748473220170126296579837799072265625f,1.025899977142363805882041560835205018520355224609375f,-1.1183000475175679522266136700636707246303558349609375f,-1.642999983400105890041231759823858737945556640625f,0.6464000117838377690304696443490684032440185546875f,0.140999999880790483075543306767940521240234375f,0.21539998970031692948623458505608141422271728515625f,2.14450000333786050532580702565610408782958984375f,0.222200017820299411397400035639293491840362548828125f,-1.104399984753131835901740487315692007541656494140625f,-0.1880000099986791628481341831502504646778106689453125f,1.233899974083900730192908667959272861480712890625f,0.36639998466372514229760781745426356792449951171875f,-1.0007999674797058986541742342524230480194091796875f,-0.150099983716011298184866973315365612506866455078125f,1.3741999736607077853278724433039315044879913330078125f,-0.1340999932229518520898636779747903347015380859375f,0.01379998194873299777185593484318815171718597412109375f,0.75829999862313268721436543273739516735076904296875f,-0.5129999725967648860347480876953341066837310791015625f,0.92000001393258568072042180574499070644378662109375f,-1.07909996453821666051453576073981821537017822265625f,0.2823999878227712034828300602384842932224273681640625f,0.6264999993741511286060585916857235133647918701171875f,-0.448900009101629304808511733426712453365325927734375f,-1.087199995505809813067799041164107620716094970703125f,-0.7366999922871588779571538907475769519805908203125f,-0.841799967397749870912093683728016912937164306640625f,0.318399993067979725225313814007677137851715087890625f,-0.167699995762854658920559813850559294223785400390625f,0.2757999951958656215111886922386474907398223876953125f,-0.63389997481033211190748488661483861505985260009765625f,-0.5811999954760072029102957458235323429107666015625f,0.114100016804039139373116995557211339473724365234375f,0.151399991071224260252847670926712453365325927734375f,-0.28700000005960479843025723312166519463062286376953125f,1.19690001261830314405187891679815948009490966796875f,0.1362000114202499556625980403623543679714202880859375f,-0.7456999842479825613139610140933655202388763427734375f,0.403700007285177520088836899958550930023193359375f,0.516099993431568204727000193088315427303314208984375f,-1.016599978284537630912609529332257807254791259765625f,-0.6002999818444252344562528378446586430072784423828125f,0.265600030073523729612361421459354460239410400390625f,1.6282000473544002705494904148508794605731964111328125f,0.896099991866945888574491618783213198184967041015625f,0.2845000040642917671362965847947634756565093994140625f,0.541299987629801027111398070701397955417633056640625f,-0.5443999834120274439186459858319722115993499755859375f,0.3800999713107942046264042801340110599994659423828125f,1.408500012360513142795070962165482342243194580078125f,-1.2799000255227088729270690237171947956085205078125f,-0.602400030291080579303297781734727323055267333984375f,2.14329999668598247808404266834259033203125f,0.08270001749992417217072215862572193145751953125f,-0.4393999981641769636553362943232059478759765625f,-1.7518000224947929410745928180404007434844970703125f,-0.5741999741077421504087396897375583648681640625f,0.671100040817261600523124798201024532318115234375f,1.61670001523494732964536524377763271331787109375f,-0.954100002312660677716849022544920444488525390625f,1.46570001058578469610438332892954349517822265625f,0.187599981629848144848438096232712268829345703125f,2.4779000361919401029808796010911464691162109375f,-0.638800009292364023849586374126374721527099609375f,0.075900032109021964288331218995153903961181640625f,0.585399949020146692646449082531034946441650390625f,-0.646299998712539292000656132586300373077392578125f,-0.97809996845722135816458830959163606166839599609375f,-0.20429998375177405023350729607045650482177734375f,0.84590001541972181797746088705025613307952880859375f,-0.1374000049591064254173033987171947956085205078125f,0.445500032424927194796282492461614310741424560546875f,-0.189499997787177409946934858453460037708282470703125f,0.2607999970212577789396846128511242568492889404296875f,-0.8691999904058871351253401371650397777557373046875f,0.1530999962478876152971452029305510222911834716796875f,-0.84329999970346702919954395838431082665920257568359375f,0.47599997073039423955975735225365497171878814697265625f,-0.077700015171617398124226383515633642673492431640625f,-0.9535000068843364484649782752967439591884613037109375f,-0.085799983982742134003274259157478809356689453125f,1.18280001194477080161959747783839702606201171875f,0.196200026023387597007285876316018402576446533203125f,-0.36819999434351924350039553246460855007171630859375f,-0.3359000526353719262573349624290131032466888427734375f,0.3488000046730039827025393606163561344146728515625f,1.11070001982450516919698202400468289852142333984375f,0.9720000042319298927395720966160297393798828125f,0.6523000384539365814617895011906512081623077392578125f,1.015899992173910248993706773035228252410888671875f,0.6697000168144706666595311617129482328891754150390625f,1.2324000225424764920489906216971576213836669921875f,0.3110000306516891299679627991281449794769287109375f,0.01540001413822178477630586712621152400970458984375f,-0.0517000220149756994914014285313896834850311279296875f,-0.584999981671571855912361570517532527446746826171875f,-0.63470000560134653966315454454161226749420166015625f,0.229099998140334992768885058467276394367218017578125f,0.34319997627586140698241479185526259243488311767578125f,-0.1515999990344047976265073884860612452030181884765625f,-0.21709998932629781531744583844556473195552825927734375f,1.383600010418891912422623136080801486968994140625f,-0.177000036418437645124868140555918216705322265625f,0.093000009998679189493486774154007434844970703125f,-0.45010001485943806898148977779783308506011962890625f,0.7786999757878481087658428805298171937465667724609375f,-1.6475999854668972322002673536189831793308258056640625f,-1.1127000177606938269292413679067976772785186767578125f,-0.495399978971480692280238145031034946441650390625f,0.2152000162333251864765770733356475830078125f,-0.9167999786108733673017923138104379177093505859375f,0.608200001496076225748765864409506320953369140625f,0.5579999705106022389600184396840631961822509765625f,-0.2565000124126672886859523714520037174224853515625f,-0.2139000348806376905486104078590869903564453125f,-0.1364000082820648884762704255990684032440185546875f,-0.2584000219017266175569602637551724910736083984375f,-0.17139998814687107397958243382163345813751220703125f,0.047899989383668017950412831851281225681304931640625f,-1.6704000328950583398324170047999359667301177978515625f,-0.52999996423721285054853069595992565155029296875f,-0.028599983298779108054077369160950183868408203125f,0.941200022745132258705780259333550930023193359375f,-0.89460003235340135319120236090384423732757568359375f,-0.170399953341483723079363699071109294891357421875f,0.8204000202476979808352552936412394046783447265625f,0.9597999842882156684709116234444081783294677734375f,-0.3838000026613475057502000709064304828643798828125f,-0.36119999562501892143018267233856022357940673828125f,0.392600006216764274569186454755254089832305908203125f,0.4348999836131930418758884115959517657756805419921875f,-0.316600011104345213652777601964771747589111328125f,-0.65819998566359305680606439636903814971446990966796875f,-0.31270000997483726745684862180496565997600555419921875f,-0.64460002190396192833787836207193322479724884033203125f,-1.059599978392571362206808771588839590549468994140625f,-0.14799999519065043518395441424218006432056427001953125f,0.10130001227259644025480156415142118930816650390625f,0.53760000405609587037503160900087095797061920166015625f,-0.332199995654821389479138815659098327159881591796875f,0.53919999448508004746827282360754907131195068359375f,0.112399992889165911691407018224708735942840576171875f,0.7408000002995127619698223497834987938404083251953125f,-0.40980000127553939393010296043939888477325439453125f,-0.6041000132650140574952501992811448872089385986328125f,-0.20830000711679463876180307124741375446319580078125f,-0.0726999898396434307557001375243999063968658447265625f,0.0542999982059002572754025095491670072078704833984375f,0.02150001604482520978223192287259735167026519775390625f,0.385700000751018734490571659989655017852783203125f,1.2165000026151535639229450680431909859180450439453125f,0.868399999326467497695603015017695724964141845703125f,0.951199981729686339093632341246120631694793701171875f,0.32640002310872073820746663841418921947479248046875f,0.40740000499635919961605168282403610646724700927734375f,0.497999990701675443460771930404007434844970703125f,-0.398299987037479985563237505630240775644779205322265625f,0.00699999789893634982718140236102044582366943359375f,-0.44790000013113040466805614414624869823455810546875f,0.5380000067204238067830601721652783453464508056640625f,-0.487399978137016631762890028767287731170654296875f,0.06109998031854591005185284302569925785064697265625f,0.91450002896785775874377577565610408782958984375f,0.72390005902051957065168608096428215503692626953125f,1.4041999723196028071470209397375583648681640625f,0.6450000152923163199147893465124070644378662109375f,0.527699990435689603174296280485577881336212158203125f,-0.2067000041149553712926945081562735140323638916015625f,-0.37979999158978472451053676195442676544189453125f,-0.43109997167587277289158009807579219341278076171875f,0.3454999853372573426213421043939888477325439453125f,0.86389997676610885690706709283404052257537841796875f,0.6474999763071540570535944425500929355621337890625f,-0.20799998697638510947172107989899814128875732421875f,-1.75299998950958269006150658242404460906982421875f,-0.698100005263089684603983187116682529449462890625f,-0.95349998900294252734965994022786617279052734375f,-0.12819997471123922849756127106957137584686279296875f,1.303399998953938432322274820762686431407928466796875f,-0.78020000043809412115791701580747030675411224365234375f,1.06840001676082607673379243351519107818603515625f,0.095699990282952285269857384264469146728515625f,-1.05460000370591888696480964426882565021514892578125f,-0.280399971185623986258406148408539593219757080078125f,0.12380002076625817863941847463138401508331298828125f,0.306300000835955099631746634258888661861419677734375f,-0.34460000497251730422476612147875130176544189453125f,1.054099994452297739400137288612313568592071533203125f,-0.701900000101327581347732120775617659091949462890625f,-0.200599996322393447911736075184307992458343505859375f,-1.0019000046089290290041162734269164502620697021484375f,0.800000017136335284106962717487476766109466552734375f,-0.77509996698945771154143358216970227658748626708984375f,0.93079997180104268039713133475743234157562255859375f,-0.0200999953947962328726362102315761148929595947265625f,0.1538999988198279655904343599104322493076324462890625f,-0.86870001676604136076065287852543406188488006591796875f,-0.043600000733137189712351755588315427303314208984375f,0.1339000103309750944191591770504601299762725830078125f,0.48759997715950031960119304130785167217254638671875f,0.149500005908310085178669623928726650774478912353515625f,0.24989999348521241273601845023222267627716064453125f,-0.840300023902952997190141104510985314846038818359375f,-0.7508999970912935140177069115452468395233154296875f,0.4930999672278761902788346560555510222911834716796875f,-0.108799996924400232956031686626374721527099609375f,0.960000005550682100619042103062383830547332763671875f,0.921299991746246771384676321758888661861419677734375f,0.70240001915246264019287991686724126338958740234375f,0.8515999979913235762296608299948275089263916015625f,-0.0537000432342293887444384381524287164211273193359375f,-0.354300006103515752187149701057933270931243896484375f,0.1446999793380501220241285409429110586643218994140625f,0.38500000260770317339620305574499070644378662109375f,0.3680999859660862849608520264155231416225433349609375f,-0.091500002652406653425032345694489777088165283203125f,-0.82489997426271433056399473571218550205230712890625f,-0.4701000372111803216057523968629539012908935546875f,-1.12460002341270381975846248678863048553466796875f,-1.9348000275015824200863789883442223072052001953125f,-0.30109991385936751839835778810083866119384765625f,2.0109000361323356997900191345252096652984619140625f,-0.045099990272522205714267329312860965728759765625f,-0.79240003366246813953921446227468550205230712890625f,-0.442299993582814909842682027374394237995147705078125f,0.1639000262938437035842298428178764879703521728515625f,-0.681500020384788740557269193232059478759765625f,-0.8174000403679908544063437147997319698333740234375f,-0.429499992664903462724623750546015799045562744140625f,0.3721000271938741388311200353200547397136688232421875f,-0.8930000103712085746110460604541003704071044921875f,-0.089600013391673893892175328801386058330535888671875f,0.415899989491701038701876314007677137851715087890625f,-0.2167000034630299776239326092763803899288177490234375f,-0.3146999979645013212348203524015843868255615234375f,0.65350002193450951182285280083306133747100830078125f,0.02739999929666492306523650768212974071502685546875f,0.096300005418062273321311295148916542530059814453125f,0.359599996292590873281369567848742008209228515625f,-0.4085999698877333941027245600707828998565673828125f,1.44509998550415019735737587325274944305419921875f,1.59060004266500509828574649873189628124237060546875f,-0.56740001688003616209243773482739925384521484375f,-0.5673000354737052219888937543146312236785888671875f,0.2383999922484161260172186302952468395233154296875f,-0.70570001000836501958701774128712713718414306640625f,0.3987000104516738030469014120171777904033660888671875f,-0.77909999560713760757835188996978104114532470703125f,0.0343999950349331840016020578332245349884033203125f,0.45230003167390808727077455841936171054840087890625f,1.402499996162951401146301577682606875896453857421875f,-0.555899980551004535556103292037732899188995361328125f,-1.1100999980024990332339029919239692389965057373046875f,-0.1570000005066394255237582910922355949878692626953125f,0.9967000072441993818728178666788153350353240966796875f,-0.58590001035332672785216345801018178462982177734375f,0.1008999746836722888332360525964759290218353271484375f,-0.1268000001430511503031084430404007434844970703125f,0.2624999965541066071494924472062848508358001708984375f,-0.5672000123441218732978086336515843868255615234375f,-0.092700013178586837536698794792755506932735443115234375f,0.2280999912932515361063678938080556690692901611328125f,-0.194400007365644056367415259956032969057559967041015625f,-0.043700000691413887210501343361102044582366943359375f,0.24539999599605810676195005726185627281665802001953125f,-0.16799999897181994157335793715901672840118408203125f,-0.8374000098392364588306691075558774173259735107421875f,0.1910999947726725878283104975707828998565673828125f,-0.34110000319182864902955998331890441477298736572265625f,0.997699982184171663135430208058096468448638916015625f,-0.39929999008476679822621235871338285505771636962890625f,-0.65410001795887939834983626496978104114532470703125f,-0.36830000465810286414125584997236728668212890625f,0.1244999985694885680231891456060111522674560546875f,0.98599999412521677211174164767726324498653411865234375f,-1.50420003311634076226255274377763271331787109375f,-0.14579997284412371527650975622236728668212890625f,1.00350003460049652659336061333306133747100830078125f,-0.21029999316930769737155060283839702606201171875f,-0.4041000274956225712941204619710333645343780517578125f,-2.35499998211860628316571819595992565155029296875f,1.4408000059247019208896745112724602222442626953125f,-1.09890000269412979605476721189916133880615234375f,1.0186999655246733453850538353435695171356201171875f,-0.76260001113414688234115601517260074615478515625f,-0.1083999897152185809545699157752096652984619140625f,-0.1462000095576048153134252061136066913604736328125f,0.16030002379119423494557850062847137451171875f,0.61920002834796861179711413569748401641845703125f,0.50730001528263102272831019945442676544189453125f,-0.58240000376701317463812301866710186004638671875f,-1.135599986636638192294412874616682529449462890625f,0.763399983978271023943307227455079555511474609375f,0.53760002990961108793044331832788884639739990234375f,-2.04179993696212846998605527915060520172119140625f,0.763299977202712920387739359284751117229461669921875f,0.955499986529350753272638030466623604297637939453125f,0.5069000492006543456824374516145326197147369384765625f,1.3826999763727183534456344204954802989959716796875f,0.12909995105266514059394467039965093135833740234375f,-1.71709997990131313372330623678863048553466796875f,1.1991000467181205380029496154747903347015380859375f,0.739799999189376222830105689354240894317626953125f,0.82630000884532961435979814268648624420166015625f,-1.40189999555647393236768039059825241565704345703125f,0.400400005644559797701731440611183643341064453125f,0.5613999947592616113212216077954508364200592041015625f,-0.17279999242424970429254926784778945147991180419921875f,-0.033200027349591376690796096227131783962249755859375f,-1.13379998694062233965951236314140260219573974609375f,0.065199993322789584482279678923077881336212158203125f,-0.93889998838901522759670115192420780658721923828125f,-0.977999966301024148407350367051549255847930908203125f,-0.84179998241066922304298714152537286281585693359375f,-1.126099965380132683634428758523426949977874755859375f,-0.134600004339218148885493064881302416324615478515625f,-1.395800014567374791596421346184797585010528564453125f,0.5789000050783155160161186358891427516937255859375f,0.9363999890223146937984211035654880106449127197265625f,-0.52969998656511307899563689716160297393798828125f,-0.3473000167354942657738092748331837356090545654296875f,0.743599972122907626470578179578296840190887451171875f,-0.3589000271692863730521594334277324378490447998046875f,-0.901399982130527632762095890939235687255859375f,0.142299976241588854719566370476968586444854736328125f,0.4434000024929642957971509531489573419094085693359375f,-1.260099960023164822331409595790319144725799560546875f,0.75829997433424001229695932124741375446319580078125f,-0.8650999890804289549350869492627680301666259765625f,-0.77329998446702941095054484321735799312591552734375f,-0.09239998897761125817851279862225055694580078125f,0.16319999821782094073796542943455278873443603515625f,-1.201700016687809569049250058014877140522003173828125f,-0.936300041627884382933189044706523418426513671875f,-0.88450003612041427913936786353588104248046875f,-1.48820005156397883183672092854976654052734375f,-1.321700006443261798949606600217521190643310546875f,0.2994000450655820433354392662295140326023101806640625f,0.721099984453618336743829786428250372409820556640625f,0.54029998370706966426268991199322044849395751953125f,1.88419995324611733167330385185778141021728515625f,1.1834000385165222013483798946253955364227294921875f,-0.65450000870227853511096327565610408782958984375f,-0.05040000188201698705370290554128587245941162109375f,-0.519100035169720808170268355752341449260711669921875f,0.34229997676312906573770078466623090207576751708984375f,-1.08290001771450050682688015513122081756591796875f,-0.1986000394761564535883735516108572483062744140625f,-1.0953999783754344576891526230610907077789306640625f,-1.27129997166693176069429682684130966663360595703125f,1.16579997537732094059492737869732081890106201171875f,-0.2719000027835372090834198388620279729366302490234375f,-0.54860002055168166634757653810083866119384765625f,-1.4401000271528960894329429720528423786163330078125f,0.1093000152677294778413852327503263950347900390625f,-0.28630002620518180123099227785132825374603271484375f,-1.2314999786615370425124638131819665431976318359375f,-0.99700001805275650923476860043592751026153564453125f,-0.00759999671727396020060041337274014949798583984375f,1.000399998231232157053227638243697583675384521484375f,-0.598900024747848647166392765939235687255859375f,0.4897999750494950177426289883442223072052001953125f,0.8005000217705962040071199226076714694499969482421875f,-0.654200013446807560058005037717521190643310546875f,0.008799978968500266063301751273684203624725341796875f,-0.26840001139640801142149939551018178462982177734375f,0.4623000012196598618174903094768524169921875f,0.62799997743964208751776823191903531551361083984375f,0.182500000726431077513467471362673677504062652587890625f,0.94899998804926877227217119070701301097869873046875f,-0.603299963344633649597881230874918401241302490234375f,-0.78350000062584879145788363530300557613372802734375f,-0.2045999970749019514215660819900222122669219970703125f,-0.597200015957653373988023304264061152935028076171875f,-0.36700000497698781476429985559661872684955596923828125f,-0.902300003808736761357067734934389591217041015625f,0.0490999967992304942043801929685287177562713623046875f,-0.7860999710798264317190842120908200740814208984375f,0.47030001075267779953037461382336914539337158203125f,0.83789997159540663407284455388435162603855133056640625f,1.5472999914780258823299163850606419146060943603515625f,-0.42530001268982875473056992632336914539337158203125f,1.405599965700507159027665693429298698902130126953125f,0.14420000510215746913900147774256765842437744140625f,1.3006999609649183557991136694909073412418365478515625f,-0.800899977421760667084527085535228252410888671875f,1.0630999967321752563975678640417754650115966796875f,0.017900004750490250415850823628716170787811279296875f,-0.2926999864682555685391207589418627321720123291015625f,-0.60890001267790783145983368740417063236236572265625f,-0.0496999979645018630236563694779761135578155517578125f,-0.9798999957203857746890207636170089244842529296875f,-0.50339989933967643764844979159533977508544921875f,1.437100030416250007192502380348742008209228515625f,1.321599968785047707342528156004846096038818359375f,0.3806000332772736527431334252469241619110107421875f,1.245599989914893512832350097596645355224609375f,-1.0032000433683396778405949589796364307403564453125f,1.621700014042854576246099895797669887542724609375f,0.30409999426603295802351567544974386692047119140625f,-1.4943000204086303028816473670303821563720703125f,0.4055000281035905373983041499741375446319580078125f,-0.6280000249743462603646548814140260219573974609375f,1.386000024318695178493499042815528810024261474609375f,0.283599968099593979786732234060764312744140625f,-1.141500014275312224754088674671947956085205078125f,-0.078499962180853088966614450328052043914794921875f,-0.007399983985722169421705984859727323055267333984375f,1.243399986064433360155589980422519147396087646484375f,0.4975999939233062274723806694964878261089324951171875f,-0.37910001095533374382284819148480892181396484375f,-0.354100012147426923547754995524883270263671875f,0.611700016427040083044630591757595539093017578125f,1.08810003951713429870551408384926617145538330078125f,0.974899973685294352065966450027190148830413818359375f,-0.3877000062681730252478473630617372691631317138671875f,0.807799991306662690249140723608434200286865234375f,-0.276600011327862826959744779742322862148284912109375f,0.876399987345934050608775578439235687255859375f,-1.45710002922415782222742564044892787933349609375f,-1.171400016887484962779808483901433646678924560546875f,0.26190003735423061925757792778313159942626953125f,0.8109999855756762077163557478343136608600616455078125f,0.4724999926984310150146484375f,-0.1083999739199874579043125777388922870159149169921875f,0.25859999656081189556289245956577360630035400390625f,-0.2470999820247288258912021774449385702610015869140625f,0.864300012511014958960231524542905390262603759765625f,-0.291600017139316047831698597292415797710418701171875f,-0.90450000438094146915091187111102044582366943359375f,-0.5109000177294016342699478627764619886875152587890625f,-0.3988000148057937455092769596376456320285797119140625f,-0.3815999944522976417005111215985380113124847412109375f,-0.3226000071108341415992981637828052043914794921875f,0.023700021217763822534152495791204273700714111328125f,-0.07819998033642772128359865746460855007171630859375f,-0.0051000092342492475694371023564599454402923583984375f,0.6296999799713491885455596275278367102146148681640625f,-0.72349999853968582907981499374727718532085418701171875f,-0.2909000076711176774324485450051724910736083984375f,-0.99049999509751796011869373614899814128875732421875f,0.647699999320507036060234895558096468448638916015625f,0.3413999993972478552706206755829043686389923095703125f,0.2952999939143656771278756423271261155605316162109375f,0.0231000130303204986859100245055742561817169189453125f,0.3120000002086162904646471361047588288784027099609375f,0.0504000291138886336028690493549220263957977294921875f,0.408399994111061204193902085535228252410888671875f,0.14330002224147275757815123142790980637073516845703125f,-0.003299989701062155944555343012325465679168701171875f,-0.7775000232085587104080559583962894976139068603515625f,0.9585000033453103807090656118816696107387542724609375f,-0.2843000472307206649702493450604379177093505859375f,-0.6773000484377149632564396597445011138916015625f,-1.42609998240470847719052471802569925785064697265625f,0.807399981862306770352688545244745910167694091796875f,-0.4303000071763989353712531737983226776123046875f,0.065700023102760241044961730949580669403076171875f,0.16399998521804803885970613919198513031005859375f,0.78809999982416645281091405195184051990509033203125f,-0.77419997604489321219034536625258624553680419921875f,-1.2789000076115135673404665794805623590946197509765625f,-0.63139998660087570669929846189916133880615234375f,0.74890000073835238936226232908666133880615234375f,-0.649499989908188357645713040255941450595855712890625f,-0.0828000119633978126643114592297933995723724365234375f,-0.588400031661986844255807227455079555511474609375f,-0.0174000186681748658656943007372319698333740234375f,-0.35519999663829793234981480054557323455810546875f,-1.69750001609325362750269050593487918376922607421875f,0.08410003151893619133261381648480892181396484375f,0.46640001517385254459213683730922639369964599609375f,0.095599981048703153163614842924289405345916748046875f,-1.178000026986003145879067233181558549404144287109375f,-0.315000001788138916936077293939888477325439453125f,0.79440006209015923133165415492840111255645751953125f,1.1903999917864798163691375521011650562286376953125f,-0.006100024425983452402988405083306133747100830078125f,-0.326299997203796987577106847311370074748992919921875f,-0.630900012122839637385141031700186431407928466796875f,-0.409199991150945407980543677695095539093017578125f,-0.5937000206589697626213819603435695171356201171875f,-0.182099999347329288834629323901026509702205657958984375f,-0.864099978247284905563674328732304275035858154296875f,-0.4265999913975597745974255303735844790935516357421875f,-0.471999993205070467183759319595992565155029296875f,-1.36289998239874865504361878265626728534698486328125f,0.197799986910820013719103371840901672840118408203125f,-0.73699998903274510642091854606405831873416900634765625f,-0.42369998675882813987669806010671891272068023681640625f,-0.3997000048562886487246714750654064118862152099609375f,0.28729999039769182900272426195442676544189453125f,0.7795999968327580464944048799225129187107086181640625f,-0.2019999972283839095865687340847216546535491943359375f,0.86130001285001578370525976424687542021274566650390625f,-0.334300012958049563849272090010344982147216796875f,1.044999995417892346694088701042346656322479248046875f,-0.447999984741211054739551400416530668735504150390625f,0.38959998794794092891180525839445181190967559814453125f,0.36380001167654985039234816213138401508331298828125f,0.9406999860733746832153201467008329927921295166015625f,-0.067300007943809025956483083064085803925991058349609375f,-0.6249000380769376139511450674035586416721343994140625f,0.331399995541572511825734181911684572696685791015625f,0.87710000925660125403737765736877918243408203125f,0.5965999942660331623045522064785473048686981201171875f,-0.6329999996796249472907902600127272307872772216796875f,-0.054400047039985910402037916355766355991363525390625f,0.5898999808192257887640153057873249053955078125f,-0.8910000007748610784119591698981821537017822265625f,-0.58389998719692215445320471189916133880615234375f,0.39180001914203188562169088982045650482177734375f,-1.4558999824136495959692183532752096652984619140625f,1.02450002218782909579886108986102044582366943359375f,-0.5626000129222870071998841012828052043914794921875f,0.265300028634071427546814447850920259952545166015625f,-0.8647000244140627955857780762016773223876953125f,-0.013000036299228412417505751363933086395263671875f,0.0405999607086187097593210637569427490234375f,0.1341000228762627699552467674948275089263916015625f,1.97050000560283677941697533242404460906982421875f,0.87920002320706913678804994560778141021728515625f,-1.22400001844763739455856921267695724964141845703125f,0.2858000287234789826840142268338240683078765869140625f,1.26199999618530256384474341757595539093017578125f,-0.447600018063187832240146235562860965728759765625f,0.41649999587237829246078035794198513031005859375f,-1.191500033721327955760216354974545538425445556640625f,-0.331700000929831873008879483677446842193603515625f,0.533000009477138547708818805404007434844970703125f,-1.9148999912500386244573746807873249053955078125f,0.325899980551003665141251985915005207061767578125f,-1.0769999991655350157770953956060111522674560546875f,-0.1643000016331672696878740680404007434844970703125f,2.0254000206947324613793171010911464691162109375f,0.467599953317641592320796917192637920379638671875f,-1.0554999705851078051210834019002504646778106689453125f,0.33569998972415948657044282299466431140899658203125f,-0.854699986527860833263048334629274904727935791015625f,0.2539999943077564381610500277020037174224853515625f,0.4935999654546379122876942346920259296894073486328125f,-0.4761999984562397258969212998636066913604736328125f,0.204999980069696885465191371622495353221893310546875f,-0.3938000051200389872718687911401502788066864013671875f,0.62179997585415824890020530801848508417606353759765625f,-0.7888000057160855504889696021564304828643798828125f,-0.75550001890212303834459817153401672840118408203125f,-0.06170001299977290187825929024256765842437744140625f,-0.7475999805867676872850324798491783440113067626953125f,1.10480001111030556870673535740934312343597412109375f,-0.564200014564395058158652318525128066539764404296875f,-0.87429999813139447351062472080229781568050384521484375f,0.0655000264644625129761834614328108727931976318359375f,0.3782000025391578645894696819595992565155029296875f,-0.4220999926418065317790251356200315058231353759765625f,0.1624999989569186897853114714962430298328399658203125f,0.4969000090047714923713328971643932163715362548828125f,0.08149999937415142170493709272705018520355224609375f,0.39670000979602326740547368899569846689701080322265625f,-1.06000000715255726646546463598497211933135986328125f,0.0994999857544900390138309376197867095470428466796875f,-0.678799998563528017569979056133888661861419677734375f,0.405000034272671172175250831060111522674560546875f,0.9765000093579292439471828402020037174224853515625f,0.10810003100484610882858760305680334568023681640625f,0.0283000025719406522739518550224602222442626953125f,-0.0866000141590834715543678612448275089263916015625f,-0.85809997985661023989223394892178475856781005859375f,0.76209999599456779151296359486877918243408203125f,0.60649998342990851796230344916693866252899169921875f,1.2427000216722490222309716045856475830078125f,0.3795000518970195191315042393398471176624298095703125f,-1.6247000066816810193159881237079389393329620361328125f,-0.17650004150718479056791920811519958078861236572265625f,-0.9592999792814254078621161170303821563720703125f,-2.22770001392364491721309605054557323455810546875f,-0.531699954438209942964022047817707061767578125f,0.631900050634891297107742502703331410884857177734375f,0.8797999836549161845056232778006233274936676025390625f,-1.0584000296130777751812956921639852225780487060546875f,0.011300014805794234007407794706523418426513671875f,1.740600019717216895287492661736905574798583984375f,0.52110001757144885914385668002068996429443359375f,-0.3096000523209578858541135559789836406707763671875f,0.413299944233894933631745516322553157806396484375f,-0.442400008311867765087299630977213382720947265625f,-0.266200008812546684566768817603588104248046875f,0.664400013065338246320834514335729181766510009765625f,-0.60690001083016387184443374280817806720733642578125f,0.0697000044465065915488821701728738844394683837890625f,1.219999961107969710383258643560111522674560546875f,-1.7259999881088727846645269892178475856781005859375f,-0.326099952900410272604858619160950183868408203125f,0.185699968564509987345445551909506320953369140625f,-1.25609999141991135473972462932579219341278076171875f,-0.15000000685453418469705866300500929355621337890625f,-0.407299987417459430361077465931884944438934326171875f,0.745799983870983229650164503254927694797515869140625f,-0.43750000849366188493405616100062616169452667236328125f,0.83450001361966119617363801808096468448638916015625f,0.14530001209378251569859230585279874503612518310546875f,-0.92970001020207992947774755521095357835292816162109375f,1.372999979898333666739063119166530668735504150390625f,-0.78909997757673266960409819148480892181396484375f,-0.20460005265623326664581327349878847599029541015625f,0.4031000051140782858283273526467382907867431640625f,1.012199966951460083919300814159214496612548828125f,0.650999982576817348700615184498019516468048095703125f,0.868199994585662881974030824494548141956329345703125f,0.33090000752210624312965592253021895885467529296875f,1.29880003153234735435717084328643977642059326171875f,-0.41959997692108164102364753489382565021514892578125f,-0.83589997473955168771908574854023754596710205078125f,-0.14600001463294010051185978227294981479644775390625f,-1.003699981357157877681629543076269328594207763671875f,-0.0843000055074691889700488900416530668735504150390625f,-0.27790001108348383329627040438936091959476470947265625f,-0.43909999970495705579054401823668740689754486083984375f,0.2749000089824196191301552971708588302135467529296875f,0.462399991101026586193256662227213382720947265625f,0.2106000027671457441869051763205789029598236083984375f,0.20570000790357578779321556794457137584686279296875f,1.982299988199770712782310511101968586444854736328125f,-1.56459997746124823692071004188619554042816162109375f,0.1762000345170491311108662557671777904033660888671875f,0.41839998699575653429150179363205097615718841552734375f,-0.47329997344017016303041600622236728668212890625f,-0.051999984860419345977788907475769519805908203125f,-0.157299995017051941204044851474463939666748046875f,-0.18309998663663851203864396666176617145538330078125f,1.63229998548030863503299769945442676544189453125f,-1.07699999380111677282911841757595539093017578125f,0.63430002875328028721924056299030780792236328125f,-0.0305000243783002389363900874741375446319580078125f,0.11030000658035277183444122783839702606201171875f,-1.984599974536894961829602834768593311309814453125f,0.84390002295970845125339110381901264190673828125f,0.378899973934888834037337801419198513031005859375f,-0.1351999659419060861864636535756289958953857421875f,-0.047899997225404167267015509423799812793731689453125f,1.13490002328753458726851022220216691493988037109375f,1.5511999844491477507091303777997381985187530517578125f,-0.1233999863624575255016679875552654266357421875f,-0.58930000461339915318603743799030780792236328125f,0.04460001715421668677663546986877918243408203125f,-0.33009997194409379517310298979282379150390625f,0.073000046581029209846747107803821563720703125f,0.351300000053644367881133803166449069976806640625f,0.849100021386146153190566110424697399139404296875f,-1.93140002683401146299502215697430074214935302734375f,-0.1974999600648885689224698580801486968994140625f,0.1898999998927113352920059696771204471588134765625f,0.5437999724864965855886111967265605926513671875f,0.7779999645128850005448839510791003704071044921875f,0.75769999023079748212694539688527584075927734375f,-0.77229996946156076464973239126265980303287506103515625f,0.1968000254750261746039541321806609630584716796875f,0.66329997276961838448272601453936658799648284912109375f,-0.04040001279711713522146965260617434978485107421875f,0.05689998952150343569655888131819665431976318359375f,0.65830001069307275685105196316726505756378173828125f,-0.4719999929815530403942602788447402417659759521484375f,0.898599957072735211482950035133399069309234619140625f,-0.624599980199337156960837091901339590549468994140625f,-0.05350001537799808914996901876293122768402099609375f,-0.494699995654821422785829554413794539868831634521484375f,-0.232000028222798970745088809053413569927215576171875f,-0.7579999940171837646829544610227458178997039794921875f,-1.264800019007921161318108715931884944438934326171875f,0.11469999785274265224899181703221984207630157470703125f,-0.615299963515997472285334879416041076183319091796875f,-0.0262999926403164607791040907613933086395263671875f,-1.099199973362684712441250667325221002101898193359375f,-0.829799982984364081772810095571912825107574462890625f,-1.17940004078149751620685492525808513164520263671875f,0.170700000862777034882356019807048141956329345703125f,1.240200035381317178462268202565610408782958984375f,-0.6071000041157004201153313260874710977077484130859375f,-0.418900040841102327959788453881628811359405517578125f,-0.16729998760372399857487835106439888477325439453125f,-0.62950004893541322559258333058096468448638916015625f,-0.7081999945297836607238650685758329927921295166015625f,0.0570000025928021347709773181122727692127227783203125f,0.2328000071763993705786788268596865236759185791015625f,-0.24200000900030094186377027654089033603668212890625f,0.05290000068992384285593288950622081756591796875f,0.142199993717670114534712411114014685153961181640625f,-1.0923999847307801491780310243484564125537872314453125f,-0.45979997564554242472922851447947323322296142578125f,0.266900003118813178293322607714799232780933380126953125f,-0.0187999724864962303172433166764676570892333984375f,-0.38720000832080792019240789159084670245647430419921875f,-0.23679998255968115472569479607045650482177734375f,0.4557999909117820269699450363987125456333160400390625f,-0.6096000168561932497368616168387234210968017578125f,0.56479999881684772589096610317938029766082763671875f,0.6782000087976456370597588829696178436279296875f,-0.93870002851933254106597814825363457202911376953125f,-0.16779999842196680059913660443271510303020477294921875f,0.76460001536607735683759301537065766751766204833984375f,-0.0322000051915647400591069526853971183300018310546875f,0.6003000126525759316820085587096400558948516845703125f,0.3727999989807602165825528572895564138889312744140625f,-1.3082000241458422440388176255510188639163970947265625f,-0.6936000038623806318582865060307085514068603515625f,-0.204799968269467225212565608671866357326507568359375f,-0.21239998178780095106077396849286742508411407470703125f,0.311399998708069614128390867335838265717029571533203125f,-0.381400019681453983366736792959272861480712890625f,-0.55790001451075077465446838687057606875896453857421875f,-1.44320001334250047619889301131479442119598388671875f,-0.6475999792084099482281089876778423786163330078125f,-0.380600026050209816474989565904252231121063232421875f,-0.43019996646344704771536271437071263790130615234375f,-0.4816999828994268373349996181786991655826568603515625f,0.75849997395277046763339967583306133747100830078125f,0.854400003528594798041240210295654833316802978515625f,0.10569998935163038566997784073464572429656982421875f,0.65809998812675463142340959166176617145538330078125f,0.183299996946752141724346074624918401241302490234375f,0.14030002326965274050962761975824832916259765625f,-0.7282999710932376302707780268974602222442626953125f,0.50589996892809896422704696306027472019195556640625f,-0.65729996316582006699036355712451040744781494140625f,-1.05839997622966652812692700535990297794342041015625f,0.2568999987602234025274583473219536244869232177734375f,-0.76539999124631297178211752907373011112213134765625f,-0.06920000482276089659450235558324493467807769775390625f,0.68010001025870436830444987208466045558452606201171875f,0.41209998295605199114532979365321807563304901123046875f,0.56150000399351096547206907416693866252899169921875f,0.56709997986406113223978309179074130952358245849609375f,0.6754999568760398087619023499428294599056243896484375f,-0.35340001236498341796732347575016319751739501953125f,0.247599982784687977499515909585170447826385498046875f,0.192799996000528384154648620096850208938121795654296875f,1.377299995240568453169771601096726953983306884765625f,-0.162599994407594206080602816655300557613372802734375f,0.311199998903274543948782593361102044582366943359375f,-0.6163999866753815393138893341529183089733123779296875f,-0.12400001170486218970978598008514381945133209228515625f,-1.1079000122755766621907014268799684941768646240234375f,-0.53400001174211553234272287227213382720947265625f,0.381299999122321597866402953513897955417633056640625f,-0.2018000069975851573644831660203635692596435546875f,1.124999998584389704348041050252504646778106689453125f,0.8527999967455865970578088308684527873992919921875f,-1.4993999935075639218240439731744118034839630126953125f,-0.56680000282525977439718189998529851436614990234375f,-0.4756000025436282196977799685555510222911834716796875f,-0.164399988031387778164571500383317470550537109375f,0.202099962094425933400998474098742008209228515625f,-0.0613999866008754224822041578590869903564453125f,1.058300011102855098243935572099871933460235595703125f,0.1043999823689460981768206693232059478759765625f,-0.18860001164823747554777355617261491715908050537109375f,0.24019998233318329283747516456060111522674560546875f,0.65980002627223743871809347183443605899810791015625f,0.29919999474585046161934087649569846689701080322265625f,-0.8852000062495470888990212188218720257282257080078125f,-0.20779999696910367301683209007023833692073822021484375f,0.167299999785423370024517453202861361205577850341796875f,-0.087800017607211788828180942800827324390411376953125f,-0.028300001864135282403367455117404460906982421875f,0.14219998805522937601608646218664944171905517578125f,-1.1740999701261518595174493384547531604766845703125f,-0.499199981856345953445952545735053718090057373046875f,0.0326999791666866546080427724518813192844390869140625f,0.8044000020198514011582346938666887581348419189453125f,0.0562999996252357970405455489526502788066864013671875f,0.1819000178150833590962065500207245349884033203125f,-0.90470001230686858395557692347210831940174102783203125f,-0.181600074136258005097488421597518026828765869140625f,-0.20219998734742417045850970680476166307926177978515625f,-0.539399992352723955946203204803168773651123046875f,0.905599995651841016552907603909261524677276611328125f,0.3550000359117984061185779864899814128875732421875f,-0.56119997088909190363636980691808275878429412841796875f,-0.6043999518215645849750217166729271411895751953125f,-0.9547000032916661549364789607352577149868011474609375f,-1.1622000665843490452289188397116959095001220703125f,0.05259999843090756854735445813275873661041259765625f,0.75800002124905585532133045489899814128875732421875f,-0.0306000008672475676263502464280463755130767822265625f,-0.683699964183569708353616078966297209262847900390625f,-0.4395999831423165193911017922800965607166290283203125f,-0.759599991635977911386135019711218774318695068359375f,-0.14819998301863679390777406297274865210056304931640625f,-0.7415000227317209979815970655181445181369781494140625f,-0.18709999156147250065629350501694716513156890869140625f,0.22739999545961619986655932734720408916473388671875f,-0.683999971918761939804198846104554831981658935546875f,-1.0827000017046923829866500454954802989959716796875f,-0.1676999908268452799120495910756289958953857421875f,-0.46310000272989260139411271666176617145538330078125f,-0.51969996775239746256147554959170520305633544921875f,-0.726299976360797128194235483533702790737152099609375f,-0.26069999706298130348613995010964572429656982421875f,0.0119999990165233771932662421022541821002960205078125f,0.2270000311657784930474690554547123610973358154296875f,-0.1874000019788741866477721487171947956085205078125f,-0.40730002508014440110173381981439888477325439453125f,0.251999994397163362691571819595992565155029296875f,0.97720003022551560345476673319353722035884857177734375f,0.47160000737905516388082105549983680248260498046875f,-0.939300007034838291275491428677923977375030517578125f,-0.3224000057041644939914704082184471189975738525390625f,-0.341199996519088610824610441341064870357513427734375f,-0.39080000264644620955323262023739516735076904296875f,0.10409999814033510379118752098293043673038482666015625f,0.11359999879598614569431447307579219341278076171875f,0.2915000130459663996163044430431909859180450439453125f,-0.1926000045776365965366494492627680301666259765625f,-1.068699987392127237484373836196027696132659912109375f,-0.0465999760121109662946992102661170065402984619140625f,0.86309999840855600172773165468242950737476348876953125f,0.046599996352195471871482368442229926586151123046875f,-0.1592000305086376243934864760376513004302978515625f,-0.019200022685527784460646216757595539093017578125f,-0.18319999281615029218528434284962713718414306640625f,0.05540000042915327327364138909615576267242431640625f,-0.3887999954715370964919429752626456320285797119140625f,-0.2281999939709902225359883232158608734607696533203125f,0.5066999967575072094660981747438199818134307861328125f,0.350200017797946738795644705533050000667572021484375f,1.17530003865510224159152130596339702606201171875f,-1.500900023987889131404926956747658550739288330078125f,-0.38450000024586905578161122321034781634807586669921875f,-0.0082000485092397212838477571494877338409423828125f,-0.59659998618215315246970931184478104114532470703125f,-1.42400002112984669366824164171703159809112548828125f,-0.3580999991908668800277837362955324351787567138671875f,-0.67530002326965377079659447190351784229278564453125f,0.82070001218765942230248811029014177620410919189453125f,0.62310003677904701735457138056517578661441802978515625f,-0.5501999957442282873643080165493302047252655029296875f,-0.060400040140748512129675873438827693462371826171875f,0.3413999979257582584324381969054229557514190673828125f,0.671000042945146812911616507335565984249114990234375f,-0.6117999867320060847220020150416530668735504150390625f,-0.3127000619053834640226341434754431247711181640625f,0.1798999948263169113005233157309703528881072998046875f,-0.36159999013096111308840363562921993434429168701171875f,-0.0928999997213482142655038842349313199520111083984375f,0.46079998144954448235921518062241375446319580078125f,-0.125600001277029360569059690533322282135486602783203125f,-0.272800018650293463196021548355929553508758544921875f,-0.507800003898143614833315950818359851837158203125f,-0.90490002224445376111106043026666156947612762451171875f,0.34219998138695972844658399480977095663547515869140625f,0.49329999575465877104107903505791909992694854736328125f,-0.085399998939037324863221556370262987911701202392578125f,-0.1557000005647537488329135157982818782329559326171875f,0.76249997422099136112905171103193424642086029052734375f,1.376899979686736941175695392303168773651123046875f,0.6202999959260224738244460240821354091167449951171875f,0.570600006753206212550821874174289405345916748046875f,-0.90830001926124126487849252953310497105121612548828125f,-0.591000013887882147400887333787977695465087890625f,-0.18009999653100994265741974231787025928497314453125f,-0.4242000168740751586682335982914082705974578857421875f,-1.2613999830618498609879907235153950750827789306640625f,-0.631999981582164860327566202613525092601776123046875f,1.5897999829471112231971119399531744420528411865234375f,0.38459999152421975310289781191386282444000244140625f,-1.0617999782383444884459322565817274153232574462890625f,-0.429599981391429963650807621888816356658935546875f,0.4981999907299876628741230888408608734607696533203125f,-0.00840001914501176116800706950016319751739501953125f,-0.138499985277652781778812141055823303759098052978515625f,-0.16980001308470960008634165205876342952251434326171875f,-0.70789998053610336814500669788685627281665802001953125f,0.23320004307031627632795789395458996295928955078125f,0.001599999928474649113496752761420793831348419189453125f,-0.43599999392032628975357511080801486968994140625f,0.345599971064924904595727639389224350452423095703125f,-0.3396000392079354668339874478988349437713623046875f,-0.0741999927341938647629149272688664495944976806640625f,0.45140000843107686012700696664978750050067901611328125f,0.352800007362663592402896028943359851837158203125f,-0.03120001231431945853955767233856022357940673828125f,-1.267899981690943089773782048723660409450531005859375f,-0.357799980950356033559955903911031782627105712890625f,1.294500003412366151422929760883562266826629638671875f,-0.7788999807149170617748268341529183089733123779296875f,0.1088000021770596514869566817651502788066864013671875f,-0.08990001620948273597377919941209256649017333984375f,-0.7834000030145047421825665878714062273502349853515625f,-0.03570004105865953736298479270772077143192291259765625f,-0.79250001028180083739727024294552393257617950439453125f,-0.77250005401671018034903681837022304534912109375f,-0.654399979686737243156358090345747768878936767578125f,0.252299971175193871175679305451922118663787841796875f,0.77500000026077009440683696084306575357913970947265625f,0.77900001129508122943434500484727323055267333984375f,0.699999970570206908604404816287569701671600341796875f,1.82599999824166214779097572318278253078460693359375f,-0.10189999432712804061651468146010302007198333740234375f,0.175199995688349152711538181392825208604335784912109375f,0.0608000004671513938347970906761474907398223876953125f,-0.77479998866543109414806167478673160076141357421875f,0.10009999947398906527951112366281449794769287109375f,0.558899973338842226411316005396656692028045654296875f,-0.26779998862445364427031790910405106842517852783203125f,0.88779998631477408110868054791353642940521240234375f,-0.7919000109419223054629810576443560421466827392578125f,-1.5808000341251500220351999814738519489765167236328125f,0.67209998113065971825363931202446110546588897705078125f,0.968899993343651289023910067044198513031005859375f,-0.15619999502897263798928406686172820627689361572265625f,-0.564300005656480863081014831550419330596923828125f,0.297399981229007515093343272383208386600017547607421875f,1.298699985194205819283297387300990521907806396484375f,0.46140000396072877730802019868860952556133270263671875f,-0.39140001603066931767926917018485255539417266845703125f,-0.263899997404217712126950345918885432183742523193359375f,-0.20960000512152909823981872250442393124103546142578125f,-0.7104000096306204437013320784899406135082244873046875f,-0.347400001718103423886674363529891707003116607666015625f,0.1769000048510730171980043223811662755906581878662109375f,0.036400015807152019675640985951758921146392822265625f,0.61730000399500095653593234601430594921112060546875f,-0.14260002782344827210181392729282379150390625f,-0.2916999953791499233801687296363525092601776123046875f,0.299200001525878889196974341757595539093017578125f,-0.9214000497817995682936498269555158913135528564453125f,-0.155600011968612506052522803656756877899169921875f,0.963400050213933045739622684777714312076568603515625f,-0.0854000057935717649115758831612765789031982421875f,0.0740000006780030528119596056058071553707122802734375f,0.2343999907135962956772345933131873607635498046875f,-0.246299999494850485604047207743860781192779541015625f,-0.239000035881996808484473149292171001434326171875f,-0.05799999327212557176380869350396096706390380859375f,0.51260000606775246723145755822770297527313232421875f,-0.753100033612549513151179780834354460239410400390625f,-0.21740000019073502102173733874224126338958740234375f,-0.0519000122085213622113997189444489777088165283203125f,-0.0336000071406363787218651850707828998565673828125f,0.19260000044256442297552212039590813219547271728515625f,-0.093200001943111399071995037957094609737396240234375f,0.18470001577138928094967695869854651391506195068359375f,0.5486999771103260314930594176985323429107666015625f,-0.03969999926835299230987175178597681224346160888671875f,0.516699994801729900473219458945095539093017578125f,-0.0103999989204108800322501338087022304534912109375f,0.1690999992579220734256750802160240709781646728515625f,-0.60759997186958825121649852007976733148097991943359375f,-0.5141999961987131495533276392961852252483367919921875f,0.785199990379810142115957205533050000667572021484375f,1.868100023889542438126909473794512450695037841796875f,-0.7203000082194801478863155352883040904998779296875f,-1.53450004908442583229088995722122490406036376953125f,0.4048999921813607461018591493484564125537872314453125f,-0.3024999591708183999116954510100185871124267578125f,-0.847800005090236563631833632825873792171478271484375f,-0.23030002326965348657950016786344349384307861328125f,-0.65629998612105833188934411737136542797088623046875f,-1.12010003706216831886877116630785167217254638671875f,-0.7142999696329237480085794231854379177093505859375f,-0.844000026047229567893737112171947956085205078125f,-0.651100003936886384536819605273194611072540283203125f,-1.497000039845705732233227536198683083057403564453125f,0.1485000028610230504710898458142764866352081298828125f,-0.41380003030300116506623453460633754730224609375f,0.48919996535331045084404877343331463634967803955078125f,0.808000018864869051782306996756233274936676025390625f,-0.45219999371767050089232498066849075257778167724609375f,-0.346600006186962072973756221472285687923431396484375f,0.132299993415176897482155027319095097482204437255859375f,0.7062000088870519221018184907734394073486328125f,0.38040000899732095973604373284615576267242431640625f,-0.12239999065399143063359588268212974071502685546875f,-0.18409999158382406303502420996665023267269134521484375f,-0.0951999920189379178481203780393116176128387451171875f,-0.2524000136017801043664121607434935867786407470703125f,-0.099999986998737078192078797655995003879070281982421875f,0.5093999985925850015888727284618653357028961181640625f,0.18820001147985454537092664395458996295928955078125f,-0.0517999956727026589220486130216158926486968994140625f,0.004400014108419281200212935800664126873016357421875f,0.82929999127686004900539273876347579061985015869140625f,0.085599992150068526797213053214363753795623779296875f,0.74239999490082231314858063342398963868618011474609375f,0.697799993467331347574145183898508548736572265625f,-1.325499992601573584494190072291530668735504150390625f,-0.475400001770257762956362057593651115894317626953125f,0.80980000917315475561508719692938029766082763671875f,0.28200000084191578775261177725042216479778289794921875f,-0.85679998054802408713470640577725134789943695068359375f,-0.0609999733567239132980830618180334568023681640625f,0.720999981403350975739385830820538103580474853515625f,0.03439998683929434974970718030817806720733642578125f,0.0036999824374912559932226940873079001903533935546875f,0.32879999871253939858206649660132825374603271484375f,-0.3796999957293272220937296879128552973270416259765625f,-0.495600007498264272243204686674289405345916748046875f,-0.0644000021129846889067493975744582712650299072265625f,0.1132000095427037056339258924708701670169830322265625f,-0.77869999793469901749887185360421426594257354736328125f,-0.203100028806924459701122032129205763339996337890625f,0.350299998049437899538816054700873792171478271484375f,0.62350001351535322413610629155300557613372802734375f,0.2144000043481586736948685256720636971294879913330078125f,0.304200018606335387261196956387721002101898193359375f,0.7228999623566869647817156874225474894046783447265625f,1.05180001582652327130062985816039144992828369140625f,-0.3506000079080460185565470965229906141757965087890625f,-0.8354000243455177443507864154526032507419586181640625f,0.03489999918490649388758129134657792747020721435546875f,0.5630999821662905358010675627156160771846771240234375f,0.13530000013560050664551681620650924742221832275390625f,-0.3479999962896105358822751441039144992828369140625f,0.6836999885469674520521721206023357808589935302734375f,0.1706000183016060933738344829180277884006500244140625f,0.05649999177455899967981167719699442386627197265625f,0.633900013702362929990385964629240334033966064453125f,0.016600000952929239161903751664794981479644775390625f,-0.944099981991201264719393293489702045917510986328125f,-0.85589998286068436073037446476519107818603515625f,-0.79010003244280824219458736479282379150390625f,-0.81029998240321876590286365171778015792369842529296875f,-1.815300002855062100337590891285799443721771240234375f,-0.4857000100269910358719016585382632911205291748046875f,-0.4501000620961195242131225313642062246799468994140625f,-0.3876999922797084163761383024393580853939056396484375f,-0.39790000273883319170664663033676333725452423095703125f,-0.16239999467730525939401786672533489763736724853515625f,-0.013600018838047844838001765310764312744140625f,-0.8678999904826281142078414632123894989490509033203125f,-0.47279998929500610671539106988348066806793212890625f,-0.283700026768446189606720508891157805919647216796875f,-0.6242000134468081995464672218076884746551513671875f,-0.433899990959465409190443097031675279140472412109375f,-0.685200006175040954303767648525536060333251953125f,0.502199988055229251671107704169116914272308349609375f,0.14159998536258944223931166561669670045375823974609375f,-1.2596000036686658685169959426275454461574554443359375f,-0.4691999976515770498508572927676141262054443359375f,0.6993000107228757311617073355591855943202972412109375f,0.145000008046627382185533861047588288784027099609375f,-0.3904999951347709608029390437877736985683441162109375f,-0.391599999630451112153650683467276394367218017578125f,-0.5440000396445394681421703353407792747020721435546875f,0.0262000027775766142212887643836438655853271484375f,0.6151000059932467678436296409927308559417724609375f,0.2390000159144403113486987422220408916473388671875f,0.4796000054940581236451180302537977695465087890625f,-0.10220001659095279133993017239845357835292816162109375f,0.105600016103685023782787766322144307196140289306640625f,0.2055999958753584877513276296667754650115966796875f,-1.16110004849135872717624806682579219341278076171875f,-0.2566000011950728687537548466934822499752044677734375f,0.54300000865757469536987400715588591992855072021484375f,-0.0088000048220158166856208481476642191410064697265625f,-0.01930000558197508819802123980480246245861053466796875f,-0.0197999940484760994507951181731186807155609130859375f,-0.73109996809959409969081889357767067849636077880859375f,-0.27119999834448105247020066599361598491668701171875f,-0.06930000137239689106394280315726064145565032958984375f,1.042199990588426938842303570709191262722015380859375f,-0.89320001300722318138269884002511389553546905517578125f,0.0571999259173860963301194715313613414764404296875f,-0.8576999679908159368579845249769277870655059814453125f,-0.408300008029490235639258344235713593661785125732421875f,0.07289999977722760871046148167806677520275115966796875f,-0.24149998702481401124941839952953159809112548828125f,-0.24790000285059210494154058324056677520275115966796875f,0.2598000011265277908734105949406512081623077392578125f,-0.514199966992438195489967256435193121433258056640625f,-0.4820000227093685918333676454494707286357879638671875f,-0.14840000007152553962441743351519107818603515625f,-0.43789998895525972244513468467630445957183837890625f,0.243499985277652708504092515795491635799407958984375f,0.19360004126429597448577624163590371608734130859375f,-0.25490000190436834515139707946218550205230712890625f,-0.088400014637410873064027327927760779857635498046875f,-0.74259998181611308609717525541782379150390625f,-1.21040000735819308630425439332611858844757080078125f,-0.676899977935850660770711328950710594654083251953125f,-1.69580002198070278751629302860237658023834228515625f,-0.069300005991757007706155491177923977375030517578125f,0.42330003792494530490131410260801203548908233642578125f,-0.51289999413341280654066167699056677520275115966796875f,-0.76940001898854959438978085017879493534564971923828125f,-0.0421000005766749563207440587575547397136688232421875f,0.383399977235496347560683716437779366970062255859375f,0.6915999784708020126089422774384729564189910888671875f,0.8458000106930736450294716632924973964691162109375f,-1.094999987334012825357376641477458178997039794921875f,-0.2239999998211867904274186003021895885467529296875f,1.5876999938443303594937106026918627321720123291015625f,0.41879999305009807386568354559130966663360595703125f,-1.327699997569620649073840468190610408782958984375f,-0.37659998566061270164340157862170599400997161865234375f,1.7575999966055155798727582805440761148929595947265625f,0.89880000675916704722112626768648624420166015625f,-0.94549999610334634070341053302399814128875732421875f,-0.15699999354034666065871306273038499057292938232421875f,0.570200005951523980485262654838152229785919189453125f,0.275600014054775410698994164704345166683197021484375f,-0.12129999986737960426808058400638401508331298828125f,-0.21280000307857982466686053157900460064411163330078125f,-0.092200010332465220930231453166925348341464996337890625f,0.044800001424551094686421492951922118663787841796875f,0.2712000078067184904995201577548868954181671142578125f,-0.0108000042855740385761009747511707246303558349609375f,-0.64680000662505621189524163128226064145565032958984375f,-0.43420000778436662614012675476260483264923095703125f,0.15649998961389066653993040745262987911701202392578125f,-0.1776000227570531109222429222427308559417724609375f,-0.6532999616309995705165647450485266745090484619140625f,-0.3105999812722206598891716566868126392364501953125f,0.6449999807775019800004656644887290894985198974609375f,0.242999997705221293387012337916530668735504150390625f,0.5413999897301196018162272594054229557514190673828125f,-0.6325000120699399985824129544198513031005859375f,0.8141999832347035503943288858863525092601776123046875f,-0.1949999384582046246805475675500929355621337890625f,-0.8824000099509949901488425894058309495449066162109375f,-0.48640007064342594134132014005444943904876708984375f,0.91329997899085302304200695289182476699352264404296875f,0.5552000115580855155172912418493069708347320556640625f,-0.08650000054761763923494299888261593878269195556640625f,0.073499999005347316227698684087954461574554443359375f,0.7631000104412433149292382950079627335071563720703125f,-0.179199949204176089967432972116512246429920196533203125f,-0.067900000261515380817201048557762987911701202392578125f,-0.0240999955885115024756260027061216533184051513671875f,0.4670000115334984602810664000571705400943756103515625f,0.6830000305250292225167640935978852212429046630859375f,-0.1857999915078283326153041343786753714084625244140625f,-0.2589999918863175043526325680431909859180450439453125f,-0.08140000023543836338291157517232932150363922119140625f,0.09059999945163699219818909114110283553600311279296875f,0.26089999742656944103913474464206956326961517333984375f,0.625000009015202362405716485227458178997039794921875f,-0.21659999735802415443686186335980892181396484375f,0.494400005018710952953142623300664126873016357421875f,-0.80469997542649529531644247981603257358074188232421875f,-0.91520001451969168471123339259065687656402587890625f,0.6074000150546428056230752190458588302135467529296875f,1.2503000412255536044625614522374235093593597412109375f,-0.0022999975502491221135414889431558549404144287109375f,-0.5977999975465235760196947012445889413356781005859375f,-1.7117000259637833448067567587713710963726043701171875f,-0.4652000014066697275438855285756289958953857421875f,0.9247999970287084092745999441831372678279876708984375f,0.1685999976038932146593651850707828998565673828125f,-0.345500014506280539450244759791530668735504150390625f,0.2904000266551971520812003291212022304534912109375f,0.4447000136852263807440976961515843868255615234375f,-0.1819999845623971879149394226260483264923095703125f,0.2086000181302425371399067444144748151302337646484375f,-0.1471999767303469042190045001916587352752685546875f,-0.21440000695586203249831669381819665431976318359375f,-0.241599998289346640234498408972285687923431396484375f,-1.027900016000866667553736988338641822338104248046875f,-0.8562000194668772934392109164036810398101806640625f,1.9333999881505963269745507204788736999034881591796875f,0.9673999992966653138637411757372319698333740234375f,-1.61359999894499761552424388355575501918792724609375f,-0.67080001158714264164473206619732081890106201171875f,-0.0759999952614303087017333382391370832920074462890625f,0.1056000209093088670897486736066639423370361328125f,0.5799999829009172547245043460861779749393463134765625f,-0.36820000797808172254832470571273006498813629150390625f,-0.89859998277723762694080278379260562360286712646484375f,-0.769200009274482621179913621745072305202484130859375f,-0.3101999989852310957161307669593952596187591552734375f,0.1152000374674795324381193495355546474456787109375f,0.3423999905422332457050060838810168206691741943359375f,-0.45640001997947710066227955394424498081207275390625f,-0.40090000796914093683653845801018178462982177734375f,-0.11779999272227303919180485536344349384307861328125f,-0.7899000021651385594623206998221576213836669921875f,-0.34980002958774603083469401099137030541896820068359375f,0.1952999914556742544480272272267029620707035064697265625f,-0.7412999834388493969328237653826363384723663330078125f,0.5794999773725868497109559029922820627689361572265625f,1.1476999952971933538492521620355546474456787109375f,0.7975999737322332361344479068065993487834930419921875f,1.7401999809920791761896907701157033443450927734375f,0.8310999662742022398020935725071467459201812744140625f,1.5538999822050325594346986690652556717395782470703125f,0.06399999825656410568086585044511593878269195556640625f,0.868100016364454774020487093366682529449462890625f,-0.772900002068281022360451970598660409450531005859375f,-0.45500001594424244188985539949499070644378662109375f,0.74709998310506353647042487864382565021514892578125f,0.07110001772046015133810215047560632228851318359375f,-0.89729997009485995551614223586511798202991485595703125f,-1.09409995492696765495566069148480892181396484375f,0.34299999226629740434191262465901672840118408203125f,1.411899984343350400450844972510822117328643798828125f,-0.27550000742822866595105324449832551181316375732421875f,0.5760999606117598315080385873443447053432464599609375f,-0.633699971373379522532331975526176393032073974609375f,-0.7532999963879571936331558390520513057708740234375f,0.3179999816864731432275448241853155195713043212890625f,1.371199970740078821762608640710823237895965576171875f,0.0100999994739889853434533506515435874462127685546875f,0.604800016325711897735573074896819889545440673828125f,0.7984000151962040714437307542539201676845550537109375f,0.8286000288963322191193583421409130096435546875f,-1.1216000229135156818216501051210798323154449462890625f,-0.08139998332261999536285657086409628391265869140625f,0.5375000202469528698401290967012755572795867919921875f,-0.23100000174343560122025564851355738937854766845703125f,0.19289998556524479145224404419423080980777740478515625f,-0.2576000248432155359523676452226936817169189453125f,-0.5801999626636502593868272015242837369441986083984375f,-0.0831999894261361561120793339796364307403564453125f,0.7002000006243587204579625904443673789501190185546875f,0.40700000952184234304098708889796398580074310302734375f,-0.4035999856829641796451824120595119893550872802734375f,-0.0207999820828437265163302072323858737945556640625f,0.8096999924138186255362370502552948892116546630859375f,0.508600007438659407199565976043231785297393798828125f,0.1670000066161156926369812936172820627689361572265625f,0.0605999885737900090276752962381578981876373291015625f,0.90870001022815716229530380587675608694553375244140625f,0.9222000086188320011615360272116959095001220703125f,0.2310999883651734254641496590920723974704742431640625f,0.567399983650445971505860143224708735942840576171875f,0.3225999953761695682175059118890203535556793212890625f,0.667200004148483127863755726139061152935028076171875f,0.55970001081675280207861078451969660818576812744140625f,0.4786000024467707003594796333345584571361541748046875f,-0.926699976491927923660796295735053718090057373046875f,-0.42839997891187664436074555851519107818603515625f,1.344699971105157931816620475728996098041534423828125f,0.168500006437302118911247816868126392364501953125f,0.74450001019239397948012992856092751026153564453125f,-0.533199962157010265428880302351899445056915283203125f,-0.2856999958336354783483557184808887541294097900390625f,0.33920000584721565761725514676072634756565093994140625f,-0.63219997078850898031276983601856045424938201904296875f,-0.0360000285468984326797681205789558589458465576171875f,-0.5324999972060322495082118621212430298328399658203125f,-0.7163000240445143163015018217265605926513671875f,0.8152999929830431202759655207046307623386383056640625f,1.17160002432912602188253003987483680248260498046875f,0.73360000851899354046281587216071784496307373046875f,0.46140003540217933153400053924997337162494659423828125f,0.7930999915912746445201264577917754650115966796875f,0.9270000371634965485867496681748889386653900146484375f,0.7515999652460221280847463276586495339870452880859375f,0.665299998012184090612208819948136806488037109375f,-0.64529996187686966546692701740539632737636566162109375f,-1.7445999791562549763312972572748549282550811767578125f,0.5970000156685706382830858274246565997600555419921875f,-0.6818999365478750629421256235218606889247894287109375f,-0.012099997261166617334993134136311709880828857421875f,-1.3570999759525059769060817416175268590450286865234375f,0.7464999740049245158246549181058071553707122802734375f,0.12460001517981267316059756922186352312564849853515625f,0.094000000193715127494442640454508364200592041015625f,-0.248400028420984853028130601160228252410888671875f,0.4937000022187827408259863659623079001903533935546875f,1.51520000796318132785245325067080557346343994140625f,-1.141600010955333477369322281447239220142364501953125f,-0.3250000038743028341059471131302416324615478515625f,1.194499977260827794367514798068441450595855712890625f,-0.03780001909732799703078853781335055828094482421875f,-0.003799977627396311508078952101641334593296051025390625f,0.03079997820854174506166600622236728668212890625f,-0.2721999920040369946860892014228738844394683837890625f,0.04479998458623857260363365639932453632354736328125f,-0.92210003686100261433011837652884423732757568359375f,0.035200007069111194368815631605684757232666015625f,0.04660001330226659632671726285479962825775146484375f,-0.219199992883205130311807806720025837421417236328125f,0.50830001717507844460897103999741375446319580078125f,-0.15960000433921806006765109486877918243408203125f,0.62850001604855076919875500607304275035858154296875f,0.63180000044107398338155689998529851436614990234375f,-0.858000007614493132024335864116437733173370361328125f,-0.51100001738965505904133124204236082732677459716796875f,-0.285699991177022383226358215324580669403076171875f,-0.0879999850392341187443889793939888477325439453125f,-0.79420000901371234913739272087696008384227752685546875f,-0.62920000420808808172523640678264200687408447265625f,0.9923999804094432164447425748221576213836669921875f,0.11719997565001272210594152056728489696979522705078125f,-0.1826999768942595137044548891935846768319606781005859375f,0.09479999248385428955998577293939888477325439453125f,-0.1905999846249818718835200570538290776312351226806640625f,0.00159999843835834099081694148480892181396484375f,-0.314400001740455736776169715085416100919246673583984375f,-0.50699996727705087806725714472122490406036376953125f,-0.72700001499801825133317834115587174892425537109375f,0.258899920886754397741924549336545169353485107421875f,-0.048999997474253209617245374829508364200592041015625f,0.12899999773502290878468556911684572696685791015625f,0.2489000069782136836948893687804229557514190673828125f,0.21030001492500360882331733591854572296142578125f,-0.45969999993890497957949037299840711057186126708984375f,-1.4467000447392468487350924988277256488800048828125f,0.6042999846830963139865389166516251862049102783203125f,1.16290003380775441854666496510617434978485107421875f,0.220900003238022113460914397364831529557704925537109375f,1.248100007647276132871638765209354460239410400390625f,0.580899995116889389379366548382677137851715087890625f,0.478000050306320201087828536401502788066864013671875f,0.2411999958857895531849635517573915421962738037109375f,0.056099981026350664858881600594031624495983123779296875f,-0.44519997547864942344375549510004930198192596435546875f,-0.23539999413341217149309159140102565288543701171875f,0.219500005461275460305614615208469331264495849609375f,0.75090001094937353087743758806027472019195556640625f,-0.81999999329447736950982061898685060441493988037109375f,0.015399958053976037231080908895819447934627532958984375f,0.08080000020638100632908162879175506532192230224609375f,-0.1541999888412652897073940039263106882572174072265625f,-0.033799997892975774949064771135454066097736358642578125f,-0.179800018188357224602214046171866357326507568359375f,-0.1669999897032976576838336768560111522674560546875f,-0.715100008302926415382216873695142567157745361328125f,0.11870001246333128364085496286861598491668701171875f,0.137600014598667830245659615684417076408863067626953125f,0.0695000193007291600100217010549386031925678253173828125f,-0.139199987220763876649698431720025837421417236328125f,0.5297999938622115490005626270431093871593475341796875f,0.6188000146567820269893900331226177513599395751953125f,-0.7008999840900302924495690604089759290218353271484375f,-0.533799986195564457602813490666449069976806640625f,-0.2148000133082270668438695793156512081623077392578125f,-0.4180000020265577376221699523739516735076904296875f,-0.2708999808490275551520198860089294612407684326171875f,-0.2753999812066558749990008436725474894046783447265625f,-0.07090002883076673700912806452834047377109527587890625f,-0.0077999872088430066696673748083412647247314453125f,0.325300012987852105172947858591214753687381744384765625f,-0.03539998094588525656689625975559465587139129638671875f,0.99089999805986861414197619524202309548854827880859375f,0.53100001746416136683137665386311709880828857421875f,0.561899998313188486775970886810682713985443115234375f,0.4671999922275542616034726961515843868255615234375f,-0.1553000027060509324883241788484156131744384765625f,0.2753999926805494879999969271011650562286376953125f,0.25999999795109030031881047761999070644378662109375f,0.14019999611675759698670162833877839148044586181640625f,-1.0603000354513529845945640772697515785694122314453125f,-0.31280000658035245209021013579331338405609130859375f,1.092500026747584396247248150757513940334320068359375f,0.069999983012676292304377057007513940334320068359375f,-1.1378000014394522576566259886021725833415985107421875f,0.8826000104822213643274153582751750946044921875f,0.5839000047057865128152798206428997218608856201171875f,1.0729000098541383589889619543100707232952117919921875f,0.7043000068858262796567260011215694248676300048828125f,0.50810005451142881138792972706141881644725799560546875f,0.7301999736905100402140078585944138467311859130859375f,0.8438999777346847253056694171391427516937255859375f,0.34729998056292554764468150096945464611053466796875f,1.444299987924098527258820467977784574031829833984375f,-0.32059999155402196802100434069870971143245697021484375f,0.09070000350773366193379843025468289852142333984375f,0.1698999870777132270571740946252248249948024749755859375f,0.2344000021874903527674405268044210970401763916015625f,-0.74659998260587467910909253987483680248260498046875f,-0.896500016417353773334752986556850373744964599609375f,-0.03490000076815480412051329039968550205230712890625f,-0.182800021239369669956431607715785503387451171875f,0.339900008088350080726058877189643681049346923828125f,-0.264499978601932195232393496553413569927215576171875f,0.60959997900724427477570088740321807563304901123046875f,-0.14109996467232743810882311663590371608734130859375f,-0.4274000002279876309074779783259145915508270263671875f,-0.863300028984248957186764528159983456134796142578125f,0.1846000038549302502133286907337605953216552734375f,-0.27059997166097116139127365386229939758777618408203125f,-0.73260000301301475555959541452466510236263275146484375f,-0.0518000153422359943533592741005122661590576171875f,-0.271499988086521704122588971586083061993122100830078125f,-0.529200009274482408017092893715016543865203857421875f,-1.4304999700635672166271206151577644050121307373046875f,-0.782399980223178914684467599727213382720947265625f,-0.19200004092603928373961252873414196074008941650390625f,-0.0377999958515164991013079998083412647247314453125f,9.9992917478086695837191655300557613372802734375e-05f,-0.026999985009431792803979988093487918376922607421875f,0.4932000007137655739342108063283376395702362060546875f,-0.130399993574619177394424696103669703006744384765625f,1.246099983820319057059577971813268959522247314453125f,0.698999992370605838232222595252096652984619140625f,-1.20759997868686941302485138294287025928497314453125f,-0.61219999014139148130198009312152862548828125f,0.33209998243451099853018604335375130176544189453125f,-0.3674000425100327760219443007372319698333740234375f,-0.6434000089749696282837021499290131032466888427734375f,-0.0129999950230119676319873178726993501186370849609375f,0.92670003892779362342224658277700655162334442138671875f,0.0507999871492383903159861802123486995697021484375f,-0.6087000111222267140220765213598497211933135986328125f,-0.0332000074565410585591962444595992565155029296875f,0.304500004567205717620481664198450744152069091796875f,0.50440001515150090227734835934825241565704345703125f,-0.5824999755248423394249357443186454474925994873046875f,-0.018999963164329614073722041212022304534912109375f,0.8245999895498157261641836157650686800479888916015625f,0.214000017106533046984395696199499070644378662109375f,0.6833000298410656370151627925224602222442626953125f,-0.063800016891956801146079669706523418426513671875f,-0.153600004681945012663391025853343307971954345703125f,0.690199988740683334498271506163291633129119873046875f,-0.4065999920308589477002669809735380113124847412109375f,-0.14710001164078700952586586936376988887786865234375f,-0.680999990753829553824516551685519516468048095703125f,-1.1072999956876039551190160636906512081623077392578125f,-0.0298000039204955324834145358181558549404144287109375f,0.481799998280405983308583017787896096706390380859375f,0.269399996301531718945199145309743471443653106689453125f,0.333700026172399244472899226821027696132659912109375f,0.920999994106590502696008115890435874462127685546875f,0.81690003124475563112127929343841969966888427734375f,-0.0470000038221478622091353827272541821002960205078125f,1.38709999599456779151296359486877918243408203125f,-0.7495000103414055647732539000571705400943756103515625f,-1.178400039634109219832680537365376949310302734375f,0.68259997801631688840728884315467439591884613037109375f,1.7152000221192833517847020630142651498317718505859375f,0.084400003752112429111775782075710594654083251953125f,-0.2278000078469515354839813880971632897853851318359375f,-0.041199998828768735759098262860788963735103607177734375f,-0.6679999931603664009571730275638401508331298828125f,-0.1566999937400221920569265421363525092601776123046875f,-1.18080001069307360950233487528748810291290283203125f,0.6103000065803525497898363028070889413356781005859375f,1.0761000308334833786005901856697164475917816162109375f,0.52459999510049815096834890937316231429576873779296875f,1.28890001223087313775295115192420780658721923828125f,-0.452099989512562583371391156106255948543548583984375f,-0.89110005854964224880632173153571784496307373046875f,-1.022200001652538770002820456284098327159881591796875f,-0.515599999600648839503946874174289405345916748046875f,0.37040000422894958642672236237558536231517791748046875f,0.0249999964237215266393832280300557613372802734375f,-1.0862000115320087356707290382473729550838470458984375f,-0.830400019800663358182646334171295166015625f,0.82500000154599562041113358645816333591938018798828125f,0.66400002221018095127647029585205018520355224609375f,1.056800009139627238852199297980405390262603759765625f,0.720800009500980110033196979202330112457275390625f,-1.1564000504150986348150809135404415428638458251953125f,-0.17160001512765887810019194148480892181396484375f,0.7726000071108340971903771787765435874462127685546875f,-0.065600010664760788348104370015789754688739776611328125f,0.3268999969162045271531269463594071567058563232421875f,0.49160000141859061528748497948981821537017822265625f,0.6401000423148277551632645554491318762302398681640625f,0.1641999949693673244155434076674282550811767578125f,-0.4093000091210010626951998347067274153232574462890625f,0.00460000731945076068996058893389999866485595703125f,-0.861000001706182960248270319425500929355621337890625f,-0.25579999832510935675600194372236728668212890625f,0.1954999944269656120265921117606922052800655364990234375f,-0.0351999979794024309143196660443209111690521240234375f,0.8030000063106415009173133512376807630062103271484375f,0.12719998358488115997033673920668661594390869140625f,-0.455299995031952564605859379298635758459568023681640625f,0.38159999486207940577742192544974386692047119140625f,1.231100015746056985932455063448287546634674072265625f,1.0440999937444919165585588416433893144130706787109375f,0.045599997067451436549845311674289405345916748046875f,0.707100019910931631983430634136311709880828857421875f,-0.72029997208416485232618242662283591926097869873046875f,-0.516100018763541612543122028000652790069580078125f,-0.8333000119969244945394848400610499083995819091796875f,-0.97570003414824701959418007390922866761684417724609375f,0.73980000956431002379076744546182453632354736328125f,0.1042000548906631962609026231802999973297119140625f,0.8795999786347152848975383676588535308837890625f,-0.1417999514162548990725554176606237888336181640625f,-0.911399996510148024952968626166693866252899169921875f,-0.5392000133909287118427755558514036238193511962890625f,-0.5436999778181317299896591066499240696430206298828125f,-1.3234999838247887282705050893127918243408203125f,-0.3587999985635279553974896771251223981380462646484375f,-0.547700039553642881173800560645759105682373046875f,0.9286999808728697569648602438974194228649139404296875f,1.051799967509508437757403953582979738712310791015625f,0.4260000001415609194310718521592207252979278564453125f,0.3821000133544207866265196571475826203823089599609375f,-0.074800001089274925636374291570973582565784454345703125f,0.294600001657009169520051727886311709880828857421875f,0.33929998878091571778981005991226993501186370849609375f,1.0122999975502493530399306109757162630558013916015625f,-0.042099997521936938227327118511311709880828857421875f,-1.171699956226349481624993131845258176326751708984375f,-0.3705999890208244718081687096855603158473968505859375f,-1.2952000154137603971093994914554059505462646484375f,-0.11470000953152748035535068993340246379375457763671875f,0.075199988032877362886807759423390962183475494384765625f,1.03259996868446446427469709306024014949798583984375f,0.6371999829888341793093786691315472126007080078125f,-0.75799999424070119147245350177399814128875732421875f,-0.26820000581741343381736442097462713718414306640625f,0.48050000564008943371163695701397955417633056640625f,-0.0920000185370444256705013685859739780426025390625f,0.0824000172898176064251174466335214674472808837890625f,0.4157999966859815543784861802123486995697021484375f,0.5754999663010238464266876690089702606201171875f,0.5851999802470206901716665015555918216705322265625f,-0.122999995209276402619025247986428439617156982421875f,0.3397999795198440864396616234444081783294677734375f,-0.9662999831780789694590794169926084578037261962890625f,-0.354799987566471575206605848507024347782135009765625f,0.8506000081688167568927383399568498134613037109375f,0.1429999974071975987754967718501575291156768798828125f,-0.5343000183597206298458104356541298329830169677734375f,-0.235400009667873888474787236191332340240478515625f,-0.783699991676211293878395736101083457469940185546875f,-0.4173999909520151874176008277572691440582275390625f,0.79409997847080260324759137802175246179103851318359375f,0.35179998524188960118408431299030780792236328125f,0.7340000263452532980323894662433303892612457275390625f,0.35819999508857680581286331289447844028472900390625f,-1.668600028114020528136052234913222491741180419921875f,-0.312199996399879609043637174181640148162841796875f,1.1906000111117960305051610703230835497379302978515625f,0.118900020128488659310050934436731040477752685546875f,-0.96799997043609664615360088646411895751953125f,-0.5511000277042388262316308100707828998565673828125f,0.16400004422664693493061349727213382720947265625f,-0.76189997655749319704909794381819665431976318359375f,-0.21610000982284560677726403810083866119384765625f,-0.37729999695420257666000907192938029766082763671875f,1.1862000242352479517649044282734394073486328125f,-0.066799967211485178353314040577970445156097412109375f,0.366399993157386916209361515939235687255859375f,-0.00160000588893893791464506648480892181396484375f,-0.55579996867179914943335461430251598358154296875f,0.413599985682964632616176459123380482196807861328125f,-0.747599984014033935864063096232712268829345703125f,0.552699990081787273510371960583142936229705810546875f,-1.316499951243400801104144193232059478759765625f,-0.39749999806284908032694147550500929355621337890625f,0.09250000521540613362958538345992565155029296875f,1.0724000021278856653594857561984099447727203369140625f,0.0847999448001388600459904409945011138916015625f,1.5044999753236771855569031686172820627689361572265625f,0.10919996218681404798189760185778141021728515625f,-0.972200019645690804992455014144070446491241455078125f,0.39970004378557177915354259312152862548828125f,-0.283099951916933267881404390209354460239410400390625f,0.1906999819755554881339776329696178436279296875f,0.95709999927282307652376402984373271465301513671875f,0.15639997482895751090836711227893829345703125f,0.1095000182390213350203111986047588288784027099609375f,0.98669997723698532610114853014238178730010986328125f,0.3256000061571597825604840181767940521240234375f,0.15509998677074943174147847457788884639739990234375f,-0.281200003820657684627803973853588104248046875f,0.43179994992613845994355870061554014682769775390625f,-0.07540001458525669164600913063623011112213134765625f,0.1565000147223465365442507390980608761310577392578125f,-0.5121999834358692549329816756653599441051483154296875f,-0.470699984508752944378784377477131783962249755859375f,0.043800018382072369860225080628879368305206298828125f,0.07980002556443199779323549591936171054840087890625f,0.08220000846981978526173406862653791904449462890625f,-0.495400015926360737239519949071109294891357421875f,-0.710200011837482492182971327565610408782958984375f,0.110199946421385419625948998145759105682373046875f,0.791800012212991788373983581550419330596923828125f,1.237700049947202618483288460993207991123199462890625f,-0.0608000059619545341060842247316031716763973236083984375f,-0.6169000244833531265697956769145093858242034912109375f,-0.193800008845329330142703838646411895751953125f,-0.728100005933642879796252600499428808689117431640625f,0.242599999213218620042198381270281970500946044921875f,0.2872000550359488091345383509178645908832550048828125f,-0.7859999939203259344822072307579219341278076171875f,0.0087999743491409265772063008625991642475128173828125f,-0.2447999906957150084707564019481651484966278076171875f,-1.07560000239461661664108760305680334568023681640625f,-0.540400020396709646064437038148753345012664794921875f,-0.28370001976490044626189046539366245269775390625f,0.2040999824941156504110040259547531604766845703125f,0.4519999805390835234675250831060111522674560546875f,0.64819996886253361534357964046648703515529632568359375f,-0.78149997746944421805892488919198513031005859375f,-1.109199988710880102615874420735053718090057373046875f,-0.176900006806850029761335463263094425201416015625f,-1.0035999713778498421135054741171188652515411376953125f,0.5601000065147871964654768817126750946044921875f,-0.5550999753713610207483952763141132891178131103515625f,0.839599981987475985079072415828704833984375f,0.520200000959634589747793143033050000667572021484375f,1.089099959397316297327051870524883270263671875f,0.61119996179938329561309728887863457202911376953125f,-1.1077999720096585178907844237983226776123046875f,-0.08480000023245803930649344692938029766082763671875f,0.272099979305266970186494290828704833984375f,-0.06580000557079890288747492377297021448612213134765625f,0.86010000869408287371697952039539813995361328125f,-0.28009999707117672063105828783591277897357940673828125f,-0.2290999959051607248738946509547531604766845703125f,-0.17029998705983173667988239685655571520328521728515625f,-0.3166000164687634566007545799948275089263916015625f,-0.4054999857097862925314757376327179372310638427734375f,-0.2987999379158026158620486967265605926513671875f,-1.1457999735891821746491814337787218391895294189453125f,0.49679999857842904731342059676535427570343017578125f,0.02839999466985465570445512639707885682582855224609375f,-0.8512000009894364893625606782734394073486328125f,0.220299982142448502742126947850920259952545166015625f,-0.3289999769851570388112804721458815038204193115234375f,-0.2454000115677714977113765826288727112114429473876953125f,0.2133999823577699128662743532913736999034881591796875f,-0.3794000012934208854176176828332245349884033203125f,-0.10060000302791660686807517777197062969207763671875f,-0.2724000028729438582786315237171947956085205078125f,-0.2991000149786469108903474989347159862518310546875f,0.74959999197125437575550677138380706310272216796875f,0.55540002620816242284718100563623011112213134765625f,-0.54080000488162038863038105773739516735076904296875f,0.38079994661808047595741300028748810291290283203125f,0.225400019800662931857004878111183643341064453125f,1.020000048428773897768451206502504646778106689453125f,0.054000006526708776988243698724545538425445556640625f,-1.045500014096498286875203120871447026729583740234375f,0.058799987238645545772897094138897955417633056640625f,0.2629999935328957150204587378539144992828369140625f,0.439999982863664484966648160479962825775146484375f,0.0576999548405405793261024882667697966098785400390625f,0.2996000032961367853800993543700315058231353759765625f,0.93100000662356652281204105747747235000133514404296875f,-0.2289999937862157164403242859407328069210052490234375f,-0.570100008489191178995270092855207622051239013671875f,-0.0152000060260295555281118140555918216705322265625f,0.5682999658405785137205157298012636601924896240234375f,0.1601999945521355162014742745668627321720123291015625f,0.91689998147487639101882450631819665431976318359375f,-0.43739998424649240149619799922220408916473388671875f,0.52459998601079060875918003148399293422698974609375f,-0.15209999955222019973888336608069948852062225341796875f,-0.53749999480321974942853557877242565155029296875f,0.14030003342106933672539526014588773250579833984375f,0.0424999892711639404296875f,-0.056000022381544223293303730315528810024261474609375f,-0.5943999844551086653154925443232059478759765625f,-0.162400026416778420212949640699662268161773681640625f,-0.1741000130414960267444257624447345733642578125f,0.24599998140335088692154386080801486968994140625f,-0.09689997491836521703589824028313159942626953125f,0.3789000429272650904977126629091799259185791015625f,1.005799965095519610258634202182292938232421875f,-0.23780002014040935165439805132336914539337158203125f,1.0043000183224677357429754920303821563720703125f,0.709299994480609985458841038052923977375030517578125f,-0.5402999991297718906935187987983226776123046875f,0.746999968767166233618581827613525092601776123046875f,0.11809996845722281477719661779701709747314453125f,1.0377999852716921491690982293221168220043182373046875f,-0.76140001781880872755436939769424498081207275390625f,-0.13269999675005672568062209393247030675411224365234375f,-0.21089998390376596404394149431027472019195556640625f,0.86499998241662989784117598901502788066864013671875f,-0.761899992501735567884679767303168773651123046875f,0.265599985444545882984357376699335873126983642578125f,-0.43739999646544447386986576020717620849609375f,-0.1998999984025957576960763617535121738910675048828125f,0.132900011456012379085223074071109294891357421875f,-0.467799980950355465125767295830883085727691650390625f,1.0831999786972996702161253779195249080657958984375f,0.1386000074386597447073654620908200740814208984375f,-0.29280001693665980155856232158839702606201171875f,-0.21079998163580881964662694372236728668212890625f,0.867500008046627524294081013067625463008880615234375f,0.462599999809265138850378207280300557613372802734375f,0.1895999889910218172417444293387234210968017578125f,0.275399998193979200777903315611183643341064453125f,-0.11439999116063059858561246073804795742034912109375f,-0.27879999588131898491383253713138401508331298828125f,0.445999987736344305488955797045491635799407958984375f,-0.617200012493133609581263954169116914272308349609375f,-0.92660001497864730168885216698981821537017822265625f,0.34120000709891318280142513685859739780426025390625f,0.239199987667799263135748333297669887542724609375f,-0.4445999952495096607663072063587605953216552734375f,0.6570999984532595039610214371350593864917755126953125f,0.108200017216801558372907265948015265166759490966796875f,-0.731299993161857120327340453513897955417633056640625f,0.0368000022292136463875067420303821563720703125f,0.019900017774105105416992955724708735942840576171875f,0.0627999903380871415947694913484156131744384765625f,0.829599982732533991480750046321190893650054931640625f,-0.019600002253055581746821189881302416324615478515625f,-0.084499983444810300436955685654538683593273162841796875f,-0.227599996605515497716254458282492123544216156005859375f,0.046299999569356486972537823021411895751953125f,0.3054000086247921075255362666212022304534912109375f,-1.3492999923944477558279686491005122661590576171875f,0.400700002092122975483334812452085316181182861328125f,0.0734999991357323523288869182579219341278076171875f,0.90830002335906001587062519320170395076274871826171875f,-1.0966999812603006603239919058978557586669921875f,0.25120000903606420905589402536861598491668701171875f,0.3419999791979790870755095966160297393798828125f,-0.015800029021501682535699728759936988353729248046875f,-0.40770000617504109641231480054557323455810546875f,0.4280000182688237231332095689140260219573974609375f,-0.3281999865949156713895717984996736049652099609375f,-1.19600000762939462362055564881302416324615478515625f,-0.27239997903108559285101364366710186004638671875f,0.66250000059604641222676946199499070644378662109375f,0.15329998059272753607729100622236728668212890625f,-0.055199991571903250786590433563105762004852294921875f,-1.0862999617576605260182986967265605926513671875f,-0.7382999659523370450386892116512171924114227294921875f,0.95719997684210511579294688999652862548828125f,0.6864000116720794242297642995254136621952056884765625f,-0.397200012195110474522152799181640148162841796875f,0.668599995964765714262512119603343307971954345703125f,-0.441099996709823471974232234060764312744140625f,-0.571399992859363603514566420926712453365325927734375f,0.09469999908208848182766814716160297393798828125f,-0.22619999499171983270429109325050376355648040771484375f,0.180400004042684969363108393736183643341064453125f,-0.762399983091652355682299457839690148830413818359375f,0.712099987947940604726682067848742008209228515625f,0.0968000306904313045919252544990740716457366943359375f,0.20330001367330530204213800971047021448612213134765625f,0.71680000804066656971968996003852225840091705322265625f,0.57090002801120309783300399431027472019195556640625f,-0.5404000007271765326777313021011650562286376953125f,-0.12750003188848513246966831502504646778106689453125f,-0.3749999982118605856840076739899814128875732421875f,1.14629999059140619266372596030123531818389892578125f,-0.50879998500347145551359062665142118930816650390625f,-0.855800008159875336133382006664760410785675048828125f,0.17159998443126678324688327847979962825775146484375f,0.3680999933421613690143203712068498134613037109375f,-0.73519998725056634469865457504056394100189208984375f,-1.744199968743324280495698985760100185871124267578125f,-0.057800016117095953660509621840901672840118408203125f,-0.0828000304594636860056056093526422046124935150146484375f,-0.35280000870376826416219273596652783453464508056640625f,0.786299990572780327369173392071388661861419677734375f,0.4406000390887261364269988916930742561817169189453125f,-0.117799962621926823658213834278285503387451171875f,-0.4499999973177910561616954510100185871124267578125f,-1.09440001559853516965858943876810371875762939453125f,-0.3618000163853167805427801795303821563720703125f,1.10819997862279340239410885260440409183502197265625f,0.424600000688433620865680495626293122768402099609375f,-0.892899973942339553190095102763734757900238037109375f,-0.1220000011026858288687435560859739780426025390625f,1.235599977099895330212575572659261524677276611328125f,-0.439599992716312559792868341901339590549468994140625f,-1.80889998719692268736025653197430074214935302734375f,0.9872999900251626659297699006856419146060943603515625f,0.36079997083246695410707616247236728668212890625f,0.64230001491010202396836348270880989730358123779296875f,-0.343999969720840681475237943232059478759765625f,0.24300003004074088863717406638897955417633056640625f,-0.6290999863684181292455832590349018573760986328125f,-0.66109998672604586289480721461586654186248779296875f,0.488000000536441547183130751363933086395263671875f,-0.388400034493207702013251036987639963626861572265625f,-0.48419999318122819431664538569748401641845703125f,-1.5411999625444412487240697373636066913604736328125f,-0.02829995734691692632623016834259033203125f,0.12549998921155935960314309340901672840118408203125f,-0.53919999288320497043969226069748401641845703125f,-0.22140000740662213463139096347731538116931915283203125f,-0.88589997986182567046853364445269107818603515625f,-0.61219996639266638371879025726229883730411529541015625f,1.287599983865021613382850773632526397705078125f,0.2501000156044959066292676652665250003337860107421875f,0.0250000059604644775390625f,0.787300009322166705061363245476968586444854736328125f,-1.2542999810695647511238348670303821563720703125f,-0.98260000249147427808793509029783308506011962890625f,0.49619998213946825416087449411861598491668701171875f,0.847699996191263238642932265065610408782958984375f,-0.90309998037815120142113300971686840057373046875f,-1.4588999588847162414140257169492542743682861328125f,0.591599990987777601958441664464771747589111328125f,0.94899999013543112624802233767695724964141845703125f,-0.48110003142952917354335795607767067849636077880859375f,-0.57180001675784597470197923030355013906955718994140625f,-0.3321000227421524453319534586626105010509490966796875f,-0.3176000037208199788807405639090575277805328369140625f,0.06320000859275476390308767804526723921298980712890625f,-0.23680000021755687189539685277850367128849029541015625f,0.6038999688871211635188274158281274139881134033203125f,-0.110399999088048872408762690611183643341064453125f,-1.929400002932548208178786808275617659091949462890625f,-0.14660000097155556630923456395976245403289794921875f,1.14069999568462332462104313890449702739715576171875f,0.26299999785423278098051014239899814128875732421875f,0.512800043535232674685175879858434200286865234375f,0.2043999930977822288014067453332245349884033203125f,-0.42560005622506213995848156628198921680450439453125f,-0.03939998937249189481235589482821524143218994140625f,0.2544999900758266875300250831060111522674560546875f,0.3950000052154063467924061114899814128875732421875f,0.4623999556362630158901083632372319698333740234375f,-0.3314000171482565093583616544492542743682861328125f,0.7274999774992456647026983773685060441493988037109375f,0.5654000083267687326582517925999127328395843505859375f,1.01930004007816332745051113306544721126556396484375f,-0.38240000510811800182864317321218550205230712890625f,-0.724699971812964083284214211744256317615509033203125f,0.157999993234872793124168310896493494510650634765625f,0.331799984049796936602660935022868216037750244140625f,-0.150200004386901841968438020558096468448638916015625f,-0.587600020819902812974078187835402786731719970703125f,-0.3233000081598758068679444477311335504055023193359375f,-0.73269998620748477691222433350048959255218505859375f,-0.5676000055648386943829564188490621745586395263671875f,-0.4492999864153564004709551227279007434844970703125f,0.0149999765120447658972580029512755572795867919921875f,0.058300020192563639653826612629927694797515869140625f,-1.29810000602677444447863308596424758434295654296875f,0.532700017201900521968127577565610408782958984375f,1.392600027376413418522815845790319144725799560546875f,-1.284299981665611767311929725110530853271484375f,-0.65250000253319750953551192651502788066864013671875f,0.11759999206066140686743892729282379150390625f,0.71189999503493339716442278586328029632568359375f,1.004199995565414837983553297817707061767578125f,1.3167000131487842651267783367075026035308837890625f,-0.8937000114202504619242972694337368011474609375f,1.1687999937951563111937502981163561344146728515625f,-0.7083000247001649540834478102624416351318359375f,0.89410002183318137980450046597979962825775146484375f,-1.608500015974044572431012056767940521240234375f,-0.07200000274181395099049041164107620716094970703125f,-0.1177000402987005145405419170856475830078125f,-0.8513999824285505280130337268928997218608856201171875f,0.19030001969337462242037872783839702606201171875f,0.368899987196922296561751863919198513031005859375f,0.456699988710880688813631422817707061767578125f,-0.1793000078916549000496161170303821563720703125f,0.018400008416175950287652085535228252410888671875f,-0.063500003159046247702690379810519516468048095703125f,0.258899990177154482040577931911684572696685791015625f,-0.266199993538856372055079191341064870357513427734375f,-0.26999999523162809822451890795491635799407958984375f,-0.14799999949336051230375232989899814128875732421875f,0.28980000310093190218907466260134242475032806396484375f,-0.21140000038444994867603554666857235133647918701171875f,-1.13499996522441559676508404663763940334320068359375f,0.4260000201463700619797236868180334568023681640625f,0.82070002086758597759086342193768359720706939697265625f,0.09900000682473175661613140619010664522647857666015625f,-0.74459998429715612555668258210062049329280853271484375f,-0.01220000280737865150371135314344428479671478271484375f,-0.24019999209344344848204855225048959255218505859375f,0.61979999635815641312319712596945464611053466796875f,0.80390002191662801589444597993860952556133270263671875f,-0.57780000702738754103648943782900460064411163330078125f,-1.032300014276802624380025008576922118663787841796875f,0.33560000634342446179658736582496203482151031494140625f,0.8998000007726252302830971530056558549404144287109375f,0.3190000329017639302264797152020037174224853515625f,0.03970000921487848444257906521670520305633544921875f,0.24699999827146523756482565659098327159881591796875f,1.029199983346462232702833716757595539093017578125f,0.61380000973939896624642642564140260219573974609375f,0.68400001770257912170336567214690148830413818359375f,-0.57260000249147413597938793827779591083526611328125f,0.111799984049797185292618451057933270931243896484375f,0.244800010961294223221784704946912825107574462890625f,0.584600011715293188530040424666367471218109130859375f,-0.38940001142621039065261356881819665431976318359375f,-0.09319997035265004114990006200969219207763671875f,-0.577599998170137407527136019780300557613372802734375f,0.2189000091016293225720801274292171001434326171875f,-1.354699975389241917156368799624033272266387939453125f,0.861000006325542965868180544930510222911834716796875f,0.7895000214613971944999093466321937739849090576171875f,0.64459998943805629778580623678863048553466796875f,0.013100011894106700083284522406756877899169921875f,0.72219998365938664619534392841160297393798828125f,0.38989998648166668271386470223660580813884735107421875f,-0.53419998029172433007261133752763271331787109375f,0.06219998365938683715370416393852792680263519287109375f,-0.21509998848438272034400142729282379150390625f,-0.593899992039799595744398175156675279140472412109375f,0.1358000323742629689149907790124416351318359375f,0.2522999971218407022632845837506465613842010498046875f,-0.02749999815598158647844684310257434844970703125f,0.80630003620758616822428166415193118155002593994140625f,0.9809999742507926612233859486877918243408203125f,1.50199999901652336831148204510100185871124267578125f,-0.23580002112388598334291600622236728668212890625f,0.8793000163853168515970537555404007434844970703125f,0.47939996776580784398902324028313159942626953125f,0.270099998170137300945725655765272676944732666015625f,-0.566799977195262982831991394050419330596923828125f,-0.1758999963462353210985611440264619886875152587890625f,0.4075999934017655590423601097427308559417724609375f,-0.19479999591112140411297559694503434002399444580078125f,0.825799978543817925213943453854881227016448974609375f,-0.4989999989271165503623706172220408916473388671875f,-0.70660000126957900334900841698981821537017822265625f,-0.5295999997198581610291512333787977695465087890625f,0.04029999197721423342954949475824832916259765625f,-0.67579999907016752302979512023739516735076904296875f,-0.1470000427886837546243015140134957619011402130126953125f,0.0845999884322285733428969933811458759009838104248046875f,0.892999996978789400969844791688956320285797119140625f,-0.4378000027060509058429715878446586430072784423828125f,-0.7888000094413758489508836646564304828643798828125f,0.33800001379847532945177590590901672840118408203125f,0.6876999814391144827396828986820764839649200439453125f,-0.5007999944508074730009639097261242568492889404296875f,0.6454000230789187497748571331612765789031982421875f,0.1425999887824058731666809762828052043914794921875f,-0.4316999957144262989316985112964175641536712646484375f,0.5050000195205213149307610365212894976139068603515625f,-0.2806999911397694003056813016883097589015960693359375f,0.0099999962747097104198701345012523233890533447265625f,0.39429996031969782865900242541101761162281036376953125f,0.39140000954866405180609945091418921947479248046875f,1.9610000281184916115506666756118647754192352294921875f,-0.166200010302662792849304196352022700011730194091796875f,-0.671700023616850661056787430425174534320831298828125f,0.1738000000536441547183130751363933086395263671875f,0.6381999997079372821673359794658608734607696533203125f,0.032200005713105273041918508170056156814098358154296875f,0.4468999885156750284664894934394396841526031494140625f,-0.8279999829530717558867536354227922856807708740234375f,0.40359999492168441292960778810083866119384765625f,0.259600028032064500393971684388816356658935546875f,-0.2120999912261964936988078989088535308837890625f,-0.025200017201900681840243123588152229785919189453125f,-0.63759997298717507874243892729282379150390625f,-0.25540001218244412495295136977802030742168426513671875f,-0.46469999716356369390268810093402862548828125f,-0.62159999439641833163250339566729962825775146484375f,0.43240000838637371316508506424725055694580078125f,0.8165999787688253963580109484610147774219512939453125f,0.064499994695186302351430640555918216705322265625f,-0.1512999812781810060613452151301316916942596435546875f,-0.460299970370531497110277996398508548736572265625f,-1.048900009995698656695140016381628811359405517578125f,1.2675999974250800050867837853729724884033203125f,0.820100003981590219837016775272786617279052734375f,0.95849999168515154224223806522786617279052734375f,1.460099997574090924246092981775291264057159423828125f,-0.58460000039041037922515897662378847599029541015625f,1.075100001783668712818098356365226209163665771484375f,-0.4122000238180163478318718262016773223876953125f,-0.6617000158682462274128965873387642204761505126953125f,0.8812000130966310962321585975587368011474609375f,0.7803999968156218880466212794999592006206512451171875f,0.2315999857723713972745827049948275089263916015625f,-0.00990000372976063136576385659282095730304718017578125f,0.305600005821883513590364600531756877899169921875f,-1.004899964241683818499950575642287731170654296875f,-0.05719997956156674945304985158145427703857421875f,0.8580000103712084325024989084340631961822509765625f,0.62070001460909818291611372842453420162200927734375f,0.10160000484585740565535161294974386692047119140625f,-0.17259999936223113792266303789801895618438720703125f,0.16699998709559427112480989308096468448638916015625f,0.3502000286385411609302309443592093884944915771484375f,-0.3789999907687306990311526533332653343677520751953125f,-1.0142999895818529676461139388266019523143768310546875f,0.89140000016093257073634958942420780658721923828125f,0.7634000135943297404850227394490502774715423583984375f,-0.1169999975636601685646809301033499650657176971435546875f,-0.1444999969489872970740407254197634756565093994140625f,0.26280000181198115427605443983338773250579833984375f,-0.3577999984592195037436113125295378267765045166015625f,-0.2388000020653009425330992598901502788066864013671875f,0.15219999375492354598549127331352792680263519287109375f,0.12360000118017200332332095058518461883068084716796875f,-0.053800007727742293894834801903925836086273193359375f,-0.117800009113550263606384760350920259952545166015625f,-0.077599999883771086928163640550337731838226318359375f,-0.493200009691715290927049863967113196849822998046875f,-0.78530001567006113560864832834340631961822509765625f,-0.07540001100897786301402447861619293689727783203125f,1.29839994374513612029886644450016319751739501953125f,-0.55240000197887439981059287674725055694580078125f,-1.418100000196694576715117364074103534221649169921875f,0.27260000293850905617176749728969298303127288818359375f,-0.0544999867603179477537622688032570295035839080810546875f,-0.3696000050842760398239761343575082719326019287109375f,0.4514000020980830640837666578590869903564453125f,1.08320005499124505377039895392954349517822265625f,0.7994999526739121620266814716160297393798828125f,-0.22620001797676092536448777536861598491668701171875f,-0.52310001554489105046741315163671970367431640625f,-0.56909998668506756036578053681296296417713165283203125f,0.38570000538900472975001321174204349517822265625f,0.260799964480847190628765019937418401241302490234375f,0.128899981236457961131236515939235687255859375f,0.16550001288577920188771486209589056670665740966796875f,0.380499998915940551569292438216507434844970703125f,-0.2915000223405659340869533480145037174224853515625f,0.443799990069866368003204115666449069976806640625f,0.2898999984025956155875292097334749996662139892578125f,-0.23389999025166030577338460716418921947479248046875f,-0.110000002458691514828359458988416008651256561279296875f,0.3118999986857176054400042630732059478759765625f,-0.16490000607669352827855391296907328069210052490234375f,-0.1924000012338158427382950321771204471588134765625f,-0.36390000612139694613489382390980608761310577392578125f,-0.18559998723268478215686627663671970367431640625f,0.76899997150897991815554632921703159809112548828125f,-0.6652999916791912937696906737983226776123046875f,-1.595800013226270852584320891764946281909942626953125f,0.0422000041484835719529655762016773223876953125f,0.315499995544552991333375757676549255847930908203125f,-0.17300000061094777947801048867404460906982421875f,0.84129998401626959747545697609893977642059326171875f,0.71740002641677858008506518672220408916473388671875f,-0.24979999740123748352971233543939888477325439453125f,-0.1400000135600569795002456885413266718387603759765625f,0.1565999966502189533201772064785473048686981201171875f,0.9251000270783900081283945837640203535556793212890625f,0.4082000129699707713371026329696178436279296875f,-0.42399997076392192951743709272705018520355224609375f,-0.40559999781250954953293330618180334568023681640625f,-0.99399998223781604878013240522705018520355224609375f,-0.41499998152256001304749588598497211933135986328125f,-0.94279994772076614051314891185029409825801849365234375f,-0.30560000228285788548276968867867253720760345458984375f,-0.07380000862181201881639935891143977642059326171875f,-0.82780001969337480005606266786344349384307861328125f,-0.8461000271081928758576395921409130096435546875f,0.970600001835823178453210857696831226348876953125f,0.78990004310607897508589303470216691493988037109375f,-0.56780001075267794163892176584340631961822509765625f,0.95739993551969515550581490970216691493988037109375f,0.23559999915361373723499127663671970367431640625f,-1.108599976816773580168273838353343307971954345703125f,-0.45859999767839898510146667831577360630035400390625f,-0.044500011794269322962236401508562266826629638671875f,-0.86380000780224808210050468915142118930816650390625f,0.67759995778799098076206064433790743350982666015625f,0.21920003207325944316608001827262341976165771484375f,-0.8417999590158462552835771930404007434844970703125f,-0.65300002527236955529588158242404460906982421875f,-0.0996000329494481917436132789589464664459228515625f,-0.20039997748136517685679791611619293689727783203125f,0.5943999993562698591631487943232059478759765625f,-0.027800007474422461228868996840901672840118408203125f,0.05729999367594729164920863695442676544189453125f,-0.77249999009072778388684810124686919152736663818359375f,1.5591999884128568965024896897375583648681640625f,-0.76039997882246979088449734263122081756591796875f,0.3605000139474867637545685283839702606201171875f,-1.27149995601177234760825740522705018520355224609375f,0.0684000251054754926371970213949680328369140625f,-0.487899990519881310291339104878716170787811279296875f,0.0582000133424998011832940392196178436279296875f,-0.31299999137222755951626140813459642231464385986328125f,-0.0685999949216835602783248759806156158447265625f,0.0307999723970890926239007967524230480194091796875f,0.0641999836444853144712396897375583648681640625f,0.66099998334050180659460238530300557613372802734375f,-0.09039998135566662540441029705107212066650390625f,-1.12329999504685407174520150874741375446319580078125f,1.4437999486446386754323611967265605926513671875f,0.5897000019133091797129964106716215610504150390625f,-0.62609997338950673650970202288590371608734130859375f,-0.4244000078499319261737809938495047390460968017578125f,-0.772099975430965201894650817848742008209228515625f,-1.2686999617993832689677446978748776018619537353515625f,1.1970999918222418045843369327485561370849609375f,0.00649999654293065276533525320701301097869873046875f,-0.64319999091625135179128847084939479827880859375f,-1.390899998432397755010470064007677137851715087890625f,-0.56859995990395528764338450855575501918792724609375f,0.83580002693533916868773303576745092868804931640625f,1.0589000606596481457444269835832528769969940185546875f,0.1650000126659871835244075555237941443920135498046875f,-0.49730001133382295552820551165495999157428741455078125f,-0.12359999704509971873989115920267067849636077880859375f,-0.26110002207904958737572087557055056095123291015625f,-0.33100000405311558182575026876293122768402099609375f,0.99679995946288091346332294051535427570343017578125f,-0.34260000562071812879594290279783308506011962890625f,-1.1515999815255402172198273547110147774219512939453125f,0.2776000004053116310132054422865621745586395263671875f,0.21830000447183894873859344443189911544322967529296875f,-0.20699999722838402504976329510100185871124267578125f,-0.97839998636245706364888974349014461040496826171875f,0.46320000432729724337832521996460855007171630859375f,-0.791699949967861638100430354825221002101898193359375f,0.188999997138977082755673109204508364200592041015625f,0.6210999650448567610538930239272303879261016845703125f,0.9692000051021576467746854177676141262054443359375f,0.23030001432895719215565577542292885482311248779296875f,0.69139998689889903271676985241356305778026580810546875f,0.928499999508261453229351900517940521240234375f,0.67659999828934669352520359097979962825775146484375f,-0.420799966660142354957230281797819770872592926025390625f,0.554400008073449246381869670585729181766510009765625f,1.089100012333691136490188000607304275035858154296875f,-0.6497999930799007728410288109444081783294677734375f,-1.810599987679719191646654508076608180999755859375f,0.340299992275237883632144075818359851837158203125f,0.300299997639655202874564565718173980712890625f,0.2934000161647798421427069115452468395233154296875f,1.2523000091731546490336768329143524169921875f,0.95559998991489403241672562216990627348423004150390625f,-0.19349999332427980647253207280300557613372802734375f,0.5557999965183439972093992764712311327457427978515625f,-1.2047999532938007405391545034945011138916015625f,-1.73499999269843119265033237752504646778106689453125f,-0.0040999963521954896350507624447345733642578125f,-0.3757000275731086080810428029508329927921295166015625f,-0.02249999441206451677999211824499070644378662109375f,-1.02049999178200945237904306850396096706390380859375f,0.3631000113725662004071637056767940521240234375f,-0.253199974673986361040078918449580669403076171875f,0.441799984943866519415678340010344982147216796875f,1.5018999668717383233484952143044210970401763916015625f,-0.5231999985158442001420553424395620822906494140625f,0.3705999698728323910046356104430742561817169189453125f,0.1305999797821044694501324556767940521240234375f,1.318599986949563263038953664363361895084381103515625f,-0.0363000273972744480488472618162631988525390625f,0.04239999683797368579263320498284883797168731689453125f,0.568099975907802701158288982696831226348876953125f,1.088599998982250571799568206188268959522247314453125f,-0.0172000140205028628770378418266773223876953125f,-1.130399989961087925394167541526257991790771484375f,0.1323000230312345593119971454143524169921875f,-0.13049999979138338090933757484890520572662353515625f,0.7025999625563610351264287601225078105926513671875f,-0.059600006127357829655011300928890705108642578125f,-0.7388999839186676243940610220306552946567535400390625f,-0.1820000092983244943667386905872263014316558837890625f,-0.22430003456473368572687832056544721126556396484375f,-0.20420002208948151434242390678264200687408447265625f,-0.4470000335872175156737284851260483264923095703125f,0.044200002568960172766310279257595539093017578125f,1.67660003733038909246033654198981821537017822265625f,-0.016600006633996855498480726964771747589111328125f,-0.5805999920010567194594841566868126392364501953125f,-0.282600002491474100452251150272786617279052734375f,-0.895600020164251109378028559149242937564849853515625f,-0.4554000108599662866026847041212022304534912109375f,-0.9852000239074232279534726330894045531749725341796875f,0.6944000021874903172403037387994118034839630126953125f,0.45690002744495838982174973352812230587005615234375f,-0.602400018146634241844594725989736616611480712890625f,0.30149995448440325418459906359203159809112548828125f,-0.149399991160631184783369462820701301097869873046875f,-0.19499999687075586507489788345992565155029296875f,-0.528799998563528106387821026146411895751953125f,-0.61880001305490728302771685775951482355594635009765625f,-0.0008000184789299369381154747316031716763973236083984375f,0.00759996976479870856024945169338025152683258056640625f,0.03559999989867212999428147668368183076381683349609375f,0.0521999918550253294569074569153599441051483154296875f,-0.0355999937891959827851451336755417287349700927734375f,0.246600006484985811994192772544920444488525390625f,-0.2864999743402005805137378047220408916473388671875f,0.8549999439716344795670011080801486968994140625f,0.5188999614179135022595801274292171001434326171875f,-1.0443999762594700086992816068232059478759765625f,-0.2621000152170658292760663243825547397136688232421875f,-1.170600001835822467910475097596645355224609375f,0.625400014585256514010325190611183643341064453125f,-1.01510000755786933268609573133289813995361328125f,-1.37090001810193040654439755599014461040496826171875f,0.824100018405914624963770620524883270263671875f,-0.421200011867284729305538348853588104248046875f,0.349400020515918896535367821343243122100830078125f,-0.753100006902217788962161648669280111789703369140625f,-0.6197999753475187389994971454143524169921875f,0.35880001093149171964569177362136542797088623046875f,-0.0431000446021556626874371431767940521240234375f,0.3465999728083610786910639944835565984249114990234375f,-0.76099998922645983867596441996283829212188720703125f,-0.966000004053115812752139390795491635799407958984375f,0.158500005543231736737652681767940521240234375f,0.170100021713972093806432894780300557613372802734375f,-0.01409999396800998283652006648480892181396484375f,0.06939996886476895721074242828763090074062347412109375f,-0.83489998415335975323614547960460186004638671875f,0.1585000015012919261092605438534519635140895843505859375f,-0.626599996426701721929930499754846096038818359375f,0.24900000585615622572532856793259270489215850830078125f,-0.60319999434351956324462662450969219207763671875f,0.9835000030100344847738824682892300188541412353515625f,-0.52059997326284690455366899186628870666027069091796875f,0.03380000325739385136358805539202876389026641845703125f,-0.4705999953724440132418749271892011165618896484375f,-0.41179999939799305508358884253539144992828369140625f,-0.66540000028014123500952337053604423999786376953125f,0.64600000971555715523209073580801486968994140625f,0.254200039151310619445212068967521190643310546875f,-0.74259999310374258385536450077779591083526611328125f,-0.95929997451305393241227648104541003704071044921875f,-0.41000000298023220324239446199499070644378662109375f,0.255599963539838714865481961169280111789703369140625f,-0.314999988675117581493623220012523233890533447265625f,-0.3400999562233686379641994790290482342243194580078125f,0.354000016212463464171378291212022304534912109375f,0.69969998563379076639279219307354651391506195068359375f,0.0458000172123313176175685157431871630251407623291015625f,-0.3734999704696233546741268582991324365139007568359375f,0.5914000187873842406816038419492542743682861328125f,0.03170001374483177869478822685778141021728515625f,0.380799990725517290002244408242404460906982421875f,-0.07499998107552574566625480656512081623077392578125f,0.4956000004947185288983746431767940521240234375f,0.5875000091642139654624088507262058556079864501953125f,0.0899999953806400743161475475062616169452667236328125f,0.60470002273768230782735599859734065830707550048828125f,-0.0644000002503393176311874412931501865386962890625f,-0.5988000377535822593699776916764676570892333984375f,0.05259999176263807640907543827779591083526611328125f,0.928399980252981205097739803022705018520355224609375f,-0.52620001671016225497368168362299911677837371826171875f,0.263300006408989428763334217364899814128875732421875f,-0.88930000913962725661576769198291003704071044921875f,-0.237099971258640351834401371888816356658935546875f,-0.4248999764978884652322221882059238851070404052734375f,0.7328999812066552266287544625811278820037841796875f,-0.74339996758699433687667124104336835443973541259765625f,0.082100026392936342745088040828704833984375f,-0.40650000444054601445031948969699442386627197265625f,0.0420000194311143104641814716160297393798828125f,-0.97139996842145936994938892894424498081207275390625f,0.4967999940335747766084750765003263950347900390625f,0.6857999967604875291016242044861428439617156982421875f,0.9965999886035916688342695124447345733642578125f,0.14000002017244683738539379191934131085872650146484375f,0.54729999496117187618438038043677806854248046875f,0.41529999218210578160181967177777551114559173583984375f,0.29900000607967314181223628111183643341064453125f,0.253600011759996579741027744603343307971954345703125f,-0.0704000075817106107933796010911464691162109375f,1.074500003039836837359644050593487918376922607421875f,0.2306999736309052195792901329696178436279296875f,0.2213999781072140127236025364254601299762725830078125f,-0.18639999278485763767321259365417063236236572265625f,0.25170000052005081503381234142580069601535797119140625f,0.70660001900196078850058256648480892181396484375f,0.83459998645782462745046359486877918243408203125f,-0.785000000298023081768405972979962825775146484375f,-0.428299978506565093283597889239899814128875732421875f,0.38300000441074377732775246840901672840118408203125f,-0.875599975758790893820560086169280111789703369140625f,-0.10919997574686934882493005716241896152496337890625f,0.28600000748038301168207908631302416324615478515625f,1.0348999984025955001243346487171947956085205078125f,0.14840000573396683325455569502082653343677520751953125f,-0.256299990740418426327096312888897955417633056640625f,-0.199600001060962650711871901876293122768402099609375f,-0.798599985086917651955218389048241078853607177734375f,0.605199981588125268672229140065610408782958984375f,0.32170000599622738945981836877763271331787109375f,-0.741199989962577632240936509333550930023193359375f,-1.46050001290440523149527507484890520572662353515625f,-0.39220002366900441614916417165659368038177490234375f,1.321299989920853956704149823053739964962005615234375f,-0.138000006794929408471261922386474907398223876953125f,-1.7295000198036436511728197729098610579967498779296875f,-0.11760000696182260071509517729282379150390625f,0.30500000238418578657473290149937383830547332763671875f,-0.07360000535249711095531210958142764866352081298828125f,0.030399999311566450188593080383725464344024658203125f,0.72140002430081384687809986644424498081207275390625f,1.59820001699328440025738018448464572429656982421875f,-0.03659998636841788055562574299983680248260498046875f,-0.9282999776124949420363918761722743511199951171875f,0.1860000070333480692852390347979962825775146484375f,-1.43560001748204246752038670820184051990509033203125f,-0.1398000024676324670735994004644453525543212890625f,0.64629999185800546257496534963138401508331298828125f,-0.396099960500002179486500608618371188640594482421875f,-0.2766999660611162425993825308978557586669921875f,0.086600005292892578978580786497332155704498291015625f,0.9130999934911727677899762056767940521240234375f,-0.120099999585747685415526575525291264057159423828125f,1.1994999902993441764920135028660297393798828125f,-0.23100000710785384416823262654361315071582794189453125f,0.271800012660026624189413269050419330596923828125f,-0.183100042217969605218286233139224350452423095703125f,0.159499996185303416496026329696178436279296875f,-0.4605999854445457231122418306767940521240234375f,-0.0505999511718755456968210637569427490234375f,0.55380001271963141817877840367145836353302001953125f,-1.1426999835252757975467829965054988861083984375f,-1.675900017207860859258516939007677137851715087890625f,0.5500000210106374964880160405300557613372802734375f,0.5341000032812355247102686917060054838657379150390625f,-1.0756000024318694840985699556767940521240234375f,-0.242300005075335445070550122181884944438934326171875f,-0.314099996724724661589789320714771747589111328125f,0.68740000637471665445588087095529772341251373291015625f,0.41069998912811200852956972084939479827880859375f,-0.871899986392259851442076978855766355991363525390625f,1.15749998390674591064453125f,-0.1524999980628489737455311114899814128875732421875f,0.550999992430210028260262333787977695465087890625f,1.2600000202655790015882075749686919152736663818359375f,-1.356400017595291274119517765939235687255859375f,-0.7349999825656414742525157635100185871124267578125f,-0.06009996598362954411953751332475803792476654052734375f,-0.64320000626444817637406003996147774159908294677734375f,-1.0712000140279531734677220811136066913604736328125f,0.22140000582337382439845896442420780658721923828125f,0.9476999693691734449885188951157033443450927734375f,-0.84540000862479214305267305462621152400970458984375f,0.211399993269144825802641207701526582241058349609375f,0.7212000067636370470580686742323450744152069091796875f,-0.3929999946691091761152847539051435887813568115234375f,0.65920000376105303274698599125258624553680419921875f,0.54580006925463653288943532970733940601348876953125f,0.20439998490214339454951186780817806720733642578125f,-0.03600002860277939920052858724375255405902862548828125f,0.02739999698698525332218878247658722102642059326171875f,0.429199974349886603786075056632398627698421478271484375f,0.74400001397728932062136664171703159809112548828125f,0.42659997444748842809758571092970669269561767578125f,-0.21900001159310367171428879373706877231597900390625f,-0.59969995601773273818935194867663085460662841796875f,-0.20799998906254746344757222686894237995147705078125f,0.14530001298785266694579831892042420804500579833984375f,-0.25520000125765796017418551855371333658695220947265625f,-0.143700001361966034352235510596074163913726806640625f,-0.10420000405907625662393911625258624553680419921875f,-1.25459999666512000970897133811376988887786865234375f,0.0305999964714052108405439867055974900722503662109375f,0.688799983513355140729572667623870074748992919921875f,0.652399994081258594036398790194652974605560302734375f,0.3885999964118003191515526850707828998565673828125f,-0.8350999970525503357521301950328052043914794921875f,0.0002000238329173953388817608356475830078125f,0.81589998598992818923392178476206026971340179443359375f,-0.1495000063180924598782439716160297393798828125f,-0.007199967640638238464134701644070446491241455078125f,-0.533699982810020401302608661353588104248046875f,-0.848500036761164810883428799570538103580474853515625f,0.891499977134168286596604957594536244869232177734375f,0.4312000145494938596613110348698683083057403564453125f,0.26489997098445883239037357270717620849609375f,1.00879998857975028414557527867145836353302001953125f,1.009699974942208200445747934281826019287109375f,1.010699963125586631207397658727131783962249755859375f,-0.366799993474781427238440301152877509593963623046875f,0.9446000082693994936988701738300733268260955810546875f,0.26229998130798293232146534137427806854248046875f,-0.331199978637695569005927609396167099475860595703125f,-1.0129000177145002226097858510911464691162109375f,-0.61340000826716423176776515902020037174224853515625f,-0.61169995980262825696627260185778141021728515625f,-0.57199998448789113325574362534098327159881591796875f,0.244600014397502008023366215638816356658935546875f,-0.61500000394880760534732644373434595763683319091796875f,0.2699000125586987763881552382372319698333740234375f,0.6579999886900187444638277156627736985683441162109375f,-0.9533999733984472868542070500552654266357421875f,0.5865999987363814671681438994710333645343780517578125f,0.53529995085001047527839546091854572296142578125f,0.59829998834133135687807225622236728668212890625f,-0.30239996733367480175047603552229702472686767578125f,0.3393999891862271578446552666719071567058563232421875f,0.4539999812506139154066886476357467472553253173828125f,0.641399984365701669730697176419198513031005859375f,0.561000024765728966258393484167754650115966796875f,-0.346800011914968564497030456550419330596923828125f,-1.1909000151216986296276445500552654266357421875f,-0.09280000374913210947624975233338773250579833984375f,0.414399962550401568250890704803168773651123046875f,0.593600004011392456959583796560764312744140625f,0.64190000963807136713512591086328029632568359375f,0.5467999833047390012552568805404007434844970703125f,0.04460000568032196355261476128362119197845458984375f,0.58799998608231529573231455287896096706390380859375f,1.69069995247125604009852395392954349517822265625f,-0.342199990141391907627621549181640148162841796875f,-1.006000008374452914239327583345584571361541748046875f,0.18060000764727579536383927916176617145538330078125f,0.9645000005811452670201333603472448885440826416015625f,-0.5898000203490258996907869004644453525543212890625f,-0.720899989938736940331409641657955944538116455078125f,-0.406799992394447418320169163052923977375030517578125f,-0.5815000051856042961873072272283025085926055908203125f,0.4843999841570851838667977062868885695934295654296875f,-0.078500032365322880423263995908200740814208984375f,0.2199999971687791600061245844699442386627197265625f,1.0817000284969804368984114262275397777557373046875f,-0.00459999822974221928006954840384423732757568359375f,-0.3585000472664834347824580618180334568023681640625f,-1.04489996543377561266652264748699963092803955078125f,-0.068199990618229122674165409989655017852783203125f,-0.60579997269511220991944355773739516735076904296875f,0.56799996730685275991845628595910966396331787109375f,-0.49000003587454532549827490583993494510650634765625f,-0.295399978673458463163115084171295166015625f,0.0364000486642124254643704261980019509792327880859375f,-0.45839999813437426467999102897010743618011474609375f,0.7501000371739270367044127851841039955615997314453125f,0.30439999577999099500402735429815948009490966796875f,0.1563999982237811448015918358578346669673919677734375f,0.661999967575073622327863631653599441051483154296875f,0.12630000005364483417480414573219604790210723876953125f,0.490799987894296574353347750729881227016448974609375f,0.0684000291660431525286867326940409839153289794921875f,0.93519996758103385303684262908063828945159912109375f,0.1039000064566735215265680380980484187602996826171875f,-0.1816000002264974710897149634547531604766845703125f,0.5889999896138906620990383089520037174224853515625f,-0.2233999841272833464245195500552654266357421875f,1.5906999914005395257987629520357586443424224853515625f,0.289800000977515903599623925401829183101654052734375f,0.77550000090897042159809871009201742708683013916015625f,-0.061600012445449436881972360424697399139404296875f,0.005900010427832835802064437302760779857635498046875f,0.4174000028729434319529900676570832729339599609375f,0.9135000011473888292812262079678475856781005859375f,-0.20980000619292216157418806687928736209869384765625f,-0.4453999810948960913492555846460163593292236328125f,-1.049200010019541107197937890305183827877044677734375f,-0.318899990549683653995316490181721746921539306640625f,-0.5421999789655209678329583766753785312175750732421875f,-0.5477999708175662618714341078884899616241455078125f,-0.0800999816298482159027116722427308559417724609375f,0.32989998603463188686646390124224126338958740234375f,1.0363999776601791058538992729154415428638458251953125f,1.047699961620569553133464069105684757232666015625f,0.605999963372946037765132132335565984249114990234375f,0.7195999672353270426583549124188721179962158203125f,0.18129998880326780863470048643648624420166015625f,0.4081999868929386821037041954696178436279296875f,-0.134999977573752261150730191729962825775146484375f,-0.46939997298121483026989153586328029632568359375f,-0.8926999943286180183577016578055918216705322265625f,-0.49249999880790706896505071199499070644378662109375f,0.0592000104665756088362371656330651603639125823974609375f,0.71710001162216041592500914703123271465301513671875f,0.44610000121742476864028503769077360630035400390625f,0.2354999913722277682381900376640260219573974609375f,0.2735999865025282584696242338395677506923675537109375f,-0.21900001882016628673710556540754623711109161376953125f,0.639299990680813667864867966272868216037750244140625f,0.41520000647306432028926792554557323455810546875f,0.36139997572302806627675408890354447066783905029296875f,0.757399976535141572497877859859727323055267333984375f,0.585200006473064693324204199598170816898345947265625f,-0.1896999998271464704657773836515843868255615234375f,-0.2225999888569116791359192575328052043914794921875f,1.107599998915195005366740588215179741382598876953125f,0.10659999188184754448371904800296761095523834228515625f,0.79489998655617200018497214841772802174091339111328125f,0.545399966901541244368445404688827693462371826171875f,-0.88250004392117220586300163631676696240901947021484375f,-0.3189999753832821749455206372658722102642059326171875f,-0.8475999887079004313505947720841504633426666259765625f,-0.0893999966740608442705706693232059478759765625f,0.0069000054657457621232197197969071567058563232421875f,-0.369799990099668463017223984934389591217041015625f,1.400199995632469107675888153607957065105438232421875f,0.0163999973297119794324316899292171001434326171875f,0.736999967947601586359951397753320634365081787109375f,0.88339996088147199060358616407029330730438232421875f,0.0998999534383415710436793233384378254413604736328125f,-0.43619999115467056327588579733856022357940673828125f,0.81960003177598161894223949275328777730464935302734375f,-0.21599999291449789584618201843113638460636138916015625f,-0.3446000004835425567506490551750175654888153076171875f,-0.08979998846054104699732079097884707152843475341796875f,-1.1431000088393687974530621431767940521240234375f,-0.2629999814629557786105351624428294599056243896484375f,-0.6377999404564496099823145414120517671108245849609375f,0.49319997929334657982280987198464572429656982421875f,-0.3285999865397808417100122824194841086864471435546875f,-0.18839999507963656188991308226832188665866851806640625f,-0.7233000186279416876544701153761707246303558349609375f,0.49899999475479095423224862315692007541656494140625f,-0.1485000243932013130887526131118647754192352294921875f,0.5148000039577482755959181304206140339374542236328125f,-0.139799996116012348323920377879403531551361083984375f,-0.0008000043228266751071942053386010229587554931640625f,-0.028800020915269453070095551083795726299285888671875f,-0.4094000217825175536034976175869815051555633544921875f,-0.289899989312887296222243094234727323055267333984375f,-0.4583999891191721243188794687739573419094085693359375f,-0.4699999818205835566686801030300557613372802734375f,0.46010003736019111642008283524774014949798583984375f,0.208600022637843718342764987028203904628753662109375f,-0.5131000074982641212528733376529999077320098876953125f,0.07239997053742452948199570528231561183929443359375f,-0.37310002873241909782109360094182193279266357421875f,0.77310001181960075200549908913671970367431640625f,0.119900013825297246938816897454671561717987060546875f,-0.51059998514652260581669906969182193279266357421875f,-0.650699978697300007723924863967113196849822998046875f,-0.60049997565150281531032305792905390262603759765625f,-0.87509998528063304146229484103969298303127288818359375f,-1.0043999976053832323685810479219071567058563232421875f,-0.34919999295771109171226953549194149672985076904296875f,-0.0214999978467822483452209780807606875896453857421875f,-0.42939998404532675468914248995133675634860992431640625f,-0.9460999698877337049651714551146142184734344482421875f,-0.91089997839033609583481165827834047377109527587890625f,-0.606899979835748837331266258843243122100830078125f,-0.119499991565942931259769466123543679714202880859375f,0.3037999960303305879705249026301316916942596435546875f,-0.18400000943243466355880855189752764999866485595703125f,-1.3735999908424911808424440096132457256317138671875f,0.20599999798089252411870120340608991682529449462890625f,0.35380002082213779512898099710582755506038665771484375f,0.562000002443789892225822768523357808589935302734375f,0.33710000612735713243495183633058331906795501708984375f,0.365799982976913629073578704264946281909942626953125f,-0.75670004526078693540824815499945543706417083740234375f,-0.62299998429417602352486937888897955417633056640625f,-0.1512000013992194080714170922874473035335540771484375f,0.47099997231364287841870464035309851169586181640625f,1.67400000716000807443606390734203159809112548828125f,0.12100000688433620865680495626293122768402099609375f,1.7563000420376659871379843025351874530315399169921875f,-0.35599999883770916397907058126293122768402099609375f,0.0609000088259581406902043454465456306934356689453125f,0.168599999541044098805286921560764312744140625f,0.616499975718557635673278127796947956085205078125f,0.071200003522634336405872090836055576801300048828125f,0.61289996794462131557423845151788555085659027099609375f,0.23279998936951173948983750960906036198139190673828125f,0.06629999774396411194032907587825320661067962646484375f,0.226199985492229693750942942642723210155963897705078125f,0.235799994041025640090225579115212894976139068603515625f,0.598399973398447304617775444057770073413848876953125f,0.4035999789401889525919386869645677506923675537109375f,0.17999999120831500221129317651502788066864013671875f,0.47989999188333687385465964325703680515289306640625f,0.349199971425533828295328930835239589214324951171875f,1.0251999834507705688935175203368999063968658447265625f,-1.0040000064522029976643580084783025085926055908203125f,-0.28699999924004071516492331284098327159881591796875f,-0.53320001386404047849509879597462713718414306640625f,-0.96070000417828538274989114142954349517822265625f,-1.017100007691979168811258205096237361431121826171875f,-0.40339999247193336628924953402020037174224853515625f,-0.362899996033311023069245493388734757900238037109375f,-0.195999994069337990509893643320538103580474853515625f,0.3182000221341849677259006057283841073513031005859375f,-0.249599999049305854015301520121283829212188720703125f,0.2036000002115965212823311958345584571361541748046875f,0.61499999940395344566468338598497211933135986328125f,-0.003999998480081590201962171704508364200592041015625f,-0.55599999600648875031083662179298698902130126953125f,-0.313300003391504322536320614744909107685089111328125f,-0.30310000064373010530971441767178475856781005859375f,0.091799981069564939861749053306994028389453887939453125f,0.6026999716416003849417393212206661701202392578125f,-0.009700014802813239356282792869023978710174560546875f,-0.4217000113606450550918225417262874543666839599609375f,-0.49739999966919412610621975545655004680156707763671875f,-0.68159998323917392326620756648480892181396484375f,-0.3431000095844269903722079106955789029598236083984375f,0.27309998842477811731299652819870971143245697021484375f,0.44449998977780336151255369259160943329334259033203125f,0.3424999920651317086850440318812616169452667236328125f,0.23099997946619998945294582881615497171878814697265625f,0.637899975842237676459944850648753345012664794921875f,0.7307999788045884681508823632611893117427825927734375f,1.317000023305415989938182974583469331264495849609375f,0.13919999586343767550289385326323099434375762939453125f,0.373900036445260042228255770169198513031005859375f,0.71119996865391765794583989190869033336639404296875f,0.8343000141873961883476340517518110573291778564453125f,-0.0473999937832356721401083632372319698333740234375f,-0.157800031949580255030696207541041076183319091796875f,-0.37559999974966018498889752663671970367431640625f,-0.438799997445940892504268049378879368305206298828125f,0.46559998514652256584867018318618647754192352294921875f,-0.55780001291334624369255834608338773250579833984375f,0.076799998056888529873731386032886803150177001953125f,0.07079998573362800140529316195170395076274871826171875f,0.176999993652105391817030977108515799045562744140625f,-0.13969998250454651600449551551719196140766143798828125f,0.3677999824404718509640588308684527873992919921875f,-0.1249999947473405814690750048612244427204132080078125f,-0.2803999912649393078112325383699499070644378662109375f,0.3642000105410814203565905700088478624820709228515625f,0.179399999952316147044939498300664126873016357421875f,0.325100020670891165508464837330393493175506591796875f,0.3849999962747094883752652094699442386627197265625f,0.0699000102117659505296387578709982335567474365234375f,-0.20239998588562035930493721025413833558559417724609375f,-0.22879998708963411591099657016457058489322662353515625f,0.0505999975889921405070026594330556690692901611328125f,-0.248400002194940849875592903117649257183074951171875f,-0.7033999732494355949796727145439945161342620849609375f,-1.6524999885261058008012469144887290894985198974609375f,-0.51700000162422654614857719934661872684955596923828125f,-0.41810000086724752321742926142178475856781005859375f,-0.063599992910027491888058648328296840190887451171875f,-0.10409999292492866374004734097979962825775146484375f,-0.480600018897652692118072081939317286014556884765625f,-0.5129000066131350621390083688311278820037841796875f,-0.299600029000639977994069340638816356658935546875f,0.5943999813258649123071108988369815051555633544921875f,-0.05199999588727932664511399707407690584659576416015625f,-0.170299979311228089073892988380976021289825439453125f,1.1414000038117169122386940216529183089733123779296875f,1.1207999683737757123935807612724602222442626953125f,0.726699996608495712990816173260100185871124267578125f,-0.422399977391958447725528458249755203723907470703125f,-0.0332999829024077931283187581357196904718875885009765625f,-0.301899992371350567310628321138210594654083251953125f,-0.11570002738684426191184684284962713718414306640625f,0.61279996664524105653981678187847137451171875f,0.45209999040663273461859716917388141155242919921875f,0.360799991619586979396672177244909107685089111328125f,-0.1811999929428098621286835623322986066341400146484375f,-1.02679998479485501405861214152537286281585693359375f,-0.71680002309083956646418300806544721126556396484375f,0.2916999883010984451203739808988757431507110595703125f,0.29650000410526977656644476155634038150310516357421875f,-0.673199987041950276278612363967113196849822998046875f,0.2546999871611597132670112841879017651081085205078125f,1.0944999562501911416489974726573564112186431884765625f,0.15259998155534282293643855155096389353275299072265625f,0.8913999956905838129017638493678532540798187255859375f,-0.366199989813566162411007098853588104248046875f,0.77940005337297957321851527012768201529979705810546875f,0.66379998261928552238941847463138401508331298828125f,0.52300004732608851298891750047914683818817138671875f,0.71219998894929859023505969162215478718280792236328125f,0.9149000207543369622698037346708588302135467529296875f,0.327999982655048594182289889431558549404144287109375f,-0.5137000449478625885291194208548404276371002197265625f,-0.29939999488592139442744155530817806720733642578125f,-1.777899994468688849025284071103669703006744384765625f,-0.1537999967753886920718287001363933086395263671875f,-1.294999992437660285560241391067393124103546142578125f,0.23739999437928194225833067321218550205230712890625f,-0.77950001773610722022311847467790357768535614013671875f,0.34199998944252729682347080597537569701671600341796875f,0.23970000420436232690946098955464549362659454345703125f,-0.57539999461770019451023472356610000133514404296875f,0.605200031246245107041659139213152229785919189453125f,0.255199988889694306948285884573124349117279052734375f,-0.08039998135566650550032363753416575491428375244140625f,0.90359998806714969532549730502068996429443359375f,-0.07590000510066741146175672838580794632434844970703125f,-0.6215999750435354798128173570148646831512451171875f,-0.54909997794926168612761330223293043673038482666015625f,0.39699998351931586881846669712103903293609619140625f,-0.406500000379979464781854403554461896419525146484375f,-0.218999983131885844755970538244582712650299072265625f,-0.411600010731816734477206409792415797710418701171875f,0.14030000598430636404856386434403248131275177001953125f,0.0019000035658478520161906999419443309307098388671875f,0.2086000033035873835185469715725048445165157318115234375f,-0.0267999991185963182260820758529007434844970703125f,0.002500011958181669324829954348388127982616424560546875f,-0.0146999732099477586899638481554575264453887939453125f,0.13770001034736623068965855054557323455810546875f,0.9485999977529047111346471865545026957988739013671875f,0.38350000718235965901925510479486547410488128662109375f,-1.2428999920845029691918171010911464691162109375f,-0.79960001685619364053536628489382565021514892578125f,0.02299999949336051230375232989899814128875732421875f,-0.089899991026520797987586774979718029499053955078125f,0.1486000205516813110762086580507457256317138671875f,-0.77120001305937790903044515289366245269775390625f,-1.0115000117421146796914399601519107818603515625f,-1.3256000045180318380744211026467382907867431640625f,-1.355399988955259260592356440611183643341064453125f,-1.08099998475611158710307790897786617279052734375f,-1.2699000094294543572459588176570832729339599609375f,-1.1084000259995459014561447474989108741283416748046875f,-0.61930001787543265123758828849531710147857666015625f,0.315799992364645287779012505779974162578582763671875f,0.811499983578920502935716285719536244869232177734375f,0.3548999907284973165388919369434006512165069580078125f,-0.035999986469745781647588955820538103580474853515625f,-0.1072999965071679273620475214556790888309478759765625f,0.2428000006198882676500261368346400558948516845703125f,0.27650000779330741185191300246515311300754547119140625f,-0.13000001356005663755155410399311222136020660400390625f,0.406599993520974922756039404703187756240367889404296875f,-0.90890002484098098012310629201238043606281280517578125f,0.232999992452561865974303145776502788066864013671875f,0.18249999348074175120615336709306575357913970947265625f,0.363199974189699126281283270145650021731853485107421875f,-0.1449000284098087565354262551409192383289337158203125f,-0.544999990314245064126907891477458178997039794921875f,-0.6328999604567879799787988304160535335540771484375f,0.8165999832391737101033868384547531604766845703125f,-0.022699993062019085954261754523031413555145263671875f,-0.31979998831152922633691559894941747188568115234375f,1.2712000409618025020819231940549798309803009033203125f,0.0716000064849851014514570124447345733642578125f,1.33690002647638284116737850126810371875762939453125f,0.63179998136758808158219835604541003704071044921875f,0.8384999887794253314865500215091742575168609619140625f,0.48479997981786748795229868846945464611053466796875f,0.3420999931633474933079241964151151478290557861328125f,0.932599994146823263463375042192637920379638671875f,-0.01160003204047743707860718131996691226959228515625f,-0.1957999901294709133026117342524230480194091796875f,-0.3779999769181008328899906700826250016689300537109375f,-0.20919998170733489217809619731269776821136474609375f,0.1259000205233693492345992126502096652984619140625f,0.049400001889467137772271598805673420429229736328125f,0.43700000017136331198486232096911408007144927978515625f,-0.03339999694228179549781998503021895885467529296875f,-0.68510002249628321191465829542721621692180633544921875f,0.046899996972083801693997884285636246204376220703125f,0.389400001293420672254796954803168773651123046875f,0.5609000050634145306815980802639387547969818115234375f,0.44320000157058225820350116919144056737422943115234375f,0.8198999930381776213295097477384842932224273681640625f,1.286599973385781137125150053179822862148284912109375f,0.4698999666631220595292006692034192383289337158203125f,-0.11850001105666141398842228227294981479644775390625f,-0.21780002066195003340709490657900460064411163330078125f,0.490399985304474927971796205383725464344024658203125f,0.12970000079572197027033553240471519529819488525390625f,-0.344599984595179709145895685651339590549468994140625f,-0.3072999900251629501468642047257162630558013916015625f,-0.47909997400045423177061820751987397670745849609375f,0.63160002064108822228405415444285608828067779541015625f,0.7527000050574539358194670057855546474456787109375f,0.0732999943017957067414869243293651379644870758056640625f,-0.370599991088360614099883605376817286014556884765625f,-0.5227000162705779384708648649393580853939056396484375f,0.003899977362156192839393042959272861480712890625f,-0.9983000075638290837787280906923115253448486328125f,-0.869399989074468582117560799815692007541656494140625f,-1.2907000118523834775174918831908144056797027587890625f,-0.52020000990033121723854492302052676677703857421875f,-0.277700009378790912961676440318115055561065673828125f,-0.0439999971389770649921047152020037174224853515625f,-0.334900009801983866708496861974708735942840576171875f,-0.0182000012725591631124189007095992565155029296875f,0.20460000143349166279449491412378847599029541015625f,0.2453000026687977008776897491770796477794647216796875f,-0.080200008112191856213257779018022119998931884765625f,-0.67800000720471131909761197675834409892559051513671875f,-0.1394000020384786875382587822969071567058563232421875f,-0.5971999960459764000830773511552251875400543212890625f,-0.58739997020214840262042343965731561183929443359375f,0.34670005261823544362442817146074958145618438720703125f,-0.135799995791912042619742351234890520572662353515625f,1.142300032344460536393171423696912825107574462890625f,-0.270999990642070809343522341805510222911834716796875f,-0.168799994130432562400301321758888661861419677734375f,0.099799991738796389739718506461940705776214599609375f,-0.7439000023588533405671796572278253734111785888671875f,-0.45899999177455885757126452517695724964141845703125f,-0.5501999700397248727057331052492372691631317138671875f,0.086799996864795669893055674037896096706390380859375f,-0.23989999793693417284856650439905934035778045654296875f,-0.09220000247210247135143390551093034446239471435546875f,1.6737000378884372420174031503847800195217132568359375f,0.05159999054074315694151664501987397670745849609375f,2.0329000254258513535887686884962022304534912109375f,0.42779998974204058725945287733338773250579833984375f,1.827000020883977615682169926003552973270416259765625f,-0.065399987167120254838437176658771932125091552734375f,0.76280003940016027907944362596026621758937835693359375f,0.5361999814689160093195141598698683083057403564453125f,0.304999993629753551882544115869677625596523284912109375f,-0.11000000178813917894871110547683201730251312255859375f,0.63640002094805236954044858066481538116931915283203125f,0.20759999939948337033257530492846854031085968017578125f,0.818199975270033075247511078487150371074676513671875f,0.3360999835222961973357769238646142184734344482421875f,-0.066700014489888825863772581215016543865203857421875f,-0.3227000168293716608758359143394045531749725341796875f,-0.23319998510479944542339580948464572429656982421875f,-0.572499986365437418811552561237476766109466552734375f,-0.42419998414739978898069239221513271331787109375f,0.2172000078365206032771084210253320634365081787109375f,0.683999993562698360705098821199499070644378662109375f,-0.2961000063210728061591225923621095716953277587890625f,-0.57329998297691364683714709826745092868804931640625f,-0.2342999878495930499155974757741205394268035888671875f,0.08819999687671664645449709496460855007171630859375f,0.184299997907876988989528399542905390262603759765625f,0.9879000099658963307547310250811278820037841796875f,0.1046999888002868583924964696052484214305877685546875f,0.0978999978959560479552237666212022304534912109375f,-0.51999999567866328487042437700438313186168670654296875f,-0.8612999816134572217407594507676549255847930908203125f,-1.26659999806582934667176232323981821537017822265625f,-1.0953999967038630103388641146011650562286376953125f,-0.5004999906271694332104971181252039968967437744140625f,0.84610001485198704784806977841071784496307373046875f,-0.026699997130036567938304870040155947208404541015625f,-0.264199995788931829565626685507595539093017578125f,-0.365500006347894756952854322662460617721080780029296875f,0.18850000096112484726518232491798698902130126953125f,0.6714999833554029518012384869507513940334320068359375f,0.220199990342557272438028803662746213376522064208984375f,0.04700000378489477270704810507595539093017578125f,0.3587000313505532833602273967699147760868072509765625f,0.676799994331597876140449443482793867588043212890625f,0.57410001650601583111921399904531426727771759033203125f,0.83139996201396026975771746947430074214935302734375f,-0.11020003476664397279449758570990525186061859130859375f,0.08399999970942728655387554681510664522647857666015625f,0.09680000444576142282215869272476993501186370849609375f,0.11420000018477427516927491524256765842437744140625f,0.7666999909833067672337847398011945188045501708984375f,0.1547999772846704313877808090182952582836151123046875f,0.352100010634959048605452380797942169010639190673828125f,0.3585999840438367503026029226020909845829010009765625f,0.154900002239645118162769676928292028605937957763671875f,-0.56199997696280501457977152313105762004852294921875f,1.2158000334545968623700673560961149632930755615234375f,-0.305399996256828121232729245093651115894317626953125f,1.251800037954748034962904057465493679046630859375f,0.56659998122751720206480285924044437706470489501953125f,1.144899992032348823300935691804625093936920166015625f,0.37379998515248313850634076516143977642059326171875f,1.158300009277463349377512713545002043247222900390625f,-0.1935999924629927537722551278420723974704742431640625f,0.908700009408593079029969885596074163913726806640625f,0.14699999901652327505274797658785246312618255615234375f,0.84150001233816140722154841569135896861553192138671875f,-0.082400001159310232168309084954671561717987060546875f,-0.206199982288479966019423272882704623043537139892578125f,-0.3478000128388405443047304288484156131744384765625f,-0.13209998839497583134061642340384423732757568359375f,0.048599997678399287082129376358352601528167724609375f,0.07639996663332038195903805899433791637420654296875f,-0.358100029850005885379005121649242937564849853515625f,-0.125200012582540320948965018033050000667572021484375f,0.7541999934047460829589226705138571560382843017578125f,0.66729998046979321912886007339693605899810791015625f,-0.1944000055029988516253069974482059478759765625f,-0.45199997934699087664967009914107620716094970703125f,0.2972000256806612839000081294216215610504150390625f,0.3938999933622777671615722283604554831981658935546875f,0.13139999848455186093332258678856305778026580810546875f,0.316200004118680766396209946833550930023193359375f,0.4002000212997198769215856373193673789501190185546875f,-0.0287999945774675136789255702751688659191131591796875f,-0.153899998037517082849490179796703159809112548828125f,0.076000004425644807071194009040482342243194580078125f,0.1151000054717064013942717792815528810024261474609375f,0.1059999995082616663921726285479962825775146484375f,0.45810001107454301205734736868180334568023681640625f,0.2647000028073787092353086336515843868255615234375f,-0.30659998405873789550923902424983680248260498046875f,-0.866999966830015456054070455138571560382843017578125f,-0.0543999855726957637358509600744582712650299072265625f,0.93489998484253877819583067321218550205230712890625f,0.44020001131594199694774260933627374470233917236328125f,-0.42969999155700211446173852891661226749420166015625f,-0.637099989736080285496200303896330296993255615234375f,-0.6113000044494862894595144098275341093540191650390625f,-0.8005999836564061666877023526467382907867431640625f,1.2591000596821313450845991610549390316009521484375f,0.18180001727938588373945094645023345947265625f,0.007199988241493482377109103254042565822601318359375f,-0.3579999766945842765153429354541003704071044921875f,-0.76449999609217089702184466659673489630222320556640625f,0.02859999771565202575374087246018461883068084716796875f,-0.286500037129968632143572904169559478759765625f,-0.68239999125003780733322855667211115360260009765625f,0.12250000270083549747113238481688313186168670654296875f,-0.05139999811202289325962055954732932150363922119140625f,-0.005700028597563910892631611204706132411956787109375f,-0.4539999959468838941489821081631816923618316650390625f,-0.177600011469423890719809833171893842518329620361328125f,0.235399989551305865997932187383412383496761322021484375f,0.24459999495148665449306690788944251835346221923828125f,-0.08899999803304670109582730219699442386627197265625f,-0.16980000861436128634096576206502504646778106689453125f,-0.04339999515414250108591431853710673749446868896484375f,-0.48910001084357512812772483812295831739902496337890625f,0.306199993538856407582215979346074163913726806640625f,0.23789998649656751705805390884052030742168426513671875f,0.647599975967407370802675359300337731838226318359375f,-0.36750006146729063960520988985081203281879425048828125f,-0.1778000038981435437790423748083412647247314453125f,-0.825400035521388186765534555888734757900238037109375f,-0.01960000284910190515574868186376988887786865234375f,0.26000000922009380843746839673258364200592041015625f,0.09079999430179608310709227225743234157562255859375f,0.333999994307756509215323603712022304534912109375f,-0.33430001735389236472428819979541003704071044921875f,-0.02010000100135800327194601777591742575168609619140625f,0.221999982252716943076364941589417867362499237060546875f,1.11179998181462291739762804354541003704071044921875f,1.255299982365965849595568215590901672840118408203125f,1.13139997229576128034977955394424498081207275390625f,1.439099964463710801254592297482304275035858154296875f,0.92499995842576065996354373055510222911834716796875f,1.04429998084604758190607753931544721126556396484375f,0.404799999114870967531487622181884944438934326171875f,0.5500000105798246163857356805237941443920135498046875f,0.4274999964237213845308360760100185871124267578125f,-0.66370002155303975399647242738865315914154052734375f,-0.596299990069866225894656963646411895751953125f,0.2175000293552873387170620844699442386627197265625f,0.069200026112794521537807668210007250308990478515625f,-0.2233000003367660024622409764560870826244354248046875f,1.08119996701478982004118734039366245269775390625f,0.64879999118745335096036797040142118930816650390625f,-0.1549000035062431113175307473284192383289337158203125f,0.17160000324398272386616781659540720283985137939453125f,-0.02519998978376403186985044158063828945159912109375f,-0.115200025770068048558414375293068587779998779296875f,0.88839995983839070703425022657029330730438232421875f,0.821299998265505148964393811183981597423553466796875f,0.18560000466704362764858160517178475856781005859375f,0.631900002112984804369943958590738475322723388671875f,0.19440002938210998451040723011828958988189697265625f,0.29419999809861163697632946423254907131195068359375f,0.628900006903708064243119224556721746921539306640625f,-0.16259999042153372439400982329971157014369964599609375f,0.365300015074014883253994412370957434177398681640625f,-0.90099997141957288704361417330801486968994140625f,-0.608599969775974525276751592173241078853607177734375f,-0.416599982494116005682371906004846096038818359375f,-1.842199975128471667318308391259051859378814697265625f,-0.340199988442659417842151015065610408782958984375f,-1.1657999735519286588214527000673115253448486328125f,0.03239999795556069983604174922220408916473388671875f,0.07110000728964827043654395311023108661174774169921875f,0.65119999696612307360510385478846728801727294921875f,-0.0682000276848675479612893468583934009075164794921875f,-0.67559997873902322140793330618180334568023681640625f,0.5137000291898845549809493604698218405246734619140625f,0.6513999727427961516923460294492542743682861328125f,0.9203999433204528290985990679473616182804107666015625f,-0.69439996597766917574290346237830817699432373046875f,-0.01949997938424374854804455026169307529926300048828125f,0.464999987483024568302170109745929948985576629638671875f,-0.425900002939999300366480383672751486301422119140625f,0.8083999675869943946082685215515084564685821533203125f,0.197399983352422647708834801960620097815990447998046875f,-0.5997999850332735860547472839243710041046142578125f,-0.46099997987598140269938085111789405345916748046875f,0.374999998956918378922864576452411711215972900390625f,-1.24790004107207064265594453900121152400970458984375f,0.16179998837113362952777606551535427570343017578125f,-0.265299996447563302126582129858434200286865234375f,0.2028000144779682489826200253446586430072784423828125f,0.67890000939965222670480216038413345813751220703125f,-0.113999990321695998662931970102363266050815582275390625f,-0.34929998270869277998684765407233498990535736083984375f,-0.5598999806515874144707822779309935867786407470703125f,-0.88619996954798718746815211488865315914154052734375f,0.017100000092387102057500669616274535655975341796875f,0.78179998360276226065934679354541003704071044921875f,0.75460000813901417782147973412065766751766204833984375f,0.064900013191997729666837813056190498173236846923828125f,0.87270001697838306409238384730997495353221893310546875f,0.919799987119436224247692734934389591217041015625f,0.71059998805224910967126561445184051990509033203125f,0.24809997918605830591332050971686840057373046875f,-0.0231999941200020654008540077484212815761566162109375f,-0.95249996401369607657017013480071909725666046142578125f,-1.177699975776672403071643202565610408782958984375f,-0.61909997832179097798643852001987397670745849609375f,-0.51919996166527260239575980449444614350795745849609375f,-0.77109997785985484597404138185083866119384765625f,-0.49119999622106558234690965036861598491668701171875f,0.1193999922037125571705473703332245349884033203125f,-0.032200006309151818495450925183831714093685150146484375f,-0.55529999760240311701409154920838773250579833984375f,0.504399999058246617522627275320701301097869873046875f,1.05199998113512993569429454510100185871124267578125f,0.604800002840161443629085624706931412220001220703125f,-0.7639000088781122688175173607305623590946197509765625f,-0.7110000025629990005882063996978104114532470703125f,0.942800043162704159982467899681068956851959228515625f,-0.2878000015139576817091437987983226776123046875f,1.1702000157490370924051603651605546474456787109375f,0.42359997506588731308596607050276361405849456787109375f,0.17530001473873901840505595828290097415447235107421875f,0.7445999714076521058814250864088535308837890625f,-1.045200024540722250065982734668068587779998779296875f,-0.3291999896049500051731229177676141262054443359375f,-1.52770004666894720912750926800072193145751953125f,-0.45120000002086124712974424255662597715854644775390625f,-1.173500024616718206971199833787977695465087890625f,-0.46959999092817295007762368186376988887786865234375f,-1.0094999967813487184997711665346287190914154052734375f,-0.6565999683380130402809982115286402404308319091796875f,0.210600020015239308701637810372631065547466278076171875f,0.4475999714970593235108253793441690504550933837890625f,0.5111999460414049512024803334497846662998199462890625f,-0.617799996149539509815440396778285503387451171875f,0.458599988886714271529854158870875835418701171875f,-0.172199988800287417944900880684144794940948486328125f,0.3344999954402447794876707121147774159908294677734375f,-0.7519999672770503185148527336423285305500030517578125f,0.103500024877488439045691848150454461574554443359375f,-0.1957999966114758461088740659761242568492889404296875f,-0.4746999985232951413394175688154064118862152099609375f,-0.12000000596046422884910498396493494510650634765625f,-1.6689000057488672101868587560602463781833648681640625f,-0.231700016352534010621866400470025837421417236328125f,0.3724999608099466286148526705801486968994140625f,-0.3140000206083060074746526879607699811458587646484375f,-0.6670999801993371391972686978988349437713623046875f,-0.16279997700154780204684357158839702606201171875f,-0.59079998878836637032918588374741375446319580078125f,-0.385699991288781074416647243197076022624969482421875f,-1.096200005906819985312949938816018402576446533203125f,-0.2380000079125166312365990961552597582340240478515625f,0.98860001026987998073991548153571784496307373046875f,0.982600011357664726574512314982712268829345703125f,0.6942000080823895746817697727237828075885772705078125f,0.81330001479089253368925938048050738871097564697265625f,0.2884000026047228715242454200051724910736083984375f,-0.0039000003844498198901646901504136621952056884765625f,-0.64910001363754243897119522443972527980804443359375f,0.03370002080798162324981603887863457202911376953125f,-0.1972999827608468148554266008432023227214813232421875f,-0.01779999477118254791463414221652783453464508056640625f,0.3989999901354313038837062777020037174224853515625f,0.378200002315640659844575566239655017852783203125f,0.128599992239475380273461269098334014415740966796875f,-0.44789999834299087932976135562057606875896453857421875f,-0.62539997094348098727323304046876728534698486328125f,-0.29249998707324287305908683265442959964275360107421875f,-0.5565999961286782760083724497235380113124847412109375f,-0.32399999937415124851014525120262987911701202392578125f,-0.5307999986231326516872286447323858737945556640625f,-0.81100000226497659383539939881302416324615478515625f,0.797699991534650099112013776903040707111358642578125f,0.12780002714395433116578715271316468715667724609375f,0.2830000056028361132831605573301203548908233642578125f,0.401799985539913251386678894050419330596923828125f,-1.3007000407606363889811973422183655202388763427734375f,0.29219999625086767736092951963655650615692138671875f,-0.7242000189974906465550930079189129173755645751953125f,0.54439999041557296521887110429815948009490966796875f,-0.034799994644522325160096443141810595989227294921875f,-0.56759998490810392723915356327779591083526611328125f,-0.4921999982625246961021048264228738844394683837890625f,-0.146600003355741304034154381952248513698577880859375f,-1.3784000263348221704262641651439480483531951904296875f,0.0161999975621700542660619248636066913604736328125f,-0.890300035078823714940199351985938847064971923828125f,-0.30279998199343705067576593137346208095550537109375f,-0.03640001193285036240609997548745013773441314697265625f,0.634999978840351086972759730997495353221893310546875f,-0.41090004995316320446363533847033977508544921875f,-0.1228000041961667676559954998083412647247314453125f,0.28260001627504838239701712154783308506011962890625f,0.576599986666440855742621351964771747589111328125f,-0.10940001286044698236565864135627634823322296142578125f,0.21899999456852670132178673156886361539363861083984375f,-0.574000009898096408278433955274522304534912109375f,0.20359999298453335114800211158581078052520751953125f,-0.2842000001102686912446415590238757431507110595703125f,0.285199985313415727006258748588152229785919189453125f,0.92470001659095280022171436939970590174198150634765625f,0.817499995976686211207606902462430298328399658203125f,0.47580000098869190683359420290798880159854888916015625f,0.9760000161230564241776619383017532527446746826171875f,0.611800001633167056525053340010344982147216796875f,0.48979999747574343160749776870943605899810791015625f,0.275399983516335655764351031393744051456451416015625f,1.1062999831408262263465758223901502788066864013671875f,1.035099970006942982081454829312860965728759765625f,0.8005999890208244096356793306767940521240234375f,0.304600009107589553281059124856255948543548583984375f,0.348799985301494697154112145653925836086273193359375f,0.82029996344149147535773636263911612331867218017578125f,0.015899977421760580487131164773018099367618560791015625f,0.0355000072419642265231232158839702606201171875f,-0.38560001383125774854221390341990627348423004150390625f,0.371099981193989680861022861790843307971954345703125f,-0.1946000157013536568939571225200779736042022705078125f,-0.21489998856782932534770225174725055694580078125f,-0.04309998604059206428473771666176617145538330078125f,-0.147199996995925896925427878159098327159881591796875f,-0.2501999994695187634619060190743766725063323974609375f,-0.56329999653697004902141998172737658023834228515625f,-0.49469999781548990025470402542850933969020843505859375f,-0.149699985857308093262219017560710199177265167236328125f,-0.04579998677670971130027055551181547343730926513671875f,0.256000020146369688944787412765435874462127685546875f,-0.5665999965012076255987949480186216533184051513671875f,-0.8239999831318856049477972192107699811458587646484375f,-0.18609999663531773261837543032015673816204071044921875f,-1.095400014827400791972422666731290519237518310546875f,-0.395599998520314410921372427765163592994213104248046875f,-1.430800031312555287144050453207455575466156005859375f,-0.1796000091075893312364541998249478638172149658203125f,-0.97300001588463747026480632484890520572662353515625f,-0.78659997087121025227673953850171528756618499755859375f,-0.69689995364695767054996622391627170145511627197265625f,-0.00259999742507943665259517729282379150390625f,-0.6260999897062777819201073725707828998565673828125f,0.266399987494945644783683746936731040477752685546875f,-0.03550000485777859982050586040713824331760406494140625f,-0.1571999898433686926324526211828924715518951416015625f,1.1942000265970824646188930273638106882572174072265625f,-0.01980000842809648275988365639932453632354736328125f,1.50180002370551246571039882837794721126556396484375f,-0.16539999897629009684152379122679121792316436767578125f,0.33000001298263670346244680331437848508358001708984375f,-0.432800005537270937594485076260752975940704345703125f,-0.28069998730271972231520294371875934302806854248046875f,-0.399999979138374595066807160037569701671600341796875f,0.45830002261400226704068927574553526937961578369140625f,0.0144000069558618548626327537931501865386962890625f,0.53020000483393658896602573804557323455810546875f,-0.06260000271499144286480031951214186847209930419921875f,0.8753000357493763150529275662847794592380523681640625f,0.17839999470710754536639797152020037174224853515625f,0.70620001097321516425608933786861598491668701171875f,0.372199988204240828082447478664107620716094970703125f,0.0871000092193485730973634417750872671604156494140625f,0.69699999573826787457875298059661872684955596923828125f,0.50079997195005443444415504927746951580047607421875f,-0.269500018388032724914182836073450744152069091796875f,0.0749000125586984921710609341971576213836669921875f,-0.30130000698268422087977569390204735100269317626953125f,0.78629999431669705955982863088138401508331298828125f,0.502499997243285090320341623737476766109466552734375f,0.5687999994575976270283490521251223981380462646484375f,0.22380001569986329723604967512073926627635955810546875f,-0.0414999692738060588226289837621152400970458984375f,0.95570002124011477206977360765449702739715576171875f,0.72409999590516083145530501496978104114532470703125f,0.138400007149577231047032910282723605632781982421875f,-0.88329997999668119490479512023739516735076904296875f,-0.2034000116944316705058781735715456306934356689453125f,0.93890000247061233817902348164352588355541229248046875f,0.617400005927681849726695872959680855274200439453125f,-0.6918000016331671275793269160203635692596435546875f,-0.16659999337196340629674295996665023267269134521484375f,-0.1941999947458505904052117330138571560382843017578125f,-0.61330001248121279733283017776557244360446929931640625f,-0.51999999865889545702657414949499070644378662109375f,0.100799990800023220316461447509936988353729248046875f,1.27969997255802159230597681016661226749420166015625f,0.857799997043609874225467137875966727733612060546875f,0.513399988597631473652427303022705018520355224609375f,0.1510000045001509416664475793368183076381683349609375f,-1.1695000119805332161604383145458996295928955078125f,-0.5051000196278092602142351097427308559417724609375f,-0.5447999757945540810766260619857348501682281494140625f,0.5385999721229079995055144536308944225311279296875f,0.991300021250546503637224304839037358760833740234375f,-0.00920000346302973781575929024256765842437744140625f,1.7506000213935972720236122768255881965160369873046875f,-0.399999987781047838808490269002504646778106689453125f,0.236900008334219425165656502940692007541656494140625f,-0.170399997746944364962473628111183643341064453125f,-0.80190003437399848262856494329753331840038299560546875f,-0.6955999811232089768964215181767940521240234375f,-0.73779997800737628921297073247842490673065185546875f,0.24379998232126265378383322968147695064544677734375f,-0.4823999696433534101203122190781868994235992431640625f,0.8549999916553492340653974679298698902130126953125f,-0.364400000920891642408605548553168773651123046875f,-0.4893999837100506677955991108319722115993499755859375f,-1.2175000068545343001602532240212894976139068603515625f,0.32439999339580527504267593030817806720733642578125f,-0.6098000169962645156118696831981651484966278076171875f,0.139799997103214224125622422434389591217041015625f,0.6186000111266969359036238529370166361331939697265625f,-0.244800003957748035787744811386801302433013916015625f,0.6823000014990565631478602881543338298797607421875f,-0.66939998862743355090287877828814089298248291015625f,-0.3759000210821630272306492770439945161342620849609375f,-0.4963999883890151121335065909079276025295257568359375f,-0.5025000028312207600578176425187848508358001708984375f,0.69759998684525470480366493575274944305419921875f,-0.90290001253634688538340924424119293689727783203125f,0.98659999828934663579360631047165952622890472412109375f,0.708199997100233957780801574699580669403076171875f,-0.1253999975979329661868177936412394046783447265625f,-0.726699981558323049313230512780137360095977783203125f,-0.3637999897718426911552569436025805771350860595703125f,0.04299998757243184144272163393907248973846435546875f,-0.4912000288918616508482273275149054825305938720703125f,0.32720000543370819645616620618966408073902130126953125f,0.260300009690970390874298345806892029941082000732421875f,-0.036000018060207139569683931767940521240234375f,0.08110001101493846675793975009582936763763427734375f,0.1202000177979467565592130995355546474456787109375f,-0.74069999270439179639424764900468289852142333984375f,-0.60299997518584158573418108062469400465488433837890625f,-1.1318999981641770791185308553394861519336700439453125f,-0.76699999409914010328748190659098327159881591796875f,-0.190900004988908467140618085977621376514434814453125f,-0.09830003244876817092290366417728364467620849609375f,0.696799990755319509361243035527877509593963623046875f,-0.04829999385476124729166258475743234157562255859375f,0.44510000822842121781519608703092671930789947509765625f,0.4287000031501053687321700635948218405246734619140625f,0.36219999915659417677460396589594893157482147216796875f,0.50180000200569618851176301177474670112133026123046875f,0.156600008049606997939662278440664522349834442138671875f,-0.8169999750256540238524394226260483264923095703125f,-0.820099987664818730337401575525291264057159423828125f,-0.45719999520778653590014073415659368038177490234375f,-0.780500011265278015315516313421539962291717529296875f,-1.156099987247586113880970515310764312744140625f,0.152799995404481858685130646335892379283905029296875f,0.288099988722800848250926719629205763339996337890625f,0.360999979019165362359444770845584571361541748046875f,0.85770002837777192805646109263761900365352630615234375f,0.83279999138116789225705360877327620983123779296875f,0.140399963492899626515963973361067473888397216796875f,-0.8617999861359593172238646729965694248676300048828125f,-0.34020001517161713167070047347806394100189208984375f,-0.72179997078776381158604635857045650482177734375f,1.20720003495663430470585808507166802883148193359375f,0.10300001022219618107556016184389591217041015625f,1.58340000068619790596358143375255167484283447265625f,0.390199980060756457245219053220353089272975921630859375f,1.2466000079192223193302879735711030662059783935546875f,-0.099399998909234899002740348805673420429229736328125f,-0.65710000713318572618248936123563908040523529052734375f,-0.1069999812841420094855493516661226749420166015625f,-2.0760000109076504060112711158581078052520751953125f,0.026399983173609253839231314486823976039886474609375f,-0.61519998698979561080335543010733090341091156005859375f,0.032599997276067771423413432785309851169586181640625f,-0.27169999966323354900765707498067058622837066650390625f,-0.2323999922931194728192849652259610593318939208984375f,-0.7129999872744081645947744618752039968967437744140625f,0.3113999840676786590165647794492542743682861328125f,0.7395999881342054660393614540225826203823089599609375f,-0.153399987107515567430482406052760779857635498046875f,0.69339998744279096598575051757507026195526123046875f,-0.06599999123811717272047872029361315071582794189453125f,-0.46859998454675089174514823753270320594310760498046875f,-0.8492999927856026420869284265791065990924835205078125f,-0.3367999955236911091560614295303821563720703125f,0.40300000239908673194122457061894237995147705078125f,0.454099997693300228007728946977294981479644775390625f,-0.0482999945253135276601597070111893117427825927734375f,-0.9655999824643135553969841566868126392364501953125f,-1.21009999623298636350909873726777732372283935546875f,-1.01629999856352792875213708612136542797088623046875f,-0.922099986532330273547586330096237361431121826171875f,-0.6181999840617180552726495079696178436279296875f,-0.75149997508525867573325740522705018520355224609375f,-1.1502999700725080600705041433684527873992919921875f,-0.566499994978308496484942224924452602863311767578125f,0.4303999646663669853552391941775567829608917236328125f,-0.30790002300441232652161716032424010336399078369140625f,-0.91930001362860169766832996174343861639499664306640625f,0.161999996483326469398633662422071211040019989013671875f,0.69029997469186810121755115687847137451171875f,0.943699979680776568358169242856092751026153564453125f,0.37869997759461426767302327789366245269775390625f,-0.4048000043302770745157204146380536258220672607421875f,-0.5402999970436095367176676518283784389495849609375f,-0.5041999980241058665342279709875583648681640625f,-0.764300016981362961843160519492812454700469970703125f,-0.2878999951392409162309604653273709118366241455078125f,0.11669999765157701432372050476260483264923095703125f,-1.014000011593103156570805367664434015750885009765625f,-0.4103000363826740937156500876881182193756103515625f,-0.126799962219596551449285470880568027496337890625f,0.03840006504058823821878831950016319751739501953125f,-0.2182000685513012516736353063606657087802886962890625f,0.20319997959137037213395160506479442119598388671875f,-0.4259999529421334418799460763693787157535552978515625f,1.1625000289082532134443681570701301097869873046875f,-0.80980000991821299294315394945442676544189453125f,1.201300035369396113082984811626374721527099609375f,-0.515999989897012678596865953295491635799407958984375f,-0.1325999771595007814539712853729724884033203125f,0.44479998071193715958315806346945464611053466796875f,-1.230999972760676630656462293700315058231353759765625f,1.125799965393543100589113237219862639904022216796875f,-1.95579995853900800995006648008711636066436767578125f,1.347799965080618722623739813570864498615264892578125f,-0.82669999184012343818039880716241896152496337890625f,0.1783999940365552649979008492664434015750885009765625f,-0.44050002199411419923080757143907248973846435546875f,0.79900000630319123473555009695701301097869873046875f,-0.70849999526143125905264241737313568592071533203125f,1.18719997129142296188319960492663085460662841796875f,0.5345000286698340374869076185859739780426025390625f,-1.11440001313984371478227330953814089298248291015625f,0.80760001202821740662329830229282379150390625f,-0.183599971228838310111086684628389775753021240234375f,-0.26870002229809841765018063597381114959716796875f,1.03300000672042369131986561114899814128875732421875f,-1.05590005416274035354717852897010743618011474609375f,-0.17889999084770646931019655312411487102508544921875f,-9.99998837709625831848825328052043914794921875e-05f,-0.9287999914109708043952196021564304828643798828125f,-0.037500000558793544769287109375f,0.243099987530708716576555161736905574798583984375f,-0.0494999991282820683835552699747495353221893310546875f,1.21670000316500637183025901322253048419952392578125f,0.344499995104968481651752654215670190751552581787109375f,1.553099993863700678531358789769001305103302001953125f,0.16629999364614478363222360712825320661067962646484375f,0.74320000127255791966263132053427398204803466796875f,0.2567999971255658930857634913991205394268035888671875f,-0.311099969068170167929565650410950183868408203125f,0.08329999974071977675293965148739516735076904296875f,0.010399996852874959785140163148753345012664794921875f,-0.29090000383406877659808742464520037174224853515625f,-0.27770000043809395240401727278367616236209869384765625f,0.229600010988861402694283242453821003437042236328125f,0.457900033453852717268972583042341284453868865966796875f,0.0079000008389353837401358759962022304534912109375f,-0.005700021240115205056753211465547792613506317138671875f,-0.20830000562667849717257695374428294599056243896484375f,0.1676000021934509476295716012828052043914794921875f,0.287400006598234281085524344234727323055267333984375f,1.6180000104457139542546428856439888477325439453125f,0.210199996973574110281646198927774094045162200927734375f,0.37940000472068735604125322424806654453277587890625f,0.04329999914467330679457290898426435887813568115234375f,0.0848000240743160826895064019481651484966278076171875f,-0.7008999850213530891096525010652840137481689453125f,0.641999974206090229245091904886066913604736328125f,-0.850699966999889145569113679812289774417877197265625f,0.7777999560654169641793487244285643100738525390625f,-0.2881000405788416429686549236066639423370361328125f,-0.7083999666929248206770353135652840137481689453125f,1.06359997972249953335222016903571784496307373046875f,-1.13519996489882490919853808009065687656402587890625f,1.47969998373389177714898323756642639636993408203125f,-0.83539997159540657634124727337621152400970458984375f,1.3246000216245636948997344006784260272979736328125f,-0.07200001563131763759884051978588104248046875f,-0.582099968874455697687153588049113750457763671875f,0.8085999546140438809516126639209687709808349609375f,-0.6208000133752822335964083322323858737945556640625f,0.382400005555152855407641254714690148830413818359375f,0.833000035870820720873553000274114310741424560546875f,-0.71020001951158040487399603080120868980884552001953125f,0.53989999875649863358972879723296500742435455322265625f,-0.4423999794781210415095529242535121738910675048828125f,-0.2141000546902416257211143602035008370876312255859375f,-0.048199961486459219628386563272215425968170166015625f,-0.108899976617097049569338196306489408016204833984375f,-0.2556000073492530333396643982268869876861572265625f,0.79660000350475324015775413499795831739902496337890625f,-0.87759997984170912133095043827779591083526611328125f,-0.2050000175088648735055585348163731396198272705078125f,0.27839999984800822829811295378021895885467529296875f,-1.07939999190568958198355176136828958988189697265625f,-0.25480001848638045292005926967249251902103424072265625f,-0.022399999613314867941138430751379928551614284515380859375f,0.34710000275596986085702155833132565021514892578125f,0.221600000561773757823402775102294981479644775390625f,0.0863000063866372624232781163300387561321258544921875f,-0.010100001075863840327428988530300557613372802734375f,-0.44840001124739625737447568099014461040496826171875f,-0.4112999996438622130057183312601409852504730224609375f,-0.5794999839663501628450603675446473062038421630859375f,0.2401000045403838267787932636565528810024261474609375f,0.842300020274520111485117013216949999332427978515625f,0.18809999669492238894008551142178475856781005859375f,0.460600017556547669528299593366682529449462890625f,0.20720000895410783936512189029599539935588836669921875f,-0.260299971860646461863098011235706508159637451171875f,-0.14909999765604731170043351085041649639606475830078125f,-0.731099990078806438731362504768185317516326904296875f,-0.13799999506026505713407459552399814128875732421875f,-1.61890001528561100485603674314916133880615234375f,-0.297600009308755464321194494914379902184009552001953125f,-0.6570000357478862351712223244248889386653900146484375f,0.059499998383224006470726408224436454474925994873046875f,-0.55629999521076689550369565040455199778079986572265625f,-0.133599999727308738073361382703296840190887451171875f,-0.90530000598430593328203030978329479694366455078125f,-0.161399998670816391399540634665754623711109161376953125f,-0.00380000117123147074238431741832755506038665771484375f,0.2467999945178627196895604356541298329830169677734375f,1.181700002866982845972643190179951488971710205078125f,-0.621100021538883328986457854625768959522247314453125f,0.11919999981224538299073856251197867095470428466796875f,-0.27959999806210344974743975399178452789783477783203125f,0.4692000176936381716785717799211852252483367919921875f,-0.59430000550746964194104293710552155971527099609375f,0.3999999850243332044641419997788034379482269287109375f,-0.572300031673908193852184922434389591217041015625f,0.04579999802708645262327991076745092868804931640625f,-0.15829995377063799111283515230752527713775634765625f,0.0577999547243124300877070709248073399066925048828125f,0.563000032722950294328256859444081783294677734375f,0.385000017508863923154649455682374536991119384765625f,1.303800038051605270084110088646411895751953125f,-0.827800020959973181788882357068359851837158203125f,-0.05029997498989047244322136975824832916259765625f,0.544199990946054423801570010255090892314910888671875f,-0.65929998941421530389561667107045650482177734375f,0.832999966636300381850332996691577136516571044921875f,-1.2588000189781194393390251207165420055389404296875f,1.111999998867511774136573876603506505489349365234375f,-0.7467000062942508176178080248064361512660980224609375f,0.1139999924451113866297191634657792747020721435546875f,-1.1453999716326597191340397330350242555141448974609375f,1.312199958327413096270674941479228436946868896484375f,-1.22839999187588677642679613200016319751739501953125f,0.70979997951984419302107198745943605899810791015625f,-0.151399973338843363279693221556954085826873779296875f,-0.06160002041757051127746080965152941644191741943359375f,1.32250000834465009091900355997495353221893310546875f,-0.8519000064343209555062230720068328082561492919921875f,-0.021800000180304035524159189662896096706390380859375f,-1.0707000364393000069185291067697107791900634765625f,-0.1320000109747052619013629737310111522674560546875f,-0.474300047603250174432787389378063380718231201171875f,-0.10009999757409093756876927727716974914073944091796875f,-0.9788000226289035321514120369101874530315399169921875f,-0.25720000612288718144782251329161226749420166015625f,-0.437700011390447674131110034068115055561065673828125f,0.0731999993354082167940077852108515799045562744140625f,0.686200003671646019398622229346074163913726806640625f,0.18360000267028808718094978758017532527446746826171875f,0.458000027358532069143848275416530668735504150390625f,-0.0781000020965934016459186750580556690692901611328125f,-0.906900029456615630607529965345747768878936767578125f,-0.070900004579126818526901843142695724964141845703125f,0.9528999703288072442575185050372965633869171142578125f,0.18079999530762425052898834110237658023834228515625f,-0.282800022338331569216052230331115424633026123046875f,-0.101300005790591285403934307396411895751953125f,-0.7480000148415566485482486314140260219573974609375f,-0.102099995361268458538006598246283829212188720703125f,-0.139500010117887984772977461034315638244152069091796875f,0.01200000076740981302947375297662802040576934814453125f,0.47870000613033758529724082109169103205204010009765625f,0.144299999584257598872483185914461500942707061767578125f,0.81519999488741146098647050166619010269641876220703125f,-0.006900001330673699584394853445701301097869873046875f,-0.5387000537395483235059145954437553882598876953125f,-0.63399998715519867431567035964690148830413818359375f,0.4827999972671268569257563285646028816699981689453125f,-1.372799998757243233882263666600920259952545166015625f,1.2862000028520821359734327415935695171356201171875f,-0.7598000039577481601327235694043338298797607421875f,0.588999990209936985507965800934471189975738525390625f,-0.34720003216266714929361114627681672573089599609375f,-0.1947999998599285742528763876180164515972137451171875f,-0.5308999874055384537996360450051724910736083984375f,0.526399994051456321386694980901665985584259033203125f,0.008199989053606859812362017692066729068756103515625f,-0.799599998974799941464652874856255948543548583984375f,-0.1216999824523927742347950697876513004302978515625f,0.870599986934661895787712637684307992458343505859375f,-1.00070001446008660650477395392954349517822265625f,0.230000011324882347452103203977458178997039794921875f,0.50169998748153421530560081009753048419952392578125f,-0.5809999802112579825319471638067625463008880615234375f,-0.0071000187560918970319789877976290881633758544921875f,0.49800001595914344942883644762332551181316375732421875f,-0.97909999903440425583767137140966951847076416015625f,0.95219998626708957090158946812152862548828125f,-0.1563000203192228543258579520625062286853790283203125f,-0.448599977859855003625710878623067401349544525146484375f,0.443400001524388631679585159872658550739288330078125f,-0.31279999682009218542333428558777086436748504638671875f,1.0374999725446107934345718604163266718387603759765625f,-1.0319999619126323242568332716473378241062164306640625f,-0.27600007483363153681921176030300557613372802734375f,-1.148200019153952755601721946732141077518463134765625f,-0.2301000033855438442476071259079617448151111602783203125f,-0.8326000077068804738900098527665250003337860107421875f,0.194500004902482093172011445858515799045562744140625f,0.1304000026643279408489206616650335490703582763671875f,0.057699999432265780541229105438105762004852294921875f,0.843299990129469989597055246122181415557861328125f,0.3671999956548213095430810426478274166584014892578125f,0.3870000125020738312997536922921426594257354736328125f,-0.2278999975979327796693496566149406135082244873046875f,0.74620000024437960206569186993874609470367431640625f,0.2716999976888299084265554483863525092601776123046875f,0.8113000174134967767969328633625991642475128173828125f,-0.09489999557137485108881946871406398713588714599609375f,-0.39439999317228735531415395598742179572582244873046875f,-0.057400000693649040617305701061923173256218433380126953125f,0.825699982143193178529827491729520261287689208984375f,0.104299991351365972658271630280069075524806976318359375f,0.1605999759078013955360120235127396881580352783203125f,0.213599997976422262269124985323287546634674072265625f,-0.256399987122416295193261248641647398471832275390625f,-0.093199995945393965257608215324580669403076171875f,0.1156999803364280321460455525084398686885833740234375f,-0.10339999999701976474142384176957421004772186279296875f,-0.9048000072360038448238128694356419146060943603515625f,-0.148900001911818957722744016791693866252899169921875f,0.86449996779859039630622419281280599534511566162109375f,0.31029999577701083968150896907900460064411163330078125f,-0.02800000858306983531065270653925836086273193359375f,0.07879998761117423811128901434130966663360595703125f,0.37720000347793103401272674091160297393798828125f,-0.384999964609741507359785828157328069210052490234375f,0.489599987426399874124172129086218774318695068359375f,-0.93089998591542144623645071987994015216827392578125f,0.9172000095129007046779179290751926600933074951171875f,0.15690001045763413145550657645799219608306884765625f,-0.29760000457763646775077859274460934102535247802734375f,0.0039999816417690681191743351519107818603515625f,-0.1988000098139044435896494178450666368007659912109375f,0.5446999809026724648219897062517702579498291015625f,-1.49699995490908666084806100116111338138580322265625f,-0.13319999330043774676823886693455278873443603515625f,0.4877999781936408485449874206096865236759185791015625f,-1.009200009106844486694853912922553718090057373046875f,0.28579999810159206763415795649052597582340240478515625f,0.77359999378547039672326945947133935987949371337890625f,-0.68659997377693660070008263573981821537017822265625f,0.7907000158011907586796951363794505596160888671875f,-0.2424000030219553014632083431934006512165069580078125f,-1.2950999896764745500377102871425449848175048828125f,1.449999993816017518355465654167346656322479248046875f,-0.247100042262673014192841947078704833984375f,-0.6297999754220249268854558977182023227214813232421875f,-0.6545999981924890409601403007400222122669219970703125f,0.7745999912261962716542029738775454461574554443359375f,0.502199964809417753741627166164107620716094970703125f,-1.27679995372593424463047995232045650482177734375f,-0.41270002554655160764696120168082416057586669921875f,0.4831000255286694855527684921980835497379302978515625f,0.0777000013694167102062948515595053322613239288330078125f,0.56290001749098317329611518289311788976192474365234375f,0.032699999339133499487441980591029278002679347991943359375f,0.4155000323317945554180141698452644050121307373046875f,0.1480000048391521272339588222166639752686023712158203125f,0.95770003102272804884620427401387132704257965087890625f,0.12890000369995835427516794879920780658721923828125f,-0.5530999890208239122557642986066639423370361328125f,-0.1965000019446014878354844768182374536991119384765625f,-0.383700041446089823438114763121120631694793701171875f,-0.1074999997019767672412626779987476766109466552734375f,0.06140003666877813515156958601437509059906005859375f,0.191000014707446208461760761565528810024261474609375f,-1.0505999987065788303652880131267011165618896484375f,-0.1553000117585063843961279417271725833415985107421875f,-1.5119000655919325737386316177435219287872314453125f,-0.295800008867681096802471074624918401241302490234375f,-2.01510007535815294232861560885794460773468017578125f,-0.196100007103383600082224802463315427303314208984375f,-0.761700064706803292580161723890341818332672119140625f,-0.278300010804832076427572928878362290561199188232421875f,-0.1103000352650884696714683741447515785694122314453125f,0.1822000052288175186987473352928645908832550048828125f,0.7593000403761867023177956070867367088794708251953125f,0.06250000637024644145611773637938313186168670654296875f,0.00310002336800152544782349650631658732891082763671875f,0.06170000271797182467281572826323099434375762939453125f,0.053800000351667431885971382143907248973846435546875f,-0.177099997335672387777094627381302416324615478515625f,0.0203999979704618805698146388749592006206512451171875f,-0.4940000123567880319086498275282792747020721435546875f,0.03460000016689301904904141338192857801914215087890625f,-0.170899953971058560231455203393124975264072418212890625f,0.310199951785803396120400066138245165348052978515625f,0.91570001300722381198937682711402885615825653076171875f,-1.362999994277954041166367460391484200954437255859375f,-1.00929998017549404920600863988511264324188232421875f,1.63840000275373398608280695043504238128662109375f,-0.7367000334143636308681379887275397777557373046875f,-0.220999991759657898882096560555510222911834716796875f,0.6679000624179849410211318172514438629150390625f,0.25439996716976143176225377828814089298248291015625f,-0.89229999084472666481815394945442676544189453125f,1.3668000098288057841955378535203635692596435546875f,-1.9342000244736681224821950308978557586669921875f,1.749199979993701248304205364547669887542724609375f,-2.4918000525951384815925848670303821563720703125f,1.63400002083182283030282633262686431407928466796875f,-1.3698000804007062214395773480646312236785888671875f,0.567600024545192649583214006270281970500946044921875f,0.3791999674022203947743037133477628231048583984375f,-1.0761999684304004443191615791874937713146209716796875f,0.58610003038644808981416645110584795475006103515625f,0.2549999767541886175337140230112709105014801025390625f,0.253000010371208450266067302436567842960357666015625f,-0.31499999813735490850064024925814010202884674072265625f,-0.37910002466440229085264945751987397670745849609375f,0.09600002633035220611645854660309851169586181640625f,0.07390000016093256629545749092358164489269256591796875f,0.814200012850761378757624697755090892314910888671875f,0.1029000012859702106737813664949499070644378662109375f,1.41090001940578257944025608594529330730438232421875f,0.25190000522360211920869232926634140312671661376953125f,0.6678000233814120889519472257234156131744384765625f,-0.014500001549720753502015213598497211933135986328125f,-0.0416000247389083366300610578036867082118988037109375f,0.1232999990701675041560037016097339801490306854248046875f,-0.280199990230798778867438159068115055561065673828125f,-0.1260999987959861290409691036984440870583057403564453125f,-0.6824000044375650997352522608707658946514129638671875f,-0.3530999956145881579772094482905231416225433349609375f,-0.059899989014863042058323117089457809925079345703125f,0.27680000781714919622089610129478387534618377685546875f,0.50800003428757245504954198622726835310459136962890625f,-0.15150000036135313763452359125949442386627197265625f,-0.311700027249008859708823138134903274476528167724609375f,0.136799992096424016807532098027877509593963623046875f,-1.0558000311821709171766769941314123570919036865234375f,-0.1173000014990568384831703951931558549404144287109375f,-1.013800029185414341981186225893907248973846435546875f,-0.1263000013202428828407164473901502788066864013671875f,-0.815400020620227206080699033918790519237518310546875f,0.1352999997258186315463035498396493494510650634765625f,0.5634000064790247730428518480039201676845550537109375f,-0.00180000118613243831333647904102690517902374267578125f,0.3222000103324649256109069028752855956554412841796875f,0.39970001368522645179837127216160297393798828125f,-0.57179999962151040193703011027537286281585693359375f,0.5432000126719473964698181589483283460140228271484375f,-0.5626000059187410418104491327540017664432525634765625f,1.561799990904332080532412874163128435611724853515625f,-1.68299996015429531581730770994909107685089111328125f,1.03770002327412402820527859148569405078887939453125f,0.290000010207295044750708257197402417659759521484375f,0.83540000348389131801241092034615576267242431640625f,-0.5261999906331300547179807836073450744152069091796875f,-0.118699993538856407582215979346074163913726806640625f,0.2921999902158975981336652694153599441051483154296875f,-0.606200003448127855421034837490878999233245849609375f,0.163199991065263727563205975457094609737396240234375f,0.3261000309079893355601598159410059452056884765625f,-0.805000007599592226625873081502504646778106689453125f,0.08740001917108930484801021520979702472686767578125f,0.37459999375939367904919663487817160785198211669921875f,0.36220003016963564501651262617087922990322113037109375f,0.40780000225901602561862091533839702606201171875f,-0.49269998143911397647798366961069405078887939453125f,1.2921999794870615563269211634178645908832550048828125f,-0.493100003325938729403787874616682529449462890625f,0.771000012621283037361763490480370819568634033203125f,-0.483899992859363692332408390939235687255859375f,0.236199991229176564644376412616111338138580322265625f,0.0167000083804129673126226407475769519805908203125f,0.0959999957084656241335096638067625463008880615234375f,0.471600039044022256717880736687220633029937744140625f,0.7185999987214801709711764488019980490207672119140625f,0.311899998313188486775970886810682713985443115234375f,0.8992000235050914280776623854762874543666839599609375f,0.226400001539289963403689398546703159809112548828125f,0.403500017538666266858626840985380113124847412109375f,0.1884999988377094037872439002967439591884613037109375f,0.14400000928342360229095220347517170011997222900390625f,-0.1481999974913894846384465608934988267719745635986328125f,0.464300016888231414924348428030498325824737548828125f,0.301900004571676261466706137071014381945133209228515625f,0.226800033000111422865074928267858922481536865234375f,-0.1026000038698315908192171264090575277805328369140625f,-0.47050001960992826610663541941903531551361083984375f,0.116299998451769359686380767016089521348476409912109375f,1.432300033015013074333410258986987173557281494140625f,0.220000014081597417003877126262523233890533447265625f,0.35700005623698327639203853323124349117279052734375f,0.019300001297891167251208344168844632804393768310546875f,-1.1825000095367432795256945610162802040576934814453125f,-0.100100000293552870989088887654361315071582794189453125f,-1.274800031971931701946232351474463939666748046875f,-0.2275000081211329128194620352587662637233734130859375f,-0.0124000009357929652509255902259610593318939208984375f,0.27980001338273297051273402757942676544189453125f,0.30880005395859566608152135813725180923938751220703125f,0.006700001525878906960542735760100185871124267578125f,-0.1161999942839144761563829888473264873027801513671875f,0.05940000155419113136989750501015805639326572418212890625f,0.97699999450892172081495345992152579128742218017578125f,0.5731999950513240182914387332857586443424224853515625f,-0.8125999828964471927150725605315528810024261474609375f,0.15560001405477574820679365075193345546722412109375f,0.5038000126451247506764730132999829947948455810546875f,-0.7367999727249152641661567031405866146087646484375f,0.660199963632226438647876420873217284679412841796875f,-0.831800002322345743976939047570340335369110107421875f,0.40439998974502089357230261157383210957050323486328125f,0.34610001654699462658726361041772179305553436279296875f,-0.5874000020533800547894998089759610593318939208984375f,0.01440003646016041471966673270799219608306884765625f,0.84139996536672168048198727774433791637420654296875f,-0.45949998575448969262424725457094609737396240234375f,-0.01699999767541893191946655861102044582366943359375f,1.241800025773049043209539377130568027496337890625f,-1.14340001318454742573749172152020037174224853515625f,1.336800054532290005226968787610530853271484375f,0.01879999617934213773651208612136542797088623046875f,-0.62549996432662002820990210238960571587085723876953125f,1.25239997329413910165385459549725055694580078125f,-1.4670000028163199257136284359148703515529632568359375f,1.46139998920857916431259582168422639369964599609375f,0.45030002658516199431915083550848066806793212890625f,-0.8434000234663485695563167610089294612407684326171875f,0.4893000427231195548216646784567274153232574462890625f,0.5209999886304139682380309750442393124103546142578125f,0.225500014431774697953869690536521375179290771484375f,-0.1754000038564205166125020696199499070644378662109375f,0.8687999994948503168501474647200666368007659912109375f,0.0633999806255101105989524512551724910736083984375f,-0.307699997681379233682719132048077881336212158203125f,-0.643000034660100805439242321881465613842010498046875f,0.1894999979361891018214691939647309482097625732421875f,-0.4205000287741429421117800302454270422458648681640625f,-0.2424000037297606713337927430984564125537872314453125f,0.49049999159574475537937132685328833758831024169921875f,0.257099997950345215969036871683783829212188720703125f,0.049100003597884966932696215735631994903087615966796875f,-0.3942000014886259950941393981338478624820709228515625f,-0.64270001273155175081086554200737737119197845458984375f,0.1423000009961426226556824303770554251968860626220703125f,0.469300014317780511685640476571279577910900115966796875f,-0.0350000008195638923069736847537569701671600341796875f,-0.12379998723864549248219191213138401508331298828125f,0.058300000225007531096110824364586733281612396240234375f,0.3119000132143499026682320618419907987117767333984375f,0.0237000026658177598282151166131370700895786285400390625f,-0.05329999683499320184409953071735799312591552734375f,-0.00809999960064888835375995768117718398571014404296875f,0.46600002200901524673781750607304275035858154296875f,0.18050000329315663005758096915087662637233734130859375f,-0.0877999601632353243729767200420610606670379638671875f,-0.0787999963656067503592339562601409852504730224609375f,0.498899998745322204030117063666693866252899169921875f,0.2144999948441981274527279310859739780426025390625f,0.5270000210702416865160557790659368038177490234375f,-0.1572000009447336033030495627826894633471965789794921875f,-1.193000012457370306862003417336381971836090087890625f,-0.05380001912712994993626125506125390529632568359375f,-0.261199995252490158037517176126129925251007080078125f,0.5637000149965292195020083454437553882598876953125f,0.346600026303529151761040338897146284580230712890625f,-0.09350007349252731359001700184307992458343505859375f,-0.3123999637573959820002755805035121738910675048828125f,0.467199985168128772983209273661486804485321044921875f,-0.64839998755455019452398346402333118021488189697265625f,0.23799998472258387938182977450196631252765655517578125f,0.095199985536933429131067896378226578235626220703125f,0.36540001423135404223785371868871152400970458984375f,-0.375199998426437364429375520558096468448638916015625f,0.73969999906346128870637812724453397095203399658203125f,-0.555200008857249738269956651492975652217864990234375f,0.0947000179320571788821325753815472126007080078125f,0.120999989226460602509405362070538103580474853515625f,0.1339000130131842158931476660654880106449127197265625f,-0.0402000070691108657428003425593487918376922607421875f,0.082999992117285703585594092146493494510650634765625f,-0.2353999955862760007363476688624359667301177978515625f,0.020300006356835265108884414075873792171478271484375f,-0.1672000082463025449897031649015843868255615234375f,-0.94449996429681792875499013462103903293609619140625f,1.2887999545305970361397385204327292740345001220703125f,0.0706999766111362504261705907993018627166748046875f,-1.260399993053078038229841695283539593219757080078125f,0.03759999973475913748188759200274944305419921875f,0.4453999943196773614317862666212022304534912109375f,-0.803999989520757907257575425319373607635498046875f,0.5225999898999926784881608909927308559417724609375f,0.2879000050485134298838829636224545538425445556640625f,0.271000015527010085225612101567094214260578155517578125f,-0.064499998979270445342848461223184131085872650146484375f,0.79430000781714937829747213982045650482177734375f,0.155500004410743741800615680404007434844970703125f,0.934200011956691778181038898765109479427337646484375f,0.2150999999955296448916897134040482342243194580078125f,1.589699997219442995088911629864014685153961181640625f,0.1494999936521053118809732040972448885440826416015625f,1.1765999978423111738123907343833707273006439208984375f,0.1414000022470951190012300457965466193854808807373046875f,1.0102000372439625675013985528494231402873992919921875f,0.3814000071644783051993954359204508364200592041015625f,0.250400028890371828538263798691332340240478515625f,-0.44240000004172319592044004821218550205230712890625f,-0.9313999951690432599349378506303764879703521728515625f,0.2713000069081783482971559351426549255847930908203125f,1.519000030070543516558245755732059478759765625f,0.2155999993771313683055268484167754650115966796875f,0.0622000254571435373662779966252855956554412841796875f,0.0664999991878867380279416465782560408115386962890625f,0.49330000585019551095911083393730223178863525390625f,0.2259999989494680949775329281692393124103546142578125f,1.627900003558397035163807231583632528781890869140625f,0.1666999940380453626431744851288385689258575439453125f,0.280299991902708267588195667485706508159637451171875f,0.1307000038802623720357587444595992565155029296875f,1.335800029915571673910790195805020630359649658203125f,0.025200016605854358431315631605684757232666015625f,0.0365999932974574448962812311947345733642578125f,0.2841999976515772186047570357914082705974578857421875f,-0.074399992650747426292667796587920747697353363037109375f,1.648100001761317656701066880486905574798583984375f,-1.435799978581071112415656898519955575466156005859375f,1.8263999919652942338643697439692914485931396484375f,-1.556999960720539366576531392638571560382843017578125f,1.2167999622941021442557030241005122661590576171875f,-1.358999939695001213380010085529647767543792724609375f,0.68009999869167803154823559452779591083526611328125f,-0.4492000012278556386746686257538385689258575439453125f,0.297400025746226592815446565509773790836334228515625f,-0.06540000728368777771493114414624869823455810546875f,-0.1427000282287593790897517465054988861083984375f,0.2522000111520288356814489816315472126007080078125f,-0.25549994510412243897690132143907248973846435546875f,0.3649999482184653754046621543238870799541473388671875f,1.4632999889373774493606106261722743511199951171875f,-1.2887999723374843341616724501363933086395263671875f,0.3337000058323145168515111436136066913604736328125f,0.2964000075370070153013557501253671944141387939453125f,-0.7875999855414039796386305170017294585704803466796875f,0.33719997129142331715456748497672379016876220703125f,0.99079998729825025094442025874741375446319580078125f,-1.076599957758188619294514865032397210597991943359375f,0.4059999948143957482216137577779591083526611328125f,-0.193800003853440205858760236878879368305206298828125f,0.3470999593377115388648235239088535308837890625f,-0.52100003147125306668385746888816356658935546875f,-0.3295000082179904143941939764772541821002960205078125f,-0.208900012528896450447746246936731040477752685546875f,0.235300007958710255540069056223728694021701812744140625f,-0.16349996639043073987096477139857597649097442626953125f,0.0421999993242323234721169455951894633471965789794921875f,-0.779099988119303699107831562287174165248870849609375f,-0.1088999961376189151707194469054229557514190673828125f,0.587999979600310584970657146186567842960357666015625f,0.1023999976202845108019801045884378254413604736328125f,0.6037000193178652107661719128373079001903533935546875f,-0.164000000938773149439242615699185989797115325927734375f,0.50949999916553512235850575962103903293609619140625f,0.1305999995633959631646803245530463755130767822265625f,0.682499998584389810929451414267532527446746826171875f,-0.1369000005483627102620403093169443309307098388671875f,-1.1294000012934211074622226078645326197147369384765625f,-0.16800000008940696449855067839962430298328399658203125f,-1.268000018939375639348554614116437733173370361328125f,-0.418300002944469451193754139239899814128875732421875f,-1.909400026774406011753626444260589778423309326171875f,-0.30489999642819165526219649109407328069210052490234375f,-1.0081000183761112953817473680828697979450225830078125f,-0.1989999981448054011679005270707421004772186279296875f,-1.1537000204354523891225881015998311340808868408203125f,-0.247300003547966495975884981817216612398624420166015625f,0.14779997234493447155756484789890237152576446533203125f,0.283399996607005544291268961387686431407928466796875f,0.5307000169932845068387905484996736049652099609375f,0.225499979376792225593817420303821563720703125f,-0.0891999991416927429099814617075026035308837890625f,1.1438000617444519235732514061965048313140869140625f,-0.567200025010108799250474476139061152935028076171875f,0.2064000612929459688160704899928532540798187255859375f,0.66859996456056880465013136927154846489429473876953125f,-1.501399964156000965687098869238980114459991455078125f,1.178599972793459738085175558808259665966033935546875f,-0.1684000349402428042822066345252096652984619140625f,-1.203199977281689481145576792187057435512542724609375f,0.42799998474121025981986576880444772541522979736328125f,-0.42340001012980887917791505969944410026073455810546875f,0.8854999898821114268088194876327179372310638427734375f,-1.33639998295009121420662268064916133880615234375f,0.9652999708175666881970755639486014842987060546875f,-0.623199956867099391644160277792252600193023681640625f,-0.6631999705016620172415287015610374510288238525390625f,1.2989999470710760220981683232821524143218994140625f,-0.754100006335974182292147816042415797710418701171875f,0.348399981519580403954705616342835128307342529296875f,-0.6192999945551154183931430452503263950347900390625f,0.801999982476234496431288789608515799045562744140625f,0.189199971202016481441887663095258176326751708984375f,-0.488399973919988017456716988817788660526275634765625f,-0.07150000730901950873885652981698513031005859375f,0.5276000033110379572320880470215342938899993896484375f,0.5320999856010073525425241314223967492580413818359375f,-1.326399985632300104754222047631628811359405517578125f,-0.13280001611709479902856401167809963226318359375f,0.9480000085085631411629947251640260219573974609375f,0.26329999951720228779095123172737658023834228515625f,-0.4779999833256010077775499667041003704071044921875f,-0.161699999216198875284788982753525488078594207763671875f,-0.65900002090632892048915891791693866252899169921875f,-0.1975000023096800116917393097537569701671600341796875f,-0.9398000283211469518818148571881465613842010498046875f,-0.23000000786036256528177545988000929355621337890625f,0.368699987950920959889344885596074163913726806640625f,0.16600001109391460119013572693802416324615478515625f,1.02110006264746289161848835647106170654296875f,0.19070000529587272097842287621460855007171630859375f,0.71460003049075648817733963369391858577728271484375f,0.020500000461936007667418380151502788066864013671875f,-0.055400019279122414417315667378716170787811279296875f,-0.321400007797777720330856254804530180990695953369140625f,-1.0343000213772064466155597983743064105510711669921875f,0.10610000140368935905854641532641835510730743408203125f,0.8620000198036432692560993018560111522674560546875f,0.08919999634772535213045330237946473062038421630859375f,0.5817000083804126919773125337087549269199371337890625f,0.2681000004574656259137555025517940521240234375f,-0.07069997988939269450980873443768359720706939697265625f,-0.197899994822591462906302695046178996562957763671875f,-0.7375000081025058218386902808560989797115325927734375f,-0.1590000033974647564161131185755948536098003387451171875f,-0.7086000323981049842103629998746328055858612060546875f,-0.115800005775690106890607466993969865143299102783203125f,-0.762800022226572504990826928406022489070892333984375f,0.328299973738193529015916283242404460906982421875f,-0.0347999661833053863801978877745568752288818359375f,-0.678000016182661369157358421944081783294677734375f,0.7167999869555234937479326617904007434844970703125f,-1.55620003839135190304432398988865315914154052734375f,0.493799996849894728967456103418953716754913330078125f,-1.81400003677606580509973355219699442386627197265625f,1.3592000138193365277317070649587549269199371337890625f,0.4059999924302104545859037898480892181396484375f,-1.317600007781386306504600725020281970500946044921875f,1.259700043189525775488846193184144794940948486328125f,-0.82540001465976209349406644832924939692020416259765625f,0.21809999587535866538701156969182193279266357421875f,-0.57739999430477606523481881595216691493988037109375f,-0.4120000446140767991209941101260483264923095703125f,0.553000024601816964064937565126456320285797119140625f,-0.84500000618398285467947061988525092601776123046875f,0.4231999696075920969207118105259723961353302001953125f,0.0314999898746604056753994882456026971340179443359375f,-0.51919999400079230866822399548254907131195068359375f,0.18880002583265298454762159963138401508331298828125f,0.2657999945998190671758720782236196100711822509765625f,-0.857100000595301114714175128028728067874908447265625f,0.60979997892379766977200006294879131019115447998046875f,-1.157200010872632134351079002954065799713134765625f,0.59360000147819513838243210557266138494014739990234375f,-1.064200014415383588328722908045165240764617919921875f,0.63159999828934676457947716698981821537017822265625f,-0.88029996947646171889800825738348066806793212890625f,0.62230001748055219223942913231439888477325439453125f,0.0151999993391334908832135397460660897195339202880859375f,0.61910001315325491333396712434478104114532470703125f,0.0839999983683228368391837648232467472553253173828125f,0.809699999231100253638260255684144794940948486328125f,-0.146700000743567926519972388632595539093017578125f,0.3958999821901318938444092054851353168487548828125f,0.3194999982342123789891275009722448885440826416015625f,1.4989000360727307015196174688753671944141387939453125f,0.2032000043272972344965410229633562266826629638671875f,1.2580000093281269446521264399052597582340240478515625f,0.24159999516010277620381430097040720283985137939453125f,0.79640002169310974977634032256901264190673828125f,0.11950000244379044289644298260100185871124267578125f,-0.03820000745654128504469326799153350293636322021484375f,0.10929999591484662602169919409789144992828369140625f,-0.185500010725111297293921097661950625479221343994140625f,-0.203000002473592733309715185896493494510650634765625f,-0.9164000336885449460311292568803764879703521728515625f,-0.21279999734163279612175756483338773250579833984375f,-0.862200008395313854947517029359005391597747802734375f,-0.136399999378621572532210848294198513031005859375f,-0.9113000089943408266179858401301316916942596435546875f,-0.091899998126924009600458020941005088388919830322265625f,0.2456000408023604197893519085482694208621978759765625f,0.231200012686848754839274988626129925251007080078125f,-0.62199997718632149545925358324893750250339508056640625f,-0.1903000012904405569003074560896493494510650634765625f,-0.0854000134676693445356931988499127328395843505859375f,0.7812999911129470120840778690762817859649658203125f,-0.7760000018179413761032492402591742575168609619140625f,0.56279999760240262407506861563888378441333770751953125f,-0.972800001737474939744743096525780856609344482421875f,0.86499998003244460420546602108515799045562744140625f,-0.642199966895580676151666921214200556278228759765625f,1.031299988058209660124475703923963010311126708984375f,-1.2375999649405482472275252803228795528411865234375f,0.58990002522468554246870553470216691493988037109375f,-0.1588000198721883915453645386151038110256195068359375f,0.8889999896883964414229239991982467472553253173828125f,-0.9313999627590181518144163419492542743682861328125f,0.991500043481587933769105802639387547969818115234375f,0.3219999721199275910521464538760483264923095703125f,0.787500019241124338265080950804986059665679931640625f,-0.36839998561739928373270913652959279716014862060546875f,0.37590001160129882240568122142576612532138824462890625f,0.05539998880624796839811097015626728534698486328125f,-0.04010001202821644739060502615757286548614501953125f,-0.1439999846220019197318151782383210957050323486328125f,-0.2654999948740002224667478003539144992828369140625f,0.788199995908140937928010316682048141956329345703125f,-0.34460000016689307233974659538944251835346221923828125f,0.0655999874562026852320428815801278688013553619384765625f,0.4012000682681808871876683042501099407672882080078125f,0.0887999678671358427806126201176084578037261962890625f,-0.720199976074695324967933629523031413555145263671875f,0.98579998349845421312664939250680617988109588623046875f,-0.5211000044584270796121927560307085514068603515625f,0.144199977087974051670471453689970076084136962890625f,0.33389999095946543139490358953480608761310577392578125f,1.250900012961029261759904329665005207061767578125f,0.1258000064089894731722552023711614310741424560546875f,0.44390002795159855253359637572430074214935302734375f,0.2164000053390860711655108161721727810800075531005859375f,0.61290003492534184115214657140313647687435150146484375f,0.13440000300705434721493247707257978618144989013671875f,0.212400033345819139185550739057362079620361328125f,-0.27259999157637349043170615914277732372283935546875f,0.115500033542514923823318895301781594753265380859375f,0.30930000208020214103044054354541003704071044921875f,0.313400024211407224328240772592835128307342529296875f,-0.0564999930039047359997539388132281601428985595703125f,0.8131000257521876495303558840532787144184112548828125f,0.15030000896453865610880029635154642164707183837890625f,-0.15959998884201009872185750282369554042816162109375f,-0.203800006759166763004031963646411895751953125f,-0.29440002200603510029708331785514019429683685302734375f,-0.0390999995931982924357583897290169261395931243896484375f,-0.4597000107049942219106242191628552973270416259765625f,-0.2062000047519803036522034744848497211933135986328125f,0.168699956881999124647109056240878999233245849609375f,0.2516999954536556227679966468713246285915374755859375f,0.9167000146389001624669390366761945188045501708984375f,0.019799998369812943366241597686894237995147705078125f,-1.00740003610253392452023035730235278606414794921875f,-0.230600012341141802352240119944326579570770263671875f,-0.57680007204115479879646954941563308238983154296875f,-0.01769999663829846525686662062071263790130615234375f,-0.318199982869625142001268613967113196849822998046875f,0.6005000437498093646127017564140260219573974609375f,0.24979997124969965938134919269941747188568115234375f,1.4109000418692829725841875188052654266357421875f,-0.80220000332891938654711339040659368038177490234375f,1.1406000223249197489394646254368126392364501953125f,-0.71920000048279753013247272974695079028606414794921875f,0.46150000011920955245159348123706877231597900390625f,-0.40540001883208720556694970582611858844757080078125f,0.4680000392794614327840463374741375446319580078125f,-0.724400000622868578403767969575710594654083251953125f,-1.0176999751806246052865390083752572536468505859375f,1.280799987447261312212276607169769704341888427734375f,0.2452999913811690912979202039423398673534393310546875f,-1.303199994269013561876136009232141077518463134765625f,-0.493400015121698398701255428022705018520355224609375f,0.67760001038908956871864575077779591083526611328125f,-0.854900023213029047752797850989736616611480712890625f,0.22179999626874924434360991654102690517902374267578125f,-0.232099990555644009049274245626293122768402099609375f,0.611199996891617747252212211606092751026153564453125f,1.405399990594386139264315715990960597991943359375f,-1.622399982905387449960699086659587919712066650390625f,1.50629999543428372277276139357127249240875244140625f,-0.7871999862670897574190576051478274166584014892578125f,-1.15260003184676218523918578284792602062225341796875f,1.43260001694560035190306734875775873661041259765625f,-1.224900079762935689586811349727213382720947265625f,0.818700008589029248895485579851083457469940185546875f,0.2740000038444996288689026187057606875896453857421875f,0.3570000357478857466730914893560111522674560546875f,-0.0911000018134713318573858487070538103580474853515625f,-0.333300011512636995547609330969862639904022216796875f,0.2842000025317072964270437296363525092601776123046875f,1.009100015500187463857173497672192752361297607421875f,-0.142700000736117382782452978062792681157588958740234375f,-0.778700028109550412835915267351083457469940185546875f,-0.2691000048086047424789057913585565984249114990234375f,-0.35329998819231978046673248172737658023834228515625f,0.1957000133052468893168907015933655202388763427734375f,-0.6966999482914797425792130525223910808563232421875f,-0.0676999984078109073859508271198137663304805755615234375f,-0.9358000122204419568561206688173115253448486328125f,-0.2395000018849968714818032822222448885440826416015625f,-1.324300081801415007731748119113035500049591064453125f,-0.1911000111266971668300129749695770442485809326171875f,0.122499979361891053741828727652318775653839111328125f,-0.0756000031396746319245494305505417287349700927734375f,0.64469999870955951859485821842099539935588836669921875f,0.1705000043362379369416004237791639752686023712158203125f,0.823200031745434213092948994017206132411956787109375f,0.0892000049903989289390437988913618028163909912109375f,-0.46469997933804929601819821982644498348236083984375f,-0.232299998965859344490780813430319540202617645263671875f,-0.68680002234578108044615873950533568859100341796875f,-0.13609999950379136901545962246018461883068084716796875f,-1.365300029975176077101650662370957434177398681640625f,0.44749999977648258209228515625f,-0.753200001198053303852475437452085316181182861328125f,-0.317899965038896059610351585433818399906158447265625f,1.1007999661386016487796268847887404263019561767578125f,-0.7251999853134147855371338664554059505462646484375f,0.362800010827183161410403045010752975940704345703125f,0.60679997909516014686914786580018699169158935546875f,-0.70100000781565852125964966035098768770694732666015625f,0.63969996564760822632678127774852328002452850341796875f,-0.391399961715936939299353980459272861480712890625f,0.69750004231929807474443805404007434844970703125f,0.31459996544718737965240507037378847599029541015625f,-0.344199954587221412793951458297669887542724609375f,0.8881999639451507722043288595159538090229034423828125f,-0.67659998305290891540408892979030497372150421142578125f,0.964999994933604998692544540972448885440826416015625f,-0.796300086033344456382110365666449069976806640625f,-0.345599942529201786811654528719373047351837158203125f,-0.6182000377058987083955798880197107791900634765625f,-0.154599999347329042365117857116274535655975341796875f,0.220200021001696999434216195368207991123199462890625f,-0.40720000809729095525568709490471519529819488525390625f,0.7041000128179792927340940877911634743213653564453125f,-0.683999998107552631410044341464526951313018798828125f,-0.764099982941150113191497439402155578136444091796875f,0.86779998266398905570895294658839702606201171875f,-0.55039999938607220286712617962621152400970458984375f,0.649799987864494443812191093456931412220001220703125f,-0.1058000270843511003704406903125345706939697265625f,-0.56479999710321437955684586995630525052547454833984375f,-0.036899996860325356973220323197892867028713226318359375f,-0.971599997618793675968618117622099816799163818359375f,-0.27979999401122324087509696255438029766082763671875f,-0.3322999931544059482035891051054932177066802978515625f,0.15560000433176757184128291555680334568023681640625f,-0.29529997662901852351069464930333197116851806640625f,-0.1583000020131468676964914266136474907398223876953125f,-1.169500043347478257516058874898590147495269775390625f,-0.28540000806599863381052273325622081756591796875f,-1.6581000410258770472182732191868126392364501953125f,-0.330299995665252144050327842705883085727691650390625f,-1.72030000367462587718137001502327620983123779296875f,-0.2645999988630414190282635900075547397136688232421875f,-0.60550002609193331881698441065964289009571075439453125f,0.20789999275505532949637199635617434978485107421875f,-0.317600038403273998710574232973158359527587890625f,-0.2169000121340156628235718017094768583774566650390625f,-0.02550005596876214752910527749918401241302490234375f,0.107000000879168499778870682348497211933135986328125f,1.3136000179439779600443216622807085514068603515625f,0.21829999549388878765654453673050738871097564697265625f,0.049199992808699022361906827427446842193603515625f,-0.184299999696016347794369494295096956193447113037109375f,-1.4836000434249643209483338068821467459201812744140625f,-0.312600012363493551248438961920328438282012939453125f,-0.194100024813414595570293386117555201053619384765625f,0.2217000007435678821110514036263339221477508544921875f,-0.9055000203549867787700122789829038083553314208984375f,0.30090003885179772424862676416523754596710205078125f,0.901199993538856602981468313373625278472900390625f,0.5737000062048436266337603228748776018619537353515625f,0.59190000263452535733676995732821524143218994140625f,0.30009996616989342843595522936084307730197906494140625f,-0.4974000043630599776633971487171947956085205078125f,0.403300001603364766111781136714853346347808837890625f,-0.351099999094009351807699204073287546634674072265625f,0.5230000041127198517898477803100831806659698486328125f,-0.095500008136033986971824560896493494510650634765625f,0.5594999961853026615443695845897309482097625732421875f,-0.17499999791383746838846491300500929355621337890625f,0.17279998147189623569630612109904177486896514892578125f,-0.751000021040439502684193939785473048686981201171875f,0.24160003926754003433785555898793973028659820556640625f,1.05009997522234943545527130481787025928497314453125f,-0.2750999760419130524269348825328052043914794921875f,0.0610000034570692406532543827779591083526611328125f,-0.27599998784810253482646658085286617279052734375f,0.218900001204013872069253920926712453365325927734375f,-1.00469998436719176737597081228159368038177490234375f,0.51389997173696766452621886855922639369964599609375f,-1.004799997624754581693196087144315242767333984375f,0.4241999772369862142795682302676141262054443359375f,-0.293699981282651290115381925716064870357513427734375f,0.4439999880492688788535815547220408916473388671875f,-0.25739998348280812567878683694289065897464752197265625f,-0.0174999997578561272693065120620303787291049957275390625f,0.81840001260712735842162146582268178462982177734375f,-0.0919999851211901198411169389146380126476287841796875f,0.9546000203952191842660113252350129187107086181640625f,-1.2437000037088992154821198710124008357524871826171875f,0.79340005609989106005741632543504238128662109375f,-0.3003000097095966935967226163484156131744384765625f,-0.732499994188547276507961214520037174224853515625f,0.2287000443518163450562497018836438655853271484375f,0.7275999920606608384332503192126750946044921875f,0.769700026537477821619859241764061152935028076171875f,-0.518899997925758693639863849966786801815032958984375f,-1.0950000078603625563999912628787569701671600341796875f,-0.38199996694177418277149627101607620716094970703125f,-0.289900027646124325286791645339690148830413818359375f,0.5332000204205513682609307579696178436279296875f,-1.0079000309765344667312092497013509273529052734375f,-0.0532999993681908534881586092524230480194091796875f,0.617200023519993035137076731189154088497161865234375f,-2.2210999526023869066193583421409130096435546875f,0.38960003026723910579676157794892787933349609375f,0.1231999643921852793937432579696178436279296875f,-1.06530000538825930789244011975824832916259765625f,-1.12779997469186810121755115687847137451171875f,0.50539997301101635684972279705107212066650390625f,-1.95370002274513154816304449923336505889892578125f,1.213900008058548252165564917959272861480712890625f,0.474799948525429726942093111574649810791015625f,0.154599993312359629271668381989002227783203125f,-1.13769998054504384299434605054557323455810546875f,-0.809199977385997470946676912717521190643310546875f,0.82850001072138590441085170823498629033565521240234375f,0.571100004737824296086046160780824720859527587890625f,0.8417999955050650129351197392679750919342041015625f,0.70390001714825611855985698639415204524993896484375f,0.27859995088875333379974108538590371608734130859375f,-0.105800007116794603234666283242404460906982421875f,-0.4959000164628026841029395654913969337940216064453125f,0.73419997500181199967528300476260483264923095703125f,0.1258999857664111488020353135652840137481689453125f,-0.2984000043556094272645395903964526951313018798828125f,1.02759997689873028292595336097292602062225341796875f,-0.043499988704919889670463817310519516468048095703125f,1.7141000083848838553279847474186681210994720458984375f,0.12210001477003107339669440989382565021514892578125f,-0.43490000693351049676493857987225055694580078125f,0.2925999889314174762233733417815528810024261474609375f,0.027199947300553528606315012439154088497161865234375f,-1.107999974906444862199350609444081783294677734375f,0.975299955841899102182424030615948140621185302734375f,-0.16910001006126407219198881648480892181396484375f,-0.6271000226303933278160229747300036251544952392578125f,-0.257700000065565149043322890065610408782958984375f,0.1303999541610474377506534437998197972774505615234375f,-0.88210000076293937354421359486877918243408203125f,0.8420000174567092710020688173244707286357879638671875f,-0.197599997872114130359477712772786617279052734375f,-1.301899978531897428268848670995794236660003662109375f,-0.7856999788463117084802433964796364307403564453125f,-0.1794999911934136438418363468372263014316558837890625f,1.97530002326965359316091053187847137451171875f,-0.500100011134147592883891775272786617279052734375f,-0.776499971807002253854079754091799259185791015625f,-1.98569996081590716130449436604976654052734375f,0.608600009748339942916572908870875835418701171875f,-0.370599987307191014451746013946831226348876953125f,0.546400008729099884163815659121610224246978759765625f,-0.226100011610984807930435636080801486968994140625f,-0.061100040519237897029825035133399069309234619140625f,-1.64880001540183940278438967652618885040283203125f,-0.50339998993873535937382257543504238128662109375f,-0.1953999777793882230980671010911464691162109375f,0.71750002879649432685482679517008364200592041015625f,0.437899996480345610194717664853669703006744384765625f,0.33669997611939883608300760897691361606121063232421875f,-0.61579998595714613429663586430251598358154296875f,-0.2165000111460688714259958942420780658721923828125f,-0.162099990928172843496213317848742008209228515625f,-1.79049999040365204194813486537896096706390380859375f,0.370500034809111866707098670303821563720703125f,0.5970000042319298927395720966160297393798828125f,1.67789997926950462669992703013122081756591796875f,0.0338000246405609772182287997566163539886474609375f,0.704299987924098758185209590010344982147216796875f,-1.58640001729726787260688070091418921947479248046875f,0.43250004231929750631024944595992565155029296875f,-0.139300034117699311764226877130568027496337890625f,2.11289994380474155377669376321136951446533203125f,-0.094499988436698600935415015555918216705322265625f,0.370499975092708933033236462506465613842010498046875f,-0.68989998916387573757447171374224126338958740234375f,-1.067200030039250346902690580463968217372894287109375f,-0.865699990022182674920259159989655017852783203125f,-0.1244000303879373436899413718492724001407623291015625f,-0.1274999988079070778468349089962430298328399658203125f,-0.303999997362494500663387952954508364200592041015625f,-0.1679000063896178840394668441149406135082244873046875f,-0.116100022153556914350502893285010941326618194580078125f,-0.8565999933719634640283402404747903347015380859375f,0.1071000009864564450623447555699385702610015869140625f,0.15560001092553132906459723017178475856781005859375f,1.14520002807974830005832700408063828945159912109375f,-0.080599997551739133161419204043340869247913360595703125f,-1.005599999190866622456042023259215056896209716796875f,-0.52239999437928208436687782523222267627716064453125f,0.119000010587275273366003602859564125537872314453125f,-0.193799987983703569938143118633888661861419677734375f,0.9365999777257443259514957389910705387592315673828125f,0.548700002181529811196014634333550930023193359375f,1.11899995142966535155437668436206877231597900390625f,0.69360000088214857072443919605575501918792724609375f,-0.1259999818503854118745266532641835510730743408203125f,0.125599996173381800446122724679298698902130126953125f,0.6646000100016598022278913049376569688320159912109375f,0.39840001497268673347207368351519107818603515625f,1.0664999931156631252093802686431445181369781494140625f,1.0412999809801579687018602271564304828643798828125f,0.2096000299692148960417625858099199831485748291015625f,0.243399991726874986852635629475116729736328125f,0.688799954009056136783328838646411895751953125f,0.368099964843690674598519763094373047351837158203125f,-0.382000044763087576171756154508329927921295166015625f,-1.1184999852031467515445228855242021381855010986328125f,-1.46719997851848660275209113024175167083740234375f,1.2519000111281872733570708078332245349884033203125f,0.1861999693989755400025387643836438655853271484375f,-0.09280002774000184473379704286344349384307861328125f,-1.262399992144107585545498295687139034271240234375f,-0.692899996778368887362375971861183643341064453125f,-0.12970001368522598994559302809648215770721435546875f,-0.384099996203184357312920838012360036373138427734375f,0.052600005769729563098735525272786617279052734375f,1.43240001166462871395879119518212974071502685546875f,-0.6870000134706497618708453956060111522674560546875f,0.400299995609372682991988767753355205059051513671875f,-1.5010000083371999313186506697093136608600616455078125f,0.135899973808228546889864674085401929914951324462890625f,0.01120002378821371991080013685859739780426025390625f,-0.7092000469744210722211619213339872658252716064453125f,1.72560000064373042505394550971686840057373046875f,-1.0458999553680410343758921953849494457244873046875f,0.31150004631280836520090815611183643341064453125f,-0.18200004111230416725675240741111338138580322265625f,-1.33260001769065805632408228120766580104827880859375f,-0.591100013212859654032627076958306133747100830078125f,0.496200013431906672423110649106092751026153564453125f,-0.38280001142323050178362109363661147654056549072265625f,0.9066000020146363080897344843833707273006439208984375f,-0.11480000872612006190820466144941747188568115234375f,-0.56889997780919099312768594245426356792449951171875f,-0.6971000028863547637314468374825082719326019287109375f,-1.0260000036060812345084514163318090140819549560546875f,-0.5429000017702581004641615436412394046783447265625f,0.4289999819397929403663738412433303892612457275390625f,-0.20150000816583624185795997618697583675384521484375f,-1.065400002142787183601058131898753345012664794921875f,-0.658199990171194126986620176467113196849822998046875f,0.0430000124201175282223630347289144992828369140625f,0.257599986472726083519546591560356318950653076171875f,0.28210000672340351801636870732181705534458160400390625f,0.426799988222122284042825413052923977375030517578125f,0.350900002977251990188278796267695724964141845703125f,-0.6236000032663344683214745600707828998565673828125f,-0.608799988635629230060430927551351487636566162109375f,-1.014300007053464458550706694950349628925323486328125f,0.76750002803280981122924231385695748031139373779296875f,-0.38909997645914551345214249522541649639606475830078125f,-0.266300002400577273675708056543953716754913330078125f,0.87180001161694509193011981551535427570343017578125f,0.083100012005866030762035734369419515132904052734375f,0.4290000101029871615310185006819665431976318359375f,1.334899970649182865400916853104718029499053955078125f,-0.739700003030896002798044719384051859378814697265625f,0.7352000160470602008189189291442744433879852294921875f,0.328900004296004799808628149548894725739955902099609375f,0.908999994792043963798278127796947956085205078125f,0.49999999623745683408060358488000929355621337890625f,-0.178800057423114555632537303608842194080352783203125f,-1.2083000526398424145924082040437497198581695556640625f,-0.100099953317641876537891221232712268829345703125f,0.53600005263090100271483606775291264057159423828125f,0.62539997777938793888097279705107212066650390625f,0.56850001753866674647497347905300557613372802734375f,-0.11939999697208403262038700631819665431976318359375f,0.774199996310472204896768744220025837421417236328125f,1.45909992661476195507930242456495761871337890625f,-0.8400999688148491628680858411826193332672119140625f,-0.46839999232292228725782479159533977508544921875f,-1.611500082969666181043066899292171001434326171875f,-0.9716999526500700312681146897375583648681640625f,1.2299000240139659201332733573508448898792266845703125f,-1.2890000217258930081953849366982467472553253173828125f,-0.50570002366155419881721400088281370699405670166015625f,0.7707999799966813725404790602624416351318359375f,0.71800002050399758246612691436894237995147705078125f,1.1756999813795090403800713829696178436279296875f,0.23099997860938259464091970585286617279052734375f,-1.22419998412877273352705742581747472286224365234375f,1.119300045330822168665463323122821748256683349609375f,0.81779998810291232302915886975824832916259765625f,-0.02629997889399504629182047210633754730224609375f,0.06570000090002992010340676642954349517822265625f,0.07480000287741450648582031135447323322296142578125f,1.39160001069456296107773596304468810558319091796875f,-0.593900037004054137668163093621842563152313232421875f,-0.0777999841168524053358623859821818768978118896484375f,-0.4374000252619383299901301143108867108821868896484375f,-1.028199966925382735638550002477131783962249755859375f,0.356000018507242277365776317310519516468048095703125f,-0.51340000554770259810766219743527472019195556640625f,0.6598999686002731124290221487171947956085205078125f,0.0944999843761327174007647045073099434375762939453125f,-0.015299998906254774766466653090901672840118408203125f,0.1742000012278550891142714363013510592281818389892578125f,0.48029997677803049072053909185342490673065185546875f,0.49550004713982342963163318927399814128875732421875f,-0.40240000376701345885521732270717620849609375f,-0.0485000283420082833885089712566696107387542724609375f,-0.00919998662471765982218130375258624553680419921875f,1.06080000175237643134096288122236728668212890625f,-0.89680000902786838334890262558474205434322357177734375f,-0.65449999715387841003888524937792681157588958740234375f,0.623899981832504391121574371936731040477752685546875f,-1.22409996688514954854554162011481821537017822265625f,0.837499979734420918475734652020037174224853515625f,-0.70689996918141861925732882809825241565704345703125f,-0.398899994647502698086327654891647398471832275390625f,-0.2620999995708468421895531719201244413852691650390625f,0.3670999872028826160885728313587605953216552734375f,0.0484999898225073611257585071143694221973419189453125f,-0.067999998897314117840551261906512081623077392578125f,0.735500013723970003098884262726642191410064697265625f,0.414300007742643305874707948532886803150177001953125f,0.0279999984875320961918987450189888477325439453125f,0.594900002276897765796093153767287731170654296875f,-0.2202999913811680698927375487983226776123046875f,0.942300017815828283573864609934389591217041015625f,-0.7375999989151953428745400742627680301666259765625f,-0.233600005948543465450484291068278253078460693359375f,1.3573000164747242024532170034945011138916015625f,0.1570000360459093968756860704161226749420166015625f,0.63279999540448184092156225233338773250579833984375f,0.8821000027000904797347402563900686800479888916015625f,-0.6170999998688699861304485239088535308837890625f,0.7391000055909149324406826053746044635772705078125f,0.15319999568462350225672707892954349517822265625f,-0.2142999806225294179284901474602520465850830078125f,-0.9204000037074084872301682480610907077789306640625f,0.52959999387115264113390367128886282444000244140625f,-0.25619998277276767595367346075363457202911376953125f,1.403000018082558586485220075701363384723663330078125f,0.032699980247020477008845773525536060333251953125f,-1.00580001635551496974585461430251598358154296875f,0.918800017786026046451297588646411895751953125f,0.21699998724460556331905536353588104248046875f,1.907599957489967579249423579312860965728759765625f,-0.109100003802776512884520343504846096038818359375f,-0.6029000005781650628478018916212022304534912109375f,-0.4508000095009805363588384352624416351318359375f,-1.7567000104665755344512945157475769519805908203125f,0.4259999999552963600990551640279591083526611328125f,-0.10330001963377011264810789725743234157562255859375f,-0.552600009793043245309718258795328438282012939453125f,-0.09789999856650843934602335139061324298381805419921875f,0.038000004708766965677568805404007434844970703125f,-0.6014999921470882693341764024808071553707122802734375f,-0.4383999951541424078271802500239573419094085693359375f,-0.4744999969303609343995731251197867095470428466796875f,0.9209000140786169907869407325051724910736083984375f,-0.03059998004287489692387680406682193279266357421875f,0.41729999375045301324149704669252969324588775634765625f,-0.68939997745156256225840252227499149739742279052734375f,-0.43980000634193405772975893341936171054840087890625f,-1.62600002588331715713820813107304275035858154296875f,0.135199997979402297687556711025536060333251953125f,0.0664999930784107018411077660857699811458587646484375f,-0.3744999987185000378531185560859739780426025390625f,-0.1377999830365179700919497918221168220043182373046875f,-0.154399995779991083821869324310682713985443115234375f,-1.08479997143596396114162416779436171054840087890625f,-0.25050001171231262020455687888897955417633056640625f,-0.05049997095763669729961975463083945214748382568359375f,0.6289000055253506360486426274292171001434326171875f,0.284200022163986876222452337970025837421417236328125f,0.94859997808337226388175622560083866119384765625f,-0.46399998085945870496260567961144261062145233154296875f,-0.304600001507997486527301589376293122768402099609375f,0.595499986007809400945234301616437733173370361328125f,-0.86880000884532915250701989862136542797088623046875f,0.501899990378320115524957145680673420429229736328125f,-0.209800010439753492619274766184389591217041015625f,0.32859998829066749959082471832516603171825408935546875f,0.9803000274233524269362760605872608721256256103515625f,0.84769998948573999086875119246542453765869140625f,-0.264500019729137392232587444595992565155029296875f,0.8483999655008318541149492375552654266357421875f,-0.0693000109463941527110364404506981372833251953125f,-0.3714000038117166724305207026191055774688720703125f,-0.39859998233020288438410716480575501918792724609375f,0.8366999791741367431541220867075026035308837890625f,-0.238000003144145377831364385201595723628997802734375f,-0.71329997306764170161841320805251598358154296875f,1.4223999700158831416274551884271204471588134765625f,-0.52120005522966295075093512423336505889892578125f,0.187700046147406762742093633278273046016693115234375f,-0.643400006031990212562732267542742192745208740234375f,-0.8019000246509910834191714457119815051555633544921875f,1.041600022950768700269463806762360036373138427734375f,-0.44589997660219615749355170919443480670452117919921875f,-0.4127000028967859890371983055956661701202392578125f,1.688299970757960455358670515124686062335968017578125f,-1.036899983859061791235944838263094425201416015625f,-0.36290001264810545222871951409615576267242431640625f,-1.8961999804258340418527950532734394073486328125f,0.23990002433061619058207725174725055694580078125f,1.6393999787926674116533831693232059478759765625f,-0.18430000915825406337944514234550297260284423828125f,0.93609998180866238470798634807579219341278076171875f,0.442700009602308153233707344043068587779998779296875f,-0.094300018024444653974569519050419330596923828125f,-1.3701000461146239928211798542179167270660400390625f,0.4681999656029052214734065273660235106945037841796875f,0.03009999757409076437397743575274944305419921875f,-2.12339996967315602915959971142001450061798095703125f,0.65519996802806890201509304461069405078887939453125f,0.3304000295609228476934049467672593891620635986328125f,-0.09189999905824652870478530530817806720733642578125f,0.6205000107064841952109190970077179372310638427734375f,0.5548999738156796812660331852384842932224273681640625f,0.06699997439235472196372711550793610513210296630859375f,-0.232799993914365899172480567358434200286865234375f,-0.5089000342100862273042594097205437719821929931640625f,-0.52210000880956641822194796986877918243408203125f,-0.6713999634668235128032165448530577123165130615234375f,-0.4539000036999583098662469637929461896419525146484375f,0.3477999860912561747028348690946586430072784423828125f,0.6921000179365275339904428619774989783763885498046875f,-0.458999987602233705530352381174452602863311767578125f,0.1607999761223795065490094202687032520771026611328125f,-0.5034000150099398052105925671639852225780487060546875f,-0.684299977344274612534036350552923977375030517578125f,-0.2933999938875440083307921668165363371372222900390625f,-0.871199981319904281917843036353588104248046875f,0.853900033465027519241630216129124164581298828125f,0.77149998775124561944949164171703159809112548828125f,-0.602199979189038447913162599434144794940948486328125f,0.553599985608458577956980661838315427303314208984375f,0.63970003097057348551146560566849075257778167724609375f,-0.373100006157159658215505260159261524677276611328125f,0.496000019140541592577164919930510222911834716796875f,-0.7652000325687235982030642844620160758495330810546875f,-0.2783000027954571908850311956484802067279815673828125f,-0.67650001736730358548044250710518099367618560791015625f,0.201399997329712476812346721999347209930419921875f,-0.2213999807894229121529861004091799259185791015625f,-0.38949998202919999812365858815610408782958984375f,-0.419899979478120854992084787227213382720947265625f,0.550999996304511796552105806767940521240234375f,1.427700027334690702218722435645759105682373046875f,-0.80560004355907466333519550971686840057373046875f,-0.50050001398474019964623948908410966396331787109375f,0.337600018361210896244983814540319144725799560546875f,0.77960003509148967371089611333445645868778228759765625f,0.864699987477809184355237448471598327159881591796875f,-0.1810000443048773721699262750917114317417144775390625f,1.19050002646446273502078838646411895751953125f,-0.7068999612092969897503280662931501865386962890625f,0.568800072622298813485031132586300373077392578125f,-0.8356000003457069169598980806767940521240234375f,0.34010001828670510803931392729282379150390625f,-0.596600015723705467962645343504846096038818359375f,-0.25690000114440891820777324028313159942626953125f,0.6087999987125396472720240126363933086395263671875f,-0.04900002038478845634017488919198513031005859375f,-1.7016000330090523817716530174948275089263916015625f,0.35759999384880103434625198133289813995361328125f,1.41439999995231602269996074028313159942626953125f,-0.4134999778270724846152006648480892181396484375f,1.0053000046432014613628780352883040904998779296875f,-0.23869998608827625030670560590806417167186737060546875f,0.13729999939426795751984400340006686747074127197265625f,0.994900019189714601708374175359494984149932861328125f,0.4007999975800515812807134352624416351318359375f,0.53879999573230719089877993610571138560771942138671875f,0.0160999977529048265978417475707828998565673828125f,0.84889995322227485008426128842984326183795928955078125f,0.4695999937593935413815415813587605953216552734375f,-0.45900001520663469278815682628192007541656494140625f,-0.714700026351213324460331932641565799713134765625f,-1.451400000719726079978499910794198513031005859375f,0.506700003500282658563946824870072305202484130859375f,0.6955000084899365386803538058302365243434906005859375f,0.28109999924302098150974416057579219341278076171875f,-1.3323999818809328932189828265109099447727203369140625f,0.83300001677870749716703357989899814128875732421875f,-0.2980999759450553288075980162830092012882232666015625f,0.79700002524256685632053631707094609737396240234375f,1.46949997372180263255359022878110408782958984375f,0.5795000005066395232233844581060111522674560546875f,1.556099985906481464326134300790727138519287109375f,0.06419999735355386150104095577262341976165771484375f,1.077799999204277714426325474050827324390411376953125f,-0.33510000174641607628700512577779591083526611328125f,0.075100024023651901217135673505254089832305908203125f,0.5301999876976015713125889305956661701202392578125f,0.1130000140219926674234329766477458178997039794921875f,-0.4818999854981900199391020578332245349884033203125f,-0.89799997535347930721627562888897955417633056640625f,-1.443900028994679729521521949209272861480712890625f,0.2449999818950896468550126883201301097869873046875f,-0.62950001548230627435032147332094609737396240234375f,-0.1794999946951865155142513685859739780426025390625f,-0.479300015342235763426970152067951858043670654296875f,0.28399999684095433849506662227213382720947265625f,0.3677999825894833207939882413484156131744384765625f,0.3850000295042992348726329510100185871124267578125f,-2.217900064206123733612230353173799812793731689453125f,-0.3563999585866923780486104078590869903564453125f,1.1260000433921817375448881648480892181396484375f,1.05319997929334618902430520392954349517822265625f,-0.658099994683265521189241553656756877899169921875f,-0.694399996972084210256070946343243122100830078125f,-0.45410000380277626419456282746978104114532470703125f,0.8526000057697302736414712853729724884033203125f,-1.164399981474876000220319838263094425201416015625f,0.593100060844420795547193847596645355224609375f,1.398200018930435106767617980949580669403076171875f,0.014699982690810742269604816101491451263427734375f,-0.98330006463527741544794480432756245136260986328125f,-1.21949999976158096615108661353588104248046875f,-0.49740004638433532591079710982739925384521484375f,-1.913900007462501662303111515939235687255859375f,-0.606200046586991447838954627513885498046875f,-0.45700000423192932430538348853588104248046875f,0.22050001990795209394491394050419330596923828125f,0.076999990224837944197133765555918216705322265625f,-0.87240004429817208375652626273222267627716064453125f,0.225900009347498098577489145100116729736328125f,0.22819999553561221006248160847462713718414306640625f,0.530299997080862173248760882415808737277984619140625f,0.0941999988436699187133172017638571560382843017578125f,0.85940001146346300942013840540312230587005615234375f,0.30660000082254423858785230549983680248260498046875f,1.3603000371277331748132155553321354091167449951171875f,-0.1523999712824821717305212587234564125537872314453125f,0.1437999945774672827525364482426084578037261962890625f,0.267300020796060522343395859934389591217041015625f,0.7916999976515768366880365647375583648681640625f,0.3767999909043311390632879920303821563720703125f,-1.25910002771913998032005110871978104114532470703125f,-0.011700009721517634631027249270118772983551025390625f,0.9654999974071978297018858938827179372310638427734375f,0.7577000055789948618212292785756289958953857421875f,1.09100000721961265526260831393301486968994140625f,0.7407000146090982894975240924395620822906494140625f,0.979099989944697046695409881067462265491485595703125f,-0.400000008344650215263982317992486059665679931640625f,0.7287000320583583157230123106273822486400604248046875f,-0.851699992734193500609762850217521190643310546875f,0.52880002225637401380708979559130966663360595703125f,-1.105200011688470596027400461025536060333251953125f,1.146199978972970878743353750905953347682952880859375f,-0.304899988567829183239155099727213382720947265625f,0.8472999959483742404842132600606419146060943603515625f,-0.305699999409914102699303839472122490406036376953125f,-0.674100038298964587824002592242322862148284912109375f,0.05840000819265867004759229530463926494121551513671875f,0.641799976450204923139608581550419330596923828125f,0.1922999784767629449788728379644453525543212890625f,-0.98400000041723245658431551419198513031005859375f,-1.479499982774257649253968338598497211933135986328125f,-0.457299999189377359698482905514538288116455078125f,0.121400007462501235977470059879124164581298828125f,0.41709999718666068702077609486877918243408203125f,0.3803000211648648321016707996022887527942657470703125f,1.1940999947130677139028875899384729564189910888671875f,-0.31110000084489597416137485197396017611026763916015625f,2.15819999032020604090575943700969219207763671875f,0.5127999925732620312146536889486014842987060546875f,-2.030799974632262916429681354202330112457275390625f,-0.22199997770786250583796572755090892314910888671875f,0.06190002319812748510230449028313159942626953125f,-1.12200001116096981235159546486102044582366943359375f,0.17920002596378292736289949971251189708709716796875f,-0.376300001171231190966182111878879368305206298828125f,0.97590002018809318684589015902020037174224853515625f,-0.6009999775290488788215270687942393124103546142578125f,-0.4512999581813819105491347727365791797637939453125f,0.09189998803138710314897252828814089298248291015625f,0.696899979686737225392789696343243122100830078125f,0.73810003774762211747884066426195204257965087890625f,-0.78449999029934414096487671486102044582366943359375f,-0.4953999955862762316627367908949963748455047607421875f,0.470399994468689186533083557151257991790771484375f,-0.24899999266862860025639747618697583675384521484375f,0.58900000783056005726479043005383573472499847412109375f,0.09949999544024468178804454510100185871124267578125f,-0.14609999387860295172458791057579219341278076171875f,-0.8226999909758567763873315925593487918376922607421875f,-0.436500011406838861471868540320429019629955291748046875f,0.19539997658729557628021211712621152400970458984375f,1.186999955356121017047144050593487918376922607421875f,0.31399999773502340616460060118697583675384521484375f,-0.11020000870823831728984032451990060508251190185546875f,0.58469999392256133408096729908720590174198150634765625f,-0.26890000366270550014036189168109558522701263427734375f,1.23380000348091112272186364862136542797088623046875f,-0.377200003030896180433728659409098327159881591796875f,-1.188999975681304999142184897209517657756805419921875f,0.593900012230873297625066697946749627590179443359375f,-0.051299988281726882632938213646411895751953125f,0.2837999769940975358650803173077292740345001220703125f,0.370500002995133304040109578636474907398223876953125f,0.53359999350607434376314586188527755439281463623046875f,0.78480000298917296230882811869378201663494110107421875f,0.8089999846220019552589519662433303892612457275390625f,7.599592066753757535479962825775146484375e-09f,-0.152300057750940187606403242170927114784717559814453125f,0.625499986082315384550156522891484200954437255859375f,-0.70410000007748596573264876496978104114532470703125f,-0.30069999948441983317337644621147774159908294677734375f,0.5456999940827487183270250170608051121234893798828125f,0.448699995326995892952481881366111338138580322265625f,0.76390000377446387602731192600913345813751220703125f,-0.30700000162422702576492383741424418985843658447265625f,0.871000029683113297096497262828052043914794921875f,1.65560001286268221321051896666176617145538330078125f,-0.38810006233453719914905377663671970367431640625f,-0.443799981650709884917205272358842194080352783203125f,0.66320001662075522830264162621460855007171630859375f,1.24980002243518839577518519945442676544189453125f,-1.3948000388264656379533334984444081783294677734375f,-0.114199985022842565030032346840016543865203857421875f,0.373900004556774856467882273136638104915618896484375f,-1.25010003177225659243276822962798178195953369140625f,0.8037000066518782404045850853435695171356201171875f,-0.526199987950920711199387369561009109020233154296875f,-0.7641999597281223799427607445977628231048583984375f,0.35360000080764297791802164283581078052520751953125f,-0.26290000251531608910227078013122081756591796875f,0.833300006222724931603806908242404460906982421875f,-0.987799985122681079019457683898508548736572265625f,0.13249999612569762774683113093487918376922607421875f,0.335299981844425332155879004858434200286865234375f,0.24500001125037673688211725675500929355621337890625f,-0.65989998618364342775066688773222267627716064453125f,-0.379899990206956861271692105219699442386627197265625f,0.7797999717712400524760596454143524169921875f,1.2085999841183419079015948227606713771820068359375f,-1.25199998053908334583184114308096468448638916015625f,-0.0012999791920182968141261881100945174694061279296875f,0.5357000031352043833976495079696178436279296875f,-1.23919999586343720920922351069748401641845703125f,0.3140000058189034159994434958207421004772186279296875f,0.800100012326240506155272669275291264057159423828125f,-0.582600005955994593165314654470421373844146728515625f,-0.4073000270545481527051379089243710041046142578125f,-0.982999978296458909454713648301549255847930908203125f,0.601300004076957606002906686626374721527099609375f,-0.1957000191539528533013481137459166347980499267578125f,-0.9600000169873237343409755339962430298328399658203125f,0.3764000265359881414184428649605251848697662353515625f,-0.005299998943507577831013577451813034713268280029296875f,0.524199994149804116005952892010100185871124267578125f,0.5584999994337558337775817562942393124103546142578125f,0.453999969385564750012918011634610593318939208984375f,-0.2272999898016454523030915879644453525543212890625f,0.39999997273087550819781199606950394809246063232421875f,-0.172300003361701925541638047434389591217041015625f,-0.717799998943507322479717913665808737277984619140625f,-0.680799995642900501735539364744909107685089111328125f,-0.9672000022485851555842373272753320634365081787109375f,0.26979998756647116664453278644941747188568115234375f,0.15500000484287734803245939474436454474925994873046875f,-0.409599990704655692042024384136311709880828857421875f,-0.45109999633729447321428551731514744460582733154296875f,0.6483999681830407979532537865452468395233154296875f,-0.810200009602308313105822890065610408782958984375f,-0.72799998258054265942718075166339986026287078857421875f,0.0451000084519386224002346352790482342243194580078125f,0.26840000483989712165566743351519107818603515625f,-0.4306000198662276456929021151154302060604095458984375f,0.666400012975930877701102872379124164581298828125f,-1.8712000427126884716244603623636066913604736328125f,-0.503599985235929192839421375538222491741180419921875f,-0.7176999951481821682364170555956661701202392578125f,-0.70950002807378798053150603664107620716094970703125f,-1.208699994730949356380733661353588104248046875f,-0.7079000232279302196047865436412394046783447265625f,-0.0298999846935270596759437466971576213836669921875f,-0.799199978019296874975907485350035130977630615234375f,-0.461199968206882804366841810406185686588287353515625f,1.4734999722391364240792199780116789042949676513671875f,0.45859997167587351896145264618098735809326171875f,-1.58619999555051283124385008704848587512969970703125f,-0.45079996941685696043577991076745092868804931640625f,0.388299968373775339358644487219862639904022216796875f,-1.03930000789165433161542750895023345947265625f,-0.4844999982714650599291417165659368038177490234375f,-1.403900005674362461149939917959272861480712890625f,-0.78100000494718546661943037179298698902130126953125f,-0.89919996933937085259458399377763271331787109375f,0.5834000170588495848278398625552654266357421875f,-1.72949992227554361079455702565610408782958984375f,0.1810000115036967116566302138380706310272216796875f,-0.144000007867813195616690791212022304534912109375f,-0.63890006855726344525692184106446802616119384765625f,1.50080000622272535792944836430251598358154296875f,-0.0434999720156188374176053912378847599029541015625f,0.8414999890923500203143703402020037174224853515625f,-1.486900024092197458713826563325710594654083251953125f,-0.760999993957579778935951253515668213367462158203125f,-1.1616000207901000518262435434735380113124847412109375f,0.371099991978705023853990496718324720859527587890625f,-0.36070000343322750069319226895458996295928955078125f,0.9513999757230282572351143244304694235324859619140625f,-0.5610999995410441698595604975707828998565673828125f,0.52409997996091828298403925145976245403289794921875f,0.254300004762411102632313486537896096706390380859375f,0.72799998358637107997992643504403531551361083984375f,0.1411000026330351264647333664470352232456207275390625f,0.9991999870717525045193951882538385689258575439453125f,-0.19069999046921726204573133145458996295928955078125f,-0.37249998070299650265724267228506505489349365234375f,0.028800008472800175951533674378879368305206298828125f,1.515699983614683166166514638462103903293609619140625f,-0.81299998563528053097115844138897955417633056640625f,-0.0389000070154670307687183594680391252040863037109375f,-0.755700008052587524076670888462103903293609619140625f,-0.5311000332176682992013638795469887554645538330078125f,0.0163000012457370502261255751363933086395263671875f,0.6939999595135455390249035190208815038204193115234375f,-0.80089995954036741210302352556027472019195556640625f,1.36660001106709305673803100944496691226959228515625f,-0.308700006204843280244176639826036989688873291015625f,0.01480000395774805355131320538930594921112060546875f,0.81830001412034025776165435672737658023834228515625f,0.121700027081370176773589264485053718090057373046875f,0.3855999961733818093279069216805510222911834716796875f,-0.018500022642314650767048078705556690692901611328125f,0.603900029851496267241373061551712453365325927734375f,-1.030099986994267080575582440360449254512786865234375f,0.7474999757856128912436588507262058556079864501953125f,0.024399995183944866994352196343243122100830078125f,-1.88150000667571948298473216709680855274200439453125f,-0.47329995794296308986304211430251598358154296875f,0.8714999813996249233838398140505887567996978759765625f,-0.8984999648630618285238824682892300188541412353515625f,0.86420001989156058908037039145710878074169158935546875f,-0.488299999666214290527932462282478809356689453125f,-0.89639998142272236947292185504920780658721923828125f,1.204399972310662203511810730560682713985443115234375f,1.38400001420080709380044936551712453365325927734375f,0.55290003201961468448644154705107212066650390625f,-1.114300029051303653204740840010344982147216796875f,1.9459000364303591368297929875552654266357421875f,0.6754000132441522197268568561412394046783447265625f,0.466600005888938795806097914464771747589111328125f,-0.95600002029538178049961061333306133747100830078125f,0.6041000032067298519677933654747903347015380859375f,0.860000015497207837000814834027551114559173583984375f,-0.0066999937772749262876459397375583648681640625f,1.488000026315450696756670367904007434844970703125f,-0.7917999992489814786722490680404007434844970703125f,-0.307699992167949520904812743538059294223785400390625f,0.235399987912177977023020503111183643341064453125f,-0.384099990112334754854828133829869329929351806640625f,1.2706999808207155577832736526033841073513031005859375f,0.1852000106826430236850455912644974887371063232421875f,0.98679997019171750149979516208986751735210418701171875f,-0.5994000003620982841567865762044675648212432861328125f,0.9878999710738660677833422596449963748455047607421875f,-0.537799979162216335026869273860938847064971923828125f,-0.84659998756051013657497605890966951847076416015625f,0.158800001245737121280399151146411895751953125f,0.78289997014254353313589263052563183009624481201171875f,-0.59569997037127631056563359379651956260204315185546875f,-0.3767000096470114467450684969662688672542572021484375f,-0.5719999945461751167385955341160297393798828125f,-0.25700000624358654288670322785037569701671600341796875f,0.7706999800756573204552069000783376395702362060546875f,0.20339999731481128719678963534533977508544921875f,-0.22249997004866628458330524154007434844970703125f,-0.8390999801844352123936232601408846676349639892578125f,1.15370002393722526079500312334857881069183349609375f,0.3475000365450979789017083021462894976139068603515625f,-0.073899989581108105340945257921703159809112548828125f,-1.014600007952749738393549705506302416324615478515625f,0.6184999939203261742903805497917346656322479248046875f,-0.99850000070035438426430118852294981479644775390625f,0.1854000001311302270323722041212022304534912109375f,-0.398199992257356694125292051467113196849822998046875f,-0.7072999896526337426649888584506697952747344970703125f,-0.76169999437332147618207045525196008384227752685546875f,0.197499988079071098212580182007513940334320068359375f,0.14739998888447847225080522548523731529712677001953125f,0.844999989587813526981108225299976766109466552734375f,-0.13990002531781786654363486377405934035778045654296875f,1.3251000232972207637516248723841272294521331787109375f,-0.3080999727785587172235182151780463755130767822265625f,0.60710001838356297820809004406328313052654266357421875f,0.26149997791647905387435457669198513031005859375f,0.0615999914348126509366920799948275089263916015625f,-1.12649997627735132255111238919198513031005859375f,1.292900024829804817017020468483678996562957763671875f,-0.221800032627582854871661766082979738712310791015625f,-1.0059000137060885027295853433315642178058624267578125f,-0.96640005455017075064461096189916133880615234375f,1.0513000380516057674640251207165420055389404296875f,0.50229997653961078185602673329412937164306640625f,-0.30880003916919296358400970348156988620758056640625f,0.42949998277425738280044242856092751026153564453125f,0.005999993808567527509012506925500929355621337890625f,1.05970000262111430089362329454161226749420166015625f,-1.97750002771616006924659814103506505489349365234375f,-0.097299931538105344230871196486987173557281494140625f,0.3707999956615271042181802840786986052989959716796875f,-0.634699996642023034354451738181523978710174560546875f,1.9343000185646113120441214050515554845333099365234375f,-0.9623000808477399914409033954143524169921875f,-0.4248000103652476155957629089243710041046142578125f,-1.7114999566078186177264797152020037174224853515625f,-0.554799984101951082493542344309389591217041015625f,0.228899964770674646530324025661684572696685791015625f,0.47860001444220567901055574111524038016796112060546875f,-1.54219997553825383107550806016661226749420166015625f,0.708399957305192895518075602012686431407928466796875f,0.51020000423788991472662246451363898813724517822265625f,-0.41899997167661806596328233354142867028713226318359375f,-0.9523999950498340805182806434459052979946136474609375f,-0.08490000749230386389854174922220408916473388671875f,-0.81769998836815338538741571028367616236209869384765625f,-0.637799984526634222703478371840901672840118408203125f,0.00470000839531425906869799291598610579967498779296875f,-0.6078000115349888599070027339621447026729583740234375f,0.185799988192319520674544719440746121108531951904296875f,-0.63730002660751328136257143341936171054840087890625f,0.85780001414269246851063144276849925518035888671875f,-0.44220001413226128761380095966160297393798828125f,-1.3829000109717244360041377149173058569431304931640625f,-0.007600008079409636962964214035309851169586181640625f,-0.19569996375888554229760529779014177620410919189453125f,0.36240000540614136781414345023222267627716064453125f,0.058899986526370184947154484689235687255859375f,0.4563000244542954053628136534825898706912994384765625f,0.8582999733097853578556168940849602222442626953125f,-0.2041999988436697943683384437463246285915374755859375f,-0.71159999696686926373700998738058842718601226806640625f,-0.9177000358283518011148771620355546474456787109375f,-0.65549999599158770280382668715901672840118408203125f,-0.92419997157454503167173243127763271331787109375f,-0.2108000337526203527005463911336846649646759033203125f,0.451399968235194737342652615552651695907115936279296875f,0.7486999827727676137811840817448683083057403564453125f,-0.884699982765317027855189735419116914272308349609375f,0.2155999772861598717810238667880184948444366455078125f,0.8213000526547435953261810936965048313140869140625f,-0.819700033950805817539730924181640148162841796875f,0.10459996857643094614331857883371412754058837890625f,0.77349998497962957344498136080801486968994140625f,-1.8931000480294226662181245046667754650115966796875f,0.21650001651048711437397287227213382720947265625f,0.034400005204975769856901024468243122100830078125f,0.6944000037148594373093146714381873607635498046875f,-0.2640000163987281656119421313633210957050323486328125f,0.036299980235099837955203838646411895751953125f,-0.89329998536109922468995137023739516735076904296875f,1.33369997208118462594939046539366245269775390625f,-0.04520000915527333518184605054557323455810546875f,-0.679399974024295971730680321343243122100830078125f,-0.1939999978095290344981549424119293689727783203125f,-0.56500001341104510999002741300500929355621337890625f,0.861199989143013766579315415583550930023193359375f,1.1024999765306713328527621342800557613372802734375f,-0.62630004945099404523034536396153271198272705078125f,1.20830000920295788091607391834259033203125f,0.33580005182027861110327648930251598358154296875f,0.397099997782706992666135192848742008209228515625f,1.22970001159906328069837400107644498348236083984375f,1.178799985897541091617313213646411895751953125f,0.2688999710477888438475702059804461896419525146484375f,-1.0751999863564962733875063349842093884944915771484375f,-0.38929999302774642711000296912970952689647674560546875f,-0.26080002335906016242006444372236728668212890625f,1.52810001301169418042036340921185910701751708984375f,0.1769000268861649072249520031618885695934295654296875f,-0.735800016951560831302003862219862639904022216796875f,0.6605999998986720189719790141680277884006500244140625f,-0.05839999172687537765114029753021895885467529296875f,0.127500013113021726240958741982467472553253173828125f,0.32330001292824750436238900874741375446319580078125f,1.2377000223800538325491515934118069708347320556640625f,0.22650000481307508426453978245262987911701202392578125f,-0.414200009460747420320103628910146653652191162109375f,0.058299988639354705100004139239899814128875732421875f,0.54460001123100487685491088996059261262416839599609375f,0.9492999792814256210249368450604379177093505859375f,0.9612999832525848109554544862476177513599395751953125f,-0.417499982118606727254928046022541821002960205078125f,0.33639998447746055632023853831924498081207275390625f,-0.3370000090748070942225922408397309482097625732421875f,-0.03699997130036358772287030660663731396198272705078125f,0.2494999966323376838772674091160297393798828125f,-1.460899969300628153945353915332816541194915771484375f,0.88530001254081724937350372783839702606201171875f,-1.0075999562606217097027183626778423786163330078125f,0.525399980759620799375397837138734757900238037109375f,-0.298999989986419745235934897209517657756805419921875f,0.5155999963223933946210308931767940521240234375f,-0.15709999193400159267497429027571342885494232177734375f,-0.79289997358843689045926339531433768570423126220703125f,-0.6752000077396635990822915118769742548465728759765625f,0.018499984681606296277323053800500929355621337890625f,-1.0046000056430699487464153207838535308837890625f,0.63330000212490578093138537951745092868804931640625f,-0.01669999601244942510902546928264200687408447265625f,-1.4139000245615844786328807458630762994289398193359375f,0.398299984094500292286511466954834759235382080078125f,0.92780000449418975616566740427515469491481781005859375f,-0.69190002081394208488518415833823382854461669921875f,-0.407700003641843888857465572073124349117279052734375f,1.849699998188018668088261620141565799713134765625f,0.65949993687868069258684045053087174892425537109375f,-0.80609996929168659107745043002068996429443359375f,0.6154000001311299428152779000811278820037841796875f,0.41879999513626131601995439268648624420166015625f,-1.58170005606412900078794336877763271331787109375f,-1.618999987602233403549689683131873607635498046875f,0.7063999988198279567086501629091799259185791015625f,-1.930400010561942991671458003111183643341064453125f,0.53249997235834545250554583617486059665679931640625f,-1.38469998641610114731292924261651933193206787109375f,-0.4581999745249742961306083088857121765613555908203125f,-0.4343999917566776503008441068232059478759765625f,0.016399980491399901438853703439235687255859375f,-1.4199999949336046256576082669198513031005859375f,0.8812000069312748706806814880110323429107666015625f,0.270699979908019638941141238319687545299530029296875f,-0.8191000059075654871065808038110844790935516357421875f,-0.0422999971032140820170752704143524169921875f,-0.70090004302412278747169693815521895885467529296875f,0.196800040748715598937224058317951858043670654296875f,1.079100073614716936987178996787406504154205322265625f,1.383199980485438462807223913841880857944488525390625f,-0.5401999745845795786181042785756289958953857421875f,-1.593200004178285222877775595407001674175262451171875f,-0.5955999854445457319940260276780463755130767822265625f,-1.6588000203937289800393273253575898706912994384765625f,-0.0020000043064356542998893928597681224346160888671875f,0.6244999958127739336788408763823099434375762939453125f,0.39019999116212134016024037919123657047748565673828125f,0.45130000064969078010790326516143977642059326171875f,0.37279999886900172523240826194523833692073822021484375f,-0.60579996137022984470377195975743234157562255859375f,0.1914000007569789385541980664129368960857391357421875f,-0.365799991470575402985332402749918401241302490234375f,0.54700001205503900880700030029402114450931549072265625f,1.61020001958608727221644585370086133480072021484375f,-0.4736999649658801825324871970224194228649139404296875f,-0.6758000130772590097194552072323858737945556640625f,0.279499998420476902794007401098497211933135986328125f,0.41050000593811286631051871154340915381908416748046875f,0.1028000027433038088275907284696586430072784423828125f,-0.147100005307793768594137873151339590549468994140625f,-0.74509996479153672854778278633602894842624664306640625f,0.07379999714791807274849588793586008250713348388671875f,-0.646399971364438830079279796336777508258819580078125f,0.58419998289942842717437088140286505222320556640625f,-0.184899986071884736560377859859727323055267333984375f,-0.427800008163600919974811631618649698793888092041015625f,0.016599999257922180839752712699919356964528560638427734375f,-0.224700004632770677925890367987449280917644500732421875f,0.75210003514736911700566679428447969257831573486328125f,0.124599997596442413350104061464662663638591766357421875f,0.147699976167828150863670089165680110454559326171875f,0.3781999769091601848458594758994877338409423828125f,-1.0315000260472297899383420372032560408115386962890625f,-0.17379999364614473478241052362136542797088623046875f,-0.395599999041855110437637677023303695023059844970703125f,-0.661200006651878169350311509333550930023193359375f,0.583599997864663766478088291478343307971954345703125f,0.19220000467002396060678393041598610579967498779296875f,0.8187999922305346789386248929076828062534332275390625f,0.6781999962061642239774528206908144056797027587890625f,0.75320003114938749444462473547901026904582977294921875f,0.538200004923343300333726801909506320953369140625f,-1.490300030124187546931580072850920259952545166015625f,-0.394599998676776753114836537861265242099761962890625f,0.223799964849650923071067154523916542530059814453125f,0.228600002074241359650841332040727138519287109375f,1.44009999403506494530802228837274014949798583984375f,0.367799987804890093912035808898508548736572265625f,-1.272099991524219486649371901876293122768402099609375f,-0.2081999876379967417960870079696178436279296875f,0.34619996597170776819751836228533647954463958740234375f,0.1020000233054165761359399766661226749420166015625f,1.47300001882761666394117128220386803150177001953125f,0.20319999210834538416747818700969219207763671875f,-1.3227999843031168492046845130971632897853851318359375f,-0.3699999913573268628397272550500929355621337890625f,0.829499996371567238639954666723497211933135986328125f,0.1920000159665942174314068324747495353221893310546875f,-0.337799987283348979172359349831822328269481658935546875f,-1.080299991269410231353731433046050369739532470703125f,-0.5503999923452733700202088584774173796176910400390625f,-1.705500014692545196481887614936567842960357666015625f,-0.255699993970990302472046096227131783962249755859375f,0.45450000914931276696506756707094609737396240234375f,-0.8987999692827466358124866019352339208126068115234375f,-0.6208999904975291972419881858513690531253814697265625f,0.23330000596195443751668108234298415482044219970703125f,-0.37039994961619371327543603911180980503559112548828125f,0.0492000021964311695654714640113525092601776123046875f,-0.7578999991998074392540729604661464691162109375f,0.2261000005096198695042630788520909845829010009765625f,-0.668200018185377331292329472489655017852783203125f,-0.564699988055229251671107704169116914272308349609375f,-0.832199971067905774901873883209191262722015380859375f,0.59229996856749078659731821971945464611053466796875f,1.1316000146806235449048472219146788120269775390625f,0.4090999993324277994588555884547531604766845703125f,-0.46539994350671776146555203013122081756591796875f,-0.23719999725669627110846704454161226749420166015625f,-0.26820000686049372262687029433436691761016845703125f,0.35759997835159307300045838928781449794769287109375f,1.26139997751116794688641675747931003570556640625f,0.58529997621923701700552555848844349384307861328125f,0.29490000525712911638720470364205539226531982421875f,0.501299966265261520703688802313990890979766845703125f,1.228600010418891219643455769983120262622833251953125f,0.4651000010386108218796152868890203535556793212890625f,-0.20240000525712975587566688773222267627716064453125f,-0.3920000163391230696419142986997030675411224365234375f,0.1092000077843664485044428147375583648681640625f,0.14650001259893186134064535508514381945133209228515625f,-0.35899998804926891438071834272705018520355224609375f,0.33459999018311503693468011988443322479724884033203125f,-0.06319999479055393720727806794457137584686279296875f,0.01469997609704687402398803897085599601268768310546875f,0.2412000173807147263005390414036810398101806640625f,0.116900008296966451126763786305673420429229736328125f,-0.095600001835823178453210857696831226348876953125f,1.318800031569600239578221589908935129642486572265625f,0.48780000233352172056555673407274298369884490966796875f,-1.551900001591443700732497745775617659091949462890625f,-0.5499999910593036389627741300500929355621337890625f,1.4191999949693678839679478187463246285915374755859375f,0.60220000414848318115446090814657509326934814453125f,-0.02770002424269890628494295015116222202777862548828125f,0.38399997657537454642806551419198513031005859375f,0.260299981583654638228608746430836617946624755859375f,-0.0590000388622282656569950631819665431976318359375f,-0.24899997065216294966916166231385432183742523193359375f,0.09619999115467070538443294935859739780426025390625f,0.43220000589936991008954692006227560341358184814453125f,0.6429999924898144314511228003539144992828369140625f,-0.0944000169023875290719161057495512068271636962890625f,-0.1028000235676760354408543207682669162750244140625f,-0.2124999732151626741227801176137290894985198974609375f,0.258599987620115268072140679578296840190887451171875f,0.7225999707520009973649166568065993487834930419921875f,0.6040000565946090915758759365417063236236572265625f,0.31899998916685579075647183344699442386627197265625f,0.9343000237613916514334277962916530668735504150390625f,-0.089799999673664532284789174809702672064304351806640625f,0.5097999852567911549527934766956605017185211181640625f,0.119800002318620590902042977177188731729984283447265625f,-0.245199993807076577212455958942882716655731201171875f,0.508999974042177338873216285719536244869232177734375f,0.64020002033114398187763072201050817966461181640625f,-0.3545000088140366489852794984471984207630157470703125f,-0.146400028957426986409018354606814682483673095703125f,0.466700004953145952679705033006030134856700897216796875f,0.01910000119507293536713632420287467539310455322265625f,0.3248999897226692024077010501059703528881072998046875f,1.045400012350082619150271057151257991790771484375f,0.36349999280273925794659817256615497171878814697265625f,-0.68329996673464865608593754586763679981231689453125f,-0.53039998485744010991993491188623011112213134765625f,-0.6749000103235243130939124966971576213836669921875f,-0.2045000064671038675356840030872263014316558837890625f,-0.419800005149841837237545405514538288116455078125f,0.19190000531673423456169302880880422890186309814453125f,-0.4847999604463577583146616234444081783294677734375f,-0.20829999236464491474407623172737658023834228515625f,-0.61750005334615654106755755492486059665679931640625f,-0.514699987272918324521242539049126207828521728515625f,-1.548700009930133791868911430356092751026153564453125f,0.49719999438822259918424606439657509326934814453125f,0.26319997959136998133544693700969219207763671875f,-0.51759996430724886096186310169287025928497314453125f,0.01900001718103905279377840997767634689807891845703125f,-0.6747000201299784283293092812527902424335479736328125f,0.0170000042319298216852985206060111522674560546875f,0.8573000195294617764574240936781279742717742919921875f,-0.055799994897842264407472612219862639904022216796875f,-0.833400012886524432786927718552760779857635498046875f,-0.19799998712539679246447121840901672840118408203125f,0.58140000880360587220962997889728285372257232666015625f,0.1242000150859359219879252123064361512660980224609375f,-0.3859999968633054390210190831567160785198211669921875f,-0.21319999330043781782251244294457137584686279296875f,0.4121000099271536765144219316425733268260955810546875f,0.028400015270710010639731990522705018520355224609375f,-0.1349999839439985915845454655936919152736663818359375f,0.021000030316412454656216368675814010202884674072265625f,1.6620000231377780597341597967897541821002960205078125f,0.6814000070154662136445722353528253734111785888671875f,-0.86800003726780439183841053818468935787677764892578125f,-0.4068000347137452621382180950604379177093505859375f,0.0461000072151425133171187553671188652515411376953125f,0.3637999867916104079768047085963189601898193359375f,0.37579999262541508020518676858046092092990875244140625f,0.6454000256121161793743112866650335490703582763671875f,0.1937000188708306591678365293773822486400604248046875f,0.06499999813735468645603532422683201730251312255859375f,0.5837999851897357483920814047451131045818328857421875f,0.46619999949932111604766760137863457202911376953125f,0.5851999945521353385657903345418162643909454345703125f,0.348300004397333384797974531466024927794933319091796875f,0.85019998367428784025179311356623657047748565673828125f,0.55990004612356347735158124123699963092803955078125f,0.075200000624358498413357665413059294223785400390625f,0.8405000051558022544639925399678759276866912841796875f,0.378799999718367974566035627503879368305206298828125f,0.23920001463890105952714293380267918109893798828125f,0.29369997874945419358283515975927002727985382080078125f,0.70240000115931044977202191148535348474979400634765625f,-0.4160999832242728846409818288520909845829010009765625f,-0.768300003316998214586419635452330112457275390625f,-0.66399998022615924764266992497141472995281219482421875f,-0.5112000248312946748541207853122614324092864990234375f,0.5900999963447450813447403561440296471118927001953125f,0.6538000164635480171426706874626688659191131591796875f,0.76060000071823574696594505439861677587032318115234375f,1.2899000327870255855344794326811097562313079833984375f,0.761699986699223785535650677047669887542724609375f,0.6440999784708021369539210354560054838657379150390625f,-0.5361000141069289792739027689094655215740203857421875f,-0.95539999551177157144365992280654609203338623046875f,-0.387199988837540143293836081284098327159881591796875f,-1.68570001438558048079130458063445985317230224609375f,0.5989999967291950166980996073107235133647918701171875f,0.323100020946562427326398392324335873126983642578125f,-0.223199994902312681688272277824580669403076171875f,-0.91600001113116735762531561704236082732677459716796875f,-0.136099974916875421371287302463315427303314208984375f,-0.08439997991025405266185543950996361672878265380859375f,-0.754499986909329845019556159968487918376922607421875f,-0.27179998524189041830823043710552155971527099609375f,-0.5829000349625943488973689454724080860614776611328125f,-0.7170000113844867684065320645458996295928955078125f,0.67190000278353689822097294381819665431976318359375f,0.1653999993860721939853419826249592006206512451171875f,0.143799995508789635323410038836300373077392578125f,-0.2580000172257417290211378713138401508331298828125f,0.740300048415363232123809211771003901958465576171875f,0.9160000112056732035625827847979962825775146484375f,-1.508800011676549868155916556133888661861419677734375f,-0.237399978137016631762890028767287731170654296875f,1.35409999603554620506429273518733680248260498046875f,0.478000022627413534426210617311880923807621002197265625f,-0.8554999932162463682772113315877504646778106689453125f,-0.646199998605251124672577134333550930023193359375f,-0.735399987465143123444022421608678996562957763671875f,-0.848799999010562800094703561626374721527099609375f,-0.16999999370425944444917831788188777863979339599609375f,-0.0041999653905631095796024965238757431507110595703125f,0.002899978822469950756612888653762638568878173828125f,-0.6130000208020209839787639793939888477325439453125f,0.8530000006854534522204858149052597582340240478515625f,0.4669999923110008666071735206060111522674560546875f,0.323800029520690701989593662801780737936496734619140625f,0.1369999835938211507624373552971519529819488525390625f,-0.30010001649856576921848727579344995319843292236328125f,0.5930999736353754769879742525517940521240234375f,0.53900001192837920793721195877878926694393157958984375f,0.6315000439658771114181945449672639369964599609375f,-0.65060000470429624197521434325608424842357635498046875f,-0.0693000613868246517768056946806609630584716796875f,-0.854099969679117609899776653037406504154205322265625f,-0.39739997836053297941560913386638276278972625732421875f,0.0058000002995133304040109578636474907398223876953125f,0.526799960878491901894449256360530853271484375f,-0.0487999954342843711430077746626920998096466064453125f,-0.67999996304512055900204359204508364200592041015625f,0.60160001795887918518701553693972527980804443359375f,-0.84049995285272505185503177926875650882720947265625f,-0.5361000028192994815157135235494934022426605224609375f,-0.40290004664137979428772950996062718331813812255859375f,0.8111000017575917819812048037420026957988739013671875f,1.113000023800880367019772165804170072078704833984375f,-0.70909998264312756877103538499795831739902496337890625f,0.35889996559023895628115496947430074214935302734375f,-0.87619997729659093721465978887863457202911376953125f,-0.75770002185851248466263996306224726140499114990234375f,-0.02250000286847342323426346410997211933135986328125f,0.95419996900409387308172881603240966796875f,-0.294799991105497039001193115836940705776214599609375f,0.049900027310848305006629743729718029499053955078125f,0.25560000254362813532083009704365395009517669677734375f,0.15330001222044209452377572233672253787517547607421875f,0.1182000045508146968131768517196178436279296875f,-0.75710001398771975544832457671873271465301513671875f,-0.9479999961405993769147926286677829921245574951171875f,-0.544999990463256978046047152020037174224853515625f,0.17339999012500040276307800013455562293529510498046875f,-0.175999991834163704851334841805510222911834716796875f,0.045299988251924272475434918305836617946624755859375f,-0.393200023400783749139009159989655017852783203125f,-0.0730999952420592347124284060555510222911834716796875f,-0.0538000072062014833562670901301316916942596435546875f,-0.86189996702074989087805079179815948009490966796875f,-0.430999972462654312721497262828052043914794921875f,-0.66570005763620143657277594684273935854434967041015625f,-0.8080000211000442078557171043939888477325439453125f,1.1229000085502860439845562723348848521709442138671875f,0.595400013542175532421651951153762638568878173828125f,-0.8652999645590779831394456778070889413356781005859375f,-0.5515999818235639562402639057836495339870452880859375f,0.02960000120997448302961885246986639685928821563720703125f,-0.44999998897314075207987116300500929355621337890625f,0.853500002153217973699383946950547397136688232421875f,0.38139999405145630362312658689916133880615234375f,-0.6263999948710203824475684086792171001434326171875f,-0.42200001227855654661880180356092751026153564453125f,-0.4272000007957219569476592369028367102146148681640625f,0.139000008463859625607028647209517657756805419921875f,0.450299999390542493582501037963083945214748382568359375f,0.559800026905536984855871196486987173557281494140625f,0.2428000125780702678213174294796772301197052001953125f,-0.0897999957621096189797071929206140339374542236328125f,0.62459998887926337918230501600191928446292877197265625f,-0.91710005619525958309168345294892787933349609375f,-0.30890000828206520822050151764415204524993896484375f,0.38389997035860989882394278538413345813751220703125f,0.235999986767768954454282948063337244093418121337890625f,1.155400000280141892261553948628716170787811279296875f,-0.790600010553001997237743125879205763339996337890625f,-0.724300048348308411760854141903109848499298095703125f,-0.31699998966604481243081181673915125429630279541015625f,-0.3591000173628329239505774239660240709781646728515625f,0.2323000064909457318407248749281279742717742919921875f,1.47409999769330024577129734097979962825775146484375f,-0.3343000118777155194038641639053821563720703125f,0.0228000170111657407545635578571818768978118896484375f,0.2496999972939490941570284121553413569927215576171875f,0.03749999675899740037010587911936454474925994873046875f,-0.042400001122057329183689944329671561717987060546875f,1.205199960465729080993924071663059294223785400390625f,0.2721000063136218560799761689850129187107086181640625f,0.83960002579689074764246470294892787933349609375f,-0.374999988414347118492742083617486059665679931640625f,-0.520600056523084386839173021144233644008636474609375f,-0.562899978487193752840767047018744051456451416015625f,-0.9608000053286549491105006381985731422901153564453125f,0.18649999997019761632799372819135896861553192138671875f,-0.5363999797463414598297504198853857815265655517578125f,0.555800004322826612934704826329834759235382080078125f,0.61630002247989212893486410393961705267429351806640625f,0.23690000509321673671792041204753331840038299560546875f,0.0008000362485649503696549800224602222442626953125f,-0.148199997249245429831177034429856576025485992431640625f,-0.2188000240445138189215867896564304828643798828125f,-1.7590999622657894629895736215985380113124847412109375f,-0.991199975061416438393280259333550930023193359375f,1.331299974721670054123023874126374721527099609375f,0.665000004917383424896115684532560408115386962890625f,-0.41179997458755945860531255675596185028553009033203125f,-0.188000000238418341069746020366437733173370361328125f,-0.137500021345913214787515244097448885440826416015625f,0.1761999978601931360344678978435695171356201171875f,-0.5671000148072844471158759915851987898349761962890625f,-0.5717999897122378882841076119802892208099365234375f,0.65520002521127473915640848645125515758991241455078125f,0.19060000801980503393195931494119577109813690185546875f,0.34279998374432307794990038019022904336452484130859375f,0.08699997510015944346406513432157225906848907470703125f,-1.3053999773696063346761775392224080860614776611328125f,-0.401799976003169856397789772017858922481536865234375f,-0.0598000217273833012399109065881930291652679443359375f,-0.39679998911619218660007391008548438549041748046875f,0.635499989844858337306732209981419146060943603515625f,0.414800025415420492436169297434389591217041015625f,0.8548000296622515481459458897006697952747344970703125f,-0.095400009667873320040598628111183643341064453125f,-0.90430000319778913731028069378226064145565032958984375f,-0.3386000066936014629703777245595119893550872802734375f,0.9768000266298653144758645794354379177093505859375f,0.60600001150369653402094627381302416324615478515625f,-0.1024999943003057634172137113637290894985198974609375f,-0.019599996497482674584489359403960406780242919921875f,0.32409999832659952545554915559478104114532470703125f,0.97009999712705674568269387236796319484710693359375f,-0.8063999661862852885718666584580205380916595458984375f,-0.4105000149160613620580306815099902451038360595703125f,0.123100001388788182765665624174289405345916748046875f,-0.396900001665950075135924635105766355991363525390625f,-0.454199965912104042242702917064889334142208099365234375f,-0.929099966028332335810091535677202045917510986328125f,0.033899998894333815968593626166693866252899169921875f,0.38259996859133327262725288164801895618438720703125f,0.36629997747838505262052422040142118930816650390625f,1.1898000163629645076213137144804932177066802978515625f,0.40170000074356781993856202461756765842437744140625f,0.543000030785799392418766728951595723628997802734375f,-0.483900011187791523337153876127558760344982147216796875f,-1.191200030717254332301990871201269328594207763671875f,-0.5509999684020880916790474657318554818630218505859375f,0.074799987566472214695068032597191631793975830078125f,0.39409996353238863076029474541428498923778533935546875f,0.55909999516010255860010147443972527980804443359375f,0.4515999751180412857820556382648646831512451171875f,1.12829997403621629104009116417728364467620849609375f,-0.3491999804779888361139228436513803899288177490234375f,-0.45990002142488928260632974343025125563144683837890625f,-0.441499984398484368597337379469536244869232177734375f,-0.48510000986754864271688347798772156238555908203125f,0.66530000915080289569658589243772439658641815185546875f,0.234500027738512084596322893048636615276336669921875f,0.357099982602148979804468353904667310416698455810546875f,0.214800012935698225735592359342263080179691314697265625f,0.3397000061415134108955271585728041827678680419921875f,0.24360000684261340353486957610584795475006103515625f,-0.321000005617737560470459357020445168018341064453125f,-0.0012000006914139049740697373636066913604736328125f,0.7071999665603041496098057905328460037708282470703125f,0.26999997317791013529131305404007434844970703125f,-1.1099999639391897598983405259787105023860931396484375f,-0.5417999771952626275606235140003263950347900390625f,-0.2361000153362751152741338955820538103580474853515625f,-0.45500000588595879147391087826690636575222015380859375f,1.13329999970346673165977335884235799312591552734375f,0.26919998405426748266933145714574493467807769775390625f,-1.9397999854058027846548384331981651484966278076171875f,-1.0262000042676930888774222694337368011474609375f,0.4728999707385899142764174030162394046783447265625f,0.14599997961521093969849971472285687923431396484375f,0.39219998276531686798307418939657509326934814453125f,0.4031999953866005625968682579696178436279296875f,0.3236999961838125106083907667198218405246734619140625f,0.4886000050544740958002876141108572483062744140625f,0.386800011579692348817616220912896096706390380859375f,0.2401999999165536081591199035756289958953857421875f,-0.1718999801337719457450248228269629180431365966796875f,0.2100000137090685115026644780300557613372802734375f,-0.522799990189075192148493442800827324390411376953125f,-0.345999993622303492202263441868126392364501953125f,0.27090000435560934732848181738518178462982177734375f,0.25619998890087070098076083013438619673252105712890625f,-0.288199997901171400815911738391150720417499542236328125f,-0.816800020539015125820014873170293867588043212890625f,-0.431799991947412475923329111537896096706390380859375f,-1.15180000655055092551037887460552155971527099609375f,-0.219899991808831796191725516109727323055267333984375f,-0.2554999778866768878060611314140260219573974609375f,-0.351699991802871370083494184655137360095977783203125f,-0.67180001027584079764665148104541003704071044921875f,0.5178000104174014239788448321633040904998779296875f,1.688700050647557571181778257596306502819061279296875f,-0.7025999996975063055515420273877680301666259765625f,-0.126600034182519027847746428960817866027355194091796875f,0.086100001906603551571350863014231435954570770263671875f,0.13739998910799666731463730684481561183929443359375f,-0.9082000128582115028308408000157214701175689697265625f,-0.370200031656028105686573326238431036472320556640625f,-0.8621000002786514126995598417124710977077484130859375f,-1.801500028729439151220503845252096652984619140625f,-0.586499985925853284385311781079508364200592041015625f,-1.7988000115275379897639140835963189601898193359375f,-0.0974999928474427068891827730112709105014801025390625f,-0.6018000129580496349035456660203635692596435546875f,0.5363999949082731699689929882879368960857391357421875f,0.06180002338886314561250401311554014682769775390625f,-0.554599976176023634621969904401339590549468994140625f,-0.787000028073787660787274944595992565155029296875f,0.419299984087050159331511167692951858043670654296875f,1.6754999637305736381875931328977458178997039794921875f,0.615499989323317731049201029236428439617156982421875f,0.673599996560812375179239097633399069309234619140625f,-0.80889998943209651116603708942420780658721923828125f,-0.074199995863437617771296572755090892314910888671875f,0.7908000052914021527072918615886010229587554931640625f,0.3996000043392182732304718228988349437713623046875f,0.71430003009438525207741577105480246245861053466796875f,0.487599998915194898785330224200151860713958740234375f,-0.6910000366866591026138166853343136608600616455078125f,0.293399999773502173638917156495153903961181640625f,0.1861000019997358645440499458345584571361541748046875f,-0.2792000188112258030059820157475769519805908203125f,-1.243400023615360172613009126507677137851715087890625f,-0.36699999946355799096409100457094609737396240234375f,1.3196000084370371752129358355887234210968017578125f,0.368999994456768121153800166212022304534912109375f,-0.26029999890625443725866716704331338405609130859375f,0.18919998394250825413109851069748401641845703125f,0.320100011618435598137466513435356318950653076171875f,0.11540000638961789292125104111619293689727783203125f,0.9744000283762812575361067501944489777088165283203125f,1.0456000384926795732098980806767940521240234375f,-0.54759998285919397886800652486272156238555908203125f,0.22540004572868355126047390513122081756591796875f,1.5368999620661136784605105276568792760372161865234375f,0.7645999933123590608374797739088535308837890625f,0.05970002270042906733493737192475236952304840087890625f,0.17579999161958692610596699523739516735076904296875f,-0.8459000267446044052377374100615270435810089111328125f,0.0504000191301112732134015459450893104076385498046875f,-0.04129999405592688077604179852642118930816650390625f,-1.171599994787573706389594008214771747589111328125f,-0.249599997298419473690245240504737012088298797607421875f,-0.380600008690356705898238942609168589115142822265625f,-0.2095999950259923583217869236250407993793487548828125f,-0.78160002108812332011211765347979962825775146484375f,0.5468999811396006993646778937545605003833770751953125f,-0.703199999260902597342237640987150371074676513671875f,-0.50040001112073628775078759645111858844757080078125f,-0.32680005527734845571785626816563308238983154296875f,-0.10419999459683904063922454952262341976165771484375f,-0.33129997546672829145109062665142118930816650390625f,-0.42679999481588615228844219018355943262577056884765625f,0.02630001227259715079753732425160706043243408203125f,-0.603099995428323598645192760159261524677276611328125f,-1.7367000024199494845333902048878371715545654296875f,-0.754899988456070314413182131829671561717987060546875f,-1.10340004537105595971979710157029330730438232421875f,0.63250000737607459111444541122182272374629974365234375f,-0.11849996918439753557095173164270818233489990234375f,0.92649999933689830555749722407199442386627197265625f,-0.435999997943639527875347994267940521240234375f,0.74160001017302246140161514631472527980804443359375f,0.73770003672242268066838732920587062835693359375f,0.42289998306930076665821616188623011112213134765625f,-0.204399994289875763797681429423391819000244140625f,0.53919999515563243885907240837696008384227752685546875f,-0.28379996354579883899305059458129107952117919921875f,0.376200009706616322890937453848891891539096832275390625f,0.29179998881816882061457363306544721126556396484375f,-0.4783999835312366943895767690264619886875152587890625f,0.1390000147223471316237919381819665431976318359375f,-0.3109000131845471859293184024863876402378082275390625f,-0.00499999463558165047061265795491635799407958984375f,0.917599980847537466388530447147786617279052734375f,0.2905999815702440258746719337068498134613037109375f,-0.70729998460486520617251926523749716579914093017578125f,-0.50480001762956394628645284683443605899810791015625f,0.21529997161850300546603875773143954575061798095703125f,-0.1184000417947768113435813575051724910736083984375f,-1.03699998970329776426524404087103903293609619140625f,0.07480002303123445273058678139932453632354736328125f,1.3153000104546548332251632018596865236759185791015625f,0.4075999971270558575042741722427308559417724609375f,-0.4411000206634401177296922469395212829113006591796875f,0.0461999898135661002385177198448218405246734619140625f,0.0473000051498413398576303734444081783294677734375f,-0.0710000077784063154950899843242950737476348876953125f,0.19299998105317384133883251706720329821109771728515625f,0.431399993455409802578515154891647398471832275390625f,-1.003600033105910060982068898738361895084381103515625f,-0.71120001589059800295444802031852304935455322265625f,-0.163799987722933249045809134258888661861419677734375f,-0.2135999892592430615678722460870631039142608642578125f,0.9515999889388677690504891870659776031970977783203125f,0.29379999245405219454596590367145836353302001953125f,-0.0173999919950962311787634462234564125537872314453125f,-0.30800001148879541101877066466840915381908416748046875f,0.2502000015184282499802748134243302047252655029296875f,-0.4144999911189077845818928835797123610973358154296875f,0.7658999829351902377538863220252096652984619140625f,0.6188000382006162425341244670562446117401123046875f,-0.5216999683707956858569332325714640319347381591796875f,-0.5653999700307839049173708190210163593292236328125f,-0.1820999959945679425032949438900686800479888916015625f,-0.1497000002928080830688628566349507309496402740478515625f,-0.401399992319196485812682340110768564045429229736328125f,0.34059998829439308565270039252936840057373046875f,-0.7824000021278856298323489681934006512165069580078125f,-1.9378000235676768880921372328884899616241455078125f,0.76839996453225645201001725581591017544269561767578125f,1.338600013250111686602394911460578441619873046875f,0.21339999850690372085182389128021895885467529296875f,-0.49069998205006160052477071076282300055027008056640625f,-0.4491999923244118786413991983863525092601776123046875f,-0.5581000024318694130442963796667754650115966796875f,-0.6958999883741139313997337012551724910736083984375f,-1.54919999794960094874340938986279070377349853515625f,-0.1144999920129776693755729866097681224346160888671875f,-0.2007000044763085355725706904195249080657958984375f,-0.76549999506026511486567187603213824331760406494140625f,-0.3403000015139581169165694518596865236759185791015625f,-0.0022000028818845718259211707845679484307765960693359375f,0.3677000074416401087518124768394045531749725341796875f,-0.35489999057948595773126498897909186780452728271484375f,-0.713500012546777639954598271287977695465087890625f,0.794199976044893229953913760255090892314910888671875f,0.05379998708963373843516819761134684085845947265625f,-1.1789000079840423751420530606992542743682861328125f,-0.7219999881386751638956411625258624553680419921875f,0.35400000801682474094178587620262987911701202392578125f,-0.1554000107109547723638343086349777877330780029296875f,-0.350000012069940691361580320517532527446746826171875f,-0.099199984538554719648573154699988663196563720703125f,1.3355000450909140230493221679353155195713043212890625f,0.44040000497400744183806864384678192436695098876953125f,-0.13820002243220808058055126821273006498813629150390625f,0.233999984025955232169735609204508364200592041015625f,-0.43870001063793873097296227570041082799434661865234375f,0.190999996453523390727013975265435874462127685546875f,0.2020000025928020692678188652280368842184543609619140625f,-0.0095999927163123999207527958787977695465087890625f,-0.54519999011903974572845754664740525186061859130859375f,-0.33239998350143462602090949076227843761444091796875f,-0.0066000152766705344475894889910705387592315673828125f,-0.39499999880790692685650355997495353221893310546875f,0.57740000074952846365050618260283954441547393798828125f,0.3398000041067599230615314809256233274936676025390625f,-0.79919998710900530536349606336443684995174407958984375f,-0.568399994707107225622166879475116729736328125f,-0.42500001467764381146707819425500929355621337890625f,-0.2847999877154829828640458799782209098339080810546875f,1.00599999727308730967934025102294981479644775390625f,0.139799989652633627201794297434389591217041015625f,-1.138600013957917322926505221403203904628753662109375f,0.24779997983276846884592714559403248131275177001953125f,-0.73889999859631039225149606863851658999919891357421875f,-0.80610008045434966561515466310083866119384765625f,0.223300001044571150288220451329834759235382080078125f,1.4262000422656539999621827519149519503116607666015625f,-0.6018999963387845486550986606744118034839630126953125f,-0.85530003865510251248593931450159288942813873291015625f,0.7018999960407612537238719596643932163715362548828125f,1.4265000539496544007533884723670780658721923828125f,0.310200010496377853730365359297138638794422149658203125f,-0.36719998678863030594499150538467802107334136962890625f,-0.366399990177154577519758049675147049129009246826171875f,-1.4236999764770275067604643481899984180927276611328125f,0.0039999987408518844489435650757513940334320068359375f,-0.03359999499619004126316212932579219341278076171875f,0.1218000031232833180183661170303821563720703125f,0.28220000578761084142342951963655650615692138671875f,0.43029999275952579562698474546778015792369842529296875f,1.501000000178814186568843069835565984249114990234375f,-0.3356999915122986788418302239733748137950897216796875f,-0.27319999248087400989248862970271147787570953369140625f,0.3662999917089939660996833481476642191410064697265625f,0.5156999963551758270341451861895620822906494140625f,-0.720799964573979767834543963545002043247222900390625f,-0.4587999992340794364054090692661702632904052734375f,0.3801999724239113476187412743456661701202392578125f,1.3142999967157835694564482764690183103084564208984375f,-0.0939000018000601954781814129091799259185791015625f,-0.81599996888637615910511158290319144725799560546875f,-0.8947000251591206154699875696678645908832550048828125f,0.015200020778179723635048503638245165348052978515625f,1.0894000228255988904635387370944954454898834228515625f,0.472800009560585099421814447850920259952545166015625f,0.33549999610334635402608682852587662637233734130859375f,0.52920001672506344903013086877763271331787109375f,-0.1196000081017614125045156470150686800479888916015625f,-0.044000001311302217033016859204508364200592041015625f,0.324699989619851248079385186429135501384735107421875f,0.33139999889433380264591733066481538116931915283203125f,0.546100005315243830494864596403203904628753662109375f,-0.0110000025629997111309421597979962825775146484375f,-0.28210000486075870185231906361877918243408203125f,0.3063999977767466464939616344054229557514190673828125f,1.0408000025719401637758210199535824358463287353515625f,0.7182000153541565623527276329696178436279296875f,-0.6322000000134109232163837077678181231021881103515625f,0.01280001462697999414785954286344349384307861328125f,-0.3841000102475284894154583525960333645343780517578125f,-0.2549999895691872353609142010100185871124267578125f,0.7981999799638987980188176152296364307403564453125f,0.3312000033736222093239121022634208202362060546875f,1.4493000360921026725691262981854379177093505859375f,0.58860002263784405585056447307579219341278076171875f,-0.5853000285595657548043391216197051107883453369140625f,-0.3006000241875650402789688087068498134613037109375f,0.1390000055581330773435411174432374536991119384765625f,0.156999987542629337866628702613525092601776123046875f,-0.78450002282112851759166005649603903293609619140625f,-1.32270005683898972392853465862572193145751953125f,0.792299997811019007798449820256792008876800537109375f,1.0933000775992873787600956347887404263019561767578125f,0.2991000083103774187520684790797531604766845703125f,-0.6740999666988845007153940969146788120269775390625f,0.365300000508129496523679335950873792171478271484375f,-0.0538999761700627022520393438753671944141387939453125f,-0.40480000090301027082517748567624948918819427490234375f,-0.9634999884441499684584186979918740689754486083984375f,-0.35539998962581165198315602538059465587139129638671875f,-0.28130003712028184281734866090118885040283203125f,0.5111999757692220480720379782724194228649139404296875f,0.00120002103149807748394550799275748431682586669921875f,-0.1895999991983174925991306736250407993793487548828125f,-0.92750001288950389977117083617486059665679931640625f,0.8742999769717456626239027173141948878765106201171875f,0.06210001506805351567663819878362119197845458984375f,0.42249999996274689717523642684682272374629974365234375f,0.507000056385994213314916123636066913604736328125f,-0.542199995617568486494519675034098327159881591796875f,-0.67579997716546102992651867680251598358154296875f,0.4089999668896202589252197867608629167079925537109375f,0.4336999819159501612375606782734394073486328125f,0.54000001642853001637689658309682272374629974365234375f,0.40329999907016844673535160836763679981231689453125f,-0.5418000133678313456897512878640554845333099365234375f,-0.562100044274330912230652756989002227783203125f,-0.110199993397295525898726964442175813019275665283203125f,-1.1371999792635438808474646066315472126007080078125f,0.685499997444451114603225505561567842960357666015625f,0.39580000614821908566653974048676900565624237060546875f,1.0727999933928249465253657035646028816699981689453125f,0.45079998655319197808921671821735799312591552734375f,-1.3054999778866764881257722663576714694499969482421875f,-0.1661999845981600998356952914036810398101806640625f,0.85109996688887468163642324725515209138393402099609375f,0.311599999220668888444407684801262803375720977783203125f,-0.3021999970890580922144863507128320634365081787109375f,-0.28899999892711658588950740522705018520355224609375f,0.630899982823431404455050142132677137851715087890625f,0.13379997822344347202516701145214028656482696533203125f,-0.2210999776735900201884987836820073425769805908203125f,-0.1432000210165984555032991920597851276397705078125f,-0.3046000052705408744913029295275919139385223388671875f,-0.212000024497509542698026052676141262054443359375f,0.111800007854402139173544128425419330596923828125f,0.21379999662637683144339462160132825374603271484375f,-1.34970002687275414388068384141661226749420166015625f,-0.542000011682510329791284675593487918376922607421875f,-0.734699998225271766472133094794116914272308349609375f,-0.673399994707107651947808335535228252410888671875f,0.6049999951198696823695399871212430298328399658203125f,0.004600005531310902284758412861265242099761962890625f,-0.2683999957501888022903813180164434015750885009765625f,-0.28419999824762331996907960274256765842437744140625f,0.41179997440129501029559833114035427570343017578125f,0.018199972327053892673376367383752949535846710205078125f,-0.8479999994002282992511254633427597582340240478515625f,-1.0854999873861672643471365518053062260150909423828125f,0.47730000332444888488936385328997857868671417236328125f,0.5323000218018894003080276888795197010040283203125f,0.92959996894896057550994328266824595630168914794921875f,0.1569000394403925469077876186929643154144287109375f,-0.51719998354762797276862329454161226749420166015625f,-0.5254000108040866034997407041373662650585174560546875f,0.515999968457967472801328767673112452030181884765625f,0.736299984351544534177946843556128442287445068359375f,0.530599985183775491037749816314317286014556884765625f,1.306100020737945843762872755178250372409820556640625f,-0.50689997745156312181080693335388787090778350830078125f,-0.59869999108016502820106552462675608694553375244140625f,0.57149997050315182178792383638210594654083251953125f,1.18229999516606287812692244187928736209869384765625f,-0.7942000079333777495804724821937270462512969970703125f,-1.062700073379279075425074552185833454132080078125f,0.39989999370872986172997798348660580813884735107421875f,0.247099999719857965629898899351246654987335205078125f,-0.7576999929502605812814408636768348515033721923828125f,-0.147800024461746115633786757825873792171478271484375f,-0.454300003197789070696899216272868216037750244140625f,0.376799971085786555846652845502831041812896728515625f,0.6557999795123936603857828231411986052989959716796875f,1.179900009876489352933504051179625093936920166015625f,-0.24949998880922807931170837036916054785251617431640625f,0.1940000030435624633895486113033257424831390380859375f,-0.6141999895304441903221004395163618028163909912109375f,-1.453899990288913368630119293811731040477752685546875f,0.97520000537410378438352154262247495353221893310546875f,0.022199991221725501322481477473047561943531036376953125f,-0.51399999730661516395002763601951301097869873046875f,-0.4644000023365020268784064683131873607635498046875f,-0.006699992994964099057852990881656296551227569580078125f,0.1054000171184537038016060250811278820037841796875f,0.5178000073254109825171553893596865236759185791015625f,-0.04639999487102020037099237015354447066783905029296875f,0.806699986587464668019720193115063011646270751953125f,0.8066000103592880776659512775950133800506591796875f,0.4542000635147100950916865258477628231048583984375f,0.1261999971151350763420850853435695171356201171875f,-0.951000013776123420683461517910473048686981201171875f,-0.3374000063002109417453766582184471189975738525390625f,0.563900027914345347568314537056721746921539306640625f,0.1557999976545569875696628514560870826244354248046875f,-0.6449000413924454999659019449609331786632537841796875f,0.24359998337328392370437768477131612598896026611328125f,-0.058199999111890943215286142731201834976673126220703125f,0.01380001603513958396973748676828108727931976318359375f,-0.4619999981969595825859187243622727692127227783203125f,-0.547999977290629658455145545303821563720703125f,0.8271000053450465738791308467625640332698822021484375f,0.507400012409686684833332037669606506824493408203125f,-0.5844000227883459075428618234582245349884033203125f,-0.49260000405609549734009533494827337563037872314453125f,1.104300008934736343491067600552923977375030517578125f,0.6802000119864939353675481470418162643909454345703125f,-0.1213000057160857902971429211902432143688201904296875f,0.6485999970264730851710055503644980490207672119140625f,0.474399967002124001158591681814868934452533721923828125f,1.09199997228756462419596573454327881336212158203125f,0.1368999973073601328366066809394396841526031494140625f,0.390600018972158569141583939199335873126983642578125f,0.41049998399615306166055006542592309415340423583984375f,1.0296999752029771801886681714677251875400543212890625f,0.78080000029951313056386652533547021448612213134765625f,0.4807000429585581713354258681647479534149169921875f,0.6725000146031374637090038959286175668239593505859375f,0.58760004972815504942218467476777732372283935546875f,0.94300001562386714937247234047390520572662353515625f,1.215700042846799444618000052287243306636810302734375f,0.1393000025272368436191072760266251862049102783203125f,0.79400000716000829203977673387271352112293243408203125f,0.03360000453293332522974878884269855916500091552734375f,-1.0309999749213460962238286811043508350849151611328125f,-0.66440000542849275699808231365750543773174285888671875f,-1.3295000359527768818423965058173052966594696044921875f,-0.01140000185593958992313901035231538116931915283203125f,0.513599948709458242745995448785834014415740966796875f,0.739500004418193856992047585663385689258575439453125f,1.2454000873029238416478392537101171910762786865234375f,0.6472999721437695530568134927307255566120147705078125f,0.727599995450674885688613358070142567157745361328125f,-0.68469997509121938161769094222108833491802215576171875f,-0.905599961602687297812508404604159295558929443359375f,0.7184999826699496328075156270642764866352081298828125f,0.555699995461106599492495661252178251743316650390625f,1.4443000341176988055025276480591855943202972412109375f,0.61420001642704047384313525981269776821136474609375f,0.5500000084564089508631923308712430298328399658203125f,0.2649999839067458395902576739899814128875732421875f,-0.22279999961406016240061944699846208095550537109375f,0.4443999990582465642319220933131873607635498046875f,0.3917000172838569493904969931463710963726043701171875f,0.284599993684887753175871694111265242099761962890625f,-0.984200018289685285566292805015109479427337646484375f,-0.27880000258684134450959390960633754730224609375f,0.8630000069066880996615509502589702606201171875f,0.3155999872326855637538756127469241619110107421875f,-0.2034000037595629084563597643864341080188751220703125f,0.37379999483823755923594944761134684085845947265625f,0.02770000677108763742495511905872263014316558837890625f,0.1400000108778478580262571995262987911701202392578125f,0.2068000302433967707571582650416530668735504150390625f,-0.264800000232458199178608992951922118663787841796875f,-1.1914000194206837424104605815955437719821929931640625f,-0.6352000124335290820454247295856475830078125f,-0.80309998309761265744555203127674758434295654296875f,-0.60099999034404749664872724679298698902130126953125f,0.334499989740550507377747635473497211933135986328125f,-0.36720001487731934730618377216160297393798828125f,0.1922999945513901298710379705880768597126007080078125f,0.402600009457766805365253048876184038817882537841796875f,-0.8384000140227378938817537346039898693561553955078125f,-0.4521999993801113504332533921115100383758544921875f,-0.298399994111061273027729612294933758676052093505859375f,0.30100001180171975789789939881302416324615478515625f,0.84270001176297626432898368875612504780292510986328125f,0.82140003637075498232889003702439367771148681640625f,0.5442999867320061913034123790566809475421905517578125f,-0.299299979281425265753568965010344982147216796875f,-0.2567000047668812623413714391062967479228973388671875f,-0.84400003293901726397763241038774140179157257080078125f,0.0858000027954578803335294878706918098032474517822265625f,-0.3856000138685107270219987185555510222911834716796875f,-0.14099999637901794447003567256615497171878814697265625f,0.191200010973215484000320429913699626922607421875f,-0.660399971073866165482968426658771932125091552734375f,-1.96169998736977557740601696423254907131195068359375f,0.37709998280704037920685323115321807563304901123046875f,-0.01889999502003225639867878271616064012050628662109375f,0.254000005856156174655069435175391845405101776123046875f,0.9445999900341039978712842639652080833911895751953125f,0.1137000011757016704194711564923636615276336669921875f,0.576899976333976383102708496153354644775390625f,-0.8754000130951400393541916855610907077789306640625f,-0.3885000624656687051583503489382565021514892578125f,-0.73849997313320658864910228658118285238742828369140625f,0.36729997043013629109964313101954758167266845703125f,-0.2704999981895088723149456200189888477325439453125f,-0.5350000104866918793078411908936686813831329345703125f,0.74429996458515557122836980852298438549041748046875f,0.749999986793845163646210494334809482097625732421875f,-0.27609999626278891593500475210021249949932098388671875f,0.5697999926328662922969670034945011138916015625f,1.1685000289753080249255390299367718398571014404296875f,0.5288000075042245118339678811025805771350860595703125f,-1.029900006709992599240877098054625093936920166015625f,-0.22119999480545493586447491907165385782718658447265625f,0.314900002425909153469518741985666565597057342529296875f,0.1410000122487546914129552533267997205257415771484375f,0.599600030080974288893003176781348884105682373046875f,0.02439998713731750257238672929815948009490966796875f,-1.4110999960020185195475050932145677506923675537109375f,-0.3755999782919889895538290147669613361358642578125f,1.529999998547136641491306363604962825775146484375f,0.4629999963641164839600605773739516735076904296875f,-0.754600006127357669782895754906348884105682373046875f,0.39260002692937856494381776428781449794769287109375f,-0.017999981537461184899484578636474907398223876953125f,-0.01679996080398549196388557902537286281585693359375f,0.30459997327029686431387744960375130176544189453125f,-0.0697999921858312166733639969606883823871612548828125f,0.24489998808354129788966702108155004680156707763671875f,-0.023800003480911158249000436626374721527099609375f,-0.218499989598989696748532196579617448151111602783203125f,-0.020399969434738096651926753111183643341064453125f,0.5356000086903571766328013836755417287349700927734375f,-0.20999999482184639187920538461185060441493988037109375f,0.97480000582039350032204083618125878274440765380859375f,0.70760000308752069031470455229282379150390625f,-1.294200010503828490726618838380090892314910888671875f,-0.46040000601708930272337738642818294465541839599609375f,0.7383000296548012375552616504137404263019561767578125f,1.425899990385770532697051748982630670070648193359375f,-0.8558000181436540287904790602624416351318359375f,0.82150000637769693412337801419198513031005859375f,0.02009997358322124227925087325274944305419921875f,-0.35730000492632374431423158966936171054840087890625f,1.1918000052839516911262762732803821563720703125f,-0.13339998971521860315903040827834047377109527587890625f,-0.056999986350536602230931748636066913604736328125f,0.4530999776959421243560655057081021368503570556640625f,-0.32599998706579214058365323580801486968994140625f,0.283900000756978965199550657416693866252899169921875f,0.13679998479485533380284323357045650482177734375f,-0.5630000263154506523477493828977458178997039794921875f,0.63260000755786904846900142729282379150390625f,1.193300004434585392942835824214853346347808837890625f,-1.5350999807357794679774087853729724884033203125f,-0.70450000259280187719923560507595539093017578125f,-0.775500006198882374519598670303821563720703125f,-0.0806000033631921741772430323180742561817169189453125f,-0.7376999994322659404133446514606475830078125f,-0.049500008143484652833876680233515799045562744140625f,0.21120000821650020128572577959857881069183349609375f,-0.561500001274049420629808082594536244869232177734375f,0.02030001015663174257497303187847137451171875f,0.000799995493888872033494408242404460906982421875f,-0.6873999846935276991644059307873249053955078125f,-0.32209999029487368371604816275066696107387542724609375f,-0.003599998870492004954257936333306133747100830078125f,-0.0556999932073056758241591523983515799045562744140625f,-0.62240003269389310158743455758667550981044769287109375f,0.17320000618994224605984300069394521415233612060546875f,0.2746999955989419817115049227140843868255615234375f,0.58719998507499671092091375612653791904449462890625f,1.8227000137746329766486041989992372691631317138671875f,-0.0293999992072582205793196408194489777088165283203125f,-0.10370001007914542157095638685859739780426025390625f,-0.55120000605583197028636277536861598491668701171875f,0.137199952217936793630315150949172675609588623046875f,0.20699999730288975996472800034098327159881591796875f,-0.347700002487003778650631602431531064212322235107421875f,0.8615999968737362291193448982085101306438446044921875f,0.12469998578280172552013027598150074481964111328125f,0.513400011992454441411837251507677137851715087890625f,1.1927000507295133502339012920856475830078125f,-0.2609999988377094126690280972979962825775146484375f,0.742099992120266005457551727886311709880828857421875f,0.31139998257756218436043127439916133880615234375f,-1.371899970149994540946636334410868585109710693359375f,0.280199988293647894721516422578133642673492431640625f,1.018299995195865559338699313229881227016448974609375f,-0.032799997043609696589783197850920259952545166015625f,0.32890003234744114024579175747931003570556640625f,0.215200017946958865877604694105684757232666015625f,-1.38079997910261109694829428917728364467620849609375f,-0.03219999848604171432953080511651933193206787109375f,0.7336000102698800873213258455507457256317138671875f,-0.5178000034511089921807069913484156131744384765625f,-1.536999977558851870895750835188664495944976806640625f,1.111200007862597427532591609633527696132659912109375f,-0.546200023229420406067902149516157805919647216796875f,1.2282999924205244557384730796911753714084625244140625f,-0.2574000532388680539952474646270275115966796875f,-0.1178000237166880737049723393283784389495849609375f,0.7470999988257887025611125864088535308837890625f,0.4046999812006950580922648441628552973270416259765625f,-0.059599988245963686495088040828704833984375f,-0.43650001874566068948979591368697583675384521484375f,0.28509998766481903231806427356787025928497314453125f,1.0132999880060553632432629456161521375179290771484375f,0.331599956305325349603663198649883270263671875f,1.0931000117823479467205061155254952609539031982421875f,-0.5374000308871274000921403057873249053955078125f,0.069499999314546645479140352108515799045562744140625f,0.378700007545948125198265188373625278472900390625f,-0.978000005900859559204718607361428439617156982421875f,1.437299969387055398328811861574649810791015625f,0.736899952715635464528531883843243122100830078125f,-0.3365999847292897584338788874447345733642578125f,-0.315599990957975418126579825184307992458343505859375f,1.0533999860644343016247148625552654266357421875f,-0.509800000679492715249807588406838476657867431640625f,1.3625000274181360282454988919198513031005859375f,0.31950001466274269290579468361102044582366943359375f,-0.749399955248833293808274902403354644775390625f,-1.57139995694756517963242004043422639369964599609375f,0.5695000081062318031399627216160297393798828125f,0.81569998868107784772973900544457137584686279296875f,0.11789999640584003515186850563623011112213134765625f,-0.47160001497864723063457859097979962825775146484375f,-0.156599978917837834302417832077480852603912353515625f,0.242799987953901119652755369315855205059051513671875f,-0.41980000380873738752285362352267839014530181884765625f,-0.36439999384284027517111326233134604990482330322265625f,-0.4544999913424252468985287123359739780426025390625f,-0.097999998301267599032371435896493494510650634765625f,0.325400024867058323962965005193836987018585205078125f,0.2924000133037569781890852027572691440582275390625f,-0.4529999758005143206673892564140260219573974609375f,0.0426000021934509476295716012828052043914794921875f,-0.5648000180393462077432786827557720243930816650390625f,0.3690000036954879103490156921907328069210052490234375f,1.100700014199316623120239455602131783962249755859375f,-0.73220000653266925638718021218664944171905517578125f,-1.652800001662969808791103787370957434177398681640625f,0.620200007814168952080535746063105762004852294921875f,0.19070003338456142927270775544457137584686279296875f,0.6359999803602693901893871952779591083526611328125f,0.320399980014562490993057508603669703006744384765625f,-0.189400025731325083455658386810682713985443115234375f,0.255499960452317598225135952816344797611236572265625f,-0.038200007754564335726854551467113196849822998046875f,-0.329700020688772355015316861681640148162841796875f,0.82300000038743004182606455287896096706390380859375f,0.32870002617239979514351944089867174625396728515625f,-0.17380000124573680153616805910132825374603271484375f,1.13879997621178663536056774319149553775787353515625f,-0.80829999579191191827476359321735799312591552734375f,-0.07869997744560208730035810731351375579833984375f,-0.846600013637542669897584346472285687923431396484375f,1.4140999692320832537006936036050319671630859375f,0.02030000218451011306797227007336914539337158203125f,0.330999999210238371460945927537977695465087890625f,-0.32380000683367249347810457038576714694499969482421875f,0.10090000811815258430215180851519107818603515625f,-0.6542999892652032745132828495115973055362701416015625f,1.0276000188827509873590315692126750946044921875f,0.66480002452135078527817313442938029766082763671875f,0.381199994134902908626827411353588104248046875f,0.4121999934196469705938170591252855956554412841796875f,-0.62010001657307167732824382255785167217254638671875f,-1.33049998105317346386300414451397955417633056640625f,0.2460000034570697380331694148480892181396484375f,0.29319999702572818733870008145458996295928955078125f,-0.05690001991987259089000872336328029632568359375f,1.483399975186586505770947042037732899188995361328125f,0.0734999587535867249243892729282379150390625f,0.787399967706203796069530653767287731170654296875f,-1.23449997800588562313350848853588104248046875f,-0.334600013726949807590926866396330296993255615234375f,-0.68889999881982788565437658689916133880615234375f,1.086400001055002118022230206406675279140472412109375f,-0.1655000509023665244967560283839702606201171875f,1.2147999793708326166097322129644453525543212890625f,-1.3898999554872517592229996807873249053955078125f,-0.87010000830292710816138423979282379150390625f,-0.43979996044635694119051549932919442653656005859375f,0.01679998106956492875951880705542862415313720703125f,0.19259998572766789948218502104282379150390625f,-0.261999986425041964110960179823450744152069091796875f,-0.054099957497417694440855484572239220142364501953125f,0.16319998883008945966821556794457137584686279296875f,-0.737900016522407664609772837138734757900238037109375f,-0.237599996531009605149620256270281970500946044921875f,0.412300014835596417839269633986987173557281494140625f,-0.210599996173381764918985936674289405345916748046875f,-0.925599977695942044420007732696831226348876953125f,0.3298000224351884668294587754644453525543212890625f,0.7001000022679573220329984906129539012908935546875f,-0.01119999026060103375357357435859739780426025390625f,0.452999981164932563615366234444081783294677734375f,-0.11379999483823777239877017564140260219573974609375f,-0.07959998694211234404605193049064837396144866943359375f,-0.09399998886883265125646857995889149606227874755859375f,-1.7383000096686183066907460670336149632930755615234375f,0.543000007092952596821078259381465613842010498046875f,-0.1438999477833509121893484916654415428638458251953125f,-0.49500001877546306872091008699499070644378662109375f,0.189599967831372229198905188241042196750640869140625f,0.562399989610910200354965127189643681049346923828125f,1.10570001192688938118635633145458996295928955078125f,0.718800022107362934775665053166449069976806640625f,-0.3092999827086921893481985534890554845333099365234375f,0.1088000036299228145963979841326363384723663330078125f,0.24550001517683284202320237454841844737529754638671875f,0.34579998565912273278399879927746951580047607421875f,-0.918399973845481554235448129475116729736328125f,-1.20739998186230668153484657523222267627716064453125f,-0.23999996311962679129692332935519516468048095703125f,-0.530500003218650650893550846376456320285797119140625f,1.1439999810457237572336453013122081756591796875f,-0.031100011312961584053482511080801486968994140625f,-0.401399978256225864470252417959272861480712890625f,-1.1206999760150908596045837839483283460140228271484375f,0.81900000876188272513900301419198513031005859375f,0.806600000673532324668713044957257807254791259765625f,0.3851000041306011922870311536826193332672119140625f,0.60950001123547548065317869259160943329334259033203125f,0.37139999613761887076179846189916133880615234375f,-0.7647999988913536384416147484444081783294677734375f,0.177200006234645712765995995141565799713134765625f,-1.134499994173645998074562157853506505489349365234375f,0.726999998308718087258739615208469331264495849609375f,0.4425000238604841484857388422824442386627197265625f,-0.4498000345051291759546074899844825267791748046875f,0.6188000101119282003736543629202060401439666748046875f,-1.0853999783754346708519733510911464691162109375f,0.47539998359084112422578982659615576267242431640625f,0.026899951374531383407884277403354644775390625f,0.072599984312057497248815707280300557613372802734375f,0.7441999969065182796157387201674282550811767578125f,1.023100023144483561310380537179298698902130126953125f,-0.25300001282989992290595182566903531551361083984375f,0.29579998219460257313784268262679688632488250732421875f,-0.084499969884753500792839986388571560382843017578125f,-0.292399986258149002793516046949662268161773681640625f,-0.256000014968215783284222197835333645343780517578125f,-0.07079997805953031075887338374741375446319580078125f,0.38830004228353498518799824523739516735076904296875f,-0.6254000126481056298644034541212022304534912109375f,-0.1605000614821907589657712378539144992828369140625f,0.21560000019669534054855830618180334568023681640625f,-0.39629998477995354022596075083129107952117919921875f,-0.4696000050842759065972131793387234210968017578125f,-0.231200047779083206478389911353588104248046875f,0.9504000046014784430781219271011650562286376953125f,1.425400022333860761136747896671295166015625f,0.02900002083182329215560457669198513031005859375f,-0.093499972797930208656680406420491635799407958984375f,0.4250000106915832187581827383837662637233734130859375f,-0.2923999780811363802257574207033030688762664794921875f,-0.6113999925613402552926345379091799259185791015625f,0.034299966913462043294202885590493679046630859375f,-0.84359999179244038458591603557579219341278076171875f,0.564799967971444605296937879757024347782135009765625f,0.146600031891465254485495961489505134522914886474609375f,-0.434499964557587770030977480928413569927215576171875f,-0.73680002458095561479467505705542862415313720703125f,-0.7153000442802908143136164653697051107883453369140625f,0.4687999923050403250357476281351409852504730224609375f,-0.337900027772784550261775393664720468223094940185546875f,-0.039600011566281512198628433907288126647472381591796875f,0.576900020180642680855953585705719888210296630859375f,-0.520400012499094177798042437643744051456451416015625f,-0.21119996470511015473903171368874609470367431640625f,0.183500064253806882685893242523889057338237762451171875f,0.48209998049735958147721248678863048553466796875f,0.10479996700286875466190394945442676544189453125f,-0.0099999761581415214095613919198513031005859375f,-0.12140000820755947330553681240417063236236572265625f,0.1061999936878681438656712998636066913604736328125f,0.19359998151063939619831444360897876322269439697265625f,-1.1734999384880069328573881648480892181396484375f,0.0240999921798703553577070124447345733642578125f,0.0619999556541444007962127216160297393798828125f,0.38810003879070276155971441767178475856781005859375f,1.118299977016448565336759202182292938232421875f,-0.264700018975138728904994422919116914272308349609375f,0.557999997518956458719685542746447026729583740234375f,0.4601000159025192548511995482840575277805328369140625f,-0.216599971914291700159083120524883270263671875f,-0.315299985569715612854224673355929553508758544921875f,-0.452899970030784970731474459171295166015625f,-0.8590999644264580847874412938836030662059783935546875f,1.0083999919876447393107810057699680328369140625f,0.009799987752735772605916508837253786623477935791015625f,-0.5209999808073035865163546986877918243408203125f,-0.02900002738833418192143653868697583675384521484375f,-0.18250001624226541707685100845992565155029296875f,0.5989999997839330347915165475569665431976318359375f,-1.196000017277896176892681978642940521240234375f,-0.476400021953880703495798343283240683376789093017578125f,1.367000013321637563734611831023357808589935302734375f,-0.526199990409612627928481742856092751026153564453125f,-0.1788999913692475018933691899292171001434326171875f,-0.2223999984025955001243346487171947956085205078125f,-1.013599996709823702900621356093324720859527587890625f,-0.33640000612139697722113851341418921947479248046875f,0.350599987083672903764863804099150002002716064453125f,-0.017200001764297478956677878159098327159881591796875f,0.9594999724179504863030842898297123610973358154296875f,-0.549399984902143589948764201835729181766510009765625f,-0.53829996688365877588466901215724647045135498046875f,-0.88420002566576005875731425476260483264923095703125f,-0.788200029659271717008550695027224719524383544921875f,0.744600014770031126687399591901339590549468994140625f,0.7943000014096497363169646632741205394268035888671875f,-0.6335999890357257857687045543571002781391143798828125f,-0.6453999945059418763548819697462022304534912109375f,-0.664199998396635127306808499270118772983551025390625f,-1.4870000182390210596850010915659368038177490234375f,0.50840000692605968879433930851519107818603515625f,0.58930000491142298102431595907546579837799072265625f,-0.52459999808073032312449868186376988887786865234375f,0.7563999745309357702893748864880762994289398193359375f,0.1694000039756298203741380348219536244869232177734375f,0.2633000224649908460605729487724602222442626953125f,0.67960001581311235696603034739382565021514892578125f,-0.247499981969594440300852511427365243434906005859375f,-0.190600001239776606354325849679298698902130126953125f,-0.4251000009268519619354265159927308559417724609375f,-0.368699978339672274074700908386148512363433837890625f,0.3365000147223469184609712101519107818603515625f,-0.838699991303682335086477905861102044582366943359375f,0.432000009000301332662274944595992565155029296875f,0.749500021368265212373671602108515799045562744140625f,-0.62009998967647561585181392729282379150390625f,0.045899987554550047974544213502667844295501708984375f,-0.3123999982535838881858580862171947956085205078125f,-1.1915999801844361360991797482711263000965118408203125f,0.3165000039935108588906587101519107818603515625f,-0.838300005179643203945261120679788291454315185546875f,0.0290000311136253685617703013122081756591796875f,-0.0017000150859356466526151052676141262054443359375f,0.632199989843368115316479816101491451263427734375f,-0.3489999602213504914516306598670780658721923828125f,0.19220000046044560804148204624652862548828125f,-1.1108999996617436867296646596514619886875152587890625f,-0.23849998110532766304459073580801486968994140625f,0.95020000751614563938574065105058252811431884765625f,-0.47280002565383938417653553187847137451171875f,0.5226999951481818840193227515555918216705322265625f,-1.21699996399879495356799452565610408782958984375f,-0.786399994647502875722011594916693866252899169921875f,0.379499995738267870137860882095992565155029296875f,-0.086200012314319618411673218361102044582366943359375f,1.109899988567828899022060795687139034271240234375f,0.2199000004887581294354959027259610593318939208984375f,-0.0768999881803988927231330308131873607635498046875f,0.15109999417662633991454868009896017611026763916015625f,0.3358999882064754860522270973888225853443145751953125f,-0.49460003011822717411405392340384423732757568359375f,0.10959996947050054671990437782369554042816162109375f,-1.00799997103214256100045531638897955417633056640625f,-0.71500001177191607748540036482154391705989837646484375f,0.81540001071095480345007899813936091959476470947265625f,0.4935000097900630322556025930680334568023681640625f,-0.15319999821782115390078615746460855007171630859375f,0.66039997554421481229525170419947244226932525634765625f,0.28739999065399157718303513320279307663440704345703125f,0.507900022557377983645210406393744051456451416015625f,-0.786199992346763298911582751316018402576446533203125f,-1.178700004193186767764700562111102044582366943359375f,0.154200000110268575781446998007595539093017578125f,-0.7846999980762598969619148192577995359897613525390625f,0.1733999903857708080323618560214526951313018798828125f,1.487099999682604423156817574636079370975494384765625f,-0.23319998152852061679141115746460855007171630859375f,0.405099985802174433757727456395514309406280517578125f,-0.5209999967515468455303562222979962825775146484375f,-0.65019996057748841167267528362572193145751953125f,0.55760002037286771070512259029783308506011962890625f,-1.205999974250793638219647618825547397136688232421875f,-0.173599997901916491827023492078296840190887451171875f,0.587499986886978575739703956060111522674560546875f,-0.35540000102519986313609479111619293689727783203125f,-0.25010003549754600271626259200274944305419921875f,0.49980000857710837891545452293939888477325439453125f,-0.4323999971359964167305633964133448898792266845703125f,-0.1725000318326054549178394381669932045042514801025390625f,-0.9158999917268744184184470213949680328369140625f,-0.7377999870598319631653794203884899616241455078125f,-0.24359996035099129585432820022106170654296875f,0.209800004181265986602511475211940705776214599609375f,0.2739999867826696799966157414019107818603515625f,0.83689998840391621737211380605003796517848968505859375f,-0.399499995291233034322431194595992565155029296875f,-0.207800007399916442096809987560845911502838134765625f,0.571099974432588197714721900410950183868408203125f,-1.10149999736249437631840919493697583675384521484375f,0.140600067698954944717115722596645355224609375f,-0.528399995601177341342236104537732899188995361328125f,0.27679999574720870469946021330542862415313720703125f,1.1306000110745431097569735356955789029598236083984375f,-0.52500000886619080375794510473497211933135986328125f,-0.25570000712126528252809976038406603038311004638671875f,0.661099999392032344758263207040727138519287109375f,-1.245099960619211021395358329755254089832305908203125f,0.55510001471042613729878212325274944305419921875f,-0.3859999800622464505295283743180334568023681640625f,1.2793999874353403356508351862430572509765625f,1.33440000263452507311967565328814089298248291015625f,-1.1121999895453456019822624512016773223876953125f,-0.187600000405311551077147669275291264057159423828125f,-0.5002999828875065446709413663484156131744384765625f,-0.19600000956654561878877984781865961849689483642578125f,-0.30750001192092923929521930404007434844970703125f,1.3386000205516814798301084010745398700237274169921875f,-0.601399980379640819450060007511638104915618896484375f,0.4092000219032170260646807946613989770412445068359375f,0.2588000160537664573467964146402664482593536376953125f,0.444400019323826001027555321343243122100830078125f,1.0146000107467176842845901774126105010509490966796875f,0.1293999947369097736071807958069257438182830810546875f,-0.36199999484419809192559114308096468448638916015625f,0.40499997973442081189432428800500929355621337890625f,1.52879997189044924965628524660132825374603271484375f,-0.77479999591112136414494671043939888477325439453125f,-1.2369000110536807479633125694817863404750823974609375f,-0.3276000002562999835475920917815528810024261474609375f,-0.60650001882016690846199935549520887434482574462890625f,0.1973999909147620412941392942229867912828922271728515625f,0.8693999769113955888855116427293978631496429443359375f,-1.05159999173283580375937162898480892181396484375f,-0.5099000103235251657451954088173806667327880859375f,0.44819999002218224859461770392954349517822265625f,0.79620000411868030454343170276843011379241943359375f,0.68500000700354579663553522550500929355621337890625f,1.0152000057280066158682529930956661701202392578125f,0.6702000107944012796679089660756289958953857421875f,0.44210000553131134637396826292388141155242919921875f,0.09359999730587009736382242408581078052520751953125f,-0.299699993717670309933964745141565799713134765625f,0.4324000252246857911586630507372319698333740234375f,1.22139998585581821544110425747931003570556640625f,-0.541000005394220551124817575328052043914794921875f,-1.213599984639883277992566945613361895084381103515625f,0.3741000104337930398656908437260426580905914306640625f,0.429900001047550883725989478989504277706146240234375f,1.1199000182211398257692280822084285318851470947265625f,0.13579999787807395250638364814221858978271484375f,0.273499998539686206555643366300500929355621337890625f,0.45889998287558597667157300747931003570556640625f,1.0881999581336978533130377400084398686885833740234375f,-0.198000005602835926765692420303821563720703125f,-1.53930001549124728654760474455542862415313720703125f,0.2756000352144241105634137056767940521240234375f,0.0719000218570233773363042928394861519336700439453125f,0.14600000323355200038122347905300557613372802734375f,0.38659998005405071808127104304730892181396484375f,-0.736899999114125847654577228240668773651123046875f,-0.3150999425329270575701912093791179358959197998046875f,-0.4580999797821050378843210637569427490234375f,0.69860000535249699993300964706577360630035400390625f,0.10899994766712239879780099727213382720947265625f,1.408399975186586328135263102012686431407928466796875f,0.136099974656104905079700984060764312744140625f,0.95629999230504036944466861314140260219573974609375f,0.3102999875068661594923469237983226776123046875f,0.0672999972522259071183725609444081783294677734375f,-0.2893999922037124861162737943232059478759765625f,1.01950001049041727441135662957094609737396240234375f,0.0859999933242789893483859486877918243408203125f,-0.96589996818304069137184342253021895885467529296875f,0.036499994754791487139300443232059478759765625f,-0.7067000132977963033908963552676141262054443359375f,-0.81509999489188167398623363624210469424724578857421875f,0.81780000881552716851530249186907894909381866455078125f,1.2846000076919796839547416311688721179962158203125f,0.02640000552535060052150583942420780658721923828125f,-0.3355000024735925734375996398739516735076904296875f,0.540800016355514667765191916259936988353729248046875f,0.22699998188018799538667735760100185871124267578125f,0.34380000288486467496795739862136542797088623046875f,0.520000056177377434352138152462430298328399658203125f,0.59760001024007802783444276428781449794769287109375f,0.329700015175342198148200623109005391597747802734375f,-0.426800016236305257422145587042905390262603759765625f,0.765799996089935319787400658242404460906982421875f,-0.2056000051140784989911480806767940521240234375f,-1.00830003036260595905559966922737658023834228515625f,0.589400015151500866750211571343243122100830078125f,1.0602999930202958722702533123083412647247314453125f,0.4569999909698960749437901540659368038177490234375f,0.88219998343586869538057726458646357059478759765625f,-0.10500000312924395728941817651502788066864013671875f,-0.54590000588297815653504585498012602329254150390625f,1.071800035756826474653280456550419330596923828125f,0.99960006588101446567407037946395576000213623046875f,0.21360002636313435431247853557579219341278076171875f,-1.328699953603743377783530377200804650783538818359375f,0.831799998652935190790458364062942564487457275390625f,0.32670001195669140514610262471251189708709716796875f,-0.00160001423358924199646935448981821537017822265625f,0.738900027728080832645218833931721746921539306640625f,0.737199986490607361844240585924126207828521728515625f,-0.119200003239512852815096266567707061767578125f,0.35800001521408553184500078714336268603801727294921875f,0.301899999058246493177648517303168773651123046875f,0.960600006455182953146731961169280111789703369140625f,-0.06320000611543719060136936604976654052734375f,0.366600034797191387525572281447239220142364501953125f,-0.629200029242038993970709270797669887542724609375f,0.868100018227100367340653974679298698902130126953125f,0.63719997375011416806955821812152862548828125f,0.34269998143911362120661578956060111522674560546875f,-0.809599980795383089571259915828704833984375f,-0.669200013446807684402983795735053718090057373046875f,0.81200000751018563960315077565610408782958984375f,1.55590000633001324104043305851519107818603515625f,-0.272000008106232371574151329696178436279296875f,1.316100007885694367359974421560764312744140625f,0.0230999475955968591733835637569427490234375f,-0.4405000286996365588265689439140260219573974609375f,-0.41330000085830675971010350622236728668212890625f,0.08600001791119549210407058126293122768402099609375f,0.471299968910216193762607872486114501953125f,-0.389499978452921169491673936136066913604736328125f,1.090699988532065844992757774889469146728515625f,1.01659999963045155624286053352989256381988525390625f,0.1567999920964240345711004920303821563720703125f,0.077599989825487103445311731775291264057159423828125f,-0.39860000028610187428057543002068996429443359375f,-0.8190999957561491129354180884547531604766845703125f,-0.07330000767558819685376647612429223954677581787109375f,0.1875999935507774996068519612890668213367462158203125f,0.73269999321103096434626422706060111522674560546875f,-0.01099999600648882136511019780300557613372802734375f,-0.433299956899881433258769902749918401241302490234375f,0.249800000232458074833630234934389591217041015625f,-0.1347000230729573644339325255714356899261474609375f,0.380199987548589746211291640065610408782958984375f,0.503299973849951953752679401077330112457275390625f,-0.6060000140741468310778827799367718398571014404296875f,-0.8322000234641133697977011252078227698802947998046875f,-0.094400018131732821302648517303168773651123046875f,2.12159997310638370748847592039965093135833740234375f,0.03120000844001769024771419935859739780426025390625f,-1.61099997171759667935475590638816356658935546875f,0.41239999706149088609663522220216691493988037109375f,0.4114000195324418740483451983891427516937255859375f,0.21039999223351468771170402760617434978485107421875f,0.171099976369738282500065906788222491741180419921875f,0.068600004011392545777425766573287546634674072265625f,0.8054999936074016542164599741226993501186370849609375f,0.628999988943338195213073049671947956085205078125f,0.78170001910924913346434550476260483264923095703125f,-0.66100000956654536565793023328296840190887451171875f,0.39139995813965777760046194089227356016635894775390625f,-0.17079999668598178086398320374428294599056243896484375f,-0.9623999919950960713066479002009145915508270263671875f,0.44500000298023234535094161401502788066864013671875f,0.3510999908983709616450141766108572483062744140625f,0.23089998453706506165161727039958350360393524169921875f,-0.6605999921500682603436871431767940521240234375f,-1.0116999852836132678390868022688664495944976806640625f,0.3129999996423720176608185283839702606201171875f,0.6148999730706214972286716147209517657756805419921875f,0.70949998605251352046252577565610408782958984375f,0.977599974924326264869023361825384199619293212890625f,-0.1977000055789943644413142465054988861083984375f,0.087599994221329691157507113530300557613372802734375f,-0.748899986042081966530759018496610224246978759765625f,0.7992999868810179986411412755842320621013641357421875f,-1.2541999812602995234556146897375583648681640625f,-1.8009999754428864804367549368180334568023681640625f,1.15639999256134018423836096189916133880615234375f,0.82949999886751157873732154257595539093017578125f,-0.1567000295400617915220209397375583648681640625f,0.7288999883890152631238379399292171001434326171875f,0.1495999779641632443372145644389092922210693359375f,0.99320000134706487582292311344644986093044281005859375f,-0.3465000139772893561485034297220408916473388671875f,0.4635999794989824618340890083345584571361541748046875f,0.474699952590466267565716407261788845062255859375f,0.3421999804556372648534079417004249989986419677734375f,-0.12509996285438518270893837325274944305419921875f,-0.8874999954551459069307384197600185871124267578125f,0.6216999993652105871433377615176141262054443359375f,1.0740999871134757626123246154747903347015380859375f,-0.12860000386238112923820153810083866119384765625f,0.8986999904096126812191869248636066913604736328125f,-0.85899999486654998737122923557762987911701202392578125f,0.251400002396106714286361238919198513031005859375f,1.604300022047758123022731524542905390262603759765625f,-0.931399997180700101040429217391647398471832275390625f,-1.4822000311195846489198402196052484214305877685546875f,-0.0354000018447637732066368698724545538425445556640625f,1.14610001257956017894912292831577360630035400390625f,-0.23180000163316716310646370402537286281585693359375f,-1.7664000316768884157880847851629368960857391357421875f,0.3788000055670737165058881146251223981380462646484375f,0.04280002040118013884040237826411612331867218017578125f,0.336799993139505371431141611537896096706390380859375f,1.20739998886585198079046676866710186004638671875f,0.32319998778700809172192975893267430365085601806640625f,0.094499999240040377657123826793394982814788818359375f,-0.028399996644258518330161678022705018520355224609375f,0.42259999474287024412433311226777732372283935546875f,-0.46140000030994432478337330394424498081207275390625f,-1.049999980777502006645818255492486059665679931640625f,-0.395799996536970066784988375729881227016448974609375f,-0.122100019240379165097465374856255948543548583984375f,0.353799999308586077262361868633888661861419677734375f,0.04950001905858503192803254933096468448638916015625f,-0.511999992311000795552899944595992565155029296875f,0.801399965590239293788954455521889030933380126953125f,-0.352399998700618777291992955724708735942840576171875f,-1.5353999827712778714072783259325660765171051025390625f,0.146199996072053917117727905861102044582366943359375f,0.99249999552965117999292488093487918376922607421875f,1.137800008629262560333472720230929553508758544921875f,-0.22360001757144942757804528810083866119384765625f,-1.47130001346468919365406691213138401508331298828125f,0.113500063359737168866558931767940521240234375f,0.3512000187218189406479496028623543679714202880859375f,-0.1602000055789947197126821265555918216705322265625f,-1.43200002580136054319837057846598327159881591796875f,0.669400028264522717336149071343243122100830078125f,0.6126000062167642479238338637514971196651458740234375f,-0.47030002349317090448721501161344349384307861328125f,0.99219999017864479728956439430476166307926177978515625f,-0.396500013232230941184752737171947956085205078125f,0.014299987700581606731020656297914683818817138671875f,0.63299996365606858717001159675419330596923828125f,0.08379998612105843402986238288576714694499969482421875f,0.133699999201297714535030536353588104248046875f,-1.184699986267089943936525742174126207828521728515625f,0.49679999537467978143467917107045650482177734375f,-0.07560000526309007540248785517178475856781005859375f,-0.08549997788667695886033470742404460906982421875f,-0.29259999757409094200966137577779591083526611328125f,0.3248999824583531648158896132372319698333740234375f,0.44069998719096170614051288794144056737422943115234375f,-0.2101999781608583361958153545856475830078125f,-1.507199970471858829768052601139061152935028076171875f,0.7476000041306019028297669137828052043914794921875f,0.6036999945074319473548030146048404276371002197265625f,-0.3631000039219856034833355806767940521240234375f,0.305900013333558984385263102012686431407928466796875f,1.59340003076791791869482040056027472019195556640625f,-0.07019999932050691171525613754056394100189208984375f,-1.3088000094413754226252422085963189601898193359375f,0.550600003474950749904337499174289405345916748046875f,0.38420001791715652217362730880267918109893798828125f,0.65180001072287563346208116854541003704071044921875f,0.13440002334713962994783287285827100276947021484375f,0.10640000373721125725978708942420780658721923828125f,0.346800002899765758002104121260344982147216796875f,0.600399998864531436737479452858678996562957763671875f,0.241699992398917817837400434655137360095977783203125f,-0.265599996471404864450960303656756877899169921875f,-0.1624999763071539593539682755363173782825469970703125f,0.5039999978840350447484297546907328069210052490234375f,0.3202000075906514808821157203055918216705322265625f,0.66740001434683815517701077624224126338958740234375f,0.57590005893111229084979640902020037174224853515625f,0.01579999892115591109131855773739516735076904296875f,-0.11439999250173560341181655530817806720733642578125f,0.001200006353855087581905536353588104248046875f,0.317899987763166080867449636571109294891357421875f,0.4515999905407426240344648249447345733642578125f,-0.91959998139143106499204805004410445690155029296875f,0.53280001999139781077019506483338773250579833984375f,-0.111699972617625320481238304637372493743896484375f,-0.33560000690221780672573004267178475856781005859375f,0.0588999909222120976437508943490684032440185546875f,0.137999995023012189676592242904007434844970703125f,0.957700010794401368485750936088152229785919189453125f,-1.959700023669004309567753807641565799713134765625f,1.006600027048587975286864093504846096038818359375f,0.535199994850158677905938020558096468448638916015625f,0.425899977421760667084527085535228252410888671875f,0.3904999947249889746814233149052597582340240478515625f,0.786099980616569382618763484060764312744140625f,0.87650001248717313018232744070701301097869873046875f,0.283499994218349371521981083787977695465087890625f,0.8154999866783618767129837578977458178997039794921875f,0.63119996485412155351468754815869033336639404296875f,-0.087299989056587146141197308679693378508090972900390625f,-0.7049999678134923897232511080801486968994140625f,0.28230000559687606909164969692938029766082763671875f,0.5131999703526499700956264859996736049652099609375f,0.11050000277161597939112880339962430298328399658203125f,0.333099995875357990371412597596645355224609375f,-0.3976000026404857834450012887828052043914794921875f,0.86689999995231659113414934836328029632568359375f,0.28500000849366191602030085050500929355621337890625f,1.0619999854564667884915252216160297393798828125f,-0.19099998646974558624833662179298698902130126953125f,1.1650999461650854982508462853729724884033203125f,0.78609998091459232227862230502068996429443359375f,-0.18720002202987728878724738024175167083740234375f,0.7661000309824945730952094891108572483062744140625f,-0.8907999847650529545717290602624416351318359375f,0.1341999857306479526641851407475769519805908203125f,-0.599499998271465273091962444595992565155029296875f,0.439799998146295667567073905956931412220001220703125f,0.584899980446696332592182443477213382720947265625f,0.422999993011355268635043103131465613842010498046875f,-0.692499999515712172382109201862476766109466552734375f,0.19199998545646668191011485760100185871124267578125f,0.38200001570582298171530055697076022624969482421875f,0.051200010973214915566131821833550930023193359375f,0.148900002656877195050810769316740334033966064453125f,0.2887999945029615656011401370051316916942596435546875f,0.0222000297598532458920317367301322519779205322265625f,-0.5052000116884709512987683410756289958953857421875f,-1.6238999848872424092149913121829740703105926513671875f,0.2209999895989895879466757833142764866352081298828125f,0.73949993341416109249308874495909549295902252197265625f,-0.46300002661347416932358100893907248973846435546875f,-1.15340000528693309433947433717548847198486328125f,-0.18040000296235092491770046763122081756591796875f,0.6087999882817261010359288775362074375152587890625f,0.053799997222423456832984811626374721527099609375f,0.35710002922415728932037382037378847599029541015625f,-0.4446000062763690863221199833787977695465087890625f,0.572999970138072800551753971376456320285797119140625f,-0.65299999472498893027250232989899814128875732421875f,-0.33930000878572474931615943205542862415313720703125f,-0.15640000448226931695216990192420780658721923828125f,-0.704499980241060530516961080138571560382843017578125f,-0.2809999983906745768535984097979962825775146484375f,-0.66450000277906606793720811765524558722972869873046875f,0.11239998685419572144184030548785813152790069580078125f,1.588099972983449514885023745591752231121063232421875f,-0.102600024694204261521690568770281970500946044921875f,0.55789997308552319310592793044634163379669189453125f,0.022899995213747192934761187643744051456451416015625f,-0.58759999861717204794331337325274944305419921875f,0.18930001355409586949463118799030780792236328125f,0.7068999665737152326983050443232059478759765625f,-0.190299991455674000206954588065855205059051513671875f,0.419299978499114711638640073942951858043670654296875f,-0.84740000451207164733347099172533489763736724853515625f,0.02950000870227853511096327565610408782958984375f,0.9801999607264997305122733450843952596187591552734375f,-0.63859999007880663413061483879573643207550048828125f,-0.85249998133629556473778166036936454474925994873046875f,-0.8311000128030769928955123759806156158447265625f,0.7571000176012514515377915813587605953216552734375f,-1.081500011444092024248675443232059478759765625f,-0.47730000187158605484682993846945464611053466796875f,-0.62160001065731051994589506648480892181396484375f,0.0840999997794629194913795799948275089263916015625f,-0.041499986708164016135924612171947956085205078125f,-0.7452999839305878282402773038484156131744384765625f,0.2473000176668165295268408954143524169921875f,-1.136699996310472471350294654257595539093017578125f,0.217800004494190346804316504858434200286865234375f,0.012899995213746962008372065611183643341064453125f,-0.05159998800754550529745756648480892181396484375f,0.90829997783601312821843976053060032427310943603515625f,-0.631899985758960003323636556160636246204376220703125f,-0.8616999975584447302168200621963478624820709228515625f,-0.61489999959468821799646320869214832782745361328125f,-0.1208000125184656159404994468786753714084625244140625f,-0.330099975389988031793109257705509662628173828125f,-0.6562000068008899678062562088598497211933135986328125f,-0.2410000177621842709640986868180334568023681640625f,0.1022000087678431867743711336515843868255615234375f,-0.621299971666932293601348646916449069976806640625f,-0.334400007477402727573689844575710594654083251953125f,-0.913399971945583200039209259557537734508514404296875f,0.84620000486373925241423421539366245269775390625f,-0.2775000356137748980245305574499070644378662109375f,-0.269200018662214546338873333297669887542724609375f,-0.405900003945827148044145360472612082958221435546875f,-0.520200010943412838315680346568115055561065673828125f,-1.0833000398993493096355678062536753714084625244140625f,0.055200005802512108754598330051521770656108856201171875f,0.637900017975270738901372169493697583675384521484375f,0.569599989214539359494438031106255948543548583984375f,1.1574000049591057770470570176257751882076263427734375f,0.0956000151351094895613158541891607455909252166748046875f,0.4773000372059648110933949283207766711711883544921875f,-0.85019999499917009444516224903054535388946533203125f,-1.304800022882222521047879126854240894317626953125f,0.5742000019729136539581304532475769519805908203125f,-0.1048999315708878388164748685085214674472808837890625f,-0.3682000031352041791166129769408144056797027587890625f,0.14109994921237267728741926475777290761470794677734375f,0.739800002318620641972302109934389591217041015625f,0.14790003879964341848562980885617434978485107421875f,-0.139499969847500293607112098470679484307765960693359375f,-0.055900008006393608894768476602621376514434814453125f,-0.75510000964403156675786021878593601286411285400390625f,-0.33769997234940518637813511304557323455810546875f,-0.9234999790191651403148398458142764866352081298828125f,0.6566999848365782099790521897375583648681640625f,-0.698100017333030731236931387684307992458343505859375f,-0.33019998240768888564389271778054535388946533203125f,0.048199956047535241765444880002178251743316650390625f,0.4118000096797942433113348670303821563720703125f,-0.2041000119984150984464577049948275089263916015625f,-1.0099000061511986814366537146270275115966796875f,-0.80950000959634760278049725457094609737396240234375f,-0.4656999827206131925549925654195249080657958984375f,0.18160001129060965308070763057912699878215789794921875f,-0.87010000893622674311700393445789813995361328125f,0.8125000080838800098348428946337662637233734130859375f,0.52929995000064433696707055787555873394012451171875f,0.13419999468997112490598055956070311367511749267578125f,0.5238999995462592096373555250465869903564453125f,-1.68350001550838346275895673898048698902130126953125f,0.258500066637992631513043306767940521240234375f,-0.57589996773600571344786658301018178462982177734375f,1.009699964511394654209652799181640148162841796875f,0.674399966275692097639193889335729181766510009765625f,0.21970003149211425608200443093664944171905517578125f,-0.36729999728947870796247343605500645935535430908203125f,0.8395999684274197960576202603988349437713623046875f,-0.52399997404217713015128765619010664522647857666015625f,-0.95170003982186290869549338822253048419952392578125f,0.52580000607371335519246713374741375446319580078125f,-0.77710000001788070989050538628362119197845458984375f,0.0249999873340129852294921875f,-0.090699991065263940726026703487150371074676513671875f,0.622200012344122033169924179674126207828521728515625f,0.365400004974007430735838397595216520130634307861328125f,0.236399999341368627359116771913249976933002471923828125f,0.7941000236585737948047380996285937726497650146484375f,-0.293200003582239077104532043449580669403076171875f,-1.0296000149190422945366663043387234210968017578125f,0.48960000508427636844999142340384423732757568359375f,1.522999974310398574317559905466623604297637939453125f,0.393799990069866101549678205628879368305206298828125f,0.13070002205967856667712112539447844028472900390625f,0.14000000521540645337381647550500929355621337890625f,-0.64360001734793159666736528379260562360286712646484375f,-0.49939999488592146104082303281757049262523651123046875f,-0.09019999537169898218280650326050817966461181640625f,-0.584399994140863299207921954803168773651123046875f,0.5675999901235098121787814307026565074920654296875f,-0.5005999900639057642592888441868126392364501953125f,-0.452100029969214833869273206801153719425201416015625f,-0.485400007879734030069585060118697583675384521484375f,0.288199984546005705254856366082094609737396240234375f,0.3330000066831708682713042435352690517902374267578125f,0.05690006568506333906043437309563159942626953125f,-0.4198000018715857262208146494231186807155609130859375f,0.1166999797701837593422169447876513004302978515625f,-0.58290002143979080528879421763122081756591796875f,0.16709999718666068702077609486877918243408203125f,0.34320000686049478844097393448464572429656982421875f,-0.9604999930858610923678497783839702606201171875f,0.7714000150620936580025954754091799259185791015625f,0.025699983465671039084554649889469146728515625f,0.7096000040039418532700210562325082719326019287109375f,-0.5972000124558807243602132075466215610504150390625f,-0.329800008837878577683255798547179438173770904541015625f,0.04010002126693734680884517729282379150390625f,1.445799977612495457179875302244909107685089111328125f,-0.6296999952077868556443718262016773223876953125f,-0.1542999926924704112707331660203635692596435546875f,0.33639997676610988719403394497931003570556640625f,0.052599982970952918748253068770281970500946044921875f,0.3050000016391276602689686114899814128875732421875f,-0.03009998036324990078327346054720692336559295654296875f,-0.9898999774664645201482926495373249053955078125f,-0.09229995590150352757774498968501575291156768798828125f,-0.4630999750137334558530710637569427490234375f,-0.4069999970793727328555178246460855007171630859375f,0.10950000751018507116896216757595539093017578125f,-0.13949999842047677844902864308096468448638916015625f,-0.8105999794840812455731793306767940521240234375f,-0.97149997784197328343225308344699442386627197265625f,0.421799994368850850179342160117812454700469970703125f,-0.09900000652670859491166766019887290894985198974609375f,0.35540002978444107384348171763122081756591796875f,-1.3940999811530112584279095244710333645343780517578125f,-0.523700012612343446249951739446260035037994384765625f,0.345600003474950678850063923164270818233489990234375f,0.17260004108548177015336477779783308506011962890625f,-0.1093999936938285788556868283194489777088165283203125f,0.42019997655898400079621524128015153110027313232421875f,0.04740000227689744605186206172220408916473388671875f,-1.373100022473931147715120459906756877899169921875f,-0.02699999931454666324270874611102044582366943359375f,1.63839997876286513900367936003021895885467529296875f,-0.5281999842107296672111260704696178436279296875f,-1.81699996548891107295276015065610408782958984375f,0.2662000012874603527279759873636066913604736328125f,0.8403000462174414764149332768283784389495849609375f,0.5638000228524209234137742896564304828643798828125f,0.59170003907680523980161524377763271331787109375f,-0.056999979197978944966962444595992565155029296875f,-0.639799963724612563709115420351736247539520263671875f,-0.122200001317262607614111402654089033603668212890625f,-0.748699964854121713386803094181232154369354248046875f,0.1049999955296516684910557160037569701671600341796875f,0.705499998301267527978097859886474907398223876953125f,0.118800026726722851577733308658935129642486572265625f,-0.62899997661262718740005084328004159033298492431640625f,-0.29419999202638857926928039887570776045322418212890625f,-0.6584000007607040405588350040488876402378082275390625f,0.823600009375810682143992380588315427303314208984375f,1.2108000141948462857754975630086846649646759033203125f,-0.3429999994933605744762417089077644050121307373046875f,-0.97840003065615999133797231479547917842864990234375f,-0.16619999186247635947211165330372750759124755859375f,0.47959998109340684635526486090384423732757568359375f,1.1881999922573565076078239144408144056797027587890625f,0.191399993902444887083902358426712453365325927734375f,1.468600021035969316329783396213315427303314208984375f,-0.412399992889165734055723078199662268161773681640625f,-0.05509998207688315829955172375775873661041259765625f,-0.1397999982953070929880823314306326210498809814453125f,-0.7195999976336953096733850543387234210968017578125f,-0.2434000009655949980924560804851353168487548828125f,-0.0287000009894369245699863313348032534122467041015625f,-0.232799991679191631277490159845910966396331787109375f,-0.09180002405941534160405126385740004479885101318359375f,-0.507200000572204157123223922098986804485321044921875f,-0.70000001817941726045546602108515799045562744140625f,0.8039999572038656339145745732821524143218994140625f,-0.9481000498920675312319872318767011165618896484375f,0.147000005424022361921743140555918216705322265625f,-1.0965000338330870288672258539008907973766326904296875f,0.1997999936014414057439125826931558549404144287109375f,0.3076999913856381940746587133617140352725982666015625f,0.41519997152984189359159472587634809315204620361328125f,0.724799968120455329056994742131792008876800537109375f,0.5619999760687353074217753601260483264923095703125f,-0.189100035803020372071614474407397210597991943359375f,0.34899997881054911630371861974708735942840576171875f,-0.14950000428780885197710404099780134856700897216796875f,-0.00060000001043081230278630755492486059665679931640625f,-0.50100000667944544208154411535360850393772125244140625f,-0.307799999614059960340028965219971723854541778564453125f,-0.79550001005455828195778167355456389486789703369140625f,-1.1283999936640258798803415629663504660129547119140625f,-1.11469999416470511022225764463655650615692138671875f,-0.6509999977201224030665116515592671930789947509765625f,0.4971000047862528248288072063587605953216552734375f,0.123800016146898173019508249126374721527099609375f,0.07540001682043051545178968808613717555999755859375f,-0.428399991503357835398446695762686431407928466796875f,0.1281000034376977847472289795405231416225433349609375f,-0.445999989859759693455742990408907644450664520263671875f,-0.6262999941304325801638697157613933086395263671875f,-0.666200004118680855214051916846074163913726806640625f,-0.85970001964569053853892910410650074481964111328125f,-0.1368999981641767504925155662931501865386962890625f,0.2895000064671038320085472150822170078754425048828125f,0.01639999777674694403373223394737578928470611572265625f,1.123899989879131311454329988919198513031005859375f,0.505000006407498869265282337437383830547332763671875f,-0.356499968826771151952925720252096652984619140625f,0.191700015234947063191839333740063011646270751953125f,0.75529999123215674217135529033839702606201171875f,0.292399980297684436436611576937139034271240234375f,1.0454000016212461332543171010911464691162109375f,0.59789999178647956767918003606610000133514404296875f,-0.263399987107515443085503648035228252410888671875f,0.89589998941719528602334321476519107818603515625f,1.2651999684751036312491123680956661701202392578125f,0.315199980321526407323062812793068587779998779296875f,0.5464999982193108341022025342681445181369781494140625f,0.298999992333352559992221131324185989797115325927734375f,0.95219998073503375746184929084847681224346160888671875f,0.2058000109910960162551418761722743511199951171875f,1.39599999727308698993510915897786617279052734375f,-0.0921999996781348585273008211515843868255615234375f,0.9902999982357021391266016507870517671108245849609375f,0.63799997341632863623317462042905390262603759765625f,0.45129995069354744696710213247570209205150604248046875f,-0.267399994230270332540300159962498582899570465087890625f,0.5529999768435953644285518748802132904529571533203125f,-0.3657999827533962022840796635136939585208892822265625f,0.3617999946668739585931007241015322506427764892578125f,-0.090799995344877260095017845742404460906982421875f,0.58060001997798682538132197805680334568023681640625f,0.247399988865852460406813406734727323055267333984375f,0.744999985173345091737928669317625463008880615234375f,-0.8429999534487730983300934894941747188568115234375f,-0.346799999025464433799470498342998325824737548828125f,-0.18479999993443474437526674591936171054840087890625f,0.4620000015869736298412817632197402417659759521484375f,-0.19659999217987067510193810448981821537017822265625f,0.9335000358670952902429007735918276011943817138671875f,-0.67099998900294277603961745626293122768402099609375f,-0.7549999359622587835616513984859921038150787353515625f,-0.0618000026762484111486628535203635692596435546875f,-1.212199964697658227663623620173893868923187255859375f,-0.29679997630119370199963668710552155971527099609375f,-0.0362999895483258061545939199277199804782867431640625f,0.5513000032573938113955591688863933086395263671875f,0.81159999247789382792461765347979962825775146484375f,0.8460999959647654833361229975707828998565673828125f,1.03739999423027029479271732270717620849609375f,0.76909999501109116426533773847040720283985137939453125f,0.6062999797135593116337304309126920998096466064453125f,0.31379999088943011376073854989954270422458648681640625f,0.2468000010371207419979100450291298329830169677734375f,0.27069998868107780776171011893893592059612274169921875f,0.7733999982833861253084251075051724910736083984375f,0.38879998977184282438201989862136542797088623046875f,0.07679999462962161516088599455542862415313720703125f,-0.39290001465976243100186593437683768570423126220703125f,-0.648100005523860378531253445544280111789703369140625f,0.71880000705719027109807939268648624420166015625f,1.14359997182488459799287738860584795475006103515625f,0.25289998038709171623850124888122081756591796875f,-0.518100005710124822400075572659261524677276611328125f,-0.6923999808192251581573373186984099447727203369140625f,-0.891599972808361229681395343504846096038818359375f,-0.758499967992305901276495205820538103580474853515625f,-0.89149996823072452656333553022705018520355224609375f,-0.38339998889565496398290633806027472019195556640625f,-0.411499993562698396232235609204508364200592041015625f,-0.7493999935448167892815263257944025099277496337890625f,-1.11570000164508797979578957892954349517822265625f,-0.82059998559355751268640233320184051990509033203125f,-1.1923999633848669787994367652572691440582275390625f,-0.671399994572997371733436011709272861480712890625f,-0.243899984477459863541071172221563756465911865234375f,-0.222199982839823118041522320709191262722015380859375f,-0.44080001736134299950009562962804920971393585205078125f,-0.67459996812939682087062465143389999866485595703125f,0.3450999996229999755570361230638809502124786376953125f,0.04339999962449070380898774601519107818603515625f,-0.1713000078767539235968797584064304828643798828125f,-0.0670000025928019216081565900822170078754425048828125f,0.02740002541095076793453699792735278606414794921875f,-0.66639998391866672733385712490417063236236572265625f,0.5832000608772045513461534937960095703601837158203125f,0.05380000571608523074473851011134684085845947265625f,-0.1315000109225514801636336414958350360393524169921875f,-0.15059999863207329529046774041489697992801666259765625f,-0.91410001665502793599671349511481821537017822265625f,-0.04499999031424550821611774154007434844970703125f,-0.754700023631751815145207729074172675609588623046875f,-0.3417999888181686429788896930404007434844970703125f,0.9231000007927419037656591171980835497379302978515625f,0.078600007289647777497521019540727138519287109375f,0.37829999493509536279134408687241375446319580078125f,0.54419999258518192419842307572253048419952392578125f,-0.8414000263869765294799663024605251848697662353515625f,0.125799995493888872033494408242404460906982421875f,0.1387999709963796401979152506100945174694061279296875f,-0.06579999012947101988402209826745092868804931640625f,0.3741000280916690901023002879810519516468048095703125f,0.74999996513128319719498904305510222911834716796875f,-0.2988000222563742536152631146251223981380462646484375f,0.263600028002262121162857511080801486968994140625f,-0.45079996926784549060585050028748810291290283203125f,0.62480000522434675502836398663930594921112060546875f,0.5252999872460961672260282284696586430072784423828125f,0.73500000569969425878724678113940171897411346435546875f,0.7803000067293641706100970623083412647247314453125f,0.512400005629658483741195595939643681049346923828125f,0.12709999897480006580963163287378847599029541015625f,0.663999994382262315184561884962022304534912109375f,1.030799983125925134430644902749918401241302490234375f,0.1690999794393774902090399336884729564189910888671875f,-0.257499993219971745617158376262523233890533447265625f,-0.4989999970644711790868086609407328069210052490234375f,-0.033399996495246941918821903527714312076568603515625f,-0.0456000052630900487571352641680277884006500244140625f,0.56719998805522919838040252216160297393798828125f,0.9572999938994646296208657076931558549404144287109375f,1.0738999786101282030159609348629601299762725830078125f,0.5261999916017059408801515019149519503116607666015625f,-0.54490002783238844497049058190896175801753997802734375f,0.195300007921457574600054840630036778748035430908203125f,-0.61799998906254760555611937888897955417633056640625f,-0.98410000670850283910340294823981821537017822265625f,-0.7766999951183795047882085782475769519805908203125f,-0.73390001051723974256901783519424498081207275390625f,0.139299999025463971946692254277877509593963623046875f,0.08360000639557851886962680509896017611026763916015625f,-0.433300005738437021829412287843297235667705535888671875f,-0.61960000337064269348985590113443322479724884033203125f,-1.1141999916911118351237064416636712849140167236328125f,-0.02240000197887415112063536071218550205230712890625f,-1.84770000360459096810927803744561970233917236328125f,-0.295800001752376251484832891947007738053798675537109375f,-0.405600000159441975711160921491682529449462890625f,-0.44139998213052766828923267894424498081207275390625f,-0.3262000127613549071980969529249705374240875244140625f,-0.0066000076770780236046221034484915435314178466796875f,-0.5796999890238041519552325553377158939838409423828125f,-0.35099998453259484421096203732304275035858154296875f,0.6015000326409933872895408057956956326961517333984375f,0.00579998357296040989439234181190840899944305419921875f,1.514100025483966049932860187254846096038818359375f,0.23279998542070412526072686887346208095550537109375f,0.1463999833971261210763259441591799259185791015625f,-0.3313999953925608199511998464004136621952056884765625f,-0.966300027043372811164090308011509478092193603515625f,-0.5973999913617966672774173275684006512165069580078125f,-1.223600029622763596393042462295852601528167724609375f,-0.4051999805450441360932245515868999063968658447265625f,-0.730899963377415584631080491817556321620941162109375f,0.75419999273419335850121569819748401641845703125f,0.523699980016052446529783992446027696132659912109375f,0.0388000098884102584406718960963189601898193359375f,0.8565999922171230185341528340359218418598175048828125f,-0.74179999209642366597705631647841073572635650634765625f,0.27920002853423386834919028842705301940441131591796875f,0.16539999677836891844862066136556677520275115966796875f,0.274299991239607354742702227667905390262603759765625f,0.67029999473392931630399971254519186913967132568359375f,0.4202000020399687674199640241567976772785186767578125f,0.36039999204725059467335768204065971076488494873046875f,0.125499963134527714458954505971632897853851318359375f,-0.1187000053852795833364552890998311340808868408203125f,0.2171000299692149582142519648186862468719482421875f,-0.4660999894082547001517013995908200740814208984375f,-0.141499995872378381278622327954508364200592041015625f,0.0866000148296353078336551334359683096408843994140625f,-0.46419996576309241476110400981269776821136474609375f,-0.628699990260601193625689120381139218807220458984375f,-0.65159999694824222160605131648480892181396484375f,-0.0107999956429001287006030906923115253448486328125f,0.58729996759891545110576771548949182033538818359375f,-0.0620000277385115428074868759722448885440826416015625f,0.41239999953880890348756338426028378307819366455078125f,0.7041000174187124205360532869235612452030181884765625f,0.111700003910064626921894159750081598758697509765625f,-0.0075000019371509463184111154987476766109466552734375f,0.083599998497962957344498136080801486968994140625f,-0.305700032639503349596310499691753648221492767333984375f,0.63909996763020782140074516064487397670745849609375f,0.548300007191300675657430474529974162578582763671875f,0.2153000165641305585495501873083412647247314453125f,-0.3095999893635512645317930946475826203823089599609375f,-0.513099986934661966841986213694326579570770263671875f,-0.198499981328844887773499294780776835978031158447265625f,-0.85370000564604975323845792445354163646697998046875f,-0.285399999237060608692218011128716170787811279296875f,-1.03069999911189125185728698852472007274627685546875f,-0.134400018429755518933887969978968612849712371826171875f,0.116100022079050957390933263013721443712711334228515625f,0.20760000055432303867064547375775873661041259765625f,1.18360002058893432064223816269077360630035400390625f,0.61959997230172181303942124941386282444000244140625f,0.51439996586590996940913100843317806720733642578125f,-0.09539999715089797494016465861932374536991119384765625f,-0.61449997867643812998750263432157225906848907470703125f,0.5209999702274803468071695533581078052520751953125f,-0.66820005175024288490703838760964572429656982421875f,-0.59359999015331244009985312004573643207550048828125f,1.1827000170528887412757512720418162643909454345703125f,-0.07819999330043803098533317097462713718414306640625f,0.515699998236447409993843393749557435512542724609375f,-0.7133999917268749602072830384713597595691680908203125f,-0.01589997509345408843728364445269107818603515625f,0.604799971920252144030882845981977880001068115234375f,0.38490000775307409153214166508405469357967376708984375f,0.82939996493458778559215716086328029632568359375f,0.7163000011712312709022398848901502788066864013671875f,-0.3435999973803756102341822042944841086864471435546875f,0.0576000156790021922148525845841504633426666259765625f,0.51859998896121961564631419605575501918792724609375f,-0.4596000537365678706436256106826476752758026123046875f,-0.2047999927818775489640756859444081783294677734375f,-0.7176000289410355836849930710741318762302398681640625f,0.3229999983757732007205731861176900565624237060546875f,-0.684000029511750096133937404374592006206512451171875f,-0.20380002627968796247159843915142118930816650390625f,0.66989997932910938516215537674725055694580078125f,0.423199975866079380892870176467113196849822998046875f,0.7039999886453152111442932437057606875896453857421875f,0.25459999405741695621685494188568554818630218505859375f,-0.5457999993681905692710643052123486995697021484375f,-0.7031000145018102553962080492055974900722503662109375f,-0.39509999019801622655023720653844065964221954345703125f,0.260599991628527705156415095189004205167293548583984375f,0.95769996653795264052178026759065687656402587890625f,1.049400009787083032364307655370794236660003662109375f,0.791800012511014728033842402510344982147216796875f,-0.215100018361211287043488482595421373844146728515625f,-0.15789999461770065636301296763122081756591796875f,0.5034999943673612587957677533267997205257415771484375f,0.80619996280521188936063481378369033336639404296875f,0.86139998384416127663598672370426356792449951171875f,0.65639998585581782464259958942420780658721923828125f,-0.171200002256035599401684521581046283245086669921875f,-1.1126999988734720403726896620355546474456787109375f,-0.5943999900430441130083636380732059478759765625f,-0.705599982613325238389734295196831226348876953125f,-0.4670999986022710270816560296225361526012420654296875f,0.219499998867511703082300300593487918376922607421875f,0.0017999944806097456506677190191112458705902099609375f,0.45130000288486460391368382261134684085845947265625f,0.57810001770555974331955439993180334568023681640625f,0.26350000308454024633419976453296840190887451171875f,-0.05239999385774130047366270446218550205230712890625f,1.067700055125355707019707551808096468448638916015625f,0.20019999634027474399999846355058252811431884765625f,-0.0862999847799537089798604938550852239131927490234375f,-0.196799991500377746689309788052923977375030517578125f,0.0624000426590447521135729402885772287845611572265625f,0.6977999924242492824077999102883040904998779296875f,0.454900010062754045492283694329671561717987060546875f,0.2673999896109104934538436282309703528881072998046875f,-0.15900001814961484125632296127150766551494598388671875f,-0.069000005185603896507018362171947956085205078125f,-0.56449999774992409840024265577085316181182861328125f,0.40779999987482984380449124728329479694366455078125f,0.1910999896690246391273149129119701683521270751953125f,-0.5319999918639657376928653320646844804286956787109375f,0.251700027900934208968664052008534781634807586669921875f,0.1359999977201222787215328935417346656322479248046875f,-1.023800023932755376421255277819000184535980224609375f,0.115199990081787273510371960583142936229705810546875f,-0.34930002331435694440386896530981175601482391357421875f,0.2033999941110611331396285095252096652984619140625f,0.194500015333294751229686880833469331264495849609375f,0.817399962788820655390509273274801671504974365234375f,-0.08510002048462605994672003362211398780345916748046875f,-0.2621999952077864737276513551478274166584014892578125f,0.4053999924570319546290875223348848521709442138671875f,-0.8227999894440169992293476752820424735546112060546875f,-0.111199995140731255904853469473891891539096832275390625f,-0.04700000482797606071727614107658155262470245361328125f,-0.2212000190198419868892187878373079001903533935546875f,0.1037000317230820645164612869848497211933135986328125f,-0.03919998856186879265806055627763271331787109375f,0.66939999987781018120358567102812230587005615234375f,0.46199998307228096194876343361102044582366943359375f,-0.71300001335144014280587043685954995453357696533203125f,-0.58509998766481885468238033354282379150390625f,-0.310699980634451033978393752477131783962249755859375f,-0.3907999840199949392882672327687032520771026611328125f,-0.4632999810397622209023893447010777890682220458984375f,0.16850000233948225769609052804298698902130126953125f,0.789199996459483799071676912717521190643310546875f,1.138099998855590655466585303656756877899169921875f,0.70309998998939970960009304690174758434295654296875f,0.2645000004693864870120023624622263014316558837890625f,-0.104099988864362114071582254837267100811004638671875f,-0.854099992664158147448461022577248513698577880859375f,-0.448300014120340595269453842774964869022369384765625f,0.4928999935746192218033456811099313199520111083984375f,0.50889998816549819604659887772868387401103973388671875f,1.201799966019392318372638328582979738712310791015625f,0.5803999516278504078314881553524173796176910400390625f,0.66299997848272340661424095742404460906982421875f,0.218599994176626122310835853568278253078460693359375f,-0.88509999131560324059364575077779591083526611328125f,-0.405399987390637317474784140358678996562957763671875f,-0.249699989470839600613771835924126207828521728515625f,-0.4782000009745359658808183667133562266826629638671875f,-0.434800000232458128124335416941903531551361083984375f,-0.30059998313486546717143710338859818875789642333984375f,-0.62090003473535215761103245313279330730438232421875f,0.25320000194311109709133233991451561450958251953125f,-0.7016000172138225909890252296463586390018463134765625f,0.77279995234012677229884502594359219074249267578125f,0.4324999862164264907704591678339056670665740966796875f,-0.373599995517730487648577764048241078853607177734375f,0.6493000076308843571126772076240740716457366943359375f,0.40339997339844746448989099008031189441680908203125f,0.58949996098130952315585773249040357768535614013671875f,-0.2749999840557577623911811315338127315044403076171875f,-0.362799967464804529182487158323056064546108245849609375f,-0.01880000839829421011017984710633754730224609375f,-0.95719999818801848601168558161589317023754119873046875f,-0.337999993532895948167293909136787988245487213134765625f,0.11300001808255955015880545033724047243595123291015625f,0.36840000357329805158457247671321965754032135009765625f,1.1414000093996516937977503403089940547943115234375f,0.144399978643656279331253244890831410884857177734375f,0.1657000101014978188374016099260188639163970947265625f,-0.235599979782104895775773911736905574798583984375f,-0.0399999595433471188954399622161872684955596923828125f,-0.787999974906444577982256305404007434844970703125f,0.0105000384598969986882366356439888477325439453125f,0.576999974727630871029759873636066913604736328125f,-0.1156000110745429854119947776780463755130767822265625f,0.48379997904300708899683058916707523167133331298828125f,-0.5279000438660386151923376019112765789031982421875f,-0.387599999511241577465625596232712268829345703125f,-0.1831999979570506642545524300658144056797027587890625f,-0.0818000251397490441007676054141484200954437255859375f,-0.24020000041946765190203905149246565997600555419921875f,-0.7407000137336552381128740307758562266826629638671875f,-0.586999995291233034322431194595992565155029296875f,-0.5631999964296816596487360584433190524578094482421875f,0.116000004947185431092293583787977695465087890625f,-0.55879997643530376905118828290142118930816650390625f,-0.7707999592840675262550576007924973964691162109375f,0.3423000166982408387639225111342966556549072265625f,1.312099993163347022573361755348742008209228515625f,0.80420000875294228581680044953827746212482452392578125f,0.103099996285140604879160264317761175334453582763671875f,-0.5490000054091215275775539339520037174224853515625f,-0.7551999835439027197736550078843720257282257080078125f,0.20579999627619993685101462688180617988109588623046875f,0.28429996393322998216035557561554014682769775390625f,-0.903300036248564364171897977939806878566741943359375f,-0.83440000457167595726559738977812230587005615234375f,-1.474700013312697510770021835924126207828521728515625f,-0.9225999862492084702125794137828052043914794921875f,-1.17279999994933614715364456060342490673065185546875f,-0.51529998810291299804475784185342490673065185546875f,-0.220799991023540531642765927244909107685089111328125f,-0.05449999544024475284231812111102044582366943359375f,-0.247800001439452355356252155615948140621185302734375f,-0.6773999966144561568626158987171947956085205078125f,0.00819999821782158022642761352472007274627685546875f,0.8211999671638015030339374789036810398101806640625f,0.946199989441037114801247298601083457469940185546875f,0.340600018990784902950252899245242588222026824951171875f,-0.0966000061869618509291512964409776031970977783203125f,1.34640005151405972583233960904181003570556640625f,0.060399989104271245565769277163781225681304931640625f,1.211800015044212752712837755098007619380950927734375f,-0.064399995779991225930416476330719888210296630859375f,0.85179998911619192458744009854854084551334381103515625f,0.133799999159574412033180124126374721527099609375f,-0.23890002478510208305806372663937509059906005859375f,-0.14300001215934710074861868633888661861419677734375f,-0.2780000017657873190302098009851761162281036376953125f,0.41019999373257143293614035428618080914020538330078125f,0.10209999329373242726859416507068090140819549560546875f,0.13119999677985905162103108523297123610973358154296875f,-0.032099957977980153600583435036242008209228515625f,0.90479998592734300899564914288930594921112060546875f,1.1263000122353437060240821665502153337001800537109375f,0.291800013405083991102628715452738106250762939453125f,2.059400012208520980294679247890599071979522705078125f,0.07479999561309824684940394945442676544189453125f,1.577100017675757470669850590638816356658935546875f,-0.230200000512599700641658273525536060333251953125f,-0.021399994274973632712999460636638104915618896484375f,-0.21459999659061423926686984486877918243408203125f,0.298700012090801880759727282566018402576446533203125f,0.14319997929334693509417775203473865985870361328125f,0.1314000006824735944377380292280577123165130615234375f,-0.081199985492229842520828242413699626922607421875f,-1.675400018906593846423902505193836987018585205078125f,-0.43489999773204346400490294399787671864032745361328125f,-0.9869999749511482178832011413760483264923095703125f,-0.477099990555644115630684609641321003437042236328125f,0.0846999990820884729458839501603506505489349365234375f,-0.60610001198351393103536111084395088255405426025390625f,-0.73419999556541437613077505375258624553680419921875f,-0.5778000117957589054640266112983226776123046875f,-0.5872000288844103632612814180902205407619476318359375f,-0.45390002169311027824250004414352588355541229248046875f,-0.384300006252527026617826777510344982147216796875f,-0.294799999561905945455464461701922118663787841796875f,-0.247199983882904206211605924181640148162841796875f,0.05660001229643807363345331395976245403289794921875f,0.648300012034177530750866935704834759235382080078125f,-0.7062000077694656763327429871424101293087005615234375f,-1.1558999911308287522615501075051724910736083984375f,-1.35389999896883939101144278538413345813751220703125f,-0.8572000004231927761111364816315472126007080078125f,-0.49100000025331969055741865304298698902130126953125f,0.107299995017051674750518941436894237995147705078125f,-0.0404000153303145648209238061099313199520111083984375f,-0.285800025743245811327142291702330112457275390625f,0.55799999003112343842758491518907248973846435546875f,1.027699973541498223994494765065610408782958984375f,0.228900006344914697109516055206768214702606201171875f,-0.54169998245239270318052149377763271331787109375f,0.0877000152647493536051115370355546474456787109375f,-0.039099993818998424188748685992322862148284912109375f,0.4453999979704617029341306988499127328395843505859375f,-0.403100009100139100581827733549289405345916748046875f,-0.165800004732608474711241797194816172122955322265625f,0.927700032736361368534971916233189404010772705078125f,0.028399997538328225488157841027714312076568603515625f,0.76550002054125076966784035903401672840118408203125f,-0.01960000672340367344759215484373271465301513671875f,-0.1663999900281425414760860803653486073017120361328125f,-0.182599981480837225689128899830393493175506591796875f,0.6575000416114928913913217911613173782825469970703125f,-0.102599987590313190821689204312860965728759765625f,0.4855000267624856036263736314140260219573974609375f,0.4243999883294105046616095933131873607635498046875f,0.31249998979270454668721868074499070644378662109375f,0.024999998509883880615234375f,1.2230000245273113801403042089077644050121307373046875f,0.237199989694357071812191861681640148162841796875f,1.803900020016730110228309058584272861480712890625f,0.2604000072836871737536057480610907077789306640625f,0.090699957686661836220309851341880857944488525390625f,-0.17140000060796722891609533689916133880615234375f,-0.8755000383108859640657328782253898680210113525390625f,-0.923999987304210179672736558131873607635498046875f,-0.07649999296665210835044490522705018520355224609375f,-0.7855999945342535539793971111066639423370361328125f,-0.130599991218744992949041261454112827777862548828125f,-0.49759998267292981921627870178781449794769287109375f,-0.5295999940201638889192281567375175654888153076171875f,-0.15579999191761029209146727225743234157562255859375f,-0.111099992462992513964081808808259665966033935546875f,-0.3897999905467035564043953854707069694995880126953125f,-0.59909998264312758209371168049983680248260498046875f,0.24940000151693819674392216256819665431976318359375f,0.6570999801993371303154845008975826203823089599609375f,0.36090000782012932045716979700955562293529510498046875f,0.03270002226829493707782603451050817966461181640625f,-0.682199971067905419630506003159098327159881591796875f,-0.65669997231960319794552560779266059398651123046875f,-0.1407999771282077983158842471311800181865692138671875f,-0.16119999374374749390170791230048052966594696044921875f,-0.5994999863691627961514996059122495353221893310546875f,-1.30639996752738962726425597793422639369964599609375f,-0.387000023305416274155277278623543679714202880859375f,1.288199964690208698669948716997168958187103271484375f,0.5421999817967415591368762761703692376613616943359375f,-0.4542000065922735529966303147375583648681640625f,-0.36409999098777756643130487645976245403289794921875f,-0.335899992099404354206626521772705018520355224609375f,-0.716799999547004684785633799037896096706390380859375f,-0.70910000134408457395096547770663164556026458740234375f,-1.0443999990582464310051591382944025099277496337890625f,-0.3486999933898449199887181748636066913604736328125f,-0.017799979162216317263300879858434200286865234375f,0.208600021594762541354839413543231785297393798828125f,-0.04080000897943991677863095901557244360446929931640625f,0.67799998936057093157359076940338127315044403076171875f,-0.254300002378225420418544899803237058222293853759765625f,-0.759499980092048776469937365618534386157989501953125f,-0.7652999836325646842993819518596865236759185791015625f,-0.45140000600963825494460479603731073439121246337890625f,0.02459999599456796914864753489382565021514892578125f,-0.884200012254714895476581659750081598758697509765625f,-0.4935999834477902137308547025895677506923675537109375f,0.390599993528425226685385496239177882671356201171875f,-0.018600011014938022668729900033213198184967041015625f,1.08999998211860660290994928800500929355621337890625f,0.0522000005722045301581601961515843868255615234375f,0.058099997980147477516510434725205413997173309326171875f,0.6093999859452245981827900322969071567058563232421875f,-0.15679999662265142745809498592279851436614990234375f,0.825199984419345611286189523525536060333251953125f,0.127000011309981619689324361388571560382843017578125f,-0.76999996662139924552548109204508364200592041015625f,-0.413799991261959210220311433658935129642486572265625f,-0.17539998806118983765145458164624869823455810546875f,0.643699996779858896189807637711055576801300048828125f,0.12940000263452511308770453979377634823322296142578125f,1.1110000043511387701755666057579219341278076171875f,0.4517999889671802105084452705341391265392303466796875f,1.2253999758422373655974979556049220263957977294921875f,-0.2613999830245974376197182209580205380916595458984375f,0.986900021409988337239838074310682713985443115234375f,0.61919998036623002946043925476260483264923095703125f,0.45479999561309814026799358543939888477325439453125f,-0.1461999948799610482552679968648590147495269775390625f,0.5729000098541379148997521042474545538425445556640625f,0.10279999212622659143789860536344349384307861328125f,0.66790001298338186330738608376123011112213134765625f,-0.45740002142488933589703492543776519596576690673828125f,-0.827499999403953356846841415972448885440826416015625f,0.1262999927520749299247881936025805771350860595703125f,1.1972000078737732309264174546115100383758544921875f,0.38940000501871108173901347981882281601428985595703125f,-0.400899997091293258666411247759242542088031768798828125f,0.08449999585002643254227905345032922923564910888671875f,1.1442000083804126919773125337087549269199371337890625f,0.4987000089615583053870295771048404276371002197265625f,-0.619400007887184411714542875415645539760589599609375f,-0.51709998303055737522981871734373271465301513671875f,-0.8645999821364880322249746313900686800479888916015625f,-0.6456999885320662269094782459433190524578094482421875f,0.16449999864399433846529063885100185871124267578125f,0.103200000378489420427285949699580669403076171875f,0.199999995529651641845703125f,0.7124999964237215266393832280300557613372802734375f,1.110399971371889282778511187643744051456451416015625f,0.8006999951630831002802324292133562266826629638671875f,0.17970000474452962890836715814657509326934814453125f,-0.17160000499486915970237532746978104114532470703125f,0.072299994793534327897077673696912825107574462890625f,0.2888000095158813618212434448651038110256195068359375f,0.0050000122189519657922573969699442386627197265625f,0.1941999963104727999763099433039315044879913330078125f,1.169199971947074256917176171555183827877044677734375f,0.403799984407424872312475372382323257625102996826171875f,-0.494499994099140138814618694595992565155029296875f,-0.38259999951124168404703596024774014949798583984375f,1.0429000234141947434096664437674917280673980712890625f,0.38559999662041644086230007815174758434295654296875f,0.127100024567544434006549636251293122768402099609375f,0.1039999804496769542083711712621152400970458984375f,-0.26809999587535815468442024211981333792209625244140625f,0.373199994120001654618334896440501324832439422607421875f,0.00449999078363172433370209546410478651523590087890625f,-0.510999995857476907445970937260426580905914306640625f,-0.92820003181993993024434530525468289852142333984375f,-0.11860000006258475391263118581264279782772064208984375f,0.15539999051988129252777071087621152400970458984375f,0.222199995356798130075048902654089033603668212890625f,0.90049997948855153850900023826397955417633056640625f,-0.2885999795734885964293425786308944225311279296875f,0.31569998674392640847230495637631975114345550537109375f,-0.09740000033974638427025638520717620849609375f,-0.486600019560754493141985221882350742816925048828125f,-0.2525999880373479555828453158028423786163330078125f,-0.4461000015154483744339586337446235120296478271484375f,0.310199983823299607621493123588152229785919189453125f,0.00639998496174831732474785894737578928470611572265625f,-0.38939998415708554357905768483760766685009002685546875f,0.3315000074207776759038779346155934035778045654296875f,0.5815999644637113163980757235549390316009521484375f,0.086199987578392534004478875431232154369354248046875f,-0.3347999790728095614866788309882394969463348388671875f,-0.3658999851703643724221137745189480483531951904296875f,0.12219999997615804687711715814657509326934814453125f,1.2400000087916851487790381725062616169452667236328125f,-0.090300012019276731933814517105929553508758544921875f,1.2886000128030772327036856950144283473491668701171875f,0.58490001591145979187302827995154075324535369873046875f,-0.09099999600648889241938377381302416324615478515625f,-0.173799994912743560604440062888897955417633056640625f,0.1235999995410441698595604975707828998565673828125f,-0.01239999475181097210452207946218550205230712890625f,-0.095200000065565060225480920053087174892425537109375f,-0.7369000043481588324567610470694489777088165283203125f,-0.3172999914035201651785200738231651484966278076171875f,-0.09100000267475849557996525618364103138446807861328125f,-0.01770000841021523996232645004056394100189208984375f,-0.0698000090986490295819066886906512081623077392578125f,-0.223599996784329402288449273328296840190887451171875f,-0.3820000157803298268532898873672820627689361572265625f,0.00709999033212671548653815989382565021514892578125f,0.671000001370906762332424477790482342243194580078125f,0.894600000613927637260758274351246654987335205078125f,0.4886000036388635781037237393320538103580474853515625f,0.5686999921977520244809056748636066913604736328125f,-0.12800001893937551500357585609890520572662353515625f,0.66239996860027350322752681677229702472686767578125f,0.123699972081184217387317403336055576801300048828125f,-0.438300013822316980593996049719862639904022216796875f,0.0188000144332651775158637974527664482593536376953125f,-0.63489999914765338218103352119214832782745361328125f,-0.6920000104159118592406230163760483264923095703125f,-0.6832999850630760985126244122511707246303558349609375f,-0.64429999440610419281227905230480246245861053466796875f,-0.342000009782612135467161351698450744152069091796875f,-0.707799993616342160152044016285799443721771240234375f,-0.26360002230256807109753935947082936763763427734375f,-0.4399999919533728043319342759787105023860931396484375f,0.568599991941451943233687416068278253078460693359375f,-0.6147999875664709179545752704143524169921875f,0.901000032216310753341304007335565984249114990234375f,0.0930000000894068978851692008902318775653839111328125f,1.468900004854798435616203278186731040477752685546875f,-0.176999991714954452159958009360707364976406097412109375f,-0.1085999795362351250105348299257457256317138671875f,0.027000000357627840230634319595992565155029296875f,-0.1290999850273129023747742394334636628627777099609375f,-0.19999998778104799423971371652442030608654022216796875f,0.28290001138150699944162624888122081756591796875f,-0.74259998013973227415362998726777732372283935546875f,-0.2260999789774416068866003115545026957988739013671875f,-0.49519998978376378317989292554557323455810546875f,0.9431000094354151652709106201655231416225433349609375f,-0.04979999620914465907617341144941747188568115234375f,1.1966999990671876030745579555514268577098846435546875f,-0.7009999508559712211308578844182193279266357421875f,-0.09930001388937359507735891384072601795196533203125f,-0.36900000339746430455534209613688290119171142578125f,-0.28319997005462571593170650885440409183502197265625f,0.652599992656707428295703721232712268829345703125f,-0.110399994841217985452885841368697583675384521484375f,0.206799992245435770854555812547914683818817138671875f,1.131300017525255885431079150293953716754913330078125f,-0.48959999137520782142019015736877918243408203125f,0.2423000025793908829285783212981186807155609130859375f,0.36200001380592594646401494173915125429630279541015625f,-0.195000001788139254443876779987476766109466552734375f,0.3133999929189681399321898425114341080188751220703125f,0.24429997913241408014073385857045650482177734375f,-0.1813000154390926876857292882050387561321258544921875f,0.29419998098090316407393629560829140245914459228515625f,0.909899989964813027398804479162208735942840576171875f,0.834899986034631780285053537227213382720947265625f,0.15270001444518543909367735977866686880588531494140625f,-1.007499957568943838026598314172588288784027099609375f,-0.1942999770462510156221469515003263950347900390625f,0.12259997581839598979058791883289813995361328125f,0.0837999832898379537482469459064304828643798828125f,-1.08280001373290968302853798377327620983123779296875f,-0.747500025331973905196036866982467472553253173828125f,-0.348300012183189178216480286209844052791595458984375f,0.1719000071048737421364194233319722115993499755859375f,0.0492000092744826478252662127488292753696441650390625f,0.04199998806416982422007322384160943329334259033203125f,0.6584999846816064206223018118180334568023681640625f,0.414999978989362805492646657512523233890533447265625f,0.6788999860048294809899971369304694235324859619140625f,0.70059999893009650140385247141239233314990997314453125f,0.129100004100799470307947558467276394367218017578125f,-0.462499981373548585406041411260957829654216766357421875f,-0.94189996092990069254113905117264948785305023193359375f,-0.410899992918968337551888225789298303425312042236328125f,0.8637999627634880273063799904775805771350860595703125f,-0.538999982833862301134786321199499070644378662109375f,-1.1005000189021227896546406554989516735076904296875f,-0.35319997661113777809305247501470148563385009765625f,0.01290000415444314540991399553604423999786376953125f,0.03439999980330465945144169381819665431976318359375f,0.5452999953113499831403032658272422850131988525390625f,-0.47899996763467844740347345577902160584926605224609375f,-0.908499993454665144554383005015552043914794921875f,-0.146799987775087625863079665577970445156097412109375f,-0.50000001519918468861902738353819586336612701416015625f,0.40279997938573386395688658012659288942813873291015625f,-0.795900031587481127104410916217602789402008056640625f,0.08920002477168953447517196764238178730010986328125f,-0.8975999913901093751888993210741318762302398681640625f,0.80099999377131414490804672823287546634674072265625f,-0.3915000099167229574703696926007978618144989013671875f,-0.02879999379515663093798139016143977642059326171875f,-0.003299985249340497972525554359890520572662353515625f,0.52959998675584785132741671986877918243408203125f,0.61990001546442508040257735046907328069210052490234375f,0.3385999919414519609972558100707828998565673828125f,-0.545800006371736312615894348709844052791595458984375f,-0.36659998264312765314798525650985538959503173828125f,-0.54219999520778683343991133369854651391506195068359375f,0.22319998719096201256206768448464572429656982421875f,0.364600003501028169239361886866390705108642578125f,0.486399984104931404349514423302025534212589263916015625f,0.62060000010356330957250747815123759210109710693359375f,0.4844999882504346100375869355048052966594696044921875f,0.32849998037889627244823032015119679272174835205078125f,0.8636999899439512518029005150310695171356201171875f,0.49879999841451638786793409963138401508331298828125f,0.471199984076619227124638200621120631694793701171875f,0.628299972844124088311446030274964869022369384765625f,0.8751999958187341999149566618143580853939056396484375f,0.4944000001385806530862510044244118034839630126953125f,0.26040001916736332798762987295049242675304412841796875f,-0.22429995961189330699880883912555873394012451171875f,0.562900050348043468062542160623706877231597900390625f,-0.03509999995827683960669673979282379150390625f,0.893500032886862971537311750580556690692901611328125f,0.252900002440810123260916952858678996562957763671875f,0.038599998572468818824887648588628508150577545166015625f,-0.737500011734664173701503386837430298328399658203125f,-0.7962999998301263371303093663300387561321258544921875f,-0.5253999999448655255918083639699034392833709716796875f,-0.57410002142339966457740274563548155128955841064453125f,0.311499988291412677199332392774522304534912109375f,1.027299997960030797372610322781838476657867431640625f,1.4930000090301032145134740858338773250579833984375f,0.90319999993145483330181377823464572429656982421875f,0.5304999762475492985913660959340631961822509765625f,0.53020000632405250851064693051739595830440521240234375f,0.214500033028423242598137221648357808589935302734375f,-0.44079999772906308663777963374741375446319580078125f,0.254899990132451126356727399979718029499053955078125f,0.8896999894708395029141456689103506505489349365234375f,-0.055399964051692729416487281923764385282993316650390625f,-0.1963999734878546232774709778823307715356349945068359375f,-1.14289999236390027448351247585378587245941162109375f,0.214399992501735692229658525320701301097869873046875f,-0.65110001526176930486400351583142764866352081298828125f,-0.446999980986118483627933528623543679714202880859375f,-1.204300015826523573281292556202970445156097412109375f,-0.211599999928474280519452577209449373185634613037109375f,-1.0239000412508847404069456388242542743682861328125f,-0.7112000023305409257545761647634208202362060546875f,-1.50240006072670251313638800638727843761444091796875f,-0.10160000097155563736350813996978104114532470703125f,-1.923000062264502485476214133086614310741424560546875f,0.21579999340772626492679364673676900565624237060546875f,-1.02760003009587563838067580945789813995361328125f,0.2391999928086996352050164205138571560382843017578125f,0.95310000485330859110177925686002708971500396728515625f,0.024200020450353021406186826425255276262760162353515625f,1.422500022035091848948695769649930298328399658203125f,-0.18399999918788656483314980505383573472499847412109375f,-0.79289999154433610240033658556058071553707122802734375f,-0.2324000174760811887608724646270275115966796875f,-0.956700020077823953812412582919932901859283447265625f,-0.8101999614715580388946136736194603145122528076171875f,0.6690000358447429906760817175381816923618316650390625f,0.38899997352063697686475052250898443162441253662109375f,0.5287999713316562377940499573014676570892333984375f,-0.83959996872544306878438646890572272241115570068359375f,-0.66009995827227785358815026484080590307712554931640625f,0.064700000572204263704634286114014685153961181640625f,0.008700012016296110317625789321027696132659912109375f,0.4078000307202340746215440958621911704540252685546875f,-0.51429999477863308499081540503539144992828369140625f,-0.43180000625252734636205786955542862415313720703125f,-0.77400000056624396194138171267695724964141845703125f,-0.695800005701184165474160181474871933460235595703125f,-0.68510000066608178759253178213839419186115264892578125f,-0.011599990838766116585389909232617355883121490478515625f,0.271400011932849682949608904891647398471832275390625f,-0.6626999876976016334850783096044324338436126708984375f,-1.3653999622821810788764196331612765789031982421875f,-0.949899983203410958054746515699662268161773681640625f,0.044499997079372466401991914608515799045562744140625f,0.2022999979972837092390136604080908000469207763671875f,0.2260000032335518493908921300317160785198211669921875f,0.4510000054687262061037245075567625463008880615234375f,-0.20180001169145089789935809676535427570343017578125f,0.123199988010525895987257172237150371074676513671875f,-0.20000001072883577535321819595992565155029296875f,-0.32010002707839024793656790279783308506011962890625f,0.239499995589256275962952713598497211933135986328125f,-0.1191999948203561476844924982287921011447906494140625f,-0.271399994498491281547103426419198513031005859375f,-0.1057000205323099795151620128308422863483428955078125f,0.36689997847601796276961749754264019429683685302734375f,0.7598999934293326763423692682408727705478668212890625f,0.588700013953447065517821101821027696132659912109375f,0.7714999943077562871707186786807142198085784912109375f,-0.6954000418916346237807601937674917280673980712890625f,-0.63539999804496716251378529705107212066650390625f,-1.3668000163853164519167648904840461909770965576171875f,0.0098000057458875744487158954143524169921875f,-0.168099982762336619401821735664270818233489990234375f,-0.04519999693632126280817828956060111522674560546875f,0.363100017742812752885583904571831226348876953125f,0.076999999240040750692060100845992565155029296875f,-0.08529999506920571405998998670838773250579833984375f,0.141399991220235943245597809436731040477752685546875f,0.52060000082999458026478123429114930331707000732421875f,0.40619997252821971045477766892872750759124755859375f,0.7150999928429728758061401094892062246799468994140625f,-0.40959997498393097004054652643389999866485595703125f,-0.0137000006541614371968762497999705374240875244140625f,-0.26600000047683691750677326126606203615665435791015625f,-0.9066000007480383704461246452410705387592315673828125f,0.387199984031915800386514092679135501384735107421875f,-0.371199956360458660498125027515925467014312744140625f,0.760999986022710572797222994267940521240234375f,0.798900021655857273117362638004124164581298828125f,0.48779997498989136062164106988348066806793212890625f,-0.016300012123584561862799091613851487636566162109375f,0.52739998603463167370364317321218550205230712890625f,-0.6436999995551999642628970832447521388530731201171875f,-0.61079997195005442112147875377559103071689605712890625f,-0.091799950391799001181425410322844982147216796875f,0.14799998906254785424607689492404460906982421875f,-0.39739998555034361515936325304210186004638671875f,0.0251999924659729312992340055643580853939056396484375f,-0.32640001737177348761775874663726426661014556884765625f,0.184300012362003384769337799298227764666080474853515625f,-0.154399986355006901828090803974191658198833465576171875f,-0.14099998863041396379713887654361315071582794189453125f,0.406100001701712454149628683808259665966033935546875f,0.0860000158995390950877890645642764866352081298828125f,-1.169999997764825661050736016477458178997039794921875f,0.0223999765723947863449438955285586416721343994140625f,1.06079994229674401395868699182756245136260986328125f,0.512999959930777738037477320176549255847930908203125f,-0.528500010311603318768902681767940521240234375f,0.0132999780595302041774630197323858737945556640625f,0.4754999841451645892220767564140260219573974609375f,0.6962999785959720355776880751363933086395263671875f,1.311399975425004971185671820421703159809112548828125f,1.1734999907165768462391497450880706310272216796875f,1.023499981775879863477030085050500929355621337890625f,0.53099998840689632828571120626293122768402099609375f,0.169700009661912787350956932641565799713134765625f,0.036099978381395114723773076548241078853607177734375f,0.7945999778151513481816436978988349437713623046875f,0.45379998381137853780131763414829038083553314208984375f,0.123999995332211288001644788892008364200592041015625f,-0.5060000089332461481461677976767532527446746826171875f,-0.80499999225139617919921875f,-0.18969999803900694512748259512591175734996795654296875f,0.2103999793812636465162313470500521361827850341796875f,-0.32250001095235336645572488123434595763683319091796875f,-0.68060001584291551779415385681204497814178466796875f,1.53679998703002951287999167107045650482177734375f,-1.1881000119686131455409849877469241619110107421875f,1.039799993228912367015936979441903531551361083984375f,0.8732000276476146183313176152296364307403564453125f,-0.82280001164674754221550756483338773250579833984375f,0.822000012502074550724273649393580853939056396484375f,-0.5323999846190214935148787844809703528881072998046875f,-0.406799992543458444060888723470270633697509765625f,0.4955999777704478415074618169455789029598236083984375f,-0.2895999979317185335503381793387234210968017578125f,0.053199998068809595253014776972122490406036376953125f,0.8284000097572796761369318119250237941741943359375f,-0.9832000163972371975518171893781982362270355224609375f,-1.1726999853134163487311525386758148670196533203125f,1.445599982985854570216588399489410221576690673828125f,-1.48740004906654377236918662674725055694580078125f,0.15800000292062765794298684340901672840118408203125f,-0.99400003882497589557942774263210594654083251953125f,0.4162000117182731440124143773573450744152069091796875f,-0.22899997407943117888606820997665636241436004638671875f,0.21059996398687419461026593126007355749607086181640625f,0.3518000346392382571281132186413742601871490478515625f,-0.28580002134740301045212618191726505756378173828125f,-0.536999997675419393772244802676141262054443359375f,0.60699999372661128660411122837103903293609619140625f,-0.369100007714331102004479134848224930465221405029296875f,-0.04159999851286412531070624254425638355314731597900390625f,-0.02920003672987203646727039085817523300647735595703125f,-0.857300007589906432059478902374394237995147705078125f,-0.1831000100128353513806445107547915540635585784912109375f,-1.43640005242675616869973964639939367771148681640625f,-0.2825000017881393876706397350062616169452667236328125f,-0.5101999958932399437117055640555918216705322265625f,0.199900005443394201964935064097517170011997222900390625f,0.42750005125999468447162144002504646778106689453125f,0.2714000035881996009123895419179461896419525146484375f,0.6176000299096109369401119693065993487834930419921875f,-0.0352999996140599403560145219671539962291717529296875f,-0.7575000052899121261162918017362244427204132080078125f,-0.29590000092834234379779445589520037174224853515625f,-1.963600063727796385393276068498380482196807861328125f,-0.35760001381635675432590915079344995319843292236328125f,-0.43720003395080653252335878278245218098163604736328125f,0.230500004149973403144713302026502788066864013671875f,1.295500037938356996392030850984156131744384765625f,0.1681000119313598162307243910618126392364501953125f,-0.3034999369978894634414245956577360630035400390625f,-0.22059999378919592505354785316740162670612335205078125f,0.37380001994669509457480671699158847332000732421875f,0.27150000384449957113730533819762058556079864501953125f,1.21340005580186893752170362859033048152923583984375f,0.20470000955015432708705702680163085460662841796875f,-0.1910999826282253621911877417005598545074462890625f,-0.1916999972790478512507661434938199818134307861328125f,1.08619998526871253119452376267872750759124755859375f,0.327800003562867647843859231215901672840118408203125f,0.570300029975175704066714388318359851837158203125f,-0.519700005992502322982318219146691262722015380859375f,0.98760000919699664212458856127341277897357940673828125f,-1.31420003531426221599076598067767918109893798828125f,1.419199994745850457178448777995072305202484130859375f,0.19550001141428907658337266184389591217041015625f,-0.39200001749396307104689185507595539093017578125f,0.29730006073117198894806278985925018787384033203125f,0.359199957269430658612918705330230295658111572265625f,0.2231000036984678303042528568767011165618896484375f,-0.2468000058799979523627143862540833652019500732421875f,-0.0167000088274480429362256472813896834850311279296875f,0.6211999977856874632919925716123543679714202880859375f,-0.9246000525072222142597411220776848495006561279296875f,0.779800031822919681445682726916857063770294189453125f,-0.113999989092350428876443402259610593318939208984375f,-0.02600002253055555545557808727608062326908111572265625f,1.74160000261068415028375966357998549938201904296875f,-1.4539999820888045434230662067420780658721923828125f,0.4859000206351282713512773625552654266357421875f,0.1705999768018724438434219337068498134613037109375f,0.8399000314831734925746786757372319698333740234375f,-0.668800004896521471664527780376374721527099609375f,2.0093000517010697336672819801606237888336181640625f,-1.2586000008821489615229438641108572483062744140625f,0.246600027644635844126241863705217838287353515625f,-0.0093999906390909160336377681232988834381103515625f,0.84459998317957030167235643602907657623291015625f,-1.55199995826184800051805723342113196849822998046875f,0.360200010645389312458064523525536060333251953125f,0.415500023484230762704783046501688659191131591796875f,0.1455000062361360146923061620327644050121307373046875f,1.7435000521838670550778260803781449794769287109375f,0.20690000907927752482606820194632746279239654541015625f,0.72770004558861278720627296934253536164760589599609375f,0.2382999975055455632588774506075424142181873321533203125f,0.79540001614987809741563751231296919286251068115234375f,0.05910000056177378002786326760542578995227813720703125f,0.8367000229835515057175143738277256488800048828125f,0.085600007200241190474798713694326579570770263671875f,0.141400013199449059442258658236823976039886474609375f,0.089300002825260182959254962042905390262603759765625f,-0.74140002824962170091538382621365599334239959716796875f,-0.327700009825825755438444275569054298102855682373046875f,0.447299963873624140120455194846726953983306884765625f,0.3542999995470046403767128140316344797611236572265625f,0.6166000114023681977215574079309590160846710205078125f,-0.29930000547021629042632184791727922856807708740234375f,-1.54730002645850195364118917495943605899810791015625f,-0.227699994738399968952080598683096468448638916015625f,-1.811499985814093438563077143044210970401763916015625f,-0.19429999280422916019261947440099902451038360595703125f,-0.5585000052452080421261371157015673816204071044921875f,-0.15390000280588866932163227829732932150363922119140625f,-1.11530003787279152760447686887346208095550537109375f,-0.3809000071123243014170611786539666354656219482421875f,-2.29170003184974202525836517452262341976165771484375f,-0.2203999987527727189018378339824266731739044189453125f,-0.209700025233626075049642167869023978710174560546875f,0.9245000021457681071979095577262341976165771484375f,-1.188599976742267916307582709123380482196807861328125f,1.7889000056743622479871191899292171001434326171875f,-1.2317999988019465718025458045303821563720703125f,1.335600014278292224645383612369187176227569580078125f,-0.58359998650252820073802695333142764866352081298828125f,0.52310000250637500585071393288671970367431640625f,-0.31340000334978090901216774000204168260097503662109375f,1.2340000235140333728622863418422639369964599609375f,-1.04999999016523393180477796704508364200592041015625f,0.8404000376820572881797488662414252758026123046875f,0.091400003960728337659702447126619517803192138671875f,-0.58300003987550785833349209497100673615932464599609375f,0.44480001617670072988630636245943605899810791015625f,0.4538999613434071545015058291028253734111785888671875f,-1.339399972906708757847127344575710594654083251953125f,0.20839996818304040715474911849014461040496826171875f,0.3584000181019304509533185409964062273502349853515625f,-0.704399994885921643117399071343243122100830078125f,1.1777999661982061496701135183684527873992919921875f,-1.6054999951720230200180594692938029766082763671875f,1.024799979817867079390225626411847770214080810546875f,-1.31600002551078798518346957280300557613372802734375f,0.3343999987602235268724371053394861519336700439453125f,-1.095400035297870999784208834171295166015625f,1.2536000059485434832140526850707828998565673828125f,-2.031799988520145205939115840010344982147216796875f,1.48679996505081657431901476229541003704071044921875f,-0.4250000062584877724702892010100185871124267578125f,-1.318200029659271077520088510937057435512542724609375f,-0.2519999946951865243960355655872263014316558837890625f,0.2013999950200326960469965342781506478786468505859375f,0.01109999626278879159002599408267997205257415771484375f,1.235400015851855215487375971861183643341064453125f,0.1915000094324350587982053184532560408115386962890625f,0.16140004017055087359722165274433791637420654296875f,-0.01150000265240669339306123220012523233890533447265625f,1.2384000065535307744113424632814712822437286376953125f,0.26490000473558905635940163847408257424831390380859375f,0.430400028741359630402030234108678996562957763671875f,-0.029900002686679361119104214594699442386627197265625f,-0.3641000134885314931665334370336495339870452880859375f,0.1267999994724988699346113207866437733173370361328125f,0.388799995061754888325822321348823606967926025390625f,0.0582999972820281708862921732361428439617156982421875f,1.233200000713765120963216759264469146728515625f,0.1682999959968029812795720090434770099818706512451171875f,-0.137800038282573977799216891071409918367862701416015625f,-0.37880001339018354311605207840329967439174652099609375f,-1.193000044122338731966692648711614310741424560546875f,-0.0104999949485062860077277946402318775653839111328125f,-0.40700000788271406548801678582094609737396240234375f,-0.21460000169426207694556296701193787157535552978515625f,-0.853900016775727355167191490181721746921539306640625f,-0.16830000249743459761475605773739516735076904296875f,-1.0248000386774542835865986489807255566120147705078125f,-0.093500004909932654673099250430823303759098052978515625f,-0.69100002156198048197666139458306133747100830078125f,-0.7210000040531152620815191767178475856781005859375f,0.184799984288215757288753593456931412220001220703125f,-0.14660001907646602603563223965466022491455078125f,-0.9543999732792378676293765238369815051555633544921875f,1.3136000219300389968424269682145677506923675537109375f,-0.84680001690685724469176420825533568859100341796875f,0.483600008332729203175404109060764312744140625f,-0.31359999440014352245498230331577360630035400390625f,1.066400003141165342412932659499347209930419921875f,-1.2101999806195500308803048028494231402873992919921875f,0.665600043410062625071077491156756877899169921875f,0.466399981087446402483465135446749627590179443359375f,0.127900043568014876171901050838641822338104248046875f,0.3555999784410002195755851062131114304065704345703125f,0.543399988746642748083104379475116729736328125f,-0.872999984815716612018832165631465613842010498046875f,1.4188999824285499773424135128152556717395782470703125f,-1.481199981468915627402793688816018402576446533203125f,0.13479993507713039502249330325867049396038055419921875f,0.1942000499546525649208206232287921011447906494140625f,-1.335900021678208648978625205927528440952301025390625f,0.902999998524784519560171247576363384723663330078125f,-1.034200022983551292554693645797669887542724609375f,0.98979999196231371882959138019941747188568115234375f,0.2845999877989289217339319293387234210968017578125f,-0.437999994501471423546945516136474907398223876953125f,-0.12459999517500397470115558462566696107387542724609375f,0.55680000558197495497125828478601761162281036376953125f,-0.2985000135153532685450272765592671930789947509765625f,-0.0008000024974348374229293767712078988552093505859375f,0.04759999571144579721959644302842207252979278564453125f,0.7221999760597939133077716178377158939838409423828125f,0.1773999922558664454896870665834285318851470947265625f,0.44150002038478763921602876507677137851715087890625f,0.312400008907914195077637486974708735942840576171875f,0.532400044223666935039318559574894607067108154296875f,-0.0682999993681907557885324422386474907398223876953125f,-0.704599984669684875626671782811172306537628173828125f,-0.068099995242059119249233845039270818233489990234375f,-0.4133000288724897330894236802123486995697021484375f,-0.426400009101629340335648521431721746921539306640625f,-0.512400039231777792991806563804857432842254638671875f,0.2760999965980648340746483881957828998565673828125f,0.38609999507069503454914638496120460331439971923828125f,-0.096700002010166652421929711636039428412914276123046875f,-0.0053999898493291009771155586349777877330780029296875f,0.250599999898672098908036787179298698902130126953125f,0.8664000178933140006165558588691055774688720703125f,0.3044999971911310776562231694697402417659759521484375f,1.4564999935626976590441472581005655229091644287109375f,0.172399993000924511843408026834367774426937103271484375f,0.33120000579506092552861673539155162870883941650390625f,0.2115000071227550737518185997032560408115386962890625f,0.2812000363796955948458844432025216519832611083984375f,-0.257599999921023836702005382903735153377056121826171875f,-1.4817000418335199984909422710188664495944976806640625f,-0.267100008772313657345875981263816356658935546875f,-0.34560002266019618222259168760501779615879058837890625f,0.27289999469220571626237870077602565288543701171875f,-0.6948000049263234334517846946255303919315338134765625f,0.4217000081196429217555987634113989770412445068359375f,0.1846000304907555733535673425649292767047882080078125f,0.1788000481843941003035070025362074375152587890625f,0.19099994295835553970164255588315427303314208984375f,0.0071000190913679261939250864088535308837890625f,0.142799996522068983750841653090901672840118408203125f,-0.993299971130489733894819437409751117229461669921875f,0.507799969625473135437232485855929553508758544921875f,-0.784200030136108949818662949837744235992431640625f,0.3800000002980237212568681570701301097869873046875f,0.7699000055551525889541153446771204471588134765625f,-0.350399979865551269853085614158771932125091552734375f,1.14829998964518242843269035802222788333892822265625f,-0.82219999680966104893542478748713620007038116455078125f,0.22290004775896654631850424266303889453411102294921875f,0.2301999793529512228218436575843952596187591552734375f,-0.371199962246418380118484492413699626922607421875f,0.3413999815344812560624632169492542743682861328125f,0.81680000174492572995177397388033568859100341796875f,-0.6529999743103982456915446164202876389026641845703125f,0.3015000207200644322114158057956956326961517333984375f,-0.07680002480447267743102202075533568859100341796875f,-0.46670001210570344341022064327262341976165771484375f,0.4061999931663275997806294981273822486400604248046875f,-1.4168000500246886730337791959755122661590576171875f,0.8288000163704156708632808658876456320285797119140625f,-0.4859000276759271041981946837040595710277557373046875f,-0.0270000168979169785643534851260483264923095703125f,0.32090000003427265884425878539332188665866851806640625f,1.38410001870393717382512477342970669269561767578125f,0.183200002688169494291514638462103903293609619140625f,0.11840001810193090392431258806027472019195556640625f,0.0431000023946166432864401940605603158473968505859375f,-0.211300004002451913276416917142341844737529754638671875f,-0.04810000034570693916435857317992486059665679931640625f,0.726399983471631482956354375346563756465911865234375f,0.29709999163597811122627945223939605057239532470703125f,1.73110004715025400656713827629573643207550048828125f,0.4047000146538020093345267014228738844394683837890625f,0.972800061714650166067031022976152598857879638671875f,-0.18660000007748600570067765147541649639606475830078125f,-0.0328000341475014334235993374022655189037322998046875f,0.3191999927714466167572027188725769519805908203125f,1.506400004743038589793968640151433646678924560546875f,0.21539999592155212315702783598680980503559112548828125f,-0.056700010541081979908994981087744235992431640625f,-0.3081000078707933909072380629368126392364501953125f,-1.50540004177987629674362324294634163379669189453125f,-0.0269000034540891608259016720694489777088165283203125f,0.4315999859213828226955911304685287177562713623046875f,0.005300000359117984505274989714962430298328399658203125f,0.241300007876753763724764212383888661861419677734375f,-0.0641999995514750398939440856338478624820709228515625f,-0.3002999979376792527574480118346400558948516845703125f,0.05130000009089707990739270826452411711215972900390625f,0.0677000059515240337759678368456661701202392578125f,0.3533000074148180402744401362724602222442626953125f,-0.54279998497366932497243396937847137451171875f,1.006900002038479069454979253350757062435150146484375f,-0.84319998674392715454217750448151491582393646240234375f,-0.36889997974634081145950403879396617412567138671875f,0.95020000244975033609762249398045241832733154296875f,-0.8670000012516976273246882556122727692127227783203125f,0.563199985775351574801561582717113196849822998046875f,0.511899980729817460911590387695468962192535400390625f,-1.102799975287914069355110768810845911502838134765625f,0.8929000290393833694224667851813137531280517578125f,-0.655000017806887768756496370770037174224853515625f,0.5867000194072726770855297218076884746551513671875f,0.038999997735023494982442571199499070644378662109375f,0.7118999947741624367836266173981130123138427734375f,-0.8123999690473080459440780032309703528881072998046875f,0.6076999767824997622511773442965932190418243408203125f,-0.808799990293383341821709109353832900524139404296875f,-0.7723000155806545308223576284945011138916015625f,1.33280000024735922892205053358338773250579833984375f,-1.748999996840954196386519470252096652984619140625f,1.00699997539818308922576761688105762004852294921875f,-0.26460000568032260304107694537378847599029541015625f,-0.3187999999046324806073471336276270449161529541015625f,1.1752000139236447040502753225155174732208251953125f,-0.74060000652968849266244433238171041011810302734375f,0.6415000197887419375319950631819665431976318359375f,-0.3868000050604341044646616865065880119800567626953125f,0.228099993118644039924447497469373047351837158203125f,-0.988699995606391990321526463958434760570526123046875f,-0.0175999964937567643374904946540482342243194580078125f,1.103300030660629094114710824214853346347808837890625f,0.4199999900162219024224441454862244427204132080078125f,0.572399987226723805378014731104485690593719482421875f,-0.2934000008165835726714476550114341080188751220703125f,-1.3254000238984826598453992119175381958484649658203125f,-0.0025000033900142160092627818812616169452667236328125f,0.146700003015995061872445148765109479427337646484375f,0.0073000040695071211072075811898685060441493988037109375f,0.7791000448554761259600809353287331759929656982421875f,0.35080000771284114424730660175555385649204254150390625f,1.409600029522180886232263219426386058330535888671875f,-0.023599999839067464790787198580801486968994140625f,-0.76340000074207747804422297122073359787464141845703125f,-0.170399997784197287931107211989001370966434478759765625f,0.06219999565482103864866303410963155329227447509765625f,0.2833999959364532639227718391339294612407684326171875f,1.4491000076025721821082470341934822499752044677734375f,0.331600002722442166458449719357304275035858154296875f,1.531100018912553650807240046560764312744140625f,0.0332000006765127420038652417133562266826629638671875f,-1.090400022706389915327918060938827693462371826171875f,-0.16490000719428066222604911672533489763736724853515625f,-0.31020002137422597598970241961069405078887939453125f,0.0602999995395541166232078467146493494510650634765625f,-0.2857000236243012469827817767509259283542633056640625f,-0.3220000088140368976752370144822634756565093994140625f,-1.6295000583976513297557175974361598491668701171875f,-0.73070002647414822316562776904902420938014984130859375f,-0.1132000039547680358964498736895620822906494140625f,-0.2124999767541897455203070421703159809112548828125f,-0.084000011146068143119691740139387547969818115234375f,-0.1111999946564434793572218040935695171356201171875f,0.542999992042779933143492598901502788066864013671875f,-0.6355999741941698033542706980369985103607177734375f,0.62579996658563674660769038382568396627902984619140625f,0.15830000093281260120647857547737658023834228515625f,-0.192800004792213375282017295830883085727691650390625f,1.000700031521917221510875606327317655086517333984375f,-1.3533999942600729582409258000552654266357421875f,0.83270001310408137129570604884065687656402587890625f,-0.4693999995797870639080429100431501865386962890625f,-0.65369999432116665527559007387026213109493255615234375f,1.11900000160932489023934977012686431407928466796875f,0.1358000213474024331361533768358640372753143310546875f,-0.24380002128779831860327931281062774360179901123046875f,1.3956999962806708737161898170597851276397705078125f,-1.4343999550253159203094810436596162617206573486328125f,1.8076999984264379150999957346357405185699462890625f,-0.909199970680475644257967360317707061767578125f,-0.5763999919652942338643697439692914485931396484375f,1.087999988988042066040407007676549255847930908203125f,-1.143899996584654132902869605459272861480712890625f,0.47439998043179532061230929684825241565704345703125f,-0.628200034949184082933015815797261893749237060546875f,0.1268000079661610879355748693342320621013641357421875f,-1.432500021755695573943967247032560408115386962890625f,0.57720000392496528807129152482957579195499420166015625f,-0.060500002101063710124861927397432737052440643310546875f,-1.001100032621622393236293646623380482196807861328125f,-0.13620000329911707837027279310859739780426025390625f,-0.539200023803115158216314739547669887542724609375f,0.062800003041326990516068917713710106909275054931640625f,-0.773299962189792911004815323394723236560821533203125f,-0.2010999901533125910901844690670259296894073486328125f,0.4412000292271383017350672162137925624847412109375f,0.3691000003010034280492845937260426580905914306640625f,1.5194000477850437835769525918294675648212432861328125f,0.212500002868473536477011975875939242541790008544921875f,-0.403799994987249277755836374126374721527099609375f,-0.2606000004015862625550425946130417287349700927734375f,-0.025900025235861490191524580950499512255191802978515625f,0.3943999955192207806931037339381873607635498046875f,0.493899986675380819889369377051480114459991455078125f,-0.1209000001460314166479292907752096652984619140625f,-1.57580001598298569120970569201745092868804931640625f,-0.2312000013992190350364808182348497211933135986328125f,-1.537100030267238626180414939881302416324615478515625f,-0.210700000713765600579563397332094609737396240234375f,-1.028700029078125854908876135596074163913726806640625f,-0.1363000006496906113540035221376456320285797119140625f,-0.62340001527071020603898432455025613307952880859375f,-0.136000007629394570329850466805510222911834716796875f,-0.01500005476176813346000926685519516468048095703125f,0.21839999668151133249693884863518178462982177734375f,0.943500017687677772215693039470352232456207275390625f,0.3107000138267874689290692913345992565155029296875f,0.38619999677985894503962072121794335544109344482421875f,-0.28219997112378447656766411455464549362659454345703125f,0.81519999097585671421484221355058252811431884765625f,0.1181000159919260994456635671667754650115966796875f,-0.0148000140160320370341651141643524169921875f,-0.43749996587634054634463609545491635799407958984375f,0.96739995734989658870972561999224126338958740234375f,0.613300015461444303355165175162255764007568359375f,-1.60720000452101174914787407033145427703857421875f,0.66440003929138224947337221237830817699432373046875f,0.23439999451339221803181089853751473128795623779296875f,-0.8297000100158153923501913595828227698802947998046875f,0.740799991992115991479295189492404460906982421875f,-0.67680000321641509142267523202463053166866302490234375f,0.149800032493471402261775438091717660427093505859375f,-0.5812000212550163524838353623636066913604736328125f,1.279800002244114498495264342636801302433013916015625f,-0.8321999975919727177142704022116959095001220703125f,1.23679997995197776816667101229541003704071044921875f,-0.4535999865770339312120995600707828998565673828125f,0.577999986529350362474133362411521375179290771484375f,-0.2569000103831289294475936912931501865386962890625f,-0.203199994865059796467221531202085316181182861328125f,-0.462799979162216601480395183898508548736572265625f,0.96839998427629492283585932455025613307952880859375f,0.2804999635815619285494904033839702606201171875f,-1.05159996081292650416116885025985538959503173828125f,-0.0080000196099287990136872394941747188568115234375f,-0.438499996975064032511681944015435874462127685546875f,-0.37690000002682200186399086305755190551280975341796875f,-1.0648000312268732781006974619231186807155609130859375f,0.0722000007584691072537452782853506505489349365234375f,-0.671500011891126735719126372714526951313018798828125f,-0.23580000458359717807610422823927365243434906005859375f,-1.487800088909268847459088647156022489070892333984375f,-0.3233000117361546354999290997511707246303558349609375f,-0.7988000496000058348045058664865791797637939453125f,-0.0818999989837407849080364030669443309307098388671875f,-0.9976000115811824997535950387828052043914794921875f,0.00100000286102297497592417130363173782825469970703125f,1.0418000486463314668839075238793157041072845458984375f,0.299900010100007108349018380977213382720947265625f,-0.57319997273683487293283178587444126605987548828125f,-0.28400000391900537266565152094699442386627197265625f,-1.72520006138384385252493302687071263790130615234375f,-0.150400002254545750446368401753716170787811279296875f,-0.8341000266015534680974496950511820614337921142578125f,-0.0849000058159232739996014061034657061100006103515625f,-0.257900049379468754295885446481406688690185546875f,-0.40100001608580360112910057068802416324615478515625f,-0.3069000781089084028963043238036334514617919921875f,0.1215999914720653496402746895910240709781646728515625f,-0.4055000060498716862866785959340631961822509765625f,-0.34379999550878992398139644137700088322162628173828125f,-1.1584000220507377765244427791913039982318878173828125f,-0.1208999990284442827004340870189480483531951904296875f,0.9470000294148928077220261911861598491668701171875f,-0.8417999947041270303316196077503263950347900390625f,1.039399979612231295078572657075710594654083251953125f,-0.1907999785810705617450366844423115253448486328125f,0.03259999511539934946569019302842207252979278564453125f,-0.37979998905658707286647768341936171054840087890625f,0.859999997168779284351103342487476766109466552734375f,-0.957800069761276073876388181815855205059051513671875f,0.34300005231797647820002339358325116336345672607421875f,-0.24930001683235136766825235099531710147857666015625f,0.6372000075012440589716788963414728641510009765625f,-0.624799996507167776371716172434389591217041015625f,0.30919998438954365838071680627763271331787109375f,0.49040004155635852356454051914624869823455810546875f,-0.4652000280052421832266418277868069708347320556640625f,-0.4467999637842172688806385849602520465850830078125f,1.48739999274015399777226775768212974071502685546875f,-1.087800009262561928835566504858434200286865234375f,1.25159999389350407028587142121978104114532470703125f,-0.47360004916191211776776981423608958721160888671875f,-0.4163999909222122486340822433703579008579254150390625f,0.1669999440312377458894843584857881069183349609375f,-0.1571999619036912321234922274015843868255615234375f,1.40079997671842448880852316506206989288330078125f,-1.4257999818593258822829739074222743511199951171875f,0.550600006902218552795602590776979923248291015625f,0.352999998897313815859888563863933086395263671875f,-0.33470001726150488963185125612653791904449462890625f,-0.2199999963492154098076980517362244427204132080078125f,1.3572000181555754494411303312517702579498291015625f,-0.930900011843442509729129596962593495845794677734375f,-0.247500000149011578631785823745303787291049957275390625f,0.09419997574687022368067346178577281534671783447265625f,0.0361999978601932337340940648573450744152069091796875f,0.418000028699636150264495881856419146060943603515625f,0.08550000414997338538114490802399814128875732421875f,-0.868099970543383836485418214579112827777862548828125f,-0.19949999421089881224133932846598327159881591796875f,-0.71329999363124307887318309440161101520061492919921875f,0.108100001165270798164641519178985618054866790771484375f,-0.55509997701048785501143356668762862682342529296875f,-0.20859999395310868663955261581577360630035400390625f,0.6721999843299395482887348407530225813388824462890625f,0.2138000016927719126869078536401502788066864013671875f,-0.002800004345179019082934246398508548736572265625f,-0.2251000048384070861828831766615621745586395263671875f,-0.807100014695525391772434886661358177661895751953125f,-0.0314999976605176978949884869507513940334320068359375f,-0.907600006663799252493163294275291264057159423828125f,-0.07250000450760130110694490213063545525074005126953125f,-0.672500024586916378410705874557606875896453857421875f,-0.15389999848455193642848826129920780658721923828125f,-0.4374999589473003158701658321660943329334259033203125f,0.2962000105634332580706313819973729550838470458984375f,1.3521000438272958721341865384601987898349761962890625f,0.2179000019565224544049186761185410432517528533935546875f,-0.2416999782055615497711187344975769519805908203125f,-0.2019999990165233239025610600947402417659759521484375f,-1.4380000163316724037798621793626807630062103271484375f,-1.0361000116854903740915005982969887554645538330078125f,0.499599991449713787261543984641321003437042236328125f,0.249999966770410697591842108522541821002960205078125f,-1.051799969446659321903325690072961151599884033203125f,0.76500004246830854981453740037977695465087890625f,0.216799953204394046935021833633072674274444580078125f,-0.33239996651411019001898239366710186004638671875f,0.767199973750114505577357704169116914272308349609375f,-0.88479999073296777822861258755438029766082763671875f,0.36459998925179248896455419526319019496440887451171875f,-1.872599986156075146936927922070026397705078125f,1.733999974191188453431777816149406135082244873046875f,0.62919996814727863210237046587280929088592529296875f,-1.7079999527037148521202425399678759276866912841796875f,0.5084999754428860541111134807579219341278076171875f,0.3658000144183635171657442697323858737945556640625f,-0.34500000223517446329424274154007434844970703125f,0.564399985572695772617635157075710594654083251953125f,-0.7900999952644112145350163700641132891178131103515625f,1.0715999953091144281103197499760426580905914306640625f,-0.327000017791986952175875558168627321720123291015625f,0.413599989557266400908019932103343307971954345703125f,-0.028399947172403017248143441975116729736328125f,0.1571999653309583688809425439103506505489349365234375f,0.7556000230699779773857471809606067836284637451171875f,-0.701399995541572618407144545926712453365325927734375f,-0.1261999729007481363396436790935695171356201171875f,0.9569999778568745174567311551072634756565093994140625f,-1.25879997963905321256561364862136542797088623046875f,-0.0719000040501363013589752881671302020549774169921875f,-0.12020000177919865080866657081060111522674560546875f,0.50080001136362550795411152648739516735076904296875f,0.41460000698417431852504932976444251835346221923828125f,1.1243000431329017718695695293718017637729644775390625f,0.010400000615417959171082884495262987911701202392578125f,0.81620002363920229893068380988552235066890716552734375f,0.2173000076085329634878462456981651484966278076171875f,0.03800000821053994837228628966840915381908416748046875f,-0.288700010675191964804042754622059874236583709716796875f,-1.342900037905574439633937799953855574131011962890625f,-0.21709999960809944763440171300317160785198211669921875f,-0.51730001073777653886054395115934312343597412109375f,0.0079000036329030798309958072422887198626995086669921875f,-0.1069999821037053155947660343372263014316558837890625f,0.03669999843388793703269357138196937739849090576171875f,1.0828999984174962900596028703148476779460906982421875f,0.2904999980032443307464973258902318775653839111328125f,1.125000039935111662003919263952411711215972900390625f,0.16830000141710044214704566911677829921245574951171875f,0.451300002139806810674826920148916542530059814453125f,-0.09630000023990870072765346776577644050121307373046875f,-0.178299986106157604126565274782478809356689453125f,0.20999999813735481524190618074499070644378662109375f,1.525800011289119684221304851234890520572662353515625f,0.30460000322163105490602674763067625463008880615234375f,0.344800021392106970097302109934389591217041015625f,-0.14210000035315750732678452550317160785198211669921875f,-0.001300007578730610902084663393907248973846435546875f,-0.2930999955028295023140572084230370819568634033203125f,1.2141999983966347276265196342137642204761505126953125f,0.3704000248670578088194815791212022304534912109375f,-1.34480001699626416922228600014932453632354736328125f,0.09570003703534657457652201628661714494228363037109375f,0.7069999925345180624702834393247030675411224365234375f,0.899700010518729609287902349024079740047454833984375f,-1.52500001110136462756372566218487918376922607421875f,0.04119998668432189870003412579535506665706634521484375f,0.49060001345872894518151952070184051990509033203125f,-1.1105000110417606595802908486803062260150909423828125f,1.0721999814987182020331601961515843868255615234375f,0.02379998954832540647430505487136542797088623046875f,-0.1744000013679265492783088120631873607635498046875f,0.446499997958540451037379170884378254413604736328125f,-0.2661999977111815240959913353435695171356201171875f,1.09619997863769480517248666728846728801727294921875f,-1.3573999732941388618456812764634378254413604736328125f,0.590400022184848438655535574071109294891357421875f,-0.022000014066696049752636099583469331264495849609375f,0.292199989396333847935238736681640148162841796875f,-0.253999988198280401974216147209517657756805419921875f,-0.052499960586429583742074100882746279239654541015625f,0.34599996277689992751902536838315427303314208984375f,1.0555999964714055661119118667556904256343841552734375f,-1.3947999659597878352457200890057720243930816650390625f,0.149499999314546716533413928118534386157989501953125f,0.5575999992132185667514931992627680301666259765625f,0.209500005125999422261884319595992565155029296875f,1.1371999963253731635859367088414728641510009765625f,0.2213000093668699985727243984001688659191131591796875f,0.433000039055944085930605069734156131744384765625f,-0.15380000173002483787598748676828108727931976318359375f,-0.529900010547041944164448068477213382720947265625f,-0.1172000006839633012845069970353506505489349365234375f,-0.6203000052392482199792311803321354091167449951171875f,0.0419000017032027383123704566969536244869232177734375f,-0.4028999978214502508677696823724545538425445556640625f,-0.09229999799728394460629488094127736985683441162109375f,-0.863800014284252792862162095843814313411712646484375f,-0.07710000195503230457916288287378847599029541015625f,0.121799984049796972129797723027877509593963623046875f,-0.087199998039007187156101963410037569701671600341796875f,-0.764000020384788314231627737171947956085205078125f,-0.2565000029131769831991505270707421004772186279296875f,0.1416999886363743410555571244913153350353240966796875f,0.1718000049486756442007617806666530668735504150390625f,0.96630004766285448880580588593147695064544677734375f,0.27530000382363795097262482158839702606201171875f,0.720400009071826996631671136128716170787811279296875f,0.054500003002583952138593303970992565155029296875f,-0.7233000183671720595413034970988519489765167236328125f,-0.3015000147968531774012035384657792747020721435546875f,-1.173400053343177429354682317352853715419769287109375f,-0.0334999990612268039313903500442393124103546142578125f,-0.54060001591742035120802256642491556704044342041015625f,-0.19260000074058769570228832890279591083526611328125f,-0.35189999540746175199501522001810371875762939453125f,0.698300000560283518069581987219862639904022216796875f,-1.719400017908215172468544551520608365535736083984375f,1.03280002565383899337803086382336914539337158203125f,-0.0532000054448840131726683466695249080657958984375f,0.68160000305771806239363286294974386692047119140625f,-0.30839999940097317931986253825016319751739501953125f,0.36460002728700668939154638792388141155242919921875f,0.2937999958813188872142063701176084578037261962890625f,0.18040001719295961635225467034615576267242431640625f,0.4975999828219415110908130372990854084491729736328125f,-0.3386000179812305166393571198568679392337799072265625f,-0.227199971440434911329475653474219143390655517578125f,-0.016800002825259952032865840010344982147216796875f,0.320799994450807535173453288734890520572662353515625f,0.341099974581599685308219704893417656421661376953125f,-0.433599970334768602953090521623380482196807861328125f,1.22360001131296147747207214706577360630035400390625f,-1.1324000065237280754359971979283727705478668212890625f,1.5049000493645667830833190237171947956085205078125f,-0.32800000850856303458158436114899814128875732421875f,1.5920999769210819607678786269389092922210693359375f,-0.887399956381321519671701025799848139286041259765625f,0.28459995456933828705814448767341673374176025390625f,0.1908000167280439196559882475412450730800628662109375f,-1.13020001064538888613242306746542453765869140625f,0.922400008013843919485452715889550745487213134765625f,-1.278300015461444782971511813229881227016448974609375f,0.76139998801648622883675443517859093844890594482421875f,-0.05059996428489643705006528762169182300567626953125f,-0.1407999917685984758719541787286289036273956298828125f,-0.07789999867826702395490201524808071553707122802734375f,-0.6789000043332571454612889283453114330768585205078125f,-0.2689999989271163460813340861932374536991119384765625f,0.11939998512566063482154277153313159942626953125f,0.16999999444931734871033768285997211933135986328125f,0.18589998673498586700958412620821036398410797119140625f,-0.054499998271465288635084789348184131085872650146484375f,0.532900005421042433084721778868697583675384521484375f,0.195400002254545679392094825743697583675384521484375f,-0.01749995209276644647644616270554251968860626220703125f,-0.215999995298683578059950605165795423090457916259765625f,-0.60340002972483564036565439892001450061798095703125f,-0.0248000016853213711698344923206605017185211181640625f,-0.327900015330314875683370701153762638568878173828125f,-0.1309000064417720210485640564002096652984619140625f,-0.6497000615179546745281413677730597555637359619140625f,-0.18900000689923768248945634695701301097869873046875f,0.03360000021159625926969738429761491715908050537109375f,0.222200005526840715930347869289107620716094970703125f,1.13850003251433395945468873833306133747100830078125f,0.3625000014528632075183622873737476766109466552734375f,1.399900017699599263920617886469699442386627197265625f,0.0825000016391277402050263845012523233890533447265625f,0.6090000253021716769552540426957421004772186279296875f,0.122200000348687165541150534409098327159881591796875f,-0.4623999720647926636729607707820832729339599609375f,-0.1619999926649033084036233276492566801607608795166015625f,0.207500013001264083101204960257746279239654541015625f,0.558699994805454736024330486543476581573486328125f,-0.4844000028580424110913327240268699824810028076171875f,-0.0969000421598547045931582033517770469188690185546875f,-0.2479999690949921120619592329603619873523712158203125f,0.3302999931693069157745412667281925678253173828125f,-0.0037999965518711320555667043663561344146728515625f,0.72829996446222100558998135966248810291290283203125f,-0.55619996317774111904697065256186760962009429931640625f,0.80749997215345492573845831429935060441493988037109375f,-1.11979997974336153987451325519941747188568115234375f,-0.66699996876716627358661071411916054785251617431640625f,1.28899995034933123605469518224708735942840576171875f,-0.9474000589758151402719477118807844817638397216796875f,-0.4007999525040393251629211590625345706939697265625f,-1.5034000471591948411287376075051724910736083984375f,1.508200022208689983216345353866927325725555419921875f,-1.28080007357597391859371782629750669002532958984375f,0.1678000252068041486808169793221168220043182373046875f,-0.349399982964992528167158525320701301097869873046875f,0.31759997827708719153605443352716974914073944091796875f,0.44300002527236959082301837042905390262603759765625f,-0.45999999418854731203509800252504646778106689453125f,0.867499979436397783416623497032560408115386962890625f,-1.02039997487366207451486843638122081756591796875f,0.3674000168800350962783340946771204471588134765625f,-0.0118000134050844085464859745115973055362701416015625f,-0.9623999601066106635016694781370460987091064453125f,1.27339995790123960972550776205025613307952880859375f,0.36530001056641403511804355730419047176837921142578125f,0.4095999956592919488684856332838535308837890625f,0.013400001934170735484741499021765775978565216064453125f,0.7173000199764967277360483421944081783294677734375f,0.196200001995265493892617314486415125429630279541015625f,1.3811000102698802738387939825770445168018341064453125f,0.086499997176229914686018673819489777088165283203125f,0.4659000002205366808283315549488179385662078857421875f,0.02499999910593031504646432949812151491641998291015625f,0.229000034987926870400087864254601299762725830078125f,-0.3041999949693678928497320157475769519805908203125f,-1.1178000258773561625957881915383040904998779296875f,-0.16960000765472660333443855051882565021514892578125f,0.536700011211633576380108934245072305202484130859375f,0.10790000106245278832517442424432374536991119384765625f,-0.074700013759732453166861887439154088497161865234375f,-0.22760001091063031264383198504219762980937957763671875f,-0.68990004105716995486119458291796036064624786376953125f,-0.08609999600201845293412361570517532527446746826171875f,-0.51350000144541263491504423654987476766109466552734375f,0.0811999979905783930700380324196885339915752410888671875f,1.2923000286005434933400692898430861532688140869140625f,0.3856000058963895416042078068130649626255035400390625f,1.831300032240152919627007577219046652317047119140625f,-0.047099999310076245961909080506302416324615478515625f,-0.03389995463788497698232049515354447066783905029296875f,0.246100009524822305184699189339880831539630889892578125f,0.522500022873283231916730073862709105014801025390625f,-0.16159999955594539500225437222979962825775146484375f,0.0317999919474127867857760065817274153232574462890625f,0.8949999907612811167467725681490264832973480224609375f,-0.56939996102303325198334960077772848308086395263671875f,0.23149998881295308361671914099133573472499847412109375f,-0.579999998733401245232244036742486059665679931640625f,0.8877999784171581865166444913484156131744384765625f,-0.8507999671816828257675524582737125456333160400390625f,0.6036999743908637583444942720234394073486328125f,-0.8883999947816130227096209637238644063472747802734375f,0.3154000223711130868053942322148941457271575927734375f,-0.097000040888785665771365529508329927921295166015625f,-0.454800031077862598749561584554612636566162109375f,-0.4177999933928244757908032624982297420501708984375f,-0.36090002406239563015333260409533977508544921875f,-0.326800002825260005323571022017858922481536865234375f,-0.972400037555396945521124507649801671504974365234375f,0.748600019583105957821089759818278253078460693359375f,-0.5649000167682765027876712338184006512165069580078125f,0.8629999980777498524986413031001575291156768798828125f,0.4099000011779365681974240942508913576602935791015625f,-0.31819999158680445372482381571899168193340301513671875f,1.34790003114417178409212283440865576267242431640625f,-1.90160001713931503530830013914965093135833740234375f,1.6321999978899963679168649832718074321746826171875f,-0.866399962610006735275192113476805388927459716796875f,0.706000067085027271929220660240389406681060791015625f,0.11039994350671822331833027419634163379669189453125f,0.8170999994963412671467040127026848495006561279296875f,-0.615799978357553623453668478759936988353729248046875f,1.011499982908369066336717878584749996662139892578125f,0.9968000381410127008763311096117831766605377197265625f,-0.2325000029429793446666963063762523233890533447265625f,-1.076900027072429377739126721280626952648162841796875f,-0.2631000015005469094830914400517940521240234375f,-0.861100029715895498583222433808259665966033935546875f,-0.24760000334829090906652027115342207252979278564453125f,-1.596100060114264973520903367898426949977874755859375f,-0.24490000589042904000081080084783025085926055908203125f,-0.19520000688284644407843870794749818742275238037109375f,0.14290000523477797145233125775121152400970458984375f,-0.78689997551441148715412055025808513164520263671875f,-0.3211000009194016069358212916995398700237274169921875f,-1.0523000383794303136397729758755303919315338134765625f,-0.11139999680817125327081384966732002794742584228515625f,-0.19859999015331253335858718855888582766056060791015625f,-0.0030000001639127749086810581502504646778106689453125f,-0.2628000173836948505368127371184527873992919921875f,-0.0214000080958009208931258626762428320944309234619140625f,0.4216999807387586951534785839612595736980438232421875f,0.19070000063925984790103029808960855007171630859375f,0.8419000012934201304659609377267770469188690185546875f,0.280299996782839233588902061455883085727691650390625f,0.3457000096172091474500120966695249080657958984375f,0.03490000011622906850305980697157792747020721435546875f,0.59879998597204708854491173042333684861660003662109375f,-0.085200000326335401101829347680904902517795562744140625f,0.4428000216305256753202002073521725833415985107421875f,0.277800008517503804750248264099354855716228485107421875f,1.6790000483244664764015396940521895885467529296875f,0.46240000913143131100468963268212974071502685546875f,-0.4192000094234937801473961371812038123607635498046875f,-0.71480002660751296161834034137427806854248046875f,0.270400014734268001603822995093651115894317626953125f,-0.8824000138998027154002556926570832729339599609375f,0.923600006619095470483671306283213198184967041015625f,-1.0323000298485160985961783808306790888309478759765625f,0.6680000082105397307685734631377272307872772216796875f,-0.2468999901548027953168684689444489777088165283203125f,-0.2118000181734560616320095505216158926486968994140625f,-1.12529995502233504112155060283839702606201171875f,1.73299995344877277858586239744909107685089111328125f,-0.45180001593828134076602509594522416591644287109375f,-0.2305999858915808164994132312131114304065704345703125f,0.471300002139806384349185464088805019855499267578125f,-0.35800000634789441722460878736455924808979034423828125f,-0.511700015234947347408933637780137360095977783203125f,0.1572000177830457090522031649015843868255615234375f,-0.824399994662403212686285769450478255748748779296875f,0.633800010931491186738639953546226024627685546875f,-0.12569997899532392438004535506479442119598388671875f,0.1111999808728698635462706079124473035335540771484375f,-0.5926999954462051078962758765555918216705322265625f,0.4149999954551457648221912677399814128875732421875f,0.19089996047169055515269064926542341709136962890625f,-0.74219995687454998201104672261863015592098236083984375f,0.82210004097744848738926748410449363291263580322265625f,-0.4750000256299971024276374009787105023860931396484375f,1.22020002480149347690030481317080557346343994140625f,-0.7659000273406508796369962510652840137481689453125f,-0.29810001073926695180915658056619577109813690185546875f,-0.4685000210404404175079662309144623577594757080078125f,0.078399998432397810521621295265504159033298492431640625f,0.312300004479288961789507084176875650882720947265625f,0.0034999996945261901970525286742486059665679931640625f,-0.9101000195533039960338328455691225826740264892578125f,-0.2530000103339553607639800247852690517902374267578125f,0.523099933663009775131058631814084947109222412109375f,0.288199992518126890672647277824580669403076171875f,0.3174999646842469047669510473497211933135986328125f,-0.319600006723403939901118064881302416324615478515625f,-0.0188000393927101328017670311965048313140869140625f,0.012599998691678049311803988530300557613372802734375f,0.7263999884635203851956930520827881991863250732421875f,0.25029999696910376627556615858338773250579833984375f,1.440400020992755880655522560118697583675384521484375f,0.20810000649243598491722195831243880093097686767578125f,0.71890004411935848338544019497931003570556640625f,0.080400006687641190072923791376524604856967926025390625f,0.684000019341707510278638437739573419094085693359375f,0.20479999829530715071967961193877272307872772216796875f,0.27659999389350364840112206366029568016529083251953125f,0.069599997875839481054072166443802416324615478515625f,-0.443300023116916686038990746965282596647739410400390625f,-0.36130000456124544694347378026577644050121307373046875f,-0.8045000389516354655228269621147774159908294677734375f,0.232300001461803906632752614314085803925991058349609375f,-0.50080001613199787158237086259759962558746337890625f,0.011200031052529535458006648696027696132659912109375f,-0.59990001740157594678493069295655004680156707763671875f,0.26510000223070384617329864340717904269695281982421875f,-0.625999993920326236462869928800500929355621337890625f,0.3094000057265162695330218411982059478759765625f,-0.090099992805719342214842981775291264057159423828125f,0.091800005321204469765916655887849628925323486328125f,0.80979998583421120006420323989004828035831451416015625f,-0.07539997546970889086281886193319223821163177490234375f,-0.56489998379945749462649473571218550205230712890625f,-0.1038999962493780682137867188430391252040863037109375f,-0.74010000934600839173072017729282379150390625f,-0.27210000527054056806974813298438675701618194580078125f,-0.08979999285638340378312705070129595696926116943359375f,0.6871000027000906396068558024126105010509490966796875f,0.6336000160813330950304589350707828998565673828125f,1.06579999005496528496905739302746951580047607421875f,-0.091199986982345659924931169371120631694793701171875f,0.456899983672797826983469349215738475322723388671875f,0.0005000002570450245986677373366546817123889923095703125f,-0.218499985985458111681367654455243609845638275146484375f,0.007700000065565149043322890065610408782958984375f,-1.187599953317642675898468951345421373844146728515625f,1.06639998779296885089706847793422639369964599609375f,-0.8840999575346712990864261882961727678775787353515625f,0.002099994206428590359791996888816356658935546875f,-0.131199966567754733315354087608284316956996917724609375f,0.78750000894069671630859375f,0.725099997536837737044379537110216915607452392578125f,0.06120000404417513806265560560859739780426025390625f,-0.03990002060532571448447924922220408916473388671875f,-0.0389999746382232448382865186431445181369781494140625f,1.561500005483626551949782879091799259185791015625f,0.125599981272220162509256624616682529449462890625f,0.78119997744560265573454671539366245269775390625f,0.0357000201225279312211569049395620822906494140625f,-0.902200029778480683262387174181640148162841796875f,-0.89560000991970323269697473733685910701751708984375f,0.589499979570507814941038304823450744152069091796875f,-0.60319999769628063202020484823151491582393646240234375f,0.35000002384185791015625f,0.69910002183318198376582586206495761871337890625f,-0.1560000317692757931808955618180334568023681640625f,0.849499995589256595707183805643580853939056396484375f,-1.04129999752044710703557939268648624420166015625f,0.18860004044473210882415514788590371608734130859375f,1.52349995815753924688351617078296840190887451171875f,0.557199992078542205575786283588968217372894287109375f,-0.0723000280976293652202002704143524169921875f,0.13280001026839016731173614971339702606201171875f,-0.80339998647421584365702074137516319751739501953125f,1.097500024847686272977398402872495353221893310546875f,0.3039000161051745863005635328590869903564453125f,-0.8016000084221364119230202049948275089263916015625f,0.5210000225305559951038958388380706310272216796875f,0.5753999822497366523066375521011650562286376953125f,0.92159999217987120800898992456495761871337890625f,0.4842000189602373438901850022375583648681640625f,-0.101900013624131258183069803635589778423309326171875f,-0.23500000558793543437019479824812151491641998291015625f,-0.339799990732967405193676313501782715320587158203125f,0.72359999268650998516250183456577360630035400390625f,-0.90909999676197739137961661981535144150257110595703125f,0.42889998853802691058945129043422639369964599609375f,0.3705000139474865505917478003539144992828369140625f,-0.60850001865625369390500054578296840190887451171875f,-0.02680001705586920746071655230480246245861053466796875f,0.579499974913895155026466454728506505489349365234375f,0.993199971656501201522360133822076022624969482421875f,0.395900013780593695855714031495153903961181640625f,1.18449997856467970080984741798602044582366943359375f,-0.4727000085592270028911343615618534386157989501953125f,-1.275700026604532855145635039662010967731475830078125f,-0.92419998915791534699337717029266059398651123046875f,-1.416099979312717582757841228158213198184967041015625f,0.787799969848990588872084117610938847064971923828125f,-1.607899991711974507779814302921295166015625f,0.073999981492757882506339228712022304534912109375f,-0.6225999846845866603217700685490854084491729736328125f,-0.20189999846220008095087905530817806720733642578125f,-0.45580000227391703759849406196735799312591552734375f,0.108600004607438904713490046560764312744140625f,-0.3965999989971515926612255498184822499752044677734375f,0.625900007969141025654380428022705018520355224609375f,-0.256899978159368325147937639485462568700313568115234375f,0.6481999982178212604821965214796364307403564453125f,-0.8649999683722857657386384744313545525074005126953125f,0.41679996140003172655497110099531710147857666015625f,-0.8031999789953232493644463829696178436279296875f,-0.9196999668955800899539099191315472126007080078125f,-0.833999967634678540662207524292171001434326171875f,1.4634000510334974620718639926053583621978759765625f,1.23160000350475229424773715436458587646484375f,-0.248100031191111014550187974236905574798583984375f,-0.2656000112235545884686871431767940521240234375f,0.3555999849975108872968121431767940521240234375f,1.927399997806549691858890582807362079620361328125f,0.0787000196158889053776874789036810398101806640625f,-0.7854000129461287116328094271011650562286376953125f,0.9148000601165005019055342927458696067333221435546875f,0.6909000102788203090398155836737714707851409912109375f,-0.77300000973790894409631846428965218365192413330078125f,0.24329999594092388548460803576745092868804931640625f,1.6436000586241490406536058799247257411479949951171875f,0.719899989834427600499111576937139034271240234375f,0.563100008913874461313753272406756877899169921875f,-1.252500057220458984375f,0.190900003163516629456353257410228252410888671875f,1.28519997745305314396091489470563828945159912109375f,-0.224199999029934371463923525880090892314910888671875f,0.460900000369548479284276254475116729736328125f,-0.0766999922312796567069881348288618028163909912109375f,1.0923000103652478198767994399531744420528411865234375f,0.41650001222640298248478529785643331706523895263671875f,0.5968000418663024220222723670303821563720703125f,0.31220002974122795791345197358168661594390869140625f,-0.40050000735372304205839100177399814128875732421875f,-0.63709998183846483499337409739382565021514892578125f,-0.4969000286370512942113464305293746292591094970703125f,-0.1252999976396562686886682058684527873992919921875f,0.76409998119026523166752440374693833291530609130859375f,0.234400000101327776746984454803168773651123046875f,-1.211000015713274091666562526370398700237274169921875f,-0.19470000429749489967434783466160297393798828125f,-0.47099998125433939488715395782492123544216156005859375f,0.966200013655424072567257098853588104248046875f,-0.92099999552220168652638676576316356658935546875f,-0.3499000040650368514860701907309703528881072998046875f,-0.35020000624954661372356667925487272441387176513671875f,0.20350000135228019093602824796107597649097442626953125f,-1.2344000395148990723015458570444025099277496337890625f,0.55859999156892292671017230532015673816204071044921875f,0.0610000068843365994553096243180334568023681640625f,0.1371999977409839033271055086515843868255615234375f,-0.949300012362003453603165326057933270931243896484375f,-0.9067000276029109073760992032475769519805908203125f,0.020100021453201666332688546390272676944732666015625f,0.423599995964765607681101755588315427303314208984375f,-1.1594999897032975955113442978472448885440826416015625f,0.32129999260306352226734816213138401508331298828125f,-0.73949999938905219831752901882282458245754241943359375f,0.85530000030323860560343973702401854097843170166015625f,-0.6939000087291000706812837961479090154170989990234375f,0.5542000211954114519130598637275397777557373046875f,-1.0968000257357954918546738554141484200954437255859375f,-0.2479999749064436542766998172737658023834228515625f,2.23960004785060817766861873678863048553466796875f,-0.1176000308036808661427130573429167270660400390625f,-0.23949998784065229529005591757595539093017578125f,-1.237700012731551169054000638425350189208984375f,0.50750002980232267191240680404007434844970703125f,1.90790006033182191202968169818632304668426513671875f,0.8125999855041499131402815692126750946044921875f,-1.0993000448465348739546243450604379177093505859375f,1.42110002293586745736320153810083866119384765625f,-0.304200015085935859815435833297669887542724609375f,-0.6435999841928481401964745600707828998565673828125f,0.041000000625848631585768089280463755130767822265625f,-0.372199995654820980917065753601491451263427734375f,0.33239998514056168232855270616710186004638671875f,-0.312100011491775575223073246888816356658935546875f,-1.19860002204179760809665822307579219341278076171875f,-0.496599992775916376785971806384623050689697265625f,1.12600001649558567606845826958306133747100830078125f,-0.78879998649358729068126194761134684085845947265625f,-0.53340001392960534332132738200016319751739501953125f,-0.6577999988317486668165656737983226776123046875f,0.8779999954700468123292012023739516735076904296875f,0.230999939680099686256653512828052043914794921875f,0.3309999811053270235561285517178475856781005859375f,-0.6768000400781630787605536170303821563720703125f,-1.866700037288666180756990797817707061767578125f,-0.45690000412464115697730449028313159942626953125f,1.530600001221149586427827671286650002002716064453125f,1.03690002211779397356394838425330817699432373046875f,1.14330001903772338067710734321735799312591552734375f,0.43189996415227671633374484372325241565704345703125f,-0.33809997531175639551292988471686840057373046875f,-0.2143000139638779888429098718916065990924835205078125f,0.608100014799833221701419461169280111789703369140625f,0.54400002567470095993229506348143331706523895263671875f,-0.14159999992847449590271935448981821537017822265625f,-0.7692999734699721425812413144740276038646697998046875f,0.7840000141263008259784328402020037174224853515625f,-0.8909999969750643789012656270642764866352081298828125f,-0.0278999992370605465197286321199499070644378662109375f,1.0632000206440685996511774646933190524578094482421875f,-0.15279999510645847493606197531335055828094482421875f,0.069799989317357091778148969751782715320587158203125f,-0.253700003373622973157353044371120631694793701171875f,-0.5680000110045078098863768900628201663494110107421875f,0.27729999993443499306522426195442676544189453125f,1.013400001040100750770989179727621376514434814453125f,-0.5677999820679426878911044695996679365634918212890625f,0.5974000067099927147040716590709052979946136474609375f,-0.40659999322295181656272688996978104114532470703125f,0.2886000313177701226408089496544562280178070068359375f,-0.3198000249683854434579188819043338298797607421875f,-0.66709996555894657266350122881704010069370269775390625f,-0.755000014007091380108249722979962825775146484375f,-0.675799984765053096680276212282478809356689453125f,0.30079998014569309106036598677746951580047607421875f,0.22659998333230568778873248447780497372150421142578125f,-0.481500010885298035390178483794443309307098388671875f,1.834299963784218334694742225110530853271484375f,0.6653000114977363210755356703884899616241455078125f,0.551399984365701811839244328439235687255859375f,-0.97950003403425256465197890065610408782958984375f,1.9441000175118450243871848215349018573760986328125f,-0.047300007832050017242409012396819889545440673828125f,-0.579599998974799746065400540828704833984375f,-0.95260001187920551046772743575274944305419921875f,0.21759998982548722779029048979282379150390625f,-1.20210002058148379688873319537378847599029541015625f,0.43759999980926522766822017729282379150390625f,0.78269997547864900155900613754056394100189208984375f,0.07349997395277085843190434388816356658935546875f,1.3691000207900998475452070124447345733642578125f,0.634399930810928935898118652403354644775390625f,-1.7018000366508960752298662555404007434844970703125f,-1.594699991780519354733769432641565799713134765625f,1.1265000336468222741359568317420780658721923828125f,-0.628700022447109319045921438373625278472900390625f,-0.470800024253130278140133668784983456134796142578125f,0.6387999921560290061961495666764676570892333984375f,1.793300042022764717586369442869909107685089111328125f,-0.383300056588649962208137367269955575466156005859375f,-0.5385000082999471526790102871018461883068084716796875f,1.423699979829788020424530259333550930023193359375f,-0.800899992620944800592042156495153903961181640625f,-0.7412999574363237087482048082165420055389404296875f,-0.10730002496838597636497070197947323322296142578125f,-0.02819999354258218016155979057657532393932342529296875f,1.01439998713731771573520745732821524143218994140625f,-0.1595999521106483687304944396601058542728424072265625f,0.033399997836351502655816148035228252410888671875f,-0.03699997595697668284486780976294539868831634521484375f,1.1426000175416468618294629777665250003337860107421875f,0.9842000127762553507437814914737828075885772705078125f,-0.523599984937906270943130948580801486968994140625f,1.128500003941356855108324452885426580905914306640625f,0.006799999547004720312770587042905390262603759765625f,0.6043999925762413294449970635469071567058563232421875f,0.424299996268749257666286212042905390262603759765625f,-0.51679999608248505804652950246236287057399749755859375f,0.0359999934732915249924189993180334568023681640625f,0.13250005614012394516976200975477695465087890625f,-0.23280001567006092244582760031335055828094482421875f,-0.9137000031873590000230933583225123584270477294921875f,-0.53350000062584879145788363530300557613372802734375f,0.621699993404746020786433291505090892314910888671875f,-0.5541000106573104044827005054685287177562713623046875f,-0.618499997235834708675383808440528810024261474609375f,0.49469999476075177113898462266661226749420166015625f,-0.193999971769750256811448707594536244869232177734375f,-0.7242000259265302108957484961138106882572174072265625f,-0.896699983979761583441359107382595539093017578125f,0.706099985980987554512466886080801486968994140625f,-0.26250001329928596138785223956801928579807281494140625f,-0.12669999079704297173520899377763271331787109375f,0.7693999497354031547047270578332245349884033203125f,0.9825000223517417197172107989899814128875732421875f,-0.787200027990341055783574120141565799713134765625f,-0.050799996536969871385736041702330112457275390625f,0.5938000130176543933657740126363933086395263671875f,-1.27310001435279840364955816767178475856781005859375f,1.02160001602172911816523992456495761871337890625f,0.840500028848648383927866234444081783294677734375f,1.39689997938871357519019511528313159942626953125f,0.9962999796390530349299297085963189601898193359375f,-0.2377000270366664835819392465054988861083984375f,-1.4714999774694437206790098571218550205230712890625f,0.5669000255823135603350237943232059478759765625f,-0.596299982619285628970828838646411895751953125f,1.360699980187416002763711730949580669403076171875f,-0.39999999031424504636333949747495353221893310546875f,1.0977999869108199249012614018283784389495849609375f,-0.101800014150142725810610500047914683818817138671875f,-1.926300010633468673404422588646411895751953125f,-0.6153999959588052348635756061412394046783447265625f,0.4219999923110009376614470966160297393798828125f,-1.45789998522996899765757916611619293689727783203125f,0.85110003068447070972979418002068996429443359375f,0.7471000262439257966207151184789836406707763671875f,1.411699972468614561194044654257595539093017578125f,-0.5978999798655504349653710960410535335540771484375f,0.4304999752044675886963887023739516735076904296875f,-0.11399999982118647068318750825710594654083251953125f,0.417700022864341491413142648525536060333251953125f,-0.20170000957250611151039265678264200687408447265625f,-0.6673999854385856966842993642785586416721343994140625f,-0.6206000079452991258222027681767940521240234375f,-0.70220000053495168490513833603472448885440826416015625f,0.354400008594989657240148517303168773651123046875f,0.5884000105395912338934749641339294612407684326171875f,0.894900010770559362072162912227213382720947265625f,0.6343000213772056472549820682615973055362701416015625f,0.54659998815655708170879734097979962825775146484375f,-1.4883000046953560602247534916386939585208892822265625f,-0.37869999920129782111644090036861598491668701171875f,-0.2832999804809690491680385093786753714084625244140625f,0.71030000434517859275729279033839702606201171875f,1.3474999817460773243738003657199442386627197265625f,0.975699980783462539335459950962103903293609619140625f,1.03679999474138018200619626441039144992828369140625f,0.6534999888539314572000193948042578995227813720703125f,-0.0790000253766776072694710819632746279239654541015625f,-0.94270001094341271397070158855058252811431884765625f,0.6714999928548930352434354063007049262523651123046875f,0.452899989402294256279901674133725464344024658203125f,1.295899958609044499979745523887686431407928466796875f,0.88380000586509677162894149660132825374603271484375f,1.13299995575845269257797554018907248973846435546875f,0.54669999377727496181478272774256765842437744140625f,-0.850899983568489393093159378622658550739288330078125f,0.41929997153282183575129238306544721126556396484375f,0.4702000058770181567524559795856475830078125f,0.311000006586313215706240953295491635799407958984375f,-0.51989999992996427380376189830712974071502685546875f,-0.97350002789497391830764172482304275035858154296875f,-0.317299969387054403568981797434389591217041015625f,2.05730007429122974116353361750952899456024169921875f,-0.084200019407273174465444753877818584442138671875f,0.8036999920487399862167876563034951686859130859375f,-1.3060000207424167228964506648480892181396484375f,-0.240400034701824694138849736191332340240478515625f,0.87190001008510620295055559836328029632568359375f,-0.106299961459637160032798419706523418426513671875f,0.732899975395202574190989253111183643341064453125f,1.2152999794602390437603389727883040904998779296875f,0.734399954652786846054368652403354644775390625f,-1.432300016772746431570340064354240894317626953125f,-1.6716999928832052546567865647375583648681640625f,0.4105000383853916190446398104541003704071044921875f,-1.03320002012252842860107193700969219207763671875f,0.17119999055862411552197954733856022357940673828125f,-0.770700021314620897783242980949580669403076171875f,-1.934000029623508254417174612171947956085205078125f,0.40230002601146708229862269945442676544189453125f,0.060300004121661032741030794568359851837158203125f,-0.2073999959439039031394713674671947956085205078125f,-1.3876999678045507380375056527554988861083984375f,-0.38429998285770494703683652915060520172119140625f,-0.0622000223279002284471062012016773223876953125f,-0.085499989509582263735865126363933086395263671875f,2.12280004204511651977327346685342490673065185546875f,-0.698400049543380418981541879475116729736328125f,0.911600028240680870794676593504846096038818359375f,-0.350400026990473179022700378482113592326641082763671875f,0.224300009232759389732336785527877509593963623046875f,-0.0202999976396556203184218247770331799983978271484375f,-0.4394000167906284559649066068232059478759765625f,-1.4024000050708647080455193645320832729339599609375f,-1.0203000123918055663807535893283784389495849609375f,-1.68619997997879966789014360983856022357940673828125f,0.7326999996185303842821667785756289958953857421875f,0.0467000274166460282998514230712316930294036865234375f,0.643099975162744641465906170196831226348876953125f,1.090399992680548990620081895031034946441650390625f,-0.1497999968051910713029428734444081783294677734375f,-0.658200004699826646259452900267206132411956787109375f,-0.71999999597668651318826960050500929355621337890625f,0.582699975664913605299943810678087174892425537109375f,-0.619800016623735405829620503936894237995147705078125f,0.78249997567385431995745648237061686813831329345703125f,-0.70050000157952307944242420489899814128875732421875f,0.28549999628961086894918253165087662637233734130859375f,0.6384000039458275210790816345252096652984619140625f,1.1813999848127363367922271208954043686389923095703125f,-0.043999997586011918571102796704508364200592041015625f,0.51099996773153566476111109295743517577648162841796875f,0.2743999823689460271225470933131873607635498046875f,-0.0733000618785617508166296829585917294025421142578125f,0.479999996274709683774517543497495353221893310546875f,0.01910000272244172236923986929468810558319091796875f,-0.6809000157177447221101829200051724910736083984375f,-0.806799966615438624017997426562942564487457275390625f,-0.05199999037384994693411499611102044582366943359375f,-0.474400002187490343885656329803168773651123046875f,-1.401900020478665798151496346690692007541656494140625f,0.183300015461445031661469329264946281909942626953125f,-0.2618000255495311900943988803192041814327239990234375f,-0.09019999663829736391562619246542453765869140625f,0.441599997507035713084633243852294981479644775390625f,-0.243599997305870008545980454073287546634674072265625f,0.3996000380903483861771974261500872671604156494140625f,1.7926999671339984843143611215054988861083984375f,0.1258999862320724361097745713777840137481689453125f,1.440699991195649065645056907669641077518463134765625f,-0.6017000279568140985730906322714872658252716064453125f,-0.87159999098777785064839918049983680248260498046875f,-1.381100026363134336548910141573287546634674072265625f,0.0095000504255295936673064716160297393798828125f,0.9238000403612858235646854154765605926513671875f,-0.61949998739361777921885732212103903293609619140625f,0.2878000018864865783285722500295378267765045166015625f,-0.45520002703666673227189676254056394100189208984375f,0.3228000566482547295521499108872376382350921630859375f,1.923299950194358842736619408242404460906982421875f,0.46699999886751175637300548260100185871124267578125f,0.51279994280338314638356678187847137451171875f,0.880000087916851470026813331060111522674560546875f,1.9683999863624563886332907713949680328369140625f,0.41029997718781174853575066663324832916259765625f,-0.66090000480264432525245865690521895885467529296875f,0.441200036416947938278099172748625278472900390625f,0.070900003051757753969042141761747188866138458251953125f,-0.207000000059604616353681194595992565155029296875f,0.650299986761808224144942869315855205059051513671875f,0.5357999991074204881869036398711614310741424560546875f,1.303500004798173517173154323245398700237274169921875f,0.13450000661611571928233388462103903293609619140625f,0.5840000123009081445246692965156398713588714599609375f,-0.270799992699921165950627255369909107685089111328125f,-0.070299996745586401658556496840901672840118408203125f,0.68709998466968524866160805686376988887786865234375f,-0.2343999571673567317020570044405758380889892578125f,0.4379000029623509870901898466399870812892913818359375f,-0.2973999860532583472405576685559935867786407470703125f,-0.535199994328618089411975233815610408782958984375f,-0.1537000381305813068166798984748311340808868408203125f,0.54780002073645572835403072531335055828094482421875f,0.3672999923720958737050068521057255566120147705078125f,-0.25319999918639657376928653320646844804286956787109375f,-0.79370003168582947683518113990430720150470733642578125f,-0.04570001751482488661082470571273006498813629150390625f,0.2488000054553147766256415707175619900226593017578125f,1.042699970859289226865485034068115055561065673828125f,-1.423300003316998907365587001550011336803436279296875f,0.3934999654591084805588252493180334568023681640625f,-0.8379999572485683501099629211239516735076904296875f,1.0167999746620655088236162555404007434844970703125f,-1.3161000285983082136453958810307085514068603515625f,-0.2840000054836273601921448062057606875896453857421875f,-0.6211999796807765594763850458548404276371002197265625f,-0.9002999848432835516831573841045610606670379638671875f,-0.6040999666988853267213244180311448872089385986328125f,0.884400048865378263229786170995794236660003662109375f,0.80369998698234468292866949923336505889892578125f,-0.758500032365322596206169691868126392364501953125f,0.27170001583099434583346010185778141021728515625f,2.4673000182628630483350207214243710041046142578125f,-0.83470003871917697324533946812152862548828125f,-1.855700007307529375566446105949580669403076171875f,0.62130008573532080617951578460633754730224609375f,0.1232000239968300547843682579696178436279296875f,-0.6826999739885337703526602126657962799072265625f,-0.454200017619132978552443091757595539093017578125f,1.418800014209747217819312936626374721527099609375f,0.84600002573430543861832120455801486968994140625f,-1.13930003545880342841201127157546579837799072265625f,-0.5841000018656254955118356519960798323154449462890625f,0.01200001138448669735225848853588104248046875f,-0.193500029869377865310298147960565984249114990234375f,-0.341500017404556377442759185214526951313018798828125f,0.5606000214308501927717998114530928432941436767578125f,1.14460000806450779009537654928863048553466796875f,-1.29030001000761984641940216533839702606201171875f,-0.6149999700486663556375788175500929355621337890625f,0.37570000760555277707908317097462713718414306640625f,0.781699969786405546301466529257595539093017578125f,-0.946499991923570149765509995631873607635498046875f,-0.83279998750686701214362983591854572296142578125f,-1.3765000671744349602931833942420780658721923828125f,-0.0634000264838332494576889075688086450099945068359375f,-0.01659999421015390108635756405419670045375823974609375f,-0.460200034263729518269059326485148631036281585693359375f,0.52459997289776794104909640736877918243408203125f,-0.7457000087603924409762612413032911717891693115234375f,0.69230000365972532705427511245943605899810791015625f,-0.099499974839389881964279993553645908832550048828125f,-0.8515000095069407137771122506819665431976318359375f,-0.292999967157840490727949145366437733173370361328125f,0.7657000354707239608842428424395620822906494140625f,1.5367999654978519163961436788667924702167510986328125f,-0.412199994164705429966488736681640148162841796875f,0.2408999717220670078177136019803583621978759765625f,0.84690000546574584205927749280817806720733642578125f,-0.08999999701976768573530307548935525119304656982421875f,-0.88050002013146866364223797063459642231464385986328125f,0.826399999900162551824678303091786801815032958984375f,-0.1983999951541424167089644470252096652984619140625f,0.339999982379377119201535606407560408115386962890625f,-0.395600006939470727473917577299289405345916748046875f,-0.25899998983740768476735638614627532660961151123046875f,0.26330000517964347039878703071735799312591552734375f,-0.16220002553164902803217728433082811534404754638671875f,-0.170299990189075600710566504858434200286865234375f,1.2652999723076820970391054288484156131744384765625f,0.11999999269843097948751164949499070644378662109375f,1.0195000003948810896048371432698331773281097412109375f,-0.66139999598860743645900583942420780658721923828125f,1.164099990727007760682454318157397210597991943359375f,1.08159997489452308627733145840466022491455078125f,-0.392900046324729856905833003111183643341064453125f,-0.6209000203371051185285978135652840137481689453125f,-1.45769998829364766379512730054557323455810546875f,1.1143000381410121946146318805404007434844970703125f,-1.6413000532507897588629930396564304828643798828125f,-0.918300019931792999017261536209844052791595458984375f,-1.71169995443820965874692774377763271331787109375f,1.1749999919533724579423505929298698902130126953125f,0.221799978089332938679945073090493679046630859375f,-0.52080000905394552290772480773739516735076904296875f,0.587799998086691033449824317358434200286865234375f,-0.14949999171495420569044654257595539093017578125f,-1.5003999712228779372935605351813137531280517578125f,-0.02530001403093340428540614084340631961822509765625f,-0.449800021839141805912731797434389591217041015625f,-1.2278000140123064287678289474570192396640777587890625f,0.3971000006139278060146580173750407993793487548828125f,1.34420003021061429393512298702262341976165771484375f,-0.023099989020824995833436332759447395801544189453125f,-1.3500000221282244527998273042612709105014801025390625f,-0.1344000118732449067238121642731130123138427734375f,0.54910000112056689403061682241968810558319091796875f,0.67459991493225146541590220294892787933349609375f,0.1574000305891036788352721487171947956085205078125f,-0.446000021934509049970074556767940521240234375f,0.311200039546192019912496107281185686588287353515625f,-0.342599960172176754014117250335402786731719970703125f,0.1643999548763035445375635390519164502620697021484375f,-0.064400020366906840507681408780626952648162841796875f,-0.5757000192284582595902975299395620822906494140625f,-1.22300000325590385585883268504403531551361083984375f,-0.55449999737739563698823985760100185871124267578125f,0.204099973814189095122628714307211339473724365234375f,-1.0753000286340712587929147048271261155605316162109375f,-0.282900015926360826057361919083632528781890869140625f,-0.60329998625516889632081074523739516735076904296875f,0.792499997802078581798923551104962825775146484375f,-0.168699998903274472894509017351083457469940185546875f,-0.2890999726966025917818114976398646831512451171875f,0.2994000012934208143633441068232059478759765625f,0.6541999903872610655497510379063896834850311279296875f,-0.94549998384714140087226041941903531551361083984375f,1.287499958947301070821822577272541821002960205078125f,-0.640699996578693475868249151972122490406036376953125f,-0.17010000632852306789999374814215116202831268310546875f,0.35769999494329096645373056162497960031032562255859375f,-0.100600006231665262124153059630771167576313018798828125f,-0.3815999895349146964207420751336030662059783935546875f,-0.265400008326768688249330807593651115894317626953125f,0.162499990165233754169094027020037174224853515625f,-0.7135000058412561685372565989382565021514892578125f,0.56209994890689873869860093691386282444000244140625f,-1.32139999073594882617044277139939367771148681640625f,0.21499997645616542030211348901502788066864013671875f,0.49479998533129698756738434894941747188568115234375f,0.0306000131234525074575003600330092012882232666015625f,0.817499974220991187934259869507513940334320068359375f,-1.16730002779960617687038393341936171054840087890625f,0.4132000214636324386674459674395620822906494140625f,1.070899998134374531133516939007677137851715087890625f,-0.85050000083446519738572533242404460906982421875f,-0.7270999600268890361576268333010375499725341796875f,0.046500009898096106297771257231943309307098388671875f,0.2586000248730186967804911546409130096435546875f,0.8040000033974641358014423531130887567996978759765625f,0.35990002250522412641231539964792318642139434814453125f,1.7986000437974922760986373759806156158447265625f,-0.45030002446174677288581733591854572296142578125f,-0.089100030624866377593207289464771747589111328125f,0.56210006573200299584414096898399293422698974609375f,-1.733499996304512791311935870908200740814208984375f,0.70279996366500796511900261975824832916259765625f,0.6235999487280849962189677171409130096435546875f,-0.5771000064253808403691436978988349437713623046875f,0.364399991607665896253820392303168773651123046875f,-0.1148000017412005302475108692306093871593475341796875f,0.9825000301003459224347125200438313186168670654296875f,0.17370001890808384192865787554183043539524078369140625f,0.881200027513504124954124563373625278472900390625f,0.49750001430511492372943394002504646778106689453125f,0.73350003802776342354263761080801486968994140625f,0.194000051155686747961226501502096652984619140625f,1.18720000519156432261524969362653791904449462890625f,-0.7539000579774388643983229485456831753253936767578125f,-0.37570001505613248582449159584939479827880859375f,0.2482000177383429928568148170597851276397705078125f,0.576999983407557426318135185283608734607696533203125f,0.2775000084936618538478114714962430298328399658203125f,-0.3629000478520989503294913447462022304534912109375f,-0.73459998735189468987982763792388141155242919921875f,-0.05240001611635047851223134784959256649017333984375f,0.0837000029265879685880236138473264873027801513671875f,0.13049997924640821533870393977849744260311126708984375f,0.34649999507144080279630315999384038150310516357421875f,-0.0591999840170134294936588048585690557956695556640625f,0.715900021529197516656495281495153903961181640625f,-1.22690003515630952080073257093317806720733642578125f,0.130999980807304350349795640795491635799407958984375f,0.23269998918771772622449134360067546367645263671875f,-1.01810000973343850461105830618180334568023681640625f,0.54409999031722555695722576274420134723186492919921875f,-0.10070001311898213458562167943455278873443603515625f,-1.3562999838858846857192474999465048313140869140625f,0.319299990308284709072950136032886803150177001953125f,-0.55690000252276650183347328493255190551280975341796875f,0.816700017526000632273053270182572305202484130859375f,-0.4858999885976313937163695300114341080188751220703125f,0.769200010168552328337909784750081598758697509765625f,-0.9465999996304510499811613044585101306438446044921875f,0.69580001739859564935386515571735799312591552734375f,0.22680003296285899949680242571048438549041748046875f,0.76249999158084402761659248426440171897411346435546875f,1.3793999905273313100195764491218142211437225341796875f,-0.18899999400973310770268653868697583675384521484375f,1.2486000392153859817057082182145677506923675537109375f,-2.0335999919414522452143501141108572483062744140625f,0.6523000039577482311869971454143524169921875f,1.279899997508525899547748849727213382720947265625f,0.169699983286857758457699674181640148162841796875f,-0.840400004564224811787198632373474538326263427734375f,-0.810499980866909108812023987411521375179290771484375f,-0.8301999785333868953074443197692744433879852294921875f,-0.078899994945526685796721721999347209930419921875f,1.130299988251923792859088280238211154937744140625f,-0.2806000137567519203685151296667754650115966796875f,0.436200006726384259536644094623625278472900390625f,0.9017999857634304561315730097703635692596435546875f,-0.93180000632703308127702257479541003704071044921875f,-1.20859997227191939828117028810083866119384765625f,0.9421000341415410872514257789589464664459228515625f,1.47330001814365374457338475622236728668212890625f,-0.409800000642240203063693115836940705776214599609375f,0.808699998456239743660489693866111338138580322265625f,0.885999991834163669324198053800500929355621337890625f,-0.89409998100399956655337518895976245403289794921875f,0.336299997427314423248390085063874721527099609375f,1.0752000078141692451794142471044324338436126708984375f,-0.28239997228831026365725165305775590240955352783203125f,0.7380999964714050065595074556767940521240234375f,0.7253999936491251077086417353712022304534912109375f,-0.80939999473690971143469141679815948009490966796875f,0.35100002683326625430026979302056133747100830078125f,-1.2250999892294409665538523768191225826740264892578125f,-1.850300027702748817404199144220910966396331787109375f,-0.2839999945685269810979889371083118021488189697265625f,0.7967999870300292997171709430404007434844970703125f,1.523799962055683021588947667623870074748992919921875f,-0.392099986755848117780942629906348884105682373046875f,-0.2300000051036474868482173405936919152736663818359375f,-0.5221999854847789723777395920478738844394683837890625f,-0.214600003594159705055943732077139429748058319091796875f,0.2815999790668488600431373924948275089263916015625f,0.44009998498260982646712591304094530642032623291015625f,-0.381599999779462795146400821977294981479644775390625f,-0.5824999821931122756524246142362244427204132080078125f,0.24579999564290044844483418273739516735076904296875f,-1.124700026760995452690394813544116914272308349609375f,-0.25519999395608916614719419158063828945159912109375f,0.418899998149276253656125845736823976039886474609375f,0.73840001020431511591368689551018178462982177734375f,-0.6157999881178140011428467914811335504055023193359375f,0.63859999317079785274842151920893229544162750244140625f,-0.4423999961674212055839916502009145915508270263671875f,-0.53499998487532141044908939875313080847263336181640625f,-0.68159998607039462559242792849545367062091827392578125f,0.59820001131221645085389582163770683109760284423828125f,-0.4431999811559916846448459182283841073513031005859375f,0.59469999163150788490384002216160297393798828125f,-0.4237000103399157779904271592386066913604736328125f,-0.43609999350607397960999378483393229544162750244140625f,0.21689999760538347661764646545634604990482330322265625f,-0.81049999288097029914723634647089056670665740966796875f,1.186999973163008537113682905328460037708282470703125f,-0.74269995223283746810238881153054535388946533203125f,0.45450003671646044267617980949580669403076171875f,0.92630000348091101614045328460633754730224609375f,0.367099996590614097158322692848742008209228515625f,-0.915300007511675506322035289485938847064971923828125f,1.345099992954730705463362028240226209163665771484375f,-0.4918000012606384974134243748267181217670440673828125f,0.484099996352195915960692218504846096038818359375f,-0.2987000224471092479916478623636066913604736328125f,-0.9566999669551847773618646897375583648681640625f,-0.7444000109791755193100470933131873607635498046875f,-0.63469998328685761634915252216160297393798828125f,0.47460002482831509240668310667388141155242919921875f,-1.72379996592998541160568493069149553775787353515625f,0.745500016629696116154946139431558549404144287109375f,0.027999996215104516750216134823858737945556640625f,-0.7142000210464001241916776052676141262054443359375f,-0.4537000000953668177317013032734394073486328125f,-0.95890000209808334830086096189916133880615234375f,0.214300030541419772589506465010344982147216796875f,1.302399998402595571178608224727213382720947265625f,-0.71010002335309962973042274825274944305419921875f,0.086600021386146863733301870524883270263671875f,-0.903099984252452969712976482696831226348876953125f,-0.175100004465878367199138665455393493175506591796875f,-0.604199993404745949732159715495072305202484130859375f,-0.6359000162392856037030242077889852225780487060546875f,0.1286999961465600250676288851536810398101806640625f,-0.29399998514354219736333106993697583675384521484375f,-0.0149999974668022506563147544511593878269195556640625f,0.7024999950826167349759998614899814128875732421875f,1.3417999801009898863668468038667924702167510986328125f,-0.083299998325109481100980701739899814128875732421875f,-0.591199999350309202128528340836055576801300048828125f,0.9324999835714697926647431813762523233890533447265625f,0.287800000470876671254671919086831621825695037841796875f,0.5257999920666218685028070467524230480194091796875f,1.179799992893636240154364713816903531551361083984375f,-0.45749997809529308057108210050500929355621337890625f,0.2100999741047619995271134030190296471118927001953125f,-0.01179999895095829032243273104541003704071044921875f,0.90299997393786934907211616518907248973846435546875f,-0.985899982115626460910107198287732899188995361328125f,-0.42980004046559283636952386586926877498626708984375f,-0.15550000426173227197068626992404460906982421875f,0.0337000000953675549197896543773822486400604248046875f,-0.1378999934256075476923797396011650562286376953125f,0.9739999952763316315440533799119293689727783203125f,0.35210000985264766626414711936376988887786865234375f,-0.47880002665221688129548738288576714694499969482421875f,-0.5354000053465366448790518916212022304534912109375f,-1.12329998495131722080486724735237658023834228515625f,-0.342399998998641930114672504714690148830413818359375f,0.470099990868568351487510881270281970500946044921875f,-0.1995999980434775000759373142500407993793487548828125f,1.227199995505809493323567949119023978710174560546875f,0.1414000010550022778943457524292171001434326171875f,-0.478200028355419970438333621132187545299530029296875f,-0.4013000136137012674453217186965048313140869140625f,-1.108200001645087695578695274889469146728515625f,0.5249000375926495820522177382372319698333740234375f,-1.0336999616503721455273989704437553882598876953125f,-0.308600019713491047212983175995759665966033935546875f,0.0248000305935740517071508293156512081623077392578125f,1.124300020818412182421752731897868216037750244140625f,-1.53939999935627014338024309836328029632568359375f,-0.686500008016825091772261657752096652984619140625f,-1.4324999538064009385607278090901672840118408203125f,-0.32510001679658895312741151428781449794769287109375f,-0.019799984884262045170544297434389591217041015625f,1.18139998421669023542790455394424498081207275390625f,-0.92509997477531413778706337325274944305419921875f,0.4693999974936247099321917630732059478759765625f,-0.25109997391104688091445495956577360630035400390625f,-1.5920000280737880427039954156498424708843231201171875f,1.0189999908804896477931833942420780658721923828125f,-0.022699981439112892900311635457910597324371337890625f,-0.6150000095367431640625f,-0.859900043255091617311336449347436428070068359375f,0.3403999938726425256163565791212022304534912109375f,1.143100017333031015454025691724382340908050537109375f,-0.30459998466968585262293345294892787933349609375f,-0.6738000003516670943781718960963189601898193359375f,-1.1677000097513197118814787245355546474456787109375f,-0.5167000146016480499611134291626513004302978515625f,0.065599999377131457123368818429298698902130126953125f,-0.01060001725852510734426914495998062193393707275390625f,-0.5749999863281851641971798017038963735103607177734375f,-0.1294000090420244220013046287931501865386962890625f,1.02049998425692312054025023826397955417633056640625f,-0.518799985599518009848907240666449069976806640625f,-0.163900022903829156728505722639965824782848358154296875f,0.2499000094853341680245506495339213870465755462646484375f,-0.04159998512044527085862455351161770522594451904296875f,-1.01070000388026226545434838044457137584686279296875f,0.19999997183680573442643435555510222911834716796875f,-0.664399984604120419362516258843243122100830078125f,0.619299960207939648171304725110530853271484375f,0.112200007128715473214697340154089033603668212890625f,-0.4238999964356420235844780108891427516937255859375f,-0.71829997522830968392781869624741375446319580078125f,0.032600037173927010059060194180347025394439697265625f,-0.50179997205436233098652110129478387534618377685546875f,0.1356999867439265816670967979007400572299957275390625f,0.7734999979436396611021109492867253720760345458984375f,1.28730003044456253746830043382942676544189453125f,-0.4701000083029267528900163597427308559417724609375f,0.695299995255469927002422991790808737277984619140625f,0.2198999958693981238155856772209517657756805419921875f,-0.34580002287477229483414475907920859754085540771484375f,-0.49249998450279264261553180404007434844970703125f,-0.1829000077679749924897123491973616182804107666015625f,-0.6179000050485132788935516146011650562286376953125f,0.1120999839991327906574269945849664509296417236328125f,0.258999993227422231623080506324185989797115325927734375f,-0.11390002880841543220213907261495478451251983642578125f,-1.7206000065296891410326907134731300175189971923828125f,0.415999994516372595398934208787977695465087890625f,1.250400001285969953102039653458632528781890869140625f,0.976799973171949442729555812547914683818817138671875f,-1.4889000403195622457275248962105251848697662353515625f,-0.277899962878226602924769395031034946441650390625f,1.5202000565409665711058551096357405185699462890625f,0.737599978351592966419048025272786617279052734375f,-2.06289998940229413193492291611619293689727783203125f,-0.29079997522831035894341766834259033203125f,0.707499977350234843243015347979962825775146484375f,-1.9014999750852581428262055851519107818603515625f,1.623300009500979967924649827182292938232421875f,-0.394900066947936778660732670687139034271240234375f,0.1616999750949437153479948392487131059169769287109375f,0.7985999627351765273175487891421653330326080322265625f,0.21850003471225509787956298168865032494068145751953125f,1.257499989569186737980999168939888477325439453125f,-1.048899987196922012344657559879124164581298828125f,0.55160003330707496616014395840466022491455078125f,-1.0678999965548516826174818561412394046783447265625f,0.2798000245213501102625741623342037200927734375f,-0.5446999890983104108954648836515843868255615234375f,0.0060999889612194380106302560307085514068603515625f,0.4666999744057651611228720867075026035308837890625f,-1.802499976158141947735202847979962825775146484375f,0.0542000095725061470375294447876513004302978515625f,-1.13230001528263102272831019945442676544189453125f,0.6769000016286970122791899484582245349884033203125f,0.76619999327808585576349287293851375579833984375f,0.717799978417158257570918067358434200286865234375f,0.46409999698549497804123120658914558589458465576171875f,0.3773999876365066175054607811034657061100006103515625f,-0.882500005066395143416002611047588288784027099609375f,0.477600017690658518176860525272786617279052734375f,1.19520002070367326041377964429557323455810546875f,0.113899993306398439329996108426712453365325927734375f,-0.6285000231266020254139448297792114317417144775390625f,0.5038999833226203151070876629091799259185791015625f,0.97509995812326710762363290996290743350982666015625f,-0.8805999854445456520579682546667754650115966796875f,0.174900020456313587402519260649569332599639892578125f,-0.6731000270187852407843820401467382907867431640625f,-1.210899958944321230802643185597844421863555908203125f,0.24139998885467661704495867525110952556133270263671875f,-0.7262999704748394069753203439177013933658599853515625f,0.55719999118447294250699997064657509326934814453125f,0.42140000407248801028714524363749660551548004150390625f,0.42640000597238536528266195091418921947479248046875f,0.987499955967069187323659207322634756565093994140625f,0.410299996298551405971011263318359851837158203125f,0.4950999610662465411081711863516829907894134521484375f,0.327599974775314439767726071295328438282012939453125f,0.7985999663487077793178059437195770442485809326171875f,-1.0274999792873860116060313885100185871124267578125f,0.01319996811747536913372869094018824398517608642578125f,-0.6134000091612339389257613220252096652984619140625f,-0.2417999939590693703195256603066809475421905517578125f,0.4145999974846841240605499478988349437713623046875f,-0.151800007891655042158163269050419330596923828125f,0.03709993966817837218741260585375130176544189453125f,-0.094499996185302137519101961515843868255615234375f,-1.02109997659325557606280199252068996429443359375f,-1.0715999853253368456762473215349018573760986328125f,0.70679997224062685035050890292041003704071044921875f,0.39399999285489339939658748335205018520355224609375f,-0.223700019317865983481397051946260035037994384765625f,0.015000036284327666891158514772541821002960205078125f,0.50630001651942746487833346691331826150417327880859375f,-1.58279998810291289146334747783839702606201171875f,0.317399995999782635891506288317032158374786376953125f,0.1102000054672360729313140836893580853939056396484375f,-0.9604999794885513697551004952401854097843170166015625f,1.1538000475883478657124214805662631988525390625f,-1.385000015795231576021251385100185871124267578125f,0.70079999489784228217104100622236728668212890625f,-0.5361000303864482674498503911308944225311279296875f,-0.36869995360374474557829671539366245269775390625f,0.4073999972105024625079749966971576213836669921875f,1.038100004220009253685930161736905574798583984375f,0.0940000346899036998138399212621152400970458984375f,-1.95019997994899796367462840862572193145751953125f,1.492300014537573105144474538974463939666748046875f,0.3970999694705010796269561978988349437713623046875f,0.04589999254643917225848781527020037174224853515625f,-0.23340000945925698516703050700016319751739501953125f,0.4434999931752685586872075873543508350849151611328125f,-0.235200022715330359091012724093161523342132568359375f,0.79540001018941364208103550481610000133514404296875f,0.6080000394284732490035594310029409825801849365234375f,0.88599998609721664077909508705488406121730804443359375f,0.12160004273205993197137786410166881978511810302734375f,0.0643000022292137263235645150416530668735504150390625f,0.1951999673016372172096311032873927615582942962646484375f,-0.5941999886550008991292770588188432157039642333984375f,-0.77090002031847770780359496711753308773040771484375f,0.5224999771267178072520209752838127315044403076171875f,0.619699973899125122756004202528856694698333740234375f,0.815199990603327595550808837288059294223785400390625f,1.3930000533610584767529871896840631961822509765625f,0.34539999059438708517433269662433303892612457275390625f,-0.3384999705255034019302229353343136608600616455078125f,-0.430499998971819675119832027121447026729583740234375f,-1.505200015413761693849892253638245165348052978515625f,0.14209999629259117970292436439194716513156890869140625f,-0.326300004151463596269167055652360431849956512451171875f,0.034099999090284112412962969074214925058186054229736328125f,0.529599970383197504730787841253913938999176025390625f,0.263599987359345089288353847223334014415740966796875f,1.22449996727705023857879496063105762004852294921875f,-0.7262999921560286953337026716326363384723663330078125f,-0.45190004681646822604079716256819665431976318359375f,0.059899998290836808934045620844699442386627197265625f,0.5154000076562166299254386103712022304534912109375f,-0.550299993169307555263003450818359851837158203125f,-1.15320003561973560834985619294457137584686279296875f,0.70559999643415205472507523154490627348423004150390625f,0.3753999972999098044823540476500056684017181396484375f,-0.58589997798055371003300706433947198092937469482421875f,-0.45820002191066766528138032299466431140899658203125f,-0.595600008355081200761560467071831226348876953125f,-0.3210000013709066735145825077779591083526611328125f,0.625600004443526369612982307444326579570770263671875f,0.57740001419782682745562851778231561183929443359375f,-0.6262999963283537585567728456226177513599395751953125f,-0.31160000343024696345395341268158517777919769287109375f,0.9387999730825422606272923076176084578037261962890625f,0.564999975264072418212890625f,0.200100014561414685232421106775291264057159423828125f,0.292599996829033148770804473315365612506866455078125f,-1.1686000052779912383726923508220352232456207275390625f,-0.5122000011682512621291607501916587352752685546875f,0.15900002153962855544477861258201301097869873046875f,-0.5578000064313408667970861642970703542232513427734375f,-1.299100008533895067586172444862313568592071533203125f,-0.71219999163150760068674571812152862548828125f,0.249500008367002035214454735978506505489349365234375f,-0.0205999819427729047305319909355603158473968505859375f,-0.31560000377297381390917507815174758434295654296875f,-0.620200007963180865999675006605684757232666015625f,0.34989998532682642373714543282403610646724700927734375f,0.086199994730949303090028479346074163913726806640625f,-0.933399989268183905011255774297751486301422119140625f,-0.7099999952316284890230235760100185871124267578125f,-0.323100006231665570766153905424289405345916748046875f,0.50890002199113393288598672370426356792449951171875f,0.42720000120550383204687250326969660818576812744140625f,0.7085000240206713595370047187316231429576873779296875f,-0.4113999821677805091013624405604787170886993408203125f,-0.0383000181436532471934697241522371768951416015625f,-0.3291999907597900065781004741438664495944976806640625f,-0.646700005847215653176363048260100185871124267578125f,0.6023000143885608448357515953830443322658538818359375f,0.351500047393143777441082420409657061100006103515625f,-0.647099972040951598728497629053890705108642578125f,-0.89199996764957856232314270528149791061878204345703125f,-0.26630000601410852567596521112136542797088623046875f,-1.100400019651651906116285317693836987018585205078125f,0.7009999982044099464673081456567160785198211669921875f,0.752100033992528782533781850361265242099761962890625f,0.7187999702140690505558495715376920998096466064453125f,2.554900005182623612398629120434634387493133544921875f,-0.5518999694794419763610449081170372664928436279296875f,-0.133600013175605880633156630210578441619873046875f,-0.33159999281317009689473707112483680248260498046875f,0.24350001079589167485295320147997699677944183349609375f,-0.10849999887496231121275513942237012088298797607421875f,0.602199985857307495962231769226491451263427734375f,-0.17020000449866046210445347242057323455810546875f,0.3675000053644181807754875990212894976139068603515625f,-0.64709999666511996974094245160813443362712860107421875f,-0.5627000616073605243627753225155174732208251953125f,-0.686099999205767918653009473928250372409820556640625f,-0.275100044214725780733488136320374906063079833984375f,-0.670999995820224182097035736660473048686981201171875f,-0.317799990785122332681567058898508548736572265625f,0.2489999618977309925327290329732932150363922119140625f,0.17219999133348462549975010915659368038177490234375f,0.77680002934932745883855886859237216413021087646484375f,-0.1360000189542773796347319148480892181396484375f,-1.0142000224620104642525575400213710963726043701171875f,-0.240999997496604834168465458787977695465087890625f,0.166800002899765653641139806495630182325839996337890625f,-0.42800002467632314306911212042905390262603759765625f,-0.926000025659799774757630075328052043914794921875f,-0.153400000667571756451934561482630670070648193359375f,0.2160000177063050585957171279005706310272216796875f,-0.364800019417703402790920108600403182208538055419921875f,-0.9757000408537688240784291338059119880199432373046875f,-0.4670000124275681674390625630621798336505889892578125f,0.48500002410262832430731805288814939558506011962890625f,-0.26060001122355469505009750719182193279266357421875f,-0.4868000126972795715829533946816809475421905517578125f,-0.05679998375177319758222438395023345947265625f,0.95800003365427255364039638152462430298328399658203125f,0.0599999886006117311154639537562616169452667236328125f,-1.472800004196166856473837469820864498615264892578125f,-0.58600000733137136421646573580801486968994140625f,-0.1510000093616545857599930968717671930789947509765625f,-0.392800013844668949314353767476859502494335174560546875f,-0.8057000211097300468310322685283608734607696533203125f,-0.591600020194054110334036522544920444488525390625f,-0.807600032368302578333896235562860965728759765625f,0.60090003449320850137382876710034906864166259765625f,0.4617999753698709408666900344542227685451507568359375f,0.714500031799076840144380184938199818134307861328125f,-0.1876999959677458917894909973256289958953857421875f,0.497700010198354192425540531985461711883544921875f,0.23450000739842646879651510971598327159881591796875f,0.14440000725388557611950091086328029632568359375f,0.09359999644905325766330861370079219341278076171875f,1.258999987602234416073088141274638473987579345703125f,-0.580700014273821896182425916776992380619049072265625f,-0.90980004881024445495540931005962193012237548828125f,0.086199990297853912313286173230153508484363555908203125f,1.0430000207833938219437186489813029766082763671875f,-0.050599999060481792856336369368364103138446807861328125f,0.19890001455917971640019459300674498081207275390625f,-0.83029999607503413017184357158839702606201171875f,0.0579999755769964320251119715976528823375701904296875f,0.089699999678134911818006003159098327159881591796875f,0.105499990627169637491533649154007434844970703125f,0.705100004614889197540605891845189034938812255859375f,1.787300045867265207988339170697145164012908935546875f,0.0309000047653913423317817432689480483531951904296875f,-0.1079000057563188619269567425362765789031982421875f,-0.3988999927476049034424931960529647767543792724609375f,0.4434999984465541889022688337718136608600616455078125f,-0.82599999323114747795671064523048698902130126953125f,-0.51780002407059111391163241933099925518035888671875f,-0.678600016826390817215042261523194611072540283203125f,-0.202800040070713283313352803816087543964385986328125f,0.16040004926770967674798384905443526804447174072265625f,-0.1362000313878066748429773724637925624847412109375f,0.0365999391317364253239929894334636628627777099609375f,0.5263999767661093898141189129091799259185791015625f,-0.92729999274462482361514048534445464611053466796875f,-0.2880000100731852086255457834340631961822509765625f,0.8689000216558577793790618670755065977573394775390625f,0.048399987554549994683839031495153903961181640625f,-0.544199989977478981728609142010100185871124267578125f,-0.1310000106096271821343179908581078052520751953125f,-1.399000025041401418235409437329508364200592041015625f,-0.418599989557266294326609568088315427303314208984375f,1.607000016674399400784523095353506505489349365234375f,0.67960001775026324111195208388380706310272216796875f,-0.8747000181555748543615891321678645908832550048828125f,-0.35800000232458106808053344138897955417633056640625f,-0.1457000122621658899646490681334398686885833740234375f,-0.00619998966455437283684659632854163646697998046875f,0.7414000061959027387814558096579276025295257568359375f,0.2228000049412248717572992973146028816699981689453125f,0.653900006568431724218726230901665985584259033203125f,0.515200011539459268306018202565610408782958984375f,0.0139000339865684718887450799229554831981658935546875f,-0.11119998281002008155837756930850446224212646484375f,0.021000013887882307273002879810519516468048095703125f,0.156400018191337863981971167959272861480712890625f,-0.6522999909937381435298675569356419146060943603515625f,-0.0637999748706814528986797085963189601898193359375f,-0.0321000143229961398816385553800500929355621337890625f,0.090900033003091351702096289955079555511474609375f,-0.6114999928921458494102125769131816923618316650390625f,-1.46270003433823614358288978110067546367645263671875f,0.51959996701180966027777685667388141155242919921875f,0.019000017404555702427160213119350373744964599609375f,-0.72470000850707261275118753474089317023754119873046875f,-0.05210006096363173355712206102907657623291015625f,0.6313999983727931475385730664129368960857391357421875f,0.80210002162456550678371058893389999866485595703125f,-0.486700000408291533204874212970025837421417236328125f,-1.030400041407347089261747896671295166015625f,-0.390600001426041065766270321546471677720546722412109375f,-0.15900001174211464416430317214690148830413818359375f,0.2171000031843781652440839025075547397136688232421875f,-0.36110001965761195918958037509582936763763427734375f,0.12759999206066130472692066177842207252979278564453125f,0.231700024771690937797075093840248882770538330078125f,-0.663599999131262219265181556693278253078460693359375f,-0.80880002933442607382374944791081361472606658935546875f,0.5064999766126276892208579738507978618144989013671875f,0.167900002589821628617983151343651115894317626953125f,0.4084999929144979002870741169317625463008880615234375f,0.434299965125322540160368589567951858043670654296875f,0.56290001298338143698174462770111858844757080078125f,0.7623000117063529756933348835445940494537353515625f,0.82400000138580775654872923041693866252899169921875f,0.5177000474512585270758791011758148670196533203125f,-0.6935999717876317749443160209921188652515411376953125f,-0.7741000211440021150366419533384032547473907470703125f,-0.338899992766231561436285346644581295549869537353515625f,-0.26620002214908566884332685731351375579833984375f,-0.2306999623432754997764959625783376395702362060546875f,-0.03599999541044240913834073580801486968994140625f,0.101599983239173685678480296701309271156787872314453125f,0.011399968421458961387315866886638104915618896484375f,0.302099999906122462789426208473742008209228515625f,0.70000000298023223876953125f,0.4998000070124863913889612376806326210498809814453125f,0.5607999940037728947572759352624416351318359375f,-0.331300000351667456310877923897351138293743133544921875f,0.171399992859363692332408390939235687255859375f,-1.1047999912917616516239149859757162630558013916015625f,-0.6589999881982802065749638131819665431976318359375f,1.487100019426643715547697865986265242099761962890625f,0.34960000135898638973230845294892787933349609375f,-1.402400013378262588759071149979718029499053955078125f,0.04440001425743123064648898434825241565704345703125f,0.897000023119151723705044787493534386157989501953125f,0.10000001341104525209857456502504646778106689453125f,-0.4174000025376678468802538191084749996662139892578125f,-0.359000003695487901467231495189480483531951904296875f,0.2790000252649189604881030390970408916473388671875f,-0.04739998417198620916934714841772802174091339111328125f,-0.121899995630979418592687579803168773651123046875f,-0.26600001746416079839718804578296840190887451171875f,-0.464900009988248097414498261059634387493133544921875f,0.192800000321864839492036480805836617946624755859375f,0.332499973364174084000666198335238732397556304931640625f,0.318299997095764120036420763426576741039752960205078125f,-0.6570999905556440534581952306325547397136688232421875f,-0.685700035321712420000039855949580669403076171875f,0.2179000017330050553709952509962022304534912109375f,0.51579999668598208728553800028748810291290283203125f,-0.7205000073909755275991528833401389420032501220703125f,-1.860300048452616206162701928406022489070892333984375f,0.66989998122900740185059476061724126338958740234375f,-0.120899990087747877254287232062779366970062255859375f,0.24500000216066830649452867874060757458209991455078125f,-0.653099981272220464489919322659261524677276611328125f,-0.36689999078810220378699114007758907973766326904296875f,-0.4424000369966023527013021521270275115966796875f,-0.188000006608664893548166219261474907398223876953125f,1.01499994531273873832333265454508364200592041015625f,0.7558999994382260556591290878714062273502349853515625f,-0.431699951308965435003983657225035130977630615234375f,0.0042999938473105014935526924091391265392303466796875f,0.4247000008702277451533291241503320634365081787109375f,0.53460000359415982273958434234373271465301513671875f,0.9357000217616564530231926255510188639163970947265625f,-0.508499995186924902412783922045491635799407958984375f,-0.0298000474318864672085283018532209098339080810546875f,0.87260001098513562123315523422206752002239227294921875f,1.737600047008693859851291563245467841625213623046875f,-0.404599983254075201699606623151339590549468994140625f,-1.0627999666452410121308957968722097575664520263671875f,-0.1336999975249171246360901932348497211933135986328125f,-0.784699993419646801839917316101491451263427734375f,0.1440999923661350212711340645910240709781646728515625f,-0.01239999125003787838750213268212974071502685546875f,-0.225699997547268882414073232212103903293609619140625f,-0.377200013983249871074576731189154088497161865234375f,0.795999991387128513764537274255417287349700927734375f,0.4725999869942665299049622262828052043914794921875f,-0.598799980905651896279096035868860781192779541015625f,-0.3846000150680541906922371708787977695465087890625f,-1.05350001958757655273757336544804275035858154296875f,-0.541600001418590437651801039464771747589111328125f,0.675700004290044287103000897332094609737396240234375f,0.311999989330768556783368694595992565155029296875f,0.2183999861761927174796937833889387547969818115234375f,0.15759997686147730888706064433790743350982666015625f,0.11250001188367608762064264738000929355621337890625f,0.250999984383583285563190656830556690692901611328125f,-1.042800004494190080350790594820864498615264892578125f,0.05000002652406720926592242904007434844970703125f,1.065000005550682526944683559122495353221893310546875f,0.239799998295307403850529226474463939666748046875f,0.37129995937347415324580879314453341066837310791015625f,0.4349999594688416237886485760100185871124267578125f,-0.0506999982550742078757366471108980476856231689453125f,-0.10880003149509409610118382261134684085845947265625f,-1.05829997057169666874187896610237658023834228515625f,-0.319599991226195978555324472836218774318695068359375f,0.62080000007599611677733264514245092868804931640625f,0.26419999318122844300660290173254907131195068359375f,-1.316899991793930357886210913420654833316802978515625f,0.106099965566396559069062277558259665966033935546875f,0.524699992860853559051292904769070446491241455078125f,1.2079000296354294174960841701249592006206512451171875f,-0.3271000083997843699279428619774989783763885498046875f,-0.31760000740885718784056734875775873661041259765625f,-0.6953999973371622900231159292161464691162109375f,-1.105300062757731183182841050438582897186279296875f,-0.2115999873742462344949899488710798323154449462890625f,0.20319997795224242764788868953473865985870361328125f,0.3031999772816897920080236872308887541294097900390625f,0.84860003306865650074541918002068996429443359375f,-0.69089998841136690543862641789019107818603515625f,-1.34050002154707925683396751992404460906982421875f,-0.72719997337758568445309492744854651391506195068359375f,-1.42489999467730488191818949417211115360260009765625f,-0.355599981234967887910869421830284409224987030029296875f,-1.675799992364644719344823897699825465679168701171875f,0.4480000002384183499515302173676900565624237060546875f,0.916900040036440433510733782895840704441070556640625f,-0.39949998240172879260256877387291751801967620849609375f,-1.25019996504783659219128821860067546367645263671875f,-0.7742999919474125025686817025416530668735504150390625f,-1.224700018751621488632963519194163382053375244140625f,-0.1326000004053116132496370482840575277805328369140625f,-0.3417999944806100476313304170616902410984039306640625f,-0.49069996748417654686136302188970148563385009765625f,-0.386999978005885481024961336515843868255615234375f,-0.18419999351650473418118281188071705400943756103515625f,-0.6374000161349773208030455862171947956085205078125f,-0.75580002131015067146080355087178759276866912841796875f,-0.21160000201463713409566480549983680248260498046875f,0.2720000225976113572556869257823564112186431884765625f,-0.33279999644756319554517176584340631961822509765625f,-0.373999993264674923665324968169443309307098388671875f,-0.195399973905086454806223628111183643341064453125f,-0.7503999921590089883238761103712022304534912109375f,-0.8715999936699869721223876695148646831512451171875f,0.7620999812051654220823593277600593864917755126953125f,0.17679999388456391073987106210552155971527099609375f,0.668499999396502619930515720625407993793487548828125f,0.27559998276233699243675800971686840057373046875f,-1.014900002872944018150747069739736616611480712890625f,0.239400013661384747365445946343243122100830078125f,0.61090000830441726353825515616335906088352203369140625f,0.1820000104904178073184084496460855007171630859375f,1.0386000021487473699011161443195305764675140380859375f,0.58000000447034860684425439103506505489349365234375f,-0.309899990840256212010217495844699442386627197265625f,0.1177999976396559844715739018283784389495849609375f,0.4268999850139021390305060776881873607635498046875f,0.349999996274709790355927907512523233890533447265625f,-0.30940000874400108710204904127749614417552947998046875f,0.1613999967336656737870725919492542743682861328125f,-0.9303000236794354282920949117396958172321319580078125f,-0.62580000026226034748333404422737658023834228515625f,0.6836000074014065841510046084295026957988739013671875f,-0.10859998627901035206377855502068996429443359375f,0.10740003796517871847981950850225985050201416015625f,-0.15539996481537787786919579957611858844757080078125f,-0.54810000470429642849268248028238303959369659423828125f,0.177699963408707528600416480912826955318450927734375f,0.524900012446939623345087966299615800380706787109375f,1.09840003225803428676954354159533977508544921875f,0.811200001026689765382116092951036989688873291015625f,2.12380002717375848675374072627164423465728759765625f,-0.878799994242191129245611591613851487636566162109375f,-0.242200047659874240224553432199172675609588623046875f,-0.542699968661368270517186829238198697566986083984375f,-0.6973000197529781729599562822841107845306396484375f,0.663399978613853580355907979537732899188995361328125f,-0.49169996561408080282262744731269776821136474609375f,0.2937999877974390994239684005151502788066864013671875f,0.280600020164250896215207831119187176227569580078125f,0.45050000973790849112532441722578369081020355224609375f,0.341900044357777321835101247415877878665924072265625f,0.8058999922111629299337209886289201676845550537109375f,0.16180005497932459235244095907546579837799072265625f,-0.717099989065527854137371832621283829212188720703125f,-1.4111000239044428905543782093445770442485809326171875f,0.05229999915212391048413564931252039968967437744140625f,-0.8547999588817365435033934772945940494537353515625f,-0.559299968627095633877388536348007619380950927734375f,-0.8101999753296365014421098749153316020965576171875f,-0.91979999751299612587018827980500645935535430908203125f,-0.10770005475282662388281096355058252811431884765625f,0.633999994866549787531084803049452602863311767578125f,1.494600012534856858792409184388816356658935546875f,0.59369999175071708208406562334857881069183349609375f,0.47960000344514863712674923590384423732757568359375f,-0.214000002540648048832139238584204576909542083740234375f,0.11299999867379673101908110766089521348476409912109375f,0.55930000610351537915221342700533568859100341796875f,0.4050000104308129067476329510100185871124267578125f,-0.11869999670535325009268490248359739780426025390625f,0.00640000157654274648422187965479679405689239501953125f,0.362899993313849034137774651753716170787811279296875f,-0.186800034415721682989897090010344982147216796875f,-0.056199950362741890597106930727022700011730194091796875f,0.05719999867230629586600798575091175734996795654296875f,-0.57500002160668362005679909998434595763683319091796875f,0.2816000069320201415479232309735380113124847412109375f,-0.533700032430887372214556307881139218807220458984375f,-0.191400014615059177458533667959272861480712890625f,0.0948000121906400483595689365756697952747344970703125f,-0.1193999872863293898461733988369815051555633544921875f,-0.598500036723911943425946446950547397136688232421875f,-0.5094000001013274658845375597593374550342559814453125f,1.098900011411309218800624876166693866252899169921875f,-0.04160002704858722921699154539965093135833740234375f,-2.073299993743002556101373556884936988353729248046875f,-0.9949999979138368644271395169198513031005859375f,0.5412000344425444975371419786824844777584075927734375f,-0.419200021195411665075880591757595539093017578125f,-1.3699000270873309403896200819872319698333740234375f,-0.4813999938279389656514695161604322493076324462890625f,0.3069000198826195013879214457119815051555633544921875f,-0.805600004518031820310852708644233644008636474609375f,0.194400000473856859883881043060682713985443115234375f,-0.579699974010884577779734172509051859378814697265625f,-0.67569998901635408561361373358522541821002960205078125f,-1.3650000241771333975293600815348327159881591796875f,-0.1401000022679566026084785335115157067775726318359375f,-0.640100030542910314323989950935356318950653076171875f,0.282199988912046062505822874300065450370311737060546875f,1.129199988748132543747715317294932901859283447265625f,-0.1703999874651433987793325286475010216236114501953125f,0.010600000997633307608936092947260476648807525634765625f,0.087000005405396318991506632301025092601776123046875f,-0.76440001204088348085718962465762160718441009521484375f,0.43169999284595250710339087163447402417659759521484375f,0.13720004236996174995510955341160297393798828125f,-0.8600000121071931236116370200761593878269195556640625f,-0.898300040718913894721708857105113565921783447265625f,0.0345000012516975262943930147230275906622409820556640625f,0.4203999921590091393142074593924917280673980712890625f,-0.29309999751448645444185103769996203482151031494140625f,0.9715000039935113296252211512182839214801788330078125f,-0.59609997119158542044914383950526826083660125732421875f,-0.72480001647472303005770299932919442653656005859375f,-0.586599982754886450919684648397378623485565185546875f,1.198400008565187135900487191975116729736328125f,-0.633099960671365646902586377109400928020477294921875f,-0.79539999398440042188695997538161464035511016845703125f,-0.034600004041194953874338580135372467339038848876953125f,1.3110999928727746777212814777158200740814208984375f,0.2849999904818834028930041313287802040576934814453125f,0.356599999071657569604809623342589475214481353759765625f,-1.1103000016070898592346338773495517671108245849609375f,-0.9814000122308730311715407879091799259185791015625f,0.136299987201392358571183649473823606967926025390625f,-0.0479999933838843872990764793939888477325439453125f,-0.432699981252849141810656874440610408782958984375f,-0.5179999755024908836276154033839702606201171875f,0.961700000445544933569408385665155947208404541015625f,0.4315999859213821565617763553746044635772705078125f,0.249599974499643106895518940291367471218109130859375f,0.477799995255470122401675325818359851837158203125f,-0.2148000039950013206890844230656512081623077392578125f,-0.27700002151727698418426371063105762004852294921875f,-0.12319999531209457899194603669457137584686279296875f,0.1948000199019912503928253499907441437244415283203125f,1.2750000146403912548720427366788499057292938232421875f,0.7079999954700468833834747783839702606201171875f,-0.48059999933987851417072079129866324365139007568359375f,0.09100000680983078016339504756615497171878814697265625f,0.1550000127404931316021929887938313186168670654296875f,-0.2014000217676166215596822439692914485931396484375f,-0.033500027894973694042590750541421584784984588623046875f,0.506599994266033082368494433467276394367218017578125f,0.7395999986767767264694839468575082719326019287109375f,0.27499998241663003994972314103506505489349365234375f,0.679900020866096355121044325642287731170654296875f,1.0052000043869018242048696265555918216705322265625f,0.078100012639164884120646092924289405345916748046875f,-0.31799995257332935860716816023341380059719085693359375f,-0.185999994460493367487430305118323303759098052978515625f,0.14449999471381236304523554281331598758697509765625f,-0.63199997245520389554940265952609479427337646484375f,0.0120000261366381977268247283063828945159912109375f,-0.9059999945908783214321147170267067849636077880859375f,-0.73600008779764181099380948580801486968994140625f,0.712899988396465733586637725238688290119171142578125f,1.42810006591081606330817521666176617145538330078125f,-0.714900003655254412393560414784587919712066650390625f,-2.0036000379860414710719851427711546421051025390625f,0.39460000191777933054027016623876988887786865234375f,0.477300027501583290501230294466949999332427978515625f,-0.6193999964132903546243369419244118034839630126953125f,-1.0144000372797254971857228156295605003833770751953125f,0.7884000091612335392454724569688551127910614013671875f,0.8214000696003440626924430034705437719821929931640625f,0.485300005276501078554929335950873792171478271484375f,-0.3641999880030748482084845818462781608104705810546875f,0.440700001458823542588305599565501324832439422607421875f,1.138000020094215525290337609476409852504730224609375f,0.1307999997779727419100481711211614310741424560546875f,-0.4830999638378623561862923452281393110752105712890625f,0.0761999983444809902977112869848497211933135986328125f,-0.126300002884864870367209732648916542530059814453125f,0.2856000017985700356604183980380184948444366455078125f,-0.866799951565266013631116948090493679046630859375f,0.8271000046744940714660287994774989783763885498046875f,-0.15519995804428976526878614095039665699005126953125f,-0.5441999957889318562109792765113525092601776123046875f,-0.34599998229742023880817214376293122768402099609375f,-0.4044000159338116962004505694494582712650299072265625f,-0.0507999832749366220241427072323858737945556640625f,0.6104999949857592422830521172727458178997039794921875f,0.117200020241738034343370600254274904727935791015625f,0.0250000208988787608888060276512987911701202392578125f,0.1660000264048573370700978557579219341278076171875f,1.0011999969288705170100683972123079001903533935546875f,0.02580000234842305673055307124741375446319580078125f,-0.107199993196129828021412322414107620716094970703125f,0.01039999476671216172007916611619293689727783203125f,0.463799992603063759855075431914883665740489959716796875f,0.695000006556510729893716415972448885440826416015625f,1.0473999864071605880866400184459052979946136474609375f,0.65379999185800574679205965367145836353302001953125f,-0.73499998370185493090644968106062151491641998291015625f,-0.127200003589689691896325030029402114450931549072265625f,0.03269999989792726768911279577878303825855255126953125f,-0.15780001157224210484031345913535915315151214599609375f,0.1919999914914367611373791078221984207630157470703125f,0.331000007629394854546944770845584571361541748046875f,-1.4830000047832729848096278146840631961822509765625f,-1.0277999949574461169277128647081553936004638671875f,-0.402099998304247918667897465638816356658935546875f,-0.44039999029636334171300404705107212066650390625f,0.7868000003293158517436722831916995346546173095703125f,0.15999999262392494703277634471305646002292633056640625f,0.4578999914512034141722551794373430311679840087890625f,0.141799984794855671310642719618044793605804443359375f,0.5692000031650064073573958012275397777557373046875f,0.772600044959784071352260070852935314178466796875f,0.0795999918222427549352460118825547397136688232421875f,0.3243999944388864520306015037931501865386962890625f,0.805900007335841106481666429317556321620941162109375f,0.608900097912550730683278743526898324489593505859375f,0.0809999945908785878856406270642764866352081298828125f,0.6856999660313132238798061734996736049652099609375f,0.36909999076426036612730285924044437706470489501953125f,0.472499998807907051201482317992486059665679931640625f,-0.147499998621642536278386614867486059665679931640625f,-1.0083999860644345947235933635965920984745025634765625f,0.833799985711276558930649116518907248973846435546875f,0.37810002597570413485073004267178475856781005859375f,0.89729998518228526460660532393376342952251434326171875f,0.38530004325583566693325110463774763047695159912109375f,-0.365799990967661192708959561059600673615932464599609375f,-0.053300009258836988923491162495338357985019683837890625f,0.51509998561590919496921969766844995319843292236328125f,0.4522000241532924125209547128179110586643218994140625f,-0.2231999874517323068090490778558887541294097900390625f,-0.1037000415205948655739121022634208202362060546875f,-0.7646999703228478661998224197304807603359222412109375f,0.05630002076625917339924853877164423465728759765625f,-0.6583999830096962657677295283065177500247955322265625f,-0.350399969919025944253831994501524604856967926025390625f,-0.17600000327080478346175596016109921038150787353515625f,-0.8060000098273152335792701705940999090671539306640625f,0.15820001572668562772605582722462713718414306640625f,0.256199993836879524877758740331046283245086669921875f,-0.840700044262409296180749151972122490406036376953125f,-0.3378000149250031114434023038484156131744384765625f,0.992700006584823047006693741423077881336212158203125f,0.42599998822063211978417029968113638460636138916015625f,-0.52319999406412220910311816624016501009464263916015625f,-0.335400017900765068379342892512795515358448028564453125f,0.4848000211499634559686455759219825267791748046875f,-0.013800027024746697179580223746597766876220703125f,-1.6987000563472511860396707561449147760868072509765625f,-0.50860000624656631629250114201568067073822021484375f,0.86760002353936471752859915795852430164813995361328125f,0.50720001815557491653407851117663085460662841796875f,0.6955000183805821745153252777527086436748504638671875f,0.021400005376339237272986792959272861480712890625f,-1.25900000928342326034226061892695724964141845703125f,-0.52180000818967808839943245402537286281585693359375f,-0.09720000897273439743884182462352328002452850341796875f,-0.340800017808378041817007897407165728509426116943359375f,-0.68170000534430152949738612733199261128902435302734375f,-0.53920001165866882075761168380267918109893798828125f,-0.8141000200450421164788394889910705387592315673828125f,-0.7354000153303150710826230351813137531280517578125f,0.8315000163987280590305317673482932150363922119140625f,0.1077999948084353842858718053321354091167449951171875f,0.66719997263252694974511314285336993634700775146484375f,0.8542000027179721399761547218076884746551513671875f,0.30250005397945678442539474417571909725666046142578125f,0.443200071829557895597417882527224719524383544921875f,-0.138699996966123506592083458599518053233623504638671875f,-0.477800017458200887432440140401013195514678955078125f,0.5778999657839538617309926848975010216236114501953125f,1.1609000473268322739528457532287575304508209228515625f,0.00960000242069358744601004218566231429576873779296875f,-0.1454999893978243530323624099764856509864330291748046875f,-0.198899991536885589749061864495160989463329315185546875f,0.5684000166676934640008767019025981426239013671875f,-0.16530000002384159785151496180333197116851806640625f,-0.65740003684759162894124528975225985050201416015625f,-0.37189999496042747129109784509637393057346343994140625f,-0.346899960613251234775589182390831410884857177734375f,0.7759000103905793199743357035913504660129547119140625f,1.25380002017021308091671016882173717021942138671875f,0.4686000018879770134816453719395212829113006591796875f,0.58470001234412194435208220966160297393798828125f,-0.42769999425411231452898164206999354064464569091796875f,0.2879000051788989100742810478550381958484649658203125f,-0.81020001047775114244586802669800817966461181640625f,-0.5346000493966049926797268199152313172817230224609375f,-0.84200000229477856006354841156280599534511566162109375f,-0.21050004181265880021101111196912825107574462890625f,-0.8580000121966000481421588119701482355594635009765625f,-0.1203000464409598180282046087086200714111328125f,-0.5100999741792680008956040182965807616710662841796875f,-1.6806000149488438122347133685252629220485687255859375f,0.3145999897733331440718984595150686800479888916015625f,-0.19150000384450027723914899979718029499053955078125f,0.7866999807760119978183865896426141262054443359375f,0.406599984431267102991114370524883270263671875f,0.531499991141259453542033952544443309307098388671875f,0.31999997884035114026346491300500929355621337890625f,-0.3330999870836734988444050031830556690692901611328125f,0.29180000304877740102682537326472811400890350341796875f,-0.49619998720586266927057295106351375579833984375f,-0.21579998893737784015911529422737658023834228515625f,0.6237999909266829234866236220113933086395263671875f,0.110000004470348411445002057007513940334320068359375f,-0.2756999865204095012671814401983283460140228271484375f,-0.374400009340047912331783663830719888210296630859375f,-0.90280003831982613693440953284152783453464508056640625f,-0.0931999986648559541890790569595992565155029296875f,0.7128000225991011351567294695996679365634918212890625f,-0.3266000178098682482641379465349018573760986328125f,-0.641200014773011250923673287616111338138580322265625f,-0.25380000772774213846361135438201017677783966064453125f,-0.52019999596774579853075692881247960031032562255859375f,-0.2321999852240088646482263357029296457767486572265625f,0.0734999885372815686679359714617021381855010986328125f,-0.464400015263259635656112322976696304976940155029296875f,-0.1833000056080520412393752849311567842960357666015625f,-0.134400009787082552747961017303168773651123046875f,-0.684400003342330531808102023205719888210296630859375f,-0.2921999896943570096397024826728738844394683837890625f,0.9845999994404615041077022397075779736042022705078125f,0.468000009216368140219088900266797281801700592041015625f,0.8709999819062648196421605462091974914073944091796875f,1.38850006157159810982193448580801486968994140625f,-0.752099973046779890495372455916367471218109130859375f,-1.4265000127851958922065023216418921947479248046875f,0.75179997212886828794609073156607337296009063720703125f,0.1087000355973832821376845458871684968471527099609375f,0.44569998447150005471684153235401026904582977294921875f,0.265200019250809493343012945842929184436798095703125f,-0.5146999669700864643573368084616959095001220703125f,-0.2497000042974939493234387555276043713092803955078125f,0.45539999282956122872434434611932374536991119384765625f,1.0607999978035687060895497779711149632930755615234375f,-0.010099998281896116480993441655300557613372802734375f,-0.3330999727040531155353164649568498134613037109375f,-0.2516000076025723952710677622235380113124847412109375f,0.7964999950528142136363385361619293689727783203125f,-0.4435000003278253277727571912691928446292877197265625f,-0.388600034074485567447965195242431946098804473876953125f,-0.6098999880835414000301852865959517657756805419921875f,-0.7010000206306574721537572258966974914073944091796875f,0.28649998015165356601841040173894725739955902099609375f,1.431299976267665474694013028056360781192779541015625f,-0.470700011796503947625325281478581018745899200439453125f,-0.1783000336967415222488853032700717449188232421875f,-0.548999986708164300353018916212022304534912109375f,-0.75210001409947846440218199859373271465301513671875f,0.1662999850034715121349648825344047509133815765380859375f,0.42310001090690441127861731729353778064250946044921875f,0.71219999097958197609159469720907509326934814453125f,0.19240004670098453942728156107477843761444091796875f,0.275499992527068027214909307076595723628997802734375f,0.1075999887824057310581338242627680301666259765625f,0.770300005201995485748511782730929553508758544921875f,0.32759999593496313963214561226777732372283935546875f,-1.0309000053614376657407092352514155209064483642578125f,-0.47440001507699480765012367555755190551280975341796875f,0.2495999932378529795329313856200315058231353759765625f,0.22659999694824239924173525650985538959503173828125f,0.3440000014230608638143849020707421004772186279296875f,0.565200008857249969196345773525536060333251953125f,-0.606900018616020719974812891450710594654083251953125f,-0.1509999957084655619610202847979962825775146484375f,1.0145999964043499463883790667750872671604156494140625f,0.5954000110089776587329879475873894989490509033203125f,-0.61910001043379292440249628270976245403289794921875f,-0.63460001029968271524239753489382565021514892578125f,1.095100000479817037302154858480207622051239013671875f,0.640000011622905873309719027020037174224853515625f,-0.088700006316602131306581213721074163913726806640625f,0.17039998761415464656465701409615576267242431640625f,-0.9832000081643463840208596593583934009075164794921875f,-0.5825999902725218504428994492627680301666259765625f,0.05300001323968184241408607704215683043003082275390625f,-0.16579998238086712802896727225743234157562255859375f,0.81399998685717578350562462219386361539363861083984375f,0.34040000765621669653882008788059465587139129638671875f,0.75530000035911781797182129594148136675357818603515625f,0.496599998885393301151225387002341449260711669921875f,-1.5233000213846563664077393696061335504055023193359375f,-0.5068999871373183196965328534133732318878173828125f,-0.788899972482025813036443651071749627590179443359375f,-1.10250001341104475471865953295491635799407958984375f,0.39350000207871194124464864216861315071582794189453125f,0.324100010620057499277635315593215636909008026123046875f,-0.83509997581839578995044348630472086369991302490234375f,-0.978200004141032142257472514756955206394195556640625f,-0.4801999843075872576036999817006289958953857421875f,-1.59219999341964690842132768011651933193206787109375f,0.3470999832913281846202835367876105010509490966796875f,-0.3397999976992611692594437045045197010040283203125f,-0.3405000000894069511758743828977458178997039794921875f,-0.66999998189508958024163121081073768436908721923828125f,0.755799983684718679199932012124918401241302490234375f,1.118100015172361683113422259339131414890289306640625f,0.080399995549023106633512725238688290119171142578125f,0.417800025504828198563700425438582897186279296875f,0.143699995587766249371952653746120631694793701171875f,-0.741699990200996861489102229825221002101898193359375f,-0.3718999990954994228076202489319257438182830810546875f,-0.49600003638863565669225863530300557613372802734375f,0.288399986176192779652183162397705018520355224609375f,1.120900022721290412164307781495153903961181640625f,0.01270000058710574197817777530872263014316558837890625f,0.126100007401406977169955325734918005764484405517578125f,0.08200000017136332974843071497161872684955596923828125f,-0.4364999935999509528272710667806677520275115966796875f,-0.0165999974325299337607475536060519516468048095703125f,-0.520900003871322070381211233325302600860595703125f,-0.417200004781782762819375420804135501384735107421875f,-0.01999997377395601461103069595992565155029296875f,0.9412999930500987044723615326802246272563934326171875f,0.348199998366832463858600021922029554843902587890625f,-0.51719999468624588967458066690596751868724822998046875f,-0.370199993099272595120652340483502484858036041259765625f,0.346300009385496654346070499741472303867340087890625f,0.1450000312924384360258045489899814128875732421875f,0.5502999766290190830630990603822283446788787841796875f,0.426600005590915376529892455437220633029937744140625f,0.27979998738020650517199783280375413596630096435546875f,0.04719999699592580810758590814657509326934814453125f,-0.5222999882370233937223247266956605017185211181640625f,-0.3307999838709836382122375653125345706939697265625f,1.263500009007751945233621881925500929355621337890625f,0.4394000054657454246154202337493188679218292236328125f,-0.20910004601031573212566172514925710856914520263671875f,0.092599972391128826387785011320374906063079833984375f,0.6375000593811275262368098992737941443920135498046875f,0.71260002812147149597876705229282379150390625f,-0.084400018727779357874396737315692007541656494140625f,0.163399993515014774203564229537732899188995361328125f,-0.58420000912547098703697656674194149672985076904296875f,-0.3378000146269801717835434828884899616241455078125f,0.6956000047415493714453305074130184948444366455078125f,0.579200020599365483775500251795165240764617919921875f,-0.5223000064536928999103793103131465613842010498046875f,-0.29080000182688225951466165497549809515476226806640625f,-0.1035000245794657214304379522218368947505950927734375f,-1.02060000734925182541701360605657100677490234375f,-0.369299991835653873550882053677923977375030517578125f,-0.4939999840259556851407296562683768570423126220703125f,0.3431999914005399165972676200908608734607696533203125f,0.3109000198714435558855484487139619886875152587890625f,-0.455600001016259026354049410656443797051906585693359375f,-0.364700035180151871383458228592644445598125457763671875f,0.7682000016823409271893297045608051121234893798828125f,0.5749000635206698017753979002009145915508270263671875f,0.3798999927401542908711462587234564125537872314453125f,0.234300022345781400190389831550419330596923828125f,-0.16460000366866578414004607111564837396144866943359375f,-0.14560002225041390744308955618180334568023681640625f,0.5179999682009224226675314639578573405742645263671875f,1.3274999807029956855330965481698513031005859375f,0.7346999758362773302877712922054342925548553466796875f,0.3947999924838538898796969078830443322658538818359375f,-0.61489999855160704100853763520717620849609375f,-1.603800010335445502818174645653925836086273193359375f,0.575800016914307821735974357579834759235382080078125f,-0.103899982130526868928654948831535875797271728515625f,-0.113699993762373996464276615370181389153003692626953125f,-0.131599997395277501510690854047425091266632080078125f,0.49189997491836567888867648434825241565704345703125f,0.5395000241994856171601213645772077143192291259765625f,0.4834000032007692571056622909964062273502349853515625f,0.18800002344697741563095405581407248973846435546875f,-0.42799999353289608361450291340588591992855072021484375f,-0.77000000335276064244993676766171120107173919677734375f,0.215899995638430208799718457157723605632781982421875f,0.27719999878406564874921969021670520305633544921875f,-0.4411999846354126564307307489798404276371002197265625f,-0.50200000505149355856104875783785246312618255615234375f,-0.658400033859908972999619436450302600860595703125f,-0.07060000243186959067998031969182193279266357421875f,1.0164000255301595121437685520504601299762725830078125f,0.07299999457597738938829934340901672840118408203125f,-1.1164000269085165850668772691278718411922454833984375f,-0.0798000060439107983256690204143524169921875f,0.4345000069141387033511136905872263014316558837890625f,0.297800032806396330897769075818359851837158203125f,0.4714000182285904561041434135404415428638458251953125f,-0.21679997719526244992493957397527992725372314453125f,-0.17619998539909731416486238231300376355648040771484375f,-0.365599998743831922087821340028312988579273223876953125f,-0.137299998537450840263574036725913174450397491455078125f,-0.26620002274513243634146419935859739780426025390625f,0.46720001014620093915397092132479883730411529541015625f,0.5319999902248380152514073415659368038177490234375f,-0.2838000288128850190361163186025805771350860595703125f,0.234199981558322889441114966757595539093017578125f,-0.38459999927282317866428229535813443362712860107421875f,-0.041799994480609559133199581992812454700469970703125f,0.519599988357722697429608160746283829212188720703125f,0.49440001574754699475988672929815948009490966796875f,-0.133200013081729462527391660842113196849822998046875f,-0.0925999920606610515960710472427308559417724609375f,0.41440002092570049629927098067128099501132965087890625f,-0.246899962811171658216835567145608365535736083984375f,-0.786899964897334935898243202245794236660003662109375f,-0.829900026156007442068585078231990337371826171875f,-0.20139999237507588691897808530484326183795928955078125f,-0.361699981223046140854648911044932901859283447265625f,-0.868500012621283179470310642500407993793487548828125f,-0.49000006735324941331555237411521375179290771484375f,0.22000000577419986935723272836185060441493988037109375f,-0.58049994972348262223249548696912825107574462890625f,-0.916499968901276940158595607499592006206512451171875f,-1.584599983775614617798055405728518962860107421875f,0.432499984577298324239791327272541821002960205078125f,1.082699951338768062925055346568115055561065673828125f,0.74670000722557272609947176533751189708709716796875f,1.8121000139690945918147235715878196060657501220703125f,-0.053099997942894716640438446120242588222026824951171875f,0.688199991419165879591446355334483087062835693359375f,-0.2665999889388681598489938551210798323154449462890625f,-0.6085999651193620962885688641108572483062744140625f,-0.2189999841004610647843264814582653343677520751953125f,-1.3898999778389935499944840557873249053955078125f,-0.40899998227506895176475154585205018520355224609375f,-1.0914999792575834636210174721782095730304718017578125f,-0.49519996746927541497740321574383415281772613525390625f,-1.177799971562623060350460946210660040378570556640625f,0.75189997376352568636548312497325241565704345703125f,0.59560001092553083168468219810165464878082275390625f,0.319799996469914749042118273791857063770294189453125f,1.26420000804513765757519649923779070377349853515625f,0.5898000202372670752737349175731651484966278076171875f,0.60220000027418141286261743516661226749420166015625f,-0.36799999614059952790512397768907248973846435546875f,0.01740000138282749020390838268212974071502685546875f,0.44509999075680983793290579342283308506011962890625f,0.635600003325939244547271300689317286014556884765625f,-0.782099960604309973888348395121283829212188720703125f,-0.6499999821931126131602241002838127315044403076171875f,-0.491100030311942159499949411838315427303314208984375f,-0.39220000243485009239208238795981742441654205322265625f,-0.1820999873891471221298843374825082719326019287109375f,-0.0877999627709389329766054288484156131744384765625f,0.6003999836280942137278771042474545538425445556640625f,-0.110800001566112082951320871870848350226879119873046875f,0.51180002046451011921135432203300297260284423828125f,0.217799990189075032276377896778285503387451171875f,0.317999985784292338308887337916530668735504150390625f,0.5592000009298327967144359718076884746551513671875f,-1.000900012364983382440186687745153903961181640625f,-0.1541999997377399012066234718076884746551513671875f,0.3015999836489555718799238093197345733642578125f,-0.25340001437664039229957779753021895885467529296875f,-0.5152000235721467813476692754193209111690521240234375f,0.1334000087141991031103316345252096652984619140625f,0.7657000198245047517531247649458236992359161376953125f,-0.230200001704692791548723107553087174892425537109375f,0.40679999205917105609131567689473740756511688232421875f,0.3403999922335148031748985886224545538425445556640625f,-0.44719999535679832991519333518226630985736846923828125f,-0.77700002114474775449792787185288034379482269287109375f,-0.1688999936044215388619704754091799259185791015625f,-0.3624999945610760665459793017362244427204132080078125f,-0.30659999814033511711386381648480892181396484375f,0.6927000042006372648728529384243302047252655029296875f,0.34829997064620243918398045934736728668212890625f,0.5087000077322125779488715124898590147495269775390625f,-0.920899962855875831024832223192788660526275634765625f,-0.6937999788939952505728570031351409852504730224609375f,0.425599997067450885879225097596645355224609375f,-1.0105000105202197602238811668939888477325439453125f,-0.19409997191429084750780020840466022491455078125f,0.16169998811483399236976765678264200687408447265625f,-0.3444000051677225116009140037931501865386962890625f,1.2491000086456536255496985177160240709781646728515625f,-0.388700031685829117122921161353588104248046875f,0.40279999123215670664421850233338773250579833984375f,0.1108999684810640928844804875552654266357421875f,0.2555999925412237860911091047455556690692901611328125f,-0.059499999146908510994080643285997211933135986328125f,-0.48479998834878201829923227705876342952251434326171875f,-0.74759996464252509440484573133289813995361328125f,0.154200016427040065281062197755090892314910888671875f,-1.2030999547481542322202585637569427490234375f,-1.064399986466765479775631320080719888210296630859375f,-0.276699976119399337903814739547669887542724609375f,0.80200001771748052892263558533159084618091583251953125f,-0.621400018191337721873424015939235687255859375f,0.77259999042153371107133352779783308506011962890625f,-0.839800019901991046111788818961940705776214599609375f,0.301599995905160955800283772987313568592071533203125f,-0.2477000199586154582931385448318906128406524658203125f,-0.587600020521879429225009516812860965728759765625f,0.1318999710440635908526019193232059478759765625f,-0.3591999946713446689727788907475769519805908203125f,0.07709999778270759662746058893389999866485595703125f,0.58340000513792045211403092253021895885467529296875f,0.0329000100404031048384467794676311314105987548828125f,-0.2018000001430511058941874580341391265392303466796875f,-0.19500003058463277749723374654422514140605926513671875f,0.7589999858140945576678859652020037174224853515625f,0.90609999969005539099953239201568067073822021484375f,0.10679999030828479789079210604541003704071044921875f,0.523500003159046212175553591805510222911834716796875f,-0.7888000206172467443366258521564304828643798828125f,-2.0239000044822699209134952980093657970428466796875f,-0.65140000001192088774359945091418921947479248046875f,0.774899974039197747543994410079903900623321533203125f,-0.740799994450807464119179712724871933460235595703125f,-0.337100023785233238182712511843419633805751800537109375f,0.110600015172362342585898886682116426527500152587890625f,0.01650000291317699208093472407199442386627197265625f,-0.734000013679266150035118698724545538425445556640625f,-0.8303000143289569479065903578884899616241455078125f,0.29660000767707828117636381648480892181396484375f,0.94529999540448184092156225233338773250579833984375f,-0.716599983537197005034613539464771747589111328125f,0.3462000000953675549197896543773822486400604248046875f,-0.2154000296354292576239686241024173796176910400390625f,-0.2152999977886675964100504643283784389495849609375f,0.3088999948710202669843738476629368960857391357421875f,-1.03710000255107814837174373678863048553466796875f,-0.456800008189678141690137636032886803150177001953125f,-0.4459999766349795891073881648480892181396484375f,-0.366700026559829694861036841757595539093017578125f,0.38719999580383301918118377216160297393798828125f,0.8080999908089638861241610356955789029598236083984375f,-0.08329998677670946705120513797737658023834228515625f,1.038799986530840602227954150293953716754913330078125f,-0.06700002598762466732296161353588104248046875f,-0.16860003202557560797458791057579219341278076171875f,-0.305199988591670745563533273525536060333251953125f,-1.79239999050497988974939289619214832782745361328125f,0.3389000187873847380615188740193843841552734375f,0.1278999768853186225214813021011650562286376953125f,-0.9228999671995641307375990436412394046783447265625f,-1.267399991101026301976162358187139034271240234375f,0.2179000382781026701195514760911464691162109375f,-0.3685999815478921970424153187195770442485809326171875f,0.72119999148994651250177412293851375579833984375f,-0.3896999888375400900031308992765843868255615234375f,-0.1850999953389163010797346942126750946044921875f,0.644600001358985874588825026876293122768402099609375f,0.003599985682964046418419457040727138519287109375f,-0.0009999669492246621160802533267997205257415771484375f,-0.3589999697208412499094265513122081756591796875f,-0.8140999853253365614591530174948275089263916015625f,0.33110001056790405726815151865594089031219482421875f,-0.47720000206232082717860976117663085460662841796875f,-0.765699962604046380221234358032234013080596923828125f,0.382599983119964459632456055260263383388519287109375f,-0.52650004571676323195106306229718029499053955078125f,-0.40260002275705364382929474231787025928497314453125f,0.3641999994397162598858130877488292753696441650390625f,-0.3024000019788741777659879517159424722194671630859375f,-1.1303000192463397155506754643283784389495849609375f,-0.52740000376701345885521732270717620849609375f,1.069400019323825556938345471280626952648162841796875f,-0.130800006222724807258828150224871933460235595703125f,-0.70690002681165975406685220150393433868885040283203125f,-0.51080002429038307010245034689432941377162933349609375f,0.1220000032447277060754231570172123610973358154296875f,-0.281800005656480667681762497522868216037750244140625f,-0.15050004814565209887433638868969865143299102783203125f,-0.44020001213550574714616914206999354064464569091796875f,0.37170001694858090246498250053264200687408447265625f,-0.1974000000417230893390296841971576213836669921875f,-1.827800031092762456097489121020771563053131103515625f,-0.088200002017617185057218875954276882112026214599609375f,0.7691999974653125349277615896426141262054443359375f,-0.621400009399652564212601646431721746921539306640625f,-0.497600028754770828953724048915319144725799560546875f,-0.2912000019207596768211487869848497211933135986328125f,0.7101999756090340287784101747092790901660919189453125f,-0.535600014352798581285242107696831226348876953125f,0.5490000119656317512095711208530701696872711181640625f,-0.7835999612450599283164365260745398700237274169921875f,0.32760000040531167542212642729282379150390625f,0.2092999697446824569624368450604379177093505859375f,0.81259996911287402099333121441304683685302734375f,0.2471999747931956648017148836515843868255615234375f,-1.163000009179114613289129920303821563720703125f,-0.9907000108838079910356100299395620822906494140625f,0.13600001552701002083267667330801486968994140625f,0.4955000286996364966540795649052597582340240478515625f,-1.22199997413158456538440077565610408782958984375f,-1.158700011271238228260926916846074163913726806640625f,0.569800018858909851360294851474463939666748046875f,-0.146400009399652475394759676419198513031005859375f,-0.556900002001225136183393260580487549304962158203125f,-0.05940001910030812126706223352812230587005615234375f,0.3045000204741956650877909851260483264923095703125f,-0.3810000070333481314577284138067625463008880615234375f,-0.663299987968802184923333697952330112457275390625f,-1.544600034365057883434246832621283829212188720703125f,0.55510003676414498841040767729282379150390625f,0.5958000099107623181993176331161521375179290771484375f,-1.058999987266957987230853177607059478759765625f,-0.6363999957278370089852614910341799259185791015625f,-0.1614999879002567695351899601519107818603515625f,0.3209000031262636554174605407752096652984619140625f,-0.381899994774162809818562891450710594654083251953125f,-0.27009998058676709664638337926589883863925933837890625f,-0.090800003242492266508634202182292938232421875f,-0.5462000163376330430509142388473264873027801513671875f,-0.28140004709959054451928750495426356792449951171875f,-0.19220001845359835357385236420668661594390869140625f,0.8305000111162659237606931128539144992828369140625f,0.475199978905916253779651015065610408782958984375f,0.32300001052021887204546146676875650882720947265625f,-0.13299999889731406454984607989899814128875732421875f,-0.23020002286434237959156234865076839923858642578125f,-0.2804000174164773540042006061412394046783447265625f,-0.6684000216782095549206133000552654266357421875f,-0.1385999962627888493216232745908200740814208984375f,1.419500020623206903991331273573450744152069091796875f,-0.289199992883205414528902110760100185871124267578125f,-0.8219000117242336589384876788244582712650299072265625f,0.7970000233054161942192195056122727692127227783203125f,1.02300004553794909867292517446912825107574462890625f,-0.45440000904202459963698856881819665431976318359375f,-0.3357999945998191293483614572323858737945556640625f,0.30800001990795156103786212042905390262603759765625f,0.485899998432397950409722398035228252410888671875f,0.17020000066161156127009235206060111522674560546875f,-0.450599971809983568249435847974382340908050537109375f,0.55340001139640815353004654753021895885467529296875f,-0.0003999774813649992211139760911464691162109375f,-0.3825999872922896116733681992627680301666259765625f,-0.43159999635219481461945179034955799579620361328125f,-0.04799999248981468014108031638897955417633056640625f,-0.26359997927546441331969617749564349651336669921875f,0.24400002887845051446902289171703159809112548828125f,1.0615000066384681076669949106872081756591796875f,1.519300026071071396671641196007840335369110107421875f,-1.4326000349760050767145003192126750946044921875f,-0.43830002663731537637659130268730223178863525390625f,-0.1013000082492823139546089805662631988525390625f,0.70920001761913287197103272774256765842437744140625f,0.0354999960660933311373810283839702606201171875f,-0.512000018686056268535367053118534386157989501953125f,-1.065799997282027788969571702182292938232421875f,0.39310002478361116828864396666176617145538330078125f,-0.8485000100135806633261381648480892181396484375f,-0.726299989026784942325321026146411895751953125f,0.743200002837181017412149230949580669403076171875f,0.7261000062465665649824586580507457256317138671875f,0.2147999784767620212733163498342037200927734375f,-0.3793999933958052128701865512994118034839630126953125f,0.14250002026557950784990680404007434844970703125f,0.47490001181364061011436206172220408916473388671875f,0.83340000305175720995976007543504238128662109375f,-0.168299981486797367580265927244909107685089111328125f,0.3293999932467936986313361558131873607635498046875f,0.34010000278949725771582279776339419186115264892578125f,0.309599988245963686495088040828704833984375f,-0.3759000008165838124796209740452468395233154296875f,-0.56159996297359526806758367456495761871337890625f,-1.095499993234872793124168310896493494510650634765625f,0.02720001904964419736643321812152862548828125f,0.62450001645088182300469270558096468448638916015625f,0.8160999973058693512939498759806156158447265625f,-1.021299986717104868461092337383888661861419677734375f,-0.295500006478279619859250715308007784187793731689453125f,-0.28740000272393206870447102119214832782745361328125f,0.5316999541401863371703484517638571560382843017578125f,-0.1933999932169914170998481495189480483531951904296875f,-0.8773999998927113352920059696771204471588134765625f,0.06659999232888225151327787898480892181396484375f,1.050099997872113721797404650715179741382598876953125f,0.67060000392198571006474594469182193279266357421875f,-0.2627999660491944222684423948521725833415985107421875f,0.5707999987721443790888997682486660778522491455078125f,0.4362000103771688230835934518836438655853271484375f,0.30260000100135808764889588928781449794769287109375f,1.119699995952844862046049456694163382053375244140625f,-0.747999985784292054091793033876456320285797119140625f,-1.4751000018954272263727034442126750946044921875f,0.0634000063300135252575273625552654266357421875f,0.9484000315129763603039236841141246259212493896484375f,0.2394000060617924585670834858319722115993499755859375f,-0.39999999940395358777323053800500929355621337890625f,-0.01079999623894689619874043273739516735076904296875f,-0.517999993085860754860050292336381971836090087890625f,-0.31979999591112129309067313442938029766082763671875f,0.17889998667538176135849425918422639369964599609375f,-0.352399998775124512206957660964690148830413818359375f,-1.388100010143219709135564698954112827777862548828125f,-0.33720000995993615333645720966160297393798828125f,0.7735999703347682387999384445720352232456207275390625f,0.3761999936878679395846347688348032534122467041015625f,0.5606000241875648271161480806767940521240234375f,0.8181999979011715939947180231683887541294097900390625f,-1.084099964165687879358301870524883270263671875f,-0.64089998368024847508195307455025613307952880859375f,-0.59199999007582704280139296315610408782958984375f,1.1258000217199326531414271812536753714084625244140625f,-1.110300002706051003542597754858434200286865234375f,-0.269900003767013618727332868729718029499053955078125f,0.090999987959861527997418306767940521240234375f,0.951699990200996381872755591757595539093017578125f,-0.29160000872015956474569975398480892181396484375f,0.9121999968469143293958723006653599441051483154296875f,-0.103800005269050643619266338646411895751953125f,-0.703899974530935335081949233426712453365325927734375f,0.010500042855738911384833045303821563720703125f,-0.417000006020069147183448876603506505489349365234375f,-0.97819996051788393742754124104976654052734375f,-0.569399982368945956068273517303168773651123046875f,1.2956999973237515177970635704696178436279296875f,-0.1381000104784966264759304976905696094036102294921875f,0.17150000786781305350814363919198513031005859375f,0.3215000148713589300797366377082653343677520751953125f,-0.216999982774257915707494248636066913604736328125f,-1.3647000211358071286582571701728738844394683837890625f,-0.25869997021853929908274949411861598491668701171875f,0.3546999987840653290049885981716215610504150390625f,-0.91449998486042005652052466757595539093017578125f,-0.222400005406141243469164692214690148830413818359375f,0.703899961417913999639495159499347209930419921875f,0.098199986743927158983069602982141077518463134765625f,-0.0266999970555303889341303147375583648681640625f,-0.1270000000596045452994076185859739780426025390625f,-0.7350000050663947348539295489899814128875732421875f,-0.407400010025501302379780099727213382720947265625f,-0.7404000107109549588813024456612765789031982421875f,0.7332000102877618274987980839796364307403564453125f,1.180299995404481716576583494315855205059051513671875f,-0.555200011092424450254156909068115055561065673828125f,0.554099989050627073083887808024883270263671875f,-0.7576000059187411039829385117627680301666259765625f,-1.45540000773072275563890798366628587245941162109375f,0.58660000216364860392559421597979962825775146484375f,0.55420000405907643425962305627763271331787109375f,0.31759999131560334717505611479282379150390625f,0.666799958419800109510333641082979738712310791015625f,0.319199994969368017194710773765109479427337646484375f,1.936100001478195498094692084123380482196807861328125f,0.1128000097095966935967226163484156131744384765625f,-1.308599992537498035716225786018185317516326904296875f,0.401400006866455161258500083931721746921539306640625f,0.2643000026762489795828514616005122661590576171875f,0.73739997202754015148684629821218550205230712890625f,0.9246999682366843220648888745927251875400543212890625f,-0.2476000226080417032648028907715342938899993896484375f,0.57059997024536102117053815163671970367431640625f,-0.2537999908149242145327662001363933086395263671875f,-0.252600039148330068883296917192637920379638671875f,0.3483999975383280656160422950051724910736083984375f,1.20250000357627850888775355997495353221893310546875f,-1.202900007730722364840403315611183643341064453125f,0.29780002148151396568209747783839702606201171875f,0.081100017124414502944773630588315427303314208984375f,-0.299599993908405082265744567848742008209228515625f,1.4609000017106532620658754240139387547969818115234375f,-0.3412000167846684917094535194337368011474609375f,0.13139999494552601078112274990417063236236572265625f,-1.0272999816060064404155127704143524169921875f,-1.714499991714954152399741360568441450595855712890625f,-0.046299955797195480045047588646411895751953125f,1.12400000011920919718022560118697583675384521484375f,0.01249997317790985107421875f,1.52909996371865286590718824299983680248260498046875f,-0.62069999687671639776453957892954349517822265625f,0.8974999803304672951753673260100185871124267578125f,-0.641000003457069311707527958787977695465087890625f,-1.07880000780224793999195753713138401508331298828125f,1.2775000035762786865234375f,1.38069999359846118380801271996460855007171630859375f,-0.0646000052332880159156047739088535308837890625f,1.788699970144033812857742304913699626922607421875f,-0.1545000325441350241817417554557323455810546875f,0.4995999629139902253882610239088535308837890625f,-1.0688999757230277509734150953590869903564453125f,-0.2348000145375730429719851599656976759433746337890625f,-0.056299987089634129233672865666449069976806640625f,0.5387000047147274539582895158673636615276336669921875f,-0.19890002653598770621101721189916133880615234375f,-0.22719999639987964457077396218664944171905517578125f,0.42650000935792942158286678022705018520355224609375f,-0.01099999973177911982702426030300557613372802734375f,0.15119998400211276390336934127844870090484619140625f,-0.6042000140428545051918263197876513004302978515625f,-0.2898000143885610668803565204143524169921875f,-0.4031999953866005625968682579696178436279296875f,-1.14030000345110860138220232329331338405609130859375f,-0.83519998486638069579157672706060111522674560546875f,-0.6039999960958954972056744736619293689727783203125f,0.3430000024735928576546939439140260219573974609375f,0.2705999738216409156166264438070356845855712890625f,-0.0036000098228452515058961580507457256317138671875f,0.810400026804208639674698133603669703006744384765625f,0.376400004631280982181351646431721746921539306640625f,-0.20229997743368155482812653644941747188568115234375f,0.17019999693632126280817828956060111522674560546875f,-0.62340000111460636134097512695007026195526123046875f,0.4921999980390072693126057856716215610504150390625f,-0.23279998944401736338249975233338773250579833984375f,0.37660000872015952921856296597979962825775146484375f,0.76330000547766696517015816425555385649204254150390625f,0.43539999789595607015968425912433303892612457275390625f,0.59400001390278323043503405642695724964141845703125f,0.46099999168515193304074273328296840190887451171875f,1.15329998282790224806149126379750669002532958984375f,-0.40439999533295623024287124280817806720733642578125f,0.09249998897314082313414473901502788066864013671875f,-0.298399995005130591607667156495153903961181640625f,-0.86930000669956175585184610099531710147857666015625f,0.3099000133037568271987538537359796464443206787109375f,0.59669998960495007622739649377763271331787109375f,-0.96940002811551106987053572083823382854461669921875f,-0.29839999172687470263554132543504238128662109375f,-1.772500007897615414975689418497495353221893310546875f,0.518800011825561568912235088646411895751953125f,0.044400043463706850843664142303168773651123046875f,-0.2062999814271933018972049467265605926513671875f,0.5961999696969986217709447373636066913604736328125f,-0.22909999754428866935995756648480892181396484375f,-0.130000002533197100973438864457421004772186279296875f,-1.0051999650478364856098778545856475830078125f,-0.6648999936342239180930846487171947956085205078125f,1.1856999941945076670890557579696178436279296875f,0.008799972486495999390854194643907248973846435546875f,0.5610000010728839470175444148480892181396484375f,0.032999983698129842224489038926549255847930908203125f,0.14849999988079076729263761080801486968994140625f,1.377900015479326167877616171608678996562957763671875f,0.1478999783754346708519733510911464691162109375f,0.485499987199902616197277893661521375179290771484375f,0.1619999888092280571072478778660297393798828125f,0.94399996383488182782883768595638684928417205810546875f,0.6876999692201621883214102126657962799072265625f,0.3686000040113928122309516766108572483062744140625f,0.61710001506805411963796359486877918243408203125f,-1.1088000136137008411196802626363933086395263671875f,0.14720003871917697324533946812152862548828125f,-0.9477000031948090708056042785756289958953857421875f,0.0136999674618245581569908608798868954181671142578125f,0.5712000039696694120294750973698683083057403564453125f,0.32600001396238820206008313107304275035858154296875f,0.380200008484721241330817065318115055561065673828125f,0.583899993790686355765728876576758921146392822265625f,0.56420002149343506658851765678264200687408447265625f,0.72070002124011534050396221573464572429656982421875f,-0.073200001421570792814463857212103903293609619140625f,-1.082399990691244706653151297359727323055267333984375f,-0.373600027853250526987949342583306133747100830078125f,-0.2460000737532976045063293213388533331453800201416015625f,-0.1315999912485480705104379239855916239321231842041015625f,0.7520000241436066712452657156973145902156829833984375f,0.28139999896883960417426351341418921947479248046875f,0.02300002191960803799020141013897955417633056640625f,0.09679999433159824917538571753539144992828369140625f,0.7948999897599211550414111115969717502593994140625f,-0.0881999940454960551505791954696178436279296875f,0.18170003587305583181432666606269776821136474609375f,-0.4610000040531156972889448297792114317417144775390625f,-1.5229000280708071546342807778273709118366241455078125f,-0.0446000116407869739987290813587605953216552734375f,1.631999970257282939201104454696178436279296875f,-1.05879999722242335025157444761134684085845947265625f,-1.3941000714540479066272382624447345733642578125f,-0.501000017464161118141419137828052043914794921875f,1.3065999663263563324022697997861541807651519775390625f,-0.5028000026315451176373017005971632897853851318359375f,-1.18610001805573705979668375221081078052520751953125f,-0.2413000146567820802800952151301316916942596435546875f,0.2967999889671801838630926795303821563720703125f,1.2641000058889388935057240814785473048686981201171875f,0.29849998654425125010902775102294981479644775390625f,0.8597000159204004621216199666378088295459747314453125f,0.1959999927282325415944796986877918243408203125f,-0.12719998388290409963019556016661226749420166015625f,0.01520000133216381499323688331060111522674560546875f,0.427899966305494483531646210394683293998241424560546875f,-1.3598999778389924131261068396270275115966796875f,-0.16899998512491609670149728117394261062145233154296875f,-1.221899998704344625366502441465854644775390625f,-0.70829999823197720143497235767426900565624237060546875f,0.4497999862253667657796540879644453525543212890625f,0.94620001633763328729997965638176538050174713134765625f,0.53529998095035580263356678187847137451171875f,-0.291900005912780979855369878350757062435150146484375f,0.4712999920070171100405787001363933086395263671875f,-0.4345000154823062121778320943121798336505889892578125f,0.5198000349521638696614900254644453525543212890625f,1.088499964341521408783819424570538103580474853515625f,-1.17829997165203081976869725622236728668212890625f,-1.10840000200867638824320238200016319751739501953125f,-0.113999935746193159502581693232059478759765625f,0.15689998177289954384150405530817806720733642578125f,-1.220599998855590229140943847596645355224609375f,-1.4666000053301451178100478500709868967533111572265625f,0.521300017674267479605987318791449069976806640625f,0.370499988280236614013318785509909503161907196044921875f,-0.182900026841462004512095518293790519237518310546875f,0.5823999967262147858804155475809238851070404052734375f,0.7597000010751184806423452755552716553211212158203125f,-1.0584000204861165883585272240452468395233154296875f,-0.525900053864717165197362191975116729736328125f,0.2608000230610372227602056227624416351318359375f,1.361000005692243775001770700328052043914794921875f,-0.8177999894440173278553629643283784389495849609375f,-2.0720000151097774931940875831060111522674560546875f,0.7369999996125695673754307790659368038177490234375f,1.39100002565979874447066322318278253078460693359375f,0.51559998440146426190722195315174758434295654296875f,-0.3197000077247622584764030762016773223876953125f,-0.266800009977817609296835144050419330596923828125f,0.939699994909763081096798487124033272266387939453125f,0.067200003403425245807056853664107620716094970703125f,-1.3320000123530622015977087357896380126476287841796875f,0.682999992787837850727328259381465613842010498046875f,0.667400017252564481395893380977213382720947265625f,-0.019000000044703480028829289949499070644378662109375f,-0.8403000188365579159466278724721632897853851318359375f,-0.4328000235676767726289426718722097575664520263671875f,0.0308999963462348592457828999613411724567413330078125f,0.6667999912023543362948885260266251862049102783203125f,-0.2070999925673009389726075824000872671604156494140625f,0.1367999865829945260742306345491670072078704833984375f,0.0962999809801576844847659231163561344146728515625f,1.0153999947667120551386688021011650562286376953125f,-0.057599993997812237722655481775291264057159423828125f,0.284099980929493955983389241737313568592071533203125f,0.639299964044988566769234239473007619380950927734375f,-0.1421000301182269165423122103675268590450286865234375f,-0.3196999791145316294205258600413799285888671875f,0.45290001413822178477630586712621152400970458984375f,0.3634999969005576758718234486877918243408203125f,-1.672199980902671967442074674181640148162841796875f,0.17450004267692520443233661353588104248046875f,1.43390003711581215384285314939916133880615234375f,-0.0067000682830808955259271897375583648681640625f,0.24489995921254159583213549922220408916473388671875f,0.33589996192455373602570034563541412353515625f,0.99659999665021903325623497948981821537017822265625f,-0.77200000154972059362989966757595539093017578125f,-1.293800032687187240298953838646411895751953125f,0.4289000099956989942029395024292171001434326171875f,0.752299990770220805558210486196912825107574462890625f,-0.2690000343918796943398774601519107818603515625f,-0.228600011610984754639730454073287546634674072265625f,-0.00149999393522737278772183344699442386627197265625f,-0.070300001960992786198545445586205460131168365478515625f,-1.03020000542998246828574337996542453765869140625f,0.0366999744057656673845713157788850367069244384765625f,0.489799964469671778033443843014538288116455078125f,0.69739996711015717067994046374224126338958740234375f,-0.798499987512826692181988619267940521240234375f,0.1705999763548373682198189271730370819568634033203125f,0.2185999549865726976349833421409130096435546875f,1.4106000201642512337230073171667754650115966796875f,0.63750001683831225562926192651502788066864013671875f,-0.60159999352097504043968001496978104114532470703125f,-0.32399998685717523727589650661684572696685791015625f,0.07680000774264339469254991854541003704071044921875f,-0.08519996460080125899594349903054535388946533203125f,0.02340000081658377695248418604023754596710205078125f,-0.330800016951560582612046346184797585010528564453125f,0.37320001580119122053247338044457137584686279296875f,-0.6905999729275702492259370046667754650115966796875f,-0.13739999572038641417748294770717620849609375f,-0.4488999931573867119283249849104322493076324462890625f,0.0014000012040138454239013299229554831981658935546875f,0.52340000111460671661234300700016319751739501953125f,0.70839999232292161224222581950016319751739501953125f,-0.60009996121525777112992727779783308506011962890625f,0.51019998739957816979995186557061970233917236328125f,0.00950000978261211770359295769594609737396240234375f,-0.0611999996110796917747620682348497211933135986328125f,0.2631999775238333949545221912558190524578094482421875f,-0.391800022420287330504606870817951858043670654296875f,-0.733899987532198760931123615591786801815032958984375f,0.5104000141382218913577162311412394046783447265625f,1.05630003104806036162699456326663494110107421875f,-1.0489999908804890083047212101519107818603515625f,0.031099957966805646947250352241098880767822265625f,-0.2351999900817869360025724745355546474456787109375f,-0.1726000368386504391082780784927308559417724609375f,0.305599990436434598706227916409261524677276611328125f,0.516800008003413413604221204877831041812896728515625f,-0.88860001220703122015720509807579219341278076171875f,-0.07749996304512052347490680404007434844970703125f,-0.734199983048438920008038621745072305202484130859375f,1.1280999970674514543134137056767940521240234375f,-0.7904000379800795172968719271011650562286376953125f,0.819100027048587975286864093504846096038818359375f,0.025500032126903438012277547386474907398223876953125f,1.3504999638795851524264435283839702606201171875f,0.24450000244379044289644298260100185871124267578125f,-0.41759998133182563151422073133289813995361328125f,0.210400011754035887179270503111183643341064453125f,-0.3807999980270864170961431227624416351318359375f,-0.4481000101804732960175670086755417287349700927734375f,0.731999960720539633030057302676141262054443359375f,0.4154999683499338569703240864328108727931976318359375f,-0.16559999885559051335803815163671970367431640625f,0.559100002834200804358033565222285687923431396484375f,-0.095099999175965788111852816655300557613372802734375f,-0.8912000048637389593153557143523357808589935302734375f,-0.32790000378191397345517543726600706577301025390625f,0.5134999812543392661012831013067625463008880615234375f,0.279999997913837006535686668939888477325439453125f,-0.2391000112533567811823331794585101306438446044921875f,-0.1323999179363255507269059307873249053955078125f,-0.5453999922335146521845672396011650562286376953125f,0.7548000254154203503276221454143524169921875f,0.3964000037372112927869238774292171001434326171875f,0.40499996803700977210382916382513940334320068359375f,-0.1382999838709830786598331542336381971836090087890625f,-0.5065000099539755940014629231882281601428985595703125f,0.3213999988198279478268659659079276025295257568359375f,1.170500026911497126747008223901502788066864013671875f,-0.54140002847313883904689646442420780658721923828125f,-1.5613000078395018466181909388978965580463409423828125f,0.5159999901577829728438473466667346656322479248046875f,0.45779998797252741127294939360581338405609130859375f,-0.430200020293891594036495007458142936229705810546875f,-0.9053000679545111228918585766223259270191192626953125f,-0.16300002020597492702336239744909107685089111328125f,1.245500038534402609258222582866437733173370361328125f,-0.472200007575750202448716663639061152935028076171875f,-0.0535000363886357632736689993180334568023681640625f,0.52659999620318398427798456395976245403289794921875f,-0.981999997675418345721709556528367102146148681640625f,-0.228399991428851922847798050497658550739288330078125f,0.13070001356601768094378712703473865985870361328125f,-0.51980000813007354310002483543939888477325439453125f,-0.39910003468543298055237755761481821537017822265625f,-0.2204000028505921360277852727449499070644378662109375f,0.5833999877594404637193292728625237941741943359375f,-0.44080000145435338509969369624741375446319580078125f,0.4885999834477896541784502915106713771820068359375f,-0.65920000733733186137897064327262341976165771484375f,-0.676700028496981520476083460380323231220245361328125f,0.192200004446506422794982427149079740047454833984375f,0.6023000080555684920824432992958463728427886962890625f,0.974800013047457003523277307976968586444854736328125f,0.0856999874889847745862425654195249080657958984375f,1.03160000618696212626446140347979962825775146484375f,-0.225400023525953230318918940611183643341064453125f,-1.3323000115573402357682653018855489790439605712890625f,0.2891000213861474321674904786050319671630859375f,0.59019997113868594151853130824747495353221893310546875f,-0.01050001974031289364575059153139591217041015625f,0.1310999824605882657646560573994065634906291961669921875f,-0.550199999879299905813923032837919890880584716796875f,0.32380000184178336919416096861823461949825286865234375f,-1.3544000351190561559633351862430572509765625f,-0.84550003495812386944407990085892379283905029296875f,0.27370003675222420724821859039366245269775390625f,-0.718200029808282902621385801467113196849822998046875f,0.94409997072219908886836492456495761871337890625f,0.255599996918439931192779113189317286014556884765625f,-0.8087999656319624364186893217265605926513671875f,-0.459899989461898694997898928704671561717987060546875f,-0.05939998087882969457496074028313159942626953125f,0.41150002194941048827558915945701301097869873046875f,-0.298900001241266810581009849556721746921539306640625f,0.11550001581013191742641765813459642231464385986328125f,-0.21159999933242801262167631648480892181396484375f,-0.24849997812509538874792269780300557613372802734375f,-0.0966999973535536128110834397375583648681640625f,0.684199981558322622987589056720025837421417236328125f,-0.6246000170052052880009796353988349437713623046875f,0.6142999948531391307682270053192041814327239990234375f,0.168099996769427662002271972596645355224609375f,0.648700019168853447837364001316018402576446533203125f,-1.95419998081326440342309069819748401641845703125f,0.7169999960362911650690875831060111522674560546875f,0.9804999753534797690690538729541003704071044921875f,0.0827999787151814103935976163484156131744384765625f,0.0768000057309858874532437766902148723602294921875f,0.281399996659159601364308400661684572696685791015625f,0.040100018621980648703129190835170447826385498046875f,-0.0569999939501286689846892841160297393798828125f,0.0468999787926680511418453534133732318878173828125f,-0.00839999947547909187051118351519107818603515625f,-0.53700000065565112183918472510413266718387603759765625f,-0.56300000843405728634394336040713824331760406494140625f,0.5544000095635652769487933255732059478759765625f,0.12399999445676801457238980219699442386627197265625f,-0.86600003311038076247996286838315427303314208984375f,-0.40260000800490391981156790279783308506011962890625f,0.313299997207522107345312178949825465679168701171875f,-0.1705999993771314393598004244267940521240234375f,0.9772999767258763181843050915631465613842010498046875f,-0.292999992936849640301488761906512081623077392578125f,-1.0688999822422864394155794798280112445354461669921875f,-0.5154000041916966257105059412424452602863311767578125f,-0.0674000025935475033378452280885539948940277099609375f,-0.0388000021398067218569849501363933086395263671875f,1.092399983948469266437086844234727323055267333984375f,-0.330199996638298021167656770558096468448638916015625f,-0.6686999894782903464118817282724194228649139404296875f,0.4435999968215822431005790349445305764675140380859375f,-0.9289999702610074461262001932482235133647918701171875f,1.49199997860193267484874013462103903293609619140625f,-0.00970002083778354062815196812152862548828125f,-0.1480000137984749386532712378539144992828369140625f,-0.60049998652934988285778672434389591217041015625f,-0.35039999729990967125559109263122081756591796875f,0.39189998274147530565869601559825241565704345703125f,0.493799983364343442193700184361659921705722808837890625f,0.38109998657703414437492028810083866119384765625f,-0.380700003582238988286690073437057435512542724609375f,-0.058999992072582330138175166212022304534912109375f,-0.2485999945491552676202218208345584571361541748046875f,-0.75519998210966576834834995679557323455810546875f,-0.62510000465214232701072205600212328135967254638671875f,1.1373999876737599379339371807873249053955078125f,0.77630000765323625699920739862136542797088623046875f,-0.9343000031232833180183661170303821563720703125f,0.046399992561340308583339719916693866252899169921875f,-0.28089999373853213882057389128021895885467529296875f,-1.1173999729961157090230017274734564125537872314453125f,0.5218999998033044818157577537931501865386962890625f,0.2267000205993652262037585387588478624820709228515625f,-0.604199983830750131374998090905137360095977783203125f,0.19119998847246166828739433185546658933162689208984375f,-0.253699996519088699642452411353588104248046875f,-0.988699985194206210081802055356092751026153564453125f,0.5264000102937220759713454754091799259185791015625f,-0.3739000110387802333633544549229554831981658935546875f,0.569499978564679931736236540018580853939056396484375f,1.9502999831855298840110890523646958172321319580078125f,-0.19120002885460873898182398988865315914154052734375f,0.3857999917685983604087596177123486995697021484375f,0.471199964183568908993038348853588104248046875f,0.208199995237588808549844543449580669403076171875f,0.07640000813305380944484568317420780658721923828125f,-0.417800007101893466909814378595910966396331787109375f,0.197999965928495225142569324816577136516571044921875f,0.40920000733733186137897064327262341976165771484375f,0.9560999876201154989985298016108572483062744140625f,0.6738000075042247516421412001363933086395263671875f,0.66500004872679685519187842146493494510650634765625f,0.02020000215172768065485797706060111522674560546875f,0.1686999939858910835255301208235323429107666015625f,0.64199999603629098743340364308096468448638916015625f,-0.0326999672085050985259613298694603145122528076171875f,-0.8168000216007234026704963980591855943202972412109375f,-1.161900020515918896535367821343243122100830078125f,0.7862000193178655393921872018836438655853271484375f,0.48900002619624149957644476671703159809112548828125f,0.32939999235272399147333999280817806720733642578125f,1.296000008970498829086182013270445168018341064453125f,-0.747200000423192900456115239649079740047454833984375f,-0.765500007838010976257692163926549255847930908203125f,-0.103799990814924303350608170148916542530059814453125f,-0.51369998467266542174769483608542941510677337646484375f,0.45840000714957718219721982677583582699298858642578125f,-0.18519998624473787973698790665366686880588531494140625f,0.60360000714063655191665702659520320594310760498046875f,1.2521000312358143702340385061688721179962158203125f,0.7854999792277812531438030418939888477325439453125f,0.1512999593734747350026736967265605926513671875f,0.23749997928738597607889460050500929355621337890625f,-0.52770001094341267844356480054557323455810546875f,0.55569999061822894503848146996460855007171630859375f,-0.58769999015629270644467396778054535388946533203125f,0.566699987965822327140585912275128066539764404296875f,-0.06370000605583214792204671539366245269775390625f,0.243700017678737612669692680356092751026153564453125f,0.596099968099593979786732234060764312744140625f,0.42950000110268593545015392010100185871124267578125f,0.7244999943971635048001189716160297393798828125f,-0.25370000858902930218619076185859739780426025390625f,0.9561999612033371676034221309237182140350341796875f,-0.40210001491904245884967394886189140379428863525390625f,-0.1247999823510639316737069748342037200927734375f,0.3357999971330165589478156107361428439617156982421875f,0.325499988317488941902411170303821563720703125f,1.1029000080287456597716300166212022304534912109375f,0.08069997482299839930419693700969219207763671875f,-1.266700001376867223967792597250081598758697509765625f,0.343700037348270370785030536353588104248046875f,-0.630699985551834263475257102982141077518463134765625f,0.71209999152421943335866671986877918243408203125f,0.41749999575316909528055475675500929355621337890625f,0.46419998677372920070638429024256765842437744140625f,1.0870999971866606159665025188587605953216552734375f,-0.0547000089913609599534538574516773223876953125f,-0.01900001643598059342110673242132179439067840576171875f,0.78059998216629100653563000378198921680450439453125f,0.24820002667903917625835674698464572429656982421875f,0.081000003010034848927034545340575277805328369140625f,0.416799986881017492379442046512849628925323486328125f,0.14750000048428713039783133353921584784984588623046875f,-0.28239999147355565600747695498284883797168731689453125f,0.384500003058463046823334252621862106025218963623046875f,0.06040001235008229940603996510617434978485107421875f,-0.593099973374604960696387934149242937564849853515625f,0.545799991470575118768238098709844052791595458984375f,-0.22269998188614881229341335711069405078887939453125f,0.12439999965429304751296513131819665431976318359375f,0.77119999040961229042068225680850446224212646484375f,-0.5007999935567379878875726717524230480194091796875f,-1.536599984431267440498913856572471559047698974609375f,0.285800003689527404304726587724871933460235595703125f,-0.5036999319225550220124887346173636615276336669921875f,0.23379998828172698921434857766143977642059326171875f,0.757899957066773932723435791558586061000823974609375f,0.7268000239849090604593584430404007434844970703125f,0.077100037717820146809799553011544048786163330078125f,0.205799997878074680812687802244909107685089111328125f,0.724499989479780115431140075088478624820709228515625f,-0.57180002726316470074152675806544721126556396484375f,-0.84909999322295259815973622607998549938201904296875f,0.66280001507401475890901565435342490673065185546875f,0.2402000281542544524171489683794789016246795654296875f,-0.2000000004470348091700770964962430298328399658203125f,0.70500001329928674298486157567822374403476715087890625f,-0.38929998166561130545915148104541003704071044921875f,1.138000003218650135750067420303821563720703125f,0.22979999963641173366113434894941747188568115234375f,0.644099962377548962422224576584994792938232421875f,0.34460001529157147093229696110938675701618194580078125f,0.246399990139901436947411639266647398471832275390625f,-1.1351000037021936606151939486153423786163330078125f,0.7629999877214430625826935283839702606201171875f,0.14899999654293072381960882921703159809112548828125f,0.950299953532219632279520737938582897186279296875f,0.7646999926745892128820969446678645908832550048828125f,0.12659999098777774406698881648480892181396484375f,-1.2432999615192414921693853102624416351318359375f,0.3891000017166135194202070124447345733642578125f,-0.226599978768825760511163025512360036373138427734375f,-0.2383000239551069654453385737724602222442626953125f,1.2052999820679428655267884096247144043445587158203125f,-0.421199989664554408363983384333550930023193359375f,0.119799995613097731705920523381792008876800537109375f,0.18179997630119260065839625895023345947265625f,0.83390000269413011579899830394424498081207275390625f,-0.189800001573562582279919297434389591217041015625f,-1.112399998700618564129172227694652974605560302734375f,-0.0983999964952460004496970213949680328369140625f,-0.1805000233352185290414126939140260219573974609375f,0.680099988782405517895313096232712268829345703125f,0.55490000808835038270672157523222267627716064453125f,0.20160000790059573461121544823981821537017822265625f,-0.50699999920278771003978590670158155262470245361328125f,-0.257199979300796410797147473203949630260467529296875f,0.5986000083327294163382248370908200740814208984375f,1.0289999672621494841706635270384140312671661376953125f,-0.6548000016480683971309417756856419146060943603515625f,0.181199996779859429096859457786194980144500732421875f,0.28260001068711293470414602779783308506011962890625f,0.00129999275207470788018326857127249240875244140625f,-0.08160000320672988749493015347979962825775146484375f,0.605300014925003182497675879858434200286865234375f,0.29139999136924732425768524990417063236236572265625f,1.20289996004700672216358725563623011112213134765625f,0.42140002370476725701564646442420780658721923828125f,-0.4558999510467049987028076429851353168487548828125f,-0.5109999878108499871132153202779591083526611328125f,-1.2222000030681481330674387209000997245311737060546875f,-0.52339999273270354507303636637516319751739501953125f,-0.6046000134102997858320804880349896848201751708984375f,-0.371600019448995677606717435992322862148284912109375f,-0.26590004686117207910456272657029330730438232421875f,0.67639999494552593972684917389415204524993896484375f,0.939000008463859447971344707184471189975738525390625f,0.414799981307983234302128039416857063770294189453125f,0.8906999833166597824174459674395620822906494140625f,0.1400000043213367462158203125f,0.79869997804164949428695763344876468181610107421875f,0.5282000047743320436666181194595992565155029296875f,0.00480005804896332932685254490934312343597412109375f,-0.10600000420212740692704755929298698902130126953125f,-0.9859000125139949499697422652388922870159149169921875f,1.6868999837100506766773833078332245349884033203125f,0.19459998586177817969655734486877918243408203125f,-0.064499993950128509112573738093487918376922607421875f,-0.70199997018277660032481435337103903293609619140625f,0.301900029009580739280949046587920747697353363037109375f,0.78369999055862393788629560731351375579833984375f,0.304699987608194167165720500634051859378814697265625f,-0.73599999094009405098404386080801486968994140625f,0.57689997402429593620354353333823382854461669921875f,0.824599972748757181761902756989002227783203125f,-0.32829998580813413155965463374741375446319580078125f,1.2868999802827829626039601862430572509765625f,-0.65360000520348560115735381259582936763763427734375f,-0.18089995357990318325391854159533977508544921875f,-0.3951999733179807350325063453055918216705322265625f,-0.642799962435663285731379801291041076183319091796875f,-0.12800000753998752589524201539461500942707061767578125f,-0.86909999128580039950975333340466022491455078125f,0.74459998288154594092702609486877918243408203125f,-0.2640000466108318732949555851519107818603515625f,0.99539996228218097229500926914624869823455810546875f,0.0565999674439439104389748536050319671630859375f,0.593599963331222557627597780083306133747100830078125f,0.313099995577335477037195232696831226348876953125f,-0.5192999885201452503480368250166065990924835205078125f,1.01469997851848603431790252216160297393798828125f,-0.220700016397237686049948024447076022624969482421875f,0.7047000130891802882615593262016773223876953125f,0.0354000074699520883569903162424452602863311767578125f,1.1431999655842783880643764859996736049652099609375f,0.521399998223781580009017488919198513031005859375f,0.25950001034140601774424794712103903293609619140625f,-0.1567999958217143330330145545303821563720703125f,-0.325699957537651574313031233032234013080596923828125f,0.749399994587898188314056824310682713985443115234375f,0.400999988034367371714239425273262895643711090087890625f,-0.814999983087182133800752126262523233890533447265625f,-0.3659999900087711477425500561366789042949676513671875f,-0.2785999980509281659379894335870631039142608642578125f,0.70230000503808198697441866897861473262310028076171875f,0.01040002456903454941539166611619293689727783203125f,-0.81689997689276883097164727587369270622730255126953125f,-0.07659999970495701138162303323042578995227813720703125f,-0.55480003057494808871297209407202899456024169921875f,0.4156000006437299276740304776467382907867431640625f,0.741500004291534509093253291212022304534912109375f,0.853800012719631240543094463646411895751953125f,1.9605999829113489596466024522669613361358642578125f,0.043599999541044098805286921560764312744140625f,-0.23569999940991426257141938549466431140899658203125f,-0.20980000023245803930649344692938029766082763671875f,1.33150001293420761072638924815692007541656494140625f,0.23439998981952658851923843030817806720733642578125f,-1.029700007277727369370268206694163382053375244140625f,-0.502200003999471622506689527654089033603668212890625f,-0.757099995696544514345305287861265242099761962890625f,0.189599990034103438318879852886311709880828857421875f,0.1296000212520358463308411955949850380420684814453125f,-1.16440003679543746528679548646323382854461669921875f,-0.4182000013470645871649367109057493507862091064453125f,-1.552800032322108503279878277680836617946624755859375f,-0.41279998720884325535962489084340631961822509765625f,-0.2986000027075410567789504057145677506923675537109375f,-0.3371999902158975270793916934053413569927215576171875f,0.238300033100694419463394524427712894976139068603515625f,0.7999999696016313333046809930237941443920135498046875f,-0.23719999645575828850496691302396357059478759765625f,-0.54799998310208319907133045489899814128875732421875f,-1.1920999873146411740520989042124710977077484130859375f,-0.204599994355440184534700165386311709880828857421875f,-1.45900002995878441680588366580195724964141845703125f,-0.1799999900162221333488332675187848508358001708984375f,0.2975000093877315787693760285037569701671600341796875f,0.218600004160403926789513207040727138519287109375f,0.7289999940842388781447880319319665431976318359375f,-0.244999984502792589324826622032560408115386962890625f,0.1705000035911796718579580556252039968967437744140625f,-0.006799999621510510738886523540713824331760406494140625f,-0.9144999873936174861199788210797123610973358154296875f,0.235799991619586979396672177244909107685089111328125f,-0.9915999656558032171460581594146788120269775390625f,0.93339996550083181858781244955025613307952880859375f,-0.5026999952226873791261141377617605030536651611328125f,0.1623999905049802183754081852384842932224273681640625f,-0.0878999961078166247574472436099313199520111083984375f,0.4431999928534029464799459674395620822906494140625f,0.735800018404424438500655014649964869022369384765625f,0.1275000101327894430625065069762058556079864501953125f,-0.27289998530447501234874607689562253654003143310546875f,0.249300031808018796031234387555741705000400543212890625f,-0.41649999490380285038781948969699442386627197265625f,-0.196499995574355157401669202954508364200592041015625f,-0.14429999552369121573747179354541003704071044921875f,-0.357399998477101465965688475989736616611480712890625f,-0.00049998146295582301945614744909107685089111328125f,0.17590003687739363957831528750830329954624176025390625f,-0.185599977584183506440496103095938451588153839111328125f,-1.06499998085200786590576171875f,-0.998699975061416722610374563373625278472900390625f,-0.427699987101555212376524650608189404010772705078125f,0.0037000155925748234864158803247846662998199462890625f,1.27040002062022683304576275986619293689727783203125f,1.48129998336434365313607486314140260219573974609375f,1.091700002792477253166225636960007250308990478515625f,-0.329699982914328781902213449939154088497161865234375f,0.1614000102192163499381649671704508364200592041015625f,0.7917000013768671351499506272375583648681640625f,0.146499984100461000391391053199185989797115325927734375f,0.403299985472858057988787550129927694797515869140625f,0.5280999996751549740992004444706253707408905029296875f,0.545299986463785302248652442358434200286865234375f,-0.28910001579821109629619968472979962825775146484375f,-0.646499999672174308074090731679461896419525146484375f,-0.3273000191569328531926430514431558549404144287109375f,0.1215999824941160767366454820148646831512451171875f,-0.0661000271826981400380418563145212829113006591796875f,-0.9387000484868888161571476302924565970897674560546875f,-0.2379999978542326477537471873802132904529571533203125f,-0.663200024332106341518056069617159664630889892578125f,0.699599985414743219536148899351246654987335205078125f,-0.38480002984851646719022255638265050947666168212890625f,0.4093999799847605203240163973532617092132568359375f,-0.16260003199577344190629446529783308506011962890625f,-0.594799999040364735236607884871773421764373779296875f,0.43169999854564655716870902324444614350795745849609375f,0.41839997511208093516898998132091946899890899658203125f,1.02609998758286291575814175303094089031219482421875f,-0.04879998664259943552679033018648624420166015625f,-0.08420000886470102585690256091766059398651123046875f,-0.2989999962449072512526981881819665431976318359375f,-1.562800016936659375232920865528285503387451171875f,-0.2635999771893029475222647306509315967559814453125f,-1.2920000161156066642575979130924679338932037353515625f,0.75459996172189758301129813844454474747180938720703125f,-0.1222000114500521039673230916378088295459747314453125f,0.30799998071789769227279975893907248973846435546875f,0.41040000154674061150217312388122081756591796875f,-0.1991999975770710751277192684938199818134307861328125f,0.078400008416176003578357267542742192745208740234375f,0.34800000098347627641715007484890520572662353515625f,0.4125999995484945781498709038714878261089324951171875f,0.2543999956309794807651769588119350373744964599609375f,0.8927000535979867645863805591943673789501190185546875f,0.66619997640252126558380041387863457202911376953125f,0.3444000156357887032498865664820186793804168701171875f,0.978700010153651600575130942161194980144500732421875f,1.00809996681809455054690261022187769412994384765625f,0.38919999474585065257770111202262341976165771484375f,0.1461000154852865051680055330507457256317138671875f,0.1023000234782696082902475609444081783294677734375f,-0.381000001817941580384285771287977695465087890625f,0.3367000083059072945701473145163618028163909912109375f,0.4788999997138976283395095379091799259185791015625f,0.23739999803006650580528003047220408916473388671875f,-0.715800003093480796678704791702330112457275390625f,-0.253500016942620476356751169078052043914794921875f,0.6395000122785565732641543945646844804286956787109375f,0.5426000164613127285662130816490389406681060791015625f,0.07200000223889946315836141366162337362766265869140625f,0.030900009068101474962730890183593146502971649169921875f,0.11530002125799632128888561055646277964115142822265625f,-0.117799996745586554869333895112504251301288604736328125f,-0.01589999880492687367450344027020037174224853515625f,0.575600006902217575799340920639224350452423095703125f,1.0178000189483160653480808832682669162750244140625f,0.62240002395808691204592832946218550205230712890625f,-0.7373999924793839255698912893421947956085205078125f,0.107900012946128942559198549133725464344024658203125f,0.32640000075697894743598226341418921947479248046875f,-0.1670000002458691401585610947222448885440826416015625f,-0.42159999603554598746057990865665487945079803466796875f,-0.380300007567554498866257972622406668961048126220703125f,0.1473999876737597247711164527572691440582275390625f,-0.02660001989603066885337057101423852145671844482421875f,-0.7407999963507052587630141715635545551776885986328125f,-0.21179998106956521297661311109550297260284423828125f,0.1627999871343372984000552605721168220043182373046875f,-0.261199982959032350748884709901176393032073974609375f,-0.37290004792660547128235748459701426327228546142578125f,0.3517999982058999108858188265003263950347900390625f,-0.967100037010014457194984061061404645442962646484375f,-0.008200001049041727441135662957094609737396240234375f,-0.5476999660909172007450251840054988861083984375f,0.828999985963105867625699829659424722194671630859375f,-0.02889997881501871201948006273596547544002532958984375f,-0.186999994397163415982277001603506505489349365234375f,-0.48210001385733480816497831256128847599029541015625f,0.34060000168681103804857457362231798470020294189453125f,0.383100000364333226343660498969256877899169921875f,0.4145999880969524209461951613775454461574554443359375f,0.0053999929785728539854972041212022304534912109375f,0.2813999950945376138378151154029183089733123779296875f,-1.0519000324368479315495505943545140326023101806640625f,-0.4864000041842455601681649568490684032440185546875f,-1.1986000121697781839458230024320073425769805908203125f,0.3389999864846469090906566634657792747020721435546875f,-0.3874000057041644407007652262109331786632537841796875f,0.428799989771842859909156686626374721527099609375f,0.3810000003464519835461032926104962825775146484375f,0.62939998534917818151512847180129028856754302978515625f,0.78740002300813838331805527559481561183929443359375f,0.528800006759166052461296203546226024627685546875f,0.90240002891272297347313724458217620849609375f,-0.467699972796439933375722830533050000667572021484375f,-1.25690001157522157626544867525808513164520263671875f,-0.899100004696845811480443444452248513698577880859375f,-0.318799986493587539371219463646411895751953125f,-0.0157000052213671636991421109996736049652099609375f,-0.203800007504224556242888866108842194080352783203125f,-0.1784000089377164588455570992664434015750885009765625f,-0.22860000088214871283298634807579219341278076171875f,0.098000003665685841980348413926549255847930908203125f,-0.7823000161767001969792545423842966556549072265625f,-0.28160002123713478994204706395976245403289794921875f,0.7008999878525734583689654755289666354656219482421875f,-0.45840002830922610410624429277959279716014862060546875f,-0.56440000859498962171301172929815948009490966796875f,0.266900008371472541313096371595747768878936767578125f,0.663400002157688017945247338502667844295501708984375f,-0.0031000137567522845216672067181207239627838134765625f,-0.003299996834993379479783470742404460906982421875f,-0.292400018742680511962817035964690148830413818359375f,0.3435999853104354517796537038520909845829010009765625f,0.12729998175501811008558661342249251902103424072265625f,-0.13060000496506685152553473017178475856781005859375f,0.511900000473856753302470679045654833316802978515625f,0.2482000000059603195268209674395620822906494140625f,0.7457000096172095027213799767196178436279296875f,0.497799998533725585048159700818359851837158203125f,0.30900000250339498819585060118697583675384521484375f,0.03550000068545333675729125388897955417633056640625f,0.135899996420741298042145217550569213926792144775390625f,0.27450000806897900673675394500605762004852294921875f,0.8187999964773648997606869670562446117401123046875f,-0.30610002312213158148779257317073643207550048828125f,-0.36939997029900606406727092689834535121917724609375f,0.4670000516548753477508171272347681224346160888671875f,0.402799999427795096806903529795818030834197998046875f,-0.72000002153217812495000771377817727625370025634765625f,0.08120000188350662284619829733856022357940673828125f,-1.0631000156939032219582941252156160771846771240234375f,0.55659999188184716700789067544974386692047119140625f,0.7118000203341245946120352527941577136516571044921875f,0.050599999749660451442423436674289405345916748046875f,0.6689000055998561844461391956428997218608856201171875f,-0.39899999698996513330939706065692007541656494140625f,0.094000028543174796169523688149638473987579345703125f,0.013800001841783426925758249126374721527099609375f,0.8571000339552761193573360287700779736042022705078125f,0.000599994087219446470271577709354460239410400390625f,0.778700035858154837598021913436241447925567626953125f,-0.41339997906088854495720852355589158833026885986328125f,-0.21399998339265591251745490808389149606227874755859375f,-0.46139998943209636905748993740417063236236572265625f,-0.9441999944850805182028352646739222109317779541015625f,-0.012600021713971454317970710690133273601531982421875f,-1.71280003202408526163935675867833197116851806640625f,-0.424400001740455223853132338263094425201416015625f,0.452100012385844962636838317848742008209228515625f,0.0391999787271029020985224633477628231048583984375f,0.78340001271888670686394107178784906864166259765625f,-0.243199979591369963571878543007187545299530029296875f,-0.90029997290372865137442204286344349384307861328125f,-0.4742000037610532015008857342763803899288177490234375f,-0.1554000102639196967402313021011650562286376953125f,-0.390099997499584905114033972495235502719879150390625f,-1.3306999919593331949130288194282911717891693115234375f,-1.07540002188682581873990784515626728534698486328125f,-0.295400001472234663424387690611183643341064453125f,-0.2070999944299460882035646136500872671604156494140625f,-0.714399984008073918317904826835729181766510009765625f,-0.5291999843895436317353642152738757431507110595703125f,-0.594299984943866821396341038052923977375030517578125f,0.0938999946475027602588170339004136621952056884765625f,0.7074999864399433846529063885100185871124267578125f,0.25509998513162113198404767899774014949798583984375f,-0.46019998278021834181572558009065687656402587890625f,0.2833000109165906277297608539811335504055023193359375f,0.78400000220537169326462390017695724964141845703125f,0.44059999818503836621630398440174758434295654296875f,0.4243999993562697081728174453019164502620697021484375f,0.45390001029372217811186374092358164489269256591796875f,-0.09549999740719794516508045489899814128875732421875f,0.26109999797642213792414622730575501918792724609375f,0.6181000047788021145578341020154766738414764404296875f,0.66850003396719703818718016918865032494068145751953125f,-0.581399981832504408885142765939235687255859375f,-0.5798999912500379938506966936984099447727203369140625f,0.093799996179342315372196026146411895751953125f,-0.17580000823438179935465086600743234157562255859375f,-0.18319997327700232592206930348766036331653594970703125f,0.75979997087717077164370493846945464611053466796875f,-0.5129000057190655770256171308574266731739044189453125f,0.124599991375208052346579279401339590549468994140625f,-0.30829997660666641490223582877661101520061492919921875f,0.703799992752074832225162026588805019855499267578125f,0.0939000091761352795316497577005065977573394775390625f,-0.5613999804913998303845801274292171001434326171875f,-0.244599984334409104036467397236265242099761962890625f,-0.19319999747276295209985619294457137584686279296875f,0.5161000131756068309840657093445770442485809326171875f,0.140399993127584554741815736633725464344024658203125f,0.30300003693252819392256469654967077076435089111328125f,0.2999999837577345118688754155300557613372802734375f,-1.4982999973192807630084644188173115253448486328125f,-0.10540001458525605215754694654606282711029052734375f,-0.1429999814257019163932227456825785338878631591796875f,-0.47279998780488963205925756483338773250579833984375f,0.07739999996721724784265461494214832782745361328125f,-0.3289999996721741926108961706631816923618316650390625f,-0.2062999857857821250917140787350945174694061279296875f,0.4203999847829342773053440396324731409549713134765625f,-0.96680003173351314327277350457734428346157073974609375f,-0.276800010424851861134953878718079067766666412353515625f,-1.304900028130412170668250837479718029499053955078125f,0.588999978810548885377329497714526951313018798828125f,0.55639996436089267373148459228104911744594573974609375f,-0.2535999830007555910782457431196235120296478271484375f,0.9996000100016593226115446668700315058231353759765625f,0.271199985939264198719911291846074163913726806640625f,0.644000004440545836814635549671947956085205078125f,0.6459000098317861926489058532752096652984619140625f,0.0251999963402749216356824035756289958953857421875f,-0.9779000171929599360964857623912394046783447265625f,-1.411899987286329150038000079803168773651123046875f,-0.708799992230534581238998725893907248973846435546875f,-0.34009999131560331164791932678781449794769287109375f,0.518100003698468203339189130929298698902130126953125f,0.92559999557733529940151129267178475856781005859375f,-0.149900009131431755093899482744745910167694091796875f,-0.9705999870836734544354840181767940521240234375f,-1.33139998086392896681218189769424498081207275390625f,-1.209599956655502683844360944931395351886749267578125f,-1.2722999713242055808137820349656976759433746337890625f,-1.0358999732494356127432411085464991629123687744140625f,-0.27279999220073236632089219710906036198139190673828125f,1.13299999725818612006378316436894237995147705078125f,0.12570000969171513105493431794457137584686279296875f,-0.684899982309341570640981444739736616611480712890625f,-1.3272999675989149981347736684256233274936676025390625f,-1.30769995491504698037488196860067546367645263671875f,-0.9337999693572525128359984591952525079250335693359375f,-0.65999996915459657742530907853506505489349365234375f,-0.233499986916780866152976159355603158473968505859375f,-0.02799998891353627783473712042905390262603759765625f,-0.605599987605213829766626076889224350452423095703125f,-0.624899999743699918752781741204671561717987060546875f,-0.0893000021547079025907578397891484200954437255859375f,-0.5999000039160249375669309301883913576602935791015625f,-0.331399996584653688813659755396656692028045654296875f,-0.49659998558610640717603246230282820761203765869140625f,0.3757999998152252718597310376935638487339019775390625f,0.8171000243440271759709503385238349437713623046875f,0.26739999363422395362022143672220408916473388671875f,0.3970000192075968659111140368622727692127227783203125f,0.58359998076558117219292398658581078052520751953125f,-1.1513000071689483139181220394675619900226593017578125f,0.06460000091195094995555336936376988887786865234375f,0.2086000130638481220302082874695770442485809326171875f,0.194799996805191000248669297434389591217041015625f,1.541200011047720863643917255103588104248046875f,0.48539997733235384913541565765626728534698486328125f,0.852299983505904634739636094309389591217041015625f,0.297799985420704071970021686865948140621185302734375f,0.5974999860674141682892468452337197959423065185546875f,-0.66339999068379373881043647998012602329254150390625f,0.5933000095382332528259894388611428439617156982421875f,0.117999987274408635329336902941577136516571044921875f,1.6326000040933497103878835332579910755157470703125f,0.2490000069737430266059163841418921947479248046875f,0.57829999810159193884828709997236728668212890625f,0.47099998959898936590207085828296840190887451171875f,-0.0208000195220119366013022954575717449188232421875f,0.3427999717116361200197616199147887527942657470703125f,0.5114999704658986789951313767232932150363922119140625f,0.06240000302195536363569772220216691493988037109375f,0.80230000108927423951854507322423160076141357421875f,0.019300007072091229787247357307933270931243896484375f,0.206399995392561041995804771431721746921539306640625f,0.3664000000864267025946219291654415428638458251953125f,0.674999997168779231060398160479962825775146484375f,1.23010000956952580963843502104282379150390625f,1.2624999825656413321439686114899814128875732421875f,1.1749999967962505564855746342800557613372802734375f,0.2705999744176867949363440857268869876861572265625f,-0.78030001298785212071607020334340631961822509765625f,-0.82819996603131329493407974950969219207763671875f,-0.518999976202845658690421259962022304534912109375f,-0.4581999819755555591882512089796364307403564453125f,0.218799998861550992756974665098823606967926025390625f,0.69950000080466256946465364308096468448638916015625f,0.450100007334351293053487097495235502719879150390625f,0.500000013858079572770520826452411711215972900390625f,0.3035000191777943445714527115342207252979278564453125f,0.01060001152157763470995632815174758434295654296875f,-0.130399988731741967029620354878716170787811279296875f,-0.6522999982953070485791613464243710041046142578125f,-0.91740000302195545689443179071531631052494049072265625f,-0.306799989041686149704446506802923977375030517578125f,-0.5488999786287547255625440811854787170886993408203125f,-0.2288999731898311296163228689692914485931396484375f,-0.32859998180866245576225992408581078052520751953125f,0.449700001913309055368017652654089033603668212890625f,0.1628999839633700208452182778273709118366241455078125f,0.7623000052988526675790126319043338298797607421875f,1.117800012615322913234194857068359851837158203125f,0.7357000144228340587915226933546364307403564453125f,0.1020000054240224329760167165659368038177490234375f,-0.3120000331401830973021560566849075257778167724609375f,-0.014200003761053014983417597250081598758697509765625f,-1.661900069652498590500044883810915052890777587890625f,-0.47379999543428397146271890960633754730224609375f,-0.49249998375773440528746505151502788066864013671875f,0.75139997721314433221095896442420780658721923828125f,0.7759999946281312954710074336617253720760345458984375f,0.259999976456165793337049763067625463008880615234375f,-0.51840000215768744507016663192189298570156097412109375f,-0.0303999989390373315245597041212022304534912109375f,-0.379499980762600497286030076793394982814788818359375f,-0.6267999815165996579935381305404007434844970703125f,0.243499996639787941177246466395445168018341064453125f,-0.456599996501207083809958930942229926586151123046875f,-0.59620002784878012302982597248046658933162689208984375f,-0.601199986982345446762110441341064870357513427734375f,0.59770002308785918909705969781498424708843231201171875f,0.22379999424219132464486392564140260219573974609375f,1.1641000088691713099109392715035937726497650146484375f,-0.19739999333620072974326831172220408916473388671875f,1.394099986256658763039695259067229926586151123046875f,-0.05020000349283204599259988754056394100189208984375f,1.9866000181823968429029036997235380113124847412109375f,0.39719998909831044642260167165659368038177490234375f,0.04480000749677426341577302082441747188568115234375f,-0.2942000048041339965720908367075026035308837890625f,-1.679100026899576381111955925007350742816925048828125f,-0.7048000055223704318763111587031744420528411865234375f,-0.2701999899327756082811902160756289958953857421875f,-0.517999993458390317613293518661521375179290771484375f,-0.5142999912396073458609180306666530668735504150390625f,-1.011800016944110591765593198942951858043670654296875f,-0.51590000454187379119730394450016319751739501953125f,-0.142500008121132726301993898232467472553253173828125f,0.23559999129325159827885727281682193279266357421875f,0.2384000225350260127044066393864341080188751220703125f,-0.77819996439218552808370077400468289852142333984375f,-0.36089999083280577263366240003961138427257537841796875f,-0.792600015585869233092353169922716915607452392578125f,-0.1795000058338043213979062784346751868724822998046875f,0.28299999554455279593412342364899814128875732421875f,0.005300002259016167727168067358434200286865234375f,-0.11830000190138800775230265571735799312591552734375f,-0.3519000253587962756540719055919907987117767333984375f,0.48869998541772374789360355862299911677837371826171875f,0.516800004017353042939930674037896096706390380859375f,0.75530001872479879398980529003893025219440460205078125f,-0.233299975377321666680785483549698255956172943115234375f,-1.202599950188398825190461138845421373844146728515625f,-0.0999999696761373346731716083013452589511871337890625f,0.7331000183761116062441942631267011165618896484375f,-0.00199997480213642830904063885100185871124267578125f,-0.7697999790728093927327790879644453525543212890625f,-0.066299982842802862581521594620426185429096221923828125f,0.9342999959334730153415193854016251862049102783203125f,0.35170000346302965343880941873067058622837066650390625f,0.226099972122907610927455834826105274260044097900390625f,-0.35079999340772616278627538122236728668212890625f,1.2697000303372742990859478595666587352752685546875f,0.154199993181228733885035353523562662303447723388671875f,1.4408000347957017339695084956474602222442626953125f,-0.14320000149607636785731301642954349517822265625f,0.23390003100633671717645256649120710790157318115234375f,-0.131599991881847511177028309248271398246288299560546875f,-0.99600001310557129130529574467800557613372802734375f,0.616199982362985565487178973853588104248046875f,-1.236200040682405631997653472353704273700714111328125f,0.37259998598843824257897949792095459997653961181640625f,-0.5618000178568063329720416732016019523143768310546875f,-0.44259998446106918468245794429094530642032623291015625f,1.076300038237870015933594913803972303867340087890625f,0.25400001069903321848642008262686431407928466796875f,1.1360000139251347217594911853666417300701141357421875f,-0.1409999961555004066582341692992486059665679931640625f,0.36420001747012153980875837078201584517955780029296875f,0.1167999897122384833636488110641948878765106201171875f,0.0816999967947602545592644673888571560382843017578125f,0.69079996569156687513668657629750669002532958984375f,0.151100005277991389363023699843324720859527587890625f,0.631799963188171886940835975110530853271484375f,-0.4291000487297775833894775132648646831512451171875f,0.84859996914267554757316247560083866119384765625f,-0.0575000452995302335779115310288034379482269287109375f,-0.0067999997705221471022696277941577136516571044921875f,-0.1152000049084429544876684303744696080684661865234375f,-0.3614999944195151915238284345832653343677520751953125f,-0.20429999453648928220417246848228387534618377685546875f,0.3123000283025205714437788628856651484966278076171875f,0.54470001353621455564280040562152862548828125f,-0.5039000073879957852795996586792171001434326171875f,-0.45400000399351103652634265017695724964141845703125f,0.243899975574016547597011594916693866252899169921875f,1.3775999770104885300270325387828052043914794921875f,0.09059998671114488644207085599191486835479736328125f,-1.1584000189959997584310258389450609683990478515625f,-1.029000007942318717368834768421947956085205078125f,-0.076499994009733285338370478712022304534912109375f,-0.0042999792069196018928778357803821563720703125f,-0.65309999662041651191657365416176617145538330078125f,-0.2160000059157611840277013470767997205257415771484375f,0.67820000604093044760389830116764642298221588134765625f,0.227100024753809137667559525652904994785785675048828125f,-0.6917999940335750608255693805404007434844970703125f,0.13470001681447030250637908466160297393798828125f,0.246300000835955046341041452251374721527099609375f,0.52320002232044959811219086986966431140899658203125f,-0.29610000177621831340957214706577360630035400390625f,-0.3785999945491551610388114568195305764675140380859375f,-0.27849999138712888679947354830801486968994140625f,-0.2557999800711872051550699325161986052989959716796875f,-0.63709997423887276823961656191386282444000244140625f,-0.517099982210993847075997109641321003437042236328125f,-0.47919999273419389140826751827262341976165771484375f,-0.26889998771846279623787268064916133880615234375f,0.5445999651305373223664219040074385702610015869140625f,-0.2627999869108201824730031148646958172321319580078125f,0.64219998561516344892652341513894498348236083984375f,0.71959997587800028640003802138380706310272216796875f,0.9934999899342653595368801688891835510730743408203125f,0.286599987262487587713621906004846096038818359375f,0.059600030900538225608897846541367471218109130859375f,-0.2012000039696690834034598083235323429107666015625f,-0.041599998438357932428743879427202045917510986328125f,0.17200001823902066888649642351083457469940185546875f,1.1406999783247704360888974406407214701175689697265625f,-0.451999978452921169491673936136066913604736328125f,0.15319999579638210462917413678951561450958251953125f,0.636999990224837997487838947563432157039642333984375f,-0.35650003975629862562612970577902160584926605224609375f,-0.56299999904632525016268118633888661861419677734375f,-0.8018000476405025001014337249216623604297637939453125f,-0.35619998713135725409983933786861598491668701171875f,-0.51370002229809796912007868741056881844997406005859375f,-0.223799999010562744583552330368547700345516204833984375f,-1.26380003339499236147958072251640260219573974609375f,0.372600001299380867436639164225198328495025634765625f,-0.17900004579126871373517815300147049129009246826171875f,-0.3757999805927279002304430832737125456333160400390625f,-0.117000008553266698907435738874482922255992889404296875f,0.0631999955356121745353448204696178436279296875f,-0.5827000551626089208667735874769277870655059814453125f,-0.77779997722506521995455841533839702606201171875f,0.157100003631412743487771876971237361431121826171875f,0.647000010713934869954755413345992565155029296875f,0.8717999938845633067785456660203635692596435546875f,0.79509999787211427246802486479282379150390625f,0.86179999120235439846737790503539144992828369140625f,0.69570000090002981352199640241451561450958251953125f,1.0460000207424158258362467677216045558452606201171875f,0.71720000735223299326293044941849075257778167724609375f,0.72499997049570108487159814103506505489349365234375f,0.392499985545873730785615407512523233890533447265625f,0.002499995231628471259455182007513940334320068359375f,0.09969999803900719825833220966160297393798828125f,-0.3821999987840651868964414461515843868255615234375f,-0.33390000396072849753181799314916133880615234375f,0.26169999347925188004637675476260483264923095703125f,0.341600014755129688381884989212267100811004638671875f,-0.212699980843067404379098661593161523342132568359375f,0.263100018934905310885596918524242937564849853515625f,-0.19959999459758404061204828394693322479724884033203125f,0.172300008968264084519006473783520050346851348876953125f,0.4202999914184213370305087664746679365634918212890625f,0.02839999496787759536431394735700450837612152099609375f,-0.4342999861359597701948587200604379177093505859375f,0.2062000198394056127426665625534951686859130859375f,0.61610003426074921861754773999564349651336669921875f,-0.025499988913536331125442302436567842960357666015625f,0.51460001715421643808667795383371412754058837890625f,-0.394399994662403940992589923553168773651123046875f,-0.0920000015124677883449066939647309482097625732421875f,-0.19880002557188281020472686577704735100269317626953125f,-0.663500022679567980077308675390668213367462158203125f,-0.107000011533498362581440233043394982814788818359375f,-0.38020000248700347444952285513863898813724517822265625f,-0.4149999879300594329833984375f,-0.061400006903708170824529588571749627590179443359375f,-0.681999999761580877333244643523357808589935302734375f,0.318399992024898548237388240522705018520355224609375f,-0.5665999792158604719816139549948275089263916015625f,-0.0884999798759817934978855191729962825775146484375f,-0.02400000235438337625737403868697583675384521484375f,-0.58049998865276553061676167999394237995147705078125f,0.1953999943196773614317862666212022304534912109375f,-0.0667000284597276671405552406213246285915374755859375f,-0.8573999650239947367680315437610261142253875732421875f,-0.7848000126749277161053441886906512081623077392578125f,-0.5555999823153019434585075941868126392364501953125f,-0.243099988704174929665668969391845166683197021484375f,0.2369999911189079877527063899833592586219310760498046875f,-0.60190002832040179203687557674129493534564971923828125f,-0.058199983763695228855539198775659315288066864013671875f,-0.186400012044609153516461219624034129083156585693359375f,-0.1114000089526172576626095178653486073017120361328125f,0.7552000468552113598974528940743766725063323974609375f,0.7275999478042134427369092009030282497406005859375f,0.87930000837594324281099034124054014682769775390625f,0.17159998025894207529518098453991115093231201171875f,1.176999996222555910918572408263571560382843017578125f,-0.118799983811378850884210578442434780299663543701171875f,0.509000023551285263323507024324499070644378662109375f,0.11390002638697627190822458942420780658721923828125f,-0.5939000020980833571826451589004136621952056884765625f,0.36510001009106651803648446730221621692180633544921875f,1.10550000292062744478016611537896096706390380859375f,0.320800001305341364599144071689806878566741943359375f,-0.55259998276606214773210012936033308506011962890625f,-0.053099974045157427582353193429298698902130126953125f,-0.08489999171569950764393297504284419119358062744140625f,-0.238099987884610564403686794321401976048946380615234375f,-0.995300016117095509571299771778285503387451171875f,-0.103299995940923761139629277749918401241302490234375f,0.529700024712085326683563835103996098041534423828125f,0.0362000078439710382127714183297939598560333251953125f,0.59500000372529004977195654646493494510650634765625f,0.74720002046525468841764450189657509326934814453125f,-0.16229999695420271876855622394941747188568115234375f,0.466000016048550680380913036060519516468048095703125f,0.285100009047984936927377930260263383388519287109375f,-0.2417000044688581983365338601288385689258575439453125f,-0.4224000060580668236553947281208820641040802001953125f,-0.52830002285614607870911640929989516735076904296875f,-0.134400010383128876156888509285636246204376220703125f,-0.2510000208914278108096596042742021381855010986328125f,0.18169998379349738826249449630267918109893798828125f,-0.7541000236958266267350836642435751855373382568359375f,-0.96729999434649938105934552368125878274440765380859375f,-1.2057000023901462970599141044658608734607696533203125f,-0.74069998361468325498435660847462713718414306640625f,-0.211899990341067212540338005055673420429229736328125f,-0.173800015215575864857555643538944423198699951171875f,0.054400003677606445506853560800664126873016357421875f,-0.4239999976232642353579649352468550205230712890625f,-0.2425999747753149193840727093629539012908935546875f,-0.53070001541003619660585854944656603038311004638671875f,-0.14339999176412832060378832466085441410541534423828125f,0.017699982742964970316990047649596817791461944580078125f,0.39359999492168407098091620355262421071529388427734375f,-0.85320005709603474741697937133722007274627685546875f,0.55400000503659185824290034361183643341064453125f,-0.07800000906735693462223935057409107685089111328125f,0.23779998631477372583731266786344349384307861328125f,0.091299946856498337410812382586300373077392578125f,-0.843599981212615812609101340058259665966033935546875f,-1.062300017145276331831382776726968586444854736328125f,0.07139998362064403636395581997931003570556640625f,-0.663399991801380650713326758705079555511474609375f,0.38399998864531514897180386469699442386627197265625f,0.56150002148374955623921778169460594654083251953125f,0.57579999743103937870358777217916212975978851318359375f,0.987200039110332827618776718736626207828521728515625f,0.62559999229907958806506940163671970367431640625f,0.8653000334024432138591009788797236979007720947265625f,0.657999998897313531642794259823858737945556640625f,0.594599989177285959129903858411125838756561279296875f,0.4079999820590021197830310484278015792369842529296875f,0.6606000026926401869076244111056439578533172607421875f,-0.48399999222159362233242063666693866252899169921875f,-0.4851999921306966445655461939168162643909454345703125f,0.103500014483928648445498765795491635799407958984375f,-1.1586999619483950851872577914036810398101806640625f,-0.745199974733591119502307265065610408782958984375f,-0.180799986255169020665789503254927694797515869140625f,0.3388999994903800772050317391403950750827789306640625f,0.2438999905869365658617198278079740703105926513671875f,0.21200000292807807955597354521160013973712921142578125f,0.1001000033855437010288369492627680301666259765625f,-0.0843999923527238848919296287931501865386962890625f,0.196599997395277004130775821977294981479644775390625f,0.61020000930428508478797766656498424708843231201171875f,-0.0102000014811753914045766578055918216705322265625f,0.057099999496340814175709965638816356658935546875f,0.64820000313520420576196556794457137584686279296875f,0.056999998271465290855530838598497211933135986328125f,-0.54770000975131960530006836052052676677703857421875f,-0.4890000079423189038863029054482467472553253173828125f,-0.35769999190717938208905479768873192369937896728515625f,-0.011600015909969674243740200836327858269214630126953125f,0.87709996894896058439172747966949827969074249267578125f,0.5929999822452665103611479935352690517902374267578125f,-0.17760000768825390249361362293711863458156585693359375f,-0.451100011629611385988170013661147095263004302978515625f,-0.44479998548030863503299769945442676544189453125f,-0.0329000051975250062952227381174452602863311767578125f,0.85829998334944257454282023900304920971393585205078125f,0.391699986214935791384306185136665590107440948486328125f,0.35079999528899785676827605129801668226718902587890625f,-0.402899988228827776737261956441216170787811279296875f,-0.0551000110968946632539200436440296471118927001953125f,0.1977999881029131046261682058684527873992919921875f,0.126899993395805932294706508400849997997283935546875f,-0.22659998189806973556414959602989256381988525390625f,-0.88190000166594983976864341457257978618144989013671875f,-0.431799978685379282072887008325778879225254058837890625f,0.55759999247044245151982977404259145259857177734375f,0.164599983775616021119958531926386058330535888671875f,0.05249997101724146464363229824812151491641998291015625f,-0.2163999955415725207075183789129368960857391357421875f,-1.076200003299117025079567611101083457469940185546875f,0.3957999895334243234401583322323858737945556640625f,0.31559999323040266450135504783247597515583038330078125f,-0.2583999842762949583629961125552654266357421875f,0.386700012776255430679839264485053718090057373046875f,0.633399992322921434606541879475116729736328125f,0.23979999959915854201852880578371696174144744873046875f,0.452199984329938964311423887920682318508625030517578125f,0.191000004388392152776532384450547397136688232421875f,-0.2317999856889250143154868055717088282108306884765625f,-0.3936999623022969796437564582447521388530731201171875f,0.57539999327659574479554294157424010336399078369140625f,0.30660001082494847679527083528228104114532470703125f,0.3757999861061573909637445467524230480194091796875f,0.328700006242096609554437236511148512363433837890625f,-0.1603999914884568500639261401374824345111846923828125f,-0.392100015608221330598581744197872467339038848876953125f,-0.6705999770998956055478856796980835497379302978515625f,0.64320001380816105296389650902710855007171630859375f,-0.8229999973326920237326476126327179372310638427734375f,-0.6963999858558178601697363774292171001434326171875f,-0.791599988380074393035101820714771747589111328125f,-0.440799999219179117204703288734890520572662353515625f,-0.0133999777197837399711488615139387547969818115234375f,-0.4561999836668373387027486387523822486400604248046875f,-1.03179999578446146557553220191039144992828369140625f,-0.7340999833881856062589577049948275089263916015625f,-0.82990000682175146806685006595216691493988037109375f,-0.06120001544356323819329190882854163646697998046875f,0.077599992582201959834264926030300557613372802734375f,0.38249999940395351671895696199499070644378662109375f,-0.140400011008977809723319296608678996562957763671875f,0.471499987527728148251071615959517657756805419921875f,1.1128000171601772905205507413484156131744384765625f,0.8108999997735022446931907325051724910736083984375f,-0.11280001604259037861766046262346208095550537109375f,-0.5021999998271464704657773836515843868255615234375f,-0.5331999945670364837724264361895620822906494140625f,-0.47779999421238894541374975233338773250579833984375f,0.342199996027350294980351463891565799713134765625f,1.2188999906241893000924392254091799259185791015625f,0.816499983653426131269270626944489777088165283203125f,0.686699981558323013786093724775128066539764404296875f,-0.01440001090466969113634831956005655229091644287109375f,-0.0741999983593821799132683736388571560382843017578125f,-0.5059999995455146670764179361867718398571014404296875f,-0.69469998842775826375373071641661226749420166015625f,-0.017099975654482957310165147646330296993255615234375f,1.2147999731123444444591541468980722129344940185546875f,0.25759999563693991575519248726777732372283935546875f,0.4431999580591912124560849406407214701175689697265625f,-0.09719999669790269081204314716160297393798828125f,-0.05900000619143275226718969861394725739955902099609375f,0.733199974524974873446581113967113196849822998046875f,0.94809998443871723594611466978676617145538330078125f,0.157599987739324820523734160815365612506866455078125f,0.9645000082179908673651880235411226749420166015625f,-0.55199997532367728325652933563105762004852294921875f,0.9931000384181738471767175724380649626255035400390625f,-0.054599996292591157498463871888816356658935546875f,0.5863999830245971711661923109204508364200592041015625f,-0.86859996422529250903465936062275432050228118896484375f,-0.54000000156462213407593253577942959964275360107421875f,0.221400001502036758438407559879124164581298828125f,-0.2083000154614451648882322842837311327457427978515625f,-0.392399994826316600438076420687139034271240234375f,1.02880003440082123944421255146153271198272705078125f,0.078000007390975678589484232361428439617156982421875f,0.08150001420080654757072124994010664522647857666015625f,-0.0524000003397464553245299612171947956085205078125f,-0.33720000239759695404018202680163085460662841796875f,-0.2005999949812888871747418306767940521240234375f,0.50750000160187480613416255437186919152736663818359375f,-0.313599993282556444018638330817339010536670684814453125f,0.638700017939508146724847392761148512363433837890625f,-0.140000000298023064004837578977458178997039794921875f,0.854600027100742121177745502791367471218109130859375f,-0.524100015574693767206326810992322862148284912109375f,0.4557999956429004129176973947323858737945556640625f,-0.178700000765919408962645320571027696132659912109375f,0.2486999956250190990658666123636066913604736328125f,0.314499994024634410560992137106950394809246063232421875f,0.2626999757207932884028878106619231402873992919921875f,0.211599971765279892821354223997332155704498291015625f,0.94229997579455382350488434894941747188568115234375f,0.56640000515282196147381910122931003570556640625f,0.3752999903380871415947694913484156131744384765625f,0.1290000017583370350848781527020037174224853515625f,-0.116000001668929986209377602790482342243194580078125f,0.4865999769806863550769548965035937726497650146484375f,1.11139995381832168419578010798431932926177978515625f,0.6746999604880812295704117786954157054424285888671875f,-0.11590003114044622467559975120821036398410797119140625f,-0.133399997389316571361206342771765775978565216064453125f,-0.9028999645173552135446470856550149619579315185546875f,-0.964500009596348295559664620668627321720123291015625f,-0.3311000102698802294298729975707828998565673828125f,-0.10259997976720303114461785298772156238555908203125f,-0.4294999867975712870560300871147774159908294677734375f,0.37310001569390316422669684470747597515583038330078125f,0.78349999034404760323013761080801486968994140625f,0.34089998353123629470928790397010743618011474609375f,0.0880000018775461967379669658839702606201171875f,0.25150001516938214063401346720638684928417205810546875f,0.153400006367266084073008869381737895309925079345703125f,-0.11740001360178020650693042625789530575275421142578125f,1.2420000150725247767269365795073099434375762939453125f,-0.2314000070154662580534932203590869903564453125f,1.006100001589954384684233446023426949977874755859375f,-0.4943999720871452563386583278770558536052703857421875f,0.5686000107355411170573233903269283473491668701171875f,0.237799998943507506776740001441794447600841522216796875f,-0.03780002544894867089197987297666259109973907470703125f,-0.6549999883770940556360073969699442386627197265625f,-0.046400030708313000360476507921703159809112548828125f,-0.6445999685764316478042701419326476752758026123046875f,0.5643000062525269644453373985015787184238433837890625f,0.01200000452995286792656770558096468448638916015625f,-0.2967999834537504710851862910203635692596435546875f,-0.2537999791920189096572357811965048313140869140625f,-1.357499998994171619415283203125f,-0.3865999762356284730202560240286402404308319091796875f,-0.0361999628424640729207339973072521388530731201171875f,0.387599991315603187302940568770281970500946044921875f,0.4671999807164066709219696349464356899261474609375f,-0.5562000009894365604168342542834579944610595703125f,0.118400004541874270813650582567788660526275634765625f,0.06199999573826797671927124611102044582366943359375f,-0.63540001365393361520972348444047383964061737060546875f,-0.06839997786879603580700859311036765575408935546875f,-0.6245000264719136051638770368299447000026702880859375f,-0.6461999924957748664411383288097567856311798095703125f,0.6018000277101995809658774305717088282108306884765625f,0.21479999196231369662513088769628666341304779052734375f,0.95560002698153301281536187161691486835479736328125f,-0.54709998165220030674760209876694716513156890869140625f,-0.8871999694287779902879265137016773223876953125f,-0.4466999779820441940358932697563432157039642333984375f,0.0314000019863247548101981010404415428638458251953125f,-0.41340001292377703823177625963580794632434844970703125f,-0.1217000033140180903501459397375583648681640625f,-0.5156000165879728314166641212068498134613037109375f,-0.388499994069337883928483279305510222911834716796875f,-0.3985999763697385400718076198245398700237274169921875f,-0.1230999735236171233054847107268869876861572265625f,-0.5251000114321708878151184762828052043914794921875f,0.8040999686360361664583251695148646831512451171875f,1.05419998662471758876790772774256765842437744140625f,0.7841999992907044969570051762275397777557373046875f,0.978000010706484346201250446029007434844970703125f,0.01540000011250375333560214130557142198085784912109375f,0.085100014729052875228632046855636872351169586181640625f,-0.43729998235106481985212667495943605899810791015625f,-0.873699987801909383478005111101083457469940185546875f,-1.04399999445676794351811622618697583675384521484375f,-0.966199972826242259316131821833550930023193359375f,-0.6070999758780004640357219614088535308837890625f,-0.68059997747242473309370325296185910701751708984375f,-0.153899980044365225495539561961777508258819580078125f,-0.3084999977201221543765541355242021381855010986328125f,0.6373999824583531648158896132372319698333740234375f,0.8487999990850685350096682668663561344146728515625f,0.8415000080168244522837994736619293689727783203125f,0.6421000047117468856328059700899757444858551025390625f,1.260000007674098032595111362752504646778106689453125f,0.184599979454279417012685371446423232555389404296875f,0.6494000027090311988331450265832245349884033203125f,0.5039999795556069361879281132132746279239654541015625f,0.94350003463774978484934763400815427303314208984375f,0.129800005447864297281057588406838476657867431640625f,0.54820002332627770780248965820646844804286956787109375f,0.137799991381168496218379004858434200286865234375f,0.9941999803662298074158343297312967479228973388671875f,0.139599993461370530667409184388816356658935546875f,1.66819999080449310469020929303951561450958251953125f,-0.7429999787807464173283733543939888477325439453125f,-0.822299981606006813450449044466949999332427978515625f,-0.192000017344950979492068654508329927921295166015625f,-1.569399996469169611401639485848136246204376220703125f,0.37219997941255600348853249670355580747127532958984375f,-0.829500014383345529722646460868418216705322265625f,0.512399987822770963674656741204671561717987060546875f,0.17129994719177432660472959469188936054706573486328125f,0.01080000473260867011049413122236728668212890625f,1.179899968972801804767414068919606506824493408203125f,0.08140000358819943215848979889415204524993896484375f,1.3499999700486651210695754343760199844837188720703125f,0.1215999816000465916232542440411634743213653564453125f,0.65889998216778078443667254759930074214935302734375f,-0.853599966609478233436902883113361895084381103515625f,-0.05389997780919075776040472192107699811458587646484375f,-0.35759999340176573667804404976777732372283935546875f,-0.59349998997151853785680941655300557613372802734375f,-1.092299978476762856161030867951922118663787841796875f,-0.420499994277954147747777824406512081623077392578125f,0.3894999988675118540726316496147774159908294677734375f,0.227699976223707523104167194105684757232666015625f,0.023899985185265482101613088161684572696685791015625f,-0.06279998929500596460684391786344349384307861328125f,0.660500001952052162579320793156512081623077392578125f,0.57079996565431390109779385966248810291290283203125f,0.30229997609257741597588164950138889253139495849609375f,0.2638999806776645140615755735780112445354461669921875f,0.06889995918273916775120824240730144083499908447265625f,1.01109997063279166695792810060083866119384765625f,0.403399994334578793075962721559335477650165557861328125f,0.0035999984793365358282102306475280784070491790771484375f,-0.7317000021219250971427072727237828075885772705078125f,-1.1757000059664246549573363154195249080657958984375f,-1.69099999578297133240312177804298698902130126953125f,-1.32659997236132642939310244400985538959503173828125f,-0.951299982097744578624087807838805019855499267578125f,0.18509998624920864784826335380785167217254638671875f,-0.327500013187527638791607387247495353221893310546875f,-0.4650999980211256712436806992627680301666259765625f,-0.2534000097572803866796675720252096652984619140625f,1.02450000199675539391819256707094609737396240234375f,0.9458000071913001960410838364623486995697021484375f,0.26049999651312827353422107989899814128875732421875f,0.4222000013172626520230323876603506505489349365234375f,-0.03099999518692497790794959655613638460636138916015625f,0.8230000024735926178465206248802132904529571533203125f,-0.3977000305384403855413211203995160758495330810546875f,-0.442600008898972940851734847456100396811962127685546875f,-0.059800033015012576953495226916857063770294189453125f,-0.4587999779999256677598395981476642191410064697265625f,-0.06470001454204332702602187055163085460662841796875f,0.4252000031203031493731714363093487918376922607421875f,-0.8141999813348055337058895020163618028163909912109375f,0.55779998665004970614944568296778015792369842529296875f,-0.05320002047642991005460544329253025352954864501953125f,-0.313399973100424222849369471077807247638702392578125f,0.64479998846054087380252894945442676544189453125f,-1.30619999786019302945305753382854163646697998046875f,-0.1534000081181521313311577614513225853443145751953125f,0.3652000002145765478189787245355546474456787109375f,-1.128900016552209617515245554386638104915618896484375f,1.26920000525116893896893088822253048419952392578125f,-0.64020003836154959486748339259065687656402587890625f,0.3088000094413756446698471336276270449161529541015625f,-0.159800008670240745178858787767239846289157867431640625f,0.04519999946951865077426901962098781950771808624267578125f,-0.41150001176074102460233916644938290119171142578125f,-0.20599998490512394511142701958306133747100830078125f,-1.3262999799370760456440621055662631988525390625f,1.020799983572959757083253862219862639904022216796875f,-0.6482000042527911176648558466695249080657958984375f,-0.259999996498227137209369175252504646778106689453125f,-0.951300009664893142513619750388897955417633056640625f,1.226200004267692378334686509333550930023193359375f,-1.30930000252723655052022877498529851436614990234375f,0.3733000380367046400209574130713008344173431396484375f,-0.20259999925047156299484640840091742575168609619140625f,-0.700900010129809292180880220257677137851715087890625f,0.03379999763220542519093214650638401508331298828125f,0.356500015392899438637641651439480483531951904296875f,0.068499998465180400586405085050500929355621337890625f,0.187199999976157993586411976139061152935028076171875f,-0.1931999999314546467843456412083469331264495849609375f,-0.6203999829947945965358258035848848521709442138671875f,0.102200002583861360161421316661289893090724945068359375f,-0.79689998676478790429200671496801078319549560546875f,-0.327399993783235476740856029209680855274200439453125f,-1.275800007489322762666006383369676768779754638671875f,-0.1544999977499246224255102788447402417659759521484375f,1.209000023886562136254951838054694235324859619140625f,0.379000008724629910972225843579508364200592041015625f,1.165500070571899637883461764431558549404144287109375f,0.0952999973788857257517292964621447026729583740234375f,1.3345000274032354781184039893560111522674560546875f,0.38680000420361759783105526366853155195713043212890625f,1.424900028353929481994555317214690148830413818359375f,0.125799997915327532727047810112708248198032379150390625f,0.57080000216215809327735541955917142331600189208984375f,0.28160000167936083403930069835041649639606475830078125f,0.4427000242054461853768998480518348515033721923828125f,-0.0918000018566846875955889117904007434844970703125f,-0.33040002628266851303351359092630445957183837890625f,0.26149999557435510411096402094699442386627197265625f,1.2833999961227171571209737521712668240070343017578125f,-0.2005000114142896361357770729227922856807708740234375f,0.074999992027878936706741797024733386933803558349609375f,-1.033500008970499539628917773370631039142608642578125f,0.634199988636374722972277595545165240764617919921875f,-0.851800003123283744344007573090493679046630859375f,0.310799989905953477631328496499918401241302490234375f,0.0908000014543528521926418761722743511199951171875f,-0.293800020766257663495935048558749258518218994140625f,-0.07399994319677383192157549274270422756671905517578125f,0.50639995791614100806299347823369316756725311279296875f,-0.7714999820142975917036665123305283486843109130859375f,1.1605999830603597189337961026467382907867431640625f,-0.509700014579296034611388677149079740047454833984375f,-0.34320000648796533671003317067516036331653594970703125f,1.08289998642206253265385385020636022090911865234375f,-1.879799974229932058022995988721959292888641357421875f,-0.116299970251322548620009911246597766876220703125f,0.612399967259169120126216512289829552173614501953125f,0.138800036635994938905014350893907248973846435546875f,-0.65700001622736436956984107382595539093017578125f,0.9818000238731503515055010211654007434844970703125f,-0.366599992626905457626662609982304275035858154296875f,0.5317999989137052185839138473966158926486968994140625f,-0.4793999897450209601856840890832245349884033203125f,-0.29799999427795409445707264239899814128875732421875f,0.9817999964177606120330210615065880119800567626953125f,-0.48429998628497106238910419051535427570343017578125f,0.20940001015961140495846848352812230587005615234375f,0.51210000627636897974070961936376988887786865234375f,0.420899976453184887503766731242649257183074951171875f,0.2562999993830918921133843468851409852504730224609375f,1.048800042968988321945289499126374721527099609375f,0.03440000919103625154349401782383210957050323486328125f,-0.76059998566806275022145200637169182300567626953125f,-0.1097000049680472155699817449203692376613616943359375f,0.73260001567900179253456371952779591083526611328125f,0.156500003173947355161743644202942959964275360107421875f,-0.088199992182850461830412314157001674175262451171875f,-0.02590000141263004707070649601519107818603515625f,0.84760000971853755036278244006098248064517974853515625f,0.18390001044273385222282968243234790861606597900390625f,0.44220003819763731289782526800991035997867584228515625f,0.0929000026643276022308981509922887198626995086669921875f,0.8117000644832856881549787431140430271625518798828125f,0.23650001103430995375020984283764846622943878173828125f,0.247300027501583752354008538532070815563201904296875f,0.0302999998375773349490458485888666473329067230224609375f,1.2901000431716445060459363958216272294521331787109375f,0.175800007824599813233135137124918401241302490234375f,-0.1026999774903052387031721082166768610477447509765625f,-0.209700004856288424459620500783785246312618255615234375f,0.21699998959153876665340021645533852279186248779296875f,0.25100000289827584243340652392362244427204132080078125f,0.48210003168284909502716573115321807563304901123046875f,0.080900000667572025125906520770513452589511871337890625f,0.366300003927946260517956034163944423198699951171875f,0.24560000511407853451828486868180334568023681640625f,1.266800059971213698872816166840493679046630859375f,1.04659998040795354512511039501987397670745849609375f,-1.266399958363175759501473294221796095371246337890625f,1.038000012457370502261255751363933086395263671875f,-0.0630000239312646925782246398739516735076904296875f,-0.618100031936169358459665090776979923248291015625f,0.2842000081568958336220021010376513004302978515625f,-0.08810001628994967859398457221686840057373046875f,-0.54040000154674050492076275986619293689727783203125f,-0.97979999516606230969273383379913866519927978515625f,1.02620001387894088651364654651843011379241943359375f,0.074900009429455405296494063804857432842254638671875f,-0.70440001448094846647762778957257978618144989013671875f,0.103700006242096520736595266498625278472900390625f,-0.04740000201612704078257820583530701696872711181640625f,0.03210004779472985791954897649702616035938262939453125f,0.2831999648392200530366835664608515799045562744140625f,-0.317799995627999098957161550060845911502838134765625f,-0.024399997717142074549201424815692007541656494140625f,0.798000015512108529236456888611428439617156982421875f,-1.39560002470910493599376422935165464878082275390625f,-0.48349998628348078710814661462791264057159423828125f,0.7506000001221895256975358279305510222911834716796875f,-0.5139999983683229967112993108457885682582855224609375f,-0.142400010994076620107762209954671561717987060546875f,-0.5025999780908225744724404648877680301666259765625f,0.83819998834580167201835365631268359720706939697265625f,-0.34699999869987296907680729418643750250339508056640625f,0.17800001245737018251702465931884944438934326171875f,0.61080004287958189479468273930251598358154296875f,-2.131300051611661583450541002093814313411712646484375f,-0.2172000095129014152206536891753785312175750732421875f,-0.681200024682283356014522723853588104248046875f,-0.25569999095350492979150658356957137584686279296875f,-0.6288999932318917185369855360477231442928314208984375f,0.1044000017032027383123704566969536244869232177734375f,-0.314500020623207365844109517638571560382843017578125f,-0.161000004276633301714127810555510222911834716796875f,-0.40700002971291571185474822414107620716094970703125f,-0.111300008808076444921653092023916542530059814453125f,0.57649998007714675640755785934743471443653106689453125f,0.31790000288784503457151231486932374536991119384765625f,-0.055800006446242722546458026045002043247222900390625f,-0.304300009232759516297761592795723117887973785400390625f,-0.3990000309646133391794364797533489763736724853515625f,0.2215000064149499348076233218307606875896453857421875f,0.82970004728734547683188793598674237728118896484375f,0.0969000032305717962088920103269629180431365966796875f,-0.12600001813471306544300887253484688699245452880859375f,-0.0238999993413686606658075106679461896419525146484375f,-0.222600013443827737802394040045328438282012939453125f,-0.1879000031113624569201192571199499070644378662109375f,-0.45960002870261718044275767169892787933349609375f,0.003699993240833254759536430356092751026153564453125f,0.8361999871313567922470610938034951686859130859375f,-0.0174000027239322729855075522209517657756805419921875f,0.438200008574127775062834189157001674175262451171875f,0.1219999961480497940868872319697402417659759521484375f,0.796100000099837501466026878915727138519287109375f,-1.7180000231862067039401154033839702606201171875f,1.600600017482042058958313646144233644008636474609375f,-1.0936000604867928132080123759806156158447265625f,0.0770000109374517904825552250258624553680419921875f,-0.28049998384714047716670393128879368305206298828125f,0.5624000054806466142309773204033263027667999267578125f,-0.8135000526309017487847086158581078052520751953125f,-0.066399985706806052832007480901665985584259033203125f,-1.351900040483474896291227196343243122100830078125f,1.04760002111792527301759037072770297527313232421875f,0.254499999165534784850706273573450744152069091796875f,-0.53679999962151059289539034580229781568050384521484375f,0.26639999300837491347238028538413345813751220703125f,0.476200013878941508238540336606092751026153564453125f,0.3484000064790251371960039250552654266357421875f,-0.7474000031709671088009372397209517657756805419921875f,0.7198000561118131912508033565245568752288818359375f,0.32239998610913733756433430244214832782745361328125f,0.639199994522333891922016846365295350551605224609375f,-0.60219997188746965388617127246106974780559539794921875f,0.398299984690547059784648808999918401241302490234375f,-0.064999986812472432262666188762523233890533447265625f,-0.072100008809567128764683729968965053558349609375f,-0.5274000004142518349681267864070832729339599609375f,0.3484999864697453375583791057579219341278076171875f,-0.9143999944388863099220543517731130123138427734375f,0.6779000525832170609419335960410535335540771484375f,0.296999974802136801343976912903599441051483154296875f,1.47570001907944714503173599950969219207763671875f,-1.074399990899860046766889354330487549304962158203125f,-0.0928999978587030650345468529849313199520111083984375f,-0.24990003274977201641604551696218550205230712890625f,-0.066100007289648099462198160836123861372470855712890625f,-0.97420003564953827623895676879328675568103790283203125f,-0.2477000012949108764814809546805918216705322265625f,-0.985700049105286524309121887199580669403076171875f,-0.3031000013142823856782115399255417287349700927734375f,-1.4776000333368777273079786027665250003337860107421875f,-0.326200000989437077780763729606405831873416900634765625f,-1.3208000205278394023622468012035824358463287353515625f,-0.1343999992072582028157512468169443309307098388671875f,0.16379999729692951149218060891143977642059326171875f,0.33400000008195640521790892307762987911701202392578125f,0.429300000068545184461754615767858922481536865234375f,-0.295100003609061189990825369022786617279052734375f,0.564599996218085653509888288681395351886749267578125f,0.197800008107721858596761421722476370632648468017578125f,0.33410002719759968048407472451799549162387847900390625f,0.0156999987766146542611522818333469331264495849609375f,0.19650001278519635405928056570701301097869873046875f,0.1144000005483627457891770973219536244869232177734375f,0.6898000038832423630452694851555861532688140869140625f,0.04439999760538337891802029844257049262523651123046875f,0.1555000173747540515023501939140260219573974609375f,-0.0457000001922249765584638225845992565155029296875f,0.232300000455975652613460624706931412220001220703125f,0.009000001870095697409368540320429019629955291748046875f,0.01030002431273491225738325738348066806793212890625f,-1.477699964861570958163383693317882716655731201171875f,1.253199971395730916157162937452085316181182861328125f,-1.09590002406239506171914399601519107818603515625f,0.05700001548230648751314220135100185871124267578125f,-1.710800014418362824386576903634704649448394775390625f,1.728400002455710904314400977455079555511474609375f,-0.89270004789829204838724763249047100543975830078125f,-0.0209999796897175627918841200880706310272216796875f,-0.60290001100897772090547732659615576267242431640625f,0.6195999922692776440413808813900686800479888916015625f,-0.7679999848902223646973652648739516735076904296875f,0.7921999813497067055578781946678645908832550048828125f,0.127800022971630511392504558898508548736572265625f,-0.63899999110400695911948787397705018520355224609375f,-0.525800009202956886156243854202330112457275390625f,0.111799997982382937067313832812942564487457275390625f,0.8258000331938273319565269048325717449188232421875f,-0.816800010350346550325184580287896096706390380859375f,1.052000016301870122248374173068441450595855712890625f,-0.69859999939203232255380271453759633004665374755859375f,0.001000028640032013527161325328052043914794921875f,0.34659999083876602554710188996978104114532470703125f,0.16019998672902513447979799821041524410247802734375f,-1.0610000050961965190055025232140906155109405517578125f,-0.7160999825909735161388880442245863378047943115234375f,0.80059997665286086743208215921185910701751708984375f,-0.03430000804066679620518698357045650482177734375f,-0.034800001797079982424065747181884944438934326171875f,1.0999000180721285602203352027572691440582275390625f,1.31110002092421051855808400432579219341278076171875f,0.095500004336237953594945793156512081623077392578125f,0.473800023075938714356425407459028065204620361328125f,0.12310000272989274350265986868180334568023681640625f,-0.8048000019460914700175635516643524169921875f,-0.356900002858042686426642831065692007541656494140625f,-1.579400051137805238710143385105766355991363525390625f,0.01139999811202285773248377154232002794742584228515625f,1.262100022965670031993568045436404645442962646484375f,0.298500007219612673026176707935519516468048095703125f,0.33010002938807048167291213758289813995361328125f,-0.2120000006183981877683208949747495353221893310546875f,-2.00310000913739205685715205618180334568023681640625f,-0.24149999605864280738387606106698513031005859375f,-1.716300011751055176745239805313758552074432373046875f,-0.26690000054836271470293240781757049262523651123046875f,0.279800001573562884260581995476968586444854736328125f,0.224100004659593121658645031857304275035858154296875f,-0.2666999700844285836609515172312967479228973388671875f,-0.16600000107288359618706863329862244427204132080078125f,-1.778000001430510845779053852311335504055023193359375f,-0.3110999886259435687207997034420259296894073486328125f,-1.870499981164930769494958440191112458705902099609375f,-0.134699993456900035670997795023140497505664825439453125f,0.078700020882487731199717018171213567256927490234375f,0.122399997396767101776049457839690148830413818359375f,-0.3385000133663418342422346540843136608600616455078125f,-0.186000003717839756944840701180510222911834716796875f,0.02029998966753421285602598800323903560638427734375f,0.5827000028967859179829247295856475830078125f,-0.92100000189244735082638726453296840190887451171875f,0.15410001631975234204219304956495761871337890625f,0.134200005027651503297647650470025837421417236328125f,-1.18049999323487231350782167282886803150177001953125f,1.79679999112784827275390853174030780792236328125f,-1.4817000116586687141762013197876513004302978515625f,0.458199988904595567618116547237150371074676513671875f,0.107499997317791695650157635100185871124267578125f,-1.102599988931417751558683448820374906063079833984375f,-0.27749997407197923848798382095992565155029296875f,1.337599981704354235034770681522786617279052734375f,-1.52939996538162237271762933232821524143218994140625f,1.6045999466717246750846470604301430284976959228515625f,-1.673799983811378400133662580628879368305206298828125f,1.24479997035563005880476339370943605899810791015625f,0.87870001618862136893994829733856022357940673828125f,-1.392600006365775744399115865235216915607452392578125f,0.32920004652738565908975942875258624553680419921875f,0.9013999748289587277128021014505065977573394775390625f,-1.212799965453147166272174217738211154937744140625f,1.726799952757358536103993174037896096706390380859375f,-1.5969999882876866337255705730058252811431884765625f,0.750199976074695573657891145558096468448638916015625f,-0.64889997184872694191426489851437509059906005859375f,0.2675999676972633523064359906129539012908935546875f,-0.55240000570416469827250693924725055694580078125f,0.7354000006526708599352559758699499070644378662109375f,0.1910000144839276714492370956577360630035400390625f,0.1039000094741584501178977006929926574230194091796875f,0.1728000044569373283565028032171539962291717529296875f,0.59410001758635022639509770669974386692047119140625f,0.162099998676776824169110113871283829212188720703125f,1.94160003636181333064314458169974386692047119140625f,0.250099998095631548267192556522786617279052734375f,1.32880002315044354332940201857127249240875244140625f,0.3582999943390487995742432758561335504055023193359375f,0.21450000952184122837707036524079740047454833984375f,-0.171399999080598330891689329291693866252899169921875f,-1.23710001596212304519895042176358401775360107421875f,-0.267599996829032793499436593265272676944732666015625f,0.494600005754829208370892956736497581005096435546875f,0.09780000713914638321710981472278945147991180419921875f,0.2689000371903185193644958417280577123165130615234375f,0.0329000042289495642222618698724545538425445556640625f,-0.110599977621435474617328509339131414890289306640625f,-0.220399998007714703618376006488688290119171142578125f,-0.94030000970959637385249152430333197116851806640625f,-0.0889000025078654498855712517979554831981658935546875f,-0.9623000211685897031799186152056790888309478759765625f,-0.1522999977737664878407741753107984550297260284423828125f,-0.6661999825865028146409940745797939598560333251953125f,-0.0082999946370720945054699768661521375179290771484375f,0.0087000484496362329167595817125402390956878662109375f,0.3666000072672963572273374666110612452030181884765625f,1.12510004145801101316237691207788884639739990234375f,0.12830000331699853433065072749741375446319580078125f,1.020200019809603730891467421315610408782958984375f,0.434900024479627944629100966267287731170654296875f,0.0507999972075221517542331639560870826244354248046875f,0.4734999834895123882461120956577360630035400390625f,-0.95060000205933992134532672935165464878082275390625f,1.3086999804258336865814271732233464717864990234375f,-1.69959998608529527786004109657369554042816162109375f,0.602100025796890037099728942848742008209228515625f,0.14860001220703100699438437004573643207550048828125f,-0.679899973070622110071781207807362079620361328125f,1.11959995151460223183903508470393717288970947265625f,-1.1977000082612025977368830353952944278717041015625f,0.15480000634193391562121178139932453632354736328125f,0.7273000152826316622167723835445940494537353515625f,-0.869800001648068477066999548696912825107574462890625f,1.27520003120899172444069336052052676677703857421875f,-0.7374000202327961606130202198983170092105865478515625f,0.5416000029087069123079345445148646831512451171875f,-0.403199999782442919382674517692066729068756103515625f,-0.2631999745249746780473287799395620822906494140625f,1.02119999011158935076082343584857881069183349609375f,-0.31350000010430800756466851453296840190887451171875f,0.2905999917775392571428483279305510222911834716796875f,0.161200041148065853491289090015925467014312744140625f,0.50519995059371058943042953615076839923858642578125f,0.61540006927251855728400187217630445957183837890625f,-0.0198000193804503954453366532106883823871612548828125f,0.8722000229239466051467388751916587352752685546875f,-1.09560000310242156018603054690174758434295654296875f,0.5344000050187105443910695612430572509765625f,0.41620001633763337167692952789366245269775390625f,-0.186999999165534946943267868846305646002292633056640625f,-1.0344000085949895950676591382944025099277496337890625f,-0.05769999958127736139346097843372263014316558837890625f,-1.077900030380487539360956361633725464344024658203125f,-0.1894000049442052802106672970694489777088165283203125f,-0.59940004554986991980314314787392504513263702392578125f,0.01979999985992908495546771519002504646778106689453125f,-0.533599984490871204201312139048241078853607177734375f,-0.14609999518245453398179733994766138494014739990234375f,-0.67010002678036661993843381424085237085819244384765625f,-0.2319000139035285457111967843957245349884033203125f,0.051299950861184939743253607957740314304828643798828125f,-0.0396999991565942789151222314103506505489349365234375f,-0.116199983182549093641000581555999815464019775390625f,0.1290000070855021885307678530807606875896453857421875f,0.264300010424852349633084713786956854164600372314453125f,0.1454999981895088723149456200189888477325439453125f,1.0373000501513482962678835974656976759433746337890625f,0.334600011715293021996586730892886407673358917236328125f,-0.3109999375194301141078767614089883863925933837890625f,-0.222399996130168364771861888584680855274200439453125f,0.39110003493130296448043736745603382587432861328125f,0.4601000074833632158544105550390668213367462158203125f,0.69660004053413882019185621174983680248260498046875f,-0.03410000372827060033387169823981821537017822265625f,1.1006000273913147946558410694706253707408905029296875f,0.33900000600516799753592067645513452589511871337890625f,0.59130004457086327018799920551828108727931976318359375f,-0.27900000131130209268803810118697583675384521484375f,0.1831999928534031596427666954696178436279296875f,-0.18669996464550475678834118298254907131195068359375f,0.8579999815374612648355423516477458178997039794921875f,-0.3370999915614720787715441474574618041515350341796875f,0.599800007981061700235159150906838476657867431640625f,0.031899985572696021307592673110775649547576904296875f,-0.38039998455941692601101067339186556637287139892578125f,-1.1692999980196356890616016244166530668735504150390625f,0.891799966466427118660931228077970445156097412109375f,-1.856100048435479710207118841935880482196807861328125f,0.78100002536177626222269054778735153377056121826171875f,-0.15600004106387521662924200427369214594364166259765625f,-0.5037999910384416413222652408876456320285797119140625f,0.423600031504035801077634459943510591983795166015625f,0.537799997565149734413125770515762269496917724609375f,0.6230999991536136928260702916304580867290496826171875f,-0.5231999909162521333882978069595992565155029296875f,1.0301000546455385542543581323116086423397064208984375f,-0.63940003213882423693803502828814089298248291015625f,-1.049599936687946044600039385841228067874908447265625f,1.5359999421387915408132585071143694221973419189453125f,-0.5104999325871464321835446753539144992828369140625f,-0.47120003995597325996413928805850446224212646484375f,-1.034599973642826853392762131989002227783203125f,1.10739996301233833975175002706237137317657470703125f,0.181600006224214904904101786087267100811004638671875f,-0.94980000403225417482389048018376342952251434326171875f,-0.2068999610975390535116957835271023213863372802734375f,0.42060003849268012832141039325506426393985748291015625f,-0.091799997833371171918059872041339986026287078857421875f,-0.048699966195225652398903548601083457469940185546875f,0.414600005568563911850787917501293122768402099609375f,0.979300053340196985374177529592998325824737548828125f,0.021099997566640382729019620455801486968994140625f,0.69309997568428460823497516685165464878082275390625f,0.353599998535215842565548882703296840190887451171875f,0.40229998659789512061024652211926877498626708984375f,-0.279800007347762669240864852326922118663787841796875f,-1.3557000437408692761209749733097851276397705078125f,-0.1647999981090426668828285983181558549404144287109375f,-1.040099994221329371413276021485216915607452392578125f,-0.03130000087320805590707095689140260219573974609375f,0.50560000317692743720954240416176617145538330078125f,0.02830000037401914081414133761427365243434906005859375f,-0.52950000639259808821179831284098327159881591796875f,-0.38210000471174720981792916063568554818630218505859375f,-0.76440002260208139706065821883385069668292999267578125f,-0.01590000077933073630020999189582653343677520751953125f,-0.173300008383393322475285458494909107685089111328125f,-0.060300002110004469191295584096224047243595123291015625f,0.07120000925958169801788244512863457202911376953125f,-0.176899996674060810963879930568509735167026519775390625f,-1.43879999655187074125706203631125390529632568359375f,-0.259599997596442644276493183497223071753978729248046875f,-0.955700025859474777689683833159506320953369140625f,-0.430100003758072890747143901535309851169586181640625f,-1.9482000522345306325888714127358980476856231689453125f,0.24720000843256695333138850401155650615692138671875f,0.29859998929649600452052027321769855916500091552734375f,0.15710003587379961498271541131543926894664764404296875f,-0.2538000379770988246264096233062446117401123046875f,-0.0638999597787854867192436358891427516937255859375f,0.907399994081258487454988426179625093936920166015625f,0.85089999321699139045449555851519107818603515625f,-1.76919997522532934652872427250258624553680419921875f,-0.147800019395351256434878450818359851837158203125f,1.54320002362430042097685145563445985317230224609375f,-0.271400049483776939496237901039421558380126953125f,-0.1142000030905006013881575199775397777557373046875f,-0.8062000003933906810971166123636066913604736328125f,0.978399996122717663382672981242649257183074951171875f,0.2298999811172477425458282596082426607608795166015625f,-1.0850000092387193095788688879110850393772125244140625f,-0.095800021421909686836215769289992749691009521484375f,0.47940000360310097704541476559825241565704345703125f,-0.114600021103024918289747802191413938999176025390625f,-0.46839999880492666495257481074077077209949493408203125f,-0.6567000330790879747411281641689129173755645751953125f,0.246400009474158299127566351671703159809112548828125f,-0.25280002595186257252635186887346208095550537109375f,0.248399993589520473591392146772705018520355224609375f,-0.77330000398755061041811131872236728668212890625f,0.522199983808398382478799248929135501384735107421875f,-0.50580004153847735182125688879750669002532958984375f,0.091800002899766308672724335337989032268524169921875f,-0.81300002825260175853827604441903531551361083984375f,0.0096000004649159365044397418387234210968017578125f,0.2098999984398484119907379863434471189975738525390625f,1.766300032389163998658432319643907248973846435546875f,0.2247000047817826473561808597878552973270416259765625f,1.0745000554174186646605448913760483264923095703125f,0.1368000080034137422302364939241670072078704833984375f,1.068700029450655453189256149926222860813140869140625f,0.1488999970689415253133347505354322493076324462890625f,-0.1905000075399880810067543279728852212429046630859375f,-0.05780000263154505546481232158839702606201171875f,-0.1712000190943481214844723581336438655853271484375f,-0.081100002223253253585966149330488406121730804443359375f,-0.050000023469328969127900563762523233890533447265625f,-0.1792999987274408457693652962916530668735504150390625f,-0.9543999939918512698255881332443095743656158447265625f,-0.3059999953359364699423394995392300188541412353515625f,-0.66699999655783115404261707226396538317203521728515625f,0.023900000086426731460420569419511593878269195556640625f,0.2738999959886074808679268244304694235324859619140625f,0.06450000218302009979964850572287105023860931396484375f,0.211900031021237555961533871595747768878936767578125f,-0.142300000753998734381866597686894237995147705078125f,0.034800034058094286848472620476968586444854736328125f,0.461200008961558438613792532123625278472900390625f,1.861400074964762563212161694536916911602020263671875f,0.311800007593631800517641750047914683818817138671875f,0.3284000255525116873656088500865735113620758056640625f,-0.02779999734163285385335484534152783453464508056640625f,-0.10860000565052008170141562004573643207550048828125f,0.523699989366530616052841651253402233123779296875f,-1.17619999622106519154840498231351375579833984375f,1.506700026857853202955084270797669887542724609375f,-0.932999997258186386517309074406512081623077392578125f,1.808300041091441823226659835199825465679168701171875f,-1.474800027874111574277549152611754834651947021484375f,1.89260002692937856494381776428781449794769287109375f,-0.793399976676702589628575879032723605632781982421875f,0.43999998927116390490255071199499070644378662109375f,-0.1801999851644040528952928070793859660625457763671875f,-0.5571000071704386158444322063587605953216552734375f,0.4678000009924172530872965580783784389495849609375f,0.41309995906949037447475348017178475856781005859375f,-0.67839998166859150074969875277020037174224853515625f,0.21939998043179542719371966086328029632568359375f,-0.2158000002622604274193918172386474907398223876953125f,-0.27779998106211412078891953569836914539337158203125f,0.38419998800307519903896036339574493467807769775390625f,-0.02470003456547853470937070596846751868724822998046875f,-0.6827999808758498101468603636021725833415985107421875f,-0.0140999864801767404998145138961263000965118408203125f,0.555599979856610470818623070954345166683197021484375f,0.10920009972453126323443939327262341976165771484375f,-0.263800063383578642373095135553739964962005615234375f,0.8229000993728643464919514372013509273529052734375f,-0.214400040334462804736403995775617659091949462890625f,0.5330000240057710669816515292041003704071044921875f,0.1788000007241963285053998333751223981380462646484375f,0.7630999810114509340763788713957183063030242919921875f,-1.042200006234645481839606873109005391597747802734375f,-0.183000003963708912646524140654946677386760711669921875f,0.39520000915527386808889787062071263790130615234375f,0.2687000148847700220500200884998776018619537353515625f,0.45510006388425938705921680593746714293956756591796875f,-0.2246999971263110296160192547176848165690898895263671875f,-0.769499997582286265895845644990913569927215576171875f,0.06780000229626892416234795746277086436748504638671875f,-0.51560001763105400840458969469182193279266357421875f,-0.2356000096589327963414461919455789029598236083984375f,0.3798000204980371830032481739181093871593475341796875f,0.28650001200288544023209169608890078961849212646484375f,1.072300047245622334202153069782070815563201904296875f,-0.0487999988242984183983708135201595723628997802734375f,-0.526899989894032305670634741545654833316802978515625f,-0.11490000533163546858617110046907328069210052490234375f,0.5568000010371207952886152270366437733173370361328125f,0.2326000056207180310963167357840575277805328369140625f,0.35760002566278037416935831060982309281826019287109375f,0.068900001502037067080408405672642402350902557373046875f,0.0564999944567678991091952411807142198085784912109375f,0.1856000001966953139032057151780463755130767822265625f,0.686200008365511759933497160091064870357513427734375f,0.3070000031515955907224224574747495353221893310546875f,-0.1352999989807610603520515724085271358489990234375f,-0.20020000818669803077654023581999354064464569091796875f,-1.331400039127469181465812653186731040477752685546875f,-0.39679999943524590921839489965350367128849029541015625f,-1.9411000782936813635615180828608572483062744140625f,-0.325300003153085715013048684340901672840118408203125f,0.68140000507831566256555788640980608761310577392578125f,0.5775999865472325467408154509030282497406005859375f,-1.423199990469217635080667605507187545299530029296875f,0.195200011036545273412912138155661523342132568359375f,0.56579998595714575682080749174929223954677581787109375f,-0.60000000251457119571085740972193889319896697998046875f,0.18659998774677533095456283263047225773334503173828125f,-1.596500067491084973170245575602166354656219482421875f,1.1134000280857083708951904554851353168487548828125f,-0.32399998790025730244224178022705018520355224609375f,-0.3672000376760957696120613036327995359897613525390625f,1.2298000105142588012085980153642594814300537109375f,-0.995200008484720566315218093222938477993011474609375f,0.00069999955892541265711770392954349517822265625f,-0.056799994853138802142211716272868216037750244140625f,-0.17319997571706746697373091592453420162200927734375f,-0.0862000191688536698819689263473264873027801513671875f,-0.4397999807491890233990261549479328095912933349609375f,0.8310000069960945534575103010865859687328338623046875f,-0.57319999626204343190494228110765106976032257080078125f,0.5488000043749805545445497045875526964664459228515625f,-0.196499995648860448227424058131873607635498046875f,0.229800011929869096860556965111754834651947021484375f,-0.9928000131368646119511822689673863351345062255859375f,1.2489999864101413162842391102458350360393524169921875f,-0.855400010412930900116634802543558180332183837890625f,0.610999997273087291915771857020445168018341064453125f,-1.2061000509500505728510688641108572483062744140625f,0.23110003753900532075249429908581078052520751953125f,-0.269899996428191624175951801589690148830413818359375f,-1.5037000141024579047410725252120755612850189208984375f,-0.215499993421137236992990438011474907398223876953125f,0.0469000012934215337878640639246441423892974853515625f,-0.05389999900609254712691154054482467472553253173828125f,0.01430001191496810264425221248529851436614990234375f,0.140699997211992666734658996574580669403076171875f,0.1647000044465063428589246541378088295459747314453125f,-0.0346000005394220266907723271287977695465087890625f,0.097199997219443279306005933904089033603668212890625f,0.0670999984532594517805392797527019865810871124267578125f,1.2390000332742931998808444404858164489269256591796875f,0.2244000101596117513480521665769629180431365966796875f,-0.354199981558322551933315480710007250308990478515625f,-0.0201000047639012802136448954115621745586395263671875f,-0.032700006175041540501524650608189404010772705078125f,0.04090000010877847813617336214520037174224853515625f,0.7930999899148940546211861146730370819568634033203125f,0.23169999850839373412014765563071705400943756103515625f,0.54570000805258722653690028892015106976032257080078125f,0.106500000454485432843654280077316798269748687744140625f,0.118000007241964244286691609886474907398223876953125f,-0.012699998612701879352471223683096468448638916015625f,0.6349000054061415543316115872585214674472808837890625f,0.25520000259876252091117976306122727692127227783203125f,0.85420001605451101323041029900196008384227752685546875f,-0.096699999216198928575494164761039428412914276123046875f,-1.38049999420344793321646648109890520572662353515625f,-0.208099954152107358140710857696831226348876953125f,1.56159994926452672103778240852989256381988525390625f,-0.85840001795291787090036450535990297794342041015625f,-0.1761999887704852607583916324074380099773406982421875f,0.2270999766975645428601637831889092922210693359375f,-0.4699999811500312763001829807762987911701202392578125f,0.4614000224754202239552114406251348555088043212890625f,0.145199972647429120797823998145759105682373046875f,0.23609999194145192547011902206577360630035400390625f,-0.4239999902844429069403986432007513940334320068359375f,-0.37520000997483737847915108432061970233917236328125f,0.018600003191828751170078248833306133747100830078125f,0.808900021246075962011445881216786801815032958984375f,-0.551600011700391856805936185992322862148284912109375f,-0.3731000000476836220286713796667754650115966796875f,0.784800013196468082554702050401829183101654052734375f,-0.9242000018984082299056126430514268577098846435546875f,0.6667999830812216810471682038041763007640838623046875f,0.598199982012808550990712319617159664630889892578125f,-1.0467999643802645692858277470804750919342041015625f,-0.024499999947845463310613922658376395702362060546875f,0.41460001197606277667517815643805079162120819091796875f,-0.72559998885318643946362726637744344770908355712890625f,0.7269999771118165199368377216160297393798828125f,0.5580000032186511749188184694503434002399444580078125f,-0.9689999828338624610069018672220408916473388671875f,1.082999982915818559803255993756465613842010498046875f,-0.71059998771697274744241212829365395009517669677734375f,-0.46499997561797468126343346739304251968860626220703125f,0.08170001456439468512371604447253048419952392578125f,-0.088200000117719223879930723342113196849822998046875f,-1.1829000189810991994221467393799684941768646240234375f,-0.23320000481158498217837404808960855007171630859375f,-0.6247000049680473399149605029379017651081085205078125f,0.08800000072270630635529187202337197959423065185546875f,-0.39029999205172050125156602007336914539337158203125f,-0.1648000009402632581867464978131465613842010498046875f,-0.830900040677189810622849108767695724964141845703125f,-0.169800002430379404216864713816903531551361083984375f,-0.266700003910064875611851675785146653652191162109375f,0.1779000047132373030223106979974545538425445556640625f,0.401900009563565419057340477593243122100830078125f,-0.0944999985694885413778365546022541821002960205078125f,0.30680002942383310937657370232045650482177734375f,0.317700006845593463378207843561540357768535614013671875f,0.627800036457181409588201859151013195514678955078125f,-0.270000003762543183682964809122495353221893310546875f,-1.362300041285157359283175537711940705776214599609375f,-0.1027999981239438032076805029646493494510650634765625f,0.57490001859366923753213995951227843761444091796875f,0.38540000575631860879610712800058536231517791748046875f,1.8667000398218631662672351012588478624820709228515625f,0.20929999846667046714543403140851296484470367431640625f,0.2641000094652172780484988834359683096408843994140625f,-0.0936000000625848704860487714540795423090457916259765625f,-1.1500000174343585346292684334912337362766265869140625f,-0.27120000154823065141584947923547588288784027099609375f,-0.2788999823540445621716798996203579008579254150390625f,-0.4631999840617186947611116920597851276397705078125f,1.10159996885955369094745037727989256381988525390625f,-0.78820003472566657620745900203473865985870361328125f,0.2660000081509352742870078145642764866352081298828125f,-0.9813999836951499133874676772393286228179931640625f,0.8791999679982664961386262802989222109317779541015625f,-0.8907999804064632431988002281286753714084625244140625f,0.991399976393580573130748234689235687255859375f,0.040299979758263049234301433898508548736572265625f,-0.107999973490834566547391659696586430072784423828125f,0.7870000224113464781794391456060111522674560546875f,-0.232400008833408389108399205724708735942840576171875f,1.2400999814808368881813294137828052043914794921875f,-0.85799997796118265824816262465901672840118408203125f,0.888800005194544606723638935363851487636566162109375f,-0.8756000032514332787059174734167754650115966796875f,0.18330000100731869139281116076745092868804931640625f,0.44720000675618631902352717588655650615692138671875f,-0.3451999933600422565405096975155174732208251953125f,0.673599993357062221122077971813268959522247314453125f,0.7622000086188318590529888751916587352752685546875f,-1.39979999211132533076806794269941747188568115234375f,1.37990001509189585959802570869214832782745361328125f,-0.7179999893605708560784250948927365243434906005859375f,0.694700008171796667966191307641565799713134765625f,0.0347999982208013758366860201931558549404144287109375f,-0.3248999914363024377195188208133913576602935791015625f,0.72539999275505540055064557236619293689727783203125f,0.072800060299039071054494343115948140621185302734375f,1.07490001531541334856001412845216691493988037109375f,0.1775000004470348446972138845012523233890533447265625f,0.055599994161724897168141978909261524677276611328125f,-0.04799999669939281066177727552712894976139068603515625f,0.013700009371340249320070370231405831873416900634765625f,0.2030999960429965833785814766088151372969150543212890625f,1.10899998838454383331963981618173420429229736328125f,0.1098999942302703036745015197084285318851470947265625f,0.603800002586841078056067999568767845630645751953125f,-0.03429999787062409932758555441978387534618377685546875f,-0.4860999852359293438297527245595119893550872802734375f,-0.0151999992460012389727808113093487918376922607421875f,0.781399980938434435273620692896656692028045654296875f,0.277699996452033470717424279428087174892425537109375f,0.6920999705135812973821884952485561370849609375f,-0.21770000896900898457175799194374121725559234619140625f,-1.03430001102089885733903429354541003704071044921875f,-0.1651999913483856996521126347943209111690521240234375f,-1.01720003313124163213387873838655650615692138671875f,-0.301100013101101016133043231093324720859527587890625f,-1.216600044780970169000511305057443678379058837890625f,-0.12639999926686285025567713091732002794742584228515625f,0.28320000045299575930357605102472007274627685546875f,0.1964000070899725169937255486729554831981658935546875f,0.889400042271614399425061492365784943103790283203125f,-0.1529000020310282703661641789949499070644378662109375f,-0.39650000004470342673812410794198513031005859375f,0.155800003726780433854770535617717541754245758056640625f,0.958200040686131071510089896037243306636810302734375f,0.695600048029422879380945232696831226348876953125f,-0.29580001136362543689983795047737658023834228515625f,1.0274000016808511048793661757372319698333740234375f,-0.426199991676211364932669312111102044582366943359375f,0.5476000004053112046875639862264506518840789794921875f,-0.050799982529878828785285804769955575466156005859375f,1.211199965859948601831774794845841825008392333984375f,-1.349199977907538094967776487465016543865203857421875f,1.4256999837636943340157813508994877338409423828125f,-0.575999988555908171150576890795491635799407958984375f,0.42150000414252275504622957669198513031005859375f,0.1992000022709368156625941992388106882572174072265625f,0.90689996180534393488414934836328029632568359375f,-1.400599941933155623274842582759447395801544189453125f,0.4086999267816542324993633883423171937465667724609375f,-0.2363999460697179255674882369930855929851531982421875f,-0.89709998601078932978225566330365836620330810546875f,1.232599964940547909719725794275291264057159423828125f,-0.51090002331733774809663373162038624286651611328125f,0.06120001104772132549669549916870892047882080078125f,-0.405999997794628253444670917815528810024261474609375f,0.49939998639225968712906933433259837329387664794921875f,0.22389997497797065051372555899433791637420654296875f,-1.0473999787330630084625227027572691440582275390625f,0.7903999935746188754137619980610907077789306640625f,-0.328600001180171741310687139048241078853607177734375f,0.42080003915429131655656647126306779682636260986328125f,0.04879998351335534945150129715329967439174652099609375f,1.3046000474780805600261146537377499043941497802734375f,-1.0631999991863967380822941777296364307403564453125f,-0.169699999752640717787244284409098327159881591796875f,-0.9104000145852564340742674175999127328395843505859375f,0.129199998620152467498911619259160943329334259033203125f,0.718800015997886898588831172673963010311126708984375f,0.1073000054106116873953169488231651484966278076171875f,-0.7977000151902438052076149688218720257282257080078125f,-0.1477000037163496148906460803118534386157989501953125f,0.01440001887679098757644169381819665431976318359375f,0.1156000077962875405290787966805510222911834716796875f,0.28560001688599623292930118623189628124237060546875f,0.0944999962970614060253637944697402417659759521484375f,0.4822999971777193195521249435842037200927734375f,0.137399997024238107456994839594699442386627197265625f,0.87170000778436651511782429224695079028606414794921875f,0.0784999975338578348527107664267532527446746826171875f,0.173999999448656961220649463939480483531951904296875f,-0.02779999782592057488983527946402318775653839111328125f,-0.739300009828805837486243035527877509593963623046875f,-0.3402000058770180412892614185693673789501190185546875f,-1.5782000184088946781457707402296364307403564453125f,-0.14309999542832374519463201067992486059665679931640625f,-0.7199999714642759673921545982011593878269195556640625f,0.039200000818073732489210669882595539093017578125f,0.2386999992758038757756366976536810398101806640625f,-0.031799998094141468385487314662896096706390380859375f,0.46770002964437029646660448634065687656402587890625f,0.1920000079944730320136159207322634756565093994140625f,0.4029000354468827982401535336975939571857452392578125f,-0.2996999458476905520143418470979668200016021728515625f,1.29739994960129312318031225004233419895172119140625f,-0.73529998236596583183199982158839702606201171875f,0.0683999940365553893428796072839759290218353271484375f,0.1127000355675826792634097728296183049678802490234375f,0.54439999831318797163248746073804795742034912109375f,-0.51570000820159922483298942097462713718414306640625f,0.2971999993056060329621459459303878247737884521484375f,-0.0053999551296239900466389372013509273529052734375f,0.56899994572997147912474247277714312076568603515625f,1.40720000653266907875149627216160297393798828125f,-1.04559996652007125561567590921185910701751708984375f,1.100400005495547617329066270031034946441650390625f,-1.1716000007480378286572886281646788120269775390625f,-0.10089998230189056727112983935512602329254150390625f,0.01260000021904705835851245865342207252979278564453125f,-0.97640000003054738808572210473357699811458587646484375f,0.584599980124831386518735598656348884105682373046875f,-1.2020000283718115241526902536861598491668701171875f,1.185199994105100795849239148083142936229705810546875f,-0.8238000034809118687917361967265605926513671875f,0.825199967059493388887858600355684757232666015625f,-0.17969996272325605701780659728683531284332275390625f,0.3981999583572155554378468877985142171382904052734375f,1.04909999083876659398129049804992973804473876953125f,-1.0275999696344140232895369990728795528411865234375f,1.5014000265359879193738379399292171001434326171875f,-1.02700001265108564751926678582094609737396240234375f,0.20420002473443776835893004317767918109893798828125f,-1.88150003603100746119025643565692007541656494140625f,-0.31020000312030315825495563331060111522674560546875f,-0.52120003861486896568067095358856022357940673828125f,-0.014200002308189851873976294882595539093017578125f,0.1364999893158671540049908799119293689727783203125f,-0.19229999717772006118110539318877272307872772216796875f,-0.2110000135153533573628692465717904269695281982421875f,0.281099990041553893238557293443591333925724029541015625f,1.384399992315470395709553486085496842861175537109375f,0.218600001142919053709334775703609921038150787353515625f,1.27720000370144770585056903655640780925750732421875f,0.2574999945610760843095476957387290894985198974609375f,0.393800004188715746522575500421226024627685546875f,0.0485000004582107069328600346125313080847263336181640625f,-0.9144000280037527517151829670183360576629638671875f,-0.243700010079145545915935144876129925251007080078125f,-2.029200043323637014935911793145351111888885498046875f,-0.35910000581443313194540678523480892181396484375f,-1.117400041988492187527981513994745910167694091796875f,0.14840000100284811423989594914019107818603515625f,-0.345499990850687055399248492904007434844970703125f,0.0077000007733702968693023649393580853939056396484375f,0.916300014880299773523120165918953716754913330078125f,0.1863000101864338287072087041451595723628997802734375f,0.309000043630600629285254399292171001434326171875f,0.04609999898225070058543906270642764866352081298828125f,0.483500001966953529830561819835565984249114990234375f,-0.16679999686479562992502678753226064145565032958984375f,0.4200000093132263412343263553339056670665740966796875f,0.6434999802112575384427373137441463768482208251953125f,0.1563999936044215832708914604154415428638458251953125f,-0.61119999298006266741367653594352304935455322265625f,-0.9967999767854809345379862861591391265392303466796875f,-0.4363000095531346378407988595427013933658599853515625f,-1.07529998780488966758639435283839702606201171875f,0.7416000184431672526130796541110612452030181884765625f,0.43409999858736991740215671597979962825775146484375f,-0.16199999924004071516492331284098327159881591796875f,0.899300002527236852500891473027877509593963623046875f,0.82150003848969976871785547700710594654083251953125f,-0.108699985752999717103506327475770376622676849365234375f,-0.6799000293970112185348853017785586416721343994140625f,-0.18169999899268152177000956726260483264923095703125f,0.832199992898106533090185621404089033603668212890625f,0.065000005587935394402165911742486059665679931640625f,0.92229999382495908566426123798009939491748809814453125f,0.28310000287890435544113643118180334568023681640625f,0.15160001557469382493792409150046296417713165283203125f,0.161200001287460370491544381366111338138580322265625f,-0.2123999954968687209344579969183541834354400634765625f,-0.7601999830782413258845053860568441450595855712890625f,-1.01439998467862579900611308403313159942626953125f,0.4648999943792819777854674612171947956085205078125f,-0.73539999722540405624471304690814577043056488037109375f,-0.0249000118136404324786781216971576213836669921875f,-0.15470000746399203972458735734107904136180877685546875f,0.531500015318393526086993006174452602863311767578125f,-0.7372999868586653793300911274855025112628936767578125f,-0.42009999742507897479981693322770297527313232421875f,0.622300002616643865849255234934389591217041015625f,-0.1514000053018331737320067986729554831981658935546875f,0.18060002612471581784348018118180334568023681640625f,0.7602000301659106451523939540493302047252655029296875f,1.976100019359588344514122582040727138519287109375f,0.492800030347705586564188706688582897186279296875f,0.01299999055266365388661142787896096706390380859375f,-2.03300002661347445354067531297914683818817138671875f,0.6732000135660172190910088829696178436279296875f,1.018199985849857824860009714029729366302490234375f,-0.255600013756751565097147249616682529449462890625f,-0.81640001044273358132841167389415204524993896484375f,0.276900015151500866750211571343243122100830078125f,0.0928999954372642378075397573411464691162109375f,0.27999999560415744781494140625f,-1.021699992957711078389593239990063011646270751953125f,-0.7197999836921695759883732534945011138916015625f,1.349700007799267353902905597351491451263427734375f,-1.12400000742077832427412431570701301097869873046875f,-0.7184000106513501737737215080414898693561553955078125f,-0.097999984741210965921709430404007434844970703125f,-0.5389000204265119720048460294492542743682861328125f,0.5142000030159952217445606947876513004302978515625f,0.502200016665458992548565220204181969165802001953125f,-1.029600005531310813466916442848742008209228515625f,0.33370001268684834627720192656852304935455322265625f,0.808999979034066285521475947462022304534912109375f,-0.805899990311264691200676679727621376514434814453125f,0.5466999701216821438976012359489686787128448486328125f,0.56139999345541002906401217842358164489269256591796875f,-0.2021999971076844648809611726392176933586597442626953125f,0.40370000508725645271823623261298052966594696044921875f,-0.04440000989884129722895522718317806720733642578125f,1.056700013297795948119528475217521190643310546875f,-0.05379998753666825894725889156688936054706573486328125f,-0.25050000225007529319753984964336268603801727294921875f,0.7737999754294750953675929849850945174694061279296875f,0.6926000045776365965366494492627680301666259765625f,0.925399987837672188817350615863688290119171142578125f,-0.474999989867210370420025355997495353221893310546875f,-0.20109996452331557748038903810083866119384765625f,0.19650000905990605559736650320701301097869873046875f,0.32970000761300310987422790276468731462955474853515625f,0.0624000003397464642063141582184471189975738525390625f,-0.64250000759959202678572864897432737052440643310546875f,0.08369998623728758246897996286861598491668701171875f,0.4408999805510045444378874890389852225780487060546875f,-0.74650000257790083413311776894261129200458526611328125f,0.43340000212043527927363584240083582699298858642578125f,0.0190000060796737813006984652020037174224853515625f,-0.04600002357363719429628190482617355883121490478515625f,0.5656999991118907278320193654508329927921295166015625f,-0.21169999508112680164373387015075422823429107666015625f,-0.423000006049871313251742321881465613842010498046875f,0.940100008638202755406609867350198328495025634765625f,0.2232999968349933528344308797386474907398223876953125f,0.0186000316902996676304837819770909845829010009765625f,0.3361000409662724397463762215920723974704742431640625f,1.590599984550475909372835303656756877899169921875f,-0.72389997274279505035110560129396617412567138671875f,-0.8889999616742141341774186003021895885467529296875f,0.47059999170303346005539424368180334568023681640625f,-0.2501999996185304553364403545856475830078125f,0.093499996006489283217888441868126392364501953125f,-0.364299976897239474737943965010344982147216796875f,-1.02910004388690001775330529198981821537017822265625f,-1.6472999920368192050545985694043338298797607421875f,0.73699998918175690931775534409098327159881591796875f,-0.4958999908328056704931441345252096652984619140625f,-0.688500025585293773389139460050500929355621337890625f,-0.39219998544752598945706267841160297393798828125f,0.282899990966915648726853760308586061000823974609375f,1.383499995112419611587029066868126392364501953125f,0.00360002867281483673878028639592230319976806640625f,-0.44570000045299540403220817097462713718414306640625f,-0.3572999925583599800660294931731186807155609130859375f,-1.1219999616891147553587870788760483264923095703125f,-0.52490003260076090185748398653231561183929443359375f,-0.531499981641768926010627183131873607635498046875f,1.1496000135779382844702922739088535308837890625f,0.8732000314474106517081963829696178436279296875f,1.3157000415802002635246026329696178436279296875f,-0.321200025129318333938499563373625278472900390625f,0.33080001387819635283449315465986728668212890625f,1.175900002101808450305497899535112082958221435546875f,0.0602000031016765024816095319692976772785186767578125f,0.4989999901726840381144256753032095730304718017578125f,0.822499993890523750650345391477458178997039794921875f,0.4884999746978289092425029593869112432003021240234375f,-0.41029998944401757654532048036344349384307861328125f,0.187599941955507276691861306971986778080463409423828125f,-0.2647999950170518701497712754644453525543212890625f,0.600100008004904150737957024830393493175506591796875f,-0.500999972462654152849381716805510222911834716796875f,0.5172000002741812263451492981403134763240814208984375f,-0.8930999921500681892894135671667754650115966796875f,0.8139000069037078954892194815329276025295257568359375f,-0.7039999816417694677994632002082653343677520751953125f,-0.98519999116212109147028286315617151558399200439453125f,-0.0935999971568584054892880885745398700237274169921875f,-0.270299993839860253075357832130976021289825439453125f,0.823800005567073778678377493633888661861419677734375f,-0.694799991440773201389902169466949999332427978515625f,-0.03260000837743282087188845252967439591884613037109375f,0.1648999877482655307403547340072691440582275390625f,-0.27979999114274978211369671043939888477325439453125f,0.518199974972009425044916497427038848400115966796875f,0.14480000619292265895410309894941747188568115234375f,-0.41809998876005416423851102081243880093097686767578125f,0.5651000181376932829380166367627680301666259765625f,-1.031499973297119066018012745189480483531951904296875f,-0.35150001412630071939702247618697583675384521484375f,-0.14279997567906954625271964687271974980831146240234375f,0.3713000028848647549040151716326363384723663330078125f,-0.331600014028698097945380141027271747589111328125f,0.559100024627149583267282650922425091266632080078125f,-0.78090000305175788497535904753021895885467529296875f,-1.591000010758638172347900763270445168018341064453125f,0.851000038027763139325543306767940521240234375f,-0.4415000236630444163665742962621152400970458984375f,0.6420999834775926728980266489088535308837890625f,-1.2016999812602993102927939617075026035308837890625f,1.048200014162063098410726524889469146728515625f,-0.060500035107136085343881859444081783294677734375f,-1.58399998700618738212142488919198513031005859375f,1.3132000029116870365442082402296364307403564453125f,0.2851999733179813034666949533857405185699462890625f,-0.57329999199211645333207343355752527713775634765625f,0.4407999948978424953338617342524230480194091796875f,1.74630000079870217888355909963138401508331298828125f,-0.0691999937772749262876459397375583648681640625f,-1.46500003337860107421875f,-0.58059999051094024480335065163671970367431640625f,-1.359799990360438304293211331241764128208160400390625f,0.377499991059302875129333187942393124103546142578125f,1.7247999657362693692874699991079978644847869873046875f,0.426999964594841685538995079696178436279296875f,-0.1658999926209450137548628845252096652984619140625f,1.2229999983012671549431615858338773250579833984375f,-0.7901000028640030592441689805127680301666259765625f,-1.04169995890855826559118213481269776821136474609375f,-1.314299994331598053776133383507840335369110107421875f,-0.449099973702431043420801870524883270263671875f,-0.0784999853707848416206616093404591083526611328125f,-0.408800009664892716187978294328786432743072509765625f,-0.65639999516904357079738474567420780658721923828125f,0.80500000774860414054501234204508364200592041015625f,0.65830001203417776167725605773739516735076904296875f,1.2617999853909014351671658005216158926486968994140625f,0.34639999464750292901271677692420780658721923828125f,0.9504000073581935215116800463874824345111846923828125f,0.7509999934732913828838718472979962825775146484375f,0.021899949623644499041574817965738475322723388671875f,-0.68089999411106116866676529753021895885467529296875f,-0.62619997830241924674510300974361598491668701171875f,0.03179999179840109491124167107045650482177734375f,-0.9669999626576906148756052061798982322216033935546875f,0.5782999867022038387176507967524230480194091796875f,0.7362000147357583568208383439923636615276336669921875f,-0.58789997569322594017648953013122081756591796875f,0.68590000361055114552755185286514461040496826171875f,-0.4513999810874462781384863774292171001434326171875f,0.5500999922469258507362610544078052043914794921875f,0.468899976952374064609330162056721746921539306640625f,0.6626000236511229157798652522615157067775726318359375f,-0.368699995028972438149139634333550930023193359375f,-1.107799952377379160139980740495957434177398681640625f,0.40029999927878368026767930132336914539337158203125f,-0.401600020305811877818769062287174165248870849609375f,-0.438599982479214833830383213353343307971954345703125f,0.7033999507114294669207765764440409839153289794921875f,0.24629999707341188042164503713138401508331298828125f,0.5981999333418908104675892900559119880199432373046875f,-0.309599991524219575467213871888816356658935546875f,0.90089999828338562792851007543504238128662109375f,0.4498000009775164897973809274844825267791748046875f,-1.7580999773979186784345074556767940521240234375f,-0.688000036150217653130312100984156131744384765625f,-1.791599997991323078849745797924697399139404296875f,0.54270000036358823081172886304557323455810546875f,-0.782100007021426790743134915828704833984375f,-0.705099987925589033466167165897786617279052734375f,0.415099985578656127671592912520281970500946044921875f,1.199700032684206885136291020899079740047454833984375f,0.2507999788045882638698458322323858737945556640625f,-0.018400041925162202716137471725232899188995361328125f,-0.092900006706267301836987826391123235225677490234375f,-0.4802999699048700055215022075572051107883453369140625f,-0.3050999616622931398524087853729724884033203125f,-0.881500036459416325129723190912045538425445556640625f,0.1230000088438389926892568837502039968967437744140625f,1.3193999997660512679686917181243188679218292236328125f,1.26309996860623385828148457221686840057373046875f,-0.1594999614655971953425250831060111522674560546875f,0.74719996600151006305168266408145427703857421875f,1.381800010033697123645879401010461151599884033203125f,-0.9696999828025696377409303750027902424335479736328125f,-0.597300019641220369948086954536847770214080810546875f,-1.48579998893737741383347383816726505756378173828125f,-1.671300001245736854826873241108842194080352783203125f,0.137500029206275797832859097979962825775146484375f,0.5036999947309492853264600853435695171356201171875f,-0.2452000230133533076326557420543394982814788818359375f,0.58139998868703823831083354889415204524993896484375f,-0.361400008505582182039006511331535875797271728515625f,-0.9513999886870383448922439129091799259185791015625f,0.0638999785542484488587433588691055774688720703125f,-0.63890000269412983158190399990417063236236572265625f,0.840399995772540098215586112928576767444610595703125f,-0.457999996811151444120469022891484200954437255859375f,-0.50550000590086019425228869295096956193447113037109375f,0.61689997793585060748000614694319665431976318359375f,0.79040000713467595261363385361619293689727783203125f,-0.1678000050902366258043230118346400558948516845703125f,-0.9963999776415526365980213086004368960857391357421875f,-0.0895000025928018860810198020772077143192291259765625f,0.7680000248812139940213228328502736985683441162109375f,-0.2076999862074853098192761535756289958953857421875f,0.28639997250065263312279739693622104823589324951171875f,0.299399991887062899564853069023229181766510009765625f,-0.07809997790083300639452090763370506465435028076171875f,0.40169999213814744365436126827262341976165771484375f,-2.022100000390410823314368826686404645442962646484375f,0.663199963647127077592813293449580669403076171875f,-0.53150002349540559354323931984254159033298492431640625f,-0.68369997750148192583452555481926538050174713134765625f,0.4629999984689054937092578256851993501186370849609375f,-0.796799983826279589749219667282886803150177001953125f,1.0384999936223027194870383027591742575168609619140625f,0.509200002419948560827833716757595539093017578125f,0.17269996191859238177102042754995636641979217529296875f,0.01209999994337562778667916063568554818630218505859375f,-1.325700028467178270830117980949580669403076171875f,-0.97570005975961748845293186604976654052734375f,-1.91959996500015250830983859486877918243408203125f,0.22219999394118783442308995290659368038177490234375f,-1.47500003635883292218977658194489777088165283203125f,-0.005200034338236125819321387098170816898345947265625f,-1.30239998737573614562279544770717620849609375f,0.3570000095963479225247283466160297393798828125f,0.52170001642704022515317774377763271331787109375f,0.79610005061477462362518053851090371608734130859375f,0.255299986016750413142517572850920259952545166015625f,0.37709997930526772957904313443577848374843597412109375f,-1.5184999656081199503887546597979962825775146484375f,-1.366199969696998639534513131366111338138580322265625f,-0.008699979829788873075813171453773975372314453125f,0.8570000079572199780386654310859739780426025390625f,1.6198999918460845748313658987171947956085205078125f,0.37189999637603765592075433232821524143218994140625f,0.13960003831386558204030734486877918243408203125f,1.385000042617321014404296875f,0.542399970686435750621967599727213382720947265625f,-0.4015000235140320228310883976519107818603515625f,-1.4071999600410460828925351961515843868255615234375f,0.80739998677969015972166744177229702472686767578125f,-0.61420001076459840305687976069748401641845703125f,-0.7513999581396575422331807203590869903564453125f,0.347800056499242060681353905238211154937744140625f,-0.1385000554621225177243104553781449794769287109375f,0.849100017735362033732826603227294981479644775390625f,-0.91060001569390269793302650214172899723052978515625f,0.51279997398406240716184356642770580947399139404296875f,-0.91559997888803490440068344469182193279266357421875f,1.137199994686245663189083643374033272266387939453125f,-0.976000013366341345744103819015435874462127685546875f,-0.37660000708103213984401236302801407873630523681640625f,-0.713799989771843002017703838646411895751953125f,-1.3969000128418207484770618975744582712650299072265625f,0.5371000076174734516598618938587605953216552734375f,0.003100006902217788962161648669280111789703369140625f,0.058099997179210161046825078301480971276760101318359375f,-0.27529999886900158312386110992520116269588470458984375f,-0.9672999754965305729825786329456605017185211181640625f,-0.1767000163152812053368734268588013947010040283203125f,0.48199998649954789442517721909098327159881591796875f,0.43179998043626588444254821297363378107547760009765625f,0.88879999334812165301400455064140260219573974609375f,-1.53669999314397554002198376110754907131195068359375f,0.283600016528367859791615046560764312744140625f,-0.1476999916091561448894253771868534386157989501953125f,-0.0112000060558319347592259873636066913604736328125f,-0.19530001559555520973532338757649995386600494384765625f,-0.85549996715784093481715899542905390262603759765625f,-0.6707000050723552764253554414608515799045562744140625f,-0.5079999942779540589299358543939888477325439453125f,0.359500043720007322889387069153599441051483154296875f,0.23460001849532119422292453236877918243408203125f,1.66169996825903698578486000769771635532379150390625f,-0.06029997543692644512702827341854572296142578125f,1.15129999006986594167756265960633754730224609375f,0.528800009441375618024494542623870074748992919921875f,-2.20519997518062638164337840862572193145751953125f,0.5377000288248066084406673326157033443450927734375f,1.4957999516844751042299321852624416351318359375f,0.7782000232517720661462590214796364307403564453125f,-0.0925000250339511609354303800500929355621337890625f,-0.62510003780722644961542755481787025928497314453125f,-1.106100003415345867097130394540727138519287109375f,-0.3471999908864500383742779376916587352752685546875f,-0.0180999976634979020673199556767940521240234375f,-0.8490000050365917871886267676018178462982177734375f,-1.3479999489784244559587023104541003704071044921875f,-0.81479999173879580354196150437928736209869384765625f,-0.87159996535778105908320867456495761871337890625f,0.32879994745254492727326578460633754730224609375f,0.3762999832153326451589236967265605926513671875f,-0.550900022721290127947213477455079555511474609375f,0.460400000131130582303740084171295166015625f,0.79680004208981980440285042277537286281585693359375f,0.71160001393556537863105404539965093135833740234375f,-0.045400013095140412389127959613688290119171142578125f,-1.378899970209598535575423738919198513031005859375f,1.04670003632009045446693562553264200687408447265625f,0.9078000102311367669472019770182669162750244140625f,0.12750002823770056892271895776502788066864013671875f,0.90490000972747708374299691058695316314697265625f,-0.431499986708164584570113220252096652984619140625f,-0.68869997364580637366771043161861598491668701171875f,0.8250999816298485445287269612890668213367462158203125f,-0.2618000509932636443721776231541298329830169677734375f,-1.14929998866915727973037064657546579837799072265625f,0.1388999828010799575395139981992542743682861328125f,0.668699976849556065872093313373625278472900390625f,-0.5027999793484807522503388099721632897853851318359375f,-0.40910001125335693217266452847979962825775146484375f,-0.8389000040352343479099772594054229557514190673828125f,-0.47330000324249255072572850622236728668212890625f,1.135499989025294365063700752216391265392303466796875f,-0.0079999941289425891000064439140260219573974609375f,1.26459996332377233585475551080889999866485595703125f,-0.7120999725997450013892375864088535308837890625f,1.6399999573454266954541935774614103138446807861328125f,-0.5081999864459039173425480839796364307403564453125f,1.2355999705433846624913485356955789029598236083984375f,-0.2544999826252460906061969581060111522674560546875f,-1.21759995525479336464513835380785167217254638671875f,0.31260000070333493482621634029783308506011962890625f,-0.848599976295232760747921929578296840190887451171875f,0.335200001332163655121121337288059294223785400390625f,-0.75920001881122534115320377168245613574981689453125f,-0.589499986797571207119972314103506505489349365234375f,0.21750000916421452501481326180510222911834716796875f,0.22040000504851331442068840260617434978485107421875f,-1.889099996165930495095608421252109110355377197265625f,0.62399998238682741202865145169198513031005859375f,-0.2604000325039022012418854501447640359401702880859375f,-0.63899999415874475516829988919198513031005859375f,-1.74600004100799566231216886080801486968994140625f,0.269599981242418351712331059388816356658935546875f,-0.0632999826788900321616893052123486995697021484375f,-1.448499997794627791591892673750407993793487548828125f,-0.9361999857902532085063285194337368011474609375f,1.0724999997019768471773204510100185871124267578125f,0.4502000031948085734256892465054988861083984375f,-0.5590999960541722657580976374447345733642578125f,2.20860000267028766529620043002068996429443359375f,-0.52130000489652150719166456838138401508331298828125f,0.95990000197887415112063536071218550205230712890625f,0.714200051146745895636058776290155947208404541015625f,0.880499995470046314949286170303821563720703125f,1.0402000434838234443901683334843255579471588134765625f,1.7699999640882013718368170884787105023860931396484375f,0.3091999725058673931243902188725769519805908203125f,-2.641199939894676163021358661353588104248046875f,0.2719000145554542058334845933131873607635498046875f,-1.765199988889694537874675006605684757232666015625f,-0.23270000438690185973200641456060111522674560546875f,0.87090002629756924079629243351519107818603515625f,-0.36350000822544092926591474679298698902130126953125f,0.043200021910667629754243534989655017852783203125f,0.12320001393854607130151634919457137584686279296875f,-0.8297999986678359363168056006543338298797607421875f,0.06510002223551314415317392558790743350982666015625f,0.375900017952918830133057781495153903961181640625f,0.10880004609823235028898125165142118930816650390625f,-0.533000010110438182664438500069081783294677734375f,-0.9312000266194342401604444603435695171356201171875f,1.2386999933153390873741273026098497211933135986328125f,-0.3844999733865261948295710681122727692127227783203125f,0.606399989171326314618681863066740334033966064453125f,-0.43220000221133236806281274766661226749420166015625f,0.9677999772623187002551503610447980463504791259765625f,-0.3232999858081342381410649977624416351318359375f,-0.706699988338350681971178346429951488971710205078125f,1.002500001490115977276218472979962825775146484375f,-0.9927999976769086298844513294170610606670379638671875f,0.703399994632601721633591296267695724964141845703125f,-1.3750000079348680959157036340911872684955596923828125f,0.6919999790489672619742123060859739780426025390625f,-1.22219999047666760816355235874652862548828125f,1.0328999700121581639677970088087022304534912109375f,-0.6469999995380637525244083008146844804286956787109375f,-1.33979996551275259975000153644941747188568115234375f,0.2685999725699426132763392160995863378047943115234375f,-0.931699999439716375349007648765109479427337646484375f,0.0833999843880537383711271104402840137481689453125f,0.614800012153386976621050052926875650882720947265625f,-0.58450000389665357491963959546410478651523590087890625f,0.424400004571676259246260087820701301097869873046875f,0.264699984963237877622077576234005391597747802734375f,-0.166400001502037042655501863919198513031005859375f,-0.143200007382035199299252781202085316181182861328125f,0.514799997550249077704620503936894237995147705078125f,1.2182999853983524918277225879137404263019561767578125f,2.01360004275441184518058435060083866119384765625f,0.040499980568885263210177072323858737945556640625f,-1.6638000465452673637400948791764676570892333984375f,-0.931099984043836315095177269540727138519287109375f,-0.285099991725385137897319509647786617279052734375f,-0.66639998484998930194933564052917063236236572265625f,1.316699978913366653188177224365063011646270751953125f,-0.86170002059936479099633288569748401641845703125f,-0.31319997452497450041164483991451561450958251953125f,1.003599993133544643342247582040727138519287109375f,0.84500003129243861366148848901502788066864013671875f,0.64579999579191138536771177314221858978271484375f,-0.3279000503480435924075209186412394046783447265625f,1.7218999868392952379281268804334104061126708984375f,0.26150003260374088398521053022705018520355224609375f,0.20989997098445911660746787674725055694580078125f,-0.2040999826431271202409334364347159862518310546875f,-0.61839996907711025642129243351519107818603515625f,0.300900009384751143670655437745153903961181640625f,1.28500001952052134157611362752504646778106689453125f,-0.9885999908983704731468833415419794619083404541015625f,0.22440004703998539525855449028313159942626953125f,-1.50970005719661770626771613024175167083740234375f,0.60190005896091491877086809836328029632568359375f,-0.613499973952770005780621431767940521240234375f,-0.49579998118877455226538586430251598358154296875f,0.9680000139474866927002949523739516735076904296875f,0.591599998438358198882269789464771747589111328125f,0.683499992728233252137215458787977695465087890625f,0.5242000133350490642669683438725769519805908203125f,-0.5847000029563904632823323481716215610504150390625f,-1.213000015325844227476181913516484200954437255859375f,-0.1265999966502189266748246154747903347015380859375f,1.1125999899372456258817010166239924728870391845703125f,-0.130999995708465544197451890795491635799407958984375f,0.510199990156292582099695209763012826442718505859375f,-0.853199992406368235009495037957094609737396240234375f,-0.64549996976554402383641217966214753687381744384765625f,0.474299992394447311738758799037896096706390380859375f,-0.2182000039920206635457589072757400572299957275390625f,-0.0302000008106233508442528545856475830078125f,-1.213299998399615109434535042964853346347808837890625f,0.96180000565648082755387804354541003704071044921875f,-1.4001999745845801204069402956520207226276397705078125f,0.34249997973442081189432428800500929355621337890625f,-0.6628999983057379807860343134962022304534912109375f,0.13960000374317171889515520888380706310272216796875f,0.586200002107024165098891899106092751026153564453125f,0.89670000949800010570100994300446473062038421630859375f,0.951600024403632005487452261149883270263671875f,0.4070000122785568663630328956060111522674560546875f,1.4839999827593566283923109949682839214801788330078125f,0.7886000284492968859240136225707828998565673828125f,0.7525999804750085342419652079115621745586395263671875f,0.20839998844861984395038234652020037174224853515625f,0.2974999693781139598058871342800557613372802734375f,0.34299999800324432186471312888897955417633056640625f,-1.434900021983683604531734090414829552173614501953125f,-0.274199997465312594879804919401067309081554412841796875f,0.031899995779991030531164142303168773651123046875f,-0.859199985283613187903029029257595539093017578125f,-0.0164999446868900889739961712621152400970458984375f,0.13500003248453129600648026098497211933135986328125f,1.639699974942207205685917870141565799713134765625f,0.08149996227025990691572587820701301097869873046875f,-1.5662999969243998066303902305662631988525390625f,0.5626999815881248423465876840054988861083984375f,-0.7982000046253201475110472529195249080657958984375f,-0.546099983000754907180862574023194611072540283203125f,-0.76039996764659889549875515513122081756591796875f,-0.3934000033497806469995339284650981426239013671875f,0.487999977886676816751787555404007434844970703125f,-1.21170004146099064001873557572253048419952392578125f,-1.57719999401569310748527641408145427703857421875f,0.601300033283234114378501544706523418426513671875f,1.328300030064582415434415452182292938232421875f,-0.4698999826073642083201775676570832729339599609375f,0.06280003697872160728365997783839702606201171875f,-0.112199978816509116086308495141565799713134765625f,0.815500015735626249124834430404007434844970703125f,1.077800041151046883669550879858434200286865234375f,0.88669999705552982049994170665740966796875f,-0.7433000509262086552553228102624416351318359375f,-1.89669999467134431370141101069748401641845703125f,0.743400025105476203179932781495153903961181640625f,-1.09480003614425669411502894945442676544189453125f,0.7791999970555298915542152826674282550811767578125f,-0.4528000010110437845156639014021493494510650634765625f,0.2502999866127968431328554288484156131744384765625f,0.04529998150914937848909858075785450637340545654296875f,0.53880001063346849576873864862136542797088623046875f,1.1969999701455236174041374397347681224346160888671875f,1.040799979251623152975980701739899814128875732421875f,0.324500002555549116323163616470992565155029296875f,-0.8540999921798704264119805884547531604766845703125f,-0.244900000078976720629952978924848139286041259765625f,-0.5299999929964542833005225475062616169452667236328125f,0.8354999927878379306633860323927365243434906005859375f,-0.47429997496008891033625332056544721126556396484375f,1.27159996729493141032207859097979962825775146484375f,0.045399997299909511383475546608678996562957763671875f,0.0898999808564779900876828833133913576602935791015625f,-0.620699993002414629472696105949580669403076171875f,-0.1474000105097890145344763368484564125537872314453125f,0.79020001317858667988502929802052676677703857421875f,-0.9574000139370555384488170602708123624324798583984375f,0.4878000098586081634266520268283784389495849609375f,0.7610000003278256652805566773167811334133148193359375f,0.71890000075697901849025583942420780658721923828125f,0.53429999716281917354621100457734428346157073974609375f,0.155599997067451312204866553656756877899169921875f,-0.8667000210463997600385255282162688672542572021484375f,-0.839699996548890936765019432641565799713134765625f,0.496600010657310075856685216422192752361297607421875f,-0.74890002266168576028348979889415204524993896484375f,0.3748999960556620436591401812620460987091064453125f,-0.5415000205338003080868247707257978618144989013671875f,-1.1228000322103497410353156737983226776123046875f,-0.066100022302567840171150237438268959522247314453125f,-0.192499988973140911951986709027551114559173583984375f,1.1708000010818244707166968510136939585208892822265625f,-0.5543000376939772877449286170303821563720703125f,-0.296300021902472732193700721836648881435394287109375f,-0.5638000083610414936430288435076363384723663330078125f,1.527300024409592449359251986606977880001068115234375f,-1.216200006055832005813499563373625278472900390625f,1.40799999368190764670316639239899814128875732421875f,-0.535500041365622792000067420303821563720703125f,-0.450699917900561075612131389789283275604248046875f,-0.78449998843669987991233938373625278472900390625f,0.231099947834014329828278278000652790069580078125f,0.46509995868206033264868892729282379150390625f,-0.469400010383129284718961571343243122100830078125f,0.27750000387430162618329632095992565155029296875f,0.39739999676346737800258779316209256649017333984375f,-0.419800002020597418095348984934389591217041015625f,0.94819999464154225421452792943455278873443603515625f,-1.0630000145435332115084747783839702606201171875f,-0.1176999868035313312475409475155174732208251953125f,0.634099976086616834436426870524883270263671875f,0.212800045919419034134989487938582897186279296875f,-0.322899978971481260714426753111183643341064453125f,0.4381999881595373125264814007095992565155029296875f,-1.54700000512599888935483249952085316181182861328125f,-0.581600037926435664559221550007350742816925048828125f,1.3446999992310997296129926326102577149868011474609375f,-0.78580000979900344049156046821735799312591552734375f,0.49959998653233039789256508811376988887786865234375f,-0.3421999871239067569916869615553878247737884521484375f,-1.03739996565729342137274215929210186004638671875f,-0.70380001733899089089163680910132825374603271484375f,-1.39629999467060006423935192287899553775787353515625f,-0.1954000055514276412704788299379288218915462493896484375f,-0.75770001796558494877587008886621333658695220947265625f,-0.913299995493888960851336378254927694797515869140625f,-1.4416000164315108111878771524061448872089385986328125f,-0.6610999971568585209524826495908200740814208984375f,0.36310003253221534436079309671185910701751708984375f,-0.676300003182888076480594463646411895751953125f,0.5671999804183844862137675590929575264453887939453125f,-0.209700012493133414182011620141565799713134765625f,-1.334599972935020861797283941996283829212188720703125f,0.208999992966651948478329359204508364200592041015625f,-0.32219999677240840352254735989845357835292816162109375f,-0.244500001102685882159448738093487918376922607421875f,-0.47040001994967484133525204015313647687435150146484375f,-0.81139999628663073139023254043422639369964599609375f,-1.7795000211074951668166477247723378241062164306640625f,0.6499999855458735709134998614899814128875732421875f,-0.54060000030845412055668930406682193279266357421875f,0.165000005215406364555974505492486059665679931640625f,1.08289999197274422471082289121113717555999755859375f,-0.927899991115927580409561414853669703006744384765625f,-0.17589997652769151503804323510848917067050933837890625f,0.1157999993681908534881586092524230480194091796875f,1.2541000276446343519864967674948275089263916015625f,0.987600024247169638869081609300337731838226318359375f,1.11739997754096975768334232270717620849609375f,-0.9987000032246111658196241478435695171356201171875f,0.0600000080466269736234607989899814128875732421875f,1.156499958992004195579283987171947956085205078125f,-0.290700025486945179409303818829357624053955078125f,-0.366299975168704605721359257586300373077392578125f,-1.2223000415086744396830908954143524169921875f,-0.5397000475108626460496452637016773223876953125f,-1.1408999869585034758756592054851353168487548828125f,0.15650001889467279880818750825710594654083251953125f,-0.05060003342628505151878925971686840057373046875f,-0.58750002775341325644831158570013940334320068359375f,0.58709999156147230081614907248876988887786865234375f,-1.940300063838064748011902338475920259952545166015625f,0.1953000363826760121810366399586200714111328125f,0.82940003381520543968008496449328958988189697265625f,-0.681600004249811153300697696977294981479644775390625f,1.239900034314393995060754605219699442386627197265625f,0.0918999776005744450912970933131873607635498046875f,-0.532300028246641243612202742951922118663787841796875f,0.42289997286200531334543484263122081756591796875f,-1.08000004164874585654843031079508364200592041015625f,-1.3828000306457273183013967354781925678253173828125f,1.08240005405843309205238256254233419895172119140625f,-1.5702000222682954699848778545856475830078125f,-1.48029998810291285593621068983338773250579833984375f,0.893699988174438519905606881366111338138580322265625f,0.8114000158071517088131940909079276025295257568359375f,-0.0202999850854277408274128902121447026729583740234375f,0.3004999947249888947453655418939888477325439453125f,-0.377799997863173420142857139580883085727691650390625f,0.181900000250339477503302987315692007541656494140625f,0.075699989724158867065995082157314755022525787353515625f,-0.47899998745322225346399136469699442386627197265625f,0.13160004206150699435085016375523991882801055908203125f,-0.84230001930594422532294629490934312343597412109375f,0.9752999827384944975250391507870517671108245849609375f,-0.78289997971653946251535671763122081756591796875f,0.7039000180050731803049757218104787170886993408203125f,-0.392000017940997924625889936578460037708282470703125f,0.08099999835342164278273457966861315071582794189453125f,0.16419999899268145071573599125258624553680419921875f,0.0371999896198509372169382913853041827678680419921875f,0.6751000211179256638160950387828052043914794921875f,0.1649999970570201757169570555561222136020660400390625f,-0.178500007592141629597648488925187848508358001708984375f,0.864100012743473033793861759477294981479644775390625f,-0.4447999993383884298481234509381465613842010498046875f,-0.1482999951213599398869291690061800181865692138671875f,-0.017100007468461964066364089376293122768402099609375f,-0.8606000215798623731444649820332415401935577392578125f,-0.34630001540184007779998864862136542797088623046875f,0.71989999389499370607836681301705539226531982421875f,-0.998000011712312673495262060896493494510650634765625f,0.705999989189207699524786221445538103580474853515625f,0.98680002167522928857579245232045650482177734375f,-1.13719998760819418492928889463655650615692138671875f,0.1703000056862833400117551718722097575664520263671875f,0.2326000072598457535377747262828052043914794921875f,-1.032500026077032284632650771527551114559173583984375f,0.7735000204443931437481296597979962825775146484375f,1.386600033977627877135319067747332155704498291015625f,0.96099998781085016474889926030300557613372802734375f,0.366400000309944129384120969916693866252899169921875f,1.8645999799013139863745891489088535308837890625f,0.8246999722599976934134247130714356899261474609375f,0.7256999933004379954581963829696178436279296875f,-1.93000000156462192535400390625f,1.65739996457695948350874459720216691493988037109375f,-0.3011000265121459573691709010745398700237274169921875f,-0.9517999752581118855232489295303821563720703125f,0.57270001392364466852313853451050817966461181640625f,-0.41849999943375593147720792330801486968994140625f,0.56929998036175977205175513518042862415313720703125f,-0.645599974492192085762098940904252231121063232421875f,-0.29880000426322228523901003427454270422458648681640625f,-1.020700035694241503136936444207094609737396240234375f,0.27000000558793579852334687529946677386760711669921875f,1.176000021934509476295716012828052043914794921875f,0.0676000042796136568767906283028423786163330078125f,1.585200011539459552523112506605684757232666015625f,0.8162000459909446448136804974637925624847412109375f,0.519199985730647739501364412717521190643310546875f,-0.8013000144332647067813013563863933086395263671875f,-0.5417999729484324067385614398517645895481109619140625f,-0.72989998476803308324889485447783954441547393798828125f,0.44180002312809241171720486818230710923671722412109375f,-0.1284999890029430158477907752967439591884613037109375f,-0.851800002117454990724354502162896096706390380859375f,0.7738000022888182893865405276301316916942596435546875f,-0.27909998595863572123931817259290255606174468994140625f,0.364499997451901425193909744848497211933135986328125f,-0.29199993576109395387874201333033852279186248779296875f,0.73799999353289624792751055792905390262603759765625f,0.0327000474885104086553155866567976772785186767578125f,-0.27199998173117645450247437111102044582366943359375f,-0.4029999817237259751578903888002969324588775634765625f,1.0576999710828065559553579078055918216705322265625f,-0.2774000054433940487541576658259145915508270263671875f,-0.218099995577335281637942898669280111789703369140625f,0.3731999937102197240079703988158144056797027587890625f,-0.21740000033974649085166674922220408916473388671875f,0.62130000251233596930688918291707523167133331298828125f,-0.700199969704449198815154886688105762004852294921875f,-0.78820001852065357805798839763156138360500335693359375f,-0.5413000200584530130498706057551316916942596435546875f,-0.4382999948978423265799619912286289036273956298828125f,0.6795000170469283062857357435859739780426025390625f,-0.27699998959153904198871032349416054785251617431640625f,0.8874000162839887906329749966971576213836669921875f,-0.04069998167753208218044846944394521415233612060546875f,-0.05660000275969512273377404198981821537017822265625f,-0.94440000676959734438042914916877634823322296142578125f,0.50840001471191609283550860709510743618011474609375f,-0.212200046840310019291564458399079740047454833984375f,-1.36820002682804986449127682135440409183502197265625f,1.06020005341172218749079547706060111522674560546875f,0.1983000069677827159608796137035824358463287353515625f,0.5315999850273129823108320124447345733642578125f,0.0100000202655792236328125f,-0.628699955093861007071609492413699626922607421875f,0.247499997355043799274199045612476766109466552734375f,-0.484099994266033117895631221472285687923431396484375f,-1.3886999977856875343462661476223729550838470458984375f,0.93479998011589060524784144945442676544189453125f,0.9069999940991397835432508145458996295928955078125f,-0.1848000021696094563594670034945011138916015625f,-0.17600004674494318379629476112313568592071533203125f,-1.77830000085830608469450453412719070911407470703125f,-1.620800032597780049314906136714853346347808837890625f,-0.97009994676113109335346962325274944305419921875f,1.8603000086665151258102923748083412647247314453125f,-0.552499998509883738506687222979962825775146484375f,1.14129998410940114439426906756125390529632568359375f,-0.5003999998331067899925983510911464691162109375f,-1.455399981988966029433640869683586061000823974609375f,0.111500014722346829643129240139387547969818115234375f,0.3677000011831521586458393358043394982814788818359375f,-1.919299983453750968465101323090493679046630859375f,-1.4083999848723411929540816345252096652984619140625f,0.251100025916098701372902723960578441619873046875f,-0.32370000024437928232146077789366245269775390625f,0.17980000529885231674853685035486705601215362548828125f,-0.474300013144314303037418767416966147720813751220703125f,0.2920000207722190932457806411548517644405364990234375f,0.67320002824366087512686362970271147787570953369140625f,-0.761199998232722219171364486101083457469940185546875f,0.4608999795079231631689253845252096652984619140625f,-0.4081000088766213451663134037517011165618896484375f,-0.07969998626708996170009413617663085460662841796875f,0.63499997425824428187723924565943889319896697998046875f,-0.2414999957606196456794123150757513940334320068359375f,0.984400017796456694441076251678168773651123046875f,0.81539997554421450587369690765626728534698486328125f,0.144799996544420661592766919056884944438934326171875f,-0.21960000568032267409535052138380706310272216796875f,0.1702999967828391358892758944421075284481048583984375f,-1.027299984809756150383464046171866357326507568359375f,0.2350000056624408362182521159411408007144927978515625f,0.05000001654028896069803522550500929355621337890625f,0.292700019772350827906848280690610408782958984375f,-0.495699995386600367197615923942066729068756103515625f,-0.5362999798253180738782930347952060401439666748046875f,-0.73979997981786738137088832445442676544189453125f,-0.5384999930262563960781108107767067849636077880859375f,0.3232999932214617455628058451111428439617156982421875f,0.8215999713554982264440695871599018573760986328125f,-1.1206999947905540437886884319595992565155029296875f,0.52589996836930563706147268021595664322376251220703125f,0.52879998843073838799000441213138401508331298828125f,1.580599931614101283372519901604391634464263916015625f,-0.7835000442862511960129268118180334568023681640625f,-1.0378999938726423124535358510911464691162109375f,-0.9257000463485720587186733609996736049652099609375f,-0.908599968099593979786732234060764312744140625f,-0.549500016450881645369008765555918216705322265625f,-1.48020002644062032004512730054557323455810546875f,0.374400015002489539028829312883317470550537109375f,0.3229999993443488648381389793939888477325439453125f,-0.64590001080036163472186672152020037174224853515625f,-0.423799990665912673648563213646411895751953125f,-1.188400039410591801924965693615376949310302734375f,0.802999975204468086076303734444081783294677734375f,1.11190000203847905169141085934825241565704345703125f,0.5355999660730361711102887056767940521240234375f,-1.682200063417852664571228160639293491840362548828125f,-0.345200007963180066639097276492975652217864990234375f,0.1705000100731848267088253123802132904529571533203125f,0.23339998323321342610370265902020037174224853515625f,0.2178000082194804232216256423271261155605316162109375f,1.7326999865055086047505028545856475830078125f,-0.3546000032961362791184001252986490726470947265625f,0.067700002151728000399089069105684757232666015625f,-1.550800011438131065233392291702330112457275390625f,0.063100078129767922519022249616682529449462890625f,2.3756000826001173464874227647669613361358642578125f,0.3558999744415274335551657713949680328369140625f,-0.7428000491976742836186531349085271358489990234375f,0.3895000009536744300930877216160297393798828125f,0.592900074934959775418974459171295166015625f,1.636899996599554985010627206065692007541656494140625f,-0.71029999048709857589756211382336914539337158203125f,1.8194999840036032434653634481946937739849090576171875f,-0.7039999673366548194053393672220408916473388671875f,0.387699958789348375631789167528040707111358642578125f,-0.84950001034140587563570079510100185871124267578125f,0.135800002199411462555644902749918401241302490234375f,-0.0043999957799911726397112943232059478759765625f,0.64650002221018088022219671984203159809112548828125f,-0.3272999953150748098096300964243710041046142578125f,1.2202999945476646903586015469045378267765045166015625f,0.488199989277124490882897589472122490406036376953125f,0.774300038848817262504553582402877509593963623046875f,-0.32410000678300843190982050145976245403289794921875f,-0.653399979023635779640244436450302600860595703125f,0.0211999952152372994618190205073915421962738037109375f,0.248000006422399998928796094332938082516193389892578125f,0.20749998681247239229463730225688777863979339599609375f,0.11989999501258108427492743430775590240955352783203125f,0.87450000154972062915703645558096468448638916015625f,0.4138000200584530663405757877626456320285797119140625f,-0.05559999080896371737026129267178475856781005859375f,-0.083300021794438183775355355464853346347808837890625f,0.5624000283539292155410294071771204471588134765625f,0.78979999058395711220015300568775273859500885009765625f,-0.42360001414418224641167398658581078052520751953125f,-1.7855000203177329165526998622226528823375701904296875f,0.34399999982118600883040926419198513031005859375f,-0.238699993613362249078591048601083457469940185546875f,-0.06429997373074247679625159435090608894824981689453125f,0.687900003707408558284441824071109294891357421875f,0.017800008666515765298754558898508548736572265625f,-0.32449998873472196692091529257595539093017578125f,0.372500024288893172297321143560111522674560546875f,0.3625000211596489663179454510100185871124267578125f,-0.757600009047985079035925082280300557613372802734375f,1.271700038480759076264803297817707061767578125f,0.04980000269114981392704066820442676544189453125f,-0.2068000039428472547342607867904007434844970703125f,0.75000002868473547579242222127504646778106689453125f,1.067999995023011905459497938863933086395263671875f,-1.1487000017344950464348585228435695171356201171875f,-1.8713999728918082610107376240193843841552734375f,0.70630000634938472359181105275638401508331298828125f,-0.809500015482306434222437019343487918376922607421875f,0.99220001972019689073789550093351863324642181396484375f,0.571399969017505782176158390939235687255859375f,-0.395699984359740852823961176909506320953369140625f,0.91100001835823007922954275272786617279052734375f,0.2024000123351807900462517864070832729339599609375f,-0.0320000068396328885000912123359739780426025390625f,-0.58919999087154817374312187894247472286224365234375f,-1.2496999999761584376756218262016773223876953125f,-1.4153000375747684103089341078884899616241455078125f,-0.69249997258186368753740680404007434844970703125f,0.77709999342411695266719107166863977909088134765625f,-0.383199997323751428979221600457094609737396240234375f,-0.30279999495744702731059305733651854097843170166015625f,-1.81810002970099393593272907310165464878082275390625f,-0.246699998620152516348724702766048721969127655029296875f,-0.154399999095499396162267657928168773651123046875f,0.65889997214674977943360545395989902317523956298828125f,0.81639999800026397114294240964227356016635894775390625f,0.58869997766911996706085119512863457202911376953125f,1.47350000345706888538188650272786617279052734375f,-0.0404999904409051314502221430302597582340240478515625f,-0.14020000334382043405412332504056394100189208984375f,-0.60049999658763397736294109563459642231464385986328125f,-0.980500023446977397867385661811567842960357666015625f,0.50599999824166308481920850681490264832973480224609375f,-0.6896000021785493316883730585686862468719482421875f,-0.3395000109001993227053617374622263014316558837890625f,-0.16389998429119645440010799575247801840305328369140625f,-0.2019999947696924369466842108522541821002960205078125f,-0.3187000102281569713369435703498311340808868408203125f,-0.042599997611343809467143728397786617279052734375f,-0.44750000245869170356627364526502788066864013671875f,-0.09309999162852766740883225793368183076381683349609375f,-0.64920000405907618556966554024256765842437744140625f,0.435499985337257589090853571178740821778774261474609375f,1.77399997478723481236784209613688290119171142578125f,-0.4095999921575188551514656865037977695465087890625f,-0.16340001527070935338770141243003308773040771484375f,-0.223199993300438048748901564977131783962249755859375f,0.10649999982118618646609320421703159809112548828125f,-0.61810001614093745825329051513108424842357635498046875f,-0.414000017702580436207426828332245349884033203125f,-0.38189999630153181886527136157383210957050323486328125f,-0.12740000615119928539797911071218550205230712890625f,-0.58749999683350317969399156936560757458209991455078125f,-0.3037999961793425018896641631727106869220733642578125f,0.17629996727108954157614562063827179372310638427734375f,-0.46960004636049301751654638792388141155242919921875f,0.149400020925700705021199610200710594654083251953125f,0.2726000038325786523074611977790482342243194580078125f,0.4266999620378015833921381272375583648681640625f,0.578799973082542607016875990666449069976806640625f,-0.3377999763309957881318723593722097575664520263671875f,-0.00119999413490301520823777536861598491668701171875f,0.183900014615058893241439363919198513031005859375f,0.17640000239610653665067729889415204524993896484375f,-0.02370000661462545732405260423547588288784027099609375f,0.0399999953806400299072265625f,-1.01360000132918326443132173153571784496307373046875f,-0.238199985253810808671914855949580669403076171875f,0.65979999933838850978418122394941747188568115234375f,0.0778000186502934010235321693471632897853851318359375f,0.78570000186860566859792243121773935854434967041015625f,0.39800000143051139644967406638897955417633056640625f,-0.8387000251293184049927731393836438655853271484375f,-0.397400012707710192927379466709680855274200439453125f,-0.1497999862998724562856978081981651484966278076171875f,-0.05879997665882097379608239862136542797088623046875f,1.420499997183680473966660429141484200954437255859375f,0.39219998432993907755417239968664944171905517578125f,-0.63079996293485141034551588745671324431896209716796875f,-0.86939999697208414364268946883385069668292999267578125f,-0.55579998644143346897550372887053526937961578369140625f,-0.7023000063419340133208379484131000936031341552734375f,-0.58889999546706661703865393064916133880615234375f,0.3832999682247633099763106656610034406185150146484375f,-0.701700008641183092805704291095025837421417236328125f,-0.80720003134310314152344290050677955150604248046875f,0.4501000030130146711826455430127680301666259765625f,-0.2132999571979043995639813147136010229587554931640625f,-0.541799995076656326631336924037896096706390380859375f,-1.456800030243396548712553340010344982147216796875f,0.57799997766315958092064875017968006432056427001953125f,0.705400004154443482917713481583632528781890869140625f,-0.1074000008612869816460033689509145915508270263671875f,-0.3450000263005497114221498122788034379482269287109375f,0.058199999260902413045215553211164660751819610595703125f,-0.461299978372454955177772717433981597423553466796875f,-0.5479999774768951059655819335603155195713043212890625f,-0.295300004419683848055910857510752975940704345703125f,0.56629999550878995506764113088138401508331298828125f,0.6521000351846220066676096394076012074947357177734375f,0.02600000219047071681188754155300557613372802734375f,-0.343300004732608687874062525224871933460235595703125f,-0.686700000333785975925593447755090892314910888671875f,-0.753900022810697567621218695421703159809112548828125f,0.09690000185221425699211295068380422890186309814453125f,0.6763999971061946947514798011980019509792327880859375f,0.176100002148747381003346390571095980703830718994140625f,-0.070199996712803969245442203828133642673492431640625f,0.64639997467994680935277074240730144083499908447265625f,0.03299996179342290503200274542905390262603759765625f,-0.1555999922990796147104219926404766738414764404296875f,0.532199996995925683762607150129042565822601318359375f,-0.33970002299845225568475370891974307596683502197265625f,-0.238800010633468673404422588646411895751953125f,1.33360002003014077587295105331577360630035400390625f,0.72340000245571101089581134147010743618011474609375f,-0.275700001011788931037926886347122490406036376953125f,0.504800026607513441234686979441903531551361083984375f,0.81980001345723874095483552082441747188568115234375f,0.292600004875659625014350240235216915607452392578125f,-0.9545999984160065121585603264975361526012420654296875f,-0.3355999870836736675983047462068498134613037109375f,0.29739997776448701127804952193400822579860687255859375f,-0.33060002090930939999680049368180334568023681640625f,-0.86220000105649230448534581228159368038177490234375f,-0.2681999946415423163870173084433190524578094482421875f,0.320200002337992339906946881455951370298862457275390625f,0.216400015956163205288476092391647398471832275390625f,0.0080999797821042773815491955247125588357448577880859375f,-0.03619998877048491436880794935859739780426025390625f,-0.5895999955847859297364266240037977695465087890625f,-0.867000010788441244358182302676141262054443359375f,0.0200000006705522981320655162562616169452667236328125f,-0.26659998711347565603091425145976245403289794921875f,0.7443000129580499280024241670616902410984039306640625f,0.04879998407214858335834151148446835577487945556640625f,-0.5110000112801789118321948990342207252979278564453125f,-0.1606000308930879416635661982581950724124908447265625f,-0.41619999335259205697212792074424214661121368408203125f,-1.6002999583005912409561233289423398673534393310546875f,0.302300000008940639162346997181884944438934326171875f,0.0925000180304049735013904864899814128875732421875f,0.206199985343217928601688981871120631694793701171875f,0.39610001995563504095798634807579219341278076171875f,0.75049997602403195617881692669470794498920440673828125f,-0.6243000001430514256384185500792227685451507568359375f,0.6325999892294407178638948607840575277805328369140625f,0.649700003999471586979552739649079740047454833984375f,0.8743999777123334649076014102320186793804168701171875f,0.7938000095158812552398330808500759303569793701171875f,-0.145000000968575404325378030989668332040309906005859375f,0.860299972233176557523393057635985314846038818359375f,-0.61670000689029669249663356822566129267215728759765625f,-0.854600029447675435534392818226478993892669677734375f,-0.2450999936997891592849185826707980595529079437255859375f,-0.391699969711899687041523066000081598758697509765625f,-0.090999998465180365059268297045491635799407958984375f,-0.436700001749395827488342547439970076084136962890625f,-0.4918000030860303350976892033941112458705902099609375f,0.2434999599456784125095509807579219341278076171875f,-0.627600015194713645172441829345189034938812255859375f,0.322499933987855147421441870392300188541412353515625f,0.2923000061929224013823613859131000936031341552734375f,1.2497000175781549646814028164953924715518951416015625f,0.050900004523247499577109920210205018520355224609375f,-0.71540000339075948687650452484376728534698486328125f,0.85260000927150247918007153202779591083526611328125f,0.3490000059306621604804377057007513940334320068359375f,0.672000013209879565323490169248543679714202880859375f,0.48059998902082412541858502663671970367431640625f,0.0622999805629251834915294239181093871593475341796875f,0.319799989056587463664982351474463939666748046875f,0.1715000152066352701041296313633210957050323486328125f,0.35279998393058775718600372783839702606201171875f,-1.226100028747320269673082293593324720859527587890625f,-0.1173999867796897689231627737171947956085205078125f,0.0243000347137449779211237910203635692596435546875f,-0.641999980539083470176819901098497211933135986328125f,-0.1207000131189822911270681515816249884665012359619140625f,-0.216599999928474229449193444452248513698577880859375f,0.91340001556873318122597993351519107818603515625f,0.263799998414516068123703007586300373077392578125f,0.8555999812722203667902931556454859673976898193359375f,0.9911999953269958751889134873636066913604736328125f,-0.8029999834373591216518661894951947033405303955078125f,-0.08879999997913812226357777035445906221866607666015625f,0.778600031131505776471612989553250372409820556640625f,0.14340001229047771857949555851519107818603515625f,-0.7361999968916175252076072865747846662998199462890625f,-0.396200020062923652375275196391157805919647216796875f,-0.079600022965669536834099062616587616503238677978515625f,0.4117999993979928330389839175040833652019500732421875f,0.01759997861236339833368447216344065964221954345703125f,0.19039999715089805931711453013122081756591796875f,-0.265300027777254532335149406208074651658535003662109375f,0.69140002716928716797184506503981538116931915283203125f,0.67509998960196970774205738052842207252979278564453125f,0.397800047335028494899233919568359851837158203125f,-0.4269999889209865617800687687122263014316558837890625f,-1.43850003176927554449093804578296840190887451171875f,-0.1096999937921763201842395574203692376613616943359375f,-0.8322999769866470387569279409945011138916015625f,-0.499099974112212851906633659382350742816925048828125f,-1.8487999937951560269766559940762817859649658203125f,0.1868999849021435455398432168294675648212432861328125f,1.3914999738186601963008115490083582699298858642578125f,-0.0710000001043080697371578935417346656322479248046875f,0.6600999839395285118115452860365621745586395263671875f,-0.6952999814718963111914717956096865236759185791015625f,-0.436000027894973385400589904747903347015380859375f,0.9084999695569280220297514460980892181396484375f,0.17670001538395840867678998620249330997467041015625f,0.637699980731308624370967663708142936229705810546875f,-0.4376999824449427922701261195470578968524932861328125f,-0.5310999680250887422516825608909130096435546875f,-0.177600030095873773205994439194910228252410888671875f,0.80029998445212857660635563661344349384307861328125f,1.2863999891340724968102904313127510249614715576171875f,-0.084299997386336311677723642787896096706390380859375f,-0.52679996743500268063797875583986751735210418701171875f,0.2363999888733030463328077530604787170886993408203125f,-0.47979996379911948878316252375952899456024169921875f,-0.5424999861046673199638235018937848508358001708984375f,-1.3704999736957252931546236141002736985683441162109375f,-1.360400027959048685488596674986183643341064453125f,-0.6438000249385833484438990126363933086395263671875f,0.173200017775595238589403379592113196849822998046875f,0.291800031137466220343412715010344982147216796875f,-0.039500024087727236832279231748543679714202880859375f,0.11780002803802513966502374387346208095550537109375f,0.9630000054538248033253466928726993501186370849609375f,0.2370000122785569374173064716160297393798828125f,-0.77370000803023575297601155398297123610973358154296875f,-0.50720000407397713981794140636338852345943450927734375f,1.2596000169306993399231942021287977695465087890625f,0.377199997293949262910928155179135501384735107421875f,-1.378899973413348245543375014676712453365325927734375f,-0.56799997889250530835880681479466147720813751220703125f,1.737199990960955897634221400949172675609588623046875f,0.940799993407726020677728229202330112457275390625f,-1.16379999759495245115203942987136542797088623046875f,-0.303199991400539936581282063343678601086139678955078125f,0.34820000546351070891404333451646380126476287841796875f,-0.3272000065326690076972226961515843868255615234375f,-1.806000027261674301115590424160473048686981201171875f,-0.9880000175237650950066381483338773250579833984375f,0.56400000060349686492600085330195724964141845703125f,0.066000003494322057040477602640748955309391021728515625f,0.77749999934807390022939443952054716646671295166015625f,0.6500000192224979489452607595012523233890533447265625f,0.2975999971270562038938578552915714681148529052734375f,0.31539999446868893784312604111619293689727783203125f,-0.8836000198438764829944602752220816910266876220703125f,-0.1356000131607064673744389438070356845855712890625f,-0.5798999890893699049598808414884842932224273681640625f,-0.6893999817728992951515465392731130123138427734375f,0.608299989309907029877422246499918401241302490234375f,0.183000012755394347863102666451595723628997802734375f,0.0812999963283538296110464216326363384723663330078125f,0.247800000321865443453361876890994608402252197265625f,0.35019999019354586256014272294123657047748565673828125f,0.439000006712972901112834733794443309307098388671875f,-0.84119997759461428987748377039679326117038726806640625f,-0.36560001040399037197659026787732727825641632080078125f,0.473700003038346639794298198467004112899303436279296875f,0.266500001367182159928148621474974788725376129150390625f,-0.06120000085905186182344550616107881069183349609375f,0.188400014730542242347155479365028440952301025390625f,0.400400004228949502049772490863688290119171142578125f,-0.6669999836124478331811360476422123610973358154296875f,-0.462499983683228588215996524013462476432323455810546875f,0.494899956344068581159945097169838845729827880859375f,-0.0482999970212578677575265828636474907398223876953125f,0.47069998853206573841134741087444126605987548828125f,-0.86829997146576687327979016117751598358154296875f,-0.0966999731950453078610507873236201703548431396484375f,0.54729996797144442322036184123135171830654144287109375f,0.17670000685304387832985639761318452656269073486328125f,0.60819997951686399773052471573464572429656982421875f,-0.101499972030520968502287360024638473987579345703125f,-0.606499986410141023185360609204508364200592041015625f,-1.160500021696090566791781384381465613842010498046875f,0.67270003206580886168097777044749818742275238037109375f,0.71260003437995944608474019332788884639739990234375f,-0.111400019495189184226546785794198513031005859375f,0.4144000166416166308636093162931501865386962890625f,1.08160001639425740194155878270976245403289794921875f,0.5868000324785713761599481586017645895481109619140625f,-0.66899999997019765629602261469699442386627197265625f,-0.2507999841690065068178228102624416351318359375f,-0.66590002242326751247247784704086370766162872314453125f,-0.9128000140309335819210900808684527873992919921875f,1.071099994735419613789417780935764312744140625f,0.22039999066889304213390232689562253654003143310546875f,-0.6181999779522417970412107024458236992359161376953125f,-0.012399976944923896127193074789829552173614501953125f,0.2333999699339273092846269719302654266357421875f,0.03259999206066144239457571529783308506011962890625f,-0.001699991877377016180616919882595539093017578125f,-0.473400016462802941674681278527714312076568603515625f,-1.4819000107929110043869513901881873607635498046875f,-0.7864000038862233310510418959893286228179931640625f,1.0341999868482349977938383744913153350353240966796875f,0.03039998850822467346688426914624869823455810546875f,-0.15820000909566889202295669747400097548961639404296875f,0.384600001955031967071363396826200187206268310546875f,-0.6268999904900791619866140536032617092132568359375f,-0.5379999841451645892220767564140260219573974609375f,1.56510002044737372983718159957788884639739990234375f,0.68239999333620016130907970364205539226531982421875f,0.5998999744117259336206871012109331786632537841796875f,-0.86160000402629355331640681470162235200405120849609375f,0.8742000134840604008701347993337549269199371337890625f,-0.2513999380230895752674769028089940547943115234375f,0.5057999767929317602721539515187032520771026611328125f,0.14359999186694627493210418833768926560878753662109375f,0.8638000028103587357719561623525805771350860595703125f,0.5412000365287070735575980506837368011474609375f,0.3425999804377557467205406283028423786163330078125f,1.58979999755024881125109459389932453632354736328125f,-0.774400011947750765983755627530626952648162841796875f,0.1451999643400305739504574376042000949382781982421875f,0.09909999959319824858194891703533357940614223480224609375f,-0.51869998489618296400038843785296194255352020263671875f,0.673199972736835849929093456012196838855743408203125f,0.94790002382397542390890521346591413021087646484375f,-0.7129999723359945473788457093178294599056243896484375f,-0.2505000073909753321999005493125878274440765380859375f,-0.631799988408386692384510752162896096706390380859375f,-0.35000001907348643470641036401502788066864013671875f,-0.55630001372545923032930659246630966663360595703125f,0.684499944329261200692826605518348515033721923828125f,0.29699999715387814802625143784098327159881591796875f,-0.3438999614179134578506591424229554831981658935546875f,0.665999962814152635104392174980603158473968505859375f,0.98939997417330705076210506376810371875762939453125f,0.307399979031086179048770645749755203723907470703125f,0.981200026917456558095409491215832531452178955078125f,0.003100009547174042978667785064317286014556884765625f,-1.31900000607967360366501452517695724964141845703125f,0.2621999838456512410544974045478738844394683837890625f,-0.2696000094056127949215806438587605953216552734375f,-0.5417000394865869594696050626225769519805908203125f,0.1058000053286551889186739572323858737945556640625f,0.8390000170320270900248260659282095730304718017578125f,-0.286600013935565556266737985424697399139404296875f,-0.25049998168647269469744287562207318842411041259765625f,-0.008799998712539558454182042623870074748992919921875f,-0.58019999794214971444716866244561970233917236328125f,-0.02999997615814198326233963598497211933135986328125f,1.45179997894614931652768063941039144992828369140625f,0.652199985074997101719418424181640148162841796875f,0.185500001318752882895068978541530668735504150390625f,0.704999999999999626965063725947402417659759521484375f,0.976099994176626406527930157608352601528167724609375f,0.8171999975919721492800817941315472126007080078125f,0.5647999947190280867204137393855489790439605712890625f,0.334799989950656851078747422434389591217041015625f,-1.469699969689548080253871376044116914272308349609375f,-0.6051999888896943957661278545856475830078125f,-0.06420001367032501793374876797315664589405059814453125f,-0.4104000169694426158883970856550149619579315185546875f,-0.46380001040995122441046305539202876389026641845703125f,-0.01599998170137428843418092583306133747100830078125f,0.76560001081377293541407880184124223887920379638671875f,-0.185200013476610525486876213108189404010772705078125f,-1.7060999998390669052383827875019051134586334228515625f,-1.0258000068187715925205338862724602222442626953125f,0.74259995387643584763281978666782379150390625f,0.9074999722838399396351860559661872684955596923828125f,-0.719399972534179976690893454360775649547576904296875f,-1.478899981385469519779007896431721746921539306640625f,0.6297999774709344134038246920681558549404144287109375f,1.936300012272596404727664776146411895751953125f,-0.31809998231530212109419153421185910701751708984375f,-0.2507000023901462260056405284558422863483428955078125f,0.559900004102289461371810830314643681049346923828125f,0.360500035852194589125474522006697952747344970703125f,0.140799998474121057512320476234890520572662353515625f,0.228900016403198236503158113919198513031005859375f,0.172399995310604570164514370844699442386627197265625f,-1.3668999519705777867528695423970930278301239013671875f,0.5656999797403814422835921504884026944637298583984375f,-0.44049995985627266037454319302923977375030517578125f,0.530799997207522356035269694984890520572662353515625f,0.056800001037121461422430002130568027496337890625f,0.654699970881641224451641392079181969165802001953125f,0.57289998016357390753228173707611858844757080078125f,-0.3912000003188846886104101940873079001903533935546875f,0.61799996827542702515501105153816752135753631591796875f,-0.2479999970346689064371048516477458178997039794921875f,-0.41019998792111866947607268230058252811431884765625f,0.18029998679906145131468520048656500875949859619140625f,1.1962999879091971155986584562924690544605255126953125f,0.079900001047551683086567209102213382720947265625f,0.69419999906718654614223851240240037441253662109375f,-0.74059997333735250180097864358685910701751708984375f,-0.03579996598958956610658788122236728668212890625f,-0.363900011932849931639566420926712453365325927734375f,0.2744000226020819610539547284133732318878173828125f,1.59670002238750452505655630375258624553680419921875f,0.642799994063377067021747279795818030834197998046875f,-0.134800006230175295485196329536847770214080810546875f,0.569600004339218202176198246888816356658935546875f,0.667800011646747737614759898860938847064971923828125f,0.152599985206126831371875596232712268829345703125f,-0.38419999973773943935384522774256765842437744140625f,-0.2966000353932383148958251695148646831512451171875f,-1.1826000102400779923073059762828052043914794921875f,-0.3970000118315220039022506171022541821002960205078125f,-1.34130001007467480889090438722632825374603271484375f,-1.02340001497268673347207368351519107818603515625f,-0.09489999119415870598714946027030237019062042236328125f,-0.7330000110790131362392685332451947033405303955078125f,-0.3441000037096439978512307789060287177562713623046875f,0.31720003335475865924308891408145427703857421875f,0.5214999819397927449671215072157792747020721435546875f,0.10980001915693282654729046043939888477325439453125f,0.4597999966561794327191137199406512081623077392578125f,-0.1890000150203707818263865192420780658721923828125f,-0.94430001388937245820898169768042862415313720703125f,-0.41759999742507947217973196529783308506011962890625f,0.3443999755889175506951005445444025099277496337890625f,0.131600006037950745252373963012360036373138427734375f,-0.47749998889863476403405684322933666408061981201171875f,-0.53340000245571150827572637354023754596710205078125f,1.655799984802305946374190170899964869022369384765625f,0.03290000728368713822646896005608141422271728515625f,0.579800011482834332099400853621773421764373779296875f,0.7908000625491151680535040213726460933685302734375f,0.7073999743372205717406586700235493481159210205078125f,-0.583999978810549880137159561854787170886993408203125f,0.3209000087514517485232090621138922870159149169921875f,0.3532000207185745921378838829696178436279296875f,0.7467999833419920463484231731854379177093505859375f,1.33950001123547490777809798601083457469940185546875f,-0.4592999923199414524788153357803821563720703125f,-0.56940001552402985662837409108760766685009002685546875f,-0.667599989639222624049352816655300557613372802734375f,0.020699979069828344790948904119431972503662109375f,-0.853399977980554336198792952927760779857635498046875f,0.8298999817691747882264508007210679352283477783203125f,-0.838299967684597024941695053712464869022369384765625f,-1.44969998021349244510247444850392639636993408203125f,0.7491999661356214534890796130639500916004180908203125f,0.835100001746415188108585425652563571929931640625f,0.52809999438524235504388570916489697992801666259765625f,1.264400034970045272331162777845747768878936767578125f,-0.52640000288039423548269724051351658999919891357421875f,-0.70900002679228801838462459272705018520355224609375f,-0.71829999977797254206990373859298415482044219970703125f,-0.7629000255376101602422522773849777877330780029296875f,-0.1350000023096800116917393097537569701671600341796875f,0.08910000745356061457869145669974386692047119140625f,-0.71579999061375854108035809986176900565624237060546875f,-0.76269999186992709638843734865076839923858642578125f,0.5275000216066840774686852455488406121730804443359375f,-0.13400000339746487298953070421703159809112548828125f,-0.7103000206992031717589952677371911704540252685546875f,0.03620001291036611945628465036861598491668701171875f,0.88009999790936699337606796689215116202831268310546875f,0.4404000076562166743343595953774638473987579345703125f,0.021200005310773650801792200581985525786876678466796875f,-0.0509999867677688456524265347979962825775146484375f,0.1673000137552620447678464188356883823871612548828125f,-0.17740001240968705786826831172220408916473388671875f,0.0106999962806704207451957699959166347980499267578125f,-0.0696000210285188103398468228988349437713623046875f,-0.92740000324547333665492487853043712675571441650390625f,-0.635400015628337921924639886128716170787811279296875f,-1.152100014509260805795065607526339590549468994140625f,-0.6248000024676318986394107923842966556549072265625f,-0.2578999918237327992898144657374359667301177978515625f,-0.2295999876499177361210968228988349437713623046875f,0.6676999917209147650254408290493302047252655029296875f,-0.1578000171601774415108820903697051107883453369140625f,0.42030001417994522494581133287283591926097869873046875f,0.5500000029802318834981633699499070644378662109375f,0.0546999724090098116136005046428181231021881103515625f,0.3581999885320659160470313508994877338409423828125f,-0.7250000163540242237303345973487012088298797607421875f,-0.0619999949932097393912044935859739780426025390625f,0.1414999896138906532172541119507513940334320068359375f,-0.36280001254081728490064051584340631961822509765625f,-1.152700029346347410097450847388245165348052978515625f,0.348699972081183862115949523285962641239166259765625f,0.85539999364912500112723137135617434978485107421875f,0.659700054365397026145956260734237730503082275390625f,-0.26990000250041479290530332946218550205230712890625f,-0.60509999716430906691044810941093601286411285400390625f,-0.025299998682737356858751809340901672840118408203125f,-0.304600027026235675720045037451200187206268310546875f,0.82329997381269948153459381501306779682636260986328125f,1.700400010710954035175745957531034946441650390625f,-0.24769998777210722185060376432375051081180572509765625f,0.07229999859332991718474659137427806854248046875f,0.35119999949932090288484687334857881069183349609375f,1.0109000005185613257907562001491896808147430419921875f,-0.73130000981390441694429682684130966663360595703125f,-0.3653000626087195001900909119285643100738525390625f,-0.5665999900937082056628923965035937726497650146484375f,-0.260199979352950805377986398525536060333251953125f,-0.5779999957680701516693488883902318775653839111328125f,0.08409997467100627233094201073981821537017822265625f,-0.72969999878406499593808121062465943396091461181640625f,-1.5043000137031083962568800416192971169948577880859375f,-0.349799986374378146791741528431884944438934326171875f,-1.58950000348687137119441104005090892314910888671875f,-0.777800006990134296103178712655790150165557861328125f,-2.0140000394582759923878256813623011112213134765625f,-0.5751999780863525923990664523444138467311859130859375f,-0.111600002461671454767611066927202045917510986328125f,-0.3074999994412064285853602996212430298328399658203125f,0.2921999724090103001117313397116959095001220703125f,-0.7229000047691165331542606509174220263957977294921875f,-0.33239999512433993089643990970216691493988037109375f,0.0363999829873444813443938983255065977573394775390625f,0.004999991357326205587696676957421004772186279296875f,0.677299980451166572237298169056884944438934326171875f,0.01499997317791024187272341805510222911834716796875f,-0.47009997436553252470048391842283308506011962890625f,0.319200034606456739538771216757595539093017578125f,-0.1737000151082874754848717202548868954181671142578125f,-0.269999992549419420839740269002504646778106689453125f,1.629799988386035014542585486196912825107574462890625f,0.9301999903798101598795255995355546474456787109375f,0.2815999944895504203401515042060054838657379150390625f,0.32279998154640221486033624387346208095550537109375f,-0.57809999937713174578135522096999920904636383056640625f,0.3662000212550164945923825143836438655853271484375f,-0.3710000046491621628064194737817160785198211669921875f,-0.78580003244876905910132336430251598358154296875f,-0.5052000193998218424695778594468720257282257080078125f,-0.2500000053644182429479769780300557613372802734375f,0.5951999993205070449420190925593487918376922607421875f,0.08559999200105661287807379267178475856781005859375f,0.0796999991193416690293815918266773223876953125f,-0.101400022363662856150767765939235687255859375f,-0.902000006057322334385162321268580853939056396484375f,0.1072000336527825226085042231716215610504150390625f,0.85820001576393856179691965735401026904582977294921875f,0.1938000252366069986464935936965048313140869140625f,0.8829999999403952415377716533839702606201171875f,-0.229100007788837045641372469617635942995548248291015625f,-0.44279998452663438257559391786344349384307861328125f,-0.4770999647021294531867852128925733268260955810546875f,-0.808599991233646786525923744193278253078460693359375f,-0.671500029921531904619769193232059478759765625f,-0.72099997346848276880137973421369679272174835205078125f,-1.94659999888539214651927977683953940868377685546875f,0.566999987691640949805105265113525092601776123046875f,0.77599998632073408089127042330801486968994140625f,0.317399979031086132419403611493180505931377410888671875f,0.378100031638144873369355991599150002002716064453125f,0.5655999793723227586639268338331021368503570556640625f,-0.226799995523691677590250037610530853271484375f,0.26149999643117183278917536881635896861553192138671875f,-1.0207999676287169421584621886722743511199951171875f,0.226499986931681718260733759962022304534912109375f,0.97560000079274100670545522007159888744354248046875f,-0.7128000090017911904993752614245750010013580322265625f,-0.39980001021623667867288531851954758167266845703125f,-0.759200000259280027847808014485053718090057373046875f,-0.620200025248527797572251074598170816898345947265625f,-0.6376999872133135571772299954318441450595855712890625f,-0.6390000287294395064918717253021895885467529296875f,0.18750000476837147544983963598497211933135986328125f,-0.58149997240304873713512279209680855274200439453125f,0.3244999894425273367914996924810111522674560546875f,0.61519999082684506674922886304557323455810546875f,0.6147000142067666761391819818527437746524810791015625f,1.0675000147521505056147361756302416324615478515625f,0.5746000028118489755257769502350129187107086181640625f,0.993200017142296331940087839029729366302490234375f,-0.37609998974353064937758972519077360630035400390625f,-0.03780000665485883093452912362408824265003204345703125f,-0.4940999552249907278422824674635194242000579833984375f,-0.392599964642524668079204275272786617279052734375f,-0.3055000366717577353625756586552597582340240478515625f,-0.242399998067319211170200787819339893758296966552734375f,-0.19389999839141969228961670523858629167079925537109375f,-0.35619999085664755256175340036861598491668701171875f,-0.0469000192493205236843323291395790874958038330078125f,-0.49880000139474844900178140960633754730224609375f,1.7778000388786201479973669847822748124599456787109375f,1.0224000028729438582786315237171947956085205078125f,-0.2529999843314290064455462925252504646778106689453125f,-0.0291999946713445979185053147375583648681640625f,-1.0502999902263281395420335684320889413356781005859375f,-0.4473999960184098512172568007372319698333740234375f,0.21899999218434096359686691357637755572795867919921875f,0.03420000927448274552489237976260483264923095703125f,-0.2764000190854074556767727699480019509792327880859375f,-0.195999995708465490906746708787977695465087890625f,-1.0337000215530398605778827914036810398101806640625f,-0.97620001559257474355035810731351375579833984375f,0.2823000030264257720347131908056326210498809814453125f,-0.06820000104904178073184084496460855007171630859375f,0.9778000049039718977184065806795842945575714111328125f,0.6442000246226788906511728782788850367069244384765625f,-0.631099987843632970196949827368371188640594482421875f,-1.8063000092923637840414130550925619900226593017578125f,0.8634000032007691505242519269813783466815948486328125f,0.1961000421956182737659446502220816910266876220703125f,0.4025999828219415377361656283028423786163330078125f,0.942299972181022127415417344309389591217041015625f,0.6666000073418019145066182318259961903095245361328125f,1.4044000209815805657598275502095930278301239013671875f,-0.055500003442168287237645785126005648635327816009521484375f,0.5210000012591480267332144649117253720760345458984375f,0.65529997789561755805465281810029409825801849365234375f,2.04209999212026627191107763792388141155242919921875f,0.584899964912235947878116348874755203723907470703125f,0.99010004734396783732108815456740558147430419921875f,0.87800001521408521654166179359890520572662353515625f,0.372200023967028670313084148801863193511962890625f,0.53800000396370872834950205287896096706390380859375f,0.771700019705295670036093724775128066539764404296875f,-0.28789998880624778632153493163059465587139129638671875f,0.06359999209046396373423704062588512897491455078125f,0.16179999950975176847833836291101761162281036376953125f,0.2971999940156937469737385981716215610504150390625f,0.6222999766513706010329087803256697952747344970703125f,1.78740001285672178710228763520717620849609375f,-0.902399999482929704441858120844699442386627197265625f,-0.2420999961435796876685344614088535308837890625f,-0.42540000173300496211226118248305283486843109130859375f,-0.94050001670420135813088791110203601419925689697265625f,0.770999983154237344962211864185519516468048095703125f,1.030399987688661145313062661443836987018585205078125f,0.5361000039741397049652960049570538103580474853515625f,-0.116600019299984314358198389527387917041778564453125f,-0.1804000004664063627757286667474545538425445556640625f,-0.2406000164389610063153668306767940521240234375f,-0.191100007736682897530045011080801486968994140625f,0.223399994111061150903196903527714312076568603515625f,-0.26560000265538741270887612699880264699459075927734375f,-0.20779999465942378122917943983338773250579833984375f,0.9026000233158468777361349566490389406681060791015625f,0.7156000247836118433042429387569427490234375f,0.4921999980017541798105185080203227698802947998046875f,0.594800005149841215512651615426875650882720947265625f,0.5225000363215805965211302464012987911701202392578125f,0.2900000119209291682409457280300557613372802734375f,0.08909999076426033948195026823668740689754486083984375f,0.1853999863475561671322111578774638473987579345703125f,-0.623000015027821429924870244576595723628997802734375f,0.099999997913837290752780972979962825775146484375f,-0.7154000066503882759860744044999592006206512451171875f,-0.43579999139606939717594968897174112498760223388671875f,1.887100017862021861247967535746283829212188720703125f,0.591399997031688773319046958931721746921539306640625f,-0.4979999941661954476757045995327644050121307373046875f,0.0147999950170518701497712754644453525543212890625f,-1.0864000273182992106768551821005530655384063720703125f,-0.112199980753659556143020381568931043148040771484375f,1.02480002005100256923242341144941747188568115234375f,0.3260000165700915886191069148480892181396484375f,0.7712000087380410651149986733798868954181671142578125f,0.04400000712275531355999191873706877231597900390625f,-0.607600004391371850687164624105207622051239013671875f,-0.365500061929226571777462595491670072078704833984375f,0.051600000636279563792641056352294981479644775390625f,-0.1962999791920185810312204921501688659191131591796875f,-0.18239999516159299819406669485033489763736724853515625f,-0.267500033080577015454082356882281601428985595703125f,0.5125999654993418364057333747041411697864532470703125f,0.43859998747110306993590711499564349651336669921875f,0.8253000107899304271796836474095471203327178955078125f,-0.0504999427199355199746833022800274193286895751953125f,0.7463999736368658322049896014505065977573394775390625f,0.551799996417760230116300590452738106250762939453125f,0.63320001617372001501138356616138480603694915771484375f,0.52270003720670998870190260277013294398784637451171875f,0.09880000139474864884192584213451482355594635009765625f,-0.7349999750778071216927855857647955417633056640625f,-0.880300011684000072165190431405790150165557861328125f,-1.3012000673368582237543478186125867068767547607421875f,-0.19009999988377102031478216304094530642032623291015625f,0.65179999477863315160419688254478387534618377685546875f,0.8616999948948620424715727494913153350353240966796875f,0.45370001693367978390369898988865315914154052734375f,0.24170000066906205393735262987320311367511749267578125f,-0.4341999874442807882957140463986434042453765869140625f,-0.2085000021904708233932979055680334568023681640625f,-0.074499988809228145925089847878552973270416259765625f,0.78089999958723754769351899085449986159801483154296875f,0.959200064706802635328131145797669887542724609375f,0.201200000505149301233132064226083457469940185546875f,0.0406000115215776613553089191555045545101165771484375f,1.029600020506978186318747248151339590549468994140625f,0.333400001263618239732977599487639963626861572265625f,-1.3868000206321478007254199837916530668735504150390625f,-0.600200011837482616527950085583142936229705810546875f,0.1908000040993094170715949076111428439617156982421875f,-0.3630000102221966340465542089077644050121307373046875f,0.312000041559338636698583968609455041587352752685546875f,-0.095600001835823178453210857696831226348876953125f,0.1818999995052814622198411598219536244869232177734375f,0.266999987840652153181508765555918216705322265625f,-0.4835000099018216257462654539267532527446746826171875f,0.0601999786078935006372603311319835484027862548828125f,-1.29919998379349710404540019226260483264923095703125f,-0.206599977874756035589598468504846096038818359375f,0.732599984386563374272327564540319144725799560546875f,0.1006000027298927790297966566868126392364501953125f,0.7695000128373503667233990199747495353221893310546875f,0.43160000551640942401121492366655729711055755615234375f,-0.967100027100741943542061562766321003437042236328125f,-0.28559999088346910145475021636229939758777618408203125f,0.37700001592934129224232719934661872684955596923828125f,-0.34700001794099843976937336265109479427337646484375f,-0.133100022250414173896615466219373047351837158203125f,-0.24440003213882466326367648434825241565704345703125f,-0.0274999868869780783597889239899814128875732421875f,0.02119998817443846661490169935859739780426025390625f,-0.97640000261962411887139978716732002794742584228515625f,-0.183800036188960458360952543444000184535980224609375f,0.52529997655451321492137140012346208095550537109375f,1.0013999992668625171887697433703579008579254150390625f,0.394699993717670560844368310426943935453891754150390625f,0.18460000016689281920889698085375130176544189453125f,-0.9187999959185717546716887227375991642475128173828125f,-0.00110005236566090847105670036398805677890777587890625f,-0.1539999996349215027890977580682374536991119384765625f,0.67359996549189116166189705836586654186248779296875f,-0.1862999841466546069312926192651502788066864013671875f,-0.20640004456639271523954448639415204524993896484375f,0.8385999769285323868217574272421188652515411376953125f,0.63530000382363738253843621350824832916259765625f,-0.7254999644756321419691857954603619873523712158203125f,-1.8121999813497058351430268885451368987560272216796875f,-0.20729999993443481987043242043000645935535430908203125f,0.19179998390078534242775276652537286281585693359375f,0.1393999986484646402828957434394396841526031494140625f,0.03660000957697651102762392838485538959503173828125f,0.82969996827393810434614351834170520305633544921875f,0.50449998400360274164455631762393750250339508056640625f,0.218400001710653046682608646733569912612438201904296875f,1.40950003559887449000598280690610408782958984375f,-0.51050001178681814195670085609890520572662353515625f,-0.038700047779083757149010125431232154369354248046875f,-0.58760001463592015369385990197770297527313232421875f,-0.536800047081710030028034452698193490505218505859375f,0.3504999994561075471466438102652318775653839111328125f,0.132800024089217316713984473608434200286865234375f,0.955699983651935980333291809074580669403076171875f,0.41699999283254196580372763492050580680370330810546875f,0.032700040112435935224510785701568238437175750732421875f,0.1955999894678586148444310310878790915012359619140625f,-1.6019000199198722977911302223219536244869232177734375f,-0.792200000870227682980839745141565799713134765625f,0.316599988901615336800432487507350742816925048828125f,0.10000000089406935188662828295491635799407958984375f,0.42739998886585228721202156521030701696872711181640625f,0.16480000477731227004341008068877272307872772216796875f,0.7450000170618296113644873912562616169452667236328125f,0.4396000031471256619397536269389092922210693359375f,-1.0928999936491248234915474313311278820037841796875f,-0.438399995303154099701714585535228252410888671875f,0.9925999899744983867577730052289552986621856689453125f,0.7480000281035901199544468909152783453464508056640625f,-0.6163999935671686802862723197904415428638458251953125f,-0.0605999809741977202293128357268869876861572265625f,0.2831000184133649621998074508155696094036102294921875f,-0.32759997507333782351679474231787025928497314453125f,0.58669998390525568421338675761944614350795745849609375f,0.058000000834464771060083876363933086395263671875f,-0.166900002597272223425761694670654833316802978515625f,0.07919999534189703371822588451323099434375762939453125f,0.3848000023186208817804754289682023227214813232421875f,0.52480002392530433752426688442938029766082763671875f,0.24039999011009971496832804405130445957183837890625f,0.09619999115467070538443294935859739780426025390625f,0.9045999622061848821630292150075547397136688232421875f,0.604499991081655085878310273983515799045562744140625f,0.70749999716877909339274310696055181324481964111328125f,0.42290006539821656250666137566440738737583160400390625f,-0.41820000611543617363707880940637551248073577880859375f,-0.6881000668048866675263752767932601273059844970703125f,-0.624699980977177826702018137439154088497161865234375f,-0.233899973040819109115773244411684572696685791015625f,-0.1712000028520821448552169385948218405246734619140625f,-0.341200007657706805286323969994555227458477020263671875f,0.158999995015561612632382093579508364200592041015625f,-0.09619999223500474982984087546356022357940673828125f,-0.6292999598726634058465378984692506492137908935546875f,-1.36910001259446101329331213491968810558319091796875f,0.1199000023886560573060933165834285318851470947265625f,0.02050001327693562469534072079113684594631195068359375f,0.8296999757245184792253667183103971183300018310546875f,-0.0449999717622997508215121342800557613372802734375f,-0.5774000035062429869725519893108867108821868896484375f,-0.095600065165758252305749920196831226348876953125f,0.37479999203681957364864274495630525052547454833984375f,1.24259997626543050586178651428781449794769287109375f,0.86439996694624454676159075461328029632568359375f,0.9012000279232850186161840611021034419536590576171875f,-0.4043999822571875402132945964694954454898834228515625f,-1.291199980835616489827089026221074163913726806640625f,0.8397999945700167501172472839243710041046142578125f,0.652300064307450799816479047876782715320587158203125f,-0.6355999693140390593981692290981300175189971923828125f,-1.559599961572885273852762111346237361431121826171875f,0.845800029207766090877385067869909107685089111328125f,0.842000020325183839986493694595992565155029296875f,1.0820999835148452650201988944900222122669219970703125f,1.04439999518394444066871074028313159942626953125f,-0.769800014910102081699960763216949999332427978515625f,0.0946000117897991543713942519389092922210693359375f,0.6592000267088413689720027832663618028163909912109375f,0.001999995887279393258495474583469331264495849609375f,0.73350001806020748151837551631615497171878814697265625f,0.70460001357793800025319796986877918243408203125f,0.36079998379647715278650821346673183143138885498046875f,0.42959998943805732807277308893389999866485595703125f,0.1377000522196294429733143260818906128406524658203125f,0.144200007784366590612989966757595539093017578125f,-0.632100011231005343148581232526339590549468994140625f,0.19200001399219057685030520588043145835399627685546875f,-0.15829999866038557687630827786051668226718902587890625f,-0.1962000005424020532274198558297939598560333251953125f,0.37839998427629450983289416399202309548854827880859375f,-0.14240003565549841368920169770717620849609375f,0.147500022463500357616794644854962825775146484375f,0.46140000239610667875922445091418921947479248046875f,-1.0341000283524393044132239083410240709781646728515625f,-0.20140002057552397474182726000435650348663330078125f,0.9555000199452046150128126100753434002399444580078125f,0.016999994993209810445478069595992565155029296875f,-1.430800022576749430669451612629927694797515869140625f,-0.637400006747245839733295724727213382720947265625f,0.60039999532550591965218700352124869823455810546875f,-0.40809994238018898471409556805156171321868896484375f,0.7543999936938285966192552223219536244869232177734375f,-0.50899996912479394950423738919198513031005859375f,0.862599969112873399268437424325384199619293212890625f,-0.45569999687671725041582249104976654052734375f,0.5449999924376605076048463160987012088298797607421875f,1.2070000364184381158594305816222913563251495361328125f,0.1692999956354499513366818064241670072078704833984375f,-0.310199990975856820796252577565610408782958984375f,0.457300004926323833132073559681884944438934326171875f,-0.335699997025727725485921837389469146728515625f,-0.21129998727589860418873968228581361472606658935546875f,-0.6034000264465806395719482679851353168487548828125f,0.7887000025540589209782638135948218405246734619140625f,0.2231000218406327118003673604107461869716644287109375f,-0.15499999474734071913673005838063545525074005126953125f,-0.223300015722215139390982585609890520572662353515625f,-0.2424999889731408730941808471470721997320652008056640625f,0.4346999969959262077878747732029296457767486572265625f,-0.4678000036001204176017154168221168220043182373046875f,-1.092100013130903324309883828391321003437042236328125f,0.2366999937027693956537177655263803899288177490234375f,-0.616399967825413508393239681026898324489593505859375f,-0.85740001065880022679266403429210186004638671875f,-0.24410005908608578550911261118017137050628662109375f,0.8134000113591548508651385418488644063472747802734375f,1.141700046527386547268179128877818584442138671875f,-0.481900004348158716993566486053168773651123046875f,0.65249998122453600757353342487476766109466552734375f,-0.770800015591829978944815593422390520572662353515625f,-0.58300002408027662426093229441903531551361083984375f,-0.69080001631826182251217005614307709038257598876953125f,-0.4098000021696091010880991234444081783294677734375f,-0.00709999632984403827862252001068554818630218505859375f,-0.376400004482269512351422235951758921146392822265625f,0.9797999943092463581706397235393524169921875f,0.38619998221397366933160810731351375579833984375f,-1.0003000110879540773822782284696586430072784423828125f,-0.191399997776746655375745831406675279140472412109375f,1.022400017178058728717360281734727323055267333984375f,0.18199998396635042041680208058096468448638916015625f,-1.270700027498602668885041566682048141956329345703125f,0.085800011289119737512010033242404460906982421875f,0.3599999871850012578278210639837197959423065185546875f,-0.16419999325573420012602809947566129267215728759765625f,0.698000016033649473001787555404007434844970703125f,-0.07140000656843170645515783689916133880615234375f,-1.48529998720884304219680416281335055828094482421875f,-0.8410000079274180251331927138380706310272216796875f,1.0484999648630617397060404982767067849636077880859375f,0.460399989551306010326925388653762638568878173828125f,0.65290002874135988353287984864437021315097808837890625f,0.850000019967556141864406527020037174224853515625f,-0.6702999941006300144152874054270796477794647216796875f,-0.0179999808669091265755923814140260219573974609375f,0.3234999813660979217644353411742486059665679931640625f,-0.07239999624192716698445337897283025085926055908203125f,0.18200000143796202234369729922036640346050262451171875f,0.02270000140666983412529589259065687656402587890625f,0.050300015223026406374629004858434200286865234375f,-1.4819999617636205879733779511298052966594696044921875f,0.82669998953044387945965354447253048419952392578125f,-0.67089998893290758719132327314582653343677520751953125f,1.3665999933719632508655195124447345733642578125f,1.06589999783635125396585863200016319751739501953125f,-0.9155000061988829429537872783839702606201171875f,0.5917999961197377700727884075604379177093505859375f,-0.87199999558925611609083716757595539093017578125f,-0.0278999923825267170940378491650335490703582763671875f,-0.61590002979934244109472274431027472019195556640625f,-0.26299999051541067540682661274331621825695037841796875f,1.0867000143408773737974115647375583648681640625f,0.863399994856119068487032564007677137851715087890625f,0.4637999690592291557322823791764676570892333984375f,0.61919999802410596867474623650196008384227752685546875f,-0.56040000378191479057932156138122081756591796875f,0.25280000129044044587800499357399530708789825439453125f,-0.08290001771450050682688015513122081756591796875f,-0.96979996804595014481265025096945464611053466796875f,0.782099980795383231679807067848742008209228515625f,-0.167299992632865990316304305451922118663787841796875f,-0.57139998421668991568367346189916133880615234375f,-0.304000007718801423806098682689480483531951904296875f,0.61530001358389796450865105725824832916259765625f,0.2871000313103200252129454383975826203823089599609375f,-0.15879999036788916555451578460633754730224609375f,-0.66229998175501858526104115298949182033538818359375f,0.718799985152482445727173399063758552074432373046875f,-0.013999999225139614367208196199499070644378662109375f,0.7132999765694141292016183797386474907398223876953125f,-0.166999992012977660493788789608515799045562744140625f,-0.4861000078111887834353410653420723974704742431640625f,-0.5374000331223012238979208632372319698333740234375f,0.365599957877396519734247704036533832550048828125f,0.159800000381469686772106797434389591217041015625f,-0.5606999748975034503217784731532447040081024169921875f,0.84519998829364784143081124057061970233917236328125f,1.52939997968673679906714824028313159942626953125f,-0.195999995708465490906746708787977695465087890625f,-0.787399987822771318946024621254764497280120849609375f,0.226800008636713013032704111537896096706390380859375f,0.26160001557469370059294533348293043673038482666015625f,-0.00600000301003456026904814279987476766109466552734375f,0.755800007191300249331789018469862639904022216796875f,0.003200008201599047197305480949580669403076171875f,0.229299986284971613059724404592998325824737548828125f,0.007400006449222562565637417719699442386627197265625f,-0.9716999901264904426767543554888106882572174072265625f,0.492000003933906526754071819595992565155029296875f,0.63920002447366730535804890678264200687408447265625f,-0.28340000454187386225157752051018178462982177734375f,0.718299983423948074090503723709844052791595458984375f,0.01020001720190055749526436557061970233917236328125f,-1.259599990779161071685621209326200187206268310546875f,0.32059999185204512972546808668994344770908355712890625f,0.076099978306889415335945159313268959522247314453125f,0.74359998285174366827732228557579219341278076171875f,-0.15470000146627427284329314716160297393798828125f,0.13220001897513855571020258139469660818576812744140625f,0.2115999902427194712117852759547531604766845703125f,0.5692999939590694946645044183242134749889373779296875f,-1.18629999454021373139767092652618885040283203125f,-0.7184000243604184987589178490452468395233154296875f,-0.2812000056087970989437962998636066913604736328125f,1.6463999799698594284080854777130298316478729248046875f,-0.149600021624564760713838040828704833984375f,-0.51799998742342001634142434340901672840118408203125f,-0.16759998133182563151422073133289813995361328125f,0.40130002076625803653087132261134684085845947265625f,-0.14130001391172442026800126768648624420166015625f,0.096899998387694363799482744070701301097869873046875f,-0.261999997898936243245771038345992565155029296875f,-0.02540000512301932023007111638435162603855133056640625f,0.921399985408783095408580265939235687255859375f,-0.604500004231929732867456550593487918376922607421875f,0.5514000013530253596627517254091799259185791015625f,0.7602000035673379674250327298068441450595855712890625f,0.09399997746944421805892488919198513031005859375f,0.516400003290176545789336159941740334033966064453125f,-0.003199985849857256425821105949580669403076171875f,-0.6607999954938890141420415602624416351318359375f,0.76860000505447345631182543002068996429443359375f,0.852699988070130476813801578828133642673492431640625f,-0.740399996182322439608469721861183643341064453125f,0.83339999135434605914696248873951844871044158935546875f,-0.4668000052839518243530392282991670072078704833984375f,0.01639997408390048150295115192420780658721923828125f,1.048099958920478247392793491599150002002716064453125f,0.642599986696243075101619979250244796276092529296875f,0.4223000166237351749032313819043338298797607421875f,-0.345200017201900521968127577565610408782958984375f,-1.0240000267922884091831292607821524143218994140625f,0.6430000033676623871770061668939888477325439453125f,0.71210005604624715402906076633371412754058837890625f,0.5692000177681446615451932302676141262054443359375f,-0.5143999770045279973373908433131873607635498046875f,0.0791999972045420719268804532475769519805908203125f,-0.01329996837377589447015679979813285171985626220703125f,-0.31159999113678937820992587148793973028659820556640625f,-0.899199992510676171519889976480044424533843994140625f,-0.3298000041067599141797472839243710041046142578125f,0.9932999957919117495208638501935638487339019775390625f,-0.1833999990284442827004340870189480483531951904296875f,0.10029998415410512624390548808150924742221832275390625f,0.43320001013874998019304030094644986093044281005859375f,0.35080002760589135135660399100743234157562255859375f,0.208400017207861054657769273035228252410888671875f,0.35529999197721462422805416281335055828094482421875f,-0.92720000504255306594814101117663085460662841796875f,-0.1403000015139586054147002869285643100738525390625f,-0.4045999959945678625672371708787977695465087890625f,-1.3867000246599314738915609268588013947010040283203125f,-0.0970000118687750212398412941183778457343578338623046875f,0.8996000172846019271588602350675500929355621337890625f,0.7588999644726517335158177957055158913135528564453125f,-1.01899998998641905245676753111183643341064453125f,-0.15410002347230911112774265347979962825775146484375f,0.567599953317643013406268437393009662628173828125f,-0.2114000177443025041412738573853857815265655517578125f,0.0506000107765203210874460637569427490234375f,0.05330000890493380438783788122236728668212890625f,-0.13430002815723440789952292107045650482177734375f,-0.142199990886449967320004361681640148162841796875f,0.966899983561038567358991713263094425201416015625f,-0.1001999814391136833791051685693673789501190185546875f,-0.141700010168552381628614966757595539093017578125f,-0.351799981442094011896415395312942564487457275390625f,0.386199998828768542580291978083550930023193359375f,0.56830001121461404256507421450805850327014923095703125f,-0.74929997808933279657139792107045650482177734375f,-0.662600004577636791935901783290319144725799560546875f,0.4999000055551530152797568007372319698333740234375f,0.1581000076472758308909760671667754650115966796875f,0.98719996562898160163967986591160297393798828125f,0.77250001497566700425778662975062616169452667236328125f,0.74789999506473492374425404705107212066650390625f,-0.094799958360195279993831718456931412220001220703125f,-0.36999998986721038818359375f,-1.30630001599788680977098920266143977642059326171875f,0.05020001928806294699825230054557323455810546875f,0.25350000120326876551501982248737476766109466552734375f,-0.9841000075094399335284833796322345733642578125f,-1.268700040049105748352076261653564870357513427734375f,-0.909600034998357731552687255316413938999176025390625f,0.25939999756813048037429325631819665431976318359375f,-0.98690000196397331677644615410827100276947021484375f,0.65880000050067888395233239862136542797088623046875f,0.84030001716017732604768752935342490673065185546875f,0.36179999850392352556127661955542862415313720703125f,0.128699981617927505794796161353588104248046875f,0.5061999923467634943108350853435695171356201171875f,2.15820000253915811327942719799466431140899658203125f,-0.03219999818801877466967198415659368038177490234375f,-0.65170000376105330364140399979078210890293121337890625f,-0.79719997896552075022924555014469660818576812744140625f,-0.70399998573958821879159586387686431407928466796875f,-0.0485999928355216326281151850707828998565673828125f,0.08729996029734632401186900096945464611053466796875f,-0.01940000472068792447544183232821524143218994140625f,-0.8194999538660052706973146996460855007171630859375f,1.0421999987840653290049885981716215610504150390625f,0.66760003519952260120362552697770297527313232421875f,0.052600000628828968984862513025291264057159423828125f,-0.0416000475749367382150012417696416378021240234375f,-0.18379998410940157071991052362136542797088623046875f,-0.6258999771237372389265374295064248144626617431640625f,-0.5986000196576120035985013601020909845829010009765625f,0.198699984598159407056527925305999815464019775390625f,0.537599991762637952064096680260263383388519287109375f,0.2291000278681512014600230031646788120269775390625f,0.49779999749064440806023412733338773250579833984375f,1.3090000049993395503378224020707421004772186279296875f,-0.77320001695603135516421389183960855007171630859375f,0.2234999934732915249924189993180334568023681640625f,1.641700004357099285101639907225035130977630615234375f,-0.376200043606758072201046161353588104248046875f,0.830599978888034939927820232696831226348876953125f,-0.42509999682903298889868892729282379150390625f,1.3360999831497668566271386225707828998565673828125f,-0.499600030565261477022431790828704833984375f,-0.851500000268220702537291799671947956085205078125f,1.1558000002622605961732915602624416351318359375f,-0.418399971833825379263771537807770073413848876953125f,-0.2359999621063479935401119291782379150390625f,0.20800000806152818544347837814711965620517730712890625f,0.076200005757808497719452134333550930023193359375f,-0.7017999791324138758596973275416530668735504150390625f,-0.51260000278949746643775142729282379150390625f,0.53299999174475676255724465590901672840118408203125f,0.1355999750137328874188824556767940521240234375f,1.03519998054504380746720926254056394100189208984375f,0.03109999529421347830293598235584795475006103515625f,-0.57919997649192822564145899377763271331787109375f,-0.854799973559380532606155611574649810791015625f,-0.3697999881625175788713022484444081783294677734375f,-0.969599992269277066725408076308667659759521484375f,-0.460500012159347438256418172386474907398223876953125f,0.7248999618947511436317654442973434925079345703125f,1.17709997364282603626861600787378847599029541015625f,0.731699972319603375581209547817707061767578125f,0.06569998031780144298608092867652885615825653076171875f,0.8759000458180903336824485450051724910736083984375f,-0.6738000259816647741217821021564304828643798828125f,0.4798999469935889550242791301570832729339599609375f,0.32900001785159105338607332669198513031005859375f,0.1021000636458400112616118349251337349414825439453125f,0.3230000208020211704962321164202876389026641845703125f,-0.399899979925155246718304624664597213268280029296875f,0.247600010091066469186671383795328438282012939453125f,0.0429000326901676221069692473975010216236114501953125f,-0.3472000002741812973994228741503320634365081787109375f,-0.372600047679245260923153182375244796276092529296875f,0.251799996864795705420192462042905390262603759765625f,0.68459999528676285240891274952446110546588897705078125f,-0.66559998123496766364581844754866324365139007568359375f,-1.3214000006638466633290818208479322493076324462890625f,1.0399999900162217869592495844699442386627197265625f,1.141000017054378812275672316900454461574554443359375f,-0.20859999156892305993693526033894158899784088134765625f,-0.4620000182949002276444616654771380126476287841796875f,-0.5321999989330767899531338116503320634365081787109375f,0.4698000154316428478296074899844825267791748046875f,-0.9682000210165977449605634319595992565155029296875f,-0.099600076945125959326787778991274535655975341796875f,-0.01780000292956829266444174209027551114559173583984375f,-0.4743999649532144591290716562070883810520172119140625f,0.7388000161468983861823289771564304828643798828125f,-0.2619999567717314903347869403660297393798828125f,0.60820000209212299324690320645458996295928955078125f,1.5225000093877316675872179985162802040576934814453125f,0.230799981337785897750336516764946281909942626953125f,-0.4058999863624563886332907713949680328369140625f,-1.1561999942839147337281247018836438655853271484375f,1.055899976825713793004979379475116729736328125f,0.08710000910758974868031145888380706310272216796875f,1.4113999992668624372527119703590869903564453125f,0.2927000121355056716510034675593487918376922607421875f,0.4265000033974644111367524601519107818603515625f,0.5506999937474728046282734794658608734607696533203125f,0.16890001856386671619247863418422639369964599609375f,-0.06670001594275232204012127112946473062038421630859375f,-0.481399999117851251639876863919198513031005859375f,-0.78779996267780683094628102480783127248287200927734375f,0.070500020559876475090277381241321563720703125f,0.63059998598471278530297468023491092026233673095703125f,0.061899940347672099960618652403354644775390625f,0.52569999829232683108415358219644986093044281005859375f,0.176399995355307925848364902776665985584259033203125f,-0.144999994728714221015053453811560757458209991455078125f,-1.02009996926188506449761916883289813995361328125f,-0.7828000283360478928074144278070889413356781005859375f,-0.1209999599456779151296359486877918243408203125f,-1.24899998797476285972152254544198513031005859375f,0.437499998547136836890558697632513940334320068359375f,0.05900000863149745811142565798945724964141845703125f,0.8629999839961524088494115858338773250579833984375f,1.74910000082254413200644194148480892181396484375f,0.30669996814727795708677149377763271331787109375f,-0.237600002193450787757456055260263383388519287109375f,0.34679999511390924293863236016477458178997039794921875f,0.694000023514032449156729853712022304534912109375f,0.454300038886071178012571181170642375946044921875f,0.5388000058650970203188990126363933086395263671875f,-0.3251999541699888141010887920856475830078125f,0.56040002322793025513192333164624869823455810546875f,0.358600032100082177777267133933492004871368408203125f,-0.289999995827674883486224644002504646778106689453125f,-0.66379997859597228426764559117145836353302001953125f,-0.88560000943541528073410518118180334568023681640625f,-0.79220001897513914190795958347734995186328887939453125f,0.20339999396204955228739663652959279716014862060546875f,0.99239998227208825465339714355650357902050018310546875f,-0.09919999094605458367368555627763271331787109375f,-0.999799944353104041994129147497005760669708251953125f,0.3179999911487101371676544658839702606201171875f,0.1940999544799328901945045799948275089263916015625f,-0.5276000053226947983375794137828052043914794921875f,0.97929996870160085364886981551535427570343017578125f,-0.76980000425577177480818136245943605899810791015625f,-0.998800000202656246273136275704018771648406982421875f,-0.141200010228156980218727767351083457469940185546875f,0.36739999244213095153099857270717620849609375f,0.3799999934434890036527576739899814128875732421875f,0.082100038462876501199616541271097958087921142578125f,-0.710799990427493977307449313229881227016448974609375f,-1.8618000150442128859396007101167924702167510986328125f,0.7993999948859211723828366302768699824810028076171875f,0.7054000166714189390404499135911464691162109375f,-0.31120001369267724644629424801678396761417388916015625f,0.56529999048709811404478386975824832916259765625f,0.031600028836727300785014449502341449260711669921875f,1.2631999724388123240714776329696178436279296875f,0.1295999864578245563961900188587605953216552734375f,0.6834000064790242134904474369250237941741943359375f,-0.59899996256828302865216073769261129200458526611328125f,0.8156000045180320512372418306767940521240234375f,-0.154099980258941560151697558467276394367218017578125f,0.0071000228166582246558391489088535308837890625f,-0.1142000249952077606252487385063432157039642333984375f,-0.25429999507665623781349495402537286281585693359375f,-1.1706999639078976560568889908608980476856231689453125f,0.795299983036517943446597200818359851837158203125f,1.107399967855215106027344518224708735942840576171875f,0.47229997355938024838906130753457546234130859375f,0.620899995005130822534056278527714312076568603515625f,-0.942699983823299447749377577565610408782958984375f,-1.0435999683976173724175851020845584571361541748046875f,0.1858000014543534916811040602624416351318359375f,-0.4061000281512736620470604975707828998565673828125f,-0.63079997969865786444643163122236728668212890625f,-0.76359998061656941814590027206577360630035400390625f,0.740799974930286708740823087282478809356689453125f,-0.8554999790787698454863630104227922856807708740234375f,0.25700003105401947323116473853588104248046875f,0.954200003463029577943643744220025837421417236328125f,-0.230800067168473788115079514682292938232421875f,0.2632999957919122113736420942586846649646759033203125f,-1.0539999914765356692214481881819665431976318359375f,-0.1876000017464161118141419137828052043914794921875f,1.567599990570545553936199212330393493175506591796875f,0.14359998329877843303847839706577360630035400390625f,-1.719499995589256702288594169658608734607696533203125f,-0.404600004041194782899992787861265242099761962890625f,0.3516999641239646479107250343076884746551513671875f,-0.217400008684396794933491037227213382720947265625f,0.0746000028491022870724691529176197946071624755859375f,0.0376000178396702633421000427915714681148529052734375f,-1.01699997681379272762569598853588104248046875f,0.7391999897539616171115994802676141262054443359375f,0.9298999984025950737986931926570832729339599609375f,0.80699999052286131018263404257595539093017578125f,-0.12529997186064800729354828945361077785491943359375f,0.90319998912811261249089511693455278873443603515625f,-0.1213999828755856214002051274292171001434326171875f,0.1145999967396260643681671353988349437713623046875f,0.334400014555453761744274743250571191310882568359375f,-0.647999984145164464877097998396493494510650634765625f,-0.442200003403425245807056853664107620716094970703125f,-0.492800019246339982004201374365948140621185302734375f,0.567199982467293306598321578348986804485321044921875f,0.071400008803605974350148244411684572696685791015625f,-1.064599998825789040068912072456441819667816162109375f,-0.36540001488327999989991212714812718331813812255859375f,0.35199998433887902393735203077085316181182861328125f,-1.08280001388192204103688709437847137451171875f,-0.81050004568696021323148670489899814128875732421875f,0.4406999855518340947213573599583469331264495849609375f,-0.4892999953746794972175848670303821563720703125f,-1.4479000148832794625519682085723616182804107666015625f,1.3903999843359002852594130672514438629150390625f,0.89209997155666354018421770888380706310272216796875f,-1.0307999630093576115541509352624416351318359375f,-1.138700009781122179930434867856092751026153564453125f,-0.288899989283084579483329434879124164581298828125f,-0.6030999866366386630289753156830556690692901611328125f,0.80450000438094093624386005103588104248046875f,1.0295999848186971892260999084101058542728424072265625f,-0.045900016164779344762791879475116729736328125f,0.2975999974250793655983216012828052043914794921875f,0.782999949276447892998476163484156131744384765625f,0.9527999688804149602816551123396493494510650634765625f,0.4275999491453177370203775353729724884033203125f,0.3865000019073487891319018672220408916473388671875f,-0.704099983835220655237208120524883270263671875f,-0.4209999594986442161825834773480892181396484375f,-0.1675999810338026918543619103729724884033203125f,-0.451400001204013801014980344916693866252899169921875f,-0.4492000025689595332778480951674282550811767578125f,-0.94249999344348911467506013650563545525074005126953125f,1.00149998372793191947494051419198513031005859375f,0.2455999968439339653514252859167754650115966796875f,-0.80360000706613021748125902377068996429443359375f,-0.794200011882185918921095435507595539093017578125f,-0.3348999942302697263585287146270275115966796875f,0.52449998098611860797291228664107620716094970703125f,0.248500049948692236512215458787977695465087890625f,0.317200006532669220860043424181640148162841796875f,-1.2524000262677663641852632281370460987091064453125f,0.5251999964892863914656118140555918216705322265625f,0.400800046306847956856245218659751117229461669921875f,0.6869999949932097393912044935859739780426025390625f,-0.2786000154852867893850998370908200740814208984375f,-0.154400020217895672658414696343243122100830078125f,-1.0581000054121023623565633897669613361358642578125f,-0.4454000239729882792971693561412394046783447265625f,0.6752999729037281184673702227883040904998779296875f,-0.34240000406503678931358081172220408916473388671875f,0.69049997960031017640858408412896096706390380859375f,-1.0332000184834004841150090214796364307403564453125f,-0.740300061900913686230296661960892379283905029296875f,-0.7850000022724270554164149871212430298328399658203125f,1.3016999492041769759254066229914315044879913330078125f,0.40880001540184007779998864862136542797088623046875f,-0.41239992747306875031654271879233419895172119140625f,0.13660000112056724930198470246978104114532470703125f,-1.15949999961256988711966187111102044582366943359375f,-0.13040000594258316368723171763122081756591796875f,0.86709999957084615829217000282369554042816162109375f,-0.2409999918341636515606296597979962825775146484375f,0.35720001353621488870970779316849075257778167724609375f,0.31160000573992741035311837549670599400997161865234375f,0.26599999324977385839474663953296840190887451171875f,-0.4236000001370907597220138995908200740814208984375f,-0.4977000206291677386616356670856475830078125f,-0.8286000350058078112169823725707828998565673828125f,1.0632000141620636668449151329696178436279296875f,0.64790002278089531273508328013122081756591796875f,-0.357100020730495515408620121888816356658935546875f,0.377499973922967857475896380492486059665679931640625f,0.596799966764450573464273475110530853271484375f,-0.00790000653862943380545402760617434978485107421875f,-0.838200005817412829856039024889469146728515625f,0.4686999869823456066342259873636066913604736328125f,-1.5187999975204462543842964805662631988525390625f,-0.25010000830292700157997387577779591083526611328125f,-0.3774999921023844962064686114899814128875732421875f,-0.626900017237663309543904688325710594654083251953125f,-0.24169997887611316400580108165740966796875f,-0.193400022795796378005661608767695724964141845703125f,-0.950999966092407778006645457935519516468048095703125f,-1.753799959671497266100459455628879368305206298828125f,1.683599959158897263478138484060764312744140625f,-0.170200012731552163813830702565610408782958984375f,1.09939998683929474054821184836328029632568359375f,0.98410001140236857963827787898480892181396484375f,-0.3805000097751616294772247783839702606201171875f,-0.036199987280368883801884294371120631694793701171875f,0.39820000045299508428797707892954349517822265625f,0.057299991366267288839253524201922118663787841796875f,0.7717999997705220494026434607803821563720703125f,0.43370001268684854611734635909670032560825347900390625f,0.11729999978542338112674769945442676544189453125f,0.4926999602794648325243542785756289958953857421875f,-0.01759999921321853122435641125775873661041259765625f,-0.50200001749396339079112294712103903293609619140625f,-0.3341999846875667312673385822563432157039642333984375f,0.2582000128209591682804102674708701670169830322265625f,-0.021099988141655867668333712572348304092884063720703125f,0.6306000087648631069470184229430742561817169189453125f,0.704800016362965298100107247591949999332427978515625f,-0.28179999492764462587501839152537286281585693359375f,-1.4088999842166902709550413419492542743682861328125f,-0.23980000872612006190820466144941747188568115234375f,-0.037300027203559960042866805451922118663787841796875f,-0.297600027525425137042702772305347025394439697265625f,-0.4682000208675862751306340214796364307403564453125f,-0.700599995130300357004671241156756877899169921875f,0.72790001085996625107554791611619293689727783203125f,-0.57480000053048119212917299591936171054840087890625f,0.21459996380805979043771003489382565021514892578125f,0.01360001220703122015720509807579219341278076171875f,-0.528200008946657195707530263462103903293609619140625f,-0.061400001502037060419070257921703159809112548828125f,0.4496999738246196809399179983302019536495208740234375f,0.78760000994205459079466891125775873661041259765625f,0.77280001268982889683911707834340631961822509765625f,0.67320000715851779915510633145458996295928955078125f,1.77190003094673098615885464823804795742034912109375f,0.20480000127553932287582938442938029766082763671875f,0.158999978959560639424353212234564125537872314453125f,-0.397000010192394281460792626603506505489349365234375f,0.5008000065207482975893071852624416351318359375f,-0.7432000034332275628656816479633562266826629638671875f,-0.59849998855590769153423025272786617279052734375f,-0.167599992507696082810753068770281970500946044921875f,0.9340999923288819672961835749447345733642578125f,0.822299999077618082310436875559389591217041015625f,0.592199978332221377286259667016565799713134765625f,-0.817199983845651178882008025539107620716094970703125f,0.1615000251531597541543305851519107818603515625f,0.340699998813867477309713649447076022624969482421875f,-0.66819999628066995001063332892954349517822265625f,-0.61800002452731117585926767787896096706390380859375f,-0.757800000917910665521048940718173980712890625f,0.062700022566318569516852221568115055561065673828125f,-0.973699961054325768827766296453773975372314453125f,-0.8588000045239925217543941471376456320285797119140625f,0.1481000375986099015790387056767940521240234375f,-0.10069997929334650876853629597462713718414306640625f,0.8591999854326246577329584397375583648681640625f,1.120200010645389543384453645558096468448638916015625f,-0.5824000051081181794643271132372319698333740234375f,0.9614000033646818987875803941278718411922454833984375f,0.0879999459981917198092560283839702606201171875f,1.797500021755695343017578125f,0.0440999582052228333850507624447345733642578125f,0.5533999771237372300447532325051724910736083984375f,0.7709999796152118278769194148480892181396484375f,0.20779999257326142725332829286344349384307861328125f,-0.30939998832941029149878886528313159942626953125f,-0.995499997109174472598169813863933086395263671875f,0.5030000078938900998082317528314888477325439453125f,-0.24159999590516090250957859097979962825775146484375f,0.0557999783575535701629632967524230480194091796875f,0.32199999484419805639845435507595539093017578125f,-0.0737000021815301664673825143836438655853271484375f,0.106799987477064206586874206550419330596923828125f,1.7118999559938909982292898348532617092132568359375f,0.35640002787709246234726379043422639369964599609375f,-0.070999946236609989824728472740389406681060791015625f,-0.362799999129772121619907920830883085727691650390625f,-1.253999989837408346460279062739573419094085693359375f,1.01599998944997782501786787179298698902130126953125f,0.086900036311149175816126444260589778423309326171875f,0.239399985349178390237057101330719888210296630859375f,0.9844000074028969038408831693232059478759765625f,0.54780001850128190454825016786344349384307861328125f,-0.30369998683333410127715978887863457202911376953125f,-0.84720002724528331583542239968664944171905517578125f,0.22549997311830427548784427926875650882720947265625f,-0.02079999966621404183797494624741375446319580078125f,-0.08260001396536864604058791883289813995361328125f,0.25780000374913214500338654033839702606201171875f,0.503599982255697131705574065563268959522247314453125f,-0.301400026684999655657293260446749627590179443359375f,0.896099994474649275133515402558259665966033935546875f,0.227800024312734716858130923355929553508758544921875f,-0.511999962806701347517446265555918216705322265625f,0.1282000117778778758292901329696178436279296875f,-0.9856999794423568772572252782993018627166748046875f,-0.041600004007667334970932415671995840966701507568359375f,0.23530000658035277183444122783839702606201171875f,0.818000002622604416302465324406512081623077392578125f,0.538499959051609522475700941868126392364501953125f,1.4450999518275262634148248253040947020053863525390625f,-0.86249998986721010396649944595992565155029296875f,-0.89989997515678421535767483874224126338958740234375f,0.92909999396800913018523715436458587646484375f,-0.490699975419044420732461730949580669403076171875f,0.421000019550323401062996708787977695465087890625f,0.563999991029501046568839228712022304534912109375f,0.3235999591588978319123270921409130096435546875f,0.84089996997118010568783574854023754596710205078125f,-0.54059999140500991643421002663671970367431640625f,-0.203299988192319869284574451739899814128875732421875f,1.1625999944448466294488753192126750946044921875f,0.5725000183284283394868907635100185871124267578125f,-0.677899995064734639527159743010997772216796875f,-0.103800005269050643619266338646411895751953125f,0.4219999848604203407376189716160297393798828125f,1.07769999306201924582637730054557323455810546875f,-0.24170001523494732964536524377763271331787109375f,-0.162700024205446158731547257048077881336212158203125f,-0.8497000026583663156998227350413799285888671875f,-1.067399991101026568429688268224708735942840576171875f,1.338099998557567005263990722596645355224609375f,-0.172400011962652310870680594234727323055267333984375f,0.5054000099658964018090046010911464691162109375f,-0.226700029987096485228903475217521190643310546875f,0.8865000048130753373953893969883210957050323486328125f,1.02040001294612903137704051914624869823455810546875f,0.35520001653134869457062450237572193145751953125f,-0.180400012722611524651483705383725464344024658203125f,-0.047100004301965814335062532336451113224029541015625f,-0.67799997833371161704008045489899814128875732421875f,-0.84389996398836275570687348590581677854061126708984375f,-0.21140000276863568640095536466105841100215911865234375f,0.1993999936752020030183984999894164502620697021484375f,-0.55140000776052477959865427692420780658721923828125f,-1.0218999735772609227524299058131873607635498046875f,-0.77940000665783859545854284078814089298248291015625f,-1.80200001391768527270187405520118772983551025390625f,1.01119998862147308926751065882854163646697998046875f,0.14140000731349022800031889346428215503692626953125f,-0.29919999690651888357706411625258624553680419921875f,-0.088200004178285329459185959422029554843902587890625f,-0.472199998933076958707033554674126207828521728515625f,-0.1463000094413755558520051636151038110256195068359375f,0.0284000057339668376954477935214526951313018798828125f,0.30690002275109318219392662285827100276947021484375f,-0.35900000786781305350814363919198513031005859375f,-0.70499997302889827466287897550500929355621337890625f,-0.92720000906586630406991389463655650615692138671875f,-0.14910004478096983149271181900985538959503173828125f,0.11880000124573708575326236314140260219573974609375f,0.34850000390410418305009443429298698902130126953125f,0.67560001956820503465905858320184051990509033203125f,-0.39209996075332131937329904758371412754058837890625f,-0.0105000256448983808610364576452411711215972900390625f,0.429599993014335268526338040828704833984375f,0.04030002014040956481721877935342490673065185546875f,-0.10790000273883304515720737981610000133514404296875f,-1.5010999910473827867463114671409130096435546875f,-0.01329997165203167241998016834259033203125f,-0.61159999150931809719367038269410841166973114013671875f,0.474800015021860755126681397086940705776214599609375f,-1.510499995600432132647483740583993494510650634765625f,0.53440001902580291925914934836328029632568359375f,-1.46709999346137021092317809234373271465301513671875f,0.016100059294700486134388484060764312744140625f,0.11549996551871277716827535186894237995147705078125f,0.1476999477624900691807852126657962799072265625f,0.0027999916791916490410585538484156131744384765625f,-0.729099974894523938928614370524883270263671875f,-0.52059998306036003867802719469182193279266357421875f,-0.073699996817111923519405536353588104248046875f,-0.46510000070333479271766918827779591083526611328125f,0.238599999839067322682240046560764312744140625f,-1.1600999981701372032460994887514971196651458740234375f,0.444000023365020979326800443232059478759765625f,-0.17840000051856019780416318099014461040496826171875f,-0.006099949622154099415638484060764312744140625f,-0.15849999690055849299596957280300557613372802734375f,0.53289999089241035790109890513122081756591796875f,-0.1831999965786934581046807579696178436279296875f,0.8065999932229512836556750698946416378021240234375f,1.01760004004240034447548168827779591083526611328125f,-0.35230001885890960267033733543939888477325439453125f,-0.05440000486969953641391839482821524143218994140625f,0.354699993270635172137872359598986804485321044921875f,-0.44720001308918000404446502216160297393798828125f,-0.091100011312961637344187693088315427303314208984375f,0.339199997353553772683198985760100185871124267578125f,0.6633000023484232343662370112724602222442626953125f,-0.42119999249577499966790128382854163646697998046875f,-0.80189996433854116020256697083823382854461669921875f,0.70460001611113565189725704840384423732757568359375f,-0.03039998031258495103656969149596989154815673828125f,0.57680000029206279776872179354541003704071044921875f,-0.45959997379183814558700760244391858577728271484375f,0.11760000427961347924110668827779591083526611328125f,-0.5431999965041869327109225196181796491146087646484375f,0.121599989572167388462986536978860385715961456298828125f,0.431099999056756555404490427463315427303314208984375f,0.442200017708540116245785611681640148162841796875f,1.578800028887390904941412372863851487636566162109375f,-0.38379999781846996231848834213451482355594635009765625f,-1.3367000224992633405918240896426141262054443359375f,0.38379999439120293658334048814140260219573974609375f,-0.0722999813079832076567754484131000936031341552734375f,-0.1250000040233134601663778084912337362766265869140625f,0.82420003490448057181794183634337969124317169189453125f,0.10420001866221462183403900780831463634967803955078125f,-0.0814999972134824446357015403918921947479248046875f,0.28219999997615818898566431016661226749420166015625f,-1.40879999513626064100435542059130966663360595703125f,1.2688000162959098826576109786401502788066864013671875f,-0.633300010693073289758103783242404460906982421875f,-0.376200015145540245242727905861102044582366943359375f,-0.9597999789237974255229346454143524169921875f,0.1016999904990196768039822927676141262054443359375f,0.087199987459182892735043424181640148162841796875f,-0.8197999747514723711816486684256233274936676025390625f,-0.11289995870590274762435001321136951446533203125f,-0.319100016319751933480119987507350742816925048828125f,0.513300002646445818754727952182292938232421875f,0.2323000222861768548909822129644453525543212890625f,0.488899984514713281669173738919198513031005859375f,0.1554999811649320218265302173676900565624237060546875f,-0.3469999967813492958157439716160297393798828125f,0.3382999959036709913817730921437032520771026611328125f,-0.52630000031441426244782633148133754730224609375f,1.0551000332996249841954750081640668213367462158203125f,0.420499987125396046394598670303821563720703125f,-0.4419999986439944184013484118622727692127227783203125f,0.64260001575350766955807557678781449794769287109375f,-0.014500017382204521965149979223497211933135986328125f,-0.587300002467632253910778672434389591217041015625f,-0.064400003379583292684174011810682713985443115234375f,-0.49909998219609263969687162898480892181396484375f,-0.4885999982744453351557467613019980490207672119140625f,1.2494000328093761709169484674930572509765625f,-0.42720001182258127148116955140721984207630157470703125f,0.794000002503395307940081693232059478759765625f,1.963299972993135522614238652749918401241302490234375f,0.169999993890523892758892543497495353221893310546875f,-0.020999997198581699109354303800500929355621337890625f,-1.2044999919384722364412709794123657047748565673828125f,0.23060000272989267244838629267178475856781005859375f,0.425100026333331992844932756270281970500946044921875f,-0.47360000371336941071831461158581078052520751953125f,-0.0783000309586527887262263902812264859676361083984375f,-0.5000000058114528744823701345012523233890533447265625f,-0.8981000411003829952960586524568498134613037109375f,-0.26339998815059662007342922152020037174224853515625f,0.00799998727440875967431566095910966396331787109375f,-0.08120001097321516425608933786861598491668701171875f,-0.745099963003397203209487997810356318950653076171875f,0.162199992823600869229494492174126207828521728515625f,1.9604999515861276648820421542041003704071044921875f,-0.0629999829530716315417748774052597582340240478515625f,-0.3744999803900717072480119895772077143192291259765625f,0.1187999991595745097328062911401502788066864013671875f,-0.2353000141054393257178389831096865236759185791015625f,0.2854000014722346545426034936099313199520111083984375f,0.38300007843226169246264589673955924808979034423828125f,-0.3176000024914740205161933772615157067775726318359375f,-1.258800038908421736749687624978832900524139404296875f,-0.3670000057220459410700641456060111522674560546875f,-0.19050000411272094424930401146411895751953125f,-0.25840001437664028571816743351519107818603515625f,0.365799981188773326579166678129695355892181396484375f,-0.170600014650821751871490050689317286014556884765625f,0.30160000022649757767112532746978104114532470703125f,-1.204399977749586181374752413830719888210296630859375f,0.5857999983251094278102755197323858737945556640625f,0.3524000080138443014021731869434006512165069580078125f,-0.418800013911724278159454115666449069976806640625f,0.344500015556812311245948876603506505489349365234375f,-0.079099995458125960112738539464771747589111328125f,-0.390200004088878671382190077565610408782958984375f,-0.7089999813437461995135890902020037174224853515625f,0.17910001497864691533123959743534214794635772705078125f,-1.40079998953342510503716766834259033203125f,-0.85839998233914371894570649601519107818603515625f,-1.11249999701976776123046875f,1.077999963283538864544652824406512081623077392578125f,-0.8703999877631662940302703646011650562286376953125f,-0.6335999718993903240260578968445770442485809326171875f,0.89119998862147298268610029481351375579833984375f,0.0996999935686588845129563196678645908832550048828125f,0.25729999665617953041873988695442676544189453125f,-0.320599975907802647867583800689317286014556884765625f,0.01979998033940777446559877716936171054840087890625f,-0.73560002258568990551879096528864465653896331787109375f,0.66470003179013747995895755593664944171905517578125f,0.49570003051608801936112058683647774159908294677734375f,-0.65099999749660497627701261080801486968994140625f,0.343000005751848302537609924911521375179290771484375f,-0.92519998948574055930293980054557323455810546875f,-0.644800021541118706380757430451922118663787841796875f,-0.64739999393224678669866989366710186004638671875f,-0.314200002568960190529878673260100185871124267578125f,-0.8820999763995402975780280030448921024799346923828125f,0.2641999888598920431803662722813896834850311279296875f,-0.62650004225224276144246005060267634689807891845703125f,0.279600003147125075741996624856255948543548583984375f,1.3708000335663552693432620799285359680652618408203125f,-0.7962000024795532482357884873636066913604736328125f,-0.143100041025876922873294461169280111789703369140625f,0.263199996429681615239815073437057435512542724609375f,0.15060001129806011022083112038671970367431640625f,0.198000002548098574806090255151502788066864013671875f,-0.706999991230666591235376472468487918376922607421875f,0.6770000173449517433255095966160297393798828125f,1.83299998056888657771423822850920259952545166015625f,-0.56540000549554836339893881813623011112213134765625f,-0.51639999673366521193429434788413345813751220703125f,0.8078000247597696414914025808684527873992919921875f,0.397600051143765398364848806522786617279052734375f,0.26280002043843264658562475233338773250579833984375f,0.410500006943940842774054544861428439617156982421875f,-0.484799983245134402665144079946912825107574462890625f,-0.9327999767780301709763079998083412647247314453125f,-0.20560000600814820614914424368180334568023681640625f,-0.8133999687045818660635632113553583621978759765625f,0.241599995532631783845545214717276394367218017578125f,0.964599985824525330002643386251293122768402099609375f,0.54520000542998303671993198804557323455810546875f,-0.70479997132420546535058747394941747188568115234375f,0.33599998989701251872475040727294981479644775390625f,0.5560999832987787439009252921096049249172210693359375f,-0.325400027549266557258533794083632528781890869140625f,-0.25399997873604274190029173041693866252899169921875f,1.137000041298568131509227896458469331264495849609375f,-0.5742000389277936989174122572876513004302978515625f,-0.4498000131219626052114790581981651484966278076171875f,-0.088599975252151352833607234060764312744140625f,-0.222299996656179388310192734934389591217041015625f,0.62949996548891107295276015065610408782958984375f,0.1925999895274639328590637887828052043914794921875f,0.30289999335110184830455182236619293689727783203125f,1.767099980199336783925900817848742008209228515625f,-0.3162000108242040141703910194337368011474609375f,-0.755099987739324518543071462772786617279052734375f,0.03780000836849239931325428187847137451171875f,-0.4837999875366687518862818251363933086395263671875f,0.9644999918639651781404609209857881069183349609375f,0.94070004739165280494006537992390803992748260498046875f,1.1576999692201610514530329965054988861083984375f,-0.66809997441768675940920729772187769412994384765625f,0.17970000802278462970207328908145427703857421875f,-1.067899979567527690704764609108678996562957763671875f,-0.14999999105930328369140625f,0.5730999844014643684886323171667754650115966796875f,0.17809997970759905427939884248189628124237060546875f,-0.5980000127553939393010296043939888477325439453125f,1.05489996979236622109965537674725055694580078125f,1.511199972975254102180997506366111338138580322265625f,0.21609996452331614591457764618098735809326171875f,0.729999961405992880969506586552597582340240478515625f,-0.00999999389052330656113554141484200954437255859375f,0.86640001655220988396877146442420780658721923828125f,-0.150799957048892707689446979202330112457275390625f,1.0587999942421912891177271376363933086395263671875f,1.06600007438659627467814061674289405345916748046875f,0.07799999904632581859686979441903531551361083984375f,-1.065300015521049470379466583835892379283905029296875f,-0.101000003755092659929459841805510222911834716796875f,-0.326000005990267016642292219330556690692901611328125f,0.740600012266635854274454686674289405345916748046875f,-0.15549995940923633241936840931884944438934326171875f,0.51780000240802781519278141786344349384307861328125f,0.5597999670028688257161775254644453525543212890625f,-0.19099999570846559748815707280300557613372802734375f,-0.118099971362947808728449672344140708446502685546875f,0.337200019124150429661312955431640148162841796875f,0.31780001496225640522652611252851784229278564453125f,-0.174999999403953498955388567992486059665679931640625f,-0.7127000166803594272124655617517419159412384033203125f,-0.1964000195324420161568923504091799259185791015625f,0.4539999653995032691256028556381352245807647705078125f,0.4690000083148480047867678877082653343677520751953125f,0.77330001993179298125369314220733940601348876953125f,0.07719998343586897959767156862653791904449462890625f,0.0466999772369849353026438620872795581817626953125f,-0.3364000198304655242509397794492542743682861328125f,-0.1785000033080574866062306682579219341278076171875f,-0.644200004059076292151075904257595539093017578125f,-0.985100017690658802393954829312860965728759765625f,0.741100017422437584713179603568278253078460693359375f,0.41949997293949081722530536353588104248046875f,0.845300009560585152712519629858434200286865234375f,0.2601999602794649035786278545856475830078125f,-0.32020002256631840964473667554557323455810546875f,0.8558000181436540287904790602624416351318359375f,0.25890000925064082792914632591418921947479248046875f,-0.371999999165534944722821819595992565155029296875f,0.2820999982297418551269174713524989783763885498046875f,0.1027999693274495029982062987983226776123046875f,1.424399985796213297106760364840738475322723388671875f,-0.48119999532699608835173421539366245269775390625f,-0.48929999135136625909581198357045650482177734375f,-0.935399986720085507840849459171295166015625f,0.09700001645088196511323985760100185871124267578125f,0.39600000204145846538494879496283829212188720703125f,0.262400000786781328887542485972517170011997222900390625f,-0.0475999862492084702125794137828052043914794921875f,0.6830999929696324191041867379681207239627838134765625f,-1.11469998030662509336252696812152862548828125f,-0.027199996250867997105160611681640148162841796875f,-0.1723000103652481129756779409945011138916015625f,0.7984000215291977564646686005289666354656219482421875f,0.48929995722770769361886777915060520172119140625f,-0.689500054299831521831265490618534386157989501953125f,-0.815499997854232105964911170303821563720703125f,-0.56089998964071252629537411849014461040496826171875f,-0.053699978637695267025264911353588104248046875f,0.3262999550521374914069383521564304828643798828125f,-0.7778999661751095917594511774950660765171051025390625f,-0.700400025314092733452753236633725464344024658203125f,0.129099982196092089026251414907164871692657470703125f,-0.327400002127885780822680317214690148830413818359375f,-0.272500015348196455988727393560111522674560546875f,-0.305199999469518701289416640065610408782958984375f,-0.756600019001961054954108476522378623485565185546875f,0.5904000050485134210020987666212022304534912109375f,1.83250002518296195574976081843487918376922607421875f,-0.15739999974370000312973161271656863391399383544921875f,-0.9563000273972750431283884609001688659191131591796875f,0.0169999878406523752261136905872263014316558837890625f,1.38560001897215823163378445315174758434295654296875f,0.25979999173879608775905580841936171054840087890625f,-0.234299991276860186673047792282886803150177001953125f,-0.165600000941753311423099148669280111789703369140625f,-0.974700012753903610729366846499033272266387939453125f,-0.267600005173683097581260881270281970500946044921875f,0.1182999823808668082847361802123486995697021484375f,-0.8220000075101854264403300476260483264923095703125f,-0.49510001634955391747183739425963722169399261474609375f,0.54319999374747285347808656297274865210056304931640625f,0.5529999943524597227906269836239516735076904296875f,0.91320000924468036629377820645458996295928955078125f,0.625200021821260776277995319105684757232666015625f,0.2170000148117541272085873060859739780426025390625f,0.25640000708997290335133811822743155062198638916015625f,-0.07599999958276748568408720529987476766109466552734375f,-0.30299999446421865822998142903088591992855072021484375f,1.2785000144839291369436296008643694221973419189453125f,0.179799993824958814769843229441903531551361083984375f,-0.2794000015541910908467571061919443309307098388671875f,-0.52819997430145770866971588475280441343784332275390625f,-0.2535999759599565361867234969395212829113006591796875f,-0.81639997482895854119533396442420780658721923828125f,-0.3776999878093600937489782154443673789501190185546875f,0.3151999918699264213728383765555918216705322265625f,-0.15630000474750982419891442987136542797088623046875f,0.61759998863339404806538368575274944305419921875f,-0.3819000304996971184579024338745512068271636962890625f,-0.18659999136030680499942491223919205367565155029296875f,0.5000000371038917368338161395513452589511871337890625f,-0.15279999585151671226412872783839702606201171875f,1.2780000185668469736555152849177829921245574951171875f,-0.18359999045133601480728202659520320594310760498046875f,0.3980999998241661064213303689029999077320098876953125f,0.199799995762109716679333359934389591217041015625f,0.04359998717308022353478236254886724054813385009765625f,0.0069999988675117919001422706060111522674560546875f,0.1029999930113552064625537241226993501186370849609375f,-0.426599980854988292122698112507350742816925048828125f,-0.60079999392926719536944801802746951580047607421875f,0.637399984097480665212742678704671561717987060546875f,0.29849999593198284220108007502858527004718780517578125f,-0.151800009828805482214875155477784574031829833984375f,0.80610005225390291005993503858917392790317535400390625f,0.5164000016510481572140633943490684032440185546875f,0.6754000033721323514868117854348383843898773193359375f,-0.6550999894529583311708620385616086423397064208984375f,-0.39549999099969868066040135090588591992855072021484375f,-0.459900005704164394071398191954358480870723724365234375f,0.04329998513758204214951774702058173716068267822265625f,-0.306899992799758603023718706026556901633739471435546875f,-0.73899998447298997916732332669198513031005859375f,-0.5598000108867886570607197427307255566120147705078125f,-0.16290000169575213906369981486932374536991119384765625f,-0.6846999792635439785470907736453227698802947998046875f,-1.2985999627351763052729438641108572483062744140625f,-0.4665999999284746735384032945148646831512451171875f,0.372599984908104087111269109300337731838226318359375f,0.2137999835133552739563356226426549255847930908203125f,-0.40980001081228234482978223240934312343597412109375f,-0.22830000368952740874561868622549809515476226806640625f,-0.1731999971002340377168593477108515799045562744140625f,-0.06339999828338627185786435802583582699298858642578125f,0.45109999924302091045547058456577360630035400390625f,0.1259999963790179311473593770642764866352081298828125f,0.72529998877346513186381571358651854097843170166015625f,0.176499999970197773979663224963587708771228790283203125f,-0.4350000005960463766996326739899814128875732421875f,-0.196199992197751971190200492856092751026153564453125f,-0.0550999895274639772679847737890668213367462158203125f,-0.20290002009868646215551279965438880026340484619140625f,0.653999988645315166735372258699499070644378662109375f,-0.05029999339282509307480495408526621758937835693359375f,-0.8426999777883292797042713573318906128406524658203125f,-0.43729997639060036451752466746256686747074127197265625f,1.080500001095234807735323556698858737945556640625f,0.34159998741149910639336439999169670045375823974609375f,1.03600001027435073552851463318802416324615478515625f,0.2863999949455260374264753409079276025295257568359375f,0.29570001360327058392840626765973865985870361328125f,0.883599996709822921303612019983120262622833251953125f,-0.55430003978014064092150192664121277630329132080078125f,-0.1903999980449675444305057681049220263957977294921875f,0.4943000029370192560662644609692506492137908935546875f,0.0968000028252600230871394160203635692596435546875f,0.229200022014975690609617231530137360095977783203125f,-0.19479999203681952479882966144941747188568115234375f,-0.53819998137950886274438744294457137584686279296875f,0.037800000768899888470286896335892379283905029296875f,0.00740000738054524820341839586035348474979400634765625f,0.053199999111890772240940350457094609737396240234375f,0.86729999531507451226985949688241817057132720947265625f,-0.00340000722408273503560849349014461040496826171875f,0.4805999788880346290653733376529999077320098876953125f,-0.4691999910950659380404204057413153350353240966796875f,-0.07390000437051058579385198754607699811458587646484375f,-0.177999993681907664466734786401502788066864013671875f,0.83990003491044074035443145476165227591991424560546875f,0.43639999092221248844225556240417063236236572265625f,0.72610001075416785720761936318012885749340057373046875f,-0.29340000081658335062684272998012602329254150390625f,-0.24429999742358887715454329736530780792236328125f,0.05159999635219580937928185448981821537017822265625f,-0.1410000146701937406845672740018926560878753662109375f,-0.716899988254904751983076494070701301097869873046875f,-1.047299959701299965075804720981977880001068115234375f,-0.8461999828845263582621782916248776018619537353515625f,-0.07389999747872355584377146442420780658721923828125f,0.411100018167495750986972780083306133747100830078125f,0.5142000033140181614044195157475769519805908203125f,0.0535999843418597521349511225707828998565673828125f,-0.404700019049644144075728036114014685153961181640625f,0.1443999784946441433675090593169443309307098388671875f,0.493999983280897225768057978712022304534912109375f,0.647299992930889356301804582471959292888641357421875f,0.2532999782085421092148180832737125456333160400390625f,0.7894999876171351349540827868622727692127227783203125f,0.5657999787300826977087808700161986052989959716796875f,0.05119999145269404916547273387550376355648040771484375f,-0.566499992072582170266059620189480483531951904296875f,-0.3332000110328199316001018814858980476856231689453125f,0.46870000285208213153254064309294335544109344482421875f,-0.166800009977817464967841942780069075524806976318359375f,0.2817999844223262329023782513104379177093505859375f,-0.21560001703500741854213629267178475856781005859375f,0.12289999163746845312061850563623011112213134765625f,0.184899983128905542884012902504764497280120849609375f,0.3670999717056755429211989394389092922210693359375f,0.29899996919929971994633888243697583675384521484375f,0.62149999505281439127202247618697583675384521484375f,0.326500012934208161397009462234564125537872314453125f,-0.657799998626857895800412734388373792171478271484375f,0.1070000100806354215166038557072170078754425048828125f,0.3297000003114345378918415008229203522205352783203125f,0.530599989616870715281038428656756877899169921875f,0.4009999533891674960983664277591742575168609619140625f,-0.5111999743163588849625966759049333631992340087890625f,-0.047100007952749878281650808276026509702205657958984375f,-0.0283999893426898353254728135652840137481689453125f,-0.863900016552209937259476646431721746921539306640625f,-0.5253999736070635862006383831612765789031982421875f,-0.1430999805271626623692782231955789029598236083984375f,-0.7877999810993669971281860853196121752262115478515625f,-0.807899992047250048443629566463641822338104248046875f,-0.43279997811913517580251209437847137451171875f,-0.58629998831898000322127018080209381878376007080078125f,0.04039999968409535568980572861619293689727783203125f,0.100300032880902723064764359151013195514678955078125f,0.64579998685121520196616984321735799312591552734375f,0.5234000130355358493261519470252096652984619140625f,0.0217999961197377079002990285516716539859771728515625f,1.043899993231892420197937099146656692028045654296875f,-0.3799999858438969368990001385100185871124267578125f,1.59350004220008845123857099679298698902130126953125f,-0.034799999934434833193108715931884944438934326171875f,0.74360005434006504199118126052781008183956146240234375f,0.6969999970793718802042349125258624553680419921875f,-0.595600014948845402074084631749428808689117431640625f,-0.056800008040666316588840345502831041812896728515625f,-1.0933000323183836854923356440849602222442626953125f,0.30759999634474499696779048463213257491588592529296875f,-0.7899000519536441000667537082335911691188812255859375f,0.421299997520446556364959178608842194080352783203125f,0.25089999731481071876260102726519107818603515625f,-0.300999993845820323912221283535473048686981201171875f,-1.22229999963641144944404004490934312343597412109375f,-1.065999993398785594678201960050500929355621337890625f,-0.78169997798204438055336140678264200687408447265625f,-0.982599977904558219421460307785309851169586181640625f,-1.1957999789536000179168695467524230480194091796875f,-0.4188999917417763896310134441591799259185791015625f,1.05529999108314509470574193983338773250579833984375f,0.97949997889995554789521747807157225906848907470703125f,0.62299997468292733771022540167905390262603759765625f,0.054399987584352604841342326835729181766510009765625f,-0.245899988895654786347222398035228252410888671875f,-0.199300007593631534064115840010344982147216796875f,0.079099976384640058313379995524883270263671875f,-0.0423000136435032203507944359444081783294677734375f,-0.6717999907553195981790850055404007434844970703125f,-0.20109998210668589280203377711586654186248779296875f,-0.30499998081475521161110009416006505489349365234375f,-0.1082000009372829918419256500783376395702362060546875f,-0.075500021398067218569849501363933086395263671875f,0.6961000084817412503213063246221281588077545166015625f,-0.18209999867677684193267850787378847599029541015625f,-0.24770000140666947885392801254056394100189208984375f,0.129699994984269206810267860419116914272308349609375f,-0.3406999941945076937344083489733748137950897216796875f,-0.22419998438954369390785359428264200687408447265625f,-0.427599990347027780757116488530300557613372802734375f,-0.9857000432938340939159616027609445154666900634765625f,-0.49259998431205753721684459378593601286411285400390625f,0.18299999509751774251498090961831621825695037841796875f,0.15900000220537169326462390017695724964141845703125f,0.7620000046789645598011020410922355949878692626953125f,0.09059999259710327379480077070184051990509033203125f,0.6916999852091072487070277929888106882572174072265625f,0.2769999909698963591608844581060111522674560546875f,1.003899994833767461699380874051712453365325927734375f,-0.4495999827325345421513702603988349437713623046875f,-0.303899965552985928951557070831768214702606201171875f,0.329599979156256051027185094426386058330535888671875f,0.1815000163614752803908913847408257424831390380859375f,0.3515999813020231901106171790161170065402984619140625f,-0.5426000110968942635736311785876750946044921875f,-0.30759997820258178080621291883289813995361328125f,-0.99709999558478568104646910796873271465301513671875f,-0.66059997426867500536218358320184051990509033203125f,0.69040001711845355725216677456046454608440399169921875f,0.252399980521202438321637373519479297101497650146484375f,-0.78570001807361844470278811058960855007171630859375f,-0.0992000043570993028652083012275397777557373046875f,-1.77300000176578720356701523996889591217041015625f,0.5111999930918214030128865488222800195217132568359375f,-0.177800042529404345170718215740635059773921966552734375f,-0.49719997211098709843923870721482671797275543212890625f,1.4797000064209100411716235612402670085430145263671875f,-0.08279999644756319554517176584340631961822509765625f,1.8794000017590821727253569406457245349884033203125f,0.08820002280473726585796612198464572429656982421875f,0.09330002022981620513064626720733940601348876953125f,-0.077400016880035504840407156734727323055267333984375f,0.8082999683737757123935807612724602222442626953125f,0.5519999799430370668318346361047588288784027099609375f,0.82729999248385421850571219692938029766082763671875f,0.26649998804926866569076082669198513031005859375f,0.258299995344877242331449451739899814128875732421875f,0.24410000939071163639226824670913629233837127685546875f,0.0708000204905865349047644485835917294025421142578125f,-0.49769999246597296238547869506874121725559234619140625f,-0.4087999897718430641901932176551781594753265380859375f,0.5976999987989664031573511238093487918376922607421875f,0.733199969179183508316555162309668958187103271484375f,-0.3781000111863013213309159255004487931728363037109375f,-0.8507999951958655771022677072323858737945556640625f,-0.136699986624717662042627353002899326384067535400390625f,0.12459999184086922863201607469818554818630218505859375f,0.038699998903274634987070612623938359320163726806640625f,-0.27459999986886973744049100787378847599029541015625f,0.369000025823712274331001026439480483531951904296875f,0.485800036472081586680360487662255764007568359375f,0.066700001451373058802829518754151649773120880126953125f,0.6511999810777606700895603353274054825305938720703125f,0.282799968433379955712325681815855205059051513671875f,0.217799987283349150146705142105929553508758544921875f,-0.3120000011771917325376080043497495353221893310546875f,-0.71469998894929887001126189716160297393798828125f,-0.3217999955236912068556875965441577136516571044921875f,0.2110000140368939458568320333142764866352081298828125f,0.61379998857974982229279703460633754730224609375f,0.075599957877396928296320766094140708446502685546875f,-0.736999978899955721090009319595992565155029296875f,0.75980003435611731532617341144941747188568115234375f,-0.57199999216198893492446586606092751026153564453125f,0.531000033818185723788474206230603158473968505859375f,-0.383599992388486743521980315563268959522247314453125f,0.69710003347098836723461090514319948852062225341796875f,0.48059997642934348949239620196749456226825714111328125f,-0.002200031864643126056080291164107620716094970703125f,-0.165400000280141679098733220598660409450531005859375f,-0.63459999053701776627889330484322272241115570068359375f,0.19259998807460088077192494893097318708896636962890625f,-0.187000039678066887205432067275978624820709228515625f,-0.5133999757826330778698320500552654266357421875f,0.75479999756887483641776270815171301364898681640625f,0.242999987833202035769630811046226881444454193115234375f,-0.40200000369176269732207629203912802040576934814453125f,0.489999980032444160116256171022541821002960205078125f,-0.283000007614493398477861774154007434844970703125f,0.75579996911883373655882678576745092868804931640625f,-0.1978000330485401025981673228670842945575714111328125f,-0.416799974326789779421886805721442215144634246826171875f,-0.1621999727629124254946191285853274166584014892578125f,-0.78919997649192807909201974325696937739849090576171875f,1.22010004041492958748449382255785167217254638671875f,0.053000000238418554232566748396493494510650634765625f,1.297900025314092875561300388653762638568878173828125f,-0.29150000913441209693388600499019958078861236572265625f,0.16449999693036065906426301808096468448638916015625f,0.410300007399917010530998595640994608402252197265625f,0.69799998146295560985663541941903531551361083984375f,0.408999979555606518744070854154415428638458251953125f,0.395799999442696393003870980464853346347808837890625f,0.9964000053018333691312591327005065977573394775390625f,1.137799976480007302370722754858434200286865234375f,0.98199999007582672305716187111102044582366943359375f,0.5487999924540520879645555396564304828643798828125f,-0.3085000177621843864272932478343136608600616455078125f,-0.368799983103573481013626178537379018962383270263671875f,0.35259998811185366829334952853969298303127288818359375f,1.0889999549686912860835263927583582699298858642578125f,0.29909999650120744352221890949294902384281158447265625f,-0.32829997627139118065997536177746951580047607421875f,-0.486499978885054584765157414949499070644378662109375f,-0.51849999451637263092607099679298698902130126953125f,-0.014300009456276630004367689252831041812896728515625f,0.6809999723136428428915678523480892181396484375f,0.107099981316923997809453794616274535655975341796875f,-0.2011999848961831816041012643836438655853271484375f,0.17099999794363984761957908631302416324615478515625f,0.53169997977018379486935373279266059398651123046875f,1.1715999743729827997640313697047531604766845703125f,1.1205999723315240856891250587068498134613037109375f,-0.0554000180125233665506812030798755586147308349609375f,-1.0338000109314915420100078335963189601898193359375f,-0.65270000282227957910663462826050817966461181640625f,-0.1091000274211173515226391828036867082118988037109375f,-0.822399998253583230933827508124522864818572998046875f,0.235500010706485074507554600131697952747344970703125f,-0.8059999668002131301136614638380706310272216796875f,0.0342000150859351759180526642012409865856170654296875f,-0.81719999699592527075964198957080952823162078857421875f,-0.28519997272193364779013791121542453765869140625f,-0.35359998389482516500947895110584795475006103515625f,0.0600000455230478291213103148038499057292938232421875f,0.64419999422430951341311811120249330997467041015625f,0.0059999954849481174079528500442393124103546142578125f,-0.02800000046193595881760529664461500942707061767578125f,-0.33600000036507837730681558241485618054866790771484375f,-0.571799971681833607561884491587989032268524169921875f,0.6551000012621279733338042206014506518840789794921875f,0.136399994200468110960855483426712453365325927734375f,0.84619999111741750486004320919164456427097320556640625f,0.7683999775707721635598090870189480483531951904296875f,0.2093000067368151473345960766891948878765106201171875f,0.44300000366568514920118104782886803150177001953125f,-0.6166000099495054787013259556260891258716583251953125f,0.081199995625019116829435006366111338138580322265625f,-0.61850000546872563322864380097598768770694732666015625f,-0.6825999595761305105412475313642062246799468994140625f,-0.663499955177307310094647618825547397136688232421875f,-0.045199997830390969966174452565610408782958984375f,-0.370200026291608974560176648083142936229705810546875f,-0.14899998328089747445801549474708735942840576171875f,-0.09930002074390653632462999667041003704071044921875f,0.012599991986155689716042616055347025394439697265625f,-0.5627999830365182365454757018596865236759185791015625f,-0.20669998945593814898558093773317523300647735595703125f,1.28429999820590001746722919051535427570343017578125f,0.75400001434981778203336943988688290119171142578125f,-0.55809997411966350000511738471686840057373046875f,0.012300018113851507450817734934389591217041015625f,0.28900000488758070815720202517695724964141845703125f,0.5643999998778104210117589900619350373744964599609375f,0.6761999865353107708187962998636066913604736328125f,-0.23129999647736543266773878713138401508331298828125f,-0.7344999754726888596678691101260483264923095703125f,-0.51329999199211595595215840148739516735076904296875f,0.0592999898612500686567727825604379177093505859375f,-0.20609999626278874274021291057579219341278076171875f,-0.3453999965548515405089347041212022304534912109375f,0.10000000804662700915059758699499070644378662109375f,-0.3476000020444391935825478867627680301666259765625f,0.562999982431530821003207165631465613842010498046875f,0.35379997427463560910609885468147695064544677734375f,0.19079999370574984851600675028748810291290283203125f,0.037200000348687201068287322414107620716094970703125f,0.1035999842673540616289074023370631039142608642578125f,0.818199975270033075247511078487150371074676513671875f,0.51999999664723894898799017028068192303180694580078125f,-0.212800007697939719264468294568359851837158203125f,0.5353000145524742237057580496184527873992919921875f,0.5024999938905239105224609375f,-0.66149999259412251007006489089690148830413818359375f,0.084199990275501956915604750975035130977630615234375f,0.5789999749362471082037018277333118021488189697265625f,0.53809996707886487143923659459687769412994384765625f,-0.8017999631881718158865623991005122661590576171875f,-0.4823999938577419044349881005473434925079345703125f,0.59879996965527571006759899319149553775787353515625f,-1.0649000435903668293491364238434471189975738525390625f,-0.1416000008225439810161105924635194242000579833984375f,-0.430600011074543209677045751959667541086673736572265625f,-0.33499999836087201998680029646493494510650634765625f,0.305300008629262986659114176291041076183319091796875f,-0.278399987703561890839409898035228252410888671875f,0.546999991826713216624966662493534386157989501953125f,-0.7607999595820909011223420748137868940830230712890625f,0.3800999780163163421775607275776565074920654296875f,-0.65799997103214291627182319643907248973846435546875f,-0.7109999900460242105992847427842207252979278564453125f,-0.15239998856782932534770225174725055694580078125f,-0.73539995527863499802379010361619293689727783203125f,0.433999984622001733214347041212022304534912109375f,-0.3406000090628860110797404558979906141757965087890625f,0.40479999859333004597061744789243675768375396728515625f,0.14379997881948913818206392534193582832813262939453125f,-0.42639998153448122053532642894424498081207275390625f,-0.493200007680058671866163422237150371074676513671875f,0.30980000008344621420519615639932453632354736328125f,-0.5183000420972714294975958182476460933685302734375f,0.78759997820258131895343467476777732372283935546875f,0.6654999486431478761261359977652318775653839111328125f,0.51429999790787661595459212549030780792236328125f,0.8120000215172762381143911625258624553680419921875f,0.312400016283989057086500906734727323055267333984375f,-0.4798000118553635839901971849030815064907073974609375f,-0.14669999869465832897930113176698796451091766357421875f,-0.08819998927712457970073955948464572429656982421875f,0.307300023850798798008554513216949999332427978515625f,-0.121100007587671143483021296560764312744140625f,0.3144000204414130195118559640832245349884033203125f,-0.977899987614154753146067378111183643341064453125f,-0.53109999358057979890190836158581078052520751953125f,-0.281899993321299557891279619070701301097869873046875f,0.0495000158548355440046861986047588288784027099609375f,0.4661000120580194305830445955507457256317138671875f,-0.0404999861195685095793805885477922856807708740234375f,-0.35769998171851058454961957977502606809139251708984375f,-0.254500022206455123185975253363722003996372222900390625f,1.29819994889497802859068542602472007274627685546875f,1.016999975770711994726980265113525092601776123046875f,1.006499989688396379250434620189480483531951904296875f,0.4381000055611135035604775112005881965160369873046875f,-0.437100004786252771538102024351246654987335205078125f,-0.4019000183552503546735579220694489777088165283203125f,0.204599989587068709084860529401339590549468994140625f,-0.45330001941025266365414836400304920971393585205078125f,0.21620000911056980186231157858856022357940673828125f,0.057400002574920616638110004714690148830413818359375f,0.175499994128942571336438049911521375179290771484375f,-0.8488999991923569243823521901504136621952056884765625f,-0.6223000124886635120446953806094825267791748046875f,-0.72119999920129718162797871627844870090484619140625f,-0.6168000072211021755919091447140090167522430419921875f,-0.400999975293875099424667496350593864917755126953125f,-0.067000010751188110447174040018580853939056396484375f,-0.54219998492598531214525792165659368038177490234375f,-0.9043000331863761953599123444291763007640838623046875f,-0.0845999840736393338147536269389092922210693359375f,-1.537200026127696350641826938954181969165802001953125f,0.83599998632073369009276575525291264057159423828125f,-0.01760001824945245374465230270288884639739990234375f,0.55959998452067372110008136587566696107387542724609375f,0.46199998772889350195924862418905831873416900634765625f,0.8035999768912791552111229975707828998565673828125f,-0.019000038955361286951983856852166354656219482421875f,0.94919995831251180451459958931081928312778472900390625f,0.29749996887519980504066552384756505489349365234375f,-1.02839995477199597218032067758031189441680908203125f,0.1799000156879422274158741856808774173259735107421875f,-0.65599998065829279170202426030300557613372802734375f,0.18640001219362023476833201129920780658721923828125f,0.428799998861550957229837877093814313411712646484375f,1.05470002787858252446540063829161226749420166015625f,0.31579999415278425800579498172737658023834228515625f,1.27240002496391557684773943037725985050201416015625f,-0.256399995392560864360120831406675279140472412109375f,1.080500029817223595074437980656512081623077392578125f,0.45239998961091032469994388520717620849609375f,0.5047999502018092332633614205406047403812408447265625f,0.509100007453560987613627730752341449260711669921875f,0.4965999939680099117822464904747903347015380859375f,0.5296999984860419896648409121553413569927215576171875f,0.333299995046854036218064720742404460906982421875f,-0.463600012952089357565199634336750023066997528076171875f,-0.2103999977096915330321280634962022304534912109375f,0.37649999207258222355676480219699442386627197265625f,0.853699982511997301770634294371120631694793701171875f,0.7274000027239322374583707642159424722194671630859375f,-0.407099991226196333826692352886311709880828857421875f,-0.0078999770343302078146052735974080860614776611328125f,-0.108399989640712846039605210535228252410888671875f,0.3126000056207178801059853867627680301666259765625f,-0.41050000455975510504913472686894237995147705078125f,-0.173600015336275337318738820613361895084381103515625f,0.43800001752376527264232208835892379283905029296875f,-0.172900009667873444385577386128716170787811279296875f,0.37479998607635511831404073745943605899810791015625f,-0.4930000112652777044530694183777086436748504638671875f,-0.77689999957978705946715081154252402484416961669921875f,-1.33450001913309090895154440659098327159881591796875f,-0.9482000105857846250501097529195249080657958984375f,-1.3452000101983543345340876840054988861083984375f,-1.0823000094711776597478092298842966556549072265625f,-1.0573000123769045188737436546944081783294677734375f,0.10220000981092436376229670713655650615692138671875f,-0.15119998534321776872957343584857881069183349609375f,0.01640000582337375334418538841418921947479248046875f,0.422499999180435903411989784217439591884613037109375f,-0.79020001031011322112362904590554535388946533203125f,-0.120599993342161315723615189199335873126983642578125f,-0.38670000901371259782735023691202513873577117919921875f,-0.2221999949097632764960508211515843868255615234375f,1.304900023399293562675893554114736616611480712890625f,-0.3449999914318322424833240802399814128875732421875f,-0.269699976134299912455816183864953927695751190185546875f,-0.401799987924098989111598712042905390262603759765625f,-1.077199995431303847226445213891565799713134765625f,0.403599987620115285835709073580801486968994140625f,-0.9925000519677997790068957328912802040576934814453125f,0.13560000511407821477405377663671970367431640625f,-0.842699984195828477595568983815610408782958984375f,0.8773999775409697665651265197084285318851470947265625f,0.267500035017729231867633643560111522674560546875f,0.4065999789178373902132079820148646831512451171875f,1.1816000120356680014310768456198275089263916015625f,0.0288000005006789905337427626363933086395263671875f,1.15230003055632135300356821971945464611053466796875f,0.5787999909639356399537746256100945174694061279296875f,1.21949999219924176685481143067590892314910888671875f,-0.06159999426603324224060997948981821537017822265625f,1.9475000333040954902941166437813080847263336181640625f,0.20640000939965208459625500836409628391265869140625f,2.1773000238135455930432726745493710041046142578125f,0.0023999957203866273403036757372319698333740234375f,0.80519999671280384490046344581060111522674560546875f,0.365799993556737756961183549719862639904022216796875f,-0.04329996647387701624865030680666677653789520263671875f,-0.058499998986721035709734906049561686813831329345703125f,0.2170000036358832318228451185859739780426025390625f,-0.57969999595284438242970281862653791904449462890625f,-0.32890000030994404056627899990417063236236572265625f,-1.041099985310435460661437900853343307971954345703125f,-1.175499956578016647057438603951595723628997802734375f,-0.6905000079125167555815778541727922856807708740234375f,0.6563999716997148681230100919492542743682861328125f,0.924099986517429528021239093504846096038818359375f,0.8865999872624874544868589509860612452030181884765625f,-0.04320001177787780033412445845897309482097625732421875f,-0.53069999404549594856916883145458996295928955078125f,-0.15579998245537307610675270552746951580047607421875f,-0.552399973517656572852274621254764497280120849609375f,-0.2105000035911801514743046936928294599056243896484375f,0.463899993455409997977767488919198513031005859375f,0.49770001034736655043388964259065687656402587890625f,-0.00830000249743445550620890571735799312591552734375f,-0.467500020191073506481416188762523233890533447265625f,-0.0318000052094458141027644160203635692596435546875f,-0.87000001311302188611307428800500929355621337890625f,-0.79659999381899826431663313996978104114532470703125f,-0.46139998205602150704862651764415204524993896484375f,-0.26229998816251764992557582445442676544189453125f,0.206999989479780044376866499078460037708282470703125f,0.99539997822642334313059109263122081756591796875f,0.6547999903231858098706652526743710041046142578125f,0.58490000406503650509648650768212974071502685546875f,0.691900031840801066351787085295654833316802978515625f,0.502400003524869465110214150627143681049346923828125f,0.223399986511469084149439368047751486301422119140625f,1.524099994526803669714354327879846096038818359375f,0.1644000072538849277492545297718606889247894287109375f,1.395399998231231730727586182183586061000823974609375f,0.61559997426867507641645715921185910701751708984375f,0.55489999151080837691552005708217620849609375f,-0.6833999874055383116910888929851353168487548828125f,-0.290999954618512912674077597330324351787567138671875f,-0.197199990141391889864053155179135501384735107421875f,-0.2727000023007388751494772805017419159412384033203125f,-0.6745999652981762295667067519389092922210693359375f,-0.17349998632073404536413363530300557613372802734375f,-0.032999998301267652323076617904007434844970703125f,0.4023000187471510447068112625856883823871612548828125f,-0.63159998547434792470767206395976245403289794921875f,0.4070000477060682353425136170699261128902435302734375f,0.81559999870657851062105692108161747455596923828125f,0.8432999991819258855940688590635545551776885986328125f,0.26920000152587864050701682572253048419952392578125f,1.4076000154554841881093807387514971196651458740234375f,0.336799990308284780127223712042905390262603759765625f,-0.09910002943277362863483403998543508350849151611328125f,0.25060000064372989214689368964172899723052978515625f,-0.1287000190198426263776809719274751842021942138671875f,-0.649599987947940604726682067848742008209228515625f,-0.042899994766712357119331500143744051456451416015625f,-0.5132000047743314752324295113794505596160888671875f,-1.393100026683509184977083350531756877899169921875f,-0.9723999984025957221689395737485028803348541259765625f,-0.29699998188018827960377166164107620716094970703125f,0.51600002647936360489211438107304275035858154296875f,0.3132999909862876908306361656286753714084625244140625f,1.0788999816462396807992263347841799259185791015625f,1.05469996913075458877528944867663085460662841796875f,0.9803000029295685369135071596247144043445587158203125f,0.5762000064097343443592080802773125469684600830078125f,0.907499997466802543755193255492486059665679931640625f,1.352299984884262062934112691436894237995147705078125f,1.6332999916195867218249304642085917294025421142578125f,1.1439999987781046542067997506819665431976318359375f,0.30879999819099879232453531585633754730224609375f,-0.5051999873995782763813622295856475830078125f,0.49630003238916398089486392564140260219573974609375f,0.012599999213218637805766775272786617279052734375f,0.574900009876489814786282295244745910167694091796875f,0.295099989974498821965198658290319144725799560546875f,0.062799991679191702331763735855929553508758544921875f,0.244099993968009965072951672482304275035858154296875f,0.7205999998241663373477194909355603158473968505859375f,1.05819999583363522077661400544457137584686279296875f,1.100100007781386057814643208985216915607452392578125f,0.5165000199377534073619244736619293689727783203125f,-0.37030001261532330403269952512346208095550537109375f,0.080500000983476649452086348901502788066864013671875f,-0.3357000254124404836630901627358980476856231689453125f,0.323899988612532663267984389676712453365325927734375f,0.4743999939918517316783663773094303905963897705078125f,0.1068999780476092009706690078019164502620697021484375f,-0.86399999147653527842294352012686431407928466796875f,-0.53369998813718499963698604915407486259937286376953125f,-0.126399984365702044986079499722109176218509674072265625f,-1.494600020357967462558690385776571929454803466796875f,-0.15680001400113052767437693546526134014129638671875f,-0.8371000033892685987524373558699153363704681396484375f,-0.061399999378621561429980602042633108794689178466796875f,-0.59270000202134232036854655234492383897304534912109375f,-0.35679998360276243829503073357045650482177734375f,-1.7270999750956887996977684451849199831485748291015625f,-0.095599994385242581529382732696831226348876953125f,-1.28340005252361333276667210157029330730438232421875f,-0.78099997887015337738603193429298698902130126953125f,-1.1356000346556311964008045833907090127468109130859375f,0.0297999880135062511499199899844825267791748046875f,-0.866500015951693125515475912834517657756805419921875f,0.302599989378452338684155620285309851169586181640625f,-0.08760001955330387612974618605221621692180633544921875f,-0.166599998140334992768885058467276394367218017578125f,-0.61849999347329098764447508074226789176464080810546875f,-0.36419998498559014166175984428264200687408447265625f,-0.2404999537840482037864830999751575291156768798828125f,0.726399977809190744437728426419198513031005859375f,0.95280001123696589360179132199846208095550537109375f,0.02240000391602503526655709720216691493988037109375f,0.5964000144287944227272646458004601299762725830078125f,-0.83899998134374598190987626367132179439067840576171875f,0.28190007989704657376250906963832676410675048828125f,-0.3975999953389166563511025742627680301666259765625f,-0.7621000043392178735501829578424803912639617919921875f,-0.35330000331699851212619023499428294599056243896484375f,-0.2652999909341335893486757413484156131744384765625f,-0.313799993571639124212424576398916542530059814453125f,-0.385299988698959428035095697850920259952545166015625f,0.2249999911338089741974499702337197959423065185546875f,0.62229998473525061086775167495943605899810791015625f,0.343699977669119416390230981050990521907806396484375f,0.08640001387000051380482545937411487102508544921875f,0.286400015881657754590605691191740334033966064453125f,0.1621999923765660156504964106716215610504150390625f,0.4581000010907647634894601651467382907867431640625f,-0.076199995625019223410845370381139218807220458984375f,0.157600028717518048093637617057538591325283050537109375f,-0.499499985940754509528005655738525092601776123046875f,-0.6891000084221363675140992199885658919811248779296875f,-0.5882000074565407743421019404195249080657958984375f,-0.37080002343356621707926024100743234157562255859375f,0.8244999981597065374927524317172355949878692626953125f,0.10529999823570246331172484133276157081127166748046875f,-0.4338999924123287943444893244304694235324859619140625f,0.38120002036094657871245772184920497238636016845703125f,-0.141599990093708161253971411497332155704498291015625f,0.15079999869763816011669632644043304026126861572265625f,0.7811000154107805570902200997807085514068603515625f,0.30270000125765772480690429802052676677703857421875f,0.737600008898973147353217427735216915607452392578125f,-0.0746999665230514420244389839353971183300018310546875f,1.02760002663135541212113821529783308506011962890625f,0.529199997204541805473354543210007250308990478515625f,-0.4306000025436282907520535445655696094036102294921875f,0.3957999895334243234401583322323858737945556640625f,-0.21180001869499687217057726229541003704071044921875f,-0.0253999983429908926524376511224545538425445556640625f,-0.363599964076280091074266920259105972945690155029296875f,-0.115999994069337919455620067310519516468048095703125f,-0.94709999394565824726299752001068554818630218505859375f,-0.78359999939203195395975853898562490940093994140625f,-0.3331999910652629903751176243531517684459686279296875f,-0.6997999710261824102275340919732116162776947021484375f,-0.889600009293854743219753800076432526111602783203125f,-0.253600007736682453440835161018185317516326904296875f,-0.88610002699643353185621208467637188732624053955078125f,0.0795999993473290867740388421225361526012420654296875f,1.1269000464439395958748946213745512068271636962890625f,0.205600001537799670359163428656756877899169921875f,1.497100056008994872769335415796376764774322509765625f,0.00359999708235270166056807283894158899784088134765625f,-0.833000028848648099710771930404007434844970703125f,-0.370599991703033371237552273669280111789703369140625f,-1.2369000076264142773396770280669443309307098388671875f,0.06439999339580548820549665833823382854461669921875f,0.4467000339359048499687787625589407980442047119140625f,-0.310400006687640672708994316053576767444610595703125f,0.38670000893920697393468799418769776821136474609375f,-0.732199987906217319988400049624033272266387939453125f,0.40069998871833056863778210754389874637126922607421875f,0.1471999687582254967566086634178645908832550048828125f,0.76189997976124301803935168209136463701725006103515625f,0.412499984055757551448806452754070051014423370361328125f,-0.3769000126183030818793895377893932163715362548828125f,0.10130000273585337833281982966582290828227996826171875f,-0.0331000018358229564086059326655231416225433349609375f,-0.23590001027882101514165924527333118021488189697265625f,-0.114500000283122016497827644343487918376922607421875f,-0.1536000083327293541657354580820538103580474853515625f,-0.93640000969767545058175528538413345813751220703125f,-0.192900002291798156051072510308586061000823974609375f,-0.37110001235604261893286093254573643207550048828125f,-0.1732999818593266017074938645237125456333160400390625f,-1.243099953630566911755295222974382340908050537109375f,-0.67209999241828910498952609486877918243408203125f,0.673099974864721506406795015209354460239410400390625f,0.104899987822771212364614257239736616611480712890625f,0.15130000199079507439137159963138401508331298828125f,0.17929998844563976856392173431231640279293060302734375f,0.139999970681965735241902848429162986576557159423828125f,0.8824999947100875541394771062186919152736663818359375f,0.982400015874206555821501751779578626155853271484375f,0.45730000813007354310002483543939888477325439453125f,-0.3935000106096264715915822307579219341278076171875f,-0.51070001311898227669416883145458996295928955078125f,-0.7681999921083448867875631549395620822906494140625f,-0.379800005000829887791269356966949999332427978515625f,-0.15559999215006836692509750719182193279266357421875f,0.2383000082343815773100459409761242568492889404296875f,-0.2098999808937305200373657498857937753200531005859375f,-0.4507999794006349869590621892712078988552093505859375f,-0.79199999719113123308744661699165590107440948486328125f,-0.137999992638826451951672424911521375179290771484375f,0.294200028012693071133298872155137360095977783203125f,0.63319998868107774114832864142954349517822265625f,0.03759999880343667388871153889340348541736602783203125f,0.43899998849630350594708261269261129200458526611328125f,-1.0353000155210498878233238428947515785694122314453125f,0.60699997308850317523365447414107620716094970703125f,-1.8147000270590181170149435274652205407619476318359375f,-0.06679998717904123139987859758548438549041748046875f,-0.7632000063762072183948248493834398686885833740234375f,0.6051999674320225341972445676219649612903594970703125f,1.1165999924406408982946459218510426580905914306640625f,-0.15319999642968173958479383145458996295928955078125f,0.4506000062689186069775360010680742561817169189453125f,-0.185599994832277292999833662179298698902130126953125f,-0.62990000406503654506451539418776519596576690673828125f,-0.51659997034966986806381328278803266584873199462890625f,-0.58659997798651464240293762486544437706470489501953125f,-0.23420000383555861223072724897065199911594390869140625f,-0.2190999821960921689623091879184357821941375732421875f,0.569999997466802188483825375442393124103546142578125f,0.01330003233701038123371063193189911544322967529296875f,0.665999996304511565625716684735380113124847412109375f,-0.65640001401901260091875656144111417233943939208984375f,0.066600000375509171846033495967276394367218017578125f,-0.19890001219362052342631841383990831673145294189453125f,0.29110000781118861024054922381765209138393402099609375f,-0.119499968878925277859792686285800300538539886474609375f,-0.1388999874949454760297840039129368960857391357421875f,-0.97260002402365153262309149795328266918659210205078125f,-0.178600008630752482563508465318591333925724029541015625f,0.4543000046133993663488581660203635692596435546875f,-0.262700003492831957174757917528040707111358642578125f,0.13519998918771758411594419158063828945159912109375f,-0.12590000938475132130633937777020037174224853515625f,0.1135999971196055557953741299570538103580474853515625f,-0.68709999744743133209823326978948898613452911376953125f,-1.29799997356534024817165118292905390262603759765625f,-0.336499976947903345347867798409424722194671630859375f,0.69379998932480813067513736314140260219573974609375f,0.167699973765015553084367638803087174892425537109375f,0.339799983617663503565609062206931412220001220703125f,0.2495999776661397273613829383975826203823089599609375f,0.34869997163414989671537114190869033336639404296875f,0.29969996264874954050583255593664944171905517578125f,0.852999961495399805500028378446586430072784423828125f,0.9504999790787700408856153444503434002399444580078125f,0.5445999796032907624976360239088535308837890625f,0.360199997904896651590433975798077881336212158203125f,-0.3231999880477787190358185398508794605731964111328125f,-0.30010000014454119021678479839465580880641937255859375f,0.6967999855399131803324053180404007434844970703125f,0.16610002397894874093253747560083866119384765625f,-0.51249998137354868532611362752504646778106689453125f,-1.0610999756246803915615828373120166361331939697265625f,-1.07260001418888517576988306245766580104827880859375f,-0.36159999776780582525503859869786538183689117431640625f,-0.2754999956190584686765987498802132904529571533203125f,0.3204000071346755351697765945573337376117706298828125f,0.550900014749168942529422565712593495845794677734375f,0.571599976235628304266356281004846096038818359375f,0.63909996979087591029156101285479962825775146484375f,-0.179799983245134686882238383986987173557281494140625f,0.5938000013947484223564288186025805771350860595703125f,-0.231799992245435737547865073793218471109867095947265625f,0.86060000369846767487302940935478545725345611572265625f,0.240599970990420075622751028276979923248291015625f,0.517599987180531240227310263435356318950653076171875f,0.08099999824166292938798505929298698902130126953125f,-0.37320000734478231407820203457958996295928955078125f,-0.95519997667074185709878975103492848575115203857421875f,1.32330007305443331233618664555251598358154296875f,-0.1331999901711942158044621464796364307403564453125f,0.4925000250712034954858609125949442386627197265625f,-0.118199999111890718950235168449580669403076171875f,-1.6064000003471965438706092754728160798549652099609375f,-0.346799978089332938679945073090493679046630859375f,-0.33680000129789144924785659895860590040683746337890625f,0.088599996560812022128317266833619214594364166259765625f,0.547399959287047099820711082429625093936920166015625f,-0.916599973851442317851478946977294981479644775390625f,0.79389998067766376355081092697218991816043853759765625f,0.171199994581937797732962280861102044582366943359375f,1.25689995819181188352331446367315948009490966796875f,-0.0481999875634908647725751507095992565155029296875f,-0.19740001211166369188276803470216691493988037109375f,-0.223700000691413602993407039321027696132659912109375f,-0.4738000046730039827025393606163561344146728515625f,0.4644000026345249665382652892731130123138427734375f,0.88270001258254016107684947201050817966461181640625f,0.6334000107258557221712180762551724910736083984375f,-0.1028000044941901336414957768283784389495849609375f,-0.322500010505318623899029262247495353221893310546875f,-0.4701000081539152830600869492627680301666259765625f,-0.53049998489022254233304920489899814128875732421875f,-1.044999970346689455169553184532560408115386962890625f,-0.2306999887555838402164454237208701670169830322265625f,-0.13830001069307318317669341922737658023834228515625f,0.153299998548626970062969121499918401241302490234375f,-0.1219999993517995040548385077272541821002960205078125f,-0.11559999509304763609662813905742950737476348876953125f,0.02310001085102542095484068340738303959369659423828125f,-0.295499979823827751967968424651189707219600677490234375f,-0.92399997270107281366335882921703159809112548828125f,-0.8221000089585783765500082154176197946071624755859375f,-0.51490000361800181138960397220216691493988037109375f,-0.360199996340274442019335765508003532886505126953125f,-0.1329000095188614949393013375811278820037841796875f,-0.10430000379383574937719458830542862415313720703125f,0.12510000472664817294798922375775873661041259765625f,0.829500023230910432658902209368534386157989501953125f,0.39409999665021899772909819148480892181396484375f,-0.1931999851047994098962590214796364307403564453125f,0.01559999699294589703413294046185910701751708984375f,-0.76179998241066915198871356551535427570343017578125f,-0.1823999880835407427781547085032798349857330322265625f,-0.8211999595642094362801799434237182140350341796875f,-0.082499971091747337226252057007513940334320068359375f,-0.57259999057054500326557899825274944305419921875f,-0.59249999850988377403382401098497211933135986328125f,0.0558000016033648460478389097261242568492889404296875f,-0.344100025521219154978069809658336453139781951904296875f,-0.47359999089837057084650950855575501918792724609375f,0.3137000138416887740078209390048868954181671142578125f,0.11819998659491570691670858650468289852142333984375f,-0.953700019355118389086101160501129925251007080078125f,-0.06600000718235943253375808126293122768402099609375f,-0.56929999746084203327001205252599902451038360595703125f,-0.135600006604194300852128662882023490965366363525390625f,0.58830002354532506814166481490246951580047607421875f,0.1891999872207643651478292667889036238193511962890625f,-0.11209999089091959856290259267552755773067474365234375f,0.64579998878836608611209157970733940601348876953125f,-0.1095000024065375665571764329797588288784027099609375f,-0.19819999449253078438459851895458996295928955078125f,1.155600032159686652022401176509447395801544189453125f,0.272400020009278431842858481104485690593719482421875f,-0.41920005438774932127188321828725747764110565185546875f,-0.07640000075697894743598226341418921947479248046875f,-1.0460000218972564933750390991917811334133148193359375f,0.4445999915242193623043931438587605953216552734375f,-0.208500025883317174901776525075547397136688232421875f,1.02730000133141885498844203539192676544189453125f,0.26270000751614563938574065105058252811431884765625f,0.22509999228417854055805946700274944305419921875f,0.53469999669790269081204314716160297393798828125f,0.066999984115362121173120613093487918376922607421875f,0.0673999893128873761583008672459982335567474365234375f,-0.99890002176761616414779609840479679405689239501953125f,-1.427900002068280826961199636571109294891357421875f,-0.76110001183450226580617936633643694221973419189453125f,0.2903000026315449844105387455783784389495849609375f,-0.0680999921500682336983345521730370819568634033203125f,-0.701199994432926132503780536353588104248046875f,-0.01729998823702327825913016567938029766082763671875f,0.94410000231266000270125005044974386692047119140625f,0.62190001023411733971357762129628099501132965087890625f,0.6913000062003729073722979592275805771350860595703125f,0.51110000766217689172066229730262421071529388427734375f,-0.0918999923527241691090239328332245349884033203125f,0.45640001260340223865341613418422639369964599609375f,0.23289999104142200536671225563623011112213134765625f,-0.66760000420510767771276050552842207252979278564453125f,-0.5285999762207271857050727703608572483062744140625f,-0.5357999935567381299961198237724602222442626953125f,-0.10899999743700039545046820421703159809112548828125f,0.080300014030933564157521686865948140621185302734375f,0.21919998736977586162311126827262341976165771484375f,-0.046600002834200626722349625197239220142364501953125f,0.324999995343387138024837668126565404236316680908203125f,0.0107999966859816665110116673531592823565006256103515625f,-0.543299983572959721556117074214853346347808837890625f,-0.05880001555085179187898347663576714694499969482421875f,1.360199969816207943296149096568115055561065673828125f,-1.64999994128942528703873904305510222911834716796875f,2.23409999344646958974180961376987397670745849609375f,-1.4021999756872656206496685626916587352752685546875f,-0.1518000058054926881823121220804750919342041015625f,0.768800000128150085032530114403925836086273193359375f,-1.61150000846385932362636594916693866252899169921875f,1.2053999856770036824826775045949034392833709716796875f,-1.3136000712156299385924285161308944225311279296875f,0.45820003591775881890413302244269289076328277587890625f,0.6459000208586458402493235553265549242496490478515625f,-0.596000011131167628519733625580556690692901611328125f,0.6437000285565854795777340768836438655853271484375f,0.396399992114305543822183608426712453365325927734375f,-0.590200014817714890824618123588152229785919189453125f,0.498400006702542253123056070762686431407928466796875f,-0.603900022885203835443235220736823976039886474609375f,0.4332000023901463325870508924708701670169830322265625f,0.932799958896636471905594589770771563053131103515625f,-1.270799952578544722570086378254927694797515869140625f,0.40249999225139543312934620189480483531951904296875f,-0.091199996891617729488643817603588104248046875f,0.3097000269472598432685117586515843868255615234375f,0.01979998100996027687870082445442676544189453125f,0.103000034660102102179735084064304828643798828125f,0.1814000056743616529075779908453114330768585205078125f,0.7487999811291690122061481815762817859649658203125f,-1.0695999880224460465427682720473967492580413818359375f,-1.1175871339474952037562616169452667236328125e-10f,-0.008500008970499239868701124578365124762058258056640625f,0.15259999712705607510798699877341277897357940673828125f,-0.42239999668896199835899096797220408916473388671875f,-0.006799993102252432919385682907886803150177001953125f,0.011700003910064982193262039800174534320831298828125f,-0.15820000462532046725527834496460855007171630859375f,-1.3278000043451783085401984862983226776123046875f,-0.3687999931991099433759018211276270449161529541015625f,-1.1136000263631335993608217904693447053432464599609375f,0.01419999854564668595457987976260483264923095703125f,-0.001200024086237316822689535911194980144500732421875f,-0.12450000590831045155226775023038499057292938232421875f,-0.55990002984404607655477548178168945014476776123046875f,-0.0614000003844499542271506697943550534546375274658203125f,0.4798000131964681447271914294105954468250274658203125f,0.24210000106096263294830350787378847599029541015625f,0.706600018703937404751513895462267100811004638671875f,0.09439999928176401322588162656757049262523651123046875f,-0.1356000027298929211383438087068498134613037109375f,-0.19100000315904619441198519780300557613372802734375f,-0.58320000075101841030544846944394521415233612060546875f,0.1580000064224005296154018651577644050121307373046875f,0.8344000201433898755709606120944954454898834228515625f,0.1688999988943338248503778231679461896419525146484375f,-0.60840000215768819114003918002708815038204193115234375f,-0.319199999737739492644550409750081598758697509765625f,-1.3958000268608332650188685875036753714084625244140625f,-0.6397999826490881059015691789682023227214813232421875f,0.6789999818652869834068042109720408916473388671875f,-0.1275999906450511200972641745465807616710662841796875f,0.240600005561113217122937157910200767219066619873046875f,-1.161800003272294379286222465452738106250762939453125f,1.10879999074041801776502325083129107952117919921875f,-0.8651000427246098301026222543441690504550933837890625f,-0.049399992352723909316836170546594075858592987060546875f,-1.002299981158971231565146808861754834651947021484375f,1.60439996478557578285517593030817806720733642578125f,-0.37620001916885303927529093925841152667999267578125f,-0.6647999698340896213011319559882394969463348388671875f,-0.2081000176310539018231793306767940521240234375f,0.475800014418363392820765511714853346347808837890625f,-0.446000016495585516196342723560519516468048095703125f,0.15160000037550924734119917047792114317417144775390625f,0.622200030896066902386110086808912456035614013671875f,-0.203200023177265709506400526151992380619049072265625f,1.1946000031471248892245284878299571573734283447265625f,-1.0261999880254266681589569998322986066341400146484375f,0.262299997252225747246257014921866357326507568359375f,0.4360000033080577708233249722979962825775146484375f,-0.230799968969821467368319645174778997898101806640625f,0.39059998216629010503453400815487839281558990478515625f,0.7825000193715094809476795489899814128875732421875f,-0.7253999963313340071380252993549220263957977294921875f,-0.448299996015429247364636466954834759235382080078125f,0.78739998767375951604918782322783954441547393798828125f,-1.748899995020032793746622701291926205158233642578125f,-0.7969999837428327627009139177971519529819488525390625f,-0.2212999923050402717450424461276270449161529541015625f,-0.53659999858736906475087380385957658290863037109375f,-0.16059999803602698076332444543368183076381683349609375f,-0.218599974656104478754059528000652790069580078125f,0.14900000779330735412031572195701301097869873046875f,0.0253999978959564831626494196825660765171051025390625f,-0.347900006724894073073528488748706877231597900390625f,-0.405100046598911855966207440360449254512786865234375f,-0.054100002871453778396926281857304275035858154296875f,-0.29470003670752087376882855096482671797275543212890625f,-0.114000004403293164756405531079508364200592041015625f,0.62609998985528891868312939550378359854221343994140625f,0.2313999973297118650794601535380934365093708038330078125f,0.08620001015365108543164751608856022357940673828125f,0.000900002343952710504026981652714312076568603515625f,0.5935000101625920176928730143117718398571014404296875f,0.208100004033744345743883741306490264832973480224609375f,0.62430001653432842800839353003539144992828369140625f,0.0479999980777501555895270257678930647671222686767578125f,-0.1437999978184703042671799266827292740345001220703125f,0.11060000068098307934860713430680334568023681640625f,0.26309998686015578783781165839172899723052978515625f,0.3462999992340802801749077843851409852504730224609375f,1.581600021088122698387223863392136991024017333984375f,0.169799998742341973212433003936894237995147705078125f,0.01779999547898747369600869205896742641925811767578125f,-0.1896000049166381706822193109474028460681438446044921875f,0.464500006113201246993327231393777765333652496337890625f,-1.389499987095593969144147195038385689258575439453125f,1.2875999629288912284863499735365621745586395263671875f,-0.3694000220060349448658598703332245349884033203125f,-0.07300000038743004182606455287896096706390380859375f,0.49889999986290956002221719245426356792449951171875f,-0.63119999860525144441680822637863457202911376953125f,-0.02610005095004996888974346802569925785064697265625f,0.0536000436484809217319025265169329941272735595703125f,0.78319996856451012945399270392954349517822265625f,-0.821399977958202409666910170926712453365325927734375f,0.638899956351518749642082184436731040477752685546875f,-0.68579995861351517039139480402809567749500274658203125f,0.646799987551569532939765849732793867588043212890625f,-0.51519998531341559822038789206999354064464569091796875f,-0.527099984595178927548886349541135132312774658203125f,0.8875999956369395871291771982214413583278656005859375f,0.0886000186145305956841866645845584571361541748046875f,-0.36680001444816578981544807902537286281585693359375f,1.039900012037157761568550995434634387493133544921875f,-0.250799991321563719992582264239899814128875732421875f,0.845900030842423600319079923792742192745208740234375f,0.109000001087784514908207711414434015750885009765625f,0.1566000046968465397867475985549390316009521484375f,-0.07239997225105820888302332605235278606414794921875f,0.4073999829053871479800363886170089244842529296875f,-0.783800000202655500203263727598823606967926025390625f,-0.703200024797767664352932115434668958187103271484375f,0.65840000595748426892583893277333118021488189697265625f,0.40619998517557898143337524743401445448398590087890625f,1.2736000556811699624404354835860431194305419921875f,-0.0958999995872378274697211963939480483531951904296875f,-1.219900025299191614891469725989736616611480712890625f,-0.3475000073015690471578409415087662637233734130859375f,-1.857400040274858721289774621254764497280120849609375f,-0.2482999991074203993690616698586381971836090087890625f,-1.69850002782046782812130913953296840190887451171875f,-0.306700003165006673810921711265109479427337646484375f,-0.003400002157687875836700186482630670070648193359375f,0.2722000074267387770277082381653599441051483154296875f,1.0208000267863273080592989572323858737945556640625f,0.29159999299943439865501204621978104114532470703125f,0.9308000051051370515864391563809476792812347412109375f,0.076000000625848773694315241300500929355621337890625f,-0.17280003265738486106783966533839702606201171875f,-0.1596000049352646055211835118825547397136688232421875f,-0.65260003043115144549801698303781449794769287109375f,-0.450699999111890736713803562452085316181182861328125f,-1.4616000165432689872346827542060054838657379150390625f,0.01319999989420173047705020508146844804286956787109375f,-0.41490002984404572572429970023222267627716064453125f,-0.1626000109106302549122347045340575277805328369140625f,0.053399957305192202738908235915005207061767578125f,0.036599999406933758638871267976355738937854766845703125f,0.627200010779500161106625455431640148162841796875f,-0.07539999689012766292961487124557606875896453857421875f,-0.605099974998831502404073034995235502719879150390625f,-0.0147999949797987806476839978131465613842010498046875f,0.07390003704130709838437951475498266518115997314453125f,0.6109999869540334582751484049367718398571014404296875f,-0.5879999875724315483438431328977458178997039794921875f,-0.775100016275048542269132667570374906063079833984375f,1.470800001379847454785476656979881227016448974609375f,-1.2917000104665756765598416677676141262054443359375f,0.861999981135129988984999727108515799045562744140625f,-1.4472999976992610982051701284945011138916015625f,1.47899997217953238504151158849708735942840576171875f,0.7091000070810320465852782945148646831512451171875f,-1.184999994337558870682869383017532527446746826171875f,0.934100020119547735930609633214771747589111328125f,-0.60759997924566278015845455229282379150390625f,-0.008799967643618789026049853418953716754913330078125f,0.591399973636865805559637010446749627590179443359375f,-0.28110005378127045361225100350566208362579345703125f,-0.520999949663877970351677504368126392364501953125f,-0.305700044262408709982992149889469146728515625f,0.38300001223385304882640411960892379283905029296875f,-0.376599971243739783943738075322471559047698974609375f,0.57279996128082333317621532842167653143405914306640625f,-0.6093000390723346537669158351491205394268035888671875f,-0.3073999580204489490142805152572691440582275390625f,-0.9133000340878965062074712477624416351318359375f,0.233000001281499891092607867904007434844970703125f,0.18399996986985200919662020169198513031005859375f,-0.251799990233779080739395794807933270931243896484375f,0.521300032612681096821916071348823606967926025390625f,0.491999970629811489430949222878552973270416259765625f,1.252200041494518689688675294746644794940948486328125f,-0.899500018686056446171050993143580853939056396484375f,0.226500008873641522910702406079508364200592041015625f,1.34730002884268795781963490298949182033538818359375f,0.0710999946609138788744530756957828998565673828125f,0.755500009775161185388014928321354091167449951171875f,0.184600003147125268920802909633493982255458831787109375f,0.1446000070959330141562304561375640332698822021484375f,-0.02619999983459708747801641948171891272068023681640625f,0.08070002451837066104189943871460855007171630859375f,0.233700005906820329482087572614545933902263641357421875f,-0.14589998755454980372547879596822895109653472900390625f,-0.24550000254809856148341395964962430298328399658203125f,0.33399997642636269912230773115879856050014495849609375f,0.305699993058293983949624816887080669403076171875f,0.057099972655623221218235130436369217932224273681640625f,-0.454100004138052515401113851112313568592071533203125f,-1.14130002415627274103826493956148624420166015625f,0.1554999963827430609075719303291407413780689239501953125f,-0.02280000758618161427193626877851784229278564453125f,-0.005600004257261759210706486555864103138446807861328125f,1.070500043973326764756848206161521375179290771484375f,0.434700013312697475242885047919116914272308349609375f,1.632200061629713871980129624716937541961669921875f,0.0798000032313168350395216066317516379058361053466796875f,0.376300000984966853678770348778925836086273193359375f,-0.149800001052021958258819722686894237995147705078125f,-1.21300002110004445654567462042905390262603759765625f,-0.1574999999627471058971650563762523233890533447265625f,-0.24149999125301813140964668491506017744541168212890625f,-0.5495999897360803743140422739088535308837890625f,0.187199967938661782085318918689154088497161865234375f,1.0389999780654903815957368351519107818603515625f,-1.13439996582865720853305901982821524143218994140625f,1.9567000320732592655303960782475769519805908203125f,-0.899400001442432373011115487315692007541656494140625f,1.1436000227868561296418192796409130096435546875f,-0.4385999902278185924586750843445770442485809326171875f,0.29530006290674204461055296633276157081127166748046875f,0.33499995738267929201725792154320515692234039306640625f,-0.4534000046163797481568735747714526951313018798828125f,0.19379999834001082614776123591582290828227996826171875f,0.201799992226808999618015150190331041812896728515625f,-0.72739997500777253680581679873284883797168731689453125f,-0.13110004489645421532628688510158099234104156494140625f,0.68340000469088568735287481104023754596710205078125f,-0.135999999731779952494292729170410893857479095458984375f,0.095399993723631115738470498399692587554454803466796875f,-0.70400000842660681588114357509766705334186553955078125f,0.311200003597140284483657524106092751026153564453125f,-0.26929994128346468329482377157546579837799072265625f,-0.07900006501376566347971674986183643341064453125f,0.7798000005304823734064711970859207212924957275390625f,-0.939999965876341381232350613572634756565093994140625f,1.093699986200034945937886732281185686588287353515625f,-0.5239999749362471703761912067420780658721923828125f,0.072500025406479817746685512247495353221893310546875f,1.03119998042583471686839402536861598491668701171875f,0.408600031429529497728481146623380482196807861328125f,-0.2058000176966192640293229487724602222442626953125f,-0.21400000820308928695112626883201301097869873046875f,-0.032100030118227262931895893416367471218109130859375f,0.034600002849102029500727439881302416324615478515625f,-0.061199995923042482814935283386148512363433837890625f,-0.1286000037878751722786319078295491635799407958984375f,0.633799987983703516647437936626374721527099609375f,0.29560000272989273017998357317992486059665679931640625f,1.3848000482141973321859040879644453525543212890625f,0.4239000075742602735573427708004601299762725830078125f,0.6883000409424309129491348357987590134143829345703125f,-0.1517999971628188615735410849083564244210720062255859375f,0.57769997756481183959209602107875980436801910400390625f,0.1910999957039951624437890131957828998565673828125f,0.347300027576088243819185663596726953983306884765625f,0.011400004966557020225081942044198513031005859375f,-0.86329998454153500375696239643730223178863525390625f,0.041799997870624050477772470912896096706390380859375f,1.0720999984532593174435533001087605953216552734375f,0.4039000015765428219793875541654415428638458251953125f,0.7512000306427477624993116478435695171356201171875f,-0.10179999760985369849919379703351296484470367431640625f,0.47539999484121864270491641946136951446533203125f,-0.027099996069073650772907058126293122768402099609375f,0.84480001289844519618554841144941747188568115234375f,0.34859999991357326631913338133017532527446746826171875f,0.16199997763335627354308599024079740047454833984375f,-0.397300010216236232363229419206618331372737884521484375f,-1.681800066900253742829818293103016912937164306640625f,0.3675999834924927878176958984113298356533050537109375f,-0.596999997004866056471428237273357808589935302734375f,1.47799999815225646671024151146411895751953125f,-0.971599974149465861472663164022378623485565185546875f,1.362800004643201390308604459278285503387451171875f,-0.831199996966123233477219400811009109020233154296875f,1.565400013020635139326941498438827693462371826171875f,-1.3443999749183657144158132723532617092132568359375f,0.9105000380873686793847809894941747188568115234375f,-0.19019999976754231596487443312071263790130615234375f,-0.19949999521672701074948008681531064212322235107421875f,0.33359998784363276147502119783894158899784088134765625f,0.9673999626398088746981329677510075271129608154296875f,-1.454599944734573657711962368921376764774322509765625f,0.68510003020763310388474565115757286548614501953125f,0.683399955442548279194170390837825834751129150390625f,-0.9512999811291695806403367896564304828643798828125f,0.84619998929202555615347591810859739780426025390625f,-0.04520000841021509785377929802052676677703857421875f,-0.4333999773472552163866566843353211879730224609375f,-0.196300013911724136050906963646411895751953125f,0.06599999287724500618423917330801486968994140625f,0.5791999681472788097380544058978557586669921875f,-0.87559996063411293931721957051195204257965087890625f,1.27259999399781253970331817981787025928497314453125f,-1.30719996838569674224572736420668661594390869140625f,-0.2340000461637981032936295378021895885467529296875f,1.06660001654326919151571928523480892181396484375f,-0.60610008239150037212539245956577360630035400390625f,0.610500010818241900523162257741205394268035888671875f,0.005699998776614645379368084832094609737396240234375f,1.055900023987889735366252352832816541194915771484375f,0.2273000129729509932730024956981651484966278076171875f,-0.1295999313235274286171261337585747241973876953125f,-0.04839999940097328146038080376456491649150848388671875f,0.37119999778568768533659749664366245269775390625f,-0.00380000009089709323006900376640260219573974609375f,0.977100018048286500516041996888816356658935546875f,0.163099998892843711661981842553359456360340118408203125f,-0.521899996450543302017877067555673420429229736328125f,-0.337599995227158000687950334395281970500946044921875f,-1.056500023663043297261765474104322493076324462890625f,-0.1953999980449676598937003291212022304534912109375f,-1.20120001216530791765535468584857881069183349609375f,-0.3174000001907348877949743837234564125537872314453125f,-1.47100002349913072663412094698287546634674072265625f,-0.248000000573694645567002226016484200954437255859375f,0.208299969714879740223523185704834759235382080078125f,0.26059999378919596058068464117241092026233673095703125f,0.848399996793269384198765692417509853839874267578125f,0.12080000205039975735843427173676900565624237060546875f,1.2502000223428011427273531808168627321720123291015625f,0.19220000008791682244435605753096751868724822998046875f,0.007699996414780141407163682742975652217864990234375f,-0.0339000039607287029230775488031213171780109405517578125f,0.9204000166714185748872978365398012101650238037109375f,0.42739999948292961562401615083217620849609375f,0.623100040355324846785833869944326579570770263671875f,-0.423599985831976777461704841698519885540008544921875f,0.93259996762275731985170068583101965487003326416015625f,1.244200001376867703584139235317707061767578125f,-1.637599995860457458007886089035309851169586181640625f,-0.0681999572396279063468682579696178436279296875f,0.73499996930360822489092242904007434844970703125f,-0.4656000158429147717242813087068498134613037109375f,-0.56920000234544243511436434346251189708709716796875f,0.94020001153945909067033426254056394100189208984375f,-0.769799992334842642094372422434389591217041015625f,0.04890001148581557544048337149433791637420654296875f,0.67459998534023757343902616412378847599029541015625f,0.2867999700427059650564842741005122661590576171875f,-0.339399976408481851564147291355766355991363525390625f,0.38569996513724280617907425039447844028472900390625f,0.09680000535845767473119849455542862415313720703125f,-0.528700036752223212488388526253402233123779296875f,0.04360002882182545391742678475566208362579345703125f,-0.2211000071406363787218651850707828998565673828125f,-0.174399980133772114498924565850757062435150146484375f,-0.2132000143110754919462124234996736049652099609375f,0.083999996691942246940243421704508364200592041015625f,0.957699985089897953827176024788059294223785400390625f,-0.9363999763935806353032376136980019509792327880859375f,-0.0017000276774172817795260925777256488800048828125f,0.184200012925267220253999767010100185871124267578125f,-0.283200005593897241595868763397447764873504638671875f,-0.1574000179976220437083611614070832729339599609375f,-0.4483000065207480844264864572323858737945556640625f,0.91280002997517595275667190435342490673065185546875f,0.07940000710487371549106683232821524143218994140625f,-0.59240000451207086573646165561513043940067291259765625f,-0.346700004170835018868501720135100185871124267578125f,-0.808700028035044926610908078146167099475860595703125f,-0.173899998521804821649539007921703159809112548828125f,-0.9728000169366595173414680175483226776123046875f,-0.16850000226497652278112582280300557613372802734375f,-0.68340000767111774848672212101519107818603515625f,-0.136600001493096367966018078732304275035858154296875f,-0.0583999965697525880159446387551724910736083984375f,0.0954000009879469867968282414949499070644378662109375f,0.81290003775656227702484102337621152400970458984375f,0.433800006796419612697945922263897955417633056640625f,0.989700066956877844148721123929135501384735107421875f,0.161599998699128610812891793102608062326908111572265625f,1.72719998745918257299081233213655650615692138671875f,0.1749999973177909229349324959912337362766265869140625f,0.9416000166177742602968692153808660805225372314453125f,0.1682000032842159154000682974583469331264495849609375f,-0.0721999794870613609276688293903134763240814208984375f,-0.0040000008270144604693996370770037174224853515625f,0.2759000013381240901111368657439015805721282958984375f,0.234700001093745225233533346909098327159881591796875f,0.50720002292394639198391814716160297393798828125f,0.0526000059187412827288454764129710383713245391845703125f,0.046400017446279750998883173451758921146392822265625f,0.247800000731646985485667755710892379283905029296875f,-0.302099995398521503631172890891321003437042236328125f,1.129700022029876294027417316101491451263427734375f,-1.12440003198981219867391700972802937030792236328125f,0.211499982237816741559299771324731409549713134765625f,0.15300003793835592613703511233325116336345672607421875f,0.10339997742175999206892811344005167484283447265625f,-0.067400012931227326618000006419606506824493408203125f,-0.34030004800558089073092560283839702606201171875f,-0.234399971863627598622770165093243122100830078125f,-1.75120003585815453561735921539366245269775390625f,1.23899999192357057609115145169198513031005859375f,-1.2670000040829183518553691101260483264923095703125f,0.61699998433887959237154063885100185871124267578125f,-0.208899963727593895868039908236823976039886474609375f,-0.06540003581941089549900425481610000133514404296875f,-0.80579996096044848030715002096258103847503662109375f,0.78059996231049344483921004211879335343837738037109375f,1.375299961150437955126335509703494608402252197265625f,-1.711199939149618653999596062931232154369354248046875f,2.28219999878406465398938962607644498348236083984375f,-1.3001999930620191658903195275343023240566253662109375f,0.17940001217126866350781710934825241565704345703125f,0.7086000055760142135596879597869701683521270751953125f,-0.01339998636245720575743689551018178462982177734375f,0.1361999995738265578637538055772893130779266357421875f,0.7159000321835282676374845323152840137481689453125f,-0.160800014269351976281541283242404460906982421875f,0.390500039651989983013891105656512081623077392578125f,0.1661999916762113560508851151098497211933135986328125f,-0.24450002717971841548205702565610408782958984375f,-0.099000022172928137109693125239573419094085693359375f,0.0306999989256262484360604503308422863483428955078125f,1.11170003207325951422035359428264200687408447265625f,0.120500002585351484452047543527442030608654022216796875f,-0.3597000003486872099500715194153599441051483154296875f,-0.132100000204145917592768455506302416324615478515625f,-0.06600001690536727583236142891109921038150787353515625f,0.1143000006273389157485098621691577136516571044921875f,0.2995000076591967097527913210797123610973358154296875f,-0.0388000013947487065735231226426549255847930908203125f,-0.0803999998331068610468719271011650562286376953125f,-0.06699999745190143851658604035037569701671600341796875f,0.03420001985430776159091692534275352954864501953125f,0.11429999898821119330705187167041003704071044921875f,1.053100020611286158356278974679298698902130126953125f,0.3119000081852079109268061074544675648212432861328125f,1.175100037062168478740886712330393493175506591796875f,0.2248000000834464717769378694356419146060943603515625f,-0.5212000035226347360861609558924101293087005615234375f,-0.06960000195503235342897596638067625463008880615234375f,-1.09100005128979749002837706939317286014556884765625f,-0.2992000100195408851533329652738757431507110595703125f,-2.05020006779134345009651951841078698635101318359375f,-0.2372000040367245876637980472878552973270416259765625f,-0.917300023776293382837820900022052228450775146484375f,0.1220000031143426699742349228472448885440826416015625f,-0.137999973490833927058929475606419146060943603515625f,0.11530000352561475818191638609278015792369842529296875f,1.23680005646944124464425840415060520172119140625f,0.26759997134804702767496564774774014949798583984375f,-0.9257999834239480918540721177123486995697021484375f,0.7853000165641308427666444913484156131744384765625f,-0.21680000267624865983862036955542862415313720703125f,0.5995000195428730194180388934910297393798828125f,0.2269999965578317624448345668497495353221893310546875f,0.7585999854221945337684473997796885669231414794921875f,-1.2819999680221083604436671521398238837718963623046875f,-0.25149999743700046650474178022705018520355224609375f,0.5433999880015851768888524020439945161342620849609375f,-0.8327000344872477199942295555956661701202392578125f,-0.055600003027915789743929053656756877899169921875f,-1.1220999942064278087627826607786118984222412109375f,0.7832000080525869378789138863794505596160888671875f,-0.18419995786547627147911043721251189708709716796875f,-0.177400033569335757732687852694652974605560302734375f,0.8392000432491304451332325697876513004302978515625f,-0.109600000166893085662422890891321003437042236328125f,-0.5211999881744382445702967743272893130779266357421875f,1.1589999995231625717906354111619293689727783203125f,-1.019700034993887616252550287754274904727935791015625f,1.14840000841617584370624172152020037174224853515625f,-1.014300020110606936896147090010344982147216796875f,1.49500001378357350034775663516484200954437255859375f,0.0346000180482857189190326607786118984222412109375f,0.170799969416857155835032244795002043247222900390625f,0.0160000237226487485031611868180334568023681640625f,0.133799991187453226615389212383888661861419677734375f,0.403500049948693373380592674948275089263916015625f,0.17520000457316609043800781364552676677703857421875f,0.1823000010147690641559847790631465613842010498046875f,0.13570001498162742592512586270458996295928955078125f,0.0302000019654631302046254859305918216705322265625f,-1.4810999793499706189692233238019980490207672119140625f,-0.20739999598115677059695372008718550205230712890625f,-0.04830002231597863016077099018730223178863525390625f,0.0320999938711523657985935642500407993793487548828125f,0.68289999029636339056281713055795989930629730224609375f,0.23660000447332862894000982123543508350849151611328125f,0.096400009623170301864547582226805388927459716796875f,-0.3918999971956014238827492590644396841526031494140625f,-1.7718000081151716873506529736914671957492828369140625f,-0.24249999500811092456586948173935525119304656982421875f,-0.02479999486803929897860143682919442653656005859375f,0.0224000025749206965741677777259610593318939208984375f,-0.263399995377659568163153380737639963626861572265625f,0.2450000059232115834362275563762523233890533447265625f,0.93630004088282614560512229218147695064544677734375f,0.1478000027060508703158347998396493494510650634765625f,-0.675400000652671028689155718893744051456451416015625f,-0.1326000029757619103065735544078052043914794921875f,-0.869700023519992981846371549181640148162841796875f,-0.07789999845474959716540297449682839214801788330078125f,1.11650002180039908949993332498706877231597900390625f,0.19539999957233666894040879924432374536991119384765625f,0.578100009435414730063484967104159295558929443359375f,0.029199998247623426550489966757595539093017578125f,-0.0559999760389324752196671397541649639606475830078125f,0.09440002137273506566117475813371129333972930908203125f,-0.0098000188589093539803798194043338298797607421875f,0.1717000244736670566680913907475769519805908203125f,0.73019998821914189335302580730058252811431884765625f,-1.2778999674975874967230993206612765789031982421875f,1.5673999472171071811743559010210447013378143310546875f,0.2722999878644944971028962754644453525543212890625f,-0.95699996727705016752452138462103903293609619140625f,1.329400007998943866738272845395840704441070556640625f,-1.0711999822139741667115231393836438655853271484375f,0.40359997853040763260423773317597806453704833984375f,0.271600013712048227176865111687220633029937744140625f,-0.68240000436305958686489248066209256649017333984375f,0.4453999838143587464145412013749592006206512451171875f,0.644500017940998759513604454696178436279296875f,-0.593799992305040547080352553166449069976806640625f,0.0332000216126449032572054420597851276397705078125f,0.357800001439451786922063547535799443721771240234375f,-0.07899999177455896415267488919198513031005859375f,0.39999999597668622897117529646493494510650634765625f,-0.6030999890208246227985000587068498134613037109375f,1.133199988010525682824436444207094609737396240234375f,-1.0364000301122668901143697439692914485931396484375f,0.183199992704391689812837284989655017852783203125f,1.0783999979853629014314719825051724910736083984375f,-1.1907999818593264507171625155024230480194091796875f,1.459700020539759890425557387061417102813720703125f,-0.904600011044740082155612981296144425868988037109375f,0.13220002381801609914191431016661226749420166015625f,-1.131100032025575519156745940563268959522247314453125f,-0.3852000012576579646150776170543394982814788818359375f,-0.879000031486153243776016097399406135082244873046875f,0.03080000193864107504992944086552597582340240478515625f,1.199300001856684172452105485717765986919403076171875f,0.2523999892011283829873491413309238851070404052734375f,1.668300021682679101786561659537255764007568359375f,0.2041000071369112045527316467996570281684398651123046875f,1.5810000434294348981012490185094065964221954345703125f,0.3258000061482191345163528239936567842960357666015625f,2.06240007089674559637160200509242713451385498046875f,0.3072000107422472314766537238028831779956817626953125f,0.5985000211894515587118803523480892181396484375f,-0.4475000041350722934652139883837662637233734130859375f,-1.253300022912025468713181908242404460906982421875f,-0.0697000054523349010793253910378552973270416259765625f,0.6790999900937075839379986064159311354160308837890625f,0.20129999725967639978563283875701017677783966064453125f,0.7745000089257951803034529802971519529819488525390625f,0.034499998718500124450514476848184131085872650146484375f,0.9572000182300808290847271564416587352752685546875f,0.2239000080585480390027441899292171001434326171875f,1.43800008510053256571836755028925836086273193359375f,0.4456000133469701918187411138205789029598236083984375f,1.2367000576287516810936040201340802013874053955078125f,0.0956000040337443568461139875580556690692901611328125f,0.0456999972492457828820988652296364307403564453125f,-0.1445000021457671923741372665972448885440826416015625f,-0.6508000198572876371372331050224602222442626953125f,-0.500300017756223791565162173355929553508758544921875f,-0.114800009620189324976990974391810595989227294921875f,0.22129999044239578864790018997155129909515380859375f,-0.5185999801695349020747016766108572483062744140625f,0.7724999697133887455180456527159549295902252197265625f,-1.412599973359703664499420483480207622051239013671875f,1.0298000237762929831575320349656976759433746337890625f,-0.569800012153387047675323628936894237995147705078125f,1.502900006650388586848521299543790519237518310546875f,-1.7165999798119069286173044019960798323154449462890625f,1.4088000204682362692665265058167278766632080078125f,-1.46519998523891015196340958937071263790130615234375f,0.338799971890448858857780578546226024627685546875f,0.0941999933302408720692255883477628231048583984375f,-0.46410002615451872998164617456495761871337890625f,-0.2713999928593633370610405108891427516937255859375f,-0.07190000263452578366241141338832676410675048828125f,-0.1644000005483625681534931572969071567058563232421875f,0.36789999312758414617974267457611858844757080078125f,-0.3315999946385621566236068247235380113124847412109375f,1.10160000216364917235978282405994832515716552734375f,-1.396599988082051435611674605752341449260711669921875f,0.754000080883503898121489328332245349884033203125f,-0.09460003540813932687569831614382565021514892578125f,0.45450004178285663414271766669116914272308349609375f,0.1213999926358459990893834401504136621952056884765625f,0.897999992638826682878061546944081783294677734375f,-0.71239997701942936458863186999224126338958740234375f,-0.186899958750605055257665298995561897754669189453125f,-1.038700005832314143816574869561009109020233154296875f,-0.0910000023767352228531990476767532527446746826171875f,-0.51210001901686208469755001715384423732757568359375f,-0.0334999997317790842998874722979962825775146484375f,0.0380000070184471905321288431878201663494110107421875f,-0.11569999829232691102021135520772077143192291259765625f,0.000199995073676628720704684383235871791839599609375f,0.1272000022485852976927844792953692376613616943359375f,0.613900001874565948156714512151665985584259033203125f,-0.2280000021383166153299271172727458178997039794921875f,-0.627900026953220002923217180068604648113250732421875f,-0.1744999982342123612255591069697402417659759521484375f,-0.6809000183999540656287763340515084564685821533203125f,-0.1652000065475702772488375558168627321720123291015625f,-0.602400015464425120370606236974708735942840576171875f,-0.17979999911487098973594811468501575291156768798828125f,-0.69030001093894199915013132340391166508197784423828125f,-0.04499999996274706148824407136999070644378662109375f,0.102899992009997109931873637833632528781890869140625f,0.3244000056520104369184309689444489777088165283203125f,0.10120001350641272264141434789053164422512054443359375f,-0.3477000034555792762347437019343487918376922607421875f,-0.6747000186771152652198679788853041827678680419921875f,-0.075199999730288957788815196181531064212322235107421875f,0.87570003874898016960059976554475724697113037109375f,0.25660001043379321306048268525046296417713165283203125f,0.711800036203861896666467146133072674274444580078125f,0.0211000047564506854058663520845584571361541748046875f,-1.0663999708801503718547110111103393137454986572265625f,-0.552199990290402897841204321593977510929107666015625f,1.084600008586048769387844004086218774318695068359375f,-1.21380002791881569379484062665142118930816650390625f,0.25279999637305738957593348459340631961822509765625f,-0.55340000036954872797423377051018178462982177734375f,0.4859999869912863257326307575567625463008880615234375f,0.21489999408125992630402834038250148296356201171875f,-0.890999966874719273590699231135658919811248779296875f,0.66190000233650181371558574028313159942626953125f,-0.492400012931227148982316066394560039043426513671875f,0.6847999923348435657999289105646312236785888671875f,-0.532199974048138013671405133209191262722015380859375f,0.053599979573487388506691786460578441619873046875f,-0.2276000035345548955234562527039088308811187744140625f,-0.680100009904801794391460134647786617279052734375f,0.469799991962313701066022986196912825107574462890625f,-0.4990999774649729214814897204632870852947235107421875f,1.03320000723302296563588242861442267894744873046875f,0.6921000288516285792184135061688721179962158203125f,-0.76140000954866404736520735241356305778026580810546875f,-0.0454000235259532924914083196199499070644378662109375f,0.03160000991225253574867792849545367062091827392578125f,-0.628200011926889345659219543449580669403076171875f,0.0307999949723483101848842125036753714084625244140625f,0.99650006680190639229977023205719888210296630859375f,-0.710200028228759716597551232553087174892425537109375f,1.600900040826201742305556763312779366970062255859375f,-1.1442000093489881340502734019537456333637237548828125f,-0.57609994843006173681487780413590371608734130859375f,1.412099997782707117011113950866274535655975341796875f,0.11680000140964985622105132279102690517902374267578125f,-0.10960001611113578956491210192325524985790252685546875f,-0.2176000073343515861523655985365621745586395263671875f,-0.20720001584589520238210980096482671797275543212890625f,0.2346000028118491176343241022550500929355621337890625f,0.338500012025237051460635484545491635799407958984375f,0.058499998986721035709734906049561686813831329345703125f,-0.39160003293454703765519298030994832515716552734375f,-0.2021000146210194525764336503925733268260955810546875f,-0.578800081786514208914695700514130294322967529296875f,-0.2686000057622790482270147549570538103580474853515625f,-1.0431000052630903240924453712068498134613037109375f,-0.23279999793767924831655591333401389420032501220703125f,0.0297999850332739679714677549782209098339080810546875f,0.245199997010827008825373241052147932350635528564453125f,0.4072000070542092231562492088414728641510009765625f,0.001200001697242270015664189486415125429630279541015625f,-0.0811000259160994385609910750645212829113006591796875f,-0.162700005169212846833914909439045004546642303466796875f,0.70000001907348619045734494648058898746967315673828125f,0.279100010061264114380463752240757457911968231201171875f,0.4061000328451402907603551284410059452056884765625f,0.033999998778104778551778508699499070644378662109375f,-0.4140000095069407137771122506819665431976318359375f,-0.0721999963998793958808164461515843868255615234375f,-0.253800009217858058008232546853832900524139404296875f,-0.09589999969899654086447071676957421004772186279296875f,0.499600019240379555895970042911358177661895751953125f,0.8524999856948858312222228050814010202884674072265625f,-1.0441999703824522693906828862964175641536712646484375f,0.04649996420741064895310046267695724964141845703125f,0.19900002515315995399447501768008805811405181884765625f,-0.388199987936019841328061374952085316181182861328125f,0.48099999026954154413004971502232365310192108154296875f,-0.59380000646114350359994205064140260219573974609375f,0.196000001743435792178615884040482342243194580078125f,-1.645400037160516415468691775458864867687225341796875f,1.063000012978911446026586418156512081623077392578125f,-1.5446000898718832416989243938587605953216552734375f,-0.019799959775805842099316578241996467113494873046875f,-1.7063999854087834595617323429905809462070465087890625f,1.6393999403476720022609924853895790874958038330078125f,0.156500012338161464953145696199499070644378662109375f,-0.5977999851226807326298740008496679365634918212890625f,0.73940000561475738294348047929815948009490966796875f,-0.225000002011656707878728411742486059665679931640625f,0.30999994799494789976535003006574697792530059814453125f,-0.54779994630515627118683141816291026771068572998046875f,1.090400021365284910501713966368697583675384521484375f,-1.12300002557039224626578288734890520572662353515625f,-0.1059999792426815634627246254240162670612335205078125f,0.6570000165998930441446646000258624553680419921875f,-1.4105000136494627582806060672737658023834228515625f,1.0824000021278852301520601031370460987091064453125f,-1.018800034177303359683719463646411895751953125f,-0.007000000432133557382030630833469331264495849609375f,0.1572000163674351913556392901227809488773345947265625f,0.31650001524388804430287791547016240656375885009765625f,-0.210599993379413985561399158541462384164333343505859375f,-1.1178000327318902140660838995245285332202911376953125f,-0.23100001046061524601071823781239800155162811279296875f,-0.10860002114772893122562891221605241298675537109375f,0.355200002598762498706719270558096468448638916015625f,1.102000027030706430508644189103506505489349365234375f,0.04230000079125162404380944281001575291156768798828125f,-0.155099993103742672673206470790319144725799560546875f,-0.02329999955445527515252024386427365243434906005859375f,-1.271100007960200617418422552873380482196807861328125f,-0.330500007018446950723955524154007434844970703125f,-0.555400041332841265528941221418790519237518310546875f,0.2453000000983476258653581680846400558948516845703125f,1.003800011751055709652291625388897955417633056640625f,-0.167800002519786328747386505710892379283905029296875f,-0.221700035277009543932535962085239589214324951171875f,0.1869999970421194479541782129672355949878692626953125f,0.5578999727129925201296600789646618068218231201171875f,0.22989999143630257538717387433280237019062042236328125f,1.17949998322129179229023066000081598758697509765625f,0.0070000000596046607626021796022541821002960205078125f,0.22440001246929153211340235429815948009490966796875f,-0.04069999900013210325511181508773006498813629150390625f,0.12029999614953990061394506483338773250579833984375f,-0.0345000015124679038081012549810111522674560546875f,-0.633499984607100774525179076590575277805328369140625f,-0.0599999988451600518857276256312616169452667236328125f,-1.062899985826014859213728414033539593219757080078125f,0.04240000451207193155056529576540924608707427978515625f,-0.21940002111196488243649582727812230587005615234375f,-1.016700024101137689314100498449988663196563720703125f,1.2096000335454935026291423127986490726470947265625f,-0.393999997287989511818295795819722115993499755859375f,-0.16500002458691565010440172045491635799407958984375f,0.5392000018239020420196538907475769519805908203125f,-0.058999992519617183717173247714526951313018798828125f,0.47909999493658528280093378270976245403289794921875f,-0.19579999482333665383748666499741375446319580078125f,-0.8611999701812871332862187045975588262081146240234375f,1.646999958410859665747238977928645908832550048828125f,-0.1657000037312510443143764859996736049652099609375f,-0.774599994876980613156547406106255948543548583984375f,1.12259998326897658671441604383289813995361328125f,-0.757999968275428148700711972196586430072784423828125f,1.282200024488568512737174387439154088497161865234375f,-0.084199982228875480672058984055183827877044677734375f,1.05569997452497421619455053587444126605987548828125f,-1.0633999601364136911030300325364805757999420166015625f,0.69259995003938712443414260633289813995361328125f,-0.981599957460165484235403710044920444488525390625f,0.08810000682771157443085030536167323589324951171875f,0.529399976781010916937475485610775649547576904296875f,1.0214999770596617789664151132456026971340179443359375f,-1.131799982559680728400053340010344982147216796875f,0.374700027394293755378384958021342754364013671875f,0.836999962583184764497445939923636615276336669921875f,-0.161800002378225116217436152510344982147216796875f,-1.285300041449069841092978094820864498615264892578125f,-0.3155000075772405043750268305302597582340240478515625f,-1.52470006285905856913132083718664944171905517578125f,-0.440999999955296484444033922045491635799407958984375f,-0.465199996861815012749730158247984945774078369140625f,0.1359000093102455952731588695314712822437286376953125f,0.134400023421645808952007428160868585109710693359375f,-0.05210000210404397424923672588192857801914215087890625f,-0.45300003316998527225223369896411895751953125f,-0.31900000898540026295080451745889149606227874755859375f,-1.8540000371485951813355086414958350360393524169921875f,-0.3228999996840953290444531376124359667301177978515625f,-1.192200056451559131431849891669116914272308349609375f,-0.1776000105381012605931800862890668213367462158203125f,0.1095999745368951838742077597999013960361480712890625f,-0.0446000021785497580140145146287977695465087890625f,-0.332800008219480691895597601615008898079395294189453125f,0.07720000191330911309961493316222913563251495361328125f,-0.618999983578919721338706949609331786632537841796875f,-0.149299994145333680961584832402877509593963623046875f,-0.684900021201371611567765285144560039043426513671875f,-0.254300004799664025600947070415713824331760406494140625f,-0.616500028952956302674692778964526951313018798828125f,0.1266000057026743785826283783535473048686981201171875f,-0.211499988570809538401817917474545538425445556640625f,-0.088600003452599107589549021213315427303314208984375f,0.903900031751394550383338355459272861480712890625f,0.29030000438243153126904871896840631961822509765625f,0.126300041776895799472413273178972303867340087890625f,0.42099998587369924507584073580801486968994140625f,-0.164599975728988656697993064881302416324615478515625f,0.05919995026588509290377260185778141021728515625f,0.06880003268718670739190201857127249240875244140625f,1.0258000142693521894443620112724602222442626953125f,-1.179600013801455293815934055601246654987335205078125f,0.3320000090003016879336428246460855007171630859375f,0.025400010785460391815604452858678996562957763671875f,-0.29750004068017066316542695858515799045562744140625f,0.26260001485943806898148977779783308506011962890625f,-1.02409999098777770853985202847979962825775146484375f,1.1833999713867904279851472892914898693561553955078125f,-1.089900044298172332446483778767287731170654296875f,-0.236599977129697602862279381952248513698577880859375f,-0.11729999792277823189579066820442676544189453125f,0.1995999947279691877355389806325547397136688232421875f,0.019800029997527612835028776316903531551361083984375f,0.094199980664253446516198664539842866361141204833984375f,-1.262799979907273328905148446210660040378570556640625f,1.185799979996680963978405998204834759235382080078125f,-1.0567000264108177276511923992075026035308837890625f,0.417200014430284316091501750634051859378814697265625f,-0.09039999908804841055598444654606282711029052734375f,0.551800005134939652862158254720270633697509765625f,-0.29610005154609808641907875426113605499267578125f,-0.520999999657272727660028976970352232456207275390625f,0.01110001161098495003898278810083866119384765625f,-0.17240000302195568337992881424725055694580078125f,-0.9608999792099002235090665635652840137481689453125f,0.91680001869499694322485083830542862415313720703125f,0.21260000826567415632695201566093601286411285400390625f,1.37310005965232928559771607979200780391693115234375f,0.26810000679045919991239088631118647754192352294921875f,0.66310002642273924156057773870998062193393707275390625f,0.16190000021308659228225224069319665431976318359375f,-0.925700024071336002862153691239655017852783203125f,-0.348000005714595328498717208276502788066864013671875f,-0.849900039827824027582892085774801671504974365234375f,0.1318000028997659833773781201671226881444454193115234375f,-0.4510000042021273802816949682892300188541412353515625f,-0.248900004109740280444640347923268564045429229736328125f,0.61640000910162928704494333942420780658721923828125f,0.2088000083237886439491148848901502788066864013671875f,0.7048000681817538914941678740433417260646820068359375f,0.19040000661313538632413155937683768570423126220703125f,0.281100028002262636306340937153436243534088134765625f,-0.05349999988079068291568773929611779749393463134765625f,0.635200007590651427591410538298077881336212158203125f,0.335299999278783733558384483330883085727691650390625f,0.361800031584501358139505100552923977375030517578125f,-0.2337999950617551114806502710052882321178913116455078125f,-0.9776000195533034453632126314914785325527191162109375f,-0.2891000048086047602424741853610612452030181884765625f,-0.649500022187829539888070939923636615276336669921875f,0.120399995623528877075614218483678996562957763671875f,0.122700000289082122861827883752994239330291748046875f,0.0870000015124678671707414423508453182876110076904296875f,0.10740001203715798805404801896656863391399383544921875f,0.416299989362060751663108248976641334593296051025390625f,-0.156900010085105900969892900320701301097869873046875f,0.29840000729858850991860208523576147854328155517578125f,-0.181899996674060648871318335295654833316802978515625f,1.4178000322476027594831293754396028816699981689453125f,0.0709999976456165970972733703092671930789947509765625f,-0.1604000018447637732066368698724545538425445556640625f,-0.59299998015910393434069192153401672840118408203125f,-0.848299975526332961095477003254927694797515869140625f,0.69929998777508739493669054354541003704071044921875f,-1.218199981975555346025430480949580669403076171875f,0.23780000687837610229280471685342490673065185546875f,-0.9130999928951271105148634887882508337497711181640625f,0.53859997793436065194327966310083866119384765625f,-1.2830999665200713000245968942181207239627838134765625f,-0.389999993592500704409076206502504646778106689453125f,0.1078000109389425364980752419796772301197052001953125f,-0.81350001113116732209817882903735153377056121826171875f,0.02640001696199212322113680784241296350955963134765625f,0.81200001503527163837503621834912337362766265869140625f,0.26409999866187572337139499722979962825775146484375f,0.41259998937845221433917686226777732372283935546875f,-1.624299989786743392272683195187710225582122802734375f,0.236199990856647445980343036353588104248046875f,-0.736299940299987465408548814593814313411712646484375f,0.91879997367858923240646618069149553775787353515625f,-1.1069999537169934455960174091160297393798828125f,-0.5580999970674513921409243266680277884006500244140625f,0.02060000921189786282639033743180334568023681640625f,-0.651000005245208512860699556767940521240234375f,-0.58050001551210872463570922263897955417633056640625f,1.5498000409871346416679216417833231389522552490234375f,-0.20580000443458601466772961430251598358154296875f,0.2659999893754712019244834664277732372283935546875f,-0.8507000120013952226827314007095992565155029296875f,0.75760002774894275034966995008289813995361328125f,0.28509999623298654114478267729282379150390625f,1.2754000591397289809947324101813137531280517578125f,-0.555500032424927070451303734444081783294677734375f,-0.1536000223398215069892103201709687709808349609375f,0.41329995615482317816713475622236728668212890625f,-0.633500036835670243817730806767940521240234375f,-0.566699992585181888671286287717521190643310546875f,-1.03189997864365512469930763472802937030792236328125f,-1.6294999818801887414565499057061970233917236328125f,-0.04630005682706883618493520771153271198272705078125f,1.74119998519420704496951657347381114959716796875f,0.1232000436663636122602838440798223018646240234375f,0.71720003931522313678215141408145427703857421875f,1.057499990016221858013523160479962825775146484375f,-1.1181999903202051172002029488794505596160888671875f,-0.70140000478029218555775514687411487102508544921875f,-0.33989998886585226500756107270717620849609375f,-2.1448000576019286000928332214243710041046142578125f,0.84090003166198812323273159563541412353515625f,-0.3902999949574468274704486248083412647247314453125f,-0.15669997708797467339536524377763271331787109375f,1.7885000028610225086822538287378847599029541015625f,-1.0475000091269615065669995601638220250606536865234375f,-0.23740000741779809789733235447783954441547393798828125f,-0.9691999770134682723465857634437270462512969970703125f,-0.36330000845789900409954498172737658023834228515625f,-1.71570001863241206052634879597462713718414306640625f,-0.346500008016824789791598959709517657756805419921875f,-1.165500017225742368509600055404007434844970703125f,0.30079999728202810871380279422737658023834228515625f,0.2202000294208528163864002635818906128406524658203125f,-0.7110000168681143151161450077779591083526611328125f,0.32409998927414396252544293020037002861499786376953125f,-0.1862000142514705913754369248636066913604736328125f,0.3994999908581375880345376572222448885440826416015625f,-0.572299978923797691976460555451922118663787841796875f,0.6233999833449723748657333999290131032466888427734375f,-0.86550000247359282212755715590901672840118408203125f,0.481399970619380557224076255806721746921539306640625f,0.88800000903010367636625232989899814128875732421875f,-1.49869997338503591066682929522357881069183349609375f,0.60700000900030115502659100457094609737396240234375f,0.0815999908387663452913329820148646831512451171875f,-0.83349997529387476191686801030300557613372802734375f,0.846900003416835911451698848395608365535736083984375f,-0.31169998483657845866900970577262341976165771484375f,1.2279000042662022718076286764699034392833709716796875f,0.84520002182126052758803780307061970233917236328125f,0.3286000075504180717445024129119701683521270751953125f,0.33169999750256540238524394226260483264923095703125f,-0.36989998692870162955159685225225985050201416015625f,0.6506999998569487075883444049395620822906494140625f,-1.592800026845932137575800879858434200286865234375f,-1.50099999451637255987179742078296840190887451171875f,0.80970003275871249570627696812152862548828125f,-0.2152000207781785690031028934754431247711181640625f,2.4187000024795537456157035194337368011474609375f,-0.994200016129017427601866074837744235992431640625f,-0.35089998576641079353066743351519107818603515625f,1.6024000305891039630523664527572691440582275390625f,-0.845900009310245337701417156495153903961181640625f,0.83380001182556195971073975670151412487030029296875f,-0.513399991130828681207276531495153903961181640625f,0.77450000222027259866308668279089033603668212890625f,0.1841999731391670280800099135376513004302978515625f,-0.68309997389614540708180356887169182300567626953125f,0.416700020152330097289450350217521190643310546875f,-0.39470002217888833229153533466160297393798828125f,1.452400004363060048717670724727213382720947265625f,-1.00370001559257548962023065541870892047882080078125f,-0.273099996471405148668054607696831226348876953125f,1.44349999406933715562217912520281970500946044921875f,-0.7982000120759007444348753779195249080657958984375f,1.643500041007996070874241922865621745586395263671875f,-0.121500029325485314757315791212022304534912109375f,-1.445400025463104753953302861191332340240478515625f,0.136799977493286206708944519050419330596923828125f,1.19770001332759878920342089259065687656402587890625f,-0.932400001382827525731045170687139034271240234375f,-0.18859998702406866044611888355575501918792724609375f,0.48969995635300900271857926782104186713695526123046875f,-1.026300013613700823356111868633888661861419677734375f,0.8545999637708068252806015152600593864917755126953125f,0.68530001447796795588374152430333197116851806640625f,1.0354000218868260052573759821825660765171051025390625f,1.3084000055849553945108709740452468395233154296875f,-0.92629999517351446769453104934655129909515380859375f,0.35379999513626092522144972463138401508331298828125f,-0.198400029054284221530224385787732899188995361328125f,-0.475599999451637334146880675689317286014556884765625f,-0.773199978473782412180526080192066729068756103515625f,-0.381899991160631113729095886810682713985443115234375f,-0.845399988731742269010283052921295166015625f,-0.06630000661015511553841861314140260219573974609375f,0.2521000158131123658478145443950779736042022705078125f,0.25340000707507126520567908301018178462982177734375f,0.7511000123187903110277829910046420991420745849609375f,0.739800001722574318563374617951922118663787841796875f,-0.6791999845758081022495389333926141262054443359375f,0.323799981129169633931041971663944423198699951171875f,-0.2635000242441896123324340805993415415287017822265625f,-0.4449999995529652085934912975062616169452667236328125f,-0.162799965378642053082103302585892379283905029296875f,-0.64260000472664824400226279976777732372283935546875f,-1.3705000086948277981946375803090631961822509765625f,0.429699978071451216266041228664107620716094970703125f,0.2887999876111744246287571513676084578037261962890625f,-0.690699988979101231478807676467113196849822998046875f,0.0949000036925073953142373284208588302135467529296875f,0.4184999904930588598972462932579219341278076171875f,-0.2804999728202819397893108543939888477325439453125f,0.298800017338991086290889143128879368305206298828125f,0.1075999923586845596901184762828052043914794921875f,0.1418999901361761573070907616056501865386962890625f,-0.4600000238791108753133585196337662637233734130859375f,0.3320999941691760160011881453101523220539093017578125f,0.7589000366687770338103291578590869903564453125f,-1.013500031620264252296692575328052043914794921875f,1.193299971801042858032815274782478809356689453125f,1.1565000233650213345981683232821524143218994140625f,-0.4989000211715701738057759939692914485931396484375f,-1.71820001803636568382671612198464572429656982421875f,-1.0746999731540682887498405762016773223876953125f,0.118300017510354127381333455559797585010528564453125f,0.669899996018409549236594102694652974605560302734375f,1.6449999982863661163179358482011593878269195556640625f,-0.19260002514123897299214149825274944305419921875f,-1.3132999677777288383140330552123486995697021484375f,0.5577000347852703043827204965054988861083984375f,-1.117200004148482861410229816101491451263427734375f,-0.090399939632415993173708557151257991790771484375f,0.6843000382900239486616555950604379177093505859375f,0.335499974608421069888208876363933086395263671875f,0.97779999465942335490353798377327620983123779296875f,0.5145000009536744300930877216160297393798828125f,-0.0429999456629150955677687306888401508331298828125f,0.38990004217475604519904663902707397937774658203125f,0.189999966956674537499338839552365243434906005859375f,-0.002100004749000128345670646012877114117145538330078125f,0.0564000107407569561956961479154415428638458251953125f,0.72470001111477611033251378103159368038177490234375f,-0.0331000027298926635666020956705324351787567138671875f,0.62499999748542889310698456029058434069156646728515625f,-0.554100006205588524466065791784785687923431396484375f,0.51900001380965166131176147246151231229305267333984375f,-0.1421999665975572924025982501916587352752685546875f,-0.1872000040739774107123594149015843868255615234375f,0.50829999489784238875245137023739516735076904296875f,-0.41539998020082702367972160573117434978485107421875f,-0.78090000498890876912128078402020037174224853515625f,-1.1838000077649948327263018654775805771350860595703125f,-0.0772000065326690076972226961515843868255615234375f,0.5762999823212624495027966986526735126972198486328125f,0.688699982474744398786015153746120631694793701171875f,-0.666800005209446045029153538052923977375030517578125f,-0.27250000745058056139669133699499070644378662109375f,-0.2321999990448355699612648095353506505489349365234375f,0.7092000055491922694272943772375583648681640625f,0.92969995098859126159140942036174237728118896484375f,0.520300018054246748988589388318359851837158203125f,0.36059998425245309849884733921498991549015045166015625f,0.376199970293045193869829745381139218807220458984375f,-1.3375000135600567663374249605112709105014801025390625f,0.41860000043511380596328308456577360630035400390625f,0.4569000282645230281985959663870744407176971435546875f,0.226000015974044732303127602790482342243194580078125f,-0.051300045092403934177127666771411895751953125f,-0.731399973338842546155547097441740334033966064453125f,0.273000008583068165535223670303821563720703125f,-0.9043000327952206429671377918566577136516571044921875f,-0.6544999788999554812818360005621798336505889892578125f,-0.05339999193176636627100606347084976732730865478515625f,0.269899978975206789044705146807245910167694091796875f,1.1757000446163126117227193390135653316974639892578125f,0.72669997828006671625189483165740966796875f,-0.85380005839169115233744378201663494110107421875f,0.3995999990493057651974595501087605953216552734375f,-0.059400004646182225087613915093243122100830078125f,-0.1646000201344488544918931438587605953216552734375f,-0.828500017166137414648119374760426580905914306640625f,-0.2092000018239019709653803147375583648681640625f,-1.22930000774264325258400276652537286281585693359375f,0.7330999917030336376910781837068498134613037109375f,1.62750002279877659105977727449499070644378662109375f,-0.217299985927343897174068843014538288116455078125f,0.1401000470459461411110169137828052043914794921875f,2.15749996185302705953290569595992565155029296875f,0.4126999789804219886946157203055918216705322265625f,0.0546999848514797548659771564416587352752685546875f,-0.762499978616833828937160433270037174224853515625f,0.694599989736079947988400817848742008209228515625f,0.732700002375244796581910122768022119998931884765625f,0.48430000669956196901466682902537286281585693359375f,-1.2937000266194347286585752954124473035335540771484375f,1.221400008654594415702376863919198513031005859375f,-0.38699998582899564070203268784098327159881591796875f,0.1060999617293483243685159322922118008136749267578125f,0.4419999864995478588980404310859739780426025390625f,1.088899954600632380419256151071749627590179443359375f,-0.00239999298229809365778919527656398713588714599609375f,1.483799976006894727476037587621249258518218994140625f,0.1441999940752982656277936257538385689258575439453125f,0.9167000390954320732106452851439826190471649169921875f,-0.3195000101923941571158138685859739780426025390625f,-0.98109999786466328686174165341071784496307373046875f,0.076700000929832423679499697755090892314910888671875f,0.34239999456554637280447650482528842985630035400390625f,-0.3022000007212162220326945316628552973270416259765625f,0.5007999908000229094540145524661056697368621826171875f,-0.122299996507167829662421354441903531551361083984375f,0.35910000778883677252650841182912699878215789794921875f,0.76959998392462747318631954840384423732757568359375f,-0.5007000067114828478764820829383097589015960693359375f,-0.279099994415044960760496906004846096038818359375f,-0.597299993675946883087135574896819889545440673828125f,1.09629997427463532488900455064140260219573974609375f,-0.796999990187585272138903746963478624820709228515625f,-0.069300004315376195762610223027877509593963623046875f,-0.66289997729510086177384664551937021315097808837890625f,0.7377999758839608457350323078571818768978118896484375f,0.8834000148981810962567351452889852225780487060546875f,0.07280000561177735196594085209653712809085845947265625f,0.1521000157758594095724902217625640332698822021484375f,0.907399995571374962111121931229718029499053955078125f,1.32160001129061033253719870117492973804473876953125f,-1.404699984441697413473093547509051859378814697265625f,-0.084499991118908379661434082663618028163909912109375f,0.9088999943494793232190431808703579008579254150390625f,-0.87800001484155654196683826739899814128875732421875f,-0.5973000079810617535258643329143524169921875f,-2.45669994370937416761080385185778141021728515625f,-0.38770000975132035136994090862572193145751953125f,0.4306000066041946183759137056767940521240234375f,-0.40480002362728129128299769945442676544189453125f,-0.64189998415708515722144511528313159942626953125f,1.903299995791911669584806077182292938232421875f,-0.20360000743865924732745043002068996429443359375f,-0.7259000167608267162222546176053583621978759765625f,-0.46699998694658262365919654257595539093017578125f,-1.6604000493049628772723735892213881015777587890625f,0.1949999848008161507095792330801486968994140625f,1.1582000290632255001810335670597851276397705078125f,0.664299973917007235968412715010344982147216796875f,-0.5482000061154366221671807579696178436279296875f,-0.609300011169910504804647644050419330596923828125f,0.55270003806352629993625669158063828945159912109375f,1.96769994895458211203731480054557323455810546875f,0.00439998337477476297863177023828029632568359375f,1.16979998298436438375347279361449182033538818359375f,-1.474600011380017061668468159041367471218109130859375f,-0.86459995725154925594324595294892787933349609375f,0.822400031483173421520405099727213382720947265625f,2.233500001072883378583355806767940521240234375f,-0.493399977011978574381600992637686431407928466796875f,0.5848999681159858798906725496635772287845611572265625f,0.3968999825552106397452689634519629180431365966796875f,1.134799976055324233215060303336940705776214599609375f,-0.6778000039726495007386120050796307623386383056640625f,-0.1504000179752702504032413344248197972774505615234375f,-1.1993999829649926169850004953332245349884033203125f,0.6893999677658084745957012273720465600490570068359375f,0.41219999803900719825833220966160297393798828125f,-0.335000001825392246246337890625f,0.0620999983787537956914093228988349437713623046875f,-0.285599958883226090478046899079345166683197021484375f,-0.34959999554753284911612354335375130176544189453125f,0.26490005133897032152390238479711115360260009765625f,-0.25949999201297746509453645558096468448638916015625f,1.5721999778106801048949137111776508390903472900390625f,0.34779999704360964329907801584340631961822509765625f,0.74540000735819322841280154534615576267242431640625f,-0.06210000687241556960316302138380706310272216796875f,-1.02970000973641884201015272992663085460662841796875f,-0.277200008022785215899830291164107620716094970703125f,0.241699992734193624954741608235053718090057373046875f,0.8390999765336513149804886779747903347015380859375f,-0.2241999560400842472773774716188199818134307861328125f,0.388299998772144494552094329264946281909942626953125f,-0.6882999991446734355804437655024230480194091796875f,0.011600003951787840605902601964771747589111328125f,0.8681999899104242413017118451534770429134368896484375f,1.137899998939037260470286128111183643341064453125f,-0.7389999905452129258520699295331723988056182861328125f,-0.9851000164985652673976801452226936817169189453125f,-1.58559997024536158960472675971686840057373046875f,0.290600038790703507629586965776979923248291015625f,1.61200001287460281673702411353588104248046875f,0.271700006871670840524757295497693121433258056640625f,-0.4485999811753631494326555184670723974704742431640625f,1.2079000179007646220696869931998662650585174560546875f,0.1102000014066693012182440725155174732208251953125f,-0.75849996709823575002928919275291264057159423828125f,-0.379399995183945293319993652403354644775390625f,-0.0450000305846334214265880291350185871124267578125f,0.72019996851235656976086829672567546367645263671875f,0.514000027798116487787183359614573419094085693359375f,1.190899985915422121252049691975116729736328125f,-0.35010004227757463013404048979282379150390625f,-2.355600004220008258926100097596645355224609375f,0.016400010442733758964095613919198513031005859375f,0.668799995434284255679813213646411895751953125f,-1.0710000237226484642860668827779591083526611328125f,-0.6549999769032002205904063885100185871124267578125f,-0.1143000000312923702949774451553821563720703125f,0.460799996015429425000320406979881227016448974609375f,-0.76340006243288538456681635580025613307952880859375f,-1.14359996035098987476885668002068996429443359375f,-1.07080002142190977565405773930251598358154296875f,1.325399968838691933115114807151257991790771484375f,-0.23319999330043739149687098688445985317230224609375f,0.568399983084201920746636460535228252410888671875f,0.6590000069737431687144635361619293689727783203125f,-0.5046000242322683515538983556325547397136688232421875f,-0.842899981951713783701052307151257991790771484375f,-0.2249000261560076818767583972658030688762664794921875f,0.8700000031292436375451870844699442386627197265625f,-0.933199994399398047306704029324464499950408935546875f,-0.1590999954394995696826953235358814708888530731201171875f,-0.42100000388547764718083499246858991682529449462890625f,-0.2028999944686891154788099811412394046783447265625f,0.321399983471632400000572715725866146385669708251953125f,-0.24890000373721132831406066543422639369964599609375f,0.5182999714657665624173432661336846649646759033203125f,0.2341999907970429006809354177676141262054443359375f,-0.50229998097270733836694489582441747188568115234375f,0.3858999953031538865388938575051724910736083984375f,-0.64780000743716958933049454572028480470180511474609375f,-1.2381999946415422897416647174395620822906494140625f,1.217500015608965924229778465814888477325439453125f,0.1506000054121017228681012056767940521240234375f,0.4958999919503928044406393382814712822437286376953125f,-0.29079998804330831063680307124741375446319580078125f,1.198099997738003796854400206939317286014556884765625f,0.062500001676380645410091574376565404236316680908203125f,1.203400015866756600502185392542742192745208740234375f,-0.8207999642014502938991427072323858737945556640625f,0.9403999935001128296363503977772779762744903564453125f,0.715399988806247666417448272113688290119171142578125f,-0.32599998609721669851069236756302416324615478515625f,0.4273999852895737916469443007372319698333740234375f,-0.59939999935627008564864581785514019429683685302734375f,-0.6482999874472614720843921531923115253448486328125f,2.03850000107288309436626150272786617279052734375f,0.8345999784298239188018442291649989783763885498046875f,0.0830000021755694650238410758902318775653839111328125f,-0.48300000020116529153568762922077439725399017333984375f,-0.41159997698068639948587588150985538959503173828125f,1.9131999897241585006923969558556564152240753173828125f,0.5535999782323841600373270921409130096435546875f,0.0682000037312509022058293339796364307403564453125f,0.90229998130798350075565394945442676544189453125f,-2.2802000163078304240116267465054988861083984375f,0.29279998869895962343434803187847137451171875f,0.2573999729216094323192010051570832729339599609375f,1.3793999749183658565243604243732988834381103515625f,1.12800003555417038825225972686894237995147705078125f,-1.2529000323176386899604040081612765789031982421875f,-0.397900026077777635435950287501327693462371826171875f,0.445699987433105793144250128534622490406036376953125f,0.06819999320730563141523816739208996295928955078125f,1.904299967956543326863538823090493679046630859375f,0.2937000269174578903630390414036810398101806640625f,1.5664000190854068250700947828590869903564453125f,-0.0919000179827218488526341388933360576629638671875f,0.77439999563097927648414042778313159942626953125f,-0.1394000066578389152027739328332245349884033203125f,-1.53570003323554971075282082892954349517822265625f,0.729400013624131648981574471690692007541656494140625f,-0.572500037252903215545529747032560408115386962890625f,-1.5387000420421366175816046961699612438678741455078125f,-0.56169998759329242687954319990240037441253662109375f,0.94919999541640276419229849125258624553680419921875f,-0.55020000479668340620520439188112504780292510986328125f,-0.35809998535141362996370162363746203482151031494140625f,-1.5367999785363668507898182724602520465850830078125f,-0.02820000008046619210944072619895450770854949951171875f,-0.1078999800890685367704691088874824345111846923828125f,-0.21479998980164527466740764793939888477325439453125f,-0.4323999721392992601209925851435400545597076416015625f,0.0443000081896780528722956660203635692596435546875f,0.3487000184610484776470684664673171937465667724609375f,0.48429998494386694574131979607045650482177734375f,0.25390001539736939406566307297907769680023193359375f,0.87099998408556000839553234982304275035858154296875f,-0.3156999679312113560314401183859445154666900634765625f,-0.9449000198602675038017650877009145915508270263671875f,0.05409999309256663391209940527915023267269134521484375f,0.16009999690353871049097733703092671930789947509765625f,-0.33109997769221688468377351455274038016796112060546875f,0.973999987453222360045401728712022304534912109375f,-0.05309993809610580317581707276985980570316314697265625f,0.60980000798106193116154827293939888477325439453125f,-0.49780000050812933665156378992833197116851806640625f,0.28829999802708616840618560672737658023834228515625f,-0.19490001017451297204985394273535348474979400634765625f,-0.8635000167191027031776684452779591083526611328125f,0.445800006744265342462085754959844052791595458984375f,-0.6800000182166694440155652046087197959423065185546875f,-0.8230000428184869321057703928090631961822509765625f,-1.08200004141032746218797910842113196849822998046875f,0.55460000836253176004220222239382565021514892578125f,1.6112000057578084177833943613222800195217132568359375f,-0.0762000638723367984539436292834579944610595703125f,-1.13399998670816426482588212820701301097869873046875f,1.672999980568885547427271376363933086395263671875f,-1.36530003265738475448642930132336914539337158203125f,-0.3637999832153315793448200565762817859649658203125f,1.34460000374317178994942878489382565021514892578125f,-0.792799992871284331386050325818359851837158203125f,0.76130002203285673800792210386134684085845947265625f,0.9993000009626147317476352327503263950347900390625f,0.04800002236664280275135752162896096706390380859375f,0.1537999805331224933979683555662631988525390625f,1.5911000207751992974181121098808944225311279296875f,-0.0914000097721814341866775066591799259185791015625f,1.24660002496242494629541397443972527980804443359375f,0.3233999898642299086759521742351353168487548828125f,0.367100021065771553452350417501293122768402099609375f,1.574899993336200676452563129714690148830413818359375f,0.9767999967157841023635000965441577136516571044921875f,-1.55129998753666864530487146112136542797088623046875f,0.6958000361740592776271796537912450730800628662109375f,-0.264199997055531099476866074837744235992431640625f,0.57150000730901950873885652981698513031005859375f,1.13809998004287482586960322805680334568023681640625f,0.934900019301474483057745601399801671504974365234375f,0.8212000221490853846262325532734394073486328125f,-2.0118000683903698444510155241005122661590576171875f,-0.5535999984979629306991455450770445168018341064453125f,0.51190000017583370262030939557007513940334320068359375f,0.1465000248923893355623704337631352245807647705078125f,-0.5150000175833699955774136469699442386627197265625f,-0.3725999773457651098595988514716736972332000732421875f,0.964199984389543462981464472250081598758697509765625f,0.13509998293370006905433911015279591083526611328125f,0.2997000028073788513438557856716215610504150390625f,0.2569000119104980495166046239319257438182830810546875f,-0.512199993568658751286193364649079740047454833984375f,-0.49239996729642143691307865083217620849609375f,0.31000000149011608385762883699499070644378662109375f,-1.6067000243246554180842622372438199818134307861328125f,0.21949997800588594287773958058096468448638916015625f,-1.402199995543062183145366361713968217372894287109375f,-0.1731000045180319801829682546667754650115966796875f,-1.7628999772205948826098165227449499070644378662109375f,0.02919999362826342093057974125258624553680419921875f,-1.38009998136907796606465126387774944305419921875f,0.8983999937757849441055668648914434015750885009765625f,-0.764799998481571652320099019561894237995147705078125f,-1.011799983677267977810743104782886803150177001953125f,-0.7398000032499429945431757005280815064907073974609375f,-0.33150000131130230585085882921703159809112548828125f,-0.53539999558627571207836126632173545658588409423828125f,0.347199996250867837233045065659098327159881591796875f,-0.18429996534883930081605285522527992725372314453125f,-0.17999997809529322267962925252504646778106689453125f,0.0256000280991199868907415293506346642971038818359375f,0.70510001403987399015704795601777732372283935546875f,-0.055600039982795834703210857696831226348876953125f,-0.89360005542039910864104967913590371608734130859375f,-0.17569998912811257696375832892954349517822265625f,-2.01389997110366802957059917389415204524993896484375f,0.498000013947486053211832768283784389495849609375f,1.57619997982978787831598310731351375579833984375f,-1.01370005433559384755426435731351375579833984375f,-0.815699987712502405656778137199580669403076171875f,-1.0990999572366479952734152902849018573760986328125f,1.98339999999701976918231594027020037174224853515625f,-0.55679998285770437860264792107045650482177734375f,0.144100026154518445764551870524883270263671875f,-0.23760002663135537659400142729282379150390625f,0.48699999856948839038750520558096468448638916015625f,-0.29600002312660222969498136080801486968994140625f,-0.287000026825815712783196431701071560382843017578125f,-0.7552000044986606486219216094468720257282257080078125f,-0.056099977748095941620931625948287546634674072265625f,0.74170000867843555170111358165740966796875f,1.330600007777660298557975693256594240665435791015625f,-0.1824000341281293291473275530734099447727203369140625f,-1.0180000172071153041741808920050971210002899169921875f,0.5315000079050660186652521588257513940334320068359375f,0.287299992297589679157709952050936408340930938720703125f,-1.44070001997351671008118501049466431140899658203125f,-0.0606000048160557636123257907456718385219573974609375f,0.4548999927401542464622252737171947956085205078125f,-0.433300002944469131449523047194816172122955322265625f,0.00880005243122539138056481533567421138286590576171875f,-0.584100012594461315273974832962267100811004638671875f,-9.9985429644622314526714035309851169586181640625e-05f,-0.284899983948469159855676480219699442386627197265625f,0.59159998718798167960386535924044437706470489501953125f,0.71159998174905769730003157746978104114532470703125f,0.12549995851516737577213689291966147720813751220703125f,-0.768700023341178706459686509333550930023193359375f,-0.4448999912872910744709997743484564125537872314453125f,-0.32909998875260360051697716698981821537017822265625f,-0.07740003565549802289069702965207397937774658203125f,-0.747299976241588836955997976474463939666748046875f,-1.1457999890491363803590729730785824358463287353515625f,-0.298500013887882165164455727790482342243194580078125f,0.17000001702457667729362356112687848508358001708984375f,-0.546400005674362088115003643906675279140472412109375f,-0.2442999947227537660410234821029007434844970703125f,-0.59219999973401427073582681259722448885440826416015625f,-1.274899990821629192083719317452050745487213134765625f,0.59219998283982278053372283466160297393798828125f,0.44289998241737482675972614742931909859180450439453125f,-0.564699999659508566907106796861626207828521728515625f,0.4271000137269498342362794574000872671604156494140625f,0.1171000094242393985144445878177066333591938018798828125f,-0.48639999686405055445703737859730608761310577392578125f,-0.93470001524984824836650432189344428479671478271484375f,0.4977000020027160243074604295543394982814788818359375f,0.0360000027120113141876345252967439591884613037109375f,0.119700006085634580443866070709191262722015380859375f,0.902100029969215455594166996888816356658935546875f,1.392299972069263702678654226474463939666748046875f,-0.63770002241730683323339690105058252811431884765625f,1.5877000297188761379629795555956661701202392578125f,-1.020200018393993435239508471568115055561065673828125f,-0.41579996867179858099916600622236728668212890625f,0.5225000068731600055826902462285943329334259033203125f,0.8430000222176314839117594601702876389026641845703125f,0.105900029538572137965957153937779366970062255859375f,-1.3859999742507937980917631648480892181396484375f,1.215700013566017645416650339029729366302490234375f,-0.504999998807907246600734652020037174224853515625f,-0.1131999871909616928178365924395620822906494140625f,0.3262999814271925203001956106163561344146728515625f,-0.894900039827823956528618509764783084392547607421875f,-1.342100009405612581758759915828704833984375f,1.504300029051304221638929448090493679046630859375f,1.0767000089764593440122553147375583648681640625f,0.287499990761279633488811668939888477325439453125f,-1.249500033736229198666478623636066913604736328125f,-0.65119998877048512753162867738865315914154052734375f,-1.3355999922990804407163523137569427490234375f,-0.3005999945342541224135857191868126392364501953125f,-1.0722999842882163790136473835445940494537353515625f,0.117599941154569176404720565187744796276092529296875f,-1.3876999603912232306157648054067976772785186767578125f,0.417800015558302373364085724460892379283905029296875f,0.6886999780416491745427265414036810398101806640625f,-0.25789999141395103521290366188623011112213134765625f,0.2950000143051145773398502569762058556079864501953125f,0.656899989521503524514400851330719888210296630859375f,-0.027599963860213794220044292160309851169586181640625f,0.9533000191867351258423468607361428439617156982421875f,-0.6539999777302145655966114645707421004772186279296875f,0.37989999795556084194458890124224126338958740234375f,0.37449999920278764786729652769281528890132904052734375f,0.7517000083990394987409899840713478624820709228515625f,0.35009996967166667758419862366281449794769287109375f,-0.6076999902307989032124169170856475830078125f,0.315999969631433952343968485365621745586395263671875f,-0.6340999893486500837980202049948275089263916015625f,-0.25639998861253243678248736614477820694446563720703125f,-0.0499000080883502672435270142159424722194671630859375f,-0.54830002153366830253133912265184335410594940185546875f,-0.4553000073254107604725504643283784389495849609375f,0.6448000361442567385239499344606883823871612548828125f,0.1321000137269496832459481083787977695465087890625f,-1.2684999868422746072127438310417346656322479248046875f,-0.921700020301341904627179246745072305202484130859375f,0.0455999870464207646136856055818498134613037109375f,-0.6621000074684619818299324833787977695465087890625f,1.22369995837211664735377780743874609470367431640625f,-0.66410000082254416753357872948981821537017822265625f,-0.2300000444985921266294326414936222136020660400390625f,-0.76719997052773847290296771461726166307926177978515625f,-0.25159997677579537533887332756421528756618499755859375f,0.30179999239444743608373755705542862415313720703125f,-0.424999976418912683850948042163508944213390350341796875f,-0.0118000267416242810014637143467552959918975830078125f,0.463699963438511275626296992413699626922607421875f,1.1195000349283219520657439716160297393798828125f,-0.922600017690658802393954829312860965728759765625f,-1.18750002577900914957353961654007434844970703125f,0.143899981385469999395354534499347209930419921875f,0.026300007653237145177627098746597766876220703125f,-0.6274999731779100642370394780300557613372802734375f,-0.7111000201977797363639410832547582685947418212890625f,-0.1002000036418435602314502830267883837223052978515625f,0.89880001040995105565656331236823461949825286865234375f,-0.174200008380413162711874974775128066539764404296875f,0.2318999984435736738674904700019396841526031494140625f,-0.1161000105306506657854015429620631039142608642578125f,-1.005300077919662360415031798765994608402252197265625f,-1.149299988818167861381880356930196285247802734375f,-1.753299963903426572642274550162255764007568359375f,0.86480001766681624530974659137427806854248046875f,0.381099985366314175649904427700676023960113525390625f,0.4742999879613518654508652616641484200954437255859375f,-0.4078000212952492820051020316896028816699981689453125f,-1.4360999508142473501948188641108572483062744140625f,0.50760000845193875562699759029783308506011962890625f,0.141799965274334027753866394050419330596923828125f,0.1520000021457672545466266456060111522674560546875f,-0.103700023937225438430687063373625278472900390625f,-0.82660002704858737132553869741968810558319091796875f,-0.874399966871738598683805321343243122100830078125f,0.78180000267624816245870533748529851436614990234375f,-0.754199992361665128015602022060193121433258056640625f,0.28719998209476482742275038617663085460662841796875f,-0.63749998606741453688329102078569121658802032470703125f,-0.73740000734329225196006518672220408916473388671875f,0.35759997939467391692147657522582449018955230712890625f,0.24459999673962595778675677138380706310272216796875f,0.2835000170543791053745508179417811334133148193359375f,-0.2602999942123889187683971613296307623386383056640625f,0.14260000074058754027106488138088025152683258056640625f,0.28499998940154924742529374270816333591938018798828125f,0.60350000114738922007973087602294981479644775390625f,-0.26340000212043523930560695589520037174224853515625f,0.611800011523813136449234661995433270931243896484375f,-0.46799999522790314276932122083962894976139068603515625f,0.18949999013170570894004640649654902517795562744140625f,-0.66049997639656066183988514239899814128875732421875f,0.941300021101534323264559134258888661861419677734375f,0.89599998736381536446060636080801486968994140625f,1.433599978362769089557104962295852601528167724609375f,0.1181999971933662962886302238985081203281879425048828125f,0.25349997996911433073563557627494446933269500732421875f,-0.820999969333410462013489450328052043914794921875f,0.46549998801946657067674095742404460906982421875f,-0.235100010538101145129985525272786617279052734375f,0.23400000321120006940844859855133108794689178466796875f,-0.482900000876188340015460198628716170787811279296875f,-0.696599997842311413620564053417183458805084228515625f,-0.375899988448619826186813952517695724964141845703125f,-0.124199999551474871140044342610053718090057373046875f,-2.263300050330162349609963712282478809356689453125f,-0.626899979686736941175695392303168773651123046875f,0.5034000337481501219372148625552654266357421875f,-2.08589994702339254217804409563541412353515625f,0.873699983089416765125179153983481228351593017578125f,0.0046000186815857713185096145025454461574554443359375f,0.2108999464645975319143644810537807643413543701171875f,-0.28160003747940010043748770840466022491455078125f,0.145499993048608278201072607771493494510650634765625f,1.232899983441830382702164570218883454799652099609375f,-1.0505999829858538863192052303929813206195831298828125f,-1.0245999465227129121558391489088535308837890625f,0.2659000417947776639948642696253955364227294921875f,-0.8868999886274337995928362943232059478759765625f,1.5251000424265868105067056603729724884033203125f,-0.22390001759529098990242346189916133880615234375f,0.4728000164151193729367150808684527873992919921875f,0.4567999518632888111824286170303821563720703125f,-1.069799997401236879568386939354240894317626953125f,-1.06089997861385398891798104159533977508544921875f,-0.568800042372942016299930401146411895751953125f,-0.4115999447941778299764337134547531604766845703125f,0.527300011706352211859893941436894237995147705078125f,-2.2871999344110491847459343262016773223876953125f,0.015399991264938961421648855321109294891357421875f,0.76800001350045210557482278090901672840118408203125f,0.0023999884188172782018000361858867108821868896484375f,-0.1229000022172925810082233510911464691162109375f,0.506199989776313419298503504251129925251007080078125f,-0.85440001049488767392858790117315948009490966796875f,0.21339999664425857162086686003021895885467529296875f,0.63369997621625717698634616681374609470367431640625f,-0.90519999067783363244643624057061970233917236328125f,1.382599991445989218163958867080509662628173828125f,-0.403199978678673520704478505649603903293609619140625f,-0.94279999423101568112315362668596208095550537109375f,-0.383500006884336386292488896287977695465087890625f,-1.17989995355010091060421473230235278606414794921875f,0.6888000007614494268892713080276735126972198486328125f,-0.7190000188574199313507051556371152400970458984375f,0.02870001857280701784702614531852304935455322265625f,0.2303999246940018341689437875174917280673980712890625f,-0.029400001591444180348844383843243122100830078125f,0.75259998256117111026242127991281449794769287109375f,-0.43389999688267710808986521442420780658721923828125f,0.11140000112950776411935294163413345813751220703125f,-0.2667000119566917959446072927676141262054443359375f,-0.456100011052190890126212252653203904628753662109375f,-0.546499996542930688292472041212022304534912109375f,-0.3908000211611384333565410997834987938404083251953125f,0.1665000230669975156416739991982467472553253173828125f,0.2059999989867213887606567368493415415287017822265625f,-1.07589997131228454207985123503021895885467529296875f,0.6490999941915276405524082292686216533184051513671875f,0.47020000639855863422411630381247960031032562255859375f,1.0314000086545942469484771208954043686389923095703125f,-0.3064999889433384083758937777020037174224853515625f,0.1849000136017797668586126746959052979946136474609375f,-1.0035000356435777035812861868180334568023681640625f,-0.778699998605251408889671438373625278472900390625f,1.09920001612901696574908783077262341976165771484375f,-0.27350001478195196114029386080801486968994140625f,-0.7349000362515454298772965557873249053955078125f,2.07909999158382419182089506648480892181396484375f,-0.0127999970436096788262148038484156131744384765625f,-0.295099982821941608790439204312860965728759765625f,1.40610002375543086117204438778571784496307373046875f,-1.00240002090334900941570595023222267627716064453125f,-1.3576000041306020005293930807965807616710662841796875f,-0.519799992036819702434513601474463939666748046875f,0.86489999512433968220648239366710186004638671875f,0.62619999115467095407439046539366245269775390625f,0.03610000639557853219230310060083866119384765625f,-0.3787999459624291631598680396564304828643798828125f,0.56789996567219436229834172991104423999786376953125f,-1.468499997794627365266251217690296471118927001953125f,0.5506000033631923695764953663456253707408905029296875f,-1.754099994489550784493303581257350742816925048828125f,-0.90120001194179122361305189770064316689968109130859375f,0.603700005459785415951046161353588104248046875f,0.990199977210908688363133478560484945774078369140625f,-1.6615000155046579788375993302906863391399383544921875f,-0.404299999360739514830953567070537246763706207275390625f,-1.0235999777108428787641969393007457256317138671875f,1.11240001725256387743456798489205539226531982421875f,1.342899989700317320284739253111183643341064453125f,-1.1748999915480613509544127737171947956085205078125f,-1.052100026653706610346716843196190893650054931640625f,-0.305000003874301928163959019002504646778106689453125f,0.798100021430850237180720796459354460239410400390625f,-0.6361999987542628076653272728435695171356201171875f,-0.5486999710008502173508304622373543679714202880859375f,1.06209999569654467421742083388380706310272216796875f,-1.47829998361021264230430460884235799312591552734375f,0.097799996596574789720079934340901672840118408203125f,-0.1210000017806887040450192216667346656322479248046875f,-0.78240000816285604212652060596155934035778045654296875f,-0.44550001044571396757731918114586733281612396240234375f,0.16699998500943191714895874611102044582366943359375f,-0.04700001112371687828073163473163731396198272705078125f,0.46349999377131467781509854830801486968994140625f,-0.7636999771475789788865995433297939598560333251953125f,0.0835000202953815051643005062942393124103546142578125f,0.710299997602402921614839215180836617946624755859375f,-0.8875999822631477353951368058915250003337860107421875f,-0.1447999737456442392868893875856883823871612548828125f,-1.14599999865144486221879560616798698902130126953125f,0.0754000130578875715769981979974545538425445556640625f,0.519999994337558835155732595012523233890533447265625f,-0.018700036733597880100887778098694980144500732421875f,-1.093199994734674884711012055049650371074676513671875f,-0.83339998291656358642143231918453238904476165771484375f,-0.30079999549388869439781046821735799312591552734375f,-0.0495999829560520577587112711626105010509490966796875f,0.57649999788403505363021395169198513031005859375f,0.30510000077784049654638920401339419186115264892578125f,-1.153299994413554241390329480054788291454315185546875f,-1.272899958407879505983828494208864867687225341796875f,-0.3100000172853467628186763249686919152736663818359375f,-0.9403999784499408320925795123912394046783447265625f,0.1533999807745214383203347097150981426239013671875f,-0.25849999243020960193462087772786617279052734375f,-1.8668000648140914421446723281405866146087646484375f,-0.8951999811410900065311579965054988861083984375f,1.11900005793571466483626863919198513031005859375f,0.91449998545646593584024230949580669403076171875f,0.3943999950349326155674134497530758380889892578125f,-1.241500015765428344138854299671947956085205078125f,-0.445699963982402636020196950994431972503662109375f,-0.03739998286813506211956337210722267627716064453125f,0.915899998618662447569249707157723605632781982421875f,0.42799995791912071041451781638897955417633056640625f,-0.095499984890222489042344022891484200954437255859375f,1.1325000232458108939681551419198513031005859375f,0.7085000487565995541672236868180334568023681640625f,-1.230600014352798865502336411736905574798583984375f,0.25949998456239686817070833058096468448638916015625f,-0.3048000149846075146342627704143524169921875f,-0.12989997813701581463874390465207397937774658203125f,1.818400028085708441949464031495153903961181640625f,1.0320000119805339267031740746460855007171630859375f,0.15460000910758964209890109486877918243408203125f,1.2261000029683106760103328269906342029571533203125f,-0.472500026226043701171875f,-0.74390000641942055636945951846428215503692626953125f,0.17649998678267031504418582699145190417766571044921875f,-0.6995999819502236594104260802851058542728424072265625f,-0.204500040814279859802127248258329927921295166015625f,-0.292899997746944418253178810118697583675384521484375f,1.2283000315919518907747942648711614310741424560546875f,-0.27009998759031272896891096024774014949798583984375f,0.4079000104501837586923329581622965633869171142578125f,0.0232999999642373722963384352624416351318359375f,-0.02520000010281808755507881869561970233917236328125f,0.58289997405409810227183697861619293689727783203125f,-1.13220004713833333198635955341160297393798828125f,-0.46260000860095029651120057678781449794769287109375f,1.27659999046623706675518405972979962825775146484375f,0.070400007879733994542448272113688290119171142578125f,0.3891999947831035200351834646426141262054443359375f,-0.141100017794966614559371009818278253078460693359375f,-2.06839999489337156290957864257507026195526123046875f,0.79560000049471835126269070315174758434295654296875f,-0.4532000220224257081014229697757400572299957275390625f,-0.669000015318393703722676946199499070644378662109375f,-0.0506000343948602715471452029305510222911834716796875f,-0.209500002741813684536964501603506505489349365234375f,-0.44009999083131534636237347513088025152683258056640625f,1.095900013035535636163331218995153903961181640625f,-0.975399971297382872847947510308586061000823974609375f,-0.01939999741911879738154311780817806720733642578125f,-0.48449997185915716357840210548602044582366943359375f,-0.704400003975629740438080261810682713985443115234375f,0.31160000182837210846997777480282820761203765869140625f,0.09190001008510595426059808232821524143218994140625f,0.17899998581409448661361238919198513031005859375f,-0.059099982605874235019882689812220633029937744140625f,-1.020499985635280548734726835391484200954437255859375f,1.7490000337213278402970217939582653343677520751953125f,-0.44890005574226421458661206997931003570556640625f,-0.5334000587821012828726452426053583621978759765625f,-1.88200001645088121904336730949580669403076171875f,-0.1435999622881407589147784165106713771820068359375f,-1.3712000126123431442692890414036810398101806640625f,-1.52980000753402745061748646548949182033538818359375f,0.6328000077724453831251594237983226776123046875f,0.925500011116266563249155296944081783294677734375f,0.2257999916195867484702830552123486995697021484375f,1.1805000234842299988713421043939888477325439453125f,0.4188999585866923780486104078590869903564453125f,0.41029999585151699648122303187847137451171875f,-0.72760005106925973450415767729282379150390625f,-0.337800011125207078066523536108434200286865234375f,-0.45560000228285790768723018118180334568023681640625f,0.7008000033169985432124349244986660778522491455078125f,-0.19719999565482115855274969362653791904449462890625f,-1.0844000026345248510750707282568328082561492919921875f,-1.785299981099367272463496192358434200286865234375f,0.5742000094234942508819585782475769519805908203125f,0.182899978077411873300661682151257991790771484375f,-0.0948999573126432238723282353021204471588134765625f,1.409499998643994000957491152803413569927215576171875f,0.39009996210932629256973314113565720617771148681640625f,-0.592300031375885271955894495476968586444854736328125f,0.11480000112652766208753973842249251902103424072265625f,-0.13529997577220120863472629935131408274173736572265625f,0.0531999988138676105364766044658608734607696533203125f,-0.2171000189796092882943412405438721179962158203125f,0.321700010019540683092742483495385386049747467041015625f,-0.272200000907480144096695084954262711107730865478515625f,0.6312999704375865661631905823014676570892333984375f,1.358299982567130914645758821279741823673248291015625f,-0.5468999871000648216323725137044675648212432861328125f,-0.57510000427961305291546523221768438816070556640625f,-0.6975999982446430269789061640040017664432525634765625f,0.47669991837739988937983071082271635532379150390625f,-0.73469999498426918904669946641661226749420166015625f,0.16219998611807806554452326963655650615692138671875f,0.03199999160319577651279132624040357768535614013671875f,0.797099981912970267927676104591228067874908447265625f,0.88230000358521909475939537514932453632354736328125f,1.0161000574320560030372462279046885669231414794921875f,-0.60699999350309352674770480007282458245754241943359375f,-0.6229000607788568455447375527000986039638519287109375f,-0.0618999936938287032006655863369815051555633544921875f,0.4365000087618824409219087101519107818603515625f,-0.2693999888509512885548247140832245349884033203125f,1.2746999662995346369598337332718074321746826171875f,-0.4434999639689924944008225793368183076381683349609375f,-0.86200002136826459064877781202085316181182861328125f,0.289199999327957646411135783637291751801967620849609375f,0.9817000004827978187904591322876513004302978515625f,-1.03560001804083601228967381757684051990509033203125f,-0.1874000010848042574451710606808774173259735107421875f,-0.41559994800537791714845070600858889520168304443359375f,-0.6473999802231791278472883277572691440582275390625f,-0.5015000489950180995180062382132746279239654541015625f,-0.431999996632337346369467923068441450595855712890625f,-0.3701000052109362226104849469265900552272796630859375f,-0.7406000089883804093915387056767940521240234375f,1.3500000357255341754125765874050557613372802734375f,0.48420001761913322724240060779266059398651123046875f,0.38089998878389597081195461214520037174224853515625f,0.2929999865293506644547960604541003704071044921875f,0.2981000067159531585758713845279999077320098876953125f,0.62679999746084202882911995402537286281585693359375f,0.007800005723536340696000479510985314846038818359375f,0.586800012660026570898708087042905390262603759765625f,0.2173999896481628724131951457820832729339599609375f,-0.3104000224828720178038565791212022304534912109375f,-1.6619999964088199018164004883146844804286956787109375f,-0.8628000167131428810307625099085271358489990234375f,1.0616999831974502921383418652112595736980438232421875f,0.481800011467933497755211647017858922481536865234375f,0.749199996683001412378644090495072305202484130859375f,0.5767999938845633778328192420303821563720703125f,0.3818000039055939875964895691140554845333099365234375f,0.5541999931812284785337396897375583648681640625f,-1.425200001145899175725162422168068587779998779296875f,-0.4387999930500985357184617896564304828643798828125f,0.1256000026926396007098674090229906141757965087890625f,0.1477000073671337343483855875092558562755584716796875f,0.0128999954372643610422954907335224561393260955810546875f,0.9240000003427264463340407019131816923618316650390625f,0.2688000070944427388752728802501223981380462646484375f,1.1906000066787008062618724579806439578533172607421875f,-0.02410000592619188086729309361544437706470489501953125f,1.183000009849667577555010211654007434844970703125f,-0.4969000056147574451159698583069257438182830810546875f,0.220199979054927386101780939497984945774078369140625f,0.90470001398324928487681972910650074481964111328125f,0.83200003567338054466517860419116914272308349609375f,0.560800003279745329365368888829834759235382080078125f,-0.14699995073676053181088718702085316181182861328125f,0.72979997348487402053507366872509010136127471923828125f,-0.5027999636650097858847630050149746239185333251953125f,0.6218000070720908212251742952503263950347900390625f,1.1012000194668782881990409805439412593841552734375f,-0.5273000136807557414186931055155582726001739501953125f,-0.3159000325560583632977795787155628204345703125f,-0.7672000007957218148391120848827995359897613525390625f,-0.3385000329613683245355559847666881978511810302734375f,0.80369996612072025499173832940869033336639404296875f,0.37730000693797993704947657533921301364898681640625f,0.4429999845921994694464274289202876389026641845703125f,-0.50229998056292490815621931687928736209869384765625f,-0.6910000052079554588857490671216510236263275146484375f,-1.93500005364418115050284541212022304534912109375f,-0.6862999897345900546241637130151502788066864013671875f,0.44609993993640006948453446966595947742462158203125f,-0.1675000160187483100315830597537569701671600341796875f,0.14620001946687732896634770440869033336639404296875f,1.2280999745666980604852369651780463755130767822265625f,0.5467999628901480946296942420303821563720703125f,-0.225599950017035144611554642324335873126983642578125f,-0.188600009822845748885811190120875835418701171875f,-1.78800004937499767976305520278401672840118408203125f,-0.4018000025272367992101862910203635692596435546875f,0.0273999915108082969794622840709052979946136474609375f,-0.37539999506473531454275871510617434978485107421875f,0.15920002942830324688117116238572634756565093994140625f,-0.25579998208284404626056129927746951580047607421875f,-0.073300018888712070719293478759936988353729248046875f,-0.379800003510713413135135851916857063770294189453125f,0.937000009149313140000003841123543679714202880859375f,0.51419998751878726039876710274256765842437744140625f,-0.136499978326260984484719074316672049462795257568359375f,-0.00440000770092030535352023434825241565704345703125f,-1.408000006422400307570796940126456320285797119140625f,-0.32299998176097854951649424037896096706390380859375f,-0.1226000274881722251762994346790947020053863525390625f,-0.393599994027614474845222503063268959522247314453125f,1.0563000409573317650568924364051781594753265380859375f,0.09300001096725463156644764239899814128875732421875f,-1.1380000312700868647652896470390260219573974609375f,-0.39540000430345489945693771005608141422271728515625f,0.5983000067442654223981435279711149632930755615234375f,0.17580000652074811995362324523739516735076904296875f,0.4169000000268219263688251885469071567058563232421875f,-0.93960001730322861845934312441386282444000244140625f,0.76489999877512426795789224343025125563144683837890625f,0.9657000847935677256828057579696178436279296875f,-0.4707000017568467864492731678183190524578094482421875f,-0.8754000315725800618338325875811278820037841796875f,-0.780799989049136389240857170079834759235382080078125f,-1.725900048649311457893418264575302600860595703125f,-0.8070999946534633817663006993825547397136688232421875f,-0.28540002307891843003062604111619293689727783203125f,0.18509998393952853401600577853969298303127288818359375f,1.08440000054836271914382450631819665431976318359375f,0.24379998861700313828038133578957058489322662353515625f,1.1574999665468930221123855517362244427204132080078125f,0.092399993373453614964319058344699442386627197265625f,0.368700029227137182630258394055999815464019775390625f,-0.0339999943450093322638849713257513940334320068359375f,-0.4844000094890592578167343162931501865386962890625f,-0.37229998879581682746930937355500645935535430908203125f,-1.1440000209808349751483547152020037174224853515625f,-0.70289997211694743572962806865689344704151153564453125f,-0.27730002228617589565828893682919442653656005859375f,0.158799983215332118913210024402360431849956512451171875f,1.3084999836385247107273244182579219341278076171875f,0.9191999959006904585834263343713246285915374755859375f,0.97950003135204344317799041164107620716094970703125f,-0.67339998327046657333738721717963926494121551513671875f,-0.48789999715089749088292592205107212066650390625f,0.60499997805804028860876542239566333591938018798828125f,1.099599997931718586841043361346237361431121826171875f,-0.3299000009357928586695152262109331786632537841796875f,-0.244599995100498457389903705916367471218109130859375f,1.07000000227242697548035721410997211933135986328125f,0.395000001788139432079560720012523233890533447265625f,0.204500002853572393490821923478506505489349365234375f,0.361000004462897983170677207453991286456584930419921875f,0.7327000243730845685519170729094184935092926025390625f,0.330599979782103758907396695576608180999755859375f,-0.3351000274881723584030623896978795528411865234375f,0.10719997851848628300786003819666802883148193359375f,-0.60979996622055754329494448029436171054840087890625f,-0.131599990391731314076650960487313568592071533203125f,-0.66170000990778188310059704235754907131195068359375f,-0.031399981236457819022689363919198513031005859375f,0.3837999935716391863849139554076828062534332275390625f,-0.22360000289380532745298069130512885749340057373046875f,-0.7650000005215407128389415447600185871124267578125f,-0.6513999916672705836617751629091799259185791015625f,0.805399978598952515085329650901257991790771484375f,0.325399976587295469698801753111183643341064453125f,-0.4644000165298580729000832434394396841526031494140625f,0.08419998990297283825157137471251189708709716796875f,-0.355499994054436552204379040631465613842010498046875f,-0.44499999225139585945498765795491635799407958984375f,-0.995100019963085724583606861415319144725799560546875f,-0.946200010228157140090843313373625278472900390625f,1.638600017832219979396768394508399069309234619140625f,0.6633999915033574890088630127138458192348480224609375f,-1.2497000345095994333632916095666587352752685546875f,-0.866199994283913810022568213753402233123779296875f,-0.288400006590783408721989644618588499724864959716796875f,-0.031700024920702229991320564295165240764617919921875f,0.206000006027519610984910514162038452923297882080078125f,-0.4058999983765183561246203680639155209064483642578125f,0.6871999984115360060599186908802948892116546630859375f,0.874700038234889287735995822004042565822601318359375f,0.678699974912405412652560698916204273700714111328125f,0.426699978578090277636647442704997956752777099609375f,-0.1289999996721744590644220807007513940334320068359375f,-0.3824000155389313260201333832810632884502410888671875f,0.876999994024634244027538443333469331264495849609375f,0.77230006430745135048709926195442676544189453125f,-0.433099983060360127495869164704345166683197021484375f,-0.061100000323354886067050983911030925810337066650390625f,-0.7462999733060600160428066374151967465877532958984375f,-0.386600033865868386584452309762127697467803955078125f,0.1392999921709299204763965462916530668735504150390625f,0.776800042015313607635107473470270633697509765625f,-0.6903999888435006226927725947462022304534912109375f,-1.897099976325035353141856830916367471218109130859375f,-0.797599999995529440610653182375244796276092529296875f,-0.84270004611015369988535894663073122501373291015625f,-0.50689996843636075940509044812642969191074371337890625f,0.406399998968840492352683213539421558380126953125f,-0.6104000082895157941464958639699034392833709716796875f,0.11950000434368768242876512886141426861286163330078125f,0.0321000058665872334273672095150686800479888916015625f,-1.02360002416521300716567566269077360630035400390625f,0.202799972605705480788174099870957434177398681640625f,0.30679998285770437860264792107045650482177734375f,0.02389999585822211169983120271353982388973236083984375f,0.390600008429586920133402827559621073305606842041015625f,0.7918999810837206165814450287143699824810028076171875f,0.38139998477548398003733609584742225706577301025390625f,0.6545000744909053036479917864198796451091766357421875f,0.2848000042557714550639502704143524169921875f,-0.93260001966506234083453819039277732372283935546875f,-0.134200005549192535880820287275128066539764404296875f,0.65349998617172222470372844327357597649097442626953125f,0.485599994385242705874361490714363753795623779296875f,-0.0088999944239855910410597061854787170886993408203125f,0.326199998009204961135765188373625278472900390625f,-0.24569998719096186601262843396398238837718963623046875f,-0.017399998104572045320992401684634387493133544921875f,-0.990800002534687251909417682327330112457275390625f,-0.69760000398159061063552144332788884639739990234375f,0.8457999922528862679627081888611428439617156982421875f,-0.1700000301003452563008977449499070644378662109375f,-1.1591999829366803620445125488913618028163909912109375f,-0.346799991053342804292469736537896096706390380859375f,-0.2212999993458388547917792266161995939910411834716796875f,-0.23920000033378574499920432572253048419952392578125f,0.8107000190049411525450295812333934009075164794921875f,0.3002000000655649092351495710317976772785186767578125f,-0.397500003464519913176644649865920655429363250732421875f,-0.01680000908374790213883898104541003704071044921875f,0.7001000091969971084182589038391597568988800048828125f,-0.9822999611169098699292590026743710041046142578125f,-0.440699974711239672586771121132187545299530029296875f,-1.62579996049478570085966566693969070911407470703125f,-0.223600000155717071326222367133595980703830718994140625f,-0.6993000145599246319960684559191577136516571044921875f,0.02020000086650242110408726148307323455810546875f,-0.047800000452250746008076021098531782627105712890625f,0.914400016269087512199575940030626952648162841796875f,0.7115000325292355487505346900434233248233795166015625f,0.05169999921619894411861650951323099434375762939453125f,-0.0339000012785195536935134441591799259185791015625f,0.493199993449449480831248138201772235333919525146484375f,-0.23109997063279141826797058456577360630035400390625f,0.16169999489486219790279619701323099434375762939453125f,-0.2563999924123285811816685964004136621952056884765625f,-0.781299973678589054770782240666449069976806640625f,-0.711500023961067373790001511224545538425445556640625f,0.124699988539517025998293320299126207828521728515625f,1.8080999952793117557803270756267011165618896484375f,-0.5915999702751639777176251300261355936527252197265625f,-0.9202000059515229590800799996941350400447845458984375f,0.04039999811947347918561490587308071553707122802734375f,0.5288000008732078871531712138676084578037261962890625f,-0.8109999659806492733338245670893229544162750244140625f,-0.76690000598728591540975685347802937030792236328125f,0.2130000030323863369208226004047901369631290435791015625f,-0.1864000004589556613865397594054229557514190673828125f,-0.74089998058825734528198836414958350360393524169921875f,-1.5956999969512228876311610292759723961353302001953125f,0.2412999911129476426907558561651967465877532958984375f,0.13360000170171282274367285936023108661174774169921875f,0.62660000588893893791464506648480892181396484375f,0.3572000147283078685944701646803878247737884521484375f,-0.3925999945938583035598412607214413583278656005859375f,0.2538000285148616086416950565762817859649658203125f,0.9458999801412224517349613961414434015750885009765625f,0.087999994277954129984209430404007434844970703125f,-1.0406999953866005181879472729633562266826629638671875f,-0.3491999779820442739719510427676141262054443359375f,0.23140002716928720349898185304482467472553253173828125f,-0.4351999739885332729727451805956661701202392578125f,1.221100000956654607620066599338315427303314208984375f,0.7514000155091284938180251629091799259185791015625f,0.44169996226131880145970853845938108861446380615234375f,0.648799987983704085081626544706523418426513671875f,-0.29739996271431434760046386145404540002346038818359375f,-0.22440000859498976382155888131819665431976318359375f,-0.578500016830861607530778201180510222911834716796875f,-0.03259999802112556466227033524774014949798583984375f,0.66320000101178877560670343882520683109760284423828125f,-0.1824000332713122674022088176570832729339599609375f,-0.23119995095878842494840910148923285305500030517578125f,0.0494000025600194181407687210594303905963897705078125f,-1.249800028842687371621877900906838476657867431640625f,-0.0675999968290330599529625033028423786163330078125f,0.9675000050663948858442608980112709105014801025390625f,-0.21800003033876436120408470742404460906982421875f,-1.191700001376867046332108657225035130977630615234375f,0.4416000343874089129059257174958474934101104736328125f,-0.59819996919781004418581460413406603038311004638671875f,-1.104199997465311611222205101512372493743896484375f,0.66029999819844942265945064718835055828094482421875f,-0.224999937340617517378404954797588288784027099609375f,0.616499989911913903739559827954508364200592041015625f,1.1307000223577023945153996464796364307403564453125f,-0.37409999057799581123617826960980892181396484375f,-0.96070000715851744388373845140449702739715576171875f,-0.505599994236230809718790624174289405345916748046875f,0.435399941867590456467951298691332340240478515625f,0.3046999902531506432268315620603971183300018310546875f,0.510199990230798761103869765065610408782958984375f,0.3469000023365020890508958473219536244869232177734375f,-0.90589999619722316737124856445007026195526123046875f,-0.2507999980270863016329485617461614310741424560546875f,-0.654400021409988763565479530370794236660003662109375f,0.3676999993577598768723646571743302047252655029296875f,0.34400002485513692107588212820701301097869873046875f,0.09790000072717652823683920360053889453411102294921875f,1.0865000013113021992694484652020037174224853515625f,-0.307499982602894572636387238162569701671600341796875f,-0.34339999988526026530877288678311742842197418212890625f,0.022700005765259323453619799693115055561065673828125f,-0.939299985651672386666177771985530853271484375f,-0.46909999311119321419027983210980892181396484375f,0.52709998541474334388112765736877918243408203125f,-0.62779998713433737833611303358338773250579833984375f,-0.891700021381676588561049356940202414989471435546875f,-0.7561000158578157215316650763270445168018341064453125f,0.2002000011086462194498380995355546474456787109375f,0.066399998521804892703812583931721746921539306640625f,-0.5584000334501268980602617375552654266357421875f,-0.683799988244473855303340315003879368305206298828125f,-0.222999985150993118576678853060002438724040985107421875f,0.1560999826095999243324996541559812612831592559814453125f,-0.296200010228157228908685283386148512363433837890625f,0.9964999884963037235507954392232932150363922119140625f,0.72859999045133605477531091310083866119384765625f,-0.33289999327659625105724217064562253654003143310546875f,-0.1296000067979095060621830270974896848201751708984375f,-0.215099991017579927898850655765272676944732666015625f,-0.28160002198219302727011381648480892181396484375f,-0.502600002305209958564091721200384199619293212890625f,0.35800003838539140588181908242404460906982421875f,1.5888000110059976588416930098901502788066864013671875f,0.44279998750686644370944122783839702606201171875f,-0.29670000286698339220237130575696937739849090576171875f,0.433200002837180964121444048942066729068756103515625f,0.814100009092688647882596342242322862148284912109375f,0.42779999354183673165863410758902318775653839111328125f,0.61210001994818441506396311524440534412860870361328125f,0.9608000211238856280715481261722743511199951171875f,0.0773999653220174277379328486858867108821868896484375f,0.10399997210502665012654688325710594654083251953125f,-0.990599970319867129120439130929298698902130126953125f,-0.33079999668598158990562296821735799312591552734375f,-0.39600001150369645852578059930237941443920135498046875f,0.00110005437731697242043082951568067073822021484375f,0.509699975687265549595394986681640148162841796875f,1.5343000184714785660844427184201776981353759765625f,-0.629899997769296060567967288079671561717987060546875f,-0.5770000136755408792765820180648006498813629150390625f,-0.046600000151991775865223388564118067733943462371826171875f,0.7588999794855710856467112535028718411922454833984375f,0.2801999881073833353894997344468720257282257080078125f,0.24970003737807200394627216155640780925750732421875f,0.3692000097587702089896311008487828075885772705078125f,-0.06759998159259483774263799205073155462741851806640625f,-0.02039999830573802075406320000183768570423126220703125f,0.5239999933019277023049653507769107818603515625f,-0.049199997279048002241097492515109479427337646484375f,-0.8527999901890748191135571687482297420501708984375f,0.752099981652200266779573212261311709880828857421875f,1.264700012195110190305058495141565799713134765625f,0.2269999976754188963923297706060111522674560546875f,-0.9765999795138837757946248530060984194278717041015625f,0.21629999293833979123746757977642118930816650390625f,-0.088799999867379852958038100041449069976806640625f,0.249700002956390332276015442403149791061878204345703125f,-1.064299981740116418649222396197728812694549560546875f,0.896599971988797594946163371787406504154205322265625f,0.408599972271918687738434528000652790069580078125f,-0.37159999277591715394208904399420134723186492919921875f,-0.792200004148483127863755726139061152935028076171875f,-0.436399984142184393931529484689235687255859375f,-0.125800014865398157581921623204834759235382080078125f,0.1280000190138819160523553364328108727931976318359375f,-0.0634000268936159017130194115452468395233154296875f,-0.7919000108860434483659673787769861519336700439453125f,-0.170600008429587168823360343594686128199100494384765625f,-1.0562999816693363630548674336750991642475128173828125f,-0.4707999807417395654596248277812264859676361083984375f,0.9006999926298855374540153206908144056797027587890625f,-0.132800003600120675173457129858434200286865234375f,0.3378999920099972076314998048474080860614776611328125f,0.1803999750971794213683097041212022304534912109375f,-0.006999982513487434943044718238525092601776123046875f,0.6098000066399578145137638784945011138916015625f,-1.11219998328685765187628931016661226749420166015625f,-0.706400001204013694433569980901665985584259033203125f,0.291400011038780437644390985951758921146392822265625f,0.249200024175644241353211327805183827877044677734375f,0.407799997676908942967344273711205460131168365478515625f,0.08279999298304330235254155923030339181423187255859375f,-0.43119998802542702787121697838301770389080047607421875f,-0.46220001666545851293221858213655650615692138671875f,0.25730003737360274129741810611449182033538818359375f,0.088399986958503262712838477455079555511474609375f,-0.85800002594292157365174489314085803925991058349609375f,-0.2278000155210495591973085538484156131744384765625f,-0.033500003829598556936986142318346537649631500244140625f,0.3736000250220303797732412931509315967559814453125f,0.6064000303357841570317532386980019509792327880859375f,0.16100001485645798471324496858869679272174835205078125f,-0.870000039562583094010506101767532527446746826171875f,1.40390004337429985525886877439916133880615234375f,-0.4047999810472131532179673740756697952747344970703125f,-0.67470002081915725344885004233219660818576812744140625f,0.530099976693839014529885389492847025394439697265625f,0.830300008722394178306558387703262269496917724609375f,-0.28719999245107175056546111591160297393798828125f,0.9957999482572084115616917188162915408611297607421875f,0.5630999775096776627236749845906160771846771240234375f,1.08889998376965468906973910634405910968780517578125f,0.434199982973933362728757856530137360095977783203125f,-0.509899975603818944591694162227213382720947265625f,-0.4148999728471043368926984840072691440582275390625f,-0.7907000317454329074706720348331145942211151123046875f,-0.786699984054267442701302570640109479427337646484375f,-0.675500036001205561575488900416530668735504150390625f,0.8413000109687442940042956252000294625759124755859375f,1.1895000490285465577500190192949958145618438720703125f,0.0072000016339122951958273688433109782636165618896484375f,0.4852999770946804236615435002022422850131988525390625f,-0.612799988810718065934679543715901672840118408203125f,-0.23520002211928403568208523211069405078887939453125f,-0.0974999897554515992936785551137290894985198974609375f,-0.3578000255048277011837853933684527873992919921875f,-0.0991999982848763561804616983863525092601776123046875f,-0.60779999898076031428217902430333197116851806640625f,0.610500004373490501308197053731419146060943603515625f,0.7662000267311934731395695052924565970897674560546875f,0.76600000919401611820802600050228647887706756591796875f,0.1802000723734504816775370272807776927947998046875f,-1.00400002221018080916792314383201301097869873046875f,-0.3210000050961971940211014953092671930789947509765625f,0.52120001529455162181392324782791547477245330810546875f,0.056800014746189564363021418103016912937164306640625f,0.0360999955177307985110246590920723974704742431640625f,-0.02880003149509402504691024660132825374603271484375f,-0.744700002509355751811881418689154088497161865234375f,0.165800008010864363683367628254927694797515869140625f,-0.318399977198243089304696695762686431407928466796875f,-0.34279997737407708058299249387346208095550537109375f,0.5640000046640636366390708644757978618144989013671875f,-0.235400010412931681713644138653762638568878173828125f,-1.4455000379011033739828917532577179372310638427734375f,-0.731400014317035385147391934879124164581298828125f,-0.37419996762573715098909588050446473062038421630859375f,-0.695799984019994877115777853759936988353729248046875f,-0.43800000835955132938437373013584874570369720458984375f,-0.757200002807378869107424179674126207828521728515625f,-0.1365000029131770153956182412002817727625370025634765625f,-0.59460001856982713785981786713819019496440887451171875f,-0.3941000070810318778313785514910705387592315673828125f,0.0094000170886514577972548067918978631496429443359375f,0.03450000121444485312593997150543145835399627685546875f,-0.34380000258684173530809857766143977642059326171875f,0.4624000035434960853564234639634378254413604736328125f,-0.1626000137418508462161526040290482342243194580078125f,0.301100026400387632552479999503702856600284576416015625f,0.123199983763694120852960622869431972503662109375f,0.0567000054746864545762718989863060414791107177734375f,0.29679999544918533871395993628539144992828369140625f,-0.56649999594688404958020555568509735167026519775390625f,-0.1315000631511207274115804466418921947479248046875f,0.7426999985381959046293331994093023240566253662109375f,1.1933000336408614572292208322323858737945556640625f,-0.60169997950941345177255925591452978551387786865234375f,-1.7465999781727783446427793023758567869663238525390625f,-0.5854999960660933755463020133902318775653839111328125f,-0.7399000413179397384055846487171947956085205078125f,0.27469998898655190178175189430476166307926177978515625f,1.5691999802917238060473437144537456333637237548828125f,-0.0204999972581862976994671043939888477325439453125f,0.27919999414980356533533267793245613574981689453125f,-0.4394999873191121064763819958898238837718963623046875f,0.8404000051230198220508782469551078975200653076171875f,0.0306999979197979389056172294658608734607696533203125f,1.5718999756634233211372020377893932163715362548828125f,-0.0779000011369586076170890009962022304534912109375f,1.4687999921932817226633005702751688659191131591796875f,-0.70019999060332771545489549680496565997600555419921875f,-0.8027000233486294433760122046805918216705322265625f,0.72859997230917239452452349723898805677890777587890625f,1.6053000083684911825088192927069030702114105224609375f,0.2710999969333409698180048508220352232456207275390625f,1.412600003646314217320423267665319144725799560546875f,0.012800001290440565782091653090901672840118408203125f,0.4273999962791794615668550250120460987091064453125f,0.7122999842509629697673290138482116162776947021484375f,0.04330000577569048658688188879750669002532958984375f,0.53550002922117723169748160216840915381908416748046875f,0.36580000950098057188597522326745092868804931640625f,0.053300005552172624589957194984890520572662353515625f,0.3972000032544134029421911691315472126007080078125f,-0.0177000195488330458459813598892651498317718505859375f,0.33599998199939751231113405083306133747100830078125f,-0.8988999903261662982600910254404880106449127197265625f,-0.4287999733805651914053669315762817859649658203125f,0.58650000291317727629802902811206877231597900390625f,0.321200009185075519013707889826036989688873291015625f,-0.2184000117316842182191294341464526951313018798828125f,-0.4886000047564511561404287931509315967559814453125f,0.3036000022232532291610596075770445168018341064453125f,-0.030999993026256333905621431767940521240234375f,0.65890002251267443256210754043422639369964599609375f,0.3344000106811524375416411203332245349884033203125f,-0.2402999951437114756203072829521261155605316162109375f,0.145200012470781791851237585433409549295902252197265625f,-1.1484999858178195353275441448204219341278076171875f,-0.8902000002145769030903466045856475830078125f,1.654199965390562798717155601480044424533843994140625f,0.6301999884799125428713750807219184935092926025390625f,0.04950003427639604325349864666350185871124267578125f,0.345199994440376578541673779909615404903888702392578125f,-1.0292999920032916971734948674566112458705902099609375f,-0.1395999711096285178513198843575082719326019287109375f,1.4948999963536857560342241413309238851070404052734375f,0.481399989283084916991128920926712453365325927734375f,-0.4627999920889733775908325696946121752262115478515625f,0.295699976983666346086465637199580669403076171875f,0.93620000534802649827526010994915850460529327392578125f,0.510600042665005471320682772784493863582611083984375f,0.43889998529702445306810432157362811267375946044921875f,-0.326699970829487984502748076920397579669952392578125f,-0.7211999629914764842197882899199612438678741455078125f,-1.32520001615881799494900405989028513431549072265625f,0.523000000685453159121607313863933086395263671875f,0.9066000673562288891815796887385658919811248779296875f,0.021099999876320385538974733208306133747100830078125f,0.39539998545348620240247328183613717555999755859375f,0.70509996233284510935845901258289813995361328125f,1.744599996143578746199409579276107251644134521484375f,0.31670000681579091317274787797941826283931732177734375f,0.519500014960766254290547294658608734607696533203125f,0.75120000098943695565623102083918638527393341064453125f,0.4036000266797843405441881259321235120296478271484375f,0.53949999536573878256007219533785246312618255615234375f,1.1940000181868668516216303032706491649150848388671875f,-0.7399999729916455493139437749050557613372802734375f,0.0366999822475022607903838434140197932720184326171875f,0.9123999850660562405124665019684471189975738525390625f,0.3536000443749129917847540127695538103580474853515625f,0.182499997392296886555840274013462476432323455810546875f,-0.588700016225874644959503712016157805919647216796875f,0.63070000548213689395282699479139409959316253662109375f,1.4124000389337545424695008478011004626750946044921875f,0.300099999213218671112457514027482829988002777099609375f,0.196100012207030882649405612028203904628753662109375f,0.934900014235079179769627444329671561717987060546875f,-0.07920001314878444276246227673254907131195068359375f,-0.28479998551756147140423536257003434002399444580078125f,-0.066000017166138125190855134860612452030181884765625f,-1.541900001554191046437836121185682713985443115234375f,-0.3889999771714212073447924922220408916473388671875f,0.44419997353032225628766127556446008384227752685546875f,-0.279800013829767879602883340339758433401584625244140625f,0.088099971418827804914286616622121073305606842041015625f,0.316999999463558168599774944595992565155029296875f,0.350100012102723123774694613530300557613372802734375f,-0.08979999889135381607729868846945464611053466796875f,0.5292000074863434377903104177676141262054443359375f,0.186399996733665584969230621936731040477752685546875f,-0.95210001719146986065567261903197504580020904541015625f,-0.028399980700015703405370004475116729736328125f,1.201900046928227272502454070490784943103790283203125f,-0.036000006139278895034294691868126392364501953125f,-1.0906000095471739541608258150517940521240234375f,-0.18879997576475116005667587160132825374603271484375f,0.91129999860078125806239768280647695064544677734375f,0.571400013720989363719127140939235687255859375f,0.7419000010699037250816445521195419132709503173828125f,0.584000006079673728009993283194489777088165283203125f,-0.7434000096827746428829186697839759290218353271484375f,-0.226199984858930613906835560555919073522090911865234375f,0.66650000356510263888054623748757876455783843994140625f,-0.164999999701976651778068116982467472553253173828125f,-0.3815999954208730837734719898435287177562713623046875f,0.43169997813105620565465869731269776821136474609375f,-0.29859998955726629876750166658894158899784088134765625f,-0.3588000064611431838557109585963189601898193359375f,-0.6380000041127200649526685083401389420032501220703125f,-1.511700043845177088286391153815202414989471435546875f,-0.8751999842330813184076987454318441450595855712890625f,-0.2269000378571448539588573112268932163715362548828125f,-0.76770000049397324470845660471240989863872528076171875f,-1.40200001121684980631698635988868772983551025390625f,0.2903999884709715928465811884962022304534912109375f,0.557100012385844944873269923846237361431121826171875f,0.666399979932606356669566594064235687255859375f,1.210000010132789238781469975947402417659759521484375f,0.286600007230043252182127844207570888102054595947265625f,-0.520599975907801937324848040589131414890289306640625f,-0.004100002051889983789578764117322862148284912109375f,1.074499992311000795552899944595992565155029296875f,0.334899992851913186342471817624755203723907470703125f,0.054099971169232929923964547924697399139404296875f,0.69460000627636875325521259583183564245700836181640625f,0.82270007293224356459404589259065687656402587890625f,0.462800012056529208592792201670818030834197998046875f,-0.2797999373495578812054418449406512081623077392578125f,0.60130000701993679967927164398133754730224609375f,0.0754000430651016184668833375326357781887054443359375f,0.38559996465742629734307911348878405988216400146484375f,1.5170999608650799217457461054436862468719482421875f,-0.14599999485164882884191683842800557613372802734375f,-0.0382000120013952226827314007095992565155029296875f,-1.2898000078693032666166118360706605017185211181640625f,-0.56499999701976744148623765795491635799407958984375f,0.14460000690966856584651623052195645868778228759765625f,-0.02879999036788927213592614862136542797088623046875f,0.855899978092312441191324978717602789402008056640625f,0.48699999052286191414395943866111338138580322265625f,-0.75089995279759147450704404036514461040496826171875f,-0.158799979043007244428054036688990890979766845703125f,0.344399978028982811650848816498182713985443115234375f,0.124000009842217207012282642608624882996082305908203125f,0.22040000659450897924074297407059930264949798583984375f,0.00160000253617764709446191773167811334133148193359375f,1.045700012373924181474649230949580669403076171875f,0.59700000069290393156506979721598327159881591796875f,-0.35830001141950529319757379198563285171985626220703125f,0.38760001322031012449542686226777732372283935546875f,0.5699000143095853676555861966335214674472808837890625f,0.3471999910354615082042073481716215610504150390625f,0.2960999939903616873237979234545491635799407958984375f,0.286999991901218998169298401990090496838092803955078125f,-0.26800000744685525955190996683086268603801727294921875f,0.379000004589557182299586202134378254413604736328125f,0.5909000016734002880269827073789201676845550537109375f,0.08499999970197702481300439103506505489349365234375f,0.57050000344216822423248913764837197959423065185546875f,0.44279999197721497949942204286344349384307861328125f,-1.0529999750182030826550771962502039968967437744140625f,-0.15139998093843498594424090697430074214935302734375f,0.35149999445676816112182905271765775978565216064453125f,-0.2032000340551134431876789676607586443424224853515625f,-0.3412999926775694969904861864051781594753265380859375f,0.164699992525577432189720639144070446491241455078125f,0.6261999959975479423945898815873079001903533935546875f,0.742900041854382298112113858223892748355865478515625f,-0.1650000007078051833531162628787569701671600341796875f,-1.01719999356865908879399285069666802883148193359375f,0.030800002646446278387060146997100673615932464599609375f,0.1297999914966527501558601898068445734679698944091796875f,-0.442099983645230676376769451962900348007678985595703125f,-0.5737000162072478648411788526573218405246734619140625f,0.304799998258054205546585535557824186980724334716796875f,0.3609000103347004273501852367189712822437286376953125f,0.79319999367296689651851693270145915448665618896484375f,-0.2813999771758906032204095026827417314052581787109375f,0.5061999673873189831141417016624473035335540771484375f,0.18100001209974170279792815563268959522247314453125f,-0.80839996356368093444189071306027472019195556640625f,-1.6373000138483935206323849342879839241504669189453125f,0.137000003710389062216989941589417867362499237060546875f,-0.8916999660238624780816962811513803899288177490234375f,0.5986999709635976163468740196549333631992340087890625f,0.881399981832503787160248975851573050022125244140625f,0.443100000867247434399587291409261524677276611328125f,0.404300020408630444990194519050419330596923828125f,0.78149999158829441814333449656260199844837188720703125f,0.15720004751086236183255095966160297393798828125f,0.08069999449253095757939036047901026904582977294921875f,0.381499983280896959314532068674452602863311767578125f,-1.356400021991133630905324025661684572696685791015625f,0.04140001163482676105331847793422639369964599609375f,0.070000005848706248201551716192625463008880615234375f,0.1302000252485271403202204965054988861083984375f,1.925300010156631547175720697850920259952545166015625f,0.5804000050857658354885870721773244440555572509765625f,-1.3654000151254239714404548067250289022922515869140625f,-0.5257999886393545097007518052123486995697021484375f,-0.0101999878838661128810372247244231402873992919921875f,-0.46100000636279581112120240504737012088298797607421875f,-0.812500049769878440741877057007513940334320068359375f,-0.052200013387203370029965299181640148162841796875f,-0.1534000315874817221839521153015084564685821533203125f,-0.301800006699561862433256465010344982147216796875f,1.2844000178709624737649619419244118034839630126953125f,0.2107999981760979579803461092524230480194091796875f,0.450200002412497912729349991423077881336212158203125f,0.191799984347820640095960698090493679046630859375f,-0.5031999871909615951182104254257865250110626220703125f,0.2884000063300131699861594825051724910736083984375f,-0.7322999984815716789654516105656512081623077392578125f,-0.408999987006188003846318679279647767543792724609375f,0.44729998764097711472231821971945464611053466796875f,0.294200029242038141319426358677446842193603515625f,0.8372999919623136388935336071881465613842010498046875f,-0.0492000052511694097034933292889036238193511962890625f,0.9211999912291763958904766695923171937465667724609375f,0.710799996387958543664353783242404460906982421875f,-0.4276999721258878395246938453055918216705322265625f,-0.681999965041875544358163097058422863483428955078125f,-0.6215999880447984349274292981135658919811248779296875f,-1.29989998439550458186886316980235278606414794921875f,0.3329000008761882067886972436099313199520111083984375f,0.8957000117219988677419451050809584558010101318359375f,0.7530999858170750460573117379681207239627838134765625f,0.471799961884320051552776931202970445156097412109375f,0.292399986518919519085102365352213382720947265625f,1.6598999827191238676249440686660818755626678466796875f,-0.52540000452697255273193377433926798403263092041015625f,-1.03809999628514137981483145267702639102935791015625f,-0.5709000152707097708315586714888922870159149169921875f,-0.6313999968826777831054641865193843841552734375f,-0.48020000583976507613215289893560111522674560546875f,0.19459999331235877662038546986877918243408203125f,0.5878999866828322762302150294999592006206512451171875f,0.8532000006020066251721800654195249080657958984375f,0.676099996635317612714288770803250372409820556640625f,-0.04049993035197285706772163393907248973846435546875f,0.35520000036358823081172886304557323455810546875f,0.22190003467202235043487235088832676410675048828125f,-0.5608000119969245300666216280660592019557952880859375f,-0.786900026774407290730550812440924346446990966796875f,0.5283999793961646762596728876815177500247955322265625f,1.15699998925626257317844647332094609737396240234375f,-0.32659999754428847840159733095788396894931793212890625f,-0.8014000243008136958877685174229554831981658935546875f,0.579099988268315879480496732867322862148284912109375f,-0.055000016875565160834327116390340961515903472900390625f,-1.300900018027424831501548396772705018520355224609375f,-0.15979999203681938269028250942938029766082763671875f,1.47210003566890978987657945253886282444000244140625f,0.5186000139951705278917870600707828998565673828125f,-0.69669999500662083136148794437758624553680419921875f,-0.4696000195384026909550811978988349437713623046875f,-1.5652000062122939372244445621618069708347320556640625f,-0.59099999034404770981154797482304275035858154296875f,0.76079999605268244788902620712178759276866912841796875f,0.1040000163614751560459126267232932150363922119140625f,-0.31050000359117968518063435112708248198032379150390625f,-0.03599998915195445903236759477294981479644775390625f,-0.26479999978542334559961091144941747188568115234375f,-0.5874000004887580672630065237171947956085205078125f,-0.3533999930679796452892560409964062273502349853515625f,-0.245399990296364389763539293198846280574798583984375f,0.952699979837238419833056468633003532886505126953125f,0.52740002380907535783904904747032560408115386962890625f,-0.3387999674946067241165792438550852239131927490234375f,-0.09019996348321457357855024383752606809139251708984375f,0.924399992427229921787557032075710594654083251953125f,0.19659999873638156486777006648480892181396484375f,-0.67490003319680713644856950850225985050201416015625f,-0.085799991060793223685010389090166427195072174072265625f,-0.12819996128156774251038996226270683109760284423828125f,-0.199199968594312881720043151290155947208404541015625f,0.52399998670816427814855842370889149606227874755859375f,-0.41880001868009575360929375165142118930816650390625f,0.548199982534348873031149196322076022624969482421875f,-0.01239996502399431932417428470216691493988037109375f,0.02109999939203255348019183657015673816204071044921875f,0.5075999981701375673992515658028423786163330078125f,0.943999998070299550789741260814480483531951904296875f,1.2884000058829787604963712510652840137481689453125f,-0.6579999883919953607147590446402318775653839111328125f,-0.7791000260055067627718017320148646831512451171875f,-0.44739999322444201634851879134657792747020721435546875f,0.240999977305531576377006786060519516468048095703125f,0.6875999980211255913076229262514971196651458740234375f,0.5774000538349153277550840357434935867786407470703125f,-0.5188999768033626391883217365830205380916595458984375f,-0.01940001306533734037884642020799219608306884765625f,-0.52779999588876957972161108045838773250579833984375f,-0.20670000480413452947914265678264200687408447265625f,-0.378300007116794290151773338948260061442852020263671875f,-0.5484000464513894002038796315900981426239013671875f,0.72009997294992234007082743119099177420139312744140625f,0.391199990335106217997918065520934760570526123046875f,0.80049999323487275315613942439085803925991058349609375f,1.548200027647614884784843525267206132411956787109375f,-0.41929998174011717804177123980480246245861053466796875f,-1.1718000211723145209674612488015554845333099365234375f,0.76959997058808815584285412114695645868778228759765625f,1.248800036151707093523555158753879368305206298828125f,-0.118399999400973232610567720257677137851715087890625f,1.113999958917498478427887675934471189975738525390625f,-0.292099998565018115215252691996283829212188720703125f,-0.57629999319911018318407514016143977642059326171875f,0.0741999823406340741627218449139036238193511962890625f,0.57299999807774992799380697761080227792263031005859375f,-0.8080999993771313949508794394205324351787567138671875f,-0.337399983054399665860501045244745910167694091796875f,1.9480000243782995550390069183777086436748504638671875f,0.8268000218987463512121394160203635692596435546875f,-0.99400001416355376893108086733263917267322540283203125f,0.104600023114681306424245121888816356658935546875f,-0.3548999988496299717866122591658495366573333740234375f,-0.7684000105023383042635032325051724910736083984375f,-0.49780004282742762455882257199846208095550537109375f,-0.23179999448060950584249439998529851436614990234375f,1.828800014768540815879305228008888661861419677734375f,0.7803999917119741613902306198724545538425445556640625f,-0.67709997893273821123472089311690069735050201416015625f,-0.3537999973714349710718352071125991642475128173828125f,-1.379200010690092792486893813475035130977630615234375f,-0.987400022840500213305858778767287731170654296875f,0.4803999825105071952435764615074731409549713134765625f,0.25680001087188752961765203508548438549041748046875f,-0.0103000051647426094092452331096865236759185791015625f,-0.1642000000357626277036615647375583648681640625f,-0.02369999502897257581679468785296194255352020263671875f,-0.43180003203153649593559748609550297260284423828125f,-0.972199976357817519101445213891565799713134765625f,-0.5399999856948856091776178800500929355621337890625f,0.62929999492764476798356554354541003704071044921875f,0.1282000117778778758292901329696178436279296875f,0.8201999883309005756615306381718255579471588134765625f,0.007900030715762174082783531048335134983062744140625f,0.55049998775869612988032031353213824331760406494140625f,0.0984000294640658967892932196264155209064483642578125f,-0.12430000081360337294000828478601761162281036376953125f,0.21989998983442848867753127706237137317657470703125f,0.4869999976009131703591492623672820627689361572265625f,-1.07810000407099693120471783913671970367431640625f,-0.4570999924927949820130379521287977695465087890625f,0.178499968290328325792870600707828998565673828125f,-0.6076999711945656468259358007344417273998260498046875f,-0.311599982643126605097450010362081229686737060546875f,0.0043999919429420497607452489319257438182830810546875f,0.25060004508644329046518350878614000976085662841796875f,0.07940000252276657732863895944319665431976318359375f,-0.39050002605468037142344428502838127315044403076171875f,0.3329000068366525511009967885911464691162109375f,1.37640002959072571542264995514415204524993896484375f,-0.8715999951973556481021887520910240709781646728515625f,-0.50120001231432009802801985642872750759124755859375f,0.44400000965595209923009178965003229677677154541015625f,0.629300039333105853955885322648100554943084716796875f,0.11589999202489853491471194502082653343677520751953125f,1.038000013947486532828179406351409852504730224609375f,-0.829400001405179221336538830655626952648162841796875f,-0.60800005671381995853153057396411895751953125f,-0.0338999996766446987095378062804229557514190673828125f,0.768999961823225053336727796704508364200592041015625f,0.84399999345093956737429152781260199844837188720703125f,0.91300002691149728661912376992404460906982421875f,-1.178500024206936558357483590953052043914794921875f,-0.5836000080347059082441774080507457256317138671875f,0.730100003646314110739012903650291264057159423828125f,0.42660001117885115728967093673418276011943817138671875f,-0.03769998009800890059750599903054535388946533203125f,-0.2921999880552288431090346421115100383758544921875f,-0.1019000188395384753903272212482988834381103515625f,0.2226000239491465748642440303228795528411865234375f,0.9000000262260439143346957280300557613372802734375f,0.469200022089481638687402664800174534320831298828125f,0.5775999778300524578611430115415714681148529052734375f,0.337599979543686146143954829312860965728759765625f,-0.0244999786019324261587826185859739780426025390625f,0.014599997782707152538250738871283829212188720703125f,0.33989999415576466201827088298159651458263397216796875f,0.046599981451034722113035968504846096038818359375f,-0.7996999789282679405033604780328460037708282470703125f,-0.2039999846220013068887055851519107818603515625f,-0.45620000359714030224722591810859739780426025390625f,-0.3063999953925609087690418164129368960857391357421875f,0.94319999486505967656313487168517895042896270751953125f,0.2719999933540819814226097150822170078754425048828125f,-0.76630000996291636194968077688827179372310638427734375f,0.10840001854896552657692154753021895885467529296875f,0.037399993298947664666087575824349187314510345458984375f,-0.48740000019073459469609588268212974071502685546875f,0.91310004329830452007854546536691486835479736328125f,0.15279999831020829592631571358651854097843170166015625f,-1.6506000500410797915407101754681207239627838134765625f,-0.2029000098168849408608593876124359667301177978515625f,0.018900019085407393504283390939235687255859375f,0.4283000128164886621817686318536289036273956298828125f,0.576799989973007853905073716305196285247802734375f,0.7724000038042662108495051143108867108821868896484375f,-0.724900012707710317272358224727213382720947265625f,-0.8269999769628046948355404310859739780426025390625f,0.03470000526607019963876155088655650615692138671875f,0.262000000953674205828036747334408573806285858154296875f,-0.6956999772071839061027276329696178436279296875f,-0.515999962180853088966614450328052043914794921875f,0.7743999922037119176820851862430572509765625f,-0.373499990344047461121590458787977695465087890625f,-0.1150999342441565431727212853729724884033203125f,-1.33239996874928490200318265124224126338958740234375f,0.66529999883174895103365997783839702606201171875f,-0.705800021123886178742168340249918401241302490234375f,0.1004000029623508538634268916212022304534912109375f,1.2794000281900166360316006830544210970401763916015625f,-0.5354000312745572642825209186412394046783447265625f,-0.0893000283807514616540856877691112458705902099609375f,-0.96729997951984358905974659137427806854248046875f,-1.146499998480081661256235747714526951313018798828125f,-0.355799988937377520414884202182292938232421875f,0.1862999996066090346857890835963189601898193359375f,-1.3354999769926081398807582445442676544189453125f,-1.579699961084127579624691861681640148162841796875f,0.544599991226196067373166442848742008209228515625f,0.5372999960601330116105600609444081783294677734375f,0.39230002794861784654045777642750181257724761962890625f,-0.16820000581741323397721998844644986093044281005859375f,-0.0419000389933590344782032843795605003833770751953125f,-0.8783999947071077230020819115452468395233154296875f,-0.50069997184276526791535388838383369147777557373046875f,-0.42380000556707375647391700113075785338878631591796875f,-0.505200011762976686213733046315610408782958984375f,-0.75340000796914097236367524601519107818603515625f,-0.7669000337779525722226026118732988834381103515625f,0.3106000055611133348065777681767940521240234375f,-0.74529997841715722728395121521316468715667724609375f,0.09519998456835754296889717807061970233917236328125f,-0.1147000156223771938357458566315472126007080078125f,0.5555999972164631373061638441868126392364501953125f,0.45320003874897896167794897337444126605987548828125f,0.6723999940812586117999671841971576213836669921875f,-0.87079999415278397378870067768730223178863525390625f,-0.38140000060796719338895854889415204524993896484375f,0.96999998182058266849026040290482342243194580078125f,-0.84960001745224023039781968691386282444000244140625f,-0.24960002222061206111902720294892787933349609375f,0.41260000874996194397681392729282379150390625f,-0.65009995242357287992263081832788884639739990234375f,0.57879999468922616046029361314140260219573974609375f,-0.294799986821413284587833913974463939666748046875f,0.8532000187814237079919621464796364307403564453125f,2.16190002751946419579098801477812230587005615234375f,-0.25619999309182173163890183786861598491668701171875f,-0.150400012797117010876490894588641822338104248046875f,-0.71590000938475117919779222575016319751739501953125f,0.4483000192612411893833268550224602222442626953125f,-1.215499983549118478975969992461614310741424560546875f,-0.0581999891281128611808526329696178436279296875f,-1.133000008061527896785491975606419146060943603515625f,0.67250002332031755258867633529007434844970703125f,-0.40529999369084823701570030607399530708789825439453125f,0.0327000282287599475239403545856475830078125f,-0.74800002869963666540797930792905390262603759765625f,-0.09790000340938576073313015513122081756591796875f,0.117599975520372712622929611825384199619293212890625f,-0.6764999810457226203652680851519107818603515625f,-0.1450999902725218504428994492627680301666259765625f,0.03589998815059658454629243351519107818603515625f,0.01239997336864462340599857270717620849609375f,0.965899961030483922286293818615376949310302734375f,1.2095000167489053932712295136298052966594696044921875f,-1.0961999673128133281352347694337368011474609375f,-0.74629998008608833259813764016143977642059326171875f,-0.1524000043630593381749349646270275115966796875f,-0.92039999864101407212046979111619293689727783203125f,0.424399994289874626929304213263094425201416015625f,0.6297000004231929626286046186578460037708282470703125f,0.033899982428550856639049015939235687255859375f,2.0604000263571737860957000521011650562286376953125f,-0.19090002570152364569366909563541412353515625f,0.22270001600980737777035756153054535388946533203125f,0.5380999934911727677899762056767940521240234375f,-0.54809998857378960934738643118180334568023681640625f,1.14180002994537321825418985099531710147857666015625f,0.06940000174045568570591058232821524143218994140625f,-1.67189999384283982664101131376810371875762939453125f,0.7621999753892421125556211336515843868255615234375f,0.4925999501883975284499683766625821590423583984375f,-0.211999984115362138936689007095992565155029296875f,1.87510003214478526700759175582788884639739990234375f,-0.14100000062584872040361005929298698902130126953125f,0.560899987107515318740524890017695724964141845703125f,-0.435800003093480992077957125729881227016448974609375f,-0.6319000172376636470517041743732988834381103515625f,-0.34999998748302463269510553800500929355621337890625f,-0.63069996692538243809877940293517895042896270751953125f,-0.52560000466704359656233691566740162670612335205078125f,-1.0807000222831966151915139562333934009075164794921875f,0.5983999921739100447126702420064248144626617431640625f,0.2514999820888039749888775986619293689727783203125f,-0.0305999949812889582290154066868126392364501953125f,1.3352000099003316080370495910756289958953857421875f,-0.09339998263716697834979640902020037174224853515625f,0.6141000002264984658495450275950133800506591796875f,-0.45360000058412541790175964706577360630035400390625f,-1.181800018620491332654864891082979738712310791015625f,-0.227999998599290876200029742904007434844970703125f,-0.31849995666742270117310908972285687923431396484375f,-0.24200000125169740528008333058096468448638916015625f,0.2622999637246135051782403024844825267791748046875f,0.09079999996423726571492807124741375446319580078125f,-1.0590999717652798128852964509860612452030181884765625f,-0.823900035327672863871839581406675279140472412109375f,0.80860001101493850228507653810083866119384765625f,1.89509998326897655118727925582788884639739990234375f,-0.30910003390312112969695590436458587646484375f,1.36970002128481826986217129160650074481964111328125f,-1.472299995613099099500686861574649810791015625f,-0.414699995803832877072636620141565799713134765625f,0.7771000037431718965308391489088535308837890625f,-0.89600000151991832098019585828296840190887451171875f,0.53730002392530451515995082445442676544189453125f,-0.749899971431494005713602746254764497280120849609375f,0.23860000386238056080401293002068996429443359375f,-0.623499999210238353697377533535473048686981201171875f,-0.2393999672442674864214495755732059478759765625f,0.65819999084174629633281483620521612465381622314453125f,-0.275900031512975374425877816975116729736328125f,0.9272000069797037280494578226353041827678680419921875f,-0.93820000641346013026122818700969219207763671875f,-0.11879999737143531746141889016143977642059326171875f,0.65249999642372102925946819595992565155029296875f,-0.8890999905407426240344648249447345733642578125f,0.514200027453899366491896216757595539093017578125f,-0.884500000506639683095500004128552973270416259765625f,-0.49049998652935045129197533242404460906982421875f,0.077999990105629191106118014431558549404144287109375f,0.29480002422332773903690394945442676544189453125f,-0.73969999222755422607633590814657509326934814453125f,0.37840000424385067390176118351519107818603515625f,0.628999981194734658629386103712022304534912109375f,0.846599964463711440743054481572471559047698974609375f,-0.156799995225668009624087062547914683818817138671875f,-0.579699995952844826518912668689154088497161865234375f,0.3944000038266182173174456693232059478759765625f,0.31470001282840998069190163732855580747127532958984375f,0.23199999745190147404372282835538499057292938232421875f,-0.94150002818927180925356879015453159809112548828125f,0.0553999791204926594900825875811278820037841796875f,-0.4274999584257608287174434735788963735103607177734375f,0.0427999909341336692847335143596865236759185791015625f,0.874599974536895530263791442848742008209228515625f,-0.4444000063598156913258208078332245349884033203125f,-0.404199983346462232702833716757595539093017578125f,0.2550000120699404959623279864899814128875732421875f,-0.16930000185668436785135781974531710147857666015625f,-0.064199996981024742837007579510100185871124267578125f,1.006199976365268700106980759301222860813140869140625f,-0.834599998080730376415203863871283829212188720703125f,-1.47050000485777854208890857989899814128875732421875f,0.60399998566508283914799903868697583675384521484375f,1.3824999803304667267411787179298698902130126953125f,0.4239999962449072512526981881819665431976318359375f,0.9253000028550619759926121332682669162750244140625f,-0.3909999940693378306377780972979962825775146484375f,0.34609997182488516642706599668599665164947509765625f,-0.9896000080645084295838387333787977695465087890625f,-1.3077000497609383433683660769020207226276397705078125f,0.7597999952405689594314708301681093871593475341796875f,1.6883000466421247409698480623774230480194091796875f,-0.1731999719917773905564217784558422863483428955078125f,0.395699991512298510087930480949580669403076171875f,0.4113999858558177180611892254091799259185791015625f,-0.190000008642673634540187777020037174224853515625f,-0.24710001320540897484789866211940534412860870361328125f,-0.043999995835125371712592823314480483531951904296875f,0.1036000201605260517823126065195538103580474853515625f,-1.030700030851363635520101524889469146728515625f,0.41979997296333326772810323745943605899810791015625f,-0.8768999795377254713457659818232059478759765625f,-1.5520000008046628270363953561172820627689361572265625f,-0.174299986732005862677397090010344982147216796875f,0.0222000014662742106708037681528367102146148681640625f,1.151299974870681808170047588646411895751953125f,1.0597999857783317878556772484444081783294677734375f,-0.07729999985992908051457561668939888477325439453125f,-1.2889999887943266454470858661807142198085784912109375f,0.0148000382304189770366065204143524169921875f,1.112499985247850542435799070517532527446746826171875f,0.02069996469020907170488499104976654052734375f,1.10139999732971194390529490192420780658721923828125f,0.59759995868206061686578323133289813995361328125f,0.231399969911575187353491855901665985584259033203125f,0.02150001680850976981673738919198513031005859375f,-0.7591999784290788255702864262275397777557373046875f,-0.096099968099593979786732234060764312744140625f,0.5938999900653956220963891610153950750827789306640625f,-0.2853000532582399273451301269233226776123046875f,-0.8121000212892888203697339122300036251544952392578125f,0.16389995433986204176335377269424498081207275390625f,0.51739998350143423522240482270717620849609375f,1.0303999734207989202872113310149870812892913818359375f,0.64480001271218068126245270832441747188568115234375f,0.0999000297136607706693922636986826546490192413330078125f,0.4307999911352993205326811221311800181865692138671875f,-0.1101000512368981443245274931541644036769866943359375f,-0.08880000541806243319342684117145836353302001953125f,-0.16439998937249133970084358224994502961635589599609375f,-0.23220000608563429178587966816849075257778167724609375f,-0.40289999305307855337332512135617434978485107421875f,0.42639999464750300006699035293422639369964599609375f,0.114600009852647843899831059388816356658935546875f,0.363799999904632631597678482648916542530059814453125f,0.05270000841021538207087360206060111522674560546875f,0.196200013953447438552757375873625278472900390625f,1.3988999761700633417405015279655344784259796142578125f,-0.8830000170767309253250232359278015792369842529296875f,-2.0714000116348270097432759939692914485931396484375f,-0.232000017195939989278485882095992565155029296875f,1.2327999472737314334835900808684527873992919921875f,-0.44959999212026591219881765937316231429576873779296875f,-0.54999999441206437467144496622495353221893310546875f,-0.06119999473094939190787044935859739780426025390625f,0.0769999473094944431039721166598610579967498779296875f,0.2308000027954577593192198037286289036273956298828125f,-0.0202999907106160559777663365821354091167449951171875f,0.124200004506111127966505591757595539093017578125f,0.52999996379017810799183507697307504713535308837890625f,0.45259994840025929985216635031974874436855316162109375f,0.08839997444152913885773159563541412353515625f,0.784199997577071261645187405520118772983551025390625f,0.707299978141486374028090722276829183101654052734375f,1.02069997569844161944274674169719219207763671875f,-0.2022999930612741081858985126018524169921875f,-0.286700011230260454198059960617683827877044677734375f,-1.161100010418891770314075984060764312744140625f,-0.412499995604157287942825860227458178997039794921875f,-0.368599996933340889881947077810764312744140625f,-0.06220002106130106955816927438718266785144805908203125f,-0.148199996578693315996133605949580669403076171875f,0.402999999791383789471410636906512081623077392578125f,0.0905999955773353349286480806767940521240234375f,1.204699998635053503903691307641565799713134765625f,-0.089699999678135355907215853221714496612548828125f,-1.2643999856472014631236788773094303905963897705078125f,0.04070000894665781743242405354976654052734375f,0.7200999527215958817549790182965807616710662841796875f,-1.3814999792575832771035493351519107818603515625f,-1.101799985316395780188258868292905390262603759765625f,-0.097899978896975881070829927921295166015625f,0.54829998975694171914341268347925506532192230224609375f,-0.05659999814033511711386381648480892181396484375f,0.2059000060319899905181273425114341080188751220703125f,-0.5079000165224076823733412311412394046783447265625f,1.077400003096461222895641185459680855274200439453125f,-0.51240001032352466836528037674725055694580078125f,0.651699995639920093282171364990063011646270751953125f,-0.52629995289146958015180643997155129909515380859375f,0.406800001856684634304883729782886803150177001953125f,0.722999978221953387702569671091623604297637939453125f,0.28660000603795054985312162898480892181396484375f,0.641600012668967156770349902217276394367218017578125f,0.36060000377297374285490150214172899723052978515625f,-1.26629996473789230293505170266143977642059326171875f,0.20340000066757202290546047152020037174224853515625f,1.243599995368719124400058717583306133747100830078125f,-0.159199978131055797092585635255090892314910888671875f,-0.0324000121116636563556312466971576213836669921875f,-0.6309999775290489054668796597979962825775146484375f,-1.246599977874755627027525406447239220142364501953125f,0.846200017678737648196829468361102044582366943359375f,0.166700060534477945139997245860286056995391845703125f,0.874400002634525552736022291355766355991363525390625f,0.924799982351064642216442734934389591217041015625f,-0.4970000121295452544245563331060111522674560546875f,-0.039799993377923836845866389921866357326507568359375f,-0.007000004082918120928979988093487918376922607421875f,-1.1991000078260896088977460749447345733642578125f,-0.33540002203583707540701652760617434978485107421875f,0.093999928519129805071230521207326091825962066650390625f,-0.084599986830353690603345739873475395143032073974609375f,1.090100024209916806938736044685356318950653076171875f,0.19979999680519089366725893341936171054840087890625f,-0.95060002910494834083010573522187769412994384765625f,-0.28600001493096360860590721131302416324615478515625f,0.917399976572394137974697514437139034271240234375f,1.445699966627359334125912937452085316181182861328125f,0.46729997780621079783713867072947323322296142578125f,-0.23330001128911970198487324523739516735076904296875f,0.3125999981701372831821572617627680301666259765625f,-0.71699998396635067354765169511665590107440948486328125f,1.6148999495267872816839371807873249053955078125f,1.3424999843537808175142345135100185871124267578125f,-0.10000000894069671630859375f,-0.14099997618794457565627453732304275035858154296875f,0.8218999990582460668520070612430572509765625f,-0.890699957239628137273257380002178251743316650390625f,-0.0665999802589425371479592286050319671630859375f,0.3819000141084192279095077537931501865386962890625f,0.213199968564509845236898399889469146728515625f,1.673599986428022656781422483618371188640594482421875f,0.0553999995350835661156452260911464691162109375f,-0.249900000488758156080848493729718029499053955078125f,-0.79569997899532296514735207892954349517822265625f,-0.694300013703107676832360084517858922481536865234375f,0.42870000814199471506071859039366245269775390625f,0.758599964076280830482801320613361895084381103515625f,-1.4126999644517894694217829965054988861083984375f,-0.2049999805167319610887943781563080847263336181640625f,-0.96079999098628832143731415271759033203125f,-0.14579999329596748935955474735237658023834228515625f,1.0011999685049062236430472694337368011474609375f,1.3075999841630459030739075387828052043914794921875f,-1.423499998688697587567730806767940521240234375f,-0.087700028079748193476916640065610408782958984375f,0.482600027823448129993266775272786617279052734375f,0.2341999872207640720489507657475769519805908203125f,-1.075699997770787064155229018069803714752197265625f,-0.632200026500225220615902799181640148162841796875f,0.69659995682686493001511962575023062527179718017578125f,-0.20439999019205579156022167808259837329387664794921875f,-0.38349999043717986069168546237051486968994140625f,-0.48680001713037501787084693205542862415313720703125f,-0.9428999999821190414195370976813137531280517578125f,0.7991999848365782810333257657475769519805908203125f,0.322099997708201579715847628904157318174839019775390625f,-0.236000000849366065036605277782655321061611175537109375f,-0.3224000197485088481386128478334285318851470947265625f,-0.4672000197947026034483997136703692376613616943359375f,0.93179999522566792080624509253539144992828369140625f,0.42460002609491365177518673590384423732757568359375f,-1.656199968206882200405516414321027696132659912109375f,0.379599994206428537069086814881302416324615478515625f,0.7350000086426733414413092759787105023860931396484375f,0.5266000005245208281934310434735380113124847412109375f,-0.360799990502000067493781898519955575466156005859375f,0.376599997097253780253822696977294981479644775390625f,0.39529998970478763542502065320149995386600494384765625f,0.97000002026557918810567571199499070644378662109375f,-0.4419999082684515911978451185859739780426025390625f,0.1175999863982202242596031283028423786163330078125f,0.955199965941905926314348107553087174892425537109375f,-0.304799992036819400453850903431884944438934326171875f,0.3769999920129780690558618516661226749420166015625f,-0.697799994063377226893862825818359851837158203125f,0.9725999807357794679774087853729724884033203125f,0.75979998592734343532129059894941747188568115234375f,-0.94419999437332080560736358165740966796875f,-0.97969999528229256835487603893852792680263519287109375f,0.376999991305172255096067601698450744152069091796875f,-0.6691000034116207917378460479085333645343780517578125f,-0.294599991524219007033025263808667659759521484375f,0.3274999532103539667815539360162802040576934814453125f,0.12579996867179943365044891834259033203125f,0.3954999912604690859296852067927829921245574951171875f,-0.58179997298568508767857565544545650482177734375f,-0.283500005431473189876356855165795423090457916259765625f,-0.95729998533129645466033252887427806854248046875f,-1.4148999896109104579267068402259610593318939208984375f,1.453000000834464344734442420303821563720703125f,1.26050000396370887045804920489899814128875732421875f,-0.6556000080943107377606793306767940521240234375f,0.173600001627206790288937554578296840190887451171875f,0.425599973970651745958093670196831226348876953125f,0.8176999873995780543367573045543394982814788818359375f,-0.50260001150667665825721996952779591083526611328125f,0.11180003504902102928753038213471882045269012451171875f,0.4223999554872506223546224646270275115966796875f,1.4995999778151514192359172739088535308837890625f,-0.82699997889995557898146216757595539093017578125f,-0.09979999501705183462263448745943605899810791015625f,-0.20439998356103927790172747336328029632568359375f,-0.6925999971270559996128213242627680301666259765625f,-1.44910003971457523874732942203991115093231201171875f,-0.15920001657605187261879109428264200687408447265625f,0.966099971415102398708540931693278253078460693359375f,-0.4258000136360526877155052716261707246303558349609375f,-0.059899999799580161408130152267403900623321533203125f,-0.4207999871492387189420014692586846649646759033203125f,-0.9270000035613772393361387003096751868724822998046875f,-0.44700000125169747633435690659098327159881591796875f,-1.0919999829232693855374236591160297393798828125f,-0.0516000197470187771386918029747903347015380859375f,-0.187399996018410064380077528767287731170654296875f,0.87860001876354232308585778810083866119384765625f,1.2074999931454666324270874611102044582366943359375f,-0.726599988901615034819769789464771747589111328125f,0.45359998329877893041839342913590371608734130859375f,0.12700000646710396523531017010100185871124267578125f,-1.0703999934628598555974576811422593891620635986328125f,0.0471999971076846602802135066667688079178333282470703125f,-0.2842000108577309447355219163000583648681640625f,-0.554400000399351000623937579803168773651123046875f,-0.297100062453746982527036379906348884105682373046875f,-0.036200005757808906281525196391157805919647216796875f,0.5684000072240829037895082365139387547969818115234375f,-0.3328000143289566725712802508496679365634918212890625f,0.482700005280971566890002577565610408782958984375f,0.40300001409649865991013939492404460906982421875f,-0.49719999304711759435804196982644498348236083984375f,-0.031999997898936261009339432348497211933135986328125f,0.277700031022728222040996115538291633129119873046875f,0.62719998209476468531420323415659368038177490234375f,-1.3977000085592266032108454965054988861083984375f,-1.328199996802210680613143267692066729068756103515625f,-0.424799990509450164921645409776829183101654052734375f,-0.7015999813020232789284591490286402404308319091796875f,0.4835999916434285950117555330507457256317138671875f,-0.6871999854475259184027891024015843868255615234375f,-0.825399972414970761747099459171295166015625f,-0.29580000488162028204897069372236728668212890625f,-1.0002999851226803684767219237983226776123046875f,1.15429998434782010718890887801535427570343017578125f,0.0793999844551080968813039362430572509765625f,0.495400026804208692965403315611183643341064453125f,-0.665199964749813688058566185645759105682373046875f,-1.4520000165998936392242057991097681224346160888671875f,0.491100004756450658760513761080801486968994140625f,0.52990000285431759774468218893161974847316741943359375f,-0.3600999932341277798286682809703052043914794921875f,0.4362000061117112803543705012998543679714202880859375f,-1.18509996464252509440484573133289813995361328125f,0.049800013792514530308608300401829183101654052734375f,-1.2018000236868857655281317420303821563720703125f,-0.393700013506412371810938566341064870357513427734375f,-0.58239999259114227925238083116710186004638671875f,-0.8544000236451625429623391028144396841526031494140625f,0.8803000101566311741407844237983226776123046875f,-0.2819000068068504560869769193232059478759765625f,-0.01949998791515827889497813885100185871124267578125f,-0.111399992859363250463644590126932598650455474853515625f,0.167200040358304935494970777654089033603668212890625f,-0.240599998259544367584794599679298698902130126953125f,0.0484999942928554528265294720767997205257415771484375f,-0.20939998773336387927201940328814089298248291015625f,-1.2567999886691563204976773704402148723602294921875f,0.0025999807357788284889466012828052043914794921875f,1.27939996716976178703362165833823382854461669921875f,0.47420001702308667290708399377763271331787109375f,1.129499995738267870137860882095992565155029296875f,-0.43659998592138293815878569148480892181396484375f,-1.37759999474287031517860668827779591083526611328125f,-0.39900000876188290721557905271765775978565216064453125f,0.535300017085671431260607278090901672840118408203125f,-0.749000021874904664542782484204508364200592041015625f,-0.16410006623864192842887632650672458112239837646484375f,-0.85899999043345454108333569820388220250606536865234375f,-0.73090000029504287315518240575329400599002838134765625f,-0.94100000926852234162112154081114567816257476806640625f,-1.19720002694726002090419569867663085460662841796875f,-0.645200008186697981926727152313105762004852294921875f,0.9504999756142495925814728252589702606201171875f,-0.089999996572732943178607456502504646778106689453125f,-0.8574999731034045336031113038188777863979339599609375f,-0.498200015056133427293616477982141077518463134765625f,-1.2979000186830760288358987963874824345111846923828125f,-0.338400008192658419137188730019261129200458526611328125f,-0.456500045232475226697488324134610593318939208984375f,-0.22060002120733290809084792272187769412994384765625f,-0.54959998660683684335026555345393717288970947265625f,0.354800009471178068309882291941903531551361083984375f,-0.1079999956190587084847720689140260219573974609375f,-0.9318000084131956572974786467966623604297637939453125f,-0.415599956834316941467477590776979923248291015625f,0.9718999881803991325313063498470000922679901123046875f,-0.97039999148845623722081654705107212066650390625f,0.41450000795722008462007579510100185871124267578125f,0.688399988448619382097604102455079555511474609375f,-0.7171999679386618087306715096929110586643218994140625f,0.43860000684261279957354418002068996429443359375f,-0.028199993970990266944909308222122490406036376953125f,-0.8166999980241058665342279709875583648681640625f,-0.9313000321656463853514651418663561344146728515625f,0.1028000277400025197493960149586200714111328125f,-1.354999983534217022906886995770037174224853515625f,-0.000399975022673970670439302921295166015625f,-1.223599991792440278004505671560764312744140625f,-0.2715999740749595048328046686947345733642578125f,-0.08159999985396859667474700472666881978511810302734375f,-0.87129998030960553734303175588138401508331298828125f,-1.785499986231326996488633085391484200954437255859375f,0.7876999994695186302351430640555918216705322265625f,-0.5774000074177980668110876649734564125537872314453125f,0.01160000320673049145625554956495761871337890625f,-0.0703000110134481825951979772071354091167449951171875f,-0.94529998232871292884738068096339702606201171875f,-0.7685999911591412381284271759795956313610076904296875f,-1.1194000208139414098695851862430572509765625f,0.7412000068008897102345144958235323429107666015625f,-0.43130000534355605878999995184130966663360595703125f,0.0940000016093254231464015902020037174224853515625f,-0.4832999959409232104690090636722743511199951171875f,0.33980000470280646851506389793939888477325439453125f,-0.1643999603152277444451101473532617092132568359375f,0.63599998497962939580929742078296840190887451171875f,1.92919997485280081406244789832271635532379150390625f,-0.61319998629689198565984042943455278873443603515625f,-1.0538000067591664077326640835963189601898193359375f,0.51620000158548340341013727083918638527393341064453125f,0.4738000066846606017634258023463189601898193359375f,-0.81100001895427720199904797482304275035858154296875f,-1.392299978923798420282764709554612636566162109375f,-0.13339999947547909187051118351519107818603515625f,-0.5977000133275982562963690725155174732208251953125f,0.15520000379085541197810016456060111522674560546875f,-0.03639998365789620660137870800099335610866546630859375f,0.05780000322759148989604227608651854097843170166015625f,-0.175599997831136167558696570267784409224987030029296875f,-0.89680000506043455743565573357045650482177734375f,-0.9021000221088524284596132929436862468719482421875f,0.44399998633563531047485639646765775978565216064453125f,-0.4147000096432866822482310453779064118862152099609375f,0.29259999958574767209285028002341277897357940673828125f,1.260800023414939463606287972652353346347808837890625f,-0.248599990600347542368808717583306133747100830078125f,-1.8517999982804054237561786067089997231960296630859375f,0.42379999349713326495248111314140260219573974609375f,-0.090500016331672927805129802436567842960357666015625f,-0.80790000437796127830125669788685627281665802001953125f,-0.53359999015331283089835778810083866119384765625f,0.03420002417564393937254862976260483264923095703125f,-0.1109000131845476744274492375552654266357421875f,1.551999956399202407197890352108515799045562744140625f,0.08999996542930688292472041212022304534912109375f,-1.315100011134147539593186593265272676944732666015625f,0.4501000480890269273004378192126750946044921875f,0.121400016030669188893398313666693866252899169921875f,0.904099958875775655542383901774883270263671875f,-1.18079998707473254881961111095733940601348876953125f,-0.0437999885797495380757027305662631988525390625f,-0.669000025004148568541495478712022304534912109375f,-0.01499998569488525390625f,-0.427800003153085750540185472345910966396331787109375f,-0.2207999978780748051576665602624416351318359375f,-0.51099999265372753054492704904987476766109466552734375f,0.274599997521936867173053542501293122768402099609375f,-0.115599995707720726301204194896854460239410400390625f,-0.192099995994567507295869290828704833984375f,-0.58600003795325716904329738099477253854274749755859375f,-0.394399988739192242093167806160636246204376220703125f,0.597000010956078686064074645400978624820709228515625f,-0.522900014846026994774774721008725464344024658203125f,-1.3279000058494510039253100330824963748455047607421875f,0.7497000096619128584052305086515843868255615234375f,0.7158999925464393232488191642914898693561553955078125f,-0.526300008547305964157203561626374721527099609375f,0.714400007402896886077314775320701301097869873046875f,-0.896500036031007585535235193674452602863311767578125f,-0.144400000697374242264459098805673420429229736328125f,0.00659996103644377996033654198981821537017822265625f,-0.0191999983966351095432401052676141262054443359375f,1.659000008612870669111316601629368960857391357421875f,-0.0911999744653702038021947373636066913604736328125f,-0.1312999992713328456517274389625526964664459228515625f,-0.1454000019565224732787100947462022304534912109375f,0.98450000477209653748644768711528740823268890380859375f,0.4399999883770941977445545489899814128875732421875f,-0.125100003087521116640346008352935314178466796875f,-0.31040000936985023827219265513122081756591796875f,-1.193099989169836394609092167229391634464263916015625f,0.080399995511770239176030372618697583675384521484375f,0.8818000210046772480154686491005122661590576171875f,0.123400013929605645301990080042742192745208740234375f,-1.157600035423040463200550220790319144725799560546875f,-0.300200003641843959911739148083142936229705810546875f,0.722200034099817145261113182641565799713134765625f,-0.356399995392560953177962801419198513031005859375f,-1.0615999957561488287183237844146788120269775390625f,0.68079999996423712360638091922737658023834228515625f,0.00550000485777868419745573191903531551361083984375f,0.61179999135136586829730731551535427570343017578125f,0.735600005970895143292409557034261524677276611328125f,-0.26639999487102039577024470418109558522701263427734375f,-1.3836999656923121282403599252575077116489410400390625f,-0.22719999297261228576871872064657509326934814453125f,0.06229999710321454386985351447947323322296142578125f,0.51879999230504036944466861314140260219573974609375f,-0.94309997948408152979027363471686840057373046875f,0.673200034725665918955428423942066729068756103515625f,-0.880500013053417518449350609444081783294677734375f,-0.077299976688623761589269633986987173557281494140625f,0.8749999856948846854720613919198513031005859375f,0.7931000135332346712147000289405696094036102294921875f,-0.208400002865493316761558162397705018520355224609375f,-0.4474000214621423054950355435721576213836669921875f,0.961999975919724192863213829696178436279296875f,-0.52030000300407408531100372783839702606201171875f,-0.02939999458789799291480449028313159942626953125f,0.012599981033801999075194544275291264057159423828125f,-0.436299997520446680709937936626374721527099609375f,-0.481999996334314229073925162083469331264495849609375f,1.0903000110507008457716437987983226776123046875f,0.255300026398896928725434918305836617946624755859375f,-1.0945000066161156837551970966160297393798828125f,-1.776700016874074972150765461265109479427337646484375f,0.54910002019405368400839506648480892181396484375f,0.07809998037815102378544906969182193279266357421875f,0.829199968743324689057772047817707061767578125f,0.43779997260570535644319534185342490673065185546875f,0.526499962866306958630957524292171001434326171875f,-0.69449996556341642151011228634160943329334259033203125f,0.858899991704523468882825909531675279140472412109375f,0.323499998111277664580853752340772189199924468994140625f,-0.477499980032444426569782081060111522674560546875f,0.0467999989509582103863749580341391265392303466796875f,0.8367000197052958387899934677989222109317779541015625f,-0.618199993151426152593330698437057435512542724609375f,0.0030999836564067351218909607268869876861572265625f,-0.170800004732608812219041283242404460906982421875f,-1.2365999795138831185425942749134264886379241943359375f,0.6476000026404860054896062138141132891178131103515625f,1.303099976205825516473169045639224350452423095703125f,-0.134399986690282968737619739840738475322723388671875f,-0.428399969412386116829338789102621376514434814453125f,0.3362000157043338877116411822498776018619537353515625f,0.5135999933384363114186044185771606862545013427734375f,-0.09600000248849394068884066655300557613372802734375f,-0.78500000610947606727307856999686919152736663818359375f,0.44159999761879442647938276422792114317417144775390625f,0.4345000317804517120379159678122960031032562255859375f,0.41560001420378700487390233320184051990509033203125f,-0.78439996389150667965850516338832676410675048828125f,0.64080000085830679523724029422737658023834228515625f,1.010999977491796375517196793225593864917755126953125f,0.24519999581873419547406456331373192369937896728515625f,0.8676999955020843202646574354730546474456787109375f,0.40259999340176566562377047375775873661041259765625f,-0.697499960511923422501467939582653343677520751953125f,-0.083800002586841504381709455628879368305206298828125f,-0.55930000059008566637430703849531710147857666015625f,0.25459998720288279372425677138380706310272216796875f,0.512999993085860861441460656351409852504730224609375f,-0.115799989533424518839410666259936988353729248046875f,-0.15189997700452817497307478333823382854461669921875f,-0.45299999561905857081711701539461500942707061767578125f,-0.19949995833635369990588515065610408782958984375f,-1.29890000463128085783637288841418921947479248046875f,0.34060001018047358911644550971686840057373046875f,1.047099986010790129142833393416367471218109130859375f,0.028500006586313020306988619267940521240234375f,-0.2256999690115455425853951965109445154666900634765625f,0.477099981093406455556760192848742008209228515625f,0.735099995115399362788366488530300557613372802734375f,0.377999996177851738110575752216391265392303466796875f,-0.1998999976575374371012827623417251743376255035400390625f,-0.05619999584853641039217109209857881069183349609375f,-0.0415000091344117638669786174432374536991119384765625f,-0.373500010274350646710672663175500929355621337890625f,-0.173099969034641920639927548108971677720546722412109375f,0.63529999823570193484556511975824832916259765625f,2.025499983847141027837324145366437733173370361328125f,-0.1504000355958936552269733510911464691162109375f,0.48349998356401935328818808557116426527500152587890625f,-0.451500001125037631055647580069489777088165283203125f,-0.5366999838493764496405447061988525092601776123046875f,0.807499963343143889460407081060111522674560546875f,0.1356999746739864232125682974583469331264495849609375f,-0.39599997991323476753677823580801486968994140625f,-0.3317000153839586573667475022375583648681640625f,-0.38809999527931182683460065163671970367431640625f,-0.4885999715268614096430610516108572483062744140625f,0.64579998029470431220033788122236728668212890625f,-0.716400000756979071780961021431721746921539306640625f,0.7835000146329402781475437222979962825775146484375f,0.3656000145018101221694450941868126392364501953125f,1.2376999914228914434488615370355546474456787109375f,0.27500000253319711873700725845992565155029296875f,-1.40090000472813880350031467969529330730438232421875f,0.42260000420510757113135014151339419186115264892578125f,0.66000005831941965794840143644250929355621337890625f,-0.7287999924540518037474612356163561344146728515625f,-1.33210000486075852421663512359373271465301513671875f,-0.508399990758299669124653519247658550739288330078125f,0.907300018076598657756903776316903531551361083984375f,0.27699998456239693922498190659098327159881591796875f,0.72990001524090697415658723912201821804046630859375f,-0.08200001034140580458142721909098327159881591796875f,-0.5345000282227996279971193871460855007171630859375f,0.0577999951064587236260194913484156131744384765625f,-0.74339998308420174311095252051018178462982177734375f,-0.07940001842975608070673843030817806720733642578125f,0.21329998580813391839683390571735799312591552734375f,0.34159999381899819326235956395976245403289794921875f,0.314199994969367679686911287717521190643310546875f,-0.238399984872341264008355210535228252410888671875f,0.586700016278027813854123451164923608303070068359375f,0.42480001140832879258368848240934312343597412109375f,-1.17189996135830920565013002487830817699432373046875f,-0.02359999700784687348686929908581078052520751953125f,-0.0233999965697526679520024117664434015750885009765625f,0.66579999921917920602254525874741375446319580078125f,0.60700002561509602827527487534098327159881591796875f,0.157100001507997522054438377381302416324615478515625f,0.91569998376369454717860207892954349517822265625f,-0.427099997186660917947165216901339590549468994140625f,1.4880999511718755456968210637569427490234375f,0.610300003525614531696419362560845911502838134765625f,-0.446800027672946686863042486947961151599884033203125f,-1.0612999850593507478180299585801549255847930908203125f,-0.0023999525070195204534684307873249053955078125f,0.026000034898519430726082646287977695465087890625f,-0.69649996741116115828162946854718029499053955078125f,-0.1903999956607815846609810250811278820037841796875f,1.19839997384548269110382534563541412353515625f,0.225699970799684734856782597489655017852783203125f,1.0081000066041951868101023137569427490234375f,-0.17790001384019848984507916611619293689727783203125f,-0.1830999893188476335126324556767940521240234375f,-0.2948000282466409771586768329143524169921875f,-0.794599972748756044893525540828704833984375f,-0.45090000677704811238299953402020037174224853515625f,0.6536999720811849101664847694337368011474609375f,0.25870004122257217460401079733856022357940673828125f,0.665999979913234341211136779747903347015380859375f,-0.736299980682134869169885860173963010311126708984375f,1.53759994914531716858618892729282379150390625f,1.173900003290176297099378643906675279140472412109375f,-0.4156000076472761151080703712068498134613037109375f,0.1974999915808439698849952037562616169452667236328125f,-0.22010001351833352600806392729282379150390625f,-0.82920000629425061333677149377763271331787109375f,0.636300013166665845432135029113851487636566162109375f,-0.114199999588727951760347423260100185871124267578125f,-0.92570002325177203061912223347462713718414306640625f,0.304800007385015447880505234934389591217041015625f,-0.74869998385310143618198708281852304935455322265625f,-0.56640000463128092889064646442420780658721923828125f,1.261199979680776461776758878841064870357513427734375f,-0.520399995809793569634393861633725464344024658203125f,-0.297500004619359881274931467487476766109466552734375f,-0.021599996650218944438393009477294981479644775390625f,-0.3775999808102843413593063814914785325527191162109375f,0.21780000836849211509615997783839702606201171875f,0.31219998209476518269411826622672379016876220703125f,-0.59239999482631677807376036071218550205230712890625f,0.24229996938705422593329785740934312343597412109375f,-0.11139999136924760847477955394424498081207275390625f,0.079300029274821159930297653772868216037750244140625f,0.542200021508336238440506349434144794940948486328125f,0.4271000416293737611539427234674803912639617919921875f,-0.2169999845623971079788816496147774159908294677734375f,-1.783699972714483550362274399958550930023193359375f,0.3346000068724155340760262333787977695465087890625f,1.600500021286309415557980173616670072078704833984375f,-0.21040000586807716675963320085429586470127105712890625f,-0.68610000071451116010479154283530078828334808349609375f,-0.07999997362494470909410893000313080847263336181640625f,-0.95719998246729343094330033636651933193206787109375f,-0.5598000118553638770890756859444081783294677734375f,-0.37920000808238985001707987976260483264923095703125f,-0.7109000094592570206941672950051724910736083984375f,0.60610000296831145760734216310083866119384765625f,-0.204400002783537093620225277845747768878936767578125f,-0.3497000089168551539842155762016773223876953125f,0.72790001831054684799937604111619293689727783203125f,-1.2982999835729600590639165602624416351318359375f,-1.12319998763799677732322379597462713718414306640625f,0.48350000077486043892349698580801486968994140625f,-0.96720001547336575953295323415659368038177490234375f,0.3760000083744525767315280972979962825775146484375f,0.5551999946266412688800073738093487918376922607421875f,0.75599998676776891670670011080801486968994140625f,0.137100014024972960413606415386311709880828857421875f,-0.2121999761343005275193718262016773223876953125f,-0.38449996370077155205535746063105762004852294921875f,0.533799971592426203415016061626374721527099609375f,-0.350299987655878108938622972345910966396331787109375f,0.76669999698102397900356663740240037441253662109375f,-0.98879997218847304196742697968147695064544677734375f,0.89789999424517130677259046933613717555999755859375f,1.202599999250471451972543945885263383388519287109375f,-0.2027000151157380969380028545856475830078125f,-1.423000000759958805218730049091391265392303466796875f,0.843100024150312155057918062084354460239410400390625f,-0.175199965289980552629600651926011778414249420166015625f,0.8923999896854155622349935583770275115966796875f,0.31130000444948680016210573739954270422458648681640625f,-0.93389998123645767691414221189916133880615234375f,0.708399969226122028231884542037732899188995361328125f,-0.0731999555259950085428499733097851276397705078125f,0.641399985036253728054589373641647398471832275390625f,0.62630001886635955354876159617560915648937225341796875f,-0.25779999212622650706094873385154642164707183837890625f,-1.236500013306736978080380140454508364200592041015625f,0.860400028294324936695147698628716170787811279296875f,1.19620006476640750037176985642872750759124755859375f,0.41100002610683450399164939881302416324615478515625f,0.3025000111758711085485629155300557613372802734375f,-0.8182000149071217975915715214796364307403564453125f,0.5157999822318559690614847568213008344173431396484375f,-0.1424000024259088892364388811984099447727203369140625f,-1.3031000211328269688948466864530928432941436767578125f,0.9355999969929456039352544394205324351787567138671875f,0.976000007592141560763820962165482342243194580078125f,-0.61540000266432759445933697861619293689727783203125f,-0.3885000208169220314857739140279591083526611328125f,-1.01120002170205136593494898988865315914154052734375f,0.08789997718334152665420333505608141422271728515625f,0.28799999636411666159574451739899814128875732421875f,-0.215799998176097407309725895174778997898101806640625f,-0.211800002527236852500891473027877509593963623046875f,-0.94250002384185815884620751603506505489349365234375f,-0.422800017607212197390254004858434200286865234375f,0.999499960422514988067632657475769519805908203125f,-0.670599989914894223375085857696831226348876953125f,-0.69899999013543112624802233767695724964141845703125f,-0.11439999801516531618972294381819665431976318359375f,-1.250699970501660818200662106391973793506622314453125f,-0.054700014430284493727185690659098327159881591796875f,0.72149995459616267368119224556721746921539306640625f,-0.112499997094273662678887149013462476432323455810546875f,0.81779999309480189140231232158839702606201171875f,0.743300016579031552765854939934797585010528564453125f,-0.04110002681017022041487507522106170654296875f,0.745500030040740835346468884381465613842010498046875f,1.03179998524188931696699000895023345947265625f,0.19909998428225517130840671597979962825775146484375f,-0.4086999834060662806223263032734394073486328125f,-1.17070001922845801090034001390449702739715576171875f,0.9871000232636930604712688364088535308837890625f,1.28839998572915792607318508089520037174224853515625f,0.0401999661281706721638329327106475830078125f,-0.410600010068714660338429212060873396694660186767578125f,-0.5913999667823315320447363774292171001434326171875f,-0.524899986034631726994348355219699442386627197265625f,-0.43169999139308856683783233165740966796875f,-1.0406000489234923378489838796667754650115966796875f,0.73139999494552654368817456997931003570556640625f,-0.07700000035762766259495037957094609737396240234375f,0.1594000193238258589190081693232059478759765625f,0.7718000145971775083353350055404007434844970703125f,-0.97240000436305962239202926866710186004638671875f,-1.245699984657764769480081668007187545299530029296875f,-0.1459999945163730217245756648480892181396484375f,-1.14720000489354134742825408466160297393798828125f,-0.153899987196922438670299015939235687255859375f,0.505499969840049967473305514431558549404144287109375f,0.8545000179409978358080479665659368038177490234375f,-0.56780000896453852732292943983338773250579833984375f,-0.20120002878010279090403855661861598491668701171875f,0.566800007519125959021266680792905390262603759765625f,0.303500020183622876146500857430510222911834716796875f,-0.63720000697970391456692595966160297393798828125f,-0.5490999894976607009766667033545672893524169921875f,0.259800016474724060344669851474463939666748046875f,-0.58089996848106384419452297152020037174224853515625f,0.51980000813007354310002483543939888477325439453125f,-0.706599982494115153031088993884623050689697265625f,0.58999998867511749267578125f,1.034999960660934359424345529987476766109466552734375f,-0.137000005722045958833632539608515799045562744140625f,-0.5838999964356421656930251629091799259185791015625f,-0.33860002815127376862847086158581078052520751953125f,-0.144300007593631818281210144050419330596923828125f,-0.495400000727176603732004878111183643341064453125f,-0.797999997109174241671780691831372678279876708984375f,-0.303600000584125506719601617078296840190887451171875f,-0.7436000091522931843002197638270445168018341064453125f,0.167199993717670469806080291164107620716094970703125f,0.7616000378519292013379526906646788120269775390625f,0.3000000061094760805957548654987476766109466552734375f,-0.318599981808662224835870802053250372409820556640625f,-0.463599998050927997184089690563268959522247314453125f,-1.0313000490039587742074900234001688659191131591796875f,0.2184000081181525221296624295064248144626617431640625f,1.785400021551550420184639733633957803249359130859375f,0.5047000189378858525657278732978738844394683837890625f,0.008999995499849422486704497714526951313018798828125f,0.31970000884234905935699089241097681224346160888671875f,-0.400999997422098797272838055505417287349700927734375f,-0.001199993427097700848804606721387244760990142822265625f,-0.003800003480911140485432042623870074748992919921875f,-0.98660000533014502455131378155783750116825103759765625f,-0.315799980890751175177655341030913405120372772216796875f,-0.70049997442215694576361784129403531551361083984375f,0.6551999766707421457567761535756289958953857421875f,0.8744000136613845342026252183131873607635498046875f,0.19159999516010284281719577847979962825775146484375f,0.8433000622510913313334413032862357795238494873046875f,0.68899996554851572483357813325710594654083251953125f,0.0978000094115738516364899624022655189037322998046875f,-0.11359999209046378609855310060083866119384765625f,-0.8506000323459510514823023186181671917438507080078125f,-0.5191999952673909124456486097187735140323638916015625f,-0.26510000640302899821421078740968368947505950927734375f,0.4267999967157838359099741865065880119800567626953125f,-0.10970000988543038289435571641661226749420166015625f,0.444999987781047767754216692992486059665679931640625f,-0.86310003797113932222373477998189628124237060546875f,0.5111999875783919122795850853435695171356201171875f,-0.889900040125847890948307394864968955516815185546875f,0.7525999539136893901059011113829910755157470703125f,-0.41200002911686883777520051808096468448638916015625f,-0.222999994575977300570457373396493494510650634765625f,-0.18400000973045838037478461046703159809112548828125f,0.40250000484287717927855965172057040035724639892578125f,0.54430001355409596275336525650345720350742340087890625f,0.9499999977648256876960886074812151491641998291015625f,1.02689996761679669390332492184825241565704345703125f,0.98619999838173411088604325414053164422512054443359375f,0.976600006261467701307310562697239220142364501953125f,0.7488000034064052901072727763676084578037261962890625f,0.57170000152587885366983755375258624553680419921875f,0.053399988001584741681426748982630670070648193359375f,0.0577000028967855627115568495355546474456787109375f,0.4782000393450260844474541954696178436279296875f,-0.3532999700129035858253701007924973964691162109375f,0.0117000150114296985748296719975769519805908203125f,-0.186200003671646019398622229346074163913726806640625f,0.1283000032424925773710810972261242568492889404296875f,0.37780000404715519124465572531335055828094482421875f,0.308300018441677092795316639239899814128875732421875f,-0.74299999472498878816395517787896096706390380859375f,-0.111100008258223414969734221813268959522247314453125f,0.38659998592138311579446963150985538959503173828125f,-0.166200036603212364383352905861102044582366943359375f,0.31489997977614425650472185225225985050201416015625f,0.198299988341331445695914226234890520572662353515625f,-0.36039999625682828110484479111619293689727783203125f,-0.860500012308359263357715462916530668735504150390625f,-1.41710000210404363230054514133371412754058837890625f,-0.858700017008185323419411361101083457469940185546875f,0.34919999005198487651568939327262341976165771484375f,0.22719998544752595392992589040659368038177490234375f,-0.6453999907806513558483629822148941457271575927734375f,-0.1005999988555907886933482586755417287349700927734375f,-0.7184000186234718032807222698465920984745025634765625f,0.1193999922037125571705473703332245349884033203125f,-0.660700016844272486338240923942066729068756103515625f,-0.081199998456239708133352905861102044582366943359375f,0.06890002251267457467065469245426356792449951171875f,0.45539999148845655696504763909615576267242431640625f,-0.8006000486254698511601191057707183063030242919921875f,-0.525800000560283198325350895174778997898101806640625f,-1.471800036725401827908399354782886803150177001953125f,-0.10680000357031804725238544051535427570343017578125f,-1.0447000152871008804567054539802484214305877685546875f,-0.5437999747216704093943917541764676570892333984375f,-0.0702000181332234074460529882344417273998260498046875f,-0.8101999768197536422320581550593487918376922607421875f,-0.0793999676540491083898132274043746292591094970703125f,0.7217999841988085307775691035203635692596435546875f,0.16080002324730191531898526591248810291290283203125f,0.6803999980449670914595117210410535335540771484375f,-0.262900014138221838067011049133725464344024658203125f,-0.6643999726831915086933122438495047390460968017578125f,-0.4566999706432219863216914745862595736980438232421875f,-0.40019998039603255080010058009065687656402587890625f,0.9837000532552602205527136902674101293087005615234375f,0.18519999946951859470800627605058252811431884765625f,1.9087000563845040179700163207598961889743804931640625f,0.2595999927163123999207527958787977695465087890625f,0.00780001954436304600903895334340631961822509765625f,0.07389999859631046774666174314916133880615234375f,0.967500004768371280050587301957421004772186279296875f,-0.0916999687433242005596412127488292753696441650390625f,-1.30119996686577810152130041387863457202911376953125f,-1.088000001952052375742141521186567842960357666015625f,-1.3549000140488143273387322551570832729339599609375f,-0.86070001200139500951991067267954349517822265625f,0.35550001037120804170399424037896096706390380859375f,0.1787999917089937440550784231163561344146728515625f,0.989099996054172425630213183467276394367218017578125f,0.8459999776780604552328668432892300188541412353515625f,0.689099967890978160056647539022378623485565185546875f,0.77879998910129077938080399690079502761363983154296875f,0.4155000043362377937228302471339702606201171875f,-0.2771999966233968937245890629128552973270416259765625f,-0.87509998967647550927040356327779591083526611328125f,-0.614499992832541419573999519343487918376922607421875f,-0.910000005513429410797243690467439591884613037109375f,-0.5441000071555375239285012867185287177562713623046875f,0.258899997627735078964406056911684572696685791015625f,0.1999999979138376016152278680237941443920135498046875f,0.9759999952614306639731012182892300188541412353515625f,0.85039998835921271069793192509678192436695098876953125f,0.32429998777508739493669054354541003704071044921875f,0.619500017866492225238062019343487918376922607421875f,0.146100026363134016804679049528203904628753662109375f,-0.0878000011414290515432412576046772301197052001953125f,0.518099991554021865880486075184307992458343505859375f,0.36740000846087905728154510143212974071502685546875f,0.23249997947364986483620441504172049462795257568359375f,-0.66999999091029138753583538345992565155029296875f,0.2772999969169502865184995243907906115055084228515625f,0.13240001300573300824225952965207397937774658203125f,0.7259000265955919406479779354413039982318878173828125f,0.92479997102618227700077113695442676544189453125f,1.24460003067702107415470891282893717288970947265625f,0.216199995028972580257686786353588104248046875f,0.721000019475817932601557913585565984249114990234375f,-0.14459999331235895425606940989382565021514892578125f,-0.578599988737702464192125262343324720859527587890625f,-0.10539999536275868052825899212621152400970458984375f,-0.841799983975290988524875501752831041812896728515625f,0.170999992877244988420670779305510222911834716796875f,-0.2902000210762023524324604295543394982814788818359375f,-0.0603999925315381602786146686412394046783447265625f,0.2056999977335329798933116762782447040081024169921875f,0.49539998478293423289642305462621152400970458984375f,1.0087000005051489637253325781784951686859130859375f,0.187799980652332720865160808898508548736572265625f,0.6283000128537421513641447745612822473049163818359375f,-0.118799995136261049566428482648916542530059814453125f,-1.535300018724799375746670193620957434177398681640625f,-0.262799998682737179223067869315855205059051513671875f,-0.41719999397844065303075922201969660818576812744140625f,0.32499998033046750833818805404007434844970703125f,0.1753999973744158058508446629275567829608917236328125f,-0.4581999819755555591882512089796364307403564453125f,-0.6338999816089867511692546031554229557514190673828125f,-0.4188999968081710267853168261353857815265655517578125f,0.3346999820947649251223765531904064118862152099609375f,-0.29430001511871817232446346679353155195713043212890625f,-0.4654999939799309149890405024052597582340240478515625f,-0.026399988016486186648279499422642402350902557373046875f,0.07230000844672301862914309822372160851955413818359375f,0.388400015568733547599578059816849417984485626220703125f,-0.5335000225305552845611600787378847599029541015625f,-0.0289999975860120162707289637182839214801788330078125f,-0.81359997361302394125459613860584795475006103515625f,0.285900008267164107422786401002667844295501708984375f,0.78579998729825017989014668273739516735076904296875f,0.606700001078844142199386624270118772983551025390625f,0.4262999939441680652407740126363933086395263671875f,-0.34739999192059034527346739196218550205230712890625f,-0.37839998543113484430477910791523754596710205078125f,0.1782000034704804836138691825908608734607696533203125f,0.97189998862743376406569950631819665431976318359375f,-0.226499995946884080666450245189480483531951904296875f,-0.98229997579455385903202113695442676544189453125f,-1.2440999944895505535669144592247903347015380859375f,-0.38450000229477865332228248007595539093017578125f,-0.46560000369846843426557825296185910701751708984375f,0.171900011873244995541654134285636246204376220703125f,0.3564000140190126675321380389505065977573394775390625f,0.66149999400973324981123369070701301097869873046875f,-0.6277000186175107909747339363093487918376922607421875f,-1.0203000041589138646713763591833412647247314453125f,0.075400008699297860204069365863688290119171142578125f,1.119700005526841124492420931346714496612548828125f,0.069600005084275995415055149351246654987335205078125f,0.1699999954923985256982632563449442386627197265625f,-0.156200000691413709574817403336055576801300048828125f,-0.5620000179596245093449624619097448885440826416015625f,-0.407799994100630114335359621691168285906314849853515625f,-1.62560003076270209732001603697426617145538330078125f,-0.046599973404408245869490201584994792938232421875f,-1.15430000066459204077773392782546579837799072265625f,0.32939998847842222318149651982821524143218994140625f,-1.269600028218329068607772569521330296993255615234375f,-0.2801999938070773854548178860568441450595855712890625f,0.017499978728591936150138508310192264616489410400390625f,-0.2041999991416929560728021897375583648681640625f,1.52810000321418026913988796877674758434295654296875f,0.430399978375435143806981841407832689583301544189453125f,0.05589999876767404174415787565521895885467529296875f,0.15199999186396606631888062111102044582366943359375f,0.3349999915435912800631967911613173782825469970703125f,0.18720000429749461545725353062152862548828125f,2.2876000117674468725681435898877680301666259765625f,0.112200012195109888324395797098986804485321044921875f,1.4550000047683706583256935118697583675384521484375f,0.36459997215271044979090220294892787933349609375f,0.2608000066697602203902306428062729537487030029296875f,0.61959998958706830052278746734373271465301513671875f,0.091999965190887156296639659558422863483428955078125f,-0.1361999860137703688423016501474194228649139404296875f,0.196400003327429129029724208521656692028045654296875f,-1.0248999671846628078952790019684471189975738525390625f,-0.7071999640643600315570438397116959095001220703125f,-0.9684999879971147862534053274430334568023681640625f,-0.23570000006183977614426794389146380126476287841796875f,-0.6196999917246401867743088587303645908832550048828125f,-0.609899967501312811890556986327283084392547607421875f,-0.0330999806389213890867040390730835497379302978515625f,-0.3628000098586081634266520268283784389495849609375f,0.52150002321600952281954732825397513806819915771484375f,-0.021100002744793844300374985323287546634674072265625f,-0.1336999969661236686846450538723729550838470458984375f,-0.3179999822080137317215076109278015792369842529296875f,-0.08999998837709444199361996652442030608654022216796875f,-0.231299992156028810796897232648916542530059814453125f,-0.4592999959707262380703696180717088282108306884765625f,-0.68349998602271089254145408631302416324615478515625f,0.0718000105738643146224831070867367088794708251953125f,1.0365999817490578749357155174948275089263916015625f,0.23640001409351807382108745514415204524993896484375f,-1.16339995834827458764948460157029330730438232421875f,-0.31209998690485962757890092689194716513156890869140625f,0.420999988183379247885795848560519516468048095703125f,-0.0390999982893467379341245759860612452030181884765625f,-0.3983999818921091229384501275490038096904754638671875f,-0.73739999348521212407803204769152216613292694091796875f,-0.54210000858604889817371486060437746345996856689453125f,-0.513899994871020115994042498641647398471832275390625f,-0.924199992734193731536151972250081598758697509765625f,-1.25749999962747072146385107771493494510650634765625f,0.81169992713183114485531177706434391438961029052734375f,0.0597999969542026832414194359444081783294677734375f,1.6374999945424495439993961554137058556079864501953125f,-0.2815999924778938012792650624760426580905914306640625f,0.90100002703815740279225110498373396694660186767578125f,-0.06560000097900620108504199379240162670612335205078125f,-1.319500054541975941901910118758678436279296875f,-0.089000004738569060691588674671947956085205078125f,-1.549100044743716875217387496377341449260711669921875f,0.1519999985694884259146419935859739780426025390625f,-0.63930001798719160444051112790475599467754364013671875f,-0.327599992656707694749229631270281970500946044921875f,-0.1404999903291461738064072051201947033405303955078125f,-0.212399993485212323918176480219699442386627197265625f,0.92480002433508612380563818078371696174144744873046875f,0.3809999754428867735356334378593601286411285400390625f,0.251299993199110394126449818941182456910610198974609375f,0.16480000813007311677438337937928736209869384765625f,-0.892200035589933992952182961744256317615509033203125f,0.8033999674379828093151445500552654266357421875f,-0.32740002295225867356975868460722267627716064453125f,0.13299999129772199779608854441903531551361083984375f,0.38139999915510391925721478401101194322109222412109375f,0.4567999730229381771806629330967552959918975830078125f,0.317499967440962704845475172987789846956729888916015625f,-0.0155999988555908242204850466805510222911834716796875f,0.3373000033244488715666875577881000936031341552734375f,0.51179998211264621232885474455542862415313720703125f,0.781199959750473293951245068456046283245086669921875f,0.11270000349283204599259988754056394100189208984375f,-0.3675999800279738938257878544391132891178131103515625f,-0.42069998983591794239345063033397309482097625732421875f,0.2855999980360268697410219829180277884006500244140625f,0.61009999772310286036969273482100106775760650634765625f,1.2260999648213388724116157391108572483062744140625f,0.3719999990165234748928924091160297393798828125f,-0.4893999914586544264238909818232059478759765625f,-0.293900012603402149835574164171703159809112548828125f,0.5661999795317651518189450143836438655853271484375f,0.50009996523857136452306804130785167217254638671875f,0.77539995766282121536505655967630445957183837890625f,-0.01850003199279282029010573751293122768402099609375f,-0.55110001012086851090998607105575501918792724609375f,-0.825499991297721891214678180404007434844970703125f,-1.01559996339082747596194167272187769412994384765625f,-0.170199979501962861405672811088152229785919189453125f,0.08200000289082520765759909409098327159881591796875f,0.386000010088086042969734990037977695465087890625f,0.400900011396407851549383849487639963626861572265625f,0.0990000327527520429526930456631816923618316650390625f,-0.9317999546945099353933983366005122661590576171875f,-0.50199998955428615232676747837103903293609619140625f,-0.66690001056939340884355260641314089298248291015625f,-0.337900000801682143247717249323613941669464111328125f,0.346399997478723520316634676419198513031005859375f,-0.39409998838007442856223860871978104114532470703125f,-1.158199987041950151933633605949580669403076171875f,-0.736499984398484297543063803459517657756805419921875f,1.031900004348158095268672695965506136417388916015625f,0.053799989324808394908217223928659223020076751708984375f,0.578999971807002911106110332184471189975738525390625f,-0.52019999201893785123473890052991919219493865966796875f,0.075899975596368296493210436892695724964141845703125f,-0.92459996261596710809271826292388141155242919921875f,0.5555000041127200471891001143376342952251434326171875f,0.170199994105100671504260390065610408782958984375f,1.2574000238463280965106605435721576213836669921875f,0.0501999929130079181049950420856475830078125f,-0.062199976544082158369519675034098327159881591796875f,0.0353999940216542796633802936412394046783447265625f,-0.49730000451654188253769461880438029766082763671875f,-0.47959999077916126708487354335375130176544189453125f,0.9955000393167143801775864631053991615772247314453125f,-0.126800002229213504278959590010344982147216796875f,-0.21319998875558354711756692267954349517822265625f,0.193599993729591357549679742078296840190887451171875f,-0.75050002311170105340210056965588591992855072021484375f,-0.18299999785423270992623656638897955417633056640625f,-0.547100007766485152416180426371283829212188720703125f,0.69359997256994265768526020110584795475006103515625f,-0.8883000252589585699070084956474602222442626953125f,0.3115999836117031573934355037636123597621917724609375f,0.711499999672174698872595399734564125537872314453125f,-0.12339998919367811680558588705025613307952880859375f,0.5803999942079183238519135556998662650585174560546875f,0.8501999809920786077555021620355546474456787109375f,0.34749996669590410558470239266171120107173919677734375f,0.572800010156631511648583909845910966396331787109375f,0.791699993777275068396193091757595539093017578125f,0.6069000154495238774643439683131873607635498046875f,-0.4073999896109103957542174612171947956085205078125f,-0.192900016149878561488861805628403089940547943115234375f,0.49509997565075780645571512650349177420139312744140625f,0.5727999974906448077405229923897422850131988525390625f,0.4268999951839447248858050443232059478759765625f,-0.12679998978674422716039771330542862415313720703125f,-0.55949997368454962298756072414107620716094970703125f,0.013800028440356049141968242111033760011196136474609375f,-0.224799988050758958735286796581931412220001220703125f,0.129600018420815477071528221131302416324615478515625f,-0.293999987751245583922354853712022304534912109375f,0.10409998010992982386824223794974386692047119140625f,1.0983999787628651034765425720252096652984619140625f,0.379499999836087287263808320858515799045562744140625f,-0.811399989879131311454329988919198513031005859375f,-0.4323999972105025957347379517159424722194671630859375f,-0.10840000379681580255919470801018178462982177734375f,-0.25989999624192716698445337897283025085926055908203125f,-0.674099992813169901495484737097285687923431396484375f,-0.4825000187754628910852261469699442386627197265625f,-0.2320000382065767752237661625258624553680419921875f,-0.0517999934375286130716631305404007434844970703125f,-0.978399959242344241516775582567788660526275634765625f,-0.4673999876737597869436058317660354077816009521484375f,-0.0641999964594841543430447927676141262054443359375f,0.066800003272295072065389831550419330596923828125f,-0.7319000007718796663169769090018235146999359130859375f,-0.63359996616244362854786231764592230319976806640625f,-0.2040999902799727205859880996285937726497650146484375f,0.7965999914348120825025034719146788120269775390625f,0.023099973057954859001483782776631414890289306640625f,-0.037399997769296200456068390849395655095577239990234375f,-0.64360001069828853825782744024763815104961395263671875f,0.0798000078320499905970564213930629193782806396484375f,-0.538200052011013507780035070027224719524383544921875f,-0.45099998617172243342565707280300557613372802734375f,-0.176100001813471518374853985733352601528167724609375f,0.2571999861925842179033452339353971183300018310546875f,-0.962600013108551255669453894370235502719879150390625f,0.452999991446733307753902408876456320285797119140625f,-0.31420001296252031419697914316202513873577117919921875f,-0.96219998045563670530100353062152862548828125f,-0.346199973012506490022133220918476581573486328125f,-0.01759999712705617724850526428781449794769287109375f,0.1179999961405993058605190526577644050121307373046875f,-0.3901999827802182796432362010818906128406524658203125f,-0.045300004047155562059145950115635059773921966552734375f,-0.07639999829828736377379527766606770455837249755859375f,-0.30270000662207596775488127605058252811431884765625f,-0.86519996534585974501396776759065687656402587890625f,-0.0571999493867154651383089003502391278743743896484375f,0.41959998437166223794747565989382565021514892578125f,0.338600006917118889759876765310764312744140625f,0.374799978923797905139281283481977880001068115234375f,0.05039999923706084405949923166190274059772491455078125f,0.210400029858946790994878028868697583675384521484375f,-1.15749998673796650194844914949499070644378662109375f,-0.1210999916061756831453521954244934022426605224609375f,0.38669998623356238720560895671951584517955780029296875f,0.7528999812252819001656689579249359667301177978515625f,0.68659997020065810513500537126674316823482513427734375f,0.5081999818265441337672427835059352219104766845703125f,0.10579998551011104979124866076745092868804931640625f,0.409900000712275502934289761469699442386627197265625f,-0.5213999851107598004773535649292171001434326171875f,-0.72629999886155172106327881920151412487030029296875f,-1.08549997788667695886033470742404460906982421875f,-0.48089998472332950996133149601519107818603515625f,-0.16460000046491618519439725787378847599029541015625f,-0.11990000950396062506797534297220408916473388671875f,0.6916000017166135105384228154434822499752044677734375f,0.55980002444684517914907928570755757391452789306640625f,-0.33459998990371830895895755020319484174251556396484375f,-0.085499988801777504487944270294974558055400848388671875f,-0.67409998666644099785116850398480892181396484375f,-0.678499999955296306808349982020445168018341064453125f,-0.97320001192688909696926202741451561450958251953125f,-0.2260999897807833836083091227919794619083404541015625f,-0.16289999268054966563568086712621152400970458984375f,-0.022000004306435894108062711893580853939056396484375f,0.5902000093042847339575018850155174732208251953125f,0.751400018787383938700941143906675279140472412109375f,0.4135000037923453053423372693941928446292877197265625f,0.49570001554042109059849963159649632871150970458984375f,-1.43900005473196568317462151753716170787811279296875f,0.6911999835550783899407178978435695171356201171875f,-1.0802000364057720016575103727518580853939056396484375f,-0.0263999863028530068476129599730484187602996826171875f,-0.5481000042386359183410604600794613361358642578125f,0.1139999957978725664275998497032560408115386962890625f,-1.0586000014036891503366177857969887554645538330078125f,0.08740000153183924425093209720216691493988037109375f,-1.9677999998375768075931091516395099461078643798828125f,0.10060000585913630999357337714172899723052978515625f,0.1782999997034664385608948578010313212871551513671875f,-0.0500000162422651328597567044198513031005859375f,1.015500040359795352884475505561567842960357666015625f,0.8169999653398993810782258151448331773281097412109375f,-0.459000041209161435684649177346727810800075531005859375f,-0.62639999822378111815623924485407769680023193359375f,-0.690800011941045344343592660152353346347808837890625f,0.09579999530762439707842759162303991615772247314453125f,-0.35469998092129817468531882695970125496387481689453125f,0.841599968785047725106096550007350742816925048828125f,-0.6713000331714749080447290907613933086395263671875f,0.436199993985891154579803696833550930023193359375f,-0.64030002807527797159536930848844349384307861328125f,0.324999995231628258096634453977458178997039794921875f,-0.23890002519488373611267206797492690384387969970703125f,-0.70359999730586952892963381600566208362579345703125f,-0.80670001884847886142182460389449261128902435302734375f,-0.384000002950429519810171541394083760678768157958984375f,0.38470000645816371243057574247359298169612884521484375f,0.1813999981492759872025999356992542743682861328125f,-0.12250000141561023792036166923935525119304656982421875f,0.033299986255168667614867672455147840082645416259765625f,0.273400003945827307916260906495153903961181640625f,0.138900033316016191520247957669198513031005859375f,-1.089399959570169773570569304865784943103790283203125f,-1.1142999953001739843472250868217088282108306884765625f,-0.25589999813437458442422212101519107818603515625f,0.054299977940321042524374206550419330596923828125f,0.957199965554476062123967494699172675609588623046875f,0.65889999054968384495367672570864669978618621826171875f,-0.2393000066250561275182917597703635692596435546875f,0.396600019896030442367873547482304275035858154296875f,0.65270002524852710479308370850048959255218505859375f,0.71829999862313254066492618221673183143138885498046875f,0.92969997278153915232223880593664944171905517578125f,1.04969999252557766311610976117663085460662841796875f,0.7439999908059835576068508089520037174224853515625f,0.404700001913309126422291228664107620716094970703125f,-0.3714999938234686549520802145707421004772186279296875f,0.02829998003393419114814832937554456293582916259765625f,1.1016999737352133337253690115176141262054443359375f,1.21969999721944333259671111591160297393798828125f,0.67929999388456341335995603003539144992828369140625f,0.5213000079512595252850815086276270449161529541015625f,-0.38949999361485243554170665447600185871124267578125f,0.154500006467104267215972868143580853939056396484375f,-0.29340002138018572708233477897010743618011474609375f,-0.319700021508336096331959197414107620716094970703125f,-0.1134999827444553854860487263067625463008880615234375f,0.3947999902486800660739163504331372678279876708984375f,-0.7155000022873283516133824377902783453464508056640625f,-0.1994000132143491921254963017418049275875091552734375f,-0.811000000663101072717608985840342938899993896484375f,0.61579997984766965402059213374741375446319580078125f,1.410700007419288137811008709832094609737396240234375f,-0.0404000023663042551191892925999127328395843505859375f,0.751300033022463509269073256291449069976806640625f,-0.16019999529719353148493610206060111522674560546875f,-0.984300007034838220221217852667905390262603759765625f,0.375399993723630753805764470598660409450531005859375f,-0.3608000220179559125455170942586846649646759033203125f,0.4023999869287013808616393362171947956085205078125f,-0.51260002473145771517692992347292602062225341796875f,0.219999989867210477001435720012523233890533447265625f,-1.1003000098958610752930553644546307623386383056640625f,0.383799991261958961530353917623870074748992919921875f,-0.6819000106811525796501882723532617092132568359375f,0.8139999782145022066970341256819665431976318359375f,-0.39240001583695394149486901369527913630008697509765625f,-0.6007999746322634226913805832737125456333160400390625f,-0.9852000044241548959433885102043859660625457763671875f,-0.007999993532896265691078951931558549404144287109375f,-1.2084000271171333462660868462990038096904754638671875f,0.3815999888271093265501576752285473048686981201171875f,-1.8593000502109529037397805950604379177093505859375f,0.03940001798272074751139371073804795742034912109375f,-0.6362000148102648910963807793450541794300079345703125f,0.2224000002652406493552916799671947956085205078125f,-0.044099997395277146239322973997332155704498291015625f,0.3502000092297794520135312268394045531749725341796875f,-0.224000013381242535359660905669443309307098388671875f,-0.5923999814897775717525973959709517657756805419921875f,-0.97679995931386986995903498609550297260284423828125f,-0.038599981063604626996266233618371188640594482421875f,1.282399995422363048191982670687139034271240234375f,0.50709999942183514587412673790822736918926239013671875f,0.18479998942911635140262660570442676544189453125f,0.86779999816119701705474653863348066806793212890625f,1.191699969637394218580084270797669887542724609375f,1.04709998422265027073763121734373271465301513671875f,0.293300005924701512327601449214853346347808837890625f,0.3797000078737735595524327436578460037708282470703125f,0.045599997067451436549845311674289405345916748046875f,0.585400007432699265308428948628716170787811279296875f,0.4363999905310571580940859348629601299762725830078125f,0.818300013375282464522797454264946281909942626953125f,0.41909999143481257988241850398480892181396484375f,-0.256099995145201597690487460567965172231197357177734375f,-0.31749997528269913171783400684944353997707366943359375f,0.0944000155985353917031943637994118034839630126953125f,-0.63560000394061189066263750646612606942653656005859375f,0.396399995839595786772946439668885432183742523193359375f,1.1895999868303537283509285771287977695465087890625f,0.429600012907385975235996511401026509702205657958984375f,-0.7172999871194363219473189019481651484966278076171875f,-0.22259997730851155406384123125462792813777923583984375f,-1.644000029996037337554071200429461896419525146484375f,-0.34660000231265986059270289842970669269561767578125f,-1.236300014731287699731865359353832900524139404296875f,-0.287199986267089979463662530179135501384735107421875f,0.85719998991787405007158895386965014040470123291015625f,0.172799991083145254577857485855929553508758544921875f,0.48019997994899699111925883698859252035617828369140625f,-0.24460001223683303095413066330365836620330810546875f,-0.0332000140503045937379056340432725846767425537109375f,-0.435199980098009309159579061088152229785919189453125f,-0.5462999988615511171019534231163561344146728515625f,-0.13079997418522903984694494283758103847503662109375f,-1.5332000209793454903461906724260188639163970947265625f,-0.123600005799531675876323788543231785297393798828125f,-1.13370003447979694755076707224361598491668701171875f,-0.364799997401237252603323213406838476657867431640625f,-0.98900001850351693111207396214012987911701202392578125f,-0.382599991129338679041183013396221213042736053466796875f,-0.18770001126006297109682918744510971009731292724609375f,0.69159997742772105766562162898480892181396484375f,-0.82960005688443771987294894643127918243408203125f,-0.473799986754357804752402216763584874570369720458984375f,-0.93530000266879798065389195471652783453464508056640625f,-0.011999995850026701216251012738212011754512786865234375f,-0.48600001833960460029260275405249558389186859130859375f,-0.4943999980151649875637076547718606889247894287109375f,-0.8487000245332716463764199943398125469684600830078125f,-0.108199995982646957060069325962103903293609619140625f,-0.470300002855062349027548407320864498615264892578125f,-0.304399996823072616081162777845747768878936767578125f,-0.754799983692169274007710555451922118663787841796875f,-0.7275000004470346670615299444762058556079864501953125f,-1.0130000120103355154554947148426435887813568115234375f,-0.74450003314018242672744918309035710990428924560546875f,0.34569997646212602404602876049466431140899658203125f,-0.53760001489669051455422277285833843052387237548828125f,-0.407900000149756525313904376162099651992321014404296875f,-0.6021000099457800569524579259450547397136688232421875f,0.451100005799531800221302546560764312744140625f,0.70960000322163097052907687611877918243408203125f,0.6504999848902226489144595689140260219573974609375f,0.56000000156462181877259354223497211933135986328125f,-0.2867999901592732658883733165566809475421905517578125f,0.13670000376105295725182031674194149672985076904296875f,0.7388000015437601319945315481163561344146728515625f,0.381399996063113366773222878691740334033966064453125f,-0.3834000240623947064477761159650981426239013671875f,0.693799992081522987064090557396411895751953125f,1.176799995076656113468516196007840335369110107421875f,0.5517000146389008374825380087713710963726043701171875f,0.1775000078976152195764370844699442386627197265625f,0.052800025877356659975703223608434200286865234375f,0.08020000468492494150041238754056394100189208984375f,0.7556000002712013330352647244581021368503570556640625f,1.1694999577961866510378285965998657047748565673828125f,0.646299971890449409528400792623870074748992919921875f,-0.21549999845027922873441639239899814128875732421875f,-0.1730999962478876330607135969330556690692901611328125f,0.1815000022053718797820920372032560408115386962890625f,0.48979997877478620438296275096945464611053466796875f,0.073600014628470322719522300758399069309234619140625f,0.612599993997811953505561177735216915607452392578125f,1.5485000316575165957289073048741556704044342041015625f,0.20739998826980610147074912674725055694580078125f,1.537200033857673897585982558666728436946868896484375f,0.5047999936014411215268182786530815064907073974609375f,0.9499999951384963114975334974587894976139068603515625f,-0.5725999732851985157822127803228795528411865234375f,-0.522800005537271683664357624365948140621185302734375f,-0.252599983716011333712003761320374906063079833984375f,-1.253900023481250070034320742706768214702606201171875f,-0.035000004023313380230320035479962825775146484375f,0.28560000753551728625012628981494344770908355712890625f,-0.37239999810457202755742400768212974071502685546875f,-0.6329999978542326655173155813827179372310638427734375f,0.25000000119209264681785498396493494510650634765625f,-1.3756000358849769238389626480056904256343841552734375f,-0.43540000147223434368015659856610000133514404296875f,-0.3799000499978664180389387183822691440582275390625f,-0.68699998575448972815138404257595539093017578125f,0.240699951130151301725845769396983087062835693359375f,-0.6455999800801277554995749596855603158473968505859375f,-0.072699995949119244897929092985577881336212158203125f,-0.395999971233308822871066467996570281684398651123046875f,-0.47739996928945205301175747081288136541843414306640625f,0.2875999841630461073549440698116086423397064208984375f,0.041300000686943560967989697019220329821109771728515625f,0.079200003314018163624865564997890032827854156494140625f,0.32230000663995728160671205841936171054840087890625f,0.387599999660253491384764856775291264057159423828125f,0.77719999938011152806893733213655650615692138671875f,0.0490000105500218996468220211681909859180450439453125f,-0.75149996942281749312542160623706877231597900390625f,-0.203100004145503110208892394439317286014556884765625f,0.5405999677121642310595461822231300175189971923828125f,-0.14790001424998033829893984147929586470127105712890625f,-0.28339999215528377529693671021959744393825531005859375f,-0.192899995008855962286276053418987430632114410400390625f,0.214399998126924007380011971690692007541656494140625f,0.115200002636015430557137051437166519463062286376953125f,-0.3596999990262090829418184512178413569927215576171875f,0.223499991238117257097428591805510222911834716796875f,0.32699998128414176079559183563105762004852294921875f,-0.465700004178285720257690627477131783962249755859375f,-1.21919997708797467339536524377763271331787109375f,-1.097399979329109154235766254714690148830413818359375f,-0.55149998702481395351782111902139149606227874755859375f,-0.096099999690055615264583366297301836311817169189453125f,0.5262999765098097526561105041764676570892333984375f,0.592399994528293394324691689689643681049346923828125f,0.000900010502338233209229656495153903961181640625f,0.257900018608570036349192378111183643341064453125f,-0.178399984723329740887720618047751486301422119140625f,0.5560000017434358898782420510542578995227813720703125f,0.681499966740608531523548663244582712650299072265625f,0.5820999790072443058619455769076012074947357177734375f,-0.149599979826807949478961745626293122768402099609375f,0.409799976241588925773839946486987173557281494140625f,-0.66790002136528470177978533683926798403263092041015625f,-0.385599992299079818991458523669280111789703369140625f,-0.06949998977780313946794876756030134856700897216796875f,-0.1008000027954576438560252427123486995697021484375f,0.7071000174894928047564235384925268590450286865234375f,-0.128199995535612121244639638462103903293609619140625f,0.810600008969753815790681983344256877899169921875f,0.01979998786449466141590391998761333525180816650390625f,0.34020000049397369767945065177627839148044586181640625f,-0.114799997699260691863543115687207318842411041259765625f,-1.08610003092661511203687041415832936763763427734375f,-0.417599988447129699675741676401230506598949432373046875f,-0.49669997256174636124370636025560088455677032470703125f,-0.576199976998567731101275057881139218807220458984375f,1.0242000282362102581146245938725769519805908203125f,0.29199998486042044731902933563105762004852294921875f,0.8420999781318008192698698621825315058231353759765625f,0.002399997694790434454858996105031110346317291259765625f,-0.07080001093521681720943661275669001042842864990234375f,-0.6745999602317815924124033699627034366130828857421875f,-0.7181999952375892615208385905134491622447967529296875f,-0.084000005334615490681926530669443309307098388671875f,-1.386900028599798329054237910895608365535736083984375f,-0.35559998961687089291672236868180334568023681640625f,-0.454500006318092619750359517638571560382843017578125f,-0.21199998858571067472666982212103903293609619140625f,-0.227100012646615834199792516301386058330535888671875f,-0.31769999783039093443903766456060111522674560546875f,0.02290000147223469895152447861619293689727783203125f,-0.219500010266899803212936603813432157039642333984375f,1.06949997174739852567881825962103903293609619140625f,0.344799988013506253370366039234795607626438140869140625f,-0.41880002465918619858342708539566956460475921630859375f,0.8834999910891057073314414083142764866352081298828125f,0.924499990075826616475751507095992565155029296875f,0.619799985033273603818315677926875650882720947265625f,-0.2871000045999883010239273062325082719326019287109375f,-0.52810001118630156557998134303488768637180328369140625f,0.36950000810623162550427878159098327159881591796875f,-0.159500003486872099500715194153599441051483154296875f,0.646100017422437389313927269540727138519287109375f,0.0818999967485662949684410705231130123138427734375f,0.09520000334382050510839690105058252811431884765625f,0.502200020018219728257236056379042565822601318359375f,-0.2024999722838405347147272550500929355621337890625f,0.27490002105236044371849857270717620849609375f,0.342599997425079294544048025272786617279052734375f,0.356899988627433994992088628350757062435150146484375f,0.4972999589562421274280268335132859647274017333984375f,-0.01160005185902079904991524017532356083393096923828125f,1.05049995419383090933251878595910966396331787109375f,0.4015999825686218382969627782586030662059783935546875f,-0.47580000734031180353866830046172253787517547607421875f,-0.66620001007914542157095638685859739780426025390625f,-0.5859999943673610545147312222979962825775146484375f,0.515800005179643417108081848709844052791595458984375f,0.4065999916210767395341463270597159862518310546875f,-0.062800003153085537377364744315855205059051513671875f,1.604399984864890438274187545175664126873016357421875f,-0.71379997189044974703620027867145836353302001953125f,-0.4441000292651360314977182497386820614337921142578125f,-0.368599981622398098490833717733039520680904388427734375f,-1.1764999718256294869433986605145037174224853515625f,0.04859999164342898581026020110584795475006103515625f,-0.81359995748251634495318285189568996429443359375f,0.630799979549646394616502220742404460906982421875f,1.241400032719970791816876953816972672939300537109375f,-0.15319997527003348380958414054475724697113037109375f,1.9554000371977686700830645349924452602863311767578125f,-0.04239999288916607156352256424725055694580078125f,0.02630000098496676486092837876640260219573974609375f,-0.2647999914407730415177866234444081783294677734375f,-1.63330001553595050012290812446735799312591552734375f,-0.0820000019967555004996029310859739780426025390625f,-0.723100038976967507409199242829345166683197021484375f,-0.48299998161196722179511198191903531551361083984375f,0.8481000219151375585369123655254952609539031982421875f,0.149400008296966202436806270270608365535736083984375f,1.0066000061869619930376984484610147774219512939453125f,-0.87219996033906976862226656521670520305633544921875f,0.3345000242739912810208124938071705400943756103515625f,0.3063999944984912016110456534079276025295257568359375f,-0.1854000084757803090695915670949034392833709716796875f,-0.4155999854445457941665154066868126392364501953125f,-1.178100005523860627221210961579345166683197021484375f,0.404500012874603243062665569595992565155029296875f,0.617299998593330290219682865426875650882720947265625f,0.2513999924123286877630789604154415428638458251953125f,-0.1173000130474566304883410339243710041046142578125f,0.4305000175982716115186121896840631961822509765625f,-0.77519999753236756845353738754056394100189208984375f,-0.20890000954866394522468908689916133880615234375f,0.709899983650446042560133719234727323055267333984375f,0.3536999867588281887265111436136066913604736328125f,0.67839999612271772999605445875204168260097503662109375f,0.359800010514259305249851195185328833758831024169921875f,0.10540000862479192988985232659615576267242431640625f,-0.032099994131922660045574957621283829212188720703125f,0.18849999526143079719986417330801486968994140625f,-0.116600024589895934212790962192229926586151123046875f,0.3504999649226669955481838769628666341304779052734375f,-0.12290004550516597792153561385930515825748443603515625f,0.4076999790921809196930780672118999063968658447265625f,1.040399974910914959735919183003716170787811279296875f,1.3620999619826676951817034932901151478290557861328125f,0.44399999669194245566217205123393796384334564208984375f,-0.734499992907047261070374588598497211933135986328125f,0.072600002342462610638307296540006063878536224365234375f,1.2952999972857532906544975048745982348918914794921875f,1.0046999753892418283385268296115100383758544921875f,0.737999964550137832475229515694081783294677734375f,0.3511999944329264877751484164036810398101806640625f,-0.301999995142221422383954632095992565155029296875f,-0.88099999503791348587355969357304275035858154296875f,-0.649200028161704967288869738695211708545684814453125f,-0.16899999624490735783410855219699442386627197265625f,-0.860200031879544635415868469863198697566986083984375f,-0.46599999257922153361732853227294981479644775390625f,-1.0388999925240873789533679882879368960857391357421875f,0.9037999744236469013003443251363933086395263671875f,0.27469998965710384908334162901155650615692138671875f,0.224799994271993597294567734934389591217041015625f,-0.2851000279352071675731394861941225826740264892578125f,-0.4695999940574167030860053273499943315982818603515625f,0.47599999533593628786576346101355738937854766845703125f,-0.025800008755922032577245772699825465679168701171875f,0.92490001788586384545709506710409186780452728271484375f,0.50379998232126244062101250165142118930816650390625f,0.6136000077366829952296711780945770442485809326171875f,0.060800010842084528661644071689806878566741943359375f,0.37220002411603936298689632167224772274494171142578125f,-0.7633999786138534471291450245189480483531951904296875f,-0.5129999763220549624520572251640260219573974609375f,-0.63779998594224440733313485907274298369884490966796875f,-0.1196000042274596442126721740351058542728424072265625f,0.189799988758564186497324044466949999332427978515625f,0.09660000706240590151452352074556984007358551025390625f,-0.6063999782934785809374034215579740703105926513671875f,-0.05249997345730694764398549523320980370044708251953125f,-0.195199998873472058136258056038059294223785400390625f,-0.1555999820917841613976406733854673802852630615234375f,0.597599986696243146155893555260263383388519287109375f,0.19199997957050796149047755534411408007144927978515625f,0.2673999932616950570007929854909889400005340576171875f,0.1784999971613288050065193601767532527446746826171875f,0.5669999983087181671947973882197402417659759521484375f,0.864499978676438463054410021868534386157989501953125f,-0.2655000330209731362884895133902318775653839111328125f,-0.69740002164840664278244730667211115360260009765625f,-0.355500016480684077890828120871447026729583740234375f,0.15229998875856409767948207445442676544189453125f,0.107500012814998768817531527020037174224853515625f,0.23040001428723311249768812558613717555999755859375f,0.1078000106781720202064889235771261155605316162109375f,0.94350001418590512258788294275291264057159423828125f,-0.369299996641278494013960198572021909058094024658203125f,-0.92259998434931045352414002991281449794769287109375f,0.140600017407536359570485728909261524677276611328125f,0.353500002488493780816725120530463755130767822265625f,0.7593000222712757985021880813292227685451507568359375f,0.28129999707341202253019218915142118930816650390625f,-0.215700007829070283804639984737150371074676513671875f,0.507600015008449201303619702230207622051239013671875f,0.5920000206232070016909574405872263014316558837890625f,0.412700004684924959263980781543068587779998779296875f,0.0937000090360642357012466163723729550838470458984375f,-0.75569999851584412908778176642954349517822265625f,-0.1556000248581170808392926119267940521240234375f,0.57939996582865749275015332386828958988189697265625f,0.589499981954693996755167972878552973270416259765625f,0.433899991816282248890956907416693866252899169921875f,0.2689000200539826135326393341529183089733123779296875f,-0.5907000133797526419954238008358515799045562744140625f,-0.6881999646902088318967116720159538090229034423828125f,0.526000004239379759241046485840342938899993896484375f,-0.119999994933605247382502057007513940334320068359375f,0.1142999954119321426304622946190647780895233154296875f,-0.47699999335408183043227836606092751026153564453125f,1.2681000525370242915101925973431207239627838134765625f,-0.19720000221133204831858165562152862548828125f,0.93730000958293668844589774380438029766082763671875f,0.6687999748706818792243211646564304828643798828125f,0.32589997317492969131080826628021895885467529296875f,-0.27159999530911438370139876496978104114532470703125f,0.5958999980598689294453151887864805757999420166015625f,-0.3107999878197909016108724244986660778522491455078125f,-0.3943000015214088715964635412092320621013641357421875f,0.18039998448491123550496695315814577043056488037109375f,-0.7262000139161941536514177641947753727436065673828125f,0.86639998913407278990916893235407769680023193359375f,0.0898999427095048542213362452457658946514129638671875f,-0.53879999305009818044709390960633754730224609375f,-0.1416999998122452364412993119913153350353240966796875f,-0.051999999314546574424866776098497211933135986328125f,-0.08419998256415095472249277008813805878162384033203125f,0.851799992394446814358843766967765986919403076171875f,-0.18270001310408179318045540640014223754405975341796875f,0.2473999931871890822776549612171947956085205078125f,-1.3907000305160881481469914433546364307403564453125f,-0.359999995380639870035111016477458178997039794921875f,-1.201700020934641344183546607382595539093017578125f,0.0031000025063752101317504639155231416225433349609375f,0.4528000135838983197800189373083412647247314453125f,-0.279100008869171301029155074502341449260711669921875f,0.0858999979481100961464790088939480483531951904296875f,0.81770001299232231151137284541619010269641876220703125f,0.94850000569224324209471888025291264057159423828125f,1.017300002691149618527788334176875650882720947265625f,0.9580999934911726967357026296667754650115966796875f,0.517699994552135223102595773525536060333251953125f,0.365200005877017730426814523525536060333251953125f,-0.32449996347725384993054831284098327159881591796875f,-1.01559996339082747596194167272187769412994384765625f,-0.56919998140931138408404876827262341976165771484375f,-0.001100002223253238042843804578296840190887451171875f,0.6097000241160392963735148441628552973270416259765625f,0.08810002061128585637561627663671970367431640625f,1.2180000215470787594540524878539144992828369140625f,0.98420000137686702856854026322253048419952392578125f,1.0907000128954649209589433667133562266826629638671875f,0.59999999724328512140658631324185989797115325927734375f,0.210999992877244968436656336052692495286464691162109375f,-0.03440000419914712725955041605629958212375640869140625f,0.268199989277124239972494024186744354665279388427734375f,0.8469999951422213513296810560859739780426025390625f,0.515900004094838937618305862997658550739288330078125f,0.345300009411573238793380369315855205059051513671875f,-0.53089999850690372529271598978084512054920196533203125f,-1.1367999942198394602854705226491205394268035888671875f,-0.13849998080730419047768009477294981479644775390625f,0.1808000156104565547821039217524230480194091796875f,-0.4314000113368032796046236398979090154170989990234375f,0.34440001585930613003938560723327100276947021484375f,0.2396000056058167348993492851150222122669219970703125f,-0.7073000279113654809037825543782673776149749755859375f,0.52520000706911107446472897208877839148044586181640625f,-0.086200047257542156131648880545981228351593017578125f,-0.76679996147453832744389501385740004479885101318359375f,-0.3999999883398415168045403333962894976139068603515625f,0.521999984040856457312429483863525092601776123046875f,0.3809000149726866624178001075051724910736083984375f,-0.213599984639883277992566945613361895084381103515625f,0.563300062251091304688088712282478809356689453125f,-0.2384000240623951327734175720252096652984619140625f,1.09700001615285902545338103664107620716094970703125f,-1.0285999793499709387134544158470816910266876220703125f,0.9377999779701224891681476947269402444362640380859375f,-0.8681999940454956377067219364107586443424224853515625f,-0.6959000148236753613417704400490038096904754638671875f,1.2888000136137005569025859585963189601898193359375f,-1.338300023061037347105184380779974162578582763671875f,0.423599980020523236845519932103343307971954345703125f,-0.3179000072836881685134358122013509273529052734375f,-0.469599999421834723989377380348742008209228515625f,-1.2593000138521208697284237132407724857330322265625f,1.142399992740154690551435123779810965061187744140625f,0.178299957942961828649686140124686062335968017578125f,-1.137599968144297424288424736005254089832305908203125f,0.172399993038176990722831760649569332599639892578125f,0.655900022572279528532135373097844421863555908203125f,0.1757000092819333836313688834707136265933513641357421875f,1.2913000222563744134873786606476642191410064697265625f,0.349499992311000706735057974583469331264495849609375f,2.215200052443146550018582274788059294223785400390625f,0.33960001026242980781688629576819948852062225341796875f,1.4137000713974245957871289647300727665424346923828125f,0.3581000076845288759841423598118126392364501953125f,0.8335000228285789347637546597979962825775146484375f,-0.204900006635487130779438302852213382720947265625f,-0.68410000432431683492495722020976245403289794921875f,-0.2281999959453939741393924123258329927921295166015625f,-0.4999000101745119106766424010857008397579193115234375f,-0.269699999417364555398535230779089033603668212890625f,-0.3479000127971172418028800166212022304534912109375f,-0.098099998147785694158073965809307992458343505859375f,-1.130499988391994836689491421566344797611236572265625f,-0.0622999995619057944651331126806326210498809814453125f,0.607999987870454727811875272891484200954437255859375f,0.3710000031590461322394958187942393124103546142578125f,1.1144000193238259299732817453332245349884033203125f,-0.122000001028180093953778850845992565155029296875f,-0.689000026047229763292989446199499070644378662109375f,-0.2646999988213181165264131777803413569927215576171875f,-1.914100040683150094622533288202248513698577880859375f,-0.373800006274878970913277953513897955417633056640625f,-0.960100055465102908414110061130486428737640380859375f,0.2720000002458691223949927007197402417659759521484375f,0.641700015532970180487382094725035130977630615234375f,0.2841000193744902535541996257961727678775787353515625f,-0.5700000178813933171539929389837197959423065185546875f,0.9090999943405382310857021366246044635772705078125f,-1.190399975618719796699451762833632528781890869140625f,1.33470002448856828181078526540659368038177490234375f,-0.9992000145643948894047525755013339221477508544921875f,0.844800052759052011452922670287080109119415283203125f,-0.348800009441375902241588846663944423198699951171875f,0.25059997799396427353713079355657100677490234375f,-0.4987999999046319743456479045562446117401123046875f,-0.5046000049352645788758309208787977695465087890625f,0.09499997891485723044979749829508364200592041015625f,-0.30110002770423793805321110994555056095123291015625f,-0.1625999737322334492972686348366551101207733154296875f,0.8410999827027314523775203269906342029571533203125f,-0.696200007098912720948646892793476581573486328125f,-0.68479998026490118689935115980915725231170654296875f,1.027800005313753484159633444505743682384490966796875f,-0.5561000044584272217207399080507457256317138671875f,-0.025599996322393625547420015209354460239410400390625f,0.564399956738949271084493375383317470550537109375f,-0.7173999578714376212928982567973434925079345703125f,0.13919997753500990000929959933273494243621826171875f,0.156600011551379925123228531447239220142364501953125f,-1.0926999999165527555078369914554059505462646484375f,1.799400009861587879100852660485543310642242431640625f,-0.1560999868750574393061469891108572483062744140625f,-0.029200007113814319126277041505090892314910888671875f,0.0516000121474267103849342674948275089263916015625f,0.0868000134050845861821699145366437733173370361328125f,0.1168000047624111470412344715441577136516571044921875f,-0.833399967512487460652437221142463386058807373046875f,-0.373999992184340879219917042064480483531951904296875f,-2.41570004627406564878810968366451561450958251953125f,-0.20880001499205835813199882977642118930816650390625f,0.181499967932700290162983947084285318851470947265625f,0.2845000091865659275214284207322634756565093994140625f,1.1267000523388392974055705053615383803844451904296875f,0.30210000787824398127412450776319019496440887451171875f,1.4179000225573779037091526333824731409549713134765625f,0.1926999944031237976815873480518348515033721923828125f,0.995200012955069102105198908247984945774078369140625f,0.1653000042706728456298748142216936685144901275634765625f,-0.1323999943792817379772941421833820641040802001953125f,-0.319400002075731770379007912197266705334186553955078125f,-1.7227000469297173435023751153494231402873992919921875f,-0.184700009587407176780970985419116914272308349609375f,0.0651999739885320561683101914240978658199310302734375f,0.2294999974891542837696079004672355949878692626953125f,1.042500025033950894481904470012523233890533447265625f,0.3058000074893236774897786744986660778522491455078125f,0.957600015679001881352405689540319144725799560546875f,-0.06540000650137667292938203900121152400970458984375f,0.6691000308483838754369799062260426580905914306640625f,0.2087000088125468000299633786198683083057403564453125f,0.6398000262349852196308575003058649599552154541015625f,0.0163000028103590377526188603951595723628997802734375f,-0.3845999793797727495103799810749478638172149658203125f,1.02729999412298145244903935235925018787384033203125f,-1.3738000069081779397350828730850480496883392333984375f,-0.352799969923495382317923940718173980712890625f,1.573199973705410759095002504182048141956329345703125f,-2.324600021922588410916432621888816356658935546875f,1.78420001709759201702354403096251189708709716796875f,-0.3678999789714811896601531771011650562286376953125f,0.021999972045421589683655838598497211933135986328125f,1.20960003503561086546369551797397434711456298828125f,-1.1131999964296819261022619684808887541294097900390625f,1.08649999475479130950361650320701301097869873046875f,-0.8307999653190376587730270330212078988552093505859375f,0.43860000438392177102286950685083866119384765625f,-0.472800003078579944570947191095910966396331787109375f,-0.583300010469555818559683757484890520572662353515625f,0.920999994739890137651627810555510222911834716796875f,-1.5190000279843811625823946087621152400970458984375f,1.1242000071138142924809244505013339221477508544921875f,-0.14979998369217017994969864957965910434722900390625f,0.0715999766826633798899592875386588275432586669921875f,1.121500029325485758846525641274638473987579345703125f,-1.086799995374679639326132019050419330596923828125f,0.49449997293949099486098930356092751026153564453125f,-0.779399976333975619269267554045654833316802978515625f,0.22840005133152008198749172152020037174224853515625f,0.1751999548405411832874278843519277870655059814453125f,0.67960000761747441089255516999401152133941650390625f,-0.8303999796420338253000181794050149619579315185546875f,-0.42869997610449761538120583281852304935455322265625f,0.21879999312460418625647662338451482355594635009765625f,0.17479999945014712547930457731126807630062103271484375f,-0.21910000611245672441640408578678034245967864990234375f,-0.22470000362694264595120330341160297393798828125f,-0.615800022725760953790086205117404460906982421875f,-0.1468999997101723964387787191299139522016048431396484375f,-0.804800043334066739220133968046866357326507568359375f,-0.299700007016956870842250282294116914272308349609375f,-0.953100013458728856363677550689317286014556884765625f,-0.215099996866285725349854374144342727959156036376953125f,0.245699983763695062322085505002178251743316650390625f,0.103299997915327512743033366859890520572662353515625f,1.27430003024339644213114297599531710147857666015625f,0.181100000099837732392416000948287546634674072265625f,-0.36049997915327580688682473919470794498920440673828125f,0.04979999967366455226880361806252039968967437744140625f,0.784700012940168445396693641669116914272308349609375f,0.35460000232756139215695156963192857801914215087890625f,0.40440003415048142443311007809825241565704345703125f,-0.2175999953016638510661096006515435874462127685546875f,-1.22690000300705381874877275549806654453277587890625f,-0.000499997183680545020934005151502788066864013671875f,-0.3635999819576742897453414116171188652515411376953125f,0.200299999465048284008616974460892379283905029296875f,1.173300036099553356194746811524964869022369384765625f,0.435800006707012688167424130369909107685089111328125f,0.2385000047981737925084644302842207252979278564453125f,-0.39450000773370275108931082286289893090724945068359375f,-0.26850000334531076706667818143614567816257476806640625f,0.282400015836954398906755159259773790836334228515625f,-0.20699998567998434406689511888544075191020965576171875f,-0.144800013792515169797070484491996467113494873046875f,0.38119999778568758319607923112926073372364044189453125f,-0.12820001818537729576519268448464572429656982421875f,-0.408400000816583397256209764236700721085071563720703125f,-0.80070007685869892810615056077949702739715576171875f,0.22520005117654751103373200749047100543975830078125f,-1.4500000272691249136869373614899814128875732421875f,0.906199981021881040277321517351083457469940185546875f,-0.11790003544688243408700145664624869823455810546875f,-1.23659998040795304774519536294974386692047119140625f,1.17930002443194315020491558243520557880401611328125f,-0.23440001425743073326657395227812230587005615234375f,0.118400045594572400631250275182537734508514404296875f,0.75359996720552480109489579263026826083660125732421875f,0.3779000126853586660757855497649870812892913818359375f,-1.070999991387128869035905154305510222911834716796875f,-0.2549999749660489811731167719699442386627197265625f,1.3493999703735113371294573880732059478759765625f,-0.335999985277652513104840181767940521240234375f,0.78199999491870375578628227231092751026153564453125f,0.04390003279447451944861313677392899990081787109375f,0.0687999651849275917214754372253082692623138427734375f,0.30220003901720104977357550524175167083740234375f,-0.385800001975894257810750787029974162578582763671875f,-0.57830003665834706083614946692250669002532958984375f,1.0388000116765498948012691471376456320285797119140625f,0.4721000069096683571245876009925268590450286865234375f,-0.18879995736181687249199967482127249240875244140625f,-0.1780999940872192155438824556767940521240234375f,-0.99039999163746761379201188901788555085659027099609375f,-0.1008000000387430095116769734886474907398223876953125f,-1.3439000038117168145390678546391427516937255859375f,-0.2331999932818113741017640450081671588122844696044921875f,-1.1721000406794248593911333955475129187107086181640625f,-0.2807000141248108260327853713533841073513031005859375f,-0.0101000281214720377676030693692155182361602783203125f,0.0113999988943338237401547985427896492183208465576171875f,-0.15879999535977817881615692385821603238582611083984375f,-0.2441999969810246806645182005013339221477508544921875f,-0.7702000072181218737199515089741908013820648193359375f,-0.14399999833106991165010413169511593878269195556640625f,-0.95080000503063200056885762023739516735076904296875f,-0.06310000313967467633347041555680334568023681640625f,0.4712999865680931321776370168663561344146728515625f,0.382200003813207123126716169281280599534511566162109375f,0.28430005930066126751398769556544721126556396484375f,-0.18929999831765886852963376441039144992828369140625f,0.1854999951720242012953576704603619873523712158203125f,0.1682999976918101159295559909878647886216640472412109375f,0.38510001612603617093810726146330125629901885986328125f,0.0567999991744756460576581957866437733173370361328125f,-0.6820000288933518728384797213948331773281097412109375f,-0.228600008295476497810483351713628508150577545166015625f,-1.1567000391513115875596895421040244400501251220703125f,-0.13250000491738322949686335050500929355621337890625f,-1.313600062051416106356782620423473417758941650390625f,-0.844899980148673623858712744549848139286041259765625f,0.94379995155036500165124380146153271198272705078125f,0.00560002236217282955976770608685910701751708984375f,0.34899999732524145112932956180884502828121185302734375f,-0.36869997457712833099918725565657950937747955322265625f,0.62059997210800654787732355543994344770908355712890625f,-1.09030005934909102194296792731620371341705322265625f,-0.06499998047947830315251849242486059665679931640625f,-0.63320003733336971407652526977472007274627685546875f,-0.0017999947786331293997363900416530668735504150390625f,-0.33250001378357385561912451521493494510650634765625f,0.08919999347925189336905305026448331773281097412109375f,-0.1913999949455256199826180818490684032440185546875f,0.1597999978482722571726526439306326210498809814453125f,-0.5723999623417856952301008277572691440582275390625f,1.08559997024536158960472675971686840057373046875f,0.31259999146461492358639588928781449794769287109375f,-1.1307999832004309581634515780024230480194091796875f,-1.15619998176693972169459811993874609470367431640625f,1.494199950563908085854336604825221002101898193359375f,-0.66250000439584244560364822973497211933135986328125f,-0.4537999897718429931359196416451595723628997802734375f,-0.21479996111691068705340512678958475589752197265625f,0.80559995936751416234500311475130729377269744873046875f,-0.8043000272631648517318581070867367088794708251953125f,0.458200016173720303669369968702085316181182861328125f,-1.239000005781651037040091978269629180431365966796875f,1.3161999845236540540582836911198683083057403564453125f,-1.04610005773007941343166748993098735809326171875f,-0.5200999743282797016519225508091039955615997314453125f,-0.03409999877363445452971291160793043673038482666015625f,-0.43550000284612166989717252363334409892559051513671875f,-0.086100001701712614021744229830801486968994140625f,0.89839998829960787674053790397010743618011474609375f,0.1868999985739587810229522801819257438182830810546875f,-0.2976999764472250120661556138657033443450927734375f,-0.0905000005364418047548724644002504646778106689453125f,0.22779999257326100092768683680333197116851806640625f,0.28649999725073571621436485656886361539363861083984375f,1.012299976763128217527309971046634018421173095703125f,0.239299989339709195945715691777877509593963623046875f,0.353599956178663799022388047887943685054779052734375f,-0.33280000933706765930963911159778945147991180419921875f,-1.962200034546852212002931992174126207828521728515625f,-0.34939999391734588574109920955379493534564971923828125f,-1.264000025972723317835288980859331786632537841796875f,0.008999999336898323321065618074499070644378662109375f,0.9263000306010249484955920706852339208126068115234375f,0.09730000623017553973426174707128666341304779052734375f,-0.675499958291648994190836674533784389495849609375f,-0.1673999943420290126283589415834285318851470947265625f,-0.6403000238656992859631600367720238864421844482421875f,-0.188000011526048227405993884531199000775814056396484375f,0.27079997012466117123352660200907848775386810302734375f,0.1719000044226646206624309343169443309307098388671875f,-1.098199988383054659379922668449580669403076171875f,-0.1739000034019351215164306267979554831981658935546875f,-0.338300001677870998406660874024964869022369384765625f,0.626300035145877842523987055756151676177978515625f,0.6681999594002967501893408552859909832477569580078125f,0.3783999958619472803178496178588829934597015380859375f,-0.61699999327957610883998995632282458245754241943359375f,1.962399977317451860159280840889550745487213134765625f,-1.57199996772408479017713034409098327159881591796875f,0.56290001503229092350011342205107212066650390625f,0.687399987003207257885151193477213382720947265625f,0.53820002340078332281336770392954349517822265625f,-0.34219999490976338307746118516661226749420166015625f,0.845399980163573872005144949071109294891357421875f,-0.948199954631924857295643960242159664630889892578125f,0.1250999613642687080528048682026565074920654296875f,0.399400017237663274016767900320701301097869873046875f,-0.695699974226950956790460622869431972503662109375f,1.169999966919422096367497942992486059665679931640625f,-0.7603000053882595921095344237983226776123046875f,-0.259999996647238607039298585732467472553253173828125f,0.32260001173019414721920838928781449794769287109375f,-0.4632000159502027702984605639358051121234893798828125f,-1.136399963355064102188407559879124164581298828125f,1.11719995400607619018273908295668661594390869140625f,-1.71330002570599315703248066711239516735076904296875f,0.6814000172972678459615281099104322493076324462890625f,0.2373000024303790755908494247705675661563873291015625f,-0.133800012198090456649879342876374721527099609375f,-1.19619999890327477487517171539366245269775390625f,1.5065999881565570461816605529747903347015380859375f,-1.35719999081194497847491220454685389995574951171875f,0.4762000015854841450391177204437553882598876953125f,0.097800008666515392263818284845910966396331787109375f,-0.20259995741546088510176559793762862682342529296875f,0.08630000430047513049203189439140260219573974609375f,0.9482000130817294092366864788345992565155029296875f,0.1778999953441321169034239346729009412229061126708984375f,0.73520000524371820205260519287548959255218505859375f,0.1257000032842159331636366914608515799045562744140625f,0.068800029483437885602370442938990890979766845703125f,-0.334099996650218944438393009477294981479644775390625f,-1.2073999948263161030581613886170089244842529296875f,-0.03259999917596574647848939321193029172718524932861328125f,-0.10860000069588393589725683341384865343570709228515625f,-0.018899998112022864393821919293259270489215850830078125f,1.280300014105439476708170332130976021289825439453125f,0.1144999999850988547933638983522541821002960205078125f,-0.81800001692771928674119408242404460906982421875f,-0.32760000249147414042028003677842207252979278564453125f,-0.8050000333040951971952381427399814128875732421875f,-0.139100000226497655386737051230738870799541473388671875f,-1.254200024399161339516695079510100185871124267578125f,-0.29200000382214785776824328422662802040576934814453125f,-0.995600075075030677140830448479391634464263916015625f,-0.3670000119805336691314323616097681224346160888671875f,-1.8868000382900242950512392781092785298824310302734375f,-0.222599993178248301006760812015272676944732666015625f,-0.4685999989449971536714656394906342029571533203125f,0.10699999797344206253768561509787105023860931396484375f,0.7156000125646591047967604026780463755130767822265625f,0.934999990463257546480235760100185871124267578125f,-1.068999986112117994707659818232059478759765625f,-0.26509998580217342123432899825274944305419921875f,0.73059998790323721351569474791176617145538330078125f,0.051699977236985661388501966939656995236873626708984375f,-0.489999980777502119888566767258453182876110076904296875f,0.38179997879713767350295938740600831806659698486328125f,-0.575599988797307116072943244944326579570770263671875f,-0.12260001396536868156772470683790743350982666015625f,0.397599976637959873215777406585402786731719970703125f,-0.6241999870344994150173079106025397777557373046875f,0.2443999904155731428545550443232059478759765625f,-0.9933000073030588961131570613360963761806488037109375f,0.3313999912202358899548926274292171001434326171875f,1.19179999105334299969172207056544721126556396484375f,-1.50539997517168533391895834938623011112213134765625f,0.020499998748302772355600609444081783294677734375f,0.7300000023841857199613514239899814128875732421875f,-0.2548000234782694661817004089243710041046142578125f,0.00500000931322563957337479223497211933135986328125f,-0.56950002256035825443092335262917913496494293212890625f,0.57660000141859057976034819148480892181396484375f,-0.1667000756591556775987328364863060414791107177734375f,-0.19799993631243761882387843797914683818817138671875f,-1.00399998998641937220099862315692007541656494140625f,1.1939999613761902441666507002082653343677520751953125f,-0.8143999740242957585678595933131873607635498046875f,0.2791999849855901771888966322876513004302978515625f,0.23120001678466817196522242738865315914154052734375f,-0.478799985674023265147525307838805019855499267578125f,-0.2344000052794814603629447447019629180431365966796875f,0.00049997468292717339721775715588591992855072021484375f,0.055800000970065599670277833865839056670665740966796875f,1.31300000843405673123243104782886803150177001953125f,0.308599994102120300798475227566086687147617340087890625f,0.65259999019801551156660934793762862682342529296875f,-0.0299000033944845089450836894684471189975738525390625f,-0.04480001423954949046901674591936171054840087890625f,-0.233499996714293889255031899665482342243194580078125f,-1.05810000824332206548206158913671970367431640625f,-0.3368000047624111203958818805404007434844970703125f,-1.32340004052817850066503524431027472019195556640625f,0.20009999638199804650184887577779591083526611328125f,1.143599996486305503395897176233120262622833251953125f,0.0141999983593821266225631916313432157039642333984375f,-0.155299989145994021821906017066794447600841522216796875f,-0.1042999996028840248296631898483610711991786956787109375f,-0.6941000049017367334869277328834868967533111572265625f,-0.00400000179558990254236050532199442386627197265625f,-0.611900052478909994846389963640831410884857177734375f,-0.25830001210868369643236519550555385649204254150390625f,-0.448200031521917008348054878297261893749237060546875f,0.013100001761317259241224064680864103138446807861328125f,0.727800026026368396259158544125966727733612060546875f,-0.0015999975815415012903031310997903347015380859375f,0.0813000054925681059359021674026735126972198486328125f,-0.03200000240653755323450013747788034379482269287109375f,0.975500037342310388766009054961614310741424560546875f,-1.6363000240445142452472282457165420055389404296875f,1.397399987301230606107083076494745910167694091796875f,-0.82060005767792443265307156252674758434295654296875f,-0.65499997045844826626392887192196212708950042724609375f,0.70809998803362173003250745750847272574901580810546875f,-0.5711999835550785054039124588598497211933135986328125f,-0.006800038886070058907762359012849628925323486328125f,-0.379399966052174963948573349625803530216217041015625f,-1.01640003726482319734714110381901264190673828125f,0.33140000552535031630441153538413345813751220703125f,-1.065799985361099544434182462282478809356689453125f,1.321799968403578162678968510590493679046630859375f,-0.96589996550083156989785493351519107818603515625f,0.55499998755753043866434381925500929355621337890625f,0.477799987059830844060570598230697214603424072265625f,-1.12480000537335822485829339711926877498626708984375f,0.0346000145651401691537785154650919139385223388671875f,0.343399992918968222088693664773018099367618560791015625f,-1.378699983573704823669459074153564870357513427734375f,0.99859998367130753393894337932579219341278076171875f,-1.905400056159496813279474736191332340240478515625f,1.057000018760561932396058182348497211933135986328125f,-1.151700037437677792695467360317707061767578125f,0.218799993869662312562240913393907248973846435546875f,0.69450000944733591978774711606092751026153564453125f,-0.90960001305639703872429890907369554042816162109375f,0.8026000098675485361354731139726936817169189453125f,-0.7990000033974642423828527171281166374683380126953125f,1.4320000500157481493346267598099075257778167724609375f,-0.363500028640032557536443391654756851494312286376953125f,0.10880000020265578886125013013952411711215972900390625f,0.291599999108910434841845926712267100811004638671875f,-0.0465000029504298861837696676957421004772186279296875f,-0.5671999964743850153325865903752855956554412841796875f,-0.194099995160102789526490596472285687923431396484375f,-0.556999995291232341543263828498311340808868408203125f,-0.0123999993711709777244323049671947956085205078125f,0.2447999730378385141449371076305396854877471923828125f,0.2746999951332806944037656649015843868255615234375f,0.578799992305039534556954095023684203624725341796875f,-0.0171000044509768966971563486367813311517238616943359375f,1.0269000429421664133400327045819722115993499755859375f,0.36940001284182077956330658707884140312671661376953125f,1.8514000609576708455961124855093657970428466796875f,0.3673999958693980882884488892159424722194671630859375f,1.9344000270724290846402482202393002808094024658203125f,0.3455000021010636856999553856439888477325439453125f,1.00160003941655162407187162898480892181396484375f,0.11260000007003545530182009315467439591884613037109375f,1.4031000026553870352330477544455789029598236083984375f,0.18240000171810388795989865684532560408115386962890625f,1.120300027441978318876181219820864498615264892578125f,0.20220000288188455517257580140721984207630157470703125f,0.3952000296443702875848202893394045531749725341796875f,-0.07480000164806842322118285437682061456143856048583984375f,-0.19970004345029612746742486706352792680263519287109375f,-0.2035000077039004207080097330617718398571014404296875f,-0.692000043272972931163167231716215610504150390625f,-0.3743000066995623154042505120742134749889373779296875f,0.402199994611739997107946464893757365643978118896484375f,-0.127100009554624637786446328391321003437042236328125f,-0.641399996212124623440331561141647398471832275390625f,-1.2390999678909775383317537489347159862518310546875f,1.4209999542087319301941761295893229544162750244140625f,-0.50169996904134706028344226069748401641845703125f,0.545799989533424234622316362219862639904022216796875f,0.9068999938428401463852424058131873607635498046875f,-1.1233999657243491920866063082939945161342620849609375f,-0.13800002624094531711307354271411895751953125f,0.260400010114908209146733497618697583675384521484375f,0.074500029340386131337936603813432157039642333984375f,-0.460600043112039170267735244124196469783782958984375f,0.7785000139623878823158520390279591083526611328125f,-0.97459997856020930129261614638380706310272216796875f,0.598899999564885821001780641381628811359405517578125f,-0.728999996319413146039778439444489777088165283203125f,-0.613399987256527001733275028527714312076568603515625f,0.7273999873757361900317164327134378254413604736328125f,0.75149996841698918359497838537208735942840576171875f,-1.494599963435530920463634174666367471218109130859375f,1.092799993318319007329364467295818030834197998046875f,-0.832999995023012029804476696881465613842010498046875f,-0.52040001208931219167652670876123011112213134765625f,0.3997999994128942358173617321881465613842010498046875f,-1.1762000890180470680235202962649054825305938720703125f,-0.038599955955147979835828664363361895084381103515625f,-0.968200048136711899360307143069803714752197265625f,0.0725999647915358536920393817126750946044921875f,-0.214700006160140066668873259914107620716094970703125f,-0.3490000378936530811557759079732932150363922119140625f,-0.1954000047877431089915489792474545538425445556640625f,-1.196100006842613527879848334123380482196807861328125f,-0.2135999953687190977547061265795491635799407958984375f,0.0792000262618071104725459008477628231048583984375f,0.3399000020161271340413122743484564125537872314453125f,1.287300021764635982179925122181884944438934326171875f,-0.0130000002756715249407903911560424603521823883056640625f,0.5365000012367964199455627749557606875896453857421875f,0.3004000038564205166125020696199499070644378662109375f,1.5176000491321086638407678037765435874462127685546875f,0.0368000002920627622415850055404007434844970703125f,0.1639999947547912118039903361932374536991119384765625f,-0.1584000051379204077051099375239573419094085693359375f,0.217399981974065015233321673804312013089656829833984375f,0.0098999982349574355833254912795382551848888397216796875f,0.662200013051926550389225667458958923816680908203125f,0.3805999975517391220591889577917754650115966796875f,0.5543000309512020606916848919354379177093505859375f,-0.33149999469891178183189595074509270489215850830078125f,-1.151000025659798975397052345215342938899993896484375f,-0.25700000577792525557896397003787569701671600341796875f,0.5004999803826206683510235961875878274440765380859375f,0.24699999883025880453857325846911408007144927978515625f,-0.02429998807311062325453576704603619873523712158203125f,-0.3205000014677643616067825860227458178997039794921875f,-1.418100035437941386362581397406756877899169921875f,-0.492200039091706731397835028474219143390655517578125f,-0.4071999927490945747621253758552484214305877685546875f,-0.26330003691911674224002126720733940601348876953125f,0.0184000281602141324555077517288736999034881591796875f,-0.6660999767422683959239293471910059452056884765625f,1.358399940168858233136006674612872302532196044921875f,0.6853000265479085584274798748083412647247314453125f,-1.579800002467631969693684368394315242767333984375f,1.2735000148564576960552585660479962825775146484375f,-0.204799993303418137458038472686894237995147705078125f,1.054299961325526613364900185842998325824737548828125f,-1.572599952572584669496791320852935314178466796875f,0.65449999201297703876889499952085316181182861328125f,-0.4566000084221359944791629459359683096408843994140625f,0.08949997740983928196101260255090892314910888671875f,-0.4111999887704846923242030243272893130779266357421875f,0.467199979561567058095050697374972514808177947998046875f,-0.713799986642599026964717268128879368305206298828125f,0.32360001813024175110200530980364419519901275634765625f,-0.072200015398859562765210284851491451263427734375f,-0.27560002433657615483753033913671970367431640625f,-0.11400000414252275948712167519261129200458526611328125f,-0.83270002379566410155575795215554535388946533203125f,0.56960001462101927938164180886815302073955535888671875f,0.82579998159855516615124315649154596030712127685546875f,-1.537599994817375748112908695475198328495025634765625f,-0.01460000568032260304107694537378847599029541015625f,0.59679999835491182036406598854227922856807708740234375f,-1.67380002911090830508555882261134684085845947265625f,-0.1434999727606769415189091887441463768482208251953125f,-0.14090000197142360516266990089206956326961517333984375f,-0.98050004702806481393650983591214753687381744384765625f,-0.161900007142126600712117578950710594654083251953125f,0.115499961271881002033978802501223981380462646484375f,0.289499994546174921339343200088478624820709228515625f,-0.08380000675916743357873883724096231162548065185546875f,0.0554000011742115106017081416212022304534912109375f,0.334800011557340848611374894971959292888641357421875f,0.018700008663535161446134225116111338138580322265625f,0.21820002407133642918779514729976654052734375f,0.0907000010490417452047040569595992565155029296875f,0.26340001437664029015905953201581723988056182861328125f,-0.15130000180453062608165737401577644050121307373046875f,0.5377000159353020336538975243456661701202392578125f,0.20040000545829539380093820000183768570423126220703125f,-0.294700022402405892307797330431640148162841796875f,-0.14320001107007274132598695359774865210056304931640625f,1.1846999889492979551874896060326136648654937744140625f,0.24330000082105396330689472961239516735076904296875f,0.90750003330409523272237493074499070644378662109375f,-0.1248999993711710221333532899734564125537872314453125f,-0.64459999815523616462087375111877918243408203125f,-0.1730000000521541014819604242802597582340240478515625f,0.4758999994382264731029863469302654266357421875f,0.2932000004716217578248915742733515799045562744140625f,0.531300014917551965965003546443767845630645751953125f,-0.0054000029996037479662618352449499070644378662109375f,0.026699993628263196665528766970965079963207244873046875f,-0.78199997621774652856174725457094609737396240234375f,0.9281999843597412702678184359683655202388763427734375f,-1.731300023448466784969923537573777139186859130859375f,1.359400019025802208716413588263094425201416015625f,-0.02099999690055920353870533290319144725799560546875f,0.064800001052021993785956510691903531551361083984375f,-0.11319997992664554420372269305516965687274932861328125f,0.794999996088444849107190748327411711215972900390625f,0.60849999788776065745565801989869214594364166259765625f,-0.610800004211068170434373314492404460906982421875f,0.87679998315572760247960104607045650482177734375f,-0.9951999727219345004414208233356475830078125f,-0.129100016617775370519893840537406504154205322265625f,0.15859999343156838680357623161398805677890777587890625f,0.86490003267526649466390153975225985050201416015625f,-0.76520001071989529606298674480058252811431884765625f,-0.2634000337481499087743941345252096652984619140625f,0.23760003907382465371256330399774014949798583984375f,0.64379996920823945316669778549112379550933837890625f,-1.5546000022530550577215535668074153363704681396484375f,1.2678000116467469382541821687482297420501708984375f,-0.41160001311600158402370652765966951847076416015625f,0.74669998900890277582220733165740966796875f,-0.377200003924965443502514972351491451263427734375f,1.4385999791264538760771074521471746265888214111328125f,-1.398199947851896940420601822552271187305450439453125f,0.26890000235885358481624507476226426661014556884765625f,0.2953999976724386300475089228712022304534912109375f,-0.4931999836146834237382563514984212815761566162109375f,-0.0811000101953747165595132173621095716953277587890625f,0.23770000226348642069496008843998424708843231201171875f,1.351700022536516332394285200280137360095977783203125f,0.11880000310838223498421939439140260219573974609375f,-0.195599961826204538084539308329112827777862548828125f,-0.262799994249641788446325563199934549629688262939453125f,-0.43290003485083572432046139510930515825748443603515625f,0.16689999216645945434578379718004725873470306396484375f,0.14350000017881303637778955817339010536670684814453125f,-0.032900001509487630801942259495262987911701202392578125f,-0.32580001754760756771389651476056315004825592041015625f,0.0117999996960163056058945585391484200954437255859375f,-0.2960000148192047930706394254229962825775146484375f,-0.319200004338473064535719458945095539093017578125f,-0.3149000323399899770038246060721576213836669921875f,-0.0582999996289610411537296386086381971836090087890625f,-1.29050002050399825748172588646411895751953125f,-0.2668000037193298812354669280466623604297637939453125f,-1.6882000319316983194539716350845992565155029296875f,-0.2176000009082257602432974863404524512588977813720703125f,-0.0481000260874626661689035245217382907867431640625f,-0.0176999982401728761516324084368534386157989501953125f,-0.27409998726248741007793796597979962825775146484375f,-0.07979999971091744637163856168626807630062103271484375f,-0.2008000015288586848072327484260313212871551513671875f,-0.2415999975815414924085189340985380113124847412109375f,0.3562999876856804970515213426551781594753265380859375f,0.19539999476671210398848188560805283486843109130859375f,1.01519999842643660059593457845039665699005126953125f,-0.41699998135864735360200938885100185871124267578125f,-0.166000016421079532591420502285473048686981201171875f,0.7523000033617019965959116234444081783294677734375f,-1.107999998301267385869550707866437733173370361328125f,-0.01509996032118721132064820267260074615478515625f,0.855799973216652798413406344479881227016448974609375f,0.323899994498491050620714304386638104915618896484375f,-0.57499997377395628550544870449812151491641998291015625f,0.35280002084821404650938347913324832916259765625f,0.50579998018294591854981945289182476699352264404296875f,0.09729999883547424399665715100127272307872772216796875f,-0.36919999549090853907529208299820311367511749267578125f,-0.272100039841234764281807656516321003437042236328125f,-0.012599984759092297537108606775291264057159423828125f,0.42789997804015966753610200612456537783145904541015625f,-0.586199964183569566245068926946260035037994384765625f,-0.5415999763846404135847478755749762058258056640625f,1.6519999607205395619757837266661226749420166015625f,-1.10850001195073133430923917330801486968994140625f,0.50459999256730092564993128689820878207683563232421875f,0.52309996082037735209979700812255032360553741455078125f,-0.837599962109327744741449350840412080287933349609375f,0.26409998964667291687646866193972527980804443359375f,-0.21460000508427601317862354335375130176544189453125f,-0.34320002042055097746242608991451561450958251953125f,-0.1940000001937149942676796854357235133647918701171875f,0.65789998284578299347913343808613717555999755859375f,-0.7453999700307847309233011401374824345111846923828125f,1.298699977147578454861331920255906879901885986328125f,0.091999995812773516234983617323450744152069091796875f,0.089000001274049278521260930574499070644378662109375f,0.549499993801116470848455719533376395702362060546875f,0.2094999986812471348684994154609739780426025390625f,-0.4241000185921794951582342036999762058258056640625f,-0.32590001126602297087941906283958815038204193115234375f,0.287999973230063321949501187191344797611236572265625f,0.296900002261996309727010157075710594654083251953125f,0.9174000156134367589544353904784657061100006103515625f,0.08499999772757294014269291437813080847263336181640625f,0.848100013756752257876314615714363753795623779296875f,0.1900000145286322439375226167612709105014801025390625f,0.51360008440315851885316078551113605499267578125f,0.2952000152274967259558025034493766725063323974609375f,0.263700046810508847983101077261380851268768310546875f,-0.098999998293817004224592892569489777088165283203125f,-0.30829998878836573084072369965724647045135498046875f,-0.1867999949276446525203709825291298329830169677734375f,0.3124000124096877328838672838173806667327880859375f,-0.0140999984011054291244136038585565984249114990234375f,0.36770000118315204762353687328868545591831207275390625f,0.28729999524056903936752860317938029766082763671875f,1.287799976852534999949284610920585691928863525390625f,0.1460999960392712626600086878170259296894073486328125f,-0.4272000087678440305438698487705551087856292724609375f,-0.0688000048965216048912907353951595723628997802734375f,-0.8274000376671555301300031715072691440582275390625f,-0.3684000018969177947525395211414434015750885009765625f,-0.4363000287383795861018143114051781594753265380859375f,-0.095199987101555194612956256605684757232666015625f,0.274599982509017070952950234641321003437042236328125f,1.767099995696543857093274709768593311309814453125f,-1.271999981582164540583335110568441450595855712890625f,0.3968999387085432317689992487430572509765625f,0.06820004590153672108243654292891733348369598388671875f,0.32609998519867666733063060746644623577594757080078125f,-1.096399988016486304331920109689235687255859375f,-0.2790999767571680223454677616246044635772705078125f,0.5546000058293338863535382188274525105953216552734375f,0.5384000300601134991751450797892175614833831787109375f,-0.109000014424324387363185451249592006206512451171875f,1.366600070559979229756208951584994792938232421875f,-0.4378000294536352754448671475984156131744384765625f,0.4951000096440321129875883343629539012908935546875f,-0.383199990692734804298424933222122490406036376953125f,-0.5065000090599056647988618351519107818603515625f,-0.11620001514554001431633878382854163646697998046875f,0.363799992156028650924781686626374721527099609375f,-0.528999998629092971214049612171947956085205078125f,0.960400001919269552530522560118697583675384521484375f,-0.24359996444881015786876332640531472861766815185546875f,1.91019995491504612772359905648045241832733154296875f,-1.347599950858950901277921730070374906063079833984375f,0.2959999894499774075740106127341277897357940673828125f,0.651600008273124586821722914464771747589111328125f,-0.722900041407346538591127682593651115894317626953125f,0.32440001209676250226721094804815948009490966796875f,0.1334999774545426731719999224878847599029541015625f,0.5010000153779985421209630658267997205257415771484375f,0.18870000408142806325173523873672820627689361572265625f,0.04949999208748323553663794882595539093017578125f,-0.26300000988692051606676614028401672840118408203125f,-0.913100039833784737908217721269465982913970947265625f,-0.0227000050947070430851226774393580853939056396484375f,1.0276999929875130224132817602367140352725982666015625f,0.36000000003725285413480605711811222136020660400390625f,1.27570002712607344363959782640449702739715576171875f,0.05960000009238719531623473812942393124103546142578125f,0.500200028303265487039652725798077881336212158203125f,0.1099000013828277388938658987171947956085205078125f,0.0576000158280134400001770700328052043914794921875f,0.2336999999463558186363343338598497211933135986328125f,1.047399990355967869248843271634541451930999755859375f,0.1921999949097632498506982301478274166584014892578125f,0.4695999819874756564530571267823688685894012451171875f,-0.09370000251680610237059454448171891272068023681640625f,0.77629996555745517383684273227117955684661865234375f,0.258699993836879638120507252097013406455516815185546875f,0.1315000033229580811422465558280237019062042236328125f,-0.168300004769861732967228817869909107685089111328125f,0.4602999994650485149350060964934527873992919921875f,0.24480001094266778949304352863691747188568115234375f,1.4437000320956112631165524362586438655853271484375f,0.2479999927133320902772339877628837712109088897705078125f,0.1009999913871280075028380451840348541736602783203125f,0.039400000138580804076582353445701301097869873046875f,-0.488899990400671669021903653629124164581298828125f,0.31419999586343738684490745072253048419952392578125f,-0.6500000053644179320855300829862244427204132080078125f,0.2137999605655664936421089805662631988525390625f,-0.33619997290074810081250689108856022357940673828125f,-1.3705000133514406712720301584340631961822509765625f,1.125399995511770168121756796608678996562957763671875f,-0.03610000326633500122852638014592230319976806640625f,-0.5173999990731477094385581949609331786632537841796875f,1.0507999928116793508792170541710220277309417724609375f,-1.0359999945163724799357396477716974914073944091796875f,0.855800045412778320752522631664760410785675048828125f,0.140199970635772164229138070368207991123199462890625f,0.1519000357151032432057036203332245349884033203125f,0.0627999960750340591175699955783784389495849609375f,0.459999987483024508350126779987476766109466552734375f,-0.18019996922016179308201344611006788909435272216796875f,1.01879996972978137392829012242145836353302001953125f,-0.7739999633878473783710205680108629167079925537109375f,1.807799976815282594344580502365715801715850830078125f,-1.798799963247776023678170531638897955417633056640625f,0.2132000090211623177793853756156750023365020751953125f,0.9627999861657621227806203023646958172321319580078125f,0.9746000009119517581979152964777313172817230224609375f,-1.714599988394975849104184817406348884105682373046875f,1.702600021527707507829063615645281970500946044921875f,-0.707999974310398183519055237411521375179290771484375f,-0.245300001141429024897888666600920259952545166015625f,0.45759999675452700529376670601777732372283935546875f,-1.0752999934673308501942301518283784389495849609375f,-0.0190000209808351971929596402333118021488189697265625f,-0.13769999637752772247978327868622727692127227783203125f,0.2815000203847886073305062382132746279239654541015625f,0.307399999818205760249156810459680855274200439453125f,-0.1908000270098455519729441220988519489765167236328125f,-0.40550001495331533529764556078589521348476409912109375f,-0.87090006876587988671190032619051635265350341796875f,0.0875999972760677092509240537765435874462127685546875f,-0.066599993148446667845519186812452971935272216796875f,0.289400002969801395380500252940692007541656494140625f,1.782700078371167951019060637918300926685333251953125f,0.290200013066828355068338396449689753353595733642578125f,-0.4122999674499026045992877698154188692569732666015625f,-0.397000000469386549184491741470992565155029296875f,-1.0094000219687817576641464256681501865386962890625f,0.1091999985456466593092272887588478624820709228515625f,1.206600029656291095392361967242322862148284912109375f,0.3812000050500035097655882054823450744152069091796875f,1.4873000614762308391192391354707069694995880126953125f,0.0222000051170587742177531254128552973270416259765625f,0.2802000395536430321641319096670486032962799072265625f,0.12180000520944600506112465154728852212429046630859375f,-0.194999987706541588750042137689888477325439453125f,0.00050000046193598990384998614899814128875732421875f,0.674999982193111858208567355177365243434906005859375f,0.25609999149441708965468933456577360630035400390625f,1.1479999904781570396750112195149995386600494384765625f,-0.0320000032261014144552291327272541821002960205078125f,-0.41709999159872512830560253860312514007091522216796875f,0.5734999561458824501869457890279591083526611328125f,-0.4547999888330698237126625826931558549404144287109375f,1.2765000080540771509873820832581259310245513916015625f,-0.8353999891042709347033223821199499070644378662109375f,0.160199962738155843311460557742975652217864990234375f,0.483400035089254220821430862997658550739288330078125f,0.4226000126242634991058366722427308559417724609375f,0.0605999981105327378827496431767940521240234375f,0.1500000384449950985299437888897955417633056640625f,0.059199963900447460929399312590248882770538330078125f,1.0642000481665139233200534363277256488800048828125f,-0.980200022193789433089250451303087174892425537109375f,0.1236000204399232416108134202659130096435546875f,0.9135999913454055931794073330820538103580474853515625f,-0.4360000082626942496943911464768461883068084716796875f,-0.524200003314017948241598787717521190643310546875f,-0.199799982276558818483636059681884944438934326171875f,0.134999990090727717273466623737476766109466552734375f,0.69960000314712456059851319878362119197845458984375f,-0.7428000220775599071743044987670145928859710693359375f,0.76300003272295047196394079946912825107574462890625f,-0.4830000038146973206920620214077644050121307373046875f,-0.8942999915003770894372792099602520465850830078125f,1.18719999975264034475230801035650074481964111328125f,-0.357200042295456654528607032261788845062255859375f,-0.580999969705939367514702098560519516468048095703125f,0.40799995702505054140374340931884944438934326171875f,-0.8573999822348354893364330564509145915508270263671875f,-0.474099995458125977876306933467276394367218017578125f,0.327500020936131619464504183270037174224853515625f,0.2905000085830689027233120214077644050121307373046875f,0.7616000351697211900869888268061913549900054931640625f,-0.08459999953359365054694762875442393124103546142578125f,0.202900008252262953334366102353669703006744384765625f,0.1796000010237097654908211552537977695465087890625f,1.84150004899501862354327386128716170787811279296875f,0.12380000571608551496183281415142118930816650390625f,0.350200013849139235588836527313105762004852294921875f,-0.0715000009387731783050412559532560408115386962890625f,-1.218799983066320091751322252093814313411712646484375f,-0.236199994581937744442257098853588104248046875f,-1.0668000120639795635923974259640090167522430419921875f,-0.05120000225603582588718154511298052966594696044921875f,0.07820000805258775500306001049466431140899658203125f,-0.1154999949112534007866770480177365243434906005859375f,1.212700039926172035364970724913291633129119873046875f,0.17310000161230565396408564993180334568023681640625f,1.488800042521953859164796085678972303867340087890625f,0.22180000394284726805693708229227922856807708740234375f,0.3964000146150592485128072439692914485931396484375f,-0.13039999826848502895160208936431445181369781494140625f,0.6320000136196615159178691101260483264923095703125f,0.187500005401670943872005636876565404236316680908203125f,0.3997000484794381858222322989604435861110687255859375f,0.23940000505596403801433780245133675634860992431640625f,-0.41430000409185918641696844133548438549041748046875f,-0.254300011914968593362829096804489381611347198486328125f,0.2457000011980532416799860584433190524578094482421875f,-1.296299989473820080121413411688990890979766845703125f,-0.8290000083148481024863940547220408916473388671875f,-1.428599987880885269220243571908213198184967041015625f,0.42639998237267151548479660050361417233943939208984375f,0.29450005377829080455143184735788963735103607177734375f,-0.6232000037312506179887350299395620822906494140625f,0.486499994084238718272672485909424722194671630859375f,0.2781999729976061264125064553809352219104766845703125f,1.2465999746337530496020917780697345733642578125f,-0.4743000001430512924116555950604379177093505859375f,0.20489993630200642460437165937037207186222076416015625f,-0.1101000102400779834255217792815528810024261474609375f,1.0325000205636023498101394579862244427204132080078125f,0.20900000444054622761314021772705018520355224609375f,0.4923999935224652180210114238434471189975738525390625f,-0.21819998838305476596133303246460855007171630859375f,0.8377999897792935968254823819734156131744384765625f,0.746100003713369375191177823580801486968994140625f,-0.9237999716669325067641693749465048313140869140625f,0.14809999185204514304814438219182193279266357421875f,-0.24429998188912904755198951534111984074115753173828125f,0.012900009593367567362065528868697583675384521484375f,0.408299994767457263833421166054904460906982421875f,0.0296999989330768432438389936578460037708282470703125f,-0.8370000399202108809504352393560111522674560546875f,-0.3854000046014787184134320341399870812892913818359375f,-0.0271000366561111594165822680224664509296417236328125f,0.2613000022888183337954615126363933086395263671875f,1.5839999705031517773790028513758443295955657958984375f,0.908600000211596370291999846813268959522247314453125f,-0.6090000004917381914992802194319665431976318359375f,0.76740004020035268439414721797220408916473388671875f,-1.61890002772808117015301831997931003570556640625f,-0.6462000205844642408692379831336438655853271484375f,0.90550000515580197912868243292905390262603759765625f,-0.2024999837577337036265134884160943329334259033203125f,1.893400027489661852087010629475116729736328125f,-0.213600006917118889759876765310764312744140625f,0.73390002161860490303979531745426356792449951171875f,-0.6024999652802940364182404664461500942707061767578125f,-0.6147999646186832478633732534945011138916015625f,0.5260999952197078499693816411308944225311279296875f,-1.9476999757766719767460017465054988861083984375f,-1.022599989676475473743266775272786617279052734375f,0.35320004336833932256922707892954349517822265625f,-0.0260000140368941146107317763380706310272216796875f,0.9092000027179718557590604177676141262054443359375f,-1.083000002473592626728304821881465613842010498046875f,1.5644000223040581687428129953332245349884033203125f,0.830300024759770050053475642926059663295745849609375f,-0.846000023722648819557434762828052043914794921875f,-0.3760999913640323821795163894421420991420745849609375f,-1.0072999766886241435059901050408370792865753173828125f,0.30489998532682649479141900883405469357967376708984375f,0.481199988174438431087764911353588104248046875f,0.93219999818801824176262016408145427703857421875f,-0.313400032854080023980714031495153903961181640625f,0.0816999876677986236472861492075026035308837890625f,-0.20520000952780248493212411631247960031032562255859375f,-0.29219999982714650599291417165659368038177490234375f,-0.2748000090613962331786979120806790888309478759765625f,-1.02590002003908153938027680851519107818603515625f,0.5257999788418412645540911398711614310741424560546875f,0.50500000916421405872114291923935525119304656982421875f,-0.051399994572996821062815797631628811359405517578125f,1.07760001187920551046772743575274944305419921875f,-0.6782000026509162893262327997945249080657958984375f,0.63350001187622539955413003553985618054866790771484375f,-0.295200031469762314006999304183409549295902252197265625f,0.18880000362992266360606663511134684085845947265625f,0.6105999638006094354381048106006346642971038818359375f,-0.8649999853968619589750232989899814128875732421875f,1.25770000662207603880915485206060111522674560546875f,0.50970001219511029688646885915659368038177490234375f,1.046199984262883742047733903746120631694793701171875f,0.5178000028550624467271745743346400558948516845703125f,0.5716999915421008271465552752488292753696441650390625f,1.0942000125527380216539086177363060414791107177734375f,0.190700007419288386500966225867159664630889892578125f,-0.633900017297267748261901942896656692028045654296875f,0.08619998448640131538667219501803629100322723388671875f,-0.55199999672546995466149155618040822446346282958984375f,-0.29270001519024313463290809522732160985469818115234375f,0.2401999887406827127733777160756289958953857421875f,-0.846799971272051532622526792692951858043670654296875f,-0.343400015419721427178956218995153903961181640625f,-0.06169999262243541782169131693080998957157135009765625f,-0.3561000233829021421882998765795491635799407958984375f,-2.29129996831417059865998453460633754730224609375f,-0.534000015616417300634566345252096652984619140625f,-1.01669999943971589573266101069748401641845703125f,0.603900023555755893767127417959272861480712890625f,2.08890005693435654166023596189916133880615234375f,-0.9214000208735466657117285649292171001434326171875f,-0.701299942088127181705203838646411895751953125f,1.6794000062108036530617027892731130123138427734375f,-1.5917000021219251948423334397375583648681640625f,-1.45969999401569339170237071812152862548828125f,0.4977000020027162463520653545856475830078125f,-0.3454000224828721599124037311412394046783447265625f,-0.370199978458881417964221327565610408782958984375f,-1.2596999886512758592971295001916587352752685546875f,0.81740001032352438414818607270717620849609375f,0.0795000048279757010050161625258624553680419921875f,-0.27450001347065011714221327565610408782958984375f,-1.00530001578181948929113787016831338405609130859375f,0.239399986466765302139947380055673420429229736328125f,-0.34849999252334240562589684486738406121730804443359375f,0.9394000063598155758626262468169443309307098388671875f,1.513599989296495973434275583713315427303314208984375f,-0.855899988746642748083104379475116729736328125f,-0.279799997997283611539387493394315242767333984375f,-1.440499963581562070658037555404007434844970703125f,0.54930002176836101313028848380781710147857666015625f,1.133699971168488307426969186053611338138580322265625f,-1.9907000350050625403497406296082772314548492431640625f,-0.16020001213550560947851408855058252811431884765625f,-1.00399998506903642692122957669198513031005859375f,0.1521999817594881232452053154702298343181610107421875f,-0.062200007873773444089238182641565799713134765625f,0.742200029145181350287430177559144794940948486328125f,1.19230003003478035594753237091936171054840087890625f,-0.56579999638795808181157553917728364467620849609375f,0.11900001785159108891321011469699442386627197265625f,0.3130999722942708896056274170405231416225433349609375f,-1.06539998970031746239328640513122081756591796875f,0.59860001317560662670302917831577360630035400390625f,0.53189999458789838371330915833823382854461669921875f,0.4985999927982687207617118474445305764675140380859375f,0.0451000035345554550758606637828052043914794921875f,1.067200013983249817783871549181640148162841796875f,0.89749998673796671511126987752504646778106689453125f,2.17839996490478515767108547152020037174224853515625f,-0.32680000580549251054662818205542862415313720703125f,0.1578999658212068002427486135275103151798248291015625f,0.309599984036386055574752163010998629033565521240234375f,-0.11620000363439320079095296023297123610973358154296875f,0.83029999337419868599141636877902783453464508056640625f,-0.27690000360310074167813354506506584584712982177734375f,-0.6874999794363976235445079510100185871124267578125f,-0.895699987377226225504500689567066729068756103515625f,-0.611199984747171409793509155861102044582366943359375f,0.8937999923236665811288048644200898706912994384765625f,0.3714999938607215224095625671907328069210052490234375f,0.5188000034250317415995823466801084578037261962890625f,-0.58360001697540297982413903810083866119384765625f,0.266999994993209810445478069595992565155029296875f,0.32630001271963049447322191554121673107147216796875f,0.794300013852119235480131465010344982147216796875f,-0.93730003048181576019715066649951040744781494140625f,0.109800003361701925541638047434389591217041015625f,1.04690000114440895373491002828814089298248291015625f,-0.60740000913143177285746787674725055694580078125f,-1.0425000059604645485933360760100185871124267578125f,-0.21149999207258218802962801419198513031005859375f,0.4078999911904332975609577260911464691162109375f,0.81379999096393618174261064268648624420166015625f,-0.23339999656975241038026069873012602329254150390625f,-0.71540000430345518367403201409615576267242431640625f,-1.243000045090913996403969576931558549404144287109375f,-0.493099977397918110000318847596645355224609375f,1.2621000416107477359872746092150919139385223388671875f,1.4608999769747257335694712310214526951313018798828125f,1.487100012050568853538834446226246654987335205078125f,0.223999976277351464659659541212022304534912109375f,-0.7183000097990035470729708322323858737945556640625f,-0.419100036287307631255316664464771747589111328125f,-2.03580005584359202686073331278748810291290283203125f,0.4970000343322755753661112976260483264923095703125f,1.21620002363920232113514430238865315914154052734375f,-0.4745000092983246986477752216160297393798828125f,-0.03470002903342273015141472569666802883148193359375f,0.604999959021807143244586768560111522674560546875f,-0.0113000206172468864451730041764676570892333984375f,-1.8708999477311969439341510224039666354656219482421875f,0.350799983274936888477668617269955575466156005859375f,0.4717000374376774107787468892638571560382843017578125f,0.366000004351139107683366091805510222911834716796875f,0.4525000007450581396284405855112709105014801025390625f,0.213199994492530908729577276972122490406036376953125f,-0.0955000083595514137613236016477458178997039794921875f,-0.55750000193715099072733210050500929355621337890625f,-0.335400002440810141024485346861183643341064453125f,0.1777999961495400071953554288484156131744384765625f,0.591099991643428523957481957040727138519287109375f,0.41180000312328335354550290503539144992828369140625f,0.149000012263655445821086686919443309307098388671875f,-0.38340000044405453394347205176018178462982177734375f,0.07519998859167065230479920501238666474819183349609375f,0.22850000926852231941666104830801486968994140625f,0.13990002142488922043384036442148499190807342529296875f,0.74869998618140820578759075942798517644405364990234375f,-0.427799974319338804917833840590901672840118408203125f,-0.4369999948441982695612750831060111522674560546875f,0.14310002981275304900776745853363536298274993896484375f,-0.532000013917684366759885961073450744152069091796875f,0.29439999328404642398027135641314089298248291015625f,0.6919000126183032506332892808131873607635498046875f,1.0760000004395842143622985531692393124103546142578125f,-0.607099997037649163900141502381302416324615478515625f,-0.96289999148845695220444440565188415348529815673828125f,0.2581999827206136188806340214796364307403564453125f,-0.549800009880960782737702174927107989788055419921875f,-0.2795999982111154746888814770500175654888153076171875f,-1.0599999915063385014235564085538499057292938232421875f,-0.56170002803131946489401116195949725806713104248046875f,0.0797000186398626464523431423003785312175750732421875f,1.808700013394653804965628296486102044582366943359375f,0.1934999641180041862753569148480892181396484375f,-2.09120001492202245430007678805850446224212646484375f,-0.55419998841285700308390005375258624553680419921875f,-0.261000013142824283107756855315528810024261474609375f,-1.70709998139142982154226046986877918243408203125f,0.6497000369124112939545057088253088295459747314453125f,0.9645999827325340003625342433224432170391082763671875f,-0.16980002925246984180063236635760404169559478759765625f,1.260900022721290980598496389575302600860595703125f,0.12400004273653042474734320421703159809112548828125f,0.29319996647834756231532082892954349517822265625f,0.969100025856494795561957289464771747589111328125f,0.18699999320507032507521216757595539093017578125f,2.0126000385522839764007585472427308559417724609375f,0.86220000534057561480949516408145427703857421875f,-0.3935000341534617973593412898480892181396484375f,1.251700017023086530798536841757595539093017578125f,0.179600033247471202457745675928890705108642578125f,-1.40889995977878612620770581997931003570556640625f,-0.7156999940454955577706641633994877338409423828125f,0.3148000295877455556592394714243710041046142578125f,0.89880003469884428568548173643648624420166015625f,1.03840002629756877894351418945007026195526123046875f,1.180600039535760981124212776194326579570770263671875f,0.270299992610514294710810645483434200286865234375f,-0.68969999749884014494938355710473842918872833251953125f,-0.091000026572495451571143121327622793614864349365234375f,0.2066000097073613928078117396580637432634830474853515625f,0.32090000839754917461021932467701844871044158935546875f,-0.52429999448060993216813585604541003704071044921875f,-0.6595999875381590715761603860300965607166290283203125f,0.067899996182322563953448479878716170787811279296875f,0.596099977263808256111587979830801486968994140625f,-0.58149999967217436136479591368697583675384521484375f,0.34269999671280382269600295330747030675411224365234375f,0.56359997948035600590088733952143229544162750244140625f,-0.47509999541342240458874357500462792813777923583984375f,0.864700015920400577584814527654089033603668212890625f,-0.9648999910265210200321917000110261142253875732421875f,0.490899963861703980683159898035228252410888671875f,-1.1154999912977217046972100433777086436748504638671875f,0.32689998147487653312737165833823382854461669921875f,-0.157599966095387511444414485595189034938812255859375f,-0.066799992096424176679647644050419330596923828125f,-1.7869999952167268109093356542871333658695220947265625f,0.72920000972151743923177491524256765842437744140625f,-0.54940001712590424975957148490124382078647613525390625f,-0.6657999813377859510410416987724602222442626953125f,0.3111999641463165655608236193074844777584075927734375f,0.5064000070154666133248611004091799259185791015625f,0.123799985599518269641095002953079529106616973876953125f,0.486400012752413513084093210636638104915618896484375f,-0.00430000900924187856588787326472811400890350341796875f,1.09260006090402672640493619837798178195953369140625f,0.6124999976158136405501863919198513031005859375f,0.9553000056862828159864875487983226776123046875f,0.5858999848723414061169023625552654266357421875f,0.317300025564431820157551555894315242767333984375f,1.05019998576045026084102573804557323455810546875f,-0.84910000730454893158594131818972527980804443359375f,0.81270000274777398630021707504056394100189208984375f,-0.3215000161007048884442838243558071553707122802734375f,0.588400028756260962126134472782723605632781982421875f,1.140900027079880363345409932662732899188995361328125f,-0.39959998001307273085558335878886282444000244140625f,-0.223200001235306810798419974162243306636810302734375f,0.0632000141620636668449151329696178436279296875f,1.67240003654956836953715537674725055694580078125f,1.218699977743625595394405536353588104248046875f,-0.9481000477313994423411713796667754650115966796875f,-1.310500002622603687996161170303821563720703125f,0.0695000061690809189940409851260483264923095703125f,0.248099956983327984971765545196831226348876953125f,-0.88289995945095967044835560955107212066650390625f,-0.23279998631477383241872303187847137451171875f,0.489700013089180430370106478221714496612548828125f,-1.036099980616569382618763484060764312744140625f,0.407400041578709792844392723054625093936920166015625f,1.354999983608722313732641850947402417659759521484375f,1.0626999879211183053229206052492372691631317138671875f,0.70449997219443361018420546315610408782958984375f,0.225699975270032382468343712389469146728515625f,-1.58790000422894905796056264080107212066650390625f,0.3817999861359595570320379920303821563720703125f,-1.489199981260299399110635931720025837421417236328125f,0.3556999903202058277429387089796364307403564453125f,0.2168000243201848586949154196190647780895233154296875f,0.396399996882677019272023244411684572696685791015625f,-0.0909999947026369770952669568941928446292877197265625f,0.57839999329149727191889951427583582699298858642578125f,-0.44720000388771286026212692377157509326934814453125f,0.658800009329616909070637120748870074748992919921875f,-0.74899998443573678752471778352628462016582489013671875f,-0.148499997645616499397647203295491635799407958984375f,-0.684600007878243932424311424256302416324615478515625f,0.37840000051856037543984712101519107818603515625f,0.602300004442036129859161519561894237995147705078125f,0.17570000298619259382348900544457137584686279296875f,0.03520000483393648238461537403054535388946533203125f,-0.84989998394846910656497129821218550205230712890625f,-0.4386000105679042082584828676772303879261016845703125f,0.12509997850060461388466137577779591083526611328125f,-0.9718000279337168922921819103066809475421905517578125f,0.4539999899864195498366825631819665431976318359375f,0.176199975508451789352193372906185686588287353515625f,-0.211599999481439482451605726964771747589111328125f,-0.0166999916538597137360966371488757431507110595703125f,0.9412999751314521379441657700226642191410064697265625f,0.06080002321004862597675355573301203548908233642578125f,0.102199995505809937412777799181640148162841796875f,-1.4319999819546946895343353389762341976165771484375f,-0.263899980938434364219347116886638104915618896484375f,-1.264099933618307858296248014084994792938232421875f,-0.372699968624115030024768202565610408782958984375f,-0.1361999643325813025285242474637925624847412109375f,-0.2308000326722865480633117840625345706939697265625f,-0.51409999918341586777614793390966951847076416015625f,0.950700018632411936181370037957094609737396240234375f,-0.06990001509189625039653037674725055694580078125f,0.0371000292241578932816992164589464664459228515625f,1.072599991017579412755367229692637920379638671875f,-0.4794000162690874589088707580231130123138427734375f,0.9693999955564738257862700265832245349884033203125f,-1.1062000028520824201905270456336438655853271484375f,-0.52659995909929335766719304956495761871337890625f,1.798500013142824371925598825328052043914794921875f,0.1472999956130980336865832214243710041046142578125f,0.04059999408721903790819851565174758434295654296875f,1.55299996387958572086063213646411895751953125f,0.6935999937295918016388895921409130096435546875f,0.93979995985031195004921755753457546234130859375f,0.21549998429417627221482689492404460906982421875f,-0.623399994260072531915284343995153903961181640625f,-0.5824000048100952398044682922773063182830810546875f,-0.43579997195005404364565038122236728668212890625f,0.34130002646595247739469414227642118930816650390625f,0.571599997469782739045740527217276394367218017578125f,0.71439996396601157524486325200996361672878265380859375f,0.269099966102839260884138639084994792938232421875f,0.1012000172317026880364210228435695171356201171875f,0.38019997406303918108250172736006788909435272216796875f,-0.2714999849200250281455737422220408916473388671875f,0.6045999536380179950612046013702638447284698486328125f,0.36159999516010277176292220246978104114532470703125f,1.0948999692335725786307421003584749996662139892578125f,-0.038099999675154538891774791409261524677276611328125f,1.038699995252489571839760174043476581573486328125f,-0.42329999758005154575357664725743234157562255859375f,-0.175100013630091755345574711100198328495025634765625f,0.2263999985218045907231498858891427516937255859375f,-0.218699984374641775985992353525944054126739501953125f,0.374200000184774506095664037275128066539764404296875f,-0.2230000135377048220419737845077179372310638427734375f,0.02640000761151295449735698639415204524993896484375f,0.0820999771818524681776807483402080833911895751953125f,-0.3185999922394753269827560870908200740814208984375f,-0.8089999883100393862633836761233396828174591064453125f,0.625099996232986399036235525272786617279052734375f,-0.504799956311285935584010076127015054225921630859375f,0.0965999990344048597989967674948275089263916015625f,-1.7230999487876894615823175627156160771846771240234375f,-0.4016999951183795047882085782475769519805908203125f,-0.7558999906837942095449989210464991629123687744140625f,0.15080000801086423933838887023739516735076904296875f,0.730400002403557291330571388243697583675384521484375f,0.373499993883073366784941526930197142064571380615234375f,-0.07079998521208796802284268778748810291290283203125f,-0.0735000027120111809608715702779591083526611328125f,-0.9993999309599406188908687909133732318878173828125f,-0.3716000346481802552034423570148646831512451171875f,-1.780299982738494435352549771778285503387451171875f,-1.1399000067472453423533806926570832729339599609375f,0.84800004792213456994431908242404460906982421875f,-0.952000030905008287618329632095992565155029296875f,-0.380399972862004887019793386571109294891357421875f,0.9038999897301192021359383943490684032440185546875f,0.10949998515844328039747779257595539093017578125f,-0.6592999956727023658231701119802892208099365234375f,-1.76299997997283952599900658242404460906982421875f,0.1590999740004530593751042033545672893524169921875f,-1.238200026679038501242757774889469146728515625f,-0.265800004136561796030946425162255764007568359375f,0.3744999780058861915676970966160297393798828125f,1.265800047945976558594338712282478809356689453125f,0.32980000634193373798552784137427806854248046875f,-1.4714000178933144269421973149292171001434326171875f,0.035999991536140640846497262828052043914794921875f,0.41290004563555182670597787364386022090911865234375f,0.321100035098939695643593950080685317516326904296875f,0.065999943815171047134526816080324351787567138671875f,0.6782999686717994247686647213413380086421966552734375f,1.1536000018134711098127809236757457256317138671875f,-0.13840006980895935839726007543504238128662109375f,0.378099978590011431833772803656756877899169921875f,1.40299999487400128828085144050419330596923828125f,-0.1618999743223188403362655662931501865386962890625f,-1.98999999463558197021484375f,-0.0831000197172162557990304776467382907867431640625f,-0.31729998890757615814806058551766909658908843994140625f,-0.9569999794960022398981891456060111522674560546875f,1.112700007665157198033512031543068587779998779296875f,0.671000027000903997986824833787977695465087890625f,1.6020000225603581167632682991097681224346160888671875f,-0.0715999859213827249959649634547531604766845703125f,0.3204999901801344197593834906001575291156768798828125f,-0.23300000023841871410468229441903531551361083984375f,-0.9309999942928548488652040759916417300701141357421875f,-0.72560000198483454170172990416176617145538330078125f,1.5820999660804861974838786409236490726470947265625f,-0.8434999983906745768535984097979962825775146484375f,-0.41940000758916173850820996449328958988189697265625f,0.2822999859273433997941538109444081783294677734375f,-0.4091999873697755862878011612338013947010040283203125f,-0.14349999243021027695021984982304275035858154296875f,-1.0002000324383377716230825171805918216705322265625f,0.522999985039234172035094161401502788066864013671875f,1.2716000095583501749985089190886355936527252197265625f,0.1708000124812125708473331542336381971836090087890625f,0.05499999994412052561898462954559363424777984619140625f,0.30439999322816746474273941203136928379535675048828125f,-1.0851999566286811838011772124445997178554534912109375f,-0.323500027000903855878277681767940521240234375f,-1.4713999499812728810610451546381227672100067138671875f,0.74210001029968264418812395888380706310272216796875f,-0.612599992172420559910506199230439960956573486328125f,-0.89789999953508381480560274212621152400970458984375f,0.8629999693557619533379465792677365243434906005859375f,0.5844999938383690363252753741107881069183349609375f,-0.09179999839216446133605131763033568859100341796875f,0.037399978174268877495478591299615800380706787109375f,-1.99440001306533787328589824028313159942626953125f,0.03630001778602576223420328460633754730224609375f,-0.231400043821334833182845613919198513031005859375f,-1.03120002512931829841136277536861598491668701171875f,1.49449999797344190710646216757595539093017578125f,0.0552999904870983272076045977883040904998779296875f,1.728999971508979882628409541212022304534912109375f,0.4945999959945677204586900188587605953216552734375f,0.9592000241756437617368646897375583648681640625f,-0.3318999442219734419268206693232059478759765625f,-0.646799993586540722390054725110530853271484375f,0.77840001638829736663183211931027472019195556640625f,-0.118699998120963545744643852231092751026153564453125f,-0.89369999432116753457222557699424214661121368408203125f,0.1986999911546707409115697373636066913604736328125f,0.302600020968913696606250596232712268829345703125f,-0.811400030708313124705455265939235687255859375f,-1.28660001721382144523886381648480892181396484375f,-0.529600012683868470730885746888816356658935546875f,1.777000048749149296867244629538618028163909912109375f,-0.648400022721290714144970479537732899188995361328125f,-0.3963000193506480695049276619101874530315399169921875f,0.4823000000834465339494272484444081783294677734375f,0.28279998456388710792452911846339702606201171875f,-0.407599988782405997511659734300337731838226318359375f,0.2353999833673239283626799078774638473987579345703125f,0.1516000107690691489636947153485380113124847412109375f,0.010400001621246435234979799133725464344024658203125f,0.0762999949499960639087703384575434029102325439453125f,-1.0304000065386293982783172396011650562286376953125f,1.4047000068679447171149377027177251875400543212890625f,-0.382900020098686066916116033098660409450531005859375f,0.5838000115647912213745485132676549255847930908203125f,0.79609999417662624665581461158581078052520751953125f,-0.26739997314512731207969409297220408916473388671875f,0.8995999819874764824589874478988349437713623046875f,-1.804299977791309128605234946007840335369110107421875f,0.688800006312131785080055124126374721527099609375f,-0.938600025580823338344771400443278253078460693359375f,0.16269998278021802207149448804557323455810546875f,0.6460999639272693162439509251271374523639678955078125f,-0.009899999929964542477733857595012523233890533447265625f,-0.075900000295043124065585971038672141730785369873046875f,0.528299979847669742838434103759936988353729248046875f,1.4675999825611707461092692028614692389965057373046875f,-0.5243000178754328999275458045303821563720703125f,-0.26280005828738239870290271937847137451171875f,0.229199978429079020969538760255090892314910888671875f,0.6100999910548330351645063274190761148929595947265625f,-0.668400007559359021769296305137686431407928466796875f,0.240500013798475353876682447662460617721080780029296875f,0.0604000059425831015147423386224545538425445556640625f,-0.0881000036239623085521088796667754650115966796875f,-0.5156999841734768441625647028558887541294097900390625f,-1.1744999705925582844656673842109739780426025390625f,-1.0647999515056607577889735694043338298797607421875f,0.5862000078439706385324825532734394073486328125f,-0.39510005203783560290276000159792602062225341796875f,-0.62710000895857742619909913628362119197845458984375f,-0.057500015497207623837994105997495353221893310546875f,-1.962499957680702067364109097979962825775146484375f,0.35119999264478707345915609039366245269775390625f,-1.703999988198280135520690237171947956085205078125f,-0.56309996756315161547945535858161747455596923828125f,-1.09979993392229147275429568253457546234130859375f,0.17649998588859983073007242637686431407928466796875f,-0.06459997870922062901399840484373271465301513671875f,-0.428799998712539487399908466613851487636566162109375f,0.430100014263391727808993891812860965728759765625f,-0.67930002622008345269932760857045650482177734375f,-0.64759997835159310852759517729282379150390625f,-0.0518000412702566137568283011205494403839111328125f,1.251899986839294598439664696343243122100830078125f,0.364699998486042176182309049181640148162841796875f,1.54390000239610714061200269497931003570556640625f,-0.86829999694675219501505125663243234157562255859375f,-0.382199973116815527873768587596714496612548828125f,0.45809999509304777376428319257684051990509033203125f,0.071000003159046087830574833787977695465087890625f,-0.14310002031326352067708285176195204257965087890625f,-1.96989999184608421955999801866710186004638671875f,0.03840005192756734686554409563541412353515625f,1.277099993014335410634885192848742008209228515625f,1.10220001010894730342215552809648215770721435546875f,0.5542999868437650246022485589492134749889373779296875f,0.74159998100399970866192234097979962825775146484375f,1.1495999666020273455302458387450315058231353759765625f,-0.085400003036856464433412838843651115894317626953125f,0.8435999893337484678568216622807085514068603515625f,-0.96789997181892406530323569313623011112213134765625f,0.11649996521323913167833552506635896861553192138671875f,-0.910299987208843308650330072850920259952545166015625f,-0.4809000097200268886155072323163039982318878173828125f,-0.1690999857723713972745827049948275089263916015625f,0.55199996407330065295582244289107620716094970703125f,0.208100011596083600551310155424289405345916748046875f,1.28410002146065238548544584773480892181396484375f,0.30070000835061083677146598347462713718414306640625f,1.702100020059943208394770408631302416324615478515625f,-0.47079998789429655658977935672737658023834228515625f,-0.0528000065431000198401534362346865236759185791015625f,0.60069999806880947090803601895458996295928955078125f,-0.374599942313134537386076772236265242099761962890625f,0.65710001119375238687325690989382565021514892578125f,-0.0164999885708094762293285384657792747020721435546875f,-0.0895999926418067360600616666488349437713623046875f,-0.28170001124888610632979180081747472286224365234375f,0.059699996548891132164271766669116914272308349609375f,-0.6239000339120630567180114667280577123165130615234375f,-0.8079999911487103503304751939140260219573974609375f,0.5665000163242221997705883040907792747020721435546875f,0.835399983143806590391022837138734757900238037109375f,0.033000004261732218679981087916530668735504150390625f,-0.625700002837180857540033684927038848400115966796875f,-0.15069996826648690557703957892954349517822265625f,0.96130004328563867233015116653405129909515380859375f,-0.33070000550076361633955457364208996295928955078125f,-0.02839999874904741705705646381829865276813507080078125f,-0.2048999928891657162921546841971576213836669921875f,0.51539999953508353058850843808613717555999755859375f,-1.3636000166773794006758180330507457256317138671875f,0.259500039994717379698840886703692376613616943359375f,-1.5473999691963200575628434307873249053955078125f,0.8364000294789679657014858094044029712677001953125f,0.17440002442747282174195788684301078319549560546875f,-1.167399973778426858217471817624755203723907470703125f,0.27070002846717766686879258486442267894744873046875f,1.979299978834390127957476579467765986919403076171875f,-0.748300058078765317759462050162255764007568359375f,-0.9602000024497510111132214660756289958953857421875f,0.2046000004649162207215340458787977695465087890625f,0.18960001365244405491239376715384423732757568359375f,-1.59880000884532957883266135468147695064544677734375f,0.535899999922513803340962112997658550739288330078125f,0.320399974203109394466082449071109294891357421875f,-0.36120003421902691087552739190869033336639404296875f,0.063299992066621513231439166702330112457275390625f,1.5260000180602073527325046597979962825775146484375f,-0.6653000381708142896286517498083412647247314453125f,-0.77530000464320192321565627935342490673065185546875f,1.1736999926447868602963353623636066913604736328125f,-1.000000025555491500739435650757513940334320068359375f,-0.3660000404492020908975291604292578995227813720703125f,0.41370001261234268241651079733856022357940673828125f,0.1655999857611953895997203289880417287349700927734375f,-0.102700004200637351470248859186540357768535614013671875f,-0.188399997258931417576377498335205018520355224609375f,0.037400001531839421886616037227213382720947265625f,-0.302500026822090595235437149312929250299930572509765625f,-0.6927000045359135160794039620668627321720123291015625f,0.220499965108931217372401079046539962291717529296875f,-0.88569998301863694933899751049466431140899658203125f,-0.6908000219061960489597140622208826243877410888671875f,-0.046599995011091355223697973997332155704498291015625f,-0.6109000157922499507634483961737714707851409912109375f,0.612299991217255534792229809681884944438934326171875f,0.51279998042881491659983339559403248131275177001953125f,-0.85270001496672609420102162403054535388946533203125f,-0.54559997613132049654183219900005497038364410400390625f,0.212499998062849027036236293497495353221893310546875f,0.7226999884054066125571580414543859660625457763671875f,-1.016400007238984048996144338161684572696685791015625f,1.1157000065252187237518910478684119880199432373046875f,-0.418700006353855069818337142351083457469940185546875f,-1.58610001306384784669489818043075501918792724609375f,-0.00490000734329232301433876273222267627716064453125f,-0.794699986751377540628027418279089033603668212890625f,-0.776600008273124586821722914464771747589111328125f,0.023800026614963776250277760482276789844036102294921875f,0.5344999994635581952451275355997495353221893310546875f,0.3040999995559452440119230232085101306438446044921875f,-1.122300013792514761234997422434389591217041015625f,0.7542000387787819448703885427676141262054443359375f,0.511700022387505004672902941820211708545684814453125f,-1.2608999669909479734997148625552654266357421875f,-0.231700005996227531568365520797669887542724609375f,-0.675700007605552599443399230949580669403076171875f,0.7709999778270724135609270888380706310272216796875f,-0.5476999543190004260395653545856475830078125f,0.177300020945072134281872422434389591217041015625f,2.12399999356269830741439363919198513031005859375f,-0.96189998818039867956031230278313159942626953125f,0.3585000028610227928993481327779591083526611328125f,-0.6160999940089884052696334038046188652515411376953125f,0.0497000340998170475614870156277902424335479736328125f,0.59040000128596992201579496395424939692020416259765625f,-0.818300054502486773344571702182292938232421875f,-0.1343000396311282429451239295303821563720703125f,-0.8980999723315239435805779066868126392364501953125f,-0.395500051945448571899532907991670072078704833984375f,-1.2852999839305869755889943917281925678253173828125f,0.43560001360774069922854323522187769412994384765625f,0.2062999802350997669009302626363933086395263671875f,-1.721700052785873680250006145797669887542724609375f,-0.29279996724128665164244011975824832916259765625f,1.661200039397180194811198816751129925251007080078125f,0.751999969363212894535308805643580853939056396484375f,0.0624000267893076721037459719809703528881072998046875f,-1.014300000739098095436929725110530853271484375f,1.302899953639506946956316824071109294891357421875f,-0.79700000154972061583436016007908619940280914306640625f,0.8120000148117541005632347150822170078754425048828125f,0.41619998921752010545560551690869033336639404296875f,-0.607399986928701451915912912227213382720947265625f,0.9602999817326665077388270219671539962291717529296875f,-0.400100002193450876575298025272786617279052734375f,-0.23390000399798116514915591324097476899623870849609375f,-0.5397999674499036615316072129644453525543212890625f,-0.777299983058869603524954072781838476657867431640625f,1.04180000029206265566017464152537286281585693359375f,-0.822599986360966983767184501630254089832305908203125f,0.597199984031915764859377304674126207828521728515625f,0.57199997949600234203870741112041287124156951904296875f,0.49730000345483416079872540649375878274440765380859375f,1.812199985149502534653720431379042565822601318359375f,-0.6756000209093091513068429776467382907867431640625f,-0.083599984937906324233836130588315427303314208984375f,-0.6022999832451343404926547009381465613842010498046875f,0.3779000033721324758317905434523709118366241455078125f,-0.70709997483491893177642850787378847599029541015625f,0.568400022199749610507524266722612082958221435546875f,-0.174200004804134334079890322755090892314910888671875f,-1.0159000066652896787644522191840223968029022216796875f,0.48920001121163370072508769226260483264923095703125f,0.4441000232860441432336529032909311354160308837890625f,0.2813999887987972403635694718104787170886993408203125f,-0.146999985307455116601005329357576556503772735595703125f,-0.6715000059306621693622219027020037174224853515625f,-0.788800018978118799850562936626374721527099609375f,0.175400036787987545494615915231406688690185546875f,-0.401500015616417016417472041212022304534912109375f,-0.74690001321434973391433231881819665431976318359375f,-1.5359999372959141084038492408581078052520751953125f,0.1991000023685398634398779904586263000965118408203125f,-1.3295999818384649504565686584101058542728424072265625f,-0.90089999973624934614946369038079865276813507080078125f,0.4021999984860418209109411691315472126007080078125f,0.828600016677379702656480731093324720859527587890625f,0.7576999930620189616092829965054988861083984375f,-0.4997000211358066934508315171115100383758544921875f,0.0256999828696251597648370079696178436279296875f,0.64269999280124867624408580013550817966461181640625f,0.33299996145814692027897763182409107685089111328125f,0.823899998857080984038248061551712453365325927734375f,0.97289996556043689679427188821136951446533203125f,0.2892000210836531692848438979126513004302978515625f,-0.15300001443177446702748056850396096706390380859375f,0.207300021280348634178380962111987173557281494140625f,0.9596000070214270039059556438587605953216552734375f,-0.058599963033199031769981957040727138519287109375f,-1.712599993848801460671893437393009662628173828125f,0.5431999687135213861211013863794505596160888671875f,0.9956999721407893133573452359996736049652099609375f,1.21950004483759411044729858986102044582366943359375f,0.1998000038832419278378438320942223072052001953125f,-0.37790003149807471771737255039624869823455810546875f,-1.1721999949097625659533150610513985157012939453125f,-0.08770000692009904952328724903054535388946533203125f,-0.9769999581128361310078389578848145902156829833984375f,-0.425300006580352718543736045830883085727691650390625f,0.030099977941811406623173752450384199619293212890625f,-0.72099996873736404978671998833306133747100830078125f,1.014599959039688581441396308946423232555389404296875f,-0.88449998607113966553328054942539893090724945068359375f,0.5475999834924938358682311445591039955615997314453125f,0.3054999929368495958925677769002504646778106689453125f,0.94829998517483493625235269064432941377162933349609375f,-0.09369999145269414242420680238865315914154052734375f,-0.7816999997004863143246211620862595736980438232421875f,-0.108100007349252624777591336169280111789703369140625f,-0.94559999218732160297662403536378405988216400146484375f,0.052999987125396774700902824406512081623077392578125f,-1.241099968360364957931096796528436243534088134765625f,0.38339999470710761642067154753021895885467529296875f,-1.0130999722942715113305212071281857788562774658203125f,-0.1029000043034553613097159541212022304534912109375f,-0.33499999061226837238081088798935525119304656982421875f,0.501300003033876340197139143128879368305206298828125f,-0.100100010873376887854391270593623630702495574951171875f,-0.90980001245141028931584514793939888477325439453125f,-0.31030001667588924529894711668021045625209808349609375f,0.47329998714923871006021727225743234157562255859375f,-1.7450999861747025221347939805127680301666259765625f,-0.01220000399947185343307864968664944171905517578125f,-0.1272000034406779445106394632603041827678680419921875f,-0.56409998651742920827700800145976245403289794921875f,1.7206000274658201920630062886630184948444366455078125f,-0.1751000024914741715065247262828052043914794921875f,-0.6029000050485127104593630065210163593292236328125f,0.59200000080466264051892721909098327159881591796875f,-0.457699999618530029010798898525536060333251953125f,-1.3036000347822902512007203768007457256317138671875f,-0.86910002764463367697089779539965093135833740234375f,1.097100025200843820272211814881302416324615478515625f,-0.2499000162839895011757107567973434925079345703125f,-0.70800001273676738122730967006646096706390380859375f,1.088100001817196460507375377346761524677276611328125f,-0.4187000329338015358615621153148822486400604248046875f,-0.7996999823182822098033284419216215610504150390625f,1.27610002271234979076552917831577360630035400390625f,-0.350699977952242392120751901529729366302490234375f,-0.622899997895956403226591646671295166015625f,1.080100024247169443469829275272786617279052734375f,0.43760001053810082538575443322770297527313232421875f,1.1128999670505521635277546010911464691162109375f,1.28230003081709131862453432404436171054840087890625f,-1.550900038293004268297181624802760779857635498046875f,-0.00050001834332979999686585870222188532352447509765625f,0.6944999875426294266844706726260483264923095703125f,0.186599976310133808254931864212267100811004638671875f,0.8809999564439063846066346741281449794769287109375f,-1.42910000357925870417830083169974386692047119140625f,0.101000041604041967957527958787977695465087890625f,0.49200003701448480342151015065610408782958984375f,0.705499993979930906107256305404007434844970703125f,-1.24670001702308663737994720577262341976165771484375f,-0.38559998324662470015056214833748526871204376220703125f,-0.04500000104308110593365199747495353221893310546875f,1.525200004163383749045124204712919890880584716796875f,-0.572199981796741585782228867174126207828521728515625f,0.5694000056892629668681138355168513953685760498046875f,-0.404400011798739189572415853035636246204376220703125f,0.679800003957748089078449993394315242767333984375f,-0.6399000024259091645717489882372319698333740234375f,0.0179000090904539721492483295151032507419586181640625f,0.364000004440546032213887883699499070644378662109375f,-0.775499996904283506182764540426433086395263671875f,-0.31889999673366542509711507591418921947479248046875f,-1.1092999919101591554948527118540368974208831787109375f,-0.25369999547600752265452683786861598491668701171875f,-0.1666999814093108600587811451987363398075103759765625f,0.808900004780292558592691420926712453365325927734375f,-0.4409000010773540534358971854089759290218353271484375f,0.71439997149109846219516839482821524143218994140625f,-0.736599997395277039657912609982304275035858154296875f,-1.056300023336708360233160419738851487636566162109375f,0.2383000035777687042326533628511242568492889404296875f,0.1499000030219554968624606772209517657756805419921875f,-0.2437999754294757348560551690752618014812469482421875f,0.213099990361929059190515545196831226348876953125f,-0.9755999980360272605395266509731300175189971923828125f,0.57250001206994038938091762247495353221893310546875f,0.928899994871020595610389136709272861480712890625f,0.444099994117021612538565022987313568592071533203125f,-0.1822999789983033025464465026743710041046142578125f,0.5808000075265766781740239821374416351318359375f,0.44959996548444092212548639508895576000213623046875f,0.860600026385486049917972195544280111789703369140625f,0.666499979257583419212096487171947956085205078125f,1.321200010526180079750702134333550930023193359375f,0.53379993016719851084417314268648624420166015625f,-1.0256000697851188618869855417869985103607177734375f,-1.79349998140335031848735525272786617279052734375f,0.033600023233891107565796119160950183868408203125f,0.95789999536275871605539578013122081756591796875f,0.3156999953866002073255003779195249080657958984375f,0.182300001275539358402966172434389591217041015625f,0.02300002072751539117234642617404460906982421875f,0.37979999457001678564438407192938029766082763671875f,-1.3592000368416312650055033373064361512660980224609375f,0.687000014662743296867120079696178436279296875f,-0.0369999707043167092024305020458996295928955078125f,-1.0171999698758131813747240812517702579498291015625f,0.32799999167025095658800637465901672840118408203125f,-0.34229999665617949489160309894941747188568115234375f,0.2451000031992791594603886551340110599994659423828125f,-0.180500012680888222149633293156512081623077392578125f,-1.586699996981024707309870791505090892314910888671875f,-0.2960999704837803392365458421409130096435546875f,0.780700011777877733720742980949580669403076171875f,0.124599989438056724111447692848742008209228515625f,-0.31940001813173335420970033737830817699432373046875f,-0.929100027346610346512534306384623050689697265625f,0.4843000129580499191206399700604379177093505859375f,-1.2329999458491809338767097870004363358020782470703125f,0.127099989885091968488950442406348884105682373046875f,-1.207299966146052394577736777137033641338348388671875f,0.20429998866915699551327634253539144992828369140625f,-0.580799949523806535722769694984890520572662353515625f,-0.25250002682208982918155015795491635799407958984375f,-0.77729999743849109705706723616458475589752197265625f,-0.89789997807741173119211453013122081756591796875f,0.4122000225886701674227197145228274166584014892578125f,0.52099999570846566854243064881302416324615478515625f,-0.6210999954432250280689231658470816910266876220703125f,0.1043000011861323628181708045303821563720703125f,-0.40149999356269827632814894968760199844837188720703125f,0.29629999178349974986446113689453341066837310791015625f,0.804400005689263064567740002530626952648162841796875f,0.7580000090301037829476626939140260219573974609375f,1.16499998617917288612488846410997211933135986328125f,-0.4757999931097032231264165602624416351318359375f,0.243999970875680982640432148400577716529369354248046875f,-0.26409999761879443536116696122917346656322479248046875f,0.039900001606345159022026791717507876455783843994140625f,-0.265500003330409517499077765023685060441493988037109375f,0.14859998393207807243499019023147411644458770751953125f,0.2618999980151652806625861558131873607635498046875f,0.1731999667763712835721889859996736049652099609375f,0.139199986922740936989839610760100185871124267578125f,0.5793999975681300984575727852643467485904693603515625f,-0.4519000069558618548626327537931501865386962890625f,0.42230000418126534267315719262114726006984710693359375f,0.620900003796815980194878648035228252410888671875f,0.1137000123143199203923359164036810398101806640625f,0.006900040781498528730253383400849997997283935546875f,-0.83449994134902993891955702565610408782958984375f,-0.1851000015974051393641275353729724884033203125f,-0.432100035333633059053681790828704833984375f,0.3328000308692455888603944913484156131744384765625f,0.768100012713670565744905616156756877899169921875f,-0.3595999872028823318714785273186862468719482421875f,-0.83669998364448616712252260185778141021728515625f,0.3082999454259862659455393441021442413330078125f,-0.9668000317335128102058661170303821563720703125f,-0.04549997937679250981091172434389591217041015625f,1.362100001060962739529713871888816356658935546875f,0.86439999637603737170366002828814089298248291015625f,0.4751000242471690171441878192126750946044921875f,-1.5295000370144844481501422706060111522674560546875f,0.0659000097572803866796675720252096652984619140625f,-1.256100020253658300362076261080801486968994140625f,-0.1811999595642088678459913353435695171356201171875f,0.94870001827478400713289374834857881069183349609375f,-1.643300023508071916467088158242404460906982421875f,-0.6030000187158588431657335604541003704071044921875f,-1.37549998444318788415330345742404460906982421875f,1.870099986398219726879688096232712268829345703125f,0.137799953532219632279520737938582897186279296875f,-0.494799979221820507291340618394315242767333984375f,1.325700010287761188010335899889469146728515625f,-0.0493000251770023822928123991005122661590576171875f,1.0618999992072577498447571997530758380889892578125f,0.789799989689886405841434680041857063770294189453125f,2.230500025290996557458811366814188659191131591796875f,-0.242399989145249239452795109173166565597057342529296875f,-0.29349999706819662748102928162552416324615478515625f,0.808300014045834291920300529454834759235382080078125f,-0.3451999685123550154486338215065188705921173095703125f,-0.48659996684789696974604567003552801907062530517578125f,-1.449899968115985426919678502599708735942840576171875f,0.2379000061661002973778522573411464691162109375f,-0.0525999673992390004428898464539088308811187744140625f,-0.159999999739229792350414527390967123210430145263671875f,-0.200899979880451606817359788692556321620941162109375f,-0.1245999913379549628444920017500407993793487548828125f,-1.6200999913156035603378768428228795528411865234375f,-0.3107999967977403965591065571061335504055023193359375f,0.4809999656081203056601225398480892181396484375f,-0.43430000137239677115985614364035427570343017578125f,-0.219500058472156478472925300593487918376922607421875f,-0.0941000041380524177014876840985380113124847412109375f,-0.07539998664557934215935119937057606875896453857421875f,-0.1668000033095478062961092291516251862049102783203125f,-0.341800019719452008448712376775802113115787506103515625f,-0.0408999994195997640389350635814480483531951904296875f,-0.117399985177815191494943292127572931349277496337890625f,-0.082000005517155027945364054176025092601776123046875f,0.003800051406770865725093244691379368305206298828125f,-0.747199991035461419386365378159098327159881591796875f,-0.913000000685453727555795921944081783294677734375f,-0.5124000027984381144818826214759610593318939208984375f,-0.313599988216161751353183717583306133747100830078125f,0.40829999504685421385374866076745092868804931640625f,0.1616000187039379198949973215349018573760986328125f,1.0028999962195752626570310894749127328395843505859375f,0.2661999796807765772399534398573450744152069091796875f,-0.746899995146691697556207145680673420429229736328125f,-0.45640003368854564769208081997931003570556640625f,-0.7719000187277795799190016623470000922679901123046875f,-0.3204000124990942222069634226500056684017181396484375f,-0.58060003830641504496412608204991556704044342041015625f,-0.140800010991096513635056908242404460906982421875f,0.381200007769465276652454122086055576801300048828125f,0.229400030946731448011632892303168773651123046875f,1.24210000176876800281888790777884423732757568359375f,0.4653999941706654652762154000811278820037841796875f,-0.0717000137448311480881102397688664495944976806640625f,0.000399985229968979894010772113688290119171142578125f,0.16049997714161856610104450737708248198032379150390625f,0.176399963653087521464613018906675279140472412109375f,-0.7930999667435882916066702819080092012882232666015625f,-0.08259997447729094943724703625775873661041259765625f,0.63299998490884934465583455676096491515636444091796875f,0.422600001336634056858798658140585757791996002197265625f,-0.56709998312368981032705050893127918243408203125f,-0.53159999217987063957480131648480892181396484375f,-0.08189999361932276400466435006819665431976318359375f,-0.21839998099803903386373349349014461040496826171875f,0.633099987232685013083255398669280111789703369140625f,-0.8976000269293782363178024752414785325527191162109375f,0.047299993489682723435407751821912825107574462890625f,0.31209999994337511708408783306367695331573486328125f,0.75520000759065109008361105225048959255218505859375f,1.274900066575408885682918480597436428070068359375f,-0.2273000050753353484811469797932659275829792022705078125f,-0.2709000125885017151716738226241432130336761474609375f,-0.7708999998852608115385010023601353168487548828125f,-0.7463000342518093521704258819227106869220733642578125f,-0.65720000697970359926358696611714549362659454345703125f,-1.0270000064671049866404928252450190484523773193359375f,-0.6262999985262751589942809005151502788066864013671875f,-1.0593999939918514741066246642731130123138427734375f,0.368299982455372987288910735514946281909942626953125f,0.39040001935362766971593373455107212066650390625f,-0.4598999901324509753663960509584285318851470947265625f,-1.0801000297605991562477356637828052043914794921875f,-0.313599998274445457280279470069217495620250701904296875f,-0.561999998867511951772257816628552973270416259765625f,-0.79630000083595486870535751222632825374603271484375f,-1.725000046044588675187014814582653343677520751953125f,0.49670001195669144511413151121814735233783721923828125f,0.51800005744025112708328606458962894976139068603515625f,0.425200002282112687890247570976498536765575408935546875f,0.672800014049560246576220379211008548736572265625f,-0.253899988277256571933548912056721746921539306640625f,0.415499969691038639751923255971632897853851318359375f,0.296700006406009020309966217610053718090057373046875f,0.661200017678738038995334136416204273700714111328125f,-0.10830000503063208494580749174929223954677581787109375f,0.2274000347614286710040687466971576213836669921875f,0.0632999819710848843357098303386010229587554931640625f,-0.1525999795436856487640397972427308559417724609375f,0.939800024037063419513060580356977880001068115234375f,-0.2416000276446350625292325275950133800506591796875f,-1.276200024421513035122188739478588104248046875f,-0.14579998714923902980444836430251598358154296875f,0.3745999714449046447128921499825082719326019287109375f,-0.025799986553192155724900658242404460906982421875f,1.2553000133976339736108229772071354091167449951171875f,0.5551999993205072314594872295856475830078125f,-0.93429999757260073778297737590037286281585693359375f,-0.17059999080896393053308202070184051990509033203125f,1.02110000475645090745047127711586654186248779296875f,0.4758000005602838200502446852624416351318359375f,0.8234999738037587224681601583142764866352081298828125f,0.8097999854803088481958184274844825267791748046875f,-0.9395999894008040254078650832525454461574554443359375f,-0.19240000175535654225456028143526054918766021728515625f,-0.203100000811368197606299190738354809582233428955078125f,-0.277800010715424983143151393960579298436641693115234375f,-0.3758000021062792850301548241986893117427825927734375f,-0.24439999011755020319469622336328029632568359375f,-0.164799997960030975008294262806884944438934326171875f,-0.04539997777938875600511892116628587245941162109375f,1.493199978287517826203156801057048141956329345703125f,0.25759997466355588624509209694224409759044647216796875f,-0.252299988684058396071208107969141565263271331787109375f,-0.3370999879479408267712869928800500929355621337890625f,0.40199998340755715986460927524603903293609619140625f,0.42010000055432339394201335380785167217254638671875f,-0.4577999945476651788567323819734156131744384765625f,0.588799993646144770309547311626374721527099609375f,-0.7875000064447517100774120990536175668239593505859375f,-1.73800004509091454707458979100920259952545166015625f,-0.5672000029936430376409361997502855956554412841796875f,-0.459500049531460064144994248636066913604736328125f,0.778899972295761244822642765939235687255859375f,1.12210003548264491968211586936376988887786865234375f,-0.3969999809861184392190125436172820627689361572265625f,-1.547500004768371351104860877967439591884613037109375f,-0.29739999769479030788943418883718550205230712890625f,-0.36909999724626540995586765347979962825775146484375f,-0.8009999629631641138161057824618183076381683349609375f,-2.2079000048995016669550750521011650562286376953125f,0.5589000044077632178840531196328811347484588623046875f,-0.308499950595199745606578289880417287349700927734375f,-0.39039998359084149281983400214812718331813812255859375f,-0.1370999886371191411882364263874478638172149658203125f,-0.835699984415620544808689373894594609737396240234375f,-0.8391000359706577427232332411222159862518310546875f,0.529399967393279435867725624120794236660003662109375f,0.791900005465745238097952096723020076751708984375f,0.17399998719245213241180181285017170011997222900390625f,1.052599977662413976986499619670212268829345703125f,0.08370000175312153345430488116107881069183349609375f,-1.131299968407302625195143264136277139186859130859375f,-1.641799974885582713568510371260344982147216796875f,-0.74759997894763952075436463928781449794769287109375f,1.4284000002950432151038739903015084564685821533203125f,0.42099999451637248881752384477294981479644775390625f,-0.503200005407631323350869934074580669403076171875f,0.15400000488758092132002275320701301097869873046875f,-0.414400002112984555679986442555673420429229736328125f,-0.2397999929308891609025522484444081783294677734375f,0.2894999942481517596348794540972448885440826416015625f,0.1802000049829484140673230285756289958953857421875f,0.16499999243766050316395421759807504713535308837890625f,-0.1280000139474868348088421043939888477325439453125f,0.0614000088036059654683640474104322493076324462890625f,0.3304000119030476856352152026374824345111846923828125f,-0.49140001178383851954123429095488972961902618408203125f,-0.05719999014139176551907439716160297393798828125f,0.03000001246109607055956303156563080847263336181640625f,-0.62560001260191189231818498228676617145538330078125f,0.399800006211548986101433911244384944438934326171875f,0.07319999449253078438459851895458996295928955078125f,0.6151000217884778908938869790290482342243194580078125f,0.70380001480579412742599743069149553775787353515625f,-0.5941000100985169840583921541110612452030181884765625f,0.070999989748000924549842238775454461574554443359375f,0.580300008592009586294580003595910966396331787109375f,0.319200004804134351843458716757595539093017578125f,-0.9141999947831036532619464196613989770412445068359375f,-0.58159999605417223023096084943972527980804443359375f,0.078799968165158829069838475334108807146549224853515625f,0.77169999318122872722369720577262341976165771484375f,-0.58219997792243971534986712867976166307926177978515625f,0.2472999854803088481958184274844825267791748046875f,-0.03609998896121979772289023458142764866352081298828125f,-1.0327000043869016820963224745355546474456787109375f,-0.22199998642504226165073077936540357768535614013671875f,-0.36230001498460762121567313442938029766082763671875f,0.4619999922737478836864966069697402417659759521484375f,0.71390003517866151838688892894424498081207275390625f,-0.323899989506602370425980552681721746921539306640625f,-0.2687000179767604635117095313034951686859130859375f,-0.2372000065326689277611649231403134763240814208984375f,-0.72280001142323035967507394161657430231571197509765625f,0.29699999842047686282597851459286175668239593505859375f,-0.29459997952878413940425161854363977909088134765625f,-0.6025000058487055554223843500949442386627197265625f,0.23949993868544650066354506634525023400783538818359375f,-0.5427999872274698045515606281696818768978118896484375f,0.125599966650456362771137719391845166683197021484375f,0.78169998271316298854571869014762341976165771484375f,0.4924000128567218581565612112171947956085205078125f,0.76400001036375730922856064353254623711109161376953125f,-0.1246999430537216735359606900601647794246673583984375f,-0.7612999994575975204469386881100945174694061279296875f,-1.184300047677755873820615306613035500049591064453125f,0.74789999167472120955579839574056677520275115966796875f,1.102099995696545153833767471951432526111602783203125f,-0.231499994270503606230704463087022304534912109375f,-0.44059999483227674232921344810165464878082275390625f,-0.004399999188631598112664278232841752469539642333984375f,0.393000007949769469828282808521180413663387298583984375f,-0.3186000106237829587740861825295723974704742431640625f,-0.190200004647672338276009895707829855382442474365234375f,0.7977000029899177224734785340842790901660919189453125f,0.096799993288517072187460144050419330596923828125f,-0.5286999975249171423996585872373543679714202880859375f,0.2622000084698201671784545396803878247737884521484375f,-0.2187000003188848129553889521048404276371002197265625f,0.1284000283837318789892378845252096652984619140625f,0.438000003945082649892128756619058549404144287109375f,-0.377200009177625028566893661263748072087764739990234375f,-0.3076000059001149367077232454903423786163330078125f,-0.31280001522302658401031294488348066806793212890625f,-1.2543000118032097134346258826553821563720703125f,-0.6849999973177904877275068429298698902130126953125f,-0.56979997929632641984198926365934312343597412109375f,-0.7501999868035316154646352515555918216705322265625f,1.0749000182956460758276762135210447013378143310546875f,-0.0740000090599064463958711712621152400970458984375f,-1.4609000223487615954809371032752096652984619140625f,-0.79380001838207281394943493069149553775787353515625f,0.09729996860474354747339020832441747188568115234375f,0.2533999988794324309537842054851353168487548828125f,0.1427999930575489795359089839621447026729583740234375f,-0.090400009071827103213081500143744051456451416015625f,-0.0674000087589023966216927874484099447727203369140625f,-0.3696000268399719512757428674376569688320159912109375f,0.601499994084238931435493213939480483531951904296875f,-0.53649997938796911700620739793521352112293243408203125f,0.595899997072666831599008219200186431407928466796875f,0.5268000344157222070151647130842320621013641357421875f,-0.316800007817148954192276733010658062994480133056640625f,0.2971999483063816338557217022753320634365081787109375f,0.9023999758273364424354667789884842932224273681640625f,0.33430004343092412089077924974844790995121002197265625f,-0.3111999892547727686320513384998776018619537353515625f,-1.248399988821148554052342660725116729736328125f,0.0411000011429190070799677414470352232456207275390625f,0.84189999367520196305036961348378099501132965087890625f,-0.452099975598603764925798031981685198843479156494140625f,-0.00949999422952441108236598665826022624969482421875f,-0.73450000806897841609810484442277811467647552490234375f,-0.66940006104707805434372858144342899322509765625f,-0.7611999674245719660348186152987182140350341796875f,0.203500006288290791189865558408200740814208984375f,-0.5874999789893629031922728245262987911701202392578125f,0.87589997771978378438006984652020037174224853515625f,0.5131999966531992640739190392196178436279296875f,1.088699998754262932010306030861102044582366943359375f,-0.6626000075206157635676618156139738857746124267578125f,-0.66670000852942568059233963140286505222320556640625f,0.348799981538951531234715730533935129642486572265625f,0.4232000301063056912909132734057493507862091064453125f,-0.6187999964773653882588178021251223981380462646484375f,-0.21440002073943686955459497767151333391666412353515625f,-0.298200006860494415406037660432048141956329345703125f,0.09259996158778616148765649995766580104827880859375f,-0.5958999842390415579984619398601353168487548828125f,-0.973400006628036607025933335535228252410888671875f,-0.224899993149936250347309396602213382720947265625f,-0.606399991667270654716048738919198513031005859375f,1.06579998741000903095255125663243234157562255859375f,0.3059999781250954953293330618180334568023681640625f,-0.8197999845862387058303966114181093871593475341796875f,-0.1414000055253505916397216424229554831981658935546875f,-0.80030001451522114752634706746903248131275177001953125f,-0.42800000821054007271726504768594168126583099365234375f,-0.65159998677819952472844988733413629233837127685546875f,-0.5661999837785962608194267886574380099773406982421875f,0.088599965417385018184859291068278253078460693359375f,-0.226400003886222833671126863919198513031005859375f,0.938200005444884244099057468702085316181182861328125f,-0.02160002406835559440878569148480892181396484375f,0.11349999842792712900774176887352950870990753173828125f,0.76480001558065424660526332445442676544189453125f,-1.03630000806301847404711224953643977642059326171875f,-0.65760000755786851556194960721768438816070556640625f,-0.353300021459162227888128882113960571587085723876953125f,-0.2216000124454495789905195124447345733642578125f,-0.164699980306625359816052878159098327159881591796875f,-0.20539997539520271629953640513122081756591796875f,0.2849999962002041087316683842800557613372802734375f,-0.47740001233518103429531720394152216613292694091796875f,0.3301000184729694364449414933915250003337860107421875f,0.0050000044703484292085704510100185871124267578125f,-1.4107000153169038103584398413659073412418365478515625f,1.56960000970363555694575552479363977909088134765625f,-0.5134999869912864056686885305680334568023681640625f,0.926499980002642331555762211792171001434326171875f,-0.392599992060661318049596957280300557613372802734375f,0.371399978852271939189222393906675279140472412109375f,0.01730000511258833473249296730500645935535430908203125f,-0.56879998746216298144418033189140260219573974609375f,-0.4890999842077494719205787987448275089263916015625f,-0.5309000064417717101861171613563783466815948486328125f,-0.79590000115185954854268857161514461040496826171875f,-0.4042000449255114258306775809614919126033782958984375f,-0.12699999909102910322644675034098327159881591796875f,0.63239999471455821122845009085722267627716064453125f,-0.590099997015297361713237478397786617279052734375f,0.23569996081590627312607466592453420162200927734375f,0.3450999900117517338316019959165714681148529052734375f,-0.58429998613595923728780689998529851436614990234375f,0.8050999749615790346268795474315993487834930419921875f,0.762099986606835866354003883316181600093841552734375f,-0.2324999962374569850709349339012987911701202392578125f,0.175900048500299277520753093995153903961181640625f,-0.849999986886978309286178046022541821002960205078125f,-1.009099991732835377433730172924697399139404296875f,-0.25899999702721832051111050532199442386627197265625f,0.7422999779924748242621035387855954468250274658203125f,-0.84399996718764336289808625224395655095577239990234375f,-0.43690002778023373508631266304291784763336181640625f,0.20310000094175340024094111868180334568023681640625f,0.44989996442794844000445664278231561183929443359375f,-0.405700004215538090335257948026992380619049072265625f,-0.04079999132156419960892890230752527713775634765625f,0.4829999779984357477502499023103155195713043212890625f,0.018799996626376991315510167623870074748992919921875f,0.560799990203976417291187317459844052791595458984375f,0.3724000146448613879357480982434935867786407470703125f,0.626599996165931205638344181352294981479644775390625f,0.2327999884009361863945741788484156131744384765625f,0.0791000010088087623927322056260891258716583251953125f,-0.02040003172159199351654024212621152400970458984375f,-0.111099969664216036147053046079236082732677459716796875f,0.111999999016523243966503287083469331264495849609375f,-0.791500030964612744099895280669443309307098388671875f,0.0571999922275541194949255441315472126007080078125f,0.11650000891089440369086105420137755572795867919921875f,-0.363000002324581405588332927436567842960357666015625f,0.255699979107081531992662348784506320953369140625f,-0.1666000276446339967151288874447345733642578125f,-0.3194999680966141841764738273923285305500030517578125f,-0.1512000030755995538811475853435695171356201171875f,-1.427999975129961907072129179141484200954437255859375f,-0.56579997701644924035235817427746951580047607421875f,0.171299973529577209685470506883575581014156341552734375f,-0.0651999954462052500048230285756289958953857421875f,0.61830000458359724024859360724803991615772247314453125f,0.04339999522864823600087902377708815038204193115234375f,-0.263699991564452951298136440527741797268390655517578125f,-0.18079999072551711236656046821735799312591552734375f,-0.5247999672636389778546117668156512081623077392578125f,-1.244099969306588615580722034792415797710418701171875f,-0.24159999970495693588645735871978104114532470703125f,-0.1571999786674982413359202837455086410045623779296875f,-0.60149997750669736973350154585205018520355224609375f,-1.17029999801218398403079845593310892581939697265625f,-0.104699997480213635725476706284098327159881591796875f,-0.3159000187724831931745939073152840137481689453125f,0.6551999882191419377619467923068441450595855712890625f,0.996200031834840071809367145760916173458099365234375f,0.19659999977946274185569563996978104114532470703125f,0.1808000170260670724786677965312264859676361083984375f,-0.04069999229460952161474551758146844804286956787109375f,-0.89589998226463851693779361085034906864166259765625f,-0.2888999975904822381522762952954508364200592041015625f,0.2651999811410903351571732855518348515033721923828125f,0.4535999837830664294102689382270909845829010009765625f,1.5788999824285507855847754399292171001434326171875f,-0.19330000666975966083782623172737658023834228515625f,0.07429995961189295172744095907546579837799072265625f,0.288999988272786278997728004469536244869232177734375f,0.418099979037046320939907673164270818233489990234375f,0.13739998987168089428223538561724126338958740234375f,0.7828000004708766113026285893283784389495849609375f,-0.4602000020027159354896184595418162643909454345703125f,-0.237500024177134338998484963667578995227813720703125f,-0.25729998771548279190568564445129595696926116943359375f,-0.23670003542601969481751211787923239171504974365234375f,0.19749999035149823356505294214002788066864013671875f,1.141399955904483842772378920926712453365325927734375f,-0.2821000182345512197201742310426197946071624755859375f,0.40340001676082604120665564551018178462982177734375f,1.0136000085189937891527733881957828998565673828125f,0.3859999843835826283111600787378847599029541015625f,0.4216000170648097622461136779747903347015380859375f,0.75200001972913721459690350457094609737396240234375f,-0.6802000234603882145023590055643580853939056396484375f,-0.3890000108480453633319484652020037174224853515625f,-0.29349998069554550372828316540108062326908111572265625f,0.1572000345468520521308164461515843868255615234375f,1.786099964523315097864042400033213198184967041015625f,0.90020000528097199321564403362572193145751953125f,-1.2678999784499407343929533453774638473987579345703125f,-0.698400000965595069146729656495153903961181640625f,0.67160000600069780229972593588172458112239837646484375f,-0.0294000145554544900505788973532617092132568359375f,-1.052300033238530385659714738721959292888641357421875f,-0.30680000491142234153585377498529851436614990234375f,1.2941000319659712314290800350136123597621917724609375f,0.47699999495595657439395154142403043806552886962890625f,-0.73710000625774296789671780061325989663600921630859375f,-0.325399994580447493586206064719590358436107635498046875f,-0.62409998892024187711058402783237397670745849609375f,-0.4817999825596803731286854599602520465850830078125f,-0.376199984151125210729560421896167099475860595703125f,-0.6942000101685525947914356947876513004302978515625f,-0.1808000265628100233783470685011707246303558349609375f,0.319000015914440382402972318232059478759765625f,0.7966000035047529070908467474509961903095245361328125f,-0.864000006526708386189739030669443309307098388671875f,0.634499970182776706906224717386066913604736328125f,0.64830002201795533522954428917728364467620849609375f,-0.4720000084787603356062390957958996295928955078125f,-1.928400037771463626512513656052760779857635498046875f,0.1110000020414592114548213430680334568023681640625f,-0.3161000427544120583434050786308944225311279296875f,0.3004999874234199896960717524052597582340240478515625f,0.1713000361144541017210940481163561344146728515625f,0.2349000068590043621696850095759145915508270263671875f,0.009600008660555658934754319489002227783203125f,0.867099983850121436290692145121283829212188720703125f,0.6181000215053558122235699556767940521240234375f,-0.413499994404614046317192332935519516468048095703125f,0.64799999710917433048962266184389591217041015625f,0.3714999914020300497696780439582653343677520751953125f,0.17730002362728125575586091144941747188568115234375f,-0.83979998697042479438579221096006222069263458251953125f,-0.7532999875776471565558267684536986052989959716796875f,0.0312000058136880920045541643048636615276336669921875f,-0.3335000401884309351174806579365395009517669677734375f,-0.196800001279264613618380508341942913830280303955078125f,0.37209993856921830257533656549639999866485595703125f,0.28650000362098193562587766791693866252899169921875f,1.735599997812510508765626582317054271697998046875f,-0.5799000147938724314400360526633448898792266845703125f,-0.98500003293156712658174001262523233890533447265625f,0.4192999906808136945102205572766251862049102783203125f,0.1396000396549705868665114394389092922210693359375f,0.95089998900741334875164056938956491649150848388671875f,0.2076000085264444461330413105315528810024261474609375f,-0.38150000473856937599492766821640543639659881591796875f,0.1624000329732897540679914527572691440582275390625f,1.93390001364648345116847849567420780658721923828125f,0.80860000594854408717537808115594089031219482421875f,-0.2942999735444782327675738997641019523143768310546875f,-0.0028000140309334398125429288484156131744384765625f,-0.6790999905779957490636888906010426580905914306640625f,-0.39160000946521744680239862645976245403289794921875f,-1.8163000028103588601169349203701131045818328857421875f,-1.087200009512901743846668978221714496612548828125f,-0.039000049963593408364204151439480483531951904296875f,-0.07180000640153849644775618799030780792236328125f,0.47850001180171963799381273929611779749393463134765625f,-0.05079999847412108859856516573927365243434906005859375f,-0.23000000141561027788839055574499070644378662109375f,-0.090799994301795639017882422194816172122955322265625f,1.170499993346631573132299308781512081623077392578125f,0.74340000573396736172071541659533977508544921875f,-1.374000022768974371700778647209517657756805419921875f,-0.42880000228881787194268326857127249240875244140625f,1.141400030373036944553177818306721746921539306640625f,0.18400001122057452196401072797016240656375885009765625f,0.600800013263523613460392880369909107685089111328125f,0.305799995791911971565468775224871933460235595703125f,-0.4035000080019235380035524940467439591884613037109375f,0.03299999569356415474175037161330692470073699951171875f,-0.748999973334372182165452613844536244869232177734375f,-0.09370001756697809991436542986775748431682586669921875f,-0.0075999981328844778971642881515435874462127685546875f,-0.853000011004507285861109266988933086395263671875f,0.573499994777142863000563011155463755130767822265625f,0.82440002096295383893220787285827100276947021484375f,0.66370001626312691289655276705161668360233306884765625f,0.13970004587173523447063416824676096439361572265625f,-0.4344999820664525724822624397347681224346160888671875f,-0.359500036120414367957209833548404276371002197265625f,-0.5717999779403211135786477825604379177093505859375f,-0.0460000214874742852089184452779591083526611328125f,-0.82570000257641051888413130654953420162200927734375f,-0.309600012981892547259121784009039402008056640625f,-0.5855999804154041754600257263518869876861572265625f,-0.41479997765719833768827129460987634956836700439453125f,-0.05810000049471854277616245099125080741941928863525390625f,-0.16269999496191733800998235892620868980884552001953125f,-0.70979997273981598748804344722884707152843475341796875f,-1.29609999834895095460751690552569925785064697265625f,0.565800013114511557432706467807292938232421875f,-0.3311999361693853671795295667834579944610595703125f,-0.71849998516589430597178989046369679272174835205078125f,-0.1176999968245622252283055786392651498317718505859375f,0.7165999991461633467082492643385194242000579833984375f,0.906900032027066149709071396500803530216217041015625f,0.231899998164177001402919131578528322279453277587890625f,-0.205000003688037668592158979663508944213390350341796875f,0.57109997797161360377771188723272643983364105224609375f,1.30689997532814761171948703122325241565704345703125f,0.7281999937474730177910942074959166347980499267578125f,0.369199986624717979566412395797669887542724609375f,-0.1858000006347897414826775275287218391895294189453125f,0.2562000187218191893379071188974194228649139404296875f,0.295299990896880582003092285958700813353061676025390625f,0.44800001543760270550365021335892379283905029296875f,-0.0664000149503352066204797665704973042011260986328125f,-0.06979998458623892787500153644941747188568115234375f,-0.07329999102354012308069286518730223178863525390625f,-0.1629999874234204781942025874741375446319580078125f,0.66149996666610222373350325142382644116878509521484375f,0.0201999973833559831604134160443209111690521240234375f,1.03430001854598518917782712378539144992828369140625f,0.56320000224113453413110619294457137584686279296875f,0.6520000162646175656533387154922820627689361572265625f,0.8904000153303144315941608510911464691162109375f,-1.016100014814734553425523699843324720859527587890625f,-0.61720001577138905446417993516661226749420166015625f,0.38900002958625545801396583556197583675384521484375f,-0.0340000063776972893947458942420780658721923828125f,-0.4165000255629418557390408750507049262523651123046875f,0.29559999296963201942389787291176617145538330078125f,-0.72839998360574231828223901175078935921192169189453125f,-0.50779998959302918848379704286344349384307861328125f,-0.09940005493760140087289300936390645802021026611328125f,0.1350000107288362727331332280300557613372802734375f,0.776600044743716910744524284382350742816925048828125f,-0.18100000345706934723466474679298698902130126953125f,-0.055200005765259074763662283658050000667572021484375f,-0.39389997125268028099753792048431932926177978515625f,-0.7128999625429515152319481785525567829608917236328125f,-1.9869999914914355798600809066556394100189208984375f,-0.1212999986380338679481383223901502788066864013671875f,-1.045699990394711509367198232212103903293609619140625f,0.5707999696403744938066893155337311327457427978515625f,0.34130000005364369286553483107127249240875244140625f,0.48029997424483317214338740086532197892665863037109375f,0.74360004439353843963544932194054126739501953125f,-0.343199997361004260909567165072076022624969482421875f,-0.01510001135766481095146218649460934102535247802734375f,-0.3368000080034134757767105838865973055362701416015625f,-0.637400018295645853783071288489736616611480712890625f,-0.0848999947890638484437886290834285318851470947265625f,-1.52950000810623176761282593361102044582366943359375f,0.6067000004455447292883718546363525092601776123046875f,0.408099982613325362734713053214363753795623779296875f,-0.4254000025898216907904725303524173796176910400390625f,0.4251999631106855304096825420856475830078125f,-0.2110000027865168714669152905116789042949676513671875f,-1.04320001747757284960016477270983159542083740234375f,0.0182000001177191617074413443333469331264495849609375f,0.01829999739378695977620736812241375446319580078125f,-0.18519999533444642114687894718372263014316558837890625f,-0.284499997228383705305532203055918216705322265625f,0.247799976368248575653296938980929553508758544921875f,1.0631000092864031358885767986066639423370361328125f,-0.64849996955692790212566478658118285238742828369140625f,-0.581400022661685778047058192896656692028045654296875f,-0.23460002389699241565068632553447969257831573486328125f,-0.2908000112891198085662836092524230480194091796875f,0.162600002007186328345511583393090404570102691650390625f,-0.1987999974831937688435345990001223981380462646484375f,0.076500007010996295964133651068550534546375274658203125f,-0.068999992370605500724423109204508364200592041015625f,-0.3417000303223731716428801519214175641536712646484375f,-0.132999991297721553706878694356419146060943603515625f,0.823499994516372790798186542815528810024261474609375f,0.373399997985362830377198406495153903961181640625f,-0.60919997619390453547083552621188573539257049560546875f,-0.7108000012308364201629728995612822473049163818359375f,0.32219996935427219764136452795355580747127532958984375f,0.1195999904811382297253885553800500929355621337890625f,0.0219000107929108178694832531618885695934295654296875f,-0.11719997524023018087291347910650074481964111328125f,0.2597999990031123473954721703194081783294677734375f,0.123399993515014738676427441532723605632781982421875f,0.38380001018643394861129536366206593811511993408203125f,0.5544000212609769828731032248470000922679901123046875f,1.2855000310838224919507410959340631961822509765625f,0.50780000091791155369946864084340631961822509765625f,-1.9268000151932238583896150885266251862049102783203125f,-0.76640000388622286919826365192420780658721923828125f,0.620000012852251369821487969602458178997039794921875f,0.000600016438961237241755952709354460239410400390625f,0.7408000076010823331529309143661521375179290771484375f,0.269199983048439062116585773765109479427337646484375f,-0.5337000019952655716082290382473729550838470458984375f,0.331800042313337595345501540577970445156097412109375f,-0.7668000110208985642401557925040833652019500732421875f,-0.488200034669787286123465719356318004429340362548828125f,-0.06500000117346647898752820537993102334439754486083984375f,0.18230000673308988101695149453007616102695465087890625f,-0.1764999995976685998044786174432374536991119384765625f,-0.3356000152468681108075543306767940521240234375f,0.311099986428022334816745342322974465787410736083984375f,0.465800053310393646910370080149732530117034912109375f,0.6375999792829156742612894959165714681148529052734375f,1.27200002632290054549457636312581598758697509765625f,-0.0279000011742114306656503686099313199520111083984375f,0.53530002963989919972931375014013610780239105224609375f,-0.74389997203499103051171914557926356792449951171875f,-0.923900042554735012600986010511405766010284423828125f,0.8622999919995664175331739897956140339374542236328125f,1.3290000397562982126231645452207885682582855224609375f,-0.18129999550879005720815939639578573405742645263671875f,-1.00989999117553264085245245951227843761444091796875f,0.8533000030562278848123014540760777890682220458984375f,1.0198000329405072506006035837344825267791748046875f,0.91590000603198984396868809199077077209949493408203125f,0.726700028049946045172191588790155947208404541015625f,-0.1602000015556812595463043180643580853939056396484375f,0.9520999827325347109052700034226290881633758544921875f,0.8213999918535350719395182750304229557514190673828125f,1.39500003010034578920794956502504646778106689453125f,-0.135699993337690838490772193836164660751819610595703125f,-1.191399999639391804606702862656675279140472412109375f,-0.5849000175133347578793063803459517657756805419921875f,-0.183600017869472775799977171118371188640594482421875f,-0.0222999991148709053589982431731186807155609130859375f,0.168600014591216762482872582040727138519287109375f,1.058299988825619042387415902339853346347808837890625f,0.56120000754594823177967555238865315914154052734375f,-0.7339999821260569934366913003032095730304718017578125f,-0.394599995398522196410340256989002227783203125f,0.40009997987896195326129600289277732372283935546875f,-0.3852000329971314585009167785756289958953857421875f,-1.06859999708235253734756042831577360630035400390625f,-0.220999996602535109246900901780463755130767822265625f,0.16569998050406586909133466178900562226772308349609375f,0.120399993239343083839543169233365915715694427490234375f,1.2729000049181280473931110464036464691162109375f,0.79980001885891027768593630753457546234130859375f,-0.810099963860213545530086776125244796276092529296875f,-0.159199979472160801918789729825221002101898193359375f,0.9992999752208586716761828938615508377552032470703125f,0.644800017070770170590776615426875650882720947265625f,0.184400019212067167728719141450710594654083251953125f,-0.033799999010562853385408743633888661861419677734375f,-1.1449000226542356273284894996322691440582275390625f,-0.3007999943017960475799554842524230480194091796875f,0.5234000138923524669820608323789201676845550537109375f,-0.33400000004470331571582164542633108794689178466796875f,0.25520000442415502472925936672254465520381927490234375f,0.38040000966787346214914578013122081756591796875f,-0.7474999958649279907518803156563080847263336181640625f,-0.0368999743223188403362655662931501865386962890625f,-0.8770000027418134447287911825696937739849090576171875f,-1.474700027543307090382995738764293491840362548828125f,0.31419999083429583919269134639762341976165771484375f,0.132800007921457297044298684340901672840118408203125f,0.401800001111626503558227341272868216037750244140625f,0.1893000187695026426126787555404007434844970703125f,0.425399966789782535414587982813827693462371826171875f,1.9937999639928332129557020380161702632904052734375f,-0.5832999963507055074529716875986196100711822509765625f,-0.9580000120103362437617988689453341066837310791015625f,-0.76589997354745886770643892305088229477405548095703125f,-1.102899984168260960615270960261113941669464111328125f,-0.683599988756328880157298044650815427303314208984375f,-1.4255000328905875761620336561463773250579833984375f,0.802100009852647399810621209326200187206268310546875f,0.09010004659891190925691262236796319484710693359375f,0.403599984416365853423513954112422652542591094970703125f,0.3501999908268456085380648801219649612903594970703125f,0.57409996993988787750140545540489256381988525390625f,0.5051999690711490575978359629516489803791046142578125f,0.35399999948590987752794490006635896861553192138671875f,0.02020002942085330488453109865076839923858642578125f,-0.9531000056728717861886934770154766738414764404296875f,-0.1737000788107516680014441590174101293087005615234375f,-0.12779999506920580731872405522153712809085845947265625f,-0.3065999912485484202306906809099018573760986328125f,0.588500005729496233897179990890435874462127685546875f,0.86790000668764211155803423025645315647125244140625f,0.238000033244490705186535706161521375179290771484375f,-0.00879998887777322380543409963138401508331298828125f,0.249499980278313326920169856748543679714202880859375f,0.469200002419948081211487078689970076084136962890625f,0.75069999175444246386490476652397774159908294677734375f,0.472200008805095994279810156513121910393238067626953125f,0.39099999818578368149246671237051486968994140625f,0.204599992418289300388778428896330296993255615234375f,0.7199999970942736471357648042612709105014801025390625f,0.321599972808360945464301039464771747589111328125f,-1.0182999841317534883700091086211614310741424560546875f,-0.183199993002414629472696105949580669403076171875f,-0.47330000849515219130125842639245092868804931640625f,-0.4792000017493960317693790784687735140323638916015625f,-0.372799987655878350967242340630036778748035430908203125f,-0.5982000052213667373735006549395620822906494140625f,-0.2807000020548702234890470208483748137950897216796875f,-0.1690000003427265529154510659282095730304718017578125f,0.05020000885725028894057686557061970233917236328125f,-0.260200015711784526928340710583142936229705810546875f,-0.09039999998211822873628307206672616302967071533203125f,0.37040002009868633336964194313623011112213134765625f,-0.2335999913454056553518967120908200740814208984375f,-0.633199999260902313125143336947076022624969482421875f,-0.96820001002699107583993054504389874637126922607421875f,-0.798400011843443113690454993047751486301422119140625f,1.075199996042251360250929792528040707111358642578125f,0.674200022387504649401535061770118772983551025390625f,1.1841000234723093598177001695148646831512451171875f,1.5950000052154056362496703513897955417633056640625f,0.42880000132024276293662978787324391305446624755859375f,-0.0417999424755575166301468925666995346546173095703125f,0.4231999760895970297269741422496736049652099609375f,-0.0688999887801713395418801155756227672100067138671875f,0.7708999835498631103547495513339526951313018798828125f,0.641000030670314746572557851322926580905914306640625f,-0.199299986433983000733149992811377160251140594482421875f,-0.271299997669458559101940409163944423198699951171875f,0.6035999879926443600908214648370631039142608642578125f,0.211400014019012427723964719916693866252899169921875f,0.12610000263303511314205707094515673816204071044921875f,0.80289998087137981475081005555693991482257843017578125f,-0.0362000024423003718965219377423636615276336669921875f,1.13329997474402244250768490019254386425018310546875f,-0.2373000078320501859963087554206140339374542236328125f,0.26479997605532379356674255177495069801807403564453125f,0.03229999587386844783765127431252039968967437744140625f,0.6928000056490299929379261811845935881137847900390625f,-0.33149999885261050014406691843760199844837188720703125f,-0.973699991154671540272147467476315796375274658203125f,0.5897000147655602209084690912277437746524810791015625f,1.3781000225484383303609092763508670032024383544921875f,-0.3686999920114875095578099717386066913604736328125f,0.6322999818295234053522335671004839241504669189453125f,-0.08910000160485509468344389460980892181396484375f,0.2180000181943186898791964267729781568050384521484375f,0.425799984429776845473725188639946281909942626953125f,-0.1299000022768979079046403057873249053955078125f,0.242199993345141084688521004864014685153961181640625f,0.221400018191337810691265985951758921146392822265625f,1.2933999963089821694239844873663969337940216064453125f,0.67179999358654018948300290503539144992828369140625f,-0.0963999820932748274771029173280112445354461669921875f,0.4806000243365768920256186902406625449657440185546875f,0.43950001119822290629457484101294539868831634521484375f,0.118799993348121635250436156638897955417633056640625f,0.5518000182107092310701546011841855943202972412109375f,0.30699998713284715190496854120283387601375579833984375f,-1.524899998179078064453051410964690148830413818359375f,-0.327599981182813859703628622810356318950653076171875f,-0.4046999968841669126362603492452763020992279052734375f,-0.67839999053478283741469567758031189441680908203125f,0.995399993537366523099763071513734757900238037109375f,0.18519999216794946761410756153054535388946533203125f,-1.328400012402236374242647798382677137851715087890625f,-0.784200008678435978026755037717521190643310546875f,0.0554000094071032123110853717662394046783447265625f,0.05360002703368704768394081838778220117092132568359375f,1.04660000864565372324932468472979962825775146484375f,0.218399990534782428852622615522705018520355224609375f,-0.437099989400804300743175190291367471218109130859375f,-0.180200003045797085832191442023031413555145263671875f,-1.041900022751093057848947864840738475322723388671875f,-0.26220001517534274881882083718664944171905517578125f,0.8251000087499618107500509722740389406681060791015625f,0.14639999345541010455917785293422639369964599609375f,-1.0329999751672147567660431377589702606201171875f,-0.7398000418066976724418282174156047403812408447265625f,-0.178299999964237176897086101234890520572662353515625f,-0.53420001672506334244872050476260483264923095703125f,0.79609997480511685008508493410772643983364105224609375f,-0.31939999063909052523513310006819665431976318359375f,0.2901000068873165194105467890040017664432525634765625f,0.2199000054806468096302296544308774173259735107421875f,0.8573999934107063847221752439509145915508270263671875f,1.188100045403093218965295818634331226348876953125f,0.084800004721432953314064207006595097482204437255859375f,-0.7758000031121075057427560750511474907398223876953125f,-0.01240000220239156902835020446218550205230712890625f,1.003799987015127737066677582333795726299285888671875f,0.0248999964281917396391463626059703528881072998046875f,1.05870000635385519416331590036861598491668701171875f,-0.61279998527169221578247970683150924742221832275390625f,-0.73050005194544809228318626992404460906982421875f,-0.7077999745056036129398080447572283446788787841796875f,-0.395200003977119518339122805627994239330291748046875f,0.8065000089481468847907308372668921947479248046875f,1.60130004870593634080933043151162564754486083984375f,0.182599995934963288402030912038753740489482879638671875f,0.1808999679595235665630070798215456306934356689453125f,0.0785999983862042395088565172045491635799407958984375f,-0.164899973294139190471696565509773790836334228515625f,-0.8692000036492941728027972203562967479228973388671875f,-0.65140003309249916441103778197430074214935302734375f,-0.26829998793154963276919033887679688632488250732421875f,-1.06489999527335132967209574417211115360260009765625f,0.19430003802925333022955101114348508417606353759765625f,-0.3565999895721672086068565477035008370876312255859375f,-0.586699997055530442224835496745072305202484130859375f,-0.60960000821352000599517850787378847599029541015625f,0.606000040970742670509707750170491635799407958984375f,0.0174000183701519262058354797773063182830810546875f,-0.6090000301450493314092682339833118021488189697265625f,0.3044000133633613103256720933131873607635498046875f,0.531100031317770504557529420708306133747100830078125f,-0.1442000021219254080051541677676141262054443359375f,-0.72720001580864190859898599228472448885440826416015625f,-0.35800000590085989671251809340901672840118408203125f,0.05490000291019681011306374784908257424831390380859375f,-0.1282000023901461727149353464483283460140228271484375f,0.60579999288618535668859976794919930398464202880859375f,0.46599999511241918526138761080801486968994140625f,-0.2387000066518780716506853423197753727436065673828125f,0.301999999463558044254796186578460037708282470703125f,0.08610000483095647805242833783267997205257415771484375f,-0.0591999930322171241670048402738757431507110595703125f,0.40929999720007181895908843216602690517902374267578125f,0.15520000148117552019044751432375051081180572509765625f,-0.123300008755922119174641693462035618722438812255859375f,0.256399997627735132255111238919198513031005859375f,-0.54690000378936520775141616468317806720733642578125f,-0.49719999997615804687711715814657509326934814453125f,1.182399993559717810143183669424615800380706787109375f,0.75160001751184513096859518554992973804473876953125f,-0.3845999763622878209190503184800036251544952392578125f,1.79790000553280204798056729487143456935882568359375f,-0.0559999989867210334892888567992486059665679931640625f,0.848999972440302119736088570789434015750885009765625f,-0.1276999957069755442518044219468720257282257080078125f,0.94599996516108575406178715638816356658935546875f,0.6483000156104561373382466626935638487339019775390625f,-0.08069995858073131245191689231432974338531494140625f,0.51419996658265620936845152755267918109893798828125f,1.093899989283084295266235130839049816131591796875f,-0.3314999890923500114325861432007513940334320068359375f,-0.288300022614002227072660389239899814128875732421875f,-0.156700004766881395568134394125081598758697509765625f,0.32879999491274336520518772886134684085845947265625f,-0.28339998908191932347477859366335906088352203369140625f,-0.6365999909877768647703533133608289062976837158203125f,-0.661499996580183324823565271799452602863311767578125f,-2.0807000171422966872114557190798223018646240234375f,0.2754999983385204576080695915152318775653839111328125f,0.2611000192850829737523099538520909845829010009765625f,0.2055999962478876341709366215582122094929218292236328125f,0.4285999989822506517356259791995398700237274169921875f,0.34469999919384708420011520502157509326934814453125f,-0.24609994954764868868579696936649270355701446533203125f,-0.471699983532726907498044965905137360095977783203125f,-0.55990000913143145311323678470216691493988037109375f,-0.27379999006986632803517522916081361472606658935546875f,-0.344999993741511357114859492867253720760345458984375f,0.401100000397860867451527155935764312744140625f,-0.14789999670386322350168484263122081756591796875f,-0.5046000045254828147989201170275919139385223388671875f,-0.676800003123283477890481663052923977375030517578125f,0.01000000510364795758277978166006505489349365234375f,-0.1147999887585644529508499545045197010040283203125f,-0.079200031812489246618724791915155947208404541015625f,-0.227999984443187031502020545303821563720703125f,1.42730000868886719445072230882942676544189453125f,0.42019998024702065464452971355058252811431884765625f,0.51820003625303545646829661563970148563385009765625f,0.8266000184059141275838555884547531604766845703125f,-1.254600036935508367008651475771330296993255615234375f,-0.473599991941451747834435082040727138519287109375f,1.1117000435471535713105595277738757431507110595703125f,0.07539999625682813899629763909615576267242431640625f,0.35860000129193092543999910049024038016796112060546875f,0.334199983644484888145598233677446842193603515625f,-0.6339000149503351000390694025554694235324859619140625f,0.149199998843669856540827822755090892314910888671875f,-0.350400011493265661766116636499646119773387908935546875f,0.062400001680850802898703477694652974605560302734375f,0.4257000093191862788444268517196178436279296875f,0.30820003263950379590596639900468289852142333984375f,-0.133900009809434283880591465276665985584259033203125f,0.0198000164747238471818491234444081783294677734375f,0.97480001748055189469965853277244605123996734619140625f,-0.04799999070167526582508799037896096706390380859375f,-0.3603999777048823016656342588248662650585174560546875f,-0.28000001579523114969560992904007434844970703125f,-0.43590001307278891662377873217337764799594879150390625f,-1.851200008440017530375598653336055576801300048828125f,0.02090002391338430243195034563541412353515625f,-0.274700001093745260760670134914107620716094970703125f,-0.712499961219728472627821247442625463008880615234375f,-0.4757000011980536680056275145034305751323699951171875f,0.0204999656677244956881622783839702606201171875f,1.2153999938726425256163565791212022304534912109375f,0.58710000068843282150510276551358401775360107421875f,0.31619998877048516305876546539366245269775390625f,0.79439996418953029433396295644342899322509765625f,0.23559997806847121637474629096686840057373046875f,0.549000018373131837279288447462022304534912109375f,1.134400014927983146861834029550664126873016357421875f,0.072899975991248311402159743010997772216796875f,0.3093000236868856944738581660203635692596435546875f,1.2482999567508699101381353102624416351318359375f,1.0517000009298325124973416677676141262054443359375f,-0.90529998571872738466481678187847137451171875f,-0.95689998609423643216587151982821524143218994140625f,-0.41829995824098631373999523930251598358154296875f,0.23620001037716864544790951185859739780426025390625f,0.5447999778807162130078722839243710041046142578125f,0.037000003915280021971057067275978624820709228515625f,-0.118499994646757755845101200975477695465087890625f,-0.66330002004355159694881649556919001042842864990234375f,1.026900005856901998413377441465854644775390625f,0.75690000030621884530290799375507049262523651123046875f,1.0094999759197236244290252216160297393798828125f,0.23119999830722814948558152536861598491668701171875f,1.1038999826520683456010374356992542743682861328125f,-0.33379999379515645774318954863701947033405303955078125f,-0.1399999783188103918973865802399814128875732421875f,0.09519999768137932250056110206060111522674560546875f,-1.5748000093221659057007855153642594814300537109375f,0.3105999924480915552749138441868126392364501953125f,0.3746999791890377462522110363352112472057342529296875f,-0.48460002102851884586698361090384423732757568359375f,0.636599976086615892967301988392136991024017333984375f,-0.225600001388788218292802412179298698902130126953125f,0.8760000132173295650517275134916417300701141357421875f,0.1413999863028525538766189129091799259185791015625f,0.8429999807178969462029272108338773250579833984375f,0.367400022095441869396381662227213382720947265625f,-1.09109997346401232931611957610584795475006103515625f,0.71100000286102282842648492078296840190887451171875f,0.758500007331371239871486977790482342243194580078125f,0.015999994516372684216776178800500929355621337890625f,-0.161400008505582892581742271431721746921539306640625f,0.34379998410940171282845767564140260219573974609375f,1.868799980086087497710423122043721377849578857421875f,-0.195799988341331498986619408242404460906982421875f,-0.666899992687999709772839196375571191310882568359375f,0.4735999878808854202105749209295026957988739013671875f,0.0236999950848517659807157542672939598560333251953125f,-0.612999992787838010599443805404007434844970703125f,-0.1390000075697894743598226341418921947479248046875f,-0.3477999869108199249012614018283784389495849609375f,-0.808599995517730096850073095993138849735260009765625f,-0.78869999137818813661482408861047588288784027099609375f,0.4392000115841625529355951584875583648681640625f,-1.01900003118813042846113603445701301097869873046875f,0.8373000057458881428829045034945011138916015625f,1.264400002336502293331932378350757062435150146484375f,-0.5669000051677226537094611558131873607635498046875f,0.6155000074654817243668958326452411711215972900390625f,-1.073200005221366382102132774889469146728515625f,-0.129399995332956319060713212820701301097869873046875f,-0.1946999986350537170665120356716215610504150390625f,0.089899981638789316917836913489736616611480712890625f,0.894599980497361002562684006989002227783203125f,-0.36759998922944081556352102779783308506011962890625f,1.360699984359740710715414024889469146728515625f,1.860300006729364241664370638318359851837158203125f,0.0481999539613724437003838829696178436279296875f,0.804599992567300859036549809388816356658935546875f,-0.534800000381469686772106797434389591217041015625f,1.36959999286532418949491329840384423732757568359375f,-0.739800013792514477017903118394315242767333984375f,-1.096300014507770814731202335678972303867340087890625f,0.35820000879764535284266457892954349517822265625f,-0.030900014004111131526997269247658550739288330078125f,0.575299965117872513786778654321096837520599365234375f,0.0262000023305415385976857578498311340808868408203125f,-0.26779998273849514589528553187847137451171875f,-1.0538999950945378003552832524292171001434326171875f,0.04480002481937361835662159137427806854248046875f,-0.157099989587068389340629437356255948543548583984375f,0.5010000237971541370285422090091742575168609619140625f,0.39200002196431160683687267010100185871124267578125f,-0.152499939054250521763833603472448885440826416015625f,0.108400020784139794471911955042742192745208740234375f,-1.017100043231248918118581059388816356658935546875f,-0.23620000918507599863005452789366245269775390625f,1.64800000604987051389116459176875650882720947265625f,0.12240001285672175157515084720216691493988037109375f,0.64810003119111048164313615416176617145538330078125f,-0.14820000119805332161604383145458996295928955078125f,-1.78809998201727893274437519721686840057373046875f,0.78040001190304764122629421763122081756591796875f,1.661200000095367723673689397401176393032073974609375f,-0.051599992179870657338369710487313568592071533203125f,-0.71210000165700915175648333388380706310272216796875f,0.80200001466274262185152110760100185871124267578125f,1.14230000410676080235816698404960334300994873046875f,0.81339998144507408284198390902020037174224853515625f,0.6948999996691942460103064149734564125537872314453125f,-0.1136000004351138681357724635745398700237274169921875f,-0.01600002565979929958217553576105274260044097900390625f,-0.1278000003591179822848289404646493494510650634765625f,1.1084999735616147820138621682417578995227813720703125f,-0.4067999976098537473490068805404007434844970703125f,-0.356599973553419768990124794072471559047698974609375f,0.044800011855363752744096927926875650882720947265625f,-1.523099967227876216924187247059307992458343505859375f,0.1525999834552408118693023197920410893857479095458984375f,0.4757999860502779565507580628036521375179290771484375f,-0.6014999791830779596324418889707885682582855224609375f,1.6176999775648113200077204965054988861083984375f,0.664500010788440675923993694595992565155029296875f,1.110700006711482501486898399889469146728515625f,1.409300042462348745431199859012849628925323486328125f,-1.2043000287532805714363348670303821563720703125f,-0.3296999852240087847121685626916587352752685546875f,0.926099966311454636525013484060764312744140625f,-1.0773000027656554067334582214243710041046142578125f,0.78510001113414773499243892729282379150390625f,1.65839999157786355254984300700016319751739501953125f,-0.102900020694733029813505709171295166015625f,0.16590001974105827997618689551018178462982177734375f,0.48019998918771733542598667554557323455810546875f,1.8862999838113783113158206106163561344146728515625f,-0.106600032114982923303614370524883270263671875f,0.6459999572634700371054350398480892181396484375f,-0.0761000175714485749267623759806156158447265625f,0.5392999864339829141357540720491670072078704833984375f,0.04380000936687000745450859540142118930816650390625f,0.69620000132471326903527142349048517644405364990234375f,-0.242399999333918092503381558344699442386627197265625f,-1.3894999729394912346691626225947402417659759521484375f,0.3323000111103056042338721454143524169921875f,-0.637599976563453907374423579312860965728759765625f,1.05329997910261141669252538122236728668212890625f,0.95529996942058226228056128093157894909381866455078125f,-0.95469998621121021642466075718402862548828125f,0.99140000517144788449996894996729679405689239501953125f,0.20940000233650213345981683232821524143218994140625f,-0.72679998017549518607438585604541003704071044921875f,-1.18240002865195226178229859215207397937774658203125f,0.44020000125765790244258823804557323455810546875f,0.82930001087188731645483130705542862415313720703125f,0.2632000079035758943746259319595992565155029296875f,0.8088000013947489463816964416764676570892333984375f,-0.40400000488758092132002275320701301097869873046875f,-0.878299992662668227438871326739899814128875732421875f,0.372600008302927054870679057785309851169586181640625f,0.14009999921321902860427144332788884639739990234375f,-0.16080000413656225788372466922737658023834228515625f,0.416399997478723360444519130396656692028045654296875f,0.375600009137392110147857238189317286014556884765625f,0.381600025036931356225977651774883270263671875f,0.2210000182092191067795283743180334568023681640625f,-0.047599997834861351719837330165319144725799560546875f,-0.7312000046774744799193967992323450744152069091796875f,-0.2330000366531310707074453603127039968967437744140625f,-0.23940000919103621157546513131819665431976318359375f,-0.80879998574852951520597343915142118930816650390625f,0.62000001117587100196715255151502788066864013671875f,0.10250001609325476437106772209517657756805419921875f,0.32500000298023223876953125f,0.78169999526739086803672762471251189708709716796875f,0.15859999685883519049411916057579219341278076171875f,-0.2109999766349790473185521477716974914073944091796875f,-0.6977999919772146508734067538171075284481048583984375f,0.3310000132918364812439904198981821537017822265625f,-0.71749996826052697684872327954508364200592041015625f,0.10220001487731877887199516408145427703857421875f,0.58310000183582300081752691767178475856781005859375f,-0.58850002729892736397232511080801486968994140625f,0.468699996370077087703975848853588104248046875f,0.06699998843669874304396216757595539093017578125f,-0.9371999984860419630194883211515843868255615234375f,0.1746000120878221650855266489088535308837890625f,0.52299999576807021384183826739899814128875732421875f,0.396099987471103531788685359060764312744140625f,0.510700005966425063519409377477131783962249755859375f,-0.5169999836683274452298064716160297393798828125f,-0.28280000121593484863069534185342490673065185546875f,0.4141999945223329149257551762275397777557373046875f,0.72650003021955489312944109769887290894985198974609375f,0.7868999951839441564516164362430572509765625f,0.331900002560019391495416130055673420429229736328125f,-0.883299970124661992798564824624918401241302490234375f,-1.2685000003278255054084411312942393124103546142578125f,-0.68339998159408565925332368351519107818603515625f,-0.121400017744302868294425934436731040477752685546875f,-0.10010000100135840739312698133289813995361328125f,1.349000028431415643126456416212022304534912109375f,0.095999979317188177674324833787977695465087890625f,-0.4311000132501126014261672025895677506923675537109375f,-0.07490000317096701110131107270717620849609375f,-0.303999990880489345812520696199499070644378662109375f,-0.386799954694510006447671912610530853271484375f,-1.1343999859452245093649480622843839228153228759765625f,0.4502000056162476226973012671805918216705322265625f,0.3769999956265090990115140812122263014316558837890625f,-0.16789998666420580253344496668432839214801788330078125f,-0.12080000108182431528547340349177829921245574951171875f,-0.3407000015147032545570482398034073412418365478515625f,-0.25360000877976407451797058456577360630035400390625f,-0.31079997627139110960570178576745092868804931640625f,0.508800010186433837588992901146411895751953125f,1.203300010320544455311164711019955575466156005859375f,0.341800019067525884253200274542905390262603759765625f,-0.672399986220896028754623330314643681049346923828125f,-0.4022000077620149216528488977928645908832550048828125f,-0.1194000459410247128033688568393699824810028076171875f,-0.24779999838471411521823029033839702606201171875f,1.05120001946687668947788552031852304935455322265625f,-0.583199998962879107011758605949580669403076171875f,0.22669994251728109446730741183273494243621826171875f,-0.71859998985528950043999429908581078052520751953125f,-1.0485000526309011803505200077779591083526611328125f,-0.39120001410245919259978109039366245269775390625f,-0.189199998098612098829107708297669887542724609375f,-0.86760001068711289917700923979282379150390625f,0.36129999230503972995620642905123531818389892578125f,0.34159998636841759633853143895976245403289794921875f,-0.43049999010562967072246465249918401241302490234375f,0.09199999707937234205701315659098327159881591796875f,0.23479997773170424579802784137427806854248046875f,-0.46120002468228360470448023988865315914154052734375f,-0.3068000012606386661673241178505122661590576171875f,-0.4097000109285116042912022749078460037708282470703125f,0.504999979473650828509789789677597582340240478515625f,0.44510001120865350099364832203718833625316619873046875f,0.125899997724592704884116756147705018520355224609375f,-0.83010000604912637101051586796529591083526611328125f,-0.73509996941089639221900142729282379150390625f,0.3631000021338461891673432546667754650115966796875f,0.6725999826729296415805947617627680301666259765625f,0.00780001045465461562145037532900460064411163330078125f,1.36969998298883410825510509312152862548828125f,0.7503999847084283914000479853712022304534912109375f,-0.519599988581239902174502276466228067874908447265625f,0.3470000277757645523735163806122727692127227783203125f,-0.57189999816417724787243059836328029632568359375f,-0.432400027161836675304584787227213382720947265625f,0.2020000307559968177884002216160297393798828125f,-0.63309998753070839683232406969182193279266357421875f,0.1704000165224073271019733510911464691162109375f,-0.69479999576210982326074372394941747188568115234375f,-1.0390000078678127692910493351519107818603515625f,-0.47269998233318322178320158855058252811431884765625f,-0.64889998481273636343757971189916133880615234375f,-0.982300015580654051206010990426875650882720947265625f,0.9376999692201621883214102126657962799072265625f,1.074199992585182172888380591757595539093017578125f,0.138799982917308994956329115666449069976806640625f,0.0775999941468239473607582112890668213367462158203125f,-0.6788000072062008172224523150362074375152587890625f,-0.93610006495714170871025316955638118088245391845703125f,-0.62280002669692091643582898541353642940521240234375f,-1.072200012940168534214535611681640148162841796875f,-0.597100043827295312581782127381302416324615478515625f,-0.663599987918138456421957016573287546634674072265625f,-1.0774000097274776255318329276633448898792266845703125f,0.2477999860167503509700281938421539962291717529296875f,0.597999979823827576552730533876456320285797119140625f,-0.079399999207258264988240625825710594654083251953125f,0.623699982959032173113200769876129925251007080078125f,-0.2873999994456768458661599652259610593318939208984375f,0.8289999789968127696937472137506119906902313232421875f,-0.601999992385506832448527347878552973270416259765625f,-0.634099984468519561886523661087267100811004638671875f,-0.0573999992966651717551940237171947956085205078125f,-0.525600023442506891768744026194326579570770263671875f,-0.303000019758939753700133223901502788066864013671875f,0.19580000339150416266420506872236728668212890625f,0.7834000145256521108194647240452468395233154296875f,0.2954000652492048129005297596449963748455047607421875f,-0.7494000093400476902871787387994118034839630126953125f,-0.2294000179827211383098983787931501865386962890625f,0.0839999887943267964374172152020037174224853515625f,0.98139995933175061537667716038413345813751220703125f,-0.03159998010993003703106296597979962825775146484375f,-0.71569999009668805456385598517954349517822265625f,-0.419200005772709882734261555015109479427337646484375f,0.0892999443754558530628173684817738831043243408203125f,-0.1549999894201754546685378954862244427204132080078125f,0.488900030857325251787415254511870443820953369140625f,-0.70550000604987150865099465590901672840118408203125f,-0.240699979591369128684164024889469146728515625f,-0.798999990433454598814932978712022304534912109375f,-0.2686999741673474773051566444337368011474609375f,0.361699983197450780636472700280137360095977783203125f,-0.84000000014901132772138225845992565155029296875f,0.3895999989748000214007106478675268590450286865234375f,-0.3537000035226345318051244248636066913604736328125f,0.216100019732117676340976686333306133747100830078125f,0.00499999620020386004171086824499070644378662109375f,-0.9914999837279323546823661672533489763736724853515625f,0.36239999814182510817772708833217620849609375f,0.215299999390542617927479795980616472661495208740234375f,0.313799998712539718326297588646411895751953125f,-0.0620000136196612317007748060859739780426025390625f,-1.2152999812483784580763312987983226776123046875f,-0.561299992305040351681100219138897955417633056640625f,0.6018999927997583654359914362430572509765625f,-0.084700005042552817258183495141565799713134765625f,-0.1022999510586259930278174579143524169921875f,0.2185999815106394184027749361121095716953277587890625f,0.72319999508857790715410374104976654052734375f,-0.16049998831748979455369408242404460906982421875f,0.993299991917609759184415452182292938232421875f,0.861399983173608774222884676419198513031005859375f,-0.59019998777210691542904896778054535388946533203125f,-0.441399992114305528279061263674520887434482574462890625f,-0.70349998036027017178639653138816356658935546875f,-0.9574999868869780161872995449812151491641998291015625f,1.2194999883621930880650552353472448885440826416015625f,0.1911999960720536240188494048197753727436065673828125f,-0.171399992412329282842620159499347209930419921875f,0.27259999355077724203511024825274944305419921875f,0.5005999882757665719879014432081021368503570556640625f,0.5598000203490258730454343094606883823871612548828125f,0.17320001885592972712402115575969219207763671875f,0.04079998252987859785889668273739516735076904296875f,-0.7918000061035159742317546260892413556575775146484375f,-0.2268000329256059099947151480591855943202972412109375f,-0.33660004433393531098062112505431286990642547607421875f,0.04060000705122945863223549167742021381855010986328125f,0.861600015947222797052518217242322862148284912109375f,0.244199986624717535477202545735053718090057373046875f,0.1663999778091902470578133943490684032440185546875f,-0.6408000090539456294891351717524230480194091796875f,-1.48089998561739921711932765902020037174224853515625f,-0.31180001295804959937640887801535427570343017578125f,0.058499970376491461365731083787977695465087890625f,-0.408399994409084143853760906495153903961181640625f,0.5689999992251395521947188171907328069210052490234375f,-0.6779999850392341986804467524052597582340240478515625f,-0.797499974817037493579618967487476766109466552734375f,0.149200007188320160622652110760100185871124267578125f,-0.1696000012844802729006232766550965607166290283203125f,0.0866000000774860279051381439785473048686981201171875f,0.41829999195486344820693602741812355816364288330078125f,-0.3309999851286409278117162102716974914073944091796875f,-0.7043999935448166382911949767731130123138427734375f,0.08860001161098474131705415857140906155109405517578125f,1.1800999504864222444666665978729724884033203125f,1.0386999665677547621811527278623543679714202880859375f,0.502399963533878946009281207807362079620361328125f,0.6877999952554703089191434628446586430072784423828125f,0.5146999922275536931692840880714356899261474609375f,-0.05650000995397552738808144567883573472499847412109375f,-0.3500999944448466294488753192126750946044921875f,0.0426999702632429034565575420856475830078125f,-1.000199997085331915513961575925350189208984375f,0.1213999991923568888552154021454043686389923095703125f,-0.75200000326335469935656874440610408782958984375f,0.03329999407827866075848533000680617988109588623046875f,0.619599967086315928099793381989002227783203125f,-0.744200002270936966652925548260100185871124267578125f,-0.442499984204768992412937222979962825775146484375f,0.0995999888420107115649670959101058542728424072265625f,-1.148799962353706405338016338646411895751953125f,-0.75039998180270206518116538063623011112213134765625f,-0.9625999604702002443445962853729724884033203125f,-1.5467999532043936739000855595804750919342041015625f,0.3726000139653677933893050067126750946044921875f,0.1769000150024890860578352658194489777088165283203125f,0.64739998618364325011498294770717620849609375f,0.6961999892175196880117482578498311340808868408203125f,-0.29380000385344029467660220689140260219573974609375f,-0.49229998089820148354789353106752969324588775634765625f,-0.36089996415972791510284878313541412353515625f,0.1226000069618222720890798882464878261089324951171875f,0.761200014996529006339187617413699626922607421875f,0.459000009208917703062979853712022304534912109375f,-1.1902999800562863441655281349085271358489990234375f,0.5600000005960463766996326739899814128875732421875f,0.9502000064730637962640003024716861546039581298828125f,0.9284000265955925090821665435214526951313018798828125f,0.972600051367283224834636712330393493175506591796875f,-0.609000017553567918326962171704508364200592041015625f,0.01349998766183801990337087772786617279052734375f,0.3086000205516814531847558100707828998565673828125f,-0.74239995131492531044159477460198104381561279296875f,0.91780000270605111012400811887346208095550537109375f,0.466300025981664756358213708153925836086273193359375f,-0.744200000482797552336933222250081598758697509765625f,-0.7620000388026237914118610206060111522674560546875f,-0.2190000200867654456260424922220408916473388671875f,0.1407000242948530654985006549395620822906494140625f,-0.05639999852180466177742346189916133880615234375f,-0.2400000588968398129718906375273945741355419158935546875f,0.5383999969050285994143223433638922870159149169921875f,0.67790001974478375501576010719873011112213134765625f,-0.403999984659254351981871877796947956085205078125f,0.2655000160150235988254507901729084551334381103515625f,-0.6889999858140942734507916611619293689727783203125f,-1.13890002385377808735711369081400334835052490234375f,-0.79459999390840518884715493186376988887786865234375f,0.74250002503395062802837855997495353221893310546875f,0.46300000262260443406603371840901672840118408203125f,-0.2456999873399734468648603069595992565155029296875f,-1.0068999746203421974399816463119350373744964599609375f,1.06569998972415902471766457892954349517822265625f,1.870900023168325265743305862997658550739288330078125f,-0.3520000367164612953274627216160297393798828125f,0.901000012844800579614457092247903347015380859375f,0.2959999606907359748220187611877918243408203125f,-0.4372999987423418222221016549156047403812408447265625f,-0.31449997621774627987178973853588104248046875f,-0.791899995630979791627623853855766355991363525390625f,1.2915999754905698182483320124447345733642578125f,0.544899979180097915332225966267287731170654296875f,0.513600002074241501759388484060764312744140625f,0.11610001503825184698825978557579219341278076171875f,-0.0906999655842781749015557579696178436279296875f,-0.691400006121396959457570119411684572696685791015625f,-0.5112000048637383997629513032734394073486328125f,0.2919999802410604416991191101260483264923095703125f,0.49160001542568210197714506648480892181396484375f,0.77950000033900124218888549876282922923564910888671875f,-0.243500043224543105679913423955440521240234375f,0.0483000111959874522948865660509909503161907196044921875f,-0.448699963773787846577079108101315796375274658203125f,-1.3523999897599219277566362507059238851070404052734375f,-0.1113999377250678435302688740193843841552734375f,-0.22639994145035746697658396442420780658721923828125f,-0.8175999646425253786219400353729724884033203125f,-1.14180001102089878628476071753539144992828369140625f,1.368999990582466352861956693232059478759765625f,0.338600019806623464546646573580801486968994140625f,-0.5190000241845840189824912158655934035778045654296875f,0.269600012162327817843987531887250952422618865966796875f,1.081400032384693954412568928091786801815032958984375f,0.210600018823146939439538982696831226348876953125f,-1.51219997315406740057142087607644498348236083984375f,0.76620000396966947420196447637863457202911376953125f,-0.133900014466046712868774193339049816131591796875f,-0.00940000308156019315219964482821524143218994140625f,0.38119997804164906796131617738865315914154052734375f,-0.46279999734163279612175756483338773250579833984375f,0.9121999624252321581252545001916587352752685546875f,-1.1318000064015389938276712200604379177093505859375f,-0.04059999170303374427248854772187769412994384765625f,0.21439998832941054018874638131819665431976318359375f,0.214699957656860396237874510916299186646938323974609375f,-0.026199998828768722436421967358910478651523590087890625f,0.02800000895559795477396392016089521348476409912109375f,0.14040001130700119347238796763122081756591796875f,0.33859999387860373332159724668599665164947509765625f,0.7915999993324280836759498924948275089263916015625f,1.00350002849102093449573658290319144725799560546875f,-0.18780000449419009811435898882336914539337158203125f,-0.181600048580765616179633070714771747589111328125f,0.024400000473856930938154619070701301097869873046875f,-1.03980000324994303895209668553434312343597412109375f,0.2305999998986720811444683931767940521240234375f,0.514300010201334867332434441777877509593963623046875f,-0.133400011917948813078282910282723605632781982421875f,-0.46169999135583683624872719519771635532379150390625f,0.0390000049248337976592893028282560408115386962890625f,0.095799993817507811399991624057292938232421875f,-0.8245999937221408782050957597675733268260955810546875f,1.2602000055789943644413142465054988861083984375f,0.91149997165799145903974931570701301097869873046875f,-0.17920000033378613579770899377763271331787109375f,1.17970001025795934168627354665659368038177490234375f,-0.9319000053167343367022112943232059478759765625f,0.20059999781250947847865973017178475856781005859375f,-1.06379997606277498789495439268648624420166015625f,-0.965599963390827209508415762684307992458343505859375f,-0.328599984788894516896107234060764312744140625f,-0.239300047677755145514311152510344982147216796875f,0.899599990630150614379090256989002227783203125f,0.23259999418407684146359315491281449794769287109375f,0.13010000074058769570228832890279591083526611328125f,0.720099988931417467341589144780300557613372802734375f,0.292100001060962455312619567848742008209228515625f,0.2918999624013902671748610373470000922679901123046875f,0.3501999936580659777973778545856475830078125f,0.459099986517429670129786245524883270263671875f,0.86260001888275184001031448133289813995361328125f,-0.5851000138163566788307434762828052043914794921875f,0.07600000666081907496618441655300557613372802734375f,0.44430000744462017081559679354541003704071044921875f,-0.518100008690357327623132732696831226348876953125f,-0.736699992734193731536151972250081598758697509765625f,-0.03700000326335395328669619630090892314910888671875f,-0.4208999838292601936018400010652840137481689453125f,-0.53750001907348643470641036401502788066864013671875f,0.66339999232292168329649939551018178462982177734375f,1.5039999771714216425522181452834047377109527587890625f,0.4362000245332720016477878743899054825305938720703125f,0.8385000214129689766195951960980892181396484375f,0.56000000521540638231954289949499070644378662109375f,0.2680000559687609751335912733338773250579833984375f,-0.07339999709129330085488618351519107818603515625f,-0.388899991071224970795583431026898324489593505859375f,0.060800001454353047591894210199825465679168701171875f,-0.296299985450506397910430678166449069976806640625f,0.16320000358223918368594240746460855007171630859375f,-0.2921000076919797461272310101776383817195892333984375f,0.6770000130236148994100631171022541821002960205078125f,0.6882000086486339540670087444595992565155029296875f,0.9378000128388404021961832768283784389495849609375f,0.3804000411093237943305211956612765789031982421875f,-0.742000007659196825215985882095992565155029296875f,-0.805200047153234788055442550103180110454559326171875f,-0.140399990445375433267827247618697583675384521484375f,0.7180000220686195699926201996277086436748504638671875f,0.401999989107251043396473733082530088722705841064453125f,0.58519998997002797835875753662548959255218505859375f,-0.562399993038177559157020368729718029499053955078125f,0.32890003324151084740378792048431932926177978515625f,0.3004000020682813243411146686412394046783447265625f,-1.1719000087440019086670872638933360576629638671875f,-0.1205999948322773462905388441868126392364501953125f,0.524499973833561394798152832663618028163909912109375f,1.327399955934286168712787912227213382720947265625f,0.0155999696493154260679148137569427490234375f,0.48120000575780874640940965036861598491668701171875f,0.6081000039219848218863262445665895938873291015625f,-0.19759997000694273783238941177842207252979278564453125f,1.25849996948242193184341886080801486968994140625f,-0.459299991798400863984852549037896096706390380859375f,1.06660000350475314689902006648480892181396484375f,0.1820000148117539851000401540659368038177490234375f,1.10299995344877288516727276146411895751953125f,0.691499996542930706056040435214526951313018798828125f,0.2725999916136263578891885117627680301666259765625f,1.13240000540614094148850199417211115360260009765625f,-0.2499000256717209822454606182873249053955078125f,0.6052000067710874287030264895292930305004119873046875f,0.20219995646476718320627696812152862548828125f,0.37500001385808001685973067651502788066864013671875f,-0.78909997012615207268027006648480892181396484375f,0.065200027036666607926918004523031413555145263671875f,-1.119499997079373088126885704696178436279296875f,-0.1079000086620449661012344222399406135082244873046875f,0.3217999709740286817805099417455494403839111328125f,-0.278599991829693471867557263976777903735637664794921875f,0.02260000845193843588276649825274944305419921875f,-0.836000001668929737519420086755417287349700927734375f,1.4495000033378602211087127216160297393798828125f,1.37189999488592118126462082727812230587005615234375f,-0.63050000202655809289353783242404460906982421875f,-0.73450001212954507678887239308096468448638916015625f,-0.181200000989437004506044104346074163913726806640625f,-0.04320000447630878426252820645458996295928955078125f,0.43839999414831343216292225406505167484283447265625f,-0.1812000004306435207990233493546838872134685516357421875f,-0.17500000899657608410819875643937848508358001708984375f,0.0074000089824199921650915712234564125537872314453125f,0.687700032997131671663737506605684757232666015625f,-0.1254000016212462043085906771011650562286376953125f,-0.524899998849630566866153458249755203723907470703125f,0.393599982404708725880482234060764312744140625f,-0.195900008788705892737169733663904480636119842529296875f,-0.8858000128537419914920292285387404263019561767578125f,-0.993400003536045961283207361702807247638702392578125f,0.4045999924182890339352525188587605953216552734375f,1.14479998168051277929180287173949182033538818359375f,-0.391799984198808459723295527510344982147216796875f,-1.280599982166289674268000453594140708446502685546875f,0.198599997305870079600254030083306133747100830078125f,0.70069999955892559029280164395458996295928955078125f,0.53900000473856923832727261469699442386627197265625f,-1.165800000485777587755364947952330112457275390625f,0.23299999487400047115670531638897955417633056640625f,0.6157000208675864616481021585059352219104766845703125f,-0.8900000114738941814351846915087662637233734130859375f,-0.77440002230405813321567620732821524143218994140625f,-0.24039999342560758321951652760617434978485107421875f,-1.65129999483823741712740229559130966663360595703125f,0.24739999408125878943565112422220408916473388671875f,0.92100000166893014608149314881302416324615478515625f,0.57080001143813152708617053576745092868804931640625f,0.5970999965906145234839641489088535308837890625f,-0.23740001138523225687748663403908722102642059326171875f,0.29510001048222189723446717835031449794769287109375f,0.0526000110037624091230412659569992683827877044677734375f,0.275499998301267368105982313863933086395263671875f,-0.849700007128715295579013400129042565822601318359375f,0.895000021755695485126125277020037174224853515625f,-0.320800010395049906009035112219862639904022216796875f,0.872199971961975961676216684281826019287109375f,-0.4800999976113440315117486534290947020053863525390625f,-0.1794000036403531339601613581180572509765625f,0.6943999632582069647668276957119815051555633544921875f,0.2722999650657182968416236690245568752288818359375f,-1.18510000711083396396361422375775873661041259765625f,0.03890004560947346590182860381901264190673828125f,0.412200010555982654381068641669116914272308349609375f,-0.038800004076957606002906686626374721527099609375f,-0.5338000215113163360314274541451595723628997802734375f,0.7881999839127065143884465214796364307403564453125f,0.358299985808133936160402299719862639904022216796875f,-0.81460000553131095557546359486877918243408203125f,-0.97969999252557737889901545713655650615692138671875f,-0.0076999744355674692997126840054988861083984375f,-1.4122000075757503712026164066628552973270416259765625f,0.007300026905536327603840618394315242767333984375f,0.8845000018477440217878893236047588288784027099609375f,-0.843900003588199609794173738919198513031005859375f,0.251999998569488514732483963598497211933135986328125f,-0.10540002710223195236949322861619293689727783203125f,0.02599999481439585480302412179298698902130126953125f,1.19839998755454946177678721142001450061798095703125f,0.233200003582239023813826861442066729068756103515625f,-0.57689996851086622342563714482821524143218994140625f,0.61480000500082976344629059894941747188568115234375f,0.674999989718198634136570035479962825775146484375f,-0.587600005620718235377353266812860965728759765625f,-1.848499976038933567679123370908200740814208984375f,-0.49660001810789111686972319148480892181396484375f,0.83399997523426971923754535964690148830413818359375f,-0.01699999216198921914156017010100185871124267578125f,0.462299980153143774685986500116996467113494873046875f,0.4079999986365436814850227165152318775653839111328125f,0.4830000059939922874008288999903015792369842529296875f,0.7702000158607962276846592430956661701202392578125f,0.68509999518990571942822498385794460773468017578125f,-0.24799998965859426647284635691903531551361083984375f,-0.750599958026409108668985936674289405345916748046875f,0.722200018900633011753598111681640148162841796875f,-0.1787999540090563499461495666764676570892333984375f,-0.2032000017940998048970868694595992565155029296875f,0.29070001401305223254212251049466431140899658203125f,0.033400016462802994965386460535228252410888671875f,0.053199998292326799997908892692066729068756103515625f,-0.44999999471008766960267166723497211933135986328125f,0.074199992845952689179966910160146653652191162109375f,0.35899999579787245096440528868697583675384521484375f,0.34480000708699254374778320197947323322296142578125f,0.4294000148534775629372006733319722115993499755859375f,-0.057800015819072569911440950818359851837158203125f,-0.562300001424551165740695068961940705776214599609375f,0.8332999725461007756166509352624416351318359375f,0.560499979972839401654027824406512081623077392578125f,0.33209998884201041846608859486877918243408203125f,-0.829399952566623799299350139335729181766510009765625f,0.1376999876976015002583153545856475830078125f,0.5561999854922294872494603623636066913604736328125f,-1.3765999838352200868030195124447345733642578125f,-0.0446000113427640343388702603988349437713623046875f,0.188499995410442267029793583787977695465087890625f,0.8941999957889319450288212465238757431507110595703125f,0.412699970747530731074448340223170816898345947265625f,0.2473999621927738257198825522209517657756805419921875f,0.2069000118732446935609914362430572509765625f,-0.49679994605183619427180019556544721126556396484375f,0.454899987375736003514248295687139034271240234375f,0.052600022011995317683386019780300557613372802734375f,0.721899986541270521911428659223020076751708984375f,0.1605000042617319433446709808777086436748504638671875f,-0.399899987375736287731342599727213382720947265625f,0.3692000346064567839476922017638571560382843017578125f,0.8325999962329859727105940692126750946044921875f,0.01270000200271592660783426254056394100189208984375f,-0.55399996949732344120320703950710594654083251953125f,-0.71030001462697978098503881483338773250579833984375f,0.471699981856346539643709547817707061767578125f,-1.44089996982216828058653845801018178462982177734375f,-1.1836999901115898836678752559237182140350341796875f,-0.1303999967038631524474112666212022304534912109375f,-0.09179999537467953274472165503539144992828369140625f,0.5935999910473821472578492830507457256317138671875f,-0.017999966599048011772765676141716539859771728515625f,0.4381999963924287921912537058233283460140228271484375f,-0.7362999720580878371123390024877153337001800537109375f,0.28259999593496321068641918827779591083526611328125f,-0.115500020802020930688058797386474907398223876953125f,0.07359997182488431377578308456577360630035400390625f,0.354199996608495215610901141189970076084136962890625f,-0.277799995404481858685130646335892379283905029296875f,-0.214599989363550680554482141815242357552051544189453125f,-0.6156000005692241483501447874004952609539031982421875f,-0.60860001783221928661760102841071784496307373046875f,-0.758999995797872362146563318674452602863311767578125f,-0.118500002190470576923786438783281482756137847900390625f,-0.203000001654028816577834959389292635023593902587890625f,-0.5913000180467964383979051490314304828643798828125f,-0.304200010168552470446456936770118772983551025390625f,0.78809999483227688443776060012169182300567626953125f,0.427799998235702361171206575818359851837158203125f,-0.630400003260373598124033378553576767444610595703125f,-0.753799993199109952257686018128879368305206298828125f,0.18319998197555553698379071647650562226772308349609375f,-0.97300002333521862230014676242717541754245758056640625f,-0.905000022873282627955404677777551114559173583984375f,0.802800025355816071481740436865948140621185302734375f,-0.4240999810039998152433327049948275089263916015625f,-0.70419999407529854096310373279266059398651123046875f,0.04710000731945029883718234486877918243408203125f,-0.18789998761415471761893059010617434978485107421875f,1.009599949204922353374058729968965053558349609375f,0.0373999813407659420505524394684471189975738525390625f,0.616599976496398749503669023397378623485565185546875f,1.058199980187416233690100852982141077518463134765625f,-0.181200009930133631996795884333550930023193359375f,0.042899977481365425546755432151257991790771484375f,-0.98170000331401752191595733165740966796875f,-0.5958000035405159877655023592524230480194091796875f,-0.1170999849677087922827922739088535308837890625f,-0.6825999961957334161155586116365157067775726318359375f,-0.2845999565437438150183879770338535308837890625f,-0.6452999756976962597576630287221632897853851318359375f,-0.6830999514698987695737741887569427490234375f,-0.8230999947950243811334303245530463755130767822265625f,0.090000017546117305755615234375f,0.211000005133450019290108912173309363424777984619140625f,-0.584599997782706992666135192848742008209228515625f,-0.9042000356495381030441649272688664495944976806640625f,1.03719997136592922970521613024175167083740234375f,0.290200010496378002500250659068115055561065673828125f,-0.619099964165688021466849022544920444488525390625f,-1.5033999989539383879133538357564248144626617431640625f,-0.1671999848514786890518735162913799285888671875f,-0.21029999532997611932927384259528480470180511474609375f,0.728599987769126045122902723960578441619873046875f,0.9177000258445737745915948835317976772785186767578125f,0.1305999849975105764343652481329627335071563720703125f,0.4126000246942045368570006758091039955615997314453125f,-0.12509998267292932183636366971768438816070556640625f,0.57259999310374265490963807678781449794769287109375f,0.248900040841102399014062029891647398471832275390625f,0.581400025045871959861187860951758921146392822265625f,0.0051000131458049935417875531129539012908935546875f,-0.15720002039074909561122694867663085460662841796875f,-1.375400020210445273249888487043790519237518310546875f,0.2183999944463371478686752880093990825116634368896484375f,1.415900003889948077784310953575186431407928466796875f,-0.30559998961687095953010384619119577109813690185546875f,-0.1624000025004150860041818305035121738910675048828125f,-0.8050000034272672966295658625313080847263336181640625f,-1.6782000141620638800077358609996736049652099609375f,0.3400000055134295706693592364899814128875732421875f,1.22740000928044334926880765124224126338958740234375f,-0.822000005871057481954267132095992565155029296875f,0.4626999640792612833450903053744696080684661865234375f,-0.0657999934077262427223331542336381971836090087890625f,-1.6890000078305593689265151624567806720733642578125f,0.11719999714493756215460962266661226749420166015625f,0.07760000696182267621026085180346854031085968017578125f,-0.35559999483227711092325762365362606942653656005859375f,0.310700003358721943413911503739655017852783203125f,0.00239999125003809155032286071218550205230712890625f,-0.056500021427870361634404616779647767543792724609375f,0.202200001317262678668384978664107620716094970703125f,1.06819998614788058688418459496460855007171630859375f,-0.276799998056888707509415326057933270931243896484375f,0.622700003045796535161571227945387363433837890625f,0.293400006926059830902886460535228252410888671875f,-0.09210000910758964209890109486877918243408203125f,0.40719999833703024449960139463655650615692138671875f,0.5553000058352939305450490792281925678253173828125f,-0.42660000573992717942672925346414558589458465576171875f,-0.344099996650218287186362431384623050689697265625f,0.60259999334588643105092842233716510236263275146484375f,-0.914500013116747112462689983658492565155029296875f,-0.87709998047873394266815694209071807563304901123046875f,0.6316999956399200755186029709875583648681640625f,-1.03099998252093794093298129155300557613372802734375f,0.21529999927878407106618396937847137451171875f,1.0270999977827071081293297538650222122669219970703125f,-0.46490002582073230996684287674725055694580078125f,-0.15400001710653299369369051419198513031005859375f,-0.506099964523315293263294734060764312744140625f,-1.7583000221669673468483097167336381971836090087890625f,0.74030001574456694157788660959340631961822509765625f,0.60970000828355563893268254105350933969020843505859375f,-0.665299994957447182741816504858434200286865234375f,-0.23969999304711819831936736591160297393798828125f,-0.7805999609321361276670359075069427490234375f,0.24350001947581778605211866306490264832973480224609375f,0.46139998421669048411786206997931003570556640625f,0.739799978625774290463823490426875650882720947265625f,-0.38730000306367884377323207445442676544189453125f,0.516400016850232734810788315371610224246978759765625f,-1.363800022926926036603845204808749258518218994140625f,-0.470399984112381763790011746095842681825160980224609375f,-0.228900000496208289035848792991600930690765380859375f,-0.7798000106632709815812631859444081783294677734375f,-0.93960003443956363611277993186376988887786865234375f,-0.29259998282194121799193453625775873661041259765625f,0.35130002508759616830502636730670928955078125f,0.3852000069200993692675183410756289958953857421875f,-0.993799981576204860544976327219046652317047119140625f,-0.160999994069337848401346491300500929355621337890625f,0.585899999326467746385560531052760779857635498046875f,-0.401800006550550481421169024542905390262603759765625f,-0.377800010603666525099697537370957434177398681640625f,-0.092399999892711637272668667719699442386627197265625f,-0.3291000122219320811467468956834636628627777099609375f,0.75100001657009141098342297482304275035858154296875f,1.732600014412402966712534180260263383388519287109375f,0.384199987220763983231108795735053718090057373046875f,-0.001800001186132771380243866587989032268524169921875f,-0.112799998086690944631982347345910966396331787109375f,-0.453799989622831301261385306133888661861419677734375f,0.0785999979019165184723760830820538103580474853515625f,-0.02899999270588160538153488232637755572795867919921875f,0.533799997669458292648414499126374721527099609375f,0.9311999929428100841732884873636066913604736328125f,-0.5212000039696693676205541123636066913604736328125f,-0.526200005310773377686928142793476581573486328125f,0.87940002066493061505525474785827100276947021484375f,0.69960002609491400704655461595393717288970947265625f,-0.5136999646678568343105553140048868954181671142578125f,-0.35679996344894249205026426352560520172119140625f,-0.840599991442263050345218289294280111789703369140625f,-0.50629997189044928518342203460633754730224609375f,-0.971600008720159724617815300007350742816925048828125f,-0.8527999606847771474349428899586200714111328125f,-0.90109996139407133597387655754573643207550048828125f,1.1466999943733213740415521897375583648681640625f,0.96060000898838016070158118964172899723052978515625f,-0.574099996054172834192286245524883270263671875f,-0.568399993067979725225313814007677137851715087890625f,-0.07449998873472196692091529257595539093017578125f,0.047199990886449771920752027654089033603668212890625f,-0.1345999964416026983826668583787977695465087890625f,1.4855000131279230135561419956502504646778106689453125f,-0.092000015854836192374932579696178436279296875f,0.01049998906254767661039295489899814128875732421875f,0.670999966502190403616623370908200740814208984375f,-0.0370000049769878547323287421022541821002960205078125f,0.1398999988496303359397643362171947956085205078125f,-0.1728000096350907899278581680846400558948516845703125f,-0.06489999683797353924319395446218550205230712890625f,0.85349998799711468411288706192863173782825469970703125f,0.566699965763092450288240797817707061767578125f,-0.0900000129640101675931873614899814128875732421875f,0.03740001628398914590434287674725055694580078125f,0.1946999874591828216807698481716215610504150390625f,0.55330000205039908678372739814221858978271484375f,-0.333800001543760327393783882143907248973846435546875f,0.4209999818503860069540678523480892181396484375f,-0.587599998468160578113383962772786617279052734375f,0.188199977207183177796423478866927325725555419921875f,0.1136000055015085052900758455507457256317138671875f,0.3360000426471234646896846243180334568023681640625f,0.0216000182569029419710204820148646831512451171875f,-0.6188000399887554348055118680349551141262054443359375f,0.4789999886453151223264512736932374536991119384765625f,0.746699977684020410606535733677446842193603515625f,0.8050000055134294285608120844699442386627197265625f,0.94650005495548317213660993729718029499053955078125f,-0.63400000548362722696538185118697583675384521484375f,-2.075800039750337777633149016764946281909942626953125f,0.045600008243322331935587499174289405345916748046875f,0.2298999800741672316917174612171947956085205078125f,0.03499997898936246798484717146493494510650634765625f,-0.0884999696314341388614366223919205367565155029296875f,0.2312000023305416096519593338598497211933135986328125f,0.54840000513792031000548377051018178462982177734375f,0.3674000091314315596946471487171947956085205078125f,1.0762000149965285089592725853435695171356201171875f,0.2964000171482563672498145024292171001434326171875f,-0.058099972778558051089703440084122121334075927734375f,0.164400005763769119226935799815692007541656494140625f,-0.74130000020265551796683212160132825374603271484375f,-0.37279999018907528096633541281335055828094482421875f,-1.063199999707937326576256964472122490406036376953125f,-0.376200012165308184108880595886148512363433837890625f,-0.268600023233890539131607511080801486968994140625f,-1.66500002831220594856631578295491635799407958984375f,-0.2311999898135663755738278268836438655853271484375f,-1.5504000172488396902537033383850939571857452392578125f,0.5159999724626544992389653998543508350849151611328125f,-0.83560005721226371377952091279439628124237060546875f,-0.59979999173879594565050865639932453632354736328125f,0.75520000077336979504849523436860181391239166259765625f,-0.096000024765729052855789404929964803159236907958984375f,1.4057999834984544751392832040437497198581695556640625f,-0.513599979871511624907043369603343307971954345703125f,0.73980000310093141369094382753246463835239410400390625f,-0.07020000304579721017717020004056394100189208984375f,0.7956000118196009385229672261630184948444366455078125f,-0.16800000306963891461009552585892379283905029296875f,1.7034000221624971960210359611664898693561553955078125f,-0.3155999838054182049518203712068498134613037109375f,0.6835000252500176287639988004229962825775146484375f,0.573199986147880480302774230949580669403076171875f,-0.49059997657835463513720242190174758434295654296875f,0.373599983000755475615051182103343307971954345703125f,0.25830001039505046112054742479813285171985626220703125f,-0.25139998987913136918592726942733861505985260009765625f,0.497700019008666227904313927865587174892425537109375f,0.15359999711960548030020845544640906155109405517578125f,0.359499997861683573407987069003866054117679595947265625f,-0.26759998926669370522546387292095459997653961181640625f,1.68950001740083077805820721550844609737396240234375f,0.01020000244975083347753752605058252811431884765625f,0.9109999912753703465995158694568090140819549560546875f,-0.21380000765323636802150986113701947033405303955078125f,0.73099998639523988686050870455801486968994140625f,0.61349999712407576879513726453296840190887451171875f,0.95800001066923101689098984934389591217041015625f,0.2845000259876251380575240546022541821002960205078125f,-0.90129998552501200403952452688827179372310638427734375f,-0.5063999876439571057318289604154415428638458251953125f,-0.7403999981939790586693561635911464691162109375f,-0.6755000144690275210024310581502504646778106689453125f,-0.25460000500977020276849316360312514007091522216796875f,-0.584500007361173690156874727108515799045562744140625f,0.24220001413226110997811701963655650615692138671875f,-0.208100025305152147581111421459354460239410400390625f,0.8874999710917474970983676030300557613372802734375f,-0.2608000141203401511802439927123486995697021484375f,-0.95530000091791134053664791281335055828094482421875f,-0.829699995356798059020775326644070446491241455078125f,-0.6065999917328357415868822499760426580905914306640625f,-1.016199989813566073593165128841064870357513427734375f,-1.3683999753355979844826606495189480483531951904296875f,-0.51190000822246106704227486261515878140926361083984375f,0.751099989259242928341109291068278253078460693359375f,0.3995000163391235759036135277710855007171630859375f,0.17550000375881780190212566594709642231464385986328125f,0.5679000113628804857768272995599545538425445556640625f,0.3071999882042408813731526606716215610504150390625f,0.40289998917877678508148164837621152400970458984375f,0.55899999222159379996810457669198513031005859375f,0.33010000256598015511144694755785167217254638671875f,0.692100004488229725296832839376293122768402099609375f,-0.070199998724460588306328645558096468448638916015625f,-0.3944000271096828047490134849795140326023101806640625f,-0.68099999257922139150878138025291264057159423828125f,-0.2807000033959743401368314152932725846767425537109375f,0.2102000006616113747526242150343023240566253662109375f,0.205900016239285665875513586797751486301422119140625f,0.8697999924838537566529339528642594814300537109375f,0.50080000864416351902264068485237658023834228515625f,0.46499999642372102925946819595992565155029296875f,1.5138000175252559476035685293027199804782867431640625f,0.00780001060366597442907732329331338405609130859375f,1.4083000130400058491630943535710684955120086669921875f,-0.71779997856616972740084747783839702606201171875f,1.11840002763867385482399186003021895885467529296875f,0.0292000058472154933042475022375583648681640625f,0.915900012700259669173874499392695724964141845703125f,0.325999994218349353758412689785473048686981201171875f,0.7100000174343590320091834655613638460636138916015625f,0.551399997478723147281698402366600930690765380859375f,0.313099984811245679594549073954112827777862548828125f,-0.3168000056564803657010997994802892208099365234375f,-0.351200046978145830056661225171410478651523590087890625f,-0.5311999786376955245970066243899054825305938720703125f,-0.10690000954493950047208272735588252544403076171875f,0.3841999817073347145424122572876513004302978515625f,-0.678000017710030267181764429551549255847930908203125f,-0.073000012308358730450663642841391265392303466796875f,-0.77030000468045412009843175837886519730091094970703125f,0.358799967867136027077634707893594168126583099365234375f,1.08429996780753157281651510857045650482177734375f,0.450299977150559360694614952080883085727691650390625f,0.53709999822974197059011203236877918243408203125f,-0.3759000070005651394922097097150981426239013671875f,-1.11820002295374809619943334837444126605987548828125f,-0.882400039306282746309761932934634387493133544921875f,0.31809997099041975587851993623189628124237060546875f,0.291299972113967076126073152408935129642486572265625f,1.18749998509883880615234375f,0.0533999921739098937223388929851353168487548828125f,-1.11759998371601110278561463928781449794769287109375f,-0.63189999637603744275793360429815948009490966796875f,0.2856999862968923054040715214796364307403564453125f,-0.970800007340312021142381126992404460906982421875f,-1.35189998192191129788852776982821524143218994140625f,-1.380799969938397264712648393469862639904022216796875f,-0.63829996360540430799801470129750669002532958984375f,-0.02999996468424825923904109004070051014423370361328125f,-0.045399984596669884506781045274692587554454803466796875f,-0.43999998744577173415137849588063545525074005126953125f,-0.48889998948797586120207370186108164489269256591796875f,-0.038300016523152458081113991283928044140338897705078125f,0.9717999688506129274401246220804750919342041015625f,0.828299963903427194367168340249918401241302490234375f,0.884699967640638629262639369699172675609588623046875f,0.1209000014871358941181966883959830738604068756103515625f,-0.882999967884272596307937419624067842960357666015625f,-0.9493999946251514554518280419870279729366302490234375f,0.55270001690387715598262730054557323455810546875f,-0.82239997470974923743369799922220408916473388671875f,1.4088000596210361603510818895301781594753265380859375f,-0.138400006181001344884862191975116729736328125f,0.833900032608211549955967711866833269596099853515625f,0.8511999992012970750465683522634208202362060546875f,1.28430001146793326682882252498529851436614990234375f,-0.02759998922944095767206817981787025928497314453125f,0.4076000169083477953080318911815993487834930419921875f,-0.6101999766707422168110497295856475830078125f,0.16150001479685283101161985541693866252899169921875f,0.02459999018311531671088232542388141155242919921875f,1.3215000199005009218211625920957885682582855224609375f,0.019000001013278922101790158194489777088165283203125f,1.492599998617171852544061039225198328495025634765625f,0.23019998024702115202444474562071263790130615234375f,1.0113999681234362437720619709580205380916595458984375f,-0.4905999866366386186200543306767940521240234375f,0.342099995882808816105580262956209480762481689453125f,0.22079998558461688684673163152183406054973602294921875f,0.9141000043056906054772525749285705387592315673828125f,0.5459999828934669618973885008017532527446746826171875f,0.04130000819340384765609996975399553775787353515625f,-0.2677999808013442617493637953884899616241455078125f,-0.250299980540573674403503900975920259952545166015625f,-0.4319999808371068894530253601260483264923095703125f,-0.31129999280795439542401936705573461949825286865234375f,0.21279999320656062256063023596652783453464508056640625f,0.6856000106833874951917096041142940521240234375f,0.66409999150931842137879357323981821537017822265625f,0.554999977648258369100631171022541821002960205078125f,-0.5032000092446800021406261294032447040081024169921875f,-0.957900000727176514914162908098660409450531005859375f,-0.6333999956011771015340627855039201676845550537109375f,-0.340999997496604922986307428800500929355621337890625f,0.2422000051915647045319701646803878247737884521484375f,0.80789999044537541550425885361619293689727783203125f,0.2117999973863362583870184607803821563720703125f,-0.2466999937772751394504666677676141262054443359375f,-0.334199994224309904211622779257595539093017578125f,-0.36479999203681945374455608543939888477325439453125f,-0.3406000121176244732623672462068498134613037109375f,0.084799999263882597233532578684389591217041015625f,-0.104400009861588483062178056570701301097869873046875f,0.08359999089837089059074060060083866119384765625f,-0.0922000072404742798681809290428645908832550048828125f,-0.20839999271407727565730283458833582699298858642578125f,0.066099998665601067404651303149876184761524200439453125f,0.1111999963700771587582494248636066913604736328125f,0.25209999919831727055452574859373271465301513671875f,-0.345899988895654875165064368047751486301422119140625f,-0.2622999803394077122931093981605954468250274658203125f,-0.6616999986946581202573725022375583648681640625f,-0.52570001177787784030215334496460855007171630859375f,0.156999996036291111778382401098497211933135986328125f,0.822800007101893271510562044568359851837158203125f,0.313800002586841486618141061626374721527099609375f,0.1424999833852051978055897052399814128875732421875f,-0.6773999950870874808828148161410354077816009521484375f,0.4707999734401706604103310382924973964691162109375f,0.0444999671280383868321450790972448885440826416015625f,0.0817999983549118070413896930404007434844970703125f,0.5236999918997284897415056548197753727436065673828125f,0.02239999333620090737895225174725055694580078125f,-0.1801000106126069422174396095215342938899993896484375f,-0.1841999937772751394504666677676141262054443359375f,0.0835999949961898636274781893007457256317138671875f,-0.0574000031709669400470374966971576213836669921875f,0.526700026261806897309725172817707061767578125f,0.3291999934792517734649663907475769519805908203125f,0.0680000038146972851649252334027551114559173583984375f,-0.38719999401569349384288898363593034446239471435546875f,1.1832000520855192604585681692697107791900634765625f,0.00720000057220460121243377216160297393798828125f,0.6182000405929983788411163914133794605731964111328125f,0.7593999734282494973314214803394861519336700439453125f,-1.4550000324286518349481411860324442386627197265625f,-0.127799993020296209778052798355929553508758544921875f,-1.523900030000508110106238746084272861480712890625f,-0.53680000178217834871929881046526134014129638671875f,0.636200027401745860089476991561241447925567626953125f,0.2772000001251695433523991596302948892116546630859375f,0.22269999451488242669938699691556394100189208984375f,-0.409599987947940835653071189881302416324615478515625f,-0.901399981832504249013027219916693866252899169921875f,0.43299997505545650966496395994909107685089111328125f,-0.038899979262053019368750028661452233791351318359375f,-0.02229999356418843614591196455876342952251434326171875f,0.31009999852403991038585218120715580880641937255859375f,0.37790000774934895400036793944309465587139129638671875f,0.0658999935708941275702699158500763587653636932373046875f,0.249299989116191778038000848027877509593963623046875f,0.14720000027418134180834385915659368038177490234375f,0.170099997723102358548885604250244796276092529296875f,-0.163899993082881056949418052681721746921539306640625f,-0.2982000101387500823335585664608515799045562744140625f,0.75319998480677607943789553246460855007171630859375f,0.2866000113651157032990113293635658919811248779296875f,-0.22079998422488589238099621070432476699352264404296875f,-0.422899995306879106227171405407716520130634307861328125f,-0.572600009793042818984076802735216915607452392578125f,-0.3276000079303982293055241825641132891178131103515625f,0.389700023669004025350659503601491451263427734375f,0.15340000923573970936786281527020037174224853515625f,0.59809998931884766903976924368180334568023681640625f,0.258099999749660913295201680739410221576690673828125f,-0.150200017201900237751033273525536060333251953125f,0.600800004136562648682229337282478809356689453125f,0.06249998763203645779640282853506505489349365234375f,-0.228100020536780245805630329414270818233489990234375f,0.5664999943077564381610500277020037174224853515625f,0.06489999966919413054711185395717620849609375f,-0.305999998837709341614754521287977695465087890625f,-0.2971999943137166866335974191315472126007080078125f,-0.206000001594424109185865745530463755130767822265625f,0.0452000096768137016312039122567512094974517822265625f,-0.322100026672333561439387494829134084284305572509765625f,-0.5715999817490577949996577444835565984249114990234375f,0.16460001342892649489613177138380706310272216796875f,0.158800003220140706350349546482902951538562774658203125f,-1.132499997187405682552707730792462825775146484375f,-0.17059999274611481467900375719182193279266357421875f,-0.276499990992248090293514906079508364200592041015625f,-0.18780000002384167334668063631397671997547149658203125f,1.6079000175654887261345038496074266731739044189453125f,-0.54319998972415906024480136693455278873443603515625f,0.54780000622645008689914902788586914539337158203125f,0.5081999778032304515562600499833934009075164794921875f,0.30220000349655773419499382725916802883148193359375f,0.629199992585181888671286287717521190643310546875f,0.8833000168770548388152974439435638487339019775390625f,0.49339996758699466994357862859033048152923583984375f,0.6627999812483784669581154957995750010013580322265625f,0.44999997660517720987627399154007434844970703125f,-1.100700053314864756970337111852131783962249755859375f,0.16480000425577134848253990639932453632354736328125f,-1.1125000064820051992597882417612709105014801025390625f,0.785999990791081959429220660240389406681060791015625f,0.15639997732490307225106107580359093844890594482421875f,-0.56839996684193661025119581609033048152923583984375f,0.8907000163599845254935871707857586443424224853515625f,0.2536000002861020785616119610494934022426605224609375f,0.365999996230006285902192075809580273926258087158203125f,0.49400000458955706239549954261747188866138458251953125f,-1.4094000552356249489349693249096162617206573486328125f,1.0012000080674885449383282320923171937465667724609375f,0.683699996221065475765499286353588104248046875f,0.385199988889694200366875520558096468448638916015625f,0.2887999870896340581793992896564304828643798828125f,-0.3670999971866606426118551098625175654888153076171875f,-0.926799975109100504511161489062942564487457275390625f,-0.47229998346865187386356410570442676544189453125f,-0.45069999546110628418915666770772077143192291259765625f,-0.447000001102685728948671339821885339915752410888671875f,-0.100600008876621516140659196025808341801166534423828125f,0.049899973741173886399025150240049697458744049072265625f,0.86509996818155077136225372669287025928497314453125f,0.159999986439943508997885146527551114559173583984375f,0.00780000374913214500338654033839702606201171875f,-0.154299994853138944250758868292905390262603759765625f,-0.47179998866915706656754991854541003704071044921875f,-0.153000013574958071416176608181558549404144287109375f,0.06299999293684965806505715590901672840118408203125f,0.8406000123411414559626564368954859673976898193359375f,0.59580002171993218240686474018730223178863525390625f,-0.095199983003735777487008817843161523342132568359375f,0.0508000282764431432269702781923115253448486328125f,-0.35180001072287581109776510857045650482177734375f,0.694699978071451340611019986681640148162841796875f,0.015500005155801854783703674911521375179290771484375f,-0.4612999787077309843397188160452060401439666748046875f,-0.47539999726265669277580627749557606875896453857421875f,-0.2931000055611132637523041921667754650115966796875f,0.567000001847743906324694762588478624820709228515625f,-0.36950001041591207240344374440610408782958984375f,0.6369999830722807843130794935859739780426025390625f,-0.375100020372868048212922076345421373844146728515625f,0.47059997844100021069380090921185910701751708984375f,-0.4634000168725844748252029603463597595691680908203125f,0.18539999551177022141246197861619293689727783203125f,0.1606999916240570147607513717957772314548492431640625f,0.3869999859035015976616023181122727692127227783203125f,0.9163000151410700677701015592901967465877532958984375f,0.49179999470412705253608010025345720350742340087890625f,0.53150000328570567997843454577377997338771820068359375f,0.1181999988883732921607361276983283460140228271484375f,0.537700007776916066859484999440610408782958984375f,0.2249999959766864066068592364899814128875732421875f,-0.520400005458295122906520191463641822338104248046875f,0.28319999836683251714930520392954349517822265625f,-0.93400000444054587234177233767695724964141845703125f,-0.82239997664690012157961973571218550205230712890625f,-0.2459999440014357663386590502341277897357940673828125f,0.358799981725216099448516615666449069976806640625f,0.9346000003904106900876058716676197946071624755859375f,0.1935999961137770952745995600707828998565673828125f,1.748300009948014821503647908684797585010528564453125f,-0.1667999993979929485021784785203635692596435546875f,0.80780000829398612705034565806272439658641815185546875f,0.738999968826770992080810174229554831981658935546875f,-0.3124000269755722314357626601122319698333740234375f,0.156599986815452840716034188517369329929351806640625f,-0.71149996681511351592774872187874279916286468505859375f,-0.413399999102949777807225473225116729736328125f,0.615999978572130402199036325328052043914794921875f,-0.2920999991238115711666978313587605953216552734375f,-0.19839999232292182540504654753021895885467529296875f,-0.7943000058799982721069454782991670072078704833984375f,-0.238000000759959196017234717146493494510650634765625f,-0.0545999906301497528460231478675268590450286865234375f,0.01440001157522186048254297929815948009490966796875f,0.4291000334560870488331829619710333645343780517578125f,-0.45739997709393520608500693924725055694580078125f,-0.17979999144077318806722587396507151424884796142578125f,-0.665800006446242154112269417964853346347808837890625f,-0.4192999923199411949070736227440647780895233154296875f,0.08680000386834141323788571753539144992828369140625f,0.419700000646710424945240447414107620716094970703125f,0.561899981772899792531461571343243122100830078125f,0.4737000028520821359734327415935695171356201171875f,-0.686899974322319195607633446343243122100830078125f,-0.0281000045180324065086097107268869876861572265625f,-0.1864000029921530909859939129091799259185791015625f,0.7679999783337116969761382279102690517902374267578125f,1.090499961793423011613413109444081783294677734375f,0.321199974241852981293732227641157805919647216796875f,0.12179999269247065996069068205542862415313720703125f,-0.42740000644922260253366630422533489763736724853515625f,-0.459899991808831731798790087850647978484630584716796875f,-0.0283000060737133851684887986266403459012508392333984375f,0.16539999119043358177805203013122081756591796875f,0.3335999987214801620893922518007457256317138671875f,1.7450999974250793744801057982840575277805328369140625f,-0.2953999767363075790171933476813137531280517578125f,1.6441999776467672855773116680211387574672698974609375f,0.0814000004589556791501081534079276025295257568359375f,1.4883999986931677472767887593363411724567413330078125f,-0.26000000089406938297287297245929948985576629638671875f,-0.1989999926686281117582666411180980503559112548828125f,-0.24299998787045495873826439492404460906982421875f,-1.461800034974515494212710109422914683818817138671875f,0.27240000227689709078049418167211115360260009765625f,-0.920600035884977074829293997026979923248291015625f,0.244999989271164064774666258017532527446746826171875f,0.5615999869272114697338338373810984194278717041015625f,-0.11019998486638105106294460711069405078887939453125f,-0.3953000149622563075268999455147422850131988525390625f,0.0780000057518479561480262418626807630062103271484375f,-1.478399992434680232378241271362639963626861572265625f,0.5451999848663806602644399390555918216705322265625f,-1.320699983651935749406902687042020261287689208984375f,0.26239998707771317043579983874224126338958740234375f,-1.0728999807596204529858141540898941457271575927734375f,-0.0018000001430511503031084430404007434844970703125f,0.4021000211775305199779495524126105010509490966796875f,-0.00359999551773082515637725009582936763763427734375f,0.8327000104591248952345949874143116176128387451171875f,0.20199999059736739592807452936540357768535614013671875f,-0.338699984002113207992579191341064870357513427734375f,0.10079998834133174767657692427746951580047607421875f,-0.72529997782111177428987502935342490673065185546875f,-0.095400021365284803920303602353669703006744384765625f,0.6912999832153321477790086646564304828643798828125f,-0.14340001512169830988341345801018178462982177734375f,-1.121000007927417385644730529747903347015380859375f,-1.2876000342309474877566799477790482342243194580078125f,-0.6940000202357765601846040226519107818603515625f,0.258600006842613527879848334123380482196807861328125f,0.55150000220537176431889747618697583675384521484375f,0.454999983012676079141556328977458178997039794921875f,0.82599997678399095235590721131302416324615478515625f,0.029499997451901460721046532853506505489349365234375f,-0.1186999777436259506657734164036810398101806640625f,0.737900020694732372561475131078623235225677490234375f,0.5062000035226343896965772728435695171356201171875f,0.23080000421106827701578367850743234157562255859375f,0.01569999896287921359316896996460855007171630859375f,0.061900008744001144833646321785636246204376220703125f,0.020200024354457557507203091518022119998931884765625f,-0.8775999992132186289239825782715342938899993896484375f,-0.751700006983428981044426109292544424533843994140625f,-0.1089000048547983379165771111729554831981658935546875f,-0.03050000439211719704957204157835803925991058349609375f,0.53709998763129096488455616054125130176544189453125f,0.79190000501871082860816386528313159942626953125f,0.75129999588131912702237968915142118930816650390625f,-0.10750001609325376961123765795491635799407958984375f,-0.243300030958652602208758253254927694797515869140625f,0.81870000456571556668450284632854163646697998046875f,0.352499989420175463550322092487476766109466552734375f,-0.24149998477101253246956957809743471443653106689453125f,-0.789399970149994079093858090345747768878936767578125f,0.59170004920959506922173432030831463634967803955078125f,0.20200000989437061349462965154089033603668212890625f,1.1762000193178649976033511848072521388530731201171875f,-0.4051999918699262792642912245355546474456787109375f,-0.1802999755114311764003787175170145928859710693359375f,0.10900000339746451771816282416693866252899169921875f,-0.1673000257879502239433122667833231389522552490234375f,-0.6105999897658822561652414151467382907867431640625f,0.2265000061169271106109590618871152400970458984375f,-0.863999980896711150535338674671947956085205078125f,-0.05669995179325315604046409134753048419952392578125f,0.44799998146295560985663541941903531551361083984375f,-0.8569000325858591171623857007944025099277496337890625f,0.0264000105917450156312042963691055774688720703125f,0.4217000087901947580348860356025397777557373046875f,0.2991999845385553413734669447876513004302978515625f,0.96489998128488652096024225102155469357967376708984375f,-0.08799998954683563301415460955468006432056427001953125f,0.008400023093819430908268941493588499724864959716796875f,0.35359997972250012399086926961899735033512115478515625f,-0.88639997317120400310841432656161487102508544921875f,0.07479997549653116806211983202956616878509521484375f,-0.9645999553516497737604140638723038136959075927734375f,0.775599983209371401926546241156756877899169921875f,0.289800010551512610135205250116996467113494873046875f,-0.77319997884631153084455945645458996295928955078125f,0.5846000305280092401716274252976290881633758544921875f,-0.355000014826655163613366994468378834426403045654296875f,-0.2142000053629278255584722501225769519805908203125f,0.1407999849885701593166231759823858737945556640625f,1.0797999892055987203320910339243710041046142578125f,0.23390000902712326791288433014415204524993896484375f,-0.616499970912933736855165989254601299762725830078125f,-0.2154999614953994058197395133902318775653839111328125f,-0.95959999107718463307037382037378847599029541015625f,-0.546799988445639595369129892787896096706390380859375f,-0.6533999896407127749853316345252096652984619140625f,0.06900000369548831002930455724708735942840576171875f,0.515799977165460887817971524782478809356689453125f,-0.37550000970065600114367043715901672840118408203125f,0.18950003209710075680050067603588104248046875f,-0.256799987104535276660755016564507968723773956298828125f,0.101999998942017555680905616100062616169452667236328125f,-0.30059999915361423905579840720747597515583038330078125f,-0.281799983900785644408415464567951858043670654296875f,-0.8368999847531319868920718363369815051555633544921875f,-0.27049996879696891483035869896411895751953125f,0.1678000413745639463058978435583412647247314453125f,0.519400029009580155303638093755580484867095947265625f,0.0385000056922433220307766532641835510730743408203125f,0.404399999095499396162267657928168773651123046875f,0.3262000098928810043474868507473729550838470458984375f,-0.5403000019609927040420416233246214687824249267578125f,0.22280000233352204030978782611782662570476531982421875f,0.4068999878451229346154605082119815051555633544921875f,0.86189998538643097791833724841126240789890289306640625f,0.515199982295930336562150841928087174892425537109375f,-0.1000000014156101624251959947287105023860931396484375f,0.3850000134482980396199991446337662637233734130859375f,-0.75019998993277514642841197201050817966461181640625f,-0.3707000112935900926203203198383562266826629638671875f,-0.5000000043213361777816317044198513031005859375f,-0.2055000078007578334648997042677365243434906005859375f,0.5199999827146530861909923260100185871124267578125f,-0.0946000100761654749703666311688721179962158203125f,0.757199967193603828974346470204181969165802001953125f,-0.01419999526739079698245404870249330997467041015625f,0.8433999877035613934594948659650981426239013671875f,1.051099979014694429935161679168231785297393798828125f,-0.0350000020861624960843982989899814128875732421875f,1.0200999919861553077993221450014971196651458740234375f,-0.00319999985694885413778365546022541821002960205078125f,0.97880001033544505872896479559130966663360595703125f,0.2397999787747864264275676760007627308368682861328125f,0.8218999713793397887684477609582245349884033203125f,-0.6213999964356418104216572828590869903564453125f,1.1369000200688841317742117098532617092132568359375f,0.15619998772740384396229273988865315914154052734375f,0.11779997387230378702582811456522904336452484130859375f,-0.403599995219707297078315377802937291562557220458984375f,-0.96970001588314769236376378103159368038177490234375f,-0.022199997889995604083424041164107620716094970703125f,-0.84760000789314482450009791136835701763629913330078125f,-0.249199979919195568900391890565515495836734771728515625f,-0.932099981614948003283416255726478993892669677734375f,-0.876699983793497228390378950280137360095977783203125f,-1.06119996269345318040677739190869033336639404296875f,-1.0620999915987254791360783201525919139385223388671875f,-0.040300001141428953843615090590901672840118408203125f,-0.1526000183612106209096737075014971196651458740234375f,0.70519997637271902846123339259065687656402587890625f,0.858699977892637189569313704851083457469940185546875f,1.179099982345104269398916585487313568592071533203125f,0.6476999826312066499411912445793859660625457763671875f,0.6406999716937544775419155484996736049652099609375f,-0.18700001741945726507765357382595539093017578125f,-0.129300003942847130389282028772868216037750244140625f,-0.82620001946687704474925340036861598491668701171875f,-0.96420000361204127869996227673254907131195068359375f,-0.896200011196732315710278271581046283245086669921875f,-0.52340001422762849614400693099014461040496826171875f,0.350700014087558020747792397742159664630889892578125f,-0.06099999868869776520341474679298698902130126953125f,-0.31230001975297927430119671043939888477325439453125f,-0.85500001668930014631087033194489777088165283203125f,-0.6129000069856640475762787900748662650585174560546875f,-0.160999994032084980943864138680510222911834716796875f,0.2852000002145766988093100735568441450595855712890625f,0.521099980169534848783996494603343307971954345703125f,0.2167999833792448871605529348016716539859771728515625f,0.9677000030457971746500334120355546474456787109375f,0.1677000005871054355566229787655174732208251953125f,-0.710499992489814768958922286401502788066864013671875f,-0.2989999835789201032554274206631816923618316650390625f,1.2094999690279359949585114009096287190914154052734375f,0.18419998707175277985470529529266059398651123046875f,1.697500006444752074230564176104962825775146484375f,-0.24660000469684562052208320892532356083393096923828125f,0.9129000358939178827455407372326590120792388916015625f,-0.844199961441755686308852091315202414989471435546875f,0.238300042991340443876424615154974162578582763671875f,-0.563199976760149212395845097489655017852783203125f,0.7490000093951818538329234797856770455837249755859375f,-0.18259998192787207926812698133289813995361328125f,1.2026000144124029400671815892565064132213592529296875f,0.04540000519752485086399929059552960097789764404296875f,0.7739000167012213271533482839004136621952056884765625f,-0.06359999998807897014785339706577360630035400390625f,0.411399990102648605017066074651665985584259033203125f,-0.63659998249411575699241438996978104114532470703125f,-0.2397999683067200127339901882805861532688140869140625f,0.671799983900785324664184372522868216037750244140625f,-0.743799997632205389663795358501374721527099609375f,0.49819998778700824715315320645458996295928955078125f,0.1239999916255476453130768277333118021488189697265625f,-0.22039998500645185952606652790564112365245819091796875f,0.2410999992057675511691883230014354921877384185791015625f,0.243799990963935897525516338646411895751953125f,0.4518000130325561247701671163667924702167510986328125f,-0.66439997625947011528069197083823382854461669921875f,0.6364000238165254952349414452328346669673919677734375f,0.56779997320175201735992231988348066806793212890625f,-0.465200042980909334033867708058096468448638916015625f,0.12470000489354127193308841015095822513103485107421875f,0.12820000343322757174746584496460855007171630859375f,-0.2015000156164171996042711043628514744341373443603515625f,0.395400002235919145743281433169613592326641082763671875f,0.0868999920174478857060051950611523352563381195068359375f,0.121499992072582330138175166212022304534912109375f,0.291199993836879666986305892351083457469940185546875f,1.0099000068962571408093253921833820641040802001953125f,0.55570000432729715900137534845271147787570953369140625f,0.60769999887347214695410002605058252811431884765625f,1.1260000075548888265331015645642764866352081298828125f,0.784799973075092083263371023349463939666748046875f,-0.19369999890327438407666704733856022357940673828125f,-0.596399981012940560987090066191740334033966064453125f,-0.0259999989867212288885411908267997205257415771484375f,0.89279999153017985946689805132336914539337158203125f,0.56260000822842126666500917053781449794769287109375f,-0.335899988299608320829747754032723605632781982421875f,-0.2138999986708165490512101314379833638668060302734375f,0.497000000730156710204710179823450744152069091796875f,0.5569000231981278137283197793294675648212432861328125f,-0.427299997103214312943464392446912825107574462890625f,0.955500027805566976013551538926549255847930908203125f,0.85839999634623520563536658301018178462982177734375f,0.747899987763166240739565182593651115894317626953125f,0.6852999928712844024403239018283784389495849609375f,0.306700012478232419965706867515109479427337646484375f,-1.0083999842762947363183911875239573419094085693359375f,-0.69439999645054351073980569708510302007198333740234375f,1.1238000348851089338353403945802710950374603271484375f,0.224600005978345507173798978328704833984375f,0.94420002359822385873400207856320776045322418212890625f,-0.00860000114291903372532033245079219341278076171875f,0.3407000106044113518777294302708469331264495849609375f,0.0037999984145165033311286606476642191410064697265625f,0.3244000200688843538188166348845697939395904541015625f,-0.407199990290403324166845777654089033603668212890625f,-0.086400008766352787148434799746610224246978759765625f,-0.71259996345043230547844359534792602062225341796875f,-0.572700015152991515066105421283282339572906494140625f,0.3138000026613469994885008418350480496883392333984375f,0.4747999521762125141322030685842037200927734375f,0.19119999473094939634876254785922355949878692626953125f,-0.0204000157773498447255633436725474894046783447265625f,-0.353600007438659158509608460008166730403900146484375f,-0.74629999163448756949179596631438471376895904541015625f,0.0519999981224537055624068671022541821002960205078125f,0.1102999693647024326281780304270796477794647216796875f,-0.83859998434185956117659088704385794699192047119140625f,-1.375500044159591706005585365346632897853851318359375f,-0.28639999807476979043485698639415204524993896484375f,-1.1885000148937105990398777066729962825775146484375f,0.33019999350905404611467020004056394100189208984375f,-1.3379000304177406377448278362862765789031982421875f,-0.3639999993741509509703746516606770455837249755859375f,-1.0477000042006370250646796193905174732208251953125f,0.266399992263317120233523382921703159809112548828125f,0.479100003653764705546791446977294981479644775390625f,-0.403399994930624838929134057252667844295501708984375f,-0.5261999996483324171236972688348032534122467041015625f,-0.57100002416968342888736742679611779749393463134765625f,0.58049997892975824242967064492404460906982421875f,0.04569999277889724709211805020458996295928955078125f,-0.181399990773201125193736515939235687255859375f,0.7088999997138980546651509939692914485931396484375f,0.9548999511659150840614529442973434925079345703125f,0.54529998743236074432161331060342490673065185546875f,-0.87359998553395268316990041057579219341278076171875f,-0.749900001308322128323879951494745910167694091796875f,-0.226099991494417285053941668593324720859527587890625f,-0.958799993385374360599371357238851487636566162109375f,-0.45659999167695641819619822854292578995227813720703125f,-0.8325000040046874882904148762463591992855072021484375f,-0.2404999769926075003922960604541003704071044921875f,0.672000010341406106562089917133562266826629638671875f,0.8359999583065513828472603563568554818630218505859375f,0.26689996031522766894994447284261696040630340576171875f,0.10499998062849069668800439103506505489349365234375f,0.472599990347027709702842912520281970500946044921875f,0.078899998819828009999355344916693866252899169921875f,0.0182999906510115106783587179961614310741424560546875f,0.13899998924136180988853084272705018520355224609375f,-0.260500022888183746516688188421539962291717529296875f,-0.0301999979421496700382476774393580853939056396484375f,-0.02339999787360425020921184113831259310245513916015625f,0.2988000039279459230101565481163561344146728515625f,1.28100000204145914040054776705801486968994140625f,0.37800000232458141891100922293844632804393768310546875f,0.467600000107287971928826664225198328495025634765625f,1.00749999776482557223289404646493494510650634765625f,-0.48799997863173505407985430792905390262603759765625f,0.1882999955311415174463718358310870826244354248046875f,-0.481799991947412298287645171512849628925323486328125f,-0.51009998874515261046980185710708610713481903076171875f,-0.1031999734818942471292757545597851276397705078125f,-1.081899992017448131065293637220747768878936767578125f,0.870199982482194656085994211025536060333251953125f,0.6424999866262073311418134835548698902130126953125f,0.3445999804973605140645531719201244413852691650390625f,1.797699996321648452379804439260624349117279052734375f,0.1150000014901160216851394579862244427204132080078125f,0.68989998743161518035549306659959256649017333984375f,-0.6938000040769569665144445025362074375152587890625f,-0.452899988061189251453697579563595354557037353515625f,-0.284199984389543747198558776290155947208404541015625f,-0.2213999738231298142210334845003671944141387939453125f,-0.7227999743938446908231298948521725833415985107421875f,-0.493299962636828237538821895213914103806018829345703125f,-0.0036000034153460536145985315670259296894073486328125f,0.8051000411972406567429061396978795528411865234375f,0.2269999978244303662222591810859739780426025390625f,0.72560004117488874353369965319870971143245697021484375f,0.363199980783462772482295122244977392256259918212890625f,-0.4854000371977686967284171259962022304534912109375f,-0.55239998737573614562279544770717620849609375f,-1.1305999928951262489817963796667754650115966796875f,1.06199998493492575590835258481092751026153564453125f,0.397599993848800625784178919275291264057159423828125f,0.2238999986708167799775992534705437719821929931640625f,0.39419998051524185456173654529266059398651123046875f,0.3145999875381588761769080520025454461574554443359375f,0.42449998964741830764779706441913731396198272705078125f,0.182500011082738555767690513675916008651256561279296875f,-0.5484999836385251459347500713192857801914215087890625f,-0.31709997349381435771675796786439605057239532470703125f,-0.55239998439550408448894813773222267627716064453125f,0.3795000090003011194994542165659368038177490234375f,0.266999995440244664024476151098497211933135986328125f,0.593400003722309943299251244752667844295501708984375f,0.4088000079885123483336428762413561344146728515625f,0.563599995890259553021905958303250372409820556640625f,0.13889999881982806329006052692420780658721923828125f,0.180999989151954532307087220033281482756137847900390625f,0.304700005228817349944847592269070446491241455078125f,0.1581000063806772271135514529305510222911834716796875f,-0.314899998253583834895152904209680855274200439453125f,-0.1975000008940697160397803600062616169452667236328125f,0.67860000222325300711645468254573643207550048828125f,0.82890002065002921227687693317420780658721923828125f,0.11679999567270282767594835604541003704071044921875f,0.60840000729858900729851711730589158833026885986328125f,0.8557999960154294427638888009823858737945556640625f,-0.022300027427077484531992013216949999332427978515625f,-0.05190001306533797986730860429815948009490966796875f,-0.1940000127106902283458111924119293689727783203125f,-1.5140999928504228488890248627285473048686981201171875f,-0.20220000012516958776132014463655650615692138671875f,-0.05820001934021679090847101178951561450958251953125f,-0.6987999575853354183863075377303175628185272216796875f,0.0658999940551814322731161155388690531253814697265625f,-0.71439997164011026509200519285514019429683685302734375f,0.0121999985046681302947035874240100383758544921875f,0.08279999451041231139925002935342490673065185546875f,-0.73930003475099781429236145413597114384174346923828125f,-0.02779998705983199869251620839349925518035888671875f,-1.0882000175520782914162509769084863364696502685546875f,0.047199992525577716406814943184144794940948486328125f,-0.73539998824745389516266413920675404369831085205078125f,0.2767999847203495011882523613166995346546173095703125f,-0.92889997143894387221507713547907769680023193359375f,0.54539997986555110998097006813623011112213134765625f,-0.551399987308680561426399435731582343578338623046875f,0.0107999978035688837252337179961614310741424560546875f,-0.25439999630153165011137161855003796517848968505859375f,0.4387999940931794906617824381100945174694061279296875f,0.09609994451850578656149082235060632228851318359375f,-0.748599985235929299420831739553250372409820556640625f,-0.28540000068992410486856670104316435754299163818359375f,0.5923999742627146236628732367535121738910675048828125f,-0.8080000216215845743050749661051668226718902587890625f,0.365799999368190409398948759189806878566741943359375f,-0.2323000177040699387731592651107348501682281494140625f,-0.6697999851822851180571660734131000936031341552734375f,-0.158199974972009105300685405381955206394195556640625f,-0.6353000380963089721575443036272190511226654052734375f,-0.03290000311136243027476666611619293689727783203125f,-0.330000007152557284229033029987476766109466552734375f,-0.272499996721744519589947230997495353221893310546875f,0.0235999907493587013362912330194376409053802490234375f,-0.024899995198846003319204100989736616611480712890625f,0.4428000220030547939842335836146958172321319580078125f,0.3342000055491922694272943772375583648681640625f,0.29390001789331432480167904941481538116931915283203125f,-0.9202999967269598347030523655121214687824249267578125f,0.0664999948665501161571000920957885682582855224609375f,0.8241999737352134758339161635376513004302978515625f,0.9432999859571460365970096972887404263019561767578125f,1.04179996825456644415908158407546579837799072265625f,-0.119600015142559801262223118101246654987335205078125f,-1.049800000679492750776944376411847770214080810546875f,-0.3902999870598315990122273433371447026729583740234375f,0.10490001851916286312871307018212974071502685546875f,0.19290000467598422684289971584803424775600433349609375f,1.0782000024273987737188917890307493507862091064453125f,0.64020002025663858002957340431748889386653900146484375f,-0.5627999939143657481821492183371447026729583740234375f,0.081300002288818173923345966613851487636566162109375f,-0.20549999301135546403429543715901672840118408203125f,-0.208799996104836349530842198873870074748992919921875f,-0.08169999571442598806925161625258624553680419921875f,-0.80100000979006313883701295708306133747100830078125f,-0.663399988001585061425657841027714312076568603515625f,-0.6314000061959028631264345676754601299762725830078125f,0.70950001447647792485184936595032922923564910888671875f,-0.2150000041723247878877600669511593878269195556640625f,0.0614000255674131967253970287856645882129669189453125f,-0.5429999755024912388989832834340631961822509765625f,-0.083799973380565884184534297673963010311126708984375f,-0.48420000510215699396354693817556835711002349853515625f,-0.81630001234710169999431172982440330088138580322265625f,-0.2957999875962735725209995507611893117427825927734375f,-1.3913000395976009571086251526139676570892333984375f,0.18159999743252985826558187909540720283985137939453125f,-0.0201000050060453627764900375041179358959197998046875f,-0.106999996185302759243995751603506505489349365234375f,-0.3196000095173718857921585367876105010509490966796875f,0.140800003838538856371087604202330112457275390625f,0.0613999798953528408418378603528253734111785888671875f,0.3709999907910823679912937222979962825775146484375f,0.6511000053897497696908658326719887554645538330078125f,0.75239996666312247697305792826227843761444091796875f,-0.4811000569850210073497009943821467459201812744140625f,-0.63319999933540771497320065464009530842304229736328125f,-1.0000000130757686900295766463386826217174530029296875f,0.080599998110532755646318037179298698902130126953125f,-0.9228999826595185584920955079724080860614776611328125f,0.300599981719255726630990466219373047351837158203125f,0.5612000281840570092839470817125402390956878662109375f,0.307800005388259467764555665780790150165557861328125f,1.071100005576014257968608944793231785297393798828125f,0.20519998948574080799289731658063828945159912109375f,1.4594000058382743389984170789830386638641357421875f,0.943699994283914378456756821833550930023193359375f,0.186299992714822337802615948021411895751953125f,0.55000000342726718116637130151502788066864013671875f,0.35900000324845304788823341368697583675384521484375f,0.437799994882940968210505161550827324390411376953125f,-0.03649999952316296258914007921703159809112548828125f,0.436899999803304961432104391860775649547576904296875f,0.77259998580217370545142330229282379150390625f,0.389299987030029104317918609012849628925323486328125f,-0.45890000299215305545885712490417063236236572265625f,-0.0615999991834161875203790259547531604766845703125f,0.28719999803900719825833220966160297393798828125f,0.0011999973386525031315841260948218405246734619140625f,-0.15549999330937869235214066065964289009571075439453125f,-1.007000006839632977317933182348497211933135986328125f,-1.2139999916255472811599247506819665431976318359375f,-0.905299999651312692350302313570864498615264892578125f,0.2700000014901160483304920489899814128875732421875f,0.3691000040262937265111986562260426580905914306640625f,0.33799998898804195501810454516089521348476409912109375f,0.275899999326467526561401655271765775978565216064453125f,0.042200004260241896769656477772514335811138153076171875f,-0.28939998750984663455909640106256119906902313232421875f,-0.3579999881684781115609439439140260219573974609375f,-0.1125999857276678284279114450328052043914794921875f,0.443400003349780913453059838502667844295501708984375f,-0.3952000104963782067812871900969184935092926025390625f,-0.2863999855577947784013304044492542743682861328125f,0.4226000180631874769687783555127680301666259765625f,-1.14489997500777196393073609215207397937774658203125f,0.2697999800413850568503448812407441437244415283203125f,-0.868099995242058941613549905014224350452423095703125f,0.438799986195564262203561156638897955417633056640625f,-1.7287000402539973720195121131837368011474609375f,0.0091999736607082382988664903677999973297119140625f,-0.73509998926669251062548937625251710414886474609375f,-0.73179996959567095160537064657546579837799072265625f,0.0675000412389634707466257168562151491641998291015625f,0.39619998430013669832305822637863457202911376953125f,-0.0562999980419871537407061623525805771350860595703125f,0.215199998426437222320828368538059294223785400390625f,0.02549999442696548879183637836831621825695037841796875f,0.2149999897181988917083117485162802040576934814453125f,1.125700023773312796748768960242159664630889892578125f,0.1669999933540822212307830341160297393798828125f,1.296600007863343062552985429647378623485565185546875f,-0.4585999774873259493546129306196235120296478271484375f,0.003799987052380549013008703695959411561489105224609375f,-0.1197999956130981757951303734444081783294677734375f,-0.54260001232624066602738821529783308506011962890625f,-0.18079999750554553994419393347925506532192230224609375f,0.73459998887926325483732625798438675701618194580078125f,-0.077999996364116697122881305404007434844970703125f,0.9429999969601627896054196753539144992828369140625f,0.273999988198280419737784541212022304534912109375f,0.2497999911427497554683441194356419146060943603515625f,-0.623199998068808991291689380886964499950408935546875f,-0.13639998667538144605515526563976891338825225830078125f,0.106700002121925319187312197755090892314910888671875f,0.5111999913036824327861040728748776018619537353515625f,-0.20930002949833859116068879302474670112133026123046875f,0.4153999742031100339545446331612765789031982421875f,-0.21770001288056395338799120509065687656402587890625f,0.24560002418756443631764341262169182300567626953125f,-0.7670999816894530365374293978675268590450286865234375f,-0.9818999837100506056231097318232059478759765625f,-0.7317999968647956876566240680404007434844970703125f,-0.2639000110387801356637282879091799259185791015625f,0.9656000232934953686481094337068498134613037109375f,1.2716000020146367432971601374447345733642578125f,0.408799997222423439069416417623870074748992919921875f,-0.927700006324052584005812605028040707111358642578125f,-0.3360999978274101795960859817569144070148468017578125f,-0.3277000174999233905737128225155174732208251953125f,0.1197999963581561910785922009381465613842010498046875f,-0.278999998927116354963118283194489777088165283203125f,-0.6101000080049037155305313717690296471118927001953125f,-0.26700000631809217566114966757595539093017578125f,-0.33130002009570613807909467141143977642059326171875f,0.7690999950110910532430352759547531604766845703125f,0.656100009748339818571594150853343307971954345703125f,0.7196999935686587690497617586515843868255615234375f,0.684099990317225348235297133214771747589111328125f,0.30159998890161521245545372948981821537017822265625f,0.57820000626444834068706768448464572429656982421875f,-0.47740000212788569200483834720216691493988037109375f,-0.370599996396899111772427204414270818233489990234375f,0.8986999977111818083130856393836438655853271484375f,-1.56819999948441957116074263467453420162200927734375f,-0.00509997637718928586991751217283308506011962890625f,0.16939999022930873895376180371385999023914337158203125f,-0.27810005065575349458839582439395599067211151123046875f,-0.14999998316168738909937019343487918376922607421875f,0.0270000348538150802824020502157509326934814453125f,0.6721999625742440809261779577354900538921356201171875f,0.6008000183299182506146962623461149632930755615234375f,-0.335400015777349125301043386571109294891357421875f,0.873500049129128708358393851085565984249114990234375f,-0.36480002414882173322752123567624948918819427490234375f,-0.140099987962841954214354700525291264057159423828125f,0.907000008180737449237085456843487918376922607421875f,0.7168999966740603468906556372530758380889892578125f,-0.369799989578127874523261198191903531551361083984375f,-0.468499996602535162537606083787977695465087890625f,0.9368000009626147317476352327503263950347900390625f,-0.5838000005006787063166484585963189601898193359375f,-0.332799996820092314209205142105929553508758544921875f,0.144700010555983027416004915721714496612548828125f,0.389999998435377914773880547727458178997039794921875f,0.10610000090077509327102234237827360630035400390625f,-0.24119999525249002925164631960797123610973358154296875f,1.133200037109852065242421303992159664630889892578125f,-0.81100002469122400849954601653735153377056121826171875f,1.215300061919540208776879808283410966396331787109375f,0.17699998486042023415620860760100185871124267578125f,0.527099962615967143619855050928890705108642578125f,0.495200005057454095691582551808096468448638916015625f,0.2429000010997056924200165894944802857935428619384765625f,1.3750000385195015883965652392362244427204132080078125f,0.1129000052347779448069786667474545538425445556640625f,-0.529299971532820823227893924922682344913482666015625f,-0.252899998678266957341520537738688290119171142578125f,-1.21270002562105627674782226677052676677703857421875f,0.0005000002384185631143509453977458178997039794921875f,0.0706000025808812825545146552030928432941436767578125f,0.08710000132173301157223477275692857801914215087890625f,0.449300028082728619693853033822961151599884033203125f,0.180700011554360440158006895217113196849822998046875f,0.67760003214478548017041248385794460773468017578125f,0.13829999851137397826050801086239516735076904296875f,-0.865700023549795805166695572552271187305450439453125f,-0.2696000052332878649252734248875640332698822021484375f,-1.615200036424398799539403626113198697566986083984375f,-0.11420000018477438619157737775822170078754425048828125f,-1.4026000115066761253501681494526565074920654296875f,-0.2766999957889317851567057005013339221477508544921875f,-0.2624000177741050432445035767159424722194671630859375f,0.14059999542832368746303473017178475856781005859375f,0.58500000879168478906677819395554251968860626220703125f,-0.092799997565150282863299935343093238770961761474609375f,-0.061099985012412094675937623833306133747100830078125f,0.21489999534785741985842832946218550205230712890625f,1.19980000477731163499583999509923160076141357421875f,0.143899996435642218983730344916693866252899169921875f,0.43259999712705610175333958977716974914073944091796875f,0.471200007396936459969083443866111338138580322265625f,-0.779399990043044166299068820080719888210296630859375f,1.518600014069675818717541915248148143291473388671875f,-1.3594000173866742642303506727330386638641357421875f,-0.2482999779105179438687400761409662663936614990234375f,1.0431999946415422275691753384307958185672760009765625f,-1.644500013917684189124202021048404276371002197265625f,1.143399999549984613622655160725116729736328125f,-0.9472000104069706605258716081152670085430145263671875f,0.58300000411272012712515788734890520572662353515625f,0.424600032949447925290087368921376764774322509765625f,0.050599972704052920136064130929298698902130126953125f,1.663200011777878017937837284989655017852783203125f,-1.10079996971488025536700661177746951580047607421875f,0.1643000019311902093477328890003263950347900390625f,1.209399999207258158406830261810682713985443115234375f,-1.060700011479854598661631825962103903293609619140625f,0.78079999415278422691955029222299344837665557861328125f,-1.698799994242190525284286195528693497180938720703125f,1.802599980363249532189229284995235502719879150390625f,-0.0605000178217895978605156415142118930816650390625f,-1.0181999968767161401927978658932261168956756591796875f,0.89059999121874522387543038348667323589324951171875f,-0.39020000360459061727880225589615292847156524658203125f,0.24500000769272378686736146846669726073741912841796875f,-0.19259999809563144168578219250775873661041259765625f,0.398299996984004422984071425162255764007568359375f,-0.72139999680817101790353262913413345813751220703125f,0.204299989414215676930552945123054087162017822265625f,1.16560000444352596105090924538671970367431640625f,0.1855999988183378857087291180505417287349700927734375f,0.5989000234812493772551533766090869903564453125f,0.23600000319629910183749643692863173782825469970703125f,-0.868500032439828206776155639090575277805328369140625f,-0.1358000091657043384429925936274230480194091796875f,0.209499958261846419560470167198218405246734619140625f,0.1929999969229101441925422477652318775653839111328125f,0.9359000075221057812768776784650981426239013671875f,0.2041000057771802378425718416110612452030181884765625f,-0.370099981033801928020920968265272676944732666015625f,-0.3379000046387314881712882197462022304534912109375f,-0.894100042918324788843165151774883270263671875f,0.14969999848604198522394881365471519529819488525390625f,-1.000800010171533305225466392585076391696929931640625f,-0.3263000075042248315781989731476642191410064697265625f,-2.060600039535760874542802412179298698902130126953125f,-0.3972999939367174437876428783056326210498809814453125f,-0.998499981924890533946381765417754650115966796875f,0.080000002793967739389557891627191565930843353271484375f,-0.23299998235702490401166642186581157147884368896484375f,0.043600000621378420806451003954862244427204132080078125f,0.733799990665912726939268395653925836086273193359375f,0.223500000588595870709696100675500929355621337890625f,1.6027000080376858637265513607417233288288116455078125f,0.21599999790638679808552069516736082732677459716796875f,-0.57949998702108873604998962036916054785251617431640625f,-0.2917999960638582024330389685928821563720703125f,-1.180700015447288198089381694444455206394195556640625f,1.940299996000528270911900108330883085727691650390625f,-1.3213999689430000472611936856992542743682861328125f,1.80470005958080292884915252216160297393798828125f,-0.3366000143080949413842972717247903347015380859375f,-0.956900015449523522192976088263094425201416015625f,1.542000031873583143493533498258329927921295166015625f,-0.1154000364899641084548420622013509273529052734375f,-0.056599980631470980796393632772378623485565185546875f,1.572899986124038917978396057151257991790771484375f,-1.607599975371361011866611079312860965728759765625f,-0.0425999759674073885662437533028423786163330078125f,1.346999979496002364243167903623543679714202880859375f,-0.03759999384880075012915767729282379150390625f,-0.152399994304776242870502755977213382720947265625f,-0.6985999791264543290481014992110431194305419921875f,1.785199966463447029951794320368207991123199462890625f,-0.9287000102281570690365697373636066913604736328125f,0.387599972242117285503582024830393493175506591796875f,-0.25829996211528782890098909774678759276866912841796875f,0.2363999719977381841484742608372471295297145843505859375f,-0.5355999626085161668953560365480370819568634033203125f,0.91179997730702166958138832342228852212429046630859375f,0.51299997425451937527185464205103926360607147216796875f,-1.26179995558857971360566807561554014682769775390625f,1.63469998626708967748299983213655650615692138671875f,-1.0021999695032837163211070219404064118862152099609375f,-0.439099997711926537391491365269757807254791259765625f,0.87479997765719896829494928169879131019115447998046875f,-0.94989998400434816350212940960773266851902008056640625f,1.0291999812602987685039579446311108767986297607421875f,0.3673999922186135247414995319559238851070404052734375f,0.0037999730825413191581674254848621785640716552734375f,-0.3239000086173415837720312993042171001434326171875f,-1.0753000427901751034909239024273119866847991943359375f,-0.085600003549456626927849356434307992458343505859375f,0.277599990198015955655819198000244796276092529296875f,0.1893000017821788727445664335391484200954437255859375f,0.55460002892613413649769427138380706310272216796875f,-0.05070000019222498099935592108522541821002960205078125f,-0.793500007629394321639892950770445168018341064453125f,-0.0381999978452920441185369782033376395702362060546875f,0.0485999923884870010937220285995863378047943115234375f,-0.08120000091493129179553989160922355949878692626953125f,-0.14680000438988205502255368628539144992828369140625f,0.120200001294910929772186136688105762004852294921875f,-0.33350000308454064157359653108869679272174835205078125f,0.138299999368190762449870589989586733281612396240234375f,0.606299994912743667185850426903925836086273193359375f,-0.0979000009506940660486407068674452602863311767578125f,-0.1366000128179789552262946017435751855373382568359375f,-0.3308000055894255719834973206161521375179290771484375f,-1.200700033533573130029026287957094609737396240234375f,-0.076399999788403505363021395169198513031005859375f,-0.6036000279277564661839505788520909845829010009765625f,-0.1258000017896294675523449768661521375179290771484375f,0.27850001098215582118200472905300557613372802734375f,0.222500000298023248301859666753443889319896697998046875f,1.582200027692318489158651573234237730503082275390625f,0.08210000553131013845131747075356543064117431640625f,-0.2903999992370602800662027220823802053928375244140625f,-0.71070003025531836016170927905477583408355712890625f,1.0088000052690506702646189296501688659191131591796875f,-0.69570001296997130424415445304475724697113037109375f,0.4595999910771848551149787454050965607166290283203125f,0.49639998868703827383797033689916133880615234375f,-1.059199988785386015166523066000081598758697509765625f,0.433700022447109034828827134333550930023193359375f,0.41039998567700397558155600563623011112213134765625f,-0.5386999860882752955149044282734394073486328125f,0.493999997138976798538578805164434015750885009765625f,0.03139999509453783588242004043422639369964599609375f,-0.2335999973058699996641962570720352232456207275390625f,0.624900038114190703453232345054857432842254638671875f,0.203200002166628479471910395659506320953369140625f,-0.48279997539967300923535731271840631961822509765625f,0.48259998494535694124607516641844995319843292236328125f,-0.626200042358785680107757798396050930023193359375f,-0.4917999778658153076094095013104379177093505859375f,-0.29060002686977437491577802575193345546722412109375f,-0.0138000047475095311000359288300387561321258544921875f,-0.50659997936487144443162833340466022491455078125f,1.052599995338916460951850240235216915607452392578125f,-0.1404000354468823985598646686412394046783447265625f,-0.5251999846428632157113725043018348515033721923828125f,0.008300021868944185143845970742404460906982421875f,-0.20980000954568378546127860317938029766082763671875f,1.38940002304911747188498338800854980945587158203125f,-0.07910001430809499023411035523167811334133148193359375f,-0.076799999211728586789860173666966147720813751220703125f,-0.120999988742172492894866309143253602087497711181640625f,0.0982000018872320745710879918988212011754512786865234375f,0.100099998356402153376620844937860965728759765625f,-0.1102999989435076155785964147071354091167449951171875f,0.270900006404519277936060461797751486301422119140625f,-0.0456999992609024019429853069595992565155029296875f,0.108799997445940821449994473368860781192779541015625f,0.16899999985843894290127309432136826217174530029296875f,-0.21089998323321379469774683457217179238796234130859375f,0.076500000230967979408802648322307504713535308837890625f,-0.1000000000745056016882017502211965620517730712890625f,0.0131999976590275736043622600845992565155029296875f,0.51990002053081951771673629991710186004638671875f,0.33850000591576101527380160405300557613372802734375f,1.368000045239925466233898987411521375179290771484375f,0.3799999973922967289041707772412337362766265869140625f,0.6753999880611889494730348815210163593292236328125f,-0.1008000080853701518890375155024230480194091796875f,0.562799995926022145198430735035799443721771240234375f,0.25780000095516442115695099346339702606201171875f,1.34910002369582660008973107323981821537017822265625f,0.0522000072404742443410441410378552973270416259765625f,-0.306899968063830907993860819260589778423309326171875f,0.0260000015199184364433904192992486059665679931640625f,-0.381600001269578825713324476964771747589111328125f,0.0566000026106834308592397064785473048686981201171875f,0.188400036430359296701908533577807247638702392578125f,0.317999977327882987765406141988933086395263671875f,-0.61179999474138002657497281688847579061985015869140625f,0.3740000116489829995458649136708118021488189697265625f,0.085400005942582790652295443578623235225677490234375f,-0.22490003420263471323181647676392458379268646240234375f,-0.587399977391958483252665246254764497280120849609375f,0.175800018590688722497361595742404460906982421875f,0.317599983343482161757265203050337731838226318359375f,1.367500026449560923680337509722448885440826416015625f,-0.803199995461106208693990993197076022624969482421875f,0.18350003426521954708050543558783829212188720703125f,0.44979997046738906529839141512638889253139495849609375f,-0.99779999769553484600947967919637449085712432861328125f,0.9504000127226108762812373242923058569431304931640625f,-0.1440999531388289511824041255749762058258056640625f,-0.033200052830576254336847341619431972503662109375f,0.5420000480413431631632192875258624553680419921875f,-0.332600011432170550307318990235216915607452392578125f,-0.42570005528926913029863499104976654052734375f,-0.0899999700486660003662109375f,-0.7343000242829322843363115680404007434844970703125f,0.260200010719894958555187258752994239330291748046875f,0.7186999756574632414185543893836438655853271484375f,-0.75819998920261877373150127823464572429656982421875f,0.3390000212788581990253078402020037174224853515625f,0.615999992430209974969557151780463755130767822265625f,-0.66200001794099794238945833058096468448638916015625f,0.36040000579357123200452406308613717555999755859375f,-0.605300018724799659963764497661031782627105712890625f,1.66150007268786481517963693477213382720947265625f,0.43250000294297930025777532136999070644378662109375f,1.65970004438161833348885920713655650615692138671875f,0.24389999572783704895329037753981538116931915283203125f,-0.1239000042587521388526283772080205380916595458984375f,-0.302499998807907066744604662744677625596523284912109375f,-0.91820001580119114947819980443455278873443603515625f,-0.02170000290423634847769562838948331773281097412109375f,-0.0551000075578691461686275943065993487834930419921875f,0.167500000633299339636295144373434595763683319091796875f,1.0573999973595136214754575121332891285419464111328125f,0.349699995394050999752977304524392820894718170166015625f,1.033400028644501755792362018837593495845794677734375f,0.129300003793835660559352618292905390262603759765625f,-0.487799997565149912048809710540808737277984619140625f,-0.17059999904185529917555186329991556704044342041015625f,0.078499986544251942888195117120631039142608642578125f,-0.240200000549852799025529748178087174892425537109375f,-0.125899985989927909457719579222612082958221435546875f,0.310800010432303042140489424127736128866672515869140625f,1.173300082665682975147092292900197207927703857421875f,0.177600012624263836613636158290319144725799560546875f,0.5882000382274394922887950087897479534149169921875f,0.2581999975845217232262029938283376395702362060546875f,-0.0214999818652874541413666520384140312671661376953125f,0.08850000025331972164366334254737012088298797607421875f,-0.0994000094890594709795550443232059478759765625f,-0.107700007106363793152326024937792681157588958740234375f,0.747100003370642529176848256611265242099761962890625f,0.7308000354290005162738452781923115253448486328125f,-0.455200013178586715412166086025536060333251953125f,0.117200011599063458334057941101491451263427734375f,-0.10619999651908873516958919935859739780426025390625f,-1.02820000954270351911645775544457137584686279296875f,0.9517999827831984394066466848016716539859771728515625f,-1.0927999980866911489130188783747144043445587158203125f,0.7043999872863293543190366108319722115993499755859375f,-0.42649998894333818189039675417006947100162506103515625f,0.49060000176131690619030223388108424842357635498046875f,0.6190999860331409809788283382658846676349639892578125f,-0.8327999775975942764461024125921539962291717529296875f,0.5438000228151678161481186180026270449161529541015625f,0.27739998700320711577660404145717620849609375f,-0.9068000204086299476102794869802892208099365234375f,0.324400005093216758922380904550664126873016357421875f,-0.02140003249645161531589110381901264190673828125f,-0.458799974200129856427565755438990890979766845703125f,0.5016000519335275242838179110549390316009521484375f,-0.501600031742453378313939538202248513698577880859375f,0.55130007992386875770307597122155129909515380859375f,0.35759995212554951393713054130785167217254638671875f,1.13990003803968420470482669770717620849609375f,-0.5953999909669160928160636103712022304534912109375f,-0.181599958279729634114119107834994792938232421875f,0.8887999586284165420835279292077757418155670166015625f,-0.9547000163301826436423880295478738844394683837890625f,0.20680001236200329373104978003539144992828369140625f,-0.00309996681809376894989327411167323589324951171875f,0.10969998969435668101368719362653791904449462890625f,0.204900001122057362490380683084367774426937103271484375f,-0.071499967262149244362490208004601299762725830078125f,0.0319000012934207433090705308131873607635498046875f,-0.59359999365508553381687306682579219341278076171875f,-0.115400002701580517427970562494010664522647857666015625f,-0.774500025093555422017743694595992565155029296875f,-0.1340999957934021491468001840985380113124847412109375f,-0.46869998131990353584797048824839293956756591796875f,-0.145399994841217961027979299615253694355487823486328125f,-0.5412000046402212216634097785572521388530731201171875f,-0.1899000092059374422692741291029960848391056060791015625f,-0.1420000240504746269465385921648703515529632568359375f,0.05450000542402266834329793709912337362766265869140625f,0.210800016131997391966024224529974162578582763671875f,0.148199998851120451348606366082094609737396240234375f,0.2673000260114672954614434274844825267791748046875f,-0.1223999949008225396340776569559238851070404052734375f,-0.16420001448988852388310988317243754863739013671875f,-0.06360000363886353369480275432579219341278076171875f,1.38359997279345936505023928475566208362579345703125f,0.370499994874000482258935562640544958412647247314453125f,1.2918000148952000216695523704402148723602294921875f,0.11100000159442424685352079904987476766109466552734375f,-0.52860000140368956778047504485584795475006103515625f,-0.091600001977384104545620857606991194188594818115234375f,-0.14660001661777499748495756648480892181396484375f,0.047799996708333514217059700968093238770961761474609375f,0.11949999007582601251442611101083457469940185546875f,1.065400007358193068540685999323613941669464111328125f,-0.52339998978972435139667140902020037174224853515625f,0.67080001799464117340221491758711636066436767578125f,0.4949999745935205197611139738000929355621337890625f,0.19380001555085168973846521112136542797088623046875f,0.2943999986857175343857306870631873607635498046875f,-0.083400013557076047021610065712593495845794677734375f,-0.331799997907876953462391611537896096706390380859375f,-0.895600017780065371653108741156756877899169921875f,0.38079998595714581455240477225743234157562255859375f,-1.2744000322133306468685987056232988834381103515625f,0.7102000107198953582354761238093487918376922607421875f,-0.066500008538365573684814080479554831981658935546875f,0.048199988904595425509569395217113196849822998046875f,0.29399997299909585990462801419198513031005859375f,-0.798599971601367197848730938858352601528167724609375f,-0.99379999826550413644099535304121673107147216796875f,0.8077999842286107679001361248083412647247314453125f,-1.24070001714229505296316347084939479827880859375f,0.05600001671910259659625808126293122768402099609375f,0.5405999799311160813886090181767940521240234375f,-1.19399996472895164600913631147705018520355224609375f,0.92200001570582390542085704510100185871124267578125f,0.095200000214576530055410330533050000667572021484375f,-0.26519999067783384116836487010004930198192596435546875f,0.60099998766183870824164614532492123544216156005859375f,-0.3668000046879049680370599162415601313114166259765625f,-0.26519998434484026716972948634065687656402587890625f,-0.97469996853470863840129823074676096439361572265625f,0.762499972879886467325150078977458178997039794921875f,0.01610000322908167191826578346081078052520751953125f,-1.168300025743246095544236595742404460906982421875f,-0.297000004976987863614112939103506505489349365234375f,-1.67960008018612949598491468350403010845184326171875f,-0.336200012761354527501822531121433712542057037353515625f,-0.7252000471532351610903788241557776927947998046875f,0.17250000599771742049171052713063545525074005126953125f,1.582100030192732820211176658631302416324615478515625f,0.154699995505809706486388677149079740047454833984375f,-0.28539999133944515818939180462621152400970458984375f,-0.3217999938100575274546599757741205394268035888671875f,-0.37599998773634357718265164294280111789703369140625f,0.32100000409036877346835581192863173782825469970703125f,0.864500020474195718378496167133562266826629638671875f,-0.044599998900294313131098533631302416324615478515625f,0.4290000501871114035878918002708815038204193115234375f,0.1836000092267990879690842120908200740814208984375f,-0.70660000849664184041643011369160376489162445068359375f,-0.28960000730082402498055671458132565021514892578125f,-0.8099000296764080619738024324760772287845611572265625f,0.079999996796250361086322300252504646778106689453125f,1.0923000035107133243172938819043338298797607421875f,0.254800001424551059159284704946912825107574462890625f,1.010000006556511120692221084027551114559173583984375f,0.1827999977141618881404383500921539962291717529296875f,1.446200010973214933329700215836055576801300048828125f,0.0964000083565712539979131179279647767543792724609375f,0.159600018495321904765660292468965053558349609375f,0.69559997933507045075884889229200780391693115234375f,-1.02639997065663379771649488247931003570556640625f,-0.3101000084519380806113986182026565074920654296875f,1.388000017970799859767794259823858737945556640625f,-0.74840004939436965969434822909533977508544921875f,0.66120002617239936881787798483856022357940673828125f,-0.00230001751780495311550112091936171054840087890625f,-0.713999996542930670528903647209517657756805419921875f,1.39890000030994343660495360381901264190673828125f,-0.972199988129734737896114893374033272266387939453125f,1.0037000108242040141703910194337368011474609375f,-0.30100000628829004511999301030300557613372802734375f,0.351800051403046154518960975110530853271484375f,-0.32120001507103435045564765459857881069183349609375f,-0.126100021445751053761341609060764312744140625f,-0.194599988991022154749543915386311709880828857421875f,0.10590005252361311960385137354023754596710205078125f,0.3851999572992326204001756195793859660625457763671875f,-0.479100013246387401721904097939841449260711669921875f,0.76180001213848591579136382279102690517902374267578125f,-0.64510000627264385553161218922468833625316619873046875f,0.12900000462681038282397594230133108794689178466796875f,0.26760002575591224083240149411722086369991302490234375f,-0.9562000103771683967579519958235323429107666015625f,-0.3326999763727176429028986603952944278717041015625f,0.396200020956992471354851659270934760570526123046875f,1.0235000052452090102406145888380706310272216796875f,-1.054600007170438669135137388366274535655975341796875f,1.527900013542176083092272165231406688690185546875f,0.6493000047624104542620671054464764893054962158203125f,-0.09320000421553852054667999027515179477632045745849609375f,0.56759998911768161367064067235332913696765899658203125f,0.17130000146925450366097720689140260219573974609375f,-0.5103999907433987015537013576249592006206512451171875f,-0.4066000017911195474340502187260426580905914306640625f,-1.797200038495659679682603382389061152935028076171875f,-0.1890000057443976810844787905807606875896453857421875f,-0.8707000278711323204561267630197107791900634765625f,-0.0174000050336122757954626649734564125537872314453125f,0.8539999887198206351968110539019107818603515625f,0.454500000134110404559351081843487918376922607421875f,1.585400069496035513338938471861183643341064453125f,0.1483000124067068714595052369986660778522491455078125f,-0.082199956613778102365586164523847401142120361328125f,0.04460000072568655327120978881794144399464130401611328125f,-0.07669998226612817493474949515075422823429107666015625f,0.0457000001922249765584638225845992565155029296875f,-0.648699994358420184425995103083550930023193359375f,-0.289299996566772410488965761032886803150177001953125f,-0.2479999529272307601246438935049809515476226806640625f,0.3197000137597323377036673264228738844394683837890625f,0.58360004409551702320158028669538907706737518310546875f,-0.03140000112950802613198675317107699811458587646484375f,-0.39560001465082172966702955818618647754192352294921875f,-0.2319000051304698217791155911982059478759765625f,-0.1325000306963921570257980420137755572795867919921875f,0.01800000180304051067281534415087662637233734130859375f,0.665600011372565969480774583644233644008636474609375f,0.16649999799579351389411385753192007541656494140625f,0.2244000060617923342221047278144396841526031494140625f,1.3788000093296168824252845297451131045818328857421875f,-1.39859999350607377976984935230575501918792724609375f,0.38750001728534666511905015795491635799407958984375f,0.3145999752819543804349677884601987898349761962890625f,-1.386399996435642378855845890939235687255859375f,1.205599988648295539661603470449335873126983642578125f,-1.70650000369548848766498849727213382720947265625f,0.95679998136758825921788229607045650482177734375f,-0.5533999994754790208162376075051724910736083984375f,-0.1297999876409772213037285837344825267791748046875f,0.633300044667720385405118577182292938232421875f,0.08519994619786785960968700237572193145751953125f,0.41950003101676724970303666850668378174304962158203125f,-0.0776000006288290189448986211573355831205844879150390625f,0.013000045351683642280704589211381971836090087890625f,0.6345999636590484715981119734351523220539093017578125f,0.04600002059340457805092228227294981479644775390625f,0.00279996880590971386482124216854572296142578125f,0.77860009237527894043751075514592230319976806640625f,-0.1906000525742765905334863418829627335071563720703125f,0.738700014027953155704153687111102044582366943359375f,-0.76599999444186661623490408601355738937854766845703125f,-0.3859000219389796537683423593989573419094085693359375f,0.44720001189708702415970265064970590174198150634765625f,-0.06130004334151717415579696535132825374603271484375f,-0.6973999737411740174053420560085214674472808837890625f,0.0644000054284925571579378811293281614780426025390625f,-0.472999964997172117620038989116437733173370361328125f,-0.2430999979615211525896256716805510222911834716796875f,-1.115400007283687155990037354058586061000823974609375f,-0.039700001801550366398174674031906761229038238525390625f,0.09489996919631948912865482270717620849609375f,0.0209000026419758899720591216464526951313018798828125f,0.7391000184059144384463024834985844790935516357421875f,0.2534000060319900882177535095252096652984619140625f,1.19340002532899358556051083724014461040496826171875f,0.24849999839067449247664853828609921038150787353515625f,0.081600018927454609496408011182211339473724365234375f,0.0113000015437603185119996851426549255847930908203125f,-1.134800034579634964160277377231977880001068115234375f,-0.344400008706748528286567534451023675501346588134765625f,-1.65090008701980206495818492840044200420379638671875f,-0.388700010600686152173466325621120631694793701171875f,-1.092400038486719626718013387289829552173614501953125f,0.15850000155717136607336215092800557613372802734375f,-0.058100025826692824892916178214363753795623779296875f,-0.18830001140087848643389634162303991615772247314453125f,-0.3253000459194190785439104729448445141315460205078125f,-0.09899999710172414923992079138770350255072116851806640625f,0.06000000488013024213529433836811222136020660400390625f,0.25689999592900270020123798531130887567996978759765625f,1.62600003810226922951187589205801486968994140625f,0.194200012589991199973837865400128066539764404296875f,-0.5090999741494652841566903589409776031970977783203125f,-0.059700006793439452934535438544116914272308349609375f,-0.17910002160966431716815350227989256381988525390625f,-0.6040999614834792197370916255749762058258056640625f,1.441799944487214713007006139378063380718231201171875f,-0.096799971234798221075834590010344982147216796875f,0.097999997407197891874375272891484200954437255859375f,1.363399992993474807434495232882909476757049560546875f,-1.010599964731932143280346281244419515132904052734375f,2.01040000728368806193202544818632304668426513671875f,-1.4549999776482585023273941260413266718387603759765625f,1.7539000098466868138302743318490684032440185546875f,-0.49919998491108419358397441101260483264923095703125f,0.5804000071346759881407706416212022304534912109375f,0.296000004425644558381236493005417287349700927734375f,-0.702900018236041201902253305888734757900238037109375f,1.037600009420514002300706124515272676944732666015625f,-0.54830005852580132597040574182756245136260986328125f,0.244600025126338049830110321636311709880828857421875f,-0.6440000025033949526687138131819665431976318359375f,0.272999998301267421396687495871447026729583740234375f,-1.45270003776550282736934605054557323455810546875f,0.989800027352571110128565123886801302433013916015625f,-0.54020003221482060240532518946565687656402587890625f,0.1559999943673611166872206013067625463008880615234375f,0.3293999990954989964819787928718142211437225341796875f,-0.4479999883919953962418958326452411711215972900390625f,1.631300026726723473302627098746597766876220703125f,-1.12540000214278723689176331390626728534698486328125f,-0.07089996073246052077365675359033048152923583984375f,1.0511999784886840725306456079124473035335540771484375f,-0.7239000301122668901143697439692914485931396484375f,0.582700055646897308037068796693347394466400146484375f,0.1985000092312694253138261046842671930789947509765625f,0.43050003473460718428356130971224047243595123291015625f,0.1499000038973987425361400482870521955192089080810546875f,0.4701000159956516544212945518665947020053863525390625f,0.070299998570978627920879944213083945214748382568359375f,0.92710002683997172479024584390572272241115570068359375f,0.245000010728836148388154470012523233890533447265625f,0.963800047960878369934789589024148881435394287109375f,-0.00089999172687527106972993351519107818603515625f,0.5984000286072495011779892593040131032466888427734375f,0.34049999759346238903390258201397955417633056640625f,0.7578000423058863788128292071633040904998779296875f,-0.070599998296797306096550528309307992458343505859375f,0.016500006899237806834435104974545538425445556640625f,0.13099999798089267954992465092800557613372802734375f,0.4849000071942806133762360332184471189975738525390625f,-0.08459999815523622235247103162691928446292877197265625f,-0.018800003108382146166377424378879368305206298828125f,-0.0945000004321336906087935858522541821002960205078125f,-0.9040000203847886606212114202207885682582855224609375f,-0.3386000078856945538774425585870631039142608642578125f,0.511200000095367368402321517351083457469940185546875f,0.389300005619227895703460262666339986026287078857421875f,1.7075000440329315409826449467800557613372802734375f,0.0607999985858798108750988831161521375179290771484375f,-0.39470000601112842364415200790972448885440826416015625f,-0.2950000024959444910876982248737476766109466552734375f,-0.828500024393200806827053384040482342243194580078125f,0.81330005867481336423452376038767397403717041015625f,0.237799990189075050039946290780790150165557861328125f,0.56730002580657623578730408553383313119411468505859375f,-0.3778000068783760045931785498396493494510650634765625f,0.44070001630410560178319201440899632871150970458984375f,-0.040799995940923761139629277749918401241302490234375f,1.257900029635429905994215005193836987018585205078125f,-1.197400008833408246999852053704671561717987060546875f,1.57270000915527408125171859865076839923858642578125f,-0.940799989756941901219988722004927694797515869140625f,0.84919998427778420335698683629743754863739013671875f,-0.7957999779850244426171457234886474907398223876953125f,-0.0209999897852550798660331565770320594310760498046875f,0.9759999628514055558525797096081078052520751953125f,0.437399996092915355205832383944652974605560302734375f,-1.340799985733628130191164018469862639904022216796875f,0.15330002201795611682655362528748810291290283203125f,0.4349999867379665818845069225062616169452667236328125f,0.69589998785257289881656106445007026195526123046875f,-0.930999999657272869768576128990389406681060791015625f,-0.186900037577748978634417653665877878665924072265625f,-0.0005999893188473048866171666304580867290496826171875f,0.1430000015795229728610138408839702606201171875f,-0.808200004625320378437436374952085316181182861328125f,0.77989998931288706529585397220216691493988037109375f,-1.106599985548853748440478739212267100811004638671875f,-0.796900002485513514471904272795654833316802978515625f,1.11039999051988136358204428688623011112213134765625f,-0.62069999762177463509260633145458996295928955078125f,0.17210001074671776866154004892450757324695587158203125f,0.0226000024169683566555022480315528810024261474609375f,0.228399989119171920037842937745153903961181640625f,0.07639999878257508481027571178856305778026580810546875f,-0.262899994915724022348513244651257991790771484375f,-0.2087000000581145098266233617323450744152069091796875f,0.15649996532499788060022183344699442386627197265625f,0.26159999646395437622459212434478104114532470703125f,0.243700015369057165770527717540971934795379638671875f,-0.0748999993711709777244323049671947956085205078125f,0.0952000058770179347078510545543394982814788818359375f,0.0922999988541006732845062288106419146060943603515625f,-0.41589999686777601173304219628334976732730865478515625f,-0.042699997979402548597960276310914196074008941650390625f,-0.999500001996755482736034537083469331264495849609375f,-0.326100001515448045807943344698287546634674072265625f,-0.10830004690289507518485834225430153310298919677734375f,0.1904999932721256339362980725127272307872772216796875f,0.568499981924890818163476069457828998565673828125f,-0.17360000211149451132541798870079219341278076171875f,-0.6623000350266698976753332317457534372806549072265625f,-0.024600002141296872792963768006302416324615478515625f,0.568100009956955975809478331939317286014556884765625f,0.23030000292956831486890223459340631961822509765625f,-0.0496000006139281079953207154176197946071624755859375f,0.0715999980658292844992729442310519516468048095703125f,0.62019999641478040786068959278054535388946533203125f,-0.0763999995276332111160400017979554831981658935546875f,0.99110001287758375809744393336586654186248779296875f,-0.733099988275765834799813092104159295558929443359375f,0.96959999599456747620962460132432170212268829345703125f,0.60169997500181171545818870072253048419952392578125f,-0.75539997800290592522287624888122081756591796875f,0.5992999968647954034395297640003263950347900390625f,-0.00940000516772254712805079179815948009490966796875f,0.7262999569892887308242279686965048313140869140625f,-1.19299993728101316747824967023916542530059814453125f,-0.9696000195384026909550811978988349437713623046875f,1.190999988928437058888221145025454461574554443359375f,-0.83599998348951398696726755588315427303314208984375f,0.6047999711751941287474210184882394969463348388671875f,-0.1585999725326896481192306964658200740814208984375f,0.80179998095780613542871151366853155195713043212890625f,-0.02689999483004246183526220193016342818737030029296875f,-0.14240001837015103802741577965207397937774658203125f,0.811999983072281050766605403623543679714202880859375f,-1.06099995353817977417065776535309851169586181640625f,1.00469996987581300373904014122672379016876220703125f,-0.3903999623566871690627522184513509273529052734375f,0.11899998715519899405990145169198513031005859375f,-0.2517999939590691571567049322766251862049102783203125f,0.42020003910660808088550766115076839923858642578125f,0.316199999275803111942195755545981228351593017578125f,-0.1310999830752610506579003413207828998565673828125f,0.0955999960988760388858054284355603158473968505859375f,0.3352999850109215085325331529020331799983978271484375f,-0.59640000440776308465729016461409628391265869140625f,1.28299999040365264590946026146411895751953125f,1.3380000227391721789871326109278015792369842529296875f,0.043899997739493823445400266791693866252899169921875f,0.5976999944031238243269399390555918216705322265625f,-0.043399997240304966084067928022705018520355224609375f,-0.18519999537169928860436129980371333658695220947265625f,-0.086599999071657662863543691855738870799541473388671875f,-0.73879999200701707007254981363075785338878631591796875f,-0.1527999998003244375155418310896493494510650634765625f,-1.133800015848874576107618850073777139186859130859375f,-0.2920999973729252463527927830000407993793487548828125f,-0.0028999960333104812804094763123430311679840087890625f,0.08539999808222054067385897724307142198085784912109375f,0.7825000213086605871382062105112709105014801025390625f,0.20040000441521410579071016400121152400970458984375f,0.950600025789440028489707401604391634464263916015625f,0.18030000088065867291930999272153712809085845947265625f,1.0993000150442124862593118450604379177093505859375f,0.10640000414699320174793939486335148103535175323486328125f,0.07980001241415768387099660685635171830654144287109375f,0.01280000002384185098236457633902318775653839111328125f,0.718600011089443935219378545298241078853607177734375f,0.02530000106692309458367162733338773250579833984375f,0.0132000264555216517692315392196178436279296875f,0.19999999705702065089241159512312151491641998291015625f,1.623900020352005757473534686141647398471832275390625f,0.3841000062614678967065628967247903347015380859375f,1.1700000379979613729375387265463359653949737548828125f,-0.19870000411868093959100178835797123610973358154296875f,-1.5556000805884604432094420189969241619110107421875f,0.73649999892711637272668667719699442386627197265625f,-0.6189999931156633383722009966732002794742584228515625f,1.0545999718546870127511283499188721179962158203125f,-0.616799970191717505940687260590493679046630859375f,1.14299996816366888907623433624394237995147705078125f,-0.81539997841268785361279469725559465587139129638671875f,-0.06789999042674932372420926185441203415393829345703125f,0.66379997963905357227787362717208452522754669189453125f,-1.10780003817081418304724138579331338405609130859375f,0.281000004500150613040432290290482342243194580078125f,-0.34670001463890098847286935779266059398651123046875f,-0.3363999971061943927708171031554229557514190673828125f,0.13370002300590311250516606378369033336639404296875f,-0.4032000101387500645699901724583469331264495849609375f,-0.2848999752685419384334863934782333672046661376953125f,-0.025199996638298305384751074598170816898345947265625f,0.0749000128194694525518571026623249053955078125f,-0.33060000600814820614914424368180334568023681640625f,-0.4242999944433567538482066083815880119800567626953125f,0.4885999986469744538197801375645212829113006591796875f,0.76709998392462619420939518022350966930389404296875f,-0.991400012156366639004545504576526582241058349609375f,0.2384000280857083708951904554851353168487548828125f,0.0611999974876642482968236436136066913604736328125f,0.22929999842941750642921761027537286281585693359375f,-0.839400002858042615372369255055673420429229736328125f,0.704899990132451303992411340004764497280120849609375f,-0.15639997624456913882795561221428215503692626953125f,-0.355600058311223765628028559149242937564849853515625f,0.9408000211238860543971895822323858737945556640625f,-0.023100001798570135580490614302107132971286773681640625f,-0.8537000369012355260878166518523357808589935302734375f,-0.3102999981984495558862136022071354091167449951171875f,-0.713300015163421452513148324214853346347808837890625f,-0.13650000313669441442954166632262058556079864501953125f,-0.431100026065111219253367380588315427303314208984375f,-0.0584999975338578170891423724242486059665679931640625f,-0.5589000006824729194221390571328811347484588623046875f,-0.3108000064648688276491839133086614310741424560546875f,0.242499968167393886719906959115178324282169342041015625f,0.27869999722689386967289237873046658933162689208984375f,1.5893000099778167300001996409264393150806427001953125f,0.394599997521936807221010212742839939892292022705078125f,0.3125000093877308149359350863960571587085723876953125f,-0.221100007475912629928416208713315427303314208984375f,-0.526900010606647040134475901140831410884857177734375f,0.27380000336915255587655337876640260219573974609375f,1.7155000427067281787429919859278015792369842529296875f,0.156800001633167263026535920289461500942707061767578125f,-0.07249998576939076233571768170804716646671295166015625f,-0.2147000022485852088749425092828460037708282470703125f,-0.5181000200897454277537690359167754650115966796875f,-0.0327000014811753836330154854294960387051105499267578125f,0.3824000040650370468853225247585214674472808837890625f,0.1529000075817108506015529201249592006206512451171875f,-0.6655999855190513070368751868954859673976898193359375f,-0.16240000216513872377532834434532560408115386962890625f,-0.319500024273992266898858360946178436279296875f,0.1224000116646291047572958632372319698333740234375f,0.1179999993443487937838654033839702606201171875f,0.01979997922182175074112819856964051723480224609375f,-0.1051999796509746065709123286069370806217193603515625f,0.2680000108182429841008342918939888477325439453125f,-0.117600006812810686795955916750244796276092529296875f,0.236999982997775138215956758358515799045562744140625f,-0.760199994701146852804640730028040707111358642578125f,-0.99190003305152085744111900567077100276947021484375f,0.9126000062912702492923244790290482342243194580078125f,0.26149997823312831624065211144625209271907806396484375f,-0.847399981117248568551758580724708735942840576171875f,1.209299984124302174137710608192719519138336181640625f,-1.088399974814057369343345271772705018520355224609375f,-0.500600013607741090027047903276979923248291015625f,1.3833999973893165158500551115139387547969818115234375f,-0.8612999751687056004811893217265605926513671875f,0.237799961355328992596014359151013195514678955078125f,0.99370001142024932949681215177406556904315948486328125f,-1.508000009700655397182345041073858737945556640625f,-0.0525999946683641805833531179814599454402923583984375f,0.49500000476837147100894753748434595763683319091796875f,-0.95880000519454444685152338934130966663360595703125f,0.1771999987095593809272031649015843868255615234375f,-0.148199968937038573102427108096890151500701904296875f,0.504799986895918539886451981146819889545440673828125f,-0.59069999061822908714702862198464572429656982421875f,0.8167999857634307137033147228066809475421905517578125f,-0.08089997384548297532091964967548847198486328125f,1.4635000322908167103008736376068554818630218505859375f,0.12229999609738588517426904900275985710322856903076171875f,0.91060000164955823276358160001109354197978973388671875f,0.224400002522766595092207353445701301097869873046875f,0.765700011777877609375764222932048141956329345703125f,-0.1072000101834535978895246444153599441051483154296875f,0.968199978250265047563516418449580669403076171875f,0.073100006119906968393706847564317286014556884765625f,-0.468300001752376626740215215249918401241302490234375f,-0.04630000597685579588613791202078573405742645263671875f,0.2856999886065718641248167841695249080657958984375f,0.16380000314563514240973063351702876389026641845703125f,0.4097000229984522068349406254128552973270416259765625f,0.02369999987185000822620395410922355949878692626953125f,-1.00800001387298099331246703513897955417633056640625f,-0.3085000036060809858184939002967439591884613037109375f,-1.034800046202540624307175676221959292888641357421875f,-0.02409999597966672180149316773167811334133148193359375f,-0.000399982547759802908871051840833388268947601318359375f,0.06989999843984839866806169084156863391399383544921875f,0.2239000038862226926728027365243178792297840118408203125f,-0.067399999818205824642092238718760199844837188720703125f,-0.29709999241828910498952609486877918243408203125f,-0.07239999799281360282066088984720408916473388671875f,-0.2324999812245367447616217759787105023860931396484375f,0.247400002947449682011438198969699442386627197265625f,0.33000002585351495554277789779007434844970703125f,-0.2710000002905725846602535966667346656322479248046875f,-0.437300016549229564333245434681884944438934326171875f,0.40100003791600524749583200900815427303314208984375f,-0.1635999941021204495683605273370631039142608642578125f,-0.1444999914541840180248755132197402417659759521484375f,-0.306400008282065261511206699651665985584259033203125f,-0.487100018942356882689637131989002227783203125f,-0.273799997818469531551954787573777139186859130859375f,0.96339999649524798996935714967548847198486328125f,-1.0521999738991265616050441167317330837249755859375f,-0.0020999885439874077519561978988349437713623046875f,0.637199999976158171222095916164107620716094970703125f,-0.2440000039935110720534794381819665431976318359375f,-0.21999999962747107673521895776502788066864013671875f,-0.1236999872803690170286472493899054825305938720703125f,0.298799986419081842203837595661752857267856597900390625f,-0.930799982231854894365596919669769704341888427734375f,1.57499999307095972511660875170491635799407958984375f,0.0752999860167502532704020268283784389495849609375f,-0.415400002142786828329690251848660409450531005859375f,0.6033999936640259686981835329788736999034881591796875f,-0.449399987882375728798223235571640543639659881591796875f,0.19429999030828459805064767351723276078701019287109375f,-0.2108000003367661301378888083490892313420772552490234375f,0.1034000211566683535835409202263690531253814697265625f,0.59639997579753423195825234870426356792449951171875f,0.25310002627372707451058886363171041011810302734375f,-0.3248000152826306941022949104080908000469207763671875f,0.3597000199437143663772076251916587352752685546875f,0.098399989864230263947320054285228252410888671875f,0.6017000223875044184751459397375583648681640625f,0.59310003938674993317903272327384911477565765380859375f,0.35570000533312562396304201683960855007171630859375f,1.21450001645088168089614555356092751026153564453125f,0.189499997712671675031970153213478624820709228515625f,0.358700043457746087227633324800990521907806396484375f,0.200900005324184949273558231652714312076568603515625f,-0.88489999944567632184089234215207397937774658203125f,-0.316299999941885456866685899512958712875843048095703125f,-1.245900051480532066960904558072797954082489013671875f,-0.337300011147558809199153984081931412220001220703125f,0.258500003308057557660504244267940521240234375f,0.3602000091552735039357457935693673789501190185546875f,0.2970000195056201608423407378722913563251495361328125f,-0.21620001108497388653262305524549447000026702880859375f,-0.8567000781178482071709368028677999973297119140625f,-0.2679000054582953982418302985024638473987579345703125f,-0.3393000144481661539686001560767181217670440673828125f,-0.080100000964105133594017615905613638460636138916015625f,-0.4695000122040511314480681903660297393798828125f,0.019299999770522102693348642787896096706390380859375f,0.08539999171197365512853139080107212066650390625f,0.042999997667968248293846045271493494510650634765625f,0.68410001550018773031069940770976245403289794921875f,0.17820000317245721088710297408397309482097625732421875f,0.231300007951259711802549645653925836086273193359375f,-0.0234000040203332648758305367664434015750885009765625f,0.141599973031877990337079609162174165248870849609375f,-0.02820000458806759535690389384399168193340301513671875f,-0.45950000959634795805186513462103903293609619140625f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_2d_8item_large_5o3i_uneven_filter_uneven_data_dilation_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{8,3,16,16};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{8,5,30,44};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1},        // move_strides
                                                                  Strides{1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0}, // below_pads
                                                                  CoordinateDiff{0,0}, // above_pads
                                                                  Strides{2,3}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f,.64f,.10f,.22f,-.81f,.95f,-.54f,.86f,-.12f,-.38f,-.48f,.01f,.89f,.96f,.51f,.00f,-.27f,.72f,-.07f,.91f,.45f,-.56f,.05f,-.76f,.23f,.79f,.88f,.51f,-.46f,.73f,-.46f,.65f,-.09f,-.49f,.43f,-.78f,.97f,.25f,.89f,-.11f,.72f,-.10f,.74f,.13f,.57f,.55f,.22f,.36f,.02f,-.92f,.66f,-.26f,-.09f,.92f,.38f,-.17f,-.79f,.40f,-.85f,-.17f,-.55f,.77f,.26f,-.79f,.01f,.03f,.58f,.51f,.24f,-.49f,-.34f,-.34f,.65f,.59f,-.81f,.08f,.26f,.69f,.62f,-.71f,-.80f,.70f,.13f,.43f,-.73f,.93f,.82f,-.75f,-.22f,-.11f,-.56f,-.46f,-.84f,.68f,.13f,-.29f,.13f,-.13f,-.41f,.85f,.37f,.64f,-.37f,.64f,-.81f,-.29f,-.31f,-.62f,-.47f,-.90f,-.15f,.39f,.75f,.87f,-.62f,-.68f,-.66f,.14f,.92f,.70f,-.35f,-.57f,-.12f,.12f,.41f,.89f,.85f,.23f,.57f,.54f,-.89f,.97f,-.66f,.89f,-.89f,-.53f,.43f,-.04f,-.39f,.14f,.91f,-.83f,.31f,-.83f,-.27f,-.65f,.76f,-.56f,-.17f,-.94f,.42f,.13f,-.77f,-.16f,.00f,.66f,-.67f,.84f,-.30f,-.52f,-.60f,-.42f,-.24f,.54f,-.48f,.60f,.72f,.22f,-.71f,.71f,.40f,.12f,.28f,.43f,-.04f,-.91f,.61f,.82f,-.88f,.40f,-.32f,-.26f,.16f,.87f,-.70f,.68f,.12f,.12f,.63f,-.00f,.12f,-.48f,-.93f,.47f,-.65f,.15f,.43f,-.49f,-.81f,-.44f,-.83f,.85f,-.85f,-.45f,.63f,-.22f,-.62f,-.55f,.30f,.58f,-.85f,-.59f,.03f,-.19f,-.84f,-.22f,.44f,.04f,-.20f,-.61f,-.14f,.72f,-.80f,-.02f,-.79f,-.49f,-.87f,.88f,-.86f,.47f,-.90f,-.86f,.77f,.77f,.67f,-.58f,-.53f,-.41f,-.75f,-.13f,.58f,-.94f,-.22f,.39f,-.89f,.89f,-.02f,-.49f,.01f,-.39f,-.93f,-.45f,.26f,-.71f,.88f,.10f,.47f,-.49f,-.41f,-.50f,-.40f,-.01f,-.19f,.33f,-.70f,-.74f,-.40f,-.08f,-.16f,-.32f,.14f,-.61f,-.64f,.23f,-.51f,-.27f,.00f,-.29f,.18f,-.77f,-.42f,.99f,-.10f,.93f,.51f,.57f,-.23f,.45f,.63f,-.23f,.70f,.33f,-.61f,.40f,-.47f,.60f,.03f,.60f,-.14f,.60f,-.81f,.50f,.03f,.50f,.98f,-.53f,-.97f,-.59f,.22f,-.68f,-.54f,.94f,-.61f,.55f,.46f,.07f,-.31f,-.15f,-.00f,.51f,.93f,-.52f,-.58f,-.33f,-.94f,-.63f,-.82f,-.35f,.75f,-.95f,.86f,.11f,.17f,.05f,-.41f,.24f,.36f,.44f,.33f,.17f,-.94f,-.31f,.67f,-.50f,-.77f,-.82f,.93f,-.41f,-.78f,-.65f,.24f,-.49f,.83f,-.17f,.56f,.29f,.76f,.71f,-.48f,-.90f,.61f,.27f,-.42f,-.18f,-.29f,.18f,.99f,.36f,.13f,.20f,-.32f,-.06f,-.37f,.06f,-.77f,-.91f,.07f,.43f,.48f,.91f,-.90f,.98f,-.49f,.93f,-.09f,.27f,-.01f,.81f,.13f,.88f,-.66f,-.25f,-.34f,.23f,-.93f,-.02f,-.46f,-.24f,-.83f,-.87f,-.73f,-.36f,.47f,.87f,.60f,.77f,-.25f,-.31f,-.14f,-.71f,.01f,.06f,-.24f,.97f,.92f,.06f,.11f,-.39f,-.49f,.72f,-.51f,-.03f,-.97f,.72f,.22f,-.50f,.49f,.03f,.32f,.63f,-.67f,-.83f,-.91f,.69f,-.64f,-.65f,.02f,.42f,-.08f,-.09f,.86f,-.36f,-.02f,.70f,.03f,-.13f,.75f,-.01f,-.89f,.44f,-.18f,-.68f,-.02f,-.41f,-.18f,.62f,.12f,.68f,-.50f,-.22f,.99f,.70f,-.06f,-.63f,.85f,.31f,.75f,.11f,.52f,.52f,.24f,.00f,.77f,-.54f,-.53f,-.60f,-.35f,.43f,-.11f,.40f,.79f,-.05f,-.35f,.06f,.46f,-.56f,.49f,.64f,.28f,-.03f,-.35f,-.86f,-.32f,.17f,.28f,.67f,-.08f,-.83f,-.75f,-.09f,-.21f,.53f,.57f,.96f,.11f,-.72f,-.87f,.51f,-.16f,.86f,-.08f,-.42f,.30f,.07f,-.85f,-.94f,-.05f,.69f,-.58f,-.93f,-.32f,.55f,-.13f,-.03f,-.26f,.87f,-.07f,.68f,.46f,-.15f,-.43f,.51f,.38f,-.66f,.71f,.36f,-.74f,-.89f,.65f,.29f,.06f,.15f,.74f,.74f,.08f,-.85f,-.58f,.16f,-.18f,-.14f,.72f,-.16f,.07f,.87f,-.07f,-.85f,.06f,.41f,.93f,.72f,.62f,-.15f,.80f,-.53f,.79f,-.62f,.61f,.34f,-.20f,-.19f,.68f,-.58f,-.44f,-.60f,-.92f,.71f,-.69f,-.14f,.81f,-.20f,.91f,-.70f,-.20f,.44f,.15f,-.74f,-.11f,-.61f,.56f,-.97f,.51f,.11f,-.38f,.43f,-.62f,.36f,.86f,.29f,-.39f,.81f,.89f,.48f,-.61f,.48f,.93f,.23f,.53f,.25f,.52f,-.27f,.55f,-.60f,-.98f,.93f,.65f,-.64f,-.41f,-.54f,-.87f,.81f,-.04f,.59f,.93f,-.74f,-.66f,.88f,-.20f,.75f,.18f,-.32f,.29f,.01f,-.50f,.91f,-.09f,.24f,.77f,.58f,.78f,-.05f,.71f,.16f,.19f,.65f,.99f,-.87f,.85f,-.43f,.74f,-.91f,-.46f,-.49f,-.08f,-.08f,-.23f,-.86f,.27f,-.30f,-.67f,-.19f,-.97f,-.19f,-.38f,.20f,-.94f,-.94f,.41f,-.57f,.26f,.78f,.64f,.18f,-.65f,-.98f,.20f,-.71f,.03f,.24f,.30f,.35f,-.91f,-.46f,.44f,.91f,.92f,.91f,.64f,-.29f,-.25f,-.99f,.89f,-.77f,-.51f,.53f,-.50f,.04f,.18f,.55f,.69f,-.39f,.54f,.63f,-.26f,-.81f,.56f,.75f,.89f,-.37f,.95f,.57f,-.39f,.31f,-.88f,-.14f,-.13f,.43f,-.41f,-.09f,-.10f,.07f,.70f,.89f,.32f,-.41f,.46f,-.92f,-.47f,.18f,-.65f,-.77f,.74f,-.60f,.16f,-.58f,-.55f,.44f,-.24f,-.36f,.90f,.24f,.91f,-.18f,.96f,-.57f,-.85f,.08f,.35f,.32f,.48f,.53f,-.89f,-.43f,.02f,.83f,-.62f,-.53f,-.46f,.06f,.54f,-.30f,.20f,.88f,.78f,.49f,.67f,-.56f,-.47f,-.70f,-.82f,-.86f,.14f,.06f,-.74f,.43f,.34f,.02f,-.55f,-.76f,.25f,-.22f,-.08f,.99f,-.60f,.40f,-.26f,-.59f,-.07f,-.10f,.85f,.10f,-.88f,.42f,-.18f,.14f,-.57f,-.10f,-.50f,.08f,-.33f,.08f,.50f,.07f,.33f,-.78f,.36f,-.34f,-.80f,.23f,.08f,.48f,-.12f,-.88f,-.67f,.87f,.44f,.55f,-.21f,.85f,-.16f,-.57f,-.18f,-.39f,-.44f,.84f,-.18f,.50f,-.87f,-.21f,.78f,.98f,-.77f,-.95f,-.83f,-.62f,.66f,-.62f,-.92f,-.33f,.02f,.11f,.58f,-.90f,-.01f,.95f,.73f,-.03f,-.92f,-.88f,.11f,-.05f,-.67f,-.94f,-.60f,.00f,.62f,-.05f,-.08f,-.71f,.75f,.49f,.50f,-.51f,.64f,-.65f,-.38f,.63f,.20f,-.59f,.76f,.49f,-.36f,-.63f,-.96f,-.90f,.39f,.58f,.60f,-.47f,.73f,.97f,-.59f,.50f,-.62f,-.14f,-.83f,.73f,.98f,.31f,.85f,.94f,-.94f,-.48f,.15f,-.15f,-.68f,.15f,-.84f,-.79f,.98f,-.74f,-.87f,.42f,.46f,.72f,.90f,.32f,-.56f,.12f,-.88f,-.87f,-.70f,.43f,.64f,.01f,.00f,-.77f,.09f,-.60f,.24f,-.95f,.09f,-.98f,.91f,-.38f,-.31f,.72f,-.73f,-.18f,.48f,.04f,-.92f,.33f,.83f,-.03f,.46f,.89f,-.78f,.76f,.40f,-.09f,.31f,.63f,-.47f,-.79f,.53f,-.94f,.17f,.87f,-.41f,.59f,-.67f,-.56f,.22f,.15f,-.85f,.02f,.14f,.41f,-.72f,-.41f,-.41f,.59f,-.10f,.08f,.23f,.75f,.83f,.81f,.04f,-.40f,-.19f,-.05f,.06f,.50f,.50f,.55f,-.10f,.39f,.06f,-.12f,.62f,-.58f,-.28f,-.67f,.55f,-.97f,-.93f,.48f,.50f,-.42f,-.76f,-.59f,-.96f,-.99f,-.80f,-.88f,-.31f,.31f,-.62f,-.33f,.82f,-.54f,.12f,.56f,-.30f,-.53f,.91f,.50f,.54f,.70f,.79f,.00f,.14f,-.49f,-.00f,.49f,-.84f,-.05f,.63f,.67f,-.50f,-.94f,-.69f,-.02f,-.23f,-.92f,.30f,.90f,.81f,-.74f,-.57f,-.97f,.98f,-.10f,-.46f,-.30f,.78f,.42f,.27f,.43f,.78f,-.45f,-.07f,.39f,-.42f,-.43f,-.90f,-.00f,-.33f,-.22f,.82f,.41f,-.40f,.33f,.63f,-.95f,-.61f,.19f,-.84f,-.17f,.36f,-.80f,.21f,-.27f,.64f,-.26f,.62f,.42f,.47f,.64f,.41f,-.74f,.33f,.73f,-.12f,-.92f,-.70f,-.87f,.99f,.58f,.17f,-.49f,-.95f,.02f,-.61f,-.23f,.07f,.18f,.86f,-.92f,.36f,.52f,.87f,.09f,.55f,-.69f,.71f,-.99f,.39f,-.34f,.91f,.34f,-.04f,-.28f,.80f,-.21f,-.27f,.50f,-.04f,.94f,.93f,-.42f,.71f,.15f,-.23f,.95f,-.42f,.20f,.88f,-.97f,-.81f,.78f,-.17f,.98f,-.45f,-.08f,.63f,-.77f,-.17f,-.56f,-.76f,.52f,.12f,.76f,-.33f,.83f,-.38f,-.25f,.54f,-.87f,-.19f,-.62f,-.81f,.07f,.87f,.03f,.96f,.75f,-.15f,.63f,.36f,.57f,-.88f,.25f,-.09f,.56f,-.11f,.34f,-.31f,-.38f,-.08f,.73f,.76f,-.37f,-.66f,-.55f,.50f,.15f,.97f,-.15f,.47f,.21f,-.77f,.39f,.68f,-.76f,-.71f,.43f,.31f,.44f,-.40f,-.44f,.88f,.25f,-.53f,-.94f,-.70f,.05f,.59f,.61f,.93f,.72f,-.93f,-.37f,.42f,.16f,.67f,.97f,.37f,-.04f,.16f,.40f,-.43f,-.38f,-.24f,.27f,-.32f,-.21f,-.46f,-.57f,.00f,.93f,-.94f,-.32f,.97f,.33f,-.82f,.56f,.47f,-.61f,.47f,-.10f,-.91f,-.93f,-.83f,.83f,.09f,-.33f,.85f,-.15f,-.48f,-.41f,.50f,.35f,-.53f,-.72f,.75f,.29f,.05f,-.87f,.57f,-.04f,.77f,.59f,.00f,-.05f,.29f,-.09f,-.52f,-.42f,-.55f,-.88f,.60f,-.23f,-.67f,-.89f,-.67f,.02f,.15f,-.71f,-.62f,.41f,-.43f,.62f,-.58f,.03f,.33f,.74f,.35f,-.99f,.37f,-.87f,.93f,.61f,-.32f,.64f,-.52f,-.43f,.52f,.28f,-.00f,.96f,.25f,-.83f,-.76f,.13f,.74f,-.47f,.27f,.22f,.10f,.13f,-.57f,.44f,.21f,.66f,.15f,.40f,.63f,.62f,.85f,-.46f,.68f,-.64f,.15f,-.04f,.52f,-.68f,-.00f,.74f,-.18f,-.10f,.84f,.74f,.56f,-.34f,-.26f,-.77f,-.75f,-.95f,.10f,.73f,.21f,.04f,.49f,.66f,-.36f,.03f,-.97f,-.06f,-.75f,.56f,.19f,.64f,-.32f,.04f,.76f,-.96f,-.64f,.59f,-.82f,.14f,-.29f,.57f,.03f,-.29f,.63f,-.75f,-.20f,-.41f,.89f,-.46f,.01f,-.37f,.48f,.11f,.90f,-.65f,-.70f,.69f,.23f,.14f,-.97f,-.64f,-.98f,-.48f,.79f,-.07f,-.83f,.77f,.75f,.54f,-.28f,-.78f,-.42f,.08f,-.90f,.17f,.83f,-.82f,.42f,-.61f,-.23f,-.13f,-.64f,-.56f,-.65f,.40f,.28f,-.29f,-.04f,.77f,.67f,-.64f,-.72f,-.71f,-.36f,-.76f,.62f,.75f,.89f,.96f,-.96f,-.68f,.61f,.74f,.10f,.75f,-.43f,.08f,-.93f,.23f,.02f,-.95f,.32f,-.34f,.85f,.57f,.56f,.86f,.44f,.91f,.58f,.34f,.36f,.14f,.45f,.87f,.89f,-.69f,-.42f,-.58f,-.22f,-.32f,.79f,.80f,.75f,-.15f,-.47f,-.72f,-.85f,.47f,-.92f,-.45f,-.79f,-.00f,-.82f,-.71f,-.91f,-.55f,.29f,-.20f,-.76f,.93f,.43f,-.88f,.99f,.21f,.73f,-.00f,.22f,-.59f,-.91f,-.83f,-.66f,-.05f,.97f,.12f,-.87f,-.23f,.12f,-.92f,.90f,.61f,-.64f,-.38f,.99f,.20f,.32f,-.83f,.90f,.14f,-.17f,-.90f,-.33f,.60f,.72f,-.34f,.29f,-.23f,-.74f,.54f,.64f,.15f,-.19f,.78f,-.18f,-.95f,.55f,-.47f,-.96f,.45f,-.99f,.71f,-.24f,-.85f,-.81f,-.90f,-.68f,-.16f,-.46f,.32f,-.80f,.16f,.26f,.61f,-.21f,.91f,-.85f,.49f,.52f,-.90f,.27f,.33f,.55f,.85f,.12f,-.14f,-.30f,-.23f,-.28f,.88f,-.04f,-.79f,.39f,-.12f,.21f,-.74f,.28f,-.91f,.20f,.24f,.69f,-.30f,.78f,.09f,.10f,-.12f,.80f,.98f,.84f,.17f,-.17f,.69f,-.04f,.93f,.09f,.56f,-.00f,.37f,-.34f,-.90f,.71f,.66f,-.36f,-.44f,-.91f,-.19f,.96f,-.65f,-.22f,-.71f,.65f,-.44f,.22f,.29f,.63f,-.45f,.35f,-.40f,.24f,.69f,-.09f,-.09f,.35f,-.27f,-.11f,-.23f,-.01f,.60f,.32f,.74f,-.88f,-.11f,.82f,-.77f,-.89f,.22f,.81f,-.58f,.16f,.72f,-.65f,-.99f,-.31f,.86f,.40f,-.20f,-.34f,-.36f,.85f,.68f,.48f,-.14f,.07f,-.09f,-.65f,-.83f,-.76f,-.23f,-.90f,-.25f,-.36f,.71f,-.80f,-.92f,.93f,-.16f,.87f,.52f,-.77f,.74f,.27f,.32f,-.52f,.85f,.01f,.67f,-.71f,.48f,.86f,-.85f,.00f,-.63f,-.06f,-.86f,.16f,.26f,.28f,.91f,.67f,.74f,.10f,.95f,.62f,-.98f,.77f,-.11f,.12f,.89f,.55f,.55f,-.61f,.25f,-.94f,.09f,.12f,.59f,.27f,-.92f,.29f,.74f,.54f,.82f,.99f,.81f,-.07f,.54f,.75f,.91f,-.72f,.65f,.41f,-.15f,-.84f,.52f,-.89f,.98f,.22f,.16f,-.03f,.73f,.01f,.01f,.07f,-.08f,.73f,-.00f,.48f,.24f,-.55f,.02f,-.77f,.49f,.20f,-.61f,.87f,.28f,-.89f,-.65f,-.03f,.39f,.12f,.12f,.58f,.18f,-.67f,-.08f,-.61f,.20f,-.03f,-.54f,-.76f,.01f,-.58f,.38f,.15f,-.16f,-.08f,.86f,-.93f,-.84f,.01f,.77f,.65f,-.96f,-.51f,.19f,.62f,-.52f,-.23f,-.95f,.12f,-.37f,.51f,.66f,-.16f,.13f,.94f,.15f,.04f,-.69f,.51f,-.24f,-.66f,.02f,-.34f,.03f,-.36f,.87f,-.75f,.22f,-.78f,-.16f,-.66f,-.98f,.79f,-.31f,-.88f,.94f,.91f,-.48f,.16f,.61f,.69f,.41f,-.17f,.15f,.37f,-.81f,.93f,-.60f,-.69f,-.70f,.40f,.98f,.69f,.10f,.04f,-.88f,.29f,.02f,.41f,-.11f,.27f,-.20f,-.86f,.06f,.00f,.96f,.92f,.95f,-.09f,-.60f,-.42f,.64f,-.72f,.78f,-.34f,-.57f,.54f,.78f,.98f,.80f,-.46f,-.71f,-.57f,.11f,-.51f,.76f,-.11f,-.57f,-.93f,.76f,.25f,-.27f,.55f,-.96f,-.97f,.86f,.86f,.52f,.30f,-.81f,-.73f,.70f,.86f,-.27f,.54f,.18f,-.52f,.17f,.51f,-.56f,-.32f,-.98f,-.77f,.21f,.76f,.06f,.81f,-.51f,.16f,.44f,.93f,-.98f,.79f,-.07f,-.39f,-.01f,-.59f,-.97f,-.09f,-.58f,-.29f,-.20f,.88f,-.89f,.35f,.08f,-.67f,-.49f,-.85f,.35f,.05f,-.07f,.96f,-.25f,.89f,.53f,-.67f,-.78f,-.22f,.36f,-.84f,-.73f,.69f,-.87f,.06f,-.70f,.55f,-.13f,.52f,.20f,-.70f,-.26f,-.35f,.75f,-.94f,-.17f,-.81f,-.66f,-.78f,.16f,.46f,-.26f,-.99f,.01f,.84f,-.31f,-.83f,-.75f,.21f,.22f,-.35f,-.41f,-.97f,-.16f,.30f,-.05f,.98f,-.66f,.19f,-.21f,-.77f,.17f,.77f,-.30f,.85f,.83f,.69f,-.05f,.69f,.47f,-.47f,-.79f,-.79f,-.28f,-.62f,-.46f,-.48f,.93f,.26f,.69f,.09f,-.40f,.13f,.77f,-.84f,-.74f,.82f,-.88f,-.68f,.80f,-.63f,-.67f,.90f,.03f,.36f,.22f,-.47f,.80f,-.66f,.75f,-.00f,-.20f,.95f,-.17f,.00f,-.74f,.46f,.40f,-.59f,.53f,.03f,.02f,-.81f,.09f,-.68f,-.63f,.24f,-.29f,-.64f,.65f,.48f,-.99f,.47f,.96f,-.85f,.68f,-.05f,.79f,.97f,-.26f,-.65f,.19f,.84f,.84f,-.30f,-.88f,.06f,-.87f,.62f,-.67f,-.49f,.95f,.28f,.47f,.44f,.59f,.06f,.90f,-.46f,.85f,.84f,-.22f,-.83f,-.46f,-.06f,.95f,-.91f,.51f,.50f,.97f,-.88f,.07f,.69f,.30f,.33f,-.34f,.53f,-.09f,-.90f,-.37f,.67f,.05f,.37f,.67f,.11f,-.58f,-.23f,-.52f,.43f,.62f,-.32f,.87f,.55f,-.85f,.97f,.40f,-.15f,-.06f,-.27f,-.71f,.63f,.68f,.75f,-.13f,-.98f,.74f,.13f,.26f,.59f,-.40f,.16f,-.45f,-.07f,-.36f,-.07f,.49f,-.97f,-.55f,-.77f,-.73f,.66f,.80f,-.90f,-.45f,.85f,.69f,.09f,.18f,-.92f,.61f,.99f,.81f,-.29f,-.32f,-.12f,.11f,.50f,.71f,-.26f,-.99f,.70f,-.70f,.21f,.84f,.27f,-.18f,-.74f,-.65f,.23f,-.99f,.39f,.83f,-.22f,.81f,.86f,.53f,.61f,.05f,-.40f,.90f,.52f,.23f,.03f,-.91f,-.04f,-.50f,-.16f,.89f,-.21f,.77f,-.07f,.90f,-.17f,.88f,-.10f,-.89f,.65f,-.74f,-.81f,.44f,.47f,-.38f,-.34f,.34f,-.37f,-.15f,.45f,.06f,.44f,.81f,.84f,-.94f,-.64f,.74f,-.00f,-.32f,-.36f,.06f,.11f,-.94f,-.16f,-.24f,-.20f,-.96f,.50f,.48f,-.00f,-.68f,.57f,-.58f,.24f,-.58f,-.39f,.52f,.88f,-.97f,.03f,.75f,.22f,.14f,-.73f,-.85f,.18f,-.63f,.62f,-.80f,.76f,.62f,-.38f,.07f,.42f,-.10f,.04f,.81f,-.89f,-.34f,.05f,-.70f,.19f,-.10f,-.98f,-.45f,.84f,-.67f,-.07f,-.84f,-.36f,-.80f,-.62f,.77f,-.94f,-.13f,-.79f,-.25f,.61f,.63f,.41f,.79f,.88f,.86f,-.38f,-.76f,-.27f,-.09f,-.29f,-.85f,-.54f,-.67f,-.53f,.19f,.42f,.07f,.35f,-.74f,.23f,-.09f,.48f,-.34f,-.81f,-.59f,.96f,-.09f,-.71f,-.46f,-.81f,-.16f,-.68f,-.34f,.63f,-.49f,-.76f,-.38f,.61f,.54f,.43f,-.63f,-.68f,-.43f,.54f,-.70f,.40f,-.57f,-.98f,-.62f,-.09f,-.99f,.81f,.41f,-.80f,.52f,.56f,.36f,-.83f,-.99f,.51f,-.95f,.04f,.37f,.32f,-.90f,-.63f,.04f,-.47f,.44f,.65f,-.11f,-.02f,-.36f,.93f,-.54f,-.16f,.98f,.00f,-.93f,-.33f,.33f,.44f,-.25f,-.97f,-.76f,-.66f,.76f,-.34f,-.93f,-.98f,.00f,-.52f,-.23f,.41f,.65f,.10f,.71f,.85f,.83f,-.38f,-.97f,-.78f,-.23f,-.79f,-.95f,.51f,.83f,-.87f,.37f,-.45f,-.88f,-.52f,-.19f,.13f,-.26f,-.10f,.73f,.12f,-.71f,-.86f,-.82f,.14f,-.21f,.16f,-.59f,.63f,.39f,-.08f,-.12f,-.99f,-.73f,.55f,-.69f,.00f,-.80f,-.25f,-.93f,-.58f,-.02f,.00f,.38f,-.47f,.31f,-.98f,.89f,.48f,-.16f,-.34f,.43f,-.62f,.32f,-.46f,-.73f,.94f,.75f,-.33f,-.56f,.09f,.77f,-.86f,.08f,-.42f,-.03f,.82f,-.87f,-.93f,.49f,.29f,.25f,.44f,.98f,.93f,-.65f,-.32f,.66f,.56f,.27f,.98f,.64f,.15f,.10f,.05f,.37f,-.25f,.93f,-.79f,.69f,-.64f,-.49f,.43f,.52f,.77f,-.74f,-.78f,-.65f,-.79f,.30f,.72f,.29f,-.59f,.93f,.76f,.90f,.51f,-.56f,-.02f,.53f,.96f,.71f,-.74f,-.50f,-.98f,.80f,-.81f,-.86f,.63f,.54f,-.40f,-.39f,.53f,.78f,.96f,-.94f,-.74f,.31f,-.30f,-.61f,-.46f,.31f,.76f,.99f,-.77f,-.05f,-.46f,-.75f,-.41f,.14f,.08f,.30f,.84f,.18f,-.83f,-.72f,-.94f,-.28f,-.65f,-.37f,-.55f,-.81f,.94f,.58f,-.52f,.47f,-.87f,.22f,.61f,-.24f,-.44f,.02f,-.18f,-.20f,-.59f,.91f,-.71f,.59f,-.22f,-.63f,.40f,.80f,.45f,.11f,-.49f,.87f,-.24f,-.51f,-.62f,.61f,.93f,-.28f,-.76f,-.60f,.14f,-.85f,.47f,-.67f,.64f,-.39f,.89f,.77f,.22f,-.90f,-.48f,-.24f,.54f,.35f,.70f,.96f,.16f,.46f,-.71f,.89f,.97f,-.97f,-.21f,.93f,-.72f,-.28f,-.72f,.54f,.23f,.00f,.67f,-.28f,.81f,-.05f,.24f,.71f,-.45f,-.09f,-.61f,.78f,.59f,.93f,.84f,-.74f,.32f,.37f,.07f,-.85f,-.59f,.10f,.05f,-.89f,.40f,-.97f,.31f,-.62f,.24f,-.64f,-.54f,-.26f,-.64f,.50f,.29f,.60f,-.28f,-.81f,-.49f,-.63f,.84f,.67f,.76f,.40f,.61f,.01f,-.66f,-.05f,.15f,.54f,-.67f,.77f,-.50f,.34f,-.12f,.24f,.91f,.69f,-.37f,.04f,.69f,.55f,-.76f,-.37f,.11f,.67f,.76f,-.61f,-.44f,-.89f,.45f,-.01f,.39f,.35f,-.29f,.38f,.95f,.87f,-.72f,.71f,.74f,-.81f,.48f,-.80f,-.90f,.18f,-.01f,.63f,.83f,-.96f,-.50f,-.43f,.84f,-.92f,-.18f,.50f,-.00f,-.26f,-.04f,.50f,.01f,-.41f,-.75f,-.17f,-.28f,.37f,.66f,-.47f,.01f,.25f,-.45f,.10f,.82f,-.01f,-.18f,.80f,.89f,-.54f,.35f,.43f,.08f,-.10f,-.59f,.98f,.88f,.45f,-.95f,-.18f,.35f,.90f,.37f,-.76f,-.32f,.66f,.60f,.89f,-.02f,-.46f,.73f,.10f,.83f,.21f,-.10f,-.39f,-.64f,-.67f,.31f,.21f,-.65f,-.44f,.96f,.44f,-.31f,-.00f,-.15f,-.40f,.40f,-.80f,-.66f,.30f,.77f,-.53f,.68f,-.25f,.54f,-.73f,.03f,.55f,.13f,.67f,.25f,.96f,-.18f,.81f,-.95f,-.74f,-.40f,.89f,.60f,.14f,.17f,-.52f,.97f,-.02f,-.68f,.05f,.39f,.43f,-.57f,.10f,.47f,.93f,.90f,.10f,-.86f,-.79f,.09f,-.79f,-.12f,.58f,.91f,-.36f,-.22f,.67f,.61f,.43f,.24f,.85f,.30f,.71f,-.10f,.34f,-.21f,.07f,-.99f,.87f,.17f,-.94f,-.12f,-.16f,.00f,.45f,-.35f,.65f,-.63f,.24f,.01f,-.64f,.59f,-.14f,-.17f,.75f,-.02f,.00f,.94f,.54f,.13f,.24f,.03f,-.41f,-.95f,.16f,.36f,.18f,.27f,-.17f,.41f,.24f,.81f,-.20f,-.98f,.47f,.14f,-.70f,-.12f,-.21f,-.99f,.83f,.96f,-.67f,.43f,.62f,-.69f,-.32f,.46f,-.50f,.53f,.91f,.16f,.34f,-.49f,.55f,-.30f,.34f,.00f,-.75f,.17f,.21f,-.16f,.00f,.56f,-.42f,.37f,.67f,.87f,.71f,.67f,-.88f,-.66f,.98f,-.79f,-.86f,-.17f,.21f,.41f,.75f,-.22f,-.36f,-.50f,-.18f,-.93f,.29f,-.85f,.53f,-.30f,.12f,.19f,.25f,.55f,.09f,.18f,.03f,-.01f,.75f,-.71f,.88f,-.12f,-.91f,.33f,.92f,.73f,.62f,-.51f,.56f,.40f,.98f,-.60f,-.49f,.26f,-.36f,-.42f,-.26f,-.57f,.41f,.97f,.55f,-.95f,.51f,.28f,.96f,-.55f,.42f,.94f,-.65f,-.29f,.01f,.50f,-.50f,.58f,.13f,.26f,-.56f,.35f,.30f,-.34f,.61f,.63f,-.82f,-.79f,.86f,-.81f,.24f,-.77f,-.17f,-.68f,.33f,-.95f,.32f,-.15f,.52f,.85f,.78f,.58f,-.50f,.73f,-.45f,.68f,.36f,-.91f,-.13f,.51f,.41f,.08f,.57f,.86f,.06f,.62f,-.29f,-.89f,.21f,.88f,.19f,.96f,-.17f,.22f,.51f,.17f,.64f,.11f,-.69f,-.14f,.12f,.28f,-.51f,.30f,-.26f,-.24f,.46f,.32f,-.21f,.78f,.17f,-.77f,.60f,-.08f,-.35f,-.36f,-.94f,.41f,-.38f,.88f,.03f,.23f,-.36f,.66f,-.56f,-.52f,.77f,.02f,-.74f,-.66f,.57f,.70f,.86f,.58f,.05f,.77f,-.60f,.70f,-.80f,-.19f,-.64f,.29f,.44f,.84f,.11f,-.71f,-.64f,.28f,.36f,.17f,-.62f,.49f,.33f,-.02f,.16f,-.39f,.77f,-.40f,-.15f,.79f,.03f,.17f,-.44f,-.76f,-.02f,.83f,-.99f,.22f,-.98f,-.92f,.11f,-.93f,.25f,.16f,.21f,.98f,-.41f,-.43f,-.53f,.74f,.83f,.75f,-.85f,-.51f,-.65f,-.37f,.98f,.16f,-.67f,-.83f,-.90f,-.49f,-.52f,-.88f,.26f,-.57f,-.15f,.02f,.27f,-.06f,.40f,-.72f,.41f,.04f,-.48f,-.58f,.98f,.17f,-.83f,-.33f,-.93f,.01f,-.07f,-.11f,.75f,.77f,-.97f,-.13f,-.04f,.92f,.35f,.90f,-.49f,.06f,.13f,-.66f,-.65f,.41f,.15f,.32f,.80f,-.34f,-.25f,-.37f,.51f,.26f,-.43f,-.54f,.45f,.67f,.37f,.65f,.23f,-.85f,-.56f,.41f,-.83f,.17f,.74f,-.85f,-.15f,-.82f,.41f,.86f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.7218999722734096291532068789820186793804168701171875f,-0.53799999543279408698026600177399814128875732421875f,-0.1766000023126601536915813994710333645343780517578125f,-0.2209999900460244415256738648167811334133148193359375f,0.7854999811649321372897247783839702606201171875f,0.85430000595450383826801044051535427570343017578125f,0.6160999824047090900336343111121095716953277587890625f,-0.7060000323653223830433489638380706310272216796875f,0.130299989891052181434361045830883085727691650390625f,0.256599995048344187154043538612313568592071533203125f,-1.092999989211559608293100609444081783294677734375f,0.0786999825119973905884762643836438655853271484375f,-0.7555999651417140139386674491106532514095306396484375f,-0.918500021636486252418762887828052043914794921875f,-0.190300012093782555666621192358434200286865234375f,-0.9162999917834995233789641133625991642475128173828125f,-0.2089999851062891433883805802906863391399383544921875f,-0.8074000163957475439957534035784192383289337158203125f,-0.1614000074252486260917294202954508364200592041015625f,0.53900000011920923270736238919198513031005859375f,-0.87740000242590898693606504821218550205230712890625f,0.0211999940231442085547541864798404276371002197265625f,0.57650000160932535209212801419198513031005859375f,-0.36739998588562006176516661071218550205230712890625f,0.6022999988168475926642031481605954468250274658203125f,0.1710000171661381074272867408581078052520751953125f,-0.31230001230239867737736858543939888477325439453125f,-0.57840001441389288228123177759698592126369476318359375f,1.16500001370906858255693805404007434844970703125f,-0.6096999904394149183417539461515843868255615234375f,0.5891999791368842664951444021426141262054443359375f,-0.376000010013580521217591012828052043914794921875f,-0.5744000129163264745102424058131873607635498046875f,-0.2633999962344766299082721161539666354656219482421875f,0.522000012576580019185712444595992565155029296875f,-0.02400001069903368033919832669198513031005859375f,-0.2486000001743435827705752672045491635799407958984375f,-1.07200002032518426631213515065610408782958984375f,-0.4679000212907791222960440791212022304534912109375f,-0.88859998326152567216240640846081078052520751953125f,-0.53000001609325408935546875f,0.646100020998716217945911921560764312744140625f,0.42000000365078449249267578125f,-0.4255000004619358122681660461239516735076904296875f,-1.2223000340580938427592627704143524169921875f,-0.5202000161588191673445180640555918216705322265625f,-1.1171999683856963514472226961515843868255615234375f,0.11980000216960906556096233543939888477325439453125f,0.30910000972598794533041655085980892181396484375f,0.69649999151378860773320411681197583675384521484375f,-0.375300012280046946244738137465901672840118408203125f,0.35770001269429929635634834994561970233917236328125f,0.223699989441036972692700146581046283245086669921875f,-0.87330004608333167137601549256942234933376312255859375f,0.8491999800682066279478021897375583648681640625f,0.7455999738216405603452585637569427490234375f,-0.704200017321110038892584270797669887542724609375f,-0.493299991470575349694627220742404460906982421875f,-0.0782000090956689319909855839796364307403564453125f,0.624600006723403655684023760841228067874908447265625f,0.18799999122321597866402953513897955417633056640625f,-1.21370000963211044364697954733856022357940673828125f,-0.467300015729665840780171492951922118663787841796875f,-0.00819999613165833807215676642954349517822265625f,-0.980899965649843608161972952075302600860595703125f,0.2354999902546399681568800588138401508331298828125f,0.7518000067368149075264227576553821563720703125f,0.40769997246116407296767647494561970233917236328125f,0.059900017513335068741753275389783084392547607421875f,-0.172700001499801913951159804128110408782958984375f,0.362299995259195650731953719514422118663787841796875f,-0.2157000087417662026467723990208469331264495849609375f,-0.103900001502037042655501863919198513031005859375f,-0.4597999954640865638566538109444081783294677734375f,0.343000010520219777987449560896493494510650634765625f,0.1681999885320664134269463829696178436279296875f,1.8652000240564348132465966045856475830078125f,-0.0917999944806098255867254920303821563720703125f,-0.46079998722374426733949803747236728668212890625f,0.2127000172764059726659979787655174732208251953125f,-0.35530002066195010002047638408839702606201171875f,-0.33720002858638764564602752216160297393798828125f,-1.925999966502189408856793306767940521240234375f,0.0206000009417532936595307546667754650115966796875f,-1.31860003128051772591788903810083866119384765625f,0.24460001208782244930262095294892787933349609375f,-0.082800028336048825394755112938582897186279296875f,-0.47760001411437968954487587325274944305419921875f,-0.980199999991059112147695486783050000667572021484375f,0.23449999141693123050345093361102044582366943359375f,-0.45139998391866686944240427692420780658721923828125f,0.5448999905422329259607749918359331786632537841796875f,0.2595000085905194708857379737310111522674560546875f,0.1823999992966651717551940237171947956085205078125f,-0.2646999981507658361579160555265843868255615234375f,0.8795000283345582037242138540023006498813629150390625f,-0.247400002537667640378771238829358480870723724365234375f,-0.11390000310391190652126169879920780658721923828125f,0.2905000360012053306490997783839702606201171875f,-0.0087999973714349977171877981163561344146728515625f,-0.597899968130886971806603469303809106349945068359375f,-0.45350000241398813471960238530300557613372802734375f,0.2668999802827836109742065673344768583774566650390625f,0.24109999454915520544773244182579219341278076171875f,0.480500000536441707055246297386474907398223876953125f,-0.84049999852478496364938109763897955417633056640625f,0.1684999979808925463231616959092207252979278564453125f,-0.297499992400407364812053856439888477325439453125f,-0.4398999921441077987083190237171947956085205078125f,0.1228999890670181560636819995124824345111846923828125f,-0.63800000698119418984788353554904460906982421875f,0.089899990206956825744555317214690148830413818359375f,-0.631599987672269325145180118852294981479644775390625f,0.40900000535324210204635164700448513031005859375f,0.1531999998569488763422441479633562266826629638671875f,0.11040000059679144772672998442430980503559112548828125f,-0.37050000709295272116605701739899814128875732421875f,-0.1077000023007392837115503425593487918376922607421875f,0.08129999789297581713753970689140260219573974609375f,-0.01449997889995557898146216757595539093017578125f,0.92740002090334883178002201020717620849609375f,-0.420899980886280555836265193647705018520355224609375f,0.8064999981075526847007495234720408916473388671875f,0.082700007367134009683695694548077881336212158203125f,0.3760999810263516707209419109858572483062744140625f,0.23349999332427984199966886080801486968994140625f,-0.95079997895359991133545918273739516735076904296875f,0.5698999947145579891838451658259145915508270263671875f,1.084500034928321809957196819595992565155029296875f,0.36000000283122091104814899154007434844970703125f,0.9397000061228866218954181022127158939838409423828125f,0.787999989509582832170053734444081783294677734375f,0.70590002421140685129330449854023754596710205078125f,0.07949999968707555098035300034098327159881591796875f,-0.163800011043250481890254377503879368305206298828125f,0.14480001196712277700129334334633313119411468505859375f,-0.77120003168582940134001546539366245269775390625f,-0.2886999935388558924387325532734394073486328125f,1.0393000603437432260989226051606237888336181640625f,-0.769899999594688466686420724727213382720947265625f,0.3318999963760376203936175443232059478759765625f,0.21129999334812143985118382261134684085845947265625f,0.2832000055193901744132745079696178436279296875f,-1.729800000381469970989201101474463939666748046875f,-0.21480002094507266718892424250952899456024169921875f,-0.63129997658431502571829696535132825374603271484375f,-0.83839996058344912199800091912038624286651611328125f,0.011799986433982834199696299037896096706390380859375f,-0.271200005906820251766475848853588104248046875f,-1.059299980622530057416952331550419330596923828125f,0.36990000868439665282494388520717620849609375f,-0.52160001661777499748495756648480892181396484375f,-0.408099987232684924265413428656756877899169921875f,0.56690000472068735604125322424806654453277587890625f,-1.376299998712539718326297588646411895751953125f,0.75530000240802763755709747783839702606201171875f,0.3210999865770336469950052560307085514068603515625f,-0.603299994599819200402635033242404460906982421875f,0.059099992179870497466254164464771747589111328125f,0.69530000061988772586119011975824832916259765625f,0.479399997419118761854406329803168773651123046875f,-0.8388000216603277436888674856163561344146728515625f,0.005600018823147312474475256749428808689117431640625f,0.186500015616417158526019193232059478759765625f,2.0740000191926952766152680851519107818603515625f,-0.3450000280141836128677823580801486968994140625f,-1.212000010192394228170087444595992565155029296875f,-0.311099960350990301094498136080801486968994140625f,0.7541000309228902409586225985549390316009521484375f,-1.158900008654594415702376863919198513031005859375f,-0.730199977862834970210315077565610408782958984375f,0.7434000048398967663842995534650981426239013671875f,0.0618000156402587208503973670303821563720703125f,-0.28260002841949471985572017729282379150390625f,-0.853800024342536545418624882586300373077392578125f,-0.0828999807596204618675983510911464691162109375f,0.1080000002756715815621646470390260219573974609375f,-0.2480000103712083348028727414202876389026641845703125f,-0.053500005133450045935461503177066333591938018798828125f,-0.06099999827891589010420148042612709105014801025390625f,-0.79050003540515945132938213646411895751953125f,0.35010000427961340818683311226777732372283935546875f,0.3919000098988412172928974541719071567058563232421875f,0.29150000518560403861556551419198513031005859375f,0.398799989771843055308409020653925836086273193359375f,0.46129999498724927331494427562574855983257293701171875f,0.0815000022053720130088549922220408916473388671875f,-1.047800004345178503939450820325873792171478271484375f,0.1330999883130192795732682498055510222911834716796875f,0.456999979645014153817328406148590147495269775390625f,-0.318099990287423306511982445954345166683197021484375f,0.6015999624148016255986703981761820614337921142578125f,-0.278500010162591848938973271287977695465087890625f,-0.38449999558925629372652110760100185871124267578125f,0.299899991957843337075928502599708735942840576171875f,-0.3549999874830245261136951739899814128875732421875f,0.077499997168779533041060858522541821002960205078125f,0.330200005728006118488337961025536060333251953125f,0.594500010490417452047040569595992565155029296875f,0.77979998726844801382185323745943605899810791015625f,0.809099989870190494656299051712267100811004638671875f,-0.4654999901056289246525921043939888477325439453125f,0.38539997777938861389657176914624869823455810546875f,0.28309999825954423879892374316114000976085662841796875f,-0.35800002244114903504623725893907248973846435546875f,-0.5378000046432018788067352943471632897853851318359375f,-0.2087999971479177485633726973901502788066864013671875f,0.30300003838539169009891338646411895751953125f,0.948900007760524744071517488919198513031005859375f,-0.41599999455362546285641656140796840190887451171875f,-0.0895000319480896422419391456060111522674560546875f,0.3277000136256216222818693495355546474456787109375f,0.77849998322874325307196841094992123544216156005859375f,-0.1125000026822089438383045489899814128875732421875f,0.090899995154142487763238023035228252410888671875f,0.81329998804330816408736382072675041854381561279296875f,1.075999987363815080243512056767940521240234375f,-0.4520000066161156127009235206060111522674560546875f,0.184399984678626172041049358085729181766510009765625f,0.11200003433227578852893202565610408782958984375f,-0.089799999785423079146085001411847770214080810546875f,-0.02650001204013818778548738919198513031005859375f,-1.18969997553825379554837127216160297393798828125f,0.488700014698505214028045884333550930023193359375f,1.4544000136613846052568987943232059478759765625f,-0.13230003018379221657596644945442676544189453125f,-0.7089000149130821881726660649292171001434326171875f,-0.1497000208377841090623405762016773223876953125f,1.90829996747970653814263641834259033203125f,0.250300025355816302408129558898508548736572265625f,0.0406000022828578721600933931767940521240234375f,-0.6034000075221062076025191345252096652984619140625f,0.104600002253055546219684401876293122768402099609375f,0.164000002391636368059835149324499070644378662109375f,-0.539600006425380751551301727886311709880828857421875f,-0.2024000092059374811270799909834749996662139892578125f,-1.098499998986721237770325387828052043914794921875f,0.176899968957901165822477196343243122100830078125f,0.3140999912858006837268476374447345733642578125f,0.52239998618364325011498294770717620849609375f,-0.343600011610984523713341332040727138519287109375f,0.35040001711845381038301638909615576267242431640625f,-0.4579000122010707940489737666212022304534912109375f,-1.1710999803185462297960839350707828998565673828125f,0.04330000234842312778482664725743234157562255859375f,0.3846999982625245451117734774015843868255615234375f,-0.54760001336932173643390342476777732372283935546875f,0.0350000090897084614738332675187848508358001708984375f,0.343800011080503509219852276146411895751953125f,-0.0198000126004220788900056504644453525543212890625f,-0.389600025200844246597853270941413938999176025390625f,0.46540001428723343224191921763122081756591796875f,0.70759999220967273458882118575274944305419921875f,-0.54480002139210714773298604995943605899810791015625f,0.03260000245422123299476879765279591083526611328125f,-0.371900005242228548496541407075710594654083251953125f,0.40310001472532752675448364243493415415287017822265625f,-0.2318999968230723851547736558131873607635498046875f,-0.01919999273419392693540430627763271331787109375f,0.277400000637769483802230752189643681049346923828125f,0.5346000040411951204077922739088535308837890625f,-0.789899972772598601977733778767287731170654296875f,0.34310000571012455594654966262169182300567626953125f,-0.1645000292658806984036345966160297393798828125f,0.20649999870359891929183504544198513031005859375f,-0.615000005960464335430515347979962825775146484375f,-0.461399992114305490531478426419198513031005859375f,0.1179999990090727646219193047727458178997039794921875f,-0.146999990224838228414228069595992565155029296875f,-0.59550000485777854208890857989899814128875732421875f,-0.889799989652633627201794297434389591217041015625f,-0.22799999070167586978641338646411895751953125f,1.1285999908983708195364670245908200740814208984375f,-0.268599968360364815822549644508399069309234619140625f,-0.16400000652670865264326494070701301097869873046875f,-0.2932000065624713602829842784558422863483428955078125f,0.05079999661147593936760813448927365243434906005859375f,0.1470000002086162549375103480997495353221893310546875f,-0.387600002454221215231200403650291264057159423828125f,0.01499999672174452680639689106101286597549915313720703125f,0.4010000034570690985447072307579219341278076171875f,0.41559997128844283764692590921185910701751708984375f,0.70599999462813112227621559213730506598949432373046875f,-0.8449999931454659218843517010100185871124267578125f,-0.166000005245208637205678314785473048686981201171875f,-0.3977999805778267283784543906222097575664520263671875f,0.28899998611211774601770230219699442386627197265625f,-0.51819998912811282565371584496460855007171630859375f,0.5247999890938401801321333550731651484966278076171875f,-0.328000007912516711172656869166530668735504150390625f,-0.35450000430643557880472371834912337362766265869140625f,-0.1953999959215522164157619044999592006206512451171875f,0.25650001695752155939089789171703159809112548828125f,-0.2100000077486037231011550829862244427204132080078125f,-0.16510000096410515357803205915843136608600616455078125f,0.3660000140368939725021846243180334568023681640625f,0.100199999916553483814141145558096468448638916015625f,-0.344799999561905767819780521676875650882720947265625f,-0.5510000097528100493349256794317625463008880615234375f,-0.067300007943809025956483083064085803925991058349609375f,-0.0462000004306435396728147679823450744152069091796875f,-0.1919999976754187542837826185859739780426025390625f,0.1292999965667724904250235340441577136516571044921875f,0.113799996514618417808861750017968006432056427001953125f,-0.8419999964833255745588758145458996295928955078125f,-0.408099992150068313634392325184307992458343505859375f,-0.3292000012278555320932582617388106882572174072265625f,-0.8545000137686731278563456726260483264923095703125f,0.211700025069713859693365520797669887542724609375f,1.1754999817609785850436310283839702606201171875f,-1.1452000109434123942264704965054988861083984375f,-0.339799978327750551443386939354240894317626953125f,0.998200005519390032304727355949580669403076171875f,-0.210700033831596300615274230949580669403076171875f,-0.1848999969124793807395690237171947956085205078125f,0.3830999959312377800557669615955092012882232666015625f,0.7529999649226668534396367249428294599056243896484375f,0.04720000254660827732067218676093034446239471435546875f,0.772800020959973466005976661108434200286865234375f,0.3521999960273500818175307358615100383758544921875f,-0.13959999651610832671622119960375130176544189453125f,-0.2443000204086303028816473670303821563720703125f,-0.443599964523315293263294734060764312744140625f,-0.713000031232834174943491234444081783294677734375f,-0.0717000062942503291196771897375583648681640625f,-0.7757999603271485966615728102624416351318359375f,-0.47260001858472833191626705229282379150390625f,-0.667100019389391007962331059388816356658935546875f,-0.2511999711871144569386160583235323429107666015625f,0.951000057399273845959442041930742561817169189453125f,-0.2946000130377710579665517798275686800479888916015625f,0.0129000044524671952927974416525103151798248291015625f,0.11490000685900458865518203310784883797168731689453125f,0.6255000070184468796696819481439888477325439453125f,-1.34550002154707915025255715590901672840118408203125f,-0.3465000029504297085480857276706956326961517333984375f,-0.55550002139806764489549095742404460906982421875f,-0.751599995160102452018691110424697399139404296875f,0.76660004582405161244196278857998549938201904296875f,1.5031000119686126481610699556767940521240234375f,-0.9532000296592713084464776329696178436279296875f,-0.240599979782103901015943847596645355224609375f,0.410600003176927685899499920196831226348876953125f,0.3454000078797339057246063021011650562286376953125f,-0.857200042891502977937534524244256317615509033203125f,-0.7566000011950730907983597717247903347015380859375f,1.83570001788735392977969240746460855007171630859375f,0.1120999985277653099302597183850593864917755126953125f,0.024600010001659455838307621888816356658935546875f,0.990000005960464335430515347979962825775146484375f,0.476200026023388289786453242413699626922607421875f,-1.368900024449825281180892488919198513031005859375f,-0.64139997076839228018485528082237578928470611572265625f,-0.419000017404556501787737943232059478759765625f,-0.9540999859213830092130592674948275089263916015625f,-0.591999984376132548646864961483515799045562744140625f,-0.4959999980926514950851924368180334568023681640625f,-0.370100012177228876453227712772786617279052734375f,-0.65439998713731772905788375283009372651576995849609375f,-0.34999998301267642997203211052692495286464691162109375f,0.34129998319670573625472798084956593811511993408203125f,-0.4181999787718060357377680702484212815761566162109375f,-0.3915000067502258929152958444319665431976318359375f,-0.011400000160932677317759953439235687255859375f,-0.59510000215619773822339766411460004746913909912109375f,1.119000020384788740557269193232059478759765625f,-0.4298999718785285750755065237171947956085205078125f,0.405599997477233298326382282539270818233489990234375f,0.67750000178813962747881305404007434844970703125f,-0.558699973720312215164085500873625278472900390625f,0.284899987524747988487661132239736616611480712890625f,-0.75200004163384459587859964813105762004852294921875f,0.3197000150263307194364870156277902424335479736328125f,0.328199998404085757641723830602131783962249755859375f,0.06749999940395345898735968148685060441493988037109375f,0.10350000407174231309426204461487941443920135498046875f,0.1887000043421982464764141695923171937465667724609375f,-0.001500016584992547308274879469536244869232177734375f,-0.95520000580251218647021005381247960031032562255859375f,-0.15450000639259808821179831284098327159881591796875f,-0.5900000348687175488748835050500929355621337890625f,-0.030699997174739923622155401972122490406036376953125f,0.371000001557171277255520180915482342243194580078125f,-0.79300002586841600304978783242404460906982421875f,-0.882500003874302052508937777020037174224853515625f,-0.885199994663893985347158377408050000667572021484375f,0.81850003400445014989372793934307992458343505859375f,-0.1875999899744986709748673092690296471118927001953125f,-0.1766999979123473618614070801413618028163909912109375f,0.403500012323260381918998973560519516468048095703125f,1.16190000688135608442053126054815948009490966796875f,0.21670000193566085755492167663760483264923095703125f,-0.6485000129938125468243015347979962825775146484375f,0.68299999979138359407215830287896096706390380859375f,-0.28769999861270167951232679115491919219493865966796875f,0.2009999778270721293438327847979962825775146484375f,0.722000023007392854879071819595992565155029296875f,0.4238000444591047966014230041764676570892333984375f,0.8877999979376784978057912667281925678253173828125f,0.20579999847412100422161529422737658023834228515625f,0.06540001205205925316477078013122081756591796875f,0.302699998426437133502986398525536060333251953125f,0.47510000457763634784669193322770297527313232421875f,0.67560002597570445459496113471686840057373046875f,0.45840001050233780688358820043504238128662109375f,0.6088000067591661235155697795562446117401123046875f,0.24719998030662537757962127216160297393798828125f,-0.3515999778747556092639570124447345733642578125f,-0.9830000219941137373780293273739516735076904296875f,0.70819999538660027837977395392954349517822265625f,1.31829994930028959743140148930251598358154296875f,-0.1368999978661538108326567453332245349884033203125f,-0.7953999718189237455590046010911464691162109375f,0.3337000251293176233957638032734394073486328125f,-0.0765000093579288886758149601519107818603515625f,0.32399999654293054618392488919198513031005859375f,-1.242600010836124369006938650272786617279052734375f,0.24560001256465913144211299368180334568023681640625f,0.37280000952333214314649012521840631961822509765625f,-0.672200013014674357947342286934144794940948486328125f,-0.00920000711381463442961603504954837262630462646484375f,0.3467999935306609327057003611116670072078704833984375f,-1.1426999809920788120365386930643580853939056396484375f,-0.48870000624209619655147207595291547477245330810546875f,-0.1062000011384487407894994248636066913604736328125f,-0.7932000016450881929586103069595992565155029296875f,-0.36120001231431952959383124834857881069183349609375f,-1.59599996732175331004555118852294981479644775390625f,0.07320002131462022276764400885440409183502197265625f,0.658799989026784604817521540098823606967926025390625f,-0.04490002403259296670512412674725055694580078125f,-0.51910000469684547397264395840466022491455078125f,-0.708699999201297003992294776253402233123779296875f,-0.343999981343745986350768362171947956085205078125f,-1.311099974656105615622436744160950183868408203125f,0.045099990570545145374126150272786617279052734375f,-0.49879996414184546438264078460633754730224609375f,-0.97839998993873678045929409563541412353515625f,-0.5952000520706182129515582346357405185699462890625f,-0.27090000245571133064004243351519107818603515625f,-0.6570000003203746441471366779296658933162689208984375f,-0.58900000578165023767951424815692007541656494140625f,0.45879998634457574979705896112136542797088623046875f,-0.5046000149190421613099033493199385702610015869140625f,-0.6734999930262564049598950077779591083526611328125f,0.309299987626075800761782375047914683818817138671875f,-0.20029999901801343042961889295838773250579833984375f,-1.30199998813867523494991473853588104248046875f,0.31799999010562896017972889239899814128875732421875f,-0.7011999986052510625000877553247846662998199462890625f,1.10850000822544103584732511080801486968994140625f,-0.56879996876120575421964531415142118930816650390625f,0.0881000019475817186531685365480370819568634033203125f,-0.35249997645616559793779742904007434844970703125f,0.51569998018741625145366924698464572429656982421875f,-0.7128999686524275514187820590450428426265716552734375f,0.68799997699260728722947533242404460906982421875f,0.307100005382299201528439880348742008209228515625f,0.540799965691566963954528546310029923915863037109375f,-0.5555000121593476336556705064140260219573974609375f,-0.640600007051229436427774999174289405345916748046875f,-0.077499999143183284644464947632513940334320068359375f,-0.2604999981895088634331614230177365243434906005859375f,-0.433400010912120270401004518134868703782558441162109375f,-0.15610000259578227232992730932892300188541412353515625f,0.39049998921155937292581938891089521348476409912109375f,-0.84109997970387351262644415328395552933216094970703125f,0.0448999922186137379043202599859796464443206787109375f,0.55699999320507043165662253159098327159881591796875f,-0.52339999724030494832049953402020037174224853515625f,0.289399990415573127311432699571014381945133209228515625f,0.317999984741211161320961764431558549404144287109375f,0.574799983170628525641632222686894237995147705078125f,1.01399995929002795236328893224708735942840576171875f,0.965999984383582699365433654747903347015380859375f,-0.50759998148083695923560298979282379150390625f,0.2718000028625129349535427536466158926486968994140625f,0.2029999793767931492993739084340631961822509765625f,-0.61429999611973773454565161955542862415313720703125f,0.46499996885657346457065841605071909725666046142578125f,1.1559999998807910515097319148480892181396484375f,-0.62440000710487364443679325631819665431976318359375f,0.6575999585330489782819540778291411697864532470703125f,-0.6179999918937681968600372783839702606201171875f,-0.0148999915480612088458656216971576213836669921875f,0.09399997001886362113509676419198513031005859375f,0.349400020217895956875509000383317470550537109375f,0.58260003020763395653602856327779591083526611328125f,-0.38579998238086687933900975622236728668212890625f,-1.4498999876737599379339371807873249053955078125f,-0.0557000120759010286519696819595992565155029296875f,0.0497999825000761120463721454143524169921875f,1.081799981069565319558023475110530853271484375f,0.263800016593933150943485088646411895751953125f,-1.23600000584125524483170011080801486968994140625f,-0.86219998984336854164212127216160297393798828125f,1.0192000191092489558286615647375583648681640625f,0.1227000094532968432758934795856475830078125f,0.06120001648664441518121748231351375579833984375f,-0.64080004377365185064263641834259033203125f,-0.363999977171420852073424612171947956085205078125f,-0.473599983596802331931030494160950183868408203125f,-0.340400032615661984891630709171295166015625f,0.71389999732971176626961096189916133880615234375f,0.1746999671936038112107780762016773223876953125f,-0.7679000075817103976305588730610907077789306640625f,-0.41969997255802127256174571812152862548828125f,-0.13169997053146431653658510185778141021728515625f,-0.5657000359177590809167668339796364307403564453125f,-1.07000002980232267191240680404007434844970703125f,-0.3487000108242028773020138032734394073486328125f,0.7617000241756439748996854177676141262054443359375f,0.2788000337302687370311105041764676570892333984375f,1.2089000119328492388603990548290312290191650390625f,0.04969999639987943140795323415659368038177490234375f,0.736099990898370748482193448580801486968994140625f,-1.1150000002980231528226795489899814128875732421875f,0.155200008261203503678871129523031413555145263671875f,-0.0731999887928367343192803673446178436279296875f,0.97529997882694008382031825021840631961822509765625f,-0.507700008521973611408384385867975652217864990234375f,0.57449999432265741461378638632595539093017578125f,0.0458999670654538505232267198152840137481689453125f,0.046100017198920539840401033870875835418701171875f,-0.19050001931190507775681908242404460906982421875f,1.51589999172687583950391854159533977508544921875f,0.01510001500844992960992385633289813995361328125f,0.446099984341859823189224698580801486968994140625f,-0.253399980327487117648388448287732899188995361328125f,-0.8359999906420707560528171597979962825775146484375f,0.365000013709068316103412144002504646778106689453125f,-0.251999975025654077143144604633562266826629638671875f,0.3669999777078629676907439716160297393798828125f,-0.71410000156760222722596154198981821537017822265625f,0.5405999691277745267115051319706253707408905029296875f,-0.388999986410141218584612943232059478759765625f,0.65479999382495890358768519945442676544189453125f,-0.276599974298477491174708120524883270263671875f,-0.416500016212463464171378291212022304534912109375f,0.1293999863922596915699614328332245349884033203125f,0.8334999868050216864645562964142300188541412353515625f,0.74950002986192743037463515065610408782958984375f,-0.2088999927103518672311111004091799259185791015625f,0.03609999253749851533257242408581078052520751953125f,0.7190000013113024834865427692420780658721923828125f,-0.29599999988079073176550082280300557613372802734375f,0.41899997277557876618203636098769493401050567626953125f,0.47749999940395326802899944595992565155029296875f,-0.30740000019073487891319018672220408916473388671875f,-0.3882999893099070565227748375036753714084625244140625f,1.053999996542930972509566345252096652984619140625f,-0.17979998950362219289900167495943605899810791015625f,0.449399966312945320368044122005812823772430419921875f,-0.20450000572204629634143202565610408782958984375f,0.28249998837709444643451206502504646778106689453125f,0.6795000097081060896897497514146380126476287841796875f,-0.27850000017881360037108606775291264057159423828125f,0.6048999988496301938312171841971576213836669921875f,-0.4201000111714001672424956268514506518840789794921875f,-0.74299999651312820247994750388897955417633056640625f,-0.6840999988853930791066204619710333645343780517578125f,-0.3811999895527959925090044634998776018619537353515625f,0.7149999935552475704980679438449442386627197265625f,0.36049999621510497860299437888897955417633056640625f,-0.0046000038549303123858180697425268590450286865234375f,-0.4865000051110985879176951129920780658721923828125f,-0.072400010621547661315844379714690148830413818359375f,-0.437499983534217040670455389772541821002960205078125f,0.113000007987022144106958876363933086395263671875f,0.8482000050723552675435712444595992565155029296875f,-0.1219999786391856577694170482573099434375762939453125f,-0.17600000450015063080400068429298698902130126953125f,-0.3469000175356864446030158433131873607635498046875f,-0.5600000135600566864013671875f,-0.6867999821126460346931708045303821563720703125f,0.56080000488161996230473960167728364467620849609375f,0.864799998127669056202648789621889591217041015625f,0.420099981052428628203188054612837731838226318359375f,0.0025000068359075644508493496687151491641998291015625f,-0.61969999416470500364084728062152862548828125f,-0.697899944847822695237482548691332340240478515625f,0.5355000018775459835751462378539144992828369140625f,0.00439999954253433855910770944319665431976318359375f,-0.058400000667572005141892077517695724964141845703125f,0.17420000942349445072210301077575422823429107666015625f,0.12990001777410498107201419770717620849609375f,0.111299964737892054245094186626374721527099609375f,0.51950002479553258893929523765109479427337646484375f,0.73350000453740360484289340092800557613372802734375f,-0.708300013151764940033672246499918401241302490234375f,-0.67770002502501036634185993534629233181476593017578125f,0.80670003147721303093931055627763271331787109375f,0.428399986809491650774361914955079555511474609375f,0.1900000213086610045820634695701301097869873046875f,0.7143000341176986012214911170303821563720703125f,0.00689994153976414281714824028313159942626953125f,-0.9793000293493268060274203890003263950347900390625f,-0.4669000166416168440264300443232059478759765625f,-1.021099980020522934864857234060764312744140625f,0.262300006937980612065075547434389591217041015625f,-0.818299998176097886926072533242404460906982421875f,0.287600005173683115344829275272786617279052734375f,0.7388000162959098560122583876363933086395263671875f,-1.0866999957144258814878412522375583648681640625f,0.142000036269426033186391578055918216705322265625f,-0.0674000283539299971380387432873249053955078125f,0.160900010353326905487847398035228252410888671875f,-0.1515999948620798676302001695148646831512451171875f,-0.731600034499168661028534188517369329929351806640625f,1.255900020337104905365777085535228252410888671875f,-0.043300014269352260498635587282478809356689453125f,-0.2598999963164327908771156216971576213836669921875f,-0.0049999971687793021146717364899814128875732421875f,0.74660000611245624924094954621978104114532470703125f,-0.5666000308483842839990529682836495339870452880859375f,-0.788500018954277237526184762828052043914794921875f,0.64109998095184572708404857621644623577594757080078125f,-0.3169999939501284558218685560859739780426025390625f,-0.065200000810623048863590156543068587779998779296875f,0.4069999996870754532807268333272077143192291259765625f,-0.6464999864660203598987209261395037174224853515625f,0.0548999943792820577215252342284657061100006103515625f,-0.70409997949525726657071800218545831739902496337890625f,-0.23950000676512672725948505103588104248046875f,-0.0679999853372574847298892564140260219573974609375f,0.4578999783754351682318883831612765789031982421875f,-0.2315000061914325168999084780807606875896453857421875f,0.027199998821318127628643424031906761229038238525390625f,-0.030299999353289586156989798837457783520221710205078125f,-0.7785000228285792189808489638380706310272216796875f,0.209999974370002728818462855997495353221893310546875f,-0.2243000015586613660190096197766251862049102783203125f,0.3495000109747052885467155647347681224346160888671875f,-0.7499999998882411755829480171087197959423065185546875f,-0.271499999485909970786678968579508364200592041015625f,-0.849500024348497362325360882095992565155029296875f,0.134299991351365832770170527510344982147216796875f,-0.7029000060170884633947707698098383843898773193359375f,0.755499991297721606997583876363933086395263671875f,-0.46840002361536026143085109652020037174224853515625f,-0.31510001165568812808714938000775873661041259765625f,0.007499992847442626953125f,-0.3623999915480613509544127737171947956085205078125f,0.459299995598196897361731316777877509593963623046875f,-0.3645000081062317320856891456060111522674560546875f,0.5636999895155430095883275498636066913604736328125f,0.39209999428093433859743299763067625463008880615234375f,0.893000012308359458756967796944081783294677734375f,0.26530000866651537450024989084340631961822509765625f,0.7843999792024494155384672922082245349884033203125f,0.84150001218915004841392146772705018520355224609375f,-0.3602999937653541540072410498396493494510650634765625f,0.063699989441037274673362844623625278472900390625f,-0.5895000060200690228384701185859739780426025390625f,-0.3570000021457673256009002216160297393798828125f,-0.85529999704360937240466000730521045625209808349609375f,0.743000025570392796936403101426549255847930908203125f,0.20560000608265405208641141143743880093097686767578125f,-0.0029999991953373328357201899052597582340240478515625f,0.3945000095963475672533604665659368038177490234375f,0.02629999096393564883555882261134684085845947265625f,-1.3943000205576421279829446575604379177093505859375f,0.5036000092267993721861785161308944225311279296875f,0.61280000896453845626865586382336914539337158203125f,-0.459199997949600202673536841757595539093017578125f,0.519800007236003835942028672434389591217041015625f,-0.041200005459785415951046161353588104248046875f,0.29460003272593038303739376715384423732757568359375f,1.85879999454021405114190201857127249240875244140625f,0.256400005525350582757937445421703159809112548828125f,0.8501999830782409617313533090054988861083984375f,-0.330300065737962000866900780238211154937744140625f,0.1187000315368180736186332069337368011474609375f,0.42170001121163380730649805627763271331787109375f,0.75820000358223893499598489142954349517822265625f,-0.50280002401471168838043013238348066806793212890625f,0.57369998966455426625543623231351375579833984375f,1.77919994162321160047213197685778141021728515625f,-0.2857999832749360535899540991522371768951416015625f,-1.07469999610185595884104259312152862548828125f,-0.92089994076490455654493416659533977508544921875f,0.695499993085860523933661170303821563720703125f,1.12309999729096876563971818541176617145538330078125f,0.55239995995759993974161261576227843761444091796875f,-0.457200005787611107876955429674126207828521728515625f,0.416999988548457434234251195448450744152069091796875f,0.639599967458844620438185302191413938999176025390625f,-0.0735999956667423571587960395845584571361541748046875f,-0.926500025153160322588519193232059478759765625f,-0.265700000452994800070882774889469146728515625f,1.175700051712990301666650339029729366302490234375f,-0.7403999870181081632836139760911464691162109375f,-1.07339995924234443691602791659533977508544921875f,0.78360001995563521859367028810083866119384765625f,0.7382000038057565660665204632095992565155029296875f,-1.48310001509785660545048813219182193279266357421875f,-0.3185000092685223993527188213192857801914215087890625f,-0.575300017458200585451777442358434200286865234375f,-1.24689999950528118688453105278313159942626953125f,0.2193000050604343442728350055404007434844970703125f,0.657200005079805915642054969794116914272308349609375f,-0.505699999558925750164917189977131783962249755859375f,-0.3969000059872864749621612645569257438182830810546875f,0.066899999654292940931554767303168773651123046875f,0.4177999850109219703853113969671539962291717529296875f,0.22650000488758070815720202517695724964141845703125f,0.7484999957084657040695674368180334568023681640625f,0.8031000019848344440021037371479906141757965087890625f,0.3915000010132789753924953402020037174224853515625f,0.35220000221133229700853917165659368038177490234375f,0.25589999444633726444209287365083582699298858642578125f,-0.564999996423721118077310165972448885440826416015625f,1.017399988642334829336277834954671561717987060546875f,-0.58620000098943680910679177031852304935455322265625f,-0.784000011593104062512793461792171001434326171875f,-0.30170002253651606594075929024256765842437744140625f,-0.59039997017979661819708780967630445957183837890625f,0.3430000209510328801343348459340631961822509765625f,0.149900004661083219303918667719699442386627197265625f,-0.330399998380243697937430624733678996562957763671875f,-0.0599999943375593147720792330801486968994140625f,0.73649998581409459319502275320701301097869873046875f,-0.6613999670431021105088120748405344784259796142578125f,-0.11099999898672052722758962772786617279052734375f,-0.0431999904692172975728681194595992565155029296875f,0.79439997421056052218091281247325241565704345703125f,-0.235499979674816284358485063421539962291717529296875f,-0.081300009664893035932209386373870074748992919921875f,-0.81769997421205065091243113784003071486949920654296875f,-0.2159999849051239539932112165843136608600616455078125f,0.2333999893799425517926238171639852225780487060546875f,-0.397999976322055193378446347196586430072784423828125f,-0.85450002658367196772815077565610408782958984375f,0.4389999929666519307147609652020037174224853515625f,0.4725000078231094136071988032199442386627197265625f,-0.473500032961368333417340181767940521240234375f,-0.1597999945700170343343415879644453525543212890625f,0.557299970541894840181385006872005760669708251953125f,-0.121000004276633266186991022550500929355621337890625f,-1.040900004467367967464497269247658550739288330078125f,-0.219400002634525304046064775320701301097869873046875f,0.1454999899566171706055683898739516735076904296875f,-0.47580000175237646686809966922737658023834228515625f,0.58609999574124793664253729730262421071529388427734375f,0.0364999965429306794106878442107699811458587646484375f,-0.5320999928280711888106679907650686800479888916015625f,-0.309900001718103779158042243579984642565250396728515625f,-0.39450000631809256645965433563105762004852294921875f,0.508300025892257423265618854202330112457275390625f,0.35350001299381261787857511080801486968994140625f,-0.35680000967979452752842917107045650482177734375f,-0.7252000309109689624165184795856475830078125f,0.27209997155666343360280734486877918243408203125f,-1.47239998946189842854437301866710186004638671875f,-0.141399999117851393748424015939235687255859375f,0.187299997811019469651228064321912825107574462890625f,-0.762699988293647823667242846568115055561065673828125f,-0.3057000054448841819265680896933190524578094482421875f,0.8034000307679178831676836125552654266357421875f,0.82590000275373398608280695043504238128662109375f,0.171899996674060417944929213263094425201416015625f,0.01729999829530726174198207445442676544189453125f,0.465700014162063524736367980949580669403076171875f,-0.69310004147291248699502830277197062969207763671875f,-0.247100006425380769314870121888816356658935546875f,0.372199998784065400059262174181640148162841796875f,0.4966000198960305311857155174948275089263916015625f,0.19160000350475314689902006648480892181396484375f,-1.05170000271797192681333399377763271331787109375f,-0.726100033664703659042061190120875835418701171875f,0.23449999540299160116774146445095539093017578125f,1.13199996686726844785653156577609479427337646484375f,-0.492000010080635430398388052708469331264495849609375f,-0.809799991440772970463513047434389591217041015625f,-0.21619999473094964059782796539366245269775390625f,0.60959999927282293441521687782369554042816162109375f,-0.3352000007361173317121938453055918216705322265625f,0.06130000959038728325367628713138401508331298828125f,0.4383000066697595453746316707110963761806488037109375f,-0.51969999986439940897753331228159368038177490234375f,0.773200010958313921349827069207094609737396240234375f,0.18740000115931032542704315346782095730304718017578125f,-0.9095000116825104896634002216160297393798828125f,-0.45759998848438243612690712325274944305419921875f,1.1086000539302833800547887221910059452056884765625f,0.505600018189847677518855562084354460239410400390625f,1.111400018489360430606893714866600930690765380859375f,0.0188000086218120809888887379202060401439666748046875f,-0.590200013588368932460070936940610408782958984375f,-0.92209998500496137552318032248876988887786865234375f,0.420700010921061018365207928582094609737396240234375f,-0.766600003374367844344305922277271747589111328125f,-0.707899961946905076359826125553809106349945068359375f,0.691500007271766747862784541212022304534912109375f,-0.5049000009357929030784362112171947956085205078125f,-0.0435000091195105742514215307892300188541412353515625f,-0.007999996066093473245928180404007434844970703125f,-0.8838999860048293299996657879091799259185791015625f,0.0818000095680353478400093081290833652019500732421875f,0.2664999928176403631852053877082653343677520751953125f,-0.545799988378584455261943730874918401241302490234375f,0.057899993276596173341630446884664706885814666748046875f,-0.82549998801946600224255234934389591217041015625f,0.340099992358684488635844900272786617279052734375f,-0.764300000180303751307064885622821748256683349609375f,0.8940000355839732293361521442420780658721923828125f,0.009000010401010616334360747714526951313018798828125f,0.05540000128597011297415519948117434978485107421875f,-0.4705000038146973651009830064140260219573974609375f,0.41469999565482140724270720966160297393798828125f,0.01170000696480266721977159249945543706417083740234375f,0.816500007271766747862784541212022304534912109375f,-0.842899997150897917208567378111183643341064453125f,0.1932999891236424350182687703636474907398223876953125f,0.465500015474855644015406141988933086395263671875f,0.388899992710352027103226646431721746921539306640625f,-0.2594999834820630457699053295073099434375762939453125f,-0.198500000774860296814949833787977695465087890625f,0.250399985080957510064081361633725464344024658203125f,0.49909998722523452041599512085667811334133148193359375f,-0.326999997898936189955065856338478624820709228515625f,0.24689999674856666356248524607508443295955657958984375f,0.1413999990433454367888543856679461896419525146484375f,0.1350000036880374398862159068812616169452667236328125f,0.5763000015810131326787768557551316916942596435546875f,0.208399998208880443684165584272705018520355224609375f,-0.778000043749809577775522484444081783294677734375f,0.31089999887943253753519456950016319751739501953125f,0.50699999324232347674978882423602044582366943359375f,-0.4055000026226041054400184293626807630062103271484375f,0.4982000105485318020015483853057958185672760009765625f,-0.5627000010713931743566718068905174732208251953125f,-0.1095000100806355902705035987310111522674560546875f,-0.231899997270107238733771737315692007541656494140625f,0.49889999449849120605193775190855376422405242919921875f,0.1394999901689590871001200866885483264923095703125f,-0.366600004398822676421332289464771747589111328125f,-0.906999995589256258199384319595992565155029296875f,0.44830001218318926703432225622236728668212890625f,0.35570000075101848580061414395458996295928955078125f,0.12279998720884321983248810283839702606201171875f,-0.520599991703033282419710303656756877899169921875f,-0.81180002756118785356420630705542862415313720703125f,1.52860005452632918832023278810083866119384765625f,0.54079995436668326647122739814221858978271484375f,-1.056600024664401615837050485424697399139404296875f,-0.6179000109530985440642325556837022304534912109375f,0.8960000205561520214558868246967904269695281982421875f,0.5072000210240485973400836883229203522205352783203125f,-0.3657999862551690739564946852624416351318359375f,1.533699956583976842239280813373625278472900390625f,0.09269999812841422937026436557061970233917236328125f,0.83120001171827340158415609039366245269775390625f,-0.30139998034238857371747144497931003570556640625f,0.0267999917984003133142323349602520465850830078125f,0.2543999957799911726397112943232059478759765625f,-1.333799994540214584048953838646411895751953125f,0.053600001180171830128529109060764312744140625f,1.115700001160800258759309144807048141956329345703125f,0.051599938833713832053717851522378623485565185546875f,-0.4532000146836046017284616027609445154666900634765625f,-0.536200015667080975845237844623625278472900390625f,-0.2650999970525500515350358909927308559417724609375f,-0.10530000844299802764680862310342490673065185546875f,0.767300000306963880802868516184389591217041015625f,0.5487999794155360433478563209064304828643798828125f,0.08180000692307931586810809676535427570343017578125f,-1.29660001512765887810019194148480892181396484375f,-0.21329997552633273016908788122236728668212890625f,0.3940999987363813517049493384547531604766845703125f,-0.61919998129755260407591777038760483264923095703125f,0.833699981394409928014965771581046283245086669921875f,0.33289999573528750165252176884678192436695098876953125f,-0.16160000926032669354981408105231821537017822265625f,-0.233500001948326652012610793462954461574554443359375f,0.0713000030152499686408873458276502788066864013671875f,0.420900019443035233734917710535228252410888671875f,-1.065600007796287940209367661736905574798583984375f,-0.5526000296115878285263534053228795528411865234375f,-0.9023000219881538441768498159945011138916015625f,0.582700009565054966031993899377994239330291748046875f,0.202999996364116697122881305404007434844970703125f,0.536499996393918987536153508699499070644378662109375f,0.51099998863041407037854924055864103138446807861328125f,0.9770000125765800902399860206060111522674560546875f,-0.576199983704090090697036430356092751026153564453125f,0.17139999963939178684313446865417063236236572265625f,0.250499988317489652445146930404007434844970703125f,-0.3702000155627729327534325420856475830078125f,-0.9847000134989614128500079459627158939838409423828125f,-0.21250000303611160656913625643937848508358001708984375f,0.76930000805929299456664693934726528823375701904296875f,0.191300004579871896215337301327963359653949737548828125f,0.1410000055432316656833791057579219341278076171875f,0.918299972099065886510516065754927694797515869140625f,-0.0133999947816134667988308137864805757999420166015625f,-0.64949998456239654842647723853588104248046875f,-0.31379999409317971270638736314140260219573974609375f,-0.5618999986484645159379169854219071567058563232421875f,-0.25150000607967371024642488919198513031005859375f,-0.7405999978125095140057965181767940521240234375f,0.029699996362626546186902487534098327159881591796875f,-0.2324999761581423296519233190338127315044403076171875f,-0.3447000251963732164739440122502855956554412841796875f,-0.727899975544214594691538877668790519237518310546875f,0.5370000057965516049307552748359739780426025390625f,-0.0716999969065188480499273282475769519805908203125f,0.45940000039935091624698770829127170145511627197265625f,-0.6789999857395887516986476839520037174224853515625f,0.169299991202354505048788269050419330596923828125f,-0.67159998022168887477079124437295831739902496337890625f,0.445000000298023223876953125f,0.27410000395178801824158654198981821537017822265625f,0.9001999903053043983192083032918162643909454345703125f,0.0175000013038515422891805428662337362766265869140625f,0.6772999768748879656499184420681558549404144287109375f,0.239899991846084625901625031474395655095577239990234375f,0.027499999944120556705229319049976766109466552734375f,-0.0683000003367661978614933104836381971836090087890625f,0.1427000030644236805432711889807251282036304473876953125f,0.4190000007152558936240893672220408916473388671875f,-0.8363000052690505725649927626363933086395263671875f,-0.00880001633316263021100667174323461949825286865234375f,0.660999988704919161364159663207828998565673828125f,0.415900038516522130294106318615376949310302734375f,-0.104000023663044061095206416212022304534912109375f,-1.604199986624717411132223787717521190643310546875f,-0.09379999275207495657014078460633754730224609375f,-0.47009999116659173523657955229282379150390625f,0.838799982321262405093875713646411895751953125f,-0.3100000047683710846513349679298698902130126953125f,0.8611000159695745281851486652158200740814208984375f,1.03609998624175769776911693043075501918792724609375f,-0.265900003908574600330894099897705018520355224609375f,0.766099969887733323048450984060764312744140625f,0.89289998970031803082747501321136951446533203125f,-0.8267000214934352442242015968076884746551513671875f,1.13799999547004659916638047434389591217041015625f,-0.97720008938312474811027641408145427703857421875f,-0.027799984824657286708315950818359851837158203125f,0.1569999917149544899075408466160297393798828125f,0.091400028324127191581283113919198513031005859375f,-0.81540002784729015417042319313623011112213134765625f,-0.288699996668100311580928973853588104248046875f,-1.24489998603463192239360068924725055694580078125f,0.47050001946091679627670600893907248973846435546875f,-1.771300009441375777896610088646411895751953125f,0.86260001173019418274634517729282379150390625f,0.2407999650955190418244455941021442413330078125f,0.9853999768853185514672077260911464691162109375f,-0.088100028359889392959303222596645355224609375f,-0.0630999827623366371653901296667754650115966796875f,0.80729999740123759011112269945442676544189453125f,0.413700007843971206966671161353588104248046875f,0.4359000113964075495687211514450609683990478515625f,1.02489997634887686217552982270717620849609375f,-0.40450000214576675716671161353588104248046875f,-0.37369998668432202748590498231351375579833984375f,0.2946000243067743440406047739088535308837890625f,1.66920000301599458225609851069748401641845703125f,-0.194000026643276868298926274292171001434326171875f,0.756600038075447400842676870524883270263671875f,0.73320000939369123216238222084939479827880859375f,0.1689999938607211760199788841418921947479248046875f,0.011099989557266098927357234060764312744140625f,-0.206700015532970127196676912717521190643310546875f,-0.53510001232624038181029391125775873661041259765625f,-0.1883000065207482975893071852624416351318359375f,-0.19859999309629206010185953346081078052520751953125f,0.2089999852180479678054325631819665431976318359375f,-0.86419998513460161149168925476260483264923095703125f,0.36899999780952918992937838993384502828121185302734375f,0.750499988615512592105005751363933086395263671875f,0.43269999440312378879980315105058252811431884765625f,0.266699991914630007983078030520118772983551025390625f,-0.29499998886138190101746658910997211933135986328125f,0.291999996781349135943628425593487918376922607421875f,-0.7491999903500079494023111692513339221477508544921875f,0.512000044465064974019696819595992565155029296875f,0.04420001448988930548011921928264200687408447265625f,-0.536199964518845550998094040551222860813140869140625f,-0.801500006079674420789160649292171001434326171875f,-0.7516000397145745637317304499447345733642578125f,-0.6594999743178491247164174637873657047748565673828125f,0.9530000166296961339185145334340631961822509765625f,-0.258799973380565706548850357648916542530059814453125f,0.0429999983012676612048608149052597582340240478515625f,-0.3965000297725200795184719027020037174224853515625f,-0.445599999153614145797064338694326579570770263671875f,0.3228999835908414883789419036475010216236114501953125f,0.9920000373125077430813689716160297393798828125f,0.88689998832941085993297747336328029632568359375f,1.09659999393075668905339625780470669269561767578125f,-0.646999989926815288754369248636066913604736328125f,-0.26110000162720670147109558456577360630035400390625f,-0.691799967584014297017347416840493679046630859375f,-1.17249997735023470113446819595992565155029296875f,0.20980000008344656947656403644941747188568115234375f,-0.7530999790370465074573758101905696094036102294921875f,-0.2669999985694886390774627216160297393798828125f,-0.542399981862306646007709787227213382720947265625f,-0.6007999772772194546632817946374416351318359375f,0.02700003343820611689807265065610408782958984375f,0.763799994242191360172000713646411895751953125f,-0.458199997621774546274764361442066729068756103515625f,-0.786499991774558537827033433131873607635498046875f,0.29899999341368665994878028868697583675384521484375f,-0.713700007583200513039400902925990521907806396484375f,0.6950000023841855778528042719699442386627197265625f,-0.293599998050928068238363266573287546634674072265625f,0.1408000035032628272091415055911056697368621826171875f,-0.1204999909996988805005457834340631961822509765625f,0.1938999958395957179391189129091799259185791015625f,-0.85540001488327987999582546763122081756591796875f,-1.272499986141919947613132535479962825775146484375f,0.6257000196754933796228215214796364307403564453125f,0.5949999988079071044921875f,-0.5800000131130218505859375f,0.00399998939037260470286128111183643341064453125f,0.8366999902009961687099348637275397777557373046875f,1.1840999523937707493814741610549390316009521484375f,-0.27369999517798415666902656084857881069183349609375f,0.051699999141693098181349341757595539093017578125f,-1.388800001692771957095828838646411895751953125f,-0.05880000079870217888355909963138401508331298828125f,-0.03439996001720402318824199028313159942626953125f,1.3673000081300727970301522873342037200927734375f,0.5369000041246412280315780662931501865386962890625f,-0.1847000470638278102342155762016773223876953125f,-2.5209999957084647803640109486877918243408203125f,0.12560003581047141096860286779701709747314453125f,0.5654999823570250327975372783839702606201171875f,-0.7890000078678127692910493351519107818603515625f,0.254000025749206770342425443232059478759765625f,0.67430002726316473626866354607045650482177734375f,1.52980000187158537983123096637427806854248046875f,-0.3652000183939936306387608055956661701202392578125f,-0.3382000126719475474601495079696178436279296875f,-0.46760000129938106283589149825274944305419921875f,-0.47580003006458326808569836430251598358154296875f,0.228800005269050643619266338646411895751953125f,0.3327999974906443725330973393283784389495849609375f,-0.401200019019842368805939258891157805919647216796875f,0.84120001708269143136931234039366245269775390625f,0.618300014269351549955899827182292938232421875f,-0.675900029277801905891465139575302600860595703125f,-0.14100003683567052803482511080801486968994140625f,-1.803299964499473162504727952182292938232421875f,0.1623000218391421611840996774844825267791748046875f,-0.19539998493194588036203640513122081756591796875f,1.77219997136592866127102752216160297393798828125f,0.077000004529953258725072373636066913604736328125f,0.195500000275671492744322677026502788066864013671875f,-0.072799999949336058335802590590901672840118408203125f,-0.068199998813867512836850437452085316181182861328125f,0.138700016784667923275264911353588104248046875f,0.38999996948987292189059417069074697792530059814453125f,-0.180000021904707097064601839520037174224853515625f,-0.296700000184774381750685279257595539093017578125f,0.7087999897718428865545092776301316916942596435546875f,-0.4429999799728392417819122783839702606201171875f,-0.423400009906291785455323406495153903961181640625f,-0.3369999894797803818846659851260483264923095703125f,-0.26749998018145593192684827954508364200592041015625f,0.372799987059831750002558692358434200286865234375f,-0.7548999669611458074314214172773063182830810546875f,0.04649998551607126273665926419198513031005859375f,-0.768300000709295272116605701739899814128875732421875f,0.21109999015331271099427112858393229544162750244140625f,-0.6865000013113018439980805851519107818603515625f,0.91569998644590366865259056794457137584686279296875f,-0.3217000133350487178773846608237363398075103759765625f,-0.01400003826618245739155099727213382720947265625f,-1.261299981725215957339969463646411895751953125f,0.537400005145370673886873191804625093936920166015625f,-0.74800000917911546594041283242404460906982421875f,-0.443899997627735132255111238919198513031005859375f,-0.332999984480440769374354204046539962291717529296875f,-0.0289999801516529487344087101519107818603515625f,0.55730000038146965124497000942938029766082763671875f,-0.6755999988183376547823399960179813206195831298828125f,0.877500019967555999755859375f,-0.3437999982655046693480471731163561344146728515625f,0.427899999646842221778797465958632528781890869140625f,0.35500001430511485267516036401502788066864013671875f,-0.0039999987781047519064259176957421004772186279296875f,-0.13859999901950359468827400633017532527446746826171875f,0.260000023543835112604938331060111522674560546875f,-0.0652999879539013505791444913484156131744384765625f,-0.5764000002354381368974145516403950750827789306640625f,-0.09850001776218419990982511080801486968994140625f,-0.87589997205734260177223404753021895885467529296875f,0.3755000022128223147177550345077179372310638427734375f,0.034000001311301986106627737171947956085205078125f,1.006399982726573938407454988919198513031005859375f,-0.1751999952226876988703452298068441450595855712890625f,-0.0550000013411044985645048654987476766109466552734375f,-0.10009999776035549690078596540843136608600616455078125f,-0.11599999887496230399630547935885260812938213348388671875f,-0.9320000218153001725340800476260483264923095703125f,0.286200013171136635747870968771167099475860595703125f,0.6145999913007020865052254521287977695465087890625f,-0.80989999795556055772749459720216691493988037109375f,-0.6829000104129312465062184855923987925052642822265625f,-0.523599975214898538666830063448287546634674072265625f,0.390099984610080330327264164225198328495025634765625f,0.3626999891132112185943014992517419159412384033203125f,-0.827900029635429746122099459171295166015625f,-0.5023000283956520206629647873342037200927734375f,-0.3915000069737430976601899601519107818603515625f,-0.162599982821941324573344900272786617279052734375f,-1.201800012063980460652601323090493679046630859375f,-0.36840003553628974941602791659533977508544921875f,1.396899997866153597669836017303168773651123046875f,0.585499959111213996720834984444081783294677734375f,-0.883700014102458908382686786353588104248046875f,-1.3702000192880632312153466045856475830078125f,-0.613399996495246568883885629475116729736328125f,1.089000030517578210265128291212022304534912109375f,-0.80910001717656854225424467585980892181396484375f,-0.35249998066574317334698207559995353221893310546875f,0.031299989511072379144707156228832900524139404296875f,-0.029499991975724615400622496963478624820709228515625f,-0.615199991869926687826364286593161523342132568359375f,-0.1328000090390444309917938880971632897853851318359375f,0.94480004508495341042362269945442676544189453125f,0.4474999612569803275619051419198513031005859375f,-0.9109000274896619231412842054851353168487548828125f,-0.90139998242855057242195471189916133880615234375f,-0.35709997930526782283777720294892787933349609375f,0.48129999573230719533967203460633754730224609375f,0.5956000094354152452069683931767940521240234375f,-1.3360000240206719723801143118180334568023681640625f,-0.346800014150142832392020864062942564487457275390625f,-0.15250004053115873148271930404007434844970703125f,-2.2886000086307518586181686259806156158447265625f,0.0306000134587289807086563087068498134613037109375f,0.876400035774707930613658390939235687255859375f,-0.3025000163912778816666104830801486968994140625f,0.2223000203490261839078812045045197010040283203125f,-0.0955999833583831559735699556767940521240234375f,-1.93570000879764592127685318700969219207763671875f,0.1395000027418138444090800476260483264923095703125f,-0.279900001233816198009662912227213382720947265625f,0.4915999934092163403676067900960333645343780517578125f,0.444999997988343221066998012247495353221893310546875f,-0.78249998442828661016079649925814010202884674072265625f,-0.16619999830722809175398424486047588288784027099609375f,-0.0899999964237212513040731209912337362766265869140625f,0.4328999742403627504216956367599777877330780029296875f,0.239199986773729389444298476519179530441761016845703125f,1.1299999958276742972884676419198513031005859375f,-0.237800004345178450648745638318359851837158203125f,0.7738000047847624074393024784512817859649658203125f,0.6105000044107438128548892564140260219573974609375f,-0.73200001451373086780449739308096468448638916015625f,0.40169997950941327413687531588948331773281097412109375f,0.1210000207424162255165356327779591083526611328125f,-0.26200000259280198378064596909098327159881591796875f,-0.93279997435659189886081321674282662570476531982421875f,-0.41000001847744016458818805404007434844970703125f,0.3115999877095223524747780174948275089263916015625f,0.87439999615251995379594518453814089298248291015625f,0.560500008396804361154863727279007434844970703125f,-0.018399992322921665532931001507677137851715087890625f,0.6446999960646031535560496195103041827678680419921875f,0.1994999938011170481644285246147774159908294677734375f,-0.3662999977812171348290348760201595723628997802734375f,0.154499987393617754793950780367595143616199493408203125f,0.49249999225139617919921875f,-0.25610001548528682491223662509582936763763427734375f,-0.5731000152096148436697831130004487931728363037109375f,0.038499989151954849830872262828052043914794921875f,0.1498999930381774703391783987171947956085205078125f,0.6172999742671849698893993263482116162776947021484375f,0.02099999585747713837236005929298698902130126953125f,-0.9444000114262103284801241898094303905963897705078125f,-0.13420000577270985164801686551072634756565093994140625f,0.08549998593330343510388047434389591217041015625f,-1.17629999498724924222869958612136542797088623046875f,0.4945000082552428732896032670396380126476287841796875f,-0.939500031650066347310712444595992565155029296875f,-0.25690001276731511126172335934825241565704345703125f,-0.636300012533366210476515334448777139186859130859375f,-0.101500033497810449034659541212022304534912109375f,-0.96290001205205921763763399212621152400970458984375f,0.3696999740853907923110455158166587352752685546875f,0.59400000265240660013432716368697583675384521484375f,-0.2178999992370602711844185250811278820037841796875f,-0.8057000213146210398917901329696178436279296875f,-0.13689998147487614232886699028313159942626953125f,0.9705000264644620955323262023739516735076904296875f,-0.4607000076055527415519463829696178436279296875f,-0.32829999251365649115541600622236728668212890625f,0.7349000380396848441932888817973434925079345703125f,0.49650000056624410404992886469699442386627197265625f,0.16439998750984674558139886357821524143218994140625f,-0.086399999937415206119339927681721746921539306640625f,1.2480000020265578086764435283839702606201171875f,0.0395999620199205537574016489088535308837890625f,-0.267599987292289398510547471232712268829345703125f,-0.40790000963062045258311627549119293689727783203125f,-0.152799978640675515606517365085892379283905029296875f,0.83200002561509600607081438283785246312618255615234375f,0.0582000114798546519523370079696178436279296875f,-1.263599997901916793807686190120875835418701171875f,-0.21200000929832452101209128159098327159881591796875f,0.5755999987065791856366558931767940521240234375f,1.396899986392259762624235008843243122100830078125f,-0.304700007277727280552426236681640148162841796875f,-0.321600024664402184271239093504846096038818359375f,-1.260000009536742737736858543939888477325439453125f,-0.4102000237584118025324642076157033443450927734375f,0.052499996442347818259577252320013940334320068359375f,-0.4313999878302217094727666335529647767543792724609375f,0.261400004445016154175362999012577347457408905029296875f,0.401399997031688826609752140939235687255859375f,-1.011299977850914189048125990666449069976806640625f,0.34610000475645019690773551701568067073822021484375f,-0.5458000085324048455959200509823858737945556640625f,-0.13069997497200969149844240746460855007171630859375f,0.9601000223100182306978922497364692389965057373046875f,-0.2758000050306321782045415602624416351318359375f,-0.76820002012252786016688332892954349517822265625f,-0.5094000118732449067238121642731130123138427734375f,-0.4535000182837247706402195035479962825775146484375f,0.33080003170371075071898303576745092868804931640625f,0.4312000222980978403342078308924101293087005615234375f,0.13280002237558363731295685283839702606201171875f,0.507699966835975402545955148525536060333251953125f,0.48990000168085057197231435566209256649017333984375f,-0.9863000192575157587526746283401735126972198486328125f,0.02709998805969970891993625627947039902210235595703125f,-0.675500000238418607523271930404007434844970703125f,0.41419999884366998088580658077262341976165771484375f,0.40590000707507112309713193099014461040496826171875f,-0.621000029683113297096497262828052043914794921875f,0.16500000014901150535706619848497211933135986328125f,0.2363999970689416585400977055542171001434326171875f,-0.255499992415308962989684005151502788066864013671875f,-0.0391000037282704937524613342247903347015380859375f,-0.37249999023973945355692194425500929355621337890625f,-0.574000003695488203447894193232059478759765625f,-0.3140000078678131245624172152020037174224853515625f,-0.861999981135129988984999727108515799045562744140625f,-0.7885000055059789847433648901642300188541412353515625f,0.280900001673400401269731219144887290894985198974609375f,0.147300002504885174658966207061894237995147705078125f,0.2419999833703041502985797706060111522674560546875f,-0.755000000298023277167658307007513940334320068359375f,0.2175999811083080270890377505565993487834930419921875f,-0.034499974578618974874189007095992565155029296875f,0.52649999982118611541181962820701301097869873046875f,-0.59369997942000640733795080450363457202911376953125f,0.7835000147819517479774731327779591083526611328125f,-0.746000000327825318890972994267940521240234375f,0.53670000350028235214239202832686714828014373779296875f,-0.37949999735876904782827523376909084618091583251953125f,-0.1508999977059662100931092254540999419987201690673828125f,-0.020499999064952145744200606714002788066864013671875f,-0.7459999948143956061130666057579219341278076171875f,-0.498699994283914538328872367856092751026153564453125f,-0.1947999988913535762691253694356419146060943603515625f,-0.854499999687075462162511030328460037708282470703125f,0.36309999907910828209622877693618647754192352294921875f,0.21710000161975617771759061724878847599029541015625f,0.876999987244605705427602515555918216705322265625f,-0.52720000295639035670092198415659368038177490234375f,0.4405999915540217415355073171667754650115966796875f,-0.231500005111098250409895626944489777088165283203125f,0.41090001743137827272533968425705097615718841552734375f,0.18350000487267970061822097704862244427204132080078125f,-0.7439999926686287068378078402020037174224853515625f,0.410099967771768714186464421800337731838226318359375f,-0.2875999976485967835060364450328052043914794921875f,0.23999999463558185919254128748434595763683319091796875f,-0.0473000163257129013771873360383324325084686279296875f,-0.203500014781951676923199556767940521240234375f,-1.783999937534332502764300443232059478759765625f,0.06299999248981436039684922434389591217041015625f,-0.5814000245988371062821897794492542743682861328125f,-1.2252000073671336366487594204954802989959716796875f,-0.25560000869035714998744879267178475856781005859375f,-0.2809000169098379018350897240452468395233154296875f,-1.8224000150918957530166153446771204471588134765625f,-0.16140000895261774616074035293422639369964599609375f,0.2363000043563545649050183783401735126972198486328125f,0.7765000127851962030689492166857235133647918701171875f,0.08810001111179610600032674483372829854488372802734375f,-0.79300002586841600304978783242404460906982421875f,-0.53420000510215714939477038569748401641845703125f,-0.432800037276745541703348862938582897186279296875f,0.006500012338161553770987666212022304534912109375f,-0.473899998521805088103064917959272861480712890625f,-0.4121000240087511201636516489088535308837890625f,0.2088000068895519145684147588326595723628997802734375f,0.2531999972864984460585446868208236992359161376953125f,-0.05460000091195105209607163487817160785198211669921875f,0.64090001270025975799171646940521895885467529296875f,0.6262999924167989007628420949913561344146728515625f,-0.097499999217689037322998046875f,0.656200000095367386165889911353588104248046875f,0.350299982440471779909785254858434200286865234375f,-0.57790001056194295614432121510617434978485107421875f,1.1315999881565570461816605529747903347015380859375f,0.1670999912261965647530814749188721179962158203125f,-0.30489999333620065868899473571218550205230712890625f,0.6561000075131655506766037433408200740814208984375f,-0.69930000625252741741633144556544721126556396484375f,-0.7895000356733803403841420731623657047748565673828125f,0.1392999938845633778328192420303821563720703125f,0.947599993848800892237704829312860965728759765625f,-0.07999999016523329231631578295491635799407958984375f,0.712300033834576851177189382724463939666748046875f,0.3653000062078231024997876374982297420501708984375f,0.10030000754892842707022282411344349384307861328125f,-0.3519000037707387118501856093644164502620697021484375f,1.1715000393092631636449141296907328069210052490234375f,0.49870002680569924802256309703807346522808074951171875f,-0.22399997270107263602767488919198513031005859375f,-0.6947000062346457838202695711515843868255615234375f,0.06629999144822373846608343228581361472606658935546875f,-0.002600001671910268097320795277482829988002777099609375f,0.547399995310604570164514370844699442386627197265625f,-0.07589997861385366917374994955025613307952880859375f,-0.5811999959602951459913811049773357808589935302734375f,-1.5908000118851663984287370112724602222442626953125f,-0.1614999931156635426532375277020037174224853515625f,-0.52589999791085728642059393678209744393825531005859375f,0.07770000319480896422419391456060111522674560546875f,0.5198999636828904868934841942973434925079345703125f,-0.197299998518824626358991736196912825107574462890625f,0.758299995791912095910447533242404460906982421875f,0.5378999789714811186058796010911464691162109375f,0.15619999081939450746858710772357881069183349609375f,1.04500003159046173095703125f,-0.204200003872812008154369323165155947208404541015625f,0.59280001507401436366961888779769651591777801513671875f,1.068000034287572841407154555781744420528411865234375f,-0.32910000186562538004864109097979962825775146484375f,0.8919999948814509860284260867047123610973358154296875f,0.7124000374436381122222883277572691440582275390625f,-0.54369998847246137074762373231351375579833984375f,0.2582999898314477515981479882611893117427825927734375f,-0.1257000120759013128690639859996736049652099609375f,0.297299995166062647200533319846726953983306884765625f,0.5745000151842829527737421813071705400943756103515625f,0.5181000396847732503147199167869985103607177734375f,-0.6026999800980092913960106670856475830078125f,0.706700002717971731414081659750081598758697509765625f,-0.128899985408782669082938809879124164581298828125f,-0.04950001153349870008923971909098327159881591796875f,0.389199998284876169662993561360053718090057373046875f,0.932000031054020183773900498636066913604736328125f,0.1888999894320964045846267254091799259185791015625f,0.3608000090911982482566600083373486995697021484375f,-0.11239997515678368245062301866710186004638671875f,0.40529997007250795348909377935342490673065185546875f,0.286299993050098233737799091613851487636566162109375f,1.414700040209293518955746549181640148162841796875f,-0.07199998366832716101271216757595539093017578125f,-0.913300013263523169371183030307292938232421875f,-0.65090005639791570501984097063541412353515625f,0.3572999941974874804628825586405582726001739501953125f,-0.894299985800683483461170908412896096706390380859375f,0.10369999517798422772330013685859739780426025390625f,0.334800012004375258101163126411847770214080810546875f,0.26939999734461306246657841256819665431976318359375f,0.39660000648498539010944341498543508350849151611328125f,0.0685999897807836322982666388270445168018341064453125f,-0.435800001193583030900668973117717541754245758056640625f,1.09440003176629563341748507809825241565704345703125f,0.56020000654757018399010348730371333658695220947265625f,-0.73000001251697543835916803800500929355621337890625f,0.550300030422210539882144075818359851837158203125f,0.071200006651878311458858661353588104248046875f,-0.590800007042288743974722819984890520572662353515625f,-0.628800000202655695602516061626374721527099609375f,-0.287799991679191347060395855805836617946624755859375f,0.4350999987661838819263948607840575277805328369140625f,0.532700005653500507918352013803087174892425537109375f,0.5020000186860562596535828561172820627689361572265625f,-0.02649999710172412648034878657199442386627197265625f,-0.389099981749057466373642455437220633029937744140625f,-0.05919997947216071310094775981269776821136474609375f,0.472899979232251632677019870243384502828121185302734375f,-0.54980000932216654963013979795505292713642120361328125f,-0.5112000083655117155245761750848032534122467041015625f,0.145500009365379823211839038776815868914127349853515625f,0.448500002413988074767559055544552393257617950439453125f,0.22979999557584529501497172532253898680210113525390625f,-0.3353999984174966808581075383699499070644378662109375f,-0.468400004541873915542282702517695724964141845703125f,-0.183399993216991408218063952517695724964141845703125f,0.398499999880790711781486379550187848508358001708984375f,0.1083000103950499948268770822323858737945556640625f,-0.286800023984909557839273475110530853271484375f,-0.5454000113815069550327052638749592006206512451171875f,0.8346999984860421495369564581778831779956817626953125f,0.529199988561868561731671434245072305202484130859375f,-0.4953000014021993191448700599721632897853851318359375f,0.1033000002622603830104708322323858737945556640625f,0.44880002672672247854279703460633754730224609375f,0.4535999955922365156624209703295491635799407958984375f,0.9705000300407409241643108543939888477325439453125f,0.7324000186681747237571471487171947956085205078125f,-0.4984000139296054232573851550114341080188751220703125f,0.7463999940514565167859473149292171001434326171875f,0.48610007210969996549465577118098735809326171875f,-0.5495999813914298481876130608725361526012420654296875f,0.197499987967312495840133124147541821002960205078125f,0.15469999058842631711740978062152862548828125f,0.26849999351054421037332531341235153377056121826171875f,0.09630000165551899637961241751327179372310638427734375f,-0.21089999541491266654702485539019107818603515625f,0.022699994738399897897807022673077881336212158203125f,0.62610000877976401678637330405763350427150726318359375f,-0.09149998126923997165960145139251835644245147705078125f,0.4325999686658388299065336468629539012908935546875f,0.406099993021786342950463222223334014415740966796875f,0.132099989736079947988400817848742008209228515625f,-0.0874999965727329964693126385100185871124267578125f,-0.511199977333844035598531263531185686588287353515625f,-0.187199997591971811772282308083958923816680908203125f,-0.42180000036656839501603144526598043739795684814453125f,0.77139999211430543279988114591105841100215911865234375f,0.63500002481043349433775802026502788066864013671875f,-0.268399986362457543265236381557770073413848876953125f,0.14640000098049643639797068317420780658721923828125f,0.577199993270635758335629361681640148162841796875f,0.45539998116940261230212172449682839214801788330078125f,-0.061900002858042757480916407075710594654083251953125f,0.47650001125782726063562222407199442386627197265625f,-0.002300000511854805029798853865941055119037628173828125f,0.01380000256821511950278136282577179372310638427734375f,-0.412200000851601355833508932846598327159881591796875f,-0.114499996483325983120948876603506505489349365234375f,-0.118399998991191413022505685148644261062145233154296875f,0.259999999254941993598322369507513940334320068359375f,0.65359999119639411446769372560083866119384765625f,-0.517500010766088536229290184564888477325439453125f,-0.8427000523686416499913320876657962799072265625f,-0.2009999823719266220933832300943322479724884033203125f,0.12140000239610670984546914041857235133647918701171875f,-0.63739999475181097210452207946218550205230712890625f,-0.75899998328089690602382688666693866252899169921875f,0.5638999791130425709440032733255065977573394775390625f,0.7002999937653537898540889727883040904998779296875f,-0.48239999229311880668547019013203680515289306640625f,-0.22229998358041047623601116356439888477325439453125f,-1.32270003508329381247676792554557323455810546875f,-0.63399996793270130268638240522705018520355224609375f,0.221399997963011241353115110541693866252899169921875f,-0.27470001092851170643172054042224772274494171142578125f,-0.14700000095367415919866971307783387601375579833984375f,0.19350000811368228692543880242737941443920135498046875f,-1.0843000439524654865408592741005122661590576171875f,-0.481200006949901393227264634333550930023193359375f,0.7809000224605202600258735401439480483531951904296875f,0.1844000029325485456865862943232059478759765625f,0.166600001120567497991942218504846096038818359375f,-0.1150000004842877121546962371212430298328399658203125f,0.0717000152349474006996388197876513004302978515625f,0.30320003025531772067324709496460855007171630859375f,0.3621999919295311176625773441628552973270416259765625f,0.3478999882847073621405797894112765789031982421875f,0.4481999958336351230769878384307958185672760009765625f,0.068099996396899176165362632673350162804126739501953125f,-0.427500004470348304863591692992486059665679931640625f,-0.02919998602867135417682220577262341976165771484375f,0.540699996988475373171922910842113196849822998046875f,-0.53779998542070384104363256483338773250579833984375f,-0.17159997876882560063904747948981821537017822265625f,0.46709998768717053696519769800943322479724884033203125f,0.4214000059723854718640723149292171001434326171875f,0.3241999979496004158363575697876513004302978515625f,-0.2996000032588839179226170017500407993793487548828125f,-0.6595999876499174519040025188587605953216552734375f,-0.31379998500347117129649632261134684085845947265625f,0.429599991188943430842073212261311709880828857421875f,-0.3470000209212305009032206726260483264923095703125f,-0.131599997246265143502341743442229926586151123046875f,0.365000016987323760986328125f,0.4857000338315966558866421109996736049652099609375f,-0.101000018656254297866325941868126392364501953125f,-0.8197000250101089680043742191628552973270416259765625f,-0.37730003495216379860721644945442676544189453125f,0.00999998778104771446351151098497211933135986328125f,0.4169000096753239237301613684394396841526031494140625f,-0.41219999908208837524625778314657509326934814453125f,0.05460001775026324111195208388380706310272216796875f,0.53669999303221704867183916576323099434375762939453125f,1.2317999876260756764168036170303821563720703125f,0.7747999908447269490352482534945011138916015625f,-0.65479998182952403595891155418939888477325439453125f,0.2221000281810763254952689749188721179962158203125f,-0.72180003501176859259658158407546579837799072265625f,0.01229999673068526977459669069503434002399444580078125f,-0.04090000130087141638757231021372717805206775665283203125f,0.20350000766664744222822491792612709105014801025390625f,-0.17659999501109080455307775991968810558319091796875f,-0.7910000135153529843279329725191928446292877197265625f,-0.1858000288724905857407065923325717449188232421875f,-0.161699993479251791228534784750081598758697509765625f,-0.38329998178482072024308990876306779682636260986328125f,-0.78250000566244093391787828295491635799407958984375f,-0.49299999442696584850409635691903531551361083984375f,0.76890000139027814274328420651727356016635894775390625f,1.07010003229379702105461547034792602062225341796875f,-0.52929996445477078825803118888870812952518463134765625f,0.1687000009894370489149650893523357808589935302734375f,-0.123699984672665408425018540583550930023193359375f,-0.371799987626075800761782375047914683818817138671875f,0.0323999963164329773945837587234564125537872314453125f,0.4295999982297420416443856083787977695465087890625f,-0.21919999705553028235271995072253048419952392578125f,-0.36489998994618655370203441634657792747020721435546875f,-0.06170002417564379726400147774256765842437744140625f,-0.2492999858379363331550848670303821563720703125f,-0.6964999704658987322858365587308071553707122802734375f,-1.5013000159978862058096638065762817859649658203125f,-0.072299998593330361273956441436894237995147705078125f,-0.6252999702960255756778451541322283446788787841796875f,-0.3317000065922730556167152826674282550811767578125f,-0.07340000737309448908263220801018178462982177734375f,0.344599987649917560705858932124101556837558746337890625f,0.859000026047229692238715870189480483531951904296875f,0.6716999948203561388027083012275397777557373046875f,-0.39629998820722089902801599237136542797088623046875f,-1.01910003390312198234823881648480892181396484375f,-0.429799970430135847010433280956931412220001220703125f,-0.641200007098913005165741196833550930023193359375f,-0.66080002499818846217749523930251598358154296875f,-0.546299979639053301383455618633888661861419677734375f,-0.487799984042346590484839907730929553508758544921875f,-0.401400016105174728409110684879124164581298828125f,-0.2589999968409539832236987422220408916473388671875f,0.731999999240040555292807766818441450595855712890625f,-0.086399951434135147110282559879124164581298828125f,0.715499995917081577090357313863933086395263671875f,-0.1558999917268752977150825245189480483531951904296875f,-0.76649999922513956107650301419198513031005859375f,-0.2449999803304674372839144780300557613372802734375f,0.747199991966783994001843893784098327159881591796875f,0.137400014197826436657123849727213382720947265625f,-0.1115999960541724789209183654747903347015380859375f,-0.189900000973045823826623745844699442386627197265625f,-0.2023000078320507100215763784945011138916015625f,-0.43220002024173709287424571812152862548828125f,-0.34359998713582751150852345745079219341278076171875f,-0.20780000300407408531100372783839702606201171875f,-0.024399991309642654613298873300664126873016357421875f,0.268000002846121798683043380151502788066864013671875f,0.1778000038608908983661649472196586430072784423828125f,0.17600000844895846707771625005989335477352142333984375f,0.01480000019520522069882417781627736985683441162109375f,0.2319000005483626836166877183131873607635498046875f,0.27100000822544156875437693088315427303314208984375f,-0.0717999921336769375557196326553821563720703125f,-0.871799991798400952802694519050419330596923828125f,-0.53759998371601103173134106327779591083526611328125f,0.543599996113777184092441530083306133747100830078125f,0.1768999896705150121078986558131873607635498046875f,0.378600010120868635254964829073287546634674072265625f,0.23400000350922345315751726957387290894985198974609375f,-0.31910000573992736150330529198981821537017822265625f,-0.1517999934375284798449001755216158926486968994140625f,0.285300005909800535874865090590901672840118408203125f,0.3665000209808351172569018672220408916473388671875f,0.170800008904933964259953427244909107685089111328125f,-0.2428000085175037181528523433371447026729583740234375f,0.552600013518333543771632321295328438282012939453125f,0.1363999910712241359078689129091799259185791015625f,-0.5450000052154064800191690665087662637233734130859375f,-0.64160000745356082774151218472979962825775146484375f,-0.32239999475181091437292479895404540002346038818359375f,0.464300002899765973385370898540713824331760406494140625f,-0.270499995917081736962472859886474907398223876953125f,-0.1233999958992004764013472595252096652984619140625f,0.185499994277954105559302888650563545525074005126953125f,-0.8869999869465825526049229665659368038177490234375f,-0.47039998403787652847540812217630445957183837890625f,0.6626999819979071393305503079318441450595855712890625f,-0.995800022315979305176369962282478809356689453125f,-0.273199995088577285429209950962103903293609619140625f,-0.28649999222159383549524136469699442386627197265625f,0.0251999916836620485582898254506289958953857421875f,0.753599989259242875050404109060764312744140625f,0.395999976336955938904793583787977695465087890625f,0.7904999914839863084381477165152318775653839111328125f,1.1193000332236291427534524700604379177093505859375f,0.6138999745309359212797062355093657970428466796875f,-0.832999994985759162346994344261474907398223876953125f,-1.031800032627582908162366948090493679046630859375f,-0.155399999386072185103557785623706877231597900390625f,0.126800002303719405727377989023807458579540252685546875f,0.382000014066696369496867191628552973270416259765625f,-0.153200000266730806952608645588043145835399627685546875f,0.351399999602138912724313968283240683376789093017578125f,0.355400016187131573275337359518744051456451416015625f,-0.324099997097253567091001968947239220142364501953125f,-0.4589999667033557440021240836358629167079925537109375f,-0.9593999892234794657497332082130014896392822265625f,0.18279998273849518142242231988348066806793212890625f,-0.06540000273883339598768316136556677520275115966796875f,0.75779999167919154245964818983338773250579833984375f,-0.5586999834060668579382991083548404276371002197265625f,0.281199989254772741986698747496120631694793701171875f,0.21200000289082510107618873007595539093017578125f,-0.01130000698268407433033644338138401508331298828125f,0.2128999947667121472871798459891579113900661468505859375f,0.6349000140488147980732946962234564125537872314453125f,0.047199993717670363224669927149079740047454833984375f,0.346900009451806379257021717421594075858592987060546875f,0.184100016319752146642940715537406504154205322265625f,0.298199992853402928716377573437057435512542724609375f,0.24180000237822529829401219103601761162281036376953125f,-0.13499999150633801292542557348497211933135986328125f,-0.016200000803172631691495553241111338138580322265625f,-0.1704999965503812831002505845390260219573974609375f,0.35590000491440310081969755628961138427257537841796875f,-0.0390999954581261466302066764910705387592315673828125f,-0.234099991881847435681862634737626649439334869384765625f,-0.102500006258487541543900078977458178997039794921875f,0.04140000269412968947335684788413345813751220703125f,0.08809999375194321746818104656995274126529693603515625f,0.99609999164342877264743947307579219341278076171875f,0.22169999586343802633336963481269776821136474609375f,-0.057399996949732301487756558344699442386627197265625f,-0.2821000156641009226632377249188721179962158203125f,-0.142999993979930906107256305404007434844970703125f,0.3353000134348869121225789058371447026729583740234375f,-0.363799995732307479556766338646411895751953125f,-0.32879999305009821597423069761134684085845947265625f,0.1671000043019652547826581212575547397136688232421875f,-0.28269998745545754648134106901125051081180572509765625f,-0.357399996726214641551422346310573630034923553466796875f,-0.0604999996237456372227825340814888477325439453125f,-0.2657000049233435579054685149458236992359161376953125f,-0.335200013178587052919965572073124349117279052734375f,-0.0085999992430210447924565642097149975597858428955078125f,1.207700021672248880122424452565610408782958984375f,0.83420000957250550754906726069748401641845703125f,-0.570500014245510111976500411401502788066864013671875f,0.723400006628036607025933335535228252410888671875f,0.577800004494189778370127896778285503387451171875f,-0.221700009721517599103890461265109479427337646484375f,-1.016100047671795181258858065120875835418701171875f,-0.462600016498565747014026783290319144725799560546875f,0.7215000181496142861448106486932374536991119384765625f,0.01469999645575870150793207358219660818576812744140625f,0.050400000540912148760952504744636826217174530029296875f,0.01710000219717622282900038044317625463008880615234375f,0.06839999068379398750039399601519107818603515625f,0.1830000117123129488305721679353155195713043212890625f,0.23460001130551100256838026325567625463008880615234375f,-0.830500038385391547990366234444081783294677734375f,-0.28100000137090663798744571977294981479644775390625f,0.7271000199854373846619637333787977695465087890625f,-0.630900020337104905365777085535228252410888671875f,-0.433400000369548621392823406495153903961181640625f,0.458900016403198218739589719916693866252899169921875f,1.0417000363945962959633106947876513004302978515625f,0.499600005829333948526027597836218774318695068359375f,-0.73480001915693282654729046043939888477325439453125f,0.3871000015079975042908699833787977695465087890625f,0.0223999744117258092757083431934006512165069580078125f,-0.61010000476390124912740020590717904269695281982421875f,-0.6901000179886818131080872262828052043914794921875f,-0.58840002272129066085426529753021895885467529296875f,0.15340000461637970374795258976519107818603515625f,-0.0793000266671182174604837200604379177093505859375f,-0.2012000293016438234872111934237182140350341796875f,0.0358999841272833464245195500552654266357421875f,0.807099963435530920463634174666367471218109130859375f,1.517900017118453348530238145031034946441650390625f,0.300999977827072218161674754810519516468048095703125f,0.359299997870623943896362106897868216037750244140625f,0.931300024342537113852813490666449069976806640625f,0.4126999755345288622976340775494463741779327392578125f,-0.222499988432973749841181643205345608294010162353515625f,-0.0325000022910533203912564204074442386627197265625f,0.4009999939948316605153877389966510236263275146484375f,0.1066999948576092815955007608863525092601776123046875f,0.456700013372302038305861060507595539093017578125f,-0.4054999878704546034668965148739516735076904296875f,0.633699983480572992533552678651176393032073974609375f,-0.227500017285347411188922706060111522674560546875f,-0.3878999819517137126467787311412394046783447265625f,0.59179998110681797385268509970046579837799072265625f,0.17739999512433968220648239366710186004638671875f,-0.14149998432397836722884676419198513031005859375f,-0.7221000201344487390286985828424803912639617919921875f,-0.142500029504299963178937105112709105014801025390625f,-0.0911999986611304602757144266433897428214550018310546875f,-0.134400003174692306284754295120364986360073089599609375f,-0.2292000126458706432686085463501513004302978515625f,-0.25710000337064276010323737864382565021514892578125f,0.79289999059438687201151196859427727758884429931640625f,1.1313000447571279760694551441702060401439666748046875f,-0.2994999921619889704516026540659368038177490234375f,-0.40490001270771014407756638320279307663440704345703125f,0.15229999024867968415719587937928736209869384765625f,0.2554999819099901259278340148739516735076904296875f,0.72149997784197328343225308344699442386627197265625f,1.74270004432201375266231480054557323455810546875f,0.08759999563693998680946606327779591083526611328125f,0.5824999996647239886016222953912802040576934814453125f,-0.101900001144409113607025574310682713985443115234375f,0.30920000592172147690916972351260483264923095703125f,-0.80079999080002295386293553747236728668212890625f,-1.5788000402867796712058634511777199804782867431640625f,0.438799999457597511565154491108842194080352783203125f,-0.6718999998405574825000030614319257438182830810546875f,-0.3283000163555147565830338862724602222442626953125f,-0.530899969226121815069063814007677137851715087890625f,0.4900999952644109480814904600265435874462127685546875f,-0.54120000203251805714899091981351375579833984375f,-0.541200018870830579231778756366111338138580322265625f,0.05279999473392960052109401658526621758937835693359375f,-0.7426999903798101598795255995355546474456787109375f,-0.50559999557733537045578486868180334568023681640625f,0.3516999903127551174719656046363525092601776123046875f,-1.32399998104572258483813129714690148830413818359375f,-0.9703999962568286008490758831612765789031982421875f,0.59239997813701628093241424721782095730304718017578125f,1.3055000067949293907076935283839702606201171875f,0.803199998068809151163804926909506320953369140625f,-0.698800001767277745301498725893907248973846435546875f,-0.18889998451471345930485767894424498081207275390625f,-0.4488000025868412734553203335963189601898193359375f,-0.28829999717026943972797425885801203548908233642578125f,0.63299997729063051110642845742404460906982421875f,0.437599973881244608264751150272786617279052734375f,-0.374899993522465280193500802852213382720947265625f,-0.62570000581741336276309084496460855007171630859375f,-0.20399998164176924575485827517695724964141845703125f,0.64380000452399255284063883664202876389026641845703125f,-0.25029999145567405349765977007336914539337158203125f,-0.08719998529851424873271525939344428479671478271484375f,0.277399996726214903564056157847517170011997222900390625f,0.46709999426230786045977083631441928446292877197265625f,0.426800005023181217023164890633779577910900115966796875f,-0.021499998088926108863461195142008364200592041015625f,0.57719999625086781946947667165659368038177490234375f,0.498800006163120368540830895653925836086273193359375f,-0.1285999944746494261238467515795491635799407958984375f,0.025199989113211973545958244358189404010772705078125f,0.2399999932944776315224544305237941443920135498046875f,0.1235999973803758589241397203295491635799407958984375f,1.0716999785780902954002158367075026035308837890625f,0.7365999921798707106290748924948275089263916015625f,-0.4917999791696667788443164681666530668735504150390625f,0.09619998996257805856657796539366245269775390625f,0.33800000649690620235787719138897955417633056640625f,0.3048999986633658121348844360909424722194671630859375f,1.170000012516975829157672706060111522674560546875f,0.9432000093936920848136651329696178436279296875f,-0.390000008046627044677734375f,-0.7438999928593634791695876629091799259185791015625f,0.14529999525547054872731678187847137451171875f,0.5090000017583362623696530135930515825748443603515625f,-0.377399983054399701387637833249755203723907470703125f,-0.0125999847590918534478987567126750946044921875f,0.2323999964654444028155921841971576213836669921875f,-0.43959998399913324806931314014946110546588897705078125f,0.06159999396801030258075115852989256381988525390625f,0.5643999930977816603672181372530758380889892578125f,-0.4951999851644039996045876250718720257282257080078125f,0.5671999895453456730365360272116959095001220703125f,-0.23419999571442584596070446423254907131195068359375f,0.8784999699667099815059145839768461883068084716796875f,0.0700999953389160879169139661826193332672119140625f,0.6883999632656578882006215280853211879730224609375f,-0.2217999888181687584420842540566809475421905517578125f,-0.0974000022768972684161781216971576213836669921875f,-0.2365999692320830405378728755749762058258056640625f,-0.17549999461323029237291848403401672840118408203125f,-0.90269998859167088767208042554557323455810546875f,-0.21420000346302980887003286625258624553680419921875f,0.27589999601095904768754962788079865276813507080078125f,1.00990001926422134914673733874224126338958740234375f,-0.0666000043615699865995338768698275089263916015625f,0.28450000642985084908787030144594609737396240234375f,0.67370002192556899256459246316808275878429412841796875f,-0.3381999909721316033284210789133794605731964111328125f,0.78839997486993684816525274072773754596710205078125f,0.7320000133775170780836560879833996295928955078125f,-0.3825999925076961627468108417815528810024261474609375f,0.509999994710087722893376849242486059665679931640625f,0.1932000205695629801994073204696178436279296875f,-0.48479996245801515453166530278394930064678192138671875f,-0.755999963596463597781394128105603158473968505859375f,-1.2311999927192918136142907314933836460113525390625f,-0.34689999831318818479530818876810371875762939453125f,0.68599997138232016169467897270806133747100830078125f,-0.01039998940229480695052188821136951446533203125f,-0.556799972724915104294041157118044793605804443359375f,0.3432999998524783880071709063486196100711822509765625f,0.3857000248908999395780483609996736049652099609375f,-0.65719997136592933628662649425677955150604248046875f,0.6828000190973277927497520067845471203327178955078125f,-0.1379999531507483112591216922737658023834228515625f,0.17749999925494197583475397550500929355621337890625f,-0.51810000321418048230270869680680334568023681640625f,-0.9203999956607820109866224811412394046783447265625f,-0.62399999594688448922852330724708735942840576171875f,0.4409999913126230186577458880492486059665679931640625f,0.2683999878525735738321600365452468395233154296875f,0.471200008738041020706077688373625278472900390625f,0.2431000040337443213189771995530463755130767822265625f,-0.383199995684623928582368534989655017852783203125f,-0.42520000438690175315059605054557323455810546875f,-0.051199998232722254698501274106092751026153564453125f,-0.7831000131607055436688824556767940521240234375f,-0.1319999842643735377123448415659368038177490234375f,0.793800002437829999024643257143907248973846435546875f,0.88740003535747558061075324076227843761444091796875f,0.5002000094532963458959784475155174732208251953125f,-0.54550002191960811348536708464962430298328399658203125f,0.7226000090479853810165877803228795528411865234375f,0.56800000321865073971139281638897955417633056640625f,-0.268000008434057246375914473901502788066864013671875f,0.70819999538660027837977395392954349517822265625f,0.285399992084503395517458557151257991790771484375f,-0.527199985708296292585828268784098327159881591796875f,1.110500002324581458879038109444081783294677734375f,0.7477999892950055738083392498083412647247314453125f,-0.574400006769597570865926172700710594654083251953125f,-0.48030001105070141420583240687847137451171875f,-0.065999985873698818750199279747903347015380859375f,0.472500009872019344214777447632513940334320068359375f,-0.2014999999701974076060650986619293689727783203125f,-0.373200024294853438533436928992159664630889892578125f,-0.21150000768154864072556620158138684928417205810546875f,-0.74589998904466625617715180851519107818603515625f,-0.359399976110458485578647014335729181766510009765625f,0.6416999922126531696875417765113525092601776123046875f,0.84939998624324786202777204380254261195659637451171875f,0.3467999751836063904164575433242134749889373779296875f,-0.732299984697997619065290564321912825107574462890625f,-0.4316000079751016738072166845086030662059783935546875f,-0.350999999433755593969408437260426580905914306640625f,0.1842000117704272466045978262627613730728626251220703125f,0.1606999941945073118176878779195249080657958984375f,-0.1410000201463699198711765347979962825775146484375f,-0.4535999927982687918159854234545491635799407958984375f,-0.17409999784231189323691069148480892181396484375f,-0.9793000433564191808955001761205494403839111328125f,0.2835999898552898912384989671409130096435546875f,-0.52980001282393895944977657563867978751659393310546875f,0.09899995005130701741791199310682713985443115234375f,-0.64179997510910080649182418710552155971527099609375f,0.4567000021591780534180315953562967479228973388671875f,0.3863000353693966104628998436965048313140869140625f,0.430999979615211969985466566868126392364501953125f,-0.5413999959886073298775954754091799259185791015625f,-0.4246000269889833589331828989088535308837890625f,-0.55859998046755787726169728557579219341278076171875f,-0.659899980930984231264346817624755203723907470703125f,-0.309100007379054630973769235424697399139404296875f,-0.07429999507665652203058925806544721126556396484375f,0.471600006149709205516273868852294981479644775390625f,-0.137999999046325427798365126363933086395263671875f,-0.380399978971481811385046967188827693462371826171875f,0.3502000105708835686613156212843023240566253662109375f,-0.1929999784827227671257787733338773250579833984375f,0.17729998250007650284487681346945464611053466796875f,0.6192999870300293085989551400416530668735504150390625f,0.2849000246286390591876624966971576213836669921875f,-0.3432999718010432133041831548325717449188232421875f,0.3958000032797455158828370258561335504055023193359375f,-0.4941999815583226762782942387275397777557373046875f,-0.38179999880194603889549398445524275302886962890625f,-0.6125999891549349651853617615415714681148529052734375f,-0.5130000193119048645939983543939888477325439453125f,0.5182000040292733533675573198706842958927154541015625f,-0.3897999912172556147282875826931558549404144287109375f,-0.3590000023543833407302372506819665431976318359375f,-0.02350001031160342535031304578296840190887451171875f,0.364299990308284804552130253796349279582500457763671875f,1.279100004398822765239174259477294981479644775390625f,0.17159999765604727617329672284540720283985137939453125f,0.04370000158548348334619504385045729577541351318359375f,-0.7378999845221638764769522822462022304534912109375f,0.2618999797612431290616541446070186793804168701171875f,0.03949999625980848971806835834286175668239593505859375f,0.50110001108944413061863087932579219341278076171875f,0.41669998677372976914057289832271635532379150390625f,-0.537200004334747571732577853254042565822601318359375f,-0.944800033462048105548092280514538288116455078125f,-0.30669999914169299159993897774256765842437744140625f,-0.23109999764114619313915000020642764866352081298828125f,-0.515799990129470753430496188229881227016448974609375f,-0.103999978959560479552237666212022304534912109375f,0.527599989080429043752928919275291264057159423828125f,-0.5346999873101712097422932856716215610504150390625f,-0.52119999640733005730197646698798052966594696044921875f,0.1224999953061342294891034043757827021181583404541015625f,-0.440399998939037029543897006078623235225677490234375f,-0.04819999359846161013365417602472007274627685546875f,0.492199983324110468263512530029402114450931549072265625f,-0.2669000064343214351225697100744582712650299072265625f,-0.14960000113546845224021808462566696107387542724609375f,0.1700000028684735264850047542495303787291049957275390625f,-0.8582000234007836070304620079696178436279296875f,-0.55060000720024060427704171161167323589324951171875f,0.46060001707226039258102900930680334568023681640625f,0.52380001726448544019376640790142118930816650390625f,0.32700001399219036368748447785037569701671600341796875f,-0.201000004425644862582345240298309363424777984619140625f,-0.779200022983550955046894159750081598758697509765625f,-0.65980002183914177038559500942938029766082763671875f,0.268900010368227915247274495413876138627529144287109375f,1.018599993431568151436295011080801486968994140625f,0.4997999902486807144441627315245568752288818359375f,-0.666599980370700340159828556352294981479644775390625f,-0.140200004088878671382190077565610408782958984375f,0.16460001447200767188405734486877918243408203125f,0.382599998803436758265661410405300557613372802734375f,-0.579200010168552381628614966757595539093017578125f,-0.27380000258684145109100427362136542797088623046875f,0.3467000016003847218115652140113525092601776123046875f,0.796100013995170829872449758113361895084381103515625f,0.61180000073909734936705717700533568859100341796875f,-0.393700014698505407206852169110788963735103607177734375f,0.9908000160574914616518071852624416351318359375f,0.593400001859664794068294213502667844295501708984375f,-0.5565000073462724738959650494507513940334320068359375f,-0.40760000159740439329425498726777732372283935546875f,-0.42260000576972966968014588928781449794769287109375f,0.12560000120252368116524621655116789042949676513671875f,0.8852000239074231391356306630768813192844390869140625f,0.3979999966621396989552295053726993501186370849609375f,-0.70280001239180567296216395334340631961822509765625f,0.28480000514984116222194643341936171054840087890625f,-0.195799988341331498986619408242404460906982421875f,-0.326799991649389109937828834517858922481536865234375f,-0.27499999232590199849113332675187848508358001708984375f,0.04579998133778584445963133475743234157562255859375f,0.492999976638704706655147447236231528222560882568359375f,-0.006200001566857071821914360043592751026153564453125f,0.550999996099621025535952867357991635799407958984375f,-0.360699970799684965783171719522215425968170166015625f,-0.261899999952316164808507892303168773651123046875f,-0.009900020307302526134662912227213382720947265625f,0.022199999789893620771863425034098327159881591796875f,-0.10870000110119580616352408242164528928697109222412109375f,0.09249999925494190033958830099436454474925994873046875f,0.20689999190568908460363672929815948009490966796875f,-0.39969998649060756168438501845230348408222198486328125f,0.23230002481937450653504129149951040744781494140625f,-0.080700008238852038999766591587103903293609619140625f,0.84509999429583537722265873526339419186115264892578125f,0.78750002793967743830449990127817727625370025634765625f,0.514099994117021008577239626902155578136444091796875f,-0.156200002293288786603397966246120631694793701171875f,0.5582000226557255473380791954696178436279296875f,0.0752999881029126072462531737983226776123046875f,0.8303000093370671574888319810270331799983978271484375f,0.3007000719785697384622835670597851276397705078125f,-0.4642999876260756053625300410203635692596435546875f,0.069099993148446170465604154742322862148284912109375f,0.200299991679191435878237825818359851837158203125f,-0.076899990415573160618123438325710594654083251953125f,-0.76539998333007108755055014626123011112213134765625f,-0.6254000147342679838402546010911464691162109375f,-0.416699973511696075689769713790155947208404541015625f,-0.1642999869555235026297168587916530668735504150390625f,-1.0591000164687631723836602759547531604766845703125f,-0.1480999945342542645221328712068498134613037109375f,0.61419999910444023072386698913760483264923095703125f,-0.024999983310699303018509453977458178997039794921875f,0.444499985754489568279268496553413569927215576171875f,0.399499990150332440208558182348497211933135986328125f,1.048300024700164811974900658242404460906982421875f,0.1121999989330768610074073876603506505489349365234375f,0.1726000046521425634882263011604663915932178497314453125f,-0.5794000083714723192684914465644396841526031494140625f,-0.276700000631808773476905116694979369640350341796875f,0.242399995608627794041467495844699442386627197265625f,-0.01580000249743473972330320975743234157562255859375f,0.110200001406669745307453922578133642673492431640625f,0.0203000003963709207965848690946586430072784423828125f,0.37449999767541886086519298260100185871124267578125f,0.20079999012947080672120137023739516735076904296875f,-0.30339999947547913183854007002082653343677520751953125f,-0.495200000587105559901601736783050000667572021484375f,-0.6644000287115574554519525918294675648212432861328125f,-0.203800004449486760194076850893907248973846435546875f,-0.731000016272068453559995759860612452030181884765625f,-0.50479999144077236650218765134923160076141357421875f,0.425900024919211916962780151152401231229305267333984375f,0.519300021004676981561942739062942564487457275390625f,0.1141999937772748552333723637275397777557373046875f,-0.5339000123053789792493262211792171001434326171875f,0.13909998979568438670639807241968810558319091796875f,-0.3138000234484668027334919315762817859649658203125f,-0.58169997857809063734890742125571705400943756103515625f,-0.40149998909234962951586567214690148830413818359375f,0.0818000174283977088407482369802892208099365234375f,0.56979997687488792568188955556252039968967437744140625f,-0.002399982234835729144606375484727323055267333984375f,-0.2653999964058397775801267925999127328395843505859375f,-0.24670000003576275648953242125571705400943756103515625f,-0.127399990542233165768948310869745910167694091796875f,-0.24999999754130819429320808922057040035724639892578125f,-0.1261999970033764462140624118546838872134685516357421875f,-1.1898000328660014446313653024844825267791748046875f,-0.52499999880790682027509319595992565155029296875f,0.8520000165626406829488814764772541821002960205078125f,-0.75210002520084362487295948085375130176544189453125f,-0.1503999998331071452639662311412394046783447265625f,0.77980000038146968233121469893376342952251434326171875f,-0.0507999944508075174098848947323858737945556640625f,0.1192000219404698579950263592763803899288177490234375f,0.32750000093132258793815481112687848508358001708984375f,-0.016200007247924919084880457376129925251007080078125f,0.2942000191092493111000294447876513004302978515625f,0.377200006197392967433046351288794539868831634521484375f,0.1587000006541610108712347937398590147495269775390625f,0.08520000163018692784788754579494707286357879638671875f,-0.030899999848008175007407771772705018520355224609375f,-0.429600008809566613621200303896330296993255615234375f,0.302200009661913071568051236681640148162841796875f,-0.2724999894201751704514435914461500942707061767578125f,-0.8319999834820628326070846014772541821002960205078125f,-1.2896000348865985785096199833787977695465087890625f,-0.029799992483854342850690954946912825107574462890625f,-0.2106999953493475796761913443333469331264495849609375f,-0.6887000081419945018978978623636066913604736328125f,0.78219999319612931731882099484209902584552764892578125f,-0.7527999830365181832547705198521725833415985107421875f,-0.6104000085502860883934772573411464691162109375f,0.269799964171648642974332688027061522006988525390625f,0.1516999944850800918771938086138106882572174072265625f,0.9637000252783298748227025498636066913604736328125f,0.306099994623660887071991965058259665966033935546875f,0.18750000193715099516822419900563545525074005126953125f,-0.18570000268816944100080945645458996295928955078125f,0.748899963206053342901213909499347209930419921875f,-0.2837000006914138783287171463598497211933135986328125f,-0.5041000014185907929231689195148646831512451171875f,-0.58429996482729951168266779859550297260284423828125f,-0.376999988324940193962220291723497211933135986328125f,-0.244600024008750693838010192848742008209228515625f,0.53849996702373059687118939109495840966701507568359375f,0.3293999981269239984982277746894396841526031494140625f,0.557400010398030332225971505977213382720947265625f,0.362599975334108315383474518966977484524250030517578125f,-0.09099999641627076751859704017988406121730804443359375f,0.0153999955490231599242179072462022304534912109375f,-0.2022999956130977494694889173842966556549072265625f,-0.923099998408555943996134374174289405345916748046875f,-0.289500025391578930111791123636066913604736328125f,-0.47619996716380175172389499493874609470367431640625f,-0.340499999828636656928892989526502788066864013671875f,0.159900000190734914440326974727213382720947265625f,-0.24900000555813317504316728445701301097869873046875f,0.83469999256283033961523187826969660818576812744140625f,1.3115000372231009340140417407383210957050323486328125f,-0.59189999086260769445289042778313159942626953125f,0.1580999990046023651046880331705324351787567138671875f,0.2061000063955784611380295245908200740814208984375f,0.016999997265636779264497135955025441944599151611328125f,0.55979998656064278161892389107379131019115447998046875f,0.7098000170335172498425890807993710041046142578125f,0.139499990224838388286343615618534386157989501953125f,0.536699991504848039625130695640109479427337646484375f,0.850600011968612790269617107696831226348876953125f,0.5170000042319298216852985206060111522674560546875f,-0.44230000194609164765324749168939888477325439453125f,-0.020899991130828965424370835535228252410888671875f,0.2046000266909597797848618938587605953216552734375f,0.35309999572634698239426143118180334568023681640625f,0.3089999965801835113410334088257513940334320068359375f,0.28980000373423087101087958217249251902103424072265625f,-0.008399998953938503376548396772705018520355224609375f,-0.73349998348951306326171106775291264057159423828125f,-0.7016000067830088227083251695148646831512451171875f,0.14329998338669536206424481861176900565624237060546875f,1.0495000364184381425047831726260483264923095703125f,0.54180000610351530809793985099531710147857666015625f,-0.7308000204160809420983468953636474907398223876953125f,-0.574000007569789971739737666212022304534912109375f,-0.37060000332593912020229254267178475856781005859375f,0.2665000026524066978339533307007513940334320068359375f,0.925900014674663651703667710535228252410888671875f,0.7076000162005424698463684762828052043914794921875f,-0.29690000371485947283645145944319665431976318359375f,0.63720000683069211167008916163467802107334136962890625f,0.1469999892562630083858721263823099434375762939453125f,-0.677699993062019334644219270558096468448638916015625f,-0.4112999972224233857787112356163561344146728515625f,-0.0867999900102613963781550410203635692596435546875f,0.373599994325637840830722780083306133747100830078125f,-0.44339999724030487726622595801018178462982177734375f,-0.2251999958932400236477633370668627321720123291015625f,0.2764999940469861083869318463257513940334320068359375f,0.07989999922215940131309253047220408916473388671875f,-0.01020001057088382179216523581999354064464569091796875f,-0.188700002516806131236393184735788963735103607177734375f,-1.1077000478982927944571201805956661701202392578125f,-0.4450000098347661747766323969699442386627197265625f,0.8503000214815139568003132808371447026729583740234375f,-0.3029000048622487728522401084774173796176910400390625f,-0.45040002538859880143462532942066900432109832763671875f,-0.150200005243718681668951830943115055561065673828125f,-0.29590000752210621204341123302583582699298858642578125f,0.0728000132113695297420008500921539962291717529296875f,0.460900001002848169751047180398018099367618560791015625f,-0.052500004023313395773442380232154391705989837646484375f,0.614599969172477500478635192848742008209228515625f,-0.13389999971389787702946705394424498081207275390625f,0.706400000272691119818091465276665985584259033203125f,0.253600029790401748641670565120875835418701171875f,-0.39750000223517378827864376944489777088165283203125f,0.628699979978800005397943095886148512363433837890625f,0.952299997401237163785481243394315242767333984375f,-0.239299993176758374335832968426984734833240509033203125f,0.5042999857261778462458323701866902410984039306640625f,0.456300010670721345462652607238851487636566162109375f,0.63759999131560274321373071870766580104827880859375f,-0.2735999885886908344900803058408200740814208984375f,0.34439999905824691950328997336328029632568359375f,0.0017000001847744528049588552676141262054443359375f,0.36140000369995839424319683530484326183795928955078125f,-0.45740001181364053906008848571218550205230712890625f,0.131099997305870186181664394098334014415740966796875f,-0.526399995802342868245204954291693866252899169921875f,-0.244000016808509911925284541212022304534912109375f,-0.42619999487996107490062058786861598491668701171875f,0.9097999967306851232251574401743710041046142578125f,0.46780003042221096620778553187847137451171875f,0.35269998028427396175032981773256324231624603271484375f,0.02820000388026233650862195645458996295928955078125f,-0.54059999852031470624069697805680334568023681640625f,-0.265999986916780617463018643320538103580474853515625f,-0.50369997606724525862631480777054093778133392333984375f,-0.40170000212192480404382877168245613574981689453125f,-0.01919999563992025315428691101260483264923095703125f,-0.39509999142736196287017946815467439591884613037109375f,-0.1815000139772890985767617166857235133647918701171875f,0.079100006075203399547035587602294981479644775390625f,-0.471699992734193773724626908006030134856700897216796875f,-0.79610001988112932824748213533894158899784088134765625f,-0.2726999839723109886335805640555918216705322265625f,-0.8081000049650668426437505331705324351787567138671875f,-0.1981000137567523466941565857268869876861572265625f,0.5729999994561072451659811122226528823375701904296875f,-0.30010000725984553593406189975212328135967254638671875f,-0.1181000146135691153403968201018869876861572265625f,-0.44719998749643552482524455626844428479671478271484375f,-0.099200001339614374273878638632595539093017578125f,0.09319999311417337395369031582958996295928955078125f,-0.566399974735826372551628082874231040477752685546875f,-0.1677999962240457332285359370871447026729583740234375f,0.0591999937772751394504666677676141262054443359375f,0.0919999923110008666071735206060111522674560546875f,-0.057500000111758708953857421875f,1.1202000139236449882673696265555918216705322265625f,0.74080000801086409722984171821735799312591552734375f,-0.5362000038951635350059632401098497211933135986328125f,0.64049999248981492883103783242404460906982421875f,0.0595999745368960276437064749188721179962158203125f,-0.658999986633658441093075452954508364200592041015625f,-0.3332000106602908129360685052233748137950897216796875f,-0.263400014935434023666260827667429111897945404052734375f,0.0315000001005828200728586807599640451371669769287109375f,0.155999996602535162537606083787977695465087890625f,-0.12080001665353723439011446316726505756378173828125f,-0.438099988760054070979776952299289405345916748046875f,-0.57969997553825347580414018011651933193206787109375f,-0.4167999876260761737967186491005122661590576171875f,0.33219997762441633160079845765721984207630157470703125f,-0.2968999981641768926010627183131873607635498046875f,0.01840001974105831550332368351519107818603515625f,0.4813999974787235291984188734204508364200592041015625f,-0.0243999750301242546157709512044675648212432861328125f,-0.18979998182952362295594639363116584718227386474609375f,-0.1080999991908668800277837362955324351787567138671875f,0.4872000041484831900362451051478274166584014892578125f,0.416800008040666636333071437547914683818817138671875f,-0.1656999995961785099307661539569380693137645721435546875f,-0.5866999869227409458716238077613525092601776123046875f,-0.560199997085332856983086458058096468448638916015625f,0.1302999949574470683888449684673105366528034210205078125f,0.22119999860525130230826107435859739780426025390625f,0.3624000064492225448020690237171947956085205078125f,0.05129999681264163502447672726702876389026641845703125f,-0.0731000011652707115672455984167754650115966796875f,-0.09720000765025582634137890636338852345943450927734375f,-0.104699997442960712756843122406280599534511566162109375f,-0.014300000664591860921603938550106249749660491943359375f,0.05640000563710934056160795080359093844890594482421875f,0.0825000017881393488128338731257827021181583404541015625f,0.51930001116991064691319479607045650482177734375f,0.43340001407861716842262467253021895885467529296875f,-0.0785000036060810035820622942992486059665679931640625f,0.568499978423118168535665972740389406681060791015625f,-0.003900014317036237798674846999347209930419921875f,-0.147799985122680777038794985855929553508758544921875f,-0.54909998580962426917295715611544437706470489501953125f,-0.91550001275539383271961924037896096706390380859375f,-0.3015999975442884561971368384547531604766845703125f,0.78839998546838774284850615003961138427257537841796875f,0.10440000591278053576616002828814089298248291015625f,0.56639998496174737141473087831400334835052490234375f,0.83790000031739442221123681520111858844757080078125f,0.72150007134676030773334787227213382720947265625f,0.150700006059557300996942785786814056336879730224609375f,-0.763000000424682855992841723491437733173370361328125f,-0.8390000382661824129826300122658722102642059326171875f,0.36779998720884332641389846685342490673065185546875f,-0.62729997173398766019403183236136101186275482177734375f,-1.08329999281167932423386446316726505756378173828125f,0.25059999602436988652698346413671970367431640625f,0.2070999972984195469649648657650686800479888916015625f,0.9010999955177307896292404620908200740814208984375f,-0.315799996834993379479783470742404460906982421875f,0.436699992398917657965284888632595539093017578125f,0.9307000138640404429679620079696178436279296875f,0.373899962870777036183511654598987661302089691162109375f,0.48940001053214043924555198827874846756458282470703125f,0.8102000305011873848570758127607405185699462890625f,-0.3305999949067828236337618363904766738414764404296875f,0.0819000008091330666815110816969536244869232177734375f,-0.37410000641047957969931303523480892181396484375f,0.5124999769777061064957024427712894976139068603515625f,-0.1218999971210956712042161598219536244869232177734375f,0.424899995198845914501362130977213382720947265625f,-0.5141999791741369563169428147375583648681640625f,-0.4206999906554818036141796255833469331264495849609375f,-0.9071000234127044592469246708787977695465087890625f,0.0906000024691222538564261412830092012882232666015625f,-0.61419997053146391241540413830080069601535797119140625f,-0.86659999434053835987157299314276315271854400634765625f,-0.126200000374764031985108658773242495954036712646484375f,-0.0010000006817281192894597552367486059665679931640625f,0.000999997217208115074527086107991635799407958984375f,-0.297099995249510140382653844426386058330535888671875f,0.8217000026434657655727278324775397777557373046875f,0.73650004214048436779194162227213382720947265625f,-0.334700004148482666010977482073940336704254150390625f,0.697600022496283056483434847905300557613372802734375f,0.140599989020824267527132178656756877899169921875f,0.38000001668930050158223821199499070644378662109375f,0.3286000067681074110481631578295491635799407958984375f,0.79839998099803910491800706950016319751739501953125f,0.39939997640848190485485247336328029632568359375f,-0.62949998255074024466892979035037569701671600341796875f,-0.10369999070763580295562178434920497238636016845703125f,-0.3714000133484602894640147496829740703105926513671875f,-0.282600004242360591799609892404987476766109466552734375f,0.793100037300586535593538428656756877899169921875f,0.2026000063657757976898210472427308559417724609375f,-0.761200013096630545561538383481092751026153564453125f,-0.7201000039815905751083846553228795528411865234375f,-0.4279999746084204303997466922737658023834228515625f,0.5528000228971243767972509886021725833415985107421875f,1.1292000164270401540989041677676141262054443359375f,0.63399999654293015538542022113688290119171142578125f,-0.6976000110596418668507112670340575277805328369140625f,-0.575200023758411393970391145558096468448638916015625f,-0.0821999943137168287421445711515843868255615234375f,0.67500000815838578915872858488000929355621337890625f,0.796900033330917523244352196343243122100830078125f,0.5882000126719475474601495079696178436279296875f,-0.506900019100308352193451355560682713985443115234375f,-0.53359999283552195237234627711586654186248779296875f,-0.44959998854398719458913546986877918243408203125f,0.23690000326782467698905065844883210957050323486328125f,-1.1495999748349188251950181438587605953216552734375f,-0.87819998912811314539794693700969219207763671875f,0.4887999787822365771461363692651502788066864013671875f,-0.105100000107288593653720454312860965728759765625f,-0.1445999959945680757300578989088535308837890625f,0.061800003011524662355213877162896096706390380859375f,0.17360003247261079906138547812588512897491455078125f,-0.2324000109195711871734602027572691440582275390625f,-0.54700002211332321433445713410037569701671600341796875f,-0.703299985361098833891446702182292938232421875f,-0.6604000087738040036811071331612765789031982421875f,0.15339998226463791297646821476519107818603515625f,0.78829999996423705255210734321735799312591552734375f,0.50460000195503251774198361090384423732757568359375f,-0.40979999162703750759106924306252039968967437744140625f,-0.111600001418590721868895343504846096038818359375f,0.897800019395351256434878450818359851837158203125f,-0.584399991011619324154935384285636246204376220703125f,0.5787999779999257743412499621626920998096466064453125f,0.7028000062823291926861202227883040904998779296875f,0.00850000405311579498857099679298698902130126953125f,-0.1185999945864082416591145374695770442485809326171875f,-0.9529999871253966858830608543939888477325439453125f,0.63069998298138363335141320931143127381801605224609375f,-0.0105999994516372542108229026780463755130767822265625f,0.14020000226348638960871539893560111522674560546875f,0.4184999708235270787781701073981821537017822265625f,0.4353999860867860949298346895375289022922515869140625f,-0.0586000274062161707888662931509315967559814453125f,0.14539995855689102910446308669634163379669189453125f,0.51610000747591211478493278264068067073822021484375f,1.50290005615949695538802188821136951446533203125f,-0.08979999352693557312932171043939888477325439453125f,0.59579998178482085346985286378185264766216278076171875f,-0.18180001146793411948010543710552155971527099609375f,-0.416499982982874161763220399734564125537872314453125f,0.01629999439120288773352740463451482355594635009765625f,0.54350001344084741816686801030300557613372802734375f,-0.3952999846756455326612922362983226776123046875f,-0.7332999598056080259311784175224602222442626953125f,-1.9337000340700143397043575532734394073486328125f,0.36299996894598063335024562547914683818817138671875f,0.2272999858900905945091608373331837356090545654296875f,0.8976999954462048236791815725155174732208251953125f,0.56509998162984853564694276428781449794769287109375f,-0.3326999987989662788123723657918162643909454345703125f,0.6488999716997145839059157879091799259185791015625f,0.206499981492758166723433532752096652984619140625f,0.56619996816962991914579106378369033336639404296875f,0.99139999405145573518893797881901264190673828125f,0.61899999207258193933967049815692007541656494140625f,-0.35839997943341739272682389128021895885467529296875f,-0.6640000153183933662148774601519107818603515625f,0.5875999972760670431171092786826193332672119140625f,-0.2482999877825380341533900718786753714084625244140625f,0.3497000032544139713763797772116959095001220703125f,-0.183300004285573692186517291702330112457275390625f,0.46419999351650476082653540288447402417659759521484375f,-0.06339998695850379561989029753021895885467529296875f,0.42069998495578797559346639900468289852142333984375f,0.299799990285932960176751294056884944438934326171875f,-0.450200018990039918520551509573124349117279052734375f,0.08160001423358931305074293049983680248260498046875f,0.664500007547438098498560066218487918376922607421875f,-0.407199973154067862424199120141565799713134765625f,-0.22999997437000274658203125f,0.324199986475706136701546711265109479427337646484375f,-0.070399988955259562573019138653762638568878173828125f,-0.373800004673003893884697390603832900524139404296875f,-0.336599994228780230454134425599477253854274749755859375f,-0.0018000075936317472269365680404007434844970703125f,0.23420001493692410576841211877763271331787109375f,0.34270000565350056120905719581060111522674560546875f,-0.7283000005602833226703296531923115253448486328125f,-0.668400021827221024750542710535228252410888671875f,0.0872999905839562106990570100606419146060943603515625f,-0.175700036811829107818994089029729366302490234375f,0.363800021660328098960235365666449069976806640625f,0.750100025513768198237585238530300557613372802734375f,-0.7727000079631807238911278545856475830078125f,-0.406999988138675661275556194595992565155029296875f,0.59270000565350056120905719581060111522674560546875f,0.18430001862049127936415970907546579837799072265625f,-0.2582000198245051336698452359996736049652099609375f,-0.55570001378953459703069484021398238837718963623046875f,0.90990002164840699805381518672220408916473388671875f,0.6748000176668167426896616234444081783294677734375f,-0.38140000708997245926212826816481538116931915283203125f,0.471400014615058982059281333931721746921539306640625f,0.188799999308586041735225080628879368305206298828125f,-0.380800006520748135496745589989586733281612396240234375f,0.50760002692937877810663849231787025928497314453125f,0.06919999407529786594750476069748401641845703125f,-0.53330001885145905671237187561928294599056243896484375f,-0.2289000143170358825273069669492542743682861328125f,0.20000001817941637227704632095992565155029296875f,0.5977000030085444226557456204318441450595855712890625f,0.0813999913692473597848220379091799259185791015625f,-0.2324000264167782603408340946771204471588134765625f,-0.4707999899059533976952707234886474907398223876953125f,-0.02450000084191561455781993572600185871124267578125f,-0.03100000055134299881132164955488406121730804443359375f,0.0133000004485249456731565942391171120107173919677734375f,-1.03070002191066745211855959496460855007171630859375f,0.16340000066757198737832368351519107818603515625f,-0.623799982172250633283283605123870074748992919921875f,-0.176500000678002866294491468579508364200592041015625f,0.1074999980628490447998046875f,0.057799985718727242556269629858434200286865234375f,0.27880000713169561521453942987136542797088623046875f,0.82760000755786933268609573133289813995361328125f,0.6668999629974370346729983793920837342739105224609375f,-0.0570999983787536802282147618825547397136688232421875f,0.0396999975919722913886289461515843868255615234375f,-0.442499992847442680243830182007513940334320068359375f,0.50639999364167442852391332053230144083499908447265625f,0.74160002555847182037496168049983680248260498046875f,0.55060000556111265979097879608161747455596923828125f,-0.8493999760359527417818981120944954454898834228515625f,-0.5546000061273570480580019648186862468719482421875f,-0.8654999929368489830494581838138401508331298828125f,0.7048999607771637698760969215072691440582275390625f,1.1773000105142585880457772873342037200927734375f,0.0342999874770644197496949345804750919342041015625f,0.15829998394548905338297117850743234157562255859375f,0.99309999945163696111194440163671970367431640625f,0.6914999910295005491889241966418921947479248046875f,-0.1278999928295612154016680506174452602863311767578125f,-0.3131000051140784279368745046667754650115966796875f,-0.39839999753832788798035835498012602329254150390625f,0.452100004301965174846600348246283829212188720703125f,-0.218699996519088557533905259333550930023193359375f,0.1115999960541724789209183654747903347015380859375f,0.222200002136826502141531136658159084618091583251953125f,-0.103399996942281635625704439007677137851715087890625f,0.3943999948859211457374840392731130123138427734375f,0.425699994827807071118286330602131783962249755859375f,0.200099999213218637805766775272786617279052734375f,-0.60639998734593358875599733437411487102508544921875f,0.4493999661266807610360274338745512068271636962890625f,0.8585999847888938774076450499705970287322998046875f,0.51179998971223827908261228003539144992828369140625f,-0.6675999815553428362591148470528423786163330078125f,-1.0604000022172925810082233510911464691162109375f,0.041499998219311218239369054572307504713535308837890625f,0.1042999956354499213606601415449404157698154449462890625f,0.1879000022545456172196054467349313199520111083984375f,0.24509999369978885397358681075274944305419921875f,0.329200010913610452423228025509160943329334259033203125f,0.6028000007688998351795817143283784389495849609375f,0.4096000050842760753511129223625175654888153076171875f,-0.21009999694079162235738067465717904269695281982421875f,-0.1967999939218161298271070336340926587581634521484375f,-0.10759999839365452789508026398834772408008575439453125f,0.06889999665915969018215037067420780658721923828125f,1.0922999887585636002995670423842966556549072265625f,0.862600007557869474794642883352935314178466796875f,-0.4311999788239597730665764174773357808589935302734375f,0.6069000151515009378044851473532617092132568359375f,0.59640002236366296273217812995426356792449951171875f,-0.03840000439286228584023774601519107818603515625f,0.466100017571449143360950984060764312744140625f,-0.03260000159740439329425498726777732372283935546875f,-0.5553999977096915063867754724924452602863311767578125f,-0.7926000206708909701092125033028423786163330078125f,-0.16119998817443814687067060731351375579833984375f,0.7504000095933676117709865138749592006206512451171875f,0.45140000805854796350757851541857235133647918701171875f,0.4514000108152627088742292471579276025295257568359375f,-0.09620000601857899014124342329523642547428607940673828125f,0.1927999987944961635122353982296772301197052001953125f,0.22700000415742394022089456484536640346050262451171875f,0.04389999900609253824512734354357235133647918701171875f,0.7878999953627587871096693561412394046783447265625f,0.250799980294704738525979337282478809356689453125f,-0.6644999886229634444845260077272541821002960205078125f,-0.16630000556707358327912515960633754730224609375f,0.0976000018954277237526184762828052043914794921875f,0.274699995543062680525281393784098327159881591796875f,0.0882000052213669505363213829696178436279296875f,0.197600020372867835050101348315365612506866455078125f,0.223600003266334557139316530083306133747100830078125f,-0.04380001003742250986761064268648624420166015625f,0.397000034630298870297337998636066913604736328125f,0.611900012916326563328084375825710594654083251953125f,-0.90210002549886691980418618186376988887786865234375f,-0.615600012862682177683382178656756877899169921875f,0.51630001208633180542761920150951482355594635009765625f,-0.2289000220656394191109939129091799259185791015625f,0.2164000149130824723897603689692914485931396484375f,0.6263000134274363528419371505151502788066864013671875f,0.637899998939037260470286128111183643341064453125f,0.03360000524073836203342580120079219341278076171875f,-0.163800002661347310350947736878879368305206298828125f,0.86389997650533933981620293707237578928470611572265625f,0.7327000161588188564820711690117605030536651611328125f,-0.007199994872510717680569314325111918151378631591796875f,-0.47609996683299582453940956838778220117092132568359375f,-0.543299986590444206058236886747181415557861328125f,-0.5923000002324574353451680508442223072052001953125f,0.32579999858587971317547271610237658023834228515625f,-0.60659998770952228142050444148480892181396484375f,-0.539999990016222231048459434532560408115386962890625f,0.6405000165924426003272174057201482355594635009765625f,0.3513000422239311859584631747566163539886474609375f,0.565699965435266793889468317502178251743316650390625f,0.6753999889180064553073634669999592006206512451171875f,0.6398000176668166005811144714243710041046142578125f,-0.5173999855875965891982559696771204471588134765625f,-0.813599985310435425134301112848334014415740966796875f,-0.532800017309188689296206575818359851837158203125f,-0.44049998701363779485262739399331621825695037841796875f,-0.032299997662007751131341137806884944438934326171875f,-0.52510001698285346805050721741281449794769287109375f,-0.241699990163743716475863720916095189750194549560546875f,0.294999998584389633293767474242486059665679931640625f,0.24620001466125263078765783575363457202911376953125f,0.32859999179243981615172742749564349651336669921875f,0.6710000060647722808226944835041649639606475830078125f,0.3010000606775289355709901428781449794769287109375f,-0.37839997488856358387465661508031189441680908203125f,-0.376999981582164966908976566628552973270416259765625f,-0.453399996495246426775338477455079555511474609375f,-0.266400005227327429935257896431721746921539306640625f,0.7854000002056358287205739543423987925052642822265625f,1.099800044488907335704652723507024347782135009765625f,-0.79830000026225977904914543614722788333892822265625f,0.7037999799743295259446540512726642191410064697265625f,1.0542000137448308549892317387275397777557373046875f,0.31299999740719730567661827080883085727691650390625f,0.0701999982029197777677609337843023240566253662109375f,1.0074000392317774554840070777572691440582275390625f,-0.6357999837219718841652138507924973964691162109375f,0.5166000014930960393400027896859683096408843994140625f,0.9594000512123113111329075763933360576629638671875f,-0.060699998068809435380899230949580669403076171875f,0.2053000079587101733835652339621447026729583740234375f,-0.9528000021100044847344179288484156131744384765625f,-0.4241999884128571096653104177676141262054443359375f,0.6695999944671988668432049962575547397136688232421875f,-0.82550003212690370446580345742404460906982421875f,-0.3662000066518782404045850853435695171356201171875f,0.609200014154613000982863013632595539093017578125f,-0.1781999923318624556856093477108515799045562744140625f,-0.2089999974370002622237052491982467472553253173828125f,0.0142999979078769767770751286661834456026554107666015625f,-0.3876000045776368807537437533028423786163330078125f,-0.424000001907348433860533987171947956085205078125f,0.0940000106990337425116877057007513940334320068359375f,-0.99760000509917734490272778202779591083526611328125f,-0.50739999430477633612923682449036277830600738525390625f,0.660699994306266358279344785842113196849822998046875f,0.2293999916076661094166411203332245349884033203125f,0.34920000867843636882525970577262341976165771484375f,0.1758000048816203975121652547386474907398223876953125f,0.150500019013881658480613623396493494510650634765625f,-0.24040000996589672155323569313623011112213134765625f,-0.493100007461011402565276284803985618054866790771484375f,0.7681000182271002785228120046667754650115966796875f,0.72640003070831316023259205394424498081207275390625f,-0.10240000205337995708987364196218550205230712890625f,-0.443799993608891885088496565003879368305206298828125f,-0.196999989219010018803857064995099790394306182861328125f,0.3616999919332564150664666158263571560382843017578125f,-0.816199985492229274086639634333550930023193359375f,-0.36959997781515152581732763792388141155242919921875f,0.689699983473122113508679831284098327159881591796875f,-1.1016999980986117435577398282475769519805908203125f,-0.5527999897420408093040578023646958172321319580078125f,0.72949998869746923713108799347537569701671600341796875f,0.908399986958503546929932781495153903961181640625f,0.7172000029563907474994266522116959095001220703125f,-0.311599982717633228190834415727294981479644775390625f,-0.3466000102102757551847389549948275089263916015625f,-0.1970000085532668254728605461423285305500030517578125f,0.16429999805688855207819187853601761162281036376953125f,-0.7991999904990194636411615647375583648681640625f,-0.58519999455213511652118540951050817966461181640625f,0.326899996934831182926473047700710594654083251953125f,0.04709999210163939853401871005189605057239532470703125f,0.78340002778768536018105805851519107818603515625f,-0.1570999939084050112114709918387234210968017578125f,-0.7952999924987553814759166925796307623386383056640625f,-0.3177000377655030405321667785756289958953857421875f,-0.13029999123215674217135529033839702606201171875f,-0.5056000056728719993515142050455324351787567138671875f,0.0075999911665913799652116722427308559417724609375f,0.23829998744726188508735731375054456293582916259765625f,0.08639999945312737406055703104357235133647918701171875f,0.32640000254511836175197458942420780658721923828125f,0.466399972295761244822642765939235687255859375f,0.376799988520145401338368174037896096706390380859375f,0.996000022530555639832527958787977695465087890625f,-0.029699993605911689797949293279089033603668212890625f,-0.626599997656047236205267836339771747589111328125f,-0.00340002816021456322204130628961138427257537841796875f,-0.43379998738765745969203635468147695064544677734375f,0.57939999577999101720848784680129028856754302978515625f,0.63140003368854547005639687995426356792449951171875f,0.621399978554248999529363572946749627590179443359375f,-0.1441999983221291525836704749963246285915374755859375f,-0.3118000172793866653364602825604379177093505859375f,-0.633800000053643675101966437068767845630645751953125f,0.665899999736249359472139985882677137851715087890625f,0.226300011229515263266875990666449069976806640625f,-0.0592999981500208495077686166041530668735504150390625f,-0.043400000742077789184225622420854051597416400909423828125f,0.356599991993606091345014874605112709105014801025390625f,-0.08459999837875376016427253489382565021514892578125f,0.13789999495297677878369313475559465587139129638671875f,0.99309999259710313168625361868180334568023681640625f,-0.05039999200999734085826275986619293689727783203125f,-0.74959999115169040351247531361877918243408203125f,-0.0940000330507755332831720807007513940334320068359375f,-0.477799995255470122401675325818359851837158203125f,0.5849999956786631205574167324812151491641998291015625f,-0.0197999681949614370068957214243710041046142578125f,0.017900008401274813962800180888734757900238037109375f,-0.49640001159757340953859738874598406255245208740234375f,-0.4276000392973429153897768628667108714580535888671875f,0.329999987185001231182468472979962825775146484375f,-0.50729996897727291926827319912263192236423492431640625f,0.02070001386404118903783455607481300830841064453125f,-0.43109998596236087653466029223636724054813385009765625f,0.47529998337179424350296130796778015792369842529296875f,-0.039299988818168429816068965010344982147216796875f,0.31900002425908979830637690611183643341064453125f,0.488299979996681177141226726234890520572662353515625f,0.70199999663233736413303631707094609737396240234375f,0.719600018942356367546153705916367471218109130859375f,-0.0469999903738498314709204350947402417659759521484375f,0.34529999957680690414463242632336914539337158203125f,0.566600029730797150051557764527387917041778564453125f,0.2809000079691410522997330190264619886875152587890625f,-0.25010000515505670382054859146592207252979278564453125f,-0.320200016568601364408408471717848442494869232177734375f,-0.07100000198557976371915856361738406121730804443359375f,1.13090003166198727058144868351519107818603515625f,0.8860000284910203305344111868180334568023681640625f,-0.449200009870529104460956659750081598758697509765625f,0.5954000075817109660647474811412394046783447265625f,0.43700000512599945778902110760100185871124267578125f,-0.230000003241002559661865234375f,-0.039899999240785799514696918777190148830413818359375f,-0.099200004617869874667945850887917913496494293212890625f,-0.06040000067129731864401520624596741981804370880126953125f,-0.23399999542534344332267437494010664522647857666015625f,-0.344600008884072384063301797141321003437042236328125f,-0.08260000312477351613882348146944423206150531768798828125f,0.46660000678300850296409407746978104114532470703125f,0.11919999526739122330809550476260483264923095703125f,-0.42429999757260083992349564141477458178997039794921875f,-0.25249998912215243507262130151502788066864013671875f,-0.348399997687339979535181555547751486301422119140625f,-0.005300003786385065751574074965901672840118408203125f,0.7412000249803067930542965768836438655853271484375f,0.4666000158727168223293801929685287177562713623046875f,-0.32260001005381344629796558365342207252979278564453125f,0.0709999939203260765907543827779591083526611328125f,-0.14420001612901689469481425476260483264923095703125f,-0.2916999944478273487646902140113525092601776123046875f,-0.5872000095129010777128542031277902424335479736328125f,-0.520200017350912258251582898083142936229705810546875f,0.1817999982431530436866040645327302627265453338623046875f,-0.47770001576766374373761436800123192369937896728515625f,-0.437400026826560928139286943405750207602977752685546875f,0.0209999970309435735060787919792346656322479248046875f,1.2440999850273133375821998924948275089263916015625f,0.34609999119639400788628336158581078052520751953125f,-0.5660999864652753732485734872170723974704742431640625f,0.270500011451542121676538954488933086395263671875f,-0.041599997879564754033054896353860385715961456298828125f,1.526999977707863109799291123636066913604736328125f,-0.5444000178337091711000539362430572509765625f,-0.03550000798702246385118996840901672840118408203125f,-0.1442000098705289445888411137275397777557373046875f,0.67560001852512385767113300971686840057373046875f,0.05639999747872348478949788841418921947479248046875f,-0.8819999839663505980524860206060111522674560546875f,-0.1425999452710158266199869103729724884033203125f,-0.394999988824129122377826206502504646778106689453125f,-1.1154000150322911366629341500811278820037841796875f,-0.2287999537110323444721871055662631988525390625f,-0.361900001740455667942342188325710594654083251953125f,-0.9520000061690809811665303641348145902156829833984375f,0.61159999523460850667788690770976245403289794921875f,0.23540000642687097798244622026686556637287139892578125f,-0.521100013399124151192154386080801486968994140625f,0.348699995326995804134639911353588104248046875f,0.35820000760555270602480959496460855007171630859375f,0.2539000131249427028024001629091799259185791015625f,0.563299977314471789213712327182292938232421875f,0.428300003689527475359000163734890520572662353515625f,-0.472800022077560555544550879858434200286865234375f,-0.51540000817775677433019154705107212066650390625f,-0.142200001913308948786607288639061152935028076171875f,0.34259999384880046591206337325274944305419921875f,0.331199952411651565853389911353588104248046875f,0.52540000177025802940988796763122081756591796875f,-0.87210001029968253760671359486877918243408203125f,0.4281000238895416032391949556767940521240234375f,0.09560000660419465390305049368180334568023681640625f,0.1470999807953834448426277958787977695465087890625f,-0.6230999943852424394208355806767940521240234375f,-0.096799993884563395596387636032886803150177001953125f,-1.4484000140786168486783935804851353168487548828125f,0.131400060808658736277720890939235687255859375f,-0.243999993115663560416805921704508364200592041015625f,0.5157000296592713084464776329696178436279296875f,0.3224999904632568359375f,-0.065600011819601178331140545196831226348876953125f,-0.516600003281235675700600040727294981479644775390625f,-0.09620002781152692250543623231351375579833984375f,0.3268000013351439747566473670303821563720703125f,0.4666000076770782101220902404747903347015380859375f,-0.1041999730646612221107716322876513004302978515625f,-0.424200007709860826832226621263544075191020965576171875f,-0.034200007113814379078320371263544075191020965576171875f,0.290299997453391522839183380710892379283905029296875f,-0.5990000188201667352672075139707885682582855224609375f,0.4214000044822692192525437349104322493076324462890625f,0.39730002403706354829893143687513656914234161376953125f,-0.5152000172019004509138540015555918216705322265625f,-0.1569999863505362469595638685859739780426025390625f,0.069200026857853202955084270797669887542724609375f,0.32780000866651537450024989084340631961822509765625f,-0.14719999282360074488451573415659368038177490234375f,-0.040900024360417841506887270952574908733367919921875f,0.1528000050902367235039491788484156131744384765625f,0.46120000993013388068675340036861598491668701171875f,-0.3527999729782340665451556560583412647247314453125f,0.5155999870836731613366055171354673802852630615234375f,-0.0151999911248685837250604890868999063968658447265625f,-0.5769999769628046948355404310859739780426025390625f,0.124400008520484106622205899839173071086406707763671875f,-0.6538000092178581912349955018726177513599395751953125f,-0.528200024704635229255700323847122490406036376953125f,-0.3358000022366642856042062703636474907398223876953125f,0.2356000004574656803146837091844645328819751739501953125f,0.4581000024132431125423181583755649626255035400390625f,0.3786000056505203215095889390795491635799407958984375f,-0.1263999970316886933829891859204508364200592041015625f,-0.346700004208087886325984072755090892314910888671875f,-0.67880002583265319771044232766143977642059326171875f,-0.37060000690221794883427719469182193279266357421875f,0.71220000683069173419426078908145427703857421875f,-0.46960000642538080040111481139319948852062225341796875f,0.58039999059438696082935393860680051147937774658203125f,0.745600006231665446421175147406756877899169921875f,0.59840000215768807123595252051018178462982177734375f,0.55940001693964003237624638131819665431976318359375f,-0.5395999527812005425175811978988349437713623046875f,-0.54360001518726352998100992408581078052520751953125f,0.96060003283023842612919906969182193279266357421875f,1.31980003703832693418007693253457546234130859375f,0.02079998878836608611209157970733940601348876953125f,-0.3417999907553195271248114295303821563720703125f,-0.040199987697601358149768202565610408782958984375f,0.805399973309040007052317378111183643341064453125f,0.4290999970972537713720384999760426580905914306640625f,0.1898999937459826536922946615959517657756805419921875f,-0.62269999764412631293453159742057323455810546875f,-0.188300000001490108747503882113960571587085723876953125f,0.1962999911129472696558195821125991642475128173828125f,-0.54390001662671583204655689769424498081207275390625f,0.05730000637918719608165929457754828035831451416015625f,-0.1301999799489976794575341045856475830078125f,-0.86339998040199361639679409563541412353515625f,-0.2668999875843525160235003568232059478759765625f,0.11599999094009394440263349679298698902130126953125f,0.372600006067752786975688650272786617279052734375f,0.000300001960992890559509760350920259952545166015625f,-0.44910000380277637077597319148480892181396484375f,0.5244999936521050898363682790659368038177490234375f,0.41070000820159913157425535246147774159908294677734375f,-0.52029998318552994618357843137346208095550537109375f,0.6968999723851680982988909818232059478759765625f,0.2222999939739704888808091709506697952747344970703125f,-0.5631000049278138686048578165355138480663299560546875f,-0.331099964709579808186390437185764312744140625f,-0.335199998985230951387137565689045004546642303466796875f,0.42109999549910426264176521726767532527446746826171875f,-0.28510000321790585076087154448032379150390625f,-0.0021999974615871942251743575980071909725666046142578125f,-0.2667999970138072995951006305404007434844970703125f,0.435800001603364961511033470742404460906982421875f,0.0732999987721443257981945862411521375179290771484375f,0.668200029659271166337930480949580669403076171875f,-1.201400001502037184764049015939235687255859375f,-0.55310000794529923240361313219182193279266357421875f,0.698099985668063194310661856434307992458343505859375f,-0.20990002932250462208685348741710186004638671875f,0.15349999503791333932412044305237941443920135498046875f,-0.49979997773170481423221644945442676544189453125f,1.233399971461295763219823129475116729736328125f,0.6341000006735324490136918029747903347015380859375f,1.377600016498565338451953721232712268829345703125f,0.0031999682664860529257566668093204498291015625f,0.3095000121295452544245563331060111522674560546875f,0.141700017619132978552443091757595539093017578125f,0.37649997746944396936896737315692007541656494140625f,0.186200010377168767572442220625816844403743743896484375f,-0.146200002256035743730677722851396538317203521728515625f,-0.24170001173317434695064775951323099434375762939453125f,0.61680002890229257417331609758548438549041748046875f,-0.25239999884963015830408039619214832782745361328125f,-0.1792000241756444012253268738277256488800048828125f,-0.19819999821782108284651258145458996295928955078125f,0.31540000743269924754486055462621152400970458984375f,0.66809999110698736757285587373189628124237060546875f,0.54640000150203693607409149990417063236236572265625f,0.27740001211166376293704161071218550205230712890625f,-0.83439997491836503940021430025808513164520263671875f,0.1166000000774858325058858099509961903095245361328125f,0.5849999864399431093175962814711965620517730712890625f,0.18830000421106785069014222244732081890106201171875f,0.5608000051796435680984131977311335504055023193359375f,0.0752000063240527705232807420543394982814788818359375f,-0.4743999908626079786699847318232059478759765625f,0.395199996191263114297953507048077881336212158203125f,-0.022199978369474404615857565659098327159881591796875f,-0.11599998944997746974649999174289405345916748046875f,-0.45260000144839285241005200077779591083526611328125f,0.58180001042485240958512804354541003704071044921875f,1.3032999877452855486126281903125345706939697265625f,0.2966000021636485683984574279747903347015380859375f,-0.1509999845325946665752780972979962825775146484375f,0.0642999807715423088438910781405866146087646484375f,0.44680002070665381097569479607045650482177734375f,-0.1980000050067900474459747783839702606201171875f,-0.782200009661913053804482842679135501384735107421875f,-0.92940003809928928291128613636828958988189697265625f,-0.21300000619888326269801837042905390262603759765625f,0.963700004863738968197139911353588104248046875f,0.28720001383423809926398462266661226749420166015625f,0.29759999280571935997841137577779591083526611328125f,0.399199988710879694053801358677446842193603515625f,0.444600004935264525585125738871283829212188720703125f,0.342000017046928395103577713598497211933135986328125f,0.181500001609325778417769470252096652984619140625f,-0.493999987304210463889830862171947956085205078125f,0.703999987900257195860831416212022304534912109375f,0.427399998402595571178608224727213382720947265625f,-0.89380003298521071286586447968147695064544677734375f,-0.27820000939369204928652834496460855007171630859375f,-0.21540000516027202337454582448117434978485107421875f,0.14800001338869339662096535903401672840118408203125f,-0.011199998903274555051012839612667448818683624267578125f,0.08030001254081753359059803187847137451171875f,0.70990002343654623473412357270717620849609375f,0.0320999974846838398434556438587605953216552734375f,0.7532999913215636667018770822323858737945556640625f,-0.0077000073671342761372216045856475830078125f,0.062100003594160124720247040386311709880828857421875f,-0.8804000040054322795413099811412394046783447265625f,0.92959998526573173194265109486877918243408203125f,0.46850000479817399678950096131302416324615478515625f,0.23439998221993452176548089482821524143218994140625f,0.6065999552249916604296231525950133800506591796875f,0.40699998724460606513986249410663731396198272705078125f,-0.37429999224543575309098741854541003704071044921875f,0.8368999878823757399004534818232059478759765625f,0.2824999971687793820507295095012523233890533447265625f,0.033500020444392930585308931767940521240234375f,0.55469999312162343585441703908145427703857421875f,0.116200003224611325691739693866111338138580322265625f,1.44269998769760121604122105054557323455810546875f,-0.1359000191450121519665117375552654266357421875f,0.07910000365376479436463341698981821537017822265625f,0.240899997687339606500245281495153903961181640625f,0.33310000302791564763538190163671970367431640625f,-0.026499995797872433200836894684471189975738525390625f,-0.80200000929832437890354412957094609737396240234375f,0.447200024116039429600277799181640148162841796875f,0.106800002974271723843457948532886803150177001953125f,0.865200013327598327350642648525536060333251953125f,-1.43500001728534698486328125f,-0.55420000703930849539347036625258624553680419921875f,0.626300002288818546958282240666449069976806640625f,0.66949997949600259516955702565610408782958984375f,0.229499988436698831861804137588478624820709228515625f,0.4068000052094458141027644160203635692596435546875f,0.91379997814893698659943765960633754730224609375f,0.33689999995231634244419183232821524143218994140625f,0.25179996080398581170811667107045650482177734375f,-0.184600003147124880342744290828704833984375f,0.2768000061035156278421709430404007434844970703125f,1.081800007891654757941068965010344982147216796875f,0.302099997335672387777094627381302416324615478515625f,0.7662000014364718225579053978435695171356201171875f,-0.1727999897420406938408632413484156131744384765625f,-0.82849998468160634956802823580801486968994140625f,-0.0240000050365924977313625277020037174224853515625f,-0.4290000062286853932391750277020037174224853515625f,-0.8327999937653540740711832768283784389495849609375f,-0.544199995416402515502340975217521190643310546875f,-0.432399997657537227269131108187139034271240234375f,0.768299971354007738000291283242404460906982421875f,0.3020000081062317320856891456060111522674560546875f,-0.0087999972224235278872583876363933086395263671875f,-0.366600003057718115684338044957257807254791259765625f,0.096000002041459087109842585050500929355621337890625f,0.143600002744793897591080167330801486968994140625f,-0.23119999201148733192212603171356022357940673828125f,-0.16819998510479905462489114142954349517822265625f,0.491600015276670632147215656004846096038818359375f,0.8178999771833428411582644912414252758026123046875f,0.5312000018835068004818822373636066913604736328125f,-0.5971999874591826795722226961515843868255615234375f,-0.85199999886751154321018475457094609737396240234375f,0.309800007683038725048163541941903531551361083984375f,0.494400004869699483123213212820701301097869873046875f,-0.1278999803125859813235365436412394046783447265625f,0.3358000093519688533660882967524230480194091796875f,-0.2057999993681907113796114572323858737945556640625f,-0.5957000030606984797287850597058422863483428955078125f,-0.16920000897645959270221283077262341976165771484375f,0.07820000462532039620100476895458996295928955078125f,-0.035399991339444714100181954563595354557037353515625f,-0.5028000128388405709500830198521725833415985107421875f,-0.0084000075221062342478717255289666354656219482421875f,0.27740000272393228186729174922220408916473388671875f,0.4316000095397234392891050447360612452030181884765625f,-0.2877999991297721660288289058371447026729583740234375f,-0.47140001420527688491546314253355376422405242919921875f,0.1361999980092047923818654453498311340808868408203125f,-0.0589999965429306438835510562057606875896453857421875f,-0.000300004047155244535360907320864498615264892578125f,0.68839998919367761942567085498012602329254150390625f,-0.52179999671578425335383144556544721126556396484375f,-1.04779998691082010253694534185342490673065185546875f,0.3502000067710877573290417785756289958953857421875f,0.087300002914667107489776753936894237995147705078125f,-0.71729997892379770974002894945442676544189453125f,0.9304999918937681968600372783839702606201171875f,0.22339999649524688862811672152020037174224853515625f,-1.061100005054473882637466886080801486968994140625f,-0.4142999774932860646003973670303821563720703125f,-0.1805999954283237229901715181767940521240234375f,0.81230000753402720192752894945442676544189453125f,-1.0430999583244329187436960637569427490234375f,-0.5643999925017357810475004953332245349884033203125f,-0.34260000576972959862587231327779591083526611328125f,0.405800004136562364465135033242404460906982421875f,0.1344000021874904859942034818232059478759765625f,-0.2776000053226947983375794137828052043914794921875f,0.22399999780952928318811245844699442386627197265625f,0.159400003081560159845508906073519028723239898681640625f,0.944499974727630586812665569595992565155029296875f,0.17550000441074331547497422434389591217041015625f,0.169800012600421990072163680451922118663787841796875f,-0.8079999799728394549447330064140260219573974609375f,0.27920002477168992527367663569748401641845703125f,-0.1317999953746795682718584430404007434844970703125f,-0.0826999975323676750349477515555918216705322265625f,0.798299979996681230431931908242404460906982421875f,0.4465999990344047265722338124760426580905914306640625f,-0.606400001502036989364796681911684572696685791015625f,0.2946000152170658026307137333787977695465087890625f,0.0424000026494264492527008769684471189975738525390625f,-0.13480000634193434194685323745943605899810791015625f,-0.195600003325939297837976482696831226348876953125f,0.00950000482797630496634155861102044582366943359375f,0.13319999315142627693830945645458996295928955078125f,-0.72080000190138804327943944372236728668212890625f,-0.1916000000774860101415697499760426580905914306640625f,-0.3879000014722346900697402816149406135082244873046875f,0.38730001148283488277002106769941747188568115234375f,-0.00380000046342610087179991751327179372310638427734375f,0.0700000022351741080228748614899814128875732421875f,0.15779999480843542869479279033839702606201171875f,-0.000800001603364908220328288734890520572662353515625f,-1.0022999771356584375325837754644453525543212890625f,0.54729999203681956032596644945442676544189453125f,-0.00640000790953632048285726341418921947479248046875f,-0.0611999696969984796623975853435695171356201171875f,0.0046999826908109554324255441315472126007080078125f,0.51740001211166397609986233874224126338958740234375f,-0.35580000443458548176067779422737658023834228515625f,-1.280099981927872221376674133352935314178466796875f,0.24280000583529481872346877935342490673065185546875f,-0.52020000036358826633886565105058252811431884765625f,-0.8354000081777570585472858510911464691162109375f,-0.118600000174343467307380706188268959522247314453125f,-0.4421999912217258188462665202678181231021881103515625f,-0.0485000140555198466785213895491324365139007568359375f,0.00719999774098389888621341015095822513103485107421875f,-0.49580001799464235023862102025304920971393585205078125f,-0.42960001558959515222113623167388141155242919921875f,-0.4278000220775606265988244558684527873992919921875f,0.8342000110626219822052007657475769519805908203125f,0.528100059056281878611116553656756877899169921875f,-0.0850000137090685115026644780300557613372802734375f,0.5731999992609022598344381549395620822906494140625f,0.06550004225969274784802109934389591217041015625f,0.57140002206563966780095142894424498081207275390625f,-0.26060001137256616488002691767178475856781005859375f,-0.192300021988154323793196454062126576900482177734375f,-0.0107999981388449128871798166073858737945556640625f,0.1128000048294663937298309974721632897853851318359375f,0.1830000045783817252953440402052365243434906005859375f,0.418200015950203063397339064977131783962249755859375f,0.074999996870756202582697369507513940334320068359375f,-0.8780999959498643203659185019205324351787567138671875f,0.65060002746582057397972675971686840057373046875f,-0.10140000030994400503914221189916133880615234375f,-0.37350001686811484802319682785309851169586181640625f,0.6190000185966493262412768672220408916473388671875f,-0.7030000169277190735783733543939888477325439453125f,-1.55570001356601750330810318700969219207763671875f,-0.614800028097629791545841726474463939666748046875f,0.349400005912780642347570392303168773651123046875f,0.391400037711858939104558885446749627590179443359375f,-0.800400014734268250293780511128716170787811279296875f,0.123999994009733160993391720694489777088165283203125f,1.3673999890893695496885129614383913576602935791015625f,-0.00279999734163283164889435283839702606201171875f,-0.580200016605854074214221327565610408782958984375f,0.0645999667882914963001894648186862468719482421875f,0.1518000061035156278421709430404007434844970703125f,-0.579599998974800190154610390891321003437042236328125f,-1.408399986958503546929932781495153903961181640625f,0.39500002562999725341796875f,0.09100000181794154485714898328296840190887451171875f,-0.929500003933906526754071819595992565155029296875f,-0.08789996824264534325266140513122081756591796875f,-0.165199993807077394336602083058096468448638916015625f,-0.067399997583031723280555524979718029499053955078125f,-0.495199968214333541283167505753226578235626220703125f,-0.32999999046325678708768691649311222136020660400390625f,-0.506000006958842174498158783535473048686981201171875f,-0.38159998867809719058641348965466022491455078125f,-0.27530000695288181500330892959027551114559173583984375f,0.4004000206947324613793171010911464691162109375f,0.014599956655501955538056790828704833984375f,0.3706000004947185288983746431767940521240234375f,-0.02039998791217811913156765513122081756591796875f,0.291399960374832289744517765939235687255859375f,0.3580999937891959916669293306767940521240234375f,0.04280001194477112136382856988348066806793212890625f,0.636200007545948409415359492413699626922607421875f,0.015199987995624386627468993538059294223785400390625f,0.223500007107853893018045710050500929355621337890625f,0.080699999149143497589875551057048141956329345703125f,0.0477000012576579701661927401801222003996372222900390625f,-0.974100016915798061489795145462267100811004638671875f,0.5925000102818014369177035405300557613372802734375f,0.2991000046223402097922416942310519516468048095703125f,-0.1783999958992001921842529554851353168487548828125f,0.85100001537799840889420011080801486968994140625f,0.17319999791979778791528588044457137584686279296875f,-1.68440001068115208227027324028313159942626953125f,0.271800038290024303933023475110530853271484375f,-0.077400000637769750255756662227213382720947265625f,0.18960000851154301670931090484373271465301513671875f,-0.619200023877620964185553020797669887542724609375f,0.004300002825260218486391750047914683818817138671875f,1.419100004994869212993080509477294981479644775390625f,-0.9693000125110149411966631305404007434844970703125f,-0.3443000057309866246413321277941577136516571044921875f,0.152199999976158295567074674181640148162841796875f,-0.41419998543262437351586413569748401641845703125f,-0.407100009852647826136262665386311709880828857421875f,0.831200015592575169875999563373625278472900390625f,-0.49510004928112039124243892729282379150390625f,-0.040400004601478745058784625143744051456451416015625f,-0.624400023198127929191514340345747768878936767578125f,-0.1386000409662724308645920245908200740814208984375f,0.25319999613165844465356713044457137584686279296875f,-0.48799999740719801621935403090901672840118408203125f,-0.26340001825094194742860054248012602329254150390625f,0.477599984908103625258490865235216915607452392578125f,-1.057600003981590486290542685310356318950653076171875f,-0.8268000218987463512121394160203635692596435546875f,-0.5064000143170357404187598149292171001434326171875f,0.6901999993205070182966665015555918216705322265625f,0.3264000524640078282345712068490684032440185546875f,-0.354000002056360063562578943674452602863311767578125f,-0.621199993091821500712512715836055576801300048828125f,0.0369999809861187856085962266661226749420166015625f,-0.4131999970257282939201104454696178436279296875f,0.568599975699186188649036921560764312744140625f,0.89419998215436891086937976069748401641845703125f,0.716000014185905531149956004810519516468048095703125f,-0.213399993515014596567880289512686431407928466796875f,-0.98720000027418119969979670713655650615692138671875f,0.3448000029891729045772308381856419146060943603515625f,-0.1393999953329563279424974098219536244869232177734375f,-0.60720000046044553254631637173588387668132781982421875f,0.159999999739229625816960833617486059665679931640625f,0.3535999955549835593870966476970352232456207275390625f,-0.4467999680869285672457635882892645895481109619140625f,-0.00919999407529825674600942875258624553680419921875f,0.50960000106096270400257708388380706310272216796875f,0.0098000148355961158586069359444081783294677734375f,-0.03359999298453342220227568759582936763763427734375f,0.58079999385476099860170506872236728668212890625f,0.96719996667206320495324689545668661594390869140625f,-0.33779998825192425471186652430333197116851806640625f,0.70239999586939827480591702624224126338958740234375f,0.31790002516508053531651967205107212066650390625f,0.39520000602602944894670145004056394100189208984375f,0.43659999262690529775454706395976245403289794921875f,-0.2887999945402146551032274146564304828643798828125f,-0.2296000043392183442847453989088535308837890625f,1.01199999618530256384474341757595539093017578125f,0.4404000135421757278209042851813137531280517578125f,0.20219999446272840515348434564657509326934814453125f,-0.5844000108301639073715705308131873607635498046875f,-0.1286000208497046770617089350707828998565673828125f,-0.518599994325636970415871473960578441619873046875f,-0.4190000114440917133862285481882281601428985595703125f,-0.1726999972343445932665417785756289958953857421875f,-0.00029999227523802574069122783839702606201171875f,-0.2225000020861624960843982989899814128875732421875f,-0.7455999872326852795367813087068498134613037109375f,-0.12719999073743792905588634312152862548828125f,-0.466000009715556995359975189785473048686981201171875f,0.114200014042854292029005591757595539093017578125f,-0.221400027132034438182017765939235687255859375f,0.27830000011324873554485748172737658023834228515625f,-0.0430999979615211969985466566868126392364501953125f,-0.961899953758717174423509277403354644775390625f,-0.533099987530708308014482099679298698902130126953125f,0.9220999697685243745581828989088535308837890625f,-0.122700032997130392686813138425350189208984375f,0.1403999871671199883849112666212022304534912109375f,-0.9455999997496604692059918306767940521240234375f,0.81400001353025430717025301419198513031005859375f,0.203900003439188015619265570421703159809112548828125f,-0.6252000060260296532277379810693673789501190185546875f,0.35440000568926333102126591256819665431976318359375f,0.09210000348240143797085011101444251835346221923828125f,-0.557299995240569057131096997181884944438934326171875f,0.449899981899559708864444473874755203723907470703125f,0.35269999678730956649275185554870404303073883056640625f,-0.051200010824203445736202411353588104248046875f,0.352399987971782735485248849727213382720947265625f,0.3446000031471252444958963678800500929355621337890625f,1.1939999658465385579120265902020037174224853515625f,0.4499999631941324906847512465901672840118408203125f,0.24839999172687543538273757803835906088352203369140625f,0.180099995636940235499423579312860965728759765625f,0.0342999971628188404793036170303821563720703125f,0.3368000049114225902258112910203635692596435546875f,-0.248700016188621475521358661353588104248046875f,0.86849996829032960476979496888816356658935546875f,0.454999992102384620551447369507513940334320068359375f,0.355999962329864416687996708787977695465087890625f,0.4255999511718755456968210637569427490234375f,0.56239999899864212551392483874224126338958740234375f,0.13330001456737505804994725622236728668212890625f,0.8257999969840046361468921531923115253448486328125f,0.32239999154806131542727598571218550205230712890625f,-0.60439998892545698794265263131819665431976318359375f,-0.34439999071359661542146568535827100276947021484375f,0.35960001193881030445709257037378847599029541015625f,0.57299999472498885921822875388897955417633056640625f,-0.2401999963402747795271352515555918216705322265625f,-0.1804000013232229804316375521011650562286376953125f,-0.4403999917864798163691375521011650562286376953125f,0.231399987792968886424205265939235687255859375f,0.6664000181913376508191504399292171001434326171875f,0.1942000140428543630832791677676141262054443359375f,-0.001600017511844242790175485424697399139404296875f,-0.445000026524067227029490823042578995227813720703125f,0.426400006568431688691589442896656692028045654296875f,0.37030003399848965273122303187847137451171875f,-0.1856000072002412792926406837068498134613037109375f,0.6195999959945677204586900188587605953216552734375f,0.3576000108361245821697593783028423786163330078125f,-0.3985999944746492218428102205507457256317138671875f,0.0909999931752683011154658743180334568023681640625f,-0.0104999954700470965462955064140260219573974609375f,-0.4858000114381315626133073237724602222442626953125f,0.7835999849379060577803102205507457256317138671875f,0.7111000131011007141523805330507457256317138671875f,0.3488000191271307670604073791764676570892333984375f,0.1972000221788885454543560626916587352752685546875f,0.2531999909162525597139392630197107791900634765625f,-0.310400012499094213325179225648753345012664794921875f,-0.450000022798776821986166396527551114559173583984375f,0.23559997307658164800159283913671970367431640625f,0.72979999978542320349106375942938029766082763671875f,-0.45419998871088029801512675476260483264923095703125f,-1.230699983614683024057967486442066729068756103515625f,-0.7335999946981666131051724732969887554645538330078125f,0.296599984058737831116303595990757457911968231201171875f,0.837999982841312718306880924501456320285797119140625f,0.18980001260042211885803453697008080780506134033203125f,-0.45339999977350220472516184599953703582286834716796875f,-0.51710001544058314948415500111877918243408203125f,-0.58580000309348090326011515571735799312591552734375f,-0.14240000123381602037397897220216691493988037109375f,0.802699990975856536579158273525536060333251953125f,0.56420000122785562979288442875258624553680419921875f,-0.48540000266432770104074734263122081756591796875f,-0.2337999575853348943610399146564304828643798828125f,0.01100001239776560169048025272786617279052734375f,-0.27040000281333931297922390513122081756591796875f,-0.21710002400875083594655734486877918243408203125f,0.9180999732255941125913523137569427490234375f,0.53099999645352369270767667330801486968994140625f,0.27340001497268673347207368351519107818603515625f,-0.5605999714374547693296335637569427490234375f,-0.3869999940991402098688922706060111522674560546875f,0.2855999958753585588056012056767940521240234375f,1.1179999918937681968600372783839702606201171875f,0.32650000056624417510420244070701301097869873046875f,0.2318000046133992242403110140003263950347900390625f,-0.316400033390522139598033390939235687255859375f,0.109300000441074462997903538052923977375030517578125f,0.313199980783462450517617980949580669403076171875f,0.20199996340274850581408827565610408782958984375f,0.318599996411800479023668231093324720859527587890625f,-0.210300011944770659511050325818359851837158203125f,-0.521299979639053390201297588646411895751953125f,-0.05189999235272413358188714482821524143218994140625f,0.621099977636337285957779386080801486968994140625f,-0.0911000479698174103759811259806156158447265625f,0.2927999907851219774101991788484156131744384765625f,0.473900006270408624686751863919198513031005859375f,0.675500012159346852058661170303821563720703125f,0.19970000981092450587084385915659368038177490234375f,0.2194999899268150755915485206060111522674560546875f,-0.758100004220008116817552945576608180999755859375f,-0.4319000130653380953305031653144396841526031494140625f,-1.23259998013973248731645071529783308506011962890625f,0.640800011289119453294915729202330112457275390625f,0.078200002241134658476084950962103903293609619140625f,0.7304999739006163839150076455553062260150909423828125f,-0.79309999367743699849597760476171970367431640625f,-0.130599998520314730665603519810247235000133514404296875f,-0.541500001981854328647614238434471189975738525390625f,-0.09049996380507963067429955117404460906982421875f,-0.18069999352395538227966653721523471176624298095703125f,0.804300011467933728681600769050419330596923828125f,-0.8934999819993976188925444148480892181396484375f,-0.3829999978542328875619205064140260219573974609375f,-0.96079998238086705697469369624741375446319580078125f,0.393800023150443934127906686626374721527099609375f,0.4783999985814093580671624295064248144626617431640625f,0.0588000186800956559096675846376456320285797119140625f,-0.19679998866915671129618203849531710147857666015625f,0.1127999943241476954458590853391797281801700592041015625f,-0.68919999709278345623175709988572634756565093994140625f,-0.5816000044919549250010959440260194242000579833984375f,0.4461999904096123348296032418147660791873931884765625f,0.1237999997556207265603234191075898706912994384765625f,-0.1312999881327154838572823791764676570892333984375f,0.14720000452101233978652317091473378241062164306640625f,-0.13540000162124632421267733661807142198085784912109375f,-0.15160000704377896152408311536419205367565155029296875f,0.372600024396181339625400141812860965728759765625f,-0.536600009912252318144965101964771747589111328125f,-0.43290005019903166072481326409615576267242431640625f,-0.0138000056415796823472419418976642191410064697265625f,0.1578000038981437480600789058371447026729583740234375f,-0.46769997573941923807439025040366686880588531494140625f,0.2270000104904175142195299486047588288784027099609375f,-0.8028000280380249709111240008496679365634918212890625f,-0.6593000432074074268484764616005122661590576171875f,-0.4905999849975106741339914151467382907867431640625f,0.2929999831020833056527408189140260219573974609375f,0.0666999788761142298199047218076884746551513671875f,0.4110000047981738902080905972979962825775146484375f,0.2954000107109546746642081416212022304534912109375f,-0.210499981760978727152178180404007434844970703125f,0.18180000818967823050797960604541003704071044921875f,-0.07399999743700025334192105219699442386627197265625f,0.319699974346161042149105924181640148162841796875f,-0.4262000158905985269797156433924101293087005615234375f,0.8014000053763388198291295339004136621952056884765625f,1.06170001925826085198423243127763271331787109375f,-0.42300001275539411693671354441903531551361083984375f,0.51079999817609778034466216922737658023834228515625f,0.310100024098157955876331470790319144725799560546875f,0.206999999612569762774683113093487918376922607421875f,-0.671000016123056486350151317310519516468048095703125f,-0.937300011110306030559513601474463939666748046875f,-0.67400002098083511281600976872141472995281219482421875f,0.33640000254511825961145632390980608761310577392578125f,0.7222000200927258806160580206778831779956817626953125f,-0.150599994832277150891286510159261524677276611328125f,0.48409999933242797709453952847979962825775146484375f,-0.04399998298287410847251521772705018520355224609375f,0.72799998787045439030407578684389591217041015625f,0.2074000012338161891278787152259610593318939208984375f,-0.63489998687282211964344469379284419119358062744140625f,-0.37809997208938028734337422065436840057373046875f,-0.35509999677315351451767355683841742575168609619140625f,0.01990001375079142320601022220216691493988037109375f,0.732899960047006970853544771671295166015625f,0.2330999860405922330386374596855603158473968505859375f,-0.1157999997407197501075870604836381971836090087890625f,0.10620000575780874640940965036861598491668701171875f,-0.02810000112801791516403682180680334568023681640625f,-0.3353000250577924390427142498083412647247314453125f,0.12570003621578251795654068700969219207763671875f,-0.196699998694658262365919654257595539093017578125f,-0.6429000035583973815533909146324731409549713134765625f,0.05249999754130829643372635473497211933135986328125f,0.17130000597685579588613791202078573405742645263671875f,-0.67100001880526516373492995626293122768402099609375f,-0.2983999733984461499858298338949680328369140625f,-0.3885000101625919466385994383017532527446746826171875f,-0.174900053238868480320888920687139034271240234375f,-0.5050999807357783311090315692126750946044921875f,0.04850000956654554329361417330801486968994140625f,-0.121299988579750106509891338646411895751953125f,0.7972999848842619030619971454143524169921875f,0.351699999886751157873732154257595539093017578125f,0.515799996089935319787400658242404460906982421875f,0.24160000052452090812948881648480892181396484375f,-0.043900002396106696522792844916693866252899169921875f,1.10999999865889531491802699747495353221893310546875f,0.0393999512255192030352191068232059478759765625f,0.2717999956727028543213009470491670072078704833984375f,0.15259999146461478147784873726777732372283935546875f,-0.228800024044513605758766061626374721527099609375f,0.182100000911951109827668915386311709880828857421875f,-1.0631000051140784279368745046667754650115966796875f,-0.25729997862577391742888721637427806854248046875f,-0.26410000424981117106426609097979962825775146484375f,0.600600002953409983774690772406756877899169921875f,-0.6870000189095737397337870788760483264923095703125f,-0.06939999846220012980069213881506584584712982177734375f,0.6853000003218649993641520268283784389495849609375f,-0.2152999828875064025623942143283784389495849609375f,-0.15899999654293051065678810118697583675384521484375f,0.40000001609325419593687911401502788066864013671875f,1.006099993878603271468819002620875835418701171875f,-0.352800003004074103074572121840901672840118408203125f,0.268599996709823596319210992078296840190887451171875f,0.6535999970078467669054589350707828998565673828125f,-0.388400009459257233856988023035228252410888671875f,-0.74400002142786991754519476671703159809112548828125f,0.35299995501339420655995127162896096706390380859375f,0.4588000292599205831578501602052710950374603271484375f,-0.6425999931037422729929176057339645922183990478515625f,-0.97060002463459937871448346413671970367431640625f,-0.5038000201702119706936855436651967465877532958984375f,-0.0206000042200089605870516606955789029598236083984375f,0.30770001109242439696345172706060111522674560546875f,-0.665399990296363430530846017063595354557037353515625f,-0.484600006127357207930117510841228067874908447265625f,0.4564999688267707966815578402020037174224853515625f,0.503399988299607858976969509967602789402008056640625f,0.354800010663270715127737275906838476657867431640625f,0.06010001709461221253150142729282379150390625f,-0.186600005143880753877283495967276394367218017578125f,-0.519599995845556161810918638366274535655975341796875f,-0.42769999939501257557594726677052676677703857421875f,-0.15479999956190571008818324116873554885387420654296875f,-0.34759999124109730583853661300963722169399261474609375f,-0.041400006009638001813755181501619517803192138671875f,0.65980002705454854350364257697947323322296142578125f,-0.25779999182820301228957760031335055828094482421875f,-0.129300029349327161298788269050419330596923828125f,0.721600011849403255581592020462267100811004638671875f,0.072199990439414829523911976139061152935028076171875f,-0.360400013542175656766630709171295166015625f,0.521200007247924812503470093361102044582366943359375f,-0.003799994987249366573678344138897955417633056640625f,-1.0553999839633700918994918538373894989490509033203125f,0.4278000144779683378004619953571818768978118896484375f,0.4470000148117543314896238371147774159908294677734375f,-0.18129998753666853872346109710633754730224609375f,0.09279999816119666178337865858338773250579833984375f,-0.053200006487965634249803770217113196849822998046875f,-0.512399987487495156557315567624755203723907470703125f,0.1506000119686128346785380927030928432941436767578125f,0.3979999987483024970202905024052597582340240478515625f,1.208200006413459703935586730949580669403076171875f,0.139599990332126111525212763808667659759521484375f,0.0827000037908554030963159675593487918376922607421875f,0.6675999947428703507057434762828052043914794921875f,-0.26960001119375220923757296986877918243408203125f,-0.01270000185370445677790485206060111522674560546875f,0.3727999889969826341484804288484156131744384765625f,-1.0731999848067754754765701363794505596160888671875f,-0.67260000994205471069875557077466510236263275146484375f,-0.99310000585913638104784695315174758434295654296875f,0.1301000664174551957330550067126750946044921875f,-0.1667999840497971231201290720491670072078704833984375f,0.17420000256896006618489991524256765842437744140625f,-0.72840000945925709174844087101519107818603515625f,-0.2046000016570090895839939548750407993793487548828125f,0.3499999919533731684850863530300557613372802734375f,-1.285399943506718045682646334171295166015625f,-0.590399988657236196587518861633725464344024658203125f,0.2723000143885609958260829444043338298797607421875f,0.998899948751926558543345890939235687255859375f,0.380199988889694306948285884573124349117279052734375f,-0.59099999511241918526138761080801486968994140625f,-0.6831999645411972466035877005197107791900634765625f,-0.34809998991489410347099919817992486059665679931640625f,-0.085999993845820466020768435555510222911834716796875f,-0.47999996915459686164240338257513940334320068359375f,-0.276799990084767355558170720541966147720813751220703125f,0.166900008296966717580289696343243122100830078125f,0.773500022530555497723980806767940521240234375f,0.03370000054240218645418281084857881069183349609375f,-0.1484000057339667222322532325051724910736083984375f,1.08079997433424068731255829334259033203125f,0.310699985402822509428233388462103903293609619140625f,-1.151100005352497124277988405083306133747100830078125f,0.6721000053822994146912606083787977695465087890625f,0.3006000054866075910098288659355603158473968505859375f,-0.09270000006556511351618610206060111522674560546875f,0.93309999334216087163440533913671970367431640625f,0.4561000062465667692634951890795491635799407958984375f,-0.62750000044703480028829289949499070644378662109375f,0.05409999940693399622659853775985538959503173828125f,0.064299998354911791498267348288209177553653717041015625f,-0.126000006586313162415535771287977695465087890625f,-0.830800001454353065355462604202330112457275390625f,-0.287400005257129720348530099727213382720947265625f,0.404799997699260671879528672434389591217041015625f,0.049100016319751915716551593504846096038818359375f,0.504199999737739545935255591757595539093017578125f,0.180200018095970193598986952565610408782958984375f,-0.766699958014488203161818091757595539093017578125f,0.10820000022947784401594617520458996295928955078125f,0.250599997141957242519083592924289405345916748046875f,-0.27389998067766452294335977057926356792449951171875f,0.071599983686208457100974555942229926586151123046875f,-0.855200011688471040116610311088152229785919189453125f,-0.474200020897388441198927466757595539093017578125f,-0.65820002861619020251282563549466431140899658203125f,0.47320000984072674299341088044457137584686279296875f,0.67210003771781945403063218691386282444000244140625f,0.14980000514984137538476716144941747188568115234375f,-0.29599999809265131744950849679298698902130126953125f,0.02529998333454130943209747783839702606201171875f,-0.089600003892183366360768559388816356658935546875f,0.642000006169080705831220257095992565155029296875f,-0.10439996776580784398902324028313159942626953125f,-0.685400002589821699672256727353669703006744384765625f,0.038000000461936134232843187419348396360874176025390625f,0.623699994991719908199456767761148512363433837890625f,0.2708000044345855172878145822323858737945556640625f,0.1138000088453292590884302626363933086395263671875f,0.18859998449087189698047950514592230319976806640625f,0.19199999856948846144177878159098327159881591796875f,-0.13139999479651454095119333942420780658721923828125f,0.0111999936133624355960591856273822486400604248046875f,-0.2970000052005052904036119798547588288784027099609375f,0.0801999982774257436091147610568441450595855712890625f,0.65349999209493392360315056066610850393772125244140625f,0.68500003099441553189308251603506505489349365234375f,-0.18140000686645496585924774990417063236236572265625f,-0.1787000358581547487801799434237182140350341796875f,-0.22960001275837438328153439215384423732757568359375f,-0.568600022414326833342101963353343307971954345703125f,0.016799984161555148176603324827738106250762939453125f,0.452400008758902572036930678223143331706523895263671875f,0.236000010237097768150960064303944818675518035888671875f,-0.171799992208182761288526307907886803150177001953125f,0.36360000114291890493944947593263350427150726318359375f,-0.4368999930977821577471331693232059478759765625f,-0.11660001542568210197714506648480892181396484375f,-0.1280000121593474204928497783839702606201171875f,0.18650000369548802581221025320701301097869873046875f,-0.79919997529983533013364649377763271331787109375f,0.8037999939441675678608589805662631988525390625f,0.38000000551342960619649602449499070644378662109375f,-0.34089999902844425605508149601519107818603515625f,0.31489998730123058834351468249224126338958740234375f,0.242299997997283911299604142186581157147884368896484375f,-0.4926999975323678171434949035756289958953857421875f,-0.548299991619586535307462327182292938232421875f,-0.496100008928775704220015541068278253078460693359375f,0.5257000111818310728040160029195249080657958984375f,-0.57010002961158789958062698133289813995361328125f,0.03600000286102300606216886080801486968994140625f,0.138000006198883085062334430404007434844970703125f,0.01020001273155202170528355054557323455810546875f,-0.162700000810623190972137308563105762004852294921875f,-0.25510000994205483948462642729282379150390625f,0.3192999786853789601082098670303821563720703125f,0.42830000592470174325399057124741375446319580078125f,0.5001999930247664760685211149393580853939056396484375f,-0.9259999798759821487692533992230892181396484375f,-0.262699990715086484360796248438418842852115631103515625f,0.3619999920129775006216732435859739780426025390625f,0.4955999893188476335126324556767940521240234375f,0.129299999994039627182473850552923977375030517578125f,0.119700006532668989933654302149079740047454833984375f,0.1888999968826770015084548504091799259185791015625f,-0.0358000019758939469483038919861428439617156982421875f,0.7156000013887882094110182151780463755130767822265625f,-0.32260000807940958367225903202779591083526611328125f,-0.12540000322312117031486877749557606875896453857421875f,0.10439999697208435236461809836328029632568359375f,0.8610000129938129020956694148480892181396484375f,0.10039999282956113546561027760617434978485107421875f,-0.01799999502301208309518187888897955417633056640625f,-0.74319999873936115619699194212444126605987548828125f,-0.458100013905763658872416499434621073305606842041015625f,-0.0438999932691455097000243767979554831981658935546875f,0.81889998939484343054573400877416133880615234375f,0.317500000186264508261757555374060757458209991455078125f,0.13230000416263931750648907836875878274440765380859375f,0.225400049602985319552317378111183643341064453125f,-0.17719999863505364601223845966160297393798828125f,0.274199999290704710119825904257595539093017578125f,0.73619998608827597053050340036861598491668701171875f,-0.36080001069307332528524057124741375446319580078125f,0.66800000649690627341215076739899814128875732421875f,0.24280004323720927317253881483338773250579833984375f,-0.4907999957919120248561739572323858737945556640625f,-0.3150000366568566079195079510100185871124267578125f,-0.3658000309586526554994634352624416351318359375f,-0.14060000246912240928764958880492486059665679931640625f,-0.13320000253915786903036178046022541821002960205078125f,0.5664999883286652160307994563481770455837249755859375f,-0.5891999915421006761562239262275397777557373046875f,-0.0846000032961367054440415813587605953216552734375f,0.90939995644092608273467703838832676410675048828125f,0.376799980473518036916402706992812454700469970703125f,-0.558999998927116603653075799229554831981658935546875f,-0.8767999852418899564554521930404007434844970703125f,0.58960000493526454334869413287378847599029541015625f,0.00900000727176664128137417719699442386627197265625f,-0.82959998377561561255788546986877918243408203125f,-0.007799998310208167140444857068359851837158203125f,-0.429999986365437347757278985227458178997039794921875f,-0.637500002421437894639666410512290894985198974609375f,-0.14279999972581858269649046633276157081127166748046875f,0.51300001186132437869247269190964289009571075439453125f,0.43180001735389250683283535181544721126556396484375f,0.09459999465346335956184020687942393124103546142578125f,-0.70739999676346776436020036271656863391399383544921875f,-0.49710000948011856536368213710375130176544189453125f,-0.031599999183416382919631359982304275035858154296875f,0.635000013262033480287982456502504646778106689453125f,0.7690000096559526099326831172220408916473388671875f,-0.2439999943822621641942305359407328069210052490234375f,0.0211999986052513467171820593648590147495269775390625f,0.66399996476620481988817346064024604856967926025390625f,0.0856000034376979412886754516875953413546085357666015625f,0.0972000063464046049066524801673949696123600006103515625f,-0.0079999954886733615211369397002272307872772216796875f,0.10899999952316274942631935118697583675384521484375f,0.33979999918937675573715750942938029766082763671875f,-0.433899985110759445205985684879124164581298828125f,-0.3069999738335604799743805415346287190914154052734375f,0.50330000905394545185345123172737658023834228515625f,0.8578999920845031823546378291212022304534912109375f,-0.0005000145435332115084747783839702606201171875f,0.030199996787309668633270121063105762004852294921875f,0.0742000056982039524200445157475769519805908203125f,-0.03060002269744899194847675971686840057373046875f,0.3174000046610832015403502737171947956085205078125f,-0.67500004947185487935712444595992565155029296875f,-0.9781999772071827692343504168093204498291015625f,-0.30530000225901599009148412733338773250579833984375f,0.69860001347362998824763735683518461883068084716796875f,-0.253200002054870143552989247837103903293609619140625f,-0.21350000645592814851880802962114103138446807861328125f,0.9368999725341797812916411203332245349884033203125f,-0.84789997360706337303781765513122081756591796875f,-0.359799992185831030155895859934389591217041015625f,-0.9171999800086023668654888751916587352752685546875f,0.07759999295473107849829830229282379150390625f,-0.146900006657838755330658386810682713985443115234375f,-0.822399994230270436901264474727213382720947265625f,0.801199995923042251888546161353588104248046875f,0.29670000107884408890868144226260483264923095703125f,-0.79540000594258319921436850563623011112213134765625f,-0.37439995800554815019722809665836393833160400390625f,-0.204699982094764643125728298400645144283771514892578125f,0.443700003001093801202614486101083457469940185546875f,0.19489998774826489125189254991710186004638671875f,0.24740000562965869246312422546907328069210052490234375f,-0.529599988916516384307442422141321003437042236328125f,-0.41059997136294867914330097846686840057373046875f,-0.38499999664723871806160104824812151491641998291015625f,0.9267000048041342807891851407475769519805908203125f,0.47469997225999804868479259312152862548828125f,0.314600000315904626546625877381302416324615478515625f,0.7610999765932560112702276455820538103580474853515625f,-0.28609999454915513439345886581577360630035400390625f,-0.15189999782890073465324576318380422890186309814453125f,0.01790000331625341001284823505557142198085784912109375f,0.1947000012613830932650671456940472126007080078125f,0.0925000030361115832544527393110911361873149871826171875f,-0.562400018668174794811420724727213382720947265625f,0.36619998579025292428923421539366245269775390625f,0.326600002759695140497342435992322862148284912109375f,0.88759997939467449867834147880785167217254638671875f,-0.971300023448467442221954115666449069976806640625f,0.47200001078844078250540405861102044582366943359375f,0.43120000873804098517894090036861598491668701171875f,-0.5263999690175058532304319669492542743682861328125f,-0.662599989974498537748104354250244796276092529296875f,0.440399986422062017510370424133725464344024658203125f,0.9886999792337416437248975853435695171356201171875f,-0.300200011017918655387148874069680459797382354736328125f,-0.5748000117808580444744848136906512081623077392578125f,-0.126500012896955116303843169589526951313018798828125f,-0.6806000270187855250014763441868126392364501953125f,-0.0235999831497668566271386225707828998565673828125f,0.2913000443100930425543992896564304828643798828125f,0.67040003350973176310390044818632304668426513671875f,-0.61379998589754070081880854559130966663360595703125f,-1.39560002597570331772658391855657100677490234375f,-0.57080002261400242247191272326745092868804931640625f,0.53959999867677677087840493186376988887786865234375f,0.47620004450082742408767444430850446224212646484375f,0.6938000179350376495079899541451595723628997802734375f,0.0377999954044818675669148433371447026729583740234375f,-0.60910000842213651850443056900985538959503173828125f,-0.6674000171780587464809286757372319698333740234375f,0.9090000290274620198260890902020037174224853515625f,1.752300011706353188856155611574649810791015625f,0.282800002557039409367689586360938847064971923828125f,-0.632199989098310322077622913639061152935028076171875f,-0.8577999922752379546864176518283784389495849609375f,0.40899998179078078663906126166693866252899169921875f,-1.00179999164938937639135474455542862415313720703125f,-0.8394999985694884259146419935859739780426025390625f,0.1841999975025654379123807302676141262054443359375f,-0.3507999989211558755641817697323858737945556640625f,-0.80829997373819395534155773930251598358154296875f,-0.644400029158592513311987204360775649547576904296875f,-0.342400004363060173062649482744745910167694091796875f,0.589799992334842926311466726474463939666748046875f,0.088399983084201494420995004475116729736328125f,-0.8034000158667566893200273625552654266357421875f,-0.4706000223994255549087029066868126392364501953125f,-0.307399996167421196702207453199662268161773681640625f,0.466000003904104342922209980315528810024261474609375f,0.27950001555681236453665405861102044582366943359375f,-0.4173999988496301938312171841971576213836669921875f,0.0464000023961066432320876629091799259185791015625f,-0.526199989962577774349483661353588104248046875f,0.95699998903274519079786841757595539093017578125f,0.507800002408027584266392295830883085727691650390625f,0.9457999713540079511631120112724602222442626953125f,-0.5727999979376789951857062987983226776123046875f,-0.033299998474121128566594052244909107685089111328125f,-0.2395000004693865758298443324747495353221893310546875f,-0.863699982102215102486297837458550930023193359375f,-0.462200002472102633443995500783785246312618255615234375f,0.070500033617019397524927626363933086395263671875f,-0.9916999919891367198943044058978557586669921875f,-0.2555999890208244806899529066868126392364501953125f,-1.459400020217895388441320392303168773651123046875f,0.4250000721216196097884676419198513031005859375f,-0.024499976664781542012860882095992565155029296875f,0.2998000140905379140576769714243710041046142578125f,-0.41100003266334539375748136080801486968994140625f,0.13499999701976772570333196199499070644378662109375f,-0.48590000707507119415140550700016319751739501953125f,0.99389999420046848399579175747931003570556640625f,0.3148999942302703747287750957184471189975738525390625f,1.89740000138282738362249801866710186004638671875f,-0.17840003762245260077179409563541412353515625f,0.1760000089704991665939814993180334568023681640625f,-0.9272999780297279670548959984444081783294677734375f,-0.1112999602675444066335330717265605926513671875f,-0.226999990671873153047499727108515799045562744140625f,-0.8008999660968783018688554875552654266357421875f,-0.189099983239174207483301870524883270263671875f,-0.44590000320076939033242524601519107818603515625f,-0.94089997444152828620644868351519107818603515625f,-0.05029998154640225038747303187847137451171875f,-0.0701999981284142648974011535756289958953857421875f,0.5192000024199483476650129887275397777557373046875f,-1.141200006949901535335811786353588104248046875f,-0.506200003969669243275575354346074163913726806640625f,-0.486200001883506871536155813373625278472900390625f,-0.4861999989032739222238888032734394073486328125f,-0.456300011974573038742164499126374721527099609375f,0.171299993050098464664188213646411895751953125f,-0.09369999979734444650603109039366245269775390625f,0.189400007998943298304084237315692007541656494140625f,0.6659000000715256106786910095252096652984619140625f,-0.161899990415573569180196500383317470550537109375f,0.6444000188767911030396362548344768583774566650390625f,0.3178000100076199263554599383496679365634918212890625f,-0.2329999994933604767766155418939888477325439453125f,0.3883999929189680955232688575051724910736083984375f,-0.3239999834299087666522609652020037174224853515625f,-0.77219998686313573443840141408145427703857421875f,-0.3658000013053415155894754207110963761806488037109375f,-0.5769999954402444952705764080747030675411224365234375f,-0.0365000120401379746226666611619293689727783203125f,0.467400012409687093395405099727213382720947265625f,0.2544000032305717695635394193232059478759765625f,-0.59589998964071266840392127051018178462982177734375f,-0.11120000829100629147205836488865315914154052734375f,-0.65400001963973064533774959272705018520355224609375f,0.2343999534606933110580939683131873607635498046875f,0.9314000333905223527608541189692914485931396484375f,0.4694000121712686990349538973532617092132568359375f,-1.0010999686956409959748270921409130096435546875f,0.633400010502338517426323960535228252410888671875f,-0.470999994218349371521981083787977695465087890625f,-1.01169999616146100152036524377763271331787109375f,-0.603400023615360492357240218552760779857635498046875f,-0.447400015985965904263821357744745910167694091796875f,0.3178999894022940253535125521011650562286376953125f,-0.277200008022785215899830291164107620716094970703125f,0.711000018060207406023209841805510222911834716796875f,0.43300003361702010806766338646411895751953125f,-0.3376000111341477261106547302915714681148529052734375f,0.1076000001072883183184103472740389406681060791015625f,0.224000010251999004395884185214526951313018798828125f,-0.452000028222799610233550993143580853939056396484375f,-0.152200008767843453227897043689154088497161865234375f,0.00200002062320692175489966757595539093017578125f,0.71560000764727593747238643118180334568023681640625f,0.28280001060366632970044520334340631961822509765625f,-0.7593999588251119092774388263933360576629638671875f,-0.63399999624490721572556140017695724964141845703125f,-0.23440001217126837929072280530817806720733642578125f,0.87259995748996743714087642729282379150390625f,0.03579999974071979007561594698927365243434906005859375f,-0.07020000207722187912651179431122727692127227783203125f,-0.1544999980852008736320613024872727692127227783203125f,0.445400020694732834414253375143744051456451416015625f,0.302199984180927305743580291164107620716094970703125f,-0.04489999914765352428958067321218550205230712890625f,1.1542999907553195271248114295303821563720703125f,0.4380000002384185631143509453977458178997039794921875f,-0.80119998817443871530485921539366245269775390625f,0.27200000929832413021358661353588104248046875f,0.0022000071287155975596760981716215610504150390625f,-0.15089999485611915730487453402020037174224853515625f,-0.953899953967333402715667034499347209930419921875f,-0.49569998823404315402285646996460855007171630859375f,-0.5994000077009200566635627183131873607635498046875f,0.81759999504089364563697017729282379150390625f,0.395500002324581156898375411401502788066864013671875f,0.33469998626708985511868377216160297393798828125f,-0.61169999854564593988470733165740966796875f,-0.513500013291835699646981083787977695465087890625f,-0.94379998679161047903107828460633754730224609375f,1.618400006628036180700291879475116729736328125f,0.735600006455182864328889991156756877899169921875f,-0.986199983406066849056514911353588104248046875f,0.506000024318694840985699556767940521240234375f,0.0072000025093554853583555086515843868255615234375f,0.1913999982237815089547439129091799259185791015625f,-1.179600000464915865450166165828704833984375f,-0.527000007361173583575464363093487918376922607421875f,0.3594000020384786608929061912931501865386962890625f,0.1393999793887132909731008112430572509765625f,0.343700009781122250984708443866111338138580322265625f,0.125999999135732565491707646287977695465087890625f,-0.399600006276369157376393559388816356658935546875f,-0.0565999984383582788183275624760426580905914306640625f,-0.1900999917626382540447593783028423786163330078125f,-0.7729000156283376554711139760911464691162109375f,-0.235600005859136540919962499174289405345916748046875f,-0.7030999702453613053876324556767940521240234375f,1.188499984383582841473980806767940521240234375f,0.5104999975562094505221466533839702606201171875f,0.884999979138374470721828402020037174224853515625f,-1.0241999770879743891782709397375583648681640625f,-0.4699999988079071044921875f,-0.179199998620152456396681373007595539093017578125f,-0.01099999794363970551103193429298698902130126953125f,-0.015400000727176677006724503371515311300754547119140625f,-0.38760001798868159994526649825274944305419921875f,-0.069400001330673699584394853445701301097869873046875f,-0.017200001764297478956677878159098327159881591796875f,0.264199998098611832375581798260100185871124267578125f,-0.4029999685734513281687441121903248131275177001953125f,-0.03599999637901785121130160405300557613372802734375f,0.006600001642107944377357853227294981479644775390625f,0.369999969191849853444864493212662637233734130859375f,-0.35179998062252959556417408748529851436614990234375f,0.99560001599192649024416823522187769412994384765625f,0.7885000150799754692343412898480892181396484375f,0.11400000309944147147689363919198513031005859375f,0.627000009298324556539228069595992565155029296875f,-0.1097999872684471966977071133442223072052001953125f,-0.42220000817179670349332809564657509326934814453125f,-0.37080000130534163105266998172737658023834228515625f,-0.3530999893188475624583588796667754650115966796875f,0.58760002335310002052892741630785167217254638671875f,0.18880001629591003364794232766143977642059326171875f,0.172999979972839668107553734444081783294677734375f,-0.095800004807114647054078204746474511921405792236328125f,0.6102000147432089516286168873193673789501190185546875f,0.537700023944676086529170788708142936229705810546875f,0.114600001135468421153973395121283829212188720703125f,0.2371999973684548734809141024015843868255615234375f,-0.142699992875754855248260355438105762004852294921875f,-0.52879999588131898491383253713138401508331298828125f,-0.00360002725720409699761148658581078052520751953125f,0.0039999756813049458514797152020037174224853515625f,0.2012000038206576135735303978435695171356201171875f,0.3083999984323977372469016700051724910736083984375f,0.299399986243248150685758446343243122100830078125f,0.4056000098824501520766716566868126392364501953125f,0.0948000045537948921037241234444081783294677734375f,-0.847599991613626091435662601725198328495025634765625f,0.7902000228643419887930576805956661701202392578125f,0.4836000247240068716791938641108572483062744140625f,-0.8632999647974965995445018052123486995697021484375f,0.531200017231702847908536568866111338138580322265625f,-0.750800026637315820465801152749918401241302490234375f,-1.0548000424027446086938653024844825267791748046875f,0.76000002667307864356871505151502788066864013671875f,0.29239999586939813269736987422220408916473388671875f,-0.8396000097036360187985337688587605953216552734375f,-0.43020000051259987827734221355058252811431884765625f,0.320000001899898023349777531620929948985576629638671875f,-0.571499995648860892316633908194489777088165283203125f,0.1292999940335750608255693805404007434844970703125f,0.2338000017672776653654409528826363384723663330078125f,0.47129997621178620903492628713138401508331298828125f,-0.01630000042617307798309411737136542797088623046875f,-0.059199998098611705810156990992254577577114105224609375f,0.760899985766410935639214585535228252410888671875f,0.26430000193118985407636500895023345947265625f,0.46560001703500741854213629267178475856781005859375f,-0.165600024485588193101648357696831226348876953125f,0.60719997971057892982571502216160297393798828125f,0.477900007432699336362702524638734757900238037109375f,-0.61969999759197236244290252216160297393798828125f,-0.91629996771812471934026689268648624420166015625f,-0.325399989998340632979534348123706877231597900390625f,0.480200000214576760981799452565610408782958984375f,1.013400002455711046422948129475116729736328125f,0.27480000336170196106877483543939888477325439453125f,0.5941000093534587467303254015860147774219512939453125f,0.3426999743983145663150935433804988861083984375f,0.312300002802908427401717972315964289009571075439453125f,-0.16300000753998755698148670489899814128875732421875f,0.32599998594820522868076295708306133747100830078125f,0.216099997901916529574606329333619214594364166259765625f,-0.24070004634857156133875832892954349517822265625f,-0.77909998800754465264617465436458587646484375f,-0.15649999982118600883040926419198513031005859375f,0.4530999851465224992352887056767940521240234375f,0.5422999741554264119258732534945011138916015625f,0.22629999364614494794523125165142118930816650390625f,-0.94200001525878906960542735760100185871124267578125f,0.590800006520748155480760033242404460906982421875f,0.304600007468462052884206059388816356658935546875f,-0.829199987816810590857130591757595539093017578125f,1.433000005006789479011786170303821563720703125f,0.68450001317262643141248190659098327159881591796875f,-1.0579000251650807484793404000811278820037841796875f,0.1517000256657610179900075308978557586669921875f,-0.207700009155273423999688020558096468448638916015625f,-0.78950002956390363806349341757595539093017578125f,1.2536999959230428203227347694337368011474609375f,0.54150000041723256316572587820701301097869873046875f,0.56679999120235446952165148104541003704071044921875f,-0.597599992358684772852939204312860965728759765625f,0.50300000366568564658109607989899814128875732421875f,-0.19819998331665988899885633145458996295928955078125f,-0.0357000052213667373735006549395620822906494140625f,0.5058000203788282078676274977624416351318359375f,-0.2168000056564807209724676795303821563720703125f,-0.2539000238537791886983541189692914485931396484375f,0.02899999713897705166942841970012523233890533447265625f,-0.31039999864101408544314608661807142198085784912109375f,-0.30470000101923933044645309564657509326934814453125f,0.16960000851154344303495236090384423732757568359375f,-0.67360000684261311931777527206577360630035400390625f,-0.889600012087822022976979496888816356658935546875f,0.5290000019818543730565352234407328069210052490234375f,-0.3643999935448170024443470538244582712650299072265625f,-0.29390000846832997627444683530484326183795928955078125f,0.31480000991821288636174358543939888477325439453125f,0.35319999568462367989241101895458996295928955078125f,-0.44019999276399612853083453956060111522674560546875f,-0.5424000170290470546063943402259610593318939208984375f,-0.4036000004351139036629092515795491635799407958984375f,0.305199999469518701289416640065610408782958984375f,0.056800001633167340742147644050419330596923828125f,0.78420002685785306084653711877763271331787109375f,0.034400041079521770370774902403354644775390625f,0.35699999666959036925817372321034781634807586669921875f,-0.17559999371469026119285672393743880093097686767578125f,-0.414299992897361468724426458720699883997440338134765625f,0.7291999921381471239101301762275397777557373046875f,-0.3909999901950360623459346243180334568023681640625f,-1.02739998230934137524172911071218550205230712890625f,0.571599998438358181118701395462267100811004638671875f,-0.375199989038705883359625659068115055561065673828125f,-0.2538000078767537193158432273776270449161529541015625f,-0.1755999836564061666877023526467382907867431640625f,0.7468000010371209640425149700604379177093505859375f,0.2738000088453294011969774146564304828643798828125f,0.055399996107816651402799834613688290119171142578125f,-0.088800000500679043824447944643907248973846435546875f,0.2349000044375653128980729889008216559886932373046875f,0.2112000053107738750668431748636066913604736328125f,-0.5837999964773654681948755751363933086395263671875f,-0.24560003938674856982515848358161747455596923828125f,0.09300000623613602357409035903401672840118408203125f,0.302499997913837415097759730997495353221893310546875f,0.2105000228881834800631622783839702606201171875f,0.5939000003099437208220479078590869903564453125f,0.01549999919533728842679920489899814128875732421875f,-0.074300000143050937140287715010344982147216796875f,0.5659000113964083311657304875552654266357421875f,0.0349999946355818991605701739899814128875732421875f,-0.352599995711445846069409526535309851169586181640625f,-0.2059999859482051220993525930680334568023681640625f,-0.1600999974995851449222072915290482342243194580078125f,-1.017200002360343802365605370141565799713134765625f,-0.30999995768070220947265625f,-0.28359999492168430634819742408581078052520751953125f,-0.0989999811947346319840335127082653343677520751953125f,0.49080000048577776539104888797737658023834228515625f,-0.05350000543147326315107648042612709105014801025390625f,-0.4764000169992446132027907879091799259185791015625f,0.67759998669624366129937698133289813995361328125f,0.39639999911785128716701365192420780658721923828125f,0.19790001160502423971365715260617434978485107421875f,-1.0596999837338927363816765137016773223876953125f,-0.4774999974668026059276826345012523233890533447265625f,-0.0201999954462053210590966045856475830078125f,0.64819998614787976976003847084939479827880859375f,0.548100015097856552159782950184307992458343505859375f,-0.36039998739063749955136017888435162603855133056640625f,0.362999996177852057854806844261474907398223876953125f,0.043099998166412134548153289870242588222026824951171875f,-0.9716999764919282966957325697876513004302978515625f,0.70269999306201924582637730054557323455810546875f,0.0919999960362911650690875831060111522674560546875f,-0.024899978435039660240590819739736616611480712890625f,0.5490999962031839487508477759547531604766845703125f,-0.0543000052839517355351972582866437733173370361328125f,0.192199976730346833164730924181640148162841796875f,0.3005999964714050065595074556767940521240234375f,0.4206000086903571855145855806767940521240234375f,0.346600022876262681137404797482304275035858154296875f,0.0379999917447566559758342918939888477325439453125f,-0.0785000077039004207080097330617718398571014404296875f,-0.1575000154972077126558360760100185871124267578125f,-0.12909997370243075920370756648480892181396484375f,-0.30279999540448176986728867632336914539337158203125f,0.593400006404519064773239733767695724964141845703125f,-1.0861999813944096526796556645422242581844329833984375f,0.6185999859809871992410990060307085514068603515625f,0.4050000107288358464074917719699442386627197265625f,-0.62629994626045260019964189268648624420166015625f,0.07720001279115695780319583718664944171905517578125f,0.83600001746416108261428234982304275035858154296875f,0.1682000274240973425321499234996736049652099609375f,0.44780001701116578516348454286344349384307861328125f,0.65020001958608641956516294158063828945159912109375f,-0.416499975830316060410041245631873607635498046875f,-0.141800007183849885450399597175419330596923828125f,-0.3756000121548772607837918258155696094036102294921875f,0.0614999909363682650820237540756352245807647705078125f,-0.3596000091075897131531746708787977695465087890625f,1.0102000329971314585009167785756289958953857421875f,0.77340004388094030218780972063541412353515625f,-0.12540000326037414879465359263122081756591796875f,0.415399998790025648531809565611183643341064453125f,0.0829000120520593242190443561412394046783447265625f,-0.066400002135336422259825894798268564045429229736328125f,-0.147800004829466369304924455718719400465488433837890625f,-0.0368000056006014819587335296091623604297637939453125f,-0.099800006565451682138956357448478229343891143798828125f,-0.484600009331107195453824942887877114117145538330078125f,-0.226700011397898304466025365400128066539764404296875f,-0.342000004976987792559839363093487918376922607421875f,-0.054200003761053050510554385255090892314910888671875f,-0.01899999207258229461103837820701301097869873046875f,0.0623999908030032912620299612171947956085205078125f,-0.58279999927878378684908966533839702606201171875f,-0.70679998509287855767979635857045650482177734375f,-0.120600011968612808033185501699335873126983642578125f,0.302600005322694709519737443770281970500946044921875f,-0.3170999682039019607060481575899757444858551025390625f,-0.35260000189542761717120811226777732372283935546875f,-0.19920000659227365957804067875258624553680419921875f,0.33809999170303317583829993964172899723052978515625f,0.15659999784231171116033465295913629233837127685546875f,-0.65140002504587191101137477744487114250659942626953125f,-0.54010002745091956199985361308790743350982666015625f,-0.164600009144842740482772569521330296993255615234375f,-0.3632000150933862858693146336008794605731964111328125f,0.4564999871365718053795035302755422890186309814453125f,-0.24719998507499685302946090814657509326934814453125f,0.5286000016644594179382465881644748151302337646484375f,-0.0370999922320247588203301347675733268260955810546875f,0.0135999964863062761111223153420723974704742431640625f,-0.1436999992758034583317794385948218405246734619140625f,0.5741000052928928454321066965349018573760986328125f,0.31700000065565081541762992856092751026153564453125f,0.7197000023603437046659792031277902424335479736328125f,0.729500023007393139096166123636066913604736328125f,0.12959999465346339064808489638380706310272216796875f,0.504900002314150331272912808344699442386627197265625f,-0.34909997251033786369589506648480892181396484375f,0.40599997469782866943432964035309851169586181640625f,-0.4617000013396139745935897735762409865856170654296875f,-0.711699995863437351317770662717521190643310546875f,0.23539999446868886678885246510617434978485107421875f,-0.7689000193089248202937824316904880106449127197265625f,-0.297000031761825766807305626571178436279296875f,-0.48440000077187972404857418950996361672878265380859375f,-0.3874000150546430543130327350809238851070404052734375f,-0.4821000225186349297246124478988349437713623046875f,-0.68919998647570590577515758923254907131195068359375f,-0.4254999945387243354133488537627272307872772216796875f,0.357100005829334321560963871888816356658935546875f,-0.48979998458623885682072796043939888477325439453125f,0.3443000274866820920038890108116902410984039306640625f,-0.89610004916191154933358120615594089031219482421875f,-0.094800001424550917050737552926875650882720947265625f,-0.75630003644973065046741567130084149539470672607421875f,-0.032900035297870999784208834171295166015625f,-0.740399979418516096529856440611183643341064453125f,0.0869000075146555417404670151881873607635498046875f,-0.2773000018715858772111459984444081783294677734375f,-0.27360000103116011160864218254573643207550048828125f,-0.3683000016406177135053212623461149632930755615234375f,-0.59490002105236072793559287674725055694580078125f,-0.03120000680088974576165128382854163646697998046875f,-0.4099000112548469854090171793359331786632537841796875f,-0.6660999889612195801191774080507457256317138671875f,0.0599999891221525416540316655300557613372802734375f,-0.79310001118630157890265763853676617145538330078125f,0.9724000407218937880315934307873249053955078125f,0.4176000045776362412652815692126750946044921875f,0.803600017943978173207142390310764312744140625f,-0.465100011357665099609448589035309851169586181640625f,0.3848000317484139287671496276743710041046142578125f,-0.1578000116467475066883707768283784389495849609375f,0.24959998437166230900174923590384423732757568359375f,0.132099993684887895284418846131302416324615478515625f,-0.041999994248151761855325503347557969391345977783203125f,-0.412099993945658216176752830506302416324615478515625f,-0.0731999956101179183320937227108515799045562744140625f,-0.19520000185370445233701275355997495353221893310546875f,-0.5607000034704803237417536365683190524578094482421875f,0.11380001450777077476317344917333684861660003662109375f,0.1193999886274337285385627183131873607635498046875f,-0.3142000153839585863124739262275397777557373046875f,0.13669999519288555944740437553264200687408447265625f,-0.002799996745586508239966860855929553508758544921875f,0.1911000086307523826434362490545026957988739013671875f,-0.19779999458491792641012807507649995386600494384765625f,-0.001800009381771161542928894050419330596923828125f,0.3265999940425157177514847717247903347015380859375f,0.463900007835030603331460952176712453365325927734375f,0.1753999892532827065139144906424917280673980712890625f,0.649199969190359382764654583297669887542724609375f,0.2824999929964544520544222905300557613372802734375f,0.424800008279085095086458068180945701897144317626953125f,0.39900000894814713348068835330195724964141845703125f,0.163399998432397719483333276002667844295501708984375f,-0.2217999989509582825508715586693142540752887725830078125f,-0.47169999297633768975401835632510483264923095703125f,-0.1656999956473708124349286663345992565155029296875f,0.1267999980568885742826523710391484200954437255859375f,0.4220999958455562417469764113775454461574554443359375f,0.08219999692142010427886589241097681224346160888671875f,-0.09020000081062296004574818653054535388946533203125f,-0.83360001384615856068194261752068996429443359375f,-0.496100020477175718269791104830801486968994140625f,-0.43619999398589126560210615934920497238636016845703125f,-0.722999989882111560035582442651502788066864013671875f,-0.2244999905228615144636705736047588288784027099609375f,-0.03300002169609062008248656638897955417633056640625f,0.68219997032284762639164910069666802883148193359375f,0.452899990370869698352862542378716170787811279296875f,-0.81060001941919335166630844469182193279266357421875f,-1.13440000993609402257789042778313159942626953125f,-0.57869999793469872884088545106351375579833984375f,-0.32840001214146585795106147998012602329254150390625f,-1.243800033357739476258529975893907248973846435546875f,0.445000002533197491771943532512523233890533447265625f,-0.4825999793946744720329888878040947020053863525390625f,0.349200019593536747919415574870072305202484130859375f,-0.4235000093802809839615974851767532527446746826171875f,0.1537999965518712652823296593851409852504730224609375f,-0.284300008673965987071596828172914683818817138671875f,-0.6721000338062648182813063613139092922210693359375f,-0.27179999887645223122234483525971882045269012451171875f,-0.443700007657706674280007064226083457469940185546875f,-0.906799995672702863203085144050419330596923828125f,0.1013999906241895843095335294492542743682861328125f,-0.705399991227686040673461320693604648113250732421875f,0.610900002604722658361424691975116729736328125f,0.190199982631206854222227775608189404010772705078125f,0.46649998991191399255740179796703159809112548828125f,-0.2568000122874976032250060598016716539859771728515625f,-0.69219998582005481946310965213342569768428802490234375f,-0.24339998565465192914558656411827541887760162353515625f,0.20019998546242678827411509701050817966461181640625f,-0.28979998801350603798709926195442676544189453125f,0.0587999955460427603526341044926084578037261962890625f,-0.4317999909415839443482809656416065990924835205078125f,0.3151999892622232568584195178118534386157989501953125f,-0.45300001130253075842801990802399814128875732421875f,-0.03930000198706984360086380547727458178997039794921875f,0.01719999785274273218504959004349075257778167724609375f,0.05570000304207212149520955790649168193340301513671875f,0.27110000505447384711033009807579219341278076171875f,-0.039799999785423256781768941436894237995147705078125f,0.14910000030100345469463718472979962825775146484375f,-0.28499998182058305928876507095992565155029296875f,0.7465999944150443212720347219146788120269775390625f,-0.117200000311434404665078545804135501384735107421875f,0.065599980750679964813798505929298698902130126953125f,-0.156999989330768752182621028623543679714202880859375f,0.19979999781101920319770215428434312343597412109375f,0.3500000032782555337007579510100185871124267578125f,-0.83259997045397771131547415279783308506011962890625f,0.2046000309005380213278613155125640332698822021484375f,0.6962000051617618368027251563034951686859130859375f,-0.14900000831484749852506865863688290119171142578125f,0.8060000273361807021643699044943787157535552978515625f,0.1911999905586245773747577914036810398101806640625f,-0.5233000124067068714595052369986660778522491455078125f,0.121800002825260100802751139781321398913860321044921875f,0.252900007209181654221907820101478137075901031494140625f,0.0731000007182358857438231325431843288242816925048828125f,0.03619999815821639543855781084857881069183349609375f,0.42860000952482213421035339706577360630035400390625f,-0.261900017684698394049291891860775649547576904296875f,-0.3743999986112118705250395578332245349884033203125f,-0.539499987840652561743581827613525092601776123046875f,-0.4081999924063681728370056589483283460140228271484375f,0.2037999900698661548403833876363933086395263671875f,0.844999983459711057065533168497495353221893310546875f,0.6445000089257957309740731943747960031032562255859375f,-0.2905999933421610226247366881580092012882232666015625f,-0.050299996224045795401025316095910966396331787109375f,0.0101000088244675989557208595215342938899993896484375f,0.0897999927818775578458598829456605017185211181640625f,0.60619998713135725409983933786861598491668701171875f,0.198099994981288940465447012684307992458343505859375f,-0.018400013631582279316489803022705018520355224609375f,0.279199980813265025147984488285146653652191162109375f,-0.1058000085324046768420203079585917294025421142578125f,-0.70679999731183063005346411955542862415313720703125f,-1.08709998109340677530099128489382565021514892578125f,-0.775999988853931288446119651780463755130767822265625f,-0.045600001090765118760828045196831226348876953125f,0.10470000176429739013883590814657509326934814453125f,-0.285399995809793249890162769588641822338104248046875f,0.342800002408027548739255507825873792171478271484375f,0.7462000082910058562646327118272893130779266357421875f,0.43900000190734844718321028267382644116878509521484375f,-0.20699999558925608056370037957094609737396240234375f,-1.125700010883807777872789301909506320953369140625f,-0.5397000051915643581423864816315472126007080078125f,-0.33939999771714202125849624280817806720733642578125f,-0.1117999920964241056253740680404007434844970703125f,-0.41090000931024572849992182455025613307952880859375f,-0.280800008308887516506047177244909107685089111328125f,0.115499994128942518045732867904007434844970703125f,-0.262800000992417182033022982068359851837158203125f,-0.316399988687038558055064640939235687255859375f,-0.5193999680638317073544385493732988834381103515625f,0.15220002545714361730233576963655650615692138671875f,-0.25259999116659148654662203625775873661041259765625f,0.5356999949768184166032369830645620822906494140625f,0.0413000068336726311457596239051781594753265380859375f,0.01519999768137936246858998856623657047748565673828125f,-0.01409999974220987883910538585041649639606475830078125f,1.0000000533461577134630715590901672840118408203125f,0.25939999771714195020422266679815948009490966796875f,0.7442000247344375818414619061513803899288177490234375f,0.52590000066757180974263974349014461040496826171875f,0.115399987912178314530819989158771932125091552734375f,0.53750000074505821512360626002191565930843353271484375f,-0.1424000258207325231296636047773063182830810546875f,-0.7669999859035012690355870290659368038177490234375f,-0.2421999993428587405475127525278367102146148681640625f,0.4320999723762273259097810296225361526012420654296875f,-0.43799997624754938296831596744596026837825775146484375f,0.39309999285787300404848565449356101453304290771484375f,0.42640000224709506682074788841418921947479248046875f,-0.29899998953938489165693681570701301097869873046875f,0.257000006876885844775415534968487918376922607421875f,0.6224000121116635142470840946771204471588134765625f,0.02279999197721505055369561887346208095550537109375f,0.467600008302927250269931391812860965728759765625f,0.89870000009536710194879560731351375579833984375f,0.245399981653690701932646334171295166015625f,0.947500000968575672999349990277551114559173583984375f,0.70010000249147363859947290620766580104827880859375f,0.262999976396561141456231780466623604297637939453125f,0.55409998949766181564058342701173387467861175537109375f,-0.2089999928176400345591900986619293689727783203125f,-0.264799993824958779242706441436894237995147705078125f,-0.3037999950617551458975640343851409852504730224609375f,-0.441700014340877800123053020797669887542724609375f,0.71319998689293839788660989142954349517822265625f,-0.19150002206116933933799373335205018520355224609375f,1.10800002229213756521630784845910966396331787109375f,0.043000012010335542100847305846400558948516845703125f,0.94780001451522088995460535443271510303020477294921875f,1.10370000665187806276890114531852304935455322265625f,-0.125400003111362234875514332088641822338104248046875f,0.90930000848770153876188260255730710923671722412109375f,-0.453800025534629725143531686626374721527099609375f,-0.5167999897122381725012019160203635692596435546875f,-0.256599998810887353073439953732304275035858154296875f,-0.1547000086188319301072624512016773223876953125f,-0.10299999494850642367538284815964289009571075439453125f,0.2921999876081944336192464106716215610504150390625f,0.941299973827600666709258803166449069976806640625f,0.3260999869495631031668381183408200740814208984375f,0.53280000225901602561862091533839702606201171875f,0.53279999391436572153679662733338773250579833984375f,0.636600005218386666427932141232304275035858154296875f,-0.1257999953448774022035649977624416351318359375f,-0.8038999801933766065076270024292171001434326171875f,-0.64890000783503065662216613418422639369964599609375f,0.116399999415874422226124806911684572696685791015625f,0.3994000012934206811365811518044210970401763916015625f,0.19340000379681587805436038252082653343677520751953125f,0.047599997946619954092284388025291264057159423828125f,0.207599998579919287067241384647786617279052734375f,0.2370000056102872232344225267297588288784027099609375f,-0.5053999964058402127875524456612765789031982421875f,-0.61949997040629423139534992515109479427337646484375f,-0.493099980154633410478481891914270818233489990234375f,0.47959999063014979725494413287378847599029541015625f,0.825199981737136933901410884573124349117279052734375f,0.2881999883830548281338224114733748137950897216796875f,-0.151400007909536338246425657416693866252899169921875f,0.25749999374151233411112116300500929355621337890625f,0.176299999085068659354647024883888661861419677734375f,0.2785999990940093429259150070720352232456207275390625f,0.56449999924004068407867862333660013973712921142578125f,0.312699999916553450507450406803400255739688873291015625f,0.027799999129772157147044708835892379283905029296875f,-0.0608000032424926839524914612411521375179290771484375f,0.15810000936090962131430615045246668159961700439453125f,-0.0275999944448470646563009722740389406681060791015625f,-0.442999997109174703524558935896493494510650634765625f,-0.0077999937653541184801042618346400558948516845703125f,0.22440000129342074774996262931381352245807647705078125f,0.5282000008627771858726873688283376395702362060546875f,0.1051999985009431648474986786823137663304805755615234375f,-0.0098000006794929372944125134381465613842010498046875f,0.0682999965369701644846145427436567842960357666015625f,-0.09750000104308120807417026298935525119304656982421875f,0.5183999915778638722940740990452468395233154296875f,1.038599966162443433148609983618371188640594482421875f,0.4285999857574702698315149973495863378047943115234375f,0.20159999486207968999451622948981821537017822265625f,0.1411000069171191029226974933408200740814208984375f,0.3569000079989432805405158433131873607635498046875f,-0.25639998913407335834335754043422639369964599609375f,0.1331000079080462139557994305505417287349700927734375f,-1.19040001950263985008859890513122081756591796875f,-0.1643999978661536687241095933131873607635498046875f,-1.090800013971328752404588158242404460906982421875f,0.070500040471554115129038109444081783294677734375f,0.8783999686300756337686834740452468395233154296875f,0.168700006838143234944027426536194980144500732421875f,0.07619999726414672380769843584857881069183349609375f,-0.1903999992370604132929656771011650562286376953125f,-0.00260000047981734372370965502341277897357940673828125f,-0.15760001012831936773039842591970227658748626708984375f,-0.292399995124339950880454352954984642565250396728515625f,-0.15719999911934134928515049978159368038177490234375f,0.77930000133514365501241627498529851436614990234375f,0.049199975150824304392926933360286056995391845703125f,0.76830000201314729846302498117438517510890960693359375f,-0.0935999832987786106741623370908200740814208984375f,0.1107999910235406559877446852624416351318359375f,-0.2403999979704616318798571228398941457271575927734375f,0.373900000458955883431144684436731040477752685546875f,-0.5315999807059765824845953829935751855373382568359375f,0.290500018157064943125078571029007434844970703125f,-0.16149999691545957603011629544198513031005859375f,-0.1463999985218047417134812349104322493076324462890625f,-0.2352999989435076155785964147071354091167449951171875f,-0.39660002093911206344500897102989256381988525390625f,-0.141000003606080781537457369267940521240234375f,-0.31400000946968786852409039056510664522647857666015625f,-0.60140001774430285053085754043422639369964599609375f,0.1015999981403350460595902404747903347015380859375f,-0.425000012293458073742158376262523233890533447265625f,0.217000006355345220754315960220992565155029296875f,0.04679999544918544973626239880104549229145050048828125f,0.0793999957427382607733079566969536244869232177734375f,0.277499999552965004312454766477458178997039794921875f,0.0887999955832958765000739731476642191410064697265625f,0.215299995441734781653764230213710106909275054931640625f,-0.0706000051140787121539688087068498134613037109375f,-0.0735999910473825735834907391108572483062744140625f,-0.2526000096440317310708678633091039955615997314453125f,0.4485000466704374133541932678781449794769287109375f,-0.238100015470385617533111144439317286014556884765625f,0.61419999258518220841551737976260483264923095703125f,1.1108999854683876407079878845252096652984619140625f,0.81769998874068239302914662403054535388946533203125f,0.50720000161528577820035934564657509326934814453125f,0.668599999541044098805286921560764312744140625f,0.4582000026136636705587079632095992565155029296875f,0.1810000000670550779346967829042114317417144775390625f,-0.04029999583289034514876902903779409825801849365234375f,-0.082899999330192708502096365918987430632114410400390625f,0.35380001093149193724940460015204735100269317626953125f,0.27740000279843801678225645446218550205230712890625f,0.198299999591708075996621118974871933460235595703125f,-0.4174000174760816861407874966971576213836669921875f,-0.5493000260710712012723888619802892208099365234375f,0.07589999545216574716732793604023754596710205078125f,-0.1872000098109243282351599191315472126007080078125f,-0.071700021344422992797262850217521190643310546875f,0.2311999955505132930966283311136066913604736328125f,0.054200002717971873522628811770118772983551025390625f,0.4139000055253505561125848544179461896419525146484375f,-0.22870002006292378116114605290931649506092071533203125f,-0.1194000041618943352972337379469536244869232177734375f,-0.12480000129416579657259944724501110613346099853515625f,-0.123300004341453328660094257429591380059719085693359375f,0.3120000008046626138735746280872263014316558837890625f,0.80310000444352602766429072289611212909221649169921875f,0.6666000106573104488916214904747903347015380859375f,0.07659999873638145828635970246978104114532470703125f,0.579900008237361763718809015699662268161773681640625f,-0.0247000104069712289600602161954157054424285888671875f,0.8302000073671340629744008765555918216705322265625f,1.337700002300739043903377023525536060333251953125f,0.8809000002205369384000732679851353168487548828125f,-0.167400007641315351492039553704671561717987060546875f,-0.5668000183224675136983705669990740716457366943359375f,0.09650000376999401385802457298268564045429229736328125f,-0.5281999934494494564063415964483283460140228271484375f,-1.320399989476799706977772075333632528781890869140625f,-0.8541000048458575744092513559735380113124847412109375f,0.1450000125169754738863048260100185871124267578125f,-0.06039998984932903880462617962621152400970458984375f,-0.314500008627772498215335872373543679714202880859375f,-0.48580000622272478949525975622236728668212890625f,-0.24770000151842819224867753291619010269641876220703125f,-0.76349998766183801990337087772786617279052734375f,-0.4459999785721304732533099013380706310272216796875f,-0.828899983881413948694216742296703159809112548828125f,-0.65260000249147420703366151428781449794769287109375f,-0.016800006997585104073777984012849628925323486328125f,-0.67300000917911517728242642988334409892559051513671875f,-0.252699989392608570515363908270956017076969146728515625f,0.39199997562170063591935331714921630918979644775390625f,-0.22330000575706332455183655838482081890106201171875f,-0.542400009503960500722996584954671561717987060546875f,0.1998000015735623691170985694043338298797607421875f,-0.51700001969188480455130729751545004546642303466796875f,-0.193300013077259080773728783242404460906982421875f,-0.5635999745070936484125923016108572483062744140625f,-0.030299976219236857133410012465901672840118408203125f,-0.143500015974044714539559208787977695465087890625f,-0.5565999698281292040746848215349018573760986328125f,-0.079499986797571420282793042133562266826629638671875f,0.891900042718649643802564241923391819000244140625f,0.248800009590386839164466437068767845630645751953125f,0.656900016827881128023136625415645539760589599609375f,0.227400011720508443691102229422540403902530670166015625f,0.0138000002026557877510271055143675766885280609130859375f,0.2136000108473003755715780016544158570468425750732421875f,-0.43590001460015770362588227726519107818603515625f,-0.6185999864280222748647020125645212829113006591796875f,-0.48749999888241291046142578125f,0.770700035619736212311181589029729366302490234375f,-0.02719999535679828994716444867663085460662841796875f,0.537300019790232230576521033071912825107574462890625f,-0.915100003087520263989063096232712268829345703125f,-0.2675999886333944033367515658028423786163330078125f,-1.007700009714067146404659069958142936229705810546875f,-0.7034000273406508796369962510652840137481689453125f,-0.205799993258714675192777576739899814128875732421875f,-0.4720000022575259190915630824747495353221893310546875f,0.353400011172890771149468491785228252410888671875f,0.6857999871492383991977703772136010229587554931640625f,0.513199997733533308519326965324580669403076171875f,0.15620001559257534751168350339867174625396728515625f,0.638599990600347222624577625538222491741180419921875f,0.11119998605102277000611366020166315138339996337890625f,1.13130002613067670580448975670151412487030029296875f,-0.1559000058829784762792769470252096652984619140625f,0.263200021314621057655358526972122490406036376953125f,0.05500001385807973264263637247495353221893310546875f,-0.115999993175268212297623904305510222911834716796875f,0.2218000026762485532572100055404007434844970703125f,0.292399995422363279118371792719699442386627197265625f,-0.0903000022590159101554263543221168220043182373046875f,0.617600004577636862990175359300337731838226318359375f,0.76839997950792326975033574854023754596710205078125f,0.091199989664554781398919658386148512363433837890625f,-0.26140001163482651236336096189916133880615234375f,-0.45860001652836768215593110653571784496307373046875f,0.251099995741248083191976547823287546634674072265625f,0.242200002509355361013376750634051859378814697265625f,-0.22279999138116846069124221685342490673065185546875f,-0.119699999082088393009826177149079740047454833984375f,-0.51519998188614835044063511304557323455810546875f,-0.793799972933530995078399428166449069976806640625f,-0.4041999868482353264198536635376513004302978515625f,0.390599994981288833884036648669280111789703369140625f,1.020599993044137843156704548164270818233489990234375f,0.4894999995380640012143658168497495353221893310546875f,0.26219999878406519133733354465221054852008819580078125f,0.5785999956294893831199033229495398700237274169921875f,0.34810000608265412314068498744745738804340362548828125f,0.0741999931439758508844306561513803899288177490234375f,0.39019998618885887164964287876500748097896575927734375f,0.504699997536092848093858265201561152935028076171875f,-0.15680000222921375296891710604541003704071044921875f,0.00260000413060179624835654976777732372283935546875f,0.217600009047985043508788294275291264057159423828125f,-0.6311999847471714275570775498636066913604736328125f,-0.647999971702695187758536121691577136516571044921875f,-0.6707999808162450339210636229836381971836090087890625f,-0.29540000415444378489837617962621152400970458984375f,-0.366299989622831390079227276146411895751953125f,0.175500002399086696414087782613933086395263671875f,-0.07439999712109557350458999280817806720733642578125f,0.40720000012516965881559372064657509326934814453125f,0.262600008973479237539550013025291264057159423828125f,-0.4695999936103820715516121708787977695465087890625f,-0.38439998803138752947461398434825241565704345703125f,0.134000029921531815801927223219536244869232177734375f,0.08139999777674677972072458942420780658721923828125f,-0.0995000368282201730352198865148238837718963623046875f,-0.204600020283460803938169192406348884105682373046875f,0.5299999880790708939315436509787105023860931396484375f,-0.207800041151047165666199134648195467889308929443359375f,0.184199999737739261718161287717521190643310546875f,0.36299998429417623668769010691903531551361083984375f,0.08119998217672108609122005873359739780426025390625f,-0.109200007188320569184725172817707061767578125f,0.58799998414516441158639281638897955417633056640625f,-0.2660000359416010429214338728343136608600616455078125f,-0.6216000091671940452897615614347159862518310546875f,-0.57779996917843856607532870839349925518035888671875f,-0.4057999749675393896808373028761707246303558349609375f,-0.5769999860525132362454314716160297393798828125f,0.58659996818900150827857942203991115093231201171875f,-0.4700000083073969658897794943186454474925994873046875f,0.392199979710579071934262174181640148162841796875f,-0.0263999874949458757100728689692914485931396484375f,0.480599988722800741669516355614177882671356201171875f,0.0344000026345252507553595933131873607635498046875f,-0.33939999592900260694250391679815948009490966796875f,-0.148199997062981037032614040072076022624969482421875f,-0.16800001271814135606064155581407248973846435546875f,-0.22060000161230541859680442939861677587032318115234375f,-0.23980000850260252409640315818251110613346099853515625f,-0.364400010774284355274943436597823165357112884521484375f,-0.64819996975660354454618072850280441343784332275390625f,-0.34779998618438856539114567567594349384307861328125f,-0.065000012069940549253033168497495353221893310546875f,-0.5155999863386153680977486146730370819568634033203125f,-0.029799990397691711319083651687833480536937713623046875f,1.2503999797910447799864641638123430311679840087890625f,0.09359999194145218748275283360271714627742767333984375f,1.16639998116195187094490393064916133880615234375f,-0.2998999921441072302741304156370460987091064453125f,-0.56079997627139110960570178576745092868804931640625f,-0.1544999679848551021876801314647309482097625732421875f,0.31999997526407231163148026098497211933135986328125f,-0.569999978244304816854537421022541821002960205078125f,0.21879999528527227514729247559444047510623931884765625f,0.0539999434947961987063536071218550205230712890625f,-0.76639998362064343240263042389415204524993896484375f,0.1555999861896035785235881121479906141757965087890625f,0.4413000133156774751341799856163561344146728515625f,-0.564000013902783425834286390454508364200592041015625f,0.2815999865174294569669655174948275089263916015625f,0.378799971294403459154409574694000184535980224609375f,-0.0988000280678271725065542341326363384723663330078125f,-0.13159999650120735026348484097979962825775146484375f,0.36020000781416872115414662403054535388946533203125f,0.293600014814734411316976547823287546634674072265625f,0.0591999976515769077423101407475769519805908203125f,-0.25899999892711633719954988919198513031005859375f,-0.322000009596347780416181194595992565155029296875f,-0.1938000031828881475348680396564304828643798828125f,0.3474000060021875668780921841971576213836669921875f,-0.26629999848902219383717238088138401508331298828125f,-0.215200006771087526402652656543068587779998779296875f,-0.4637000181257722175587332458235323429107666015625f,0.06080000599920776238604958052746951580047607421875f,-0.320399996256828245577708003111183643341064453125f,-0.31719999639987950246222681016661226749420166015625f,0.020200015562772843935590572073124349117279052734375f,0.058800003629922770187476999126374721527099609375f,-0.19449998664855971952647450962103903293609619140625f,0.260800008010864114993410112219862639904022216796875f,-0.477799991530179823939761263318359851837158203125f,-0.70269999201893806883845172706060111522674560546875f,-0.093699984672665603824270874611102044582366943359375f,-0.25180000923275969171299948357045650482177734375f,0.01460000761747348718699868186376988887786865234375f,-0.345100002938508954031249231775291264057159423828125f,0.086600017362833181522319137002341449260711669921875f,-0.328599992090463643990005948580801486968994140625f,-0.26149999922513966765791337820701301097869873046875f,0.576999987542629266812355126603506505489349365234375f,1.1838999718487264178889972754404880106449127197265625f,0.61429998971223842563205153055605478584766387939453125f,-0.340799985212087708230654925500857643783092498779296875f,-0.927599967734515917783255645190365612506866455078125f,-0.2985999862790109649068881481071002781391143798828125f,0.34560000198483453726083780566114000976085662841796875f,0.430500007316469979201656315126456320285797119140625f,0.2053999990880488457634100996074266731739044189453125f,-0.22139999837279322747463083942420780658721923828125f,-0.8402999903380869994862223393283784389495849609375f,-0.53390000947415838794540832168422639369964599609375f,0.318999985814094610958591147209517657756805419921875f,0.047000003114342714383155907853506505489349365234375f,0.2196000362277032991187297739088535308837890625f,0.43839999470710733220357724349014461040496826171875f,0.090800012481212721837664503254927694797515869140625f,0.3025999741792677610874306992627680301666259765625f,-0.68199998009204865212495860760100185871124267578125f,0.2185999982744453173921783672994934022426605224609375f,0.02540002784729011864328640513122081756591796875f,0.57799997758865373498338158242404460906982421875f,0.0422000041484835719529655762016773223876953125f,1.3000000140070913090539761469699442386627197265625f,-0.05900000831484764063361581065692007541656494140625f,1.0266000063732267921778884556260891258716583251953125f,-0.4051000322937969855274786823429167270660400390625f,-0.2637999924540519458560083876363933086395263671875f,-0.23870000605583197028636277536861598491668701171875f,0.06959996470212903574292795383371412754058837890625f,-0.415599987828731531891435224679298698902130126953125f,0.13879999044239499372821455835946835577487945556640625f,-0.8187000036716458595265066833235323429107666015625f,-0.18459999629259105091705350787378847599029541015625f,-0.6402999941378830772720220920746214687824249267578125f,-0.040300039660930764284785254858434200286865234375f,-0.39279997901320484743337146937847137451171875f,0.118699990148841916237643090425990521907806396484375f,1.0332000335335731477925946819595992565155029296875f,0.05379998455643697496952881920151412487030029296875f,0.8606000232562427409988004001206718385219573974609375f,-0.226000014930963555315202029305510222911834716796875f,0.5633999936640259331710467449738644063472747802734375f,-0.154600031794608072654995112316100858151912689208984375f,-0.2584999973475934353928096243180334568023681640625f,-0.265199994403123806563371545053087174892425537109375f,-0.4282999979153274683341123818536289036273956298828125f,-0.2932000058546659904123998785507865250110626220703125f,0.1435999964863061695297119513270445168018341064453125f,-0.0768000033095479484046563811716623604297637939453125f,-0.309699985596537441523423694889061152935028076171875f,0.132199989694357089575760255684144794940948486328125f,-0.347699999693035888270742361783050000667572021484375f,0.0025000289082530713358210050500929355621337890625f,0.59799998578429214290963500388897955417633056640625f,0.1323000103279949879464538753381930291652679443359375f,-0.2052999998748301635487223393283784389495849609375f,0.049500008702278108785321819595992565155029296875f,-0.0212000126123428334068421463598497211933135986328125f,0.377999981611967239558680375921539962291717529296875f,-0.0358000072658060108921063147136010229587554931640625f,0.46339999798536302133555864202207885682582855224609375f,0.427799988680332976542786127538420259952545166015625f,0.2060999922022224151163527494645677506923675537109375f,-0.0656000049650666827716349871479906141757965087890625f,0.319499978154897945614720811136066913604736328125f,-0.0542000098705290866973882657475769519805908203125f,0.0834000012636184617775825245189480483531951904296875f,0.173000001467764341622768142769928090274333953857421875f,0.0106999978452920474492060520788072608411312103271484375f,0.29320001580119114947819980443455278873443603515625f,0.465500028252601172340519042336381971836090087890625f,-0.0095999913752080612283634764025919139385223388671875f,0.17879999468922613825583312063827179372310638427734375f,0.309899989946186560363372564097517170011997222900390625f,0.53160000797510154058045372948981821537017822265625f,0.5116000207901001406440855134860612452030181884765625f,0.5351000170946118572601335472427308559417724609375f,0.17180000126063799115172514575533568859100341796875f,-0.020399980312586052377810119651257991790771484375f,-0.28769995878934917499236689764074981212615966796875f,0.322400034016370629075254328199662268161773681640625f,-0.0724000110685825148948424612171947956085205078125f,0.3590999871134759047208717674948275089263916015625f,0.11639999420046809319728708942420780658721923828125f,-0.16759999563694005786373963928781449794769287109375f,-0.1226999890387059366503308410756289958953857421875f,-0.43980000060498714020695842918939888477325439453125f,-0.6630000032186507130660402253852225840091705322265625f,-0.90610000311732274980158763355575501918792724609375f,-0.458899994200467897798034755396656692028045654296875f,-0.216400000905990541610890431911684572696685791015625f,-0.4099000028729438138697105387109331786632537841796875f,0.14460001067221173176591264564194716513156890869140625f,0.608800008398294512090842545148916542530059814453125f,0.7142999968647956166023504920303821563720703125f,0.5105999993026255623362885671667754650115966796875f,-0.30960000225305550625165551537065766751766204833984375f,-0.81860000520348530361758321305387653410434722900390625f,-0.199999995976686328891247512729023583233356475830078125f,-0.392000007361173796738285091123543679714202880859375f,0.27000001419335595276294270661310292780399322509765625f,0.66240000838637325131230682018212974071502685546875f,-0.10040000221729261653536013909615576267242431640625f,0.4884000083416701887273347892914898693561553955078125f,-0.52719999250695115389220291035599075257778167724609375f,0.31059998764246710134528939306619577109813690185546875f,-0.53820000874176610050625413350644521415233612060546875f,0.68449999275803552478691926808096468448638916015625f,-0.12659998413324391464129803352989256381988525390625f,0.51929999686479533238525618799030780792236328125f,0.09160000570267445851868615136481821537017822265625f,0.06059999893009659910347863842616789042949676513671875f,0.040999999508261664171726579297683201730251312255859375f,0.1845000084042549559626422706060111522674560546875f,0.31619998131990456613493734039366245269775390625f,0.0508999828606844229028638437739573419094085693359375f,-0.7167000231698157364235157729126513004302978515625f,-0.49979998972713934879408270717249251902103424072265625f,-0.635100000666081854205913259647786617279052734375f,-0.39790000355839705292737562558613717555999755859375f,0.39499999612569780538251507095992565155029296875f,-0.4543000289767984423150437578442506492137908935546875f,-0.91510003616809854065650142729282379150390625f,-0.3550000026822086596212102449499070644378662109375f,-0.69510001791417597161171215702779591083526611328125f,0.480500007390975980570146930404007434844970703125f,-0.3661999878764152782650853623636066913604736328125f,0.352500017210841232184748150757513940334320068359375f,0.82870001350641242510164374834857881069183349609375f,0.343399985766410953402782979537732899188995361328125f,0.70109999618828300338435610683518461883068084716796875f,0.57869998206496209292026833281852304935455322265625f,-0.14679999313950542472184679354541003704071044921875f,0.664100002871453654051947523839771747589111328125f,-0.4577000289738180072163231670856475830078125f,-0.388599987471103691660800905083306133747100830078125f,-0.2869000019639731391407622140832245349884033203125f,-0.85190001187324515541376968030817806720733642578125f,0.056600003504752915972630944452248513698577880859375f,-0.85230002034902596630416837797383777797222137451171875f,-0.345400016373396123725569850648753345012664794921875f,0.0323999995946884222774997397209517657756805419921875f,-0.1358000020503997706811105672386474907398223876953125f,0.52370001201629623466260454733856022357940673828125f,0.214700003328919475364955360419116914272308349609375f,-0.56960000523328790933419440989382565021514892578125f,-0.7430999917030334245282574556767940521240234375f,-0.72120000240504733568513984209857881069183349609375f,-0.321799996268749222139149424037896096706390380859375f,-0.61019999082684517333063922706060111522674560546875f,-0.332699997010827086540984964813105762004852294921875f,0.13520002092719085595717842807061970233917236328125f,-0.454999983683228581554658376262523233890533447265625f,-0.4627999998003244908062470130971632897853851318359375f,0.5648000002324577994983201278955675661563873291015625f,0.86620001529455148414626819430850446224212646484375f,0.422200001242756695063462757389061152935028076171875f,-0.0381999983668326326124997649458236992359161376953125f,-0.464100006932020026084728669957257807254791259765625f,-0.0420999983787535558832360038650222122669219970703125f,0.063800010633468406950896678608842194080352783203125f,-0.65889997020959878426538125495426356792449951171875f,-0.303899998223781775408269822946749627590179443359375f,-0.238600009673833657330987989553250372409820556640625f,0.2270999749839308634591361624188721179962158203125f,-0.17010000658929325112467267899774014949798583984375f,0.399599998527765176703496763366274535655975341796875f,0.176299996253848068050729125388897955417633056640625f,0.2886000001370907508402297025895677506923675537109375f,0.15639999755322919750000210115103982388973236083984375f,-0.0609999979063868824624705666792578995227813720703125f,-0.02940000170320267169898897918756119906902313232421875f,0.00519999797940240426896707504056394100189208984375f,0.4171000102996824665524400188587605953216552734375f,-0.33560001226663604967370702070184051990509033203125f,0.003400004541874057650829854537732899188995361328125f,0.6121999992311000227118711336515843868255615234375f,0.00069999665319908643823509919457137584686279296875f,0.3995999940574168629581208733725361526012420654296875f,0.97219998947083940965541160039720125496387481689453125f,0.750400006613135328592534278868697583675384521484375f,-0.1356000100314618261876375981955789029598236083984375f,0.35229999844431869604477469692938029766082763671875f,0.145200002598762534233856058563105762004852294921875f,0.242199999454617564964564735419116914272308349609375f,0.3885999923512339915276925239595584571361541748046875f,0.42670000022202725364905973037821240723133087158203125f,0.1404000043034552280829529991024173796176910400390625f,0.199500005908310351632195533966296352446079254150390625f,1.078300023657083439587722750729881227016448974609375f,0.2797999845862391143924696734757162630558013916015625f,1.031100016081333503592531997128389775753021240234375f,0.20319999896287921359316896996460855007171630859375f,0.120199998575448940840715295053087174892425537109375f,0.28020000069886441540489840917871333658695220947265625f,-0.1778000033766031773296845130971632897853851318359375f,0.93159997727870924588700063395663164556026458740234375f,-0.10820003629028829283953427875530906021595001220703125f,-0.40219997151494002451954656862653791904449462890625f,0.05679998166561173178479293710552155971527099609375f,-0.50419999184012365134321953519247472286224365234375f,-0.55850001761317269455275891232304275035858154296875f,0.1355999993026255623362885671667754650115966796875f,-0.386700011546909916404501927900128066539764404296875f,-0.611299972486495590828781132586300373077392578125f,0.68239997202754043570394060225225985050201416015625f,-0.4206999989256259286918293582857586443424224853515625f,1.0450999890804286707179926452226936817169189453125f,-0.2131999883830548725427433964796364307403564453125f,0.84329999880939698897464040783233940601348876953125f,-0.9222999988168478768812974522006697952747344970703125f,0.18459998586177850388168053541448898613452911376953125f,-0.8155000060498711622614109728601761162281036376953125f,-0.32579998577088120637057500061928294599056243896484375f,0.292799981397390662873903011131915263831615447998046875f,-0.275799998138844648654099955820129252970218658447265625f,1.157700046706199970003581256605684757232666015625f,0.30979998756647120217166957445442676544189453125f,0.8761000173479320807246040203608572483062744140625f,0.9099999982118607277925548260100185871124267578125f,-0.180199984270334567781901569105684757232666015625f,0.655399995176493721515953438938595354557037353515625f,-0.4758000173240901631288579665124416351318359375f,-0.6435999873220918932048562055570073425769805908203125f,-0.551799999547004649258497011032886803150177001953125f,0.5377000069200992271589711890555918216705322265625f,-0.496599992775916820875181656447239220142364501953125f,0.40810002023875735499558459196123294532299041748046875f,-0.54360001194626084153327383319265209138393402099609375f,-0.293799986046552830121214583414257504045963287353515625f,-0.51500000171363369272370391627191565930843353271484375f,0.08430001176595691703141710604541003704071044921875f,0.13669997842907921636879109428264200687408447265625f,-0.08719999878406525795071502216160297393798828125f,-0.1688999971807003674939551274292171001434326171875f,0.28080001434385781777791635249741375446319580078125f,0.2093999790906906355303362943232059478759765625f,0.7754999698400499852368739084340631961822509765625f,0.55849998974800119100336814881302416324615478515625f,0.0331999962061644282584893517196178436279296875f,0.2452999923124909997795839444734156131744384765625f,0.3187999998673796131498647810076363384723663330078125f,0.470600009733438628956037064199335873126983642578125f,0.33039999029636391014719265513122081756591796875f,0.050899996048212159394097398035228252410888671875f,-0.33020000930428494712032261304557323455810546875f,-0.773300017696618713358702734694816172122955322265625f,-0.0998999974340197827160636734333820641040802001953125f,0.04680000103712078640683103003539144992828369140625f,0.051700002866983396643263404257595539093017578125f,-0.233000006869435338785478961654007434844970703125f,-0.125800006669759767419236595742404460906982421875f,0.041399994051456445731673738919198513031005859375f,0.4291000134885312178312233299948275089263916015625f,-0.169999996125698216165034182267845608294010162353515625f,-0.76669998397976169002276947139762341976165771484375f,-0.228199993300437942167491200962103903293609619140625f,-0.17979999889135367396875153644941747188568115234375f,0.015600005263090022111782673164270818233489990234375f,-0.2772999938994644697487501616706140339374542236328125f,-0.0048000017225742208637484509381465613842010498046875f,0.08719999513328080542606812741723842918872833251953125f,-0.169800004330277365394152866429067216813564300537109375f,-0.27619999681711193684208183185546658933162689208984375f,-0.586699994261562718378399949870072305202484130859375f,-0.44430000193119045803769040503539144992828369140625f,0.00060000600814813509487066767178475856781005859375f,0.46510001769065834054117658524774014949798583984375f,-0.267500009536743466043162698042578995227813720703125f,0.210600011745095461179744233959354460239410400390625f,-0.1442000020846723185030668901163153350353240966796875f,-0.14290000992864382300950865101185627281665802001953125f,-0.081400008207559604311853718172642402350902557373046875f,0.35279999320656063588330653146840631961822509765625f,0.030899996122717876545493709272705018520355224609375f,-0.2154000066876410601768299102332093752920627593994140625f,-0.325399997858702771935668351943604648113250732421875f,-0.5495000289678575455809550476260483264923095703125f,-0.43899999028444280924077247618697583675384521484375f,-0.4071000040411949516538925308850593864917755126953125f,-0.1176999939560889885115102515555918216705322265625f,-0.62799998131394385580961170489899814128875732421875f,-0.2150999750360844675611815546290017664432525634765625f,-0.20730000883787857990370184779749251902103424072265625f,-0.429799981009960363476096745216636918485164642333984375f,-0.19929999235719453309911841643042862415313720703125f,-0.4354000147342684812201696331612765789031982421875f,0.548599987322091919850208796560764312744140625f,-0.454400036199391177405004782485775649547576904296875f,-0.402100022220611919010480050928890705108642578125f,-0.13999999597668644213399602449499070644378662109375f,-0.18349999827891583237260419991798698902130126953125f,0.36950001913309105106009155861102044582366943359375f,0.352199988651275663897877166164107620716094970703125f,0.28270000028908259803728242331999354064464569091796875f,-0.4044000294566156838982351473532617092132568359375f,-0.74979998264908775951198549591936171054840087890625f,-0.3101999936208128527681537889293394982814788818359375f,-0.4736999960348009519606193862273357808589935302734375f,0.2641999850600960098034875045414082705974578857421875f,-0.2644999983087179540319766601896844804286956787109375f,0.79539999715089759746433628606610000133514404296875f,0.076199988025427156657087834901176393032073974609375f,0.65999999251216656670493421188439242541790008544921875f,0.413600006172060830067493952810764312744140625f,0.1827999930202961476055634193471632897853851318359375f,0.3211999970406294035996097591123543679714202880859375f,0.3365000059679150723468410433270037174224853515625f,0.32439999302327626740094501656130887567996978759765625f,0.37749999858438965105733586824499070644378662109375f,1.127900037682056932908380986191332340240478515625f,0.2015999997794626352742852759547531604766845703125f,0.84730001159459344961533133755438029766082763671875f,-0.36480000123828626445998679628246463835239410400390625f,0.5446000009119504881027751252986490726470947265625f,-0.30500002034008544260501594180823303759098052978515625f,0.6254000096306204792284688664949499070644378662109375f,-0.2689999933391808983884629924432374536991119384765625f,0.47040001406371600989331227538059465587139129638671875f,0.68659999547675243380950860228040255606174468994140625f,0.15029999354183676274487879709340631961822509765625f,-0.39879999498724938433724673814140260219573974609375f,-0.575399987167120041675616448628716170787811279296875f,-0.5035999963372945753548037828295491635799407958984375f,-0.2444000109791755193100470933131873607635498046875f,-0.328800012123584561862799091613851487636566162109375f,0.2458000032424927372431966432486660778522491455078125f,0.3702000288248061821150258765555918216705322265625f,-0.01519996638894127727326122112572193145751953125f,0.478500026330351602155133150517940521240234375f,-0.201999998643994260749678915090044029057025909423828125f,-0.45429999852254976389076546183787286281585693359375f,-0.53540001283437022028266483175684697926044464111328125f,-0.28919998379349731720822092029266059398651123046875f,-0.911799961251020807395661904592998325824737548828125f,-0.6107999839454889556833450114936567842960357666015625f,0.511399994051456641130926072946749627590179443359375f,0.76379996697306662412074729218147695064544677734375f,0.410699989500642015372022797237150371074676513671875f,0.27379999126195908587533267564140260219573974609375f,0.85659998398423198295859037898480892181396484375f,0.499899998179078153270893380977213382720947265625f,0.436399997106194259544054148136638104915618896484375f,0.4105000010207293659192373525002039968967437744140625f,0.43740001050978916108391558736911974847316741943359375f,-0.33240000808835035162047688572783954441547393798828125f,-0.34780000084340567667595678358338773250579833984375f,-0.121199997636675771417458236101083457469940185546875f,0.43219999938011166573659238565596751868724822998046875f,0.36149999140203004088789384695701301097869873046875f,0.125499993383884511644055237411521375179290771484375f,-0.553200007456541076322764638462103903293609619140625f,-0.51299999979138366512643187888897955417633056640625f,-0.455200000438094054544535538298077881336212158203125f,-0.27419998736977591047292435177951119840145111083984375f,-0.568199973370135058559071694617159664630889892578125f,-0.508299986106157231091629000729881227016448974609375f,-0.0430000042617321442950384380310424603521823883056640625f,0.0517999985598027734567949664779007434844970703125f,0.00489999903589488638999682734720408916473388671875f,0.08260000040531156884071606327779591083526611328125f,0.3550999986171721189975869492627680301666259765625f,0.51310001628994950095830063219182193279266357421875f,-0.5229999984502795573604316814453341066837310791015625f,0.185600042925775365887375301099382340908050537109375f,0.67020000259876244541601408855058252811431884765625f,0.210799993109703098781437802244909107685089111328125f,0.66059999833405014246778819142491556704044342041015625f,-0.449200014936923963659864966757595539093017578125f,-0.58439997998476034268833245732821524143218994140625f,-0.28559998194277280703090582392178475856781005859375f,-1.4630000446438788941350139793939888477325439453125f,-0.04419999780058869731647064327262341976165771484375f,-1.2212000393971804701465089237899519503116607666015625f,0.54830002434626246010651584583683870732784271240234375f,0.5129999931976196858585126392426900565624237060546875f,0.52070000486746426116013708451646380126476287841796875f,0.1418000046133993663488581660203635692596435546875f,0.6147999664068226621793655795045197010040283203125f,0.2617999854654076141713403558242134749889373779296875f,-0.440800015163421932129494962282478809356689453125f,0.02140000492930393960477886139415204524993896484375f,-0.5154000174537303191613091257750056684017181396484375f,-0.01760000547170648133032955229282379150390625f,-0.4801999872505664512800649390555918216705322265625f,-0.153799994428455821804391234763897955417633056640625f,-0.87810000701397694644612101910752244293689727783203125f,-0.019800021541117984735791424100170843303203582763671875f,-0.8129000084385271662767991074360907077789306640625f,0.273800002959370569755037649883888661861419677734375f,-0.24439999712109550245031641679815948009490966796875f,0.30740001688003559809914122524787671864032745361328125f,-0.440099992284178842538722165045328438282012939453125f,0.3859999837875367489914424368180334568023681640625f,-0.45630001119226204497891785649699158966541290283203125f,0.963800008547305964157203561626374721527099609375f,-0.10359999909400929851699402206577360630035400390625f,0.9102000168293715720579939443268813192844390869140625f,0.463999995984136770488248657784424722194671630859375f,0.40339997615516220985654172181966714560985565185546875f,0.476999983370304192487054706361959688365459442138671875f,0.14720000617876649595672233772347681224346160888671875f,-0.0870000002458690691042875187122263014316558837890625f,0.1178000080518425696229911636692122556269168853759765625f,-0.44570003561973603467549764900468289852142333984375f,-0.635599986785650106213552135159261524677276611328125f,-0.40990000842362650512740174235659651458263397216796875f,0.51270002211928389357353808009065687656402587890625f,0.37749999694526170657127295271493494510650634765625f,0.02519998620748520323786578956060111522674560546875f,0.60519998620748527429213936557061970233917236328125f,0.450999995112419060916408852790482342243194580078125f,-0.470799985361099349034930128254927694797515869140625f,-0.98809996353983908790041823522187769412994384765625f,-0.420199985089898309098543904838152229785919189453125f,0.295400008624792154154903300877776928246021270751953125f,-0.08469999934285876719286534353159368038177490234375f,0.031299998563528053097115844138897955417633056640625f,0.00659998249411586357382475398480892181396484375f,-0.54370002676844553235468993079848587512969970703125f,0.066099988439679346896582501358352601528167724609375f,0.71620000009536699536738524329848587512969970703125f,1.2594000176846975591615773737430572509765625f,0.736700014564394933813673560507595539093017578125f,-0.2744000085949895861858749412931501865386962890625f,-0.595800018441676737523948759189806878566741943359375f,0.0735999961137772107377941210870631039142608642578125f,0.1384000014126300914796274810214526951313018798828125f,0.763300001901388025515871049719862639904022216796875f,0.2425999995112417817466621272615157067775726318359375f,-0.9766000091671944716154030174948275089263916015625f,-1.48829999638795840155580663122236728668212890625f,-0.956899989372491432959577650763094425201416015625f,0.543599989706277764156538978568278253078460693359375f,0.70099998423457154927973533631302416324615478515625f,0.409999987557530420900775425252504646778106689453125f,0.62079999176859868015299070975743234157562255859375f,0.74329996598959002795936612528748810291290283203125f,0.164199979621172165167308776290155947208404541015625f,0.4549999888241291756685313885100185871124267578125f,0.625099977308511967066806391812860965728759765625f,0.5366999837189911914947515469975769519805908203125f,-0.047199992823600656066673764144070446491241455078125f,-0.733400004839896979547120281495153903961181640625f,-0.3827999983847139020554095623083412647247314453125f,0.4498000123023988550130525254644453525543212890625f,0.33799999144673353868029153090901672840118408203125f,0.061099997976422404377672137343324720859527587890625f,-0.211800003272295089828958225552923977375030517578125f,-0.331499992519617148190036459709517657756805419921875f,0.0937000077694654098792170771048404276371002197265625f,-0.13359999447464954158704131259582936763763427734375f,-0.12049999319761983684884398826397955417633056640625f,0.13280000628232979664744561887346208095550537109375f,-0.718799975168705085337705895653925836086273193359375f,0.03160002742111689411075303723919205367565155029296875f,0.0468999978661539529412038973532617092132568359375f,0.1197999956130981757951303734444081783294677734375f,0.220900000295042975295700671267695724964141845703125f,-0.7099000063374638624935641928459517657756805419921875f,0.43319999024569977308374291169457137584686279296875f,-0.600100020298361958026589491055347025394439697265625f,-0.933299992811679857140916283242404460906982421875f,0.089599996441602769436940434388816356658935546875f,-0.683499983228742724605808689375407993793487548828125f,-0.5939999902844430579307299922220408916473388671875f,-0.23300002363324079368567254277877509593963623046875f,-0.7254000010624521710411727326572872698307037353515625f,-0.5170000155568121869009701185859739780426025390625f,-0.3299999773502353406229303800500929355621337890625f,-0.3041999879658223715495068972813896834850311279296875f,0.50230000231862081960798604995943605899810791015625f,-0.32939998654127133903557478333823382854461669921875f,0.2923000029146669564994454049156047403812408447265625f,1.0760999952197067841552780009806156158447265625f,0.187999976545572877739687100984156131744384765625f,1.17510000766962807716708994121290743350982666015625f,-0.5917999777913092174230769160203635692596435546875f,0.089999992251396321307765902020037174224853515625f,-0.633399990460276285375584848225116729736328125f,-0.2704999895095827611157801584340631961822509765625f,0.50739998648166650951907286071218550205230712890625f,-0.3925000147521495730273954904987476766109466552734375f,-0.75049998503923376347302109934389591217041015625f,-0.18279998556971577272634021937847137451171875f,-0.738699983815848781887325458228588104248046875f,-0.3954000192046169814830136601813137531280517578125f,0.63719998090267182533352752216160297393798828125f,-0.223400029426813251376415792037732899188995361328125f,-0.403000021398067787004038109444081783294677734375f,0.501799991053342608893217402510344982147216796875f,-0.426400040431321070144576879101805388927459716796875f,-0.266300003182887934372047311626374721527099609375f,-0.3403999940216539954462859896011650562286376953125f,-0.1436999853804705740145664094598032534122467041015625f,0.1011000208497048191702560870908200740814208984375f,0.10109999767839905615574025432579219341278076171875f,-0.1160000068843365372828202453092671930789947509765625f,0.4942000000357626987579351407475769519805908203125f,0.02459999800622458820953397662378847599029541015625f,0.3879999895095824768986858543939888477325439453125f,0.706199978339672185256858938373625278472900390625f,0.4051999988734724666983311180956661701202392578125f,0.2664000010550020558497408273979090154170989990234375f,-0.17849999362230306587662198580801486968994140625f,-0.02260000174641607628700512577779591083526611328125f,0.11279999585151667673699193983338773250579833984375f,0.644499997824430348458690787083469331264495849609375f,0.4440000057071447070455860739457421004772186279296875f,0.10820002056956301572654410847462713718414306640625f,-0.23139998689889917926620910293422639369964599609375f,-0.454699998039007180494763815659098327159881591796875f,0.1469999809861182171744076185859739780426025390625f,1.158699990260600998226436786353588104248046875f,0.68689998967051479894507792778313159942626953125f,0.496999991714954347798993694595992565155029296875f,0.987699985611438791011096327565610408782958984375f,0.4446999876081942915106992586515843868255615234375f,0.106200011718273312766314120381139218807220458984375f,-0.53350000077486026128781304578296840190887451171875f,-0.185499999642372070951523710391484200954437255859375f,-0.365400012350082459278155511128716170787811279296875f,-0.4120000076591967541617123060859739780426025390625f,0.03510000822842140877355632255785167217254638671875f,-0.0708000002622604096558234232361428439617156982421875f,-0.3196000028491019495646696668700315058231353759765625f,0.00740000391602524398848572673159651458263397216796875f,0.137200000721216408550162668689154088497161865234375f,-0.2368000184714791345186313265003263950347900390625f,0.17079999810159218753824461600743234157562255859375f,0.13179997943043719743627661955542862415313720703125f,0.88159997012615232137022758251987397670745849609375f,0.484299981069564733360266473027877509593963623046875f,-0.21719998179674160354579726117663085460662841796875f,-0.914199966061115532056646770797669887542724609375f,-0.6165999911367894270597389549948275089263916015625f,0.08779999994933607165847888609278015792369842529296875f,0.11969999770373107583765204253722913563251495361328125f,0.100899998618662334326501195391756482422351837158203125f,0.345000014305114621748771241982467472553253173828125f,0.28769999309927174468981547761359252035617828369140625f,0.90980001423954970363183747394941747188568115234375f,-0.355999992132186804383309208787977695465087890625f,0.6862000176787377281328872413723729550838470458984375f,-0.29580000801086470119116711430251598358154296875f,-0.14120000724792447499567060731351375579833984375f,-0.3750000058114526524377652094699442386627197265625f,-0.711099983596801710206136704073287546634674072265625f,0.4861999753594401507683642194024287164211273193359375f,-0.5859000024557110553047323264763690531253814697265625f,0.04509999146461485253212231327779591083526611328125f,-0.588999985516071244973090870189480483531951904296875f,-0.05569999240636824833217133345897309482097625732421875f,0.1927999895930287976852923748083412647247314453125f,0.81299998012185081819325205287896096706390380859375f,0.1657999593958254802572582775610499083995819091796875f,0.361500030815602002576270024292171001434326171875f,-1.0015999666988850691495827049948275089263916015625f,0.132100050272047742083714183536358177661895751953125f,-0.134299980175494937384428340010344982147216796875f,-0.1111999963700771587582494248636066913604736328125f,-0.294499989815056206765575552708469331264495849609375f,-0.69110001876354232308585778810083866119384765625f,0.55799999055266358283233785186894237995147705078125f,-0.5053000293418767885356146507547236979007720947265625f,-0.0154000129461291379584508831612765789031982421875f,-0.474199995118379291625387850217521190643310546875f,0.088400015531480580222023490932770073413848876953125f,-0.3632000129699708423913762089796364307403564453125f,-0.02040000013113019150523541611619293689727783203125f,-0.24280000441968441204920736709027551114559173583984375f,-0.87800002676248567468064720742404460906982421875f,0.06160000767707796143213272443972527980804443359375f,-0.6940000170320275163504675219883210957050323486328125f,0.3010000198483471223198648658581078052520751953125f,-0.68299996760487591274113583494909107685089111328125f,0.1184000358715653344887641651439480483531951904296875f,0.0178000071763992906426210538484156131744384765625f,0.73199997323751464506358388462103903293609619140625f,0.1525999836415052879345921610365621745586395263671875f,-0.14190000058561558216041476043756119906902313232421875f,-0.031599997097253862410326519238878972828388214111328125f,-0.14010000033080582948485925953718833625316619873046875f,0.234900010621547750133686349727213382720947265625f,0.025000001862645060413115061237476766109466552734375f,0.1163999930083752243348271804279647767543792724609375f,0.26779998862445364427031790910405106842517852783203125f,0.516799999472498772234985153772868216037750244140625f,-0.171599989423155907530826880247332155704498291015625f,-0.2655999804154041132875363473431207239627838134765625f,-0.33839998416453587193331031812704168260097503662109375f,-0.60620000739693669089547256589867174625396728515625f,-0.680099977010488743189853266812860965728759765625f,-0.1474999859184027428682384197600185871124267578125f,-0.1602000017046929514208386535756289958953857421875f,0.14880000452399233523692601011134684085845947265625f,0.320100022831559360980691053555347025394439697265625f,0.36699999931454652113416159409098327159881591796875f,0.00989999706149102820518237422220408916473388671875f,0.405500002101063738990660567651502788066864013671875f,-0.25960000463724153263456173590384423732757568359375f,0.1902000106453893835123380995355546474456787109375f,-0.39700000229477883095796642010100185871124267578125f,-0.422000011235475536164329923849436454474925994873046875f,-0.39190000825971349485143946367315948009490966796875f,-0.206800000813603279681274216272868216037750244140625f,-0.02679999962151047299130368628539144992828369140625f,0.1604999990090727468583509107702411711215972900390625f,0.18180000532120466072427689141477458178997039794921875f,0.10700002375245087904431784409098327159881591796875f,-0.27339996699094815113539880258031189441680908203125f,0.274300026816129527418297584517858922481536865234375f,-0.2773999943792819777854674612171947956085205078125f,-0.0887999915599824163336961646564304828643798828125f,-0.26349999183416361603349287179298698902130126953125f,0.293999997735023388401032207184471189975738525390625f,0.7767999990999696624527359745115973055362701416015625f,0.65220001323819165595097047116723842918872833251953125f,0.105599977695941760202913428656756877899169921875f,0.99469998552203175989916417165659368038177490234375f,0.633599996933341014226925835828296840190887451171875f,0.623000020056962977577086348901502788066864013671875f,0.95660001661777460668645289842970669269561767578125f,0.418700005608796832490270389826036989688873291015625f,0.15820000477433193708520775544457137584686279296875f,0.95820001326799353336127751390449702739715576171875f,0.543100016289949305559048298164270818233489990234375f,-0.319399997568130533664998438325710594654083251953125f,0.4178000147387386231656591917271725833415985107421875f,-0.64340000968277433202047177474014461040496826171875f,-0.5627999769270422003586418213672004640102386474609375f,-0.56259998669624333711425379078718833625316619873046875f,0.45799998619407367161926458720699883997440338134765625f,0.251199978339672502780643981168395839631557464599609375f,0.4219999784156680533442340674810111522674560546875f,0.311900011575222180226774071343243122100830078125f,-0.525599998408555535434061312116682529449462890625f,0.4161000357136135363589346525259315967559814453125f,-0.00540003619194084905075214919634163379669189453125f,-0.695399994319676917342576416558586061000823974609375f,0.00119999491721400897148441799799911677837371826171875f,-1.30170000569820398794718130375258624553680419921875f,0.036399994498491405892082184436731040477752685546875f,-1.094700003366172236241027349024079740047454833984375f,1.394200030136108381384474341757595539093017578125f,0.055199987995624866243815631605684757232666015625f,1.12980001654922990184104492072947323322296142578125f,0.5075000020489095486908581733587197959423065185546875f,-0.2577999942868947069740670485771261155605316162109375f,0.5123000181511045969529050125856883823871612548828125f,-0.00490001423507935296441928585409186780452728271484375f,-0.320599989020824482910398955937125720083713531494140625f,-0.018900001390278475810191594064235687255859375f,-1.0456000402808189875258904066868126392364501953125f,-0.10499999955296512865743352449499070644378662109375f,-0.8194000301644208672513514102320186793804168701171875f,0.7925999875903126934417741722427308559417724609375f,-0.05779999987483019907585912733338773250579833984375f,0.6839999902471898263200955625507049262523651123046875f,-0.41960002147555375273668687441386282444000244140625f,-0.6195999924182888918267053668387234210968017578125f,-0.46880000295937052090522456637700088322162628173828125f,0.2765000174045564307334643672220408916473388671875f,-0.94699998098611803953872367856092751026153564453125f,0.0893000408232216269510672645992599427700042724609375f,-0.3660000106096266137001293827779591083526611328125f,0.1222000010192392238650427316315472126007080078125f,-0.494200029800832663084975138190202414989471435546875f,0.273199981677532122148477355949580669403076171875f,-0.7409999845325945244667309452779591083526611328125f,0.0833999978736041924776145606301724910736083984375f,-0.020100003236532160144633962772786617279052734375f,0.47490000451207148302046334720216691493988037109375f,-0.02560001003146172848801143118180334568023681640625f,0.27589998725652709055111699854023754596710205078125f,-0.317800015074014563509763320325873792171478271484375f,0.0838000039279460651187037001363933086395263671875f,0.42270000960230813547013895004056394100189208984375f,-0.114900008013844701082462051999755203723907470703125f,0.23359999924302099483242045607767067849636077880859375f,0.2100999951899051865211731637828052043914794921875f,0.145999998092651406267350466805510222911834716796875f,0.59440000412464133461298843030817806720733642578125f,0.6632000028371809463578756549395620822906494140625f,0.48279999883174884445224961382336914539337158203125f,0.14279999495744710724665083034778945147991180419921875f,0.4572999868586660188185533115756697952747344970703125f,-0.1526000147849320975890208273995085619390010833740234375f,0.0396000013589858923523934208787977695465087890625f,0.465000003278255302774368828977458178997039794921875f,0.4162000108242034368544182143523357808589935302734375f,0.0866000009715557350631343069835565984249114990234375f,-0.49260000442862494907103609875775873661041259765625f,-0.186300005418062131212764143128879368305206298828125f,-0.265000019073486470233547152020037174224853515625f,0.027399988716840795177631662227213382720947265625f,0.278500008001923760048157419078052043914794921875f,0.178399993664026312867321166777401231229305267333984375f,0.45489998411610732542698087854660116136074066162109375f,0.137999992936849669167287402160582132637500762939453125f,0.46079998789429676975260008475743234157562255859375f,1.049399966126680627809264478855766355991363525390625f,0.4441999871462585236514541975338943302631378173828125f,0.1181999931514263746379356234683655202388763427734375f,0.45199998284876363729978265837416984140872955322265625f,-0.218900015062093611373228441152605228126049041748046875f,-0.3207999993681909245424321852624416351318359375f,-0.25259997954368618167109161731787025928497314453125f,0.152000016748905064645214224583469331264495849609375f,-0.0968000049114225991075954880216158926486968994140625f,-0.13110001108944391301491805279511027038097381591796875f,-0.296800018098950457723361751050106249749660491943359375f,0.25080000964999182855308390571735799312591552734375f,0.256400013273954119341624391381628811359405517578125f,-0.1271999903649090324364578918903134763240814208984375f,-0.3738000073924661048607731572701595723628997802734375f,-0.24699999652057902377322307074791751801967620849609375f,0.972900029635429319796458003111183643341064453125f,0.2829999889135361712533267564140260219573974609375f,0.74930001072287577557062832056544721126556396484375f,0.667900025761127835721708834171295166015625f,0.1460000006258486138221996952779591083526611328125f,0.4725000067427753691617908771149814128875732421875f,-0.3420999994218349993246874873875640332698822021484375f,0.10459999435544020673916065788944251835346221923828125f,-0.33170000495314588828676960474695079028606414794921875f,-0.85759999444484691366596962325274944305419921875f,0.0855999946832657343520622816868126392364501953125f,-0.848800006908178250597529768128879368305206298828125f,0.6743000209674239187052080524154007434844970703125f,0.27299999479949466074657493663835339248180389404296875f,0.46510000163465736733314770390279591083526611328125f,-0.0470000203251839110407672706060111522674560546875f,-0.6103999822497367944151847041212022304534912109375f,-0.0977999999493361915625655456096865236759185791015625f,-0.59319999583363536288516115746460855007171630859375f,0.51699998471140862221773204510100185871124267578125f,-0.42900000820308914484257911681197583675384521484375f,0.06879999334812136879691024660132825374603271484375f,-0.764599991673230672262207008316181600093841552734375f,0.08980002694278998998100860262638889253139495849609375f,0.13710000208541739397105629905126988887786865234375f,-0.042599998058378718557293041158118285238742828369140625f,0.0306999998383223893227977896458469331264495849609375f,0.0241999976515767656337629887275397777557373046875f,-0.183999992072582330138175166212022304534912109375f,-0.1698000030636788171278794834506697952747344970703125f,0.817800019320845450465640169568359851837158203125f,0.3217999909043312012357773710391484200954437255859375f,0.5839999990388751172076808870770037174224853515625f,-0.04580000860691058051088475622236728668212890625f,-0.6035999889612195801191774080507457256317138671875f,0.0390000166594983710410815547220408916473388671875f,0.45750002436339887168514906079508364200592041015625f,0.2710000047981735438185069142491556704044342041015625f,0.39510001225173463357265291051589883863925933837890625f,0.1375999959349627488336409442126750946044921875f,0.46479996879100848872212736750952899456024169921875f,0.0439999727755786551597338984720408916473388671875f,0.7855000152699649351717425815877504646778106689453125f,0.6333000013053420307329588467837311327457427978515625f,0.0509999790191653090687395888380706310272216796875f,0.746299954605103010862876544706523418426513671875f,0.4592999846458436508100930950604379177093505859375f,-0.34880001271963134712450482766143977642059326171875f,-0.05019999768137939355483467807061970233917236328125f,0.160000008046627062441302769002504646778106689453125f,-0.08860001310110110495088520110584795475006103515625f,0.525299998384713973109683138318359851837158203125f,0.45230001096129424098535309894941747188568115234375f,0.1130000054910780260541969255427829921245574951171875f,-0.12390000245198595774098748734104447066783905029296875f,-0.148700009725242943137146767185186035931110382080078125f,-0.6018000201106072921675149700604379177093505859375f,-0.673600009673833710621693171560764312744140625f,-0.122299989876150760892414837144315242767333984375f,-0.1395999991238117132752449833787977695465087890625f,-0.10829999817609792245320932124741375446319580078125f,0.166800011765956934794985500047914683818817138671875f,0.075800003652274627885532254367717541754245758056640625f,-0.01339999826475979372020219670957885682582855224609375f,0.006500000510364756234338301510433666408061981201171875f,0.275200006473064473500045323817175813019275665283203125f,0.087299999152123941570380338816903531551361083984375f,0.0621999977409839477360264936578460037708282470703125f,0.095200000959634767383477083058096468448638916015625f,-0.163799998861551276974068969138897955417633056640625f,0.1758000025719403947022101419861428439617156982421875f,0.5013999930083752332166113774292171001434326171875f,0.4885999789774420065668891766108572483062744140625f,0.401099986279010778389420011080801486968994140625f,0.0847999933779239878361977389431558549404144287109375f,0.15849998624622829712649263456114567816257476806640625f,0.5046000097036362763702754818950779736042022705078125f,-0.0295999959945678625672371708787977695465087890625f,-0.411899993246793716394904549815692007541656494140625f,-0.2296000056058169480621700131450779736042022705078125f,0.42320001639723792141722924498026259243488311767578125f,0.423500004127621565430672490037977695465087890625f,0.1589999993741511019607060006819665431976318359375f,-0.0749999945983288451856196843436919152736663818359375f,-0.39930000444576119189576957069220952689647674560546875f,-0.5159000203557313479763024588464759290218353271484375f,-0.38099998766183862386469627381302416324615478515625f,-0.15310001018047358911644550971686840057373046875f,0.501100004197656989646247893688268959522247314453125f,-0.60899999020993711429383665745262987911701202392578125f,0.2912000028893351188941096552298404276371002197265625f,-1.1529999832510942070484816213138401508331298828125f,1.666799968180060265154907028772868216037750244140625f,-0.67640000329017624380867346189916133880615234375f,0.65600005471706435145051727886311709880828857421875f,-0.2926000318467616434503497657715342938899993896484375f,0.3521000234127047434640189749188721179962158203125f,0.349000028431415643126456416212022304534912109375f,-0.71580000190138814986084980773739516735076904296875f,0.2339000095486643004960569669492542743682861328125f,0.3674000216484074599065934307873249053955078125f,-0.950400015702843603548899409361183643341064453125f,0.2530000136494638951489832834340631961822509765625f,0.91519998561143756088398504289216361939907073974609375f,-1.17279999316930716446449878276325762271881103515625f,0.5200000064820049505698307257262058556079864501953125f,0.4624999782443044438196011469699442386627197265625f,-0.734599991151690279167496555601246654987335205078125f,0.2582000070095062937980401329696178436279296875f,0.2972999863743783777181306504644453525543212890625f,-0.013199980187416304744374428992159664630889892578125f,0.1348999998927116195091002737171947956085205078125f,-0.2214000354766856304422617540694773197174072265625f,-0.330599999600648342124031842104159295558929443359375f,-0.0927000100493433620840733055956661701202392578125f,0.028799954605101873994499328546226024627685546875f,0.08140001997947710066227955394424498081207275390625f,-0.01370000903606438669157796539366245269775390625f,0.746700014340877515905958716757595539093017578125f,-1.00300000761449314978790425811894237995147705078125f,0.431000017166137894264466012828052043914794921875f,-0.60130001480579409189886064268648624420166015625f,0.3017999912768605241808472783304750919342041015625f,-0.3222000038504599928046445711515843868255615234375f,0.810600034916401313012102036736905574798583984375f,-1.40140001878738384988309917389415204524993896484375f,0.4991000246644023974340598215349018573760986328125f,0.07590002361536107855499722063541412353515625f,0.4912000071734183848093380220234394073486328125f,0.0192999965667723927253973670303821563720703125f,0.0245999915987251682736314251087605953216552734375f,0.161100013101101779966484173201024532318115234375f,0.20139999144375320128119710716418921947479248046875f,-0.024100000748038308273635266232304275035858154296875f,-0.380400005793571693857302307151257991790771484375f,-0.4100000075995922088623046875f,-0.13510000167191027475865894302842207252979278564453125f,-0.10390000817030664581608334628981538116931915283203125f,-0.649200030508637837556307204067707061767578125f,-0.341100010195374558907843720589880831539630889892578125f,-0.59270003500878853941458146437071263790130615234375f,-0.37460002900064015562975328066386282444000244140625f,-0.154900010100007090585449986974708735942840576171875f,0.075999961733817400499901850707828998565673828125f,0.27660001773536191382873994371038861572742462158203125f,0.1773000011265277731098422009381465613842010498046875f,0.187400011217594197887592599727213382720947265625f,0.1679999923408030948479563448927365243434906005859375f,-0.1179000018820166584276876164949499070644378662109375f,-0.3125000092387200112398204510100185871124267578125f,0.59039998597502663102432052255608141422271728515625f,0.160499994017183678085558540260535664856433868408203125f,0.253100003772973369819965228089131414890289306640625f,0.3902000204056503829264102023444138467311859130859375f,0.20740000577867034881052177297533489763736724853515625f,0.80970002750605385433146921059233136475086212158203125f,-0.45100000680983065581841628954862244427204132080078125f,-0.1682000016264617314742935150206903927028179168701171875f,-0.407700016140192744718007133997161872684955596923828125f,0.42340000938475130798366308226832188665866851806640625f,0.161900001889467237692343815069762058556079864501953125f,0.271400013571977627435671820421703159809112548828125f,-0.697400025895237973827534005977213382720947265625f,-0.0919000056892633754301868975744582712650299072265625f,0.0563999746799463963498055818490684032440185546875f,-0.7096000122368335549793982863775454461574554443359375f,-0.30470000105649230892623791078221984207630157470703125f,-0.8445000154823063542863792463322170078754425048828125f,0.544999985620379323592032960732467472553253173828125f,0.0156999973982572260666756847058422863483428955078125f,-0.3012000084400181521004924434237182140350341796875f,-1.0366000390440230916055952548049390316009521484375f,-0.4001000148594380245725687927915714681148529052734375f,-1.4018000758409510098090322571806609630584716796875f,-0.17620002848207949597281185560859739780426025390625f,-0.40969999699592563047190196812152862548828125f,0.64359998955726638314445153810083866119384765625f,-0.35699998061358950707244730438105762004852294921875f,0.33110000192522992534804870956577360630035400390625f,-0.5995000140294433066401325049810111522674560546875f,0.3811999996483326214047337998636066913604736328125f,-0.3207000086858868836969804760883562266826629638671875f,-0.58630004371404720320271053424221463501453399658203125f,0.27680001295805001237937403857358731329441070556640625f,-0.3205000143200160689360700416727922856807708740234375f,-0.656400033390521997489486238919198513031005859375f,0.13060002932846526135790554690174758434295654296875f,-0.33470001606941224281399627216160297393798828125f,0.282400036251544417353898097644560039043426513671875f,0.04959997148215811169080780018703080713748931884765625f,0.14440001217126852139926995732821524143218994140625f,0.57649998208880415262456153868697583675384521484375f,-0.7141999860286711854229224627488292753696441650390625f,0.314000007346272536068454428459517657756805419921875f,0.804099978917836466507651493884623050689697265625f,-0.74419997485280031668253286625258624553680419921875f,0.44259999459385834796876224572770297527313232421875f,0.02630001745075016827968283905647695064544677734375f,-0.3804000109344720659265703943674452602863311767578125f,0.0475000064447523850930110711487941443920135498046875f,-0.440500004615634654925315771833993494510650634765625f,0.48139999665915966797768987817107699811458587646484375f,-0.25160000352337952289616396228666417300701141357421875f,0.491600013190507834082154658972285687923431396484375f,-0.38000000260770316895531095724436454474925994873046875f,0.2700000070035457611083984375f,-1.2476000251412386887750471942126750946044921875f,1.117000025019049047614316805265843868255615234375f,-0.675100030207634205225986079312860965728759765625f,-0.52990001889169224824627235648222267627716064453125f,0.6881999967277050433978047294658608734607696533203125f,-0.315200008037686441042524165823124349117279052734375f,1.1839999789595605506065112422220408916473388671875f,-0.983399962893128698482314575812779366970062255859375f,0.631900004720687746839757892303168773651123046875f,-0.67340001288652473476759041659533977508544921875f,1.2790000095814466618548976839520037174224853515625f,-0.42510001173019418274634517729282379150390625f,0.374699951398373087840809603221714496612548828125f,0.2019000085949897993486956693232059478759765625f,0.24800001461803933278105205317842774093151092529296875f,0.09940000442266468116958577638797578401863574981689453125f,0.23700001499801903737107977576670236885547637939453125f,0.554400036460161782514433070900849997997283935546875f,0.3159000127002598024006374544114805757999420166015625f,0.3621000284790998335893164039589464664459228515625f,-0.4415999999284745403116403394960798323154449462890625f,-0.1167000020846724606116140421363525092601776123046875f,-0.5381000176310539728774529066868126392364501953125f,0.6613999947220083885923713751253671944141387939453125f,0.0552999964475631156091139928321354091167449951171875f,0.0590000054836270493296979111619293689727783203125f,-0.078400000220537169326462390017695724964141845703125f,-0.119599997037649175002371748632867820560932159423828125f,-0.609599987500905715620547198341228067874908447265625f,0.570800004732608723401199313229881227016448974609375f,0.160300001104176026434089408212457783520221710205078125f,0.137900012499093893580948133603669703006744384765625f,0.3333999937385316858495798442163504660129547119140625f,0.15799999409168952180237965876585803925991058349609375f,0.09869998757839137937253326526843011379241943359375f,-0.904000005185604305069091424229554831981658935546875f,-0.2441000044360757126327143851085565984249114990234375f,-1.0243000344157220382612649700604379177093505859375f,0.215600009435414907699168907129205763339996337890625f,0.175899996681511350260507242637686431407928466796875f,0.153899998819827743545829434879124164581298828125f,0.64560002478361167010945109723252244293689727783203125f,0.17040000877380379051828640513122081756591796875f,0.4876000316977506798821195843629539012908935546875f,-0.9976000372111801794972052448429167270660400390625f,-0.2204000120520592798101233711349777877330780029296875f,-0.2012000364542014807511804974637925624847412109375f,0.01320001684427296595458756200969219207763671875f,0.0621000084742904245871386592625640332698822021484375f,-0.3850999852061267603176020202226936817169189453125f,0.20100000323355171616412917501293122768402099609375f,0.09149999788403506695289024719386361539363861083984375f,-0.34140000656843216830793608096428215503692626953125f,-0.62919999541640247997520418721251189708709716796875f,-0.38479999538958065130600516567938029766082763671875f,-0.86260000517368240480209351517260074615478515625f,-1.0088000265777115060927826561965048313140869140625f,-0.1964000044822693524793066899292171001434326171875f,0.23000001665204783396490029190317727625370025634765625f,-0.2418000099778174760700721890316344797611236572265625f,0.131500009022653241430589332594536244869232177734375f,0.5931000608444225719040332478471100330352783203125f,-0.061000018209219408760191072360612452030181884765625f,0.309600021028519023502667550928890705108642578125f,-0.35289998254776033803636892116628587245941162109375f,0.7431999789953234181183461259934119880199432373046875f,-0.22149999743700021781478426419198513031005859375f,1.11659998174905705781156939337961375713348388671875f,-1.2497999854058023938563337651430629193782806396484375f,0.6173000093221663320264269714243710041046142578125f,0.34939998743534061986792949028313159942626953125f,-0.032600003981590131019174805260263383388519287109375f,0.163900000831484593533104998641647398471832275390625f,0.8517000055491923404815679532475769519805908203125f,-0.644999992549419420839740269002504646778106689453125f,0.45980000768303863623032157192938029766082763671875f,0.45610001161098434607765739201568067073822021484375f,-0.1131999933749436859642401032033376395702362060546875f,0.236900001442432284193273517303168773651123046875f,-0.7754999769926076425008432124741375446319580078125f,1.461199968206882804366841810406185686588287353515625f,-0.47590000066757198737832368351519107818603515625f,0.1979000338077536724767924170009791851043701171875f,0.194399974769354333403725831885822117328643798828125f,0.0949000082373618880637877737171947956085205078125f,0.730400017714500915388953217188827693462371826171875f,-0.93960000672340393546022596638067625463008880615234375f,0.41900001829862620894573410623706877231597900390625f,-1.54340004000663810757032479159533977508544921875f,1.3606000156939028755687104421667754650115966796875f,-0.840300036382675585855395183898508548736572265625f,0.27230003018379278501015505753457546234130859375f,0.424199986922740635009176912717521190643310546875f,0.1106000080943108088149529066868126392364501953125f,0.564000025153159612045783433131873607635498046875f,0.11399998886883278004233943647705018520355224609375f,0.2718000090837477955574286170303821563720703125f,-0.337800009560584868495425325818359851837158203125f,0.18119998057484654196969131589867174625396728515625f,-0.19719999103546115293283946812152862548828125f,-1.62720001964569149777162238024175167083740234375f,-0.357400005853175883885342045687139034271240234375f,0.890600002059340312143831397406756877899169921875f,0.288999999634921511670881955069489777088165283203125f,0.510700016248225807657945551909506320953369140625f,0.359399996748566596949103768565692007541656494140625f,0.045599998892843718323319990304298698902130126953125f,0.677700013029575387690783827565610408782958984375f,-0.61059999073445769823820228339172899723052978515625f,-0.01159999832659952545554915559478104114532470703125f,0.11109998329877868172843591310083866119384765625f,0.35640000969767571259438909692107699811458587646484375f,0.091200001958012588687552124611102044582366943359375f,0.1574000118136407166957724257372319698333740234375f,0.13239999870061869291504308421281166374683380126953125f,-0.028600000658631330452408292330801486968994140625f,-0.0531999995589256258199384319595992565155029296875f,-0.503399986883998007414220410282723605632781982421875f,-0.14580000067204235136841816711239516735076904296875f,-0.7523000203490255444194190204143524169921875f,0.7964000064194201033984654714004136621952056884765625f,0.324999998807907086728619105997495353221893310546875f,0.90120001261234250478082685731351375579833984375f,0.19339999895393855666725357878021895885467529296875f,-0.04219999829977749694620570153347216546535491943359375f,-0.40110000028610226507908009807579219341278076171875f,0.497200012046098815954309202425065450370311737060546875f,0.145400003334879901473186691873706877231597900390625f,0.42360001772046107504365863860584795475006103515625f,-0.12220001226961640927726193694979883730411529541015625f,-0.0977000057280064115872164620668627321720123291015625f,-0.08060001226663615625511738471686840057373046875f,-0.4664000140190125431871592809329740703105926513671875f,-0.176800003719329856810560386293218471109867095947265625f,-0.132400011962652275343543806229718029499053955078125f,0.5338000173389911839905153101426549255847930908203125f,0.222800003749132169428293082091840915381908416748046875f,0.47170002216398732031166218803264200687408447265625f,0.10840000096559510023297434599953703582286834716796875f,0.074599998676776857475800852625980041921138763427734375f,0.070400001472234574606545720598660409450531005859375f,0.9709999956339594451293351085041649639606475830078125f,0.3465999971345066210659524585935287177562713623046875f,0.9081000229954714342284205486066639423370361328125f,0.4242000135213139788703529120539315044879913330078125f,-0.10159999545812592458560175145976245403289794921875f,1.09990000168085089171654544770717620849609375f,-0.987799979832768570986445411108434200286865234375f,0.607800005388259734218081575818359851837158203125f,0.1370999718546865864254868938587605953216552734375f,-0.932399984022974859243504397454671561717987060546875f,0.123899997329711908378158113919198513031005859375f,-1.166899964487551955016897409223020076751708984375f,1.23899997813999629414638548041693866252899169921875f,-0.638400002455711046422948129475116729736328125f,0.484600009703636036562102162861265242099761962890625f,-0.47560000585913642101587583965738303959369659423828125f,0.26800000977516180711290871840901672840118408203125f,0.287599992731213394137057548505254089832305908203125f,-0.38599999697506415241576860353234224021434783935546875f,0.1602000039026141298137417834368534386157989501953125f,-0.2970999563574796553666601539589464664459228515625f,0.861799952831864946034556851373054087162017822265625f,-0.2031999834656712522473753779195249080657958984375f,0.63560001137256616488002691767178475856781005859375f,-0.8211999886214733646028207658673636615276336669921875f,0.3754000051975250329405753291212022304534912109375f,0.6520999962925912996070110239088535308837890625f,-0.40579999005496514286051024100743234157562255859375f,0.33700000736117363686616954510100185871124267578125f,0.5218000081896783104440373790566809475421905517578125f,-0.68740000019073477233177982270717620849609375f,0.301700009721517670158164037275128066539764404296875f,-0.117800018799305572514413142926059663295745849609375f,-0.10499999620020361579264545071055181324481964111328125f,-0.05670000450611123454791595577262341976165771484375f,-0.66160002540946027949075869400985538959503173828125f,0.3428000047922135085087802508496679365634918212890625f,-0.35100001254677781759028221131302416324615478515625f,0.51870002214908594861952906285296194255352020263671875f,-0.46400001032650461940676223093760199844837188720703125f,0.2917000106900931033493407085188664495944976806640625f,0.110600007051229187737817483139224350452423095703125f,0.04259999593496333059050584779470227658748626708984375f,0.05550000023841850094186156638897955417633056640625f,0.93349998468160588771524999174289405345916748046875f,-1.1171999717384577532897083074203692376613616943359375f,0.5382999954938885167621265281923115253448486328125f,0.16880002702474694586953773978166282176971435546875f,0.712900001919269499239817378111183643341064453125f,0.90640000969767609007021746947430074214935302734375f,0.2012000066518784269220532223698683083057403564453125f,0.8068000293493273744616089970804750919342041015625f,0.56860001183450226136528726783581078052520751953125f,0.1838000018790364453735719507676549255847930908203125f,0.7929000203967095927737318561412394046783447265625f,0.00240000399053086344025587095529772341251373291015625f,0.086999997414648533311520850475062616169452667236328125f,0.427399993485212181809629328199662268161773681640625f,0.24340000893771662759945684229023754596710205078125f,0.1187000040441751613773391227368847467005252838134765625f,0.6146000275105241161810454286751337349414825439453125f,-0.63920004667639762629960387130267918109893798828125f,-0.389200011062622142077316311770118772983551025390625f,-1.013200049924851242622025893069803714752197265625f,-0.1774000281304122239589560194872319698333740234375f,-0.210500004559755371502660636906512081623077392578125f,-0.579000017404556643896285095252096652984619140625f,0.52760003229379714984048632686608470976352691650390625f,0.2986000138461590847072102405945770442485809326171875f,0.369600038015843068706089979968965053558349609375f,-0.09360000170171256517193114632391370832920074462890625f,-0.022500000149011600836246316248434595763683319091796875f,-0.168300008457899163971660527749918401241302490234375f,0.6168000148952008654390510855591855943202972412109375f,0.06270000546723608625399037919123657047748565673828125f,0.254100025409460528180716210044920444488525390625f,0.6268000329256062652660830281092785298824310302734375f,0.2630000117123127978402408189140260219573974609375f,0.283600030684471704489624244160950183868408203125f,1.1376000303566460303272833698429167270660400390625f,0.326400011336803519412796958931721746921539306640625f,1.238800058317185204259658348746597766876220703125f,-0.64700004245340869690750196241424418985843658447265625f,-0.32530001283884057983186721685342490673065185546875f,-0.5234000420928008878718173946253955364227294921875f,-1.107200014579296176719935829169116914272308349609375f,-0.23530000341385604034627476721652783453464508056640625f,-0.6415000306665898932578784297220408916473388671875f,0.066600018629431989580780282267369329929351806640625f,0.18930000662505630515397569979541003704071044921875f,0.5348000260114673665157170034945011138916015625f,-0.203199999707937006832025872427038848400115966796875f,-0.3106000074982642189524995046667754650115966796875f,0.954599986457825622210293659009039402008056640625f,-1.54059996920228048367107476224191486835479736328125f,0.564700008916855011875668424181640148162841796875f,0.367900003111362394747629878111183643341064453125f,-0.86440000002682193525060938554815948009490966796875f,0.2399000076413155824184286757372319698333740234375f,-0.34039998437315244217415965977124869823455810546875f,0.15179998952597395511787681243731640279293060302734375f,-0.16849999881908284127263186746858991682529449462890625f,-0.05740000682175150359398685395717620849609375f,-0.33660000030100345469463718472979962825775146484375f,-0.000900003275275285119505497277714312076568603515625f,-0.245800059866905229455369408242404460906982421875f,0.25900004675984344970629535964690148830413818359375f,-0.15750002086162595560381305404007434844970703125f,0.19039995244741447777414578013122081756591796875f,-0.2147999624580148037011895212344825267791748046875f,0.09219998686313601865549571812152862548828125f,0.547400044298173238388471872895024716854095458984375f,-0.07900000965595299629029568677651695907115936279296875f,0.28750001341104525209857456502504646778106689453125f,-0.03689999484866834933427526266314089298248291015625f,0.19380000217705950493751743124448694288730621337890625f,-0.03059999961927528655536434598616324365139007568359375f,0.8936999977111816928498910783673636615276336669921875f,-1.2168000001430510081945612910203635692596435546875f,0.507900016969442535952339312643744051456451416015625f,0.80220004706382841419554097228683531284332275390625f,-0.4054000147342684545748170421575196087360382080078125f,0.4279000218868258542670446331612765789031982421875f,1.00340000364780479458204354159533977508544921875f,-1.7325999960839748581520325387828052043914794921875f,0.6071000168561937471167766489088535308837890625f,-0.639000058233738510438115554279647767543792724609375f,0.30460003257691858902234116612817160785198211669921875f,-0.34670002387762099971268980880267918109893798828125f,-1.4172999991893766580375313424156047403812408447265625f,1.7561999904096123881203084238222800195217132568359375f,-0.803400019294023604032872754032723605632781982421875f,-0.3475999762654300440090082702226936817169189453125f,0.390599983879923673413259166409261524677276611328125f,-0.1799000016808509627708190237171947956085205078125f,0.1462000144004829138566492474637925624847412109375f,1.2346000460624697581124564749188721179962158203125f,0.12160001721382140971172702847979962825775146484375f,0.16820000410377977662079729270772077143192291259765625f,0.63900003558397333591756250825710594654083251953125f,0.3592000076353547566299084792262874543666839599609375f,0.2332999985113739516151554198586381971836090087890625f,0.5783000130772588676109080552123486995697021484375f,0.821600006186961717702388341422192752361297607421875f,0.45159999724626531669713358496665023267269134521484375f,1.32880003238916355456922246958129107952117919921875f,-0.598600036868452978211507797823287546634674072265625f,-0.395300005313754088120958840590901672840118408203125f,-0.818000035405159309220834984444081783294677734375f,-0.171599996948242239369619710487313568592071533203125f,0.05549999699741603453873040052712894976139068603515625f,0.54749998927116383384827713598497211933135986328125f,-0.7217999936610457467622836702503263950347900390625f,-0.26339999377578493522378266789019107818603515625f,-0.875499992787837300056708045303821563720703125f,0.9194000174611807185698353350744582712650299072265625f,0.26590000379681588693614457952207885682582855224609375f,0.59580002678632748569498289725743234157562255859375f,0.4974000057041643163557864681934006512165069580078125f,0.126899999319016909549162619441631250083446502685546875f,0.36460001275837417011871366412378847599029541015625f,0.22519998941123497981919854282750748097896575927734375f,-0.089899998849630347041994582468760199844837188720703125f,-0.178299996238946878435172038734890520572662353515625f,-0.09160000305771820450218001496978104114532470703125f,-0.1645999992728233163319373488775454461574554443359375f,-0.4924000055551527310626624966971576213836669921875f,-0.4751999844193459665575574035756289958953857421875f,-0.1043999972701072920244769193232059478759765625f,-0.87239999542236290608343551866710186004638671875f,-0.4750000057369465622514326241798698902130126953125f,-0.3307999954566358535856807066011242568492889404296875f,-0.8099000159859652825389275676570832729339599609375f,0.324200006070733071084077892010100185871124267578125f,-0.0043999964877963204656907691969536244869232177734375f,-0.2850999798417088726409929222427308559417724609375f,-0.491200013208389574259626897401176393032073974609375f,-0.3728000038981438279961366788484156131744384765625f,-1.13520001273155202170528355054557323455810546875f,0.212200002285837907578525118879042565822601318359375f,0.01490000078678122008568607270717620849609375f,-0.31459999003410388240808970294892787933349609375f,0.12879997457265890403732555569149553775787353515625f,-0.14980000321269049123884542495943605899810791015625f,0.12530001611709540298988940776325762271881103515625f,-0.0882000104367730575205541754257865250110626220703125f,0.0798000003814696157178332214243710041046142578125f,0.3184000045418731161817049724049866199493408203125f,-0.551999997675418629938803860568441450595855712890625f,0.2135999922394749006571146310307085514068603515625f,-0.3104000603318208817427148460410535335540771484375f,0.047400046160816611262589503894560039043426513671875f,-0.17670001761913312066099024377763271331787109375f,-0.7238999857068062482312598149292171001434326171875f,0.341799988222122319569962201057933270931243896484375f,-0.3672000020623205074343786691315472126007080078125f,-0.54769999932050694724239292554557323455810546875f,0.9051999748080969965258191223256289958953857421875f,-0.33410000171661380363730131648480892181396484375f,1.10060001494884485140346441767178475856781005859375f,-1.1909999993592499389905015050317160785198211669921875f,0.618700017082691289260765188373625278472900390625f,0.57260000100135755030095197071204893290996551513671875f,-0.73700000162422629745861968331155367195606231689453125f,0.3281000077217817167962721214280463755130767822265625f,0.57029996608644761835194003651849925518035888671875f,-0.78779996958822040564740518675534985959529876708984375f,0.31699999869987360856526947827660478651523590087890625f,0.26159998890161517692831694148480892181396484375f,-0.25199999722838395399548971909098327159881591796875f,0.13160000573992736150330529198981821537017822265625f,-0.1787999645888813660121741122566163539886474609375f,1.03439994884133401598091950290836393833160400390625f,-0.153599990823864995803660349338315427303314208984375f,-0.12350001746415983916449476964771747589111328125f,0.0446000138014550628895449335686862468719482421875f,-0.08219999639987918271799571812152862548828125f,0.7862999936461445571467265835963189601898193359375f,-0.590800002571940208184742004959844052791595458984375f,0.4151000084519385069370400742627680301666259765625f,0.0991999717235554356875582016073167324066162109375f,0.063199987936020551870797135052271187305450439453125f,0.02200000154972059362989966757595539093017578125f,-0.0006999822735789251737514859996736049652099609375f,-0.09030002535581616029958240687847137451171875f,0.4579999938309191609420167878852225840091705322265625f,0.2907999972075223649170538919861428439617156982421875f,0.7229999984502786247730909963138401508331298828125f,0.258600009897351323928660349338315427303314208984375f,-0.0575999955996870927066311196540482342243194580078125f,-0.4350999822258945215480707702226936817169189453125f,-0.146399998223781580009017488919198513031005859375f,0.088199998292326942106456044712103903293609619140625f,0.328999988198280135520690237171947956085205078125f,0.45340002309381954859190955176018178462982177734375f,0.211700003463029862160738048260100185871124267578125f,-0.118199987936019823564492980949580669403076171875f,-0.52820003867447429257708790828473865985870361328125f,-0.28210001246035110167298398664570413529872894287109375f,-0.57320003651380613263199848006479442119598388671875f,-0.366000012546777941935260969330556690692901611328125f,-0.238200003284215977572557676467113196849822998046875f,-0.728000009775161771585771930404007434844970703125f,-0.300000000596046145773243551957421004772186279296875f,-0.193499997608363560885891274665482342243194580078125f,-0.124100004994868839958144235424697399139404296875f,-0.741000018283724859458061473560519516468048095703125f,-0.379200002010166681287728351890109479427337646484375f,-1.029700012791156638058964745141565799713134765625f,0.128000016182661102703832511906512081623077392578125f,0.105100001709163226593091167160309851169586181640625f,-0.235500010073184995462725055404007434844970703125f,-0.240799997133016585593168201739899814128875732421875f,0.03830000156611201855838544361176900565624237060546875f,0.32370001797676106747303492738865315914154052734375f,1.058000001132488154809152547386474907398223876953125f,0.3243999982386827074520851965644396841526031494140625f,0.906600018703937138297987985424697399139404296875f,-0.890199995669722188296191234258003532886505126953125f,-0.31399999572336667608141169694135896861553192138671875f,-0.62910000875741178827382782401400618255138397216796875f,0.4016000184059145272641444535111077129840850830078125f,0.1717000046178699113053056635180837474763393402099609375f,0.6207000175148251752688111082534305751323699951171875f,-0.69980000433027722550605176365934312343597412109375f,-0.1174000000789761077868433858384378254413604736328125f,-0.1821000219225883398621590458787977695465087890625f,0.74759999384880060357971842677216045558452606201171875f,-0.669900020456314138073139474727213382720947265625f,0.17300000500679058035302659845910966396331787109375f,-0.1335999733150010637672266966546885669231414794921875f,0.1089999911785124453444950631819665431976318359375f,0.7998999992966648875380997196771204471588134765625f,-0.597600010165571848830268208985216915607452392578125f,0.418900014019012445487533113919198513031005859375f,-0.5365999802589414002795820124447345733642578125f,0.532799994957446898524722200818359851837158203125f,-0.28580000324249255072572850622236728668212890625f,0.802600050771236084301563096232712268829345703125f,-0.04260002953708141859578972798772156238555908203125f,0.403700020956993199661155813373625278472900390625f,-0.409800051641465046259327209554612636566162109375f,0.25080002611577523197183836600743234157562255859375f,-0.2300000205636028027811335050500929355621337890625f,-0.0280000228881842616601716144941747188568115234375f,0.156399997180700633947481037466786801815032958984375f,-0.0370000036358835160399394226260483264923095703125f,-0.3625000143051142487138349679298698902130126953125f,-0.097999997258186422044445862411521375179290771484375f,-0.18129999871253943410920328460633754730224609375f,-0.4205000362992290519059679354541003704071044921875f,0.533400001040101212623767423792742192745208740234375f,-0.257600011134147877100986079312860965728759765625f,0.3515000501871110571983081172220408916473388671875f,0.279799977582693149003034704946912825107574462890625f,0.16000001847744016458818805404007434844970703125f,-0.6797000053405763964065045001916587352752685546875f,0.531799988669157119858255100552923977375030517578125f,-0.35800001320242902380641680792905390262603759765625f,1.099999992847442431553872665972448885440826416015625f,-1.4017999745130538258308661170303821563720703125f,0.63430000520944584962990120402537286281585693359375f,-1.003299998213350363585050217807292938232421875f,0.939599976995587393702180634136311709880828857421875f,-0.5597000012800095181120241250027902424335479736328125f,0.19790002047121557637865407741628587245941162109375f,-0.4632000087976455571237011099583469331264495849609375f,0.133300010916590938592207749024964869022369384765625f,-1.069699990141391054976338637061417102813720703125f,1.13219999632537327016734707285650074481964111328125f,-0.59140000984668716910164221189916133880615234375f,1.041399979448318635633086159941740334033966064453125f,-0.94730001587867729284653250942938029766082763671875f,0.4107999962389465853362935376935638487339019775390625f,0.158599995666742266120508020321722142398357391357421875f,0.055599993640183864584969342104159295558929443359375f,0.40720000072121609324682367514469660818576812744140625f,0.20869999795332550007742611342109739780426025390625f,0.859700008264928872137033977196551859378814697265625f,0.35279999257326100092768683680333197116851806640625f,0.131099998162686748326422048194217495620250701904296875f,-0.031099990898371121517129722633399069309234619140625f,0.1846000080645085195119037280164775438606739044189453125f,0.0887000024423003352591621251121978275477886199951171875f,0.1936000101208688040088645720970816910266876220703125f,0.5164000433743003437569996094680391252040863037109375f,0.28130001104325064176236992352642118930816650390625f,0.691700043547154308498647878877818584442138671875f,0.1379999860078094664483927545006736181676387786865234375f,-0.101100004309415847369990615334245376288890838623046875f,-0.00150000675022621265952693647705018520355224609375f,0.53700003321468869899035780690610408782958984375f,0.284100011476874414473314800488878972828388214111328125f,1.005200056093932037271088120178319513797760009765625f,-0.427599984163045565566108052735216915607452392578125f,-0.287899994207918563660086874733678996562957763671875f,-0.351899989521502476463865605182945728302001953125f,0.32919999809861177908487661625258624553680419921875f,0.054499999724328507255677322973497211933135986328125f,-0.173500001966953476539856637828052043914794921875f,0.0254000167459249581725089228712022304534912109375f,0.1291000028342008665305229442310519516468048095703125f,-0.0139999923706053408523075631819665431976318359375f,-0.830400026804209101527476377668790519237518310546875f,-0.326400007909536438166497873680782504379749298095703125f,-1.154000037521124344408463002764619886875152587890625f,-0.50340002368986624947666541629587300121784210205078125f,-0.3057000057429075656756367607158608734607696533203125f,-0.63240002254247684732035850174725055694580078125f,0.60440001112818730888420759583823382854461669921875f,0.20710000422745944437252774150692857801914215087890625f,0.96670003609657317866776793380267918109893798828125f,-0.810400030976534235804820127668790519237518310546875f,-0.31340000923573974045410750477458350360393524169921875f,-0.6238000381261115112607740229577757418155670166015625f,-0.404400003975629918073764201835729181766510009765625f,0.0456000012397765885907574556767940521240234375f,0.62520001049637752288390402100048959255218505859375f,-0.2595999934613704152042146233725361526012420654296875f,0.32740000361800181138960397220216691493988037109375f,0.020299998403340513419834678643383085727691650390625f,-0.43699999007582668308913298460538499057292938232421875f,0.04799999627098439081152037033461965620517730712890625f,0.595699991065263390055406489409506320953369140625f,-0.20559998544454582969365219469182193279266357421875f,0.30829999415278397378870067768730223178863525390625f,0.15240000995099556302392329598660580813884735107421875f,-0.1262000040441751680386772704878239892423152923583984375f,0.0865000038444996288689026187057606875896453857421875f,0.30410005357265479375428185448981821537017822265625f,-0.2868000446975227379908801594865508377552032470703125f,0.1809000227212909095442228135652840137481689453125f,0.29249996691942181659129573745303787291049957275390625f,-0.73079998029470427667320109321735799312591552734375f,0.1784999981671571145369625810417346656322479248046875f,0.11500002041459111978838336654007434844970703125f,-0.47520002189576615592159214429557323455810546875f,0.10060000734925278464970688219182193279266357421875f,-0.3859000024557115438028631615452468395233154296875f,-0.104400033777951595226340941735543310642242431640625f,-0.197899986720084797298113699071109294891357421875f,0.689300001931190564619100769050419330596923828125f,-0.40759998833537114393266165279783308506011962890625f,0.36090000603199001716347993351519107818603515625f,-0.001399961268901961375377140939235687255859375f,0.4975999723166228960735679720528423786163330078125f,-0.0221999919295310377265195711515843868255615234375f,-0.5146000208795076247270117164589464664459228515625f,1.015000002831220715648896657512523233890533447265625f,-0.32830001352429416527911598677746951580047607421875f,-0.312600040042400717510417962330393493175506591796875f,0.12560001457631597698849645894370041787624359130859375f,-0.17700001078844085355967763462103903293609619140625f,0.297099988543987336697682621888816356658935546875f,-0.93039999282956120651988385361619293689727783203125f,0.21090000007152553962441743351519107818603515625f,-0.8646000358551744202628697166801430284976959228515625f,0.7640000144243239699193281921907328069210052490234375f,-0.48220001957118530100387943093664944171905517578125f,-0.2120999819874764824589874478988349437713623046875f,-0.28169999943971646416684961877763271331787109375f,-0.551800001260638328659524631802923977375030517578125f,-0.0947000025838613535000831689103506505489349365234375f,-0.7244000297546389077751882723532617092132568359375f,-0.541000005170702902290713609545491635799407958984375f,-0.16670000063180923088879126225947402417659759521484375f,-0.50420001374483103262491567875258624553680419921875f,-0.293999988347291907331282345694489777088165283203125f,0.1110000024139881080742497942992486059665679931640625f,0.5596000192403796091866752249188721179962158203125f,0.795000036284327915581116030807606875896453857421875f,0.298200008685886974735268495351192541420459747314453125f,0.975700028616190540020625121542252600193023681640625f,-0.4764000229597094016043001829530112445354461669921875f,-0.170400006762147004923946269627776928246021270751953125f,-0.130600018972158782304404667229391634464263916015625f,-0.574799985480308084362377485376782715320587158203125f,-0.07499999552965153082340066248434595763683319091796875f,0.24959999331235938058171086595393717288970947265625f,0.28659998703896949479030809015966951847076416015625f,0.09979999423474061881478291979874484241008758544921875f,-0.30850000405311650553130675689317286014556884765625f,0.105799995568394589184890719479881227016448974609375f,0.04689999659955501609687189557007513940334320068359375f,0.61719999550580961766854670713655650615692138671875f,-0.0415999991834163918014155569835565984249114990234375f,0.0427999977886676929994536067169974558055400848388671875f,0.3307999899059532733502919654711149632930755615234375f,0.56800003629922901637883114744909107685089111328125f,0.3401000096812845097105082459165714681148529052734375f,0.3555000237822536490739366854541003704071044921875f,0.64580002194345009769449461600743234157562255859375f,0.23830000801086426154284936274052597582340240478515625f,0.12020003865957296085298366961069405078887939453125f,0.7698000041812658178486117321881465613842010498046875f,0.11280000058263539575165168571402318775653839111328125f,0.3973000178158283546281381859444081783294677734375f,0.306400006270408642450320257921703159809112548828125f,0.016100001217424886323925647957366891205310821533203125f,-0.326500002950429912829122258699499070644378662109375f,0.2655999964714050864955652286880649626255035400390625f,0.00530000088065862851038900771527551114559173583984375f,0.22590001080036170577614029753021895885467529296875f,0.22020001906454578222138707133126445114612579345703125f,0.34360000490546216411757995956577360630035400390625f,-0.4261999679088592785092259873636066913604736328125f,1.170799980071187018637601795489899814128875732421875f,-0.28079999772906294452923248172737658023834228515625f,-0.5330000002384185364689983543939888477325439453125f,0.733399992099404318679489733767695724964141845703125f,-0.311100005054473882637466886080801486968994140625f,-0.975999985426664551368958200328052043914794921875f,0.839999974071979682577193671022541821002960205078125f,-0.515200011017918679812055415823124349117279052734375f,0.5915000443756586179233636357821524143218994140625f,-0.968400019517540844304903657757677137851715087890625f,0.36110002666115814662362026865594089031219482421875f,-0.68360003411173853038462766562588512897491455078125f,0.3910000147819518989678044817992486059665679931640625f,-0.36410001855492613032083681900985538959503173828125f,-1.1741999744057647347972306306473910808563232421875f,0.823399976229667540650325463502667844295501708984375f,-0.6212999984145159970694294315762817859649658203125f,0.689899994826316032003887812606990337371826171875f,-0.05539998917877664297293449635617434978485107421875f,0.3431999953866000652169532258994877338409423828125f,-0.273800010782480285342899151146411895751953125f,-0.339199990871548617832331729005090892314910888671875f,-0.109800003883242514035600834176875650882720947265625f,-0.330399990445375379977122065611183643341064453125f,0.1091999971300363636572683390113525092601776123046875f,-0.1637999997183678946299778544926084578037261962890625f,0.611500068068504987195410649292171001434326171875f,0.01479997206926331188014955841936171054840087890625f,0.315500022888183906388803734444081783294677734375f,0.964200022983551008337599341757595539093017578125f,-0.47100001187622542175859052804298698902130126953125f,0.5087000123143194940666944603435695171356201171875f,1.0426999868035313756564619325217790901660919189453125f,-1.411999986872076728872116291313432157039642333984375f,0.594500011831522012784034814103506505489349365234375f,0.7751000151574611862770325387828052043914794921875f,-0.444400002336502009114838074310682713985443115234375f,0.40230001572966589407087667495943605899810791015625f,0.310899977198243249176812241785228252410888671875f,-0.6733999828606844761935690257814712822437286376953125f,0.1903999984547495305520214969874359667301177978515625f,0.082800028485059851135474673355929553508758544921875f,1.273500004351138414904198725707828998565673828125f,-0.29140001349747191028427550918422639369964599609375f,-0.24170000323951246201659159851260483264923095703125f,-0.43280001194477080161959747783839702606201171875f,0.62119998608827575736768267233856022357940673828125f,0.07539999610781666916636822861619293689727783203125f,0.12599999213218637805766775272786617279052734375f,0.13999999389052397713584241500939242541790008544921875f,-0.06169999925345182045788305913447402417659759521484375f,-0.14069999717473979927717664395458996295928955078125f,0.3640000014603135269908307236619293689727783203125f,0.25489999784380190650523445583530701696872711181640625f,0.97990002403259257590661945869214832782745361328125f,-0.8022000444561248055919122634804807603359222412109375f,-0.3552000138863922185095134409493766725063323974609375f,-0.7371000505328186847009419579990208148956298828125f,0.14660001192390925695008263573981821537017822265625f,0.1579000023663044149913048386224545538425445556640625f,0.2906000077962875849379997816868126392364501953125f,-0.128800018531084248252227553166449069976806640625f,-0.25070000488609078814761232933960855007171630859375f,-0.59050001782178895837205345742404460906982421875f,-0.7346000070959328720476833041175268590450286865234375f,-0.232900002329051447613750269738375209271907806396484375f,-0.30720002527087919563797413502470590174198150634765625f,0.1086000095993278069528287232969887554645538330078125f,0.093299999070167505266226726234890520572662353515625f,-0.14259999802112588440650142729282379150390625f,0.156200002703070495169157538839499466121196746826171875f,0.1215999988481402116491381093510426580905914306640625f,0.14170000197291354737672008923254907131195068359375f,-0.47040000407993798337002999687683768570423126220703125f,-0.084000001199543483654252895576064474880695343017578125f,-0.31800001238286501603624856215901672840118408203125f,0.592400044000149339495919775799848139286041259765625f,0.3214000149130822325815870499354787170886993408203125f,0.3008000419855125784351912443526089191436767578125f,-0.0822000001996754381394794108928181231021881103515625f,0.08790000244081019875608262736932374536991119384765625f,0.51620001887083066804962072637863457202911376953125f,0.3544000054657459042317668718169443309307098388671875f,0.09410000119507312688060807204237789846956729888916015625f,-0.12489999709874399780318299235659651458263397216796875f,0.18799999651312815363013442038209177553653717041015625f,0.21279999823570250327975372783839702606201171875f,-0.104600030267238963688214425928890705108642578125f,-0.16839999329149701878804989974014461040496826171875f,-0.055200003194808999751330702565610408782958984375f,0.987799977970123421755488379858434200286865234375f,-1.10939996150732067548005943535827100276947021484375f,0.547900002813339170870676753111183643341064453125f,0.378499979242682638158612462575547397136688232421875f,-0.49959998265802874062302407764946110546588897705078125f,0.2088999999001621699079578320379368960857391357421875f,-0.16350000733137193265065434388816356658935546875f,-0.2057999995172021812095408677123486995697021484375f,-0.0518000117659571657213746220804750919342041015625f,-0.75030005217790662896959474892355501651763916015625f,0.6250000291317701073268153777462430298328399658203125f,-0.4212000245332720993474140414036810398101806640625f,-0.015599974119663073679475928656756877899169921875f,0.20239998864233488262698301696218550205230712890625f,-0.0099999961256980185453357989899814128875732421875f,0.303299964499473162504727952182292938232421875f,-0.486199983555078318886444321833550930023193359375f,0.160399999833106932101145503111183643341064453125f,-0.99140000280588846948148784576915204524993896484375f,0.7815999915093183592063041942310519516468048095703125f,-0.5398000057831404685515508390381000936031341552734375f,0.25540002814531259645036698202602565288543701171875f,0.2265999755650762725878166747861541807651519775390625f,0.11590000871419903205605805851519107818603515625f,0.1243000132560727610808726240065880119800567626953125f,0.101199995103478490587889382368302904069423675537109375f,0.061500000566244050759223682689480483531951904296875f,-0.57659999366998671010975385797792114317417144775390625f,0.83619999145269385820711249834857881069183349609375f,-0.3333000068932772119723040304961614310741424560546875f,0.7026000647783288144410107634030282497406005859375f,-0.091000031769275846471600743825547397136688232421875f,0.3647000244140627955857780762016773223876953125f,-0.555800012257694930895013385452330112457275390625f,0.2978000061333176962108382213045842945575714111328125f,-0.2813000158116223392568144845427013933658599853515625f,0.68210001443475487548084856825880706310272216796875f,-0.3193999992072582561064564288244582712650299072265625f,0.3546000098899007024755292150075547397136688232421875f,0.617499998975544972523721298784948885440826416015625f,-0.8432000391960148277803455130197107791900634765625f,-0.4449999967962503522045381032512523233890533447265625f,-0.08839999951273203038226711214520037174224853515625f,-0.518500008523464117615731083787977695465087890625f,-0.4584000304698945260639675325364805757999420166015625f,-0.3366000065594911827560054007335565984249114990234375f,-0.7550000339746478772440241300500929355621337890625f,-0.1705999829858537708560106693767011165618896484375f,-0.136899991980194979390716980560682713985443115234375f,0.370400036787987829711710219271481037139892578125f,-0.290599998371302958855011411287705413997173309326171875f,0.0395000013448298104012934572892845608294010162353515625f,-0.069600004823505923212678681011311709880828857421875f,-0.7509999831169842376965561925317160785198211669921875f,-0.126799997870624014950635682907886803150177001953125f,-0.27230000902414275287810596637427806854248046875f,0.687399986109137106637945180409587919712066650390625f,0.24709999390840520216983122736564837396144866943359375f,0.85700001317262586297829329851083457469940185546875f,0.733800002512335680648902780376374721527099609375f,0.1587000002443790247497190648573450744152069091796875f,0.3190000144243239077468388131819665431976318359375f,0.2990000016838310070710349464206956326961517333984375f,0.07149999549984931146440203519887290894985198974609375f,0.61719999550580961766854670713655650615692138671875f,-0.34540001138150699944162624888122081756591796875f,-0.1796000034824013769085837566308327950537204742431640625f,-0.6455000260919334653664236611803062260150909423828125f,0.36560000258088098945563615416176617145538330078125f,0.2151999979048967170935924286823137663304805755615234375f,0.8480000080615279767215497486176900565624237060546875f,-0.409000007942319054876634254469536244869232177734375f,-0.14000000301748516395861088312813080847263336181640625f,-0.86010002603530910647577911731787025928497314453125f,-0.42639998548328883476443706967984326183795928955078125f,-0.042499998882412903800087633499060757458209991455078125f,-0.25950000147521468107925102231092751026153564453125f,0.04960000031590461322394958187942393124103546142578125f,0.003499999675899749529417448457024875096976757049560546875f,-0.081900004776567281172816592516028322279453277587890625f,0.185599994534253909250764991156756877899169921875f,-0.1052000018537044834232574430643580853939056396484375f,-0.1743999957799911015854377183131873607635498046875f,-0.630399982696771221668541329563595354557037353515625f,0.129299999547004773603475769050419330596923828125f,-0.38549998146295560985663541941903531551361083984375f,0.8607999787300826266545072940061800181865692138671875f,-0.242499999701976776123046875f,-0.13060004594326013460658941767178475856781005859375f,-0.0435999686956407561666537731071002781391143798828125f,-0.0808000103950501369354242342524230480194091796875f,-0.51040001980066307396555203013122081756591796875f,-0.3874000221699471779857049114070832729339599609375f,-0.232499998211860514629734097979962825775146484375f,-0.487399982830881650652798953160527162253856658935546875f,0.8407999949723479193863795444485731422901153564453125f,-0.28300000657141211046763373815338127315044403076171875f,-1.09249996095895784975482456502504646778106689453125f,1.2495999498754744028161667301901616156101226806640625f,-0.6098999942302700816298965946771204471588134765625f,0.5065999701261514331918078823946416378021240234375f,-0.889399969032406811919599931570701301097869873046875f,0.3122999875664707047917545423842966556549072265625f,1.0141999940752981501645990647375583648681640625f,-1.099799984958767851139782578684389591217041015625f,0.565100008749961801868266775272786617279052734375f,0.06100000010430761676616384647786617279052734375f,-0.6600000031292434510277189474436454474925994873046875f,0.0726000067010523775223873599315993487834930419921875f,-0.076100003787875181160416104830801486968994140625f,0.31840000372231003211709321476519107818603515625f,-0.066299999681115107108553274883888661861419677734375f,-0.04859999633729472634513513185083866119384765625f,-0.31219999565482137171557042165659368038177490234375f,0.007699996861815439075371614308096468448638916015625f,-0.0534999879598618832687861868180334568023681640625f,0.731999993428587902855042557348497211933135986328125f,-0.0772999988913536384416147484444081783294677734375f,-0.55809996607303669069466423025005497038364410400390625f,0.8861999571800236452645549434237182140350341796875f,-0.3247999937504528755738419931731186807155609130859375f,0.145900005156546885842772098840214312076568603515625f,-0.05379999930858614387574334614328108727931976318359375f,0.07400000352784996682231621889513917267322540283203125f,0.50339997503757416552616632543504238128662109375f,-0.76739999497532807026800583116710186004638671875f,0.2825000044703482871000232989899814128875732421875f,-0.3825999579370016334678439307026565074920654296875f,0.806300046098233025304580223746597766876220703125f,0.468799998414516583267186433658935129642486572265625f,0.0058000021249056121774856364936567842960357666015625f,-0.1953999980449676598937003291212022304534912109375f,-0.917399991175531948073285093414597213268280029296875f,-0.31589999463260165502020981875830329954624176025390625f,-0.80840000931024480479436533642001450061798095703125f,-0.0794000022619962830816575660719536244869232177734375f,0.04699999909102915707226344466107548214495182037353515625f,0.56130000776499511250250407101702876389026641845703125f,-0.598200045230985022470804324257187545299530029296875f,-0.243600013957917749252146677463315427303314208984375f,-0.140300055158139258537630666978657245635986328125f,1.1014000409156086135453733731992542743682861328125f,0.3347000069051981885337454514228738844394683837890625f,1.1068000335216527929560470511205494403839111328125f,-0.6768000400781633008051585420616902410984039306640625f,-0.3594000080734491842093802915769629180431365966796875f,-0.7286000366449361109744131681509315967559814453125f,0.7232000236988069818977464819909073412418365478515625f,0.213700008328259027923223811740172095596790313720703125f,0.230700035172701323205046719522215425968170166015625f,-1.278400031810999593062661006115376949310302734375f,-0.392600015231967081064112790045328438282012939453125f,-1.504600072884560546526699909009039402008056640625f,0.623800002512335804993881538393907248973846435546875f,0.23359999920576812737493810345767997205257415771484375f,0.91150000548362708485683469916693866252899169921875f,0.767599992507695727539385188720189034938812255859375f,0.4044999936148523378420804874622263014316558837890625f,1.2725000110268585018502562888897955417633056640625f,0.4889999917000533713462573359720408916473388671875f,0.108600000248849415385166139458306133747100830078125f,0.615900005137920203424073406495153903961181640625f,-0.631600023025274737165091210044920444488525390625f,-0.09860000982284555792745095459395088255405426025390625f,0.07239997903108541521532970364205539226531982421875f,0.366600002238154143441306587192229926586151123046875f,0.245699998590350077165567199699580669403076171875f,0.93080002887248980414369725622236728668212890625f,-0.4010000081509350611241870865342207252979278564453125f,-0.24700000091642138055902933047036640346050262451171875f,-0.56790002009868612020682121510617434978485107421875f,-0.4923999940812586739724565632059238851070404052734375f,0.550400026059151059598661959171295166015625f,0.9761999905586247194833049434237182140350341796875f,-0.909999977052212027928135285037569701671600341796875f,0.5264000101447106061414160649292171001434326171875f,-0.9225999866962428797023676452226936817169189453125f,1.079999968782067387707002126262523233890533447265625f,-0.5289999968409535568980572861619293689727783203125f,-0.56510001236349349795773377991281449794769287109375f,0.0962000025540590275596741776098497211933135986328125f,-0.2764000112995506075463936213054694235324859619140625f,-0.8623000513434408986768175964243710041046142578125f,0.353800047215818924684072044328786432743072509765625f,-0.452800026249885689821894629858434200286865234375f,1.02960004636049262671804171986877918243408203125f,-1.5526000286430114982749728369526565074920654296875f,0.60970003633499203488099738024175167083740234375f,-0.53020005981922135873674051254056394100189208984375f,0.3272000411033626487977699071052484214305877685546875f,-0.2974000234365465900054914527572691440582275390625f,1.03430002204775828289484707056544721126556396484375f,-0.8000000089406966718996727649937383830547332763671875f,0.56050001767277723985216653090901672840118408203125f,-1.0355999946832668001661659218370914459228515625f,1.893399982041120921394394827075302600860595703125f,-0.63610001161098495003898278810083866119384765625f,0.3597000011682514042377079022116959095001220703125f,-0.833599980095029113869031789363361895084381103515625f,0.2371000028491021538457061978988349437713623046875f,0.2828999932765956515368088730610907077789306640625f,-0.604799974900484205164730155956931412220001220703125f,0.196699987816810306640036287717521190643310546875f,0.38549997431039884077108581550419330596923828125f,-0.9201999600559478409422808908857405185699462890625f,0.24719999520778657142727752216160297393798828125f,-1.13840002361536107855499722063541412353515625f,0.936000001221895416847473825328052043914794921875f,-0.6120000250935557772891115746460855007171630859375f,-0.1196000100016600953267698059789836406707763671875f,-0.226200001063942668366735233576036989688873291015625f,-0.029900012111664153735546278767287731170654296875f,-0.20150001204013801014980344916693866252899169921875f,0.1170000037103890999645727788447402417659759521484375f,-0.1191999982476232844419428147375583648681640625f,-0.87909997489452340602156255044974386692047119140625f,-0.171699981558322889441114966757595539093017578125f,0.503400017133355248688530991785228252410888671875f,0.141000004909932641350422954928944818675518035888671875f,-0.164099987560510474082775544957257807254791259765625f,0.6195999925673003616566347773186862468719482421875f,0.187399993187189028986949779209680855274200439453125f,0.599199999141692529747160733677446842193603515625f,-0.159999998807907051201482317992486059665679931640625f,0.029499998085200818120910071229445748031139373779296875f,0.527699989932775448409074670053087174892425537109375f,0.5194000086694956497268549355794675648212432861328125f,0.053300003167986886865037376992404460906982421875f,-0.003999985218047896751158987171947956085205078125f,-0.063199984806776132728600714472122490406036376953125f,0.1847000024721026090190889590303413569927215576171875f,0.5397000270962717394240826251916587352752685546875f,0.972000018686056677097440115176141262054443359375f,0.136200010153651351885173426126129925251007080078125f,-0.032199980902671399007886066101491451263427734375f,0.9096000134289266014775421353988349437713623046875f,0.31140000381171706322902537067420780658721923828125f,0.84220002381801606361477752216160297393798828125f,-0.484400015151500884513779965345747768878936767578125f,-0.07480000462830072027742289719753898680210113525390625f,0.26499999612569791196392543497495353221893310546875f,-0.47500002630054982688534437329508364200592041015625f,-0.29750000901520257112764511475688777863979339599609375f,-0.84920004175901464549269803683273494243621826171875f,-0.34520001556277291054897204958251677453517913818359375f,-0.151000004798173936837457631554570980370044708251953125f,-0.209600016409158929064915355411358177661895751953125f,-0.30460003049075634606879248167388141155242919921875f,-0.13700000814348467503833717273664660751819610595703125f,0.09949998664855908003801232553087174892425537109375f,0.755599994981289047046857376699335873126983642578125f,0.1303000021472573433101160844671539962291717529296875f,0.0467000059962270341884504887275397777557373046875f,-0.745999985426664125043316744267940521240234375f,-0.1011999964445828847914299331023357808589935302734375f,-0.1214000020980829930294930818490684032440185546875f,0.094400000101327929957761853074771352112293243408203125f,0.0220000005811452903348168774755322374403476715087890625f,0.1208000042110681793161575114936567842960357666015625f,0.6132000409841540378153013080009259283542633056640625f,0.2951999963402744953100409475155174732208251953125f,0.95610003053545966622550622560083866119384765625f,-0.429200009796023351782423560507595539093017578125f,0.49650001904368412652956976671703159809112548828125f,0.59639999077320027254245360381901264190673828125f,-0.8655999849975106741339914151467382907867431640625f,0.35080000085830675971010350622236728668212890625f,-0.6774999929964540257287808344699442386627197265625f,0.215999996602535215828311265795491635799407958984375f,-0.3389000029921529488774467608891427516937255859375f,0.90860000058412548895603322307579219341278076171875f,-0.9147999973267315620972794931731186807155609130859375f,0.496800014150142743574178894050419330596923828125f,-0.5812999796390538875812126207165420055389404296875f,0.654199969488382659932312890305183827877044677734375f,-0.308800007057190128989532240666449069976806640625f,1.7020000146627438653013086877763271331787109375f,-1.545399978524446993333185673691332340240478515625f,0.922900028443336850614286959171295166015625f,0.88640001103878063304364331997931003570556640625f,-1.0985999858319761557368110516108572483062744140625f,0.51230001051425944069706019945442676544189453125f,-1.0236000274062160286803191411308944225311279296875f,0.617800004494190257986474534845910966396331787109375f,-0.5363000228524210655223214416764676570892333984375f,-0.110000025033951231989703956060111522674560546875f,0.20500001393258582282896895776502788066864013671875f,-0.0831000045180321222915154066868126392364501953125f,-0.400800022166967639947188217774964869022369384765625f,0.22360000848174099719045671008643694221973419189453125f,-0.216200009781122304275413625873625278472900390625f,-0.555900054311752711555527639575302600860595703125f,0.04080003103315821277874420047737658023834228515625f,-0.2797000253081325382709110272116959095001220703125f,1.2608999869585044706354892696253955364227294921875f,-1.133399959242344490206733098602853715419769287109375f,0.684800006341934164311169297434389591217041015625f,-1.2029999704360960777194122783839702606201171875f,1.27239995980858822122172568924725055694580078125f,-0.66320000104904153204188332892954349517822265625f,0.0725999956369400845090922302915714681148529052734375f,-0.174199995341897062584024524767301045358180999755859375f,0.0466999994769692339247058043838478624820709228515625f,0.35980005074739462855859528644941747188568115234375f,-0.24099999224394530461523800113354809582233428955078125f,0.094599986681342229655200526394764892756938934326171875f,-0.13000000048428772547737253262312151491641998291015625f,-0.12470000399947167579739470966160297393798828125f,-0.1641999868482353353016378605389036238193511962890625f,0.01899999948590991305508168807136826217174530029296875f,-0.12949999678134904712578645558096468448638916015625f,-0.7062000010639426506031668395735323429107666015625f,-0.3839999991878866314465312825632281601428985595703125f,-1.2443000373959538507051547640003263950347900390625f,0.023399995228647885170403242227621376514434814453125f,-0.16290000385642056102142305462621152400970458984375f,-0.553800008249282882388797588646411895751953125f,-0.116600012221932392009193790727294981479644775390625f,-0.0898000035479664671100863415631465613842010498046875f,0.4266999899029730869415288907475769519805908203125f,0.33240002656579026307781532523222267627716064453125f,0.1980000050812960044055444086552597582340240478515625f,-0.1773999930381773282306312466971576213836669921875f,-0.3660000179111959628386330223293043673038482666015625f,-0.18090000565946107169423839877708815038204193115234375f,-0.76810003152638717249800492936628870666027069091796875f,0.098199999037384966271702069207094609737396240234375f,-0.042099999421834954915766502381302416324615478515625f,-0.16959999480247489600515109486877918243408203125f,-0.384799988609552112706069237901829183101654052734375f,-0.2154999945387243709404856417677365243434906005859375f,-0.0512999799370756903726942255161702632904052734375f,-0.01399999117851269403445257921703159809112548828125f,0.013700001399219008391128227231092751026153564453125f,-0.371899991905689120130773517303168773651123046875f,0.59320000745654100082759896395145915448665618896484375f,0.054200000929832459206636485760100185871124267578125f,0.2240000153183938635947924922220408916473388671875f,0.816200006130337829546306238626129925251007080078125f,0.087100002104044005335481415386311709880828857421875f,-0.104600000762939515652760746888816356658935546875f,0.896000040560961164004538659355603158473968505859375f,0.35410001322776085697086045911419205367565155029296875f,1.11390006825924015032569514005444943904876708984375f,0.816000030428171374552448469330556690692901611328125f,0.1377000094905496130337496651918627321720123291015625f,0.0679000406622893848407329642213881015777587890625f,-0.59960000672340385552416819336940534412860870361328125f,0.2828000153347851597374074117396958172321319580078125f,0.3158999640107158501933781735715456306934356689453125f,-0.696799969968199928160856870817951858043670654296875f,0.186099997156858432134640679578296840190887451171875f,-0.2092999708622702570437468239106237888336181640625f,0.5055999607086187896953788367682136595249176025390625f,-0.1298999923303723047496305298409424722194671630859375f,-0.20229998637437773822966846637427806854248046875f,0.5425999854296446045509583200328052043914794921875f,-0.152799985122680226368174771778285503387451171875f,0.5105999890208234859301228425465524196624755859375f,-0.467400008907913555589175302884541451930999755859375f,0.2649000100255012313255065237171947956085205078125f,-0.4749000296950338650958656216971576213836669921875f,-0.334599981167912563506661172141321003437042236328125f,-0.2156000086903571144603120046667754650115966796875f,0.634600061559677186551198246888816356658935546875f,0.19479996774792684988142354995943605899810791015625f,0.307600023055076832179111079312860965728759765625f,-0.02450000814348474165171865024603903293609619140625f,0.4188000092923640504949389651301316916942596435546875f,-0.0473000034734607499586900303256697952747344970703125f,0.31579999050199969445884562446735799312591552734375f,-0.31039999744892110555838371510617434978485107421875f,0.1686000024840235145262568039470352232456207275390625f,-0.5567000146389009529457325697876513004302978515625f,-0.040600024634599218842367918114177882671356201171875f,-0.279999997615814066875827847979962825775146484375f,0.235500018119811471706270822323858737945556640625f,0.27259997715950046170974019332788884639739990234375f,0.098800006461143397018531686626374721527099609375f,0.876799993586539816448066630982793867588043212890625f,-1.18599999913573217469320297823287546634674072265625f,0.49740001449584969606121376273222267627716064453125f,1.4968000081896786213064842741005122661590576171875f,-1.374799980935454524200167725211940705776214599609375f,0.81060002388954188745628925971686840057373046875f,0.6485000240206719723801143118180334568023681640625f,-1.022200024265050632976681299624033272266387939453125f,0.3952000160098076975145886535756289958953857421875f,1.3505000216960905135010762023739516735076904296875f,-1.245400021141767066268357666558586061000823974609375f,0.7352000288248063952778466045856475830078125f,-0.537400004959106336599461428704671561717987060546875f,0.0641000222802166064184348215349018573760986328125f,0.410000007301568825113236016477458178997039794921875f,0.2658999990656971679214848336414434015750885009765625f,0.2523000000834461076237857923842966556549072265625f,0.495800030139089020764231463545002043247222900390625f,0.251800010313093736158407409675419330596923828125f,0.34090003196001106289259041659533977508544921875f,0.1833999995499848711943968737614341080188751220703125f,0.0808999987304210577132579373937915079295635223388671875f,0.3988000032573937314595013958751223981380462646484375f,0.56000001609325433804542626603506505489349365234375f,0.1972000048935413918371750696678645908832550048828125f,0.9500000268220905041971491300500929355621337890625f,0.5510000098645686517073727372917346656322479248046875f,0.304300001148879506462918698161956854164600372314453125f,0.49760001463592029580240705399774014949798583984375f,0.3352000073671341784375954375718720257282257080078125f,-0.0255999963223934035028150901780463755130767822265625f,-0.1643999832630154145363121642731130123138427734375f,-0.442400016134977480675161132239736616611480712890625f,-0.064900005890429046662148948598769493401050567626953125f,0.233299980443715693212425321689806878566741943359375f,-0.09660002026855973866759086376987397670745849609375f,-0.2223000077575445487809702171944081783294677734375f,-0.756800037693977856179117225110530853271484375f,0.440599998147785498758821631781756877899169921875f,0.1620999980993568512221969513120711781084537506103515625f,0.338200006078183346147625343292020261287689208984375f,0.9721999949842687893664106013602577149868011474609375f,0.323799996291100899981074690003879368305206298828125f,0.78030001641511859133970574475824832916259765625f,0.568800006759166532077642841613851487636566162109375f,0.32909999877363438347543933559791184961795806884765625f,0.4993000104248519477323497994802892208099365234375f,-0.347999995768069947388312357361428439617156982421875f,-0.282599996456503799180381975020281970500946044921875f,-0.821800010275840264739599660970270633697509765625f,-0.032600003758072759740826995766838081181049346923828125f,0.07370000084042549470808580736047588288784027099609375f,0.0147999972522256939555518329143524169921875f,0.3374000218719241939169251054408960044384002685546875f,0.248200000341236570733371991082094609737396240234375f,0.66010001858472833191626705229282379150390625f,-0.17360000080764292906820855932892300188541412353515625f,0.4092000101685524526828885427676141262054443359375f,0.4355000336170196106877483543939888477325439453125f,0.099199996310472382532452684245072305202484130859375f,0.22090000543594356940957368351519107818603515625f,0.5317000417590147520741084008477628231048583984375f,-0.193000018343329582393153032171539962291717529296875f,0.283800015997886845298125990666449069976806640625f,0.00659999657571308517844954621978104114532470703125f,-0.2579999949485063392984329766477458178997039794921875f,0.0231999983668327303121259319595992565155029296875f,0.234000003695488345556441345252096652984619140625f,-0.86639999792575839165920115192420780658721923828125f,0.176000007033348282448059762828052043914794921875f,0.49500002443790436235104834850062616169452667236328125f,-0.16960000009238729301586090514319948852062225341796875f,0.2678000033766030352211373610771261155605316162109375f,0.72879999334812151090545739862136542797088623046875f,-0.7644000011444089803802626192918978631496429443359375f,0.3962000114202499645443822373636066913604736328125f,-0.9301000163495540817848450387828052043914794921875f,0.583000009924173223652132946881465613842010498046875f,-0.48840001750588424300758561003021895885467529296875f,0.467599982821941040356250596232712268829345703125f,-0.35459998325407493524608071311376988887786865234375f,0.2339000059723854718640723149292171001434326171875f,0.474399998611211570764822909040958620607852935791015625f,-0.449999988824129115716488058751565404236316680908203125f,0.26579999985247837468449461084674112498760223388671875f,1.03729998816251711701852400437928736209869384765625f,-1.1531999723643064914568867607158608734607696533203125f,0.5892999989509579705782016390003263950347900390625f,0.5147000256061551315411861651227809488773345947265625f,-0.1046000006139278237782264113775454461574554443359375f,0.275000001937150795328079766477458178997039794921875f,0.0910000004768374282093645888380706310272216796875f,-0.024800011408328881401530452421866357326507568359375f,0.028600013399124435409248690120875835418701171875f,-0.1013999958395952472045564718428067862987518310546875f,0.376400012603401667998781476853764615952968597412109375f,-0.066800007444620224106301975552923977375030517578125f,0.02110003753900446810121138696558773517608642578125f,-0.0176000362426036449647881454438902437686920166015625f,0.0261000104188919834768967120908200740814208984375f,0.12560001602917925112024022382684051990509033203125f,0.645500010818242486720919259823858737945556640625f,0.6163999731898304190735871088691055774688720703125f,0.064599993908405206610723325866274535655975341796875f,-0.123200004625321213325150893069803714752197265625f,-0.13960000568032260304107694537378847599029541015625f,-0.2174000010848045061351285767159424722194671630859375f,-0.8476000242471695145241028512828052043914794921875f,0.375600002729892690211954686674289405345916748046875f,-0.0630000002384185631143509453977458178997039794921875f,-0.4432000076055526704976728069595992565155029296875f,-0.06580000011324871334039698922424577176570892333984375f,0.067000001698732380983614120850688777863979339599609375f,0.325700009244680455111620176467113196849822998046875f,-0.7086000352293257975588858244009315967559814453125f,-0.393300013673305581818340215249918401241302490234375f,-1.254200065898896099270132253877818584442138671875f,-0.188000009179115412649707650416530668735504150390625f,-0.17530000229626896413037684396840631961822509765625f,-0.437500007450580596923828125f,-0.0109999988377094126690280972979962825775146484375f,0.01610000043511390643846681314244051463901996612548828125f,0.14680000487416988708133658292354084551334381103515625f,0.0888000016927719126869078536401502788066864013671875f,0.09090000029504298473259638058152631856501102447509765625f,0.52750001203268770400001130838063545525074005126953125f,-0.2326000089734792108941974220215342938899993896484375f,-0.12500000152736900904670847012312151491641998291015625f,0.02349999153614046321081332280300557613372802734375f,0.399999990612268430112408168497495353221893310546875f,0.1420999976709484968750984990037977695465087890625f,0.79310000392198531926624127663671970367431640625f,-0.018400023168325729816530156313092447817325592041015625f,-0.17630000724345451512675708727329038083553314208984375f,-0.1997000159204009861468875897116959095001220703125f,-0.3494000001758335027801649630418978631496429443359375f,-0.0486999985307454696936702021048404276371002197265625f,0.0459999951124192563156611868180334568023681640625f,0.417600016498565818068300359300337731838226318359375f,0.2187000062420964285880842226106324233114719390869140625f,0.8145000434964899671541616044123657047748565673828125f,-0.23060000019669513182662967665237374603748321533203125f,0.092000003114342698840033563101314939558506011962890625f,0.183900007611513149896609320421703159809112548828125f,-0.441799992915987704833469251752831041812896728515625f,-0.8539999767616393011593345363507978618144989013671875f,0.88349998373165694598441177731729112565517425537109375f,-0.52529998119249954324772033942281268537044525146484375f,-0.19949999568238852010182426965911872684955596923828125f,-0.29299999338388449388048684340901672840118408203125f,-0.27190000054836271914382450631819665431976318359375f,-0.633799984481930867019627839908935129642486572265625f,-0.0960000004768370995833492997917346656322479248046875f,-0.3167999876633287303917541066766716539859771728515625f,-0.390799993854761051892410250729881227016448974609375f,0.81949998605251295202833716757595539093017578125f,-0.60649997940659527984053056570701301097869873046875f,0.6460999937295912598500535750645212829113006591796875f,-0.573999970316886987120597041212022304534912109375f,-0.6857000187814232816663206904195249080657958984375f,-0.7567999723523859412210867958492599427700042724609375f,-0.586500032901764001280753291212022304534912109375f,0.4293000098288057841955378535203635692596435546875f,0.7048999900579453470328417097334749996662139892578125f,0.554500015109777422139814007095992565155029296875f,0.09410000603795065643453199299983680248260498046875f,0.612700003194808662243531216518022119998931884765625f,0.76949999365210519641777864308096468448638916015625f,-0.3692999855399132780320314850541763007640838623046875f,-0.0267000004082917907766159260063432157039642333984375f,-0.600999993920325881191502048750407993793487548828125f,0.657699972125887821761125451303087174892425537109375f,-0.67500001460313752144060117643675766885280609130859375f,0.61050001111626617245065062888897955417633056640625f,-0.0929999874234199719325033584027551114559173583984375f,-0.23429998878091584657568091643042862415313720703125f,0.4194999890327453240246313725947402417659759521484375f,-0.5199999895319342702038056813762523233890533447265625f,0.5742999896749854116251299274154007434844970703125f,-1.28600003266334539375748136080801486968994140625f,0.3265999774277208445028009009547531604766845703125f,-0.63620000840276436093745360267348587512969970703125f,0.533000005379319130582871366641484200954437255859375f,-0.66559999445974826759453435442992486059665679931640625f,-0.3081999959081411777361836357158608734607696533203125f,-0.37449999599903804892164771445095539093017578125f,0.136599996948242097261072558467276394367218017578125f,-0.72629999770671105352448648773133754730224609375f,1.209500027179718273373509873636066913604736328125f,-0.97440003899335891901500872336328029632568359375f,-0.2782000326395035472160088829696178436279296875f,-1.65679997749328578038330306299030780792236328125f,0.3314000065684314932923371088691055774688720703125f,0.238400018399953950165581773035228252410888671875f,-0.4547000074267391056537235272116959095001220703125f,-0.689500041782856509797738908673636615276336669921875f,-0.71729999382495890358768519945442676544189453125f,1.12259998729228982483618892729282379150390625f,-0.0701999951481813155851341434754431247711181640625f,0.0640000190436840199481594027020037174224853515625f,1.2079999787807462752198262023739516735076904296875f,0.22399999982118590224899890017695724964141845703125f,-0.7215999900937077882190351374447345733642578125f,-0.456799958717823528786539100110530853271484375f,-0.5954000516891486682879985892213881015777587890625f,0.650300001811981331911738379858434200286865234375f,0.02080001754760729681947850622236728668212890625f,-0.62920001195669161830892335274256765842437744140625f,-0.583200026958435646662337603629566729068756103515625f,0.1527000621661545043394880849518813192844390869140625f,1.0109000328168271654050158758764155209064483642578125f,0.504499999463558168599774944595992565155029296875f,-0.14150001412630075492415926419198513031005859375f,0.02050000023841835883331441436894237995147705078125f,-0.0408000056266786259584478102624416351318359375f,-1.164000018894672194846862112171947956085205078125f,0.26040002397298867009567402419634163379669189453125f,0.30539998433589943260813015513122081756591796875f,-0.60989996979236593688256107270717620849609375f,-0.5533000106930732187038302072323858737945556640625f,0.9956000191211700212079449556767940521240234375f,-0.5552000365734102160786278545856475830078125f,0.108400011992454636811089585535228252410888671875f,-0.00350001895427709541763761080801486968994140625f,-1.05189998803138706762183574028313159942626953125f,-0.6441000118494031312366132624447345733642578125f,0.69829997865557658087709569372236728668212890625f,1.072299982649088434527584468014538288116455078125f,-0.6977000215232376234553157701157033443450927734375f,0.853700001734495117489132098853588104248046875f,-0.3989000238537787623727126629091799259185791015625f,-0.734700020539760689786135117174126207828521728515625f,1.150700016546249315751992980949580669403076171875f,-1.012900011940299993540293144178576767444610595703125f,-0.219499995589256258199384319595992565155029296875f,-0.71379998902678476468963708612136542797088623046875f,0.4149000060394403277541641728021204471588134765625f,0.73050001558661481482204180792905390262603759765625f,-0.5223000055968760602098654999281279742717742919921875f,0.0496999867513776560912219792953692376613616943359375f,0.61749998509883852193524944595992565155029296875f,0.71329999504685392963665435672737658023834228515625f,0.3480999927461148057972195601905696094036102294921875f,-0.34799999129772185568754139239899814128875732421875f,0.7055999808251858240737419691868126392364501953125f,-0.299999996423721271288087564244051463901996612548828125f,1.318500019848347193374138441868126392364501953125f,-0.30359998523592945929294728557579219341278076171875f,0.73169997258037344778358601615764200687408447265625f,0.343500003755092819801575387828052043914794921875f,-0.329699979263544218355264092679135501384735107421875f,-0.34510000040531141340949261575588025152683258056640625f,-0.89499999580904809892700768614304251968860626220703125f,0.538300024979561531068839030922390520572662353515625f,0.18540000890418883994215093480306677520275115966796875f,-0.39999999105930328369140625f,-0.167000007063150324171374450088478624820709228515625f,-0.34899998987466107625010636184015311300754547119140625f,-0.3050000309944156384744928800500929355621337890625f,-0.522400007939338717477539830724708735942840576171875f,0.1725999963074922316508263975265435874462127685546875f,0.50300000441074388390916283242404460906982421875f,-0.57169997157454499614459564327262341976165771484375f,-0.0017999994724988699346113207866437733173370361328125f,-0.8770000128746033851712127216160297393798828125f,-0.4532000204205512972066571819595992565155029296875f,-0.656199996891617676197938635596074163913726806640625f,0.853499994516372595398934208787977695465087890625f,-0.7769999797940254637751422706060111522674560546875f,0.3184999964535235594809137182892300188541412353515625f,0.392000043421983690450360882095992565155029296875f,0.196600006037950691961668781004846096038818359375f,-0.5445999674960975145410202458151616156101226806640625f,0.33350001075863833222001630929298698902130126953125f,-0.6284000017106532443023070300114341080188751220703125f,-0.39989999303817758136148086123284883797168731689453125f,0.34650002723932260551009676419198513031005859375f,-0.090499996364115986580145545303821563720703125f,0.5756000173330306779462262056767940521240234375f,0.533799971592426203415016061626374721527099609375f,-0.60860001131296126430925141903571784496307373046875f,-0.19930004067420981073155417107045650482177734375f,-0.84139999554157185457370360381901264190673828125f,-0.3892000015258787470884271897375583648681640625f,1.123200020420551226152383605949580669403076171875f,-0.151200053143501378372093313373625278472900390625f,-0.33540000787973411888742703013122081756591796875f,-1.3666999782800672846860834397375583648681640625f,-0.20469997553825436398255988024175167083740234375f,0.7873000000834462497323329444043338298797607421875f,-0.63160001602172854973105131648480892181396484375f,-0.75139997646808609488289221189916133880615234375f,0.660200029420852985140300006605684757232666015625f,-0.402899995809793409762278315611183643341064453125f,1.4628000038981436858875895268283784389495849609375f,0.024399976110457632927364102215506136417388916015625f,1.0288000034995377074409361739526502788066864013671875f,0.2899999834597111192380225475062616169452667236328125f,-0.39140000153928988790852372403605841100215911865234375f,1.1021000216245653291480266489088535308837890625f,-0.0385999925374989061310770921409130096435546875f,-0.629200007486343082518942537717521190643310546875f,-0.17239999959468832457787357270717620849609375f,0.67490002701282492125756107270717620849609375f,0.667300037336350015948482905514538288116455078125f,-0.7371000120878221650855266489088535308837890625f,0.4547999830961231282344670034945011138916015625f,0.137000006914139049740697373636066913604736328125f,-0.02359999223947539803702966310083866119384765625f,1.301599961781501946234129718504846096038818359375f,-0.38580001128911955987632609321735799312591552734375f,-1.501499980151653090842955862171947956085205078125f,0.251700032520293603965910733677446842193603515625f,0.4872999810099596373902386403642594814300537109375f,0.01100000435113912544693448580801486968994140625f,-1.188499995112419327369934762828052043914794921875f,-0.00350000852346443735996217583306133747100830078125f,-0.2928999835908410176443794625811278820037841796875f,-0.8608999550700193736929577426053583621978759765625f,-0.00090000394582734344339769450016319751739501953125f,-0.1664000180423261809892210294492542743682861328125f,0.58549997591227320725693061831407248973846435546875f,0.3689999965429304751296513131819665431976318359375f,-0.035400006687641205616046136128716170787811279296875f,-0.3822000009447334978318622233928181231021881103515625f,0.663000005006789905337427626363933086395263671875f,-0.5402999821424483428700114018283784389495849609375f,0.3716000136747952264926198040484450757503509521484375f,-0.3914999971389772071006518672220408916473388671875f,-0.41400002530217161478276466368697583675384521484375f,-0.72389999215155842460234225654858164489269256591796875f,-0.019000015616416732200377737171947956085205078125f,0.42579998789429662764405293273739516735076904296875f,0.866399967862665931761512183584272861480712890625f,-0.394000022768974389464347041212022304534912109375f,-0.051799991500377728925741394050419330596923828125f,0.4491999951183796024878347452613525092601776123046875f,0.25850003728270554148593873833306133747100830078125f,0.8646999931216241552789369961828924715518951416015625f,0.0403000066548584445769165540696121752262115478515625f,-0.25049999219179153175929286589962430298328399658203125f,-0.17999999886378648117357670344063080847263336181640625f,-0.71809998626410964650546020493493415415287017822265625f,0.008999991178512800615862943232059478759765625f,-0.4496999862670900682815045001916587352752685546875f,-0.665200007255374980985607180627994239330291748046875f,-0.75150002098083490409408113919198513031005859375f,0.6189000116348264413090873858891427516937255859375f,-0.13070000041574214577622115029953420162200927734375f,0.3639999959468838142129243351519107818603515625f,0.4324999916553498024995860760100185871124267578125f,0.432200003887712735917148165754042565822601318359375f,0.5165000087618825119761822861619293689727783203125f,0.565599984401464528360747863189317286014556884765625f,-0.122299992670118928828060234081931412220001220703125f,0.4720000021457675387637209496460855007171630859375f,0.592999993234872846414873492904007434844970703125f,0.95099996884912274541790111470618285238742828369140625f,0.0029999951720238726693423814140260219573974609375f,-0.635100002044439282400389856775291264057159423828125f,0.194999988600611739997248150757513940334320068359375f,0.22299997788667713649601864744909107685089111328125f,-0.40059998186826728527876184671185910701751708984375f,0.360799969677627563324762149932212196290493011474609375f,0.60650001323223090565761594916693866252899169921875f,-0.24559999982416613750757505840738303959369659423828125f,1.2955000014305113609225372783839702606201171875f,0.74059995653629329126488300971686840057373046875f,-0.8206000164389610773696404066868126392364501953125f,-0.0743000017821788816263506305404007434844970703125f,0.416300001543760345157352276146411895751953125f,-0.021900001889467279880818750825710594654083251953125f,-0.563399969226122010468316148035228252410888671875f,1.6901999510407446081217130995355546474456787109375f,-0.041000012695789234129506439785473048686981201171875f,1.2323000021696088879252783954143524169921875f,0.2496999403715136622849968262016773223876953125f,-0.71310002001523997705589863471686840057373046875f,0.53030002297163036928395740687847137451171875f,0.53470002709627095782707328908145427703857421875f,0.312900018012523872812380432151257991790771484375f,-0.9782000132679939952140557579696178436279296875f,-0.43389998868703827383797033689916133880615234375f,0.446500002801418105491393362171947956085205078125f,0.20170000122785580742856836877763271331787109375f,-1.4208999783158304808239336125552654266357421875f,0.21330000532865511786440038122236728668212890625f,-1.474400005614757702687711571343243122100830078125f,-0.235000010728835917461765347979962825775146484375f,0.9542000122547147356044661137275397777557373046875f,-0.68260001947879800354712642729282379150390625f,0.477400010025501586596874403767287731170654296875f,0.08439999876022330482783218030817806720733642578125f,-1.2306000334262847673016949556767940521240234375f,-0.52439993975162479955542949028313159942626953125f,0.8562000307917596586548825143836438655853271484375f,1.173599983596801621388294734060764312744140625f,-0.1164000235557551832243916578590869903564453125f,0.050600022399425625962976482696831226348876953125f,0.61889999852180466177742346189916133880615234375f,0.14999996840953855326006305404007434844970703125f,-1.04720001517534200274894828908145427703857421875f,0.353300000672042369131986561114899814128875732421875f,-0.635000005811452883364154331502504646778106689453125f,-0.31540000929534439677581758587621152400970458984375f,0.01990000443756567705122506595216691493988037109375f,0.566599974037707010410258590127341449260711669921875f,-0.40080000074654764663506512079038657248020172119140625f,0.43969999416470528785794158466160297393798828125f,-0.22099999935925007665815655855112709105014801025390625f,0.112500023245811320293796597979962825775146484375f,-0.13830000696778288471477935672737658023834228515625f,-0.898499975517392357460266794078052043914794921875f,0.0815000060796737813006984652020037174224853515625f,0.2284000012636184795411509185214526951313018798828125f,-0.011899996823072467311277478074771352112293243408203125f,0.46200000020861597960220024106092751026153564453125f,0.9947999650657177728163560459506697952747344970703125f,0.132799992051720661123681566095910966396331787109375f,0.021000024914741288739605806767940521240234375f,-0.3504000199496744016869342885911464691162109375f,-0.791599975825846513544092886149883270263671875f,-0.8065000135302540229531587101519107818603515625f,0.2866000100612637879748945124447345733642578125f,-0.5451000077813858979425276629626750946044921875f,0.14149999922513956107650301419198513031005859375f,0.11229999710321436623416957445442676544189453125f,0.693899994535744024659607021021656692028045654296875f,-0.421500011742114821799987112171947956085205078125f,-0.722399992740154317516498849727213382720947265625f,0.0490999895721675461146560337510891258716583251953125f,-0.1580000073909761937329676584340631961822509765625f,0.48739997456073780313090537674725055694580078125f,0.91479999173879600338210593690746463835239410400390625f,0.3925000113248824362699451739899814128875732421875f,0.416000005990266874533745067310519516468048095703125f,0.3997000067189333538664186562527902424335479736328125f,-0.208500013589858923523934208787977695465087890625f,0.25260002648234358701984092476777732372283935546875f,0.80669999627321953283853872562758624553680419921875f,-0.93549999427795427209275658242404460906982421875f,-0.2744000009953975194321174058131873607635498046875f,-0.84869996988326334985686116851866245269775390625f,0.914499996185302421736196265555918216705322265625f,-0.3966999960124493185276151052676141262054443359375f,-0.2552999946594238789288056068471632897853851318359375f,0.1555000000149011629702044956502504646778106689453125f,-0.51369999894052742472894124148297123610973358154296875f,-0.08310000146329403480915942736828583292663097381591796875f,0.5079999896585938312654207038576714694499969482421875f,0.161299994949996416959692169257323257625102996826171875f,-0.0257999998524784945885812703636474907398223876953125f,-0.950999994665384207337410771287977695465087890625f,-0.0707000123739245367460171109996736049652099609375f,0.278700043308734990432640188373625278472900390625f,1.959899968004226167295200866647064685821533203125f,0.040499994874000577738115680404007434844970703125f,0.70490000555515308633403037674725055694580078125f,0.75130002136230444875764078460633754730224609375f,0.464500015556811973738149390555918216705322265625f,0.34030000469908117111117462627589702606201171875f,-0.069700009866803913638477752101607620716094970703125f,0.1823000129543246305985348953981883823871612548828125f,-1.119600001657008903066525817848742008209228515625f,-0.099599938178062785709698800928890705108642578125f,0.84240001628398886168724857270717620849609375f,-0.64330000782459961783388280309736728668212890625f,-1.63339995823651573658708002767525613307952880859375f,0.656800011356174717747080649132840335369110107421875f,0.0445999984532594595521004521287977695465087890625f,-0.05750000014901157641133977449499070644378662109375f,-0.2193000053584575059772987515316344797611236572265625f,0.225500002902001117632835303083993494510650634765625f,0.3477999893322587521282684974721632897853851318359375f,-0.7458000210866331247672178506036289036273956298828125f,-0.18239999005794516051537357270717620849609375f,0.02250000119209261129071819595992565155029296875f,0.8407000019431105641842805198393762111663818359375f,0.341600035244226774011622183024883270263671875f,1.390300015968083613415728905238211154937744140625f,-0.3981000314891343094814146752469241619110107421875f,0.1707999973565339502101778634823858737945556640625f,0.28729999129176153616072042495943605899810791015625f,-0.2947000025093553521315925536327995359897613525390625f,0.114499999333173008153607952408492565155029296875f,0.400299992554634886943176752538420259952545166015625f,-0.1803000056676566931201932675321586430072784423828125f,1.0166999842405317622251459397375583648681640625f,0.3473999662160878187933121807873249053955078125f,-0.679000018894672763281050720252096652984619140625f,-0.314799990397691686894177109934389591217041015625f,1.0053999968528746222773406771011650562286376953125f,0.003999984920024513002090316149406135082244873046875f,-0.103000024676322965433428180404007434844970703125f,0.660400025463104611844755709171295166015625f,0.51460003145933175261461656191386282444000244140625f,-1.43510002715289619601435333606787025928497314453125f,0.299699988576769715820091732894070446491241455078125f,-0.2704999972581862976994671043939888477325439453125f,1.00099997395277018341630537179298698902130126953125f,-0.54590000461637944173531877822824753820896148681640625f,-1.135999986171722042627152404747903347015380859375f,0.41900000652670854606185457669198513031005859375f,-0.7417999867692588455980740036466158926486968994140625f,-0.49100000805780297241653897799551486968994140625f,-0.0418000055074692067336172840441577136516571044921875f,-0.2755999968253075582680367006105370819568634033203125f,-0.2750000119209285998067571199499070644378662109375f,0.45200001704692827075859895558096468448638916015625f,0.65619998079836394655472986414679326117038726806640625f,-0.376000015117227803784771822392940521240234375f,-0.519499990224838281704933251603506505489349365234375f,0.6085999827027321895656086780945770442485809326171875f,0.255000000372529012082623012247495353221893310546875f,-0.11369999644458295140481141061172820627689361572265625f,0.02379999930858611723039075513952411711215972900390625f,0.80950000629946583519114255977910943329334259033203125f,-0.112299995296448518189436072134412825107574462890625f,-0.06550000051781534882167079558712430298328399658203125f,-0.967999982357024890688990126363933086395263671875f,0.33289997941851634521981395664624869823455810546875f,-0.049199998173117709399093655520118772983551025390625f,0.26150002202391675609760568477213382720947265625f,0.53710000195503226905202609486877918243408203125f,-0.4017000112488862129112021648325026035308837890625f,0.258500000849366085020619721035473048686981201171875f,0.01930000245273111314503466928726993501186370849609375f,-0.11109999540597204070735415371018461883068084716796875f,0.149500007864087791631391155533492565155029296875f,0.1291999989926815306517937642638571560382843017578125f,-0.11349999472126370836377873274614103138446807861328125f,0.1355000348091124351412872783839702606201171875f,-0.2447000034034250148806677316315472126007080078125f,-0.657899969583750188206749953678809106349945068359375f,0.22250001773238192726012130151502788066864013671875f,0.5973999915480614486540389407309703528881072998046875f,0.0580999995633959542828961275517940521240234375f,-0.602499999105930328369140625f,0.545800017398595738171707125729881227016448974609375f,-0.14649998182803403157237198684015311300754547119140625f,0.244500005125999120281221621553413569927215576171875f,-0.97230002362728118470158733543939888477325439453125f,0.2296999953567979702029333566315472126007080078125f,0.6813999588847163835225728689692914485931396484375f,0.26600001001358020147335992078296840190887451171875f,-0.31459997572898856788015109486877918243408203125f,1.760899989938735643590916879475116729736328125f,0.1458999896407124907682373304851353168487548828125f,0.0508999647557732970426513929851353168487548828125f,1.1568999892234810289437518804334104061126708984375f,0.37490003058910392752522966475225985050201416015625f,-0.182499967515468597412109375f,1.619499964594840690779165015555918216705322265625f,-0.06950001466274269290579468361102044582366943359375f,0.89949998426437360876661841757595539093017578125f,0.8611999679088597758891410194337368011474609375f,-0.7552000154137612497606824035756289958953857421875f,-0.18919999824762356865903711877763271331787109375f,0.6676999814391137988422997295856475830078125f,-0.22709999271631176043229061178863048553466796875f,0.3182000019431114878898370079696178436279296875f,1.524999998509883880615234375f,-0.5516000339031226218367010005749762058258056640625f,0.5305999931931495439130230806767940521240234375f,0.30549999159574525720017845742404460906982421875f,-0.673300024402141872315041837282478809356689453125f,1.2319000097870826948565081693232059478759765625f,-1.000600017929077267808679607696831226348876953125f,-0.37759999504089325483846550923772156238555908203125f,1.0930000027716157973145527648739516735076904296875f,-0.2173000593900678012460048194043338298797607421875f,-0.65570002161264451245870077400468289852142333984375f,-1.2897999622344968884135596454143524169921875f,0.85390001342296528719089110381901264190673828125f,0.2904999892115593951302798814140260219573974609375f,-0.2836000108659266771837792475707828998565673828125f,1.2241000339031220534025123924948275089263916015625f,0.57270003359317822599905412062071263790130615234375f,1.33349999414384345897133243852294981479644775390625f,0.27329994661808054701168657629750669002532958984375f,-0.445300005835294943068447537370957434177398681640625f,-0.12069996707439401006922707892954349517822265625f,1.570600007796286945449537597596645355224609375f,0.2139999953508375796218388131819665431976318359375f,-1.12899999624490732230697176419198513031005859375f,-0.408099992224574048549357030424289405345916748046875f,-0.52299998801946667725815132143907248973846435546875f,0.40749997720122355104876987752504646778106689453125f,-0.340399978710711315077475092039094306528568267822265625f,0.0320000140666962806790252216160297393798828125f,1.04709998392462733107777239638380706310272216796875f,-0.1008000029444693357305595782236196100711822509765625f,-0.53599998676776916539665762684307992458343505859375f,0.7322000038504601349131917231716215610504150390625f,-0.313799963434040984910922134076827205717563629150390625f,0.2175000068545340781156482989899814128875732421875f,0.872999972522258804730199699406512081623077392578125f,-0.1275000058859585561066296577337197959423065185546875f,0.31950003075599653357130591757595539093017578125f,0.02550000068545354992011198191903531551361083984375f,-0.6434999696314338546443423183518461883068084716796875f,0.434499984562396690535024390555918216705322265625f,0.31489999780654898131615482270717620849609375f,0.0728999991253018730930079982499592006206512451171875f,0.479500045359134929867650498636066913604736328125f,0.56120000203251851900176916387863457202911376953125f,-0.37489998502880350628174710436724126338958740234375f,0.48850002580881124458755948580801486968994140625f,-0.16989999452829351866967044770717620849609375f,-0.296699987407028853425572378910146653652191162109375f,-0.4120000149607658812556110206060111522674560546875f,-0.901099999690055941670152606093324720859527587890625f,-0.6497999986305831310318126270431093871593475341796875f,0.05150002098083472645839719916693866252899169921875f,-0.55190002245306946093705846578814089298248291015625f,-0.622499981038272753863793695927597582340240478515625f,0.569999981522560261737453402020037174224853515625f,0.80489998782277094591108834720216691493988037109375f,0.735599962496757964203197843744419515132904052734375f,-0.54800000739097587398873656638897955417633056640625f,0.9005000187158582747315449523739516735076904296875f,-0.122799991455674051277213720823056064546108245849609375f,-0.405499975800514445012368014431558549404144287109375f,-0.266800016310810850228563140262849628925323486328125f,-0.92649996823072466867188268224708735942840576171875f,-0.19999999582767458150556194595992565155029296875f,0.93109999223947514934707214706577360630035400390625f,-0.2246000108584758070406905972049571573734283447265625f,0.672500010430812977801906527020037174224853515625f,-0.4368000064015387096105769160203635692596435546875f,0.2231999994099140849357354454696178436279296875f,-0.570899991875887025116753648035228252410888671875f,0.0702999948084350734234249102883040904998779296875f,0.73230001677274714211307582445442676544189453125f,0.68189999071359608251441386528313159942626953125f,0.3167000092744824968349348637275397777557373046875f,-1.07660001678541306091574369929730892181396484375f,-0.196899978177994317007915014983154833316802978515625f,-0.2203000226922335880175296551897190511226654052734375f,-0.94860002651214614388663903810083866119384765625f,0.10050002288818404849735088646411895751953125f,1.0163000598073015368072447017766535282135009765625f,0.55949997323751432531935279257595539093017578125f,-1.494600001955031842726384638808667659759521484375f,-0.15539996913671405565082750399596989154815673828125f,-0.29749997735023470113446819595992565155029296875f,-0.4275999640464789308680337853729724884033203125f,-0.5424000320792199403285849257372319698333740234375f,-0.16250004589557676126787555404007434844970703125f,1.16449999856948949172874563373625278472900390625f,0.642500046193600127253375831060111522674560546875f,0.334800002821534992136776054394431412220001220703125f,-1.1456999895378949805291313168709166347980499267578125f,0.291299993534385459614810542916529811918735504150390625f,-1.08000000663101669573507024324499070644378662109375f,-0.73379997159242638105070000165142118930816650390625f,0.405800001901388096570144625729881227016448974609375f,0.3745999929398298178284676396287977695465087890625f,0.34960000448822992069608517340384423732757568359375f,-0.8144000123202796981303208667668513953685760498046875f,0.37390000865459427359382971189916133880615234375f,0.841399957394600050974986515939235687255859375f,0.6084000286817552449747381615452468395233154296875f,0.421400015211105483103892765939235687255859375f,0.02620001261234250478082685731351375579833984375f,0.5286000336647038722048819181509315967559814453125f,0.3167999885201453480476629920303821563720703125f,0.8396999961018565272752312012016773223876953125f,0.0759000149726869466348944115452468395233154296875f,-0.3630000041127203758151154033839702606201171875f,-0.733799974274635502524688490666449069976806640625f,0.67380000169277121102595629054121673107147216796875f,0.387400029098987630504780099727213382720947265625f,0.084399999803304648349211447566631250083446502685546875f,-0.25449999216198904150587623007595539093017578125f,-0.3232999990701674875026583322323858737945556640625f,-0.080899999661743660084312068647705018520355224609375f,-0.96199999171495420569044654257595539093017578125f,0.32659998934864997721660984097979962825775146484375f,-0.7185999907121061180959031844395212829113006591796875f,1.088000011507421760370561969466507434844970703125f,0.0313000103354454939363904486526735126972198486328125f,0.8671999910913406051093943460728041827678680419921875f,-0.6310000386238101555136381648480892181396484375f,0.58390001237988453652860698639415204524993896484375f,0.4602000032693147613116479988093487918376922607421875f,-0.2025000318884853101053522550500929355621337890625f,-0.956699982005357885128660200280137360095977783203125f,-0.121499991625547476559177084709517657756805419921875f,0.93449999499321023677111952565610408782958984375f,-0.35309998440146461717858983320184051990509033203125f,0.4054999679028992165541467329603619873523712158203125f,-0.728999993562698733740035095252096652984619140625f,0.868800009590387389835086651146411895751953125f,-0.2159999599829320882093952604918740689754486083984375f,-0.62399998903647035941588683272129856050014495849609375f,-0.57570000438317647795116727138520218431949615478515625f,-0.11160000169798733882231545067043043673038482666015625f,0.271999988734722197847304414608515799045562744140625f,-0.040999994888901714062967585050500929355621337890625f,0.82799998235702521043322121840901672840118408203125f,0.789000002428889235517317501944489777088165283203125f,-0.16379998045861732247630015990580432116985321044921875f,-0.1601999962657689735578969703055918216705322265625f,-1.08000000536441831400225055404007434844970703125f,0.58049997252225882249376809340901672840118408203125f,-0.5457999834612012879375697593786753714084625244140625f,-1.006000027894973669617684208787977695465087890625f,0.11080000354051566802127126720733940601348876953125f,-0.424800005522369961141748717636801302433013916015625f,-0.338999993264675225645987666212022304534912109375f,0.41010000472664831505653637577779591083526611328125f,-0.390599981272220730943445232696831226348876953125f,-0.6109999894499775763279103557579219341278076171875f,-0.090400001919269445949112196103669703006744384765625f,0.2469999966323375151233676660922355949878692626953125f,-0.2504999865293502381291546043939888477325439453125f,0.4527999991297715354221509187482297420501708984375f,0.0577999971181153426869059330783784389495849609375f,-0.37820001103281963850122338044457137584686279296875f,0.5398000267565252752177684669732116162776947021484375f,0.256199992272257759395870380103588104248046875f,-0.41010000115036948642455172375775873661041259765625f,-0.4025000022351739037418383304611779749393463134765625f,-0.371899999058246777394742821343243122100830078125f,1.599499968171119945736791123636066913604736328125f,0.3047000056385993360663633211515843868255615234375f,-0.84839997324943539069863618351519107818603515625f,0.86489998767375908528265426866710186004638671875f,0.247299991440772970463513047434389591217041015625f,-0.3667999966412782697489092242904007434844970703125f,0.97160000007748603678692234097979962825775146484375f,0.0357999977290630599924270427436567842960357666015625f,-1.03329997582435595404604100622236728668212890625f,0.10660001065730995151170645840466022491455078125f,0.97380001033544516531037515960633754730224609375f,-0.1357000002853572251382274771458469331264495849609375f,0.0538000048220157456313472721376456320285797119140625f,0.068800001879036454255356147768907248973846435546875f,0.8085999853104350876265016268007457256317138671875f,1.081699960622191980519346543587744235992431640625f,-0.2794999996870757286160369403660297393798828125f,0.84039999566078193993234890513122081756591796875f,-0.2949999868869781494140625f,-0.3091999940752980791103254887275397777557373046875f,0.83320001773834206915125832892954349517822265625f,0.44359992697238936898429528810083866119384765625f,-0.760200016605853789997127023525536060333251953125f,-0.6565999938748774411578779108822345733642578125f,-1.285899948457629005105218311655335128307342529296875f,0.4967000008739528293943976677837781608104705810546875f,0.681299983662366770431617624126374721527099609375f,-0.9254999987483021328671384253539144992828369140625f,-0.010899962222575165782245676382444798946380615234375f,0.93249998271465273091962444595992565155029296875f,0.849499949097634043937432579696178436279296875f,-0.4734999974966047631141918827779591083526611328125f,0.25590000562220804880553259863518178462982177734375f,0.113399977943301255578489872277714312076568603515625f,-0.46360000989735106191602653780137188732624053955078125f,0.0381000009417533647138043306767940521240234375f,-0.70710000500976999404656453407369554042816162109375f,-0.749000016137957747019981979974545538425445556640625f,-0.02920000905096531429450124051072634756565093994140625f,-0.0925999957114458371876253295340575277805328369140625f,0.340499994352459811608468953636474907398223876953125f,-0.40469999036490922339481812741723842918872833251953125f,-0.031899998722970501763285255947266705334186553955078125f,-0.1329999865293505223462489084340631961822509765625f,1.026399978405237334300181828439235687255859375f,-0.06519998542517446704636085996753536164760589599609375f,0.299499993801116914937665569595992565155029296875f,0.70969997881650925819485564716160297393798828125f,0.4202999810621144138877980367396958172321319580078125f,0.219500005349516857933167557348497211933135986328125f,0.60039999797046184060178575236932374536991119384765625f,0.09510000021904707612208085265592671930789947509765625f,-0.512500014305114603985202847979962825775146484375f,0.5892999800264835386087725055404007434844970703125f,0.5300999768614771401331609013141132891178131103515625f,0.00999999824911357304557668612687848508358001708984375f,0.078500001464039070864231462110183201730251312255859375f,0.0753999982312321959110335001241764985024929046630859375f,-0.23849997253716015421787233208306133747100830078125f,0.321899990862608120778531883843243122100830078125f,-0.71569996517449663286924987914972007274627685546875f,-0.2155000032186507041842560283839702606201171875f,-0.42619998176693929536895666387863457202911376953125f,-0.4982999935939906865911552813486196100711822509765625f,0.377500001788138916936077293939888477325439453125f,-0.18520002837777127524532261304557323455810546875f,-0.5228999913021923351408304370124824345111846923828125f,-0.15800001411512454296826035715639591217041015625f,-0.3864999873042107569887093632132746279239654541015625f,0.593199973202497066182559137814678251743316650390625f,-0.48550003495812443787826850893907248973846435546875f,-0.6289999826848509112409146837308071553707122802734375f,-0.3380999926716088399558657329180277884006500244140625f,-0.07449998068809549067736952565610408782958984375f,0.10839999053478255319760137354023754596710205078125f,-0.7254999644756321419691857954603619873523712158203125f,0.4179999980404971449132744965027086436748504638671875f,-0.147100004823505881024203745255363173782825469970703125f,-0.107800000098347670274279153090901672840118408203125f,0.988000022590160398294756305404007434844970703125f,0.54960001685619364053536628489382565021514892578125f,1.053999994717538246646881816559471189975738525390625f,0.740599969053268747387619441724382340908050537109375f,-0.3335999978274106769760010138270445168018341064453125f,0.6608999919690192736965173025964759290218353271484375f,-0.2552999742448329723032429683371447026729583740234375f,-0.39889999244958118662651713748346082866191864013671875f,0.9930999991536140214520855806767940521240234375f,0.207199992227554474766293424181640148162841796875f,-0.99700000095367347086039444548077881336212158203125f,-0.061499993264675367754534818232059478759765625f,0.672899950361252052744021057151257991790771484375f,0.75710003294944794305365576292388141155242919921875f,1.15680002160072348260655417107045650482177734375f,-0.729299989414215588112710975110530853271484375f,0.0459000173568728797590665635652840137481689453125f,0.423599992910027367543079890310764312744140625f,0.3725999758929017247055526240728795528411865234375f,0.0526000129967929552776695345528423786163330078125f,0.3687000097066164272519017686136066913604736328125f,-0.53490001404881493130005765124224126338958740234375f,-0.4377000239074233522984513911069370806217193603515625f,-0.343500014930963715187317575328052043914794921875f,1.4693999895215039686036107013933360576629638671875f,-0.101399976468085295522314481786452233791351318359375f,1.41910003986358645988730131648480892181396484375f,-1.039300051403046154518960975110530853271484375f,-0.1744999958872792689135167165659368038177490234375f,0.916400001036375755347762606106698513031005859375f,0.526899983579665498467647921643219888210296630859375f,-0.0430999961920084384559004320180974900722503662109375f,-0.9868999975681305159014300443232059478759765625f,0.51590000722408291267129243351519107818603515625f,0.3773000069379808252278962754644453525543212890625f,-0.9764999871551989230056278756819665431976318359375f,0.1578000268459316401958858477883040904998779296875f,0.31920000182390229070961140678264200687408447265625f,-0.048800004412233644046636982238851487636566162109375f,-1.599399951970577493653991041355766355991363525390625f,-0.4432000072330233297890345056657679378986358642578125f,-0.8403999901473522271544425166212022304534912109375f,0.4134999951124189720985668827779591083526611328125f,0.5907000013470646848645628779195249080657958984375f,1.05729999740123759011112269945442676544189453125f,0.387699998426437375531605766809661872684955596923828125f,-0.3474999830126763722404348300187848508358001708984375f,0.0741999955281615886093504741438664495944976806640625f,-0.8254999733790757687756922678090631961822509765625f,0.03499999105930340359549290951690636575222015380859375f,-0.40039997194930931900813675383687950670719146728515625f,-0.3547999973267315088065743111656047403812408447265625f,0.5709999864697454796669262577779591083526611328125f,-0.422499979138374559539670372032560408115386962890625f,-0.5491999949321149099290551021113060414791107177734375f,-0.9515000141263012523040742962621152400970458984375f,0.626599966251850215570584623492322862148284912109375f,-0.2304999879449607380621500851702876389026641845703125f,-0.913500023126601945477887056767940521240234375f,-0.59849999243021034800449342583306133747100830078125f,-0.7311000075504179296359552608919329941272735595703125f,-0.3849999976903202281164340092800557613372802734375f,0.132999992787838028363012199406512081623077392578125f,-0.3841999828994275834048721662838943302631378173828125f,0.304000009432435103207126303459517657756805419921875f,-0.50500000707805148270068684723810292780399322509765625f,-0.08540000434070826873522719324682839214801788330078125f,0.37949998590350109139990308904089033603668212890625f,0.8139000083565710585986607839004136621952056884765625f,0.0225000121444463019315662677399814128875732421875f,-0.8170000253915787880032439716160297393798828125f,-0.88940002811551099881626214482821524143218994140625f,-0.8232000089839097700661341150407679378986358642578125f,-0.6254999842010438015904583153314888477325439453125f,0.0860999953687191510454113085870631039142608642578125f,-0.75029998041018852728001320429029874503612518310546875f,0.233499989748001013367684208787977695465087890625f,0.59539999566078183335093854111619293689727783203125f,0.55329998916089540017537728999741375446319580078125f,0.30399998283386242547976507921703159809112548828125f,0.38330000339150416266420506872236728668212890625f,0.6189999757930639834313524261233396828174591064453125f,0.6264999780654909589117096402333118021488189697265625f,-0.9909999671354892125663127444568090140819549560546875f,0.4020999766603114977669974905438721179962158203125f,-0.15449999886751175637300548260100185871124267578125f,0.584699981051683526089846054674126207828521728515625f,0.418299991731345632839378367862082086503505706787109375f,0.691500025749206770342425443232059478759765625f,0.7457000424742705746439241920597851276397705078125f,-0.28969999565482140724270720966160297393798828125f,-0.341500002056360330016104853712022304534912109375f,0.402100011342763963284596684388816356658935546875f,-1.2807000079035759654288995079696178436279296875f,-0.3617999903082846913093817420303821563720703125f,0.4974000007867811490314124966971576213836669921875f,-0.417400011664629033703022287227213382720947265625f,1.4032000156521799993925014859996736049652099609375f,0.082000023901463325870508924708701670169830322265625f,-0.58959998168945304541921359486877918243408203125f,-0.13819999449253117518310318700969219207763671875f,-0.34700001943111402624708716757595539093017578125f,0.63820002802014386134032974950969219207763671875f,0.335600030893086653804857633076608180999755859375f,0.2768000026762482690401157015003263950347900390625f,0.366500008016824807555167353712022304534912109375f,-0.2756000033259393688922500587068498134613037109375f,-0.851400044715405091011461991001851856708526611328125f,0.680200010645389596675158827565610408782958984375f,0.776299986195564173385719186626374721527099609375f,0.024100003206729780913519789464771747589111328125f,0.1524999999254941229764881427399814128875732421875f,-0.76420001165866846548624380375258624553680419921875f,0.2852000207781795193540119726094417273998260498046875f,-1.39220002083778382484524627216160297393798828125f,0.49430004246234915399327292107045650482177734375f,0.409300015342235923299085698090493679046630859375f,1.3443000163853167094885066035203635692596435546875f,-0.0597000679254531263495664461515843868255615234375f,-0.83870002781152752646676162839867174625396728515625f,0.0981999956846237864738213829696178436279296875f,1.241300023150444076236453838646411895751953125f,0.59210003831386615047449595294892787933349609375f,-0.4655000069439410026461700908839702606201171875f,-0.677499964684248112689601839520037174224853515625f,0.5695000103414056269457432790659368038177490234375f,0.29079999474883067023256444372236728668212890625f,-0.019900018519162898655849858187139034271240234375f,0.4185000307261947938286539283581078052520751953125f,-1.38030003936290768251637928187847137451171875f,-0.3185999949216835602783248759806156158447265625f,0.555000014603137969970703125f,-1.150000002384185648907077847979962825775146484375f,0.42119999979734412676179999834857881069183349609375f,-0.31100001254677778206314542330801486968994140625f,0.01159999173283576823223484097979962825775146484375f,0.1843999933585525052848197447019629180431365966796875f,0.298499994814395819275887333787977695465087890625f,0.22709999286532411844063972239382565021514892578125f,0.891899987733363985853429767303168773651123046875f,0.19399998745322211135544421267695724964141845703125f,0.8554000172674653956761403605923987925052642822265625f,0.0488000062376261478647165859001688659191131591796875f,0.892999986529350309183428180404007434844970703125f,-0.0758000002622605251190179842524230480194091796875f,0.5281999740034344359429496762459166347980499267578125f,-0.837999998599290307765841134823858737945556640625f,0.1474000067472456265704749966971576213836669921875f,-0.5824000092059369304564597769058309495449066162109375f,0.84700002554059050652313089813105762004852294921875f,-0.5070999982297419439447594413650222122669219970703125f,-0.04000000465661290860452936613000929355621337890625f,-0.5399999898672103171293201739899814128875732421875f,0.28769999291300774046931110206060111522674560546875f,-0.63439998862743374186123901381506584584712982177734375f,-0.483000022068619916382203882676549255847930908203125f,-0.33930000617802147377943811079603619873523712158203125f,-0.0420000016614794358105200444697402417659759521484375f,0.491999989032745332906415569595992565155029296875f,0.674100020343065153838324476964771747589111328125f,0.8463999921143054994132626234204508364200592041015625f,0.10450001883506754296604412957094609737396240234375f,-0.5794000269234178546184921287931501865386962890625f,-0.7940999856233599274446532945148646831512451171875f,-0.85650001025199884452376863919198513031005859375f,0.841500013530254165061705862171947956085205078125f,-0.408899988798796965028259364771656692028045654296875f,-0.398500003904104005414410494267940521240234375f,-0.075799991023540513879197533242404460906982421875f,0.334499992460012496309218477108515799045562744140625f,-0.764500015705824154110814561136066913604736328125f,0.074499992311000795552899944595992565155029296875f,-0.2980999887600541686794031193130649626255035400390625f,0.2965000007152553962441743351519107818603515625f,0.289099999779462990545653156004846096038818359375f,0.9451000081539149277887190692126750946044921875f,0.230999991834163864723450387828052043914794921875f,0.049999996870756291400539339520037174224853515625f,0.64060002955198314111839863471686840057373046875f,1.5119000115752214696840383112430572509765625f,-0.462700016903877298091174452565610408782958984375f,-0.55260003698766269053521682508289813995361328125f,-1.2420000233799211031282538897357881069183349609375f,0.37680002852976368643567184335552155971527099609375f,0.43250000476837158203125f,-0.254900006449222615856342599727213382720947265625f,0.39890002623796494418684233096428215503692626953125f,1.18969999043941498939602752216160297393798828125f,-0.443600009226798874806263484060764312744140625f,-0.1770000075101858527659715036861598491668701171875f,0.1994000184297561872881487943232059478759765625f,1.17009997954368571981831337325274944305419921875f,0.6347000151753423580203161691315472126007080078125f,-0.31099999531731015167679288424551486968994140625f,0.555499989863485144070409660344012081623077392578125f,-0.4505000119172033468117888332926668226718902587890625f,-0.113900010442733901072642765939235687255859375f,-1.298500013589858781415387056767940521240234375f,-0.50830001456737505804994725622236728668212890625f,-0.4952000285267832424551670555956661701202392578125f,-0.7249000152409070807379976031370460987091064453125f,-0.05450000289082534976614624611102044582366943359375f,-0.068700009036064102474483661353588104248046875f,-0.4805999887228011857587262056767940521240234375f,-0.69580004347562862676568329334259033203125f,0.06240000734329242959574912674725055694580078125f,-1.7660999692916874437287333421409130096435546875f,-0.0221000067234040642460968228988349437713623046875f,0.39459998880475755100860624224878847599029541015625f,-0.802599997052550140352877861005254089832305908203125f,-0.01219998351037465678103899335837922990322113037109375f,-0.2841999945595858889646478928625583648681640625f,0.69040002356320595566785414121113717555999755859375f,-0.393800032277405787084489929839037358760833740234375f,-0.56329998446702944647768163122236728668212890625f,-0.336699983942509106782381422817707061767578125f,0.430100003385543772083110525272786617279052734375f,0.02599999004602437935318448580801486968994140625f,0.372400021052360585827045724727213382720947265625f,0.82860002859830839128107982105575501918792724609375f,-0.8419000129163265455645159818232059478759765625f,0.70299999293684944934312852637958712875843048095703125f,0.124000011742115390234175720252096652984619140625f,0.5213000078022478334105471731163561344146728515625f,-0.6272000045955177238710120946052484214305877685546875f,-0.078000019982457757805605069734156131744384765625f,-0.42420000629425036464681397774256765842437744140625f,0.5352000118374820036848404924967326223850250244140625f,-0.8425000178813935036714610760100185871124267578125f,-0.084800004255771721517476180451922118663787841796875f,-0.3574999970197676457672741889837197959423065185546875f,-0.64699997174739820593458716757595539093017578125f,-0.52869999979734405570752642233856022357940673828125f,-0.7563999766543509917227083860780112445354461669921875f,-0.986999993801116914937665569595992565155029296875f,0.79829997716546063912801400874741375446319580078125f,-0.479799991366267164494274766184389591217041015625f,0.8184999958388505802986401249654591083526611328125f,0.2033000001132487799537784667336381971836090087890625f,0.228499993454664984682267458993010222911834716796875f,0.758499992132186662274762056767940521240234375f,-0.840399999535083708224192378111183643341064453125f,0.41229999345242962505153627716936171054840087890625f,0.44250000581145254585635484545491635799407958984375f,-0.3069000036776063922161483787931501865386962890625f,0.496900009526311858820690758875571191310882568359375f,0.966500031709671247881487943232059478759765625f,-0.478099991255998446604280616156756877899169921875f,0.27989999870061854636560383369214832782745361328125f,-0.00750001549720735738446819595992565155029296875f,-0.95849998825788507161860252381302416324615478515625f,0.2596999776616695232434040008229203522205352783203125f,-0.2720000149980188464127195402397774159908294677734375f,-0.50799999036639942318061002879403531551361083984375f,-0.14919999590069055184216040288447402417659759521484375f,0.7235000258460644317892729304730892181396484375f,0.289000009804964097526180921704508364200592041015625f,0.173899993827939081114664077176712453365325927734375f,-0.145000014603137827862155972979962825775146484375f,0.0771999953567981123114805086515843868255615234375f,0.3775999798044564759180730106891132891178131103515625f,-1.1084999954104421959755200077779591083526611328125f,0.474000003844499584459981633699499070644378662109375f,-0.258499986730515995958512576180510222911834716796875f,-0.070500014692544965555498492904007434844970703125f,0.967900029933452810126937038148753345012664794921875f,0.3316999957516788555267339688725769519805908203125f,0.10539998310655374069710887852124869823455810546875f,0.066000009156763628226372020435519516468048095703125f,-0.706599996799230467559027601964771747589111328125f,-0.53059997426867511194359394721686840057373046875f,0.4283999996244904906461670179851353168487548828125f,-1.46010000815391549622290767729282379150390625f,-0.788699992942810013119014911353588104248046875f,1.09250001639127702901532757095992565155029296875f,0.8845000280737878028958220966160297393798828125f,0.281899973726272179419538588263094425201416015625f,-1.00190004882812555564441936439834535121917724609375f,-1.353600005650520188282825984060764312744140625f,-0.58679993934631369256749167107045650482177734375f,0.7716000327110297263288885005749762058258056640625f,-0.12740003476142902627543662674725055694580078125f,-0.62679999895095761530683375895023345947265625f,-0.5740000096559523257155888131819665431976318359375f,0.4572999887585638134623877704143524169921875f,1.4023999659180645949163590557873249053955078125f,-0.3568000078916551132124368450604379177093505859375f,0.5082000310003760290555874234996736049652099609375f,1.1886000163793557504732234519906342029571533203125f,0.25480000574588768103012625942938029766082763671875f,0.0983000252962114018373540602624416351318359375f,1.43109999909400897877276293002068996429443359375f,-0.0745000161528590609805178246460855007171630859375f,0.00019999149739735599951018230058252811431884765625f,-0.34790000460147840755098513909615576267242431640625f,-0.150700003880262389799327138462103903293609619140625f,0.2536999871313572185727025498636066913604736328125f,-0.8083999798059462449373313575051724910736083984375f,-0.1901999921679493610326971975155174732208251953125f,0.12589999917745586799355805851519107818603515625f,-0.096200005161762192074093036353588104248046875f,0.48300002348423021203416283242404460906982421875f,-0.42570001207590113523338004597462713718414306640625f,-0.43440000226199604327348424703814089298248291015625f,-0.018000001952052091525047217146493494510650634765625f,-1.5631000234425069805865859962068498134613037109375f,-0.2197999938249584062077701673842966556549072265625f,0.80760001500844946775714561226777732372283935546875f,-0.1369000030629337061327532865107059478759765625f,0.29489999441653491629722338984720408916473388671875f,-0.33299999893456710964301237254403531551361083984375f,0.013499994069337883928483279305510222911834716796875f,-0.274399988776445502569600876086042262613773345947265625f,-0.0355000054538248122071308898739516735076904296875f,0.00160000126957893229473484097979962825775146484375f,0.51569998409897099822529753510025329887866973876953125f,-0.3470000116825104896634002216160297393798828125f,0.237199982243776474888363736681640148162841796875f,0.9691999876305459338254877366125583648681640625f,0.6575000265240671382116488530300557613372802734375f,-0.363200010883808044326315211947076022624969482421875f,-0.4815000039935111164624004231882281601428985595703125f,-0.006000042796134863465340458787977695465087890625f,0.21480002362728090048449303139932453632354736328125f,0.925199971753359218240575501113198697566986083984375f,0.851000008225440751630230806767940521240234375f,-0.50299998325109473995553344138897955417633056640625f,0.3218000117659565173511282409890554845333099365234375f,0.121500029623508254417174612171947956085205078125f,0.53489999169707314052857327624224126338958740234375f,-0.497099971184134847845825788681395351886749267578125f,-0.71749999880790671369368283194489777088165283203125f,0.6188000005006788484251956106163561344146728515625f,-0.6279000053092833599777122799423523247241973876953125f,0.017000017046928661557103623636066913604736328125f,0.71619999219775198895376888685859739780426025390625f,-0.29960000318437796096304737147875130176544189453125f,0.199500002145767130201647887588478624820709228515625f,-0.236099998870491933899984360323287546634674072265625f,0.0911000028565524810897358065631124190986156463623046875f,0.06599998825788500056432894780300557613372802734375f,-0.543899979895353435921379059436731040477752685546875f,-0.007400000749528345966865572336246259510517120361328125f,-0.726500011146068658263175166212022304534912109375f,0.0880999942362308274823590181767940521240234375f,-0.1782999950468540095727121297386474907398223876953125f,0.341499998107552382720086825429461896419525146484375f,-0.153700000169873174371559798601083457469940185546875f,0.38889999904334526803495464264415204524993896484375f,0.09999999523162816927879248396493494510650634765625f,0.46309999125599876634851170820184051990509033203125f,1.0081999978080389279710971095482818782329559326171875f,0.2860000061951577521313083707354962825775146484375f,-0.05750000476837158203125f,1.1872000223279002284471062012016773223876953125f,-0.60159997698068679028438054956495761871337890625f,0.024000010550022210509268916212022304534912109375f,-0.023100011372566342515710857696831226348876953125f,0.569099993371963819299708120524883270263671875f,0.43670003103017851486811196082271635532379150390625f,-1.02889998749494537833015783689916133880615234375f,-1.05519996176958130718048778362572193145751953125f,0.8554000311255460786696858122013509273529052734375f,-0.1734000429868700621227617375552654266357421875f,-2.21579999608993460924466489814221858978271484375f,0.3180000297427181266129991854541003704071044921875f,-0.59620003466606164010954671539366245269775390625f,-1.58030000598430575564634636975824832916259765625f,0.123899997627734848038016934879124164581298828125f,-0.246399981087446207084212801419198513031005859375f,0.3036999807238576210011160583235323429107666015625f,0.661500007867812822581754517159424722194671630859375f,-0.8956000027298927079755230806767940521240234375f,0.868900016999244684257064363919198513031005859375f,0.28689999756813033826574610429815948009490966796875f,0.4606000040709972154218121431767940521240234375f,0.69349998304247861824478604830801486968994140625f,-0.49630001555085190290128593915142118930816650390625f,0.49639999986290916922371252439916133880615234375f,0.966099964970350555404365877620875835418701171875f,-0.5573000070869920108407313819043338298797607421875f,-0.195099994146823885188268832280300557613372802734375f,0.2874999941140412573759022052399814128875732421875f,0.18329999936819063588444578272174112498760223388671875f,0.9043999998033047660328520578332245349884033203125f,-0.6946999874591828216807698481716215610504150390625f,-0.41149999445676765930102192214690148830413818359375f,-0.72919999735355389702817774377763271331787109375f,1.2382999811887742680482915602624416351318359375f,-0.1257000087976454238969381549395620822906494140625f,-0.906300011080503509219852276146411895751953125f,-0.239699987608194220456425682641565799713134765625f,-0.368900024300813811350963078439235687255859375f,-1.2765999729573724863485040259547531604766845703125f,-0.1132999788045889744125815923325717449188232421875f,0.67809999527931186236173743964172899723052978515625f,1.251900038993358776906461571343243122100830078125f,0.68979999199956643085585028529749251902103424072265625f,-0.90800001364946325566052109934389591217041015625f,-0.543999987602234114092425443232059478759765625f,-0.770400007581710344339853691053576767444610595703125f,-0.559000010550022352617816068232059478759765625f,0.455199999469518612471574670053087174892425537109375f,-0.17279998657554396235269678072654642164707183837890625f,-0.35400004988908762015853426419198513031005859375f,-0.06929999418258692145400345907546579837799072265625f,0.7497999654755000964456712608807720243930816650390625f,-0.276500022768974673681441345252096652984619140625f,-1.03599998900294298920243818429298698902130126953125f,0.43630000895708764829805659246630966663360595703125f,0.288000005006789905337427626363933086395263671875f,-0.68130000243782973257111734710633754730224609375f,0.6784000066280360119463921364513225853443145751953125f,-0.68700000140070915932710704510100185871124267578125f,0.4340999731063843736222906954935751855373382568359375f,-0.0122000039994716313884737246553413569927215576171875f,0.2825000080466271157320079510100185871124267578125f,0.734500001251697565152198876603506505489349365234375f,0.4456999961689114275742440440808422863483428955078125f,0.3060000155270099497784030972979962825775146484375f,0.110099997872114219177319682785309851169586181640625f,-0.3491999912440777453781493022688664495944976806640625f,0.358000009030103427676294813863933086395263671875f,0.2272999936759472205949350609444081783294677734375f,-0.4087999850779773236553182869101874530315399169921875f,-0.097499997355043888092041015625f,0.253199991139769597925379684966173954308032989501953125f,0.05469999867230652956795466934636351652443408966064453125f,-0.12650001263618460001225685118697583675384521484375f,-0.6858999808490278127237615990452468395233154296875f,-0.455300000061094556347285333686158992350101470947265625f,0.698500004649162065106793306767940521240234375f,0.75829998923540120614461557124741375446319580078125f,0.3488999991551040569248698375304229557514190673828125f,1.157000006765127153585126507095992565155029296875f,-0.07519999261498444553808440105058252811431884765625f,0.781599989087879531979297098587267100811004638671875f,0.05699998545646689507293558563105762004852294921875f,0.41969998626708981959154698415659368038177490234375f,0.808199966105819012085476060747168958187103271484375f,-0.5240000126361845644851200631819665431976318359375f,-0.5153000062823291926861202227883040904998779296875f,0.51070000946819771314721947419457137584686279296875f,1.39399995660781872430789007921703159809112548828125f,0.166400002247094835894358766381628811359405517578125f,-0.9443000010371207508796942420303821563720703125f,-0.7401999719023706347797997295856475830078125f,0.4496000052332878027527840458787977695465087890625f,-0.6968999984622001875322894193232059478759765625f,0.71969999908208848182766814716160297393798828125f,-0.211500021576881636065081693232059478759765625f,0.079700001429021227750126854516565799713134765625f,0.81159997269660255625467470963485538959503173828125f,-0.35520000427514286656105468864552676677703857421875f,0.0022999921858311012101694359444081783294677734375f,1.2235000195503236142258174368180334568023681640625f,0.49010003229379695000034189433790743350982666015625f,-0.453000026166439084818193805404007434844970703125f,-1.667699963259696716022517648525536060333251953125f,-0.007299982500075685720730689354240894317626953125f,1.23550003600120561486619408242404460906982421875f,0.84349997842311807971782400272786617279052734375f,-0.715500021696090726663896930404007434844970703125f,0.1817999980568885121101629920303821563720703125f,0.584699989843368683750668424181640148162841796875f,0.32090002421140706445612522657029330730438232421875f,0.084100042545795616888426593504846096038818359375f,1.832899984931945169819300645031034946441650390625f,0.0552999866127965589157611248083412647247314453125f,-0.4551000074088573654762512887828052043914794921875f,-0.3857999916195868905788302072323858737945556640625f,0.30860000296831113786311107105575501918792724609375f,0.1243999893724918592852191068232059478759765625f,-0.1897999848842619741162707214243710041046142578125f,-0.441400009846687257919484181911684572696685791015625f,0.50360001876354232308585778810083866119384765625f,-1.91150000190734914440326974727213382720947265625f,0.051699984836577783653410733677446842193603515625f,0.564600009107589784207448246888816356658935546875f,0.06389999136924728873054846189916133880615234375f,-0.46970002202987704009728986420668661594390869140625f,-0.69040001205205925316477078013122081756591796875f,0.188500010013580521217591012828052043914794921875f,0.98670003937482864131425230880267918109893798828125f,-1.1854999871253966148287872783839702606201171875f,0.54999999247491349052552322973497211933135986328125f,-0.604499978899955880962124865618534386157989501953125f,0.697499973997473876607955389772541821002960205078125f,-0.64589997831583045861947312005213461816310882568359375f,0.113999984323978509337393916212022304534912109375f,-0.05609999581575397797905679908581078052520751953125f,0.720599980415403962297204998321831226348876953125f,0.790000014007091522216796875f,0.4471999962508679260508870356716215610504150390625f,0.431799994815885934684729363652877509593963623046875f,0.40250000197440360949485693708993494510650634765625f,0.2952999928712845001399500688421539962291717529296875f,-0.120899993105232805845616894657723605632781982421875f,-0.65150000548362729801965542719699442386627197265625f,0.81620001246333107047803423483856022357940673828125f,-0.3140999875605106073095384999760426580905914306640625f,0.3544999693632124859732357435859739780426025390625f,-0.80029997141361253198965641786344349384307861328125f,0.61649998860806232148235039858263917267322540283203125f,0.018500008225441177955872262828052043914794921875f,-0.036200006353855229690452688373625278472900390625f,-0.8945000019967552784549980060546658933162689208984375f,-0.5590000087618829383018237422220408916473388671875f,0.384099996501207296972779658972285687923431396484375f,-0.3005999896913766900041764529305510222911834716796875f,-0.1449999904632566227746792719699442386627197265625f,0.9765999795138835537500199279747903347015380859375f,-0.38520000911802021459351408338989131152629852294921875f,-0.06549999994039534811918201739899814128875732421875f,-0.01130000139474862663746534963138401508331298828125f,0.332699997532367619523796520297764800488948822021484375f,0.48800000485777861314318215590901672840118408203125f,-0.2749999828636646714841162975062616169452667236328125f,0.03520000125765798681953810955747030675411224365234375f,-0.430500003218650562075708876363933086395263671875f,-1.09710001178979865699147921986877918243408203125f,-0.0537000162258742808063516349648125469684600830078125f,0.197500016391277455340969027020037174224853515625f,-0.235900001859664865122567789512686431407928466796875f,-0.32369999782294034407215121973422355949878692626953125f,-0.7840000177025796546104174922220408916473388671875f,0.570099996978044476492186731775291264057159423828125f,0.26280000367462630350701147108338773250579833984375f,-0.262000044465064974019696819595992565155029296875f,-0.02669999910444020851940649663447402417659759521484375f,0.4727000332951547534321434795856475830078125f,0.17589999679326950854374445043504238128662109375f,0.4445000155568124000637908466160297393798828125f,0.0621000208795070562928231083787977695465087890625f,1.8542999733209608592687800410203635692596435546875f,0.28790000579357144516734479111619293689727783203125f,-0.6533999748885630509676047950051724910736083984375f,0.0524999970197672638505537179298698902130126953125f,0.5166999902009958844928405596874654293060302734375f,-0.025799990278482454186814720742404460906982421875f,0.2816999866247176242950445157475769519805908203125f,0.236700006592273748395882648765109479427337646484375f,0.07030000188648699577242950908839702606201171875f,-0.19060000541210175839523799368180334568023681640625f,-0.1498000054478645370892309074406512081623077392578125f,0.6028999884150920696157527345349080860614776611328125f,0.4911999714106325409801456771674565970897674560546875f,-0.15739999575763941042083615684532560408115386962890625f,-0.1355999973654746781903668306767940521240234375f,-0.26729999829530726174198207445442676544189453125f,0.517100010001659171621213317848742008209228515625f,-0.64760001560449609314673580229282379150390625f,0.750500010371208503556772484444081783294677734375f,0.23230001558065449529522084048949182033538818359375f,-1.0259000383675100920299883000552654266357421875f,-0.878499997347592653795800288207828998565673828125f,0.3507000065624712448197897174395620822906494140625f,0.6008000063717364724880098947323858737945556640625f,-1.2020000289678574034724078956060111522674560546875f,-0.5054000224828723020209508831612765789031982421875f,0.115699982869625017656289855949580669403076171875f,1.234000003695488345556441345252096652984619140625f,0.6154000332117082194827162311412394046783447265625f,-0.7754999935701487601136250304989516735076904296875f,-0.97799995907396120031762620783410966396331787109375f,0.299999997578561217181913889362476766109466552734375f,0.01430005319118521356358542107045650482177734375f,2.0184999831914893775319796986877918243408203125f,-0.1549000216484071046352255507372319698333740234375f,-0.833599994772672658882584073580801486968994140625f,1.2674000076413154403098815237171947956085205078125f,-0.019599998974799692774695358821190893650054931640625f,0.419600022537261363453353624208830296993255615234375f,-0.265299997192621150876590263578691519796848297119140625f,-0.93450001794099790686232154257595539093017578125f,0.1517999876260756053625300410203635692596435546875f,-0.46990001144111115394252919941209256649017333984375f,-0.43399999117851262298017900320701301097869873046875f,1.0711999786376953380795384873636066913604736328125f,-0.4221999921157955082406942892703227698802947998046875f,-0.19399999862909300674118640017695724964141845703125f,0.331099977040290927021715106093324720859527587890625f,0.358799981725216043937365384408622048795223236083984375f,-0.29300000455975538926622903090901672840118408203125f,0.2350999878883361926540374042815528810024261474609375f,-0.0794000009208917223446633215644396841526031494140625f,0.145499999567866300509422217146493494510650634765625f,-0.16590000163614748718288183226832188665866851806640625f,0.012899998119473454760708364119636826217174530029296875f,-0.24299998585879822865507549067842774093151092529296875f,0.08969999362453828783969811411225236952304840087890625f,-0.490799976271391213966666100532165728509426116943359375f,-0.57650000160932535209212801419198513031005859375f,0.059899991250038198131733224727213382720947265625f,0.034699999044835561079480612534098327159881591796875f,0.1709999954104421959755200077779591083526611328125f,0.604500007957220031329370613093487918376922607421875f,0.2962000051617621476651720513473264873027801513671875f,0.30500000372529001424481975845992565155029296875f,-0.1471999995291231044802771066315472126007080078125f,0.824800007869302742591344212996773421764373779296875f,0.22450000482797616285779440659098327159881591796875f,-0.928000012457370626606234509381465613842010498046875f,-0.127100002141296963831251787269138731062412261962890625f,-0.8999999928474426980073985760100185871124267578125f,0.8421000034451484594910652958787977695465087890625f,-0.42539997446388022606100776101811788976192474365234375f,0.18649998391419675414226730936206877231597900390625f,-0.266299989473819831431455895653925836086273193359375f,0.6694999730512503077761721215210855007171630859375f,0.181500005483627546709612943232059478759765625f,1.01759997864961615476886436226777732372283935546875f,-0.3211000176459546384677423702669329941272735595703125f,0.63649999192357054056401466368697583675384521484375f,0.8302000034928320726379524785443209111690521240234375f,0.397699990789592394690998844453133642673492431640625f,0.86450003613904147226065788345295004546642303466796875f,-0.6573000215411184399272315204143524169921875f,0.03160000708103183342245756648480892181396484375f,1.015399996554851469454661128111183643341064453125f,-0.26220001338720333450282851117663085460662841796875f,0.8352999943986532116468879394233226776123046875f,1.56139993709176838621033311937935650348663330078125f,-0.614800000344216446279688170761801302433013916015625f,-0.701399979783594584858974485541693866252899169921875f,0.706799987030028997736508244997821748256683349609375f,0.5623999960929151331612274589133448898792266845703125f,0.545400019800663216074099182151257991790771484375f,1.087200017857551159750073566101491451263427734375f,-0.039399996078014254408117267303168773651123046875f,-0.4030999918520450364667340181767940521240234375f,0.2504000016212462043085906771011650562286376953125f,-0.038600005650520241573531166068278253078460693359375f,0.44019999753236760398067417554557323455810546875f,0.80869997357130074533415609039366245269775390625f,-0.4563000136137009832282274146564304828643798828125f,1.47490003118515033975199912674725055694580078125f,-0.4975000166893011055435636080801486968994140625f,-0.1317000062942508264995922218076884746551513671875f,0.14089999977350231574746430851519107818603515625f,-0.021200002181529953304561786353588104248046875f,0.696000039815903193129997816868126392364501953125f,-0.533700021906942101423965141293592751026153564453125f,0.8982000393077733679803031918709166347980499267578125f,0.335400020210445182211600467780954204499721527099609375f,0.002700004349649010038092455943115055561065673828125f,-0.575699993300438084276038352982141077518463134765625f,-0.1039000067919493286439092116779647767543792724609375f,-1.352200040209293518955746549181640148162841796875f,-0.046999966979026197577695711515843868255615234375f,0.38060001673698451440941425971686840057373046875f,-1.0292999944806098255867254920303821563720703125f,0.002700016307830566120173898525536060333251953125f,0.85270000408887813847513825749047100543975830078125f,0.618300013822317140466111595742404460906982421875f,-1.5477000231623652126700108055956661701202392578125f,-0.121700013446808252837172403815202414989471435546875f,-0.158199999912828292991662237909622490406036376953125f,0.6858999885603787038945711174164898693561553955078125f,-0.228299997468292270585976666552596725523471832275390625f,0.4323000126004217236186377704143524169921875f,0.3444000234961510642506254953332245349884033203125f,-0.0231000063247976950009388019680045545101165771484375f,0.87719996652305154416495724944979883730411529541015625f,0.2532000221714370002956684402306564152240753173828125f,0.3325999799907208842597583497990854084491729736328125f,0.364899990020692399639301584102213382720947265625f,0.716900009340047716932531329803168773651123046875f,0.151399993865191928588131986543885432183742523193359375f,0.3751999821469189644318475984618999063968658447265625f,0.56800000072270595552481609047390520572662353515625f,-0.65629998440742465248831649660132825374603271484375f,0.5263999776229264515592376483255065977573394775390625f,0.0064000000119204258908212068490684032440185546875f,0.24919996397495314255365883582271635532379150390625f,0.610500005193054473551228511496447026729583740234375f,1.364100021088123781964895897544920444488525390625f,-0.35949997472763062233980235760100185871124267578125f,-0.903400005249678894614362434367649257183074951171875f,-0.506600034201145632550833397544920444488525390625f,-0.374899992591142261488812437164597213268280029296875f,-0.1055999959871172233505376425455324351787567138671875f,-0.6336000233829022221243576495908200740814208984375f,-0.09050000038743011288033812888897955417633056640625f,0.6303999796047807802068518867599777877330780029296875f,0.058400013184547017175418659462593495845794677734375f,0.70559999445974741494325144230970181524753570556640625f,-0.4558999668791894332997571837040595710277557373046875f,-0.15429997488558200302577461116015911102294921875f,0.15599998974800133311191530083306133747100830078125f,0.445499984778464030998890166301862336695194244384765625f,-0.083700002181530397393771636416204273700714111328125f,-0.5889999838396906550741505270707421004772186279296875f,0.31749999038875109658164319625939242541790008544921875f,0.1307000030979512672502096393145620822906494140625f,0.522299994271993028860379126854240894317626953125f,-0.524199990163743745341662361170165240764617919921875f,0.521800027263164878377210698090493679046630859375f,-0.01460001037418834357595187611877918243408203125f,0.87859998568296437948532684458768926560878753662109375f,0.6226000258862971481477188717690296471118927001953125f,0.55149998838454450389434668977628462016582489013671875f,0.061699999849498254889113013632595539093017578125f,0.34090001009255654906837662565521895885467529296875f,-0.3146999958038332323440045001916587352752685546875f,-0.4976000099420547773121370482840575277805328369140625f,-0.389899989461898854870014474727213382720947265625f,0.0988000344753263703978518606163561344146728515625f,0.668599990525841736399570436333306133747100830078125f,0.8659000233173372862438554875552654266357421875f,0.560400011754035975997112473123706877231597900390625f,-0.4155000095888972122537552422727458178997039794921875f,0.6065999748945234415486993384547531604766845703125f,0.2721999710679057216111687012016773223876953125f,-0.5640999810039997175437065379810519516468048095703125f,-0.32719999446272840515348434564657509326934814453125f,-0.416000010907649819813514113775454461574554443359375f,-0.116800001782178863862782236537896096706390380859375f,1.33310002746582068056113712373189628124237060546875f,0.98240000972747765217718551866710186004638671875f,-0.59360002166926861821849570333142764866352081298828125f,0.533800004971027419742313213646411895751953125f,0.2177999919772144465923702227883040904998779296875f,-0.3807000049978495059832539482158608734607696533203125f,-1.142599997685849633199950403650291264057159423828125f,-0.6211999967053538629357944955700077116489410400390625f,0.686299980663508168987618773826397955417633056640625f,-0.35649998879432676091028042719699442386627197265625f,-0.2629999895095824768986858543939888477325439453125f,0.19399999576061965900208861057762987911701202392578125f,-0.41340002927780172825578119955025613307952880859375f,-0.03679999418258628196554127498529851436614990234375f,0.53600002118945122564497296480112709105014801025390625f,0.6124000064492225448020690237171947956085205078125f,0.4264000035881991834685322828590869903564453125f,-0.2353000067666173311664579159696586430072784423828125f,-0.817000006914138765523603069595992565155029296875f,-0.538399993515014330114354379475116729736328125f,0.518000008434057246375914473901502788066864013671875f,0.92639999583959564688484533689916133880615234375f,0.7370000069141386944693294935859739780426025390625f,-0.277999994240701209236021895776502788066864013671875f,0.649700036633014565978783139144070446491241455078125f,0.09580000473260863458335734321735799312591552734375f,-0.7156000125274062373392780500580556690692901611328125f,0.5460000138878822184551609097979962825775146484375f,0.268200009095668878700280401972122490406036376953125f,-0.3029999995306134064065872735227458178997039794921875f,0.6352000329971314585009167785756289958953857421875f,0.94470005213022290035951300524175167083740234375f,-0.577199993866681637655347003601491451263427734375f,0.2554999892488123425238200070452876389026641845703125f,0.7182999838709829276695018052123486995697021484375f,-0.20499999701976800992042626603506505489349365234375f,0.6944000097870823839940612742793746292591094970703125f,0.214000031411648361512334304279647767543792724609375f,0.05290000638961789292125104111619293689727783203125f,-0.4265999933347106587433472668635658919811248779296875f,-1.2377999985337257982109804288484156131744384765625f,0.4687999896228312035617591391201131045818328857421875f,-0.5511999781906606887815769368899054825305938720703125f,-0.4904000002801414126452073105610907077789306640625f,-0.57979997102618252569072865298949182033538818359375f,0.4777000075533985867792807766818441450595855712890625f,-0.6042999995470044183321078890003263950347900390625f,0.132499999999999840127884453977458178997039794921875f,0.55759997891038681760988993119099177420139312744140625f,0.35440000054836229281818305025808513164520263671875f,0.0376000131271780890784839357365854084491729736328125f,-0.8857000109583135216695382041507400572299957275390625f,-0.256100071103871673727780944318510591983795166015625f,0.166499986857175930055063872714526951313018798828125f,0.10900000216811889242052302506635896861553192138671875f,0.52700000482797637602061513462103903293609619140625f,-0.3355999985575675026439057546667754650115966796875f,0.18299999241530884308559734563459642231464385986328125f,0.65100002640485765681432894780300557613372802734375f,-0.19699999380111687941052878159098327159881591796875f,0.6550999730244282392987997809541411697864532470703125f,0.3990999906897538807015735073946416378021240234375f,0.2543999829649923327679061912931501865386962890625f,0.255400003036856393379139262833632528781890869140625f,1.205000038743019530329547706060111522674560546875f,-0.48809999200105647076952664065174758434295654296875f,0.819299976077676017638395933317951858043670654296875f,0.322500009536742737736858543939888477325439453125f,0.477899970626831649411769831203855574131011962890625f,0.270999995149672212590985509450547397136688232421875f,-0.16780002222657230959157459437847137451171875f,0.056299984854459861338682458153925836086273193359375f,0.8042999926924705444974961210391484200954437255859375f,0.715500027656555293020801400416530668735504150390625f,0.210499971926212392503430237411521375179290771484375f,-0.28329997973591092286227421936928294599056243896484375f,0.57139999434947963408149007591418921947479248046875f,0.16159998443126699640970400650985538959503173828125f,-0.52339998897016049017594241377082653343677520751953125f,0.6763000016927716018244609585963189601898193359375f,0.5802000148177146598982290015555918216705322265625f,-0.1652999948829412257822468745871447026729583740234375f,-0.214499992907047243306806194595992565155029296875f,-0.26879999275207477893445684458129107952117919921875f,0.0528000066548586222126004940946586430072784423828125f,-0.32470001100301715268869884312152862548828125f,0.0457999990701676296112054842524230480194091796875f,0.37479999646991490891423381981439888477325439453125f,-0.6224000228405000001430380507372319698333740234375f,-0.1455999970674515253676872816868126392364501953125f,0.6130000065714120704996048516477458178997039794921875f,0.2792000334143640571937794447876513004302978515625f,-0.029400001144409326769846302340738475322723388671875f,-0.488700020584464045469985649106092751026153564453125f,-0.05229999235346916020006347025628201663494110107421875f,-0.235200002934038698843011161443428136408329010009765625f,-0.142800000713020569520494973403401672840118408203125f,0.2448999954223634034633505507372319698333740234375f,-0.011400014913082401335486792959272861480712890625f,-0.2414999998956919302628421064582653343677520751953125f,-0.8653000259518623948906679288484156131744384765625f,-0.6084000230193140623669023625552654266357421875f,0.3537000043049454145460686049773357808589935302734375f,0.2802999881029126783005267498083412647247314453125f,0.197399993932247497241405653767287731170654296875f,-0.2048999869287013719798551392159424722194671630859375f,0.64830000026226031195619725622236728668212890625f,0.16239998350143469707518306677229702472686767578125f,-0.701199989292025538389907524106092751026153564453125f,0.2303000053882602315979966078884899616241455078125f,0.31699999856948846144177878159098327159881591796875f,-0.050000012852251529693603515625f,-0.05079999638795840155580663122236728668212890625f,0.17600001746416094050573519780300557613372802734375f,0.325299996633827692704699074965901672840118408203125f,0.151499985665083158892230130732059478759765625f,0.284599993908404957920765809831209480762481689453125f,0.15519999652653926336398626517620868980884552001953125f,0.61390001536011684635241181240417063236236572265625f,-0.702399982011318257946186349727213382720947265625f,0.272799989593028868739565950818359851837158203125f,0.36279999883174884889314171232399530708789825439453125f,0.03080002917051327671060789725743234157562255859375f,-0.44290000206828050721696854452602565288543701171875f,0.532099977964163084465099018416367471218109130859375f,0.126099988365172777093903278000652790069580078125f,0.337799974542856151771275108330883085727691650390625f,0.4701000090852376356309605398564599454402923583984375f,0.9609000459671026561636608676053583621978759765625f,-0.45149996748566678661518380977213382720947265625f,-0.7243999721989038409475369917345233261585235595703125f,-0.8416000008225434925179797573946416378021240234375f,-0.35479998488426200964340750942938029766082763671875f,-0.3382000091701744537431295611895620822906494140625f,0.0489999834299084113808930851519107818603515625f,0.31409999560713774968689904198981821537017822265625f,-0.55939997488111270484978376771323382854461669921875f,-1.102600029611587206801459615235216915607452392578125f,0.42939998059943318420295099713257513940334320068359375f,0.352299989969283411372913406012230552732944488525390625f,0.631100008761137587498524226248264312744140625f,0.299299998205899697722998098470270633697509765625f,0.646699979621172094113035200280137360095977783203125f,0.785500022292136890200708876363933086395263671875f,0.2986000078856942963057008455507457256317138671875f,-0.740000010281800957301356902462430298328399658203125f,-0.276400030112267103277190471999347209930419921875f,-0.125100000256299637158008408732712268829345703125f,-0.363699991601705630017704606871120631694793701171875f,-0.841700005698204023474318091757595539093017578125f,0.3253999856770040111086927936412394046783447265625f,-0.14549998619407433775307936230092309415340423583984375f,-0.82109998925924276846899374504573643207550048828125f,-0.426699966508150563271328792325221002101898193359375f,-0.673399968965351813920960921677760779857635498046875f,-1.297000011682509779120664461515843868255615234375f,-0.3817999915003777999800149700604379177093505859375f,0.39370000080317246737848790871794335544109344482421875f,0.62090001944303541137060165056027472019195556640625f,-0.4000999691128737101308843193692155182361602783203125f,0.237599988894164670938380368170328438282012939453125f,0.183600007587671143483021296560764312744140625f,-0.4562000066518778762514330082922242581844329833984375f,0.155100008712708958835690964406239800155162811279296875f,0.4420000200271605450552669935859739780426025390625f,-0.0590000016093252810378544381819665431976318359375f,-0.5700999990642070525126428037765435874462127685546875f,0.097200003701448434156873190659098327159881591796875f,-0.003800003778934524234500713646411895751953125f,-0.153400002194941043054399187894887290894985198974609375f,0.488599992239475255928482511080801486968994140625f,0.079599976027012520063408373971469700336456298828125f,-0.6519999852329492195934790288447402417659759521484375f,0.3003000247597693572743082768283784389495849609375f,-0.048199987936019539347398676909506320953369140625f,-0.3585999982371925742796747726970352232456207275390625f,-0.78420001136064509950074352673254907131195068359375f,-0.6402000204801563398859798326157033443450927734375f,0.27239999419301741401255867458530701696872711181640625f,-0.06850000077486040339636019780300557613372802734375f,0.1808000118851662563201898592524230480194091796875f,0.24429999854117634416894588866853155195713043212890625f,-0.50700000184774385303398958058096468448638916015625f,-0.083799983215332218833282240666449069976806640625f,0.63099999418109653515074342067237012088298797607421875f,-0.5084000087141991031103316345252096652984619140625f,-0.04239998707771253094733765465207397937774658203125f,0.500200005802512226438238940318115055561065673828125f,0.32570000522136677290063744294457137584686279296875f,0.21239999959468836010501036071218550205230712890625f,-0.24240000272393225078104705971782095730304718017578125f,-0.5277000163078309213915417785756289958953857421875f,-0.0395999971866611844006911269389092922210693359375f,0.5597999953895806957149261506856419146060943603515625f,-0.82489999542236347451762412674725055694580078125f,-0.646399990773200983085189363919198513031005859375f,0.39560000176131726590256221243180334568023681640625f,1.1869999914169309107592198415659368038177490234375f,0.5893999871373178933708913973532617092132568359375f,-0.7912999817997217189002867598901502788066864013671875f,0.2236000039741397049652960049570538103580474853515625f,0.19180000856220724703149471679353155195713043212890625f,-0.00669999955147504433483618413447402417659759521484375f,0.61289998321831207217513792784302495419979095458984375f,0.26339998189210922507896839306340552866458892822265625f,-0.50129998183697466185293478702078573405742645263671875f,-1.014299990606307932949903261032886803150177001953125f,0.82430003016889108469200664330855943262577056884765625f,0.5884999656081202346058489638380706310272216796875f,0.5623999952360987375499234985909424722194671630859375f,0.4912000176787376659603978623636066913604736328125f,0.0994999984204769649664967801072634756565093994140625f,-0.131699998247623406566475523504777811467647552490234375f,-0.287700009155273495053961596568115055561065673828125f,0.30699999424815160864454810507595539093017578125f,-0.6103999623566871424173996274475939571857452392578125f,-1.383999975979327334840718322084285318851470947265625f,0.4274999625980857587137506925500929355621337890625f,0.6342000054374334894191633793525397777557373046875f,0.724200031626224927094881422817707061767578125f,0.46250000566244064970078397891484200954437255859375f,-0.32430000848770124122211200301535427570343017578125f,0.29969999014139148130198009312152862548828125f,-0.44279998676180820638137447531335055828094482421875f,-0.33799998958408838944933449965901672840118408203125f,-0.493600013697147232960560359060764312744140625f,-0.3296999889492990831740826251916587352752685546875f,0.33929999023377899192155382479541003704071044921875f,0.982500009238719940185546875f,-0.467999987423419749887898433371447026729583740234375f,-0.854199967774749158166969209560193121433258056640625f,-0.865400006091594065082972520031034946441650390625f,-0.1071999968469139474791518296115100383758544921875f,-0.32389999751597631227895135452854447066783905029296875f,-0.82270002822875998305107714259065687656402587890625f,-0.4835999730169771027021852205507457256317138671875f,-0.6495000145509834954538064266671426594257354736328125f,-0.5331000551819808919162824167869985103607177734375f,0.45699996981024781916858046315610408782958984375f,0.181699984426796579128904340905137360095977783203125f,1.117299996507167492154621868394315242767333984375f,0.06709998705387132389432736090384423732757568359375f,0.818799983252584961945785835268907248973846435546875f,0.119600031459331290761838317848742008209228515625f,-0.131700002159178264360406274136039428412914276123046875f,0.4951999941051006270953394050593487918376922607421875f,0.5344000176474452690200678262044675648212432861328125f,0.10149999453127389603679375795763917267322540283203125f,0.11309999807327979215454405448326724581420421600341796875f,-0.40889998194426302458026611930108629167079925537109375f,0.154499989926815128882253702613525092601776123046875f,0.5260000056549907210268202106817625463008880615234375f,-0.1566000017166135904744805884547531604766845703125f,-0.47780002982616398554682746180333197116851806640625f,-0.42519999682456255385432086768560111522674560546875f,-1.113699984896182826332733384333550930023193359375f,-0.9006000030279164292323912377469241619110107421875f,0.39359998043030497427707814495079219341278076171875f,-0.492500007301569009410258104253443889319896697998046875f,-0.33220000120550385869222509427345357835292816162109375f,0.2716000043243169681517201752285473048686981201171875f,-0.5598000102162359326030127704143524169921875f,-0.14960000493526504072860916494391858577728271484375f,0.4615999880075454253613997934735380113124847412109375f,-0.64630000556707367653785922811948694288730621337890625f,-0.0037999786704780991186680694227106869220733642578125f,0.848299991470575331931058826739899814128875732421875f,0.2988000000536441547183130751363933086395263671875f,0.1815999988853931323973256439785473048686981201171875f,-0.16449999853223562507054111847537569701671600341796875f,0.991800003682076880551221620407886803150177001953125f,0.496999991007149255484165450980071909725666046142578125f,-0.688099995670467645680901114246807992458343505859375f,-1.146399976170062728897391934879124164581298828125f,-0.6603999792695045556456534541212022304534912109375f,0.681699979881942308423958820640109479427337646484375f,0.438900023853779241989059300976805388927459716796875f,-0.03240001807212866680174556677229702472686767578125f,-0.6369000201061367771870891374419443309307098388671875f,0.346299997669458292648414499126374721527099609375f,0.1307999932587147196016985617461614310741424560546875f,-0.29069999482780695121419967108522541821002960205078125f,0.214800005596876120161908829686581157147884368896484375f,0.04099999708682300347817317742737941443920135498046875f,-0.2366000005990266663591370388530776835978031158447265625f,0.3898000325679777944287707214243710041046142578125f,0.020200014221668283198596327565610408782958984375f,-0.4141000072300433654248763559735380113124847412109375f,0.23510001500844968091996634029783308506011962890625f,0.011199994432926185794485718361102044582366943359375f,-0.35740000853538511638163299721782095730304718017578125f,-0.68050000649690645104783470742404460906982421875f,-0.5060000088214877678183256648480892181396484375f,0.2425999966055154555277795225265435874462127685546875f,-0.040999999359250249852948400075547397136688232421875f,1.1165000501871116256324967253021895885467529296875f,0.759699988949298354867778471088968217372894287109375f,-0.6290999717652800971023907550261355936527252197265625f,-0.7490999847292894031625110073946416378021240234375f,0.63889998973011952188016948639415204524993896484375f,-0.48020000233799198241513295215554535388946533203125f,0.39379996742010092702912515960633754730224609375f,0.158699993706494513201477047914522700011730194091796875f,-0.059200001246482170103035969077609479427337646484375f,0.398800007075816420609726264956407248973846435546875f,-0.303199967074394560739847293007187545299530029296875f,-0.7613999988943336294511254891403950750827789306640625f,-0.6582000406861308050565639859996736049652099609375f,-0.64649997407943027294408011584891937673091888427734375f,-0.0485000038668512711836910966667346656322479248046875f,0.49189999097436665209670536569319665431976318359375f,-0.049799999561905838874054097686894237995147705078125f,0.1874999958649277986832970555042265914380550384521484375f,-0.0044999963343144155913932991097681224346160888671875f,0.0372999963022768721288002780056558549404144287109375f,0.470999994292855272970399482801440171897411346435546875f,-0.229799986188113702922919401316903531551361083984375f,0.1219999973773955304068294935859739780426025390625f,-0.6556999706879260703118461606209166347980499267578125f,0.1090999865174300254011541255749762058258056640625f,0.501800003123282767347745902952738106250762939453125f,-0.109199986624717748640023273765109479427337646484375f,-0.54720000370144816770334728062152862548828125f,0.096999994471669293005788858863525092601776123046875f,0.1495999945044517998038458017617813311517238616943359375f,-0.0955999969184400111288368862005881965160369873046875f,0.136299993851035916581082574339234270155429840087890625f,0.01950000130757689742466709503787569701671600341796875f,-0.165300001607090241151354348403401672840118408203125f,0.3780999666690831872983835637569427490234375f,0.543799995918571976716293647768907248973846435546875f,0.494200001823902113073927466757595539093017578125f,0.19739999899864191235110411071218550205230712890625f,-0.380899981035292267694813972411793656647205352783203125f,-0.795700010287761383409588233917020261287689208984375f,0.287999996513128575514883777941577136516571044921875f,-0.61190001489072998186458107738872058689594268798828125f,-0.25870004956722336686425478546880185604095458984375f,-0.53889999367892682524683323208591900765895843505859375f,-0.7454000097051263207248439357499592006206512451171875f,-0.5170999928653241539677765103988349437713623046875f,-0.461799995374679195236922168987803161144256591796875f,0.10930000409185891552255043279728852212429046630859375f,0.05300000888109224206345970742404460906982421875f,-0.22960001953840247779226046986877918243408203125f,-0.439700005824863904280164206284098327159881591796875f,-0.54769999976754224491060085711069405078887939453125f,-0.536600005590915696274123547482304275035858154296875f,0.02600000312179329142736605717800557613372802734375f,0.2870000018477441017239470966160297393798828125f,-0.08500002086162528058821408194489777088165283203125f,-0.55399999509006736531091519282199442386627197265625f,0.5998000242233274548198096454143524169921875f,0.073000001132488279154131305404007434844970703125f,-0.653500002190470663521182359545491635799407958984375f,0.48869997655153252225090909632854163646697998046875f,0.20279997767210033998708240687847137451171875f,-0.43349998017400503957929913667612709105014801025390625f,0.68990004131793991604126858874224126338958740234375f,0.09719999699592563047190196812152862548828125f,-0.81480002169013021617871572743752039968967437744140625f,0.68770002495050430724177203956060111522674560546875f,0.2776000054717062681675088242627680301666259765625f,-0.52990000887066124324320526284282095730304718017578125f,-0.2551000081539154251686341012828052043914794921875f,-0.16059999498128885164760504267178475856781005859375f,0.2779000081032514390955157068674452602863311767578125f,0.28310002344250673189662848017178475856781005859375f,0.044000018596649592694802777259610593318939208984375f,-0.2182999974682926502822510883561335504055023193359375f,0.4165999805569651215364501695148646831512451171875f,0.2911999735713006742798825143836438655853271484375f,-0.3298999947890639550251989930984564125537872314453125f,-0.63000001654028903175230880151502788066864013671875f,-0.547600025439262783066851625335402786731719970703125f,0.090099998877942510944194509647786617279052734375f,-0.165199976894259581428059391328133642673492431640625f,-0.3449999867379662799038442244636826217174530029296875f,-0.102599999995529600482768728397786617279052734375f,-0.1800999980211255291351335472427308559417724609375f,-0.36420002506971371758481836877763271331787109375f,-0.2751000079303980161427034545340575277805328369140625f,0.53850001060962693344436047482304275035858154296875f,0.15929999120235383003318929695524275302886962890625f,0.3756999754190448737034557780134491622447967529296875f,-0.384099984133243754769182487507350742816925048828125f,-0.05810000079274146855823346413671970367431640625f,0.47459999748468373326204527984373271465301513671875f,-0.7330999723687769975555283963331021368503570556640625f,-1.1371000094056125107044863398186862468719482421875f,0.4808999762296684021833925726241432130336761474609375f,-0.6478000076606866830530861989245750010013580322265625f,-0.4250000466406351762316262465901672840118408203125f,0.43449999290704699461684867856092751026153564453125f,-0.50749996811151543152362819455447606742382049560546875f,-1.01949998307228018035175409750081598758697509765625f,0.47929996795654350449922276311554014682769775390625f,0.4135999926492572864589192249695770442485809326171875f,0.0999999961256978764367886469699442386627197265625f,0.087399998551607183117084787227213382720947265625f,-0.0804000034093856896788565791212022304534912109375f,-0.58799999040365236169236595742404460906982421875f,0.50119998474717153413848791387863457202911376953125f,-0.0436999888822435789137443862273357808589935302734375f,-0.6889000229597090907418532879091799259185791015625f,0.16559998618960403149458215921185910701751708984375f,0.47839999537765987014381607878021895885467529296875f,0.0867999992489814076179754920303821563720703125f,0.09749998822808247922466762247495353221893310546875f,0.79629998135268686976218077688827179372310638427734375f,1.48070003800392147041975476895458996295928955078125f,0.2315999671459201270096173175261355936527252197265625f,0.723400007522105870094719648477621376514434814453125f,0.8606000392377382013364695012569427490234375f,-0.18859996839761805631496827118098735809326171875f,-0.6928999714836481249591315645375289022922515869140625f,-1.4856999900221818933232498238794505596160888671875f,0.4096000044882297519421854303800500929355621337890625f,-0.8673000121906395687432222985080443322658538818359375f,-0.6897000510871418299529977957718074321746826171875f,0.53729995917975947872236019975389353930950164794921875f,0.444500000432133557382030630833469331264495849609375f,0.8917000173956157738075489760376513004302978515625f,0.50810000496506635414561969810165464878082275390625f,-0.6910999803930522045192219593445770442485809326171875f,-0.8443000090837475823946078890003263950347900390625f,-0.363999980598688210875479853712022304534912109375f,-0.140799999815225618249314720742404460906982421875f,-0.270699986445903650889022173942066729068756103515625f,-0.09479998488426222280622823745943605899810791015625f,0.278699989329278474681217403485788963735103607177734375f,-0.91189999011754974134191797929815948009490966796875f,-0.7722000029563904632823323481716215610504150390625f,0.29039999092966317650876817424432374536991119384765625f,1.005400009816884931979075190611183643341064453125f,0.79440000721663228677726920068380422890186309814453125f,-0.432300005522370300869994252934702672064304351806640625f,-0.921200048673153304434890742413699626922607421875f,-0.4256000188231467973309918306767940521240234375f,0.60590002104490991996499360539019107818603515625f,-0.1830000318288806937516710604541003704071044921875f,0.06900000399351124968916337820701301097869873046875f,0.4251000218629836790995568662765435874462127685546875f,0.985800002050400081543557462282478809356689453125f,0.706999992907046959089711890555918216705322265625f,-0.43970000809729103963263696641661226749420166015625f,-0.814100009167194826886770897544920444488525390625f,-0.6650000083446503396089610760100185871124267578125f,0.20060000611990691510300166555680334568023681640625f,-0.697199977512657387279659815249033272266387939453125f,-0.265199970375001592426400520707829855382442474365234375f,0.66029998171404002960116486065089702606201171875f,0.044799991217255641373640173696912825107574462890625f,0.3160000071823596545783630062942393124103546142578125f,0.26299999889731406899073817839962430298328399658203125f,1.0003999978959559058466766146011650562286376953125f,0.4795999854147432461815014903550036251544952392578125f,-0.70819999501407149278264796521398238837718963623046875f,-1.1163000100374222967047899146564304828643798828125f,-0.86860000654458957569659105502068996429443359375f,0.4051000106126070310352815795340575277805328369140625f,-1.0602000264406203910994008765555918216705322265625f,-0.6996000162601472283085968228988349437713623046875f,0.5712000064283608846693596206023357808589935302734375f,-0.28019999991655364368625669158063828945159912109375f,-0.361200009334087468459983938373625278472900390625f,-0.07709999945908785345949354450567625463008880615234375f,-0.812399981713294749852138920687139034271240234375f,-0.30199997532367728325652933563105762004852294921875f,0.693199980112910285612315419712103903293609619140625f,-0.1164999914765356692214481881819665431976318359375f,0.37560000079274136197682310012169182300567626953125f,-0.077400000190734896676758580724708735942840576171875f,0.1696000003159046087830574833787977695465087890625f,0.515599999898672223253015545196831226348876953125f,0.594299984794854907477201777510344982147216796875f,-0.3441999913558364543320067241438664495944976806640625f,0.4041999961614610725746388197876513004302978515625f,-0.55699998178705545370803520199842751026153564453125f,0.0611000038437545800462658007745631039142608642578125f,-0.926900015654414932697591211763210594654083251953125f,-0.12859998657703375357641562004573643207550048828125f,-0.9289000162169334107176155157503671944141387939453125f,-0.5513000377535828278041662997566163539886474609375f,-0.289500003635883018660024390555918216705322265625f,0.69059996029883652379766090234625153243541717529296875f,1.21380001391172331892676083953119814395904541015625f,-0.38369997148513856899398888344876468181610107421875f,0.4943999967113137383734056129469536244869232177734375f,-0.2255999842524527565501557546667754650115966796875f,0.45589997876286592060068869614042341709136962890625f,-0.88060001371949869763966489699669182300567626953125f,-0.52820006154775711593174492008984088897705078125f,-0.1679999996237457882131138831027783453464508056640625f,0.2121000059038399554456333362395525909960269927978515625f,0.9849000044003133336190103364060632884502410888671875f,-0.59119997453987604973946190511924214661121368408203125f,-0.2253999955117702569395987666212022304534912109375f,-0.4430000097006558945622600731439888477325439453125f,0.08379999729692944043790703290142118930816650390625f,0.543999978289008367937640286982059478759765625f,-0.1215999991834169069448989830561913549900054931640625f,0.56429997674822818254369849455542862415313720703125f,-0.6668999837845566158733845441020093858242034912109375f,0.1023000045537951763208184274844825267791748046875f,0.3014000035881991834685322828590869903564453125f,-0.3336000104933976029286668563145212829113006591796875f,0.5375999998092648723968522972427308559417724609375f,0.41999998569488550259620751603506505489349365234375f,-0.2823000073477623939055547452881000936031341552734375f,-0.1263000225543979837539154686965048313140869140625f,-0.23539999059438709849700899212621152400970458984375f,-0.31259998833537105955571178128593601286411285400390625f,0.28259998073577907717890411731787025928497314453125f,0.745599983507394536985657396144233644008636474609375f,0.453099971660971601039591405424289405345916748046875f,-0.86320001580119143369529410847462713718414306640625f,-0.4571999965488908745925300536327995359897613525390625f,0.592800008815526968675158059340901672840118408203125f,0.42529999406337726242099961382336914539337158203125f,0.2871999992311000671207921186578460037708282470703125f,-0.17109999339431525289256796895642764866352081298828125f,-0.155899996495246995209527085535228252410888671875f,-0.31740001151561703096604105667211115360260009765625f,-0.2717999957844614566937480049091391265392303466796875f,0.2020999915242196465214874478988349437713623046875f,0.0955999848484996306297034607268869876861572265625f,-0.2506999944552779080453319693333469331264495849609375f,0.218899999862909311332259676419198513031005859375f,0.0151999866545200479350796740618534386157989501953125f,-0.33439999995231628471259455182007513940334320068359375f,-0.8034000108003613860319092054851353168487548828125f,-0.3438000019907949678099612356163561344146728515625f,0.5438999974787235291984188734204508364200592041015625f,-0.01510000224933027990203981971717439591884613037109375f,0.001799998019635651314018787161330692470073699951171875f,0.0087000000022351697825939709218800999224185943603515625f,-0.0480999802291395273101670682081021368503570556640625f,-0.33839999873042092559671800700016319751739501953125f,-0.3213000007614492670171557620051316916942596435546875f,-0.080500001430511503031084430404007434844970703125f,-0.121600002908706983362208120524883270263671875f,0.03819999806880947090803601895458996295928955078125f,0.6320999989747999592282212688587605953216552734375f,0.1751999942541129229311991366557776927947998046875f,-0.5615999801844357985913802622235380113124847412109375f,-0.383000003665685373466232022110489197075366973876953125f,0.09440001220852101138092393739498220384120941162109375f,0.6090999874860048013403002187260426580905914306640625f,-0.45380003626346621103948564268648624420166015625f,-0.210200007665157340142059183563105762004852294921875f,0.4292000212326645414151471413788385689258575439453125f,0.1552000106453892414037909475155174732208251953125f,-0.19500000059604616353681194595992565155029296875f,-0.3941999912813305417813580788788385689258575439453125f,0.5388999965101479716622634441591799259185791015625f,0.26339999433457872424213519479962997138500213623046875f,-0.368099990697205059486663003553985618054866790771484375f,1.15150002217292790618330400320701301097869873046875f,0.5257999844670298017490495112724602222442626953125f,0.02299999569356447892687356215901672840118408203125f,-0.26539999983310702535987957162433303892612457275390625f,0.4274000033199787385029821962234564125537872314453125f,-0.1184000010401009550520257107564248144626617431640625f,0.470799984392523906961969260009936988353729248046875f,0.3032000093936917384240814499207772314548492431640625f,0.434499984413385664794304830138571560382843017578125f,-0.806199975545704550228265361511148512363433837890625f,-1.051400014615058609024345059879124164581298828125f,-0.01299998816847836025090145994909107685089111328125f,-0.59659996990263497451678631477989256381988525390625f,-1.11459998824596340227799373678863048553466796875f,0.172200001093745225233533346909098327159881591796875f,-0.40129999129921201106441230876953341066837310791015625f,-0.8137000130593776692222718338598497211933135986328125f,-0.17109997882843064331837013014592230319976806640625f,-0.912599968926608884345341721200384199619293212890625f,-0.643800001394748022676139953546226024627685546875f,-0.0228999969087541332957158601857372559607028961181640625f,-0.08929999982640145217427374291219166480004787445068359375f,-0.12490000389739874808725517141283489763736724853515625f,0.620799963828921885777845091070048511028289794921875f,-0.02359999894499775763279103557579219341278076171875f,0.0655999896913765923045502859167754650115966796875f,0.18059999453425401583217535517178475856781005859375f,0.4542999750718477169897369094542227685451507568359375f,0.7930999985575670763182642986066639423370361328125f,0.31539997196793567724171225563623011112213134765625f,0.54390001070350368905792493023909628391265869140625f,0.25470005451440869137513800524175167083740234375f,-0.5753999673672025583215372535050846636295318603515625f,-0.0463000013016163780577016950701363384723663330078125f,0.247299993210285951050764197134412825107574462890625f,-0.02210000419020641260203774436376988887786865234375f,0.416299962204695062073511735434294678270816802978515625f,1.077100008809565689915643815766088664531707763671875f,0.55269996117353503706226547365076839923858642578125f,0.2146999947607517444936320316628552973270416259765625f,0.2299000025749204922931312466971576213836669921875f,0.037499993555247801424457065877504646778106689453125f,0.344299998801946560700315558278816752135753631591796875f,0.01270002189576635576173657682375051081180572509765625f,-0.07710000389218318872508461936376988887786865234375f,0.14689999868571756991286747506819665431976318359375f,-0.1364999872669578895312270105932839214801788330078125f,-0.36800000195205206932058672464336268603801727294921875f,-0.20859999980181453960170756545267067849636077880859375f,0.16969999109134070724991261158720590174198150634765625f,0.047399989871680314745816531285527162253856658935546875f,-0.100199999376386461591437182505615055561065673828125f,0.8766999818563459001552473637275397777557373046875f,0.33859997614622105999160339706577360630035400390625f,-0.6610999834477901959672863085870631039142608642578125f,-1.0706000200152399060016250587068498134613037109375f,-0.74700000631809171380837142351083457469940185546875f,0.4632000185206532894000019950908608734607696533203125f,-0.832500014603137827862155972979962825775146484375f,-0.54120000516176247629118734039366245269775390625f,0.5547000009819864629889707430265843868255615234375f,-0.3322000017642976477105776211828924715518951416015625f,-0.326400001353025270844909755396656692028045654296875f,0.0594000062853098087511938274474232457578182220458984375f,0.40170001538395894158384180627763271331787109375f,0.2782000128209591860439786614733748137950897216796875f,-0.13720000433474777157272228578221984207630157470703125f,0.430799969565867968412931077182292938232421875f,0.050999964714050438630010830820538103580474853515625f,-0.5846999771773815357533976566628552973270416259765625f,-0.6805000142455099876315216533839702606201171875f,-0.43539999506473492374425404705107212066650390625f,0.517600015343725683436559847905300557613372802734375f,-0.494499984711408602233717601848184131085872650146484375f,-0.52359999779015786725011594171519391238689422607421875f,0.0672999930053949368957688648151815868914127349853515625f,0.0801999850153922722029165015555918216705322265625f,0.1045999876499177361210968228988349437713623046875f,-0.0787999935343861590553160567651502788066864013671875f,0.14609998255372058650891631259582936763763427734375f,0.204999985694885200615544817992486059665679931640625f,-0.02889999755322936181300974567420780658721923828125f,0.6306999858871102926372032015933655202388763427734375f,0.66320000231564002479700548065011389553546905517578125f,0.0358000001505016651748292133561335504055023193359375f,-0.33749999955296505760315994848497211933135986328125f,-0.148399998879432448717352599487639963626861572265625f,0.176299998079240349824203804018907248973846435546875f,1.0420000250935554930720172706060111522674560546875f,-0.74280003265738514528493396937847137451171875f,0.5989999692365530759019520701258443295955657958984375f,0.16589999411106115534408900202834047377109527587890625f,0.4406999998942016105729635455645620822906494140625f,0.0993000015772878275033264117155340500175952911376953125f,0.479799967096001311883668449809192679822444915771484375f,0.58339999252781193916916890884749591350555419921875f,0.25109999060034748907810353557579219341278076171875f,0.82069996521174937154086137525155209004878997802734375f,0.43870001916885303927529093925841152667999267578125f,0.36389998153448122053532642894424498081207275390625f,-0.78849997842311891904643061934621073305606842041015625f,-0.14689998981952623324787055025808513164520263671875f,0.25019999946951809732809124398045241832733154296875f,0.3833999804764987029415124197839759290218353271484375f,1.4166000219821928141072930884547531604766845703125f,0.3071999817222359485668903289479203522205352783203125f,-0.09560000161230564064140935442992486059665679931640625f,0.20119999912679187303865546709857881069183349609375f,-0.120500003740191541368176331161521375179290771484375f,0.55500000510364755346159881810308434069156646728515625f,0.4554000265061859398230126316775567829608917236328125f,0.3122999942719939525659356149844825267791748046875f,-0.459600012646615263545157858970924280583858489990234375f,-1.095600020313263645022061609779484570026397705078125f,0.20789998388886399283137507154606282711029052734375f,0.6880000034794206964505747237126342952251434326171875f,1.7348000751852996614843505085445940494537353515625f,0.538599976984411643599059971165843307971954345703125f,0.0003999988831580036929835841874592006206512451171875f,0.526399995336681580937465696479193866252899169921875f,-0.1538999854087830243543066899292171001434326171875f,-0.5994999876916409231597526741097681224346160888671875f,-1.0139000110387801356637282879091799259185791015625f,-0.2788999731898311740252438539755530655384063720703125f,-0.4376999920934439014530425993143580853939056396484375f,-0.8397000057876109480048398836515843868255615234375f,-0.6389999716952445840689733813633210957050323486328125f,0.84349996814131766864619521584245376288890838623046875f,0.6319000098243350294069387018680572509765625f,-0.0785999922394753358645402840920723974704742431640625f,-0.70079997224807766276200027277809567749500274658203125f,-0.75960000538229888178420878830365836620330810546875f,0.34069998704195025851504396996460855007171630859375f,0.6389000232949852975394833265454508364200592041015625f,-0.434500010788440693687562088598497211933135986328125f,-0.15619999741315826469190142233856022357940673828125f,0.367400004623830322980637674845638684928417205810546875f,0.04979999442100524476018108543939888477325439453125f,0.283800008547306248374297865666449069976806640625f,0.18690000159144404268118933032383210957050323486328125f,0.19939997402429554540503886528313159942626953125f,-0.064400034821033624865549427340738475322723388671875f,-0.4257999875962734659395891867461614310741424560546875f,0.8893999880313874228932036203332245349884033203125f,0.6423999888658524781703818007372319698333740234375f,-0.4269999925345182578695357733522541821002960205078125f,-0.806599991583823783258822004427202045917510986328125f,-0.63480000783205081660298674250952899456024169921875f,0.324599981242418234028690449122223071753978729248046875f,1.0353000313162805667843713308684527873992919921875f,0.611000012993812458006459564785473048686981201171875f,-0.571700013595819456213575904257595539093017578125f,-0.435399990892410215792551753111183643341064453125f,-0.49680000580549243949235460604541003704071044921875f,-0.0063000049337744723487730880151502788066864013671875f,-0.556700019854307281974570287275128066539764404296875f,-0.2209999964535234173723665662691928446292877197265625f,0.500900012588501031274290653527714312076568603515625f,0.2881999804854391555863912799395620822906494140625f,0.2927999836325643201462298748083412647247314453125f,-0.1550999966427684118031038451590575277805328369140625f,0.40520002241730690428767047706060111522674560546875f,0.051800012958050256628439456108026206493377685546875f,-0.3535999955549835593870966476970352232456207275390625f,-0.9005000184178353350716861314140260219573974609375f,-0.76880002642869982310003251768648624420166015625f,0.2063000009849667026884389997576363384723663330078125f,-0.588399999177455779175716088502667844295501708984375f,-0.16299998861551312501205757143907248973846435546875f,0.55839998908191912363463416113518178462982177734375f,-0.813900018489360999041082322946749627590179443359375f,-0.52520000319480875106137318653054535388946533203125f,0.451700012589991095612873550635413266718387603759765625f,0.0281999938964840879407347529195249080657958984375f,0.278200013864040585076509159989655017852783203125f,0.1985000059902668478883924763067625463008880615234375f,0.057399986183643392223530099727213382720947265625f,-0.0233000017523767866123307612724602222442626953125f,-0.139299994182586761581887913052923977375030517578125f,-0.2293999932095409644006167582119815051555633544921875f,0.0542000058472158485756153822876513004302978515625f,-0.53249998122453678917054276098497211933135986328125f,-0.4457999833121897292897983788861893117427825927734375f,-0.7818000090837475823946078890003263950347900390625f,0.30180000357031833146947974455542862415313720703125f,-0.5846000146210190706597131793387234210968017578125f,-1.065400034105778548365606184233911335468292236328125f,-0.36599997931718863952710307785309851169586181640625f,0.210400005793571320822366033098660409450531005859375f,-0.567999976396560413149927626363933086395263671875f,0.434200005251168530406857826164923608303070068359375f,-0.21580000011324873554485748172737658023834228515625f,0.485799991172552125817674095742404460906982421875f,-0.1307000019431114878898370079696178436279296875f,0.7749000035434960853564234639634378254413604736328125f,0.180900019443035464661306832567788660526275634765625f,0.537399983501434252985973216709680855274200439453125f,-0.1672999957248568758672035983181558549404144287109375f,0.2110999979019165806448654620908200740814208984375f,-0.120700000825524345060557607212103903293609619140625f,0.0634999946653842239907561406653258018195629119873046875f,0.6247000172615051472035929691628552973270416259765625f,-0.32059997471570955696051896666176617145538330078125f,-0.6785000061020252104526662151329219341278076171875f,-1.3513000443100935399343143217265605926513671875f,0.3448999654710294127113456852384842932224273681640625f,0.8455000101476903839881060775951482355594635009765625f,0.9407000539481646939066195045597851276397705078125f,0.5767999976098536762947333045303821563720703125f,-0.82840001221597159286602618522010743618011474609375f,-0.35360004200935435392239014618098735809326171875f,-0.282899983888864614556268861633725464344024658203125f,-0.269900001159310232168309084954671561717987060546875f,0.12769997890591611167110386304557323455810546875f,0.207799988400936275212416148860938847064971923828125f,-0.257400004586577413334680386469699442386627197265625f,0.408600001627206665943958796560764312744140625f,-0.5464999917745583246642127051018178462982177734375f,-0.51779999424964184839836889295838773250579833984375f,-0.87300003153085725671189720742404460906982421875f,-0.097599973285198426964370810310356318950653076171875f,-0.219799989652633642744916642186581157147884368896484375f,0.35140001014471078377710000495426356792449951171875f,-0.150400027549267178983427584171295166015625f,-0.64550001078099017437494921978213824331760406494140625f,-0.2670000236034397733675405106623657047748565673828125f,0.140400017863512527327429779688827693462371826171875f,0.52020002256631858728042061557061970233917236328125f,-0.88400001651048665252119462820701301097869873046875f,-0.54700000274181359571912253159098327159881591796875f,0.53740000730603931788920135659282095730304718017578125f,-0.5066999657630919529083257657475769519805908203125f,-0.40179997421503088617100729607045650482177734375f,0.2898999817132950074238806337234564125537872314453125f,-1.031599996054172407866644789464771747589111328125f,-0.8496000076174734516598618938587605953216552734375f,0.290699996094405666013926747837103903293609619140625f,0.8968999905645846926205422278144396841526031494140625f,0.48399998376518504894505667834891937673091888427734375f,-0.6318999889999628027936751095694489777088165283203125f,0.782199995505809653195683495141565799713134765625f,0.71060001494884517114769550971686840057373046875f,-0.121399989022314525044521360541693866252899169921875f,0.365900003796815642687079161987639963626861572265625f,0.418800019872188844516358585678972303867340087890625f,0.015900006143748812714733276152401231229305267333984375f,1.0690000364780427588584643672220408916473388671875f,0.731800021302699832403959590010344982147216796875f,-0.4975000184774398537257411589962430298328399658203125f,-0.079300002825259952032865840010344982147216796875f,0.252400016582011943455654545687139034271240234375f,0.48109999384135004873996876995079219341278076171875f,-0.265000013485551022540676058270037174224853515625f,-0.07639999636113647962787354117608629167079925537109375f,0.316400007648766024015429820792633108794689178466796875f,0.80030002595186255920367557337158359587192535400390625f,0.3987999974459406349325263363425619900226593017578125f,-0.611800018620491048437770587042905390262603759765625f,-0.30290001026391966121309451409615576267242431640625f,-0.03780000121593474204928497783839702606201171875f,0.31619999916404484263665608523297123610973358154296875f,-1.24589999649524685310097993351519107818603515625f,-0.81199999737739592120533416164107620716094970703125f,0.631999988213181485008362869848497211933135986328125f,-0.310400011754035975997112473123706877231597900390625f,-0.844399999505281328993078204803168773651123046875f,0.6497999974012369506226605153642594814300537109375f,-0.055299999986588854739011367200873792171478271484375f,-0.366500004291534509093253291212022304534912109375f,-0.5454000037819140001005280282697640359401702880859375f,0.0437999917834996921328638563863933086395263671875f,0.2766000046223400232747735572047531604766845703125f,0.1854999945759772117526154033839702606201171875f,-0.14410000588893889794661617997917346656322479248046875f,0.64870000635385505205476874834857881069183349609375f,0.30609997510314013169363533961586654186248779296875f,0.4592999938100577050903439157991670072078704833984375f,1.2236999971151352184506322373636066913604736328125f,0.604999985694885111797702847979962825775146484375f,-0.8208999766394498021071512994240038096904754638671875f,-0.2129000078797336215075119980610907077789306640625f,-0.049599996162205950422929845444741658866405487060546875f,0.10240000099167223535090442965156398713588714599609375f,-0.713199989444762483259410146274603903293609619140625f,-0.5828999956607816557152546010911464691162109375f,0.78289999200999726980398918385617434978485107421875f,0.39730003077983866432987269945442676544189453125f,-0.4514000034391874027761559773352928459644317626953125f,0.2014000005334615206464832226629368960857391357421875f,-0.0838000002026557666567896376363933086395263671875f,-0.3412999969244001619017581106163561344146728515625f,0.66020000181645155379328571143560111522674560546875f,-0.04700000005960447424513404257595539093017578125f,-0.55219997866749803705488375271670520305633544921875f,0.6045999884694814863195233556325547397136688232421875f,0.9578000196933746934746523038484156131744384765625f,-0.1006000019475816742442475515417754650115966796875f,0.33279997990727439027835998786031268537044525146484375f,0.75920000547468624585434326945687644183635711669921875f,0.01800000847131000281109436400583945214748382568359375f,0.02050000739097601609728371840901672840118408203125f,-0.73990001714080566586062559508718550205230712890625f,-0.2440999810040000994604270090349018573760986328125f,-0.293799989287555352035496980533935129642486572265625f,-0.181399994945525833145438809879124164581298828125f,0.3185999959647656254446701495908200740814208984375f,-0.72930000003129225039089078563847579061985015869140625f,0.10949996012449236815200492856092751026153564453125f,-0.560499981015920578641953397891484200954437255859375f,0.6247000126048922741262003910378552973270416259765625f,-0.30510000055432329180149508829344995319843292236328125f,-0.2497999982208012337281388681731186807155609130859375f,0.11930000371932984737366467697938787750899791717529296875f,-0.217500000894069511758743828977458178997039794921875f,0.0684000075221060654939719825051724910736083984375f,0.328799992901086801655452518389211036264896392822265625f,-0.7481000092864036332684918306767940521240234375f,-0.09059998097419708074085065163671970367431640625f,0.7770000012144446799311481299810111522674560546875f,0.9560000323653223830433489638380706310272216796875f,0.38900000429153447356611650320701301097869873046875f,-0.7154000108972191629419512537424452602863311767578125f,-0.325200043725967447016955702565610408782958984375f,0.1103999825477597340750435250811278820037841796875f,0.469200009423494268645526972250081598758697509765625f,0.91720000474452945127268321812152862548828125f,0.6756000113725662004071637056767940521240234375f,-0.366599995458126048930580509477294981479644775390625f,0.30400002038478834975876452517695724964141845703125f,-0.132200004446506369504277245141565799713134765625f,-0.57250000156462188538597501974436454474925994873046875f,-0.65589999783635111185731147998012602329254150390625f,-0.674800021541118955070714946486987173557281494140625f,0.0363999883890151476606433789129368960857391357421875f,0.10870001529455208810759359039366245269775390625f,-0.2002000157117844736376355285756289958953857421875f,-0.4304000114187598313719718134962022304534912109375f,0.211900002411007815084076355560682713985443115234375f,0.1872000069797039589758469446678645908832550048828125f,-0.03689999835044144305129520944319665431976318359375f,0.09149999013543119730229591368697583675384521484375f,0.158200001645087962032221184927038848400115966796875f,0.123500002861022917244326890795491635799407958984375f,-0.80940001127719884976841058232821524143218994140625f,-0.48459999986886970191335421986877918243408203125f,0.48430000461339950401651321953977458178997039794921875f,0.2789999973624943674366249979357235133647918701171875f,0.3058000074148179425748139692586846649646759033203125f,-0.0126999954462051201087291474323137663304805755615234375f,-0.23130000929236427253954389016143977642059326171875f,0.133400010800361901175392631557770073413848876953125f,0.3433000048071146448336321554961614310741424560546875f,0.307700004535913451686468533807783387601375579833984375f,-0.363399997985362599450809284462593495845794677734375f,0.18569998745173232901350957035901956260204315185546875f,-0.0321000008001923742284589025075547397136688232421875f,0.26190000170320271166701786569319665431976318359375f,-0.3663999800443651366776975919492542743682861328125f,-0.2510999916806817555681163867120631039142608642578125f,-0.2091000075280664560750665259547531604766845703125f,-0.6104999804198740775973419658839702606201171875f,-0.77259997444003847277826935169287025928497314453125f,-0.52100001031160303455180837772786617279052734375f,0.22949998724460574095473930356092751026153564453125f,0.78310001167058918003505141314235515892505645751953125f,0.30230003793239657028379951952956616878509521484375f,-0.5735999573707584175963347661308944225311279296875f,-0.5856000143527981816049532426404766738414764404296875f,-0.6384000257015234325308483676053583621978759765625f,-0.38540000281333863796362493303604423999786376953125f,0.6091999801427128158337609420414082705974578857421875f,0.0171999934196467307856437400914728641510009765625f,0.56269997041225483513926519663073122501373291015625f,-0.1257999938920140170495187703636474907398223876953125f,-0.4370000134706497618708453956060111522674560546875f,0.6523999998927110244295590746332891285419464111328125f,-0.4990999941542744178235579965985380113124847412109375f,-0.0763000110805034381655787001363933086395263671875f,0.487899997001886021052996511571109294891357421875f,-0.3987999816134573105586014207801781594753265380859375f,-0.6752000064730641071264471975155174732208251953125f,-0.14680000185668451440079707026598043739795684814453125f,0.22519999663829803893122516456060111522674560546875f,0.1288000038534402591494654188863933086395263671875f,-0.2006999941945075693894295909558422863483428955078125f,0.44429997496008899471320319207734428346157073974609375f,0.57749999836087173576970599242486059665679931640625f,0.12969999550580979530423064716160297393798828125f,-0.3054000002428889359862296259962022304534912109375f,0.32339999709129330085488618351519107818603515625f,-0.32029999570250489160372353580896742641925811767578125f,0.0900000001117586823085048308712430298328399658203125f,-0.1595999986767768774598152958787977695465087890625f,-0.5326999853134151141631491555017419159412384033203125f,-0.764699982616305451443849960924126207828521728515625f,-1.001100025171041352223255671560764312744140625f,-0.022900000075250781517155473920865915715694427490234375f,0.06960000102370977881349745075567625463008880615234375f,-0.81360000386238073843969687004573643207550048828125f,-0.5568000031232838153982811491005122661590576171875f,0.50879999155998223425712012613075785338878631591796875f,-0.617900017714500648935427307151257991790771484375f,-0.46460001149177543311452609486877918243408203125f,0.2210000101998448318596501849242486059665679931640625f,-0.67630004520416253654957472463138401508331298828125f,-0.0654000102639198388487784541212022304534912109375f,0.75930001295804971928049553753226064145565032958984375f,-0.459300019812584281453382573090493679046630859375f,0.1396000287771226311406280728988349437713623046875f,0.838400013929605503193442928022705018520355224609375f,0.1266000079751012918904962134547531604766845703125f,0.4408000357270243085849870112724602222442626953125f,0.3791000013813377034210816418635658919811248779296875f,-0.824999999403953410137546597979962825775146484375f,-0.508399996495246586647454023477621376514434814453125f,0.4300999988779425908802522826590575277805328369140625f,0.172700002896785775874377577565610408782958984375f,0.08439999130964270790400405530817806720733642578125f,-0.229900005666911599888635464594699442386627197265625f,0.482200012791156762403943503159098327159881591796875f,0.0975999944448471268287903512828052043914794921875f,-0.499900001792609682826906691843760199844837188720703125f,0.562400001680850802898703477694652974605560302734375f,0.09239998707771324149007341475225985050201416015625f,-0.618299990166723656415115328854881227016448974609375f,-0.1397999978110193719516018973081372678279876708984375f,-0.104600000464916131903692075866274535655975341796875f,0.028999998815357697079519994076690636575222015380859375f,0.01609999119639393683200978557579219341278076171875f,-0.003600003266334805829274046118371188640594482421875f,0.07530000274330372889153295545838773250579833984375f,0.7022999953150748098096300964243710041046142578125f,0.310799982678890529541604337282478809356689453125f,-0.6476999907150864377314292141818441450595855712890625f,1.118900005376338935292324094916693866252899169921875f,0.84420000420808793961668925476260483264923095703125f,-0.5046000005766748675029020887450315058231353759765625f,-0.21400000950694053614142831065692007541656494140625f,0.082399997210502728961500906734727323055267333984375f,0.2784999963417649215813298724242486059665679931640625f,-0.3319000059127805712932968162931501865386962890625f,-0.0442000042080881172523731947876513004302978515625f,0.29920000122785506135869582067243754863739013671875f,0.36239998111724869289673733874224126338958740234375f,1.3704000120520589689476764760911464691162109375f,0.273699991601705772126251758891157805919647216796875f,-0.51220000537782894856064785926719196140766143798828125f,-0.16700001823902166364632648765109479427337646484375f,-0.5728999615371233034011311247013509273529052734375f,-0.4439000034391880067374813734204508364200592041015625f,-0.0810999952197075657522873370908200740814208984375f,-0.71839999724030434435917413793504238128662109375f,0.4646999736383559120866948433103971183300018310546875f,1.0947000014662737754633781150914728641510009765625f,-0.73019999812841351882752860547043383121490478515625f,0.33449999957531684202649557846598327159881591796875f,0.32730000067949305275760707445442676544189453125f,0.14639999747872334268095073639415204524993896484375f,-0.625499971739948090743155262316577136516571044921875f,-0.152699992465972211874714048462919890880584716796875f,-0.02219998909831044642260167165659368038177490234375f,-0.63439998698830601941978102331631816923618316650390625f,-1.14520002822875976988825641456060111522674560546875f,0.302599986547231747380237720790319144725799560546875f,0.20990000112205742244242401284282095730304718017578125f,-0.11569999493956562020002820645458996295928955078125f,0.391999979346990823358964917133562266826629638671875f,0.17109999890774485464817189495079219341278076171875f,-0.244899989908933690685444162227213382720947265625f,0.054299999770522078268442101034452207386493682861328125f,-0.259499988473951948009244006243534386157989501953125f,-0.25950000061839795240103967444156296551227569580078125f,0.0541999985456464994371117427363060414791107177734375f,0.6705999709159138344460870939656160771846771240234375f,1.0214000117838377690304696443490684032440185546875f,0.07300000008940710216620573191903531551361083984375f,-0.4253999832183124052420453153899870812892913818359375f,-1.3266000035047529337361993384547531604766845703125f,-0.534299983751773677198571022017858922481536865234375f,0.2110000029727816528435369036742486059665679931640625f,-0.8038000027358531696108912001363933086395263671875f,-0.4093999682128435235739516429021023213863372802734375f,-0.659899970574677308121636087889783084392547607421875f,-0.845499995321034703010809607803821563720703125f,0.442699989634751656097932936972938477993011474609375f,-0.0938999879792332681205380140454508364200592041015625f,-0.516199984076619156070364624611102044582366943359375f,-0.6278000034511086457911233082995750010013580322265625f,-0.0967000021219253103055280007538385689258575439453125f,-0.10730001230239860632309500942938029766082763671875f,0.206200004565715744320186786353588104248046875f,0.34009999988377093149694019302842207252979278564453125f,0.78930000997781757376969835604541003704071044921875f,0.4485999949216843418753342120908200740814208984375f,-0.57220000299364326412643322328221984207630157470703125f,0.51659999516010268738597233095788396894931793212890625f,0.1541999865502121647153899175464175641536712646484375f,-0.5120999886929988864636698053800500929355621337890625f,-0.68720001413226139419521132367663085460662841796875f,-0.624200023281574534195215164800174534320831298828125f,0.095600000606477220088663671049289405345916748046875f,0.229299971532821444952787715010344982147216796875f,0.49600000494718532451088321977294981479644775390625f,0.3269000116869806848995949621894396841526031494140625f,0.3264000107407571960038694669492542743682861328125f,-0.1370000164508820006403766456060111522674560546875f,-0.5224000030592084176106482118484564125537872314453125f,-0.5732000138640405140222355839796364307403564453125f,-0.47120001171827308183992499834857881069183349609375f,0.1709000081554055139321235401439480483531951904296875f,-0.0606999927043911924329222529195249080657958984375f,-0.36940002498626700599970718030817806720733642578125f,-0.4304000058308243836791007197462022304534912109375f,-0.1909999985396861887920749722979962825775146484375f,0.021400009846687328973757757921703159809112548828125f,0.25249999701976777455314504550187848508358001708984375f,0.57640000448226924589789632591418921947479248046875f,0.358399999773502120348211974487639963626861572265625f,-0.30140000276863576633701313767232932150363922119140625f,-0.475899986064433733190526254475116729736328125f,-0.0243999674677848332748908433131873607635498046875f,0.705699989612400546690196279087103903293609619140625f,0.350300019693374764528925879858434200286865234375f,0.1136000053524970354601464350707828998565673828125f,-0.2759000086024403497475532276439480483531951904296875f,-1.0368000224947930831831399700604379177093505859375f,-0.54840000513792031000548377051018178462982177734375f,0.6261000071033835379097354234545491635799407958984375f,-0.6053000393629073272450114018283784389495849609375f,0.4103000170111652522564327227883040904998779296875f,0.70009997067749518695478627705597318708896636962890625f,-0.5007999775752427495945084956474602222442626953125f,-0.2096000017315144958729433710686862468719482421875f,-0.52749997451901453615619175252504646778106689453125f,-0.2346999961763620579091593754128552973270416259765625f,-0.3047000056385993360663633211515843868255615234375f,-0.10129999379515641777516066213138401508331298828125f,-0.187199984814226727536379257799126207828521728515625f,-1.059600006723403708974728942848742008209228515625f,0.21589998893290773818165462216711603105068206787109375f,0.008400002828240327179543101010494865477085113525390625f,-0.466799991165101513246327158412896096706390380859375f,0.5084999967515468899392772073042578995227813720703125f,-0.76210001022517648294751779758371412754058837890625f,-0.463700041371584603666633483953773975372314453125f,-0.729199976640939606653546434245072305202484130859375f,0.4819999667555097122573215528973378241062164306640625f,0.4551999969363205167383057414554059505462646484375f,0.62089998487234066004702981445007026195526123046875f,0.441900001516938090162511798553168773651123046875f,0.42030003101825741396169178187847137451171875f,0.33699999052286155887259155861102044582366943359375f,-0.40030000259429210363038009745650924742221832275390625f,-0.03230000902414342789370493846945464611053466796875f,0.641699996310471920679674440179951488971710205078125f,-0.62839997038096218062719344743527472019195556640625f,-0.8511999935388558924387325532734394073486328125f,-0.2428999931275843682243475996074266731739044189453125f,-0.17139999483376744393581248004920780658721923828125f,-0.101000008076429281800301396287977695465087890625f,-0.109999997168779284351103342487476766109466552734375f,0.406999984338879794432131120629492215812206268310546875f,0.059799994122981647848291686386801302433013916015625f,-0.40099999436736144531323589035309851169586181640625f,0.66580000704228847752119690994732081890106201171875f,1.44860004081726145841457764618098735809326171875f,0.147199991482496184147521489649079740047454833984375f,0.41260000204443925131414516727090813219547271728515625f,0.30980001260042211441714243846945464611053466796875f,-0.0003999941706656073847625521011650562286376953125f,-0.89609999659806482963375628969515673816204071044921875f,-0.37770002793073675917412401759065687656402587890625f,-0.6382999625623231310100891278125345706939697265625f,-0.0461000014595687457319428403934580273926258087158203125f,-0.96020001302957513900082631153054535388946533203125f,-0.6738000112295150501040552626363933086395263671875f,0.47040000113695856764906011449056677520275115966796875f,-0.39529999838471407969109350233338773250579833984375f,-0.569400021111964971254337797290645539760589599609375f,-0.2381999989256262661996288443333469331264495849609375f,-0.35979999337792367697375084389932453632354736328125f,-0.126399993753433026455468279891647398471832275390625f,0.261099992649257206522861451958306133747100830078125f,-0.32760000517368315087196606327779591083526611328125f,-0.240800003838538945188929574214853346347808837890625f,0.12350000386685133779707257417612709105014801025390625f,0.172600000703334810481237582280300557613372802734375f,0.1258000021994114536738607057486660778522491455078125f,-0.060099998393654818773512715779361315071582794189453125f,-0.12549999038502590575916428861091844737529754638671875f,-0.250599997179210054465414714286453090608119964599609375f,-0.1084999979250132751129598318584612570703029632568359375f,-0.70189999846220008095087905530817806720733642578125f,-0.3515999906897544491357621154747903347015380859375f,0.48989999553412200139490551009657792747020721435546875f,0.09259998639821986898823524825274944305419921875f,0.02039998522996899765757916611619293689727783203125f,-0.1778999933138489808470694697462022304534912109375f,0.087300001275539163003713838406838476657867431640625f,0.35860002382993716452119770110584795475006103515625f,0.269000002838671259386416068082326091825962066650390625f,0.32870001335740095527171433786861598491668701171875f,0.316800023090839655282024978077970445156097412109375f,0.049200001004338300703011555015109479427337646484375f,-0.946900002038479016164274071343243122100830078125f,-0.79780001105070130762442204286344349384307861328125f,0.2334999973475933021660466692992486059665679931640625f,0.00059999095797558243958746970747597515583038330078125f,0.2870000047534702058982247763196937739849090576171875f,0.319899994677305177237514044463750906288623809814453125f,0.3070000030398372103945803246460855007171630859375f,-0.172200033354759085568730370141565799713134765625f,-0.6846000025138259115209393712575547397136688232421875f,0.6102999824404715667469645268283784389495849609375f,0.266399970805645036620035170926712453365325927734375f,-0.59099999049305917964147738530300557613372802734375f,1.081100043834746227133791762753389775753021240234375f,0.05519999997243285605463825049810111522674560546875f,0.35210000106096206451411489979363977909088134765625f,-0.312699991720915004833614148083142936229705810546875f,0.477700017201900806185221881605684757232666015625f,0.6920999852657316875337301098625175654888153076171875f,-0.8103999759539963765320180755225010216236114501953125f,-0.8036000069916244825662943185307085514068603515625f,-0.1353999880611900241689227186725474894046783447265625f,-0.49909996703416148466914137316052801907062530517578125f,-0.43269998963475142517154381494037806987762451171875f,0.1341999969065188480499273282475769519805908203125f,-0.270899989752471592741045469665550626814365386962890625f,-0.088499993622302763895959287765435874462127685546875f,-0.0667000004082918263037527140113525092601776123046875f,0.1837999999418854224497721361331059597432613372802734375f,0.1102000096023083575147438750718720257282257080078125f,0.37209997518882198530576488337828777730464935302734375f,0.048099998054653425594295867995242588222026824951171875f,0.228499996844679031937630497850477695465087890625f,-0.02579999862313275826863900874741375446319580078125f,-0.31479999024868021706424769945442676544189453125f,0.19119999622106575998259359039366245269775390625f,0.02920000450611137665646310779266059398651123046875f,-0.51710000355690699525013087622937746345996856689453125f,-0.88270002852678342009085099562071263790130615234375f,-0.628299994301795283746514542144723236560821533203125f,-0.12899999807029960408044644282199442386627197265625f,-0.339000019341707314879386103712022304534912109375f,-0.2566000088691711145116869374760426580905914306640625f,0.85930000256448957340893457512720488011837005615234375f,0.98210004352927260384831242845393717288970947265625f,0.586899984306097355357678679865784943103790283203125f,-0.840500003367662174014185438863933086395263671875f,-0.32650004571676305431537912227213382720947265625f,-0.56889996868222991732721993685117922723293304443359375f,-0.0204000003546476182947344568674452602863311767578125f,-0.08280000117868180353752904920838773250579833984375f,0.62570000104904099913483150885440409183502197265625f,-0.5448999682277444467359828195185400545597076416015625f,-1.0792999828577034548970914329402148723602294921875f,0.0634000122904776475252219825051724910736083984375f,-0.52929999451786269304420784465037286281585693359375f,-1.398500014185905637731366368825547397136688232421875f,-0.0210000060833992474584874798892997205257415771484375f,-0.3443999926134944100653001441969536244869232177734375f,0.41580001039505010140828744624741375446319580078125f,0.06559998872280115023158941767178475856781005859375f,-0.49940000464618206077460627057007513940334320068359375f,0.45850000256299938694581896925228647887706756591796875f,-0.1242000163525338596315350514487363398075103759765625f,-0.73949998381733894614598057160037569701671600341796875f,-0.3709999949261544305301185886492021381855010986328125f,-0.24040000095069380403600689533050172030925750732421875f,0.108999996133148702170956312329508364200592041015625f,-0.2417000006318090754575678147375583648681640625f,-0.41560002329349554628379337373189628124237060546875f,-0.1838000074297190256089606918976642191410064697265625f,0.32389999375343325738185740192420780658721923828125f,0.209399991607666091653072726330719888210296630859375f,-0.21209999644160271170534315388067625463008880615234375f,0.43980001558065406896957938442938029766082763671875f,0.0551999975323678171434949035756289958953857421875f,-0.50190000066012141477500563269131816923618316650390625f,-0.7577999854207035923536750487983226776123046875f,-0.584599990034103456082448246888816356658935546875f,0.421999990113079537223939041723497211933135986328125f,-1.13580002589225781406412352225743234157562255859375f,-0.69460000821351997046804171986877918243408203125f,0.65470001092851159985031017640721984207630157470703125f,-0.128400007335841781497265401412732899188995361328125f,-0.154200016464293432338905631695524789392948150634765625f,-0.0969000045530498399504182316377409733831882476806640625f,0.1305999999359250818287137008155696094036102294921875f,0.17820000529587265436504139870521612465381622314453125f,0.067199998784065240187146628159098327159881591796875f,0.283599986875056941926231957040727138519287109375f,0.37879999930858598844451989862136542797088623046875f,-0.0211999958485364903282288651098497211933135986328125f,-0.6282999951958654349937205552123486995697021484375f,-0.22179999060630795071347165503539144992828369140625f,0.4671999895825982296315714847878552973270416259765625f,-0.273099989318847491404085303656756877899169921875f,0.015800027978420505547774155274964869022369384765625f,0.4851999980166554760074859586893580853939056396484375f,0.471599985809624033805675935582257807254791259765625f,0.184799981866777096595200191586627624928951263427734375f,-0.4310999872662126808364746466395445168018341064453125f,-0.3108000145673752712127679842524230480194091796875f,-0.39739998886585237158897143672220408916473388671875f,-0.10380001040995090466623196334694512188434600830078125f,-0.419999993182718522888308143592439591884613037109375f,0.125700002222508044891213785376749001443386077880859375f,-0.7156999935984611482808759319595992565155029296875f,-0.285299977374076974001582129858434200286865234375f,-0.1905999963223934390299518781830556690692901611328125f,-0.53539998955130574387339947861619293689727783203125f,-0.11579999676048746692913482547737658023834228515625f,-0.058400002753734636673499380776775069534778594970703125f,0.18260000457763680969947017729282379150390625f,0.633599977040290696095325984060764312744140625f,0.54490000197887411559349857270717620849609375f,-1.31809998693466212671410175971686840057373046875f,0.1047000351428977182877133600413799285888671875f,-0.09119998295903197771394843584857881069183349609375f,0.414400010681152508595914696343243122100830078125f,0.8533999905347826597790117375552654266357421875f,0.148499988853931341736824833787977695465087890625f,0.68180001400113088294574481551535427570343017578125f,0.2211999796807759821604122407734394073486328125f,0.2539000058233737977531063734204508364200592041015625f,0.06199999380111709257334950962103903293609619140625f,-0.272200005042552817258183495141565799713134765625f,0.142900001174211421783866171608678996562957763671875f,-0.0279000311255455102354972041212022304534912109375f,-0.8594999876916400438631171709857881069183349609375f,-0.57460000806450828303439948285813443362712860107421875f,0.13740000585317613257529956172220408916473388671875f,-0.613799999308586308188750990666449069976806640625f,-0.062099996590614381375416996888816356658935546875f,0.01060000198483468381027705618180334568023681640625f,0.76379997852146619408131300588138401508331298828125f,0.472100000613927817116888263626606203615665435791015625f,-0.69810002746582000554553815163671970367431640625f,-0.072699961471556662218063138425350189208984375f,-0.439200009572505933874708716757595539093017578125f,-0.479800004851818062689972066436894237995147705078125f,-0.1841999975025654379123807302676141262054443359375f,0.1246000057548284356556678176275454461574554443359375f,-0.4189000049665569935797293510404415428638458251953125f,-0.658299986366927214476163499057292938232421875f,-0.392700005318224409922578388432157225906848907470703125f,0.20129997963905310598420328460633754730224609375f,-0.596200005161762192074093036353588104248046875f,0.7436000005841254534288964350707828998565673828125f,0.2162000266194343822689916123636066913604736328125f,-0.6957999606251714652671580552123486995697021484375f,-0.26160001021027579071187574299983680248260498046875f,0.47440000308156005104365249280817806720733642578125f,0.083200019377470457726531094522215425968170166015625f,-0.66579998834133125029666189220733940601348876953125f,0.36999998509883891273375411401502788066864013671875f,1.1574999710917470707727261469699442386627197265625f,-0.2496000022530555639832527958787977695465087890625f,-0.440000005066394805908203125f,0.300799966138601604370705899782478809356689453125f,-0.482400024330616350454192797769792377948760986328125f,1.079000006377696774251262468169443309307098388671875f,0.8584000435829164388223944115452468395233154296875f,-0.26340001407861723947689824854023754596710205078125f,0.04959998914003360681590493186376988887786865234375f,-0.11809997531175664420288740075193345546722412109375f,0.49599998442083570449057106088730506598949432373046875f,-0.43979999032318606300151486721006222069263458251953125f,-0.13599998578056737841279755230061709880828857421875f,0.216999998420476902794007401098497211933135986328125f,-0.349999993592500668881939418497495353221893310546875f,-0.51749999940395330355613623396493494510650634765625f,0.588800029110908784701905460678972303867340087890625f,0.019599998378753813454977716901339590549468994140625f,-0.64400001442432408538252275320701301097869873046875f,-0.08840001333355917978451543604023754596710205078125f,0.34260000025629988584796592476777732372283935546875f,-0.1377999556183819862553718849085271358489990234375f,0.5752000157117844736376355285756289958953857421875f,-0.6980000067949294617619671043939888477325439453125f,-1.186800026965141086066068965010344982147216796875f,-0.0893999986112117284164924058131873607635498046875f,0.7134000032007694613866988220252096652984619140625f,0.008900040245055862442313809879124164581298828125f,-0.6722000245630741499525129256653599441051483154296875f,-0.1123999848425387693140464762109331786632537841796875f,0.3363000310480597221385323791764676570892333984375f,-0.12860000311732289191013478557579219341278076171875f,0.06500000223517421460428522550500929355621337890625f,-0.401099981361627389020441114553250372409820556640625f,-0.2466000051438808071679886779747903347015380859375f,-0.27390000433325756290514618740417063236236572265625f,-0.598199969458580227410493534989655017852783203125f,1.1873999858856194578038412146270275115966796875f,0.463900004631280893363509676419198513031005859375f,-0.06770001630783095691867856658063828945159912109375f,-0.037300020349025686527966172434389591217041015625f,0.236700004655122864249960912275128066539764404296875f,1.17559997382164027612816425971686840057373046875f,-0.613399996495246568883885629475116729736328125f,-0.1067000006318092886203885427676141262054443359375f,0.33279998810291289146334747783839702606201171875f,-0.655999981701373968689949833787977695465087890625f,-0.4167999995470046403767128140316344797611236572265625f,0.7825999971270558575042741722427308559417724609375f,0.1381999351859093394523370079696178436279296875f,0.474699996548891167691408554674126207828521728515625f,-0.2845999912261962805359871708787977695465087890625f,-0.3616000130414960267444257624447345733642578125f,-0.025500005900859878948949699406512081623077392578125f,0.10310005123317222963663652990362606942653656005859375f,0.401899981586634869046292806160636246204376220703125f,-0.137400013545900645528519135041278786957263946533203125f,-0.541499993860721673399893916212022304534912109375f,0.023500015079974900800152681767940521240234375f,-0.101999995887279482076337444595992565155029296875f,-0.57880001450777029958771890960633754730224609375f,0.82360001906156554696281091310083866119384765625f,0.277000003635883285113550300593487918376922607421875f,-0.26349997961521154365982511080801486968994140625f,0.103699975657463028255733661353588104248046875f,0.2285999962627887072130761225707828998565673828125f,-1.2180000157356261070162872783839702606201171875f,0.24280003638267544374684803187847137451171875f,-0.139799997103214224125622422434389591217041015625f,-0.113699995923042251888546161353588104248046875f,0.3804999799728392417819122783839702606201171875f,0.46090000290274613092833533301018178462982177734375f,0.14430002562403654309264311450533568859100341796875f,-1.0390999878585347460102639161050319671630859375f,-0.3203999889552593405284142136224545538425445556640625f,-0.59750000476837161755838678800500929355621337890625f,-0.176899991309642956593961571343243122100830078125f,0.06490000793933869971397143672220408916473388671875f,-0.6929000189065934733889662311412394046783447265625f,0.730799981486796923491056077182292938232421875f,-0.453599996411800265860847503063268959522247314453125f,-0.004000015318393668195540158194489777088165283203125f,0.15159998681545250320823470246978104114532470703125f,-0.04019999812841401620744363754056394100189208984375f,0.60700001794099822660655263462103903293609619140625f,0.090700043964385912431680480949580669403076171875f,0.15120000933408750398712072637863457202911376953125f,-0.49620000277757636553133124834857881069183349609375f,-0.8136000199556345791052081040106713771820068359375f,0.226600007528066527129340101964771747589111328125f,0.361599997991323363066840101964771747589111328125f,0.5568999734282495950310476473532617092132568359375f,0.55120001350641256721019090036861598491668701171875f,0.03960000225305559951038958388380706310272216796875f,-0.231600002312660535608301870524883270263671875f,-0.6242000247716905647621388197876513004302978515625f,0.0100000110268592123929920489899814128875732421875f,0.959900000935793418221919637289829552173614501953125f,-0.074400005875527863707930009695701301097869873046875f,-0.4606000056728720704057877810555510222911834716796875f,-0.5031999975286425286213898289133794605731964111328125f,-0.10620000754594816072540197637863457202911376953125f,-0.26060002418756500475183202070184051990509033203125f,-0.539899994826317453089359332807362079620361328125f,0.18720000310540196863939854665659368038177490234375f,-0.54160001706480986882752404198981821537017822265625f,-0.031600024962425976582380826584994792938232421875f,-0.28539998791217779938733656308613717555999755859375f,0.33140000507831590681462330394424498081207275390625f,0.744099986815452751898192218504846096038818359375f,-0.7471999972939489254031286691315472126007080078125f,0.1045999951004983330449249478988349437713623046875f,0.84759997835159328616327911731787025928497314453125f,-0.19180000684857390069737448357045650482177734375f,0.622800002259015883510073763318359851837158203125f,1.08129998917579683848089189268648624420166015625f,0.4236000040113925280138573725707828998565673828125f,0.2030000057518481781926311668939888477325439453125f,-0.51379998798370341006602757261134684085845947265625f,0.217399998253583692786605752189643681049346923828125f,0.359199984091520096995964195230044424533843994140625f,-0.1092999901592737188593673636205494403839111328125f,0.0232000056684018574060246464796364307403564453125f,0.237699994552135418501848107553087174892425537109375f,-0.010600023591518237253694678656756877899169921875f,-0.65619998549222913197809248231351375579833984375f,-0.156199997711181648440970093361102044582366943359375f,0.029800020051002462651013047434389591217041015625f,0.3835999925374977692626998759806156158447265625f,0.3906000094354151741526948171667754650115966796875f,-0.946200021553039505306514911353588104248046875f,-0.20339996252059933112832368351519107818603515625f,-0.178499990493059090823635415290482342243194580078125f,0.798899979448318475760970613919198513031005859375f,0.6398999703884129530706559307873249053955078125f,0.2329999918937684100228580064140260219573974609375f,-0.053799989771842859909156686626374721527099609375f,0.8181999944925308909660088829696178436279296875f,0.220399994766712126192942378111183643341064453125f,0.97620000903606385378452614531852304935455322265625f,-0.827200011599064310985340853221714496612548828125f,-0.312499997764825732105009592487476766109466552734375f,-0.645399994990229597391362403868697583675384521484375f,-0.482599979804456236109899691655300557613372802734375f,-0.2642999986529350753272638030466623604297637939453125f,-0.732200005936622488889042870141565799713134765625f,-0.38920000212192462640814483165740966796875f,-0.13140000969767573479884958942420780658721923828125f,0.109100011253357109808348468504846096038818359375f,-0.115699994492530322531820274889469146728515625f,-0.31170001463890084636432220577262341976165771484375f,0.811199989962577916458030813373625278472900390625f,-0.11779999167919186220387928187847137451171875f,0.03680000684857365200741696753539144992828369140625f,0.7593999778985978110767973703332245349884033203125f,-0.94979998071193705300174769945442676544189453125f,-0.26729999188780784180607952293939888477325439453125f,1.266099984788894516896107234060764312744140625f,0.25749995529651670267412555404007434844970703125f,0.214799992930889249720394218456931412220001220703125f,-0.5053000041961670518730898038484156131744384765625f,0.72349998795986181221451261080801486968994140625f,0.33580000100731854928426400874741375446319580078125f,-0.0853000062823294769032145268283784389495849609375f,0.567299961042404987665577209554612636566162109375f,0.3245999849677085880017557428800500929355621337890625f,-0.055300003004074227419550879858434200286865234375f,-0.65769999928325406646223427742370404303073883056640625f,-0.46260000874996176634112998726777732372283935546875f,-0.37760000040531149778644248726777732372283935546875f,-0.2424999943375585331750698969699442386627197265625f,-0.1862000047147274184311527278623543679714202880859375f,-0.0021999995291233087613136376603506505489349365234375f,0.258100000196695322784989912179298698902130126953125f,-0.80039998850822424714124281308613717555999755859375f,0.13759998878240597974809134029783308506011962890625f,1.243799944323301875925835702219046652317047119140625f,-0.22980000172257408763698549591936171054840087890625f,-0.33479999099373802806667299591936171054840087890625f,-0.47209999748468334246354061178863048553466796875f,0.194199992436170365550651695230044424533843994140625f,-0.725800024253130171558723304769955575466156005859375f,-0.458100020611286851135446340776979923248291015625f,0.180400005346536662642620285623706877231597900390625f,0.404800014835596133622175329946912825107574462890625f,0.358600004160404495223701815120875835418701171875f,0.719400002783536773875994185800664126873016357421875f,-0.068999997884035213502329497714526951313018798828125f,-0.933099985444545865220788982696831226348876953125f,0.3248000197529794519368806504644453525543212890625f,0.3068000183224679489057962200604379177093505859375f,0.52079997195005489629693329334259033203125f,-0.314800005745887734320831441436894237995147705078125f,0.331800010871887263164126125047914683818817138671875f,0.74300001126527792649767434340901672840118408203125f,0.43079999996423712360638091922737658023834228515625f,0.26460000940561290150299100787378847599029541015625f,0.2526000185847276924278048682026565074920654296875f,0.431200002181530095413108938373625278472900390625f,-0.853799994242191218063453561626374721527099609375f,-1.115199998128414193843127577565610408782958984375f,-0.61899999386072135365566282416693866252899169921875f,0.39119998906850828035430822637863457202911376953125f,0.259300009977817325079740840010344982147216796875f,0.105800001081824301962797107989899814128875732421875f,0.11840000163614750050555812777020037174224853515625f,-0.3582999868884682026504151508561335504055023193359375f,-0.483399988895654442178084764236700721085071563720703125f,0.214599988171458366803534545397269539535045623779296875f,0.50669998431503759928062891049194149672985076904296875f,0.6233999985814093758307308235089294612407684326171875f,0.13600000476092100054614775217487476766109466552734375f,-0.49160000052452090812948881648480892181396484375f,-0.9709999676942828727987944148480892181396484375f,-0.35769998948574066588435016456060111522674560546875f,0.254500003188848467061689007095992565155029296875f,-0.25290000087618835777902859263122081756591796875f,-0.09270000006556511351618610206060111522674560546875f,1.25419995205402390325843953178264200687408447265625f,-0.970199980247020477008845773525536060333251953125f,-0.317199993270635527409240239649079740047454833984375f,-0.691500007271766747862784541212022304534912109375f,-0.6180999613046651575132273137569427490234375f,-0.22779998303651805002800756483338773250579833984375f,-0.4294999946951865155142513685859739780426025390625f,-0.27410000022649683160125277936458587646484375f,-0.374300017875432988745387774542905390262603759765625f,0.5103000033020972381336832768283784389495849609375f,0.585899985468387285436620004475116729736328125f,0.277700002896785758110809183563105762004852294921875f,-0.8894999917149544188532672706060111522674560546875f,0.9270999873518945832984172739088535308837890625f,0.295899994856119175068442928022705018520355224609375f,0.80339998636245724128457368351519107818603515625f,0.74180000073909724278564681299030780792236328125f,0.229200007188320231676925686770118772983551025390625f,0.796000012099742804139168583787977695465087890625f,-0.16080001009702726832983898930251598358154296875f,0.028700002926588030760512992856092751026153564453125f,0.50340003404617306159707368351519107818603515625f,0.8441999699354170161313959397375583648681640625f,0.26969998969435682312223434564657509326934814453125f,-1.17219997732639313881009002216160297393798828125f,-0.074199958610535077241365797817707061767578125f,-0.29849999466538434944595792330801486968994140625f,0.008199980783462734734712284989655017852783203125f,-0.6067999876260756764168036170303821563720703125f,0.050700007605552599443399230949580669403076171875f,-0.438000002771615992713805098901502788066864013671875f,0.20939999868571756991286747506819665431976318359375f,0.163300002869963656326746104241465218365192413330078125f,0.418999977357685793322161771357059478759765625f,-0.51279998612850885564284908468835055828094482421875f,-0.066399996286630680319973407677025534212589263916015625f,-0.82439998013377202568108259583823382854461669921875f,-0.9064999911785118769103064551018178462982177734375f,-0.584200001674890501135450904257595539093017578125f,-0.573800022703409240421024151146411895751953125f,0.5622999422669412439290681504644453525543212890625f,0.278599980467557628571739769540727138519287109375f,-1.011600003653764900946043781004846096038818359375f,-0.6259000063300135252575273625552654266357421875f,0.2582000068232417344660234448383562266826629638671875f,-0.3784000044301152332337778716464526951313018798828125f,-0.4569000142760575311484672056394629180431365966796875f,0.30319999330043767571396529092453420162200927734375f,0.12279999763965587789016353781335055828094482421875f,0.3460000001788134937896757037378847599029541015625f,0.859000004664063343540192363434471189975738525390625f,0.205000009909272085106834992984659038484096527099609375f,-0.594899983464181492109901228104718029499053955078125f,-0.123600000882148730596554742078296840190887451171875f,0.157799998980760580735704934340901672840118408203125f,0.0910000071078538308455563310417346656322479248046875f,-0.59140000496655698025705305553856305778026580810546875f,0.38919999400079252627193682201323099434375762939453125f,0.47630001203045235769195642205886542797088623046875f,0.5407999929606912559165721177123486995697021484375f,-0.5621999834358692993419026606716215610504150390625f,-0.4232000162482254523865776718594133853912353515625f,-0.6776000288665298132428915778291411697864532470703125f,-0.1600000159442427349887339005363173782825469970703125f,0.5735999950706958117052636225707828998565673828125f,-0.262399999892711566218395091709680855274200439453125f,0.069599996441602751673372040386311709880828857421875f,0.1804000019937752608001346743549220263957977294921875f,-0.2374000040277838563529400062179774977266788482666015625f,-0.0037999995693564214693793701371760107576847076416015625f,0.1973000010706484896871870660106651484966278076171875f,-0.30500001370906826281270696199499070644378662109375f,-0.22679998404979695436622932902537286281585693359375f,-0.265299974691867390674815396778285503387451171875f,-0.449600011193752369109688515891321003437042236328125f,0.044200002121925319187312197755090892314910888671875f,0.701999990969896625614410368143580853939056396484375f,0.10359999343156811590915822307579219341278076171875f,-0.46339999858140945576678859652020037174224853515625f,0.00900000220537133799325602012686431407928466796875f,0.1146000134289266725318157114088535308837890625f,0.57850001165270814595942283631302416324615478515625f,0.538099967265129208726648357696831226348876953125f,-1.22089997622966706103397882543504238128662109375f,-0.643100003772973938254153836169280111789703369140625f,-0.5782999698638917607240728102624416351318359375f,-0.3692999906063079151863348670303821563720703125f,-0.52120001052618025738638607435859739780426025390625f,-0.43890000211670987528833620672230608761310577392578125f,0.068900016570836442042491398751735687255859375f,-0.1182000009186565947505442863985081203281879425048828125f,0.58400001680850976981673738919198513031005859375f,0.56319996170997654871825943700969219207763671875f,0.122199983584880822462537253159098327159881591796875f,-0.4097999860391021709205006118281744420528411865234375f,1.35759997086375960861914791166782379150390625f,0.475199992950260663437944685938418842852115631103515625f,0.044099998438358323227248547482304275035858154296875f,-0.0607000035822391481588056194595992565155029296875f,0.0590000007897615308394279054482467472553253173828125f,0.30799998898804192837275195415713824331760406494140625f,-0.549000004999339541456038205069489777088165283203125f,0.026100003359466772412389445889857597649097442626953125f,-0.241099989557266525252998690120875835418701171875f,0.3553000041961666966017219237983226776123046875f,-0.1667999993979929485021784785203635692596435546875f,0.46089999366402611968851488200016319751739501953125f,-1.060700020867585635642171837389469146728515625f,-0.3887000100791453416348986138473264873027801513671875f,0.107899997150898041553546136128716170787811279296875f,-0.3200999945938587387672669137828052043914794921875f,-0.0544999980479479173567369798547588288784027099609375f,0.15050000003352759225805357345961965620517730712890625f,-0.3344999961294234935849090106785297393798828125f,-0.0998999977134168160031180150326690636575222015380859375f,-0.465999983787536820045716012828052043914794921875f,-0.63339997622966848211945034563541412353515625f,-0.20549998563528060202543201739899814128875732421875f,0.71279999555349338180576523882336914539337158203125f,-0.568400005435943711518120835535228252410888671875f,-0.20129999901056283562184034963138401508331298828125f,0.1146000341415405188172371708787977695465087890625f,-0.178999992966652143877581693232059478759765625f,-0.288300006669759856237078565754927694797515869140625f,0.9148000012755392873486925964243710041046142578125f,0.17450004178285549727434045053087174892425537109375f,0.017800006580352967233693561865948140621185302734375f,-0.499799989652633502856815539416857063770294189453125f,-0.0987000237882131870037483167834579944610595703125f,-0.6164000099956989942029395024292171001434326171875f,-0.2042000082314014974826932302676141262054443359375f,0.9237999662280085289012276916764676570892333984375f,-0.12700000199675542944532935507595539093017578125f,-0.5451999887406824285562834120355546474456787109375f,0.184099995905160795928168226964771747589111328125f,-0.680999998837709341614754521287977695465087890625f,-0.097000011831521959493329632095992565155029296875f,0.89949995476007504890958443866111338138580322265625f,-0.5778000134348868499500895268283784389495849609375f,-0.1711999914526938226799757103435695171356201171875f,0.1103000193953516117062463308684527873992919921875f,-0.457599987143277875389912878745235502719879150390625f,0.203199986296892287640503127477131783962249755859375f,0.610799980294704170091790729202330112457275390625f,-0.842800032806396703932705349870957434177398681640625f,-0.041399992561340415164750083931721746921539306640625f,0.7598000182628634746606621774844825267791748046875f,-0.432400016433000633497840681229718029499053955078125f,0.120800007265806197409574451739899814128875732421875f,0.139400027072429821828336571343243122100830078125f,0.45580000950098042977742807124741375446319580078125f,-0.39020001064538956114802203956060111522674560546875f,-1.24129999781847022433112215367145836353302001953125f,-0.06439999741911872632726954179815948009490966796875f,-0.20119998802542671256787798483856022357940673828125f,-0.5893999956309787791042253957130014896392822265625f,-0.48359999417662624665581461158581078052520751953125f,0.52819997974038113142114525544457137584686279296875f,1.2909999590516090250957859097979962825775146484375f,-0.109600007170438829007252934388816356658935546875f,-0.4814000120818615613416113774292171001434326171875f,0.366799986284970458427778794430196285247802734375f,-0.0154000022172926520624969271011650562286376953125f,-0.62979998905658707286647768341936171054840087890625f,-0.73969999826252408325899523333646357059478759765625f,-0.44140001193285005598454517894424498081207275390625f,-0.36420001851320282781898640678264200687408447265625f,0.6244999651908873516958919935859739780426025390625f,-0.330800007116794692052508253254927694797515869140625f,-0.0653000028550625444268007413484156131744384765625f,-0.0634000009655952823095503845252096652984619140625f,-0.340399961686134844285334111191332340240478515625f,-0.3225999892294406645731896787765435874462127685546875f,0.957299984884262045170544297434389591217041015625f,-0.9722999968051908581401221454143524169921875f,-0.45160000246167175674827376496978104114532470703125f,0.2481000316381454240399762056767940521240234375f,-0.561299966824055474035048973746597766876220703125f,-0.4363999946475030089487745499354787170886993408203125f,0.8824999648332596535738048260100185871124267578125f,-0.99249998509883852193524944595992565155029296875f,-0.393499997347593222229988896287977695465087890625f,-0.103399986958503831147027085535228252410888671875f,-0.961999975919724192863213829696178436279296875f,-0.30069998793601993014590334496460855007171630859375f,0.5698999796271326800933820777572691440582275390625f,-0.4740999877095219972034101374447345733642578125f,-0.081699998843669963122238186770118772983551025390625f,0.6160000144839283819919728557579219341278076171875f,-1.2024000076413159376897965557873249053955078125f,-0.382299996358156146669671215931884944438934326171875f,-0.0314999902844430579307299922220408916473388671875f,-0.536700001823902539399568922817707061767578125f,-0.062899993574619283975835060118697583675384521484375f,-1.3818000066995619334875300410203635692596435546875f,0.28320002012252842860107193700969219207763671875f,0.04399999997019765629602261469699442386627197265625f,-0.78510001113414773499243892729282379150390625f,-0.289499965488911215061307302676141262054443359375f,-0.0714999819397925673314375671907328069210052490234375f,1.4182999585390092533998540602624416351318359375f,-0.2209000286817541791606345213949680328369140625f,0.03519998963475234887710030307061970233917236328125f,0.49010002424716958557837642729282379150390625f,-0.474299998652934817755522090010344982147216796875f,-0.4140000136792660878626293197157792747020721435546875f,-0.9104999939799309771615298814140260219573974609375f,-0.5374999542534357743761574965901672840118408203125f,-0.32889998496174810416192713091732002794742584228515625f,0.66219998686313630287259002216160297393798828125f,-0.855600004220008258926100097596645355224609375f,-0.4672000108540057539130430086515843868255615234375f,1.0881999924063681106645162799395620822906494140625f,-0.14959996783137352593939795042388141155242919921875f,0.3207999938547609897199208717211149632930755615234375f,-0.1159999919831753434351639953092671930789947509765625f,-0.4887999866425989381468752981163561344146728515625f,0.205399992829561117702041883603669703006744384765625f,-0.898000005453825078660656799911521375179290771484375f,-0.5753000232696532378895426518283784389495849609375f,-0.41240000767856843655323473285534419119358062744140625f,0.97860001425594089763393412795267067849636077880859375f,1.0162000306986274100751188598223961889743804931640625f,0.4084000237643721220592851750552654266357421875f,-0.17580000756382929694154881872236728668212890625f,0.1585999743580810417142856749705970287322998046875f,0.58860002233982111619070565211586654186248779296875f,-0.00840000424385056732035081950016319751739501953125f,-1.01849998259544438639068175689317286014556884765625f,-0.327000011980533855648900498636066913604736328125f,0.729599986159801261464963317848742008209228515625f,0.4613000204682347060725078335963189601898193359375f,-0.001799998503923205817045527510344982147216796875f,-0.613999988347291747459166799671947956085205078125f,0.185899994409084001745213754475116729736328125f,0.71759999306648947214881673062336631119251251220703125f,-0.27539998485744010547904281338560394942760467529296875f,-0.9875999840326603607110200755414552986621856689453125f,0.227200005489587741891455152654089033603668212890625f,0.562800006729364366009349396335892379283905029296875f,0.34880000728070736926156314439140260219573974609375f,-0.596400008207559562123378782416693866252899169921875f,0.339199989902973175759370860760100185871124267578125f,0.3946000230401749409026024295599199831485748291015625f,0.25020001541376135634209276759065687656402587890625f,-0.91020000289678559823869363754056394100189208984375f,-0.3985000505447384711033009807579219341278076171875f,0.49720000921487805811693760915659368038177490234375f,-0.73600002163648614583735252381302416324615478515625f,-0.7314000301122671743314640480093657970428466796875f,-0.148800004822015718985994681133888661861419677734375f,-0.433800001245737032462557181133888661861419677734375f,0.0263999955415723519536186358891427516937255859375f,0.6355999920010564352423898526467382907867431640625f,-0.1651999820351601755419324035756289958953857421875f,-0.46939998952150308042519100126810371875762939453125f,-0.5604000075817108239562003291212022304534912109375f,0.2627999997258185782555983678321354091167449951171875f,-0.48269998933672919605442075408063828945159912109375f,0.346099994176626069020130671560764312744140625f,0.0841000000774860811958433259860612452030181884765625f,-0.5588999940514565167859473149292171001434326171875f,-0.391099977338313919972279109060764312744140625f,-0.472000003933906508990503425593487918376922607421875f,1.035500003423541759417503271833993494510650634765625f,0.1826999626077707716831355355679988861083984375f,0.40360000376924876075435122402268461883068084716796875f,0.420500012159347846818491234444081783294677734375f,0.4877000115394594104145653545856475830078125f,-0.02460000895857827885038204840384423732757568359375f,-1.0703999908924100026297310250811278820037841796875f,0.77100001716613775215591886080801486968994140625f,0.3117000009298322993345209397375583648681640625f,0.371399984812736505546126863919198513031005859375f,0.081999950885773387199151329696178436279296875f,0.33789998970031742686614961712621152400970458984375f,0.3169000404834747541826800443232059478759765625f,-0.4264999649524696678781765513122081756591796875f,-0.44739999691247955837525296374224126338958740234375f,-0.93809998507201697304225262996624223887920379638671875f,0.771899992837011605928410062915645539760589599609375f,0.162000000525266030226845259676338173449039459228515625f,0.5005999994516372453290387056767940521240234375f,0.7669999707043171355280719581060111522674560546875f,0.40079999668598176310041480974177829921245574951171875f,0.203600001776218508808824481093324720859527587890625f,-0.5992000079333781314971929532475769519805908203125f,0.0021999991565941900972802613978274166584014892578125f,-0.3030000091791151817233185283839702606201171875f,-0.318799949240684554752078838646411895751953125f,-0.49279999570250510032565216533839702606201171875f,-0.64820000045299508428797707892954349517822265625f,0.1402000145196922176182852126657962799072265625f,-0.25400001382827763762861650320701301097869873046875f,0.0165000159144401692401515902020037174224853515625f,-0.5270999813914301057593547739088535308837890625f,-0.343800000351667467413108170148916542530059814453125f,-0.3045999793052676096749564749188721179962158203125f,0.77499998390674562642743694595992565155029296875f,0.138999997735023583800284541212022304534912109375f,0.420799982380867021447556908242404460906982421875f,-0.7319000115752221091725004953332245349884033203125f,0.5396000161111356163701202603988349437713623046875f,-0.4163999997138976283395095379091799259185791015625f,-0.5926000208944082192630276040290482342243194580078125f,-0.065999997198581628055080727790482342243194580078125f,0.219199997204542196271859211265109479427337646484375f,-0.1157999845415356166000719895237125456333160400390625f,-0.46219999803900702062264826963655650615692138671875f,-0.26519999589323983713029520004056394100189208984375f,0.79329996569156691066382336430251598358154296875f,-0.38559999274611467257045660517178475856781005859375f,0.24839999127984047078143703402020037174224853515625f,0.93879996116161379404729814268648624420166015625f,-0.430400004452466955484624122618697583675384521484375f,0.060999996304512027478494928800500929355621337890625f,0.7107999882668256663720285359886474907398223876953125f,0.35020000975131981846288908855058252811431884765625f,-0.08200000274181373782766968361102044582366943359375f,0.265499958813190772843881859444081783294677734375f,-0.01060000001043082118457050455617718398571014404296875f,0.03059999911636113179014273555367253720760345458984375f,0.09349999878183001822407049985486082732677459716796875f,-0.34200000751018511113699105408159084618091583251953125f,-0.53439998981952652190585695279878564178943634033203125f,-0.05140000068247285724964967812411487102508544921875f,0.135999993324279699891121708787977695465087890625f,-0.291800008487701489912069519050419330596923828125f,-0.6451999820351606018675738596357405185699462890625f,-0.45279998482465710907263201079331338405609130859375f,-0.04700000661611536401096600457094609737396240234375f,-0.00600002461671778064555837772786617279052734375f,0.6163999975159761834930804980103857815265655517578125f,0.2846000009492037907676831309800036251544952392578125f,-0.3221999856524171423899360888754017651081085205078125f,0.52660002004504224970560244400985538959503173828125f,-0.31260001396536818418780967476777732372283935546875f,-0.984699998337031257022999852779321372509002685546875f,-0.36139999926686261488839591038413345813751220703125f,-0.42299999263882614997100972686894237995147705078125f,-0.1223000009775159213631923194043338298797607421875f,-0.2961999949544666943523907320923171937465667724609375f,0.1777999945104122847538974383496679365634918212890625f,0.15069999721199278663874565609148703515529632568359375f,-0.635800012481212650783390927244909107685089111328125f,-0.45270001541376103659786167554557323455810546875f,-0.482600000405311480022874093265272676944732666015625f,0.4434000222742557895116988220252096652984619140625f,-0.0429999997168779568568197646527551114559173583984375f,-0.209500000059604563062976012588478624820709228515625f,0.0500999987661838730446106637828052043914794921875f,0.147500001415610260124822161742486059665679931640625f,0.863899991965293878593001863919198513031005859375f,-0.80069997363090550379638443700969219207763671875f,-0.429799992930889107611847066436894237995147705078125f,1.031499966144561852843253291212022304534912109375f,-0.3077000049829479166874079965054988861083984375f,-0.07710000106096259742116671986877918243408203125f,0.731399995839595806756960882921703159809112548828125f,-0.5274000018298625747092955862171947956085205078125f,-0.1855000012069940584780169956502504646778106689453125f,0.523599981212615972481216886080801486968994140625f,0.4785999901533131151154520921409130096435546875f,0.00729999293088923195682582445442676544189453125f,0.1150000004470348446972138845012523233890533447265625f,0.014399996934831182926473047700710594654083251953125f,0.0043999985739588687305712255692924372851848602294921875f,-0.164499985978007412512624796363525092601776123046875f,-0.889699961009622342089642188511788845062255859375f,-0.45749998800456526115709721125313080847263336181640625f,-0.754799984288215597416638047434389591217041015625f,-0.0647999979972837536479346454143524169921875f,-0.06730000559687621120019684894941747188568115234375f,-0.213400038814544501519776531495153903961181640625f,-0.6993999588251114118975237943232059478759765625f,-0.1626999845683574363874868140555918216705322265625f,-0.17189999278113254899125195152009837329387664794921875f,1.081699958442897013810579665005207061767578125f,0.419999987799674290300799839315004646778106689453125f,-1.01889999256134000660267702187411487102508544921875f,0.464900021946430541674999403767287731170654296875f,0.080699995386600331670479135937057435512542724609375f,-0.28029999346733092124850372783839702606201171875f,-0.844499953866005625968682579696178436279296875f,-0.3766999852836132589573026052676141262054443359375f,0.0401999836742878979833903940743766725063323974609375f,-0.29479999181330196478256766567938029766082763671875f,0.01400000470131634866532976957387290894985198974609375f,0.4589999926686285647292606881819665431976318359375f,0.4938999930083749489995170733891427516937255859375f,-0.185600006082653923300540554919280111789703369140625f,-0.60459999904930583625173312611877918243408203125f,-0.374000001497566625374702198314480483531951904296875f,0.003399992658197792394503267132677137851715087890625f,-0.0980000133141876172970796687877736985683441162109375f,-0.4074999762885280762958473133039660751819610595703125f,-0.49059999885559069099372209166176617145538330078125f,0.07459997632503512221546770888380706310272216796875f,-0.08009998699426734702910835039801895618438720703125f,0.28580002604126963916542081278748810291290283203125f,-0.69340000111460664555806943099014461040496826171875f,-0.19310005935430485379811216262169182300567626953125f,0.64080000368952738654115819372236728668212890625f,-0.7308000218689443272523931227624416351318359375f,-1.3584000096082693431753796176053583621978759765625f,0.03280000523924841981937561286031268537044525146484375f,-0.40879999826550472707964445362449623644351959228515625f,-0.2500000112503766303007068927399814128875732421875f,-0.013000002920627640179418449406512081623077392578125f,0.190000004619359952329205043497495353221893310546875f,-0.2428999878376718601913353268173523247241973876953125f,-0.69899999028444259607795174815692007541656494140625f,0.06959998466968553287870236090384423732757568359375f,0.999299979430436469129972465452738106250762939453125f,0.6718000241339208145063821575604379177093505859375f,-0.6824000179231166640647643362171947956085205078125f,-1.5653000366806981702438861248083412647247314453125f,0.0575999911293386956945283827735693193972110748291015625f,-0.70719999852329495926284153028973378241062164306640625f,-0.5492000037051736338611362953088246285915374755859375f,-0.20979999278187744238266532192938029766082763671875f,0.30139999658465388421291208942420780658721923828125f,0.8198999799251556197532408987171947956085205078125f,-0.12099999630451208076920011080801486968994140625f,0.4041999890089034153106695157475769519805908203125f,0.6990999854743478181262616999447345733642578125f,0.2359999986141918526527661015279591083526611328125f,0.676999990150332298100011030328460037708282470703125f,-0.326399958474934859253835384151898324489593505859375f,0.0240000050365924977313625277020037174224853515625f,0.040800005775690095788377220742404460906982421875f,0.49719997330308007832400107872672379016876220703125f,-0.66540002740025538940926708164624869823455810546875f,-0.0463000058650970824913883916451595723628997802734375f,-0.50959998988509180861683489638380706310272216796875f,-0.73799996544420753963322567869909107685089111328125f,-0.413699993389844922209164224113919772207736968994140625f,-0.525299982962012190768064101575873792171478271484375f,-0.100300007586180850438495326670818030834197998046875f,-0.0074000089824200754318184181101969443261623382568359375f,-0.3432000064134594907727660029195249080657958984375f,-0.67419999079704240330102038569748401641845703125f,-0.07980000291466726736189229995943605899810791015625f,-0.0405000193119047224854512023739516735076904296875f,-0.09349994802474981270279386080801486968994140625f,-0.36369999309182166058462826185859739780426025390625f,-1.410300000023841704432925325818359851837158203125f,0.30090001020431600409210659563541412353515625f,-0.11850001269578935847448519780300557613372802734375f,-0.266999996408820106097437019343487918376922607421875f,-0.26619997126162076028776937164366245269775390625f,-0.23419999489486220678458039401448331773281097412109375f,-0.349300009977817627060403538052923977375030517578125f,0.0960999959647654833361229975707828998565673828125f,0.1612000056833029493219555661198683083057403564453125f,0.98639999494552643710676420596428215503692626953125f,-0.890999981105327520936043583787977695465087890625f,-0.299100000971555868289897262002341449260711669921875f,-1.674900038337707286473232670687139034271240234375f,0.36330004019737316411919891834259033203125f,-0.04659999754428856277854720246978104114532470703125f,-0.61569998653903612595428285203524865210056304931640625f,-0.46929997456893346452488913200795650482177734375f,-0.38809999790564175814466807423741556704044342041015625f,0.95009998788833627259009517729282379150390625f,-0.0217000092744825678892084397375583648681640625f,0.02729999725222587159123577293939888477325439453125f,0.81979998548030863503299769945442676544189453125f,0.171999973237515035862088552676141262054443359375f,0.1407999902784824453050305237411521375179290771484375f,-0.40079999504685392963665435672737658023834228515625f,0.80359995670020634150887417490594089031219482421875f,0.57869998921751975018423763685859739780426025390625f,0.6632999877452849801784395822323858737945556640625f,0.107299998295307119633434922434389591217041015625f,-0.0366999988436698121319068377488292753696441650390625f,-0.2194999842643738929837127216160297393798828125f,-0.2050000259280206904577426030300557613372802734375f,0.3230000111162663056774135839077644050121307373046875f,-0.0636000001370906620223877325770445168018341064453125f,-0.17490001583695402587181888520717620849609375f,0.100200009155273495053961596568115055561065673828125f,0.565200012731552181577399096568115055561065673828125f,0.6188999961376193681417134939692914485931396484375f,-0.56599999958276736578000054578296840190887451171875f,-0.01640001699924464872992757591418921947479248046875f,0.906599986070393715209547735867090523242950439453125f,-0.37759999474287031517860668827779591083526611328125f,0.72639998809099193266547445091418921947479248046875f,0.703199996280670092119180480949580669403076171875f,0.2715999823451038519550593264284543693065643310546875f,-0.7380000160336497305735292684403248131275177001953125f,-0.6122000099599365086078250897116959095001220703125f,-0.40300001782178895837205345742404460906982421875f,0.48460000731945029883718234486877918243408203125f,0.161300033432245637499136137194000184535980224609375f,-0.05400000712275510039717119070701301097869873046875f,-0.63820000507235530307070803246460855007171630859375f,-0.2830000151395797303166546043939888477325439453125f,0.5226000190317632299041861188015900552272796630859375f,-0.2078000033020972470154674738296307623386383056640625f,-0.6607000145345927055728907362208701670169830322265625f,-0.30120000054240225750845638685859739780426025390625f,0.56659999888539314127910984097979962825775146484375f,1.1438000049710268513081246055662631988525390625f,-0.17619999696612342887647173483856022357940673828125f,-0.25739999586939799058882272220216691493988037109375f,-0.613700004118680197962021338753402233123779296875f,0.01319999583363529183088758145458996295928955078125f,-0.77120001603960997016429246286861598491668701171875f,0.025399976587294759156065993010997772216796875f,0.5097999966561792550834297799156047403812408447265625f,-0.0169999927580357645950925871147774159908294677734375f,-0.2818999899685383780933989328332245349884033203125f,0.3952000161588193893891229890868999063968658447265625f,-0.5452000033438204606994759160443209111690521240234375f,-0.5588000238955022069831102271564304828643798828125f,0.09620001142025014218006617738865315914154052734375f,0.58120002572536488827381617738865315914154052734375f,0.684100010359287580286036245524883270263671875f,-0.30480000813007368520857198745943605899810791015625f,0.16080000041127195942181060672737658023834228515625f,-0.07310000764727586641811285517178475856781005859375f,0.413900003737211363841197453439235687255859375f,0.0753999990880489523448204636224545538425445556640625f,0.7979999933838843872990764793939888477325439453125f,0.6053999816536901334984577260911464691162109375f,0.326700002717971837995492023765109479427337646484375f,0.93600000584125542246738405083306133747100830078125f,-0.765199996936321014118220773525536060333251953125f,-0.306000008374452736603643643320538103580474853515625f,0.667799970221519600954707129858434200286865234375f,0.0261999744653707011821097694337368011474609375f,0.2861999971151352184506322373636066913604736328125f,-0.59559998976588257590947250719182193279266357421875f,-0.1824000092804425321446615271270275115966796875f,-0.3380000164806842288811594698927365243434906005859375f,-0.03160000410079977228861025650985538959503173828125f,-0.231000021636486252418762887828052043914794921875f,0.191400005972385489627640708931721746921539306640625f,-0.32409999635219577385214506648480892181396484375f,-0.56089997920989986823769868351519107818603515625f,-0.411600000971555690654213321977294981479644775390625f,-0.605000007599592048990189141477458178997039794921875f,0.5620000170469285905028300476260483264923095703125f,0.104600000390410396988727370626293122768402099609375f,1.304300011467933728681600769050419330596923828125f,0.0618999707460403669756487943232059478759765625f,0.1355999918520449654124604421667754650115966796875f,-0.862700022864341775630236952565610408782958984375f,-0.496099962437153152450264315120875835418701171875f,-0.1388999828755854704098737784079276025295257568359375f,0.130300020885467660036738379858434200286865234375f,-0.530899985766410509313573129475116729736328125f,-0.52750001370906840492125411401502788066864013671875f,-0.13289998210072528905811850563623011112213134765625f,0.7124999798834323883056640625f,0.1768999930977821488653489723219536244869232177734375f,-0.49009999936223014316283297375775873661041259765625f,0.1419000322878360975664691068232059478759765625f,-0.1778999990880489878719572516274638473987579345703125f,0.880400013542175230440989253111183643341064453125f,0.61659999396800913018523715436458587646484375f,0.32710000940561290150299100787378847599029541015625f,1.1884000084161758792333785095252096652984619140625f,0.8177999965965749851193322683684527873992919921875f,-0.677200023072958234848783831694163382053375244140625f,-0.340600023740530222227107515209354460239410400390625f,0.40859999074935871021807543002068996429443359375f,0.54600003102421801326471495485748164355754852294921875f,0.30120001872181922930593600540305487811565399169921875f,-0.31019999775588491530697865528054535388946533203125f,0.507400011962652275343543806229718029499053955078125f,-0.588600008183717715581906304578296840190887451171875f,-0.76950002121925376030731058563105762004852294921875f,0.38899998402595503677048327517695724964141845703125f,-0.62179998345375064872087023104541003704071044921875f,-1.1082999734401699498675952781923115253448486328125f,0.0996000000178813849061043583787977695465087890625f,-0.8654000035583973016173331416212022304534912109375f,-0.1238000443100926162287578335963189601898193359375f,-0.6147999966189263698623790332931093871593475341796875f,0.4919999910071493065544245837372727692127227783203125f,0.8273999954409896329110551960184238851070404052734375f,0.0626000127732755373699546908028423786163330078125f,0.7238000081002713415045946021564304828643798828125f,-0.09509997656345348104878212325274944305419921875f,0.0696000095546247532496408894076012074947357177734375f,0.5228000225245954535324699463672004640102386474609375f,0.1640000200867657298431367962621152400970458984375f,-0.34940002140998860369336398434825241565704345703125f,0.70200001883506768507459128159098327159881591796875f,0.3461000494599346666291239671409130096435546875f,0.631200001734494975380584946833550930023193359375f,0.282799998682737196986636263318359851837158203125f,-0.7919999765157701432372050476260483264923095703125f,0.3956000089138747011219265914405696094036102294921875f,-0.1296000024020671492763767673750407993793487548828125f,-0.67979999103099142399031507011386565864086151123046875f,-0.614000025525689441252552569494582712650299072265625f,0.403000002846121641031373883379274047911167144775390625f,0.821400022624433123752396568306721746921539306640625f,0.259000005930662080544379932689480483531951904296875f,0.106799994629621419761633660527877509593963623046875f,0.0474999845027923583984375f,0.4703999929785727118769500521011650562286376953125f,-0.7613999946475029645398535649292171001434326171875f,-0.44840000901222243356869512354023754596710205078125f,0.4962000108242035079086917903623543679714202880859375f,0.2542000055491924204176257262588478624820709228515625f,0.2642999956727027921488115680404007434844970703125f,-1.2037000075459474146555294282734394073486328125f,-0.488400011545419676650681140017695724964141845703125f,-0.190799998399615144961671830969862639904022216796875f,0.9912000220745795786569942720234394073486328125f,0.40920001322329035975400302049820311367511749267578125f,-0.711099999988078934620716609060764312744140625f,0.25750002235174207498857867904007434844970703125f,-0.144700003999471693560963103664107620716094970703125f,-1.20999997079372434427568805404007434844970703125f,0.030600016140937214004225097596645355224609375f,-0.1529999938309192231145061668939888477325439453125f,-0.015500003814697294046709430404007434844970703125f,-0.4842999703407286915535223670303821563720703125f,-0.545400001472234663424387690611183643341064453125f,0.74889998006299152155662568475236184895038604736328125f,-0.47950000529363734358412330038845539093017578125f,0.0391000018842518060058210949136991985142230987548828125f,-0.18150001323223108329329988919198513031005859375f,0.6022999920368192761088721454143524169921875f,0.5271000095546245489686043583787977695465087890625f,0.18770000453591340061620940105058252811431884765625f,0.490900002008675784281876985915005207061767578125f,0.354300013703107818940907236537896096706390380859375f,0.212000010490417167829946265555918216705322265625f,0.0193999671697611120180226862430572509765625f,0.32910000737905509282654747948981821537017822265625f,-0.785499999642372159769365680404007434844970703125f,1.061699970531464032319490797817707061767578125f,0.49069999300241473605410646996460855007171630859375f,-0.6985999920904635285268113875645212829113006591796875f,0.42980000820457942012353669269941747188568115234375f,0.12399999881535761492301617181510664522647857666015625f,0.7518000066623089505668531273840926587581634521484375f,-0.54260003285259017502539791166782379150390625f,-0.020700002725422328442306252327398397028446197509765625f,0.1708999878525734317236128845252096652984619140625f,0.4610999844908718614533427171409130096435546875f,0.141399993157386827391519545926712453365325927734375f,-0.2837999695062638494391649146564304828643798828125f,0.10700000989437086218458716757595539093017578125f,-0.31990000689625741614463549922220408916473388671875f,0.08980000470280646851506389793939888477325439453125f,-0.010400048559903840583729106583632528781890869140625f,0.109800004330277534148052609452861361205577850341796875f,-0.307799997863173413481518991829943843185901641845703125f,-0.292700007329881106699076553923077881336212158203125f,0.2835999999880789435025008060620166361331939697265625f,0.2438000043749808387616440086276270449161529541015625f,0.091800001782178952680624206550419330596923828125f,0.62540001816153534264230984263122081756591796875f,0.28040001935362823815012234263122081756591796875f,0.17469998179674206539857550524175167083740234375f,-0.55340001690983786630795293604023754596710205078125f,0.14259999727606764707843467476777732372283935546875f,0.05050001186132480057722204946912825107574462890625f,0.3552000133275985405134633765555918216705322265625f,0.4319999848604201275747982435859739780426025390625f,0.3935999925374975560998791479505598545074462890625f,-0.2276000010013579100132119492627680301666259765625f,0.6696000150680543328007843228988349437713623046875f,0.2494000434637069218979377183131873607635498046875f,-0.617400010621547590261570803704671561717987060546875f,-0.277999991595744955219515759381465613842010498046875f,0.4480999988555911528465003357268869876861572265625f,-0.08680001474618936896376908407546579837799072265625f,-0.77700002151727698418426371063105762004852294921875f,-0.119000027686357867651167907752096652984619140625f,-0.53300001677870767480271751992404460906982421875f,-0.25920000972151768792173243127763271331787109375f,0.5762999811291695806403367896564304828643798828125f,0.0156000018358231073989372816868126392364501953125f,0.18400000607967381682783525320701301097869873046875f,-0.04219998872578134552213668939657509326934814453125f,0.2274000010848045150169127737171947956085205078125f,0.116999996483325929830243694595992565155029296875f,-0.43309997665286115164917646325193345546722412109375f,0.419000009655952521114841147209517657756805419921875f,-0.467600001597404446584960169275291264057159423828125f,-0.492100018644332948269948246888816356658935546875f,0.01340000633001336982630391503334976732730865478515625f,0.39280000866651543223184717135154642164707183837890625f,0.20110000922679915902335778810083866119384765625f,0.18740001389980331936158108874224126338958740234375f,0.48700002062320724149913075962103903293609619140625f,0.7084999925792221375786539283581078052520751953125f,-0.2146000034078955620575612783795804716646671295166015625f,-0.080500000089406942294090185896493494510650634765625f,-0.2993999973818659565694133561919443309307098388671875f,-0.084399979873001740315885399468243122100830078125f,-0.136599996314942850883511482606991194188594818115234375f,0.22480000082850448706039969692938029766082763671875f,0.59919997604489338982602930627763271331787109375f,0.2358999950051308136522720815264619886875152587890625f,0.4892999953746794972175848670303821563720703125f,1.13469999103546115293283946812152862548828125f,0.3482000031352043833976495079696178436279296875f,-0.2079000195026399211428724811412394046783447265625f,-0.70650000429153436698470613919198513031005859375f,-0.07029999361634242660556992632336914539337158203125f,0.634800001573562866497013601474463939666748046875f,0.8651999638557441585362539626657962799072265625f,0.3419999812841414410513607435859739780426025390625f,0.144800014686584432865856797434389591217041015625f,0.16679999060630734675214625895023345947265625f,0.35620000963211051470125312334857881069183349609375f,0.3370000084042548138540951185859739780426025390625f,-1.0675999720931059755457681603729724884033203125f,-0.433899988985061657587039007921703159809112548828125f,-0.106399999117851251639876863919198513031005859375f,-0.694399997866153029235647409223020076751708984375f,-0.5397000150263309148357393496553413569927215576171875f,0.543499992132186804383309208787977695465087890625f,-0.9191000029087064149280195124447345733642578125f,-0.4150000047683715109769764239899814128875732421875f,-0.0700999983191490372291809762828052043914794921875f,0.1510999995410440277510133455507457256317138671875f,0.13650000429153441583451922269887290894985198974609375f,-0.4799999874830245261136951739899814128875732421875f,0.380599990808963895005945232696831226348876953125f,0.2083999957501887489996761360089294612407684326171875f,-0.3999999967217444662992420489899814128875732421875f,0.2382000186324120249992120079696178436279296875f,-0.127500001788139361025287144002504646778106689453125f,0.225799998399615287070218982989899814128875732421875f,0.3183999946326014907072021742351353168487548828125f,0.24370000508725642163199154310859739780426025390625f,0.966200002479553177181514911353588104248046875f,0.685199991571902700115970219485461711883544921875f,0.2856000058591365853288834841805510222911834716796875f,0.28330000005736943879952605129801668226718902587890625f,0.1577000303149223014997915015555918216705322265625f,0.588799993348121386560478640603832900524139404296875f,-0.756800019812584157108403815072961151599884033203125f,-1.315200000512600553292941185645759105682373046875f,0.10420000942349449957191609428264200687408447265625f,-0.44040000385642041891287590260617434978485107421875f,-0.04510002096891430056757599231787025928497314453125f,0.679800023627281202465155729441903531551361083984375f,0.214999982714652926318876779987476766109466552734375f,-0.1501000257372862023430570843629539012908935546875f,0.9644000243902208779900320223532617092132568359375f,0.2506000197172166821246719337068498134613037109375f,-1.0165999710202218153654030174948275089263916015625f,0.47160001110434546234273511799983680248260498046875f,0.59820003189444559410503643448464572429656982421875f,-0.23779996753930987551939324475824832916259765625f,0.29280000851750376256177332834340631961822509765625f,-0.41939999771714209231276981881819665431976318359375f,-0.120800038558244171582600756664760410785675048828125f,-0.3728000076234341264580507413484156131744384765625f,0.2488000030338763934878443251363933086395263671875f,0.95399999445676808562666337820701301097869873046875f,-0.4644000090420246085187727658194489777088165283203125f,-0.0422000019133090820133702436578460037708282470703125f,0.24140000343918810443710754043422639369964599609375f,-0.5943999998033044906975419507944025099277496337890625f,-0.0654000050485132877753358116024173796176910400390625f,0.3691999924361706320041776052676141262054443359375f,-0.0293999992072582205793196408194489777088165283203125f,-0.0617999985784292160673203397891484200954437255859375f,0.16449998784065267276588429012917913496494293212890625f,0.22399999877810472526107332669198513031005859375f,-0.1836000129520893864309982745908200740814208984375f,-0.799599981838465367900425917468965053558349609375f,-0.75039999700188619868868045159615576267242431640625f,0.40359999775290500423352568759582936763763427734375f,1.2563999806404115844316038419492542743682861328125f,-0.4526000035345554284305080727790482342243194580078125f,0.9868000095307827290724844715441577136516571044921875f,0.9533000257432462376527837477624416351318359375f,0.14280000643134105331455430132336914539337158203125f,0.37099999555945384344113335828296840190887451171875f,0.5937999717414381706248605041764676570892333984375f,0.159800004106759985234020859934389591217041015625f,0.51040000221729275864390729111619293689727783203125f,-1.3743999913096427434311408433131873607635498046875f,0.209600000464917002318543381989002227783203125f,-0.157600015008449556574987582280300557613372802734375f,0.0407000076055528126062199589796364307403564453125f,-0.166299975466728255923953838646411895751953125f,-0.264600001358985981170235390891321003437042236328125f,0.0931999834656718206815639859996736049652099609375f,1.139799977433682443006546236574649810791015625f,0.365999983936548201057803453295491635799407958984375f,-0.923199980187416002763711730949580669403076171875f,1.5088000070571894184467964805662631988525390625f,0.59770000751614560385860386304557323455810546875f,-0.164699988055229340488949674181640148162841796875f,1.1008999941110602094340720213949680328369140625f,0.5804000145852565850645987666212022304534912109375f,0.02609998538494107123142384807579219341278076171875f,0.119300016534328534589803894050419330596923828125f,-0.184599998676776788641973325866274535655975341796875f,1.0575000029802321677152576739899814128875732421875f,-0.28730001647472391823612269945442676544189453125f,-0.07530000270605086143405060283839702606201171875f,0.10749999716877933764180852449499070644378662109375f,-0.69169999213814747918149805627763271331787109375f,-0.1983999958992004319924262745189480483531951904296875f,0.21849998989701280294184471131302416324615478515625f,-0.8882999759733678502016118727624416351318359375f,-0.2703999923825264328769435451249592006206512451171875f,0.2108999947071075187210453805164434015750885009765625f,-0.073299997133016603356736595742404460906982421875f,-0.074499999128282035076864531220053322613239288330078125f,-0.9304999918937681968600372783839702606201171875f,0.128699999797344588614578242413699626922607421875f,0.0544999935030936466517914595897309482097625732421875f,1.26289998016357429833078640513122081756591796875f,-0.70509999802112588440650142729282379150390625f,-0.122499994933605194091796875f,0.93269999440312378879980315105058252811431884765625f,0.1240999675929543855090741999447345733642578125f,0.4066000045478344038230034129810519516468048095703125f,0.8074999749660491943359375f,0.526699970233440950551084824837744235992431640625f,0.2097999918878079572692740839556790888309478759765625f,-1.0217000241756437617368646897375583648681640625f,-0.0843999621033670877068288973532617092132568359375f,-0.22260000293850890074054404976777732372283935546875f,-0.74239999616742124999291263520717620849609375f,0.02869996738731916963160983868874609470367431640625f,0.301800006997585246182325136032886803150177001953125f,0.431800002229213664151075136032886803150177001953125f,0.1961000011801719011828026850707828998565673828125f,-0.617800007325410849290392434340901672840118408203125f,0.541600004547834412704787609982304275035858154296875f,1.06750000238418607523271930404007434844970703125f,-0.519799995687603821892253108671866357326507568359375f,0.091999994620680924928279864616342820227146148681640625f,0.52649998771399264541059892508201301097869873046875f,-0.0826000050246715744606262887828052043914794921875f,-0.61660001170039180351523100398480892181396484375f,0.1738999819815152392266099923290312290191650390625f,0.45160001348853118230408654198981821537017822265625f,0.61919999899268152177000956726260483264923095703125f,-0.3081999679684646054056429420597851276397705078125f,-0.44180001161694537614721411955542862415313720703125f,-0.43900002172589314142214789171703159809112548828125f,-0.22010001351833352600806392729282379150390625f,0.07720000727772724502528944867663085460662841796875f,-0.49380000690817826836109816213138401508331298828125f,-0.13660002317428610041361025650985538959503173828125f,-0.5870000100433827583401580341160297393798828125f,-0.425600018674135327501062420196831226348876953125f,0.668499954581260880104309762828052043914794921875f,0.221800002229213699678211924037896096706390380859375f,0.138000009477138529945250411401502788066864013671875f,0.0173000038087369745198884629644453525543212890625f,-0.212000008702278197603163789608515799045562744140625f,0.306000000178813902351748765795491635799407958984375f,0.121200027662515363857664851821027696132659912109375f,0.8688000042259691468871096731163561344146728515625f,-0.0651999917209149515429089660756289958953857421875f,-1.293199989128112292746664024889469146728515625f,-0.19000001147389422584410567651502788066864013671875f,-0.02759999876618390857174745178781449794769287109375f,-0.13699998784065225976291912957094609737396240234375f,0.43360001384615909358899443759582936763763427734375f,-0.35019999857544892307714690105058252811431884765625f,-0.00960002698898332340604611090384423732757568359375f,0.65140002415180209283107615192420780658721923828125f,0.63480000470280639746079032192938029766082763671875f,0.0213999964356421656930251629091799259185791015625f,-0.8563999596297744432149556814692914485931396484375f,-0.57669999198913568516644545525196008384227752685546875f,0.3489999968409538411151515902020037174224853515625f,0.7695999736428262849585735239088535308837890625f,0.354799994570016874462226041941903531551361083984375f,1.0588999904751776881539626629091799259185791015625f,-0.4277000166058542163227684795856475830078125f,0.037600003683567084777905620285309851169586181640625f,0.4425999854668976052352036276715807616710662841796875f,-0.652399984060227922100239084102213382720947265625f,-0.163699994321167496824642739738919772207736968994140625f,0.263700020360946751907249563373625278472900390625f,-0.548099987828731372019319678656756877899169921875f,-0.4709000106513501204830163260339759290218353271484375f,-0.26949998903274519079786841757595539093017578125f,1.08049997520446794396775658242404460906982421875f,0.599099994564056270718310770462267100811004638671875f,-0.445100018882751413684673025272786617279052734375f,-0.9824999874830240287337801419198513031005859375f,-0.3969000033795833104477424058131873607635498046875f,-0.0864999977350233706374638131819665431976318359375f,-0.25209997572898856788015109486877918243408203125f,-0.29580000190138822091512338374741375446319580078125f,0.65969997434616090004055877216160297393798828125f,-1.10969999282360021197746391408145427703857421875f,-0.557800005984306235262693007825873792171478271484375f,0.1100000050663947348539295489899814128875732421875f,0.426199992346763423256561509333550930023193359375f,0.1563000010967254826965699976426549255847930908203125f,0.02540001026391980332164166611619293689727783203125f,-0.081800015044212415205038269050419330596923828125f,0.130399999088048890172331084613688290119171142578125f,-1.29610000803470626351554528810083866119384765625f,1.1182999913215638798646978102624416351318359375f,0.39370000158548368318633947637863457202911376953125f,-0.2663000010967255803961961646564304828643798828125f,-0.3107000016450882640128838829696178436279296875f,-0.054299997162818858242872011032886803150177001953125f,0.2305000091791153948861392564140260219573974609375f,0.38530001254081724937350372783839702606201171875f,-0.112400004512071660656147287227213382720947265625f,0.223999979853630293291644193232059478759765625f,-0.0754000069670379691189054938149638473987579345703125f,0.399000006526708528298286182689480483531951904296875f,-0.729800019603967786707698905956931412220001220703125f,-0.6533000398993493718080571852624416351318359375f,-0.43420000227093691336222036625258624553680419921875f,-0.60400000980496404423547573969699442386627197265625f,0.532899953490495903452028869651257991790771484375f,0.1556000025808810249827729421667754650115966796875f,0.0318000032722949299568426795303821563720703125f,0.23139997333884299024475694750435650348663330078125f,0.062000006467104018526015352108515799045562744140625f,-0.224600003296136829789020339376293122768402099609375f,-0.024200022238492824300237771240063011646270751953125f,-0.0185999983489513187606689825770445168018341064453125f,0.1060000001788139467606697508017532527446746826171875f,-0.0372999973267316153879846751806326210498809814453125f,-0.10620000117570149722467931496794335544109344482421875f,-0.38499999303370702197213404360809363424777984619140625f,-0.2087000007100401344217743826447986066341400146484375f,0.425600003474950749904337499174289405345916748046875f,-0.279999994188547152162982456502504646778106689453125f,-0.359600011193752067129025817848742008209228515625f,-0.29519999797940243979610386304557323455810546875f,0.1381999985158444133048760704696178436279296875f,0.72439999399185150963376145227812230587005615234375f,0.2554000098168851540236801156424917280673980712890625f,0.5948000288426877890657351599656976759433746337890625f,0.2785000226795677491509195533581078052520751953125f,0.2456000060081482416762810316868126392364501953125f,0.197400009131431630748920724727213382720947265625f,-0.68319998510479873488066004938445985317230224609375f,0.03779999465942385228345301584340631961822509765625f,-0.80139999852180476835883382591418921947479248046875f,-0.009500016748905437680150498636066913604736328125f,0.329800001201033421427410985415917821228504180908203125f,0.475999997720122192124136972779524512588977813720703125f,0.052100009666383062523209446226246654987335205078125f,-0.82500000938773165870543380151502788066864013671875f,-0.23660002660155310394429761799983680248260498046875f,0.9570999495029450798710968228988349437713623046875f,-0.715999998986720953553231083787977695465087890625f,0.385799985957145707970994408242404460906982421875f,1.285599980825185451038805695134215056896209716796875f,-0.52640002128332874509197836232488043606281280517578125f,-0.1751000016719102825302201154045178554952144622802734375f,-0.5915000153183935793776981881819665431976318359375f,0.007100014770031748412293381989002227783203125f,-0.289500010341405822344995613093487918376922607421875f,0.572699986505508462641955702565610408782958984375f,-1.024699972856044638547245995141565799713134765625f,-0.6160000004768371173469176937942393124103546142578125f,0.3744999800920485455435482435859739780426025390625f,0.271299990665912815757110365666449069976806640625f,0.0335999992430211502636439035995863378047943115234375f,0.02329998983144765389852182124741375446319580078125f,-0.071899987435340761976476642303168773651123046875f,-0.13219999967813489405443760915659368038177490234375f,-0.070400005048513403238530372618697583675384521484375f,0.0453999995350837792784659541212022304534912109375f,0.0725000018626451581127412282512523233890533447265625f,-0.27999999091029170728006647550500929355621337890625f,-0.449799970541894911235658582882024347782135009765625f,-0.26419999146759509667248266850947402417659759521484375f,-0.2438999964356423078015723149292171001434326171875f,0.393700004565715744320186786353588104248046875f,-0.016300000798702196647127493633888661861419677734375f,0.8033000068187714504119867342524230480194091796875f,-0.27230000097751627663456019945442676544189453125f,-0.09540000206828125328684109263122081756591796875f,0.41710001491904247217235024436376988887786865234375f,0.825299992424248785027884878218173980712890625f,0.4410000113546848066192751502967439591884613037109375f,-0.8114000140190125165418066899292171001434326171875f,0.443400005733966651177979656495153903961181640625f,0.3047000108540058871398059636703692376613616943359375f,0.099200028347969038122755591757595539093017578125f,-0.51939997372627288996227434836328029632568359375f,-0.536000003308057859641166942310519516468048095703125f,0.1665999973580241100279408783535473048686981201171875f,0.829999961443245837244830909185111522674560546875f,0.413899990288913277591831274548894725739955902099609375f,0.90789996109008797020578640513122081756591796875f,-1.29909999873638071221648715436458587646484375f,-0.539100007528066527129340101964771747589111328125f,1.31959998466968553287870236090384423732757568359375f,-0.664800010812282238248371868394315242767333984375f,-0.128900005078315782469644545926712453365325927734375f,0.06719998500049084722007819436839781701564788818359375f,0.0855000154376028831393341533839702606201171875f,-0.540000011622905784491877057007513940334320068359375f,0.014399996078014343225959237315692007541656494140625f,0.58999999701976779675760553800500929355621337890625f,-0.7453999961823221109824544328148476779460906982421875f,-0.363000003144144767208700841365498490631580352783203125f,0.366099980802834235049658673233352601528167724609375f,-0.05879999699890614550668033189140260219573974609375f,-0.085999996677041057324686335050500929355621337890625f,0.520199964265525949258517357520759105682373046875f,-0.25080001486539860167113147326745092868804931640625f,-0.54500002115964907289935581502504646778106689453125f,0.0125999866962427375938204932026565074920654296875f,-0.238199997770786264794651287957094609737396240234375f,0.273799994540214530758248656638897955417633056640625f,0.487699991720914827197930208058096468448638916015625f,-0.482800007623434002113071983330883085727691650390625f,-0.080599996322393341330325711169280111789703369140625f,0.2802000025987627651602451805956661701202392578125f,0.5265999953091142771199884009547531604766845703125f,-0.8518000089347363967817727825604379177093505859375f,-1.4135000100135801659462231327779591083526611328125f,0.549800016176700712122737968456931412220001220703125f,-0.665600009286403615504923436674289405345916748046875f,-0.6235000308752063347128569148480892181396484375f,-0.23940001153796919286520505920634604990482330322265625f,-0.05280000233352188931945647709653712809085845947265625f,0.6147000083394342784259833933901973068714141845703125f,0.13100000084936613831132490304298698902130126953125f,0.183799999383091883231600149883888661861419677734375f,-0.17449999190121889824922618572600185871124267578125f,-0.11719999729394903198453903314657509326934814453125f,0.65640001372098932819199035293422639369964599609375f,0.8982000210165974607434691279195249080657958984375f,0.30140000045895565250475556240417063236236572265625f,0.03240001106858247936770567321218550205230712890625f,0.30309998335838361782634819974191486835479736328125f,0.64940000934004782351394169381819665431976318359375f,-0.06839999157786369465839015902020037174224853515625f,-1.116499995201825523594152400619350373744964599609375f,-0.45559999419897778683008482403238303959369659423828125f,0.17899999020993717646632603646139614284038543701171875f,0.5921999883718787049957654744503088295459747314453125f,0.60780003191232712111968794488348066806793212890625f,-0.8921000088468191702162357614724896848201751708984375f,-0.8523000004746013047451924649067223072052001953125f,-0.442199975426495417440975188583252020180225372314453125f,-0.62909998005405076693108412655419670045375823974609375f,-0.709399999207258158406830261810682713985443115234375f,0.0155999988555908242204850466805510222911834716796875f,-0.6734000029772519990700629932689480483531951904296875f,-0.52020000062435844956354458190617151558399200439453125f,-0.105599988126755028883252407467807643115520477294921875f,-0.489399997828900923391159949460416100919246673583984375f,0.2225000053644183850565241300500929355621337890625f,-0.70139997259378450422673267894424498081207275390625f,0.2417000337123871300804012207663618028163909912109375f,-0.8200000053644185271650712820701301097869873046875f,-0.19680001504421174018943929695524275302886962890625f,-0.8664000038117165569673261416028253734111785888671875f,1.3049000347614292394382573547773063182830810546875f,0.20500000506639448616397203295491635799407958984375f,1.0417000146389006065561488867388106882572174072265625f,0.49910000752806649160220331395976245403289794921875f,-0.17139999971389752175809917389415204524993896484375f,0.519100021088123586565643563517369329929351806640625f,-0.5353000058352943568706905352883040904998779296875f,-0.361399981534481273826031610951758921146392822265625f,-0.3644999864250422216827018928597681224346160888671875f,-0.3616000315934424502728461447986774146556854248046875f,0.71159997400045404969404216899420134723186492919921875f,-0.314000055924058329992476501502096652984619140625f,-0.6414000013530252175542045733891427516937255859375f,-0.095399992233514918638093149638734757900238037109375f,-0.44279998940676446039788061170838773250579833984375f,0.36639999937862166579094491680734790861606597900390625f,-0.5645999812424182806580574833787977695465087890625f,0.30580002000629902259021264399052597582340240478515625f,-0.03389999762773499014656408689916133880615234375f,0.57519998427033414145626011304557323455810546875f,-0.19970003644675049514489728608168661594390869140625f,-0.8386000161558389009996972163207828998565673828125f,-0.4595999866068363193249979303800500929355621337890625f,-0.6945999944299459105678806736250407993793487548828125f,-0.6785000120624899988541756101767532527446746826171875f,0.3321999951332805789405711038853041827678680419921875f,-0.607700024168193575491159208468161523342132568359375f,-0.544300050210953401119695627130568027496337890625f,0.5795000172704458663019977393560111522674560546875f,0.115799975228309648400681908242404460906982421875f,0.85159996520876912740050102001987397670745849609375f,0.378499986693263235082440587575547397136688232421875f,-0.129800000530481352001288541941903531551361083984375f,-0.1766000012695789767036558259860612452030181884765625f,0.26430001772642153223813465956482104957103729248046875f,-0.52959998377561579019356940989382565021514892578125f,-1.10449996057152777240162322414107620716094970703125f,-0.613599988737702606300672414363361895084381103515625f,0.0400000122189521079008045489899814128875732421875f,-0.3407999840199948948793462477624416351318359375f,-0.35279999421238894541374975233338773250579833984375f,-0.638200007605552510625557260937057435512542724609375f,-0.79140001387000058485909903538413345813751220703125f,-0.110099988856911412682393347495235502719879150390625f,0.0951999984264375598286278545856475830078125f,-0.054500013917684331232749173068441450595855712890625f,0.2759999945908784280135250810417346656322479248046875f,0.096799984645843772934625803827657364308834075927734375f,0.33279996040538006862874453872791491448879241943359375f,-0.321800035160779873688596808278816752135753631591796875f,0.308999996393918952009016720694489777088165283203125f,0.447499990910291689516498081502504646778106689453125f,0.2364999981075526225282601444632746279239654541015625f,0.182000002145767059147374311578460037708282470703125f,0.732400014793872511376093825674615800380706787109375f,0.3242000046551225533875140172312967479228973388671875f,-0.01960001223683338622549854335375130176544189453125f,0.05619997491240535936185551690869033336639404296875f,0.330399989923834791483159278868697583675384521484375f,0.6612000084400175836663038353435695171356201171875f,1.0688000073552128554865703335963189601898193359375f,0.54700000646710389418103659409098327159881591796875f,-0.2016000108063220608300980529747903347015380859375f,-0.0323000084280966603955675964243710041046142578125f,0.34980000082850448706039969692938029766082763671875f,0.0101999872505664779254175300593487918376922607421875f,-0.5719000193238255036476402892731130123138427734375f,-0.02090000729858854100484677474014461040496826171875f,0.198399996644258447275888102012686431407928466796875f,0.2040999956071376519872728749760426580905914306640625f,0.48700000780820851264962811910663731396198272705078125f,0.27279999987483005696731197531335055828094482421875f,-1.248000041998923226316264845081605017185211181640625f,0.45580001248121249091127538122236728668212890625f,-0.5633999862134455582918235450051724910736083984375f,0.2480000267997386487195399240590631961822509765625f,-0.4576000381052498511280646198429167270660400390625f,-0.480199993509053957296828230028040707111358642578125f,-0.3266000091299414709311577098560519516468048095703125f,0.2909000072240832679426603135652840137481689453125f,0.31219999192953107325365635915659368038177490234375f,0.4381000010907649677704966961755417287349700927734375f,-0.012800001066923361037197537370957434177398681640625f,0.6127999743938448151681086528697051107883453369140625f,0.023999977022409468840891122454195283353328704833984375f,0.1423999596595759697947869426570832729339599609375f,-0.5881999791443348613029229454696178436279296875f,0.29820000265091639590764316380955278873443603515625f,-0.69910001244544961451765630044974386692047119140625f,-0.190999979913235140571714509860612452030181884765625f,-0.56629999901056315980696354017709381878376007080078125f,0.8440000345222653521659594844095408916473388671875f,0.456400003848969448849715035976259969174861907958984375f,0.62200001151487216421998027726658619940280914306640625f,-0.26849999988079087387404797482304275035858154296875f,0.031999993056059050644535091123543679714202880859375f,-0.31150000492483365110985005230759270489215850830078125f,0.24560001137256648462425800971686840057373046875f,-0.3579999939799307639987091533839702606201171875f,0.0844000104576350107521420795819722115993499755859375f,-0.4142000056982038103114973637275397777557373046875f,-0.46759997179508250297885751933790743350982666015625f,-0.3349999823421241362808586927712894976139068603515625f,-0.474000006079673852354972041212022304534912109375f,-0.0172000080227849849734411691315472126007080078125f,-0.599200019481778145546968517010100185871124267578125f,-0.2501000057697293499359147972427308559417724609375f,-0.74139997169971483259587330394424498081207275390625f,-0.1672999734103679525532015759381465613842010498046875f,-0.84149999549984944025027289171703159809112548828125f,0.214399989074468777516813133843243122100830078125f,-0.6072999914035197566164470117655582726001739501953125f,-0.83850001701712617574457908631302416324615478515625f,-0.38139999405145630362312658689916133880615234375f,-0.72289999953508365937437929460429586470127105712890625f,-1.11790003321170861028122089919634163379669189453125f,-0.28629998649358778806117697968147695064544677734375f,0.021200012910365995111305892351083457469940185546875f,-0.333599978232383964638074758113361895084381103515625f,0.0906000131607054282056878946605138480663299560546875f,-0.21860001965761188813530679908581078052520751953125f,-0.0647000148773191341433630441315472126007080078125f,0.19769999306201935240778766456060111522674560546875f,0.50820000820159894061589511693455278873443603515625f,0.60780001209378209381384294829331338405609130859375f,0.495900014600157756916587459272705018520355224609375f,-0.4217999907553195981790850055404007434844970703125f,-0.24649998357892055622642146772705018520355224609375f,-0.504499989703297568865991706843487918376922607421875f,-0.01360000475645062323337697307579219341278076171875f,0.437999995768070249368975055404007434844970703125f,-0.0488000110805035802741258521564304828643798828125f,-0.334800001573562600043487691436894237995147705078125f,-0.9609000051379201767787208154913969337940216064453125f,-0.60019999939501256225327097126864828169345855712890625f,0.3950000011548400191685459503787569701671600341796875f,0.28579999316558246658104280868428759276866912841796875f,0.328499996695667484392089363609557040035724639892578125f,0.35860000892877597067354145110584795475006103515625f,0.2994999955892563292536578956060111522674560546875f,0.46829999966621382867515421821735799312591552734375f,0.14060000824332252733483983320184051990509033203125f,0.1022000011682511200206135981716215610504150390625f,-0.30450001764297507378387308563105762004852294921875f,-0.39779999763965623316153141786344349384307861328125f,-0.6344999777078630387450175476260483264923095703125f,-0.42909999396801001836365685448981821537017822265625f,-0.28459998824596421940213986090384423732757568359375f,-0.946199969547987240758857296896167099475860595703125f,-0.3844999855309725322882741238572634756565093994140625f,-0.666199994283914520565303973853588104248046875f,-0.80409998934865001274374662898480892181396484375f,-0.6798999887913463879840492154471576213836669921875f,0.09799999502301215414945545489899814128875732421875f,0.565799991768598520280875163734890520572662353515625f,0.3648000052243471902357896397006697952747344970703125f,-0.100599996024369975344825434149242937564849853515625f,0.19409999456405646611756310448981821537017822265625f,0.148099999973177798295864704414270818233489990234375f,-0.181000013440847595802551950328052043914794921875f,0.45900000395625806248744993354193866252899169921875f,-0.684400014257430910902257892303168773651123046875f,0.01719999848604203407376189716160297393798828125f,-0.5112000108987093671686352536198683083057403564453125f,-0.0222999845862386081307704444043338298797607421875f,-0.56359998314976689215427541057579219341278076171875f,0.0211000389173630065187126092496328055858612060546875f,-0.80640001580715203743920937995426356792449951171875f,-0.045000013858079501716247250442393124103546142578125f,-0.78900002012401804218910683630383573472499847412109375f,1.205899990236758867467870004475116729736328125f,0.0779999944269658129769595689140260219573974609375f,1.053699987168609908394500962458550930023193359375f,0.7596000222206118479562064749188721179962158203125f,-0.09039999089241046448250926914624869823455810546875f,0.5580000152885913333733469698927365243434906005859375f,0.1332999944508070910842434386722743511199951171875f,0.546999976813793420404863354633562266826629638671875f,0.2672999882370235003037350907106883823871612548828125f,-0.8144000031001865647084514421294443309307098388671875f,0.0937999993085860683805776716326363384723663330078125f,-0.74140000770464542512172556598670780658721923828125f,-0.9541000216841695191760663874447345733642578125f,-0.1435999798715119624148428556509315967559814453125f,-0.849500014998019192802303223288618028163909912109375f,0.391600024664402468488333397544920444488525390625f,0.570200000661611028363040531985461711883544921875f,0.28099999500066008550902552087791264057159423828125f,0.4595000161528588478176970966160297393798828125f,0.1351999881446364071280186180956661701202392578125f,0.498900012417137883602435977081768214702606201171875f,-0.283600001478195196114029386080801486968994140625f,0.34379998008608847470668479218147695064544677734375f,-0.075000006444752020939858994097448885440826416015625f,1.09429998226165725583314269897527992725372314453125f,0.099199994224310028556601537275128066539764404296875f,1.06489998867958757244878142955712974071502685546875f,0.081799985539912967169584590010344982147216796875f,-0.48679998941421498415138557902537286281585693359375f,-0.014999999180436152101947300252504646778106689453125f,0.232899980312585963559968149638734757900238037109375f,-0.3995999711096291928669188564526848495006561279296875f,0.177899996368586499340125328672002069652080535888671875f,-0.020800005477667138364950005779974162578582763671875f,0.46990000644922247374779544770717620849609375f,0.32440000084638587196650405530817806720733642578125f,-0.08490000004172326697471362422220408916473388671875f,0.316200004342198415230313912616111338138580322265625f,-0.0847999999344348776020297009381465613842010498046875f,-0.2678999999821185529214062626124359667301177978515625f,-0.15380000154376027854397079863701947033405303955078125f,-0.473199974524975086609401841997168958187103271484375f,-1.2373999446094041676502683912985958158969879150390625f,-0.72779997186064726566456783984904177486896514892578125f,0.37379999856352785769786351011134684085845947265625f,0.18890000254511818411629064939916133880615234375f,0.435300011274218690005000098608434200286865234375f,0.398800011974573376249963985173963010311126708984375f,0.385899993216991532563042710535228252410888671875f,0.0267999876260756053625300410203635692596435546875f,-0.330600007349252766886138488189317286014556884765625f,-0.136400000309944147147689363919198513031005859375f,-0.2680999996006487862132416921667754650115966796875f,0.320399992829561330864862611633725464344024658203125f,0.99009997954368600403540767729282379150390625f,0.497999994426965741922685992904007434844970703125f,-0.4079999999403955968091395334340631961822509765625f,-0.173599979424476913436592440120875835418701171875f,-0.54519999052882184287227573804557323455810546875f,-0.391400007164478314081179632921703159809112548828125f,-0.648800002139806597512006192118860781192779541015625f,-0.4123000047773122123118128001806326210498809814453125f,-0.28860001459121686906428294605575501918792724609375f,0.05879997799992597862228649319149553775787353515625f,-0.358500024542212525346940310555510222911834716796875f,0.70339999232292171882363618351519107818603515625f,0.9429999816119671862679751939140260219573974609375f,0.48549998496472834830228748614899814128875732421875f,-0.11599999451637277303461814881302416324615478515625f,-0.4794999802410604416991191101260483264923095703125f,0.24560001278817633618700710940174758434295654296875f,0.1139999967664480085005607179482467472553253173828125f,0.29689999522119758790239529844257049262523651123046875f,0.344600004003942028685258947007241658866405487060546875f,-0.208599988663196678206901424346142448484897613525390625f,-0.580299971041083662015580557635985314846038818359375f,-0.1440999885290863147258022536334465257823467254638671875f,0.4937999921560287663879762476426549255847930908203125f,-0.41729999941289419584933284568251110613346099853515625f,-1.222700007367134134028674452565610408782958984375f,0.1141999896049501472816700697876513004302978515625f,-1.014300007072090892279447871260344982147216796875f,-9.9997574090959773229769780300557613372802734375e-05f,0.1481999962806701542916698599583469331264495849609375f,0.043499999880790729545054773552692495286464691162109375f,0.174599990183114783803830505348742008209228515625f,0.583199965286255395113812483032234013080596923828125f,0.352599976041913176771913640550337731838226318359375f,-1.093900010740757267058143042959272861480712890625f,-0.045200017052888341595462406985461711883544921875f,-0.95050000757724006916760117746889591217041015625f,-0.1839000018000600533696342608891427516937255859375f,0.4538000055670732280077572795562446117401123046875f,-0.249500024124980512851834646426141262054443359375f,0.8246000091075895710446275188587605953216552734375f,-0.11820000298619248724207864142954349517822265625f,0.722800013174116795511281452490948140621185302734375f,0.10149999266862863578353426419198513031005859375f,-0.44739998201131836452759671374224126338958740234375f,-0.0808999988049270424284031832939945161342620849609375f,1.35600001567602124197264856775291264057159423828125f,0.128799976658821702102386552724055945873260498046875f,1.16320000522136712817200532299466431140899658203125f,0.3994000139594078291338519193232059478759765625f,0.1277999954044819475029726163484156131744384765625f,0.4516000154629351559520955561310984194278717041015625f,1.187400033271313048999218153767287731170654296875f,0.0018000154912467536405529244802892208099365234375f,0.98400002839416256250615333556197583675384521484375f,-0.6391999791741369563169428147375583648681640625f,0.162999993532896070291826617904007434844970703125f,-0.72259999746233205542012001387774944305419921875f,-0.9061000098228451093973490060307085514068603515625f,-0.34419998930692674576903300476260483264923095703125f,-0.6516999902009963374638346067513339221477508544921875f,-0.31730001062601809191932034082128666341304779052734375f,-0.355799991172552065865630765983951278030872344970703125f,-0.304899997471272943272424527094699442386627197265625f,-0.2606999996334311564538666061707772314548492431640625f,0.04239999735951427428659599172533489763736724853515625f,-0.1104999982640146871659680982702411711215972900390625f,0.02510001068711265048705172375775873661041259765625f,0.662400017774105176471266531734727323055267333984375f,0.0236000278532504381701073725707828998565673828125f,-0.69049996417760883815617489744909107685089111328125f,-0.488799985003471437750022232648916542530059814453125f,0.548199998515844555413423222489655017852783203125f,0.5334999714195731712607084773480892181396484375f,0.020899978092312920807671616785228252410888671875f,0.28020000379085541197810016456060111522674560546875f,0.427300002374499943158525638864375650882720947265625f,0.1432999982133507099746339008561335504055023193359375f,-0.26440000233650184924272252828814089298248291015625f,-0.07280001373290989619135871180333197116851806640625f,-0.443800014954805366329537719138897955417633056640625f,0.047399988269806014873353205985040403902530670166015625f,0.8428999773696066455386244342662394046783447265625f,0.214299991127848699079549987800419330596923828125f,-0.0660000006258487648125310442992486059665679931640625f,-0.11979999926388262831977726818877272307872772216796875f,0.045600001761317232595871473677107132971286773681640625f,-0.224799996954202663257404992691590450704097747802734375f,-0.61049999042227864887166788321337662637233734130859375f,0.018800009292364083801629703884827904403209686279296875f,0.261999996036291094014814007095992565155029296875f,0.54439998460412031278110589482821524143218994140625f,-0.119200003537535348385745237465016543865203857421875f,-0.0877999879539010930074027783120982348918914794921875f,-0.7346000018060205860592759563587605953216552734375f,-0.26989999512433993089643990970216691493988037109375f,-0.0537999982655048558655153101426549255847930908203125f,-0.25299999420344831957407905065338127315044403076171875f,-0.006499998331070011570176347959204576909542083740234375f,-0.0447999994874000717626216783173731528222560882568359375f,-0.2534999980740249014843357144854962825775146484375f,-0.068199998776614645379368084832094609737396240234375f,0.25759998818635931883136436226777732372283935546875f,0.0150999835669995974285484408028423786163330078125f,0.32079999839961548246947131701745092868804931640625f,-0.25739999572038652075889331172220408916473388671875f,-0.6981999933004379155221386099583469331264495849609375f,-0.42649999468028532145780218343134038150310516357421875f,0.1145999930143357659062530728988349437713623046875f,0.034399982219934788219006804865784943103790283203125f,-0.2657000257104635831950645297183655202388763427734375f,-0.52640001088976884346948281745426356792449951171875f,-0.086699995341896762823807875975035130977630615234375f,-0.28750001341104525209857456502504646778106689453125f,0.866599969232082489867252661497332155704498291015625f,-0.15230004188120382924154228021507151424884796142578125f,-0.2224999907612801308687267010100185871124267578125f,0.64319999136328664945949640241451561450958251953125f,-0.3191000197842719376950526566361077129840850830078125f,-0.09629999519214027525748633706825785338878631591796875f,0.11319999624341725574794281783397309482097625732421875f,-0.1673000066585839107347055687569081783294677734375f,1.08440000978708273038364495732821524143218994140625f,0.186800009977817094153351717977784574031829833984375f,0.91360000185072387512974501078133471310138702392578125f,0.8313000015065072911824017865001223981380462646484375f,-0.4551999818116427842795701508293859660625457763671875f,0.5591000101357697094073273547110147774219512939453125f,-0.13420000338852400290079458500258624553680419921875f,-0.1219999951422214845564440111047588288784027099609375f,-0.08379999632835388734264370214077644050121307373046875f,-0.53990001373216500724083743989467620849609375f,-0.180599993453919915875616197808994911611080169677734375f,-0.32789999683424830845268616030807606875896453857421875f,0.535999999880790056749901850707828998565673828125f,0.3847999753475193074336857534945011138916015625f,0.327199979785084860139932061429135501384735107421875f,-0.50640003175139458591047514346428215503692626953125f,0.43219999505877471079884344362653791904449462890625f,-0.306200032617152118064041133038699626922607421875f,-0.27159999910891041707827753270976245403289794921875f,0.0030000011324882169816419263952411711215972900390625f,-0.17979999654442069267901160856126807630062103271484375f,-0.18890000062659384472141255173482932150363922119140625f,0.09539999569803481183072335625183768570423126220703125f,-0.11490000073090189669500205127405934035778045654296875f,-0.95990000078678150430278037674725055694580078125f,-0.0230000100731846401913571753539144992828369140625f,-0.806299994689225751898220551083795726299285888671875f,0.1067999934375283288545688265003263950347900390625f,0.363799983960390260762096659163944423198699951171875f,0.2041999886736274305576444021426141262054443359375f,0.27420003520250357809118213481269776821136474609375f,0.501799998205899822067976856487803161144256591796875f,0.19880001149028536655549714851076714694499969482421875f,1.1671000227421519657156068205949850380420684814453125f,0.2315999926269054487448784129810519516468048095703125f,0.194600001358985696953141086851246654987335205078125f,0.06150000950694067824997546267695724964141845703125f,-0.131299998265504758165889143128879368305206298828125f,-0.2543999846041202772539691068232059478759765625f,-0.93859997927546512386243193759582936763763427734375f,-0.662599991688132217149131975020281970500946044921875f,0.102599991613626428943462087772786617279052734375f,-0.2029000078797338346703327260911464691162109375f,-0.34990002112686635626914721797220408916473388671875f,-0.26199997815489783903331044712103903293609619140625f,-0.952499961555004492907983149052597582340240478515625f,-0.5579999793767929094912005894002504646778106689453125f,0.1167999929904937062019598670303821563720703125f,-0.37410000752806649160220331395976245403289794921875f,0.123599997976422404377672137343324720859527587890625f,-0.260199987846613023378949947073124349117279052734375f,-0.636999969661236065121556748636066913604736328125f,-0.2640999918818476288606689195148646831512451171875f,-0.179000003993511125344184620189480483531951904296875f,-0.94040001339316336981255517457611858844757080078125f,-0.312100001135468207991152667091228067874908447265625f,-0.015000007599592191098736293497495353221893310546875f,-0.311200012612342646889374009333550930023193359375f,0.1457000006020070514978215214796364307403564453125f,0.618600002372264867744888761080801486968994140625f,1.052499995231628293623771241982467472553253173828125f,0.7975000016391275980964792324812151491641998291015625f,0.35579999713301635466677907970733940601348876953125f,0.35560000287890414227831570315174758434295654296875f,0.520300014477968364445814586360938847064971923828125f,-0.67839997608065605305682765902020037174224853515625f,-1.09619995911717449388333989190869033336639404296875f,-0.770999979391694179042815449065528810024261474609375f,-0.00759999668002109274311806075274944305419921875f,-0.31440000591278050023902324028313159942626953125f,-0.50920002365410343969642781303264200687408447265625f,0.54719999520778639379159358213655650615692138671875f,0.469499993279576255389429206843487918376922607421875f,0.4538000013202430071856952054076828062534332275390625f,-0.09859998151063908977675964706577360630035400390625f,-0.65329998506307607186727182124741375446319580078125f,-0.533499996453523639416971491300500929355621337890625f,-0.609800000977516187816718229441903531551361083984375f,-0.718099992522597485589130883454345166683197021484375f,-0.046499983876943762339806198724545538425445556640625f,0.4435999706327919955839433896471746265888214111328125f,-0.072300005783140275372744554260862059891223907470703125f,0.1225999932527539471038835472427308559417724609375f,0.74159997340440764190816480549983680248260498046875f,0.2445999742388724751407380608725361526012420654296875f,0.076300045651197745399940686183981597423553466796875f,0.6623999803721904999775915712234564125537872314453125f,0.070100016312301505205795137953828088939189910888671875f,-0.0535000138878825026722552138380706310272216796875f,0.6009999703764918876913725398480892181396484375f,0.1064999712854624025482053184532560408115386962890625f,-0.9706999962806701631734540569595992565155029296875f,0.13219999014139194315475833718664944171905517578125f,-0.7490999935209747828679383019334636628627777099609375f,-0.058799992752074814461593632586300373077392578125f,0.13419999780058855520792349125258624553680419921875f,0.0650000039115548045032255686237476766109466552734375f,-0.5822000124931339115619266522116959095001220703125f,0.40979998533129702309452113695442676544189453125f,-0.32960000988990056924876625998876988887786865234375f,-0.6746000102996827507695343228988349437713623046875f,-0.11459999882578841834401828236877918243408203125f,-0.500799996127188062899904252844862639904022216796875f,-0.8263000112295149079955081106163561344146728515625f,-0.3959999854266644803146846243180334568023681640625f,-0.8729000062778593527212933622649870812892913818359375f,-0.949000014275312775424708888749592006206512451171875f,-0.2310000143349164591910493982140906155109405517578125f,-0.8516000057026740233112604983034543693065643310546875f,0.82579997999668108832338475622236728668212890625f,0.284799992930889089848278672434389591217041015625f,0.886199978041648517290695963311009109020233154296875f,0.365200031656027324089563990128226578235626220703125f,0.7319999979734415074261733025196008384227752685546875f,0.330000001564621670002708242463995702564716339111328125f,-1.0158999979108573885611122022964991629123687744140625f,0.10939998147487683954892645488143898546695709228515625f,-0.94590000536143736820093863570946268737316131591796875f,0.0606999888300894241410787799395620822906494140625f,0.72719998373389227452889826963655650615692138671875f,0.1392999678447839340122982321190647780895233154296875f,0.695399989104270588313738699071109294891357421875f,0.3051999955952167109529682420543394982814788818359375f,0.2033999924719334106981705190264619886875152587890625f,0.487799981695413720217402442358434200286865234375f,0.2556999962806703052820012089796364307403564453125f,0.504600006276369139612825165386311709880828857421875f,0.46529999302029612096021082834340631961822509765625f,0.059899993559718200941688337479718029499053955078125f,-0.69760001053810116733444601777591742575168609619140625f,-0.863800002456456272881268887431360781192779541015625f,-0.3445999953612683963655172192375175654888153076171875f,-0.1059999976456165171612155972979962825775146484375f,0.09820001162886615730940320645458996295928955078125f,-0.447200019720197072814471539459191262722015380859375f,0.3695999921202659521668465458787977695465087890625f,0.981899993544816940271857674815692007541656494140625f,0.6767999893397089739011107667465694248676300048828125f,-0.3743999951839445117229843162931501865386962890625f,-0.45590000022053711603575720801018178462982177734375f,0.042000010713934887718323807348497211933135986328125f,0.096999989479780168721845257095992565155029296875f,-0.14190001023411724645484355278313159942626953125f,-0.44790002911388882012033718638122081756591796875f,0.409199988114833879127019145016674883663654327392578125f,1.0199999702535571532990843479637987911701202392578125f,0.461999995440244948241570455138571560382843017578125f,-0.383200004774332025903049725457094609737396240234375f,-0.101600000450015048869545353227294981479644775390625f,-0.2143999992817640087849895280669443309307098388671875f,-0.1696000001668929169085231478675268590450286865234375f,-0.55979999956190573673353583217249251902103424072265625f,0.079199999290704592436185293991002254188060760498046875f,0.41460001178979855041006885585375130176544189453125f,0.36970001934766738571624955511651933193206787109375f,-0.141299999755621019659201920148916542530059814453125f,0.47080001188516629184732664725743234157562255859375f,0.72340000215768807123595252051018178462982177734375f,0.128199989277124171138666497427038848400115966796875f,0.148799999159574536378158882143907248973846435546875f,-0.10560000466704355659430802916176617145538330078125f,-0.136200012090802236031095162616111338138580322265625f,0.14479998756647116664453278644941747188568115234375f,0.634599972450733407214329417911358177661895751953125f,0.0487999906659128956931681386777199804782867431640625f,0.062600018584728189807719900272786617279052734375f,-0.225599982762336781494383330937125720083713531494140625f,-0.04880000631213210482428621617145836353302001953125f,-0.16459999673962588673248319537378847599029541015625f,-0.125400001956522455515141700743697583675384521484375f,-0.309099999332428154730223468504846096038818359375f,0.54799998310208319907133045489899814128875732421875f,-0.3741000256702304849198981173685751855373382568359375f,0.51979999533370124797926337123499251902103424072265625f,-0.35759999120384444726283845739089883863925933837890625f,0.61700002190843239446849111118353903293609619140625f,1.0606000513076789815158917917869985103607177734375f,0.37960000255107839706170125282369554042816162109375f,0.8518000219732522193538670762791298329830169677734375f,-0.28829997910261084825833677314221858978271484375f,-0.59819997214078934888448202400468289852142333984375f,-0.398699971336126335330618530861102044582366943359375f,-0.0761000372409821324026779620908200740814208984375f,-0.48339997354745900537409397657029330730438232421875f,0.0462999937951560358584401910775341093540191650390625f,0.3361000607848172450076162931509315967559814453125f,0.76039998180270185201834465260617434978485107421875f,0.2715000240728262514977586761233396828174591064453125f,0.0443999884225423357975159888155758380889892578125f,-0.275999997906386684842772183401393704116344451904296875f,-0.14280000715777274589157741502276621758937835693359375f,0.8848000016480683793673733816831372678279876708984375f,-0.27679999060630799956328473854227922856807708740234375f,0.77440001224577426075512676106882281601428985595703125f,-0.661000005170702564782914123497903347015380859375f,-0.1891999828994277432769877123064361512660980224609375f,-0.43859998858869087001721709384582936763763427734375f,0.0349000207543372908958190237171947956085205078125f,0.69679996750950845552097234758548438549041748046875f,-0.0749000184074042341109134213184006512165069580078125f,0.02820000998973881678466568700969219207763671875f,0.33200000423192932430538348853588104248046875f,-0.129800015953183578432117428747005760669708251953125f,-0.2063999803423879342290092608891427516937255859375f,0.43079999161958681952455663122236728668212890625f,-0.17399999803304666556869051419198513031005859375f,0.6110000007748599415435819537378847599029541015625f,-0.0340000030994414004226200631819665431976318359375f,0.3933999975383282166063736440264619886875152587890625f,-0.37940000770092030535352023434825241565704345703125f,0.099799987044930649204843575716949999332427978515625f,0.2992000018239020509014380877488292753696441650390625f,0.66100000412762149881729101252858527004718780517578125f,0.109999995827674779125260329237789846956729888916015625f,0.0025999972760677447780608417815528810024261474609375f,0.04959999890029431757199063213192857801914215087890625f,0.243300000262260340821995896476437337696552276611328125f,-0.20499999150633829714251987752504646778106689453125f,-0.774099976831674752020262531004846096038818359375f,-0.643699995252489998165401630103588104248046875f,-0.44539998210072528905811850563623011112213134765625f,-0.81819996439218556361083756200969219207763671875f,-0.6018999810278415907305316068232059478759765625f,-0.2705999940872192421892350466805510222911834716796875f,-0.69379998850524426945440836789202876389026641845703125f,-0.437899998268485080021861222121515311300754547119140625f,-0.17359998806715015717827554908581078052520751953125f,-0.0004999825060370000073817209340631961822509765625f,-0.47959999249279494648590116412378847599029541015625f,-0.051799999509751781801014658412896096706390380859375f,-0.067099999291449774485585066940984688699245452880859375f,-0.0721000009305775291235107715692720375955104827880859375f,0.25579999564290034630431591722299344837665557861328125f,-0.15520000386536114689306486980058252811431884765625f,-0.002100005084276101996465513366274535655975341796875f,0.3689999962449075354697924922220408916473388671875f,0.49299998131394406897243243292905390262603759765625f,0.429099988082051186921717089717276394367218017578125f,0.10940001425743073326657395227812230587005615234375f,-0.22899996361136487621479318477213382720947265625f,0.22490002589523783171898685395717620849609375f,-0.0576000057697294287617495456288452260196208953857421875f,0.538599980709701942060974033665843307971954345703125f,0.1117999945923686677673458689241670072078704833984375f,0.457999999493360565594457511906512081623077392578125f,0.9192999976098537029400858955341391265392303466796875f,0.54399998670816385182291696764877997338771820068359375f,0.25999999716877919553326137247495353221893310546875f,-0.11469999580383305470832056016661226749420166015625f,0.125799998772144316916410389239899814128875732421875f,-0.06539999446868904886542850363184697926044464111328125f,-0.247499984912574466644485937649733386933803558349609375f,0.0875000107288359807444777516138856299221515655517578125f,-0.337800003302097362478662034845910966396331787109375f,-0.33130003842413469072880616295151412487030029296875f,0.273400016686320412873101304285228252410888671875f,0.1139999934136868287026800317107699811458587646484375f,0.0981999991863966581462364047183655202388763427734375f,-0.323400001487135657640692443237639963626861572265625f,-0.3209999861717225400070674368180334568023681640625f,-0.264799988721311108097467013067216612398624420166015625f,0.24950001645088182300469270558096468448638916015625f,0.742799965006113449561553352396003901958465576171875f,0.31969999107271462879253931532730348408222198486328125f,0.5491999887108800493251692387275397777557373046875f,0.33859998136162783310965096461586654186248779296875f,0.5989999813064932343564805705682374536991119384765625f,0.15560000340044499722580439993180334568023681640625f,0.3569999899268150311826275355997495353221893310546875f,0.24459999595731496402351012875442393124103546142578125f,1.0868999975681301606300621642731130123138427734375f,0.34819998346567171410015362198464572429656982421875f,0.8940999793648718441119171984610147774219512939453125f,0.109300001614540842620471039481344632804393768310546875f,0.0336000005096196707743416709490702487528324127197265625f,0.1027000005684792771631919094943441450595855712890625f,-0.690199984344840089534045546315610408782958984375f,0.4369999771118167064543058586423285305500030517578125f,-0.5703999995723363358735014116973616182804107666015625f,-0.6195999998688694887505334918387234210968017578125f,-0.13619998236298602733995721791870892047882080078125f,-0.6061999974504115318296726400149054825305938720703125f,-0.9036000154852867893850998370908200740814208984375f,-0.05180001727938599032086131046526134014129638671875f,-0.7114000121936199239058851162553764879703521728515625f,0.73019998864755020218098024997743777930736541748046875f,-0.39199997871369152147025261001545004546642303466796875f,0.5290000003986057297566958368406631052494049072265625f,-0.3268999777495853464870378957130014896392822265625f,-0.276199976849556438907029587426222860813140869140625f,-0.4348999829798938510094785669934935867786407470703125f,-0.8436999834060667780022413353435695171356201171875f,0.26819998137950928907002889900468289852142333984375f,-0.6984999939203258012554442757391370832920074462890625f,-0.3943000086739658627266180701553821563720703125f,-0.13599999950826158201522275703609921038150787353515625f,-0.2769000004738568776474494370631873607635498046875f,-0.4405000211000444920728114084340631961822509765625f,-0.465600009956955940282341543934307992458343505859375f,0.3993999991327523702011603745631873607635498046875f,0.1385999946609139943376476367120631039142608642578125f,0.19869999476820232597873427948798052966594696044921875f,0.2237999965891244169569063160452060401439666748046875f,0.27149998236820127583968087492394261062145233154296875f,0.352299985796958037287396336978417821228504180908203125f,0.1158000068187714504119867342524230480194091796875f,-0.195499991893768321205016036401502788066864013671875f,0.3700999996602531982858863557339645922183990478515625f,0.243600013846158702790489769540727138519287109375f,0.256500032007693778979273702134378254413604736328125f,-0.2655999940872193487706454106955789029598236083984375f,0.7398000076830388849202790879644453525543212890625f,1.21619998847246169049185482435859739780426025390625f,0.665099992209672752352389579755254089832305908203125f,0.23640000060796728664769261740730144083499908447265625f,0.172800003376603061866489952080883085727691650390625f,0.0477999965965747453111589493346400558948516845703125f,0.0613999980747699236616199414129368960857391357421875f,0.16089999761283391155330946276080794632434844970703125f,0.326300011453032612340763307656743563711643218994140625f,-0.356399989283084916991128920926712453365325927734375f,-1.0238999708056450987925245499354787170886993408203125f,-0.57300000418722663031445563319721259176731109619140625f,0.817800017458200301234683138318359851837158203125f,1.37170001955628340084558658418245613574981689453125f,0.871300019052624552529096035868860781192779541015625f,0.46459999867677692630962837938568554818630218505859375f,0.377399987357109889529738211422227323055267333984375f,0.1958999921366573193637350414064712822437286376953125f,-0.44039999253153805369720430462621152400970458984375f,-0.64729997951984419302107198745943605899810791015625f,-0.555999995410442426901909129810519516468048095703125f,-0.4317999906063079151863348670303821563720703125f,-0.5182999843180180477020257967524230480194091796875f,-0.4404999951720238726693423814140260219573974609375f,-0.096600007304548984876646500197239220142364501953125f,0.3573999823465945357980899643735028803348541259765625f,0.48829999582916483458205902934423647820949554443359375f,-0.25499999418854724098082442651502788066864013671875f,-0.318999984771013433970665573724545538425445556640625f,-0.4790999923288821182865149239660240709781646728515625f,0.07359999387860316488740863860584795475006103515625f,0.63770000662207593222774448804557323455810546875f,-0.83319999207109207706167808282771147787570953369140625f,0.310199987250566577845489746323437429964542388916015625f,-0.73980000492632369546441850616247393190860748291015625f,-0.5656999978639181136941260774619877338409423828125f,-0.1915999737024313420707954946919926442205905914306640625f,-0.51629999113157420964625998749397695064544677734375f,-1.0995000045299523350195158855058252811431884765625f,-0.2371999762833123526206691167317330837249755859375f,-0.8728999921962621311166685700300149619579315185546875f,0.3743000248789787320902178180404007434844970703125f,0.63019996474981354595001903362572193145751953125f,0.2342999865084889332678130813292227685451507568359375f,-0.5856000098824498678595773526467382907867431640625f,-0.09299998250603724869733923696912825107574462890625f,-0.7170000190958385477557612830423749983310699462890625f,-0.244399985423684018570611442555673420429229736328125f,0.245199988144636282783039860078133642673492431640625f,-0.2495999998688696042137280528550036251544952392578125f,-0.39670002276003391017411559005267918109893798828125f,-0.3711999971151349608788905243272893130779266357421875f,-0.34490000514537089149058601833530701696872711181640625f,0.0339000302612788573242141865193843841552734375f,0.498599994623660780490581601043231785297393798828125f,0.198300013896822946435349876992404460906982421875f,-0.8263000258326531621833055396564304828643798828125f,-0.28440000188946701342729284078814089298248291015625f,-0.9389000313416124843257648535654880106449127197265625f,-0.58839999558255084988189764771959744393825531005859375f,0.34059998794049028969510573006118647754192352294921875f,-0.58140001150444142741235964422230608761310577392578125f,0.722100007915496888699635746888816356658935546875f,0.2361999926447868602963353623636066913604736328125f,0.71650000123679635777307339594699442386627197265625f,0.713299995344877313385723027749918401241302490234375f,0.0632000053703782871394878384307958185672760009765625f,0.693099996285140296237159418524242937564849853515625f,-0.07920001668781007087005718858563341200351715087890625f,-0.876199953454733559965461608953773975372314453125f,-0.09139998190701048574879905572743155062198638916015625f,0.755600001984834346302477570134215056896209716796875f,0.3201999795019629946324357661069370806217193603515625f,0.6781999898359181155882424718583934009075164794921875f,-1.2245000188350676495474544935859739780426025390625f,-0.109400003901124254213073072605766355991363525390625f,0.0370000031888484404163364160922355949878692626953125f,0.28050000739097580293446299037896096706390380859375f,-0.028299997654557262904972958494909107685089111328125f,-0.487200003254413704922853867174126207828521728515625f,-0.34709999018311510354806159739382565021514892578125f,-0.409199994000792433013202753500081598758697509765625f,-0.313199991512298492324362086947076022624969482421875f,-0.8961999847471713298574513828498311340808868408203125f,-0.59060001167058973070567162721999920904636383056640625f,-0.47879998947381974261361392564140260219573974609375f,-0.6435999764442446036127876141108572483062744140625f,-0.16819998197555552366111442097462713718414306640625f,0.4604000069856646337740357921575196087360382080078125f,0.528199980336427898919282597489655017852783203125f,0.1305999890208244806899529066868126392364501953125f,-0.141400006270408606923183469916693866252899169921875f,-0.3009000109493731311971487230039201676845550537109375f,0.26750001206994078462031438903068192303180694580078125f,0.33920000525116922318602519226260483264923095703125f,0.14419999750256540238524394226260483264923095703125f,0.025799996313452755458683895994909107685089111328125f,0.272200002658367079533263677149079740047454833984375f,0.70350000695884207235764051802107132971286773681640625f,0.276300000500678877290994250870426185429096221923828125f,-0.65680000237822522279884651652537286281585693359375f,-0.903399995452165427423096843995153903961181640625f,-0.7074000106960536271571982069872319698333740234375f,0.40720000399947142710743719362653791904449462890625f,0.31310001092553108037463971413671970367431640625f,0.57860002084970485469739287509582936763763427734375f,0.21820000686049478844097393448464572429656982421875f,-0.031200003969669154457733384333550930023193359375f,-0.3681000231444837567096328712068498134613037109375f,0.0359999913871289489719629273167811334133148193359375f,0.4488999795973303008622679044492542743682861328125f,-0.00260001158118239317218467476777732372283935546875f,0.3514000037372111417965925284079276025295257568359375f,0.5610000071823595391151684452779591083526611328125f,0.09729999397397026683620424591936171054840087890625f,-0.52240001479387299099244046374224126338958740234375f,-0.392299998444318731571911484934389591217041015625f,-0.4636000049799680056139550288207828998565673828125f,-0.5179999960660932600831074523739516735076904296875f,0.59500001639127741981383223901502788066864013671875f,0.39930000386834141323788571753539144992828369140625f,0.00759999638199815308325923979282379150390625f,0.2810999971196055380318057359545491635799407958984375f,1.0191000061869619486287774634547531604766845703125f,-0.11220000504255267514963634312152862548828125f,0.92550001126527803307908470742404460906982421875f,0.1758000272333628544174644048325717449188232421875f,0.548000000834464540133694754331372678279876708984375f,0.2906000096589325121243518879055045545101165771484375f,0.1703999771833419885069815791212022304534912109375f,-0.31719998894929890553839868516661226749420166015625f,0.2803999975234268138279958293423987925052642822265625f,-0.24940001440644277153069197083823382854461669921875f,0.37200000333785965267452411353588104248046875f,-0.3274000306636100088297780530410818755626678466796875f,-0.628100033873320029442766099236905574798583984375f,-0.464399991756677454901591772795654833316802978515625f,-0.44670000245720142828531606937758624553680419921875f,-0.43699999946355827518118530861102044582366943359375f,0.437599986398220064387487582280300557613372802734375f,-0.41500001758337023982647906450438313186168670654296875f,0.11710000828802602512723751715384423732757568359375f,-0.1515999972462653833105150624760426580905914306640625f,-0.0278999983057379719042501164949499070644378662109375f,0.9730000234842304251969835604541003704071044921875f,0.17840000811815226455792071647010743618011474609375f,0.9782000220596789308302732024458236992359161376953125f,-1.199500024944544218641340194153599441051483154296875f,-0.2146000150680535956126959717948921024799346923828125f,-1.0543000157892701285078373985015787184238433837890625f,0.15460003086328555355066782794892787933349609375f,0.8089999946057790936038145446218550205230712890625f,0.095999993957578855230394765385426580905914306640625f,0.61190000606179228981318374280817806720733642578125f,-0.07739999214410797634400296374224126338958740234375f,0.431500004440545925632477519684471189975738525390625f,0.05000002190470720364601220353506505489349365234375f,0.2393999914586544264238909818232059478759765625f,-0.0762000002816318744436330234748311340808868408203125f,1.102700032401084939692736952565610408782958984375f,-0.082600007259845398266406846232712268829345703125f,1.00270003418922470928009715862572193145751953125f,0.83960001208782220061266343691386282444000244140625f,-0.37679998986124996207536241854541003704071044921875f,0.045600014501810282041560640209354460239410400390625f,-0.6289999981820582064528935006819665431976318359375f,-0.18820000022947791507021975121460855007171630859375f,-0.05400001144409205533492013273644261062145233154296875f,0.4950000087171790852380581782199442386627197265625f,0.14459999912381160669383461936376988887786865234375f,0.442400005108118055119348355219699442386627197265625f,0.758100001239776499772915485664270818233489990234375f,0.2241999937772749529329985307413153350353240966796875f,0.63859998419284824677788492408581078052520751953125f,0.92309996219873458045412917272187769412994384765625f,0.602299988311529421736167932976968586444854736328125f,0.300400011605024275240793940611183643341064453125f,0.11020000960230813547013895004056394100189208984375f,-0.309600001433491645030926520121283829212188720703125f,-0.418999991476535882384268916212022304534912109375f,-0.854399976166337893346280907280743122100830078125f,-0.265899987964332229495312276412732899188995361328125f,-0.1860000068843365994553096243180334568023681640625f,0.115300005686282958095034700818359851837158203125f,0.3752000128805637046980336890555918216705322265625f,-0.2506000139057638076423017992055974900722503662109375f,-0.0598000006794929817033334984444081783294677734375f,0.031500003397464837462393916212022304534912109375f,-0.30640001610517497709906820091418921947479248046875f,-0.391100022190808971345177269540727138519287109375f,0.1540000055581334237331248004920780658721923828125f,0.1645999937593938255986358853988349437713623046875f,0.86019998143911369226088936557061970233917236328125f,0.560700000303983703275889638462103903293609619140625f,0.28359999551773074077942737858393229544162750244140625f,0.6533999916896224835483053539064712822437286376953125f,0.39619999234676350763351138084544800221920013427734375f,-0.389400003230571611911869922550977207720279693603515625f,-0.85240001065880033337407439830712974071502685546875f,-0.299299998354911611642137359012849628925323486328125f,-0.107799993616342515423411896335892379283905029296875f,0.1037000073969363000969678978435695171356201171875f,-0.13669999534189702927733378601260483264923095703125f,0.34859999432563792964856475009582936763763427734375f,0.519799977582693362165855432976968586444854736328125f,-0.1443000224202870551692967637791298329830169677734375f,-0.1676000075578689685329436542815528810024261474609375f,0.62300001439452190599155301242717541754245758056640625f,-0.7206000206112861405927105806767940521240234375f,0.49839999917745547719505339046008884906768798828125f,-0.5090000279843811537006104117608629167079925537109375f,0.605400032839179313981503582908771932125091552734375f,-0.415199999767541516604296703008003532886505126953125f,0.4338000349223618545835279292077757418155670166015625f,-0.023699988770484736733124009333550930023193359375f,0.1561999983072279718498975853435695171356201171875f,-0.1713000076532364968073807176551781594753265380859375f,-0.7409000027537349097883634385652840137481689453125f,-0.07800000306963905671864267787896096706390380859375f,-0.7923000033989546775359258390381000936031341552734375f,0.12380000884532904592560953460633754730224609375f,0.8163999681234361815995725919492542743682861328125f,0.042999972000718145181963336654007434844970703125f,-0.09540000663176162465362040165928192436695098876953125f,-0.03859999846071004991898689695517532527446746826171875f,0.0840000034905969972243156007607467472553253173828125f,-0.0981000107765197526532574556767940521240234375f,-0.794599983775615470449338317848742008209228515625f,-0.06529997983276825124221431906335055828094482421875f,0.311200012016296323480446517351083457469940185546875f,-0.4018000011861320164285871214815415441989898681640625f,0.291800027002394657404948929979582317173480987548828125f,-0.282500024437904784235797706060111522674560546875f,-0.54259998937845210775776649825274944305419921875f,-0.13609998866319639176936107105575501918792724609375f,0.0863999791502951808297439129091799259185791015625f,-0.72419997127652191437618967029266059398651123046875f,-0.0553999978587034203059147330350242555141448974609375f,-0.17219999070018499054413041449151933193206787109375f,-0.27199998567998429077618993687792681157588958740234375f,-0.25499998901039366838716659913188777863979339599609375f,-0.0918000190302732388403228469542227685451507568359375f,0.0482000023156403667456970651983283460140228271484375f,0.08239999601840963805443607270717620849609375f,0.5704999924898146446139435283839702606201171875f,-0.02039999819397930735931367962621152400970458984375f,0.4594999906346199924911388734471984207630157470703125f,0.383600025320053106270279386080801486968994140625f,0.55539998925328237788789920159615576267242431640625f,0.195799995382130109788931804359890520572662353515625f,0.83630000988841057818490298814140260219573974609375f,-0.35689999563097962731461620933259837329387664794921875f,0.210599999749660427017516894920845516026020050048828125f,0.24350000066310162338822919991798698902130126953125f,0.092699997755885110706230989308096468448638916015625f,-0.1754000025153159558755078251124359667301177978515625f,0.09039999476671223277435274212621152400970458984375f,-0.133100000271201057699954617419280111789703369140625f,-0.2958000045835973423891118727624416351318359375f,0.210900007671117606378174968995153903961181640625f,-0.3786999965935944345574171165935695171356201171875f,0.0456000049650668870526715181767940521240234375f,-0.313099997961521214762115050689317286014556884765625f,0.3058000199317933986975504012661986052989959716796875f,-0.391199995923042109779999009333550930023193359375f,-0.06959999420642848377838163287378847599029541015625f,-0.382800011348724655846353925880976021289825439453125f,-0.275200016903877298091174452565610408782958984375f,-0.26500001102685910581158168497495353221893310546875f,0.169599997037649163900141502381302416324615478515625f,-0.004400001740455738996615764335729181766510009765625f,-0.62309999721646303072475348017178475856781005859375f,-0.231000000998377696959096283535473048686981201171875f,0.30940001485347767840039523434825241565704345703125f,0.013999997735023583800284541212022304534912109375f,-0.17710000709593298751087786513380706310272216796875f,-0.0885999882161616625353417475707828998565673828125f,-0.66509999250769613610145825077779591083526611328125f,-0.392299995240569021603960209176875650882720947265625f,-0.03860000505447391816460367408581078052520751953125f,0.041199997487664230533255249611102044582366943359375f,0.1131000013142824389689167219330556690692901611328125f,0.12539999297857296056690756813623011112213134765625f,0.409099985474348226688334762002341449260711669921875f,0.3524999906122683324127820014837197959423065185546875f,0.291200002181529971068130180356092751026153564453125f,0.530700000602006838335000793449580669403076171875f,0.0535999936550854982897362788207828998565673828125f,-0.21760000532269485606917669429094530642032623291015625f,-0.116899996972084085911092188325710594654083251953125f,-0.0315999966502189533201772064785473048686981201171875f,-0.52420000748634354437172078178264200687408447265625f,-0.36449999037384994693411499611102044582366943359375f,-0.576700000035762716521503534750081598758697509765625f,-0.1562000012874603382950766672365716658532619476318359375f,0.69099999097734698949579978943802416324615478515625f,-0.1644999910071491822094458257197402417659759521484375f,0.127599992284178842538722165045328438282012939453125f,-0.190300000545382375083391934822429902851581573486328125f,0.6665999959051607248738946509547531604766845703125f,-0.123399995452165622822349178022705018520355224609375f,0.54759999999552955163295564489089883863925933837890625f,1.460900015270709673131932504475116729736328125f,-0.02920001359581902988793444819748401641845703125f,1.1859000077828767416576738469302654266357421875f,-1.0453000438332562538334968849085271358489990234375f,-0.2262000024795529640186941833235323429107666015625f,-0.8057000216126442015962538789608515799045562744140625f,1.219200017619133546986631699837744235992431640625f,-0.083999975085259137586035649292171001434326171875f,1.039200014787911907632178554194979369640350341796875f,-0.1749999916553495182824917719699442386627197265625f,-0.55319998376369472481428601895458996295928955078125f,-0.0693999650835988024510925242793746292591094970703125f,-0.5524999889731407876070079510100185871124267578125f,0.4753999765872958249701696331612765789031982421875f,-0.368099999973177549605907188379205763339996337890625f,-0.250600008243322402989861075184307992458343505859375f,0.6901999920189376691581628620042465627193450927734375f,-0.246400034098327669340733336866833269596099853515625f,-0.20310001465082194727074238471686840057373046875f,0.5659999836385249949444187222979962825775146484375f,-0.07170002644807116354286335990764200687408447265625f,-0.0513999996393919023063290296704508364200592041015625f,-0.21499999657273283215630499398685060441493988037109375f,-0.0403999910786746740942732003532000817358493804931640625f,-0.182499996274709364030286451452411711215972900390625f,-0.3823999764978889270850004322710447013378143310546875f,-0.211899985013902336650204460966051556169986724853515625f,0.183700014698505498245140188373625278472900390625f,0.21299999338388442282621326739899814128875732421875f,0.0428999961078166958117208196199499070644378662109375f,0.2681000039219856301286881716805510222911834716796875f,-0.25419999720454200531349897573818452656269073486328125f,0.25930001415014281018756037155981175601482391357421875f,1.330300025504827399203122695325873792171478271484375f,0.0503999871671203525380633436725474894046783447265625f,1.181700020934641770509188063442707061767578125f,-0.00950000127032402108540054541663266718387603759765625f,0.079299999323487302405055743292905390262603759765625f,0.737600004577636969571585723315365612506866455078125f,1.159999977648258351337062777020037174224853515625f,0.6391999884128569675567632657475769519805908203125f,0.13340000320076939033242524601519107818603515625f,0.071500006824731787702376095694489777088165283203125f,-0.2033000106185676170156284570111893117427825927734375f,-0.131799989414215001914953973027877509593963623046875f,0.45410001512765862941023442544974386692047119140625f,-0.119700000572204867665959682199172675609588623046875f,0.213799978744983576461891061626374721527099609375f,1.17999998196959499097147272550500929355621337890625f,0.589099988231062798860193652217276394367218017578125f,-0.16320002205967920616558330948464572429656982421875f,0.41659998636841777397421537898480892181396484375f,0.24300000090897100335496361367404460906982421875f,0.124800009173154702324382014921866357326507568359375f,0.2192000140428540522208322727237828075885772705078125f,-0.22019999254047861736438562729745171964168548583984375f,-0.45539999879002568405894635361619293689727783203125f,-0.821799994480609807823157098027877509593963623046875f,-0.6233000014543532696364991352311335504055023193359375f,-0.0079999977052211956873861709027551114559173583984375f,-0.400199996936321245044609895558096468448638916015625f,-0.0077999956279992677110612930846400558948516845703125f,-0.0355999936401845129552157231955789029598236083984375f,-0.552799983036518227663691504858434200286865234375f,-0.0735999930590391926443771808408200740814208984375f,-0.05399999676644806623215799845638684928417205810546875f,-0.126699993702769353465242829770431853830814361572265625f,-0.1715999966129660159186443024736945517361164093017578125f,0.392199998337030564243832486681640148162841796875f,0.64419998528361333001157618127763271331787109375f,0.58009999176263793430052828625775873661041259765625f,-0.56979999009966864065290792495943605899810791015625f,-1.21299996894598027807887774542905390262603759765625f,-0.650499987274408386639379386906512081623077392578125f,-0.6618000065505504903029532215441577136516571044921875f,-0.84209999510049815540924100787378847599029541015625f,-0.258499990567564896792873696540482342243194580078125f,-0.42519998844265938231501422706060111522674560546875f,-0.31379998083114646334479402867145836353302001953125f,-0.491999986723065330096460456843487918376922607421875f,0.583000017672777204325029742904007434844970703125f,0.39810002549141654260012046506744809448719024658203125f,-0.6240000141263006838698856881819665431976318359375f,-0.046399981236458387456877971999347209930419921875f,-0.740800021719933088348852834315039217472076416015625f,0.19510002871751819242263081832788884639739990234375f,0.439599990183114908148809263366274535655975341796875f,0.12850000394135718817523184043238870799541473388671875f,1.11130000526905092783636064268648624420166015625f,-0.18399998149275820225057032075710594654083251953125f,0.8542999999567861824090186928515322506427764892578125f,0.4310000281929973198202787898480892181396484375f,-0.7821999786674975752021055086515843868255615234375f,0.17560004083961278098513503209687769412994384765625f,0.5258000109910963004722361802123486995697021484375f,-0.68239999512433957562507202965207397937774658203125f,0.3950000327825551327265429790713824331760406494140625f,0.4592000001847740264793173992075026035308837890625f,0.403599974656105420223184410133399069309234619140625f,0.3235999783813954255862199715920723974704742431640625f,0.5964000068664550013863845379091799259185791015625f,0.3517999904572963743021318805404007434844970703125f,0.6386000012174248841034795987070538103580474853515625f,-0.57260000800490384875729432678781449794769287109375f,0.11839999411106116866676529753021895885467529296875f,-0.4098000052988528540964807689306326210498809814453125f,-0.64519999857544885202287332504056394100189208984375f,0.206399989283085005808970890939235687255859375f,-0.4460000011473892467250834670267067849636077880859375f,0.18469999967813477415035094963968731462955474853515625f,0.174999989941716338481825232520350255072116851806640625f,0.1702999933183193537189481503446586430072784423828125f,-0.678800005418061846995669839088805019855499267578125f,-0.3389999749362473391300909497658722102642059326171875f,-0.6745999969258906769908890055376105010509490966796875f,0.2604000031113624658019034541212022304534912109375f,0.33099998304247879588046998833306133747100830078125f,0.4322000004604460432489076993078924715518951416015625f,0.34869999070763579851472968584857881069183349609375f,-0.41959999018311489038524086936376988887786865234375f,0.472500019110739355454597898642532527446746826171875f,1.090999984830617552944431736250407993793487548828125f,0.0771999940156937736190911891753785312175750732421875f,0.9701999833017584951022627137717790901660919189453125f,-0.87670002536773683488036112976260483264923095703125f,0.402000003710389020028515005833469331264495849609375f,0.2560000074803827629921215702779591083526611328125f,-0.153299989160895488993219260009936988353729248046875f,-0.2339999969899653198268651976832188665866851806640625f,-0.285800001752376520158804851234890520572662353515625f,0.03510000100135801659462231327779591083526611328125f,-0.162299999338388456493476041941903531551361083984375f,-0.548199998217821171664354551467113196849822998046875f,-1.0150999874413013657203919137828052043914794921875f,-0.47649998991191377939458106993697583675384521484375f,-0.173599992239475364730338924346142448484897613525390625f,-0.6154999783709647065421677325502969324588775634765625f,-0.19759999131560335161594821329344995319843292236328125f,0.120000004619360112201320589520037174224853515625f,-0.17530000345110874349074947531335055828094482421875f,-0.394600020730495604226462091901339590549468994140625f,-0.13180001131892193910744026652537286281585693359375f,0.45399998879432690301882757921703159809112548828125f,0.331299990218877571379607616108842194080352783203125f,0.32800001588463789659044778090901672840118408203125f,-0.090899991428852189301323960535228252410888671875f,0.2566000105082988369531449279747903347015380859375f,0.1988000060141086322573755751363933086395263671875f,0.117400007790326998957652904209680855274200439453125f,-0.1232000069350005500012912307283841073513031005859375f,-0.071399976617097848929915926419198513031005859375f,-0.7588999745309354949540647794492542743682861328125f,-0.52609998724758622046238087932579219341278076171875f,-0.095200005578994773003387308563105762004852294921875f,0.10569999508857730319277834496460855007171630859375f,-0.074000001683831140297797901439480483531951904296875f,-0.10720000683069219604703903314657509326934814453125f,-0.1812000115692613544382538748322986066341400146484375f,0.12759999578595160318883472427842207252979278564453125f,0.4609999943673610545147312222979962825775146484375f,1.30739998260736456359154544770717620849609375f,0.771300009292364308066680678166449069976806640625f,0.1125999987661838730446106637828052043914794921875f,-0.006400007462501466903859181911684572696685791015625f,0.2219000050187112105248843363369815051555633544921875f,0.022000001400709123799970257095992565155029296875f,0.218800005120038942862947806133888661861419677734375f,-0.3728000124663115588674600076046772301197052001953125f,-0.21219999043941495386889073415659368038177490234375f,0.01160003263652331639832482323981821537017822265625f,-0.3130999923735859891138488819706253707408905029296875f,0.73219996331930203847804250472108833491802215576171875f,-0.2691000211626288773913984186947345733642578125f,0.8299000001907348433860533987171947956085205078125f,-0.236799990159273221479452331550419330596923828125f,0.7005000072792171295077423565089702606201171875f,0.2368999922037122729534530662931501865386962890625f,0.08239999199509639993266318924725055694580078125f,0.387100001843273755497421007021330296993255615234375f,-0.2900999979838727593772773616365157067775726318359375f,0.1741999904245139507708017845288850367069244384765625f,-0.13490000000447022188154733157716691493988037109375f,0.158500029534101916084409822360612452030181884765625f,0.7373999936342234828856589956558309495449066162109375f,0.130099997723102378532900047503062523901462554931640625f,0.38260001173019464459912342135794460773468017578125f,-0.758599961990118476506950173643417656421661376953125f,0.27160003416389211228221256533288396894931793212890625f,-1.1419000303506852134205473703332245349884033203125f,0.19739999721050249803511178470216691493988037109375f,-0.9495000317990782701116359021398238837718963623046875f,0.0166000192999842255403564195148646831512451171875f,0.4227999898910521636707926518283784389495849609375f,-0.0290000037699937873725275494507513940334320068359375f,-0.15850001895427734410759512684307992458343505859375f,0.790799973589181970368144902749918401241302490234375f,-0.02750004094094060658193257040693424642086029052734375f,0.449500004976987721505565787083469331264495849609375f,-0.0818000010371207064707732570241205394268035888671875f,0.360700006264448258530563862223061732947826385498046875f,-0.28440000367760642774328516679815948009490966796875f,-0.270399984037876350839724182151257991790771484375f,-0.19079998759627347926226548224803991615772247314453125f,-0.1180000303387647164754525874741375446319580078125f,-0.67819998957514737725205122842453420162200927734375f,-0.3138000129058957643479743637726642191410064697265625f,-0.57100001865625404917636842583306133747100830078125f,-0.110199999767541800821391007048077881336212158203125f,-0.47200000866502522800516317147412337362766265869140625f,0.890400033509731514413942932151257991790771484375f,0.44239998886585230053469786071218550205230712890625f,0.68320000499784949710146975121460855007171630859375f,1.211900002634525463918180321343243122100830078125f,0.000299998850375404746415597401210106909275054931640625f,0.451600016766786627187002522987313568592071533203125f,0.6211000095248220276289430330507457256317138671875f,0.10680000163316716310646370402537286281585693359375f,0.01620001723170272356355781084857881069183349609375f,-0.59489997634887714639262412674725055694580078125f,-0.55269999537169933745417438331060111522674560546875f,-0.15199999186396606631888062111102044582366943359375f,-0.074199987220764374029613463790155947208404541015625f,-0.2937999865680931410594212138676084578037261962890625f,0.0668000071465968403572333045303821563720703125f,0.0418000117659567127503805750166065990924835205078125f,-0.10719999587833883847309834891348145902156829833984375f,-0.010399999833107020918987473123706877231597900390625f,0.01830000227391714862079652448301203548908233642578125f,0.092400004139542579739696748220012523233890533447265625f,0.23779999648481597862570424695149995386600494384765625f,0.07469999731257559449915106597472913563251495361328125f,0.091499996226280966293842311642947606742382049560546875f,0.169799998742341973212433003936894237995147705078125f,0.4505999949812888871747418306767940521240234375f,0.00289999715089805931711453013122081756591796875f,-0.32799999919533728842679920489899814128875732421875f,-0.631599991434812491064576533972285687923431396484375f,-0.4890000073462725804773754134657792747020721435546875f,-0.3650000189244748671768547865212894976139068603515625f,-0.0393000072956084522957098670303821563720703125f,-0.146900003454089045362707111053168773651123046875f,0.21699999529123292774102083058096468448638916015625f,0.587799997937679119530685056815855205059051513671875f,0.6215000150948764012781566634657792747020721435546875f,0.23899999833106999602705400320701301097869873046875f,0.0038999904751779723710569669492542743682861328125f,-0.36030002237558367284009364084340631961822509765625f,-0.236000001668929926257334273032029159367084503173828125f,0.14689999395459896192051019170321524143218994140625f,-0.017599999511241914973425082280300557613372802734375f,-0.21799996998906134848539295489899814128875732421875f,-1.06849996322393447911736075184307992458343505859375f,-0.598799979788064984376205757143907248973846435546875f,-0.585399987018108358682866310118697583675384521484375f,-1.2153999721169472802984046211349777877330780029296875f,-0.70489999475181119859001910299411974847316741943359375f,-0.154200001972913780523555260515422560274600982666015625f,-0.1832000082202256407981622032821178436279296875f,-0.0559000081181526553564253845252096652984619140625f,0.3609999930262564049598950077779591083526611328125f,-0.18230003159940277868145130923949182033538818359375f,-0.027699994403123984199055485078133642673492431640625f,0.9079999667108060723563767169252969324588775634765625f,0.045299973015487271510437494725920259952545166015625f,0.637399984544515074702530910144560039043426513671875f,0.1663999827265743025606070659705437719821929931640625f,0.5401999786078930387844820870668627321720123291015625f,0.115400001770257887301340815611183643341064453125f,0.225199988591671118598469547578133642673492431640625f,0.0777999898165464554011805375921539962291717529296875f,-0.103100005486607582128044668934307992458343505859375f,-0.13899999788403505363021395169198513031005859375f,-0.083099999526143053518723036177107132971286773681640625f,-0.464499987598508579100808901785057969391345977783203125f,0.2211999865353109495647032645138096995651721954345703125f,-0.4142999977402386235780795686878263950347900390625f,0.3644000047206876757854843162931501865386962890625f,0.117799991828203332033808692358434200286865234375f,0.214199993591010606763802570640109479427337646484375f,0.4348000227332118328149590524844825267791748046875f,0.31639999643564209463875158689916133880615234375f,0.4688000127196312316613102666451595723628997802734375f,0.83630002300143235771656691213138401508331298828125f,-0.35400000086426697265551410964690148830413818359375f,0.725700034315884590085943273152224719524383544921875f,-0.514400023645162907115491179865784943103790283203125f,-0.6401999949991699079276941120042465627193450927734375f,-0.5114000007942316461395648730103857815265655517578125f,0.15710003712177300627672593691386282444000244140625f,0.819999981373548347818314141477458178997039794921875f,0.09089999995976694169286247415584512054920196533203125f,0.6738999885752796625837390820379368960857391357421875f,-0.3793999806553127740471609286032617092132568359375f,0.5555000029578802678287274829926900565624237060546875f,-0.2407000290632250738553921109996736049652099609375f,0.75459997602701189833851458388380706310272216796875f,-0.0359000457435849629206359168165363371372222900390625f,0.37280000553727177248219959437847137451171875f,0.377399995571374713421164415194652974605560302734375f,0.524600000427663193391936147236265242099761962890625f,-0.704400046909601229572217562235891819000244140625f,0.2931000148743390099070893484167754650115966796875f,-0.176399995243549323475917844916693866252899169921875f,-0.670399994468688920079557647113688290119171142578125f,-0.2825999971270560795488790972740389406681060791015625f,0.086600008273124473578974402698804624378681182861328125f,-0.6360999752148988051203559734858572483062744140625f,-0.1204999930113554995614322251640260219573974609375f,-0.21079998975694191898355711600743234157562255859375f,-0.380199980358779665579049833468161523342132568359375f,-0.512999988205730783619173962506465613842010498046875f,0.20980001111030599503237681346945464611053466796875f,-0.15160000365376458120181268895976245403289794921875f,-0.117300008651614273702534774201922118663787841796875f,-0.26739999363422395362022143672220408916473388671875f,-0.4404999860823155533040562659152783453464508056640625f,-0.22949999260902409492501874410663731396198272705078125f,-0.011199999946355898572392106871120631694793701171875f,-0.43659999620318412638653171597979962825775146484375f,-0.052599996009468963364952287520281970500946044921875f,0.82600000569224363289322354830801486968994140625f,1.19129998619556420891285597463138401508331298828125f,0.726400002396106803104203208931721746921539306640625f,0.387200006532669060987927878159098327159881591796875f,0.6801000053226946562290322617627680301666259765625f,0.02599998937547187694008243852294981479644775390625f,-0.229200004953145797248481585484114475548267364501953125f,-0.5349000124655660659556133396108634769916534423828125f,-0.495000021904707043773896657512523233890533447265625f,0.0011999988287687557431127061136066913604736328125f,0.18869999700337647396963802748359739780426025390625f,0.1808000033542513929063488831161521375179290771484375f,-0.60400001353025434269738980219699442386627197265625f,-0.64260000487565971383219221024774014949798583984375f,-0.526400013422966051024332045926712453365325927734375f,-0.2162000005424022930355931748636066913604736328125f,-0.047499999850988405825091831502504646778106689453125f,-0.4404999994188545375806143056252039968967437744140625f,0.422199996548890954528587826644070446491241455078125f,1.0768999936938283834564344942918978631496429443359375f,0.71390001275241388167813738618860952556133270263671875f,-0.244399991607666067228166184577275998890399932861328125f,-0.6241999789319934155429336897213943302631378173828125f,-0.257599994891882122516335584805347025394439697265625f,0.349200006294250187011130037717521190643310546875f,-0.30380000109672522512482828460633754730224609375f,-0.3582000286161888258362751003005541861057281494140625f,-0.077999984294176538668352804961614310741424560546875f,-0.18840000092834208178516064435825683176517486572265625f,0.102499987781047963153469027020037174224853515625f,-0.6007999857336283611175531405024230480194091796875f,0.09069999821782115390078615746460855007171630859375f,0.14539997740685972615892751491628587245941162109375f,-0.500399976512790001237362957908771932125091552734375f,0.1019999938383697735133637252147309482097625732421875f,0.3657999838709837803207847173325717449188232421875f,-0.144399980655312898392139686620794236660003662109375f,0.171200004267692662551780813373625278472900390625f,0.695299989593028300305377342738211154937744140625f,-1.32399998097121684992316659190692007541656494140625f,0.428300005030632036095994408242404460906982421875f,0.548400046265125951094887568615376949310302734375f,0.20679998472034899492655313224531710147857666015625f,0.2682000123739243235831963829696178436279296875f,-0.1590000065267087592246753047220408916473388671875f,0.553200010362267402541647243197076022624969482421875f,-0.12080000488162045968465463374741375446319580078125f,-0.302800027143954597619313062750734388828277587890625f,0.05680001422464819871294139375095255672931671142578125f,-0.16680000401735295412208870402537286281585693359375f,-0.629699987012148909570896648801863193511962890625f,0.354799961265922281228313295287080109119415283203125f,-0.316900004497170595385568958590738475322723388671875f,-0.596400031900405469542647551861591637134552001953125f,0.27660002116262882854158533518784679472446441650390625f,-0.32010001232624052391884106327779591083526611328125f,0.574399994066357777455777977593243122100830078125f,-0.3145999897360802766144161068950779736042022705078125f,0.2911000066563487198578741299570538103580474853515625f,0.340300051581859719362910254858434200286865234375f,-0.52420004526078667339561434346251189708709716796875f,0.22240001866817493691996787674725055694580078125f,0.0189999661445618972521742762182839214801788330078125f,-0.5031999801874160738179853069595992565155029296875f,0.027799997192621273001122972345910966396331787109375f,-0.31750001188367615867491622339002788066864013671875f,-0.1511999990522860937147697768523357808589935302734375f,-0.1393000083759428076035646881791763007640838623046875f,-0.3021000478506099540254581370390951633453369140625f,0.5342000211954118782387013197876513004302978515625f,0.762999988391995120906585725606419146060943603515625f,0.169099994787573759680299190222285687923431396484375f,0.092799990487097971936236717738211154937744140625f,-0.29640001908540758446264362646616064012050628662109375f,-0.2499000067844987515996990623534657061100006103515625f,-0.507900018310547096689333557151257991790771484375f,-0.970999986693262595593978403485380113124847412109375f,-0.2784999931752680790708609492867253720760345458984375f,-0.7958000074148170455146100721322000026702880859375f,-0.222399989461898872633582868729718029499053955078125f,0.066400000011920923270736238919198513031005859375f,0.29920000510215771782895899377763271331787109375f,-0.840200043353438896787110934383235871791839599609375f,-0.45730001207888137937374040120630525052547454833984375f,-1.36000004619360037594333334709517657756805419921875f,-0.1554000201731919883485488753649406135082244873046875f,-0.1353000020354986343562586625921539962291717529296875f,-0.0360000046491624203781611868180334568023681640625f,0.4805999880149958158881418057717382907867431640625f,0.32999999610707153596678153917309828102588653564453125f,0.515499999605118386369895233656279742717742919921875f,0.4050000011175869385482428697287105023860931396484375f,0.14999999795109031364148677312186919152736663818359375f,0.52450000572204569238010662957094609737396240234375f,0.6628000326573852962752653183997608721256256103515625f,0.17300001081824312176848934541339986026287078857421875f,0.24880003298521113919150593574158847332000732421875f,-0.067800027963519138296533128595910966396331787109375f,-0.1584000042065978330896314218989573419094085693359375f,-0.2927000166058544294855892076157033443450927734375f,0.96840003091692938852474981104023754596710205078125f,0.342600007110834159362866557785309851169586181640625f,1.110400046920776873093927861191332340240478515625f,-0.18200001749396310657402864308096468448638916015625f,-0.262300000865757443335724019561894237995147705078125f,-0.695500018417835264017412555404007434844970703125f,-0.594600037941336978519757394678890705108642578125f,-0.207600012959539947932086079163127578794956207275390625f,-0.068900049334765345321329732541926205158233642578125f,0.04479998935461038112038067993125878274440765380859375f,-0.1123000030264258430889867668156512081623077392578125f,0.0542999929904937062019598670303821563720703125f,-0.0096000102996829372870024599251337349414825439453125f,-0.092900011754036171396364807151257991790771484375f,1.2239999932646750124831669381819665431976318359375f,-1.008199967297911836539014984737150371074676513671875f,0.661900005912780642347570392303168773651123046875f,-0.056100033664704174185544616193510591983795166015625f,-0.15499999403953512722154073344427160918712615966796875f,-0.03300000470876707225897916941903531551361083984375f,-1.0439999753832811535403379821218550205230712890625f,1.349999962821602839113666050252504646778106689453125f,-0.601399996733665176407157559879124164581298828125f,-0.6415999781727794726293723215349018573760986328125f,0.622399971878528912583306009764783084392547607421875f,-0.34320000149607654549299695645458996295928955078125f,-0.3268000543832787485598601051606237888336181640625f,0.584200022462010526425046919030137360095977783203125f,-0.2199000189661983739597417297773063182830810546875f,-0.19260004481077164228963738423772156238555908203125f,-0.2131999674469235994678228962584398686885833740234375f,-0.088800015699863621421172865666449069976806640625f,0.34490000242590923562602256424725055694580078125f,0.08520003225207262165241672846605069935321807861328125f,0.18169998532086573117538819133187644183635711669921875f,0.28249999999999975131004248396493494510650634765625f,-0.5149999944120644546075027392362244427204132080078125f,0.1760000016689300395000827847979962825775146484375f,0.93120003436803866492255110642872750759124755859375f,-0.8484000186979769875250667610089294612407684326171875f,0.5110000264048579765585600398480892181396484375f,0.1582999510884288696388466632924973964691162109375f,-0.376799954470992570776388674858026206493377685546875f,0.089299985837936191046537715010344982147216796875f,0.785600022101402117868929053656756877899169921875f,-1.22840001631379092117413165397010743618011474609375f,0.47300001811981218224900658242404460906982421875f,0.288499966502190119399529066868126392364501953125f,-0.36639997155070336276594389346428215503692626953125f,0.145499999046325712015459430404007434844970703125f,-0.92290003991723068565988796763122081756591796875f,0.463000033691525203494165907613933086395263671875f,-0.48640002310872088031601379043422639369964599609375f,0.090699971842765680918319048942066729068756103515625f,-0.60079998416900626256875739272800274193286895751953125f,0.07249999880790714001932428800500929355621337890625f,-0.06990002462864008947462934884242713451385498046875f,-0.86329999072551633076955113210715353488922119140625f,-0.3847999944210049871884393724030815064907073974609375f,-0.17019999641478056329191304030246101319789886474609375f,-0.09099999481439535742310908972285687923431396484375f,-0.15960002013444896107330350787378847599029541015625f,-0.2004000037819146928796953943674452602863311767578125f,-0.6150000333786014294901178800500929355621337890625f,0.490200018170475981804656839813105762004852294921875f,0.1407000024646520852655839917133562266826629638671875f,0.7536000241279605660338347661308944225311279296875f,-0.099399983784556500410189983085729181766510009765625f,0.03220000284463164774706456228159368038177490234375f,-0.503499989449977647382183931767940521240234375f,0.4796000155150892396704875864088535308837890625f,0.1912000059813261376717719031148590147495269775390625f,0.0926000176906587313396812533028423786163330078125f,-0.81020002666413837300041222988511435687541961669921875f,-0.166300012198090652049131676903925836086273193359375f,-0.3828000420451171947888724389486014842987060546875f,0.20579999683499339280245976624428294599056243896484375f,0.066899999468028548132991772945388220250606536865234375f,0.596400008207559562123378782416693866252899169921875f,0.16439999667406102190625460934825241565704345703125f,-0.0131999965041875721993847037083469331264495849609375f,0.4134000245094302528059415635652840137481689453125f,0.23400001770257983224610143224708735942840576171875f,0.2457000080153346477374043388408608734607696533203125f,0.5481000301480296599265784607268869876861572265625f,-0.53559999140501002301562039065174758434295654296875f,-0.1501999976813792603280717230518348515033721923828125f,-0.5868000162363049554414828890003263950347900390625f,-0.8729999971836803762670342621277086436748504638671875f,-0.257699999693036030379289513803087174892425537109375f,-0.449600018644332521944306790828704833984375f,-0.63900000585615668313721471349708735942840576171875f,-0.17160000391453511525696740136481821537017822265625f,-0.96550001782178895837205345742404460906982421875f,0.4654000012487171655806150738499127328395843505859375f,0.059599999533593628342487136251293122768402099609375f,0.0143000087857245716804754920303821563720703125f,0.14099999861419210134272361756302416324615478515625f,-0.03539999614506961211901625574682839214801788330078125f,-0.28689998087882973010209752828814089298248291015625f,-0.0764000107407571960038694669492542743682861328125f,-0.2041999899029729448329817387275397777557373046875f,-0.54820000730752926898503574193455278873443603515625f,0.1707999871492387189420014692586846649646759033203125f,-0.285599999749660327097444678656756877899169921875f,0.243599979424477197653686744160950183868408203125f,-0.19339996877908749439711755258031189441680908203125f,0.11729999978542338112674769945442676544189453125f,0.2748000084280963761784732923842966556549072265625f,-0.855000016316771027646836955682374536991119384765625f,0.1917000125527383858070606947876513004302978515625f,0.1689000351786607012627428048290312290191650390625f,0.603799961534142948238468306954018771648406982421875f,0.048200009989738390459024230949580669403076171875f,0.7074000252246861464300309307873249053955078125f,-0.29120000143647217782927327789366245269775390625f,0.37100001120567327461685636080801486968994140625f,-1.101800010573864785357045548153109848499298095703125f,1.267199997964501445579799110419116914272308349609375f,-0.61780002058744454274119561887346208095550537109375f,-0.08880001789778457776947107049636542797088623046875f,-0.46419998502284298691478170439950190484523773193359375f,-0.01170000594034792396058719532447867095470428466796875f,0.003599962735176376327217440120875835418701171875f,-0.62179997957944888042902675806544721126556396484375f,0.037499990463256693828952847979962825775146484375f,-0.082499977946280722562732989899814128875732421875f,0.25459999510049780013787312782369554042816162109375f,-0.040200002598762551997424452565610408782958984375f,-0.471399971699714370743095059879124164581298828125f,0.83219997464418415944464868516661226749420166015625f,-0.286099991941451747834435082040727138519287109375f,-1.12599998885393137726396162179298698902130126953125f,1.0513999714761974590970794452005065977573394775390625f,-0.6223000009775159213631923194043338298797607421875f,-0.375499987423420833465570467524230480194091796875f,1.15619997260272544536974237416870892047882080078125f,-0.2600000032782556758093051030300557613372802734375f,0.79609999462366065614560284302569925785064697265625f,-0.7783999947816131470545997217413969337940216064453125f,0.4347000083208083509589414461515843868255615234375f,0.47349998050928121529068448580801486968994140625f,-0.31819998979866515043113395222462713718414306640625f,0.24579999772906280242068532970733940601348876953125f,0.337899994766711841975848074071109294891357421875f,0.29460001767575771935980810667388141155242919921875f,-0.185800011363625561244816708494909107685089111328125f,-0.0935000042021274513359685442992486059665679931640625f,0.3933999881505965134920188575051724910736083984375f,-0.23040000549554828790377314362558536231517791748046875f,-0.050700001756846913014697975086164660751819610595703125f,-0.100100009196996797555812008795328438282012939453125f,-1.063599978083372032955367103568278253078460693359375f,-0.2515999959796666463063274932210333645343780517578125f,-0.614600007021426364417493459768593311309814453125f,0.06139999561607822897713049314916133880615234375f,-0.0338000047847628604102965255151502788066864013671875f,0.4500999899744986265659463242627680301666259765625f,0.426400010442733901072642765939235687255859375f,0.26570000540763138996425141158397309482097625732421875f,0.86310003998279594128462122171185910701751708984375f,-0.066399999862909453440806828439235687255859375f,-0.01970000180155040414575751128722913563251495361328125f,-0.48550001454353353125270587042905390262603759765625f,0.878800003927945994064430124126374721527099609375f,0.22670000141412016692044062438071705400943756103515625f,0.44030002446174609787021836382336914539337158203125f,-0.00599997648596772847895408631302416324615478515625f,0.086400001874566090265261664171703159809112548828125f,-0.429400004124641299085851642303168773651123046875f,-0.5566000146061182984880133517435751855373382568359375f,-0.16250000461935998341544973300187848508358001708984375f,-0.58980003226995503240459584048949182033538818359375f,0.176999994620680833889991845353506505489349365234375f,0.1173000006049871313251742321881465613842010498046875f,-0.2916000064849857409399191965349018573760986328125f,0.86220000750124459187873071641661226749420166015625f,0.33990000086128713263633471797220408916473388671875f,1.22200001943111402624708716757595539093017578125f,-0.60780000471770723180497952853329479694366455078125f,-0.3530999969929455861716860454180277884006500244140625f,-0.5212000120162958438640998792834579944610595703125f,0.190599996545910810308299687676480971276760101318359375f,-0.0289000006079673799064266859204508364200592041015625f,0.01300000157952307944242420489899814128875732421875f,-0.57279998981654600687107858902891166508197784423828125f,-0.217399994379281924494762279209680855274200439453125f,-0.41820000175684624021954505224130116403102874755859375f,0.8864000061213967995854545733891427516937255859375f,0.3299000118881465493103632979909889400005340576171875f,-0.5732000251889228792379071819595992565155029296875f,-0.181599987486004810222084415727294981479644775390625f,-0.27240000853538504088646732270717620849609375f,-0.318100002580881113800614912179298698902130126953125f,0.34160000082254404762949206997291184961795806884765625f,-0.17710000523328783827992083388380706310272216796875f,-1.351399957394600281901375637971796095371246337890625f,1.57799993154406603679262843797914683818817138671875f,-0.76199999171495402805476260255090892314910888671875f,-0.197900012052059537381865084171295166015625f,-0.50059999788701503575794049538671970367431640625f,-0.068399998283386054254151531495153903961181640625f,0.0270999962925912996070110239088535308837890625f,-0.2395999948024751802222453989088535308837890625f,0.0430999908089635397345773526467382907867431640625f,0.2379000192046163419945514760911464691162109375f,0.30259999071955689942114986479282379150390625f,0.0962000069499016063900853623636066913604736328125f,1.1566999922871588069028803147375583648681640625f,-1.190799981933832185632127220742404460906982421875f,0.6435000067353247032997387577779591083526611328125f,0.259000062704085820541877183131873607635498046875f,0.59119995256066371069891829392872750759124755859375f,0.0958000220179560102451432612724602222442626953125f,-0.50139999971389759281237274990417063236236572265625f,0.8278000035256145583417719535646028816699981689453125f,-0.301100007140636449776138761080801486968994140625f,0.4740000111460691556430901982821524143218994140625f,0.251400019308924083105694080586545169353485107421875f,0.2216999973535536128110834397375583648681640625f,0.516800002229214072713148198090493679046630859375f,-1.06139997937381291848168984870426356792449951171875f,0.33290000281333931297922390513122081756591796875f,-0.558200023400782896487726247869431972503662109375f,0.09960000292360771112498696311376988887786865234375f,-0.29719999997615786924143321812152862548828125f,0.3443999857217072690929171585594303905963897705078125f,-0.5479999899566172505416261628852225840091705322265625f,0.1972000026211142564847023095353506505489349365234375f,-0.62919999128133008436947193331434391438961029052734375f,0.5461999789357185886018442033673636615276336669921875f,-0.349699995096027560492757402244023978710174560546875f,0.7366000093162057282825116999447345733642578125f,0.0251000150084497164471031283028423786163330078125f,-0.540799980294704329963906275224871933460235595703125f,-0.0030999961361288796979351900517940521240234375f,0.30910001751184523755000554956495761871337890625f,0.5953999949902293309378364938311278820037841796875f,0.37479999654442064382919852505438029766082763671875f,1.19970001875162068927238578908145427703857421875f,0.41700001488626015078153841386665590107440948486328125f,0.1818000040359795399513842539818142540752887725830078125f,0.5363000221259894839676007904927246272563934326171875f,0.072399997657537351614109866204671561717987060546875f,0.1190000004172324654660997111932374536991119384765625f,-0.1987999945402147972117745666764676570892333984375f,0.8112000035226343275240878938348032534122467041015625f,0.229800000642239987680426338556571863591670989990234375f,0.08260001616328926932197873611585237085819244384765625f,-0.148600004830956422541277106574852950870990753173828125f,-0.09200000039488076541971395272412337362766265869140625f,-0.1931000058591365586835308931767940521240234375f,-0.6678000142171978126270914799533784389495849609375f,-0.32010000176504249669307000658591277897357940673828125f,-0.586600020454823845028613504837267100811004638671875f,0.639000005409120941379796931869350373744964599609375f,0.359399994029104608017632926930673420429229736328125f,0.730100008451938009557125042192637920379638671875f,-0.67880002285242113657659501768648624420166015625f,-0.171700007002055732296952328397310338914394378662109375f,-0.100500011414289769362540027941577136516571044921875f,-0.268400002902746070976292003251728601753711700439453125f,-0.11229999945014712547930457731126807630062103271484375f,-0.1515000065267084750075810006819665431976318359375f,-0.24480000574588778317064452494378201663494110107421875f,-0.077100001899153003392939353943802416324615478515625f,-0.084100010005384751021750844302005134522914886474609375f,-0.08700002382695704028492400539107620716094970703125f,-0.0789000096976757436806337864254601299762725830078125f,0.2721999701738351262747528380714356899261474609375f,-0.59340000998079800442752684830338694155216217041015625f,-0.13050000530481342675415135090588591992855072021484375f,-0.31460002102851891692125718691386282444000244140625f,0.1457999966114760237445580060011707246303558349609375f,0.1905000001639127749086810581502504646778106689453125f,0.4515999999284741051042146864347159862518310546875f,-0.264999995306134383810103827272541821002960205078125f,0.74550002646446245080369408242404460906982421875f,-1.1876999751806263105891048326157033443450927734375f,1.023599952304363736033110399148426949977874755859375f,-0.6353000033020972381336832768283784389495849609375f,0.04009999176263789877339149825274944305419921875f,-0.3157999779850246824253190425224602222442626953125f,0.060399983739852558528582449071109294891357421875f,0.281500013623386724503916411777026951313018798828125f,-0.41260000644028183014455635202466510236263275146484375f,0.17140000654980547256656109311734326183795928955078125f,0.2196000085115432653992684208787977695465087890625f,0.442800004047155137953950543305836617946624755859375f,0.0873999933362004099990372196771204471588134765625f,1.29080000603646016799075368908233940601348876953125f,-0.8521999870121479414564191756653599441051483154296875f,0.6887000090733168544687714529572986066341400146484375f,-0.0869000091910361316394073583069257438182830810546875f,0.032800004568695972917424796833074651658535003662109375f,-0.049500001698732365440491776098497211933135986328125f,-0.63320002593398072576746926642954349517822265625f,0.40360000520348549013505135008017532527446746826171875f,-0.34940000758916134326881319793756119906902313232421875f,0.4615000248551357486803681240417063236236572265625f,-0.244800011482834367626537641626782715320587158203125f,0.259300002527236728155912715010344982147216796875f,-1.06670003624558518140474916435778141021728515625f,0.8158000041365625065736821852624416351318359375f,-0.57460002624392547687648402643389999866485595703125f,-0.3221000067234036645658079578424803912639617919921875f,0.189800001573562526768768066176562570035457611083984375f,-0.173800002437829892443232893128879368305206298828125f,-0.343700005627423532672537476173602044582366943359375f,0.25360000088214873503744684057892300188541412353515625f,-0.18590000444874166252162694945582188665866851806640625f,-0.316000029981136520973450387828052043914794921875f,-0.27719997307956223409064477891661226749420166015625f,-0.141400010442733758964095613919198513031005859375f,-0.778599999690056332468657274148426949977874755859375f,0.90819998756349107349450378023902885615825653076171875f,-0.43810000958442696372685531969182193279266357421875f,-0.14679999388456277387149384594522416591644287109375f,0.2626000191062682898035518519463948905467987060546875f,-0.07510001024007806336157955229282379150390625f,0.088000043004750949648951063863933086395263671875f,-0.3037000284075741518563518184237182140350341796875f,0.12920001374483103262491567875258624553680419921875f,0.15639999949038030369052876267232932150363922119140625f,0.51260000010728834496376293827779591083526611328125f,-0.756999999240040910564175646868534386157989501953125f,-0.098600002335011982523838014458306133747100830078125f,-0.165100010240077921253032400272786617279052734375f,-0.073400001338124187810763032757677137851715087890625f,0.138799998451769379670395210268907248973846435546875f,0.450099984610080383617969346232712268829345703125f,-0.8119999872446059807629126225947402417659759521484375f,-0.16149999874085185780359097407199442386627197265625f,-0.266500003397464269028205308131873607635498046875f,-0.433000023260712740835742806666530668735504150390625f,-0.172100007691979417501215721131302416324615478515625f,0.069999967217444947209514793939888477325439453125f,-0.5907999865531916583449856261722743511199951171875f,-0.19309999278336753558704685929114930331707000732421875f,-0.43829999907016681248705936013720929622650146484375f,-0.886600040087104357411362798302434384822845458984375f,-0.25910001438260088502119060649420134723186492919921875f,-0.29100004845857707636014310992322862148284912109375f,-0.384800027650594955730412038974463939666748046875f,-0.20230000853985552478064846582128666341304779052734375f,-0.20450003075599720858690488967113196849822998046875f,0.8528000141799452205049192343722097575664520263671875f,0.1906000060826540387637351159355603158473968505859375f,0.8210000380277637788140054908581078052520751953125f,0.20359999879598600358576732105575501918792724609375f,-0.0197000042602419878079444970353506505489349365234375f,0.3766999928832053257110601407475769519805908203125f,-0.913399984499811967708637894247658550739288330078125f,-0.2409999944046139486175661659217439591884613037109375f,-0.9720999956965439281475482857786118984222412109375f,-0.138599997976422084633441045298241078853607177734375f,0.143400003387033958546226131147705018520355224609375f,0.331700007784366590612989966757595539093017578125f,0.3631999983668325882035787799395620822906494140625f,0.1218999957427382430097395626944489777088165283203125f,0.8408999973893163115690185804851353168487548828125f,-0.493599996262788831558054880588315427303314208984375f,-0.0049999996274709967991611847537569701671600341796875f,0.1026000081539155672771812533028423786163330078125f,-0.7154000140637162274970251019112765789031982421875f,-0.3141000261545183747102782945148646831512451171875f,-0.2051999680280687243794091045856475830078125f,0.12459997870922112639391343691386282444000244140625f,-0.09729999218583085252021191990934312343597412109375f,-1.1924999853968625274092119070701301097869873046875f,1.329199968370795570393738671555183827877044677734375f,-0.661300010037422225650516338646411895751953125f,-0.5150999807357781179462108411826193332672119140625f,0.526600008198618407817548359162174165248870849609375f,-0.27200000616908059924980989308096468448638916015625f,-1.19049997520446826371198767446912825107574462890625f,1.2339999506473546819762532322783954441547393798828125f,-0.6584999998807905541298168827779591083526611328125f,-0.7490000290274618777175419381819665431976318359375f,0.189200022312998594742339264485053718090057373046875f,-0.38400001382827753104720613919198513031005859375f,-0.65769998441934518496054806746542453765869140625f,0.6985999764442443193956933100707828998565673828125f,-0.3709999948143956061130666057579219341278076171875f,-1.2634000406026846263785046176053583621978759765625f,0.997000020101666439842347244848497211933135986328125f,-0.6857000293612482977323452359996736049652099609375f,-0.4527000374674798877094872295856475830078125f,0.091000025063753042786629521287977695465087890625f,-0.2366000145316125014005592674948275089263916015625f,0.85319998793602014330872407299466431140899658203125f,-1.4745999872028827670789041803800500929355621337890625f,0.5111000098228455357229904620908200740814208984375f,0.10289999178647946109776967205107212066650390625f,-0.71740000078678090034145498066209256649017333984375f,0.09080000726580639280882678576745092868804931640625f,-0.8310999725699428353209441411308944225311279296875f,1.469199946466088757546231136075221002101898193359375f,-0.5030999970674514543134137056767940521240234375f,-0.5488999895811073059803675278089940547943115234375f,0.748000006720423105122108609066344797611236572265625f,-0.30110001354813586971204131259582936763763427734375f,-0.0067000064432622963295216322876513004302978515625f,-0.69839999068379388091898363200016319751739501953125f,0.045100001075863982435976140550337731838226318359375f,-0.9543999927997592891415479243732988834381103515625f,1.013799975913763251611499072168953716754913330078125f,-0.521900012469291851857633446343243122100830078125f,-0.46159999873638213330195867456495761871337890625f,-0.29360001041889205453117028810083866119384765625f,0.43239999303817722164922088268212974071502685546875f,0.121099996151030009361448946947348304092884063720703125f,0.301500002056359850399758215644396841526031494140625f,0.505400009444355813315041814348660409450531005859375f,0.31359999980181452183813917145016603171825408935546875f,1.06710002460479724817332680686376988887786865234375f,-0.83180002689063570198158004131983034312725067138671875f,-0.38140000736936929826015330036170780658721923828125f,-1.307700047954172273279027649550698697566986083984375f,0.265800032225251836592860854580067098140716552734375f,0.2388000133529306623358934302814304828643798828125f,0.15850002700090470852956059388816356658935546875f,0.8058000167280441328188089755713008344173431396484375f,0.1455000103712082992757359534152783453464508056640625f,0.3966000291347508976969038485549390316009521484375f,-0.524600019761920055572090859641321003437042236328125f,-0.32110000199973587342583414283581078052520751953125f,-0.881200009036064102474483661353588104248046875f,0.0278000030785801044430627371184527873992919921875f,0.21220000060945753972418970079161226749420166015625f,0.52809998842477767766467877663671970367431640625f,0.930599994944035557864481233991682529449462890625f,0.28619999430254094985315305166295729577541351318359375f,0.655500011153518524764649555436335504055023193359375f,-0.05399999967217450347334306570701301097869873046875f,-0.05159999955594540832493066773167811334133148193359375f,-0.62900000637769704070478837820701301097869873046875f,-0.633200019824504689580635385937057435512542724609375f,-0.31420000085532662215115351500571705400943756103515625f,-0.5128000195443629394276285893283784389495849609375f,0.474400035864114943962022152845747768878936767578125f,0.3530000067204237534923549901577644050121307373046875f,1.0262000322818760622567424434237182140350341796875f,0.749200028795004246973121553310193121433258056640625f,0.3668000107973814483131036467966623604297637939453125f,0.9918000537872320165888595511205494403839111328125f,-0.048600001701712525203902259818278253078460693359375f,0.07919999977499248000611942188697867095470428466796875f,0.39209999778270709924754555686376988887786865234375f,0.6615999942660326649246371744084171950817108154296875f,0.267799993839859862276853164075873792171478271484375f,0.37420000152587817865423858165740966796875f,-0.488999992296099694755184827954508364200592041015625f,0.13879999528527253715992628713138401508331298828125f,0.504099989050626362541152047924697399139404296875f,-0.58739998469352716625735411071218550205230712890625f,0.28579999862313254510581828071735799312591552734375f,0.0011000086307522138895365060307085514068603515625f,-0.311400003513693679479956699651665985584259033203125f,0.0375999959349631041050088242627680301666259765625f,-0.382700017630309030636226452770642936229705810546875f,0.86120000821650022349018627210170961916446685791015625f,-0.25870000734105713657839942243299447000026702880859375f,0.2567000536799444176949691609479486942291259765625f,0.347999983027576842431471959571354091167449951171875f,0.11810001375675227563988300971686840057373046875f,1.09320000015497331702363226213492453098297119140625f,-1.3523999774664645645572136345435865223407745361328125f,0.6174000162839892169586164527572691440582275390625f,-0.1654000412583354062689977581612765789031982421875f,0.230000011101365142707209088257513940334320068359375f,-0.11280001134872463808278553187847137451171875f,-0.42229997087716952819391735829412937164306640625f,0.5762000100046389405861191335134208202362060546875f,-0.225600004816055133005647803656756877899169921875f,1.0688999912202350461853939123102463781833648681640625f,-1.159199981111287858226432945230044424533843994140625f,0.60270000256150935147303471239865757524967193603515625f,0.3770000112354754406851498060859739780426025390625f,0.2111999941349029796811009873636066913604736328125f,0.167000006094574882098413581843487918376922607421875f,-0.6268000382900231759464304559514857828617095947265625f,0.3160000229775901114948055692366324365139007568359375f,-0.340800013524293898825590076739899814128875732421875f,-0.0423999465465545455344908987171947956085205078125f,-0.056200037497281751797117976821027696132659912109375f,0.003900014615059177458533667959272861480712890625f,0.4678000208854680153081062599085271358489990234375f,-0.585200007516145870312129773083142936229705810546875f,0.28280000598430632408053497783839702606201171875f,-0.4028999980449672335680588730610907077789306640625f,0.248400007894634899940911054727621376514434814453125f,-0.2049000076413154403098815237171947956085205078125f,0.79740000495910567934743085061199963092803955078125f,-0.5063999886870382827197545339004136621952056884765625f,0.4318000001430508660860141390003263950347900390625f,-1.1129999868273738883317491854541003704071044921875f,1.200800028872489377818055800162255764007568359375f,0.6776000261843206917689030888141132891178131103515625f,0.296200006651878400276700631366111338138580322265625f,0.64500002995133431937802015454508364200592041015625f,-0.040000004470348182739058984225266613066196441650390625f,-0.10789999852925535872572027074056677520275115966796875f,-0.1793000023782251872717097285203635692596435546875f,-0.58699999059736729378755626385100185871124267578125f,-0.0286000009194016246993896857020445168018341064453125f,-0.0489000149130820460641189129091799259185791015625f,-0.6671999840319156049872617586515843868255615234375f,-0.1460999968960881023605224982020445168018341064453125f,-0.5803000003218645730385105707682669162750244140625f,-0.5854000030368566864780177638749592006206512451171875f,-0.12710000091195106097785583187942393124103546142578125f,-0.3316000134885310757226761779747903347015380859375f,0.16900002612173548044438575743697583675384521484375f,0.1955000042617320854532181328977458178997039794921875f,0.0282000231027605963163296109996736049652099609375f,-0.0471999996781349018259987815326894633471965789794921875f,-0.0049999996274709690435855691248434595763683319091796875f,-0.0606000001966953139032057151780463755130767822265625f,-0.144000008314848049195688872714526951313018798828125f,0.0242999946668744082334256972899311222136020660400390625f,0.4940999810785051682415769391809590160846710205078125f,0.384199992510676491264121068525128066539764404296875f,0.08590000096559524678241359652020037174224853515625f,0.5018000132560729298347723670303821563720703125f,-0.3189999829083678406504986924119293689727783203125f,-0.1714999960213898777539043294382281601428985595703125f,0.0012000036716468542863367474637925624847412109375f,0.52139999785125235032268165014102123677730560302734375f,0.1313999999932944084957142649727757088840007781982421875f,-0.106299985171109667714262059234897606074810028076171875f,0.79680002279281680888090022563119418919086456298828125f,0.2199000125214458645217519006109796464443206787109375f,0.670900041198731145186684443615376949310302734375f,-0.149600002253055475165410825866274535655975341796875f,-0.0259000021204351948966859708889387547969818115234375f,0.4082999850630759652858614572323858737945556640625f,-0.29919999787509421462772252198192290961742401123046875f,-0.163399998171627480747503113889251835644245147705078125f,-0.2490000031739469932290376164019107818603515625f,-0.781600026899576416639092713012360036373138427734375f,0.147399982905387361142857116647064685821533203125f,0.6194000342249872392841325563495047390460968017578125f,-0.55520001451969136496700230054557323455810546875f,0.350200015562772914989864148083142936229705810546875f,0.1044999839663509622056380976573564112186431884765625f,-0.280199986654520227791209663337212987244129180908203125f,0.060600000345707005777740050689317286014556884765625f,-1.002499963939190052997219027020037174224853515625f,1.19419996449649357117550607654266059398651123046875f,-0.5571999996781347164187536691315472126007080078125f,-0.71329996747970625392554211430251598358154296875f,1.1357999500453477281780578778125345706939697265625f,-0.42079999370574938666322850622236728668212890625f,-0.7535999940276145725448486700770445168018341064453125f,0.913399987182021089182626383262686431407928466796875f,-0.426100006693601596197140679578296840190887451171875f,0.196000048458575548693261225707828998565673828125f,0.44199995513260414981004942092113196849822998046875f,0.0806000125646590959149762056767940521240234375f,-0.03879999807924028321082232650951482355594635009765625f,0.1125999987289309223204014642760739661753177642822265625f,-0.02570000050887466525040281339897774159908294677734375f,-0.6225000024586915259305897052399814128875732421875f,0.175400007432699123199881796608678996562957763671875f,-0.310200007441639780125797187793068587779998779296875f,0.2897999708771710203336624545045197010040283203125f,-0.729399967542291083333338974625803530216217041015625f,0.18829999191761004340150975622236728668212890625f,-0.49599998885393148384537198580801486968994140625f,-0.0250000325590367111772138741798698902130126953125f,-0.2488999866008754224822041578590869903564453125f,0.951099993934481968693717135465703904628753662109375f,-0.53799998809397220345118739714962430298328399658203125f,0.49990000169947712560514219148899428546428680419921875f,0.847700005578996051980311676743440330028533935546875f,-1.153199984061718641470406510052271187305450439453125f,0.49130001271963141817877840367145836353302001953125f,-0.5489000003099437918763214838691055774688720703125f,0.017600004279613390423264718265272676944732666015625f,-0.268300001454353065355462604202330112457275390625f,-0.28480000455379472779071647892124019563198089599609375f,0.08939999190568936882073103333823382854461669921875f,-0.1528999970763920135397029298474080860614776611328125f,-0.6544000330328945125302197993732988834381103515625f,-0.194000004887580956847159541212022304534912109375f,0.1001999964147806121417261238093487918376922607421875f,0.07949999774992466683443126385100185871124267578125f,0.68139999852180466177742346189916133880615234375f,0.16899998864531540210265347923268564045429229736328125f,-0.006999997824430448378763003347557969391345977783203125f,0.225900009459257145039146053022705018520355224609375f,-0.3384000049889082095688763729413039982318878173828125f,-0.2327999939143656771278756423271261155605316162109375f,-0.0951999936580651962003685184754431247711181640625f,0.3896000460624697847578090659226290881633758544921875f,0.3187000095948577804705337257473729550838470458984375f,0.7557000329375274105814241920597851276397705078125f,1.084800019007921445535203019971959292888641357421875f,0.363300008346140401727097923867404460906982421875f,1.20390005842447322947919019497931003570556640625f,0.1816000094652177043741403394960798323154449462890625f,0.1190000017583370539586695713296649046242237091064453125f,0.4758000064462424294475795250036753714084625244140625f,0.050199992093443668306207428031484596431255340576171875f,-0.08920000372380021413931672213948331773281097412109375f,-0.1049000060021878510951864882372319698333740234375f,-0.88859999037682946276817119724000804126262664794921875f,-0.2120999922692774486421285473625175654888153076171875f,-0.46020000885724954287070431746542453765869140625f,1.176200043532252603739607366151176393032073974609375f,0.340600011782348222055816222564317286014556884765625f,1.227500068843365710336001939140260219573974609375f,0.35799999785423264331285508887958712875843048095703125f,0.1948999971546232501395934377796947956085205078125f,0.860900007168203362795111388550139963626861572265625f,-0.503999990284442755950067294179461896419525146484375f,-0.1022999967679381061458343538106419146060943603515625f,-0.5749000052571293650771622196771204471588134765625f,-0.411199993091821536239649503841064870357513427734375f,-0.110199995967745767444512239308096468448638916015625f,-0.625399996256827961360613699071109294891357421875f,0.1841999855071307923282120100338943302631378173828125f,-0.14969999751746654315098794540972448885440826416015625f,-0.51939999369382849891962905530817806720733642578125f,-0.1731999876379968217321447809808887541294097900390625f,0.04509999984651802407142895390279591083526611328125f,-0.197300001871585806156872422434389591217041015625f,-0.524800040912628329436984131461940705776214599609375f,-0.14040000296235088939056367962621152400970458984375f,-0.3442000090509651499814935959875583648681640625f,-0.2205999885737897070470125981955789029598236083984375f,-0.1489000063821672714681199067854322493076324462890625f,0.143499963857234291708664386533200740814208984375f,-0.51299996242672263324635650860727764666080474853515625f,0.09919998854324207915311717442818917334079742431640625f,-0.43460003652572520849162174272350966930389404296875f,-0.1941999784290795449948063833289779722690582275390625f,-0.2129000075817106818476531771011650562286376953125f,0.01490007231235512819012001273222267627716064453125f,0.1187999456644062146182250216952525079250335693359375f,0.0193000239849094867849998991005122661590576171875f,0.9176999906778338100821201805956661701202392578125f,-1.442999985784292338308887337916530668735504150390625f,0.55080000383853899847963475622236728668212890625f,-0.06859998605549311179174765129573643207550048828125f,-0.1308000044345856149874407492461614310741424560546875f,-0.018399996122717698909809769247658550739288330078125f,0.2042999819636339697837001949665136635303497314453125f,-0.436199995550512975572843288318836130201816558837890625f,0.119200001376867259494929385255090892314910888671875f,-1.166399980938433333932380264741368591785430908203125f,1.283999982610344314792882869369350373744964599609375f,-0.65280000270605054168981951079331338405609130859375f,1.0743000207066533135957797640003263950347900390625f,-1.751800027337669707350187309202738106250762939453125f,0.6434000268936159727672929875552654266357421875f,-0.062500006388872986207161375205032527446746826171875f,-0.38779998810291294031316056134528480470180511474609375f,0.00419999338611953820787903168820776045322418212890625f,-0.4444999729394911724966732435859739780426025390625f,0.959599971854686817351876015891321003437042236328125f,-0.27769999604225148459590855054557323455810546875f,-0.2387999864935874683169458876363933086395263671875f,0.746999979048967421846327852108515799045562744140625f,-0.16609999894499782868706461158581078052520751953125f,0.7677999564379458519169929786585271358489990234375f,-0.7821999625742439565811991997179575264453887939453125f,0.4086999967053530635752167654572986066341400146484375f,-0.18829996420145089786046810331754386425018310546875f,0.6693999653816227191072130153770558536052703857421875f,-0.12779999272227282602898412733338773250579833984375f,-0.6857000707864759903031881549395620822906494140625f,0.1615000296235091781227311003021895885467529296875f,0.309400017312169151040279757580719888210296630859375f,0.135800004173815269670200223117717541754245758056640625f,0.150100017243623984342093535815365612506866455078125f,-0.246599995234608737604276029742322862148284912109375f,-0.07830000041127205268054467524052597582340240478515625f,-0.73140001491308215264552927692420780658721923828125f,-0.74399999773502301092520383463124744594097137451171875f,-0.1391999942243098420391334002488292753696441650390625f,-0.334400005614757134253522963263094425201416015625f,-0.65039999923706037776582888909615576267242431640625f,-0.3173999969869851778270231079659424722194671630859375f,-0.9614000056743616795529305818490684032440185546875f,0.54159999873638131617781255044974386692047119140625f,0.286399999415874406683002462159493006765842437744140625f,0.932400031632184767005355752189643681049346923828125f,-0.42960003190636675274305389393703080713748931884765625f,-0.3036000102698803715384201495908200740814208984375f,-0.5572000324606900534263331792317330837249755859375f,0.167400002947449610957164622959680855274200439453125f,0.1529999991580843765603958672727458178997039794921875f,0.6975000110268592123929920489899814128875732421875f,0.65720002575516733855920392670668661594390869140625f,0.216400008058548032341406042178277857601642608642578125f,0.465400032019615661482703217188827693462371826171875f,-0.73260000748336295828266884200274944305419921875f,-0.1925999986544251196818322569015435874462127685546875f,-0.9429000227808952416808097041212022304534912109375f,0.140400002664327505641495008603669703006744384765625f,-0.0501000027522444657535061196540482342243194580078125f,0.4271000004649163628300811978988349437713623046875f,0.1232000107347968054227749234996736049652099609375f,0.2527000055417418789005523649393580853939056396484375f,0.7321000225186349297246124478988349437713623046875f,0.4771999977409837612185583566315472126007080078125f,0.187299998556077429423538660557824186980724334716796875f,0.158900006717443087467245277366600930690765380859375f,0.580800026935339275269143399782478809356689453125f,0.281100007699430054497469200214254669845104217529296875f,0.742900030082464635228234328678809106349945068359375f,-0.2762000010639429348202611436136066913604736328125f,-0.0787000020325183680114378148573450744152069091796875f,-0.7682000326395037603788296109996736049652099609375f,-0.6492000070393084687481177752488292753696441650390625f,0.34010002543926276530328323133289813995361328125f,0.3861000248730181994005761225707828998565673828125f,-0.179800017294287517444217883166857063770294189453125f,0.20560001152157791892705063219182193279266357421875f,0.1176000278234479168304460472427308559417724609375f,0.572399977764487477571719864499755203723907470703125f,0.0186000101208687595999435870908200740814208984375f,-1.029999991655348612340503677842207252979278564453125f,1.212399995124339380225819695624522864818572998046875f,-0.57820001028776157880884056794457137584686279296875f,-0.334600004935264649930104496888816356658935546875f,0.56199998068809531304168558563105762004852294921875f,-0.21399999713897699393783113919198513031005859375f,0.174399983114003731543562025763094425201416015625f,-0.43959998586177828627796770888380706310272216796875f,0.12739998603463131843227529316209256649017333984375f,-0.235400048112869964000992695218883454799652099609375f,-0.06499997839331606019896980797057040035724639892578125f,-0.12830001248121281065550647326745092868804931640625f,-0.2678999995350839213870131061412394046783447265625f,-0.065200000736117313948625451303087174892425537109375f,-0.1167000059962273184055447927676141262054443359375f,0.7468000253260138610045260065817274153232574462890625f,-0.7594000118732451287684170893044210970401763916015625f,0.419100017809868052864885612507350742816925048828125f,-0.5498999903559678159581380896270275115966796875f,1.26199999834597065273555926978588104248046875f,-0.3519000071048736799639300443232059478759765625f,-0.0329000108599668550368733122013509273529052734375f,-0.732000007957219533949455581023357808589935302734375f,0.02410000752806684687357119400985538959503173828125f,-0.41569998435974042649831972084939479827880859375f,0.4370000028908247458048208500258624553680419921875f,-0.211800010275840833173788269050419330596923828125f,0.6573000029146669476176612079143524169921875f,-0.3351999875485898172655652160756289958953857421875f,0.34909999977946259974714848794974386692047119140625f,0.38270003314614342571076122112572193145751953125f,-0.454600013428926530423268559388816356658935546875f,0.22880001346468947787116121617145836353302001953125f,0.09120003198385262521696859039366245269775390625f,0.6539999911040066393752567819319665431976318359375f,0.001000010013580521217591012828052043914794921875f,-0.65279999525546994476599138579331338405609130859375f,-0.914800020498037103067190400906838476657867431640625f,-0.58860001511275772401177164283581078052520751953125f,-0.13020000233799222666419836968998424708843231201171875f,0.12590000007152557515155422152020037174224853515625f,0.463200023102760649607034793007187545299530029296875f,0.0984000065535307888442417834085063077509403228759765625f,-0.1331999969512220882705832991632632911205291748046875f,0.7472000250101091811671949471929110586643218994140625f,0.32050000757724050881591892903088591992855072021484375f,0.86390003905296364195010028197430074214935302734375f,0.791999982327222351585760407033376395702362060546875f,0.2495999909281729767229762728675268590450286865234375f,0.9931999980688086537838898948393762111663818359375f,0.256200020733475586354188635596074163913726806640625f,0.303600001515448081335080132703296840190887451171875f,0.879300021898746564374960144050419330596923828125f,-0.83120000233054136540289391632541082799434661865234375f,-0.328299998045712637662063571042381227016448974609375f,-0.8009000163324173637846570272813551127910614013671875f,-0.0348000023186205709180285339243710041046142578125f,-0.213299999181926214220084148109890520572662353515625f,-0.656500011146068374046080862171947956085205078125f,-0.093600005203485547866648630588315427303314208984375f,-0.121500003062188610680749434322933666408061981201171875f,-0.546900027221441487057518315850757062435150146484375f,-0.4938000425964599315875602769665420055389404296875f,-0.3327000139236451214941325815743766725063323974609375f,-0.8198000498533257740518820355646312236785888671875f,0.706399998372793103129652081406675279140472412109375f,0.09250000122934576296529485261999070644378662109375f,0.048700020658969833675655536353588104248046875f,0.5596000081390144487158977426588535308837890625f,0.137600004763901273552306747660622932016849517822265625f,0.383800019350648113913848646916449069976806640625f,-0.8142000349789861868288198820664547383785247802734375f,-0.300300010566413477786085195475607179105281829833984375f,-0.71360003928989235166824300904409028589725494384765625f,0.15340001296103000782977687777020037174224853515625f,0.07700000352412461612772176522412337362766265869140625f,-0.09149998283386207020839719916693866252899169921875f,0.6290000045150516694292264219257049262523651123046875f,0.391299998936057047416170462383888661861419677734375f,0.92200001525878860775264911353588104248046875f,0.334200007709860802407320079510100185871124267578125f,-0.20519999172091463179867787403054535388946533203125f,-1.085900033450126755951714585535228252410888671875f,0.68120000702440730577791327959857881069183349609375f,-0.5735000246167185622425677138380706310272216796875f,0.8808000422090298542343589360825717449188232421875f,-0.536400012230873191043656333931721746921539306640625f,0.463800027993321872799015181954018771648406982421875f,0.57630002285242110104945822968147695064544677734375f,-0.7136000074386597002984444770845584571361541748046875f,0.34010001083612451111548580229282379150390625f,0.5949999713897700104325849679298698902130126953125f,-1.176199960905313535164395943866111338138580322265625f,0.372099991822242515127072692848742008209228515625f,-0.3056999632000927391572986380197107791900634765625f,0.287199970248341873713116001454181969165802001953125f,-0.146699997949600202673536841757595539093017578125f,-0.768300003335624648315160811762325465679168701171875f,0.83419998498559000399410479076323099434375762939453125f,-0.44060000151917322330774595684488303959369659423828125f,0.50229999024868021706424769945442676544189453125f,-0.5019999963343142468374935560859739780426025390625f,0.25990001270771045938090537674725055694580078125f,0.2781000085413456446303825941868126392364501953125f,-0.038199998515844324487034100457094609737396240234375f,0.1296000102996828218238078989088535308837890625f,-0.14920005069971153943697572685778141021728515625f,0.0828000327318907380913515225984156131744384765625f,-0.097000015258789318295384873636066913604736328125f,1.1880999791860578085334054776467382907867431640625f,-1.186199979382753344481216117856092751026153564453125f,0.6504000057935712675316608510911464691162109375f,0.66539999804496818835986005069571547210216522216796875f,-0.83799998503923445181129636694095097482204437255859375f,0.3796000069469214199813222876400686800479888916015625f,-0.81280003828257374465238171978853642940521240234375f,0.8722000168889760818302647749078460037708282470703125f,-0.4627000229015949539501662002294324338436126708984375f,0.306100017571449001252403832040727138519287109375f,0.03379998150169871706793855992145836353302001953125f,0.1508000071167945321803927072323858737945556640625f,0.29200001674890518899019298260100185871124267578125f,-0.1825999897509815728113835575641132891178131103515625f,0.1754999948740003645752949523739516735076904296875f,0.7398000190079212501359506859444081783294677734375f,0.341900030946732158554368652403354644775390625f,0.2372000077247620186682297571678645908832550048828125f,0.11440000125616790749294438001015805639326572418212890625f,0.40060000731199985057884305206243880093097686767578125f,0.133000000461935830031734440126456320285797119140625f,0.047899999237060508772145794864627532660961151123046875f,-0.234599998229742201516501154401339590549468994140625f,0.3446000216990707798458970501087605953216552734375f,0.1846000018432736311524422490037977695465087890625f,-0.2618999960780143965166644193232059478759765625f,-0.4432000219106677629810064900084398686885833740234375f,-0.2675000081583858158040811758837662637233734130859375f,-0.2089000184893610168046507169492542743682861328125f,0.043200016546249830895476407022215425968170166015625f,0.046800010722875651225649562547914683818817138671875f,-0.505199976968764730145267094485461711883544921875f,-0.364400023645162551844123299815692007541656494140625f,-0.281800004613399490693836924037896096706390380859375f,-0.745200037765503253694987506605684757232666015625f,-0.569800013867020727076351249706931412220001220703125f,-0.12170000651776791134039967801072634756565093994140625f,0.05919996904134716686485262471251189708709716796875f,0.105000004693865722771306536742486059665679931640625f,0.1995000014007091149181860600947402417659759521484375f,0.7742000271558762136692166677676141262054443359375f,-0.559000008240342349807860955479554831981658935546875f,-0.0806000041082501894607048598118126392364501953125f,0.00829999221563326727846288122236728668212890625f,-0.00240000421404840125205737422220408916473388671875f,-0.0561000012174248663399112047045491635799407958984375f,-0.1611000083327294163382248370908200740814208984375f,0.3067999947786332892718519360641948878765106201171875f,0.01420000227093698441649394226260483264923095703125f,0.050800007414817827111619408242404460906982421875f,0.5610000199228526440720088430680334568023681640625f,0.27270000498294832969037315706373192369937896728515625f,0.64940002588033696184766085934825241565704345703125f,-0.1274000014573335448631041799671947956085205078125f,-0.0997000004231929637388276432830025441944599151611328125f,-0.4400000127404928296215302907512523233890533447265625f,-0.20399999073147778716474931570701301097869873046875f,-0.0090000000447034711470450929482467472553253173828125f,-0.6766000195980073073087623924948275089263916015625f,-0.29740000771582131289250128247658722102642059326171875f,0.15080002261400249352618629927746951580047607421875f,0.08680001023858796571630591643042862415313720703125f,-0.2354000025153160091662130071199499070644378662109375f,0.0627000041261316365392985971993766725063323974609375f,0.3850000075995918535909368074499070644378662109375f,0.026199995848536605791423426126129925251007080078125f,0.18770000274777398630021707504056394100189208984375f,0.72290005615949581851964467205107212066650390625f,0.1529999654442075751603624667041003704071044921875f,0.3520000218153001014798064716160297393798828125f,-0.52530004293918697300114217796362936496734619140625f,-0.0543999939918513586434301032568328082561492919921875f,-0.2671000114917756462773468228988349437713623046875f,0.4188000225543984100795569247566163539886474609375f,0.352799992275237617178618165780790150165557861328125f,0.184800006341934164311169297434389591217041015625f,0.0063999728918080478479168959893286228179931640625f,-0.023199970501661670851945018512196838855743408203125f,-0.01240000317096701110131107270717620849609375f,-1.0082000052213668794820478069595992565155029296875f,0.708800001618266062308748587383888661861419677734375f,-0.5370000113844870526236263685859739780426025390625f,-0.4550000035762788996862582280300557613372802734375f,0.230999996230006221509256647550500929355621337890625f,-0.22190001440644291363923912285827100276947021484375f,-0.9705000070929532540731088374741375446319580078125f,0.884199987742305015814281432540155947208404541015625f,-0.526600015425682244085692218504846096038818359375f,0.1358999951541424167089644470252096652984619140625f,-0.1525999941468239029518372262828052043914794921875f,0.071400002172589349669351577176712453365325927734375f,0.50619997953176554261744968243874609470367431640625f,-0.6397999726653103014228918254957534372806549072265625f,0.28349999883770937714189130929298698902130126953125f,0.41800002586841600304978783242404460906982421875f,-0.4004000092953443612486807978712022304534912109375f,0.2422000092148781646983479731716215610504150390625f,0.1314000050038099942639746586792171001434326171875f,0.07559999773800374356369502493180334568023681640625f,0.05320000596642504575584098347462713718414306640625f,0.12400004571676248588119051419198513031005859375f,0.74499997392296801734801192651502788066864013671875f,0.01570001371502893761089580948464572429656982421875f,-0.273100005877763063466545645496807992458343505859375f,-0.72150001859664936176841365522705018520355224609375f,1.035200007367134134028674452565610408782958984375f,0.35440000442266461622153883581631816923618316650390625f,1.20960005053281793863106940989382565021514892578125f,0.1626000176161530585972059270716272294521331787109375f,0.1782000057801604864238242953433655202388763427734375f,0.4471000147700312510323783499188721179962158203125f,0.514800035250187359991969060502015054225921630859375f,0.19620001410245901940498924886924214661121368408203125f,-0.08879995251893912922014351352117955684661865234375f,1.036600012817979088453057556762360036373138427734375f,0.217700005131959972715094409068115055561065673828125f,0.185800012779235856896775658242404460906982421875f,0.3349999936670062794519253657199442386627197265625f,-0.0194000045344233651434251441969536244869232177734375f,-0.4487000242352490175790080684237182140350341796875f,0.32420002425014959879234766049194149672985076904296875f,0.2416000022009014980284291596035473048686981201171875f,0.47050002050399797326463158242404460906982421875f,-0.6636000089287759085010520720970816910266876220703125f,-0.139500003375113035275489892228506505489349365234375f,-0.58430002368688604974522604607045650482177734375f,-0.4198000119298697097036665581981651484966278076171875f,-0.153800003667175777533060454516089521348476409912109375f,-0.084500015705823994238699015113525092601776123046875f,-0.55780000188648681813674556906335055828094482421875f,-0.187399999855458687658682492838124744594097137451171875f,-0.116700013297796001410233657225035130977630615234375f,-0.0723999953478575430931840628545614890754222869873046875f,0.023499999657273284991987338798935525119304656982421875f,0.00309999982416631070236689993180334568023681640625f,0.487999997705221399968422701931558549404144287109375f,0.072700003753602526757049417938105762004852294921875f,0.456900022304058239797086571343243122100830078125f,-1.074399999505281311229509810800664126873016357421875f,-0.3142999999567865909710917549091391265392303466796875f,-0.8685000270009037848240041057579219341278076171875f,-0.839000021651387317689341216464526951313018798828125f,-0.406600005181133761222866951356991194188594818115234375f,-1.299300041717291076537321714567951858043670654296875f,-0.17539998441040527410450522438623011112213134765625f,-0.0015999972835183395858393851085565984249114990234375f,-0.5328999894022938832449654000811278820037841796875f,0.54540002076923865814706005039624869823455810546875f,0.226400003886222833671126863919198513031005859375f,0.828799976956844375308719463646411895751953125f,-1.374399974322319195607633446343243122100830078125f,0.5031999953866002073255003779195249080657958984375f,-0.10809997113943037305716643459163606166839599609375f,0.1757999897569415548304050389560870826244354248046875f,-0.05199999529123289221388404257595539093017578125f,0.8982000388979916039033923880197107791900634765625f,-0.257800026547908789353868996840901672840118408203125f,0.4617000197052956167453885427676141262054443359375f,1.6254000156283385791766704642213881015777587890625f,-1.450199978682399137852598869358189404010772705078125f,0.885100023651123279933017329312860965728759765625f,0.92950000780820829504591529257595539093017578125f,-0.5971999893218278288031797274015843868255615234375f,0.483900015211105483103892765939235687255859375f,0.136300042521952935459239597548730671405792236328125f,0.07719996249973826163426338098361156880855560302734375f,0.07590001229047782516090592253021895885467529296875f,-0.70689999160766614494377790833823382854461669921875f,1.1577999889969825542124226558371447026729583740234375f,-0.4186000086307526402151779620908200740814208984375f,-0.62270001794695861718764717807061970233917236328125f,0.3140000043660402528900021934532560408115386962890625f,-0.32910000931620597697246921597979962825775146484375f,-0.831700004357099231810934725217521190643310546875f,0.49119999257028101879996029310859739780426025390625f,-0.44270000364184358687680287403054535388946533203125f,-0.14209998727738881729720787916448898613452911376953125f,0.25719998634159579875557710693101398646831512451171875f,-0.082499998174607735990093715372495353221893310546875f,0.5134999697804456531002870178781449794769287109375f,-0.991399969166517625041024075471796095371246337890625f,0.31119999666810027605379218584857881069183349609375f,-1.1610999824047087969347558100707828998565673828125f,1.461799967807531519525809926562942564487457275390625f,-0.666800001335143832648100215010344982147216796875f,-0.3905000167787076037484439439140260219573974609375f,0.766200002703070648379934937111102044582366943359375f,-0.25840000409483909749042140902020037174224853515625f,0.06890003190040516756198485381901264190673828125f,0.64239995958507112305824193754233419895172119140625f,-0.00449999096989639468802124611102044582366943359375f,0.243099995577335192820100928656756877899169921875f,-0.7899000124096868802325843716971576213836669921875f,-0.05379999044239536232225873391143977642059326171875f,0.1470999975591897879212410771287977695465087890625f,0.67960000016689292579030734486877918243408203125f,0.077600005620718004450964144780300557613372802734375f,0.00580000227391719302971750948927365243434906005859375f,-0.44659999605417244339378157746978104114532470703125f,-0.93020001519024386738010434783063828945159912109375f,-0.32200000523775818006555482497788034379482269287109375f,-0.92630003566741958564989545266143977642059326171875f,-0.58699999700486638065655142781906761229038238525390625f,-0.191499998293816975358794252315419726073741912841796875f,-0.31080001095384324205639359206543304026126861572265625f,-0.025999994069337617474957369267940521240234375f,0.1787000037088990467282201279886066913604736328125f,0.526300012123584792789188213646411895751953125f,0.343200000602007282424210643512196838855743408203125f,-0.0410999936923384101561396164470352232456207275390625f,-0.4480999920010564352423898526467382907867431640625f,-0.169200002568960172766310279257595539093017578125f,-0.0632999999269843183213879456161521375179290771484375f,0.4471000147700312510323783499188721179962158203125f,0.1854000173419716457345884919050149619579315185546875f,0.1752000053927302847256441964418627321720123291015625f,0.5251000185847285450790877803228795528411865234375f,-0.6057999921411276744720453280024230480194091796875f,-0.04099999999254944071935824467800557613372802734375f,0.13250000745058088114092242904007434844970703125f,0.1326000003308056562900674180127680301666259765625f,0.04889999867081640250177088091732002794742584228515625f,-0.11540000430345553894539989414624869823455810546875f,-0.24760001962780953110865311828092671930789947509765625f,-0.17240000309646130727259105697157792747020721435546875f,-0.35380001718997977633307527867145836353302001953125f,1.1200000070035454058370305574499070644378662109375f,0.251899998722970419606781433685682713985443115234375f,0.568100028359889819284944678656756877899169921875f,-0.0828000146269800563203489218722097575664520263671875f,-0.0825000053271651712094580943812616169452667236328125f,0.0514999902844426316050885361619293689727783203125f,0.3742000249952079915516378605389036238193511962890625f,0.2882000065997243343218769950908608734607696533203125f,0.3825000205636026606725863530300557613372802734375f,0.434800034356117581779699321486987173557281494140625f,0.318900008654594557810924015939235687255859375f,-0.6481999755680565300508533255197107791900634765625f,0.405199976447224941011882037855684757232666015625f,-0.328800000873207931562092198873870074748992919921875f,0.4564000137209891505563064129091799259185791015625f,0.074599991673231169642122040386311709880828857421875f,0.2169000074028968327866095933131873607635498046875f,-0.858300004434585872559182462282478809356689453125f,1.14059998775422588579431248945184051990509033203125f,-0.5018000075936317472269365680404007434844970703125f,-0.742999994389712536957404154236428439617156982421875f,0.63399998380243782758469706095638684928417205810546875f,-0.4083999998107550322146153121138922870159149169921875f,-0.502499987185000662748279864899814128875732421875f,0.5536000034153456539343096665106713771820068359375f,-0.26510001068711286364987245178781449794769287109375f,0.9532999895334253181999883963726460933685302734375f,-0.707399974411726528700228300294838845729827880859375f,0.5006000086903572565688591566868126392364501953125f,-0.59530002803802517519216053187847137451171875f,-0.116800000590085772955717402510344982147216796875f,-0.28510001113414773499243892729282379150390625f,-0.12189997640848115878497992525808513164520263671875f,0.0743999885529278426332666640519164502620697021484375f,-0.0528999962568281745234344271011650562286376953125f,-1.16069998763799731023027561604976654052734375f,1.0933999625951056344774769968353211879730224609375f,-0.6286000095248223118460373370908200740814208984375f,0.298400009906291785455323406495153903961181640625f,-0.027600001671910234790630056522786617279052734375f,0.14900000414252279057336636469699442386627197265625f,-0.113400031363963815778106436482630670070648193359375f,-0.01039997405409842645696016916190274059772491455078125f,-0.06460001000165949136544440989382565021514892578125f,1.48549999248981379196266061626374721527099609375f,-1.6777999930202955880531590082682669162750244140625f,0.832800014328956450526675325818359851837158203125f,-0.14870002632141154919054315541870892047882080078125f,-0.1925999875903128266685371272615157067775726318359375f,-0.065600008094310879869226482696831226348876953125f,0.26830004824399988905270220129750669002532958984375f,0.2079999798983334091673214061302132904529571533203125f,0.1349000109195710450649130507372319698333740234375f,0.8662000436067582853638668893836438655853271484375f,0.2525000057369470862767002472537569701671600341796875f,0.64800002914667143016913541941903531551361083984375f,0.33210000795274974283444180400692857801914215087890625f,0.95070005111694388943988087703473865985870361328125f,-0.416199985417723627989516899106092751026153564453125f,0.0652000013694167268596402209368534386157989501953125f,0.3627000058770182278067295555956661701202392578125f,0.0222000124186274572224419898702763020992279052734375f,0.076199999089539005581173114478588104248046875f,-0.089499992907047243306806194595992565155029296875f,0.08980001476109034097561334419879131019115447998046875f,0.09440000296980144423031333644757978618144989013671875f,0.22930001295805002570205033407546579837799072265625f,-0.016600000301003448033299036978860385715961456298828125f,-0.044899999967217441021460899719386361539363861083984375f,-0.091400000905990541610890431911684572696685791015625f,0.06159999866187571004871870172792114317417144775390625f,0.0583999978736043090510321462716092355549335479736328125f,0.4255999931186436047170218444080092012882232666015625f,0.4868000108718870677648737910203635692596435546875f,0.22870000043064353523192266948171891272068023681640625f,0.42750001430511463951233963598497211933135986328125f,-0.5045999997943637627173529835999943315982818603515625f,-0.094799998109042604710339219309389591217041015625f,-0.396300013315677546188453561626374721527099609375f,-0.01260000554621232282670462154783308506011962890625f,-0.057000003673136234727536475475062616169452667236328125f,-0.634100030475855280798214153037406504154205322265625f,0.3997999995619057056472911426681093871593475341796875f,-0.02810000205934048977951533743180334568023681640625f,0.056400002098083490409408113919198513031005859375f,0.158800004971027419742313213646411895751953125f,0.2690000002682209068183283307007513940334320068359375f,0.7748000045537946078866298194043338298797607421875f,0.1441999992161988597416666380013339221477508544921875f,-0.01069999862760308284581611815156065858900547027587890625f,-0.43959999253004788943854919125442393124103546142578125f,-0.915599991554021830353349287179298698902130126953125f,-0.1706999985530972541170058320858515799045562744140625f,-0.1658999932169908930745805264450609683990478515625f,-1.05739999877512413917202138691209256649017333984375f,-0.29169999660849565970011099125258624553680419921875f,-0.8574000153899188347850213176570832729339599609375f,-0.5526000321075326127129301312379539012908935546875f,-0.07740000333860519443618386503658257424831390380859375f,0.331500023066997329124205862171947956085205078125f,-0.48920002268552753577068870072253048419952392578125f,0.2091000073790549862451371154747903347015380859375f,-1.0332999698638918317783463862724602222442626953125f,1.012199963393807689016057338449172675609588623046875f,-0.55580000368952742206829498172737658023834228515625f,0.07030002803802393174237295170314610004425048828125f,0.3179999649971729791531060982379131019115447998046875f,0.01990000808835024059817442321218550205230712890625f,-0.045299978566169851745826235855929553508758544921875f,0.08879997971355935160175931741832755506038665771484375f,-0.02149999356269827188725685118697583675384521484375f,0.0555999966204165918526314271730370819568634033203125f,-0.108400000593066148102394663510494865477085113525390625f,0.030600001984834701573845450184307992458343505859375f,-0.22059999837130295219367326353676617145538330078125f,-0.09859999495893723153727705721394158899784088134765625f,-0.09750000132247793604989283267059363424777984619140625f,0.46370002006292310170465498231351375579833984375f,-0.3138000082492826692259768606163561344146728515625f,0.2566000067830085384912308654747903347015380859375f,-0.57689998594522418073893277323804795742034912109375f,0.9043999934703108589673092865268699824810028076171875f,-0.3381000036239623085521088796667754650115966796875f,0.457100018495321336331471684388816356658935546875f,0.09980000269114963629135672817938029766082763671875f,0.21200001093745246549815419712103903293609619140625f,0.68719998686313576996553820208646357059478759765625f,-1.0261999961465593234066773220547474920749664306640625f,0.39130001137852676862394218915142118930816650390625f,-0.38979997892379714130584034137427806854248046875f,0.406600003802775944450331735424697399139404296875f,-0.1965000081658363484393703402020037174224853515625f,0.57779999678283910924392330343835055828094482421875f,-0.1451999977558851018244467923068441450595855712890625f,0.2876000027522442881178221796290017664432525634765625f,-1.4438999913692480703275577980093657970428466796875f,1.373199954110384535255207083537243306636810302734375f,-0.789900010323524526256733224727213382720947265625f,-1.148599988663196125315835161018185317516326904296875f,1.514599975356459626851801658631302416324615478515625f,-0.66250000625848759483460526098497211933135986328125f,-0.44740004504323138689869665540754795074462890625f,-0.6717000024199482055564658367075026035308837890625f,-0.6128000122427941231961767698521725833415985107421875f,-0.16900000376999380069520384495262987911701202392578125f,-0.25900001829862606683718695421703159809112548828125f,-0.6187999957323071509307510496000759303569793701171875f,-0.15519999920874827381567229167558252811431884765625f,0.15419998494833742075371674218331463634967803955078125f,-0.22159998577237149941510097050922922790050506591796875f,0.05359999931752679191987454032641835510730743408203125f,-0.09159999933242790604026595246978104114532470703125f,-0.099800003138184489870354809681884944438934326171875f,0.0713000011712312531386714908876456320285797119140625f,0.4928000161170960069512148038484156131744384765625f,-0.39719999788999549306112157864845357835292816162109375f,-0.13739999855160711650370330971782095730304718017578125f,-0.44400000429153418934902219916693866252899169921875f,-0.441400011560320937320511802681721746921539306640625f,-0.14540000463873148373039612124557606875896453857421875f,-0.06210002162456529362088986090384423732757568359375f,1.041600010359287065142552819452248513698577880859375f,0.349500000618398143359399909968487918376922607421875f,1.326100042903423315010513761080801486968994140625f,0.73620003235638176164457036065869033336639404296875f,0.37650001002848154652014045495889149606227874755859375f,0.89480004299879123408345549250952899456024169921875f,-0.393000013612210652436118607511161826550960540771484375f,-0.0219000012002885380280048366330447606742382049560546875f,-0.172200015659630611963848423329181969165802001953125f,0.13039999849200245574110113011556677520275115966796875f,-0.02149999985843897842840988232637755572795867919921875f,-0.10649999624490735783410855219699442386627197265625f,-0.522400011813640485769383303704671561717987060546875f,-0.1694000012189149639851848405669443309307098388671875f,-0.9310000338554385024281145888380706310272216796875f,0.1014000072389837914244026251253671944141387939453125f,0.1427999977514147200707839147071354091167449951171875f,-0.0985000007748606520863177138380706310272216796875f,0.89580001076757886924184504096047021448612213134765625f,0.1799999999627470703700282683712430298328399658203125f,0.4919000226020813215654925443232059478759765625f,-0.578599995182454307496300316415727138519287109375f,-0.1863999960817395995515965978484018705785274505615234375f,-0.408900005860626247766731466981582343578338623046875f,0.8034000379949810533020126968040131032466888427734375f,0.60609998959451905076178945819265209138393402099609375f,0.4565000120401379035683930851519107818603515625f,-0.28909998130202296806601225398480892181396484375f,-0.78849996575713188207146231434307992458343505859375f,-1.1780000255703928502271082834340631961822509765625f,0.482299958658218397289374479441903531551361083984375f,-0.24500000681728106854961879434995353221893310546875f,0.3185000109076501217941768118180334568023681640625f,-0.485599975013732976236724425689317286014556884765625f,-0.3420999896615743995909042496350593864917755126953125f,-0.268000023186207414482851163484156131744384765625f,-0.536900005912780642347570392303168773651123046875f,0.5102000123962755484541276018717326223850250244140625f,-1.01299999547004659916638047434389591217041015625f,0.15260000070333479271766918827779591083526611328125f,-0.1966000029087064948640772854560054838657379150390625f,-0.5909999819993974057297236868180334568023681640625f,0.7007999796986581486635259352624416351318359375f,-0.36579997999668145691742893177433870732784271240234375f,0.22699998590350123350845024106092751026153564453125f,0.7704999938309191609420167878852225840091705322265625f,0.0243999996915460481972104389569722115993499755859375f,-0.13399999833107045787983224727213382720947265625f,1.17559998618960381833176143118180334568023681640625f,-0.4209999728724362899612287947093136608600616455078125f,0.3839999972879883927134869736619293689727783203125f,-0.085500001877546250028672147891484200954437255859375f,0.26200000460445860284153241082094609737396240234375f,0.862500004023313948664508643560111522674560546875f,0.1164000138700007624947829754091799259185791015625f,0.6992999709740284064451998347067274153232574462890625f,0.2334999837875368910999895888380706310272216796875f,0.75890000284314140799324377439916133880615234375f,0.78769996840059797182931333736632950603961944580078125f,-0.7960000079274180961874662898480892181396484375f,-0.4223000202000140035352160339243710041046142578125f,-0.442799984005093794081631131120957434177398681640625f,-0.911499991774558537827033433131873607635498046875f,0.35800001722574226192818969138897955417633056640625f,-0.14329999854862673913657999946735799312591552734375f,0.9605000266134737785250763408839702606201171875f,0.2338000021398070060740792541764676570892333984375f,0.9089000063821668362606942537240684032440185546875f,0.470500016927719144632646930404007434844970703125f,0.0923000248193748262792723835445940494537353515625f,-1.19249999620020386004171086824499070644378662109375f,-0.03359998359680194113252582610584795475006103515625f,0.643600002521276248756976201548241078853607177734375f,0.052000028669834108541181194595992565155029296875f,1.282699957895278686237361398525536060333251953125f,0.0242999864339825677461703890003263950347900390625f,0.9607000056684018574060246464796364307403564453125f,-0.3744999872446062028075175476260483264923095703125f,-0.66110000773668264884008749504573643207550048828125f,0.53969998075366021339505095966160297393798828125f,-0.818699986833333781532928696833550930023193359375f,0.3331000124156471287051317631267011165618896484375f,-0.0718000135540961537117254920303821563720703125f,-1.18219997524023057167141814716160297393798828125f,0.26159999694824165317186270840466022491455078125f,-1.1777999797582623386915656737983226776123046875f,-0.782099953675270853636902756989002227783203125f,1.0884999983906737952565890736877918243408203125f,0.37040004155635841698313015513122081756591796875f,1.83039996109008740177159779705107212066650390625f,0.155399989998340259944598074071109294891357421875f,-0.67730001707077036599002894945442676544189453125f,-0.47469999461173983945627696812152862548828125f,-0.2789000152111054120496191899292171001434326171875f,-0.236299998582154557880130596458911895751953125f,-1.020100001280754842269971049972809851169586181640625f,0.4975000268034641681680341207538731396198272705078125f,0.72830000108182435525350228999741375446319580078125f,-0.05029999763965609105298426584340631961822509765625f,-0.384100000375509065264623131952248513698577880859375f,0.4327999895185232759331483975984156131744384765625f,-0.02719998254179945007535934564657509326934814453125f,-0.6662000163376331496323246028623543679714202880859375f,0.57239996860027275715765426866710186004638671875f,0.58319995694160553512119804508984088897705078125f,0.339600044274330770122105604968965053558349609375f,0.35259999522715812503292909241281449794769287109375f,-0.786399990847706842345132827176712453365325927734375f,0.16160000164210763795580305668408982455730438232421875f,0.65310003253221537988792988471686840057373046875f,1.1412000075459474146555294282734394073486328125f,-0.150000009536743306171047152020037174224853515625f,-0.344199988412857038611036841757595539093017578125f,-0.3020999898105858338936968721100129187107086181640625f,0.039499994397163451509413789608515799045562744140625f,0.433899991741776513975992202176712453365325927734375f,0.74049998384714132981798684340901672840118408203125f,-0.0774999874830246682222423260100185871124267578125f,0.6798999680042268067836630507372319698333740234375f,-0.26149999963492154275712664457387290894985198974609375f,0.15749999582767504335834019002504646778106689453125f,-0.61139998019337671308903736644424498081207275390625f,-0.497900001844763517855341206086450256407260894775390625f,-0.832499988526105738628757535479962825775146484375f,-0.4277999925732611785633707768283784389495849609375f,-0.3258999835312368364981239210464991629123687744140625f,-0.283499990642070542889996431767940521240234375f,-0.5255999924480914131663666921667754650115966796875f,0.198099999786913283372769001289270818233489990234375f,-0.55100000405311533313579275272786617279052734375f,0.18409999203085902763632475398480892181396484375f,0.762599968591332721956632667570374906063079833984375f,-0.488499984979629431336434208787977695465087890625f,0.9531999663293362345939385704696178436279296875f,-0.6152999989435075089971860506921075284481048583984375f,0.8705000044107435996920685283839702606201171875f,-0.21439999399185172279658218030817806720733642578125f,0.6350999959349630241689510512514971196651458740234375f,-0.4720000334940852582121806335635483264923095703125f,-0.3272000047445298154258352951728738844394683837890625f,0.2405999890394510121183202500105835497379302978515625f,-0.035500001132488190336289335391484200954437255859375f,-0.29959998891651640207101081614382565021514892578125f,-0.44209999539852129490924426136189140379428863525390625f,0.553500012621283676850225674570538103580474853515625f,-0.32519997793734078950222965431748889386653900146484375f,-0.1700999949291348567470549824065528810024261474609375f,-0.873000005006790757988710538484156131744384765625f,0.3131999918103218760734307579696178436279296875f,-0.5897999602228407578508040387532673776149749755859375f,-0.4689999945685265902994842690532095730304718017578125f,-0.432199997480213593537001770528149791061878204345703125f,-0.1581999964669347225054707450908608734607696533203125f,-0.293499992430209744043168029747903347015380859375f,0.4278999999821184729853484896011650562286376953125f,-0.6397000057131048134095863133552484214305877685546875f,-0.244999997317790985107421875f,-1.06500000655651039238591692992486059665679931640625f,0.479000014815479602248160517774522304534912109375f,0.604899995180219196555526650627143681049346923828125f,-0.2449000056110322010027857686509378254413604736328125f,-0.576000020742416296570809208787977695465087890625f,0.618900050377845900584361515939235687255859375f,0.83110003515481967184541645110584795475006103515625f,0.241700011658668501013380591757595539093017578125f,0.87399997866153711356673738919198513031005859375f,-0.06360000535249721309583037509582936763763427734375f,0.19780001254081724937350372783839702606201171875f,-0.42670001761913312066099024377763271331787109375f,-0.9967000331163404780454584397375583648681640625f,0.01720002828836442176907439716160297393798828125f,0.512899969434737812434832449071109294891357421875f,0.53650000727176649917282702517695724964141845703125f,-1.56340003643035885261269868351519107818603515625f,-0.4915999337673184754748945124447345733642578125f,1.1858000297665594047202830552123486995697021484375f,-0.26990001062154789224223350174725055694580078125f,-1.28470001934766742124338634312152862548828125f,0.442900024271011716336943209171295166015625f,0.0444000097870826948565081693232059478759765625f,0.69989999572038641417748294770717620849609375f,-0.2601000036835667827972429222427308559417724609375f,-1.004299970042705325568022090010344982147216796875f,0.229100024366378107743003056384623050689697265625f,0.284299993288517072187460144050419330596923828125f,-0.29479999088197939016708915005438029766082763671875f,1.085000002458691437112747735227458178997039794921875f,-0.34260003072917510902328785959980450570583343505859375f,0.107699997495114718759623428923077881336212158203125f,0.389099982196092764041850387002341449260711669921875f,0.1033000019758938403668935279711149632930755615234375f,-0.528299980443716066247361595742404460906982421875f,1.4762999799370764009154299856163561344146728515625f,0.161099993133544838741499916068278253078460693359375f,-0.246299992305040404971805401146411895751953125f,0.686399986153841012992415926419198513031005859375f,-0.1290000056326388033767216256819665431976318359375f,-0.114199987965822202795607154257595539093017578125f,0.2032999832749364799155955552123486995697021484375f,0.590299995255469500676781535730697214603424072265625f,-0.7118000141501426014656317420303821563720703125f,-0.57779997871518151697500798036344349384307861328125f,-0.0369999969489871460837093763984739780426025390625f,0.1446999996781348496455166241503320634365081787109375f,-0.40099999993666990416585349521483294665813446044921875f,-0.66700000482797605627638404257595539093017578125f,0.723700022894143923934962003841064870357513427734375f,0.13900000697374348401780252970638684928417205810546875f,-0.100499989807605771829912555404007434844970703125f,0.395699987190961888217088926467113196849822998046875f,-0.31189999391734601896786216457257978618144989013671875f,1.163499996900558386414559208787977695465087890625f,-0.6036999997973442333432103623636066913604736328125f,0.1398999859228731157401171003584749996662139892578125f,-0.7195000048279762694392047706060111522674560546875f,0.451299968612194124517600357648916542530059814453125f,-0.21390000474303949573595673427917063236236572265625f,-0.39450000393390638464552466757595539093017578125f,0.44700002166628838295991954510100185871124267578125f,0.9742999974235890814355798283941112458705902099609375f,-0.374000026643276584081831970252096652984619140625f,-0.479800011855363806034802109934389591217041015625f,0.3217999980196355469530544723966158926486968994140625f,0.304499996483325929830243694595992565155029296875f,0.27510000129938116941730186226777732372283935546875f,-0.09630000329464660779876794549636542797088623046875f,0.3699999803304674372839144780300557613372802734375f,0.41479999665617928172878237091936171054840087890625f,0.63459996511191174572985573831829242408275604248046875f,0.6620000271424653082164013540023006498813629150390625f,0.51819999751001599719302248558960855007171630859375f,0.10359999808818098898655080120079219341278076171875f,-0.2174999906122685455756027295137755572795867919921875f,0.2207999897941947953228236656286753714084625244140625f,-0.166899989223480371691721302340738475322723388671875f,0.17200000646710389418103659409098327159881591796875f,0.9427999797582626850811493568471632897853851318359375f,0.1005999959871173299319480065605603158473968505859375f,0.3479999997913836295992950908839702606201171875f,0.3692999923944473295023271930404007434844970703125f,0.09059999740272761670212275930680334568023681640625f,-0.69099998095631587347043023328296840190887451171875f,0.3306999733328821466926683569909073412418365478515625f,-0.0765999964267016775210095147485844790935516357421875f,-0.145000014603137827862155972979962825775146484375f,0.45410001631975216440650910953991115093231201171875f,-0.59500001825392256904478927026502788066864013671875f,0.21790000698566469594652517116628587245941162109375f,0.300100012177229036325343258795328438282012939453125f,-0.12490002224445362344340537674725055694580078125f,1.80909998025894225293086492456495761871337890625f,-0.020900005733967219612168264575302600860595703125f,-1.000000007450580596923828125f,-0.1105999765038490068036480806767940521240234375f,0.3256000077962877270465469337068498134613037109375f,0.69760002752542504822486080229282379150390625f,-0.108100026869774268334367661736905574798583984375f,-0.91870004628896762000067610642872750759124755859375f,-0.20240001084506520356853798148222267627716064453125f,0.81259999682903316653437286731787025928497314453125f,0.342600014561414756286694682785309851169586181640625f,1.136200022745132542922874563373625278472900390625f,0.024799980413913402799153118394315242767333984375f,-0.236400001800060266532454988919198513031005859375f,-1.091400005972385400809798738919198513031005859375f,0.311100019955635076485123136080801486968994140625f,0.463100001835822894236116553656756877899169921875f,0.9828999867200849394066608510911464691162109375f,1.17529995889663751995612983591854572296142578125f,-0.5605000038146972229924358543939888477325439453125f,-0.47710001354068509726857882924377918243408203125f,-1.10899996677786116805464189383201301097869873046875f,0.518800003518163688198683303198777139186859130859375f,-0.072900005048513349947825190611183643341064453125f,0.5940000165104866169940578402020037174224853515625f,-0.108800008249283042260913134668953716754913330078125f,-0.5082000070095062937980401329696178436279296875f,0.9471000055313112397925578989088535308837890625f,0.16150001680851033825092599727213382720947265625f,0.37089998993873596333514797152020037174224853515625f,-0.384699990886449683102910057641565799713134765625f,-0.2154999951720235618068954863701947033405303955078125f,0.113299999032914655572312767617404460906982421875f,-0.17599999790638687358068636967800557613372802734375f,0.2176000058814882009983193711377680301666259765625f,0.3026999777138232872175649390555918216705322265625f,-1.4522999842882153842538173194043338298797607421875f,-0.3253000009179114471180582768283784389495849609375f,1.25659998621940616203573881648480892181396484375f,0.56319999024569999956923993522650562226772308349609375f,0.045999999582767348016432151780463755130767822265625f,0.33410000357925895286825834773480892181396484375f,0.33400000607967361698769082067883573472499847412109375f,0.110000040531158305157077847979962825775146484375f,0.978999993711710203569964505732059478759765625f,-0.2121999673053627244456720291054807603359222412109375f,0.314499986946582765767743694595992565155029296875f,0.24740000719428056896731504821218550205230712890625f,0.680499987535178707531713371281512081623077392578125f,0.6890000188946725501182299922220408916473388671875f,-0.505700012820958999526510524447076022624969482421875f,-0.2962000088870524461270861138473264873027801513671875f,-0.303999993562698467286509185214526951313018798828125f,0.587799997714161914785790941095910966396331787109375f,-0.0631999864459039384367855518576106987893581390380859375f,-0.5320000012516974408072201185859739780426025390625f,-0.28940000799894338712192620732821524143218994140625f,-0.78439999779164770732364786454127170145511627197265625f,0.1995000015497208067927203956060111522674560546875f,0.53609999998807911225640054908581078052520751953125f,0.64629999219328160275921391075826250016689300537109375f,0.000500025868415576724146376363933086395263671875f,0.24219999520778667800868788617663085460662841796875f,-0.799299965646863341817152104340493679046630859375f,-0.320999996788799535352154634892940521240234375f,-0.320099992358684470872276506270281970500946044921875f,0.4271999947607516556757900616503320634365081787109375f,0.19350001552701012741408703732304275035858154296875f,0.2931000098824501076677506716805510222911834716796875f,-0.024699995505809813067799041164107620716094970703125f,0.15949998962879163855177466757595539093017578125f,0.65770000617504109641231480054557323455810546875f,0.175900002194941063038413631147705018520355224609375f,0.00999999605119228363037109375f,-0.35019998620748526985124726706999354064464569091796875f,-0.1597999954268336242346748576892423443496227264404296875f,-0.36849999880045647415727216866798698902130126953125f,-0.0419000017404556057698528093169443309307098388671875f,-0.099099997209012524734816906857304275035858154296875f,0.21049999576807021384183826739899814128875732421875f,-0.941599980705976502548537609982304275035858154296875f,0.095100002491474044941099919014959596097469329833984375f,0.593500041007995804420716012828052043914794921875f,-1.4752149724017726839520037174224853515625e-08f,-0.778900007835030550040755770169198513031005859375f,-0.4759999819248914576519382535479962825775146484375f,-0.09880001458227649635546185891143977642059326171875f,-0.16050001811981218224900658242404460906982421875f,0.9923999918460850722112809307873249053955078125f,0.5876000370621685675587286823429167270660400390625f,-0.27970002836287033431972304242663085460662841796875f,1.36690001366138513816395061439834535121917724609375f,0.408700029450655311080708997906185686588287353515625f,-0.275999997645616446106942021287977695465087890625f,-0.1391999881148340278969044447876513004302978515625f,0.891200009930133152380449246265925467014312744140625f,0.78410002704858783317831694148480892181396484375f,0.014199968147277530761130037717521190643310546875f,-0.89300002676248535493641611537896096706390380859375f,1.5953999688386915067894733510911464691162109375f,-0.5661000271081917389892623759806156158447265625f,-0.364099984431266676665472914464771747589111328125f,0.09660001959800723625448881648480892181396484375f,0.744599986755847709218869567848742008209228515625f,-0.17240000257492038571172088268212974071502685546875f,0.696700009404867870443922583945095539093017578125f,1.025899977142363805882041560835205018520355224609375f,-0.3925000059790908490953142973012290894985198974609375f,-0.7258000415384771031312993727624416351318359375f,-1.642999983400105890041231759823858737945556640625f,0.4424000015318387823981538531370460987091064453125f,0.204000010251998986632315791212022304534912109375f,0.140999999880790483075543306767940521240234375f,0.34300000202655755998648601234890520572662353515625f,-0.12760001232624063050025142729282379150390625f,2.14450000333786050532580702565610408782958984375f,0.0943000266671187858946723281405866146087646484375f,0.127899991153180625502727707498706877231597900390625f,-1.104399984753131835901740487315692007541656494140625f,-0.3938000075787304599117533143726177513599395751953125f,0.20579999758005129706361913122236728668212890625f,1.233899974083900730192908667959272861480712890625f,-0.29330000443458548176067779422737658023834228515625f,0.659699989098310624058285611681640148162841796875f,-1.0007999674797058986541742342524230480194091796875f,0.179200007784366288632327268715016543865203857421875f,-0.3292999915003775868171942420303821563720703125f,0.6646999723345043964428668914479203522205352783203125f,0.7095000013262033888850055518560111522674560546875f,-0.1340999932229518520898636779747903347015380859375f,0.67129998790919775952801273888326250016689300537109375f,-0.65750000596046476175615680404007434844970703125f,0.75829999862313268721436543273739516735076904296875f,-0.1764999793320896070980552394757978618144989013671875f,-0.336499993264675278936692848219536244869232177734375f,0.92000001393258568072042180574499070644378662109375f,-0.11509997559487838980629703655722551047801971435546875f,-0.96399998894333815968593626166693866252899169921875f,0.2823999878227712034828300602384842932224273681640625f,0.0360000018179416070296383622917346656322479248046875f,0.5904999975562095215764202293939888477325439453125f,-0.448900009101629304808511733426712453365325927734375f,-0.385700009095668594483186097932048141956329345703125f,-0.701499986410141218584612943232059478759765625f,-0.7366999922871588779571538907475769519805908203125f,-0.992299959887564586580310788122005760669708251953125f,0.1504999924898147156682171043939888477325439453125f,0.318399993067979725225313814007677137851715087890625f,-0.160700006674229900482941957307048141956329345703125f,-0.0069999890886247584376178565435111522674560546875f,0.2757999951958656215111886922386474907398223876953125f,-0.60039998819157480713926133830682374536991119384765625f,-0.03349998661875730476822354830801486968994140625f,-0.5811999954760072029102957458235323429107666015625f,0.716100009115039792817469788133166730403900146484375f,-0.60199999231100065344435279257595539093017578125f,0.151399991071224260252847670926712453365325927734375f,-0.24599999839067454576735372029361315071582794189453125f,-0.041000001668930252662903512828052043914794921875f,1.19690001261830314405187891679815948009490966796875f,-0.291799981069564839941676837042905390262603759765625f,0.4279999924898147956042748774052597582340240478515625f,-0.7456999842479825613139610140933655202388763427734375f,0.178199993039667659022740053842426277697086334228515625f,0.2255000142455099165772480773739516735076904296875f,0.516099993431568204727000193088315427303314208984375f,-0.298099984215200120019062524079345166683197021484375f,-0.71849999406933751089354700525291264057159423828125f,-0.6002999818444252344562528378446586430072784423828125f,-0.349899993708729706298754535964690148830413818359375f,0.61550002378225343591111595742404460906982421875f,0.8371000247538089578114295363775454461574554443359375f,0.791100022600591312738060878473334014415740966796875f,0.896099991866945888574491618783213198184967041015625f,-0.1789000015020369982465808789129368960857391357421875f,0.46340000556632876538287746370770037174224853515625f,0.541299987629801027111398070701397955417633056640625f,-0.1295000010468065898550094061647541821002960205078125f,-0.414899982365220854063636579667218029499053955078125f,0.3800999713107942046264042801340110599994659423828125f,0.568100009100139136108964521554298698902130126953125f,0.840400003260374006686106440611183643341064453125f,-1.2799000255227088729270690237171947956085205078125f,-0.320300001513957877108396132825873792171478271484375f,-0.2821000287771227021949016489088535308837890625f,2.14329999668598247808404266834259033203125f,-0.03449998247623398128780536353588104248046875f,0.11719999997615815345852752216160297393798828125f,-0.4393999981641769636553362943232059478759765625f,-0.3292000083804129673126226407475769519805908203125f,-1.42260001411437997376197017729282379150390625f,-0.5741999741077421504087396897375583648681640625f,1.107000049829483856456135981716215610504150390625f,-0.43590000901222225593301118351519107818603515625f,1.61670001523494732964536524377763271331787109375f,0.080499987125396188503145822323858737945556640625f,-1.03459998943805686621999484486877918243408203125f,1.46570001058578469610438332892954349517822265625f,0.101099974954127702631012653000652790069580078125f,0.086500006675720442217425443232059478759765625f,2.4779000361919401029808796010911464691162109375f,-0.199900006747245839733295724727213382720947265625f,-0.43890000254511818411629064939916133880615234375f,0.075900032109021964288331218995153903961181640625f,-0.326700030583143785634092637337744235992431640625f,0.91209997960329047828054171986877918243408203125f,-0.646299998712539292000656132586300373077392578125f,-0.18549997550249042177483715931884944438934326171875f,-0.792599992954730936389751150272786617279052734375f,-0.20429998375177405023350729607045650482177734375f,0.53810001465082191174360559671185910701751708984375f,0.30780000076889990623385529033839702606201171875f,-0.1374000049591064254173033987171947956085205078125f,-0.053999993860721406946368006174452602863311767578125f,0.499500026285648601742650498636066913604736328125f,0.1669999988302588445066021449747495353221893310546875f,-0.3564999966174362544535370034282095730304718017578125f,0.2607999970212577789396846128511242568492889404296875f,-0.692699997606873285604933698778040707111358642578125f,-0.176499992799013849520406438386999070644378662109375f,0.1530999962478876152971452029305510222911834716796875f,-0.4032999918989836896088263529236428439617156982421875f,-0.44000000780448333959071760546066798269748687744140625f,0.47599997073039423955975735225365497171878814697265625f,0.117799994752556091981432473403401672840118408203125f,-0.19550000992417349010565885691903531551361083984375f,-0.9535000068843364484649782752967439591884613037109375f,-0.328299998883903043633836205117404460906982421875f,0.24250001490116090963056194595992565155029296875f,1.18280001194477080161959747783839702606201171875f,-0.151299970400333716469276623683981597423553466796875f,0.3474999964237213134765625f,-0.36819999434351924350039553246460855007171630859375f,0.0590999950483441782722593416110612452030181884765625f,-0.39500004768371610452959430404007434844970703125f,0.3488000046730039827025393606163561344146728515625f,0.90219998418092739456142226117663085460662841796875f,0.208500035643577774635559762828052043914794921875f,0.9720000042319298927395720966160297393798828125f,0.0263000135391948663965422383625991642475128173828125f,0.626000024914741715065247262828052043914794921875f,1.015899992173910248993706773035228252410888671875f,0.4851999920189380421930991360568441450595855712890625f,0.18450002479553262446643202565610408782958984375f,1.2324000225424764920489906216971576213836669921875f,-0.4424999917298553242517300532199442386627197265625f,0.7535000223815444542196928523480892181396484375f,0.01540001413822178477630586712621152400970458984375f,0.3772999915152788208416723136906512081623077392578125f,-0.4290000135302545203330737422220408916473388671875f,-0.584999981671571855912361570517532527446746826171875f,-0.51719998354762797276862329454161226749420166015625f,-0.11750002205371856689453125f,0.229099998140334992768885058467276394367218017578125f,0.49969998250454683574872660756227560341358184814453125f,-0.15650000622868542876631181570701301097869873046875f,-0.1515999990344047976265073884860612452030181884765625f,-0.19009999790936704666677314889966510236263275146484375f,-0.0269999914169307686506726895458996295928955078125f,0.67800001633167283898728783242404460906982421875f,0.705599994087219073435335303656756877899169921875f,-0.177000036418437645124868140555918216705322265625f,-0.2487999993085860950259302626363933086395263671875f,0.3418000093072652845194170367904007434844970703125f,-0.45010001485943806898148977779783308506011962890625f,-0.0193000026762486509568361725541763007640838623046875f,0.797999978464096759722679053083993494510650634765625f,-1.6475999854668972322002673536189831793308258056640625f,-0.081399977026879344332854770982521586120128631591796875f,-1.031300040733814427085235365666449069976806640625f,-0.495399978971480692280238145031034946441650390625f,0.47980001915693293312870082445442676544189453125f,-0.26460000292360774665212375111877918243408203125f,-0.9167999786108733673017923138104379177093505859375f,0.304600002178549544851193786598742008209228515625f,0.303599999317526680897572077810764312744140625f,0.5579999705106022389600184396840631961822509765625f,-0.3678000033766029019943744060583412647247314453125f,0.11129999096393561330842203460633754730224609375f,-0.2139000348806376905486104078590869903564453125f,-0.8085000100135797396205816767178475856781005859375f,0.67210000173151485114431125111877918243408203125f,-0.2584000219017266175569602637551724910736083984375f,0.17860002390444318365325671038590371608734130859375f,-0.350000012051314257632839144207537174224853515625f,0.047899989383668017950412831851281225681304931640625f,-0.1169000020198522893366543939919210970401763916015625f,-1.55350003087520605049576261080801486968994140625f,-0.52999996423721285054853069595992565155029296875f,1.071000011205672564074120600707828998565673828125f,-1.09959999450445167212819796986877918243408203125f,0.941200022745132258705780259333550930023193359375f,0.45059999974966036262458146666176617145538330078125f,-1.345200032103061715815783827565610408782958984375f,-0.170399953341483723079363699071109294891357421875f,0.45300001484155671960252220742404460906982421875f,0.3674000054061412612327330862171947956085205078125f,0.9597999842882156684709116234444081783294677734375f,0.17600000822544092926591474679298698902130126953125f,-0.55980001088678843501611481769941747188568115234375f,-0.36119999562501892143018267233856022357940673828125f,0.263600002521276355338386565563268959522247314453125f,0.12900000369548791923079988919198513031005859375f,0.6303999775931241611459654450300149619579315185546875f,-0.1954999939799311192700770334340631961822509765625f,-0.316600011104345213652777601964771747589111328125f,-0.43269998326450609393845070371753536164760589599609375f,-0.225500002399086962867613692651502788066864013671875f,-0.31270000997483726745684862180496565997600555419921875f,-0.1691000019773840623571459218510426580905914306640625f,-0.47550001992657786598073244022089056670665740966796875f,-1.059599978392571362206808771588839590549468994140625f,-0.28549999280646443100550868621212430298328399658203125f,0.1374999976158139958215542719699442386627197265625f,0.10130001227259644025480156415142118930816650390625f,0.73610000863075220056686021052883006632328033447265625f,-0.1985000045746563301918286015279591083526611328125f,-0.332199995654821389479138815659098327159881591796875f,0.28469999107271437566168970079161226749420166015625f,0.2545000034123656718065831228159368038177490234375f,0.112399992889165911691407018224708735942840576171875f,-0.2386999899253250401187642637523822486400604248046875f,0.97949999022483780208858661353588104248046875f,-0.40980000127553939393010296043939888477325439453125f,0.1269000027835367472306415947969071567058563232421875f,-0.731000016048550804725891794078052043914794921875f,-0.20830000711679463876180307124741375446319580078125f,-0.4851999910876154675776206204318441450595855712890625f,0.4125000012479720368219204829074442386627197265625f,0.0542999982059002572754025095491670072078704833984375f,0.27250000400468687988819738166057504713535308837890625f,-0.250999987959861670105965458787977695465087890625f,0.385700000751018734490571659989655017852783203125f,0.9980000006482001584373620062251575291156768798828125f,0.2185000019669534054855830618180334568023681640625f,0.868399999326467497695603015017695724964141845703125f,0.547699988405406568148237056448124349117279052734375f,0.4034999933242797709453952847979962825775146484375f,0.32640002310872073820746663841418921947479248046875f,0.91589999593645321507295875562704168260097503662109375f,-0.50849999094009401545690707280300557613372802734375f,0.497999990701675443460771930404007434844970703125f,-0.469299986918270572999745127162896096706390380859375f,0.070999999880790642947658852790482342243194580078125f,0.00699999789893634982718140236102044582366943359375f,0.41459999867677677087840493186376988887786865234375f,-0.8624999988079071755464610760100185871124267578125f,0.1627999996513127545227916925796307623386383056640625f,0.3752000070691110522602684795856475830078125f,-0.487399978137016631762890028767287731170654296875f,0.38880000675916637220552729559130966663360595703125f,-0.327700026440620462153674452565610408782958984375f,0.91450002896785775874377577565610408782958984375f,0.73610003992319139598521360312588512897491455078125f,-0.01219998090267182533352752216160297393798828125f,1.4041999723196028071470209397375583648681640625f,0.5364000047802921500306183588691055774688720703125f,0.108600010512024169884170987643301486968994140625f,0.527699990435689603174296280485577881336212158203125f,-0.3105000056587157164500467843026854097843170166015625f,0.103800001543760345157352276146411895751953125f,-0.37979999158978472451053676195442676544189453125f,0.52640002221465120868515441543422639369964599609375f,-0.9574999938905239815767345135100185871124267578125f,0.3454999853372573426213421043939888477325439453125f,0.34349998676776838379964829073287546634674072265625f,0.5203999899983404731074188021011650562286376953125f,0.6474999763071540570535944425500929355621337890625f,0.02230000947117805054631389793939888477325439453125f,-0.23029999644756316001803497783839702606201171875f,-1.75299998950958269006150658242404460906982421875f,0.40040002725124423932356876321136951446533203125f,-1.098500032514333923927551950328052043914794921875f,-0.95349998900294252734965994022786617279052734375f,0.5395000180900098030178924091160297393798828125f,-0.66769999280124903151545368018560111522674560546875f,1.303399998953938432322274820762686431407928466796875f,0.28820000142157076172821916770772077143192291259765625f,-1.06840000185966488288613618351519107818603515625f,1.06840001676082607673379243351519107818603515625f,0.21979998458623839496794971637427806854248046875f,-0.12409999430328610969809233210980892181396484375f,-1.05460000370591888696480964426882565021514892578125f,0.536400031267107113563952225376851856708526611328125f,-0.81680000245273109982235837378539144992828369140625f,0.12380002076625817863941847463138401508331298828125f,0.469600009852647826136262665386311709880828857421875f,-0.1633000090166927265045160311274230480194091796875f,-0.34460000497251730422476612147875130176544189453125f,0.176000008560717180472465770435519516468048095703125f,0.8780999858915805589276715181767940521240234375f,0.08359999805092811764328786239275359548628330230712890625f,-0.7854999981522556851132321753539144992828369140625f,-0.200599996322393447911736075184307992458343505859375f,-0.2739000022843478543421724680229090154170989990234375f,-0.728000002324581174661943805404007434844970703125f,0.800000017136335284106962717487476766109466552734375f,-0.07259998129457245763518358216970227658748626708984375f,-0.70249998569488525390625f,0.93079997180104268039713133475743234157562255859375f,-0.3440999914720654917488218416110612452030181884765625f,0.32399999607726925887618563137948513031005859375f,0.1538999988198279655904343599104322493076324462890625f,-0.10320000356361258564419358663144521415233612060546875f,-0.7655000132024287751164592918939888477325439453125f,-0.043600000733137189712351755588315427303314208984375f,-0.1231000007554887698546508545405231416225433349609375f,0.25700001108646386427381003159098327159881591796875f,0.48759997715950031960119304130785167217254638671875f,0.56799999312311388877105855499394237995147705078125f,-0.41849998721480385910354016232304275035858154296875f,0.24989999348521241273601845023222267627716064453125f,-0.495299976815283482523000202490948140621185302734375f,-0.345000047087669514667140902020037174224853515625f,-0.7508999970912935140177069115452468395233154296875f,0.3815999782845380661200351823936216533184051513671875f,0.1114999889433381241587994736619293689727783203125f,-0.108799996924400232956031686626374721527099609375f,0.85050000075995890735924831460579298436641693115234375f,0.1095000047907233042820962509722448885440826416015625f,0.921299991746246771384676321758888661861419677734375f,0.20489999829083682669761401484720408916473388671875f,0.497500020861625813495265902020037174224853515625f,0.8515999979913235762296608299948275089263916015625f,0.5522999960973857813684162465506233274936676025390625f,-0.606000039331615170112854684703052043914794921875f,-0.354300006103515752187149701057933270931243896484375f,0.1561999875038864171727936991374008357524871826171875f,-0.011500008165836295148665158194489777088165283203125f,0.38500000260770317339620305574499070644378662109375f,0.4760999908611178721429268989595584571361541748046875f,-0.10800000489503158718207487254403531551361083984375f,-0.091500002652406653425032345694489777088165283203125f,0.140600002878904228875711623913957737386226654052734375f,-0.9654999771416186149508575908839702606201171875f,-0.2804000098168852872504430706612765789031982421875f,-0.1897000273942950343553093262016773223876953125f,-1.12460002341270381975846248678863048553466796875f,-0.6438000067591662656241169315762817859649658203125f,-1.291000020742416154462262056767940521240234375f,-0.30109991385936751839835778810083866119384765625f,1.0601000266313551634311806992627680301666259765625f,0.9508000095009805363588384352624416351318359375f,-0.045099990272522205714267329312860965728759765625f,-1.01930002726316448757870603003539144992828369140625f,0.2268999936006963480394915677607059478759765625f,-0.442299993582814909842682027374394237995147705078125f,0.3611000095807013199333823649794794619083404541015625f,-0.19719998328685761634915252216160297393798828125f,-0.681500020384788740557269193232059478759765625f,-0.7729000394701959208987318561412394046783447265625f,-0.044500000897794933507611858658492565155029296875f,-0.429499992664903462724623750546015799045562744140625f,0.2735000076852738271071530107292346656322479248046875f,0.0986000195086003117239670245908200740814208984375f,-0.8930000103712085746110460604541003704071044921875f,-0.21580002201795611682655362528748810291290283203125f,0.126200008626282222934378296486102044582366943359375f,0.415899989491701038701876314007677137851715087890625f,0.0935000005513428877890191870392300188541412353515625f,-0.310200004014372865412951796315610408782958984375f,-0.3146999979645013212348203524015843868255615234375f,0.185300007548928391543086036108434200286865234375f,0.46820001438558112027976676472462713718414306640625f,0.02739999929666492306523650768212974071502685546875f,-0.268000008434057246375914473901502788066864013671875f,0.364300013852119519697225769050419330596923828125f,0.359599996292590873281369567848742008209228515625f,0.2100000101327896828706798260100185871124267578125f,-0.618599980020523076973404386080801486968994140625f,1.44509998550415019735737587325274944305419921875f,0.31970000325441372268642226117663085460662841796875f,1.2709000394105913755993242375552654266357421875f,-0.56740001688003616209243773482739925384521484375f,-0.3058000297665603994801131193526089191436767578125f,-0.261500005707144822508780634962022304534912109375f,0.2383999922484161260172186302952468395233154296875f,-0.15740000548064703167483457946218550205230712890625f,-0.5483000045277179879121831618249416351318359375f,-0.5927999795347453915184132711146958172321319580078125f,0.991499989986419194565314683131873607635498046875f,-0.77909999560713760757835188996978104114532470703125f,0.4669000039756294739845543517731130123138427734375f,-0.432500008940696289982952293939888477325439453125f,0.45230003167390808727077455841936171054840087890625f,0.769499987281859088028568294248543679714202880859375f,0.6330000088810923131177332834340631961822509765625f,-0.555899980551004535556103292037732899188995361328125f,-0.4706000028789041333965315061504952609539031982421875f,-0.6394999951235948998373714857734739780426025390625f,-0.1570000005066394255237582910922355949878692626953125f,-0.1672999928750098508345445225131697952747344970703125f,1.16400000011920923270736238919198513031005859375f,-0.58590001035332672785216345801018178462982177734375f,0.4243999796122313039603568540769629180431365966796875f,-0.32350000492855901512712080148048698902130126953125f,-0.1268000001430511503031084430404007434844970703125f,0.0474999977461993960758945831912569701671600341796875f,0.21499999880790721107359786401502788066864013671875f,-0.5672000123441218732978086336515843868255615234375f,0.125799983572960016875441624506493099033832550048828125f,-0.2184999967515468544121404192992486059665679931640625f,0.2280999912932515361063678938080556690692901611328125f,-0.182899997933208935396720562494010664522647857666015625f,-0.011500009432435120970694697462022304534912109375f,-0.043700000691413887210501343361102044582366943359375f,0.24089999623447655441310644164332188665866851806640625f,0.004499999761581552348843615618534386157989501953125f,-0.16799999897181994157335793715901672840118408203125f,-0.2899000047728417239767395585658960044384002685546875f,-0.5475000050663947348539295489899814128875732421875f,0.1910999947726725878283104975707828998565673828125f,-0.37209999979436381156716606710688211023807525634765625f,0.030999996602535162537606083787977695465087890625f,0.997699982184171663135430208058096468448638916015625f,0.13819999322593223833877118522650562226772308349609375f,-0.537499983310699036564983543939888477325439453125f,-0.65410001795887939834983626496978104114532470703125f,-0.7673000079810616824715907569043338298797607421875f,0.3990000033229588183303349069319665431976318359375f,0.1244999985694885680231891456060111522674560546875f,0.18400000194460142122210299930884502828121185302734375f,0.801999992180615350889638648368418216705322265625f,-0.7620000182390214149563689716160297393798828125f,-0.74220001487731934730618377216160297393798828125f,-0.14579997284412371527650975622236728668212890625f,0.980400037086010200937380432151257991790771484375f,0.02309999751448632565598018118180334568023681640625f,-0.21029999316930769737155060283839702606201171875f,-0.1665000008642671947001190346782095730304718017578125f,-0.23760002663135537659400142729282379150390625f,-2.35499998211860628316571819595992565155029296875f,0.2108000166535379804599870112724602222442626953125f,1.2299999892711639404296875f,-1.09890000269412979605476721189916133880615234375f,-0.3360999943256377520128808100707828998565673828125f,1.3547999598503110973979346454143524169921875f,-0.76260001113414688234115601517260074615478515625f,-0.5047999935269356086564584984444081783294677734375f,0.396400003811717027701888582669198513031005859375f,-0.1462000095576048153134252061136066913604736328125f,0.06100000695884233437027432955801486968994140625f,0.09930001683235190057530417107045650482177734375f,0.61920002834796861179711413569748401641845703125f,0.7934000328540804503063554875552654266357421875f,-0.28610001757144942757804528810083866119384765625f,-0.58240000376701317463812301866710186004638671875f,-0.813800017786025620125656132586300373077392578125f,-0.3217999688506125721687567420303821563720703125f,0.763399983978271023943307227455079555511474609375f,0.81000001639127727770528508699499070644378662109375f,-0.27239998648166618977484176866710186004638671875f,-2.04179993696212846998605527915060520172119140625f,0.384999994635581543889202293939888477325439453125f,0.378299982567131376498537065344862639904022216796875f,0.955499986529350753272638030466623604297637939453125f,0.0281000160664321985137803494581021368503570556640625f,0.4788000331342221471686571021564304828643798828125f,1.3826999763727183534456344204954802989959716796875f,0.34489999840259510932582998066209256649017333984375f,-0.2158000473499299687318853102624416351318359375f,-1.71709997990131313372330623678863048553466796875f,0.2865000120401379746226666611619293689727783203125f,0.912600034677982563380282954312860965728759765625f,0.739799999189376222830105689354240894317626953125f,-0.151600000822543989897894789464771747589111328125f,0.977900009667873604257692932151257991790771484375f,-0.66639997803270833998112721019424498081207275390625f,-0.735500017523765592386553180404007434844970703125f,0.400400005644559797701731440611183643341064453125f,0.3588999997511506112601864515454508364200592041015625f,0.20249999500811100006103515625f,-0.17279999242424970429254926784778945147991180419921875f,0.0542999970883131510834829214218189008533954620361328125f,-0.087500024437904500018703402020037174224853515625f,-1.13379998694062233965951236314140260219573974609375f,0.529699993680417424712913998519070446491241455078125f,-0.464500000357627840230634319595992565155029296875f,-0.93889998838901522759670115192420780658721923828125f,-0.640499986566603229931615715031512081623077392578125f,-0.337499979734420918475734652020037174224853515625f,-0.84179998241066922304298714152537286281585693359375f,-0.753099961639941195556957609369419515132904052734375f,-0.373000003740191488077471149154007434844970703125f,-0.134600004339218148885493064881302416324615478515625f,-0.27779999644756314669535868233651854097843170166015625f,-1.118000018119811755923365126363933086395263671875f,0.5789000050783155160161186358891427516937255859375f,-0.3466000058516855997226002727984450757503509521484375f,1.282999994874000293521021376363933086395263671875f,-0.52969998656511307899563689716160297393798828125f,0.4811999959602950571735391349648125469684600830078125f,-0.8285000126957893229473484097979962825775146484375f,0.743599972122907626470578179578296840190887451171875f,-0.0669000107184050119002449719118885695934295654296875f,-0.292000016450881361151914461515843868255615234375f,-0.901399982130527632762095890939235687255859375f,0.4647999658107762055436751325032673776149749755859375f,-0.3224999895691873508241087620262987911701202392578125f,0.4434000024929642957971509531489573419094085693359375f,-0.445099974328279746060843535815365612506866455078125f,-0.81499998569488507627056605997495353221893310546875f,0.75829997433424001229695932124741375446319580078125f,-0.6585999961137769531660524080507457256317138671875f,-0.206499992966652001769034541212022304534912109375f,-0.77329998446702941095054484321735799312591552734375f,0.3891000159099693433972788625396788120269775390625f,-0.4815000048875806015757916611619293689727783203125f,0.16319999821782094073796542943455278873443603515625f,-0.76070000667422965623387653977260924875736236572265625f,-0.4410000100135800238376759807579219341278076171875f,-0.735400024867058021982302307151257991790771484375f,-0.2009000167608263609508867375552654266357421875f,-0.88450003612041427913936786353588104248046875f,-0.65830003483295485011694836430251598358154296875f,-0.82990001673102398171977256424725055694580078125f,-1.321700006443261798949606600217521190643310546875f,0.8009000294268133757213945500552654266357421875f,-0.5014999843612313323859552838257513940334320068359375f,0.721099984453618336743829786428250372409820556640625f,0.431899997344612762706361763775930739939212799072265625f,0.108399986362456957067479379475116729736328125f,1.88419995324611733167330385185778141021728515625f,-0.0923999814152711707038179156370460987091064453125f,1.2758000199317933720521978102624416351318359375f,-0.65450000870227853511096327565610408782958984375f,-0.5479000192046168393744665081612765789031982421875f,0.49750001732259985232076360261999070644378662109375f,-0.519100035169720808170268355752341449260711669921875f,-0.65990002857744711750598298749537207186222076416015625f,1.00220000534057618324368377216160297393798828125f,-1.08290001771450050682688015513122081756591796875f,-0.384700009214878235752621549181640148162841796875f,0.1860999697387217821642479975707828998565673828125f,-1.0953999783754344576891526230610907077789306640625f,-0.49339999828338587661846759147010743618011474609375f,-0.7778999733835458840758292353712022304534912109375f,1.16579997537732094059492737869732081890106201171875f,0.1943999846041200019186589997843839228153228759765625f,-0.466299987387657211002078838646411895751953125f,-0.54860002055168166634757653810083866119384765625f,-0.6630000398755075963208582834340631961822509765625f,-0.77709998727738849311208468861877918243408203125f,0.1093000152677294778413852327503263950347900390625f,-0.19450001130998106901870414731092751026153564453125f,-0.0918000148952007322122881305404007434844970703125f,-1.2314999786615370425124638131819665431976318359375f,-0.48410000529289209936223414842970669269561767578125f,-0.51290001275986440987253445200622081756591796875f,-0.00759999671727396020060041337274014949798583984375f,-0.231200011830031915138761178241111338138580322265625f,1.23160001006126407219198881648480892181396484375f,-0.598900024747848647166392765939235687255859375f,-0.1459000075221066339281605905853211879730224609375f,0.63569998257160165167078957892954349517822265625f,-0.2169999990910291831625045233522541821002960205078125f,1.01750002086162538716962444595992565155029296875f,-0.654200013446807560058005037717521190643310546875f,0.437300005820393078437291478621773421764373779296875f,-0.4285000268518928123739897273480892181396484375f,-0.26840001139640801142149939551018178462982177734375f,0.6612999951362608097582551636151038110256195068359375f,-0.1989999939166009479407648541382513940334320068359375f,0.62799997743964208751776823191903531551361083984375f,0.1519999957196414286375585334099014289677143096923828125f,0.030500005006789621120333322323858737945556640625f,0.94899998804926877227217119070701301097869873046875f,-0.380299970854819147092484854510985314846038818359375f,-0.222999992489814502505396376363933086395263671875f,-0.78350000062584879145788363530300557613372802734375f,-0.70460000158250313262442432460375130176544189453125f,0.5000000045076011812028582426137290894985198974609375f,-0.597200015957653373988023304264061152935028076171875f,-0.13500000640749931779538428600062616169452667236328125f,-0.231999998569488496968915569595992565155029296875f,-0.902300003808736761357067734934389591217041015625f,-0.4663999968826770814445126234204508364200592041015625f,0.51549999368190757564889281638897955417633056640625f,-0.7860999710798264317190842120908200740814208984375f,0.155300005164742349617057470823056064546108245849609375f,0.315000005587935394402165911742486059665679931640625f,0.83789997159540663407284455388435162603855133056640625f,0.3287999874249101228684821762726642191410064697265625f,1.218500004053115759461434208787977695465087890625f,-0.42530001268982875473056992632336914539337158203125f,0.575099981033801999075194544275291264057159423828125f,0.830499984666705159952471149154007434844970703125f,0.14420000510215746913900147774256765842437744140625f,0.5906999758660795851739067074959166347980499267578125f,0.70999998509883877062520696199499070644378662109375f,-0.800899977421760667084527085535228252410888671875f,0.3795999900341033850281746708787977695465087890625f,0.683500006698071871369393193162977695465087890625f,0.017900004750490250415850823628716170787811279296875f,0.4252999994650481507818540194421075284481048583984375f,-0.7179999859333037193209747783839702606201171875f,-0.60890001267790783145983368740417063236236572265625f,-0.7671999884277582726355149134178645908832550048828125f,0.717499990463256409611858543939888477325439453125f,-0.0243999901175495637062340392731130123138427734375f,-0.95550000560283621098278672434389591217041015625f,-0.50339989933967643764844979159533977508544921875f,1.009400024688243746595617267303168773651123046875f,0.42770000572800626059688511304557323455810546875f,1.321599968785047707342528156004846096038818359375f,-0.4312000044167039192188894958235323429107666015625f,0.81180003769397757196202292107045650482177734375f,1.245599989914893512832350097596645355224609375f,-0.7210000293850900021652705618180334568023681640625f,-0.28220001398324967567532439716160297393798828125f,1.621700014042854576246099895797669887542724609375f,-0.15610000654459010860364287509582936763763427734375f,0.46020000081062306662715855054557323455810546875f,-1.4943000204086303028816473670303821563720703125f,-0.0592999831557268208825917099602520465850830078125f,0.464800011259317358280895859934389591217041015625f,-0.6280000249743462603646548814140260219573974609375f,0.076600006484985438959256498492322862148284912109375f,1.3094000178337097395342425443232059478759765625f,0.283599968099593979786732234060764312744140625f,-0.639800012004375417973278672434389591217041015625f,-0.5017000022709368067808100022375583648681640625f,-0.078499962180853088966614450328052043914794921875f,-0.39690002722144157587536028586328029632568359375f,0.389500043235719406453654301003552973270416259765625f,1.243399986064433360155589980422519147396087646484375f,0.1325999918371436603337087944964878261089324951171875f,0.365000002086162567138671875f,-0.37910001095533374382284819148480892181396484375f,-0.052900008773804074735380709171295166015625f,-0.301200003373622848812374286353588104248046875f,0.611700016427040083044630591757595539093017578125f,0.77690002379417411049189468030817806720733642578125f,0.311200015722960188213619403541088104248046875f,0.974899973685294352065966450027190148830413818359375f,-0.4775000044889743389120440042461268603801727294921875f,0.089799998220801313664196641184389591217041015625f,0.807799991306662690249140723608434200286865234375f,-0.294200009498000181196175617515109479427337646484375f,0.017599998170137354236430837772786617279052734375f,0.876399987345934050608775578439235687255859375f,-0.159200011807680397168951458297669887542724609375f,-1.297900017416477425058474182151257991790771484375f,-0.557900001509487264428344133193604648113250732421875f,-0.613500015377997698351464350707828998565673828125f,0.26190003735423061925757792778313159942626953125f,0.5694999772608284605013295731623657047748565673828125f,0.241500008314847747215026174671947956085205078125f,0.4724999926984310150146484375f,-0.4458999870330095927073443817789666354656219482421875f,0.33750001311302213480303180404007434844970703125f,0.25859999656081189556289245956577360630035400390625f,-0.6521000073567029264864913784549571573734283447265625f,0.4050000253319741005952892010100185871124267578125f,0.864300012511014958960231524542905390262603759765625f,-0.036599984952807584903666793252341449260711669921875f,-0.25500003218650846292803180404007434844970703125f,-0.90450000438094146915091187111102044582366943359375f,-0.0694000058382748807872530960594303905963897705078125f,-0.44150001189112675348269476671703159809112548828125f,-0.3988000148057937455092769596376456320285797119140625f,-0.2491000016048550147473861215985380113124847412109375f,-0.132499992847442626953125f,-0.3226000071108341415992981637828052043914794921875f,-0.869299991090595636222815301152877509593963623046875f,0.893000012308359458756967796944081783294677734375f,-0.07819998033642772128359865746460855007171630859375f,0.4643999863550067885853422922082245349884033203125f,-0.4694999955892560361547793945646844804286956787109375f,0.6296999799713491885455596275278367102146148681640625f,-0.52450000944733587981971822955529205501079559326171875f,-0.19899998909234994926009676419198513031005859375f,-0.2909000076711176774324485450051724910736083984375f,-0.19099999563395986257319236756302416324615478515625f,-0.7994999994635580975455013685859739780426025390625f,0.647699999320507036060234895558096468448638916015625f,-0.0490999939307570354429799408535473048686981201171875f,0.3904999933280048907136006164364516735076904296875f,0.2952999939143656771278756423271261155605316162109375f,-0.2919000004552303462190820937394164502620697021484375f,0.31500001348555084490499211824499070644378662109375f,0.3120000002086162904646471361047588288784027099609375f,-0.1405999917775393459606902979430742561817169189453125f,0.1910000208914279795635593472979962825775146484375f,0.408399994111061204193902085535228252410888671875f,-0.12819998920261876929060917973401956260204315185546875f,0.2715000114440915268687604111619293689727783203125f,-0.533900001502036758438407559879124164581298828125f,0.530600011800974602493852216866798698902130126953125f,-0.7775000232085587104080559583962894976139068603515625f,-0.335300016377866494377002481996896676719188690185546875f,1.2938000197231769305972193251363933086395263671875f,-0.2843000472307206649702493450604379177093505859375f,-0.55810001971721678870608229772187769412994384765625f,-0.11920002872049817455035736202262341976165771484375f,-1.42609998240470847719052471802569925785064697265625f,-0.381300002139806526457732616108842194080352783203125f,1.188699984002113296810421161353588104248046875f,-0.4303000071763989353712531737983226776123046875f,-0.031299985003471419986453838646411895751953125f,0.097000008106231661031415569595992565155029296875f,0.16399998521804803885970613919198513031005859375f,0.38200001466274269290579468361102044582366943359375f,0.4060999851614237599051193683408200740814208984375f,-0.77419997604489321219034536625258624553680419921875f,0.1984000064790245598800311199738644063472747802734375f,-1.47730001409053812722049769945442676544189453125f,-0.63139998660087570669929846189916133880615234375f,0.983800019276142023727516061626374721527099609375f,-0.234900018537789634365253732539713382720947265625f,-0.649499989908188357645713040255941450595855712890625f,0.2867000041894611772619327894062735140323638916015625f,-0.369500016152858989926244248636066913604736328125f,-0.588400031661986844255807227455079555511474609375f,-0.2766000136375428297696998924948275089263916015625f,0.259199994969367963904005591757595539093017578125f,-0.35519999663829793234981480054557323455810546875f,-0.61440000591278032260333930025808513164520263671875f,-1.0831000101804733048993512056767940521240234375f,0.08410003151893619133261381648480892181396484375f,0.3157000061154366932214543339796364307403564453125f,0.15070000905841585137068250332958996295928955078125f,0.095599981048703153163614842924289405345916748046875f,-0.26120000747144211938888247459544800221920013427734375f,-0.91680001951456091546788229607045650482177734375f,-0.315000001788138916936077293939888477325439453125f,0.94140004858970716128396816202439367771148681640625f,-0.146999986499547929952314007095992565155029296875f,1.1903999917864798163691375521011650562286376953125f,-0.191400011634826672235476507921703159809112548828125f,0.18529998720884321983248810283839702606201171875f,-0.37729999945014702777967841029749251902103424072265625f,0.05100000224635004020257156298612244427204132080078125f,-0.630900012122839637385141031700186431407928466796875f,-0.184200003667920830796589370947913266718387603759765625f,-0.22499998748302463269510553800500929355621337890625f,-0.5937000206589697626213819603435695171356201171875f,-0.778599991717934702961656512343324720859527587890625f,0.596499992370605358615875957184471189975738525390625f,-0.864099978247284905563674328732304275035858154296875f,0.4108999996617435090939807196264155209064483642578125f,-0.83749999105930328369140625f,-0.471999993205070467183759319595992565155029296875f,-0.78889997482895868330388111644424498081207275390625f,-0.574000007569789971739737666212022304534912109375f,0.197799986910820013719103371840901672840118408203125f,-0.17999999798834320774432171674561686813831329345703125f,-0.556999991044401898676596829318441450595855712890625f,-0.42369998675882813987669806010671891272068023681640625f,-0.2071999911472202082762805730453692376613616943359375f,-0.192500013709068440448390902020037174224853515625f,0.28729999039769182900272426195442676544189453125f,0.9780999896168707241628226256580092012882232666015625f,-0.1984999927841126776684177457354962825775146484375f,-0.2019999972283839095865687340847216546535491943359375f,0.22130000629350521368365889429696835577487945556640625f,0.6400000065565105700216008699499070644378662109375f,-0.334300012958049563849272090010344982147216796875f,0.42900000537186822047175382977002300322055816650390625f,0.615999990046024237244637333787977695465087890625f,-0.447999984741211054739551400416530668735504150390625f,0.00159999665021903769712707799044437706470489501953125f,0.387999991297721891214678180404007434844970703125f,0.36380001167654985039234816213138401508331298828125f,0.710199988554418126796008436940610408782958984375f,0.2304999975189565564193117097602225840091705322265625f,-0.067300007943809025956483083064085803925991058349609375f,-0.0709000024929642425064457711414434015750885009765625f,-0.5540000355839733714446992962621152400970458984375f,0.331399995541572511825734181911684572696685791015625f,0.52360000088214864177871277206577360630035400390625f,0.35350000837445261225866488530300557613372802734375f,0.5965999942660331623045522064785473048686981201171875f,-0.0545000018849968181910981002147309482097625732421875f,-0.5784999977946281290996921597979962825775146484375f,-0.663000025570392725882129525416530668735504150390625f,0.608599978530406815480091609060764312744140625f,0.5898999808192257887640153057873249053955078125f,0.4163000115275377055468197795562446117401123046875f,-1.30730001230239878395877894945442676544189453125f,-0.58389998719692215445320471189916133880615234375f,0.90930002666711828851475729607045650482177734375f,-0.51750000752508640289306640625f,-1.4558999824136495959692183532752096652984619140625f,0.57890001871287832813095519668422639369964599609375f,0.4456000034749507676679058931767940521240234375f,-0.5626000129222870071998841012828052043914794921875f,-0.022199995207786482609435552149079740047454833984375f,0.28750002384185791015625f,-0.8647000244140627955857780762016773223876953125f,-0.804300019514560204925146535970270633697509765625f,0.79129998321533179250764078460633754730224609375f,0.0405999607086187097593210637569427490234375f,0.2110000240206719723801143118180334568023681640625f,-0.0769000011444092024248675443232059478759765625f,1.97050000560283677941697533242404460906982421875f,0.1247000273942955317352243582718074321746826171875f,0.7544999958127736050528255873359739780426025390625f,-1.22400001844763739455856921267695724964141845703125f,-0.2239999841749662490286709726206026971340179443359375f,0.50980001289844523171268519945442676544189453125f,1.26199999618530256384474341757595539093017578125f,-0.5684000239133837339977617375552654266357421875f,0.120800005850195901757615501992404460906982421875f,0.41649999587237829246078035794198513031005859375f,-0.103900002023577631149464650661684572696685791015625f,-1.087600031697750324610751704312860965728759765625f,-0.331700000929831873008879483677446842193603515625f,0.132499996721744395244968472979962825775146484375f,0.40050001275539415246385033242404460906982421875f,-1.9148999912500386244573746807873249053955078125f,0.1068999826669685404567644582130014896392822265625f,0.2189999978840351246844875277020037174224853515625f,-1.0769999991655350157770953956060111522674560546875f,-0.1870000095963479935790019226260483264923095703125f,0.0227000079631807238911278545856475830078125f,2.0254000206947324613793171010911464691162109375f,-0.282000024795532766574979177676141262054443359375f,0.74959997811317435889577609486877918243408203125f,-0.0550000046193599434474208464962430298328399658203125f,-1.000499965965747861673662555404007434844970703125f,0.33569998972415948657044282299466431140899658203125f,-0.633699962805212013705613571801222860813140869140625f,-0.221000023722648819557434762828052043914794921875f,0.2539999943077564381610500277020037174224853515625f,0.8615999888643621762440716338460333645343780517578125f,-0.368000023409724263956377399154007434844970703125f,-0.4761999984562397258969212998636066913604736328125f,0.502999987311661289623998527531512081623077392578125f,-0.29800000724196440415880715590901672840118408203125f,-0.3938000051200389872718687911401502788066864013671875f,-0.22519999887347219580391310955747030675411224365234375f,0.84699997472763044470411841757595539093017578125f,-0.7888000057160855504889696021564304828643798828125f,0.10799997680634287888778999331407248973846435546875f,-0.8634999957084659172323881648480892181396484375f,-0.06170001299977290187825929024256765842437744140625f,-0.6300999698579314856061728278291411697864532470703125f,-0.117500010728836201678859652020037174224853515625f,1.10480001111030556870673535740934312343597412109375f,-0.3056999898731707077104147174395620822906494140625f,-0.258500024691224350448237601085565984249114990234375f,-0.87429999813139447351062472080229781568050384521484375f,-0.2954999978542327543351575513952411711215972900390625f,0.361000024318695267311341012828052043914794921875f,0.3782000025391578645894696819595992565155029296875f,-0.4665999939680098851368938994710333645343780517578125f,0.04450000132620335335786876385100185871124267578125f,0.1624999989569186897853114714962430298328399658203125f,-0.1370999994590877957278962639975361526012420654296875f,0.6340000084638592880992291611619293689727783203125f,0.08149999937415142170493709272705018520355224609375f,0.82320000574290708161839802414760924875736236572265625f,-0.4264999959468838142129243351519107818603515625f,-1.06000000715255726646546463598497211933135986328125f,0.0104999859333039236020113094127736985683441162109375f,0.08899999982118611541181962820701301097869873046875f,-0.678799998563528017569979056133888661861419677734375f,0.062999995768070193857823824146180413663387298583984375f,0.34200003850460092280627577565610408782958984375f,0.9765000093579292439471828402020037174224853515625f,-0.30139999200254674605048421653918921947479248046875f,0.409500023007392854879071819595992565155029296875f,-0.7411000068426130127363649080507457256317138671875f,0.7694000094145536650103167630732059478759765625f,-0.0866000141590834715543678612448275089263916015625f,0.13740001322925099458416298148222267627716064453125f,-0.995499993085861234476396930404007434844970703125f,0.76209999599456779151296359486877918243408203125f,0.42989999542236301266484588268212974071502685546875f,0.17659998800754550529745756648480892181396484375f,1.2427000216722490222309716045856475830078125f,0.6145000376105311801211428246460855007171630859375f,-0.2349999857135116609896385853062383830547332763671875f,-1.6247000066816810193159881237079389393329620361328125f,-0.33030002889484200156999804676161147654056549072265625f,0.153799987387657211002078838646411895751953125f,-0.9592999792814254078621161170303821563720703125f,-0.73410002257823947502402006648480892181396484375f,-1.493599991345405442189075984060764312744140625f,-0.531699954438209942964022047817707061767578125f,1.159500037312508169407010427676141262054443359375f,-0.527599986677616872299267924972809851169586181640625f,0.8797999836549161845056232778006233274936676025390625f,0.235999998651444886643702147921430878341197967529296875f,-1.294400028264522717336149071343243122100830078125f,0.011300014805794234007407794706523418426513671875f,0.210900003647804368256402085535228252410888671875f,1.5297000160694125270310905762016773223876953125f,0.52110001757144885914385668002068996429443359375f,-1.1707000397920612755342517630197107791900634765625f,0.861099987471103389680138207040727138519287109375f,0.413299944233894933631745516322553157806396484375f,-0.581100008332729345283951261080801486968994140625f,0.138700000020861580196651630103588104248046875f,-0.266200008812546684566768817603588104248046875f,0.248600014665723012541320713353343307971954345703125f,0.4157999983996152337795138009823858737945556640625f,-0.60690001083016387184443374280817806720733642578125f,-0.1034999907910820748924152212566696107387542724609375f,0.17319999523758866644129739142954349517822265625f,1.219999961107969710383258643560111522674560546875f,-0.4486000028192993482889505685307085514068603515625f,-1.277399985289573436375576420687139034271240234375f,-0.326099952900410272604858619160950183868408203125f,0.380299991381168212001284700818359851837158203125f,-0.1946000228166582246558391489088535308837890625f,-0.50609998583197601806915599809144623577594757080078125f,-0.75000000558793544769287109375f,-0.15000000685453418469705866300500929355621337890625f,-0.576799993437528524253821160527877509593963623046875f,0.169500006020069093892743694595992565155029296875f,0.745799983870983229650164503254927694797515869140625f,0.50649999281764024328111872819135896861553192138671875f,-0.94400000131130212821517488919198513031005859375f,0.83450001361966119617363801808096468448638916015625f,-0.4711999811708926788611506708548404276371002197265625f,0.61649999326467519455974297670763917267322540283203125f,-0.92970001020207992947774755521095357835292816162109375f,0.51849997536838066114484036006615497171878814697265625f,0.8545000045299531166165252216160297393798828125f,-0.78909997757673266960409819148480892181396484375f,0.20689999455064533862014286569319665431976318359375f,-0.41150004720687860526595613919198513031005859375f,0.4031000051140782858283273526467382907867431640625f,0.9356999668136243286653552786447107791900634765625f,0.07650000013783564423164307299884967505931854248046875f,0.650999982576817348700615184498019516468048095703125f,0.185699995181709542890757802524603903293609619140625f,0.6824999994039533390832730219699442386627197265625f,0.33090000752210624312965592253021895885467529296875f,0.89929999958425732575761912812595255672931671142578125f,0.399500031948090139621854177676141262054443359375f,-0.41959997692108164102364753489382565021514892578125f,-0.95789999476671194855725843808613717555999755859375f,0.1220000200271602608381726895458996295928955078125f,-0.14600001463294010051185978227294981479644775390625f,-0.57619997100085063035379562279558740556240081787109375f,-0.42750001035630713630553145776502788066864013671875f,-0.0843000055074691889700488900416530668735504150390625f,-0.10889999777674674863447989991982467472553253173828125f,-0.169000013306737084661790504469536244869232177734375f,-0.43909999970495705579054401823668740689754486083984375f,-0.1785999984979629306991455450770445168018341064453125f,0.453500007480382549829300842247903347015380859375f,0.462399991101026586193256662227213382720947265625f,-0.2388999835833909290983001483255065977573394775390625f,0.4494999863505366732852053246460855007171630859375f,0.20570000790357578779321556794457137584686279296875f,0.90829996543079649651275531141436658799648284912109375f,1.074000022768974105247252737171947956085205078125f,-0.3498000020951030908378243111656047403812408447265625f,-1.2147999753661451460828857307205908000469207763671875f,0.1762000345170491311108662557671777904033660888671875f,0.60119999894052733591109927147044800221920013427734375f,-0.18280001194477080161959747783839702606201171875f,-0.47329997344017016303041600622236728668212890625f,-0.524299996268748458305708481930196285247802734375f,0.47230001140832911232791957445442676544189453125f,-0.157299995017051941204044851474463939666748046875f,0.04790000907182712097664989414624869823455810546875f,-0.23099999570846563301529386080801486968994140625f,1.63229998548030863503299769945442676544189453125f,-0.03530000360012053306490997783839702606201171875f,-1.0416999902009962397642084397375583648681640625f,0.63430002875328028721924056299030780792236328125f,0.0422999732613558165894573903642594814300537109375f,-0.07279999763965605552584747783839702606201171875f,0.11030000658035277183444122783839702606201171875f,-0.628900008058547399514282005839049816131591796875f,-1.35569996647834756231532082892954349517822265625f,0.84390002295970845125339110381901264190673828125f,0.2766999821543691950864740647375583648681640625f,0.102199991780519638950863736681640148162841796875f,-0.1351999659419060861864636535756289958953857421875f,0.358600002670287576478358460008166730403900146484375f,-0.4064999998956917437453739694319665431976318359375f,1.13490002328753458726851022220216691493988037109375f,-0.1109000246584421756068650211091153323650360107421875f,1.6621000091075899263159953989088535308837890625f,-0.1233999863624575255016679875552654266357421875f,-0.69440000114440891820777324028313159942626953125f,0.10509999653100976502173580229282379150390625f,0.04460001715421668677663546986877918243408203125f,0.396600005888938511589003610424697399139404296875f,-0.726699977833032306762106600217521190643310546875f,0.073000046581029209846747107803821563720703125f,0.022599985951185175281352712772786617279052734375f,0.32870001410245919259978109039366245269775390625f,0.849100021386146153190566110424697399139404296875f,-0.73570004187822402030860757804475724697113037109375f,-1.19569998495578744268641457892954349517822265625f,-0.1974999600648885689224698580801486968994140625f,1.0851000105381007898586176452226936817169189453125f,-0.89520001064538945456661167554557323455810546875f,0.076700005102157575720411841757595539093017578125f,0.467099967384339009868199354968965053558349609375f,0.7779999645128850005448839510791003704071044921875f,0.4483999979853621198344626463949680328369140625f,0.30929999224543536229248275049030780792236328125f,-0.77229996946156076464973239126265980303287506103515625f,0.12730003137588585815365149755962193012237548828125f,0.06949999409914031645030263462103903293609619140625f,0.66329997276961838448272601453936658799648284912109375f,0.53849999094009337596844488871283829212188720703125f,-0.57890000373721051118991454131901264190673828125f,0.05689998952150343569655888131819665431976318359375f,0.5029000048995015959008014760911464691162109375f,0.15540000579357116095025048707611858844757080078125f,-0.4719999929815530403942602788447402417659759521484375f,0.3951999787569047128954480285756289958953857421875f,0.503399978315830498587502006557770073413848876953125f,-0.624599980199337156960837091901339590549468994140625f,-0.341400013720988937393485684879124164581298828125f,0.28789999834299084824351666611619293689727783203125f,-0.494699995654821422785829554413794539868831634521484375f,-0.975900000965595371127392354537732899188995361328125f,0.74389997274279640038230354548431932926177978515625f,-0.7579999940171837646829544610227458178997039794921875f,-1.0624000165820124408355695777572691440582275390625f,-0.202400002425908720482539138174615800380706787109375f,0.11469999785274265224899181703221984207630157470703125f,-0.20049999591708189683458840590901672840118408203125f,-0.414799967598915575450746473507024347782135009765625f,-0.0262999926403164607791040907613933086395263671875f,-0.841900009340047716932531329803168773651123046875f,-0.257299964022636995508719337522052228450775146484375f,-0.829799982984364081772810095571912825107574462890625f,-1.297800017905235137050112825818359851837158203125f,0.11839997712373762084325790056027472019195556640625f,0.170700000862777034882356019807048141956329345703125f,1.039500025391578930111791123636066913604736328125f,0.20070000998973824835047707892954349517822265625f,-0.6071000041157004201153313260874710977077484130859375f,0.0952999523401256709576045977883040904998779296875f,-0.514199993181227998917393051669932901859283447265625f,-0.16729998760372399857487835106439888477325439453125f,-1.110100029164552637439555837772786617279052734375f,0.48059998022913941184697250719182193279266357421875f,-0.4959999895617366316713514606817625463008880615234375f,-0.212200004968047029052513607894070446491241455078125f,0.0570000025928021347709773181122727692127227783203125f,0.2492999971628189204153613900416530668735504150390625f,-0.0164999899864195498366825631819665431976318359375f,-0.24200000900030094186377027654089033603668212890625f,-0.31569999885112043358503797207958996295928955078125f,0.36859999954104427644097086158581078052520751953125f,0.142199993717670114534712411114014685153961181640625f,-0.2160000014081597807802381794317625463008880615234375f,-0.876399983322620368397792844916693866252899169921875f,-0.45979997564554242472922851447947323322296142578125f,0.6631999857380985741173162750783376395702362060546875f,-0.39629998261928545133514489862136542797088623046875f,-0.0187999724864962303172433166764676570892333984375f,0.53359998389482499181468710958142764866352081298828125f,-0.9207999922156329120070950011722743511199951171875f,-0.23679998255968115472569479607045650482177734375f,0.8484999780133366531487126849242486059665679931640625f,-0.392699987101554626178767648525536060333251953125f,-0.6096000168561932497368616168387234210968017578125f,-0.26400000436604020848108120844699442386627197265625f,0.828800003182887934372047311626374721527099609375f,0.6782000087976456370597588829696178436279296875f,-0.29899999799579379811120816157199442386627197265625f,-0.639700030523538742954769986681640148162841796875f,-0.16779999842196680059913660443271510303020477294921875f,0.65140001573264594281198469616356305778026580810546875f,0.113199999633431414025608319207094609737396240234375f,-0.0322000051915647400591069526853971183300018310546875f,-0.1249999997392297057530186066287569701671600341796875f,0.72530001239180563743502716533839702606201171875f,0.3727999989807602165825528572895564138889312744140625f,-0.45920000733733179476558916576323099434375762939453125f,-0.84900001680851033825092599727213382720947265625f,-0.6936000038623806318582865060307085514068603515625f,0.294000019937753709342587171704508364200592041015625f,-0.498799988207220934555152780376374721527099609375f,-0.21239998178780095106077396849286742508411407470703125f,0.452999989695847038451148591775563545525074005126953125f,-0.14159999098777742432275772443972527980804443359375f,-0.381400019681453983366736792959272861480712890625f,-0.26630001130402092268667502139578573405742645263671875f,-0.2916000032067298519677933654747903347015380859375f,-1.0865000340938575362770279753021895885467529296875f,-0.35669997924864305094416749852825887501239776611328125f,-0.6475999792084099482281089876778423786163330078125f,-0.8236000121325250944437357247807085514068603515625f,0.442999986082315277968746158876456320285797119140625f,-0.43019996646344704771536271437071263790130615234375f,-0.60659998115301050347625277936458587646484375f,0.1248999982535836661412531611858867108821868896484375f,0.75849997395277046763339967583306133747100830078125f,0.822500013262033036198772606439888477325439453125f,0.031899990266561761842467603855766355991363525390625f,0.10569998935163038566997784073464572429656982421875f,1.012099981242418067495236755348742008209228515625f,-0.35399999311566343607182716368697583675384521484375f,0.183299996946752141724346074624918401241302490234375f,0.633299988341331498986619408242404460906982421875f,-0.492999965071678758476991788484156131744384765625f,-0.7282999710932376302707780268974602222442626953125f,-0.275500013649462971443426795303821563720703125f,0.78139998257756193567047375836409628391265869140625f,-0.65729996316582006699036355712451040744781494140625f,-0.56249998599290762513192021287977695465087890625f,-0.49589999023675890299500679248012602329254150390625f,0.2568999987602234025274583473219536244869232177734375f,-0.4495000015497208067927203956060111522674560546875f,-0.315899989696592220500548364725545980036258697509765625f,-0.06920000482276089659450235558324493467807769775390625f,0.483200016937404885908335927524603903293609619140625f,0.19689999332129948239611394456005655229091644287109375f,0.41209998295605199114532979365321807563304901123046875f,0.54250000543892351601016343920491635799407958984375f,0.018999998554587449461905634962022304534912109375f,0.56709997986406113223978309179074130952358245849609375f,0.1738999918162817959199628603528253734111785888671875f,0.50159996505975801284193948959000408649444580078125f,-0.35340001236498341796732347575016319751739501953125f,0.32099998497962900501079275272786617279052734375f,-0.073400002194941027511276843142695724964141845703125f,0.192799996000528384154648620096850208938121795654296875f,0.77199999819695952485432144385413266718387603759765625f,0.60529999704360903933775261975824832916259765625f,-0.162599994407594206080602816655300557613372802734375f,-0.059399999356269717054601642303168773651123046875f,0.370599998259544261003384235664270818233489990234375f,-0.70559998924434186307053096243180334568023681640625f,0.089200002568960268245490397021058015525341033935546875f,-0.12400001170486218970978598008514381945133209228515625f,-0.2501000015228986850246428730315528810024261474609375f,-0.8578000107526779771660585538484156131744384765625f,-0.53400001174211553234272287227213382720947265625f,0.402899991600215390263883818988688290119171142578125f,-0.0215999924778937923974808654747903347015380859375f,-0.2018000069975851573644831660203635692596435546875f,-0.229300000664591774324208017787896096706390380859375f,1.3542999992489814786722490680404007434844970703125f,0.8527999967455865970578088308684527873992919921875f,-0.7036999903723597515892151932348497211933135986328125f,-0.7957000031352041702348287799395620822906494140625f,-0.56680000282525977439718189998529851436614990234375f,0.3349000030592084176106482118484564125537872314453125f,-0.810500005602836637308428180404007434844970703125f,-0.164399988031387778164571500383317470550537109375f,0.74939998661577700289626591256819665431976318359375f,-0.54730002452135106949526743846945464611053466796875f,-0.0613999866008754224822041578590869903564453125f,0.66680001843422653795556698241853155195713043212890625f,0.39149999266862867131067105219699442386627197265625f,0.1043999823689460981768206693232059478759765625f,-0.40769999429136516422289560068747960031032562255859375f,0.2190999826431276886751220445148646831512451171875f,0.24019998233318329283747516456060111522674560546875f,-0.08659999840110543800619780085980892181396484375f,0.74640002467334287672429127269424498081207275390625f,0.29919999474585046161934087649569846689701080322265625f,-0.677800013397634071310449144220910966396331787109375f,-0.2073999928519130175885720746009610593318939208984375f,-0.20779999696910367301683209007023833692073822021484375f,0.013399999475479151822554513273644261062145233154296875f,0.1539000003099442182019629399292171001434326171875f,-0.087800017607211788828180942800827324390411376953125f,-0.406599997358024101146156681352294981479644775390625f,0.378299995493888818742789226234890520572662353515625f,0.14219998805522937601608646218664944171905517578125f,-0.368099993491172783333098550428985618054866790771484375f,-0.805999976634979020673199556767940521240234375f,-0.499199981856345953445952545735053718090057373046875f,0.4310999857947230839982921679620631039142608642578125f,-0.39840000662803642939024939551018178462982177734375f,0.46620000784397053195107218925841152667999267578125f,0.338199994175880924718313735866104252636432647705078125f,0.0562999996252357970405455489526502788066864013671875f,0.030700006096810117384166005649603903293609619140625f,0.151200011718273241712040544371120631694793701171875f,-0.90470001230686858395557692347210831940174102783203125f,-0.55030006201267323007186860195361077785491943359375f,0.368699987876415224974380180356092751026153564453125f,-0.20219998734742417045850970680476166307926177978515625f,-0.25739999601840946041875213268212974071502685546875f,-0.28199999633431449552745107212103903293609619140625f,0.905599995651841016552907603909261524677276611328125f,0.060400042748451454599489807151257991790771484375f,0.2945999931633469515190881793387234210968017578125f,-0.56119997088909190363636980691808275878429412841796875f,-0.06879998083114546858496396453119814395904541015625f,-0.53559997099041911639005775214172899723052978515625f,-0.9547000032916661549364789607352577149868011474609375f,-0.7343000683903699865595626761205494403839111328125f,-0.4278999981939790586693561635911464691162109375f,0.05259999843090756854735445813275873661041259765625f,0.571400021171569960642955265939235687255859375f,0.18660000007748589467837518895976245403289794921875f,-0.0306000008672475676263502464280463755130767822265625f,-0.4085999958157540135061935870908200740814208984375f,-0.275099968367815694847422491875477135181427001953125f,-0.4395999831423165193911017922800965607166290283203125f,-0.9127999922752376704693233477883040904998779296875f,0.153200000639259759083188328077085316181182861328125f,-0.14819998301863679390777406297274865210056304931640625f,-0.93540001078546020085724421733175404369831085205078125f,0.193899988053739258386798383071436546742916107177734375f,-0.18709999156147250065629350501694716513156890869140625f,-0.070300002966821040217837435193359851837158203125f,0.29769999842643724008439676254056394100189208984375f,-0.683999971918761939804198846104554831981658935546875f,-1.174800000977515690436803197371773421764373779296875f,0.092099999272823307450153151876293122768402099609375f,-0.1676999908268452799120495910756289958953857421875f,-0.615299994510412062709292513318359851837158203125f,0.15219999178051946131517979665659368038177490234375f,-0.51969996775239746256147554959170520305633544921875f,-0.007700001406668821601897434447892010211944580078125f,-0.71859997495412830659233804908581078052520751953125f,-0.196099991308152699076572389458306133747100830078125f,-0.064600005754828604409567560651339590549468994140625f,0.0119999990165233771932662421022541821002960205078125f,0.0417000028297305203039968546363525092601776123046875f,0.185300028336047972743472200818359851837158203125f,-0.1874000019788741866477721487171947956085205078125f,-0.547600009606778570514507009647786617279052734375f,0.14029998452663416941277318983338773250579833984375f,0.251999994397163362691571819595992565155029296875f,0.04380000422596930231833312063827179372310638427734375f,0.9334000259995463011364336125552654266357421875f,0.47160000737905516388082105549983680248260498046875f,-0.58650000693649051441980191157199442386627197265625f,-0.352800000098347776855689517105929553508758544921875f,-0.3224000057041644939914704082184471189975738525390625f,0.022999999940395365882750411401502788066864013671875f,-0.36419999645948397670736085274256765842437744140625f,-0.39080000264644620955323262023739516735076904296875f,0.09499999940395353892341745449812151491641998291015625f,0.00909999873638156486777006648480892181396484375f,0.11359999879598614569431447307579219341278076171875f,0.0240999958679080084067436473560519516468048095703125f,0.267400017178058391209560795687139034271240234375f,-0.1926000045776365965366494492627680301666259765625f,-0.545999997608363596413028062670491635799407958984375f,-0.52269998978376375209364823604119010269641876220703125f,-0.0465999760121109662946992102661170065402984619140625f,0.62889998525977131027531186191481538116931915283203125f,0.2342000131487846914524197927676141262054443359375f,0.046599996352195471871482368442229926586151123046875f,-0.2779000106364488686949698603712022304534912109375f,0.118699980127811244301483384333550930023193359375f,-0.019200022685527784460646216757595539093017578125f,-0.255899989230930824390952693647705018520355224609375f,0.072699996414780532205668350798077881336212158203125f,0.05540000042915327327364138909615576267242431640625f,0.01720000142902133877242931703221984207630157470703125f,-0.40599999690055843526437229229486547410488128662109375f,-0.2281999939709902225359883232158608734607696533203125f,0.2698999961674213299289704082184471189975738525390625f,0.23680000059008587953712776652537286281585693359375f,0.350200017797946738795644705533050000667572021484375f,0.1945999997571110640137703740037977695465087890625f,0.9807000388979911775777509319595992565155029296875f,-1.39430002696514154791884720907546579837799072265625f,-0.10659999702274769450838221018784679472446441650390625f,-0.38450000024586905578161122321034781634807586669921875f,-0.53890002318322682839379922370426356792449951171875f,0.53069997467398710710995146655477583408355712890625f,-0.59659998618215315246970931184478104114532470703125f,-1.04980004180669794777713832445442676544189453125f,-0.37419997932314874589110331726260483264923095703125f,-0.3580999991908668800277837362955324351787567138671875f,-0.7201000153064729403240562533028423786163330078125f,0.04479999203681916952746178139932453632354736328125f,0.82070001218765942230248811029014177620410919189453125f,0.36550004464387964020488652749918401241302490234375f,0.25759999213516737714968485306599177420139312744140625f,-0.5501999957442282873643080165493302047252655029296875f,-0.48300002534687536126511986367404460906982421875f,0.422599985206126849135443990235216915607452392578125f,0.3413999979257582584324381969054229557514190673828125f,0.9150000268220903620886019780300557613372802734375f,-0.243999983876943549176985470694489777088165283203125f,-0.6117999867320060847220020150416530668735504150390625f,-0.9722000247120856641913633211515843868255615234375f,0.659499962806702200168729177676141262054443359375f,0.1798999948263169113005233157309703528881072998046875f,0.0774999952316282048059292719699442386627197265625f,-0.43909998536258931789433290759916417300701141357421875f,-0.0928999997213482142655038842349313199520111083984375f,0.32269998095482588240656696143560111522674560546875f,0.1381000004947185999526482191868126392364501953125f,-0.125600001277029360569059690533322282135486602783203125f,-0.407200022625923363506217356189154088497161865234375f,0.1344000039756299003101958078332245349884033203125f,-0.507800003898143614833315950818359851837158203125f,-0.884200020599365643647615797817707061767578125f,-0.02070000164508811746344463244895450770854949951171875f,0.34219998138695972844658399480977095663547515869140625f,0.4545999978572126121179053370724432170391082763671875f,0.03869999789744615892317369798547588288784027099609375f,-0.085399998939037324863221556370262987911701202392578125f,0.264199996198713815687142414390109479427337646484375f,-0.4198999967634675645200559301883913576602935791015625f,0.76249997422099136112905171103193424642086029052734375f,0.90730000410675959443551619187928736209869384765625f,0.46959997557997734674017920042388141155242919921875f,-0.002300006565451651052711667944095097482204437255859375f,0.6226000024914741803883089232840575277805328369140625f,0.570600006753206212550821874174289405345916748046875f,-0.1683000025719404157964476098641171120107173919677734375f,-0.74000001668930082132646930404007434844970703125f,-0.591000013887882147400887333787977695465087890625f,0.200100009867548944697546176030300557613372802734375f,-0.380200006398558887354965918348170816898345947265625f,-0.4242000168740751586682335982914082705974578857421875f,-0.0917999992862343905386524056666530668735504150390625f,-1.169599983775615470449338317848742008209228515625f,-0.631999981582164860327566202613525092601776123046875f,0.7447999823510647043889321139431558549404144287109375f,0.8450000005960465188081798260100185871124267578125f,0.38459999152421975310289781191386282444000244140625f,-0.5738999912202358277824032484204508364200592041015625f,-0.4878999870181086606635290081612765789031982421875f,-0.429599981391429963650807621888816356658935546875f,0.2697000009819865429250285160378552973270416259765625f,0.22849998974800111994909457280300557613372802734375f,-0.00840001914501176116800706950016319751739501953125f,-0.32879999200701715000860758664202876389026641845703125f,0.190300006729364368229795445586205460131168365478515625f,-0.16980001308470960008634165205876342952251434326171875f,-0.435599998632073381887863661177107132971286773681640625f,-0.272299981904029930745991805451922118663787841796875f,0.23320004307031627632795789395458996295928955078125f,0.6403999888062477108263692571199499070644378662109375f,-0.6387999888777731172240237356163561344146728515625f,-0.43599999392032628975357511080801486968994140625f,0.317799989816546446519396340590901672840118408203125f,0.0277999812483784580763312987983226776123046875f,-0.3396000392079354668339874478988349437713623046875f,-0.53909998968392613960531889460980892181396484375f,0.4648999969497322748424039673409424722194671630859375f,0.45140000843107686012700696664978750050067901611328125f,-0.008299998585879819756883080117404460906982421875f,0.361100005948543412159779109060764312744140625f,-0.03120001231431945853955767233856022357940673828125f,-0.539099995793402175792152775102294981479644775390625f,-0.72879998589754091398162927362136542797088623046875f,-0.357799980950356033559955903911031782627105712890625f,0.61069998234808442472143497070646844804286956787109375f,0.68380002106428161567919232766143977642059326171875f,-0.2813999980747698970162673504091799259185791015625f,-0.4974999826401471647585594837437383830547332763671875f,0.1088000021770596514869566817651502788066864013671875f,-0.47840000387132164405556977726519107818603515625f,0.38849998766183890808179057785309851169586181640625f,-0.7834000030145047421825665878714062273502349853515625f,-0.4562000319838528383797893184237182140350341796875f,0.42049999092519330101680452571599744260311126708984375f,-0.79250001028180083739727024294552393257617950439453125f,-0.81490005554854949565424249158240854740142822265625f,0.04240000153183931530520567321218550205230712890625f,-0.654399979686737243156358090345747768878936767578125f,0.126399979150295660446090550976805388927459716796875f,0.125899992024898210729588754475116729736328125f,0.77500000026077009440683696084306575357913970947265625f,0.38780004979372062479114902089349925518035888671875f,0.391199961501360604643195983953773975372314453125f,0.699999970570206908604404816287569701671600341796875f,1.583599999094008836664215778000652790069580078125f,0.24239999914765331112675994518212974071502685546875f,-0.10189999432712804061651468146010302007198333740234375f,-0.35589998576641068694925706950016319751739501953125f,0.531099981454759895171946482150815427303314208984375f,0.0608000004671513938347970906761474907398223876953125f,-0.042799994566291577058336770278401672840118408203125f,-0.731999994099139517089724904508329927921295166015625f,0.10009999947398906527951112366281449794769287109375f,0.351699977535009367102247779257595539093017578125f,0.207199995803832859309068226139061152935028076171875f,-0.26779998862445364427031790910405106842517852783203125f,0.22339999619722394896825790056027472019195556640625f,0.6643999901175501321404226473532617092132568359375f,-0.7919000109419223054629810576443560421466827392578125f,-1.1863000431180008575893225497566163539886474609375f,-0.394499991007149219957028662975062616169452667236328125f,0.67209998113065971825363931202446110546588897705078125f,0.5501000111714002827056901878677308559417724609375f,0.4187999821722510063182198791764676570892333984375f,-0.15619999502897263798928406686172820627689361572265625f,-0.55180000550746921561540148104541003704071044921875f,-0.01250000014901164746561335050500929355621337890625f,0.297399981229007515093343272383208386600017547607421875f,1.160599983656406042342723594629205763339996337890625f,0.13810000153779977694057379267178475856781005859375f,0.65720000083297491766387565803597681224346160888671875f,-0.19579999687224614035585545934736728668212890625f,-0.39140001603066931767926917018485255539417266845703125f,-0.1870000030398369095241406512286630459129810333251953125f,-0.076899994364380830358385310319135896861553192138671875f,-0.20960000512152909823981872250442393124103546142578125f,-0.1357000011980533660249648164608515799045562744140625f,-0.574700008432567077676367262029089033603668212890625f,-0.347400001718103423886674363529891707003116607666015625f,0.28140000759288674725411283361609093844890594482421875f,-0.1045000027418137023005328956060111522674560546875f,0.036400015807152019675640985951758921146392822265625f,0.29810000813156367627243525930680334568023681640625f,0.3191999958634372802634970867075026035308837890625f,-0.14260002782344827210181392729282379150390625f,-0.6937999937579035769630309005151502788066864013671875f,0.4020999983787536535828621708787977695465087890625f,0.299200001525878889196974341757595539093017578125f,-0.1365999975442886427146049754810519516468048095703125f,-0.784800052237510925579044851474463939666748046875f,-0.155600011968612506052522803656756877899169921875f,0.741200021925568552916274711606092751026153564453125f,0.2222000282883644928233479731716215610504150390625f,-0.0854000057935717649115758831612765789031982421875f,-0.4906000194564461569513014183030463755130767822265625f,0.5646000201344492097632610239088535308837890625f,0.2343999907135962956772345933131873607635498046875f,-0.594100014121830444224769962602294981479644775390625f,0.347800014626979958620722754858434200286865234375f,-0.239000035881996808484473149292171001434326171875f,-0.830200017984211502408697924693115055561065673828125f,0.772200024712085930644889231189154088497161865234375f,0.51260000606775246723145755822770297527313232421875f,-0.426800019104778771694697070415713824331760406494140625f,-0.3263000145077707969676339416764676570892333984375f,-0.21740000019073502102173733874224126338958740234375f,0.2624000093922018717051969360909424722194671630859375f,-0.31430002160072323391659665503539144992828369140625f,-0.0336000071406363787218651850707828998565673828125f,0.31230000637918708950024893056252039968967437744140625f,-0.11970000593662266652472681016661226749420166015625f,-0.093200001943111399071995037957094609737396240234375f,0.07400000429153441583451922269887290894985198974609375f,0.1107000114798548651151577359996736049652099609375f,-0.0274000119626520710625072752009145915508270263671875f,0.5760999890729781025555666928994469344615936279296875f,-0.03969999926835299230987175178597681224346160888671875f,0.1795000066533685156855426612310111522674560546875f,0.337199988148361440298828028971911408007144927978515625f,-0.0103999989204108800322501338087022304534912109375f,0.088400001356750745884482967085205018520355224609375f,0.0806999979011713275411921131308190524578094482421875f,-0.60759997186958825121649852007976733148097991943359375f,-0.372799985755979335078080794119159691035747528076171875f,-0.141400010442733758964095613919198513031005859375f,0.785199990379810142115957205533050000667572021484375f,1.296800040376186569091032652067951858043670654296875f,0.5712999835133558690358768217265605926513671875f,-0.7203000082194801478863155352883040904998779296875f,-1.36590005252361379461945034563541412353515625f,-0.16859999656081203767143961158581078052520751953125f,0.4048999921813607461018591493484564125537872314453125f,0.2197000121951102613593320711515843868255615234375f,-0.52219997136592866127102752216160297393798828125f,-0.847800005090236563631833632825873792171478271484375f,-0.593400018548965846321152639575302600860595703125f,0.36309999527931235974165247171185910701751708984375f,-0.65629998612105833188934411737136542797088623046875f,-1.19230004746913920143924769945442676544189453125f,0.07220001040697088257047653314657509326934814453125f,-0.7142999696329237480085794231854379177093505859375f,-1.638300024998187609526212327182292938232421875f,0.794299998950958041632475215010344982147216796875f,-0.651100003936886384536819605273194611072540283203125f,-1.267100053811073934184605604968965053558349609375f,-0.229899986034631798048621931229718029499053955078125f,0.1485000028610230504710898458142764866352081298828125f,-0.5379000178635120477110831416212022304534912109375f,0.1240999875605108826448486070148646831512451171875f,0.48919996535331045084404877343331463634967803955078125f,1.07919999884366912823452366865240037441253662109375f,-0.271199979978800076452216671896167099475860595703125f,-0.45219999371767050089232498066849075257778167724609375f,-0.41660000231266014480979720246978104114532470703125f,0.069999996125698071836040980997495353221893310546875f,0.132299993415176897482155027319095097482204437255859375f,0.325500006794929408471261922386474907398223876953125f,0.380700002092122513630556568386964499950408935546875f,0.4613000034064054233340357313863933086395263671875f,-0.08089999440908446359799199854023754596710205078125f,-0.12239999065399143063359588268212974071502685546875f,0.02220000638365748901748020216473378241062164306640625f,-0.20629999796748155205250441213138401508331298828125f,-0.0951999920189379178481203780393116176128387451171875f,0.1993000010743737338003711556666530668735504150390625f,-0.451700014676153838166783316410146653652191162109375f,-0.099999986998737078192078797655995003879070281982421875f,0.5968000062339007261158485562191344797611236572265625f,-0.0874000076413157245269758277572691440582275390625f,0.18820001147985454537092664395458996295928955078125f,0.275900002753734607807700740522705018520355224609375f,-0.3276999984264372667297493535443209111690521240234375f,0.004400014108419281200212935800664126873016357421875f,0.44619999301731583241092948810546658933162689208984375f,0.3830999982595442165944632506580092012882232666015625f,0.085599992150068526797213053214363753795623779296875f,-0.39279999011456967483724156409152783453464508056640625f,1.1351999850153919879858221975155174732208251953125f,0.697799993467331347574145183898508548736572265625f,-0.6203999766990542230615801599924452602863311767578125f,-0.7051000159025193614326099122990854084491729736328125f,-0.475400001770257762956362057593651115894317626953125f,0.41030001194477083714673426584340631961822509765625f,0.399499997228383862957201699828146956861019134521484375f,0.28200000084191578775261177725042216479778289794921875f,-0.2108999929934739780978958378909737803041934967041015625f,-0.64589998755455013679238618351519107818603515625f,-0.0609999733567239132980830618180334568023681640625f,0.661899983859062235325154688325710594654083251953125f,0.0590999975442887404142311424948275089263916015625f,0.03439998683929434974970718030817806720733642578125f,-0.1616000053673982694846245067310519516468048095703125f,0.165299987804889525477847200818359851837158203125f,0.32879999871253939858206649660132825374603271484375f,0.0779999989718198616373001641477458178997039794921875f,-0.45769999470114708373102985206060111522674560546875f,-0.495600007498264272243204686674289405345916748046875f,-0.0084000020086765214699653370189480483531951904296875f,-0.056000000104308167436784060555510222911834716796875f,0.1132000095427037056339258924708701670169830322265625f,0.17779999980032445972000232359278015792369842529296875f,-0.95649999773502347721887417719699442386627197265625f,-0.380100006365776010852641775272786617279052734375f,0.176999977558851551151519743143580853939056396484375f,0.350299998049437899538816054700873792171478271484375f,0.633100018227100491685632732696831226348876953125f,-0.00960000471174726754952644114382565021514892578125f,0.2144000043481586736948685256720636971294879913330078125f,0.5284000207841399454622433040640316903591156005859375f,-0.2242000021778045582010463476763106882572174072265625f,0.7228999623566869647817156874225474894046783447265625f,1.528900006158649294008000651956535875797271728515625f,-0.477099990332126022707370793796144425868988037109375f,-0.3506000079080460185565470965229906141757965087890625f,-0.4978000369787221046635750099085271358489990234375f,-0.3375999873667956396872114055440761148929595947265625f,0.03489999918490649388758129134657792747020721435546875f,0.34009999690353875934079042053781449794769287109375f,0.2229999852627517764602771421778015792369842529296875f,0.13530000013560050664551681620650924742221832275390625f,-0.14189998855292795809646122506819665431976318359375f,-0.20610000773668257778581391903571784496307373046875f,0.6836999885469674520521721206023357808589935302734375f,-0.0626999680280686533251355285756289958953857421875f,0.2332999863296747466989700114936567842960357666015625f,0.05649999177455899967981167719699442386627197265625f,0.7889000151365994639718337566591799259185791015625f,-0.155000001434236478470296560772112570703029632568359375f,0.016600000952929239161903751664794981479644775390625f,-0.374200000724941528318368000327609479427337646484375f,-0.56989998126625973640102529316209256649017333984375f,-0.85589998286068436073037446476519107818603515625f,-0.7443000502109526905769598670303821563720703125f,-0.0457999822318555516176274977624416351318359375f,-0.81029998240321876590286365171778015792369842529296875f,-1.30830002738237372028606841922737658023834228515625f,-0.506999975472688380051522472058422863483428955078125f,-0.4857000100269910358719016585382632911205291748046875f,-0.9525000464916235642931496840901672840118408203125f,0.5023999843955040400800271527259610593318939208984375f,-0.3876999922797084163761383024393580853939056396484375f,-0.1097000090658664106513242586515843868255615234375f,-0.28819999367296678105532237168517895042896270751953125f,-0.16239999467730525939401786672533489763736724853515625f,-0.22160000760257236862571517121978104114532470703125f,0.20799998876452452378771340590901672840118408203125f,-0.0257999998524784945885812703636474907398223876953125f,-0.842099990630149619619260192848742008209228515625f,-0.47279998929500610671539106988348066806793212890625f,0.576599985176324825175697696977294981479644775390625f,-0.8603000119447710147824182058684527873992919921875f,-0.6242000134468081995464672218076884746551513671875f,0.306199998120963545744643852231092751026153564453125f,-0.7400999890804289549350869492627680301666259765625f,-0.685200006175040954303767648525536060333251953125f,0.0827999948084354731037137753446586430072784423828125f,0.419399993246793723056242697566631250083446502685546875f,0.14159998536258944223931166561669670045375823974609375f,-0.3872999931544065521649145011906512081623077392578125f,-0.872300010514259316352081441436894237995147705078125f,-0.4691999976515770498508572927676141262054443359375f,0.5940000004172325542839416812057606875896453857421875f,0.10530001030564317687776565435342490673065185546875f,0.145000008046627382185533861047588288784027099609375f,-0.0538000029966234638578725935076363384723663330078125f,-0.336699992138147496945066450280137360095977783203125f,-0.391599999630451112153650683467276394367218017578125f,-0.016099999280273891155701448951731435954570770263671875f,-0.52790004036426552147531765513122081756591796875f,0.0262000027775766142212887643836438655853271484375f,0.768500019177794424507510484545491635799407958984375f,-0.153400013184547656663880843552760779857635498046875f,0.2390000159144403113486987422220408916473388671875f,0.435400007618963746924833912999019958078861236572265625f,0.04419999787509443223143534851260483264923095703125f,-0.10220001659095279133993017239845357835292816162109375f,-0.285699999894201750461064648334286175668239593505859375f,0.3913000159978867742438524146564304828643798828125f,0.2055999958753584877513276296667754650115966796875f,-0.3904000125735997972498125818674452602863311767578125f,-0.77070003591775904094873794747400097548961639404296875f,-0.2566000011950728687537548466934822499752044677734375f,0.72550001856684687151499701940338127315044403076171875f,-0.182500009909272176145123012247495353221893310546875f,-0.0088000048220158166856208481476642191410064697265625f,0.1557999997407197856347238484886474907398223876953125f,-0.17510000532269487383274508829344995319843292236328125f,-0.0197999940484760994507951181731186807155609130859375f,0.204300004613399532882311859793844632804393768310546875f,-0.935399972712993577061979522113688290119171142578125f,-0.3554000012114642004235065542161464691162109375f,0.08420000286698314795330588822253048419952392578125f,-0.06930000137239689106394280315726064145565032958984375f,0.71429999209642414115251085604541003704071044921875f,0.327899998492002797689792714663781225681304931640625f,-0.89320001300722318138269884002511389553546905517578125f,-0.51440005896091545167791991843841969966888427734375f,0.57159998487830154800803938996978104114532470703125f,-0.8576999679908159368579845249769277870655059814453125f,-0.5541000067830079700570422573946416378021240234375f,0.145799998753517734417783913158928044140338897705078125f,0.07289999977722760871046148167806677520275115966796875f,-0.279899989182502029194665738032199442386627197265625f,0.038400002157688017945247338502667844295501708984375f,-0.24790000285059210494154058324056677520275115966796875f,0.46449999573826783461072409409098327159881591796875f,-0.2046999946117400437373134991503320634365081787109375f,-0.514199966992438195489967256435193121433258056640625f,-0.91139999956488537691257079131901264190673828125f,0.4293999768555167850792031458695419132709503173828125f,-0.14840000007152553962441743351519107818603515625f,0.13519999037981023093379917554557323455810546875f,-0.57309997933506995337893386022187769412994384765625f,0.243499985277652708504092515795491635799407958984375f,0.833500028491020117371590458787977695465087890625f,-0.63989998722672414288581421715207397937774658203125f,-0.25490000190436834515139707946218550205230712890625f,-0.301300017935037800498321303166449069976806640625f,0.212900003297626927434293975238688290119171142578125f,-0.74259998181611308609717525541782379150390625f,-1.1666000183314082505603437311947345733642578125f,-0.04379998902678483574391066213138401508331298828125f,-0.676899977935850660770711328950710594654083251953125f,-1.423700031983852198891327134333550930023193359375f,-0.27209998999685069964726835678447969257831573486328125f,-0.069300005991757007706155491177923977375030517578125f,0.61350002212077381358312777592800557613372802734375f,-0.19019998419582850868181367331999354064464569091796875f,-0.51289999413341280654066167699056677520275115966796875f,-0.63010001836121087848141542053781449794769287109375f,-0.13930000062733871590836542964098043739795684814453125f,-0.0421000005766749563207440587575547397136688232421875f,0.09189999991506336840529911569319665431976318359375f,0.291499977320432979155384600744582712650299072265625f,-0.349400008893012981037173858567257411777973175048828125f,1.040999987363814938134964904747903347015380859375f,0.8458000106930736450294716632924973964691162109375f,-0.308499979168176619026553453295491635799407958984375f,-0.7865000081658362063308231881819665431976318359375f,-0.2239999998211867904274186003021895885467529296875f,0.6610999816969036491087763351970352232456207275390625f,0.9266000121474267103849342674948275089263916015625f,0.41879999305009807386568354559130966663360595703125f,-0.6231000061944127477175925378105603158473968505859375f,-0.7045999913752079013562479303800500929355621337890625f,-0.37659998566061270164340157862170599400997161865234375f,0.4995999913007020953870096491300500929355621337890625f,1.2580000053048134844857486314140260219573974609375f,0.89880000675916704722112626768648624420166015625f,-0.5450999940350651495890588194015435874462127685546875f,-0.400400002068281135603200482364627532660961151123046875f,-0.15699999354034666065871306273038499057292938232421875f,0.345399997970461836160893653868697583675384521484375f,0.22480000798106214432436900096945464611053466796875f,0.275600014054775410698994164704345166683197021484375f,0.0819999974519014518392623358522541821002960205078125f,-0.2032999973192810561073429198586381971836090087890625f,-0.21280000307857982466686053157900460064411163330078125f,0.002499999180436140999717054000939242541790008544921875f,-0.0947000095129013619299485071678645908832550048828125f,0.044800001424551094686421492951922118663787841796875f,0.1643000050976872461472311215402442030608654022216796875f,0.106900002709031216596713420585729181766510009765625f,-0.0108000042855740385761009747511707246303558349609375f,-0.11520000222623349994677255381247960031032562255859375f,-0.53160000439882271194846907746978104114532470703125f,-0.43420000778436662614012675476260483264923095703125f,0.18489999981820581798075409096782095730304718017578125f,-0.02840001020431515144082368351519107818603515625f,-0.1776000227570531109222429222427308559417724609375f,-0.2834999934360384887810369036742486059665679931640625f,-0.36979996819496108173552784137427806854248046875f,-0.3105999812722206598891716566868126392364501953125f,0.2328999809086322869688956416212022304534912109375f,0.4120999998688696930315700228675268590450286865234375f,0.242999997705221293387012337916530668735504150390625f,-0.27449999588727946875366114909411408007144927978515625f,0.8158999856173989595475859459838829934597015380859375f,-0.044200010019540769690138404257595539093017578125f,-0.588300002050399228892274550162255764007568359375f,0.8141999832347035503943288858863525092601776123046875f,0.1950000247359273686242886469699442386627197265625f,-0.389999963194131993304836214520037174224853515625f,-0.8824000099509949901488425894058309495449066162109375f,-0.6332000678062446041849398170597851276397705078125f,0.14679999716281866284361967700533568859100341796875f,0.91329997899085302304200695289182476699352264404296875f,0.52370001589059800295444802031852304935455322265625f,0.031499995667487457051691990272956900298595428466796875f,-0.08650000054761763923494299888261593878269195556640625f,0.550299990207701927857897317153401672840118408203125f,-0.47679999120235461163019863306544721126556396484375f,0.7631000104412433149292382950079627335071563720703125f,-0.0832999528765672181407353491522371768951416015625f,-0.095899996327608871826697622964275069534778594970703125f,-0.067900000261515380817201048557762987911701202392578125f,0.2736999983258545654507543076761066913604736328125f,-0.2977999939143660679263803103822283446788787841796875f,0.4670000115334984602810664000571705400943756103515625f,0.4750000362098223405382668715901672840118408203125f,0.207999994315206937489648453265544958412647247314453125f,-0.1857999915078283326153041343786753714084625244140625f,-0.07099999768286924251015079789794981479644775390625f,-0.1879999942034482618424817701452411711215972900390625f,-0.08140000023543836338291157517232932150363922119140625f,-0.05059999922811986294846064993180334568023681640625f,0.14119999867975685514664974107290618121623992919921875f,0.26089999742656944103913474464206956326961517333984375f,0.31090001355707652663795670378021895885467529296875f,0.314099995458125835767759781447239220142364501953125f,-0.21659999735802415443686186335980892181396484375f,0.2082000025391579356437432579696178436279296875f,0.286200002479553017309399365331046283245086669921875f,-0.80469997542649529531644247981603257358074188232421875f,-1.18550000321865045549429851234890520572662353515625f,0.27029998869895877078306511975824832916259765625f,0.6074000150546428056230752190458588302135467529296875f,1.3162000421166428765218370244838297367095947265625f,-0.0659000008910892720592755722464062273502349853515625f,-0.0022999975502491221135414889431558549404144287109375f,-0.40750000067055225372314453125f,-0.190299996875971377807701401252415962517261505126953125f,-0.7034000060319900438088325245189480483531951904296875f,-1.0083000199317933009979242342524230480194091796875f,-0.4652000014066697275438855285756289958953857421875f,0.6980000018775462944375931328977458178997039794921875f,0.22679999515116211483700681128539144992828369140625f,0.1685999976038932146593651850707828998565673828125f,-0.082199997554719461678729430786916054785251617431640625f,-0.2633000169515611332826665602624416351318359375f,0.2904000266551971520812003291212022304534912109375f,0.765000008046627044677734375f,-0.3202999943614006639336366788484156131744384765625f,-0.1819999845623971879149394226260483264923095703125f,0.3406999980315565945687694693333469331264495849609375f,-0.1320999799013140574288627249188721179962158203125f,-0.1471999767303469042190045001916587352752685546875f,0.1345999952495098295202069493825547397136688232421875f,-0.3490000022053718620185236432007513940334320068359375f,-0.241599998289346640234498408972285687923431396484375f,0.2014000020235776899912849557949812151491641998291015625f,-1.2293000180244444408117487910203635692596435546875f,-0.8562000194668772934392109164036810398101806640625f,0.5534999963045119653060055497917346656322479248046875f,1.379899991846084361668545170687139034271240234375f,0.9673999992966653138637411757372319698333740234375f,-0.63229998652338981202092327293939888477325439453125f,-0.9813000124216078035033206106163561344146728515625f,-0.67080001158714264164473206619732081890106201171875f,0.4140000035464763694648127057007513940334320068359375f,-0.489999998807906678166546043939888477325439453125f,0.1056000209093088670897486736066639423370361328125f,0.822599981108307787280864431522786617279052734375f,-0.2425999982073905325563600854366086423397064208984375f,-0.36820000797808172254832470571273006498813629150390625f,-0.13399999736249446069535906644887290894985198974609375f,-0.76459998541474316624544371734373271465301513671875f,-0.769200009274482621179913621745072305202484130859375f,0.0542999903455376742300586556666530668735504150390625f,-0.3644999893307687699461894226260483264923095703125f,0.1152000374674795324381193495355546474456787109375f,0.6626999926522374462223297086893580853939056396484375f,-0.3203000021100042005173236248083412647247314453125f,-0.45640001997947710066227955394424498081207275390625f,-0.08720000340342526357062524766661226749420166015625f,-0.3137000045657156732659132103435695171356201171875f,0.01379999454021429983185953460633754730224609375f,-0.13159998726248733902366438996978104114532470703125f,-0.7899000021651385594623206998221576213836669921875f,-0.2535000335574153496054350398480892181396484375f,-0.09629999603033068122925897114328108727931976318359375f,0.1952999914556742544480272272267029620707035064697265625f,0.0729000021427868016843376608449034392833709716796875f,-0.8141999855816361986171614262275397777557373046875f,0.5794999773725868497109559029922820627689361572265625f,1.0735000103116032477146291057579219341278076171875f,0.07419998498559010613462305627763271331787109375f,0.7975999737322332361344479068065993487834930419921875f,1.5436000128030773481668802560307085514068603515625f,0.196599968189001828022810514084994792938232421875f,0.8310999662742022398020935725071467459201812744140625f,1.4242999891161911563131070579402148723602294921875f,0.1295999930888414031215916111250407993793487548828125f,0.06399999825656410568086585044511593878269195556640625f,0.44720000839531426350959009141661226749420166015625f,0.42090000796914051051089700195007026195526123046875f,-0.772900002068281022360451970598660409450531005859375f,0.035499977588653308657740126363933086395263671875f,-0.49049999353289575054759552585892379283905029296875f,0.74709998310506353647042487864382565021514892578125f,-0.22689998445510894953258684836328029632568359375f,0.29800000217556910087068899883888661861419677734375f,-0.89729997009485995551614223586511798202991485595703125f,-0.40649999207258158406830261810682713985443115234375f,-0.68759996285438607088735807337798178195953369140625f,0.34299999226629740434191262465901672840118408203125f,0.93980000514984141091190394945442676544189453125f,0.472099979193509045050092254314222373068332672119140625f,-0.27550000742822866595105324449832551181316375732421875f,-0.147500023730099627528034034185111522674560546875f,0.7235999843418594590360726215294562280178070068359375f,-0.633699971373379522532331975526176393032073974609375f,-0.17209999673962528277115779928863048553466796875f,-0.5811999996483319108619980397634208202362060546875f,0.3179999816864731432275448241853155195713043212890625f,0.7895999757289882126087832148186862468719482421875f,0.581599995011090609153825425892136991024017333984375f,0.0100999994739889853434533506515435874462127685546875f,0.3769000148534775718189848703332245349884033203125f,0.227900001472234325916588204563595354557037353515625f,-0.2814000075370073350455868421704508364200592041015625f,1.0798000227332114064893175964243710041046142578125f,0.8286000288963322191193583421409130096435546875f,-0.3899999976530670320329363676137290894985198974609375f,-0.731600025260448649788713737507350742816925048828125f,-0.08139998332261999536285657086409628391265869140625f,0.8386000152990222833437883309670723974704742431640625f,-0.30109999505206930248135677175014279782772064208984375f,-0.23100000174343560122025564851355738937854766845703125f,0.0449999976530671419450158055042265914380550384521484375f,0.1478999879121776217516526230610907077789306640625f,-0.2576000248432155359523676452226936817169189453125f,-0.5673999802231788347484098267159424722194671630859375f,-0.0127999824404714246384173748083412647247314453125f,-0.0831999894261361561120793339796364307403564453125f,0.0456999906554818036141796255833469331264495849609375f,0.65450000996887691684378296486102044582366943359375f,0.40700000952184234304098708889796398580074310302734375f,-0.301399995541572540691532822165754623711109161376953125f,-0.1021999901413916944648008211515843868255615234375f,-0.0207999820828437265163302072323858737945556640625f,0.2316999985828995800574148233863525092601776123046875f,0.57799999383091904547882222686894237995147705078125f,0.508600007438659407199565976043231785297393798828125f,-0.0822999957621097788518227389431558549404144287109375f,0.2493000023782254714888040325604379177093505859375f,0.0605999885737900090276752962381578981876373291015625f,-0.36029999763965603332138698533526621758937835693359375f,1.269000007867813195616690791212022304534912109375f,0.9222000086188320011615360272116959095001220703125f,-0.558299995195865594865836101234890520572662353515625f,0.7893999835610390203299857603269629180431365966796875f,0.567399983650445971505860143224708735942840576171875f,-0.4119999924227595311521099574747495353221893310546875f,0.73459998779892909936961586936376988887786865234375f,0.667200004148483127863755726139061152935028076171875f,-0.1124999930337071685215732941287569701671600341796875f,0.67220000385045997060018407864845357835292816162109375f,0.4786000024467707003594796333345584571361541748046875f,-0.340299999129772157147044708835892379283905029296875f,-0.58639997736215576651375158689916133880615234375f,-0.42839997891187664436074555851519107818603515625f,0.371099990190565609537998170708306133747100830078125f,0.97359998091459232227862230502068996429443359375f,0.6868000156402587208503973670303821563720703125f,-0.518300009202956601939149550162255764007568359375f,0.74450001019239397948012992856092751026153564453125f,0.05650002753734639782123849727213382720947265625f,-0.589699989694356663250118799624033272266387939453125f,-0.2856999958336354783483557184808887541294097900390625f,0.2143000059545041580122415325604379177093505859375f,0.1248999998927115273605892298292019404470920562744140625f,-0.63219997078850898031276983601856045424938201904296875f,-0.6709999997876578969879801661591045558452606201171875f,0.6349999712407594643082120455801486968994140625f,-0.5324999972060322495082118621212430298328399658203125f,-0.9057000132679942083768764859996736049652099609375f,0.1893999892234798920753746642731130123138427734375f,0.8152999929830431202759655207046307623386383056640625f,1.2841000324130060761262939195148646831512451171875f,-0.11250000808388005424376387964002788066864013671875f,0.73360000851899354046281587216071784496307373046875f,0.48680004745423854917163453137618489563465118408203125f,-0.02540001205205921763763399212621152400970458984375f,0.7930999915912746445201264577917754650115966796875f,1.3171000326514246125952922739088535308837890625f,-0.3900999954879280640085426057339645922183990478515625f,0.7515999652460221280847463276586495339870452880859375f,0.5635999941766254295316684874705970287322998046875f,0.1017000038355586610805403324775397777557373046875f,-0.64529996187686966546692701740539632737636566162109375f,-1.2248999887168399158809961591032333672046661376953125f,-0.5196999904394150604503010981716215610504150390625f,0.5970000156685706382830858274246565997600555419921875f,-0.3533999598383896767472833744250237941741943359375f,-0.3284999767094853861948422490968368947505950927734375f,-0.012099997261166617334993134136311709880828857421875f,-0.8104999907761811783757366356439888477325439453125f,-0.5465999851763247985303451059735380113124847412109375f,0.7464999740049245158246549181058071553707122802734375f,0.3905000099241728861443334608338773250579833984375f,-0.26589999474436021298373589161201380193233489990234375f,0.094000000193715127494442640454508364200592041015625f,-0.52840000636726625060646256315521895885467529296875f,0.27999997794628139757833196199499070644378662109375f,0.4937000022187827408259863659623079001903533935546875f,1.3513000240445141031386810936965048313140869140625f,0.16389998391866722471377215697430074214935302734375f,-0.210200001108646450376227221568115055561065673828125f,-0.931400009846687026993095059879124164581298828125f,-0.3250000038743028341059471131302416324615478515625f,0.76329997716546049701946685672737658023834228515625f,0.431200000095367297348047941341064870357513427734375f,-0.03780001909732799703078853781335055828094482421875f,-0.623699991527199681939919173601083457469940185546875f,0.6199000138998034259429914527572691440582275390625f,0.03079997820854174506166600622236728668212890625f,-0.7526000070363283445118440795340575277805328369140625f,0.480400015032291349825754878111183643341064453125f,0.04479998458623857260363365639932453632354736328125f,-0.65770000978857279250178180518560111522674560546875f,-0.264400027072429821828336571343243122100830078125f,0.035200007069111194368815631605684757232666015625f,0.48230002060979604294743694481439888477325439453125f,-0.4357000073075294466207196819595992565155029296875f,-0.219199992883205130311807806720025837421417236328125f,0.36560000801980496731857783743180334568023681640625f,0.142700009155273477290393202565610408782958984375f,-0.15960000433921806006765109486877918243408203125f,-0.282599998244643213496374301030300557613372802734375f,0.911100014293193982695129307103343307971954345703125f,0.63180000044107398338155689998529851436614990234375f,-0.463200013826787493353975833088043145835399627685546875f,-0.3947999937877055831592087997705675661563873291015625f,-0.51100001738965505904133124204236082732677459716796875f,-0.11810000656694175091843135305680334568023681640625f,-0.16759998461008063230792686226777732372283935546875f,-0.0879999850392341187443889793939888477325439453125f,0.15669999314397575762569658763823099434375762939453125f,-0.95090000215768810676308930851519107818603515625f,-0.62920000420808808172523640678264200687408447265625f,0.4320999930515885267823250615037977695465087890625f,0.560299987357854689662417513318359851837158203125f,0.11719997565001272210594152056728489696979522705078125f,-0.60109998560845856463430436633643694221973419189453125f,0.4184000087141990231742738615139387547969818115234375f,0.09479999248385428955998577293939888477325439453125f,-0.45480000135004516881309655218501575291156768798828125f,0.264200016725063324685152110760100185871124267578125f,0.00159999843835834099081694148480892181396484375f,-0.374800005745887732100385392186581157147884368896484375f,0.0604000040054319953242156771011650562286376953125f,-0.1781000104784968840476722107268869876861572265625f,-0.32889995679855399401958493399433791637420654296875f,-0.72700001499801825133317834115587174892425537109375f,-0.24860006018877101041653077118098735809326171875f,0.507499981075525408158455320517532527446746826171875f,-0.048999997474253209617245374829508364200592041015625f,-0.44659999501109126640585600398480892181396484375f,0.57559999274611417519054157310165464878082275390625f,0.2489000069782136836948893687804229557514190673828125f,-0.1642999795794484185762485140003263950347900390625f,0.3745999945044520273995658499188721179962158203125f,-0.45969999993890497957949037299840711057186126708984375f,-1.13810004385709806484783257474191486835479736328125f,-0.30860000088214878388725992408581078052520751953125f,0.6042999846830963139865389166516251862049102783203125f,1.1579000412583351220519034541212022304534912109375f,0.00499999254941929649476151098497211933135986328125f,0.220900003238022113460914397364831529557704925537109375f,0.761300030303001751263991536688990890979766845703125f,0.4867999773442743816076472285203635692596435546875f,0.580899995116889389379366548382677137851715087890625f,0.799700030672550354893246549181640148162841796875f,-0.321699980366230153805418012780137360095977783203125f,0.2411999958857895531849635517573915421962738037109375f,-0.5376000142633916567547203158028423786163330078125f,0.5936999952897423771247531476547010242938995361328125f,-0.44519997547864942344375549510004930198192596435546875f,-0.2507999859198921654979130835272371768951416015625f,0.01539999178647999400482149212621152400970458984375f,0.219500005461275460305614615208469331264495849609375f,0.501100009822845748885811190120875835418701171875f,0.24980000112652778199162639793939888477325439453125f,-0.81999999329447736950982061898685060441493988037109375f,-0.32120003347396863802032385137863457202911376953125f,0.336599991527944675251404760274454019963741302490234375f,0.08080000020638100632908162879175506532192230224609375f,-0.386799980380386099199085947475396096706390380859375f,0.2325999915391208094916919435490854084491729736328125f,-0.033799997892975774949064771135454066097736358642578125f,-0.435400007060170235462237542378716170787811279296875f,0.2555999888718130108600234962068498134613037109375f,-0.1669999897032976576838336768560111522674560546875f,-0.489400010234117388563390704803168773651123046875f,-0.225699998068809026818826168891973793506622314453125f,-0.016499996803700955894100843579508364200592041015625f,0.135200009267032239534955806448124349117279052734375f,0.137600014598667830245659615684417076408863067626953125f,0.0631000003643334139713516606207122094929218292236328125f,0.00640001893639574603867004043422639369964599609375f,-0.139199987220763876649698431720025837421417236328125f,-0.0597000006094574597881319277803413569927215576171875f,0.589499994471669008788694554823450744152069091796875f,0.6188000146567820269893900331226177513599395751953125f,0.0235000048354268020744939349242486059665679931640625f,-0.7243999889254570945240629953332245349884033203125f,-0.533799986195564457602813490666449069976806640625f,0.4137999944284558306861754317651502788066864013671875f,-0.628600007736682897530045011080801486968994140625f,-0.4180000020265577376221699523739516735076904296875f,0.3166000046968460157614799754810519516468048095703125f,-0.5874999855458735709134998614899814128875732421875f,-0.2753999812066558749990008436725474894046783447265625f,0.49649998506903647577104266019887290894985198974609375f,-0.567400013899803212780170724727213382720947265625f,-0.0077999872088430066696673748083412647247314453125f,0.63390000358819964532131052692420780658721923828125f,-0.308599990600347540148362668332993052899837493896484375f,-0.03539998094588525656689625975559465587139129638671875f,0.365399992755055469384473099125898443162441253662109375f,0.6255000053048132002686543273739516735076904296875f,0.53100001746416136683137665386311709880828857421875f,-0.182099990779161391429852301371283829212188720703125f,0.7439999890923498782058231881819665431976318359375f,0.4671999922275542616034726961515843868255615234375f,-0.3711999905586242931576634873636066913604736328125f,0.21589998785257336066933930851519107818603515625f,0.2753999926805494879999969271011650562286376953125f,-0.040799999256432073480027611367404460906982421875f,0.30079999720752237379883808898739516735076904296875f,0.14019999611675759698670162833877839148044586181640625f,-0.214299993251264087046337181163835339248180389404296875f,-0.8460000422000888420370756648480892181396484375f,-0.31280000658035245209021013579331338405609130859375f,0.662600023278594019160436801030300557613372802734375f,0.429900003468990377086811349727213382720947265625f,0.069999983012676292304377057007513940334320068359375f,-0.5338000001281499873329039473901502788066864013671875f,-0.604000001311302270323722041212022304534912109375f,0.9920000173449512459455945645458996295928955078125f,-0.1094000068627298816181792062707245349884033203125f,0.5839000047057865128152798206428997218608856201171875f,0.758300035764277513550268849940039217472076416015625f,0.3145999740898608454386931043700315058231353759765625f,0.7043000068858262796567260011215694248676300048828125f,1.1059000455200678203482311801053583621978759765625f,-0.59779999100863900896030145304393954575061798095703125f,0.7301999736905100402140078585944138467311859130859375f,0.50100000144541212421245290897786617279052734375f,0.3428999762892726010932165081612765789031982421875f,0.34729998056292554764468150096945464611053466796875f,1.26489999363422356282171676866710186004638671875f,0.179399994289874964437103699310682713985443115234375f,-0.32059999155402196802100434069870971143245697021484375f,0.12980000544786474137026743846945464611053466796875f,-0.039100001940131079436469008214771747589111328125f,0.1698999870777132270571740946252248249948024749755859375f,0.8254999816119668043512547228601761162281036376953125f,-0.59109997942447645158381419605575501918792724609375f,-0.74659998260587467910909253987483680248260498046875f,-0.65380001748800253835725015960633754730224609375f,-0.242699998929351290488654058208339847624301910400390625f,-0.03490000076815480412051329039968550205230712890625f,0.222699985108524511900895959115587174892425537109375f,-0.405500006347894181857327566831372678279876708984375f,0.339900008088350080726058877189643681049346923828125f,-0.117299981606006298306965618394315242767333984375f,-0.147199996995925896925427878159098327159881591796875f,0.60959997900724427477570088740321807563304901123046875f,0.20640001312494238305816907086409628391265869140625f,-0.3474999777972698211669921875f,-0.4274000002279876309074779783259145915508270263671875f,-0.8974000249266627093902570777572691440582275390625f,0.034099995942413752203492549597285687923431396484375f,0.1846000038549302502133286907337605953216552734375f,-0.00339998032748678458148106074077077209949493408203125f,-0.26719999133348437680979259312152862548828125f,-0.73260000301301475555959541452466510236263275146484375f,-0.33420001344680816401933043380267918109893798828125f,0.28239999810457216966597115970216691493988037109375f,-0.271499988086521704122588971586083061993122100830078125f,-0.807900002217292634298928533098660409450531005859375f,0.2786999929428102262818356393836438655853271484375f,-0.4436000014036893812630069078295491635799407958984375f,-0.98689996865987783536411370732821524143218994140625f,-0.782399980223178914684467599727213382720947265625f,0.36929998431056743068978676092228852212429046630859375f,-0.5613000252366067144293992896564304828643798828125f,-0.0377999958515164991013079998083412647247314453125f,0.6018000102385878680166797494166530668735504150390625f,-0.6017000173211097813208425577613525092601776123046875f,-0.026999985009431792803979988093487918376922607421875f,0.7656000044807791393708384930505417287349700927734375f,-0.27240000376701356543662768672220408916473388671875f,-0.130399993574619177394424696103669703006744384765625f,0.25580000436007976905017358149052597582340240478515625f,0.99029997946023939903170685283839702606201171875f,0.698999992370605838232222595252096652984619140625f,-0.39059997952133418408493525930680334568023681640625f,-0.816999999165535228939916123636066913604736328125f,-0.61219999014139148130198009312152862548828125f,0.38930000774264339469254991854541003704071044921875f,-0.0572000253081323961623638751916587352752685546875f,-0.3674000425100327760219443007372319698333740234375f,-0.4509999877735972884096327106817625463008880615234375f,-0.19240002120137233987406943924725055694580078125f,-0.0129999950230119676319873178726993501186370849609375f,0.34390001744627951563160195291857235133647918701171875f,0.582800021481514107790644629858434200286865234375f,0.0507999871492383903159861802123486995697021484375f,-0.7054000108599662866026847041212022304534912109375f,0.0966999997377395725806081827613525092601776123046875f,-0.0332000074565410585591962444595992565155029296875f,-0.1722999991521239337988191664408077485859394073486328125f,0.476800003719329623663725215010344982147216796875f,0.50440001515150090227734835934825241565704345703125f,-0.0888999969944357371076648632879368960857391357421875f,-0.4935999785304066023172708810307085514068603515625f,-0.018999963164329614073722041212022304534912109375f,0.7269999830350279967916549139772541821002960205078125f,0.09760000651478772937252870178781449794769287109375f,0.214000017106533046984395696199499070644378662109375f,0.1903999972254037942320792353712022304534912109375f,0.492900032615661842783083557151257991790771484375f,-0.063800016891956801146079669706523418426513671875f,-0.795700023922324195524424794712103903293609619140625f,0.6421000192403791828610337688587605953216552734375f,0.154200020897388601071043012780137360095977783203125f,0.53599996784329473342722849338315427303314208984375f,-0.4065999920308589477002669809735380113124847412109375f,0.3753999679446220483214347041212022304534912109375f,-0.52249997958540905784730057348497211933135986328125f,-0.680999990753829553824516551685519516468048095703125f,-0.552600013220310160022563650272786617279052734375f,-0.5546999824672937950964524134178645908832550048828125f,-0.0298000039204955324834145358181558549404144287109375f,0.485800004211068170434373314492404460906982421875f,-0.004000005930662187125790296704508364200592041015625f,0.269399996301531718945199145309743471443653106689453125f,0.677400018370151624225172781734727323055267333984375f,-0.343699992197752379752273554913699626922607421875f,0.920999994106590502696008115890435874462127685546875f,0.430200062501430835482096881605684757232666015625f,0.38669996874332479563918241183273494243621826171875f,-0.0470000038221478622091353827272541821002960205078125f,0.7514000006079672999703689129091799259185791015625f,0.6356999953866004915425946819595992565155029296875f,-0.7495000103414055647732539000571705400943756103515625f,-1.0415000323057181930153092253021895885467529296875f,-0.1369000073283910268173713120631873607635498046875f,0.68259997801631688840728884315467439591884613037109375f,1.2242000329673288216980608922312967479228973388671875f,0.49099998915195453008664117078296840190887451171875f,0.084400003752112429111775782075710594654083251953125f,-0.389200005102157575720411841757595539093017578125f,0.1613999972552060402364304536604322493076324462890625f,-0.041199998828768735759098262860788963735103607177734375f,-0.142399998775124547734094448969699442386627197265625f,-0.525599994385241853223078578594140708446502685546875f,-0.1566999937400221920569265421363525092601776123046875f,-0.7487000179767608898373509873636066913604736328125f,-0.43209999271631271966498388792388141155242919921875f,0.6103000065803525497898363028070889413356781005859375f,1.3775000286102301316759621840901672840118408203125f,-0.3013999977767467530753719984204508364200592041015625f,0.52459999510049815096834890937316231429576873779296875f,0.68620001693367971284942541387863457202911376953125f,0.60269999529719342490352573804557323455810546875f,-0.452099989512562583371391156106255948543548583984375f,-1.118100039982795834703210857696831226348876953125f,0.22699998143315358589688912616111338138580322265625f,-0.3243999986484646935736009254469536244869232177734375f,-0.6978000030040740764292195308371447026729583740234375f,-0.515599999600648839503946874174289405345916748046875f,0.35360000170171257405371534332516603171825408935546875f,0.016800002527237012373007019050419330596923828125f,0.0249999964237215266393832280300557613372802734375f,-0.1024000017926096628428922485909424722194671630859375f,-0.9838000097393990728278367896564304828643798828125f,-0.830400019800663358182646334171295166015625f,0.2468999967113136850827004309394396841526031494140625f,0.57810000483468193532843315551872365176677703857421875f,0.66400002221018095127647029585205018520355224609375f,0.091100000640004896723667116020806133747100830078125f,0.9657000084996223421285321819595992565155029296875f,0.720800009500980110033196979202330112457275390625f,-0.3395000042691826980245650702272541821002960205078125f,-0.8169000461459159367905158433131873607635498046875f,-0.17160001512765887810019194148480892181396484375f,0.7563000110432505085356069685076363384723663330078125f,0.016299996067583588654770210268907248973846435546875f,-0.065600010664760788348104370015789754688739776611328125f,-0.1773999954782426169419551342798513360321521759033203125f,0.504299992394447116339506465010344982147216796875f,0.49160000141859061528748497948981821537017822265625f,-0.1702999947711825168283894527121447026729583740234375f,0.8104000370860102719916540081612765789031982421875f,0.1641999949693673244155434076674282550811767578125f,-0.7534000239506364327013443471514619886875152587890625f,0.3441000148296353700061445124447345733642578125f,0.00460000731945076068996058893389999866485595703125f,-0.3885999919787048728636591476970352232456207275390625f,-0.4724000097274780873846111717284657061100006103515625f,-0.25579999832510935675600194372236728668212890625f,0.32870000605583193920011808586423285305500030517578125f,-0.13320001162886629941795035847462713718414306640625f,-0.0351999979794024309143196660443209111690521240234375f,0.1885000095292925781365056536742486059665679931640625f,0.614499996781348922780807697563432157039642333984375f,0.12719998358488115997033673920668661594390869140625f,-0.680399985900521198090018515358678996562957763671875f,0.22509999086856868899531036731787025928497314453125f,0.38159999486207940577742192544974386692047119140625f,0.07529999283403154830551784471026621758937835693359375f,1.15580002291202532660463475622236728668212890625f,0.724300006401538354339209035970270633697509765625f,0.3197999873429535622193498056731186807155609130859375f,0.045599997067451436549845311674289405345916748046875f,0.7728000117212534547661562100984156131744384765625f,-0.065699991810321822782725575962103903293609619140625f,-0.72029997208416485232618242662283591926097869873046875f,-1.13070001088380767129137893789447844028472900390625f,0.61459999212026605874825690989382565021514892578125f,-0.8333000119969244945394848400610499083995819091796875f,-0.2773000495553024080663817585445940494537353515625f,-0.69839998459294461152779831536463461816310882568359375f,0.73980000956431002379076744546182453632354736328125f,0.517800046087057097565775620751082897186279296875f,-0.4135999911963939013048729975707828998565673828125f,0.8795999786347152848975383676588535308837890625f,0.3840000066757198737832368351519107818603515625f,-0.5257999580919747728557922528125345706939697265625f,-0.911399996510148024952968626166693866252899169921875f,-0.660600014352798581285242107696831226348876953125f,0.1214000009618698694424665518454276025295257568359375f,-0.5436999778181317299896591066499240696430206298828125f,-0.841699998135864380088833058835007250308990478515625f,-0.481799985688924348181672030477784574031829833984375f,-0.3587999985635279553974896771251223981380462646484375f,-1.0824000273108485004058820777572691440582275390625f,0.5346999877572056192320815171115100383758544921875f,0.9286999808728697569648602438974194228649139404296875f,0.6943000031232831048555453890003263950347900390625f,0.357499964386225332901858564582653343677520751953125f,0.4260000001415609194310718521592207252979278564453125f,0.3552000169038773691454480285756289958953857421875f,0.0268999964505434174810716285719536244869232177734375f,-0.074800001089274925636374291570973582565784454345703125f,0.343600006172060989939609498833306133747100830078125f,-0.04900000451505182041955777094699442386627197265625f,0.33929998878091571778981005991226993501186370849609375f,0.6465000175535677851001992166857235133647918701171875f,0.365799979996681567939731394289992749691009521484375f,-0.042099997521936938227327118511311709880828857421875f,-0.6256999955356121745353448204696178436279296875f,-0.545999960690737307089648311375640332698822021484375f,-0.3705999890208244718081687096855603158473968505859375f,-0.8366000088691709635213555884547531604766845703125f,-0.458600006544589433588043903000652790069580078125f,0.0110000037178397125359197161742486059665679931640625f,-0.12570001324936719289127040610765106976032257080078125f,0.075199988032877362886807759423390962183475494384765625f,0.14859999568536874647861623088829219341278076171875f,0.883999972999095717796080862171947956085205078125f,0.6371999829888341793093786691315472126007080078125f,-0.38009998576492076693966737366281449794769287109375f,-0.377900008475780424532786128111183643341064453125f,-0.26820000581741343381736442097462713718414306640625f,0.21360000121742483969455861370079219341278076171875f,0.2669000044226645940170783433131873607635498046875f,-0.0920000185370444256705013685859739780426025390625f,-0.5406999991491436841073436880833469331264495849609375f,0.62310001643896129053246113471686840057373046875f,0.4157999966859815543784861802123486995697021484375f,-0.420000015757977962493896484375f,0.9954999820590018089205841533839702606201171875f,0.5851999802470206901716665015555918216705322265625f,-0.601699987258017099378548664390109479427337646484375f,0.4786999920487406967595234164036810398101806640625f,0.3397999795198440864396616234444081783294677734375f,-0.3572000032171607220021769535378552973270416259765625f,-0.6090999799609182474569024634547531604766845703125f,-0.354799987566471575206605848507024347782135009765625f,0.37939997812211512240310185006819665431976318359375f,0.47120003004670163448963648988865315914154052734375f,0.1429999974071975987754967718501575291156768798828125f,-0.5186000220790505377266299547045491635799407958984375f,-0.015699996280670092119180480949580669403076171875f,-0.235400009667873888474787236191332340240478515625f,-0.123100005933642453470611144439317286014556884765625f,-0.66059998574256884040778459166176617145538330078125f,-0.4173999909520151874176008277572691440582275390625f,0.36429998494386672813760696953977458178997039794921875f,0.429799993526935875109984408481977880001068115234375f,0.35179998524188960118408431299030780792236328125f,-0.197900008475780542216426738377776928246021270751953125f,0.93190003482103378473766497336328029632568359375f,0.35819999508857680581286331289447844028472900390625f,-0.788400023429095764271323787397705018520355224609375f,-0.8802000046849247638647284475155174732208251953125f,-0.312199996399879609043637174181640148162841796875f,0.6805999872699379782403639183030463755130767822265625f,0.510000023841858052264797152020037174224853515625f,0.511900003081560139861494462820701301097869873046875f,-0.3929999829530714805514435283839702606201171875f,-0.96799997043609664615360088646411895751953125f,-0.2432999913215638798646978102624416351318359375f,-0.3078000363826749463669329998083412647247314453125f,0.16400004422664693493061349727213382720947265625f,-0.31799999949336044124947875388897955417633056640625f,-0.4438999770641327557996191899292171001434326171875f,-0.21610000982284560677726403810083866119384765625f,0.03589999843239777277403845801018178462982177734375f,-0.4131999953866003494340475299395620822906494140625f,1.1862000242352479517649044282734394073486328125f,0.438400012737512501104220064007677137851715087890625f,-0.5051999799489976794575341045856475830078125f,0.366399993157386916209361515939235687255859375f,-0.165400009965896543917551753111183643341064453125f,0.163800004076957606002906686626374721527099609375f,-0.55579996867179914943335461430251598358154296875f,-0.448999993264675101301008908194489777088165283203125f,0.86259997894763973391718536731787025928497314453125f,-0.747599984014033935864063096232712268829345703125f,-0.283800001692771974859397232648916542530059814453125f,0.836499991774559248369769193232059478759765625f,-1.316499951243400801104144193232059478759765625f,-0.651199993538856602981468313373625278472900390625f,0.25369999547600752265452683786861598491668701171875f,0.09250000521540613362958538345992565155029296875f,0.1596000073194503432461033298750407993793487548828125f,0.91279999480843532211338242632336914539337158203125f,0.0847999448001388600459904409945011138916015625f,0.2898999900579453115057049217284657061100006103515625f,1.214599985265731874051198246888816356658935546875f,0.10919996218681404798189760185778141021728515625f,0.071699985134601629255257648765109479427337646484375f,-1.0439000047802924342477126629091799259185791015625f,0.39970004378557177915354259312152862548828125f,-0.052399989610910413517785855219699442386627197265625f,-0.230699962306022854363618534989655017852783203125f,0.1906999819755554881339776329696178436279296875f,-0.218900014764070505179915926419198513031005859375f,1.17600001403689358170367995626293122768402099609375f,0.15639997482895751090836711227893829345703125f,0.3423000123023987040227211764431558549404144287109375f,-0.23279999406337736900240997783839702606201171875f,0.8716999723196021676585587556473910808563232421875f,0.11500000491738315844258977449499070644378662109375f,0.3256000061571597825604840181767940521240234375f,0.57829998558461692681476051802746951580047607421875f,-0.423199998813867495073282043449580669403076171875f,-0.281200003820657684627803973853588104248046875f,0.483999958693981824353613774292171001434326171875f,-0.05220000876784336441005507367663085460662841796875f,-0.07540001458525669164600913063623011112213134765625f,-0.5364999776184564694858636357821524143218994140625f,0.6929999923408030060301143748802132904529571533203125f,-0.5121999834358692549329816756653599441051483154296875f,-1.0574999769032000784818592364899814128875732421875f,0.586799992394447134103074859012849628925323486328125f,0.043800018382072369860225080628879368305206298828125f,-0.6253999700307844022972858510911464691162109375f,0.70519999559521640009052134701050817966461181640625f,0.08220000846981978526173406862653791904449462890625f,-0.06740001062154732380804489366710186004638671875f,-0.428000005304813413431475055404007434844970703125f,-0.710200011837482492182971327565610408782958984375f,0.3455999455094342920347116887569427490234375f,-0.235399999088048872408762690611183643341064453125f,0.791800012212991788373983581550419330596923828125f,0.5887000251293184049927731393836438655853271484375f,0.64900002481788410246821285909391008317470550537109375f,-0.0608000059619545341060842247316031716763973236083984375f,-0.5783000194288792084051920028286986052989959716796875f,-0.03860000505447391816460367408581078052520751953125f,-0.193800008845329330142703838646411895751953125f,-0.3896999893218282551288211834616959095001220703125f,-0.338400016611814624667431417037732899188995361328125f,0.242599999213218620042198381270281970500946044921875f,0.0784000366538759596579666322213597595691680908203125f,0.2088000183820728494765717186965048313140869140625f,-0.7859999939203259344822072307579219341278076171875f,-0.4932000213146210398917901329696178436279296875f,0.5019999956637619664689964338322170078754425048828125f,-0.2447999906957150084707564019481651484966278076171875f,-0.75859998504966485999290171093889512121677398681640625f,-0.317000017344951867670488354633562266826629638671875f,-0.540400020396709646064437038148753345012664794921875f,0.0190999820470807435413007624447345733642578125f,-0.30280000181198118980319122783839702606201171875f,-0.575300010007619988527949317358434200286865234375f,0.7793999925017356389389533433131873607635498046875f,0.4519999805390835234675250831060111522674560546875f,0.17289999744892126098960716262809000909328460693359375f,0.47529997141361235435397247783839702606201171875f,-0.78149997746944421805892488919198513031005859375f,-0.384899993932247053152195803704671561717987060546875f,-0.7242999947786330494636786170303821563720703125f,-0.176900006806850029761335463263094425201416015625f,0.0104999935328959903557688448927365243434906005859375f,-1.01409996491074583246927431900985538959503173828125f,0.5601000065147871964654768817126750946044921875f,0.0029000007271766659044942571199499070644378662109375f,-0.5579999760985376866528895334340631961822509765625f,0.839599981987475985079072415828704833984375f,0.290300000172853511770654222345910966396331787109375f,0.229900000786781077977138920687139034271240234375f,1.089099959397316297327051870524883270263671875f,0.27059998857378975145593358320184051990509033203125f,0.3405999732255935441571637056767940521240234375f,-1.1077999720096585178907844237983226776123046875f,-0.65989999944567667711226022220216691493988037109375f,0.575099999213218637805766775272786617279052734375f,0.272099979305266970186494290828704833984375f,0.38870000650286673504751888685859739780426025390625f,-0.45450001207366563793499381063156761229038238525390625f,0.86010000869408287371697952039539813995361328125f,0.2299999970756470835109297468079603277146816253662109375f,-0.510099994146823831897563650272786617279052734375f,-0.2290999959051607248738946509547531604766845703125f,-0.08770000304579739225374623856623657047748565673828125f,-0.082599984014034344426136158290319144725799560546875f,-0.3166000164687634566007545799948275089263916015625f,0.0564999985545873162351426799432374536991119384765625f,-0.46199998426437360876661841757595539093017578125f,-0.2987999379158026158620486967265605926513671875f,-0.40619998966455472810821447637863457202911376953125f,-0.7395999839246274465409669574000872671604156494140625f,0.49679999857842904731342059676535427570343017578125f,0.08739999985545876537429421659908257424831390380859375f,-0.0590000051856041096698390902020037174224853515625f,-0.8512000009894364893625606782734394073486328125f,-0.464500011086463882037378425593487918376922607421875f,0.6847999932289123847795053734444081783294677734375f,-0.08219997047185945149294639122672379016876220703125f,-0.2468000065132976150739096965480712242424488067626953125f,-0.2454000115677714977113765826288727112114429473876953125f,0.0101999915905295335250002608518116176128387451171875f,0.2031999907672403793412740924395620822906494140625f,-0.3794000012934208854176176828332245349884033203125f,0.18320000015497139855824570986442267894744873046875f,-0.2838000031828880054263208876363933086395263671875f,-0.2724000028729438582786315237171947956085205078125f,0.20529998184442543873728936887346208095550537109375f,-0.50439999682307234962763686780817806720733642578125f,0.74959999197125437575550677138380706310272216796875f,0.347400015091896108287983224727213382720947265625f,0.20800001111626631455919778090901672840118408203125f,-0.54080000488162038863038105773739516735076904296875f,-0.17480004895925471686268792836926877498626708984375f,0.555599995577335192820100928656756877899169921875f,0.225400019800662931857004878111183643341064453125f,0.3262000188708302772511160583235323429107666015625f,0.693800029557943620517335148178972303867340087890625f,0.054000006526708776988243698724545538425445556640625f,-0.623100013160705401560335303656756877899169921875f,-0.422400000935792885314867817214690148830413818359375f,0.058799987238645545772897094138897955417633056640625f,0.4427999895930287976852923748083412647247314453125f,-0.17979999606013308266483363695442676544189453125f,0.439999982863664484966648160479962825775146484375f,0.7784999748468397484657543827779591083526611328125f,-0.7208000200062991691396518945111893117427825927734375f,0.2996000032961367853800993543700315058231353759765625f,0.96280000639408858109646871525910682976245880126953125f,-0.0317999997705220582844276577816344797611236572265625f,-0.2289999937862157164403242859407328069210052490234375f,-0.001500008650123785258756470284424722194671630859375f,-0.5685999998390673937365136225707828998565673828125f,-0.0152000060260295555281118140555918216705322265625f,0.979299963039160115130243866587989032268524169921875f,-0.4109999971985816014097281367867253720760345458984375f,0.1601999945521355162014742745668627321720123291015625f,0.5996999849259854187266682856716215610504150390625f,0.31719999654889097229215622064657509326934814453125f,-0.43739998424649240149619799922220408916473388671875f,-0.229400020068883048196539675700478255748748779296875f,0.754000006079673656955719707184471189975738525390625f,-0.07939999399185193595940290833823382854461669921875f,-0.07270000556036826377948045774246565997600555419921875f,-0.53749999480321974942853557877242565155029296875f,-0.2410000034384429390410531368615920655429363250732421875f,0.3813000368595123035220240126363933086395263671875f,0.0424999892711639404296875f,-0.227000013917684650976980265113525092601776123046875f,0.1709999915361404276836765347979962825775146484375f,-0.5943999844551086653154925443232059478759765625f,-0.298299997282028162004507976234890520572662353515625f,0.135899970865249741791558335535228252410888671875f,-0.1741000130414960267444257624447345733642578125f,0.3165000054836273335467922152020037174224853515625f,-0.0705000240802764466252483543939888477325439453125f,-0.09689997491836521703589824028313159942626953125f,-0.2844999988675116497915951185859739780426025390625f,0.663400041794776740289307781495153903961181640625f,1.005799965095519610258634202182292938232421875f,0.26849999079108233246415693429298698902130126953125f,-0.5063000109314916841185549856163561344146728515625f,1.0043000183224677357429754920303821563720703125f,0.328000006198883031771629248396493494510650634765625f,0.3812999882817269536872117896564304828643798828125f,-0.5402999991297718906935187987983226776123046875f,-0.146100002670288109385410280083306133747100830078125f,0.893099971437454343003992107696831226348876953125f,0.11809996845722281477719661779701709747314453125f,0.136799988222122248515688625047914683818817138671875f,0.9009999970495699006534096042742021381855010986328125f,-0.76140001781880872755436939769424498081207275390625f,-0.14279999834746115450201386920525692403316497802734375f,0.010100001597404428821391775272786617279052734375f,-0.21089998390376596404394149431027472019195556640625f,-0.165699994790554028245566087207407690584659576416015625f,1.03069997720718387057559084496460855007171630859375f,-0.761899992501735567884679767303168773651123046875f,-0.316399998670816362533741994411684572696685791015625f,0.58199998411536224551809937111102044582366943359375f,-0.43739999646544447386986576020717620849609375f,-0.0959999980926513618584294817992486059665679931640625f,-0.10390000030994439583764687995426356792449951171875f,0.132900011456012379085223074071109294891357421875f,-0.174199994671344615682073708740063011646270751953125f,-0.2935999862790108494436935870908200740814208984375f,1.12439997581243478208534725126810371875762939453125f,-0.04119999711513511186922187334857881069183349609375f,0.1386000074386597447073654620908200740814208984375f,0.40340000089108940528603852726519107818603515625f,-0.696200017827749206844600848853588104248046875f,-0.21079998163580881964662694372236728668212890625f,0.89550000232458160098758526146411895751953125f,-0.027999994277954076693504248396493494510650634765625f,0.462599999809265138850378207280300557613372802734375f,0.6007999990701673453941111802123486995697021484375f,-0.411200010079145528152366750873625278472900390625f,0.275399998193979200777903315611183643341064453125f,-0.01039998970031685843196100904606282711029052734375f,-0.10400000146031374015365145169198513031005859375f,-0.27879999588131898491383253713138401508331298828125f,0.28239998864233495368125659297220408916473388671875f,0.163599999094009351807699204073287546634674072265625f,-0.617200012493133609581263954169116914272308349609375f,-0.8198000134944916084123178734444081783294677734375f,-0.10680000148415569327653429354541003704071044921875f,0.34120000709891318280142513685859739780426025390625f,0.7637999960303307744879930396564304828643798828125f,-0.5246000083625315113522447063587605953216552734375f,-0.4445999952495096607663072063587605953216552734375f,0.0337000030755998380982418893836438655853271484375f,0.6233999953776596658627795477514155209064483642578125f,0.108200017216801558372907265948015265166759490966796875f,-0.419899977354705189469541437574662268161773681640625f,-0.311400015807151930857799015939235687255859375f,0.0368000022292136463875067420303821563720703125f,0.393900013273954296977308331406675279140472412109375f,-0.3739999954998491915603153756819665431976318359375f,0.0627999903380871415947694913484156131744384765625f,0.735799984169005938383634202182292938232421875f,0.093799998563528053097115844138897955417633056640625f,-0.019600002253055581746821189881302416324615478515625f,-0.3374999830126765854032555580488406121730804443359375f,0.252999999567866284966299872394301928579807281494140625f,-0.227599996605515497716254458282492123544216156005859375f,-0.327900001211464342532053706236183643341064453125f,0.374200000780820829504591529257595539093017578125f,0.3054000086247921075255362666212022304534912109375f,-0.65529995874762558827342218137346208095550537109375f,-0.69400003364682216755454646772705018520355224609375f,-0.096800007742643412456118312547914683818817138671875f,0.497500009834766387939453125f,0.0734999991357323523288869182579219341278076171875f,0.02480000142455096590055063643376342952251434326171875f,0.883500021934509049970074556767940521240234375f,-1.0966999812603006603239919058978557586669921875f,-0.47519998918771744200739703956060111522674560546875f,0.7263999982237816510632910649292171001434326171875f,0.3419999791979790870755095966160297393798828125f,0.221899995630979507410529549815692007541656494140625f,-0.2377000246524811899462292785756289958953857421875f,-0.40770000617504109641231480054557323455810546875f,0.070600007051229596299890545196831226348876953125f,0.3574000112175941268333190237171947956085205078125f,-0.3281999865949156713895717984996736049652099609375f,-0.26159999903440478430383109298418276011943817138671875f,-0.9344000085949897282944220933131873607635498046875f,-0.27239997903108559285101364366710186004638671875f,-0.25680000193119045803769040503539144992828369140625f,0.9193000025272368702644598670303821563720703125f,0.15329998059272753607729100622236728668212890625f,0.092499997913837450624896519002504646778106689453125f,-0.147699989485740701411486952565610408782958984375f,-1.0862999617576605260182986967265605926513671875f,-0.461299988430738494571414776146411895751953125f,-0.2769999775215985504672744355048052966594696044921875f,0.95719997684210511579294688999652862548828125f,0.315399997486174143990211860000272281467914581298828125f,0.37100001418590533575070367078296840190887451171875f,-0.397200012195110474522152799181640148162841796875f,-0.121900002485513692107588212820701301097869873046875f,0.79049999845027940637010033242404460906982421875f,-0.441099996709823471974232234060764312744140625f,-0.171500003993511285216300166212022304534912109375f,-0.399899988865852318298266254714690148830413818359375f,0.09469999908208848182766814716160297393798828125f,0.06239999817907804224859091846155934035778045654296875f,-0.2885999931707978749528820117120631039142608642578125f,0.180400004042684969363108393736183643341064453125f,-0.035100000591576085984257815653108991682529449462890625f,-0.7272999825000763252091928734444081783294677734375f,0.712099987947940604726682067848742008209228515625f,0.4119000018894671821811925838119350373744964599609375f,-0.315099971199035877589267329312860965728759765625f,0.33950001540780050390821998007595539093017578125f,-0.13620000173449520186608197036548517644405364990234375f,0.71680000804066656971968996003852225840091705322265625f,0.55030001991689214690950393560342490673065185546875f,0.0206000080943109509235000587068498134613037109375f,-0.5404000007271765326777313021011650562286376953125f,0.1494999750256535975267979665659368038177490234375f,-0.27700000691413872999646628159098327159881591796875f,-0.3749999982118605856840076739899814128875732421875f,0.5398999959439034768138299114070832729339599609375f,0.60639999464750271584989604889415204524993896484375f,-0.50879998500347145551359062665142118930816650390625f,-0.445600010925530920502524168114177882671356201171875f,-0.41019999723434441563085783855058252811431884765625f,0.17159998443126678324688327847979962825775146484375f,-0.05230000514984123327622000942938029766082763671875f,0.42039999849200260229054038063623011112213134765625f,-0.73519998725056634469865457504056394100189208984375f,-1.384599972152710023465260746888816356658935546875f,-0.359599996590614257030438238871283829212188720703125f,-0.057800016117095953660509621840901672840118408203125f,0.35419997619390475307454835274256765842437744140625f,-0.43700000665336846683572957772412337362766265869140625f,-0.35280000870376826416219273596652783453464508056640625f,0.1026999893926083817774497219943441450595855712890625f,0.6836000011801719455917236700770445168018341064453125f,0.4406000390887261364269988916930742561817169189453125f,-0.3753999573647970322554101585410535335540771484375f,0.2575999947428702085971963242627680301666259765625f,-0.4499999973177910561616954510100185871124267578125f,-0.36080001099109626494509939220733940601348876953125f,-0.733600004607438904713490046560764312744140625f,-0.3618000163853167805427801795303821563720703125f,0.9595999778151504955303607857786118984222412109375f,0.14860000080764290686374806682579219341278076171875f,0.424600000688433620865680495626293122768402099609375f,-0.305299969811737792468875341000966727733612060546875f,-0.5876000041306017607212197617627680301666259765625f,-0.1220000011026858288687435560859739780426025390625f,0.55219999014139187210048476117663085460662841796875f,0.683399986958503458112090811482630670070648193359375f,-0.439599992716312559792868341901339590549468994140625f,-1.131899969404936090455748853855766355991363525390625f,-0.677000017791986596904507678118534386157989501953125f,0.4078999916374683731845607326249592006206512451171875f,0.579399998387694292745209168060682713985443115234375f,0.36079997083246695410707616247236728668212890625f,0.37170000115334989043702762501197867095470428466796875f,0.270600013756752133531335857696831226348876953125f,-0.343999969720840681475237943232059478759765625f,-0.38530000032186517699983596685342490673065185546875f,0.628300030362606065637010033242404460906982421875f,-0.6290999863684181292455832590349018573760986328125f,-0.41470000310540211518883779717725701630115509033203125f,-0.24639998362064385872827187995426356792449951171875f,0.488000000536441547183130751363933086395263671875f,-0.108000001877546214501535359886474907398223876953125f,-0.2804000326156614875117156771011650562286376953125f,-0.48419999318122819431664538569748401641845703125f,0.0059000036478042972021285095252096652984619140625f,-1.547099966192245545926198246888816356658935546875f,-0.02829995734691692632623016834259033203125f,-0.18169998960495004070025970577262341976165771484375f,0.307199978816509400303402799181640148162841796875f,-0.53919999288320497043969226069748401641845703125f,-0.194900002723932264103723355219699442386627197265625f,-0.02650000468268987052766760825761593878269195556640625f,-0.88589997986182567046853364445269107818603515625f,-0.38299999809637663705785826095961965620517730712890625f,-0.22919996829628974666093199630267918109893798828125f,1.287599983865021613382850773632526397705078125f,0.5539000107407570094864013299229554831981658935546875f,-0.3037999951362611028571336646564304828643798828125f,0.0250000059604644775390625f,-0.181200001883506711664040267351083457469940185546875f,0.968500011205673416725403512828052043914794921875f,-1.2542999810695647511238348670303821563720703125f,-0.59860000639557853219230310060083866119384765625f,-0.38399999609589574589563198969699442386627197265625f,0.49619998213946825416087449411861598491668701171875f,0.377599999362230265287365682524978183209896087646484375f,0.4700999968290329178444153512828052043914794921875f,-0.90309998037815120142113300971686840057373046875f,-0.33349999392032625422643832280300557613372802734375f,-1.12539996496438998718758739414624869823455810546875f,0.591599990987777601958441664464771747589111328125f,0.06049999353289603476468982989899814128875732421875f,0.8884999966025350914833325077779591083526611328125f,-0.2721000230401753317011070976150222122669219970703125f,-0.20900000838935384184225085846264846622943878173828125f,-0.57180001675784597470197923030355013906955718994140625f,-0.186100018353760443545752423233352601528167724609375f,-0.1460000043883919740306254198003443889319896697998046875f,-0.3176000037208199788807405639090575277805328369140625f,-0.0846000071145593945942664504400454461574554443359375f,0.14780001570731415849735412848531268537044525146484375f,-0.23680000021755687189539685277850367128849029541015625f,0.125499955851585187627250661535072140395641326904296875f,0.478400013035535920380425523035228252410888671875f,-0.110399999088048872408762690611183643341064453125f,-1.057199993568658680231919788639061152935028076171875f,-0.87220000936388952794686701963655650615692138671875f,-0.14660000097155556630923456395976245403289794921875f,1.18549999487400015141247422434389591217041015625f,-0.04479999918937682679143108543939888477325439453125f,0.26299999785423278098051014239899814128875732421875f,-0.0579999874234200518685611314140260219573974609375f,0.5708000309586527265537370112724602222442626953125f,0.2043999930977822288014067453332245349884033203125f,0.222999970436095651393770822323858737945556640625f,-0.64860002666115779135225238860584795475006103515625f,-0.03939998937249189481235589482821524143218994140625f,1.01490000793933887734965537674725055694580078125f,-0.7604000178635121898196302936412394046783447265625f,0.3950000052154063467924061114899814128875732421875f,1.46139999166727108104169019497931003570556640625f,-0.9990000360310080651515818317420780658721923828125f,-0.3314000171482565093583616544492542743682861328125f,1.121300000500678351045280578546226024627685546875f,-0.3938000230014326863425822011777199804782867431640625f,0.5654000083267687326582517925999127328395843505859375f,0.64790003395676620812082546763122081756591796875f,0.37140000612139711932968566543422639369964599609375f,-0.38240000510811800182864317321218550205230712890625f,-0.94509998371601167121980324736796319484710693359375f,0.220400011903047587935589035623706877231597900390625f,0.157999993234872793124168310896493494510650634765625f,0.22899998015165312637009265017695724964141845703125f,0.102800003898143810232568284845910966396331787109375f,-0.150200004386901841968438020558096468448638916015625f,-0.290600002580881255909162064199335873126983642578125f,-0.297000018239021557064916123636066913604736328125f,-0.0472000027328729476749913374078460037708282470703125f,-0.276100005427002859192953110323287546634674072265625f,-0.73269998620748477691222433350048959255218505859375f,-0.425100005844235406815556643778108991682529449462890625f,-0.14249999972060323205624854381312616169452667236328125f,-0.4492999864153564004709551227279007434844970703125f,-0.2474999992735683884692576839370303787291049957275390625f,0.26249997578561312661094007125939242541790008544921875f,0.058300020192563639653826612629927694797515869140625f,-0.137199996418505831830003671711892820894718170166015625f,-1.16090000960826866815978064551018178462982177734375f,0.532700017201900521968127577565610408782958984375f,0.215500006347894679237242598901502788066864013671875f,1.177100021028518739285573246888816356658935546875f,-1.284299981665611767311929725110530853271484375f,-0.52409999828934683563375074299983680248260498046875f,-0.12840000424385067390176118351519107818603515625f,0.11759999206066140686743892729282379150390625f,0.17530000047087668235690216533839702606201171875f,0.536599994564056714807520620524883270263671875f,1.004199995565414837983553297817707061767578125f,0.2733999908328055283845969825051724910736083984375f,1.043300022315978736742181354202330112457275390625f,-0.8937000114202504619242972694337368011474609375f,-0.3613000000536441547183130751363933086395263671875f,1.53009999384880046591206337325274944305419921875f,-0.7083000247001649540834478102624416351318359375f,-0.16080000041127195942181060672737658023834228515625f,1.05490002224445333922631107270717620849609375f,-1.608500015974044572431012056767940521240234375f,-0.6781000145018101221694450941868126392364501953125f,0.60610001175999617117895468254573643207550048828125f,-0.1177000402987005145405419170856475830078125f,0.1792999977588654036964044280466623604297637939453125f,-1.0306999801874159317094381549395620822906494140625f,0.19030001969337462242037872783839702606201171875f,-0.017800001513958108034785254858434200286865234375f,0.38669998871088040459653711877763271331787109375f,0.456699988710880688813631422817707061767578125f,0.13909999307394027567852390347979962825775146484375f,-0.31840000096559517572814002051018178462982177734375f,0.018400008416175950287652085535228252410888671875f,-0.031100000584125542246738405083306133747100830078125f,-0.032400002574920705455951974727213382720947265625f,0.162099983477592246572385192848742008209228515625f,0.096800006699562235468192739062942564487457275390625f,-0.266199993538856372055079191341064870357513427734375f,-0.519200000631808933349020662717521190643310546875f,0.24920000540018083512450175476260483264923095703125f,-0.14799999949336051230375232989899814128875732421875f,-0.4782000058919189111605874131782911717891693115234375f,0.76800000899285081334966207577963359653949737548828125f,-0.21140000038444994867603554666857235133647918701171875f,-1.34519997139945690634021957521326839923858642578125f,0.2102000061750413095751355285756289958953857421875f,0.4260000201463700619797236868180334568023681640625f,0.3555000205039977601018108543939888477325439453125f,0.46520000036358821748905256754369474947452545166015625f,0.09900000682473175661613140619010664522647857666015625f,-0.37339998852312561439248383976519107818603515625f,-0.37119999577403051116419874233542941510677337646484375f,-0.01220000280737865150371135314344428479671478271484375f,0.2307999921411276744720453280024230480194091796875f,-0.47099998423457112295409388025291264057159423828125f,0.61979999635815641312319712596945464611053466796875f,0.3813000070571899158267115126363933086395263671875f,0.42260001485943810006773446730221621692180633544921875f,-0.57780000702738754103648943782900460064411163330078125f,-0.82590001966655268716976934229023754596710205078125f,-0.2063999946102498539435288193999440409243106842041015625f,0.33560000634342446179658736582496203482151031494140625f,0.4100000039674343010887014315812848508358001708984375f,0.4897999968051909291943957214243710041046142578125f,0.3190000329017639302264797152020037174224853515625f,0.38210001506805468807215220294892787933349609375f,-0.34240000585317620362957313773222267627716064453125f,0.24699999827146523756482565659098327159881591796875f,1.04839998457431793354999172152020037174224853515625f,-0.01920000122785570084715800476260483264923095703125f,0.61380000973939896624642642564140260219573974609375f,0.7884000215291973034936745534650981426239013671875f,-0.10440000382661818179030888131819665431976318359375f,-0.57260000249147413597938793827779591083526611328125f,-0.03740002030730238402611576020717620849609375f,0.149200004357099569318734211265109479427337646484375f,0.244800010961294223221784704946912825107574462890625f,0.1514000044077636886186155606992542743682861328125f,0.433200007307529499911424863967113196849822998046875f,0.22019999023079872557673297706060111522674560546875f,-0.6096000016570091162293465458787977695465087890625f,-0.09319997035265004114990006200969219207763671875f,-0.0792999895632267026712725055404007434844970703125f,-0.498300008606910704855863514239899814128875732421875f,0.2189000091016293225720801274292171001434326171875f,0.0399000026494265025434060589759610593318939208984375f,-1.394599978038668641744379783631302416324615478515625f,0.861000006325542965868180544930510222911834716796875f,0.271899997214227806541231302617234177887439727783203125f,0.517600024247169443469829275272786617279052734375f,0.64459998943805629778580623678863048553466796875f,0.301300003778934399889521955628879368305206298828125f,-0.288199991884827699806237433222122490406036376953125f,0.72219998365938664619534392841160297393798828125f,0.25649999691545966040706616695388220250606536865234375f,0.133399989566207022306798535282723605632781982421875f,-0.53419998029172433007261133752763271331787109375f,-0.14929999321401121736840877929353155195713043212890625f,0.211499976873398054522112943232059478759765625f,-0.21509998848438272034400142729282379150390625f,0.2325000111758708243314686114899814128875732421875f,-0.826400003215670420075866786646656692028045654296875f,0.1358000323742629689149907790124416351318359375f,-0.15379999938309196760855002139578573405742645263671875f,0.40609999650493266987183460514643229544162750244140625f,-0.02749999815598158647844684310257434844970703125f,0.1107000063575803994186941281441249884665012359619140625f,0.69560002985000579656116315163671970367431640625f,0.9809999742507926612233859486877918243408203125f,0.30540000206828121775970430462621152400970458984375f,1.1965999969482421505517777404747903347015380859375f,-0.23580002112388598334291600622236728668212890625f,-0.0813000063121318561343287001363933086395263671875f,0.9606000226974487077313824556767940521240234375f,0.47939996776580784398902324028313159942626953125f,0.300899994856119068487032564007677137851715087890625f,-0.030799996685981767541306908242404460906982421875f,-0.566799977195262982831991394050419330596923828125f,-0.4166999995887279428785632262588478624820709228515625f,0.2408000032424926217800020822323858737945556640625f,0.4075999934017655590423601097427308559417724609375f,0.18900000309944153809027511670137755572795867919921875f,-0.383799999010562942203250713646411895751953125f,0.260199986840784713848506726208142936229705810546875f,0.5655999917030332113654367276467382907867431640625f,-0.4989999989271165503623706172220408916473388671875f,-0.81119999651908880622386277536861598491668701171875f,0.1045999952495098028748543583787977695465087890625f,-0.5295999997198581610291512333787977695465087890625f,0.15829999400377214868740338715724647045135498046875f,-0.11800000202655791525785389239899814128875732421875f,-0.67579999907016752302979512023739516735076904296875f,0.2309999560713773547604432678781449794769287109375f,-0.3779999988600610816291691662627272307872772216796875f,0.0845999884322285733428969933811458759009838104248046875f,0.3485999901719389892917888573720119893550872802734375f,0.5444000068068504116780559343169443309307098388671875f,-0.4378000027060509058429715878446586430072784423828125f,-0.62420001270174996221840046928264200687408447265625f,-0.16459999673962588673248319537378847599029541015625f,0.33800001379847532945177590590901672840118408203125f,0.7452999831855304790906302514486014842987060546875f,-0.0576000017464159963509473527665250003337860107421875f,-0.5007999944508074730009639097261242568492889404296875f,-0.19640000656843170645515783689916133880615234375f,0.8418000296473504562300149700604379177093505859375f,0.1425999887824058731666809762828052043914794921875f,-0.827500011324882933649860206060111522674560546875f,0.3958000156104566347181616947636939585208892822265625f,0.5050000195205213149307610365212894976139068603515625f,0.1771000116407872582158233853988349437713623046875f,-0.4578000027805566585215046870871447026729583740234375f,0.0099999962747097104198701345012523233890533447265625f,0.93829997504204520897275187962804920971393585205078125f,-0.54400001472234738031374945421703159809112548828125f,0.39140000954866405180609945091418921947479248046875f,1.237999999940395667863413109444081783294677734375f,0.7230000281780959436872535661677829921245574951171875f,-0.166200010302662792849304196352022700011730194091796875f,-0.718900026126206181942279727081768214702606201171875f,0.04720000250935552088549229665659368038177490234375f,0.1738000000536441547183130751363933086395263671875f,0.4383999918013810059846946387551724910736083984375f,0.199800007906556220671490109452861361205577850341796875f,0.032200005713105273041918508170056156814098358154296875f,-0.127700000773370181406107803923077881336212158203125f,0.5745999892890452098725972973625175654888153076171875f,-0.078399999475479098531849331266130320727825164794921875f,-0.7495999834775926018437530728988349437713623046875f,0.40359999492168441292960778810083866119384765625f,-0.0605000075399875214543499168939888477325439453125f,0.3201000355720520218483216012828052043914794921875f,-0.2120999912261964936988078989088535308837890625f,-0.354800008428096891321956718456931412220001220703125f,0.32959999122619620948171359486877918243408203125f,-0.63759997298717507874243892729282379150390625f,-0.52989999765753736937767826020717620849609375f,0.27449998547509324442472689042915590107440948486328125f,-0.46469999716356369390268810093402862548828125f,-0.1020000009350478797909289596645976416766643524169921875f,-0.51959999346137042408599882037378847599029541015625f,0.43240000838637371316508506424725055694580078125f,-0.0517000054001808262427175577613525092601776123046875f,0.86829998416900622260072850622236728668212890625f,0.064499994695186302351430640555918216705322265625f,0.0762000075459481340800493853748776018619537353515625f,-0.22749998882412914014139460050500929355621337890625f,-0.460299970370531497110277996398508548736572265625f,-0.331099993878603005015293092583306133747100830078125f,-0.7178000161170956516798469237983226776123046875f,1.2675999974250800050867837853729724884033203125f,0.46099999347329134735673505929298698902130126953125f,0.35910001050829887248028171597979962825775146484375f,0.95849999168515154224223806522786617279052734375f,0.4349000055551528465258570577134378254413604736328125f,1.0251999920189380777202359240618534386157989501953125f,-0.58460000039041037922515897662378847599029541015625f,-0.26110000188797710674037944045267067849636077880859375f,1.336200003671645930580780259333550930023193359375f,-0.4122000238180163478318718262016773223876953125f,-0.0683000001132487710719942697323858737945556640625f,-0.5934000157549974563409023176063783466815948486328125f,0.8812000130966310962321585975587368011474609375f,0.199599995659291706839866264999727718532085418701171875f,0.580800001156330125695603783242404460906982421875f,0.2315999857723713972745827049948275089263916015625f,0.06949999484419822071146199959912337362766265869140625f,-0.0793999985739588520772258561919443309307098388671875f,0.305600005821883513590364600531756877899169921875f,0.098400003089010701362582267393008805811405181884765625f,-1.10329996733069446435138161177746951580047607421875f,0.0622000019133093218215435626916587352752685546875f,-0.1193999814748760712745934142731130123138427734375f,0.8580000103712084325024989084340631961822509765625f,0.1411000274062157444632248370908200740814208984375f,0.47959998720288243845288889133371412754058837890625f,0.10160000484585740565535161294974386692047119140625f,0.2570000182390206333593596355058252811431884765625f,-0.42960001760125177128202267340384423732757568359375f,0.16699998709559427112480989308096468448638916015625f,-0.13479996879100841766785379149951040744781494140625f,0.4849999974295495785980847358587197959423065185546875f,-0.3789999907687306990311526533332653343677520751953125f,-0.6422999950356780285432023447356186807155609130859375f,-0.37199999454617493910291159409098327159881591796875f,0.89140000016093257073634958942420780658721923828125f,0.4276000108361248663868536823429167270660400390625f,0.3358000027582048740981690571061335504055023193359375f,-0.1169999975636601685646809301033499650657176971435546875f,-0.3369000007160007914563948361319489777088165283203125f,0.19240000376701349438235411071218550205230712890625f,0.26280000181198115427605443983338773250579833984375f,-0.314999998062848618474163231439888477325439453125f,-0.0428000003963708852694480810896493494510650634765625f,-0.2388000020653009425330992598901502788066864013671875f,0.000599985779822009845929642324335873126983642578125f,0.15160000797510153613956163098919205367565155029296875f,0.12360000118017200332332095058518461883068084716796875f,0.0073999966889619628318541799671947956085205078125f,-0.061200004416704256726688981871120631694793701171875f,-0.117800009113550263606384760350920259952545166015625f,-0.2102000005871058618822644348256289958953857421875f,0.132600000703334774954100794275291264057159423828125f,-0.493200009691715290927049863967113196849822998046875f,-0.3145000203251839820950408466160297393798828125f,-0.47079999534487715351360748172737658023834228515625f,-0.07540001100897786301402447861619293689727783203125f,0.93339995626211180734799199854023754596710205078125f,0.36499998748302431295087444595992565155029296875f,-0.55240000197887439981059287674725055694580078125f,-1.1415000016093248547122129821218550205230712890625f,-0.27659999858736983302520684446790255606174468994140625f,0.27260000293850905617176749728969298303127288818359375f,0.44210000202953825265694831614382565021514892578125f,-0.4965999887898562281662862005759961903095245361328125f,-0.3995999949514865701161170363775454461574554443359375f,0.029999989867210530292140902020037174224853515625f,0.4514000020980830640837666578590869903564453125f,0.53210001417398444800710421986877918243408203125f,0.551100040817260605763294734060764312744140625f,0.7994999526739121620266814716160297393798828125f,0.17169998066425318228311880375258624553680419921875f,-0.3978999986410141076476065791212022304534912109375f,-0.52310001554489105046741315163671970367431640625f,0.00899999609589574589563198969699442386627197265625f,-0.57809998278096330626141252650995738804340362548828125f,0.38570000538900472975001321174204349517822265625f,-0.0261000003047287265633968900147010572254657745361328125f,0.2868999647855758894365862943232059478759765625f,0.128899981236457961131236515939235687255859375f,0.44040000832676895470285671763122081756591796875f,-0.27489999544098975281514185553533025085926055908203125f,0.380499998915940551569292438216507434844970703125f,0.0658999981157481762306105110837961547076702117919921875f,-0.357400020456314138073139474727213382720947265625f,0.443799990069866368003204115666449069976806640625f,0.2810000031590462743480429708142764866352081298828125f,0.008899995243549341239486238919198513031005859375f,-0.23389999025166030577338460716418921947479248046875f,-0.175899999624490710470325893766130320727825164794921875f,0.065899997165799195641966434777714312076568603515625f,0.3118999986857176054400042630732059478759765625f,0.12750000007450579264656198574812151491641998291015625f,-0.2924000061511993209251158987171947956085205078125f,-0.1924000012338158427382950321771204471588134765625f,-0.06810000317692754823184486667742021381855010986328125f,-0.2958000029444693979030489572323858737945556640625f,-0.18559998723268478215686627663671970367431640625f,-0.25370000486373900372427669935859739780426025390625f,1.0226999763727189218798230285756289958953857421875f,-0.6652999916791912937696906737983226776123046875f,-0.322600002044439282400389856775291264057159423828125f,-1.273200011181831570183931034989655017852783203125f,0.0422000041484835719529655762016773223876953125f,-0.1076999999165533239420255995355546474456787109375f,0.423199995461106315275401357212103903293609619140625f,-0.17300000061094777947801048867404460906982421875f,0.04260000420510767771276050552842207252979278564453125f,0.79869997981116191976269647057051770389080047607421875f,0.18500001728534698486328125f,0.53240000913143159522178393672220408916473388671875f,-0.24979999740123748352971233543939888477325439453125f,-0.1022000139832499598924187012016773223876953125f,-0.0377999995768070196078269873396493494510650634765625f,0.1565999966502189533201772064785473048686981201171875f,0.0075000081956384523351744064711965620517730712890625f,0.91760001888275155579322017729282379150390625f,0.4082000129699707713371026329696178436279296875f,-0.9439999694228173865440112422220408916473388671875f,0.51999999865889545702657414949499070644378662109375f,-0.40559999781250954953293330618180334568023681640625f,-1.3107999734401705183017838862724602222442626953125f,0.31679999120235446952165148104541003704071044921875f,-0.41499998152256001304749588598497211933135986328125f,-1.1579999516606331866341861314140260219573974609375f,0.21520000393986704612103721956373192369937896728515625f,-0.30560000228285788548276968867867253720760345458984375f,-0.4532000073820352525899579632095992565155029296875f,0.37939999876022323377355860429815948009490966796875f,-0.82780001969337480005606266786344349384307861328125f,-0.382700032997131955880831810645759105682373046875f,-0.463399994111060919976807781495153903961181640625f,0.970600001835823178453210857696831226348876953125f,0.5767000313282011347837396897375583648681640625f,0.21320001177787784030215334496460855007171630859375f,-0.56780001075267794163892176584340631961822509765625f,0.03659994151592282918272758251987397670745849609375f,0.920799994003772326323087327182292938232421875f,0.23559999915361373723499127663671970367431640625f,-0.90279997320175198183278553187847137451171875f,-0.205800003615021598335488306474871933460235595703125f,-0.45859999767839898510146667831577360630035400390625f,0.055499996103346216358431775006465613842010498046875f,-0.10000000789761553932066817651502788066864013671875f,-0.86380000780224808210050468915142118930816650390625f,-0.0740000434815879515326741966418921947479248046875f,0.75160000126957893229473484097979962825775146484375f,0.21920003207325944316608001827262341976165771484375f,-0.7445999512910841389157212688587605953216552734375f,-0.097200007724762116367855924181640148162841796875f,-0.65300002527236955529588158242404460906982421875f,-0.3786000241279605660338347661308944225311279296875f,0.278999991178512374290221487171947956085205078125f,-0.34279999436140062840649989084340631961822509765625f,0.142400016880035451549701974727213382720947265625f,0.5943999993562698591631487943232059478759765625f,-0.0017000076353550497287869802676141262054443359375f,-0.026099999839067411500082016573287546634674072265625f,0.05729999367594729164920863695442676544189453125f,0.09509999839365479434860617402591742575168609619140625f,-0.867599988484382578235454275272786617279052734375f,1.5591999884128568965024896897375583648681640625f,0.68150000205636018790755770169198513031005859375f,-1.4418999808788299787920550443232059478759765625f,0.3605000139474867637545685283839702606201171875f,0.06210000135898585682525663287378847599029541015625f,-1.33359995737075820443351403810083866119384765625f,0.0684000251054754926371970213949680328369140625f,-0.02529998974204072936800002935342490673065185546875f,-0.462600000777840580923339075525291264057159423828125f,0.0582000133424998011832940392196178436279296875f,-0.08139999978840350980391349366982467472553253173828125f,-0.231599991583824049712347914464771747589111328125f,-0.0685999949216835602783248759806156158447265625f,-0.4012000178277492778988744248636066913604736328125f,0.4319999902248383705227752216160297393798828125f,0.0641999836444853144712396897375583648681640625f,0.44660000723004333877952376496978104114532470703125f,0.2143999761104584678150786203332245349884033203125f,-0.09039998135566662540441029705107212066650390625f,-0.33190000486969939430537124280817806720733642578125f,-0.791399990177154677439830265939235687255859375f,1.4437999486446386754323611967265605926513671875f,0.55839998427629478072731217253021895885467529296875f,0.03130001763701439898568423814140260219573974609375f,-0.62609997338950673650970202288590371608734130859375f,-0.381499996691942289128718357460456900298595428466796875f,-0.04290001115798958153391140513122081756591796875f,-0.772099975430965201894650817848742008209228515625f,-0.6094000021874903527674405268044210970401763916015625f,-0.65929995961189291620030417107045650482177734375f,1.1970999918222418045843369327485561370849609375f,0.47530000777244563181511693983338773250579833984375f,-0.468800011229514979049781686626374721527099609375f,-0.64319999091625135179128847084939479827880859375f,-0.420500011116266225741355810896493494510650634765625f,-0.970399987316131529269114253111183643341064453125f,-0.349999964237213134765625f,-0.21859999566674215287775950855575501918792724609375f,0.83580002693533916868773303576745092868804931640625f,0.2635000308752069031470455229282379150390625f,0.7954000297844412425973814606550149619579315185546875f,0.1650000126659871835244075555237941443920135498046875f,-0.399900013750791316624599858187139034271240234375f,-0.09739999758303163890360565346782095730304718017578125f,-0.12359999704509971873989115920267067849636077880859375f,0.442899969099462553767665440318523906171321868896484375f,-0.70399999117851219665453754714690148830413818359375f,-0.33100000405311558182575026876293122768402099609375f,0.7731999685645103426168134319595992565155029296875f,0.22359999089837057084650950855575501918792724609375f,-0.34260000562071812879594290279783308506011962890625f,-1.0121999865651130079413633211515843868255615234375f,-0.1393999949604272092784640335594303905963897705078125f,0.2776000004053116310132054422865621745586395263671875f,-0.19249999772757242499920948830549605190753936767578125f,0.41080000219941137373780293273739516735076904296875f,-0.20699999722838402504976329510100185871124267578125f,-1.0697999857783315746928565204143524169921875f,0.09139999941587451104396677692420780658721923828125f,0.46320000432729724337832521996460855007171630859375f,-0.282299953740835718463131343014538288116455078125f,-0.509399996227025919637299011810682713985443115234375f,0.188999997138977082755673109204508364200592041015625f,1.0424999599903828073621525618364103138446807861328125f,-0.4213999949455260463082595379091799259185791015625f,0.9692000051021576467746854177676141262054443359375f,0.6429000102639204072829670622013509273529052734375f,-0.41259999593496321512731128677842207252979278564453125f,0.69139998689889903271676985241356305778026580810546875f,0.86569999516308282494492232217453420162200927734375f,0.06280000434517862828442957834340631961822509765625f,0.67659999828934669352520359097979962825775146484375f,-0.10299996462464378055301494896411895751953125f,-0.317800002035498574404215332833700813353061676025390625f,0.554400008073449246381869670585729181766510009765625f,0.609900016619265183948073172359727323055267333984375f,0.4791999957144259525421148282475769519805908203125f,-0.6497999930799007728410288109444081783294677734375f,-1.2331999959826465129708594758994877338409423828125f,-0.5773999916970726786757950321771204471588134765625f,0.19220000295639039222805877216160297393798828125f,0.148099989318847491404085303656756877899169921875f,0.300299997639655202874564565718173980712890625f,0.47050002154707915025255715590901672840118408203125f,-0.17710000538229930810985024436376988887786865234375f,1.2523000091731546490336768329143524169921875f,0.4013000049710273575698238346376456320285797119140625f,0.55429998494386667484690178753226064145565032958984375f,-0.19349999332427980647253207280300557613372802734375f,-0.1765999977119266095559879659049329347908496856689453125f,0.73239999423027057900981162674725055694580078125f,-1.2047999532938007405391545034945011138916015625f,-0.54209999048113832742501472239382565021514892578125f,-1.19290000221729286522531765513122081756591796875f,-0.0040999963521954896350507624447345733642578125f,-0.02380000064969056694508253713138401508331298828125f,-0.3519000269234180411359602658194489777088165283203125f,-0.02249999441206451677999211824499070644378662109375f,0.17720000962466009308826642154599539935588836669921875f,-1.197700001406669656489611952565610408782958984375f,0.3631000113725662004071637056767940521240234375f,0.13160000484585765434530912898480892181396484375f,-0.384799979519844015385388047434389591217041015625f,0.441799984943866519415678340010344982147216796875f,0.3841000005245207571391574674635194242000579833984375f,1.117799966347217566209337746840901672840118408203125f,-0.5231999985158442001420553424395620822906494140625f,-0.1685999964863060807118699813145212829113006591796875f,0.539199966359138471716505591757595539093017578125f,0.1305999797821044694501324556767940521240234375f,0.4483000100970269130584711092524230480194091796875f,0.870299976852536349980482555110938847064971923828125f,-0.0363000273972744480488472618162631988525390625f,0.22869999435842036650257114160922355949878692626953125f,-0.186299997520446680709937936626374721527099609375f,0.568099975907802701158288982696831226348876953125f,0.4959999985396861266195855932892300188541412353515625f,0.5926000004425644451799826128990389406681060791015625f,-0.0172000140205028628770378418266773223876953125f,0.199300004650652395898902113913209177553653717041015625f,-1.329699994611740265781918424181640148162841796875f,0.1323000230312345593119971454143524169921875f,-0.12799999442696563534127562888897955417633056640625f,-0.00250000536441774556806194595992565155029296875f,0.0863999797463410601494615548290312290191650390625f,0.616199982810019974976967205293476581573486328125f,-0.059600006127357829655011300928890705108642578125f,-0.0878999822497377181207411922514438629150390625f,-0.6510000016689299062733198297792114317417144775390625f,-0.1820000092983244943667386905872263014316558837890625f,0.4920999833285808477967293583787977695465087890625f,-0.71640001789331453352360767894424498081207275390625f,-0.20420002208948151434242390678264200687408447265625f,0.3269999881386755902212826185859739780426025390625f,-0.774000021725893105895011103712022304534912109375f,0.044200002568960172766310279257595539093017578125f,1.04900001025199873794235827517695724964141845703125f,0.627600027078390354517978266812860965728759765625f,-0.016600006633996855498480726964771747589111328125f,-0.9990000063776971472861987422220408916473388671875f,0.418400014376640427826714585535228252410888671875f,-0.282600002491474100452251150272786617279052734375f,-0.729600016409158502739273899351246654987335205078125f,-0.1660000037550926066387546597979962825775146484375f,-0.4554000108599662866026847041212022304534912109375f,-0.577000004827976198384931194595992565155029296875f,-0.4082000190794470295685414384934119880199432373046875f,0.6944000021874903172403037387994118034839630126953125f,0.6867000344574449854917475022375583648681640625f,-0.22980000701248659566999776870943605899810791015625f,-0.602400018146634241844594725989736616611480712890625f,0.094699957470596007880203615059144794940948486328125f,0.206799997013807246304395448532886803150177001953125f,-0.149399991160631184783369462820701301097869873046875f,-0.444599993610381716280244290828704833984375f,0.24959999673962585120534640736877918243408203125f,-0.528799998563528106387821026146411895751953125f,-0.02280001820325860961702346685342490673065185546875f,-0.59599999485164867341069339090608991682529449462890625f,-0.0008000184789299369381154747316031716763973236083984375f,0.4791999778889117767022298721713013947010040283203125f,-0.47160000812411306814198042047792114317417144775390625f,0.03559999989867212999428147668368183076381683349609375f,0.473000001952052162579320793156512081623077392578125f,-0.4208000100970268331224133362411521375179290771484375f,-0.0355999937891959827851451336755417287349700927734375f,0.4703999999821188993109899456612765789031982421875f,-0.2237999934971330873167971731163561344146728515625f,-0.4645000133216381499323688331060111522674560546875f,0.1780000389814375694186310283839702606201171875f,0.8549999439716344795670011080801486968994140625f,0.15069997273683544136702039395458996295928955078125f,0.36819998868107806089255973347462713718414306640625f,-1.0443999762594700086992816068232059478759765625f,-0.3842999955236912068556875965441577136516571044921875f,0.12219998030662537757962127216160297393798828125f,-1.170600001835822467910475097596645355224609375f,-0.3947000054895877241278867586515843868255615234375f,1.0201000200748442381382119492627680301666259765625f,-1.01510000755786933268609573133289813995361328125f,-0.34540000117421154612884492962621152400970458984375f,-1.025500016927718860415552626363933086395263671875f,0.824100018405914624963770620524883270263671875f,0.31460000270009036427154569537378847599029541015625f,-0.73580001456737509357708404422737658023834228515625f,0.349400020515918896535367821343243122100830078125f,0.033100003325939209020134512684307992458343505859375f,-0.786200010228156997982296161353588104248046875f,-0.6197999753475187389994971454143524169921875f,-0.276699996906518919104200904257595539093017578125f,0.63550000783801063874989267787896096706390380859375f,-0.0431000446021556626874371431767940521240234375f,0.2577999951064586792170985063421539962291717529296875f,0.08879997770190239947396548814140260219573974609375f,-0.76099998922645983867596441996283829212188720703125f,-0.492500006854534044808957560235285200178623199462890625f,-0.4734999971985818234543330618180334568023681640625f,0.158500005543231736737652681767940521240234375f,0.021800004762411173686587062547914683818817138671875f,0.1483000169515609201198458322323858737945556640625f,-0.01409999396800998283652006648480892181396484375f,-0.286100006544590002022232511080801486968994140625f,0.35549997540935895923297493936843238770961761474609375f,-0.83489998415335975323614547960460186004638671875f,-0.2424999993480742299656327531920396722853183746337890625f,0.401000000849366156074893297045491635799407958984375f,-0.626599996426701721929930499754846096038818359375f,-0.18059999714195729136889667643117718398571014404296875f,0.42960000299811351709422524436376988887786865234375f,-0.60319999434351956324462662450969219207763671875f,-0.2141999948203563430837448322563432157039642333984375f,1.19769999783039082785762730054557323455810546875f,-0.7161999777436260927743205684237182140350341796875f,0.19560000448077918822065157655742950737476348876953125f,0.03380000325739385136358805539202876389026641845703125f,-0.56059999611750210846139452769421041011810302734375f,0.09000000074505809521951960050500929355621337890625f,-0.41179999939799305508358884253539144992828369140625f,-0.863000004559754785304903634823858737945556640625f,0.19760000427961355029538026428781449794769287109375f,0.64600000971555715523209073580801486968994140625f,-0.23439995591938522778718834160827100276947021484375f,0.48859999507069584723240041057579219341278076171875f,-0.74259999310374258385536450077779591083526611328125f,-1.2044999836683274452298064716160297393798828125f,0.24520000915527351281752999057061970233917236328125f,-0.41000000298023220324239446199499070644378662109375f,-0.28460002579689014368113930686376988887786865234375f,0.540199989336728858546621268033050000667572021484375f,-0.314999988675117581493623220012523233890533447265625f,-0.8262999730080367033480115424026735126972198486328125f,0.486200016784668065383812063373625278472900390625f,0.354000016212463464171378291212022304534912109375f,0.3334999870657924248007475398480892181396484375f,0.36619999856799834159204465322545729577541351318359375f,0.0458000172123313176175685157431871630251407623291015625f,-0.2728999823428688298321276306523941457271575927734375f,-0.1005999881267545248419992276467382907867431640625f,0.5914000187873842406816038419492542743682861328125f,0.26850001150369706692799809388816356658935546875f,-0.2367999977588652882332098670303821563720703125f,0.380799990725517290002244408242404460906982421875f,0.05440001842975572543537055025808513164520263671875f,-0.1293999995052814711016253568232059478759765625f,0.4956000004947185288983746431767940521240234375f,0.7637000039696690834034598083235323429107666015625f,-0.1761999948054551179410509575973264873027801513671875f,0.0899999953806400743161475475062616169452667236328125f,0.09269999168366205299918192395125515758991241455078125f,0.5120000310540202548281740746460855007171630859375f,-0.0644000002503393176311874412931501865386962890625f,0.00639996901750539137765372288413345813751220703125f,-0.60520000677108765074763141456060111522674560546875f,0.05259999176263807640907543827779591083526611328125f,0.771399984663724946898355483426712453365325927734375f,0.156999995589256258199384319595992565155029296875f,0.127299997401237430239007153431884944438934326171875f,-0.65350001411139968521268883705488406121730804443359375f,0.263300006408989428763334217364899814128875732421875f,0.160600006548315354937273014002130366861820220947265625f,-1.049900015687942556041889474727213382720947265625f,-0.237099971258640351834401371888816356658935546875f,-0.1313999900281428434567487784079276025295257568359375f,-0.2934999864697456217754734097979962825775146484375f,0.7328999812066552266287544625811278820037841796875f,0.53310000332593909799783205016865395009517669677734375f,-1.276499970912933434874503291212022304534912109375f,0.082100026392936342745088040828704833984375f,-0.19289999759793285960540742962621152400970458984375f,-0.2136000068426131548449120600707828998565673828125f,0.0420000194311143104641814716160297393798828125f,-0.168400000071525557387985827517695724964141845703125f,-0.802999968349933812561403101426549255847930908203125f,0.4967999940335747766084750765003263950347900390625f,0.0601999973088502837725854988093487918376922607421875f,0.6255999994516372453290387056767940521240234375f,0.9965999886035916688342695124447345733642578125f,0.33530000151395800145337489084340631961822509765625f,-0.19529998134151116406798109892406500875949859619140625f,0.54729999496117187618438038043677806854248046875f,0.1769000034354627215460453726336709223687648773193359375f,0.23839998874664303230019868351519107818603515625f,0.29900000607967314181223628111183643341064453125f,0.336800016385316869360622149542905390262603759765625f,-0.0832000046253202896195944049395620822906494140625f,-0.0704000075817106107933796010911464691162109375f,0.183999999225139543312934620189480483531951904296875f,0.890500003814697294046709430404007434844970703125f,0.2306999736309052195792901329696178436279296875f,0.1915999942660331356591996154747903347015380859375f,0.0297999838411808770644029209506697952747344970703125f,-0.18639999278485763767321259365417063236236572265625f,-0.00289999666661022725833163349307142198085784912109375f,0.2545999971866610422921439749188721179962158203125f,0.70660001900196078850058256648480892181396484375f,0.10359999909400929851699402206577360630035400390625f,0.73099998736381532893346957280300557613372802734375f,-0.785000000298023081768405972979962825775146484375f,-0.250700000751018503564182537957094609737396240234375f,-0.1775999777555465897194153512828052043914794921875f,0.110600016885995966475775276194326579570770263671875f,0.272399987524747810851977192214690148830413818359375f,-0.875599975758790893820560086169280111789703369140625f,-0.5761999968171114261394905042834579944610595703125f,0.46700002107024207731456044712103903293609619140625f,0.28600000748038301168207908631302416324615478515625f,0.257099981987476411404713871888816356658935546875f,0.77780001641511919974192323934403248131275177001953125f,0.14840000573396683325455569502082653343677520751953125f,-0.57469999856054787557013696641661226749420166015625f,0.318400007820129449243040653527714312076568603515625f,-0.199600001060962650711871901876293122768402099609375f,-0.615200003343820522871965295053087174892425537109375f,-0.183399981743097129083253093995153903961181640625f,0.605199981588125268672229140065610408782958984375f,-0.053700004565715886428733938373625278472900390625f,0.375400010561943275888552307151257991790771484375f,-0.741199989962577632240936509333550930023193359375f,-0.8259000116944310576627685804851353168487548828125f,-0.63460000120997417383250649436376988887786865234375f,-0.39220002366900441614916417165659368038177490234375f,0.8194999699592582231844062334857881069183349609375f,0.501800019961595733519743589567951858043670654296875f,-0.138000006794929408471261922386474907398223876953125f,-1.0258999980598690893174307348090223968029022216796875f,-0.70360002174377456185538903810083866119384765625f,-0.11760000696182260071509517729282379150390625f,0.491400004482269281425033113919198513031005859375f,-0.18640000209808349485030021241982467472553253173828125f,-0.07360000535249711095531210958142764866352081298828125f,0.144799993750452937746331372181884944438934326171875f,-0.11439999443888648755773829179815948009490966796875f,0.72140002430081384687809986644424498081207275390625f,0.7196000192403797512952223769389092922210693359375f,0.87859999775290464896215780754573643207550048828125f,-0.03659998636841788055562574299983680248260498046875f,-1.2430999791860575243163111736066639423370361328125f,0.314800001573562582279919297434389591217041015625f,0.1860000070333480692852390347979962825775146484375f,-0.51939998072981818921789454179815948009490966796875f,-0.9162000367522242783024921664036810398101806640625f,-0.1398000024676324670735994004644453525543212890625f,1.042300008130073507572888047434389591217041015625f,-0.39600001627206804499792269780300557613372802734375f,0.135400003558397319380901535623706877231597900390625f,-0.5314999640583994988674021442420780658721923828125f,-0.2766999660611162425993825308978557586669921875f,-0.437099990630149815018512526876293122768402099609375f,0.523699995923042393997093313373625278472900390625f,0.9130999934911727677899762056767940521240234375f,0.294199998843669874304396216757595539093017578125f,-0.414299998429417559719922792282886803150177001953125f,1.1994999902993441764920135028660297393798828125f,0.40999999724328517469729149524937383830547332763671875f,-0.64100000435113901886552412179298698902130126953125f,0.271800012660026624189413269050419330596923828125f,0.044400001889467244353681962820701301097869873046875f,-0.22750004410743684957196819595992565155029296875f,0.159499996185303416496026329696178436279296875f,0.368600002372264867744888761080801486968994140625f,-0.829199987816810590857130591757595539093017578125f,-0.0505999511718755456968210637569427490234375f,-0.27929998971223835013688585604541003704071044921875f,0.83310000243186976831566425971686840057373046875f,-1.1426999835252757975467829965054988861083984375f,-0.57050000992417349010565885691903531551361083984375f,-1.105400007283687369152858082088641822338104248046875f,0.5500000210106374964880160405300557613372802734375f,0.1989000024706124580831101411604322493076324462890625f,0.33520000081062306662715855054557323455810546875f,-1.0756000024318694840985699556767940521240234375f,-0.35540000102519986313609479111619293689727783203125f,0.113099995949864418065544668934307992458343505859375f,-0.314099996724724661589789320714771747589111328125f,-0.12530000099241733746424642959027551114559173583984375f,0.81270000736713399192012730054557323455810546875f,0.41069998912811200852956972084939479827880859375f,0.356800011318922027925282236537896096706390380859375f,-1.22869999771118187936735921539366245269775390625f,1.15749998390674591064453125f,0.408900001055002348948619328439235687255859375f,-0.5613999991178513226941504399292171001434326171875f,0.550999992430210028260262333787977695465087890625f,0.2934000030517578405664380625239573419094085693359375f,0.9666000172138211610217695124447345733642578125f,-1.356400017595291274119517765939235687255859375f,-0.5116000057399272549218949279747903347015380859375f,-0.223399976825714219330620835535228252410888671875f,-0.42129997040033373423284501768648624420166015625f,0.36120000441670419011330750436172820627689361572265625f,-0.64320000626444817637406003996147774159908294677734375f,-0.80900001539289956298262040945701301097869873046875f,-0.26219999863505361048510167165659368038177490234375f,0.22140000582337382439845896442420780658721923828125f,0.89529997737407729374581322190351784229278564453125f,0.05239999199509615124270567321218550205230712890625f,-0.84540000862479214305267305462621152400970458984375f,-0.0586000160813338055731946951709687709808349609375f,0.270000009350478631375835902872495353221893310546875f,0.7212000067636370470580686742323450744152069091796875f,-0.2533999854125077444422231565113179385662078857421875f,-0.13960000925660143167306159739382565021514892578125f,0.65920000376105303274698599125258624553680419921875f,-0.02619995241165185007048421539366245269775390625f,0.57200002166628838295991954510100185871124267578125f,0.20439998490214339454951186780817806720733642578125f,0.266399966335296056740844505839049816131591796875f,-0.30239999493807545594137309308280237019062042236328125f,0.02739999698698525332218878247658722102642059326171875f,0.67159996962323820213214276009239256381988525390625f,-0.24239999527335154283491647220216691493988037109375f,0.74400001397728932062136664171703159809112548828125f,0.9920000131726265379938922706060111522674560546875f,-0.56540003872513810989630655967630445957183837890625f,-0.21900001159310367171428879373706877231597900390625f,-0.2946999588489536137103641522116959095001220703125f,-0.30499999716877912447898779646493494510650634765625f,-0.20799998906254746344757222686894237995147705078125f,-0.1696999969959254173090812400914728641510009765625f,0.31500000998377808425487955901189707219600677490234375f,-0.25520000125765796017418551855371333658695220947265625f,-0.410500012233853350807066817651502788066864013671875f,0.26680001087188731645483130705542862415313720703125f,-0.10420000405907625662393911625258624553680419921875f,-0.5506000007182361155599892299505881965160369873046875f,-0.7039999959468838941489821081631816923618316650390625f,0.0305999964714052108405439867055974900722503662109375f,0.6843999860942364676930083078332245349884033203125f,0.004399997419118673036564359790645539760589599609375f,0.652399994081258594036398790194652974605560302734375f,0.7137999993085859529173831106163561344146728515625f,-0.32520000289678563376583042554557323455810546875f,-0.010099997276067806950550220790319144725799560546875f,-0.824999999776482528801579974242486059665679931640625f,0.0002000238329173953388817608356475830078125f,-0.21880000146925449033830091138952411711215972900390625f,1.0346999874591826795722226961515843868255615234375f,-0.1495000063180924598782439716160297393798828125f,-0.053499997943639687747463540290482342243194580078125f,0.046300030303001449283328838646411895751953125f,-0.533699982810020401302608661353588104248046875f,-0.55480001096129427651248988695442676544189453125f,-0.293700025799870534370938912616111338138580322265625f,0.891499977134168286596604957594536244869232177734375f,0.6083000057756902112515717817586846649646759033203125f,-0.177099991226196351590260746888816356658935546875f,0.26489997098445883239037357270717620849609375f,0.41430000073909756252987790503539144992828369140625f,0.594499987840652721615697373636066913604736328125f,1.009699974942208200445747934281826019287109375f,0.26749998241662975573262883699499070644378662109375f,0.74319998070895676445246635921648703515529632568359375f,-0.366799993474781427238440301152877509593963623046875f,-0.165699998310953333646722285266150720417499542236328125f,1.11030000658035277183444122783839702606201171875f,0.26229998130798293232146534137427806854248046875f,0.295900004243850656138192789512686431407928466796875f,-0.6270999828815462251441203989088535308837890625f,-1.0129000177145002226097858510911464691162109375f,-0.27150000652670858158899136469699442386627197265625f,-0.3419000017404556501787737943232059478759765625f,-0.61169995980262825696627260185778141021728515625f,-0.206399982428550732294070257921703159809112548828125f,-0.365600002059340400961673367419280111789703369140625f,0.244600014397502008023366215638816356658935546875f,-0.02409999970495702026340723023167811334133148193359375f,-0.590900004243850585083919213502667844295501708984375f,0.2699000125586987763881552382372319698333740234375f,0.0854000012487172721620254378649406135082244873046875f,0.57259998744130147230180227779783308506011962890625f,-0.9533999733984472868542070500552654266357421875f,-0.2818999907135963933768607603269629180431365966796875f,0.8684999894499778605450046597979962825775146484375f,0.53529995085001047527839546091854572296142578125f,0.344899987673759511608295724727213382720947265625f,0.253400000667571845269776531495153903961181640625f,-0.305599968643486707975398530834354460239410400390625f,0.003200001309811906224922495312057435512542724609375f,0.3393999891862271578446552666719071567058563232421875f,0.7171999761901795000795800660853274166584014892578125f,-0.263199994939565584672891418449580669403076171875f,0.641399984365701669730697176419198513031005859375f,0.3284000230193129254985251463949680328369140625f,0.232600001746416040759868337772786617279052734375f,-0.346800011914968564497030456550419330596923828125f,-0.51530000836849243484039106988348066806793212890625f,-0.67560000675320619478725348017178475856781005859375f,-0.09280000374913210947624975233338773250579833984375f,0.923399979805946458100152085535228252410888671875f,-0.509000017255544889849261380732059478759765625f,0.593600004011392456959583796560764312744140625f,1.1547000068306925157912701251916587352752685546875f,-0.5127999971926211486561442143283784389495849609375f,0.5467999833047390012552568805404007434844970703125f,0.5544000082969660070375539362430572509765625f,-0.50980000261664404348493917495943605899810791015625f,0.58799998608231529573231455287896096706390380859375f,1.354299971234798505292928894050419330596923828125f,0.336399981236457534805595059879124164581298828125f,-0.342199990141391907627621549181640148162841796875f,-0.400199980098009167051031909068115055561065673828125f,-0.60580002827644374718829567427746951580047607421875f,0.18060000764727579536383927916176617145538330078125f,0.9187000161141158915967253051348961889743804931640625f,0.0457999844670293754234080552123486995697021484375f,-0.5898000203490258996907869004644453525543212890625f,-1.0214999786615379662180203013122081756591796875f,0.300599988722801025886610659654252231121063232421875f,-0.406799992394447418320169163052923977375030517578125f,-0.352899995064735350069895503111183643341064453125f,-0.2286000101208689461174117241171188652515411376953125f,0.4843999841570851838667977062868885695934295654296875f,0.6529000017702575320299729355610907077789306640625f,-0.7314000341355804124532369314692914485931396484375f,0.2199999971687791600061245844699442386627197265625f,0.59230002839565276673283733543939888477325439453125f,0.48940000010132767016557409078814089298248291015625f,-0.00459999822974221928006954840384423732757568359375f,0.1998999787330628663539755507372319698333740234375f,-0.5584000259995463011364336125552654266357421875f,0.26040001968890436057080250975559465587139129638671875f,-1.30529998512268008425962761975824832916259765625f,-0.068199990618229122674165409989655017852783203125f,-1.04529997841715793782668697531335055828094482421875f,0.43950000572204572790724341757595539093017578125f,0.56799996730685275991845628595910966396331787109375f,0.23169996714144946992064433288760483264923095703125f,-0.7217000030159947954189192387275397777557373046875f,-0.295399978673458463163115084171295166015625f,-0.6096999867886305768394095139228738844394683837890625f,0.646100035452843002303779940120875835418701171875f,-0.45839999813437426467999102897010743618011474609375f,0.4211000358626251216520586240221746265888214111328125f,0.3290000013113019150523541611619293689727783203125f,0.30439999577999099500402735429815948009490966796875f,0.1241999830484388223084124547312967479228973388671875f,0.03220001517534232249317938112653791904449462890625f,0.661999967575073622327863631653599441051483154296875f,0.02019998799562483515757094210130162537097930908203125f,0.1061000120580199990172332036308944225311279296875f,0.490799987894296574353347750729881227016448974609375f,0.2270999955847858853275056389975361526012420654296875f,-0.1586999664187427327988189063034951686859130859375f,0.93519996758103385303684262908063828945159912109375f,0.0002999982729556815996829755022190511226654052734375f,0.10360000818371783992688506259582936763763427734375f,-0.1816000002264974710897149634547531604766845703125f,0.13880001547634623904059480992145836353302001953125f,0.45019997413754442305844349903054535388946533203125f,-0.2233999841272833464245195500552654266357421875f,0.5327999972298738118325900359195657074451446533203125f,1.05789999417066571396617291611619293689727783203125f,0.289800000977515903599623925401829183101654052734375f,1.016699988785385588840881609939970076084136962890625f,-0.2411999878764152782650853623636066913604736328125f,-0.061600012445449436881972360424697399139404296875f,-0.388400001040100750770989179727621376514434814453125f,0.3943000114679335865730536170303821563720703125f,0.4174000028729434319529900676570832729339599609375f,0.22229999226033658743517662514932453632354736328125f,0.69120000888705224184604958281852304935455322265625f,-0.20980000619292216157418806687928736209869384765625f,0.76460002933591653118128306232392787933349609375f,-1.2100000104308126225305386469699442386627197265625f,-0.650800011885166451719442193279974162578582763671875f,-0.3983999981343746554784956970252096652984619140625f,-0.318899990549683653995316490181721746921539306640625f,-0.4819999955892562137904633345897309482097625732421875f,-0.0601999833762647540424950420856475830078125f,-0.5477999708175662618714341078884899616241455078125f,-0.37009998148083678159991904976777732372283935546875f,0.28999999985098856569720737752504646778106689453125f,0.32989998603463188686646390124224126338958740234375f,0.517199995356798059020775326644070446491241455078125f,0.5191999823033810468331239462713710963726043701171875f,1.047699961620569553133464069105684757232666015625f,0.4397999808609487359944978379644453525543212890625f,0.166199982511997301770634294371120631694793701171875f,0.7195999672353270426583549124188721179962158203125f,0.31769999015629313277031542384065687656402587890625f,-0.13640000135302532413561493740417063236236572265625f,0.4081999868929386821037041954696178436279296875f,-0.2716000001519915940662031061947345733642578125f,0.136600022578239332915472914464771747589111328125f,-0.46939997298121483026989153586328029632568359375f,-0.41989998521506777251488529145717620849609375f,-0.4728000091135502458428163663484156131744384765625f,-0.49249999880790706896505071199499070644378662109375f,-0.103399994111061044321786539512686431407928466796875f,0.1626000045776366531580237051457515917718410491943359375f,0.71710001162216041592500914703123271465301513671875f,0.1933000011935828155174021958373486995697021484375f,0.25280000002384195312288284185342490673065185546875f,0.2354999913722277682381900376640260219573974609375f,0.315399994990229526337088827858678996562957763671875f,-0.0418000084877012678674645940191112458705902099609375f,-0.21900001882016628673710556540754623711109161376953125f,0.208499993398785610221324304802692495286464691162109375f,0.4307999972820280021323924302123486995697021484375f,0.41520000647306432028926792554557323455810546875f,0.02699998560547811621290748007595539093017578125f,0.33439999011754995006384660882758907973766326904296875f,0.757399976535141572497877859859727323055267333984375f,0.36639999226331720905136535293422639369964599609375f,0.218800014209747484272838846663944423198699951171875f,-0.1896999998271464704657773836515843868255615234375f,-0.08660000030100345469463718472979962825775146484375f,-0.13599998855590822444128207280300557613372802734375f,0.0886999964445827071557459930772893130779266357421875f,1.0189000024706122982109945951378904283046722412109375f,0.10659999188184754448371904800296761095523834228515625f,0.90929999902546410073256311079603619873523712158203125f,-0.11440001246929210054759096237830817699432373046875f,0.545399966901541244368445404688827693462371826171875f,-0.01860002885907874325965849493513815104961395263671875f,-0.863900015062093462603343141381628811359405517578125f,-0.3189999753832821749455206372658722102642059326171875f,-0.7711000012546780357780562553671188652515411376953125f,-0.07649998745322239557253851671703159809112548828125f,-0.0893999966740608442705706693232059478759765625f,-0.2584999931752680613072925552842207252979278564453125f,0.2653999986410138234305122750811278820037841796875f,-0.369799990099668463017223984934389591217041015625f,0.104000009246170588284030600334517657756805419921875f,1.2961999863862985193918575532734394073486328125f,0.0163999973297119794324316899292171001434326171875f,1.008199982646107439876459466177038848400115966796875f,-0.2712000146985058535165080684237182140350341796875f,0.88339996088147199060358616407029330730438232421875f,-0.1814000446781516284744384392979554831981658935546875f,0.2812999981164931995181177626363933086395263671875f,-0.43619999115467056327588579733856022357940673828125f,0.3505000219941141192947497984278015792369842529296875f,0.46910000978186749964748969432548619806766510009765625f,-0.21599999291449789584618201843113638460636138916015625f,0.267700003697723065698710342985577881336212158203125f,-0.6123000041812656224493593981605954468250274658203125f,-0.08979998846054104699732079097884707152843475341796875f,-0.5535000009983779278854854055680334568023681640625f,-0.5896000078409908695675767376087605953216552734375f,-0.2629999814629557786105351624428294599056243896484375f,-0.4397999890938402156592701430781744420528411865234375f,-0.1979999513626093943230443983338773250579833984375f,0.49319997929334657982280987198464572429656982421875f,-0.2687999850779767552211296788300387561321258544921875f,-0.0598000014618040864888826035894453525543212890625f,-0.18839999507963656188991308226832188665866851806640625f,-0.1845999997571110551319861770025454461574554443359375f,-0.538700018870830632522483938373625278472900390625f,0.49899999475479095423224862315692007541656494140625f,-0.4399000324517492455100864390260539948940277099609375f,0.29140000805854793242133382591418921947479248046875f,0.3951999960422514224234191715368069708347320556640625f,0.1196000079154968809280745745127205736935138702392578125f,-0.139799996116012348323920377879403531551361083984375f,0.0136000017389654725974423854495398700237274169921875f,-0.01440000606179214770463659078814089298248291015625f,-0.028800020915269453070095551083795726299285888671875f,-0.4514000292927027313538701491779647767543792724609375f,0.04200000751018517775037253159098327159881591796875f,-0.289899989312887296222243094234727323055267333984375f,-0.1685999945491551965659482448245398700237274169921875f,-0.28979999457001692775293122394941747188568115234375f,-0.4699999818205835566686801030300557613372802734375f,0.25550002184510223202096312888897955417633056640625f,0.2046000155150888843991197063587605953216552734375f,0.208600022637843718342764987028203904628753662109375f,-0.1756999882340430918503670909558422863483428955078125f,-0.3374000192642210294025062466971576213836669921875f,0.07239997053742452948199570528231561183929443359375f,-0.513300027009844672676308618974871933460235595703125f,0.140199998277425574855215018033050000667572021484375f,0.77310001181960075200549908913671970367431640625f,0.26330000421106802832582616247236728668212890625f,-0.143399990385770781387009265017695724964141845703125f,-0.51059998514652260581669906969182193279266357421875f,-0.182099995249509927219833116396330296993255615234375f,-0.4685999834477900805040917475707828998565673828125f,-0.60049997565150281531032305792905390262603759765625f,-0.471099986949563120930406512343324720859527587890625f,-0.40399999833106992053188832869636826217174530029296875f,-1.0043999976053832323685810479219071567058563232421875f,-0.42559999259710290520075659514986909925937652587890625f,0.0763999996393918134884870596579276025295257568359375f,-0.0214999978467822483452209780807606875896453857421875f,-0.08159999966770403734273031659540720283985137939453125f,-0.347799984377622717346412173355929553508758544921875f,-0.9460999698877337049651714551146142184734344482421875f,-0.65949998702108858505965827134787105023860931396484375f,-0.2513999913692475107751533869304694235324859619140625f,-0.606899979835748837331266258843243122100830078125f,-0.2966999907970429006809354177676141262054443359375f,0.177199999231099969421165951644070446491241455078125f,0.3037999960303305879705249026301316916942596435546875f,-0.13519999581873387572983347126864828169345855712890625f,-0.048800013613700787828975080628879368305206298828125f,-0.92659998160004608536155501496978104114532470703125f,-0.44700000924244520650319145715911872684955596923828125f,0.20599999798089252411870120340608991682529449462890625f,-0.35820001410618440562672049054526723921298980712890625f,0.71200003492832220075570148765109479427337646484375f,0.562000002443789892225822768523357808589935302734375f,0.57480000932216623876769290291122160851955413818359375f,-0.23770000319480910633274106658063828945159912109375f,0.365799982976913629073578704264946281909942626953125f,-0.16210001439750187923749535912065766751766204833984375f,-0.5946000308632850561707527958787977695465087890625f,-0.62299998429417602352486937888897955417633056640625f,-0.3787999970361590751366520635201595723628997802734375f,0.227599995636939667065234971232712268829345703125f,0.47099997231364287841870464035309851169586181640625f,0.12819997083693746020571779808960855007171630859375f,1.5458000363230706142303461092524230480194091796875f,0.12100000688433620865680495626293122768402099609375f,1.2760000271126628046403084226767532527446746826171875f,0.480300014925003182497675879858434200286865234375f,-0.35599999883770916397907058126293122768402099609375f,0.2733000131890179762450543421437032520771026611328125f,-0.2124000043630598355548499966971576213836669921875f,0.168599999541044098805286921560764312744140625f,-0.08940000587552798805290876771323382854461669921875f,0.70589998159408562372618689551018178462982177734375f,0.071200003522634336405872090836055576801300048828125f,0.70209998403638562702866465770057402551174163818359375f,-0.0892000160917643114544262061826884746551513671875f,0.23279998936951173948983750960906036198139190673828125f,0.104399991197884123295125391450710594654083251953125f,-0.03809999345392001135479631557245738804340362548828125f,0.226199985492229693750942942642723210155963897705078125f,-0.038900003103911673374426527516334317624568939208984375f,0.2746999971449373134646521066315472126007080078125f,0.598399973398447304617775444057770073413848876953125f,0.3804999814257026269359585057827644050121307373046875f,0.02309999751448632565598018118180334568023681640625f,0.17999999120831500221129317651502788066864013671875f,0.12569999557286504199282717308960855007171630859375f,0.3541999963104718318618324701674282550811767578125f,0.349199971425533828295328930835239589214324951171875f,0.2221999799340965697780347909429110586643218994140625f,0.8030000035166739991154827293939888477325439453125f,-0.7776000016719104568352349815540947020053863525390625f,-0.22640000478029254082912302692420780658721923828125f,-0.28699999924004071516492331284098327159881591796875f,-0.31060000645518304196457393118180334568023681640625f,-0.22260000740885743653052486479282379150390625f,-0.96070000417828538274989114142954349517822265625f,-0.59570000768005826330409036017954349517822265625f,-0.421400000011920905507167844916693866252899169921875f,-0.40339999247193336628924953402020037174224853515625f,-0.368500000998377874594780223560519516468048095703125f,0.00560000496506685152553473017178475856781005859375f,-0.195999994069337990509893643320538103580474853515625f,0.3478000227481128359130480021121911704540252685546875f,-0.02960000061392786818714739638380706310272216796875f,-0.249599999049305854015301520121283829212188720703125f,-0.3416000083476304904905873627285473048686981201171875f,0.545200008559227011772918558563105762004852294921875f,0.61499999940395344566468338598497211933135986328125f,0.442600003087520565969725794275291264057159423828125f,-0.44660000156760215617168796597979962825775146484375f,-0.55599999600648875031083662179298698902130126953125f,-0.538500001072883538455471352790482342243194580078125f,0.22519999768137921591915073804557323455810546875f,-0.30310000064373010530971441767178475856781005859375f,0.050799996685981785304875302244909107685089111328125f,0.040999984383583154556873751062084920704364776611328125f,0.6026999716416003849417393212206661701202392578125f,0.126299989250302502341583021916449069976806640625f,-0.136000004053115741697865814785473048686981201171875f,-0.4217000113606450550918225417262874543666839599609375f,-0.20280000680387011424699039707775227725505828857421875f,-0.2945999928653240118592293583787977695465087890625f,-0.68159998323917392326620756648480892181396484375f,-0.491300004374981114096954115666449069976806640625f,0.1481999947905541237247462049708701670169830322265625f,0.27309998842477811731299652819870971143245697021484375f,0.270499992340803074863941901639918796718120574951171875f,0.173999997437000286648611790951690636575222015380859375f,0.3424999920651317086850440318812616169452667236328125f,0.08619999562501912127032710486673749983310699462890625f,0.14479998384118086818261872394941747188568115234375f,0.637899975842237676459944850648753345012664794921875f,0.5733999921739101335305122120189480483531951904296875f,0.15739998663067833462037015124224126338958740234375f,0.83720000347793088746328749039093963801860809326171875f,0.4798000198274852134971979467081837356090545654296875f,0.13919999586343767550289385326323099434375762939453125f,0.4498000141650440042440095567144453525543212890625f,-0.0758999777197839620157537865452468395233154296875f,0.71119996865391765794583989190869033336639404296875f,0.0839000054135923267750740706105716526508331298828125f,0.7504000087738038615725599811412394046783447265625f,-0.0473999937832356721401083632372319698333740234375f,0.707800011907517845344273155205883085727691650390625f,-0.8656000438570981003749693627469241619110107421875f,-0.37559999974966018498889752663671970367431640625f,-0.6700000183284282595508329904987476766109466552734375f,0.2312000208824873670465649411198683083057403564453125f,0.46559998514652256584867018318618647754192352294921875f,0.2184000020831824706579737949141417630016803741455078125f,-0.77620001499652868659495652536861598491668701171875f,0.076799998056888529873731386032886803150177001953125f,-0.77100002275407331087109241707366891205310821533203125f,0.84180000848770131227638557902537286281585693359375f,0.176999993652105391817030977108515799045562744140625f,0.80240000574141745470768682935158722102642059326171875f,-0.94209998824596397071218234486877918243408203125f,0.3677999824404718509640588308684527873992919921875f,-0.750099998393654487927051377482712268829345703125f,0.6251000036463139064579763726214878261089324951171875f,-0.2803999912649393078112325383699499070644378662109375f,0.4233000100225210449167434489936567842960357666015625f,-0.05909999948143962456015287898480892181396484375f,0.179399999952316147044939498300664126873016357421875f,0.021700001227855647556452822755090892314910888671875f,0.303400019443035517952012014575302600860595703125f,0.3849999962747094883752652094699442386627197265625f,0.364800006528198661470696606556884944438934326171875f,-0.2948999963164327109410578486858867108821868896484375f,-0.20239998588562035930493721025413833558559417724609375f,-0.28909999042898426591108318461920134723186492919921875f,0.0603000033393501500000866144546307623386383056640625f,0.0505999975889921405070026594330556690692901611328125f,-0.01690000252276657732863895944319665431976318359375f,-0.231499999672174272546953943674452602863311767578125f,-0.7033999732494355949796727145439945161342620849609375f,-0.280699976164102527054211577706155367195606231689453125f,-1.3718000123620033292581865680404007434844970703125f,-0.064999995306134206174419887247495353221893310546875f,-0.45200000631809233997415731209912337362766265869140625f,-0.41810000086724752321742926142178475856781005859375f,-0.151199995103478368463356673601083457469940185546875f,0.087600002193450876575298025272786617279052734375f,-0.10409999292492866374004734097979962825775146484375f,-0.392200014653802053743447686429135501384735107421875f,-0.08840000424385063837462439551018178462982177734375f,-0.5129000066131350621390083688311278820037841796875f,-0.542800022897124367915466791600920259952545166015625f,0.243199993896484389921397450962103903293609619140625f,0.5943999813258649123071108988369815051555633544921875f,-0.10880000884532903260293323910445906221866607666015625f,0.0568000129580497059578192420303821563720703125f,-0.170299979311228089073892988380976021289825439453125f,0.3428000212579964678383248610771261155605316162109375f,0.79859998255372044440036916057579219341278076171875f,1.1207999683737757123935807612724602222442626953125f,0.80529998169541361363599207834340631961822509765625f,-0.078599985086917900645175905083306133747100830078125f,-0.422399977391958447725528458249755203723907470703125f,-0.1798999905794860243446464664884842932224273681640625f,0.1466000076770782312163277083527646027505397796630859375f,-0.301899992371350567310628321138210594654083251953125f,-0.0226999976441264461612945524393580853939056396484375f,-0.0930000297427178157505522904102690517902374267578125f,0.61279996664524105653981678187847137451171875f,0.196099989631772331222236971370875835418701171875f,0.25600000077486040339636019780300557613372802734375f,0.360799991619586979396672177244909107685089111328125f,0.0187999970734120669391131741576828062534332275390625f,-0.1999999900162219290677967364899814128875732421875f,-1.02679998479485501405861214152537286281585693359375f,-0.704200015011430036082629158045165240764617919921875f,-0.012600008079409530381553850020281970500946044921875f,0.2916999883010984451203739808988757431507110595703125f,0.40389999937862153256418196178856305778026580810546875f,-0.10739999527335175599773720023222267627716064453125f,-0.673199987041950276278612363967113196849822998046875f,-0.2800999971270561328395842792815528810024261474609375f,0.53479998428821584610659556346945464611053466796875f,1.0944999562501911416489974726573564112186431884765625f,0.54899997933208954936645795896765775978565216064453125f,-0.396399997776746726430019407416693866252899169921875f,0.4589000024706122449202894131303764879703521728515625f,0.432499993219971567981474436237476766109466552734375f,-0.366199989813566162411007098853588104248046875f,0.45970001405775551273080736791598610579967498779296875f,0.3197000393152240604877079022116959095001220703125f,0.66379998261928552238941847463138401508331298828125f,0.2553000062078238929785811706096865236759185791015625f,0.2677000411182646200103363298694603145122528076171875f,0.71219998894929859023505969162215478718280792236328125f,0.2869000138103963148950015238369815051555633544921875f,0.6280000069439406473748022108338773250579833984375f,0.327999982655048594182289889431558549404144287109375f,0.3975999874413014278928812927915714681148529052734375f,-0.911300032389164016422000713646411895751953125f,-0.29939999488592139442744155530817806720733642578125f,-0.716900015151500813459506389335729181766510009765625f,-1.060999979317188035565777681767940521240234375f,-0.1537999967753886920718287001363933086395263671875f,-1.094399984343349618853835636400617659091949462890625f,-0.2006000080943106667064057546667754650115966796875f,0.23739999437928194225833067321218550205230712890625f,-0.099600014285743387887350763776339590549468994140625f,-0.67990000345036383233576771090156398713588714599609375f,0.34199998944252729682347080597537569701671600341796875f,-0.52930001245513558227884232110227458178997039794921875f,0.76900001665949790918830331065692007541656494140625f,-0.57539999461770019451023472356610000133514404296875f,0.556400033725798603967405142611823976039886474609375f,0.04879999752044650307425399660132825374603271484375f,0.255199988889694306948285884573124349117279052734375f,-0.04000001549720766380602299250313080847263336181640625f,-0.040399965858458841694300645031034946441650390625f,0.90359998806714969532549730502068996429443359375f,0.116800003719329748008703973027877509593963623046875f,-0.19270000881999715947046070141368545591831207275390625f,-0.6215999750435354798128173570148646831512451171875f,-0.18649998469650752586090902696014381945133209228515625f,-0.362599993252754160266704275272786617279052734375f,0.39699998351931586881846669712103903293609619140625f,-0.195600002245604809303358706529252231121063232421875f,-0.2108999981343746554784956970252096652984619140625f,-0.218999983131885844755970538244582712650299072265625f,-0.373699996891617924887896151631139218807220458984375f,-0.0379000138401988095893102581612765789031982421875f,-0.164100005963444706136300510479486547410488128662109375f,0.30440001194775112569601560608134604990482330322265625f,0.0019000035658478520161906999419443309307098388671875f,0.04239999978095287946899816233781166374683380126953125f,0.1662000035226345040495488092346931807696819305419921875f,-0.0267999991185963182260820758529007434844970703125f,0.021099999429285531959976651705801486968994140625f,-0.018599987471103862635146697357413358986377716064453125f,-0.0146999732099477586899638481554575264453887939453125f,-0.303699987429380424686087280861102044582366943359375f,0.441399997776746655375745831406675279140472412109375f,0.9485999977529047111346471865545026957988739013671875f,0.72830000711679454550306900273426435887813568115234375f,-0.34479999993443488648381389793939888477325439453125f,-1.2428999920845029691918171010911464691162109375f,-0.7570000155568124000637908466160297393798828125f,-0.04260000129938124047157543827779591083526611328125f,0.02299999949336051230375232989899814128875732421875f,0.194900003692507706176684223464690148830413818359375f,-0.2847999947190285041642709984444081783294677734375f,0.1486000205516813110762086580507457256317138671875f,-0.43000000774860414054501234204508364200592041015625f,-0.34120000531077376848543281084857881069183349609375f,-1.0115000117421146796914399601519107818603515625f,-0.662400009280442958470302983187139034271240234375f,-0.6631999952375888796041181194595992565155029296875f,-1.355399988955259260592356440611183643341064453125f,-0.8485999918669460129194703768007457256317138671875f,-0.2323999928891655741836075321771204471588134765625f,-1.2699000094294543572459588176570832729339599609375f,-0.70800001677870749716703357989899814128875732421875f,-0.4004000092208384042891111675999127328395843505859375f,-0.61930001787543265123758828849531710147857666015625f,-0.176000000476836948593017950770445168018341064453125f,0.491799992841482236372030456550419330596923828125f,0.811499983578920502935716285719536244869232177734375f,0.5570999837011096911254526276024989783763885498046875f,-0.202199992972612374586560690659098327159881591796875f,-0.035999986469745781647588955820538103580474853515625f,-0.2706999980688096218983673679758794605731964111328125f,0.16340000156164169453631984652020037174224853515625f,0.2428000006198882676500261368346400558948516845703125f,0.16309999878108494275608109091990627348423004150390625f,0.1134000090122224690958319115452468395233154296875f,0.4842999960824847516249747059191577136516571044921875f,-0.61430000964254138917652880991226993501186370849609375f,0.406599993520974922756039404703187756240367889404296875f,-0.53510002406090517723669108818285167217254638671875f,-0.37380000078007580288641520382952876389026641845703125f,0.232999992452561865974303145776502788066864013671875f,-0.31960000897720453050254718618816696107387542724609375f,0.50210000245794628170870055328123271465301513671875f,0.363199974189699126281283270145650021731853485107421875f,0.4058999827302993690381072156014852225780487060546875f,-0.550800011140108125573533470742404460906982421875f,-0.544999990314245064126907891477458178997039794921875f,-0.6149999993667005782072010333649814128875732421875f,-0.01789996109008740177159779705107212066650390625f,0.8165999832391737101033868384547531604766845703125f,0.180900015866756636029322180547751486301422119140625f,-0.2036000089287757219835839350707828998565673828125f,-0.31979998831152922633691559894941747188568115234375f,-0.1593999948486686246695853697019629180431365966796875f,1.4306000358104711267515085637569427490234375f,0.0716000064849851014514570124447345733642578125f,1.16740001837015139329878365970216691493988037109375f,0.1695000081062314478685948415659368038177490234375f,0.63179998136758808158219835604541003704071044921875f,0.3030999978870154709653661484480835497379302978515625f,0.5353999908924098605211838730610907077789306640625f,0.48479997981786748795229868846945464611053466796875f,0.6821999893963337502356125696678645908832550048828125f,-0.34009999623298625692768837325274944305419921875f,0.932599994146823263463375042192637920379638671875f,-0.0897000146538022846698368084616959095001220703125f,0.07809998261332484759122962714172899723052978515625f,-0.1957999901294709133026117342524230480194091796875f,0.1925000048428773258279989022412337362766265869140625f,-0.570499981760978158717989572323858737945556640625f,-0.20919998170733489217809619731269776821136474609375f,-0.62189998426884418591953362920321524143218994140625f,0.74780000479221353515413284185342490673065185546875f,0.049400001889467137772271598805673420429229736328125f,0.63119999856799824389241848621168173849582672119140625f,-0.19419999839663493190755616524256765842437744140625f,-0.03339999694228179549781998503021895885467529296875f,-0.20720000060945753528329760229098610579967498779296875f,-0.47790002188682567663136069313623011112213134765625f,-0.26210000046491632730294440989382565021514892578125f,0.308999997437000128996942294179461896419525146484375f,0.389400001293420672254796954803168773651123046875f,0.338300003316998498803513939492404460906982421875f,0.2226000017464160318780841407715342938899993896484375f,0.44320000157058225820350116919144056737422943115234375f,0.269300005954504040328600922293844632804393768310546875f,0.5505999870836735254897575941868126392364501953125f,1.286599973385781137125150053179822862148284912109375f,0.8114999835789202808911113606882281601428985595703125f,-0.34160001691579822136191069148480892181396484375f,-0.11850001105666141398842228227294981479644775390625f,-0.271600013414025287517006290727294981479644775390625f,0.05379999275207525410991138414829038083553314208984375f,0.490399985304474927971796205383725464344024658203125f,0.1606999959081412132633204237208701670169830322265625f,-0.03099999511241924299298489131615497171878814697265625f,-0.344599984595179709145895685651339590549468994140625f,-0.0336999931663275464899243161198683083057403564453125f,-0.27359999685883540365693988860584795475006103515625f,-0.47909997400045423177061820751987397670745849609375f,0.148200015801191131714631410432048141956329345703125f,0.48340000483989709056942274401080794632434844970703125f,0.7527000050574539358194670057855546474456787109375f,0.32869999681711181693799517233856022357940673828125f,-0.25540000251531613795208386363810859620571136474609375f,-0.370599991088360614099883605376817286014556884765625f,-0.1436999958857894110764163997373543679714202880859375f,-0.3790000203847885273944484652020037174224853515625f,0.003899977362156192839393042959272861480712890625f,-0.4921000134289261751519006793387234210968017578125f,-0.506199994134902908626827411353588104248046875f,-0.869399989074468582117560799815692007541656494140625f,-0.7320999996453523284145603611250407993793487548828125f,-0.55860001220703114910293152206577360630035400390625f,-0.52020000990033121723854492302052676677703857421875f,-0.489900014719367060678223424474708735942840576171875f,0.21220000534057614771654698415659368038177490234375f,-0.0439999971389770649921047152020037174224853515625f,-0.1581000038474797975140972994267940521240234375f,-0.176800005954504069194399562547914683818817138671875f,-0.0182000012725591631124189007095992565155029296875f,-0.119600000092387137584637457621283829212188720703125f,0.324200001525878800379132371745072305202484130859375f,0.7652999993905422737583421621820889413356781005859375f,-0.51999999672174457288065241300500929355621337890625f,-0.080200008112191856213257779018022119998931884765625f,-0.50579999836236211319118183382670395076274871826171875f,-0.172200008842349205906430142931640148162841796875f,-0.1394000020384786875382587822969071567058563232421875f,-0.2216000077888369279577318593510426580905914306640625f,-0.375599988257139472125345491804182529449462890625f,-0.58739997020214840262042343965731561183929443359375f,-0.53019997981861244351620143788750283420085906982421875f,0.87690003243684788714062960934825241565704345703125f,-0.135799995791912042619742351234890520572662353515625f,0.76410003479719190710994780602050013840198516845703125f,0.378199997547268740305526080192066729068756103515625f,-0.270999990642070809343522341805510222911834716796875f,0.2372000038877128957892637117765843868255615234375f,-0.405999998018145458189565033535473048686981201171875f,0.099799991738796389739718506461940705776214599609375f,-0.2797999994501469966934337207931093871593475341796875f,-0.4641000029087063438737459364347159862518310546875f,-0.45899999177455885757126452517695724964141845703125f,-0.2756999815285203769832378384307958185672760009765625f,-0.274499988511204495722495266818441450595855712890625f,0.086799996864795669893055674037896096706390380859375f,-0.3771000047490000728345194147550500929355621337890625f,0.13720000681206589998595291035599075257778167724609375f,-0.09220000247210247135143390551093034446239471435546875f,0.18980001284256597937627475403132848441600799560546875f,1.4839000250458713736634308588691055774688720703125f,0.05159999054074315694151664501987397670745849609375f,1.2405000210627916690242500408203341066837310791015625f,0.79240000436305990660912357270717620849609375f,0.42779998974204058725945287733338773250579833984375f,0.808199995349347499740133571322076022624969482421875f,1.01880002553463011594203635468147695064544677734375f,-0.065399987167120254838437176658771932125091552734375f,0.95580003084689379999616676286677829921245574951171875f,-0.192999991446733520916723136906512081623077392578125f,0.5361999814689160093195141598698683083057403564453125f,-0.179600013466179320165139188247849233448505401611328125f,0.4846000070959328720476833041175268590450286865234375f,-0.11000000178813917894871110547683201730251312255859375f,0.41500001020729559986222056977567262947559356689453125f,0.2214000107407567696782280108891427516937255859375f,-0.21579999620169398877322919361176900565624237060546875f,0.42339999560117735910580449854023754596710205078125f,0.818199975270033075247511078487150371074676513671875f,0.2730999887973071249547274419455789029598236083984375f,0.06299999472498907238104948191903531551361083984375f,-0.066700014489888825863772581215016543865203857421875f,-0.4405000284761191320370699031627736985683441162109375f,0.11780001164674747116123398882336914539337158203125f,-0.23319998510479944542339580948464572429656982421875f,-0.457299995687603821892253108671866357326507568359375f,-0.1151999906778335691637238369366968981921672821044921875f,-0.42419998414739978898069239221513271331787109375f,-0.102599992842972387308009274420328438282012939453125f,0.3198000006794929905851176954456605017185211181640625f,0.683999993562698360705098821199499070644378662109375f,0.1288999914437531923994129101629368960857391357421875f,-0.42499999776482599855853550252504646778106689453125f,-0.57329998297691364683714709826745092868804931640625f,-0.491299985674023442783209247863851487636566162109375f,0.2569999978244303928676117720897309482097625732421875f,0.08819999687671664645449709496460855007171630859375f,0.003299993556737934596867489744909107685089111328125f,0.1810000043511390543926609097979962825775146484375f,0.9879000099658963307547310250811278820037841796875f,0.42270000230073900837624023552052676677703857421875f,-0.3180000135004521499837437659152783453464508056640625f,0.0978999978959560479552237666212022304534912109375f,-0.16160000514388084269512546597979962825775146484375f,-0.35839999053478244217529891102458350360393524169921875f,-0.8612999816134572217407594507676549255847930908203125f,-0.66719999692142006875172910440596751868724822998046875f,-0.59940000114440916689773075631819665431976318359375f,-1.0953999967038630103388641146011650562286376953125f,-0.74709999830424766997793994960375130176544189453125f,0.246600007677078181256291600220720283687114715576171875f,0.84610001485198704784806977841071784496307373046875f,0.245100001671910039391377722495235502719879150390625f,-0.27179999880194660732968259253539144992828369140625f,-0.264199995788931829565626685507595539093017578125f,-0.376100003117322945200839967583306133747100830078125f,0.010599996769428188247985644920845516026020050048828125f,0.18850000096112484726518232491798698902130126953125f,0.032699999096989618152520051808096468448638916015625f,0.6387999842584133336487184351426549255847930908203125f,0.198599993990361711748704465207993052899837493896484375f,0.0215999963521955606893243384547531604766845703125f,0.04700000378489477270704810507595539093017578125f,-0.1394000119104983337336989279720000922679901123046875f,0.49810004326105161709392632474191486835479736328125f,0.676799994331597876140449443482793867588043212890625f,0.39710001167803998800565068449941463768482208251953125f,0.1770000048279758431135633145458996295928955078125f,0.83139996201396026975771746947430074214935302734375f,0.1519999720081688376893680469947867095470428466796875f,-0.26220000677481281048386563270469196140766143798828125f,0.08399999970942728655387554681510664522647857666015625f,-0.14340000575259337356470723534584976732730865478515625f,0.24020001019835479638686592807061970233917236328125f,0.11420000018477427516927491524256765842437744140625f,0.0659999951496719194921070084092207252979278564453125f,0.700699995833634847741677731391973793506622314453125f,0.1547999772846704313877808090182952582836151123046875f,0.6884999948516490331229533694568090140819549560546875f,-0.336399984216690040028652219916693866252899169921875f,0.3585999840438367503026029226020909845829010009765625f,-0.316200000728666552607393214202602393925189971923828125f,0.4711000029683116707701628911308944225311279296875f,-0.56199997696280501457977152313105762004852294921875f,0.2349000079020855391576105830608867108821868896484375f,0.980900025552511323212456773035228252410888671875f,-0.305399996256828121232729245093651115894317626953125f,0.81850003109872371265254287209245376288890838623046875f,0.4333000068560242112880587228573858737945556640625f,0.56659998122751720206480285924044437706470489501953125f,0.498899991257488739648806586046703159809112548828125f,0.6460000007748600836521291057579219341278076171875f,0.37379998515248313850634076516143977642059326171875f,0.7309999996200204908092246114392764866352081298828125f,0.4273000096574428585682881021057255566120147705078125f,-0.1935999924629927537722551278420723974704742431640625f,0.2071000003159044755562945283600129187107086181640625f,0.7016000090926886034736753572360612452030181884765625f,0.14699999901652327505274797658785246312618255615234375f,0.63100000334531058943099424141109921038150787353515625f,0.2105000089928508177905541742802597582340240478515625f,-0.082400001159310232168309084954671561717987060546875f,0.144700007799267893471295565177570097148418426513671875f,-0.35089999008774785949071883806027472019195556640625f,-0.0454000034093855475703094271011650562286376953125f,-0.30240000942945499673442100174725055694580078125f,-0.13209998839497583134061642340384423732757568359375f,0.300000010654330129256095460732467472553253173828125f,-0.25140001297593084217396608437411487102508544921875f,0.07639996663332038195903805899433791637420654296875f,-0.459700029629468875924658277654089033603668212890625f,0.101599999779462990545653156004846096038818359375f,-0.125200012582540320948965018033050000667572021484375f,0.4314000014275312100409109916654415428638458251953125f,0.3227999919772148729180116788484156131744384765625f,0.66729998046979321912886007339693605899810791015625f,0.20359999011605989238660185947082936763763427734375f,-0.39799999561905874401190885691903531551361083984375f,-0.45199997934699087664967009914107620716094970703125f,0.169200028124451673505745930015109479427337646484375f,0.1279999975562095826386865837775985710322856903076171875f,0.3938999933622777671615722283604554831981658935546875f,0.09939999633878471296810630519757978618144989013671875f,0.03200000214576714796521628159098327159881591796875f,0.316200004118680766396209946833550930023193359375f,0.31960001640915880471993659739382565021514892578125f,0.0806000048905610722016490399255417287349700927734375f,-0.0287999945774675136789255702751688659191131591796875f,-0.08209999774545428952166048475191928446292877197265625f,-0.07180000029206279332782969504478387534618377685546875f,0.076000004425644807071194009040482342243194580078125f,-0.0334999994337559225954237263067625463008880615234375f,0.148600004905462323989695505588315427303314208984375f,0.1059999995082616663921726285479962825775146484375f,0.185700003135204294579807537957094609737396240234375f,0.272400007939338717477539830724708735942840576171875f,0.2647000028073787092353086336515843868255615234375f,0.02099999772012228760331709054298698902130126953125f,-0.32759998177886018311255611479282379150390625f,-0.866999966830015456054070455138571560382843017578125f,-0.6927999895185232848149325945996679365634918212890625f,0.6384000039458275210790816345252096652984619140625f,0.93489998484253877819583067321218550205230712890625f,0.677600011506676924710745879565365612506866455078125f,-0.23740000019073492776300327022909186780452728271484375f,-0.42969999155700211446173852891661226749420166015625f,-0.165299995553493506150743996840901672840118408203125f,-0.47179999418258677934545630705542862415313720703125f,-0.4738000086218117079539524638676084578037261962890625f,-0.13749999582767458150556194595992565155029296875f,-0.8005999836564061666877023526467382907867431640625f,-0.0510999646723266920389505685307085514068603515625f,1.3102000243544580371235497295856475830078125f,0.18180001727938588373945094645023345947265625f,1.074800015170871869685242927516810595989227294921875f,-1.0676000269293783873081338242627680301666259765625f,-0.3579999766945842765153429354541003704071044921875f,-0.8536000031173232027725816806196235120296478271484375f,0.08910000702515230575073701402288861572742462158203125f,0.02859999771565202575374087246018461883068084716796875f,-0.0533000009141861852413057931698858737945556640625f,-0.2332000362157824469022671109996736049652099609375f,-0.68239999125003780733322855667211115360260009765625f,-0.0764000000864266670674851411604322493076324462890625f,0.19890000278726216453861752597731538116931915283203125f,-0.05139999811202289325962055954732932150363922119140625f,0.094499999780207843969037639908492565155029296875f,-0.100200028377771754861669251113198697566986083984375f,-0.4539999959468838941489821081631816923618316650390625f,-0.121800010424852389601113600292592309415340423583984375f,-0.05580000104457150111869623287930153310298919677734375f,0.235399989551305865997932187383412383496761322021484375f,-0.05280001052916050152674642959027551114559173583984375f,0.297400005480647156019813337479718029499053955078125f,-0.08899999803304670109582730219699442386627197265625f,0.23920000573545696642696611888823099434375762939453125f,-0.4090000143498182527679318809532560408115386962890625f,-0.04339999515414250108591431853710673749446868896484375f,-0.36080001117736115734402346788556315004825592041015625f,-0.12829999966621397078370137023739516735076904296875f,0.306199993538856407582215979346074163913726806640625f,-0.17270001593530193151337925883126445114612579345703125f,0.41060000243186944857143316767178475856781005859375f,0.647599975967407370802675359300337731838226318359375f,0.47699998180568214944941018984536640346050262451171875f,-0.844500043272972789054620079696178436279296875f,-0.1778000038981435437790423748083412647247314453125f,-0.8765000375211240424278003047220408916473388671875f,0.051100001999735855662265748833306133747100830078125f,-0.01960000284910190515574868186376988887786865234375f,0.12530000691562903636366854698280803859233856201171875f,0.13470000230446477207379984974977560341358184814453125f,0.0749999973177910561616954510100185871124267578125f,0.01579999698400502694539682124741375446319580078125f,0.333999994307756509215323603712022304534912109375f,-0.2911000166028738789236740558408200740814208984375f,-0.04320000075101848580061414395458996295928955078125f,-0.02010000100135800327194601777591742575168609619140625f,0.080999999359250007824329031791421584784984588623046875f,0.1409999828934669352520359097979962825775146484375f,1.11179998181462291739762804354541003704071044921875f,0.611899993470311098775482605560682713985443115234375f,0.64339998889565475082008561003021895885467529296875f,1.13139997229576128034977955394424498081207275390625f,0.824499989032745350669983963598497211933135986328125f,0.61459997543096545058460833388380706310272216796875f,0.92499995842576065996354373055510222911834716796875f,0.779099979141354648248807279742322862148284912109375f,0.26520000170469282263496779705747030675411224365234375f,0.404799999114870967531487622181884944438934326171875f,0.1941999989926814773610885822563432157039642333984375f,0.35580001158714313902464709826745092868804931640625f,0.4274999964237213845308360760100185871124267578125f,-0.18910001602172865631246168049983680248260498046875f,-0.474600005531311097684010746888816356658935546875f,-0.596299990069866225894656963646411895751953125f,0.106900009936094164686437579803168773651123046875f,0.1106000194191931740306245046667754650115966796875f,0.069200026112794521537807668210007250308990478515625f,-0.3960999917179346407891671333345584571361541748046875f,0.17279999138116863832692615687847137451171875f,1.08119996701478982004118734039366245269775390625f,0.49479999367594729164920863695442676544189453125f,0.15399999751150605931115933344699442386627197265625f,-0.1549000035062431113175307473284192383289337158203125f,-0.14520001224726442057288977593998424708843231201171875f,0.31680001549124714443905759253539144992828369140625f,-0.02519998978376403186985044158063828945159912109375f,-0.251799995151162026019164841272868216037750244140625f,0.13659996938109397746075046597979962825775146484375f,0.88839995983839070703425022657029330730438232421875f,0.423099981421232396172626977204345166683197021484375f,0.3982000168442727527917668339796364307403564453125f,0.18560000466704362764858160517178475856781005859375f,0.135300000992417235323728164075873792171478271484375f,0.4966000011205675690462157945148646831512451171875f,-0.74780000129044044143711289507336914539337158203125f,0.9422000306725504259475201251916587352752685546875f,0.29419999809861163697632946423254907131195068359375f,0.7120000085905193731861118067172355949878692626953125f,-0.0831000016868113089429925821605138480663299560546875f,-0.16259999042153372439400982329971157014369964599609375f,-0.0578999946177006508118978445054381154477596282958984375f,0.423200009691715450799165409989655017852783203125f,-0.90099997141957288704361417330801486968994140625f,0.187800030831992703639343744725920259952545166015625f,-0.79640000060796722891609533689916133880615234375f,-0.416599982494116005682371906004846096038818359375f,-0.861799993474781533819850665167905390262603759765625f,-0.9803999816536901334984577260911464691162109375f,-0.340199988442659417842151015065610408782958984375f,-0.980599974603950652607409210759215056896209716796875f,-0.185199998947978006214043489308096468448638916015625f,0.03239999795556069983604174922220408916473388671875f,-0.23360000550150872289378867208142764866352081298828125f,0.3047000127911569933303326251916587352752685546875f,0.65119999696612307360510385478846728801727294921875f,0.1424999784305687189345235310611315071582794189453125f,-0.2107000061154362668958128779195249080657958984375f,-0.67559997873902322140793330618180334568023681640625f,-0.0304999717399476732992980032577179372310638427734375f,0.5442000009298322282802473637275397777557373046875f,0.6513999727427961516923460294492542743682861328125f,0.4111999686166643019902267042198218405246734619140625f,0.5091999747037885271083723637275397777557373046875f,-0.69439996597766917574290346237830817699432373046875f,0.2555999958753580880710387646104209125041961669921875f,-0.27509997525960183661908331487211398780345916748046875f,0.464999987483024568302170109745929948985576629638671875f,-0.223899994386732414941576507771969772875308990478515625f,-0.202000008553266940936055107158608734607696533203125f,0.8083999675869943946082685215515084564685821533203125f,-0.2040000399053096824530939557007513940334320068359375f,0.401400023257732385673079988919198513031005859375f,-0.5997999850332735860547472839243710041046142578125f,0.1972000378623608440875614178366959095001220703125f,-0.65820001773834224678694226895458996295928955078125f,0.374999998956918378922864576452411711215972900390625f,-0.60520003251284404388599114099633879959583282470703125f,-0.64270000855922670979225586052052676677703857421875f,0.4958000020503998683807367342524230480194091796875f,-0.33400001367926623885296066873706877231597900390625f,-0.265299996447563302126582129858434200286865234375f,0.2236000193595888418940376141108572483062744140625f,-0.0208000048816205929114175887661986052989959716796875f,0.67890000939965222670480216038413345813751220703125f,0.040600000941753311423099148669280111789703369140625f,-0.154599991263449310086031118771643377840518951416015625f,-0.34929998270869277998684765407233498990535736083984375f,-0.433699990390986112043236744284513406455516815185546875f,-0.12619999026060124691639430238865315914154052734375f,-0.88619996954798718746815211488865315914154052734375f,-0.524899999818206008939114326494745910167694091796875f,0.54199999991059311099661499611102044582366943359375f,0.78179998360276226065934679354541003704071044921875f,0.718000003889203153306652893661521375179290771484375f,0.03660000424981102451482684045913629233837127685546875f,0.064900013191997729666837813056190498173236846923828125f,0.38230002176463606655687499369378201663494110107421875f,0.49039999521374699753550885361619293689727783203125f,0.919799987119436224247692734934389591217041015625f,0.7342000038355589452976346365176141262054443359375f,-0.02360001578330983562636902206577360630035400390625f,0.24809997918605830591332050971686840057373046875f,0.42859998903572549266982605331577360630035400390625f,-0.4517999831557275580706800610641948878765106201171875f,-0.95249996401369607657017013480071909725666046142578125f,-0.66329997850656496893861913122236728668212890625f,-0.514399997270107434133024071343243122100830078125f,-0.61909997832179097798643852001987397670745849609375f,0.019600028255581936065254922141321003437042236328125f,-0.53879998992085453846101472663576714694499969482421875f,-0.77109997785985484597404138185083866119384765625f,-0.3231999877870084247888371464796364307403564453125f,-0.16800000843405715755807250388897955417633056640625f,0.1193999922037125571705473703332245349884033203125f,0.362199999305605757626835838891565799713134765625f,-0.394400005614757576122286764075397513806819915771484375f,-0.55529999760240311701409154920838773250579833984375f,-0.183999996393918952009016720694489777088165283203125f,0.68839999545216556953164399601519107818603515625f,1.05199998113512993569429454510100185871124267578125f,0.664000000342726881541466354974545538425445556640625f,-0.0591999975025654379123807302676141262054443359375f,-0.6071999948352575504628703129128552973270416259765625f,-0.156700014042854718354647047817707061767578125f,-0.7110000025629990005882063996978104114532470703125f,-0.046299979266523738630212392308749258518218994140625f,0.98910002242922789861268029198981821537017822265625f,-0.2878000015139576817091437987983226776123046875f,0.7143000177264215988515161370742134749889373779296875f,0.455899998022615549064795459344168193638324737548828125f,0.42359997506588731308596607050276361405849456787109375f,0.42769998304098855168575710195000283420085906982421875f,-0.25239996830224953328070114366710186004638671875f,0.7445999714076521058814250864088535308837890625f,-0.079799995501339093806336677516810595989227294921875f,-0.965400029039383156259646057151257991790771484375f,-0.3291999896049500051731229177676141262054443359375f,-0.94140001752078550367741627269424498081207275390625f,-0.58630002914816170545009299530647695064544677734375f,-0.45120000002086124712974424255662597715854644775390625f,-0.48170000838041293622637795124319382011890411376953125f,-0.6918000162363053817671243450604379177093505859375f,-0.46959999092817295007762368186376988887786865234375f,-0.7314000040352344189642508354154415428638458251953125f,-0.278099992746114299535520331119187176227569580078125f,-0.6565999683380130402809982115286402404308319091796875f,-0.248099967637658125330091252180864103138446807861328125f,0.4586999876528974340317290625534951686859130859375f,0.4475999714970593235108253793441690504550933837890625f,0.5432999783948064820293666343786753714084625244140625f,-0.032100032353401530826886300928890705108642578125f,-0.617799996149539509815440396778285503387451171875f,0.120700002539158024461585227982141077518463134765625f,0.337899986347556247068268930888734757900238037109375f,-0.172199988800287417944900880684144794940948486328125f,0.4379000010251996588550582600873894989490509033203125f,-0.103400005584954879367387547972612082958221435546875f,-0.7519999672770503185148527336423285305500030517578125f,-0.072599975520372506121447031546267680823802947998046875f,0.1761000003978610006782901109545491635799407958984375f,-0.1957999966114758461088740659761242568492889404296875f,0.2625000151619316657303215833962894976139068603515625f,-0.7372000136852268070697391522116959095001220703125f,-0.12000000596046422884910498396493494510650634765625f,-0.8116000160962342224735266427160240709781646728515625f,-0.8572999896526329877133321133442223072052001953125f,-0.166100007662177251432922275853343307971954345703125f,-0.065600008690356759188944124616682529449462890625f,0.3724999608099466286148526705801486968994140625f,-0.266400030931830400504622957669198513031005859375f,-0.0475999896764756069700297302915714681148529052734375f,-0.6670999801993371391972686978988349437713623046875f,-0.21419999400079259288531829952262341976165771484375f,0.05140001699924479083847472793422639369964599609375f,-0.59079998878836637032918588374741375446319580078125f,-0.028499988183379176831522272550500929355621337890625f,-0.35720000310540189758512497064657509326934814453125f,-1.096200005906819985312949938816018402576446533203125f,-0.5928000105291604260315807550796307623386383056640625f,0.3548000026166437947949816589243710041046142578125f,0.98860001026987998073991548153571784496307373046875f,0.633600006470083965126605107798241078853607177734375f,0.349000004887580761447907207184471189975738525390625f,0.6942000080823895746817697727237828075885772705078125f,0.36590000744760031725633098176331259310245513916015625f,0.4474000073432922164329283987171947956085205078125f,0.2884000026047228715242454200051724910736083984375f,0.45110000661909577246433400432579219341278076171875f,-0.4550000070035455923544986944762058556079864501953125f,-0.64910001363754243897119522443972527980804443359375f,-0.149900000116228948598973147454671561717987060546875f,0.183600020924210571848789186333306133747100830078125f,-0.1972999827608468148554266008432023227214813232421875f,-0.23339998990148302926428414139081723988056182861328125f,0.215599995130300481349649999174289405345916748046875f,0.3989999901354313038837062777020037174224853515625f,0.25840000208318247842953496729023754596710205078125f,0.11980000023245818141504059894941747188568115234375f,0.128599992239475380273461269098334014415740966796875f,-0.3405000145435332914445325513952411711215972900390625f,-0.1073999837994576156408044198542484082281589508056640625f,-0.62539997094348098727323304046876728534698486328125f,-0.1516999883010985428200001479126513004302978515625f,-0.14079999877214433023908668474177829921245574951171875f,-0.5565999961286782760083724497235380113124847412109375f,-0.19559999870657918119576379467616789042949676513671875f,-0.1284000006675720673143814565264619886875152587890625f,-0.5307999986231326516872286447323858737945556640625f,-0.25099999824166285833371148328296840190887451171875f,-0.5600000040233137355016879155300557613372802734375f,-0.309000020124018615064187542884610593318939208984375f,1.1067000116586687141762013197876513004302978515625f,0.12780002714395433116578715271316468715667724609375f,0.8507000035822385175521276323706842958927154541015625f,-0.56769999797940240426896707504056394100189208984375f,0.401799985539913251386678894050419330596923828125f,-0.3689000075370072462277448721579276025295257568359375f,-0.9318000332236291427534524700604379177093505859375f,0.29219999625086767736092951963655650615692138671875f,-0.6832000319316986480799869241309352219104766845703125f,-0.040999987065791998475106083787977695465087890625f,0.54439999041557296521887110429815948009490966796875f,0.147000005498528540925917695858515799045562744140625f,-0.1818000001430508660860141390003263950347900390625f,-0.56759998490810392723915356327779591083526611328125f,-0.3121999930471182072011515629128552973270416259765625f,-0.1800000052154064889009532635100185871124267578125f,-0.146600003355741304034154381952248513698577880859375f,-0.2754000091090798019166641097399406135082244873046875f,-1.103000017225742368509600055404007434844970703125f,0.0161999975621700542660619248636066913604736328125f,-0.980000017620623165015558697632513940334320068359375f,0.08969998254179945007535934564657509326934814453125f,-0.30279998199343705067576593137346208095550537109375f,0.213100008355080916544466163031756877899169921875f,-0.24950002028793127895056613851920701563358306884765625f,0.634999978840351086972759730997495353221893310546875f,-0.2027000420495870258719151024706661701202392578125f,-0.2082000079035761785917202359996736049652099609375f,-0.1228000041961667676559954998083412647247314453125f,-0.24420000472962843929281007149256765842437744140625f,0.5268000210046768216898271930404007434844970703125f,0.576599986666440855742621351964771747589111328125f,0.3889999934509396073423204143182374536991119384765625f,-0.49840000631138658970797905567451380193233489990234375f,0.21899999456852670132178673156886361539363861083984375f,-0.309800009936839426671895125764422118663787841796875f,-0.264199999961256981606538829510100185871124267578125f,0.20359999298453335114800211158581078052520751953125f,-0.1258000004857777742728330849786289036273956298828125f,-0.1583999996244909169718084740452468395233154296875f,0.285199985313415727006258748588152229785919189453125f,-0.04600000323355180054107904652482829988002777099609375f,0.97070001982450460076279341592453420162200927734375f,0.599499996036291005196972037083469331264495849609375f,0.217999999940395261521786096636787988245487213134765625f,0.47580000098869190683359420290798880159854888916015625f,0.457200012493133634006170495922560803592205047607421875f,0.51880000362992273466034021112136542797088623046875f,0.611800001633167056525053340010344982147216796875f,0.61320001393854628446433707722462713718414306640625f,-0.12340001646280285285683930851519107818603515625f,0.275399983516335655764351031393744051456451416015625f,0.4054999949485063037712961886427365243434906005859375f,0.70079998819231992257527963374741375446319580078125f,1.035099970006942982081454829312860965728759765625f,0.5905999824643135553969841566868126392364501953125f,0.2100000065565108542386951739899814128875732421875f,0.304600009107589553281059124856255948543548583984375f,-0.0313999991178512960487978489254601299762725830078125f,0.3801999844193459932029099945793859660625457763671875f,0.82029996344149147535773636263911612331867218017578125f,0.381499982535839221586826397469849325716495513916015625f,-0.365600005114078641099695232696831226348876953125f,0.0355000072419642265231232158839702606201171875f,-0.427600000181794115405864431522786617279052734375f,0.0419999863505363391080749124739668332040309906005859375f,0.371099981193989680861022861790843307971954345703125f,0.0919999919757248374452274219947867095470428466796875f,-0.2866000076770784943391845445148646831512451171875f,-0.21489998856782932534770225174725055694580078125f,0.0497000139832497467295979731716215610504150390625f,-0.09280000002384181101433568983338773250579833984375f,-0.147199996995925896925427878159098327159881591796875f,0.2384000021576879735363263534964062273502349853515625f,-0.4886000016272067369982323725707828998565673828125f,-0.56329999653697004902141998172737658023834228515625f,-0.459100004026293806447256429237313568592071533203125f,-0.03559999378919609380744759619119577109813690185546875f,-0.149699985857308093262219017560710199177265167236328125f,0.0422000101834535124023517482783063314855098724365234375f,-0.08799999696016325145819791941903531551361083984375f,0.256000020146369688944787412765435874462127685546875f,-0.3052000069200995202578496900969184935092926025390625f,-0.261399989581108105340945257921703159809112548828125f,-0.8239999831318856049477972192107699811458587646484375f,-0.40809999252259732127612323893117718398571014404296875f,0.22199999588727958865774780861102044582366943359375f,-0.67679999429434556379447940344107337296009063720703125f,-0.418600020533055339200245725805871188640594482421875f,-0.395599998520314410921372427765163592994213104248046875f,-0.421800007202476179291039670715690590441226959228515625f,-1.009000024110079163364162013749592006206512451171875f,-0.1796000091075893312364541998249478638172149658203125f,-0.9530000195354222380927922131377272307872772216796875f,-0.0199999963492152321720141117111779749393463134765625f,-0.78659997087121025227673953850171528756618499755859375f,-0.03059996685534704941034078728989697992801666259765625f,-0.666299986791610621139625436626374721527099609375f,-0.00259999742507943665259517729282379150390625f,-0.75469999654889097229215622064657509326934814453125f,0.12860000684261319037204884807579219341278076171875f,0.266399987494945644783683746936731040477752685546875f,0.03699998925626279966394349685288034379482269287109375f,-0.0724999941140413994844493572600185871124267578125f,-0.1571999898433686926324526211828924715518951416015625f,-0.2086999935016036200607913997373543679714202880859375f,1.4029000200986860846796844271011650562286376953125f,-0.01980000842809648275988365639932453632354736328125f,1.1701000138163568653482116133091039955615997314453125f,0.33170000988915548933988475255318917334079742431640625f,-0.16539999897629009684152379122679121792316436767578125f,0.21570001283958573079502230029902420938014984130859375f,0.11430000014305097266742450301535427570343017578125f,-0.432800005537270937594485076260752975940704345703125f,0.15770001872926997510404589775134809315204620361328125f,-0.43840000603198969741924884147010743618011474609375f,-0.399999979138374595066807160037569701671600341796875f,-0.31999998748302471707205540951690636575222015380859375f,0.7783000100970269841127446852624416351318359375f,0.0144000069558618548626327537931501865386962890625f,0.758500013403594408600838505662977695465087890625f,-0.228300008569657819634812767617404460906982421875f,-0.06260000271499144286480031951214186847209930419921875f,-0.1599000046983360956431852173409424722194671630859375f,1.03520004044771241069611278362572193145751953125f,0.17839999470710754536639797152020037174224853515625f,0.81040001771450054235401694313623011112213134765625f,-0.1042000067412853780979276052676141262054443359375f,0.372199988204240828082447478664107620716094970703125f,0.0507999881550669218910343261086381971836090087890625f,0.036300021064281651206329115666449069976806640625f,0.21240001032352451293405692922533489763736724853515625f,0.484599985414743361644696051371283829212188720703125f,0.50079997195005443444415504927746951580047607421875f,0.035099982523918438204191261320374906063079833984375f,-0.30460000091195116311837409739382565021514892578125f,0.0749000125586984921710609341971576213836669921875f,-0.37270000118315227410903389682061970233917236328125f,0.07139999420046805322925820291857235133647918701171875f,0.78629999431669705955982863088138401508331298828125f,0.212899996778368905125944365863688290119171142578125f,0.28960000046491618519439725787378847599029541015625f,0.5687999994575976270283490521251223981380462646484375f,0.34360000192522999196143018707516603171825408935546875f,-0.11979998622536669472538051195442676544189453125f,-0.0414999692738060588226289837621152400970458984375f,0.41330002611577487670047048595733940601348876953125f,0.5423999951243398953693031216971576213836669921875f,0.72409999590516083145530501496978104114532470703125f,0.282399991920590398564172573969699442386627197265625f,-0.14399998477101316751713966368697583675384521484375f,-0.88329997999668119490479512023739516735076904296875f,-0.633400011694431608333388794562779366970062255859375f,0.4299999999999999378275106209912337362766265869140625f,0.93890000247061233817902348164352588355541229248046875f,0.673400001040100892879536331747658550739288330078125f,-0.055999995112419043152840458787977695465087890625f,-0.6918000016331671275793269160203635692596435546875f,-0.15759999787211409483234092476777732372283935546875f,-0.00899999549984931146440203519887290894985198974609375f,-0.1941999947458505904052117330138571560382843017578125f,-0.07449999842047694276203628760413266718387603759765625f,-0.53880001406073585457079389016143977642059326171875f,-0.51999999865889545702657414949499070644378662109375f,-0.325599998334050066972622516914270818233489990234375f,0.42639998913407328728908396442420780658721923828125f,1.27969997255802159230597681016661226749420166015625f,0.729800004553795123030113245476968586444854736328125f,0.12799999248981475119535389239899814128875732421875f,0.513399988597631473652427303022705018520355224609375f,0.4204000029623509160359162706299684941768646240234375f,-0.2693999984621999743694686912931501865386962890625f,-1.1695000119805332161604383145458996295928955078125f,-0.59130000720620135012950413511134684085845947265625f,0.08619998757839208991526902536861598491668701171875f,-0.3650000004470348446972138845012523233890533447265625f,-0.1797999753475192363794121774844825267791748046875f,0.5385999721229079995055144536308944225311279296875f,-0.272000001363455812253278054413385689258575439453125f,1.2633000226140023158905023592524230480194091796875f,-0.00920000346302973781575929024256765842437744140625f,1.0451000139281154677206586711690761148929595947265625f,0.705500007465481804302953605656512081623077392578125f,-0.399999987781047838808490269002504646778106689453125f,0.461700010950863326542048525880090892314910888671875f,-0.22480000261664390137639202293939888477325439453125f,-0.170399997746944364962473628111183643341064453125f,-0.28800000277161597050934460639837197959423065185546875f,-0.51390003160238251211922033689916133880615234375f,-0.6955999811232089768964215181767940521240234375f,-0.38890000332742946653752369456924498081207275390625f,-0.3488999746799468226754470379091799259185791015625f,0.24379998232126265378383322968147695064544677734375f,-0.4416999966084953488376640962087549269199371337890625f,-0.040699973034858061282648122869431972503662109375f,0.8549999916553492340653974679298698902130126953125f,0.0916999966084954820644270512275397777557373046875f,-0.4560999975293871244730325997807085514068603515625f,-0.4893999837100506677955991108319722115993499755859375f,-0.5364999914765358202117795372032560408115386962890625f,-0.6810000153779984799484736868180334568023681640625f,0.32439999339580527504267593030817806720733642578125f,-0.4554000147342682769391331021324731409549713134765625f,-0.154400002261996238672736581065692007541656494140625f,0.139799997103214224125622422434389591217041015625f,-0.0994000055029988782706595884519629180431365966796875f,0.71800001662969581417428344138897955417633056640625f,-0.244800003957748035787744811386801302433013916015625f,0.71240002991855189140579796003294177353382110595703125f,-0.0301000284194952172356352093629539012908935546875f,-0.66939998862743355090287877828814089298248291015625f,-0.2353000095605850550128934628446586430072784423828125f,-0.140600011521577972217755814199335873126983642578125f,-0.4963999883890151121335065909079276025295257568359375f,-0.190999994665384253966777805544552393257617950439453125f,-0.311500008165836561602191068232059478759765625f,0.69759998684525470480366493575274944305419921875f,-0.07330001564770949329385985038243234157562255859375f,-0.8295999968886373920895493938587605953216552734375f,0.47899999400973314322982332669198513031005859375f,0.50760000427961349256378298377967439591884613037109375f,0.708199997100233957780801574699580669403076171875f,0.316199996519088699642452411353588104248046875f,-0.4415999941170216658292702049948275089263916015625f,-0.726699981558323049313230512780137360095977783203125f,-0.6365999911367892227787024239660240709781646728515625f,0.27280000136494653162344548036344349384307861328125f,0.04299998757243184144272163393907248973846435546875f,-0.339400026848912350629916545585729181766510009765625f,-0.1518000020429493002183107819291763007640838623046875f,0.32720000543370819645616620618966408073902130126953125f,0.418500021915882924883334226251463405787944793701171875f,-0.15820001222491253400903588044457137584686279296875f,-0.036000018060207139569683931767940521240234375f,0.3868999998033046949785784818232059478759765625f,-0.30579998878836622822063873172737658023834228515625f,0.1202000177979467565592130995355546474456787109375f,-0.47290000415444399806119690765626728534698486328125f,-0.267799988549947853844201972606242634356021881103515625f,-0.60299997518584158573418108062469400465488433837890625f,-0.482099989289045349760698400132241658866405487060546875f,-0.64980000887513167384668122394941747188568115234375f,-0.76699999409914010328748190659098327159881591796875f,-0.513300001752376555685941639239899814128875732421875f,0.32239999676346808854532355326227843761444091796875f,-0.09830003244876817092290366417728364467620849609375f,0.490599994385242599292951126699335873126983642578125f,0.20619999637007691006829190882854163646697998046875f,-0.04829999385476124729166258475743234157562255859375f,0.235900006404519135827513309777714312076568603515625f,0.20920000182390208198768277725321240723133087158203125f,0.4287000031501053687321700635948218405246734619140625f,0.12179999857842915833572305928100831806659698486328125f,0.2404000005781650184388809066149406135082244873046875f,0.50180000200569618851176301177474670112133026123046875f,0.349600000539421917888915913863456808030605316162109375f,-0.1929999924898149199492536354227922856807708740234375f,-0.8169999750256540238524394226260483264923095703125f,-0.28069999203383932950828238972462713718414306640625f,-0.539399995630979400829119185800664126873016357421875f,-0.45719999520778653590014073415659368038177490234375f,-0.412899995064735403360600685118697583675384521484375f,-0.3676000162005426119549156283028423786163330078125f,-0.79149998894333839505321748220012523233890533447265625f,-0.364599998304247829850055495626293122768402099609375f,0.152799995404481858685130646335892379283905029296875f,-0.447400013303756782789832868729718029499053955078125f,0.73550000202655763104075958835892379283905029296875f,0.360999979019165362359444770845584571361541748046875f,0.73669999333024038801198685177951119840145111083984375f,0.1210000350475315400444742408581078052520751953125f,0.83279999138116789225705360877327620983123779296875f,0.0525999972760675671423769017565064132213592529296875f,0.0877999662168320593735870716045610606670379638671875f,-0.8617999861359593172238646729965694248676300048828125f,0.05360000052824620553337808814831078052520751953125f,-0.393800015699863337204078561626374721527099609375f,-0.72179997078776381158604635857045650482177734375f,-0.28799998045712715821764504653401672840118408203125f,1.495200015413761462923503131605684757232666015625f,0.10300001022219618107556016184389591217041015625f,1.2254000006154175395067795761860907077789306640625f,0.35800000007078036645680185756646096706390380859375f,0.390199980060756457245219053220353089272975921630859375f,0.3953999881543219796498078721924684941768646240234375f,0.85120001976490033968048010137863457202911376953125f,-0.099399998909234899002740348805673420429229736328125f,0.86380002870112682078485022429958917200565338134765625f,-1.520900035834312546967339585535228252410888671875f,-0.1069999812841420094855493516661226749420166015625f,-1.25090001646280324365534397657029330730438232421875f,-0.82509999444484716235592713928781449794769287109375f,0.026399983173609253839231314486823976039886474609375f,-0.68369999469369602262958096616785041987895965576171875f,0.068500007703900411826225536060519516468048095703125f,0.032599997276067771423413432785309851169586181640625f,-0.02550000143051145418127134689711965620517730712890625f,-0.246199998232722094826385728083550930023193359375f,-0.2323999922931194728192849652259610593318939208984375f,-0.2901999963402748239360562365618534386157989501953125f,-0.42279999093413334065871822531335055828094482421875f,0.3113999840676786590165647794492542743682861328125f,-0.2145999970003960033437806487199850380420684814453125f,0.95419998513460146938314210274256765842437744140625f,-0.153399987107515567430482406052760779857635498046875f,0.89399999702721799632598731477628462016582489013671875f,-0.20060000958442714136253925971686840057373046875f,0.38019999660104508709679294042871333658695220947265625f,-0.4461999878391622598172716607223264873027801513671875f,-0.46859998454675089174514823753270320594310760498046875f,-0.5520999929584562426754246189375407993793487548828125f,-0.297199999827146399411503807641565799713134765625f,-0.3367999955236911091560614295303821563720703125f,0.099199996981024440856344881467521190643310546875f,0.30380000541806229108487968915142118930816650390625f,0.454099997693300228007728946977294981479644775390625f,0.2274999966472386336846511767362244427204132080078125f,-0.27579999117255216134481088374741375446319580078125f,-0.9655999824643135553969841566868126392364501953125f,-0.58229999695420264771428264793939888477325439453125f,-0.6277999992787837157948160893283784389495849609375f,-1.01629999856352792875213708612136542797088623046875f,-0.596299993571638875522467060363851487636566162109375f,-0.3257999929606913980251192697323858737945556640625f,-0.6181999840617180552726495079696178436279296875f,-0.030099993401766056422275141812860965728759765625f,-0.72139998168349261931098226341418921947479248046875f,-1.1502999700725080600705041433684527873992919921875f,-0.79269998255670071074519000831060111522674560546875f,0.226199987578392214260247783386148512363433837890625f,0.4303999646663669853552391941775567829608917236328125f,0.06249998174607773737676552627817727625370025634765625f,-0.3704000047504900638983826866024173796176910400390625f,-0.91930001362860169766832996174343861639499664306640625f,-0.495400004005431882081467165335197933018207550048828125f,0.6574000004887583514801008277572691440582275390625f,0.69029997469186810121755115687847137451171875f,0.71329998670220362555483006872236728668212890625f,0.230399992978572942803339174133725464344024658203125f,0.37869997759461426767302327789366245269775390625f,0.007199987384677086765805142931640148162841796875f,-0.4119999917149541612815255575696937739849090576171875f,-0.5402999970436095367176676518283784389495849609375f,-0.30359999633729461976372476783581078052520751953125f,-0.20060000168681124677050320315174758434295654296875f,-0.764300016981362961843160519492812454700469970703125f,-0.0938999984472987936356958016403950750827789306640625f,-0.19399999669194212259526466368697583675384521484375f,0.11669999765157701432372050476260483264923095703125f,-0.455600002878904231096157673164270818233489990234375f,-0.55840000871419892547464769450016319751739501953125f,0.2291000032067298519677933654747903347015380859375f,-0.6394000395894039456834434531629085540771484375f,-0.126799962219596551449285470880568027496337890625f,-0.3148000128984449474955908954143524169921875f,0.35320007793903318571437921491451561450958251953125f,-0.2182000685513012516736353063606657087802886962890625f,0.2089000295162204423604634939692914485931396484375f,-0.00570004992485007022651188890449702739715576171875f,-0.4259999529421334418799460763693787157535552978515625f,0.0154999835491178572510762023739516735076904296875f,1.147000045359135356193291954696178436279296875f,-0.80980000991821299294315394945442676544189453125f,0.6137000307917599428719768184237182140350341796875f,0.5876000045776361702110079932026565074920654296875f,-0.515999989897012678596865953295491635799407958984375f,0.337599995040893219311328721232712268829345703125f,-0.470199972200394000765300006605684757232666015625f,0.44479998071193715958315806346945464611053466796875f,-0.2442000000357626987579351407475769519805908203125f,-0.986799972724913931898527152952738106250762939453125f,1.125799965393543100589113237219862639904022216796875f,-0.55269999753236742634499023552052676677703857421875f,-1.4030999610066405836050762445665895938873291015625f,1.347799965080618722623739813570864498615264892578125f,-0.762199999976157727132886066101491451263427734375f,-0.06449999186396571104751274106092751026153564453125f,0.1783999940365552649979008492664434015750885009765625f,-0.04949999305605867760959881707094609737396240234375f,-0.391000028938055521621208754368126392364501953125f,0.79900000630319123473555009695701301097869873046875f,-0.25470001472830805511193830170668661594390869140625f,-0.453799980533123203940704115666449069976806640625f,1.18719997129142296188319960492663085460662841796875f,-0.2954000031113626079104506061412394046783447265625f,0.829900031781196645397358224727213382720947265625f,-1.11440001313984371478227330953814089298248291015625f,0.4865000218749049309963083942420780658721923828125f,0.3210999901533124756269899080507457256317138671875f,-0.183599971228838310111086684628389775753021240234375f,0.185899983382224576189400977455079555511474609375f,-0.454600005680322993839581613428890705108642578125f,1.03300000672042369131986561114899814128875732421875f,-0.30130000601410866778451236314140260219573974609375f,-0.754600048148631685762666165828704833984375f,0.0037000015854840029305705684237182140350341796875f,-0.18259999243319047224076712154783308506011962890625f,-9.99998837709625831848825328052043914794921875e-05f,-0.48720000474452973548977752216160297393798828125f,-0.4415999866664410689054420799948275089263916015625f,-0.037500000558793544769287109375f,0.07029999972581890688161365687847137451171875f,0.172799987804889809694941504858434200286865234375f,-0.0494999991282820683835552699747495353221893310546875f,0.2403000110507012010430116788484156131744384765625f,0.97639999211430517078724733437411487102508544921875f,0.344499995104968481651752654215670190751552581787109375f,0.902500007599591480556000533397309482097625732421875f,0.65059998626410919797535825637169182300567626953125f,0.16629999364614478363222360712825320661067962646484375f,0.21939999548196720269288562121801078319549560546875f,0.5238000057905907169697456993162631988525390625f,0.2567999971255658930857634913991205394268035888671875f,0.0143000147461886939481701119802892208099365234375f,-0.3253999838143588618777357623912394046783447265625f,0.08329999974071977675293965148739516735076904296875f,0.52560001256465938013207050971686840057373046875f,-0.515200015711784420346930346568115055561065673828125f,-0.29090000383406877659808742464520037174224853515625f,-0.9727000332951547534321434795856475830078125f,0.69500003285706080102812620680197142064571380615234375f,0.229600010988861402694283242453821003437042236328125f,0.310700039140135719772928268866962753236293792724609375f,0.1471999943137169974960443141753785312175750732421875f,0.0079000008389353837401358759962022304534912109375f,0.307700008261203805659533827565610408782958984375f,-0.313400029501319010716287039031158201396465301513671875f,-0.20830000562667849717257695374428294599056243896484375f,-0.29200002285838166926623671315610408782958984375f,0.4596000250518326168958083144389092922210693359375f,0.287400006598234281085524344234727323055267333984375f,0.933600014591217330917061190120875835418701171875f,0.68439999585449673435988415803876705467700958251953125f,0.210199996973574110281646198927774094045162200927734375f,0.5304000086992974871691330918110907077789306640625f,-0.15100000397861013112787986756302416324615478515625f,0.04329999914467330679457290898426435887813568115234375f,0.47960001477003100234242083388380706310272216796875f,-0.3947999906957149196529144319356419146060943603515625f,-0.723500014185905371277840458787977695465087890625f,0.0226000291645522821681879577226936817169189453125f,0.641999974206090229245091904886066913604736328125f,-0.026799993512034436804469805792905390262603759765625f,-0.8238999734878547087646438740193843841552734375f,0.7777999560654169641793487244285643100738525390625f,-0.4467999968647955455480769160203635692596435546875f,0.158699956285953902579421992413699626922607421875f,-0.7083999666929248206770353135652840137481689453125f,0.11549998965859398225575205287896096706390380859375f,0.948099990063905551096468116156756877899169921875f,-1.13519996489882490919853808009065687656402587890625f,0.54629999796748140994395726011134684085845947265625f,0.933399985766410367205025977455079555511474609375f,-0.83539997159540657634124727337621152400970458984375f,0.5108999991774556548307373304851353168487548828125f,0.81370002244710804006899707019329071044921875f,-0.07200001563131763759884051978588104248046875f,0.413500004649161922998246154747903347015380859375f,-0.99559997352361762068539974279701709747314453125f,0.8085999546140438809516126639209687709808349609375f,-0.524500002741813631246259319595992565155029296875f,-0.0963000106334686023501490126363933086395263671875f,0.382400005555152855407641254714690148830413818359375f,-0.08649999952316278495345613919198513031005859375f,0.919500035393983505827009139466099441051483154296875f,-0.71020001951158040487399603080120868980884552001953125f,0.50060002260431668563711582464748062193393707275390625f,0.03929997615218194795261297258548438549041748046875f,-0.4423999794781210415095529242535121738910675048828125f,0.045499993458390175504746366641484200954437255859375f,-0.2596000481486318012258607268449850380420684814453125f,-0.048199961486459219628386563272215425968170166015625f,-0.135300017458200638742482624365948140621185302734375f,0.02640004084110358917314442805945873260498046875f,-0.2556000073492530333396643982268869876861572265625f,0.045000012516975829157672706060111522674560546875f,0.75159999098777741100008142893784679472446441650390625f,-0.87759997984170912133095043827779591083526611328125f,0.4280000006109474508519951996277086436748504638671875f,-0.633000018119812324357553734444081783294677734375f,0.27839999984800822829811295378021895885467529296875f,-0.3202000166058542873770420555956661701202392578125f,-0.75919997529983529460650970577262341976165771484375f,-0.111000017166137610047371708787977695465087890625f,-0.14380000132024284287268756088451482355594635009765625f,-0.022399999613314867941138430751379928551614284515380859375f,0.206900006341189268699309877774794586002826690673828125f,0.1401999964147806476688629118143580853939056396484375f,0.221600000561773757823402775102294981479644775390625f,0.32490000346898995076116989366710186004638671875f,-0.2385999970823526883378917773370631039142608642578125f,-0.010100001075863840327428988530300557613372802734375f,0.16599999809265142403091886080801486968994140625f,-0.61440000934004768140539454179815948009490966796875f,-0.4112999996438622130057183312601409852504730224609375f,-0.8815000147223468474066976341418921947479248046875f,0.3020000307559966845616372665972448885440826416015625f,0.2401000045403838267787932636565528810024261474609375f,0.5957000293612484398408923880197107791900634765625f,0.246599990913271671644224625197239220142364501953125f,0.18809999669492238894008551142178475856781005859375f,-0.2652000145196922176182852126657962799072265625f,0.725800032076239887146584806032478809356689453125f,0.20720000895410783936512189029599539935588836669921875f,0.21970002739429528304526684223674237728118896484375f,-0.479999999254941744908364853472448885440826416015625f,-0.14909999765604731170043351085041649639606475830078125f,-0.24210000135898557260816232883371412754058837890625f,-0.488999988719820866123200175934471189975738525390625f,-0.13799999506026505713407459552399814128875732421875f,-0.6912999894738192097065621055662631988525390625f,-0.92760002581179179514947463758289813995361328125f,-0.297600009308755464321194494914379902184009552001953125f,-0.67100003854930445612581024761311709880828857421875f,0.0140000028014182209545879231882281601428985595703125f,0.059499998383224006470726408224436454474925994873046875f,0.216699996012449158655499559245072305202484130859375f,-0.77299999122321605415919520964962430298328399658203125f,-0.133599999727308738073361382703296840190887451171875f,-0.3593000096797940301485141390003263950347900390625f,-0.54599999630451190313351617078296840190887451171875f,-0.161399998670816391399540634665754623711109161376953125f,-0.388800008100270932942521540098823606967926025390625f,0.38500000692903946220013722268049605190753936767578125f,0.2467999945178627196895604356541298329830169677734375f,0.525300002408027211231456021778285503387451171875f,0.656400000458955634741187168401665985584259033203125f,-0.2254000105619429206171844271011650562286376953125f,-0.395700010976940408369273427524603903293609619140625f,0.11919999981224538299073856251197867095470428466796875f,-0.19990000732466584043578450291533954441547393798828125f,-0.07969999073743760931165525107644498348236083984375f,0.4692000176936381716785717799211852252483367919921875f,-0.052900012648105843027224182151257991790771484375f,-0.54139999285936379891381875495426356792449951171875f,0.3999999850243332044641419997788034379482269287109375f,-0.286200002777576401058468036353588104248046875f,-0.286100028896331792793716886080801486968994140625f,0.04579999802708645262327991076745092868804931640625f,-0.16579999847412096869447850622236728668212890625f,0.00750004470348297758164335391484200954437255859375f,0.0577999547243124300877070709248073399066925048828125f,0.0146000138759614372929718228988349437713623046875f,0.5484000188469888570352850365452468395233154296875f,0.385000017508863923154649455682374536991119384765625f,0.2540999963521954896350507624447345733642578125f,1.0497000416994097804490593262016773223876953125f,-0.827800020959973181788882357068359851837158203125f,0.5692000286459926172710765968076884746551513671875f,-0.6195000036358830897142979665659368038177490234375f,0.544199990946054423801570010255090892314910888671875f,-0.3408000047326087411647677072323858737945556640625f,-0.3184999846816065627308489638380706310272216796875f,0.832999966636300381850332996691577136516571044921875f,-0.2113999976277352033093848149292171001434326171875f,-1.0474000213503842360296403057873249053955078125f,1.111999998867511774136573876603506505489349365234375f,-0.5902000178724529089180350638343952596187591552734375f,-0.1564999884217979086997729609720408916473388671875f,0.1139999924451113866297191634657792747020721435546875f,-0.0788999986335634506673386567854322493076324462890625f,-1.066499972999096268466701076249592006206512451171875f,1.312199958327413096270674941479228436946868896484375f,-0.60140000120401371219713837490417063236236572265625f,-0.626999990671873064229657757095992565155029296875f,0.70979997951984419302107198745943605899810791015625f,-0.35489999974369990098921334720216691493988037109375f,0.203500026404856537709520125645212829113006591796875f,-0.06160002041757051127746080965152941644191741943359375f,0.11690000501871100624384780530817806720733642578125f,1.2056000033259390846751557546667754650115966796875f,-0.65050000590085943485973984934389591217041015625f,-0.2014000005334615206464832226629368960857391357421875f,-0.021800000180304035524159189662896096706390380859375f,-0.26730001319646845558963832445442676544189453125f,-0.8034000232428315513288907823152840137481689453125f,-0.1320000109747052619013629737310111522674560546875f,-0.1577000499844558589757070876657962799072265625f,-0.316599997618794315457080301712267100811004638671875f,-0.10009999757409093756876927727716974914073944091796875f,-0.64020000706911073251603738754056394100189208984375f,-0.3386000155597927996353746493696235120296478271484375f,-0.25720000612288718144782251329161226749420166015625f,-0.302100013577938142361745121888816356658935546875f,-0.135599997812509531769364912179298698902130126953125f,0.0731999993354082167940077852108515799045562744140625f,0.1653999932765959357539031771011650562286376953125f,0.520800010395050083644719052244909107685089111328125f,0.18360000267028808718094978758017532527446746826171875f,0.87280003072023415455760186887346208095550537109375f,-0.414800003361702085413753593456931412220001220703125f,-0.0781000020965934016459186750580556690692901611328125f,-0.5845000179409982621336894226260483264923095703125f,-0.322400011515617368473840542719699442386627197265625f,-0.070900004579126818526901843142695724964141845703125f,0.362099979007243888418088317848742008209228515625f,0.5907999913215633558394301871885545551776885986328125f,0.18079999530762425052898834110237658023834228515625f,0.09499999772757228289066233628545887768268585205078125f,-0.37780002006590385210671456661657430231571197509765625f,-0.101300005790591285403934307396411895751953125f,-0.1940000212788586253509492962621152400970458984375f,-0.5539999935626980231972993351519107818603515625f,-0.102099995361268458538006598246283829212188720703125f,-0.335700004327296852579820551909506320953369140625f,0.196199994209408867806843090875190682709217071533203125f,0.01200000076740981302947375297662802040576934814453125f,0.137300003212690313603161484934389591217041015625f,0.34140000291764727169407933615730144083499908447265625f,0.144299999584257598872483185914461500942707061767578125f,0.34960000795274959184411045498563908040523529052734375f,0.4655999869346618691423600466805510222911834716796875f,-0.006900001330673699584394853445701301097869873046875f,-0.04310000332593943994652363471686840057373046875f,-0.4956000504136088835593909607268869876861572265625f,-0.47490000168085089171654544770717620849609375f,-0.15909998547434778259912491193972527980804443359375f,0.4827999972671268569257563285646028816699981689453125f,-0.106699999365210462798359003500081598758697509765625f,-1.26609999939203277108390466310083866119384765625f,1.2862000028520821359734327415935695171356201171875f,-0.696400017595291132010970613919198513031005859375f,-0.0633999863624570281217529554851353168487548828125f,0.588999990209936985507965800934471189975738525390625f,-0.0712999972224235278872583876363933086395263671875f,-0.27590003494024362140635275864042341709136962890625f,-0.1947999998599285742528763876180164515972137451171875f,-0.1375000047683716530855235760100185871124267578125f,-0.393399982637166800714112468995153903961181640625f,0.526399994051456321386694980901665985584259033203125f,-0.217000003114342643328882331843487918376922607421875f,0.2251999921679495031412443495355546474456787109375f,-0.799599998974799941464652874856255948543548583984375f,0.167000000953674287984540569595992565155029296875f,-0.2886999834060670622193356393836438655853271484375f,0.870599986934661895787712637684307992458343505859375f,-0.1956000015377998835219841566868126392364501953125f,-0.8051000129222867229827897972427308559417724609375f,0.230000011324882347452103203977458178997039794921875f,-0.407500010132789469707859097979962825775146484375f,0.90919999761432368501345990807749330997467041015625f,-0.5809999802112579825319471638067625463008880615234375f,0.4851000011876223094731130913714878261089324951171875f,-0.492200019943714206505092079169116914272308349609375f,0.49800001595914344942883644762332551181316375732421875f,-0.27360001548528689596651020110584795475006103515625f,-0.705499983549117359871161170303821563720703125f,0.95219998626708957090158946812152862548828125f,-0.403600002670287949513294734060764312744140625f,0.2472999823510650951874367819982580840587615966796875f,-0.448599977859855003625710878623067401349544525146484375f,0.14679999693930145809872556128539144992828369140625f,0.296600004585087173580859598587267100811004638671875f,-0.31279999682009218542333428558777086436748504638671875f,0.1706000015005468828377388490480370819568634033203125f,0.86689997104406391059683301136828958988189697265625f,-1.0319999619126323242568332716473378241062164306640625f,0.47990000153183931530520567321218550205230712890625f,-0.75590007636547085212441743351519107818603515625f,-0.731200005310773892830411568866111338138580322265625f,-0.4170000138431788627713103778660297393798828125f,-0.2301000033855438442476071259079617448151111602783203125f,-0.8784000247329475374868934522964991629123687744140625f,0.045800017026067063596883599529974162578582763671875f,0.194500004902482093172011445858515799045562744140625f,0.471600008273124871038817218504846096038818359375f,-0.3412000056087969301898965568398125469684600830078125f,0.057699999432265780541229105438105762004852294921875f,0.15689998475313160497535136528313159942626953125f,0.68640000537633849564400634335470385849475860595703125f,0.3671999956548213095430810426478274166584014892578125f,0.69280001045465411380064324475824832916259765625f,-0.3057999979525802825008895524661056697368621826171875f,-0.2278999975979327796693496566149406135082244873046875f,-0.1889999935626978100344786071218550205230712890625f,0.93519999380707741210017047706060111522674560546875f,0.2716999976888299084265554483863525092601776123046875f,1.1943000069975848731473888619802892208099365234375f,-0.3829999895840880963504559986176900565624237060546875f,-0.09489999557137485108881946871406398713588714599609375f,-0.09539999059438653006282038404606282711029052734375f,-0.29900000257790082525133357194135896861553192138671875f,-0.057400000693649040617305701061923173256218433380126953125f,0.260699998012930256319208410786814056336879730224609375f,0.56499998413026286669946784968487918376922607421875f,0.104299991351365972658271630280069075524806976318359375f,-0.231800028008223524267350512673147022724151611328125f,0.3924000039160249198033625361858867108821868896484375f,0.213599997976422262269124985323287546634674072265625f,0.57380001182556128469514078460633754730224609375f,-0.830199998947977579888402033247984945774078369140625f,-0.093199995945393965257608215324580669403076171875f,-0.03750000923871965596845257095992565155029296875f,0.1531999895751476881144981234683655202388763427734375f,-0.10339999999701976474142384176957421004772186279296875f,-0.29320000253915790011660646996460855007171630859375f,-0.6116000046968459447072063994710333645343780517578125f,-0.148900001911818957722744016791693866252899169921875f,-0.12090001884698864387246430851519107818603515625f,0.98539998664557904017868850132799707353115081787109375f,0.31029999577701083968150896907900460064411163330078125f,0.85600001418590476731651506270281970500946044921875f,-0.8840000227689746026271677692420780658721923828125f,0.09379999737143496219005101011134684085845947265625f,-0.015000009760260724078761995770037174224853515625f,0.37720000347793103401272674091160297393798828125f,-0.043200001123547604464647520217113196849822998046875f,-0.3417999634861939028951383079402148723602294921875f,0.489599987426399874124172129086218774318695068359375f,-0.18259999742507915243550087325274944305419921875f,-0.74829998849034229380094984662719070911407470703125f,0.9172000095129007046779179290751926600933074951171875f,-0.412700012955069528430840364308096468448638916015625f,0.569600023412703659886346940766088664531707763671875f,-0.29760000457763646775077859274460934102535247802734375f,0.31520000230073907943051381153054535388946533203125f,-0.31120002065897001131133947637863457202911376953125f,-0.1988000098139044435896494178450666368007659912109375f,-0.1525999980211256712436806992627680301666259765625f,0.697299978923798136065670405514538288116455078125f,-1.49699995490908666084806100116111338138580322265625f,0.441799999547004773603475769050419330596923828125f,-0.57499999284744252037171463598497211933135986328125f,0.4877999781936408485449874206096865236759185791015625f,-0.31269999991655339499629917554557323455810546875f,-0.696500009190291091698554737376980483531951904296875f,0.28579999810159206763415795649052597582340240478515625f,-0.35740000833049412332087513277656398713588714599609375f,1.131000002115964520044144592247903347015380859375f,-0.68659997377693660070008263573981821537017822265625f,0.59410000812411301041038313996978104114532470703125f,0.19660000767707774826931199640966951847076416015625f,-0.2424000030219553014632083431934006512165069580078125f,0.122199999082088339719120995141565799713134765625f,-1.4172999887585628897568312822841107845306396484375f,1.449999993816017518355465654167346656322479248046875f,-0.776900015449523806410070392303168773651123046875f,0.529799973186850792217228445224463939666748046875f,-0.6297999754220249268854558977182023227214813232421875f,0.2873000005677341928134183035581372678279876708984375f,-0.94189999876022323377355860429815948009490966796875f,0.7745999912261962716542029738775454461574554443359375f,-0.5104000069856642340937469271011650562286376953125f,1.012599971795081987835374093265272676944732666015625f,-1.27679995372593424463047995232045650482177734375f,0.58379999215602840223482417059130966663360595703125f,-0.99650001770258000988178537227213382720947265625f,0.4747000221788884033458089106716215610504150390625f,0.0084000033497810822069595815264619886875152587890625f,0.0777000013694167102062948515595053322613239288330078125f,0.5869000177592040845553356120944954454898834228515625f,-0.02400000026822091125922042920137755572795867919921875f,0.032699999339133499487441980591029278002679347991943359375f,0.316900002504140132497667536881635896861553192138671875f,0.098600029827654422920346632963628508150577545166015625f,0.1480000048391521272339588222166639752686023712158203125f,0.201300018493831611721134322579018771648406982421875f,0.75640001252889643712506995143485255539417266845703125f,0.12890000369995835427516794879920780658721923828125f,0.4957000242948534918241421109996736049652099609375f,-1.04880001331567740407990640960633754730224609375f,-0.1965000019446014878354844768182374536991119384765625f,-0.3307000365138055286706730839796364307403564453125f,-0.053000004932284294767441679141484200954437255859375f,-0.1074999997019767672412626779987476766109466552734375f,-0.2378000024080275665028239018283784389495849609375f,0.29920003907680570165439348784275352954864501953125f,0.191000014707446208461760761565528810024261474609375f,-0.210199968028067729619579040445387363433837890625f,-0.8404000306785111007457089726813137531280517578125f,-0.1553000117585063843961279417271725833415985107421875f,-0.3557000454545029555220025940798223018646240234375f,-1.156200020137429618216629023663699626922607421875f,-0.295800008867681096802471074624918401241302490234375f,-0.927500042617321440729938331060111522674560546875f,-1.08760003274083150159867727779783308506011962890625f,-0.196100007103383600082224802463315427303314208984375f,-0.145700028169155615387353464029729366302490234375f,-0.616000036537647677192808259860612452030181884765625f,-0.278300010804832076427572928878362290561199188232421875f,-0.790900039857626691031100563122890889644622802734375f,0.6806000045925382213596321889781393110752105712890625f,0.1822000052288175186987473352928645908832550048828125f,0.7093000218987466354292337200604379177093505859375f,0.0500000184774400668885618870262987911701202392578125f,0.06250000637024644145611773637938313186168670654296875f,-0.20309998276233631742115903762169182300567626953125f,0.20620000613033784286898253412800841033458709716796875f,0.06170000271797182467281572826323099434375762939453125f,0.24820001177787798241070049698464572429656982421875f,-0.194400011426210550524729114840738475322723388671875f,0.262800011497736019094872972345910966396331787109375f,-0.439900008833408406871967599727213382720947265625f,0.0203999979704618805698146388749592006206512451171875f,-0.2095000089630485451408503649872727692127227783203125f,-0.284500003393739486767799462541006505489349365234375f,0.03460000016689301904904141338192857801914215087890625f,-0.13780000443831086354151693740277551114559173583984375f,-0.033099949532747696689938265990349464118480682373046875f,0.310199951785803396120400066138245165348052978515625f,-0.02539998310655333657592791496426798403263092041015625f,0.941099996113777148565304742078296840190887451171875f,-1.362999994277954041166367460391484200954437255859375f,0.54340001527071013498471074854023754596710205078125f,-1.552699995446204184190719388425350189208984375f,1.63840000275373398608280695043504238128662109375f,-0.859700018453597891721074120141565799713134765625f,0.1229999850392342608529361314140260219573974609375f,-0.220999991759657898882096560555510222911834716796875f,0.066100000286102300606216886080801486968994140625f,0.601800062131882640414914931170642375946044921875f,0.25439996716976143176225377828814089298248291015625f,0.28780002326965359316091053187847137451171875f,-1.18010001411438025797906448133289813995361328125f,1.3668000098288057841955378535203635692596435546875f,-0.6609000251054766295055742375552654266357421875f,-1.27329999936819149297662079334259033203125f,1.749199979993701248304205364547669887542724609375f,-0.736900016045570538381070946343243122100830078125f,-1.754900036549567943211513920687139034271240234375f,1.63400002083182283030282633262686431407928466796875f,-0.9573000403165821126094670034945011138916015625f,-0.4125000400841241088301103445701301097869873046875f,0.567600024545192649583214006270281970500946044921875f,-0.24970001949667963714318830170668661594390869140625f,0.62889998689890003191749201505444943904876708984375f,-1.0761999684304004443191615791874937713146209716796875f,0.3786000020742417149222092120908200740814208984375f,0.20750002831220637489195723901502788066864013671875f,0.2549999767541886175337140230112709105014801025390625f,0.0892000098705292288059354177676141262054443359375f,0.163800000500679221460131884668953716754913330078125f,-0.31499999813735490850064024925814010202884674072265625f,0.10150000280141835418135087820701301097869873046875f,-0.4806000274658206450340003357268869876861572265625f,-0.2089999798536297248574555851519107818603515625f,0.30500000618398193097391413175500929355621337890625f,0.07390000016093256629545749092358164489269256591796875f,0.43260000368356710254147401428781449794769287109375f,0.381600009167194331727301914725103415548801422119140625f,0.1029000012859702106737813664949499070644378662109375f,0.32150001457333576837527289171703159809112548828125f,1.0894000048324468110649831942282617092132568359375f,0.25190000522360211920869232926634140312671661376953125f,0.714800029326975394639021033071912825107574462890625f,-0.047000005945563305687073807348497211933135986328125f,-0.014500001549720753502015213598497211933135986328125f,-0.501800023984909415730726323090493679046630859375f,0.4601999992460010791006652652868069708347320556640625f,0.1232999990701675041560037016097339801490306854248046875f,0.0728000069528815796360277090570889413356781005859375f,-0.3529999971836803585034658681252039968967437744140625f,-0.1260999987959861290409691036984440870583057403564453125f,-0.1004000072091815187746988158323802053928375244140625f,-0.581999997228383580960553445038385689258575439453125f,-0.3530999956145881579772094482905231416225433349609375f,-0.7647000145792954839407684630714356899261474609375f,0.704800025564432441882445345981977880001068115234375f,0.27680000781714919622089610129478387534618377685546875f,0.8632000361412768274504969667759723961353302001953125f,-0.35520000185370437240095498054870404303073883056640625f,-0.15150000036135313763452359125949442386627197265625f,-0.474900017420202436024823100524372421205043792724609375f,0.163199990171193576315999962389469146728515625f,0.136799992096424016807532098027877509593963623046875f,-0.379600028181076964983731159009039402008056640625f,-0.6762000030010939521929458351223729550838470458984375f,-0.1173000014990568384831703951931558549404144287109375f,-0.6100000160932541604097423260100185871124267578125f,-0.403800013092160181571443899883888661861419677734375f,-0.1263000013202428828407164473901502788066864013671875f,-0.8448000266075137432153496774844825267791748046875f,0.029400005987286537134650643565692007541656494140625f,0.1352999997258186315463035498396493494510650634765625f,0.5502000061750411674665883765555918216705322265625f,0.0132000003039836055762634714483283460140228271484375f,-0.00180000118613243831333647904102690517902374267578125f,-0.26840000931024565744564824854023754596710205078125f,0.5906000196427105830565551514155231416225433349609375f,0.196700009870529157751661841757595539093017578125f,0.203000003814697294046709430404007434844970703125f,-0.57179999962151040193703011027537286281585693359375f,0.136300007653236576743438490666449069976806640625f,0.4069000050187108197263796682818792760372161865234375f,-0.5626000059187410418104491327540017664432525634765625f,0.236200007992982907722989693866111338138580322265625f,1.32559998291134917280942318029701709747314453125f,-1.68299996015429531581730770994909107685089111328125f,0.75570000771731138389242232733522541821002960205078125f,0.2820000155568127553351587266661226749420166015625f,0.290000010207295044750708257197402417659759521484375f,0.1186000041604042820608810870908200740814208984375f,0.71679999932348703595152983325533568859100341796875f,-0.5261999906331300547179807836073450744152069091796875f,0.385300003823637826627646063570864498615264892578125f,-0.50399999736249423420986204291693866252899169921875f,0.2921999902158975981336652694153599441051483154296875f,-0.268699998679756912878247021581046283245086669921875f,-0.3375000047683709425427878159098327159881591796875f,0.163199991065263727563205975457094609737396240234375f,-0.194099990391730869987441110424697399139404296875f,0.5202000212997202055476009263657033443450927734375f,-0.805000007599592226625873081502504646778106689453125f,0.315700013044476701651319672237150371074676513671875f,-0.228299993873387396803309457027353346347808837890625f,0.37459999375939367904919663487817160785198211669921875f,-0.14159999655708654930919010439538396894931793212890625f,0.5038000267267221943257027305662631988525390625f,0.40780000225901602561862091533839702606201171875f,0.228100002431869341990022803656756877899169921875f,-0.72079998387098331846800647326745092868804931640625f,1.2921999794870615563269211634178645908832550048828125f,-0.4319000071048737510182036203332245349884033203125f,-0.0611999962210649783855842542834579944610595703125f,0.771000012621283037361763490480370819568634033203125f,-0.0825000050663948769624767010100185871124267578125f,-0.4013999877929688153699316899292171001434326171875f,0.236199991229176564644376412616111338138580322265625f,-0.20010000383257864342567700077779591083526611328125f,0.21680001221299161073829964152537286281585693359375f,0.0959999957084656241335096638067625463008880615234375f,0.099200005474686658857308430015109479427337646484375f,0.37240003356933559786057230667211115360260009765625f,-0.07200000125169747633435690659098327159881591796875f,0.7905999999731776473055333553929813206195831298828125f,0.311899998313188486775970886810682713985443115234375f,0.79740002224445305500921676866710186004638671875f,0.1018000012606383730684456168091855943202972412109375f,0.226400001539289963403689398546703159809112548828125f,0.46690000203847858983863261528313159942626953125f,-0.063399984499812322980005774297751486301422119140625f,0.1884999988377094037872439002967439591884613037109375f,0.5610000070333480692852390347979962825775146484375f,-0.41699999774992446699428683132282458245754241943359375f,-0.1481999974913894846384465608934988267719745635986328125f,-0.231900000119953986210674656831542961299419403076171875f,0.6962000170081854566461743161198683083057403564453125f,0.301900004571676261466706137071014381945133209228515625f,0.676200029748678321794841394876129925251007080078125f,-0.44939999674856689892976646660827100276947021484375f,-0.1026000038698315908192171264090575277805328369140625f,-0.5813000210642815801520555396564304828643798828125f,0.11080000145435331404542012023739516735076904296875f,0.116299998451769359686380767016089521348476409912109375f,0.537499997466802437173782891477458178997039794921875f,0.89480003554821063715962736750952899456024169921875f,0.220000014081597417003877126262523233890533447265625f,0.215800050330162918044152320362627506256103515625f,0.14120000590682035834788621286861598491668701171875f,0.019300001297891167251208344168844632804393768310546875f,-0.501100012356042956440660418593324720859527587890625f,-0.6813999971807003230850341424229554831981658935546875f,-0.100100000293552870989088887654361315071582794189453125f,-0.35900001256167879404301856993697583675384521484375f,-0.91580001941025290790321378153748810291290283203125f,-0.2275000081211329128194620352587662637233734130859375f,-0.8300000405311589446455400320701301097869873046875f,0.8176000395953659793946144418441690504550933837890625f,0.27980001338273297051273402757942676544189453125f,0.46460004952699029967533306262339465320110321044921875f,-0.1557999955683946335938117044861428439617156982421875f,0.006700001525878906960542735760100185871124267578125f,-0.30340000126361843513222993351519107818603515625f,0.1872000069797039589758469446678645908832550048828125f,0.05940000155419113136989750501015805639326572418212890625f,0.03060000254362826854759305206243880093097686767578125f,0.9463999919652934522673604078590869903564453125f,-0.1321999985232948926494600527803413569927215576171875f,0.70539999357461891094089878606610000133514404296875f,-0.8125999828964471927150725605315528810024261474609375f,0.4078999968528744801687935250811278820037841796875f,-0.25229998279809873196199987432919442653656005859375f,0.5038000126451247506764730132999829947948455810546875f,-0.1382000096917153086906182579696178436279296875f,-0.5985999630331999554755384451709687709808349609375f,0.660199963632226438647876420873217284679412841796875f,-0.317299999189376791264294297434389591217041015625f,-0.514500003132968952712644750135950744152069091796875f,0.40439998974502089357230261157383210957050323486328125f,-0.28170000210329881884518954393570311367511749267578125f,0.62780001865029344543245315435342490673065185546875f,-0.5874000020533800547894998089759610593318939208984375f,0.35650000742077825321985073969699442386627197265625f,-0.342099970960617838500184006989002227783203125f,0.84139996536672168048198727774433791637420654296875f,-0.22119999309182158953035468584857881069183349609375f,-0.23829999266266810309389256872236728668212890625f,-0.01699999767541893191946655861102044582366943359375f,-0.10740000197887411559349857270717620849609375f,1.349200027751923158803037949837744235992431640625f,-1.14340001318454742573749172152020037174224853515625f,0.733500026702881058326966012828052043914794921875f,0.603300027829408946900002774782478809356689453125f,0.01879999617934213773651208612136542797088623046875f,0.29210001462101953251249142340384423732757568359375f,-0.91759997894763956072239352579344995319843292236328125f,1.25239997329413910165385459549725055694580078125f,-0.5238000050455331457754937218851409852504730224609375f,-0.943199997770786779938134714029729366302490234375f,1.46139998920857916431259582168422639369964599609375f,-0.55610001339912429330070153810083866119384765625f,1.00640003998428628761985237360931932926177978515625f,-0.8434000234663485695563167610089294612407684326171875f,0.5535000253245236923049787947093136608600616455078125f,-0.06419998260140413748331411625258624553680419921875f,0.5209999886304139682380309750442393124103546142578125f,-0.061299998787045506531967475893907248973846435546875f,0.28680001321882020448583716643042862415313720703125f,-0.1754000038564205166125020696199499070644378662109375f,0.1527000077024103230627360971993766725063323974609375f,0.71609999179243999378741136752068996429443359375f,0.339199982899426988325330967199988663196563720703125f,-0.275800002273916877726378515944816172122955322265625f,-0.307699997681379233682719132048077881336212158203125f,-0.879800021839141521695637493394315242767333984375f,0.236799987179040716256395171512849628925323486328125f,0.1894999979361891018214691939647309482097625732421875f,-0.1319000020384795135441891034133732318878173828125f,-0.2886000267356634285675909268320538103580474853515625f,-0.2424000037297606713337927430984564125537872314453125f,-0.1599000094294549256801474257372319698333740234375f,0.65040000102519968105951875259052030742168426513671875f,0.257099997950345215969036871683783829212188720703125f,0.6197000102765855000797046159277670085430145263671875f,-0.570600006678700477635857168934307992458343505859375f,-0.3942000014886259950941393981338478624820709228515625f,-1.0299000183701512156630997196771204471588134765625f,0.387200005638599353829931715154089033603668212890625f,0.1423000009961426226556824303770554251968860626220703125f,0.254900011385232161398306516275624744594097137451171875f,0.214400002932548350287333960295654833316802978515625f,-0.0350000008195638923069736847537569701671600341796875f,-0.1305999928951262489817963796667754650115966796875f,0.00680000565648075649960446753539144992828369140625f,0.058300000225007531096110824364586733281612396240234375f,0.224300005358457621440493312547914683818817138671875f,0.0876000078558922812277387492940761148929595947265625f,0.0237000026658177598282151166131370700895786285400390625f,-0.1054999952465294210668389496277086436748504638671875f,0.0521999984115362192227394189103506505489349365234375f,-0.00809999960064888835375995768117718398571014404296875f,0.17460000940561304361153815989382565021514892578125f,0.2914000126034022031262793461792171001434326171875f,0.18050000329315663005758096915087662637233734130859375f,0.58760002335310002052892741630785167217254638671875f,-0.6753999835163353449019041363499127328395843505859375f,-0.0787999963656067503592339562601409852504730224609375f,-0.2768999990582461379062806372530758380889892578125f,0.775799997803568341936397700919769704341888427734375f,0.2144999948441981274527279310859739780426025390625f,0.93500002503395052144696819595992565155029296875f,-0.4080000039637088349309124168939888477325439453125f,-0.1572000009447336033030495627826894633471965789794921875f,-0.4320000162273645027966040288447402417659759521484375f,-0.76099999623000591508770185100729577243328094482421875f,-0.32660001348853118230408654198981821537017822265625f,0.2727999943614012323678252869285643100738525390625f,-0.261199995252490158037517176126129925251007080078125f,0.129000014424324405126753845252096652984619140625f,0.4347000005722048143752545001916587352752685546875f,0.346600026303529151761040338897146284580230712890625f,0.2013999871969218702361104078590869903564453125f,-0.29490006068944918382612740970216691493988037109375f,-0.3123999637573959820002755805035121738910675048828125f,-0.1389000181913377929276975919492542743682861328125f,0.606100003359466565910906865610741078853607177734375f,-0.64839998755455019452398346402333118021488189697265625f,0.34690000142380583714185604549129493534564971923828125f,-0.1089000167012219577600262709893286228179931640625f,0.095199985536933429131067896378226578235626220703125f,-0.081899992799758791761632892303168773651123046875f,0.44730000703111283399948661099188029766082763671875f,-0.375199998426437364429375520558096468448638916015625f,0.24710000432059164854337041106191463768482208251953125f,0.4925999947428696401630077161826193332672119140625f,-0.555200008857249738269956651492975652217864990234375f,0.26900000891089437260461636469699442386627197265625f,-0.17429999097883719372248378931544721126556396484375f,0.120999989226460602509405362070538103580474853515625f,-0.0879999996796250183450638360227458178997039794921875f,0.22190001269280923423821150208823382854461669921875f,-0.0402000070691108657428003425593487918376922607421875f,0.111200007024407465650028825621120631694793701171875f,-0.02820001490712176206443473347462713718414306640625f,-0.2353999955862760007363476688624359667301177978515625f,-0.000300003823638039790466791600920259952545166015625f,0.0206000101804733048993512056767940521240234375f,-0.1672000082463025449897031649015843868255615234375f,0.0306000006437301408368512056767940521240234375f,-0.97509996494054806959184134029783308506011962890625f,1.2887999545305970361397385204327292740345001220703125f,-0.552499998509883738506687222979962825775146484375f,0.6231999751210199889328578137792646884918212890625f,-1.260399993053078038229841695283539593219757080078125f,0.388599998497962673127403832040727138519287109375f,-0.350999998763203535645516240037977695465087890625f,0.4453999943196773614317862666212022304534912109375f,-0.205500000163912677209054891136474907398223876953125f,-0.598499989356845230048520534182898700237274169921875f,0.0583999726533885876733620534650981426239013671875f,0.46420001724660409081479883752763271331787109375f,0.2879000050485134298838829636224545538425445556640625f,0.3262000185728073375912572373636066913604736328125f,-0.055200003045797252365645135796512477099895477294921875f,-0.064499998979270445342848461223184131085872650146484375f,-0.156300000500678937243037580628879368305206298828125f,0.950600008317828315540509720449335873126983642578125f,0.155500004410743741800615680404007434844970703125f,0.00660001602172854973105131648480892181396484375f,0.927599995934963228449987582280300557613372802734375f,0.2150999999955296448916897134040482342243194580078125f,0.8647000193476674922976599191315472126007080078125f,0.724999977871775502791251710732467472553253173828125f,0.1494999936521053118809732040972448885440826416015625f,0.8037999885797493249128820025362074375152587890625f,0.3728000092625618488995087318471632897853851318359375f,0.1414000022470951190012300457965466193854808807373046875f,0.4610000163465739486667871460667811334133148193359375f,0.54920002089738861883461140678264200687408447265625f,0.3814000071644783051993954359204508364200592041015625f,0.9552000443220141079336826805956661701202392578125f,-0.7048000154316422793954188819043338298797607421875f,-0.44240000004172319592044004821218550205230712890625f,-1.16320003085136391973719582892954349517822265625f,0.2318000356823206598022579782991670072078704833984375f,0.2713000069081783482971559351426549255847930908203125f,0.5538000356674199764484001207165420055389404296875f,0.9651999944031235401098456350155174732208251953125f,0.2155999993771313683055268484167754650115966796875f,0.1590000275373455451699555851519107818603515625f,-0.0968000020802020078036775885266251862049102783203125f,0.0664999991878867380279416465782560408115386962890625f,0.4006999924063681106645162799395620822906494140625f,0.09260001344382740029459455399774014949798583984375f,0.2259999989494680949775329281692393124103546142578125f,0.8415000188946724080096828402020037174224853515625f,0.786399984663724627154124391381628811359405517578125f,0.1666999940380453626431744851288385689258575439453125f,0.1064999831318846901240249280817806720733642578125f,0.173800008770823577464170739403925836086273193359375f,0.1307000038802623720357587444595992565155029296875f,0.73820003323555027918700943700969219207763671875f,0.597599996680021394723780758795328438282012939453125f,-0.444600019240379396023854496888816356658935546875f,0.4698000358462337544551701284945011138916015625f,0.0365999932974574448962812311947345733642578125f,0.242500007152557373046875f,0.0416999904990198455578820357914082705974578857421875f,-0.074399992650747426292667796587920747697353363037109375f,0.0206999988883733720967939007095992565155029296875f,1.6274000028729442846042729797773063182830810546875f,-1.435799978581071112415656898519955575466156005859375f,0.881500019788742150694815791212022304534912109375f,0.9448999721765520831695539527572691440582275390625f,-1.556999960720539366576531392638571560382843017578125f,0.5618000007390975270027411170303821563720703125f,0.6549999615550046172529619070701301097869873046875f,-1.358999939695001213380010085529647767543792724609375f,0.40589999291896816657754243351519107818603515625f,0.27420000577270986497069316101260483264923095703125f,-0.4492000012278556386746686257538385689258575439453125f,0.167600004503130950439526714035309851169586181640625f,0.129800021243095642375919851474463939666748046875f,-0.06540000728368777771493114414624869823455810546875f,0.08949999678134901159864966757595539093017578125f,-0.23220002501010839068840141408145427703857421875f,0.2522000111520288356814489816315472126007080078125f,-0.153699998903274348549530259333550930023193359375f,-0.10179994620084809042737106210552155971527099609375f,0.3649999482184653754046621543238870799541473388671875f,-0.0544999872446059185904232435859739780426025390625f,1.51779997618198336795103386975824832916259765625f,-1.2887999723374843341616724501363933086395263671875f,0.823400001263618008806588477455079555511474609375f,-0.4896999954313034919550773338414728641510009765625f,0.2964000075370070153013557501253671944141387939453125f,-0.2505000068321823203376652600127272307872772216796875f,-0.537099978709221659300965256989002227783203125f,0.33719997129142331715456748497672379016876220703125f,-0.26630000392794617170011406415142118930816650390625f,1.2570999912261964226445343228988349437713623046875f,-1.076599957758188619294514865032397210597991943359375f,0.6818000055074691090339911170303821563720703125f,-0.2758000106930733608123773592524230480194091796875f,-0.193800003853440205858760236878879368305206298828125f,-0.1143000081896783370893899700604379177093505859375f,0.4613999675273898759542134939692914485931396484375f,-0.1314000140190128007589009939692914485931396484375f,-0.3896000174522402659249564749188721179962158203125f,-0.3295000082179904143941939764772541821002960205078125f,-0.9081000444531444060203284607268869876861572265625f,0.699200031924247955572582213790155947208404541015625f,0.235300007958710255540069056223728694021701812744140625f,0.051500021725893407875673801754601299762725830078125f,-0.214999988116324092235487341895350255072116851806640625f,0.0421999993242323234721169455951894633471965789794921875f,-0.03450000069290393156506979721598327159881591796875f,-0.744599987426399767542761765071190893650054931640625f,-0.1088999961376189151707194469054229557514190673828125f,-0.0038000136137004147940388065762817859649658203125f,0.591799993214010999764695952762849628925323486328125f,0.1023999976202845108019801045884378254413604736328125f,0.6779000186085699652949188021011650562286376953125f,-0.0741999992907047545287468892638571560382843017578125f,-0.164000000938773149439242615699185989797115325927734375f,-0.21790000199377512757337171933613717555999755859375f,0.72740000115931024993187747895717620849609375f,0.1305999995633959631646803245530463755130767822265625f,0.5855000172257422974553264793939888477325439453125f,0.096999981358647513474124934873543679714202880859375f,-0.1369000005483627102620403093169443309307098388671875f,-0.3434000131845476033731756615452468395233154296875f,-0.7859999881088735040890469463192857801914215087890625f,-0.16800000008940696449855067839962430298328399658203125f,-0.5694000105321403992775231017731130123138427734375f,-0.698600008407235240071031512343324720859527587890625f,-0.418300002944469451193754139239899814128875732421875f,-1.179000026941299239524596487171947956085205078125f,-0.730399999833106772229029957088641822338104248046875f,-0.30489999642819165526219649109407328069210052490234375f,-0.9839000072389836315522870791028253734111785888671875f,-0.024200011137127663829460288980044424533843994140625f,-0.1989999981448054011679005270707421004772186279296875f,-0.6087000063538550165276319603435695171356201171875f,-0.5450000140815973725949561412562616169452667236328125f,-0.247300003547966495975884981817216612398624420166015625f,-0.76040002937465922538962104226811788976192474365234375f,0.908200001719593696947185890167020261287689208984375f,0.283399996607005544291268961387686431407928466796875f,1.069500019729137108015493140555918216705322265625f,-0.5388000027358526011767025920562446117401123046875f,0.12639999136924728873054846189916133880615234375f,0.09909998800754493686326895840466022491455078125f,-0.0891999991416927429099814617075026035308837890625f,0.033500008821487625709778512828052043914794921875f,1.1103000529229642978634728933684527873992919921875f,-0.567200025010108799250474476139061152935028076171875f,0.58490002954602271501016730326227843761444091796875f,-0.378499968253076690682945582011598162353038787841796875f,0.66859996456056880465013136927154846489429473876953125f,-0.219999995138496107216496966429986059665679931640625f,-1.2813999690175048584706019028089940547943115234375f,1.178599972793459738085175558808259665966033935546875f,-0.69609999969005542652666918002068996429443359375f,0.5276999647498126222444625454954802989959716796875f,-1.203199977281689481145576792187057435512542724609375f,0.3322999962091444103862158954143524169921875f,0.09569998853206584943364987339009530842304229736328125f,-0.42340001012980887917791505969944410026073455810546875f,0.0614000081330539071444718501879833638668060302734375f,0.8240999817490575196643476374447345733642578125f,-1.33639998295009121420662268064916133880615234375f,0.48530000658035277183444122783839702606201171875f,0.4799999642372139163626343361102044582366943359375f,-0.623199956867099391644160277792252600193023681640625f,0.26079999623894689619874043273739516735076904296875f,-0.9239999667406089134402691342984326183795928955078125f,1.2989999470710760220981683232821524143218994140625f,-0.533699995476007327255274503841064870357513427734375f,-0.2204000108599668550368733122013509273529052734375f,0.348399981519580403954705616342835128307342529296875f,-0.152199992823600638303105370141565799713134765625f,-0.4671000017315147800900376751087605953216552734375f,0.801999982476234496431288789608515799045562744140625f,-0.291500000044703444501692501944489777088165283203125f,0.4806999712467199259435801650397479534149169921875f,-0.488399973919988017456716988817788660526275634765625f,0.24950000140070915932710704510100185871124267578125f,-0.32100000870972866806596357491798698902130126953125f,0.5276000033110379572320880470215342938899993896484375f,-0.2012000030383467308325862177298404276371002197265625f,0.7332999886393540833751103491522371768951416015625f,-1.326399985632300104754222047631628811359405517578125f,0.44990000317096701110131107270717620849609375f,-0.58270001928806181012987508438527584075927734375f,0.458600020551681808456123690120875835418701171875f,0.4893999879568813327068710350431501865386962890625f,0.26329999951720228779095123172737658023834228515625f,0.14780001611709536746275261975824832916259765625f,-0.6257999994426963752403025864623486995697021484375f,-0.161699999216198875284788982753525488078594207763671875f,-0.050400009518861477175732943578623235225677490234375f,-0.608600011387467443313425974338315427303314208984375f,-0.1975000023096800116917393097537569701671600341796875f,0.0175999998092652987224937533028423786163330078125f,-0.9574000281304122506043086104909889400005340576171875f,-0.23000000786036256528177545988000929355621337890625f,-0.459500041782856527561307302676141262054443359375f,0.828200029733777487450652188272215425968170166015625f,0.16600001109391460119013572693802416324615478515625f,0.402100040400029001830262131989002227783203125f,0.61900002224743377876592376196640543639659881591796875f,0.19070000529587272097842287621460855007171630859375f,0.5488000294089321329238373436965048313140869140625f,0.16580000108182435525350228999741375446319580078125f,0.020500000461936007667418380151502788066864013671875f,0.606000014185905655494934762828052043914794921875f,-0.661400033465028069912250430206768214702606201171875f,-0.321400007797777720330856254804530180990695953369140625f,-0.928500034600496793046886523370631039142608642578125f,-0.1057999867767096535686732750036753714084625244140625f,0.10610000140368935905854641532641835510730743408203125f,0.52340002301931409789403915056027472019195556640625f,0.33859999678432917136206015129573643207550048828125f,0.08919999634772535213045330237946473062038421630859375f,0.30770000021457644123756836052052676677703857421875f,0.2740000081658362507397441731882281601428985595703125f,0.2681000004574656259137555025517940521240234375f,0.539300011169910220587553340010344982147216796875f,-0.60999999105930291509736207444802857935428619384765625f,-0.197899994822591462906302695046178996562957763671875f,-0.214299996622651145461446731133037246763706207275390625f,-0.52320001147985462086609231846523471176624298095703125f,-0.1590000033974647564161131185755948536098003387451171875f,-0.2346000199109317119194884071475826203823089599609375f,-0.47400001248717327229087459272705018520355224609375f,-0.115800005775690106890607466993969865143299102783203125f,-0.160400024122000051107761464663781225681304931640625f,-0.60239999810457245388306546374224126338958740234375f,-0.325300013732910286989863379858434200286865234375f,0.65359998747110381600577966310083866119384765625f,-0.0347999661833053863801978877745568752288818359375f,0.334199983644484888145598233677446842193603515625f,-1.01219999982714625730295665562152862548828125f,0.7167999869555234937479326617904007434844970703125f,-0.54180000595450383826801044051535427570343017578125f,-1.0144000324368480647763135493732988834381103515625f,0.493799996849894728967456103418953716754913330078125f,-0.531900017833709881642789696343243122100830078125f,-1.28210001894235592345694385585375130176544189453125f,1.3592000138193365277317070649587549269199371337890625f,-0.7143000245809556503218118450604379177093505859375f,1.1203000170111661049077156349085271358489990234375f,-1.317600007781386306504600725020281970500946044921875f,0.6297000250101092433396843262016773223876953125f,0.630000018179416532149161866982467472553253173828125f,-0.82540001465976209349406644832924939692020416259765625f,0.36490001598596588650025296374224126338958740234375f,-0.146800020110607221113241394050419330596923828125f,-0.57739999430477606523481881595216691493988037109375f,-0.03590000305175777839394868351519107818603515625f,-0.3761000415623190207270454266108572483062744140625f,0.553000024601816964064937565126456320285797119140625f,-0.23380001793503790707973166718147695064544677734375f,-0.611199988248944947599738952703773975372314453125f,0.4231999696075920969207118105259723961353302001953125f,-0.3144000048324466778382202392094768583774566650390625f,0.345899994707107083513619727455079555511474609375f,-0.51919999400079230866822399548254907131195068359375f,0.2033000017523765023952364572323858737945556640625f,-0.01449997591972351784761485760100185871124267578125f,0.2657999945998190671758720782236196100711822509765625f,-0.0066000029087067701993873924948275089263916015625f,-0.850499997686594344514787735533900558948516845703125f,0.60979997892379766977200006294879131019115447998046875f,-0.45940000064149477676522792535251937806606292724609375f,-0.697800010231137246563548615085892379283905029296875f,0.59360000147819513838243210557266138494014739990234375f,-0.382200009438395493788220846909098327159881591796875f,-0.682000004976988094540502061136066913604736328125f,0.63159999828934676457947716698981821537017822265625f,-0.37420000912547113358641581726260483264923095703125f,-0.506099960350990585311592440120875835418701171875f,0.278100014054775357408288982696831226348876953125f,0.34420000342577683483114014961756765842437744140625f,0.0151999993391334908832135397460660897195339202880859375f,0.168500008262693956595512645435519516468048095703125f,0.450600004890560956738454478909261524677276611328125f,0.0839999983683228368391837648232467472553253173828125f,0.6825000140070915932710704510100185871124267578125f,0.127199985224008660367189804674126207828521728515625f,-0.146700000743567926519972388632595539093017578125f,-0.5233000148653985661439946852624416351318359375f,0.9191999970555304599884038907475769519805908203125f,0.3194999982342123789891275009722448885440826416015625f,1.1501000156044955957668207702226936817169189453125f,0.3488000204682351057527966986526735126972198486328125f,0.2032000043272972344965410229633562266826629638671875f,0.5096000222206118479562064749188721179962158203125f,0.7483999871075150966959199649863876402378082275390625f,0.24159999516010277620381430097040720283985137939453125f,0.6234000040948384224748224369250237941741943359375f,0.1730000175982713273015178856439888477325439453125f,0.11950000244379044289644298260100185871124267578125f,-0.39780000061988829429537872783839702606201171875f,0.35959999316334700925068545984686352312564849853515625f,0.10929999591484662602169919409789144992828369140625f,0.081299990908056007921089758383459411561489105224609375f,-0.26680000163316730521501085604541003704071044921875f,-0.203000002473592733309715185896493494510650634765625f,-0.663600026512146001778091886080801486968994140625f,-0.2528000071763989442530373707995750010013580322265625f,-0.21279999734163279612175756483338773250579833984375f,-0.406400003886222549454032559879124164581298828125f,-0.455800004509091305493484469479881227016448974609375f,-0.136399999378621572532210848294198513031005859375f,-0.73470001457929612342923064716160297393798828125f,-0.1765999944150446754331795773396152071654796600341796875f,-0.091899998126924009600458020941005088388919830322265625f,-0.13399999825656383478644784190692007541656494140625f,0.3796000390589242545757997504551894962787628173828125f,0.231200012686848754839274988626129925251007080078125f,0.1114000271320350066162063740193843841552734375f,-0.73340000431835650207545995726832188665866851806640625f,-0.1903000012904405569003074560896493494510650634765625f,-0.36080001367330538641908788122236728668212890625f,0.2754000002056360418833946823724545538425445556640625f,0.2929999981522559693303264793939888477325439453125f,0.488299992960690987242600158424465917050838470458984375f,-0.7760000018179413761032492402591742575168609619140625f,0.28290000873655085644742257500183768570423126220703125f,0.2798999888658517676276460406370460987091064453125f,-0.972800001737474939744743096525780856609344482421875f,0.19770000602602966210952217807061970233917236328125f,0.667299974006414942095943843014538288116455078125f,-0.642199966895580676151666921214200556278228759765625f,0.352000002816319490506202782853506505489349365234375f,0.67929998524189016961827292107045650482177734375f,-1.2375999649405482472275252803228795528411865234375f,0.420699997472762987626992980949580669403076171875f,0.16920002775192255484171255375258624553680419921875f,-0.1588000198721883915453645386151038110256195068359375f,0.103400009310245621918511460535228252410888671875f,0.7855999803781508195044125386630184948444366455078125f,-0.9313999627590181518144163419492542743682861328125f,0.449499993950128295949753010063432157039642333984375f,0.54200004953145963781935279257595539093017578125f,0.3219999721199275910521464538760483264923095703125f,0.250600017929077267808679607696831226348876953125f,0.536900001312047070456401343108154833316802978515625f,-0.36839998561739928373270913652959279716014862060546875f,0.28810000204071395391025589560740627348423004150390625f,0.087800009560584868495425325818359851837158203125f,0.05539998880624796839811097015626728534698486328125f,0.0271000129818919077706595999188721179962158203125f,-0.06720002501010835516126462607644498348236083984375f,-0.1439999846220019197318151782383210957050323486328125f,-0.0368000022292136463875067420303821563720703125f,-0.2286999926447865760792410583235323429107666015625f,0.788199995908140937928010316682048141956329345703125f,-0.165200002598762551997424452565610408782958984375f,-0.17939999756813052034232214282383210957050323486328125f,0.0655999874562026852320428815801278688013553619384765625f,-0.0961999956995247540447735445923171937465667724609375f,0.49740006396770564123244184884242713451385498046875f,0.0887999678671358427806126201176084578037261962890625f,0.2494000243902210200985791743732988834381103515625f,-0.969600000464916345066512803896330296993255615234375f,0.98579998349845421312664939250680617988109588623046875f,-0.53870000873804091412466732435859739780426025390625f,0.01760000427961383451247456832788884639739990234375f,-0.40660001453161243034628569148480892181396484375f,0.550799991619586482016757145174778997898101806640625f,0.33389999095946543139490358953480608761310577392578125f,1.05269998888969329442488742643035948276519775390625f,0.19820002407133596733501690323464572429656982421875f,0.1258000064089894731722552023711614310741424560546875f,-0.3098999897599217234755997196771204471588134765625f,0.75380001771152027600919609540142118930816650390625f,0.2164000053390860711655108161721727810800075531005859375f,0.1221000237852336578470158201525919139385223388671875f,0.49080001114010818330513075125054456293582916259765625f,0.13440000300705434721493247707257978618144989013671875f,0.42220001904964465921921146218664944171905517578125f,-0.20979998570382552003366072312928736209869384765625f,-0.27259999157637349043170615914277732372283935546875f,-0.58109998329877754486005869694054126739501953125f,0.696600016841292468683377592242322862148284912109375f,0.30930000208020214103044054354541003704071044921875f,1.0150000154972076416015625f,-0.701599991285800417273321727407164871692657470703125f,-0.0564999930039047359997539388132281601428985595703125f,0.0469000050187116102051732013933360576629638671875f,0.7662000207334760393251826826599426567554473876953125f,0.15030000896453865610880029635154642164707183837890625f,0.514400034075976009262376464903354644775390625f,-0.67400002291798610798423396772705018520355224609375f,-0.203800006759166763004031963646411895751953125f,-0.1172000220298770045701530762016773223876953125f,-0.17719999997615809572693024165346287190914154052734375f,-0.0390999995931982924357583897290169261395931243896484375f,0.0049000001162291528800096784834749996662139892578125f,-0.464600010821223374790633897646330296993255615234375f,-0.2062000047519803036522034744848497211933135986328125f,-0.9353000408530238729554184828884899616241455078125f,1.103999997735022997602527539129368960857391357421875f,0.2516999954536556227679966468713246285915374755859375f,0.5255000190138812143914037733338773250579833984375f,0.3911999956250189480755352633423171937465667724609375f,0.019799998369812943366241597686894237995147705078125f,0.02140000209808334830086096189916133880615234375f,-1.02880003820061727282109131920151412487030029296875f,-0.230600012341141802352240119944326579570770263671875f,-0.295400041556359127525865915231406688690185546875f,-0.28140003048479567127060363418422639369964599609375f,-0.10140000478029254082912302692420780658721923828125f,0.0837000081419940755722564063034951686859130859375f,-0.318199982869625142001268613967113196849822998046875f,0.0791999934792517734649663907475769519805908203125f,0.521300050270557591147735365666449069976806640625f,0.24979997124969965938134919269941747188568115234375f,0.2452000192880632312153466045856475830078125f,1.1657000225812197413688409142196178436279296875f,-0.80220000332891938654711339040659368038177490234375f,0.624400017759203951328572657075710594654083251953125f,0.516200004565715797610891968361102044582366943359375f,-0.71920000048279753013247272974695079028606414794921875f,0.30020000885725028894057686557061970233917236328125f,0.161299991261959263511016615666449069976806640625f,-0.40540001883208720556694970582611858844757080078125f,0.08560001494884517114769550971686840057373046875f,0.3824000243306162616363508277572691440582275390625f,-0.724400000622868578403767969575710594654083251953125f,0.24620001082420372995329671539366245269775390625f,-1.2638999860048283352398357237689197063446044921875f,1.280799987447261312212276607169769704341888427734375f,-0.6905000121593474204928497783839702606201171875f,0.9358000035405165117907699823263101279735565185546875f,-1.303199994269013561876136009232141077518463134765625f,0.538600017422437904457410695613361895084381103515625f,-1.032000032544136303158666123636066913604736328125f,0.67760001038908956871864575077779591083526611328125f,-0.551600021386146721624754718504846096038818359375f,-0.303300001826882326128043132484890520572662353515625f,0.22179999626874924434360991654102690517902374267578125f,-0.162200003179907792372205221909098327159881591796875f,-0.0698999873757362166770690237171947956085205078125f,0.611199996891617747252212211606092751026153564453125f,-0.0820999945044516010739243938587605953216552734375f,1.4874999850988377403382401098497211933135986328125f,-1.622399982905387449960699086659587919712066650390625f,0.830500009179115039614771376363933086395263671875f,0.67579998625516868315799001720733940601348876953125f,-0.7871999862670897574190576051478274166584014892578125f,0.37460000508427615528717069537378847599029541015625f,-1.527200036931038340526356478221714496612548828125f,1.43260001694560035190306734875775873661041259765625f,-0.8373000355482105305782170034945011138916015625f,-0.387600044214725159008594346232712268829345703125f,0.1542999926924704112707331660203635692596435546875f,0.664400015896558837624752413830719888210296630859375f,0.2740000038444996288689026187057606875896453857421875f,0.9142000311046840632656085290363989770412445068359375f,-0.5571999953567983165925170396803878247737884521484375f,-0.0911000018134713318573858487070538103580474853515625f,-0.5279000162243843163878409541212022304534912109375f,0.194600004711747320840231623151339590549468994140625f,0.2842000025317072964270437296363525092601776123046875f,0.8343000147461889781652644160203635692596435546875f,0.17480000075399859671421154416748322546482086181640625f,-0.142700000736117382782452978062792681157588958740234375f,-0.581899999803304535106462935800664126873016357421875f,-0.196800028306245877729452331550419330596923828125f,-0.2691000048086047424789057913585565984249114990234375f,-0.6373000233292582805688653024844825267791748046875f,0.28400003513693850010213282075710594654083251953125f,0.1957000133052468893168907015933655202388763427734375f,-0.2240999602913849031438076053746044635772705078125f,-0.472599988000094839435405447147786617279052734375f,-0.0676999984078109073859508271198137663304805755615234375f,-0.273800003220140641957414118223823606967926025390625f,-0.662000009000301314898706550593487918376922607421875f,-0.2395000018849968714818032822222448885440826416015625f,-1.0579000358939172343752943561412394046783447265625f,-0.266400045907497773356453762971796095371246337890625f,-0.1911000111266971668300129749695770442485809326171875f,0.2478999822497360838724489440210163593292236328125f,-0.125400002887845030130620216368697583675384521484375f,-0.0756000031396746319245494305505417287349700927734375f,0.2232999826788901742702364572323858737945556640625f,0.42140001603066934432462176118860952556133270263671875f,0.1705000043362379369416004237791639752686023712158203125f,0.4090000197142364957159088589833118021488189697265625f,0.414200012031197772888191366291721351444721221923828125f,0.0892000049903989289390437988913618028163909912109375f,0.30470001919865641326623517670668661594390869140625f,-0.76939999853670570928443339653313159942626953125f,-0.232299998965859344490780813430319540202617645263671875f,-0.321600019150972027404122854932211339473724365234375f,-0.365200003194809053042035884573124349117279052734375f,-0.13609999950379136901545962246018461883068084716796875f,-0.85530000300407404978386693983338773250579833984375f,-0.510000026971102027317783722537569701671600341796875f,0.0348999909520149032005065237171947956085205078125f,0.4126000088244676788917786325328052043914794921875f,-0.753200001198053303852475437452085316181182861328125f,0.260400005421042468611858566873706877231597900390625f,-0.57829997045993852822221015230752527713775634765625f,1.1007999661386016487796268847887404263019561767578125f,-0.350099997723102518421001150272786617279052734375f,-0.3750999875903122671161327161826193332672119140625f,0.362800010827183161410403045010752975940704345703125f,-0.185900012588501084564995835535228252410888671875f,0.79269999168366123143414370133541524410247802734375f,-0.70100000781565852125964966035098768770694732666015625f,0.41640001485720279372770846748608164489269256591796875f,0.2232999507904054325990728102624416351318359375f,-0.391399961715936939299353980459272861480712890625f,0.114999991059303141582859097979962825775146484375f,0.582500051259994933161578956060111522674560546875f,0.31459996544718737965240507037378847599029541015625f,0.275200016903877298091174452565610408782958984375f,-0.61939997149109871088512591086328029632568359375f,0.8881999639451507722043288595159538090229034423828125f,-0.35609999592751251373812237943639047443866729736328125f,-0.3204999871253964016659665503539144992828369140625f,0.964999994933604998692544540972448885440826416015625f,-0.2261999986052511957268507103435695171356201171875f,-0.5701000874280932606552596553228795528411865234375f,-0.345599942529201786811654528719373047351837158203125f,-0.26710003533363391170496470294892787933349609375f,-0.3511000023722647966906151850707828998565673828125f,-0.154599999347329042365117857116274535655975341796875f,-0.1653999971508977040457466500811278820037841796875f,0.385600018152594703479962845449335873126983642578125f,-0.40720000809729095525568709490471519529819488525390625f,0.220800009575486289037371534504927694797515869140625f,0.4833000032424930036967225532862357795238494873046875f,-0.683999998107552631410044341464526951313018798828125f,0.279500009447336328349820178118534386157989501953125f,-1.04359999238848644154131761752068996429443359375f,0.86779998266398905570895294658839702606201171875f,-0.57010000144839256819295769673772156238555908203125f,0.0197000020623203653258315171115100383758544921875f,0.649799987864494443812191093456931412220001220703125f,-0.03680000193119070672764792107045650482177734375f,-0.0690000251531603936427927692420780658721923828125f,-0.24740000704526909913738563773222267627716064453125f,-0.31739999005794528041946023222408257424831390380859375f,-0.036899996860325356973220323197892867028713226318359375f,-0.556599990540742606270896430942229926586151123046875f,-0.4150000070780510696977216866798698902130126953125f,-0.27979999401122324087509696255438029766082763671875f,-0.74730000306367827533904346637427806854248046875f,0.4150000099092723271354543612687848508358001708984375f,0.15560000433176757184128291555680334568023681640625f,0.522300026905536896038029226474463939666748046875f,-0.81760000353455541954872387577779591083526611328125f,-0.1583000020131468676964914266136474907398223876953125f,-0.2777000201821326896833852515555918216705322265625f,-0.891800023165345567832673623342998325824737548828125f,-0.28540000806599863381052273325622081756591796875f,-0.99330003572702452174780773930251598358154296875f,-0.6648000052988525254704654798842966556549072265625f,-0.330299995665252144050327842705883085727691650390625f,-0.754900007045268495176060241647064685821533203125f,-0.96539999662935738200530977337621152400970458984375f,-0.2645999988630414190282635900075547397136688232421875f,-1.0525000080466266894063664949499070644378662109375f,0.44699998195469337058938208429026417434215545654296875f,0.20789999275505532949637199635617434978485107421875f,0.4911999962210646941684899502433836460113525390625f,-0.8088000346243386928790641832165420055389404296875f,-0.2169000121340156628235718017094768583774566650390625f,-0.53110004305243574407313644769601523876190185546875f,0.505599987083673596544031170196831226348876953125f,0.107000000879168499778870682348497211933135986328125f,0.899800024819374044682263047434389591217041015625f,0.4137999931246039153620586148463189601898193359375f,0.21829999549388878765654453673050738871097564697265625f,0.53420000241994802792078189668245613574981689453125f,-0.48500000961124900555887506925500929355621337890625f,-0.184299999696016347794369494295096956193447113037109375f,-0.698200000602006820571432399447076022624969482421875f,-0.7854000428229575003769014074350707232952117919921875f,-0.312600012363493551248438961920328438282012939453125f,-0.47610003694295954801418702118098735809326171875f,0.282000012129544952443893635063432157039642333984375f,0.2217000007435678821110514036263339221477508544921875f,-0.0106999840617181263269230839796364307403564453125f,-0.8948000362932686524430891950032673776149749755859375f,-0.26009998844712978183224549866281449794769287109375f,0.561000027298927506080872262828052043914794921875f,0.901199993538856602981468313373625278472900390625f,0.2301999967873096242243491360568441450595855712890625f,0.3435000094175340024094111868180334568023681640625f,0.59190000263452535733676995732821524143218994140625f,0.65709999238103622420936744674690999090671539306640625f,-0.357000026211142795773412217386066913604736328125f,-0.4974000043630599776633971487171947956085205078125f,0.487799994510412115999997695325873792171478271484375f,-0.08449999290704734988821655861102044582366943359375f,-0.351099999094009351807699204073287546634674072265625f,-0.355499987423420094057036067169974558055400848388671875f,0.8784999915361400013580350787378847599029541015625f,-0.095500008136033986971824560896493494510650634765625f,0.1895000008046625605828694460797123610973358154296875f,0.3699999953806401009615001385100185871124267578125f,-0.17499999791383746838846491300500929355621337890625f,0.22879998761117470440495935690705664455890655517578125f,-0.05600000613927846870865323580801486968994140625f,-0.751000021040439502684193939785473048686981201171875f,-0.00640001103878018451354137141606770455837249755859375f,0.248000050306320218851396930404007434844970703125f,1.05009997522234943545527130481787025928497314453125f,-0.4675999784260991276596541865728795528411865234375f,0.19250000238418607523271930404007434844970703125f,0.0610000034570692406532543827779591083526611328125f,-0.53050001044571359898327500559389591217041015625f,0.2545000225976110641568084247410297393798828125f,0.218900001204013872069253920926712453365325927734375f,-0.3446999908864498696203781946678645908832550048828125f,-0.6599999934807418977555926176137290894985198974609375f,0.51389997173696766452621886855922639369964599609375f,0.144199994745850545996290748007595539093017578125f,-1.1489999923706051276894868351519107818603515625f,0.4241999772369862142795682302676141262054443359375f,-0.397200005601346273209628634504042565822601318359375f,0.103500024318694983094246708787977695465087890625f,0.4439999880492688788535815547220408916473388671875f,-0.2328999841496350864389341950300149619579315185546875f,-0.02449999933317303923985264191287569701671600341796875f,-0.0174999997578561272693065120620303787291049957275390625f,-0.03109999877735973583536832620666245929896831512451171875f,0.8495000113844870526236263685859739780426025390625f,0.3259000132031741880922481868765316903591156005859375f,-0.417899998324364307933365125791169703006744384765625f,0.9546000203952191842660113252350129187107086181640625f,-0.053400020970404693532174178471905179321765899658203125f,-1.1902999827384945774610969237983226776123046875f,0.79340005609989106005741632543504238128662109375f,0.1449999865889548544828357989899814128875732421875f,-0.44529999629855154807955841533839702606201171875f,-0.732499994188547276507961214520037174224853515625f,0.5789000159561634717420020024292171001434326171875f,-0.35019997160434712668575230054557323455810546875f,0.7275999920606608384332503192126750946044921875f,0.8220000116825101343920323415659368038177490234375f,-0.052299985145032312772173099801875650882720947265625f,-0.518899997925758693639863849966786801815032958984375f,-0.0967000126272441473673779910313896834850311279296875f,-0.99829999523311840903261327184736728668212890625f,-0.38199996694177418277149627101607620716094970703125f,-0.105600017072260410344597403309307992458343505859375f,-0.1843000105738639149421942420303821563720703125f,0.5332000204205513682609307579696178436279296875f,-0.5818000350117689123408126761205494403839111328125f,-0.426099995964765554390396573580801486968994140625f,-0.0532999993681908534881586092524230480194091796875f,0.700100019180774690852331332280300557613372802734375f,-0.0828999956607816557152546010911464691162109375f,-2.2210999526023869066193583421409130096435546875f,0.1663000195741659581472049467265605926513671875f,0.22330001069307314764955663122236728668212890625f,0.1231999643921852793937432579696178436279296875f,-0.5626000078558917039117659442126750946044921875f,-0.50269999753236760398067417554557323455810546875f,-1.12779997469186810121755115687847137451171875f,0.71740001628398886168724857270717620849609375f,-0.21200004327297250483752577565610408782958984375f,-1.95370002274513154816304449923336505889892578125f,0.338700037646294305204719421453773975372314453125f,0.8751999704122539469608454965054988861083984375f,0.474799948525429726942093111574649810791015625f,-0.82139999226331639192721922881901264190673828125f,0.97599998557567602119888761080801486968994140625f,-1.13769998054504384299434605054557323455810546875f,0.0297000235199931239549187012016773223876953125f,-0.838900000905990594901595613919198513031005859375f,0.453999984119087596212693824782036244869232177734375f,0.37450002660229830819815788345295004546642303466796875f,0.571100004737824296086046160780824720859527587890625f,0.15429999967738983190912449572351761162281036376953125f,0.687499995827675292048297706060111522674560546875f,0.70390001714825611855985698639415204524993896484375f,0.70409996975362343363258332828991115093231201171875f,-0.425500018864870099832842242904007434844970703125f,-0.105800007116794603234666283242404460906982421875f,0.3270999885439873633430352128925733268260955810546875f,-0.8230000050067900474459747783839702606201171875f,0.73419997500181199967528300476260483264923095703125f,-0.0426000096440313225087948012514971196651458740234375f,0.1684999954104424713108301148167811334133148193359375f,-0.2984000043556094272645395903964526951313018798828125f,0.147099985116720344269225506650400348007678985595703125f,0.88049999178200977212327416054904460906982421875f,-0.043499988704919889670463817310519516468048095703125f,0.8170999737545849850306467487826012074947357177734375f,0.897000034630298870297337998636066913604736328125f,0.12210001477003107339669440989382565021514892578125f,0.16760000424236043414794039563275873661041259765625f,-0.60250001117587093091287897550500929355621337890625f,0.2925999889314174762233733417815528810024261474609375f,0.171699995937943494794808430015109479427337646484375f,-0.14450004863738996618849341757595539093017578125f,-1.107999974906444862199350609444081783294677734375f,0.403299968299270172877868390060029923915863037109375f,0.571999987542628929304555640555918216705322265625f,-0.16910001006126407219198881648480892181396484375f,-0.0646000038549303656765232517500407993793487548828125f,-0.562500018775462962139499722979962825775146484375f,-0.257700000065565149043322890065610408782958984375f,0.4068999840825797686960640930919907987117767333984375f,-0.276500029921532330945410649292171001434326171875f,-0.88210000076293937354421359486877918243408203125f,0.4155000137612219202054575362126342952251434326171875f,0.42650000369548735079661128111183643341064453125f,-0.197599997872114130359477712772786617279052734375f,-0.545899965538084952498820712207816541194915771484375f,-0.756000012993812475770027958787977695465087890625f,-0.7856999788463117084802433964796364307403564453125f,-0.5259999904185532759726129370392300188541412353515625f,0.3464999992251396321307765902020037174224853515625f,0.636700022089481620923834270797669887542724609375f,1.338600001180171972237076261080801486968994140625f,-0.500100011134147592883891775272786617279052734375f,-0.477699988293647237469485844485461711883544921875f,-0.29879998351335501638459390960633754730224609375f,-1.98569996081590716130449436604976654052734375f,0.3871000105977060457007610239088535308837890625f,0.221499999150633897215811884962022304534912109375f,-0.370599987307191014451746013946831226348876953125f,0.332600005844235102614447896485216915607452392578125f,0.2138000028848647815493677626363933086395263671875f,-0.226100011610984807930435636080801486968994140625f,-0.017300000679492999466901892446912825107574462890625f,-0.04380003983974489756292314268648624420166015625f,-1.64880001540183940278438967652618885040283203125f,0.03320002727508608586504124104976654052734375f,-0.53660001721382144523886381648480892181396484375f,-0.1953999777793882230980671010911464691162109375f,0.4552000219702723171621983055956661701202392578125f,0.26230000682622200969262848957441747188568115234375f,0.437899996480345610194717664853669703006744384765625f,-0.77230003890097187468910533425514586269855499267578125f,1.109000015020370710772112943232059478759765625f,-0.61579998595714613429663586430251598358154296875f,0.0648000036597249362557704444043338298797607421875f,-0.281300014805793807681766338646411895751953125f,-0.162099990928172843496213317848742008209228515625f,-0.47950001853704460330618530861102044582366943359375f,-1.310999971866607438641949556767940521240234375f,0.370500034809111866707098670303821563720703125f,0.28049999308586137658494408242404460906982421875f,0.31650001114606851615462801419198513031005859375f,1.67789997926950462669992703013122081756591796875f,-0.4018999919056884806423113332130014896392822265625f,0.4357000165462494578605401329696178436279296875f,0.704299987924098758185209590010344982147216796875f,-0.48190001991987241325432478333823382854461669921875f,-1.10449999737739545935255591757595539093017578125f,0.43250004231929750631024944595992565155029296875f,-0.140500025868416145158334984444081783294677734375f,0.00119999175071683339410810731351375579833984375f,2.11289994380474155377669376321136951446533203125f,-0.24049999457597692753552109934389591217041015625f,0.146000006139278326600106083787977695465087890625f,0.732999988801777302427353788516484200954437255859375f,-0.3625000137090683693941173260100185871124267578125f,-0.68989998916387573757447171374224126338958740234375f,-0.79169999075978980673795604161568917334079742431640625f,-0.275500039279460651187037001363933086395263671875f,-0.865699990022182674920259159989655017852783203125f,0.4740999643519525630352973166736774146556854248046875f,-0.59849999473988990672523868852294981479644775390625f,-0.1274999988079070778468349089962430298328399658203125f,-0.169499997153878201316956619848497211933135986328125f,-0.1345000002086162993464313331060111522674560546875f,-0.1679000063896178840394668441149406135082244873046875f,-0.114099998550117043283336215608869679272174835205078125f,-0.002000023603439871067166677676141262054443359375f,-0.8565999933719634640283402404747903347015380859375f,0.3056000127881761674331073663779534399509429931640625f,-0.19850001180171972237076261080801486968994140625f,0.15560001092553132906459723017178475856781005859375f,0.3287000006541609398169612177298404276371002197265625f,0.8165000274255873602413657863507978618144989013671875f,-0.080599997551739133161419204043340869247913360595703125f,-0.102099999906122729242952118511311709880828857421875f,-0.903499999284743893213089904747903347015380859375f,-0.52239999437928208436687782523222267627716064453125f,-0.719500000022351482442672931938432157039642333984375f,0.8385000106096267558086765347979962825775146484375f,-0.193799987983703569938143118633888661861419677734375f,0.3351000002562998236754765457590110599994659423828125f,0.601499977469444502276019193232059478759765625f,0.548700002181529811196014634333550930023193359375f,0.83549996078759480866438025259412825107574462890625f,0.283499990642070542889996431767940521240234375f,0.69360000088214857072443919605575501918792724609375f,-0.4314999940097330455301971596782095730304718017578125f,0.3055000121593476336556705064140260219573974609375f,0.125599996173381800446122724679298698902130126953125f,-0.3359000003695487013288811795064248144626617431640625f,1.000500010371208503556772484444081783294677734375f,0.39840001497268673347207368351519107818603515625f,0.7634999797642232355343594463192857801914215087890625f,0.303000013351439889675020822323858737945556640625f,1.0412999809801579687018602271564304828643798828125f,-0.3093999725341800566269512273720465600490570068359375f,0.5190000025033949526687138131819665431976318359375f,-0.62180002517700216912999167107045650482177734375f,0.86520001690387715598262730054557323455810546875f,0.688799954009056136783328838646411895751953125f,-0.9184000131845468928304399014450609683990478515625f,1.286499978028237567428959664539434015750885009765625f,-0.382000044763087576171756154508329927921295166015625f,-0.7140000057071447248091544679482467472553253173828125f,-0.40449997949600202673536841757595539093017578125f,-1.46719997851848660275209113024175167083740234375f,0.4652000106453897387837059795856475830078125f,0.7867000004827975345733648282475769519805908203125f,0.1861999693989755400025387643836438655853271484375f,-0.43180000997781764482397193205542862415313720703125f,0.33899998223781580009017488919198513031005859375f,-1.262399992144107585545498295687139034271240234375f,0.23340002391338376952489852556027472019195556640625f,-0.92630002069175265688727449742145836353302001953125f,-0.12970001368522598994559302809648215770721435546875f,0.428100004070997020022559809149242937564849853515625f,-0.81220000027418137733548064716160297393798828125f,0.052600005769729563098735525272786617279052734375f,0.70700000870227786009536430356092751026153564453125f,0.7254000029623508538634268916212022304534912109375f,-0.6870000134706497618708453956060111522674560546875f,-0.34320001237392450121888032299466431140899658203125f,0.743500007983297184210869090748019516468048095703125f,-1.5010000083371999313186506697093136608600616455078125f,-0.209000017516315528265380407901830039918422698974609375f,0.3448999913245440751552450819872319698333740234375f,0.01120002378821371991080013685859739780426025390625f,-0.9654000411093239808479893326875753700733184814453125f,0.256199994134902908626827411353588104248046875f,1.72560000064373042505394550971686840057373046875f,0.0138000252366072828635878977365791797637939453125f,-1.05969998060464831723948009312152862548828125f,0.31150004631280836520090815611183643341064453125f,0.16509998431205730184956337325274944305419921875f,-0.34710002542436146910631578066386282444000244140625f,-1.33260001769065805632408228120766580104827880859375f,-0.292600010985136105290393970790319144725799560546875f,-0.29850000222772354874223310616798698902130126953125f,0.496200013431906672423110649106092751026153564453125f,0.01979999196231341240803658365621231496334075927734375f,-0.40260000338554391419165767729282379150390625f,0.3466000067830081743380787884234450757503509521484375f,0.55999999523162813375165569595992565155029296875f,-0.11480000872612006190820466144941747188568115234375f,-0.5543999959290026868785616898094303905963897705078125f,-0.0144999818801883062491242526448331773281097412109375f,-0.6971000028863547637314468374825082719326019287109375f,-0.7234999678432945113826235683518461883068084716796875f,-0.302500035762786723125827847979962825775146484375f,-0.5429000017702581004641615436412394046783447265625f,0.4464999679327015247309873302583582699298858642578125f,-0.01749998599290858436461348901502788066864013671875f,-0.20150000816583624185795997618697583675384521484375f,-0.514899985215067967914137625484727323055267333984375f,-0.5505000169277192156869205064140260219573974609375f,-0.658199990171194126986620176467113196849822998046875f,-0.082499994374811702613214947632513940334320068359375f,0.125500006794929230835577982361428439617156982421875f,0.257599986472726083519546591560356318950653076171875f,0.60810000690221743813168586711981333792209625244140625f,-0.3260000001788139201153171597979962825775146484375f,0.426799988222122284042825413052923977375030517578125f,0.434399995109438830098724793060682713985443115234375f,-0.08349999213218683991044599679298698902130126953125f,-0.6236000032663344683214745600707828998565673828125f,-0.340299995963275481170029479471850208938121795654296875f,-0.26849999267235380440155267933732829988002777099609375f,-1.014300007053464458550706694950349628925323486328125f,-0.24350000351294875944319073823862709105014801025390625f,1.011000031545758570672433052095584571361541748046875f,-0.38909997645914551345214249522541649639606475830078125f,-0.048799997632205549535910904523916542530059814453125f,-0.217500004768371724139797152020037174224853515625f,0.87180001161694509193011981551535427570343017578125f,-0.483899983285367429886036916286684572696685791015625f,0.56699999529123346064807265065610408782958984375f,0.4290000101029871615310185006819665431976318359375f,0.68639996898025312788149676634930074214935302734375f,0.648500001668929737519420086755417287349700927734375f,-0.739700003030896002798044719384051859378814697265625f,0.54270000349283176177550558350048959255218505859375f,0.1925000125542284390434133456437848508358001708984375f,0.328900004296004799808628149548894725739955902099609375f,0.129500000394880743215253460220992565155029296875f,0.77949999439716322058302466757595539093017578125f,-0.472600007259845522611385604250244796276092529296875f,0.972600003497302356691989189130254089832305908203125f,-0.178800057423114555632537303608842194080352783203125f,-0.4480000177472833744474201012053526937961578369140625f,-0.76030003489255904014498810283839702606201171875f,-0.100099953317641876537891221232712268829345703125f,0.76750001817941626569563595694489777088165283203125f,-0.23149996554851526298079988919198513031005859375f,0.62539997777938793888097279705107212066650390625f,0.83160001900196078850058256648480892181396484375f,-0.26310000146329404202560908743180334568023681640625f,-0.11939999697208403262038700631819665431976318359375f,-0.009800001424550952577874340931884944438934326171875f,0.7839999977350231574746430851519107818603515625f,1.45909992661476195507930242456495761871337890625f,-0.4180999911069864793944361736066639423370361328125f,-0.42199997770786268347364966757595539093017578125f,-0.46839999232292228725782479159533977508544921875f,-0.695600054883957596985055715776979923248291015625f,-0.91590002808570858405801118351519107818603515625f,-0.9716999526500700312681146897375583648681640625f,0.9669000226020809662941246642731130123138427734375f,0.2630000014118849538391486930777318775653839111328125f,-1.2890000217258930081953849366982467472553253173828125f,-0.47200002237409377503496443750918842852115631103515625f,-0.033700001287460423782249563373625278472900390625f,0.7707999799966813725404790602624416351318359375f,0.61180001742839795753070575301535427570343017578125f,0.106200003075599624935421161353588104248046875f,1.1756999813795090403800713829696178436279296875f,0.6877000151157375285038142465054988861083984375f,-0.45670003650635493386289454065263271331787109375f,-1.22419998412877273352705742581747472286224365234375f,0.478800008137523658291456740698777139186859130859375f,0.64050003719329851037400658242404460906982421875f,0.81779998810291232302915886975824832916259765625f,0.1702000088572503955219872295856475830078125f,-0.19649998775124544181380770169198513031005859375f,0.06570000090002992010340676642954349517822265625f,-0.41670001896023745047159536625258624553680419921875f,0.491500021837651956957415677607059478759765625f,1.39160001069456296107773596304468810558319091796875f,-0.675600033761561302725340283359400928020477294921875f,0.0816999967575071650571771897375583648681640625f,0.05419999478310355556232025264762341976165771484375f,-0.1319999788999559608981826386298052966594696044921875f,-0.4374000252619383299901301143108867108821868896484375f,-0.585199981290102311248801925103180110454559326171875f,-0.4429999856352804243897480773739516735076904296875f,0.356000018507242277365776317310519516468048095703125f,0.40960001197606299427889098296873271465301513671875f,-0.923000017523765592386553180404007434844970703125f,0.6598999686002731124290221487171947956085205078125f,-0.1140000119283792745505934362881816923618316650390625f,0.208499996304511991951358140795491635799407958984375f,-0.015299998906254774766466653090901672840118408203125f,0.2146999955058097875326694747855071909725666046142578125f,-0.040499994277954698418398038484156131744384765625f,0.48029997677803049072053909185342490673065185546875f,-0.73249996978789599921810804517008364200592041015625f,1.228000016927719428849741234444081783294677734375f,-0.40240000376701345885521732270717620849609375f,0.5359999732077123102413906963192857801914215087890625f,-0.58450000154972059362989966757595539093017578125f,-0.00919998662471765982218130375258624553680419921875f,0.6292999995470045515588708440191112458705902099609375f,0.43150000220537176875978957468760199844837188720703125f,-0.89680000902786838334890262558474205434322357177734375f,0.127499994561076024357504365980275906622409820556640625f,-0.7819999917149544899075408466160297393798828125f,0.623899981832504391121574371936731040477752685546875f,-0.23659998476654309218503158263047225773334503173828125f,-0.9874999821186065673828125f,0.837499979734420918475734652020037174224853515625f,-0.42389998112469928770451588206924498081207275390625f,-0.282999988056719331552812946029007434844970703125f,-0.398899994647502698086327654891647398471832275390625f,0.4404000123500819707800246760598383843898773193359375f,-0.702500011920928812969577847979962825775146484375f,0.3670999872028826160885728313587605953216552734375f,-0.6435000077038999233280947009916417300701141357421875f,0.6919999975264072844538532081060111522674560546875f,-0.067999998897314117840551261906512081623077392578125f,0.220999986059963848816778408945538103580474853515625f,0.51450002766400615428210585378110408782958984375f,0.414300007742643305874707948532886803150177001953125f,-0.4525000017508862271142788813449442386627197265625f,0.480500000238418323306177626363933086395263671875f,0.433400022125244532844590139575302600860595703125f,0.16149998015165323295150301419198513031005859375f,-0.2202999913811680698927375487983226776123046875f,0.55030002654790877159030060283839702606201171875f,0.391999991267919511983564007095992565155029296875f,-0.7375999989151953428745400742627680301666259765625f,0.089600013577938231179587091901339590549468994140625f,-0.3232000195264816966300713829696178436279296875f,1.3573000164747242024532170034945011138916015625f,0.401300030303001875608970294706523418426513671875f,-0.2442999942570924787332842242904007434844970703125f,0.63279999540448184092156225233338773250579833984375f,0.0975999984681605869951681597740389406681060791015625f,0.7845000042319298927395720966160297393798828125f,-0.6170999998688699861304485239088535308837890625f,0.1230999896168700757925762445665895938873291015625f,0.61600001597404485664810636080801486968994140625f,0.15319999568462350225672707892954349517822265625f,0.513800041925907891027236473746597766876220703125f,-0.7281000225484373089557266212068498134613037109375f,-0.9204000037074084872301682480610907077789306640625f,-0.08180000312328328249122932902537286281585693359375f,0.611399996994435923625133000314235687255859375f,-0.25619998277276767595367346075363457202911376953125f,0.299000001870095122313841784489341080188751220703125f,1.104000016212463464171378291212022304534912109375f,0.032699980247020477008845773525536060333251953125f,-0.15270001034736679912384715862572193145751953125f,-0.8531000060081481706220074556767940521240234375f,0.918800017786026046451297588646411895751953125f,-0.078000018417835548234506859444081783294677734375f,0.295000005662441111553562222979962825775146484375f,1.907599957489967579249423579312860965728759765625f,-0.0226000024914743136150718783028423786163330078125f,-0.0865000013113021992694484652020037174224853515625f,-0.6029000005781650628478018916212022304534912109375f,-0.0031000025808811670913200941868126392364501953125f,-0.4477000069200993692675183410756289958953857421875f,-1.7567000104665755344512945157475769519805908203125f,-0.06670001165866867864906453178264200687408447265625f,0.49270001161396503874811969581060111522674560546875f,-0.10330001963377011264810789725743234157562255859375f,-0.172300007385015607752620780956931412220001220703125f,-0.38030000240802763755709747783839702606201171875f,0.74260000077784049654638920401339419186115264892578125f,-0.840499999344348935892412555404007434844970703125f,0.038000004708766965677568805404007434844970703125f,-0.1974999835342170495522395867737941443920135498046875f,-0.40400000861287121978193681570701301097869873046875f,-0.4383999951541424078271802500239573419094085693359375f,-0.1799999947845936087986729035037569701671600341796875f,-0.2945000021457673256009002216160297393798828125f,0.9209000140786169907869407325051724910736083984375f,-0.080099983604252411595325611415319144725799560546875f,0.049500003561377514671448807348497211933135986328125f,0.41729999375045301324149704669252969324588775634765625f,0.05009999936223030747584061828092671930789947509765625f,-0.739499976813792869734243140555918216705322265625f,-0.43980000634193405772975893341936171054840087890625f,-0.49449999119341381259573608986102044582366943359375f,-1.131500034689903344542472041212022304534912109375f,0.135199997979402297687556711025536060333251953125f,-0.5855000010207291882835534124751575291156768798828125f,0.65199999409913989012466117856092751026153564453125f,-0.3744999987185000378531185560859739780426025390625f,0.7061999975249169114732694652047939598560333251953125f,-0.8439999805614348815652192570269107818603515625f,-0.154399995779991083821869324310682713985443115234375f,-0.46679998878091577552140734042041003704071044921875f,-0.6179999826550481856202168273739516735076904296875f,-0.25050001171231262020455687888897955417633056640625f,-0.77999999038875100776380122624686919152736663818359375f,0.7295000194311143104641814716160297393798828125f,0.6289000055253506360486426274292171001434326171875f,0.483699997487664123951844885596074163913726806640625f,-0.1994999753236772477293925476260483264923095703125f,0.94859997808337226388175622560083866119384765625f,-0.53549997695535445085823766930843703448772430419921875f,0.07149999609589574589563198969699442386627197265625f,-0.304600001507997486527301589376293122768402099609375f,0.166499995425343494392933507697307504713535308837890625f,0.42899999058246596206345202517695724964141845703125f,-0.86880000884532915250701989862136542797088623046875f,0.64039998727887859075735832448117434978485107421875f,-0.138499996900558475232401178800500929355621337890625f,-0.209800010439753492619274766184389591217041015625f,-0.30589999895393849005387210127082653343677520751953125f,0.634499987244605989644696819595992565155029296875f,-0.1709000143952665862201456548064015805721282958984375f,1.15120004181861901315642171539366245269775390625f,0.84769998948573999086875119246542453765869140625f,-0.575300014030933226649722200818359851837158203125f,0.31079999430179583441713475622236728668212890625f,0.8483999655008318541149492375552654266357421875f,0.7812000066518773877533021732233464717864990234375f,-0.85050001759827154046433861367404460906982421875f,-0.3714000038117166724305207026191055774688720703125f,-0.17960000858604896478709633811376988887786865234375f,-0.21899997374415391959701082669198513031005859375f,0.8366999791741367431541220867075026035308837890625f,0.198299983274936142407796069164760410785675048828125f,-0.4362999864190815202391604543663561344146728515625f,-0.71329997306764170161841320805251598358154296875f,0.1796999999016524185435628169216215610504150390625f,1.2426999701142307230838923715054988861083984375f,-0.52120005522966295075093512423336505889892578125f,-0.005399961984156931293910020031034946441650390625f,0.193100008131563694036003653309307992458343505859375f,-0.643400006031990212562732267542742192745208740234375f,-0.2184000111728907622676842947839759290218353271484375f,-0.58350001347810032115148715092800557613372802734375f,1.041600022950768700269463806762360036373138427734375f,0.07140001439154197715453165073995478451251983642578125f,-0.517299990993738134648083359934389591217041015625f,-0.4127000028967859890371983055956661701202392578125f,0.822099999570845785257233728771097958087921142578125f,0.866199971187114670101436786353588104248046875f,-1.036899983859061791235944838263094425201416015625f,0.10730003048181568914287709048949182033538818359375f,-0.4702000431299211413715966045856475830078125f,-1.8961999804258340418527950532734394073486328125f,0.13180001564025900506749167107045650482177734375f,0.1081000086903571855145855806767940521240234375f,1.6393999787926674116533831693232059478759765625f,-0.2322000184535983891009891522116959095001220703125f,0.04790000929534432572154400986619293689727783203125f,0.93609998180866238470798634807579219341278076171875f,0.192700002151727556309879219043068587779998779296875f,0.250000007450580596923828125f,-0.094300018024444653974569519050419330596923828125f,-0.80770004223585178948496832163073122501373291015625f,-0.56240000387877220333621153258718550205230712890625f,0.5711999682255093357952091537299565970897674560546875f,-0.103000002622604114321802626363933086395263671875f,0.03009999757409076437397743575274944305419921875f,-0.8634000090122218029620171364513225853443145751953125f,-1.2599999606609344482421875f,0.65519996802806890201509304461069405078887939453125f,-0.5375999680697922666894328358466736972332000732421875f,0.867999997630715114382837782613933086395263671875f,-0.09189999905824652870478530530817806720733642578125f,0.7229999959543345067203290454926900565624237060546875f,-0.10249998524785031150940994848497211933135986328125f,0.5548999738156796812660331852384842932224273681640625f,-0.02450000613182778952392482096911408007144927978515625f,0.09149998052418251148765193647705018520355224609375f,-0.232799993914365899172480567358434200286865234375f,0.402099975505471551873171165425446815788745880126953125f,-0.91100000971555772366627934388816356658935546875f,-0.52210000880956641822194796986877918243408203125f,-0.818399960844219176436808993457816541194915771484375f,0.1469999973773956636335924486047588288784027099609375f,-0.4539000036999583098662469637929461896419525146484375f,0.014799992297589825707149202571599744260311126708984375f,0.3329999937936662934845344352652318775653839111328125f,0.6921000179365275339904428619774989783763885498046875f,0.234000002652406724479305921704508364200592041015625f,-0.69299999025464043000965830287896096706390380859375f,0.1607999761223795065490094202687032520771026611328125f,0.2735999945864080462598622034420259296894073486328125f,-0.7770000095963478514704547706060111522674560546875f,-0.684299977344274612534036350552923977375030517578125f,-0.4793999806553126408203979735844768583774566650390625f,0.185999986767768632489605806767940521240234375f,-0.871199981319904281917843036353588104248046875f,0.638400004318356195653905160725116729736328125f,0.215500029146671323587725055404007434844970703125f,0.77149998775124561944949164171703159809112548828125f,-0.30869998765289796693878088262863457202911376953125f,-0.293499991536140480974381716805510222911834716796875f,0.553599985608458577956980661838315427303314208984375f,-0.23529999316930771957601109534152783453464508056640625f,0.8750000241398812050874767010100185871124267578125f,-0.373100006157159658215505260159261524677276611328125f,0.0049999979138373173981335639837197959423065185546875f,0.4910000212267042751790313559467904269695281982421875f,0.1220000014007087685286023770458996295928955078125f,-0.8872000339694323667316666615079157054424285888671875f,-0.2783000027954571908850311956484802067279815673828125f,0.32890001077800989914834417504607699811458587646484375f,-1.005400028145313484628786682151257991790771484375f,0.201399997329712476812346721999347209930419921875f,0.2858000050306319650417208322323858737945556640625f,-0.507199985820054877194706932641565799713134765625f,-0.38949998202919999812365858815610408782958984375f,0.365100002044439264636821462772786617279052734375f,-0.78499998152256011962890625f,0.550999996304511796552105806767940521240234375f,0.068000005304813981865663663484156131744384765625f,1.35970002202987672035305877216160297393798828125f,-0.80560004355907466333519550971686840057373046875f,-0.060000005662441679987750831060111522674560546875f,-0.44050000832229851965848865802399814128875732421875f,0.337600018361210896244983814540319144725799560546875f,0.71040002702772631071326259188936091959476470947265625f,0.069200008063763362997633521445095539093017578125f,0.864699987477809184355237448471598327159881591796875f,-0.4863000139303503832621800029301084578037261962890625f,0.30529996962547301109225372783839702606201171875f,1.19050002646446273502078838646411895751953125f,0.7661000387310981096788964350707828998565673828125f,-1.472999999940395099429224501363933086395263671875f,0.568800072622298813485031132586300373077392578125f,0.6082000084996224131828057579696178436279296875f,-1.443800008845329330142703838646411895751953125f,0.34010001828670510803931392729282379150390625f,0.342299997699260671879528672434389591217041015625f,-0.938900013422966139842174015939235687255859375f,-0.25690000114440891820777324028313159942626953125f,0.0500999950408935745826966012828052043914794921875f,0.558700003671646072689327411353588104248046875f,-0.04900002038478845634017488919198513031005859375f,-0.5886000142931937517687401850707828998565673828125f,-1.11300001871585863000291283242404460906982421875f,0.35759999384880103434625198133289813995361328125f,0.36039999208450268497472279705107212066650390625f,1.054000007867813337725237943232059478759765625f,-0.4134999778270724846152006648480892181396484375f,-0.164500003039837139340306748636066913604736328125f,1.1698000076830386007031847839243710041046142578125f,-0.4661999893665313976498509873636066913604736328125f,0.22750000327825514734314538145554251968860626220703125f,0.13729999939426795751984400340006686747074127197265625f,0.62590001132190187238535372671321965754032135009765625f,0.3690000078678128403453229111619293689727783203125f,0.4007999975800515812807134352624416351318359375f,0.63980000947117765530691713138367049396038055419921875f,-0.1010000137388704644081371952779591083526611328125f,0.0160999977529048265978417475707828998565673828125f,0.35939998028278374864186162085388787090778350830078125f,0.48949997293949110144239966757595539093017578125f,0.4695999937593935413815415813587605953216552734375f,0.46549997859448222214950874331407248973846435546875f,-0.924499993801116914937665569595992565155029296875f,-0.714700026351213324460331932641565799713134765625f,-0.8438999926358461411979305921704508364200592041015625f,-0.6075000080838799387805693186237476766109466552734375f,0.506700003500282658563946824870072305202484130859375f,0.1135000032708048944840584226767532527446746826171875f,0.58200000521913164419629538315348327159881591796875f,0.28109999924302098150974416057579219341278076171875f,-0.0944000001199543081753517981269396841526031494140625f,-1.2379999817609785850436310283839702606201171875f,0.83300001677870749716703357989899814128875732421875f,-0.5885999648585917842780190767371095716953277587890625f,0.2904999889135364554704210604541003704071044921875f,0.79700002524256685632053631707094609737396240234375f,0.85049997539073263208564412707346491515636444091796875f,0.61899999833106988944564363919198513031005859375f,0.5795000005066395232233844581060111522674560546875f,0.92459998817145816474294406361877918243408203125f,0.631499997735023299583190237171947956085205078125f,0.06419999735355386150104095577262341976165771484375f,0.719299994555115507210985015262849628925323486328125f,0.358500004649162207215340458787977695465087890625f,-0.33510000174641607628700512577779591083526611328125f,-0.274899996241927180307129674474708735942840576171875f,0.350000020265579081524265347979962825775146484375f,0.5301999876976015713125889305956661701202392578125f,0.6760000065118070278202822009916417300701141357421875f,-0.56299999248981436039684922434389591217041015625f,-0.4818999854981900199391020578332245349884033203125f,-0.6649999974668023838830777094699442386627197265625f,-0.23299997788667692333319791941903531551361083984375f,-0.515500011265277890970537555404007434844970703125f,-0.9284000177294018385509843938052654266357421875f,0.2449999818950896468550126883201301097869873046875f,-0.24680000796616052838317045825533568859100341796875f,-0.382700007516145745967151015065610408782958984375f,-0.1794999946951865155142513685859739780426025390625f,0.338100009584426874909013349679298698902130126953125f,-0.81740002492666263833598350174725055694580078125f,0.28399999684095433849506662227213382720947265625f,-0.0128000044941902757500429288484156131744384765625f,0.380599987083673596544031170196831226348876953125f,0.3850000295042992348726329510100185871124267578125f,-0.832800029230118088463541425880976021289825439453125f,-1.38510003497600564514868892729282379150390625f,-0.3563999585866923780486104078590869903564453125f,0.5472000217318537806931999512016773223876953125f,0.578800021660327956851688213646411895751953125f,1.05319997929334618902430520392954349517822265625f,-0.205600001537799670359163428656756877899169921875f,-0.452499993145465850830078125f,-0.694399996972084210256070946343243122100830078125f,0.79240002909898787919473761576227843761444091796875f,-1.246500032901764143389300443232059478759765625f,0.8526000057697302736414712853729724884033203125f,0.178400004839897263764214585535228252410888671875f,-1.3427999863147732639845344237983226776123046875f,0.593100060844420795547193847596645355224609375f,0.479299993884563235724272090010344982147216796875f,0.918900025045871871043345890939235687255859375f,0.014699982690810742269604816101491451263427734375f,-0.67190003392696429074248953838832676410675048828125f,-0.311400030708313124705455265939235687255859375f,-1.21949999976158096615108661353588104248046875f,-0.77930004574060518507394590415060520172119140625f,0.2818999993562698591631487943232059478759765625f,-1.913900007462501662303111515939235687255859375f,-0.09630002642870039153422112576663494110107421875f,-0.50990002015829105630473350174725055694580078125f,-0.45700000423192932430538348853588104248046875f,-0.47680000044107373469159938395023345947265625f,0.69730002034902582863651332445442676544189453125f,0.076999990224837944197133765555918216705322265625f,-0.61940001753568640907587905530817806720733642578125f,-0.25300002676248567468064720742404460906982421875f,-0.78459999036937944083547336049377918243408203125f,1.01049999971687753941296250559389591217041015625f,0.22819999553561221006248160847462713718414306640625f,0.689300000925361810999447698122821748256683349609375f,-0.15900000384449963775068681570701301097869873046875f,0.0941999988436699187133172017638571560382843017578125f,0.23839999995976679514342322363518178462982177734375f,0.621000011503696214276715181767940521240234375f,0.30660000082254423858785230549983680248260498046875f,0.5518000084877010547046438659890554845333099365234375f,0.80850002864003212010857168934307992458343505859375f,-0.1523999712824821717305212587234564125537872314453125f,-0.1666999925479293320762508301413618028163909912109375f,0.3104999871253966148287872783839702606201171875f,0.267300020796060522343395859934389591217041015625f,0.9566999967575071650571771897375583648681640625f,-0.164999999105930328369140625f,0.3767999909043311390632879920303821563720703125f,-0.55859999738037569017023997730575501918792724609375f,-0.7005000303387642901498111314140260219573974609375f,-0.011700009721517634631027249270118772983551025390625f,0.2804999872744085021025739479227922856807708740234375f,0.68500001013278932759931194595992565155029296875f,0.7577000055789948618212292785756289958953857421875f,0.72800001115351875569103867746889591217041015625f,0.36299999606609389957156963646411895751953125f,0.7407000146090982894975240924395620822906494140625f,0.778599970930815121761270347633399069309234619140625f,0.2005000190138819249341395334340631961822509765625f,-0.400000008344650215263982317992486059665679931640625f,-0.5313000031083821017574564393726177513599395751953125f,1.26000003516674041748046875f,-0.851699992734193500609762850217521190643310546875f,0.60080000592470117481980196316726505756378173828125f,-0.07199998366832716101271216757595539093017578125f,-1.105200011688470596027400461025536060333251953125f,0.140199995483457851008779471158050000667572021484375f,1.005999983489513027734574279747903347015380859375f,-0.304899988567829183239155099727213382720947265625f,0.5517999966040252335375271286466158926486968994140625f,0.2954999993443490069466861314140260219573974609375f,-0.305699999409914102699303839472122490406036376953125f,-0.391100006768107189003558232798241078853607177734375f,-0.283000031530857398820444359444081783294677734375f,-0.18089999835789172255573475922574289143085479736328125f,0.2393000065505503926033270545303821563720703125f,0.641799976450204923139608581550419330596923828125f,-0.5742000150859354334897943772375583648681640625f,0.7664999935626983784686672152020037174224853515625f,-0.98400000041723245658431551419198513031005859375f,-0.441000008225441053610893504810519516468048095703125f,-1.038499974548816595643074833787977695465087890625f,-0.457299999189377359698482905514538288116455078125f,0.693299998474120826585931354202330112457275390625f,-0.5718999910116195906084612943232059478759765625f,0.41709999718666068702077609486877918243408203125f,-0.0578999992370601290758713730610907077789306640625f,0.4382000204019249611775421726633794605731964111328125f,1.1940999947130677139028875899384729564189910888671875f,0.10530000423341989712611166396527551114559173583984375f,-0.416400005078315871287486515939235687255859375f,2.15819999032020604090575943700969219207763671875f,0.0150000202655800052298218361102044582366943359375f,0.49779997230768202598483185283839702606201171875f,-2.030799974632262916429681354202330112457275390625f,-0.03559998395442942609179226565174758434295654296875f,-0.18639999375343307974617346189916133880615234375f,0.06190002319812748510230449028313159942626953125f,-0.355700021910667629754243534989655017852783203125f,-0.76629998925030218259735192987136542797088623046875f,0.17920002596378292736289949971251189708709716796875f,0.369200003165006673810921711265109479427337646484375f,-0.7455000043362378647771038231439888477325439453125f,0.97590002018809318684589015902020037174224853515625f,0.3541000003755090386192705409484915435314178466796875f,-0.9550999779045579174407976097427308559417724609375f,-0.4512999581813819105491347727365791797637939453125f,0.15609998627901067180800964706577360630035400390625f,-0.06419999824762356865903711877763271331787109375f,0.696899979686737225392789696343243122100830078125f,0.71470003961324746200034496723674237728118896484375f,0.0233999981343746554784956970252096652984619140625f,-0.78449999029934414096487671486102044582366943359375f,0.3563000116020440533048940778826363384723663330078125f,-0.85170000718832028496763086877763271331787109375f,0.470399994468689186533083557151257991790771484375f,0.09420000033378617132484578178264200687408447265625f,-0.3431999930024147715812432579696178436279296875f,0.01650000231713055764970476957387290894985198974609375f,0.572500005513429499615085660479962825775146484375f,0.09949999544024468178804454510100185871124267578125f,-0.16209999718666079360218645888380706310272216796875f,0.01600000330805784187759854830801486968994140625f,-0.8226999909758567763873315925593487918376922607421875f,-0.298999995313584843170673366330447606742382049560546875f,-0.1375000160932540183011951739899814128875732421875f,0.19539997658729557628021211712621152400970458984375f,0.68949997144937535953346241512917913496494293212890625f,0.497499983906745768535984097979962825775146484375f,0.31399999773502340616460060118697583675384521484375f,0.3592999868810178298872415325604379177093505859375f,-0.46949999558925614717708185708033852279186248779296875f,0.58469999392256133408096729908720590174198150634765625f,-0.54589999686777568310702690723701380193233489990234375f,0.276999993205070182966665015555918216705322265625f,1.23380000348091112272186364862136542797088623046875f,-0.066199988546967514224661499611102044582366943359375f,-0.3110000144839286662090671597979962825775146484375f,-1.188999975681304999142184897209517657756805419921875f,-0.010599995875358647623443175689317286014556884765625f,0.604500008106231945248509873636066913604736328125f,-0.051299988281726882632938213646411895751953125f,0.2012999883189798833171835212851874530315399169921875f,0.082499988675117652547896796022541821002960205078125f,0.370500002995133304040109578636474907398223876953125f,0.45109998151063945837080382261774502694606781005859375f,0.082500011995434885392342039267532527446746826171875f,0.78480000298917296230882811869378201663494110107421875f,0.3035000004768371173469176937942393124103546142578125f,0.50549998414516483791203427244909107685089111328125f,7.599592066753757535479962825775146484375e-09f,0.7416999629318719033932438833289779722690582275390625f,-0.8940000206828120354884958942420780658721923828125f,0.625499986082315384550156522891484200954437255859375f,-0.223599991196393954595578179578296840190887451171875f,-0.480500008881092011137070585391484200954437255859375f,-0.30069999948441983317337644621147774159908294677734375f,0.0406999967649579463824238700908608734607696533203125f,0.5049999973177907719446011469699442386627197265625f,0.448699995326995892952481881366111338138580322265625f,0.52239999739676701295820748782716691493988037109375f,0.2415000063776968630691044381819665431976318359375f,0.72270001474320844092602555974735878407955169677734375f,-1.02970001636743546669094939716160297393798828125f,0.871000029683113297096497262828052043914794921875f,0.39960001089572916299630378489382565021514892578125f,1.256000001966953050214215181767940521240234375f,-0.38810006233453719914905377663671970367431640625f,0.1321000228166582246558391489088535308837890625f,-0.575900004467368109573044421267695724964141845703125f,0.66320001662075522830264162621460855007171630859375f,0.40100001224875458927243698781239800155162811279296875f,0.848800010186433695480445749126374721527099609375f,-1.3948000388264656379533334984444081783294677734375f,-0.154599994802474771660172336851246654987335205078125f,0.04040000977963220663013999001123011112213134765625f,0.373900004556774856467882273136638104915618896484375f,-0.57810003931224407391908926001633517444133758544921875f,-0.671999992460012407491376507095992565155029296875f,0.8037000066518782404045850853435695171356201171875f,-0.000700006413459686172018336947076022624969482421875f,-0.525499981537461025027369032613933086395263671875f,-0.7641999597281223799427607445977628231048583984375f,0.22419999757707120835448222351260483264923095703125f,0.1294000032305717695635394193232059478759765625f,-0.26290000251531608910227078013122081756591796875f,-0.021300005567074009604766615666449069976806640625f,0.8546000117897989412085735239088535308837890625f,-0.987799985122681079019457683898508548736572265625f,0.03399999684095345031664692214690148830413818359375f,0.098499999284744177430184208787977695465087890625f,0.335299981844425332155879004858434200286865234375f,0.326700012552738172644239966757595539093017578125f,-0.08170000130236143576212271000258624553680419921875f,-0.65989998618364342775066688773222267627716064453125f,-0.220500008136033986971824560896493494510650634765625f,-0.1593999820709228742998675443232059478759765625f,0.7797999717712400524760596454143524169921875f,0.597400000786780793760044616647064685821533203125f,0.6111999833315611141415502061136066913604736328125f,-1.25199998053908334583184114308096468448638916015625f,0.1684000228703024237830732090515084564685821533203125f,-0.16970000206232072059719939716160297393798828125f,0.5357000031352043833976495079696178436279296875f,-0.146100002372264725636341609060764312744140625f,-1.09309999349117248357288190163671970367431640625f,0.0800000036135315450991356556187383830547332763671875f,0.2340000022053718709003078402020037174224853515625f,0.800100012326240506155272669275291264057159423828125f,0.519400000660121374806976746185682713985443115234375f,-1.10200000661611596797229140065610408782958984375f,-0.4073000270545481527051379089243710041046142578125f,-0.8799999763816597209142855717800557613372802734375f,-0.103000001914799188540428076521493494510650634765625f,0.601300004076957606002906686626374721527099609375f,0.222799999651312863324648105844971723854541778564453125f,-0.41850001880526566111484498833306133747100830078125f,-0.9600000169873237343409755339962430298328399658203125f,-0.35010000141113994942543286015279591083526611328125f,0.7265000279471280908438757251133210957050323486328125f,-0.005299998943507577831013577451813034713268280029296875f,0.029699995431303971571423971909098327159881591796875f,0.49449999871850014443452892010100185871124267578125f,0.5584999994337558337775817562942393124103546142578125f,0.354499990113079643805349405738525092601776123046875f,0.0994999792724851062075686058960855007171630859375f,-0.2272999898016454523030915879644453525543212890625f,0.46649996897578283050478376026148907840251922607421875f,-0.06649999624490732230697176419198513031005859375f,-0.172300003361701925541638047434389591217041015625f,-0.045300001029670244889757668715901672840118408203125f,-0.6724999979138370775899602449499070644378662109375f,-0.680799995642900501735539364744909107685089111328125f,-0.4637000038579104366220917654572986066341400146484375f,-0.5034999983906747189621455618180334568023681640625f,0.26979998756647116664453278644941747188568115234375f,-0.19849998997151863111554348506615497171878814697265625f,0.353499994814395979148002879810519516468048095703125f,-0.409599990704655692042024384136311709880828857421875f,0.21889999054968367619977698268485255539417266845703125f,-0.6699999868869781494140625f,0.6483999681830407979532537865452468395233154296875f,-0.143199996057152834083581183222122490406036376953125f,-0.667000013545155479022241706843487918376922607421875f,-0.72799998258054265942718075166339986026287078857421875f,-0.2693999933958053372151653093169443309307098388671875f,0.314500001847743959615399944595992565155029296875f,0.26840000483989712165566743351519107818603515625f,0.0639000007569789918449032484204508364200592041015625f,-0.49450002062320663753780536353588104248046875f,-0.0107999969840051335268071852624416351318359375f,0.677200009959936011227910057641565799713134765625f,-1.8712000427126884716244603623636066913604736328125f,-0.101399994349479438682237741886638104915618896484375f,-0.4021999908864497541571836336515843868255615234375f,-0.7176999951481821682364170555956661701202392578125f,-0.17070002042055154589661469799466431140899658203125f,-0.538800007653236434634891338646411895751953125f,-1.208699994730949356380733661353588104248046875f,-0.070900010502338517426323960535228252410888671875f,-0.6370000127255917021784625831060111522674560546875f,-0.0298999846935270596759437466971576213836669921875f,0.428900015509128706980845890939235687255859375f,-1.228099993528425581956753376289270818233489990234375f,-0.461199968206882804366841810406185686588287353515625f,0.4299999985843896421755516712437383830547332763671875f,1.043499973654746781903668306767940521240234375f,0.45859997167587351896145264618098735809326171875f,-0.6785999958157535871805521310307085514068603515625f,-0.90759999973475924406329795601777732372283935546875f,-0.45079996941685696043577991076745092868804931640625f,0.267600001299381329289417408290319144725799560546875f,0.12069996707439401006922707892954349517822265625f,-1.03930000789165433161542750895023345947265625f,-0.6983000103950498527183299302123486995697021484375f,0.213800012123584792789188213646411895751953125f,-1.403900005674362461149939917959272861480712890625f,0.09550000113248824362699451739899814128875732421875f,-0.87650000607967371024642488919198513031005859375f,-0.89919996933937085259458399377763271331787109375f,0.951200024235248520199093036353588104248046875f,-0.3678000071763989353712531737983226776123046875f,-1.72949992227554361079455702565610408782958984375f,0.1369000088930132363884695223532617092132568359375f,0.04410000261068347526816069148480892181396484375f,-0.144000007867813195616690791212022304534912109375f,-0.65880004967451188946370166377164423465728759765625f,0.01989998111724844420677982270717620849609375f,1.50080000622272535792944836430251598358154296875f,0.0720000221133236806281274766661226749420166015625f,-0.115499994128942518045732867904007434844970703125f,0.8414999890923500203143703402020037174224853515625f,-0.248500009417533807010158852790482342243194580078125f,-1.238400014674663651703667710535228252410888671875f,-0.402499968297779897596910814172588288784027099609375f,-0.35850002565979988133904043934307992458343505859375f,-1.1616000207901000518262435434735380113124847412109375f,-0.154900006411969659581018277094699442386627197265625f,0.52599999839067468343500877381302416324615478515625f,-0.36070000343322750069319226895458996295928955078125f,0.4523999821603299498207206852384842932224273681640625f,0.49899999356269830741439363919198513031005859375f,-0.5610999995410441698595604975707828998565673828125f,0.61459998764991752295827609486877918243408203125f,-0.09050000768899923997423684340901672840118408203125f,0.254300004762411102632313486537896096706390380859375f,0.379499999202787818841642319966922514140605926513671875f,0.3484999843835832056271328838192857801914215087890625f,0.1411000026330351264647333664470352232456207275390625f,0.88469997300505653914370896018226630985736846923828125f,0.114500014066695854353383765555918216705322265625f,-0.19069999046921726204573133145458996295928955078125f,-0.696499966368079537204494044999592006206512451171875f,0.323999985665083034547251372714526951313018798828125f,0.028800008472800175951533674378879368305206298828125f,0.68319998003840465283786897998652420938014984130859375f,0.83250000357627840230634319595992565155029296875f,-0.81299998563528053097115844138897955417633056640625f,0.2371000110447403219637863003299571573734283447265625f,-0.2760000180602073527325046597979962825775146484375f,-0.755700008052587524076670888462103903293609619140625f,0.0623999857366086541077265792409889400005340576171875f,-0.593500018954276953309090458787977695465087890625f,0.0163000012457370502261255751363933086395263671875f,0.5914999869316817093789495629607699811458587646484375f,0.102499972581863829645953956060111522674560546875f,-0.80089995954036741210302352556027472019195556640625f,0.53409997262209696344825715641491115093231201171875f,0.8325000384449960932897738530300557613372802734375f,-0.308700006204843280244176639826036989688873291015625f,0.13229999978542306138251660740934312343597412109375f,-0.117499995827675007831203402020037174224853515625f,0.81830001412034025776165435672737658023834228515625f,-0.285299978789687269653541079605929553508758544921875f,0.40700000587105744642713034409098327159881591796875f,0.3855999961733818093279069216805510222911834716796875f,-0.005999999396502975201883600675500929355621337890625f,-0.0125000232458116755651644780300557613372802734375f,0.703600007140636307667591609060764312744140625f,-0.099699977289140040426218547509051859378814697265625f,-1.030099986994267080575582440360449254512786865234375f,-0.2103000197678805971435167521121911704540252685546875f,0.95779999555349348838717560283839702606201171875f,0.024399995183944866994352196343243122100830078125f,-1.07080001218318887623581758816726505756378173828125f,-0.81069999449253060674891457892954349517822265625f,-0.47329995794296308986304211430251598358154296875f,0.741899997270106581481741159223020076751708984375f,0.1295999841295183419020986548275686800479888916015625f,-0.8984999648630618285238824682892300188541412353515625f,-0.27329999411553107524497363556292839348316192626953125f,1.137500014007091664325344027020037174224853515625f,-0.488299999666214290527932462282478809356689453125f,-0.2646999990820884107733945711515843868255615234375f,-0.63169998234063395869952728389762341976165771484375f,1.204399972310662203511810730560682713985443115234375f,0.02019999641478087415435993534629233181476593017578125f,1.36380001778602633066839189268648624420166015625f,0.55290003201961468448644154705107212066650390625f,-0.735300010454654540126284700818359851837158203125f,-0.37900001859664911307845613919198513031005859375f,1.9459000364303591368297929875552654266357421875f,0.1331000089883804804458122816868126392364501953125f,0.54230000425577173928104457445442676544189453125f,0.466600005888938795806097914464771747589111328125f,-0.63600002372264885508457155083306133747100830078125f,-0.3199999965727329254150390625f,0.6041000032067298519677933654747903347015380859375f,0.691900026774406651242088628350757062435150146484375f,0.1680999887228011857587262056767940521240234375f,-0.0066999937772749262876459397375583648681640625f,0.653900028324127191581283113919198513031005859375f,0.83409999799132350517538725398480892181396484375f,-0.7917999992489814786722490680404007434844970703125f,-0.362999999046325516616207096376456320285797119140625f,0.05530000687837599571139435283839702606201171875f,0.235399987912177977023020503111183643341064453125f,0.24520000379085526986955301254056394100189208984375f,-0.629299993903190024724381146370433270931243896484375f,1.2706999808207155577832736526033841073513031005859375f,0.136700012291968431288324836714309640228748321533203125f,0.04849999839067464790787198580801486968994140625f,0.63129998187422764477361170065705664455890655517578125f,0.3554999883174898567261834614328108727931976318359375f,-0.5994000003620982841567865762044675648212432861328125f,0.292899972563982313733532691912841983139514923095703125f,0.6949999985098838095609607989899814128875732421875f,-0.537799979162216335026869273860938847064971923828125f,-0.47659999352097504043968001496978104114532470703125f,-0.369999994039535096135296043939888477325439453125f,0.158800001245737121280399151146411895751953125f,0.5153999835535885143400491870124824345111846923828125f,0.26749998658895501879584344351314939558506011962890625f,-0.59569997037127631056563359379651956260204315185546875f,0.1162999953597785296466327054076828062534332275390625f,-0.4930000050067899763917012023739516735076904296875f,-0.5719999945461751167385955341160297393798828125f,-0.69199999979883408673941858069156296551227569580078125f,0.4349999935552475438527153528411872684955596923828125f,0.7706999800756573204552069000783376395702362060546875f,0.252399992069602063793354318477213382720947265625f,-0.048999994754790776596564683131873607635498046875f,-0.22249997004866628458330524154007434844970703125f,-0.9346000006884331856582548425649292767047882080078125f,0.09550002050399797326463158242404460906982421875f,1.15370002393722526079500312334857881069183349609375f,-0.0824999834224582340169718008837662637233734130859375f,0.4300000199675562129186801030300557613372802734375f,-0.073899989581108105340945257921703159809112548828125f,-0.340599996061623055965128514799289405345916748046875f,-0.67400001189112668242842119070701301097869873046875f,0.6184999939203261742903805497917346656322479248046875f,-0.01599999891221515202488490103860385715961456298828125f,-0.98250000178813934326171875f,0.1854000001311302270323722041212022304534912109375f,-0.260699986445903864051842901972122490406036376953125f,-0.13750000581145283007344914949499070644378662109375f,-0.7072999896526337426649888584506697952747344970703125f,-0.39769999693632118731301261504995636641979217529296875f,-0.3639999974370002888690578402020037174224853515625f,0.197499988079071098212580182007513940334320068359375f,-0.1325999973878264626137024606578052043914794921875f,0.27999998627230493486450768614304251968860626220703125f,0.844999989587813526981108225299976766109466552734375f,0.21459999798759821221239008082193322479724884033203125f,-0.354500023305416078756024944595992565155029296875f,0.656400031788647364550115526071749627590179443359375f,0.6686999915085733992015093463123776018619537353515625f,-0.3080999727785587172235182151780463755130767822265625f,-0.36649999188631732227605652951751835644245147705078125f,0.973600010269880300484146573580801486968994140625f,0.26149997791647905387435457669198513031005859375f,-0.0982999938547610696559786447323858737945556640625f,0.159899985289573720592670724727213382720947265625f,-1.12649997627735132255111238919198513031005859375f,-0.02269999961853041980930356658063828945159912109375f,1.315600024448335236826324035064317286014556884765625f,-0.221800032627582854871661766082979738712310791015625f,-0.6850000219792129474427611057762987911701202392578125f,-0.3208999917268755552868242375552654266357421875f,-0.96640005455017075064461096189916133880615234375f,-0.5494000220060346606487655662931501865386962890625f,1.60070006005764042811279068700969219207763671875f,0.50229997653961078185602673329412937164306640625f,-1.166900059854985016727368929423391819000244140625f,0.85810002068579205314335922594182193279266357421875f,0.42949998277425738280044242856092751026153564453125f,-0.200300004047155422171044847345910966396331787109375f,0.206299997855722949680057354271411895751953125f,1.05970000262111430089362329454161226749420166015625f,-0.473100025640428523132641203119419515132904052734375f,-1.504400002075731546113956937915645539760589599609375f,-0.097299931538105344230871196486987173557281494140625f,0.6295000147372487031560694958898238837718963623046875f,-0.258700019075721598937889211811125278472900390625f,-0.634699996642023034354451738181523978710174560546875f,0.2433000067628918738904530982836149632930755615234375f,1.691000011801719438153668306767940521240234375f,-0.9623000808477399914409033954143524169921875f,-0.221699993479251844519239966757595539093017578125f,-0.2031000168859957710765229421667754650115966796875f,-1.7114999566078186177264797152020037174224853515625f,-0.15969999967813475194589045713655650615692138671875f,-0.39509998442381633054765188717283308506011962890625f,0.228899964770674646530324025661684572696685791015625f,0.74390002318322678842577033719862811267375946044921875f,-0.26530000874102110941521459608338773250579833984375f,-1.54219997553825383107550806016661226749420166015625f,-0.344400011128187077957818473805673420429229736328125f,1.052799968433379973475894075818359851837158203125f,0.52320001282095862649157425039447844028472900390625f,-0.01300000858306871176495178588083945214748382568359375f,-0.41899997167661806596328233354142867028713226318359375f,-0.3583999971657989558337931157439015805721282958984375f,-0.5939999978840351246844875277020037174224853515625f,-0.08490000749230386389854174922220408916473388671875f,-0.73519998866617675137291598730371333658695220947265625f,-0.082499999701976634014499722979962825775146484375f,-0.637799984526634222703478371840901672840118408203125f,0.07970000005066391057795271990471519529819488525390625f,-0.0749999916553496515092547269887290894985198974609375f,-0.6078000115349888599070027339621447026729583740234375f,-0.77819999821782104287848369494895450770854949951171875f,0.963999986410140508041877183131873607635498046875f,-0.63730002660751328136257143341936171054840087890625f,0.51579998696297391092002726509235799312591552734375f,0.342000027179718557590604177676141262054443359375f,-0.44220001413226128761380095966160297393798828125f,-1.0074000146076078632262351675308309495449066162109375f,-0.375499996364116572777902547386474907398223876953125f,-0.007600008079409636962964214035309851169586181640625f,-0.65769999701082693110976151729119010269641876220703125f,0.4620000332519413888121562195010483264923095703125f,0.36240000540614136781414345023222267627716064453125f,-0.2305999887973071427182958359480835497379302978515625f,0.2894999753236773276654503206373192369937896728515625f,0.4563000244542954053628136534825898706912994384765625f,0.9822999709144237812097344431094825267791748046875f,-0.123999997604638423354117549024522304534912109375f,-0.2041999988436697943683384437463246285915374755859375f,0.24540000637099124158879703827551566064357757568359375f,-0.95700000333786050532580702565610408782958984375f,-0.9177000358283518011148771620355546474456787109375f,-1.0249999796599151835607699467800557613372802734375f,0.36949998366832748075694325962103903293609619140625f,-0.92419997157454503167173243127763271331787109375f,0.4736999857902528532349606393836438655853271484375f,-0.6845000195428732059355070305173285305500030517578125f,0.451399968235194737342652615552651695907115936279296875f,0.0916999961242079830725515421363525092601776123046875f,0.656999986648559630708632539608515799045562744140625f,-0.884699982765317027855189735419116914272308349609375f,0.5295999830678106423675899350200779736042022705078125f,-0.314000005781650770586566068232059478759765625f,0.5326000373601917914356818073429167270660400390625f,0.288700015294551803890499286353588104248046875f,-0.819700033950805817539730924181640148162841796875f,-0.71790003678798708364183767116628587245941162109375f,0.82250000536441802978515625f,0.77349998497962957344498136080801486968994140625f,-0.9095000176429746119310948415659368038177490234375f,-0.98360003038644805428702966310083866119384765625f,0.21650001651048711437397287227213382720947265625f,0.0463000022888184759040086646564304828643798828125f,-0.0118999970838427060471076401881873607635498046875f,0.6944000037148594373093146714381873607635498046875f,-0.4272000210240485262858101123129017651081085205078125f,0.163200004625320360673867980949580669403076171875f,0.036299980235099837955203838646411895751953125f,0.20910001363754293635111025650985538959503173828125f,-1.10239999899864216104106162674725055694580078125f,1.33369997208118462594939046539366245269775390625f,0.0724999904632568359375f,-0.11769999961853017111934605054557323455810546875f,-0.679399974024295971730680321343243122100830078125f,-0.23400000160932510340217049815692007541656494140625f,0.04000000379979606890401555574499070644378662109375f,-0.56500001341104510999002741300500929355621337890625f,-0.085000008195638798724758089520037174224853515625f,0.946199997338652565304073505103588104248046875f,1.1024999765306713328527621342800557613372802734375f,-0.44610001287758382915171750937588512897491455078125f,-0.1802000365734102160786278545856475830078125f,1.20830000920295788091607391834259033203125f,0.6123000415086750081172795034945011138916015625f,-0.27649998968839639701400301419198513031005859375f,0.397099997782706992666135192848742008209228515625f,0.76690000591278018049479214823804795742034912109375f,0.46280000568628310020358185283839702606201171875f,1.178799985897541091617313213646411895751953125f,-0.06019999485015858908809605054557323455810546875f,0.3290999658979474329356662565260194242000579833984375f,-1.0751999863564962733875063349842093884944915771484375f,-0.46259999403506479875858303785207681357860565185546875f,0.07330000100731837164858006872236728668212890625f,-0.26080002335906016242006444372236728668212890625f,0.50540002710223230764086110866628587245941162109375f,1.02269998590946187277950230054557323455810546875f,-0.5555999805644156186446025458280928432941436767578125f,0.7325000074505805258695545489899814128875732421875f,-0.735800016951560831302003862219862639904022216796875f,0.0810999900043010146788446945720352232456207275390625f,0.579500009894371004293134319595992565155029296875f,-0.05839999172687537765114029753021895885467529296875f,-0.539499991714954330035425300593487918376922607421875f,0.66700000482797605627638404257595539093017578125f,0.32330001292824750436238900874741375446319580078125f,0.6722000053405758901448052711202763020992279052734375f,0.565500017039477942404346322291530668735504150390625f,0.22650000481307508426453978245262987911701202392578125f,-0.018699999387562282748831421486102044582366943359375f,-0.39550001007318513757127220742404460906982421875f,0.058299988639354705100004139239899814128875732421875f,-0.16989999687522649995941037559532560408115386962890625f,0.714500008106231376814321265555918216705322265625f,0.9492999792814256210249368450604379177093505859375f,0.5632999997034666694872839798335917294025421142578125f,0.3979999835491181414681705064140260219573974609375f,-0.417499982118606727254928046022541821002960205078125f,0.251399984924495445426373407826758921146392822265625f,0.084999999552965110893865130492486059665679931640625f,-0.3370000090748070942225922408397309482097625732421875f,0.0849999962002038478292575973682687617838382720947265625f,-0.121999967500567407796552288345992565155029296875f,0.2494999966323376838772674091160297393798828125f,-0.784399973949790219052147222100757062435150146484375f,-0.676499995350837934893206693232059478759765625f,0.88530001254081724937350372783839702606201171875f,-0.204099968002736886774073354899883270263671875f,-0.8034999882578848229286450077779591083526611328125f,0.525399980759620799375397837138734757900238037109375f,-0.028499998390674630144303591805510222911834716796875f,-0.270499991595745115091631305404007434844970703125f,0.5155999963223933946210308931767940521240234375f,-0.5260999869868159262153994859545491635799407958984375f,0.36899999505281433354042519567883573472499847412109375f,-0.79289997358843689045926339531433768570423126220703125f,0.0408000113636253214366433894610963761806488037109375f,-0.7160000191032889205189349013380706310272216796875f,0.018499984681606296277323053800500929355621337890625f,-0.11659999586790803505209623835980892181396484375f,-0.88800000977516191369431908242404460906982421875f,0.02949999693036087222708374611102044582366943359375f,0.6038000051945449087043016334064304828643798828125f,-0.01669999601244942510902546928264200687408447265625f,-0.9427000417888169447877544371294789016246795654296875f,-0.47119998277276753384512630873359739780426025390625f,0.398299984094500292286511466954834759235382080078125f,0.45729999404847576638388773062615655362606048583984375f,0.47050001044571398978177967364899814128875732421875f,-0.69190002081394208488518415833823382854461669921875f,-0.318100014501810246514423852204345166683197021484375f,-0.08959998914003364234304171986877918243408203125f,1.849699998188018668088261620141565799713134765625f,-0.28490002671480230134193334379233419895172119140625f,0.9443999635934829939287737943232059478759765625f,-0.80609996929168659107745043002068996429443359375f,-0.0010999856829645437983344891108572483062744140625f,0.61649998581409448661361238919198513031005859375f,0.41879999513626131601995439268648624420166015625f,-0.949800021839141805912731797434389591217041015625f,-0.631900034224987194875211571343243122100830078125f,-1.618999987602233403549689683131873607635498046875f,0.2052000073671340629744008765555918216705322265625f,0.501199991452693893734249286353588104248046875f,-1.930400010561942991671458003111183643341064453125f,0.04779999495744657878049110877327620983123779296875f,0.4846999774008988737250547274015843868255615234375f,-1.38469998641610114731292924261651933193206787109375f,0.0251000264823442176265189118566922843456268310546875f,-0.483300001007318513757127220742404460906982421875f,-0.4343999917566776503008441068232059478759765625f,0.38380001733899149485296220518648624420166015625f,-0.36740003684759159341410850174725055694580078125f,-1.4199999949336046256576082669198513031005859375f,0.477200018155574667844120995141565799713134765625f,0.4039999887757002028365604928694665431976318359375f,0.270699979908019638941141238319687545299530029296875f,-0.0486999956436455772035287736798636615276336669921875f,-0.77040001026391990990305203013122081756591796875f,-0.0422999971032140820170752704143524169921875f,-0.3295000209212304298489470966160297393798828125f,-0.37140002210289235762274984153918921947479248046875f,0.196800040748715598937224058317951858043670654296875f,0.69940003370344661082214088310138322412967681884765625f,0.3797000399112704371873405762016773223876953125f,1.176999976813792425645033290493302047252655029296875f,0.20620000367164603716219062334857881069183349609375f,-0.5401999745845795786181042785756289958953857421875f,-1.3690000209808346198769868351519107818603515625f,-0.224199983197450603000788760255090892314910888671875f,-0.5955999854445457319940260276780463755130767822265625f,-1.1072000300765036939765195711515843868255615234375f,-0.5515999903172252860628077542060054838657379150390625f,-0.0020000043064356542998893928597681224346160888671875f,0.510800010171532647973435814492404460906982421875f,0.1136999856412412857054050618899054825305938720703125f,0.39019999116212134016024037919123657047748565673828125f,0.58540000713467588155936027760617434978485107421875f,-0.1341000064849851014514570124447345733642578125f,0.37279999886900172523240826194523833692073822021484375f,-0.39199997174739831251599753159098327159881591796875f,-0.213799989622831532187774428166449069976806640625f,0.1914000007569789385541980664129368960857391357421875f,-0.0797999892055987203320910339243710041046142578125f,-0.286000002264976682653241368825547397136688232421875f,0.54700001205503900880700030029402114450931549072265625f,0.86540004841089324116865100222639739513397216796875f,0.744799971175194031047794851474463939666748046875f,-0.4736999649658801825324871970224194228649139404296875f,-1.0833000124812119935313603491522371768951416015625f,0.4074999994039529838119051419198513031005859375f,0.279499998420476902794007401098497211933135986328125f,0.071500017404556359679190791212022304534912109375f,0.33899998853355650663132792033138684928417205810546875f,0.1028000027433038088275907284696586430072784423828125f,-0.21440000602453945788283817819319665431976318359375f,0.067300000716745689288700305041857063770294189453125f,-0.74509996479153672854778278633602894842624664306640625f,-0.32230000000893987976979815357481129467487335205078125f,0.3960999971568579525182940415106713771820068359375f,-0.646399971364438830079279796336777508258819580078125f,0.244400000846386689090650179423391819000244140625f,0.33979998205304173808372070197947323322296142578125f,-0.184899986071884736560377859859727323055267333984375f,-0.442100007021426932851682067848742008209228515625f,0.014299998857826012876870436230092309415340423583984375f,0.016599999257922180839752712699919356964528560638427734375f,0.044599999682605251383193945002858527004718780517578125f,-0.26930000431537592930908431299030780792236328125f,0.367500010505318608355906917495303787291049957275390625f,0.384600024642050453138608645531348884105682373046875f,0.124599997596442413350104061464662663638591766357421875f,-0.421900018634647100412848885753192007541656494140625f,0.5695999948024752512765189749188721179962158203125f,0.3781999769091601848458594758994877338409423828125f,-0.4139000110387802688904912429279647767543792724609375f,-0.617600015008449521047850794275291264057159423828125f,-0.17379999364614473478241052362136542797088623046875f,0.5872000045582652649756028040428645908832550048828125f,-0.9828000036001203199020892498083412647247314453125f,-0.661200006651878169350311509333550930023193359375f,0.4265999966502189710837456004810519516468048095703125f,0.1570000012144447953943426909972913563251495361328125f,0.19220000467002396060678393041598610579967498779296875f,0.04299999822676181526759364714962430298328399658203125f,0.77579999400377286367103124575805850327014923095703125f,0.6781999962061642239774528206908144056797027587890625f,-0.259700002062320745022105938915046863257884979248046875f,1.01290003321170818395557944313623011112213134765625f,0.538200004923343300333726801909506320953369140625f,-0.70650001949071883355912859769887290894985198974609375f,-0.7838000106334686023501490126363933086395263671875f,-0.394599998676776753114836537861265242099761962890625f,0.512800002221763140397570168715901672840118408203125f,-0.28900003737211221732650301419198513031005859375f,0.228600002074241359650841332040727138519287109375f,0.7314000094369053073251052410341799259185791015625f,0.70869998459815963798291704733856022357940673828125f,0.367799987804890093912035808898508548736572265625f,-0.46649998015165328180131609769887290894985198974609375f,-0.80560001137256609382575334166176617145538330078125f,-0.2081999876379967417960870079696178436279296875f,0.80129999707341192927145812063827179372310638427734375f,-0.455100031101704161073939758352935314178466796875f,0.1020000233054165761359399766661226749420166015625f,0.77430002707690004370988390292041003704071044921875f,0.6986999917507166202312873792834579944610595703125f,0.20319999210834538416747818700969219207763671875f,-0.6123999764233827303172574829659424722194671630859375f,-0.71040000787973411888742703013122081756591796875f,-0.3699999913573268628397272550500929355621337890625f,0.603099999041855294734659764799289405345916748046875f,0.22639999732971194390529490192420780658721923828125f,0.066500011052936347510922132642008364200592041015625f,0.1255000049136578699204846998327411711215972900390625f,-0.337799987283348979172359349831822328269481658935546875f,-0.880200028117001043170830598683096468448638916015625f,-0.2000999631524091881829008343629539012908935546875f,-0.5503999923452733700202088584774173796176910400390625f,-1.4152000267386437570849011535756289958953857421875f,-0.290299987953901439396986461360938847064971923828125f,-0.255699993970990302472046096227131783962249755859375f,0.1771000059783460756079875864088535308837890625f,0.27740000317096669135707998066209256649017333984375f,-0.8987999692827466358124866019352339208126068115234375f,-0.4091999997377389064467934076674282550811767578125f,-0.211699990759790235284043546926113776862621307373046875f,0.23330000596195443751668108234298415482044219970703125f,0.1885000201091173721579252742230892181396484375f,-0.55889996972531108543336131333489902317523956298828125f,0.0492000021964311695654714640113525092601776123046875f,-0.72360000099390742178684376995079219341278076171875f,-0.03429999820590001746722919051535427570343017578125f,0.2261000005096198695042630788520909845829010009765625f,-0.6335000234246255246262080618180334568023681640625f,-0.0346999947607518066661214106716215610504150390625f,-0.564699988055229251671107704169116914272308349609375f,-0.115100005471706179349666854250244796276092529296875f,-0.7170999655961995955522070289589464664459228515625f,0.59229996856749078659731821971945464611053466796875f,1.12670002119541123875023913569748401641845703125f,0.0048999934852123061546080862171947956085205078125f,0.4090999993324277994588555884547531604766845703125f,-0.05449996310472471350294654257595539093017578125f,-0.4108999804019930479626054875552654266357421875f,-0.23719999725669627110846704454161226749420166015625f,0.376399998521804945994517765939235687255859375f,-0.64460000538229866862138806027360260486602783203125f,0.35759997835159307300045838928781449794769287109375f,0.93240001270770989094671676866710186004638671875f,0.3289999648034580559396999888122081756591796875f,0.58529997621923701700552555848844349384307861328125f,0.25530001045465411380064324475824832916259765625f,0.03959999480247500258656145888380706310272216796875f,0.501299966265261520703688802313990890979766845703125f,1.461299996626376440644889953546226024627685546875f,-0.232699986207485221001434183563105762004852294921875f,0.6168000126972794650015430306666530668735504150390625f,-0.15170001165866864312192774377763271331787109375f,-0.20240000525712975587566688773222267627716064453125f,0.0821000095173716193386326267500407993793487548828125f,-0.47410002585649468898054692544974386692047119140625f,0.1092000077843664485044428147375583648681640625f,0.80190000259727234332984835418756119906902313232421875f,-0.6553999899983404819892029991024173796176910400390625f,-0.35899998804926891438071834272705018520355224609375f,0.45349998855590822888217417130363173782825469970703125f,-0.118899998372793191947494051419198513031005859375f,-0.06319999479055393720727806794457137584686279296875f,0.10490000134557486699549144759657792747020721435546875f,-0.09020002524852799297150340862572193145751953125f,0.2412000173807147263005390414036810398101806640625f,0.468600022935867333018222780083306133747100830078125f,-0.35170001463890088189145899377763271331787109375f,-0.095600001835823178453210857696831226348876953125f,0.306800003123283371309071299037896096706390380859375f,1.01200002844631686826915029087103903293609619140625f,0.48780000233352172056555673407274298369884490966796875f,-0.726000012248754433841213540290482342243194580078125f,-0.8258999893426892668912842054851353168487548828125f,-0.5499999910593036389627741300500929355621337890625f,0.5271999859690665868328096621553413569927215576171875f,0.89200000900030129713513815659098327159881591796875f,0.60220000414848318115446090814657509326934814453125f,-0.45280000229626893304413215446402318775653839111328125f,0.425099978053570026759189204312860965728759765625f,0.38399997657537454642806551419198513031005859375f,-0.212099997447431132258088837261311709880828857421875f,0.47239997903108577048669758369214832782745361328125f,-0.0590000388622282656569950631819665431976318359375f,-0.72579998241811993775485234436928294599056243896484375f,0.47680001176595698808569068205542862415313720703125f,0.09619999115467070538443294935859739780426025390625f,-0.49700000367313623694798252472537569701671600341796875f,0.9292000095725061470375294447876513004302978515625f,0.6429999924898144314511228003539144992828369140625f,-0.5422000130519271099416300785378552973270416259765625f,0.4477999961495395808697139727883040904998779296875f,-0.1028000235676760354408543207682669162750244140625f,-0.7890999820843338508069564340985380113124847412109375f,0.57660000886917117668417631648480892181396484375f,-0.022599997127056070667094900272786617279052734375f,0.281199984747171338739235579851083457469940185546875f,0.7225999707520009973649166568065993487834930419921875f,1.2838000222563739072256794315762817859649658203125f,-0.67979996566176481564980349503457546234130859375f,0.31899998916685579075647183344699442386627197265625f,0.8882000245928765025382745079696178436279296875f,0.0460999991685151488951532883220352232456207275390625f,-0.089799999673664532284789174809702672064304351806640625f,0.498999990209937127616512952954508364200592041015625f,0.0107999950468540273362805237411521375179290771484375f,0.119800002318620590902042977177188731729984283447265625f,0.300200011092424556835567273083142936229705810546875f,-0.54540000489950113404802323202602565288543701171875f,0.508999974042177338873216285719536244869232177734375f,0.821800010275840264739599660970270633697509765625f,-0.18159998994469628286196893895976245403289794921875f,-0.3545000088140366489852794984471984207630157470703125f,-0.2221000243067745572034255019389092922210693359375f,0.075699995349347570794407147332094609737396240234375f,0.466700004953145952679705033006030134856700897216796875f,-0.25249999351799468083612509872182272374629974365234375f,0.271599994713067616203261422924697399139404296875f,0.3248999897226692024077010501059703528881072998046875f,1.3221000076174735937684090458787977695465087890625f,-0.2766999952673909746181379887275397777557373046875f,0.36349999280273925794659817256615497171878814697265625f,-0.32690000561475773821484835934825241565704345703125f,-0.3563999611198909178710891865193843841552734375f,-0.53039998485744010991993491188623011112213134765625f,-1.25879999930858588186310953460633754730224609375f,0.5838999889850615687691970379091799259185791015625f,-0.2045000064671038675356840030872263014316558837890625f,-0.7188999976277354875264791189692914485931396484375f,0.2990999924778936502889337134547531604766845703125f,0.19190000531673423456169302880880422890186309814453125f,-0.13849998825788478740150821977294981479644775390625f,-0.34629997218847297091315340367145836353302001953125f,-0.20829999236464491474407623172737658023834228515625f,-1.24910002317428592277792631648480892181396484375f,0.63159996982812938171036876155994832515716552734375f,-0.514699987272918324521242539049126207828521728515625f,-1.168300007116794603234666283242404460906982421875f,-0.380400002813339188634245147113688290119171142578125f,-0.36239999624192709148928770446218550205230712890625f,0.8595999906301496906735337688587605953216552734375f,0.26319997959136998133544693700969219207763671875f,-0.7587999811291694651771422286401502788066864013671875f,0.2412000168219206042152791269472800195217132568359375f,0.01900001718103905279377840997767634689807891845703125f,-0.1716999948576092283047955788788385689258575439453125f,-0.5030000252723692000245137023739516735076904296875f,0.0170000042319298216852985206060111522674560546875f,0.6997000058621167806194307559053413569927215576171875f,0.157600013667344995837993337772786617279052734375f,-0.055799994897842264407472612219862639904022216796875f,-0.21569999955892560361547793945646844804286956787109375f,-0.61770001332759871814914731658063828945159912109375f,-0.19799998712539679246447121840901672840118408203125f,0.64260000569522379709752613052842207252979278564453125f,-0.061199996891617924887896151631139218807220458984375f,0.1242000150859359219879252123064361512660980224609375f,0.1590000031366944899247073408332653343677520751953125f,-0.5449999999999999289457264239899814128875732421875f,-0.21319999330043781782251244294457137584686279296875f,0.3847999959856271967595375826931558549404144287109375f,0.02730001394152647975488434894941747188568115234375f,0.028400015270710010639731990522705018520355224609375f,0.0843999956309795518194505348219536244869232177734375f,-0.219399979574978143403996000415645539760589599609375f,0.021000030316412454656216368675814010202884674072265625f,0.42989998944327234564610762390657328069210052490234375f,1.2321000336945058251103546353988349437713623046875f,0.6814000070154662136445722353528253734111785888671875f,-0.75610001943409443914134726583142764866352081298828125f,-0.1119000178337099526970632723532617092132568359375f,-0.4068000347137452621382180950604379177093505859375f,-0.3905999904364347852236960534355603158473968505859375f,0.43669999765157729854081480880267918109893798828125f,0.3637999867916104079768047085963189601898193359375f,-0.22880001074522737791738791202078573405742645263671875f,0.604600003370642458122574680601246654987335205078125f,0.6454000256121161793743112866650335490703582763671875f,0.0137000080674887225740121721173636615276336669921875f,0.1800000108033419365938243572600185871124267578125f,0.06499999813735468645603532422683201730251312255859375f,-0.199700009028613567618748447785037569701671600341796875f,0.783499994218349371521981083787977695465087890625f,0.093800018382072636313750990666449069976806640625f,0.37239998111724847973391661071218550205230712890625f,0.5851999945521353385657903345418162643909454345703125f,0.1036000408172608189261154620908200740814208984375f,0.244699963580072565871859069375204853713512420654296875f,0.85019998367428784025179311356623657047748565673828125f,1.0974000181093810368793128873221576213836669921875f,-0.53749997198581755952773164608515799045562744140625f,0.075200000624358498413357665413059294223785400390625f,0.553600026810169509872139315120875835418701171875f,0.2868999783456327445918532248470000922679901123046875f,0.378799999718367974566035627503879368305206298828125f,0.456400015360117095042369328439235687255859375f,-0.21720000072121603551522639463655650615692138671875f,0.29369997874945419358283515975927002727985382080078125f,1.02930000133514365501241627498529851436614990234375f,-0.32690000017583320524039436349994502961635589599609375f,-0.4160999832242728846409818288520909845829010009765625f,-0.610500012233853084353540907613933086395263671875f,-0.15779999108314513023287872783839702606201171875f,-0.66399998022615924764266992497141472995281219482421875f,-0.50960001447200742319409982883371412754058837890625f,-0.0016000103592872516600209564785473048686981201171875f,0.5900999963447450813447403561440296471118927001953125f,0.8817000213444234901771778822876513004302978515625f,-0.2279000048808754730345071948249824345111846923828125f,0.76060000071823574696594505439861677587032318115234375f,1.3622000319018965353023986608604900538921356201171875f,-0.07229999911487094976791922817938029766082763671875f,0.761699986699223785535650677047669887542724609375f,-0.0243000098288062016393951125792227685451507568359375f,0.668399988299608338593316148035228252410888671875f,-0.5361000141069289792739027689094655215740203857421875f,-0.6573000298857696321874755085445940494537353515625f,-0.29809996562600193925618441426195204257965087890625f,-0.387199988837540143293836081284098327159881591796875f,-1.044400013661384463148351642303168773651123046875f,-0.64130000072419590662065047581563703715801239013671875f,0.5989999967291950166980996073107235133647918701171875f,0.280600017258525014085535076446831226348876953125f,0.042500003688037413240863315877504646778106689453125f,-0.223199994902312681688272277824580669403076171875f,-0.51360001854002479415584048183518461883068084716796875f,-0.40239999259114256346947513520717620849609375f,-0.1850999987661838819263948607840575277805328369140625f,0.0490000238493084605551075583207421004772186279296875f,-0.08439997991025405266185543950996361672878265380859375f,-0.079399999319016867360687683685682713985443115234375f,-0.6750999875903129776588684762828052043914794921875f,-0.27179998524189041830823043710552155971527099609375f,0.4937999814644455920387144942651502788066864013671875f,-1.0767000164270399409360834397375583648681640625f,-0.7170000113844867684065320645458996295928955078125f,0.495300006803870207505724465590901672840118408203125f,0.176599995979666690715248478227294981479644775390625f,0.1653999993860721939853419826249592006206512451171875f,-0.04749999754130840301513671875f,0.191299993050098038338546757586300373077392578125f,-0.2580000172257417290211378713138401508331298828125f,-0.552899977071583226262418975238688290119171142578125f,1.29320002548694645838622818700969219207763671875f,0.9160000112056732035625827847979962825775146484375f,-0.596800013405084595063954111537896096706390380859375f,-0.911999998271465273091962444595992565155029296875f,-0.237399978137016631762890028767287731170654296875f,0.864999986253678798675537109375f,0.48910000978186729536645316329668276011943817138671875f,0.478000022627413534426210617311880923807621002197265625f,0.0104000023849308842471828029374592006206512451171875f,-0.8658999956011772525243941345252096652984619140625f,-0.646199998605251124672577134333550930023193359375f,0.316499998480081590201962171704508364200592041015625f,-1.0518999859452247136459845933131873607635498046875f,-0.848799999010562800094703561626374721527099609375f,0.3102999911203980776264188534696586430072784423828125f,-0.48029998482465752207559717135154642164707183837890625f,-0.0041999653905631095796024965238757431507110595703125f,0.630899989193677956933470341027714312076568603515625f,-0.6280000103712080061768574523739516735076904296875f,-0.6130000208020209839787639793939888477325439453125f,0.0997000025093555120037080996553413569927215576171875f,0.753299998176097940216777715249918401241302490234375f,0.4669999923110008666071735206060111522674560546875f,-0.436999997265636930254828484976314939558506011962890625f,0.76080002678632763224442214777809567749500274658203125f,0.1369999835938211507624373552971519529819488525390625f,-0.83990001777410505656717987221782095730304718017578125f,0.5398000012755392873486925964243710041046142578125f,0.351700005400180426562428692704997956752777099609375f,0.241399968235194994914394328588969074189662933349609375f,0.53900001192837920793721195877878926694393157958984375f,0.8038000346615916669179568998515605926513671875f,-0.1722999906957145554997623548842966556549072265625f,-0.65060000470429624197521434325608424842357635498046875f,-0.36460006245374732003483586595393717288970947265625f,0.29530000106692266825803017127327620983123779296875f,-0.854099969679117609899776653037406504154205322265625f,-0.248099983954428893184740445576608180999755859375f,-0.14929999440610408623086868828977458178997039794921875f,0.0058000002995133304040109578636474907398223876953125f,-0.17499999888241291046142578125f,0.701799959760904812355875037610530853271484375f,-0.0487999954342843711430077746626920998096466064453125f,-0.1971999814987182020331601961515843868255615234375f,-0.48279998154640235696888339589349925518035888671875f,0.60160001795887918518701553693972527980804443359375f,-0.36639997169971394441745360381901264190673828125f,-0.47409998115301110743757817544974386692047119140625f,-0.5361000028192994815157135235494934022426605224609375f,0.0134999557733532782322072307579219341278076171875f,-0.41640000241473307251993674071854911744594573974609375f,0.8111000017575917819812048037420026957988739013671875f,0.7387000387080018271035442012362182140350341796875f,0.374299985092878539916227964567951858043670654296875f,-0.70909998264312756877103538499795831739902496337890625f,-0.22150001531839347279628782416693866252899169921875f,0.5803999809086324290774427936412394046783447265625f,-0.87619997729659093721465978887863457202911376953125f,-0.28060002329349487126819440163671970367431640625f,-0.47709999856501761339444556142552755773067474365234375f,-0.02250000286847342323426346410997211933135986328125f,0.38669997407048928295125733711756765842437744140625f,0.56749999493360459013047147891484200954437255859375f,-0.294799991105497039001193115836940705776214599609375f,0.203600017869472793563545565120875835418701171875f,-0.153699990558624488556915821391157805919647216796875f,0.25560000254362813532083009704365395009517669677734375f,-0.34799998399615272859364267787896096706390380859375f,0.50129999621659482311741840021568350493907928466796875f,0.1182000045508146968131768517196178436279296875f,-0.50820000712126489617048719082958996295928955078125f,-0.2489000068664548592778373858891427516937255859375f,-0.0097999959856271967595375826931558549404144287109375f,-0.93820000015497218015525504597462713718414306640625f,-0.544999990463256978046047152020037174224853515625f,0.58699999607354402808567783722537569701671600341796875f,-0.4136000059485436253225998370908200740814208984375f,-0.175999991834163704851334841805510222911834716796875f,0.387200002807378762526013815659098327159881591796875f,-0.3419000145554544900505788973532617092132568359375f,-0.393200023400783749139009159989655017852783203125f,-0.1074999959394336013218662628787569701671600341796875f,0.0344000006973743666094378568232059478759765625f,-0.0538000072062014833562670901301316916942596435546875f,-0.15410000007748603234603024247917346656322479248046875f,-0.70779996694326374750971808680333197116851806640625f,-0.430999972462654312721497262828052043914794921875f,0.49039997781664135256818326524808071553707122802734375f,-1.1561000354528427891409592120908200740814208984375f,-0.8080000211000442078557171043939888477325439453125f,0.5764000154346227144941394726629368960857391357421875f,0.546499993115663329490416799671947956085205078125f,0.595400013542175532421651951153762638568878173828125f,-0.01789999029636379912489019261556677520275115966796875f,-0.84739997426271418401455548519152216613292694091796875f,-0.5515999818235639562402639057836495339870452880859375f,0.498799977701902375049058946387958712875843048095703125f,-0.46919997649192790589722790173254907131195068359375f,-0.44999998897314075207987116300500929355621337890625f,0.15749998796731234929069387362687848508358001708984375f,0.69600001418590551338638761080801486968994140625f,0.38139999405145630362312658689916133880615234375f,-0.5521000099271535788147957646287977695465087890625f,-0.074299984943866803632772644050419330596923828125f,-0.42200001227855654661880180356092751026153564453125f,-0.4617999999195336879864726142841391265392303466796875f,0.034599999123811731038813377381302416324615478515625f,0.139000008463859625607028647209517657756805419921875f,0.138800001208484180548197173266089521348476409912109375f,0.31149999818205831303430386469699442386627197265625f,0.559800026905536984855871196486987173557281494140625f,0.3006000064179301656253073815605603158473968505859375f,-0.057799993839859897803989952080883085727691650390625f,-0.0897999957621096189797071929206140339374542236328125f,0.01120000013262034688210633248672820627689361572265625f,0.61339998874664303230019868351519107818603515625f,-1.0364000482916839729341518250294029712677001953125f,0.1192999920964243898424683720804750919342041015625f,-0.30890000828206520822050151764415204524993896484375f,0.4414999741911884711953462101519107818603515625f,-0.05760000383257857237140342476777732372283935546875f,0.235999986767768954454282948063337244093418121337890625f,0.757600003683566836087948104250244796276092529296875f,0.39779999659657505617360584437847137451171875f,-0.790600010553001997237743125879205763339996337890625f,-0.851000048160553745901779620908200740814208984375f,0.126699999812245334140925479005090892314910888671875f,-0.31699998966604481243081181673915125429630279541015625f,-0.5098000050008295591652540679206140339374542236328125f,0.15069998763799663521467664395458996295928955078125f,0.2323000064909457318407248749281279742717742919921875f,1.0547000014662746281146610272116959095001220703125f,0.41939999622702561765663631376810371875762939453125f,-0.3343000118777155194038641639053821563720703125f,0.63970000861883136167307384312152862548828125f,-0.6168999916076656209185102852643467485904693603515625f,0.2496999972939490941570284121553413569927215576171875f,-0.1986999948799610393734837998636066913604736328125f,0.23619999163895843974358967898297123610973358154296875f,-0.042400001122057329183689944329671561717987060546875f,0.77439998046904801043410770944319665431976318359375f,0.430799979996681070559816362219862639904022216796875f,0.2721000063136218560799761689850129187107086181640625f,0.9105000160336498282731554354541003704071044921875f,-0.0708999902367590806306907325051724910736083984375f,-0.374999988414347118492742083617486059665679931640625f,-1.255800027978420274621385033242404460906982421875f,0.735199971455335887782212012098170816898345947265625f,-0.562899978487193752840767047018744051456451416015625f,-0.728900009250640579239188809879124164581298828125f,-0.2318999960780143698713118283194489777088165283203125f,0.18649999997019761632799372819135896861553192138671875f,0.23290000966787349767628256813623011112213134765625f,-0.7692999894142149575060329880216158926486968994140625f,0.555800004322826612934704826329834759235382080078125f,0.48580003170371099940894055180251598358154296875f,0.13049999077618112952592355213710106909275054931640625f,0.23690000509321673671792041204753331840038299560546875f,0.43210002065598995812933935667388141155242919921875f,-0.43129998440742500775968437665142118930816650390625f,-0.317599997499585173788005931783118285238742828369140625f,0.1694000002503397439568288973532617092132568359375f,-0.2188000240445138189215867896564304828643798828125f,-0.4235000009611249449648084919317625463008880615234375f,-1.3355999613046645180247651296667754650115966796875f,-0.991199975061416438393280259333550930023193359375f,0.5375999805867672787229594177915714681148529052734375f,0.7936999941349027754000644563348032534122467041015625f,0.665000004917383424896115684532560408115386962890625f,-0.28729999062120914476992084019002504646778106689453125f,-0.1244999839663503138353917165659368038177490234375f,-0.188000000238418341069746020366437733173370361328125f,-0.1147000077620148328350069277803413569927215576171875f,-0.0228000135838983819525083163171075284481048583984375f,0.1761999978601931360344678978435695171356201171875f,0.2341999999985098501742442067552474327385425567626953125f,-0.80130001480579426953454458271153271198272705078125f,-0.5717999897122378882841076119802892208099365234375f,0.47610002263784412246394595058518461883068084716796875f,0.1791000025734306166924625358660705387592315673828125f,0.19060000801980503393195931494119577109813690185546875f,0.03710000035315752509035291950567625463008880615234375f,0.30569998339116555285954746068455278873443603515625f,0.08699997510015944346406513432157225906848907470703125f,-0.381299994130432640115913045519846491515636444091796875f,-0.9240999832391736390491132624447345733642578125f,-0.401799976003169856397789772017858922481536865234375f,0.6933999861761928062975357534014619886875152587890625f,-0.753200007903576107537446659989655017852783203125f,-0.39679998911619218660007391008548438549041748046875f,0.613700001026689978544936820981092751026153564453125f,0.0217999888181683587617953890003263950347900390625f,0.414800025415420492436169297434389591217041015625f,0.5068000067740678904470996712916530668735504150390625f,0.34800002288818365769884621840901672840118408203125f,-0.095400009667873320040598628111183643341064453125f,-0.62310000824332234525826379467616789042949676513671875f,-0.281199994954466792052016899106092751026153564453125f,-0.3386000066936014629703777245595119893550872802734375f,-0.074200000967085344427687232382595539093017578125f,1.0510000275969506589035518118180334568023681640625f,0.60600001150369653402094627381302416324615478515625f,-0.6147000077620148328350069277803413569927215576171875f,0.51220001346170906941779321641661226749420166015625f,-0.638799988784640948580317854066379368305206298828125f,0.61919999228715827399582849466241896152496337890625f,0.32409999832659952545554915559478104114532470703125f,0.39050002974271791345017845742404460906982421875f,0.57959996738433883223251541494391858577728271484375f,-0.8063999661862852885718666584580205380916595458984375f,0.1007999758243567356430503423325717449188232421875f,-0.5112999907404180977010810238425619900226593017578125f,0.123100001388788182765665624174289405345916748046875f,-0.6228999982684850778014151728712022304534912109375f,0.225999996602535002665490537765435874462127685546875f,-0.454199965912104042242702917064889334142208099365234375f,-0.558599990153311853902096117963083088397979736328125f,-0.37049997587502048190799541771411895751953125f,0.033899998894333815968593626166693866252899169921875f,0.0411000091522932819998459308408200740814208984375f,0.34149995943903999062740695080719888210296630859375f,0.36629997747838505262052422040142118930816650390625f,1.368300014269351549955899827182292938232421875f,-0.178499997906386986823434881443972699344158172607421875f,0.40170000074356781993856202461756765842437744140625f,0.42930002245754028677993119345046579837799072265625f,0.113700008328259105638835535501129925251007080078125f,-0.483900011187791523337153876127558760344982147216796875f,-1.029900034686923593341134619549848139286041259765625f,-0.16129999603033073896085625165142118930816650390625f,-0.5509999684020880916790474657318554818630218505859375f,0.030199997830391733799615394673310220241546630859375f,0.04459998973608048089545263792388141155242919921875f,0.39409996353238863076029474541428498923778533935546875f,1.33809998484849845823418945656158030033111572265625f,-0.7789999896883958996340879821218550205230712890625f,0.4515999751180412857820556382648646831512451171875f,0.6043999904155725744203664362430572509765625f,0.52389998362064371661972472793422639369964599609375f,-0.3491999804779888361139228436513803899288177490234375f,-0.8268000108718869256563266390003263950347900390625f,0.36689998944699764304999689557007513940334320068359375f,-0.441499984398484368597337379469536244869232177734375f,-0.3751000039070842007049577659927308559417724609375f,-0.11000000596046444201192571199499070644378662109375f,0.66530000915080289569658589243772439658641815185546875f,-0.00109997614622070472023551701568067073822021484375f,0.235600003884732789316558410064317286014556884765625f,0.300199985015392301068715141809661872684955596923828125f,0.05689999758675667873575321209500543773174285888671875f,0.214800012935698225735592359342263080179691314697265625f,0.1233000055335462530337053976836614310741424560546875f,0.2164000006079671578618217608891427516937255859375f,0.24360000684261340353486957610584795475006103515625f,-0.0779999964386224320378460106439888477325439453125f,-0.243000009179115128432613346376456320285797119140625f,-0.0012000006914139049740697373636066913604736328125f,0.288899998484551889799121227042633108794689178466796875f,0.4182999680757522042995333322323858737945556640625f,0.26999997317791013529131305404007434844970703125f,-0.3047999810099601969426430514431558549404144287109375f,-0.8051999829292295629556974745355546474456787109375f,-0.5417999771952626275606235140003263950347900390625f,0.5378999961823225373080958888749592006206512451171875f,-0.77400001151859765258222978445701301097869873046875f,-0.45500000588595879147391087826690636575222015380859375f,0.410899999363720402900668204893008805811405181884765625f,0.72240000033974638427025638520717620849609375f,0.26919998405426748266933145714574493467807769775390625f,-0.64449999521672729496657439085538499057292938232421875f,-1.295299990189075600710566504858434200286865234375f,-1.0262000042676930888774222694337368011474609375f,0.429199984650313837164503638632595539093017578125f,0.0436999860882760771119137643836438655853271484375f,0.14599997961521093969849971472285687923431396484375f,-0.02300001059472567277452981215901672840118408203125f,0.4151999933600425407576040015555918216705322265625f,0.4031999953866005625968682579696178436279296875f,-0.1856999987021088482919140005833469331264495849609375f,0.509399994885921358900304767303168773651123046875f,0.4886000050544740958002876141108572483062744140625f,-0.141199993427097769682632133481092751026153564453125f,0.5280000050067901185002483543939888477325439453125f,0.2401999999165536081591199035756289958953857421875f,-0.3508999957501888200539497120189480483531951904296875f,0.17900001561641687430892488919198513031005859375f,0.2100000137090685115026644780300557613372802734375f,-0.084600005531310973339031988871283829212188720703125f,-0.43819998465776421880946145392954349517822265625f,-0.345999993622303492202263441868126392364501953125f,0.25569998342841859795271375332958996295928955078125f,0.0152000209271907493757680640555918216705322265625f,0.637599988037347742420024587772786617279052734375f,-0.38139999913647704143926375763840042054653167724609375f,-0.288199997901171400815911738391150720417499542236328125f,-0.5326000222913922499401451204903423786163330078125f,-0.284199998247622875879869752679951488971710205078125f,-0.431799991947412475923329111537896096706390380859375f,-0.899000025451183404356925166212022304534912109375f,-0.25279998109936752115345370839349925518035888671875f,-0.219899991808831796191725516109727323055267333984375f,-0.12629999528527235952424234710633754730224609375f,-0.1291999826014045282818187843076884746551513671875f,-0.351699991802871370083494184655137360095977783203125f,-0.956899985349178194837804767303168773651123046875f,0.28509997507333739719115328625775873661041259765625f,0.5178000104174014239788448321633040904998779296875f,1.6002000452160842769444570876657962799072265625f,0.088500005431473294237321169930510222911834716796875f,-0.7025999996975063055515420273877680301666259765625f,-0.32580002999007728892166824152809567749500274658203125f,0.199199995807558261073921812567277811467647552490234375f,0.086100001906603551571350863014231435954570770263671875f,-0.5474999933503568172454833984375f,0.68489998245835348456012070528231561183929443359375f,-0.9082000128582115028308408000157214701175689697265625f,-0.0854000722527512579063113662414252758026123046875f,-0.284799959403276847780261959997005760669708251953125f,-0.8621000002786514126995598417124710977077484130859375f,-1.3945000498294835011847681016661226749420166015625f,-0.4069999788999556500357357435859739780426025390625f,-0.586499985925853284385311781079508364200592041015625f,-1.340100022161006876331157400272786617279052734375f,-0.4586999893665311134327566833235323429107666015625f,-0.0974999928474427068891827730112709105014801025390625f,-0.6831000024318694130442963796667754650115966796875f,0.081299989473819778140750713646411895751953125f,0.5363999949082731699689929882879368960857391357421875f,0.3228000217795372606133241788484156131744384765625f,-0.26099999839067411500082016573287546634674072265625f,-0.554599976176023634621969904401339590549468994140625f,-1.049800023627280864957356243394315242767333984375f,0.2627999955534932041700812987983226776123046875f,0.419299984087050159331511167692951858043670654296875f,1.092099998080730216543088317848742008209228515625f,0.5833999656498434216445048150490038096904754638671875f,-0.3540000020936131530646662213257513940334320068359375f,0.9694999914169308841138672505621798336505889892578125f,0.673599996560812375179239097633399069309234619140625f,-0.53619998504519461590689388685859739780426025390625f,-0.272700004386901895259143202565610408782958984375f,-0.074199995863437617771296572755090892314910888671875f,0.2416000007852912023764702098560519516468048095703125f,0.54920000450611095033082165173254907131195068359375f,0.3996000043392182732304718228988349437713623046875f,-0.31139999747872348923038998691481538116931915283203125f,1.0257000275731087413078057579696178436279296875f,0.487599998915194898785330224200151860713958740234375f,-0.7324000179231167084736853212234564125537872314453125f,0.0413999812364576058598686358891427516937255859375f,0.293399999773502173638917156495153903961181640625f,0.2007000039547681691232128287083469331264495849609375f,-0.01460000195503230457916288287378847599029541015625f,-0.2792000188112258030059820157475769519805908203125f,-0.31119999622106553349709656686172820627689361572265625f,-0.93220002739429475013821502216160297393798828125f,-0.36699999946355799096409100457094609737396240234375f,0.7621000108212232504456551396287977695465087890625f,0.55749999761581392476728069595992565155029296875f,0.368999994456768121153800166212022304534912109375f,-0.41609999529421326514011525432579219341278076171875f,0.155799996387958827881448087282478809356689453125f,0.18919998394250825413109851069748401641845703125f,-0.107300014500320006760603064321912825107574462890625f,0.4274000261187556048980695777572691440582275390625f,0.11540000638961789292125104111619293689727783203125f,-0.3434000112846493646401313526439480483531951904296875f,1.31780003966093062217623810283839702606201171875f,1.0456000384926795732098980806767940521240234375f,-0.43450001410394900602796042221598327159881591796875f,-0.1130999687552449728400461026467382907867431640625f,0.22540004572868355126047390513122081756591796875f,0.5161999842628837154023813127423636615276336669921875f,1.02069997780322996305812921491451561450958251953125f,0.7645999933123590608374797739088535308837890625f,-0.43189997961223125511054377057007513940334320068359375f,0.4916000023126603224454811424948275089263916015625f,0.17579999161958692610596699523739516735076904296875f,-0.3746999986350536548940226566628552973270416259765625f,-0.47120002810955075034371475339867174625396728515625f,0.409000015914440240294425166212022304534912109375f,-0.3585999967843289670810236202669329941272735595703125f,-0.04129999405592688077604179852642118930816650390625f,-1.0320999991983175192444832646287977695465087890625f,-0.1394999955892561871451107435859739780426025390625f,-0.249599997298419473690245240504737012088298797607421875f,-0.133200009393692031522959950962103903293609619140625f,-0.247399999296664674375278991647064685821533203125f,-0.2095999950259923583217869236250407993793487548828125f,-0.8752000201821328317919324035756289958953857421875f,0.09359999909400951167981475009582936763763427734375f,0.5468999811396006993646778937545605003833770751953125f,-0.140700023102760862769855521037243306636810302734375f,-0.5624999761581417345723821199499070644378662109375f,-0.50040001112073628775078759645111858844757080078125f,-0.86040004304647510480208438821136951446533203125f,0.53359998776912664908422812004573643207550048828125f,-0.10419999459683904063922454952262341976165771484375f,-0.098599988067149979542591609060764312744140625f,-0.23269998739957831190849901759065687656402587890625f,-0.42679999481588615228844219018355943262577056884765625f,0.22280000807046906885489079286344349384307861328125f,-0.19649999579787191805735346861183643341064453125f,-0.603099995428323598645192760159261524677276611328125f,-1.484300040674209952840101323090493679046630859375f,-0.2523999617457395316932888817973434925079345703125f,-0.754899988456070314413182131829671561717987060546875f,-1.334900034463405660289936349727213382720947265625f,0.23149998909234970057013924815692007541656494140625f,0.63250000737607459111444541122182272374629974365234375f,0.58130003089904835889001333271153271198272705078125f,-0.699800000083445894460965064354240894317626953125f,0.92649999933689830555749722407199442386627197265625f,0.09050000292062776452439720742404460906982421875f,-0.52650000086426729239974520169198513031005859375f,0.74160001017302246140161514631472527980804443359375f,1.20360002859830927945949952118098735809326171875f,-0.465899991875886598791112191975116729736328125f,0.42289998306930076665821616188623011112213134765625f,-0.376699990499020032075350172817707061767578125f,0.172299996209144268277668743394315242767333984375f,0.53919999515563243885907240837696008384227752685546875f,0.37040002754926693029347006813623011112213134765625f,-0.654199991095065769286520662717521190643310546875f,-0.361199991676211362712223262860788963735103607177734375f,0.737400001382827685603160716709680855274200439453125f,0.29179998881816882061457363306544721126556396484375f,-0.6142999926924705977882013030466623604297637939453125f,0.13590000916123390339862453402020037174224853515625f,0.1390000147223471316237919381819665431976318359375f,0.0910999967098236052009951890795491635799407958984375f,-0.4020000098943707911303135915659368038177490234375f,-0.00499999463558165047061265795491635799407958984375f,0.450800001268088834649461205117404460906982421875f,0.4667999795794486317390692420303821563720703125f,0.2905999815702440258746719337068498134613037109375f,-0.32059998667389155713181025930680334568023681640625f,-0.38669999793097364904070900593069382011890411376953125f,-0.50480001762956394628645284683443605899810791015625f,-0.11530000399127604548965564390528015792369842529296875f,0.33059997560977905095569440163671970367431640625f,-0.1184000417947768113435813575051724910736083984375f,-0.69099998266994955287145785405300557613372802734375f,-0.3460000070333482113937861868180334568023681640625f,0.07480002303123445273058678139932453632354736328125f,0.6602000002145766988093100735568441450595855712890625f,0.6551000102400781344158531283028423786163330078125f,0.4075999971270558575042741722427308559417724609375f,-0.3797000096991658413259074222878552973270416259765625f,-0.061400010964274276403784824651665985584259033203125f,0.0461999898135661002385177198448218405246734619140625f,0.0662999982655048114565943251363933086395263671875f,-0.01899999311566347159896395169198513031005859375f,-0.0710000077784063154950899843242950737476348876953125f,-0.15940000945180654579047541119507513940334320068359375f,0.35239999050498038712930792826227843761444091796875f,0.431399993455409802578515154891647398471832275390625f,0.000599994347989518672648046049289405345916748046875f,-1.0042000274538995796547169447876513004302978515625f,-0.71120001589059800295444802031852304935455322265625f,0.4187000163003802288841370682348497211933135986328125f,-0.5825000040233134779299462024937383830547332763671875f,-0.2135999892592430615678722460870631039142608642578125f,0.53369999804645773533451347248046658933162689208984375f,0.4178999908924101447382781771011650562286376953125f,0.29379999245405219454596590367145836353302001953125f,-0.2778999913394450960169024256174452602863311767578125f,0.2604999993443488648381389793939888477325439453125f,-0.48510000226795690903003333005472086369991302490234375f,0.177099990779161498011262665386311709880828857421875f,0.2502000015184282499802748134243302047252655029296875f,-0.3113999843657015986764236004091799259185791015625f,-0.1031000067532061859054692831705324351787567138671875f,0.7658999829351902377538863220252096652984619140625f,1.038300015312433099978761674719862639904022216796875f,-0.419499977111816857444637207663618028163909912109375f,-0.5216999683707956858569332325714640319347381591796875f,-0.4520999813914290399452511337585747241973876953125f,-0.1132999886393548649721196852624416351318359375f,-0.1820999959945679425032949438900686800479888916015625f,-0.7412999915599822742251490126363933086395263671875f,0.59159999126717421891186177163035608828067779541015625f,-0.401399992319196485812682340110768564045429229736328125f,-0.1914000052087008185708327800966799259185791015625f,0.5319999935030939042235331726260483264923095703125f,-0.7824000021278856298323489681934006512165069580078125f,-1.4760000278949743091061463928781449794769287109375f,-0.461799995672702578985990840010344982147216796875f,0.76839996453225645201001725581591017544269561767578125f,1.47560000064372953687552580959163606166839599609375f,-0.13699998739361785027313089813105762004852294921875f,0.21339999850690372085182389128021895885467529296875f,-0.31660000350475314689902006648480892181396484375f,-0.17409997854530845362575064427801407873630523681640625f,-0.4491999923244118786413991983863525092601776123046875f,-0.647600010165572115283794119022786617279052734375f,0.0895000077337027022394977393560111522674560546875f,-0.6958999883741139313997337012551724910736083984375f,-1.0447000352919102450499622136703692376613616943359375f,-0.5044999626576907036934471761924214661121368408203125f,-0.1144999920129776693755729866097681224346160888671875f,-0.5616999954164025865566145512275397777557373046875f,0.36099999094009405098404386080801486968994140625f,-0.76549999506026511486567187603213824331760406494140625f,-0.17590000633001334762184342253021895885467529296875f,-0.1643999951839447692947260293294675648212432861328125f,-0.0022000028818845718259211707845679484307765960693359375f,0.5446000009864568891515546056325547397136688232421875f,-0.1768999935448167803997421287931501865386962890625f,-0.35489999057948595773126498897909186780452728271484375f,-0.96570000760555263497053601895458996295928955078125f,0.25219999505877499501593774766661226749420166015625f,0.705699982124567082308885801467113196849822998046875f,0.088499993920326147645027958787977695465087890625f,0.05379998708963373843516819761134684085845947265625f,-0.16810000265538693753342158743180334568023681640625f,-1.01080000532865543760863147326745092868804931640625f,-0.7219999881386751638956411625258624553680419921875f,0.56390001480132345701434815055108629167079925537109375f,-0.2099000067844987160725622743484564125537872314453125f,-0.1554000107109547723638343086349777877330780029296875f,0.013499998837709414889474146548309363424777984619140625f,-0.36350001090765005073990323580801486968994140625f,-0.099199984538554719648573154699988663196563720703125f,0.4772000145047903263417765629128552973270416259765625f,0.8583000305861236967075456050224602222442626953125f,0.44040000497400744183806864384678192436695098876953125f,-0.59850001649558548511009803405613638460636138916015625f,0.46029999406337740452954676584340631961822509765625f,0.233999984025955232169735609204508364200592041015625f,-0.40349999760836363638105694917612709105014801025390625f,-0.0352000130295750945919053265242837369441986083984375f,0.190999996453523390727013975265435874462127685546875f,0.26289999580979339643960202010930515825748443603515625f,-0.06089999321699135492735877051018178462982177734375f,-0.0095999927163123999207527958787977695465087890625f,0.08849999626725912893476788667612709105014801025390625f,-0.6336999863862988746632254333235323429107666015625f,-0.33239998350143462602090949076227843761444091796875f,0.5327999852716922557505085933371447026729583740234375f,-0.53940000054836279019809808232821524143218994140625f,-0.39499999880790692685650355997495353221893310546875f,0.22160000249892475299162697410793043673038482666015625f,0.355799998250603710658879208494909107685089111328125f,0.3398000041067599230615314809256233274936676025390625f,-0.03500000048428775212272512362687848508358001708984375f,-0.7641999866247175532407709397375583648681640625f,-0.568399994707107225622166879475116729736328125f,0.26719999379217629353888696641661226749420166015625f,-0.6922000084698201050059651606716215610504150390625f,-0.2847999877154829828640458799782209098339080810546875f,0.65679999962151047743219578478601761162281036376953125f,0.34919999765157694326944692875258624553680419921875f,0.139799989652633627201794297434389591217041015625f,-0.32099999805539847219648663667612709105014801025390625f,-0.8176000159025189617523210472427308559417724609375f,0.29909996647536740699280244371038861572742462158203125f,-0.0512999866425989381468752981163561344146728515625f,-0.73889999859631039225149606863851658999919891357421875f,-1.22730004448890728241394754149951040744781494140625f,0.42119996403455761679879287839867174625396728515625f,0.223300001044571150288220451329834759235382080078125f,1.357700034785271014925456256605684757232666015625f,0.068500007480382929525575264051440171897411346435546875f,-0.6018999963387845486550986606744118034839630126953125f,-0.71750002671033163981206826065317727625370025634765625f,-0.1378000119447708726738710538484156131744384765625f,0.7018999960407612537238719596643932163715362548828125f,1.419100052976608683508175090537406504154205322265625f,0.007400000973045717245213381829671561717987060546875f,0.310200010496377853730365359297138638794422149658203125f,-0.35099999772012202559068327900604344904422760009765625f,-0.01619998906850828035430822637863457202911376953125f,-0.366399990177154577519758049675147049129009246826171875f,-1.010000007748603767510076067992486059665679931640625f,-0.4136999687284237392503882801975123584270477294921875f,0.0039999987408518844489435650757513940334320068359375f,0.0032000061899424281364190392196178436279296875f,-0.03680000118613246939958116854541003704071044921875f,0.1218000031232833180183661170303821563720703125f,0.24300000649690645104783470742404460906982421875f,0.03919999929070439037559481221251189708709716796875f,0.43029999275952579562698474546778015792369842529296875f,1.27790001145601284093800131813623011112213134765625f,0.223099988722801345630841751699335873126983642578125f,-0.3356999915122986788418302239733748137950897216796875f,0.0242999971628190536421243450604379177093505859375f,-0.29749998964369306353461297476314939558506011962890625f,0.3662999917089939660996833481476642191410064697265625f,0.193500007405877028077156865037977695465087890625f,0.3221999889492987989569883211515843868255615234375f,-0.720799964573979767834543963545002043247222900390625f,-0.342399998402594718527325312606990337371826171875f,-0.1164000008314847178780837566591799259185791015625f,0.3801999724239113476187412743456661701202392578125f,1.2433999954521650632699447669438086450099945068359375f,0.0709000012636185061865035095252096652984619140625f,-0.0939000018000601954781814129091799259185791015625f,-0.47550000828504579430955345742404460906982421875f,-0.34049996060133036479555812547914683818817138671875f,-0.5404000092208385286340899256174452602863311767578125f,-0.354300015938282086835897644050419330596923828125f,0.015200020778179723635048503638245165348052978515625f,0.6076000186592340757130159545340575277805328369140625f,0.4818000041663648147505227825604379177093505859375f,0.472800009560585099421814447850920259952545166015625f,-0.06270000226348637628603910343372263014316558837890625f,0.3981999983668327303121259319595992565155029296875f,0.52920001672506344903013086877763271331787109375f,0.1221000025138259115209393712575547397136688232421875f,-0.24170001061558732402545501827262341976165771484375f,-0.044000001311302217033016859204508364200592041015625f,0.2111000024095177618477237047045491635799407958984375f,0.1135999872103334862316614817245863378047943115234375f,0.33139999889433380264591733066481538116931915283203125f,0.169299999211728557924061533412896096706390380859375f,0.37680000610351527257080306299030780792236328125f,-0.0110000025629997111309421597979962825775146484375f,-0.43689999220371245058913700631819665431976318359375f,0.15479998734295374873681794269941747188568115234375f,0.3063999977767466464939616344054229557514190673828125f,0.111200000318884828498511296857031993567943572998046875f,0.9296000022530552797661584918387234210968017578125f,0.7182000153541565623527276329696178436279296875f,-0.2951999916091561093622885891818441450595855712890625f,-0.3370000084042548138540951185859739780426025390625f,0.01280001462697999414785954286344349384307861328125f,0.5263000044867396365333434005151502788066864013671875f,-0.910400014734268125948801753111183643341064453125f,-0.2549999895691872353609142010100185871124267578125f,0.8420999978572130117981942021287977695465087890625f,-0.04390001789331421377937658689916133880615234375f,0.3312000033736222093239121022634208202362060546875f,0.493499992318451319306404911912977695465087890625f,0.9558000437736513532627213862724602222442626953125f,0.58860002263784405585056447307579219341278076171875f,-0.5265000197142362114988145549432374536991119384765625f,-0.0588000088453295433055245666764676570892333984375f,-0.3006000241875650402789688087068498134613037109375f,-0.3422999955385923609441078951931558549404144287109375f,0.4813000010967254382876490126363933086395263671875f,0.156999987542629337866628702613525092601776123046875f,-0.47979999691694974472966350731439888477325439453125f,-0.304700025904178772861996549181640148162841796875f,-0.971700048911571911958162672817707061767578125f,-0.35100000792741781197037198580801486968994140625f,0.792299997811019007798449820256792008876800537109375f,1.530300063204765859126155191916041076183319091796875f,-0.4369999856054784803660595571273006498813629150390625f,0.2991000083103774187520684790797531604766845703125f,-0.044099980705976093986464547924697399139404296875f,-0.6299999859929084067289295489899814128875732421875f,0.365300000508129496523679335950873792171478271484375f,0.121300014507770725913360365666449069976806640625f,-0.1751999906778334281653997095418162643909454345703125f,-0.40480000090301027082517748567624948918819427490234375f,-0.40520002033114455031181933009065687656402587890625f,-0.5582999681130054181465993679012171924114227294921875f,-0.35539998962581165198315602538059465587139129638671875f,-0.7066000052556393740132989478297531604766845703125f,0.4252999681353575311959502869285643100738525390625f,0.5111999757692220480720379782724194228649139404296875f,0.4607999951958650086680790991522371768951416015625f,-0.45959997416436693118413359115947969257831573486328125f,-0.1895999991983174925991306736250407993793487548828125f,-0.482400007417798093456440255977213382720947265625f,-0.44510000547170580631473058019764721393585205078125f,0.8742999769717456626239027173141948878765106201171875f,0.4079000081777568453844651230610907077789306640625f,-0.34579999310970332970782692427746951580047607421875f,0.42249999996274689717523642684682272374629974365234375f,0.94490003446340598003416744177229702472686767578125f,-0.43789997807741176671925131813623011112213134765625f,-0.542199995617568486494519675034098327159881591796875f,0.005799983274936693078416283242404460906982421875f,-0.681599960440397723004934960044920444488525390625f,0.4089999668896202589252197867608629167079925537109375f,0.6109999861717216873557845246978104114532470703125f,-0.1773000042557715261182238464243710041046142578125f,0.54000001642853001637689658309682272374629974365234375f,-0.10799998980760516786858715931884944438934326171875f,0.51129998887777361460393876768648624420166015625f,-0.5418000133678313456897512878640554845333099365234375f,-0.7314000411391265998872768250294029712677001953125f,0.1692999968647956876566240680404007434844970703125f,-0.110199993397295525898726964442175813019275665283203125f,-0.619399999356269770345306824310682713985443115234375f,-0.517799979907274110502157782320864498615264892578125f,0.3488999990433454545524227796704508364200592041015625f,0.3365999984011056600508027258911170065402984619140625f,0.39580000614821908566653974048676900565624237060546875f,0.0703999973744153795252032068674452602863311767578125f,1.0023999960184095670001624966971576213836669921875f,0.45079998655319197808921671821735799312591552734375f,-0.6853999914884567612460841701249592006206512451171875f,-0.620099986398219726879688096232712268829345703125f,-0.1661999845981600998356952914036810398101806640625f,0.55979997836500405838933147606439888477325439453125f,0.29129998852387062324709177119075320661067962646484375f,0.311599999220668888444407684801262803375720977783203125f,0.0415000012181698796354112346307374536991119384765625f,-0.3436999983072279718498975853435695171356201171875f,-0.28899999892711658588950740522705018520355224609375f,0.210999996714293924782168687670491635799407958984375f,0.41989998610913747967288145446218550205230712890625f,0.13379997822344347202516701145214028656482696533203125f,-0.4838999857440591245705263645504601299762725830078125f,0.2628000080704691043820275808684527873992919921875f,-0.1432000210165984555032991920597851276397705078125f,-0.4253000131741166978116552854771725833415985107421875f,0.120700007903575823320352355949580669403076171875f,-0.212000024497509542698026052676141262054443359375f,-0.51910001248270276619223295710980892181396484375f,0.630900020337104905365777085535228252410888671875f,0.21379999662637683144339462160132825374603271484375f,-0.53810001114904881802658564993180334568023681640625f,-0.81160001572370532585409819148480892181396484375f,-0.542000011682510329791284675593487918376922607421875f,0.354500004194676865409974197973497211933135986328125f,-1.0892000024199486318821072927676141262054443359375f,-0.673399994707107651947808335535228252410888671875f,0.6211999964073300350975159744848497211933135986328125f,-0.0162000012874603527279759873636066913604736328125f,0.004600005531310902284758412861265242099761962890625f,0.1253000013649463451059773433371447026729583740234375f,-0.393699997115135147396358661353588104248046875f,-0.28419999824762331996907960274256765842437744140625f,0.165699998888373389860362294712103903293609619140625f,0.246099975512921620435236036428250372409820556640625f,0.018199972327053892673376367383752949535846710205078125f,-0.3991999813161790644500115377013571560382843017578125f,-0.44880001808404923480111392564140260219573974609375f,-0.6628000071763988643169795977883040904998779296875f,-0.422699980209768344519005722759175114333629608154296875f,0.47730000332444888488936385328997857868671417236328125f,0.371700031588972024110262282192707061767578125f,0.1605999902129173761977654066868126392364501953125f,0.92959996894896057550994328266824595630168914794921875f,0.47680001623630463569725179695524275302886962890625f,-0.31989997679591208878946417826227843761444091796875f,-0.51719998354762797276862329454161226749420166015625f,-0.22560002359151809514514752663671970367431640625f,-0.2997999872125685083545931775006465613842010498046875f,0.515999968457967472801328767673112452030181884765625f,0.4859999922253184223563948762603104114532470703125f,0.250299992126226111821551967295818030834197998046875f,0.530599985183775491037749816314317286014556884765625f,1.171000019550323401062996708787977695465087890625f,0.135100001187622442699876046390272676944732666015625f,-0.50689997745156312181080693335388787090778350830078125f,-1.173699988099932145502179992035962641239166259765625f,0.57499999701976722832341692992486059665679931640625f,0.57149997050315182178792383638210594654083251953125f,0.67709999569654399920182186178863048553466796875f,0.50519999946951887892510058009065687656402587890625f,-0.7942000079333777495804724821937270462512969970703125f,-1.2938000672578819916225256747566163539886474609375f,0.2310999938786029161974511225707828998565673828125f,0.39989999370872986172997798348660580813884735107421875f,0.02669999794960009609212647774256765842437744140625f,0.220400001770257869537772421608678996562957763671875f,-0.7576999929502605812814408636768348515033721923828125f,-0.362500016987323814277033307007513940334320068359375f,0.214699992525577698643246549181640148162841796875f,-0.454300003197789070696899216272868216037750244140625f,0.372099976921081321279416442848742008209228515625f,0.004699994164705234567236402654089033603668212890625f,0.6557999795123936603857828231411986052989959716796875f,1.025400015628337602180408794083632528781890869140625f,0.154499994248151750753095257095992565155029296875f,-0.24949998880922807931170837036916054785251617431640625f,0.01010000219345119631952911731787025928497314453125f,0.1839000008501112670700194939854554831981658935546875f,-0.6141999895304441903221004395163618028163909912109375f,-1.166600001828372423773316768347285687923431396484375f,-0.2872999884605409448568025254644453525543212890625f,0.861300013464689317999045670148916542530059814453125f,0.11389999190941446638447587247355841100215911865234375f,0.022199991221725501322481477473047561943531036376953125f,-0.055200000382214842176153979380615055561065673828125f,-0.458799996924400321773873656638897955417633056640625f,-0.4644000023365020268784064683131873607635498046875f,0.107100000949203966182921021754737012088298797607421875f,-0.1137999939441680652407740126363933086395263671875f,0.1054000171184537038016060250811278820037841796875f,0.331999992124736298393372635473497211933135986328125f,0.1858000152006746841237827538861893117427825927734375f,-0.04639999487102020037099237015354447066783905029296875f,-0.182200001801550381941297018784098327159881591796875f,0.98889998838901504996101721189916133880615234375f,0.8066000103592880776659512775950133800506591796875f,-0.3046999758362769483710508211515843868255615234375f,0.758900039350987043462737346999347209930419921875f,0.1261999971151350763420850853435695171356201171875f,-0.72560001953095198956589229055680334568023681640625f,-0.225399994245171431117569227353669703006744384765625f,-0.3374000063002109417453766582184471189975738525390625f,-0.05980000198334456396054292781627736985683441162109375f,0.62370002989768991152885746487299911677837371826171875f,0.1557999976545569875696628514560870826244354248046875f,-0.649000015728175672080624281079508364200592041015625f,0.0040999743357301721147223361185751855373382568359375f,0.24359998337328392370437768477131612598896026611328125f,0.16949999767541878981091940659098327159881591796875f,-0.227699996787309733026205549322185106575489044189453125f,0.01380001603513958396973748676828108727931976318359375f,0.3167999984294176396559805652941577136516571044921875f,-0.7787999966263772222418992896564304828643798828125f,-0.547999977290629658455145545303821563720703125f,0.5291000147923827778839722668635658919811248779296875f,0.29799999055266379599515857989899814128875732421875f,0.507400012409686684833332037669606506824493408203125f,0.27319999840408559776960828457958996295928955078125f,-0.85760002119243150531247010803781449794769287109375f,-0.49260000405609549734009533494827337563037872314453125f,0.482400015762448342027113312724395655095577239990234375f,0.621899993172287945952803056570701301097869873046875f,0.6802000119864939353675481470418162643909454345703125f,0.0479000020682811555872149256174452602863311767578125f,-0.1692000077843669458843578468076884746551513671875f,0.55589999526590094802713792887516319751739501953125f,0.0927000017605721093882920058604213409125804901123046875f,0.474399967002124001158591681814868934452533721923828125f,0.62839999152198355858445211197249591350555419921875f,0.4635999807655810656115136225707828998565673828125f,0.1368999973073601328366066809394396841526031494140625f,0.6953000129878521562432069913484156131744384765625f,-0.304699994015693587101623052149079740047454833984375f,0.41049998399615306166055006542592309415340423583984375f,0.71449999409913989012466117856092751026153564453125f,0.315199981103837234552855761648970656096935272216796875f,0.78080000029951313056386652533547021448612213134765625f,1.0504000436797742423777890508063137531280517578125f,-0.569700000721216071042363182641565799713134765625f,0.6725000146031374637090038959286175668239593505859375f,0.0289000628948219429048549500294029712677001953125f,0.55869998683333310651732972473837435245513916015625f,0.94300001562386714937247234047390520572662353515625f,0.76820005618333908614658867008984088897705078125f,0.44749998666346046949371384471305646002292633056640625f,0.1393000025272368436191072760266251862049102783203125f,0.4721000170797111650244914926588535308837890625f,0.32189999008029712701528524121385999023914337158203125f,0.03360000453293332522974878884269855916500091552734375f,-0.5220000190213323065790973487310111522674560546875f,-0.5089999559000137896447313323733396828174591064453125f,-0.66440000542849275699808231365750543773174285888671875f,-1.0720000360831622998603052110411226749420166015625f,-0.257499999869614526470940063518355600535869598388671875f,-0.01140000185593958992313901035231538116931915283203125f,0.179399990098923378667450378998182713985443115234375f,0.3341999586105348640785450697876513004302978515625f,0.739500004418193856992047585663385689258575439453125f,1.920300064098835690629130112938582897186279296875f,-0.6748999767959118489812908592284657061100006103515625f,0.6472999721437695530568134927307255566120147705078125f,0.6480999990046019121336939861066639423370361328125f,0.079499996446072973554919371963478624820709228515625f,-0.68469997509121938161769094222108833491802215576171875f,-0.21829998185932575349710305090411566197872161865234375f,-0.68729997974336154431540535370004363358020782470703125f,0.7184999826699496328075156270642764866352081298828125f,0.443700014773011019997284165583550930023193359375f,0.111999980688095579495211495668627321720123291015625f,0.5827000195860863041019683805643580853939056396484375f,0.8616000145316125014005592674948275089263916015625f,0.61420001642704047384313525981269776821136474609375f,-0.3267999973490833820477519111591391265392303466796875f,0.8768000058054923329109442420303821563720703125f,0.2649999839067458395902576739899814128875732421875f,-0.785799996574223058587449486367404460906982421875f,0.56299999696016289618683003936894237995147705078125f,0.4443999990582465642319220933131873607635498046875f,-0.28629999740868805613303038626327179372310638427734375f,0.67800001469254500552352737940964289009571075439453125f,0.284599993684887753175871694111265242099761962890625f,-0.568100007274746854335489842924289405345916748046875f,-0.4161000110149384312308029620908200740814208984375f,-0.27880000258684134450959390960633754730224609375f,0.2179000091835856522948233759962022304534912109375f,0.6450999977231024473667275742627680301666259765625f,0.3155999872326855637538756127469241619110107421875f,-0.6014999900981783387265977580682374536991119384765625f,0.39809998633861543027023799368180334568023681640625f,0.37379999483823755923594944761134684085845947265625f,-0.05129999655187134077749533389578573405742645263671875f,0.079000003322958978202450452954508364200592041015625f,0.1400000108778478580262571995262987911701202392578125f,0.09100000166893007502721957280300557613372802734375f,0.1158000285744666957299386922386474907398223876953125f,-0.264800000232458199178608992951922118663787841796875f,-0.4357000103250145972566542695858515799045562744140625f,-0.75570000909566914515380631200969219207763671875f,-0.6352000124335290820454247295856475830078125f,0.15730000272840261033024944481439888477325439453125f,-0.9603999858260152677758014760911464691162109375f,-0.60099999034404749664872724679298698902130126953125f,0.488799986903369454438461616518907248973846435546875f,-0.15429999716281894706071398104541003704071044921875f,-0.36720001487731934730618377216160297393798828125f,-0.24959999979436398032106581013067625463008880615234375f,0.44189999434575411019210378071875311434268951416015625f,0.402600009457766805365253048876184038817882537841796875f,-0.0218999942339956898962327613844536244869232177734375f,-0.816500019788742203985520973219536244869232177734375f,-0.4521999993801113504332533921115100383758544921875f,0.534700013983249622384619215154089033603668212890625f,-0.8331000080943109509235000587068498134613037109375f,0.6101000159025193880779625033028423786163330078125f,-0.30910000410079963018006310448981821537017822265625f,0.84270001176297626432898368875612504780292510986328125f,0.43710005083084180199648471898399293422698974609375f,0.3842999855399131803324053180404007434844970703125f,0.5442999867320061913034123790566809475421905517578125f,0.0251000117301938274749772972427308559417724609375f,-0.3243999910116190932285462622530758380889892578125f,-0.2567000047668812623413714391062967479228973388671875f,-0.9679000320196156081919980351813137531280517578125f,0.12389999908059834421436562479357235133647918701171875f,0.0858000027954578803335294878706918098032474517822265625f,-0.5118000066623091814932422494166530668735504150390625f,0.126199992793798454471243530861102044582366943359375f,-0.14099999637901794447003567256615497171878814697265625f,0.19620001112222684724883947637863457202911376953125f,-0.00500000014901136324851904646493494510650634765625f,-0.660399971073866165482968426658771932125091552734375f,-1.640000004768371155705608543939888477325439453125f,-0.32169998260140442170040842029266059398651123046875f,0.37709998280704037920685323115321807563304901123046875f,0.16710000582933393076245920383371412754058837890625f,-0.18600000084936618716113798654987476766109466552734375f,0.254000005856156174655069435175391845405101776123046875f,0.298000019535422655536649472196586430072784423828125f,0.6465999704986813423346347917686216533184051513671875f,0.1137000011757016704194711564923636615276336669921875f,0.12089999090731140540810883976519107818603515625f,0.45599998542666497769459965638816356658935546875f,-0.8754000130951400393541916855610907077789306640625f,-0.5766000601291665361713967286050319671630859375f,0.1880999976634978310130463796667754650115966796875f,-0.73849997313320658864910228658118285238742828369140625f,-0.19210001387596076227737285080365836620330810546875f,0.5593999843060970533770159818232059478759765625f,-0.2704999981895088723149456200189888477325439453125f,-0.21730000872612009743534144945442676544189453125f,-0.3177000017605717818724997414392419159412384033203125f,0.74429996458515557122836980852298438549041748046875f,0.6902999939702443299438527901656925678253173828125f,0.059699992823600833702357704169116914272308349609375f,-0.27609999626278891593500475210021249949932098388671875f,0.36910001199841513397359449299983680248260498046875f,0.20069998063445115832337251049466431140899658203125f,0.135799997021257834450835844108951278030872344970703125f,1.0327000319540502459858544170856475830078125f,0.5288000075042245118339678811025805771350860595703125f,-0.65810001558214425987358708880492486059665679931640625f,-0.37179999112784845038959247176535427570343017578125f,-0.22119999480545493586447491907165385782718658447265625f,0.163899995169043577458722893425147049129009246826171875f,0.151000007256865576010795848560519516468048095703125f,0.1410000122487546914129552533267997205257415771484375f,0.038400001077353973499839412397705018520355224609375f,0.5612000290036203153931637643836438655853271484375f,0.02439998713731750257238672929815948009490966796875f,-0.7807000161364674450936718130833469331264495849609375f,-0.63039997986555107445383328013122081756591796875f,-0.3755999782919889895538290147669613361358642578125f,0.476699982191622240179640357382595539093017578125f,1.05330001635551440131166600622236728668212890625f,0.4629999963641164839600605773739516735076904296875f,-0.780100002491474153742956332280300557613372802734375f,0.0254999963641164839600605773739516735076904296875f,0.39260002692937856494381776428781449794769287109375f,0.451300003108382252747787788393907248973846435546875f,-0.4692999846458434376472723670303821563720703125f,-0.01679996080398549196388557902537286281585693359375f,0.70679999366104606650651476229541003704071044921875f,-0.4022000203907492021926373126916587352752685546875f,-0.0697999921858312166733639969606883823871612548828125f,0.53300000988692042280803207177086733281612396240234375f,-0.288100021803379124918365050689317286014556884765625f,-0.023800003480911158249000436626374721527099609375f,0.324100009614229189747192094728234224021434783935546875f,-0.54259999921321888649572429130785167217254638671875f,-0.020399969434738096651926753111183643341064453125f,0.70740000019073490111765067922533489763736724853515625f,-0.171799991500377779996000526807620190083980560302734375f,-0.20999999482184639187920538461185060441493988037109375f,-0.006899999579787229053717823035185574553906917572021484375f,0.98170000540018076407022817875258624553680419921875f,0.70760000308752069031470455229282379150390625f,-0.48709999893754718502947298475191928446292877197265625f,-0.80710001156628141671944831614382565021514892578125f,-0.46040000601708930272337738642818294465541839599609375f,0.458599995107948743555681403449852950870990753173828125f,0.279700034546852549510731478221714496612548828125f,0.159699994462728422917052739649079740047454833984375f,1.266199995923042109779999009333550930023193359375f,-0.8558000181436540287904790602624416351318359375f,-0.23459999986886970191335421986877918243408203125f,1.056100006246566636036732234060764312744140625f,0.02009997358322124227925087325274944305419921875f,0.078099997663497955358025137684307992458343505859375f,-0.435400002589821699672256727353669703006744384765625f,1.1918000052839516911262762732803821563720703125f,0.41140000291764733386656871516606770455837249755859375f,-0.5447999926328659370255991234444081783294677734375f,-0.056999986350536602230931748636066913604736328125f,-0.0286999947309494185532230403623543679714202880859375f,0.48179997242689154290928854607045650482177734375f,-0.32599998706579214058365323580801486968994140625f,0.137900010561943009435026397113688290119171142578125f,0.14599999019503595576452426030300557613372802734375f,0.13679998479485533380284323357045650482177734375f,0.1787999982655048558655153101426549255847930908203125f,-0.7418000245809555082132646930404007434844970703125f,0.63260000755786904846900142729282379150390625f,0.287200007724762063077150742174126207828521728515625f,0.906099996709823329865685082040727138519287109375f,-1.5350999807357794679774087853729724884033203125f,-0.6179999881684778983981232158839702606201171875f,-0.08650001442432397880111238919198513031005859375f,-0.775500006198882374519598670303821563720703125f,-0.264500008553266496846845257095992565155029296875f,0.1839000051900743226696022247779183089733123779296875f,-0.7376999994322659404133446514606475830078125f,-0.239299997945129849785672604411956854164600372314453125f,0.1897999898016451414406446929206140339374542236328125f,0.21120000821650020128572577959857881069183349609375f,-0.14319999803155647466468280981644056737422943115234375f,-0.4183000032424928349428228102624416351318359375f,0.02030001015663174257497303187847137451171875f,-0.365000005960464335430515347979962825775146484375f,0.36580000145435320746400975622236728668212890625f,-0.6873999846935276991644059307873249053955078125f,-0.27749999150633808397969914949499070644378662109375f,-0.04459999878853559973634901325567625463008880615234375f,-0.003599998870492004954257936333306133747100830078125f,-0.007899999739974739032533079807762987911701202392578125f,-0.0477999934673309923027773038484156131744384765625f,-0.1186000094316901343205472585395909845829010009765625f,-0.50380002326220296726688729904708452522754669189453125f,0.17320000618994224605984300069394521415233612060546875f,0.81130001162067078013251375523395836353302001953125f,-0.53660001602172879842100883251987397670745849609375f,0.58719998507499671092091375612653791904449462890625f,1.2804999993443484385124975233338773250579833984375f,0.5422000144302845381361066756653599441051483154296875f,-0.0293999992072582205793196408194489777088165283203125f,-0.3297000153243541120673398836515843868255615234375f,0.22600000524520869049638349679298698902130126953125f,-0.55120000605583197028636277536861598491668701171875f,0.0377999505519870382386216078884899616241455078125f,0.099400001665949755391693543060682713985443115234375f,0.20699999730288975996472800034098327159881591796875f,-0.107099999980628357576506459736265242099761962890625f,-0.240600002506375421074125142695265822112560272216796875f,0.8615999968737362291193448982085101306438446044921875f,0.787700014035403572876248290413059294223785400390625f,-0.663000028252601847356118014431558549404144287109375f,0.513400011992454441411837251507677137851715087890625f,0.7993000403761865158003274700604379177093505859375f,0.3934000103533268344335738220252096652984619140625f,-0.2609999988377094126690280972979962825775146484375f,-0.142500024884939069380607179482467472553253173828125f,0.88460001700520507483815890736877918243408203125f,0.31139998257756218436043127439916133880615234375f,-0.99129998500347227263773675076663494110107421875f,-0.380599985146522268308899583644233644008636474609375f,0.280199988293647894721516422578133642673492431640625f,0.253299985659122484094041283242404460906982421875f,0.765000009536743075244658029987476766109466552734375f,-0.032799997043609696589783197850920259952545166015625f,-0.545500005006790189554521930404007434844970703125f,0.874400037354231329800313687883317470550537109375f,0.215200017946958865877604694105684757232666015625f,-0.8855999899148940812665387056767940521240234375f,-0.49519998918771701568175558350048959255218505859375f,-0.03219999848604171432953080511651933193206787109375f,0.013600009524822098683216609060764312744140625f,0.7200000007450579886381092364899814128875732421875f,-0.5178000034511089921807069913484156131744384765625f,-1.362399977838993692103031207807362079620361328125f,-0.174599999719858178792719627381302416324615478515625f,0.29439999646916981124178391837631352245807647705078125f,0.81680001139342761629080769125721417367458343505859375f,-0.546200023229420406067902149516157805919647216796875f,-0.144500002238899416528994379405048675835132598876953125f,1.37279999465942381675631622783839702606201171875f,-0.2574000532388680539952474646270275115966796875f,0.06169998304843904435301737976260483264923095703125f,-0.17950000676512711805798971909098327159881591796875f,0.7470999988257887025611125864088535308837890625f,0.1846999972939491474477335941628552973270416259765625f,0.21999998390674591064453125f,-0.059599988245963686495088040828704833984375f,-0.3170000051259993512076107435859739780426025390625f,-0.11950001361966133828218517010100185871124267578125f,0.28509998766481903231806427356787025928497314453125f,0.185899998730421123216416390278027392923831939697265625f,0.8273999892756342955379977865959517657756805419921875f,0.331599956305325349603663198649883270263671875f,0.406299994055926816383106370267341844737529754638671875f,0.6868000177264210748262485140003263950347900390625f,-0.5374000308871274000921403057873249053955078125f,0.043800002437829999024643257143907248973846435546875f,0.02569999687671664645449709496460855007171630859375f,0.378700007545948125198265188373625278472900390625f,-0.043900002396106696522792844916693866252899169921875f,-0.9341000035047528626819257624447345733642578125f,1.437299969387055398328811861574649810791015625f,0.6124999840557574515287342364899814128875732421875f,0.1243999686598780129997976473532617092132568359375f,-0.3365999847292897584338788874447345733642578125f,0.044800006192922570136261128936894237995147705078125f,-0.3603999971508979882628409541212022304534912109375f,1.0533999860644343016247148625552654266357421875f,0.278099993193149597203728262684307992458343505859375f,-0.7878999938726423124535358510911464691162109375f,1.3625000274181360282454988919198513031005859375f,0.52850001299381244024289117078296840190887451171875f,-0.208999998331069747337096487171947956085205078125f,-0.749399955248833293808274902403354644775390625f,-0.16769998427033439014621762908063828945159912109375f,-1.403699972677230789486202411353588104248046875f,0.5695000081062318031399627216160297393798828125f,0.05079999355673781025188873172737658023834228515625f,0.7648999951243400374778502737171947956085205078125f,-0.120100008451938577991313650272786617279052734375f,0.23800000485777861314318215590901672840118408203125f,-0.47160001497864723063457859097979962825775146484375f,-1.0509999936223035632565370178781449794769287109375f,0.894400014704465728954119185800664126873016357421875f,0.242799987953901119652755369315855205059051513671875f,-0.857199997889996012645497103221714496612548828125f,0.43739999408125862512264347969903610646724700927734375f,-0.36439999384284027517111326233134604990482330322265625f,-0.586899994960427218160248230560682713985443115234375f,0.132400003618001971261719518224708735942840576171875f,-0.097999998301267599032371435896493494510650634765625f,-0.271800003123283229200524147017858922481536865234375f,0.5972000279903415531634891522116959095001220703125f,0.2924000133037569781890852027572691440582275390625f,-0.687799980950355660525019629858434200286865234375f,0.2348000051498413398576303734444081783294677734375f,0.0426000021934509476295716012828052043914794921875f,0.0666000020146366722428865614347159862518310546875f,-0.6314000200539828799861652441904880106449127197265625f,0.3690000036954879103490156921907328069210052490234375f,0.50110002416521337575971983824274502694606781005859375f,0.59959999003410313633821715484373271465301513671875f,-0.73220000653266925638718021218664944171905517578125f,-1.2813999940514566588944944669492542743682861328125f,-0.371400007611513149896609320421703159809112548828125f,0.620200007814168952080535746063105762004852294921875f,0.51590003046989441060077297152020037174224853515625f,-0.3251999970853329813280652160756289958953857421875f,0.6359999803602693901893871952779591083526611328125f,1.11059997680187194646350690163671970367431640625f,-0.790199996787309455470449393033050000667572021484375f,-0.189400025731325083455658386810682713985443115234375f,0.7208999705672258784261430264450609683990478515625f,-0.465400010114908280201007073628716170787811279296875f,-0.038200007754564335726854551467113196849822998046875f,0.25769999723434455773940499057061970233917236328125f,-0.58740001792311691275472185225225985050201416015625f,0.82300000038743004182606455287896096706390380859375f,1.25270001988410939475215855054557323455810546875f,-0.92399999371170959960863910964690148830413818359375f,-0.17380000124573680153616805910132825374603271484375f,1.069199967253208427564459270797669887542724609375f,0.06960000895857820779610847239382565021514892578125f,0.41400000101327893986535855219699442386627197265625f,-1.2222999968051908581401221454143524169921875f,-0.07869997744560208730035810731351375579833984375f,-0.102300002467632378255757430451922118663787841796875f,-0.7443000111699102916418269160203635692596435546875f,1.4140999692320832537006936036050319671630859375f,0.549099986219406144272170422482304275035858154296875f,-0.528799984034896031204198152408935129642486572265625f,0.330999999210238371460945927537977695465087890625f,-0.03790000288784500792615972386556677520275115966796875f,-0.28590000394582748555194484652020037174224853515625f,0.10090000811815258430215180851519107818603515625f,0.0105000027716160015955892959027551114559173583984375f,-0.6647999920368192761088721454143524169921875f,1.0276000188827509873590315692126750946044921875f,0.47240001062154757249800240970216691493988037109375f,0.192400013899803212780170724727213382720947265625f,0.381199994134902908626827411353588104248046875f,0.08769999559521668430761565105058252811431884765625f,0.3244999978244302862862014080747030675411224365234375f,-0.62010001657307167732824382255785167217254638671875f,-0.00289999554902309331083642973680980503559112548828125f,-1.32759998550415048157447017729282379150390625f,0.2460000034570697380331694148480892181396484375f,-0.14250000849366184496602727449499070644378662109375f,0.435700005519390032304727355949580669403076171875f,-0.05690001991987259089000872336328029632568359375f,0.242300004106760002997589253936894237995147705078125f,1.24109997107982650277335778810083866119384765625f,0.0734999587535867249243892729282379150390625f,0.1808999857664108645849410095252096652984619140625f,0.6064999819397929314845896442420780658721923828125f,-1.23449997800588562313350848853588104248046875f,-0.462700003343820664980512447073124349117279052734375f,0.1280999896168708573895855806767940521240234375f,-0.68889999881982788565437658689916133880615234375f,-0.198299999517202341081656413734890520572662353515625f,1.284700000572204459103886620141565799713134765625f,-0.1655000509023665244967560283839702606201171875f,0.2122999930799007728410288109444081783294677734375f,1.002499986290931843768703402020037174224853515625f,-1.3898999554872517592229996807873249053955078125f,-0.54099998691678052864517667330801486968994140625f,-0.32910002138614657951620756648480892181396484375f,0.204200031924248293080381699837744235992431640625f,-0.64399999237060523427089719916693866252899169921875f,0.01679998106956492875951880705542862415313720703125f,0.1819999848604201275747982435859739780426025390625f,0.0106000008672477719073867774568498134613037109375f,-0.261999986425041964110960179823450744152069091796875f,-0.788299975414573861343114913324825465679168701171875f,0.73420001791715616690225942875258624553680419921875f,0.16319998883008945966821556794457137584686279296875f,-0.598500013887882431617981637828052043914794921875f,-0.139400002634525232991791199310682713985443115234375f,-0.237599996531009605149620256270281970500946044921875f,-0.3064999966919419449595807236619293689727783203125f,0.718800011527538362798850357648916542530059814453125f,-0.210599996173381764918985936674289405345916748046875f,-1.00019998739957838296277259360067546367645263671875f,0.07460000970363633854276486090384423732757568359375f,0.3298000224351884668294587754644453525543212890625f,0.45569999784529269248878335929475724697113037109375f,0.24440000442266462954421513131819665431976318359375f,-0.01119999026060103375357357435859739780426025390625f,0.23659998353719746688739178352989256381988525390625f,0.21639999762773509672797445091418921947479248046875f,-0.11379999483823777239877017564140260219573974609375f,-0.687399985438585048314052983187139034271240234375f,0.60779999849647270426800105269649066030979156494140625f,-0.09399998886883265125646857995889149606227874755859375f,-1.1448999793477352415749237479758448898792266845703125f,-0.593400030320883065115822319057770073413848876953125f,0.543000007092952596821078259381465613842010498046875f,0.3829000556379555941788339623599313199520111083984375f,-0.52680000342130650636818245402537286281585693359375f,-0.49500001877546306872091008699499070644378662109375f,0.5009999793171875381858626496978104114532470703125f,-0.311400011485815308986957461456768214702606201171875f,0.562399989610910200354965127189643681049346923828125f,0.94629999930858588186310953460633754730224609375f,0.15940001261830349932324679684825241565704345703125f,0.718800022107362934775665053166449069976806640625f,-0.05709997118413401295811127056367695331573486328125f,-0.2522000115245581763900872829253785312175750732421875f,0.1088000036299228145963979841326363384723663330078125f,0.45950000680238012318312712523038499057292938232421875f,-0.2139999916255472811599247506819665431976318359375f,0.40740001658201219214561206172220408916473388671875f,-0.0616000309228894593616132624447345733642578125f,-0.918399973845481554235448129475116729736328125f,-0.20159999396800998283652006648480892181396484375f,-1.00579998789429669869832650874741375446319580078125f,-0.23999996311962679129692332935519516468048095703125f,-0.062699983525276226092870501815923489630222320556640625f,-0.467800019693374480311831575818359851837158203125f,1.1439999810457237572336453013122081756591796875f,0.4108999899387359988622847595252096652984619140625f,-0.4420000012516975829157672706060111522674560546875f,-0.401399978256225864470252417959272861480712890625f,-0.1588999916672706458342645419179461896419525146484375f,-0.9617999843478202137703192420303821563720703125f,0.81900000876188272513900301419198513031005859375f,0.208499999135732583255276040290482342243194580078125f,0.5981000015377997414134370046667754650115966796875f,0.3851000041306011922870311536826193332672119140625f,0.18170001121163370516597979076323099434375762939453125f,0.4278000000238417754871989018283784389495849609375f,0.37139999613761887076179846189916133880615234375f,0.0292000000357628408664822927676141262054443359375f,-0.793999998927116479308097041212022304534912109375f,0.177200006234645712765995995141565799713134765625f,0.0839000049293041616493837864254601299762725830078125f,-1.21839999910295038176855086931027472019195556640625f,0.726999998308718087258739615208469331264495849609375f,0.276900005037337637592287364896037615835666656494140625f,0.165600018823146566404602708644233644008636474609375f,-0.4498000345051291759546074899844825267791748046875f,0.1442000010043382740576589640113525092601776123046875f,0.4746000091075899263159953989088535308837890625f,-1.0853999783754346708519733510911464691162109375f,-0.56370000709891332490997228887863457202911376953125f,1.0390999906897544491357621154747903347015380859375f,0.026899951374531383407884277403354644775390625f,0.158199986445903828524706113967113196849822998046875f,-0.0856000021338463312758904066868126392364501953125f,0.7441999969065182796157387201674282550811767578125f,0.54630001510381698270890638013952411711215972900390625f,0.4768000080406664675791716945241205394268035888671875f,-0.25300001282989992290595182566903531551361083984375f,-0.06490000156909225825785370034282095730304718017578125f,0.3606999837636948313956963829696178436279296875f,-0.119299972873926574123970567598007619380950927734375f,0.03480000298917307333113058120943605899810791015625f,-0.292399986258149002793516046949662268161773681640625f,-0.613800000835954318034737298148684203624725341796875f,0.35779998586773853475051510031335055828094482421875f,-0.07079997805953031075887338374741375446319580078125f,0.08330002589225760090130279422737658023834228515625f,0.3050000163912773842866954510100185871124267578125f,-0.6254000126481056298644034541212022304534912109375f,-0.116700064110755619140036287717521190643310546875f,-0.0437999973714351398257349501363933086395263671875f,0.21560000019669534054855830618180334568023681640625f,0.18690000151693819674392216256819665431976318359375f,-0.5831999862968917369698829133994877338409423828125f,-0.4696000050842759065972131793387234210968017578125f,0.2287999659299853050242745666764676570892333984375f,-0.4600000137090685115026644780300557613372802734375f,0.9504000046014784430781219271011650562286376953125f,0.6612000245332723125102347694337368011474609375f,0.7641999978005884486265131272375583648681640625f,0.02900002083182329215560457669198513031005859375f,-0.2922999801158905341935678734444081783294677734375f,0.198800007317960325536887467023916542530059814453125f,0.4250000106915832187581827383837662637233734130859375f,-0.4473999810613686900495622467133216559886932373046875f,0.1550000029802323098238048260100185871124267578125f,-0.6113999925613402552926345379091799259185791015625f,-0.07330002827644310769983349018730223178863525390625f,0.10759999518990515099403637577779591083526611328125f,-0.84359999179244038458591603557579219341278076171875f,-0.07700003135204269710811786353588104248046875f,0.641799999323487302405055743292905390262603759765625f,0.146600031891465254485495961489505134522914886474609375f,-0.458699966679513071454721284680999815464019775390625f,0.02420000212192530142374380375258624553680419921875f,-0.73680002458095561479467505705542862415313720703125f,-0.333500043094158371559387887828052043914794921875f,-0.3818000011861324427542285775416530668735504150390625f,0.4687999923050403250357476281351409852504730224609375f,0.49430000267624851773007321753539144992828369140625f,-0.8322000304490331235029998424579389393329620361328125f,-0.039600011566281512198628433907288126647472381591796875f,0.638300009314715754982216822099871933460235595703125f,-0.06139998913407307412626323639415204524993896484375f,0.239999997019767707939763567992486059665679931640625f,-0.76040000951886188573780600563623011112213134765625f,-0.21119996470511015473903171368874609470367431640625f,-0.101599982643127362269552804718841798603534698486328125f,0.2851000468969342449554460472427308559417724609375f,0.48209998049735958147721248678863048553466796875f,0.04339999589920040534707368351519107818603515625f,0.061399971103668349314830265939235687255859375f,-0.0099999761581415214095613919198513031005859375f,-0.32260000442862502012530967476777732372283935546875f,0.2011999962210655468197728623636066913604736328125f,0.1061999936878681438656712998636066913604736328125f,0.10790000013113021370969590861932374536991119384765625f,0.085699981379509182488618534989655017852783203125f,-1.1734999384880069328573881648480892181396484375f,-0.52349998497962957344498136080801486968994140625f,0.54759997715949992880268837325274944305419921875f,0.0619999556541444007962127216160297393798828125f,0.42160000290870680572652418049983680248260498046875f,-0.033499964118004044166809762828052043914794921875f,1.118299977016448565336759202182292938232421875f,0.28369999487996100384634701185859739780426025390625f,-0.548400013855099732751341434777714312076568603515625f,0.557999997518956458719685542746447026729583740234375f,0.4129000105619431426617893521324731409549713134765625f,0.0472000053405761121894101961515843868255615234375f,-0.216599971914291700159083120524883270263671875f,-0.362799996298551530315990021335892379283905029296875f,0.047500010728835917461765347979962825775146484375f,-0.452899970030784970731474459171295166015625f,-0.53489999750852579296633848571218550205230712890625f,-0.3241999669179322918211028081714175641536712646484375f,1.0083999919876447393107810057699680328369140625f,0.352600007446110252917748084655613638460636138916015625f,-0.342800019693374480311831575818359851837158203125f,-0.5209999808073035865163546986877918243408203125f,-0.4491000094652175533838089904747903347015380859375f,0.42009998207688337146237245178781449794769287109375f,-0.18250001624226541707685100845992565155029296875f,0.1387999991595745274963746851426549255847930908203125f,0.4602000006243585072951418624143116176128387451171875f,-1.196000017277896176892681978642940521240234375f,-0.360500005789101074515912159768049605190753936767578125f,-0.11590001616477962897988618351519107818603515625f,0.8448000126004213683472698903642594814300537109375f,0.522200000721216195387341940659098327159881591796875f,-0.526199990409612627928481742856092751026153564453125f,-0.623699997413158513381858938373625278472900390625f,0.4448000060439110114884897484444081783294677734375f,-0.2223999984025955001243346487171947956085205078125f,-0.721599992924928823612162887002341449260711669921875f,-0.29200000378489487928845846909098327159881591796875f,-0.33640000612139697722113851341418921947479248046875f,0.579199988412856470176848233677446842193603515625f,-0.228600001329183566411984429578296840190887451171875f,-0.017200001764297478956677878159098327159881591796875f,0.4152999868363143054494912576046772301197052001953125f,0.544199985581636180853593032225035130977630615234375f,-0.549399984902143589948764201835729181766510009765625f,-1.0382999844670290912063137511722743511199951171875f,0.50000001758337031532164473901502788066864013671875f,-0.88420002566576005875731425476260483264923095703125f,-0.8376000424265868105067056603729724884033203125f,0.049400012767315093498154965345747768878936767578125f,0.744600014770031126687399591901339590549468994140625f,0.30430001876950285577549948357045650482177734375f,0.489999982640146936052616410961491055786609649658203125f,-0.6335999890357257857687045543571002781391143798828125f,-0.856599984617531173825000223587267100811004638671875f,0.211199990111589297470118253841064870357513427734375f,-0.664199998396635127306808499270118772983551025390625f,-0.8472000026583668130797377671115100383758544921875f,-0.63980001558065424660526332445442676544189453125f,0.50840000692605968879433930851519107818603515625f,1.195300011050701272097285254858434200286865234375f,-0.60600000613927829107296929578296840190887451171875f,-0.52459999808073032312449868186376988887786865234375f,0.57019996981620835185822215862572193145751953125f,0.1862000047147274184311527278623543679714202880859375f,0.1694000039756298203741380348219536244869232177734375f,-0.17889999434947956302721649990417063236236572265625f,0.44220001681447040908778944867663085460662841796875f,0.67960001581311235696603034739382565021514892578125f,-0.3088999834716315007199227693490684032440185546875f,0.061400001502037060419070257921703159809112548828125f,-0.190600001239776606354325849679298698902130126953125f,-0.354900006374716525670010014437139034271240234375f,-0.0701999945521354362654165015555918216705322265625f,0.150199994254112123570621406543068587779998779296875f,-0.5188999725937843976453223149292171001434326171875f,0.3365000147223469184609712101519107818603515625f,-0.118700000840425534676114693866111338138580322265625f,-0.71999999046325680041036321199499070644378662109375f,0.432000009000301332662274944595992565155029296875f,0.080799999815225564958609538734890520572662353515625f,0.668700021553039647415062063373625278472900390625f,-0.62009998967647561585181392729282379150390625f,-0.39830000652074826206217039725743234157562255859375f,0.444199994075298310036714610760100185871124267578125f,-0.3123999982535838881858580862171947956085205078125f,-0.1379000000566243944177813318674452602863311767578125f,-1.0536999801278117416813984164036810398101806640625f,0.3165000039935108588906587101519107818603515625f,-0.068499997645616428343373627285473048686981201171875f,-0.769800007534026775601887493394315242767333984375f,0.0290000311136253685617703013122081756591796875f,-0.339300011616945340620077331550419330596923828125f,0.3375999965310096939674622262828052043914794921875f,0.632199989843368115316479816101491451263427734375f,0.4800000090897083016017177214962430298328399658203125f,-0.8289999693110587930533483813633210957050323486328125f,0.19220000046044560804148204624652862548828125f,-0.168000003106892059623334034768049605190753936767578125f,-0.9428999965548516826174818561412394046783447265625f,-0.23849998110532766304459073580801486968994140625f,-0.27119999756216994768465156084857881069183349609375f,1.22140000507831558707039221189916133880615234375f,-0.47280002565383938417653553187847137451171875f,-0.0098999998927116195091002737171947956085205078125f,0.532599995040893503528423025272786617279052734375f,-1.21699996399879495356799452565610408782958984375f,-0.616899991458654373133185799815692007541656494140625f,-0.16950000318884850258882579510100185871124267578125f,0.379499995738267870137860882095992565155029296875f,0.195700001943111434599131825962103903293609619140625f,-0.2819000142574310530108050443232059478759765625f,1.109899988567828899022060795687139034271240234375f,0.6378000098586082966534149818471632897853851318359375f,-0.4179000093698501672179190791212022304534912109375f,-0.0768999881803988927231330308131873607635498046875f,-0.06149999669194217144507774719386361539363861083984375f,0.21259999086856851135962642729282379150390625f,0.7038999975346025284039797043078579008579254150390625f,-0.36800000932812704235175260691903531551361083984375f,-0.49460003011822717411405392340384423732757568359375f,-0.24320002101659810023193131200969219207763671875f,0.35279999048709864695183568983338773250579833984375f,-1.00799997103214256100045531638897955417633056640625f,-0.4620000206232059980493431794457137584686279296875f,-0.25299999114871007943605718537583015859127044677734375f,0.81540001071095480345007899813936091959476470947265625f,0.71470001390874404734177005593664944171905517578125f,-0.22120000411868101508616746286861598491668701171875f,-0.15319999821782115390078615746460855007171630859375f,0.408999971061945633010736855794675648212432861328125f,0.25140000448226917928451484840479679405689239501953125f,0.28739999065399157718303513320279307663440704345703125f,-0.04790000154674078913785706390626728534698486328125f,0.555800024104118772783067470300011336803436279296875f,-0.786199992346763298911582751316018402576446533203125f,-1.308900007164478296317611238919198513031005859375f,0.130200002971291528552910676808096468448638916015625f,0.154200000110268575781446998007595539093017578125f,-0.289699994425475448878160023014061152935028076171875f,-0.4950000036507844480837547962437383830547332763671875f,0.1733999903857708080323618560214526951313018798828125f,0.90569999803155598616655197474756278097629547119140625f,0.58140000165104854801256806240417063236236572265625f,-0.23319998152852061679141115746460855007171630859375f,-0.301700016278027671745576299144886434078216552734375f,0.7068000020802021055033037555404007434844970703125f,-0.5209999967515468455303562222979962825775146484375f,-1.27779997379779874933092287392355501651763916015625f,0.62760001322031033765824759029783308506011962890625f,0.55760002037286771070512259029783308506011962890625f,-0.48859995856285109994132653810083866119384765625f,-0.717400015687942538278321080724708735942840576171875f,-0.173599997901916491827023492078296840190887451171875f,0.7676999882936481611750423326157033443450927734375f,-0.1802000014066695854353383765555918216705322265625f,-0.35540000102519986313609479111619293689727783203125f,0.22109997607171560929373299586586654186248779296875f,-0.47120001156926161200999558786861598491668701171875f,0.49980000857710837891545452293939888477325439453125f,0.31000002324581199530939557007513940334320068359375f,-0.7424000203818084120399589664884842932224273681640625f,0.26219998960122470332834154760348610579967498779296875f,-0.434700021433830130490605370141565799713134765625f,-0.9158999917268744184184470213949680328369140625f,-0.4102000093042850181745961890555918216705322265625f,-0.32759997775554694499078323133289813995361328125f,-0.24359996035099129585432820022106170654296875f,-0.52079998774528490912416600622236728668212890625f,0.730599991926550895726677481434307992458343505859375f,0.2739999867826696799966157414019107818603515625f,0.38160000804960725329095794222666881978511810302734375f,0.45529998035430896408115586382336914539337158203125f,-0.399499995291233034322431194595992565155029296875f,-0.2278999970018862342158172396011650562286376953125f,0.020099989601969792119007252040319144725799560546875f,0.571099974432588197714721900410950183868408203125f,0.27649999065697195010926634495262987911701202392578125f,-1.3779999880194662154053730773739516735076904296875f,0.140600067698954944717115722596645355224609375f,-0.188199985700845839886596877477131783962249755859375f,-0.34020000990033150145563922706060111522674560546875f,0.27679999574720870469946021330542862415313720703125f,0.185000001937150992947778149755322374403476715087890625f,0.9456000091373921723203466171980835497379302978515625f,-0.52500000886619080375794510473497211933135986328125f,-0.15540000709742318729666976651060394942760467529296875f,-0.10030000002384209523142999387346208095550537109375f,0.661099999392032344758263207040727138519287109375f,0.052600003236532355543886296800337731838226318359375f,-1.2976999638557433769392446265555918216705322265625f,0.55510001471042613729878212325274944305419921875f,0.0325999969780443876743447617627680301666259765625f,-0.418599977040290838203873136080801486968994140625f,1.2793999874353403356508351862430572509765625f,0.63400000548362722696538185118697583675384521484375f,0.7003999971508978461542938021011650562286376953125f,-1.1121999895453456019822624512016773223876953125f,-0.437499994933605140801091692992486059665679931640625f,0.2498999945282935897239440237171947956085205078125f,-0.5002999828875065446709413663484156131744384765625f,-0.32729999814629551213585045843501575291156768798828125f,0.1312999885797498933470706106163561344146728515625f,-0.30750001192092923929521930404007434844970703125f,0.16060000973343857566533188219182193279266357421875f,1.1780000108182429041647765188827179372310638427734375f,-0.6693999893724917882309455308131873607635498046875f,0.068000008992850968780885523301549255847930908203125f,0.4092000219032170260646807946613989770412445068359375f,0.4280000113211575030192079793778248131275177001953125f,-0.1691999952673910456724115647375583648681640625f,0.444400019323826001027555321343243122100830078125f,0.766600006186962446008692495524883270263671875f,0.2480000045597552382758976818877272307872772216796875f,0.1293999947369097736071807958069257438182830810546875f,0.0215999887526034939355668029747903347015380859375f,-0.38359998359680158586115794605575501918792724609375f,0.40499997973442081189432428800500929355621337890625f,0.875799965989589423998040729202330112457275390625f,0.65300000590085982565824451739899814128875732421875f,-0.77479999591112136414494671043939888477325439453125f,-1.49430001355409558527753688395023345947265625f,0.2574000025004148373142243144684471189975738525390625f,-0.3276000002562999835475920917815528810024261474609375f,0.02949998527020192273795373694156296551227569580078125f,-0.63600000409036872017765062992111779749393463134765625f,0.1973999909147620412941392942229867912828922271728515625f,0.6507999952517449226974122211686335504055023193359375f,0.218599981659650666188099421560764312744140625f,-1.05159999173283580375937162898480892181396484375f,-0.3006999977707867088838611380197107791900634765625f,-0.209200012552738456861334270797669887542724609375f,0.44819999002218224859461770392954349517822265625f,1.1062000018835060899391464772634208202362060546875f,-0.30999999776482578539571477449499070644378662109375f,0.68500000700354579663553522550500929355621337890625f,1.170599997365474820298913982696831226348876953125f,-0.1553999916374682044306609896011650562286376953125f,0.6702000107944012796679089660756289958953857421875f,0.6791000058889391510774657945148646831512451171875f,-0.23700000035762780470349753159098327159881591796875f,0.09359999730587009736382242408581078052520751953125f,-0.0960999922394751848742089350707828998565673828125f,-0.2036000014781951250597558100707828998565673828125f,0.4324000252246857911586630507372319698333740234375f,0.875000002980232949312267010100185871124267578125f,0.346399982875585266128837247379124164581298828125f,-0.541000005394220551124817575328052043914794921875f,-0.985199980247021045443034381605684757232666015625f,-0.228400004392862232549532564007677137851715087890625f,-0.1750000019371509285548427214962430298328399658203125f,0.549100012370943968420533565222285687923431396484375f,0.429900001047550883725989478989504277706146240234375f,0.2381000097334384779657057151780463755130767822265625f,0.8818000084877013478035223670303821563720703125f,0.13579999787807395250638364814221858978271484375f,0.144600008809566471512653151876293122768402099609375f,0.12889998973011973504299021442420780658721923828125f,0.45889998287558597667157300747931003570556640625f,0.1573999930381775325116677777259610593318939208984375f,0.930799965095520320801369962282478809356689453125f,-0.198000005602835926765692420303821563720703125f,0.03969999476075170008471104665659368038177490234375f,-1.579000010251998986632315791212022304534912109375f,0.2756000352144241105634137056767940521240234375f,-0.1254999921917914207369904033839702606201171875f,0.1974000140488147980732946962234564125537872314453125f,0.14600000323355200038122347905300557613372802734375f,-0.15370000646561388091271282974048517644405364990234375f,0.54029998651966459899398387278779409825801849365234375f,-0.736899999114125847654577228240668773651123046875f,-0.1431999986089766141450496661491342820227146148681640625f,-0.171899943923950360158414696343243122100830078125f,-0.4580999797821050378843210637569427490234375f,-0.61280000687837610229280471685342490673065185546875f,1.311400012230873102225814363919198513031005859375f,0.10899994766712239879780099727213382720947265625f,0.086699980217218364231257510255090892314910888671875f,1.321699994969367963904005591757595539093017578125f,0.136099974656104905079700984060764312744140625f,0.21019999976754188963923297706060111522674560546875f,0.746099992537498479805435636080801486968994140625f,0.3102999875068661594923469237983226776123046875f,0.3157000062644481630513837444595992565155029296875f,-0.24840000901222225593301118351519107818603515625f,-0.2893999922037124861162737943232059478759765625f,0.00910000529289245463360202847979962825775146484375f,1.0104000051975248197777546010911464691162109375f,0.0859999933242789893483859486877918243408203125f,0.11080001069307332528524057124741375446319580078125f,-1.07669997887611401665708399377763271331787109375f,0.036499994754791487139300443232059478759765625f,-0.12780000851750372703463654033839702606201171875f,-0.5789000047802925763562598149292171001434326171875f,-0.271100005948543554268326261080801486968994140625f,-0.54399998894333811971790737516130320727825164794921875f,0.81780000881552716851530249186907894909381866455078125f,1.09320000306069875506409516674466431140899658203125f,0.19140000463128092889064646442420780658721923828125f,0.02640000552535060052150583942420780658721923828125f,0.020899997240305001611204716027714312076568603515625f,-0.356399999713897575048804355901665985584259033203125f,0.540800016355514667765191916259936988353729248046875f,0.8474000130057337543121320777572691440582275390625f,-0.62040003112554575892545472015626728534698486328125f,0.34380000288486467496795739862136542797088623046875f,0.417400050705670988548945388174615800380706787109375f,0.10260000547170644580319276428781449794769287109375f,0.59760001024007802783444276428781449794769287109375f,0.6777000172019000956424861215054988861083984375f,-0.348000002026557897494285498396493494510650634765625f,-0.426800016236305257422145587042905390262603759765625f,0.085999973952770147889168583787977695465087890625f,0.67980002213716517189823207445442676544189453125f,-0.2056000051140784989911480806767940521240234375f,-0.45770003001689918420424874057061970233917236328125f,-0.550600000345706774851350928656756877899169921875f,0.589400015151500866750211571343243122100830078125f,1.331500007867813195616690791212022304534912109375f,-0.2712000148475173233464374789036810398101806640625f,0.4569999909698960749437901540659368038177490234375f,1.093199997174739479532945551909506320953369140625f,-0.21100001373887078415236828732304275035858154296875f,-0.10500000312924395728941817651502788066864013671875f,-0.32629999185800517835787104559130966663360595703125f,-0.219600014024972978177174809388816356658935546875f,1.071800035756826474653280456550419330596923828125f,0.30300006788969113813436706550419330596923828125f,0.69659999799132332753970331395976245403289794921875f,0.21360002636313435431247853557579219341278076171875f,-0.995299968135356039056205190718173980712890625f,-0.333399985468387338727325186482630670070648193359375f,0.831799998652935190790458364062942564487457275390625f,0.588900016105174728409110684879124164581298828125f,-0.26220000414848332326300806016661226749420166015625f,-0.00160001423358924199646935448981821537017822265625f,-0.14229999918937696889997823745943605899810791015625f,0.881200026917457801545197071391157805919647216796875f,-0.35759999891519544945595043827779591083526611328125f,1.094799985405802811300191024201922118663787841796875f,-0.119200003239512852815096266567707061767578125f,0.24430001094639297587463033778476528823375701904296875f,0.11370000426769255597037044935859739780426025390625f,0.301899999058246493177648517303168773651123046875f,0.070600000196695322784989912179298698902130126953125f,0.8900000062584876303617420489899814128875732421875f,-0.06320000611543719060136936604976654052734375f,0.144600008809566471512653151876293122768402099609375f,0.22200002598762491601291912957094609737396240234375f,-0.629200029242038993970709270797669887542724609375f,-0.004900002276897463815430455724708735942840576171875f,0.873000020503997831156084430404007434844970703125f,0.63719997375011416806955821812152862548828125f,0.34109999611377705974746277206577360630035400390625f,0.0015999853253365614591530174948275089263916015625f,-0.809599980795383089571259915828704833984375f,-0.405400002217292776407475685118697583675384521484375f,-0.2638000112295149079955081106163561344146728515625f,0.81200000751018563960315077565610408782958984375f,0.122499994933605194091796875f,1.43340001139640804694863618351519107818603515625f,-0.272000008106232371574151329696178436279296875f,0.0518000065505503926033270545303821563720703125f,1.2643000013351439747566473670303821563720703125f,0.0230999475955968591733835637569427490234375f,0.07659998279213908745077787898480892181396484375f,-0.5171000114917756462773468228988349437713623046875f,-0.41330000085830675971010350622236728668212890625f,-0.08609999715685834331679870956577360630035400390625f,0.172100015068053835420869290828704833984375f,0.471299968910216193762607872486114501953125f,0.62600001761317258797134854830801486968994140625f,-1.015499996066093757463022484444081783294677734375f,1.090699988532065844992757774889469146728515625f,0.49160001065731062652730543049983680248260498046875f,0.5249999889731409297155551030300557613372802734375f,0.1567999920964240345711004920303821563720703125f,0.387600005322694673992600655765272676944732666015625f,-0.3100000154972075705472889239899814128875732421875f,-0.39860000028610187428057543002068996429443359375f,-0.056700001823902113073927466757595539093017578125f,-0.7623999939322469998614906216971576213836669921875f,0.49529999290853721216620897394022904336452484130859375f,-0.5686000005841254090199754500645212829113006591796875f,0.1875999935507774996068519612890668213367462158203125f,0.46849999645352369270767667330801486968994140625f,0.26419999675750727163858755375258624553680419921875f,-0.01099999600648882136511019780300557613372802734375f,-0.781099971675872861709422068088315427303314208984375f,0.34780001477599142845065216533839702606201171875f,0.249800000232458074833630234934389591217041015625f,0.2274999669194226470381181570701301097869873046875f,-0.362199989992380011472050682641565799713134765625f,0.380199987548589746211291640065610408782958984375f,0.7204999817609785139893574523739516735076904296875f,-0.2172000079110265602366780512966215610504150390625f,-0.6060000140741468310778827799367718398571014404296875f,0.0501999734483657977790471704793162643909454345703125f,-0.882399996912479167576748295687139034271240234375f,-0.094400018131732821302648517303168773651123046875f,1.2747999675989145629273480153642594814300537109375f,0.84680000550746914456112790503539144992828369140625f,0.03120000844001769024771419935859739780426025390625f,-1.27739995757341429083453476778231561183929443359375f,-0.33360001414418238852022113860584795475006103515625f,0.41239999706149088609663522220216691493988037109375f,0.44080001933574664008119725622236728668212890625f,-0.0293999998033047660328520578332245349884033203125f,0.21039999223351468771170402760617434978485107421875f,0.48849998788535575755531681352294981479644775390625f,-0.317400011515617475055250906734727323055267333984375f,0.068600004011392545777425766573287546634674072265625f,0.8204999958425760464564291396527551114559173583984375f,-0.0150000022351743922399691655300557613372802734375f,0.628999988943338195213073049671947956085205078125f,0.53569999970793702459559426642954349517822265625f,0.24600001940131210886875123833306133747100830078125f,-0.66100000956654536565793023328296840190887451171875f,-0.15060004713535324327722264570184051990509033203125f,0.54200000527501102087768458659411408007144927978515625f,-0.17079999668598178086398320374428294599056243896484375f,-0.6061999990522861647690433528623543679714202880859375f,-0.35619999294280990653760454733856022357940673828125f,0.44500000298023234535094161401502788066864013671875f,0.88149999088048947015749945421703159809112548828125f,-0.53039999998211850851248527760617434978485107421875f,-0.10109999957829718386648210071143694221973419189453125f,0.33199998411536224551809937111102044582366943359375f,-0.6605999921500682603436871431767940521240234375f,-0.1040999950110912397605034129810519516468048095703125f,-0.90759999027252202807858338928781449794769287109375f,0.3129999996423720176608185283839702606201171875f,0.1816000026106834308592397064785473048686981201171875f,0.433299970459938066369431908242404460906982421875f,0.70949998605251352046252577565610408782958984375f,0.239099997693300370116276098997332155704498291015625f,0.738499977231025894752747262828052043914794921875f,-0.1977000055789943644413142465054988861083984375f,0.0512999976694583637026880751363933086395263671875f,0.036299996551871327454819038393907248973846435546875f,-0.748899986042081966530759018496610224246978759765625f,-0.4849000063002109062182398702134378254413604736328125f,1.284199993181228904859381145797669887542724609375f,-1.2541999812602995234556146897375583648681640625f,-0.50420000331401837456724024377763271331787109375f,-1.2967999721288681058695146930404007434844970703125f,1.15639999256134018423836096189916133880615234375f,0.450999998390674505799324833787977695465087890625f,0.378500000476837072937996708787977695465087890625f,-0.1567000295400617915220209397375583648681640625f,0.1218999954819679487627581693232059478759765625f,0.6069999929070473143610797706060111522674560546875f,0.1495999779641632443372145644389092922210693359375f,0.06509999086856843586446075278217904269695281982421875f,0.928100010478496439958462360664270818233489990234375f,-0.3465000139772893561485034297220408916473388671875f,-0.1633000037640333079735910359886474907398223876953125f,0.6268999832630157698076800443232059478759765625f,0.474699952590466267565716407261788845062255859375f,0.3488999873459339706727178054279647767543792724609375f,-0.0067000068902967058193098637275397777557373046875f,-0.12509996285438518270893837325274944305419921875f,-0.3592000019729137960666776052676141262054443359375f,-0.528299993482232110864060814492404460906982421875f,0.6216999993652105871433377615176141262054443359375f,0.12769999753236771500297663806122727692127227783203125f,0.9463999895811081586316504399292171001434326171875f,-0.12860000386238112923820153810083866119384765625f,0.074200003761053068274122779257595539093017578125f,0.8244999866485596129450641456060111522674560546875f,-0.52599998836964367843194168017362244427204132080078125f,-0.333000006496906308939287555404007434844970703125f,0.251400002396106714286361238919198513031005859375f,0.99310000034570666826994056464172899723052978515625f,0.611200021702051454752790959901176393032073974609375f,-0.931399997180700101040429217391647398471832275390625f,-1.360800028872489519926602952182292938232421875f,-0.1214000022470951289932372674229554831981658935546875f,-0.0354000018447637732066368698724545538425445556640625f,0.50529998966753464362255954256397671997547149658203125f,0.64080002291202564634886584826745092868804931640625f,-0.23180000163316716310646370402537286281585693359375f,-1.230400010114907960456775981583632528781890869140625f,-0.5360000215619804553313088035793043673038482666015625f,0.3788000055670737165058881146251223981380462646484375f,0.53160002224296365458400259740301407873630523681640625f,-0.488800001841783515743600219138897955417633056640625f,0.336799993139505371431141611537896096706390380859375f,0.932799988251924450111118858330883085727691650390625f,0.27460000061392764170165037285187281668186187744140625f,0.32319998778700809172192975893267430365085601806640625f,-0.02269999939501321506440945086069405078887939453125f,0.117199998635053592721533277654089033603668212890625f,-0.028399996644258518330161678022705018520355224609375f,-0.210599996769428088327913428656756877899169921875f,0.63319999151229833245224654092453420162200927734375f,-0.46140000030994432478337330394424498081207275390625f,-1.19499998629093173718729303800500929355621337890625f,0.145000005513429730541474782512523233890533447265625f,-0.395799996536970066784988375729881227016448974609375f,-0.1739000187873838143559623858891427516937255859375f,0.051799999547004649258497011032886803150177001953125f,0.353799999308586077262361868633888661861419677734375f,-0.05409998271763338806294996174983680248260498046875f,0.103600001776218419990982511080801486968994140625f,-0.511999992311000795552899944595992565155029296875f,0.050999961733818821585373370908200740814208984375f,0.750400003856420472203581084613688290119171142578125f,-0.352399998700618777291992955724708735942840576171875f,-1.1887999794155363897374400039552710950374603271484375f,-0.346600003355741481669838321977294981479644775390625f,0.146199996072053917117727905861102044582366943359375f,0.480300000321864928309878450818359851837158203125f,0.51219999520778625168304643011651933193206787109375f,0.01039999413341285983136685899808071553707122802734375f,1.1274000144958495894798033987171947956085205078125f,-0.22360001757144942757804528810083866119384765625f,-0.06760000532269483386471620178781449794769287109375f,-1.4037000081419943597893507103435695171356201171875f,0.113500063359737168866558931767940521240234375f,-0.263999988049268718981466008699499070644378662109375f,0.6152000067710876596294156115618534386157989501953125f,-0.1602000055789947197126821265555918216705322265625f,-0.132200005154311683863710413788794539868831634521484375f,-1.29980002064704880382350893341936171054840087890625f,0.669400028264522717336149071343243122100830078125f,0.204500007510185266568214501603506505489349365234375f,0.4080999987065789813556193621479906141757965087890625f,-0.47030002349317090448721501161344349384307861328125f,0.02710000258833178410355913001694716513156890869140625f,0.96509998759031301318600526428781449794769287109375f,-0.396500013232230941184752737171947956085205078125f,-0.0457000029861927004048993694595992565155029296875f,0.059999990686774307135920025757513940334320068359375f,0.63299996365606858717001159675419330596923828125f,0.30409998644292357550256156173418276011943817138671875f,-0.2203000003218651414726991788484156131744384765625f,0.133699999201297714535030536353588104248046875f,0.044900001382827792184571080724708735942840576171875f,-1.2295999876499177361210968228988349437713623046875f,0.49679999537467978143467917107045650482177734375f,0.04009999429583555041745057678781449794769287109375f,-0.1156999995589256258199384319595992565155029296875f,-0.08549997788667695886033470742404460906982421875f,-0.20369999666810034710806576185859739780426025390625f,-0.088900000905990594901595613919198513031005859375f,0.3248999824583531648158896132372319698333740234375f,0.26489999825358390150853438171907328069210052490234375f,0.17579998893737780463197850622236728668212890625f,-0.2101999781608583361958153545856475830078125f,-0.3275999950408934324741494492627680301666259765625f,-1.179599975430965397293903151876293122768402099609375f,0.7476000041306019028297669137828052043914794921875f,0.1477999953299760083069713800796307623386383056640625f,0.4558999991774559390478316345252096652984619140625f,-0.3631000039219856034833355806767940521240234375f,-0.069700001019239454791431853664107620716094970703125f,0.3756000143527984391766949556767940521240234375f,0.79660000022649768425253569148480892181396484375f,0.79680003054142023444228470907546579837799072265625f,-0.07019999932050691171525613754056394100189208984375f,-1.320400011008977525506224992568604648113250732421875f,0.011600001567602102880982783972285687923431396484375f,0.550600003474950749904337499174289405345916748046875f,0.1674000095784666797271711402572691440582275390625f,0.21680000833868984244645616854541003704071044921875f,0.65180001072287563346208116854541003704071044921875f,0.007600018584728474024814204312860965728759765625f,0.12680000476241115592301866854541003704071044921875f,0.10640000373721125725978708942420780658721923828125f,0.24379999766945825712127771112136542797088623046875f,0.10300000523030750088082641013897955417633056640625f,0.600399998864531436737479452858678996562957763671875f,0.577899996070563748418180693988688290119171142578125f,-0.336200003671645930580780259333550930023193359375f,-0.265599996471404864450960303656756877899169921875f,-0.2416999797701837593422169447876513004302978515625f,0.0792000034630297999882486692513339221477508544921875f,0.5039999978840350447484297546907328069210052490234375f,0.41560000340044478406298367190174758434295654296875f,-0.09539999580979330318086795159615576267242431640625f,0.66740001434683815517701077624224126338958740234375f,-0.0564999554157257222186672152020037174224853515625f,0.63240001434683801306846362422220408916473388671875f,0.01579999892115591109131855773739516735076904296875f,-0.85340000305175767181253831950016319751739501953125f,0.73900001055002206840072176419198513031005859375f,0.001200006353855087581905536353588104248046875f,-0.284900007641315511364155099727213382720947265625f,0.6027999954044815922316047362983226776123046875f,0.4515999905407426240344648249447345733642578125f,-0.5109999605417261392403815989382565021514892578125f,-0.40860002084970492575166645110584795475006103515625f,0.53280001999139781077019506483338773250579833984375f,0.36450003433227617932743669371120631694793701171875f,-0.47620000694990149980867499834857881069183349609375f,-0.33560000690221780672573004267178475856781005859375f,0.5714999938607210339114317321218550205230712890625f,-0.512600002938508936267680837772786617279052734375f,0.137999995023012189676592242904007434844970703125f,0.635299991530180019339013597345910966396331787109375f,0.32240001926422134914673733874224126338958740234375f,-0.59469999625086789052375024766661226749420166015625f,-1.36500002741813641904400355997495353221893310546875f,1.006600027048587975286864093504846096038818359375f,0.373300009500980412013859677244909107685089111328125f,0.1618999853491782658920783433131873607635498046875f,0.425899977421760667084527085535228252410888671875f,0.3560000015199185074976639953092671930789947509765625f,0.034499993205070467183759319595992565155029296875f,0.786099980616569382618763484060764312744140625f,0.50780000464320185216138270334340631961822509765625f,0.3687000078439712780209447373636066913604736328125f,0.283499994218349371521981083787977695465087890625f,0.0744999976754188164562719975947402417659759521484375f,0.74099998900294306025671176030300557613372802734375f,0.63119996485412155351468754815869033336639404296875f,0.357499992847442660259815738754696212708950042724609375f,-0.444799981904029806401013047434389591217041015625f,-0.7049999678134923897232511080801486968994140625f,-0.2444999902248383705227752216160297393798828125f,0.52679999582171443961442491854541003704071044921875f,0.5131999703526499700956264859996736049652099609375f,0.30799999323487281532862880339962430298328399658203125f,-0.1974999904632568359375f,0.333099995875357990371412597596645355224609375f,0.43710001164078704505300265736877918243408203125f,-0.8347000142812728284980039461515843868255615234375f,0.86689999995231659113414934836328029632568359375f,0.31720000012516980092414087266661226749420166015625f,-0.03219999163150788490384002216160297393798828125f,1.0619999854564667884915252216160297393798828125f,0.25629999305009842913705142564140260219573974609375f,-0.447299979519844015385388047434389591217041015625f,1.1650999461650854982508462853729724884033203125f,0.5574999806284903769437732989899814128875732421875f,0.2286000002861019453348490060307085514068603515625f,-0.18720002202987728878724738024175167083740234375f,0.1935000109076501217941768118180334568023681640625f,0.57260002007484445130103267729282379150390625f,-0.8907999847650529545717290602624416351318359375f,-0.45010000368356717359574759029783308506011962890625f,0.58429998941421512625993273104541003704071044921875f,-0.599499998271465273091962444595992565155029296875f,-0.145799998325109481100980701739899814128875732421875f,0.585599996471405148668054607696831226348876953125f,0.3994999783039094864989237976260483264923095703125f,0.1854000021427868460932586458511650562286376953125f,0.422999993011355268635043103131465613842010498046875f,-0.203299993817508184434927898109890520572662353515625f,-0.48920000569820398794718130375258624553680419921875f,0.19199998545646668191011485760100185871124267578125f,-0.07499997347593367891249727108515799045562744140625f,0.456999989181756660627797828055918216705322265625f,0.051200010973214915566131821833550930023193359375f,-0.013499994814395677167340181767940521240234375f,0.162399997471272872218150951084680855274200439453125f,0.2887999945029615656011401370051316916942596435546875f,-0.3039999666102237707576705361134372651576995849609375f,0.3261999963700770166497022728435695171356201171875f,-0.5052000116884709512987683410756289958953857421875f,-1.0196999958038333033982780762016773223876953125f,-0.6041999890834091058167132359812967479228973388671875f,0.2209999895989895879466757833142764866352081298828125f,1.053499960504472365840911152190528810024261474609375f,-0.31400002709031138437012486974708735942840576171875f,-0.46300002661347416932358100893907248973846435546875f,-0.44259997924566363280973746441304683685302734375f,-0.7108000260412694615297368727624416351318359375f,-0.18040000296235092491770046763122081756591796875f,1.0682000082015985498173904488794505596160888671875f,-0.459400019919872448781461571343243122100830078125f,0.053799997222423456832984811626374721527099609375f,0.17150002187490454019780372618697583675384521484375f,0.1856000073492527491225700941868126392364501953125f,-0.4446000062763690863221199833787977695465087890625f,0.050399968391656724264748845598660409450531005859375f,0.52260000174641607628700512577779591083526611328125f,-0.65299999472498893027250232989899814128875732421875f,-0.611500009059906091124503291212022304534912109375f,0.27220000027418134180834385915659368038177490234375f,-0.15640000448226931695216990192420780658721923828125f,-0.813499982446432401417268920340575277805328369140625f,0.1090000022053718709003078402020037174224853515625f,-0.2809999983906745768535984097979962825775146484375f,-0.12110001608133291739477499504573643207550048828125f,-0.54339998669773315054243312260950915515422821044921875f,0.11239998685419572144184030548785813152790069580078125f,0.99749996205791813252972133341245353221893310546875f,0.590600010925531382355302412179298698902130126953125f,0.129299995374679621562563625047914683818817138671875f,-0.23190002006888388308425419381819665431976318359375f,0.55789997308552319310592793044634163379669189453125f,0.372499995380640103181946187760331667959690093994140625f,-0.3496000001668928547360337688587605953216552734375f,-0.58759999861717204794331337325274944305419921875f,0.00019999991655339499629917554557323455810546875f,0.1891000136375424744983320124447345733642578125f,0.7068999665737152326983050443232059478759765625f,0.16959999182224283487130378489382565021514892578125f,-0.359899983277916835078258372959680855274200439453125f,0.419299978499114711638640073942951858043670654296875f,0.28849999257922165352141519178985618054866790771484375f,-1.13589999709129330085488618351519107818603515625f,0.02950000870227853511096327565610408782958984375f,-0.104200005400180817360933360760100185871124267578125f,1.0843999661266805478732067058444954454898834228515625f,-0.63859999007880663413061483879573643207550048828125f,-0.17579999709576366040408856861176900565624237060546875f,-0.676699984240531904333693091757595539093017578125f,-0.8311000128030769928955123759806156158447265625f,-0.3442000162780285421604276052676141262054443359375f,1.101300033879279993698219186626374721527099609375f,-1.081500011444092024248675443232059478759765625f,-0.48350000911951074300532127381302416324615478515625f,0.0062000072479246881584913353435695171356201171875f,-0.62160001065731051994589506648480892181396484375f,-0.12120000084042548138540951185859739780426025390625f,0.20530000061988840087678909185342490673065185546875f,-0.041499986708164016135924612171947956085205078125f,-0.2576000057697296341530091012828052043914794921875f,-0.487699978160858194087268202565610408782958984375f,0.2473000176668165295268408954143524169921875f,-0.20710000270009043532581927138380706310272216796875f,-0.92959999361038203602447538287378847599029541015625f,0.217800004494190346804316504858434200286865234375f,0.0890999988853931057519730529747903347015380859375f,-0.0762000036716461437436009873636066913604736328125f,-0.05159998800754550529745756648480892181396484375f,-0.14319999851584430672346570645458996295928955078125f,1.051499976351857323919603004469536244869232177734375f,-0.631899985758960003323636556160636246204376220703125f,-0.224399999859184073880413734514149837195873260498046875f,-0.6372999976992606008252550964243710041046142578125f,-0.662699991832673251934693325893022119998931884765625f,0.047799992237985033938230117200873792171478271484375f,-0.1208000125184656159404994468786753714084625244140625f,-0.4596999754078694433445662070880644023418426513671875f,0.1296000000178814115514569493825547397136688232421875f,-0.6562000068008899678062562088598497211933135986328125f,-0.673600013250112539253677823580801486968994140625f,0.4325999954879282682895791367627680301666259765625f,0.1022000087678431867743711336515843868255615234375f,-0.60449996817111983915538075962103903293609619140625f,-0.01680000349581245444596788729541003704071044921875f,-0.334400007477402727573689844575710594654083251953125f,-0.766000000365078204112023740890435874462127685546875f,-0.14739997158050499592718551866710186004638671875f,0.84620000486373925241423421539366245269775390625f,0.2570999906301505433248166809789836406707763671875f,-0.534600026243925441349347238428890705108642578125f,-0.269200018662214546338873333297669887542724609375f,-0.3676999999165531107792048715054988861083984375f,-0.038200004029274037264940488967113196849822998046875f,-0.520200010943412838315680346568115055561065673828125f,-0.42870001492202280957144466810859739780426025390625f,-0.6546000249773265000641231381450779736042022705078125f,0.055200005802512108754598330051521770656108856201171875f,0.604300019104778929346366567187942564487457275390625f,0.03359999887049180955500560230575501918792724609375f,0.569599989214539359494438031106255948543548583984375f,0.834800001387297374577656228211708366870880126953125f,0.3226000035718084024694007894140668213367462158203125f,0.0956000151351094895613158541891607455909252166748046875f,-0.2150999866776166502546629999415017664432525634765625f,0.69240002388358146134805792826227843761444091796875f,-0.85019999499917009444516224903054535388946533203125f,-1.737800023567675822278033592738211154937744140625f,0.4330000006854533012301544658839702606201171875f,0.5742000019729136539581304532475769519805908203125f,-0.4386999573290353282573050819337368011474609375f,0.3338000257581474894408302134252153337001800537109375f,-0.3682000031352041791166129769408144056797027587890625f,-0.08030005482286173279504737365641631186008453369140625f,0.22140000403523441008246663841418921947479248046875f,0.739800002318620641972302109934389591217041015625f,-0.18169996524155163086788888904266059398651123046875f,0.3296000040411950493535186978988349437713623046875f,0.431400011523067894447791559286997653543949127197265625f,-0.570899981370568188054903657757677137851715087890625f,-0.055900008006393608894768476602621376514434814453125f,-0.0026000091969967942251429349198588170111179351806640625f,-0.75250000044703480028829289949499070644378662109375f,-0.33769997234940518637813511304557323455810546875f,-0.3143999993562698325177962033194489777088165283203125f,-0.6090999796628953077970436424948275089263916015625f,0.6566999848365782099790521897375583648681640625f,0.282499998360872250913189418497495353221893310546875f,-0.98060001569390298215012080618180334568023681640625f,-0.33019998240768888564389271778054535388946533203125f,-0.123399995601177148163429819760494865477085113525390625f,0.171599951648712334417723468504846096038818359375f,0.4118000096797942433113348670303821563720703125f,0.4992000174701214376682401052676141262054443359375f,-0.7033000294685365361146978102624416351318359375f,-1.0099000061511986814366537146270275115966796875f,-0.28640000180006008889677104889415204524993896484375f,-0.5231000077962875138837262056767940521240234375f,-0.4656999827206131925549925654195249080657958984375f,-0.30530000166296955566025417283526621758937835693359375f,0.4869000129535792087409618034143932163715362548828125f,-0.87010000893622674311700393445789813995361328125f,-0.239199997465312452771257767381030134856700897216796875f,1.05170000554919251811725189327262341976165771484375f,0.52929995000064433696707055787555873394012451171875f,0.279299981367587990543910336782573722302913665771484375f,-0.14509998667761692114908100847969762980937957763671875f,0.5238999995462592096373555250465869903564453125f,0.1900000046379864138135218354364042170345783233642578125f,-1.873500020146369848816902958787977695465087890625f,0.258500066637992631513043306767940521240234375f,-0.13059998261332506075405035517178475856781005859375f,-0.44529998512268065269381622783839702606201171875f,1.009699964511394654209652799181640148162841796875f,0.5816999936282634120487955442513339221477508544921875f,0.0926999726474286855903983450843952596187591552734375f,0.21970003149211425608200443093664944171905517578125f,-0.14349999649077649355177754841861315071582794189453125f,-0.2238000007987022144106958876363933086395263671875f,0.8395999684274197960576202603988349437713623046875f,0.59659999858736989519769622347666881978511810302734375f,-1.1205999726295470253489838796667754650115966796875f,-0.8147000446796415218386755441315472126007080078125f,-0.13699999514222138685681784409098327159881591796875f,0.52580000607371335519246713374741375446319580078125f,-0.09649998968839579305267761810682713985443115234375f,-0.6806000103294849168378277681767940521240234375f,0.0249999873340129852294921875f,0.2493000102758404779024203890003263950347900390625f,-0.340000001341104418628447092487476766109466552734375f,0.622200012344122033169924179674126207828521728515625f,0.9228000196933745513661051518283784389495849609375f,-0.5574000147193671761414179854909889400005340576171875f,0.236399999341368627359116771913249976933002471923828125f,0.6077000115767121624088531461893580853939056396484375f,0.186400012081861632395884953439235687255859375f,-0.293200003582239077104532043449580669403076171875f,-0.27520002167224877354101408855058252811431884765625f,-0.75439999324679352099565221578814089298248291015625f,0.48960000508427636844999142340384423732757568359375f,1.484799972963333658526607905514538288116455078125f,0.038200001347064915790951999952085316181182861328125f,0.393799990069866101549678205628879368305206298828125f,-0.02129998381137898633141958271153271198272705078125f,0.1520000058710575530085407081060111522674560546875f,0.14000000521540645337381647550500929355621337890625f,-0.215999998763203304719127118005417287349700927734375f,-0.42760001858472829194823816578718833625316619873046875f,-0.49939999488592146104082303281757049262523651123046875f,-0.04579999288618541442019704845733940601348876953125f,-0.044400002485513567762609454803168773651123046875f,-0.584399994140863299207921954803168773651123046875f,0.2413999937534327955290791578590869903564453125f,0.3261999963700770166497022728435695171356201171875f,-0.5005999900639057642592888441868126392364501953125f,-0.12890002385377830051993441884405910968780517578125f,-0.323200006115436533349338787957094609737396240234375f,-0.485400007879734030069585060118697583675384521484375f,0.138799983215331934616187936626374721527099609375f,0.149400001330673770638668429455719888210296630859375f,0.3330000066831708682713042435352690517902374267578125f,-0.4208999554984276247893149047740735113620758056640625f,0.4778000211834909638497492778697051107883453369140625f,-0.4198000018715857262208146494231186807155609130859375f,-0.53190003020167342384638686780817806720733642578125f,0.64860000997185718318860381259582936763763427734375f,-0.3297999927818775489640756859444081783294677734375f,-0.2531000286579132563247185316868126392364501953125f,0.16709999718666068702077609486877918243408203125f,0.33390001208186159686874816543422639369964599609375f,0.009299994778633191572225769050419330596923828125f,-0.9604999930858610923678497783839702606201171875f,-0.216299998563528106387821026146411895751953125f,0.9877000136256217643904165015555918216705322265625f,0.025699983465671039084554649889469146728515625f,0.194200003463029791106464472250081598758697509765625f,0.5154000005409120621635565839824266731739044189453125f,-0.5972000124558807243602132075466215610504150390625f,-0.087499999515712245656828827122808434069156646728515625f,-0.2423000093221663320264269714243710041046142578125f,0.04010002126693734680884517729282379150390625f,-0.106899998164176945891767900320701301097869873046875f,1.552699975776672403071643202565610408782958984375f,-0.6296999952077868556443718262016773223876953125f,-0.0976999906778335258650258765555918216705322265625f,-0.056600002014636885405707289464771747589111328125f,0.33639997676610988719403394497931003570556640625f,0.283599992686510038453207016573287546634674072265625f,-0.23100000971555711970495394780300557613372802734375f,0.3050000016391276602689686114899814128875732421875f,0.17100000509619717181664100280613638460636138916015625f,-0.201099985459447072599914463353343307971954345703125f,-0.9898999774664645201482926495373249053955078125f,-0.45919999340474604299089378400822170078754425048828125f,0.3669000375032425154131487943232059478759765625f,-0.4630999750137334558530710637569427490234375f,-0.447500001490116261493312777020037174224853515625f,0.0405000044107435286377949523739516735076904296875f,0.10950000751018507116896216757595539093017578125f,-0.22930000029206265566017464152537286281585693359375f,0.0898000018715858772111459984444081783294677734375f,-0.8105999794840812455731793306767940521240234375f,-0.445900000816583652607505428022705018520355224609375f,-0.525599977025389630824747655424289405345916748046875f,0.421799994368850850179342160117812454700469970703125f,0.33180000282526000976446312051848508417606353759765625f,-0.43080000935196860467613078071735799312591552734375f,0.35540002978444107384348171763122081756591796875f,-0.0719999976754188697469771796022541821002960205078125f,-1.32209998347759238868093234486877918243408203125f,-0.03449998903274575923205702565610408782958984375f,-0.489200023579597687017894713790155947208404541015625f,0.345600003474950678850063923164270818233489990234375f,0.1278000383943320716895186706096865236759185791015625f,0.0448000026911496984638461071881465613842010498046875f,-0.1093999936938285788556868283194489777088165283203125f,0.26079997213631933572486332195694558322429656982421875f,0.1594000044226646650713519193232059478759765625f,0.04740000227689744605186206172220408916473388671875f,-0.48349999652802910787841028650291264057159423828125f,-0.88960002594590203983671017340384423732757568359375f,-0.02699999931454666324270874611102044582366943359375f,1.06799999725818661744369819643907248973846435546875f,0.5703999815046785215599811635911464691162109375f,-0.5281999842107296672111260704696178436279296875f,-1.1715999495625499804418723215349018573760986328125f,-0.6454000159263610925108878291212022304534912109375f,0.2662000012874603527279759873636066913604736328125f,0.3073000349521635143901221454143524169921875f,0.5330000112652779620248111314140260219573974609375f,0.5638000228524209234137742896564304828643798828125f,-0.280299967539310301845034700818359851837158203125f,0.872000006616115541646649944595992565155029296875f,-0.056999979197978944966962444595992565155029296875f,-1.29639997587203925633048129384405910968780517578125f,0.656600012147426692621365873492322862148284912109375f,-0.122200001317262607614111402654089033603668212890625f,-1.0332999790281063301478070570738054811954498291015625f,0.2846000141739846167610039628925733268260955810546875f,0.1049999955296516684910557160037569701671600341796875f,-0.0317000124037267738685841322876513004302978515625f,0.737200010704994301846681992174126207828521728515625f,0.118800026726722851577733308658935129642486572265625f,-0.61939997581243488866675761528313159942626953125f,-0.0096000008001922709777176123679964803159236907958984375f,-0.29419999202638857926928039887570776045322418212890625f,-0.2047999988354740619200811124756000936031341552734375f,-0.453600001925229978638753891573287546634674072265625f,0.823600009375810682143992380588315427303314208984375f,0.890400018608570320566286682151257991790771484375f,0.3203999955862759652092108808574266731739044189453125f,-0.3429999994933605744762417089077644050121307373046875f,-0.08339999608546555531773947222973220050334930419921875f,-0.8950000345706943249979303800500929355621337890625f,-0.38679998118132363771337622893042862415313720703125f,0.2205999893188472782412645756267011165618896484375f,0.47959998109340684635526486090384423732757568359375f,0.3949999876320362535153662975062616169452667236328125f,0.79320000462532025409245761693455278873443603515625f,0.191399993902444887083902358426712453365325927734375f,0.79340000524967912998164365490083582699298858642578125f,0.675200015786290297370442203828133642673492431640625f,-0.412399992889165734055723078199662268161773681640625f,0.435600017035007613941388626699335873126983642578125f,-0.490699999111890772240940350457094609737396240234375f,-0.1397999982953070929880823314306326210498809814453125f,-0.467499995678663127218754880232154391705989837646484375f,-0.252100001955032126943478942848742008209228515625f,-0.2434000009655949980924560804851353168487548828125f,-0.0752999882519242991207875093095935881137847900390625f,0.0465999872624873745508011779747903347015380859375f,-0.232799991679191631277490159845910966396331787109375f,0.07739999952018250528595899595529772341251373291015625f,-0.16920002357959784689001025981269776821136474609375f,-0.507200000572204157123223922098986804485321044921875f,-0.24840000990629185206870488400454632937908172607421875f,-0.45160000827312529736445867456495761871337890625f,0.8039999572038656339145745732821524143218994140625f,-0.1956000224739309345522997318767011165618896484375f,-0.7525000274181365966796875f,0.147000005424022361921743140555918216705322265625f,-0.58450002516806165431262343190610408782958984375f,-0.5120000086650253745546024219947867095470428466796875f,0.1997999936014414057439125826931558549404144287109375f,-0.3538000069826841009756890343851409852504730224609375f,0.6614999983683222950503477477468550205230712890625f,0.41519997152984189359159472587634809315204620361328125f,0.506499978289008279119798316969536244869232177734375f,0.218299989831447049937196425162255764007568359375f,0.5619999760687353074217753601260483264923095703125f,0.045299962063133580869589422945864498615264892578125f,-0.2343999978661539529412038973532617092132568359375f,0.34899997881054911630371861974708735942840576171875f,-0.1190000048503276897093883235356770455837249755859375f,-0.03049999943748116226771571746212430298328399658203125f,-0.00060000001043081230278630755492486059665679931640625f,-0.03169999968186019978499956550876959227025508880615234375f,-0.4693000069975852284187567420303821563720703125f,0.0298000036224721487343458647956140339374542236328125f,-0.337600003236532109074374830015585757791996002197265625f,-0.79550001005455828195778167355456389486789703369140625f,-0.448000002473592562335369393622386269271373748779296875f,-0.68039999119043326203382093808613717555999755859375f,-1.11469999416470511022225764463655650615692138671875f,-0.706999999538064027859718407853506505489349365234375f,0.0560000018179416247932067562942393124103546142578125f,0.4971000047862528248288072063587605953216552734375f,0.02140000105500217131293538841418921947479248046875f,0.10240001509189600170657286071218550205230712890625f,0.07540001682043051545178968808613717555999755859375f,-0.396799994629621455288770448532886803150177001953125f,-0.03159999687373638010967624722979962825775146484375f,0.1281000034376977847472289795405231416225433349609375f,0.14200000948458912919392105322913266718387603759765625f,-0.5879999993443487671385128123802132904529571533203125f,-0.6262999941304325801638697157613933086395263671875f,-0.2639999922215938710223781527020037174224853515625f,-0.402200011897086984191673764144070446491241455078125f,-0.85970001964569053853892910410650074481964111328125f,-0.203700002032518145966832889826036989688873291015625f,0.066800003868341395474317323532886803150177001953125f,0.2895000064671038320085472150822170078754425048828125f,-0.18320000164508820628128660246147774159908294677734375f,0.1995999994218351503150188364088535308837890625f,1.123899989879131311454329988919198513031005859375f,0.6331999943435189237561644404195249080657958984375f,-0.128199987936020054490882102982141077518463134765625f,-0.356499968826771151952925720252096652984619140625f,0.0769000181317327502483749412931501865386962890625f,0.114799997103214312943464392446912825107574462890625f,0.75529999123215674217135529033839702606201171875f,0.11219999543130398933499236591160297393798828125f,0.180199984866380447101619211025536060333251953125f,1.0454000016212461332543171010911464691162109375f,0.493699995028972438149139634333550930023193359375f,0.10419999675750712953004040173254907131195068359375f,-0.263399987107515443085503648035228252410888671875f,0.2346999999016521343264685128815472126007080078125f,0.6611999895155431516968747018836438655853271484375f,1.2651999684751036312491123680956661701202392578125f,0.71999997936189163993958572973497211933135986328125f,-0.404799999040365232616522916941903531551361083984375f,0.4062000055156647082554854932823218405246734619140625f,0.1402999927036461258467170409858226776123046875f,0.298999992333352559992221131324185989797115325927734375f,0.25389999477788804949085488260607235133647918701171875f,0.698299985957145707970994408242404460906982421875f,0.2058000109910960162551418761722743511199951171875f,0.78669999251067590506636406644247472286224365234375f,0.60930000476241108486874509253539144992828369140625f,-0.0921999996781348585273008211515843868255615234375f,0.4208999988794326352348207365139387547969818115234375f,0.5693999993562695038917809142731130123138427734375f,0.63799997341632863623317462042905390262603759765625f,0.4317999694466589932773104010266251862049102783203125f,0.01949998124688845368979173144907690584659576416015625f,-0.267399994230270332540300159962498582899570465087890625f,0.0050999924704431265354287461377680301666259765625f,0.5478999843731522378931231287424452602863311767578125f,-0.3657999827533962022840796635136939585208892822265625f,0.5959000077083704827174415186163969337940216064453125f,-0.2341000130414965241243407945148646831512451171875f,-0.090799995344877260095017845742404460906982421875f,-0.02869999134093514925325507647357881069183349609375f,0.6093000113189219746345770545303821563720703125f,0.247399988865852460406813406734727323055267333984375f,0.443699993613362320132864624611102044582366943359375f,0.301299991559982771605064044706523418426513671875f,-0.8429999534487730983300934894941747188568115234375f,0.080500010892748274926589147071354091167449951171875f,-0.4273000099182127087260596454143524169921875f,-0.18479999993443474437526674591936171054840087890625f,-0.2815000009760260102353868205682374536991119384765625f,0.743500002562999640076668583787977695465087890625f,-0.19659999217987067510193810448981821537017822265625f,0.4894999996870754710442952273297123610973358154296875f,0.4440000361800198191986055462621152400970458984375f,-0.67099998900294277603961745626293122768402099609375f,0.2066000416144733176082581849186681210994720458984375f,-0.96159997757673210116990958340466022491455078125f,-0.0618000026762484111486628535203635692596435546875f,-0.752999975688755096570048408466391265392303466796875f,-0.4591999890089031310935752117075026035308837890625f,-0.29679997630119370199963668710552155971527099609375f,-0.6103999906688930554565786223975010216236114501953125f,0.57410000112056724930198470246978104114532470703125f,-0.04529999860823163970735549810342490673065185546875f,0.59660000186562545110291466698981821537017822265625f,0.81159999247789382792461765347979962825775146484375f,0.691899993693828374574650297290645539760589599609375f,0.154200002270937108761472700280137360095977783203125f,1.03739999423027029479271732270717620849609375f,0.467500001341104365337741910479962825775146484375f,0.30159999366998679892759582799044437706470489501953125f,0.6062999797135593116337304309126920998096466064453125f,0.18119999063313019238563583712675608694553375244140625f,0.132600000256299921375102712772786617279052734375f,0.2468000010371207419979100450291298329830169677734375f,0.15190000308156015318417075832257978618144989013671875f,0.1187999855995176545775393606163561344146728515625f,0.7733999982833861253084251075051724910736083984375f,0.48139999464750271584989604889415204524993896484375f,-0.092600004875659891467876150272786617279052734375f,0.07679999462962161516088599455542862415313720703125f,-0.170500011935830197984387268661521375179290771484375f,-0.22240000272393223301747866571531631052494049072265625f,-0.648100005523860378531253445544280111789703369140625f,0.00700000900030151029795888462103903293609619140625f,0.71179999805688876080012050806544721126556396484375f,1.14359997182488459799287738860584795475006103515625f,0.611299989250302377996604263898916542530059814453125f,-0.358400008863210661758103015017695724964141845703125f,-0.518100005710124822400075572659261524677276611328125f,-0.0353999981939792096596875126124359667301177978515625f,-0.6569999826252459484976498060859739780426025390625f,-0.891599972808361229681395343504846096038818359375f,-0.6320999875009061241826202603988349437713623046875f,-0.126399980491399777093874945421703159809112548828125f,-0.89149996823072452656333553022705018520355224609375f,-0.366599996054172816428717851522378623485565185546875f,-0.016799992841482147554188486537896096706390380859375f,-0.411499993562698396232235609204508364200592041015625f,-0.0213999919652938519476492729154415428638458251953125f,-0.72800000157952293733387705287896096706390380859375f,-1.11570000164508797979578957892954349517822265625f,-0.55019999693632115622676792554557323455810546875f,-0.27039998865723635645963440765626728534698486328125f,-1.1923999633848669787994367652572691440582275390625f,-0.62819999218285094144675895222462713718414306640625f,-0.04320000239014643028667705948464572429656982421875f,0.733100016476214033644964729319326579570770263671875f,-0.97700000095367389718603590154089033603668212890625f,-0.222199982839823118041522320709191262722015380859375f,-0.96200000076740999066515769300167448818683624267578125f,0.521199983406066991165062063373625278472900390625f,-0.67459996812939682087062465143389999866485595703125f,0.2729999978914852132305668419576250016689300537109375f,0.072100001731514762326469281106255948543548583984375f,0.04339999962449070380898774601519107818603515625f,-0.056700005027651823041878742515109479427337646484375f,-0.114600002849102100555001015891321003437042236328125f,-0.0670000025928019216081565900822170078754425048828125f,-0.139600002960860669620757334996596910059452056884765625f,0.1670000283718113820441431016661226749420166015625f,-0.66639998391866672733385712490417063236236572265625f,0.0258000395268205284793339160387404263019561767578125f,0.5574000213503840228668195777572691440582275390625f,0.05380000571608523074473851011134684085845947265625f,0.4500000144168734284022548308712430298328399658203125f,-0.5815000253394249085658884723670780658721923828125f,-0.15059999863207329529046774041489697992801666259765625f,-0.35710000407844777470245389849878847599029541015625f,-0.5570000125765801612942595966160297393798828125f,-0.04499999031424550821611774154007434844970703125f,-0.681600018815696540031012773397378623485565185546875f,-0.0731000048160552751141949556767940521240234375f,-0.3417999888181686429788896930404007434844970703125f,0.0537000178277493578349321978748776018619537353515625f,0.8693999829649925459307269193232059478759765625f,0.078600007289647777497521019540727138519287109375f,0.89399998793750956593839873676188290119171142578125f,-0.515699993002414203147054649889469146728515625f,0.54419999258518192419842307572253048419952392578125f,-0.2897000087678434088189760586828924715518951416015625f,-0.55170001761913312066099024377763271331787109375f,0.125799995493888872033494408242404460906982421875f,-0.3905000142455101741489897904102690517902374267578125f,0.5292999852418898143469050410203635692596435546875f,-0.06579999012947101988402209826745092868804931640625f,0.6585000047981737214541908542742021381855010986328125f,-0.2843999767065046313518905662931501865386962890625f,0.74999996513128319719498904305510222911834716796875f,-0.0487999993085856953456413975800387561321258544921875f,-0.25000002294778855826962171704508364200592041015625f,0.22440001217126859245354353333823382854461669921875f,0.03920001583099352870931397774256765842437744140625f,-0.45079996926784549060585050028748810291290283203125f,0.170800017547607208001636536209844052791595458984375f,0.453999987676739547026727450429461896419525146484375f,0.5252999872460961672260282284696586430072784423828125f,0.64779999857097869675470747097278945147991180419921875f,0.08720000712871556203253931016661226749420166015625f,0.7803000067293641706100970623083412647247314453125f,0.26560000667870031776374162291176617145538330078125f,0.246799998950958165977453973027877509593963623046875f,0.12709999897480006580963163287378847599029541015625f,0.268600004085898458328074411838315427303314208984375f,0.395399990296363856856487473123706877231597900390625f,1.030799983125925134430644902749918401241302490234375f,0.5018999878078698184680206395569257438182830810546875f,-0.3328000083684923282589807058684527873992919921875f,-0.257499993219971745617158376262523233890533447265625f,-0.3393999965995548873110010390519164502620697021484375f,-0.159600000464916291775807621888816356658935546875f,-0.033399996495246941918821903527714312076568603515625f,-0.3646000000178812872064781913650222122669219970703125f,0.31899999475479123844934292719699442386627197265625f,0.56719998805522919838040252216160297393798828125f,0.50530000322759160980012893560342490673065185546875f,0.451999990671873075331888003347557969391345977783203125f,1.0738999786101282030159609348629601299762725830078125f,0.752400005406141492159122208249755203723907470703125f,-0.2262000138044355512789707063348032534122467041015625f,-0.54490002783238844497049058190896175801753997802734375f,0.177899997597933012816184827897814102470874786376953125f,0.01740001032352456178387001273222267627716064453125f,-0.61799998906254760555611937888897955417633056640625f,-0.425100013890862715726370879565365612506866455078125f,-0.558999992817640123377032068674452602863311767578125f,-0.7766999951183795047882085782475769519805908203125f,-0.51149999557435510411096402094699442386627197265625f,-0.22240001494288463845805381424725055694580078125f,0.139299999025463971946692254277877509593963623046875f,0.2848000088751316827284654209506697952747344970703125f,-0.20120000247955316385883861585170961916446685791015625f,-0.433300005738437021829412287843297235667705535888671875f,-0.26660000298321240297383383222040720283985137939453125f,-0.3530000003874302905160220689140260219573974609375f,-0.0921999770283694619621428500977344810962677001953125f,-1.0220000146627423731615635915659368038177490234375f,-0.02240000197887415112063536071218550205230712890625f,-0.7700000090897083371288545095012523233890533447265625f,-1.077699994514882408935818602913059294223785400390625f,-0.295800001752376251484832891947007738053798675537109375f,-0.9884999862834806805267362506128847599029541015625f,0.5828999861240387048155753291212022304534912109375f,-0.44139998213052766828923267894424498081207275390625f,0.3620999924182888296542159878299571573734283447265625f,-0.688300005179643736852312940754927694797515869140625f,-0.0066000076770780236046221034484915435314178466796875f,-0.6007000041037795501353002691757865250110626220703125f,0.0210000150799753981800677138380706310272216796875f,-0.35099998453259484421096203732304275035858154296875f,-0.160799984951317309622709217364899814128875732421875f,0.7623000175923106969122500231605954468250274658203125f,0.00579998357296040989439234181190840899944305419921875f,0.687500016316771933588825049810111522674560546875f,0.8266000091671941163440351374447345733642578125f,0.23279998542070412526072686887346208095550537109375f,0.687799997267127150024634829605929553508758544921875f,-0.541400013870001028948308885446749627590179443359375f,-0.3313999953925608199511998464004136621952056884765625f,-0.545200003008544431537529817433096468448638916015625f,-0.4211000240348283796265604905784130096435546875f,-0.5973999913617966672774173275684006512165069580078125f,-0.52750000631436699816134705542935989797115325927734375f,-0.69610002330839648720939294435083866119384765625f,-0.4051999805450441360932245515868999063968658447265625f,-0.460299989853799385031152269220910966396331787109375f,-0.270599973523616199599928222596645355224609375f,0.75419999273419335850121569819748401641845703125f,-0.146400001315772687604521706816740334033966064453125f,0.6700999813318251341343056992627680301666259765625f,0.0388000098884102584406718960963189601898193359375f,0.70709999103993137481438679969869554042816162109375f,0.1495000011771916437197660343372263014316558837890625f,-0.74179999209642366597705631647841073572635650634765625f,0.13510002894103567694372713958728127181529998779296875f,0.144099999593198191405463148839771747589111328125f,0.16539999677836891844862066136556677520275115966796875f,0.194899999929964540257287808344699442386627197265625f,0.0793999913096428144854144193232059478759765625f,0.29549999755620948160839134288835339248180389404296875f,0.374799997177719834695608369656838476657867431640625f,0.4202000020399687674199640241567976772785186767578125f,0.45180001306980865916074208143982104957103729248046875f,-0.09140002102255806448738439939916133880615234375f,0.125499963134527714458954505971632897853851318359375f,-0.3013000269502399408594328633625991642475128173828125f,0.1826000215649603575229775742627680301666259765625f,0.2171000299692149582142519648186862468719482421875f,-0.32049998950958258348009621840901672840118408203125f,-0.14559999989867211667160518118180334568023681640625f,-0.141499995872378381278622327954508364200592041015625f,-0.0647999975502491221135414889431558549404144287109375f,0.151400012379884429947196622379124164581298828125f,-0.46419996576309241476110400981269776821136474609375f,-0.119099990093708196781108199502341449260711669921875f,-0.5096000001668929968445809208787977695465087890625f,-0.65159999694824222160605131648480892181396484375f,0.014000023663044203203753568232059478759765625f,-0.0248000193059443319043566589243710041046142578125f,0.58729996759891545110576771548949182033538818359375f,-0.107000021517276611149327436578460037708282470703125f,0.0449999937787650683418405606062151491641998291015625f,0.41239999953880890348756338426028378307819366455078125f,0.5159000151775778864049470939789898693561553955078125f,0.18820000224113453413110619294457137584686279296875f,0.111700003910064626921894159750081598758697509765625f,-0.2773000045537947766405295624281279742717742919921875f,0.26980000261664383032211844692938029766082763671875f,0.083599998497962957344498136080801486968994140625f,-0.3499000073432920743243812466971576213836669921875f,0.044199974703788724728070747005403973162174224853515625f,0.63909996763020782140074516064487397670745849609375f,0.171099986055493591408094289363361895084381103515625f,0.37720002113580708424933618516661226749420166015625f,0.2153000165641305585495501873083412647247314453125f,-0.00939999555647386131340681458823382854461669921875f,-0.3001999938070774032183862800593487918376922607421875f,-0.513099986934661966841986213694326579570770263671875f,-0.0748999878972769206342263714759610593318939208984375f,-0.123599993431567967139272923304815776646137237548828125f,-0.85370000564604975323845792445354163646697998046875f,-0.198599997454881549430183440563268959522247314453125f,-0.08680000178217905926203457056544721126556396484375f,-0.25489998771101252117432522936724126338958740234375f,-0.77580001140087884170526422167313285171985626220703125f,-0.134400018429755518933887969978968612849712371826171875f,-0.7234000080808993260461647878400981426239013671875f,0.839500030159950227925946819595992565155029296875f,0.20760000055432303867064547375775873661041259765625f,0.66970002057701361053432265180163085460662841796875f,0.5139000000119207101079155108891427516937255859375f,0.61959997230172181303942124941386282444000244140625f,0.3592999664291738337595916163991205394268035888671875f,0.1550999994367361356495393920340575277805328369140625f,-0.09539999715089797494016465861932374536991119384765625f,0.1615000042170286198572881630752817727625370025634765625f,-0.775999982893466722089215181767940521240234375f,0.5209999702274803468071695533581078052520751953125f,-0.58780000423341949300493070040829479694366455078125f,-0.0804000475168233919021076872013509273529052734375f,-0.59359999015331244009985312004573643207550048828125f,0.0851999926149844544198685980518348515033721923828125f,1.0975000244379042868558826739899814128875732421875f,-0.07819999330043803098533317097462713718414306640625f,0.8123000194549561125967329644481651484966278076171875f,-0.296600021218508758114040801956434734165668487548828125f,-0.7133999917268749602072830384713597595691680908203125f,-0.29199999415501931565586346550844609737396240234375f,0.27610001906156522721857982105575501918792724609375f,0.604799971920252144030882845981977880001068115234375f,0.18069998983591817331983975236653350293636322021484375f,0.204200017917155918212301912717521190643310546875f,0.82939996493458778559215716086328029632568359375f,0.11479998156875392822939829784445464611053466796875f,0.6015000196024773426728415870456956326961517333984375f,-0.3435999973803756102341822042944841086864471435546875f,0.3997000265747312486297460054629482328891754150390625f,-0.3421000108957290564148934208787977695465087890625f,0.51859998896121961564631419605575501918792724609375f,-0.3529000353723767968716629184200428426265716552734375f,-0.10670001836419107377196269226260483264923095703125f,-0.2047999927818775489640756859444081783294677734375f,0.017899997486173990779434461728669703006744384765625f,-0.7355000264272095744644275328028015792369842529296875f,0.3229999983757732007205731861176900565624237060546875f,-0.470300021071732299304812841000966727733612060546875f,-0.213700008440017796829124563373625278472900390625f,-0.554600016260147210545028428896330296993255615234375f,0.350799989980459248073429989744909107685089111328125f,0.66989997932910938516215537674725055694580078125f,0.10159998130202296806601225398480892181396484375f,0.321599994564056412826857922482304275035858154296875f,0.7039999886453152111442932437057606875896453857421875f,0.37039999268054957237694679861306212842464447021484375f,-0.11579999862313261616009185672737658023834228515625f,-0.5457999993681905692710643052123486995697021484375f,-0.55630001972317710823290326516143977642059326171875f,-0.1467999947786331471633047840441577136516571044921875f,-0.39509999019801622655023720653844065964221954345703125f,-0.290999996826052653720040552798309363424777984619140625f,0.551599988454580358876455647987313568592071533203125f,0.95769996653795264052178026759065687656402587890625f,0.483599993431568453416957709123380482196807861328125f,0.56580001635551457894734994624741375446319580078125f,0.791800012511014728033842402510344982147216796875f,0.1654999915212389538510251441039144992828369140625f,-0.380600009882450240894513626699335873126983642578125f,-0.15789999461770065636301296763122081756591796875f,0.00230000619292258789982952293939888477325439453125f,0.5011999881744386708959382303874008357524871826171875f,0.80619996280521188936063481378369033336639404296875f,0.298599983671307800392469289363361895084381103515625f,0.562800000172853476243517434340901672840118408203125f,0.65639998585581782464259958942420780658721923828125f,0.423600000509619878386047275853343307971954345703125f,-0.594800002765655477787731797434389591217041015625f,-1.1126999988734720403726896620355546474456787109375f,-0.60379999893605695859832849237136542797088623046875f,0.00940000889301284558996485429815948009490966796875f,-0.705599982613325238389734295196831226348876953125f,-0.463499995782971296875984990037977695465087890625f,-0.0036000028192997302056710395845584571361541748046875f,0.219499998867511703082300300593487918376922607421875f,0.1496000032961368741979413243825547397136688232421875f,-0.14780000881552712854727360536344349384307861328125f,0.45130000288486460391368382261134684085845947265625f,0.363300014343857835541484746499918401241302490234375f,0.214800003361701907778069653431884944438934326171875f,0.26350000308454024633419976453296840190887451171875f,0.23000000692903999066629694425500929355621337890625f,-0.2824000007867812911399596487171947956085205078125f,0.86040002941191195962034043986932374536991119384765625f,0.20730002571344385842166957445442676544189453125f,0.20019999634027474399999846355058252811431884765625f,0.0369000077754260136231323485844768583774566650390625f,-0.1231999925553797226029928424395620822906494140625f,-0.196799991500377746689309788052923977375030517578125f,-0.2271999935686588312222511376603506505489349365234375f,0.28960003622770358333582407794892787933349609375f,0.6977999924242492824077999102883040904998779296875f,0.312200009177625137368750074529089033603668212890625f,0.14270000088512890812353361980058252811431884765625f,0.2673999896109104934538436282309703528881072998046875f,0.18149999371170999484803587620262987911701202392578125f,-0.3405000118613248361043588374741375446319580078125f,-0.069000005185603896507018362171947956085205078125f,-0.46370001328289500719392890459857881069183349609375f,-0.1007999844670290912063137511722743511199951171875f,0.40779999987482984380449124728329479694366455078125f,-0.2546000168934468010917271385551430284976959228515625f,0.445700006562471440219042051467113196849822998046875f,-0.5319999918639657376928653320646844804286956787109375f,0.429500027291477071145209265523590147495269775390625f,-0.1777999993905428066653939822572283446788787841796875f,0.1359999977201222787215328935417346656322479248046875f,-0.308600008817017101847568483208306133747100830078125f,-0.71520001511573827457368679461069405078887939453125f,0.115199990081787273510371960583142936229705810546875f,-0.72960002214610575332898179112817160785198211669921875f,0.380299998831748808925112825818359851837158203125f,0.2033999941110611331396285095252096652984619140625f,0.519900005629658767958289899979718029499053955078125f,-0.32539999029636401672860301914624869823455810546875f,0.817399962788820655390509273274801671504974365234375f,-0.17880001640766829407169780097319744527339935302734375f,0.093699995923042234124977767351083457469940185546875f,-0.2621999952077864737276513551478274166584014892578125f,0.049700004073977510632431631165673024952411651611328125f,0.355699988383054499507807122427038848400115966796875f,-0.8227999894440169992293476752820424735546112060546875f,0.2527000159725549810474376499769277870655059814453125f,-0.3639000111132861814411398881929926574230194091796875f,-0.04700000482797606071727614107658155262470245361328125f,-0.3527000107944011642047144050593487918376922607421875f,0.1314999917745591773154956172220408916473388671875f,-0.2370999886557461078240294227725826203823089599609375f,0.34080002037882817234049070975743234157562255859375f,-0.03919998856186879265806055627763271331787109375f,0.21580000853240477454164647497236728668212890625f,0.45359999134540540666193919605575501918792724609375f,0.46199998307228096194876343361102044582366943359375f,-0.176400007760524335509444426861591637134552001953125f,-0.53660000559091580729642600999795831739902496337890625f,-0.58509998766481885468238033354282379150390625f,-0.2064999911785125874530422152020037174224853515625f,-0.104199989455938446525351537275128066539764404296875f,-0.3907999840199949392882672327687032520771026611328125f,-0.34809998544454556768101838315487839281558990478515625f,-0.11519999559521665322137096154619939625263214111328125f,0.16850000233948225769609052804298698902130126953125f,0.039000003397464677590278370189480483531951904296875f,0.750199993062019121481398542528040707111358642578125f,1.138099998855590655466585303656756877899169921875f,0.6376999899327753240640959120355546474456787109375f,0.06540000005662438553599713486619293689727783203125f,0.2645000004693864870120023624622263014316558837890625f,0.28050000802427532686778022252838127315044403076171875f,-0.38459999688863744093936247736564837396144866943359375f,-0.854099992664158147448461022577248513698577880859375f,-0.54450001049041762968272450962103903293609619140625f,0.096199996370077034413270666846074163913726806640625f,0.4928999935746192218033456811099313199520111083984375f,-0.11549999643862218778878059310954995453357696533203125f,0.62439998460412038383537947083823382854461669921875f,1.201799966019392318372638328582979738712310791015625f,0.6431999745994805284254880461958236992359161376953125f,-0.06280002297163012059399989084340631961822509765625f,0.66299997848272340661424095742404460906982421875f,0.386999988585710497090985882095992565155029296875f,-0.168399994409084374780150028527714312076568603515625f,-0.88509999131560324059364575077779591083526611328125f,-0.443799994614720194618939785868860781192779541015625f,0.03840000722408287714415564551018178462982177734375f,-0.249699989470839600613771835924126207828521728515625f,-0.2749999973922967466677391712437383830547332763671875f,-0.2032000035822392192130791954696178436279296875f,-0.434800000232458128124335416941903531551361083984375f,0.1654000164479019208130239348975010216236114501953125f,-0.46599999958276738798446103828609921038150787353515625f,0.40069999499544461940558903734199702739715576171875f,-1.0216000297307967770166214904747903347015380859375f,0.25320000194311109709133233991451561450958251953125f,-0.8612000352621083099080578904249705374240875244140625f,0.1596000180482857189190326607786118984222412109375f,0.77279995234012677229884502594359219074249267578125f,0.0083999823391442962616793010965920984745025634765625f,0.424100003877282194508779866737313568592071533203125f,-0.373599995517730487648577764048241078853607177734375f,0.4847000234082343705921402943204157054424285888671875f,0.16459998422264998652053691330365836620330810546875f,0.40339997339844746448989099008031189441680908203125f,0.19239997370392092346236267985659651458263397216796875f,0.39709998727738859969349505263380706310272216796875f,-0.2749999840557577623911811315338127315044403076171875f,0.314299996343254728348171056495630182325839996337890625f,-0.6770999638080592575306582148186862468719482421875f,-0.01880000839829421011017984710633754730224609375f,-0.782499982975422891939842884312383830547332763671875f,-0.17470001521259559407184269730350933969020843505859375f,-0.337999993532895948167293909136787988245487213134765625f,-0.18010000271499160273691586553468368947505950927734375f,0.2931000207975511528957213158719241619110107421875f,0.36840000357329805158457247671321965754032135009765625f,0.389700012195110190305058495141565799713134765625f,0.7516999972045415034926918451674282550811767578125f,0.144399978643656279331253244890831410884857177734375f,0.5294999858662488012583935415023006498813629150390625f,-0.3637999757647509824209919315762817859649658203125f,-0.235599979782104895775773911736905574798583984375f,-0.5213999807149172216469423801754601299762725830078125f,0.481400021171570102751502417959272861480712890625f,-0.787999974906444577982256305404007434844970703125f,0.31620004278719449075651937164366245269775390625f,-0.3057000043272974920682827359996736049652099609375f,0.576999974727630871029759873636066913604736328125f,-0.11650001479685290206589343142695724964141845703125f,0.0009000037223099166538986537489108741283416748046875f,0.48379997904300708899683058916707523167133331298828125f,0.0244999858289958183377166278660297393798828125f,-0.5524000296950344335300542297773063182830810546875f,-0.387599999511241577465625596232712268829345703125f,-0.3616000020146368232332179104560054838657379150390625f,0.17840000405758615897866548039019107818603515625f,0.2603999880611898021243177936412394046783447265625f,-0.3422000132009388462250853990553878247737884521484375f,-0.24020000041946765190203905149246565997600555419921875f,-0.274300004184991064182241871094447560608386993408203125f,-0.466400009548664229441783390939235687255859375f,-0.586999995291233034322431194595992565155029296875f,-0.463200001645088121904336730949580669403076171875f,-0.0999999947845935377443993274937383830547332763671875f,0.116000004947185431092293583787977695465087890625f,-0.095799996164441125756638939492404460906982421875f,-0.46299998027086264329454934340901672840118408203125f,-0.7707999592840675262550576007924973964691162109375f,-0.47329998513758209099933083052746951580047607421875f,0.81560000183582292976325334166176617145538330078125f,1.312099993163347022573361755348742008209228515625f,0.7718000088602305908125345013104379177093505859375f,0.03239999989271169500426594822783954441547393798828125f,0.103099996285140604879160264317761175334453582763671875f,-0.30780001365840436999832263609278015792369842529296875f,-0.2411999917507171853348069134881370700895786285400390625f,-0.7551999835439027197736550078843720257282257080078125f,-0.21119999141544110177193260824424214661121368408203125f,0.4169999876916410386229472351260483264923095703125f,0.28429996393322998216035557561554014682769775390625f,-0.281500038489699289101508838939480483531951904296875f,-0.6217999977588650750703891390003263950347900390625f,-0.83440000457167595726559738977812230587005615234375f,-0.771100012877583562698191599338315427303314208984375f,-0.70360000043511394807183023658581078052520751953125f,-0.9225999862492084702125794137828052043914794921875f,-0.67719999587833878962328526540659368038177490234375f,-0.495600004070997357530359295196831226348876953125f,-0.51529998810291299804475784185342490673065185546875f,-0.286799992394447311738758799037896096706390380859375f,0.06600000137090678009599287179298698902130126953125f,-0.05449999544024475284231812111102044582366943359375f,0.259800007906556107428741597686894237995147705078125f,-0.5076000093460084627849937533028423786163330078125f,-0.6773999966144561568626158987171947956085205078125f,-0.55579999638795829497439626720733940601348876953125f,0.563999994605779875200823880732059478759765625f,0.8211999671638015030339374789036810398101806640625f,0.54019998672902591607680733432061970233917236328125f,0.406000002712011198724439964280463755130767822265625f,-0.3594000074028969038408831693232059478759765625f,0.7000000263936818623022872998262755572795867919921875f,-0.0966000061869618509291512964409776031970977783203125f,0.63700002861395521591703072772361338138580322265625f,0.70940002290010450991530888131819665431976318359375f,0.060399989104271245565769277163781225681304931640625f,0.70300002333521882658118329345597885549068450927734375f,0.508799991708993815109351999126374721527099609375f,-0.064399995779991225930416476330719888210296630859375f,0.40759999206066110932766832775087095797061920166015625f,0.444199997055530815259771770797669887542724609375f,0.133799999159574412033180124126374721527099609375f,0.51199999435991028207126873894594609737396240234375f,-0.7509000191450123651293324655853211879730224609375f,-0.14300001215934710074861868633888661861419677734375f,-0.8713000253111122805904642518726177513599395751953125f,0.5933000235453249615602544508874416351318359375f,0.41019999373257143293614035428618080914020538330078125f,0.4577000015184282677438432074268348515033721923828125f,-0.35560000822469584047524904235615395009517669677734375f,0.13119999677985905162103108523297123610973358154296875f,-0.1612000041000545280667211045511066913604736328125f,0.1291000461220743744661376695148646831512451171875f,0.90479998592734300899564914288930594921112060546875f,0.0101000049874186981213597391615621745586395263671875f,1.11620000724792500790272242738865315914154052734375f,0.291800013405083991102628715452738106250762939453125f,1.079600004078447472721791200456209480762481689453125f,0.979800008130073507572888047434389591217041015625f,0.07479999561309824684940394945442676544189453125f,0.99260001254975804840796627104282379150390625f,0.584500005125999422261884319595992565155029296875f,-0.230200000512599700641658273525536060333251953125f,0.598900017669797168906598017201758921146392822265625f,-0.62030001194477080161959747783839702606201171875f,-0.21459999659061423926686984486877918243408203125f,-0.499100000450015013342408565222285687923431396484375f,0.7978000125408168941021358477883040904998779296875f,0.14319997929334693509417775203473865985870361328125f,0.7902000131040816111038793678744696080684661865234375f,-0.658800012421608016666141338646411895751953125f,-0.081199985492229842520828242413699626922607421875f,-0.7040000118911269311183787067420780658721923828125f,-0.971400007015466915305523798451758921146392822265625f,0.22410000328123569346416843472979962825775146484375f,-0.65900000101327915746907137872767634689807891845703125f,-0.9869999749511482178832011413760483264923095703125f,-0.42009998766481881915524354553781449794769287109375f,-0.057000002890825296475441064103506505489349365234375f,0.0846999990820884729458839501603506505489349365234375f,-0.11010000316202639414342456802842207252979278564453125f,-0.496000008821487536891936542815528810024261474609375f,-0.73419999556541437613077505375258624553680419921875f,-0.400799999219179081677566500729881227016448974609375f,-0.177000012576579823786460110568441450595855712890625f,-0.5872000288844103632612814180902205407619476318359375f,0.07169999027550233439143312352825887501239776611328125f,-0.52560001196861261263393316767178475856781005859375f,-0.384300006252527026617826777510344982147216796875f,-0.4536000143676996998465256183408200740814208984375f,0.158800014805793754391061156638897955417633056640625f,-0.247199983882904206211605924181640148162841796875f,-0.0857999944508076595184320467524230480194091796875f,0.14240000674724573315188536071218550205230712890625f,0.648300012034177530750866935704834759235382080078125f,-0.0035999999135734039867884348495863378047943115234375f,-0.70260000785589227234595455229282379150390625f,-1.1558999911308287522615501075051724910736083984375f,-0.74830000443458555281495137023739516735076904296875f,-0.6055999945342538381964914151467382907867431640625f,-0.8572000004231927761111364816315472126007080078125f,-0.72180000364482399533017087378539144992828369140625f,0.230800003391504304772752220742404460906982421875f,0.107299995017051674750518941436894237995147705078125f,0.3836000123560430186131497976020909845829010009765625f,-0.424000027686357583434073603712022304534912109375f,-0.285800025743245811327142291702330112457275390625f,0.2886000075131658348936980473808944225311279296875f,0.26939998251795760353388686780817806720733642578125f,1.027699973541498223994494765065610408782958984375f,0.6939000084310773530660299002192914485931396484375f,-0.465000002086162655956513845012523233890533447265625f,-0.54169998245239270318052149377763271331787109375f,-0.03429999313950560235753073357045650482177734375f,0.1220000084042549559626422706060111522674560546875f,-0.039099993818998424188748685992322862148284912109375f,0.2766000093907117207692181182210333645343780517578125f,0.168799988579749982164912580628879368305206298828125f,-0.7907000179246068682914483360946178436279296875f,0.387600008824467767709620602545328438282012939453125f,-0.165800004732608474711241797194816172122955322265625f,0.520200022752583368657042228733189404010772705078125f,0.4075000099837779998779296875f,0.028399997538328225488157841027714312076568603515625f,0.32410000562816865254944787011481821537017822265625f,0.441400014913082117118392488919198513031005859375f,-0.01960000672340367344759215484373271465301513671875f,0.5102000222682956387387775976094417273998260498046875f,-0.6766000122964381802148636779747903347015380859375f,-0.182599981480837225689128899830393493175506591796875f,-0.4467999889299274496323732819291763007640838623046875f,1.104300030541420341023695073090493679046630859375f,-0.102599987590313190821689204312860965728759765625f,1.00350003385543828926529386080801486968994140625f,-0.5180000070929526856389202293939888477325439453125f,0.4243999883294105046616095933131873607635498046875f,-0.40800002147257341533759245066903531551361083984375f,0.7205000112652779620248111314140260219573974609375f,0.024999998509883880615234375f,0.4796999959528445156564657736453227698802947998046875f,0.7433000285744668644838384352624416351318359375f,0.237199989694357071812191861681640148162841796875f,0.83070002790838515238647232763469219207763671875f,0.973199992108344957841836730949580669403076171875f,0.2604000072836871737536057480610907077789306640625f,0.965599992746114299535520331119187176227569580078125f,-0.8749000350594524633152104797773063182830810546875f,-0.17140000060796722891609533689916133880615234375f,-0.7453000214070082396489169695996679365634918212890625f,-0.13020001690387772441681590862572193145751953125f,-0.923999987304210179672736558131873607635498046875f,-0.00979996789693782233143792836926877498626708984375f,-0.06670002506971428601900697685778141021728515625f,-0.7855999945342535539793971111066639423370361328125f,-0.212499998323619099238612761837430298328399658203125f,0.081900007104874106289571500383317470550537109375f,-0.49759998267292981921627870178781449794769287109375f,0.1957000295475126439015411961008794605731964111328125f,-0.72530002356767653282076935283839702606201171875f,-0.15579999191761029209146727225743234157562255859375f,-0.592900013169646467048323756898753345012664794921875f,0.481800020706653953084241948090493679046630859375f,-0.2249999976158143233373465363911236636340618133544921875f,-0.1647999929308892053114732334506697952747344970703125f,-0.59909998264312758209371168049983680248260498046875f,-0.073599988886714040603465036838315427303314208984375f,0.322999990403652237347387199406512081623077392578125f,0.6570999801993371303154845008975826203823089599609375f,0.61609999715685848098445376308518461883068084716796875f,-0.2551999893367291605272839660756289958953857421875f,0.03270002226829493707782603451050817966461181640625f,-0.296599983090162222509889033972285687923431396484375f,-0.3855999879777431971206169691868126392364501953125f,-0.65669997231960319794552560779266059398651123046875f,-0.344199989604950129518101675785146653652191162109375f,0.2034000124767423312022174286539666354656219482421875f,-0.16119999374374749390170791230048052966594696044921875f,-0.198300002069026259921002974806469865143299102783203125f,-0.4011999843001365917416478623636066913604736328125f,-1.30639996752738962726425597793422639369964599609375f,-0.815800003540516183164754693279974162578582763671875f,0.4287999802350999090094774146564304828643798828125f,1.288199964690208698669948716997168958187103271484375f,0.8095999830305575795108552483725361526012420654296875f,-0.26740000123381602037397897220216691493988037109375f,-0.4542000065922735529966303147375583648681640625f,-0.072899997597932753023997065611183643341064453125f,-0.29119999338984481340730781084857881069183349609375f,-0.335899992099404354206626521772705018520355224609375f,-0.3683999990284443359911392690264619886875152587890625f,-0.3484000005185603487944945300114341080188751220703125f,-0.70910000134408457395096547770663164556026458740234375f,-0.6012000062048434845252131708548404276371002197265625f,-0.4431999928534029464799459674395620822906494140625f,-0.3486999933898449199887181748636066913604736328125f,-0.2393999914586544264238909818232059478759765625f,0.221600012296438109160590101964771747589111328125f,0.208600021594762541354839413543231785297393798828125f,-0.2718000048369171306461566928192041814327239990234375f,0.23099999585747721386752573380363173782825469970703125f,0.67799998936057093157359076940338127315044403076171875f,0.229099994713068022544888435731991194188594818115234375f,-0.483399997091293442963433335535228252410888671875f,-0.759499980092048776469937365618534386157989501953125f,-0.4758999933660030823290298940264619886875152587890625f,-0.2893999902665616019703520578332245349884033203125f,0.03030000311583280758753744521527551114559173583984375f,-0.48170000912547106253214224125258624553680419921875f,0.02459999599456796914864753489382565021514892578125f,-0.29130000258684152214527784963138401508331298828125f,-0.592900009667873373331303810118697583675384521484375f,-0.4935999834477902137308547025895677506923675537109375f,-0.597499993108213001136164166382513940334320068359375f,0.98809998663663822782154966262169182300567626953125f,-0.018600011014938022668729900033213198184967041015625f,0.82289998373985284718656885161180980503559112548828125f,0.2670999983787538667456828989088535308837890625f,0.0522000005722045301581601961515843868255615234375f,0.3458999999225138566316672950051724910736083984375f,-0.2878000019423663236040056290221400558948516845703125f,0.6093999859452245981827900322969071567058563232421875f,-0.24040002432689089317818798008374869823455810546875f,0.083600027704239465720092994160950183868408203125f,0.825199984419345611286189523525536060333251953125f,0.277600002118945088369628138025291264057159423828125f,-0.15059999080896346868030377663671970367431640625f,-0.76999996662139924552548109204508364200592041015625f,-0.332999973416328476361059074406512081623077392578125f,-0.0808000178456307338592523592524230480194091796875f,-0.17539998806118983765145458164624869823455810546875f,0.0269999993517993086555861736997030675411224365234375f,0.6166999974280595875342214640113525092601776123046875f,0.12940000263452511308770453979377634823322296142578125f,0.574299994555115489447416621260344982147216796875f,0.5367000097960232807281499844975769519805908203125f,0.4517999889671802105084452705341391265392303466796875f,0.5622999994874000595501684074406512081623077392578125f,0.663099976354837306047329548164270818233489990234375f,-0.2613999830245974376197182209580205380916595458984375f,0.634699992525577183499763123109005391597747802734375f,0.3522000288844111537400749512016773223876953125f,0.61919998036623002946043925476260483264923095703125f,0.225399992233514812056682785623706877231597900390625f,0.229400003379583328211310799815692007541656494140625f,-0.1461999948799610482552679968648590147495269775390625f,0.0708000007465481306923038573586381971836090087890625f,0.502100009107589784207448246888816356658935546875f,0.10279999212622659143789860536344349384307861328125f,0.55990000976473108806885647936724126338958740234375f,0.1080000032186507752385296043939888477325439453125f,0.02559998089969173218349851595121435821056365966796875f,-0.48300000232458106808053344138897955417633056640625f,-0.827499999403953356846841415972448885440826416015625f,-0.6813000107824802586975465601426549255847930908203125f,0.807600003534555188622334753745235502719879150390625f,1.1972000078737732309264174546115100383758544921875f,0.59560000295341009035610113642178475856781005859375f,-0.20619999793469900861708765660296194255352020263671875f,-0.400899997091293258666411247759242542088031768798828125f,-0.44330000715404749289660912836552597582340240478515625f,0.527800003004073925438888181815855205059051513671875f,1.1442000083804126919773125337087549269199371337890625f,0.75810001204311838751692675941740162670612335205078125f,-0.25940000308156008212989718231256119906902313232421875f,-0.619400007887184411714542875415645539760589599609375f,-0.0688999913692472099047137135130469687283039093017578125f,-0.4481999916613101930806806194595992565155029296875f,-0.8645999821364880322249746313900686800479888916015625f,-0.50289999357461934170743234062683768570423126220703125f,-0.14279999495744688520204590531648136675357818603515625f,0.16449999864399433846529063885100185871124267578125f,0.017800002184510166358677452080883085727691650390625f,0.085399998193979254068608497618697583675384521484375f,0.199999995529651641845703125f,0.1307000019431114878898370079696178436279296875f,0.5817999944806100387495462200604379177093505859375f,1.110399971371889282778511187643744051456451416015625f,0.6274999930709601603240344047662802040576934814453125f,0.173200002092122939956198024447076022624969482421875f,0.17970000474452962890836715814657509326934814453125f,-0.080000001490116101621197230997495353221893310546875f,-0.091600003504753058081178096472285687923431396484375f,0.072299994793534327897077673696912825107574462890625f,0.1796000017315149133168006301275454461574554443359375f,0.1092000077843664485044428147375583648681640625f,0.0050000122189519657922573969699442386627197265625f,-0.32779999212622623616653072531335055828094482421875f,0.5219999884366990361428406686172820627689361572265625f,1.169199971947074256917176171555183827877044677734375f,0.6069999820291995806798013290972448885440826416015625f,-0.203199997621774652856174725457094609737396240234375f,-0.494499994099140138814618694595992565155029296875f,-0.2206000000476837641372185316868126392364501953125f,-0.16199999946355791990981742856092751026153564453125f,0.3116000020518898683263842031010426580905914306640625f,0.731300021362304875083282240666449069976806640625f,0.38559999662041644086230007815174758434295654296875f,0.751100012467801558813107476453296840190887451171875f,-0.6239999879002571248065578402020037174224853515625f,0.1039999804496769542083711712621152400970458984375f,-0.7552000102728602115575995412655174732208251953125f,0.48710001439750205687317929914570413529872894287109375f,0.373199994120001654618334896440501324832439422607421875f,0.46610000129193063234112059944891370832920074462890625f,-0.46160001050829890800741850398480892181396484375f,-0.510999995857476907445970937260426580905914306640625f,-0.57900000485032787622685646056197583675384521484375f,-0.349200026969612054017488844692707061767578125f,-0.11860000006258475391263118581264279782772064208984375f,-0.16940000703036783846755497506819665431976318359375f,0.3247999975502491309953256859444081783294677734375f,0.222199995356798130075048902654089033603668212890625f,0.425399997188150802429618124733678996562957763671875f,0.475099982300400736079382113530300557613372802734375f,-0.2885999795734885964293425786308944225311279296875f,0.38109999313354447902923993751755915582180023193359375f,-0.0654000063896180705569349811412394046783447265625f,-0.09740000033974638427025638520717620849609375f,-0.088000003181397890017478857771493494510650634765625f,-0.3986000163793566031245063641108572483062744140625f,-0.2525999880373479555828453158028423786163330078125f,-0.5324000133410098367647833583760075271129608154296875f,0.08630001182556146233082472463138401508331298828125f,0.310199983823299607621493123588152229785919189453125f,0.02309999445974841858486570345121435821056365966796875f,-0.0167000094980001012601178445038385689258575439453125f,-0.38939998415708554357905768483760766685009002685546875f,-0.0086999927937985443993085254987818188965320587158203125f,0.340200000214576192547610844485461711883544921875f,0.5815999644637113163980757235549390316009521484375f,0.255799975153804037830695961019955575466156005859375f,-0.1695999875754115038262170855887234210968017578125f,-0.3347999790728095614866788309882394969463348388671875f,-0.3475999875903130753584946432965807616710662841796875f,-0.01829999758005129706361913122236728668212890625f,0.12219999997615804687711715814657509326934814453125f,0.0725000028312206001857020964962430298328399658203125f,1.1675000059604645485933360760100185871124267578125f,-0.5805000146180390174777130596339702606201171875f,0.490200002598762285543898542528040707111358642578125f,1.2886000128030772327036856950144283473491668701171875f,0.83370001656115044763595278709544800221920013427734375f,-0.248800000649690655762924507143907248973846435546875f,-0.09099999600648889241938377381302416324615478515625f,0.071000012770295217734428661060519516468048095703125f,-0.24480000768303877833886872394941747188568115234375f,0.1235999995410441698595604975707828998565673828125f,0.16220000824630265157111352891661226749420166015625f,-0.1746000029981136236756356083787977695465087890625f,-0.095200000065565060225480920053087174892425537109375f,-0.2581000069022179044253562096855603158473968505859375f,-0.478799997445940928031404837383888661861419677734375f,-0.3172999914035201651785200738231651484966278076171875f,-0.35539999458044768676501234949682839214801788330078125f,0.2643999919056891911850470933131873607635498046875f,-0.01770000841021523996232645004056394100189208984375f,0.297399990728497432002086497959680855274200439453125f,-0.3671999998271464615839931866503320634365081787109375f,-0.223599996784329402288449273328296840190887451171875f,-0.2267999990254641051734552092966623604297637939453125f,-0.15520001675486572167983467807061970233917236328125f,0.00709999033212671548653815989382565021514892578125f,0.396600000375509242900307071977294981479644775390625f,0.2744000009953975194321174058131873607635498046875f,0.894600000613927637260758274351246654987335205078125f,0.5034000081926583991531742867664434015750885009765625f,-0.014800004553794821049450547434389591217041015625f,0.5686999921977520244809056748636066913604736328125f,-0.11300000865757464652006092364899814128875732421875f,-0.0150000102818008684835149324499070644378662109375f,0.66239996860027350322752681677229702472686767578125f,0.375699983167648188242537798942066729068756103515625f,-0.2520000110864639708552203956060111522674560546875f,-0.438300013822316980593996049719862639904022216796875f,0.0936000130265954854991150568821467459201812744140625f,-0.07479999859333030798325125942938029766082763671875f,-0.63489999914765338218103352119214832782745361328125f,-0.451000017240643646942999112070538103580474853515625f,-0.240999993175268212297623904305510222911834716796875f,-0.6832999850630760985126244122511707246303558349609375f,-0.28729999315440657436937499369378201663494110107421875f,-0.35700000125169761844290405861102044582366943359375f,-0.042399997545778500551705292309634387493133544921875f,-0.299600012236833634915456059388816356658935546875f,-0.707799993616342160152044016285799443721771240234375f,-0.461000001482665400232008323655463755130767822265625f,0.197399979180097329134468964184634387493133544921875f,-0.4399999919533728043319342759787105023860931396484375f,0.204200000333785991468715792507282458245754241943359375f,0.364399991607665896253820392303168773651123046875f,-0.6147999875664709179545752704143524169921875f,0.3120000152587891761868377216160297393798828125f,0.589000016957521577154466285719536244869232177734375f,0.0930000000894068978851692008902318775653839111328125f,0.5072000107422471870677327387966215610504150390625f,0.96169999411255113752616807687445543706417083740234375f,-0.176999991714954452159958009360707364976406097412109375f,0.8749000030592082310931800748221576213836669921875f,-0.983499982595443356103714904747903347015380859375f,0.027000000357627840230634319595992565155029296875f,-0.822299983021616665013198144151829183101654052734375f,0.6931999979943037626384239047183655202388763427734375f,-0.19999998778104799423971371652442030608654022216796875f,0.6332000072330234985429342486895620822906494140625f,-0.3502999958515164991013079998083412647247314453125f,-0.74259998013973227415362998726777732372283935546875f,-0.2586999647796150991752028858172707259654998779296875f,0.0325999858021734922886025742627680301666259765625f,-0.49519998978376378317989292554557323455810546875f,-0.1182000047743321236026758924708701670169830322265625f,1.0613000142097472888735865126363933086395263671875f,-0.04979999620914465907617341144941747188568115234375f,0.7841000055164097926052590992185287177562713623046875f,0.41259999355077781046929885633289813995361328125f,-0.7009999508559712211308578844182193279266357421875f,0.25440001690238656095743863261304795742034912109375f,-0.353700030791760156034797546453773975372314453125f,-0.36900000339746430455534209613688290119171142578125f,-0.19530000121593449335932746180333197116851806640625f,-0.08789996883869122257237904705107212066650390625f,0.652599992656707428295703721232712268829345703125f,0.014900008609890935673547573969699442386627197265625f,-0.12530000345110892112643341533839702606201171875f,0.206799992245435770854555812547914683818817138671875f,0.014499995924532438351661767228506505489349365234375f,1.11680002160072344707941738306544721126556396484375f,-0.421199995550513239805923149106092751026153564453125f,-0.068399995824694581614267008262686431407928466796875f,0.2423000025793908829285783212981186807155609130859375f,0.28440001340061427104188851444632746279239654541015625f,0.07760000040531167542212642729282379150390625f,-0.195000001788139254443876779987476766109466552734375f,0.4155999998986719123905686501529999077320098876953125f,-0.102200006979703772458378807641565799713134765625f,0.24429997913241408014073385857045650482177734375f,-0.20630000653564906087922281585633754730224609375f,0.0249999910965563731934935276512987911701202392578125f,0.29419998098090316407393629560829140245914459228515625f,0.336099987825006107922121145747951231896877288818359375f,0.5738000021398068639655321021564304828643798828125f,0.834899986034631780285053537227213382720947265625f,0.600099995860457369190044119022786617279052734375f,-0.44739998141527193009636675924411974847316741943359375f,-1.007499957568943838026598314172588288784027099609375f,-0.60989998171329506959637001273222267627716064453125f,0.41560000466704405397422306123189628124237060546875f,0.12259997581839598979058791883289813995361328125f,0.39519998643100251456417026929557323455810546875f,-0.3114000031411645608159233233891427516937255859375f,-1.08280001373290968302853798377327620983123779296875f,-0.742900023078918447794194435118697583675384521484375f,-0.00460000225305545740184243186376988887786865234375f,-0.348300012183189178216480286209844052791595458984375f,0.0994999993145466721244929431122727692127227783203125f,0.072400007790327070011926480219699442386627197265625f,0.0492000092744826478252662127488292753696441650390625f,-0.34120000687539586703422855862299911677837371826171875f,0.38319999493956569125430178246460855007171630859375f,0.6584999846816064206223018118180334568023681640625f,0.03419998319745065629149394226260483264923095703125f,0.380799995791912149201152715249918401241302490234375f,0.6788999860048294809899971369304694235324859619140625f,0.57999999560415738120155992874060757458209991455078125f,0.12060000332593912020229254267178475856781005859375f,0.129100004100799470307947558467276394367218017578125f,-0.0622999981462954988131741629331372678279876708984375f,-0.400199983227253086592867248327820561826229095458984375f,-0.94189996092990069254113905117264948785305023193359375f,-0.6348999911010266838928828292409889400005340576171875f,0.223999998182058401852145834709517657756805419921875f,0.83689999414086335693951923531130887567996978759765625f,0.02689996862262467036686075516627170145511627197265625f,-0.538999982833862301134786321199499070644378662109375f,0.0028999910041687115835884469561278820037841796875f,-1.103400009906291501238229102455079555511474609375f,-0.35319997661113777809305247501470148563385009765625f,-0.87059999006390587084069920820184051990509033203125f,0.8834999942183490162506132037378847599029541015625f,0.03439999980330465945144169381819665431976318359375f,0.6924999928101895907417429043562151491641998291015625f,-0.147199997498839663112590869786799885332584381103515625f,-0.47899996763467844740347345577902160584926605224609375f,-0.1557999801270666118124808008360560052096843719482421875f,-0.75270001332759850498632658855058252811431884765625f,-0.146799987775087625863079665577970445156097412109375f,-0.78210001346617963324803213254199363291263580322265625f,0.2820999982669949446290047490037977695465087890625f,0.40279997938573386395688658012659288942813873291015625f,0.407899988582730355091143792378716170787811279296875f,-1.2038000201702114821955547085963189601898193359375f,0.08920002477168953447517196764238178730010986328125f,-1.0366000168412929927086452153162099421024322509765625f,0.1390000254511836175197458942420780658721923828125f,0.80099999377131414490804672823287546634674072265625f,0.2828000063195823532424810764496214687824249267578125f,-0.674300016236305310712850769050419330596923828125f,-0.02879999379515663093798139016143977642059326171875f,-0.497700004424154851534467525198124349117279052734375f,0.49440001917481435356194197083823382854461669921875f,0.52959998675584785132741671986877918243408203125f,0.37919999176561824949516221749945543706417083740234375f,0.2407000236988068309074151329696178436279296875f,0.3385999919414519609972558100707828998565673828125f,0.060299999725818675955224534845910966396331787109375f,-0.60610000609755498857111888355575501918792724609375f,-0.36659998264312765314798525650985538959503173828125f,-0.61889999673366558052833852343610487878322601318359375f,0.0767000015258787470884271897375583648681640625f,0.22319998719096201256206768448464572429656982421875f,0.0288999900653958974316992680542171001434326171875f,0.3357000134356322718076626188121736049652099609375f,0.486399984104931404349514423302025534212589263916015625f,0.43629999727830293121400018208078108727931976318359375f,0.18430000282526037835850729607045650482177734375f,0.237899992457032194437260841368697583675384521484375f,0.2465999957934024156003260941361077129840850830078125f,0.32849998037889627244823032015119679272174835205078125f,0.369699984907358725205739347074995748698711395263671875f,0.4940000050365924710860099366982467472553253173828125f,0.49879999841451638786793409963138401508331298828125f,0.301199998232722254698501274106092751026153564453125f,0.16999998584389697242613692651502788066864013671875f,0.628299972844124088311446030274964869022369384765625f,0.4043999898195267395095697793294675648212432861328125f,0.470800005999207460405386882484890520572662353515625f,0.4944000001385806530862510044244118034839630126953125f,0.60900001230090861081833963908138684928417205810546875f,-0.3485999931335452828307097661308944225311279296875f,-0.22429995961189330699880883912555873394012451171875f,0.196700034606456686248066034750081598758697509765625f,0.366200015741586781814476125873625278472900390625f,-0.03509999995827683960669673979282379150390625f,0.523500023946166237465149606578052043914794921875f,0.370000008940696734072162144002504646778106689453125f,0.252900002440810123260916952858678996562957763671875f,0.2108000031679868602196847859886474907398223876953125f,-0.172200004595518041394797137400018982589244842529296875f,-0.737500011734664173701503386837430298328399658203125f,-0.2181000019848344795292405251529999077320098876953125f,-0.578199997845291857601068841177038848400115966796875f,-0.5253999999448655255918083639699034392833709716796875f,-0.58230000995546593589580197658506222069263458251953125f,0.0081999885320662990739748465784941799938678741455078125f,0.311499988291412677199332392774522304534912109375f,0.080499994389712892228772034286521375179290771484375f,0.94680000357031790514383828849531710147857666015625f,1.4930000090301032145134740858338773250579833984375f,0.95520001384913921782526813331060111522674560546875f,-0.05200001391768438452345435507595539093017578125f,0.5304999762475492985913660959340631961822509765625f,0.4485999913454055132433495600707828998565673828125f,0.08160001497864699526729737044661305844783782958984375f,0.214500033028423242598137221648357808589935302734375f,-0.25899998849630367914187445421703159809112548828125f,-0.1818000092327594074959051795303821563720703125f,0.254899990132451126356727399979718029499053955078125f,0.3825000029057263883913719837437383830547332763671875f,0.50719998656511311452277368516661226749420166015625f,0.494100033530593396591257260297425091266632080078125f,-0.5494999975822860704965933109633624553680419921875f,-0.1963999734878546232774709778823307715356349945068359375f,-0.51329999115392599406959561747498810291290283203125f,-0.6296000012099742804139168583787977695465087890625f,0.214399992501735692229658525320701301097869873046875f,-0.63280001499950888632639589559403248131275177001953125f,-0.01830000026226041853760762023739516735076904296875f,-0.446999980986118483627933528623543679714202880859375f,-0.11269998643100265223182532281498424708843231201171875f,-1.091600029395520810027164770872332155704498291015625f,-0.211599999928474280519452577209449373185634613037109375f,-0.9984000210449099910192671814002096652984619140625f,-0.02550002020597474938767845742404460906982421875f,-0.7112000023305409257545761647634208202362060546875f,0.05590000778287684823908421094529330730438232421875f,-1.5583000685095793613754722173325717449188232421875f,-0.10160000097155563736350813996978104114532470703125f,-1.2945000500902537066139075250248424708843231201171875f,-0.6285000121742487788623066080617718398571014404296875f,0.21579999340772626492679364673676900565624237060546875f,-0.5853000216305257463744737833621911704540252685546875f,-0.4423000084653498920062020260957069694995880126953125f,0.2391999928086996352050164205138571560382843017578125f,-0.2425000032782553827104266019887290894985198974609375f,1.195600008131564084834508321364410221576690673828125f,0.024200020450353021406186826425255276262760162353515625f,1.100600010105967196949450226384215056896209716796875f,0.32190001192912454097694308075006119906902313232421875f,-0.18399999918788656483314980505383573472499847412109375f,0.21970001482144019266939949375228025019168853759765625f,-1.012600006365776295069736079312860965728759765625f,-0.2324000174760811887608724646270275115966796875f,-1.1206000157684083973208544193767011165618896484375f,0.163899995690584443508441836456768214702606201171875f,-0.8101999614715580388946136736194603145122528076171875f,0.0763000244915483794017063701176084578037261962890625f,0.59270001135319461127437534742057323455810546875f,0.38899997352063697686475052250898443162441253662109375f,0.4370999846324326032487306292750872671604156494140625f,0.0916999866992236345453193280263803899288177490234375f,-0.83959996872544306878438646890572272241115570068359375f,0.06950001984089604345484758596285246312618255615234375f,-0.72959997811317389704299785080365836620330810546875f,0.149499999612569656193272749078460037708282470703125f,-0.0847999990403653924886384629644453525543212890625f,0.008700012016296110317625789321027696132659912109375f,0.3958000254452229693669096377561800181865692138671875f,0.0120000052750111052546344581060111522674560546875f,-0.51429999477863308499081540503539144992828369140625f,0.128200004029273895156393336947076022624969482421875f,-0.560000010281801241518451206502504646778106689453125f,-0.77400000056624396194138171267695724964141845703125f,-0.447400005927681920780969448969699442386627197265625f,-0.248399999773502300204341963762999512255191802978515625f,-0.68510000066608178759253178213839419186115264892578125f,-0.193799996924400252940046129879192449152469635009765625f,0.18220000608563413635465622064657509326934814453125f,0.271400011932849682949608904891647398471832275390625f,0.0415000026524066090161113606882281601428985595703125f,-0.70419999035000824250118967029266059398651123046875f,-1.3653999622821810788764196331612765789031982421875f,-0.83329998297691343367432637023739516735076904296875f,-0.116600000226497524380420145462267100811004638671875f,0.044499997079372466401991914608515799045562744140625f,0.1722999990403650816261915679206140339374542236328125f,0.029999998956918627612822092487476766109466552734375f,0.2260000032335518493908921300317160785198211669921875f,0.19100000859797006125262441855738870799541473388671875f,0.25999999687075614485110008899937383830547332763671875f,-0.20180001169145089789935809676535427570343017578125f,0.052999995395541343867762407171539962291717529296875f,0.070199992614984552119494765065610408782958984375f,-0.20000001072883577535321819595992565155029296875f,-0.466500028133392419249503291212022304534912109375f,0.14640000105500217131293538841418921947479248046875f,0.239499995589256275962952713598497211933135986328125f,-0.2654000020682809601879625915898941457271575927734375f,0.146200007247924812503470093361102044582366943359375f,-0.271399994498491281547103426419198513031005859375f,-0.1759000035732984912328902282752096652984619140625f,0.0701999830409885117177282154443673789501190185546875f,0.36689997847601796276961749754264019429683685302734375f,0.476099987564235826997816047878586687147617340087890625f,0.2838000058650969048557044516201131045818328857421875f,0.588700013953447065517821101821027696132659912109375f,0.0821000007629393291352926098625175654888153076171875f,0.68939999354481695803542606881819665431976318359375f,-0.2219000077381727553671453279093839228153228759765625f,-0.4735000341534618684136148658581078052520751953125f,-0.63539999804496716251378529705107212066650390625f,-0.3675000043213365596983521754737012088298797607421875f,-0.999300012063979892218412715010344982147216796875f,0.0098000057458875744487158954143524169921875f,-0.740500002920627675706555237411521375179290771484375f,0.57240002015829105630473350174725055694580078125f,-0.04519999693632126280817828956060111522674560546875f,0.6288000285148622747755098316702060401439666748046875f,-0.2657000107720495218899259270983748137950897216796875f,0.076999999240040750692060100845992565155029296875f,-0.0965000020191073559772121370770037174224853515625f,0.01120000694990164191722215036861598491668701171875f,0.141399991220235943245597809436731040477752685546875f,-0.04900000559538597588726815956761129200458526611328125f,0.5696000064253805561520493938587605953216552734375f,0.40619997252821971045477766892872750759124755859375f,0.7166000010088089400284161456511355936527252197265625f,-0.0015000081658360642222760361619293689727783203125f,-0.40959997498393097004054652643389999866485595703125f,0.010300008629262169534968052175827324390411376953125f,-0.0240000092834236067318443019757978618144989013671875f,-0.26600000047683691750677326126606203615665435791015625f,-0.0500000005960463400622728613598155789077281951904296875f,-0.856600000151992002628276168252341449260711669921875f,0.387199984031915800386514092679135501384735107421875f,-0.698600008705258179730890333303250372409820556640625f,0.3274000523447995192327653057873249053955078125f,0.760999986022710572797222994267940521240234375f,0.2752000176861884028767235577106475830078125f,0.523700003969668870240639080293476581573486328125f,0.48779997498989136062164106988348066806793212890625f,0.369699981200695138028322617174126207828521728515625f,-0.385999993324279699891121708787977695465087890625f,0.52739998603463167370364317321218550205230712890625f,-0.2978000119075179252803309282171539962291717529296875f,-0.345899987647681983471414923769771121442317962646484375f,-0.61079997195005442112147875377559103071689605712890625f,-0.32969996646717181221220016595907509326934814453125f,0.23790001607537281103077475563623011112213134765625f,0.14799998906254785424607689492404460906982421875f,0.0804999966993928950387271470390260219573974609375f,-0.4778999822497365101980904000811278820037841796875f,0.090999999433755807132229165290482342243194580078125f,-0.0658000069677828758329951597261242568492889404296875f,-0.32640001737177348761775874663726426661014556884765625f,0.285100005620718188747986232556286267936229705810546875f,-0.10079999325871480397864843325805850327014923095703125f,-0.154399986355006901828090803974191658198833465576171875f,-0.32539998888075338800973668185179121792316436767578125f,0.18440000025033942421259780530817806720733642578125f,0.406100001701712454149628683808259665966033935546875f,0.4072000113755465111609055384178645908832550048828125f,-0.321199995476007416073116473853588104248046875f,-1.169999997764825661050736016477458178997039794921875f,-0.5811999970406291904367890310822986066341400146484375f,0.6035999736130239767817329266108572483062744140625f,1.06079994229674401395868699182756245136260986328125f,0.608799967345595494094823152408935129642486572265625f,-0.0958000074148177560573458322323858737945556640625f,-0.528500010311603318768902681767940521240234375f,0.158299998027086274987595970742404460906982421875f,-0.1450000199675560708101329510100185871124267578125f,0.4754999841451645892220767564140260219573974609375f,0.23689999473690992459751214482821524143218994140625f,0.45939998385906211098017593030817806720733642578125f,1.311399975425004971185671820421703159809112548828125f,0.7763000010222198543630156564177013933658599853515625f,0.3971999896943569918761340886703692376613616943359375f,1.023499981775879863477030085050500929355621337890625f,0.6055999849975106652522072181454859673976898193359375f,-0.0745999965906143369664960118825547397136688232421875f,0.169700009661912787350956932641565799713134765625f,-0.073900000160932677317759953439235687255859375f,0.109999978542327792041533029987476766109466552734375f,0.7945999778151513481816436978988349437713623046875f,0.26499998837709437538023848901502788066864013671875f,0.1887999954342841346655035295043489895761013031005859375f,0.123999995332211288001644788892008364200592041015625f,0.0761999959602952525727914689923636615276336669921875f,-0.582200004893541400718959266669116914272308349609375f,-0.80499999225139617919921875f,-0.267899990445375379977122065611183643341064453125f,0.07819999240636843484963947048527188599109649658203125f,0.2103999793812636465162313470500521361827850341796875f,-0.2379000262826681810768292280044988729059696197509765625f,-0.0845999846696852131344712688587605953216552734375f,0.59199999171495409910903617856092751026153564453125f,-1.2726000075578696169031900353729724884033203125f,1.53679998703002951287999167107045650482177734375f,-0.72300001811981218224900658242404460906982421875f,-0.4650999938488009632919784053228795528411865234375f,1.039799993228912367015936979441903531551361083984375f,-0.2946000076174737358769561978988349437713623046875f,1.16780003526508835420827381312847137451171875f,-0.82280001164674754221550756483338773250579833984375f,0.626400024971366153891949579701758921146392822265625f,0.19559998753070839683232406969182193279266357421875f,-0.5323999846190214935148787844809703528881072998046875f,0.12819999970793727328555178246460855007171630859375f,-0.53499999225139571734644050593487918376922607421875f,0.4955999777704478415074618169455789029598236083984375f,-0.30799998980760534550427109934389591217041015625f,0.0183999918758868119539329200051724910736083984375f,0.053199998068809595253014776972122490406036376953125f,-0.00139999956488612298244333942420780658721923828125f,0.82980000932216579911937515134923160076141357421875f,-0.9832000163972371975518171893781982362270355224609375f,0.46420001910924924004575586877763271331787109375f,-1.6369000044226655887769084074534475803375244140625f,1.445599982985854570216588399489410221576690673828125f,-0.889900019264221242565326974727213382720947265625f,-0.597500029802322529803859652020037174224853515625f,0.15800000292062765794298684340901672840118408203125f,-0.30550001394748704797166283242404460906982421875f,-0.68850002487748884760776491020806133747100830078125f,0.4162000117182731440124143773573450744152069091796875f,-0.3662000154808164875674947325023822486400604248046875f,0.13720004140138530868142652252572588622570037841796875f,0.21059996398687419461026593126007355749607086181640625f,0.0681000098079442839349439964280463755130767822265625f,0.283700024831293973193169222213327884674072265625f,-0.28580002134740301045212618191726505756378173828125f,0.17820000164508797979578957892954349517822265625f,-0.715199999320507373568034381605684757232666015625f,0.60699999372661128660411122837103903293609619140625f,-0.387300006937980612065075547434389591217041015625f,0.018199999223649510060596412586164660751819610595703125f,-0.04159999851286412531070624254425638355314731597900390625f,0.01179999980777501900064407891477458178997039794921875f,-0.04100003653764705546791446977294981479644775390625f,-0.163499979559331654055398530545062385499477386474609375f,-0.6938000280305747224929291405715048313140869140625f,-0.1831000100128353513806445107547915540635585784912109375f,-0.479000035621226505355707558919675648212432861328125f,-0.957400016805529663344032087479718029499053955078125f,-0.2825000017881393876706397350062616169452667236328125f,-1.0306000173330307490004997816868126392364501953125f,0.52040002143979080528879421763122081756591796875f,0.199900005443394201964935064097517170011997222900390625f,0.189500024348497664306023580138571560382843017578125f,0.238000026911497020165597859886474907398223876953125f,0.2714000035881996009123895419179461896419525146484375f,0.5388000279188158714305245666764676570892333984375f,0.0788000019907950655095874026301316916942596435546875f,-0.0352999996140599403560145219671539962291717529296875f,-0.1536999932408331659416944603435695171356201171875f,-0.6038000120490789601745973413926549255847930908203125f,-0.29590000092834234379779445589520037174224853515625f,-1.10440001794546827085241602617315948009490966796875f,-0.859200045782328114540860042325221002101898193359375f,-0.35760001381635675432590915079344995319843292236328125f,-0.81720005511045545887327534728683531284332275390625f,0.38000002115964892634991656450438313186168670654296875f,0.230500004149973403144713302026502788066864013671875f,0.3359000203371049764200506615452468395233154296875f,0.9596000176012520199719801894389092922210693359375f,0.1681000119313598162307243910618126392364501953125f,0.1113000496745115697194705717265605926513671875f,-0.4147999866724010331608951673842966556549072265625f,-0.22059999378919592505354785316740162670612335205078125f,-0.38879999603032988630957333953119814395904541015625f,0.762600015977024980884380056522786617279052734375f,0.27150000384449957113730533819762058556079864501953125f,0.28500002712011340832987116300500929355621337890625f,0.9284000286817555291918324655853211879730224609375f,0.20470000955015432708705702680163085460662841796875f,0.16830001784563108913062023930251598358154296875f,-0.3594000004738564513218079810030758380889892578125f,-0.1916999972790478512507661434938199818134307861328125f,-0.0236000169754024824442240060307085514068603515625f,1.10980000224411501363874776870943605899810791015625f,0.327800003562867647843859231215901672840118408203125f,1.009100032413005720854926039464771747589111328125f,-0.438800002437830016788211651146411895751953125f,0.29750001499429357121329076107940636575222015380859375f,-0.81720002098679600521791144274175167083740234375f,0.98760000919699664212458856127341277897357940673828125f,-0.46460002054423121808923724529449827969074249267578125f,-0.8496000147700311089238311978988349437713623046875f,1.419199994745850457178448777995072305202484130859375f,-0.511600018405914624963770620524883270263671875f,0.70710002982020370154714328236877918243408203125f,-0.39200001749396307104689185507595539093017578125f,0.37730001543164259913964997394941747188568115234375f,-0.0799999547004706101915871840901672840118408203125f,0.359199957269430658612918705330230295658111572265625f,-0.043599991941452032051529386080801486968994140625f,0.2666999956399198623557822429575026035308837890625f,-0.2468000058799979523627143862540833652019500732421875f,0.1415000041052698609433946330682374536991119384765625f,-0.15820001293271790387962028034962713718414306640625f,0.6211999977856874632919925716123543679714202880859375f,-0.1308000034287573054569975283811800181865692138671875f,-0.7938000490784649088027435936965048313140869140625f,0.779800031822919681445682726916857063770294189453125f,-0.4515000254511836175197458942420780658721923828125f,0.337500036358833188643302491982467472553253173828125f,-0.02600002253055555545557808727608062326908111572265625f,0.17870001112222677619456590036861598491668701171875f,1.56289999148845737408919376321136951446533203125f,-1.4539999820888045434230662067420780658721923828125f,0.85210001447200767188405734486877918243408203125f,-0.36619999383687940053277998231351375579833984375f,0.1705999768018724438434219337068498134613037109375f,-0.2004999898076054165585446753539144992828369140625f,1.0404000212907789091332233510911464691162109375f,-0.668800004896521471664527780376374721527099609375f,0.5592000125527381015899663907475769519805908203125f,1.45010003914833163207731558941304683685302734375f,-1.2586000008821489615229438641108572483062744140625f,0.788200033831596869049462839029729366302490234375f,-0.54160000618696102492322097532451152801513671875f,-0.0093999906390909160336377681232988834381103515625f,-0.273099997067451027987772249616682529449462890625f,1.117699980247021329660128685645759105682373046875f,-1.55199995826184800051805723342113196849822998046875f,0.651900000548362612562414142303168773651123046875f,-0.29169998990297330010434961877763271331787109375f,-0.2216999845385547729392783367075026035308837890625f,0.637200008022785535644061383209191262722015380859375f,0.1455000062361360146923061620327644050121307373046875f,0.8155000321269039176286241854541003704071044921875f,0.92800002005696313744920189492404460906982421875f,0.20690000907927752482606820194632746279239654541015625f,0.13470004706382798786989951622672379016876220703125f,0.59299999852478479933637345311581157147884368896484375f,0.2382999975055455632588774506075424142181873321533203125f,0.5830000108927484436804888900951482355594635009765625f,0.21240000525712965373514862221782095730304718017578125f,0.05910000056177378002786326760542578995227813720703125f,0.19510000666379934131100526428781449794769287109375f,0.64160001631975216440650910953991115093231201171875f,0.085600007200241190474798713694326579570770263671875f,0.002400008535385467212108778767287731170654296875f,0.139000004664063592230149879469536244869232177734375f,0.089300002825260182959254962042905390262603759765625f,-0.038599996709823614082779386080801486968994140625f,-0.70280003153979808683260444013285450637340545654296875f,-0.327700009825825755438444275569054298102855682373046875f,-0.57930003657639073111340621835552155971527099609375f,1.026600000450014871233861413202248513698577880859375f,0.3542999995470046403767128140316344797611236572265625f,1.1762000352621075904835379333235323429107666015625f,-0.5596000238597393927619805253925733268260955810546875f,-0.29930000547021629042632184791727922856807708740234375f,-0.79850003445148498570915762684307992458343505859375f,-0.7487999920070169679320315481163561344146728515625f,-0.227699994738399968952080598683096468448638916015625f,-1.001099995815753374017731403000652790069580078125f,-0.810399989998340064545345740043558180332183837890625f,-0.19429999280422916019261947440099902451038360595703125f,-0.13129999573230666243262021453119814395904541015625f,-0.4272000095129013796935169011703692376613616943359375f,-0.15390000280588866932163227829732932150363922119140625f,-0.09370000844001769024771419935859739780426025390625f,-1.0216000294327738373567626695148646831512451171875f,-0.3809000071123243014170611786539666354656219482421875f,-1.2527000398516658918879329576157033443450927734375f,-1.03899999199807613337043221690692007541656494140625f,-0.2203999987527727189018378339824266731739044189453125f,-0.2723000257134433610417545423842966556549072265625f,0.062600000479817285992112374515272676944732666015625f,0.433200022506714077508149784989655017852783203125f,0.4912999796390540296897597727365791797637939453125f,-1.188599976742267916307582709123380482196807861328125f,0.3164000018000603375867285649292171001434326171875f,1.472500003874301910400390625f,-1.2317999988019465718025458045303821563720703125f,0.7976000162005423277378213242627680301666259765625f,0.537999998077749896907562288106419146060943603515625f,-0.58359998650252820073802695333142764866352081298828125f,0.308999998405575571069903162424452602863311767578125f,0.214100004100799434780810770462267100811004638671875f,-0.31340000334978090901216774000204168260097503662109375f,0.1256000049650669581069450941868126392364501953125f,1.108400018548966414755341247655451297760009765625f,-1.04999999016523393180477796704508364200592041015625f,0.603400023913383876106308889575302600860595703125f,0.2370000137686734120734399766661226749420166015625f,0.091400003960728337659702447126619517803192138671875f,0.11470000236034394447415252216160297393798828125f,-0.69770004223585180280764461713260971009731292724609375f,0.44480001617670072988630636245943605899810791015625f,-0.3835000170916320616498751405742950737476348876953125f,0.8373999784350392161513809696771204471588134765625f,-1.339399972906708757847127344575710594654083251953125f,0.5052999952554699802931281737983226776123046875f,-0.29690002707242957313837905530817806720733642578125f,0.3584000181019304509533185409964062273502349853515625f,-0.182700008261203805659533827565610408782958984375f,-0.52169998662471783745786524377763271331787109375f,1.1777999661982061496701135183684527873992919921875f,-0.33540000042915352196359890513122081756591796875f,-1.2700999947428694980544605641625821590423583984375f,1.024799979817867079390225626411847770214080810546875f,-0.6867000065922734819423567387275397777557373046875f,-0.62930001891851450324111283407546579837799072265625f,0.3343999987602235268724371053394861519336700439453125f,-0.33350000971555715523209073580801486968994140625f,-0.76190002558231384455211809836328029632568359375f,1.2536000059485434832140526850707828998565673828125f,-0.46530001790523556337575428187847137451171875f,-1.566499970614909642563361558131873607635498046875f,1.48679996505081657431901476229541003704071044921875f,-0.8589000009059901685759541578590869903564453125f,0.4338999946475023961056649568490684032440185546875f,-0.78060003759861018579613300971686840057373046875f,-0.537599992060660891723955501220189034938812255859375f,-0.2519999946951865243960355655872263014316558837890625f,-0.1399999958276740841256469138897955417633056640625f,0.3413999908477067801726434481679461896419525146484375f,0.01109999626278879159002599408267997205257415771484375f,0.50679998911619161816588530200533568859100341796875f,0.72860002673566359732149066985584795475006103515625f,0.1915000094324350587982053184532560408115386962890625f,0.1112000442028051594434145954437553882598876953125f,0.05019999596774571415380705730058252811431884765625f,-0.01150000265240669339306123220012523233890533447265625f,0.31259999533891669187823936226777732372283935546875f,0.9258000112146140825331031010136939585208892822265625f,0.26490000473558905635940163847408257424831390380859375f,0.59760002514123922168209901428781449794769287109375f,-0.167199996399879591280068780179135501384735107421875f,-0.029900002686679361119104214594699442386627197265625f,-0.455700023400783749139009159989655017852783203125f,0.0916000099122522559724757229560054838657379150390625f,0.1267999994724988699346113207866437733173370361328125f,-0.0409999998807908383469111868180334568023681640625f,0.429799994942545726672733508166857063770294189453125f,0.0582999972820281708862921732361428439617156982421875f,0.3530000044107435286377949523739516735076904296875f,0.8801999963030215923254218068905174732208251953125f,0.1682999959968029812795720090434770099818706512451171875f,0.521400013013183727395016830996610224246978759765625f,-0.659200051295757649683082490810193121433258056640625f,-0.37880001339018354311605207840329967439174652099609375f,-0.7040000485479840364178016898222267627716064453125f,-0.488999995574354695548890958889387547969818115234375f,-0.0104999949485062860077277946402318775653839111328125f,0.2592000039100650354839672218076884746551513671875f,-0.66620001179277910097198400762863457202911376953125f,-0.21460000169426207694556296701193787157535552978515625f,-0.87610001101493839570366617408581078052520751953125f,0.022199994239211040536474683904089033603668212890625f,-0.16830000249743459761475605773739516735076904296875f,-0.65420002417564404595395899377763271331787109375f,-0.3706000145018102376326396552030928432941436767578125f,-0.093500004909932654673099250430823303759098052978515625f,-0.27820001930296456293945084325969219207763671875f,-0.41280000225901591903721055132336914539337158203125f,0.029400005316734478810758446343243122100830078125f,-0.7504000093698497408922776230610907077789306640625f,0.184799984288215757288753593456931412220001220703125f,-0.389199997949599918456442537717521190643310546875f,0.242599978873133892420810298062860965728759765625f,-0.9543999732792378676293765238369815051555633544921875f,0.1742000020101668322780597009113989770412445068359375f,1.139400019919872164564367267303168773651123046875f,-0.84680001690685724469176420825533568859100341796875f,0.61280001909732817466647247783839702606201171875f,-0.12920001076459897149106836877763271331787109375f,-0.31359999440014352245498230331577360630035400390625f,-0.04480000112652771093735282192938029766082763671875f,1.11120000426769305335028548142872750759124755859375f,-1.2101999806195500308803048028494231402873992919921875f,0.624900011217594197887592599727213382720947265625f,0.04070003219246842718348489142954349517822265625f,0.466399981087446402483465135446749627590179443359375f,-0.009499988809227755126585179823450744152069091796875f,0.13740003237724263129848623066209256649017333984375f,0.3555999784410002195755851062131114304065704345703125f,0.0532000070095062227437665569595992565155029296875f,0.4901999817371365253393378225155174732208251953125f,-0.872999984815716612018832165631465613842010498046875f,0.29210000180602069264068632037378847599029541015625f,1.1267999806225292847017271924414671957492828369140625f,-1.481199981468915627402793688816018402576446533203125f,0.64260000722259269512193213813588954508304595947265625f,-0.5078000721454623000994388348772190511226654052734375f,0.1942000499546525649208206232287921011447906494140625f,-0.279900027459860201162200610269792377948760986328125f,-1.0559999942183484478164245956577360630035400390625f,0.902999998524784519560171247576363384723663330078125f,-0.56490001270771017516381107270717620849609375f,-0.469300010275841117390882573090493679046630859375f,0.98979999196231371882959138019941747188568115234375f,-0.2982000114798548651151577359996736049652099609375f,0.58279999927878378684908966533839702606201171875f,-0.437999994501471423546945516136474907398223876953125f,0.297800014254451017592373318620957434177398681640625f,-0.42240000942945499229352890324662439525127410888671875f,0.55680000558197495497125828478601761162281036376953125f,-0.2428000106781722511328780456096865236759185791015625f,-0.055700002837181017412149230949580669403076171875f,-0.300000004470348358154296875f,0.2992000019729135207313674982287921011447906494140625f,0.04759999571144579721959644302842207252979278564453125f,0.48199999588727937549492708058096468448638916015625f,0.2401999801725145378128445372567512094974517822265625f,0.1773999922558664454896870665834285318851470947265625f,0.0770999778151502113132664817385375499725341796875f,0.36440004256963742790276228333823382854461669921875f,0.312400008907914195077637486974708735942840576171875f,0.667800044131279690873270737938582897186279296875f,-0.135399999907612755833952178363688290119171142578125f,-0.0682999993681907557885324422386474907398223876953125f,-0.2714000038862227626168532879091799259185791015625f,-0.433199980783462113009818494901992380619049072265625f,-0.068099995242059119249233845039270818233489990234375f,0.317099998676777516948277479968965053558349609375f,-0.7304000275492672500377011601813137531280517578125f,-0.426400009101629340335648521431721746921539306640625f,-1.170800034236908260254494962282478809356689453125f,0.658399995005130467262688398477621376514434814453125f,0.2760999965980648340746483881957828998565673828125f,0.14389999762773442171237547881901264190673828125f,0.24219999744296061283677090614219196140766143798828125f,-0.096700002010166652421929711636039428412914276123046875f,-0.242600004726648332820104769780300557613372802734375f,0.2372000148773192318429892111453227698802947998046875f,0.250599999898672098908036787179298698902130126953125f,0.62520000736713399192012730054557323455810546875f,0.2412000105261800086964285583235323429107666015625f,0.3044999971911310776562231694697402417659759521484375f,0.8441000055909153587663240614347159862518310546875f,0.6123999879717823002778231966658495366573333740234375f,0.172399993000924511843408026834367774426937103271484375f,0.07259998159259473116122762803570367395877838134765625f,0.25860002420246619436738910735584795475006103515625f,0.2115000071227550737518185997032560408115386962890625f,0.8162000424146658161816958454437553882598876953125f,-0.5350000060349702213358114022412337362766265869140625f,-0.257599999921023836702005382903735153377056121826171875f,-0.647500009983777768951540565467439591884613037109375f,-0.8342000318497422295394017055514268577098846435546875f,-0.267100008772313657345875981263816356658935546875f,-0.25240002533644478877050687515293247997760772705078125f,-0.093199997323751393452084812452085316181182861328125f,0.10409999486207954788596907746978104114532470703125f,0.1687999998301261683764096233062446117401123046875f,-0.6948000049263234334517846946255303919315338134765625f,0.1254000085130335673255785877699963748455047607421875f,0.29629999960660935443002017564140260219573974609375f,0.1846000304907555733535673425649292767047882080078125f,0.1431999885320660581555785029195249080657958984375f,0.035600059652328042147928499616682529449462890625f,0.19099994295835553970164255588315427303314208984375f,0.024700017261505280430355924181640148162841796875f,-0.017599998170137354236430837772786617279052734375f,0.142799996522068983750841653090901672840118408203125f,-0.020999998167157141182315172045491635799407958984375f,-0.9722999729633325927125042653642594814300537109375f,0.507799969625473135437232485855929553508758544921875f,-0.504599992418288678663884638808667659759521484375f,-0.27960003771782027115477831102907657623291015625f,0.3800000002980237212568681570701301097869873046875f,-0.183200007605552883660493534989655017852783203125f,0.9531000131607054726146088796667754650115966796875f,-0.350399979865551269853085614158771932125091552734375f,0.498500001668929826337262056767940521240234375f,0.64979998797625260209542830125428736209869384765625f,-0.82219999680966104893542478748713620007038116455078125f,0.35710001173391947570934235045569948852062225341796875f,-0.13419996397495292939083810779266059398651123046875f,0.2301999793529512228218436575843952596187591552734375f,-0.063499995410442267029793583787977695465087890625f,-0.30769996683597611308869090862572193145751953125f,0.3413999815344812560624632169492542743682861328125f,-0.150800001454353349572556908242404460906982421875f,0.96760000319927907952433088212274014949798583984375f,-0.6529999743103982456915446164202876389026641845703125f,0.5171000049725174019243922884925268590450286865234375f,-0.215599984252452969712976482696831226348876953125f,-0.07680002480447267743102202075533568859100341796875f,-0.08719999878406525795071502216160297393798828125f,-0.37950001332163818545950562111102044582366943359375f,0.4061999931663275997806294981273822486400604248046875f,-0.22300000247359275107328357989899814128875732421875f,-1.19380004755109592196049561607651412487030029296875f,0.8288000163704156708632808658876456320285797119140625f,-0.6426000292018059223408954494516365230083465576171875f,0.1567000015258788181427007657475769519805908203125f,-0.7210000219345094052414424368180334568023681640625f,0.69400000503659242667708895169198513031005859375f,0.32090000003427265884425878539332188665866851806640625f,0.75850001180171933157225794275291264057159423828125f,0.62560000690221795327516929319244809448719024658203125f,0.183200002688169494291514638462103903293609619140625f,0.4912000176787376659603978623636066913604736328125f,-0.37279999957680676203608527430333197116851806640625f,0.0431000023946166432864401940605603158473968505859375f,0.1056999941945075960347821819595992565155029296875f,-0.316999998196959509311199099101941101253032684326171875f,-0.04810000034570693916435857317992486059665679931640625f,-0.1408000054776668008571505197323858737945556640625f,0.867199988949298283813504895078949630260467529296875f,0.29709999163597811122627945223939605057239532470703125f,0.91690000650882641508587767020799219608306884765625f,0.814200040641427591481260606087744235992431640625f,0.4047000146538020093345267014228738844394683837890625f,1.1684000638484963729979426716454327106475830078125f,-0.195600002133846206930911648669280111789703369140625f,-0.18660000007748600570067765147541649639606475830078125f,-0.80680002130270001003964353003539144992828369140625f,0.7739999871551985766160441926331259310245513916015625f,0.3191999927714466167572027188725769519805908203125f,0.93340000651627696992562732702936045825481414794921875f,0.572999998226761508846038850606419146060943603515625f,0.21539999592155212315702783598680980503559112548828125f,0.93150001442432373011115487315692007541656494140625f,-0.98820002496540571002014985424466431140899658203125f,-0.3081000078707933909072380629368126392364501953125f,-0.93800004374980971988406963646411895751953125f,-0.56739999803006657685955360648222267627716064453125f,-0.0269000034540891608259016720694489777088165283203125f,0.2571999838829039930487851961515843868255615234375f,0.1744000020384788296468059343169443309307098388671875f,0.005300000359117984505274989714962430298328399658203125f,-0.085699997919797876733127850457094609737396240234375f,0.32700000579655164045789206284098327159881591796875f,-0.0641999995514750398939440856338478624820709228515625f,-0.5067000030159949375274663907475769519805908203125f,0.2064000050783156847700183789129368960857391357421875f,0.05130000009089707990739270826452411711215972900390625f,0.30350000941753396688227439881302416324615478515625f,-0.23580000346600993310630656196735799312591552734375f,-0.2032000055193901033590009319595992565155029296875f,0.556500012934208143633441068232059478759765625f,-0.54279998497366932497243396937847137451171875f,0.3205000017285346558537639793939888477325439453125f,0.686400000309944413601215273956768214702606201171875f,-0.84319998674392715454217750448151491582393646240234375f,0.39280000628232958348462489084340631961822509765625f,-0.761699986028670394944128929637372493743896484375f,0.95020000244975033609762249398045241832733154296875f,-0.422000011086463899800946819595992565155029296875f,-0.4449999901652337275237414360162802040576934814453125f,0.563199985775351574801561582717113196849822998046875f,-0.251400001949071860707363157416693866252899169921875f,0.7632999826788893216189535451121628284454345703125f,-1.102799975287914069355110768810845911502838134765625f,0.4520999956965443544731897418387234210968017578125f,0.44080003334283901494927704334259033203125f,-0.655000017806887768756496370770037174224853515625f,0.27610001220703139779288903810083866119384765625f,0.3106000072002412792926406837068498134613037109375f,0.038999997735023494982442571199499070644378662109375f,0.13690001172423382769238742184825241565704345703125f,0.57499998304992860909123919554986059665679931640625f,-0.8123999690473080459440780032309703528881072998046875f,0.3451999898955222523255770283867605030536651611328125f,0.2624999868869775099256003159098327159881591796875f,-0.808799990293383341821709109353832900524139404296875f,0.187499996423721171368015347979962825775146484375f,-0.959800012004375702190372976474463939666748046875f,1.33280000024735922892205053358338773250579833984375f,-0.561400015807151930857799015939235687255859375f,-1.187599981033802265528720454312860965728759765625f,1.00699997539818308922576761688105762004852294921875f,-0.6357000067114828567582662799395620822906494140625f,0.37110000103116025371718933456577360630035400390625f,-0.3187999999046324806073471336276270449161529541015625f,0.1987000060558319347592259873636066913604736328125f,0.9765000078678127692910493351519107818603515625f,-0.74060000652968849266244433238171041011810302734375f,0.51240002060532585659302640124224126338958740234375f,0.12909999918341608093896866193972527980804443359375f,-0.3868000050604341044646616865065880119800567626953125f,0.088300003764033352382512020994909107685089111328125f,0.139799989354610687541935476474463939666748046875f,-0.890499999698250999102810965268872678279876708984375f,-0.0981999959081409912187154986895620822906494140625f,-0.0175999964937567643374904946540482342243194580078125f,0.5153000277400021644780281349085271358489990234375f,0.588000002920626929636682689306326210498809814453125f,0.4199999900162219024224441454862244427204132080078125f,1.1507999931097021573123129201121628284454345703125f,-0.578400005882978351934298189007677137851715087890625f,-0.2934000008165835726714476550114341080188751220703125f,-1.0378000294536351422181041925796307623386383056640625f,-0.28759999444484740660499255682225339114665985107421875f,-0.0025000033900142160092627818812616169452667236328125f,-0.007700003790855447505236952565610408782958984375f,0.154400006806850509377682101330719888210296630859375f,0.0073000040695071211072075811898685060441493988037109375f,-0.2344999826997515857129883443121798336505889892578125f,1.0136000275552277116730692796409130096435546875f,0.35080000771284114424730660175555385649204254150390625f,1.23240002850294150249510494177229702472686767578125f,0.177200001019239383737158277654089033603668212890625f,-0.023599999839067464790787198580801486968994140625f,-0.040999994516372595398934208787977695465087890625f,-0.72240000622570488264528876243275590240955352783203125f,-0.170399997784197287931107211989001370966434478759765625f,-0.50880001502931104795379724237136542797088623046875f,0.57100001068413208660246027648099698126316070556640625f,0.2833999959364532639227718391339294612407684326171875f,0.379700007128715100179761066101491451263427734375f,1.0694000004738570819284859680919907987117767333984375f,0.331600002722442166458449719357304275035858154296875f,0.983500023424625169354840181767940521240234375f,0.54759999548792848145239986479282379150390625f,0.0332000006765127420038652417133562266826629638671875f,-0.3130000085830690892407801584340631961822509765625f,-0.777400014123320826087137902504764497280120849609375f,-0.16490000719428066222604911672533489763736724853515625f,-0.15420002864599258174393980880267918109893798828125f,-0.15599999272823339424576261080801486968994140625f,0.0602999995395541166232078467146493494510650634765625f,0.2509000009655952823095503845252096652984619140625f,-0.5366000245898965292923321612761355936527252197265625f,-0.3220000088140368976752370144822634756565093994140625f,-0.6267000334143641993023265968076884746551513671875f,-1.00280002498328713045339100062847137451171875f,-0.19340000254884370800567694459459744393825531005859375f,-0.53730002392530451515995082445442676544189453125f,-0.1132000039547680358964498736895620822906494140625f,-0.2545000104904175941555877216160297393798828125f,0.0420000337362278486352806794457137584686279296875f,-0.084000011146068143119691740139387547969818115234375f,0.050400000429153379855051753111183643341064453125f,-0.1615999950855968592122735572047531604766845703125f,0.542999992042779933143492598901502788066864013671875f,-0.127699993434548186854726736783050000667572021484375f,-0.507899980759621616499543961253948509693145751953125f,0.62579996658563674660769038382568396627902984619140625f,-0.28279999942779543431470301584340631961822509765625f,0.4411000003606080355211815913207828998565673828125f,-0.192800004792213375282017295830883085727691650390625f,0.225300004568695921847165664075873792171478271484375f,0.7754000269532212996637099422514438629150390625f,-1.3533999942600729582409258000552654266357421875f,0.4759000206351284845140980905853211879730224609375f,0.35679999246895288678160795825533568859100341796875f,-0.4693999995797870639080429100431501865386962890625f,0.1989000051900744470145809827954508364200592041015625f,-0.85259999951124110229017105666571296751499176025390625f,1.11900000160932489023934977012686431407928466796875f,-0.4887000114947557261046995336073450744152069091796875f,0.624500032842158159240852910443209111690521240234375f,-0.24380002128779831860327931281062774360179901123046875f,0.33360001071691502971816589706577360630035400390625f,1.06209998556375584399802391999401152133941650390625f,-1.4343999550253159203094810436596162617206573486328125f,0.6183000008583068307643770822323858737945556640625f,1.189399997568131084335618652403354644775390625f,-0.909199970680475644257967360317707061767578125f,0.631200012612342931106468313373625278472900390625f,-1.2076000045776371649708380573429167270660400390625f,1.087999988988042066040407007676549255847930908203125f,-0.6584000146746635806493941345252096652984619140625f,-0.4854999819099905522534754709340631961822509765625f,0.47439998043179532061230929684825241565704345703125f,-0.259300000962614962674024354782886803150177001953125f,-0.36890003398656912025899146101437509059906005859375f,0.1268000079661610879355748693342320621013641357421875f,-0.2053000071763992906426210538484156131744384765625f,-1.227200014579296283301346193184144794940948486328125f,0.751600006037949963655364626902155578136444091796875f,-0.17440000211298467558407310207257978618144989013671875f,-0.060500002101063710124861927397432737052440643310546875f,-0.523500027149915947433100882335565984249114990234375f,-0.47760000547170644580319276428781449794769287109375f,-0.13620000329911707837027279310859739780426025390625f,-0.69060003402233149927269550971686840057373046875f,0.151400010219216341056380770169198513031005859375f,0.062800003041326990516068917713710106909275054931640625f,-0.105899984425306126212262824992649257183074951171875f,-0.66739997776448678479255249840207397937774658203125f,-0.2010999901533125910901844690670259296894073486328125f,-0.7097999890565862557423315593041479587554931640625f,1.151000018283724557477398775517940521240234375f,0.3691000003010034280492845937260426580905914306640625f,1.2912000370502472179623509873636066913604736328125f,0.2282000107347965656146016044658608734607696533203125f,0.212500002868473536477011975875939242541790008544921875f,0.33040001182854172867564557236619293689727783203125f,-0.73420000681579100643148194649256765842437744140625f,-0.2606000004015862625550425946130417287349700927734375f,-0.463700015350430982952190106516354717314243316650390625f,0.437799990114569492760665525565855205059051513671875f,0.3943999955192207806931037339381873607635498046875f,1.12689999905824578263491275720298290252685546875f,-0.633000012382864962745543380151502788066864013671875f,-0.1209000001460314166479292907752096652984619140625f,-0.80800001156330125695603783242404460906982421875f,-0.76780000441968443425366785959340631961822509765625f,-0.2312000013992190350364808182348497211933135986328125f,-1.0659000283837318789892378845252096652984619140625f,-0.471200001883506747191177055356092751026153564453125f,-0.210700000713765600579563397332094609737396240234375f,-0.877300023925304373051403672434389591217041015625f,-0.151400005152821481857472463161684572696685791015625f,-0.1363000006496906113540035221376456320285797119140625f,-0.0826000051736830442905556992627680301666259765625f,-0.54080001009702716174842862528748810291290283203125f,-0.136000007629394570329850466805510222911834716796875f,-0.717600041532516996767299133352935314178466796875f,0.70259998677074886330728986649774014949798583984375f,0.21839999668151133249693884863518178462982177734375f,0.87170001851320222385766101069748401641845703125f,0.071799999174475548358032028772868216037750244140625f,0.22079999176859832488162282970733940601348876953125f,0.08990002205818914404744646162725985050201416015625f,0.38619999677985894503962072121794335544109344482421875f,0.01720000796690601685412502774852328002452850341796875f,-0.299399979090690493421789142303168773651123046875f,0.81519999097585671421484221355058252811431884765625f,-0.2009999966025350914833325077779591083526611328125f,0.3191000125944611909289960749447345733642578125f,-0.0148000140160320370341651141643524169921875f,0.15910000573992721939475813996978104114532470703125f,-0.596599971616267765739394235424697399139404296875f,0.96739995734989658870972561999224126338958740234375f,-0.357699990677833312702205148525536060333251953125f,0.9710000061392776160573703236877918243408203125f,-1.60720000452101174914787407033145427703857421875f,0.58660001423358920646933256648480892181396484375f,0.07780002505779293198173718337784521281719207763671875f,0.23439999451339221803181089853751473128795623779296875f,0.0366000007480382638647142812260426580905914306640625f,-0.86630001076385365621490564080886542797088623046875f,0.740799991992115991479295189492404460906982421875f,-0.47850000742450349289214273085235618054866790771484375f,-0.1982999957919115985305325011722743511199951171875f,0.149800032493471402261775438091717660427093505859375f,-0.08120001916885399850798421539366245269775390625f,-0.5000000020861623539758511469699442386627197265625f,1.279800002244114498495264342636801302433013916015625f,-0.3245000161528590609805178246460855007171630859375f,-0.507699981439113656733752577565610408782958984375f,1.23679997995197776816667101229541003704071044921875f,-0.332299999785423239018200547434389591217041015625f,-0.1212999867916106921938990126363933086395263671875f,0.577999986529350362474133362411521375179290771484375f,-0.105299992871284331386050325818359851837158203125f,-0.1516000175118445980615433654747903347015380859375f,-0.203199994865059796467221531202085316181182861328125f,-0.06960000076293937354421359486877918243408203125f,-0.393199978399277227936181589029729366302490234375f,0.96839998427629492283585932455025613307952880859375f,-0.255399998641014036593333003111183643341064453125f,0.535899962222575965142823406495153903961181640625f,-1.05159996081292650416116885025985538959503173828125f,0.33429998703002894444580306299030780792236328125f,-0.3423000066399577434594903024844825267791748046875f,0.4917000059962273184055447927676141262054443359375f,-0.930200002971291350917226736783050000667572021484375f,-0.37690000002682200186399086305755190551280975341796875f,-1.1918000364273784175139780927565880119800567626953125f,0.12700000520050525043558309334912337362766265869140625f,0.0722000007584691072537452782853506505489349365234375f,-0.012699999022483865473986952565610408782958984375f,-0.658800012868642870245139420148916542530059814453125f,-0.23580000458359717807610422823927365243434906005859375f,-0.82800003987550763184799507143907248973846435546875f,-0.65980004903376132663339603823260404169559478759765625f,-0.3233000117361546354999290997511707246303558349609375f,-0.527000044167042602794026606716215610504150390625f,-0.2718000054329632320104792597703635692596435546875f,-0.0818999989837407849080364030669443309307098388671875f,-0.7812000203609468229615231393836438655853271484375f,-0.21639999122023567679207189939916133880615234375f,0.00100000286102297497592417130363173782825469970703125f,0.34600001329183616149975932785309851169586181640625f,0.6958000353544953053841481960262171924114227294921875f,0.299900010100007108349018380977213382720947265625f,0.413600040817261316306030494160950183868408203125f,-0.98680001355409618923886228003539144992828369140625f,-0.28400000391900537266565152094699442386627197265625f,-1.05060004832744624536644550971686840057373046875f,-0.67460001305639760715848751715384423732757568359375f,-0.150400002254545750446368401753716170787811279296875f,-0.818500012695789536110169137828052043914794921875f,-0.0156000139057639319872805572231300175189971923828125f,-0.0849000058159232739996014061034657061100006103515625f,0.1846999972939489254031286691315472126007080078125f,-0.4426000466734176796990141156129539012908935546875f,-0.40100001608580360112910057068802416324615478515625f,-1.14710006722211943497313768602907657623291015625f,0.8401999891132110320768333622254431247711181640625f,0.1215999914720653496402746895910240709781646728515625f,0.1872999953150742413754414883442223072052001953125f,-0.592800001364945927662120084278285503387451171875f,-0.34379999550878992398139644137700088322162628173828125f,-1.02100001865625333863363266573287546634674072265625f,-0.1374000033944844378908101134584285318851470947265625f,-0.1208999990284442827004340870189480483531951904296875f,0.0210000076293948012562395888380706310272216796875f,0.9260000217854980064657866023480892181396484375f,-0.198000008583068876077959430404007434844970703125f,-0.6437999861210581542536601773463189601898193359375f,1.039399979612231295078572657075710594654083251953125f,-0.395299993541836425237079311045818030834197998046875f,0.204500014960765863492042626603506505489349365234375f,0.03259999511539934946569019302842207252979278564453125f,0.10140000478029254082912302692420780658721923828125f,-0.4811999938368796136956007103435695171356201171875f,0.859999997168779284351103342487476766109466552734375f,-0.2976000012993811338901650742627680301666259765625f,-0.660200068461894939986223107553087174892425537109375f,0.34300005231797647820002339358325116336345672607421875f,-0.35910003062486683944598553352989256381988525390625f,0.10980001379251547177773318253457546234130859375f,0.6372000075012440589716788963414728641510009765625f,0.010800000858306901818650658242404460906982421875f,-0.6355999973654746781903668306767940521240234375f,0.30919998438954365838071680627763271331787109375f,-0.3258000041365622934108614572323858737945556640625f,0.81620004569292081697540197637863457202911376953125f,-0.4652000280052421832266418277868069708347320556640625f,0.4404000212907792644045912311412394046783447265625f,-0.887199985074996533285229816101491451263427734375f,1.48739999274015399777226775768212974071502685546875f,-0.53390000507831558707039221189916133880615234375f,-0.553900004184246341765174292959272861480712890625f,1.25159999389350407028587142121978104114532470703125f,-0.35070001535415684656982193700969219207763671875f,-0.12290003380775527119794787722639739513397216796875f,-0.4163999909222122486340822433703579008579254150390625f,-0.0401000081539155672771812533028423786163330078125f,0.20709995218515331316666561178863048553466796875f,-0.1571999619036912321234922274015843868255615234375f,0.0854999930858610923678497783839702606201171875f,1.31529998363256339644067338667809963226318359375f,-1.4257999818593258822829739074222743511199951171875f,0.727800010156631316249331575818359851837158203125f,-0.1772000032544127634537289850413799285888671875f,0.352999998897313815859888563863933086395263671875f,-0.1270999927163121157036584918387234210968017578125f,-0.20760002454519277392819276428781449794769287109375f,-0.2199999963492154098076980517362244427204132080078125f,-0.0948000045537948921037241234444081783294677734375f,1.452000022709370341544854454696178436279296875f,-0.31470000750124427213449962437152862548828125f,-0.616200004342198237594629972591064870357513427734375f,-0.247500000149011578631785823745303787291049957275390625f,-0.388600013846158720554058163543231785297393798828125f,0.48279998959302894423473162532900460064411163330078125f,0.0361999978601932337340940648573450744152069091796875f,0.289600003445148246328244567848742008209228515625f,0.128400025254487903936251314007677137851715087890625f,0.08550000414997338538114490802399814128875732421875f,-0.2776999814391132304081111215054988861083984375f,-0.590399989104270606077307093073613941669464111328125f,-0.19949999421089881224133932846598327159881591796875f,-0.6336999997973435938547481782734394073486328125f,-0.07959999383389948501843491612817160785198211669921875f,0.108100001165270798164641519178985618054866790771484375f,0.542500014752149706254158445517532527446746826171875f,-1.097599991762637561265592012205161154270172119140625f,-0.20859999395310868663955261581577360630035400390625f,-0.410600011372565631972975097596645355224609375f,1.0827999957025051802617099383496679365634918212890625f,0.2138000016927719126869078536401502788066864013671875f,0.4684000173568723113248779554851353168487548828125f,-0.4712000217020513304078122018836438655853271484375f,-0.2251000048384070861828831766615621745586395263671875f,-0.5713000192761423789988839416764676570892333984375f,-0.235799995419383012773550944984890520572662353515625f,-0.0314999976605176978949884869507513940334320068359375f,-0.3623999954223631192462562466971576213836669921875f,-0.545200011241436133246907047578133642673492431640625f,-0.07250000450760130110694490213063545525074005126953125f,-0.362900022333860761136747896671295166015625f,-0.309600002253055617273957977886311709880828857421875f,-0.15389999848455193642848826129920780658721923828125f,-0.7772999965071676342631690204143524169921875f,0.3398000375598673183930031882482580840587615966796875f,0.2962000105634332580706313819973729550838470458984375f,0.7069000348210341400090328534133732318878173828125f,0.6452000090062617321251536850468255579471588134765625f,0.2179000019565224544049186761185410432517528533935546875f,0.4143000191420316280499491767841391265392303466796875f,-0.6559999973475931778210679112817160785198211669921875f,-0.2019999990165233239025610600947402417659759521484375f,-0.7472000196456907161746130441315472126007080078125f,-0.6907999966859816876052491352311335504055023193359375f,-0.3213000022143124301265970643726177513599395751953125f,-0.7148000094711779439649035339243710041046142578125f,0.499599991449713787261543984641321003437042236328125f,-0.39020000334382043405412332504056394100189208984375f,0.640199970114231131645965433563105762004852294921875f,-1.051799969446659321903325690072961151599884033203125f,0.3738999992668627925240798504091799259185791015625f,0.3911000432014457572904575499705970287322998046875f,0.216799953204394046935021833633072674274444580078125f,0.1817000185132027212375760427676141262054443359375f,-0.5140999850273129112565584364347159862518310546875f,0.767199973750114505577357704169116914272308349609375f,-0.305899995303153815484620281495153903961181640625f,-0.57889999542981396274399230605922639369964599609375f,0.36459998925179248896455419526319019496440887451171875f,-0.29520000861510642398144455000874586403369903564453125f,-1.5773999775409688339777858345769345760345458984375f,1.733999974191188453431777816149406135082244873046875f,-0.878000008583068591860865126363933086395263671875f,1.50719997673034722396323559223674237728118896484375f,-1.7079999527037148521202425399678759276866912841796875f,0.8428000065803527007801676518283784389495849609375f,-0.33430003113746664666905417107045650482177734375f,0.3658000144183635171657442697323858737945556640625f,-0.19680001325607321405186667107045650482177734375f,-0.1481999889791012492423760704696178436279296875f,0.564399985572695772617635157075710594654083251953125f,-0.106800000962615104782571506802923977375030517578125f,-0.6832999943017961097524448632611893117427825927734375f,1.0715999953091144281103197499760426580905914306640625f,-0.398100011819601196094708939199335873126983642578125f,0.0710999940276142439188333810307085514068603515625f,0.413599989557266400908019932103343307971954345703125f,0.00089999843239763066549130599014461040496826171875f,-0.02929994560480064791363474796526134014129638671875f,0.1571999653309583688809425439103506505489349365234375f,-0.0056999855518338193860472529195249080657958984375f,0.7613000086218117967717944338801316916942596435546875f,-0.701399995541572618407144545926712453365325927734375f,0.42280000754892821390740209608338773250579833984375f,-0.54899998044967635024704577517695724964141845703125f,0.9569999778568745174567311551072634756565093994140625f,-0.3340999963521955606893243384547531604766845703125f,-0.92469998328685765187628931016661226749420166015625f,0.57770000580251146260479799821041524410247802734375f,-0.6496000098526477639637732863775454461574554443359375f,-0.12020000177919865080866657081060111522674560546875f,-0.595400024271011574228396057151257991790771484375f,1.09620003563463708218250758363865315914154052734375f,0.41460000698417431852504932976444251835346221923828125f,1.39390004203319595177390510798431932926177978515625f,-0.26959999890029429092663804112817160785198211669921875f,0.010400000615417959171082884495262987911701202392578125f,-0.14160000462234023643759428523480892181396484375f,0.95780002826154253536827809512033127248287200927734375f,0.2173000076085329634878462456981651484966278076171875f,0.684400039589405650986009277403354644775390625f,-0.64640003137886570261372298773494549095630645751953125f,-0.288700010675191964804042754622059874236583709716796875f,-0.588900035700202550970061565749347209930419921875f,-0.75400000220537177764157377168885432183742523193359375f,-0.21709999960809944763440171300317160785198211669921875f,-0.67350001448392848857338321977294981479644775390625f,0.1562000037461519497128392686136066913604736328125f,0.0079000036329030798309958072422887198626995086669921875f,-0.2745999807208774345923529835999943315982818603515625f,0.1675999986171721189975869492627680301666259765625f,0.03669999843388793703269357138196937739849090576171875f,0.10710000135898578577098305686376988887786865234375f,0.9757999970585105042886198134510777890682220458984375f,0.2904999980032443307464973258902318775653839111328125f,0.8922000267982479471129408921115100383758544921875f,0.232800013136863714890978371840901672840118408203125f,0.16830000141710044214704566911677829921245574951171875f,0.45010000934600835620358338928781449794769287109375f,0.001199992793798454471243530861102044582366943359375f,-0.09630000023990870072765346776577644050121307373046875f,-0.31630000288486481707650455064140260219573974609375f,0.13800001677870721294993927585892379283905029296875f,0.20999999813735481524190618074499070644378662109375f,0.48460000731945029883718234486877918243408203125f,1.041200003969669385384122506366111338138580322265625f,0.30460000322163105490602674763067625463008880615234375f,0.7976000273764132231235635117627680301666259765625f,-0.4528000059843062530262614018283784389495849609375f,-0.14210000035315750732678452550317160785198211669921875f,-0.33030001194477076609246068983338773250579833984375f,0.329000004366040155190376026439480483531951904296875f,0.3892999957472083050191713482490740716457366943359375f,-0.68239999125003780733322855667211115360260009765625f,1.2141999983966347276265196342137642204761505126953125f,-0.4089000104427338300183691899292171001434326171875f,0.779300035309791638837850769050419330596923828125f,-1.34480001699626416922228600014932453632354736328125f,0.4811000232338908944029753911308944225311279296875f,-0.38539998619854431982645337484427727758884429931640625f,0.7069999925345180624702834393247030675411224365234375f,-0.22750000465661290860452936613000929355621337890625f,1.127200015175342517892431715154089033603668212890625f,-1.52500001110136462756372566218487918376922607421875f,0.64830002499818828454181129927746951580047607421875f,-0.60710003831386638584177717348211444914340972900390625f,0.49060001345872894518151952070184051990509033203125f,-0.3400000170618298067637397252838127315044403076171875f,-0.770499993979930852816551123396493494510650634765625f,1.0721999814987182020331601961515843868255615234375f,-0.44900000488758085026574917719699442386627197265625f,0.472799994435906256740054232068359851837158203125f,-0.1744000013679265492783088120631873607635498046875f,0.239600001284479890983902805601246654987335205078125f,0.20689999667406056005347636528313159942626953125f,-0.2661999977111815240959913353435695171356201171875f,0.1180000011324882080998577293939888477325439453125f,0.97819997750520659707262893789447844028472900390625f,-1.3573999732941388618456812764634378254413604736328125f,0.5706999980688092222180785029195249080657958984375f,0.0197000241160392164374570711515843868255615234375f,-0.022000014066696049752636099583469331264495849609375f,0.020900006031990159272027085535228252410888671875f,0.271299983364343688663211651146411895751953125f,-0.253999988198280401974216147209517657756805419921875f,0.140499999716877876920761991641484200954437255859375f,-0.192999960303307460662836092524230480194091796875f,0.34599996277689992751902536838315427303314208984375f,-0.101199992942810013119014911353588104248046875f,1.1567999894142155792309267781092785298824310302734375f,-1.3947999659597878352457200890057720243930816650390625f,0.660200004237890158975687882048077881336212158203125f,-0.51070000492334344244227395392954349517822265625f,0.5575999992132185667514931992627680301666259765625f,-0.2885000061392783976543796597979962825775146484375f,0.4980000112652778199162639793939888477325439453125f,0.0585999615430825571138484519906342029571533203125f,1.07860003478229060647208825685083866119384765625f,0.2213000093668699985727243984001688659191131591796875f,0.574400043761730927371900179423391819000244140625f,-0.141400004705786841441295109689235687255859375f,-0.15380000173002483787598748676828108727931976318359375f,-0.58050000798702239279691639239899814128875732421875f,0.05059999743998044863246832392178475856781005859375f,-0.1172000006839633012845069970353506505489349365234375f,-0.6051000096440315445533997262828052043914794921875f,-0.0151999955952166754258314540493302047252655029296875f,0.0419000017032027383123704566969536244869232177734375f,-0.1679000019192695702940909541212022304534912109375f,-0.2349999959021806805736787282512523233890533447265625f,-0.09229999799728394460629488094127736985683441162109375f,-0.75160000141859040212466425145976245403289794921875f,-0.112200012865662390737497844384051859378814697265625f,-0.07710000195503230457916288287378847599029541015625f,0.298599983596801621388294734060764312744140625f,-0.176799999547004649258497011032886803150177001953125f,-0.087199998039007187156101963410037569701671600341796875f,-0.609200001674890412317608934245072305202484130859375f,-0.154800018709897901914018802926875650882720947265625f,-0.2565000029131769831991505270707421004772186279296875f,-0.6309000295758249166055975365452468395233154296875f,0.7726000182121992576611546610365621745586395263671875f,0.1718000049486756442007617806666530668735504150390625f,0.3245000310540202548281740746460855007171630859375f,0.64180001660883423397763181128539144992828369140625f,0.27530000382363795097262482158839702606201171875f,0.1656000209093093644696637056767940521240234375f,0.554799988162517632162007430451922118663787841796875f,0.054500003002583952138593303970992565155029296875f,-0.20669998155832303154966211877763271331787109375f,-0.5166000368088490279916413783212192356586456298828125f,-0.3015000147968531774012035384657792747020721435546875f,-0.6696000478506096698083638329990208148956298828125f,-0.503800005492567759546318484353832900524139404296875f,-0.0334999990612268039313903500442393124103546142578125f,0.0887999879837035877017115126363933086395263671875f,-0.62940000390112393890973407906130887567996978759765625f,-0.19260000074058769570228832890279591083526611328125f,-0.3893000144481657542883112910203635692596435546875f,0.03740001904070400229329607100225985050201416015625f,-0.722100022816658082547291996888816356658935546875f,1.420400023376941600616873984108678996562957763671875f,-1.719400017908215172468544551520608365535736083984375f,0.80190003303289447700308301136828958988189697265625f,0.230899992620944516374947852455079555511474609375f,-0.0532000054448840131726683466695249080657958984375f,0.10570000447630878426252820645458996295928955078125f,0.575899998581409278131104656495153903961181640625f,-0.30839999940097317931986253825016319751739501953125f,0.289900010323524526256733224727213382720947265625f,0.07470001696348216313481316319666802883148193359375f,0.2937999958813188872142063701176084578037261962890625f,0.02140000284314158562892771442420780658721923828125f,0.159000014349818030723326955921947956085205078125f,0.4975999828219415110908130372990854084491729736328125f,0.0422000052288173943537685772753320634365081787109375f,-0.3808000232100479109931256971322000026702880859375f,-0.227199971440434911329475653474219143390655517578125f,-0.175600007796287371775179053656756877899169921875f,0.158800004971027419742313213646411895751953125f,0.320799994450807535173453288734890520572662353515625f,0.052800002333521778297154014580883085727691650390625f,0.2882999722480779070110656903125345706939697265625f,-0.433599970334768602953090521623380482196807861328125f,0.149700002360344086582699674181640148162841796875f,1.07390000895261739088937247288413345813751220703125f,-1.1324000065237280754359971979283727705478668212890625f,0.5967000176191330496067166677676141262054443359375f,0.908200031745433733476602355949580669403076171875f,-0.32800000850856303458158436114899814128875732421875f,0.475400012350082334933176753111183643341064453125f,1.1166999645709996258347018738277256488800048828125f,-0.887399956381321519671701025799848139286041259765625f,0.595999994516372311181839904747903347015380859375f,-0.31140003994703402412369541707448661327362060546875f,0.1908000167280439196559882475412450730800628662109375f,-0.1800000125169756159948519780300557613372802734375f,-0.9501999981284132701375710894353687763214111328125f,0.922400008013843919485452715889550745487213134765625f,-0.516200014400482132259639911353588104248046875f,-0.762100001060962650711871901876293122768402099609375f,0.76139998801648622883675443517859093844890594482421875f,-0.42520000528097146030859221355058252811431884765625f,0.374600040996075023258526925928890705108642578125f,-0.2831999927043913345414694049395620822906494140625f,0.1424000009357928586695152262109331786632537841796875f,-0.07789999867826702395490201524808071553707122802734375f,-0.32089998755454995915670224349014461040496826171875f,-0.3580000167787071863045866848551668226718902587890625f,-0.2689999989271163460813340861932374536991119384765625f,-0.2092000074863431535732161137275397777557373046875f,0.3285999926120037883947588852606713771820068359375f,0.16999999444931734871033768285997211933135986328125f,0.00149999922513899264231440611183643341064453125f,0.18439998750984687436726972009637393057346343994140625f,-0.054499998271465288635084789348184131085872650146484375f,-0.162099997633695647181184540386311709880828857421875f,0.69500000305473808026590631925500929355621337890625f,0.195400002254545679392094825743697583675384521484375f,0.846500030815601434142081416212022304534912109375f,-0.86399998290836788061852757891756482422351837158203125f,-0.215999995298683578059950605165795423090457916259765625f,-0.61320000544488362237416367861442267894744873046875f,0.0097999757200479820085092796944081783294677734375f,-0.0248000016853213711698344923206605017185211181640625f,0.5549000130057333279864906216971576213836669921875f,-0.882800028336048203669861322850920259952545166015625f,-0.1309000064417720210485640564002096652984619140625f,-0.24410003479719222241328679956495761871337890625f,-0.4056000267207624521148545682081021368503570556640625f,-0.18900000689923768248945634695701301097869873046875f,-0.153400020933151637336777639575302600860595703125f,0.18700002114474789660647502387291751801967620849609375f,0.222200005526840715930347869289107620716094970703125f,0.4285000213384630995960833388380706310272216796875f,0.71000001117587085985860539949499070644378662109375f,0.3625000014528632075183622873737476766109466552734375f,0.8761000196576116394453492830507457256317138671875f,0.52379999804198751345296614090329967439174652099609375f,0.0825000016391277402050263845012523233890533447265625f,0.440200016009807626460315077565610408782958984375f,0.1688000092923640504949389651301316916942596435546875f,0.122200000348687165541150534409098327159881591796875f,0.3194000082969665754717425443232059478759765625f,-0.7817999803617592391447033151052892208099365234375f,-0.1619999926649033084036233276492566801607608795166015625f,-0.484900000749527659849036353989504277706146240234375f,0.69240001375079174295024131424725055694580078125f,0.0396000088095664892762215458787977695465087890625f,0.5190999859958882467481089406646788120269775390625f,-0.4844000028580424110913327240268699824810028076171875f,0.2751000052854417621261973181390203535556793212890625f,-0.37200004744529646671935552149079740047454833984375f,-0.2479999690949921120619592329603619873523712158203125f,-0.1816000112533568966455277404747903347015380859375f,0.51190000442266381242006900720298290252685546875f,-0.0037999965518711320555667043663561344146728515625f,0.258199996280669807902086176909506320953369140625f,0.47009996818155119768789518275298178195953369140625f,-0.55619996317774111904697065256186760962009429931640625f,0.25699999426677810898667075889534316956996917724609375f,0.550499977886676816751787555404007434844970703125f,-1.11979997974336153987451325519941747188568115234375f,0.3439999980330465945144169381819665431976318359375f,-1.010999966800212757078725189785473048686981201171875f,1.28899995034933123605469518224708735942840576171875f,-0.5780999935656783161874727738904766738414764404296875f,-0.36930006541013682408447493799030780792236328125f,-0.4007999525040393251629211590625345706939697265625f,-0.156700028944015912202303297817707061767578125f,-1.3467000182151789289264343096874654293060302734375f,1.508200022208689983216345353866927325725555419921875f,-0.7484000310659411070446367375552654266357421875f,-0.53240004251003281154908108874224126338958740234375f,0.1678000252068041486808169793221168220043182373046875f,-0.2789000187873842406816038419492542743682861328125f,-0.070499964177608287485554683371447026729583740234375f,0.31759997827708719153605443352716974914073944091796875f,-0.03949999499320977491834128159098327159881591796875f,0.482500020265579365741359652020037174224853515625f,-0.45999999418854731203509800252504646778106689453125f,0.282300002765655477787731797434389591217041015625f,0.585199976670742305628891699598170816898345947265625f,-1.02039997487366207451486843638122081756591796875f,0.34820000045299526192366101895458996295928955078125f,0.01920001642703983435467307572253048419952392578125f,-0.0118000134050844085464859745115973055362701416015625f,0.0173000039577484443498178734444081783294677734375f,-0.97969996406435910785148735158145427703857421875f,1.27339995790123960972550776205025613307952880859375f,-0.55879999442845551538283643822069279849529266357421875f,0.924100004994869550500879995524883270263671875f,0.18540000724643468377195176799432374536991119384765625f,0.22419998841285726509653386528952978551387786865234375f,0.013400001934170735484741499021765775978565216064453125f,0.269100010210275630839760196977294981479644775390625f,0.448200009766221096896288145217113196849822998046875f,0.196200001995265493892617314486415125429630279541015625f,0.859500022560358178935757678118534386157989501953125f,0.5215999877095220949030363044585101306438446044921875f,0.086499997176229914686018673819489777088165283203125f,0.14109999998807865040362230502068996429443359375f,0.3248000002324580304247092499281279742717742919921875f,0.02499999910593031504646432949812151491641998291015625f,0.56620001589059842928008947637863457202911376953125f,-0.337199980902671558880001612124033272266387939453125f,-0.3041999949693678928497320157475769519805908203125f,-0.90880000586509623872188967652618885040283203125f,-0.2090000200122599238738985150121152400970458984375f,-0.16960000765472660333443855051882565021514892578125f,0.32270000528097142478145542554557323455810546875f,0.214000005930662151598653508699499070644378662109375f,0.10790000106245278832517442424432374536991119384765625f,0.63310001912117019884362889570184051990509033203125f,-0.707800032880902652010490783140994608402252197265625f,-0.22760001091063031264383198504219762980937957763671875f,-0.31430003937035866368177039475995115935802459716796875f,-0.3756000016868112911794241881580092012882232666015625f,-0.08609999600201845293412361570517532527446746826171875f,-0.81209999957084644250926430686376988887786865234375f,0.29859999812543380759422007031389512121677398681640625f,0.0811999979905783930700380324196885339915752410888671875f,0.010499992918222844640041557795484550297260284423828125f,1.28180003568232070421117896330542862415313720703125f,0.3856000058963895416042078068130649626255035400390625f,1.4461000452876096034060537931509315967559814453125f,0.385199986952543316220953784068115055561065673828125f,-0.047099999310076245961909080506302416324615478515625f,-0.364699998486042176182309049181640148162841796875f,0.33080004384815719919998855402809567749500274658203125f,0.246100009524822305184699189339880831539630889892578125f,0.41250003330409601431938426685519516468048095703125f,0.109999989569187217597345807007513940334320068359375f,-0.16159999955594539500225437222979962825775146484375f,-0.62780000479221342857272247783839702606201171875f,0.6595999967396262153584984844201244413852691650390625f,0.6927000270739200882275099502294324338436126708984375f,0.2022999636873610285192626179195940494537353515625f,-0.56939996102303325198334960077772848308086395263671875f,0.13219998852089032137513413545093499124050140380859375f,0.0993000002920627622415850055404007434844970703125f,-0.579999998733401245232244036742486059665679931640625f,0.09390000075697901849025583942420780658721923828125f,0.79389997766017916802638865192420780658721923828125f,-0.8507999671816828257675524582737125456333160400390625f,0.43949999827146513098341529257595539093017578125f,0.16419997611939862736107897944748401641845703125f,-0.8883999947816130227096209637238644063472747802734375f,0.1332000003412365796151561880833469331264495849609375f,0.1822000220298765071902380441315472126007080078125f,-0.097000040888785665771365529508329927921295166015625f,0.075700027871132391510400339029729366302490234375f,-0.53050005894899499025996192358434200286865234375f,-0.4177999933928244757908032624982297420501708984375f,-0.242400018072128631274608778767287731170654296875f,-0.118500005990266998878723825328052043914794921875f,-0.326800002825260005323571022017858922481536865234375f,-0.031700002792477643964730305015109479427337646484375f,-0.94070003476291930155639420263469219207763671875f,0.748600019583105957821089759818278253078460693359375f,-0.5130000229999425176430349893053062260150909423828125f,-0.051899993768333985144636244513094425201416015625f,0.8629999980777498524986413031001575291156768798828125f,-0.0810000056549908808989357567043043673038482666015625f,0.490900006832927449096359850955195724964141845703125f,-0.31819999158680445372482381571899168193340301513671875f,0.26120000547841193405673720917548052966594696044921875f,1.086700025665759739013083162717521190643310546875f,-1.90160001713931503530830013914965093135833740234375f,0.660900028681755458137558889575302600860595703125f,0.9712999692082409097793060936965048313140869140625f,-0.866399962610006735275192113476805388927459716796875f,0.51830000041127188836753703071735799312591552734375f,0.18770006667375549458398609203868545591831207275390625f,0.11039994350671822331833027419634163379669189453125f,0.0991000250369313828713302427786402404308319091796875f,0.71799997445940988427537376992404460906982421875f,-0.615799978357553623453668478759936988353729248046875f,0.374100004175305311804322627722285687923431396484375f,0.63739997873306375453239525086246430873870849609375f,1.1444000422716147369328609784133732318878173828125f,-0.1476000041306020360565298688015900552272796630859375f,-0.2325000029429793446666963063762523233890533447265625f,-0.4865000111460684451003544381819665431976318359375f,-0.590400015926360932638772283098660409450531005859375f,-0.2631000015005469094830914400517940521240234375f,-0.17330001367330538641908788122236728668212890625f,-0.687800016042590112164134552585892379283905029296875f,-0.24760000334829090906652027115342207252979278564453125f,-0.92870003883838680991402725339867174625396728515625f,-0.667400021275878163606876114499755203723907470703125f,-0.24490000589042904000081080084783025085926055908203125f,-0.47920002711862352828831035367329604923725128173828125f,0.2840000202357770842098716457257978618144989013671875f,0.14290000523477797145233125775121152400970458984375f,0.388700020062923812247390742413699626922607421875f,-1.17559999557733529940151129267178475856781005859375f,-0.3211000009194016069358212916995398700237274169921875f,-1.0251000340819356182464616722427308559417724609375f,-0.0272000042974946953933113036327995359897613525390625f,-0.11139999680817125327081384966732002794742584228515625f,-0.36659999426603295802351567544974386692047119140625f,0.16800000411272042466492848689085803925991058349609375f,-0.0030000001639127749086810581502504646778106689453125f,-0.04899999803304666556869051419198513031005859375f,-0.2138000193506481849681222229264676570892333984375f,-0.0214000080958009208931258626762428320944309234619140625f,0.4500999799162144210384894904564134776592254638671875f,-0.028399999177455725885010906495153903961181640625f,0.19070000063925984790103029808960855007171630859375f,0.47749999195337267110517132095992565155029296875f,0.3644000093400474593607896167668513953685760498046875f,0.280299996782839233588902061455883085727691650390625f,0.55430000818967783970947493799030780792236328125f,-0.2085999985724686922594628413207828998565673828125f,0.03490000011622906850305980697157792747020721435546875f,0.38760000189542775927975526428781449794769287109375f,0.21119998407661932926515646613552235066890716552734375f,-0.085200000326335401101829347680904902517795562744140625f,-0.168000011414289662781129663926549255847930908203125f,0.6108000330448153381013298712787218391895294189453125f,0.277800008517503804750248264099354855716228485107421875f,0.622400033122301632459993925294838845729827880859375f,1.056600015202164843941545768757350742816925048828125f,0.275300023865700183023363933898508548736572265625f,0.18709998526573112798132569878362119197845458984375f,-0.4192000094234937801473961371812038123607635498046875f,0.1013000094413758489508836646564304828643798828125f,-0.8161000360488888105692240060307085514068603515625f,0.270400014734268001603822995093651115894317626953125f,-0.426700014042854292029005591757595539093017578125f,-0.4556999998569484233712501008994877338409423828125f,0.923600006619095470483671306283213198184967041015625f,-0.2897000050425528883124570711515843868255615234375f,-0.7426000248059632102837213096790947020053863525390625f,0.6680000082105397307685734631377272307872772216796875f,-0.4134000147119165635700710481614805757999420166015625f,0.16650002455711376825320257921703159809112548828125f,-0.2118000181734560616320095505216158926486968994140625f,0.101600010955333885931395343504846096038818359375f,-1.226899965977668927052945946343243122100830078125f,1.73299995344877277858586239744909107685089111328125f,-0.71499999642372102925946819595992565155029296875f,0.26319998048543968849344310001470148563385009765625f,-0.2305999858915808164994132312131114304065704345703125f,0.1342999986529349598640692420303821563720703125f,0.337000003486871424485116222058422863483428955078125f,-0.35800000634789441722460878736455924808979034423828125f,0.184000008389353819637790365959517657756805419921875f,-0.695700023624301167046724003739655017852783203125f,0.1572000177830457090522031649015843868255615234375f,-0.347300016549229706441792586701922118663787841796875f,-0.4770999781131735062444931827485561370849609375f,0.633800010931491186738639953546226024627685546875f,-0.2568000084877013478035223670303821563720703125f,0.13110002949237742342347701196558773517608642578125f,0.1111999808728698635462706079124473035335540771484375f,0.0737000024795531061272413353435695171356201171875f,-0.66639999792575821402351721189916133880615234375f,0.4149999954551457648221912677399814128875732421875f,-0.3449000082373618880637877737171947956085205078125f,0.53579996870905244321647842298261821269989013671875f,-0.74219995687454998201104672261863015592098236083984375f,0.29789999817535284254432781381183303892612457275390625f,0.52420004280209564484493967029266059398651123046875f,-0.4750000256299971024276374009787105023860931396484375f,0.29830001978278186669513161177746951580047607421875f,0.9219000050187116102051732013933360576629638671875f,-0.308299989682435882087929712724871933460235595703125f,-0.457600037658214997549066538340412080287933349609375f,-0.29810001073926695180915658056619577109813690185546875f,-0.421900032883883113754563964903354644775390625f,-0.0465999881565573315089778816400212235748767852783203125f,0.078399998432397810521621295265504159033298492431640625f,0.02510000122487543450233715702779591083526611328125f,0.287200003254413527287169927149079740047454833984375f,0.0034999996945261901970525286742486059665679931640625f,0.1269000074028969748951567453332245349884033203125f,-1.0370000269562009709289895909023471176624298095703125f,-0.2530000103339553607639800247852690517902374267578125f,-0.4091000395655637333902632235549390316009521484375f,0.932199973228573508521321855369023978710174560546875f,0.288199992518126890672647277824580669403076171875f,0.94229999650716678161188610829412937164306640625f,-0.6248000318229198768449350609444081783294677734375f,-0.319600006723403939901118064881302416324615478515625f,-0.419600035333633769596417550928890705108642578125f,0.4007999959409236367946505197323858737945556640625f,0.012599998691678049311803988530300557613372802734375f,-0.395800013375282588867776212282478809356689453125f,1.1222000018388029740634692643652670085430145263671875f,0.25029999696910376627556615858338773250579833984375f,0.561600012445449436881972360424697399139404296875f,0.878800008547306443773550199694000184535980224609375f,0.20810000649243598491722195831243880093097686767578125f,0.5173000261604787652913728379644453525543212890625f,0.2016000179588797180940673570148646831512451171875f,0.080400006687641190072923791376524604856967926025390625f,-0.061599982196092195607661778922192752361297607421875f,0.74560000153779970588630021666176617145538330078125f,0.20479999829530715071967961193877272307872772216796875f,0.16079999519586518630376303917728364467620849609375f,0.11579999869763846209735902448301203548908233642578125f,0.069599997875839481054072166443802416324615478515625f,0.347099994150548984972459720665938220918178558349609375f,-0.79040001726746567101145046763122081756591796875f,-0.36130000456124544694347378026577644050121307373046875f,-1.12270003836154952381320981658063828945159912109375f,0.3181999994099140582903828544658608734607696533203125f,0.232300001461803906632752614314085803925991058349609375f,0.38800000746548146679515411960892379283905029296875f,-0.888800023597479338377524982206523418426513671875f,-0.48429997745603348136000931845046579837799072265625f,0.495500008508563016818015967146493494510650634765625f,-0.59990001740157594678493069295655004680156707763671875f,-0.27090000629276034249670601639081723988056182861328125f,0.5360000085234641886700046597979962825775146484375f,-0.625999993920326236462869928800500929355621337890625f,-0.3875999954134226044288880075328052043914794921875f,0.6970000011399388739619098487310111522674560546875f,-0.090099992805719342214842981775291264057159423828125f,0.68379998809546227445110844200826250016689300537109375f,-0.59199998277425780468519178612041287124156951904296875f,0.80979998583421120006420323989004828035831451416015625f,-0.61539997487366215889181830789311788976192474365234375f,0.53999999940395326802899944595992565155029296875f,-0.56489998379945749462649473571218550205230712890625f,0.3146000125721093443331710659549571573734283447265625f,-0.4185000088214874125469577847979962825775146484375f,-0.74010000934600839173072017729282379150390625f,-0.96609999965280268785505768391885794699192047119140625f,0.693999994382262119785309550934471189975738525390625f,-0.08979999285638340378312705070129595696926116943359375f,0.0615999959051609646820679699885658919811248779296875f,0.62550000679492967492478783242404460906982421875f,0.6336000160813330950304589350707828998565673828125f,0.49729999226033694270654450519941747188568115234375f,0.568499997794628342262512887828052043914794921875f,-0.091199986982345659924931169371120631694793701171875f,0.38989998409748094498894488424411974847316741943359375f,0.06699999957531688199452446497161872684955596923828125f,0.0005000002570450245986677373366546817123889923095703125f,-0.010999999694526196858390676425187848508358001708984375f,-0.2074999862909319148229769780300557613372802734375f,0.007700000065565149043322890065610408782958984375f,-0.79409997131824516802822699901298619806766510009765625f,-0.3934999819993976188925444148480892181396484375f,1.06639998779296885089706847793422639369964599609375f,-0.5600999658346179632673056403291411697864532470703125f,-0.32399999170005333581912054796703159809112548828125f,0.002099994206428590359791996888816356658935546875f,-0.449199983495474020056548170032328926026821136474609375f,0.31800001692771928674119408242404460906982421875f,0.78750000894069671630859375f,0.206600001828372459300453556352294981479644775390625f,0.5184999957084652777439259807579219341278076171875f,-0.12570000402927394844709851895458996295928955078125f,0.1869000080734490865097541245631873607635498046875f,-0.03990002060532571448447924922220408916473388671875f,-0.5223000163257121020166096059256233274936676025390625f,0.483300041687488857178323087282478809356689453125f,1.561500005483626551949782879091799259185791015625f,-0.034900014793873168628124403767287731170654296875f,0.1604999960660933311373810283839702606201171875f,0.78119997744560265573454671539366245269775390625f,0.1328000056862830291493082768283784389495849609375f,-0.097099985563755097928151371888816356658935546875f,-0.902200029778480683262387174181640148162841796875f,-0.436000027298927506080872262828052043914794921875f,-0.45959998262077572661610247450880706310272216796875f,0.589499979570507814941038304823450744152069091796875f,0.51930000468790515899542015176848508417606353759765625f,-1.122500002384185791015625f,0.35000002384185791015625f,-0.02899999535083708224192378111183643341064453125f,0.7281000171840190660077496431767940521240234375f,-0.1560000317692757931808955618180334568023681640625f,-0.728800018978118746559857754618860781192779541015625f,1.5783000145673753422670415602624416351318359375f,-1.04129999752044710703557939268648624420166015625f,-0.1820999813914294662708925898186862468719482421875f,0.37070002183616157509504773770458996295928955078125f,1.52349995815753924688351617078296840190887451171875f,0.148899993902444460758260902366600930690765380859375f,0.40829999817609774481752538122236728668212890625f,-0.0723000280976293652202002704143524169921875f,0.5669000255823135603350237943232059478759765625f,-0.43410001531392339302328764460980892181396484375f,-0.80339998647421584365702074137516319751739501953125f,0.413200007121264700771234856802038848400115966796875f,0.68430001772642157220616354607045650482177734375f,0.3039000161051745863005635328590869903564453125f,0.238700010824203445736202411353588104248046875f,-1.0403000192463398576592226163484156131744384765625f,0.5210000225305559951038958388380706310272216796875f,0.4284000105023384463720503845252096652984619140625f,0.14699997174739820593458716757595539093017578125f,0.92159999217987120800898992456495761871337890625f,0.55140002564191803458015783689916133880615234375f,-0.06720000668168069068997283466160297393798828125f,-0.64890001316219514393424105946905910968780517578125f,0.546999999538063885751171255833469331264495849609375f,-0.23500000558793543437019479824812151491641998291015625f,-0.018800003965198985866891234763897955417633056640625f,-0.3209999867677684193267850787378847599029541015625f,0.72359999268650998516250183456577360630035400390625f,-0.61160000868290620434919446779531426727771759033203125f,-0.297499988079071187030422152020037174224853515625f,0.42889998853802691058945129043422639369964599609375f,-0.278499983787536820045716012828052043914794921875f,0.6489999977350233706374638131819665431976318359375f,-0.60850001865625369390500054578296840190887451171875f,0.31719998518675562859669980753096751868724822998046875f,-0.3440000022426248360574163598357699811458587646484375f,0.579499974913895155026466454728506505489349365234375f,0.125199994663894198509979105438105762004852294921875f,0.8679999769926070030123810283839702606201171875f,0.395900013780593695855714031495153903961181640625f,0.7669999807998539864684062195010483264923095703125f,0.41749999776482571434144119848497211933135986328125f,-0.4727000085592270028911343615618534386157989501953125f,-0.35070000335872164587414090419770218431949615478515625f,-0.925000023245811320293796597979962825775146484375f,-0.92419998915791534699337717029266059398651123046875f,-0.93760000029355250461549076135270297527313232421875f,-0.478499979019165078142350466805510222911834716796875f,0.787799969848990588872084117610938847064971923828125f,-0.56189998736083512920203020257758907973766326904296875f,-1.046000004351139267555481637828052043914794921875f,0.073999981492757882506339228712022304534912109375f,-0.4065999848410488670680251743760891258716583251953125f,-0.2159999998435377932537448941729962825775146484375f,-0.20189999846220008095087905530817806720733642578125f,0.36620000016987308555371782858856022357940673828125f,-0.822000002443790123152211890555918216705322265625f,0.108600004607438904713490046560764312744140625f,-0.6026000009641048205111246716114692389965057373046875f,0.20600000196695322784989912179298698902130126953125f,0.625900007969141025654380428022705018520355224609375f,0.58060000157505253781664578127674758434295654296875f,-0.837499979734420918475734652020037174224853515625f,0.6481999982178212604821965214796364307403564453125f,-0.3814999656602744426692197521333582699298858642578125f,-0.4835000027120113230694187222979962825775146484375f,-0.53490003237724348394976914278231561183929443359375f,0.95169999377727521050474024377763271331787109375f,-0.8031999789953232493644463829696178436279296875f,-0.0703999905943870629698722041212022304534912109375f,-0.849299976301193026984037715010344982147216796875f,-0.833999967634678540662207524292171001434326171875f,0.4474000070452692767730695777572691440582275390625f,1.0160000439882281852987944148480892181396484375f,1.23160000350475229424773715436458587646484375f,-0.65260003288984336222711135633289813995361328125f,0.404500001698732347676923382095992565155029296875f,-0.2656000112235545884686871431767940521240234375f,0.075600009733438611192468670196831226348876953125f,0.279999975264072276104343472979962825775146484375f,1.927399997806549691858890582807362079620361328125f,-0.0103999774813647860582932480610907077789306640625f,0.089099997097253691435980726964771747589111328125f,-0.7854000129461287116328094271011650562286376953125f,0.250600025379658308821717582759447395801544189453125f,0.6642000347368421930838167099864222109317779541015625f,0.6909000102788203090398155836737714707851409912109375f,-0.09950000397115943417958305872161872684955596923828125f,-0.6735000057667495099167354055680334568023681640625f,0.24329999594092388548460803576745092868804931640625f,0.8767000413119798718497577283415012061595916748046875f,0.7669000173121691688038481515832245349884033203125f,0.719899989834427600499111576937139034271240234375f,-0.281100005725026136360611417330801486968994140625f,0.8442000146389005976743646897375583648681640625f,-1.252500057220458984375f,-0.223099999749660327097444678656756877899169921875f,0.41400000291317695655379793606698513031005859375f,1.28519997745305314396091489470563828945159912109375f,-0.526200016374886114789433122496120631694793701171875f,0.3020000173449517433255095966160297393798828125f,0.460900000369548479284276254475116729736328125f,0.3991000124454497921533402404747903347015380859375f,-0.4758000046767294488603283753036521375179290771484375f,1.0923000103652478198767994399531744420528411865234375f,0.53510001847296961852151753191719762980937957763671875f,-0.118600006246566636036732234060764312744140625f,0.5968000418663024220222723670303821563720703125f,0.630000035762787291560016456060111522674560546875f,-0.31780000602155933364656448247842490673065185546875f,0.207999993719160392036116036251769401133060455322265625f,-0.608500001072883378583355806767940521240234375f,-0.63709998183846483499337409739382565021514892578125f,-0.5438999929338692584934733531554229557514190673828125f,0.0469999642968179642821269226260483264923095703125f,-0.1252999976396562686886682058684527873992919921875f,0.66759996199756910240097340647480450570583343505859375f,0.09650001919269612926655099727213382720947265625f,0.234400000101327776746984454803168773651123046875f,-0.813500009305774707257796762860380113124847412109375f,-0.3975000064074993844087657635100185871124267578125f,-0.19470000429749489967434783466160297393798828125f,-0.26649999579787253534135516019887290894985198974609375f,-0.2044999854564668595457987976260483264923095703125f,0.966200013655424072567257098853588104248046875f,-0.53049996682256483904183141930843703448772430419921875f,-0.39050002869963673646225288393907248973846435546875f,-0.3499000040650368514860701907309703528881072998046875f,0.015299997937679332693505784845910966396331787109375f,-0.36550000418722594641707246410078369081020355224609375f,0.20350000135228019093602824796107597649097442626953125f,-0.5819000115752215318565276902518235146999359130859375f,-0.652500027939677540445018166792578995227813720703125f,0.55859999156892292671017230532015673816204071044921875f,0.26450000013411045785005626385100185871124267578125f,-0.20349999324977385839474663953296840190887451171875f,0.1371999977409839033271055086515843868255615234375f,-0.623299995791911864984058411209844052791595458984375f,-0.3260000165700915886191069148480892181396484375f,-0.9067000276029109073760992032475769519805908203125f,-0.356400000458955812376871108426712453365325927734375f,0.37650002191215747870955965481698513031005859375f,0.423599995964765607681101755588315427303314208984375f,-0.4094999858289958272195008248672820627689361572265625f,-0.750000003874301768291843472979962825775146484375f,0.32129999260306352226734816213138401508331298828125f,-0.384000000938772956260436330921947956085205078125f,-0.35549999845027924205709268790087662637233734130859375f,0.85530000030323860560343973702401854097843170166015625f,0.0481000065296888212884596214280463755130767822265625f,-0.74200001525878889196974341757595539093017578125f,0.5542000211954114519130598637275397777557373046875f,-0.14879999804198718038605875335633754730224609375f,-0.9480000276938083114686151020578108727931976318359375f,-0.1752999991297716775306980707682669162750244140625f,-0.0726999757766719767460017465054988861083984375f,2.23960004785060817766861873678863048553466796875f,-0.1810000225305561372124429908581078052520751953125f,0.06339999172687527106972993351519107818603515625f,-0.23949998784065229529005591757595539093017578125f,-0.584699991035460442390103708021342754364013671875f,-0.653000021696090726663896930404007434844970703125f,0.50750002980232267191240680404007434844970703125f,0.62050002914667157227768257143907248973846435546875f,1.28740003118515033975199912674725055694580078125f,0.8125999855041499131402815692126750946044921875f,-0.9478000328063966861691369558684527873992919921875f,-0.15150001204013818778548738919198513031005859375f,1.42110002293586745736320153810083866119384765625f,-0.00310000422000911157738300971686840057373046875f,-0.301100010865926748238052823580801486968994140625f,-0.6435999841928481401964745600707828998565673828125f,0.752400026118755338444543667719699442386627197265625f,-0.711400025492906706858775578439235687255859375f,-0.372199995654820980917065753601491451263427734375f,-0.27760002171397246684136916883289813995361328125f,0.610000006854534149169921875f,-0.312100011491775575223073246888816356658935546875f,0.09310001614093810218264479772187769412994384765625f,-1.291700038182735710279303020797669887542724609375f,-0.496599992775916376785971806384623050689697265625f,0.8502000326991083767325108055956661701202392578125f,0.27579998379647729933594746398739516735076904296875f,-0.78879998649358729068126194761134684085845947265625f,-0.27859999626278852957739218254573643207550048828125f,-0.25480001766681681374393519945442676544189453125f,-0.6577999988317486668165656737983226776123046875f,-0.3148000108122825935197397484444081783294677734375f,1.192800006282329405848940950818359851837158203125f,0.230999939680099686256653512828052043914794921875f,-0.6164000244498257785608075209893286228179931640625f,0.94740000555515280211693607270717620849609375f,-0.6768000400781630787605536170303821563720703125f,-0.6184000209331514952282304875552654266357421875f,-1.2483000163555146855287603102624416351318359375f,-0.45690000412464115697730449028313159942626953125f,0.8823000066399568908082073903642594814300537109375f,0.648299994581192695619620280922390520572662353515625f,0.75789999458044743363416273496113717555999755859375f,0.279000027537346539929785649292171001434326171875f,1.14330001903772338067710734321735799312591552734375f,-0.28960000735670288207757039344869554042816162109375f,0.721499971508979598411315237171947956085205078125f,-0.33809997531175639551292988471686840057373046875f,0.1186999925330283200963776835124008357524871826171875f,-0.333000006496906308939287555404007434844970703125f,0.608100014799833221701419461169280111789703369140625f,0.25900000719726079534410700944135896861553192138671875f,0.28500001847744016458818805404007434844970703125f,-0.14159999992847449590271935448981821537017822265625f,-0.8778000001728532009082073273020796477794647216796875f,0.108500026702881058326966012828052043914794921875f,0.7840000141263008259784328402020037174224853515625f,-0.1304999855607749115193882971652783453464508056640625f,-0.76050001141428946738187732989899814128875732421875f,-0.0278999992370605465197286321199499070644378662109375f,0.1697000009447335866497041934053413569927215576171875f,0.893500019699335013001473271287977695465087890625f,-0.15279999510645847493606197531335055828094482421875f,0.642300003920495132803125670761801302433013916015625f,-0.5725000146031380410249767010100185871124267578125f,-0.253700003373622973157353044371120631694793701171875f,-0.4124999976530669965057995796087197959423065185546875f,-0.1555000133514408133805773104541003704071044921875f,0.27729999993443499306522426195442676544189453125f,0.84790000750720462274756528131547383964061737060546875f,0.165499993532896017001121435896493494510650634765625f,-0.5677999820679426878911044695996679365634918212890625f,-0.0080999971047044094785150036841514520347118377685546875f,0.6055000038146971519381622783839702606201171875f,-0.40659999322295181656272688996978104114532470703125f,0.3651000010386107330617733168764971196651458740234375f,-0.0764999697208406104209643672220408916473388671875f,-0.3198000249683854434579188819043338298797607421875f,-0.78009997712224754540244475720101036131381988525390625f,0.1130000115633009727389435283839702606201171875f,-0.755000014007091380108249722979962825775146484375f,-0.4457999919176103276186040602624416351318359375f,-0.229999992847442769061672152020037174224853515625f,0.30079998014569309106036598677746951580047607421875f,0.80260000051706992252320560510270297527313232421875f,-0.57600001718476423473447312062489800155162811279296875f,-0.372800003041326599717564249658607877790927886962890625f,-0.10870000784397149118376546539366245269775390625f,1.834299963784218334694742225110530853271484375f,0.0533000139713291076759560382924973964691162109375f,0.611999997526407213399579632095992565155029296875f,0.551399984365701811839244328439235687255859375f,-0.412400016134977676074413466267287731170654296875f,-0.567100017899274888577565434388816356658935546875f,1.9441000175118450243871848215349018573760986328125f,-0.005299988400935706778227540780790150165557861328125f,-0.0420000194311143104641814716160297393798828125f,-0.579599998974799746065400540828704833984375f,-0.65940001217126820165503886528313159942626953125f,-0.2931999997079373088126885704696178436279296875f,0.21759998982548722779029048979282379150390625f,-0.49000001803040493797425369848497211933135986328125f,-0.712100002551078858914479496888816356658935546875f,0.43759999980926522766822017729282379150390625f,0.15420000092983254802447845577262341976165771484375f,0.628499974548816453534527681767940521240234375f,0.07349997395277085843190434388816356658935546875f,0.35350002789497381172623136080801486968994140625f,1.01559999289512603581897565163671970367431640625f,0.634399930810928935898118652403354644775390625f,-1.0942000238776206089141851407475769519805908203125f,-0.60760001277327546631568111479282379150390625f,-1.594699991780519354733769432641565799713134765625f,0.575500024527311637712045921944081783294677734375f,0.5510000091195106364239109097979962825775146484375f,-0.628700022447109319045921438373625278472900390625f,-0.330300014925003271315517849870957434177398681640625f,-0.1405000093281270068246158189140260219573974609375f,0.6387999921560290061961495666764676570892333984375f,0.37620001648664480597972215036861598491668701171875f,1.417100025536119911606647292501293122768402099609375f,-0.383300056588649962208137367269955575466156005859375f,-0.7165000285059217599581415925058536231517791748046875f,0.178000020205974607279131305404007434844970703125f,1.423699979829788020424530259333550930023193359375f,-0.0467000056982040945285916677676141262054443359375f,-0.7541999869227407060634504887275397777557373046875f,-0.7412999574363237087482048082165420055389404296875f,0.35289999789595594137381340260617434978485107421875f,-0.4602000228643419177387841045856475830078125f,-0.02870000093355773440606526492047123610973358154296875f,0.00050000739097555424450547434389591217041015625f,1.01439998713731771573520745732821524143218994140625f,-0.2440999776512386976179413977661170065402984619140625f,0.0845000255405903288874469581060111522674560546875f,0.033399997836351502655816148035228252410888671875f,-0.45349998397380131276435122344992123544216156005859375f,0.41650000801682462991948341368697583675384521484375f,1.1426000175416468618294629777665250003337860107421875f,0.0847000136107205481295068238978274166584014892578125f,0.89949999916553480261427466757595539093017578125f,-0.523599984937906270943130948580801486968994140625f,0.269500001586973514378087202203460037708282470703125f,0.8590000023543833407302372506819665431976318359375f,0.006799999547004720312770587042905390262603759765625f,0.2813999950200318789228504101629368960857391357421875f,0.3229999975562094505221466533839702606201171875f,0.424299996268749257666286212042905390262603759765625f,0.41319999587088818149283042657771147787570953369140625f,-0.92999999195337323953935992904007434844970703125f,0.0359999934732915249924189993180334568023681640625f,-0.54699996686726848338366835378110408782958984375f,0.67950002300739242855343036353588104248046875f,-0.23280001567006092244582760031335055828094482421875f,-0.6176999727591876432342132829944603145122528076171875f,-0.296000030428171356788880075328052043914794921875f,-0.53350000062584879145788363530300557613372802734375f,0.593199988904595354455295819207094609737396240234375f,0.0285000045001506663311374722979962825775146484375f,-0.5541000106573104044827005054685287177562713623046875f,-0.225500000573694736605290245279320515692234039306640625f,-0.39299999666214002758124479441903531551361083984375f,0.49469999476075177113898462266661226749420166015625f,-0.085999987028539337519106311447103507816791534423828125f,-0.1079999847412109748034936274052597582340240478515625f,-0.7242000259265302108957484961138106882572174072265625f,-0.8181999936357140512654950725845992565155029296875f,-0.0784999903440475321758640347979962825775146484375f,0.706099985980987554512466886080801486968994140625f,-0.36249999675899735596118489411310292780399322509765625f,0.09999998345971139457333265454508364200592041015625f,-0.12669999079704297173520899377763271331787109375f,0.6023999708354477178318120422773063182830810546875f,0.166999978899955436872915015555918216705322265625f,0.3459000221252441775732222595252096652984619140625f,0.636600000226497542143988539464771747589111328125f,-0.787200027990341055783574120141565799713134765625f,-0.2285999964118001770430055330507457256317138671875f,0.1777999998748303056572694913484156131744384765625f,0.5938000130176543933657740126363933086395263671875f,0.21680001742839838385634720907546579837799072265625f,-1.48990003178119678750590537674725055694580078125f,1.02160001602172911816523992456495761871337890625f,0.177600005471706623438876704312860965728759765625f,0.66290002337694176048898953013122081756591796875f,1.39689997938871357519019511528313159942626953125f,-0.1951000093460084627849937533028423786163330078125f,1.19139998898506149771492346189916133880615234375f,-0.2377000270366664835819392465054988861083984375f,-0.0564999709129327953860411071218550205230712890625f,-1.41500000655651092529296875f,0.5669000255823135603350237943232059478759765625f,0.38910000141859057976034819148480892181396484375f,-0.98539998403787620873117703013122081756591796875f,1.360699980187416002763711730949580669403076171875f,0.17549999725818654638942462042905390262603759765625f,-0.575499987572431592752764117904007434844970703125f,1.0977999869108199249012614018283784389495849609375f,-0.125800013673305510764066639239899814128875732421875f,0.02399999952316278495345613919198513031005859375f,-1.926300010633468673404422588646411895751953125f,0.2533000062227248605495333322323858737945556640625f,-0.868700002181530095413108938373625278472900390625f,0.4219999923110009376614470966160297393798828125f,0.17259999474287024412433311226777732372283935546875f,-1.6304999799728392417819122783839702606201171875f,0.85110003068447070972979418002068996429443359375f,0.3309000096082694852839267696253955364227294921875f,0.416200016635656311336788348853588104248046875f,1.411699972468614561194044654257595539093017578125f,-0.36029999600052775576841668225824832916259765625f,-0.2375999838650226791969544137828052043914794921875f,0.4304999752044675886963887023739516735076904296875f,0.42429999358653969210308787296526134014129638671875f,-0.53829999340772616278627538122236728668212890625f,0.417700022864341491413142648525536060333251953125f,-0.04830001426935215391722522326745092868804931640625f,-0.15339999530315395759316743351519107818603515625f,-0.4998999759018423194589786362485028803348541259765625f,-0.1675000095367433772253207280300557613372802734375f,-0.6206000079452991258222027681767940521240234375f,-0.28269999540895229817039080444374121725559234619140625f,-0.41950000512599938673474753159098327159881591796875f,0.354400008594989657240148517303168773651123046875f,-0.277099995063245174709010143487830646336078643798828125f,0.865500005602836353091333876363933086395263671875f,0.894900010770559362072162912227213382720947265625f,0.8773000058203932471911912216455675661563873291015625f,-0.2429999844431875999362091533839702606201171875f,0.54659998815655708170879734097979962825775146484375f,-0.6807999927744268209295341875986196100711822509765625f,-0.80750001192092923929521930404007434844970703125f,-0.37869999920129782111644090036861598491668701171875f,-0.8182999769046903626446010093786753714084625244140625f,0.5349999964237213134765625f,0.71030000434517859275729279033839702606201171875f,0.85499998740851879119873046875f,0.4924999943375585331750698969699442386627197265625f,0.975699980783462539335459950962103903293609619140625f,0.451299989585578487094608135521411895751953125f,0.58550000515580169491158812888897955417633056640625f,0.6534999888539314572000193948042578995227813720703125f,0.2604999934583902554408041396527551114559173583984375f,-0.3395000188350678627102752216160297393798828125f,-0.94270001094341271397070158855058252811431884765625f,0.2564999898746609385824513083207421004772186279296875f,0.415000002980232096660984097979962825775146484375f,0.452899989402294256279901674133725464344024658203125f,0.52639999580234275722290249177603982388973236083984375f,0.769499962806701631734540569595992565155029296875f,0.88380000586509677162894149660132825374603271484375f,0.97649996323883581084146499051712453365325927734375f,0.156499992519616881736510549671947956085205078125f,0.54669999377727496181478272774256765842437744140625f,-0.467899995847046223929055486223660409450531005859375f,-0.38299998772144316916410389239899814128875732421875f,0.41929997153282183575129238306544721126556396484375f,0.010199999618530186662468395297764800488948822021484375f,0.4600000062584879145788363530300557613372802734375f,0.311000006586313215706240953295491635799407958984375f,0.33259998814910662900956594967283308506011962890625f,-0.852499988079070902813327847979962825775146484375f,-0.22790001622438449402352489414624869823455810546875f,-0.7456000116705894242841168306767940521240234375f,-0.317299969387054403568981797434389591217041015625f,0.72810002716779731457563684671185910701751708984375f,1.329200047123432426587896770797669887542724609375f,-0.084200019407273174465444753877818584442138671875f,-0.3964000256419186740686200209893286228179931640625f,1.20010001769065866028540767729282379150390625f,-1.3060000207424167228964506648480892181396484375f,-0.268800015103817457884360919706523418426513671875f,0.02839998040199276374551118351519107818603515625f,0.87190001008510620295055559836328029632568359375f,0.80990002522468529377874801866710186004638671875f,-0.916199986684322453811546438373625278472900390625f,0.732899975395202574190989253111183643341064453125f,0.2186999908566473749260694603435695171356201171875f,0.9965999886035916688342695124447345733642578125f,0.734399954652786846054368652403354644775390625f,-1.017200002956389681685323012061417102813720703125f,-0.41510001381635674988501705229282379150390625f,-1.6716999928832052546567865647375583648681640625f,0.2575000166893008923807428800500929355621337890625f,0.153000021696090726663896930404007434844970703125f,-1.03320002012252842860107193700969219207763671875f,-0.11980001140832907680078278644941747188568115234375f,0.291000001966953192322762333787977695465087890625f,-0.770700021314620897783242980949580669403076171875f,-0.689300017130374698126615840010344982147216796875f,-1.24470001249313355629055877216160297393798828125f,0.40230002601146708229862269945442676544189453125f,0.53330000562667834174135350622236728668212890625f,-0.473000001505017309000322711654007434844970703125f,-0.2073999959439039031394713674671947956085205078125f,-0.07760000256597976431294227950274944305419921875f,-1.31009996523857097372456337325274944305419921875f,-0.38429998285770494703683652915060520172119140625f,0.65509999384880046591206337325274944305419921875f,-0.71730001617670069435916957445442676544189453125f,-0.085499989509582263735865126363933086395263671875f,1.13490004519224196855020636576227843761444091796875f,0.9878999968528745512230671010911464691162109375f,-0.698400049543380418981541879475116729736328125f,-0.342799997341632689540347200818359851837158203125f,1.2544000255823135603350237943232059478759765625f,0.050599991069734173709804281315882690250873565673828125f,-0.4010000180602073527325046597979962825775146484375f,0.224300009232759389732336785527877509593963623046875f,0.4441999937772751483322508647688664495944976806640625f,-0.4644999914169307686506726895458996295928955078125f,-0.4394000167906284559649066068232059478759765625f,-0.87290001067370137644019223444047383964061737060546875f,-0.52949999439716322058302466757595539093017578125f,-1.0203000123918055663807535893283784389495849609375f,-0.59770000095963471409277190105058252811431884765625f,-1.088499979019164953797371708787977695465087890625f,0.7326999996185303842821667785756289958953857421875f,-0.3412999734178188138145060293027199804782867431640625f,0.3880000008344648421143574523739516735076904296875f,0.643099975162744641465906170196831226348876953125f,0.49839999053478234447567274401080794632434844970703125f,0.59200000214576675716671161353588104248046875f,-0.1497999968051910713029428734444081783294677734375f,-0.637199976506829468547721262439154088497161865234375f,-0.021000028192997177711731637828052043914794921875f,-0.71999999597668651318826960050500929355621337890625f,0.533199989314377287286106366082094609737396240234375f,0.049499986350536318013837444595992565155029296875f,-0.619800016623735405829620503936894237995147705078125f,0.08949998258799329864388027999666519463062286376953125f,0.6929999930858610213135762023739516735076904296875f,-0.70050000157952307944242420489899814128875732421875f,0.06499999426305291816419185124686919152736663818359375f,0.220500002026557950784990680404007434844970703125f,0.6384000039458275210790816345252096652984619140625f,0.7278999943941830697013983808574266731739044189453125f,0.453499990418553267090828740037977695465087890625f,-0.043999997586011918571102796704508364200592041015625f,0.39749999050050977000836383012938313186168670654296875f,0.113499977231025894752747262828052043914794921875f,0.2743999823689460271225470933131873607635498046875f,0.8836999584466223023326847396674565970897674560546875f,-0.9570000203251840531493144226260483264923095703125f,0.479999996274709683774517543497495353221893310546875f,0.29760000498741856489459678414277732372283935546875f,-0.2785000022649768425253569148480892181396484375f,-0.6809000157177447221101829200051724910736083984375f,-0.530299984377622823927822537370957434177398681640625f,-0.27649998223781580009017488919198513031005859375f,0.45340001556873321675311672152020037174224853515625f,-0.50540000594258316368723171763122081756591796875f,-0.474400002187490343885656329803168773651123046875f,-0.1584000073730944535554954200051724910736083984375f,-1.243500013105571344596000926685519516468048095703125f,0.183300015461445031661469329264946281909942626953125f,0.5747000109285116398183390629128552973270416259765625f,-0.836500036478042829912737943232059478759765625f,-0.09019999663829736391562619246542453765869140625f,0.0552000014066695854353383765555918216705322265625f,0.386399996100366127649294867296703159809112548828125f,-0.243599997305870008545980454073287546634674072265625f,-0.1935999996155499669470145818195305764675140380859375f,0.5932000377058983531242120079696178436279296875f,1.7926999671339984843143611215054988861083984375f,-0.4413000097393986465021953335963189601898193359375f,0.56719999597147108261196990497410297393798828125f,1.440699991195649065645056907669641077518463134765625f,-0.6393000302247708876990373028093017637729644775390625f,0.03760000226795678912594667053781449794769287109375f,-0.87159999098777785064839918049983680248260498046875f,-0.190200007218122468799492708058096468448638916015625f,-1.19090001914501186774941743351519107818603515625f,0.0095000504255295936673064716160297393798828125f,0.9127000398516660339964801096357405185699462890625f,0.0111000005096197895682053058408200740814208984375f,-0.61949998739361777921885732212103903293609619140625f,0.2307000003039834101770111374207772314548492431640625f,0.0571000015825031681515611126087605953216552734375f,-0.45520002703666673227189676254056394100189208984375f,0.6114000345826153814954295739880762994289398193359375f,-0.28859997793436065194327966310083866119384765625f,1.923299950194358842736619408242404460906982421875f,-0.10650000578165030873378782416693866252899169921875f,0.573500004649162065106793306767940521240234375f,0.51279994280338314638356678187847137451171875f,0.687200033652782593662777799181640148162841796875f,0.19280005426406887636403553187847137451171875f,1.9683999863624563886332907713949680328369140625f,0.024199973809718500206145108677446842193603515625f,0.386100003378093248329605557955801486968994140625f,-0.66090000480264432525245865690521895885467529296875f,0.42110002200454488274772302247583866119384765625f,0.020100014412403055530376150272786617279052734375f,-0.5140999905407428460790697499760426580905914306640625f,0.584999993592500544536960660479962825775146484375f,-0.207000000059604616353681194595992565155029296875f,0.4837999920070170656316577151301316916942596435546875f,0.1664999947547911585132851541857235133647918701171875f,0.5357999991074204881869036398711614310741424560546875f,0.755499994426965582050570446881465613842010498046875f,0.548000010371207935122583876363933086395263671875f,0.13450000661611571928233388462103903293609619140625f,0.615000012330710443819725696812383830547332763671875f,-0.0310000000298022992950564002967439591884613037109375f,-0.270799992699921165950627255369909107685089111328125f,-0.199799996209144570258331441436894237995147705078125f,0.129499999463558168599774944595992565155029296875f,0.68709998466968524866160805686376988887786865234375f,-0.6449000027611848651076797978021204471588134765625f,0.41050004559382813340562279336154460906982421875f,0.4379000029623509870901898466399870812892913818359375f,-0.5238999781258406596151644407655112445354461669921875f,0.226499992072582312374606772209517657756805419921875f,-0.535199994328618089411975233815610408782958984375f,0.1612999892875552898630076015251688659191131591796875f,-0.3150000274181365966796875f,0.54780002073645572835403072531335055828094482421875f,0.6747999912545086242943170873331837356090545654296875f,-0.307499998882412750589310235227458178997039794921875f,-0.25319999918639657376928653320646844804286956787109375f,0.04829999616444113907931523499428294599056243896484375f,-0.842000027850270615914496374898590147495269775390625f,-0.04570001751482488661082470571273006498813629150390625f,-0.0941999974653124905188406046363525092601776123046875f,0.3430000029206272671444821753539144992828369140625f,1.042699970859289226865485034068115055561065673828125f,-0.093799997594952611024154975893907248973846435546875f,-1.32950000572204629634143202565610408782958984375f,0.3934999654591084805588252493180334568023681640625f,-0.6524999772757293925451449467800557613372802734375f,-0.18549997997283895756481797434389591217041015625f,1.0167999746620655088236162555404007434844970703125f,-0.48110001284033021562436260865069925785064697265625f,-0.83500001575797799802103327238000929355621337890625f,-0.2840000054836273601921448062057606875896453857421875f,-0.2591999989926814240703834002488292753696441650390625f,-0.3619999806880951354060016456060111522674560546875f,0.1246000076174738069312297739088535308837890625f,-1.0248999924607573586143871580134145915508270263671875f,-0.6040999666988853267213244180311448872089385986328125f,0.30629999696165322031760069876327179372310638427734375f,0.57810005190372493188988300971686840057373046875f,0.80369998698234468292866949923336505889892578125f,0.620500008285045367983912001363933086395263671875f,-1.379000040650367964190081693232059478759765625f,0.27170001583099434583346010185778141021728515625f,0.8193000251770019559671709430404007434844970703125f,1.6479999930858610923678497783839702606201171875f,-0.83470003871917697324533946812152862548828125f,-0.251299994838237950034454115666449069976806640625f,-1.60440001246929142553199199028313159942626953125f,0.62130008573532080617951578460633754730224609375f,0.3578999941706655363304889760911464691162109375f,-0.23469997017383548154612071812152862548828125f,-0.6826999739885337703526602126657962799072265625f,-0.2807000228047371592765557579696178436279296875f,-0.173499994814395819275887333787977695465087890625f,1.418800014209747217819312936626374721527099609375f,0.1732000061154366221671807579696178436279296875f,0.67280001961886881645114044658839702606201171875f,-1.13930003545880342841201127157546579837799072265625f,-0.4221000220716002448995141094201244413852691650390625f,-0.16199997979402525061232154257595539093017578125f,0.01200001138448669735225848853588104248046875f,-0.697000039100647228451634873636066913604736328125f,0.503500009231269363141336725675500929355621337890625f,-0.341500017404556377442759185214526951313018798828125f,-0.3405000129789115259626441911677829921245574951171875f,0.901100034409761718734444002620875835418701171875f,1.14460000806450779009537654928863048553466796875f,-0.4974000191152097016811239882372319698333740234375f,-0.7928999908924101447382781771011650562286376953125f,-0.6149999700486663556375788175500929355621337890625f,0.00419999303221718633949421928264200687408447265625f,0.37150001457333559073958895169198513031005859375f,0.781699969786405546301466529257595539093017578125f,-0.656700014787912067504294100217521190643310546875f,-0.2897999771356580822612158954143524169921875f,-0.83279998750686701214362983591854572296142578125f,-0.8118000469326975832018433720804750919342041015625f,-0.56470002024173737709134002216160297393798828125f,-0.42440000997334681454020710589247755706310272216796875f,0.36099998348951356508251819832366891205310821533203125f,-0.01659999421015390108635756405419670045375823974609375f,0.78679997265040901854860067032859660685062408447265625f,-1.247000006914138481306508765555918216705322265625f,0.52459997289776794104909640736877918243408203125f,-0.6912000125750894508058763676672242581844329833984375f,-0.054499996185302990170384873636066913604736328125f,0.69230000365972532705427511245943605899810791015625f,0.787500013597309145296776478062383830547332763671875f,-0.8869999884366990272610564716160297393798828125f,-0.8515000095069407137771122506819665431976318359375f,-1.008999972999096161885290712234564125537872314453125f,0.716000005841255671157341566868126392364501953125f,0.7657000354707239608842428424395620822906494140625f,0.9907999799221756109801617640187032520771026611328125f,0.5459999855756763054159819148480892181396484375f,-0.412199994164705429966488736681640148162841796875f,0.3473999652847652441778336651623249053955078125f,-0.1064999935626982363601200631819665431976318359375f,0.84690000546574584205927749280817806720733642578125f,-0.15149999661743629442156588993384502828121185302734375f,0.061499999597668608686262814444489777088165283203125f,-0.88050002013146866364223797063459642231464385986328125f,-0.208100007908046225058029676802107132971286773681640625f,1.034500007808208721371556748636066913604736328125f,-0.1983999951541424167089644470252096652984619140625f,0.2644999787136914637386553295073099434375762939453125f,0.0755000036656856554628802769002504646778106689453125f,-0.395600006939470727473917577299289405345916748046875f,-0.237500000596046423328999708246556110680103302001953125f,-0.0214999892413612059272054466418921947479248046875f,0.26330000517964347039878703071735799312591552734375f,-0.74770000624954657819642989124986343085765838623046875f,0.58549998071789755016425260691903531551361083984375f,-0.170299990189075600710566504858434200286865234375f,0.6832999720990660108554948237724602222442626953125f,0.58200000020861608618361060507595539093017578125f,0.11999999269843097948751164949499070644378662109375f,-0.2649999999627470348428914803662337362766265869140625f,1.284500000357628124447728623636066913604736328125f,-0.66139999598860743645900583942420780658721923828125f,0.526099967838824067456471311743371188640594482421875f,0.6380000228881836932259830064140260219573974609375f,-0.234400033032894583584493375383317470550537109375f,1.316000007927417669861824833787977695465087890625f,-0.392900046324729856905833003111183643341064453125f,-0.3221000073194506541085502249188721179962158203125f,-0.298800013017654464420047588646411895751953125f,-1.45769998829364766379512730054557323455810546875f,0.4771000198364259148320343228988349437713623046875f,0.637200018304586279782597557641565799713134765625f,-1.6413000532507897588629930396564304828643798828125f,-0.230300005388259787508786757825873792171478271484375f,-0.6880000145435332115084747783839702606201171875f,-1.71169995443820965874692774377763271331787109375f,0.5067000033140178771873252117075026035308837890625f,0.66829998863935458075502538122236728668212890625f,0.221799978089332938679945073090493679046630859375f,-0.45920000450611109243936880375258624553680419921875f,-0.06160000454783443046835600398480892181396484375f,0.587799998086691033449824317358434200286865234375f,-0.0740000020563602589618312777020037174224853515625f,-0.0754999896585939467286152648739516735076904296875f,-1.5003999712228779372935605351813137531280517578125f,-0.47960002043247218495025663287378847599029541015625f,0.4543000064015387806648504920303821563720703125f,-0.449800021839141805912731797434389591217041015625f,-0.92160001661777446457790574640966951847076416015625f,-0.3061999973945319641899232010473497211933135986328125f,0.3971000006139278060146580173750407993793487548828125f,0.25550000377744430668514041826711036264896392822265625f,1.088700026433170098272285031271167099475860595703125f,-0.023099989020824995833436332759447395801544189453125f,-0.4605000082105397130050050691352225840091705322265625f,-0.8895000139176847397948222351260483264923095703125f,-0.1344000118732449067238121642731130123138427734375f,0.17639999852180476835883382591418921947479248046875f,0.3727000025987621256717829965054988861083984375f,0.67459991493225146541590220294892787933349609375f,0.8148000367403032129232087754644453525543212890625f,-0.65740000615119953408793662674725055694580078125f,-0.446000021934509049970074556767940521240234375f,0.9532000314474107227624699589796364307403564453125f,-0.641999991901218702849973851698450744152069091796875f,-0.342599960172176754014117250335402786731719970703125f,0.2566000026851893434098883517435751855373382568359375f,-0.0922000478088857988723248126916587352752685546875f,0.486099966758489987483926597633399069309234619140625f,-0.5504999871253968279916080064140260219573974609375f,-0.5757000192284582595902975299395620822906494140625f,-0.82649998287111536843241310634766705334186553955078125f,-0.396500020384788598448722041212022304534912109375f,-0.55449999737739563698823985760100185871124267578125f,0.365099994109570946676512903650291264057159423828125f,-0.16100002029538185155388418934307992458343505859375f,-1.0753000286340712587929147048271261155605316162109375f,-0.137400009131431577458215542719699442386627197265625f,-0.145500006794929248599146376363933086395263671875f,-0.60329998625516889632081074523739516735076904296875f,0.6864999851062894009601222933270037174224853515625f,0.1060000126957891808388012577779591083526611328125f,-0.168699998903274472894509017351083457469940185546875f,-0.4285999812498690886286567547358572483062744140625f,0.139500008553266496846845257095992565155029296875f,0.2994000012934208143633441068232059478759765625f,-0.0392999934002757900231017629266716539859771728515625f,0.69349998378753685557285280083306133747100830078125f,-0.94549998384714140087226041941903531551361083984375f,0.412499971762299910693627680302597582340240478515625f,0.8749999871850011601281948969699442386627197265625f,-0.640699996578693475868249151972122490406036376953125f,-0.0596000072821973869707790072425268590450286865234375f,-0.11049999904632568092921474089962430298328399658203125f,0.35769999494329096645373056162497960031032562255859375f,0.1018999991327524112794122856939793564379215240478515625f,-0.2025000053644177011591409609536640346050262451171875f,-0.3815999895349146964207420751336030662059783935546875f,-0.693900008803605583551643576356582343578338623046875f,0.42850000047683689530231276876293122768402099609375f,0.162499990165233754169094027020037174224853515625f,0.6354999996423718044979978003539144992828369140625f,-1.349000005483627973035254399292171001434326171875f,0.56209994890689873869860093691386282444000244140625f,-0.56039997178167144653571085655130445957183837890625f,-0.7610000189542773796347319148480892181396484375f,0.21499997645616542030211348901502788066864013671875f,0.3538000015437601231127473511151038110256195068359375f,0.1409999837875368644546369978343136608600616455078125f,0.0306000131234525074575003600330092012882232666015625f,0.472999977439642282917020565946586430072784423828125f,0.34449999678134890501723930356092751026153564453125f,-0.71600002968311304840653974679298698902130126953125f,-0.451299998116493128463844186626374721527099609375f,0.4132000214636324386674459674395620822906494140625f,-0.089800012451410449187960693961940705776214599609375f,1.1607000105857849803214776329696178436279296875f,-0.85050000083446519738572533242404460906982421875f,-0.1630999702453603816820759675465524196624755859375f,-0.56399998978152865447555086575448513031005859375f,0.046500009898096106297771257231943309307098388671875f,-0.1515000087060032196717429542331956326961517333984375f,0.4101000335790219164522341088741086423397064208984375f,0.8040000033974641358014423531130887567996978759765625f,-0.00180000465065244252826914816978387534618377685546875f,0.361700027155876568940584547817707061767578125f,1.7986000437974922760986373759806156158447265625f,-0.499200030136108807710115797817707061767578125f,0.04890000567436203482429846189916133880615234375f,-0.089100030624866377593207289464771747589111328125f,0.55390003279447608264263180899433791637420654296875f,0.008200032937526913201509159989655017852783203125f,-1.733499996304512791311935870908200740814208984375f,-0.28010003050565757121148635633289813995361328125f,0.9828999941706655363304889760911464691162109375f,0.6235999487280849962189677171409130096435546875f,-0.7068000168323518295210305950604379177093505859375f,0.12970001040697098915188689716160297393798828125f,0.364399991607665896253820392303168773651123046875f,0.44500001966953295351459019002504646778106689453125f,-0.5598000214107334837621010592556558549404144287109375f,0.9825000301003459224347125200438313186168670654296875f,0.54490003003031051154181341189541853964328765869140625f,-0.371200011122226669613155536353588104248046875f,0.881200027513504124954124563373625278472900390625f,0.67240002641677865113933876273222267627716064453125f,-0.17490001211166372740990482270717620849609375f,0.73350003802776342354263761080801486968994140625f,0.8007000388979914617948452359996736049652099609375f,-0.6066999877423047138336187344975769519805908203125f,1.18720000519156432261524969362653791904449462890625f,0.0373999767959112272563970691408030688762664794921875f,-0.79130003477335009165472001768648624420166015625f,-0.37570001505613248582449159584939479827880859375f,-0.4249000031709666558299431926570832729339599609375f,0.67310002090930964868675800971686840057373046875f,0.126999975956976651758623120258562266826629638671875f,0.45000000745058077455951206502504646778106689453125f,0.2775000084936618538478114714962430298328399658203125f,0.2785999922022224239981369464658200740814208984375f,-0.641500040054321374327628291212022304534912109375f,-0.73459998735189468987982763792388141155242919921875f,-0.66890000418424566674957532086409628391265869140625f,0.6164999880678951882373439730145037174224853515625f,0.0837000029265879685880236138473264873027801513671875f,0.43649998166039605251853572553955018520355224609375f,-0.30600000241398783717983178576105274260044097900390625f,0.34649999507144080279630315999384038150310516357421875f,-0.4362000111967321291928101345547474920749664306640625f,0.377000027179718699699151329696178436279296875f,0.715900021529197516656495281495153903961181640625f,-0.46440000833421901660358344088308513164520263671875f,-0.7625000268220905041971491300500929355621337890625f,0.130999980807304350349795640795491635799407958984375f,-0.13829999758005140364502949523739516735076904296875f,0.3709999867677691298695208388380706310272216796875f,-1.01810000973343850461105830618180334568023681640625f,0.34859996415078675635612626138026826083660125732421875f,0.195500026166438800601099501363933086395263671875f,-0.10070001311898213458562167943455278873443603515625f,-0.674299973097443938740980229340493679046630859375f,-0.6820000107884407469782672706060111522674560546875f,0.319299990308284709072950136032886803150177001953125f,-0.25139999878257501819689423427917063236236572265625f,-0.30550000374019148363657905065338127315044403076171875f,0.816700017526000632273053270182572305202484130859375f,0.1261000108659267038291318385745398700237274169921875f,-0.6119999994635580975455013685859739780426025390625f,0.769200010168552328337909784750081598758697509765625f,-0.0215999912858009235350209564785473048686981201171875f,-0.925000008344650126446140347979962825775146484375f,0.69580001739859564935386515571735799312591552734375f,-0.167199991072714215789574154769070446491241455078125f,0.394000024035573215286376580479554831981658935546875f,0.76249999158084402761659248426440171897411346435546875f,0.2333999951168894693154243213939480483531951904296875f,1.14599999541044184070415212772786617279052734375f,-0.18899999400973310770268653868697583675384521484375f,0.7326000065520401616225854013464413583278656005859375f,0.516000032663345820083122816868126392364501953125f,-0.4117000039100648933754200697876513004302978515625f,-1.6218999880313873518389300443232059478759765625f,0.6523000039577482311869971454143524169921875f,0.390899995899200547455620835535228252410888671875f,0.88900000160932535209212801419198513031005859375f,0.169699983286857758457699674181640148162841796875f,-0.90870000128745953560382986324839293956756591796875f,0.068299996723234723816631230874918401241302490234375f,-0.810499980866909108812023987411521375179290771484375f,-0.2299000032454727726616283689509145915508270263671875f,-0.600299975287914122645815950818359851837158203125f,-0.078899994945526685796721721999347209930419921875f,0.6072999896526329877133321133442223072052001953125f,0.5229999985992908051457561668939888477325439453125f,-0.2806000137567519203685151296667754650115966796875f,-0.38340000692605968879433930851519107818603515625f,0.819600013652443948330983403138816356658935546875f,0.9017999857634304561315730097703635692596435546875f,-0.21560000384747990409550766344182193279266357421875f,-0.716200002479553177181514911353588104248046875f,-1.20859997227191939828117028810083866119384765625f,0.7282000296592716637178455130197107791900634765625f,0.213900004482269423533580265939235687255859375f,1.47330001814365374457338475622236728668212890625f,-0.0085000010728837338547236868180334568023681640625f,-0.401299999569356469208969429018907248973846435546875f,0.808699998456239743660489693866111338138580322265625f,0.383700008961558314268813774106092751026153564453125f,0.5022999828726053550553842796944081783294677734375f,-0.89409998100399956655337518895976245403289794921875f,0.1137000124633311681776604018523357808589935302734375f,0.2225999849639832550707296832115389406681060791015625f,1.0752000078141692451794142471044324338436126708984375f,0.20700002289563446122855339126545004546642303466796875f,-0.4893999951839447248858050443232059478759765625f,0.7380999964714050065595074556767940521240234375f,0.31410000588893893791464506648480892181396484375f,0.4112999877601861697939966688863933086395263671875f,-0.80939999473690971143469141679815948009490966796875f,0.2200000189244748494132863925187848508358001708984375f,0.1310000079087914048869834005017764866352081298828125f,-1.2250999892294409665538523768191225826740264892578125f,-0.39390001606792213184604634079732932150363922119140625f,-1.456400011634826796580455265939235687255859375f,-0.9694999816939235959267762154922820627689361572265625f,0.6854999871253966148287872783839702606201171875f,0.7967999870300292997171709430404007434844970703125f,0.991299976062774756968565270653925836086273193359375f,0.5324999859929082646203823969699442386627197265625f,-0.392099986755848117780942629906348884105682373046875f,-0.485499995805322992481478650006465613842010498046875f,0.2554999907016755056332613094127736985683441162109375f,-0.5221999854847789723777395920478738844394683837890625f,0.136899991160631284703441679084789939224720001220703125f,-0.3514999947547909897593854111619293689727783203125f,0.2815999790668488600431373924948275089263916015625f,0.32309998194277300687105025644996203482151031494140625f,0.11700000303983681959607565659098327159881591796875f,-0.381599999779462795146400821977294981479644775390625f,-0.252999994575977382726961195658077485859394073486328125f,-0.3294999876171349484366146498359739780426025390625f,0.24579999564290044844483418273739516735076904296875f,-0.70669998956769697340263292062445543706417083740234375f,-0.418000037193298368265459430404007434844970703125f,-0.25519999395608916614719419158063828945159912109375f,0.574399980506301144345115972100757062435150146484375f,-0.155499982357024890688990126363933086395263671875f,0.73840001020431511591368689551018178462982177734375f,-0.405299991232156653353513320325873792171478271484375f,-0.2104999968856573477893334711552597582340240478515625f,0.63859999317079785274842151920893229544162750244140625f,0.07410000357925887459753511166127282194793224334716796875f,-0.516499999746680149570465800934471189975738525390625f,-0.53499998487532141044908939875313080847263336181640625f,-0.2390999905407429348969117199885658919811248779296875f,-0.44249999552965169069551620850688777863979339599609375f,0.59820001131221645085389582163770683109760284423828125f,-0.3896999800831080218443958074203692376613616943359375f,-0.05350000107288366280045011080801486968994140625f,0.59469999163150788490384002216160297393798828125f,0.165299998347461229997179543715901672840118408203125f,-0.589000008687377007987606702954508364200592041015625f,-0.43609999350607397960999378483393229544162750244140625f,-0.209599992381036326349885712261311709880828857421875f,0.42649998998641980296753217771765775978565216064453125f,-0.81049999288097029914723634647089056670665740966796875f,0.196499982461333433381156510222353972494602203369140625f,0.990499990701675159243677626363933086395263671875f,0.59170001970529551016397817875258624553680419921875f,-1.33439997193813297826636699028313159942626953125f,0.45450003671646044267617980949580669403076171875f,0.58869999026060071400934248231351375579833984375f,0.33760001322031030213111080229282379150390625f,0.367099996590614097158322692848742008209228515625f,-0.62590002778768560887101557455025613307952880859375f,-0.289399979723989897451019714935682713985443115234375f,1.345099992954730705463362028240226209163665771484375f,0.4741000050693748857355558357085101306438446044921875f,-0.965900006330013383148980210535228252410888671875f,0.484099996352195915960692218504846096038818359375f,0.2340999984383582699365433654747903347015380859375f,-0.53280002088546751792819122783839702606201171875f,-0.9566999669551847773618646897375583648681640625f,-0.1569000050187110417709845933131873607635498046875f,-0.5875000059604644775390625f,-0.63469998328685761634915252216160297393798828125f,0.6207000207185746631921574589796364307403564453125f,-0.14609999589025957078547435230575501918792724609375f,-1.72379996592998541160568493069149553775787353515625f,-0.289400013214349716150763924815692007541656494140625f,1.03490002984404583230571006424725055694580078125f,0.027999996215104516750216134823858737945556640625f,0.0948000118553640191976228379644453525543212890625f,-0.809000032901764143389300443232059478759765625f,-0.4537000000953668177317013032734394073486328125f,-0.162300005447864492680309922434389591217041015625f,-0.796599996650218855620551039464771747589111328125f,0.214300030541419772589506465010344982147216796875f,0.242500003278255604755031527020037174224853515625f,1.05989999512433996642357669770717620849609375f,-0.71010002335309962973042274825274944305419921875f,0.1459000245094301817516679875552654266357421875f,-0.0593000031232833180183661170303821563720703125f,-0.903099984252452969712976482696831226348876953125f,0.37150000458955734217170174815692007541656494140625f,-0.546600009055435709370840413612313568592071533203125f,-0.604199993404745949732159715495072305202484130859375f,0.0429999974071979540468646519002504646778106689453125f,-0.678900013646483557749888859689235687255859375f,0.1286999961465600250676288851536810398101806640625f,-0.17650000779330721201176856993697583675384521484375f,-0.1174999773502349853515625f,-0.2289999904334545366424435997032560408115386962890625f,0.213999992966652285986128845252096652984619140625f,0.7024999950826167349759998614899814128875732421875f,0.7727999659746889182798668116447515785694122314453125f,0.5690000141263009680869799922220408916473388671875f,-0.083299998325109481100980701739899814128875732421875f,-0.1762000041186808640958361138473264873027801513671875f,-0.4149999952316283380326922269887290894985198974609375f,0.9324999835714697926647431813762523233890533447265625f,-0.118200001794099673890769963691127486526966094970703125f,0.4060000022649763451454418827779591083526611328125f,0.5257999920666218685028070467524230480194091796875f,0.57379999718517049611676839049323461949825286865234375f,0.60599999570846563301529386080801486968994140625f,-0.45749997809529308057108210050500929355621337890625f,0.5855999922245740396675728334230370819568634033203125f,-0.375500018119812040140459430404007434844970703125f,-0.01179999895095829032243273104541003704071044921875f,0.405499991819262550762914543156512081623077392578125f,0.497499982118606798309201622032560408115386962890625f,-0.985899982115626460910107198287732899188995361328125f,0.46169997380971938838456480880267918109893798828125f,-0.891500014275312224754088674671947956085205078125f,-0.15550000426173227197068626992404460906982421875f,-0.7813000052690501906482722915825434029102325439453125f,0.81500000536441774556806194595992565155029296875f,-0.1378999934256075476923797396011650562286376953125f,0.70300000612437685276745469309389591217041015625f,0.2709999891519547787765986868180334568023681640625f,0.35210000985264766626414711936376988887786865234375f,0.49419998401701470402969107453827746212482452392578125f,-0.97300001066923158532517845742404460906982421875f,-0.5354000053465366448790518916212022304534912109375f,-0.68479998387843377116723786457441747188568115234375f,-0.4385000010728834496376293827779591083526611328125f,-0.342399998998641930114672504714690148830413818359375f,0.1180999960988760033586686404305510222911834716796875f,0.3519999947696923481288422408397309482097625732421875f,-0.1995999980434775000759373142500407993793487548828125f,0.493199993076920417678365993197076022624969482421875f,0.734000002428889075645201955921947956085205078125f,0.1414000010550022778943457524292171001434326171875f,0.511799999658763216103807280887849628925323486328125f,-0.990000028014183186542140902020037174224853515625f,0.5827000163078306371744474745355546474456787109375f,-0.984000029921531904619769193232059478759765625f,-1.108200001645087695578695274889469146728515625f,0.787200020539760458859745995141565799713134765625f,-0.2622999829471108768075282569043338298797607421875f,-1.0336999616503721455273989704437553882598876953125f,-0.06090000811815254877501502051018178462982177734375f,-0.247700011595338498437968155485577881336212158203125f,0.0248000305935740517071508293156512081623077392578125f,0.554800012712180379281790010281838476657867431640625f,0.5695000081062318031399627216160297393798828125f,-1.53939999935627014338024309836328029632568359375f,-0.2386000232338911786200696951709687709808349609375f,-0.4478999847829339131521919625811278820037841796875f,-1.4324999538064009385607278090901672840118408203125f,0.02769998948574059483007658855058252811431884765625f,-0.35280000628232954795748810283839702606201171875f,-0.019799984884262045170544297434389591217041015625f,0.15820000641345988157127067097462713718414306640625f,1.0231999778032303538566338829696178436279296875f,-0.92509997477531413778706337325274944305419921875f,-0.0774999874830246682222423260100185871124267578125f,0.5468999849766493781544340890832245349884033203125f,-0.25109997391104688091445495956577360630035400390625f,-0.7701000223100187280778072818065993487834930419921875f,-0.821900005763769314626188133843243122100830078125f,1.0189999908804896477931833942420780658721923828125f,0.209200020897389205032368408865295350551605224609375f,-0.2319000023365020979326800443232059478759765625f,-0.6150000095367431640625f,-0.555800046455861007643761695362627506256103515625f,-0.30409999679923060966757475398480892181396484375f,0.3403999938726425256163565791212022304534912109375f,-0.060600004220008774069583523669280111789703369140625f,1.20370002155303978952360921539366245269775390625f,-0.30459998466968585262293345294892787933349609375f,-0.279999990463256409611858543939888477325439453125f,-0.3938000098884106847663133521564304828643798828125f,-1.1677000097513197118814787245355546474456787109375f,-0.21890001550912874250798267894424498081207275390625f,-0.29779999909251930745313075021840631961822509765625f,0.065599999377131457123368818429298698902130126953125f,0.48880001651942739382405989090329967439174652099609375f,-0.49940003377795250116832903586328029632568359375f,-0.2774999797716739546871167476638220250606536865234375f,-0.29750000655651120951006305404007434844970703125f,-0.1294000090420244220013046287931501865386962890625f,0.74050000482052524830578477121889591217041015625f,0.27999997943639787223446546704508364200592041015625f,-0.518799985599518009848907240666449069976806640625f,0.380599969463050802342962697366601787507534027099609375f,-0.54449999236687995907146842000656761229038238525390625f,0.2499000094853341680245506495339213870465755462646484375f,0.05890000498518344063114682285231538116931915283203125f,-0.100499990105628711489771376363933086395263671875f,-1.01070000388026226545434838044457137584686279296875f,-0.1035000074803829051006687222979962825775146484375f,0.30349997931718863952710307785309851169586181640625f,-0.664399984604120419362516258843243122100830078125f,0.5652999651551251503178718849085271358489990234375f,0.0539999950528144978534328402020037174224853515625f,0.112200007128715473214697340154089033603668212890625f,0.2275999971270561417213684762828052043914794921875f,-0.651499993562698165305846487171947956085205078125f,-0.71829997522830968392781869624741375446319580078125f,-0.5738999825403094945386328618042171001434326171875f,0.606500019714236504597693055984564125537872314453125f,-0.50179997205436233098652110129478387534618377685546875f,-0.1932999932587147196016985617461614310741424560546875f,0.32899998000264130126879535964690148830413818359375f,0.7734999979436396611021109492867253720760345458984375f,0.38279999909251927192599396221339702606201171875f,0.9045000313520432655423064716160297393798828125f,-0.4701000083029267528900163597427308559417724609375f,0.392299996358155933506850487901829183101654052734375f,0.30299999889731399349557250388897955417633056640625f,0.2198999958693981238155856772209517657756805419921875f,0.17719999643713235659703286728472448885440826416015625f,-0.523000019311904651431177626363933086395263671875f,-0.49249998450279264261553180404007434844970703125f,-0.7684000074103469746233940895763225853443145751953125f,0.58549999964237198213368174037896096706390380859375f,-0.6179000050485132788935516146011650562286376953125f,0.533099993379413827909729661769233644008636474609375f,-0.4210000093802810372523026671842671930789947509765625f,0.258999993227422231623080506324185989797115325927734375f,0.10259999887794257755757598715717904269695281982421875f,-0.21650002768635800975971505977213382720947265625f,-0.1388000138372185165991368194227106869220733642578125f,-1.581799992692470624433553894050419330596923828125f,0.415999994516372595398934208787977695465087890625f,0.5513999940514562325688530108891427516937255859375f,0.699000007234513720533186642569489777088165283203125f,0.976799973171949442729555812547914683818817138671875f,-0.0147999971777201810851920527056790888309478759765625f,-1.474100043141842064642332843504846096038818359375f,-0.277899962878226602924769395031034946441650390625f,0.9927000455141072876585894846357405185699462890625f,0.527500011026859283447265625f,0.737599978351592966419048025272786617279052734375f,-0.51560001941919342272058202070184051990509033203125f,-1.5472999699831007092143408954143524169921875f,-0.29079997522831035894341766834259033203125f,1.048200009989738390459024230949580669403076171875f,-0.3407000326395035472160088829696178436279296875f,-1.9014999750852581428262055851519107818603515625f,0.187100004339217917959103942848742008209228515625f,1.436200005161762049965545884333550930023193359375f,-0.394900066947936778660732670687139034271240234375f,-0.5367000149369243189312328468076884746551513671875f,0.6983999900318680342792276860564015805721282958984375f,0.7985999627351765273175487891421653330326080322265625f,-0.13679999511390927846576914816978387534618377685546875f,0.355300029826164376345332129858434200286865234375f,1.257499989569186737980999168939888477325439453125f,-0.122900005793571409640208003111183643341064453125f,-0.925999981403350602704449556767940521240234375f,0.55160003330707496616014395840466022491455078125f,-0.1416000303268436510961691965349018573760986328125f,-0.92629996622800803152131265960633754730224609375f,0.2798000245213501102625741623342037200927734375f,0.23659999039173129631308256648480892181396484375f,-0.7812999794900417072085474501363933086395263671875f,0.0060999889612194380106302560307085514068603515625f,0.5052999934673305659771358477883040904998779296875f,-0.038600019061565404854263761080801486968994140625f,-1.802499976158141947735202847979962825775146484375f,-0.2266999982476232133876692387275397777557373046875f,0.28090000782012936042519868351519107818603515625f,-1.13230001528263102272831019945442676544189453125f,0.12810001167058970850121113471686840057373046875f,0.5487999899581073037779788137413561344146728515625f,0.319699996139108988391086541014374233782291412353515625f,0.4464999971389769228835575631819665431976318359375f,0.717799978417158257570918067358434200286865234375f,0.96809998265057795041599320029490627348423004150390625f,-0.5039999856650829723747619937057606875896453857421875f,0.3773999876365066175054607811034657061100006103515625f,-0.673999982535839148312106772209517657756805419921875f,-0.2085000225305559951038958388380706310272216796875f,0.477600017690658518176860525272786617279052734375f,0.91370000419318653239741934157791547477245330810546875f,0.2815000165104866169940578402020037174224853515625f,0.113899993306398439329996108426712453365325927734375f,-0.4014999935626983873504514122032560408115386962890625f,-0.22700002956390363806349341757595539093017578125f,0.5038999833226203151070876629091799259185791015625f,0.95759995991140645088535165996290743350982666015625f,0.01749999821186065673828125f,-0.8805999854445456520579682546667754650115966796875f,0.532900001025199632209705669083632528781890869140625f,-0.3579999805688860448071864084340631961822509765625f,-0.6731000270187852407843820401467382907867431640625f,-0.86839997697472603999102602756465785205364227294921875f,-0.34249998196959519081161715803318656980991363525390625f,0.24139998885467661704495867525110952556133270263671875f,-0.6172999718457463647069971557357348501682281494140625f,-0.1089999986290930422683231881819665431976318359375f,0.55719999118447294250699997064657509326934814453125f,-0.44760000498741836505445235161460004746913909912109375f,0.869000009059906375341597595252096652984619140625f,0.42640000597238536528266195091418921947479248046875f,0.612999974384904167123977458686567842960357666015625f,0.374499981582165020199681748636066913604736328125f,0.410299996298551405971011263318359851837158203125f,0.5790999638676648242352484885486774146556854248046875f,-0.08400000280141828312707730219699442386627197265625f,0.327599974775314439767726071295328438282012939453125f,0.462599983157217564677665677663753740489482879638671875f,0.3359999831914901591289890347979962825775146484375f,-1.0274999792873860116060313885100185871124267578125f,0.38319999136328697364461959296022541821002960205078125f,-0.370000023245811604510890902020037174224853515625f,-0.6134000091612339389257613220252096652984619140625f,-0.0398000080183147364465412465506233274936676025390625f,-0.20199998594075463387298441375605762004852294921875f,0.2648999912500382691860068007372319698333740234375f,0.14970000623464585487454314716160297393798828125f,-0.151800007891655042158163269050419330596923828125f,-0.73480002839565283778711091144941747188568115234375f,0.771899968063831209974523517303168773651123046875f,-0.094499996185302137519101961515843868255615234375f,-0.4576999903798100177709784475155174732208251953125f,-0.5633999862134455582918235450051724910736083984375f,-1.0715999853253368456762473215349018573760986328125f,-0.19820001684427257515608289395458996295928955078125f,0.904999989084899425506591796875f,0.39399999285489339939658748335205018520355224609375f,0.127000000171362703582644826383329927921295166015625f,-0.35070001948922868706404187832958996295928955078125f,0.015000036284327666891158514772541821002960205078125f,0.64160002906024471425183719475171528756618499755859375f,-0.13530001254081724937350372783839702606201171875f,-1.58279998810291289146334747783839702606201171875f,0.443199998962878538577569997869431972503662109375f,-0.125800002963095902686063709552399814128875732421875f,0.1102000054672360729313140836893580853939056396484375f,0.179200006778538256657640204139170236885547637939453125f,-1.13969998626708957090158946812152862548828125f,1.1538000475883478657124214805662631988525390625f,0.233799975168704321504264953546226024627685546875f,-1.618799990963935897525516338646411895751953125f,0.70079999489784228217104100622236728668212890625f,0.2105999729275698229002955486066639423370361328125f,-0.7467000033140180903501459397375583648681640625f,-0.36869995360374474557829671539366245269775390625f,0.6733000154614448007350802072323858737945556640625f,-0.265900018250942338227105210535228252410888671875f,1.038100004220009253685930161736905574798583984375f,0.2651000206708911122177596553228795528411865234375f,-0.171099985980987412403919734060764312744140625f,-1.95019997994899796367462840862572193145751953125f,0.2790000111460688714259958942420780658721923828125f,1.2133000033915042337184786447323858737945556640625f,0.3970999694705010796269561978988349437713623046875f,-0.52350000464916224274247724679298698902130126953125f,0.5693999971956014150009650620631873607635498046875f,-0.23340000945925698516703050700016319751739501953125f,-0.6758000048816199534229554046760313212871551513671875f,1.1192999980568885121101629920303821563720703125f,-0.2178000120192766786431093350984156131744384765625f,-0.017400010696053680447903388994745910167694091796875f,0.79540001018941364208103550481610000133514404296875f,0.5510000509917742927967765353969298303127288818359375f,0.0569999884366989562067828956060111522674560546875f,0.88599998609721664077909508705488406121730804443359375f,0.61160002496242515945823470246978104114532470703125f,-0.48999998223036522748685683836811222136020660400390625f,0.0643000022292137263235645150416530668735504150390625f,-0.52690000602453945788283817819319665431976318359375f,0.72209997332617670284804489710950292646884918212890625f,-0.5941999886550008991292770588188432157039642333984375f,-0.3538000179164111358431910048238933086395263671875f,-0.41710000240206657196040396229363977909088134765625f,0.5224999771267178072520209752838127315044403076171875f,0.2364999929666513622805723571218550205230712890625f,0.383199980932473760475431845407001674175262451171875f,0.815199990603327595550808837288059294223785400390625f,1.442000051319599407406712998636066913604736328125f,-0.0489999979585409306537258089520037174224853515625f,0.34539999059438708517433269662433303892612457275390625f,-0.2553999883592130704101919036475010216236114501953125f,-0.0830999821662903315200310316868126392364501953125f,-0.430499998971819675119832027121447026729583740234375f,-1.217700034189224567171550006605684757232666015625f,-0.287499981224537126678342247032560408115386962890625f,0.14209999629259117970292436439194716513156890869140625f,-0.33710000344514856607247565989382565021514892578125f,0.010799999293684969803308604241465218365192413330078125f,0.034099999090284112412962969074214925058186054229736328125f,-0.027499998491257449106939247940317727625370025634765625f,0.557099968874454898326575857936404645442962646484375f,0.263599987359345089288353847223334014415740966796875f,0.434799999189376507047199993394315242767333984375f,0.78969996808767373153159496723674237728118896484375f,-0.7262999921560286953337026716326363384723663330078125f,-0.7575000280141832575964144780300557613372802734375f,0.30559998119771503155561731546185910701751708984375f,0.059899998290836808934045620844699442386627197265625f,0.3463000013202428561953638563863933086395263671875f,0.16910000633597377373007475398480892181396484375f,-0.550299993169307555263003450818359851837158203125f,-1.1171000371217729707495891489088535308837890625f,-0.03609999849796263760026704403571784496307373046875f,0.04109999682158227418682372444891370832920074462890625f,0.664499999612569780538251507095992565155029296875f,0.3753999972999098044823540476500056684017181396484375f,-0.39639999252408741892139687479357235133647918701171875f,-0.1894999854564662911116101895458996295928955078125f,-0.45820002191066766528138032299466431140899658203125f,-0.32660001538842919899252592585980892181396484375f,-0.268999992966652001769034541212022304534912109375f,-0.3210000013709066735145825077779591083526611328125f,-0.053699999573826762144790336606092751026153564453125f,0.679300004017353131757772644050419330596923828125f,0.57740001419782682745562851778231561183929443359375f,-0.1618999930232762896054055090644396841526031494140625f,-0.46440000330507746895136733655817806720733642578125f,-0.31160000343024696345395341268158517777919769287109375f,0.169100002908706692483775668733869679272174835205078125f,0.769699970173835623654667870141565799713134765625f,0.564999975264072418212890625f,-0.426699985581636465070687336265109479427337646484375f,0.6268000001430511503031084430404007434844970703125f,0.292599996829033148770804473315365612506866455078125f,-0.3984999908655881828423162005492486059665679931640625f,-0.770100014412403055530376150272786617279052734375f,-0.5122000011682512621291607501916587352752685546875f,0.554800004144012870455071606556884944438934326171875f,-0.395799982604384315010292993974871933460235595703125f,-0.5578000064313408667970861642970703542232513427734375f,-0.185600004182755906612101171049289405345916748046875f,-1.11350000435113916097407127381302416324615478515625f,-0.71219999163150760068674571812152862548828125f,0.62820000518411400758367335583898238837718963623046875f,-0.37869999681711197236921861986047588288784027099609375f,-0.0205999819427729047305319909355603158473968505859375f,0.4642999949276447324564287555404007434844970703125f,-0.77989999870061854636560383369214832782745361328125f,-0.620200007963180865999675006605684757232666015625f,0.31239999024420972428828235933906398713588714599609375f,0.03749999508261669944886307348497211933135986328125f,0.086199994730949303090028479346074163913726806640625f,-0.049900001308321950688196011469699442386627197265625f,-0.883499987959861954323059762828052043914794921875f,-0.7099999952316284890230235760100185871124267578125f,0.27139999352991583947414255817420780658721923828125f,-0.594499999761581410240296463598497211933135986328125f,0.56280002550482777223805896937847137451171875f,-0.05390000351369383935207224567420780658721923828125f,0.42720000120550383204687250326969660818576812744140625f,0.0108000260412695059386578577687032520771026611328125f,0.697699997979401853598346860962919890880584716796875f,-0.4113999821677805091013624405604787170886993408203125f,-0.3878000012159343867779170977883040904998779296875f,0.349499983072281139584447373636066913604736328125f,-0.3291999907597900065781004741438664495944976806640625f,-0.2156000086903571144603120046667754650115966796875f,-0.431099997156858538716051043593324720859527587890625f,0.6023000143885608448357515953830443322658538818359375f,0.2323000477671630648046630085445940494537353515625f,0.119199999625980712636419411865063011646270751953125f,-0.647099972040951598728497629053890705108642578125f,-0.53669999370276866290652151292306371033191680908203125f,-0.355299973946809899416621192358434200286865234375f,-0.26630000601410852567596521112136542797088623046875f,-1.18510000934600867594781448133289813995361328125f,0.084699989694356769831529163639061152935028076171875f,0.7009999982044099464673081456567160785198211669921875f,0.481000042498112012623323607840575277805328369140625f,0.27109999149441676991045824252068996429443359375f,0.7187999702140690505558495715376920998096466064453125f,1.8744000288605686677101402892731130123138427734375f,0.68049997632205483366618636864586733281612396240234375f,-0.5518999694794419763610449081170372664928436279296875f,-0.406299995061754515290886047296226024627685546875f,0.2726999818861486346577294170856475830078125f,-0.33159999281317009689473707112483680248260498046875f,0.5896000037431718965308391489088535308837890625f,-0.34609999294728022167788594742887653410434722900390625f,-0.10849999887496231121275513942237012088298797607421875f,0.1678999879494309510619132197462022304534912109375f,0.4342999979078765449003185494802892208099365234375f,-0.17020000449866046210445347242057323455810546875f,0.53019999365806569358028355054557323455810546875f,-0.1626999882936475128047959515242837369441986083984375f,-0.64709999666511996974094245160813443362712860107421875f,-1.04670003803372413386796324630267918109893798828125f,0.48399997642636360950518792378716170787811279296875f,-0.686099999205767918653009473928250372409820556640625f,-0.159300045144558310994398198090493679046630859375f,-0.115799999070167469739089938229881227016448974609375f,-0.140200000922381828871721154428087174892425537109375f,-0.5307999948978423532253145822323858737945556640625f,-0.317799990785122332681567058898508548736572265625f,0.4170999908536672240444431736250407993793487548828125f,-0.16810002895593623151171414065174758434295654296875f,0.17219999133348462549975010915659368038177490234375f,0.38970000593662257326599274165346287190914154052734375f,0.3871000234127048855725661269389092922210693359375f,-0.1360000189542773796347319148480892181396484375f,-0.6332000175148249088152851982158608734607696533203125f,-0.381000004947185555437272341805510222911834716796875f,-0.240999997496604834168465458787977695465087890625f,0.249100004175305367315473858980112709105014801025390625f,-0.0823000012755397136743340524844825267791748046875f,-0.42800002467632314306911212042905390262603759765625f,-0.46059999751448632565598018118180334568023681640625f,-0.46540002814531344910164989414624869823455810546875f,-0.153400000667571756451934561482630670070648193359375f,0.45910001840591430077864742997917346656322479248046875f,-0.24310000069960924218293030207860283553600311279296875f,-0.364800019417703402790920108600403182208538055419921875f,-0.0957000067301095047600512089047697372734546661376953125f,-0.8800000341236593470739535405300557613372802734375f,-0.4670000124275681674390625630621798336505889892578125f,0.51920001743286847695202368413447402417659759521484375f,-0.0341999933302401526447056312463246285915374755859375f,-0.26060001122355469505009750719182193279266357421875f,-0.2350000033155083822844488850023481063544750213623046875f,-0.251800009381771161542928894050419330596923828125f,-0.05679998375177319758222438395023345947265625f,0.376900018168985884159383203950710594654083251953125f,0.58110001548528666948101317757391370832920074462890625f,0.0599999886006117311154639537562616169452667236328125f,-0.700800001752376555685941639239899814128875732421875f,-0.77200000244379030078789583058096468448638916015625f,-0.58600000733137136421646573580801486968994140625f,0.30609999622553585307827006545267067849636077880859375f,-0.45710000558719043883826316232443787157535552978515625f,-0.392800013844668949314353767476859502494335174560546875f,0.0408999984882771894234565479564480483531951904296875f,-0.84660001959800723625448881648480892181396484375f,-0.591600020194054110334036522544920444488525390625f,0.35699999767541878981091940659098327159881591796875f,-1.16460003004372136814481564215384423732757568359375f,0.48540003932118480634017032571136951446533203125f,0.11549999517202369503365844138897955417633056640625f,0.4617999753698709408666900344542227685451507568359375f,1.2337999954342837582998981815762817859649658203125f,-0.519299963635206918155517996638081967830657958984375f,-0.1876999959677458917894909973256289958953857421875f,-0.1728999843358991483910358510911464691162109375f,0.670599994534253340816576383076608180999755859375f,0.23450000739842646879651510971598327159881591796875f,0.04690001455545456110485247336328029632568359375f,0.0974999926984310150146484375f,0.09359999644905325766330861370079219341278076171875f,0.520800009948015230065720970742404460906982421875f,0.738199977654219186007367170532234013080596923828125f,-0.580700014273821896182425916776992380619049072265625f,-0.748300049138070022536339820362627506256103515625f,-0.16149999967217443241906948969699442386627197265625f,0.086199990297853912313286173230153508484363555908203125f,0.567800018799304861971677382825873792171478271484375f,0.475200001984088959972041266155429184436798095703125f,-0.050599999060481792856336369368364103138446807861328125f,0.1994000095076859935261381906457245349884033203125f,-0.00049999494850627712594359763897955417633056640625f,-0.83029999607503413017184357158839702606201171875f,-0.4279000124990941511526898466399870812892913818359375f,0.485899988076090583177801818237639963626861572265625f,0.089699999678134911818006003159098327159881591796875f,0.2145000077337027022394977393560111522674560546875f,-0.1090000171065330647479640902020037174224853515625f,0.705100004614889197540605891845189034938812255859375f,1.318700064170361230253547546453773975372314453125f,0.46859998169690386671248916172771714627742767333984375f,0.0309000047653913423317817432689480483531951904296875f,-0.47190000259727238329787724069319665431976318359375f,0.36399999684095352137092049815692007541656494140625f,-0.3988999927476049034424931960529647767543792724609375f,0.1642999977588655013960305950604379177093505859375f,0.2792000006876886875062382387113757431507110595703125f,-0.82599999323114747795671064523048698902130126953125f,-0.80480003366693875221926646190695464611053466796875f,0.28700000959634763830763404257595539093017578125f,-0.678600016826390817215042261523194611072540283203125f,-0.201000042200089268362717120908200740814208984375f,-0.001799997870624014950635682907886803150177001953125f,-0.25009997421652074844899971139966510236263275146484375f,0.4105000234842304251969835604541003704071044921875f,-0.1362000313878066748429773724637925624847412109375f,-0.7917000274538994464279539897688664495944976806640625f,0.828299966585635871751946979202330112457275390625f,0.5263999767661093898141189129091799259185791015625f,-0.43009997905939822970822206116281449794769287109375f,-0.497200013685226593906918424181640148162841796875f,-0.2880000100731852086255457834340631961822509765625f,0.2277000001028179898554526516818441450595855712890625f,0.64120002155303978952360921539366245269775390625f,0.048399987554549994683839031495153903961181640625f,-0.703600009301304840647617311333306133747100830078125f,0.1594000193238258589190081693232059478759765625f,-0.1310000106096271821343179908581078052520751953125f,-0.433000015921890746284361739526502788066864013671875f,-0.96600000911951067195104769780300557613372802734375f,-0.418599989557266294326609568088315427303314208984375f,0.75429999857842922050821243828977458178997039794921875f,0.852700018095970069254008194548077881336212158203125f,0.67960001775026324111195208388380706310272216796875f,-0.335200005578994708610451880304026417434215545654296875f,-0.5395000125765800902399860206060111522674560546875f,-0.35800000232458106808053344138897955417633056640625f,0.3331000076100230078424146995530463755130767822265625f,-0.47880001987218889780706376768648624420166015625f,-0.00619998966455437283684659632854163646697998046875f,0.5303000113487243982746122128446586430072784423828125f,0.211099994847178340506843596813268959522247314453125f,0.2228000049412248717572992973146028816699981689453125f,0.046600002610683421977455509477294981479644775390625f,0.6073000039577483022412707214243710041046142578125f,0.515200011539459268306018202565610408782958984375f,-0.1653999980449676332483477381174452602863311767578125f,0.1793000320315361051370928180404007434844970703125f,-0.11119998281002008155837756930850446224212646484375f,-0.279400003230571680745697449310682713985443115234375f,0.3004000171184539880187003291212022304534912109375f,0.156400018191337863981971167959272861480712890625f,-0.0993999986112117372982766028144396841526031494140625f,-0.5528999923825264062315909541212022304534912109375f,-0.0637999748706814528986797085963189601898193359375f,0.5593999956682325080947748574544675648212432861328125f,-0.591500009991228647976413412834517657756805419921875f,-0.5647999774336813771924425964243710041046142578125f,0.6557000104367727288945388863794505596160888671875f,-0.6114999928921458494102125769131816923618316650390625f,-1.2791000446319582550813720445148646831512451171875f,-0.18359998970627788850151773658581078052520751953125f,0.51959996701180966027777685667388141155242919921875f,-0.01679999060630876783761777915060520172119140625f,0.035800008010864470264777992269955575466156005859375f,-0.72470000850707261275118753474089317023754119873046875f,-0.63190004584789338792916169040836393833160400390625f,0.57979998488426165437203962937928736209869384765625f,0.6313999983727931475385730664129368960857391357421875f,0.4042000259637834602699513197876513004302978515625f,0.39789999566078204651375926914624869823455810546875f,-0.486700000408291533204874212970025837421417236328125f,-0.867500047683716246638141456060111522674560546875f,-0.162899993723630842623606440611183643341064453125f,-0.390600001426041065766270321546471677720546722412109375f,0.317399983799457441335789553704671561717987060546875f,-0.476399995541572085500092725851573050022125244140625f,0.2171000031843781652440839025075547397136688232421875f,-0.5321000122368335638611824833787977695465087890625f,0.17099999257922160467160210828296840190887451171875f,0.12759999206066130472692066177842207252979278564453125f,0.73280001552104945261589818983338773250579833984375f,-0.501099990749358514818823095993138849735260009765625f,-0.663599999131262219265181556693278253078460693359375f,-0.603600031280517868026436190120875835418701171875f,-0.20519999805390820579731325778993777930736541748046875f,0.5064999766126276892208579738507978618144989013671875f,0.557299999412893765082799291121773421764373779296875f,-0.38939999682307213646481613977812230587005615234375f,0.4084999929144979002870741169317625463008880615234375f,0.0073000024676321828565050964243710041046142578125f,0.426999962657690357303863493143580853939056396484375f,0.56290001298338143698174462770111858844757080078125f,0.7313000320911413609792361967265605926513671875f,0.0309999796152116147140986868180334568023681640625f,0.82400000138580775654872923041693866252899169921875f,0.98080004019737287990210461430251598358154296875f,-0.4630999927461143528262255131267011165618896484375f,-0.6935999717876317749443160209921188652515411376953125f,-0.5748000167727465026246136403642594814300537109375f,-0.1993000043712556124120283129741437733173370361328125f,-0.403499985482543677139943838483304716646671295166015625f,0.0645999927163121157036584918387234210968017578125f,-0.26620002214908566884332685731351375579833984375f,-0.4849999866262078018763759246212430298328399658203125f,0.254300024282932302099879962042905390262603759765625f,-0.03599999541044240913834073580801486968994140625f,-0.346200009036064126899390203107031993567943572998046875f,0.4477999922752378125778704998083412647247314453125f,0.011399968421458961387315866886638104915618896484375f,-0.6911999973759055393429662217386066913604736328125f,0.9932999972820280021323924302123486995697021484375f,0.70000000298023223876953125f,-0.4311999922722578038047913651098497211933135986328125f,0.930999999284744195193752602790482342243194580078125f,0.5607999940037728947572759352624416351318359375f,-0.5691999933302402947532527832663618028163909912109375f,0.237899992978572782931223628111183643341064453125f,0.171399992859363692332408390939235687255859375f,-0.1782999950468540095727121297386474907398223876953125f,-0.92649999624490764205120285623706877231597900390625f,-0.6589999881982802065749638131819665431976318359375f,0.46100000304728749878080407142988406121730804443359375f,1.026100016379356105744591332040727138519287109375f,0.34960000135898638973230845294892787933349609375f,-0.8571999872356652616645078524015843868255615234375f,-0.545200026142597327094563297578133642673492431640625f,0.04440001425743123064648898434825241565704345703125f,0.776100012765824853744334177463315427303314208984375f,0.12090001035332686996071061003021895885467529296875f,0.10000001341104525209857456502504646778106689453125f,0.0322999996736645922368325045681558549404144287109375f,-0.44970000221133243911708632367663085460662841796875f,-0.359000003695487901467231495189480483531951904296875f,0.1169000018894672532354661598219536244869232177734375f,0.1621000233754517072526368792750872671604156494140625f,-0.04739998417198620916934714841772802174091339111328125f,-0.1380000006854533722844280418939888477325439453125f,0.0161000050544739536917404620908200740814208984375f,-0.26600001746416079839718804578296840190887451171875f,-0.38230000049322843125310100731439888477325439453125f,-0.082600009495019666161397253745235502719879150390625f,0.192800000321864839492036480805836617946624755859375f,0.307299993415176830868773549809702672064304351806640625f,0.025199979948997253131892648525536060333251953125f,-0.243999981231987461693933028072933666408061981201171875f,0.56229997832775158173035379149951040744781494140625f,-0.6570999905556440534581952306325547397136688232421875f,-1.0235000165700913754562861868180334568023681640625f,0.3377999812483789554562463308684527873992919921875f,0.2179000017330050553709952509962022304534912109375f,0.119899997359514287609272287227213382720947265625f,0.39589999932646779967626571306027472019195556640625f,-0.7205000073909755275991528833401389420032501220703125f,-1.506100050652027899644735953188501298427581787109375f,-0.354199997800588306517965975217521190643310546875f,0.66989998122900740185059476061724126338958740234375f,0.03990000704526863728460739366710186004638671875f,-0.160799997133016514538894625729881227016448974609375f,0.24500000216066830649452867874060757458209991455078125f,-0.53659998845458023453147688996978104114532470703125f,-0.116499992817640229958442432689480483531951904296875f,-0.36689999078810220378699114007758907973766326904296875f,-0.80490000823736185253665098571218550205230712890625f,0.36249997124075949983534883358515799045562744140625f,-0.188000006608664893548166219261474907398223876953125f,0.637999984145164233950708876363933086395263671875f,0.37699996116757450437262377818115055561065673828125f,0.7558999994382260556591290878714062273502349853515625f,0.2059000325560571553751287865452468395233154296875f,-0.637599983865022590379112443770281970500946044921875f,0.0042999938473105014935526924091391265392303466796875f,0.4763000095903873187808130751363933086395263671875f,-0.0516000087201595736274839509860612452030181884765625f,0.53460000359415982273958434234373271465301513671875f,0.701400030559301779220504613476805388927459716796875f,0.2342999912023546738026880120742134749889373779296875f,-0.508499995186924902412783922045491635799407958984375f,-0.32730001930594454506717738695442676544189453125f,0.297499971874058133369800316359032876789569854736328125f,0.87260001098513562123315523422206752002239227294921875f,1.2130000577196486943876152508892118930816650390625f,0.524599989289045165463676312356255948543548583984375f,-0.404599983254075201699606623151339590549468994140625f,-0.591399998819827743545829434879124164581298828125f,-0.4713999678254132685850663619930855929851531982421875f,-0.1336999975249171246360901932348497211933135986328125f,-0.1681000018358229652903901296667754650115966796875f,-0.6165999915838238365495271864347159862518310546875f,0.2367999988391995547232227181666530668735504150390625f,-0.0927000064730645334520886535756289958953857421875f,-0.01239999125003787838750213268212974071502685546875f,0.188200010958313956876963857212103903293609619140625f,-0.41390000850558283929103708942420780658721923828125f,-0.377200013983249871074576731189154088497161865234375f,0.075999999731779066536319078295491635799407958984375f,0.71999999165534944722821819595992565155029296875f,0.4725999869942665299049622262828052043914794921875f,-0.529299993959069237092762705287896096706390380859375f,-0.06949998694658265918633333058096468448638916015625f,-0.3846000150680541906922371708787977695465087890625f,-0.37320000555664301078451217108522541821002960205078125f,-0.68030001403093365297536365687847137451171875f,-0.541600001418590437651801039464771747589111328125f,0.339700013647973531050183737534098327159881591796875f,0.3359999906420707560528171597979962825775146484375f,0.311999989330768556783368694595992565155029296875f,-0.1988999996021389993217098890454508364200592041015625f,0.417299985778331716801403672434389591217041015625f,0.15759997686147730888706064433790743350982666015625f,-0.486699987556040269964796607382595539093017578125f,0.59919999943971635758543925476260483264923095703125f,0.250999984383583285563190656830556690692901611328125f,-0.56749999582767485239997995449812151491641998291015625f,-0.47530000866651533897311310283839702606201171875f,0.05000002652406720926592242904007434844970703125f,0.756100006730854534708896608208306133747100830078125f,0.30889999881982799223578695091418921947479248046875f,0.239799998295307403850529226474463939666748046875f,-0.20049999636411663939128402489586733281612396240234375f,0.5717999557375907926370928180404007434844970703125f,0.4349999594688416237886485760100185871124267578125f,-0.3863999823167920677491338210529647767543792724609375f,0.335699984061717859873397173942066729068756103515625f,-0.10880003149509409610118382261134684085845947265625f,-0.589299984608590676771200378425419330596923828125f,-0.46899998596310599197067858767695724964141845703125f,-0.319599991226195978555324472836218774318695068359375f,0.18969999412745242040045923204161226749420166015625f,0.43110000594854369637687341310083866119384765625f,0.26419999318122844300660290173254907131195068359375f,-0.303200008387863650938243154087103903293609619140625f,-1.013699983406066706947967759333550930023193359375f,0.1508999774217603118131592054851353168487548828125f,-0.044800011855363752744096927926875650882720947265625f,0.524699992860853559051292904769070446491241455078125f,0.893100031042099118394617107696831226348876953125f,0.3147999985933302991014670624281279742717742919921875f,-0.3271000083997843699279428619774989783763885498046875f,-0.0755000188648704551042101229541003704071044921875f,-0.24209998854398673273635722580365836620330810546875f,-0.6953999973371622900231159292161464691162109375f,-1.7874000457882885939397965557873249053955078125f,0.682099983030557410756955505348742008209228515625f,-0.2115999873742462344949899488710798323154449462890625f,-0.1975999962329861858734147972427308559417724609375f,0.40079997418522861352130348677746951580047607421875f,0.3031999772816897920080236872308887541294097900390625f,1.1596000102996821823353457148186862468719482421875f,-0.3109999772310256815899265347979962825775146484375f,-0.69089998841136690543862641789019107818603515625f,-1.4157000281691551890617120079696178436279296875f,0.07520000662207593222774448804557323455810546875f,-0.72719997337758568445309492744854651391506195068359375f,-1.51559999766349751126881528762169182300567626953125f,0.090700002986192629350625793449580669403076171875f,-0.355599981234967887910869421830284409224987030029296875f,-1.117999998152255702876800569356419146060943603515625f,-0.55779999421238901646802332834340631961822509765625f,0.4480000002384183499515302173676900565624237060546875f,1.0988000338792804910781342186965048313140869140625f,-0.181899993842840057567400435800664126873016357421875f,-0.39949998240172879260256877387291751801967620849609375f,-0.92950000512599917357192680356092751026153564453125f,-0.3206999599218374186193614150397479534149169921875f,-0.7742999919474125025686817025416530668735504150390625f,-1.7727000061750413095751355285756289958953857421875f,0.547999987423419820942172009381465613842010498046875f,-0.1326000004053116132496370482840575277805328369140625f,-0.4793999878823758109547270578332245349884033203125f,0.1375999934017657633233966407715342938899993896484375f,-0.49069996748417654686136302188970148563385009765625f,-0.289899992591141852926739375106990337371826171875f,-0.0970999854147436280982219614088535308837890625f,-0.18419999351650473418118281188071705400943756103515625f,-0.70860001250505444403415822307579219341278076171875f,0.07119999637007712323111263685859739780426025390625f,-0.12160000220090150246932125810417346656322479248046875f,-0.6342000191092491689914822927676141262054443359375f,-0.21160000201463713409566480549983680248260498046875f,0.5227000018909573331171714016818441450595855712890625f,-0.2506999792933459758614844758994877338409423828125f,-0.33279999644756319554517176584340631961822509765625f,0.036999987542629231285218338598497211933135986328125f,-0.410999980807304154950543306767940521240234375f,-0.195399973905086454806223628111183643341064453125f,0.448299991545081155663865501992404460906982421875f,-1.1986999837040901439877416123636066913604736328125f,-0.8715999936699869721223876695148646831512451171875f,0.4794999843761325042379439764772541821002960205078125f,0.2825999968290329178444153512828052043914794921875f,0.17679999388456391073987106210552155971527099609375f,-0.258099992783367593318644139799289405345916748046875f,0.926599992179870213249159860424697399139404296875f,0.27559998276233699243675800971686840057373046875f,-0.822199979114532464308240378159098327159881591796875f,-0.19270002375841155384250669158063828945159912109375f,0.239400013661384747365445946343243122100830078125f,0.57060001334697008079643865130492486059665679931640625f,0.040299994957447182741816504858434200286865234375f,0.1820000104904178073184084496460855007171630859375f,0.2194000065833330292974778785719536244869232177734375f,0.8191999955654143406036382657475769519805908203125f,0.58000000447034860684425439103506505489349365234375f,-0.314499988324940193962220291723497211933135986328125f,0.0045999974846839819520027958787977695465087890625f,0.1177999976396559844715739018283784389495849609375f,0.0189999956116080337409357525757513940334320068359375f,0.4078999894022941052895703251124359667301177978515625f,0.349999996274709790355927907512523233890533447265625f,-0.1399999929964542422222706363754696212708950042724609375f,-0.1694000157475468171242027892731130123138427734375f,0.1613999967336656737870725919492542743682861328125f,0.3234000005558133050698188526439480483531951904296875f,-1.2537000242352487333619137643836438655853271484375f,-0.62580000026226034748333404422737658023834228515625f,0.8238000076159834872413512130151502788066864013671875f,-0.1402000002145769030903466045856475830078125f,-0.10859998627901035206377855502068996429443359375f,0.15930001288354400656999132479541003704071044921875f,-0.0518999749183652880901718162931501865386962890625f,0.090200017946958865877604694105684757232666015625f,-0.24559998276233674374680049368180334568023681640625f,-0.54810000470429642849268248028238303959369659423828125f,-0.25290002576112780019457204616628587245941162109375f,0.430599989169835328794988527079112827777862548828125f,0.524900012446939623345087966299615800380706787109375f,1.088500045180321507132248370908200740814208984375f,0.009899987077712779637295170687139034271240234375f,0.811200001026689765382116092951036989688873291015625f,1.646400042331219282232268596999347209930419921875f,0.47739998484253920452147212927229702472686767578125f,-0.878799994242191129245611591613851487636566162109375f,-0.152000052213669079037572373636066913604736328125f,-0.090199995446205161186981058563105762004852294921875f,-0.542699968661368270517186829238198697566986083984375f,-1.116299990367888739228874328546226024627685546875f,0.4189999706149105662689180462621152400970458984375f,0.663399978613853580355907979537732899188995361328125f,0.163400019741057889177682227455079555511474609375f,-0.65509998535513869200030967476777732372283935546875f,0.2937999877974390994239684005151502788066864013671875f,0.5798000152826308095654894714243710041046142578125f,-0.299199995118379913350281640305183827877044677734375f,0.45050000973790849112532441722578369081020355224609375f,0.62770004611015384199390609865076839923858642578125f,-0.285800001752376520158804851234890520572662353515625f,0.8058999922111629299337209886289201676845550537109375f,0.4235000374317170468430049368180334568023681640625f,-0.26169998245239245449056397774256765842437744140625f,-0.717099989065527854137371832621283829212188720703125f,-1.131500034689903344542472041212022304534912109375f,-0.2795999892145395460119061681325547397136688232421875f,0.05229999915212391048413564931252039968967437744140625f,-0.33849999473989011988805941655300557613372802734375f,-0.51629996414184642361533406074158847332000732421875f,-0.559299968627095633877388536348007619380950927734375f,-0.743299981188773273288461496122181415557861328125f,-0.0668999941408632281536483787931501865386962890625f,-0.91979999751299612587018827980500645935535430908203125f,-0.21060002895593665783735559671185910701751708984375f,0.1028999742031100339545446331612765789031982421875f,0.633999994866549787531084803049452602863311767578125f,1.120400037682056648691286682151257991790771484375f,0.3741999748528002101011225022375583648681640625f,0.054899998402595517887903042719699442386627197265625f,0.538799993348121564196162580628879368305206298828125f,0.47960000344514863712674923590384423732757568359375f,-0.146599992626905428760863969728234224021434783935546875f,-0.0674000099137426200712752688559703528881072998046875f,0.11299999867379673101908110766089521348476409912109375f,0.11560000243186951962570674368180334568023681640625f,0.4437000036716458595265066833235323429107666015625f,0.4050000104308129067476329510100185871124267578125f,-0.1544999977499246224255102788447402417659759521484375f,0.0358000010445713723328253763611428439617156982421875f,0.00640000157654274648422187965479679405689239501953125f,0.001999999500811089347962479223497211933135986328125f,0.36089999381303794478981217253021895885467529296875f,-0.186800034415721682989897090010344982147216796875f,-0.71649999106675388294007689182762987911701202392578125f,0.660300040704012047854121192358434200286865234375f,0.05719999867230629586600798575091175734996795654296875f,-0.80090001892149442763724209726206026971340179443359375f,0.225899997314810807580442997277714312076568603515625f,0.2816000069320201415479232309735380113124847412109375f,0.082000000506639469932679276098497211933135986328125f,-0.6157000329375268421472355839796364307403564453125f,-0.191400014615059177458533667959272861480712890625f,0.4740000055208801743589219768182374536991119384765625f,-0.37919999333024012599935304024256765842437744140625f,-0.1193999872863293898461733988369815051555633544921875f,0.374999991618096828460693359375f,-0.973500028342008771886639806325547397136688232421875f,-0.5094000001013274658845375597593374550342559814453125f,0.676500019267201491146579428459517657756805419921875f,0.42239999214410772765404544770717620849609375f,-0.04160002704858722921699154539965093135833740234375f,-0.698499984420835939857852281420491635799407958984375f,-1.3748000093221666162435212754644453525543212890625f,-0.9949999979138368644271395169198513031005859375f,0.6185000133663416388429823200567625463008880615234375f,-0.07729997892379714130584034137427806854248046875f,-0.419200021195411665075880591757595539093017578125f,-0.46520001548826694914851032081060111522674560546875f,-0.90470001159906388021880729866097681224346160888671875f,-0.4813999938279389656514695161604322493076324462890625f,0.6554000042662024849704494044999592006206512451171875f,-0.348499984383582983582527958787977695465087890625f,-0.407800013583898390834292513318359851837158203125f,-0.397799990934133429476560195325873792171478271484375f,0.194400000473856859883881043060682713985443115234375f,-0.2567999898612498554939520545303821563720703125f,-0.322899984149634666774630886720842681825160980224609375f,-0.67569998901635408561361373358522541821002960205078125f,-1.0037000253155827778073216904886066913604736328125f,-0.361299998861550619722038391046226024627685546875f,-0.1401000022679566026084785335115157067775726318359375f,-0.6765000269412997369045115192420780658721923828125f,0.036399996398389367069370337048894725739955902099609375f,0.282199988912046062505822874300065450370311737060546875f,0.403000001914799010904744136496447026729583740234375f,0.72619998683333353284297118079848587512969970703125f,-0.1703999874651433987793325286475010216236114501953125f,-0.5459999769330021734958791057579219341278076171875f,0.55659997793063542559366396744735538959503173828125f,0.087000005405396318991506632301025092601776123046875f,-0.454200021288543975828133625327609479427337646484375f,-0.31019999075233950502905599933001212775707244873046875f,0.43169999284595250710339087163447402417659759521484375f,0.41850001530349256739782504155300557613372802734375f,-0.28129997293353081744271548814140260219573974609375f,-0.8600000121071931236116370200761593878269195556640625f,-0.7540000457167632674781998503021895885467529296875f,-0.144299995002150627243509006802923977375030517578125f,0.0345000012516975262943930147230275906622409820556640625f,0.4064999984800816701380199447157792747020721435546875f,0.013899993678927469176187514676712453365325927734375f,-0.29309999751448645444185103769996203482151031494140625f,0.4905000175237657078497477414202876389026641845703125f,0.4809999864697456217754734097979962825775146484375f,-0.59609997119158542044914383950526826083660125732421875f,-1.28249998539686149712224505492486059665679931640625f,0.5576999689221384670645420555956661701202392578125f,-0.586599982754886450919684648397378623485565185546875f,0.475000012218952605280719581060111522674560546875f,0.723399996346234530619767610915005207061767578125f,-0.633099960671365646902586377109400928020477294921875f,-0.835099990570544292722843238152563571929931640625f,0.03969999658614387083588326277094893157482147216796875f,-0.034600004041194953874338580135372467339038848876953125f,0.824200009758770502088509601890109479427337646484375f,0.486899983114004175632771875825710594654083251953125f,0.12030000300407406310654323533526621758937835693359375f,0.16469998747780933978646089599351398646831512451171875f,0.356599999071657569604809623342589475214481353759765625f,0.1551999989293515458399497219943441450595855712890625f,-1.26550000053644140507458359934389591217041015625f,-0.9814000122308730311715407879091799259185791015625f,0.481599992887675742991859806352294981479644775390625f,-0.34530000568628338442067615687847137451171875f,-0.0479999933838843872990764793939888477325439453125f,0.26910000130683187080649076960980892181396484375f,-0.701799982559681012617147644050419330596923828125f,-0.5179999755024908836276154033839702606201171875f,0.456499998144805463340389906079508364200592041015625f,0.5052000023007394702290184795856475830078125f,0.4315999859213821565617763553746044635772705078125f,-0.292700020219385681485846362193115055561065673828125f,0.5422999947190287883813653024844825267791748046875f,0.477799995255470122401675325818359851837158203125f,-0.1232000037685037074908223075908608734607696533203125f,-0.0916000002264976131982621154747903347015380859375f,-0.27700002151727698418426371063105762004852294921875f,-0.2837000050872564571591283311136066913604736328125f,0.16050000977516187816718229441903531551361083984375f,0.1948000199019912503928253499907441437244415283203125f,0.12470000269562009354018528028973378241062164306640625f,1.15030001194477105030955499387346208095550537109375f,0.7079999954700468833834747783839702606201171875f,-0.65880000698268414982550211789202876389026641845703125f,0.1782000076428056356547813265933655202388763427734375f,0.09100000680983078016339504756615497171878814697265625f,-0.0681999999314546467843456412083469331264495849609375f,0.223200012671947778386538630002178251743316650390625f,-0.2014000217676166215596822439692914485931396484375f,-0.5045000121295453165970457121147774159908294677734375f,0.470999984234571567043303730315528810024261474609375f,0.506599994266033082368494433467276394367218017578125f,-0.043699997115135225111970385114545933902263641357421875f,0.783299995791912007092605563229881227016448974609375f,0.27499998241663003994972314103506505489349365234375f,-0.68899998838454479255233309231698513031005859375f,1.368900009250641147673377417959272861480712890625f,1.0052000043869018242048696265555918216705322265625f,-0.563000004634261141944762130151502788066864013671875f,0.64110001727342602606540822307579219341278076171875f,0.2309000161647798421427069115452468395233154296875f,-0.54889996873810920074987507177866064012050628662109375f,-0.185999994460493367487430305118323303759098052978515625f,-0.494400001274794220762487384490668773651123046875f,0.63889999598860658380772292730398476123809814453125f,-0.63199997245520389554940265952609479427337646484375f,0.30680001951456148390207090415060520172119140625f,-0.2947999933779232861752461758442223072052001953125f,-0.9059999945908783214321147170267067849636077880859375f,-1.0872000524282459110736454022116959095001220703125f,0.3511999646306041000798359164036810398101806640625f,0.712899988396465733586637725238688290119171142578125f,1.806500024259090508849112666212022304534912109375f,-0.37839995834827444554093744955025613307952880859375f,-0.714900003655254412393560414784587919712066650390625f,-1.622100065433979665385777479968965053558349609375f,-0.3814999725520618056862076628021895885467529296875f,0.39460000191777933054027016623876988887786865234375f,0.04940002707242996393688372336328029632568359375f,0.427900000429153326564346571103669703006744384765625f,-0.6193999964132903546243369419244118034839630126953125f,-0.8266000291347506134798095445148646831512451171875f,-0.1878000081449748837059132711146958172321319580078125f,0.7884000091612335392454724569688551127910614013671875f,1.0416000547647483731594775235862471163272857666015625f,-0.2201999851644043104670345201157033443450927734375f,0.485300005276501078554929335950873792171478271484375f,-0.383899980640411087051688809879124164581298828125f,0.019699992637336183332052996775018982589244842529296875f,0.440700001458823542588305599565501324832439422607421875f,0.507900024308264530503720379783771932125091552734375f,0.630099995785950994786617229692637920379638671875f,0.1307999997779727419100481711211614310741424560546875f,0.0420000064671040007624469581060111522674560546875f,-0.5250999703049663569487393033341504633426666259765625f,0.0761999983444809902977112869848497211933135986328125f,-0.3845999927163123999207527958787977695465087890625f,0.258299989831447529553543063229881227016448974609375f,0.2856000017985700356604183980380184948444366455078125f,-0.2939999749362467440505497506819665431976318359375f,-0.5727999766290192695805671974085271358489990234375f,0.8271000046744940714660287994774989783763885498046875f,-0.1004999623894686777703100233338773250579833984375f,-0.05469999565482108749847611761651933193206787109375f,-0.1775000139325857428929111847537569701671600341796875f,-0.366699981856346113318068091757595539093017578125f,-0.34599998229742023880817214376293122768402099609375f,0.1830999992653727925784323815605603158473968505859375f,-0.5875000151991844887788829510100185871124267578125f,-0.0507999832749366220241427072323858737945556640625f,0.7363000053808093081642027755151502788066864013671875f,-0.125800010395050065881150658242404460906982421875f,0.117200020241738034343370600254274904727935791015625f,0.4207000180736184535845723075908608734607696533203125f,-0.3956999971747396926957662799395620822906494140625f,0.1660000264048573370700978557579219341278076171875f,0.7258999873682856485146430713939480483531951904296875f,0.275300009560584868495425325818359851837158203125f,0.02580000234842305673055307124741375446319580078125f,-0.281699997726082784765821998007595539093017578125f,0.174500004529952956744409675593487918376922607421875f,0.01039999476671216172007916611619293689727783203125f,-0.242600001299381251573805684529361315071582794189453125f,0.70639999390244501142888111644424498081207275390625f,0.695000006556510729893716415972448885440826416015625f,-0.0305000009089708701282006586552597582340240478515625f,1.0778999873161314582148406771011650562286376953125f,0.65379999185800574679205965367145836353302001953125f,-0.5616999828249216175635183390113525092601776123046875f,-0.17330000087693331334293134204926900565624237060546875f,-0.127200003589689691896325030029402114450931549072265625f,0.03209999906793237212099256794317625463008880615234375f,0.0006000008299948955681202278356067836284637451171875f,-0.15780001157224210484031345913535915315151214599609375f,-0.1014000054508448211976201491779647767543792724609375f,0.29339999694228158233499925700016319751739501953125f,0.331000007629394854546944770845584571361541748046875f,-0.0504999928623437455144085106439888477325439453125f,-1.43250001192092923929521930404007434844970703125f,-1.0277999949574461169277128647081553936004638671875f,0.631500021651387299925772822462022304534912109375f,-1.03360001995563521859367028810083866119384765625f,-0.44039999029636334171300404705107212066650390625f,0.779400013363361399143514063325710594654083251953125f,0.0073999869659544526001582198659889400005340576171875f,0.15999999262392494703277634471305646002292633056640625f,0.1441999968692659894742291726288385689258575439453125f,0.31369999458193742469802600680850446224212646484375f,-0.199999984800815155949749168939888477325439453125f,0.341799969595670827260391888557933270931243896484375f,0.5692000031650064073573958012275397777557373046875f,1.26080003393888517848608898930251598358154296875f,-0.488199988979101107133828918449580669403076171875f,0.0795999918222427549352460118825547397136688232421875f,0.6464000038862227626168532879091799259185791015625f,-0.3220000094473363105862517841160297393798828125f,0.805900007335841106481666429317556321620941162109375f,1.333100072169304706193315723794512450695037841796875f,-0.7241999742567539755100369802676141262054443359375f,0.0809999945908785878856406270642764866352081298828125f,0.4421999862670897840644101961515843868255615234375f,0.2434999797642234398153959773480892181396484375f,0.36909999076426036612730285924044437706470489501953125f,0.32390000329017620828153667389415204524993896484375f,0.148599995517730842919945644098334014415740966796875f,-0.147499998621642536278386614867486059665679931640625f,-0.38230001155734072426639613695442676544189453125f,-0.6260999745070938704571972266421653330326080322265625f,0.833799985711276558930649116518907248973846435546875f,0.5450000278651714324951171875f,-0.16690000188946729764438714482821524143218994140625f,0.89729998518228526460660532393376342952251434326171875f,0.32490004131793970287844786071218550205230712890625f,0.0604000019378959918103788595544756390154361724853515625f,-0.365799990967661192708959561059600673615932464599609375f,-0.39020000842884172698177280835807323455810546875f,0.336899999170004738058281645862734876573085784912109375f,0.51509998561590919496921969766844995319843292236328125f,0.8223000196412204587659289245493710041046142578125f,-0.3700999954879280462449742117314599454402923583984375f,-0.2231999874517323068090490778558887541294097900390625f,-0.6600000022351739659143277094699442386627197265625f,0.556299960714579100340415607206523418426513671875f,-0.7646999703228478661998224197304807603359222412109375f,0.2565000209808356856910904753021895885467529296875f,-0.20020000021457651229184193653054535388946533203125f,-0.6583999830096962657677295283065177500247955322265625f,-0.0327999970436092525005733477883040904998779296875f,-0.317599972875416691753258646713220514357089996337890625f,-0.17600000327080478346175596016109921038150787353515625f,-0.306400008468330042887828312814235687255859375f,-0.4996000013589851906914418577798642218112945556640625f,-0.279899990579485891117883511469699442386627197265625f,0.438100006306171518843939338694326579570770263671875f,0.256199993836879524877758740331046283245086669921875f,-0.314700007724761976479754821411916054785251617431640625f,-0.5260000365376473752121455618180334568023681640625f,-0.3378000149250031114434023038484156131744384765625f,0.4056000190839170738144048300455324351787567138671875f,0.5870999875009059731922889113775454461574554443359375f,0.42599998822063211978417029968113638460636138916015625f,-0.2990999938935041679854975882335565984249114990234375f,-0.22410000017061804111762057800660841166973114013671875f,-0.335400017900765068379342892512795515358448028564453125f,-0.110100005266815459403773047597496770322322845458984375f,0.5949000264167789708835698547773063182830810546875f,-0.013800027024746697179580223746597766876220703125f,-0.8319000272959471420364252480794675648212432861328125f,-0.86680002905130404400324550806544721126556396484375f,-0.50860000624656631629250114201568067073822021484375f,0.52360002043992281972606406270642764866352081298828125f,0.344000003099441897802535095252096652984619140625f,0.50720001815557491653407851117663085460662841796875f,0.2310999963745474783394229234545491635799407958984375f,0.46440002200603469617590235429815948009490966796875f,0.021400005376339237272986792959272861480712890625f,-0.52329999750554556658954652448301203548908233642578125f,-0.7357000117778778047750165569595992565155029296875f,-0.52180000818967808839943245402537286281585693359375f,0.3259000046908855363625434620189480483531951904296875f,-0.42310001366361993380138528664247132837772369384765625f,-0.340800017808378041817007897407165728509426116943359375f,0.05670000009164211096646113219321705400943756103515625f,-0.7384000054359436404638472595252096652984619140625f,-0.53920001165866882075761168380267918109893798828125f,0.2475999936997890227274865537765435874462127685546875f,-1.0617000137448311392063260427676141262054443359375f,-0.7354000153303150710826230351813137531280517578125f,0.4539999947920441147886094768182374536991119384765625f,0.3775000216066839442419222905300557613372802734375f,0.1077999948084353842858718053321354091167449951171875f,-0.45730001431405542522412588368752039968967437744140625f,1.12449998694658237496923902654089033603668212890625f,0.8542000027179721399761547218076884746551513671875f,-0.48519998050779100662310838742996565997600555419921875f,0.787700034487247791048503131605684757232666015625f,0.51530006022453367364732912392355501651763916015625f,-0.072099988394975778049911241396330296993255615234375f,-0.138699996966123506592083458599518053233623504638671875f,-0.643100014799833363809966613189317286014556884765625f,0.1652999973416324763775264727883040904998779296875f,0.5778999657839538617309926848975010216236114501953125f,1.4123000301837915770875042653642594814300537109375f,-0.251399982856959358645809743393328972160816192626953125f,0.00960000242069358744601004218566231429576873779296875f,-0.574799991124123277330681958119384944438934326171875f,0.4293000017262988965427439325139857828617095947265625f,-0.198899991536885589749061864495160989463329315185546875f,-0.05129999090805625883149332366883754730224609375f,0.6197000075757497228323700255714356899261474609375f,-0.16530000002384159785151496180333197116851806640625f,-0.77370002930164361032439046539366245269775390625f,0.11629999245405198138314517564140260219573974609375f,-0.37189999496042747129109784509637393057346343994140625f,0.27530001760721223291739079286344349384307861328125f,-0.622199978220463467692979975254274904727935791015625f,0.7759000103905793199743357035913504660129547119140625f,0.8063000553369530365444006747566163539886474609375f,0.44749996483326004437230949406512081623077392578125f,0.4686000018879770134816453719395212829113006591796875f,0.3126000164985658358318687533028423786163330078125f,0.2720999958455561085202134563587605953216552734375f,-0.42769999425411231452898164206999354064464569091796875f,0.07670000033378610027057220577262341976165771484375f,0.2112000048451127820481332264535012654960155487060546875f,-0.81020001047775114244586802669800817966461181640625f,-0.8194000524602838009258221063646487891674041748046875f,0.28480000306367880824609528644941747188568115234375f,-0.84200000229477856006354841156280599534511566162109375f,-0.55720004646778154011599326622672379016876220703125f,0.346700004655122739904982154257595539093017578125f,-0.8580000121966000481421588119701482355594635009765625f,-0.2640000510811812972633561003021895885467529296875f,0.1437000046402214792351514915935695171356201171875f,-0.5100999741792680008956040182965807616710662841796875f,-1.1369000167906280207574809537618421018123626708984375f,-0.5436999981582157914772324147634208202362060546875f,0.3145999897733331440718984595150686800479888916015625f,-0.6505999994516376006004065857268869876861572265625f,0.45909999560713732336125758592970669269561767578125f,0.3186999987170100467892552842386066913604736328125f,0.467999982059001951029131305404007434844970703125f,0.406599984431267102991114370524883270263671875f,-0.276999990560114373039368729223497211933135986328125f,0.808499981701373826581402681767940521240234375f,0.31999997884035114026346491300500929355621337890625f,-0.601699981670081651685677570640109479427337646484375f,0.2685999945864081528412725674570538103580474853515625f,0.29180000304877740102682537326472811400890350341796875f,0.0388000002771615726260279188863933086395263671875f,-0.5349999874830242418966008699499070644378662109375f,-0.21579998893737784015911529422737658023834228515625f,0.39099998658150436625646761967800557613372802734375f,0.23280000434517855723015600233338773250579833984375f,0.110000004470348411445002057007513940334320068359375f,-0.117199997666478150648572409409098327159881591796875f,-0.1584999888539313506186090307892300188541412353515625f,-0.374400009340047912331783663830719888210296630859375f,-0.2130000051558017803987610250260331667959690093994140625f,-0.6898000331640243842912241234444081783294677734375f,-0.0931999986648559541890790569595992565155029296875f,0.7748000101417302776241058381856419146060943603515625f,-0.0619999875426291424673763685859739780426025390625f,-0.3266000178098682482641379465349018573760986328125f,-0.3526000131830573369740022826590575277805328369140625f,-0.2886000015899539139496710049570538103580474853515625f,-0.25380000772774213846361135438201017677783966064453125f,0.00599999950826168859663312105112709105014801025390625f,-0.5261999954760074871273900498636066913604736328125f,-0.2321999852240088646482263357029296457767486572265625f,0.48119999268203972331292561648297123610973358154296875f,-0.40770000414475815464498964502126909792423248291015625f,-0.464400015263259635656112322976696304976940155029296875f,-0.0522000002555549347249552738503552973270416259765625f,-0.131100005352497106514420011080801486968994140625f,-0.134400009787082552747961017303168773651123046875f,-0.059699997554719441694714987534098327159881591796875f,-0.6247000057876110901133870356716215610504150390625f,-0.2921999896943570096397024826728738844394683837890625f,0.52650000116229056512651141019887290894985198974609375f,0.45809999827817093898119082950870506465435028076171875f,0.468000009216368140219088900266797281801700592041015625f,0.0255000001452863056528030938352458178997039794921875f,0.8454999817609785139893574523739516735076904296875f,1.7566000362873079865266845445148646831512451171875f,-0.3680999747157098767047500587068498134613037109375f,-0.752099973046779890495372455916367471218109130859375f,-1.642500002384185364689983543939888477325439453125f,0.2159999895989894724834812222979962825775146484375f,0.75179997212886828794609073156607337296009063720703125f,0.39540002486705727591242975904606282711029052734375f,-0.286699989269673938263593981901067309081554412841796875f,0.44569998447150005471684153235401026904582977294921875f,0.4009000062927601248929931898601353168487548828125f,-0.135699987041950631549980244017206132411956787109375f,-0.5146999669700864643573368084616959095001220703125f,-0.28749999493360434144051396287977695465087890625f,0.0377999906361103921170752073521725833415985107421875f,0.45539999282956122872434434611932374536991119384765625f,0.47940001500248907717605106881819665431976318359375f,0.5813999828010796289134987091529183089733123779296875f,-0.010099998281896116480993441655300557613372802734375f,0.1895000017732382247004352393560111522674560546875f,-0.522599974477291340235751704312860965728759765625f,-0.2516000076025723952710677622235380113124847412109375f,0.2324000001907347012775062466971576213836669921875f,0.564099994862079512358832289464771747589111328125f,-0.4435000003278253277727571912691928446292877197265625f,-0.5723000331640246685083184274844825267791748046875f,0.183699999089539101060353232242050580680370330810546875f,-0.6098999880835414000301852865959517657756805419921875f,-0.6435000175759195695235348466667346656322479248046875f,-0.057500003054737902630222379229962825775146484375f,0.28649998015165356601841040173894725739955902099609375f,0.9440999878585334403879869569209404289722442626953125f,0.4871999884091320343060260711354203522205352783203125f,-0.470700011796503947625325281478581018745899200439453125f,-0.287100036134571467982823378406465053558349609375f,0.1088000024378299457339380751363933086395263671875f,-0.548999986708164300353018916212022304534912109375f,-0.708600009822845322560169734060764312744140625f,-0.04350000427663314184201226453296840190887451171875f,0.1662999850034715121349648825344047509133815765380859375f,0.8554999977052208937067234728601761162281036376953125f,-0.43239998679831648242810615556663833558559417724609375f,0.71219999097958197609159469720907509326934814453125f,0.61500002758577476669188399682752788066864013671875f,-0.42259998088479022726460243575274944305419921875f,0.295099996568262579188512972905300557613372802734375f,-0.019600004041194551973603665828704833984375f,0.1075999887824057310581338242627680301666259765625f,0.028899989692866778767665891791693866252899169921875f,0.741400015509128706980845890939235687255859375f,0.32759999593496313963214561226777732372283935546875f,-0.55770000643581152388605914893560111522674560546875f,-0.4731999989256261418546500863158144056797027587890625f,-0.47440001507699480765012367555755190551280975341796875f,-0.0311000036388635603401553453295491635799407958984375f,0.280699996876716539873086730949580669403076171875f,0.22659999694824239924173525650985538959503173828125f,-0.1899999960139393540003993621212430298328399658203125f,0.53399999743700021781478426419198513031005859375f,0.565200008857249969196345773525536060333251953125f,-0.020700002129375949522227529087103903293609619140625f,-0.5862000164866447704525853623636066913604736328125f,-0.1509999957084655619610202847979962825775146484375f,0.5584000036850571557778266651439480483531951904296875f,0.456199992719292790610552401631139218807220458984375f,0.5954000110089776587329879475873894989490509033203125f,0.178299998101592027666129069984890520572662353515625f,-0.797400008535384952068625352694652974605560302734375f,-0.63460001029968271524239753489382565021514892578125f,0.357099999943375545630175338374101556837558746337890625f,0.738000000536441547183130751363933086395263671875f,0.640000011622905873309719027020037174224853515625f,-0.238299993966519796373404460609890520572662353515625f,0.149599987649917665066823246888816356658935546875f,0.17039998761415464656465701409615576267242431640625f,-0.0627000004753470729923492399393580853939056396484375f,-0.92050000768899931102851041941903531551361083984375f,-0.5825999902725218504428994492627680301666259765625f,0.61080000760108232871203881586552597582340240478515625f,-0.55779999436140048629795273882336914539337158203125f,-0.16579998238086712802896727225743234157562255859375f,0.55699999115616083411595127472537569701671600341796875f,0.256999995701014949389673347468487918376922607421875f,0.34040000765621669653882008788059465587139129638671875f,0.0544000029698014364587521640714840032160282135009765625f,0.7008999973893164092686447474989108741283416748046875f,0.496599998885393301151225387002341449260711669921875f,-0.3747999934896826257357815848081372678279876708984375f,-1.1485000278949737406719577847979962825775146484375f,-0.23770002197027206847224078956060111522674560546875f,-0.26919996516704625122429206385277211666107177734375f,-0.788899972482025813036443651071749627590179443359375f,-1.219300011169909936370459035970270633697509765625f,0.11679999775886518165179950301535427570343017578125f,0.39350000207871194124464864216861315071582794189453125f,-0.10009998759031280002318453625775873661041259765625f,0.424199998210370299300819851850974373519420623779296875f,-0.83509997581839578995044348630472086369991302490234375f,-0.7903000113114710956097042071633040904998779296875f,-0.187899992829561046647768307593651115894317626953125f,-0.4801999843075872576036999817006289958953857421875f,-1.02700001168251020544630591757595539093017578125f,-0.56519998173713670297502176254056394100189208984375f,0.3470999832913281846202835367876105010509490966796875f,-0.0321000117897991543713942519389092922210693359375f,-0.307699985909462014888049452565610408782958984375f,-0.3405000000894069511758743828977458178997039794921875f,0.0146999916315078138495664461515843868255615234375f,-0.68469997352659739409119765696232207119464874267578125f,0.755799983684718679199932012124918401241302490234375f,0.6614000212460755534493728191591799259185791015625f,0.456699993926286129664049440179951488971710205078125f,0.080399995549023106633512725238688290119171142578125f,0.93480002109408388832889613695442676544189453125f,-0.516999995589255689765195711515843868255615234375f,0.143699995587766249371952653746120631694793701171875f,-0.537900012052059395273317932151257991790771484375f,-0.203799978148937466215784297673963010311126708984375f,-0.3718999990954994228076202489319257438182830810546875f,-0.7351000159025193880779625033028423786163330078125f,0.23909997951388373138570386799983680248260498046875f,0.288399986176192779652183162397705018520355224609375f,1.354400016045570254163976642303168773651123046875f,-0.23349999332427984199966886080801486968994140625f,0.01270000058710574197817777530872263014316558837890625f,0.53749999672174464393492598901502788066864013671875f,-0.411399989320337666764970663280109874904155731201171875f,0.08200000017136332974843071497161872684955596923828125f,-0.1587999985262751323489283095113933086395263671875f,-0.2776999950736758204783427572692744433879852294921875f,-0.0165999974325299337607475536060519516468048095703125f,-0.727400004735589078563862130977213382720947265625f,0.2065000008642670081826508976519107818603515625f,0.187300001536309768113142126821912825107574462890625f,-0.6045000063180925309325175476260483264923095703125f,-0.01999997377395601461103069595992565155029296875f,0.7262000047147274539582895158673636615276336669921875f,0.215099988335371250514072016812860965728759765625f,0.348199998366832463858600021922029554843902587890625f,0.15819999691396946506216636407771147787570953369140625f,-0.675399991600215354736747030983678996562957763671875f,-0.370199993099272595120652340483502484858036041259765625f,0.412499985639005917192889683065004646778106689453125f,-0.0661999762535092628468191833235323429107666015625f,0.1450000312924384360258045489899814128875732421875f,0.3670999858617782773961835118825547397136688232421875f,0.1831999907672408056669155484996736049652099609375f,0.426600005590915376529892455437220633029937744140625f,0.29459999372214074053744070624816231429576873779296875f,-0.0148000063419342353654428734444081783294677734375f,0.04719999699592580810758590814657509326934814453125f,0.0482000003784894825997753287083469331264495849609375f,-0.570499988615512876322100055404007434844970703125f,-0.3307999838709836382122375653125345706939697265625f,0.463699987615644904082046195981092751026153564453125f,0.7998000213921070411515756859444081783294677734375f,0.4394000054657454246154202337493188679218292236328125f,-0.45800001383572819246836616002838127315044403076171875f,0.248899967825412460342704434879124164581298828125f,0.092599972391128826387785011320374906063079833984375f,-0.3190999780982732847434135692310519516468048095703125f,0.956600037479400810980223468504846096038818359375f,0.71260002812147149597876705229282379150390625f,-0.382300014984607638979241528431884944438934326171875f,0.29789999625682828110484479111619293689727783203125f,0.163399993515014774203564229537732899188995361328125f,-0.22909999560713767419173336747917346656322479248046875f,-0.3551000135183333128452431992627680301666259765625f,-0.3378000146269801717835434828884899616241455078125f,0.1653000009924172619690807550796307623386383056640625f,0.53030000374913210947624975233338773250579833984375f,0.579200020599365483775500251795165240764617919921875f,0.0205000039264560118823510492802597582340240478515625f,-0.54280001038014891179273035959340631961822509765625f,-0.29080000182688225951466165497549809515476226806640625f,0.329800003696978094591685248815338127315044403076171875f,-0.433300028276443871533274432295002043247222900390625f,-0.4336000133991237959207865060307085514068603515625f,-0.5869999939501280294962271000258624553680419921875f,-0.369299991835653873550882053677923977375030517578125f,-0.036900013363361239271398517303168773651123046875f,-0.4570999706625944458693311389652080833911895751953125f,0.3431999914005399165972676200908608734607696533203125f,0.2796000146210193548768074833787977695465087890625f,0.0313000052504242010087409653351642191410064697265625f,-0.455600001016259026354049410656443797051906585693359375f,-0.046000035680830719808653839209000580012798309326171875f,-0.3186999994993211515748043893836438655853271484375f,0.7682000016823409271893297045608051121234893798828125f,1.2178000283360486122319343849085271358489990234375f,-0.6428999648153788104565364847076125442981719970703125f,0.3798999927401542908711462587234564125537872314453125f,0.3503000141799450517510194913484156131744384765625f,-0.1159999918341636515606296597979962825775146484375f,-0.16460000366866578414004607111564837396144866943359375f,-0.24940001246929188738477023434825241565704345703125f,0.103799990218877979941680678166449069976806640625f,0.5179999682009224226675314639578573405742645263671875f,1.3575999819278710134540233411826193332672119140625f,-0.0301000012248753279209267930127680301666259765625f,0.7346999758362773302877712922054342925548553466796875f,0.3554999872744077915598381878226064145565032958984375f,0.0393000052094460983198587200604379177093505859375f,-0.61489999855160704100853763520717620849609375f,-1.183300010693073556211629693279974162578582763671875f,-0.4204999996423719466065449523739516735076904296875f,0.575800016914307821735974357579834759235382080078125f,-0.075399984633921945942347520031034946441650390625f,-0.028499997496604922986307428800500929355621337890625f,-0.113699993762373996464276615370181389153003692626953125f,-0.465299997341632742831052382825873792171478271484375f,0.33369999994635524132036152877844870090484619140625f,0.49189997491836567888867648434825241565704345703125f,1.12990001807212792073187301866710186004638671875f,-0.5903999938726423035717516540898941457271575927734375f,0.4834000032007692571056622909964062273502349853515625f,0.1790000208318236474269724567420780658721923828125f,0.00900000261515376820398159907199442386627197265625f,-0.42799999353289608361450291340588591992855072021484375f,-0.13360000572502606086544574282015673816204071044921875f,-0.636399997627734581584491024841554462909698486328125f,0.024200001563131845472298664390109479427337646484375f,0.1916999940752983633274197927676141262054443359375f,0.27719999878406564874921969021670520305633544921875f,-0.060599995800852735072794530424289405345916748046875f,-0.3805999888345599213579362185555510222911834716796875f,-0.50200000505149355856104875783785246312618255615234375f,-0.090200002412497981563177518182783387601375579833984375f,-0.56820003144741093592529068700969219207763671875f,-0.07060000243186959067998031969182193279266357421875f,0.5458000105813145541588937703636474907398223876953125f,0.4706000149488449579848747816868126392364501953125f,0.07299999457597738938829934340901672840118408203125f,-0.5107000026136636616769237662083469331264495849609375f,-0.6057000242948529233899535029195249080657958984375f,-0.0798000060439107983256690204143524169921875f,0.5781000021338462691034010276780463755130767822265625f,-0.1435999952197075657522873370908200740814208984375f,0.297800032806396330897769075818359851837158203125f,0.5919999944344163100851119452272541821002960205078125f,-0.1205999762058258539809685316868126392364501953125f,-0.21679997719526244992493957397527992725372314453125f,0.028100002469122475901031066314317286014556884765625f,-0.20429998786821979006589344862732104957103729248046875f,-0.365599998743831922087821340028312988579273223876953125f,-0.148599997846037112925188239387352950870990753173828125f,0.01129999930858627266161420266143977642059326171875f,-0.26620002274513243634146419935859739780426025390625f,-0.35540000039190056124738248399808071553707122802734375f,0.8226000105381015004013534053228795528411865234375f,0.5319999902248380152514073415659368038177490234375f,-0.5272000133872032368032023441628552973270416259765625f,0.24339998457431821776708602556027472019195556640625f,0.234199981558322889441114966757595539093017578125f,-0.12109999983906749587703188808518461883068084716796875f,-0.26349999943375568278725040727294981479644775390625f,-0.041799994480609559133199581992812454700469970703125f,0.228999999038875134971249281079508364200592041015625f,0.2905999893188475624583588796667754650115966796875f,0.49440001574754699475988672929815948009490966796875f,0.284100004287064056285316837602294981479644775390625f,-0.4173000173687935188127084984444081783294677734375f,-0.0925999920606610515960710472427308559417724609375f,0.4988000120863318453956480880151502788066864013671875f,-0.08439999116063134909637710734386928379535675048828125f,0.14510001862198151911798049695789813995361328125f,-0.391999981433153177334816064103506505489349365234375f,-0.786899964897334935898243202245794236660003662109375f,-1.431300002884864142060905578546226024627685546875f,0.601399976728856699992320500314235687255859375f,-0.20139999237507588691897808530484326183795928955078125f,0.111399998857081072856090031564235687255859375f,-0.473099980080127213710738942609168589115142822265625f,-0.868500012621283179470310642500407993793487548828125f,-0.830100053155423012185565312393009662628173828125f,0.34009998580217359887001293827779591083526611328125f,0.22000000577419986935723272836185060441493988037109375f,-0.08559998842477778424608914065174758434295654296875f,-0.4948999612987048379864063463173806667327880859375f,-0.916499968901276940158595607499592006206512451171875f,-0.869699994611739413130635512061417102813720703125f,-0.71489998916387520466741989366710186004638671875f,0.432499984577298324239791327272541821002960205078125f,0.740499990701675159243677626363933086395263671875f,0.342199960637092903681377720204181969165802001953125f,0.74670000722557272609947176533751189708709716796875f,1.2963000446081167638112674467265605926513671875f,0.5157999693609778280034561248612590134143829345703125f,-0.053099997942894716640438446120242588222026824951171875f,0.161699996217340213888746802695095539093017578125f,0.526499995201825665702699552639387547969818115234375f,-0.2665999889388681598489938551210798323154449462890625f,-0.3741999916911122880947004887275397777557373046875f,-0.234399973428249808193868375383317470550537109375f,-0.2189999841004610647843264814582653343677520751953125f,-1.1526000093460080364593522972427308559417724609375f,-0.2372999684929855135351317585445940494537353515625f,-0.40899998227506895176475154585205018520355224609375f,-0.885799990129470415922696702182292938232421875f,-0.2056999891281130476983207699959166347980499267578125f,-0.49519996746927541497740321574383415281772613525390625f,-0.52599998632073319271285072318278253078460693359375f,-0.651799985241889867637610223027877509593963623046875f,0.75189997376352568636548312497325241565704345703125f,0.818300009799003191801602952182292938232421875f,-0.22269999887347236011692075408063828945159912109375f,0.319799996469914749042118273791857063770294189453125f,0.875400032913685066660036682151257991790771484375f,0.38879997513145247989285735457087866961956024169921875f,-0.4128999883219599542627520349924452602863311767578125f,1.002700008559227029536486952565610408782958984375f,0.60220000027418141286261743516661226749420166015625f,-0.56229999926388263276066936668939888477325439453125f,0.1943000031232831048555453890003263950347900390625f,0.01740000138282749020390838268212974071502685546875f,-0.16559999852031470624069697805680334568023681640625f,0.6106999892771245441736027714796364307403564453125f,0.635600003325939244547271300689317286014556884765625f,-0.03379999580681325443975993039202876389026641845703125f,-0.74829996479749671944858846472925506532192230224609375f,-0.6499999821931126131602241002838127315044403076171875f,0.249799979221820844799140104441903531551361083984375f,-0.74090000953376300429908951628021895885467529296875f,-0.39220000243485009239208238795981742441654205322265625f,0.463299995158612698542555108360829763114452362060546875f,-0.6453999825477598761835906771011650562286376953125f,-0.0877999627709389329766054288484156131744384765625f,0.7852999860912561747028348690946586430072784423828125f,-0.18490000246316196097495776484720408916473388671875f,-0.110800001566112082951320871870848350226879119873046875f,0.0927999994836747610360561111519928090274333953857421875f,0.419000020980835330419722595252096652984619140625f,0.217799990189075032276377896778285503387451171875f,-0.420900024658441562763755428022705018520355224609375f,0.738900010442733901072642765939235687255859375f,0.5592000009298327967144359718076884746551513671875f,-0.447600003460049578052348806522786617279052734375f,-0.55330000890493380438783788122236728668212890625f,-0.1541999997377399012066234718076884746551513671875f,0.48839999154061075614663423039019107818603515625f,-0.18680000789165518426671042107045650482177734375f,-0.25340001437664039229957779753021895885467529296875f,-0.1239999960586428695563654400757513940334320068359375f,-0.3912000275135039117913038353435695171356201171875f,0.1334000087141991031103316345252096652984619140625f,0.7321000089585780745693455173750407993793487548828125f,0.0336000108659266771837792475707828998565673828125f,-0.230200001704692791548723107553087174892425537109375f,-0.23779999871999024652069465446402318775653839111328125f,0.6445999907791613026120103313587605953216552734375f,0.3403999922335148031748985886224545538425445556640625f,-0.37399998894333841281678587620262987911701202392578125f,-0.0732000064134599170984074589796364307403564453125f,-0.35609999276101589327225838133017532527446746826171875f,-0.420900028383731861225669490522705018520355224609375f,-0.1688999936044215388619704754091799259185791015625f,0.1157000094681979174282560052233748137950897216796875f,-0.4782000040292739839742353069595992565155029296875f,-0.30659999814033511711386381648480892181396484375f,0.07280000509023665244967560283839702606201171875f,0.6198999991104006124231773355859331786632537841796875f,0.34829997064620243918398045934736728668212890625f,0.360200000773370276885287921686540357768535614013671875f,0.148500006958842245552432359545491635799407958984375f,-0.920899962855875831024832223192788660526275634765625f,-0.6059999955594539411407595252967439591884613037109375f,-0.08779998333454130943209747783839702606201171875f,0.425599997067450885879225097596645355224609375f,0.3720000085532664257925716810859739780426025390625f,-1.382500019073486186016452847979962825775146484375f,-0.19409997191429084750780020840466022491455078125f,-0.163899999713897681630214719916693866252899169921875f,0.325599987828731673999982376699335873126983642578125f,-0.3444000051677225116009140037931501865386962890625f,0.0740000004917382714353379924432374536991119384765625f,1.175100008153915354114360525272786617279052734375f,-0.388700031685829117122921161353588104248046875f,-0.00710000717043879348011614638380706310272216796875f,0.4098999984025955001243346487171947956085205078125f,0.1108999684810640928844804875552654266357421875f,0.223499994069337848401346491300500929355621337890625f,0.0320999984718859376897626134450547397136688232421875f,-0.059499999146908510994080643285997211933135986328125f,-0.0041999996259808602960816870108828879892826080322265625f,-0.4805999887228011857587262056767940521240234375f,-0.74759996464252509440484573133289813995361328125f,-0.371299992752075258550803482648916542530059814453125f,0.525500009179115323831865680404007434844970703125f,-1.2030999547481542322202585637569427490234375f,-0.67689999235272413358188714482821524143218994140625f,-0.387499994114041346193744175252504646778106689453125f,-0.276699976119399337903814739547669887542724609375f,-0.21239999475181103871790355697157792747020721435546875f,1.014400012469291567640539142303168773651123046875f,-0.621400018191337721873424015939235687255859375f,-0.20520000453591347167048297706060111522674560546875f,0.977799994957447182741816504858434200286865234375f,-0.5352000239074232723623936180956661701202392578125f,-0.304599995994567773749395200866274535655975341796875f,0.301599995905160955800283772987313568592071533203125f,0.1873000007539988853721979467081837356090545654296875f,-0.43500002071261434366533649154007434844970703125f,-0.587600020521879429225009516812860965728759765625f,0.2660999716758727373644433100707828998565673828125f,-0.1342000006318091465118413907475769519805908203125f,-0.3591999946713446689727788907475769519805908203125f,0.0984999858736994582386614638380706310272216796875f,-0.02139998809099186161120087490417063236236572265625f,0.58340000513792045211403092253021895885467529296875f,0.0081000009417540042022665147669613361358642578125f,0.0248000090986491006361802647006697952747344970703125f,-0.2018000001430511058941874580341391265392303466796875f,0.28839996114224242251822261096094734966754913330078125f,-0.4833999917268752000154563575051724910736083984375f,0.7589999858140945576678859652020037174224853515625f,1.15469999461173955523918266408145427703857421875f,-0.24859999492168416423965027206577360630035400390625f,0.10679999030828479789079210604541003704071044921875f,-0.191500002354383358493805644684471189975738525390625f,0.7150000055134295706693592364899814128875732421875f,-0.7888000206172467443366258521564304828643798828125f,-1.7004999999403960231347809894941747188568115234375f,-0.32340000454187389777871430851519107818603515625f,-0.65140000001192088774359945091418921947479248046875f,0.21909996081292693048681030631996691226959228515625f,0.555800013226270817057184103759936988353729248046875f,-0.740799994450807464119179712724871933460235595703125f,-0.788900026535987564102470059879124164581298828125f,0.451800002750754325919757548035704530775547027587890625f,0.110600015172362342585898886682116426527500152587890625f,-0.252899987651407531785707760718651115894317626953125f,0.269399990564584523866642484790645539760589599609375f,-0.734000013679266150035118698724545538425445556640625f,-0.639300012063980460652601323090493679046630859375f,-0.1910000022649764872539890347979962825775146484375f,0.29660000767707828117636381648480892181396484375f,0.83290000206828107565115715260617434978485107421875f,0.112399993336200765270405099727213382720947265625f,-0.716599983537197005034613539464771747589111328125f,-0.125000003874301768291843472979962825775146484375f,0.4712000039696693232116331273573450744152069091796875f,-0.2855000106692313632805735323927365243434906005859375f,0.070099981033802105656604908290319144725799560546875f,-0.2152999977886675964100504643283784389495849609375f,0.0910000051707029466996345945517532527446746826171875f,0.217899989700317320284739253111183643341064453125f,-1.03710000255107814837174373678863048553466796875f,-0.566600011551380067231775683467276394367218017578125f,0.109800003361701925541638047434389591217041015625f,-0.4459999766349795891073881648480892181396484375f,-0.298899994349479669608626863919198513031005859375f,-0.06780003221035002525240997783839702606201171875f,0.38719999580383301918118377216160297393798828125f,0.391800010424852462875833225552923977375030517578125f,0.4162999803841114232483278101426549255847930908203125f,-0.08329998677670946705120513797737658023834228515625f,-0.17239999806731931553116510258405469357967376708984375f,1.21119998459816002878142171539366245269775390625f,-0.06700002598762466732296161353588104248046875f,0.2127999927222727905018473393283784389495849609375f,-0.38140002474784839847643524990417063236236572265625f,-0.305199988591670745563533273525536060333251953125f,0.02850000539422037348913363530300557613372802734375f,-1.820899995899200263238526531495153903961181640625f,0.3389000187873847380615188740193843841552734375f,-0.087400009727478078502826974727213382720947265625f,0.2152999866127967010243082768283784389495849609375f,-0.9228999671995641307375990436412394046783447265625f,-0.57109999387860288511120643306639976799488067626953125f,-0.6962999972224235278872583876363933086395263671875f,0.2179000382781026701195514760911464691162109375f,-0.231999995291233052085999588598497211933135986328125f,-0.1365999862566591449564157301210798323154449462890625f,0.72119999148994651250177412293851375579833984375f,0.243200001756846917455590073586790822446346282958984375f,-0.6328999905943870629698722041212022304534912109375f,-0.1850999953389163010797346942126750946044921875f,-0.332000011682510365318421463598497211933135986328125f,0.9766000130414962399072464904747903347015380859375f,0.003599985682964046418419457040727138519287109375f,0.0275999981701373631182150347740389406681060791015625f,-0.02859996511936202523429528810083866119384765625f,-0.3589999697208412499094265513122081756591796875f,-0.43969999505877499501593774766661226749420166015625f,-0.37439999026656156644321526982821524143218994140625f,-0.1906999809324734229676323593594133853912353515625f,0.52179999150037748023578387801535427570343017578125f,-0.47720000206232082717860976117663085460662841796875f,-1.1278999762892727432017636601813137531280517578125f,0.362200013685226362980529302149079740047454833984375f,0.382599983119964459632456055260263383388519287109375f,0.07429998166561091466064681299030780792236328125f,-0.60080002738237414661170987528748810291290283203125f,-0.40260002275705364382929474231787025928497314453125f,0.093599992835521561573841609060764312744140625f,0.2706000066041946983119714786880649626255035400390625f,-0.3024000019788741777659879517159424722194671630859375f,-0.5215000074207782887469875277020037174224853515625f,-0.608800011825561426803687936626374721527099609375f,-0.52740000376701345885521732270717620849609375f,0.583199995684623218039632774889469146728515625f,0.486200023639202338898712696391157805919647216796875f,-0.130800006222724807258828150224871933460235595703125f,-0.589700016218424050151725168689154088497161865234375f,-0.11720001059323570391512703281478025019168853759765625f,-0.51080002429038307010245034689432941377162933349609375f,-0.2182000199176374888310192545759491622447967529296875f,0.340200023162365194906442411593161523342132568359375f,-0.281800005656480667681762497522868216037750244140625f,0.0764999605566261120515036964206956326961517333984375f,-0.22700000870227821092584008511039428412914276123046875f,-0.44020001213550574714616914206999354064464569091796875f,-0.16790001018941413946095053688623011112213134765625f,0.5396000271379950419259330374188721179962158203125f,-0.1974000000417230893390296841971576213836669921875f,-1.117400018966197450254185241647064685821533203125f,-0.710400012126565005843303879373706877231597900390625f,-0.088200002017617185057218875954276882112026214599609375f,0.555599998520314830585675736074335873126983642578125f,0.213599998944997704342085853568278253078460693359375f,-0.621400009399652564212601646431721746921539306640625f,-0.3500000256299973244722423260100185871124267578125f,-0.147600003124773504481481722905300557613372802734375f,-0.2912000019207596768211487869848497211933135986328125f,0.5057999804250894460011522824061103165149688720703125f,0.204399995183944582777257892303168773651123046875f,-0.535600014352798581285242107696831226348876953125f,-0.085800010395050918532433570362627506256103515625f,0.6348000223606826697420046912156976759433746337890625f,0.4074000130057335855582323347334749996662139892578125f,-1.19099997425079351387466886080801486968994140625f,0.32760000040531167542212642729282379150390625f,-0.012700006473064462397815077565610408782958984375f,0.2219999762177469193602519226260483264923095703125f,0.81259996911287402099333121441304683685302734375f,0.3561999862372875469418431748636066913604736328125f,-0.109000011444091882140128291212022304534912109375f,-1.163000009179114613289129920303821563720703125f,-0.46330000994801512348431060672737658023834228515625f,-0.52740000093579286755129942321218550205230712890625f,0.13600001552701002083267667330801486968994140625f,-0.0532999992191791616136242737411521375179290771484375f,0.548800027918815658267703838646411895751953125f,-1.22199997413158456538440077565610408782958984375f,-0.621500000864267487798997535719536244869232177734375f,-0.53720001040697074046192938112653791904449462890625f,0.569800018858909851360294851474463939666748046875f,0.13650000026822084464583895169198513031005859375f,-0.282900009667873320040598628111183643341064453125f,-0.556900002001225136183393260580487549304962158203125f,-0.352100013056397376232098395121283829212188720703125f,0.292699993956089254965036161593161523342132568359375f,0.3045000204741956650877909851260483264923095703125f,-0.1051000033110380815770668050390668213367462158203125f,-0.275900003722310049880661608767695724964141845703125f,-0.663299987968802184923333697952330112457275390625f,-0.34290000303685663762820468036807142198085784912109375f,-1.2017000313282011347837396897375583648681640625f,0.55510003676414498841040767729282379150390625f,0.182400012260675481456928537227213382720947265625f,0.4133999976500868367423890958889387547969818115234375f,-1.058999987266957987230853177607059478759765625f,-0.346499994643032549479499948574812151491641998291015625f,-0.2899000010848045150169127737171947956085205078125f,-0.1614999879002567695351899601519107818603515625f,-0.312800006729364366009349396335892379283905029296875f,0.633700009855628021426809937111102044582366943359375f,-0.381899994774162809818562891450710594654083251953125f,-0.31490000481009483568328732872032560408115386962890625f,0.04480002422332773903690394945442676544189453125f,-0.090800003242492266508634202182292938232421875f,-0.29480001677274714211307582445442676544189453125f,-0.2513999995648859009378384143928997218608856201171875f,0.6671999949097635607131451251916587352752685546875f,-0.94860004200935410523243263014592230319976806640625f,-0.19220001845359835357385236420668661594390869140625f,1.1461000008821482509802081040106713771820068359375f,-0.315599989765882327219514991156756877899169921875f,0.475199978905916253779651015065610408782958984375f,0.2546000114917745804632431827485561370849609375f,0.06839999902844429158221828402020037174224853515625f,-0.13299999889731406454984607989899814128875732421875f,0.35179999746084167355775207397527992725372314453125f,-0.5820000203251840531493144226260483264923095703125f,-0.2804000174164773540042006061412394046783447265625f,-0.1149999964237213845308360760100185871124267578125f,-0.5534000252544881703897772240452468395233154296875f,-0.1385999962627888493216232745908200740814208984375f,0.740900012588500800347901531495153903961181640625f,0.678600008034706103643429742078296840190887451171875f,-0.289199992883205414528902110760100185871124267578125f,-0.540700005370378544711229551467113196849822998046875f,-0.2812000063538551142272581273573450744152069091796875f,0.7970000233054161942192195056122727692127227783203125f,0.66140003562569660289227613247931003570556640625f,0.36160000991225249578064904198981821537017822265625f,-0.45440000904202459963698856881819665431976318359375f,-0.73400001412630100361411678022705018520355224609375f,0.39820001952648187426575532299466431140899658203125f,0.30800001990795156103786212042905390262603759765625f,0.32229999799728403786502894945442676544189453125f,0.163600000435113912544693448580801486968994140625f,0.17020000066161156127009235206060111522674560546875f,-0.381799973097443956504548623342998325824737548828125f,-0.06879999871253961174488722463138401508331298828125f,0.55340001139640815353004654753021895885467529296875f,0.1652000267386437570849011535756289958953857421875f,-0.1656000042200087563060151296667754650115966796875f,-0.3825999872922896116733681992627680301666259765625f,-0.513599999094008552447121473960578441619873046875f,0.08200000274181373782766968361102044582366943359375f,-0.04799999248981468014108031638897955417633056640625f,-0.6837999891757959858296089805662631988525390625f,0.42020000990033157250991280307061970233917236328125f,0.24400002887845051446902289171703159809112548828125f,0.2784999924302109519658188219182193279266357421875f,0.78300001420825726672347855128464289009571075439453125f,0.424500011980533553668237800593487918376922607421875f,1.0948000140905378430034033954143524169921875f,-1.4326000349760050767145003192126750946044921875f,-0.57180001817345615933163571753539144992828369140625f,0.1334999915361407829550444148480892181396484375f,-0.1013000082492823139546089805662631988525390625f,0.19689999399185165174230860429815948009490966796875f,0.5123000236272812202287241234444081783294677734375f,0.0354999960660933311373810283839702606201171875f,-0.121800007146597000229348850552923977375030517578125f,-0.390200011539459268306018202565610408782958984375f,-1.065799997282027788969571702182292938232421875f,-0.34420000241994852530069692875258624553680419921875f,0.7373000272035596935893408954143524169921875f,-0.8485000100135806633261381648480892181396484375f,-0.36210000389218333083363177138380706310272216796875f,-0.36419998513460161149168925476260483264923095703125f,0.743200002837181017412149230949580669403076171875f,0.07689999652504919680495731881819665431976318359375f,0.64920000972151736817750133923254907131195068359375f,0.2147999784767620212733163498342037200927734375f,0.3685000106096267824540291258017532527446746826171875f,-0.7479000040054319953242156771011650562286376953125f,0.14250002026557950784990680404007434844970703125f,-0.10030000211000444920728114084340631961822509765625f,0.575200013923645059321643202565610408782958984375f,0.83340000305175720995976007543504238128662109375f,0.279900005555152819880504466709680855274200439453125f,-0.44819998704195018746077039395458996295928955078125f,0.3293999932467936986313361558131873607635498046875f,0.21889999911785118502649538640980608761310577392578125f,0.121200003671646072689327411353588104248046875f,0.309599988245963686495088040828704833984375f,0.3349000065982341567405455862171947956085205078125f,-0.7108000074148179692201665602624416351318359375f,-0.56159996297359526806758367456495761871337890625f,-0.235899982339143665655001314007677137851715087890625f,-0.859600010895729127469166996888816356658935546875f,0.02720001904964419736643321812152862548828125f,0.08940001246929174527622308232821524143218994140625f,0.53510000398159007772846962325274944305419921875f,0.8160999973058693512939498759806156158447265625f,0.24880000944137581342374687665142118930816650390625f,-1.270099996158480681884839214035309851169586181640625f,0.351499988663941553834746400752919726073741912841796875f,-0.64699999514222117369399711606092751026153564453125f,-0.28740000272393206870447102119214832782745361328125f,0.8396999683856964935557698481716215610504150390625f,-0.3080000142455101563854213964077644050121307373046875f,-0.1933999932169914170998481495189480483531951904296875f,-0.0111999877274033821095144958235323429107666015625f,-0.866200012165307953182491473853588104248046875f,0.06659999232888225151327787898480892181396484375f,1.330099996381997495831228661700151860713958740234375f,-0.27999999850988377403382401098497211933135986328125f,0.67060000392198571006474594469182193279266357421875f,0.2548000328660009472514502704143524169921875f,-0.5175999989151953695198926652665250003337860107421875f,0.5707999987721443790888997682486660778522491455078125f,0.69560000615715988914189438219182193279266357421875f,-0.25939999577999106605830093030817806720733642578125f,0.30260000100135808764889588928781449794769287109375f,0.70269999008178718469252999057061970233917236328125f,0.417000005871057677353519466123543679714202880859375f,-0.747999985784292054091793033876456320285797119140625f,-0.76950000393390638464552466757595539093017578125f,-0.70559999796152084172717877663671970367431640625f,0.0634000063300135252575273625552654266357421875f,0.1508000065207486528606750653125345706939697265625f,0.7976000249922277074432486188015900552272796630859375f,0.2394000060617924585670834858319722115993499755859375f,-0.552400002127885869640522287227213382720947265625f,0.15240000272393228186729174922220408916473388671875f,-0.01079999623894689619874043273739516735076904296875f,-0.518599995219707121663077487028203904628753662109375f,0.00060000213384636680302719469182193279266357421875f,-0.31979999591112129309067313442938029766082763671875f,0.1302999855697155595635194913484156131744384765625f,0.04860000110566620179497476783581078052520751953125f,-0.352399998775124512206957660964690148830413818359375f,-0.901700004617869144141195647534914314746856689453125f,-0.486400005525350564994369051419198513031005859375f,-0.33720000995993615333645720966160297393798828125f,0.48359997078180327889640466310083866119384765625f,0.2899999995529649599035337814711965620517730712890625f,0.3761999936878679395846347688348032534122467041015625f,0.0970000027418134180834385915659368038177490234375f,0.4636000214457514090327094891108572483062744140625f,-0.023499996472150119775079701867070980370044708251953125f,0.84169999437332165825864649377763271331787109375f,-1.084099964165687879358301870524883270263671875f,-0.5052999930202961564873476163484156131744384765625f,-0.13559999065995231859460545820184051990509033203125f,-0.59199999007582704280139296315610408782958984375f,-0.2200999938488006346659631162765435874462127685546875f,1.34590001556873328780739029753021895885467529296875f,-1.110300002706051003542597754858434200286865234375f,-0.397300005149841428675472343456931412220001220703125f,0.127400001382827809948139474727213382720947265625f,0.090999987959861527997418306767940521240234375f,0.34090000558495514582091345801018178462982177734375f,0.61079998461604123605184213374741375446319580078125f,-0.29160000872015956474569975398480892181396484375f,0.1402000048339366866656519050593487918376922607421875f,0.7719999920129776427302203956060111522674560546875f,-0.103800005269050643619266338646411895751953125f,0.030600001090764994415849287179298698902130126953125f,-0.7344999756217003294977985206060111522674560546875f,0.010500042855738911384833045303821563720703125f,-0.279999991357326560859064557007513940334320068359375f,-0.137000014662742586324384319595992565155029296875f,-0.97819996051788393742754124104976654052734375f,-0.18309997903704644528488643118180334568023681640625f,-0.38630000333189951078338708612136542797088623046875f,1.2956999973237515177970635704696178436279296875f,0.4748000000834464717769378694356419146060943603515625f,-0.61290001056194309825286836712621152400970458984375f,0.17150000786781305350814363919198513031005859375f,0.1115000065267086615250491377082653343677520751953125f,0.2100000083446502685546875f,-0.216999982774257915707494248636066913604736328125f,-0.2373999982535839325947790712234564125537872314453125f,-1.12730002288222319606347809894941747188568115234375f,-0.25869997021853929908274949411861598491668701171875f,-0.097199987459182624061071464893757365643978118896484375f,0.4518999862432480085772112943232059478759765625f,-0.91449998486042005652052466757595539093017578125f,-0.441500000417232474347883908194489777088165283203125f,0.21909999501109123087871921597979962825775146484375f,0.703899961417913999639495159499347209930419921875f,0.327599987888336219299389995285309851169586181640625f,-0.229400001144409060316320392303168773651123046875f,-0.3343000108718872098734209430404007434844970703125f,0.3076000138163568209392906283028423786163330078125f,-0.1270000000596045452994076185859739780426025390625f,-0.7330000014305113609225372783839702606201171875f,-0.0020000036358833739313922706060111522674560546875f,-0.407400010025501302379780099727213382720947265625f,-0.05580000964999243251440930180251598358154296875f,-0.6846000010609625263668931438587605953216552734375f,0.7332000102877618274987980839796364307403564453125f,1.61770000706911076804317417554557323455810546875f,-0.437400011664629051466590681229718029499053955078125f,-0.555200011092424450254156909068115055561065673828125f,0.1512999766588212224860399146564304828643798828125f,0.4028000123918058505978478933684527873992919921875f,-0.7576000059187411039829385117627680301666259765625f,-1.1070000075101855685488771996460855007171630859375f,-0.34840000022053718709003078402020037174224853515625f,0.58660000216364860392559421597979962825775146484375f,1.20899999445676797904525301419198513031005859375f,-0.6547999903976915447856299579143524169921875f,0.31759999131560334717505611479282379150390625f,1.32099995905160927378574342583306133747100830078125f,-0.654200000631809164275409784750081598758697509765625f,0.319199994969368017194710773765109479427337646484375f,1.061699980515241836798168151290155947208404541015625f,0.8744000209629536612965239328332245349884033203125f,0.1128000097095966935967226163484156131744384765625f,-1.0207999972820278600238452781923115253448486328125f,-0.287799995255470175692380507825873792171478271484375f,0.401400006866455161258500083931721746921539306640625f,0.82049998921155964382023739744909107685089111328125f,-0.55619998653531066423738593584857881069183349609375f,0.73739997202754015148684629821218550205230712890625f,1.5602999636650078940647290437482297420501708984375f,-0.6355999954283235719998401691555045545101165771484375f,-0.2476000226080417032648028907715342938899993896484375f,0.49059996830821006597034283913671970367431640625f,0.0800000019371509552001953125f,-0.2537999908149242145327662001363933086395263671875f,0.0927999721586711956433646264486014842987060546875f,-0.3454000113070012645266615436412394046783447265625f,0.3483999975383280656160422950051724910736083984375f,0.68450000572204583448865378159098327159881591796875f,0.5179999978542326743990997783839702606201171875f,-0.392699997979402581904651015065610408782958984375f,-0.81020000975131978293575230054557323455810546875f,0.29780002148151396568209747783839702606201171875f,0.107900006538629522623295997618697583675384521484375f,-0.0267999894142150196785223670303821563720703125f,-0.299599993908405082265744567848742008209228515625f,-0.2713000078022480554551520981476642191410064697265625f,1.73220000951290131752102752216160297393798828125f,-0.3412000167846684917094535194337368011474609375f,0.07010000308752051267902061226777732372283935546875f,0.0612999918580054981021021376363933086395263671875f,-1.0272999816060064404155127704143524169921875f,-0.528600001180171918946371079073287546634674072265625f,-1.185899990534782233453370281495153903961181640625f,-0.046299955797195480045047588646411895751953125f,-0.24209999718666086465646003489382565021514892578125f,1.366099997305870061836685636080801486968994140625f,0.01249997317790985107421875f,0.12619999607205389935415951185859739780426025390625f,1.4028999676465989665530287311412394046783447265625f,-0.62069999687671639776453957892954349517822265625f,-0.16289999834299084824351666611619293689727783203125f,1.06039997867345814341888399212621152400970458984375f,-0.641000003457069311707527958787977695465087890625f,-0.12429999790787693569882321753539144992828369140625f,-0.954500009894371004293134319595992565155029296875f,1.2775000035762786865234375f,0.47790000519752506846771211712621152400970458984375f,0.90279998840093611534030060283839702606201171875f,-0.0646000052332880159156047739088535308837890625f,0.073000004857778577616045367904007434844970703125f,1.71569996528625523524169693700969219207763671875f,-0.1545000325441350241817417554557323455810546875f,0.11849998483061785492509443429298698902130126953125f,0.38109997808337237046316658961586654186248779296875f,-1.0688999757230277509734150953590869903564453125f,-0.4245000014007092037360280301072634756565093994140625f,0.189699986863136160764042870141565799713134765625f,-0.056299987089634129233672865666449069976806640625f,-0.1637999940931795794796244081226177513599395751953125f,0.7024999988079070334379139239899814128875732421875f,-0.19890002653598770621101721189916133880615234375f,0.03959999480247500258656145888380706310272216796875f,-0.26679999120235464715733542107045650482177734375f,0.0153000019609930149044885183684527873992919921875f,0.41120000739693640667837826185859739780426025390625f,-0.01099999973177911982702426030300557613372802734375f,0.00559999408721889579965136363171041011810302734375f,0.1455999899148938681037179776467382907867431640625f,-0.6042000140428545051918263197876513004302978515625f,-0.9228000143289563084181281737983226776123046875f,0.6329999999403952415377716533839702606201171875f,-0.4031999953866005625968682579696178436279296875f,-1.420299990785121480030284146778285503387451171875f,0.27999998733401287864808182348497211933135986328125f,-0.83519998486638069579157672706060111522674560546875f,-0.224400005912780642347570392303168773651123046875f,-0.3795999901831148548581040813587605953216552734375f,0.3430000024735928576546939439140260219573974609375f,0.96219996838569699093568488024175167083740234375f,-0.6915999945640560753190584364347159862518310546875f,-0.0036000098228452515058961580507457256317138671875f,0.155800004434585304124993854202330112457275390625f,0.654600022369623335549704279401339590549468994140625f,0.376400004631280982181351646431721746921539306640625f,-0.5298999918460847169399130507372319698333740234375f,0.32760001441240316211178651428781449794769287109375f,0.17019999693632126280817828956060111522674560546875f,-0.49020001064538920587665415951050817966461181640625f,-0.1331999904692171554643209674395620822906494140625f,0.4921999980390072693126057856716215610504150390625f,-0.0977999869108199249012614018283784389495849609375f,-0.13500000253319743848123835050500929355621337890625f,0.37660000872015952921856296597979962825775146484375f,0.887699999022483865473986952565610408782958984375f,-0.12439999354481690030382878831005655229091644287109375f,0.43539999789595607015968425912433303892612457275390625f,0.3814000014275311656319900066591799259185791015625f,0.21260001247525206480304404976777732372283935546875f,0.46099999168515193304074273328296840190887451171875f,0.523899965739250461638221167959272861480712890625f,0.62940001708865178642327009583823382854461669921875f,-0.40439999533295623024287124280817806720733642578125f,-0.20730002660751356557966573745943605899810791015625f,0.299800015580654388713810476474463939666748046875f,-0.298399995005130591607667156495153903961181640625f,-0.922900003111362110530535574071109294891357421875f,0.05359999641180035467868947307579219341278076171875f,0.1096000013589859545248827998875640332698822021484375f,0.2003000119447708726738710538484156131744384765625f,0.59669998960495007622739649377763271331787109375f,0.11099999198317522797196943429298698902130126953125f,-1.08040002009868629784250515513122081756591796875f,-0.29839999172687470263554132543504238128662109375f,-0.238300003391504144900636674719862639904022216796875f,-1.53420000450611127007505274377763271331787109375f,0.518800011825561568912235088646411895751953125f,0.11230000455379496315799769945442676544189453125f,-0.067899961090088112314333557151257991790771484375f,-0.2062999814271933018972049467265605926513671875f,-0.4579000010251998986632315791212022304534912109375f,1.05409997072219852043417631648480892181396484375f,-0.22909999754428866935995756648480892181396484375f,-0.013999993711709901589301807689480483531951904296875f,-0.116000008821487199384137056767940521240234375f,-1.0051999650478364856098778545856475830078125f,-0.225599989616870999498132732696831226348876953125f,-0.4393000040173529185949519160203635692596435546875f,1.1856999941945076670890557579696178436279296875f,0.523800001096725420524080618633888661861419677734375f,-0.5150000286102294211332264239899814128875732421875f,0.5610000010728839470175444148480892181396484375f,0.282100003296136936370430703391321003437042236328125f,-0.249100019598007094145941664464771747589111328125f,0.14849999988079076729263761080801486968994140625f,0.312200007277727120680310690659098327159881591796875f,1.065700008201599047197305480949580669403076171875f,0.1478999783754346708519733510911464691162109375f,0.0906999973237514467427899944595992565155029296875f,0.394799989876151169454487899201922118663787841796875f,0.1619999888092280571072478778660297393798828125f,0.20690000002682207291826443906757049262523651123046875f,0.737099963808059754910573246888816356658935546875f,0.6876999692201621883214102126657962799072265625f,0.30169999154210103142759180627763271331787109375f,0.0669000124692917808033598703332245349884033203125f,0.61710001506805411963796359486877918243408203125f,-0.0642000039100647512668729177676141262054443359375f,-1.04460000970363608985280734486877918243408203125f,0.14720003871917697324533946812152862548828125f,0.0157000129699707002828290569595992565155029296875f,-0.963400016164779771088433335535228252410888671875f,0.1108999593913555514745894470252096652984619140625f,-0.0971999919295309933175985861453227698802947998046875f,0.5712000039696694120294750973698683083057403564453125f,0.3530000078380108874398501939140260219573974609375f,-0.02699999387562268537976706284098327159881591796875f,0.380200008484721241330817065318115055561065673828125f,0.764099992515146819727078764117322862148284912109375f,-0.18019999872446046396134988754056394100189208984375f,0.56420002149343506658851765678264200687408447265625f,0.6567000235795976692543263197876513004302978515625f,0.06399999766051767124963589594699442386627197265625f,-0.073200001421570792814463857212103903293609619140625f,-0.30639998887330310850529713206924498081207275390625f,-0.776000001817941598147854165290482342243194580078125f,-0.373600027853250526987949342583306133747100830078125f,0.5547999571681021535596300964243710041046142578125f,-0.80080003092139973031038380213431082665920257568359375f,-0.1315999912485480705104379239855916239321231842041015625f,0.5672000215269628053960104807629249989986419677734375f,0.184800002616643865849255234934389591217041015625f,0.28139999896883960417426351341418921947479248046875f,-0.27039998604953296990061062388122081756591796875f,0.29340000796914100789081203402020037174224853515625f,0.09679999433159824917538571753539144992828369140625f,0.4054999921917907812485282192938029766082763671875f,0.389399997568130373792882892303168773651123046875f,-0.0881999940454960551505791954696178436279296875f,-0.4364999924451111734668984354357235133647918701171875f,0.6182000283181670052812251014984212815761566162109375f,-0.4610000040531156972889448297792114317417144775390625f,-0.6753000370532273866075456680846400558948516845703125f,-0.8475999910175797680267351097427308559417724609375f,-0.0446000116407869739987290813587605953216552734375f,1.14819994859695473365945872501470148563385009765625f,0.48380002166032820554164572968147695064544677734375f,-1.05879999722242335025157444761134684085845947265625f,-0.769300052893161279143896535970270633697509765625f,-0.624800018560886627483341726474463939666748046875f,-0.501000017464161118141419137828052043914794921875f,0.652799954426289019693285808898508548736572265625f,0.6538000119000673127089839908876456320285797119140625f,-0.5028000026315451176373017005971632897853851318359375f,-0.9175000143423677201326427166350185871124267578125f,-0.26860000371336933966404103557579219341278076171875f,-0.5953000038981437480600789058371447026729583740234375f,0.35399998924136166777998369070701301097869873046875f,0.2967999889671801838630926795303821563720703125f,0.3118000041663646637601914335391484200954437255859375f,0.95230000172257422974553264793939888477325439453125f,0.29849998654425125010902775102294981479644775390625f,0.142299999636411655945522625188459642231464385986328125f,0.71740001628398886168724857270717620849609375f,0.1959999927282325415944796986877918243408203125f,0.23920000942349428640909536625258624553680419921875f,-0.366399993306398386039290926419198513031005859375f,0.01520000133216381499323688331060111522674560546875f,0.012599999362230274169149879526230506598949432373046875f,0.4152999669432642093624963308684527873992919921875f,-1.3598999778389924131261068396270275115966796875f,-0.560700003135204294579807537957094609737396240234375f,0.39170001801028819787831025678315199911594390869140625f,-1.221899998704344625366502441465854644775390625f,-0.40409999402388929734541989091667346656322479248046875f,-0.304200004208087904089552466757595539093017578125f,0.4497999862253667657796540879644453525543212890625f,0.28509999876618385972193436828092671930789947509765625f,0.66110001757144942757804528810083866119384765625f,0.53529998095035580263356678187847137451171875f,0.0929999859333037193209747783839702606201171875f,-0.384899991846084699176344656734727323055267333984375f,0.4712999920070171100405787001363933086395263671875f,0.1154999980777502432971459711552597582340240478515625f,-0.550000013560056455474978065467439591884613037109375f,0.5198000349521638696614900254644453525543212890625f,-0.038799997892975834901108100893907248973846435546875f,1.1272999622344972436849275254644453525543212890625f,-1.17829997165203081976869725622236728668212890625f,-0.44179999313950535366757321753539144992828369140625f,-0.666600008869171034575629164464771747589111328125f,-0.113999935746193159502581693232059478759765625f,-0.47329999102354047835206074523739516735076904296875f,0.63019997279644002219356480054557323455810546875f,-1.220599998855590229140943847596645355224609375f,-0.618100006753206088205843116156756877899169921875f,-0.8484999985769390296042047339142300188541412353515625f,0.521300017674267479605987318791449069976806640625f,-0.022200002397596840797433515035663731396198272705078125f,0.39269999067783345481075230054557323455810546875f,-0.3091000309228899567415282945148646831512451171875f,0.126200004081427952229432776221074163913726806640625f,0.5823999967262147858804155475809238851070404052734375f,0.4259000007234512441556262274389155209064483642578125f,0.3338000003516672364867190481163561344146728515625f,-1.0584000204861165883585272240452468395233154296875f,-0.6023000590920446484233252704143524169921875f,0.076400005227327483225963078439235687255859375f,0.2608000230610372227602056227624416351318359375f,0.7482000034332276783288762089796364307403564453125f,0.6128000022590160966728944913484156131744384765625f,-0.8177999894440173278553629643283784389495849609375f,-1.624799989354610119107746868394315242767333984375f,-0.4472000257551673740863407147116959095001220703125f,0.7369999996125695673754307790659368038177490234375f,1.420400028145312631977503770031034946441650390625f,-0.02940000248551388750684054684825241565704345703125f,0.51559998440146426190722195315174758434295654296875f,0.1129000043034551481468952260911464691162109375f,-0.43260001202821740662329830229282379150390625f,-0.266800009977817609296835144050419330596923828125f,0.6348999852895733653213028446771204471588134765625f,0.304800009620189715775495642446912825107574462890625f,0.067200003403425245807056853664107620716094970703125f,-0.7783999998480075621642981786862947046756744384765625f,-0.553600012505054639433410557103343307971954345703125f,0.682999992787837850727328259381465613842010498046875f,0.638800016593933150943485088646411895751953125f,0.028600000658631330452408292330801486968994140625f,-0.019000000044703480028829289949499070644378662109375f,-0.34729999684244372559760449803434312343597412109375f,-0.4930000219941141903490233744378201663494110107421875f,-0.4328000235676767726289426718722097575664520263671875f,0.047899991935491215144793386571109294891357421875f,-0.0169999955892563558990104866097681224346160888671875f,0.6667999912023543362948885260266251862049102783203125f,0.4307000049233433713880003779195249080657958984375f,-0.6377999974906443103606079603196121752262115478515625f,0.1367999865829945260742306345491670072078704833984375f,0.47029998735785483177096466533839702606201171875f,-0.3740000063776971472861987422220408916473388671875f,1.0153999947667120551386688021011650562286376953125f,0.4718000248789788741987649700604379177093505859375f,-0.529400018876791111921420451835729181766510009765625f,-0.26900000891089437260461636469699442386627197265625f,0.553099989840388328588005606434307992458343505859375f,0.639299964044988566769234239473007619380950927734375f,0.3737999920070171899766364731476642191410064697265625f,-0.51590002212524410651894868351519107818603515625f,-0.3196999791145316294205258600413799285888671875f,-0.54060001077651964607184709166176617145538330078125f,0.993500024914741430848152958787977695465087890625f,0.3634999969005576758718234486877918243408203125f,0.092900004601478514132395503111183643341064453125f,-1.76509998550415048157447017729282379150390625f,0.17450004267692520443233661353588104248046875f,-0.14779998944401739890963654033839702606201171875f,1.5817000265598295527524896897375583648681640625f,-0.0067000682830808955259271897375583648681640625f,0.16999998226761814379415227449499070644378662109375f,0.074899976944923452037983224727213382720947265625f,0.33589996192455373602570034563541412353515625f,0.32799998980760580735704934340901672840118408203125f,0.668600006842613225899185636080801486968994140625f,-0.77200000154972059362989966757595539093017578125f,-0.399500007808208490445167626603506505489349365234375f,-0.894300024878978749853786212042905390262603759765625f,0.4289000099956989942029395024292171001434326171875f,0.246500009283423526795786528964526951313018798828125f,0.5057999814867972787624239572323858737945556640625f,-0.2690000343918796943398774601519107818603515625f,0.214699997740984027672084266669116914272308349609375f,-0.443300009351968782311814720742404460906982421875f,-0.00149999393522737278772183344699442386627197265625f,0.045300004196167031889075360595597885549068450927734375f,-0.11560000615715981808762080618180334568023681640625f,-1.03020000542998246828574337996542453765869140625f,-0.4461000111639499277060849635745398700237274169921875f,0.48279998556971559509065627935342490673065185546875f,0.489799964469671778033443843014538288116455078125f,0.3892999919474124936868975055404007434844970703125f,0.30809997516274467699304295820184051990509033203125f,-0.798499987512826692181988619267940521240234375f,-0.1591999960124492741186941202613525092601776123046875f,0.329799972367286642338513047434389591217041015625f,0.2185999549865726976349833421409130096435546875f,0.5182999992191792415496820467524230480194091796875f,0.8923000209450719921733252704143524169921875f,0.332700014221668283198596327565610408782958984375f,0.30480000261664397243066559894941747188568115234375f,-0.60159999352097504043968001496978104114532470703125f,-1.095200002896785207440188969485461711883544921875f,0.77120001603960997016429246286861598491668701171875f,0.07680000774264339469254991854541003704071044921875f,-0.6955999940872192865981560316868126392364501953125f,0.61040002948641802760221253265626728534698486328125f,0.02340000081658377695248418604023754596710205078125f,-1.096400016254186038366924549336545169353485107421875f,0.76559999930262545575487820315174758434295654296875f,0.37320001580119122053247338044457137584686279296875f,-0.4831999694585800142476728069595992565155029296875f,-0.20740000346899023497826419770717620849609375f,-0.13739999572038641417748294770717620849609375f,-0.16809998782873147860073004267178475856781005859375f,-0.2808000053286552333275949422386474907398223876953125f,0.0014000012040138454239013299229554831981658935546875f,0.3568000005900859861185381305404007434844970703125f,0.16660000052452073049380487645976245403289794921875f,0.70839999232292161224222581950016319751739501953125f,-0.2350999710500243367050643428228795528411865234375f,-0.36499999016523343442486293497495353221893310546875f,0.51019998739957816979995186557061970233917236328125f,0.05950000959634760278049725457094609737396240234375f,-0.049999999813735485076904296875f,-0.0611999996110796917747620682348497211933135986328125f,0.2577999880470336346860449339146725833415985107421875f,0.0053999894767997602684772573411464691162109375f,-0.391800022420287330504606870817951858043670654296875f,-0.640499998189509422985565834096632897853851318359375f,-0.093399989342689337945557781495153903961181640625f,0.5104000141382218913577162311412394046783447265625f,0.59889999837279361827313550747931003570556640625f,0.4574000326752667433538590557873249053955078125f,-1.0489999908804890083047212101519107818603515625f,-0.6141000690698614761231510783545672893524169921875f,0.6452000270366671230704014305956661701202392578125f,-0.2351999900817869360025724745355546474456787109375f,-0.2854000355958934420641526230610907077789306640625f,0.112799998757243002955874544568359851837158203125f,0.305599990436434598706227916409261524677276611328125f,0.236400001762806954985762786236591637134552001953125f,0.2804000062406064586184584186412394046783447265625f,0.34569999151229868772361442097462713718414306640625f,-1.234300003719329907880819519050419330596923828125f,-0.07749996304512052347490680404007434844970703125f,-0.165499988913536011381211210391484200954437255859375f,-0.568699994134902908626827411353588104248046875f,1.1280999970674514543134137056767940521240234375f,0.37649999952316282048059292719699442386627197265625f,-1.16690003750324233777746485429815948009490966796875f,0.819100027048587975286864093504846096038818359375f,0.351000012099742964011284129810519516468048095703125f,-0.32549997997283952599900658242404460906982421875f,1.3504999638795851524264435283839702606201171875f,0.54709999510049822646351458388380706310272216796875f,-0.3025999926567077835670716012828052043914794921875f,-0.41759998133182563151422073133289813995361328125f,-0.12999999240040782666483210050500929355621337890625f,0.34040000415444371384410260361619293689727783203125f,-0.3807999980270864170961431227624416351318359375f,-0.1318999987602234025274583473219536244869232177734375f,-0.316200011420249893490108661353588104248046875f,0.731999960720539633030057302676141262054443359375f,0.5668999936938285966192552223219536244869232177734375f,-0.1514000253438947396489311358891427516937255859375f,-0.16559999885559051335803815163671970367431640625f,0.23460000448822970753326444537378847599029541015625f,0.324499998345971096824769119848497211933135986328125f,-0.095099999175965788111852816655300557613372802734375f,-0.0935000009238719786974769476728397421538829803466796875f,-0.79770000393986695286230315105058252811431884765625f,-0.32790000378191397345517543726600706577301025390625f,-0.174300011020898815150559357789461500942707061767578125f,0.68779999227523802574069122783839702606201171875f,0.279999997913837006535686668939888477325439453125f,0.2359000090122223003419321685214526951313018798828125f,-0.475000020265579081524265347979962825775146484375f,-0.1323999179363255507269059307873249053955078125f,-0.4965999827921390163965043029747903347015380859375f,-0.048800009441375635788062936626374721527099609375f,0.7548000254154203503276221454143524169921875f,0.06740000555515290869834643672220408916473388671875f,0.32899999818205838408857744070701301097869873046875f,0.40499996803700977210382916382513940334320068359375f,0.207799987804889674247732500589336268603801727294921875f,-0.346099971675872808418716886080801486968994140625f,-0.15909999843835809230085942544974386692047119140625f,-0.3474000115156175017006034977384842932224273681640625f,0.3213999988198279478268659659079276025295257568359375f,0.635500009179115199486886922386474907398223876953125f,0.53500001773238192726012130151502788066864013671875f,-0.54140002847313883904689646442420780658721923828125f,-1.2661000116109857316359921242110431194305419921875f,-0.2951999962285161149821988146868534386157989501953125f,0.5159999901577829728438473466667346656322479248046875f,0.6215999957375226703248927151435054838657379150390625f,-0.1638000077649952590519433215376920998096466064453125f,-0.430200020293891594036495007458142936229705810546875f,-0.0159000253848733397177284132339991629123687744140625f,-0.88940004256963778317413016338832676410675048828125f,-0.16300002020597492702336239744909107685089111328125f,0.8013000112295145527241402305662631988525390625f,0.444200027304888056534082352300174534320831298828125f,-0.472200007575750202448716663639061152935028076171875f,-0.47150004303455350651574917719699442386627197265625f,0.41800000664591774324208017787896096706390380859375f,0.52659999620318398427798456395976245403289794921875f,-0.5485999931335445722879740060307085514068603515625f,-0.433400004541873773433735550497658550739288330078125f,-0.228399991428851922847798050497658550739288330078125f,-0.0982999943017954791457668761722743511199951171875f,0.22900000786781316008955400320701301097869873046875f,-0.51980000813007354310002483543939888477325439453125f,-0.27230002988576895717187653644941747188568115234375f,-0.1268000047996640233805010211654007434844970703125f,-0.2204000028505921360277852727449499070644378662109375f,0.52579999510273278673366803559474647045135498046875f,0.05759999265670767698566123726777732372283935546875f,-0.44080000145435338509969369624741375446319580078125f,0.22719998954534492696666347910650074481964111328125f,0.26139999390244472721178681240417063236236572265625f,-0.65920000733733186137897064327262341976165771484375f,-0.0985000115036971379822716698981821537017822265625f,-0.578200016993284382493811790482141077518463134765625f,0.192200004446506422794982427149079740047454833984375f,1.0120999933868650710877545861876569688320159912109375f,-0.409799985331296579005311286891810595989227294921875f,0.974800013047457003523277307976968586444854736328125f,0.8339000223636627850964941899292171001434326171875f,-0.74820003487467801051025162450969219207763671875f,0.39160000365376479436463341698981821537017822265625f,0.6400000025331973318998279864899814128875732421875f,-0.225400023525953230318918940611183643341064453125f,0.0275999957859516253932952167815528810024261474609375f,-1.35990000734329186116156051866710186004638671875f,0.2891000213861474321674904786050319671630859375f,-0.03290001041293155736866538063623011112213134765625f,0.62309998155161749888719668888370506465435028076171875f,-0.01050001974031289364575059153139591217041015625f,0.1778999960891902432091882246822933666408061981201171875f,-0.046800013628601977444532167282886803150177001953125f,-0.550199999879299905813923032837919890880584716796875f,-0.33960001015067098339983431287691928446292877197265625f,0.663400011992454352593995281495153903961181640625f,-1.3544000351190561559633351862430572509765625f,-0.48500001713633533739766789949499070644378662109375f,-0.360500017821788532046412001363933086395263671875f,0.27370003675222420724821859039366245269775390625f,-0.126299998265504864747299507143907248973846435546875f,-0.5919000315427780378740862943232059478759765625f,0.94409997072219908886836492456495761871337890625f,0.560599998408556121631818314199335873126983642578125f,-0.3050000014901161904390392010100185871124267578125f,-0.8087999656319624364186893217265605926513671875f,-0.267899984335899343790288185118697583675384521484375f,-0.1920000051259993512076107435859739780426025390625f,-0.05939998087882969457496074028313159942626953125f,-0.2500000037252902984619140625f,0.66150002567470078673750322195701301097869873046875f,-0.298900001241266810581009849556721746921539306640625f,-0.26330000853240498326357510450179688632488250732421875f,0.3788000243425369006899927626363933086395263671875f,-0.21159999933242801262167631648480892181396484375f,-0.31760000904798513232663026428781449794769287109375f,0.06910003092288974357870756648480892181396484375f,-0.0966999973535536128110834397375583648681640625f,-0.36100001418590554891352439881302416324615478515625f,1.045199995744228171901113455533050000667572021484375f,-0.6246000170052052880009796353988349437713623046875f,-0.1819999999850987482119535343372263014316558837890625f,0.7962999948382378789801805396564304828643798828125f,0.168099996769427662002271972596645355224609375f,0.428000013351440333764230672386474907398223876953125f,0.22070000581741311407313332892954349517822265625f,-1.5143999922037121308449059142731130123138427734375f,-0.4397999886095522725781847839243710041046142578125f,0.7169999960362911650690875831060111522674560546875f,1.425699970054626675164399784989655017852783203125f,-0.4451999947011469060953459120355546474456787109375f,0.0827999787151814103935976163484156131744384765625f,-0.06699999409914081383021766669116914272308349609375f,0.14379999983012670128346144338138401508331298828125f,0.281399996659159601364308400661684572696685791015625f,-0.243299979512393793612545778159983456134796142578125f,0.283399998134374442315674968995153903961181640625f,-0.0569999939501286689846892841160297393798828125f,0.064899978435040139856937457807362079620361328125f,-0.0179999996423720887150921043939888477325439453125f,-0.00839999947547909187051118351519107818603515625f,-0.30819999017119403816877820645458996295928955078125f,-0.22880001048445708367040651864954270422458648681640625f,-0.56300000843405728634394336040713824331760406494140625f,-0.18280001097619535954663660959340631961822509765625f,0.73720002053976063649542993516661226749420166015625f,0.12399999445676801457238980219699442386627197265625f,-0.4002000109434131758234798326157033443450927734375f,-0.46580002216696758665648303576745092868804931640625f,-0.40260000800490391981156790279783308506011962890625f,0.4462999975204464675471172085963189601898193359375f,-0.133000000312924360201805029646493494510650634765625f,-0.1705999993771314393598004244267940521240234375f,0.4262999810174108450411267767776735126972198486328125f,0.550999995708465473143178314785473048686981201171875f,-0.292999992936849640301488761906512081623077392578125f,-0.9020999750584366427830218526651151478290557861328125f,-0.166800007183849796632557627162896096706390380859375f,-0.5154000041916966257105059412424452602863311767578125f,0.1169999948255713295708346777246333658695220947265625f,-0.1843999974191188329086799058131873607635498046875f,-0.0388000021398067218569849501363933086395263671875f,0.52259997835159310852759517729282379150390625f,0.569800005596876157909491666941903531551361083984375f,-0.330199996638298021167656770558096468448638916015625f,-0.8610999918669464125997592418571002781391143798828125f,0.192400002388656066187877513584680855274200439453125f,0.4435999968215822431005790349445305764675140380859375f,-0.3593999796308577909798032123944722115993499755859375f,-0.56959999063014965514639698085375130176544189453125f,-0.10249999076128002428731633699499070644378662109375f,1.5944999693632126991360564716160297393798828125f,-0.00970002083778354062815196812152862548828125f,0.157699998575449029658557265065610408782958984375f,-0.3057000123739239683118285029195249080657958984375f,-0.60049998652934988285778672434389591217041015625f,-0.06290000042915355749073569313623011112213134765625f,-0.28749999687075611376485539949499070644378662109375f,0.39189998274147530565869601559825241565704345703125f,0.269299996119737594657550516785704530775547027587890625f,0.22449998724460584753614966757595539093017578125f,0.38109998657703414437492028810083866119384765625f,0.033899997776746904065703347441740334033966064453125f,-0.4146000013589858923523934208787977695465087890625f,-0.058999992072582330138175166212022304534912109375f,0.0334000042438507005471137745189480483531951904296875f,-0.281999998793005968167335595353506505489349365234375f,-0.75519998210966576834834995679557323455810546875f,-0.42530000129044054357763116058777086436748504638671875f,-0.1998000033617017834330908954143524169921875f,1.1373999876737599379339371807873249053955078125f,0.35639999077320094755805257591418921947479248046875f,0.41990001688003530944115482270717620849609375f,-0.9343000031232833180183661170303821563720703125f,-0.47490000808835031165244799922220408916473388671875f,0.521300000649690620235787719138897955417633056640625f,-0.28089999373853213882057389128021895885467529296875f,-0.216700000855326646576060056759160943329334259033203125f,-0.900699972140789117958092901972122490406036376953125f,0.5218999998033044818157577537931501865386962890625f,0.0892999975353478969708476142841391265392303466796875f,0.137400023064017329232910924474708735942840576171875f,-0.604199983830750131374998090905137360095977783203125f,-0.41790000430345541904131323462934233248233795166015625f,0.60909999277591708732870756648480892181396484375f,-0.253699996519088699642452411353588104248046875f,-0.15530000091791151817233185283839702606201171875f,-0.833399984276294691909470202517695724964141845703125f,0.5264000102937220759713454754091799259185791015625f,0.0633999989539384412040590177639387547969818115234375f,-0.437300009992718674567413472686894237995147705078125f,0.569499978564679931736236540018580853939056396484375f,0.4110999983489513898149425585870631039142608642578125f,1.53919998483657849419614649377763271331787109375f,-0.05040002367496487778453229111619293689727783203125f,-0.1408000051796438611972916987724602222442626953125f,0.3857999917685983604087596177123486995697021484375f,0.949599977219104829373463871888816356658935546875f,-0.478400013035535920380425523035228252410888671875f,0.208199995237588808549844543449580669403076171875f,0.30160001125335700322693810448981821537017822265625f,-0.225200003120303193782092421315610408782958984375f,-0.417800007101893466909814378595910966396331787109375f,0.492999961309135148468385523301549255847930908203125f,-0.29499999538063992332581619848497211933135986328125f,0.40920000733733186137897064327262341976165771484375f,0.888899990177154819548377417959272861480712890625f,0.0671999974429606794501523836515843868255615234375f,0.6738000075042247516421412001363933086395263671875f,0.1704000370860097035574654000811278820037841796875f,0.49460001164078715163441302138380706310272216796875f,0.02020000215172768065485797706060111522674560546875f,-0.1927000079631806528368542785756289958953857421875f,0.36140000194907173636238439939916133880615234375f,0.64199999603629098743340364308096468448638916015625f,-0.40249996878206761863339124829508364200592041015625f,0.3698000015735625201074299184256233274936676025390625f,-0.8168000216007234026704963980591855943202972412109375f,-0.9631000255286696898338050232268869876861572265625f,-0.1987999949872492067015627981163561344146728515625f,0.7862000193178655393921872018836438655853271484375f,0.631000027596950729957825387828052043914794921875f,-0.14200000140070923038138062111102044582366943359375f,0.32939999235272399147333999280817806720733642578125f,0.6193999963760372651222496642731130123138427734375f,0.676600012594461563963932348997332155704498291015625f,-0.747200000423192900456115239649079740047454833984375f,-1.3357000076055527415519463829696178436279296875f,0.570199999767541765294254219043068587779998779296875f,-0.103799990814924303350608170148916542530059814453125f,-0.59489999065399157274214303470216691493988037109375f,0.08120000598132615099444819861673749983310699462890625f,0.45840000714957718219721982677583582699298858642578125f,0.191800010238587947952737522427923977375030517578125f,-0.37699999648332582768972542908159084618091583251953125f,0.60360000714063655191665702659520320594310760498046875f,0.77510000852644456159623587154783308506011962890625f,0.47700002270936980863780263462103903293609619140625f,-0.3582000029861927004048993694595992565155029296875f,1.143699982213973953548702411353588104248046875f,0.1512999593734747350026736967265605926513671875f,0.08639998734593401508163879043422639369964599609375f,0.1510999919414519609972558100707828998565673828125f,-0.52770001094341267844356480054557323455810546875f,-0.039799999338388403202770859934389591217041015625f,0.59549998995661734824125232989899814128875732421875f,-0.58769999015629270644467396778054535388946533203125f,-0.422200003999471606963567182901897467672824859619140625f,0.988899991965293878593001863919198513031005859375f,-0.06370000605583214792204671539366245269775390625f,0.066400004184246075311648382921703159809112548828125f,0.177300013494491537358044297434389591217041015625f,0.596099968099593979786732234060764312744140625f,0.45599999824166292938798505929298698902130126953125f,-0.02649999713897699393783113919198513031005859375f,0.7244999943971635048001189716160297393798828125f,0.18509999578595159874794262577779591083526611328125f,-0.4388000043749809009341333876363933086395263671875f,0.9561999612033371676034221309237182140350341796875f,0.60209999331235886099733534138067625463008880615234375f,-1.00420000823140131984700929024256765842437744140625f,-0.1247999823510639316737069748342037200927734375f,-0.3129000101149082002649493006174452602863311767578125f,0.648700007247924759212764911353588104248046875f,0.325499988317488941902411170303821563720703125f,0.3894000105321406834946174058131873607635498046875f,0.71349999749660497627701261080801486968994140625f,0.08069997482299839930419693700969219207763671875f,0.094999990910291653989361293497495353221893310546875f,-1.3616999922871588779571538907475769519805908203125f,0.343700037348270370785030536353588104248046875f,-0.122599997127056159484936870285309851169586181640625f,-0.508099988424778103990320232696831226348876953125f,0.71209999152421943335866671986877918243408203125f,0.180099998170137443054272807785309851169586181640625f,0.237399997583031652226281948969699442386627197265625f,0.46419998677372920070638429024256765842437744140625f,0.309600002476572988552305787379737012088298797607421875f,0.777499994710087793947650425252504646778106689453125f,-0.0547000089913609599534538574516773223876953125f,0.19200000609457490430287407434661872684955596923828125f,-0.211000022530555497723980806767940521240234375f,0.1173999638319020988319607567973434925079345703125f,0.66320001833438890770366924698464572429656982421875f,0.24820002667903917625835674698464572429656982421875f,-0.163999996840953343735236558131873607635498046875f,0.244999999850988192662271103472448885440826416015625f,0.416799986881017492379442046512849628925323486328125f,-0.167499988824129530939899268560111522674560546875f,0.31499998930841666133773060209932737052440643310546875f,-0.28239999147355565600747695498284883797168731689453125f,-0.163299991600960592297298035191488452255725860595703125f,0.54779999465942363912063228781335055828094482421875f,0.06040001235008229940603996510617434978485107421875f,-0.9876999730944628907991500454954802989959716796875f,0.394599999719857930102762111346237361431121826171875f,0.545799991470575118768238098709844052791595458984375f,-0.237099982136488307560284738428890705108642578125f,0.01440000025033949526687138131819665431976318359375f,0.12439999965429304751296513131819665431976318359375f,0.18960000284910183410147510585375130176544189453125f,0.5815999875605104563192071509547531604766845703125f,-0.5007999935567379878875726717524230480194091796875f,-1.2375999610662464789356818073429167270660400390625f,-0.299000023365020961563232049229554831981658935546875f,0.285800003689527404304726587724871933460235595703125f,-0.0438999482303861210397144532180391252040863037109375f,-0.45979998369216890097277428139932453632354736328125f,0.23379998828172698921434857766143977642059326171875f,1.008099969798326167591540070134215056896209716796875f,-0.2502000127315522348681042785756289958953857421875f,0.7268000239849090604593584430404007434844970703125f,0.333300035130978500319542945362627506256103515625f,-0.256199997413158353509743392351083457469940185546875f,0.205799997878074680812687802244909107685089111328125f,0.454699992376565997886928016669116914272308349609375f,0.26979999710321411754421205841936171054840087890625f,-0.57180002726316470074152675806544721126556396484375f,-0.98870000247955402983279782347381114959716796875f,0.13960000925660143167306159739382565021514892578125f,0.66280001507401475890901565435342490673065185546875f,0.002200019794702967601551790721714496612548828125f,0.2380000083595514848155971776577644050121307373046875f,-0.2000000004470348091700770964962430298328399658203125f,-0.05540001901835200914803181149181909859180450439453125f,0.7604000323176387521328933871700428426265716552734375f,-0.45249999225139614367208196199499070644378662109375f,0.063200010585784838212930480949580669403076171875f,1.138000003218650135750067420303821563720703125f,0.38590000916123390339862453402020037174224853515625f,-0.1561000095248221697374901850707828998565673828125f,0.644099962377548962422224576584994792938232421875f,0.3803999876141548330821251511224545538425445556640625f,-0.03579997232258336214982819001306779682636260986328125f,0.246399990139901436947411639266647398471832275390625f,-0.078300005943328143676041008802712894976139068603515625f,-1.05679999775886557245030417107045650482177734375f,0.7629999877214430625826935283839702606201171875f,0.34250000387430201698180098901502788066864013671875f,-0.1935000073313712931621921597979962825775146484375f,0.950299953532219632279520737938582897186279296875f,0.5220999875009060264829940933850593864917755126953125f,0.2426000051736831863991028512828052043914794921875f,0.12659999098777774406698881648480892181396484375f,0.061999996781349153707196819595992565155029296875f,-1.305299958300590645876582129858434200286865234375f,0.3891000017166135194202070124447345733642578125f,0.0042999959707259449714911170303821563720703125f,-0.230899974739551705482654142542742192745208740234375f,-0.2383000239551069654453385737724602222442626953125f,0.1150999985426663929644064410240389406681060791015625f,1.09019998352527647256238196860067546367645263671875f,-0.421199989664554408363983384333550930023193359375f,-0.118600000882148837177965106093324720859527587890625f,0.238399996495246568883885629475116729736328125f,0.18179997630119260065839625895023345947265625f,0.37670001314878476250669336877763271331787109375f,0.45719998954534535329230493516661226749420166015625f,-0.189800001573562582279919297434389591217041015625f,0.001000001370906833386698053800500929355621337890625f,-1.113400000071525397515870281495153903961181640625f,-0.0983999964952460004496970213949680328369140625f,-0.1706000142037868982924919691868126392364501953125f,-0.009900009131431630748920724727213382720947265625f,0.680099988782405517895313096232712268829345703125f,0.4937000099301338540414008093648590147495269775390625f,0.0611999981582165286653207658673636615276336669921875f,0.20160000790059573461121544823981821537017822265625f,-0.05980000175982713717104388706502504646778106689453125f,-0.44719999744296057286874201963655650615692138671875f,-0.028799990256130225674269240698777139186859130859375f,-0.2283999890446661851228782325051724910736083984375f,0.5986000083327294163382248370908200740814208984375f,1.17659997251033843213008367456495761871337890625f,-0.1476000052481889479594201475265435874462127685546875f,-0.6548000016480683971309417756856419146060943603515625f,-0.186600018294155400866429772577248513698577880859375f,0.36780001507401482996328923036344349384307861328125f,0.28260001068711293470414602779783308506011962890625f,0.4751000018954272263727034442126750946044921875f,-0.47380000914335251849252017564140260219573974609375f,-0.08160000320672988749493015347979962825775146484375f,0.21509999861717243874181804130785167217254638671875f,0.39020001630783074375585783855058252811431884765625f,0.29139999136924732425768524990417063236236572265625f,0.494699956464767609531918424181640148162841796875f,0.70820000358223911263166883145458996295928955078125f,0.42140002370476725701564646442420780658721923828125f,-0.643499956369399939148934208787977695465087890625f,0.1876000053226949404461265658028423786163330078125f,-0.5109999878108499871132153202779591083526611328125f,-1.050200001406668803838329040445387363433837890625f,-0.1720000016614793292291096804547123610973358154296875f,-0.52339999273270354507303636637516319751739501953125f,-0.1760000073127445663345724824466742575168609619140625f,-0.428600006097555219497508005588315427303314208984375f,-0.371600019448995677606717435992322862148284912109375f,0.3108999699711798569978782325051724910736083984375f,-0.57680001683235193610244095907546579837799072265625f,0.67639999494552593972684917389415204524993896484375f,1.1476000063657760819069153512828052043914794921875f,-0.208599997901916633935570644098334014415740966796875f,0.414799981307983234302128039416857063770294189453125f,0.765299974393844451014956575818359851837158203125f,0.1254000089228153314024893916212022304534912109375f,0.1400000043213367462158203125f,0.33189997059702935899849762790836393833160400390625f,0.4668000074446201352884600055404007434844970703125f,0.5282000047743320436666181194595992565155029296875f,-0.574599962019920695865948800928890705108642578125f,0.57940002006888402519280134583823382854461669921875f,-0.10600000420212740692704755929298698902130126953125f,-0.819500011980533571431806194595992565155029296875f,-0.166400000533461434049087301900726743042469024658203125f,0.309100008720159580288822098737000487744808197021484375f,1.37779997498989104087740997783839702606201171875f,0.19459998586177817969655734486877918243408203125f,0.173900004035234534427445396431721746921539306640625f,-0.2383999979853630435400191345252096652984619140625f,-0.70199997018277660032481435337103903293609619140625f,-0.463999996691942195869984288947307504713535308837890625f,0.765900025701522935150933335535228252410888671875f,0.78369999055862393788629560731351375579833984375f,0.231699995714425899251409646240063011646270751953125f,0.0729999918937682679143108543939888477325439453125f,-0.73599999094009405098404386080801486968994140625f,-0.22429999567270275662167478003539144992828369140625f,0.801199969696998692825218313373625278472900390625f,0.824599972748757181761902756989002227783203125f,0.24139998853802691058945129043422639369964599609375f,-0.569699974346161042149105924181640148162841796875f,1.2868999802827829626039601862430572509765625f,0.58650000414252279057336636469699442386627197265625f,-1.24010000934600839173072017729282379150390625f,-0.18089995357990318325391854159533977508544921875f,-0.143999981492757722634223682689480483531951904296875f,-0.251199991825223012398282662616111338138580322265625f,-0.642799962435663285731379801291041076183319091796875f,-0.38269998993277554166780873856623657047748565673828125f,0.2546999823927880157725667231716215610504150390625f,-0.86909999128580039950975333340466022491455078125f,-0.42340000603199001716347993351519107818603515625f,1.1679999889135359580905060283839702606201171875f,-0.2640000466108318732949555851519107818603515625f,0.13019999216794975183120186557061970233917236328125f,0.8651999701142312204638074035756289958953857421875f,0.0565999674439439104389748536050319671630859375f,0.113099987530708379068755675689317286014556884765625f,0.4804999758005141785588421043939888477325439453125f,0.313099995577335477037195232696831226348876953125f,0.0844000020384789717553530863369815051555633544921875f,-0.603699990558624222103389911353588104248046875f,1.01469997851848603431790252216160297393798828125f,0.579599999123811659984539801371283829212188720703125f,-0.800300015521049346034487825818359851837158203125f,0.7047000130891802882615593262016773223876953125f,0.213600001180171972237076261080801486968994140625f,-0.1781999937102198838800859448383562266826629638671875f,1.5717999703407290468248902470804750919342041015625f,-0.428600004756450658760513761080801486968994140625f,0.521399998223781580009017488919198513031005859375f,0.32210001775026331216622565989382565021514892578125f,-0.062600007408857294421977712772786617279052734375f,-0.1567999958217143330330145545303821563720703125f,-0.3948999656200413710394059307873249053955078125f,0.069200008082389796726374697755090892314910888671875f,0.749399994587898188314056824310682713985443115234375f,0.1774000077903270522483580862171947956085205078125f,0.223599980244040319465881339056068100035190582275390625f,-0.814999983087182133800752126262523233890533447265625f,-0.715799986888467909551536649814806878566741943359375f,0.3497999968796967618089865936781279742717742919921875f,-0.2785999980509281659379894335870631039142608642578125f,-0.13829999530762482340406904768315143883228302001953125f,0.84060000034570681037848771666176617145538330078125f,0.01040002456903454941539166611619293689727783203125f,-0.86849997693300196033305837772786617279052734375f,0.05160000004023312936141110185417346656322479248046875f,-0.07659999970495701138162303323042578995227813720703125f,0.030999997105449372813978925478295423090457916259765625f,-0.5858000276803974060157997882924973964691162109375f,0.4156000006437299276740304776467382907867431640625f,1.327100004637241426053151371888816356658935546875f,-0.5856000003457069169598980806767940521240234375f,0.853800012719631240543094463646411895751953125f,1.5707999856591232656910506193526089191436767578125f,0.3897999972522256939555518329143524169921875f,0.043599999541044098805286921560764312744140625f,0.0233000023484226659320484031923115253448486328125f,-0.25900000175833692850346778868697583675384521484375f,-0.20980000023245803930649344692938029766082763671875f,0.488899992263316818252860684879124164581298828125f,0.84260002067089079247352856327779591083526611328125f,0.23439998981952658851923843030817806720733642578125f,-0.8779000025153163022650915081612765789031982421875f,-0.151800004762411067105176698532886803150177001953125f,-0.502200003999471622506689527654089033603668212890625f,-0.5262999991595744830874537001363933086395263671875f,-0.230799996536970031257851587724871933460235595703125f,0.189599990034103438318879852886311709880828857421875f,-0.1225999867707494583868310655816458165645599365234375f,0.25220000802278530471767226117663085460662841796875f,-0.4092000020846722208034407231025397777557373046875f,-0.75520003471076524448335476336069405078887939453125f,-0.4182000013470645871649367109057493507862091064453125f,-0.62300001376122227103593331776210106909275054931640625f,-0.929800018560886343266247422434389591217041015625f,-0.41279998720884325535962489084340631961822509765625f,-0.7645999972611665640442879521287977695465087890625f,0.4659999945536255072653375464142300188541412353515625f,-0.3371999902158975270793916934053413569927215576171875f,0.49120001428872350768273236099048517644405364990234375f,-0.2528999811880290327081866053049452602863311767578125f,0.7999999696016313333046809930237941443920135498046875f,-0.20230001265630104256842969334684312343597412109375f,-0.0348999837994572459365372196771204471588134765625f,-0.54799998310208319907133045489899814128875732421875f,-0.0130999633535739956613497270154766738414764404296875f,-1.17900002396106717839074917719699442386627197265625f,-0.204599994355440184534700165386311709880828857421875f,-1.14560002310723074714360336656682193279266357421875f,-0.3134000068515536696622802992351353168487548828125f,-0.1799999900162221333488332675187848508358001708984375f,-0.1561999936878681882745922848698683083057403564453125f,0.453700003075599767043968313373625278472900390625f,0.218600004160403926789513207040727138519287109375f,0.56410000700652584981753534520976245403289794921875f,0.16489998707771302832725268672220408916473388671875f,-0.244999984502792589324826622032560408115386962890625f,0.21970000385045984625520532063092105090618133544921875f,-0.04920000025928017439724726500571705400943756103515625f,-0.006799999621510510738886523540713824331760406494140625f,-0.029599999421834943813536256129737012088298797607421875f,-0.88489998797178248679529133369214832782745361328125f,0.235799991619586979396672177244909107685089111328125f,-0.76809999796152095274948123915237374603748321533203125f,-0.2234999676942823754188793827779591083526611328125f,0.93339996550083181858781244955025613307952880859375f,-0.1111000026702877452322582030319608747959136962890625f,-0.39159999255239963389385593472979962825775146484375f,0.1623999905049802183754081852384842932224273681640625f,-0.378800001841783362532822820867295376956462860107421875f,0.29090000573396679328652680851519107818603515625f,0.4431999928534029464799459674395620822906494140625f,0.397900000093877270757047881488688290119171142578125f,0.3379000183105471677436071331612765789031982421875f,0.246100005203485405758101478568278253078460693359375f,-0.1185999950706959626955949715920723974704742431640625f,-0.27289998530447501234874607689562253654003143310546875f,0.110300013658404305605387207833700813353061676025390625f,0.1390000181496144904258471797220408916473388671875f,-0.41649999490380285038781948969699442386627197265625f,-0.285299996671080524635044639580883085727691650390625f,0.088800001096725367233375436626374721527099609375f,-0.14429999552369121573747179354541003704071044921875f,-0.181600001493096296911744502722285687923431396484375f,-0.17579999698400516905394397326745092868804931640625f,-0.00049998146295582301945614744909107685089111328125f,0.257900025463104309864093011128716170787811279296875f,-0.08199998858571067028577772362041287124156951904296875f,-0.185599977584183506440496103095938451588153839111328125f,-0.38199998955428593472305465184035710990428924560546875f,-0.6829999912977218201604046043939888477325439453125f,-0.998699975061416722610374563373625278472900390625f,-0.5376999954462053921133701805956661701202392578125f,0.110000008344650179736845529987476766109466552734375f,0.0037000155925748234864158803247846662998199462890625f,0.36460001663267604943285959961940534412860870361328125f,0.9058000039875508946352056227624416351318359375f,1.48129998336434365313607486314140260219573974609375f,0.933699990335106644323559521581046283245086669921875f,0.15800001245737060884266611537896096706390380859375f,-0.329699982914328781902213449939154088497161865234375f,-0.3263999908477066558276646901504136621952056884765625f,0.487800001066923005765829657320864498615264892578125f,0.7917000013768671351499506272375583648681640625f,0.151099995592236524544205167330801486968994140625f,-0.004600011491775524152814114131615497171878814697265625f,0.403299985472858057988787550129927694797515869140625f,0.0554999937564135681355992346652783453464508056640625f,0.472600005918741405963601209805347025394439697265625f,0.545299986463785302248652442358434200286865234375f,0.1598999890893697539695494924671947956085205078125f,-0.44900000488758085026574917719699442386627197265625f,-0.646499999672174308074090731679461896419525146484375f,-0.5807000084996223332467479849583469331264495849609375f,0.25339998934268948005410493351519107818603515625f,0.1215999824941160767366454820148646831512451171875f,-0.25030002125799644119297227007336914539337158203125f,0.1841999940752983011549304137588478624820709228515625f,-0.5654000338450078100294149408000521361827850341796875f,-0.373300014641881006127732689492404460906982421875f,-0.2379999978542326477537471873802132904529571533203125f,-0.249500000990927173205591316218487918376922607421875f,-0.41370002334117916831246475339867174625396728515625f,0.699599985414743219536148899351246654987335205078125f,-0.30190004193633834805865490125142969191074371337890625f,-0.08289998791217811913156765513122081756591796875f,0.4093999799847605203240163973532617092132568359375f,-0.062100004339217917959103942848742008209228515625f,-0.10050002765655552394719052244909107685089111328125f,-0.594799999040364735236607884871773421764373779296875f,-0.05510000223070365965583050638088025152683258056640625f,0.4868000007763502168245395296253263950347900390625f,0.41839997511208093516898998132091946899890899658203125f,0.42279998225420722945955276372842490673065185546875f,0.60330000532865568629858898930251598358154296875f,-0.04879998664259943552679033018648624420166015625f,0.6975000139698384060693570063449442386627197265625f,-0.78170002283453943192625956726260483264923095703125f,-0.2989999962449072512526981881819665431976318359375f,-0.64130000877082349308722086789202876389026641845703125f,-0.9215000081658359931680024601519107818603515625f,-0.2635999771893029475222647306509315967559814453125f,-1.04050001309067052801538011408410966396331787109375f,-0.2515000030249361362422177990083582699298858642578125f,0.75459996172189758301129813844454474747180938720703125f,-0.1851000297605989164395623447489924728870391845703125f,0.062900018310546812472239253111183643341064453125f,0.30799998071789769227279975893907248973846435546875f,-0.1125000077113507135351255783461965620517730712890625f,0.5229000092580913250372987022274173796176910400390625f,-0.1991999975770710751277192684938199818134307861328125f,0.371500012040138383184739723219536244869232177734375f,-0.2931000036239623796063824556767940521240234375f,0.34800000098347627641715007484890520572662353515625f,-0.3661000218555334750902829910046420991420745849609375f,0.778700021404028053240153894876129925251007080078125f,0.2543999956309794807651769588119350373744964599609375f,0.7629000153675675743869533107499592006206512451171875f,0.1298000382304191901994272484444081783294677734375f,0.66619997640252126558380041387863457202911376953125f,0.0532000073447826959949225056334398686885833740234375f,0.29120000829100600725496406084857881069183349609375f,0.401500019714236433543419479974545538425445556640625f,0.57719999043941516703171146218664944171905517578125f,1.00809996681809455054690261022187769412994384765625f,0.614199982824921608681734142010100185871124267578125f,-0.224999988079070956104033029987476766109466552734375f,0.1461000154852865051680055330507457256317138671875f,-0.09709999197125451786405392340384423732757568359375f,0.19940001544952412615430148434825241565704345703125f,-0.381000001817941580384285771287977695465087890625f,0.0511000070661307148611740558408200740814208984375f,0.2856000012397765797089732586755417287349700927734375f,0.4788999997138976283395095379091799259185791015625f,0.23279999875724310953728490858338773250579833984375f,0.004599999272823396267995121888816356658935546875f,-0.715800003093480796678704791702330112457275390625f,-0.475900021007657159088921616785228252410888671875f,0.22240000406503668273217044770717620849609375f,0.6395000122785565732641543945646844804286956787109375f,0.26000000081956364805790826721931807696819305419921875f,0.28260001564174908050830481442972086369991302490234375f,0.07200000223889946315836141366162337362766265869140625f,0.051700000538676904593415883937268517911434173583984375f,-0.0207999914705754296306849937536753714084625244140625f,0.11530002125799632128888561055646277964115142822265625f,-0.377400005108118274943507231000694446265697479248046875f,0.25960000836253172007417333588819019496440887451171875f,-0.01589999880492687367450344027020037174224853515625f,0.091600000673532466777260196977294981479644775390625f,0.4840000062286851090220807236619293689727783203125f,1.0178000189483160653480808832682669162750244140625f,0.71660001982152454758079329621978104114532470703125f,-0.094199995863437691046016198015422560274600982666015625f,-0.7373999924793839255698912893421947956085205078125f,-0.203699995774030639950069598853588104248046875f,0.311600008720159582509268147987313568592071533203125f,0.32640000075697894743598226341418921947479248046875f,-0.043200003954768195768565419712103903293609619140625f,-0.1237999962911009443899956750101409852504730224609375f,-0.42159999603554598746057990865665487945079803466796875f,-0.5415000100471079047537159567582421004772186279296875f,0.16120000247955346139860921539366245269775390625f,0.1473999876737597247711164527572691440582275390625f,-0.25820002459287660911968487198464572429656982421875f,0.23160000469684594026631430097040720283985137939453125f,-0.218699996779858796269735421446966938674449920654296875f,-0.5220999995708464069821275188587605953216552734375f,-0.21179998106956521297661311109550297260284423828125f,-0.6184999987632038287443947410793043673038482666015625f,0.78129998589754112714445000165142118930816650390625f,-0.261199982959032350748884709901176393032073974609375f,0.63989999173432543511097492228145711123943328857421875f,-1.01280003966093090639333240687847137451171875f,0.3517999982058999108858188265003263950347900390625f,-0.793800040175021148769474166329018771648406982421875f,-0.1732999968349933084255098947323858737945556640625f,-0.008200001049041727441135662957094609737396240234375f,-0.23570000328421591984096039595897309482097625732421875f,-0.31199996280670116988176232553087174892425537109375f,0.828999985963105867625699829659424722194671630859375f,-0.16619999096840609720260317772044800221920013427734375f,0.1373000121533873851831231149844825267791748046875f,-0.186999994397163415982277001603506505489349365234375f,0.002900006091594686807866310118697583675384521484375f,-0.485000019948929494972844622679986059665679931640625f,0.34060000168681103804857457362231798470020294189453125f,-0.436800031342357897301553748548030853271484375f,0.819900031706691123645214247517287731170654296875f,0.4145999880969524209461951613775454461574554443359375f,0.6451000031620264252296692575328052043914794921875f,-0.63970001018345357124417205341160297393798828125f,0.2813999950945376138378151154029183089733123779296875f,-0.5469000187277794911011596923344768583774566650390625f,-0.505000013709068440448390902020037174224853515625f,-0.4864000041842455601681649568490684032440185546875f,-0.592600008302926806180721541750244796276092529296875f,-0.6060000038668513777651014606817625463008880615234375f,0.3389999864846469090906566634657792747020721435546875f,-0.3846000065743921947358785473625175654888153076171875f,-0.0027999991297722459648866788484156131744384765625f,0.428799989771842859909156686626374721527099609375f,-0.0852000210016967685078270733356475830078125f,0.466200021348148696542779134688316844403743743896484375f,0.62939998534917818151512847180129028856754302978515625f,0.43999999718740578913411809480749070644378662109375f,0.3474000258207325941839371807873249053955078125f,0.528800006759166052461296203546226024627685546875f,0.23739999937117062245306442491710186004638671875f,0.66500002954155235102007281966507434844970703125f,-0.171099982404709027861144932103343307971954345703125f,-0.29659999039173090551457789842970669269561767578125f,-1.25690001157522157626544867525808513164520263671875f,-0.698100010031461604143032673164270818233489990234375f,-0.200999994665384207337410771287977695465087890625f,-0.318799986493587539371219463646411895751953125f,0.126299999159574127816085820086300373077392578125f,-0.1420000043809412915152279310859739780426025390625f,-0.203800007504224556242888866108842194080352783203125f,-0.4368000194400549762718810598016716539859771728515625f,0.258400010502338517426323960535228252410888671875f,-0.22860000088214871283298634807579219341278076171875f,0.297600009047985114563061870285309851169586181640625f,-0.1996000053822992725827134563587605953216552734375f,-0.7823000161767001969792545423842966556549072265625f,-0.48920001508593546901693116524256765842437744140625f,0.2075999938488006790748841012828052043914794921875f,0.7008999878525734583689654755289666354656219482421875f,0.04139998786747456360757269067107699811458587646484375f,-0.4998000161767006677138169834506697952747344970703125f,-0.56440000859498962171301172929815948009490966796875f,-0.147699996712803649501211111783050000667572021484375f,0.4146000050842761908143074833787977695465087890625f,0.663400002157688017945247338502667844295501708984375f,0.3252999979376791639396060418221168220043182373046875f,-0.32840001169443144846127324854023754596710205078125f,-0.003299996834993379479783470742404460906982421875f,-0.390200009676814119075061171315610408782958984375f,0.097799990934133607112244135350920259952545166015625f,0.3435999853104354517796537038520909845829010009765625f,0.35949999216198913476461029858910478651523590087890625f,-0.23220001040697102467902368516661226749420166015625f,-0.13060000496506685152553473017178475856781005859375f,0.116900002560019533603963282075710594654083251953125f,0.3949999979138372196985073969699442386627197265625f,0.2482000000059603195268209674395620822906494140625f,0.42090001281201860905412104330025613307952880859375f,0.32479999680519089366725893341936171054840087890625f,0.497799998533725585048159700818359851837158203125f,0.6016000128924847700773170799948275089263916015625f,-0.29260001038908978188146647880785167217254638671875f,0.03550000068545333675729125388897955417633056640625f,-0.387099997559189556994851955096237361431121826171875f,0.52299999397993079952584594138897955417633056640625f,0.49540002453178200170214040554128587245941162109375f,-0.220900016462802994965386460535228252410888671875f,0.8187999964773648997606869670562446117401123046875f,0.01329997295588203343186251004226505756378173828125f,-0.3193999960780136149196550832130014896392822265625f,-0.36939997029900606406727092689834535121917724609375f,-0.1891999806270006079245149521739222109317779541015625f,0.65620003228187595567533207940869033336639404296875f,0.402799999427795096806903529795818030834197998046875f,0.56900001375377173840064415344386361539363861083984375f,-1.2890000352859498633506518672220408916473388671875f,0.08120000188350662284619829733856022357940673828125f,-1.0410000274479391801918382043368183076381683349609375f,-0.0220999882459640417664559208787977695465087890625f,0.55659999188184716700789067544974386692047119140625f,0.1027000014811754180499292488093487918376922607421875f,0.60910001885294917656210600398480892181396484375f,0.050599999749660451442423436674289405345916748046875f,0.3751000052481889834865569355315528810024261474609375f,0.29380000035166720095958226011134684085845947265625f,-0.39899999698996513330939706065692007541656494140625f,0.372800025467574958071281798765994608402252197265625f,-0.2787999969244001619017581106163561344146728515625f,0.013800001841783426925758249126374721527099609375f,-0.3602000072553754872473064096993766725063323974609375f,1.21730004121065160660464243846945464611053466796875f,0.000599994087219446470271577709354460239410400390625f,1.020900032183528427509600078337825834751129150390625f,-0.2421999963253733678669732398702763020992279052734375f,-0.41339997906088854495720852355589158833026885986328125f,-0.2479999826177955168926558826569817028939723968505859375f,0.0339999992251396321307765902020037174224853515625f,-0.46139998943209636905748993740417063236236572265625f,0.0758000204160811374975992293911986052989959716796875f,-1.02000001490116165570043449406512081623077392578125f,-0.012600021713971454317970710690133273601531982421875f,-0.90140001264065461317187555323471315205097198486328125f,-0.811400019383430759489783667959272861480712890625f,-0.424400001740455223853132338263094425201416015625f,-0.75380000407695746389435953460633754730224609375f,1.205900016462802426531197852455079555511474609375f,0.0391999787271029020985224633477628231048583984375f,1.00210001056045339140609939931891858577728271484375f,-0.2186999978415666845421583275310695171356201171875f,-0.060399989253282715395698687643744051456451416015625f,-0.18279999033808724817617985536344349384307861328125f,-0.90029997290372865137442204286344349384307861328125f,-0.600799999219179259313250440754927694797515869140625f,0.1265999954581260578123647064785473048686981201171875f,-0.1554000102639196967402313021011650562286376953125f,0.08949999924004092832774404087103903293609619140625f,-0.479599996739625833441778013366274535655975341796875f,-1.3306999919593331949130288194282911717891693115234375f,-0.80800001335144056024972769591840915381908416748046875f,-0.26740000853538514746787768672220408916473388671875f,-0.295400001472234663424387690611183643341064453125f,0.1925000027567147498075428302399814128875732421875f,-0.3995999971866608380111074438900686800479888916015625f,-0.714399984008073918317904826835729181766510009765625f,-0.1545999877989292503599472183850593864917755126953125f,-0.374599996590614381375416996888816356658935546875f,-0.594299984943866821396341038052923977375030517578125f,-0.4150999954879284192799104857840575277805328369140625f,0.508999990135431179538727519684471189975738525390625f,0.7074999864399433846529063885100185871124267578125f,0.20249998472630981183328913175500929355621337890625f,0.0526000004053113201507585472427308559417724609375f,-0.46019998278021834181572558009065687656402587890625f,0.254100016096234337936721203732304275035858154296875f,0.0291999948203562897930396502488292753696441650390625f,0.78400000220537169326462390017695724964141845703125f,0.220399999013543013148819227353669703006744384765625f,0.220199999171495353067484757048077881336212158203125f,0.4243999993562697081728174453019164502620697021484375f,0.41970001025795944382679181217099539935588836669921875f,0.03420000003576273428507192875258624553680419921875f,-0.09549999740719794516508045489899814128875732421875f,-0.266700006890296936745698985760100185871124267578125f,0.527800004866719074669845213065855205059051513671875f,0.6181000047788021145578341020154766738414764404296875f,0.69870002002567066501370618425426073372364044189453125f,-0.030199986058473626826526015065610408782958984375f,-0.581399981832504408885142765939235687255859375f,-0.3606999931514263124654462444595992565155029296875f,-0.2191999980986116813852504492388106882572174072265625f,0.093799996179342315372196026146411895751953125f,-0.08379999923408032458382876939140260219573974609375f,-0.0920000090003014747708220966160297393798828125f,-0.13429999919310209310907566759851761162281036376953125f,-0.0488999740839002328129936358891427516937255859375f,0.75979997087717077164370493846945464611053466796875f,0.0926999995440247470668282403494231402873992919921875f,-0.6056000052630903240924453712068498134613037109375f,0.124599991375208052346579279401339590549468994140625f,-0.48420000394731743664777923186193220317363739013671875f,0.1759000273406510217455434030853211879730224609375f,0.703799992752074832225162026588805019855499267578125f,0.3655000111907720228288098951452411711215972900390625f,-0.2716000020146367432971601374447345733642578125f,-0.5613999804913998303845801274292171001434326171875f,-0.144199974815547360407208543620072305202484130859375f,-0.10040000951886174362925885361619293689727783203125f,-0.19319999747276295209985619294457137584686279296875f,0.0608000068932772474994408185011707246303558349609375f,0.45530000628232958348462489084340631961822509765625f,0.140399993127584554741815736633725464344024658203125f,0.48750000450760133663408169013564474880695343017578125f,-0.1844999675750731427115169935859739780426025390625f,0.2999999837577345118688754155300557613372802734375f,-0.303499991275369751519974670372903347015380859375f,-1.1948000060439110114884897484444081783294677734375f,-0.10540001458525605215754694654606282711029052734375f,-0.9089999948665494766686379080056212842464447021484375f,0.76600001344084756027541516232304275035858154296875f,-0.47279998780488963205925756483338773250579833984375f,0.516400017371773767393960952176712453365325927734375f,-0.439000017404556519551306337234564125537872314453125f,-0.3289999996721741926108961706631816923618316650390625f,-0.432400006188452146194123315581236965954303741455078125f,0.226100020402669965591258005588315427303314208984375f,0.4203999847829342773053440396324731409549713134765625f,0.187100001098215729111728933276026509702205657958984375f,-1.153900032831729038917956131626851856708526611328125f,-0.276800010424851861134953878718079067766666412353515625f,-1.070900023242831178293954508262686431407928466796875f,-0.23400000488758099237429632921703159809112548828125f,0.588999978810548885377329497714526951313018798828125f,-0.11460002210885289475328363550943322479724884033203125f,0.670999986469745568484768227790482342243194580078125f,-0.2535999830007555910782457431196235120296478271484375f,0.5569999964833256544949335875571705400943756103515625f,0.442600013518333668116611079312860965728759765625f,-0.055000014156103116391705043497495353221893310546875f,0.3262000000953673151116163353435695171356201171875f,0.644000004440545836814635549671947956085205078125f,0.495700006040930674089395324699580669403076171875f,0.1502000037908555185595105285756289958953857421875f,0.0251999963402749216356824035756289958953857421875f,-0.12230001178085814217411098070442676544189453125f,-0.8556000054121017939223747816868126392364501953125f,-1.411899987286329150038000079803168773651123046875f,-0.8934000047653913867407027282752096652984619140625f,0.184600012534856805501704002381302416324615478515625f,-0.34009999131560331164791932678781449794769287109375f,-0.158099995949864347011271092924289405345916748046875f,0.676199999648332550350460223853588104248046875f,0.92559999557733529940151129267178475856781005859375f,0.534499989181756784972776586073450744152069091796875f,-0.68439999831318854006667606881819665431976318359375f,-0.9705999870836734544354840181767940521240234375f,-0.73719999915659428779690642841160297393798828125f,-0.59419998170733467901527546928264200687408447265625f,-1.209599956655502683844360944931395351886749267578125f,-0.8014999826848507868959359257132746279239654541015625f,-0.4707999886393547939178461092524230480194091796875f,-1.0358999732494356127432411085464991629123687744140625f,-0.54859998635351658879955039083142764866352081298828125f,0.27579999415278422247865819372236728668212890625f,1.13299999725818612006378316436894237995147705078125f,0.580700007307529464384288075962103903293609619140625f,-0.454999997615814333329353758017532527446746826171875f,-0.684899982309341570640981444739736616611480712890625f,-0.4716999909460544149197858132538385689258575439453125f,-0.85559997665286058321498785517178475856781005859375f,-1.30769995491504698037488196860067546367645263671875f,-0.903599988663196906912844497128389775753021240234375f,-0.0301999806940556059231539620668627321720123291015625f,-0.65999996915459657742530907853506505489349365234375f,-0.249499992311001062006425854633562266826629638671875f,0.0160000053942201958534496952779591083526611328125f,-0.02799998891353627783473712042905390262603759765625f,-0.243399989268183514212751106242649257183074951171875f,-0.36219999833703031555387497064657509326934814453125f,-0.624899999743699918752781741204671561717987060546875f,-0.3318999980896710777500402400619350373744964599609375f,0.242599995934963175159282400272786617279052734375f,-0.1761000010311601915446999555570073425769805908203125f,-0.42380000288486474602223097463138401508331298828125f,-0.331399996584653688813659755396656692028045654296875f,-0.46119999812096346136769398071919567883014678955078125f,-0.035399987465142945808338481583632528781890869140625f,0.3757999998152252718597310376935638487339019775390625f,-0.052800004456937499330848595491261221468448638916015625f,0.8699000288009646197906477027572691440582275390625f,0.26739999363422395362022143672220408916473388671875f,0.9839000292927029267531224832055158913135528564453125f,-0.586900010085106060842008446343243122100830078125f,0.58359998076558117219292398658581078052520751953125f,-0.4559000273033977368442037914064712822437286376953125f,-0.6953999798655505770739182480610907077789306640625f,0.06460000091195094995555336936376988887786865234375f,-0.7615999966874718030140911650960333645343780517578125f,0.970200009751319925044299452565610408782958984375f,0.194799996805191000248669297434389591217041015625f,0.98780000978410242851168732158839702606201171875f,0.55340000126361843513222993351519107818603515625f,0.48539997733235384913541565765626728534698486328125f,0.669599989996850553097829106263816356658935546875f,0.18269999350905408164180698804557323455810546875f,0.297799985420704071970021686865948140621185302734375f,0.3350999921351671684277562235365621745586395263671875f,0.2623999939322469998614906216971576213836669921875f,-0.66339999068379373881043647998012602329254150390625f,0.0374000058904291332595448693609796464443206787109375f,0.55590000364780411956644456950016319751739501953125f,0.117999987274408635329336902941577136516571044921875f,0.59010000409334917303993961468222551047801971435546875f,1.042500000000000426325641456060111522674560546875f,0.2490000069737430266059163841418921947479248046875f,1.10969999736845448268240943434648215770721435546875f,-0.53139999926686254383412233437411487102508544921875f,0.47099998959898936590207085828296840190887451171875f,-0.343200013901293399243286330602131783962249755859375f,0.322399994379281462641984035144560039043426513671875f,0.3427999717116361200197616199147887527942657470703125f,0.258799980384111616427134094919892959296703338623046875f,0.25269999008178700705684605054557323455810546875f,0.06240000302195536363569772220216691493988037109375f,0.32729999721497282649806948029436171054840087890625f,0.4750000038743014130204755929298698902130126953125f,0.102500009760260635260920025757513940334320068359375f,-0.083200002688169405473672668449580669403076171875f,0.206399995392561041995804771431721746921539306640625f,-0.2278000124663115411038916136021725833415985107421875f,0.5942000125527382436985135427676141262054443359375f,0.674999997168779231060398160479962825775146484375f,0.558100013384223103685144451446831226348876953125f,0.671999996185302705953290569595992565155029296875f,1.2624999825656413321439686114899814128875732421875f,0.87240000697076336422242093249224126338958740234375f,0.30259998982548719226315370178781449794769287109375f,0.2705999744176867949363440857268869876861572265625f,-0.29390002444982510354520854889415204524993896484375f,-0.4863999885380270171708616544492542743682861328125f,-0.82819996603131329493407974950969219207763671875f,-0.5865999807804822552270707092247903347015380859375f,0.0676000045776365965366494492627680301666259765625f,-0.4581999819755555591882512089796364307403564453125f,0.21260000517368293770914533524774014949798583984375f,0.006199993687868055047829329851083457469940185546875f,0.69950000080466256946465364308096468448638916015625f,0.333900000682473052648902012151665985584259033203125f,0.1162000066518782404045850853435695171356201171875f,0.500000013858079572770520826452411711215972900390625f,0.3783000185161828898827707234886474907398223876953125f,-0.07479999933838854531131801195442676544189453125f,0.01060001152157763470995632815174758434295654296875f,0.380400014957785526092948202858678996562957763671875f,-0.51080000368952749312256855773739516735076904296875f,-0.6522999982953070485791613464243710041046142578125f,-0.62880000482201570122242628713138401508331298828125f,-0.28859999819993975567200550358393229544162750244140625f,-0.306799989041686149704446506802923977375030517578125f,-0.1156999941200018700016016737208701670169830322265625f,-0.43319998450875285556094240746460855007171630859375f,-0.2288999731898311296163228689692914485931396484375f,-0.42159998070597648478496921597979962825775146484375f,0.0929999988973140290227092918939888477325439453125f,0.449700001913309055368017652654089033603668212890625f,-0.3075000146776438736395675732637755572795867919921875f,0.4703999986410138944847858510911464691162109375f,0.7623000052988526675790126319043338298797607421875f,0.61000000782310959124288274324499070644378662109375f,0.50780000479221332199131211382336914539337158203125f,-0.091500006712973147582346200579195283353328704833984375f,0.8272000211358072618850201251916587352752685546875f,0.1020000054240224329760167165659368038177490234375f,0.6492000098705290600520356747438199818134307861328125f,-0.96120004301071215735419173142872750759124755859375f,-0.014200003761053014983417597250081598758697509765625f,-0.93300004252046375707863035131595097482204437255859375f,-0.72890002713203472239911206997931003570556640625f,-0.47379999543428397146271890960633754730224609375f,-0.82730001662373542359318889793939888477325439453125f,0.3348000328660010183057238464243710041046142578125f,0.75139997721314433221095896442420780658721923828125f,0.1925999984309079149369381411815993487834930419921875f,0.58339999619722338053406929248012602329254150390625f,0.259999976456165793337049763067625463008880615234375f,0.63339999142885228256005802904837764799594879150390625f,-1.151799993586539727630224660970270633697509765625f,-0.0303999989390373315245597041212022304534912109375f,-0.876999988511204531249632054823450744152069091796875f,0.4975000077486040339636019780300557613372802734375f,-0.6267999815165996579935381305404007434844970703125f,0.314900020493567023294190221349708735942840576171875f,-0.07140002385377908211694375495426356792449951171875f,-0.456599996501207083809958930942229926586151123046875f,-0.13240001095682385479079812284908257424831390380859375f,-0.46380001689195626823902784963138401508331298828125f,-0.601199986982345446762110441341064870357513427734375f,-0.36139999174177639407190554265980608761310577392578125f,0.9591000148296355831689652404747903347015380859375f,0.22379999424219132464486392564140260219573974609375f,0.9167000106900931033493407085188664495944976806640625f,0.247399998179078206561598562984727323055267333984375f,-0.19739999333620072974326831172220408916473388671875f,0.27520000800043342703560256268247030675411224365234375f,1.1188999782562252249817902338691055774688720703125f,-0.05020000349283204599259988754056394100189208984375f,0.9836999802023169703346638925722800195217132568359375f,1.002900037980079872568239807151257991790771484375f,0.39719998909831044642260167165659368038177490234375f,0.73590000778287656402198990690521895885467529296875f,-0.691100000286102300606216886080801486968994140625f,-0.2942000048041339965720908367075026035308837890625f,-0.709499998420476618576913097058422863483428955078125f,-0.96960002847909976253504282794892787933349609375f,-0.383200003805756583830088857212103903293609619140625f,-0.3216000017166138480462223014910705387592315673828125f,-0.2701999899327756082811902160756289958953857421875f,-0.04319999136328700473086428246460855007171630859375f,-0.474800002095103312882429236196912825107574462890625f,-0.5142999912396073458609180306666530668735504150390625f,-0.56400000172108410989579851957387290894985198974609375f,-0.44780001522302637084749221685342490673065185546875f,-0.51590000454187379119730394450016319751739501953125f,-0.298100004965066833761966336169280111789703369140625f,0.1555999968439341074599724379368126392364501953125f,0.23559999129325159827885727281682193279266357421875f,0.2325999958977102988200158506515435874462127685546875f,0.005800026637315713884390788734890520572662353515625f,-0.77819996439218552808370077400468289852142333984375f,-0.292699984717369243725215710583142936229705810546875f,-0.0682000061154365011528710738275549374520778656005859375f,-0.792600015585869233092353169922716915607452392578125f,-0.1861000032290813788193872824194841086864471435546875f,0.00659999739527705742148100398480892181396484375f,0.28299999554455279593412342364899814128875732421875f,0.28830000354051588118409199523739516735076904296875f,-0.28300000128149971345692392787896096706390380859375f,-0.11830000190138800775230265571735799312591552734375f,-0.507900019726157392341292506898753345012664794921875f,0.1559999943673611166872206013067625463008880615234375f,0.48869998541772374789360355862299911677837371826171875f,0.180599996173381904807087039444013498723506927490234375f,0.336200007843971082621692403336055576801300048828125f,0.75530001872479879398980529003893025219440460205078125f,0.129500001549720578086777322823763824999332427978515625f,-0.36279997692704224476756280637346208095550537109375f,-1.202599950188398825190461138845421373844146728515625f,-0.6891999789506197959809696840238757431507110595703125f,0.58920000927448246130779807572253048419952392578125f,0.7331000183761116062441942631267011165618896484375f,0.03140000083148475340522054466418921947479248046875f,-0.03339997563362118171426118351519107818603515625f,-0.7697999790728093927327790879644453525543212890625f,-0.52369998407661899619824907858856022357940673828125f,0.457400001233816133616727483968134038150310516357421875f,0.9342999959334730153415193854016251862049102783203125f,0.32829999489784233990263828673050738871097564697265625f,0.02340000856518731353617113200016319751739501953125f,-0.018500024616718402370452167815528810024261474609375f,0.24459999673962595778675677138380706310272216796875f,-0.35079999340772616278627538122236728668212890625f,0.2734000127375126876216882010339759290218353271484375f,0.99630001759976150044195719601702876389026641845703125f,0.154199993181228733885035353523562662303447723388671875f,0.9251000125870110224468589876778423786163330078125f,0.5157000222086907115226495079696178436279296875f,-0.14320000149607636785731301642954349517822265625f,0.537100031720102233379066092311404645442962646484375f,-0.30320000071376551620261352582019753754138946533203125f,-0.131599991881847511177028309248271398246288299560546875f,-0.292399996726214916886732453349395655095577239990234375f,-0.7036000163793563189074120600707828998565673828125f,0.616199982362985565487178973853588104248046875f,-0.4722000327959658960708111408166587352752685546875f,-0.76400000788643984694914479405269958078861236572265625f,0.37259998598843824257897949792095459997653961181640625f,-0.50820000587329250407719882787205278873443603515625f,-0.0536000119835138288948428453295491635799407958984375f,-0.44259998446106918468245794429094530642032623291015625f,-0.0769999957010150948288895733639947138726711273193359375f,1.1533000339388852495403625653125345706939697265625f,0.25400001069903321848642008262686431407928466796875f,0.9767000175818798535232190261012874543666839599609375f,0.15929999634325486823627215926535427570343017578125f,-0.1409999961555004066582341692992486059665679931640625f,0.04769999820291992431720018430496565997600555419921875f,0.31650001926720161549155818647705018520355224609375f,0.1167999897122384833636488110641948878765106201171875f,0.1162999972596765463350720892776735126972198486328125f,-0.034600000464916291775807621888816356658935546875f,0.69079996569156687513668657629750669002532958984375f,0.165399980387091805056343218893744051456451416015625f,-0.014299975109100415693319519050419330596923828125f,0.631799963188171886940835975110530853271484375f,-0.1819000168651339066627770080231130123138427734375f,-0.24720003186464367672670050524175167083740234375f,0.84859996914267554757316247560083866119384765625f,-0.10260005274564054644770294544287025928497314453125f,0.0451000074461103128697914144140668213367462158203125f,-0.0067999997705221471022696277941577136516571044921875f,-0.1391000058144331585907593762385658919811248779296875f,0.02390000090599020410309094586409628391265869140625f,0.043900003439187873510718418401665985584259033203125f,-0.4053999978587030650345468529849313199520111083984375f,-0.20429999453648928220417246848228387534618377685546875f,-0.227499995175749092357619929316570051014423370361328125f,0.5398000234782696082902475609444081783294677734375f,0.54470001353621455564280040562152862548828125f,-0.051699993553757650488478248007595539093017578125f,-0.4522000138342381347911214106716215610504150390625f,-0.45400000399351103652634265017695724964141845703125f,-0.655300014477968151282993858330883085727691650390625f,0.8991999900519846988800054532475769519805908203125f,1.3775999770104885300270325387828052043914794921875f,0.90139999427497397022079894668422639369964599609375f,-0.8108000075638290837787280906923115253448486328125f,-1.1584000189959997584310258389450609683990478515625f,-0.82520001682937138554052580730058252811431884765625f,-0.20379999111294733182830896112136542797088623046875f,-0.076499994009733285338370478712022304534912109375f,0.1305000134259461930241741356439888477325439453125f,-0.1347999926328657949170519714243710041046142578125f,-0.65309999662041651191657365416176617145538330078125f,-0.505800007936358486659855770994909107685089111328125f,0.2898000020205973026321544239181093871593475341796875f,0.67820000604093044760389830116764642298221588134765625f,0.6261000201046467150689522895845584571361541748046875f,-0.398999995350837632912543995189480483531951904296875f,-0.6917999940335750608255693805404007434844970703125f,-0.03129998418390744774342238088138401508331298828125f,0.16600000099837775024980146554298698902130126953125f,0.246300000835955046341041452251374721527099609375f,0.3148000082418322964628032423206605017185211181640625f,0.2084000140786173016493876275490038096904754638671875f,-0.29610000177621831340957214706577360630035400390625f,0.025400001397728910745854591368697583675384521484375f,-0.4039999959468840717846660481882281601428985595703125f,-0.27849999138712888679947354830801486968994140625f,-0.11959999293982992440987800364382565021514892578125f,-0.1361999871313572807451919288723729550838470458984375f,-0.63709997423887276823961656191386282444000244140625f,-0.060899985095858699679638448287732899188995361328125f,-0.456199997115135147396358661353588104248046875f,-0.47919999273419389140826751827262341976165771484375f,0.181300023075938288030783951398916542530059814453125f,-0.450200010794401084268656632048077881336212158203125f,-0.24750002566725048946949527817196212708950042724609375f,0.7920999907977879228582196446950547397136688232421875f,-0.2627999869108201824730031148646958172321319580078125f,0.71550000286474801924896382843144237995147705078125f,-0.0733000172495845703224404132924973964691162109375f,0.71959997587800028640003802138380706310272216796875f,-0.1999000527545813366003812916460447013378143310546875f,1.193400042688846696137261460535228252410888671875f,0.286599987262487587713621906004846096038818359375f,1.072400029732287318751104976399801671504974365234375f,-1.012799998831749093142207129858434200286865234375f,-0.2012000039696690834034598083235323429107666015625f,-1.100400003707408647102283794083632528781890869140625f,1.0588000052690507146735399146564304828643798828125f,0.17200001823902066888649642351083457469940185546875f,1.0988000144332643603917176733375526964664459228515625f,0.041899963891506075697179767303168773651123046875f,-0.451999978452921169491673936136066913604736328125f,0.07169998703449920185448718257248401641845703125f,0.08150000876188290277468695421703159809112548828125f,0.636999990224837997487838947563432157039642333984375f,0.16109999007880670962578051330638118088245391845703125f,-0.517600029835105335251910219085402786731719970703125f,-0.56299999904632525016268118633888661861419677734375f,-0.5350000117346642714011295538512058556079864501953125f,-0.26680003590583822870030417107045650482177734375f,-0.35619998713135725409983933786861598491668701171875f,-0.0750000071898104803125306716538034379482269287109375f,-0.43870001510828748880754801575676538050174713134765625f,-0.223799999010562744583552330368547700345516204833984375f,-0.40350000640004868301957685616798698902130126953125f,-0.8603000269949436784600038663484156131744384765625f,0.372600001299380867436639164225198328495025634765625f,-0.68310003201067492906162215149379335343837738037109375f,0.504099986219406215326443998492322862148284912109375f,-0.3757999805927279002304430832737125456333160400390625f,0.4937000042676922273443551603122614324092864990234375f,-0.61070001282095898176294213044457137584686279296875f,0.0631999955356121745353448204696178436279296875f,-0.3941000080868601873618217723560519516468048095703125f,-0.188600047075748733504951815120875835418701171875f,-0.77779997722506521995455841533839702606201171875f,-0.029200003127753948461986510665155947208404541015625f,0.1863000067591666919497583876363933086395263671875f,0.00360001228153716823499053134582936763763427734375f,0.64339999843239770171976488200016319751739501953125f,0.8717999938845633067785456660203635692596435546875f,0.64290000609159481115284506813623011112213134765625f,0.15219999178051946131517979665659368038177490234375f,0.86179999120235439846737790503539144992828369140625f,0.302899994766712143956510772113688290119171142578125f,0.392800006133317669565485630300827324390411376953125f,1.0460000207424158258362467677216045558452606201171875f,0.75500002451241010614779725074185989797115325927734375f,-0.03780001716017711288486680132336914539337158203125f,0.72499997049570108487159814103506505489349365234375f,0.228299990278482578531793478759936988353729248046875f,0.16419999526739115225382192875258624553680419921875f,0.002499995231628471259455182007513940334320068359375f,0.2134999982416629915604744383017532527446746826171875f,-0.1138000002026557933021422286401502788066864013671875f,-0.3821999987840651868964414461515843868255615234375f,-0.19830000182688234389161152648739516735076904296875f,-0.13560000213384615364020646666176617145538330078125f,0.26169999347925188004637675476260483264923095703125f,0.449800008353590907717034497181884944438934326171875f,-0.1081999935984612193351495079696178436279296875f,-0.212699980843067404379098661593161523342132568359375f,0.3577000160098073866521417585317976772785186767578125f,-0.0945999970749020757665448400075547397136688232421875f,-0.19959999459758404061204828394693322479724884033203125f,-0.251299999886006097948865090074832551181316375732421875f,0.423600008854270182467871563858352601528167724609375f,0.4202999914184213370305087664746679365634918212890625f,0.44999999877065410824883429086185060441493988037109375f,-0.421600003802776512884520343504846096038818359375f,-0.4342999861359597701948587200604379177093505859375f,-0.311199991974234535518917255103588104248046875f,0.5174000118136401482615838176570832729339599609375f,0.61610003426074921861754773999564349651336669921875f,-0.101499991178512605216610609204508364200592041015625f,0.076000002264976274091168306767940521240234375f,0.51460001715421643808667795383371412754058837890625f,-0.165999997645616570451920779305510222911834716796875f,-0.228399997016787370540669144247658550739288330078125f,-0.0920000015124677883449066939647309482097625732421875f,-0.15660000249892480628233215611544437706470489501953125f,-0.04220002307295800392239470966160297393798828125f,0.228499987810849791713962986250407993793487548828125f,-0.89200001049041777179127166164107620716094970703125f,-0.107000011533498362581440233043394982814788818359375f,-0.90780001332312798556500865743146277964115142822265625f,0.52760001083612451111548580229282379150390625f,-0.4149999879300594329833984375f,0.299000014238059552695858656079508364200592041015625f,-0.360400021141767723520388244651257991790771484375f,-0.681999999761580877333244643523357808589935302734375f,-0.467200007724761778860056438134051859378814697265625f,0.785599999749660327097444678656756877899169921875f,-0.5665999792158604719816139549948275089263916015625f,0.5486000161185859980150780756957828998565673828125f,-0.63709999599456779151296359486877918243408203125f,-0.02400000235438337625737403868697583675384521484375f,-0.5332999935939906066550975083373486995697021484375f,-0.04719999505877492396166417165659368038177490234375f,0.1953999943196773614317862666212022304534912109375f,0.0234000015988946596934283661539666354656219482421875f,-0.090100030058622326833983606775291264057159423828125f,-0.8573999650239947367680315437610261142253875732421875f,-0.0832999940782788161897087775287218391895294189453125f,-0.7015000185966488999156354111619293689727783203125f,-0.5555999823153019434585075941868126392364501953125f,-0.5204999931976199700756069432827644050121307373046875f,0.277400004493445095921089205148746259510517120361328125f,0.2369999911189079877527063899833592586219310760498046875f,0.25879999578818679850655826157890260219573974609375f,-0.86070002410858859054343383832019753754138946533203125f,-0.058199983763695228855539198775659315288066864013671875f,-0.7983000213846569437237121746875345706939697265625f,0.611900009340047734696099723805673420429229736328125f,-0.1114000089526172576626095178653486073017120361328125f,0.6007000152051451546952876014984212815761566162109375f,0.15450003165006620520216529257595539093017578125f,0.7275999478042134427369092009030282497406005859375f,-0.02290001603811919750341985491104423999786376953125f,0.9022000244140624403144101961515843868255615234375f,0.17159998025894207529518098453991115093231201171875f,0.6502000024497511798671212090994231402873992919921875f,0.52679999377280484207375366167980246245861053466796875f,-0.118799983811378850884210578442434780299663543701171875f,0.464799999524652618365649914267123676836490631103515625f,0.044200024026632700469008341315202414989471435546875f,0.4441000266015528996632610869710333645343780517578125f,-0.3302000002145766277550364975468255579471588134765625f,-0.5939000020980833571826451589004136621952056884765625f,-0.29490000019073481229980870921281166374683380126953125f,0.66000001028180133033629317651502788066864013671875f,1.10550000292062744478016611537896096706390380859375f,0.713799993497132856390408051083795726299285888671875f,-0.3929999921917914917912639793939888477325439453125f,-0.55259998276606214773210012936033308506011962890625f,-0.2240999901309610464750221581198275089263916015625f,0.171000016085803618892668964690528810024261474609375f,-0.08489999171569950764393297504284419119358062744140625f,-0.085499996121972843265979236093699000775814056396484375f,-0.15259999176263772113770755822770297527313232421875f,-0.995300016117095509571299771778285503387451171875f,-0.3809000075221060654939719825051724910736083984375f,0.277600011581182304354342704755254089832305908203125f,0.529700024712085326683563835103996098041534423828125f,0.0182000035822389438777690884307958185672760009765625f,0.01800000426173209433500232989899814128875732421875f,0.59500000372529004977195654646493494510650634765625f,0.542800022897124367915466791600920259952545166015625f,0.204399997568130320502177710295654833316802978515625f,-0.16229999695420271876855622394941747188568115234375f,0.225400007060170270989374330383725464344024658203125f,0.2406000089883804093915387056767940521240234375f,0.285100009047984936927377930260263383388519287109375f,-0.0827000023007393725293923125718720257282257080078125f,-0.1590000021681188258071415475569665431976318359375f,-0.4224000060580668236553947281208820641040802001953125f,-0.54110002168789517185132353915832936763763427734375f,0.012799998831749093142207129858434200286865234375f,-0.134400010383128876156888509285636246204376220703125f,-0.4440000200867653123992795372032560408115386962890625f,0.19299999919533750158961993292905390262603759765625f,0.18169998379349738826249449630267918109893798828125f,-0.329900027534365758441481375484727323055267333984375f,-0.4241999961614608682936022887588478624820709228515625f,-0.96729999434649938105934552368125878274440765380859375f,-0.5958999932169912749913009974989108741283416748046875f,-0.609800009173155022068613106966949999332427978515625f,-0.74069998361468325498435660847462713718414306640625f,-0.170099992731213678354151852545328438282012939453125f,-0.041799997609853534186186152510344982147216796875f,-0.355500008694827673849658822291530668735504150390625f,0.18169999347925180899210317875258624553680419921875f,0.054400003677606445506853560800664126873016357421875f,0.2623000004932283246716906432993710041046142578125f,-0.686299998116492560029655578546226024627685546875f,-0.2425999747753149193840727093629539012908935546875f,-0.77310000451803251309002007474191486835479736328125f,0.24239998910799631648416152529534883797168731689453125f,-0.14339999176412832060378832466085441410541534423828125f,0.3150000003539024095999820929137058556079864501953125f,-0.297300017610937439282992045264109037816524505615234375f,0.39359999492168407098091620355262421071529388427734375f,-0.25230003050044214063518666080199182033538818359375f,-0.600900026595592606781792710535228252410888671875f,0.55400000503659185824290034361183643341064453125f,-0.34190002725869472754993694252334535121917724609375f,0.2639000181913377929276975919492542743682861328125f,0.23779998631477372583731266786344349384307861328125f,0.10909999366998679448670372948981821537017822265625f,-0.01780004681348845707589134690351784229278564453125f,-0.843599981212615812609101340058259665966033935546875f,-0.08660001460611843615566840526298619806766510009765625f,-0.9757000025391580066980168339796364307403564453125f,0.07139998362064403636395581997931003570556640625f,-0.79050000090897043492077500559389591217041015625f,0.127100009107589784207448246888816356658935546875f,0.38399998864531514897180386469699442386627197265625f,0.03369998657256356722200507647357881069183349609375f,0.5278000349111859890172127052210271358489990234375f,0.57579999743103937870358777217916212975978851318359375f,0.494400011966377395111749137868173420429229736328125f,0.4928000271439554325070275808684527873992919921875f,0.62559999229907958806506940163671970367431640625f,0.4080000005364416981734621003852225840091705322265625f,0.4573000328660015156856388784945011138916015625f,0.657999998897313531642794259823858737945556640625f,0.316700000445544471716630141600035130977630615234375f,0.2778999887317414874132737168110907077789306640625f,0.4079999820590021197830310484278015792369842529296875f,0.3964999866709113529594787905807606875896453857421875f,0.264100016021728833948145620524883270263671875f,-0.48399999222159362233242063666693866252899169921875f,0.3855000396147374441824240420828573405742645263671875f,-0.8707000317454340887479702359996736049652099609375f,0.912300001126527870809468367951922118663787841796875f,-0.8087999866425992223639696021564304828643798828125f,-1.1586999619483950851872577914036810398101806640625f,-0.84299998295307165818712746840901672840118408203125f,0.09780000821948053868482020334340631961822509765625f,-0.180799986255169020665789503254927694797515869140625f,0.3369000038266178886914303802768699824810028076171875f,0.001999995663762188513601358863525092601776123046875f,0.2438999905869365658617198278079740703105926513671875f,0.28359999793916934596182954919640906155109405517578125f,-0.07159999501109126640585600398480892181396484375f,0.1001000033855437010288369492627680301666259765625f,-0.23799999025464035895538472686894237995147705078125f,0.15359999790191647406345509807579219341278076171875f,0.196599997395277004130775821977294981479644775390625f,0.3060000027120113319512029192992486059665679931640625f,0.30420000659227375283677474726573564112186431884765625f,-0.0102000014811753914045766578055918216705322265625f,0.05349999831616880641149691655300557613372802734375f,0.00360000118017200776421304908581078052520751953125f,0.64820000313520420576196556794457137584686279296875f,0.053799994987249188937994404113851487636566162109375f,0.00320000328421610191753643448464572429656982421875f,-0.54770000975131960530006836052052676677703857421875f,-0.363800012123584703971346243633888661861419677734375f,-0.1251999958187341999149566618143580853939056396484375f,-0.35769999190717938208905479768873192369937896728515625f,-0.419200002159178242155945781632908619940280914306640625f,0.4075999862492085679122055807965807616710662841796875f,0.87709996894896058439172747966949827969074249267578125f,0.459199983048439064337031823015422560274600982666015625f,0.1337999991968275015352674017776735126972198486328125f,-0.17760000768825390249361362293711863458156585693359375f,-0.250500018585473382959349919474334456026554107666015625f,-0.2005999930441380030288200941868126392364501953125f,-0.44479998548030863503299769945442676544189453125f,-0.428299997580051439172166283242404460906982421875f,0.3953999923825264328769435451249592006206512451171875f,0.85829998334944257454282023900304920971393585205078125f,0.393099988424777946338650735924602486193180084228515625f,-0.0014000022098421549543445507879368960857391357421875f,0.35079999528899785676827605129801668226718902587890625f,0.441700008063763360777187472194782458245754241943359375f,-0.84459999629259119302560065989382565021514892578125f,-1.1386000097855923929301980024320073425769805908203125f,1.083499998688697729676277958787977695465087890625f,0.1977999881029131046261682058684527873992919921875f,1.118299998474121093039457264239899814128875732421875f,-0.991400005078315160744750755839049816131591796875f,-0.22659998189806973556414959602989256381988525390625f,-0.785599994273483925866230492829345166683197021484375f,-0.09630000739246591390241292174323461949825286865234375f,-0.431799978685379282072887008325778879225254058837890625f,-0.1058999910190703275247869896702468395233154296875f,0.66349998348951277904461676371283829212188720703125f,0.164599983775616021119958531926386058330535888671875f,0.52909997705519196120604874522541649639606475830078125f,-0.476600006037950496562416446977294981479644775390625f,-0.2163999955415725207075183789129368960857391357421875f,-0.426199997860193191545619129101396538317203521728515625f,-0.650000005438923889045099713257513940334320068359375f,0.3957999895334243234401583322323858737945556640625f,-0.56220001871436842133533673404599539935588836669921875f,0.87780001194477108583669178187847137451171875f,-0.2583999842762949583629961125552654266357421875f,0.625400008699297682568385425838641822338104248046875f,-0.238699995923042251888546161353588104248046875f,0.633399992322921434606541879475116729736328125f,-0.127100012236833759260434817406348884105682373046875f,0.36690001183599230127896362319006584584712982177734375f,0.452199984329938964311423887920682318508625030517578125f,0.353299995009601186524150762124918401241302490234375f,-0.1622999906212090337476183776743710041046142578125f,-0.2317999856889250143154868055717088282108306884765625f,-0.2910999894082546557427804145845584571361541748046875f,-0.102599972894042379412127274918020702898502349853515625f,0.57539999327659574479554294157424010336399078369140625f,-0.08039999638721351260528535931371152400970458984375f,0.387000007212161989400556194595992565155029296875f,0.3757999861061573909637445467524230480194091796875f,0.390399994133412864272258957498706877231597900390625f,-0.0616999878913162547178217209875583648681640625f,-0.1603999914884568500639261401374824345111846923828125f,-0.206699992398917675728853282635100185871124267578125f,-0.185400023209303654869728461562772281467914581298828125f,-0.6705999770998956055478856796980835497379302978515625f,-0.18919999681338683927833699272014200687408447265625f,0.83240001062154789224223350174725055694580078125f,-0.2810000093430280454498415565467439591884613037109375f,-0.5419999879896639782828060560859739780426025390625f,-0.6963999858558178601697363774292171001434326171875f,-0.68199999134242528242566550034098327159881591796875f,-0.10959999703764911060943632037378847599029541015625f,-0.440799999219179117204703288734890520572662353515625f,0.0502000086337328621510778248193673789501190185546875f,-0.063599986353516602122226686333306133747100830078125f,-0.4561999836668373387027486387523822486400604248046875f,-0.54319999713748690073344960183021612465381622314453125f,-0.48859999864697467586438506259582936763763427734375f,-0.7340999833881856062589577049948275089263916015625f,-0.53109999365508553381687306682579219341278076171875f,-0.298800013166665934249976999126374721527099609375f,-0.06120001544356323819329190882854163646697998046875f,0.092399994304776189579797573969699442386627197265625f,-0.01480000172257422974553264793939888477325439453125f,0.38249999940395351671895696199499070644378662109375f,-0.065200005280971584653570971568115055561065673828125f,-0.07520000572800622506974832504056394100189208984375f,0.471499987527728148251071615959517657756805419921875f,0.4808000036895274664772159667336381971836090087890625f,0.6320000134706498240433347746147774159908294677734375f,0.8108999997735022446931907325051724910736083984375f,0.38599999801814544042599663953296840190887451171875f,-0.4988000140607358190436571021564304828643798828125f,-0.5021999998271464704657773836515843868255615234375f,-0.16639999874532218626654866966418921947479248046875f,-0.36679999582171429750587776652537286281585693359375f,-0.47779999421238894541374975233338773250579833984375f,-0.384400001665949897500240695080719888210296630859375f,0.726599997693300192480592158972285687923431396484375f,1.2188999906241893000924392254091799259185791015625f,0.61689999198019496162714858655817806720733642578125f,0.199599991673231169642122040386311709880828857421875f,0.686699981558323013786093724775128066539764404296875f,0.2323999940812588871352772912359796464443206787109375f,-0.24680000498592857827162561079603619873523712158203125f,-0.0741999983593821799132683736388571560382843017578125f,-0.02979999661892651641181828381377272307872772216796875f,-0.47620000292658815066459965237299911677837371826171875f,-0.69469998842775826375373071641661226749420166015625f,-0.292099993833899507222895408631302416324615478515625f,0.27500001817941654991273026098497211933135986328125f,0.1795999895870685758580975743825547397136688232421875f,1.0351999835252758686010565725155174732208251953125f,0.25759999563693991575519248726777732372283935546875f,0.9871999811261888968516586828627623617649078369140625f,-0.5440000230669976843955737422220408916473388671875f,-0.09719999669790269081204314716160297393798828125f,-0.3676000086009503231565531677915714681148529052734375f,0.30860000240951757088936346917762421071529388427734375f,0.733199974524974873446581113967113196849822998046875f,0.3093999759986996167526740464381873607635498046875f,0.63870000844001761919344062334857881069183349609375f,0.157599987739324820523734160815365612506866455078125f,0.66690000721663256211257930772262625396251678466796875f,0.2976000010013581942303062533028423786163330078125f,-0.55199997532367728325652933563105762004852294921875f,0.2880000254958867689225598951452411711215972900390625f,0.70510001292228707825415767729282379150390625f,-0.054599996292591157498463871888816356658935546875f,0.521100006470084142762289047823287546634674072265625f,0.0652999765545130284039032630971632897853851318359375f,-0.86859996422529250903465936062275432050228118896484375f,0.00770000982582541571019874027115292847156524658203125f,-0.54770001139044754978613127605058252811431884765625f,0.221400001502036758438407559879124164581298828125f,-0.6237000290036205374377686894149519503116607666015625f,0.41540001354217537254953640513122081756591796875f,-0.392399994826316600438076420687139034271240234375f,0.46260003341138400401177932508289813995361328125f,0.56620000098943723543243322637863457202911376953125f,0.078000007390975678589484232361428439617156982421875f,0.56140000187456584601619624663726426661014556884765625f,-0.4798999876737592984454749966971576213836669921875f,-0.0524000003397464553245299612171947956085205078125f,-0.63090000189691775034361853613518178462982177734375f,0.293699999499320796303436509333550930023193359375f,-0.2005999949812888871747418306767940521240234375f,0.1257000009745359303536815787083469331264495849609375f,0.38180000062733887578048097566352225840091705322265625f,-0.313599993282556444018638330817339010536670684814453125f,0.349400013102591111557870817705406807363033294677734375f,0.289300004836916979655825343797914683818817138671875f,-0.140000000298023064004837578977458178997039794921875f,0.381100016789138518191748516983352601528167724609375f,0.47350001031160360298599698580801486968994140625f,-0.392300000828504469296831302926875650882720947265625f,-0.13180001474618929790949550806544721126556396484375f,0.4557999956429004129176973947323858737945556640625f,-0.08290000393092622488211418385617434978485107421875f,-0.095799996834993184080531136714853346347808837890625f,0.2486999956250190990658666123636066913604736328125f,0.2153000033766030441029215580783784389495849609375f,0.099199990648031366458070579028571955859661102294921875f,0.2626999757207932884028878106619231402873992919921875f,-0.130800017398595702644570337724871933460235595703125f,0.34239998916387559546592456172220408916473388671875f,0.94229997579455382350488434894941747188568115234375f,0.698400005659461253770814437302760779857635498046875f,-0.132000000506639292296995336073450744152069091796875f,0.3752999903380871415947694913484156131744384765625f,0.2602000033438205406355336890555918216705322265625f,-0.131200001585483505550655536353588104248046875f,-0.116000001668929986209377602790482342243194580078125f,-0.1380000045597553626208764399052597582340240478515625f,0.6245999815404417176978313364088535308837890625f,1.11139995381832168419578010798431932926177978515625f,0.679099974894523672475088460487313568592071533203125f,-0.0044000144064424429046766817918978631496429443359375f,-0.11590003114044622467559975120821036398410797119140625f,0.325599989914894194509287217442761175334453582763671875f,-0.458999987304210765870493560214526951313018798828125f,-0.9028999645173552135446470856550149619579315185546875f,-0.63649999654293087925083227673894725739955902099609375f,-0.3280000130534173052865298814140260219573974609375f,-0.3311000102698802294298729975707828998565673828125f,0.12480001565516030126445912173949182033538818359375f,-0.227399995422363332409076974727213382720947265625f,-0.4294999867975712870560300871147774159908294677734375f,-0.21749999359250071773175250200438313186168670654296875f,0.59060000928640388195844934671185910701751708984375f,0.78349999034404760323013761080801486968994140625f,0.57069999240636803961024270392954349517822265625f,-0.22980000887513174490095479995943605899810791015625f,0.0880000018775461967379669658839702606201171875f,0.21310001181960114280400375719182193279266357421875f,0.03840000334978099783000971001456491649150848388671875f,0.153400006367266084073008869381737895309925079345703125f,-0.1592000021219253935722548476405791006982326507568359375f,0.0417999885201452148209000370115973055362701416015625f,0.1875999885961416868696005622041411697864532470703125f,1.054400026476383089857336017303168773651123046875f,-0.2314000070154662580534932203590869903564453125f,0.8086000246867541818573954515159130096435546875f,0.197499976903200202826837994507513940334320068359375f,-0.4943999720871452563386583278770558536052703857421875f,0.142499996535479567238979825560818426311016082763671875f,0.4261000142000614943071923335082828998565673828125f,0.237799998943507506776740001441794447600841522216796875f,0.29570000244602556715989294389146380126476287841796875f,-0.333500027894974238051872816868126392364501953125f,-0.6549999883770940556360073969699442386627197265625f,-0.4263000051200389606265162001363933086395263671875f,0.379899974411725960266039692214690148830413818359375f,-0.6445999685764316478042701419326476752758026123046875f,0.2786999993503089800839234158047474920749664306640625f,0.285600006902217984361413982696831226348876953125f,0.01200000452995286792656770558096468448638916015625f,0.2608000065207480844264864572323858737945556640625f,-0.55759998997449855551167274825274944305419921875f,-0.2537999791920189096572357811965048313140869140625f,-0.6657999880060554431793207186274230480194091796875f,-0.6917000109881161762359624844975769519805908203125f,-0.3865999762356284730202560240286402404308319091796875f,-0.5388999858558178868150889684329740703105926513671875f,0.50270002301335381389435497112572193145751953125f,0.387599991315603187302940568770281970500946044921875f,0.5737000088497996586056615342386066913604736328125f,-0.106500028133392987683691899292171001434326171875f,-0.5562000009894365604168342542834579944610595703125f,0.000700005966425276682230105507187545299530029296875f,0.11769999857544899413142047706060111522674560546875f,0.06199999573826797671927124611102044582366943359375f,0.17709999975711110398179926050943322479724884033203125f,-0.8125000134110447191915227449499070644378662109375f,-0.06839997786879603580700859311036765575408935546875f,-0.8263000172272330079437097083427943289279937744140625f,0.201799990755319402779832671512849628925323486328125f,-0.6461999924957748664411383288097567856311798095703125f,0.1840000163987280945576685553533025085926055908203125f,0.41780001131147148640820887521840631961822509765625f,0.21479999196231369662513088769628666341304779052734375f,0.31059999703019858241503925455617718398571014404296875f,0.64500002995133431937802015454508364200592041015625f,0.14990001095682392584507169885910116136074066162109375f,-0.6969999926090242325926737976260483264923095703125f,-0.8871999694287779902879265137016773223876953125f,-0.7638999830991028971993728191591799259185791015625f,0.3172000051170587031634795494028367102146148681640625f,0.0314000019863247548101981010404415428638458251953125f,-0.02580000164061779788227113385801203548908233642578125f,-0.38760001128315924034950512577779591083526611328125f,-0.1217000033140180903501459397375583648681640625f,-0.5340000214278699530723315547220408916473388671875f,0.01840000483989712165566743351519107818603515625f,-0.388499994069337883928483279305510222911834716796875f,0.0056000078707933997890222599380649626255035400390625f,-0.40419998424053193986082987976260483264923095703125f,-0.1230999735236171233054847107268869876861572265625f,-0.56109998732209209748589273658581078052520751953125f,0.03599997588992120967077426030300557613372802734375f,0.8040999686360361664583251695148646831512451171875f,0.43579998938441288913736570975743234157562255859375f,0.6183999972403046996305420179851353168487548828125f,0.7841999992907044969570051762275397777557373046875f,0.70060001249015346758142186445184051990509033203125f,0.27739999821633087861982858157716691493988037109375f,0.01540000011250375333560214130557142198085784912109375f,0.034099996817857081143898767550126649439334869384765625f,0.051000017911195794084733279305510222911834716796875f,-0.43729998235106481985212667495943605899810791015625f,-0.238099993118643826761626769439317286014556884765625f,-0.63559999468326555671637834166176617145538330078125f,-1.04399999445676794351811622618697583675384521484375f,-0.517399990504979978567234866204671561717987060546875f,-0.448799982321262280748896955628879368305206298828125f,-0.6070999758780004640357219614088535308837890625f,-0.453999986037612046629874384962022304534912109375f,-0.22659999143481268646382886799983680248260498046875f,-0.153899980044365225495539561961777508258819580078125f,-0.3284999841600655390294605240342207252979278564453125f,0.0199999864399433846529063885100185871124267578125f,0.6373999824583531648158896132372319698333740234375f,0.54299999539554111294137328513897955417633056640625f,0.30580000368952742206829498172737658023834228515625f,0.8415000080168244522837994736619293689727783203125f,0.44530000665485847122226914507336914539337158203125f,0.196799998056888469921688056274433620274066925048828125f,0.09900000719726109732476970748393796384334564208984375f,1.16100000047683682424803919275291264057159423828125f,0.184599979454279417012685371446423232555389404296875f,1.13239999843984850969036415335722267627716064453125f,-0.48299999573081742187952158928965218365192413330078125f,0.5039999795556069361879281132132746279239654541015625f,-0.435400016038119730410471674986183643341064453125f,1.37890005067586951525981930899433791637420654296875f,0.129800005447864297281057588406838476657867431640625f,1.172500036880374185699338340782560408115386962890625f,-0.6243000135540963668745462200604379177093505859375f,0.137799991381168496218379004858434200286865234375f,-0.4563000112295150234587026716326363384723663330078125f,1.450499991595744830874537001363933086395263671875f,0.139599993461370530667409184388816356658935546875f,1.23229998179227084875719810952432453632354736328125f,0.43590000901222225593301118351519107818603515625f,-0.7429999787807464173283733543939888477325439453125f,0.249100030922889903450823112507350742816925048828125f,-1.07140001252889671690127215697430074214935302734375f,-0.192000017344950979492068654508329927921295166015625f,-1.037400010770558989037226638174615800380706787109375f,-0.53199998569861062236441284767352044582366943359375f,0.37219997941255600348853249670355580747127532958984375f,-0.46700000156834686748652529786340892314910888671875f,-0.36250001281499866223612116300500929355621337890625f,0.512399987822770963674656741204671561717987060546875f,-0.24590002775043273697264112342963926494121551513671875f,0.41719997494220706357737071812152862548828125f,0.01080000473260867011049413122236728668212890625f,0.547199991109966976665646143374033272266387939453125f,0.63269997786283482810176792554557323455810546875f,0.08140000358819943215848979889415204524993896484375f,0.6414999807476993165522571871406398713588714599609375f,0.708499989300965804517318247235380113124847412109375f,0.1215999816000465916232542440411634743213653564453125f,0.6170999841108919525822784635238349437713623046875f,0.04179999805688883185439408407546579837799072265625f,-0.853599966609478233436902883113361895084381103515625f,-0.04419997395873076495576015076949261128902435302734375f,-0.0097000038504599928046445711515843868255615234375f,-0.35759999340176573667804404976777732372283935546875f,-0.1503000021472573610736844784696586430072784423828125f,-0.4431999878242611767831249380833469331264495849609375f,-0.868099979037046498575591613189317286014556884765625f,-0.22419999943971635758543925476260483264923095703125f,-0.420499994277954147747777824406512081623077392578125f,0.1405000129044056134119955459027551114559173583984375f,0.248999985963106240660636103712022304534912109375f,0.227699976223707523104167194105684757232666015625f,0.3492999817401170048469793982803821563720703125f,-0.325399996554851522745366310118697583675384521484375f,-0.06279998929500596460684391786344349384307861328125f,0.117500015124678558464665911742486059665679931640625f,0.5429999868273736041146548814140260219573974609375f,0.57079996565431390109779385966248810291290283203125f,0.223299983721971795347371880779974162578582763671875f,0.07899999237060562062850976872141472995281219482421875f,0.2638999806776645140615755735780112445354461669921875f,-0.1719000226020813426597300122011802159249782562255859375f,0.24079998178482053816651387023739516735076904296875f,1.01109997063279166695792810060083866119384765625f,0.40559999475777175348412129096686840057373046875f,-0.002200000423192960408158569407532922923564910888671875f,0.0035999984793365358282102306475280784070491790771484375f,-0.09090000394582749276839450658371788449585437774658203125f,-0.6407999981760976737632518052123486995697021484375f,-1.1757000059664246549573363154195249080657958984375f,-0.80120000076591946225335050257854163646697998046875f,-0.8897999950170518701497712754644453525543212890625f,-1.32659997236132642939310244400985538959503173828125f,-0.847499978467821879490884384722448885440826416015625f,-0.1038000036299226991332034231163561344146728515625f,0.18509998624920864784826335380785167217254638671875f,-0.209900012633204458012414761469699442386627197265625f,-0.11760000055432318077919262577779591083526611328125f,-0.4650999980211256712436806992627680301666259765625f,-0.43060001301169403831181625719182193279266357421875f,0.17720000325441365163214868516661226749420166015625f,1.02450000199675539391819256707094609737396240234375f,0.511400005599856211091491786646656692028045654296875f,0.434400001591443984949592049815692007541656494140625f,0.26049999651312827353422107989899814128875732421875f,0.373799999755620948604928344138897955417633056640625f,0.0484000015616417034181040435214526951313018798828125f,-0.03099999518692497790794959655613638460636138916015625f,0.324200003165006467309439130985992960631847381591796875f,0.4987999993085860950259302626363933086395263671875f,-0.406400013348460209527956976671703159809112548828125f,0.0086999828100198239866358562721870839595794677734375f,-0.442600008898972940851734847456100396811962127685546875f,0.020000005513429730541474782512523233890533447265625f,-0.07980003852844230749497000942938029766082763671875f,-0.4587999779999256677598395981476642191410064697265625f,-0.0232000058174133272359540569595992565155029296875f,-0.04150000872462999979006781359203159809112548828125f,0.4252000031203031493731714363093487918376922607421875f,-0.0587000014737248942964242814923636615276336669921875f,-0.75549997986108063940946522052399814128875732421875f,0.55779998665004970614944568296778015792369842529296875f,-0.39040000555142750915393889954430051147937774658203125f,0.3371999850749975990993334562517702579498291015625f,-0.313399973100424222849369471077807247638702392578125f,0.172500002980232380878078402020037174224853515625f,0.472299985480308492924450547434389591217041015625f,-1.30619999786019302945305753382854163646697998046875f,0.3150000068545342202241954510100185871124267578125f,-0.4684000149726863515553532124613411724567413330078125f,0.3652000002145765478189787245355546474456787109375f,-0.264200002866983307825421434245072305202484130859375f,-0.864700013685226309689824120141565799713134765625f,1.26920000525116893896893088822253048419952392578125f,-0.509100017213821587347410968504846096038818359375f,-0.13110002114772800752007242408581078052520751953125f,0.3088000094413756446698471336276270449161529541015625f,-0.0933000077128411930971196852624416351318359375f,-0.066500000957399552081739102504798211157321929931640625f,0.04519999946951865077426901962098781950771808624267578125f,-0.03650000013783571972680874750949442386627197265625f,-0.375000011622905304875530418939888477325439453125f,-0.20599998490512394511142701958306133747100830078125f,-0.1676000060677527159214150742627680301666259765625f,-1.1586999738693233297226470313034951686859130859375f,1.020799983572959757083253862219862639904022216796875f,-0.618500006437301408368512056767940521240234375f,-0.0296999978154897092963437899015843868255615234375f,-0.259999996498227137209369175252504646778106689453125f,0.006900000324845390053951632580719888210296630859375f,-0.9582000099897385325675713829696178436279296875f,1.226200004267692378334686509333550930023193359375f,-0.546300018978119084067657240666449069976806640625f,-0.76299998354911746645257153431884944438934326171875f,0.71190003869533580171946596237830817699432373046875f,-0.338600000658631217209659780564834363758563995361328125f,-0.20259999925047156299484640840091742575168609619140625f,-0.3803000064313408756788703612983226776123046875f,-0.320600003698468416502009858959354460239410400390625f,0.03379999763220542519093214650638401508331298828125f,0.4331000116705894242841168306767940521240234375f,-0.076599996277689985646475179237313568592071533203125f,0.068499998465180400586405085050500929355621337890625f,0.298799994540214441940406686626374721527099609375f,-0.111599994564056448353994710487313568592071533203125f,-0.1931999999314546467843456412083469331264495849609375f,-0.5638000046730038405939922085963189601898193359375f,-0.0565999783217907281862579793596523813903331756591796875f,0.102200002583861360161421316661289893090724945068359375f,0.10270001071989565133435462485067546367645263671875f,-0.8995999974846835556263613398186862468719482421875f,-0.327399993783235476740856029209680855274200439453125f,-1.149200011062621484825285733677446842193603515625f,-0.12659999642670138886302311220788396894931793212890625f,-0.1544999977499246224255102788447402417659759521484375f,-0.065599993789195565341287874616682529449462890625f,1.274600017675757701596239712671376764774322509765625f,0.379000008724629910972225843579508364200592041015625f,1.377500060498714873347125831060111522674560546875f,-0.211999989926815235463664066628552973270416259765625f,0.0952999973788857257517292964621447026729583740234375f,0.5207000007510185213277509319595992565155029296875f,0.813800026652216956790653057396411895751953125f,0.38680000420361759783105526366853155195713043212890625f,1.02090002927780165720150762354023754596710205078125f,0.403999999076127824793047693674452602863311767578125f,0.125799997915327532727047810112708248198032379150390625f,0.02139999706894124553713254499598406255245208740234375f,0.5494000050932168477402228745631873607635498046875f,0.28160000167936083403930069835041649639606475830078125f,0.93710003116130824452056913287378847599029541015625f,-0.4944000069558620591436692848219536244869232177734375f,-0.0918000018566846875955889117904007434844970703125f,-0.572600023651123279933017329312860965728759765625f,0.24219999736845476689950373838655650615692138671875f,0.26149999557435510411096402094699442386627197265625f,0.73839998934268891161991632543504238128662109375f,0.5450000067800282455010574267362244427204132080078125f,0.04339999694228158233499925700016319751739501953125f,-0.2439000083565712184707763299229554831981658935546875f,0.074999992027878936706741797024733386933803558349609375f,-0.134599997037649021791594350361265242099761962890625f,-0.8989000119328505178373234230093657970428466796875f,0.634199988636374722972277595545165240764617919921875f,-0.4704000186085703916205602581612765789031982421875f,-0.3813999845147133527234473149292171001434326171875f,0.310799989905953477631328496499918401241302490234375f,-0.198800000500678919479469186626374721527099609375f,0.2896000019550317716721110627986490726470947265625f,-0.293800020766257663495935048558749258518218994140625f,0.1459000152707101705118475365452468395233154296875f,-0.21989995846748400243342302928795106709003448486328125f,0.50639995791614100806299347823369316756725311279296875f,-0.1300999894529579758994941585115157067775726318359375f,-0.64139999256133961580417235381901264190673828125f,1.1605999830603597189337961026467382907867431640625f,-0.39830000026226031195619725622236728668212890625f,-0.111400014317035722655191420926712453365325927734375f,-0.34320000648796533671003317067516036331653594970703125f,-0.04299999547004684785633799037896096706390380859375f,1.1258999818921093805101918405853211879730224609375f,-1.879799974229932058022995988721959292888641357421875f,0.678200005221366808427774230949580669403076171875f,-0.794499975472689357047784142196178436279296875f,0.612399967259169120126216512289829552173614501953125f,-0.414900001606345103510875560459680855274200439453125f,0.553700038242340042415889911353588104248046875f,-0.65700001622736436956984107382595539093017578125f,0.3298000149846078699056306504644453525543212890625f,0.65200000888854248159987037070095539093017578125f,-0.366599992626905457626662609982304275035858154296875f,0.3447000058993696658404815025278367102146148681640625f,0.18709999301433555274343234486877918243408203125f,-0.4793999897450209601856840890832245349884033203125f,0.1355999920010564352423898526467382907867431640625f,-0.43359998627901052969946249504573643207550048828125f,0.9817999964177606120330210615065880119800567626953125f,-0.27310000496506692257980830618180334568023681640625f,-0.211199981319904139809295884333550930023193359375f,0.20940001015961140495846848352812230587005615234375f,-0.098900006270408624686751863919198513031005859375f,0.61100001254677760442746148328296840190887451171875f,-0.45170001612901700127622461877763271331787109375f,0.872599992582201888779991350020281970500946044921875f,0.2562999993830918921133843468851409852504730224609375f,0.8744000214099880707863121642731130123138427734375f,0.1744000215590002511589773348532617092132568359375f,0.03440000919103625154349401782383210957050323486328125f,-0.35399996763467722615814636810682713985443115234375f,-0.4066000180333855240633056382648646831512451171875f,-0.1097000049680472155699817449203692376613616943359375f,0.2948000039577483022412707214243710041046142578125f,0.43780001172125349029329299810342490673065185546875f,0.156500003173947355161743644202942959964275360107421875f,0.485600018674135380791767602204345166683197021484375f,-0.57380001085698584262217991636134684085845947265625f,-0.02590000141263004707070649601519107818603515625f,0.2553999801635740141136921010911464691162109375f,0.59220002955496353624909033896983601152896881103515625f,0.18390001044273385222282968243234790861606597900390625f,0.36500003397464819698825522209517657756805419921875f,0.07720000422298911590957004591473378241062164306640625f,0.0929000026643276022308981509922887198626995086669921875f,0.5481000281363728188210870939656160771846771240234375f,0.263600036346912869333891649148426949977874755859375f,0.23650001103430995375020984283764846622943878173828125f,0.2797000287353998970729662687517702579498291015625f,-0.032400001233816144718957730219699442386627197265625f,0.0302999998375773349490458485888666473329067230224609375f,0.4531000099569559846912625289405696094036102294921875f,0.83700003321468852135467386688105762004852294921875f,0.175800007824599813233135137124918401241302490234375f,0.604700037229061848620403907261788845062255859375f,-0.7074000147193670873235760154784657061100006103515625f,-0.209700004856288424459620500783785246312618255615234375f,-0.16680002484172595789146953393355943262577056884765625f,0.383800014433264724544869750388897955417633056640625f,0.25100000289827584243340652392362244427204132080078125f,0.73010002722740185987504446529783308506011962890625f,-0.24799999554455276484787873414461500942707061767578125f,0.080900000667572025125906520770513452589511871337890625f,0.100699997621774617329037937452085316181182861328125f,0.26560000630617164318891809671185910701751708984375f,0.24560000511407853451828486868180334568023681640625f,0.56160002317428592277792631648480892181396484375f,0.705200036796927776094889850355684757232666015625f,0.22520000960230834863295967807061970233917236328125f,0.8213999708056451964921507169492542743682861328125f,-1.266399958363175759501473294221796095371246337890625f,0.487399993634223704930263920687139034271240234375f,0.5506000188231467973309918306767940521240234375f,-0.0630000239312646925782246398739516735076904296875f,0.272300012302398641850231797434389591217041015625f,-0.89040004423856800030989688821136951446533203125f,0.2842000081568958336220021010376513004302978515625f,-0.454300024580955863484632573090493679046630859375f,0.366200008291006184890648000873625278472900390625f,-0.54040000154674050492076275986619293689727783203125f,0.21840000677704818343727311003021895885467529296875f,-1.198200001943110493130006943829357624053955078125f,1.02620001387894088651364654651843011379241943359375f,-0.6419000193238257878647345933131873607635498046875f,0.716800028753281193161228657118044793605804443359375f,-0.70440001448094846647762778957257978618144989013671875f,0.39940002021789577923982506035827100276947021484375f,-0.29570001397579925850322979385964572429656982421875f,-0.04740000201612704078257820583530701696872711181640625f,-0.13480001057013890619629137290758080780506134033203125f,0.1669000583648687641158403494046069681644439697265625f,0.2831999648392200530366835664608515799045562744140625f,0.078300017696619317320028130779974162578582763671875f,-0.3961000133246184162771896808408200740814208984375f,-0.024399997717142074549201424815692007541656494140625f,-0.188500008746981695395561473560519516468048095703125f,0.986500024259090224632018362171947956085205078125f,-1.39560002470910493599376422935165464878082275390625f,0.57010002961158789958062698133289813995361328125f,-1.05360001589506868668877359596081078052520751953125f,0.7506000001221895256975358279305510222911834716796875f,-0.5667000134840607383779342853813432157039642333984375f,0.0527000151157377416666349745355546474456787109375f,-0.142400010994076620107762209954671561717987060546875f,0.0489000000119208522164626629091799259185791015625f,-0.551499978102743426688903127796947956085205078125f,0.83819998834580167201835365631268359720706939697265625f,-0.31140000580474735958347309860982932150363922119140625f,-0.035599992895125609493334195576608180999755859375f,0.17800001245737018251702465931884944438934326171875f,-0.01300000977516191369431908242404460906982421875f,0.62380005265474369746669935921090655028820037841796875f,-1.04890001461505821822584039182402193546295166015625f,-1.082400036996603365224700610269792377948760986328125f,-0.2172000095129014152206536891753785312175750732421875f,-0.2820000265836721808909715036861598491668701171875f,-0.3991999980986111751235512201674282550811767578125f,-0.25569999095350492979150658356957137584686279296875f,-0.56949999409913942827188293449580669403076171875f,-0.0593999991327522902651026015519164502620697021484375f,0.1044000017032027383123704566969536244869232177734375f,0.0206999971747396926957662799395620822906494140625f,-0.335200017797947058539875797578133642673492431640625f,-0.161000004276633301714127810555510222911834716796875f,0.246999999165534944722821819595992565155029296875f,-0.65400002887845065657757004373706877231597900390625f,-0.111300008808076444921653092023916542530059814453125f,-0.162500034272671456392345135100185871124267578125f,0.73900001434981821279990299444762058556079864501953125f,0.31790000288784503457151231486932374536991119384765625f,0.52040002069473256796072746510617434978485107421875f,-0.576200027140975290507185491151176393032073974609375f,-0.304300009232759516297761592795723117887973785400390625f,-1.005800050181151039652149847825057804584503173828125f,0.6068000192165377004727133680717088282108306884765625f,0.2215000064149499348076233218307606875896453857421875f,0.7091000330090526659887473215349018573760986328125f,0.12060001427829281084314061445184051990509033203125f,0.0969000032305717962088920103269629180431365966796875f,-0.3590000096559524678241359652020037174224853515625f,0.23299999152123940238112709266715683043003082275390625f,-0.0238999993413686606658075106679461896419525146484375f,-0.0232000001549721446281182579696178436279296875f,-0.199400013288855593174275782075710594654083251953125f,-0.1879000031113624569201192571199499070644378662109375f,-0.5906000232934953686481094337068498134613037109375f,0.1309999945908781882053517620079219341278076171875f,0.003699993240833254759536430356092751026153564453125f,0.515799988937377662523431354202330112457275390625f,0.3203999981939791297236297396011650562286376953125f,-0.0174000027239322729855075522209517657756805419921875f,0.2752000002145766899275258765555918216705322265625f,0.163000008359551085135308312601409852504730224609375f,0.1219999961480497940868872319697402417659759521484375f,-0.2813000162959102823378998436965048313140869140625f,1.0774000163957477838039267226122319698333740234375f,-0.0793999787926669142734681372530758380889892578125f,-1.6386000443935397896666472661308944225311279296875f,1.600600017482042058958313646144233644008636474609375f,-0.90030004234314020550300483591854572296142578125f,-0.19330001814365260770500754006206989288330078125f,0.0770000109374517904825552250258624553680419921875f,-0.113400000071525397515870281495153903961181640625f,-0.16709998377561507965083364979363977909088134765625f,0.5624000054806466142309773204033263027667999267578125f,-0.107700005280971566890002577565610408782958984375f,-0.7058000473499301818947060382924973964691162109375f,-0.066399985706806052832007480901665985584259033203125f,-0.34740001866817493691996787674725055694580078125f,-1.004500021815299959371259319595992565155029296875f,1.04760002111792527301759037072770297527313232421875f,-0.554500025987625377865697373636066913604736328125f,0.809000025153160162716403647209517657756805419921875f,-0.53679999962151059289539034580229781568050384521484375f,0.44220000951290128199389073415659368038177490234375f,-0.1758000165045263685215104487724602222442626953125f,0.476200013878941508238540336606092751026153564453125f,-0.13130000422596932452279361314140260219573974609375f,0.47970001070499446171879753819666802883148193359375f,-0.7474000031709671088009372397209517657756805419921875f,0.2894000088930130942799223703332245349884033203125f,0.430400047218800096970880986191332340240478515625f,0.32239998610913733756433430244214832782745361328125f,0.1952000166058542873770420555956661701202392578125f,0.443999977916479604544974790769629180431365966796875f,-0.60219997188746965388617127246106974780559539794921875f,0.248899998745322204030117063666693866252899169921875f,0.1493999859452248557545317453332245349884033203125f,-0.064999986812472432262666188762523233890533447265625f,0.064900001382827809948139474727213382720947265625f,-0.137000010192394938712823204696178436279296875f,-0.5274000004142518349681267864070832729339599609375f,-0.0350999983191488951206338242627680301666259765625f,0.38359998478889423267901293002068996429443359375f,-0.9143999944388863099220543517731130123138427734375f,0.238400007224083054779839585535228252410888671875f,0.4395000453591340061620940105058252811431884765625f,0.296999974802136801343976912903599441051483154296875f,0.20260001530647286926978267729282379150390625f,1.27310000377297427576195332221686840057373046875f,-0.33960000080019181467605449142865836620330810546875f,-0.734799990099668232090834862901829183101654052734375f,-0.0928999978587030650345468529849313199520111083984375f,-0.0613000145077702285334453335963189601898193359375f,-0.18860001824200178788260018336586654186248779296875f,-0.066100007289648099462198160836123861372470855712890625f,0.157799979907273790757926690275780856609344482421875f,-1.132000015556811955974580996553413569927215576171875f,-0.2477000012949108764814809546805918216705322265625f,-0.5811000324726105503714279620908200740814208984375f,-0.4046000166326759739376939251087605953216552734375f,-0.3031000013142823856782115399255417287349700927734375f,-0.7532000248908996553609540569595992565155029296875f,-0.7244000084459780719470245458069257438182830810546875f,-0.326200000989437077780763729606405831873416900634765625f,-0.824000023216008958826250818674452602863311767578125f,-0.4967999973118304435359959825291298329830169677734375f,-0.1343999992072582028157512468169443309307098388671875f,-0.5432000129699705581742819049395620822906494140625f,0.70700001026690006966646251385100185871124267578125f,0.33400000008195640521790892307762987911701202392578125f,1.00750000894069646761863623396493494510650634765625f,-0.578200008872151283156881618197076022624969482421875f,-0.295100003609061189990825369022786617279052734375f,-0.18080002887248980414369725622236728668212890625f,0.745400025090575457653585544903762638568878173828125f,0.197800008107721858596761421722476370632648468017578125f,0.029700025755167391849909108714200556278228759765625f,0.30440000144243228863416561580379493534564971923828125f,0.0156999987766146542611522818333469331264495849609375f,0.193300005626678483849900658242404460906982421875f,0.00320000715851787020937990746460855007171630859375f,0.1144000005483627457891770973219536244869232177734375f,0.2643999993562697881088752183131873607635498046875f,0.4254000045269725749363942668423987925052642822265625f,0.04439999760538337891802029844257049262523651123046875f,0.63390001282691965656113097793422639369964599609375f,-0.47839999545216560505878078402020037174224853515625f,-0.0457000001922249765584638225845992565155029296875f,0.1176999993205072314594872295856475830078125f,0.114600001135468421153973395121283829212188720703125f,0.009000001870095697409368540320429019629955291748046875f,-0.362499988228082603569646380492486059665679931640625f,0.372800012540817515827029637875966727733612060546875f,-0.24799999528378224855629241574206389486789703369140625f,-1.2296999695777888206293937400914728641510009765625f,1.253199971395730916157162937452085316181182861328125f,-0.674500001549720451521352515555918216705322265625f,-0.421400022512674610197791480459272861480712890625f,0.05700001548230648751314220135100185871124267578125f,-0.21130001167654999250089531415142118930816650390625f,-1.499500002741812831885681589483283460140228271484375f,1.728400002455710904314400977455079555511474609375f,-0.84230002094507216980900921043939888477325439453125f,-0.05040002695321987857823842205107212066650390625f,-0.0209999796897175627918841200880706310272216796875f,-0.041300001692771814987281686626374721527099609375f,-0.56160000931620590591819563996978104114532470703125f,0.6195999922692776440413808813900686800479888916015625f,-0.32780000136494624740635117632336914539337158203125f,-0.44019998352527611729101408855058252811431884765625f,0.7921999813497067055578781946678645908832550048828125f,-0.26769999812841405173458042554557323455810546875f,0.395500021100044563127084984444081783294677734375f,-0.63899999110400695911948787397705018520355224609375f,0.2490000063776971472861987422220408916473388671875f,-0.7748000155806540334424425964243710041046142578125f,0.111799997982382937067313832812942564487457275390625f,-0.39929999954700434727783431299030780792236328125f,1.2251000327408316792343612178228795528411865234375f,-0.816800010350346550325184580287896096706390380859375f,0.65290002442002315063973583164624869823455810546875f,0.399099991881846971608638341422192752361297607421875f,-0.69859999939203232255380271453759633004665374755859375f,0.235200006622076074336291640065610408782958984375f,-0.2341999779820440608091303147375583648681640625f,0.34659999083876602554710188996978104114532470703125f,-0.12689999920725814064326186780817806720733642578125f,0.287099985936283275123059866018593311309814453125f,-1.0610000050961965190055025232140906155109405517578125f,0.2040000076815486895753792850882746279239654541015625f,-0.920099990272522205714267329312860965728759765625f,0.80059997665286086743208215921185910701751708984375f,-0.4952000064730643913435415015555918216705322265625f,0.4608999984323975951383545179851353168487548828125f,-0.034800001797079982424065747181884944438934326171875f,0.22570000104904153204188332892954349517822265625f,0.8742000170230870281784518738277256488800048828125f,0.7961000065445897888594117830507457256317138671875f,0.51500001437962072969867222127504646778106689453125f,0.095500004336237953594945793156512081623077392578125f,0.389600019240379680240948800928890705108642578125f,0.084200003835559034115476606530137360095977783203125f,0.12310000272989274350265986868180334568023681640625f,0.4082000129699707713371026329696178436279296875f,-1.2130000149160622413546661846339702606201171875f,-0.356900002858042686426642831065692007541656494140625f,-1.294400050616264508107633446343243122100830078125f,-0.285000000521540730602509938762523233890533447265625f,0.01139999811202285773248377154232002794742584228515625f,0.43890000329017642144435740192420780658721923828125f,0.823200019675493610549210643512196838855743408203125f,0.298500007219612673026176707935519516468048095703125f,1.124300033819675803670179448090493679046630859375f,-0.794200004431605321997267310507595539093017578125f,-0.2120000006183981877683208949747495353221893310546875f,-0.9879000245690345849425284541212022304534912109375f,-1.0151999845683572498700186770292930305004119873046875f,-0.24149999605864280738387606106698513031005859375f,-0.797300011408328401785183814354240894317626953125f,-0.91900000034272666393775352844386361539363861083984375f,-0.26690000054836271470293240781757049262523651123046875f,-0.71500001966953252718894873396493494510650634765625f,0.994800021243095411449530729441903531551361083984375f,0.224100004659593121658645031857304275035858154296875f,0.310100028121471638087314204312860965728759765625f,-0.5767999982059002217482657215441577136516571044921875f,-0.16600000107288359618706863329862244427204132080078125f,-0.67880002046823495476246534963138401508331298828125f,-1.099199980962275891016588502679951488971710205078125f,-0.3110999886259435687207997034420259296894073486328125f,-1.2668999942898739874408420291729271411895751953125f,-0.603599986875056782054116411018185317516326904296875f,-0.134699993456900035670997795023140497505664825439453125f,0.014500009149313708434192449203692376613616943359375f,0.064200011733174022765524568967521190643310546875f,0.122399997396767101776049457839690148830413818359375f,0.221300001394748591110328561626374721527099609375f,-0.5598000147610904253525632157106883823871612548828125f,-0.186000003717839756944840701180510222911834716796875f,-0.58990001837015171304301475174725055694580078125f,0.61020000803768592589904073975048959255218505859375f,-0.017300002169609030033825547434389591217041015625f,0.600000005066394948016750277020037174224853515625f,-0.92100000189244735082638726453296840190887451171875f,0.3503000140309335819210900808684527873992919921875f,-0.1961999977111812398788970313034951686859130859375f,0.134200005027651503297647650470025837421417236328125f,-0.09550000664591795640490090590901672840118408203125f,-1.0849999865889543571029207669198513031005859375f,1.79679999112784827275390853174030780792236328125f,-0.652200007724761832150761620141565799713134765625f,-0.8295000039339068820254396996460855007171630859375f,0.458199988904595567618116547237150371074676513671875f,-0.4278000110507012010430116788484156131744384765625f,0.5353000083684928966931693139486014842987060546875f,-1.102599988931417751558683448820374906063079833984375f,0.34180000908374807977452292107045650482177734375f,-0.6192999831557273182625067420303821563720703125f,1.337599981704354235034770681522786617279052734375f,-0.391100003266334539375748136080801486968994140625f,-1.13829996211528783334188119624741375446319580078125f,1.6045999466717246750846470604301430284976959228515625f,-0.6597999950170514438241298194043338298797607421875f,-1.013999988794326956309532761224545538425445556640625f,1.24479997035563005880476339370943605899810791015625f,-0.58000000298023213218812088598497211933135986328125f,1.4587000191688535011280691833235323429107666015625f,-1.392600006365775744399115865235216915607452392578125f,0.801400025343895094920299015939235687255859375f,-0.47219997881650943583053958718664944171905517578125f,0.9013999748289587277128021014505065977573394775390625f,-0.2914999959468840273757450631819665431976318359375f,-0.9212999695062631388964291545562446117401123046875f,1.726799952757358536103993174037896096706390380859375f,-0.566099996709823471974232234060764312744140625f,-1.0308999915778631617513383389450609683990478515625f,0.750199976074695573657891145558096468448638916015625f,-0.55120000262856461148430753382854163646697998046875f,-0.097699969220162330429957364685833454132080078125f,0.2675999676972633523064359906129539012908935546875f,-0.05569999255537982918440320645458996295928955078125f,-0.49670001314878486908810373279266059398651123046875f,0.7354000006526708599352559758699499070644378662109375f,-0.2948000111103059595052400254644453525543212890625f,0.485800025594233630954477121122181415557861328125f,-0.19430000193119045803769040503539144992828369140625f,0.2982000114053489081555881057283841073513031005859375f,0.1728000044569373283565028032171539962291717529296875f,0.0503000134348869920586366788484156131744384765625f,0.54380000415146323433646102785132825374603271484375f,0.162099998676776824169110113871283829212188720703125f,0.891800032329559400068319519050419330596923828125f,1.04980000403225393057482506264932453632354736328125f,0.250099998095631548267192556522786617279052734375f,0.9092000271558760005063959397375583648681640625f,0.41959999599456754282300607883371412754058837890625f,0.3582999943390487995742432758561335504055023193359375f,0.9010999964117996086088169249705970287322998046875f,-0.68659998688995838023174655972979962825775146484375f,-0.171399999080598330891689329291693866252899169921875f,-0.677900013244151722346941824071109294891357421875f,-0.55920000271797132285200859769247472286224365234375f,-0.267599996829032793499436593265272676944732666015625f,-0.116800012063979608001318410970270633697509765625f,0.611400017818808816372211367706768214702606201171875f,0.09780000713914638321710981472278945147991180419921875f,-0.12329998059272728738733349018730223178863525390625f,0.3922000177830458067518293319153599441051483154296875f,0.0329000042289495642222618698724545538425445556640625f,0.0284000227212910516527699655853211879730224609375f,-0.139000000342726526270098474924452602863311767578125f,-0.220399998007714703618376006488688290119171142578125f,-0.453500003755092251367386779747903347015380859375f,-0.48680000595450412248510474455542862415313720703125f,-0.0889000025078654498855712517979554831981658935546875f,-0.6613000228524210655223214416764676570892333984375f,-0.3009999983161686376575971735292114317417144775390625f,-0.1522999977737664878407741753107984550297260284423828125f,-0.5530000058263537798808329171151854097843170166015625f,-0.11319997676014903476016115746460855007171630859375f,-0.0082999946370720945054699768661521375179290771484375f,-0.5894999803900713430948599125258624553680419921875f,0.5982000288397075760116194942384026944637298583984375f,0.3666000072672963572273374666110612452030181884765625f,0.5181000253796579357867813087068498134613037109375f,0.60700001607835307737559560337103903293609619140625f,0.12830000331699853433065072749741375446319580078125f,0.61440001723766357599743059836328029632568359375f,0.405800002571940154894036822952330112457275390625f,0.01360000103116032477146291057579219341278076171875f,0.42130002344846761985763805569149553775787353515625f,0.0507999972075221517542331639560870826244354248046875f,0.2113000052690505725649927626363933086395263671875f,0.262199978220461815681119333021342754364013671875f,-0.95060000205933992134532672935165464878082275390625f,0.1924999973177907719446011469699442386627197265625f,1.116199983108042914636826026253402233123779296875f,-1.69959998608529527786004109657369554042816162109375f,0.65520000855922688742793980054557323455810546875f,-0.053099982762336850328210857696831226348876953125f,0.14860001220703100699438437004573643207550048828125f,-0.01300000977516191369431908242404460906982421875f,-0.666899963295460196377462125383317470550537109375f,1.11959995151460223183903508470393717288970947265625f,-0.3988999904751775460454155108891427516937255859375f,-0.798800017786025051691467524506151676177978515625f,0.15480000634193391562121178139932453632354736328125f,-0.41180000252723658604736556299030780792236328125f,1.1391000178098682482641379465349018573760986328125f,-0.869800001648068477066999548696912825107574462890625f,0.6084000212311746480509100365452468395233154296875f,0.66680000997781707638978332397527992725372314453125f,-0.7374000202327961606130202198983170092105865478515625f,0.368100017631054043931726482696831226348876953125f,0.1734999852776528683762080618180334568023681640625f,-0.403199999782442919382674517692066729068756103515625f,0.0945000084042550980711894226260483264923095703125f,-0.357699982929229776118518202565610408782958984375f,1.02119999011158935076082343584857881069183349609375f,-0.23850000345706945381607511080801486968994140625f,-0.07499999664723855374859340372495353221893310546875f,0.2905999917775392571428483279305510222911834716796875f,-0.062499995902180582874052561237476766109466552734375f,0.223700037050246436365341651253402233123779296875f,0.50519995059371058943042953615076839923858642578125f,0.0781000149488448869306012056767940521240234375f,0.53730005432367367035340066649951040744781494140625f,-0.0198000193804503954453366532106883823871612548828125f,0.2832000147581101856530949589796364307403564453125f,0.589000008165836419493643916212022304534912109375f,-1.09560000310242156018603054690174758434295654296875f,0.3572000166654589747849968262016773223876953125f,0.1771999883532515696060727350413799285888671875f,0.70860001697540297982413903810083866119384765625f,-0.29240000063776960814720951020717620849609375f,-0.186999999165534946943267868846305646002292633056640625f,-0.671200012463331052714465840836055576801300048828125f,-0.3631999961316585423531932974583469331264495849609375f,-0.05769999958127736139346097843372263014316558837890625f,-0.61390001521110537652248240192420780658721923828125f,-0.464000015169382162838473959709517657756805419921875f,-0.1894000049442052802106672970694489777088165283203125f,-0.7710000410079960175835367408581078052520751953125f,0.17159999545812609778039359298418276011943817138671875f,0.01979999985992908495546771519002504646778106689453125f,0.17560000302791589632533941767178475856781005859375f,-0.709199987518787100526651556720025837421417236328125f,-0.14609999518245453398179733994766138494014739990234375f,0.10030001254081799544337627594359219074249267578125f,-0.77040003932118461538181009018444456160068511962890625f,-0.2319000139035285457111967843957245349884033203125f,-0.138100047079474597122583645614213310182094573974609375f,0.1893999979406595368658372535719536244869232177734375f,-0.0396999991565942789151222314103506505489349365234375f,-0.1635999949216841997667870600707828998565673828125f,0.047400011739135106125786478514783084392547607421875f,0.1290000070855021885307678530807606875896453857421875f,-0.1940999883055685160115899634547531604766845703125f,0.458399998730420865644674677241710014641284942626953125f,0.1454999981895088723149456200189888477325439453125f,0.26490000749230357968144744518212974071502685546875f,0.7724000426590447165864361522835679352283477783203125f,0.334600011715293021996586730892886407673358917236328125f,0.722400043702126293254650590824894607067108154296875f,-1.0333999812215564073625273522338829934597015380859375f,-0.222399996130168364771861888584680855274200439453125f,-0.41390000835657048128268797881901264190673828125f,0.80500004328787344576312534627504646778106689453125f,0.4601000074833632158544105550390668213367462158203125f,1.3120000391006474416144556016661226749420166015625f,-0.61539999856650862142259938991628587245941162109375f,-0.03410000372827060033387169823981821537017822265625f,0.20600000464916234932388761080801486968994140625f,0.89460002274215233430965099614695645868778228759765625f,0.33900000600516799753592067645513452589511871337890625f,0.84430003728419567021745706369983963668346405029296875f,-0.252999992713332400029457858181558549404144287109375f,-0.31940000367760656985183231881819665431976318359375f,0.04040000236630447716379421763122081756591796875f,0.1831999928534031596427666954696178436279296875f,-0.00239999318718897569624459720216691493988037109375f,-0.1842999714583157810920965857803821563720703125f,0.8579999815374612648355423516477458178997039794921875f,-0.1430999956145881935043462362955324351787567138671875f,-0.1939999959468838852671979111619293689727783203125f,0.599800007981061700235159150906838476657867431640625f,-0.1405000002384184654147247783839702606201171875f,0.172399985811114486722317451494745910167694091796875f,-0.38039998455941692601101067339186556637287139892578125f,0.1069999974146484955639380132197402417659759521484375f,-1.2762999954342841846255396376363933086395263671875f,0.891799966466427118660931228077970445156097412109375f,-0.6790000063776968630691044381819665431976318359375f,-1.177100042057782847138014403753913938999176025390625f,0.78100002536177626222269054778735153377056121826171875f,-0.62960002808794401918390803984948433935642242431640625f,0.47359998702406880255466603557579219341278076171875f,-0.5037999910384416413222652408876456320285797119140625f,0.256500003620982131025130001944489777088165283203125f,0.1671000278830536700525044579990208148956298828125f,0.537799997565149734413125770515762269496917724609375f,0.057000001549720735738446819595992565155029296875f,0.5660999976038929570876234720344655215740203857421875f,-0.5231999909162521333882978069595992565155029296875f,0.313500001519918303216627464280463755130767822265625f,0.7166000531256202510377306680311448872089385986328125f,-0.63940003213882423693803502828814089298248291015625f,0.404500025987625466683539343648590147495269775390625f,-1.45409996267557151128357872948981821537017822265625f,1.5359999421387915408132585071143694221973419189453125f,-0.8074999958276745104512883699499070644378662109375f,0.297000063240528078267743694595992565155029296875f,-0.47120003995597325996413928805850446224212646484375f,0.199000022172928225927535095252096652984619140625f,-1.233599995815755079320297227241098880767822265625f,1.10739996301233833975175002706237137317657470703125f,-0.66430001444816610955967917107045650482177734375f,0.845900020672381014463780957157723605632781982421875f,-0.94980000403225417482389048018376342952251434326171875f,0.4856000129744411086818445255630649626255035400390625f,-0.6924999740719801621935403090901672840118408203125f,0.57960003622770361886296086595393717288970947265625f,-0.15899999773502349054155047269887290894985198974609375f,-0.091799997833371171918059872041339986026287078857421875f,-0.682899998492002335837014470598660409450531005859375f,0.6342000322967766834381109219975769519805908203125f,0.414600005568563911850787917501293122768402099609375f,1.242500055134296843561969581060111522674560546875f,-0.263200001794099858187792051467113196849822998046875f,0.021099997566640382729019620455801486968994140625f,-0.48390002981424373729169019497931003570556640625f,1.17700000549852834552666536183096468448638916015625f,0.353599998535215842565548882703296840190887451171875f,1.370500023186206561831568251363933086395263671875f,-0.96820003658831144122132172924466431140899658203125f,-0.279800007347762669240864852326922118663787841796875f,-1.210300040853024228226786362938582897186279296875f,-0.1454000028878450478941886103712022304534912109375f,-0.1647999981090426668828285983181558549404144287109375f,-0.5334999888539311285740041057579219341278076171875f,-0.506600005367398242839271915727294981479644775390625f,-0.03130000087320805590707095689140260219573974609375f,0.0067999935865401539558661170303821563720703125f,0.498800009590387227742525055873556993901729583740234375f,0.02830000037401914081414133761427365243434906005859375f,0.217900015479326469858278869651257991790771484375f,-0.74740002187192455807007718249224126338958740234375f,-0.38210000471174720981792916063568554818630218505859375f,-0.91680002249479297660172960604541003704071044921875f,0.15239999989271157954107138721155934035778045654296875f,-0.01590000077933073630020999189582653343677520751953125f,0.13330000413656239999227182124741375446319580078125f,-0.306600012519955722467557279742322862148284912109375f,-0.060300002110004469191295584096224047243595123291015625f,0.349400009042024617400556962820701301097869873046875f,-0.278199999782442919382674517692066729068756103515625f,-0.176899996674060810963879930568509735167026519775390625f,-0.779799998444318465118385574896819889545440673828125f,-0.658999998107552276138676461414434015750885009765625f,-0.259599997596442644276493183497223071753978729248046875f,-0.347500005811452350457102511427365243434906005859375f,-0.608200020048022427232581321732141077518463134765625f,-0.430100003758072890747143901535309851169586181640625f,-1.163200019526481554521524230949580669403076171875f,-0.7850000327080490780673471817863173782825469970703125f,0.001700002121925336950880591757595539093017578125f,0.24550000631064161638050791225396096706390380859375f,0.29859998929649600452052027321769855916500091552734375f,0.09580000408068289896590385978925041854381561279296875f,0.06130003179311671601681155152618885040283203125f,-0.2538000379770988246264096233062446117401123046875f,0.069400004720687746839757892303168773651123046875f,-0.1332999644994732335590015281923115253448486328125f,0.907399994081258487454988426179625093936920166015625f,-0.116599996948242079497504164464771747589111328125f,0.967499990165233469951999722979962825775146484375f,-1.76919997522532934652872427250258624553680419921875f,0.59350000762939458809341886080801486968994140625f,-0.741300027024745844528297311626374721527099609375f,1.54320002362430042097685145563445985317230224609375f,-0.466300027918816084593345294706523418426513671875f,0.19489997843503914509710739366710186004638671875f,-0.1142000030905006013881575199775397777557373046875f,0.091199998903274348549530259333550930023193359375f,-0.8973999992966650296466468716971576213836669921875f,0.978399996122717663382672981242649257183074951171875f,-0.4938000151038171026129930396564304828643798828125f,0.7236999962210648451588212992646731436252593994140625f,-1.0850000092387193095788688879110850393772125244140625f,0.415800015461444960607195753254927694797515869140625f,-0.511600036883354647443411522544920444488525390625f,0.47940000360310097704541476559825241565704345703125f,-0.2987000111222268827759762643836438655853271484375f,0.184099990019201964486228462192229926586151123046875f,-0.46839999880492666495257481074077077209949493408203125f,0.1157000030234456300348711010883562266826629638671875f,-0.7724000361025336047759992652572691440582275390625f,0.246400009474158299127566351671703159809112548828125f,-0.39200002166628866717701384914107620716094970703125f,0.1391999957144260946506619802676141262054443359375f,0.248399993589520473591392146772705018520355224609375f,0.0422000026583670972968320711515843868255615234375f,-0.8155000066459177077149433898739516735076904296875f,0.522199983808398382478799248929135501384735107421875f,-0.44100000151991824992592228227294981479644775390625f,-0.0648000400185591018953346065245568752288818359375f,0.091800002899766308672724335337989032268524169921875f,-0.062500003576278828631984652020037174224853515625f,-0.75050002467632292990629139239899814128875732421875f,-0.7132000070095063648523137089796364307403564453125f,0.722800007474422301356753450818359851837158203125f,0.2098999984398484119907379863434471189975738525390625f,0.9611000166773795427843651850707828998565673828125f,0.805200015711784455874067134573124349117279052734375f,0.2247000047817826473561808597878552973270416259765625f,0.2291000216841698744474342674948275089263916015625f,0.84540003373324890123541308639687485992908477783203125f,0.1368000080034137422302364939241670072078704833984375f,0.393500037431717686331467120908200740814208984375f,0.675199992018937766857789029018022119998931884765625f,0.1488999970689415253133347505354322493076324462890625f,-0.1240999984383588383707319735549390316009521484375f,-0.0664000091016292426360223544179461896419525146484375f,-0.05780000263154505546481232158839702606201171875f,0.164999988079070902813327847979962825775146484375f,-0.3362000071734190242978002061136066913604736328125f,-0.081100002223253253585966149330488406121730804443359375f,-0.145000013858080034623299070517532527446746826171875f,0.09499999038875106549539850675500929355621337890625f,-0.1792999987274408457693652962916530668735504150390625f,-0.5859999945163725243446606327779591083526611328125f,-0.3683999994754787454809275004663504660129547119140625f,-0.3059999953359364699423394995392300188541412353515625f,-0.5450000005960458082654440659098327159881591796875f,-0.12199999596178534577717300635413266718387603759765625f,0.023900000086426731460420569419511593878269195556640625f,-0.029700001019239419264295065659098327159881591796875f,0.3035999970078469001322218900895677506923675537109375f,0.06450000218302009979964850572287105023860931396484375f,0.58290002784729022522469676914624869823455810546875f,-0.370999996826052669263162897550500929355621337890625f,-0.142300000753998734381866597686894237995147705078125f,-0.692000021815299959371259319595992565155029296875f,0.726800055873394246219731940072961151599884033203125f,0.461200008961558438613792532123625278472900390625f,1.24980003674030371030312380753457546234130859375f,0.611600038224458852909037887002341449260711669921875f,0.311800007593631800517641750047914683818817138671875f,0.382800025653839526285082683898508548736572265625f,-0.0544000001013278389194738338119350373744964599609375f,-0.02779999734163285385335484534152783453464508056640625f,-0.35779999138116824752842148882336914539337158203125f,0.24919998573064816582700586877763271331787109375f,0.011999992609023735212758765555918216705322265625f,0.51169999675750688084008288569748401641845703125f,-1.17619999622106519154840498231351375579833984375f,0.3294000079989434226490629953332245349884033203125f,1.1773000188589097803060212754644453525543212890625f,-0.932999997258186386517309074406512081623077392578125f,0.6382000171422959056144463829696178436279296875f,1.170100023949145917612213452230207622051239013671875f,-1.474800027874111574277549152611754834651947021484375f,0.66130003298521078392013805569149553775787353515625f,1.2312999939441677810236797085963189601898193359375f,-0.793399976676702589628575879032723605632781982421875f,0.6516000046968457581897382624447345733642578125f,-0.21160001542568185328718755044974386692047119140625f,-0.1801999851644040528952928070793859660625457763671875f,-0.0953000041961669097645426518283784389495849609375f,-0.4618000029742717060798895545303821563720703125f,0.4678000009924172530872965580783784389495849609375f,-0.25590000647902488850604640902020037174224853515625f,0.66899996554851526298079988919198513031005859375f,-0.67839998166859150074969875277020037174224853515625f,0.35300000157952293733387705287896096706390380859375f,-0.13360002114772751014015739201568067073822021484375f,-0.2158000002622604274193918172386474907398223876953125f,-0.0702999946594236035934954998083412647247314453125f,-0.20749998640269051719542403589002788066864013671875f,0.38419998800307519903896036339574493467807769775390625f,-0.12159999987259506593062496904167346656322479248046875f,0.0968999653071165312212542630732059478759765625f,-0.6827999808758498101468603636021725833415985107421875f,0.0908999896779654203982090621138922870159149169921875f,-0.1049999761581421608980235760100185871124267578125f,0.555599979856610470818623070954345166683197021484375f,-0.09199998977780321496311444207094609737396240234375f,0.2012000895023344781975538353435695171356201171875f,-0.263800063383578642373095135553739964962005615234375f,0.140500033020973802422304288484156131744384765625f,0.6824000663518905440696471487171947956085205078125f,-0.214400040334462804736403995775617659091949462890625f,0.361600027346611341272364370524883270263671875f,0.1713999966591597257092871586792171001434326171875f,0.1788000007241963285053998333751223981380462646484375f,0.0707999976173041556393172868411056697368621826171875f,0.692299983394146778437061584554612636566162109375f,-0.50339998993873535937382257543504238128662109375f,-0.53880001629591001144348183515830896794795989990234375f,-0.183000003963708912646524140654946677386760711669921875f,-0.548400020188093417772279281052760779857635498046875f,0.943600029343367285861177151673473417758941650390625f,0.2687000148847700220500200884998776018619537353515625f,0.9819000628352174686597209074534475803375244140625f,-0.52679999895095808160050410151598043739795684814453125f,-0.2246999971263110296160192547176848165690898895263671875f,-0.7251000044099982222434164214064367115497589111328125f,-0.04439999317228793263012676106882281601428985595703125f,0.06780000229626892416234795746277086436748504638671875f,0.40080002261400249352618629927746951580047607421875f,-0.9164000402450565019307759939692914485931396484375f,-0.2356000096589327963414461919455789029598236083984375f,-0.1926000224590307396965727093629539012908935546875f,0.5724000429570679226998208832810632884502410888671875f,0.28650001200288544023209169608890078961849212646484375f,0.64970004169941031335611114627681672573089599609375f,0.422600005546212020846041923505254089832305908203125f,-0.0487999988242984183983708135201595723628997802734375f,-0.2850999798417088726409929222427308559417724609375f,-0.241800010052323433029641819302923977375030517578125f,-0.11490000533163546858617110046907328069210052490234375f,0.20679997898340207740375262801535427570343017578125f,0.3500000220537187178848625990212894976139068603515625f,0.2326000056207180310963167357840575277805328369140625f,0.6092000277519229456402172218076884746551513671875f,-0.251600002089142626982010142455692403018474578857421875f,0.068900001502037067080408405672642402350902557373046875f,0.028499993920326094354322776780463755130767822265625f,0.0280000005364418047548724644002504646778106689453125f,0.1856000001966953139032057151780463755130767822265625f,0.3548000027656552646249110694043338298797607421875f,0.331400005599856495308586090686731040477752685546875f,0.3070000031515955907224224574747495353221893310546875f,0.6899000183701513577716468716971576213836669921875f,-0.825200017350912418123698444105684757232666015625f,-0.20020000818669803077654023581999354064464569091796875f,-0.5696000319063667660657301894389092922210693359375f,-0.761800007221102415400082463747821748256683349609375f,-0.39679999943524590921839489965350367128849029541015625f,-1.24490003952980021750818195869214832782745361328125f,-0.69620003876388114605333612416870892047882080078125f,0.1649000121116639405727255507372319698333740234375f,-0.490200015264749655585774235078133642673492431640625f,0.68140000507831566256555788640980608761310577392578125f,-0.2883000117361547154359868727624416351318359375f,0.8658999982833872621768023236654698848724365234375f,-1.423199990469217635080667605507187545299530029296875f,0.518900011634826796580455265939235687255859375f,-0.323700000598281523167543127783574163913726806640625f,0.56579998595714575682080749174929223954677581787109375f,-0.20339999919608231504497553032706491649150848388671875f,-0.39660000331848888066588187939487397670745849609375f,0.18659998774677533095456283263047225773334503173828125f,-0.19890000748112812711809738175361417233943939208984375f,-1.39760006000995673502984573133289813995361328125f,1.1134000280857083708951904554851353168487548828125f,-0.756100042009354211813842994160950183868408203125f,0.43210005410909690937160121393389999866485595703125f,-0.3672000376760957696120613036327995359897613525390625f,0.247900023972988492459990084171295166015625f,0.9818999865412703087486079311929643154144287109375f,-0.995200008484720566315218093222938477993011474609375f,0.5309000134825705430330344825051724910736083984375f,-0.5302000139236451303759167785756289958953857421875f,-0.056799994853138802142211716272868216037750244140625f,-0.2602000034928320104654630995355546474456787109375f,0.08700002777576454349173218361102044582366943359375f,-0.0862000191688536698819689263473264873027801513671875f,0.0611000068426132969534592120908200740814208984375f,-0.5008999875918023203524853670387528836727142333984375f,0.8310000069960945534575103010865859687328338623046875f,-0.28720001068636780150455933835473842918872833251953125f,-0.28599998557567563040038294275291264057159423828125f,0.5488000043749805545445497045875526964664459228515625f,-0.1596000097036363030156280728988349437713623046875f,-0.0368999859452241452117959852330386638641357421875f,0.229800011929869096860556965111754834651947021484375f,-0.01300000977516191369431908242404460906982421875f,-0.9798000033617026982568631865433417260646820068359375f,1.2489999864101413162842391102458350360393524169921875f,-0.559700014132261447485916505684144794940948486328125f,-0.2956999962806694526307182968594133853912353515625f,0.610999997273087291915771857020445168018341064453125f,-0.2027999913811679988384639727883040904998779296875f,-1.003300059568882574012604891322553157806396484375f,1.19270002495050420066036167554557323455810546875f,-0.96159998741149887990786737645976245403289794921875f,-0.269899996428191624175951801589690148830413818359375f,-1.192100014770030469435369013808667659759521484375f,-0.3115999993324274353057035114034079015254974365234375f,-0.215499993421137236992990438011474907398223876953125f,-0.10269999216794900576132931746542453765869140625f,0.1495999934613705395491933813900686800479888916015625f,-0.05389999900609254712691154054482467472553253173828125f,-0.17649999654293058171106167719699442386627197265625f,0.190800008457898684355313889682292938232421875f,0.140699997211992666734658996574580669403076171875f,0.0623000078320501415873877704143524169921875f,0.1023999966144562012715368837234564125537872314453125f,-0.0346000005394220266907723271287977695465087890625f,0.105799999666214006310838158242404460906982421875f,-0.008600002446770727004832224338315427303314208984375f,0.0670999984532594517805392797527019865810871124267578125f,0.5754000066131352841836132938624359667301177978515625f,0.663600026661157915697231146623380482196807861328125f,0.2244000101596117513480521665769629180431365966796875f,0.0850000357627873626142900320701301097869873046875f,-0.439200017321109914547605512780137360095977783203125f,-0.0201000047639012802136448954115621745586395263671875f,0.1944999881386753060041883145458996295928955078125f,-0.227199994313716902016864196411916054785251617431640625f,0.04090000010877847813617336214520037174224853515625f,0.01229999665617942383732952293939888477325439453125f,0.7807999932587146307838565917336381971836090087890625f,0.23169999850839373412014765563071705400943756103515625f,0.6279000107109542483385666855610907077789306640625f,-0.08220000265836702180166639664093963801860809326171875f,0.106500000454485432843654280077316798269748687744140625f,0.439199998992681361897894021240063011646270751953125f,-0.321199991750717117611202411353588104248046875f,-0.012699998612701879352471223683096468448638916015625f,-0.150300002110004271571597200818359851837158203125f,0.7852000075161458259032087880768813192844390869140625f,0.25520000259876252091117976306122727692127227783203125f,0.6522000206887721418524961336515843868255615234375f,0.20199999536573887137791416535037569701671600341796875f,-0.096699999216198928575494164761039428412914276123046875f,-0.289499993652105214181347037083469331264495849609375f,-1.091000000551342719035119444015435874462127685546875f,0.538800007653236434634891338646411895751953125f,-0.746899961805343792775602196343243122100830078125f,1.56159994926452672103778240852989256381988525390625f,-0.46949999201297742956739966757595539093017578125f,-0.38890002593994044133296483778394758701324462890625f,-0.1761999887704852607583916324074380099773406982421875f,-0.1908000026464460319175486802123486995697021484375f,0.417899979344010574777712463401257991790771484375f,-0.4699999811500312763001829807762987911701202392578125f,0.2266999992534517449627173846238292753696441650390625f,0.23470002322196847899249405600130558013916015625f,0.145199972647429120797823998145759105682373046875f,0.1135000055432318077919262577779591083526611328125f,0.12259998639822011767819276428781449794769287109375f,-0.4239999902844429069403986432007513940334320068359375f,0.08220000072121624867804712266661226749420166015625f,-0.4574000106960536271571982069872319698333740234375f,0.018600003191828751170078248833306133747100830078125f,-0.218500009939074590903373973560519516468048095703125f,1.0274000311851505529148198547773063182830810546875f,-0.551600011700391856805936185992322862148284912109375f,0.5432000183343888011222588829696178436279296875f,-0.9163000183820724231509302626363933086395263671875f,0.784800013196468082554702050401829183101654052734375f,-0.4909000233173372862438554875552654266357421875f,-0.4332999785810709436617571554961614310741424560546875f,0.6667999830812216810471682038041763007640838623046875f,-0.2465000012740492518759083395707421004772186279296875f,0.8446999832868578028666206591879017651081085205078125f,-1.0467999643802645692858277470804750919342041015625f,0.484299997907876811353844459517858922481536865234375f,-0.50879999785572227466445838217623531818389892578125f,0.41460001197606277667517815643805079162120819091796875f,-0.26430001159831884027795467773103155195713043212890625f,-0.461299977254867599185672588646411895751953125f,0.7269999771118165199368377216160297393798828125f,-0.2667999989509581837410223670303821563720703125f,0.8248000021696093586598408364807255566120147705078125f,-0.9689999828338624610069018672220408916473388671875f,0.471500006228685375475606633699499070644378662109375f,0.6114999766871331843276493600569665431976318359375f,-0.71059998771697274744241212829365395009517669677734375f,0.33200000417605035618606734715285710990428924560546875f,-0.7969999797940250374495008145458996295928955078125f,0.59710001178979865699147921986877918243408203125f,-0.51539999722540397186776317539624869823455810546875f,-0.088200000117719223879930723342113196849822998046875f,-0.690700000751018450273477355949580669403076171875f,-0.4922000182300807491486693834303878247737884521484375f,-0.23320000481158498217837404808960855007171630859375f,-0.345300017011165749636347754858434200286865234375f,-0.2793999879568815902786127480794675648212432861328125f,0.08800000072270630635529187202337197959423065185546875f,0.21830000681877148593912352225743234157562255859375f,-0.608599998870491987190689542330801486968994140625f,-0.1648000009402632581867464978131465613842010498046875f,-0.7091000222802161800927933654747903347015380859375f,-0.121800018396973630530055743292905390262603759765625f,-0.169800002430379404216864713816903531551361083984375f,-0.693900019681453983366736792959272861480712890625f,0.427200015771389107754885117174126207828521728515625f,0.1779000047132373030223106979974545538425445556640625f,0.0929000174164773540042006061412394046783447265625f,0.3089999921470880650531398714520037174224853515625f,-0.0944999985694885413778365546022541821002960205078125f,-0.4485999913454055132433495600707828998565673828125f,0.7554000207692386226199232623912394046783447265625f,0.317700006845593463378207843561540357768535614013671875f,1.131200050908327536802744361921213567256927490234375f,-0.50340001445114612721454250277020037174224853515625f,-0.270000003762543183682964809122495353221893310546875f,-0.977300040018558746623966726474463939666748046875f,-0.385000001266598612659208811237476766109466552734375f,-0.1027999981239438032076805029646493494510650634765625f,-0.5301000030875204771518838242627680301666259765625f,1.10500002168118971468402378377504646778106689453125f,0.38540000575631860879610712800058536231517791748046875f,1.21590003881454489231828119955025613307952880859375f,0.6508000010073182739489539017085917294025421142578125f,0.20929999846667046714543403140851296484470367431640625f,0.34930001087188689012918985099531710147857666015625f,-0.0852000014066696120806909675593487918376922607421875f,-0.0936000000625848704860487714540795423090457916259765625f,-0.4930000097006559389711810581502504646778106689453125f,-0.65700000773370259565808737534098327159881591796875f,-0.27120000154823065141584947923547588288784027099609375f,-0.8669000289350747845418254655669443309307098388671875f,0.588000046581030222370145565946586430072784423828125f,-0.0751000010013580521217591012828052043914794921875f,-0.388099983060360642639352590776979923248291015625f,1.10159996885955369094745037727989256381988525390625f,-0.257900003409385902841677307151257991790771484375f,-0.53030003131628067336578169488348066806793212890625f,0.2660000081509352742870078145642764866352081298828125f,-0.2863000118255616399665086646564304828643798828125f,-0.69509997186958827342095901258289813995361328125f,0.8791999679982664961386262802989222109317779541015625f,-0.3829000029250979597605919479974545538425445556640625f,-0.50789997748136528343820828013122081756591796875f,0.991399976393580573130748234689235687255859375f,-0.3139999968409536990066044381819665431976318359375f,0.3542999765992167482409058720804750919342041015625f,-0.107999973490834566547391659696586430072784423828125f,0.16769999663829793234981480054557323455810546875f,0.6193000257730485458296243450604379177093505859375f,-0.232400008833408389108399205724708735942840576171875f,0.32410001125335696769980131648480892181396484375f,0.9159999702274799204815280972979962825775146484375f,-0.85799997796118265824816262465901672840118408203125f,0.488200003507733182317451792187057435512542724609375f,0.4006000016868114244061871431767940521240234375f,-0.8756000032514332787059174734167754650115966796875f,0.26440000218749037941279311780817806720733642578125f,-0.081100001180171688019981957040727138519287109375f,0.44720000675618631902352717588655650615692138671875f,-0.0816999967575071650571771897375583648681640625f,-0.2634999966025350914833325077779591083526611328125f,0.673599993357062221122077971813268959522247314453125f,-0.1733000041365624355194086092524230480194091796875f,0.935500012755394294572397484444081783294677734375f,-1.39979999211132533076806794269941747188568115234375f,0.5582000132679940662683293339796364307403564453125f,0.82170000182390179332969637471251189708709716796875f,-0.7179999893605708560784250948927365243434906005859375f,0.452500004470348216045749722979962825775146484375f,0.24220000370144845192044158466160297393798828125f,0.0347999982208013758366860201931558549404144287109375f,0.1096000060155988276022753780125640332698822021484375f,-0.43449999745190126532179419882595539093017578125f,0.72539999275505540055064557236619293689727783203125f,-0.269199999067187167867132302490063011646270751953125f,0.3420000593662262389216266456060111522674560546875f,0.572300001871585806156872422434389591217041015625f,0.50260001344382754240314170601777732372283935546875f,0.1775000004470348446972138845012523233890533447265625f,-0.1178000047922136417355432058684527873992919921875f,0.173399998953938538903685184777714312076568603515625f,-0.04799999669939281066177727552712894976139068603515625f,-0.32049998235702492621612691436894237995147705078125f,0.334199991728365175536197284600348211824893951416015625f,0.2030999960429965833785814766088151372969150543212890625f,0.355199998538195504949044334352947771549224853515625f,0.753799989846348328370595481828786432743072509765625f,0.1098999942302703036745015197084285318851470947265625f,0.3166000058889384405347300344146788120269775390625f,0.287199996697902637521337965154089033603668212890625f,-0.03429999787062409932758555441978387534618377685546875f,-0.40649998402595510782475685118697583675384521484375f,-0.0796000012099742360049958733725361526012420654296875f,-0.0151999992460012389727808113093487918376922607421875f,-0.1790000025033950947772609652020037174224853515625f,0.960399983441829530050881658098660409450531005859375f,0.277699996452033470717424279428087174892425537109375f,0.938500010907649340197167475707828998565673828125f,-0.246400040394067987303827749201445840299129486083984375f,-0.21770000896900898457175799194374121725559234619140625f,-0.2899000301420695535625782213173806667327880859375f,-0.74439998087882941479875853474368341267108917236328125f,-0.1651999913483856996521126347943209111690521240234375f,-0.596200002181529242761826026253402233123779296875f,-0.42100003094971238937205271213315427303314208984375f,-0.301100013101101016133043231093324720859527587890625f,-0.829400046443939942264478304423391819000244140625f,-0.38719999833703033775833546314970590174198150634765625f,-0.12639999926686285025567713091732002794742584228515625f,-0.37000000774860364316509730997495353221893310546875f,0.6532000082015994024686733609996736049652099609375f,0.1964000070899725169937255486729554831981658935546875f,0.8946000460624699002210036269389092922210693359375f,-0.005200003790855500795942134573124349117279052734375f,-0.1529000020310282703661641789949499070644378662109375f,-0.6217000161290169302219510427676141262054443359375f,0.2252000160843135034838269348256289958953857421875f,0.155800003726780433854770535617717541754245758056640625f,0.23360001451671141836641254485584795475006103515625f,0.724600026169419653143677351181395351886749267578125f,-0.2097999938249586193705908954143524169921875f,0.905400041854381498751536128111183643341064453125f,-0.29580001136362543689983795047737658023834228515625f,0.466300020766258427329375990666449069976806640625f,0.5610999809145926775499901850707828998565673828125f,-0.426199991676211364932669312111102044582366943359375f,0.29380000035166720095958226011134684085845947265625f,0.253800000053644059239132957372930832207202911376953125f,-0.050799982529878828785285804769955575466156005859375f,0.13799999193102097105878556249081157147884368896484375f,1.0731999739289275197506867698393762111663818359375f,-1.349199977907538094967776487465016543865203857421875f,0.6054000040054319242699421010911464691162109375f,0.8202999797582624097458392498083412647247314453125f,-0.575999988555908171150576890795491635799407958984375f,0.42890000194907162978097403538413345813751220703125f,-0.00739999780654887473474445869214832782745361328125f,0.1992000022709368156625941992388106882572174072265625f,-0.0162000012874603527279759873636066913604736328125f,0.9230999630928042876121253357268869876861572265625f,-1.400599941933155623274842582759447395801544189453125f,0.545799990129470558031243854202330112457275390625f,-0.1371000633478163255318804658600129187107086181640625f,-0.2363999460697179255674882369930855929851531982421875f,-0.065500022441149052809805652941577136516571044921875f,-0.831599963569640276972450010362081229686737060546875f,1.232599964940547909719725794275291264057159423828125f,-0.48489999333620037447190043167211115360260009765625f,-0.026000029981137373624733299948275089263916015625f,0.06120001104772132549669549916870892047882080078125f,-0.03420000450611127007505274377763271331787109375f,-0.371799993288516983369618174037896096706390380859375f,0.49939998639225968712906933433259837329387664794921875f,-0.21689999905824652870478530530817806720733642578125f,0.44079997403621717921851086430251598358154296875f,-1.0473999787330630084625227027572691440582275390625f,0.2866999949693678217954584397375583648681640625f,0.5036999986052510536183035583235323429107666015625f,-0.328600001180171741310687139048241078853607177734375f,0.25400001189112675348269476671703159809112548828125f,0.16680002726316456307387170454603619873523712158203125f,0.04879998351335534945150129715329967439174652099609375f,0.0872000077992678424010364324203692376613616943359375f,1.2174000396788127176250782213173806667327880859375f,-0.36620001112222677619456590036861598491668701171875f,-0.69699998806416996188772827736102044582366943359375f,-0.169699999752640717787244284409098327159881591796875f,-0.976800018620490817511381465010344982147216796875f,0.0664000040352343834371140474104322493076324462890625f,0.129199998620152467498911619259160943329334259033203125f,0.672800005239248211097446983330883085727691650390625f,0.04600001075863868749138418934307992458343505859375f,0.1073000054106116873953169488231651484966278076171875f,-0.3871000052332878027527840458787977695465087890625f,-0.4106000099569560024548309229430742561817169189453125f,-0.1477000037163496148906460803118534386157989501953125f,-0.0392000128507614675754666677676141262054443359375f,0.05360003172755245515190836158581078052520751953125f,0.1156000077962875405290787966805510222911834716796875f,-0.3995999715566629362228923127986490726470947265625f,0.68519998844265916915219349903054535388946533203125f,0.0944999962970614060253637944697402417659759521484375f,-0.1243000093817716589228439261205494403839111328125f,0.6066000065594909784749688697047531604766845703125f,0.137399997024238107456994839594699442386627197265625f,0.847900007283687529024973628111183643341064453125f,0.02380000050067898609285066413576714694499969482421875f,0.0784999975338578348527107664267532527446746826171875f,0.5409999927282331810829418827779591083526611328125f,-0.366999993279576219862292418838478624820709228515625f,-0.02779999782592057488983527946402318775653839111328125f,0.21230001230239903264873646548949182033538818359375f,-0.951600022131204870134979501017369329929351806640625f,-0.3402000058770180412892614185693673789501190185546875f,-1.0772000285863878588088482501916587352752685546875f,-0.500999989822506819336922490037977695465087890625f,-0.14309999542832374519463201067992486059665679931640625f,-0.66579998863935418995652071316726505756378173828125f,-0.0541999828249217774356338850338943302631378173828125f,0.039200000818073732489210669882595539093017578125f,-0.228699994432925990395233384333550930023193359375f,0.4673999937087298661708700819872319698333740234375f,-0.031799998094141468385487314662896096706390380859375f,-0.273299996387958543664353783242404460906982421875f,0.74100002603232884013095826958306133747100830078125f,0.1920000079944730320136159207322634756565093994140625f,0.4249000356554990531776638817973434925079345703125f,-0.0220000002086162549375103480997495353221893310546875f,0.1723000095084310512305592055781744420528411865234375f,-0.471999955356121603244901052676141262054443359375f,1.29739994960129312318031225004233419895172119140625f,-0.3148999885678289700763343716971576213836669921875f,-0.4203999937981368617556654498912394046783447265625f,0.0683999940365553893428796072839759290218353271484375f,-0.1998000043675303150081390413106419146060943603515625f,0.312500039935112994271548814140260219573974609375f,0.54439999831318797163248746073804795742034912109375f,0.1257000084996224842370793339796364307403564453125f,-0.64140001670122170907006875495426356792449951171875f,0.2971999993056060329621459459303878247737884521484375f,-0.3368000084877014188577959430404007434844970703125f,0.331400053358077428811157005839049816131591796875f,0.56899994572997147912474247277714312076568603515625f,0.133900017595291132010970613919198513031005859375f,1.273299988937377946740525658242404460906982421875f,-1.04559996652007125561567590921185910701751708984375f,0.683500007629394445984871708787977695465087890625f,0.4168999978661531713441945612430572509765625f,-1.1716000007480378286572886281646788120269775390625f,0.2789000113368036437577757169492542743682861328125f,-0.37979999363869421102890555630438029766082763671875f,0.01260000021904705835851245865342207252979278564453125f,-0.17990000169947728547725773751153610646724700927734375f,-0.7964999983310701026084643672220408916473388671875f,0.584599980124831386518735598656348884105682373046875f,-0.42160000767707828117636381648480892181396484375f,-0.7804000206947332429763264372013509273529052734375f,1.185199994105100795849239148083142936229705810546875f,-0.46780001552104977236012928187847137451171875f,-0.3559999879598620964316069148480892181396484375f,0.825199967059493388887858600355684757232666015625f,-0.2293999990582467063404692453332245349884033203125f,0.04970003633499064932266264804638922214508056640625f,0.3981999583572155554378468877985142171382904052734375f,0.00300001290440565782091653090901672840118408203125f,1.0460999779343609361603739671409130096435546875f,-1.0275999696344140232895369990728795528411865234375f,0.5649000070452689925559752737171947956085205078125f,0.936500019490718926817862666212022304534912109375f,-1.02700001265108564751926678582094609737396240234375f,0.5252000204801561267231591045856475830078125f,-0.32099999574571835836422906140796840190887451171875f,-1.3679000203967088822309960960410535335540771484375f,-0.5136000156342983569146554145845584571361541748046875f,-0.31020000312030315825495563331060111522674560546875f,-0.647000020027160616109540569595992565155029296875f,0.12579998141229165042886961600743234157562255859375f,-0.014200002308189851873976294882595539093017578125f,0.2714999873042103217812837101519107818603515625f,-0.1349999979883431677762928302399814128875732421875f,-0.19229999717772006118110539318877272307872772216796875f,-0.76839999679326975723370196647010743618011474609375f,0.5573999832779163998708327198983170092105865478515625f,0.281099990041553893238557293443591333925724029541015625f,1.034799992372094745718413832946680486202239990234375f,0.349599999943375649991139653138816356658935546875f,0.218600001142919053709334775703609921038150787353515625f,0.34700000743567915861831352231092751026153564453125f,0.9301999962657685472322555142454802989959716796875f,0.2574999945610760843095476957387290894985198974609375f,0.3556000024318688446101077715866267681121826171875f,0.03820000175684691579025553664905601181089878082275390625f,0.0485000004582107069328600346125313080847263336181640625f,0.06860000166445967550998830120079219341278076171875f,-0.98300002966821242722517126821912825107574462890625f,-0.243700010079145545915935144876129925251007080078125f,-0.886200037050247857450813171453773975372314453125f,-1.143000006273389157485098621691577136516571044921875f,-0.35910000581443313194540678523480892181396484375f,-1.38480004270076761940799769945442676544189453125f,0.267400000712275431880016185459680855274200439453125f,0.14840000100284811423989594914019107818603515625f,-0.00989999035596866860942100174725055694580078125f,-0.335600000494718386789827491156756877899169921875f,0.0077000007733702968693023649393580853939056396484375f,0.0168999895215034001694220933131873607635498046875f,0.899400025358796373353698072605766355991363525390625f,0.1863000101864338287072087041451595723628997802734375f,0.1442000462293631102284052758477628231048583984375f,0.1647999974012375190568491234444081783294677734375f,0.04609999898225070058543906270642764866352081298828125f,0.63590000543594360493671047152020037174224853515625f,-0.152400003468990019594997420426807366311550140380859375f,-0.16679999686479562992502678753226064145565032958984375f,-0.165399999088048588191668386571109294891357421875f,0.5854000084012749294259947419050149619579315185546875f,1.0089999919235703718101149206631816923618316650390625f,-0.36550001171231283336737760691903531551361083984375f,0.1563999936044215832708914604154415428638458251953125f,-0.0716999984711408355764206135063432157039642333984375f,-0.5394999945089218318372559224371798336505889892578125f,-0.9967999767854809345379862861591391265392303466796875f,-0.3112999910756946153611579575226642191410064697265625f,-0.125000018477440022479640902020037174224853515625f,-1.07529998780488966758639435283839702606201171875f,0.3755999988928435673329886412830092012882232666015625f,0.366000019550323685280091012828052043914794921875f,0.43409999858736991740215671597979962825775146484375f,-0.28000000037252892326478104223497211933135986328125f,0.1180000011324882080998577293939888477325439453125f,0.899300002527236852500891473027877509593963623046875f,-0.0629999961033462785309211540152318775653839111328125f,0.8845000345930460472487766310223378241062164306640625f,-0.108699985752999717103506327475770376622676849365234375f,-0.0758999985814095001757095815264619886875152587890625f,-0.604000030815601718359175720252096652984619140625f,-0.18169999899268152177000956726260483264923095703125f,0.4652000035673378164347013807855546474456787109375f,0.366999989330768716655484240618534386157989501953125f,0.065000005587935394402165911742486059665679931640625f,0.69179997905790824308525088781607337296009063720703125f,0.2305000147670508425790103501640260219573974609375f,0.28310000287890435544113643118180334568023681640625f,-0.15489999557137490437952465072157792747020721435546875f,0.3065000111460687293174487422220408916473388671875f,0.161200001287460370491544381366111338138580322265625f,0.2330999944597482720354264529305510222911834716796875f,-0.44549998995661699296988444984890520572662353515625f,-0.7601999830782413258845053860568441450595855712890625f,-0.1228999982684850778014151728712022304534912109375f,-0.8914999864101407212046979111619293689727783203125f,0.4648999943792819777854674612171947956085205078125f,-0.73689999063909039644926224355003796517848968505859375f,0.0014999934136863402045491966418921947479248046875f,-0.0249000118136404324786781216971576213836669921875f,0.53430000785440168176165798286092467606067657470703125f,-0.6890000153183937214862453402020037174224853515625f,0.531500015318393526086993006174452602863311767578125f,-0.1927999915674327713333013889496214687824249267578125f,-0.54449999529123260799678973853588104248046875f,0.1307999996662143615822060382924973964691162109375f,-0.55089999709129333638202297152020037174224853515625f,0.622300002616643865849255234934389591217041015625f,0.3619000044226645673717257523094303905963897705078125f,-0.5133000097244977411037325509823858737945556640625f,0.18060002612471581784348018118180334568023681640625f,0.6108000126302239873865573827060870826244354248046875f,0.149400017535686657765836571343243122100830078125f,1.976100019359588344514122582040727138519287109375f,0.1047000205397612404567553312517702579498291015625f,0.3881000098079443461074333754368126392364501953125f,0.01299999055266365388661142787896096706390380859375f,-0.5348000249683859674831865049782209098339080810546875f,-1.4982000016450882640128838829696178436279296875f,0.6732000135660172190910088829696178436279296875f,0.122400005555153512659671832807362079620361328125f,0.89579998029470431220033788122236728668212890625f,-0.255600013756751565097147249616682529449462890625f,-0.48279999704360943013625728781335055828094482421875f,-0.333600013399124151192154386080801486968994140625f,0.276900015151500866750211571343243122100830078125f,-0.0471000070214273591773235239088535308837890625f,0.14000000245869159698486328125f,0.27999999560415744781494140625f,-0.174000004068016966840559689444489777088165283203125f,-0.84769998888969411154903355054557323455810546875f,-0.7197999836921695759883732534945011138916015625f,0.8854000072836871737536057480610907077789306640625f,0.4643000005155801801492998492904007434844970703125f,-1.12400000742077832427412431570701301097869873046875f,-0.1714000070154668708966028134454973042011260986328125f,-0.547000003635883302877118694595992565155029296875f,-0.097999984741210965921709430404007434844970703125f,-0.1060000123977662411789424368180334568023681640625f,-0.43290000802874573082590359263122081756591796875f,0.5142000030159952217445606947876513004302978515625f,0.612000031054020343646016044658608734607696533203125f,-0.10980001438856135109745082445442676544189453125f,-1.029600005531310813466916442848742008209228515625f,0.30300000649690606024933003936894237995147705078125f,0.030700006189942286027871887199580669403076171875f,0.808999979034066285521475947462022304534912109375f,-0.591600015798210865369810562697239220142364501953125f,-0.2142999745130538258308661170303821563720703125f,0.6166999685570602185435973296989686787128448486328125f,-0.06999999843537807464599609375f,0.56139999345541002906401217842358164489269256591796875f,0.2197999978110194152502998576892423443496227264404296875f,-0.421999994918703880131261030328460037708282470703125f,0.40370000508725645271823623261298052966594696044921875f,0.21460000214129681950225858599878847599029541015625f,-0.2590000120401381167312138131819665431976318359375f,1.056700013297795948119528475217521190643310546875f,-0.46780000099241692002038917053141631186008453369140625f,0.414000013455748661073130278964526951313018798828125f,-0.25050000225007529319753984964336268603801727294921875f,-0.1722000018015503730595128217828460037708282470703125f,0.945999977231025468427105806767940521240234375f,0.6926000045776365965366494492627680301666259765625f,0.954399980506301037763705608085729181766510009765625f,-0.0289999926686288489463549922220408916473388671875f,-0.474999989867210370420025355997495353221893310546875f,-0.5085999785304071707514594891108572483062744140625f,0.3075000140070915932710704510100185871124267578125f,0.19650000905990605559736650320701301097869873046875f,0.20670000476688132895475291661568917334079742431640625f,0.12300000284612178091947498614899814128875732421875f,0.0624000003397464642063141582184471189975738525390625f,-0.11299999830126761235504773139837197959423065185546875f,-0.52950000929832441443068091757595539093017578125f,0.08369998623728758246897996286861598491668701171875f,0.5538999813109637404551222061854787170886993408203125f,-0.113000000759959196017234717146493494510650634765625f,-0.74650000257790083413311776894261129200458526611328125f,-0.11160000145584347830407523360918276011943817138671875f,0.5450000035762787575777110760100185871124267578125f,0.0190000060796737813006984652020037174224853515625f,0.43899999147653578912553484769887290894985198974609375f,-0.48500001505017298342181675252504646778106689453125f,0.5656999991118907278320193654508329927921295166015625f,0.09880000485926865305685851126327179372310638427734375f,-0.3104999999403954547005923814140260219573974609375f,-0.423000006049871313251742321881465613842010498046875f,0.191100004719197524849505498423241078853607177734375f,0.74900000391900523055710436892695724964141845703125f,0.2232999968349933528344308797386474907398223876953125f,-0.0343999995425343652044603004469536244869232177734375f,0.05300003123283403283494408242404460906982421875f,0.5386000192105770967287980965920723974704742431640625f,-0.202499978244304656982421875f,1.590599984550475909372835303656756877899169921875f,-0.18459999688863693023677114979363977909088134765625f,-0.5392999758541581201143344515003263950347900390625f,-0.8889999616742141341774186003021895885467529296875f,-0.12760001888275152026608338928781449794769287109375f,0.5982000105857849803214776329696178436279296875f,-0.2501999996185304553364403545856475830078125f,-0.043399985170363919451119727455079555511474609375f,0.1368999811768532026690081693232059478759765625f,-0.364299976897239474737943965010344982147216796875f,-0.82690002260208128603835575631819665431976318359375f,-0.2022000212848187317149495356716215610504150390625f,-1.6472999920368192050545985694043338298797607421875f,-0.16810000809431091539636327070184051990509033203125f,0.90509999727606782471411861479282379150390625f,-0.4958999908328056704931441345252096652984619140625f,-0.775900018548965508813353153527714312076568603515625f,0.087399992963671735424213693477213382720947265625f,-0.39219998544752598945706267841160297393798828125f,0.389000008836388300181852173409424722194671630859375f,-0.10610001786947265145499841310083866119384765625f,1.383499995112419611587029066868126392364501953125f,0.19930002458095597006604293710552155971527099609375f,-0.1956999959081411333272626507095992565155029296875f,-0.44570000045299540403220817097462713718414306640625f,0.4893000141501426814016895150416530668735504150390625f,-0.846600006708502661467719008214771747589111328125f,-1.1219999616891147553587870788760483264923095703125f,0.44479999985992879629748131264932453632354736328125f,-0.969700032460689698154965299181640148162841796875f,-0.531499981641768926010627183131873607635498046875f,0.70010002514123925720923580229282379150390625f,0.4494999884366990272610564716160297393798828125f,0.8732000314474106517081963829696178436279296875f,0.556800031435489728437460144050419330596923828125f,0.758900010144710535087142488919198513031005859375f,-0.321200025129318333938499563373625278472900390625f,-0.09500000596046476175615680404007434844970703125f,0.4258000198386611145906499586999416351318359375f,1.175900002101808450305497899535112082958221435546875f,-0.5023000229753555867517889055307023227214813232421875f,0.5625000260770320892333984375f,0.1155000037029385584474994175252504646778106689453125f,0.3834999864697454796669262577779591083526611328125f,0.822499993890523750650345391477458178997039794921875f,-0.0835000074803826652924954032641835510730743408203125f,0.57199998217821157453499836265109479427337646484375f,-0.41029998944401757654532048036344349384307861328125f,0.5725999622210864448135225757141597568988800048828125f,-0.3850000202655792236328125f,-0.2647999950170518701497712754644453525543212890625f,-0.385400003856420259040760356583632528781890869140625f,0.9855000118613244097787173814140260219573974609375f,-0.500999972462654152849381716805510222911834716796875f,0.1377000017046927649033705165493302047252655029296875f,0.37949999856948846144177878159098327159881591796875f,-0.8930999921500681892894135671667754650115966796875f,0.4638999972179529418525589790078811347484588623046875f,0.35000000968575495363666050252504646778106689453125f,-0.7039999816417694677994632002082653343677520751953125f,-0.41219999636262649733708940402721054852008819580078125f,-0.57299999479949459413319345912896096706390380859375f,-0.0935999971568584054892880885745398700237274169921875f,-0.079799995091557551774030798696912825107574462890625f,-0.1904999987483027013013270334340631961822509765625f,0.823800005567073778678377493633888661861419677734375f,-0.20029997923672215875967594911344349384307861328125f,-0.494500012204051042630226220353506505489349365234375f,-0.03260000837743282087188845252967439591884613037109375f,0.114899994677305272716694162227213382720947265625f,0.0499999930709602580236605717800557613372802734375f,-0.27979999114274978211369671043939888477325439453125f,0.71519998248219496250754900756874121725559234619140625f,-0.1970000075101854264403300476260483264923095703125f,0.14480000619292265895410309894941747188568115234375f,0.65040000575631828905187603595550172030925750732421875f,-1.068499994516372453290387056767940521240234375f,0.5651000181376932829380166367627680301666259765625f,-0.595499981164932190580429960391484200954437255859375f,-0.4359999921321868754375827847979962825775146484375f,-0.35150001412630071939702247618697583675384521484375f,-0.47979999348968271899451565332128666341304779052734375f,0.337000017810613172741796006448566913604736328125f,0.3713000028848647549040151716326363384723663330078125f,-0.40460000130310636023978076991625130176544189453125f,0.07299998727440826229440062888897955417633056640625f,0.170600021691620806763012296869419515132904052734375f,0.38850000293552877650427035405300557613372802734375f,-0.78090000305175788497535904753021895885467529296875f,-0.487100012832879958324383551371283829212188720703125f,-1.10389999792575821402351721189916133880615234375f,0.851000038027763139325543306767940521240234375f,-0.0298000215411189373071465524844825267791748046875f,-0.41170000212192547905942774377763271331787109375f,0.6420999834775926728980266489088535308837890625f,-0.2225000059604642643762417719699442386627197265625f,-0.9791999752998350459165521897375583648681640625f,1.048200014162063098410726524889469146728515625f,0.006399975574016281143485684879124164581298828125f,-0.0669000106811523664873675443232059478759765625f,-1.58399998700618738212142488919198513031005859375f,0.4362000251293185471013202914036810398101806640625f,0.87699997778236848944288794882595539093017578125f,0.2851999733179813034666949533857405185699462890625f,0.15280001023113687352861234103329479694366455078125f,-0.7261000022232533268606857745908200740814208984375f,0.4407999948978424953338617342524230480194091796875f,0.59500001102685917686585526098497211933135986328125f,1.151299989771843002017703838646411895751953125f,-0.0691999937772749262876459397375583648681640625f,-0.96020001571178426047481480054557323455810546875f,-0.50480001766681681374393519945442676544189453125f,-0.58059999051094024480335065163671970367431640625f,-0.71200001436471893612178973853588104248046875f,-0.647799975995719368171421592705883085727691650390625f,0.377499991059302875129333187942393124103546142578125f,0.4168999909371133849589341480168513953685760498046875f,1.3078999747991559843285358510911464691162109375f,0.426999964594841685538995079696178436279296875f,-1.00120002095699334176970296539366245269775390625f,0.8353000283360483280148400808684527873992919921875f,1.2229999983012671549431615858338773250579833984375f,-0.25440000784993177518344964482821524143218994140625f,-0.53569999501407128406071933568455278873443603515625f,-1.04169995890855826559118213481269776821136474609375f,0.030299997490644603459486461360938847064971923828125f,-1.34459999182224265723561984486877918243408203125f,-0.449099973702431043420801870524883270263671875f,0.6539000077605248151257910649292171001434326171875f,-0.73239999313130965674645267426967620849609375f,-0.7643000054329629477933849557302892208099365234375f,0.355499995768070231605406661401502788066864013671875f,-0.65639999516904357079738474567420780658721923828125f,-0.0475000059604644142563500963660771958529949188232421875f,0.85250001370906858255693805404007434844970703125f,0.65830001203417776167725605773739516735076904296875f,0.6682999849140645043377162437536753714084625244140625f,0.593500000476836930829449556767940521240234375f,0.34639999464750292901271677692420780658721923828125f,0.2389000006824730792942546031554229557514190673828125f,0.711500006675720442217425443232059478759765625f,0.7509999934732913828838718472979962825775146484375f,0.535899977160990470537171859177760779857635498046875f,-0.514000027537345971495597041212022304534912109375f,-0.68089999411106116866676529753021895885467529296875f,0.22579999322146182549886361812241375446319580078125f,-0.8519999715238810722439666278660297393798828125f,0.03179999179840109491124167107045650482177734375f,-0.7219999626576905082941948421648703515529632568359375f,-0.24500000000000010658141036401502788066864013671875f,0.5782999867022038387176507967524230480194091796875f,0.329200000669062020630661891118506900966167449951171875f,0.4070000140666962806790252216160297393798828125f,-0.58789997569322594017648953013122081756591796875f,-0.62559998637586833325485713430680334568023681640625f,1.311499989986419478782408987171947956085205078125f,-0.4513999810874462781384863774292171001434326171875f,0.6190999910250305493519817900960333645343780517578125f,-0.0689999987781046986157207356882281601428985595703125f,0.468899976952374064609330162056721746921539306640625f,0.320599984550476169165023065943387337028980255126953125f,0.34200003910064680212599341757595539093017578125f,-0.368699995028972438149139634333550930023193359375f,-0.815799961407483120723327374435029923915863037109375f,-0.29199999096989603941665336606092751026153564453125f,0.40029999927878368026767930132336914539337158203125f,-0.76459999938458178547762145171873271465301513671875f,0.362999979078769907658852389431558549404144287109375f,-0.438599982479214833830383213353343307971954345703125f,0.5623999734804037142765764656360261142253875732421875f,0.14099997723102575264420011080801486968994140625f,0.24629999707341188042164503713138401508331298828125f,0.9201999722748996912713437268394045531749725341796875f,-0.322000038933008880803754436783492565155029296875f,-0.537300027501583343791935476474463939666748046875f,0.2277000359773637683247216045856475830078125f,0.90089999828338562792851007543504238128662109375f,0.4335000165700915175648333388380706310272216796875f,0.016299984407424972232547588646411895751953125f,-1.7580999773979186784345074556767940521240234375f,-0.040300009560585436929613933898508548736572265625f,-0.6477000265896322162006981670856475830078125f,-1.791599997991323078849745797924697399139404296875f,0.550200019139051477168322890065610408782958984375f,-0.007500018775463246356594027020037174224853515625f,-0.782100007021426790743134915828704833984375f,-0.12889999732971180179674774990417063236236572265625f,-0.57619999059587723166941941599361598491668701171875f,0.415099985578656127671592912520281970500946044921875f,0.83550002236664300259150195415713824331760406494140625f,0.364200010317563993567091529257595539093017578125f,0.2507999788045882638698458322323858737945556640625f,-0.677400024926662513991004743729718029499053955078125f,0.65899998300150031127486727200448513031005859375f,-0.092900006706267301836987826391123235225677490234375f,0.2619000080175692968253997605643235146999359130859375f,-0.74219997792243930234690196812152862548828125f,-0.3050999616622931398524087853729724884033203125f,-0.562300040912628418254826101474463939666748046875f,-0.319199995546787906874897089437581598758697509765625f,0.1230000088438389926892568837502039968967437744140625f,0.5056000009045000798124647189979441463947296142578125f,0.813799998861551188156226999126374721527099609375f,1.26309996860623385828148457221686840057373046875f,-0.3255000042617322009164126939140260219573974609375f,0.16600004279613500557388761080801486968994140625f,0.74719996600151006305168266408145427703857421875f,0.879800028693676239299747976474463939666748046875f,0.501999981340020884346131424535997211933135986328125f,-0.9696999828025696377409303750027902424335479736328125f,-0.206299985711276112620993217205977998673915863037109375f,-0.39100003392994420181594250607304275035858154296875f,-1.48579998893737741383347383816726505756378173828125f,-0.246200000542402097636340840836055576801300048828125f,-1.425100000703334757190532400272786617279052734375f,0.137500029206275797832859097979962825775146484375f,0.4632999996662139352565645822323858737945556640625f,0.040399995064735350069895503111183643341064453125f,0.4972999954640864306298908559256233274936676025390625f,-0.742500018477439738262546597979962825775146484375f,0.58139998868703823831083354889415204524993896484375f,-0.411900012320279618194263093755580484867095947265625f,0.05050000381469743615525658242404460906982421875f,-0.9513999886870383448922439129091799259185791015625f,0.2958999954521654984773704200051724910736083984375f,-0.232000016897917049618627061136066913604736328125f,-0.63890000269412983158190399990417063236236572265625f,0.661399998633563246386302125756628811359405517578125f,0.178999997138976851829283987171947956085205078125f,-0.457999996811151444120469022891484200954437255859375f,0.16600000617653110079885436789481900632381439208984375f,-0.6715000120773912950511430608457885682582855224609375f,0.61689997793585060748000614694319665431976318359375f,0.1728999971508979882628409541212022304534912109375f,0.61750000998377796435079289949499070644378662109375f,-0.1678000050902366258043230118346400558948516845703125f,-0.1649000008240342207699313803459517657756805419921875f,-0.83149997681751841582808992825448513031005859375f,-0.0895000025928018860810198020772077143192291259765625f,-0.5214999742843213770271404428058303892612457275390625f,1.28949999916553537104846327565610408782958984375f,-0.2076999862074853098192761535756289958953857421875f,0.71389996324330606825014911009930074214935302734375f,-0.42749999074265343512735171316307969391345977783203125f,0.299399991887062899564853069023229181766510009765625f,0.10489999834671615008829803628032095730304718017578125f,-0.1829999762475491564828189439140260219573974609375f,0.40169999213814744365436126827262341976165771484375f,-0.74059997464120408405818807295872829854488372802734375f,-1.281500025749206628233878291212022304534912109375f,0.663199963647127077592813293449580669403076171875f,-0.4355000099614259045921471624751575291156768798828125f,-0.09600001353397968895109215736738406121730804443359375f,-0.68369997750148192583452555481926538050174713134765625f,-0.1579999948553740818368851250852458178997039794921875f,0.620999993324279575546142950770445168018341064453125f,-0.796799983826279589749219667282886803150177001953125f,0.59100000137090658025584843926480971276760101318359375f,0.4474999922513962502534923260100185871124267578125f,0.509200002419948560827833716757595539093017578125f,0.91969998045563705613147931217099539935588836669921875f,-0.74700001853704467436045888462103903293609619140625f,-0.01699999849498257109559062882908619940280914306640625f,0.029099998438358198882269789464771747589111328125f,-1.325700028467178270830117980949580669403076171875f,-0.5901000400424010194910806603729724884033203125f,-0.3856000197172164689618512056767940521240234375f,-1.91959996500015250830983859486877918243408203125f,0.0864000086545946288651975919492542743682861328125f,0.13579998528659320555789236095733940601348876953125f,-1.47500003635883292218977658194489777088165283203125f,-0.487400021499395652568864534259773790836334228515625f,0.48219998716115952674954314716160297393798828125f,-1.30239998737573614562279544770717620849609375f,0.21480001856088648537479457445442676544189453125f,0.14219999103546143714993377216160297393798828125f,0.52170001642704022515317774377763271331787109375f,0.84770004789829300761994090862572193145751953125f,-0.05159999728351838399476037011481821537017822265625f,0.255299986016750413142517572850920259952545166015625f,-0.25850000301003428493373803576105274260044097900390625f,0.635599982315302014512781170196831226348876953125f,-1.5184999656081199503887546597979962825775146484375f,-0.105299998533725958083095974870957434177398681640625f,-1.260899971163272681451417156495153903961181640625f,-0.008699979829788873075813171453773975372314453125f,0.706900000250339388685461017303168773651123046875f,0.1501000077068805893532044137828052043914794921875f,1.6198999918460845748313658987171947956085205078125f,0.16830000443458548176067779422737658023834228515625f,0.20359999194145217416007653810083866119384765625f,0.13960003831386558204030734486877918243408203125f,0.734600033247470918240651371888816356658935546875f,0.650400009369850096163645503111183643341064453125f,0.542399970686435750621967599727213382720947265625f,-0.5076000060677525738128679222427308559417724609375f,0.1060999825537205509817795245908200740814208984375f,-1.4071999600410460828925351961515843868255615234375f,-0.25559999051094006716766671161167323589324951171875f,1.0629999772906302268893341533839702606201171875f,-0.61420001076459840305687976069748401641845703125f,0.0782000222086907115226495079696178436279296875f,-0.829599980348348253755830228328704833984375f,0.347800056499242060681353905238211154937744140625f,-0.3320000422298914344310105661861598491668701171875f,0.19349998676776891670670011080801486968994140625f,0.089099999257922168904855197979486547410488128662109375f,0.7600000184774398093168201739899814128875732421875f,-0.91060001569390269793302650214172899723052978515625f,0.32979999579936236653310288602369837462902069091796875f,0.182999978184700040628740680404007434844970703125f,-0.91559997888803490440068344469182193279266357421875f,0.590199986580014268611193983815610408782958984375f,0.547000008106231394577889659558422863483428955078125f,-0.976000013366341345744103819015435874462127685546875f,0.25540000489950143158779383156797848641872406005859375f,-0.632000011980533571431806194595992565155029296875f,-0.713799989771843002017703838646411895751953125f,-0.1763999959141016038444149671704508364200592041015625f,-1.220500016927719144632646930404007434844970703125f,0.5371000076174734516598618938587605953216552734375f,-0.3728999874278902115776190839824266731739044189453125f,0.3759999943301080005397807326517067849636077880859375f,0.058099997179210161046825078301480971276760101318359375f,0.04919999835938215770880788113572634756565093994140625f,-0.32449999722838374083266899106092751026153564453125f,-0.9672999754965305729825786329456605017185211181640625f,-0.1836999964073300350975159744848497211933135986328125f,0.0069999800920488297606425476260483264923095703125f,0.48199998649954789442517721909098327159881591796875f,0.77279996541589535130611920976662077009677886962890625f,-0.34099998497962946686357099679298698902130126953125f,0.88879999334812165301400455064140260219573974609375f,-0.42119998754113918693064988474361598491668701171875f,-1.115500005602836353091333876363933086395263671875f,0.283600016528367859791615046560764312744140625f,-0.3286999923840163706501016349648125469684600830078125f,0.1810000007748602257606762577779591083526611328125f,-0.0112000060558319347592259873636066913604736328125f,-0.45979999330341814189893057118752039968967437744140625f,0.26449997770786293216360718361102044582366943359375f,-0.85549996715784093481715899542905390262603759765625f,0.216299990814924403270680386413005180656909942626953125f,-0.8869999958872796241848845966160297393798828125f,-0.5079999942779540589299358543939888477325439453125f,-0.688999972701072938008337587234564125537872314453125f,1.04850001642108026089772465638816356658935546875f,0.23460001849532119422292453236877918243408203125f,0.61769996933192050647676296648569405078887939453125f,1.043999998927116479308097041212022304534912109375f,0.3863000066101545115770932170562446117401123046875f,-0.4465999820470809567041214904747903347015380859375f,1.15129999006986594167756265960633754730224609375f,0.370499996662139619019171732361428439617156982421875f,0.1583000127792359990053228102624416351318359375f,-2.20519997518062638164337840862572193145751953125f,0.1588000174880033199542594957165420055389404296875f,0.37890001133680328848640783689916133880615234375f,1.4957999516844751042299321852624416351318359375f,-0.15400000220537179984603426419198513031005859375f,0.9322000254571438659922932856716215610504150390625f,-0.0925000250339511609354303800500929355621337890625f,-0.52310001971721664659753514570184051990509033203125f,-0.1020000180900098030178924091160297393798828125f,-1.106100003415345867097130394540727138519287109375f,0.2791000078260896799520196509547531604766845703125f,-0.626299998712539718326297588646411895751953125f,-0.0180999976634979020673199556767940521240234375f,-0.356500006973742955551642808131873607635498046875f,-0.4924999980628488316369839594699442386627197265625f,-1.3479999489784244559587023104541003704071044921875f,-0.27100001090764980204994571977294981479644775390625f,-0.54379998083114600149201578460633754730224609375f,-0.87159996535778105908320867456495761871337890625f,-0.44600002908706670723404386080801486968994140625f,0.7747999765396116345073096454143524169921875f,0.3762999832153326451589236967265605926513671875f,-0.892700002300738759686282719485461711883544921875f,0.3417999795794486317390692420303821563720703125f,0.460400000131130582303740084171295166015625f,0.3972000145792957681578627671115100383758544921875f,0.39960002751052403624498765566386282444000244140625f,0.71160001393556537863105404539965093135833740234375f,0.103600001329183566411984429578296840190887451171875f,-0.14900001442432397880111238919198513031005859375f,-1.378899970209598535575423738919198513031005859375f,0.33690001127719870765986343030817806720733642578125f,0.709800025042891746807072195224463939666748046875f,0.9078000102311367669472019770182669162750244140625f,-0.36320001751482511309632172924466431140899658203125f,0.49070004575252568201904068700969219207763671875f,0.90490000972747708374299691058695316314697265625f,0.704500003039836286689023836515843868255615234375f,-1.135999989748000871259137056767940521240234375f,-0.52719998231828224533046522992663085460662841796875f,-0.16149999132752412833724520169198513031005859375f,0.8250999816298485445287269612890668213367462158203125f,0.051699996571242856635564066891674883663654327392578125f,-0.313500047564506445496590458787977695465087890625f,-1.14929998866915727973037064657546579837799072265625f,0.2258999703437094552782582468353211879730224609375f,-0.086999987542629497738744248636066913604736328125f,0.668699976849556065872093313373625278472900390625f,-0.4987999839976431371013632087851874530315399169921875f,-0.00399999535083761514897560118697583675384521484375f,-0.40910001125335693217266452847979962825775146484375f,-0.5544000035285947536323192252893932163715362548828125f,-0.2845000005066395942776580341160297393798828125f,-0.47330000324249255072572850622236728668212890625f,0.203500005953013651804894834640435874462127685546875f,0.93199998307228071325880591757595539093017578125f,-0.0079999941289425891000064439140260219573974609375f,0.5365999845430255987821510643698275089263916015625f,0.72799997878074673707260444643907248973846435546875f,-0.7120999725997450013892375864088535308837890625f,0.6469999696239832065458585930173285305500030517578125f,0.992999987721443488908334984444081783294677734375f,-0.5081999864459039173425480839796364307403564453125f,0.6405999639868740214154740897356532514095306396484375f,0.59500000655651064107587444595992565155029296875f,-0.2544999826252460906061969581060111522674560546875f,-0.44059997724890731518598840921185910701751708984375f,-0.776999978005886049459149944595992565155029296875f,0.31260000070333493482621634029783308506011962890625f,-0.4125999731361869038437362178228795528411865234375f,-0.436000003159045856904185711755417287349700927734375f,0.335200001332163655121121337288059294223785400390625f,-0.43520000915527301543761495850048959255218505859375f,-0.3240000096559523257155888131819665431976318359375f,-0.589499986797571207119972314103506505489349365234375f,0.27199999484419823403413829510100185871124267578125f,-0.0544999856799837090193250332958996295928955078125f,0.22040000504851331442068840260617434978485107421875f,-0.59610000308006971803109763641259633004665374755859375f,-1.292999993085860666042208322323858737945556640625f,0.62399998238682741202865145169198513031005859375f,-0.6624000102117653110411765737808309495449066162109375f,0.401999977707863109799291123636066913604736328125f,-0.6354000051975248197777546010911464691162109375f,-0.00359998896121993539054528810083866119384765625f,-1.74600004100799566231216886080801486968994140625f,-0.011200004267692520443233661353588104248046875f,0.280799985510110872155564720742404460906982421875f,-0.0632999826788900321616893052123486995697021484375f,-0.872100018346309457939469211851246654987335205078125f,-0.57639997944831833365242346189916133880615234375f,-0.9361999857902532085063285194337368011474609375f,-0.1396000213265420342167999478988349437713623046875f,1.2121000210285188813941203989088535308837890625f,0.4502000031948085734256892465054988861083984375f,-0.95900002425909036674056551419198513031005859375f,0.39990002820491810098246787674725055694580078125f,2.20860000267028766529620043002068996429443359375f,-0.274800009918212850834606797434389591217041015625f,-0.24649999497830865635705777094699442386627197265625f,0.95990000197887415112063536071218550205230712890625f,0.427100012236833581624750877381302416324615478515625f,0.2871000389099123140113078989088535308837890625f,0.880499995470046314949286170303821563720703125f,0.825700021016597673906289855949580669403076171875f,0.2145000224672257704838784775347448885440826416015625f,1.7699999640882013718368170884787105023860931396484375f,0.39500000257044998175359751257929019629955291748046875f,-0.08580003006458269965150975622236728668212890625f,-2.641199939894676163021358661353588104248046875f,0.1396000067234037800290025188587605953216552734375f,0.13230000783205042580448207445442676544189453125f,-1.765199988889694537874675006605684757232666015625f,0.09579999907016745197552154422737658023834228515625f,-0.328500003457069311707527958787977695465087890625f,0.87090002629756924079629243351519107818603515625f,0.25810000988245018760380844469182193279266357421875f,-0.62160001810789111686972319148480892181396484375f,0.043200021910667629754243534989655017852783203125f,0.82460002847909930068226458388380706310272216796875f,-0.701400014540553229380748234689235687255859375f,-0.8297999986678359363168056006543338298797607421875f,0.40500001855194600608456312329508364200592041015625f,-0.33989999631643286193138919770717620849609375f,0.375900017952918830133057781495153903961181640625f,0.73230001796483978893093080841936171054840087890625f,-0.623499971866607438641949556767940521240234375f,-0.559999988414347171783447265625f,0.026999978303908989119008765555918216705322265625f,-0.9312000266194342401604444603435695171356201171875f,0.3021999858945607631000029869028367102146148681640625f,0.93650000742077832427412431570701301097869873046875f,-0.3844999733865261948295710681122727692127227783203125f,-0.009099997358024136673293469357304275035858154296875f,0.61549998652935045129197533242404460906982421875f,-0.43220000221133236806281274766661226749420166015625f,0.6102999683216216997294623070047236979007720947265625f,0.35750000894069700052568805404007434844970703125f,-0.3232999858081342381410649977624416351318359375f,-0.758700001809000657004844470066018402576446533203125f,0.052000013470649975033666123636066913604736328125f,1.002500001490115977276218472979962825775146484375f,-0.6097999992266294189136033310205675661563873291015625f,-0.382999998450279210970847998396493494510650634765625f,0.703399994632601721633591296267695724964141845703125f,-0.0739999967291951055159415773232467472553253173828125f,-1.301000011205672990399762056767940521240234375f,0.6919999790489672619742123060859739780426025390625f,-0.535200008298456264554943118127994239330291748046875f,-0.68699998217821123258630677810288034379482269287109375f,1.0328999700121581639677970088087022304534912109375f,-0.5419999876171350816633776048547588288784027099609375f,-0.10500001192092867086103069595992565155029296875f,-1.33979996551275259975000153644941747188568115234375f,0.4825999902725219836696624042815528810024261474609375f,-0.2140000177025793703933231881819665431976318359375f,-0.931699999439716375349007648765109479427337646484375f,0.1833999823018910291239080834202468395233154296875f,-0.099999997913837290752780972979962825775146484375f,0.614800012153386976621050052926875650882720947265625f,0.03700000128895053475019949473789893090724945068359375f,-0.6215000051856041096698390902020037174224853515625f,0.424400004571676259246260087820701301097869873046875f,0.271700005958974255548810106120072305202484130859375f,-0.007000020995736377926732529886066913604736328125f,-0.166400001502037042655501863919198513031005859375f,0.567799994584917921969235976575873792171478271484375f,-0.7110000019669531212684887577779591083526611328125f,0.514799997550249077704620503936894237995147705078125f,0.0287999987497926657198377142776735126972198486328125f,1.189499986648559826107884873636066913604736328125f,0.357600009196996637683696462772786617279052734375f,1.656000033557415207496887887828052043914794921875f,0.040499980568885263210177072323858737945556640625f,-0.8315000245571138037803393672220408916473388671875f,-0.83230002198815355995975551195442676544189453125f,-0.931099984043836315095177269540727138519287109375f,0.6055000203549862902718814439140260219573974609375f,-0.8906000120803714281692009535618126392364501953125f,-0.66639998484998930194933564052917063236236572265625f,0.144199997986853123421724376385100185871124267578125f,1.172499980926513529766452847979962825775146484375f,-0.86170002059936479099633288569748401641845703125f,-0.48829999370574928008181814220733940601348876953125f,0.17510001918077477967017330229282379150390625f,1.003599993133544643342247582040727138519287109375f,-0.07340000901222243356869512354023754596710205078125f,0.9184000403046610472301836125552654266357421875f,0.64579999579191138536771177314221858978271484375f,-0.571000018060207281678231083787977695465087890625f,0.2430999677121636892707101651467382907867431640625f,1.7218999868392952379281268804334104061126708984375f,0.27730003226995503240459584048949182033538818359375f,-0.0157999996662141484193853102624416351318359375f,0.20989997098445911660746787674725055694580078125f,0.6715000317096713189357615192420780658721923828125f,-0.8756000143527984391766949556767940521240234375f,-0.61839996907711025642129243351519107818603515625f,0.8752000204801557714517912245355546474456787109375f,-0.5743000110954046277811357867904007434844970703125f,1.28500001952052134157611362752504646778106689453125f,0.2984000112473965682369225760339759290218353271484375f,-1.28700000214576704138380591757595539093017578125f,0.22440004703998539525855449028313159942626953125f,0.105599974417685871230787597596645355224609375f,-1.61530003161430357749850372783839702606201171875f,0.60190005896091491877086809836328029632568359375f,0.4495000107884408180325408466160297393798828125f,-1.0629999847412108238131622783839702606201171875f,-0.49579998118877455226538586430251598358154296875f,0.7778000053882596631638079998083412647247314453125f,0.190200008559227029536486952565610408782958984375f,0.591599998438358198882269789464771747589111328125f,0.193800012421608158774688490666449069976806640625f,0.48969998030662509336252696812152862548828125f,0.660700004513561367502916255034506320953369140625f,-0.1364999911785123032359479111619293689727783203125f,-0.5847000029563904632823323481716215610504150390625f,-1.040500001169740951212361323996447026729583740234375f,-0.172500014156103276263820589520037174224853515625f,-0.1265999966502189266748246154747903347015380859375f,0.6380999972164629330251273131580092012882232666015625f,0.47449999272078269285657370346598327159881591796875f,-0.130999995708465544197451890795491635799407958984375f,0.733699989143013819870020597591064870357513427734375f,-0.223499998986721237770325387828052043914794921875f,-0.853199992406368235009495037957094609737396240234375f,-0.60649997888505480236887024148018099367618560791015625f,-0.0389999908804892214675419381819665431976318359375f,0.474299992394447311738758799037896096706390380859375f,-0.2832000013098119328702750863158144056797027587890625f,0.06499999731779126932451617904007434844970703125f,-0.0302000008106233508442528545856475830078125f,-0.659800013419985287299596166121773421764373779296875f,-0.55349998497962982213493887684307992458343505859375f,0.96180000565648082755387804354541003704071044921875f,-0.5296999594450000348189178112079389393329620361328125f,-0.870500015139580085588022484444081783294677734375f,0.34249997973442081189432428800500929355621337890625f,-0.1668999906763435348011626047082245349884033203125f,-0.496000007629394445984871708787977695465087890625f,0.13960000374317171889515520888380706310272216796875f,0.55919999482035631643839224125258624553680419921875f,0.027000007286667848660499657853506505489349365234375f,0.89670000949800010570100994300446473062038421630859375f,0.137600007595121809345073415897786617279052734375f,0.814000016808510196142378845252096652984619140625f,0.4070000122785568663630328956060111522674560546875f,0.8799999814480543580685889537562616169452667236328125f,0.604000001311302270323722041212022304534912109375f,0.7886000284492968859240136225707828998565673828125f,0.9825999959722160337349805558915250003337860107421875f,-0.230000015497207499493015347979962825775146484375f,0.20839998844861984395038234652020037174224853515625f,0.695999974921345909706360544078052043914794921875f,-0.3985000055432319499004734097979962825775146484375f,0.34299999800324432186471312888897955417633056640625f,-0.28089999802261578221163063062704168260097503662109375f,-1.15400002396106771129780099727213382720947265625f,0.002199995990097403097962569518131203949451446533203125f,-0.276399993455409997977767488919198513031005859375f,0.031899995779991030531164142303168773651123046875f,0.5283000086069105094566111802123486995697021484375f,-1.3874999938905236973596402094699442386627197265625f,-0.0164999446868900889739961712621152400970458984375f,0.81350001120567316803544599679298698902130126953125f,-0.67849997872114187202896573580801486968994140625f,1.639699974942207205685917870141565799713134765625f,0.10729999710321447281557993846945464611053466796875f,-0.0258000348329545658998540602624416351318359375f,-1.5662999969243998066303902305662631988525390625f,-0.078300003242492977051369962282478809356689453125f,0.640999984830617819397957646287977695465087890625f,-0.7982000046253201475110472529195249080657958984375f,0.058200021910667754099222293007187545299530029296875f,-0.6043000049114226612800848670303821563720703125f,-0.76039996764659889549875515513122081756591796875f,-0.5552000130295748903108687954954802989959716796875f,0.1618000096797942433113348670303821563720703125f,0.487999977886676816751787555404007434844970703125f,-0.53740000674724530682624390465207397937774658203125f,-0.67430003471374533319249167107045650482177734375f,-1.57719999401569310748527641408145427703857421875f,0.252600015902519459132236079312860965728759765625f,0.34870001738071465524626546539366245269775390625f,1.328300030064582415434415452182292938232421875f,0.4491000276446346362035910715349018573760986328125f,-0.91900001025199884452376863919198513031005859375f,0.06280003697872160728365997783839702606201171875f,0.80920002536773694146177149377763271331787109375f,-0.921400004184246057548079988919198513031005859375f,0.815500015735626249124834430404007434844970703125f,0.309699999082088339719120995141565799713134765625f,0.76810004206895854395042988471686840057373046875f,0.88669999705552982049994170665740966796875f,-0.487500019371509552001953125f,-0.2558000315546991032533696852624416351318359375f,-1.89669999467134431370141101069748401641845703125f,0.349100013637542616606879164464771747589111328125f,0.3943000114679335865730536170303821563720703125f,-1.09480003614425669411502894945442676544189453125f,-0.2646000210285190945569411269389092922210693359375f,1.04380001808404898611115640960633754730224609375f,0.0357000006578862549844188833958469331264495849609375f,-0.4885000016689300395000827847979962825775146484375f,0.2502999866127968431328554288484156131744384765625f,0.08429999772161245186197220391477458178997039794921875f,-0.03900001621246307337287362315692007541656494140625f,0.53880001063346849576873864862136542797088623046875f,0.8439999731853606856901706123608164489269256591796875f,0.3529999969601629317139668273739516735076904296875f,1.040799979251623152975980701739899814128875732421875f,0.193999992780387486845938838087022304534912109375f,0.1305000097751616294772247783839702606201171875f,-0.8540999921798704264119805884547531604766845703125f,0.302100011752545416499060593196190893650054931640625f,-0.54700001183152213712901357212103903293609619140625f,-0.5299999929964542833005225475062616169452667236328125f,-0.3449999877810479009809796480112709105014801025390625f,1.180499980568885831644365680404007434844970703125f,-0.47429997496008891033625332056544721126556396484375f,0.68909997683167478754739931900985538959503173828125f,0.5824999904632566227746792719699442386627197265625f,0.045399997299909511383475546608678996562957763671875f,-0.0871000034078954765703883822425268590450286865234375f,0.176999984264373466658071265555918216705322265625f,-0.620699993002414629472696105949580669403076171875f,0.7076000014111396563265543591114692389965057373046875f,-0.85500001192092867086103069595992565155029296875f,0.79020001317858667988502929802052676677703857421875f,-0.5764000009432430626787891014828346669673919677734375f,-0.381000012993812475770027958787977695465087890625f,0.4878000098586081634266520268283784389495849609375f,0.4889999934136868287026800317107699811458587646484375f,0.2720000069141388365778766456060111522674560546875f,0.71890000075697901849025583942420780658721923828125f,0.46879999662637705792889164513326250016689300537109375f,0.065500000536442115617319359444081783294677734375f,0.155599997067451312204866553656756877899169921875f,-0.6087000139534468612367845707922242581844329833984375f,-0.25800000709295289880174095742404460906982421875f,-0.839699996548890936765019432641565799713134765625f,0.435600005412101420887438507634215056896209716796875f,0.061000005245208654969246708787977695465087890625f,-0.74890002266168576028348979889415204524993896484375f,-0.045600003884732842607263592071831226348876953125f,0.4204999999403948862664037733338773250579833984375f,-0.2297000126421451238201143496553413569927215576171875f,-0.31180000789165518426671042107045650482177734375f,-1.1228000322103497410353156737983226776123046875f,-0.7368000293493270902445146930404007434844970703125f,0.670700007046759250073364455602131783962249755859375f,-0.192499988973140911951986709027551114559173583984375f,-0.3201000001817939644155330825014971196651458740234375f,1.49090000126361843513222993351519107818603515625f,-0.5543000376939772877449286170303821563720703125f,-0.3459000212311753585936457966454327106475830078125f,0.049599999328702626399945074808783829212188720703125f,-0.5638000083610414936430288435076363384723663330078125f,0.336000020630657980635902504218393005430698394775390625f,1.191300003778934524234500713646411895751953125f,-1.216200006055832005813499563373625278472900390625f,-0.09919998483657810339764182572253048419952392578125f,1.50719997851848575010080821812152862548828125f,-0.535500041365622792000067420303821563720703125f,-0.240099961960314800535343238152563571929931640625f,-0.21059995594024627507678815163671970367431640625f,-0.78449998843669987991233938373625278472900390625f,-0.885700060057640570221337839029729366302490234375f,1.1168000078916549000496161170303821563720703125f,0.46509995868206033264868892729282379150390625f,-0.70250002264976529886553180404007434844970703125f,0.233100012266636014146570232696831226348876953125f,0.27750000387430162618329632095992565155029296875f,0.31570000015497168277534001390449702739715576171875f,0.081699996608495695227247779257595539093017578125f,-0.419800002020597418095348984934389591217041015625f,-0.01580000249743473972330320975743234157562255859375f,0.96399999713897699393783113919198513031005859375f,-1.0630000145435332115084747783839702606201171875f,-0.1699999818205828461259443429298698902130126953125f,0.0522999950170515148784033954143524169921875f,0.634099976086616834436426870524883270263671875f,0.952500055730343575532970135100185871124267578125f,-0.73970000981092454139798064716160297393798828125f,-0.322899978971481260714426753111183643341064453125f,0.56550002020597478491481524542905390262603759765625f,-0.12730003204643747238833384471945464611053466796875f,-1.54700000512599888935483249952085316181182861328125f,-0.384600006276369033031414801371283829212188720703125f,-0.197000031650066631527806748636066913604736328125f,0.1406999997079372288766307974583469331264495849609375f,1.2039999995231625007363618351519107818603515625f,-0.78580000979900344049156046821735799312591552734375f,0.5755999926716086623201817928929813206195831298828125f,-0.0760000061392782644276167047792114317417144775390625f,-0.3421999871239067569916869615553878247737884521484375f,-0.389400002373754772211356112165958620607852935791015625f,-0.6479999632835387046725372783839702606201171875f,-0.70380001733899089089163680910132825374603271484375f,-0.9217999752953651704245885412092320621013641357421875f,-0.47450001937523500483706584418541751801967620849609375f,-0.1954000055514276412704788299379288218915462493896484375f,-0.01170000109747049155117792906821705400943756103515625f,-0.7460000168681144572246921597979962825775146484375f,-0.913299995493888960851336378254927694797515869140625f,-0.6425999966427682341674199051340110599994659423828125f,-0.79900001978874257702045724727213382720947265625f,-0.6610999971568585209524826495908200740814208984375f,-0.95939996865987797747266085934825241565704345703125f,1.322500001192093321833453956060111522674560546875f,-0.676300003182888076480594463646411895751953125f,0.4756999558612708955962489199009723961353302001953125f,0.09150002455711359061751863919198513031005859375f,-0.209700012493133414182011620141565799713134765625f,-0.746099982590973542784240635228343307971954345703125f,-0.588499990344047319013043306767940521240234375f,0.208999992966651948478329359204508364200592041015625f,-0.28469999900758258259969579739845357835292816162109375f,-0.0374999977648258209228515625f,-0.244500001102685882159448738093487918376922607421875f,0.01510000115036957968328579227090813219547271728515625f,-0.48550002110004442101853783242404460906982421875f,-0.81139999628663073139023254043422639369964599609375f,-0.4874999945238231990884969491162337362766265869140625f,-1.29200002658367196772815077565610408782958984375f,0.6499999855458735709134998614899814128875732421875f,-0.34859998415559534379326578346081078052520751953125f,-0.1920000161528587767634235206060111522674560546875f,0.165000005215406364555974505492486059665679931640625f,0.475399993164837386672161301248706877231597900390625f,0.607499998807906838038661589962430298328399658203125f,-0.927899991115927580409561414853669703006744384765625f,0.43810000571012464032349953413358889520168304443359375f,-0.6139999822378161553615427692420780658721923828125f,0.5509000091612339389257613220252096652984619140625f,-0.435100009793043085437602712772786617279052734375f,1.2541000276446343519864967674948275089263916015625f,0.175700008797645690350464064977131783962249755859375f,0.8119000154495239485186175443232059478759765625f,1.11739997754096975768334232270717620849609375f,-0.396600005590915571929144789464771747589111328125f,-0.6020999976336955938904793583787977695465087890625f,0.0600000080466269736234607989899814128875732421875f,0.09479999889135370949588832445442676544189453125f,1.061699960100650486083395662717521190643310546875f,-0.290700025486945179409303818829357624053955078125f,-0.343899974381923101418578880839049816131591796875f,-0.02240000078678150430278037674725055694580078125f,-1.2223000415086744396830908954143524169921875f,-0.5805000291466715367505457834340631961822509765625f,0.0407999816358088907009005197323858737945556640625f,-1.1408999869585034758756592054851353168487548828125f,-0.49650000280141792785570942214690148830413818359375f,0.653000021696090726663896930404007434844970703125f,-0.05060003342628505151878925971686840057373046875f,-0.69940003929138239158191936439834535121917724609375f,0.1119000115379691351336077786982059478759765625f,0.58709999156147230081614907248876988887786865234375f,-0.371300010670721380989789395243860781192779541015625f,-1.569000053167343367022112943232059478759765625f,0.1953000363826760121810366399586200714111328125f,0.778700035560131453848953242413699626922607421875f,0.05069999825507398583113172207958996295928955078125f,-0.681600004249811153300697696977294981479644775390625f,0.57100002402067195905743801631615497171878814697265625f,0.668900010293722147025619051419198513031005859375f,0.0918999776005744450912970933131873607635498046875f,-1.069100034201145188461623547482304275035858154296875f,0.5368000059545039448494208045303821563720703125f,0.42289997286200531334543484263122081756591796875f,-0.40330000756382933246868560672737658023834228515625f,-0.676700034084916524079744704067707061767578125f,-1.3828000306457273183013967354781925678253173828125f,0.75540004177987629674362324294634163379669189453125f,0.327000012278556795308759319595992565155029296875f,-1.5702000222682954699848778545856475830078125f,-0.29280001045465464670769506483338773250579833984375f,-1.187499977648258209228515625f,0.80869998087286953936114741736673749983310699462890625f,0.08500000730156909156676192651502788066864013671875f,0.8114000158071517088131940909079276025295257568359375f,0.0672000059738755428639933597878552973270416259765625f,-0.08749999105930328369140625f,0.3004999947249888947453655418939888477325439453125f,-0.699299990233778867576575066777877509593963623046875f,0.32149999237060544743371792719699442386627197265625f,0.181900000250339477503302987315692007541656494140625f,0.409699989247322149399366253419429995119571685791015625f,-0.3339999995231632823333711712621152400970458984375f,-0.47899998745322225346399136469699442386627197265625f,-0.65339996211081785570939928220468573272228240966796875f,0.78500000417232485006024944595992565155029296875f,-0.84230001930594422532294629490934312343597412109375f,0.3477999952554700069384807648020796477794647216796875f,0.62749998748302449058655838598497211933135986328125f,-0.78289997971653946251535671763122081756591796875f,0.269899996130168406960336824340629391372203826904296875f,0.434000021874904717833487666212022304534912109375f,-0.392000017940997924625889936578460037708282470703125f,-0.32000000201165679225567828325438313186168670654296875f,0.4010000003650784350384128629229962825775146484375f,0.16419999899268145071573599125258624553680419921875f,-0.1028000038236378532729986545746214687824249267578125f,0.13999999344348879048993694595992565155029296875f,0.6751000211179256638160950387828052043914794921875f,0.94800001275539358402966172434389591217041015625f,-0.7830000156983734083127046687877736985683441162109375f,-0.178500007592141629597648488925187848508358001708984375f,-0.042399999147653577580285855219699442386627197265625f,0.90650001189112661137414761469699442386627197265625f,-0.4447999993383884298481234509381465613842010498046875f,-0.2773000007539987432636507946881465613842010498046875f,0.1290000056326388033767216256819665431976318359375f,-0.017100007468461964066364089376293122768402099609375f,-0.3785999920904636883989269335870631039142608642578125f,-0.482000029489398684745538048446178436279296875f,-0.34630001540184007779998864862136542797088623046875f,0.56940001064389900164997015963308513164520263671875f,0.1504999832510947044283966533839702606201171875f,-0.998000011712312673495262060896493494510650634765625f,0.09049998567253358938700102953589521348476409912109375f,0.6155000035166739991154827293939888477325439453125f,0.5447000372290613512404888751916587352752685546875f,0.4420999844461679373353035771287977695465087890625f,-1.13719998760819418492928889463655650615692138671875f,-0.1145999907791612759666577403550036251544952392578125f,0.284899996465444615978412912227213382720947265625f,0.2326000072598457535377747262828052043914794921875f,-0.578800022852421047758753047673963010311126708984375f,-0.453700003224611236873897723853588104248046875f,0.7735000204443931437481296597979962825775146484375f,0.807100028777122613377059678896330296993255615234375f,0.57950000520050526375825938885100185871124267578125f,0.96099998781085016474889926030300557613372802734375f,-0.096999997377395619224671463598497211933135986328125f,0.46339999768733974860879243351519107818603515625f,1.8645999799013139863745891489088535308837890625f,-0.4788000288128859693870253977365791797637939453125f,1.30350000107288366280045011080801486968994140625f,0.7256999933004379954581963829696178436279296875f,-1.1867000250697135044219976407475769519805908203125f,-0.7432999764949084209320062655024230480194091796875f,1.65739996457695948350874459720216691493988037109375f,0.0919999897778034370077193671022541821002960205078125f,-0.3931000162899493943768902681767940521240234375f,-0.9517999752581118855232489295303821563720703125f,0.62160001199841463659367946092970669269561767578125f,-0.048899998074769968070540926419198513031005859375f,-0.41849999943375593147720792330801486968994140625f,-0.0133000004112719949489473947323858737945556640625f,0.58259998077303176700070252991281449794769287109375f,-0.645599974492192085762098940904252231121063232421875f,-0.71620001238882535776753002210170961916446685791015625f,0.41740000812560307252851998782716691493988037109375f,-1.020700035694241503136936444207094609737396240234375f,0.24450001876056226546296556989545933902263641357421875f,0.025499986827373533060381305404007434844970703125f,1.176000021934509476295716012828052043914794921875f,0.1750000172853471980261019780300557613372802734375f,-0.107400013005733541149311349727213382720947265625f,1.585200011539459552523112506605684757232666015625f,0.0700000238418585496447121840901672840118408203125f,0.746200022149086095168968313373625278472900390625f,0.519199985730647739501364412717521190643310546875f,-0.549800015878677328373669297434389591217041015625f,-0.2514999985545873784076320589520037174224853515625f,-0.3707999642014507823972735423012636601924896240234375f,-0.171000008746981624341287897550500929355621337890625f,-0.72989998476803308324889485447783954441547393798828125f,-0.10519999615401032233563682893873192369937896728515625f,0.54700001928210273405284169712103903293609619140625f,-0.1284999890029430158477907752967439591884613037109375f,-0.124799995948374320420271033071912825107574462890625f,-0.72700000616908067030408346909098327159881591796875f,0.7738000022888182893865405276301316916942596435546875f,0.02840000346153981336527749590459279716014862060546875f,-0.307499989420175534604595668497495353221893310546875f,0.364499997451901425193909744848497211933135986328125f,-0.56649998678266999529995473494636826217174530029296875f,0.2745000510215760414212127216160297393798828125f,0.73799999353289624792751055792905390262603759765625f,-0.5542999737307432450705846349592320621013641357421875f,0.5870000212192536537259002216160297393798828125f,-0.27199998173117645450247437111102044582366943359375f,-0.8239999851062893565512013083207421004772186279296875f,0.421000003382563381393310919520445168018341064453125f,1.0576999710828065559553579078055918216705322265625f,0.2415999933347106054526420848560519516468048095703125f,-0.5189999987781046542067997506819665431976318359375f,-0.218099995577335281637942898669280111789703369140625f,0.3172000005349515650010516765178181231021881103515625f,0.0559999931752681590069187222979962825775146484375f,-0.21740000033974649085166674922220408916473388671875f,0.10979999430924658465613674707128666341304779052734375f,0.5115000082030893846507524358457885682582855224609375f,-0.700199969704449198815154886688105762004852294921875f,-0.16369999391138556621427824211423285305500030517578125f,-0.6245000246092680118437101555173285305500030517578125f,-0.5413000200584530130498706057551316916942596435546875f,-0.1833000014543533229272043172386474907398223876953125f,-0.2549999934434890036527576739899814128875732421875f,0.6795000170469283062857357435859739780426025390625f,-0.25599998553842306936445538667612709105014801025390625f,-0.0210000040531159726242549368180334568023681640625f,0.8874000162839887906329749966971576213836669921875f,-0.20619998579025267115838460085797123610973358154296875f,0.1655000041127205889779361314140260219573974609375f,-0.05660000275969512273377404198981821537017822265625f,-0.51690000237375477443180216141627170145511627197265625f,-0.427500004395842569948626987752504646778106689453125f,-0.0370999979317190309302532114088535308837890625f,0.54550001264363512376576181850396096706390380859375f,-0.212200046840310019291564458399079740047454833984375f,-0.75080001195967149829613163092290051281452178955078125f,-0.6174000148683784772174476529471576213836669921875f,1.06020005341172218749079547706060111522674560546875f,-0.0330000108182433304904179749428294599056243896484375f,0.231300017786026046451297588646411895751953125f,0.5315999850273129823108320124447345733642578125f,0.659000016212463179954283987171947956085205078125f,-0.648999995946883956321471487171947956085205078125f,-0.628699955093861007071609492413699626922607421875f,0.375700003135204241289102355949580669403076171875f,-0.128200005780160442014903310337103903293609619140625f,-0.484099994266033117895631221472285687923431396484375f,-0.0272999998599290361056546316831372678279876708984375f,-1.361399997925758498240611515939235687255859375f,0.93479998011589060524784144945442676544189453125f,0.1533999753355974604573930264450609683990478515625f,0.75360001876354232308585778810083866119384765625f,-0.1848000021696094563594670034945011138916015625f,0.0769999994635579554369542165659368038177490234375f,-0.25300004620850113923324897768907248973846435546875f,-1.77830000085830608469450453412719070911407470703125f,-0.387700001555681073028836181038059294223785400390625f,-1.2331000310420989762860699556767940521240234375f,-0.97009994676113109335346962325274944305419921875f,0.9559000218272206694791748304851353168487548828125f,0.9043999868392944563311175443232059478759765625f,-0.552499998509883738506687222979962825775146484375f,-0.07370000456571634828151218243874609470367431640625f,1.21499998867511749267578125f,-0.5003999998331067899925983510911464691162109375f,-0.69560000004768340886585065163671970367431640625f,-0.759799981941282620567790218046866357326507568359375f,0.111500014722346829643129240139387547969818115234375f,0.3398999943047760208258978309459052979946136474609375f,0.027800006878376137819941504858434200286865234375f,-1.919299983453750968465101323090493679046630859375f,-0.2801000212669375599716659053228795528411865234375f,-1.128299963605403632982415729202330112457275390625f,0.251100025916098701372902723960578441619873046875f,0.51429998673200572056884993799030780792236328125f,-0.8379999869763850028903107158839702606201171875f,-0.40219998183399463531628725831978954374790191650390625f,0.5819999871328469520648241086746565997600555419921875f,-0.474300013144314303037418767416966147720813751220703125f,-0.2030000026971101600992142266477458178997039794921875f,0.495000023469329253344994867802597582340240478515625f,0.67320002824366087512686362970271147787570953369140625f,0.277800001886487013535997903090901672840118408203125f,-1.03900000011920923270736238919198513031005859375f,0.4608999795079231631689253845252096652984619140625f,-0.39959999618083230643605929799377918243408203125f,-0.0085000126957890387302541057579219341278076171875f,-0.07969998626708996170009413617663085460662841796875f,0.50149997884780195267495628286269493401050567626953125f,0.1334999954104423292022829627967439591884613037109375f,-0.2414999957606196456794123150757513940334320068359375f,0.181899996934831109651753422440378926694393157958984375f,0.802500020861625529278171597979962825775146484375f,0.81539997554421450587369690765626728534698486328125f,0.803299994637071801406591475824825465679168701171875f,-0.658499998092651139813824556767940521240234375f,-0.21960000568032267409535052138380706310272216796875f,-0.5357000013098118795795699043083004653453826904296875f,0.705999998092651015468845798750407993793487548828125f,-1.027299984809756150383464046171866357326507568359375f,0.57000001043081238716325742643675766885280609130859375f,-0.335000004768371439922702847979962825775146484375f,0.05000001654028896069803522550500929355621337890625f,0.8631999955728648643571432330645620822906494140625f,-0.5704999758005140364502949523739516735076904296875f,-0.495699995386600367197615923942066729068756103515625f,-0.5512999804213645216321992847952060401439666748046875f,0.01500000059604644775390625f,-0.73979997981786738137088832445442676544189453125f,-0.65499998629093170166015625f,0.1164999932646753055820454392232932150363922119140625f,0.3232999932214617455628058451111428439617156982421875f,0.464099979104101889593181340387673117220401763916015625f,0.3574999922513963923620394780300557613372802734375f,-1.1206999947905540437886884319595992565155029296875f,0.36589997790604844585260480016586370766162872314453125f,0.1599999904632571912088678800500929355621337890625f,0.52879998843073838799000441213138401508331298828125f,0.67259996311515612976705824621603824198246002197265625f,0.907999968498945264627764117904007434844970703125f,-0.5275000277161598916109142010100185871124267578125f,-0.25600001657009130440201261080801486968994140625f,-1.0378999938726423124535358510911464691162109375f,-0.6151000296115878285263534053228795528411865234375f,-0.3106000167369842301923199556767940521240234375f,-0.908599968099593979786732234060764312744140625f,-0.830600016736983803866678499616682529449462890625f,0.281100000286102158497669734060764312744140625f,-1.48020002644062032004512730054557323455810546875f,-0.0225999881863589990871332702226936817169189453125f,0.3970000031888485381159625831060111522674560546875f,0.3229999993443488648381389793939888477325439453125f,-0.29000000745058063245096491300500929355621337890625f,-0.35590000334978100227090180851519107818603515625f,-0.423799990665912673648563213646411895751953125f,-0.506000034749508387221794691868126392364501953125f,-0.68240000466108341470317100174725055694580078125f,0.802999975204468086076303734444081783294677734375f,-0.21280000866651516133742916281335055828094482421875f,1.32470001070499421302884002216160297393798828125f,0.5355999660730361711102887056767940521240234375f,-1.0302000395536428101195269846357405185699462890625f,-0.652000023864209854451701176003552973270416259765625f,-0.345200007963180066639097276492975652217864990234375f,0.2272000043720006079439599488978274166584014892578125f,-0.0566999942988157812351346365176141262054443359375f,0.23339998323321342610370265902020037174224853515625f,0.2132000050723552586617870474583469331264495849609375f,0.00460000314712516455983859486877918243408203125f,1.7326999865055086047505028545856475830078125f,-0.1344999860525124546484221355058252811431884765625f,-0.22010001724362382446997798979282379150390625f,0.067700002151728000399089069105684757232666015625f,-0.3969000126183033216875628568232059478759765625f,-1.153899998819827743545829434879124164581298828125f,0.063100078129767922519022249616682529449462890625f,0.8931000289559367644187659607268869876861572265625f,1.48250005364418058206865680404007434844970703125f,0.3558999744415274335551657713949680328369140625f,-0.9509000444769863946703480905853211879730224609375f,0.2080999952793121110516949556767940521240234375f,0.3895000009536744300930877216160297393798828125f,0.52370002930164361032439046539366245269775390625f,0.06920004563331616509458399377763271331787109375f,0.63439998408257947115629349355003796517848968505859375f,1.00250001251697540283203125f,-0.71029999048709857589756211382336914539337158203125f,0.4944999959245321985434884481946937739849090576171875f,1.324999988079071044921875f,-0.7039999673366548194053393672220408916473388671875f,0.558199985849857416297936651972122490406036376953125f,-0.170500027060509040666147484444081783294677734375f,-0.84950001034140587563570079510100185871124267578125f,0.051299994391202918819772094138897955417633056640625f,0.08450000780820854373587280861102044582366943359375f,-0.0043999957799911726397112943232059478759765625f,-0.283499995969235862869339825920178554952144622802734375f,0.930000018179416798602687777020037174224853515625f,-0.3272999953150748098096300964243710041046142578125f,0.4397999916270374232141193715506233274936676025390625f,0.7805000029206272671444821753539144992828369140625f,0.488199989277124490882897589472122490406036376953125f,0.407300001536309519423184610786847770214080810546875f,0.3670000373125077430813689716160297393798828125f,-0.32410000678300843190982050145976245403289794921875f,-0.8403999832555653082266644560149870812892913818359375f,0.1870000042319295285864200195646844804286956787109375f,0.0211999952152372994618190205073915421962738037109375f,0.487000007957219704923801373297465033829212188720703125f,-0.239000001534819705995005278964526951313018798828125f,0.20749998681247239229463730225688777863979339599609375f,-0.04359999831169847350764712246018461883068084716796875f,0.163499993324279557782574556767940521240234375f,0.87450000154972062915703645558096468448638916015625f,-0.2696999912962316781062099835253320634365081787109375f,0.683500011354684744446785771287977695465087890625f,-0.05559999080896371737026129267178475856781005859375f,0.233700005981325897863598584081046283245086669921875f,-0.3170000277757640816389539395458996295928955078125f,0.5624000283539292155410294071771204471588134765625f,0.64279997472912120404231473003164865076541900634765625f,0.14700001585483590815783827565610408782958984375f,-0.42360001414418224641167398658581078052520751953125f,-0.9420000126883383284592810014146380126476287841796875f,-0.84350000762939458809341886080801486968994140625f,0.34399999982118600883040926419198513031005859375f,-0.329699988725781434339978659409098327159881591796875f,0.09099999511241918526138761080801486968994140625f,-0.68679999459236829029151749637094326317310333251953125f,0.622500020861625813495265902020037174224853515625f,0.687900003707408558284441824071109294891357421875f,0.169100020790100558087942772544920444488525390625f,-0.151300012123584792789188213646411895751953125f,-0.32449998873472196692091529257595539093017578125f,0.600900033748150264045762014575302600860595703125f,-0.22840000945925709174844087101519107818603515625f,0.3625000211596489663179454510100185871124267578125f,0.493700019466877026985685006366111338138580322265625f,-1.251300028514862106021610088646411895751953125f,1.271700038480759076264803297817707061767578125f,0.09170000182390225518247461877763271331787109375f,-0.0418999991327524412554339505732059478759765625f,-0.2068000039428472547342607867904007434844970703125f,0.27300001059472567277452981215901672840118408203125f,0.4770000180900098030178924091160297393798828125f,1.067999995023011905459497938863933086395263671875f,-0.6630000251233578723031314439140260219573974609375f,-0.48569997661113717413172707892954349517822265625f,-1.8713999728918082610107376240193843841552734375f,0.5040000054836273335467922152020037174224853515625f,0.20230000086575739004501883755438029766082763671875f,-0.809500015482306434222437019343487918376922607421875f,0.20010001508295560679329128106473945081233978271484375f,0.79210000463724128394460421986877918243408203125f,0.571399969017505782176158390939235687255859375f,-0.482200005936622488889042870141565799713134765625f,0.086500021576881636065081693232059478759765625f,0.91100001835823007922954275272786617279052734375f,0.7350000128149982714376164949499070644378662109375f,-0.53260000047981748139136470854282379150390625f,-0.0320000068396328885000912123359739780426025390625f,-0.19520000148117500060607198975048959255218505859375f,-0.39399998939037317313704988919198513031005859375f,-1.2496999999761584376756218262016773223876953125f,-0.5073000391244892881559280795045197010040283203125f,-0.9079999984502791221530060283839702606201171875f,-0.69249997258186368753740680404007434844970703125f,0.6704999963641160576344191213138401508331298828125f,0.10659999706000089503277195035479962825775146484375f,-0.383199997323751428979221600457094609737396240234375f,0.08240000644922262917901889522909186780452728271484375f,-0.385200001406669656489611952565610408782958984375f,-1.913600027108191881097809528000652790069580078125f,0.09549999740719794516508045489899814128875732421875f,-0.246699998620152516348724702766048721969127655029296875f,0.030100002491474153742956332280300557613372802734375f,-0.184500001586973549905223990208469331264495849609375f,0.65889997214674977943360545395989902317523956298828125f,1.500499989435076120258827359066344797611236572265625f,-0.68409999143481226013818741193972527980804443359375f,0.58869997766911996706085119512863457202911376953125f,1.09310000839233367742053815163671970367431640625f,0.3803999950647352079613483510911464691162109375f,-0.0404999904409051314502221430302597582340240478515625f,-0.5349000022768972684161781216971576213836669921875f,0.39469999893307683436205479665659368038177490234375f,-0.60049999658763397736294109563459642231464385986328125f,-1.02970002262592341679692253819666802883148193359375f,0.049199999178946018929536876385100185871124267578125f,0.50599999824166308481920850681490264832973480224609375f,-0.26480000924766067260662794069503434002399444580078125f,-0.42479999293088877010404758038930594921112060546875f,-0.3395000109001993227053617374622263014316558837890625f,0.2860999901533123335184427560307085514068603515625f,-0.44999997444450878791855075178318656980991363525390625f,-0.2019999947696924369466842108522541821002960205078125f,-0.5468000057309865269417059607803821563720703125f,0.2280999955028295556047623904305510222911834716796875f,-0.042599997611343809467143728397786617279052734375f,-0.3570000096708536574396930518560111522674560546875f,-0.09049999278783804612658059340901672840118408203125f,-0.09309999162852766740883225793368183076381683349609375f,-0.42590000841617570159769456950016319751739501953125f,-0.223299995642900483971970970742404460906982421875f,0.435499985337257589090853571178740821778774261474609375f,1.110699988234042923096467347932048141956329345703125f,0.663299986553191889271374748204834759235382080078125f,-0.4095999921575188551514656865037977695465087890625f,0.22479998667240153054081019945442676544189453125f,-0.38820000194311088392851161188445985317230224609375f,-0.223199993300438048748901564977131783962249755859375f,-0.20520000185370435019649448804557323455810546875f,0.31170000167489053666258769226260483264923095703125f,-0.61810001614093745825329051513108424842357635498046875f,-0.18170003550052715723950313986279070377349853515625f,-0.23229998220205327896792368846945464611053466796875f,-0.21319999464154248958180914996773935854434967041015625f,-0.168700001659989329283462211606092751026153564453125f,-0.12740000615119928539797911071218550205230712890625f,-0.00280000117868184350555793571402318775653839111328125f,-0.5846999956548213361884336336515843868255615234375f,-0.3037999961793425018896641631727106869220733642578125f,0.34669998915791511162609594975947402417659759521484375f,-0.1704000218868255700499503291212022304534912109375f,-0.46960004636049301751654638792388141155242919921875f,-0.408499995447695329886528270435519516468048095703125f,0.55790001637339603490772788063623011112213134765625f,0.2726000038325786523074611977790482342243194580078125f,-0.3671000056803226385682137333787977695465087890625f,0.7937999677181242219603518606163561344146728515625f,0.578799973082542607016875990666449069976806640625f,-0.4845999818384647550573163243825547397136688232421875f,0.146800005507468966925443965010344982147216796875f,-0.00119999413490301520823777536861598491668701171875f,-0.10979999680519103577580608543939888477325439453125f,0.29370001142024992901724544935859739780426025390625f,0.17640000239610653665067729889415204524993896484375f,-0.20790000814050435540281114299432374536991119384765625f,0.1842000015258788980787585387588478624820709228515625f,0.0399999953806400299072265625f,-0.1911999993503093742130971577353193424642086029052734375f,-0.822400001978873973484951420687139034271240234375f,-0.238199985253810808671914855949580669403076171875f,0.74309999259710313168625361868180334568023681640625f,-0.0832999932587146219020723947323858737945556640625f,0.0778000186502934010235321693471632897853851318359375f,0.28369999823272229466653016061172820627689361572265625f,0.5020000036358833739313922706060111522674560546875f,0.39800000143051139644967406638897955417633056640625f,-0.2150000035762786865234375f,-0.6237000215530397184693356393836438655853271484375f,-0.397400012707710192927379466709680855274200439453125f,0.2783000095754861735741769734886474907398223876953125f,-0.4280999958753586298598747816868126392364501953125f,-0.05879997665882097379608239862136542797088623046875f,0.5177999990552664133502958065946586430072784423828125f,0.9026999981284140606163646225468255579471588134765625f,0.39219998432993907755417239968664944171905517578125f,-0.73209999085366728177604045413318090140819549560546875f,0.1013000279188158714305245666764676570892333984375f,-0.9261000014781952671683029620908200740814208984375f,0.05670000450611112352561349325696937739849090576171875f,-0.55579998644143346897550372887053526937961578369140625f,-0.6874000139743088322319408689509145915508270263671875f,-0.01489999236762518108889707946218550205230712890625f,-0.58889999546706661703865393064916133880615234375f,-0.3429000339567662702933148466399870812892913818359375f,0.726200002181529580269625512300990521907806396484375f,-0.701700008641183092805704291095025837421417236328125f,-0.850500026464462877129335538484156131744384765625f,0.04329999512135973560589263797737658023834228515625f,0.4501000030130146711826455430127680301666259765625f,0.1577000167548658904337344210944138467311859130859375f,-0.37099997395277028999771573580801486968994140625f,-0.541799995076656326631336924037896096706390380859375f,-1.633800010037422367759063490666449069976806640625f,0.17699997979402581904651015065610408782958984375f,0.57799997766315958092064875017968006432056427001953125f,0.76360000147819473426125114201568067073822021484375f,-0.058199997323751251343537660432048141956329345703125f,-0.1074000008612869816460033689509145915508270263671875f,-0.4542000229835512215004200697876513004302978515625f,0.1091999966830015100782702575088478624820709228515625f,0.058199999260902413045215553211164660751819610595703125f,-0.078599998125433945261875123833306133747100830078125f,-0.38269998024702100991589759360067546367645263671875f,-0.5479999774768951059655819335603155195713043212890625f,-0.0832000013470644006474685738794505596160888671875f,-0.212100003072619447408442283631302416324615478515625f,0.56629999550878995506764113088138401508331298828125f,0.9331000228464605417144639432081021368503570556640625f,-0.280999987661838535046854303800500929355621337890625f,0.02600000219047071681188754155300557613372802734375f,-0.470000006258487701416015625f,0.126700001525879013541953099775128066539764404296875f,-0.686700000333785975925593447755090892314910888671875f,-0.443100020909309666450326403719373047351837158203125f,-0.310800001901387901170892291702330112457275390625f,0.09690000185221425699211295068380422890186309814453125f,0.53490000287294403591431546374224126338958740234375f,0.1414999942332506588371643374557606875896453857421875f,0.176100002148747381003346390571095980703830718994140625f,-0.588299987074732744218863444984890520572662353515625f,0.518099990361928774973421241156756877899169921875f,0.17499999761581419566169870449812151491641998291015625f,0.4713999770641326136910720379091799259185791015625f,0.03299996179342290503200274542905390262603759765625f,-0.6524999837577343253514072785037569701671600341796875f,0.49689999145865471064098528586328029632568359375f,0.532199996995925683762607150129042565822601318359375f,-0.01530000456869606839660491459653712809085845947265625f,-0.3244000184297561872881487943232059478759765625f,-0.238800010633468673404422588646411895751953125f,0.19320000857412811257063367520458996295928955078125f,1.140400011456012663302317378111183643341064453125f,0.72340000245571101089581134147010743618011474609375f,-0.588400005547702331654136287397705018520355224609375f,0.31270000453591340061620940105058252811431884765625f,0.504800026607513441234686979441903531551361083984375f,0.24849999954551460490392855717800557613372802734375f,0.571300013911724136050906963646411895751953125f,0.292600004875659625014350240235216915607452392578125f,-0.298600007177889370524326295708306133747100830078125f,-0.6559999912381171416342340307892300188541412353515625f,-0.3355999870836736675983047462068498134613037109375f,0.47239998938739302669631570097408257424831390380859375f,-0.17500001162290601541826617904007434844970703125f,-0.33060002090930939999680049368180334568023681640625f,-0.232799991716444498734972512465901672840118408203125f,-0.629400009340047805750373299815692007541656494140625f,-0.2681999946415423163870173084433190524578094482421875f,0.498000002659857277098609529275563545525074005126953125f,-0.1778000003218649371916626478196121752262115478515625f,0.216400015956163205288476092391647398471832275390625f,0.46699999082088472501794740310288034379482269287109375f,-0.4589000110387804198808225919492542743682861328125f,-0.03619998877048491436880794935859739780426025390625f,0.495800003056228177911179955117404460906982421875f,-1.0853999986410141076476065791212022304534912109375f,-0.867000010788441244358182302676141262054443359375f,0.3716999875932931463040631570038385689258575439453125f,-0.3516999869227408481719976407475769519805908203125f,-0.26659998711347565603091425145976245403289794921875f,0.211099996560812019907871217583306133747100830078125f,0.5332000163972379080945529494783841073513031005859375f,0.04879998407214858335834151148446835577487945556640625f,-0.6467000102430582320067742330138571560382843017578125f,0.1356999989628793201745793339796364307403564453125f,-0.5515000318586829042288854907383210957050323486328125f,0.39090000096559496256531929248012602329254150390625f,-0.41619999335259205697212792074424214661121368408203125f,-1.06099999302625658259557894780300557613372802734375f,-0.5392999652743346583605443811393342912197113037109375f,0.302300000008940639162346997181884944438934326171875f,-0.0628999812066552976830280385911464691162109375f,0.1553999992370602711844185250811278820037841796875f,0.206199985343217928601688981871120631694793701171875f,0.644600015068053977529416442848742008209228515625f,-0.24849999511241893657143009477294981479644775390625f,0.75049997602403195617881692669470794498920440673828125f,0.03129999215602818907200344256125390529632568359375f,-0.6555999922990796147104219926404766738414764404296875f,0.6325999892294407178638948607840575277805328369140625f,0.6130000096261500885930217918939888477325439453125f,0.036699994373321498386530947755090892314910888671875f,0.8743999777123334649076014102320186793804168701171875f,0.797200009810924203890181161114014685153961181640625f,-0.0034000002950429486503480802639387547969818115234375f,-0.145000000968575404325378030989668332040309906005859375f,0.410199984642863224593156701303087174892425537109375f,0.45009998759031333293023635633289813995361328125f,-0.61670000689029669249663356822566129267215728759765625f,-0.73990003416538296932003504480235278606414794921875f,-0.114699995282292466214357773424126207828521728515625f,-0.2450999936997891592849185826707980595529079437255859375f,0.2205000044107439105545154234278015792369842529296875f,-0.6121999741226435975960384894278831779956817626953125f,-0.090999998465180365059268297045491635799407958984375f,-0.0230000116378068497624553856439888477325439453125f,-0.413699990111588977725887161795981228351593017578125f,-0.4918000030860303350976892033941112458705902099609375f,-0.3506000322341922270652503357268869876861572265625f,0.59409999217987063957480131648480892181396484375f,-0.627600015194713645172441829345189034938812255859375f,0.135599949681758147335131070576608180999755859375f,0.186899984306097000086310799815692007541656494140625f,0.2923000061929224013823613859131000936031341552734375f,0.8063000222563747598769623436965048313140869140625f,0.443399995321780149293289241541060619056224822998046875f,0.050900004523247499577109920210205018520355224609375f,-0.397100001340359387569378668558783829212188720703125f,-0.318300002050400099307125856284983456134796142578125f,0.399400008893012969934943612315692007541656494140625f,0.453200000378489509245127919712103903293609619140625f,0.3490000059306621604804377057007513940334320068359375f,-0.107799997304379946427843606215901672840118408203125f,0.7798000105142595117513337754644453525543212890625f,0.48059998902082412541858502663671970367431640625f,-0.5595000133216381232870162421022541821002960205078125f,0.6217999938845633067785456660203635692596435546875f,0.319799989056587463664982351474463939666748046875f,-0.4829999899193644363748489922727458178997039794921875f,0.654500005125999706478978623636066913604736328125f,0.35279998393058775718600372783839702606201171875f,-0.547800007027387625413439309340901672840118408203125f,-0.6783000217199326442596429842524230480194091796875f,-0.1173999867796897689231627737171947956085205078125f,0.7981000083923339705194166526780463755130767822265625f,-0.7737999736785889925982928616576828062534332275390625f,-0.641999980539083470176819901098497211933135986328125f,0.29459999033212669328207766739069484174251556396484375f,-0.41530000345110895665357020334340631961822509765625f,-0.216599999928474229449193444452248513698577880859375f,0.2037000012874603527279759873636066913604736328125f,0.7097000142812728284980039461515843868255615234375f,0.263799998414516068123703007586300373077392578125f,-0.5101000054717064191578401732840575277805328369140625f,1.36569998674392678594813332892954349517822265625f,0.9911999953269958751889134873636066913604736328125f,-0.602999986976385127235289473901502788066864013671875f,-0.1999999964609739944165767155936919152736663818359375f,-0.08879999997913812226357777035445906221866607666015625f,0.086099996560812019907871217583306133747100830078125f,0.6925000345706937565637417719699442386627197265625f,0.14340001229047771857949555851519107818603515625f,-0.6393000057309865535870585517841391265392303466796875f,-0.096899991160630971620548734790645539760589599609375f,-0.396200020062923652375275196391157805919647216796875f,-0.330700008276104961968400175464921630918979644775390625f,0.251099985310435425134301112848334014415740966796875f,0.4117999993979928330389839175040833652019500732421875f,0.16599999924749131441359395466861315071582794189453125f,-0.1484000206351279160799094825051724910736083984375f,0.19039999715089805931711453013122081756591796875f,0.3307000038430094601693554068333469331264495849609375f,-0.5960000316202640480156560442992486059665679931640625f,0.89470002586692576329596704454161226749420166015625f,-0.20329999869763859532412197950179688632488250732421875f,0.67509998960196970774205738052842207252979278564453125f,0.58310002411305905667404658743180334568023681640625f,-0.18529997677803056177481266786344349384307861328125f,-0.4269999889209865617800687687122263014316558837890625f,-1.4270000304579735228571735206060111522674560546875f,-0.01150000131130202163376452517695724964141845703125f,-0.1096999937921763201842395574203692376613616943359375f,-0.7704999972581862976994671043939888477325439453125f,-0.0617999797284607410574608366005122661590576171875f,-0.499099974112212851906633659382350742816925048828125f,-1.405900004541873471453072852455079555511474609375f,-0.4428999892532825555235831416212022304534912109375f,0.1868999849021435455398432168294675648212432861328125f,0.9033000086069105094566111802123486995697021484375f,0.4881999652117496868442003687960095703601837158203125f,-0.0710000001043080697371578935417346656322479248046875f,0.6817999812930821690315497107803821563720703125f,-0.0216999973535536572200044247438199818134307861328125f,-0.6952999814718963111914717956096865236759185791015625f,-0.6709000142276284606168701429851353168487548828125f,0.2348999863326550752162802382372319698333740234375f,0.9084999695569280220297514460980892181396484375f,0.7337000012874597132395138032734394073486328125f,-0.55699998590350130456272381707094609737396240234375f,0.637699980731308624370967663708142936229705810546875f,-0.387499991357326933894000831060111522674560546875f,-0.050199991087615802864974057229119352996349334716796875f,-0.5310999680250887422516825608909130096435546875f,-0.628699990297853172904751772875897586345672607421875f,0.45109996020197939969875733368098735809326171875f,0.80029998445212857660635563661344349384307861328125f,0.60309999885559051335803815163671970367431640625f,0.6832999902784819834522522796760313212871551513671875f,-0.084299997386336311677723642787896096706390380859375f,0.024900002276897481578998849727213382720947265625f,-0.55169996971190016221697760556708090007305145263671875f,0.2363999888733030463328077530604787170886993408203125f,0.07200000415742380255323951132595539093017578125f,-0.55179996795654329133640203508548438549041748046875f,-0.5424999861046673199638235018937848508358001708984375f,-0.7409000033497807891080810804851353168487548828125f,-0.6295999703459445040465425336151383817195892333984375f,-0.60200000970810652045628330597537569701671600341796875f,-0.758400018250942054010010906495153903961181640625f,-0.6438000249385833484438990126363933086395263671875f,0.195600003959238488704386327299289405345916748046875f,-0.02239998618364325011498294770717620849609375f,0.291800031137466220343412715010344982147216796875f,0.4503999886199832047850577509962022304534912109375f,-0.489900012707710441617336982744745910167694091796875f,0.11780002803802513966502374387346208095550537109375f,0.7969000081479549191243449968169443309307098388671875f,0.16609999730586988420100169605575501918792724609375f,0.2370000122785569374173064716160297393798828125f,0.09900000626593830066468626682762987911701202392578125f,-0.87270001429617405364069782081060111522674560546875f,-0.50720000407397713981794140636338852345943450927734375f,0.462500006929039975123174599502817727625370025634765625f,0.79710001000165942031117083388380706310272216796875f,0.377199997293949262910928155179135501384735107421875f,-0.56379999819099901881003233938827179372310638427734375f,-0.815099975222349115711040212772786617279052734375f,-0.56799997889250530835880681479466147720813751220703125f,0.434499985679984102038275750601314939558506011962890625f,1.302700005280971851107096881605684757232666015625f,0.940799993407726020677728229202330112457275390625f,-0.611700009386241472242318195640109479427337646484375f,-0.552099988208710978909721234231255948543548583984375f,-0.303199991400539936581282063343678601086139678955078125f,0.32299998796358708741394138996838591992855072021484375f,0.025200017499923621500101944548077881336212158203125f,-0.3272000065326690076972226961515843868255615234375f,-0.409900002537667729196613208841881714761257171630859375f,-1.396100024724006516407825984060764312744140625f,-0.9880000175237650950066381483338773250579833984375f,0.612400012484192846073938198969699442386627197265625f,-0.048400011880695981147937345667742192745208740234375f,0.066000003494322057040477602640748955309391021728515625f,0.22110000492408865557791841638390906155109405517578125f,0.556399994423985244651476023136638104915618896484375f,0.6500000192224979489452607595012523233890533447265625f,0.0844000068068504472051927223219536244869232177734375f,0.2131999903202057566886651329696178436279296875f,0.31539999446868893784312604111619293689727783203125f,-0.0190000011995434814338068463257513940334320068359375f,-0.864600018644333001560653428896330296993255615234375f,-0.619500016152858989926244248636066913604736328125f,0.4839000029921525225518053048290312290191650390625f,-0.5798999890893699049598808414884842932224273681640625f,-0.1166999913930890642177473637275397777557373046875f,-0.57269999037981023093379917554557323455810546875f,0.608299989309907029877422246499918401241302490234375f,0.906700003314018232458693091757595539093017578125f,-0.723699990558623884595590425305999815464019775390625f,0.0812999963283538296110464216326363384723663330078125f,0.5337000032246113079281712998636066913604736328125f,-0.285900002902745864474809422972612082958221435546875f,0.35019999019354586256014272294123657047748565673828125f,0.35300000530481323579579111537896096706390380859375f,0.086000001408159665317043618415482342243194580078125f,-0.84119997759461428987748377039679326117038726806640625f,-0.44400000272691209080022645139251835644245147705078125f,0.07839999232292171882363618351519107818603515625f,0.473700003038346639794298198467004112899303436279296875f,-0.033899976766109229942003366886638104915618896484375f,0.300399978133291389870151988361612893640995025634765625f,-0.06120000085905186182344550616107881069183349609375f,0.719999980237334913368840716429986059665679931640625f,-0.53159996550679267102168523706495761871337890625f,0.400400004228949502049772490863688290119171142578125f,-0.689599987053870933095822692848742008209228515625f,0.0226000034414230999146866452065296471118927001953125f,-0.462499983683228588215996524013462476432323455810546875f,-0.176500014014541795059898277031606994569301605224609375f,0.671399970358610431730994605459272861480712890625f,-0.0482999970212578677575265828636474907398223876953125f,-0.0387000059068203228207494248636066913604736328125f,0.50939999443888606123209683573804795742034912109375f,-0.86829997146576687327979016117751598358154296875f,0.0577000261425979488194570876657962799072265625f,-0.1543999993376432566805078749894164502620697021484375f,0.54729996797144442322036184123135171830654144287109375f,0.8752999982729547934212632753769867122173309326171875f,-0.69859999141991091509140687776380218565464019775390625f,0.60819997951686399773052471573464572429656982421875f,0.3362000001698728368637603125534951686859130859375f,-0.437699972200393805366047672578133642673492431640625f,-0.606499986410141023185360609204508364200592041015625f,-1.178500019848346624939949833787977695465087890625f,0.017999998152256058148168449406512081623077392578125f,-0.14059999501854181236382146380492486059665679931640625f,0.8133000270843506740447992342524230480194091796875f,0.71260003437995944608474019332788884639739990234375f,-0.1468000044271349224800360389053821563720703125f,0.035399984931945738253489253111183643341064453125f,0.4144000166416166308636093162931501865386962890625f,0.496400002992153144276699094916693866252899169921875f,0.585200013402104257664859687793068587779998779296875f,0.5868000324785713761599481586017645895481109619140625f,0.0132000059664250102287041954696178436279296875f,-0.68220000593662266652472681016661226749420166015625f,-0.2507999841690065068178228102624416351318359375f,0.67619999949932096949822835085797123610973358154296875f,-1.3421000219225884819707061978988349437713623046875f,-0.9128000140309335819210900808684527873992919921875f,0.6395000036731361969799536382197402417659759521484375f,0.4315999910622834168094641427160240709781646728515625f,0.22039999066889304213390232689562253654003143310546875f,-0.3106999898731708231736092784558422863483428955078125f,-0.3074999880790709738676014239899814128875732421875f,-0.012399976944923896127193074789829552173614501953125f,0.4962999858602881175784204970113933086395263671875f,-0.2629000159263608082937935250811278820037841796875f,0.03259999206066144239457571529783308506011962890625f,0.27939999991506336840529911569319665431976318359375f,-0.28109999179244038458591603557579219341278076171875f,-0.473400016462802941674681278527714312076568603515625f,-0.255000005476176738739013671875f,-1.2269000053167342656479377183131873607635498046875f,-0.7864000038862233310510418959893286228179931640625f,0.6615999891251325148999740122235380113124847412109375f,0.37259999772310248289386436226777732372283935546875f,0.03039998850822467346688426914624869823455810546875f,-0.41679999075531959373819290703977458178997039794921875f,0.25859998165965070171523620956577360630035400390625f,0.384600001955031967071363396826200187206268310546875f,0.18990000428855413616702207946218550205230712890625f,-0.81679999477863329815363613306544721126556396484375f,-0.5379999841451645892220767564140260219573974609375f,0.52599999906122685278120343355112709105014801025390625f,1.039100021386146988078280628542415797710418701171875f,0.68239999333620016130907970364205539226531982421875f,-0.6122000245630740966618077436578460037708282470703125f,1.21209999897480003028249484486877918243408203125f,-0.31030000009834768359695544859278015792369842529296875f,-0.551300003927945869719451366108842194080352783203125f,0.8742000134840604008701347993337549269199371337890625f,0.5106000393867500264377667917869985103607177734375f,-0.761999977409839601705243694595992565155029296875f,0.5057999767929317602721539515187032520771026611328125f,0.6762000018835063741562407813034951686859130859375f,-0.53260001001656009922413659296580590307712554931640625f,0.8638000028103587357719561623525805771350860595703125f,0.716600036957860453412649803794920444488525390625f,-0.175400000429153379855051753111183643341064453125f,0.3425999804377557467205406283028423786163330078125f,1.17060000153779952825061627663671970367431640625f,0.41919999601244928300047831726260483264923095703125f,-0.774400011947750765983755627530626952648162841796875f,0.141999965786933302069883211515843868255615234375f,0.0031999985530972718805742260883562266826629638671875f,0.09909999959319824858194891703533357940614223480224609375f,-0.26409998830556868920638180497917346656322479248046875f,-0.25459999659061427479400663287378847599029541015625f,0.673199972736835849929093456012196838855743408203125f,1.310399991786479478861338066053576767444610595703125f,-0.362499967962504054952432852587662637233734130859375f,-0.7129999723359945473788457093178294599056243896484375f,-0.8757999943017953370372197241522371768951416015625f,0.6252999869108200048373191748396493494510650634765625f,-0.631799988408386692384510752162896096706390380859375f,-0.6450000219047069549560546875f,0.29500000283122052024964432348497211933135986328125f,-0.55630001372545923032930659246630966663360595703125f,0.35529995979070605471861199475824832916259765625f,0.329199984538555145974214610760100185871124267578125f,0.29699999715387814802625143784098327159881591796875f,0.12660002079010013176230131648480892181396484375f,-0.4704999822080135896129604589077644050121307373046875f,0.665999962814152635104392174980603158473968505859375f,0.73319999821782033677664003334939479827880859375f,0.25619997595548671398546503041870892047882080078125f,0.307399979031086179048770645749755203723907470703125f,1.284999993145465424504436668939888477325439453125f,-0.303799966228008866409027177724055945873260498046875f,0.003100009547174042978667785064317286014556884765625f,-1.026100016230344635914661921560764312744140625f,-0.29289998984932896775035260361619293689727783203125f,0.488099978105723864008069767805864103138446807861328125f,-0.22589999426007256744242113200016319751739501953125f,-0.2696000094056127949215806438587605953216552734375f,-0.111700004319846613043409888632595539093017578125f,-0.4300000351667403464261951739899814128875732421875f,0.1058000053286551889186739572323858737945556640625f,0.7861000124305486114195673508220352232456207275390625f,0.05290000460147847860525871510617434978485107421875f,-0.286600013935565556266737985424697399139404296875f,-0.44149999214708801620332678794511593878269195556640625f,0.19100001046061532150588391232304275035858154296875f,-0.008799998712539558454182042623870074748992919921875f,-0.179900001196563241734338589594699442386627197265625f,-0.400299996745586472712830072850920259952545166015625f,-0.02999997615814198326233963598497211933135986328125f,0.51989999955743559922893837210722267627716064453125f,0.931899979388713717298742267303168773651123046875f,0.652199985074997101719418424181640148162841796875f,-0.478199984918534770628184560337103903293609619140625f,0.66369998623728765352325353887863457202911376953125f,0.704999999999999626965063725947402417659759521484375f,-0.002700001555681286191656909068115055561065673828125f,0.9787999957323076927195870666764676570892333984375f,0.8171999975919721492800817941315472126007080078125f,-0.3006000036239624417788718346855603158473968505859375f,0.865399998342990528499285574071109294891357421875f,0.334799989950656851078747422434389591217041015625f,-0.675299988810718065934679543715901672840118408203125f,-0.79439998087883001431919183232821524143218994140625f,-0.6051999888896943957661278545856475830078125f,0.31539998731613161364606412462308071553707122802734375f,-0.379600000986456631579812892596237361431121826171875f,-0.4104000169694426158883970856550149619579315185546875f,0.00479999255836005556119516768376342952251434326171875f,-0.46860000296831127997165822307579219341278076171875f,-0.01599998170137428843418092583306133747100830078125f,0.63249999437481163599983347012312151491641998291015625f,0.1331000164389612994142453317181207239627838134765625f,-0.185200013476610525486876213108189404010772705078125f,-0.37400001427531248676672248620889149606227874755859375f,-1.332099985563754529493962763808667659759521484375f,-1.0258000068187715925205338862724602222442626953125f,0.5037999787077307445315454970113933086395263671875f,0.2387999751687051031012742896564304828643798828125f,0.6342999725759028706306708045303821563720703125f,0.2731999997079370690045152514358051121234893798828125f,-0.719399972534179976690893454360775649547576904296875f,-0.864600007170438278336632720311172306537628173828125f,-0.6142999742150312414423751761205494403839111328125f,0.6297999774709344134038246920681558549404144287109375f,1.3298000376343726003369738464243710041046142578125f,0.6064999746382238043906909297220408916473388671875f,-0.31809998231530212109419153421185910701751708984375f,-0.4864999908804890083047212101519107818603515625f,0.2357999884903427822990806816960684955120086669921875f,0.559900004102289461371810830314643681049346923828125f,0.69830003766417547694800305180251598358154296875f,-0.337800001811980887822528529795818030834197998046875f,0.140799998474121057512320476234890520572662353515625f,0.5732000009000302043205010704696178436279296875f,-0.344299984496831967817342956550419330596923828125f,0.172399995310604570164514370844699442386627197265625f,-0.665599987232685208482507732696831226348876953125f,-0.7012999647378925782703618097002618014812469482421875f,0.5656999797403814422835921504884026944637298583984375f,0.1669000069558617127540856017731130123138427734375f,-0.60739996681213437312862879480235278606414794921875f,0.530799997207522356035269694984890520572662353515625f,-0.182799978566169585292300325818359851837158203125f,0.23959997960329104671473032794892787933349609375f,0.654699970881641224451641392079181969165802001953125f,0.193099999153613310909349820576608180999755859375f,0.37979998100996059662293191649951040744781494140625f,-0.3912000003188846886104101940873079001903533935546875f,0.1991999648690221391689192387275397777557373046875f,0.41880000340640488598609181281062774360179901123046875f,-0.2479999970346689064371048516477458178997039794921875f,0.022399999073147824901752755977213382720947265625f,-0.43259998699426649437782543827779591083526611328125f,0.18029998679906145131468520048656500875949859619140625f,0.69149999177455878651699094916693866252899169921875f,0.5047999961346383290816675071255303919315338134765625f,0.079900001047551683086567209102213382720947265625f,0.0394999995380640456232868018560111522674560546875f,0.65469999952912250051895171054638922214508056640625f,-0.74059997333735250180097864358685910701751708984375f,0.20539999715089862775130313821136951446533203125f,-0.2411999631404881938578910194337368011474609375f,-0.310899999177455921284263240522705018520355224609375f,-0.053000012755394010355303180404007434844970703125f,0.2744000226020819610539547284133732318878173828125f,0.46720001726150517384894556016661226749420166015625f,1.1295000051259993512076107435859739780426025390625f,0.642799994063377067021747279795818030834197998046875f,-0.671300000984966782624496772768907248973846435546875f,0.536499994754791487139300443232059478759765625f,0.569600004339218202176198246888816356658935546875f,-0.022999999940395365882750411401502788066864013671875f,0.6908000115871431034975103102624416351318359375f,0.152599985206126831371875596232712268829345703125f,-0.63470000653266911427863306016661226749420166015625f,0.25050000679492967492478783242404460906982421875f,-0.2966000353932383148958251695148646831512451171875f,-0.736900007328391115635213282075710594654083251953125f,-0.445700002911686876672092694207094609737396240234375f,-0.3970000118315220039022506171022541821002960205078125f,0.04479999825805425217595256981439888477325439453125f,-1.386100008332729061066856957040727138519287109375f,-1.02340001497268673347207368351519107818603515625f,0.561399995951354480183681516791693866252899169921875f,-0.65629998714551318617083097706199623644351959228515625f,-0.7330000110790131362392685332451947033405303955078125f,-0.0797000043533742097423555605928413569927215576171875f,-0.2643999993562697881088752183131873607635498046875f,0.31720003335475865924308891408145427703857421875f,0.8221999877572059300945284121553413569927215576171875f,-0.3007000058174131851274069049395620822906494140625f,0.10980001915693282654729046043939888477325439453125f,0.6032999995172023677270090047386474907398223876953125f,-0.1435000028610229350078952847979962825775146484375f,-0.1890000150203707818263865192420780658721923828125f,-0.10140000571161511544460154254920780658721923828125f,-0.84290000817775734276438015513122081756591796875f,-0.41759999742507947217973196529783308506011962890625f,0.542399997285008428349328823969699442386627197265625f,-0.1980000216960908776542282794252969324588775634765625f,0.131600006037950745252373963012360036373138427734375f,0.39160001311600212137165044623543508350849151611328125f,-0.869100002014636885405707289464771747589111328125f,-0.53340000245571150827572637354023754596710205078125f,0.446999992348253716301087479223497211933135986328125f,1.2087999924540522300731026916764676570892333984375f,0.603300006222724505278165452182292938232421875f,-0.57039999893903736705169649212621152400970458984375f,0.579800011482834332099400853621773421764373779296875f,0.892600060606003609109393437393009662628173828125f,-0.1017999980568884410558894160203635692596435546875f,0.7073999743372205717406586700235493481159210205078125f,-0.011400004780293127026880029006861150264739990234375f,-0.57259997403025675311027953284792602062225341796875f,0.3209000087514517485232090621138922870159149169921875f,-0.0754999713301653940789037733338773250579833984375f,0.4286999920487399862167876563034951686859130859375f,0.7467999833419920463484231731854379177093505859375f,0.5204000180125234464867389760911464691162109375f,0.81909999322295146129135900991968810558319091796875f,-0.4592999923199414524788153357803821563720703125f,-0.8757000141620636668449151329696178436279296875f,0.30629999863803381021654104188201017677783966064453125f,-0.667599989639222624049352816655300557613372802734375f,-0.5696000193148851309388192021287977695465087890625f,0.5902999983847134757297681062482297420501708984375f,-0.853399977980554336198792952927760779857635498046875f,0.078200003135204809723290964029729366302490234375f,0.7516999786339699785031598366913385689258575439453125f,-0.838299967684597024941695053712464869022369384765625f,-0.5891000046037131454568225308321416378021240234375f,-0.86059997560977929964565191767178475856781005859375f,0.7491999661356214534890796130639500916004180908203125f,1.3315999880075448658089953823946416378021240234375f,-0.4964999862611296777004099567420780658721923828125f,0.52809999438524235504388570916489697992801666259765625f,1.05890002832412744027124062995426356792449951171875f,0.205500006645917832059922147891484200954437255859375f,-0.52640000288039423548269724051351658999919891357421875f,-0.65320003323555031471414622501470148563385009765625f,-0.0557999935567377036704783677123486995697021484375f,-0.71829999977797254206990373859298415482044219970703125f,-0.29630003656148939938930197968147695064544677734375f,-0.4665999889761207608529502977035008370876312255859375f,-0.1350000023096800116917393097537569701671600341796875f,-0.0581999984413386073356377892196178436279296875f,0.14730000589489922191432924591936171054840087890625f,-0.71579999061375854108035809986176900565624237060546875f,-0.24900002425909040226770230219699442386627197265625f,-0.513699967610836694120735046453773975372314453125f,0.326600004845857661006647276735748164355754852294921875f,0.2009000167608263609508867375552654266357421875f,-0.13400000339746487298953070421703159809112548828125f,-0.2899000068590045220418005555984564125537872314453125f,-0.420400013840198649717194712138734757900238037109375f,0.03620001291036611945628465036861598491668701171875f,0.55960000467449422245636014849878847599029541015625f,0.32049999323487277091970781839336268603801727294921875f,0.4404000076562166743343595953774638473987579345703125f,0.06660000275969506222661919991878676228225231170654296875f,-0.045399997448921425302614807151257991790771484375f,-0.0509999867677688456524265347979962825775146484375f,0.1266000061497092321616264598560519516468048095703125f,0.0407000076055528126062199589796364307403564453125f,-0.17740001240968705786826831172220408916473388671875f,-0.2915000048875808769111017682007513940334320068359375f,0.30220000116825129765629753819666802883148193359375f,-0.0696000210285188103398468228988349437713623046875f,-0.42780000233352188931945647709653712809085845947265625f,-0.49960000091195144733546840143389999866485595703125f,-0.635400015628337921924639886128716170787811279296875f,-0.161999995924532402824524979223497211933135986328125f,-0.9901000185847284029705406283028423786163330078125f,-0.6248000024676318986394107923842966556549072265625f,0.5273000137552619204228676608181558549404144287109375f,-0.7852000055789947197126821265555918216705322265625f,-0.2295999876499177361210968228988349437713623046875f,0.6987999936461448680091734786401502788066864013671875f,-0.0311000019252301029837326495908200740814208984375f,-0.1578000171601774415108820903697051107883453369140625f,-0.24630000362992288120977946164202876389026641845703125f,0.6666000178098681061555907945148646831512451171875f,0.5500000029802318834981633699499070644378662109375f,-0.3190000196397304588202814557007513940334320068359375f,0.3736999920487402704338819603435695171356201171875f,0.3581999885320659160470313508994877338409423828125f,-0.2100000062212348250767490753787569701671600341796875f,-0.5150000101327893986535855219699442386627197265625f,-0.0619999949932097393912044935859739780426025390625f,0.5287999974459409724403258223901502788066864013671875f,-0.38730000783205031922307171043939888477325439453125f,-0.36280001254081728490064051584340631961822509765625f,0.05980000224411485820752432118752039968967437744140625f,-1.212500031590462157282672706060111522674560546875f,-0.1587000106751919048519994248636066913604736328125f,0.507399982756375766967948948149569332599639892578125f,0.85539999364912500112723137135617434978485107421875f,1.16780004085302380190114490687847137451171875f,-0.508099986487626775755188646144233644008636474609375f,-0.26990000250041479290530332946218550205230712890625f,-0.054700014728307877476254361681640148162841796875f,-0.55039998243600118943419374772929586470127105712890625f,-0.025299998682737356858751809340901672840118408203125f,-0.39770000442415476271662555518560111522674560546875f,0.093099977397919086996580517734400928020477294921875f,0.82329997381269948153459381501306779682636260986328125f,1.31930001534223517722921314998529851436614990234375f,0.38109999536871885794653280754573643207550048828125f,-0.24769998777210722185060376432375051081180572509765625f,-0.38649999952316260731777219916693866252899169921875f,0.45879999811649252450251879054121673107147216796875f,0.35119999949932090288484687334857881069183349609375f,0.2896000290751459260718547739088535308837890625f,0.7212999714434153997189014262403361499309539794921875f,-0.73130000981390441694429682684130966663360595703125f,-0.578500050991774372732834308408200740814208984375f,0.2131999883830548725427433964796364307403564453125f,-0.5665999900937082056628923965035937726497650146484375f,0.1034000036478044393106756615452468395233154296875f,-0.3635999830007552446886620600707828998565673828125f,-0.5779999957680701516693488883902318775653839111328125f,-0.1636000283002854160940842120908200740814208984375f,0.24770000297129168842502622283063828945159912109375f,-0.72969999878406499593808121062465943396091461181640625f,-1.0709000425398353950612317930790595710277557373046875f,-0.43339997116327300119564824854023754596710205078125f,-0.349799986374378146791741528431884944438934326171875f,-1.221000031471252356141121708787977695465087890625f,-0.36849997201561901505328933126293122768402099609375f,-0.777800006990134296103178712655790150165557861328125f,-1.591000065147877506888107745908200740814208984375f,-0.4229999743103984854997179354541003704071044921875f,-0.5751999780863525923990664523444138467311859130859375f,-0.4255999988555903001952174236066639423370361328125f,0.313999996393918845427606356679461896419525146484375f,-0.3074999994412064285853602996212430298328399658203125f,-0.0711000047564507298147873370908200740814208984375f,0.36329997716546102992651867680251598358154296875f,-0.1058000046022236351195289216775563545525074005126953125f,-0.61710000016689292579030734486877918243408203125f,-0.33239999512433993089643990970216691493988037109375f,0.3794999977871775787008346014772541821002960205078125f,-0.34310001479983309735644070315174758434295654296875f,0.004999991357326205587696676957421004772186279296875f,0.340599992336332757503214452299289405345916748046875f,0.336699988114833814734083716757595539093017578125f,0.01499997317791024187272341805510222911834716796875f,-0.53939998851567505511184208444319665431976318359375f,0.0693000141501425304113581660203635692596435546875f,0.319200034606456739538771216757595539093017578125f,0.3530999954655766881472800378105603158473968505859375f,-0.52680001057386416363215175806544721126556396484375f,-0.269999992549419420839740269002504646778106689453125f,0.44400000719726084863481219144887290894985198974609375f,1.1857999811887740548854708322323858737945556640625f,0.9301999903798101598795255995355546474456787109375f,-0.4432999913960695703707415304961614310741424560546875f,0.72489998588561999071089303470216691493988037109375f,0.32279998154640221486033624387346208095550537109375f,-0.56099998430907727087202374605112709105014801025390625f,-0.0171000150680544749093314749188721179962158203125f,0.3662000212550164945923825143836438655853271484375f,0.4699000112175941712422400087234564125537872314453125f,-0.8409000158667563340486594825051724910736083984375f,-0.78580003244876905910132336430251598358154296875f,0.1251999959304928022874037196743302047252655029296875f,-0.6304000153303146447569815791212022304534912109375f,-0.2500000053644182429479769780300557613372802734375f,0.4425999977231025450663537412765435874462127685546875f,0.1526000015974044998756653512828052043914794921875f,0.08559999200105661287807379267178475856781005859375f,-0.20230000451654195359196819481439888477325439453125f,0.28200000363588362262134978664107620716094970703125f,-0.101400022363662856150767765939235687255859375f,-0.46140000451952223325946533805108629167079925537109375f,-0.44060000153779999010339452070184051990509033203125f,0.1072000336527825226085042231716215610504150390625f,0.75590000167340043457642195789958350360393524169921875f,0.10230001409053812722049769945442676544189453125f,0.1938000252366069986464935936965048313140869140625f,0.174900000786781362194233224727213382720947265625f,0.708099999153613879343538428656756877899169921875f,-0.244900009094178694457610845347517170011997222900390625f,0.015800001305341648816238375729881227016448974609375f,-0.44279998452663438257559391786344349384307861328125f,-0.0840000022053715156289399601519107818603515625f,-0.3930999624967579375578452527406625449657440185546875f,-0.808599991233646786525923744193278253078460693359375f,-0.9482000307023525920158135704696178436279296875f,0.2767000007808206873960443772375583648681640625f,-0.72099997346848276880137973421369679272174835205078125f,-1.41019999574422794097472433350048959255218505859375f,-0.536400003141164205544555443339049816131591796875f,0.566999987691640949805105265113525092601776123046875f,0.757799996447563017909487825818359851837158203125f,0.018199989873171062981782597489655017852783203125f,0.317399979031086132419403611493180505931377410888671875f,0.999000003695487581723000403144396841526031494140625f,-0.6208999720573427083536444115452468395233154296875f,0.5655999793723227586639268338331021368503570556640625f,-0.3303999917864803848033261601813137531280517578125f,0.1035999962627887072130761225707828998565673828125f,0.26149999643117183278917536881635896861553192138671875f,-0.28849997723102571711706332280300557613372802734375f,-0.73229999039769122504139886586926877498626708984375f,0.226499986931681718260733759962022304534912109375f,0.29969999759197207822580821812152862548828125f,0.67590000320076892847964700195007026195526123046875f,-0.7128000090017911904993752614245750010013580322265625f,-0.2244000294566159681153294513933360576629638671875f,-0.17539998075962071055755586712621152400970458984375f,-0.759200000259280027847808014485053718090057373046875f,-0.575200035977363910433268756605684757232666015625f,-0.044999989271163887138982317992486059665679931640625f,-0.6376999872133135571772299954318441450595855712890625f,-1.1329000278472900475890128291212022304534912109375f,0.49389999911785054109714110381901264190673828125f,0.18750000476837147544983963598497211933135986328125f,0.06030002416372326479176990687847137451171875f,-0.64179999656677200192689269897527992725372314453125f,0.3244999894425273367914996924810111522674560546875f,0.86769999961853017111934605054557323455810546875f,-0.2525000087916851043701171875f,0.6147000142067666761391819818527437746524810791015625f,0.8247000452756889643524118582718074321746826171875f,0.242799969476461541262324317358434200286865234375f,-0.6835999988332390753242862047045491635799407958984375f,1.2582000016450880508500631549395620822906494140625f,0.993200017142296331940087839029729366302490234375f,-0.4237999863445758297331167341326363384723663330078125f,0.0476999966010451803555270089418627321720123291015625f,-0.03780000665485883093452912362408824265003204345703125f,-0.0456000009417534268862937096855603158473968505859375f,-0.4484999542832373009559887577779591083526611328125f,-0.392599964642524668079204275272786617279052734375f,0.2539999814555049528763674970832653343677520751953125f,-0.559500018127262688238943155738525092601776123046875f,-0.242399998067319211170200787819339893758296966552734375f,0.46260001066848632778061300996341742575168609619140625f,-0.6565000090599060200702297152020037174224853515625f,-0.35619999085664755256175340036861598491668701171875f,0.5701000046521425002055138975265435874462127685546875f,-0.6170000239014630238898462266661226749420166015625f,-0.49880000139474844900178140960633754730224609375f,0.3461000245377421347114932359545491635799407958984375f,1.4317000143408780132858737488277256488800048828125f,1.0224000028729438582786315237171947956085205078125f,-0.6195000091865658919942916327272541821002960205078125f,0.3665000248551368855487453402020037174224853515625f,-0.0291999946713445979185053147375583648681640625f,-0.4397000032916664746807100527803413569927215576171875f,-0.61059998693466166486132351565174758434295654296875f,-0.4473999960184098512172568007372319698333740234375f,0.26439998754709959083442072369507513940334320068359375f,-0.045399995362758627237553810118697583675384521484375f,0.03420000927448274552489237976260483264923095703125f,0.123299998176097880264734385491465218365192413330078125f,-0.399700017261505280430355924181640148162841796875f,-0.195999995708465490906746708787977695465087890625f,0.404200013148784620398146216757595539093017578125f,-1.4379000347018244809760290081612765789031982421875f,-0.97620001559257474355035810731351375579833984375f,0.6762000203236937512230042557348497211933135986328125f,-0.3939000172972679791882910649292171001434326171875f,-0.06820000104904178073184084496460855007171630859375f,0.3832000040665268780770702505833469331264495849609375f,0.594600000837445019641336330096237361431121826171875f,0.6442000246226788906511728782788850367069244384765625f,0.051500007197260888602841077954508364200592041015625f,-0.6825999950408938587997909053228795528411865234375f,-1.1473000106632709194087738069356419146060943603515625f,-0.65899999862909286463263924815692007541656494140625f,0.8634000032007691505242519269813783466815948486328125f,0.0546000424861912136975661269389092922210693359375f,0.141499999709427115579529754540999419987201690673828125f,0.4025999828219415377361656283028423786163330078125f,0.58620000750869483141514137969352304935455322265625f,0.35609996467232729600027596461586654186248779296875f,0.6666000073418019145066182318259961903095245361328125f,1.4910000210404401599362245178781449794769287109375f,-0.0866000000588595941763969676685519516468048095703125f,-0.055500003442168287237645785126005648635327816009521484375f,0.447300010849535467638560248815338127315044403076171875f,0.07369999040961250358350298483856022357940673828125f,0.65529997789561755805465281810029409825801849365234375f,1.64930002636909467383929950301535427570343017578125f,0.3927999657511715980717781349085271358489990234375f,0.584899964912235947878116348874755203723907470703125f,1.531300019276141455293327453546226024627685546875f,-0.54119997193217361797223929897882044315338134765625f,0.87800001521408521654166179359890520572662353515625f,-0.07999994695186529725106083787977695465087890625f,0.452199970918893967564144986681640148162841796875f,0.53800000396370872834950205287896096706390380859375f,0.51440002319812805353649309836328029632568359375f,0.257299996507167616499600626411847770214080810546875f,-0.28789998880624778632153493163059465587139129638671875f,0.16969999341964747685551628819666802883148193359375f,-0.1061000013291835131212792475707828998565673828125f,0.16179999950975176847833836291101761162281036376953125f,0.28340000439286239242164811003021895885467529296875f,0.01379998962283135455209048814140260219573974609375f,0.6222999766513706010329087803256697952747344970703125f,1.3451000269293782451995866722427308559417724609375f,0.4422999859273435419027009629644453525543212890625f,-0.902399999482929704441858120844699442386627197265625f,0.039599988543987052480588317848742008209228515625f,-0.281699984687566740149122779257595539093017578125f,-0.42540000173300496211226118248305283486843109130859375f,-0.29940001693964024553906710934825241565704345703125f,-0.64109999976456111259182080175378359854221343994140625f,0.770999983154237344962211864185519516468048095703125f,0.5550000111013648762536831782199442386627197265625f,0.475399976587296269059379483223892748355865478515625f,0.472400005070865158796067362345638684928417205810546875f,0.063699998903274490658077411353588104248046875f,-0.116600019299984314358198389527387917041778564453125f,-0.3076000058069825460194124389090575277805328369140625f,0.12720000534057618324368377216160297393798828125f,-0.2406000164389610063153668306767940521240234375f,-0.4328999987900257195860831416212022304534912109375f,0.2417999910533428220560381305404007434844970703125f,0.223399994111061150903196903527714312076568603515625f,-0.09709999621808529379762831013067625463008880615234375f,-0.168500006437302118911247816868126392364501953125f,-0.20779999465942378122917943983338773250579833984375f,0.0668000138148665545401172494166530668735504150390625f,0.8358000095009803231960177072323858737945556640625f,0.7156000247836118433042429387569427490234375f,-0.1806999909535049742004275685758329927921295166015625f,0.67289998895525915401094607659615576267242431640625f,0.594800005149841215512651615426875650882720947265625f,-0.1120999915614724340429120275075547397136688232421875f,0.6346000278830530305640422739088535308837890625f,0.2900000119209291682409457280300557613372802734375f,-0.4220000071376561745495337163447402417659759521484375f,0.51109999790191651403148398458142764866352081298828125f,0.1853999863475561671322111578774638473987579345703125f,-0.455999995894730114631698825178318656980991363525390625f,-0.16700001913309137080432265065610408782958984375f,0.099999997913837290752780972979962825775146484375f,0.20270000598877668807062946143560111522674560546875f,-0.9181000126391649640567038659355603158473968505859375f,-0.43579999139606939717594968897174112498760223388671875f,0.701300005082786004351191877503879368305206298828125f,1.185800012779235856896775658242404460906982421875f,0.591399997031688773319046958931721746921539306640625f,-0.7809999976828694290276189349242486059665679931640625f,0.283000003516673981351914335391484200954437255859375f,0.0147999950170518701497712754644453525543212890625f,-0.323499994702637183596749537173309363424777984619140625f,-0.7629000326156620825912568761850707232952117919921875f,-0.112199980753659556143020381568931043148040771484375f,0.85710002728700640517445208388380706310272216796875f,0.167699992763996164057971327565610408782958984375f,0.3260000165700915886191069148480892181396484375f,0.2620000101923942725790084296022541821002960205078125f,0.50919999854564679253599024377763271331787109375f,-0.2232999977290628379478221177123486995697021484375f,0.26730000485181815150781403644941747188568115234375f,-0.607600004391371850687164624105207622051239013671875f,-0.972800043535233083247248941916041076183319091796875f,0.6072999816060065114697863464243710041046142578125f,0.051600000636279563792641056352294981479644775390625f,0.010800008308887498742478783242404460906982421875f,-0.2070999875009060797736992753925733268260955810546875f,-0.18239999516159299819406669485033489763736724853515625f,-0.53280000136494631846062475233338773250579833984375f,0.265299968284369303006542395451106131076812744140625f,0.5125999654993418364057333747041411697864532470703125f,0.9753999926805487774572611670009791851043701171875f,-0.53680000520944570752135405200533568859100341796875f,0.8253000107899304271796836474095471203327178955078125f,0.4149000487685210458721485338173806667327880859375f,-0.4653999914884565658468318360974080860614776611328125f,0.7463999736368658322049896014505065977573394775390625f,0.4572000096619124320795890525914728641510009765625f,0.094599986755847798036711537861265242099761962890625f,0.63320001617372001501138356616138480603694915771484375f,0.37000003844499662619682567310519516468048095703125f,0.15269999876171336250507692966493777930736541748046875f,0.09880000139474864884192584213451482355594635009765625f,-0.3279999899193644097294964012689888477325439453125f,-0.40699998515844271196328918449580669403076171875f,-0.880300011684000072165190431405790150165557861328125f,-1.229900066649914691652156761847436428070068359375f,-0.0713000006869435321021910567651502788066864013671875f,-0.19009999988377102031478216304094530642032623291015625f,0.58790000445246703986157399413059465587139129638671875f,0.06389999032616611174262288841418921947479248046875f,0.8616999948948620424715727494913153350353240966796875f,0.186500032007694382940599098219536244869232177734375f,0.267199984925985400963099891669116914272308349609375f,0.24170000066906205393735262987320311367511749267578125f,0.12250001624226580787535567651502788066864013671875f,-0.5567000036865465961710697229136712849140167236328125f,-0.2085000021904708233932979055680334568023681640625f,0.3663000083237886173037622938863933086395263671875f,-0.440799997133016763228852141764946281909942626953125f,0.78089999958723754769351899085449986159801483154296875f,1.00050005209446002396589392446912825107574462890625f,-0.04129998738765738863776277867145836353302001953125f,0.356300005231797722871078804018907248973846435546875f,-0.15510000472664842163794673979282379150390625f,0.0406000115215776613553089191555045545101165771484375f,0.33110000125467775600185404982767067849636077880859375f,0.69850001925230031929459073580801486968994140625f,0.333400001263618239732977599487639963626861572265625f,-0.4939000097721815141227352796704508364200592041015625f,-0.8929000108599662866026847041212022304534912109375f,-0.600200011837482616527950085583142936229705810546875f,0.3727999972298741138132527339621447026729583740234375f,-0.18199999313056469674165782635100185871124267578125f,-0.3630000102221966340465542089077644050121307373046875f,-0.190500002548098568144752107400563545525074005126953125f,0.5025000441074372048433360760100185871124267578125f,-0.095600001835823178453210857696831226348876953125f,-0.8044000191748141848080422278144396841526031494140625f,0.9863000186800956470278833876363933086395263671875f,0.266999987840652153181508765555918216705322265625f,-0.8850000051036477355381748566287569701671600341796875f,0.4014999952018261097919094027020037174224853515625f,0.0601999786078935006372603311319835484027862548828125f,-0.57529999004006382445908229783526621758937835693359375f,-0.723899993753433168564015431911684572696685791015625f,-0.206599977874756035589598468504846096038818359375f,0.6988999890595675168469824711792171001434326171875f,0.033699995326995857425345093361102044582366943359375f,0.1006000027298927790297966566868126392364501953125f,0.089600002737343309444639771754737012088298797607421875f,0.67990001010000700176760801696218550205230712890625f,0.43160000551640942401121492366655729711055755615234375f,-0.293000006981194049959782432779320515692234039306640625f,-0.6741000201195479490934303612448275089263916015625f,-0.28559999088346910145475021636229939758777618408203125f,0.60750001765787609020463833076064474880695343017578125f,-0.2305000017285347979623111314140260219573974609375f,-0.34700001794099843976937336265109479427337646484375f,-0.161500004738569291617977796704508364200592041015625f,0.0283999824881551177213623304851353168487548828125f,-0.24440003213882466326367648434825241565704345703125f,-0.40179999209642414115251085604541003704071044921875f,0.37430000520944606279272193205542862415313720703125f,0.02119998817443846661490169935859739780426025390625f,-0.42140000008642675144443501267232932150363922119140625f,-0.55500000253319736742696477449499070644378662109375f,0.0288999636530871129025399568490684032440185546875f,-0.212699999842047571263492500293068587779998779296875f,0.52529997655451321492137140012346208095550537109375f,0.9393000026762481358133527464815415441989898681640625f,0.062099996590614381375416996888816356658935546875f,0.394699993717670560844368310426943935453891754150390625f,-0.061700004804134511715574262780137360095977783203125f,0.246300004971027330924471243633888661861419677734375f,-0.9187999959185717546716887227375991642475128173828125f,-0.529600036227703352409434955916367471218109130859375f,0.52849998386204244393837825555237941443920135498046875f,-0.1539999996349215027890977580682374536991119384765625f,-0.014000007048249329955069697462022304534912109375f,0.68759997254014049161696675582788884639739990234375f,-0.1862999841466546069312926192651502788066864013671875f,-0.7107000245928762893754537799395620822906494140625f,0.50429998002648357413590929354541003704071044921875f,0.8385999769285323868217574272421188652515411376953125f,0.3830000059008593638054662733338773250579833984375f,0.2522999979227780187329699401743710041046142578125f,-0.7254999644756321419691857954603619873523712158203125f,-1.4818999902665608292551269187242724001407623291015625f,-0.330299991083145005887899969820864498615264892578125f,-0.20729999993443481987043242043000645935535430908203125f,0.35069997944235797859846570645458996295928955078125f,-0.1588999955415726361707129399292171001434326171875f,0.1393999986484646402828957434394396841526031494140625f,0.1678000094115734697197694913484156131744384765625f,-0.13119999983459695869214556296356022357940673828125f,0.82969996827393810434614351834170520305633544921875f,0.60849999235570362277059075495344586670398712158203125f,-0.104000008352100881126034437329508364200592041015625f,0.218400001710653046682608646733569912612438201904296875f,0.97800003980100191913749085870222188532352447509765625f,0.431499995797872681890794410719536244869232177734375f,-0.51050001178681814195670085609890520572662353515625f,-0.524500042974949565177666954696178436279296875f,0.485799995195865808028656829264946281909942626953125f,-0.58760001463592015369385990197770297527313232421875f,-0.88840003851652227240265347063541412353515625f,0.351599991434812242374619017937220633029937744140625f,0.3504999994561075471466438102652318775653839111328125f,0.2093000218987466354292337200604379177093505859375f,-0.0764999978095293187152492464520037174224853515625f,0.182399997322261364640638703349395655095577239990234375f,0.77329998632967456018150187446735799312591552734375f,0.41699999283254196580372763492050580680370330810546875f,-0.452099978001415714334854101252858527004718780517578125f,0.48480001811385164955936488695442676544189453125f,0.1955999894678586148444310310878790915012359619140625f,-0.48720002113580707092665988966473378241062164306640625f,-1.114699998784065115842167870141565799713134765625f,-0.792200000870227682980839745141565799713134765625f,0.495499996364116679359312911401502788066864013671875f,-0.17890000746250134255888042389415204524993896484375f,0.10000000089406935188662828295491635799407958984375f,0.32509999209791418106618721139966510236263275146484375f,0.1022999967679381061458343538106419146060943603515625f,0.16480000477731227004341008068877272307872772216796875f,0.0737000016599893559288148026098497211933135986328125f,0.671300015401840255435672588646411895751953125f,0.4396000031471256619397536269389092922210693359375f,-0.4918000023037194523567450232803821563720703125f,-0.6010999913454053711348024080507457256317138671875f,-0.438399995303154099701714585535228252410888671875f,0.3231999914377927662911815787083469331264495849609375f,0.669399998536705620466591426520608365535736083984375f,0.7480000281035901199544468909152783453464508056640625f,0.0779000092580914849094142482499592006206512451171875f,-0.6943000028252601651956865680404007434844970703125f,-0.0605999809741977202293128357268869876861572265625f,0.7685999885514378515694033922045491635799407958984375f,-0.48549997013807288936959594138897955417633056640625f,-0.32759997507333782351679474231787025928497314453125f,0.33269998199790717929857919443747960031032562255859375f,0.2540000019073485049148075631819665431976318359375f,0.058000000834464771060083876363933086395263671875f,-0.1425999983191490461109651732840575277805328369140625f,-0.0243000042781231773147965213865973055362701416015625f,0.07919999534189703371822588451323099434375762939453125f,0.0608999966442584916848090870189480483531951904296875f,0.3239000056743623900956663419492542743682861328125f,0.52480002392530433752426688442938029766082763671875f,0.28729999423474072983708538231439888477325439453125f,-0.046900004124641014868757338263094425201416015625f,0.09619999115467070538443294935859739780426025390625f,0.0753999916747212228784746912424452602863311767578125f,0.829199970531463659284554523765109479427337646484375f,0.6182000026881695475822198204696178436279296875f,-0.013700011606514461703909546486102044582366943359375f,0.70749999716877909339274310696055181324481964111328125f,0.34070005163848426921191503424779511988162994384765625f,0.08220001375973229329474634141661226749420166015625f,-0.41820000611543617363707880940637551248073577880859375f,-1.1046000572383409821242139514652080833911895751953125f,0.416499990433454314597838674671947956085205078125f,-0.624699980977177826702018137439154088497161865234375f,0.152499999403953978571735206060111522674560546875f,-0.386399972444773087687508450471796095371246337890625f,-0.1712000028520821448552169385948218405246734619140625f,-0.1252000127315522348681042785756289958953857421875f,-0.215999994926154570418219691418926231563091278076171875f,0.158999995015561612632382093579508364200592041015625f,0.12660000227540724182517806184478104114532470703125f,-0.2227999945104119916550189373083412647247314453125f,-0.6292999598726634058465378984692506492137908935546875f,-1.6369000062108032267360613332130014896392822265625f,0.26779999361634221344274919829331338405609130859375f,0.1199000023886560573060933165834285318851470947265625f,0.7891000127434733002473876695148646831512451171875f,-0.76859999946653767555204694872372783720493316650390625f,0.8296999757245184792253667183103971183300018310546875f,0.56170000309050038822533679194748401641845703125f,-0.6066999748528001390468489262275397777557373046875f,-0.5774000035062429869725519893108867108821868896484375f,-0.2722000473618511051654422772116959095001220703125f,0.1765999821960928528596923570148646831512451171875f,0.37479999203681957364864274495630525052547454833984375f,1.06159999068975441360862532746978104114532470703125f,0.1809999855756760922531611868180334568023681640625f,0.86439996694624454676159075461328029632568359375f,1.1012000084400170862863888032734394073486328125f,-0.199999980516732123181355973429162986576557159423828125f,-0.4043999822571875402132945964694954454898834228515625f,-0.61679999891370496101217213436029851436614990234375f,-0.674399981921911528814916891860775649547576904296875f,0.8397999945700167501172472839243710041046142578125f,0.91940003184080154596813372336328029632568359375f,-0.267099967533350746151654675486497581005096435546875f,-0.6355999693140390593981692290981300175189971923828125f,-0.9567999953746788577291226829402148723602294921875f,-0.602799966198206416123639428406022489070892333984375f,-0.197699987064301918593400841928087174892425537109375f,1.0435000162720680094707859097979962825775146484375f,0.842000020325183839986493694595992565155029296875f,-0.3251000045403837912516564756515435874462127685546875f,1.407199988055229056271855370141565799713134765625f,1.04439999518394444066871074028313159942626953125f,-0.564399993023276369541463282075710594654083251953125f,-0.2054000218868257121584974811412394046783447265625f,0.0946000117897991543713942519389092922210693359375f,0.4619000193238258500372239723219536244869232177734375f,0.1973000073850155189347788109444081783294677734375f,0.001999995887279393258495474583469331264495849609375f,-0.23480000365972519826840425594127736985683441162109375f,0.96830002171993267978677977225743234157562255859375f,0.70460001357793800025319796986877918243408203125f,-0.42470001048147676403488048890721984207630157470703125f,0.7854999942779539168213887023739516735076904296875f,0.42959998943805732807277308893389999866485595703125f,-0.5921999873101710942790987246553413569927215576171875f,0.7299000395298005372524130507372319698333740234375f,0.144200007784366590612989966757595539093017578125f,-0.6899000124096870134593473267159424722194671630859375f,0.0578000011786816703107660941896028816699981689453125f,0.19200001399219057685030520588043145835399627685546875f,0.11640000384896997953632080680108629167079925537109375f,-0.27470000250935555641262908466160297393798828125f,-0.1962000005424020532274198558297939598560333251953125f,0.10770000379085542530077646006247960031032562255859375f,0.27069998048543908453211770392954349517822265625f,-0.14240003565549841368920169770717620849609375f,-0.591099985273182415568271608208306133747100830078125f,0.738600007736682773185066253063268959522247314453125f,0.46140000239610667875922445091418921947479248046875f,-0.426600000338256346577026079103234224021434783935546875f,-0.607500028014182902325046597979962825775146484375f,-0.20140002057552397474182726000435650348663330078125f,0.4521999937549232573275048707728274166584014892578125f,0.50330002619028135768530773930251598358154296875f,0.016999994993209810445478069595992565155029296875f,-0.528200011740624919553965810337103903293609619140625f,-0.90260001083612451111548580229282379150390625f,-0.637400006747245839733295724727213382720947265625f,0.40940000259727238329787724069319665431976318359375f,0.190999992728233536354309762828052043914794921875f,0.17290005049705570172591251321136951446533203125f,-0.58099999287724468644000808126293122768402099609375f,0.7543999936938285966192552223219536244869232177734375f,0.1012000096321106212826634873636066913604736328125f,-0.6101999787569045707869008765555918216705322265625f,0.862599969112873399268437424325384199619293212890625f,-0.086199995326996514677375671453773975372314453125f,-0.369500001549720735738446819595992565155029296875f,0.5449999924376605076048463160987012088298797607421875f,1.0042000423550607735023731947876513004302978515625f,0.2027999940633773423570573868346400558948516845703125f,0.1692999956354499513366818064241670072078704833984375f,0.0880999998986720100901948171667754650115966796875f,-0.3982999908745288308864473947323858737945556640625f,0.457300004926323833132073559681884944438934326171875f,-0.267099995100497977773557067848742008209228515625f,-0.068600001925229747712364769540727138519287109375f,-0.21129998727589860418873968228581361472606658935546875f,-1.09009999712705596408568453625775873661041259765625f,0.48669997068047532451373626827262341976165771484375f,0.7887000025540589209782638135948218405246734619140625f,0.5523000224351886089380059274844825267791748046875f,-0.329200000594555841626487335815909318625926971435546875f,-0.15499999474734071913673005838063545525074005126953125f,-0.3238000106707215053347681532613933086395263671875f,0.100499994948506365943785567651502788066864013671875f,-0.2424999889731408730941808471470721997320652008056640625f,-0.0988999942004678000984085883828811347484588623046875f,0.53359999119639400788628336158581078052520751953125f,-0.4678000036001204176017154168221168220043182373046875f,-0.9338000124216083719375092186965048313140869140625f,-0.158300000709294952372374609694816172122955322265625f,0.2366999937027693956537177655263803899288177490234375f,-0.3633000090539457715976823237724602222442626953125f,-0.253099958771467736795557357254438102245330810546875f,-0.85740001065880022679266403429210186004638671875f,-0.75540004632473056744856876321136951446533203125f,0.51129998723864478193945615203119814395904541015625f,0.8134000113591548508651385418488644063472747802734375f,0.8114000542521484504732143250294029712677001953125f,0.3302999922752380967949648038484156131744384765625f,-0.481900004348158716993566486053168773651123046875f,0.15969998209476443662424571812152862548828125f,0.49279999912977157094928770675323903560638427734375f,-0.150499997090548287559386153589002788066864013671875f,-0.62030001850128169138542943983338773250579833984375f,-0.58300002408027662426093229441903531551361083984375f,0.12940000505596394031471163543756119906902313232421875f,-0.82020002137422576282688169158063828945159912109375f,-0.4098000021696091010880991234444081783294677734375f,0.57480000749677417903882314931252039968967437744140625f,-0.5819000038266182173174456693232059478759765625f,-0.376400004482269512351422235951758921146392822265625f,0.271699995155632478827101294882595539093017578125f,0.708099999153613879343538428656756877899169921875f,0.38619998221397366933160810731351375579833984375f,-0.608200004364550128599375966587103903293609619140625f,-0.3921000067234039487829022618825547397136688232421875f,-0.191399997776746655375745831406675279140472412109375f,0.28470000325441358057787510915659368038177490234375f,0.737700013923645148139485172578133642673492431640625f,0.18199998396635042041680208058096468448638916015625f,-0.744200005921721530199874905520118772983551025390625f,-0.5265000215768811386851666611619293689727783203125f,0.085800011289119737512010033242404460906982421875f,0.8223000020951032329463714631856419146060943603515625f,-0.462300014910101975118550399201922118663787841796875f,-0.16419999325573420012602809947566129267215728759765625f,0.45220001100301743246490104866097681224346160888671875f,0.24580000503063192951458404422737658023834228515625f,-0.07140000656843170645515783689916133880615234375f,-0.36389999464750288904468789041857235133647918701171875f,-1.121399992561340042129813809879124164581298828125f,-0.8410000079274180251331927138380706310272216796875f,0.418899983769655204213222532416693866252899169921875f,0.6295999810934065354928179658600129187107086181640625f,0.460399989551306010326925388653762638568878173828125f,-0.30739998469352725063430398222408257424831390380859375f,0.9603000134348871341671838308684527873992919921875f,0.850000019967556141864406527020037174224853515625f,-0.2101000043168663733439416319015435874462127685546875f,-0.460199989783763641071345773525536060333251953125f,-0.0179999808669091265755923814140260219573974609375f,0.530699985142052188535899404087103903293609619140625f,-0.2072000037759542667714640629128552973270416259765625f,-0.07239999624192716698445337897283025085926055908203125f,0.2062000011757016137980969006093800999224185943603515625f,-0.024199999737739563698823985760100185871124267578125f,0.10980000693798075417362269945442676544189453125f,-0.08710000553131092004832680686376988887786865234375f,0.050300015223026406374629004858434200286865234375f,-0.099300000739097615820583087042905390262603759765625f,-1.3826999610245229721527948640868999063968658447265625f,0.82669998953044387945965354447253048419952392578125f,0.23469999680966135979787168253096751868724822998046875f,-0.9055999857425689469891949556767940521240234375f,1.3665999933719632508655195124447345733642578125f,0.58740000331997865856692442321218550205230712890625f,0.478499994516372595398934208787977695465087890625f,-0.9155000061988829429537872783839702606201171875f,-0.291200001734495117489132098853588104248046875f,0.8829999978542328875619205064140260219573974609375f,-0.87199999558925611609083716757595539093017578125f,-0.3756000049650669581069450941868126392364501953125f,0.3477000125825402410129072450217790901660919189453125f,-0.61590002979934244109472274431027472019195556640625f,0.02110000136643652712820085071143694221973419189453125f,-0.2840999918818472025350274634547531604766845703125f,1.0867000143408773737974115647375583648681640625f,0.274800001126527693173784427926875650882720947265625f,0.588599993729591375313248136080801486968994140625f,0.4637999690592291557322823791764676570892333984375f,0.29859999402761461251287755658267997205257415771484375f,0.320600003996491356161868679919280111789703369140625f,-0.56040000378191479057932156138122081756591796875f,-0.17730000030696391188911320568877272307872772216796875f,0.4301000015974043577671181992627680301666259765625f,-0.08290001771450050682688015513122081756591796875f,0.0494999993145466277155719581060111522674560546875f,-1.01929996736049677252822220907546579837799072265625f,0.782099980795383231679807067848742008209228515625f,0.231099996113777184092441530083306133747100830078125f,-0.398399988746643174408745835535228252410888671875f,-0.57139998421668991568367346189916133880615234375f,-0.532900006389617875157682647113688290119171142578125f,0.22889999867081645135158396442420780658721923828125f,0.61530001358389796450865105725824832916259765625f,0.2553000114977359569223835933371447026729583740234375f,0.0318000198125840682905618450604379177093505859375f,-0.15879999036788916555451578460633754730224609375f,-0.30540001130700122899952475563623011112213134765625f,-0.3568999704480173562615163973532617092132568359375f,0.8461999851942056949383186292834579944610595703125f,-0.127400000041723249211145230219699442386627197265625f,-0.013999999225139614367208196199499070644378662109375f,0.4934999834895135162327051148167811334133148193359375f,0.219799993079900612968913264921866357326507568359375f,-0.166999992012977660493788789608515799045562744140625f,0.0967000007063150146535690510063432157039642333984375f,-0.5828000085175037980889101163484156131744384765625f,-0.5374000331223012238979208632372319698333740234375f,0.63739996681213284546174691058695316314697265625f,-0.271800008934736325727499206550419330596923828125f,0.159800000381469686772106797434389591217041015625f,0.0421000120133163679980725646601058542728424072265625f,-0.60279998691081981831985103781335055828094482421875f,0.84519998829364784143081124057061970233917236328125f,1.515399981951713215266863699071109294891357421875f,0.013999997735023583800284541212022304534912109375f,-0.195999995708465490906746708787977695465087890625f,-0.55459997662305848820096798590384423732757568359375f,-0.232800011199712830745056635350920259952545166015625f,0.226800008636713013032704111537896096706390380859375f,0.4440000180006027363788234652020037174224853515625f,-0.18240000242590903578587813171907328069210052490234375f,-0.00600000301003456026904814279987476766109466552734375f,0.119199999514222110263972354005090892314910888671875f,0.636600007677078139067816664464771747589111328125f,0.003200008201599047197305480949580669403076171875f,-0.1873000173687930924870670423842966556549072265625f,0.416600003653764705546791446977294981479644775390625f,0.007400006449222562565637417719699442386627197265625f,-0.7656999854773280933528667446807958185672760009765625f,-0.20600000464916234932388761080801486968994140625f,0.492000003933906526754071819595992565155029296875f,0.513000013649463681986162555404007434844970703125f,0.12620001082420362337188635137863457202911376953125f,-0.28340000454187386225157752051018178462982177734375f,0.167499975115060539820888152462430298328399658203125f,0.55080000830888753426961557124741375446319580078125f,0.01020001720190055749526436557061970233917236328125f,-0.55619999055862390235915881930850446224212646484375f,-0.70340000022053705830415992750204168260097503662109375f,0.32059999185204512972546808668994344770908355712890625f,0.735099995115399362788366488530300557613372802734375f,-0.65900001680850994745242132921703159809112548828125f,0.149299985688924774507313486537896096706390380859375f,0.594299997162818893770008799037896096706390380859375f,-0.15470000146627427284329314716160297393798828125f,-0.11760000137388704199992162102716974914073944091796875f,0.249800020349025597710124202421866357326507568359375f,0.2115999902427194712117852759547531604766845703125f,-0.0644000049442052802106672970694489777088165283203125f,0.63369999890327477487517171539366245269775390625f,-1.18629999454021373139767092652618885040283203125f,-0.628600017422437762348863543593324720859527587890625f,-0.089800006937980736410054305451922118663787841796875f,-0.2812000056087970989437962998636066913604736328125f,0.0244000034540892141166068540769629180431365966796875f,1.621999976515770214291478623636066913604736328125f,-0.149600021624564760713838040828704833984375f,0.18519999574422829624609221355058252811431884765625f,-0.7031999831676483125875165569595992565155029296875f,-0.16759998133182563151422073133289813995361328125f,-0.05019999395608909509292061557061970233917236328125f,0.4515000147223471316237919381819665431976318359375f,-0.14130001391172442026800126768648624420166015625f,0.02370000069141386944693294935859739780426025390625f,0.073199997696280494352549794712103903293609619140625f,-0.261999997898936243245771038345992565155029296875f,-0.07419999966323376217047780301072634756565093994140625f,0.048799994540214441940406686626374721527099609375f,0.921399985408783095408580265939235687255859375f,0.21389999613761911945175597793422639369964599609375f,-0.818400000369548852319212528527714312076568603515625f,0.5514000013530253596627517254091799259185791015625f,0.2362000040441751824715765906148590147495269775390625f,0.52399999952316278495345613919198513031005859375f,0.09399997746944421805892488919198513031005859375f,0.202999999642372142005797286401502788066864013671875f,0.31340000364780440378353887354023754596710205078125f,-0.003199985849857256425821105949580669403076171875f,-0.1814000038862229047254004399292171001434326171875f,-0.4793999916076661094166411203332245349884033203125f,0.76860000505447345631182543002068996429443359375f,0.2469000066578388441485003568232059478759765625f,0.605799981412291632665301222004927694797515869140625f,-0.740399996182322439608469721861183643341064453125f,-0.09059999565184118086591524843242950737476348876953125f,0.923999987006187240012877737171947956085205078125f,0.2925999887078998273892693759989924728870391845703125f,-0.75939999399185165174230860429815948009490966796875f,0.01639997408390048150295115192420780658721923828125f,0.98589996073245966812237384147010743618011474609375f,0.062199998188018579270419650129042565822601318359375f,0.642599986696243075101619979250244796276092529296875f,0.1843000089347359704561313265003263950347900390625f,0.238000007688999204447100055404007434844970703125f,-0.345200017201900521968127577565610408782958984375f,-0.601000011205673700942497816868126392364501953125f,-0.4230000155866147082406314439140260219573974609375f,0.6430000033676623871770061668939888477325439453125f,0.30770005058050120538837290951050817966461181640625f,0.4044000054657459486406878568232059478759765625f,0.5692000177681446615451932302676141262054443359375f,-0.406199962991475871376678696833550930023193359375f,-0.1082000140130521259607121464796364307403564453125f,0.0791999972045420719268804532475769519805908203125f,-0.458499963372946073292268920340575277805328369140625f,0.44519999499917017882211212054244242608547210693359375f,-0.31159999113678937820992587148793973028659820556640625f,-0.518199992480874005451596531202085316181182861328125f,-0.3810000000298021660682934452779591083526611328125f,-0.3298000041067599141797472839243710041046142578125f,0.54229998518228494930326633038930594921112060546875f,0.4510000106096268002175975198042578995227813720703125f,-0.1833999990284442827004340870189480483531951904296875f,-0.3633000156849621742338740659761242568492889404296875f,0.46359999983906730047777955405763350427150726318359375f,0.43320001013874998019304030094644986093044281005859375f,-0.04899998216331002964807339594699442386627197265625f,0.39980000976920138100467738695442676544189453125f,0.208400017207861054657769273035228252410888671875f,0.1307000076055526704976728069595992565155029296875f,0.22459998437166195373038135585375130176544189453125f,-0.92720000504255306594814101117663085460662841796875f,-0.414900002276898050013187457807362079620361328125f,0.2746000007629394445984871708787977695465087890625f,-0.4045999959945678625672371708787977695465087890625f,-0.7359000045418735425073464284650981426239013671875f,-0.6508000201180579313842144983937032520771026611328125f,-0.0970000118687750212398412941183778457343578338623046875f,0.4817999995283781533572664557141251862049102783203125f,0.41780001775622377380159377935342490673065185546875f,-0.1879000058680773133090724513749592006206512451171875f,0.9467999703407290468248902470804750919342041015625f,-1.01899998998641905245676753111183643341064453125f,-0.35110000505447391816460367408581078052520751953125f,0.1969999815821648070368610206060111522674560546875f,0.567599953317643013406268437393009662628173828125f,0.4311999880254269168489145158673636615276336669921875f,-0.64260000576972942099018837325274944305419921875f,0.0506000107765203210874460637569427490234375f,-0.0806000069022179133071404066868126392364501953125f,0.1339000158071517176949782879091799259185791015625f,-0.13430002815723440789952292107045650482177734375f,0.272900000578164991793528315611183643341064453125f,-0.41509999146461495911353267729282379150390625f,0.966899983561038567358991713263094425201416015625f,0.5653000068783760045931785498396493494510650634765625f,-0.66549998831748968797228371840901672840118408203125f,-0.141700010168552381628614966757595539093017578125f,0.0887999991595744830874537001363933086395263671875f,-0.440599980601668494983869095449335873126983642578125f,0.386199998828768542580291978083550930023193359375f,-0.00940000211298464005693631406757049262523651123046875f,0.5777000133275986826220105285756289958953857421875f,-0.74929997808933279657139792107045650482177734375f,-0.41430000059008609269994849455542862415313720703125f,-0.248300003987550699235953288734890520572662353515625f,0.4999000055551530152797568007372319698333740234375f,0.057699998500943205925750589813105762004852294921875f,0.100400009146332624965225477353669703006744384765625f,0.98719996562898160163967986591160297393798828125f,0.43799999494850638814824606015463359653949737548828125f,0.334500020027160616109540569595992565155029296875f,0.74789999506473492374425404705107212066650390625f,0.239800001871585788393304028431884944438934326171875f,-0.334599960231781068387135746888816356658935546875f,-0.36999998986721038818359375f,-0.50950001227855690189016968361102044582366943359375f,-0.796800003719329907880819519050419330596923828125f,0.05020001928806294699825230054557323455810546875f,-0.091199994581937726678688704851083457469940185546875f,0.34469999578520649219370852733845822513103485107421875f,-0.9841000075094399335284833796322345733642578125f,-0.324600004357844529323529059183783829212188720703125f,-0.94410003569126121902854720246978104114532470703125f,-0.954600036786497074814406005316413938999176025390625f,0.04500000178813934326171875f,0.25939999756813048037429325631819665431976318359375f,-0.3302999829620123506401796475984156131744384765625f,-0.65660001900196096613626650650985538959503173828125f,0.65880000050067888395233239862136542797088623046875f,1.3525000122189521079008045489899814128875732421875f,-0.51219999505877478185311701963655650615692138671875f,0.36179999850392352556127661955542862415313720703125f,0.7918999916076661094166411203332245349884033203125f,-0.6632000099897386036218449589796364307403564453125f,0.5061999923467634943108350853435695171356201171875f,1.301599984133243737005614093504846096038818359375f,0.85660001840591437627381310448981821537017822265625f,-0.03219999818801877466967198415659368038177490234375f,-0.9240999957561495392610595445148646831512451171875f,0.27239999199509623561965554472408257424831390380859375f,-0.79719997896552075022924555014469660818576812744140625f,-0.75579999214112714156499350792728364467620849609375f,0.051800006401538922773397644050419330596923828125f,-0.0485999928355216326281151850707828998565673828125f,0.405499978035688712907358421944081783294677734375f,-0.31820001773834238889548942097462713718414306640625f,-0.01940000472068792447544183232821524143218994140625f,-0.36269995908737229228790965862572193145751953125f,-0.4567999947786329784094050410203635692596435546875f,1.0421999987840653290049885981716215610504150390625f,0.697600031697749756176563096232712268829345703125f,-0.02999999649822715497293756925500929355621337890625f,0.052600000628828968984862513025291264057159423828125f,0.43539996977001482747482441482134163379669189453125f,-0.47700001734495156568982565659098327159881591796875f,-0.18379998410940157071991052362136542797088623046875f,-0.25969996123313876523752696812152862548828125f,-0.3662000158905984736890104613848961889743804931640625f,-0.5986000196576120035985013601020909845829010009765625f,-0.032300017368793731975529226474463939666748046875f,0.231000001966953139032057151780463755130767822265625f,0.537599991762637952064096680260263383388519287109375f,0.00470001554787113917655005934648215770721435546875f,0.22440001232028006228347294381819665431976318359375f,0.49779999749064440806023412733338773250579833984375f,0.5273999832034110823997252737171947956085205078125f,0.7816000217959284679380971283535473048686981201171875f,-0.327300010029971610858723352066590450704097747802734375f,-0.44590000692605968879433930851519107818603515625f,0.2234999934732915249924189993180334568023681640625f,0.23810000168681144661064763567992486059665679931640625f,1.403600002670287949513294734060764312744140625f,-0.376200043606758072201046161353588104248046875f,0.065499993383884458353350055404007434844970703125f,0.76509998550415048157447017729282379150390625f,-0.42509999682903298889868892729282379150390625f,-0.0150999948918819626442200387828052043914794921875f,1.351199978041648819271358661353588104248046875f,-0.499600030565261477022431790828704833984375f,-0.033500005394220266907723271287977695465087890625f,-0.8179999948740004356295685283839702606201171875f,1.1558000002622605961732915602624416351318359375f,0.35959999703764911060943632037378847599029541015625f,-0.777999968871474489873207858181558549404144287109375f,-0.2359999621063479935401119291782379150390625f,-0.35759998900592326886993532752967439591884613037109375f,0.5655999970674514543134137056767940521240234375f,0.076200005757808497719452134333550930023193359375f,-0.0765999979913234430028978749760426580905914306640625f,-0.625199981141090432856799452565610408782958984375f,-0.51260000278949746643775142729282379150390625f,-0.05340000022053725814430436003021895885467529296875f,0.586399991965294020701549015939235687255859375f,0.1355999750137328874188824556767940521240234375f,0.48010000323653212461749717476777732372283935546875f,0.555099977308511682849712087772786617279052734375f,0.03109999529421347830293598235584795475006103515625f,-0.035399999833106876589994271853356622159481048583984375f,-0.543799976658821293540313490666449069976806640625f,-0.854799973559380532606155611574649810791015625f,-0.22679998390078548453629991854541003704071044921875f,-0.14300000426173209433500232989899814128875732421875f,-0.969599992269277066725408076308667659759521484375f,-0.511200008440017672484145805356092751026153564453125f,0.0506999962806702342277276329696178436279296875f,0.7248999618947511436317654442973434925079345703125f,0.43419998930692671468278831525822170078754425048828125f,0.74289998433589943260813015513122081756591796875f,0.731699972319603375581209547817707061767578125f,0.3297999927818775489640756859444081783294677734375f,-0.26410001246407610597799475726787932217121124267578125f,0.81650004094839090384994051419198513031005859375f,0.0594000048696994298325080308131873607635498046875f,-0.6738000259816647741217821021564304828643798828125f,-0.1215000495910647515529490192420780658721923828125f,0.60139999658465370657722814939916133880615234375f,0.32900001785159105338607332669198513031005859375f,-0.735299958300590361659487825818359851837158203125f,0.8374000219464303729210996607434935867786407470703125f,0.3230000208020211704962321164202876389026641845703125f,-0.583299987894296378954095416702330112457275390625f,0.183400007969141132235790792037732899188995361328125f,0.247600010091066469186671383795328438282012939453125f,-0.23589998278617851923399939551018178462982177734375f,0.2788000154763461413409686429076828062534332275390625f,-0.3472000002741812973994228741503320634365081787109375f,0.09339995637387066285128867093590088188648223876953125f,-0.466000004053115812752139390795491635799407958984375f,0.251799996864795705420192462042905390262603759765625f,0.265000005513429837122885146527551114559173583984375f,0.41959998977333301528602760299690999090671539306640625f,-0.66559998123496766364581844754866324365139007568359375f,-0.8025999871991571499307838166714645922183990478515625f,-0.5188000134646895133982980041764676570892333984375f,1.0399999900162217869592495844699442386627197265625f,0.917800022673606719081362825818359851837158203125f,0.22319999438077198217200702856644056737422943115234375f,-0.20859999156892305993693526033894158899784088134765625f,-0.024000007774680531813515926842228509485721588134765625f,-0.4380000105202197513420969698927365243434906005859375f,-0.5321999989330767899531338116503320634365081787109375f,0.053799993199110218711211928166449069976806640625f,0.4160000222325326291183955618180334568023681640625f,-0.9682000210165977449605634319595992565155029296875f,-0.44760007997751216635151649825274944305419921875f,0.34800000303238631804703118177712894976139068603515625f,-0.01780000292956829266444174209027551114559173583984375f,-0.7083999735660857499652820479241199791431427001953125f,0.23400000861287129083621039171703159809112548828125f,0.7388000161468983861823289771564304828643798828125f,-0.11659995857775218297547326073981821537017822265625f,-0.14539999819397930735931367962621152400970458984375f,0.60820000209212299324690320645458996295928955078125f,0.982499994337558746337890625f,0.5400000150501729212493273735162802040576934814453125f,0.284500001102685917686585526098497211933135986328125f,-0.053700019764900019936249009333550930023193359375f,-0.4058999863624563886332907713949680328369140625f,-0.4263000125706195575503443251363933086395263671875f,-0.72989998171329517617778037674725055694580078125f,1.055899976825713793004979379475116729736328125f,0.505500002175569562723467242904007434844970703125f,-0.41839999306797981404315578402020037174224853515625f,1.4113999992668624372527119703590869903564453125f,0.5480000070929527122842728203977458178997039794921875f,-0.25529999495744704063326935283839702606201171875f,0.4265000033974644111367524601519107818603515625f,0.2566000067830085384912308654747903347015380859375f,0.2940999869644642661370426139910705387592315673828125f,0.16890001856386671619247863418422639369964599609375f,-0.17010000021904703171315986764966510236263275146484375f,0.10339998427629470967303859652020037174224853515625f,-0.481399999117851251639876863919198513031005859375f,-0.00109999432563778754001759807579219341278076171875f,-0.78669996835216904340626342673203907907009124755859375f,0.070500020559876475090277381241321563720703125f,-0.2180999944411217972284333654897636733949184417724609375f,0.848699980425834610286983661353588104248046875f,0.061899940347672099960618652403354644775390625f,0.4288999913692475018933691899292171001434326171875f,0.09680000692307932919078439226723276078701019287109375f,0.176399995355307925848364902776665985584259033203125f,-0.074500001493841339073043172902544029057025909423828125f,-0.07049999323487288194201028090901672840118408203125f,-1.02009996926188506449761916883289813995361328125f,-0.4989999932646751457099298932007513940334320068359375f,-0.28380003507137274709748453460633754730224609375f,-0.1209999599456779151296359486877918243408203125f,-0.47730000589489929296860282192938029766082763671875f,-0.77169998207986345573061726099695079028606414794921875f,0.437499998547136836890558697632513940334320068359375f,0.190799996555596484970607207287685014307498931884765625f,-0.1317999879240989713480303180404007434844970703125f,0.8629999839961524088494115858338773250579833984375f,0.59020000706911079912941886504995636641979217529296875f,1.158899993753433221854720613919198513031005859375f,0.30669996814727795708677149377763271331787109375f,0.232499994784593599916888706502504646778106689453125f,-0.4700999969780443876743447617627680301666259765625f,0.29019999324828382736285448117996565997600555419921875f,0.05660000186562541557577787898480892181396484375f,0.694000023514032449156729853712022304534912109375f,-0.1744999705553045288297653314657509326934814453125f,0.6288000094413757068423365126363933086395263671875f,0.5388000058650970203188990126363933086395263671875f,-0.5407999662876132873634560382924973964691162109375f,0.2156000121176244732623672462068498134613037109375f,0.56040002322793025513192333164624869823455810546875f,-0.40419997388422412853969944990240037441253662109375f,0.762800005984306306316966583835892379283905029296875f,-0.289999995827674883486224644002504646778106689453125f,-1.129799983990192657756779226474463939666748046875f,0.46600000539422037348913363530300557613372802734375f,-0.88560000943541528073410518118180334568023681640625f,-0.500200025248528135080050560645759105682373046875f,-0.29199999372661100682790902283159084618091583251953125f,0.20339999396204955228739663652959279716014862060546875f,0.33639999297112221920968977428856305778026580810546875f,0.655999989300966035443707369267940521240234375f,-0.09919999094605458367368555627763271331787109375f,-1.130799953621626219302243043784983456134796142578125f,0.131000009268522177308113896287977695465087890625f,0.3179999911487101371676544658839702606201171875f,0.6042999655723573226850930950604379177093505859375f,-0.410200011092424432490588515065610408782958984375f,-0.5276000053226947983375794137828052043914794921875f,0.4400999770104885300270325387828052043914794921875f,0.53919999169111232362183727673254907131195068359375f,-0.76980000425577177480818136245943605899810791015625f,-0.63720000265836773678529425524175167083740234375f,-0.361599997544288509487842020462267100811004638671875f,-0.141200010228156980218727767351083457469940185546875f,0.01839998993873592780801118351519107818603515625f,0.34900000250339502372298738919198513031005859375f,0.3799999934434890036527576739899814128875732421875f,-0.5126999628126629460211916011758148670196533203125f,0.594800001275539447220808142446912825107574462890625f,-0.710799990427493977307449313229881227016448974609375f,-1.24979998518228541115604457445442676544189453125f,-0.6120000298619274747835561356623657047748565673828125f,0.7993999948859211723828366302768699824810028076171875f,1.1104000271022318457880828646011650562286376953125f,-0.4050000104308129067476329510100185871124267578125f,-0.02549999953061343749283196302712894976139068603515625f,-0.285700014162063808953462284989655017852783203125f,0.56529999048709811404478386975824832916259765625f,0.516400017297268032478996246936731040477752685546875f,-0.484799988460540731693981797434389591217041015625f,1.2631999724388123240714776329696178436279296875f,0.62899999624490732230697176419198513031005859375f,-0.4994000097870827659107817453332245349884033203125f,0.6834000064790242134904474369250237941741943359375f,0.50420001910924916455059019426698796451091766357421875f,-1.1031999816775321932027509319595992565155029296875f,0.8156000045180320512372418306767940521240234375f,0.185599997663497884303751561674289405345916748046875f,-0.339699977922439444455449120141565799713134765625f,0.0071000228166582246558391489088535308837890625f,-0.398700010824203587844749563373625278472900390625f,0.2844999858289958272195008248672820627689361572265625f,-0.25429999507665623781349495402537286281585693359375f,0.0060000024512409932953005409217439591884613037109375f,-1.17669996635913864935218953178264200687408447265625f,0.795299983036517943446597200818359851837158203125f,0.285299995702505082562083771335892379283905029296875f,0.822099972152710023465260746888816356658935546875f,0.47229997355938024838906130753457546234130859375f,0.168599990451336001484605731093324720859527587890625f,0.452300004553794821049450547434389591217041015625f,-0.942699983823299447749377577565610408782958984375f,-0.5434000021576881334084418995189480483531951904296875f,-0.500199966239929239009143202565610408782958984375f,0.1858000014543534916811040602624416351318359375f,-0.27850001373887067757095792330801486968994140625f,-0.1276000144124029844761025742627680301666259765625f,-0.63079997969865786444643163122236728668212890625f,-0.20419999616146089493895487976260483264923095703125f,-0.559399984455108523206945392303168773651123046875f,0.740799974930286708740823087282478809356689453125f,0.4350999914646147548324961462640203535556793212890625f,-1.2905999705433846003188591566868126392364501953125f,0.25700003105401947323116473853588104248046875f,-0.133799995732307497320334732648916542530059814453125f,1.08799999919533707526397847686894237995147705078125f,-0.230800067168473788115079514682292938232421875f,0.2094999875426293289848445056122727692127227783203125f,0.053800008249282882388797588646411895751953125f,-0.7759999884068964348671215702779591083526611328125f,-0.278000003069639234354326617904007434844970703125f,-0.1876000017464161118141419137828052043914794921875f,0.621399968123436341471688137971796095371246337890625f,0.94620002244710921246451107435859739780426025390625f,0.14359998329877843303847839706577360630035400390625f,-1.1908999979853636119742077426053583621978759765625f,-0.528599997603893090314386427053250372409820556640625f,-0.404600004041194782899992787861265242099761962890625f,0.413699966418743514395828242413699626922607421875f,-0.0620000022947788664851032081060111522674560546875f,-0.217400008684396794933491037227213382720947265625f,-0.4886000055015085052900758455507457256317138671875f,0.5632000083506107923625449984683655202388763427734375f,0.0376000178396702633421000427915714681148529052734375f,-0.5787999866425987960383281460963189601898193359375f,-0.4381999901711939315873678424395620822906494140625f,0.7391999897539616171115994802676141262054443359375f,0.793299999964236945970696979202330112457275390625f,0.1365999984383581278279962134547531604766845703125f,0.80699999052286131018263404257595539093017578125f,0.0487000324308866083811153657734394073486328125f,-0.17400000429153461567466365522705018520355224609375f,0.90319998912811261249089511693455278873443603515625f,0.420200020480155700397517648525536060333251953125f,-0.5416000033557413217977227759547531604766845703125f,0.1145999967396260643681671353988349437713623046875f,0.110000025928020050969280418939888477325439453125f,0.224399988627433710774994324310682713985443115234375f,-0.647999984145164464877097998396493494510650634765625f,-0.3837999936461446992552737356163561344146728515625f,-0.058400009757280546551783118047751486301422119140625f,-0.492800019246339982004201374365948140621185302734375f,-0.12040001987516912862474782741628587245941162109375f,0.687600002342462435223069405765272676944732666015625f,0.071400008803605974350148244411684572696685791015625f,-0.69459997721910493595487423590384423732757568359375f,-0.37000002160668399309173537403694353997707366943359375f,-0.36540001488327999989991212714812718331813812255859375f,0.02019999864995458693783803028054535388946533203125f,0.33179998568892443699951400049030780792236328125f,-1.08280001388192204103688709437847137451171875f,-0.5417000423550604182310053147375583648681640625f,-0.26880000333189979500048139016143977642059326171875f,-0.2728000018119811631578386368346400558948516845703125f,0.71349998736381525787919599679298698902130126953125f,-0.4892999953746794972175848670303821563720703125f,-0.2985000006258486937582574682892300188541412353515625f,-1.14940001425743076879371074028313159942626953125f,1.3903999843359002852594130672514438629150390625f,0.54429998852014538357479978003539144992828369140625f,0.3477999830365181566094179288484156131744384765625f,-1.0307999630093576115541509352624416351318359375f,-0.489799991589784600165558003936894237995147705078125f,-0.648900018191337579764876863919198513031005859375f,-0.288899989283084579483329434879124164581298828125f,-0.0963999977767466820210984224104322493076324462890625f,-0.50669998885989198100787689327262341976165771484375f,0.80450000438094093624386005103588104248046875f,0.2725000067055225461132295095012523233890533447265625f,0.7570999781131746431128703989088535308837890625f,-0.045900016164779344762791879475116729736328125f,0.2643999993562697881088752183131873607635498046875f,0.0331999980688095774894463829696178436279296875f,0.782999949276447892998476163484156131744384765625f,0.5847999883115291286372894319356419146060943603515625f,0.367999980568885831644365680404007434844970703125f,0.4275999491453177370203775353729724884033203125f,0.5069999914169311949763141456060111522674560546875f,-0.1204999895095824058444122783839702606201171875f,-0.704099983835220655237208120524883270263671875f,-0.0945999934613706017216827603988349437713623046875f,-0.3263999660372736144609007169492542743682861328125f,-0.1675999810338026918543619103729724884033203125f,-0.322999996960163127113219161401502788066864013671875f,-0.12840000424385067390176118351519107818603515625f,-0.4492000025689595332778480951674282550811767578125f,-0.33680001236200329817194187853601761162281036376953125f,-0.6056999810814858165031182579696178436279296875f,1.00149998372793191947494051419198513031005859375f,0.348399996048212035049118640017695724964141845703125f,-0.102799999204278069697693354100920259952545166015625f,-0.80360000706613021748125902377068996429443359375f,-0.36030000859200950191763013208401389420032501220703125f,-0.433900003290176528025767765939235687255859375f,-0.3348999942302697263585287146270275115966796875f,-0.56660001393556580495669550145976245403289794921875f,1.09109999492168441292960778810083866119384765625f,-0.3634999808073047233847319148480892181396484375f,0.612000030755996959896947373636066913604736328125f,0.317200006532669220860043424181640148162841796875f,-0.733200012969970060794366872869431972503662109375f,-0.5192000132977963033908963552676141262054443359375f,0.5251999964892863914656118140555918216705322265625f,0.341600041204690452190106952912174165248870849609375f,0.0592000051021575046661382657475769519805908203125f,0.6869999949932097393912044935859739780426025390625f,0.2052000034928322946825574035756289958953857421875f,-0.483800018978119084067657240666449069976806640625f,-0.154400020217895672658414696343243122100830078125f,-0.19549998444318816837039776146411895751953125f,-0.8626000209689141939861656283028423786163330078125f,-0.4454000239729882792971693561412394046783447265625f,0.78429997510909998936767806299030780792236328125f,-0.1090000022053718709003078402020037174224853515625f,-0.34240000406503678931358081172220408916473388671875f,0.32429997667372223446591306128539144992828369140625f,0.3662000029265879419426710228435695171356201171875f,-1.0332000184834004841150090214796364307403564453125f,-0.458900057828426355399642488919198513031005859375f,-0.281400004072487330830654173041693866252899169921875f,-0.7850000022724270554164149871212430298328399658203125f,0.2876999520652001596232594238244928419589996337890625f,1.01399999713897681630214719916693866252899169921875f,0.40880001540184007779998864862136542797088623046875f,-1.1489999375343327159271211712621152400970458984375f,0.73660001006126396561057845246978104114532470703125f,0.13660000112056724930198470246978104114532470703125f,-0.93609998925924298163181447307579219341278076171875f,-0.223400010353326905487847398035228252410888671875f,-0.13040000594258316368723171763122081756591796875f,0.47749999672174414655501095694489777088165283203125f,0.3896000028491020117371590458787977695465087890625f,-0.2409999918341636515606296597979962825775146484375f,-0.01760000636577618848832571529783308506011962890625f,0.37480001990199107719803350846632383763790130615234375f,0.31160000573992741035311837549670599400997161865234375f,-0.02380000430047513049203189439140260219573974609375f,0.2897999975502489888867785339243710041046142578125f,-0.4236000001370907597220138995908200740814208984375f,-0.05790001831054691905364961712621152400970458984375f,-0.43980000231862081960798604995943605899810791015625f,-0.1077999995768070817803163663484156131744384765625f,-0.72080003542900072943666600622236728668212890625f,1.0632000141620636668449151329696178436279296875f,0.50020001183748252771010811557061970233917236328125f,0.14770001094341278502497516456060111522674560546875f,-0.357100020730495515408620121888816356658935546875f,0.159499999314546503370593200088478624820709228515625f,0.217999974608421354105303180404007434844970703125f,0.596799966764450573464273475110530853271484375f,0.45769999648928649804702217807061970233917236328125f,-0.4656000030279159318524762056767940521240234375f,-0.838200005817412829856039024889469146728515625f,-0.2179000063896179284483878291212022304534912109375f,0.68659999337196353508261381648480892181396484375f,-1.5187999975204462543842964805662631988525390625f,-0.66800001022219657187406482989899814128875732421875f,0.4179000019192695702940909541212022304534912109375f,-0.3774999921023844962064686114899814128875732421875f,-0.31260000398159026868682985877967439591884613037109375f,-0.3143000132560729298347723670303821563720703125f,-0.24169997887611316400580108165740966796875f,-0.53460001178979865699147921986877918243408203125f,0.341199988994002278985817611101083457469940185546875f,-0.950999966092407778006645457935519516468048095703125f,-0.66019999693632114290409163004369474947452545166015625f,-1.09359996273517623421867028810083866119384765625f,1.683599959158897263478138484060764312744140625f,0.7393999933958053105698127183131873607635498046875f,-0.9096000061273574743836434208787977695465087890625f,1.09939998683929474054821184836328029632568359375f,0.4419999975264072844538532081060111522674560546875f,0.5421000138759612951844246708787977695465087890625f,-0.3805000097751616294772247783839702606201171875f,-0.222800008666515392263818284845910966396331787109375f,0.1866000213861465084619339904747903347015380859375f,0.39820000045299508428797707892954349517822265625f,-0.04930000416636470816911241854541003704071044921875f,0.106599995532631997008365942747332155704498291015625f,0.7717999997705220494026434607803821563720703125f,0.32540000229179855129046927686431445181369781494140625f,0.1083000103950499948268770822323858737945556640625f,0.11729999978542338112674769945442676544189453125f,-0.1974000073432922164329283987171947956085205078125f,0.69009996762275704895728267729282379150390625f,0.095200013178587283846354694105684757232666015625f,-0.11280001239180581507071110536344349384307861328125f,-0.50200001749396339079112294712103903293609619140625f,-0.849599998080730500760182621888816356658935546875f,0.5154000133931637694928440396324731409549713134765625f,0.2582000128209591682804102674708701670169830322265625f,-0.0934999946653842783916843472979962825775146484375f,0.072400006523728410723350634725647978484630584716796875f,0.6306000087648631069470184229430742561817169189453125f,0.395800012220442365418193730874918401241302490234375f,0.30900000414252293268191351671703159809112548828125f,-0.28179999492764462587501839152537286281585693359375f,-1.05189997551441205558830915833823382854461669921875f,-0.35700000870227821536673218361102044582366943359375f,-0.23980000872612006190820466144941747188568115234375f,0.32589998248815543746559342253021895885467529296875f,-0.363200009691715397508460227982141077518463134765625f,-0.297600027525425137042702772305347025394439697265625f,-0.128000015437603309464975609444081783294677734375f,-0.3402000054299829656656584120355546474456787109375f,-0.700599995130300357004671241156756877899169921875f,0.3250999901235100963958757347427308559417724609375f,0.40280002073645615467967218137346208095550537109375f,-0.57480000053048119212917299591936171054840087890625f,-0.3702000401496885473306974745355546474456787109375f,0.58480000395774833776840750942938029766082763671875f,0.01360001220703122015720509807579219341278076171875f,-0.1123999989986419478782408987171947956085205078125f,-0.415800009948015247829289364744909107685089111328125f,-0.061400001502037060419070257921703159809112548828125f,0.8412999940186736136382705808500759303569793701171875f,-0.39160002019405393269835258251987397670745849609375f,0.78760000994205459079466891125775873661041259765625f,1.150600023889541745347742107696831226348876953125f,-0.37780001119971284850862502935342490673065185546875f,0.67320000715851779915510633145458996295928955078125f,1.357100021326541394728337763808667659759521484375f,0.41480000962018959143051688442938029766082763671875f,0.20480000127553932287582938442938029766082763671875f,0.1729999781847002537915614084340631961822509765625f,-0.013999999225139614367208196199499070644378662109375f,-0.397000010192394281460792626603506505489349365234375f,-0.0514000196814539123124632169492542743682861328125f,0.5522000262022022099017704022116959095001220703125f,0.3729000087738036928186602381174452602863311767578125f,-1.116100012207031255684341886080801486968994140625f,-0.59849998855590769153423025272786617279052734375f,-0.253900002396106660995656056911684572696685791015625f,0.08630000988841057818490298814140260219573974609375f,0.9340999923288819672961835749447345733642578125f,0.3397000008702277806804659121553413569927215576171875f,0.4825999982073903016299709634040482342243194580078125f,0.592199978332221377286259667016565799713134765625f,0.395500001467764372709012832274311222136020660400390625f,-1.2126999853134154960798696265555918216705322265625f,0.1615000251531597541543305851519107818603515625f,0.048300007563829350232254000729881227016448974609375f,0.2923999912500381270774596487171947956085205078125f,-0.66819999628066995001063332892954349517822265625f,-0.31460000270009036427154569537378847599029541015625f,-0.3034000218272208115877219825051724910736083984375f,-0.757800000917910665521048940718173980712890625f,-0.290600009286403615504923436674289405345916748046875f,0.353300031852722185021775658242404460906982421875f,-0.973699961054325768827766296453773975372314453125f,-0.6248999964654446959144706852384842932224273681640625f,-0.23390000805854782583992346189916133880615234375f,0.1481000375986099015790387056767940521240234375f,-0.22309999676942826596359736868180334568023681640625f,0.12240001747608175719506107270717620849609375f,0.8591999854326246577329584397375583648681640625f,0.19519999544620514342341266456060111522674560546875f,0.925000015199184399961040980997495353221893310546875f,-0.5824000051081181794643271132372319698333740234375f,-0.2838000051945448465318122543976642191410064697265625f,1.245200008559226745319392648525536060333251953125f,0.0879999459981917198092560283839702606201171875f,0.259599994504451814236745121888816356658935546875f,1.537900027251243528780833003111183643341064453125f,0.0440999582052228333850507624447345733642578125f,0.1555999914050101295970307546667754650115966796875f,0.39779998571872710044772247783839702606201171875f,0.7709999796152118278769194148480892181396484375f,0.23559999408721932212529281969182193279266357421875f,-0.0278000015139578948719645268283784389495849609375f,-0.30939998832941029149878886528313159942626953125f,-0.268300004881620424157517845742404460906982421875f,-0.72719999222755404844065196812152862548828125f,0.218200002893060462927365961149916984140872955322265625f,0.28480000500082969239201702293939888477325439453125f,-0.24159999590516090250957859097979962825775146484375f,-0.116400019681453414932548184879124164581298828125f,0.1721999980390069850955114816315472126007080078125f,0.32199999484419805639845435507595539093017578125f,0.6368999908626076233986168517731130123138427734375f,-0.710599993044137789865999366156756877899169921875f,0.106799987477064206586874206550419330596923828125f,1.21869995419979115780506617738865315914154052734375f,0.49320000179409984042422365746460855007171630859375f,0.35640002787709246234726379043422639369964599609375f,-0.589799955677985821012043743394315242767333984375f,0.518800009441375831187315270653925836086273193359375f,-0.362799999129772121619907920830883085727691650390625f,-0.943799992305040635898194523178972303867340087890625f,-0.31019999753236771056208453956060111522674560546875f,1.01599998944997782501786787179298698902130126953125f,0.5977000285267823898038841434754431247711181640625f,-0.510799992215633213987757699214853346347808837890625f,0.239399985349178390237057101330719888210296630859375f,0.44499998837709409116314418497495353221893310546875f,0.53940001902580281267773898434825241565704345703125f,0.54780001850128190454825016786344349384307861328125f,-0.258699976849555923763546161353588104248046875f,-0.04500000998377817751361362752504646778106689453125f,-0.84720002724528331583542239968664944171905517578125f,-0.0565000290274628724773720023222267627716064453125f,0.28200000214576714796521628159098327159881591796875f,-0.02079999966621404183797494624741375446319580078125f,0.262599992060660980541797471232712268829345703125f,-0.345200006026029626582385390065610408782958984375f,0.25780000374913214500338654033839702606201171875f,0.9377999940633774400566835538484156131744384765625f,-0.434200011807680308351109488285146653652191162109375f,-0.301400026684999655657293260446749627590179443359375f,0.047699980843066924762752023525536060333251953125f,0.848400013631582350370763379032723605632781982421875f,0.227800024312734716858130923355929553508758544921875f,-0.830199972200393432331111398525536060333251953125f,0.3182000093936920848136651329696178436279296875f,0.1282000117778778758292901329696178436279296875f,-0.6260999979019157279935825499705970287322998046875f,-0.359599981540441149263642728328704833984375f,-0.048900000812858335219601713106385432183742523193359375f,0.007299996805191000248669297434389591217041015625f,0.23530000658035277183444122783839702606201171875f,-0.065900000071525521860849039512686431407928466796875f,0.883900002694129938163314363919198513031005859375f,0.538499959051609522475700941868126392364501953125f,0.1785999844908714440094854580820538103580474853515625f,1.2664999673366548194053393672220408916473388671875f,-0.86249998986721010396649944595992565155029296875f,-0.30859999909400936957126759807579219341278076171875f,-0.591299976062774845786407240666449069976806640625f,0.92909999396800913018523715436458587646484375f,0.47190001097917555483718388131819665431976318359375f,-0.96259998639821997556964561226777732372283935546875f,0.421000019550323401062996708787977695465087890625f,0.038200000900030062211953918449580669403076171875f,0.5257999901294709843568853102624416351318359375f,0.3235999591588978319123270921409130096435546875f,0.51909999396800987625510970246978104114532470703125f,0.32179997600317022943272604607045650482177734375f,-0.54059999140500991643421002663671970367431640625f,-0.048499997943639794328873904305510222911834716796875f,-0.154799990248680074955700547434389591217041015625f,1.1625999944448466294488753192126750946044921875f,0.5686000072896479906603417475707828998565673828125f,0.003900011038780348826549015939235687255859375f,-0.677899995064734639527159743010997772216796875f,-0.584600019836425843777760746888816356658935546875f,0.480800014567375200158494408242404460906982421875f,0.4219999848604203407376189716160297393798828125f,0.029799990248680074955700547434389591217041015625f,1.047900002813339170870676753111183643341064453125f,-0.24170001523494732964536524377763271331787109375f,-0.058800002139806739620553344138897955417633056640625f,-0.1039000220656394191109939129091799259185791015625f,-0.8497000026583663156998227350413799285888671875f,-0.339100009614229147558717158972285687923431396484375f,-0.7282999814867974208709711092524230480194091796875f,1.338099998557567005263990722596645355224609375f,0.521600007826089839824135196977294981479644775390625f,-0.694000019788742150694815791212022304534912109375f,0.5054000099658964018090046010911464691162109375f,0.2702000122845173990526745910756289958953857421875f,-0.4969000422716138842815780662931501865386962890625f,1.1100999901980161954639925170340575277805328369140625f,-0.22359998538494085806860312004573643207550048828125f,1.02040001294612903137704051914624869823455810546875f,0.3146000216245656844193945289589464664459228515625f,0.0405999949067830101512299734167754650115966796875f,-0.180400012722611524651483705383725464344024658203125f,-0.444899991063774091770710583659820258617401123046875f,0.39779998676180827743564805132336914539337158203125f,-0.67799997833371161704008045489899814128875732421875f,-1.3006999804854384450436555198393762111663818359375f,0.45680001649707568933678203393355943262577056884765625f,-0.21140000276863568640095536466105841100215911865234375f,-0.5208000151820482148679047895711846649646759033203125f,0.72020000885725021788630328956060111522674560546875f,-0.55140000776052477959865427692420780658721923828125f,-1.49389999524354966098371733096428215503692626953125f,0.47200002166628873823128742515109479427337646484375f,-0.77940000665783859545854284078814089298248291015625f,-1.381600004398823244855520897544920444488525390625f,-0.42040000951886202784635315765626728534698486328125f,1.01119998862147308926751065882854163646697998046875f,0.47500001609325437357256305404007434844970703125f,-0.33360000877976414557224416057579219341278076171875f,-0.29919999690651888357706411625258624553680419921875f,-0.2863999955415721387907979078590869903564453125f,0.198199991363286809331611948437057435512542724609375f,-0.472199998933076958707033554674126207828521728515625f,-0.30890001014471035745145854889415204524993896484375f,0.1626000007033348015994533852790482342243194580078125f,0.0284000057339668376954477935214526951313018798828125f,-0.23949999246001230090996614308096468448638916015625f,0.546400015211105483103892765939235687255859375f,-0.35900000786781305350814363919198513031005859375f,-1.1791999982476237818218578468076884746551513671875f,0.47420002521872550715897887130267918109893798828125f,-0.92720000906586630406991389463655650615692138671875f,-0.447300053727627044963810476474463939666748046875f,0.29820000894665721347109865746460855007171630859375f,0.11880000124573708575326236314140260219573974609375f,0.11269999425411203475277943653054535388946533203125f,0.2358000096499921482973149977624416351318359375f,0.67560001956820503465905858320184051990509033203125f,-0.27649995906651003707565905642695724964141845703125f,-0.115600001686811282297639991156756877899169921875f,-0.2020999993473291400647440241300500929355621337890625f,0.19159997370243075920370756648480892181396484375f,0.429599993014335268526338040828704833984375f,0.588000016629695920755693805404007434844970703125f,-0.54769999648928635593847502605058252811431884765625f,-0.10790000273883304515720737981610000133514404296875f,-0.047600006663799321326990821034996770322322845458984375f,-1.4534999843835834099081694148480892181396484375f,-0.01329997165203167241998016834259033203125f,-0.1611999860882757928948194603435695171356201171875f,-0.45040000542104230429885092235053889453411102294921875f,0.474800015021860755126681397086940705776214599609375f,0.062899999702721809402561348179006017744541168212890625f,-1.5733999953031538865388938575051724910736083984375f,0.53440001902580291925914934836328029632568359375f,0.04679999731183048794491696753539144992828369140625f,-1.513899990773200698868095059879124164581298828125f,0.016100059294700486134388484060764312744140625f,-0.19739999318718925991333890124224126338958740234375f,0.312899958705902037081614253111183643341064453125f,0.1476999477624900691807852126657962799072265625f,0.5014999965429307593467456172220408916473388671875f,-0.498700004863739110305687063373625278472900390625f,-0.729099974894523938928614370524883270263671875f,-0.26029998929500575144402318983338773250579833984375f,-0.260299993765354287234004004858434200286865234375f,-0.073699996817111923519405536353588104248046875f,-0.2211000034153460802599511225707828998565673828125f,-0.24399999728798871245771806570701301097869873046875f,0.238599999839067322682240046560764312744140625f,0.1011000025212764885651495205820538103580474853515625f,-1.261200000691413691811249009333550930023193359375f,0.444000023365020979326800443232059478759765625f,0.08270000036358826633886565105058252811431884765625f,-0.261100000882148464143028832040727138519287109375f,-0.006099949622154099415638484060764312744140625f,-0.43240000197887429322918251273222267627716064453125f,0.2739000050783158002332129399292171001434326171875f,0.53289999089241035790109890513122081756591796875f,0.1515999951601028072900589904747903347015380859375f,-0.3347999917387962653947397484444081783294677734375f,0.8065999932229512836556750698946416378021240234375f,0.57050001186132426322927813089336268603801727294921875f,0.44710002818107597022390109486877918243408203125f,0.098699995177984334304710500873625278472900390625f,-0.45100001403689393697504783631302416324615478515625f,-0.05440000486969953641391839482821524143218994140625f,0.792500000447034391726219837437383830547332763671875f,-0.43780000717639921958834747783839702606201171875f,-0.44720001308918000404446502216160297393798828125f,0.043499984681606207459481083787977695465087890625f,-0.134599995994567844803668776876293122768402099609375f,0.339199997353553772683198985760100185871124267578125f,0.48709999465346331959381132037378847599029541015625f,0.17620000769495991477242569089867174625396728515625f,-0.42119999249577499966790128382854163646697998046875f,-0.8130999785900119292136878357268869876861572265625f,0.01120001425147076901112086488865315914154052734375f,0.70460001611113565189725704840384423732757568359375f,-0.0919999857544890886629218584857881069183349609375f,0.06160000544190413762635216698981821537017822265625f,0.57680000029206279776872179354541003704071044921875f,-0.17199996593594590876818983815610408782958984375f,-0.28760000785589223681881776428781449794769287109375f,0.11760000427961347924110668827779591083526611328125f,-0.10139999434947899459302789182402193546295166015625f,-0.441800002154707882606743396536330692470073699951171875f,0.121599989572167388462986536978860385715961456298828125f,0.413899993269145394236829815781675279140472412109375f,0.017200005787611161167660611681640148162841796875f,0.442200017708540116245785611681640148162841796875f,0.911000014483928310937699279747903347015380859375f,0.66780001440346248298141063060029409825801849365234375f,-0.38379999781846996231848834213451482355594635009765625f,-0.73770001537650820022662401242996565997600555419921875f,-0.59900000712275502934289761469699442386627197265625f,0.38379999439120293658334048814140260219573974609375f,0.206100018018484210102769793593324720859527587890625f,-0.2783999993264674177595452420064248144626617431640625f,-0.1250000040233134601663778084912337362766265869140625f,-0.00640000612139690616686493740417063236236572265625f,0.83060004102587747798480677374755032360553741455078125f,0.10420001866221462183403900780831463634967803955078125f,-0.52830001523792713413740784744732081890106201171875f,0.44680001802444468950170630705542862415313720703125f,0.28219999997615818898566431016661226749420166015625f,-0.74979999769926042318957115639932453632354736328125f,-0.65899999743700021781478426419198513031005859375f,0.4254999981522560315028158584027551114559173583984375f,0.84330001814365385115479512023739516735076904296875f,-0.633300010693073289758103783242404460906982421875f,-0.282200007873773639488490516669116914272308349609375f,-0.09400000727176660575423738919198513031005859375f,-0.9597999789237974255229346454143524169921875f,-0.4393000001430511503031084430404007434844970703125f,0.54099999064207082710709073580801486968994140625f,0.087199987459182892735043424181640148162841796875f,0.1228999980449676510119161321199499070644378662109375f,-0.94269997279644002219356480054557323455810546875f,-0.11289995870590274762435001321136951446533203125f,-0.119999990165233771932662421022541821002960205078125f,-0.19910002615451816154745756648480892181396484375f,0.513300002646445818754727952182292938232421875f,0.4986000180184841923392013995908200740814208984375f,-0.266299995732307337448219186626374721527099609375f,0.488899984514713281669173738919198513031005859375f,0.4373000009775160901170920624281279742717742919921875f,-0.2818000198125840682905618450604379177093505859375f,-0.3469999967813492958157439716160297393798828125f,0.011600009763240937132877661497332155704498291015625f,0.3266999861404300542488954306463710963726043701171875f,-0.52630000031441426244782633148133754730224609375f,-0.104600000651180746746859995255363173782825469970703125f,1.15970003395080567543118377216160297393798828125f,0.420499987125396046394598670303821563720703125f,0.17639999956488594534675939939916133880615234375f,-0.6183999982088803637481078112614341080188751220703125f,0.64260001575350766955807557678781449794769287109375f,0.037399999259412286534143277094699442386627197265625f,-0.05190001664161680849929325631819665431976318359375f,-0.587300002467632253910778672434389591217041015625f,0.059100006484985367904982922482304275035858154296875f,-0.12350000986456866058915693429298698902130126953125f,-0.49909998219609263969687162898480892181396484375f,-0.1653999947667121883654317571199499070644378662109375f,-0.323200003507733146790315004182048141956329345703125f,1.2494000328093761709169484674930572509765625f,0.38840001325905315621156432825955562293529510498046875f,-0.8156000250816344276927338796667754650115966796875f,0.794000002503395307940081693232059478759765625f,0.430700005966424992465135801467113196849822998046875f,1.5325999670267105301491028512828052043914794921875f,-0.2854000090718269433409659541212022304534912109375f,0.455400002962350836099858497618697583675384521484375f,-0.020999997198581699109354303800500929355621337890625f,-0.7918999748438599883826327641145326197147369384765625f,-0.41260001709461224805863821529783308506011962890625f,0.23060000272989267244838629267178475856781005859375f,0.257300027054548241522979878936894237995147705078125f,0.16779999927878375132195287733338773250579833984375f,-0.47360000371336941071831461158581078052520751953125f,0.158699976998567304775633601821027696132659912109375f,-0.2370000079572200935018599921022541821002960205078125f,-0.5000000058114528744823701345012523233890533447265625f,-0.03050001089274889665148293715901672840118408203125f,-0.86760003020763409864457571529783308506011962890625f,-0.26339998815059662007342922152020037174224853515625f,0.7258000095009808916302063153125345706939697265625f,-0.71780002222657213195589065435342490673065185546875f,-0.08120001097321516425608933786861598491668701171875f,0.0023000298857684953190982923842966556549072265625f,-0.747399992889165698528586290194652974605560302734375f,0.162199992823600869229494492174126207828521728515625f,1.2938999522536998032506971867405809462070465087890625f,0.6665999993324278616313449674635194242000579833984375f,-0.0629999829530716315417748774052597582340240478515625f,-0.49909998517632470083071893895976245403289794921875f,0.1246000047862529935827069493825547397136688232421875f,0.1187999991595745097328062911401502788066864013671875f,-0.126099998944997793159927823580801486968994140625f,-0.1092000151604415325579111595288850367069244384765625f,0.2854000014722346545426034936099313199520111083984375f,-0.28139994916170885108641641636495478451251983642578125f,0.6644000275939705435490623131045140326023101806640625f,-0.3176000024914740205161933772615157067775726318359375f,-0.97380002579539992613177901148446835577487945556640625f,-0.2850000131130219216402110760100185871124267578125f,-0.3670000057220459410700641456060111522674560546875f,0.355500000536441262966036447323858737945556640625f,-0.546000004649162207215340458787977695465087890625f,-0.25840001437664028571816743351519107818603515625f,0.720599987828730803585131070576608180999755859375f,-0.354800006639957477005964392446912825107574462890625f,-0.170600014650821751871490050689317286014556884765625f,-0.0458000065207482265350336092524230480194091796875f,0.34740000674724580420615893672220408916473388671875f,-0.37279999361634263976839065435342490673065185546875f,-0.831599984133243541606361759477294981479644775390625f,0.5857999983251094278102755197323858737945556640625f,0.2317999990254639985920448452816344797611236572265625f,0.12060000898838030281012834166176617145538330078125f,-0.418800013911724278159454115666449069976806640625f,0.001000001370906833386698053800500929355621337890625f,0.34350001418590547785925082280300557613372802734375f,-0.079099995458125960112738539464771747589111328125f,-0.29820000894665721347109865746460855007171630859375f,-0.09199999514222145791109142010100185871124267578125f,-0.7089999813437461995135890902020037174224853515625f,-0.40650000056624413513617355420137755572795867919921875f,0.58560001554489105046741315163671970367431640625f,-1.40079998953342510503716766834259033203125f,-0.53649999132752412833724520169198513031005859375f,-0.3218999910116195906084612943232059478759765625f,-1.11249999701976776123046875f,-0.2641999994397163931125760427676141262054443359375f,1.342199962723255257657228867174126207828521728515625f,-0.8703999877631662940302703646011650562286376953125f,-0.2921999949842690735835049054003320634365081787109375f,-0.34139997691512125044255299144424498081207275390625f,0.89119998862147298268610029481351375579833984375f,0.2221999987840652668324992191628552973270416259765625f,-0.12250000521540638231954289949499070644378662109375f,0.25729999665617953041873988695442676544189453125f,0.1355999998986721077898209841805510222911834716796875f,-0.4561999758064747556574047848698683083057403564453125f,0.01979998033940777446559877716936171054840087890625f,0.20409999810308210310694221334415487945079803466796875f,-0.9397000206887720086257331786327995359897613525390625f,0.66470003179013747995895755593664944171905517578125f,0.11570000548213721369705808683647774159908294677734375f,0.3800000250339508056640625f,-0.65099999749660497627701261080801486968994140625f,-0.385000007599592297680146657512523233890533447265625f,0.72800001335144060021775658242404460906982421875f,-0.92519998948574055930293980054557323455810546875f,-0.439000007271766801153489723219536244869232177734375f,-0.2058000142693519052272677072323858737945556640625f,-0.64739999393224678669866989366710186004638671875f,-0.2131000016868114244061871431767940521240234375f,-0.101100000882148766123691530083306133747100830078125f,-0.5826999941051003162328925100155174732208251953125f,-0.2993999822944399813451354930293746292591094970703125f,0.2641999888598920431803662722813896834850311279296875f,-0.03970001126378763789404047201969660818576812744140625f,-0.586800030988455123548419578582979738712310791015625f,0.279600003147125075741996624856255948543548583984375f,1.0170000300854440400399880672921426594257354736328125f,0.3538000034809112293032740126363933086395263671875f,-0.7962000024795532482357884873636066913604736328125f,-0.441100037837028224885216332040727138519287109375f,0.297999996811151302011921870871447026729583740234375f,0.263199996429681615239815073437057435512542724609375f,0.0366000082731243736589021864347159862518310546875f,0.1140000030249357365619289339520037174224853515625f,0.198000002548098574806090255151502788066864013671875f,-0.074599990741908595026643524761311709880828857421875f,-0.63240000048875799620873294770717620849609375f,0.6770000173449517433255095966160297393798828125f,1.4849999853968629537348533631302416324615478515625f,0.34799999517202362397938486537896096706390380859375f,-0.56540000549554836339893881813623011112213134765625f,-0.7944000079989432805405158433131873607635498046875f,0.27800001126527806860622149542905390262603759765625f,0.8078000247597696414914025808684527873992919921875f,-0.26259994802772990141193076851777732372283935546875f,0.66019999917149529977677957504056394100189208984375f,0.26280002043843264658562475233338773250579833984375f,-0.0054999778866768878060611314140260219573974609375f,0.415999984830617730580115676275454461574554443359375f,-0.484799983245134402665144079946912825107574462890625f,-0.97719997076988196482716375612653791904449462890625f,0.04439999399185179385085575631819665431976318359375f,-0.20560000600814820614914424368180334568023681640625f,-0.78899996972084096569233224727213382720947265625f,-0.0243999989837409003712309640832245349884033203125f,0.241599995532631783845545214717276394367218017578125f,0.474599983589351293034042100771330296993255615234375f,0.490000002235174036968601285479962825775146484375f,0.54520000542998303671993198804557323455810546875f,-0.644999969899654246319187222979962825775146484375f,-0.05980000142455121903140025096945464611053466796875f,0.33599998989701251872475040727294981479644775390625f,0.7344999717473985612059550476260483264923095703125f,-0.1783999884486198173050297555164434015750885009765625f,-0.1023999919950961956516266582184471189975738525390625f,-0.2230000355541703616069071358651854097843170166015625f,-0.25399997873604274190029173041693866252899169921875f,0.09960000921934841766613999425317160785198211669921875f,1.037400032079219602820785439689643681049346923828125f,-0.5742000389277936989174122572876513004302978515625f,-0.1254000095933676117709865138749592006206512451171875f,-0.3244000035285949934404925443232059478759765625f,-0.088599975252151352833607234060764312744140625f,-0.3786000022232531847521386225707828998565673828125f,0.1563000055670737964419458876363933086395263671875f,0.62949996548891107295276015065610408782958984375f,0.26129998828172684710580142564140260219573974609375f,-0.06869999875426291424673763685859739780426025390625f,0.30289999335110184830455182236619293689727783203125f,0.179300000441074358636939223288209177553653717041015625f,1.587799979758262480800112825818359851837158203125f,-0.3162000108242040141703910194337368011474609375f,0.065900011545419800995659898035228252410888671875f,-0.82099999928474431953873136080801486968994140625f,0.03780000836849239931325428187847137451171875f,-0.17150000414252275504622957669198513031005859375f,-0.3122999833941459968400522484444081783294677734375f,0.9644999918639651781404609209857881069183349609375f,0.67120001365542403259922821234795264899730682373046875f,0.26950003373622877234083716757595539093017578125f,1.1576999692201610514530329965054988861083984375f,0.37219999431371686426928135915659368038177490234375f,-1.04029996873140362367848865687847137451171875f,0.17970000802278462970207328908145427703857421875f,-0.070299995851516694500560333835892379283905029296875f,-0.997599983716010996204204275272786617279052734375f,-0.14999999105930328369140625f,-0.01420000033378610027057220577262341976165771484375f,0.58729998473525046875920452293939888477325439453125f,0.17809997970759905427939884248189628124237060546875f,0.089899994230270452444386819479404948651790618896484375f,-0.68790000698566444725656765513122081756591796875f,1.05489996979236622109965537674725055694580078125f,0.608099999749660558023833800689317286014556884765625f,0.9030999732255935441571637056767940521240234375f,0.21609996452331614591457764618098735809326171875f,0.106799988818168767323868451057933270931243896484375f,0.62319997258782411364563813549466431140899658203125f,0.139600011491776143657261854968965053558349609375f,-0.14960000538229945021839739638380706310272216796875f,0.86640001655220988396877146442420780658721923828125f,0.2160000451803210097523333388380706310272216796875f,-0.3668000022292137174417803180404007434844970703125f,1.0587999942421912891177271376363933086395263671875f,0.471000046968459429308495600707828998565673828125f,0.59500002741813684536964501603506505489349365234375f,0.07799999904632581859686979441903531551361083984375f,-0.8617000071883200718048101407475769519805908203125f,-0.203600008332729398574656443088315427303314208984375f,-0.101000003755092659929459841805510222911834716796875f,-0.006199993985891438796898000873625278472900390625f,-0.319800012004375577845394218456931412220001220703125f,0.740600012266635854274454686674289405345916748046875f,0.4885000329613689018515287898480892181396484375f,-0.64399999237060523427089719916693866252899169921875f,0.51780000240802781519278141786344349384307861328125f,1.09119998817443875083199600339867174625396728515625f,-0.53140002117156992511581847793422639369964599609375f,-0.19099999570846559748815707280300557613372802734375f,-0.2076999793529505922151656704954802989959716796875f,0.089600007990002783486715998151339590549468994140625f,0.337200019124150429661312955431640148162841796875f,0.264800008316338431058056812616996467113494873046875f,0.053000006645917974168469299911521375179290771484375f,-0.174999999403953498955388567992486059665679931640625f,-0.0011000189870593590768521607969887554645538330078125f,-0.7115999976933000681356134009547531604766845703125f,-0.1964000195324420161568923504091799259185791015625f,0.8293999713420863173496400122530758380889892578125f,-0.3754000059425830482240371566149406135082244873046875f,0.4690000083148480047867678877082653343677520751953125f,0.5019000242412090528887347318232059478759765625f,0.27139999569058392836495841038413345813751220703125f,0.07719998343586897959767156862653791904449462890625f,0.409699994015692681159634958021342754364013671875f,-0.3630000167787077458569910959340631961822509765625f,-0.3364000198304655242509397794492542743682861328125f,-0.4787000075459477699268973083235323429107666015625f,0.300200004237890283320666640065610408782958984375f,-0.644200004059076292151075904257595539093017578125f,-0.3859000191450121519665117375552654266357421875f,-0.599199998545646650427443091757595539093017578125f,0.627200014132261340904506141669116914272308349609375f,0.11390000329017624380867346189916133880615234375f,0.41949997293949081722530536353588104248046875f,0.518300008606910722619431908242404460906982421875f,0.3270000009536744300930877216160297393798828125f,0.2601999602794649035786278545856475830078125f,0.55019999693632115622676792554557323455810546875f,-0.8704000195026395658715046010911464691162109375f,0.8558000181436540287904790602624416351318359375f,0.35010000606775282250282543827779591083526611328125f,-0.0911999968171119945736791123636066913604736328125f,-0.371999999165534944722821819595992565155029296875f,-0.1273000008285045669964574699406512081623077392578125f,0.4093999990582464221233749412931501865386962890625f,0.1027999693274495029982062987983226776123046875f,0.343600005203485547866648630588315427303314208984375f,1.0807999805927277492401117342524230480194091796875f,-0.48119999532699608835173421539366245269775390625f,0.01030000881552695091158966533839702606201171875f,-0.4996000001668932100074016489088535308837890625f,-0.935399986720085507840849459171295166015625f,-0.2549999897181987051908436114899814128875732421875f,0.35200000616908067030408346909098327159881591796875f,0.39600000204145846538494879496283829212188720703125f,0.218600009524822225248641416328609921038150787353515625f,0.043799991261959103638901069643907248973846435546875f,-0.0475999862492084702125794137828052043914794921875f,-0.1041999972790479400686081135063432157039642333984375f,0.787299990248680359172794851474463939666748046875f,-1.11469998030662509336252696812152862548828125f,-0.46030000330209741576936721685342490673065185546875f,0.43310000705122941866420660517178475856781005859375f,-0.1723000103652481129756779409945011138916015625f,0.1397000053405761388347627871553413569927215576171875f,0.658700016188621617629905813373625278472900390625f,0.48929995722770769361886777915060520172119140625f,0.072599975073337486008995256270281970500946044921875f,-0.762100029373169007840260746888816356658935546875f,-0.815499997854232105964911170303821563720703125f,-0.26660000201463684987857050145976245403289794921875f,-0.2942999876260756764168036170303821563720703125f,-0.053699978637695267025264911353588104248046875f,-0.2981000043690205103530388441868126392364501953125f,0.624399959421158001759977196343243122100830078125f,-0.8420999678500000751313336877501569688320159912109375f,0.064200001674890483371882510255090892314910888671875f,-0.700400025314092733452753236633725464344024658203125f,-0.3723000298857694900789283565245568752288818359375f,0.501400012081861579105179771431721746921539306640625f,-0.327400002127885780822680317214690148830413818359375f,-0.429300011169910789021741948090493679046630859375f,0.1567999958217143330330145545303821563720703125f,-0.305199999469518701289416640065610408782958984375f,0.006799996119737361510715345502831041812896728515625f,-0.7634000151216984164648238220252096652984619140625f,0.5904000050485134210020987666212022304534912109375f,1.57550001782178838993786484934389591217041015625f,0.25700000736117367683419843160663731396198272705078125f,-0.15739999974370000312973161271656863391399383544921875f,-0.24390001051723952940619710716418921947479248046875f,-0.7124000168800355137221913537359796464443206787109375f,0.0169999878406523752261136905872263014316558837890625f,0.5042000016748904300811773282475769519805908203125f,0.88140001729726780155260712490417063236236572265625f,0.25979999173879608775905580841936171054840087890625f,-0.7269000088930130942799223703332245349884033203125f,0.492600017616152907606874578050337731838226318359375f,-0.165600000941753311423099148669280111789703369140625f,-0.66370001168102021882333474422921426594257354736328125f,-0.311000001072883502928334564785473048686981201171875f,-0.267600005173683097581260881270281970500946044921875f,-0.11150000548362726249251863919198513031005859375f,0.2297999878644940707772548194043338298797607421875f,-0.8220000075101854264403300476260483264923095703125f,-0.2059000289797783267431441345252096652984619140625f,-0.28919998736977559072869325973442755639553070068359375f,0.54319999374747285347808656297274865210056304931640625f,0.59139999397695053318102509365417063236236572265625f,-0.03839999962449081039039811003021895885467529296875f,0.91320000924468036629377820645458996295928955078125f,0.1708000247001653093548156903125345706939697265625f,0.4543999971210954669231796287931501865386962890625f,0.2170000148117541272085873060859739780426025390625f,0.0288000037789348795058685936965048313140869140625f,0.22760000331103802384546952453092671930789947509765625f,-0.07599999958276748568408720529987476766109466552734375f,-0.31499999765306718746415981513564474880695343017578125f,0.0120000031888485292341783861047588288784027099609375f,1.1350000043213370748418356015463359653949737548828125f,0.1435000101625920621017939993180334568023681640625f,0.179799993824958814769843229441903531551361083984375f,0.0786999968916177738975648026098497211933135986328125f,-0.3580999984458088647443219088017940521240234375f,-0.52819997430145770866971588475280441343784332275390625f,-0.5180999861523509775196316695655696094036102294921875f,0.2645000101923944413329081726260483264923095703125f,-0.81639997482895854119533396442420780658721923828125f,0.0575000290945173464507433891412802040576934814453125f,-0.4352000169038774401997216045856475830078125f,0.3151999918699264213728383765555918216705322265625f,-0.24320001662075529935691520222462713718414306640625f,0.0869000118732454751580007723532617092132568359375f,0.61759998863339404806538368575274944305419921875f,0.2098999943420289948647905475809238851070404052734375f,-0.5918000248417261133226929814554750919342041015625f,-0.18659999136030680499942491223919205367565155029296875f,-0.3579999991953373150721517959027551114559173583984375f,0.8580000362992290519059679354541003704071044921875f,-0.15279999585151671226412872783839702606201171875f,0.5876000243216754626018882845528423786163330078125f,0.6903999942451715110536270003649406135082244873046875f,-0.18359999045133601480728202659520320594310760498046875f,0.6428000043451784772940982293221168220043182373046875f,-0.244700004521012370872767860419116914272308349609375f,0.199799995762109716679333359934389591217041015625f,-0.12750000447034837147697317050187848508358001708984375f,0.1710999916434285950117555330507457256317138671875f,0.0069999988675117919001422706060111522674560546875f,0.2142999984294176041288437772891484200954437255859375f,-0.111300005418062397666290053166449069976806640625f,-0.426599980854988292122698112507350742816925048828125f,-0.266499993488192643553702509962022304534912109375f,-0.33430000044107455181574550806544721126556396484375f,0.637399984097480665212742678704671561717987060546875f,-0.36510002685487286111509774855221621692180633544921875f,0.663600022786855703316177823580801486968994140625f,-0.151800009828805482214875155477784574031829833984375f,0.54780002159327312316605684827663935720920562744140625f,0.2583000306606297868938781903125345706939697265625f,0.5164000016510481572140633943490684032440185546875f,0.1693000033840534879203687523840926587581634521484375f,0.5060999999880788635664430330507457256317138671875f,-0.449299993661046226378630308317951858043670654296875f,-0.2057999957919121047922317302436567842960357666015625f,-0.39549999099969868066040135090588591992855072021484375f,-0.419299999397993061744926990286330692470073699951171875f,-0.0406000063061713323264712016680277884006500244140625f,0.04329998513758204214951774702058173716068267822265625f,-0.283300013226270575028564735475811176002025604248046875f,-0.0235999795734880279951539705507457256317138671875f,-0.73899998447298997916732332669198513031005859375f,-0.5468000109463933000597535283304750919342041015625f,-0.0129999999403953570009662144002504646778106689453125f,-0.16290000169575213906369981486932374536991119384765625f,0.172500008791685199849297305263462476432323455810546875f,-0.85719998805522923390753931016661226749420166015625f,-1.2985999627351763052729438641108572483062744140625f,-0.78819999032020593432434907299466431140899658203125f,0.32159999039173126078594577847979962825775146484375f,0.372599984908104087111269109300337731838226318359375f,0.4471999914824962285564424746553413569927215576171875f,-0.233400007969140954600106852012686431407928466796875f,-0.40980001081228234482978223240934312343597412109375f,-0.210300005835294623324216445325873792171478271484375f,-0.01799999785423278542140224089962430298328399658203125f,-0.1731999971002340377168593477108515799045562744140625f,-0.02219999848604203851465399566222913563251495361328125f,-0.0411999997973442333432103623636066913604736328125f,0.45109999924302091045547058456577360630035400390625f,-0.152999997183680402912386853131465613842010498046875f,0.2789999935626983340597462301957421004772186279296875f,0.72529998877346513186381571358651854097843170166015625f,0.5285000033080577974686775633017532527446746826171875f,-0.352000003337860079000165569595992565155029296875f,-0.4350000005960463766996326739899814128875732421875f,-0.17599999704957003388017255929298698902130126953125f,-0.020199995148181937310027933563105762004852294921875f,-0.0550999895274639772679847737890668213367462158203125f,-0.35350000882148757685996542932116426527500152587890625f,0.1505999887228011147044526296667754650115966796875f,0.653999988645315166735372258699499070644378662109375f,0.2546999979645014899887200954253785312175750732421875f,-0.30499999135732658306352504951064474880695343017578125f,-0.8426999777883292797042713573318906128406524658203125f,-0.20389998764395722563591561993234790861606597900390625f,-0.23339998874664313888160904753021895885467529296875f,0.6247999943837518888045678977505303919315338134765625f,0.4557000067114829189307556589483283460140228271484375f,0.34159998741149910639336439999169670045375823974609375f,0.44029999529272323410822309597278945147991180419921875f,0.595700014981627390397989074699580669403076171875f,0.2863999949455260374264753409079276025295257568359375f,0.4771000022903084669678719365037977695465087890625f,-0.18139998868703788303946566884405910968780517578125f,0.883599996709822921303612019983120262622833251953125f,-0.05500001884996941203809228682075627148151397705078125f,-0.49930002093017122888340963982045650482177734375f,-0.1903999980449675444305057681049220263957977294921875f,-0.3783000064089894198815500203636474907398223876953125f,0.87260000934600867594781448133289813995361328125f,0.0968000028252600230871394160203635692596435546875f,0.973000013276934527794992391136474907398223876953125f,-0.74379999126195883718537515960633754730224609375f,-0.19479999203681952479882966144941747188568115234375f,-0.6845999861598013325192368938587605953216552734375f,0.14640000478029246977484945091418921947479248046875f,0.037800000768899888470286896335892379283905029296875f,0.19860000993460424911773998246644623577594757080078125f,-0.191200002554059000914321586606092751026153564453125f,0.053199999111890772240940350457094609737396240234375f,-0.13000000208616258046134817050187848508358001708984375f,0.9972999974012370927312076673842966556549072265625f,-0.00340000722408273503560849349014461040496826171875f,0.7504999904781579633805677076452411711215972900390625f,-0.26990001159012333431519436999224126338958740234375f,-0.4691999910950659380404204057413153350353240966796875f,-0.34089999846965068908133389413706026971340179443359375f,0.26699999409914010328748190659098327159881591796875f,-0.177999993681907664466734786401502788066864013671875f,0.31500000774860381635988915149937383830547332763671875f,0.52490002716183692399454230326227843761444091796875f,0.43639999092221248844225556240417063236236572265625f,0.42910000324398278603865719560417346656322479248046875f,0.29700000751018507116896216757595539093017578125f,-0.29340000081658335062684272998012602329254150390625f,0.139200010876357804789904548670165240764617919921875f,-0.383500008299946681944447846035473048686981201171875f,0.05159999635219580937928185448981821537017822265625f,-0.1641000007107853431165267465985380113124847412109375f,0.023099986040591602431959472596645355224609375f,-0.395300011870265033397942033843719400465488433837890625f,-0.32159997638463977409628569148480892181396484375f,-1.047299959701299965075804720981977880001068115234375f,-0.5511999888449909956733563376474194228649139404296875f,-0.294999994039535362588821953977458178997039794921875f,-0.07389999747872355584377146442420780658721923828125f,0.067900010263919785558073272113688290119171142578125f,0.3432000079035759654288995079696178436279296875f,0.5142000033140181614044195157475769519805908203125f,0.30879999722242335025157444761134684085845947265625f,-0.25520001288056359811662332504056394100189208984375f,-0.404700019049644144075728036114014685153961181640625f,0.2045999955475330533971600743825547397136688232421875f,-0.060200017052888910029651015065610408782958984375f,0.493999983280897225768057978712022304534912109375f,0.42990000063776978578289345023222267627716064453125f,0.217399992293119570518911132239736616611480712890625f,0.2532999782085421092148180832737125456333160400390625f,0.53829999616444113019753103799303062260150909423828125f,0.25119999145269400475655174886924214661121368408203125f,0.5657999787300826977087808700161986052989959716796875f,0.13319998868107807421523602897650562226772308349609375f,-0.08199999722838402504976329510100185871124267578125f,-0.566499992072582170266059620189480483531951904296875f,-0.4704000129461289869681195341399870812892913818359375f,0.137200001913309055368017652654089033603668212890625f,0.46870000285208213153254064309294335544109344482421875f,0.035199999916553481593695096307783387601375579833984375f,-0.20200000989437094656153703908785246312618255615234375f,0.2817999844223262329023782513104379177093505859375f,-0.00860000401139249248672058456577360630035400390625f,-0.2070000130236149260554157081060111522674560546875f,0.12289999163746845312061850563623011112213134765625f,-0.247900012722611418070073341368697583675384521484375f,0.43279999585151696095408624387346208095550537109375f,0.3670999717056755429211989394389092922210693359375f,-0.11920003065764905869627909851260483264923095703125f,0.418199999856948778642617980949580669403076171875f,0.62149999505281439127202247618697583675384521484375f,0.394900004661083325885329031734727323055267333984375f,-0.0683999917268751922438951851290767081081867218017578125f,-0.657799998626857895800412734388373792171478271484375f,-0.1671999963626265017779815025278367102146148681640625f,0.274200006443261923294585358235053718090057373046875f,0.4288000011339783146269155622576363384723663330078125f,-0.0991000008225437767350740614347159862518310546875f,0.530599989616870715281038428656756877899169921875f,0.0896999852240083495047429096302948892116546630859375f,0.311299968165159146593623518128879368305206298828125f,-0.5111999743163588849625966759049333631992340087890625f,0.299899993000924069974644226022064685821533203125f,-0.347000000953674003767446265555918216705322265625f,-0.0283999893426898353254728135652840137481689453125f,-0.387000007361173903319695455138571560382843017578125f,-0.4769000091910360339397811912931501865386962890625f,-0.5253999736070635862006383831612765789031982421875f,-0.2972999774336815281827739454456605017185211181640625f,0.154199996906518865813495722250081598758697509765625f,-0.7877999810993669971281860853196121752262115478515625f,0.074000024743378300939866676344536244869232177734375f,-0.88190001679062834938349624280817806720733642578125f,-0.43279997811913517580251209437847137451171875f,-0.76609999533146633243774203947396017611026763916015625f,0.179800007012486329216471858671866357326507568359375f,0.04039999968409535568980572861619293689727783203125f,0.041400000086426747003542914171703159809112548828125f,0.05890003279447597606122144497931003570556640625f,0.64579998685121520196616984321735799312591552734375f,0.12530000799596319183137893560342490673065185546875f,0.39810000503957265749477301142178475856781005859375f,0.0217999961197377079002990285516716539859771728515625f,0.24369999577403067547720638685859739780426025390625f,0.800199997457861744720730712288059294223785400390625f,-0.3799999858438969368990001385100185871124267578125f,0.63580001039504996374063239272800274193286895751953125f,0.95770003180503859852024106658063828945159912109375f,-0.034799999934434833193108715931884944438934326171875f,0.78110002632588171334049320648773573338985443115234375f,-0.03749997198581667134931194595992565155029296875f,0.6969999970793718802042349125258624553680419921875f,-0.135100004577636934044448935310356318950653076171875f,-0.46050001037120846802963569643907248973846435546875f,-0.056800008040666316588840345502831041812896728515625f,-0.50110001291483630136980309544014744460582733154296875f,-0.59220001940354716207792762361350469291210174560546875f,0.30759999634474499696779048463213257491588592529296875f,-0.3866000187598170878544578954461030662059783935546875f,-0.40330003319382701221229581278748810291290283203125f,0.350899999475478896471258849487639963626861572265625f,0.0703999980449676598937003291212022304534912109375f,0.25089999731481071876260102726519107818603515625f,0.411000006213784185860049547045491635799407958984375f,-0.71200000005960450977227083058096468448638916015625f,-1.22229999963641144944404004490934312343597412109375f,-0.682399998924136053091160647454671561717987060546875f,-0.38359999447464954158704131259582936763763427734375f,-0.78169997798204438055336140678264200687408447265625f,-0.342399990504980156202918806229718029499053955078125f,-0.6401999873995780632185415015555918216705322265625f,-1.1957999789536000179168695467524230480194091796875f,-0.69589998554289334009581580176018178462982177734375f,0.27699999380111695046480235760100185871124267578125f,1.05529999108314509470574193983338773250579833984375f,0.432699993062019228062808906543068587779998779296875f,0.54679998583793631983240857152850367128849029541015625f,0.62299997468292733771022540167905390262603759765625f,0.244199985283613418829418151290155947208404541015625f,-0.18979999769926081398807582445442676544189453125f,-0.245899988895654786347222398035228252410888671875f,-0.44349999898672098908036787179298698902130126953125f,0.24419999139308945501625203178264200687408447265625f,0.079099976384640058313379995524883270263671875f,0.5130999912559985887128277681767940521240234375f,-0.5554000048995018090636222041212022304534912109375f,-0.6717999907553195981790850055404007434844970703125f,0.01690000732839092023596094804815948009490966796875f,-0.2179999894350768130379947251640260219573974609375f,-0.30499998081475521161110009416006505489349365234375f,-0.3851999854996799310669075566693209111690521240234375f,0.27699998456239693922498190659098327159881591796875f,-0.075500021398067218569849501363933086395263671875f,0.4250999948918821047527671908028423786163330078125f,0.2710000135898591455685391338192857801914215087890625f,-0.18209999867677684193267850787378847599029541015625f,-0.035900000667572040669028865522705018520355224609375f,-0.211800000739097438184899147017858922481536865234375f,0.129699994984269206810267860419116914272308349609375f,0.1077000000655650158165599350468255579471588134765625f,-0.44839999426007270955096828402020037174224853515625f,-0.22419998438954369390785359428264200687408447265625f,-0.415199993732571659421637377818115055561065673828125f,-0.0123999966144561490910547263410990126430988311767578125f,-0.570800027531385811840891619795002043247222900390625f,-0.4149000157624482820750699829659424722194671630859375f,-0.49259998431205753721684459378593601286411285400390625f,-0.37129999610483654937098663140204735100269317626953125f,0.5542999912023542918859675410203635692596435546875f,0.15900000220537169326462390017695724964141845703125f,0.6887000020325182436664590568398125469684600830078125f,0.0733000026464463161346429842524230480194091796875f,0.09059999259710327379480077070184051990509033203125f,0.2053000054627658332861983581096865236759185791015625f,0.486399979746341415420829434879124164581298828125f,0.2769999909698963591608844581060111522674560546875f,0.620999983973800961933875441900454461574554443359375f,0.382900010859966499765505432151257991790771484375f,-0.4495999827325345421513702603988349437713623046875f,0.14270001299232237812475432292558252811431884765625f,-0.446599978545308307076311393757350742816925048828125f,0.329599979156256051027185094426386058330535888671875f,-0.5085999976038928505062131080194376409053802490234375f,0.690100013965368130897104492760263383388519287109375f,0.3515999813020231901106171790161170065402984619140625f,0.607899998380243999918093322776257991790771484375f,-1.150500009477138263491724501363933086395263671875f,-0.30759997820258178080621291883289813995361328125f,-0.96429999109059583162206763518042862415313720703125f,-0.0328000044941898494244014727883040904998779296875f,-0.66059997426867500536218358320184051990509033203125f,0.1322000296667218410817667972878552973270416259765625f,0.55819998745173171617039997727260924875736236572265625f,0.252399980521202438321637373519479297101497650146484375f,0.51339997976869344853412258089520037174224853515625f,-1.29909999784231189323691069148480892181396484375f,-0.0992000043570993028652083012275397777557373046875f,-1.069699986602365537891046187723986804485321044921875f,-0.703300015163421665675969052244909107685089111328125f,0.5111999930918214030128865488222800195217132568359375f,-0.5799000298067931158385590606485493481159210205078125f,0.4020999872773887151566896136500872671604156494140625f,-0.49719997211098709843923870721482671797275543212890625f,0.33670000677853784143422899433062411844730377197265625f,1.1429999996423720887150921043939888477325439453125f,-0.08279999644756319554517176584340631961822509765625f,1.0046000011354683234543472281075082719326019287109375f,0.87480000062361373824870725002256222069263458251953125f,-0.16139999986290920475084931240417063236236572265625f,0.249600022667646470608815434388816356658935546875f,0.09330002022981620513064626720733940601348876953125f,-0.206999997526407408798831966123543679714202880859375f,0.129599980646371903958424809388816356658935546875f,0.8082999683737757123935807612724602222442626953125f,0.54139999479651468305974049144424498081207275390625f,0.0105999851465223837720941446605138480663299560546875f,0.82729999248385421850571219692938029766082763671875f,0.265899997389316578022544490522705018520355224609375f,0.000599990659952087668216336169280111789703369140625f,0.258299995344877242331449451739899814128875732421875f,0.2406999993354081990304393912083469331264495849609375f,0.00340001005530343736182885550078935921192169189453125f,0.0708000204905865349047644485835917294025421142578125f,-0.38989999661445617906707639122032560408115386962890625f,-0.1077999958515167833184023038484156131744384765625f,-0.4087999897718430641901932176551781594753265380859375f,0.09830001344978811506081228799303062260150909423828125f,0.49939998534917828809653883581631816923618316650390625f,0.733199969179183508316555162309668958187103271484375f,0.2692999843850734098538168836967088282108306884765625f,-0.6473999955713747311847328091971576213836669921875f,-0.8507999951958655771022677072323858737945556640625f,-0.16569998450875278450666883145458996295928955078125f,0.028999997884035122464041478451690636575222015380859375f,0.12459999184086922863201607469818554818630218505859375f,-0.185700002986192547194121971187996678054332733154296875f,0.2244000018894671821811925838119350373744964599609375f,-0.27459999986886973744049100787378847599029541015625f,-0.052800002333521778297154014580883085727691650390625f,0.4218000281572340526281550410203635692596435546875f,0.485800036472081586680360487662255764007568359375f,-0.2064999891668559683921557734720408916473388671875f,0.273199990618229027194985292226192541420459747314453125f,0.6511999810777606700895603353274054825305938720703125f,0.497999986678361761249789196881465613842010498046875f,-0.215200018244981805537463515065610408782958984375f,0.217799987283349150146705142105929553508758544921875f,0.3119999970048665804966958603472448885440826416015625f,-0.62399999818205831303430386469699442386627197265625f,-0.71469998894929887001126189716160297393798828125f,-0.6145999968886375341980965458787977695465087890625f,0.2928000013649463273424089493346400558948516845703125f,0.1867000186622143065307000142638571560382843017578125f,0.024299995374679639326132019050419330596923828125f,0.61379998857974982229279703460633754730224609375f,-0.027300033611059593141590084997005760669708251953125f,0.1028999914884565214379108510911464691162109375f,-0.736999978899955721090009319595992565155029296875f,-0.0310999834477900805040917475707828998565673828125f,0.79090001780390739583026515902020037174224853515625f,-0.57199999216198893492446586606092751026153564453125f,0.533900026498735247315607921336777508258819580078125f,-0.00289999268054952352713371510617434978485107421875f,-0.383599992388486743521980315563268959522247314453125f,-0.03269998125284889756159145690617151558399200439453125f,0.7298000147238372647962023620493710041046142578125f,0.48059997642934348949239620196749456226825714111328125f,0.574799987715482796346577742951922118663787841796875f,-0.5770000195801259224026580341160297393798828125f,-0.165400000280141679098733220598660409450531005859375f,-0.3980000048950314006646067355177365243434906005859375f,-0.23659998564198636561428656932548619806766510009765625f,0.19259998807460088077192494893097318708896636962890625f,-0.346799999230354760726413587690331041812896728515625f,0.1597999595522878735209815204143524169921875f,-0.5133999757826330778698320500552654266357421875f,0.32039999659210405269504917669110000133514404296875f,0.4344000009767707837227135314606130123138427734375f,0.242999987833202035769630811046226881444454193115234375f,0.32859999054446820121455630214768461883068084716796875f,-0.7305999942362308985366325941868126392364501953125f,0.489999980032444160116256171022541821002960205078125f,-0.58420000919997672195194127198192290961742401123046875f,0.3012000015854834344963819603435695171356201171875f,0.75579996911883373655882678576745092868804931640625f,0.1233999577149746951221231938689015805721282958984375f,-0.32119999076351479772029051673598587512969970703125f,-0.416799974326789779421886805721442215144634246826171875f,-0.3723999813966454297542441054247319698333740234375f,0.2102000086337330042596249768394045531749725341796875f,-0.78919997649192807909201974325696937739849090576171875f,0.1734000373989346144298906438052654266357421875f,1.04670000301599497305460317875258624553680419921875f,0.053000000238418554232566748396493494510650634765625f,1.010300006431341213186669847345910966396331787109375f,0.28760001888275166237463054130785167217254638671875f,-0.1536999973386526663343687459928332827985286712646484375f,-0.1378000117957594028439416433684527873992919921875f,0.16449999693036065906426301808096468448638916015625f,0.442700021299720081202622168348170816898345947265625f,-0.03240001389980307067162357270717620849609375f,0.69799998146295560985663541941903531551361083984375f,0.428999992817640229958442432689480483531951904296875f,-0.02000001326203371121437157853506505489349365234375f,0.395799999442696393003870980464853346347808837890625f,0.5240000158399344964976762639707885682582855224609375f,0.472399989461898872633582868729718029499053955078125f,1.137799976480007302370722754858434200286865234375f,0.5701999871015548393415883765555918216705322265625f,0.41180000297427188371557349455542862415313720703125f,0.5487999924540520879645555396564304828643798828125f,-0.14570002384781854942730205948464572429656982421875f,-0.1627999939143658369999911883496679365634918212890625f,-0.368799983103573481013626178537379018962383270263671875f,-0.0843999921292067634137623599599464796483516693115234375f,0.436999980241060459462687504128552973270416259765625f,1.0889999549686912860835263927583582699298858642578125f,0.71089998189210901191614766503334976732730865478515625f,-0.4117999853909015683939287555404007434844970703125f,-0.32829997627139118065997536177746951580047607421875f,-0.519099986889958397995314953732304275035858154296875f,0.0326000080049038132301575387828052043914794921875f,-0.51849999451637263092607099679298698902130126953125f,-0.441700004878640140049128604005090892314910888671875f,0.42739999542236351004476091475225985050201416015625f,0.6809999723136428428915678523480892181396484375f,-0.000500027433037786295244586654007434844970703125f,0.107600008749961784104698381270281970500946044921875f,-0.2011999848961831816041012643836438655853271484375f,0.11180000804066647646095589152537286281585693359375f,0.0591999899029733711586231947876513004302978515625f,0.53169997977018379486935373279266059398651123046875f,0.480399987688660434770326901343651115894317626953125f,0.691199986684322364993704468361102044582366943359375f,1.1205999723315240856891250587068498134613037109375f,0.59239998171329499854209643672220408916473388671875f,-0.6477999997258183650927776398020796477794647216796875f,-1.0338000109314915420100078335963189601898193359375f,-0.40889999487102013375761089264415204524993896484375f,-0.2438000079512594453490237356163561344146728515625f,0.111200001212954535656507459862041287124156951904296875f,-0.22030002863407194269029787392355501651763916015625f,-0.822399998253583230933827508124522864818572998046875f,-0.17569999177306849791335707777761854231357574462890625f,0.41120000247955346139860921539366245269775390625f,-0.8059999668002131301136614638380706310272216796875f,0.251600026527047049285101820714771747589111328125f,-0.2174000114411118733670491565135307610034942626953125f,-0.81719999699592527075964198957080952823162078857421875f,-0.217399981340765435788853210397064685821533203125f,-0.067799991381168212001284700818359851837158203125f,-0.35359998389482516500947895110584795475006103515625f,-0.2553999903708696894710783453774638473987579345703125f,0.3154000358939175185923886601813137531280517578125f,0.64419999422430951341311811120249330997467041015625f,0.1287999972596762798815461792401038110256195068359375f,-0.122800001774728162473593329195864498615264892578125f,-0.02800000046193595881760529664461500942707061767578125f,-0.03159999817758796236688567660166881978511810302734375f,-0.3044000021874904149399299058131873607635498046875f,-0.571799971681833607561884491587989032268524169921875f,-0.4337999859347940745379901272826828062534332275390625f,1.08889998719692204787179434788413345813751220703125f,0.136399994200468110960855483426712453365325927734375f,0.96829998219460222674825899957795627415180206298828125f,-0.122099991077184721888215790386311709880828857421875f,0.7683999775707721635598090870189480483531951904296875f,-0.0415000164732337584183596845832653343677520751953125f,0.2508000232100489057529557612724602222442626953125f,0.44300000366568514920118104782886803150177001953125f,0.0593999964877960362485964651568792760372161865234375f,-0.67600000643730151494992242078296840190887451171875f,0.081199995625019116829435006366111338138580322265625f,-0.53240000741779802684305877846782095730304718017578125f,-0.086099998050927606385585022508166730403900146484375f,-0.6825999595761305105412475313642062246799468994140625f,-0.139699975836277301421972651951364241540431976318359375f,-0.5237999793410299531615237356163561344146728515625f,-0.045199997830390969966174452565610408782958984375f,-0.649399990415572947455302710295654833316802978515625f,0.27919996412396397289512606221251189708709716796875f,-0.14899998328089747445801549474708735942840576171875f,0.43739999121278483329433584003709256649017333984375f,-0.5367000119566913696189658367075026035308837890625f,0.19639998101294064980493203620426356792449951171875f,-0.183799989026784960088889420148916542530059814453125f,-0.5627999830365182365454757018596865236759185791015625f,-0.53989999288916579178732035870780237019062042236328125f,0.33320000343322764280173942097462713718414306640625f,1.28429999820590001746722919051535427570343017578125f,0.721800001558661197265109876752831041812896728515625f,0.032200012791156584768259563134051859378814697265625f,-0.55809997411966350000511738471686840057373046875f,-0.29969998984336854164212127216160297393798828125f,0.312000007957220049092939007095992565155029296875f,0.28900000488758070815720202517695724964141845703125f,0.413800012794137028748764350893907248973846435546875f,0.1505999870836733922629946391680277884006500244140625f,0.6761999865353107708187962998636066913604736328125f,0.28209999629259119302560065989382565021514892578125f,-0.5133999927699566256933394470252096652984619140625f,-0.7344999754726888596678691101260483264923095703125f,-0.21829998811781425871458850451745092868804931640625f,-0.2950000038743016972375698969699442386627197265625f,0.0592999898612500686567727825604379177093505859375f,0.2323000002324580037793566589243710041046142578125f,-0.43839999649524674651956956950016319751739501953125f,-0.3453999965548515405089347041212022304534912109375f,0.07680001340508457730038571753539144992828369140625f,0.0231999946415424318502118694595992565155029296875f,-0.3476000020444391935825478867627680301666259765625f,0.22939999466240390546545313554815948009490966796875f,0.333599987769126915537754030083306133747100830078125f,0.35379997427463560910609885468147695064544677734375f,-0.027600019031762901278170829755254089832305908203125f,0.218400012737512749794177580042742192745208740234375f,0.037200000348687201068287322414107620716094970703125f,-0.0668000026017427916968927092966623604297637939453125f,0.170399986869096853325800111633725464344024658203125f,0.818199975270033075247511078487150371074676513671875f,0.3437999885052442916588688603951595723628997802734375f,0.17620000814199465732912130988552235066890716552734375f,-0.212800007697939719264468294568359851837158203125f,0.0166999989181757513279080740176141262054443359375f,0.51860001563429847237784997560083866119384765625f,0.5024999938905239105224609375f,-0.120300002780556525294741732068359851837158203125f,-0.54119998981356598477532315882854163646697998046875f,-0.27600000785291189941972334054298698902130126953125f,0.360199998128413856335328091518022119998931884765625f,0.5789999749362471082037018277333118021488189697265625f,0.380399979977309732337431569249019958078861236572265625f,0.157699987101555194612956256605684757232666015625f,-0.8017999631881718158865623991005122661590576171875f,-0.0722999976247553632902054232545197010040283203125f,-0.41009999623298654114478267729282379150390625f,0.59879996965527571006759899319149553775787353515625f,-0.2671000162973997671400638864724896848201751708984375f,-0.797800027292967062209072537370957434177398681640625f,-0.1416000008225439810161105924635194242000579833984375f,-0.71660002064108851982382475398480892181396484375f,0.28600000956654536565793023328296840190887451171875f,-0.33499999836087201998680029646493494510650634765625f,0.127800014291703956104129247250966727733612060546875f,0.17749999433755903055498492904007434844970703125f,-0.278399987703561890839409898035228252410888671875f,0.3017000113233923030975347501225769519805908203125f,0.245299980503320913527431912370957434177398681640625f,-0.7607999595820909011223420748137868940830230712890625f,0.231500013493000977859992417506873607635498046875f,0.1485999645233153643175683100707828998565673828125f,-0.65799997103214291627182319643907248973846435546875f,0.2221999996781345299012855321052484214305877685546875f,-0.933199989724158740500570274889469146728515625f,-0.15239998856782932534770225174725055694580078125f,-0.70879997889399515287323083612136542797088623046875f,-0.0265999763846398451505592674948275089263916015625f,0.433999984622001733214347041212022304534912109375f,-0.1105999972164628530890695401467382907867431640625f,-0.2300000118464231579906709157512523233890533447265625f,0.40479999859333004597061744789243675768375396728515625f,-0.21960002661645450228178333418327383697032928466796875f,0.3634000054359436404638472595252096652984619140625f,-0.42639998153448122053532642894424498081207275390625f,0.17020001377463334080175627605058252811431884765625f,-0.663400021454692012667919698287732899188995361328125f,0.30980000008344621420519615639932453632354736328125f,-0.53080002510994706010905019866186194121837615966796875f,0.0124999830126757416337568429298698902130126953125f,0.78759997820258131895343467476777732372283935546875f,-0.1233000512614850219250683949212543666362762451171875f,0.78879999990463289805120439268648624420166015625f,0.3842999949276446614021551795303821563720703125f,0.13000000298023195455243694595992565155029296875f,0.8120000215172762381143911625258624553680419921875f,0.52860002397894856329685353557579219341278076171875f,-0.216200007694959506210352628841064870357513427734375f,-0.4798000118553635839901971849030815064907073974609375f,0.17109999626278871165396822107140906155109405517578125f,-0.31779999495744704063326935283839702606201171875f,-0.08819998927712457970073955948464572429656982421875f,0.068500017091631892895975397550500929355621337890625f,0.238800006759166905112579115666449069976806640625f,-0.121100007587671143483021296560764312744140625f,0.37900000183284276999984285794198513031005859375f,-0.0645999813914297504879868938587605953216552734375f,-0.977899987614154753146067378111183643341064453125f,-0.5017000001847744528049588552676141262054443359375f,-0.02939999339580534609694950631819665431976318359375f,-0.281899993321299557891279619070701301097869873046875f,-0.4075000046193599789745576345012523233890533447265625f,0.4570000204741955229792438331060111522674560546875f,0.4661000120580194305830445955507457256317138671875f,-0.19750000067055228925028131925500929355621337890625f,0.1570000145509837796709007307072170078754425048828125f,-0.35769998171851058454961957977502606809139251708984375f,-0.473699994526058321131500861156382597982883453369140625f,0.21919997231960319794552560779266059398651123046875f,1.29819994889497802859068542602472007274627685546875f,0.760799979847669671784160527749918401241302490234375f,0.2561999959230423229428197373636066913604736328125f,1.006499989688396379250434620189480483531951904296875f,0.5815000039935112052802423932007513940334320068359375f,-0.14339999843239770171976488200016319751739501953125f,-0.437100004786252771538102024351246654987335205078125f,-0.4037000119417906152108344031148590147495269775390625f,0.00179999358654026053727648104541003704071044921875f,0.204599989587068709084860529401339590549468994140625f,-0.2941000163942575351683217377285473048686981201171875f,-0.15920000301599512848582662627450190484523773193359375f,0.21620000911056980186231157858856022357940673828125f,-0.039199999290704834464804662275128066539764404296875f,0.09660000186562545110291466698981821537017822265625f,0.175499994128942571336438049911521375179290771484375f,-0.2817000100940465490140240945038385689258575439453125f,-0.56719998909831037536832809564657509326934814453125f,-0.7506999964669347491508233360946178436279296875f,0.1283999839782712371061279554851353168487548828125f,-0.72119999920129718162797871627844870090484619140625f,-0.0111999995738265578637538055772893130779266357421875f,-0.60560000764727561772815533913671970367431640625f,-0.400999975293875099424667496350593864917755126953125f,-0.396999981246888733465993936988525092601776123046875f,0.3299999704957006230188198969699442386627197265625f,-0.54219998492598531214525792165659368038177490234375f,0.3197999953523276150946230700355954468250274658203125f,-1.224100028538703810454535414464771747589111328125f,-0.0845999840736393338147536269389092922210693359375f,-0.998900016030669579691902981721796095371246337890625f,-0.5383000100970267709499239572323858737945556640625f,0.83599998632073369009276575525291264057159423828125f,-0.3065000298097733111291063323733396828174591064453125f,0.2889000115603208573844540296704508364200592041015625f,0.55959998452067372110008136587566696107387542724609375f,0.26829998971968882504057773985550738871097564697265625f,0.193699998009204676918670884333550930023193359375f,0.8035999768912791552111229975707828998565673828125f,0.034699957470595510500288582989014685153961181640625f,-0.0536999964259567974522724398411810398101806640625f,0.94919995831251180451459958931081928312778472900390625f,-0.02350003219768392881405816297046840190887451171875f,0.3210000010728837338547236868180334568023681640625f,-1.02839995477199597218032067758031189441680908203125f,0.0730000318288801519628350433777086436748504638671875f,0.106899983859062075453039142303168773651123046875f,-0.65599998065829279170202426030300557613372802734375f,-0.0692999998822809715193216106854379177093505859375f,0.25570001207590120628765362198464572429656982421875f,0.428799998861550957229837877093814313411712646484375f,0.27629999919682723952263359024072997272014617919921875f,0.778400028681755173920464585535228252410888671875f,0.31579999415278425800579498172737658023834228515625f,0.598200009207427552127001035842113196849822998046875f,0.674200015756488024720738394535146653652191162109375f,-0.256399995392560864360120831406675279140472412109375f,0.536600017586350563902897192747332155704498291015625f,0.5439000122308730311715407879091799259185791015625f,0.45239998961091032469994388520717620849609375f,0.4004999899938701535262453035102225840091705322265625f,0.1042999602079390797371161170303821563720703125f,-0.071900001814961367330170105560682713985443115234375f,0.58100000926852235494379783631302416324615478515625f,0.4965999939680099117822464904747903347015380859375f,0.66810000824332238522629268118180334568023681640625f,-0.1384000097572803955614517690264619886875152587890625f,0.333299995046854036218064720742404460906982421875f,-0.0690000068247318409930812777020037174224853515625f,-0.394600006127357516572118356634746305644512176513671875f,-0.2103999977096915330321280634962022304534912109375f,-0.14869999696612357098501888685859739780426025390625f,0.5251999890387057945417836890555918216705322265625f,0.853699982511997301770634294371120631694793701171875f,0.7379999926388263187249094698927365243434906005859375f,-0.0105999899148940812665387056767940521240234375f,-0.407099991226196333826692352886311709880828857421875f,-0.2330999954283237141083873211755417287349700927734375f,0.225200018393993506293782047578133642673492431640625f,-0.108399989640712846039605210535228252410888671875f,0.1870000098943709332388607435859739780426025390625f,0.1255999957263469468671246431767940521240234375f,-0.41050000455975510504913472686894237995147705078125f,-0.442200022923946445274623329169116914272308349609375f,0.26860000758767110795588450855575501918792724609375f,0.43800001752376527264232208835892379283905029296875f,-0.103300000560283766759539503254927694797515869140625f,-0.06960000910758967762603788287378847599029541015625f,0.37479998607635511831404073745943605899810791015625f,-0.194000012040138170021918995189480483531951904296875f,-0.2989999992251395344311504231882281601428985595703125f,-0.77689999957978705946715081154252402484416961669921875f,-0.6261000119090081028616623370908200740814208984375f,-0.70840000722408280608988206950016319751739501953125f,-0.9482000105857846250501097529195249080657958984375f,-0.56979999978542306138251660740934312343597412109375f,-0.77540001041293127315157107659615576267242431640625f,-1.0823000094711776597478092298842966556549072265625f,-0.82810001770555974331955439993180334568023681640625f,-0.22919999467134477555418925476260483264923095703125f,0.10220000981092436376229670713655650615692138671875f,-0.298599993282556486207113266573287546634674072265625f,0.147400007939338717477539830724708735942840576171875f,0.01640000582337375334418538841418921947479248046875f,-0.115700003656745042945885870722122490406036376953125f,0.5382000028371809463578756549395620822906494140625f,0.5922000262394551128863895428366959095001220703125f,-1.38240003654956833401001858874224126338958740234375f,-0.120599993342161315723615189199335873126983642578125f,-1.24700002945214549754382460378110408782958984375f,0.86030002043843278869417190435342490673065185546875f,-0.2221999949097632764960508211515843868255615234375f,0.6687000208079816321316002358798868954181671142578125f,0.6362000025913119305442933182348497211933135986328125f,-0.3449999914318322424833240802399814128875732421875f,0.5900000172853470115086338410037569701671600341796875f,-0.85969999341964697947560125612653791904449462890625f,-0.401799987924098989111598712042905390262603759765625f,-0.86289998575150950177459208134678192436695098876953125f,-0.2143000096797944564741555950604379177093505859375f,0.403599987620115285835709073580801486968994140625f,-0.1221000196501613732635860287700779736042022705078125f,-0.8704000323176384057433097041212022304534912109375f,0.13560000511407821477405377663671970367431640625f,-0.715600031861663321564037687494419515132904052734375f,-0.127099952334165156031531296321190893650054931640625f,0.8773999775409697665651265197084285318851470947265625f,-0.0714999880492683814736665226519107818603515625f,0.339000023066997613341300166212022304534912109375f,0.4065999789178373902132079820148646831512451171875f,0.1811999937251211889588375925086438655853271484375f,1.000400018310546812472239253111183643341064453125f,0.0288000005006789905337427626363933086395263671875f,0.76600000986456873164343051030300557613372802734375f,0.386300020691752621360137709416449069976806640625f,0.5787999909639356399537746256100945174694061279296875f,0.44290000419169661682872174424119293689727783203125f,0.7765999880075451500260896864347159862518310546875f,-0.06159999426603324224060997948981821537017822265625f,0.8338000060886143760541244773776270449161529541015625f,1.1137000272154811142399921664036810398101806640625f,0.20640000939965208459625500836409628391265869140625f,1.1287000288918613843947014174773357808589935302734375f,1.048599994921683986603966332040727138519287109375f,0.0023999957203866273403036757372319698333740234375f,0.859399990788102030592199298553168773651123046875f,-0.05419999407529818569173585274256765842437744140625f,0.365799993556737756961183549719862639904022216796875f,-0.06640001092702163543179949556360952556133270263671875f,0.0231000444531446191831491887569427490234375f,-0.129100001567602096219644636221346445381641387939453125f,0.07060000258088106050990973017178475856781005859375f,0.2170000036358832318228451185859739780426025390625f,-0.14269999902248375889257658855058252811431884765625f,-0.43699999693036062353712623007595539093017578125f,-0.32890000030994404056627899990417063236236572265625f,-0.570900006404519100300376521772705018520355224609375f,-0.47019997890591636036106137908063828945159912109375f,-1.175499956578016647057438603951595723628997802734375f,-0.7642999832302332929856447663041763007640838623046875f,0.07379997531771653740406691213138401508331298828125f,0.6563999716997148681230100919492542743682861328125f,0.46889999315738695173649830394424498081207275390625f,0.45519999336004257628474078956060111522674560546875f,0.8865999872624874544868589509860612452030181884765625f,0.48639998898506153768295234840479679405689239501953125f,-0.52960000076293933801707680686376988887786865234375f,-0.53069999404549594856916883145458996295928955078125f,0.025200005504488753871328299283050000667572021484375f,-0.180999987959861829978081004810519516468048095703125f,-0.552399973517656572852274621254764497280120849609375f,-0.0586999873548748052343171366374008357524871826171875f,-0.15180001623630534623998755705542862415313720703125f,0.463899993455409997977767488919198513031005859375f,0.2054999959170817902531780418939888477325439453125f,0.29220001443028476018071160069666802883148193359375f,-0.00830000249743445550620890571735799312591552734375f,-0.290700018259883119497999359737150371074676513671875f,-0.17680000193119038698341682902537286281585693359375f,-0.0318000052094458141027644160203635692596435546875f,-0.41400001397728924956709306570701301097869873046875f,-0.4559999991357326365459812222979962825775146484375f,-0.79659999381899826431663313996978104114532470703125f,-0.618200001421570721760190281202085316181182861328125f,0.156800019365549214711563763557933270931243896484375f,-0.26229998816251764992557582445442676544189453125f,0.0518000102758406910652411170303821563720703125f,0.155199979203939353311625382048077881336212158203125f,0.99539997822642334313059109263122081756591796875f,0.458999992445111359984366572462022304534912109375f,0.1957999978780744498862986802123486995697021484375f,0.58490000406503650509648650768212974071502685546875f,0.497300018709898150603976318961940705776214599609375f,0.19460001313090291574781076633371412754058837890625f,0.073999999281018968844136907136999070644378662109375f,0.42840000424385049626607724349014461040496826171875f,0.223399986511469084149439368047751486301422119140625f,0.30699999063462024562198848798288963735103607177734375f,1.217100003892183313070063377381302416324615478515625f,0.1644000072538849277492545297718606889247894287109375f,1.074499998755752638857074998668394982814788818359375f,0.32089999947547909187051118351519107818603515625f,0.61559997426867507641645715921185910701751708984375f,0.4618999832257628668230609036982059478759765625f,0.0930000082850455100924591533839702606201171875f,-0.6833999874055383116910888929851353168487548828125f,-0.013199969644844156135832236032001674175262451171875f,-0.2777999849736687565382453612983226776123046875f,-0.197199990141391889864053155179135501384735107421875f,-0.3913999892830848370550711479154415428638458251953125f,0.118699986982345961905593867413699626922607421875f,-0.6745999652981762295667067519389092922210693359375f,0.23310002105832072771818275214172899723052978515625f,-0.4066000073790547730823163874447345733642578125f,-0.032999998301267652323076617904007434844970703125f,-0.2344000015541911619010306822019629180431365966796875f,0.6367000203013422066078419447876513004302978515625f,-0.63159998547434792470767206395976245403289794921875f,0.4765000358447435413467019316158257424831390380859375f,-0.0694999881386753060041883145458996295928955078125f,0.81559999870657851062105692108161747455596923828125f,-0.0965000226572161334814836664008907973766326904296875f,0.9398000218391420190755525254644453525543212890625f,0.26920000152587864050701682572253048419952392578125f,0.7806000070512293387281488321605138480663299560546875f,0.62700000840425484938123190659098327159881591796875f,0.336799990308284780127223712042905390262603759765625f,0.66020000259876254755653235406498424708843231201171875f,-0.759300032031536176191366394050419330596923828125f,0.25060000064372989214689368964172899723052978515625f,-0.5989000344336037340298162234830670058727264404296875f,0.4702000154137611076521352515555918216705322265625f,-0.649599987947940604726682067848742008209228515625f,0.237000028371811222172027555643580853939056396484375f,-0.2799000231385235792913590557873249053955078125f,-0.5132000047743314752324295113794505596160888671875f,-0.320500006608664733676050673238933086395263671875f,-1.07260002007484445130103267729282379150390625f,-0.5591999931812285939969342507538385689258575439453125f,-0.41320000522136712817200532299466431140899658203125f,-0.29699998188018827960377166164107620716094970703125f,0.257000024497509027554542626603506505489349365234375f,0.259000001981854577337571754469536244869232177734375f,0.3132999909862876908306361656286753714084625244140625f,0.33309999524205913257191014054114930331707000732421875f,0.74579998640418043720501373172737658023834228515625f,1.05469996913075458877528944867663085460662841796875f,0.760500000312924751000309697701595723628997802734375f,0.219800002616643841424348693180945701897144317626953125f,0.5762000064097343443592080802773125469684600830078125f,0.173299999219179046150429712724871933460235595703125f,0.7341999982476234976047635427676141262054443359375f,1.352299984884262062934112691436894237995147705078125f,0.8332999935567377836065361407236196100711822509765625f,0.79999999806284893821839432348497211933135986328125f,1.1439999987781046542067997506819665431976318359375f,0.75579998826682537327315003494732081890106201171875f,-0.44699999007582658094861471909098327159881591796875f,-0.5051999873995782763813622295856475830078125f,0.200900031810999024628472398035228252410888671875f,0.29540000057816495626639152760617434978485107421875f,0.012599999213218637805766775272786617279052734375f,0.3249000085353852540492880507372319698333740234375f,0.250000001341104560736994244507513940334320068359375f,0.295099989974498821965198658290319144725799560546875f,-0.0100000038743017771736276699812151491641998291015625f,0.0727999955534934795053914058371447026729583740234375f,0.244099993968009965072951672482304275035858154296875f,0.1596000002413988649863085811375640332698822021484375f,0.5609999995827674723614109097979962825775146484375f,1.05819999583363522077661400544457137584686279296875f,0.702699996414780425624257986783050000667572021484375f,0.39740001136660563219038522220216691493988037109375f,0.5165000199377534073619244736619293689727783203125f,-0.21970001092851187962651238194666802883148193359375f,-0.1506000016868114244061871431767940521240234375f,0.080500000983476649452086348901502788066864013671875f,-0.2157000067859888847721094862208701670169830322265625f,-0.12000001862645159889098067651502788066864013671875f,0.323899988612532663267984389676712453365325927734375f,0.4392000013768673216674187642638571560382843017578125f,0.03519999261498441001094761304557323455810546875f,0.2222999963581562266057289889431558549404144287109375f,-0.1154000183105470256350599811412394046783447265625f,-0.86399999147653527842294352012686431407928466796875f,0.0018000204831365440583113013417460024356842041015625f,-0.53550000862032154369529735049582086503505706787109375f,-0.126399984365702044986079499722109176218509674072265625f,-0.50210000456273584656941011417075060307979583740234375f,-0.9925000157952315049669778090901672840118408203125f,-0.15680001400113052767437693546526134014129638671875f,-1.0243000132933255397205130066140554845333099365234375f,0.187200009904056940968075650744140148162841796875f,-0.061399999378621561429980602042633108794689178466796875f,0.05060000122115015930290837786742486059665679931640625f,-0.6433000032424924796714549302123486995697021484375f,-0.35679998360276243829503073357045650482177734375f,-0.7783000038012863885938941166386939585208892822265625f,-0.948799971294402411103874328546226024627685546875f,-0.095599994385242581529382732696831226348876953125f,-1.00759998639821990451537203625775873661041259765625f,-0.2758000661253934282513000653125345706939697265625f,-0.78099997887015337738603193429298698902130126953125f,-0.0972000193849209548346834708354435861110687255859375f,-1.0384000152707102415661211125552654266357421875f,0.0297999880135062511499199899844825267791748046875f,-0.874200011695921208598747398355044424533843994140625f,0.00769999574422808308327148552052676677703857421875f,0.302599989378452338684155620285309851169586181640625f,-0.06430001765191561968748601429979316890239715576171875f,-0.0233000019013882564422601717524230480194091796875f,-0.166599998140334992768885058467276394367218017578125f,-0.05769999708533279925148917754995636641979217529296875f,-0.5607999963879581883929859031923115253448486328125f,-0.36419998498559014166175984428264200687408447265625f,-0.6529999958053231967625151810352690517902374267578125f,0.412500042021274992976032081060111522674560546875f,0.726399977809190744437728426419198513031005859375f,0.21229999490529305461450348957441747188568115234375f,0.74050001633167283898728783242404460906982421875f,0.02240000391602503526655709720216691493988037109375f,0.604300010015070387936475526657886803150177001953125f,-0.0078999955862759652092108808574266731739044189453125f,-0.83899998134374598190987626367132179439067840576171875f,-0.027099965223669642000459134578704833984375f,0.30900004512071621576296820421703159809112548828125f,-0.126999995738267923428566064103506505489349365234375f,-0.270599999600648732922536510159261524677276611328125f,-0.7621000043392178735501829578424803912639617919921875f,-0.55690001403391364842576649607508443295955657958984375f,0.203600010716915136299576261080801486968994140625f,-0.2652999909341335893486757413484156131744384765625f,-0.00579999720752244485311166499741375446319580078125f,-0.307999996364116679359312911401502788066864013671875f,-0.385299988698959428035095697850920259952545166015625f,-0.2551999952226877699246188058168627321720123291015625f,0.48019998635649674412206877605058252811431884765625f,0.62229998473525061086775167495943605899810791015625f,0.514899982681870316270078546949662268161773681640625f,-0.17120000501275089987984756589867174625396728515625f,0.08640001387000051380482545937411487102508544921875f,0.391000025138259044155120136565528810024261474609375f,-0.10460000925660128956451444537378847599029541015625f,0.1621999923765660156504964106716215610504150390625f,0.3864999984800814303298466256819665431976318359375f,0.071600002610683333159613539464771747589111328125f,-0.076199995625019223410845370381139218807220458984375f,0.3022000200927259516703315966879017651081085205078125f,-0.144599991375207903576693979630363173782825469970703125f,-0.499499985940754509528005655738525092601776123046875f,-0.1162999921560288196786814296501688659191131591796875f,-0.57280001626610754783541779033839702606201171875f,-0.5882000074565407743421019404195249080657958984375f,-0.70340002294480807876198014128021895885467529296875f,0.332599999511241861682719900272786617279052734375f,0.8244999981597065374927524317172355949878692626953125f,0.19769999529719350928047560955747030675411224365234375f,-0.092399997061491045968750768224708735942840576171875f,-0.4338999924123287943444893244304694235324859619140625f,0.28360001533627510195145760008017532527446746826171875f,0.0976000050246714767610001217690296471118927001953125f,-0.141599990093708161253971411497332155704498291015625f,0.0990000068992376025533985739457421004772186279296875f,0.05179999179840055756329775249469093978404998779296875f,0.7811000154107805570902200997807085514068603515625f,0.12049999979138359407215830287896096706390380859375f,0.182200001466274130734745995141565799713134765625f,0.737600008898973147353217427735216915607452392578125f,0.6169000173121690355770851965644396841526031494140625f,-0.69159998383522047760152418049983680248260498046875f,0.78210000955462455340949645687942393124103546142578125f,0.2455000170767309697339442209340631961822509765625f,0.529199997204541805473354543210007250308990478515625f,0.3245000038221477201005882307072170078754425048828125f,-0.755100006365776010852641775272786617279052734375f,0.3957999895334243234401583322323858737945556640625f,-0.601900017759203986855709445080719888210296630859375f,0.390099999064207114685132182785309851169586181640625f,-0.0253999983429908926524376511224545538425445556640625f,0.401300002735853145185984658382949419319629669189453125f,-0.76489996681213323626025157864205539226531982421875f,-0.115999994069337919455620067310519516468048095703125f,-0.75349997220188347224478775387979112565517425537109375f,-0.1936000217437747750182097661308944225311279296875f,-0.78359999939203195395975853898562490940093994140625f,-0.1403999897003169738951555700623430311679840087890625f,-0.192800001364946016479962054290808737277984619140625f,-0.6997999710261824102275340919732116162776947021484375f,-0.158199974413216148949601347339921630918979644775390625f,-0.73140003488063864978130368399433791637420654296875f,-0.253600007736682453440835161018185317516326904296875f,-0.58620001700818524792424568659043870866298675537109375f,-0.2999000099882482839319663980859331786632537841796875f,0.0795999993473290867740388421225361526012420654296875f,-0.1605000067949295949887300594127736985683441162109375f,1.2874000532388691908636246807873249053955078125f,0.205600001537799670359163428656756877899169921875f,1.18060004355907466333519550971686840057373046875f,0.316500012449920209434139906079508364200592041015625f,0.00359999708235270166056807283894158899784088134765625f,0.1086999987542629497738744248636066913604736328125f,-0.9417000276029110494846463552676141262054443359375f,-0.370599991703033371237552273669280111789703369140625f,-0.6889000021725892874968621981679461896419525146484375f,-0.54800000545382498984281482989899814128875732421875f,0.06439999339580548820549665833823382854461669921875f,-0.4356000058591364965110415141680277884006500244140625f,0.882300039795041346479820276726968586444854736328125f,-0.310400006687640672708994316053576767444610595703125f,0.60650003107637218136005685664713382720947265625f,-0.21980002213716520742536886245943605899810791015625f,-0.732199987906217319988400049624033272266387939453125f,-0.28840000077933070077307320389081723988056182861328125f,0.6890999894976612694108553114347159862518310546875f,-0.145000020489096659304095737752504646778106689453125f,0.2921999892473221560607044011703692376613616943359375f,0.76189997976124301803935168209136463701725006103515625f,0.5210999900043009613881395125645212829113006591796875f,-0.108600005948543465450484291068278253078460693359375f,-0.3769000126183030818793895377893932163715362548828125f,0.06829999936819086681083490475430153310298919677734375f,0.033000003367662511521984924911521375179290771484375f,-0.0331000018358229564086059326655231416225433349609375f,-0.44990001501739029787785284497658722102642059326171875f,0.2140000047385692827361935997032560408115386962890625f,-0.114500000283122016497827644343487918376922607421875f,-0.0630000044107437151552630894002504646778106689453125f,-0.09060000392198563901047236868180334568023681640625f,-0.93640000969767545058175528538413345813751220703125f,-0.362700001332163513012574185268022119998931884765625f,0.16979999904036535696150167495943605899810791015625f,-0.37110001235604261893286093254573643207550048828125f,0.342099997410178158219196120626293122768402099609375f,-0.5153999792695047599266899851500056684017181396484375f,-1.243099953630566911755295222974382340908050537109375f,-0.7938999779582023297308523979154415428638458251953125f,0.1217999855399132247413263030466623604297637939453125f,0.673099974864721506406795015209354460239410400390625f,0.1058999875545503233098543205414898693561553955078125f,-0.0009999997317791109452400633017532527446746826171875f,0.15130000199079507439137159963138401508331298828125f,0.108700003001093836729751274106092751026153564453125f,0.07059998544454593183417046020622365176677703857421875f,0.139999970681965735241902848429162986576557159423828125f,0.45629998850524422504548738288576714694499969482421875f,0.42620000620484332909398972333292476832866668701171875f,0.982400015874206555821501751779578626155853271484375f,0.45830000920295699273765421821735799312591552734375f,-0.0010000010728834496376293827779591083526611328125f,-0.3935000106096264715915822307579219341278076171875f,-0.35550001946091658311388528090901672840118408203125f,-0.15519999365806569358028355054557323455810546875f,-0.7681999921083448867875631549395620822906494140625f,-0.52560001539587997143598840921185910701751708984375f,0.145800010395050083644719052244909107685089111328125f,-0.15559999215006836692509750719182193279266357421875f,0.157900008252263024388639678363688290119171142578125f,0.0803999999821185529214062626124359667301177978515625f,0.0170000143647197621277200596523471176624298095703125f,-0.22689999525845028216508580953814089298248291015625f,-0.4507999794006349869590621892712078988552093505859375f,-0.31269998497813944471346303544123657047748565673828125f,-0.479300012212991788373983581550419330596923828125f,-0.137999992638826451951672424911521375179290771484375f,-0.323499996490776542401590631925500929355621337890625f,0.61770002450346961353488950408063828945159912109375f,0.63319998868107774114832864142954349517822265625f,0.49309999494403589093138862153864465653896331787109375f,-0.4554999961405992170426770826452411711215972900390625f,0.43899998849630350594708261269261129200458526611328125f,-0.3815000117421150083174552491982467472553253173828125f,-0.6538000037789348795058685936965048313140869140625f,0.60699997308850317523365447414107620716094970703125f,-0.4000000106915828634868148583336733281612396240234375f,-1.4147000163674352535281286691315472126007080078125f,-0.06679998717904123139987859758548438549041748046875f,-1.1121000016942621524407286415225826203823089599609375f,0.3488999953180549340459037921391427516937255859375f,0.6051999674320225341972445676219649612903594970703125f,0.4986999808356167296352623452548868954181671142578125f,0.61790001160502416865938357659615576267242431640625f,-0.15319999642968173958479383145458996295928955078125f,0.3977000101611019022840309844468720257282257080078125f,0.0528999961078167046935050166212022304534912109375f,-0.185599994832277292999833662179298698902130126953125f,0.02470000150352713585988340128096751868724822998046875f,-0.65460000556856368092439879546873271465301513671875f,-0.51659997034966986806381328278803266584873199462890625f,-0.54999998025596152917415793126565404236316680908203125f,-0.0365999977305531132287796935997903347015380859375f,-0.23420000383555861223072724897065199911594390869140625f,-0.2056000049650668071166137451655231416225433349609375f,-0.01349997723102536184569544275291264057159423828125f,0.569999997466802188483825375442393124103546142578125f,0.03550000340491521466645963300834409892559051513671875f,-0.02219997106790483343274900107644498348236083984375f,0.665999996304511565625716684735380113124847412109375f,0.09460000642538057835650988636189140379428863525390625f,-0.75100002044439317927526644780300557613372802734375f,0.066600000375509171846033495967276394367218017578125f,-0.60320001792460697931375079861027188599109649658203125f,0.4043000057309864558874323847703635692596435546875f,0.290899996271729577301812241785228252410888671875f,0.00020001153945903293873698203242383897304534912109375f,-0.119499968878925277859792686285800300538539886474609375f,0.31230002183914162827704785740934312343597412109375f,-0.4512000093340871043068318613222800195217132568359375f,-0.97260002402365153262309149795328266918659210205078125f,-0.5580000175237653792237324523739516735076904296875f,0.3794000088930129521713752183131873607635498046875f,0.4543000046133993663488581660203635692596435546875f,-0.10490000153183931530520567321218550205230712890625f,-0.157800001960992641869552244315855205059051513671875f,0.13519998918771758411594419158063828945159912109375f,-0.078500005692243579602518366300500929355621337890625f,-0.047400003692507741703821011469699442386627197265625f,0.1135999971196055557953741299570538103580474853515625f,0.09869999983459709635980061648297123610973358154296875f,-0.7857999972820284284580338862724602222442626953125f,-1.29799997356534024817165118292905390262603759765625f,-0.816299980309605377470916209858842194080352783203125f,0.47980000336170203212304841144941747188568115234375f,0.69379998932480813067513736314140260219573974609375f,0.342899998417496743030596917378716170787811279296875f,-0.1752000246524811899462292785756289958953857421875f,0.339799983617663503565609062206931412220001220703125f,0.4609999969005584841141853758017532527446746826171875f,-0.21140001923441875675280243740417063236236572265625f,0.34869997163414989671537114190869033336639404296875f,-0.229300020334124443621703903772868216037750244140625f,0.528999982982873984127536459709517657756805419921875f,0.852999961495399805500028378446586430072784423828125f,0.5200999863982203041956609013141132891178131103515625f,0.43039999268054973668995444313623011112213134765625f,0.5445999796032907624976360239088535308837890625f,0.65519998993277539511836948804557323455810546875f,-0.294999992027878743527935512247495353221893310546875f,-0.3231999880477787190358185398508794605731964111328125f,-0.34389999185353514743468394954106770455837249755859375f,0.043799991708993957217899151146411895751953125f,0.6967999855399131803324053180404007434844970703125f,0.60670000897645959270221283077262341976165771484375f,-0.44059998499751085176967535517178475856781005859375f,-0.51249998137354868532611362752504646778106689453125f,-0.537099985638260779552410895121283829212188720703125f,-0.5239999899864196120091719421907328069210052490234375f,-0.38869999849349234910533823494915850460529327392578125f,-0.68390001569539293768684729002416133880615234375f,-0.36159999776780582525503859869786538183689117431640625f,-0.5329000015467404427482733808574266731739044189453125f,0.257400005927681974071674630977213382720947265625f,0.3204000071346755351697765945573337376117706298828125f,0.384600009182095359250297406106255948543548583984375f,0.16630000556707358327912515960633754730224609375f,0.571599976235628304266356281004846096038818359375f,0.02529997495263813789279083721339702606201171875f,0.61379999483823777239877017564140260219573974609375f,-0.179799983245134686882238383986987173557281494140625f,0.49760000133663384591642397936084307730197906494140625f,0.09620000005811457644000483924173749983310699462890625f,-0.231799992245435737547865073793218471109867095947265625f,0.07040000795423984047971543986932374536991119384765625f,0.790199995744227834393313969485461711883544921875f,0.240599970990420075622751028276979923248291015625f,0.896799999733269626034370958223007619380950927734375f,-0.3792000125527383858070606947876513004302978515625f,0.08099999824166292938798505929298698902130126953125f,-0.4534000207468869003690770114189945161342620849609375f,0.0802000134021045862908749768394045531749725341796875f,-0.95519997667074185709878975103492848575115203857421875f,0.049400047412515135647481656633317470550537109375f,1.273900025641918176688704988919198513031005859375f,-0.1331999901711942158044621464796364307403564453125f,1.0081000248208642489089470473118126392364501953125f,-0.51559999974966075342308613471686840057373046875f,-0.118199999111890718950235168449580669403076171875f,-0.5881999987021087061833668485633097589015960693359375f,-1.018200001645087837687242426909506320953369140625f,-0.346799978089332938679945073090493679046630859375f,-0.93979999024868021706424769945442676544189453125f,0.60299998895078876781639110049582086503505706787109375f,0.088599996560812022128317266833619214594364166259765625f,0.56239998849332317742977238594903610646724700927734375f,-0.01500002920627618863136376603506505489349365234375f,-0.916599973851442317851478946977294981479644775390625f,-0.09999999519437563488821751889190636575222015380859375f,0.89389997587203939843902844586409628391265869140625f,0.171199994581937797732962280861102044582366943359375f,0.83809997512549128106940088400733657181262969970703125f,0.41879998306632071347621604218147695064544677734375f,0.28640002765357497338527537067420780658721923828125f,-0.33460001521706583815785052138380706310272216796875f,-0.19740001211166369188276803470216691493988037109375f,0.246700007188320302731199262780137360095977783203125f,-0.4704000078797339057246063021011650562286376953125f,-0.4738000046730039827025393606163561344146728515625f,0.19720000116825087133065608213655650615692138671875f,0.26720000146627409520760920713655650615692138671875f,0.88270001258254016107684947201050817966461181640625f,0.558400010129809221126606644247658550739288330078125f,0.075000000596046501044611432007513940334320068359375f,-0.1028000044941901336414957768283784389495849609375f,0.06969999870955945198147674091160297393798828125f,-0.392200009214878075880506003159098327159881591796875f,-0.4701000081539152830600869492627680301666259765625f,0.01650000920891780964439021772705018520355224609375f,-0.5469999940991403519774394226260483264923095703125f,-1.044999970346689455169553184532560408115386962890625f,-0.4960999862045050168291027148370631039142608642578125f,0.26539999744892117661265729111619293689727783203125f,-0.13830001069307318317669341922737658023834228515625f,0.400100008898973680260269247810356318950653076171875f,-0.2468000103503467101973001263104379177093505859375f,-0.1219999993517995040548385077272541821002960205078125f,-0.15819999996870748315558330432395450770854949951171875f,0.0426000048756598470589551652665250003337860107421875f,0.02310001085102542095484068340738303959369659423828125f,-0.103299995493888852049479964989586733281612396240234375f,-0.19219998432993889991848845966160297393798828125f,-0.92399997270107281366335882921703159809112548828125f,-0.516900000846386209474303541355766355991363525390625f,-0.3052000081121921670757046740618534386157989501953125f,-0.51490000361800181138960397220216691493988037109375f,-0.085599999898672063380899999174289405345916748046875f,-0.27459999644160237863843576633371412754058837890625f,-0.1329000095188614949393013375811278820037841796875f,-0.50890002020299451856999439769424498081207275390625f,0.404600016409158769192799809388816356658935546875f,0.12510000472664817294798922375775873661041259765625f,0.511500021055340869935434966464526951313018798828125f,0.318000002175569562723467242904007434844970703125f,0.39409999665021899772909819148480892181396484375f,-0.064000013381242837340323603712022304534912109375f,-0.1291999717235565725559354177676141262054443359375f,0.10680001228749780306515049232984893023967742919921875f,-0.0912000152945520170533200143836438655853271484375f,-0.76179998241066915198871356551535427570343017578125f,-0.0017999773070214164505387088865973055362701416015625f,-0.180600010776519326327615999616682529449462890625f,-0.8211999595642094362801799434237182140350341796875f,-0.021399969613552283220769822946749627590179443359375f,-0.061100001478195054005482234060764312744140625f,-0.57259999057054500326557899825274944305419921875f,-0.190699992853402999770651149447076022624969482421875f,-0.401800005656480774263172861537896096706390380859375f,0.0558000016033648460478389097261242568492889404296875f,-0.320000004135072291244767939133453182876110076904296875f,-0.024100021386146863733301870524883270263671875f,-0.47359999089837057084650950855575501918792724609375f,0.0921000091448427316009883725200779736042022705078125f,0.22160000469684604240683256648480892181396484375f,0.11819998659491570691670858650468289852142333984375f,0.378200008313358093658962388872168958187103271484375f,-1.3319000276684764827450635493732988834381103515625f,-0.06600000718235943253375808126293122768402099609375f,-1.0869000139221547396317646416719071567058563232421875f,0.51760001646131270636175258914590813219547271728515625f,-0.135600006604194300852128662882023490965366363525390625f,0.4636000203654173645873015630058944225311279296875f,0.12470000317990770355436325189657509326934814453125f,0.1891999872207643651478292667889036238193511962890625f,0.09089999295622119834803243065834976732730865478515625f,-0.2029999838471407969109350233338773250579833984375f,0.64579998878836608611209157970733940601348876953125f,-0.0264000042960048642015635778079740703105926513671875f,-0.08309999811053270235561285517178475856781005859375f,-0.19819999449253078438459851895458996295928955078125f,0.037700017425418064220821179333142936229705810546875f,1.11790001473426858780157999717630445957183837890625f,0.272400020009278431842858481104485690593719482421875f,0.94890000198632418193511739445966668426990509033203125f,-1.3681000563740735032070006127469241619110107421875f,-0.07640000075697894743598226341418921947479248046875f,-1.1667000444039705708831888841814361512660980224609375f,0.120700022506714077508149784989655017852783203125f,0.4445999915242193623043931438587605953216552734375f,0.004699996027350383798193433904089033603668212890625f,-0.2132000219106675586999699589796364307403564453125f,0.650699985756725052254978436394594609737396240234375f,0.376600015574693802733463598997332155704498291015625f,0.26270000751614563938574065105058252811431884765625f,0.023099993565678378359962152899242937564849853515625f,0.201999998718500162198097314103506505489349365234375f,0.53469999669790269081204314716160297393798828125f,0.009999986886978007305515347979962825775146484375f,0.056999997228384113867605265113525092601776123046875f,0.0673999893128873761583008672459982335567474365234375f,-0.22270001496672631180473445056122727692127227783203125f,-0.7762000068008898523430616478435695171356201171875f,-1.427900002068280826961199636571109294891357421875f,-0.8729000131696462716490714228712022304534912109375f,0.11180000133514400584289205653476528823375701904296875f,0.2903000026315449844105387455783784389495849609375f,0.03289999938607213181285260361619293689727783203125f,-0.1009999915361403655111871557892300188541412353515625f,-0.701199994432926132503780536353588104248046875f,-0.106699993255734426611525123007595539093017578125f,0.08940000501871114835239495732821524143218994140625f,0.94410000231266000270125005044974386692047119140625f,0.354900008535385058650035716709680855274200439453125f,0.26700000169873228106354190458660013973712921142578125f,0.6913000062003729073722979592275805771350860595703125f,0.31350000278651701801635454103234224021434783935546875f,0.197600004875659873704307756270281970500946044921875f,-0.0918999923527241691090239328332245349884033203125f,0.15640000455677505186713460716418921947479248046875f,0.300000008046627186786281527020037174224853515625f,0.23289999104142200536671225563623011112213134765625f,-0.279600010523199937750860044616274535655975341796875f,-0.38799999368190773996190046091214753687381744384765625f,-0.5285999762207271857050727703608572483062744140625f,-0.18599998915195481430373547482304275035858154296875f,-0.34980000440478331569238434894941747188568115234375f,-0.10899999743700039545046820421703159809112548828125f,0.252300021392107165496554443961940705776214599609375f,-0.172000007361173601339032757095992565155029296875f,0.21919998736977586162311126827262341976165771484375f,-0.173800000724196213042205272358842194080352783203125f,0.12719999788999558631985564716160297393798828125f,0.324999995343387138024837668126565404236316680908203125f,0.1531999970257282017715994015816249884665012359619140625f,-0.1424000003397465352605877342284657061100006103515625f,-0.648800007057189986880985088646411895751953125f,0.105500023484230265324868014431558549404144287109375f,-0.05880001555085179187898347663576714694499969482421875f,0.06230000842809690908552511245943605899810791015625f,1.297899961388111034210623984108678996562957763671875f,-1.64999994128942528703873904305510222911834716796875f,0.7414999942332503035657964574056677520275115966796875f,1.49259999921321906413140823133289813995361328125f,-1.4021999756872656206496685626916587352752685546875f,0.814300016832351758466757019050419330596923828125f,-0.9661000226378444466490691411308944225311279296875f,0.768800000128150085032530114403925836086273193359375f,-0.524700017261505280430355924181640148162841796875f,-1.08679999120235404319601002498529851436614990234375f,1.2053999856770036824826775045949034392833709716796875f,-0.6041000106573104488916214904747903347015380859375f,-0.70950006055831960072310948817175813019275665283203125f,0.45820003591775881890413302244269289076328277587890625f,-0.3914000266104940006783863282180391252040863037109375f,1.0373000474691398409277098835445940494537353515625f,-0.596000011131167628519733625580556690692901611328125f,0.5543000242829325685534058720804750919342041015625f,0.089400004273652911024328204803168773651123046875f,0.396399992114305543822183608426712453365325927734375f,0.013600001552700913265425697318278253078460693359375f,-0.6038000163704158040900438209064304828643798828125f,0.498400006702542253123056070762686431407928466796875f,-0.329000012561678989442270903964526951313018798828125f,-0.27490001032352484600096431677229702472686767578125f,0.4332000023901463325870508924708701670169830322265625f,-0.1657000076055528126062199589796364307403564453125f,1.098499966502189284511814548750407993793487548828125f,-1.270799952578544722570086378254927694797515869140625f,0.62009999295473061664552005822770297527313232421875f,-0.21760000070333518351617385633289813995361328125f,-0.091199996891617729488643817603588104248046875f,-0.0836000113129617972163032391108572483062744140625f,0.3933000382602216404848149977624416351318359375f,0.01979998100996027687870082445442676544189453125f,0.19140001923441918307844389346428215503692626953125f,-0.08839998457431708089870880939997732639312744140625f,0.1814000056743616529075779908453114330768585205078125f,-0.0401000045776367386451966012828052043914794921875f,0.7888999857068057508513447828590869903564453125f,-0.79659999247789325949042904539965093135833740234375f,-0.272999995544552842563490457905572839081287384033203125f,-1.1175871339474952037562616169452667236328125e-10f,-0.158900004929304117240462801419198513031005859375f,0.150399995958804877371761676840833388268947601318359375f,0.15259999712705607510798699877341277897357940673828125f,0.1509999961554999714508085162378847599029541015625f,-0.57339999284446196980979948421008884906768798828125f,-0.006799993102252432919385682907886803150177001953125f,0.250100017988682310488002258352935314178466796875f,-0.238400014078617272783588987294933758676052093505859375f,-0.15820000462532046725527834496460855007171630859375f,-0.19420001299977318609535359428264200687408447265625f,-1.13359999134540512244484489201568067073822021484375f,-0.3687999931991099433759018211276270449161529541015625f,-1.3604000165224068297220583190210163593292236328125f,0.2467999901592732303612365285516716539859771728515625f,0.01419999854564668595457987976260483264923095703125f,0.368399995303153815484620281495153903961181640625f,-0.369600019389391132307309817406348884105682373046875f,-0.12450000590831045155226775023038499057292938232421875f,-0.38430002696514176108166793710552155971527099609375f,-0.17560000287890431547310754467616789042949676513671875f,-0.0614000003844499542271506697943550534546375274658203125f,-0.0512000035971402756018733271048404276371002197265625f,0.531000016793608420329064756515435874462127685546875f,0.24210000106096263294830350787378847599029541015625f,0.1218000180244445118660223670303821563720703125f,0.584800000679492892885491528431884944438934326171875f,0.09439999928176401322588162656757049262523651123046875f,0.28240001017451277220970951020717620849609375f,-0.4180000129044056933480533189140260219573974609375f,-0.19100000315904619441198519780300557613372802734375f,-0.65800002050399797326463158242404460906982421875f,0.07480001975297956295918311298009939491748809814453125f,0.1580000064224005296154018651577644050121307373046875f,0.230200014817715015169596881605684757232666015625f,0.6042000053256748604013637304888106882572174072265625f,0.1688999988943338248503778231679461896419525146484375f,0.2256000030279157186896554776467382907867431640625f,-0.83400000518560390982969465767382644116878509521484375f,-0.319199999737739492644550409750081598758697509765625f,-0.6158000186651941820770161939435638487339019775390625f,-0.780000008195639082941852393560111522674560546875f,-0.2653000021100044847344179288484156131744384765625f,-0.3744999805390836211671512501197867095470428466796875f,0.6789999818652869834068042109720408916473388671875f,-0.22120000046789645153921810560859739780426025390625f,0.0936000098228453314419539310620166361331939697265625f,0.240600005561113217122937157910200767219066619873046875f,0.040299994808435268822677244315855205059051513671875f,-1.202099998080729648108899709768593311309814453125f,1.10879999074041801776502325083129107952117919921875f,-0.6475000175833702797945079510100185871124267578125f,-0.2176000251412395503081143033341504633426666259765625f,-0.049399992352723909316836170546594075858592987060546875f,-0.111900006359815673562252413830719888210296630859375f,-0.890399974799155558002894395031034946441650390625f,1.60439996478557578285517593030817806720733642578125f,-0.547399996018409495945888920687139034271240234375f,0.17119997684955645667059798142872750759124755859375f,-0.6647999698340896213011319559882394969463348388671875f,0.1227999964475632310723085538484156131744384765625f,-0.3309000140786171328954878845252096652984619140625f,0.475800014418363392820765511714853346347808837890625f,-0.195600010776519894761804607696831226348876953125f,-0.250400005719065621434538115863688290119171142578125f,0.15160000037550924734119917047792114317417144775390625f,-0.133800003257393829159127562888897955417633056640625f,0.7560000341534607315452376496978104114532470703125f,-0.203200023177265709506400526151992380619049072265625f,0.39240001330375662291771732270717620849609375f,0.8021999898433682663068111651227809488773345947265625f,-1.0261999880254266681589569998322986066341400146484375f,0.454300007742643341401844736537896096706390380859375f,-0.1920000104904175941555877216160297393798828125f,0.4360000033080577708233249722979962825775146484375f,-0.1300000044703484292085704510100185871124267578125f,-0.100799964499473038159749194164760410785675048828125f,0.39059998216629010503453400815487839281558990478515625f,-0.06229999382495865489772768341936171054840087890625f,0.84480001319646813584540723240934312343597412109375f,-0.7253999963313340071380252993549220263957977294921875f,0.4617000122547150198215604177676141262054443359375f,-0.910000008270144267186196884722448885440826416015625f,0.78739998767375951604918782322783954441547393798828125f,-0.505900004318356355526020706747658550739288330078125f,-1.2429999907016764382206019945442676544189453125f,0.1220000011026858288687435560859739780426025390625f,-0.9189999848455185915696574738831259310245513916015625f,-0.2212999923050402717450424461276270449161529541015625f,-0.4109999945163718138019248726777732372283935546875f,-0.12560000407099725094894893118180334568023681640625f,-0.16059999803602698076332444543368183076381683349609375f,-0.66899999088048911488613157416693866252899169921875f,0.45040001622438463613207204616628587245941162109375f,0.14900000779330735412031572195701301097869873046875f,0.54900003468990377086811349727213382720947265625f,-0.5236000367939472877054640775895677506923675537109375f,-0.347900006724894073073528488748706877231597900390625f,-0.75830003632307096950171398930251598358154296875f,0.353199989724159113535506548942066729068756103515625f,-0.054100002871453778396926281857304275035858154296875f,-0.373300012481212917236916837282478809356689453125f,0.07859997577369204346808828631765209138393402099609375f,-0.114000004403293164756405531079508364200592041015625f,0.0612999921560284377619609585963189601898193359375f,0.56479999769926048092116843690746463835239410400390625f,0.2313999973297118650794601535380934365093708038330078125f,0.5110000069588418458721434944891370832920074462890625f,-0.4247999968051907604404959784005768597126007080078125f,0.000900002343952710504026981652714312076568603515625f,0.10229999173879633644901332445442676544189453125f,0.4912000184237956812438596898573450744152069091796875f,0.208100004033744345743883741306490264832973480224609375f,0.338700021851062960109857158386148512363433837890625f,0.285599994683265467898536371649242937564849853515625f,0.0479999980777501555895270257678930647671222686767578125f,-0.3524000023514035628835472380160354077816009521484375f,0.208600004532933258616367311333306133747100830078125f,0.11060000068098307934860713430680334568023681640625f,-0.201500008761883009356097318232059478759765625f,0.46459999562203879719390897662378847599029541015625f,0.3462999992340802801749077843851409852504730224609375f,1.0452000151157374574495406704954802989959716796875f,0.536400005972385240937683192896656692028045654296875f,0.169799998742341973212433003936894237995147705078125f,0.653200014162063524736367980949580669403076171875f,-0.63540001868307605104035928889061324298381805419921875f,-0.1896000049166381706822193109474028460681438446044921875f,-0.128500014018267549875673694259603507816791534423828125f,0.59300002013146879686900092565338127315044403076171875f,-0.137500004321336799506525494507513940334320068359375f,-1.25199998277425716963762170053087174892425537109375f,1.2875999629288912284863499735365621745586395263671875f,-0.6950000023841855778528042719699442386627197265625f,0.32559998037815063298694440163671970367431640625f,-0.07300000038743004182606455287896096706390380859375f,0.15229999933838822556708691990934312343597412109375f,0.346600000524521334455130272544920444488525390625f,-0.63119999860525144441680822637863457202911376953125f,0.213800004673004195865360088646411895751953125f,-0.23990005562305416475510355667211115360260009765625f,0.0536000436484809217319025265169329941272735595703125f,-0.1401000170946122835857750033028423786163330078125f,0.9232999856591224130397677072323858737945556640625f,-0.821399977958202409666910170926712453365325927734375f,0.49200000855326653237398204510100185871124267578125f,0.146899947798252217268100139335729181766510009765625f,-0.68579995861351517039139480402809567749500274658203125f,0.10459998705387096862295948085375130176544189453125f,0.542200000497698564316806368879042565822601318359375f,-0.51519998531341559822038789206999354064464569091796875f,0.306599998810887175437756013707257807254791259765625f,-0.83369998340606610298664236324839293956756591796875f,0.8875999956369395871291771982214413583278656005859375f,-0.4557000070095060806352194049395620822906494140625f,0.5443000256240366763194060695241205394268035888671875f,-0.36680001444816578981544807902537286281585693359375f,0.297500011920929008368830182007513940334320068359375f,0.7424000001162287531997208134271204471588134765625f,-0.250799991321563719992582264239899814128875732421875f,0.377000003412365725097288304823450744152069091796875f,0.4689000274300578752217916189692914485931396484375f,0.109000001087784514908207711414434015750885009765625f,0.2272000086188317169444417231716215610504150390625f,-0.070600003921985177157694124616682529449462890625f,-0.07239997225105820888302332605235278606414794921875f,-0.009200017023086815015631145797669887542724609375f,0.4165999999284739629956675344146788120269775390625f,-0.783800000202655500203263727598823606967926025390625f,0.2579000054955482568175284541212022304534912109375f,-0.961100030293315921170460569555871188640594482421875f,0.65840000595748426892583893277333118021488189697265625f,-0.51530002001002450651157005268032662570476531982421875f,0.921500005185603487944945300114341080188751220703125f,1.160000060461462201288895812467671930789947509765625f,0.1135999952197075391069347460870631039142608642578125f,-0.0958999995872378274697211963939480483531951904296875f,-0.1372999976992606008252550964243710041046142578125f,-1.082600027599931014066214629565365612506866455078125f,-0.3475000073015690471578409415087662637233734130859375f,-1.1346000406980518704358473769389092922210693359375f,-0.722799999576806850853927244315855205059051513671875f,-0.2482999991074203993690616698586381971836090087890625f,-0.6543000189185139703340610140003263950347900390625f,-1.04420000890195385778724812553264200687408447265625f,-0.306700003165006673810921711265109479427337646484375f,-0.972600022459030100208110525272786617279052734375f,0.969200020301342224371410338790155947208404541015625f,0.2722000074267387770277082381653599441051483154296875f,0.4160000332593920546742083388380706310272216796875f,0.604799993526935253385090618394315242767333984375f,0.29159999299943439865501204621978104114532470703125f,1.0355999931186430362828332363278605043888092041015625f,-0.104799988013505984696394079946912825107574462890625f,0.076000000625848773694315241300500929355621337890625f,-0.079200002717971784704786841757595539093017578125f,-0.093600029939413076363052823580801486968994140625f,-0.1596000049352646055211835118825547397136688232421875f,0.1035999853849407514871927560307085514068603515625f,-0.75620001581609219698520973906852304935455322265625f,-0.450699999111890736713803562452085316181182861328125f,-1.270200029420852416706111398525536060333251953125f,-0.19139998712241668155087381819612346589565277099609375f,0.01319999989420173047705020508146844804286956787109375f,-0.19409999665021882009341425145976245403289794921875f,-0.2208000331938269056308854487724602222442626953125f,-0.1626000109106302549122347045340575277805328369140625f,0.2117999608039848880025601829402148723602294921875f,-0.1584000034987926852636519470252096652984619140625f,0.036599999406933758638871267976355738937854766845703125f,0.547000013321638167695937227108515799045562744140625f,0.080199997457861993410688228323124349117279052734375f,-0.07539999689012766292961487124557606875896453857421875f,-0.49329999072551711236656046821735799312591552734375f,-0.111799984273314390037512566777877509593963623046875f,-0.0147999949797987806476839978131465613842010498046875f,-0.6246999913334843057555190171115100383758544921875f,0.69860002837479140413989853186649270355701446533203125f,0.2671000085487962305563769405125640332698822021484375f,0.34389997840523722771877146442420780658721923828125f,-0.5879999875724315483438431328977458178997039794921875f,0.195300002333521849351427590590901672840118408203125f,-0.9704000186085703916205602581612765789031982421875f,1.470800001379847454785476656979881227016448974609375f,-0.577000019431114452572728623636066913604736328125f,-0.7146999910354612239871130441315472126007080078125f,0.861999981135129988984999727108515799045562744140625f,-0.411499998927116195091002737171947956085205078125f,-1.035799998772144903114167391322553157806396484375f,1.47899997217953238504151158849708735942840576171875f,-0.60670000987052929986020899377763271331787109375f,1.3158000169515613464454872882924973964691162109375f,-1.184999994337558870682869383017532527446746826171875f,0.72290001831054695458078640513122081756591796875f,0.211200001809000781349823228083550930023193359375f,-0.60759997924566278015845455229282379150390625f,0.155399996629357328714604591368697583675384521484375f,-0.1641999642729761177406544447876513004302978515625f,0.591399973636865805559637010446749627590179443359375f,-0.10969999327063550964567184564657509326934814453125f,-0.1714000605106349439665791578590869903564453125f,-0.520999949663877970351677504368126392364501953125f,-0.061900019025802777150602196343243122100830078125f,-0.243800025236605932832389953546226024627685546875f,0.38300001223385304882640411960892379283905029296875f,-0.167500000596046305645359097979962825775146484375f,-0.209099970647693478298378977342508733272552490234375f,0.57279996128082333317621532842167653143405914306640625f,-0.1340999929621815578428822846035473048686981201171875f,-0.47520004611015309592403355054557323455810546875f,-0.3073999580204489490142805152572691440582275390625f,-0.221700015830993635290724341757595539093017578125f,-0.691600018256902870916746906004846096038818359375f,0.233000001281499891092607867904007434844970703125f,-0.346300015997886845298125990666449069976806640625f,0.530299985867738854494746192358434200286865234375f,-0.251799990233779080739395794807933270931243896484375f,0.267000001177191581547276655328460037708282470703125f,0.2543000314354895152746394160203635692596435546875f,0.491999970629811489430949222878552973270416259765625f,0.093300011587142961388963158242404460906982421875f,1.158900029907375728299712136504240334033966064453125f,-1.09510004749298150983349842135794460773468017578125f,0.195600028806925063662447428214363753795623779296875f,0.226500008873641522910702406079508364200592041015625f,0.690900044476986607833168818615376949310302734375f,0.65639998436570134998646608437411487102508544921875f,0.0710999946609138788744530756957828998565673828125f,-0.016900005614758129013353027403354644775390625f,0.772400015389919314401367955724708735942840576171875f,0.184600003147125268920802909633493982255458831787109375f,-0.036399991816282284418093695421703159809112548828125f,0.1809999989122152985743241515592671930789947509765625f,-0.02619999983459708747801641948171891272068023681640625f,-0.14250000655651096082010553800500929355621337890625f,0.2232000310748816218620049767196178436279296875f,0.233700005906820329482087572614545933902263641357421875f,0.33070002228319683723611888126470148563385009765625f,-0.47660000983774664096159767723293043673038482666015625f,-0.24550000254809856148341395964962430298328399658203125f,-0.37400001502037039102788185118697583675384521484375f,0.70799999144673309015018958234577439725399017333984375f,0.305699993058293983949624816887080669403076171875f,0.939499998029320604331360300420783460140228271484375f,-0.88240002537369743862427640124224126338958740234375f,-0.454100004138052515401113851112313568592071533203125f,-1.345500024378299741556475055404007434844970703125f,0.204200000222027000518210115842521190643310546875f,0.1554999963827430609075719303291407413780689239501953125f,0.230200000177323449435107249883003532886505126953125f,-0.253000007763505063707043518661521375179290771484375f,-0.005600004257261759210706486555864103138446807861328125f,0.448299987596273208367847473709844052791595458984375f,0.6222000563770535563890007324516773223876953125f,0.434700013312697475242885047919116914272308349609375f,1.114400064921380106852666358463466167449951171875f,0.51779999670833376512746326625347137451171875f,0.0798000032313168350395216066317516379058361053466796875f,0.281100012691319012247959108208306133747100830078125f,0.09519998829364784143081124057061970233917236328125f,-0.149800001052021958258819722686894237995147705078125f,-0.646600020194053826116942218504846096038818359375f,-0.56640000090599063042873240192420780658721923828125f,-0.1574999999627471058971650563762523233890533447265625f,-0.9149000159859657088645690237171947956085205078125f,0.67340002473294757745492233880213461816310882568359375f,-0.3045000167489053666258769226260483264923095703125f,-0.2450999729871750076881653512828052043914794921875f,0.187199967938661782085318918689154088497161865234375f,-0.116099997305870061836685636080801486968994140625f,1.155099975371360443432422471232712268829345703125f,-1.13439996582865720853305901982821524143218994140625f,0.63010000278949718222065712325274944305419921875f,1.3266000292837620833097389549948275089263916015625f,-0.899400001442432373011115487315692007541656494140625f,0.70610002397894877645967426360584795475006103515625f,0.43749999880790735318214501603506505489349365234375f,-0.4385999902278185924586750843445770442485809326171875f,0.2360000076293946591476924368180334568023681640625f,0.05930005527734738546286052951472811400890350341796875f,0.33499995738267929201725792154320515692234039306640625f,0.0230000153630973702689743731752969324588775634765625f,-0.476400019979477118425847947946749627590179443359375f,0.19379999834001082614776123591582290828227996826171875f,-0.25670000391006464468546255375258624553680419921875f,0.458499996136873644303477703942917287349700927734375f,-0.72739997500777253680581679873284883797168731689453125f,0.28299999556317911864056213744333945214748382568359375f,-0.414100040459633333966849022544920444488525390625f,0.68340000469088568735287481104023754596710205078125f,-0.26780001254081753359059803187847137451171875f,0.131800012809037581096305302708060480654239654541015625f,0.095399993723631115738470498399692587554454803466796875f,0.06749999899417147286584395260433666408061981201171875f,-0.7715000074207782887469875277020037174224853515625f,0.311200003597140284483657524106092751026153564453125f,-0.39210001074671740894928007037378847599029541015625f,0.1228000694632527256544562987983226776123046875f,-0.07900006501376566347971674986183643341064453125f,0.090100021862983936671298579312860965728759765625f,0.6896999786674984367351726177730597555637359619140625f,-0.939999965876341381232350613572634756565093994140625f,0.39390000027269123084039392779232002794742584228515625f,0.6997999859273438261197952670045197010040283203125f,-0.5239999749362471703761912067420780658721923828125f,0.360600007572770220320990119944326579570770263671875f,-0.288099982166290402574304607696831226348876953125f,1.03119998042583471686839402536861598491668701171875f,-0.2094999994635582396540485206060111522674560546875f,0.618100030893087737382529667229391634464263916015625f,-0.200800002869963467588831917964853346347808837890625f,-0.005000014826655796440491030807606875896453857421875f,-0.21400000820308928695112626883201301097869873046875f,-0.602900028145313626737333834171295166015625f,0.570799998027086363805437940754927694797515869140625f,0.034600002849102029500727439881302416324615478515625f,-0.277999991893768338968584430404007434844970703125f,0.216799995970725856153649147017858922481536865234375f,-0.1286000037878751722786319078295491635799407958984375f,-0.3846000186443330193242218228988349437713623046875f,1.0184000066280365359716597595252096652984619140625f,0.29560000272989273017998357317992486059665679931640625f,0.72000002592802037071351151098497211933135986328125f,0.66480002228617696147239257697947323322296142578125f,0.4239000075742602735573427708004601299762725830078125f,1.0573000364422799890462556504644453525543212890625f,-0.3689999954998490760971208146656863391399383544921875f,-0.1517999971628188615735410849083564244210720062255859375f,-0.3331000033259388093398456476279534399509429931640625f,0.91079998089075064893194166870671324431896209716796875f,0.1910999957039951624437890131957828998565673828125f,0.5267000068902962794936684076674282550811767578125f,-0.179399979314208035674482744070701301097869873046875f,0.011400004966557020225081942044198513031005859375f,-0.48669998334646180637719226069748401641845703125f,-0.37660000119507319737977013573981821537017822265625f,0.041799997870624050477772470912896096706390380859375f,0.321899994885921358900304767303168773651123046875f,0.7502000035673379585432485328055918216705322265625f,0.4039000015765428219793875541654415428638458251953125f,0.9980000264644619534237790503539144992828369140625f,-0.246799995821714190924467402510344982147216796875f,-0.10179999760985369849919379703351296484470367431640625f,0.09280000330209769998646152089349925518035888671875f,0.38259999153912094271845489856787025928497314453125f,-0.027099996069073650772907058126293122768402099609375f,-0.163199989724159166826211730949580669403076171875f,1.00800000262260436301176014239899814128875732421875f,0.34859999991357326631913338133017532527446746826171875f,1.14500001758336988899600328295491635799407958984375f,-0.98300003995001361545291729271411895751953125f,-0.397300010216236232363229419206618331372737884521484375f,-1.186600037032366383726866843062452971935272216796875f,-0.49520002986788735910295145004056394100189208984375f,-0.1498999989241360797365132384584285318851470947265625f,0.5174999824166288675542091368697583675384521484375f,-0.596999997004866056471428237273357808589935302734375f,0.2728000098586083055351991788484156131744384765625f,1.2051999882936481611750423326157033443450927734375f,-0.971599974149465861472663164022378623485565185546875f,0.642600009346008249622173025272786617279052734375f,0.7201999952971931406864314340054988861083984375f,-0.831199996966123233477219400811009109020233154296875f,0.389800016847253072427292863721959292888641357421875f,1.17559999617338206689964863471686840057373046875f,-1.3443999749183657144158132723532617092132568359375f,0.6658000077128409799342989572323858737945556640625f,0.244700030374527699450482032261788845062255859375f,-0.19019999976754231596487443312071263790130615234375f,0.1550000011920928955078125f,-0.35449999640881990625729258681531064212322235107421875f,0.33359998784363276147502119783894158899784088134765625f,-0.2002999970436094567816098788171075284481048583984375f,1.167699959683418331479742846568115055561065673828125f,-1.454599944734573657711962368921376764774322509765625f,0.66219999371767013229828080511651933193206787109375f,0.0229000364899629715864648460410535335540771484375f,0.683399955442548279194170390837825834751129150390625f,-0.0283999896407127749853316345252096652984619140625f,-0.92289999148845680565500515513122081756591796875f,0.84619998929202555615347591810859739780426025390625f,-0.49400001218914990630537431570701301097869873046875f,0.44880000377893480845159501768648624420166015625f,-0.4333999773472552163866566843353211879730224609375f,0.26689999160766575414527324028313159942626953125f,-0.46320000551938989019618020392954349517822265625f,0.06599999287724500618423917330801486968994140625f,-0.2342000003337858515806146897375583648681640625f,0.81339996848106466131866909563541412353515625f,-0.87559996063411293931721957051195204257965087890625f,0.44359999894499768657851745956577360630035400390625f,0.828999995052814853124800720252096652984619140625f,-1.30719996838569674224572736420668661594390869140625f,0.4990000028014183186542140902020037174224853515625f,-0.73300004896521642194784362800419330596923828125f,1.06660001654326919151571928523480892181396484375f,-0.44130002657771161267419302021153271198272705078125f,-0.164800055813788759451199439354240894317626953125f,1.012500003427266204170109631377272307872772216796875f,-0.40199999260902419262464491112041287124156951904296875f,0.005699998776614645379368084832094609737396240234375f,-0.0491000035047530758447464904747903347015380859375f,1.105000027492642811210998843307606875896453857421875f,0.2273000129729509932730024956981651484966278076171875f,0.3428000578403480602673880639486014842987060546875f,-0.47239998916387548888451419770717620849609375f,-0.04839999940097328146038080376456491649150848388671875f,0.20059999557733565467287917272187769412994384765625f,0.17060000220835203066371832392178475856781005859375f,-0.00380000009089709323006900376640260219573974609375f,0.35990001419782657876567100174725055694580078125f,0.617200003850459921750370995141565799713134765625f,0.163099998892843711661981842553359456360340118408203125f,0.207900001770257691902088481583632528781890869140625f,-0.72979999822080099391996554913930594921112060546875f,-0.337599995227158000687950334395281970500946044921875f,-1.008900010740756414406860130839049816131591796875f,-0.047600012922286882854905343265272676944732666015625f,-0.1953999980449676598937003291212022304534912109375f,-0.77519999842643727561153355054557323455810546875f,-0.42600001373887064204382113530300557613372802734375f,-0.3174000001907348877949743837234564125537872314453125f,-1.0188000064611433259642581106163561344146728515625f,-0.45220001703798740066986283636651933193206787109375f,-0.248000000573694645567002226016484200954437255859375f,-0.200100021564960428577251150272786617279052734375f,0.408399991279840168800774335977621376514434814453125f,0.26059999378919596058068464117241092026233673095703125f,0.76759999235868381362024592817761003971099853515625f,0.080800004434585570578519764239899814128875732421875f,0.12080000205039975735843427173676900565624237060546875f,0.50520002375841155384250669158063828945159912109375f,0.7449999985843895888848464892362244427204132080078125f,0.19220000008791682244435605753096751868724822998046875f,0.22550001156330079510325958835892379283905029296875f,-0.217800015148520653696095905615948140621185302734375f,-0.0339000039607287029230775488031213171780109405517578125f,0.3062000022560356082834687185822986066341400146484375f,0.6142000144153829666038291179575026035308837890625f,0.42739999948292961562401615083217620849609375f,1.246100041115283829640247859060764312744140625f,-0.623000000759958982854413989116437733173370361328125f,0.26289998895525901190239892457611858844757080078125f,-0.686499974787235789364103766274638473987579345703125f,0.93259996762275731985170068583101965487003326416015625f,-0.3868000017821788816263506305404007434844970703125f,1.6310000031590465852104898658581078052520751953125f,-1.637599995860457458007886089035309851169586181640625f,0.8994000214099884260576800443232059478759765625f,-0.96759997864961633240454830229282379150390625f,0.73499996930360822489092242904007434844970703125f,-0.5144999994635579554369542165659368038177490234375f,0.0488999836206431837126729078590869903564453125f,-0.56920000234544243511436434346251189708709716796875f,0.05690000069737433108230106881819665431976318359375f,0.88330001084208475958803319372236728668212890625f,-0.769799992334842642094372422434389591217041015625f,0.48260001187920575915768495178781449794769287109375f,-0.433700000393390183717201580293476581573486328125f,0.67459998534023757343902616412378847599029541015625f,-0.2697999962091444103862158954143524169921875f,0.5565999662518503754427001695148646831512451171875f,-0.339399976408481851564147291355766355991363525390625f,0.27769999708533266158383412403054535388946533203125f,0.107999968051910144595240126363933086395263671875f,0.09680000535845767473119849455542862415313720703125f,0.02759999623298625692768837325274944305419921875f,-0.556300032985209469416076899506151676177978515625f,0.04360002882182545391742678475566208362579345703125f,-0.2867000098705290156431146897375583648681640625f,0.0656000027298926369212495046667754650115966796875f,-0.174399980133772114498924565850757062435150146484375f,0.0575999917626379698276650742627680301666259765625f,-0.2708000060737134617738774977624416351318359375f,0.083999996691942246940243421704508364200592041015625f,-0.128800009664893355676440478418953716754913330078125f,1.08649999475479130950361650320701301097869873046875f,-0.9363999763935806353032376136980019509792327880859375f,0.592500005066394663799655972979962825775146484375f,-0.5942000327438119455791820655576884746551513671875f,0.184200012925267220253999767010100185871124267578125f,-0.300500019982457455824942371691577136516571044921875f,0.01730001438856021422907360829412937164306640625f,-0.1574000179976220437083611614070832729339599609375f,0.043199992704391121378648676909506320953369140625f,-0.4914999992251392058051351341418921947479248046875f,0.247599998617172190051860525272786617279052734375f,0.66520003135800376270481137908063828945159912109375f,0.07940000710487371549106683232821524143218994140625f,-0.00519997339248590151328244246542453765869140625f,-0.58720003111958496422317921314970590174198150634765625f,-0.346700004170835018868501720135100185871124267578125f,-0.6661000294923784537104438641108572483062744140625f,-0.142599998542666472900464214035309851169586181640625f,-0.173899998521804821649539007921703159809112548828125f,-0.718599996709823329865685082040727138519287109375f,-0.254200020226836187475782935507595539093017578125f,-0.16850000226497652278112582280300557613372802734375f,0.16220000176429749672024627216160297393798828125f,-0.8456000094354152452069683931767940521240234375f,-0.136600001493096367966018078732304275035858154296875f,0.086199995923042393997093313373625278472900390625f,-0.1445999924927949820130379521287977695465087890625f,0.0954000009879469867968282414949499070644378662109375f,0.2123000042557716682267709984444081783294677734375f,0.60060003350079060879807002493180334568023681640625f,0.433800006796419612697945922263897955417633056640625f,1.175300052177906451333910808898508548736572265625f,-0.18559998522102849616288722245371900498867034912109375f,0.161599998699128610812891793102608062326908111572265625f,0.630800006073713337428898739744909107685089111328125f,1.096399981385469235561913592391647398471832275390625f,0.1749999973177909229349324959912337362766265869140625f,0.0820000063180916782812346355058252811431884765625f,0.8596000102996825820156345798750407993793487548828125f,0.1682000032842159154000682974583469331264495849609375f,0.2460000278949738827805049368180334568023681640625f,-0.3182000073820352437081737662083469331264495849609375f,-0.0040000008270144604693996370770037174224853515625f,-0.051900006210804150441617821343243122100830078125f,0.3278000075489282405527546870871447026729583740234375f,0.234700001093745225233533346909098327159881591796875f,0.276000005245208512860699556767940521240234375f,0.23120001767873787912321859039366245269775390625f,0.0526000059187412827288454764129710383713245391845703125f,-0.2529999916702505569077175096026621758937835693359375f,0.2994000091165303079066006830544210970401763916015625f,0.247800000731646985485667755710892379283905029296875f,0.45390001431703552725593908689916133880615234375f,-0.756000009715557030887111977790482342243194580078125f,-0.02910000946521762443808256648480892181396484375f,1.158800031495093918465499882586300373077392578125f,-1.12440003198981219867391700972802937030792236328125f,0.631800034117699027547132573090493679046630859375f,-0.420300051879882285987832801765762269496917724609375f,0.15300003793835592613703511233325116336345672607421875f,-0.23320001684427271726463004597462713718414306640625f,0.3365999942660327093335581594146788120269775390625f,-0.067400012931227326618000006419606506824493408203125f,0.15650000727176660575423738919198513031005859375f,-0.4968000552773474964851629920303821563720703125f,-0.234399971863627598622770165093243122100830078125f,-0.23680001981258413934483542107045650482177734375f,-1.5144000160455703962725237943232059478759765625f,1.23899999192357057609115145169198513031005859375f,-0.814100024366378960394285968504846096038818359375f,-0.4528999797165393914610831416212022304534912109375f,0.61699998433887959237154063885100185871124267578125f,-0.249800003436207784801581510691903531551361083984375f,0.040900039708613888933541602455079555511474609375f,-0.06540003581941089549900425481610000133514404296875f,0.04960000493526450782155734486877918243408203125f,-0.85539996589571298812870736583136022090911865234375f,0.78059996231049344483921004211879335343837738037109375f,-0.45030000194236630139954513651900924742221832275390625f,1.8255999630928041455035781837068498134613037109375f,-1.711199939149618653999596062931232154369354248046875f,0.994800001573562298062824993394315242767333984375f,1.28739999721050235592656463268212974071502685546875f,-1.3001999930620191658903195275343023240566253662109375f,0.7097000142812728284980039461515843868255615234375f,-0.53030000211000416499018683680333197116851806640625f,0.7086000055760142135596879597869701683521270751953125f,-0.29860001310110106942374841310083866119384765625f,0.28520002673864386366631151759065687656402587890625f,0.1361999995738265578637538055772893130779266357421875f,0.1441000023126601803369339904747903347015380859375f,0.571800029870868087300550541840493679046630859375f,-0.160800014269351976281541283242404460906982421875f,0.289400017610192517025780034600757062435150146484375f,0.10110002204179746598811107105575501918792724609375f,0.1661999916762113560508851151098497211933135986328125f,0.0539999989271162661452763131819665431976318359375f,-0.2985000261068346816273333388380706310272216796875f,-0.411800016385317046996306089567951858043670654296875f,0.3127999942123889098866129643283784389495849609375f,0.0306999989256262484360604503308422863483428955078125f,0.7441000222802163222013405174948275089263916015625f,0.36760000979304319201901307678781449794769287109375f,0.120500002585351484452047543527442030608654022216796875f,-0.165099997574090995300366557785309851169586181640625f,-0.1946000027745962146497049616300500929355621337890625f,-0.132100000204145917592768455506302416324615478515625f,-0.72480001457482579052538085306878201663494110107421875f,0.6587999976694585146930194241576828062534332275390625f,0.1143000006273389157485098621691577136516571044921875f,0.6411000089287757219835839350707828998565673828125f,-0.3416000012695790122307926139910705387592315673828125f,-0.0388000013947487065735231226426549255847930908203125f,0.0883999994754791629247847595252096652984619140625f,-0.168799999308586023971656686626374721527099609375f,-0.06699999745190143851658604035037569701671600341796875f,0.28980002213716549164246316649951040744781494140625f,-0.255600002282857730051546241156756877899169921875f,0.11429999898821119330705187167041003704071044921875f,0.53189998475313160497535136528313159942626953125f,0.521200035858154553380927609396167099475860595703125f,0.3119000081852079109268061074544675648212432861328125f,0.7111000309824948573123037931509315967559814453125f,0.464000006079673621428582919179461896419525146484375f,0.2248000000834464717769378694356419146060943603515625f,0.0824000055551525889541153446771204471588134765625f,-0.6036000090777873250402763005695305764675140380859375f,-0.06960000195503235342897596638067625463008880615234375f,-0.2550000192224981532262972905300557613372802734375f,-0.83600003206729933680207977886311709880828857421875f,-0.2992000100195408851533329652738757431507110595703125f,-1.1660000439882285405701622948981821537017822265625f,-0.88420002380311490952635722351260483264923095703125f,-0.2372000040367245876637980472878552973270416259765625f,-1.1217000417590146099655612488277256488800048828125f,0.204400017982721227127740348805673420429229736328125f,0.1220000031143426699742349228472448885440826416015625f,-0.2101999841213224584635099745355546474456787109375f,0.072200010630488531404580498929135501384735107421875f,0.11530000352561475818191638609278015792369842529296875f,0.39560001643896125500532434671185910701751708984375f,0.84120004003047998963893405743874609470367431640625f,0.25490001285672203579224515124224126338958740234375f,0.0126999584913249918827204965054988861083984375f,-0.9257999834239480918540721177123486995697021484375f,0.0623999909520147610919593716971576213836669921875f,0.722900025612116081674685119651257991790771484375f,-0.21680000267624865983862036955542862415313720703125f,0.36990001315474518861492470023222267627716064453125f,0.22960000638812783080311419325880706310272216796875f,0.2269999965578317624448345668497495353221893310546875f,0.0899000061884522860822244183509610593318939208984375f,0.66869997923374224768622298142872750759124755859375f,-1.2819999680221083604436671521398238837718963623046875f,0.405900003945827592133355210535228252410888671875f,-0.65740000138282805863809699076227843761444091796875f,0.5433999880015851768888524020439945161342620849609375f,-0.3525000083446503396089610760100185871124267578125f,-0.4802000261425973803852684795856475830078125f,-0.055600003027915789743929053656756877899169921875f,-0.2332000102877618274987980839796364307403564453125f,-0.888899983918665981263984576798975467681884765625f,0.7832000080525869378789138863794505596160888671875f,-0.46950001153349862903496614308096468448638916015625f,0.2853000536680223575558557058684527873992919921875f,-0.177400033569335757732687852694652974605560302734375f,0.1672000190496447658006218262016773223876953125f,0.6720000241994856793326107435859739780426025390625f,-0.109600000166893085662422890891321003437042236328125f,0.34520000617504109641231480054557323455810546875f,-0.8663999943494793409826115748728625476360321044921875f,1.1589999995231625717906354111619293689727783203125f,-0.491700013893842768908370999270118772983551025390625f,-0.528000021100044847344179288484156131744384765625f,1.14840000841617584370624172152020037174224853515625f,-0.3410000171661380363730131648480892181396484375f,-0.673300002944468900523133925162255764007568359375f,1.49500001378357350034775663516484200954437255859375f,-0.42800001633167283898728783242404460906982421875f,0.4626000343799585579063204932026565074920654296875f,0.170799969416857155835032244795002043247222900390625f,0.221200012612342788997921161353588104248046875f,-0.2051999888896940404947599745355546474456787109375f,0.133799991187453226615389212383888661861419677734375f,-0.10030000121593474204928497783839702606201171875f,0.503800051164628115429877652786672115325927734375f,-0.11320000920742767647197979385964572429656982421875f,0.2884000137805937669099876075051724910736083984375f,0.1823000010147690641559847790631465613842010498046875f,0.2979000120520591821104972041212022304534912109375f,-0.16219999707043175618537134141661226749420166015625f,0.0302000019654631302046254859305918216705322265625f,-0.4968999954819679487627581693232059478759765625f,-0.9841999838680026702064651544787921011447906494140625f,-0.20739999598115677059695372008718550205230712890625f,-0.567900006687640512836878770031034946441650390625f,0.51959998437166188267610777984373271465301513671875f,0.0320999938711523657985935642500407993793487548828125f,-0.1011000202536589398505384451709687709808349609375f,0.78400001055002233041335557572892867028713226318359375f,0.23660000447332862894000982123543508350849151611328125f,0.70500002324581156898375411401502788066864013671875f,-0.608600013622641267119206531788222491741180419921875f,-0.3918999971956014238827492590644396841526031494140625f,-1.1980000145435329983456540503539144992828369140625f,-0.5737999935716386890049989233375526964664459228515625f,-0.24249999500811092456586948173935525119304656982421875f,-0.42480000067949230668773452634923160076141357421875f,0.400000005811453007709133089520037174224853515625f,0.0224000025749206965741677777259610593318939208984375f,-0.3868000055074691800882646930404007434844970703125f,0.123400010129809611925111312302760779857635498046875f,0.2450000059232115834362275563762523233890533447265625f,0.8255000339150431187817957834340631961822509765625f,0.11080000696778302682332650874741375446319580078125f,0.1478000027060508703158347998396493494510650634765625f,-0.0363999931573868451550879399292171001434326171875f,-0.639000007495284183534067778964526951313018798828125f,-0.1326000029757619103065735544078052043914794921875f,-0.58490002313852329507426475174725055694580078125f,-0.284800000381469686772106797434389591217041015625f,-0.07789999845474959716540297449682839214801788330078125f,0.22950001823902166364632648765109479427337646484375f,0.8870000035613774258536068373359739780426025390625f,0.19539999957233666894040879924432374536991119384765625f,0.13570000224113432096828546491451561450958251953125f,0.442400007194280409095199502189643681049346923828125f,0.029199998247623426550489966757595539093017578125f,0.11160000529289249016073881648480892181396484375f,-0.1675999813318249653804059562389738857746124267578125f,-0.12770000561624772483781953269499354064464569091796875f,0.222100026988982790498994290828704833984375f,-0.0098000188589093539803798194043338298797607421875f,0.1190000075697899006854640902020037174224853515625f,0.05270001690387715598262730054557323455810546875f,0.73019998821914189335302580730058252811431884765625f,-0.01560000258088134472700403421185910701751708984375f,-1.26229996491670615199609528644941747188568115234375f,1.5673999472171071811743559010210447013378143310546875f,-0.7155999976634976889044992276467382907867431640625f,0.987899985527992186007395503111183643341064453125f,-0.95699996727705016752452138462103903293609619140625f,0.536400006270408624686751863919198513031005859375f,0.793000001728535242051520981476642191410064697265625f,-1.0711999822139741667115231393836438655853271484375f,0.46100000822544107137446189881302416324615478515625f,-0.0574000296950334387702241656370460987091064453125f,0.271600013712048227176865111687220633029937744140625f,-0.06720000295639039222805877216160297393798828125f,-0.61520000140666919463683370850048959255218505859375f,0.4453999838143587464145412013749592006206512451171875f,-0.338499997794627915936871431767940521240234375f,0.98300001573562667545047588646411895751953125f,-0.593799992305040547080352553166449069976806640625f,0.5121000180482866426245891489088535308837890625f,-0.4788999964356417393673837068490684032440185546875f,0.357800001439451786922063547535799443721771240234375f,-0.238200014758110256707368534989655017852783203125f,0.159200022983551292554693645797669887542724609375f,0.39999999597668622897117529646493494510650634765625f,0.06240000019073477233177982270717620849609375f,-0.6654999892115593951302798814140260219573974609375f,1.133199988010525682824436444207094609737396240234375f,-0.3967000071883202139133572927676141262054443359375f,-0.6397000229239466762010124512016773223876953125f,0.183199992704391689812837284989655017852783203125f,-0.3251000120282174776775718783028423786163330078125f,1.40350001001358037910904386080801486968994140625f,-1.1907999818593264507171625155024230480194091796875f,0.7611000172734261326468185870908200740814208984375f,0.6986000032663337577787387999705970287322998046875f,-0.904600011044740082155612981296144425868988037109375f,0.393100016140937924546960857696831226348876953125f,-0.26089999232292182540504654753021895885467529296875f,-0.68950002703070634169080221909098327159881591796875f,-0.441600004994869177465943721472285687923431396484375f,-0.3852000012576579646150776170543394982814788818359375f,-1.1020000277757642237475010915659368038177490234375f,0.222999996289610979971484994166530668735504150390625f,0.03080000193864107504992944086552597582340240478515625f,0.32070001535415659787986442097462713718414306640625f,0.878599986502527574572241064743138849735260009765625f,0.2523999892011283829873491413309238851070404052734375f,1.1100999983191481845778980641625821590423583984375f,0.5582000233635309172086635953746736049652099609375f,0.2041000071369112045527316467996570281684398651123046875f,0.417000027962029395922627372783608734607696533203125f,1.1640000154674055021786216457257978618144989013671875f,0.3258000061482191345163528239936567842960357666015625f,1.0838000446081164085398995666764676570892333984375f,0.97860002628862918783170243841595947742462158203125f,0.3072000107422472314766537238028831779956817626953125f,1.2409000397086149547476452426053583621978759765625f,-0.6424000185191633960357648902572691440582275390625f,-0.4475000041350722934652139883837662637233734130859375f,-1.1649000124096868802325843716971576213836669921875f,-0.0884000105023385884805975365452468395233154296875f,-0.0697000054523349010793253910378552973270416259765625f,0.3454999892115591109131855773739516735076904296875f,0.3336000008821484730248130290419794619083404541015625f,0.20129999725967639978563283875701017677783966064453125f,0.57230001051425904989855553139932453632354736328125f,0.2021999984115361304048974488978274166584014892578125f,0.034499998718500124450514476848184131085872650146484375f,0.066599999481439464688037332962267100811004638671875f,0.890600018748641364396689823479391634464263916015625f,0.2239000080585480390027441899292171001434326171875f,0.78020003776550339580353465862572193145751953125f,0.65780004733502916991483289166353642940521240234375f,0.4456000133469701918187411138205789029598236083984375f,1.0809000468611724699030673946253955364227294921875f,0.1558000107675792111905366255086846649646759033203125f,0.0956000040337443568461139875580556690692901611328125f,-0.1200999917626379698276650742627680301666259765625f,0.165799989011883752709763939492404460906982421875f,-0.1445000021457671923741372665972448885440826416015625f,-0.5342000211954118782387013197876513004302978515625f,-0.11659999866187575889853178523480892181396484375f,-0.669900010770559273254320942214690148830413818359375f,0.1695999930143354816891587688587605953216552734375f,-0.114800009620189324976990974391810595989227294921875f,0.065800015163421932129494962282478809356689453125f,0.15549997527897385651840522768907248973846435546875f,-0.5185999801695349020747016766108572483062744140625f,0.1075999936625360309250254431390203535556793212890625f,0.6648999760508527145930202095769345760345458984375f,-1.412599973359703664499420483480207622051239013671875f,0.419999998211860514629734097979962825775146484375f,0.6098000255644324685277979369857348501682281494140625f,-0.569800012153387047675323628936894237995147705078125f,0.34090001523345725420455210041836835443973541259765625f,1.16199999141693144366627166164107620716094970703125f,-1.7165999798119069286173044019960798323154449462890625f,0.6829000099658966149718253291212022304534912109375f,0.725900010502339654294701176695525646209716796875f,-1.46519998523891015196340958937071263790130615234375f,0.455900015568733607551621389575302600860595703125f,-0.11710004367828474869384081102907657623291015625f,0.0941999933302408720692255883477628231048583984375f,-0.089000004291534651201800443232059478759765625f,-0.37510002186298407877984573133289813995361328125f,-0.2713999928593633370610405108891427516937255859375f,-0.169800005745887716557263047434389591217041015625f,0.09790000311136193289485163404606282711029052734375f,-0.1644000005483625681534931572969071567058563232421875f,0.0690999957561491129354180884547531604766845703125f,0.29879999737143503324432458612136542797088623046875f,-0.3315999946385621566236068247235380113124847412109375f,0.16620000262856482464712826185859739780426025390625f,0.9353999995350843477126545622013509273529052734375f,-1.396599988082051435611674605752341449260711669921875f,0.5483000133752824467592290602624416351318359375f,0.205700067508221451362260268069803714752197265625f,-0.09460003540813932687569831614382565021514892578125f,0.131800019216537833699476323090493679046630859375f,0.32270002256631880044324134360067546367645263671875f,0.1213999926358459990893834401504136621952056884765625f,0.1548000069379806831193491234444081783294677734375f,0.7431999857008459997587124234996736049652099609375f,-0.71239997701942936458863186999224126338958740234375f,0.393000010147690925776942094671539962291717529296875f,-0.57989996889829598103460739366710186004638671875f,-0.426499991625547192342082780669443309307098388671875f,-0.61220001420676706249679455140721984207630157470703125f,-0.0910000023767352228531990476767532527446746826171875f,-0.249500018835068004818822373636066913604736328125f,-0.26260000018179407987872764351777732372283935546875f,-0.0334999997317790842998874722979962825775146484375f,0.3702000043869020373676903545856475830078125f,-0.3321999973684548468355615113978274166584014892578125f,-0.11569999829232691102021135520772077143192291259765625f,-0.49600000494718532451088321977294981479644775390625f,0.496200000020861953231587904156185686588287353515625f,0.1272000022485852976927844792953692376613616943359375f,0.6841000014185905087060746154747903347015380859375f,-0.070199999544024560549360103323124349117279052734375f,-0.2280000021383166153299271172727458178997039794921875f,-0.6443000171303747691808894160203635692596435546875f,0.016399990177154766257672235951758921146392822265625f,-0.1744999982342123612255591069697402417659759521484375f,-0.4909000069260596177400657325051724910736083984375f,-0.1900000114738944478887106015463359653949737548828125f,-0.1652000065475702772488375558168627321720123291015625f,-0.1090000132322312964561206172220408916473388671875f,-0.493400002232193823914485619752667844295501708984375f,-0.17979999911487098973594811468501575291156768798828125f,-0.29490000507086489012209540305775590240955352783203125f,-0.39540000586807710902803592034615576267242431640625f,-0.04499999996274706148824407136999070644378662109375f,-0.71630003179311785288518876768648624420166015625f,0.819200023803114962817062405520118772983551025390625f,0.3244000056520104369184309689444489777088165283203125f,0.6736000314295294000288549796096049249172210693359375f,-0.57240001792311667738744063171907328069210052490234375f,-0.3477000034555792762347437019343487918376922607421875f,-0.49490001986026754821068607270717620849609375f,-0.1797999988168477170091819061781279742717742919921875f,-0.075199999730288957788815196181531064212322235107421875f,0.093699998456239885769036845886148512363433837890625f,0.782000040292740283831562919658608734607696533203125f,0.25660001043379321306048268525046296417713165283203125f,0.1548000201255088636997925277682952582836151123046875f,0.5570000160783530329666746183647774159908294677734375f,0.0211000047564506854058663520845584571361541748046875f,0.0170000224113469045050806016661226749420166015625f,-1.0833999932914972763597916127764619886875152587890625f,0.32410001587271697331971154198981821537017822265625f,-0.876300006163119871160915863583795726299285888671875f,1.084600008586048769387844004086218774318695068359375f,-0.49340001825094237375424199854023754596710205078125f,-0.720400009667873320040598628111183643341064453125f,0.25279999637305738957593348459340631961822509765625f,-0.3656000089883804093915387056767940521240234375f,-0.18779999138116831858269506483338773250579833984375f,0.4859999869912863257326307575567625463008880615234375f,-0.12639999777674670866645101341418921947479248046875f,0.3412999918580066349704793537966907024383544921875f,-0.890999966874719273590699231135658919811248779296875f,0.2252000028967859890371983055956661701202392578125f,0.4366999994397158246783874346874654293060302734375f,-0.492400012931227148982316066394560039043426513671875f,0.226700018215179710523443645797669887542724609375f,0.4580999741196638552764852647669613361358642578125f,-0.532199974048138013671405133209191262722015380859375f,0.24260000234246259509518495178781449794769287109375f,-0.18900002276897520658849316532723605632781982421875f,-0.2276000035345548955234562527039088308811187744140625f,-0.088000001728534726908037555404007434844970703125f,-0.59210000817626706748342257924377918243408203125f,0.469799991962313701066022986196912825107574462890625f,-0.3250000046566128286684715931187383830547332763671875f,-0.174099972808360092813018127344548702239990234375f,1.03320000723302296563588242861442267894744873046875f,-0.14770000081062306662715855054557323455810546875f,0.83980002966225153482326959419879131019115447998046875f,-0.76140000954866404736520735241356305778026580810546875f,0.4707000155031682009365567864733748137950897216796875f,-0.516100039029121493427965106093324720859527587890625f,0.03160000991225253574867792849545367062091827392578125f,-0.27250000745058056139669133699499070644378662109375f,-0.35570000447630878426252820645458996295928955078125f,0.0307999949723483101848842125036753714084625244140625f,-0.17850000032782542547238335828296840190887451171875f,1.1750000671297318177721535903401672840118408203125f,-0.710200028228759716597551232553087174892425537109375f,0.625900038143992976102936154347844421863555908203125f,0.97500000268220876620262060896493494510650634765625f,-1.1442000093489881340502734019537456333637237548828125f,0.54050002080202119714158470742404460906982421875f,-1.11659996923208293395646251155994832515716552734375f,0.784900000488757854100185795687139034271240234375f,0.627199997293949262910928155179135501384735107421875f,0.11680000140964985622105132279102690517902374267578125f,0.3220000115334986645621029310859739780426025390625f,-0.43160002764463445412701503300922922790050506591796875f,-0.2176000073343515861523655985365621745586395263671875f,-0.34580002738237425319312023930251598358154296875f,0.13860001153647905081101043833768926560878753662109375f,0.2346000028118491176343241022550500929355621337890625f,0.419500013470649690816571819595992565155029296875f,-0.081000001445412639355936335050500929355621337890625f,0.058499998986721035709734906049561686813831329345703125f,0.465000002235174125786443255492486059665679931640625f,-0.856600035169721163441636235802434384822845458984375f,-0.2021000146210194525764336503925733268260955810546875f,-0.0698000498533257740518820355646312236785888671875f,-0.509000031933188434862813664949499070644378662109375f,-0.2686000057622790482270147549570538103580474853515625f,0.0044999791979787318041417165659368038177490234375f,-1.047599984461069055896587087772786617279052734375f,-0.23279999793767924831655591333401389420032501220703125f,-0.6490000130832189739749082946218550205230712890625f,0.6787999981164929419463760496000759303569793701171875f,0.245199997010827008825373241052147932350635528564453125f,0.289000002950429379922070438624359667301177978515625f,0.118200004103779843234178770217113196849822998046875f,0.001200001697242270015664189486415125429630279541015625f,-0.060499994128942358173617321881465613842010498046875f,-0.0206000317871570803873737531830556690692901611328125f,-0.162700005169212846833914909439045004546642303466796875f,-0.19360001883804844879932716139592230319976806640625f,0.89360003791153463925667210787651129066944122314453125f,0.279100010061264114380463752240757457911968231201171875f,0.68610004342496466733791748993098735809326171875f,-0.2800000105798243765775623614899814128875732421875f,0.033999998778104778551778508699499070644378662109375f,0.207999991893768054751490126363933086395263671875f,-0.6220000014007087685286023770458996295928955078125f,-0.0721999963998793958808164461515843868255615234375f,-0.0404000063896177152855671010911464691162109375f,-0.213400002828240342722665445762686431407928466796875f,-0.09589999969899654086447071676957421004772186279296875f,0.14240000644922279349202653975225985050201416015625f,0.357200012791156706892792271901271305978298187255859375f,0.0127000025987626941059716045856475830078125f,0.8397999830961231371162512004957534372806549072265625f,-1.0441999703824522693906828862964175641536712646484375f,0.474900003021955452453539692214690148830413818359375f,-0.428400038814544803500439229537732899188995361328125f,0.19900002515315995399447501768008805811405181884765625f,-0.23210001596212403995878048590384423732757568359375f,-0.156099971973895801369280889048241078853607177734375f,0.48099999026954154413004971502232365310192108154296875f,-0.08980000276565558436914216144941747188568115234375f,-0.50400000369548791923079988919198513031005859375f,0.196000001743435792178615884040482342243194580078125f,-0.252000008776783968045265282853506505489349365234375f,-1.3934000283837324474234264926053583621978759765625f,1.063000012978911446026586418156512081623077392578125f,-0.749100026750564751409910968504846096038818359375f,-0.7955000631213184902890134253539144992828369140625f,-0.019799959775805842099316578241996467113494873046875f,-0.400000024437904500018703402020037174224853515625f,-1.3063999609708789595430289409705437719821929931640625f,1.6393999403476720022609924853895790874958038330078125f,-0.742699995595216488908363317023031413555145263671875f,0.89920000793337795386150901322253048419952392578125f,-0.5977999851226807326298740008496679365634918212890625f,0.48510000413060172519408297375775873661041259765625f,0.2543000014841556577493975055404007434844970703125f,-0.225000002011656707878728411742486059665679931640625f,0.13640000492930415276759958942420780658721923828125f,0.17359994306564374699775044064153917133808135986328125f,-0.54779994630515627118683141816291026771068572998046875f,0.1064999841004607983308005714206956326961517333984375f,0.98390003726482400114861093243234790861606597900390625f,-1.12300002557039224626578288734890520572662353515625f,0.5590000279098752411499617664958350360393524169921875f,-0.6650000071525568046126863919198513031005859375f,0.6570000165998930441446646000258624553680419921875f,-0.3585000174641610470871455618180334568023681640625f,-1.0519999961853017111934605054557323455810546875f,1.0824000021278852301520601031370460987091064453125f,-0.5772000139832496046210508211515843868255615234375f,-0.4416000201940537550626686424948275089263916015625f,-0.007000000432133557382030630833469331264495849609375f,-0.21910000708103183342245756648480892181396484375f,0.3763000234484670247780968566075898706912994384765625f,0.561300000053643888264787165098823606967926025390625f,-0.24479998480975584396190924962866120040416717529296875f,-0.210599993379413985561399158541462384164333343505859375f,-0.27059999147951518949639648781158030033111572265625f,-0.8472000412523750245696874117129482328891754150390625f,-0.23100001046061524601071823781239800155162811279296875f,-0.655800044220686828566613257862627506256103515625f,0.54720002307295789734098434564657509326934814453125f,0.355200002598762498706719270558096468448638916015625f,0.941200029897689915969749563373625278472900390625f,0.160799997133016514538894625729881227016448974609375f,0.04230000079125162404380944281001575291156768798828125f,-0.23389999196529398517441222793422639369964599609375f,0.078799998861551312501205757143907248973846435546875f,-0.02329999955445527515252024386427365243434906005859375f,-0.15410000112056732035625827847979962825775146484375f,-1.117000006839633297062164274393580853939056396484375f,-0.330500007018446950723955524154007434844970703125f,-1.19240004727840442910746787674725055694580078125f,0.637000005945563163578526655328460037708282470703125f,0.2453000000983476258653581680846400558948516845703125f,0.8336000265121459307238183100707828998565673828125f,0.170199985238909778928473315318115055561065673828125f,-0.167800002519786328747386505710892379283905029296875f,-0.661100028598308853133858065120875835418701171875f,0.439399993321299309201322103035636246204376220703125f,0.1869999970421194479541782129672355949878692626953125f,-0.1833000089049345859848472173325717449188232421875f,0.7411999816179271061145072962972335517406463623046875f,0.22989999143630257538717387433280237019062042236328125f,0.82229999620914373537061692331917583942413330078125f,0.357199987012148056919613736681640148162841796875f,0.0070000000596046607626021796022541821002960205078125f,0.4564000062704085536324782879091799259185791015625f,-0.23199999380111702151907593361102044582366943359375f,-0.04069999900013210325511181508773006498813629150390625f,-0.38729999680519089366725893341936171054840087890625f,0.50759999295473079428120399825274944305419921875f,-0.0345000015124679038081012549810111522674560546875f,-0.2644999935030938331692595966160297393798828125f,-0.368999991104006941355919479974545538425445556640625f,-0.0599999988451600518857276256312616169452667236328125f,-0.5652999919772145887009173748083412647247314453125f,-0.497599993848800270512811039225198328495025634765625f,0.19360000252127651521050211158581078052520751953125f,-0.15119999800920458365993681582040153443813323974609375f,-0.21940002111196488243649582727812230587005615234375f,-0.0728999860495325169296165768173523247241973876953125f,-0.9438000380516051723844839216326363384723663330078125f,1.2096000335454935026291423127986490726470947265625f,-0.541800031882524901760689317598007619380950927734375f,0.147800034594535389942393521778285503387451171875f,-0.16500002458691565010440172045491635799407958984375f,0.10250000655651092529296875f,0.4366999952673911167266851407475769519805908203125f,-0.058999992519617183717173247714526951313018798828125f,0.217399997806548839207607670687139034271240234375f,0.26169999713003644359332611202262341976165771484375f,-0.19579999482333665383748666499741375446319580078125f,0.1368000029370188830313281869166530668735504150390625f,-0.9979999731183060163175468915142118930816650390625f,1.646999958410859665747238977928645908832550048828125f,-0.599699998188018668088261620141565799713134765625f,0.4339999944567676237738851341418921947479248046875f,-0.774599994876980613156547406106255948543548583984375f,0.270299999427795256679019075818359851837158203125f,0.852299983841181330035396968014538288116455078125f,-0.757999968275428148700711972196586430072784423828125f,0.447100010672211833906430911156348884105682373046875f,0.8351000138163566788307434762828052043914794921875f,-0.084199982228875480672058984055183827877044677734375f,0.4204000016212461332543171010911464691162109375f,0.63529997290372808294023343478329479694366455078125f,-1.0633999601364136911030300325364805757999420166015625f,0.38379999185800528493928140960633754730224609375f,0.3087999581813818394948611967265605926513671875f,-0.981599957460165484235403710044920444488525390625f,0.207599992060661264758891775272786617279052734375f,-0.11949998523294969032804146991111338138580322265625f,0.529399976781010916937475485610775649547576904296875f,-0.0913999977394939211450264338054694235324859619140625f,1.11289997479915570011144154705107212066650390625f,-1.131799982559680728400053340010344982147216796875f,0.6032000079035757522660787799395620822906494140625f,-0.2284999805092819968876938219182193279266357421875f,0.836999962583184764497445939923636615276336669921875f,-0.1633999985814096334024725365452468395233154296875f,0.0015999962031845171850363840349018573760986328125f,-0.958500013887881863183793029747903347015380859375f,-0.326800027561187977909185065072961151599884033203125f,-0.3155000075772405043750268305302597582340240478515625f,-0.9151000376582150153126349323429167270660400390625f,-0.60960002520084355381868590484373271465301513671875f,-0.440999999955296484444033922045491635799407958984375f,-1.203800025534629725143531686626374721527099609375f,0.738600028672814712393801528378389775753021240234375f,0.1359000093102455952731588695314712822437286376953125f,0.2638000326871878797874160227365791797637939453125f,-0.129400009265542070835408594575710594654083251953125f,-0.05210000210404397424923672588192857801914215087890625f,0.07819999538660038496118431794457137584686279296875f,-0.53120002855658565721341801690869033336639404296875f,-0.31900000898540026295080451745889149606227874755859375f,-0.89980003823041965205220549250952899456024169921875f,-0.9541999989181755292833031489863060414791107177734375f,-0.3228999996840953290444531376124359667301177978515625f,-0.9138000285148617507502422085963189601898193359375f,-0.278400027936697380681607683072797954082489013671875f,-0.1776000105381012605931800862890668213367462158203125f,0.1963999752759928441037118318490684032440185546875f,-0.0868000007390976602295040720491670072078704833984375f,-0.0446000021785497580140145146287977695465087890625f,-0.3996000210285188813941203989088535308837890625f,0.066800012809038189498522797293844632804393768310546875f,0.07720000191330911309961493316222913563251495361328125f,0.1734000076711179616495428490452468395233154296875f,-0.792399991250037682988249798654578626155853271484375f,-0.149299994145333680961584832402877509593963623046875f,-0.151700001227855096885832608677446842193603515625f,-0.533200019973516514681932676467113196849822998046875f,-0.254300004799664025600947070415713824331760406494140625f,-0.649500032693147932860711080138571560382843017578125f,0.03300000374019163018601830117404460906982421875f,0.1266000057026743785826283783535473048686981201171875f,-0.1788999803423880763375564129091799259185791015625f,-0.032600008228421462064261504565365612506866455078125f,-0.088600003452599107589549021213315427303314208984375f,0.440300015819072854128535254858434200286865234375f,0.46360001593232169625480310060083866119384765625f,0.29030000438243153126904871896840631961822509765625f,0.77230002332925806740604457445442676544189453125f,-0.645999981552362267933631301275454461574554443359375f,0.2107000204205515814237514859996736049652099609375f,0.2102999654531476636520892498083412647247314453125f,-0.164599975728988656697993064881302416324615478515625f,0.092599997425079294544048025272786617279052734375f,-0.033400047159194201640275423415005207061767578125f,0.06880003268718670739190201857127249240875244140625f,-0.04660000767707828117636381648480892181396484375f,1.0724000219464304706207258277572691440582275390625f,-1.179600013801455293815934055601246654987335205078125f,0.596600026750564893518458120524883270263671875f,-0.2646000177502632055848152958787977695465087890625f,0.025400010785460391815604452858678996562957763671875f,-0.13270000841021545312514717807061970233917236328125f,-0.164800032269955210040279780514538288116455078125f,0.26260001485943806898148977779783308506011962890625f,-0.1143000081896783370893899700604379177093505859375f,-0.90979998279809937145046205841936171054840087890625f,1.1833999713867904279851472892914898693561553955078125f,-0.5292999965667721795625766390003263950347900390625f,-0.5606000477314001528839071397669613361358642578125f,-0.236599977129697602862279381952248513698577880859375f,-0.268100016140937924546960857696831226348876953125f,0.150800018218159692651170189492404460906982421875f,0.1995999947279691877355389806325547397136688232421875f,0.0586000084444882052281400319770909845829010009765625f,-0.0387999784469605923931112556601874530315399169921875f,0.094199980664253446516198664539842866361141204833984375f,-0.019699992972612268005150326644070446491241455078125f,-1.2430999869346610608999981195665895938873291015625f,1.185799979996680963978405998204834759235382080078125f,-0.6804000069856641630394733510911464691162109375f,-0.3763000194251535646117190481163561344146728515625f,0.417200014430284316091501750634051859378814697265625f,-0.22190000725388525637526981881819665431976318359375f,0.13150000816583684581928537227213382720947265625f,0.551800005134939652862158254720270633697509765625f,0.0387999912619587661311015835963189601898193359375f,-0.3349000428080568525501803378574550151824951171875f,-0.520999999657272727660028976970352232456207275390625f,-0.13680001087188742303624167107045650482177734375f,0.147900022482872373075224459171295166015625f,-0.17240000302195568337992881424725055694580078125f,0.09910000231266025139120756648480892181396484375f,-1.0599999815225604749002741300500929355621337890625f,0.620599999302625437991309809149242937564849853515625f,0.296200019392371505233541029156185686588287353515625f,0.21260000826567415632695201566093601286411285400390625f,0.79790004036426598332809589919634163379669189453125f,0.5752000192880633022696201805956661701202392578125f,0.26810000679045919991239088631118647754192352294921875f,0.50310002567768119519087122171185910701751708984375f,0.16000000074505804636970651699812151491641998291015625f,0.16190000021308659228225224069319665431976318359375f,0.3061000044584272217207399080507457256317138671875f,-1.2318000285297632245828935992904007434844970703125f,-0.348000005714595328498717208276502788066864013671875f,-1.151900052106380911709493375383317470550537109375f,0.302000012278556884126601289608515799045562744140625f,0.1318000028997659833773781201671226881444454193115234375f,0.2556000138312579661459267299505881965160369873046875f,-0.70660001803338534642762169823981821537017822265625f,-0.248900004109740280444640347923268564045429229736328125f,-0.251200016039609952400724068866111338138580322265625f,0.867600025141239239445667408290319144725799560546875f,0.2088000083237886439491148848901502788066864013671875f,0.0750000452995305266767900320701301097869873046875f,0.6298000228822233648173778419732116162776947021484375f,0.19040000661313538632413155937683768570423126220703125f,0.422900027921796262120324172428809106349945068359375f,-0.14179999991953362581398323527537286281585693359375f,-0.05349999988079068291568773929611779749393463134765625f,0.26099999958276764999709484982304275035858154296875f,0.374200008007883777594315688475035130977630615234375f,0.335299999278783733558384483330883085727691650390625f,0.877400024032592540379482670687139034271240234375f,-0.515599992448091182239977570134215056896209716796875f,-0.2337999950617551114806502710052882321178913116455078125f,-0.7408000058501953422052110909135080873966217041015625f,-0.236800013703108103158001540577970445156097412109375f,-0.2891000048086047602424741853610612452030181884765625f,-0.9341000198960305311857155174948275089263916015625f,0.284599997708200991297644577571190893650054931640625f,0.120399995623528877075614218483678996562957763671875f,-0.171300003778934950560142169706523418426513671875f,0.294000004068017073421970053459517657756805419921875f,0.0870000015124678671707414423508453182876110076904296875f,0.1976000091969967176197542357840575277805328369140625f,-0.09019999715983872956570621681748889386653900146484375f,0.720799996872246051538013489334844052791595458984375f,-0.3045000075101853553860564716160297393798828125f,-0.156900010085105900969892900320701301097869873046875f,-0.28609999808818098454565870270016603171825408935546875f,0.58450000538676949446426078793592751026153564453125f,-0.181899996674060648871318335295654833316802978515625f,0.7523000068262215567216344425105489790439605712890625f,0.66550002542138120276149493292905390262603759765625f,0.0709999976456165970972733703092671930789947509765625f,-0.181900002112984626734260018565692007541656494140625f,0.0215000002682208535276231486932374536991119384765625f,-0.59299998015910393434069192153401672840118408203125f,-0.380299986016750413142517572850920259952545166015625f,-0.467999989509582547952959430404007434844970703125f,0.69929998777508739493669054354541003704071044921875f,-0.415199987697601358149768202565610408782958984375f,-0.8029999942779539878756622783839702606201171875f,0.23780000687837610229280471685342490673065185546875f,0.3614000092506406414116781888878904283046722412109375f,-1.274500002145767751926541677676141262054443359375f,0.53859997793436065194327966310083866119384765625f,-0.5150999630033974430176613168441690504550933837890625f,-0.7680000035166738570069355773739516735076904296875f,-0.389999993592500704409076206502504646778106689453125f,-0.1641999982848763028897565163788385689258575439453125f,0.272000009223818839387831758358515799045562744140625f,-0.81350001113116732209817882903735153377056121826171875f,-0.32260000424236057181559544915216974914073944091796875f,0.349000021204352695036732256994582712650299072265625f,0.81200001503527163837503621834912337362766265869140625f,0.32059999878108502713303096243180334568023681640625f,-0.0565000001192093037616359652020037174224853515625f,0.41259998937845221433917686226777732372283935546875f,-0.60629999908506848615985518335946835577487945556640625f,-1.01799999070167501713513047434389591217041015625f,0.236199990856647445980343036353588104248046875f,-0.786800000143050848322445744997821748256683349609375f,0.050500059843063382913896930404007434844970703125f,0.91879997367858923240646618069149553775787353515625f,-0.50649997001886415404214858426712453365325927734375f,-0.60049998369812929155386882484890520572662353515625f,-0.5580999970674513921409243266680277884006500244140625f,-0.2878999973744154061705557978712022304534912109375f,0.30850000658631326899694613530300557613372802734375f,-0.7341000061869618065202303114347159862518310546875f,0.0831000009417532936595307546667754650115966796875f,-0.58050001551210872463570922263897955417633056640625f,0.2577000217467551035355199928744696080684661865234375f,1.2921000192403795381324016489088535308837890625f,-0.20580000443458601466772961430251598358154296875f,-0.0030000130534180158292656415142118930816650390625f,0.26900000242888921775374910794198513031005859375f,-0.8507000120013952226827314007095992565155029296875f,0.466700029465556553986971266567707061767578125f,0.29089999828338619636269868351519107818603515625f,0.28509999623298654114478267729282379150390625f,0.6077000365734104292414485826157033443450927734375f,0.667700022566318551753283827565610408782958984375f,-0.555500032424927070451303734444081783294677734375f,-0.8401000457048422020989164593629539012908935546875f,0.68650002336502069510970613919198513031005859375f,0.41329995615482317816713475622236728668212890625f,-0.8003000193953511143263312987983226776123046875f,0.1667999825596808705086004920303821563720703125f,-0.566699992585181888671286287717521190643310546875f,-0.70270000736713367217589620850048959255218505859375f,-0.3291999712765214525234114262275397777557373046875f,-1.6294999818801887414565499057061970233917236328125f,-0.18790002635717417689420472015626728534698486328125f,0.1415999695301053407092695124447345733642578125f,1.74119998519420704496951657347381114959716796875f,-0.0289999747872347057864317321218550205230712890625f,0.1522000184535983180467155762016773223876953125f,0.71720003931522313678215141408145427703857421875f,0.7532000248908996553609540569595992565155029296875f,0.3042999651253222026525691035203635692596435546875f,-1.1181999903202051172002029488794505596160888671875f,-0.52919999765157665905235262471251189708709716796875f,-0.17220000712871552650540252216160297393798828125f,-0.33989998886585226500756107270717620849609375f,-0.4855000085830685208065915503539144992828369140625f,-1.65930004901886007928624167107045650482177734375f,0.84090003166198812323273159563541412353515625f,0.2165000075697900427940112422220408916473388671875f,-0.6068000025272368702644598670303821563720703125f,-0.15669997708797467339536524377763271331787109375f,0.8731000188231465841681711026467382907867431640625f,0.9153999840378759245140827260911464691162109375f,-0.6509999847188592259072947854292578995227813720703125f,-0.396500024408102280659704774734564125537872314453125f,-0.23740000741779809789733235447783954441547393798828125f,-0.0167000020474195043362897195038385689258575439453125f,-0.952499974966048768010296043939888477325439453125f,-0.36330000845789900409954498172737658023834228515625f,-0.90869999122917644029939765459857881069183349609375f,-0.8070000274032356202269511413760483264923095703125f,-0.346500008016824789791598959709517657756805419921875f,-0.151000001370906744568856083787977695465087890625f,-1.0145000158548356239407439716160297393798828125f,0.30079999728202810871380279422737658023834228515625f,-0.3902999898910521903161452428321354091167449951171875f,0.6105000193119050067025455064140260219573974609375f,-0.7110000168681143151161450077779591083526611328125f,-0.21490000726878644154993480697157792747020721435546875f,0.538999996542930404075377737171947956085205078125f,-0.1862000142514705913754369248636066913604736328125f,-0.4020000030025838722025355309597216546535491943359375f,0.8014999938607214602370731881819665431976318359375f,-0.572299978923797691976460555451922118663787841796875f,0.1239000008687376475080554882879368960857391357421875f,0.49949998247623472735767791164107620716094970703125f,-0.86550000247359282212755715590901672840118408203125f,0.557899960605800604440673851058818399906158447265625f,-0.076499989986420047216597595252096652984619140625f,0.88800000903010367636625232989899814128875732421875f,-0.39369996504038606843778325128369033336639404296875f,-1.105000008344649842229046043939888477325439453125f,0.60700000900030115502659100457094609737396240234375f,-0.39340000618100123830345182796008884906768798828125f,0.47499999701976758359478480997495353221893310546875f,-0.83349997529387476191686801030300557613372802734375f,0.080400011642277002810175190461450256407260894775390625f,0.76649999177455896415267488919198513031005859375f,-0.31169998483657845866900970577262341976165771484375f,0.2868999924644828336539603697019629180431365966796875f,0.941000011801719438153668306767940521240234375f,0.84520002182126052758803780307061970233917236328125f,1.0236000117227430639132990108919329941272735595703125f,-0.695000004172324992168796597979962825775146484375f,0.33169999750256540238524394226260483264923095703125f,0.28459999584555628615589739638380706310272216796875f,-0.654499982774257915707494248636066913604736328125f,0.1069999893307684857290951185859739780426025390625f,0.543700010526180221859249286353588104248046875f,-1.592800026845932137575800879858434200286865234375f,-0.21580000562667844832276387023739516735076904296875f,-1.28519998888969411154903355054557323455810546875f,0.80970003275871249570627696812152862548828125f,0.0099000028729445688213672838173806667327880859375f,-0.22510002365112313782447017729282379150390625f,2.4187000024795537456157035194337368011474609375f,-0.062900010263920336228693486191332340240478515625f,-0.931300005865097091373172588646411895751953125f,-0.35089998576641079353066743351519107818603515625f,1.0553000381708148580628403578884899616241455078125f,0.54709999241828910498952609486877918243408203125f,-0.845900009310245337701417156495153903961181640625f,0.23910002764463467173072785953991115093231201171875f,0.59469998418092728798001189716160297393798828125f,-0.513399991130828681207276531495153903961181640625f,0.02180000282525984545145547599531710147857666015625f,0.75269999939501275321163120679557323455810546875f,0.1841999731391670280800099135376513004302978515625f,-0.04979998540580243826525475014932453632354736328125f,-0.63329998849034296881654881872236728668212890625f,0.416700020152330097289450350217521190643310546875f,-0.271300020617246673282352276146411895751953125f,-0.12340000156164165900918305851519107818603515625f,1.452400004363060048717670724727213382720947265625f,-0.16620001708269160900499628041870892047882080078125f,-0.837499998509883880615234375f,-0.273099996471405148668054607696831226348876953125f,0.89910000648498478170722592039965093135833740234375f,0.544399987584352373914953204803168773651123046875f,-0.7982000120759007444348753779195249080657958984375f,0.253000029742718179903704367461614310741424560546875f,1.390500011265277890970537555404007434844970703125f,-0.121500029325485314757315791212022304534912109375f,-0.901500030219555270605269470252096652984619140625f,-0.543899995243549483348033390939235687255859375f,0.136799977493286206708944519050419330596923828125f,0.73840002659559278441747665056027472019195556640625f,0.4592999867320060047859442420303821563720703125f,-0.932400001382827525731045170687139034271240234375f,-0.30999999433755842659365953295491635799407958984375f,0.12140000731348976614754064939916133880615234375f,0.60069997143298425879009982963907532393932342529296875f,-0.1110000150799752560715205618180334568023681640625f,-1.026300013613700823356111868633888661861419677734375f,-0.0804000088110565780397109847399406135082244873046875f,0.9349999725818634033203125f,0.68530001447796795588374152430333197116851806640625f,0.7993999713420871788827071213745512068271636962890625f,0.23600005054473882637466886080801486968994140625f,1.3084000055849553945108709740452468395233154296875f,-0.23379996656328483339848389732651412487030029296875f,-0.692500028610229634296047152020037174224853515625f,0.35379999513626092522144972463138401508331298828125f,0.516099991568923055496043161838315427303314208984375f,-0.7145000206232072770262675476260483264923095703125f,-0.475599999451637334146880675689317286014556884765625f,-0.306699992957711220498140392010100185871124267578125f,-0.4664999855160711916823856881819665431976318359375f,-0.381899991160631113729095886810682713985443115234375f,-0.351899984604120419362516258843243122100830078125f,-0.493500004127621849647766794078052043914794921875f,-0.06630000661015511553841861314140260219573974609375f,-0.5708999851703644434763873505289666354656219482421875f,0.82300000098347680932420189492404460906982421875f,0.25340000707507126520567908301018178462982177734375f,0.4495999815031888058314279987826012074947357177734375f,0.3015000308156015051963549922220408916473388671875f,0.739800001722574318563374617951922118663787841796875f,-0.12169998904615653145810938440263271331787109375f,-0.5574999955296515707914295489899814128875732421875f,0.323799981129169633931041971663944423198699951171875f,0.4674999945610760487824109077337197959423065185546875f,-0.73100001880526566111484498833306133747100830078125f,-0.4449999995529652085934912975062616169452667236328125f,-0.315799988862872194061992558999918401241302490234375f,0.1530000234842301409798892564140260219573974609375f,-0.64260000472664824400226279976777732372283935546875f,-0.80399998860806232148235039858263917267322540283203125f,-0.5665000200867655877345896442420780658721923828125f,0.429699978071451216266041228664107620716094970703125f,0.2172999979227780986690277131856419146060943603515625f,0.0714999896883963259597294381819665431976318359375f,-0.690699988979101231478807676467113196849822998046875f,-0.1110999957412481808916027148370631039142608642578125f,0.2059999994337555762058400432579219341278076171875f,0.6636000026702877363504740060307085514068603515625f,-0.245100012177228876453227712772786617279052734375f,-0.2804999728202819397893108543939888477325439453125f,0.464300015938281962490918886032886803150177001953125f,-0.165499998599290876200029742904007434844970703125f,0.1075999923586845596901184762828052043914794921875f,-0.361600019598007360599467574502341449260711669921875f,0.503500009734183517906558336107991635799407958984375f,-0.4600000238791108753133585196337662637233734130859375f,-0.6220000194683670269313324752147309482097625732421875f,0.954100013637543042932520620524883270263671875f,0.7589000366687770338103291578590869903564453125f,-1.057700030612945596431018202565610408782958984375f,0.0441999989926813441343256272375583648681640625f,1.193299971801042858032815274782478809356689453125f,-0.345899998432397381975533789955079555511474609375f,1.5024000217974187165737021132372319698333740234375f,-0.4989000211715701738057759939692914485931396484375f,-0.65330002022981670251056129927746951580047607421875f,-1.06489999780654898131615482270717620849609375f,-1.0746999731540682887498405762016773223876953125f,0.841900005912780358130476088263094425201416015625f,-0.723599988402426230749142632703296840190887451171875f,0.669899996018409549236594102694652974605560302734375f,0.5629000019937751009280191283323802053928375244140625f,1.08209999629259101538991671986877918243408203125f,-0.19260002514123897299214149825274944305419921875f,0.1228000187993050218437929288484156131744384765625f,-1.436099986577033860157825984060764312744140625f,0.5577000347852703043827204965054988861083984375f,0.38810000213384654443871113471686840057373046875f,-1.505300006282329405848940950818359851837158203125f,-0.090399939632415993173708557151257991790771484375f,0.8068000183224679489057962200604379177093505859375f,-0.122499980032444000244140625f,0.335499974608421069888208876363933086395263671875f,0.51249999880790664263940925593487918376922607421875f,0.46529999585151671226412872783839702606201171875f,0.5145000009536744300930877216160297393798828125f,0.6961000417113307747740691411308944225311279296875f,-0.7390999873742458703418378718197345733642578125f,0.38990004217475604519904663902707397937774658203125f,-0.371100034521520516506143394508399069309234619140625f,0.561100001478195054005482234060764312744140625f,0.423899997963011088142337712270091287791728973388671875f,-0.42600000271201121648800835828296840190887451171875f,0.0564000107407569561956961479154415428638458251953125f,0.1282000016078352899739911663345992565155029296875f,0.59650000950694082035852261469699442386627197265625f,-0.0331000027298926635666020956705324351787567138671875f,0.1794999972283839451137055220897309482097625732421875f,0.44550000025704494799327903820085339248180389404296875f,-0.554100006205588524466065791784785687923431396484375f,-0.15750000359490512469307077481062151491641998291015625f,0.67650001740455678600483224727213382720947265625f,-0.1421999665975572924025982501916587352752685546875f,-0.6017000039845701309104697429575026035308837890625f,0.414499999910592720198110328055918216705322265625f,0.50829999489784238875245137023739516735076904296875f,-0.1618999898567796691395415109582245349884033203125f,-0.25349999034404735454018009477294981479644775390625f,-0.78090000498890876912128078402020037174224853515625f,-0.8742999972745775227878084479016251862049102783203125f,-0.30950001049041730993849341757595539093017578125f,-0.0772000065326690076972226961515843868255615234375f,0.78229997937083251002832184894941747188568115234375f,-0.2059999970495700605255251502967439591884613037109375f,0.688699982474744398786015153746120631694793701171875f,0.295199993658065817925262308563105762004852294921875f,-0.9619999988675118629544158466160297393798828125f,-0.27250000745058056139669133699499070644378662109375f,-0.7841999791368843286676337811513803899288177490234375f,0.5519999800920487587063689716160297393798828125f,0.7092000055491922694272943772375583648681640625f,0.87219997065812382430749494233168661594390869140625f,0.0574999803304674372839144780300557613372802734375f,0.520300018054246748988589388318359851837158203125f,0.94409997727870964556728949901298619806766510009765625f,-0.5834999930262565470684421597979962825775146484375f,0.376199970293045193869829745381139218807220458984375f,-0.0765000017583370439666623497032560408115386962890625f,-1.26100001180171972237076261080801486968994140625f,0.41860000043511380596328308456577360630035400390625f,-0.5755999840661882860359810365480370819568634033203125f,1.032500012330711314234577002935111522674560546875f,0.226000015974044732303127602790482342243194580078125f,0.5526999764844777729422276024706661701202392578125f,-0.6040000215768817071193552692420780658721923828125f,0.352799991232156440190692592295818030834197998046875f,-1.0841999645709989863462396897375583648681640625f,0.273000008583068165535223670303821563720703125f,0.249599989140033784451588871888816356658935546875f,-1.1539000219352544274187266637454740703105926513671875f,-0.6544999788999554812818360005621798336505889892578125f,0.35270000665932899064358707619248889386653900146484375f,-0.40609999859109535691459313966333866119384765625f,0.269899978975206789044705146807245910167694091796875f,-0.2320999983228737573170974428649060428142547607421875f,1.40780004293918636903981678187847137451171875f,0.72669997828006671625189483165740966796875f,-1.268100067400933284034181269817054271697998046875f,0.414300009009242131696737487800419330596923828125f,0.3995999990493057651974595501087605953216552734375f,-0.16100000472366815529312589205801486968994140625f,0.101600000077485930205511976964771747589111328125f,-0.1646000201344488544918931438587605953216552734375f,-0.574200010168552044120815480710007250308990478515625f,-0.254300006997585370527303894050419330596923828125f,-0.2092000018239019709653803147375583648681640625f,-0.17940000144243217761186315328814089298248291015625f,-1.0499000063002110749721396132372319698333740234375f,0.7330999917030336376910781837068498134613037109375f,0.51410000082254381226221084943972527980804443359375f,1.1134000219762327787975664250552654266357421875f,-0.217299985927343897174068843014538288116455078125f,0.1601000130712986191383606637828052043914794921875f,-0.01999996602535247802734375f,2.15749996185302705953290569595992565155029296875f,-0.1605000127553939393010296043939888477325439453125f,0.573199991735815927995645324699580669403076171875f,0.0546999848514797548659771564416587352752685546875f,0.0407000060409306030351217486895620822906494140625f,-0.8031999846577644319722821819595992565155029296875f,0.694599989736079947988400817848742008209228515625f,0.622999999046325303453386368346400558948516845703125f,0.1097000033289194931285237544216215610504150390625f,0.48430000669956196901466682902537286281585693359375f,-0.4809000268936161059940559425740502774715423583984375f,-0.81279999972581862266451935283839702606201171875f,1.221400008654594415702376863919198513031005859375f,0.29199999469518633787856742856092751026153564453125f,-0.6789999805241819785806001164019107818603515625f,-0.3684000177666539865839467893238179385662078857421875f,0.4744999794960023109524627216160297393798828125f,0.4419999864995478588980404310859739780426025390625f,0.692899963213503777836876906803809106349945068359375f,0.39599999138712871360468170678359456360340118408203125f,-0.00239999298229809365778919527656398713588714599609375f,0.87979999240934836368666083217249251902103424072265625f,0.6039999835975464748116792179644107818603515625f,0.1441999940752982656277936257538385689258575439453125f,0.3001999942727386905261255378718487918376922607421875f,0.61650004482269338268451974727213382720947265625f,-0.3195000101923941571158138685859739780426025390625f,-0.88310000366121510495531765627674758434295654296875f,-0.0979999942034481819064239971339702606201171875f,0.076700000929832423679499697755090892314910888671875f,-0.33659999795407047340489725684165023267269134521484375f,0.67899999251961684620937376166693866252899169921875f,-0.3022000007212162220326945316628552973270416259765625f,0.0757999973565339768555304544861428439617156982421875f,0.424999993443488932598484097979962825775146484375f,-0.122299996507167829662421354441903531551361083984375f,0.25959999729841942706087820624816231429576873779296875f,0.09950001049041734546563020558096468448638916015625f,0.76959998392462747318631954840384423732757568359375f,0.5333000021994112049839031897136010229587554931640625f,-1.0340000089108940528603852726519107818603515625f,-0.279099994415044960760496906004846096038818359375f,-0.810800002497434224579819783684797585010528564453125f,0.213500008821487341492684208787977695465087890625f,1.09629997427463532488900455064140260219573974609375f,-0.318500001408159594262770042405463755130767822265625f,-0.47849998877942567787613370455801486968994140625f,-0.069300004315376195762610223027877509593963623046875f,-0.43689998233169335950520917322137393057346343994140625f,-0.2259999949634075022686374722979962825775146484375f,0.7377999758839608457350323078571818768978118896484375f,0.336399992412328707747093403668259270489215850830078125f,0.547000022485852444020792972878552973270416259765625f,0.07280000561177735196594085209653712809085845947265625f,-0.0603999994233250792063927292474545538425445556640625f,0.2125000151991844887788829510100185871124267578125f,0.907399995571374962111121931229718029499053955078125f,0.32410000018924467468650618684478104114532470703125f,0.9975000111013656578506925143301486968994140625f,-0.171300004374980829879859811626374721527099609375f,-1.233399980066716583593233735882677137851715087890625f,-0.084499991118908379661434082663618028163909912109375f,0.5906999932259318075722376306657679378986358642578125f,0.31820000112354751564680555020458996295928955078125f,-0.87800001484155654196683826739899814128875732421875f,-0.43860001742243781563956872560083866119384765625f,-0.15869999055862393788629560731351375579833984375f,-2.45669994370937416761080385185778141021728515625f,0.22229998965263320087615284137427806854248046875f,-0.60999999940395355224609375f,0.4306000066041946183759137056767940521240234375f,0.18839999917745586799355805851519107818603515625f,-0.5932000228047371592765557579696178436279296875f,-0.64189998415708515722144511528313159942626953125f,0.8465000200867649482461274601519107818603515625f,1.0567999757051467213386786170303821563720703125f,-0.20360000743865924732745043002068996429443359375f,-0.1018000010371213903681564261205494403839111328125f,-0.62410001572370532585409819148480892181396484375f,-0.46699998694658262365919654257595539093017578125f,-0.3932000287652019920869861380197107791900634765625f,-1.2672000205397608851853874512016773223876953125f,0.1949999848008161507095792330801486968994140625f,-0.0113999916672700152275865548290312290191650390625f,1.169600020730495515408620121888816356658935546875f,0.664299973917007235968412715010344982147216796875f,-0.54910001274347308708456694148480892181396484375f,0.00090000662803646491738618351519107818603515625f,-0.609300011169910504804647644050419330596923828125f,0.13810000869035743420454309671185910701751708984375f,0.41460002937316886573171359486877918243408203125f,1.96769994895458211203731480054557323455810546875f,-0.37050000470876653935192734934389591217041015625f,0.37489998808354130233055911958217620849609375f,1.16979998298436438375347279361449182033538818359375f,-0.472000021107495282279842285788618028163909912109375f,-1.00259999027252177938862587325274944305419921875f,-0.86459995725154925594324595294892787933349609375f,0.810400013244152006564036128111183643341064453125f,0.0120000182390214149563689716160297393798828125f,2.233500001072883378583355806767940521240234375f,-0.13850000941753393135513761080801486968994140625f,-0.354899967594444643026463381829671561717987060546875f,0.44539997669905451171956656253314577043056488037109375f,0.1394999914169312571488035246147774159908294677734375f,0.3968999825552106397452689634519629180431365966796875f,0.7902999734625222316708459402434527873992919921875f,0.344500002592802001544214363093487918376922607421875f,-0.6778000039726495007386120050796307623386383056640625f,0.0100999971643089547779226222701254300773143768310546875f,-0.160500015139579232936739572323858737945556640625f,-1.1993999829649926169850004953332245349884033203125f,0.4868999671697620268417949773720465600490570068359375f,0.20250000059604644775390625f,0.41219999803900719825833220966160297393798828125f,0.332499994896352291107177734375f,-0.667499996721744537353515625f,0.0620999983787537956914093228988349437713623046875f,0.362900007618963460487293559708632528781890869140625f,-0.648499966502189550965340458787977695465087890625f,-0.34959999554753284911612354335375130176544189453125f,-0.71609996672123710226287585101090371608734130859375f,0.98100001806020742378677823580801486968994140625f,-0.25949999201297746509453645558096468448638916015625f,0.6191999957516787222999710138537921011447906494140625f,0.952999982059001382594942697323858737945556640625f,0.34779999704360964329907801584340631961822509765625f,0.89539999662935709778821546933613717555999755859375f,-0.1499999892711638693754139239899814128875732421875f,-0.06210000687241556960316302138380706310272216796875f,-0.85119999197423446002375158059294335544109344482421875f,-0.1785000177621842709640986868180334568023681640625f,-0.277200008022785215899830291164107620716094970703125f,0.073699992495775301648563981871120631694793701171875f,0.168000000238418323306177626363933086395263671875f,0.8390999765336513149804886779747903347015380859375f,-0.5591999951183794781428559872438199818134307861328125f,0.335000039078295230865478515625f,0.388299998772144494552094329264946281909942626953125f,-0.3727999756604436498719223891384899616241455078125f,-0.315500023484229785708521376363933086395263671875f,0.011600003951787840605902601964771747589111328125f,0.7076999717906120590527052627294324338436126708984375f,0.16050001811981218224900658242404460906982421875f,1.137899998939037260470286128111183643341064453125f,-0.359499995850026454746739545953460037708282470703125f,-0.3794999946951864711053303835797123610973358154296875f,-1.0218000186204907464571078890003263950347900390625f,0.03670000212192547905942774377763271331787109375f,-1.58559997024536158960472675971686840057373046875f,0.021000012993813044204216566868126392364501953125f,0.2696000257968904634253703989088535308837890625f,1.61200001287460281673702411353588104248046875f,0.194200011956692009107428020797669887542724609375f,0.077499994914978831417329274700023233890533447265625f,-0.4485999811753631494326555184670723974704742431640625f,0.278600000919401236121331066897255368530750274658203125f,0.9293000169813634414595071575604379177093505859375f,0.1102000014066693012182440725155174732208251953125f,-0.23459999718666058043936573085375130176544189453125f,-0.52389996991157516958992346189916133880615234375f,-0.379399995183945293319993652403354644775390625f,-0.4414000280261038966500564129091799259185791015625f,0.39639999744147047522346838377416133880615234375f,0.72019996851235656976086829672567546367645263671875f,-0.209399997456371611548320288420654833316802978515625f,0.723400025254488099335503648035228252410888671875f,1.190899985915422121252049691975116729736328125f,-0.311100005203485352467396296560764312744140625f,-0.039000037074089277666644193232059478759765625f,-2.355600004220008258926100097596645355224609375f,0.018600008332729345283951261080801486968994140625f,-0.00219999788999558631985564716160297393798828125f,0.668799995434284255679813213646411895751953125f,-0.60520001899003972312129917554557323455810546875f,-0.4658000047326087411647677072323858737945556640625f,-0.6549999769032002205904063885100185871124267578125f,-0.09960000657439227467193632037378847599029541015625f,-0.01469999345690009562304112478159368038177490234375f,0.460799996015429425000320406979881227016448974609375f,-0.54620002371370801608208012112299911677837371826171875f,-0.21720003871917725746243377216160297393798828125f,-1.14359996035098987476885668002068996429443359375f,-0.75280002684593227968434803187847137451171875f,-0.31799999457597749596970970742404460906982421875f,1.325399968838691933115114807151257991790771484375f,-0.15139999315738617013948896783404052257537841796875f,-0.081800000143051221357382019050419330596923828125f,0.568399983084201920746636460535228252410888671875f,0.3220000042319295374682042165659368038177490234375f,0.337000002741813631246259319595992565155029296875f,-0.4585999897807837566432453968445770442485809326171875f,-0.046000034451484594910652958787977695465087890625f,-0.842899981951713783701052307151257991790771484375f,0.2385999790892008309839411595021374523639678955078125f,-0.463500005245208512860699556767940521240234375f,0.8700000031292436375451870844699442386627197265625f,-0.512700003679096294462169680627994239330291748046875f,-0.42049999072030164182223188618081621825695037841796875f,-0.1590999954394995696826953235358814708888530731201171875f,-0.03949999780580404351582046729163266718387603759765625f,-0.38150000607967360366501452517695724964141845703125f,-0.2028999944686891154788099811412394046783447265625f,-0.6481000007927415484942912371479906141757965087890625f,0.9694999842643738929837127216160297393798828125f,-0.24890000373721132831406066543422639369964599609375f,0.5412999651104215548258480339427478611469268798828125f,-0.0229999936446549924085047678090631961822509765625f,0.2341999907970429006809354177676141262054443359375f,-0.36779998434036986765249821473844349384307861328125f,-0.1344999966323374707144466810859739780426025390625f,0.3858999953031538865388938575051724910736083984375f,-0.66529999849647258880480649168021045625209808349609375f,0.01749999105930299947431194595992565155029296875f,-1.2381999946415422897416647174395620822906494140625f,0.42550000720471103932140977121889591217041015625f,0.792000008404254884908368694595992565155029296875f,0.1506000054121017228681012056767940521240234375f,0.0163999994903802903678524671704508364200592041015625f,0.47949999246001251407278687111102044582366943359375f,-0.29079998804330831063680307124741375446319580078125f,0.4730999867483973719828327375580556690692901611328125f,0.7250000109896064248715674693812616169452667236328125f,0.062500001676380645410091574376565404236316680908203125f,0.048399996197223682514731990522705018520355224609375f,1.155000019669532917987453402020037174224853515625f,-0.8207999642014502938991427072323858737945556640625f,0.5134000035732981803704433332313783466815948486328125f,0.4269999899268146492659070645458996295928955078125f,0.715399988806247666417448272113688290119171142578125f,0.03900000287592408909631558344699442386627197265625f,-0.3649999889731407876070079510100185871124267578125f,0.4273999852895737916469443007372319698333740234375f,-0.10539998999834077064718940164311788976192474365234375f,-0.494000009357929315001456416212022304534912109375f,-0.7554000192046164130488250521011650562286376953125f,0.1071000317573549409644328989088535308837890625f,2.03850000107288309436626150272786617279052734375f,-0.3343000254750254640612183720804750919342041015625f,1.1689000039048493828630626012454740703105926513671875f,0.0830000021755694650238410758902318775653839111328125f,-0.51119999923855041767950524445041082799434661865234375f,0.0281999990373851261438176152296364307403564453125f,-0.41159997698068639948587588150985538959503173828125f,0.5780000038146966279128946553100831806659698486328125f,1.33519998590946187277950230054557323455810546875f,0.5535999782323841600373270921409130096435546875f,-0.7302000023007391149576505995355546474456787109375f,0.79840000603199001716347993351519107818603515625f,0.90229998130798350075565394945442676544189453125f,-0.8225000059604639091048738919198513031005859375f,-1.4577000103473665149067528545856475830078125f,0.29279998869895962343434803187847137451171875f,0.1277999827384945774610969237983226776123046875f,0.1295999901831148548581040813587605953216552734375f,1.3793999749183658565243604243732988834381103515625f,0.48080001575946784697634939220733940601348876953125f,0.64720001979470254127591033466160297393798828125f,-1.2529000323176386899604040081612765789031982421875f,-0.448700027513504462461924049421213567256927490234375f,0.0508000014357268270259737619198858737945556640625f,0.445699987433105793144250128534622490406036376953125f,-0.19710000353828061037120278342626988887786865234375f,0.265299996745586241786440950818359851837158203125f,1.904299967956543326863538823090493679046630859375f,0.0403000080704689622734804288484156131744384765625f,0.2534000188469889280895586125552654266357421875f,1.5664000190854068250700947828590869903564453125f,-0.5106000358104711978057821397669613361358642578125f,0.418700017827749348953148000873625278472900390625f,0.77439999563097927648414042778313159942626953125f,-0.1220000011026858288687435560859739780426025390625f,-0.01740000555515308633403037674725055694580078125f,-1.53570003323554971075282082892954349517822265625f,-0.14910000678300860954550444148480892181396484375f,0.878500020407140258527078913175500929355621337890625f,-0.572500037252903215545529747032560408115386962890625f,-0.6005000278800729507366895632003434002399444580078125f,-0.9382000141620636668449151329696178436279296875f,-0.90970000928938343776053443434648215770721435546875f,0.348000021696091010880991234444081783294677734375f,0.94919999541640276419229849125258624553680419921875f,-0.35420000521391603598431174759753048419952392578125f,-0.19599999958276737022089264428359456360340118408203125f,-0.35809998535141362996370162363746203482151031494140625f,-0.75979999136626708011732489467249251902103424072265625f,-0.776999987170099881694795840303413569927215576171875f,-0.02820000008046619210944072619895450770854949951171875f,-0.0733999982088804348023813872714526951313018798828125f,-0.0344999818801881019680877216160297393798828125f,-0.21479998980164527466740764793939888477325439453125f,-0.9148999703511597747507266831235028803348541259765625f,0.482499998211860514629734097979962825775146484375f,0.0443000081896780528722956660203635692596435546875f,-0.5622999921485780649987873403006233274936676025390625f,0.911000010609626542645855806767940521240234375f,0.48429998494386694574131979607045650482177734375f,0.988399993701278845037450082600116729736328125f,-0.73449997830390945097178700962103903293609619140625f,0.87099998408556000839553234982304275035858154296875f,-0.4316999718353156101358081286889500916004180908203125f,0.11600000390410425410436801030300557613372802734375f,-0.9449000198602675038017650877009145915508270263671875f,-0.16440000911653041004711894856882281601428985595703125f,0.218500002209097043959218353847973048686981201171875f,0.16009999690353871049097733703092671930789947509765625f,-0.07109999676570299964595278652268461883068084716796875f,-0.2599999809265138850378207280300557613372802734375f,0.973999987453222360045401728712022304534912109375f,-0.52659997403770708590542426463798619806766510009765625f,0.473500035941601282729607191868126392364501953125f,0.60980000798106193116154827293939888477325439453125f,-0.34330000059753640329063273384235799312591552734375f,-0.1544999999105929333609310560859739780426025390625f,0.28829999802708616840618560672737658023834228515625f,-0.41190000099539736400089395829127170145511627197265625f,0.216999990820884391951040015555918216705322265625f,-0.8635000167191027031776684452779591083526611328125f,0.3057999966114757217638953079585917294025421142578125f,0.1400000101327896206981904470012523233890533447265625f,-0.6800000182166694440155652046087197959423065185546875f,-0.417000001810491072173903148723184131085872650146484375f,-0.406000041007995804420716012828052043914794921875f,-0.76050003510713626297956579946912825107574462890625f,-0.3215000063031911992084133089520037174224853515625f,0.55460000836253176004220222239382565021514892578125f,0.4544000153005123276983567848219536244869232177734375f,1.1567999904572960900850375765003263950347900390625f,-0.0762000638723367984539436292834579944610595703125f,-0.73120001320838934333323777536861598491668701171875f,-0.40279997349977492149264435283839702606201171875f,1.672999980568885547427271376363933086395263671875f,-0.01160000320672960327783584943972527980804443359375f,-1.3537000294506551512085934518836438655853271484375f,-0.3637999832153315793448200565762817859649658203125f,0.67240001896619805421551063773222267627716064453125f,0.67219998477697373573391814716160297393798828125f,-0.792799992871284331386050325818359851837158203125f,0.12020000587701762384540415951050817966461181640625f,0.64110001615583911416251794435083866119384765625f,0.9993000009626147317476352327503263950347900390625f,-0.28210001290738606627428453066386282444000244140625f,0.33010003527402886902564205229282379150390625f,0.1537999805331224933979683555662631988525390625f,0.5964000250458720842061666189692914485931396484375f,0.99469999572932721321194549091160297393798828125f,-0.0914000097721814341866775066591799259185791015625f,0.20680000900924166984395924373529851436614990234375f,1.03980001595318327645145473070442676544189453125f,0.3233999898642299086759521742351353168487548828125f,0.17920001262724394308634146000258624553680419921875f,0.187900008438527610366008957498706877231597900390625f,1.574899993336200676452563129714690148830413818359375f,0.2889000120073557109634521111729554831981658935546875f,0.6878999847084283914000479853712022304534912109375f,-1.55129998753666864530487146112136542797088623046875f,-0.1548999905049799341583138811984099447727203369140625f,0.850700026679039211785493534989655017852783203125f,-0.264199997055531099476866074837744235992431640625f,-0.072400001680851033825092599727213382720947265625f,0.643900008989870542563949129544198513031005859375f,1.13809998004287482586960322805680334568023681640625f,-0.388300007377564870836295085609890520572662353515625f,1.32320002667903935389404068700969219207763671875f,0.8212000221490853846262325532734394073486328125f,-1.0538000359654429161082589416764676570892333984375f,-0.95800003242492692834275658242404460906982421875f,-0.1925999995857476942973107725265435874462127685546875f,-0.360999998912215236401834772550500929355621337890625f,0.51190000017583370262030939557007513940334320068359375f,0.0440000058189033982358751018182374536991119384765625f,0.10250001907348593732649533194489777088165283203125f,-0.5150000175833699955774136469699442386627197265625f,-0.6770999720409516253738502200576476752758026123046875f,0.3044999946951865155142513685859739780426025390625f,0.964199984389543462981464472250081598758697509765625f,0.0120999910399319787757121957838535308837890625f,0.12299999189376809027862691436894237995147705078125f,0.2997000028073788513438557856716215610504150390625f,0.9083999998107550322146153121138922870159149169921875f,-0.6514999879002569826980106881819665431976318359375f,-0.512199993568658751286193364649079740047454833984375f,-0.381899984864890740254850243218243122100830078125f,-0.110499982431530696658228407613933086395263671875f,0.31000000149011608385762883699499070644378662109375f,-0.5786999951779840944965371818398125469684600830078125f,-1.028000029146671323587725055404007434844970703125f,0.21949997800588594287773958058096468448638916015625f,-0.394200012366473284686207989579997956752777099609375f,-1.0079999831765888984591583721339702606201171875f,-0.1731000045180319801829682546667754650115966796875f,-0.7553999786362053381338910185149870812892913818359375f,-1.007499998584389544475925504229962825775146484375f,0.02919999362826342093057974125258624553680419921875f,-0.8530999905481932987783011412830092012882232666015625f,-0.5269999908208846672863501225947402417659759521484375f,0.8983999937757849441055668648914434015750885009765625f,-0.4667999878868460683634111774154007434844970703125f,-0.298000010594725583956687842146493494510650634765625f,-1.011799983677267977810743104782886803150177001953125f,-0.2057999962761998258287121643661521375179290771484375f,-0.5340000069737431687144635361619293689727783203125f,-0.33150000131130230585085882921703159809112548828125f,-0.56140000898987019173347334799473173916339874267578125f,0.0260000134035944796551120816729962825775146484375f,0.347199996250867837233045065659098327159881591796875f,-0.58179998799860477287637650078977458178997039794921875f,0.3975000226497655830826261080801486968994140625f,-0.17999997809529322267962925252504646778106689453125f,-0.8829000033721323692503801794373430311679840087890625f,0.908500031471252356141121708787977695465087890625f,-0.35520000297129161737075264682061970233917236328125f,1.06030001701116560752780060283839702606201171875f,-0.055600039982795834703210857696831226348876953125f,-1.02280004025697746072864902089349925518035888671875f,0.129199984836578352087599341757595539093017578125f,-0.17569998912811257696375832892954349517822265625f,-0.48990001240968705786826831172220408916473388671875f,-1.523999958693980971702330862171947956085205078125f,0.498000013947486053211832768283784389495849609375f,0.4379999847412108238131622783839702606201171875f,1.13819999508857705450282082892954349517822265625f,-1.01370005433559384755426435731351375579833984375f,-0.020899991130828965424370835535228252410888671875f,-0.7947999965816734402324073016643524169921875f,-1.0990999572366479952734152902849018573760986328125f,0.29349999503791313060219181352294981479644775390625f,1.68990000495910663858012412674725055694580078125f,-0.55679998285770437860264792107045650482177734375f,-0.355399985527991901790301199071109294891357421875f,0.499500011682510347554853069595992565155029296875f,-0.23760002663135537659400142729282379150390625f,-0.21940000948905957756096540833823382854461669921875f,0.706400008058547967948470613919198513031005859375f,-0.29600002312660222969498136080801486968994140625f,-0.73920002954006225337479918380267918109893798828125f,0.452200002714246540591602752101607620716094970703125f,-0.7552000044986606486219216094468720257282257080078125f,-0.301400011746585538841003426568931899964809417724609375f,0.24530003399848965273122303187847137451171875f,0.74170000867843555170111358165740966796875f,0.7249000022768967710362630896270275115966796875f,0.605700005500763527521712603629566729068756103515625f,-0.1824000341281293291473275530734099447727203369140625f,-1.0292000157937402082808375780587084591388702392578125f,0.011199998586624904106656686053611338138580322265625f,0.5315000079050660186652521588257513940334320068359375f,-0.481100019918382304506820901224273256957530975341796875f,0.7684000122159719836645308532752096652984619140625f,-1.44070001997351671008118501049466431140899658203125f,-0.0545000106394295524836479671648703515529632568359375f,-0.006099994176626211128677823580801486968994140625f,0.4548999927401542464622252737171947956085205078125f,-0.610100020372867479778733468265272676944732666015625f,0.17680001742839834832921042107045650482177734375f,-0.54769997003972570315255552486632950603961944580078125f,0.5565000224709510945331203402020037174224853515625f,-0.584100012594461315273974832962267100811004638671875f,-0.548099995502829617777251769439317286014556884765625f,0.548000010073184995462725055404007434844970703125f,-0.284899983948469159855676480219699442386627197265625f,0.03960000530779350658150406161439605057239532470703125f,0.5519999818801881730223612976260483264923095703125f,0.71159998174905769730003157746978104114532470703125f,0.93599995636940047649687812736374326050281524658203125f,-0.810499997854233100724741234444081783294677734375f,-0.768700023341178706459686509333550930023193359375f,-0.6548999788448217618253011096385307610034942626953125f,0.20999998755753068735430133529007434844970703125f,-0.32909998875260360051697716698981821537017822265625f,0.70559997411966346447798059671185910701751708984375f,-0.783000009775161487368677626363933086395263671875f,-0.747299976241588836955997976474463939666748046875f,-1.0647999918505546279590134872705675661563873291015625f,-0.08099999719858175240005948580801486968994140625f,-0.298500013887882165164455727790482342243194580078125f,-0.27799999900907279570816399427712894976139068603515625f,0.448000016033649473001787555404007434844970703125f,-0.546400005674362088115003643906675279140472412109375f,-0.30879999752044684502294558114954270422458648681640625f,0.06450000279769307898192209904664196074008941650390625f,-0.59219999973401427073582681259722448885440826416015625f,-0.13790000178888439652524766643182374536991119384765625f,-1.13699998903274490658077411353588104248046875f,0.59219998283982278053372283466160297393798828125f,-0.4596000062391158991204065387137234210968017578125f,0.90249998865649072588013268614304251968860626220703125f,-0.564699999659508566907106796861626207828521728515625f,-0.19290000175163146689527593480306677520275115966796875f,0.62000001547858130113155539220315404236316680908203125f,0.1171000094242393985144445878177066333591938018798828125f,0.00360000112429259555568705764017067849636077880859375f,-0.489999997988343150012724436237476766109466552734375f,-0.93470001524984824836650432189344428479671478271484375f,-0.3048000029146671341351293449406512081623077392578125f,0.80250000491738315844258977449499070644378662109375f,0.0360000027120113141876345252967439591884613037109375f,0.056699995714426076887093586265109479427337646484375f,0.063000010371208503556772484444081783294677734375f,0.816300029706955143637969740666449069976806640625f,0.08580000026226031195619725622236728668212890625f,1.392299972069263702678654226474463939666748046875f,-0.19570000432729717232405164395458996295928955078125f,-0.442000018090009660909345257095992565155029296875f,1.5877000297188761379629795555956661701202392578125f,0.067300002467632236147210278431884944438934326171875f,-1.08750002086162567138671875f,-0.41579996867179858099916600622236728668212890625f,1.1438000180840486308397885295562446117401123046875f,-0.6213000112108886252570982833276502788066864013671875f,0.8430000222176314839117594601702876389026641845703125f,0.536400021432340334825994432321749627590179443359375f,-0.4304999918937681968600372783839702606201171875f,-1.3859999742507937980917631648480892181396484375f,0.13200000780820886348010390065610408782958984375f,1.083700005757808781936546438373625278472900390625f,-0.504999998807907246600734652020037174224853515625f,-0.4526000024914740293979775742627680301666259765625f,0.3394000153005123365801409818232059478759765625f,0.3262999814271925203001956106163561344146728515625f,-0.462500020265579347977791258017532527446746826171875f,-0.43240001956224460855082725174725055694580078125f,-1.342100009405612581758759915828704833984375f,0.511300022554397770591094740666449069976806640625f,0.99300000649690645104783470742404460906982421875f,1.0767000089764593440122553147375583648681640625f,-0.724700027394295176463856478221714496612548828125f,1.01220001815557480995266814716160297393798828125f,-1.249500033736229198666478623636066913604736328125f,-0.17510001053810153592849019332788884639739990234375f,-0.476099978232383591603138484060764312744140625f,-1.3355999922990804407163523137569427490234375f,-0.133600025022030166610420565120875835418701171875f,-0.1669999695122239558031651540659368038177490234375f,-1.0722999842882163790136473835445940494537353515625f,-0.37370003168582943686715225339867174625396728515625f,0.491299972840398613271872818586416542530059814453125f,-1.3876999603912232306157648054067976772785186767578125f,0.196300023634731868327207848778925836086273193359375f,0.2214999919235705050368778756819665431976318359375f,0.6886999780416491745427265414036810398101806640625f,-0.06169999973773965251666595577262341976165771484375f,-0.1961999916762113826962377061136066913604736328125f,0.1094999991655349891317428046022541821002960205078125f,0.1855000151395795882081074523739516735076904296875f,0.656899989521503524514400851330719888210296630859375f,-0.258099985109269791649921899079345166683197021484375f,0.23050002124905599742987760691903531551361083984375f,0.9533000191867351258423468607361428439617156982421875f,0.0385000139251350237401538834092207252979278564453125f,-0.692499991655349589336765347979962825775146484375f,0.37989999795556084194458890124224126338958740234375f,0.622500002160668142181521034217439591884613037109375f,-0.24800000295788049431422450652462430298328399658203125f,0.7517000083990394987409899840713478624820709228515625f,0.19660000436156988001812351285479962825775146484375f,0.15349996531009679756607511080801486968994140625f,-0.6076999902307989032124169170856475830078125f,0.522499972134828905012682298547588288784027099609375f,-0.2065000025033949526687138131819665431976318359375f,-0.6340999893486500837980202049948275089263916015625f,-0.60989999698698504904115225144778378307819366455078125f,0.35350000837445261225866488530300557613372802734375f,-0.0499000080883502672435270142159424722194671630859375f,-0.20530000251978630654292601320776157081127166748046875f,-0.343000019013881995988413109444081783294677734375f,-0.4553000073254107604725504643283784389495849609375f,0.4358000011563298858874304642085917294025421142578125f,0.209000034987926852636519470252096652984619140625f,0.1321000137269496832459481083787977695465087890625f,-0.7549999866634606870974266712437383830547332763671875f,-0.5135000001788139201153171597979962825775146484375f,-0.921700020301341904627179246745072305202484130859375f,-0.4899000012710690299400084768421947956085205078125f,0.53549998831748979455369408242404460906982421875f,-0.6621000074684619818299324833787977695465087890625f,0.60369996790885949167204671539366245269775390625f,0.61999999046325715568173109204508364200592041015625f,-0.66410000082254416753357872948981821537017822265625f,0.3839999651014809334270694307633675634860992431640625f,-0.61400000960007294903419960974133573472499847412109375f,-0.76719997052773847290296771461726166307926177978515625f,-0.16459999191537499907411756794317625463008880615234375f,-0.08699998486042037626475575962103903293609619140625f,0.30179999239444743608373755705542862415313720703125f,-0.265999982334673645834044464209000580012798309326171875f,-0.158999994084239038016903577954508364200592041015625f,-0.3833000191122295507994977015187032520771026611328125f,0.371499992370605269798033987171947956085205078125f,0.463699963438511275626296992413699626922607421875f,0.334300020110607221113241394050419330596923828125f,0.785200014817714730952502577565610408782958984375f,-0.922600017690658802393954829312860965728759765625f,-0.2750000059604644775390625f,-0.91250001981854467203447711654007434844970703125f,0.143899981385469999395354534499347209930419921875f,-0.03089998978972374743534601293504238128662109375f,0.057199997442960892612973111681640148162841796875f,-0.6274999731779100642370394780300557613372802734375f,0.248899991965293221340971285826526582241058349609375f,-0.9600000121630729577049123690812848508358001708984375f,-0.1002000036418435602314502830267883837223052978515625f,0.44320000302344542131294247155892662703990936279296875f,0.455600007386505634343620840809307992458343505859375f,-0.174200008380413162711874974775128066539764404296875f,-0.2406000072747469520351160099380649626255035400390625f,0.472500005718320625902606479940004646778106689453125f,-0.1161000105306506657854015429620631039142608642578125f,-0.924900031892955443169057616614736616611480712890625f,-0.080400046026706917245974182151257991790771484375f,-1.149299988818167861381880356930196285247802734375f,-0.192499984502791932072796043939888477325439453125f,-1.56079997940063464056947850622236728668212890625f,0.86480001766681624530974659137427806854248046875f,0.385599979186057595370584749616682529449462890625f,-0.004499993819743419720680321916006505489349365234375f,0.4742999879613518654508652616641484200954437255859375f,-0.397300014351308294902054285557824186980724334716796875f,-0.0105000069439409315918965148739516735076904296875f,-1.4360999508142473501948188641108572483062744140625f,-0.17589999917745569035787411849014461040496826171875f,0.683500007629394445984871708787977695465087890625f,0.141799965274334027753866394050419330596923828125f,-0.5208000118851661142116427072323858737945556640625f,0.67280001403093336875826935283839702606201171875f,-0.103700023937225438430687063373625278472900390625f,-0.87270002018213244099342773552052676677703857421875f,0.04609999313354506966788903810083866119384765625f,-0.874399966871738598683805321343243122100830078125f,0.28859999998807861487648551701568067073822021484375f,0.4932000026881695475822198204696178436279296875f,-0.037699996116757450437262377818115055561065673828125f,-0.7164999962449076775783396442420780658721923828125f,0.28719998209476482742275038617663085460662841796875f,-0.42149997784197357209023948598769493401050567626953125f,-0.2160000082254409647930515347979962825775146484375f,-0.73740000734329225196006518672220408916473388671875f,-0.35840000513792025227388649000204168260097503662109375f,0.71599998453259416919536306522786617279052734375f,0.24459999673962595778675677138380706310272216796875f,0.6490000013485548446823258927906863391399383544921875f,-0.36549998429417573930777507484890520572662353515625f,-0.2602999942123889187683971613296307623386383056640625f,0.0120999987140297804444344365037977695465087890625f,0.13050000202655775982663044487708248198032379150390625f,0.28499998940154924742529374270816333591938018798828125f,0.6244999990910289344725470073171891272068023681640625f,-0.0209999979436397143928161312942393124103546142578125f,-0.26340000212043523930560695589520037174224853515625f,-0.252199999976158106829160487905028276145458221435546875f,0.86400001149997118776724391864263452589511871337890625f,-0.46799999522790314276932122083962894976139068603515625f,-0.12900000259652733003434832426137290894985198974609375f,0.3184999927282330389743947307579219341278076171875f,-0.66049997639656066183988514239899814128875732421875f,0.29530001526027882530200940891518257558345794677734375f,0.646000005841255386940247262828052043914794921875f,0.89599998736381536446060636080801486968994140625f,0.9115999735534192627284255650010891258716583251953125f,0.52200000480934971580637693477910943329334259033203125f,0.1181999971933662962886302238985081203281879425048828125f,0.00699999832734465865513584503787569701671600341796875f,0.24649998164176967208049973123706877231597900390625f,-0.820999969333410462013489450328052043914794921875f,0.289499982476234707373663468388258479535579681396484375f,0.1760000055432318077919262577779591083526611328125f,-0.235100010538101145129985525272786617279052734375f,-0.41299999267607934161361527003464289009571075439453125f,0.6469999958872794110220638685859739780426025390625f,-0.482900000876188340015460198628716170787811279296875f,-0.288600004756450534415535003063268959522247314453125f,-0.4079999930858608792050290503539144992828369140625f,-0.375899988448619826186813952517695724964141845703125f,0.053799997557699708039535835268907248973846435546875f,-0.17799999710917457917958017787896096706390380859375f,-0.93120003556013131174040609039366245269775390625f,-1.332100014770031037869557621888816356658935546875f,-0.626899979686736941175695392303168773651123046875f,0.84270001600980748435176792554557323455810546875f,-0.33929998226165736241455306299030780792236328125f,-2.08589994702339254217804409563541412353515625f,0.4042999894142145222986073349602520465850830078125f,0.469399993675202242826571819023229181766510009765625f,0.0046000186815857713185096145025454461574554443359375f,-0.9248000361070041197564250978757627308368682861328125f,1.13569998257160165167078957892954349517822265625f,-0.28160003747940010043748770840466022491455078125f,-0.033999978959560195335143362171947956085205078125f,0.179499972008168473536215969943441450595855712890625f,1.232899983441830382702164570218883454799652099609375f,0.2021000219970943145852970701525919139385223388671875f,-1.25270000498294820090450230054557323455810546875f,-1.0245999465227129121558391489088535308837890625f,0.9672000434875496210906931082718074321746826171875f,-0.701300001692771957095828838646411895751953125f,-0.8868999886274337995928362943232059478759765625f,0.89830004407167507451958954334259033203125f,0.6267999983549117359871161170303821563720703125f,-0.22390001759529098990242346189916133880615234375f,-0.5665000057816502732066510361619293689727783203125f,1.0393000221967696461433661170303821563720703125f,0.4567999518632888111824286170303821563720703125f,-0.902200015473365368734448566101491451263427734375f,-0.16759998192787151083393837325274944305419921875f,-1.06089997861385398891798104159533977508544921875f,-0.045300012540817391482050879858434200286865234375f,-0.523500029832124624817879521287977695465087890625f,-0.4115999447941778299764337134547531604766845703125f,0.833600025320052839816753476043231785297393798828125f,-0.30630001361370062795685953460633754730224609375f,-2.2871999344110491847459343262016773223876953125f,0.2355999973654743229189989506267011165618896484375f,-0.2202000061005353614973500953055918216705322265625f,0.76800001350045210557482278090901672840118408203125f,0.0555999972164629152615589191555045545101165771484375f,-0.0532000087976456370597588829696178436279296875f,-0.1229000022172925810082233510911464691162109375f,-0.741500019490718642600768362171947956085205078125f,1.247700009267032061899271866423077881336212158203125f,-0.73939999917000509555009557516314089298248291015625f,-0.1150000113248825783784923260100185871124267578125f,0.21339999664425857162086686003021895885467529296875f,0.90119998992532479586969884621794335544109344482421875f,-0.2675000137090677299056551419198513031005859375f,-0.90519999067783363244643624057061970233917236328125f,0.5170999644041065490540631799376569688320159912109375f,0.86550002704188255808759322462719865143299102783203125f,-0.403199978678673520704478505649603903293609619140625f,-0.14729999697282913917462110475753434002399444580078125f,-0.795499997258186652970834984444081783294677734375f,-0.383500006884336386292488896287977695465087890625f,-0.7748999696433547246243733752635307610034942626953125f,-0.4049999839067461859798413570388220250606536865234375f,0.6888000007614494268892713080276735126972198486328125f,-0.3839999658092860812530489056371152400970458984375f,-0.33500005304813385009765625f,0.02870001857280701784702614531852304935455322265625f,0.8843999713048342758980879807495512068271636962890625f,-0.654000046610832441729144193232059478759765625f,-0.029400001591444180348844383843243122100830078125f,0.47259998494535704338659343193285167217254638671875f,0.279999997615814066875827847979962825775146484375f,-0.43389999688267710808986521442420780658721923828125f,-0.29959999457895758467884661513380706310272216796875f,0.410999995708465348798199556767940521240234375f,-0.2667000119566917959446072927676141262054443359375f,-0.645599997700750716234097126289270818233489990234375f,0.189499986648559826107884873636066913604736328125f,-0.546499996542930688292472041212022304534912109375f,0.3126999741002920440990919814794324338436126708984375f,-0.70349999526143047745563308126293122768402099609375f,0.1665000230669975156416739991982467472553253173828125f,0.2755000076889990712203371003852225840091705322265625f,-0.06950000870227768245968036353588104248046875f,-1.07589997131228454207985123503021895885467529296875f,0.5630999799311162679060771552030928432941436767578125f,0.086000014260411372646331074065528810024261474609375f,0.47020000639855863422411630381247960031032562255859375f,0.0184000060319899905181273425114341080188751220703125f,1.0130000026226042564303497783839702606201171875f,-0.3064999889433384083758937777020037174224853515625f,0.217399998104572222956676341709680855274200439453125f,-0.0324999845027924560980636670137755572795867919921875f,-0.9932000380039216480554387089796364307403564453125f,-0.01029999763965605552584747783839702606201171875f,-0.778699998605251408889671438373625278472900390625f,0.31510001143217092334225526428781449794769287109375f,0.78410000469684604240683256648480892181396484375f,-0.27350001478195196114029386080801486968994140625f,-0.994300042164326214333414100110530853271484375f,0.2594000059127807844561175443232059478759765625f,2.07909999158382419182089506648480892181396484375f,0.0226999990224836523111662245355546474456787109375f,-0.0354999960660933311373810283839702606201171875f,-0.295099982821941608790439204312860965728759765625f,0.73010001649856537397909050923772156238555908203125f,0.6760000072568654871929538785479962825775146484375f,-1.00240002090334900941570595023222267627716064453125f,-0.3456000118196012049764931362005881965160369873046875f,-1.011999992311000795552899944595992565155029296875f,-0.519799992036819702434513601474463939666748046875f,0.7228000038981434727247687987983226776123046875f,0.14209999122619620948171359486877918243408203125f,0.62619999115467095407439046539366245269775390625f,0.467000020325183839986493694595992565155029296875f,-0.430900013929605307794190593995153903961181640625f,-0.3787999459624291631598680396564304828643798828125f,0.591000010907649642177830173750407993793487548828125f,-0.023100045235455279879488443839363753795623779296875f,-1.468499997794627365266251217690296471118927001953125f,-0.3286999961465595365694980500848032534122467041015625f,0.87929999950975190614599341643042862415313720703125f,-1.754099994489550784493303581257350742816925048828125f,-0.12230001341998641972708128378144465386867523193359375f,-0.778899998521804803885970613919198513031005859375f,0.603700005459785415951046161353588104248046875f,0.13749998748302427742373765795491635799407958984375f,0.852699989727884410939395820605568587779998779296875f,-1.6615000155046579788375993302906863391399383544921875f,-0.127099975021182809431508076158934272825717926025390625f,-0.27720002433955670539944549091160297393798828125f,-1.0235999777108428787641969393007457256317138671875f,0.55720000660717428075940915732644498348236083984375f,0.555200010645389596675158827565610408782958984375f,1.342899989700317320284739253111183643341064453125f,-0.0844999958872794110220638685859739780426025390625f,-1.09039999566078193993234890513122081756591796875f,-0.650600012825429008245237127994187176227569580078125f,-0.4015000138282776021014797152020037174224853515625f,-0.305000003874301928163959019002504646778106689453125f,0.073099994012713498392486144439317286014556884765625f,0.725000027418136738788234652020037174224853515625f,-0.6361999987542628076653272728435695171356201171875f,-0.3151999961540103978308025034493766725063323974609375f,-0.233499974846839819520027958787977695465087890625f,1.06209999569654467421742083388380706310272216796875f,-0.54829998182207351220540658687241375446319580078125f,-0.9300000017881391300988980219699442386627197265625f,0.097799996596574789720079934340901672840118408203125f,-0.0790000013485550045544414388132281601428985595703125f,-0.042000000432133699490577782853506505489349365234375f,-0.78240000816285604212652060596155934035778045654296875f,-0.25000000126659871479972707675187848508358001708984375f,-0.1955000091791152527775921043939888477325439453125f,0.16699998500943191714895874611102044582366943359375f,0.67449998243898157124220915648038499057292938232421875f,-0.721499993562698449522940791212022304534912109375f,0.46349999377131467781509854830801486968994140625f,-0.3096999822437764837701479336828924715518951416015625f,-0.45399999490380249511645160964690148830413818359375f,0.0835000202953815051643005062942393124103546142578125f,0.2537999974831939287156501450226642191410064697265625f,0.4565000001192089928991890701581723988056182861328125f,-0.8875999822631477353951368058915250003337860107421875f,0.336700015085935167036268467199988663196563720703125f,-0.4814999888315794063231578547856770455837249755859375f,-1.14599999865144486221879560616798698902130126953125f,-0.3180999990418553746707175378105603158473968505859375f,0.39350001209974294624771573580801486968994140625f,0.519999994337558835155732595012523233890533447265625f,0.43729999479353420799299101418000645935535430908203125f,-0.45600003152713208809387879227870143949985504150390625f,-1.093199994734674884711012055049650371074676513671875f,-0.31339999625310299702363181495456956326961517333984375f,-0.519999986663460589397800504229962825775146484375f,-0.30079999549388869439781046821735799312591552734375f,0.2584000083416697624016933332313783466815948486328125f,-0.3079999912977218201604046043939888477325439453125f,0.57649999788403505363021395169198513031005859375f,-0.61639998861253253448211353315855376422405242919921875f,0.921499989390373031028502737171947956085205078125f,-0.84580001218318923150718546821735799312591552734375f,-0.307499982230365009883144011837430298328399658203125f,-1.272899958407879505983828494208864867687225341796875f,-0.2451000138908623338096504085115157067775726318359375f,-0.06490000339448442900902591645717620849609375f,-0.9403999784499408320925795123912394046783447265625f,0.5726000129967925289520280784927308559417724609375f,-0.41920003222227109063169336877763271331787109375f,-0.25849999243020960193462087772786617279052734375f,-0.6329000347018247651931233122013509273529052734375f,-1.233900030112266676951549015939235687255859375f,-0.8951999811410900065311579965054988861083984375f,0.75940001366138432103980449028313159942626953125f,0.3596000442743303437964641489088535308837890625f,0.91449998545646593584024230949580669403076171875f,0.4927000017046925250951971975155174732208251953125f,-0.0983000066697599095277837477624416351318359375f,-1.241500015765428344138854299671947956085205078125f,0.524900035208464288416507770307362079620361328125f,-0.9705999991908669244367047213017940521240234375f,-0.03739998286813506211956337210722267627716064453125f,-0.293800018419325237317707433248870074748992919921875f,1.20970001703798768488695714040659368038177490234375f,0.42799995791912071041451781638897955417633056640625f,-0.388600010567903719760352032608352601528167724609375f,0.29310002567768123071800800971686840057373046875f,1.1325000232458108939681551419198513031005859375f,0.7397000265002251495616292231716215610504150390625f,-0.031199977743625595394405536353588104248046875f,-1.230600014352798865502336411736905574798583984375f,-0.06180001087188724540055773104541003704071044921875f,0.321299995434284113571266061626374721527099609375f,-0.3048000149846075146342627704143524169921875f,0.04540001503229174062425954616628587245941162109375f,-0.175299993169307555263003450818359851837158203125f,1.818400028085708441949464031495153903961181640625f,0.2800000140070917353796176030300557613372802734375f,0.7519999979734421913235564716160297393798828125f,0.15460000910758964209890109486877918243408203125f,0.1343999972701066525360147352330386638641357421875f,1.091700005698204023474318091757595539093017578125f,-0.472500026226043701171875f,-0.70610002025365847799776020110584795475006103515625f,-0.037799986165762078371699317358434200286865234375f,-0.37999998949468138942364703325438313186168670654296875f,0.5564999762773517044678328602458350360393524169921875f,-0.6995999819502236594104260802851058542728424072265625f,0.493499964267015922558812235365621745586395263671875f,-0.6980000050812957823609394836239516735076904296875f,-0.292899997746944418253178810118697583675384521484375f,0.0857999961271882494173723898711614310741424560546875f,1.142500035464763641357421875f,-0.27009998759031272896891096024774014949798583984375f,0.4849000114038581887854206797783263027667999267578125f,-0.0770000009536744300930877216160297393798828125f,0.0232999999642373722963384352624416351318359375f,0.89280000013560023575109880766831338405609130859375f,-0.918000000238418323306177626363933086395263671875f,0.58289997405409810227183697861619293689727783203125f,-0.523200009915232300272691645659506320953369140625f,-0.609000037223101031713667907752096652984619140625f,-0.46260000860095029651120057678781449794769287109375f,0.17159999653846025324810398160479962825775146484375f,1.104999993927776813507080078125f,0.070400007879733994542448272113688290119171142578125f,0.778199980001151647712731573847122490406036376953125f,-0.388999985218048127677548109204508364200592041015625f,-0.141100017794966614559371009818278253078460693359375f,-0.8743999858334661201553217324544675648212432861328125f,-1.1940000090599056647988618351519107818603515625f,0.79560000049471835126269070315174758434295654296875f,-0.5642000037237997478456463795737363398075103759765625f,0.1109999817013740397442234097979962825775146484375f,-0.669000015318393703722676946199499070644378662109375f,0.2453999774068595929321645598975010216236114501953125f,-0.296000011801719864479309762828052043914794921875f,-0.209500002741813684536964501603506505489349365234375f,-0.19259999381154779829472545316093601286411285400390625f,-0.2474999970197675480676480219699442386627197265625f,1.095900013035535636163331218995153903961181640625f,-0.24089999716579890698398003223701380193233489990234375f,-0.734499974131583854841665015555918216705322265625f,-0.01939999741911879738154311780817806720733642578125f,-0.60449998407810934253348023048602044582366943359375f,0.120000012218952178955078125f,-0.704400003975629740438080261810682713985443115234375f,-0.56739999188333756219293491085409186780452728271484375f,0.87899999371170967066291268565692007541656494140625f,-0.03149999892711630167241310118697583675384521484375f,0.12340000901222225593301118351519107818603515625f,0.17899998581409448661361238919198513031005859375f,0.54880002762079271860784501768648624420166015625f,-0.607900010226666953627727707498706877231597900390625f,-1.020499985635280548734726835391484200954437255859375f,0.0526999974578619134746304553118534386157989501953125f,1.696300036263465926822391338646411895751953125f,-0.44890005574226421458661206997931003570556640625f,-0.5965000212788584832424021442420780658721923828125f,0.06309996249675720036975690163671970367431640625f,-1.88200001645088121904336730949580669403076171875f,0.03020002286434220195587840862572193145751953125f,-0.1737999851524829608706568251363933086395263671875f,-1.3712000126123431442692890414036810398101806640625f,-0.19140002474784889585635028197430074214935302734375f,-1.33839998278617855476113618351519107818603515625f,0.6328000077724453831251594237983226776123046875f,0.3043999981641771768181570223532617092132568359375f,0.6211000129520893864309982745908200740814208984375f,0.2257999916195867484702830552123486995697021484375f,-0.4196000150680543328007843228988349437713623046875f,1.60010003855228433167212642729282379150390625f,0.4188999585866923780486104078590869903564453125f,-1.06370002155303922108942060731351375579833984375f,1.47400001740455621757064363919198513031005859375f,-0.72760005106925973450415767729282379150390625f,-0.5174000112175942689418661757372319698333740234375f,0.1796000000923871908753426396287977695465087890625f,-0.45560000228285790768723018118180334568023681640625f,0.3128000156700612155447061013546772301197052001953125f,0.3879999876469373276677288231439888477325439453125f,-0.19719999565482115855274969362653791904449462890625f,-0.4987999985635276356532585850800387561321258544921875f,-0.5856000040709972154218121431767940521240234375f,-1.785299981099367272463496192358434200286865234375f,0.5231000133097172266616325941868126392364501953125f,0.051099996113777024220325984060764312744140625f,0.182899978077411873300661682151257991790771484375f,0.48330002410411854185667834826745092868804931640625f,-0.57819998141676176572900658356957137584686279296875f,1.409499998643994000957491152803413569927215576171875f,-0.06690003623664464438292043269029818475246429443359375f,0.45699999834597093695265357382595539093017578125f,-0.7221000226676466127173625864088535308837890625f,0.129799991291761340761468090931884944438934326171875f,0.11480000112652766208753973842249251902103424072265625f,0.44320001952648180321148174698464572429656982421875f,-0.57849999529868301184620804633595980703830718994140625f,0.0531999988138676105364766044658608734607696533203125f,-0.40120001399070037706451330450363457202911376953125f,0.18409999501109108877017206395976245403289794921875f,0.321700010019540683092742483495385386049747467041015625f,-0.315099990868568102797553365235216915607452392578125f,0.042899989961087958700858280280954204499721527099609375f,0.6312999704375865661631905823014676570892333984375f,0.815299997527896724847096265875734388828277587890625f,0.542999985039234189798662555404007434844970703125f,-0.5468999871000648216323725137044675648212432861328125f,-0.3765000117421148928542606881819665431976318359375f,-0.19859999253749816006120454403571784496307373046875f,-0.6975999982446430269789061640040017664432525634765625f,-0.0416000476121904938509032945148646831512451171875f,0.51829996598959038323073400533758103847503662109375f,-0.73469999498426918904669946641661226749420166015625f,0.22769998412132252951778355054557323455810546875f,-0.06549999800324446397326028090901672840118408203125f,0.03199999160319577651279132624040357768535614013671875f,0.86919999169111239467611085274256765842437744140625f,-0.072100009778142126748434748151339590549468994140625f,0.88230000358521909475939537514932453632354736328125f,0.9171000447213655260014775194576941430568695068359375f,0.09900001271069047703576870844699442386627197265625f,-0.60699999350309352674770480007282458245754241943359375f,-1.2086000478208067665519820366171188652515411376953125f,0.585699987041949921007244483917020261287689208984375f,-0.0618999936938287032006655863369815051555633544921875f,-0.1690999823451040384725274634547531604766845703125f,0.6055999911069864793944361736066639423370361328125f,-0.2693999888509512885548247140832245349884033203125f,0.5610000010728839470175444148480892181396484375f,0.7136999652266506899422893184237182140350341796875f,-0.4434999639689924944008225793368183076381683349609375f,-0.8267000188112252345717934076674282550811767578125f,-0.03530000255703935607698440435342490673065185546875f,0.289199999327957646411135783637291751801967620849609375f,1.030799996984005151290375579264946281909942626953125f,-0.049099996501207332499916446977294981479644775390625f,-0.5717999941453337786612109994166530668735504150390625f,-0.4638000238955022336284628181601874530315399169921875f,-0.1874000010848042574451710606808774173259735107421875f,0.40760001608878371204269797090091742575168609619140625f,-0.823199964094161629191148676909506320953369140625f,-0.6473999802231791278472883277572691440582275390625f,0.405299975287914282517931496840901672840118408203125f,-0.9068000242829323820359377350541763007640838623046875f,-0.431999996632337346369467923068441450595855712890625f,0.7155000136122107168290540357702411711215972900390625f,-1.085600018823146939439538982696831226348876953125f,-0.7406000089883804093915387056767940521240234375f,0.534200007449090463751417701132595539093017578125f,0.8158000282764437116611588862724602222442626953125f,0.48420001761913322724240060779266059398651123046875f,-0.40320000667423006035505750332958996295928955078125f,0.7840999954581260311670121154747903347015380859375f,0.2929999865293506644547960604541003704071044921875f,-0.6283999853566288873452094776439480483531951904296875f,0.926499992072582045921080862171947956085205078125f,0.62679999746084202882911995402537286281585693359375f,-0.415899991242587585560386287397705018520355224609375f,0.42369999696612392625638676690869033336639404296875f,0.586800012660026570898708087042905390262603759765625f,0.191699996943771822088820044882595539093017578125f,0.0256999927043910503243751008994877338409423828125f,-0.3104000224828720178038565791212022304534912109375f,-0.4635999897807836500618350328295491635799407958984375f,-1.1984000066280362517545654554851353168487548828125f,-0.8628000167131428810307625099085271358489990234375f,0.5057999868512152996657960102311335504055023193359375f,0.55589999634623499247254585498012602329254150390625f,0.481800011467933497755211647017858922481536865234375f,-0.075299993839859968858263528090901672840118408203125f,0.8244999905228613812369076185859739780426025390625f,0.5767999938845633778328192420303821563720703125f,-0.3796999929353594982472941410378552973270416259765625f,0.7614999968409534858437837101519107818603515625f,0.5541999931812284785337396897375583648681640625f,-0.491799998988211140016346689662896096706390380859375f,-0.9334000021576880357088157325051724910736083984375f,-0.4387999930500985357184617896564304828643798828125f,0.6436999917134642590355042557348497211933135986328125f,-0.51809998902082465832563684671185910701751708984375f,0.2436000083327289900125833810307085514068603515625f,-0.0959000009655952556641977935214526951313018798828125f,0.0128999954372643610422954907335224561393260955810546875f,0.4593000185459852335867481087916530668735504150390625f,0.46469998179674121274729259312152862548828125f,0.2688000070944427388752728802501223981380462646484375f,1.2504000084757809219127011601813137531280517578125f,-0.0598000017970801156508287022006697952747344970703125f,-0.02410000592619188086729309361544437706470489501953125f,0.83350001813471319422887972905300557613372802734375f,0.34949999171495438332613048260100185871124267578125f,-0.4969000056147574451159698583069257438182830810546875f,0.3150999884843823650726335472427308559417724609375f,-0.094900009429454978970852607744745910167694091796875f,0.90470001398324928487681972910650074481964111328125f,0.59590004924535833197296597063541412353515625f,0.23609998642802221269221263355575501918792724609375f,0.560800003279745329365368888829834759235382080078125f,0.126800046336651206502210698090493679046630859375f,-0.27379999707341173831309788511134684085845947265625f,0.72979997348487402053507366872509010136127471923828125f,0.20380000646114293516575344256125390529632568359375f,-0.7065999701261527210505164475762285292148590087890625f,0.6218000070720908212251742952503263950347900390625f,0.484600043827295934306675917468965053558349609375f,0.6165999756395823538923650630749762058258056640625f,-0.5273000136807557414186931055155582726001739501953125f,-0.7813000419259079620815100497566163539886474609375f,0.4654000093698495987837304710410535335540771484375f,-0.7672000007957218148391120848827995359897613525390625f,0.0183999696731564910123779554851353168487548828125f,-0.3569000026345248155479339402518235146999359130859375f,0.80369996612072025499173832940869033336639404296875f,0.668099996918439131832201383076608180999755859375f,-0.29079998998045919478272480773739516735076904296875f,0.4429999845921994694464274289202876389026641845703125f,-0.1405999940872195708152503357268869876861572265625f,-0.36169998647570533734096898115240037441253662109375f,-0.6910000052079554588857490671216510236263275146484375f,-1.8314000649809845100435268250294029712677001953125f,-0.1035999886631966404593185870908200740814208984375f,-0.6862999897345900546241637130151502788066864013671875f,-0.1639000244498252101266189129091799259185791015625f,0.60999996438622527961115338257513940334320068359375f,-0.0812000107496977374665902971173636615276336669921875f,-0.0863000052690505725649927626363933086395263671875f,0.14620001946687732896634770440869033336639404296875f,0.3947000093638897144643351566628552973270416259765625f,0.83339996520280834602090180851519107818603515625f,0.5467999628901480946296942420303821563720703125f,-0.562699986244738159513190112193115055561065673828125f,0.33710003622770301490163546986877918243408203125f,-0.188600009822845748885811190120875835418701171875f,-0.77860001932233569021946095745079219341278076171875f,-1.0094000300526619895435942453332245349884033203125f,-0.4018000025272367992101862910203635692596435546875f,0.7684000101670622751015571338939480483531951904296875f,-0.74100001865625397812209484982304275035858154296875f,-0.37539999506473531454275871510617434978485107421875f,0.59020001142770051050234769718372263014316558837890625f,-0.4309999819993972636211765347979962825775146484375f,-0.25579998208284404626056129927746951580047607421875f,0.3214999874532222801093439557007513940334320068359375f,-0.3948000063419343508286374344606883823871612548828125f,-0.379800003510713413135135851916857063770294189453125f,0.1107000089466571857155230418356950394809246063232421875f,0.826300000202655926528905183658935129642486572265625f,0.51419998751878726039876710274256765842437744140625f,-0.524400003193318919869625460705719888210296630859375f,0.38790002486705787987375515513122081756591796875f,-0.00440000770092030535352023434825241565704345703125f,-0.577000014588236798118714432348497211933135986328125f,-0.8309999918341635094520825077779591083526611328125f,-0.32299998176097854951649424037896096706390380859375f,0.6355999905481934941775534753105603158473968505859375f,-0.758200018036365719353852909989655017852783203125f,-0.393599994027614474845222503063268959522247314453125f,0.5886000121325254408333194078295491635799407958984375f,0.4677000288248063242235730285756289958953857421875f,0.09300001096725463156644764239899814128875732421875f,-0.379200008119642717474562232382595539093017578125f,-0.7588000231504441472907274146564304828643798828125f,-0.39540000430345489945693771005608141422271728515625f,0.4623000152081251368230141451931558549404144287109375f,0.1359999915361402855751293827779591083526611328125f,0.17580000652074811995362324523739516735076904296875f,0.1083000077873468303124582234886474907398223876953125f,0.308599992239475096056366965058259665966033935546875f,-0.598100014352798581285242107696831226348876953125f,-0.34150000295043003717410101671703159809112548828125f,0.76489999877512426795789224343025125563144683837890625f,1.23010005106926012530266234534792602062225341796875f,-0.26439996627569239961985658737830817699432373046875f,-0.4707000017568467864492731678183190524578094482421875f,-1.5531000104784968840476722107268869876861572265625f,0.677699978905916822213839623145759105682373046875f,-0.780799989049136389240857170079834759235382080078125f,-1.466800064814091086873304448090493679046630859375f,-0.25909998383522037102011381648480892181396484375f,-0.8070999946534633817663006993825547397136688232421875f,-0.2903000155210495591973085538484156131744384765625f,0.00489999244213112916668251273222267627716064453125f,0.18509998393952853401600577853969298303127288818359375f,1.00110001131296133536352499504573643207550048828125f,0.0832999892354013837802995112724602222442626953125f,0.24379998861700313828038133578957058489322662353515625f,0.79899998611211753285488157416693866252899169921875f,0.3584999804347754892575039775692857801914215087890625f,0.092399993373453614964319058344699442386627197265625f,0.5556000104039906517527924734167754650115966796875f,-0.186899981176853469122534079360775649547576904296875f,-0.0339999943450093322638849713257513940334320068359375f,-0.8672000019133090376044492586515843868255615234375f,0.382799992424249779787714942358434200286865234375f,-0.37229998879581682746930937355500645935535430908203125f,-0.91350002938508989558386019780300557613372802734375f,-0.23049999159574507956449451739899814128875732421875f,-0.70289997211694743572962806865689344704151153564453125f,-0.29970001189708650457532712607644498348236083984375f,0.02239998961091060891703818924725055694580078125f,0.158799983215332118913210024402360431849956512451171875f,0.978800007951259321004044977598823606967926025390625f,0.329699975687265389723279440659098327159881591796875f,0.9191999959006904585834263343713246285915374755859375f,0.9924000311851504108062727027572691440582275390625f,-0.01289999983310696762828229111619293689727783203125f,-0.67339998327046657333738721717963926494121551513671875f,-0.30459999390840497568433420383371412754058837890625f,-0.18330000324249251519859171821735799312591552734375f,0.60499997805804028860876542239566333591938018798828125f,0.78340001229047739883526446647010743618011474609375f,0.316199985641241188005778894876129925251007080078125f,0.2238000009477139062852302231476642191410064697265625f,-0.55370000188350676495474544935859739780426025390625f,-0.244599995100498457389903705916367471218109130859375f,0.41739999322444198970316620034282095730304718017578125f,0.6526000090479850967994934762828052043914794921875f,0.395000001788139432079560720012523233890533447265625f,-0.391700000855326635473829810507595539093017578125f,0.596200003708899028964651733986102044582366943359375f,0.361000004462897983170677207453991286456584930419921875f,-0.2826999942354857520143696092418394982814788818359375f,1.015400018608570320566286682151257991790771484375f,0.330599979782103758907396695576608180999755859375f,-0.876000017128884866934868114185519516468048095703125f,0.540899989640712508531805724487639963626861572265625f,0.10719997851848628300786003819666802883148193359375f,-0.6180999851092696673049431410618126392364501953125f,0.00830001888871212400999866076745092868804931640625f,-0.131599990391731314076650960487313568592071533203125f,-0.02770000710636377760920368018560111522674560546875f,-0.634000002801418105491393362171947956085205078125f,-0.031399981236457819022689363919198513031005859375f,0.742999992229044448066588302026502788066864013671875f,-0.3591999986574052616816743466188199818134307861328125f,-0.22360000289380532745298069130512885749340057373046875f,0.11349999697506429896520785405300557613372802734375f,-0.87849999749660501180414939881302416324615478515625f,-0.6513999916672705836617751629091799259185791015625f,0.45490000190436841176477855697157792747020721435546875f,0.3504999766945839922982486314140260219573974609375f,0.325399976587295469698801753111183643341064453125f,-0.2029999956563115492969018305302597582340240478515625f,-0.2614000208735465236031814129091799259185791015625f,0.08419998990297283825157137471251189708709716796875f,0.322899996182322457372038115863688290119171142578125f,-0.678399990236759009576417156495153903961181640625f,-0.44499999225139585945498765795491635799407958984375f,0.324399995593726675480183985200710594654083251953125f,-1.3195000155568124000637908466160297393798828125f,-0.946200010228157140090843313373625278472900390625f,0.596400003550946689045986204291693866252899169921875f,1.04220001428127329035078219021670520305633544921875f,0.6633999915033574890088630127138458192348480224609375f,-0.63340001612752683257667740690521895885467529296875f,-0.61630001838207260078661420266143977642059326171875f,-0.77910000186562466950590533087961375713348388671875f,-0.08709999241828914051666288287378847599029541015625f,-0.288400006590783408721989644618588499724864959716796875f,0.083199974524974518175213233917020261287689208984375f,-0.11489999944567674816653379821218550205230712890625f,0.206000006027519610984910514162038452923297882080078125f,-0.323599991941451836652277052053250372409820556640625f,-0.0823000064350665194723433160106651484966278076171875f,0.6871999984115360060599186908802948892116546630859375f,0.474400035826862132015691031483584083616733551025390625f,0.400300002408027211231456021778285503387451171875f,0.678699974912405412652560698916204273700714111328125f,0.099499991416930555487851961515843868255615234375f,0.327199987161159722148795481189154088497161865234375f,-0.1289999996721744590644220807007513940334320068359375f,-0.6917999976098538894575540325604379177093505859375f,0.3093999820709225634374206492793746292591094970703125f,0.876999994024634244027538443333469331264495849609375f,1.36940003854632408319957903586328029632568359375f,-0.5970999742388727327124797739088535308837890625f,-0.433099983060360127495869164704345166683197021484375f,-0.107500005960464051213421043939888477325439453125f,0.046400005637109165146370060028857551515102386474609375f,-0.7462999733060600160428066374151967465877532958984375f,-0.797500008754431721769151408807374536991119384765625f,0.4108999748885633351846990990452468395233154296875f,0.1392999921709299204763965462916530668735504150390625f,1.0997000074267386793280820711515843868255615234375f,-0.3228999654114250716929745976813137531280517578125f,-0.6903999888435006226927725947462022304534912109375f,-1.430000014603137969970703125f,-0.467099961721897383171153705916367471218109130859375f,-0.797599999995529440610653182375244796276092529296875f,-1.016800039482117057332288823090493679046630859375f,0.17409999337196335744692987645976245403289794921875f,-0.50689996843636075940509044812642969191074371337890625f,-0.10929998881816782585474356892518699169158935546875f,0.51569998778700831820742678246460855007171630859375f,-0.6104000082895157941464958639699034392833709716796875f,0.3811999777436252401230376563034951686859130859375f,-0.26169997339993755769427252744208090007305145263671875f,0.0321000058665872334273672095150686800479888916015625f,-0.98070001285821195580183484707958996295928955078125f,-0.042900011307001051363840815611183643341064453125f,-0.120900009012222309223716365522705018520355224609375f,0.32369998161792779001189046539366245269775390625f,0.30679998285770437860264792107045650482177734375f,-0.150799994003772752648728783242404460906982421875f,0.17469998986199486434855998595594428479671478271484375f,0.390600008429586920133402827559621073305606842041015625f,0.174600003501027833952008450069115497171878814697265625f,0.6172999775826928381405878099030815064907073974609375f,0.38139998477548398003733609584742225706577301025390625f,-0.3879999794512987154604388706502504646778106689453125f,1.0425000539422040191084306570701301097869873046875f,0.2848000042557714550639502704143524169921875f,-0.9283000257804989718835031453636474907398223876953125f,-0.0042999938845633689510350450291298329830169677734375f,-0.134200005549192535880820287275128066539764404296875f,-0.1379000043034553646403850279966718517243862152099609375f,0.79139999047517761709968908689916133880615234375f,0.485599994385242705874361490714363753795623779296875f,-0.4309999859482049888725896380492486059665679931640625f,0.42209999152421939783152993186376988887786865234375f,0.326199998009204961135765188373625278472900390625f,-0.16929999209642410118448196953977458178997039794921875f,-0.07639999509453776482814646442420780658721923828125f,-0.017399998104572045320992401684634387493133544921875f,-0.035400001211464360295622100238688290119171142578125f,-0.955400001323222891613795582088641822338104248046875f,-0.69760000398159061063552144332788884639739990234375f,0.5028999915257095221221561587299220263957977294921875f,0.34290000072717674584055203013122081756591796875f,-0.1700000301003452563008977449499070644378662109375f,-0.73689999388009308489699833444319665431976318359375f,-0.4222999890565872771475142144481651484966278076171875f,-0.346799991053342804292469736537896096706390380859375f,0.1766999965712428288799884512627613730728626251220703125f,-0.39799999591708168367176767787896096706390380859375f,-0.23920000033378574499920432572253048419952392578125f,0.1377999963730573984577176815946586430072784423828125f,0.672900022631883754087311899638734757900238037109375f,0.3002000000655649092351495710317976772785186767578125f,-0.5246000125721097528952441280125640332698822021484375f,0.127100009107589784207448246888816356658935546875f,-0.01680000908374790213883898104541003704071044921875f,-0.29940000784993170412917606881819665431976318359375f,0.9995000170469288125474349726573564112186431884765625f,-0.68069999054372232194509706459939479827880859375f,-0.3015999705731875479841619380749762058258056640625f,-0.440699974711239672586771121132187545299530029296875f,-1.017899979865550363911097520031034946441650390625f,-0.60789998062923522592626568439300172030925750732421875f,-0.223600000155717071326222367133595980703830718994140625f,-0.567200017242878828938046353869140148162841796875f,-0.132099997317045858569173333307844586670398712158203125f,0.02020000086650242110408726148307323455810546875f,-0.576999982457607973884705643285997211933135986328125f,0.5291999820053572278766296221874654293060302734375f,0.914400016269087512199575940030626952648162841796875f,0.447600037956238594460955937393009662628173828125f,0.2638999945729969542895787526504136621952056884765625f,0.05169999921619894411861650951323099434375762939453125f,0.10730000105202197602238811668939888477325439453125f,-0.14120000233054152971590156084857881069183349609375f,0.493199993449449480831248138201772235333919525146484375f,-0.017999974906444560218687911401502788066864013671875f,-0.213099995726346858049282673164270818233489990234375f,0.16169999489486219790279619701323099434375762939453125f,-0.17789999506473552770557944313623011112213134765625f,-0.0784999973475930534760891532641835510730743408203125f,-0.781299973678589054770782240666449069976806640625f,-1.135700019973516106119859614409506320953369140625f,0.424199996012448732329858103184960782527923583984375f,0.124699988539517025998293320299126207828521728515625f,1.15270000721812237998165073804557323455810546875f,0.6553999880611893757986763375811278820037841796875f,-0.5915999702751639777176251300261355936527252197265625f,-1.1831999879360193261845779488794505596160888671875f,0.2629999819844963671044979491853155195713043212890625f,0.04039999811947347918561490587308071553707122802734375f,0.49359999589025971289402150432579219341278076171875f,0.0352000049829481742591497095418162643909454345703125f,-0.8109999659806492733338245670893229544162750244140625f,-0.860600004667043005923687815084122121334075927734375f,0.093699998679757090513930961606092751026153564453125f,0.2130000030323863369208226004047901369631290435791015625f,-0.3138000055670737697965932966326363384723663330078125f,0.127400005108118108410053537227213382720947265625f,-0.74089998058825734528198836414958350360393524169921875f,-1.25170002924203860317220460274256765842437744140625f,-0.3439999677091842844589564265334047377109527587890625f,0.350199996228516052809709435678087174892425537109375f,-0.1089000051155684101189535795128904283046722412109375f,0.13360000170171282274367285936023108661174774169921875f,0.205699997994303718229502919712103903293609619140625f,0.420900007894635219685142146772705018520355224609375f,0.3572000147283078685944701646803878247737884521484375f,-0.0577999986827373302133992183371447026729583740234375f,-0.3347999959111209733464420423842966556549072265625f,0.2538000285148616086416950565762817859649658203125f,0.8243999824061989389889504309394396841526031494140625f,0.1214999977350235127460109652020037174224853515625f,0.087999994277954129984209430404007434844970703125f,-0.1186000002861022917244326890795491635799407958984375f,-0.92209999510049822646351458388380706310272216796875f,-0.3491999779820442739719510427676141262054443359375f,0.75689999164491894578787878344883210957050323486328125f,-0.525499964475631742288896930404007434844970703125f,-0.4351999739885332729727451805956661701202392578125f,0.253999981716275247123348890454508364200592041015625f,0.96710001924037936049671770888380706310272216796875f,0.7514000155091284938180251629091799259185791015625f,-0.38690000881850716840659742956631816923618316650390625f,0.82859997107982596986630596802569925785064697265625f,0.648799987983704085081626544706523418426513671875f,-0.40819997430145738004370059570646844804286956787109375f,0.1108000115871430324432367342524230480194091796875f,-0.22440000859498976382155888131819665431976318359375f,-0.345300001774728304582140481215901672840118408203125f,-0.23320001505613330294863771996460855007171630859375f,-0.03259999802112556466227033524774014949798583984375f,0.28820001025050878684652388983522541821002960205078125f,0.3749999907612799887601795489899814128875732421875f,-0.1824000332713122674022088176570832729339599609375f,-0.7508999840155243798989204151439480483531951904296875f,0.51970003305673595495051131365471519529819488525390625f,0.0494000025600194181407687210594303905963897705078125f,-0.632300012153387047675323628936894237995147705078125f,-0.6175000166893003239465542719699442386627197265625f,-0.0675999968290330599529625033028423786163330078125f,0.6747999956130981136226409944356419146060943603515625f,0.2927000094532967722216199035756289958953857421875f,-0.21800003033876436120408470742404460906982421875f,-0.7297000074639916622487589847878552973270416259765625f,-0.4619999939128753840833496724371798336505889892578125f,0.8436000098228451093973490060307085514068603515625f,-0.401999975435436140980272057277034036815166473388671875f,-0.59819996919781004418581460413406603038311004638671875f,-0.85139999438673186205051024444401264190673828125f,-0.25280000307857986019399731958401389420032501220703125f,0.66029999819844942265945064718835055828094482421875f,0.587100027957558889823985737166367471218109130859375f,-0.81209996529817640720239069196395576000213623046875f,0.616499989911913903739559827954508364200592041015625f,1.199700014281273041660824674181640148162841796875f,-0.0689999919235706471454250277020037174224853515625f,-0.37409999057799581123617826960980892181396484375f,-0.6993000111699103626961004920303821563720703125f,-0.26139999598860708118763795937411487102508544921875f,-0.505599994236230809718790624174289405345916748046875f,0.089199970829486829870802466757595539093017578125f,0.3461999710381036265971488319337368011474609375f,0.3046999902531506432268315620603971183300018310546875f,1.0074999800324437870813198969699442386627197265625f,-0.4972999898016450259774501319043338298797607421875f,0.3469000023365020890508958473219536244869232177734375f,-0.4447000062346457838202695711515843868255615234375f,-0.46119998996257738355097899329848587512969970703125f,-0.2507999980270863016329485617461614310741424560546875f,-1.2276000007033349703533531283028423786163330078125f,0.573199979293346206787873597932048141956329345703125f,0.3676999993577598768723646571743302047252655029296875f,0.5101000218629838656170250033028423786163330078125f,-0.16609999700784694454114287509582936763763427734375f,0.09790000072717652823683920360053889453411102294921875f,0.36110002144575137350557270110584795475006103515625f,0.72539997986555082576387576409615576267242431640625f,-0.307499982602894572636387238162569701671600341796875f,-0.770699976909160255900133051909506320953369140625f,0.42729997702389999059136016512638889253139495849609375f,0.022700005765259323453619799693115055561065673828125f,-0.42890001934617760781520701129920780658721923828125f,-0.51039996630549477885097076068632304668426513671875f,-0.46909999311119321419027983210980892181396484375f,-0.171100008332729203175404109060764312744140625f,0.69819999374747254705653176642954349517822265625f,-0.62779998713433737833611303358338773250579833984375f,-1.18660002317428592277792631648480892181396484375f,0.294900001792609278705725728286779485642910003662109375f,-0.6357999890118837260644113484886474907398223876953125f,-0.12030002684593199546725372783839702606201171875f,0.2002000011086462194498380995355546474456787109375f,0.462500008344650215263982317992486059665679931640625f,-0.396100009822845322560169734060764312744140625f,-0.5584000334501268980602617375552654266357421875f,-0.19760000241696840106442323303781449794769287109375f,-0.486199985827505454238917081966064870357513427734375f,-0.222999985150993118576678853060002438724040985107421875f,0.3882999857150017941620490091736428439617156982421875f,-0.23220000310540189758512497064657509326934814453125f,-0.296200010228157228908685283386148512363433837890625f,0.0168999990582465731137062903144396841526031494140625f,0.9795999894380571504370891489088535308837890625f,0.72859999045133605477531091310083866119384765625f,-0.425899993813037891499106990522705018520355224609375f,0.09300000053644164044186481987708248198032379150390625f,-0.1296000067979095060621830270974896848201751708984375f,-0.27299999204277991537992420489899814128875732421875f,0.057900001025199987481073549133725464344024658203125f,-0.28160002198219302727011381648480892181396484375f,-0.402099999682605291351222831508493982255458831787109375f,-0.1005000026226046117017176584340631961822509765625f,0.35800003838539140588181908242404460906982421875f,0.6506000069767237103945944909355603158473968505859375f,0.93820000402927394844709851895458996295928955078125f,0.44279998750686644370944122783839702606201171875f,-0.71459999778270721915163221638067625463008880615234375f,0.417899994915723826949260910623706877231597900390625f,0.433200002837180964121444048942066729068756103515625f,-0.034800000381469742283258028692216612398624420166015625f,0.848900009474158334654703139676712453365325927734375f,0.42779999354183673165863410758902318775653839111328125f,-0.60930000181943189119238013518042862415313720703125f,1.2214000217676161952340407879091799259185791015625f,0.9608000211238856280715481261722743511199951171875f,-0.4064000131249427827384579359204508364200592041015625f,0.48379997844696021047639078460633754730224609375f,0.10399997210502665012654688325710594654083251953125f,-0.582199978891015046400525534409098327159881591796875f,-0.40839999142885208271991359652020037174224853515625f,-0.33079999668598158990562296821735799312591552734375f,0.10459999718666079804307855738443322479724884033203125f,-0.5006000086903572565688591566868126392364501953125f,0.4956000322341918007396088796667754650115966796875f,-0.49449997785687482831917805015109479427337646484375f,0.509699975687265549595394986681640148162841796875f,0.8565000180006023811074555851519107818603515625f,0.6778000004708761849769871332682669162750244140625f,-0.629899997769296060567967288079671561717987060546875f,-0.526300015699863621421172865666449069976806640625f,-0.0506999979756772578554091523983515799045562744140625f,-0.046600000151991775865223388564118067733943462371826171875f,0.393399985654652117883500750394887290894985198974609375f,0.3654999938309189122520592718501575291156768798828125f,0.2801999881073833353894997344468720257282257080078125f,0.6882000125229357223588522174395620822906494140625f,-0.43849997514486371841258005588315427303314208984375f,0.3692000097587702089896311008487828075885772705078125f,-0.4171999621272082237055656150914728641510009765625f,0.34959998053461338596292762304074130952358245849609375f,-0.02039999830573802075406320000183768570423126220703125f,-0.089999995194375514984130859375f,0.6139999884963032172890962101519107818603515625f,-0.049199997279048002241097492515109479427337646484375f,-0.0767999901592730793709051795303821563720703125f,-0.7760000000298017397426519892178475856781005859375f,0.752099981652200266779573212261311709880828857421875f,1.185300013434886778895815950818359851837158203125f,0.0793999987602234114092425443232059478759765625f,0.2269999976754188963923297706060111522674560546875f,-0.54979999710321436623416957445442676544189453125f,-0.4267999824106694095604552785516716539859771728515625f,0.21629999293833979123746757977642118930816650390625f,-0.108900000309944289256236515939235687255859375f,0.020100000442564436298198415897786617279052734375f,0.249700002956390332276015442403149791061878204345703125f,-0.43389999293386927181614964865730144083499908447265625f,-0.63039998880624725785537521005608141422271728515625f,0.896599971988797594946163371787406504154205322265625f,0.143799979043006231904655578546226024627685546875f,0.26479999322891245583377894945442676544189453125f,-0.37159999277591715394208904399420134723186492919921875f,-1.01559999349117280331711299368180334568023681640625f,0.223399989342689675453357267542742192745208740234375f,-0.436399984142184393931529484689235687255859375f,-0.25720000832080813779612071812152862548828125f,0.131399993455409980214199094916693866252899169921875f,-0.3904999984502792731433373774052597582340240478515625f,0.5185000174641611891956927138380706310272216796875f,-0.0634000268936159017130194115452468395233154296875f,-0.8222000123813748562184855472878552973270416259765625f,0.03030000149533151887482063102652318775653839111328125f,-0.170600008429587168823360343594686128199100494384765625f,-0.1531000013329089082247946862480603158473968505859375f,-0.903199980336427454830072747427038848400115966796875f,-0.4707999807417395654596248277812264859676361083984375f,0.6273999792546034459661541404784657061100006103515625f,0.2733000133752820914878611802123486995697021484375f,-0.132800003600120675173457129858434200286865234375f,-0.4287999996811151515174742598901502788066864013671875f,0.7666999916911123591489740647375583648681640625f,0.1803999750971794213683097041212022304534912109375f,-0.751299988020956543977035835268907248973846435546875f,0.7443000055074691090339911170303821563720703125f,0.6098000066399578145137638784945011138916015625f,-0.3646999968469142316962461336515843868255615234375f,-0.74749998643994342018004317651502788066864013671875f,-0.706400001204013694433569980901665985584259033203125f,0.079399995481967966526326563325710594654083251953125f,0.2120000155568124711180644226260483264923095703125f,0.249200024175644241353211327805183827877044677734375f,0.1254999987855554000049096430302597582340240478515625f,0.282299998891353542962434630680945701897144317626953125f,0.08279999298304330235254155923030339181423187255859375f,-0.28109999536871910219559822508017532527446746826171875f,-0.1500999926567079256756187533028423786163330078125f,-0.46220001666545851293221858213655650615692138671875f,-0.4137999974831938487795923720113933086395263671875f,0.67110003485679659007701047812588512897491455078125f,0.088399986958503262712838477455079555511474609375f,-0.80780002483427526538406482359278015792369842529296875f,-0.050200001108646308267680069548077881336212158203125f,-0.2278000155210495591973085538484156131744384765625f,-0.205100000479817412557537181783118285238742828369140625f,0.171599996650218855620551039464771747589111328125f,0.3736000250220303797732412931509315967559814453125f,0.284900012297928384441547677852213382720947265625f,0.3215000180378557725902055608457885682582855224609375f,0.16100001485645798471324496858869679272174835205078125f,-0.131000001296400991890322984545491635799407958984375f,-0.7390000382661821021201831172220408916473388671875f,0.7740000457167628411525583942420780658721923828125f,0.6298999976575370141063103801570832729339599609375f,-0.4047999810472131532179673740756697952747344970703125f,-1.0676000084519383648284929222427308559417724609375f,0.39289998763278111137964287991053424775600433349609375f,0.530099976693839014529885389492847025394439697265625f,0.82170000612661198147179675288498401641845703125f,0.008600002595782196834761634818278253078460693359375f,-0.28719999245107175056546111591160297393798828125f,0.4311999795317651429371608173823915421962738037109375f,0.56459996872544326862453090143389999866485595703125f,0.5630999775096776627236749845906160771846771240234375f,0.6530999979615206285643580486066639423370361328125f,0.43579998580813406050538105773739516735076904296875f,0.434199982973933362728757856530137360095977783203125f,-0.033399986362457667610215139575302600860595703125f,-0.4764999892413612769814790226519107818603515625f,-0.4148999728471043368926984840072691440582275390625f,-1.4001000063657755845270003192126750946044921875f,0.6093999746203426770563282843795605003833770751953125f,-0.786699984054267442701302570640109479427337646484375f,-0.4343000292003154072517645545303821563720703125f,-0.241200006800890154323724345886148512363433837890625f,0.8413000109687442940042956252000294625759124755859375f,1.068500054419041500608500427915714681148529052734375f,0.120999994609505001630367360121454112231731414794921875f,0.0072000016339122951958273688433109782636165618896484375f,-0.2507999962203204535882150594261474907398223876953125f,0.736099973315000877249758559628389775753021240234375f,-0.612799988810718065934679543715901672840118408203125f,-0.732400020456314138073139474727213382720947265625f,0.49719999833703010239105424261651933193206787109375f,-0.0974999897554515992936785551137290894985198974609375f,-0.7539000205755232997262282879091799259185791015625f,0.396099995070695598542442894540727138519287109375f,-0.0991999982848763561804616983863525092601776123046875f,-0.0687999958813190204409693251363933086395263671875f,-0.53900000309944129384120969916693866252899169921875f,0.610500004373490501308197053731419146060943603515625f,0.725700041282177465973290964029729366302490234375f,0.040499985449015951655127310004900209605693817138671875f,0.76600000919401611820802600050228647887706756591796875f,0.872000047780574050193536095321178436279296875f,-0.6917999754071235685159990680404007434844970703125f,-0.54810001930743457165817744680680334568023681640625f,-0.4559000029027462375097456970252096652984619140625f,-0.3210000050961971940211014953092671930789947509765625f,0.15929999790787696678506790703977458178997039794921875f,0.36190001738667465502885534078814089298248291015625f,0.056800014746189564363021418103016912937164306640625f,-0.3057999913215636578200928852311335504055023193359375f,0.3418999868392944563311175443232059478759765625f,-0.02880003149509402504691024660132825374603271484375f,-0.577799989593029028611681496840901672840118408203125f,-0.16690001291632672320019992184825241565704345703125f,0.165800008010864363683367628254927694797515869140625f,0.321100005277991318308750123833306133747100830078125f,-0.639499982476234407613446819595992565155029296875f,-0.34279997737407708058299249387346208095550537109375f,0.5053999873906374062926261103712022304534912109375f,0.0586000172734262303464447541045956313610076904296875f,-0.235400010412931681713644138653762638568878173828125f,-0.3466000119611620799986440033535473048686981201171875f,-1.09890002593994129398424774990417063236236572265625f,-0.731400014317035385147391934879124164581298828125f,0.57090001191794870205598044776706956326961517333984375f,-0.9450999795436858530450763282715342938899993896484375f,-0.695799984019994877115777853759936988353729248046875f,0.47709998727738855972546616612817160785198211669921875f,-0.9150999956369398891098398962640203535556793212890625f,-0.757200002807378869107424179674126207828521728515625f,0.35159999471306802032444238648167811334133148193359375f,-0.4880999976262450079644850120530463755130767822265625f,-0.59460001856982713785981786713819019496440887451171875f,-0.1266000033557415083151909129810519516468048095703125f,-0.2675000037252903695161876385100185871124267578125f,0.0094000170886514577972548067918978631496429443359375f,0.47600000043958423656675904567237012088298797607421875f,-0.44149999922513938344081907416693866252899169921875f,-0.34380000258684173530809857766143977642059326171875f,0.24029999473392960052109401658526621758937835693359375f,0.22210000880956648483532944737817160785198211669921875f,-0.1626000137418508462161526040290482342243194580078125f,-0.477000000469386564727614086223184131085872650146484375f,0.7781000268697741972800940857268869876861572265625f,0.123199983763694120852960622869431972503662109375f,-0.7892000205248594912887938335188664495944976806640625f,0.8459000259995459458650657325051724910736083984375f,0.14669999526739108119954835274256765842437744140625f,0.15010000018179425751441158354282379150390625f,-0.56649999594688404958020555568509735167026519775390625f,-0.4181000246345998316854775112005881965160369873046875f,0.2865999614834791042738970645586960017681121826171875f,0.7426999985381959046293331994093023240566253662109375f,0.8191000325620176880647704820148646831512451171875f,0.374200001078843769164450350217521190643310546875f,-0.60169997950941345177255925591452978551387786865234375f,-1.22289999357461898199517236207611858844757080078125f,-0.5236999845981593626476069402997381985187530517578125f,-0.5854999960660933755463020133902318775653839111328125f,-0.7867000299870969826088185072876513004302978515625f,0.04679998866915724420323385857045650482177734375f,0.27469998898655190178175189430476166307926177978515625f,1.04469999014139158788339045713655650615692138671875f,0.5244999901503322181639532573171891272068023681640625f,-0.0204999972581862976994671043939888477325439453125f,-0.0413000053435563785342310438863933086395263671875f,0.32049999949335994386956372181884944438934326171875f,-0.4394999873191121064763819958898238837718963623046875f,0.4137000278115277041024455684237182140350341796875f,0.4266999773114921179484326785313896834850311279296875f,0.0306999979197979389056172294658608734607696533203125f,0.998299992290139215356248314492404460906982421875f,0.5735999833732841057809537232969887554645538330078125f,-0.0779000011369586076170890009962022304534912109375f,0.8153000047177076936577577725984156131744384765625f,0.6534999874755740290055427976767532527446746826171875f,-0.70019999060332771545489549680496565997600555419921875f,-0.5478000223383308053826112882234156131744384765625f,-0.25490000101029863799340091645717620849609375f,0.72859997230917239452452349723898805677890777587890625f,1.3570000094473357865609841610421426594257354736328125f,0.248299998921155395947835131664760410785675048828125f,0.2710999969333409698180048508220352232456207275390625f,0.8659000215291978719278631615452468395233154296875f,0.54669998211711645641486256863572634756565093994140625f,0.012800001290440565782091653090901672840118408203125f,-0.14599999857693912730383090092800557613372802734375f,0.57339999485611858887068592594005167484283447265625f,0.7122999842509629697673290138482116162776947021484375f,0.3959000045418736846158935804851353168487548828125f,-0.35259999876618319802901169168762862682342529296875f,-0.32649999795854089956748111944762058556079864501953125f,0.8620000271797181312649627216160297393798828125f,0.36580000950098057188597522326745092868804931640625f,-0.593500003531575170967471422045491635799407958984375f,0.6468000090837477955574286170303821563720703125f,0.3972000032544134029421911691315472126007080078125f,-0.3465000036582350784186701275757513940334320068359375f,0.32879998410940203257268876768648624420166015625f,0.33599998199939751231113405083306133747100830078125f,-0.1667999976098537562307910775416530668735504150390625f,-0.7320999927163125420292999478988349437713623046875f,-0.4287999733805651914053669315762817859649658203125f,0.5689000075742602913209111648029647767543792724609375f,0.0175999953389169849771178633091039955615997314453125f,0.321200009185075519013707889826036989688873291015625f,0.3420999920830130047733064202475361526012420654296875f,-0.5605000038146972229924358543939888477325439453125f,-0.4886000047564511561404287931509315967559814453125f,0.3004999959170817636078254508902318775653839111328125f,0.0031000063061714655532341566868126392364501953125f,-0.030999993026256333905621431767940521240234375f,0.00850000211596491084264926030300557613372802734375f,0.65040002039670952171945828013122081756591796875f,0.3344000106811524375416411203332245349884033203125f,-0.3752000037908553853327475735568441450595855712890625f,0.134900008647143909712440290604718029499053955078125f,0.145200012470781791851237585433409549295902252197265625f,0.0653999969087541155321474661832326091825962066650390625f,-1.2138999827265735120818135328590869903564453125f,-0.8902000002145769030903466045856475830078125f,0.4995999789699911985962899052537977695465087890625f,1.154599986420571600120865696226246654987335205078125f,0.6301999884799125428713750807219184935092926025390625f,-0.688099978906661302602287832996807992458343505859375f,0.737600013183057345855786479660309851169586181640625f,0.345199994440376578541673779909615404903888702392578125f,-0.5645000066719949582960680345422588288784027099609375f,-0.4647999853312967388774268329143524169921875f,-0.1395999711096285178513198843575082719326019287109375f,0.55569999423176053010564601208898238837718963623046875f,0.939200002121925336950880591757595539093017578125f,0.481399989283084916991128920926712453365325927734375f,-0.6613999962493777307059872327954508364200592041015625f,0.19860000416040435311515466310083866119384765625f,0.1079999976307153275456585106439888477325439453125f,0.1876999793529510185408071265555918216705322265625f,0.93620000534802649827526010994915850460529327392578125f,0.62500004321336799506525494507513940334320068359375f,-0.114400000548362523744572172290645539760589599609375f,0.43889998529702445306810432157362811267375946044921875f,0.04049999248981439592398601234890520572662353515625f,-0.367199963319302380426734089269302785396575927734375f,-0.7211999629914764842197882899199612438678741455078125f,-1.4603999846339217327795267920009791851043701171875f,0.13519996847510373783052273211069405078887939453125f,0.523000000685453159121607313863933086395263671875f,1.524200036096573285249178297817707061767578125f,-0.6175999687403443960675986090791411697864532470703125f,0.021099999876320385538974733208306133747100830078125f,-0.0397000031799077390815000399015843868255615234375f,0.43509998863339394148397332173772156238555908203125f,0.70509996233284510935845901258289813995361328125f,1.665500006794928822273504920303821563720703125f,0.079099989348649923925904658972285687923431396484375f,0.31670000681579091317274787797941826283931732177734375f,0.49150002723932306736287500825710594654083251953125f,0.027999987721443186927672286401502788066864013671875f,0.75120000098943695565623102083918638527393341064453125f,0.53040002933740648671800954616628587245941162109375f,-0.1268000026576221461738214202341623604297637939453125f,0.53949999536573878256007219533785246312618255615234375f,0.6887000189080836332067292460124008357524871826171875f,0.50529999927878321841490105725824832916259765625f,-0.7399999729916455493139437749050557613372802734375f,0.0787999912619596898366580717265605926513671875f,-0.0421000090144574290462742283125407993793487548828125f,0.9123999850660562405124665019684471189975738525390625f,0.285200044210255132526299348683096468448638916015625f,0.0684000001646578592584546640864573419094085693359375f,0.182499997392296886555840274013462476432323455810546875f,-0.425500008098781190568615784286521375179290771484375f,-0.1632000081270934543908879277296364307403564453125f,0.63070000548213689395282699479139409959316253662109375f,1.4051000384032732437589174878667108714580535888671875f,0.007300000530481298710583359934389591217041015625f,0.300099999213218671112457514027482829988002777099609375f,-0.444699988949298852247693503159098327159881591796875f,0.64080000115632973489709911518730223178863525390625f,0.637800007809698588090441262465901672840118408203125f,0.29710000642538059167918618186376988887786865234375f,-0.07920001314878444276246227673254907131195068359375f,-0.52949999622255561337880180872161872684955596923828125f,0.2447000107049941419745664461515843868255615234375f,-0.066000017166138125190855134860612452030181884765625f,-0.495900011806190033070151912397705018520355224609375f,-1.045999989748001013367684208787977695465087890625f,-0.3889999771714212073447924922220408916473388671875f,0.821299983662366894776596382143907248973846435546875f,-0.37710001013204463848893510657944716513156890869140625f,-0.279800013829767879602883340339758433401584625244140625f,0.070699999242275934907553391894907690584659576416015625f,0.017399972176551870006733224727213382720947265625f,0.316999999463558168599774944595992565155029296875f,0.378100009658932645351114842924289405345916748046875f,-0.0279999975562095215764202293939888477325439453125f,-0.08979999889135381607729868846945464611053466796875f,0.03609999819993969794040822307579219341278076171875f,0.49310000928640373984990219469182193279266357421875f,0.186399996733665584969230621936731040477752685546875f,-0.40270000025182961511660550968372263014316558837890625f,-0.54940001693964024553906710934825241565704345703125f,-0.028399980700015703405370004475116729736328125f,0.69830001236945393738864140686928294599056243896484375f,0.50360003455877322409151020110584795475006103515625f,-0.036000006139278895034294691868126392364501953125f,-0.66030002114623786990676990171778015792369842529296875f,-0.430299988400935973231753450818359851837158203125f,-0.18879997576475116005667587160132825374603271484375f,0.46389999900609257821315623004920780658721923828125f,0.4473999995946886798492414527572691440582275390625f,0.571400013720989363719127140939235687255859375f,0.1188999985963106187369930921704508364200592041015625f,0.62300000247359310634465145994909107685089111328125f,0.584000006079673728009993283194489777088165283203125f,-0.0797000052288174831716105472878552973270416259765625f,-0.663700004453957159711308122496120631694793701171875f,-0.226199984858930613906835560555919073522090911865234375f,0.68579999342635267201018223204300738871097564697265625f,-0.01929998986125003312963599455542862415313720703125f,-0.164999999701976651778068116982467472553253173828125f,-0.1824000000044703195811734985909424722194671630859375f,-0.19919999541640276419229849125258624553680419921875f,-0.011100004458427292775013484060764312744140625f,0.44279998258948349842967218137346208095550537109375f,-0.29859998955726629876750166658894158899784088134765625f,-0.70700000661611550611951315659098327159881591796875f,0.34820000015497232226380219799466431140899658203125f,-0.6380000041127200649526685083401389420032501220703125f,-1.187200056898594535681468187249265611171722412109375f,-0.3244999869465825526049229665659368038177490234375f,-0.8751999842330813184076987454318441450595855712890625f,-0.3656000444531439796946870046667754650115966796875f,0.1387000065959990979802540778109687380492687225341796875f,-0.76770000049397324470845660471240989863872528076171875f,-0.80490003906413953860266019546543247997760772705078125f,-0.5970999721527103787366286269389092922210693359375f,0.2903999884709715928465811884962022304534912109375f,0.3768000207066535267586004920303821563720703125f,0.180299991679191418114669431815855205059051513671875f,0.666399979932606356669566594064235687255859375f,1.06880001659393286672639078460633754730224609375f,0.141199993538856372055079191341064870357513427734375f,0.286600007230043252182127844207570888102054595947265625f,-0.022599979245662371596381490235216915607452392578125f,-0.4979999966621395657284665503539144992828369140625f,-0.004100002051889983789578764117322862148284912109375f,0.479900007939338735241108224727213382720947265625f,0.59459998437166206031179171986877918243408203125f,0.334899992851913186342471817624755203723907470703125f,0.082099985265731589834103942848742008209228515625f,-0.02800001409649865991013939492404460906982421875f,0.69460000627636875325521259583183564245700836181640625f,1.48220004885196754429443899425677955150604248046875f,-0.6594999759197239797003931016661226749420166015625f,0.462800012056529208592792201670818030834197998046875f,0.221200037944317529081672546453773975372314453125f,-0.5009999752938754102871143913944251835346221923828125f,0.60130000701993679967927164398133754730224609375f,0.278900038903952207647307659499347209930419921875f,-0.2034999958388505891804243219667114317417144775390625f,0.38559996465742629734307911348878405988216400146484375f,1.2163999785915009521630736344377510249614715576171875f,0.3006999822735789695826724710059352219104766845703125f,-0.14599999485164882884191683842800557613372802734375f,-0.179200007933378202551466529257595539093017578125f,0.1409999959319829798687351285479962825775146484375f,-0.57120000698715445164310722248046658933162689208984375f,-0.71860000088214892599580707610584795475006103515625f,-0.56499999701976744148623765795491635799407958984375f,0.26740000723153356521066825735033489763736724853515625f,-0.1228000003218649993641520268283784389495849609375f,-0.02879999036788927213592614862136542797088623046875f,0.22500000335276126861572265625f,0.630899974739551172575602322467602789402008056640625f,0.48699999052286191414395943866111338138580322265625f,-0.314799976353347277235883439061581157147884368896484375f,-0.43609997644424414176000937004573643207550048828125f,-0.158799979043007244428054036688990890979766845703125f,0.40569997828751802604330123358522541821002960205078125f,-0.06130000025853521439245241708704270422458648681640625f,0.124000009842217207012282642608624882996082305908203125f,0.15500000173226002875281892556813545525074005126953125f,0.0654000048622489504879240485024638473987579345703125f,0.00160000253617764709446191773167811334133148193359375f,-0.03940000095814466529731134869507513940334320068359375f,1.085100013332068957794263042160309851169586181640625f,0.59700000069290393156506979721598327159881591796875f,-0.65100000090524556828341928849113173782825469970703125f,0.2926999894857402750858454965054988861083984375f,0.38760001322031012449542686226777732372283935546875f,0.1239000096604228051688778577954508364200592041015625f,0.4460000046491625624867083388380706310272216796875f,0.3471999910354615082042073481716215610504150390625f,-0.3232000082015991093697948599583469331264495849609375f,0.619300002191960796693592783412896096706390380859375f,0.286999991901218998169298401990090496838092803955078125f,-0.47959999649748208394584025882068090140819549560546875f,0.21159998905062682439393029198981821537017822265625f,0.379000004589557182299586202134378254413604736328125f,0.2359999964162706742598629716667346656322479248046875f,0.35490000525712961376711973571218550205230712890625f,0.08499999970197702481300439103506505489349365234375f,-0.30969999349415300304855236390721984207630157470703125f,0.8801999969363212272810415015555918216705322265625f,0.44279999197721497949942204286344349384307861328125f,-0.5540999896094203069907280223560519516468048095703125f,-0.49889998540878277566434917389415204524993896484375f,-0.15139998093843498594424090697430074214935302734375f,0.50509998222589491678746753677842207252979278564453125f,-0.153599987769126755665638484060764312744140625f,-0.5821000099271536054601483556325547397136688232421875f,0.378899975872040162272469387971796095371246337890625f,-0.3412999926775694969904861864051781594753265380859375f,0.2026999984264374887743542785756289958953857421875f,-0.038000005900860056584633639431558549404144287109375f,0.6261999959975479423945898815873079001903533935546875f,1.0790000379681590203517771442420780658721923828125f,-0.336099996113776722239663286018185317516326904296875f,-0.1650000007078051833531162628787569701671600341796875f,-0.8933999990284444692179022240452468395233154296875f,-0.12379999454021461957609062665142118930816650390625f,0.030800002646446278387060146997100673615932464599609375f,0.457599984908104051584132321295328438282012939453125f,-0.3277999934114512736726965158595703542232513427734375f,-0.442099983645230676376769451962900348007678985595703125f,-0.650500010874122125414942274801433086395263671875f,0.076799994666874316084914653401938267052173614501953125f,0.304799998258054205546585535557824186980724334716796875f,0.412800014589727037872535220230929553508758544921875f,-0.0519000042550266105223499835119582712650299072265625f,0.79319999367296689651851693270145915448665618896484375f,0.1680000238195062944868141130427829921245574951171875f,-0.449400000995396897707223615725524723529815673828125f,0.5061999673873189831141417016624473035335540771484375f,0.4561999848961821868442712002433836460113525390625f,-0.27519997279644048404634304461069405078887939453125f,-0.80839996356368093444189071306027472019195556640625f,-1.69960001149177486468033748678863048553466796875f,0.0622999976433813440479525525006465613842010498046875f,0.137000003710389062216989941589417867362499237060546875f,-0.381799990420043366956548425150685943663120269775390625f,-0.5098999756038191666362990872585214674472808837890625f,0.5986999709635976163468740196549333631992340087890625f,0.6714999990761274517581114196218550205230712890625f,0.209899982756376335402137556229718029499053955078125f,0.443100000867247434399587291409261524677276611328125f,0.357900027251243812997927307151257991790771484375f,0.04639999315738663199226721189916133880615234375f,0.78149999158829441814333449656260199844837188720703125f,0.35670003877878198039752533077262341976165771484375f,-0.19949999126791961856497437111102044582366943359375f,0.08069999449253095757939036047901026904582977294921875f,0.58829998387098303425091216922737658023834228515625f,-0.206800000590086074936380100552923977375030517578125f,-0.90910001088082792453093361473293043673038482666015625f,-0.4473000111103058173966928734444081783294677734375f,0.04140001163482676105331847793422639369964599609375f,0.548400000853836555592124568647705018520355224609375f,-0.478399995005130307390572852455079555511474609375f,0.1302000252485271403202204965054988861083984375f,0.83339998103529211448403657414019107818603515625f,1.091900029121339432691684123710729181766510009765625f,0.5804000050857658354885870721773244440555572509765625f,-0.6848000162698328896482280470081605017185211181640625f,-0.68059999885559108179222675971686840057373046875f,-0.5257999886393545097007518052123486995697021484375f,0.356300008025765446717514350893907248973846435546875f,-0.3664999959096315595985515756183303892612457275390625f,-0.46100000636279581112120240504737012088298797607421875f,-0.111399997627735114491542844916693866252899169921875f,-0.7011000521421433262503342120908200740814208984375f,-0.052200013387203370029965299181640148162841796875f,0.7881000147253274246139653769205324351787567138671875f,-0.9415000463128091467979174922220408916473388671875f,-0.301800006699561862433256465010344982147216796875f,0.8400000206008553238490321746212430298328399658203125f,0.444399997270107149915929767303168773651123046875f,0.2107999981760979579803461092524230480194091796875f,-0.265099991650879385218786410405300557613372802734375f,0.7152999940633772979481364018283784389495849609375f,0.191799984347820640095960698090493679046630859375f,-0.6336999835550785054039124588598497211933135986328125f,0.1304999963641169102857020334340631961822509765625f,0.2884000063300131699861594825051724910736083984375f,0.2050999960467219107584924131515435874462127685546875f,-0.9373999945282935897239440237171947956085205078125f,-0.408999987006188003846318679279647767543792724609375f,0.70149998201429841770959683344699442386627197265625f,-0.2541999943733213029872786137275397777557373046875f,0.294200029242038141319426358677446842193603515625f,0.7494999816566705330700415288447402417659759521484375f,0.08780001030564310582349207834340631961822509765625f,-0.0492000052511694097034933292889036238193511962890625f,-0.2214000088781118424918759046704508364200592041015625f,1.1426000001072882383823525742627680301666259765625f,0.710799996387958543664353783242404460906982421875f,-0.6312999937579035769630309005151502788066864013671875f,0.2036000216320157374383370552095584571361541748046875f,-0.120299986016750182216128450818359851837158203125f,-0.561699979025125362142034646240063011646270751953125f,-0.6215999880447984349274292981135658919811248779296875f,-0.5748000200510023915967394714243710041046142578125f,-0.72509996434450219027212369837798178195953369140625f,0.3329000008761882067886972436099313199520111083984375f,0.9097000130891803593158329022116959095001220703125f,-0.0140000013671814915738877971307374536991119384765625f,0.7530999858170750460573117379681207239627838134765625f,0.103099994571506758944678949774242937564849853515625f,0.36869996731281329260809798142872750759124755859375f,0.292399986518919519085102365352213382720947265625f,1.1404000078797338346703327260911464691162109375f,0.5194999748393900329546113425749354064464569091796875f,-0.52540000452697255273193377433926798403263092041015625f,-0.408200037668645876465234323404729366302490234375f,-0.62989995861649550334959712927229702472686767578125f,-0.5709000152707097708315586714888922870159149169921875f,-0.2577000163078313477171832346357405185699462890625f,-0.3736999805748464353882809518836438655853271484375f,-0.48020000583976507613215289893560111522674560546875f,0.13700000363588316076857154257595539093017578125f,0.05759998967647561585181392729282379150390625f,0.5878999866828322762302150294999592006206512451171875f,0.4807000204205511550981100299395620822906494140625f,0.372499980181455470074070035479962825775146484375f,0.676099996635317612714288770803250372409820556640625f,0.382900048112869484384646057151257991790771484375f,-0.42339997846484234145236769109033048152923583984375f,0.35520000036358823081172886304557323455810546875f,0.38280002207756069765309803187847137451171875f,-0.16089998740553834721822568099014461040496826171875f,-0.5608000119969245300666216280660592019557952880859375f,0.027999941825866070388428852311335504055023193359375f,-0.81489996860027336111897966475225985050201416015625f,0.5283999793961646762596728876815177500247955322265625f,0.9859999894499775763279103557579219341278076171875f,0.17099999980628499685053611756302416324615478515625f,-0.32659999754428847840159733095788396894931793212890625f,-0.5722000266492368591997319526853971183300018310546875f,-0.2291999976515768366880365647375583648681640625f,0.579099988268315879480496732867322862148284912109375f,-0.232899989998340828378786682151257991790771484375f,0.177899973122775667544459565760917030274868011474609375f,-0.801900008669495623081502344575710594654083251953125f,-0.49900000935792920842004605219699442386627197265625f,-0.15979999203681938269028250942938029766082763671875f,0.48680000316053628761636673516477458178997039794921875f,0.985300032508373391237910254858434200286865234375f,0.5186000139951705278917870600707828998565673828125f,-0.58400001118332145466638394282199442386627197265625f,-0.1126999838232993766951040015555918216705322265625f,-0.4696000195384026909550811978988349437713623046875f,-0.4787000093713402737449769119848497211933135986328125f,-1.08649999684095366347946765017695724964141845703125f,-0.59099999034404770981154797482304275035858154296875f,0.71159998901337384591414547685417346656322479248046875f,0.0492000070393086019748807302676141262054443359375f,0.1040000163614751560459126267232932150363922119140625f,0.15520000133216382831591317881247960031032562255859375f,-0.4657000049233435134965475299395620822906494140625f,-0.03599998915195445903236759477294981479644775390625f,0.50099999809265138850378207280300557613372802734375f,-0.7657999978780747341033929842524230480194091796875f,-0.5874000004887580672630065237171947956085205078125f,0.2768999983131885667120286598219536244869232177734375f,-0.630299991381168212001284700818359851837158203125f,-0.245399990296364389763539293198846280574798583984375f,0.5048999802231788347484098267159424722194671630859375f,0.4477999996140595850846466419170610606670379638671875f,0.52740002380907535783904904747032560408115386962890625f,0.1074000067472458130879431337234564125537872314453125f,-0.44619997424185253720452237757854163646697998046875f,-0.09019996348321457357855024383752606809139251708984375f,0.612199982914328533212255933904089033603668212890625f,0.3122000095129013885753010981716215610504150390625f,0.19659999873638156486777006648480892181396484375f,-0.256600006298720817454750431352294981479644775390625f,-0.418300026898086318993819077149964869022369384765625f,-0.085799991060793223685010389090166427195072174072265625f,0.57540001993104816779833754480932839214801788330078125f,-0.7035999812126159103087275070720352232456207275390625f,-0.199199968594312881720043151290155947208404541015625f,0.69359998329877858846970184458768926560878753662109375f,-0.1695999965906143103211434208787977695465087890625f,-0.41880001868009575360929375165142118930816650390625f,-0.290700001682341113706797841587103903293609619140625f,0.8388999842166899867379470379091799259185791015625f,0.1733000300645830549228776362724602222442626953125f,-0.18569999508857737424705192097462713718414306640625f,0.02109999939203255348019183657015673816204071044921875f,0.49790001026391994543018881813623011112213134765625f,0.00969998790621762196906274766661226749420166015625f,0.943999998070299550789741260814480483531951904296875f,0.502000023901462810727025498636066913604736328125f,0.7863999819815159497693457524292171001434326171875f,-0.6579999883919953607147590446402318775653839111328125f,-0.8336000300884247593558029620908200740814208984375f,0.05450000408291799658400123007595539093017578125f,-0.44739999322444201634851879134657792747020721435546875f,0.36579998536109936679849852225743234157562255859375f,-0.124800008055567790421491736196912825107574462890625f,0.6875999980211255913076229262514971196651458740234375f,0.9996000351846221487761567914276383817195892333984375f,-0.422199981349706821021072755684144794940948486328125f,-0.5188999768033626391883217365830205380916595458984375f,-0.3864999989271158398196348571218550205230712890625f,0.36709998586177849944078843691386282444000244140625f,-0.52779999588876957972161108045838773250579833984375f,-0.03980002154111872414432582445442676544189453125f,-0.16689998326301580533481683232821524143218994140625f,-0.378300007116794290151773338948260061442852020263671875f,0.011699957120418158496022442704997956752777099609375f,-0.560100003571807558699902074295096099376678466796875f,0.72009997294992234007082743119099177420139312744140625f,0.6256999952748412141545486520044505596160888671875f,-0.234500004939734996156630586483515799045562744140625f,0.80049999323487275315613942439085803925991058349609375f,1.64210001961290874561427699518389999866485595703125f,-0.093899991965293860829433469916693866252899169921875f,-0.41929998174011717804177123980480246245861053466796875f,-0.95530001403093312006831183680333197116851806640625f,-0.2165000071413814008991494119982235133647918701171875f,0.76959997058808815584285412114695645868778228759765625f,1.5692000333771105591296191050787456333637237548828125f,-0.3203999972254034656060639463248662650585174560546875f,-0.118399999400973232610567720257677137851715087890625f,0.4687999739021062595156763563863933086395263671875f,0.645199985015392218912211319548077881336212158203125f,-0.292099998565018115215252691996283829212188720703125f,-0.228100009584426999253992107696831226348876953125f,-0.34819998361468318393008303246460855007171630859375f,-0.43640000381171706322902537067420780658721923828125f,0.5105999861523511373917472155881114304065704345703125f,0.57299999807774992799380697761080227792263031005859375f,0.049799998667836131716057934681884944438934326171875f,-0.8578999980449675266669373741024173796176910400390625f,-0.337399983054399665860501045244745910167694091796875f,0.74209999063014986386832561038318090140819549560546875f,1.20590003374814980219298377051018178462982177734375f,0.8268000218987463512121394160203635692596435546875f,-0.59850001332908864259962911091861315071582794189453125f,-0.3955000008344651263314517564140260219573974609375f,0.104600023114681306424245121888816356658935546875f,0.5754999971091747656970483149052597582340240478515625f,-0.930399995958804737483660574071109294891357421875f,-0.7684000105023383042635032325051724910736083984375f,0.279699992637336247724988425034098327159881591796875f,-0.777500035464763872283810997032560408115386962890625f,-0.23179999448060950584249439998529851436614990234375f,0.773600022600591241683787302463315427303314208984375f,1.05519999216794957419551792554557323455810546875f,0.7803999917119741613902306198724545538425445556640625f,-0.51159999560713764754638077647541649639606475830078125f,-0.165499983325600563688340116641484200954437255859375f,-0.3537999973714349710718352071125991642475128173828125f,-0.128900001427531218922695188666693866252899169921875f,-1.2503000092625615735641986248083412647247314453125f,-0.987400022840500213305858778767287731170654296875f,0.4185999913081526191405146164470352232456207275390625f,0.0617999912023545761030618450604379177093505859375f,0.25680001087188752961765203508548438549041748046875f,0.2742000074118375874121511515113525092601776123046875f,-0.28450001257658019682139638462103903293609619140625f,-0.1642000000357626277036615647375583648681640625f,0.21070001058578491370809615546022541821002960205078125f,-0.2344000056147574895248908433131873607635498046875f,-0.43180003203153649593559748609550297260284423828125f,-0.299900006374716809887104318477213382720947265625f,-0.6722999699831007092143408954143524169921875f,-0.5399999856948856091776178800500929355621337890625f,0.28019997771382332274470172706060111522674560546875f,0.34910001721382144523886381648480892181396484375f,0.1282000117778778758292901329696178436279296875f,-0.1930000019893050033914505547727458178997039794921875f,1.01319999032020557905298119294457137584686279296875f,0.60279999674558570887938913074322044849395751953125f,-0.594899966029823534796605599694885313510894775390625f,0.55049998775869612988032031353213824331760406494140625f,0.30090002134293314384194673039019107818603515625f,-0.2024999918788672470526535107637755572795867919921875f,-0.12430000081360337294000828478601761162281036376953125f,0.61929999142587188742936632479541003704071044921875f,-0.3994000015914433987518350477330386638641357421875f,0.4869999976009131703591492623672820627689361572265625f,-0.4510000186562539425949580618180334568023681640625f,-0.6270999854147429886097597773186862468719482421875f,-0.4570999924927949820130379521287977695465087890625f,-0.3775000274181365966796875f,0.555999995708464922472558100707828998565673828125f,-0.6076999711945656468259358007344417273998260498046875f,0.0131000146508224446506574167869985103607177734375f,-0.324699997293949049748107427149079740047454833984375f,0.0043999919429420497607452489319257438182830810546875f,0.7288000221073627216128443251363933086395263671875f,-0.47819997702091943114766081635025329887866973876953125f,0.07940000252276657732863895944319665431976318359375f,-0.93979999781101941636052288231439888477325439453125f,0.54929997175633904493707859728601761162281036376953125f,0.3329000068366525511009967885911464691162109375f,0.7213000269502405359389740624465048313140869140625f,0.65510000264048517948367589269764721393585205078125f,-0.8715999951973556481021887520910240709781646728515625f,-0.0752000231623650705614636535756289958953857421875f,-0.42599998915195502746655620285309851169586181640625f,0.44400000965595209923009178965003229677677154541015625f,0.58560004713535374065713767777197062969207763671875f,0.043699992197752113298747644876129925251007080078125f,0.11589999202489853491471194502082653343677520751953125f,0.45710001029968250207957680686376988887786865234375f,0.580900003647804030748602599487639963626861572265625f,-0.829400001405179221336538830655626952648162841796875f,-0.998200042176247137604150339029729366302490234375f,0.390199985462427179072619765065610408782958984375f,-0.0338999996766446987095378062804229557514190673828125f,0.6320999801993369970887215458787977695465087890625f,0.136899981623888056248006250825710594654083251953125f,0.84399999345093956737429152781260199844837188720703125f,0.97400003111362476460044490522705018520355224609375f,-0.06100000420212747798132113530300557613372802734375f,-0.28320000451356175386052882458898238837718963623046875f,-0.8953000196933746934746523038484156131744384765625f,-0.5836000080347059082441774080507457256317138671875f,0.4660000103116035408135076067992486059665679931640625f,0.2640999933347105699255052968510426580905914306640625f,0.42660001117885115728967093673418276011943817138671875f,0.16020000274777412840876422706060111522674560546875f,-0.1978999828457830290062702260911464691162109375f,-0.2921999880552288431090346421115100383758544921875f,0.0612000002816319721432591904886066913604736328125f,-0.163100019121170447533586411736905574798583984375f,0.2226000239491465748642440303228795528411865234375f,0.55890001923441889886134958942420780658721923828125f,0.34110000699162501547334613860584795475006103515625f,0.469200022089481638687402664800174534320831298828125f,0.1601999996930361103153472868143580853939056396484375f,0.417399978137016347545795724727213382720947265625f,0.337599979543686146143954829312860965728759765625f,-0.2703999898493290032774893916212022304534912109375f,0.245900011247396577118706773035228252410888671875f,0.014599997782707152538250738871283829212188720703125f,-0.26090000461637974371598147627082653343677520751953125f,0.6007999987721444057342523592524230480194091796875f,0.046599981451034722113035968504846096038818359375f,-0.6123999898716807610554724305984564125537872314453125f,-0.187299989056587179447888047434389591217041015625f,-0.2039999846220013068887055851519107818603515625f,0.14640000470578673485988474567420780658721923828125f,-0.6026000083029270371071106637828052043914794921875f,-0.3063999953925609087690418164129368960857391357421875f,0.44800000314414500923732020964962430298328399658203125f,0.4951999917209146673258146620355546474456787109375f,0.2719999933540819814226097150822170078754425048828125f,-0.27799999271333220018931342565338127315044403076171875f,-0.488300017249584161760367351234890520572662353515625f,0.10840001854896552657692154753021895885467529296875f,0.78830000439733272088460580562241375446319580078125f,-0.75090001109838500070736699854023754596710205078125f,-0.48740000019073459469609588268212974071502685546875f,0.38780000907629730289016833921778015792369842529296875f,0.52530003422200710616607466363348066806793212890625f,0.15279999831020829592631571358651854097843170166015625f,-0.6019000201433897245806292630732059478759765625f,-1.0487000298976900669600809123949147760868072509765625f,0.2470000008046626671642798100947402417659759521484375f,-0.44990001062154760802513919770717620849609375f,0.018900019085407393504283390939235687255859375f,-0.07549999785423278098051014239899814128875732421875f,0.5038000106707214431622787742526270449161529541015625f,0.576799989973007853905073716305196285247802734375f,0.425500011302530734003113366270554251968860626220703125f,0.34689999250173553235754297929815948009490966796875f,-0.724900012707710317272358224727213382720947265625f,-0.100699997174739763750039855949580669403076171875f,-0.7262999797880649310855005751363933086395263671875f,0.03470000526607019963876155088655650615692138671875f,0.055300000619888212138874905576813034713268280029296875f,0.206700000333785993689161841757595539093017578125f,-0.6956999772071839061027276329696178436279296875f,-0.469199999588727933996779029257595539093017578125f,-0.04679996259212515496983542107045650482177734375f,0.7743999922037119176820851862430572509765625f,0.061399997329711908378158113919198513031005859375f,-0.43489998767375936949974857270717620849609375f,-0.1150999342441565431727212853729724884033203125f,-0.46549998906254774766466653090901672840118408203125f,-0.8668999796867371543385161203332245349884033203125f,0.66529999883174895103365997783839702606201171875f,0.063599995219707494698013761080801486968994140625f,-0.769400016343593673440182101330719888210296630859375f,0.1004000029623508538634268916212022304534912109375f,0.1712000077694655342241958351223729550838470458984375f,1.108200020420551101807404847932048141956329345703125f,-0.5354000312745572642825209186412394046783447265625f,-0.1618000075191259323759140897891484200954437255859375f,0.072499979138374470721828402020037174224853515625f,-0.96729997951984358905974659137427806854248046875f,-0.439500002294778813194398026098497211933135986328125f,-0.7069999961853028480618377216160297393798828125f,-0.355799988937377520414884202182292938232421875f,-0.4692000089764594150665288907475769519805908203125f,0.65550000858306844975231797434389591217041015625f,-1.3354999769926081398807582445442676544189453125f,-0.58809998410344110908454240416176617145538330078125f,-0.99159997698068647054014945751987397670745849609375f,0.544599991226196067373166442848742008209228515625f,0.07449999871850021548880249611102044582366943359375f,0.46279999734163279612175756483338773250579833984375f,-0.0242999932885172853502808720804750919342041015625f,0.41660002123713513189073864850797690451145172119140625f,-0.16820000581741323397721998844644986093044281005859375f,-0.5021000195384026643097286068950779736042022705078125f,0.4601999805450436298315253225155174732208251953125f,-0.8783999947071077230020819115452468395233154296875f,-1.2194999940991397835432508145458996295928955078125f,0.71880002225637451562789692616206593811511993408203125f,-0.42380000556707375647391700113075785338878631591796875f,-0.927800022152066361513789161108434200286865234375f,0.42260001038908967530005611479282379150390625f,-0.75340000796914097236367524601519107818603515625f,-0.5623000277996066387231621774844825267791748046875f,-0.204600005978345933499440434388816356658935546875f,0.3106000055611133348065777681767940521240234375f,-0.230299986016749613781939842738211154937744140625f,-0.51499999240040761350201137247495353221893310546875f,0.09519998456835754296889717807061970233917236328125f,0.138099983656406521959070232696831226348876953125f,-0.2527999992787837157948160893283784389495849609375f,0.5555999972164631373061638441868126392364501953125f,0.10760003288984254510296523221768438816070556640625f,0.345600005859136416574983741156756877899169921875f,0.6723999940812586117999671841971576213836669921875f,-0.2731999910652636032182272174395620822906494140625f,-0.59760000308752037057047346024774014949798583984375f,-0.38140000060796719338895854889415204524993896484375f,0.6383999932169910351831276784650981426239013671875f,0.33159998860359163330713272443972527980804443359375f,-0.84960001745224023039781968691386282444000244140625f,-0.3590000239610677823520745732821524143218994140625f,0.1094000017404557212330473703332245349884033203125f,0.41260000874996194397681392729282379150390625f,-0.3704999535977844260514757479541003704071044921875f,-0.27959999882578845387115507037378847599029541015625f,0.57879999468922616046029361314140260219573974609375f,0.3486000199556347212137552560307085514068603515625f,-0.6434000067770480058015891700051724910736083984375f,0.8532000187814237079919621464796364307403564453125f,1.6619000264763830188030624412931501865386962890625f,0.50000000104308117698792557348497211933135986328125f,-0.25619999309182173163890183786861598491668701171875f,-0.1500000116229056601468982989899814128875732421875f,-0.000400001174211350729592595598660409450531005859375f,-0.2663999973297119794324316899292171001434326171875f,-0.44950001205503919976536053582094609737396240234375f,0.4483000192612411893833268550224602222442626953125f,0.07810000064373012751417491017491556704044342041015625f,-1.293599984192848495467842440120875835418701171875f,-0.0581999891281128611808526329696178436279296875f,-0.291900001144409060316320392303168773651123046875f,-0.841100006917118836469171583303250372409820556640625f,0.67250002332031755258867633529007434844970703125f,0.05219999856054780007497129190596751868724822998046875f,-0.457499992251396037090671597979962825775146484375f,0.0327000282287599475239403545856475830078125f,-0.31190000516772276029087151982821524143218994140625f,-0.43610002353191390511710778810083866119384765625f,-0.09790000340938576073313015513122081756591796875f,0.186300010782480374160741121158935129642486572265625f,-0.068700035262107661537811509333550930023193359375f,-0.6764999810457226203652680851519107818603515625f,-0.104199993777275068396193091757595539093017578125f,-0.0408999964952467820467063575051724910736083984375f,0.03589998815059658454629243351519107818603515625f,0.291300001692771814987281686626374721527099609375f,-0.278900028324127191581283113919198513031005859375f,0.965899961030483922286293818615376949310302734375f,0.5996999949097634452499505641753785312175750732421875f,0.60980002183914194802127894945442676544189453125f,-1.0961999673128133281352347694337368011474609375f,-0.52629999617934242195360639016143977642059326171875f,-0.21999998390674591064453125f,-0.1524000043630593381749349646270275115966796875f,-0.43340001974105835103046047152020037174224853515625f,-0.486999978899955721090009319595992565155029296875f,0.424399994289874626929304213263094425201416015625f,0.3674000048100947157792006692034192383289337158203125f,0.26229999561309824684940394945442676544189453125f,0.033899982428550856639049015939235687255859375f,0.2646999990820884107733945711515843868255615234375f,1.795700027275085375322305480949580669403076171875f,-0.19090002570152364569366909563541412353515625f,0.15780000330209720260654648882336914539337158203125f,0.06490001270771017516381107270717620849609375f,0.5380999934911727677899762056767940521240234375f,-0.00860000579953190680271291057579219341278076171875f,-0.5394999827742577025446735206060111522674560546875f,0.5664000116348262281462666578590869903564453125f,0.57540001831054699010792319313623011112213134765625f,0.06940000174045568570591058232821524143218994140625f,-1.00389999941587415577259889687411487102508544921875f,-0.6679999944269656708684124168939888477325439453125f,0.7621999753892421125556211336515843868255615234375f,1.4659999828934662247093001496978104114532470703125f,-0.973400032705068696259331773035228252410888671875f,-0.211999984115362138936689007095992565155029296875f,1.272900011456012947519411682151257991790771484375f,0.60220002068877231948818007367663085460662841796875f,-0.14100000062584872040361005929298698902130126953125f,-0.0765000293254853858115893672220408916473388671875f,0.637400016433000704552114257239736616611480712890625f,-0.435800003093480992077957125729881227016448974609375f,-1.00930000669956232428603470907546579837799072265625f,0.37739998946189867723433053470216691493988037109375f,-0.34999998748302463269510553800500929355621337890625f,-0.75609997465610501166111134807579219341278076171875f,0.12540000773072257356233194514061324298381805419921875f,-0.52560000466704359656233691566740162670612335205078125f,-0.54690001388490205869175042607821524143218994140625f,-0.5338000083982945564997635301551781594753265380859375f,0.5983999921739100447126702420064248144626617431640625f,0.924700004893541205319706932641565799713134765625f,-0.6732000228047372303308293339796364307403564453125f,-0.0305999949812889582290154066868126392364501953125f,0.84460000553131120426542111090384423732757568359375f,0.49060000436902040377162848017178475856781005859375f,-0.09339998263716697834979640902020037174224853515625f,-0.01550001662969524574009483330883085727691650390625f,0.62960001685619371158963986090384423732757568359375f,-0.45360000058412541790175964706577360630035400390625f,-0.996400005525350795920758173451758921146392822265625f,-0.18540001309514053673410671763122081756591796875f,-0.227999998599290876200029742904007434844970703125f,-0.6362999802350994826838359585963189601898193359375f,0.31780002356767678151072686887346208095550537109375f,-0.24200000125169740528008333058096468448638916015625f,0.01489995667934440604085466475225985050201416015625f,0.24740000704526909913738563773222267627716064453125f,0.09079999996423726571492807124741375446319580078125f,-0.58189998028278377972810631035827100276947021484375f,-0.4771999914824960331571901406277902424335479736328125f,0.242499985247850435854388706502504646778106689453125f,-1.0664000205755232997262282879091799259185791015625f,0.80860001101493850228507653810083866119384765625f,0.35900000682473187652021806570701301097869873046875f,1.536099976444244674667061190120875835418701171875f,-0.30910003390312112969695590436458587646484375f,0.11659998666644089126975813996978104114532470703125f,1.25310003461837737859241315163671970367431640625f,-1.472299995613099099500686861574649810791015625f,-0.610799994897842424279588158242404460906982421875f,0.19609999909400954720695153810083866119384765625f,0.7771000037431718965308391489088535308837890625f,0.08929999403357502529843259253539144992828369140625f,-0.985299995553493346278628450818359851837158203125f,0.53730002392530451515995082445442676544189453125f,0.032099999719858107738446051371283829212188720703125f,-0.7819999711513521134520487976260483264923095703125f,0.23860000386238056080401293002068996429443359375f,-0.021099998497962957344498136080801486968994140625f,-0.602400000712275396352879397454671561717987060546875f,-0.2393999672442674864214495755732059478759765625f,-0.24449999670684341435134001585538499057292938232421875f,0.90269998754858971068415485206060111522674560546875f,-0.275900031512975374425877816975116729736328125f,0.1213999947965145320694091424229554831981658935546875f,0.8058000121831891959800486802123486995697021484375f,-0.93820000641346013026122818700969219207763671875f,-0.33789999908804890793589947861619293689727783203125f,0.2191000017166135904744805884547531604766845703125f,0.65249999642372102925946819595992565155029296875f,0.08070000209212313535545035847462713718414306640625f,-0.96979999263286575938991518341936171054840087890625f,0.514200027453899366491896216757595539093017578125f,0.015800001305341648816238375729881227016448974609375f,-0.900300001811981331911738379858434200286865234375f,-0.49049998652935045129197533242404460906982421875f,-0.118199992108344975605405124952085316181182861328125f,0.1961999822139741667115231393836438655853271484375f,0.29480002422332773903690394945442676544189453125f,-0.07549999785423278098051014239899814128875732421875f,-0.6641999943733214450958257657475769519805908203125f,0.37840000424385067390176118351519107818603515625f,0.21770000200271599766210783855058252811431884765625f,0.41129997919201866096727826516143977642059326171875f,0.48719996764063910887898600776679813861846923828125f,0.359399996823072331864068473805673420429229736328125f,-0.156799995225668009624087062547914683818817138671875f,-0.231899992352724293454002690850757062435150146484375f,-0.34780000360012053306490997783839702606201171875f,0.3944000038266182173174456693232059478759765625f,0.446100008779764412025770070613361895084381103515625f,-0.13139999595135443133386843328480608761310577392578125f,0.23199999745190147404372282835538499057292938232421875f,-0.199299993195384550492832431700662709772586822509765625f,-0.7422000349938873142718875897116959095001220703125f,0.0553999791204926594900825875811278820037841796875f,0.232100036525725528235852834768593311309814453125f,-0.6595999949514863569532963083474896848201751708984375f,0.0427999909341336692847335143596865236759185791015625f,0.713799982470273874923805124126374721527099609375f,0.16079999206662165533998631872236728668212890625f,-0.4444000063598156913258208078332245349884033203125f,-1.0703999907433985327998016146011650562286376953125f,0.6662000073969363000969678978435695171356201171875f,0.2550000120699404959623279864899814128875732421875f,-0.117300003957748089078449993394315242767333984375f,-0.05199999789893627877290782635100185871124267578125f,-0.064199996981024742837007579510100185871124267578125f,0.52359997148960868429412585101090371608734130859375f,0.482600004875660015812854908290319144725799560546875f,-0.834599998080730376415203863871283829212188720703125f,-1.1183000083088874276882052072323858737945556640625f,-0.35219999654889111440070337266661226749420166015625f,0.60399998566508283914799903868697583675384521484375f,1.22429998017549479527588118799030780792236328125f,0.1582000001549719314652975299395620822906494140625f,0.4239999962449072512526981881819665431976318359375f,0.71349999809265085559673025272786617279052734375f,0.2118000047624111203958818805404007434844970703125f,-0.3909999940693378306377780972979962825775146484375f,-0.11090003374814916270452158642001450061798095703125f,0.4570000055730343291315875831060111522674560546875f,-0.9896000080645084295838387333787977695465087890625f,-0.6799000273108486425144292297773063182830810546875f,-0.6278000224500897008539368471247144043445587158203125f,0.7597999952405689594314708301681093871593475341796875f,1.076500022657215449584100497304461896419525146484375f,0.611800023984909291385747565072961151599884033203125f,-0.0526999929130076427696849350468255579471588134765625f,-0.12049997907876974778673684340901672840118408203125f,0.395699991512298510087930480949580669403076171875f,0.03039999431967732590464947861619293689727783203125f,0.38099999153614039215653974679298698902130126953125f,-0.190000008642673634540187777020037174224853515625f,0.104700004893541365191822478664107620716094970703125f,-0.35180001809895034003972114078351296484470367431640625f,-0.043999995835125371712592823314480483531951904296875f,0.149300006308406618682482758231344632804393768310546875f,-0.0456999861478806224113213829696178436279296875f,-1.030700030851363635520101524889469146728515625f,-0.2109000130355358493261519470252096652984619140625f,0.63069998599886911705425518448464572429656982421875f,-0.8768999795377254713457659818232059478759765625f,-0.2648999952733517293523846092284657061100006103515625f,-1.287100005531311097684010746888816356658935546875f,-0.174299986732005862677397090010344982147216796875f,-0.1032999981760978069900147602311335504055023193359375f,0.1254999996423720176608185283839702606201171875f,1.151299974870681808170047588646411895751953125f,0.3846999962508679260508870356716215610504150390625f,0.675099989527463861804790212772786617279052734375f,-0.07729999985992908051457561668939888477325439453125f,-0.134400000250339435314828051559743471443653106689453125f,-1.1545999885439872656434090458787977695465087890625f,0.0148000382304189770366065204143524169921875f,-0.269999996274709719301654331502504646778106689453125f,1.382499981522560261737453402020037174224853515625f,0.02069996469020907170488499104976654052734375f,0.13479998786449431946721233543939888477325439453125f,0.96660000946521762443808256648480892181396484375f,0.59759995868206061686578323133289813995361328125f,0.233299984765052759172476726234890520572662353515625f,-0.0019000148534775718189848703332245349884033203125f,0.02150001680850976981673738919198513031005859375f,-0.1900999957859514921665322617627680301666259765625f,-0.569099982643127333403754164464771747589111328125f,-0.096099968099593979786732234060764312744140625f,-0.44660000708103186894959435448981821537017822265625f,1.0404999971464274910459835155052132904529571533203125f,-0.2853000532582399273451301269233226776123046875f,0.182699991311132958760055089442175813019275665283203125f,-0.9948000126004217236186377704143524169921875f,0.40829994475543518461790881701745092868804931640625f,-0.2443999904155731428545550443232059478759765625f,0.51739998350143423522240482270717620849609375f,0.891799965274334027753866394050419330596923828125f,0.1386000081464648925333449369645677506923675537109375f,0.64480001271218068126245270832441747188568115234375f,0.4849000132106244809193640321609564125537872314453125f,-0.3849999834969637380055473840911872684955596923828125f,0.4307999911352993205326811221311800181865692138671875f,0.6350999836973845180665421139565296471118927001953125f,-0.74520003493428266239106960711069405078887939453125f,-0.08880000541806243319342684117145836353302001953125f,-0.031399981832503698342407005839049816131591796875f,-0.13300000753998764135843657641089521348476409912109375f,-0.23220000608563429178587966816849075257778167724609375f,-0.0797000021368265976207112544216215610504150390625f,-0.32319999091625195575261386693455278873443603515625f,0.42639999464750300006699035293422639369964599609375f,0.245600003772973973781290624174289405345916748046875f,-0.130999993920326129881459564785473048686981201171875f,0.363799999904632631597678482648916542530059814453125f,0.26050000590085975460397094138897955417633056640625f,-0.2077999974906443725330973393283784389495849609375f,0.196200013953447438552757375873625278472900390625f,0.8712999824702745144122673082165420055389404296875f,0.5275999936997888273282342197489924728870391845703125f,-0.8830000170767309253250232359278015792369842529296875f,-1.1826000097930435828175177448429167270660400390625f,-0.888800001841783426925758249126374721527099609375f,-0.232000017195939989278485882095992565155029296875f,1.0503999497652056760443883831612765789031982421875f,0.18239999750852586846150416022283025085926055908203125f,-0.44959999212026591219881765937316231429576873779296875f,-0.508399997761845412469483562745153903961181640625f,-0.04159999665021896220196140347979962825775146484375f,-0.06119999473094939190787044935859739780426025390625f,0.561599966251850712950499655562452971935272216796875f,-0.4846000189423562698465275389025919139385223388671875f,0.2308000027954577593192198037286289036273956298828125f,0.4493000195890666237374944103066809475421905517578125f,-0.469600010299682679715260746888816356658935546875f,0.124200004506111127966505591757595539093017578125f,1.02799998891353627783473712042905390262603759765625f,-0.49800002512335816984290204345597885549068450927734375f,-0.55100001075863824784306643778108991682529449462890625f,1.00359995915889754769523278810083866119384765625f,0.08839997444152913885773159563541412353515625f,0.1546999920785427917735432856716215610504150390625f,0.629500005498528469871644119848497211933135986328125f,0.707299978141486374028090722276829183101654052734375f,0.41160000141859054423321140347979962825775146484375f,0.60909997427985096418723287570173852145671844482421875f,-0.2022999930612741081858985126018524169921875f,0.0916000039331614501758593860358814708888530731201171875f,-0.378300015163421932129494962282478809356689453125f,-1.161100010418891770314075984060764312744140625f,-0.27249999642372113584087855997495353221893310546875f,-0.139999999180436152101947300252504646778106689453125f,-0.368599996933340889881947077810764312744140625f,-0.18600000248849390960259597704862244427204132080078125f,0.12379998142719284004442670266143977642059326171875f,-0.148199996578693315996133605949580669403076171875f,0.162600005620717968923827356775291264057159423828125f,0.24039999417066582054758328013122081756591796875f,0.0905999955773353349286480806767940521240234375f,0.17340000230669971870156587101519107818603515625f,1.031299996328353785202125436626374721527099609375f,-0.089699999678135355907215853221714496612548828125f,-0.0514999940097331521116075236932374536991119384765625f,-1.21289999163746831101207135361619293689727783203125f,0.04070000894665781743242405354976654052734375f,-0.2808000127792358302514230672386474907398223876953125f,1.000899965500831712006402085535228252410888671875f,-1.3814999792575832771035493351519107818603515625f,-0.567899998641014036593333003111183643341064453125f,-0.533899986675381743594925865181721746921539306640625f,-0.097899978896975881070829927921295166015625f,-0.18149999512732029049999482595012523233890533447265625f,0.72979998488426200964340750942938029766082763671875f,-0.05659999814033511711386381648480892181396484375f,-0.1347999984443186693994221059256233274936676025390625f,0.340700004476308659917549448437057435512542724609375f,-0.5079000165224076823733412311412394046783447265625f,-0.047399999073147736083910785964690148830413818359375f,1.1248000021696089589795519714243710041046142578125f,-0.51240001032352466836528037674725055694580078125f,-0.19930000088810917446835446753539144992828369140625f,0.850999996528029267750525832525454461574554443359375f,-0.6090999605894094059976850985549390316009521484375f,0.08280000769793982584587865858338773250579833984375f,0.406800001856684634304883729782886803150177001953125f,0.629799974937737427893580388627015054225921630859375f,0.09320000328421595980898928246460855007171630859375f,0.28660000603795054985312162898480892181396484375f,-0.344199983569979384157022650470025837421417236328125f,0.98579999623894654092737255268730223178863525390625f,0.36060000377297374285490150214172899723052978515625f,-1.064699964213371430332699674181640148162841796875f,-0.20160000052452087260235202847979962825775146484375f,0.20340000066757202290546047152020037174224853515625f,0.58599999243021017036880948580801486968994140625f,0.657600002938508954031249231775291264057159423828125f,-0.159199978131055797092585635255090892314910888671875f,-0.34999999433755846212079632095992565155029296875f,0.3175999822258948057651650742627680301666259765625f,-0.6309999775290489054668796597979962825775146484375f,-0.9423999677062031565810684696771204471588134765625f,-0.304200010168552470446456936770118772983551025390625f,0.846200017678737648196829468361102044582366943359375f,0.6259000379204753272688321885652840137481689453125f,-0.459199977385997382128834942704997956752777099609375f,0.874400002634525552736022291355766355991363525390625f,1.2031999906182289095113446819595992565155029296875f,-0.2784000082671642672949019470252096652984619140625f,-0.4970000121295452544245563331060111522674560546875f,-0.3560000088214874125469577847979962825775146484375f,0.316200015443563575701091394876129925251007080078125f,-0.007000004082918120928979988093487918376922607421875f,-0.35790001354217526596812604111619293689727783203125f,-0.84119999428391434292962003382854163646697998046875f,-0.33540002203583707540701652760617434978485107421875f,0.736199949729442693069358938373625278472900390625f,-0.642200021210312943509279648424126207828521728515625f,-0.084599986830353690603345739873475395143032073974609375f,0.502500017248094454913598383427597582340240478515625f,0.58760000696182235202513766125775873661041259765625f,0.19979999680519089366725893341936171054840087890625f,-0.0602000094532968432758934795856475830078125f,-0.89040001965165149755421225563623011112213134765625f,-0.28600001493096360860590721131302416324615478515625f,1.0391999776840208369321771897375583648681640625f,-0.121800001111626698957479675300419330596923828125f,0.622799990487098664715404083835892379283905029296875f,0.82289997614026066941050885361619293689727783203125f,0.46729997780621079783713867072947323322296142578125f,0.211699997353553881485055399025441147387027740478515625f,-0.44500000864267352795877741300500929355621337890625f,0.3125999981701372831821572617627680301666259765625f,0.27270000706911090571082922906498424708843231201171875f,-0.989699991035461579258480924181640148162841796875f,1.6148999495267872816839371807873249053955078125f,0.6860999882161618046438888995908200740814208984375f,0.656399996137619012870345613919198513031005859375f,-0.10000000894069671630859375f,-0.00370000113844870526236263685859739780426025390625f,-0.1372999750494958703939119004644453525543212890625f,0.8218999990582460668520070612430572509765625f,0.132300001871585859447577604441903531551361083984375f,-1.022999959111213996720834984444081783294677734375f,-0.0665999802589425371479592286050319671630859375f,-0.203899991816282266654525301419198513031005859375f,0.5858000059247014945640330552123486995697021484375f,0.213199968564509845236898399889469146728515625f,0.400600006753206283605095450184307992458343505859375f,1.2729999796748163731763270334340631961822509765625f,0.0553999995350835661156452260911464691162109375f,0.291800008189678106163000848027877509593963623046875f,-0.541700008678436262243849341757595539093017578125f,-0.79569997899532296514735207892954349517822265625f,-0.369599999123811695511676589376293122768402099609375f,-0.324700014579295981320683495141565799713134765625f,0.42870000814199471506071859039366245269775390625f,0.138500000625848773694315241300500929355621337890625f,0.620099963450432056788486079312860965728759765625f,-1.4126999644517894694217829965054988861083984375f,-0.5795999967396259222596199833787977695465087890625f,0.3746000162228939611708256052224896848201751708984375f,-0.96079999098628832143731415271759033203125f,-0.316599990577995760165919136852608062326908111572265625f,0.170799997282028215295213158242404460906982421875f,1.0011999685049062236430472694337368011474609375f,0.3158999874055385959081831970252096652984619140625f,0.991699996757507307165724341757595539093017578125f,-1.423499998688697587567730806767940521240234375f,-0.60370000635385512310904232435859739780426025390625f,0.51599997827410692963212568429298698902130126953125f,-0.1075999849081039627662903512828052043914794921875f,0.5902000127315520927595571265555918216705322265625f,0.2341999872207640720489507657475769519805908203125f,-0.72289998791217868756575626321136951446533203125f,-0.352800009858608376589472754858434200286865234375f,-0.632200026500225220615902799181640148162841796875f,0.313599966013430986322418903000652790069580078125f,0.38299999081343394369270072274957783520221710205078125f,-0.20439999019205579156022167808259837329387664794921875f,-0.455699985942989493903354514259262941777706146240234375f,0.07219999550580968872282028314657509326934814453125f,-0.48680001713037501787084693205542862415313720703125f,-0.603100005561113761132219224236905574798583984375f,-0.3397999944210052802873178734444081783294677734375f,0.7991999848365782810333257657475769519805908203125f,0.6092999944806098966409990680404007434844970703125f,-0.287199996772408316925151439136243425309658050537109375f,-0.236000000849366065036605277782655321061611175537109375f,0.106799987886846192708389935432933270931243896484375f,-0.4292000076353550408470027832663618028163909912109375f,-0.4672000197947026034483997136703692376613616943359375f,0.0921999774754045375857458566315472126007080078125f,0.83960001775026338322049923590384423732757568359375f,0.42460002609491365177518673590384423732757568359375f,-0.982999960601329547671412001363933086395263671875f,-0.673200007605552652734104412957094609737396240234375f,0.379599994206428537069086814881302416324615478515625f,0.713800008547305964157203561626374721527099609375f,0.0212000000953673772841057143523357808589935302734375f,0.5266000005245208281934310434735380113124847412109375f,0.1998000182628629772807471454143524169921875f,-0.560600008764863044774529043934307992458343505859375f,0.376599997097253780253822696977294981479644775390625f,0.59450000911206013487486643498414196074008941650390625f,-0.19920001940727249944984578178264200687408447265625f,0.97000002026557918810567571199499070644378662109375f,0.234000078797340194114440237171947956085205078125f,-0.6759999870657917853122853557579219341278076171875f,0.1175999863982202242596031283028423786163330078125f,0.95079996598958960163372466922737658023834228515625f,0.004399999952316324680623438325710594654083251953125f,-0.304799992036819400453850903431884944438934326171875f,0.08699998366832772944690077565610408782958984375f,0.2900000083446503396089610760100185871124267578125f,0.009800009322166403080700547434389591217041015625f,-0.70760000338554362997456337325274944305419921875f,0.9725999807357794679774087853729724884033203125f,0.43559998976588243380092535517178475856781005859375f,0.32419999616146100152036524377763271331787109375f,-0.94419999437332080560736358165740966796875f,-0.5801000148594379624000794137828052043914794921875f,-0.39959998042285460595479662515572272241115570068359375f,0.376999991305172255096067601698450744152069091796875f,0.036899997661262784465208142137271352112293243408203125f,-0.706000001072883520691902958787977695465087890625f,-0.294599991524219007033025263808667659759521484375f,-0.2253000074744224701106531938421539962291717529296875f,0.552799960684776436892207129858434200286865234375f,0.12579996867179943365044891834259033203125f,0.36049999606609350877306496840901672840118408203125f,0.0349999951943755771566202383837662637233734130859375f,-0.58179997298568508767857565544545650482177734375f,-0.260199993099272275376421248438418842852115631103515625f,-0.02330001233220091449993560672737658023834228515625f,-0.95729998533129645466033252887427806854248046875f,-0.4575000049173831850879423654987476766109466552734375f,-0.957399984693527272838764474727213382720947265625f,1.453000000834464344734442420303821563720703125f,0.67570001222491260506330945645458996295928955078125f,0.5847999917387962653947397484444081783294677734375f,-0.6556000080943107377606793306767940521240234375f,-0.038399998283386249653403865522705018520355224609375f,0.21199999991059303994234142010100185871124267578125f,0.425599973970651745958093670196831226348876953125f,0.337699996936321245044609895558096468448638916015625f,0.4799999904632568092921474089962430298328399658203125f,-0.50260001150667665825721996952779591083526611328125f,0.00940000207573177259945396144757978618144989013671875f,0.102400032973289256688076420687139034271240234375f,0.4223999554872506223546224646270275115966796875f,0.5722000038504599928046445711515843868255615234375f,0.9273999739646914264312727027572691440582275390625f,-0.82699997889995557898146216757595539093017578125f,-0.25259999653100972949459901428781449794769287109375f,0.1528000015139578948719645268283784389495849609375f,-0.20439998356103927790172747336328029632568359375f,-0.219799995315074880863903672434389591217041015625f,-0.4728000018119811187489176518283784389495849609375f,-0.5161000160813333792475532391108572483062744140625f,-0.93300002363324185949977618292905390262603759765625f,-0.15920001657605187261879109428264200687408447265625f,1.0106999804854392976949384319595992565155029296875f,-0.044600009070336898986397500266321003437042236328125f,-0.4258000136360526877155052716261707246303558349609375f,-0.6298999880276625606967400017310865223407745361328125f,0.5699999882280823992886098494636826217174530029296875f,-0.4207999871492387189420014692586846649646759033203125f,-0.6077999922007319977268480215570889413356781005859375f,-0.31920001136064524160929067875258624553680419921875f,-0.44700000125169747633435690659098327159881591796875f,-0.4207999802947046674717057612724602222442626953125f,-0.6712000026285647180657178978435695171356201171875f,-0.0516000197470187771386918029747903347015380859375f,0.263999998331069463120002183131873607635498046875f,-0.45139999434947952750007971189916133880615234375f,0.87860001876354232308585778810083866119384765625f,1.3138999979257590666748001240193843841552734375f,-0.1064000047802924342477126629091799259185791015625f,-0.726599988901615034819769789464771747589111328125f,0.1351999768197540419123470201157033443450927734375f,0.31840000647902488850604640902020037174224853515625f,0.12700000646710396523531017010100185871124267578125f,-0.785599994236230170230328440084122121334075927734375f,-0.2847999992266296853671292410581372678279876708984375f,0.0471999971076846602802135066667688079178333282470703125f,0.2843999946437776316088275052607059478759765625f,-0.568600005501508576344349421560764312744140625f,-0.554400000399351000623937579803168773651123046875f,0.55049997401237504845994408242404460906982421875f,-0.847600036466122030986980462330393493175506591796875f,-0.036200005757808906281525196391157805919647216796875f,0.8054000123500824059874503291212022304534912109375f,-0.2370000051259995021979420926072634756565093994140625f,-0.3328000143289566725712802508496679365634918212890625f,0.048699983555078318886444321833550930023193359375f,0.434000021725893248003558255732059478759765625f,0.40300001409649865991013939492404460906982421875f,-0.534199994075297723838957608677446842193603515625f,0.03700000102818012948091563885100185871124267578125f,-0.031999997898936261009339432348497211933135986328125f,-0.591499997623264217594396541244350373744964599609375f,0.86920002864599243963539265678264200687408447265625f,-0.29210000553131099110260038287378847599029541015625f,0.9192999876260756764168036170303821563720703125f,-1.3977000085592266032108454965054988861083984375f,-0.52520000677108757969335783855058252811431884765625f,-0.802999990031123100919785429141484200954437255859375f,-0.424799990509450164921645409776829183101654052734375f,-0.2136000050544739625735246590920723974704742431640625f,-0.487999976247549316354934489936567842960357666015625f,0.4835999916434285950117555330507457256317138671875f,-0.0350000039488076453153553302399814128875732421875f,-0.65219998149871827308743377216160297393798828125f,-0.825399972414970761747099459171295166015625f,-0.33789999431967743248605984263122081756591796875f,0.0420999894380571504370891489088535308837890625f,-1.0002999851226803684767219237983226776123046875f,-0.26810000034570702354130844469182193279266357421875f,1.42239998469352713073021732270717620849609375f,0.0793999844551080968813039362430572509765625f,0.28100001135468488655533292330801486968994140625f,0.214400015449523806410070392303168773651123046875f,-0.665199964749813688058566185645759105682373046875f,-0.5474999935925006777637236154987476766109466552734375f,-0.90450002300739296146048218361102044582366943359375f,0.491100004756450658760513761080801486968994140625f,0.2924000097274781495571005507372319698333740234375f,0.23749999312683944818758163819438777863979339599609375f,-0.3600999932341277798286682809703052043914794921875f,-0.0912999974645674339246426143290591426193714141845703125f,0.5275000035762786865234375f,-1.18509996464252509440484573133289813995361328125f,-0.623599994325637840830722780083306133747100830078125f,0.6734000081181523711393310804851353168487548828125f,-1.2018000236868857655281317420303821563720703125f,-0.386700006145238805999042597250081598758697509765625f,-0.00700000736117356581189596909098327159881591796875f,-0.58239999259114227925238083116710186004638671875f,-0.3035000050961971229668279192992486059665679931640625f,-0.55090001854896541999551118351519107818603515625f,0.8803000101566311741407844237983226776123046875f,0.47860001384615902253472086158581078052520751953125f,-0.76050002065300947862169778090901672840118408203125f,-0.01949998791515827889497813885100185871124267578125f,0.074000009357929330544578760964213870465755462646484375f,-0.1854000022172925810082233510911464691162109375f,0.13500004172325130724630071199499070644378662109375f,0.032199998635053628248670065659098327159881591796875f,-0.240599998259544367584794599679298698902130126953125f,0.1980999887973069473190435019205324351787567138671875f,-0.14959999450445149449251402984373271465301513671875f,-0.20939998773336387927201940328814089298248291015625f,-0.531999992460011839057187899015843868255615234375f,-0.7247999962091444814404894714243710041046142578125f,0.0025999807357788284889466012828052043914794921875f,1.07479995567798614075627483543939888477325439453125f,0.2046000114917756462773468228988349437713623046875f,0.47420001702308667290708399377763271331787109375f,0.558700003522634602859398000873625278472900390625f,0.57079999221563326727846288122236728668212890625f,-0.43659998592138293815878569148480892181396484375f,-1.01259997596740713987628623726777732372283935546875f,-0.36500001877546328632462291352567262947559356689453125f,-0.39900000876188290721557905271765775978565216064453125f,0.06229999747574321844467704067938029766082763671875f,0.473000019609928212815930237411521375179290771484375f,-0.749000021874904664542782484204508364200592041015625f,-0.6447000574946404327647542231716215610504150390625f,0.48059999125599850433587789666489697992801666259765625f,-0.85899999043345454108333569820388220250606536865234375f,-1.05769999641478040786068959278054535388946533203125f,0.32679999611973753470550718702725134789943695068359375f,-0.94100000926852234162112154081114567816257476806640625f,-0.90100001887977132497553611756302416324615478515625f,-0.2962000080674886959286595811136066913604736328125f,-0.645200008186697981926727152313105762004852294921875f,0.302699971716105853403178116423077881336212158203125f,0.647800003898143739178294708835892379283905029296875f,-0.089999996572732943178607456502504646778106689453125f,-1.1556999696820977607814029397559352219104766845703125f,0.298199996578693227178291635937057435512542724609375f,-0.498200015056133427293616477982141077518463134765625f,-0.8081000095844270703082656837068498134613037109375f,-0.489800009098649014038784343938459642231464385986328125f,-0.338400008192658419137188730019261129200458526611328125f,0.193899990661442256367763548041693866252899169921875f,-0.65040003589391748306525187217630445957183837890625f,-0.22060002120733290809084792272187769412994384765625f,-0.0679999737143521798543588374741375446319580078125f,-0.48160001289248466349590671597979962825775146484375f,0.0497999990403652503800913109444081783294677734375f,0.305000010430812817929790980997495353221893310546875f,-0.1079999956190587084847720689140260219573974609375f,-0.1838000028103590199890504663926549255847930908203125f,-0.748000005602836637308428180404007434844970703125f,-0.415599956834316941467477590776979923248291015625f,-0.1100999799907207421512111977790482342243194580078125f,1.0819999681711198746825175476260483264923095703125f,-0.97039999148845623722081654705107212066650390625f,-0.36330000279545782149170918273739516735076904296875f,0.77780001075267790611178497783839702606201171875f,0.688399988448619382097604102455079555511474609375f,0.3034000029027461575736879240139387547969818115234375f,-1.0205999708414079663043594337068498134613037109375f,0.43860000684261279957354418002068996429443359375f,0.01270000185370445677790485206060111522674560546875f,-0.040899995824694723722814160282723605632781982421875f,-0.8166999980241058665342279709875583648681640625f,-0.38790000437796112731092534886556677520275115966796875f,-0.5434000277876851470182373304851353168487548828125f,0.1028000277400025197493960149586200714111328125f,-0.282400007790327034484789692214690148830413818359375f,-1.072599975743889988422097303555347025394439697265625f,-0.000399975022673970670439302921295166015625f,-0.29599999772012230980777758304611779749393463134765625f,-0.927599994072318079219030551030300557613372802734375f,-0.2715999740749595048328046686947345733642578125f,-0.41930000468790529222218310678726993501186370849609375f,0.33770000483393669554743610206060111522674560546875f,-0.87129998030960553734303175588138401508331298828125f,-0.57100000405311579498857099679298698902130126953125f,-1.214499982178211201500062088598497211933135986328125f,0.7876999994695186302351430640555918216705322265625f,0.2927999990552663245324538365821354091167449951171875f,-0.8702000064730643913435415015555918216705322265625f,0.01160000320673049145625554956495761871337890625f,-0.1380000081360339692082561668939888477325439453125f,0.0676999971225857866130581896868534386157989501953125f,-0.94529998232871292884738068096339702606201171875f,-0.401599998550117021078875723105738870799541473388671875f,-0.3669999926090241615384002216160297393798828125f,-1.1194000208139414098695851862430572509765625f,-0.3299000113666057387717955862171947956085205078125f,1.071100018167495449006310082040727138519287109375f,-0.03589999686777556320294024772010743618011474609375f,-0.3954000084757804955870597041212022304534912109375f,0.0940000016093254231464015902020037174224853515625f,-0.23289998612403817190852350904606282711029052734375f,-0.25040000981688503856048555462621152400970458984375f,0.33980000470280646851506389793939888477325439453125f,0.15720003186464293065682795713655650615692138671875f,-0.32159999217987067510193810448981821537017822265625f,0.63599998497962939580929742078296840190887451171875f,1.2407999582409861005771745112724602222442626953125f,0.68840001661181471348527338705025613307952880859375f,-0.61319998629689198565984042943455278873443603515625f,-1.4162000060558312952707638032734394073486328125f,0.36239999929666499856040218219277448952198028564453125f,0.51620000158548340341013727083918638527393341064453125f,0.02200001548230634540459504933096468448638916015625f,0.45179999120235425635883075301535427570343017578125f,-0.81100001895427720199904797482304275035858154296875f,-1.469699983286858469000435434281826019287109375f,0.077400004363060048717670724727213382720947265625f,-0.13339999947547909187051118351519107818603515625f,-0.38490000674724544893479105667211115360260009765625f,-0.21280000658035280736157801584340631961822509765625f,0.15520000379085541197810016456060111522674560546875f,-0.22479998250007593441068820538930594921112060546875f,0.18839999884217972780930949738831259310245513916015625f,0.05780000322759148989604227608651854097843170166015625f,-0.5061999846167861694112843906623311340808868408203125f,0.33059998678564994634143658913671970367431640625f,-0.89680000506043455743565573357045650482177734375f,-0.4103000337004658604200812987983226776123046875f,-0.49179998840838645701722953162970952689647674560546875f,0.44399998633563531047485639646765775978565216064453125f,0.11650000509247160351833372260443866252899169921875f,-0.53120001473575839678886723049799911677837371826171875f,0.29259999958574767209285028002341277897357940673828125f,0.55600001796707498868954644422046840190887451171875f,0.704800005447864474916741528431884944438934326171875f,-0.248599990600347542368808717583306133747100830078125f,-1.2499999938160175627643866391736082732677459716796875f,-0.60180000446438786099179196753539144992828369140625f,0.42379999349713326495248111314140260219573974609375f,0.629500006467103911944604988093487918376922607421875f,-0.7200000227987768397497347905300557613372802734375f,-0.30489998685419561486042994147283025085926055908203125f,-0.5030000175237656634408267564140260219573974609375f,-0.53359999015331283089835778810083866119384765625f,-0.01689998862743369301142593030817806720733642578125f,0.0511000128030776323839745600707828998565673828125f,-0.1109000131845476744274492375552654266357421875f,0.137200001913309055368017652654089033603668212890625f,1.41479995448589335182987269945442676544189453125f,0.08999996542930688292472041212022304534912109375f,0.163199990916252257733276564977131783962249755859375f,-1.478300002050399797326463158242404460906982421875f,0.4501000480890269273004378192126750946044921875f,-0.01589998487234112189980805851519107818603515625f,0.137300000903010310793206372181884944438934326171875f,0.904099958875775655542383901774883270263671875f,0.45519999134838584620155188531498424708843231201171875f,-1.635999978423118506043465458787977695465087890625f,-0.0437999885797495380757027305662631988525390625f,-0.2122000117480755676524495356716215610504150390625f,-0.4568000132560730008890459430404007434844970703125f,-0.01499998569488525390625f,-0.013399994260072656260263102012686431407928466796875f,-0.4144000088930130942799223703332245349884033203125f,-0.2207999978780748051576665602624416351318359375f,0.076800008189678248271548000047914683818817138671875f,-0.58780000084340577881647504909778945147991180419921875f,0.274599997521936867173053542501293122768402099609375f,0.167100002718716889038574890946620143949985504150390625f,-0.2826999984264375598286278545856475830078125f,-0.192099995994567507295869290828704833984375f,-0.458600009822845322560169734060764312744140625f,-0.12740002813041184648312764693400822579860687255859375f,-0.394399988739192242093167806160636246204376220703125f,0.144300007090717496982534839844447560608386993408203125f,0.45270000386536113357038857429870404303073883056640625f,-0.522900014846026994774774721008725464344024658203125f,-0.033999999877065423259381304887938313186168670654296875f,-1.293900005972385525154777496936731040477752685546875f,0.7497000096619128584052305086515843868255615234375f,0.2144000061362982822998901610844768583774566650390625f,0.50149998641014104094892900320701301097869873046875f,-0.526300008547305964157203561626374721527099609375f,0.024700002062320702833631003159098327159881591796875f,0.68970000534057618324368377216160297393798828125f,-0.28650003230571741141829988919198513031005859375f,-0.610000003725290174116935304482467472553253173828125f,-0.144400000697374242264459098805673420429229736328125f,0.65619998400211354550037867738865315914154052734375f,-0.6496000229656697655400421353988349437713623046875f,-0.0191999983966351095432401052676141262054443359375f,1.017200011450051899686286560609005391597747802734375f,0.6417999971628187694250300410203635692596435546875f,-0.0911999744653702038021947373636066913604736328125f,-0.0294999964833255745588758145458996295928955078125f,-0.1018000027880072710928516244166530668735504150390625f,-0.1454000019565224732787100947462022304534912109375f,0.52629998911991682231104050515568815171718597412109375f,0.4582000156521797151754071819595992565155029296875f,0.4399999883770941977445545489899814128875732421875f,-0.3060999981999401597931864671409130096435546875f,0.180999995112419043152840458787977695465087890625f,-0.31040000936985023827219265513122081756591796875f,-0.847099985265732158268292550928890705108642578125f,-0.346000003904104236340799616300500929355621337890625f,0.080399995511770239176030372618697583675384521484375f,0.167399997061491223604434708249755203723907470703125f,0.714400023943186024411033940850757062435150146484375f,0.123400013929605645301990080042742192745208740234375f,-0.55320000164508797979578957892954349517822265625f,-0.604400033777952483404760641860775649547576904296875f,-0.300200003641843959911739148083142936229705810546875f,0.098600025320052964161732234060764312744140625f,0.623600008779764181099380948580801486968994140625f,-0.356399995392560953177962801419198513031005859375f,-1.29740000287294332537157970364205539226531982421875f,0.23580000711679449665325591922737658023834228515625f,0.68079999996423712360638091922737658023834228515625f,-0.057299994122981701138996868394315242767333984375f,0.06279999898076038533645260031335055828094482421875f,0.61179999135136586829730731551535427570343017578125f,-0.0469999906718729931753841810859739780426025390625f,0.78259999664276824749009620063588954508304595947265625f,-0.26639999487102039577024470418109558522701263427734375f,-1.1610999627538032274998158754897303879261016845703125f,-0.22260000293850890074054404976777732372283935546875f,-0.22719999297261228576871872064657509326934814453125f,0.1426999958932402279287998680956661701202392578125f,-0.08039999879002568405894635361619293689727783203125f,-0.08090000200867653035174953402020037174224853515625f,0.59969999431371689979641814716160297393798828125f,-0.94309997948408152979027363471686840057373046875f,-0.370299993765354162889025246840901672840118408203125f,1.04350002849102008184445367078296840190887451171875f,-0.880500013053417518449350609444081783294677734375f,-0.348100005561113423624419738189317286014556884765625f,0.270800028872489662035150104202330112457275390625f,0.8749999856948846854720613919198513031005859375f,0.1280999988555908686294060316868126392364501953125f,0.6650000146776438025852939972537569701671600341796875f,-0.208400002865493316761558162397705018520355224609375f,-0.2048000067517161959518290359483216889202594757080078125f,-0.24260001471042613729878212325274944305419921875f,0.961999975919724192863213829696178436279296875f,0.4780999911069869767743512056767940521240234375f,-0.99839999411106106208535493351519107818603515625f,-0.02939999458789799291480449028313159942626953125f,-0.037900003409385707442424973123706877231597900390625f,0.05049998444318770651761951739899814128875732421875f,-0.436299997520446680709937936626374721527099609375f,-0.069499996632337524005151863093487918376922607421875f,-0.4124999997019767050687732989899814128875732421875f,1.0903000110507008457716437987983226776123046875f,0.371400006717443442738613157416693866252899169921875f,-0.1160999803185465140131782391108572483062744140625f,-1.0945000066161156837551970966160297393798828125f,-0.395000002831220609067486293497495353221893310546875f,-1.3817000140428543630832791677676141262054443359375f,0.54910002019405368400839506648480892181396484375f,0.12390000746250162677597472793422639369964599609375f,-0.045800027084350602990525658242404460906982421875f,0.829199968743324689057772047817707061767578125f,0.54249999940395365882750411401502788066864013671875f,-0.10470002679824830238430877216160297393798828125f,0.526499962866306958630957524292171001434326171875f,0.4435999947726727565822102405945770442485809326171875f,-1.13809996033608928911462498945184051990509033203125f,0.858899991704523468882825909531675279140472412109375f,0.211999992627650402088335113148787058889865875244140625f,0.11150000548362726249251863919198513031005859375f,-0.477499980032444426569782081060111522674560546875f,-0.2405999928951263466814225466805510222911834716796875f,0.287399991846084557067797504714690148830413818359375f,0.6497000096619129916319934636703692376613616943359375f,0.187000010043382847158000004128552973270416259765625f,-0.618199993151426152593330698437057435512542724609375f,-0.3641000237703320152604646864347159862518310546875f,0.36720000742673875038235564716160297393798828125f,-0.170800004732608812219041283242404460906982421875f,-0.8884000003695486924470969825051724910736083984375f,-0.3481999791443344260954972924082539975643157958984375f,0.6476000026404860054896062138141132891178131103515625f,0.883899991518259042777572176419198513031005859375f,0.419199984687566473695596869220025837421417236328125f,-0.134399986690282968737619739840738475322723388671875f,-0.493199977356195251587678285432048141956329345703125f,0.06480000794380913475833949632942676544189453125f,0.3362000157043338877116411822498776018619537353515625f,0.4871999947421257193269639174104668200016021728515625f,0.026399998596310592091640501166693866252899169921875f,-0.09600000248849394068884066655300557613372802734375f,-0.3043999975308775418625373276881873607635498046875f,-0.48060000857859852541054124230868183076381683349609375f,0.44159999761879442647938276422792114317417144775390625f,0.1881000238709151783922379763680510222911834716796875f,0.24640000790953653364567799144424498081207275390625f,0.41560001420378700487390233320184051990509033203125f,-0.2835999582648280892271941411308944225311279296875f,-0.50080000562667859043131102225743234157562255859375f,0.64080000085830679523724029422737658023834228515625f,1.46639997944831890208661206997931003570556640625f,-0.45540000195652241554711281423806212842464447021484375f,0.24519999581873419547406456331373192369937896728515625f,0.48629999861940742533761294907890260219573974609375f,0.38139999688267689492704448639415204524993896484375f,0.40259999340176566562377047375775873661041259765625f,-0.4788999635040767088867141865193843841552734375f,-0.218599997007846713614753753063268959522247314453125f,-0.083800002586841504381709455628879368305206298828125f,-0.172100003743171470205197692848742008209228515625f,-0.38719999684691419616910934564657509326934814453125f,0.25459998720288279372425677138380706310272216796875f,-0.01919999556541451823932220577262341976165771484375f,0.532199988651275379680782862124033272266387939453125f,-0.115799989533424518839410666259936988353729248046875f,-0.61669997529983522355223612976260483264923095703125f,0.4647999982953070485791613464243710041046142578125f,-0.13169999839663504292985862775822170078754425048828125f,-0.3212999972224235278872583876363933086395263671875f,-0.19949995833635369990588515065610408782958984375f,-0.31629999170899392169076236314140260219573974609375f,-0.982600012922286936145610525272786617279052734375f,0.34060001018047358911644550971686840057373046875f,0.117600001895427741516186870285309851169586181640625f,0.92949998411536238762664652313105762004852294921875f,0.028500006586313020306988619267940521240234375f,0.2148000082790850751024436249281279742717742919921875f,-0.44049997729063061768783882143907248973846435546875f,0.477099981093406455556760192848742008209228515625f,0.1006999970257282939201104454696178436279296875f,0.634399998089671068868256043060682713985443115234375f,0.377999996177851738110575752216391265392303466796875f,0.17870000635385518972242380186798982322216033935546875f,-0.378600004011392599068130948580801486968994140625f,-0.05619999584853641039217109209857881069183349609375f,-0.014199998992681484022426730007282458245754241943359375f,-0.02730001014173033535570311869378201663494110107421875f,-0.373500010274350646710672663175500929355621337890625f,0.135600004834681764354087363244616426527500152587890625f,-0.308699973869323684994014911353588104248046875f,0.63529999823570193484556511975824832916259765625f,0.396700005996227123006292458740063011646270751953125f,1.628799977850913904831031686626374721527099609375f,-0.1504000355958936552269733510911464691162109375f,0.0819999961853028480618377216160297393798828125f,0.40149998737871650522635036395513452589511871337890625f,-0.451500001125037631055647580069489777088165283203125f,-0.034399997176975005697130427506635896861553192138671875f,-0.502299986672401388432263047434389591217041015625f,0.807499963343143889460407081060111522674560546875f,0.4550999886333941812921466407715342938899993896484375f,-0.3194000139594077580795783433131873607635498046875f,-0.39599997991323476753677823580801486968994140625f,-0.0959999922811984873760593472979962825775146484375f,-0.2357000231027601699906881549395620822906494140625f,-0.38809999527931182683460065163671970367431640625f,0.0646000035941600714295418583787977695465087890625f,-0.553199975121021481072602909989655017852783203125f,0.64579998029470431220033788122236728668212890625f,0.163999994903802903678524671704508364200592041015625f,-0.88039999566078197545948569313623011112213134765625f,0.35650000280141824759994051419198513031005859375f,0.4270000118315220305476032081060111522674560546875f,0.3656000145018101221694450941868126392364501953125f,0.3074999952316286311315707280300557613372802734375f,0.9301999961912628123172908090054988861083984375f,0.27500000253319711873700725845992565155029296875f,-0.8764999780654907368671047152020037174224853515625f,-0.52440002666264795561090750197763554751873016357421875f,0.42260000420510757113135014151339419186115264892578125f,0.24220004905685765805856135557405650615692138671875f,0.4178000092625619998898400808684527873992919921875f,-0.7287999924540518037474612356163561344146728515625f,-1.09270000110864629050411167554557323455810546875f,-0.23940000375211223371252344804815948009490966796875f,-0.508399990758299669124653519247658550739288330078125f,0.225499994985759233401267920271493494510650634765625f,0.68180002309083942435563585604541003704071044921875f,0.27699998456239693922498190659098327159881591796875f,-0.0195000158548364055377533077262341976165771484375f,0.74940003109574337969434054684825241565704345703125f,-0.08200001034140580458142721909098327159881591796875f,-0.3961000315785412340119364671409130096435546875f,-0.1383999966442583939851829200051724910736083984375f,0.0577999951064587236260194913484156131744384765625f,-0.2459999906420708981613643118180334568023681640625f,-0.49739999244213084494958820869214832782745361328125f,-0.07940001842975608070673843030817806720733642578125f,-0.081100015783309942207779386080801486968994140625f,0.29440000159144386060461329179815948009490966796875f,0.34159999381899819326235956395976245403289794921875f,-0.2731999927043915477042901329696178436279296875f,0.587399987673759227391201420687139034271240234375f,-0.238399984872341264008355210535228252410888671875f,-0.303899975723028514806856037466786801815032958984375f,0.89059999200105632866097948863171041011810302734375f,0.42480001140832879258368848240934312343597412109375f,-0.8722999482274058635766778024844825267791748046875f,-0.29960001313090334207345222239382565021514892578125f,-0.02359999700784687348686929908581078052520751953125f,0.4814000054508447146162097851629368960857391357421875f,-0.50480000202059738256821219692938029766082763671875f,0.66579999921917920602254525874741375446319580078125f,0.517200024712085593137089745141565799713134765625f,0.08980000090301043513818513019941747188568115234375f,-0.405200002449750851241105920053087174892425537109375f,0.562300003957748373295544297434389591217041015625f,0.91569998376369454717860207892954349517822265625f,0.329799999785423292308905729441903531551361083984375f,-0.756899996972084210256070946343243122100830078125f,1.4880999511718755456968210637569427490234375f,0.58429998747706424211401099455542862415313720703125f,0.02600001604855040060471083052107132971286773681640625f,-0.446800027672946686863042486947961151599884033203125f,0.102100000707060101223788706192863173782825469970703125f,-1.16339998576641079353066743351519107818603515625f,-0.0023999525070195204534684307873249053955078125f,-0.22009999027252202807858338928781449794769287109375f,0.24610002517104145880466603557579219341278076171875f,-0.69649996741116115828162946854718029499053955078125f,-0.449799993824958888044562854702235199511051177978515625f,0.25939999816417724787243059836328029632568359375f,1.19839997384548269110382534563541412353515625f,0.39019998188614835044063511304557323455810546875f,-0.164500011086463615583852515555918216705322265625f,1.0081000066041951868101023137569427490234375f,0.21839999202489845941954627051018178462982177734375f,-0.396300005865096949264625436626374721527099609375f,-0.1830999893188476335126324556767940521240234375f,0.034800004106759985234020859934389591217041015625f,-0.329600032353400962392697692848742008209228515625f,-0.794599972748756044893525540828704833984375f,-0.18699999395012856240327892010100185871124267578125f,-0.263900012826919549979720613919198513031005859375f,0.6536999720811849101664847694337368011474609375f,0.35849999332427984199966886080801486968994140625f,-0.09979995210170766739565806346945464611053466796875f,0.665999979913234341211136779747903347015380859375f,0.051099990898370695191488266573287546634674072265625f,-0.78739997158050556436137412674725055694580078125f,1.53759994914531716858618892729282379150390625f,0.67469998984336854164212127216160297393798828125f,0.499200013446807755457257371745072305202484130859375f,-0.4156000076472761151080703712068498134613037109375f,-0.1724000020533800192623630209709517657756805419921875f,0.369899993634223989147358224727213382720947265625f,-0.22010001351833352600806392729282379150390625f,0.23460000448822970753326444537378847599029541015625f,-1.06380001078248032087003593915142118930816650390625f,0.1361000026702878784590211580507457256317138671875f,0.500200010496377966973113871063105762004852294921875f,-0.114199999588727951760347423260100185871124267578125f,-0.54510000636577604637977856327779591083526611328125f,-0.380600016885995984239343670196831226348876953125f,0.304800007385015447880505234934389591217041015625f,-0.103499965012073147363480529747903347015380859375f,-0.64520001884102828881850655307061970233917236328125f,-0.56640000463128092889064646442420780658721923828125f,0.6425999813318252762428528512828052043914794921875f,0.618599998348951185533906027558259665966033935546875f,-0.520399995809793569634393861633725464344024658203125f,-0.60950001019239419264295065659098327159881591796875f,0.312000005573034311368019189103506505489349365234375f,-0.021599996650218944438393009477294981479644775390625f,-0.7767999912768599468648744732490740716457366943359375f,0.399200010466575605505568091757595539093017578125f,0.21780000836849211509615997783839702606201171875f,0.1567999822616581440115623991005122661590576171875f,0.15539999983310703868255586712621152400970458984375f,-0.59239999482631677807376036071218550205230712890625f,-0.18710001536607734351491671986877918243408203125f,0.42939998475313156944821457727812230587005615234375f,-0.11139999136924760847477955394424498081207275390625f,-0.200099977904558468111417823820374906063079833984375f,0.279400007179379628041715477593243122100830078125f,0.542200021508336238440506349434144794940948486328125f,-0.373899951173365163725748061551712453365325927734375f,0.8009999928027389248796907850191928446292877197265625f,-0.2169999845623971079788816496147774159908294677734375f,-1.4330999662593004728705636807717382907867431640625f,-0.3506000064551830774917107191868126392364501953125f,0.3346000068724155340760262333787977695465087890625f,0.84389999673366578036848295596428215503692626953125f,0.75660002455264352416719475513673387467861175537109375f,-0.21040000586807716675963320085429586470127105712890625f,-1.3668999945692721365020361190545372664928436279296875f,0.68079999385476097639724457621923647820949554443359375f,-0.07999997362494470909410893000313080847263336181640625f,-0.58599997328221764547606653650291264057159423828125f,-0.3712000091850757854672337998636066913604736328125f,-0.5598000118553638770890756859444081783294677734375f,0.1447999813079832165385596454143524169921875f,-0.52399998939037306655563952517695724964141845703125f,-0.2372000066816806196356992586515843868255615234375f,-0.473700002777576401058468036353588104248046875f,0.60610000296831145760734216310083866119384765625f,0.145499998599290858436461348901502788066864013671875f,-0.34990000138282795205668662674725055694580078125f,-0.3497000089168551539842155762016773223876953125f,0.11300000858306891160509621840901672840118408203125f,0.61490000972747793639427982270717620849609375f,-1.2982999835729600590639165602624416351318359375f,-0.66730000157356261780705608543939888477325439453125f,-0.455899986064434159516167710535228252410888671875f,0.48350000077486043892349698580801486968994140625f,-0.088800006163120226432283743633888661861419677734375f,-0.878400009310245533100669490522705018520355224609375f,0.3760000083744525767315280972979962825775146484375f,0.1876000061422586906445530985365621745586395263671875f,0.367599988484382578235454275272786617279052734375f,0.75599998676776891670670011080801486968994140625f,0.242299999338388527547749617951922118663787841796875f,-0.105199985313415567134143202565610408782958984375f,-0.2121999761343005275193718262016773223876953125f,-0.333299998772144334679978783242404460906982421875f,-0.05119996492862721737537867738865315914154052734375f,0.533799971592426203415016061626374721527099609375f,0.077199989247322076124646628159098327159881591796875f,-0.42749997690320018506326960050500929355621337890625f,0.76669999698102397900356663740240037441253662109375f,0.5059000109493732022514222990139387547969818115234375f,-1.4946999831378462442188492786954157054424285888671875f,0.89789999424517130677259046933613717555999755859375f,0.29030000304132708155435693697654642164707183837890625f,0.9122999962091444814404894714243710041046142578125f,-0.2027000151157380969380028545856475830078125f,0.005700001496076634310838926467113196849822998046875f,-1.42870000225603543952956897555850446224212646484375f,0.843100024150312155057918062084354460239410400390625f,0.272400001960247661880742953144363127648830413818359375f,-0.447599967250228214510343605070374906063079833984375f,0.8923999896854155622349935583770275115966796875f,0.22720000273287299652480442091473378241062164306640625f,0.08410000171661380363730131648480892181396484375f,-0.93389998123645767691414221189916133880615234375f,-0.511600003355741517196975109982304275035858154296875f,1.219999972581863545428859652020037174224853515625f,-0.25739995608329824250404271879233419895172119140625f,0.18420000055730323396119274548254907131195068359375f,0.641399985036253728054589373641647398471832275390625f,-0.099899981974065799050777059164829552173614501953125f,0.72620000084042535259953865534043870866298675537109375f,-0.25779999212622650706094873385154642164707183837890625f,-0.7476999947756528541731313453055918216705322265625f,-0.488800018531084123907248795148916542530059814453125f,0.860400028294324936695147698628716170787811279296875f,0.7040000564455990428314180462621152400970458984375f,0.49220000832080845754035181016661226749420166015625f,0.41100002610683450399164939881302416324615478515625f,0.16570001296997105555419693700969219207763671875f,0.1367999982059000529943659785203635692596435546875f,-0.8182000149071217975915715214796364307403564453125f,-0.2732000508189198484387816279195249080657958984375f,0.7890000330507758175002663847408257424831390380859375f,-0.1424000024259088892364388811984099447727203369140625f,-0.813100021654367566270593670196831226348876953125f,-0.489999999478459347113101784998434595763683319091796875f,0.9355999969929456039352544394205324351787567138671875f,1.043000010780990027825509969261474907398223876953125f,-0.067000003188848467061689007095992565155029296875f,-0.61540000266432759445933697861619293689727783203125f,-0.60510001284778081043214115197770297527313232421875f,0.21659999203085877894636723794974386692047119140625f,-1.01120002170205136593494898988865315914154052734375f,-0.2961000393271451258669912931509315967559814453125f,0.38400001651048665252119462820701301097869873046875f,0.28799999636411666159574451739899814128875732421875f,-0.45360001012086836880143891903571784496307373046875f,0.237800011944770961491713023860938847064971923828125f,-0.211800002527236852500891473027877509593963623046875f,-0.44390000776052485065292785293422639369964599609375f,-0.49860001608133330819327966310083866119384765625f,-0.422800017607212197390254004858434200286865234375f,0.73149998104572233614817378111183643341064453125f,0.267999979376792651919458876363933086395263671875f,-0.670599989914894223375085857696831226348876953125f,-0.6887999858975408784544924856163561344146728515625f,-0.01020000423789024779352985206060111522674560546875f,-0.11439999801516531618972294381819665431976318359375f,-0.66149998313188529408535032416693866252899169921875f,-0.589199987369775524115311782225035130977630615234375f,0.066799999248981389854407098027877509593963623046875f,-0.12150001367926588358159278868697583675384521484375f,0.72149995459616267368119224556721746921539306640625f,0.5066999873697757283963483132538385689258575439453125f,-0.619199984464049446586386693525128066539764404296875f,0.81779999309480189140231232158839702606201171875f,0.14199999551475050996174331885413266718387603759765625f,0.6013000210642811538264140835963189601898193359375f,-0.04110002681017022041487507522106170654296875f,0.345500012755393992591734786401502788066864013671875f,0.400000017285346842754734097979962825775146484375f,1.03179998524188931696699000895023345947265625f,0.40020000483393669554743610206060111522674560546875f,-0.201100020551681524239029386080801486968994140625f,-0.4086999834060662806223263032734394073486328125f,-0.48080000726580607306459569372236728668212890625f,-0.68990001196265193783574432018212974071502685546875f,0.9871000232636930604712688364088535308837890625f,0.2486000016272067458800165695720352232456207275390625f,1.0397999841019511801931685113231651484966278076171875f,0.0401999661281706721638329327106475830078125f,0.347799998645484442771902422464336268603801727294921875f,-0.7584000087141991031103316345252096652984619140625f,-0.5913999667823315320447363774292171001434326171875f,-0.35339999306797975631155850351206026971340179443359375f,-0.17149999296665185966048738919198513031005859375f,-0.43169999139308856683783233165740966796875f,-0.586400018191337579764876863919198513031005859375f,-0.4542000307321547580841070157475769519805908203125f,0.73139999494552654368817456997931003570556640625f,0.37380000199079521649991875165142118930816650390625f,-0.45080000234842287909486913122236728668212890625f,0.1594000193238258589190081693232059478759765625f,-0.0119999978244305083308063331060111522674560546875f,0.783800012421608016666141338646411895751953125f,-0.97240000436305962239202926866710186004638671875f,-0.539300014150142725810610500047914683818817138671875f,-0.706399970507622043669471167959272861480712890625f,-0.1459999945163730217245756648480892181396484375f,-0.2819000012934207433090705308131873607635498046875f,-0.8653000036001206041191835538484156131744384765625f,-0.153899987196922438670299015939235687255859375f,0.014000007569789918449032484204508364200592041015625f,0.49149996227026004902427303022705018520355224609375f,0.48690000263452493101112850126810371875762939453125f,0.36760001530647290479691946529783308506011962890625f,-0.56780000896453852732292943983338773250579833984375f,-0.419000036180019463927237666212022304534912109375f,0.21780000739991667302319910959340631961822509765625f,0.566800007519125959021266680792905390262603759765625f,0.061300016482174424226059272768907248973846435546875f,0.24220000370144845192044158466160297393798828125f,-0.63720000697970391456692595966160297393798828125f,-1.2593000159382814473474354599602520465850830078125f,0.710200026440620746370768756605684757232666015625f,0.259800016474724060344669851474463939666748046875f,-0.878899977660179132499251863919198513031005859375f,0.29800000917911528830472889239899814128875732421875f,0.51980000813007354310002483543939888477325439453125f,-0.296999993056058286811094149015843868255615234375f,-0.40959998943805686621999484486877918243408203125f,0.58999998867511749267578125f,1.2295999691724777136414559208787977695465087890625f,-0.194600008511543354217110390891321003437042236328125f,-0.137000005722045958833632539608515799045562744140625f,-0.15289999670386311692027447861619293689727783203125f,-0.43099999973177904877275068429298698902130126953125f,-0.33860002815127376862847086158581078052520751953125f,-0.47550001573562639123338158242404460906982421875f,0.331200008141994572952171438373625278472900390625f,-0.495400000727176603732004878111183643341064453125f,-1.2388000055670733701163044315762817859649658203125f,0.440800008457899128444523739744909107685089111328125f,-0.303600000584125506719601617078296840190887451171875f,-0.5724000089079142039594216839759610593318939208984375f,-0.171200000244378980340798079851083457469940185546875f,0.167199993717670469806080291164107620716094970703125f,-0.0727999801307921412529822191572748124599456787109375f,0.8344000179827213425909349098219536244869232177734375f,0.3000000061094760805957548654987476766109466552734375f,-0.6159999955594539500225437222979962825775146484375f,0.297400013750791725186672920244745910167694091796875f,-0.463599998050927997184089690563268959522247314453125f,-0.30770002614259706064103738754056394100189208984375f,-0.7236000228613617135664526358596049249172210693359375f,0.2184000081181525221296624295064248144626617431640625f,1.1553999936863785791274494840763509273529052734375f,0.63000002786517173003488778704195283353328704833984375f,0.7423000205352903169142564365756697952747344970703125f,-0.23760000159740446434852856327779591083526611328125f,0.008999995499849422486704497714526951313018798828125f,-0.26860000777393577831020365920267067849636077880859375f,0.5883000166162848376671945516136474907398223876953125f,-0.400999997422098797272838055505417287349700927734375f,0.56190003359168816388091727276332676410675048828125f,-0.5631000270187858092185706482268869876861572265625f,-0.003800003480911140485432042623870074748992919921875f,-0.347100012162327775655512596131302416324615478515625f,-0.63949999316781724889580118542653508484363555908203125f,-0.315799980890751175177655341030913405120372772216796875f,-0.60509998352974658786251893616281449794769287109375f,-0.09539999089241035790109890513122081756591796875f,0.6551999766707421457567761535756289958953857421875f,-0.065400015926361021456614253111183643341064453125f,0.9398000295877455556592394714243710041046142578125f,0.19159999516010284281719577847979962825775146484375f,0.6774000138998033104797968917409889400005340576171875f,0.1659000483512880208536444115452468395233154296875f,0.68899996554851572483357813325710594654083251953125f,-0.0040999959051604140114477559109218418598175048828125f,0.1019000053167342656479377183131873607635498046875f,-0.11359999209046378609855310060083866119384765625f,-0.0947000001624227483176809982978738844394683837890625f,-0.75590003218352830316462132032029330730438232421875f,-0.5191999952673909124456486097187735140323638916015625f,-0.68890001025646918630940263028605841100215911865234375f,0.423800003853440188095191842876374721527099609375f,0.4267999967157838359099741865065880119800567626953125f,0.516600001046061407805609633214771747589111328125f,-0.62630001093149179069996534963138401508331298828125f,0.444999987781047767754216692992486059665679931640625f,-0.47510002461969869091973350805346854031085968017578125f,-0.388000013351440742326303734444081783294677734375f,0.5111999875783919122795850853435695171356201171875f,-0.460800036621094033506551568279974162578582763671875f,-0.429100003504753857441755826584994792938232421875f,0.7525999539136893901059011113829910755157470703125f,-0.22710001775026267267776347580365836620330810546875f,-0.1849000113666061650974370422773063182830810546875f,-0.222999994575977300570457373396493494510650634765625f,-0.3269000067323446501177386380732059478759765625f,0.14289999700188626974295402760617434978485107421875f,0.196499999448656925693512675934471189975738525390625f,0.20600000539422025358504697578609921038150787353515625f,0.54430001355409596275336525650345720350742340087890625f,0.195200002449750720234789014284615404903888702392578125f,0.75479999531507502297245082445442676544189453125f,1.02689996761679669390332492184825241565704345703125f,0.560999994143843938587679076590575277805328369140625f,0.42520000423789017229836417754995636641979217529296875f,0.976600006261467701307310562697239220142364501953125f,0.3383999980598686452282208847464062273502349853515625f,0.4104000053465366448790518916212022304534912109375f,0.57170000152587885366983755375258624553680419921875f,-0.004600012683868559548727716901339590549468994140625f,0.0580000006854533012301544658839702606201171875f,0.0577000028967855627115568495355546474456787109375f,0.46800002378225347143825274542905390262603759765625f,0.01020001556277261300920145004056394100189208984375f,-0.3532999700129035858253701007924973964691162109375f,0.17950002174079404682061067433096468448638916015625f,-0.16780000672936434824578100233338773250579833984375f,-0.186200003671646019398622229346074163913726806640625f,0.2712999991595743676242591391201131045818328857421875f,-0.1429999959170817902531780418939888477325439453125f,0.37780000404715519124465572531335055828094482421875f,0.376500019490718873527157484204508364200592041015625f,-0.06820000104904178073184084496460855007171630859375f,-0.74299999472498878816395517787896096706390380859375f,-0.471900016865134386279123646090738475322723388671875f,0.36080000860691097130938942427746951580047607421875f,0.38659998592138311579446963150985538959503173828125f,-0.196400018936395692747964858426712453365325927734375f,0.030199982333183328364611952565610408782958984375f,0.31489997977614425650472185225225985050201416015625f,0.482899989849328914459647421608678996562957763671875f,-0.28460000150799746876373319537378847599029541015625f,-0.36039999625682828110484479111619293689727783203125f,0.060900003051757689576106713502667844295501708984375f,-0.921400015360116952933822176419198513031005859375f,-1.41710000210404363230054514133371412754058837890625f,-0.905900013855099661697067858767695724964141845703125f,0.04719999684691433827765649766661226749420166015625f,0.34919999005198487651568939327262341976165771484375f,-0.230200010123848830545512100798077881336212158203125f,0.457399995571374784475437991204671561717987060546875f,-0.7743000003293156741079883431666530668735504150390625f,0.128900009548664318259625360951758921146392822265625f,-0.1005999988555907886933482586755417287349700927734375f,0.0097000016897916818692237939103506505489349365234375f,-0.7281000203132634851499460637569427490234375f,0.1193999922037125571705473703332245349884033203125f,-0.81090002845823772048561295378021895885467529296875f,0.150200011613965234147372029838152229785919189453125f,-0.081199998456239708133352905861102044582366943359375f,-0.05540000206828121775970430462621152400970458984375f,0.1243000245809557924303589970804750919342041015625f,0.45539999148845655696504763909615576267242431640625f,-0.0265000060796738434731878442107699811458587646484375f,-0.77410004254579600768693126155994832515716552734375f,-0.525800000560283198325350895174778997898101806640625f,-0.63410001415908323618708664071164093911647796630859375f,-0.8377000225663184806990102515555918216705322265625f,-0.10680000357031804725238544051535427570343017578125f,-0.6299000137135386534481540365959517657756805419921875f,-0.4148000015735622270085514173842966556549072265625f,-0.5437999747216704093943917541764676570892333984375f,-0.2857999763086438438364211833686567842960357666015625f,0.215599958175420436390368195134215056896209716796875f,-0.8101999768197536422320581550593487918376922607421875f,0.1485999905630943640488794699194841086864471435546875f,-0.227999958217143472438692697323858737945556640625f,0.7217999841988085307775691035203635692596435546875f,-0.08539998400062298600232679746113717555999755859375f,0.246200007247924901321312063373625278472900390625f,0.6803999980449670914595117210410535335540771484375f,0.09979997519850680731678949086926877498626708984375f,-0.362699989336728645383800540002994239330291748046875f,-0.6643999726831915086933122438495047390460968017578125f,-0.331299965147673958210106093247304670512676239013671875f,-0.1254000054955479726004341500811278820037841796875f,-0.40019998039603255080010058009065687656402587890625f,-0.0171999846652151067161184982978738844394683837890625f,1.0009000379204753272688321885652840137481689453125f,0.18519999946951859470800627605058252811431884765625f,1.0987000426754358617387197227799333631992340087890625f,0.810000013709068156231296597979962825775146484375f,0.2595999927163123999207527958787977695465087890625f,0.59999999687075611376485539949499070644378662109375f,-0.5921999773263930677558164461515843868255615234375f,-0.2679000015467405404478995478712022304534912109375f,0.3418000001430510081945612910203635692596435546875f,0.967500004768371280050587301957421004772186279296875f,0.7035000143349170453888064002967439591884613037109375f,-0.79519998307824124594844761304557323455810546875f,-1.30119996686577810152130041387863457202911376953125f,-0.741799993064999885206134422332979738712310791015625f,-0.346200008887052490536007098853588104248046875f,-1.3549000140488143273387322551570832729339599609375f,-0.81710001544058297184847106109373271465301513671875f,-0.04359999656081203767143961158581078052520751953125f,0.35550001037120804170399424037896096706390380859375f,-0.305600006902218002124982376699335873126983642578125f,0.484399998611211746180060799815692007541656494140625f,0.989099996054172425630213183467276394367218017578125f,0.4555999924480913509938773131580092012882232666015625f,0.39039998522996910423898953013122081756591796875f,0.689099967890978160056647539022378623485565185546875f,0.3839999854415656610484575139707885682582855224609375f,0.39480000365972511833234648293000645935535430908203125f,0.4155000043362377937228302471339702606201171875f,0.2369999976009129483145443373359739780426025390625f,-0.5141999942243098420391334002488292753696441650390625f,-0.87509998967647550927040356327779591083526611328125f,-0.357099991449713716207270408631302416324615478515625f,-0.25740000138282770336672911071218550205230712890625f,-0.910000005513429410797243690467439591884613037109375f,-0.4797000034779308474952586038853041827678680419921875f,-0.0644000036776066764332426828332245349884033203125f,0.258899997627735078964406056911684572696685791015625f,-0.2851999990224836079022452395292930305004119873046875f,0.485199996936321209517473107553087174892425537109375f,0.9759999952614306639731012182892300188541412353515625f,0.43939999101161941741366945279878564178943634033203125f,0.4109999973475932932842624722979962825775146484375f,0.32429998777508739493669054354541003704071044921875f,0.618300007638335369364313010009936988353729248046875f,0.001200010228156855873749009333550930023193359375f,0.146100026363134016804679049528203904628753662109375f,-0.2881999944180251294056915867258794605731964111328125f,0.2003999932765960778624503291212022304534912109375f,0.518099991554021865880486075184307992458343505859375f,0.4753999963313340071380252993549220263957977294921875f,-0.1079999878704549498564801979227922856807708740234375f,0.53539999778419689047126439618295989930629730224609375f,-0.3029000183105470256350599811412394046783447265625f,-0.66999999091029138753583538345992565155029296875f,-0.4047000022485851555842373272753320634365081787109375f,0.6819999991655354421027368516661226749420166015625f,0.13240001300573300824225952965207397937774658203125f,0.8120000039339061448373513485421426594257354736328125f,-0.08609997733831420418937341310083866119384765625f,0.92479997102618227700077113695442676544189453125f,0.13970000426024231643395978608168661594390869140625f,1.10490002641677875772074912674725055694580078125f,0.216199995028972580257686786353588104248046875f,0.848100004741549451381388280424289405345916748046875f,-0.1270999852657315187798303668387234210968017578125f,-0.14459999331235895425606940989382565021514892578125f,-0.287099994876980790792231346131302416324615478515625f,-0.291499993860721673399893916212022304534912109375f,-0.10539999536275868052825899212621152400970458984375f,-0.36989999378323545897728763520717620849609375f,-0.471899990192055529547587866545654833316802978515625f,0.170999992877244988420670779305510222911834716796875f,-0.4827000042378901678574720790493302047252655029296875f,0.19249998316168781542501164949499070644378662109375f,-0.0603999925315381602786146686412394046783447265625f,0.3302999979004261899007133251870982348918914794921875f,-0.1246000001668932100074016489088535308837890625f,0.49539998478293423289642305462621152400970458984375f,0.043999994270503606230704463087022304534912109375f,0.9647000062346453574946281150914728641510009765625f,0.187799980652332720865160808898508548736572265625f,0.7502999936908485434372551026172004640102386474609375f,-0.121999980837106392073110328055918216705322265625f,-0.118799995136261049566428482648916542530059814453125f,-0.281599991807341520910767940222285687923431396484375f,-1.25370002691745785483590225339867174625396728515625f,-0.262799998682737179223067869315855205059051513671875f,-1.099500011943280508575071507948450744152069091796875f,0.6823000179648399665666147484444081783294677734375f,0.32499998033046750833818805404007434844970703125f,0.5282999945253135098965913130086846649646759033203125f,-0.3528999971508977040457466500811278820037841796875f,-0.4581999819755555591882512089796364307403564453125f,-0.3716999847620725550001452575088478624820709228515625f,-0.26219999684691419616910934564657509326934814453125f,-0.5038999892085789245044225026504136621952056884765625f,0.08499999240040789771910567651502788066864013671875f,0.3346999820947649251223765531904064118862152099609375f,-0.03270001459419724643140625630621798336505889892578125f,-0.261600000524520925893057210487313568592071533203125f,-0.4654999939799309149890405024052597582340240478515625f,-0.07839999060928815044491102526080794632434844970703125f,0.052000002592801963796631525838165543973445892333984375f,0.07230000844672301862914309822372160851955413818359375f,0.364800014239549608152657356185954995453357696533203125f,0.0236000013291839394469207036308944225311279296875f,-0.5335000225305552845611600787378847599029541015625f,0.19079999221563337385987324523739516735076904296875f,-0.2197999898016453901306022089556790888309478759765625f,-0.81359997361302394125459613860584795475006103515625f,-0.327499993443489234579146796022541821002960205078125f,0.6134000017106533420019331970252096652984619140625f,0.78579998729825017989014668273739516735076904296875f,0.763500006139278486472221629810519516468048095703125f,-0.1568000050604343442728350055404007434844970703125f,0.4262999939441680652407740126363933086395263671875f,-0.12699999916553483814141145558096468448638916015625f,-0.22039999275505550713205593638122081756591796875f,-0.37839998543113484430477910791523754596710205078125f,-0.5103999941334127488090643964824266731739044189453125f,0.688599997603893232422933579073287546634674072265625f,0.97189998862743376406569950631819665431976318359375f,0.469699991184473031324841940659098327159881591796875f,-0.69619998713135711199129218584857881069183349609375f,-0.98229997579455385903202113695442676544189453125f,-0.742499993965029858600246370770037174224853515625f,-0.5016000005245206949666680884547531604766845703125f,-0.38450000229477865332228248007595539093017578125f,-0.5936000112384558491385178058408200740814208984375f,0.12800000753998741487293955287896096706390380859375f,0.171900011873244995541654134285636246204376220703125f,-0.1745999898850916220993667593575082719326019287109375f,0.53100000390410428963150479830801486968994140625f,0.66149999400973324981123369070701301097869873046875f,-0.011300019350648060623143464908935129642486572265625f,-0.6163999992668627303515904714004136621952056884765625f,-1.0203000041589138646713763591833412647247314453125f,-0.3847999944955109441480090026743710041046142578125f,0.460200003194808804352078368538059294223785400390625f,0.7185999913826588425536101567558944225311279296875f,0.4011000141441822819388107745908200740814208984375f,0.069600005084275995415055149351246654987335205078125f,0.2693000068113207134956610389053821563720703125f,-0.0993000113189221877973977825604379177093505859375f,-0.156200000691413709574817403336055576801300048828125f,0.0127000024497510022314372690743766725063323974609375f,-0.5747000204093755115763997309841215610504150390625f,-0.407799994100630114335359621691168285906314849853515625f,-0.47550000234320755065908770120586268603801727294921875f,-1.15010002841949443563862587325274944305419921875f,-0.046599973404408245869490201584994792938232421875f,-0.8605000095516448510579721187241375446319580078125f,-0.29379999111294718971976180910132825374603271484375f,0.32939998847842222318149651982821524143218994140625f,-0.1079999986365436370761017315089702606201171875f,-1.161600029581785431531670838012360036373138427734375f,-0.2801999938070773854548178860568441450595855712890625f,-1.109000020198524172343468308099545538425445556640625f,1.1264999989271160529824555851519107818603515625f,-0.2041999991416929560728021897375583648681640625f,0.87330000070929525435303730773739516735076904296875f,0.65480000250488490376454819852369837462902069091796875f,0.430399978375435143806981841407832689583301544189453125f,0.59959998694211247283192278700880706310272216796875f,-0.543699988174438431087764911353588104248046875f,0.15199999186396606631888062111102044582366943359375f,-0.6475000063702462949066784858587197959423065185546875f,0.982499997913837574969875277020037174224853515625f,0.18720000429749461545725353062152862548828125f,1.00069999780803886579860773053951561450958251953125f,1.28690001395940800676953585934825241565704345703125f,0.112200012195109888324395797098986804485321044921875f,1.14610001012086826222002855502068996429443359375f,0.3088999946475023961056649568490684032440185546875f,0.36459997215271044979090220294892787933349609375f,0.2084999767839910145283965903217904269695281982421875f,0.0523000298857692058618340524844825267791748046875f,0.61959998958706830052278746734373271465301513671875f,-0.1119000079616905285462280517094768583774566650390625f,0.2038999731525776848428677112678997218608856201171875f,-0.1361999860137703688423016501474194228649139404296875f,0.371099997045099438963688953663222491741180419921875f,-0.174699993717670309933964745141565799713134765625f,-0.4918999875098466478817726965644396841526031494140625f,-0.532999979674816160013506305404007434844970703125f,-0.7071999640643600315570438397116959095001220703125f,-0.51349997864663610158686424256302416324615478515625f,-0.45500000935047868466654108488000929355621337890625f,-0.23570000006183977614426794389146380126476287841796875f,-0.264300002024322744365036896851961500942707061767578125f,-0.35539998970031749792042319313623011112213134765625f,-0.609899967501312811890556986327283084392547607421875f,-0.2268999866530301279254899782245047390460968017578125f,0.19380000601410873883878593915142118930816650390625f,-0.3628000098586081634266520268283784389495849609375f,0.38610001980662378429087766562588512897491455078125f,0.13540000340938573852866966262809000909328460693359375f,-0.021100002744793844300374985323287546634674072265625f,0.0466999989926815128882253702613525092601776123046875f,-0.180399995958805181572870424133725464344024658203125f,-0.3179999822080137317215076109278015792369842529296875f,0.11780001224279390559246394332149066030979156494140625f,-0.207800000619888347586083909845910966396331787109375f,-0.231299992156028810796897232648916542530059814453125f,-0.0256999961316586311710352674708701670169830322265625f,-0.43359999983906760689933435060083866119384765625f,-0.68349998602271089254145408631302416324615478515625f,-0.652999995321035253681429821881465613842010498046875f,0.7248000058948995683039129289682023227214813232421875f,1.0365999817490578749357155174948275089263916015625f,0.48739998879134649456545957946218550205230712890625f,-0.2509999746978284207443721243180334568023681640625f,-1.16339995834827458764948460157029330730438232421875f,-0.653899981236457872313394545926712453365325927734375f,0.34179999433159824473449361903476528823375701904296875f,0.420999988183379247885795848560519516468048095703125f,0.2024999932944775427046124605112709105014801025390625f,-0.241599991583824280638737036497332155704498291015625f,-0.3983999818921091229384501275490038096904754638671875f,-0.55159999039173113200007492196164093911647796630859375f,-0.185800003093480992077957125729881227016448974609375f,-0.54210000858604889817371486060437746345996856689453125f,-0.080499997481703611246217633379274047911167144775390625f,-0.43339999738931656025897609652020037174224853515625f,-0.924199992734193731536151972250081598758697509765625f,-0.6985000001043080164464527115342207252979278564453125f,-0.55899999952316259399509590366506017744541168212890625f,0.08109994943588938465239834840758703649044036865234375f,0.730599977695941760202913428656756877899169921875f,0.0597999969542026832414194359444081783294677734375f,0.743599980802833737669743641163222491741180419921875f,0.8939000137396158063296525142504833638668060302734375f,-0.2815999924778938012792650624760426580905914306640625f,0.80770002444759025905796079314313828945159912109375f,0.09330000259056714373429031184059567749500274658203125f,-0.06560000097900620108504199379240162670612335205078125f,0.197100007077306571545705082826316356658935546875f,-1.516600061619282513447615201584994792938232421875f,-0.089000004738569060691588674671947956085205078125f,-1.19800003827363266140082487254403531551361083984375f,-0.351100006470084213816562623833306133747100830078125f,0.1519999985694884259146419935859739780426025390625f,-0.29770001016110192448849147695000283420085906982421875f,-0.3416000078260896799520196509547531604766845703125f,-0.327599992656707694749229631270281970500946044921875f,-0.2099999918788670871805379647412337362766265869140625f,0.06950000154972091337413075962103903293609619140625f,-0.212399993485212323918176480219699442386627197265625f,0.11250001128762965318941269288188777863979339599609375f,0.812300013047456470616225487901829183101654052734375f,0.3809999754428867735356334378593601286411285400390625f,0.791900002038479211563526405370794236660003662109375f,-0.54060000883936876192592535517178475856781005859375f,0.16480000813007311677438337937928736209869384765625f,-0.657800024610758260479315140401013195514678955078125f,-0.234400010979175732472867821343243122100830078125f,0.8033999674379828093151445500552654266357421875f,-0.015000031776726263643695347127504646778106689453125f,-0.312399991175532409926063337479718029499053955078125f,0.13299999129772199779608854441903531551361083984375f,-0.34820000071376544514833994981017895042896270751953125f,0.729599999868869364405554733821190893650054931640625f,0.4567999730229381771806629330967552959918975830078125f,0.66479998339414603236718903644941747188568115234375f,-0.34730001595318338303286509471945464611053466796875f,-0.0155999988555908242204850466805510222911834716796875f,-0.1875000030174851506359345876262523233890533447265625f,0.5248000063419340222026221454143524169921875f,0.51179998211264621232885474455542862415313720703125f,0.59019999585598714730849678744561970233917236328125f,0.190999963894486146642748281010426580905914306640625f,0.553699994954466756524880111101083457469940185546875f,-0.440999991461634710532280223560519516468048095703125f,-0.3675999800279738938257878544391132891178131103515625f,-0.47969998533576718724447118802345357835292816162109375f,0.058999995499849244851020557689480483531951904296875f,0.2855999980360268697410219829180277884006500244140625f,0.41090001005530363276108118952834047377109527587890625f,0.19919998766779922760861154529266059398651123046875f,1.2260999648213388724116157391108572483062744140625f,0.67259999638199818861039602779783308506011962890625f,-0.30059999736547471371750361868180334568023681640625f,-0.4893999914586544264238909818232059478759765625f,-0.37529999786317347343356232158839702606201171875f,0.081399985259771323597988157416693866252899169921875f,0.5661999795317651518189450143836438655853271484375f,-0.07650001382827742446579577517695724964141845703125f,0.57659997906684878898886381648480892181396484375f,0.77539995766282121536505655967630445957183837890625f,0.474499981656670843932488423888571560382843017578125f,-0.493000013649463664222594161401502788066864013671875f,-0.55110001012086851090998607105575501918792724609375f,-0.079499994695186426696409398573450744152069091796875f,-0.745999996602535464518268781830556690692901611328125f,-1.01559996339082747596194167272187769412994384765625f,-0.547399988567829343111270645749755203723907470703125f,0.37720000906586648170559783466160297393798828125f,0.08200000289082520765759909409098327159881591796875f,0.277200005266070359510877096909098327159881591796875f,0.108800004822015683458857893128879368305206298828125f,0.400900011396407851549383849487639963626861572265625f,0.1782000064134596772902341399458236992359161376953125f,-0.07919997366070763433754109428264200687408447265625f,-0.9317999546945099353933983366005122661590576171875f,-0.41819998190104978874614971573464572429656982421875f,-0.083800007653236419091768993894220329821109771728515625f,-0.66690001056939340884355260641314089298248291015625f,-0.186500002577900669820110124419443309307098388671875f,-0.15139999822378147342760712490417063236236572265625f,0.346399997478723520316634676419198513031005859375f,-0.1651000027149915894142395700328052043914794921875f,-0.22899998566508283914799903868697583675384521484375f,-1.158199987041950151933633605949580669403076171875f,-0.6657000003039836855123212444595992565155029296875f,-0.070799984094500612030742558999918401241302490234375f,0.01290001588910782981400870994548313319683074951171875f,1.018999988459050154432361523504368960857391357421875f,0.053799989324808394908217223928659223020076751708984375f,0.92119998824894422373432689710170961916446685791015625f,-0.3422000164419414236505190274328924715518951416015625f,-0.52019999201893785123473890052991919219493865966796875f,-0.31559999591261134188613368678488768637180328369140625f,0.3914999715089795273570416611619293689727783203125f,-0.92459996261596710809271826292388141155242919921875f,0.1407000010490413455244151919032447040081024169921875f,0.414800003063678701664684922434389591217041015625f,0.170199994105100671504260390065610408782958984375f,0.43220000038594008628933806903660297393798828125f,0.8252000234603880102213224745355546474456787109375f,0.0501999929130079181049950420856475830078125f,0.581800012548268075107671393197961151599884033203125f,-0.643999989092350233477191068232059478759765625f,0.0353999940216542796633802936412394046783447265625f,-0.68859999369233815258439790341071784496307373046875f,0.19129998917579627004670328460633754730224609375f,-0.47959999077916126708487354335375130176544189453125f,0.2883000193729998716918316858937032520771026611328125f,0.7072000199437145084857547772116959095001220703125f,-0.126800002229213504278959590010344982147216796875f,0.79160002801716355946837211376987397670745849609375f,-1.00480001677274710658593903644941747188568115234375f,0.193599993729591357549679742078296840190887451171875f,-0.86720002165734777488381723742350004613399505615234375f,0.1166999985456467214817166677676141262054443359375f,-0.18299999785423270992623656638897955417633056640625f,0.030699998217821100610080975457094609737396240234375f,-0.5778000059843062530262614018283784389495849609375f,0.69359997256994265768526020110584795475006103515625f,-0.3414000266849994691398251234204508364200592041015625f,-0.5468999985739591007671833722270093858242034912109375f,0.3115999836117031573934355037636123597621917724609375f,-0.347099998527764963540676035336218774318695068359375f,1.0585999981999396624132714350707828998565673828125f,-0.12339998919367811680558588705025613307952880859375f,0.8275999968662854922030192028614692389965057373046875f,-0.24720000265836716835110564716160297393798828125f,0.8501999809920786077555021620355546474456787109375f,-0.029000021986663515605187058099545538425445556640625f,0.37649998868256762118988945076125673949718475341796875f,0.418200010585784820449362086947076022624969482421875f,0.1545999995708466911992218228988349437713623046875f,0.791699993777275068396193091757595539093017578125f,0.4938999967336652474614311358891427516937255859375f,0.11300001871585863000291283242404460906982421875f,-0.4073999896109103957542174612171947956085205078125f,-0.39450000639259830137461904087103903293609619140625f,0.201599990242719739885757235242635942995548248291015625f,0.49509997565075780645571512650349177420139312744140625f,0.151999990075826818536341988874482922255992889404296875f,0.42080000741481793369302977225743234157562255859375f,0.4268999951839447248858050443232059478759765625f,0.33679999649226655122902229777537286281585693359375f,-0.463599986279010778389420011080801486968994140625f,-0.55949997368454962298756072414107620716094970703125f,-0.064999975636601536876924001262523233890533447265625f,0.078800004076957586018892243373556993901729583740234375f,-0.224799988050758958735286796581931412220001220703125f,-0.0549999970942736116086280162562616169452667236328125f,0.1846000155150890886801562373875640332698822021484375f,-0.293999987751245583922354853712022304534912109375f,-0.066499996095895852477042353712022304534912109375f,0.17059997620582567634528459166176617145538330078125f,1.0983999787628651034765425720252096652984619140625f,0.69829999616444116128377572749741375446319580078125f,-0.318799996328353874019967406638897955417633056640625f,-0.811399989879131311454329988919198513031005859375f,-0.1733999929189682376318160095252096652984619140625f,-0.2590000042915343581029219421907328069210052490234375f,-0.10840000379681580255919470801018178462982177734375f,0.023900000980496383107265501166693866252899169921875f,-0.28379999722242355009171888013952411711215972900390625f,-0.674099992813169901495484737097285687923431396484375f,-0.1180999908089637451258369083006982691586017608642578125f,-0.36440002796649917371496485429815948009490966796875f,-0.2320000382065767752237661625258624553680419921875f,0.30059998976588264696374608320184051990509033203125f,-0.35239998320341126003540921374224126338958740234375f,-0.978399959242344241516775582567788660526275634765625f,-0.5169999884366991427242510326323099434375762939453125f,0.049600000762939355780645200866274535655975341796875f,-0.0641999964594841543430447927676141262054443359375f,-0.21999999985098828148011307348497211933135986328125f,0.28680000312328335354550290503539144992828369140625f,-0.8415999944895504736308566862135194242000579833984375f,0.1096999937176708073138797772116959095001220703125f,-0.63359996616244362854786231764592230319976806640625f,-0.0272999910309914550765597596182487905025482177734375f,-0.176799999248981265509428340010344982147216796875f,0.7965999914348120825025034719146788120269775390625f,-0.109800027538836442175806951127015054225921630859375f,0.1329000005967913011772907339036464691162109375f,-0.037399997769296200456068390849395655095577239990234375f,0.01929999941661952878035890535102225840091705322265625f,-0.662900010114908067038186345598660409450531005859375f,0.0798000078320499905970564213930629193782806396484375f,-0.640500016033649810509587041451595723628997802734375f,0.1022999640226363027295519714243710041046142578125f,-0.45099998617172243342565707280300557613372802734375f,0.2138999902516600659652112881303764879703521728515625f,-0.3899999920651315843400652738637290894985198974609375f,0.2571999861925842179033452339353971183300018310546875f,-0.296800001670419977273240874637849628925323486328125f,-0.6658000114381312783962130197323858737945556640625f,0.452999991446733307753902408876456320285797119140625f,-0.52440002480000280637995047072763554751873016357421875f,0.210200011837482492182971327565610408782958984375f,-0.96219998045563670530100353062152862548828125f,0.0148000367030505231014103628695011138916015625f,-0.361000009715557013123543583787977695465087890625f,-0.01759999712705617724850526428781449794769287109375f,-0.1477999975651502762019617875921539962291717529296875f,0.265799993705749582062480840249918401241302490234375f,-0.3901999827802182796432362010818906128406524658203125f,0.262400011627375973066733649829984642565250396728515625f,-0.30770001567453153512587959994561970233917236328125f,-0.07639999829828736377379527766606770455837249755859375f,-0.12630000154376030963021548814140260219573974609375f,-0.1764000050783156581246657879091799259185791015625f,-0.86519996534585974501396776759065687656402587890625f,-0.0615999597698448919658176237135194242000579833984375f,0.00440001038312942682750872336328029632568359375f,0.41959998437166223794747565989382565021514892578125f,0.16879999819099911206876640790142118930816650390625f,0.16980000872611977769111035740934312343597412109375f,0.374799978923797905139281283481977880001068115234375f,0.13619999294281026624986452588927932083606719970703125f,-0.08579999370574942219036529422737658023834228515625f,0.455400013318657759242569227353669703006744384765625f,-0.24499998345971096824769119848497211933135986328125f,-1.15749998673796650194844914949499070644378662109375f,-0.560099992582201888779991350020281970500946044921875f,0.4390000009760262056346391545957885682582855224609375f,0.38669998623356238720560895671951584517955780029296875f,0.451299991429597036063370296687935478985309600830078125f,0.3015999897956849196134498924948275089263916015625f,0.68659997020065810513500537126674316823482513427734375f,0.106799987477064373120327900323900394141674041748046875f,0.40139999434947970513576365192420780658721923828125f,0.10579998551011104979124866076745092868804931640625f,0.383699990335106821959243461606092751026153564453125f,0.0262000103771686809750462998636066913604736328125f,-0.5213999851107598004773535649292171001434326171875f,0.07550000366568543341827535186894237995147705078125f,-0.80180000252723715448155417107045650482177734375f,-1.08549997788667695886033470742404460906982421875f,-0.657499984055757380474460660479962825775146484375f,0.176599999332427870513129164464771747589111328125f,-0.16460000046491618519439725787378847599029541015625f,-0.392300009247660508293620296171866357326507568359375f,0.272399999743699883225644953199662268161773681640625f,0.6916000017166135105384228154434822499752044677734375f,0.65200002054870143108900037987041287124156951904296875f,-0.0921999961018562796954967097917688079178333282470703125f,-0.33459998990371830895895755020319484174251556396484375f,-0.105699995721876660592641883340547792613506317138671875f,0.02020000692009915610469761304557323455810546875f,-0.67409998666644099785116850398480892181396484375f,-0.4771000032216310415833504521287977695465087890625f,-0.201399996733665265224999529891647398471832275390625f,-0.97320001192688909696926202741451561450958251953125f,-0.351700000557303216197624351480044424533843994140625f,0.1256000107765198325893152286880649626255035400390625f,-0.16289999268054966563568086712621152400970458984375f,-0.067199998709559505272181922919116914272308349609375f,0.045199994403123611164119211025536060333251953125f,0.5902000093042847339575018850155174732208251953125f,0.46640001617968085412258005817420780658721923828125f,0.285000002607703084578361085732467472553253173828125f,0.4135000037923453053423372693941928446292877197265625f,0.54250001836568129132132298764190636575222015380859375f,-0.04680000282526020072282335604541003704071044921875f,-1.05110003373920957159270983538590371608734130859375f,-0.387900020992756111581911682151257991790771484375f,0.6911999835550783899407178978435695171356201171875f,-0.13930003080219055533461869345046579837799072265625f,-0.9409000056035814463228916793013922870159149169921875f,-0.0263999863028530068476129599730484187602996826171875f,-0.86070000546351099757202973705716431140899658203125f,0.31260000122487507923096927697770297527313232421875f,0.1139999957978725664275998497032560408115386962890625f,0.1261999921232461741027464086073450744152069091796875f,-1.1847999935269353244393641944043338298797607421875f,0.08740000153183924425093209720216691493988037109375f,-1.0173999949380754248551284035784192383289337158203125f,-0.9504000048995013827379807480610907077789306640625f,0.10060000585913630999357337714172899723052978515625f,-0.7026000024542213839851001466740854084491729736328125f,0.880900002157687822545995004475116729736328125f,-0.0500000162422651328597567044198513031005859375f,0.946700014080107621339266188442707061767578125f,0.068800026279687731545209317118860781192779541015625f,0.8169999653398993810782258151448331773281097412109375f,0.048999992556870264337476328364573419094085693359375f,-0.50800003376603175553327673696912825107574462890625f,-0.62639999822378111815623924485407769680023193359375f,-0.52640001185834395247553629815229214727878570556640625f,-0.16440000008270139186805636200006119906902313232421875f,0.09579999530762439707842759162303991615772247314453125f,-0.07900000073388213639447030800511129200458526611328125f,-0.27569998018741603829084851895458996295928955078125f,0.841599968785047725106096550007350742816925048828125f,-0.14430000897198891340877935363096185028553009033203125f,-0.5270000241994861056582521996460855007171630859375f,0.436199993985891154579803696833550930023193359375f,-0.27840002111941597462418940267525613307952880859375f,-0.3619000069558619969711799058131873607635498046875f,0.324999995231628258096634453977458178997039794921875f,-0.28830001739859600906612513426807709038257598876953125f,0.0493999922037122729534530662931501865386962890625f,-0.70359999730586952892963381600566208362579345703125f,-0.0361999879509207200811715665622614324092864990234375f,-0.77050003089755814134065303733223117887973785400390625f,-0.384000002950429519810171541394083760678768157958984375f,-0.70550001521408545190894301413209177553653717041015625f,1.090200021672249164339518756605684757232666015625f,0.2099999981373550372865111057762987911701202392578125f,-0.0285999999880790500839111700770445168018341064453125f,-0.12250000141561023792036166923935525119304656982421875f,-0.314900008833408351360816368469386361539363861083984375f,0.34819999508857701897568404092453420162200927734375f,0.273400003945827307916260906495153903961181640625f,0.50590001012980945205299576628021895885467529296875f,-0.36699997681379326053274780861102044582366943359375f,-1.089399959570169773570569304865784943103790283203125f,-0.6620999851912261480180177386500872671604156494140625f,-0.4522000101089478363292073481716215610504150390625f,-0.25589999813437458442422212101519107818603515625f,-0.44550000634789466147367420489899814128875732421875f,0.49979998428821570399804841144941747188568115234375f,0.957199965554476062123967494699172675609588623046875f,0.4936999851197005906300319111323915421962738037109375f,0.16520000542998325432364481457625515758991241455078125f,-0.2393000066250561275182917597703635692596435546875f,-0.04160000231266014480979720246978104114532470703125f,0.438200022208690587177670749952085316181182861328125f,0.65270002524852710479308370850048959255218505859375f,0.050900007075071140860700324992649257183074951171875f,0.66739999154806139980422585722408257424831390380859375f,0.92969997278153915232223880593664944171905517578125f,0.401099986577034162138488682103343307971954345703125f,0.64860000594854361199992354158894158899784088134765625f,0.7439999908059835576068508089520037174224853515625f,0.29409999240338802195537937222979962825775146484375f,0.110600009509921104466911856434307992458343505859375f,-0.3714999938234686549520802145707421004772186279296875f,-0.41950000702589751444548937797662802040576934814453125f,0.4477999870598317055936377073521725833415985107421875f,1.1016999737352133337253690115176141262054443359375f,0.743499989077448741880971283535473048686981201171875f,0.476200008141994590715739832376129925251007080078125f,0.67929999388456341335995603003539144992828369140625f,0.411900000697374313318732674815692007541656494140625f,0.109400007253885267477500065069762058556079864501953125f,-0.38949999361485243554170665447600185871124267578125f,-0.077299998444318784862616666941903531551361083984375f,0.23180000491142305207858953508548438549041748046875f,-0.29340002138018572708233477897010743618011474609375f,0.314099997022748045338857991737313568592071533203125f,-0.63380001853108414167081718915142118930816650390625f,-0.4901999996930361813696208628243766725063323974609375f,0.3767000169485807958835721365176141262054443359375f,0.3947999902486800660739163504331372678279876708984375f,0.3196999998643994533864542972878552973270416259765625f,-1.035200002151727804999836735078133642673492431640625f,-0.1994000132143491921254963017418049275875091552734375f,-0.909799995799362104520469074486754834651947021484375f,0.098799995136261031802860088646411895751953125f,0.61579997984766965402059213374741375446319580078125f,0.259399979053437590437170001678168773651123046875f,1.151300028365850547373838708153925836086273193359375f,-0.0404000023663042551191892925999127328395843505859375f,1.015500026799738719773813500069081783294677734375f,-0.26419999377727521050474024377763271331787109375f,-0.16019999529719353148493610206060111522674560546875f,-0.39519999481290568610347690992057323455810546875f,-0.589100012221932534117740942747332155704498291015625f,0.375399993723630753805764470598660409450531005859375f,-0.4667000197052957322085831037838943302631378173828125f,0.1058999976873398196630660095252096652984619140625f,0.4023999869287013808616393362171947956085205078125f,-0.054700031827390471761418666574172675609588623046875f,-0.457899992904067243415511256898753345012664794921875f,0.219999989867210477001435720012523233890533447265625f,-0.5709000095710157207662405198789201676845550537109375f,-0.529400000324845354526814844575710594654083251953125f,0.383799991261958961530353917623870074748992919921875f,-0.6036000232711435931065580007270909845829010009765625f,-0.07829998741000887552132780911051668226718902587890625f,0.8139999782145022066970341256819665431976318359375f,-0.03490001755058769195017021047533489763736724853515625f,-0.3574999982863662495446988032199442386627197265625f,-0.6007999746322634226913805832737125456333160400390625f,-0.4792999849811196977356075876741670072078704833984375f,-0.50590001944303519820778092253021895885467529296875f,-0.007999993532896265691078951931558549404144287109375f,-0.55250002373009954315108416267321445047855377197265625f,-0.6559000033870339141373051461414434015750885009765625f,0.3815999888271093265501576752285473048686981201171875f,-0.49450001257658005027195713410037569701671600341796875f,-1.3648000376343727424455209984444081783294677734375f,0.03940001798272074751139371073804795742034912109375f,-1.0902000235721474030725630655069835484027862548828125f,0.4540000087618825119761822861619293689727783203125f,0.05879999878704555982267265790142118930816650390625f,0.16360000147819508953261902206577360630035400390625f,-0.044099997395277146239322973997332155704498291015625f,0.4770000187605620833863895313697867095470428466796875f,-0.1268000095307826313728583045303821563720703125f,-0.224000013381242535359660905669443309307098388671875f,0.1275999958604574491261018920340575277805328369140625f,-0.71999997735023502087869928800500929355621337890625f,-0.97679995931386986995903498609550297260284423828125f,-0.760799975228309666164250302244909107685089111328125f,0.72219999416470503916798406862653791904449462890625f,1.282399995422363048191982670687139034271240234375f,0.7475000041723252053316173260100185871124267578125f,-0.24040000475049005945749058810179121792316436767578125f,0.18479998942911635140262660570442676544189453125f,0.0119999996870756575617633643560111522674560546875f,0.85579999847412135949298317427746951580047607421875f,1.191699969637394218580084270797669887542724609375f,0.8384999809563158379432934452779591083526611328125f,0.20860000326633443279433777206577360630035400390625f,0.293300005924701512327601449214853346347808837890625f,0.2757000101387501178606953544658608734607696533203125f,0.10399999773502344169173738919198513031005859375f,0.045599997067451436549845311674289405345916748046875f,0.3992000119566918581170966717763803899288177490234375f,0.186199995476007462702483508110162802040576934814453125f,0.4363999905310571580940859348629601299762725830078125f,0.418500003606081083518120067310519516468048095703125f,0.39980000976920138100467738695442676544189453125f,0.41909999143481257988241850398480892181396484375f,-0.166500003471970359214537893421947956085205078125f,-0.089599991673231238475949567146017216145992279052734375f,-0.31749997528269913171783400684944353997707366943359375f,0.340400017565488599569079042339581064879894256591796875f,-0.246000001966953207865884678540169261395931243896484375f,-0.63560000394061189066263750646612606942653656005859375f,-0.132799998980760503020093210579943843185901641845703125f,0.5291999948203562897930396502488292753696441650390625f,1.1895999868303537283509285771287977695465087890625f,0.736000008001923777811725813080556690692901611328125f,-0.306399995094537747064578070421703159809112548828125f,-0.7172999871194363219473189019481651484966278076171875f,-0.10979998965263371157874416894628666341304779052734375f,-0.1127999876558778424850970623083412647247314453125f,-0.51249999560415704369376044269301928579807281494140625f,-1.131500034391880404882613220252096652984619140625f,-0.34660000231265986059270289842970669269561767578125f,-0.923100012490153165600759166409261524677276611328125f,-0.31320000224113453413110619294457137584686279296875f,-0.287199986267089979463662530179135501384735107421875f,-0.20599998460710045034005588604486547410488128662109375f,1.06319997452497450041164483991451561450958251953125f,0.172799991083145254577857485855929553508758544921875f,1.067999976098537029400858955341391265392303466796875f,-0.5877999961495401493039025808684527873992919921875f,-0.24460001223683303095413066330365836620330810546875f,-0.7240000021681185504718314405181445181369781494140625f,0.690799988117813956733925806474871933460235595703125f,-0.435199980098009309159579061088152229785919189453125f,0.565200005131959670734431711025536060333251953125f,-1.1115000039935107878363851341418921947479248046875f,-0.13079997418522903984694494283758103847503662109375f,-1.1513000082120303790844673130777664482593536376953125f,-0.38190001276731511126172335934825241565704345703125f,-0.123600005799531675876323788543231785297393798828125f,-0.36970000992268314821131980352220125496387481689453125f,-0.76400002455711391036174973123706877231597900390625f,-0.364799997401237252603323213406838476657867431640625f,-0.63120000456571567770680530884419567883014678955078125f,-0.357800013937801253405268653295934200286865234375f,-0.382599991129338679041183013396221213042736053466796875f,-0.36400000013783573304948504301137290894985198974609375f,0.1762999888777727619526558555662631988525390625f,0.69159997742772105766562162898480892181396484375f,0.0344999589696524111559483571909368038177490234375f,-0.8641000158540901310288973036222159862518310546875f,-0.473799986754357804752402216763584874570369720458984375f,-0.64089999334737657310512304320582188665866851806640625f,-0.2944000093214214075487689115107059478759765625f,-0.011999995850026701216251012738212011754512786865234375f,-0.12559999742135408151710862512118183076381683349609375f,-0.3604000209182505187754941289313137531280517578125f,-0.4943999980151649875637076547718606889247894287109375f,-0.5036000108659266505384266565670259296894073486328125f,-0.345100013667344995837993337772786617279052734375f,-0.108199995982646957060069325962103903293609619140625f,-0.147899999088048961226604660623706877231597900390625f,-0.3224000037670133878009437466971576213836669921875f,-0.1586000032663346104300217120908200740814208984375f,-0.145799993556738005651141065754927694797515869140625f,-0.754799983692169274007710555451922118663787841796875f,-0.3792999956727027832670273710391484200954437255859375f,-0.348200004774331883794502573437057435512542724609375f,-1.0130000120103355154554947148426435887813568115234375f,-0.65530001760721201531367796633276157081127166748046875f,-0.089200015532970411413771216757595539093017578125f,0.34569997646212602404602876049466431140899658203125f,-0.350600015619396909727356614894233644008636474609375f,-0.18699999927729360482686615796410478651523590087890625f,-0.407900000149756525313904376162099651992321014404296875f,-0.5221000135220588145301690019550733268260955810546875f,-0.0799999964237212424222889239899814128875732421875f,0.451100005799531800221302546560764312744140625f,0.24760000435411910757466102950274944305419921875f,0.4619999988675118629544158466160297393798828125f,0.6504999848902226489144595689140260219573974609375f,0.59379999535977834312916456838138401508331298828125f,-0.033799993795156524356571026146411895751953125f,-0.2867999901592732658883733165566809475421905517578125f,0.05510001202821734889170102178468368947505950927734375f,0.081599991732835608360119294957257807254791259765625f,0.7388000015437601319945315481163561344146728515625f,0.21459999934732909565582303912378847599029541015625f,0.166799996715784271117399839567951858043670654296875f,-0.3834000240623947064477761159650981426239013671875f,0.09279999234974400934561344911344349384307861328125f,0.60099999973177897771847710828296840190887451171875f,1.176799995076656113468516196007840335369110107421875f,0.7781000119686127813878329106955789029598236083984375f,-0.22639999732971194390529490192420780658721923828125f,0.1775000078976152195764370844699442386627197265625f,0.22800001998245722489855324965901672840118408203125f,-0.17519999410510056492285002605058252811431884765625f,0.08020000468492494150041238754056394100189208984375f,0.279400018951296846836385157075710594654083251953125f,0.476199981319904541710030798640218563377857208251953125f,1.1694999577961866510378285965998657047748565673828125f,0.600499973565340017245262060896493494510650634765625f,0.04579999832510939228313873172737658023834228515625f,-0.21549999845027922873441639239899814128875732421875f,-0.120300001290440494727818077080883085727691650390625f,-0.0527999949574471383328955198521725833415985107421875f,0.4368999951839447337675892413244582712650299072265625f,-0.2553999929785728539854972041212022304534912109375f,0.48979997877478620438296275096945464611053466796875f,-0.094800005261599817885098673286847770214080810546875f,0.1684000198900701406046209740452468395233154296875f,0.612599993997811953505561177735216915607452392578125f,0.3034000080063938842300785836414434015750885009765625f,1.245100023651122711498828721232712268829345703125f,0.20739998826980610147074912674725055694580078125f,0.967900009444356168586409694398753345012664794921875f,0.5693000244133177289995728642679750919342041015625f,0.5047999936014411215268182786530815064907073974609375f,0.5299000048659741057832661681459285318851470947265625f,0.420099990272522205714267329312860965728759765625f,-0.5725999732851985157822127803228795528411865234375f,0.294500005871057179973604434053413569927215576171875f,-0.81730001140832886363796205841936171054840087890625f,-0.252599983716011333712003761320374906063079833984375f,-0.79610000483095666456989647485897876322269439697265625f,-0.45780001865029351648672673036344349384307861328125f,-0.035000004023313380230320035479962825775146484375f,-0.32760001109689473874908571815467439591884613037109375f,0.6132000186324120249992120079696178436279296875f,-0.37239999810457202755742400768212974071502685546875f,0.3920000191330912375775596956373192369937896728515625f,-1.025000016987323903094875277020037174224853515625f,0.25000000119209264681785498396493494510650634765625f,-0.7986000128775836426342493723495863378047943115234375f,-0.57700002300739328120471327565610408782958984375f,-0.43540000147223434368015659856610000133514404296875f,-0.6856000158801673904918061452917754650115966796875f,0.305699965882300972452867426909506320953369140625f,-0.68699998575448972815138404257595539093017578125f,0.256099994325637680958607234060764312744140625f,-0.015400043195486379232761464663781225681304931640625f,-0.6455999800801277554995749596855603158473968505859375f,0.003399987256526626477892705224803648889064788818359375f,-0.0760999832056459268869730294682085514068603515625f,-0.395999971233308822871066467996570281684398651123046875f,-0.26629997928515092286971821522456593811511993408203125f,-0.211099990004301130142039255588315427303314208984375f,0.2875999841630461073549440698116086423397064208984375f,-0.150099999548494567047640657619922421872615814208984375f,0.1914000002354381280156303546391427516937255859375f,-0.243599997752904806613827304317965172231197357177734375f,0.322800001066922970238692869315855205059051513671875f,0.32230000663995728160671205841936171054840087890625f,-0.014000002950429912829122258699499070644378662109375f,0.4016000026106834042138871154747903347015380859375f,0.77719999938011152806893733213655650615692138671875f,0.596400008207559562123378782416693866252899169921875f,-0.5473999976575376624765567612485028803348541259765625f,-0.75149996942281749312542160623706877231597900390625f,-0.394099991360306933785295768757350742816925048828125f,0.1909999872148038235764033743180334568023681640625f,0.5405999677121642310595461822231300175189971923828125f,0.19989999035596872634101828225539065897464752197265625f,-0.3478000046059490646399581237346865236759185791015625f,-0.28339999215528377529693671021959744393825531005859375f,-0.362099996162205839400627382929087616503238677978515625f,0.169200001153349877114351329510100185871124267578125f,0.214399998126924007380011971690692007541656494140625f,0.24060000336319209424118525930680334568023681640625f,-0.125400000727176663684048207869636826217174530029296875f,-0.3596999990262090829418184512178413569927215576171875f,0.263700011569261594246427193866111338138580322265625f,-0.04020002033114433714899860206060111522674560546875f,0.32699998128414176079559183563105762004852294921875f,0.327099985712766549994512388366274535655975341796875f,-0.79279998989105227025220301584340631961822509765625f,-1.21919997708797467339536524377763271331787109375f,-0.883599984490871293019154109060764312744140625f,-0.213799994838237805705460914396098814904689788818359375f,-0.55149998702481395351782111902139149606227874755859375f,-0.125699990767240532552051490711164660751819610595703125f,0.02959999107718491728746812441386282444000244140625f,0.5262999765098097526561105041764676570892333984375f,0.44019999008178700705684605054557323455810546875f,0.152200004446506387267845639144070446491241455078125f,0.000900010502338233209229656495153903961181640625f,0.246700010615587217444044654257595539093017578125f,0.011200007992982818905147723853588104248046875f,-0.178399984723329740887720618047751486301422119140625f,0.3346000151425598811982808911125175654888153076171875f,0.221399986600876008679961159941740334033966064453125f,0.681499966740608531523548663244582712650299072265625f,0.2684999870657922560468477968242950737476348876953125f,0.313599991941452049815097780083306133747100830078125f,-0.073300001379847490312613444984890520572662353515625f,-0.07629997844696045916634830064140260219573974609375f,0.409799976241588925773839946486987173557281494140625f,-0.14510000316202609216276186998584307730197906494140625f,-0.52280001820325860961702346685342490673065185546875f,-0.385599992299079818991458523669280111789703369140625f,-0.56680000967979438097898992054979316890239715576171875f,0.49730001990199124151104115298949182033538818359375f,-0.1008000027954576438560252427123486995697021484375f,0.5643000247672202984716705032042227685451507568359375f,0.1427999927222725062847530352883040904998779296875f,-0.128199995535612121244639638462103903293609619140625f,0.02339999727755781577798188664019107818603515625f,0.787200011692196000012700096704065799713134765625f,0.01979998786449466141590391998761333525180816650390625f,0.71260001070573952830500275013037025928497314453125f,-0.37240001021176583062555209835409186780452728271484375f,-0.114799997699260691863543115687207318842411041259765625f,-0.34720000597387556950934595079161226749420166015625f,-0.7389000249527395425275244633667171001434326171875f,-0.417599988447129699675741676401230506598949432373046875f,-0.53069999831095338027608931952272541821002960205078125f,0.03400002574920701903238295926712453365325927734375f,-0.576199976998567731101275057881139218807220458984375f,-0.00539995702952122957896108346176333725452423095703125f,1.0295999852657313766712832148186862468719482421875f,0.29199998486042044731902933563105762004852294921875f,1.0799999873340129230570028084912337362766265869140625f,-0.2379000092022121037871329463087022304534912109375f,0.002399997694790434454858996105031110346317291259765625f,-0.09490000207200643966842790177906863391399383544921875f,0.024099991136789622458991289022378623485565185546875f,-0.6745999602317815924124033699627034366130828857421875f,-0.046699984687567142049857693564263172447681427001953125f,-0.67150001055002217498213212820701301097869873046875f,-0.084000005334615490681926530669443309307098388671875f,-0.55890000600963818389033122002729214727878570556640625f,-0.8280000225901602561862091533839702606201171875f,-0.35559998961687089291672236868180334568023681640625f,-0.642800003898143845759705072850920259952545166015625f,0.1882999975800512260093455552123486995697021484375f,-0.21199998858571067472666982212103903293609619140625f,0.038900002582371140391614972031675279140472412109375f,-0.26600001522898697459140748833306133747100830078125f,0.02690000874400144681430901982821524143218994140625f,-0.344600006574392381253346684388816356658935546875f,0.02290000147223469895152447861619293689727783203125f,-0.365900002679228730784188883262686431407928466796875f,0.1463999924123289275712522794492542743682861328125f,1.06949997174739852567881825962103903293609619140625f,0.481399999415874635388945534941740334033966064453125f,-0.136600011402368382018579495706944726407527923583984375f,-0.41880002465918619858342708539566956460475921630859375f,0.225899997836351562607859477793681435286998748779296875f,0.6575999932527540892124306992627680301666259765625f,0.924499990075826616475751507095992565155029296875f,0.5251999909013509437727407203055918216705322265625f,0.094599994131922660045574957621283829212188720703125f,-0.2871000045999883010239273062325082719326019287109375f,-0.31470001618117093844517739853472448885440826416015625f,-0.21339999500513062713480394450016319751739501953125f,0.36950000810623162550427878159098327159881591796875f,-0.220900011098385196106619332567788660526275634765625f,0.06140000761151309660590413841418921947479248046875f,0.646100017422437389313927269540727138519287109375f,-0.0465000037699940804714060504920780658721923828125f,0.12840000051856037543984712101519107818603515625f,0.09520000334382050510839690105058252811431884765625f,0.338000004187226199547922078636474907398223876953125f,0.16420001583099352870931397774256765842437744140625f,-0.2024999722838405347147272550500929355621337890625f,0.30750001892447453855083949747495353221893310546875f,-0.03259999787211409483234092476777732372283935546875f,0.342599997425079294544048025272786617279052734375f,0.2593000078916549711038896930404007434844970703125f,0.097599980735779023888198935310356318950653076171875f,0.4972999589562421274280268335132859647274017333984375f,-0.11880003037750708205777527837199158966541290283203125f,0.10719997851848628300786003819666802883148193359375f,1.05049995419383090933251878595910966396331787109375f,0.563599985310435425134301112848334014415740966796875f,-0.1620000027418135868373383345897309482097625732421875f,-0.47580000734031180353866830046172253787517547607421875f,-0.320600007200241010618668724418967030942440032958984375f,-0.34560000287890435544113643118180334568023681640625f,-0.5859999943673610545147312222979962825775146484375f,0.05380000742971891014576613088138401508331298828125f,0.461999997749924506962315717828460037708282470703125f,-0.83990002578347944250936052412725985050201416015625f,1.24650001740455618204350685118697583675384521484375f,-0.062800003153085537377364744315855205059051513671875f,1.043500014595687641616450491710565984249114990234375f,0.5608999702692027966577370534650981426239013671875f,-0.71379997189044974703620027867145836353302001953125f,0.3020999924182887763635108058224432170391082763671875f,-0.746200021683424807861229055561125278472900390625f,-0.368599981622398098490833717733039520680904388427734375f,-0.52799999221041804542409181522089056670665740966796875f,-0.6484999796152113304970043827779591083526611328125f,0.04859999164342898581026020110584795475006103515625f,-0.7212999943912026257208935930975712835788726806640625f,-0.09229996309131360820998679628246463835239410400390625f,0.630799979549646394616502220742404460906982421875f,-0.05369998884499038727113884306163527071475982666015625f,1.2951000215649610680657133343629539012908935546875f,-0.15319997527003348380958414054475724697113037109375f,1.0229000142499800052320324539323337376117706298828125f,0.932500022947788664851032081060111522674560546875f,-0.04239999288916607156352256424725055694580078125f,0.66110000673085433930964427418075501918792724609375f,-0.6348000057458875744487158954143524169921875f,-0.2647999914407730415177866234444081783294677734375f,-0.53719999528229234186937901540659368038177490234375f,-1.096100020253658158253529109060764312744140625f,-0.0820000019967555004996029310859739780426025390625f,-0.876300014097988633210434272768907248973846435546875f,0.1531999751210211258012350299395620822906494140625f,-0.48299998161196722179511198191903531551361083984375f,0.0869999934658405305043515909346751868724822998046875f,0.7611000284492970280325607745908200740814208984375f,0.149400008296966202436806270270608365535736083984375f,0.5363000168919562771208120466326363384723663330078125f,0.4702999892950057159168864018283784389495849609375f,-0.87219996033906976862226656521670520305633544921875f,0.2375000180304047692203539554611779749393463134765625f,0.097000006243586511800458538345992565155029296875f,0.3063999944984912016110456534079276025295257568359375f,0.1566000008225441053610893504810519516468048095703125f,-0.34200000929832441443068091757595539093017578125f,-0.4155999854445457941665154066868126392364501953125f,-0.155199983823299358931535607553087174892425537109375f,-1.022900021700561268289675354026257991790771484375f,0.211300013166666023067818969138897955417633056640625f,0.193199999707937219994846600457094609737396240234375f,0.617299998593330290219682865426875650882720947265625f,0.2163999997138976727484305229154415428638458251953125f,0.0349999926984310150146484375f,-0.1173000130474566304883410339243710041046142578125f,0.43510001046359558785070476005785167217254638671875f,-0.00459999286532397633209257037378847599029541015625f,-0.77519999753236756845353738754056394100189208984375f,-0.531500006824731752175239307689480483531951904296875f,0.322599997276067806950550220790319144725799560546875f,0.709899983650446042560133719234727323055267333984375f,0.0392999891906977705247072663041763007640838623046875f,0.3143999975681304182018038773094303905963897705078125f,0.67839999612271772999605445875204168260097503662109375f,0.5820000090003014658890378996147774159908294677734375f,-0.2221999984860421051280354731716215610504150390625f,0.10540000862479192988985232659615576267242431640625f,-0.268899992784857655436780987656675279140472412109375f,0.23679999865293499539120603003539144992828369140625f,0.18849999526143079719986417330801486968994140625f,-0.15840001102387901710244477726519107818603515625f,0.041799986433983082889653815072961151599884033203125f,0.3504999649226669955481838769628666341304779052734375f,-0.08090002182722078050147729300078935921192169189453125f,-0.042000023677945197420058320858515799045562744140625f,0.4076999790921809196930780672118999063968658447265625f,0.45279999134391546444788900771527551114559173583984375f,0.58759998356699949528803017528844065964221954345703125f,1.3620999619826676951817034932901151478290557861328125f,0.87719999610185650507077070869854651391506195068359375f,-0.43319999940991404940859865746460855007171630859375f,-0.734499992907047261070374588598497211933135986328125f,-0.4903999946177004520819764366024173796176910400390625f,0.5629999969601631182314349644002504646778106689453125f,1.2952999972857532906544975048745982348918914794921875f,0.839099996799230307686912055942229926586151123046875f,0.16559997859001163167391723618493415415287017822265625f,0.737999964550137832475229515694081783294677734375f,0.17559998499751117151390644721686840057373046875f,0.1756000094354153162612419691868126392364501953125f,-0.301999995142221422383954632095992565155029296875f,-0.188800004300475166019168682396411895751953125f,-0.69219999073743831985439101117663085460662841796875f,-0.876300029520690859641263159574009478092193603515625f,0.2271000013589858923523934208787977695465087890625f,-0.16899999624490735783410855219699442386627197265625f,0.081499999150633772870833126944489777088165283203125f,-0.9417000310301784082867015968076884746551513671875f,-0.46599999257922153361732853227294981479644775390625f,-0.7443000015959142512400603663991205394268035888671875f,-0.294599990928173127713307621888816356658935546875f,0.9037999744236469013003443251363933086395263671875f,-0.14400001729279754414392300532199442386627197265625f,0.418700006949901393227264634333550930023193359375f,0.224799994271993597294567734934389591217041015625f,0.2052999894067644159889596267021261155605316162109375f,-0.49040001734197158356209911289624869823455810546875f,-0.4695999940574167030860053273499943315982818603515625f,-0.51680000349581234342366542477975599467754364013671875f,0.99279999883174863128942888579331338405609130859375f,-0.025800008755922032577245772699825465679168701171875f,0.81339999571293597480092785190208815038204193115234375f,0.1115000221729278706561672152020037174224853515625f,0.50379998232126244062101250165142118930816650390625f,0.0330999969184400111288368862005881965160369873046875f,0.5805000108182429841008342918939888477325439453125f,0.060800010842084528661644071689806878566741943359375f,0.4171000068351624623375073497300036251544952392578125f,-0.04489998271912309935061102805775590240955352783203125f,-0.7633999786138534471291450245189480483531951904296875f,-0.077699956963955985056458075632690452039241790771484375f,-0.435300019358098921884447918273508548736572265625f,-0.63779998594224440733313485907274298369884490966796875f,-0.5921000018432736045070896580000407993793487548828125f,0.47249999761581396029441748396493494510650634765625f,0.189799988758564186497324044466949999332427978515625f,0.52609999887049208044942361084395088255405426025390625f,-0.42949999180808617893490009009838104248046875f,-0.6063999782934785809374034215579740703105926513671875f,-0.57609998618587854313233265202143229544162750244140625f,0.523600012728571595488347156788222491741180419921875f,-0.195199998873472058136258056038059294223785400390625f,0.3084000141531231253821943028015084564685821533203125f,-0.46399999624490728677983497618697583675384521484375f,0.597599986696243146155893555260263383388519287109375f,-0.36080001657903204570487787350430153310298919677734375f,0.5527999961495400071953554288484156131744384765625f,0.4509999901950358935920348812942393124103546142578125f,-0.183599996933340836591241895803250372409820556640625f,0.1784999971613288050065193601767532527446746826171875f,0.27960000646263349910469742098939605057239532470703125f,0.28739999184608466809009996723034419119358062744140625f,0.864499978676438463054410021868534386157989501953125f,0.2314999898374081599428109257132746279239654541015625f,-0.497000022858381296231300439103506505489349365234375f,-0.69740002164840664278244730667211115360260009765625f,-0.051699999886750891420206244220025837421417236328125f,-0.30380001659393318647062187665142118930816650390625f,0.15229998875856409767948207445442676544189453125f,-0.242500007301568842876804410479962825775146484375f,0.3500000201165676116943359375f,0.23040001428723311249768812558613717555999755859375f,-0.163199992033839169636166843702085316181182861328125f,0.2710000027120111898426557672792114317417144775390625f,0.94350001418590512258788294275291264057159423828125f,0.312300000903010133157522432156838476657867431640625f,-0.6815999975442885716603313994710333645343780517578125f,-0.92259998434931045352414002991281449794769287109375f,-0.348399988523125703210325809777714312076568603515625f,0.48900000593066206278081153868697583675384521484375f,0.353500002488493780816725120530463755130767822265625f,0.3563000072807074314340525234001688659191131591796875f,0.40300001499056836706813555792905390262603759765625f,0.28129999707341202253019218915142118930816650390625f,-0.267300014612078751241597274201922118663787841796875f,0.051600006783008467436957289464771747589111328125f,0.507600015008449201303619702230207622051239013671875f,0.3642000194072723129323776447563432157039642333984375f,0.227800001215934688758579795830883085727691650390625f,0.412700004684924959263980781543068587779998779296875f,0.2140999987363814138774387174635194242000579833984375f,-0.1203999897003171781761921010911464691162109375f,-0.75569999851584412908778176642954349517822265625f,-0.6330000146180392306405337876640260219573974609375f,0.4773999897599221498012411757372319698333740234375f,0.57939996582865749275015332386828958988189697265625f,0.236499985143542534871130555984564125537872314453125f,0.3529999968111514618840374168939888477325439453125f,0.433899991816282248890956907416693866252899169921875f,0.5997000070542097827086536199203692376613616943359375f,-0.33079998700022716917601428576745092868804931640625f,-0.73739998432099829184238615198410116136074066162109375f,0.14669997094124564984696235114824958145618438720703125f,-0.6881999646902088318967116720159538090229034423828125f,0.1226999998793001367403121548704802989959716796875f,0.403300004360079622500734330969862639904022216796875f,-0.119999994933605247382502057007513940334320068359375f,0.1798999996319412542078453043359331786632537841796875f,-0.06560000422000911157738300971686840057373046875f,-0.47699999335408183043227836606092751026153564453125f,0.1006000295892361950933491243631578981876373291015625f,1.167500022947788096416843472979962825775146484375f,-0.19720000221133204831858165562152862548828125f,0.95930002543777259660373601946048438549041748046875f,-0.02200001585483590815783827565610408782958984375f,0.6687999748706818792243211646564304828643798828125f,-0.094800045308470970439884695224463939666748046875f,0.42070001848340066175069296150468289852142333984375f,-0.27159999530911438370139876496978104114532470703125f,0.18690000982433563336826409795321524143218994140625f,0.4089999882355332960770510908332653343677520751953125f,-0.3107999878197909016108724244986660778522491455078125f,0.53300001145154229931222289451397955417633056640625f,-0.9273000129729511709086864357232116162776947021484375f,0.18039998448491123550496695315814577043056488037109375f,-0.85310002131909123956887697204365395009517669677734375f,0.1269000074028969748951567453332245349884033203125f,0.86639998913407278990916893235407769680023193359375f,0.0106999677076931032360107565182261168956756591796875f,0.0791999750018117509853254887275397777557373046875f,-0.53879999305009818044709390960633754730224609375f,-0.0036000093013048850565382963395677506923675537109375f,-0.13809999051094035138476101565174758434295654296875f,-0.051999999314546574424866776098497211933135986328125f,-0.19149999635666603570172128456761129200458526611328125f,0.10730001379251508097922851447947323322296142578125f,0.851799992394446814358843766967765986919403076171875f,0.19169998714625802183064706696313805878162384033203125f,-0.37440000025033981501110247336328029632568359375f,0.2473999931871890822776549612171947956085205078125f,-0.43580001095384357512330097961239516735076904296875f,-0.95490001956224457302369046374224126338958740234375f,-0.359999995380639870035111016477458178997039794921875f,-0.86510000476390114254598984189215116202831268310546875f,-0.3366000161707400906152543029747903347015380859375f,-0.2939000076860189825112001926754601299762725830078125f,0.29700001019239419264295065659098327159881591796875f,0.4528000135838983197800189373083412647247314453125f,-0.03410000179111971618794996174983680248260498046875f,-0.245000007078051584841205112752504646778106689453125f,0.0858999979481100961464790088939480483531951904296875f,0.07230000287741422670961810581502504646778106689453125f,0.7454000101149080848017547396011650562286376953125f,0.94850000569224324209471888025291264057159423828125f,0.820300019022822457515076166600920259952545166015625f,0.19699998366832716101271216757595539093017578125f,0.9580999934911726967357026296667754650115966796875f,0.59349999570846545537960992078296840190887451171875f,-0.07580000115633023227701414725743234157562255859375f,0.365200005877017730426814523525536060333251953125f,0.35830002045333397830972899100743234157562255859375f,-0.6827999839305878282402773038484156131744384765625f,-1.01559996339082747596194167272187769412994384765625f,-0.723399984723329669833447042037732899188995361328125f,0.154200003314018285749398273765109479427337646484375f,-0.001100002223253238042843804578296840190887451171875f,0.1921000041902065635923690933850593864917755126953125f,0.41760001992583273278114575077779591083526611328125f,0.08810002061128585637561627663671970367431640625f,0.543400018399953665948487468995153903961181640625f,0.6746000031471250935055650188587605953216552734375f,0.98420000137686702856854026322253048419952392578125f,0.815100012847781218994214214035309851169586181640625f,0.2756000000476837019647291526780463755130767822265625f,0.59999999724328512140658631324185989797115325927734375f,0.191799996119737581334874221283826045691967010498046875f,0.0191999967575073871017821147688664495944976806640625f,-0.03440000419914712725955041605629958212375640869140625f,0.234399995183944553911459252049098722636699676513671875f,0.0337999940931796860610347721376456320285797119140625f,0.8469999951422213513296810560859739780426025390625f,0.28930000073909756252987790503539144992828369140625f,0.226600003355741375088427957962267100811004638671875f,0.345300009411573238793380369315855205059051513671875f,-0.0599000026494265203069744529784657061100006103515625f,-0.47099999585747720498574153680237941443920135498046875f,-1.1367999942198394602854705226491205394268035888671875f,-0.64709998824596370869954853333183564245700836181640625f,0.50860000743865951822186843855888582766056060791015625f,-0.05129999677538865654469191213138401508331298828125f,0.23210001238584521132679583388380706310272216796875f,-0.4314000113368032796046236398979090154170989990234375f,0.1446000091075898552617218228988349437713623046875f,0.19980000675171627477766378433443605899810791015625f,0.2396000056058167348993492851150222122669219970703125f,0.0724000069335104523560175948659889400005340576171875f,-0.779700034844875933259800149244256317615509033203125f,0.52520000706911107446472897208877839148044586181640625f,-0.42530001611709611353262516786344349384307861328125f,0.339099968859553957400976287317462265491485595703125f,-0.76679996147453832744389501385740004479885101318359375f,0.2098999942675232599498258423409424722194671630859375f,-0.6098999826073647767543661757372319698333740234375f,0.521999984040856457312429483863525092601776123046875f,-0.319500004529952974507978069595992565155029296875f,0.7004000195026396369257781771011650562286376953125f,-0.213599984639883277992566945613361895084381103515625f,0.371200000095367244057342759333550930023193359375f,0.19210006215572406063074595294892787933349609375f,-0.2384000240623951327734175720252096652984619140625f,0.133100017929077552025773911736905574798583984375f,0.96389999822378147342760712490417063236236572265625f,-1.0285999793499709387134544158470816910266876220703125f,0.5466000023126600382283868384547531604766845703125f,0.3911999756574624509397608562721870839595794677734375f,-0.8681999940454956377067219364107586443424224853515625f,0.236300004225969306759225219138897955417633056640625f,-0.9322000190496446681009956591879017651081085205078125f,1.2888000136137005569025859585963189601898193359375f,-0.537800024610758153897904776385985314846038818359375f,-0.8004999984502791932072796043939888477325439453125f,0.423599980020523236845519932103343307971954345703125f,-0.4153000044941901336414957768283784389495849609375f,0.0973999972105019651280599646270275115966796875f,-0.469599999421834723989377380348742008209228515625f,0.09499999165534944722821819595992565155029296875f,-1.3543000055074703169566419092006981372833251953125f,1.142399992740154690551435123779810965061187744140625f,-0.73010001888275155579322017729282379150390625f,0.908399976825713384442906317417509853839874267578125f,-1.137599968144297424288424736005254089832305908203125f,0.52239999482631649385666605667211115360260009765625f,-0.350000001788139503133834296022541821002960205078125f,0.1406999941945077381433293339796364307403564453125f,0.515200028377771790388806039118207991123199462890625f,0.1757000092819333836313688834707136265933513641357421875f,0.3655000314563519037136529732379131019115447998046875f,0.925799990800022509773725687409751117229461669921875f,0.349499992311000706735057974583469331264495849609375f,1.282200018453597323286885512061417102813720703125f,0.9330000339895490046870918376953341066837310791015625f,0.33960001026242980781688629576819948852062225341796875f,0.7785000455528503504609716401319019496440887451171875f,0.635200025844574245326157324598170816898345947265625f,0.3581000076845288759841423598118126392364501953125f,0.94630003596544298716253251768648624420166015625f,-0.1128000131368640523987778578884899616241455078125f,-0.204900006635487130779438302852213382720947265625f,-0.243900013124942915965220890939235687255859375f,-0.44019999119937391895973632927052676677703857421875f,-0.2281999959453939741393924123258329927921295166015625f,-0.1878999947667114867044801940210163593292236328125f,-0.3120000154078004239721622070646844804286956787109375f,-0.269699999417364555398535230779089033603668212890625f,-0.59270001600980748435176792554557323455810546875f,0.2448000032126902425488879089243710041046142578125f,-0.098099998147785694158073965809307992458343505859375f,-0.521099983894824703156700707040727138519287109375f,-0.609400004497170133532790714525617659091949462890625f,-0.0622999995619057944651331126806326210498809814453125f,0.0987999784469604236392115126363933086395263671875f,0.509200009423494304172663760255090892314910888671875f,0.3710000031590461322394958187942393124103546142578125f,0.9636000366449355425402245600707828998565673828125f,0.1507999826788903874330571852624416351318359375f,-0.122000001028180093953778850845992565155029296875f,-0.2646000156641008516089641489088535308837890625f,-0.424400010383128911684025297290645539760589599609375f,-0.2646999988213181165264131777803413569927215576171875f,-0.6955000130534170210694355773739516735076904296875f,-1.218600027629733073553097710828296840190887451171875f,-0.373800006274878970913277953513897955417633056640625f,-1.419500065028667989963651052676141262054443359375f,0.459400009563565081549540991545654833316802978515625f,0.2720000002458691223949927007197402417659759521484375f,0.3445000125765798060228917165659368038177490234375f,0.297200002956390374464490378159098327159881591796875f,-0.298200009989738390459024230949580669403076171875f,0.5823000293642286440132238567457534372806549072265625f,-0.5700000178813933171539929389837197959423065185546875f,0.3259000186234715101818437688052654266357421875f,0.5831999757170667209038583678193390369415283203125f,-1.190399975618719796699451762833632528781890869140625f,0.373199990916251778116929926909506320953369140625f,0.96150003357231650369385533849708735942840576171875f,-0.9992000145643948894047525755013339221477508544921875f,0.544500021144747936574503910378552973270416259765625f,0.3003000316143040748784187599085271358489990234375f,-0.348800009441375902241588846663944423198699951171875f,0.18730000753402720192752894945442676544189453125f,0.0632999704599370716096018441021442413330078125f,-0.4987999999046319743456479045562446117401123046875f,0.0477000011086463615583852515555918216705322265625f,-0.552300006043910940434216172434389591217041015625f,0.09499997891485723044979749829508364200592041015625f,-0.2865999951601025941272382624447345733642578125f,-0.01450003254413534392597284750081598758697509765625f,-0.1625999737322334492972686348366551101207733154296875f,-0.0136000038623809160753808100707828998565673828125f,0.854699986565112368452901137061417102813720703125f,-0.696200007098912720948646892793476581573486328125f,0.44880001078248010770721521112136542797088623046875f,-1.1335999910473812946065663709305226802825927734375f,1.027800005313753484159633444505743682384490966796875f,-0.6129000117540357450707233510911464691162109375f,0.0568000072956085233499834430404007434844970703125f,-0.025599996322393625547420015209354460239410400390625f,0.05119998877048459462457685731351375579833984375f,0.513199967968464676459916518069803714752197265625f,-0.7173999578714376212928982567973434925079345703125f,0.28329999892115598214559213374741375446319580078125f,-0.1441000213861460821362925344146788120269775390625f,0.156600011551379925123228531447239220142364501953125f,-0.094800002765655477787731797434389591217041015625f,-0.9978999971508972777201051940210163593292236328125f,1.799400009861587879100852660485543310642242431640625f,-0.611200013208389236751827411353588104248046875f,0.4551000263333317974456804222427308559417724609375f,-0.029200007113814319126277041505090892314910888671875f,0.23830000383853899847963475622236728668212890625f,-0.1866999916911122880947004887275397777557373046875f,-0.6916000052928925612150123924948275089263916015625f,0.7784000186979771473971823070314712822437286376953125f,0.1168000047624111470412344715441577136516571044921875f,0.3160000278949741669975992408581078052520751953125f,-1.149399995407461627650036462000571191310882568359375f,-0.373999992184340879219917042064480483531951904296875f,-1.271500006377696223580642254091799259185791015625f,-1.14420003989636942520746742957271635532379150390625f,-0.20880001499205835813199882977642118930816650390625f,-0.30650006419420350312066148035228252410888671875f,0.488000032126903793283645427436567842960357666015625f,0.2845000091865659275214284207322634756565093994140625f,0.2207000293612484398408923880197107791900634765625f,0.9060000229775908575646781173418276011943817138671875f,0.30210000787824398127412450776319019496440887451171875f,0.80810003640651739686973087373189628124237060546875f,0.6097999861508605068394217596505768597126007080078125f,0.1926999944031237976815873480518348515033721923828125f,0.6177999982357018637912915437482297420501708984375f,0.377400014719367238313907364499755203723907470703125f,0.1653000042706728456298748142216936685144901275634765625f,0.6930000198334458350046816121903248131275177001953125f,-0.825400014212727572981975754373706877231597900390625f,-0.319400002075731770379007912197266705334186553955078125f,-0.926100020700693082886800766573287546634674072265625f,-0.7966000262290242606155743487761355936527252197265625f,-0.184700009587407176780970985419116914272308349609375f,-0.31160003747940123730586492456495761871337890625f,0.3768000114679332934741751159890554845333099365234375f,0.2294999974891542837696079004672355949878692626953125f,0.1441000026106831199967928114347159862518310546875f,0.89840002242326766346280919606215320527553558349609375f,0.3058000074893236774897786744986660778522491455078125f,0.820200033295154895540690631605684757232666015625f,0.137399982383846985811715057934634387493133544921875f,-0.06540000650137667292938203900121152400970458984375f,0.4052999937653538609083625487983226776123046875f,0.2638000370830300145286173574277199804782867431640625f,0.2087000088125468000299633786198683083057403564453125f,0.2592000259637838865955927758477628231048583984375f,0.3806000002712013330352647244581021368503570556640625f,0.0163000028103590377526188603951595723628997802734375f,0.1260000082254411069015986868180334568023681640625f,-0.5105999876052138564119786678929813206195831298828125f,-0.0873000114083293254907403024844825267791748046875f,1.11460000553131077793977965484373271465301513671875f,-1.3738000069081779397350828730850480496883392333984375f,0.6273000218391420190755525254644453525543212890625f,-0.9800999917626374013934764661826193332672119140625f,1.573199973705410759095002504182048141956329345703125f,-0.5893000043153762135261786170303821563720703125f,-1.735300017607212197390254004858434200286865234375f,1.78420001709759201702354403096251189708709716796875f,-0.95740003297328968301371787674725055694580078125f,0.5895000540018084933535646996460855007171630859375f,0.021999972045421589683655838598497211933135986328125f,0.30110001816749587533195153810083866119384765625f,0.90850001686811499013174397987313568592071533203125f,-1.1131999964296819261022619684808887541294097900390625f,0.5233000148653985661439946852624416351318359375f,0.56319997988939274335962181794457137584686279296875f,-0.8307999653190376587730270330212078988552093505859375f,0.3329999936819074690674824523739516735076904296875f,0.1056000107020143019553870544768869876861572265625f,-0.472800003078579944570947191095910966396331787109375f,0.088800005939603021687389627913944423198699951171875f,-0.6721000164091588402470733853988349437713623046875f,0.920999994739890137651627810555510222911834716796875f,-0.39860000997185718318860381259582936763763427734375f,-1.12040001801252397939379079616628587245941162109375f,1.1242000071138142924809244505013339221477508544921875f,-0.6195000218153001725340800476260483264923095703125f,0.46970003812312999258438139804638922214508056640625f,0.0715999766826633798899592875386588275432586669921875f,0.2369000106811522954330939683131873607635498046875f,0.884600018644333463413431672961451113224029541015625f,-1.086799995374679639326132019050419330596923828125f,0.51040001503229159851571239414624869823455810546875f,-0.0159000420928006036547230905853211879730224609375f,-0.779399976333975619269267554045654833316802978515625f,0.0341999916911124302032476407475769519805908203125f,0.19420005964040765178424408077262341976165771484375f,0.1751999548405411832874278843519277870655059814453125f,0.0962000215530398605778827914036810398101806640625f,0.58339998606443455031467237859033048152923583984375f,-0.8303999796420338253000181794050149619579315185546875f,0.3322000053405762542979573481716215610504150390625f,-0.76089998144507386967916318099014461040496826171875f,-0.1530000050067901185002483543939888477325439453125f,0.37179999813139430475672497777850367128849029541015625f,0.17479999945014712547930457731126807630062103271484375f,0.26150000518560378992560799815692007541656494140625f,-0.48060001129806051434201208394370041787624359130859375f,-0.22470000362694264595120330341160297393798828125f,-0.5572000217318535675303792231716215610504150390625f,-0.0586000009939073862597069819457828998565673828125f,-0.1468999997101723964387787191299139522016048431396484375f,-0.171200000430643317628209842951036989688873291015625f,-0.63360004290342342159192412509582936763763427734375f,-0.299700007016956870842250282294116914272308349609375f,-0.203100018227100775902727036736905574798583984375f,-0.749999995231628080460950513952411711215972900390625f,-0.215099996866285725349854374144342727959156036376953125f,-0.338300011736154093711093082674778997898101806640625f,0.58399999549984915603317858767695724964141845703125f,0.103299997915327512743033366859890520572662353515625f,0.787900019800662931857004878111183643341064453125f,0.48640001044273351027413809788413345813751220703125f,0.181100000099837732392416000948287546634674072265625f,-0.139099990093708214544676593504846096038818359375f,-0.22139998905956759234214814568986184895038604736328125f,0.04979999967366455226880361806252039968967437744140625f,-0.11290000132322308701304791611619293689727783203125f,0.897600014263391532409741557785309851169586181640625f,0.35460000232756139215695156963192857801914215087890625f,1.20980002809762954285588421043939888477325439453125f,-0.8053999939471481184227741323411464691162109375f,-0.2175999953016638510661096006515435874462127685546875f,-1.0099000040650363274608025676570832729339599609375f,-0.21699999894201749128797018784098327159881591796875f,-0.000499997183680545020934005151502788066864013671875f,-0.620800000262260454064744408242404460906982421875f,0.2572000183045861643194029966252855956554412841796875f,0.200299999465048284008616974460892379283905029296875f,0.4319000127673149336260394193232059478759765625f,0.741400023332238422568707392201758921146392822265625f,0.435800006707012688167424130369909107685089111328125f,1.214100037777424034857176593504846096038818359375f,-0.9756000329792502423487121632206253707408905029296875f,-0.39450000773370275108931082286289893090724945068359375f,-1.139300035198033356209634803235530853271484375f,0.8708000318527224781206541592837311327457427978515625f,-0.11829998700022681390464640571735799312591552734375f,0.400700002837181212811401564977131783962249755859375f,-0.20699998567998434406689511888544075191020965576171875f,0.21659999754428849172427362645976245403289794921875f,-0.361400011336803661521344110951758921146392822265625f,0.38119999778568758319607923112926073372364044189453125f,-0.21080000294446943343018574523739516735076904296875f,0.082599984759092193176144292010576464235782623291015625f,-0.408400000816583397256209764236700721085071563720703125f,0.05520000487118970067257350820000283420085906982421875f,-0.855900081729888739801026531495153903961181640625f,0.22520005117654751103373200749047100543975830078125f,-0.44550003659725234683719463646411895751953125f,-1.0044999906718725668497427250258624553680419921875f,0.906199981021881040277321517351083457469940185546875f,-0.55240000153183910214238494518212974071502685546875f,0.43449996608495666805538348853588104248046875f,-1.23659998040795304774519536294974386692047119140625f,0.28919999720454203639974366524256765842437744140625f,0.890100027227401113805171917192637920379638671875f,-0.23440001425743073326657395227812230587005615234375f,0.4555000127553938682467560283839702606201171875f,-0.337099967160821467615505753201432526111602783203125f,0.75359996720552480109489579263026826083660125732421875f,-0.2044999961480498118504556259722448885440826416015625f,0.5824000088334084779262411757372319698333740234375f,-1.070999991387128869035905154305510222911834716796875f,0.3661000065445900730765060870908200740814208984375f,-0.621099981510639054249622859060764312744140625f,1.3493999703735113371294573880732059478759765625f,-0.393000001430511503031084430404007434844970703125f,0.057000016152858989926244248636066913604736328125f,0.78199999491870375578628227231092751026153564453125f,-0.0184000027537347676798162865452468395233154296875f,0.06230003554820928712842942331917583942413330078125f,0.0687999651849275917214754372253082692623138427734375f,0.0381000101804733759536247816868126392364501953125f,0.2641000288367276738199507235549390316009521484375f,-0.385800001975894257810750787029974162578582763671875f,0.17660000290870669914511381648480892181396484375f,-0.75490003956705375998126328340731561183929443359375f,1.0388000116765498948012691471376456320285797119140625f,-0.4501000213786963133344443122041411697864532470703125f,0.92220002828836467045903191319666802883148193359375f,0.4014000247478488603292134939692914485931396484375f,-0.59019998210966573282121316879056394100189208984375f,-0.1780999940872192155438824556767940521240234375f,-0.8085999895572655304931686259806156158447265625f,-0.18180000208020208329884326303726993501186370849609375f,-0.1008000000387430095116769734886474907398223876953125f,-0.7605000178217888873177798814140260219573974609375f,-0.58339998598992781619898551070946268737316131591796875f,-0.2331999932818113741017640450081671588122844696044921875f,-0.6569000027649096740134382343967445194721221923828125f,-0.51520003791451518537769516115076839923858642578125f,-0.2807000141248108260327853713533841073513031005859375f,0.0198999760508531409186616656370460987091064453125f,-0.0300000041723251786862647350062616169452667236328125f,0.0113999988943338237401547985427896492183208465576171875f,0.2348000031381844987521390066831372678279876708984375f,-0.39359999849796267756829593054135330021381378173828125f,-0.2441999969810246806645182005013339221477508544921875f,-0.520600006902217415927225374616682529449462890625f,-0.2496000003159044577927261343575082719326019287109375f,-0.14399999833106991165010413169511593878269195556640625f,-0.44600000792741756328041446977294981479644775390625f,-0.5047999971032144372884431504644453525543212890625f,-0.06310000313967467633347041555680334568023681640625f,-0.49050002378225343591111595742404460906982421875f,0.9618000103503465680887529742904007434844970703125f,0.382200003813207123126716169281280599534511566162109375f,1.281500047653913565426364584709517657756805419921875f,-0.997199988353252297912376889144070446491241455078125f,-0.18929999831765886852963376441039144992828369140625f,-0.2069000044226640966371633112430572509765625f,0.3923999995946882979325209817034192383289337158203125f,0.1682999976918101159295559909878647886216640472412109375f,0.6135000026375052595284387280116789042949676513671875f,-0.22839998651146908859033146654837764799594879150390625f,0.0567999991744756460576581957866437733173370361328125f,-0.127000001847743959615399944595992565155029296875f,-0.5550000270456079132230797767988406121730804443359375f,-0.228600008295476497810483351713628508150577545166015625f,-0.415300029528141489976178490906022489070892333984375f,-0.7414000096231700975835110511980019509792327880859375f,-0.13250000491738322949686335050500929355621337890625f,-0.498600027108192733749092440120875835418701171875f,-0.815000034943223372607690180302597582340240478515625f,-0.447100006052851828286520685651339590549468994140625f,-0.397799974095821795572192058898508548736572265625f,0.94379995155036500165124380146153271198272705078125f,-0.258699991750717117611202411353588104248046875f,0.26430001411288994717097011744044721126556396484375f,0.34899999732524145112932956180884502828121185302734375f,0.10140000695958739651558744299109093844890594482421875f,-0.470099981536715727514774698647670447826385498046875f,0.62059997210800654787732355543994344770908355712890625f,-0.27799998982623186360996214716578833758831024169921875f,-0.812300069522859047310703317634761333465576171875f,-0.06499998047947830315251849242486059665679931640625f,-0.406200025725365065909500117413699626922607421875f,-0.22700001160800464816702515236102044582366943359375f,-0.0017999947786331293997363900416530668735504150390625f,-0.109100007155537692682401029742322862148284912109375f,-0.223400006628036162936723485472612082958221435546875f,0.08919999347925189336905305026448331773281097412109375f,-0.12799999457597710517120503936894237995147705078125f,-0.06340000036954851481141304248012602329254150390625f,0.1597999978482722571726526439306326210498809814453125f,-0.0486999965190886285881788353435695171356201171875f,-0.523699965822697066641921992413699626922607421875f,1.08559997024536158960472675971686840057373046875f,-0.32109999656081189556289245956577360630035400390625f,0.633699988025426819149288348853588104248046875f,-1.1307999832004309581634515780024230480194091796875f,0.39609999596476530570043905754573643207550048828125f,-1.5522999777317050273950371774844825267791748046875f,1.494199950563908085854336604825221002101898193359375f,-0.850600004816055133005647803656756877899169921875f,0.18810000042021268740199957392178475856781005859375f,-0.4537999897718429931359196416451595723628997802734375f,0.127799999651312834458849465590901672840118408203125f,-0.342599960768223521512254592380486428737640380859375f,0.80559995936751416234500311475130729377269744873046875f,-0.209799993675947149540661484934389591217041015625f,-0.5945000335872177021911966221523471176624298095703125f,0.458200016173720303669369968702085316181182861328125f,-0.328400016760826307660181555547751486301422119140625f,-0.91059998902082472937991042272187769412994384765625f,1.3161999845236540540582836911198683083057403564453125f,-0.5261000104188919834768967120908200740814208984375f,-0.5200000473111874299547707778401672840118408203125f,-0.282099987947940888943776371888816356658935546875f,-0.2379999863803388127081461789202876389026641845703125f,-0.03409999877363445452971291160793043673038482666015625f,-0.58489999840259532248865070869214832782745361328125f,0.14939999555647365259147818505880422890186309814453125f,-0.086100001701712614021744229830801486968994140625f,-0.1540000039935112141620265902020037174224853515625f,1.05239999229311909090256449417211115360260009765625f,0.1868999985739587810229522801819257438182830810546875f,0.0981000241875644718447802006267011165618896484375f,-0.395800000634789483910935814492404460906982421875f,-0.0905000005364418047548724644002504646778106689453125f,-0.4046000108957290564148934208787977695465087890625f,0.63240000346899005734258025768212974071502685546875f,0.28649999725073571621436485656886361539363861083984375f,0.33209999957084601618362285080365836620330810546875f,0.680199977192282201343687120242975652217864990234375f,0.239299989339709195945715691777877509593963623046875f,0.941199986386298093066216097213327884674072265625f,-0.587600030207634294043828049325384199619293212890625f,-0.33280000933706765930963911159778945147991180419921875f,-1.04380004609823284766889628372155129909515380859375f,-0.918399988448619364334035708452574908733367919921875f,-0.34939999391734588574109920955379493534564971923828125f,-1.27300002437829906654087608330883085727691650390625f,0.008999998405575748705587102449499070644378662109375f,0.008999999336898323321065618074499070644378662109375f,0.563100010478496670884851482696831226348876953125f,0.3632000201225282776107405879884026944637298583984375f,0.09730000623017553973426174707128666341304779052734375f,0.15430002279281662680432418710552155971527099609375f,-0.82979998108446562099516086163930594921112060546875f,-0.1673999943420290126283589415834285318851470947265625f,-0.0014999938607207496943374280817806720733642578125f,-0.6388000300049785362688226086902432143688201904296875f,-0.188000011526048227405993884531199000775814056396484375f,-0.14620004386752916136771318633691407740116119384765625f,0.417000013992190332601239788345992565155029296875f,0.1719000044226646206624309343169443309307098388671875f,-0.1125999765634535521030556992627680301666259765625f,-0.9856000118196011072768669691868126392364501953125f,-0.1739000034019351215164306267979554831981658935546875f,-0.1517000173211098257297635427676141262054443359375f,-0.186599984356761172676897331257350742816925048828125f,0.6178000015139577527634173748083412647247314453125f,0.0085000336319200897605696809478104114532470703125f,0.6681999594002967501893408552859909832477569580078125f,-0.0371999894335865999295265282853506505489349365234375f,0.415599985295533880247376146144233644008636474609375f,-0.61699999327957610883998995632282458245754241943359375f,0.232300006118416835221296423696912825107574462890625f,1.730099971199035024937984417192637920379638671875f,-1.57199996772408479017713034409098327159881591796875f,0.94110000445842700855791918002068996429443359375f,-0.3781999894261360850578057579696178436279296875f,0.687399987003207257885151193477213382720947265625f,-0.228900001502037042655501863919198513031005859375f,0.767100024902820365468869567848742008209228515625f,-0.34219999490976338307746118516661226749420166015625f,0.4086000071406363787218651850707828998565673828125f,0.4367999730229374932832797640003263950347900390625f,-0.948199954631924857295643960242159664630889892578125f,0.281499988794326583274596487171947956085205078125f,-0.1564000274300578752217916189692914485931396484375f,0.399400017237663274016767900320701301097869873046875f,-0.1191000073790551283536842674948275089263916015625f,-0.5765999668478958284367763553746044635772705078125f,1.169999966919422096367497942992486059665679931640625f,-0.3687999858975405942373981815762817859649658203125f,-0.3915000194907189978721362422220408916473388671875f,-0.259999996647238607039298585732467472553253173828125f,-0.1737000075459480541439916123636066913604736328125f,0.49630001927614220136320000165142118930816650390625f,-0.4632000159502027702984605639358051121234893798828125f,0.2733000148653985661439946852624416351318359375f,-1.409699978220462668332402245141565799713134765625f,1.11719995400607619018273908295668661594390869140625f,-0.75810000004768340886585065163671970367431640625f,-0.95520002565830974816663001547567546367645263671875f,0.6814000172972678459615281099104322493076324462890625f,-0.5137000192061067505022720069973729550838470458984375f,0.751000021636485826093121431767940521240234375f,-0.133800012198090456649879342876374721527099609375f,0.3817000086784361201353021897375583648681640625f,-1.57790000758171089501047390513122081756591796875f,1.5065999881565570461816605529747903347015380859375f,-0.862800022673607003298457129858434200286865234375f,-0.49439996813833797517645507468841969966888427734375f,-0.1826000105381009319671647972427308559417724609375f,0.65880001212358507700628251768648624420166015625f,0.097800008666515392263818284845910966396331787109375f,0.068800034475327009886314044706523418426513671875f,-0.27139999189078789498807964264415204524993896484375f,0.08630000430047513049203189439140260219573974609375f,0.46880002195835146494573564268648624420166015625f,0.4793999911233779442909508361481130123138427734375f,0.1778999953441321169034239346729009412229061126708984375f,0.381800001074373174247966744587756693363189697265625f,0.353400004169345027804638448287732899188995361328125f,0.1257000032842159331636366914608515799045562744140625f,0.6686000271081926626948188641108572483062744140625f,-0.599799997624754777092448421171866357326507568359375f,-0.334099996650218944438393009477294981479644775390625f,-0.996999999761580824042539461515843868255615234375f,-0.2103999950647352790156219271011650562286376953125f,-0.03259999917596574647848939321193029172718524932861328125f,0.16259999597221630462939856442972086369991302490234375f,-0.2711999966681002405266553978435695171356201171875f,-0.018899998112022864393821919293259270489215850830078125f,0.374500012725591968631988493143580853939056396484375f,0.90580000137984750807618183898739516735076904296875f,0.1144999999850988547933638983522541821002960205078125f,-0.0044000011444094155876882723532617092132568359375f,-0.8136000157833098711535058100707828998565673828125f,-0.32760000249147414042028003677842207252979278564453125f,-0.66240002492666238964602598571218550205230712890625f,-0.14260000837743280754921215702779591083526611328125f,-0.139100000226497655386737051230738870799541473388671875f,-0.567000008255243326260597314103506505489349365234375f,-0.68720001614391812427840022792224772274494171142578125f,-0.29200000382214785776824328422662802040576934814453125f,-0.609600023487210318506868134136311709880828857421875f,-0.38600005158782035863396231434307992458343505859375f,-0.3670000119805336691314323616097681224346160888671875f,-1.0252000550508508780467309406958520412445068359375f,-0.8615999832391734170045083374134264886379241943359375f,-0.222599993178248301006760812015272676944732666015625f,-0.48820000462531965013113222084939479827880859375f,0.0196000056803224964596665813587605953216552734375f,0.10699999797344206253768561509787105023860931396484375f,0.26439999667406066663488672929815948009490966796875f,0.4512000158905984381618736733798868954181671142578125f,-0.151200001287460139565155259333550930023193359375f,1.0861999917507176860453910194337368011474609375f,-1.068999986112117994707659818232059478759765625f,0.594500010490417452047040569595992565155029296875f,-0.859599996292590873281369567848742008209228515625f,0.73059998790323721351569474791176617145538330078125f,-0.453900014019012587596080265939235687255859375f,0.50559999125599830449573346413671970367431640625f,-0.489999980777502119888566767258453182876110076904296875f,0.28419999567717291188984063410316593945026397705078125f,0.0975999831199647616131187533028423786163330078125f,-0.575599988797307116072943244944326579570770263671875f,0.08859999477267255230117370956577360630035400390625f,-0.2112000087380412338688984164036810398101806640625f,0.397599976637959873215777406585402786731719970703125f,-0.1455999952793121110516949556767940521240234375f,-0.4785999917551873039656129549257457256317138671875f,0.2443999904155731428545550443232059478759765625f,-0.2479999990835783929554736459976993501186370849609375f,-0.74530000821948050315768341533839702606201171875f,0.3313999912202358899548926274292171001434326171875f,-0.38140000969767573479884958942420780658721923828125f,1.573200000751018734490571659989655017852783203125f,-1.50539997517168533391895834938623011112213134765625f,0.856900020813942120412320946343243122100830078125f,-0.83640002206563934805672033689916133880615234375f,0.7300000023841857199613514239899814128875732421875f,-0.4674000106215476790794127737171947956085205078125f,0.21259998714327821289771236479282379150390625f,0.00500000931322563957337479223497211933135986328125f,0.0942999995470046314949286170303821563720703125f,-0.66380002210736288592585196965956129133701324462890625f,0.57660000141859057976034819148480892181396484375f,-0.3727000093787908863163238493143580853939056396484375f,0.205999933719635208717591012828052043914794921875f,-0.19799993631243761882387843797914683818817138671875f,0.08939998207092259008277324028313159942626953125f,-1.09339997205734196228377186344005167484283447265625f,1.1939999613761902441666507002082653343677520751953125f,-0.6127999952554699092388545977883040904998779296875f,-0.201599978768825849329004995524883270263671875f,0.2791999849855901771888966322876513004302978515625f,-0.107599998021125742297954275272786617279052734375f,0.33880001480579391426317670266143977642059326171875f,0.5474000305891042472694607567973434925079345703125f,-1.026200016263127512416986064636148512363433837890625f,-0.2344000052794814603629447447019629180431365966796875f,-0.2683000178456307338592523592524230480194091796875f,0.26879999252855790725647011640830896794795989990234375f,0.055800000970065599670277833865839056670665740966796875f,0.607000014811754251553566064103506505489349365234375f,0.705999993622302479678864983725361526012420654296875f,0.308599994102120300798475227566086687147617340087890625f,0.657200006830691574322145243058912456035614013671875f,-0.004600016632676062755535895121283829212188720703125f,-0.0299000033944845089450836894684471189975738525390625f,0.5319999902248380152514073415659368038177490234375f,-0.57680000446438750572042408748529851436614990234375f,-0.233499996714293889255031899665482342243194580078125f,-0.103700002181529526978920330293476581573486328125f,-0.954400006061792538503141258843243122100830078125f,-0.3368000047624111203958818805404007434844970703125f,-1.3120000450611115638821502216160297393798828125f,-0.01139999546706693678288502269424498081207275390625f,0.20009999638199804650184887577779591083526611328125f,0.64179999179840052647705306299030780792236328125f,0.50180000468790508794114657575846649706363677978515625f,0.0141999983593821266225631916313432157039642333984375f,0.164300010871887280927694519050419330596923828125f,-0.319600000017881302749600536117213778197765350341796875f,-0.1042999996028840248296631898483610711991786956787109375f,-0.165900006609409900182328101436723954975605010986328125f,-0.52819999829232688881575086270458996295928955078125f,-0.00400000179558990254236050532199442386627197265625f,0.29589998606443401740762055851519107818603515625f,-0.907800038543344012254010522156022489070892333984375f,-0.25830001210868369643236519550555385649204254150390625f,-0.31120003496408532583927808445878326892852783203125f,-0.136999996557831682508776793838478624820709228515625f,0.013100001761317259241224064680864103138446807861328125f,0.279200011509657120001293151290155947208404541015625f,0.44860001451671127625786539283581078052520751953125f,-0.0015999975815415012903031310997903347015380859375f,-0.16969999490976306333323009312152862548828125f,0.2510000004023311692691322605242021381855010986328125f,-0.03200000240653755323450013747788034379482269287109375f,0.22550000560283667283556496840901672840118408203125f,0.750000031739473715930444086552597582340240478515625f,-0.09600000524520879707779386080801486968994140625f,-1.5403000187993054481694343849085271358489990234375f,1.397399987301230606107083076494745910167694091796875f,-0.84180002398490927362217917107045650482177734375f,0.02119996630698484096910760854370892047882080078125f,-0.65499997045844826626392887192196212708950042724609375f,0.05209998919591296573372574130189605057239532470703125f,0.6559999988377087642987817162065766751766204833984375f,-0.5711999835550785054039124588598497211933135986328125f,0.369599995696544336709621347836218774318695068359375f,-0.3764000345826143956173837068490684032440185546875f,-0.379399966052174963948573349625803530216217041015625f,-0.161700013446807844275099341757595539093017578125f,-0.854700023818015353072041762061417102813720703125f,0.33140000552535031630441153538413345813751220703125f,-0.447200009512901175412480370141565799713134765625f,-0.6185999758481983690217020921409130096435546875f,1.321799968403578162678968510590493679046630859375f,-0.3972999938249586193705908954143524169921875f,-0.56859997167587295052726403810083866119384765625f,0.55499998755753043866434381925500929355621337890625f,-0.29590000379681580255919470801018178462982177734375f,0.773699990856646646619765306240878999233245849609375f,-1.12480000537335822485829339711926877498626708984375f,0.44150001382827763762861650320701301097869873046875f,-0.4068999992631374684748379877419210970401763916015625f,0.343399992918968222088693664773018099367618560791015625f,-0.22340000113323321695446566081955097615718841552734375f,-1.155299982440471495692690950818359851837158203125f,0.99859998367130753393894337932579219341278076171875f,-0.6182000084996222000199850299395620822906494140625f,-1.2872000476598746132594897062517702579498291015625f,1.057000018760561932396058182348497211933135986328125f,-0.7007000335335735741182361380197107791900634765625f,-0.4510000039041042185772312222979962825775146484375f,0.218799993869662312562240913393907248973846435546875f,-0.23040000653862957591400117962621152400970458984375f,0.924900015985965495701748295687139034271240234375f,-0.90960001305639703872429890907369554042816162109375f,0.501100024724006942733467440120875835418701171875f,0.3014999851435415934020056738518178462982177734375f,-0.7990000033974642423828527171281166374683380126953125f,0.1910000066235661986269178669317625463008880615234375f,1.2410000433921819507077088928781449794769287109375f,-0.351900032436848420047681429423391819000244140625f,-0.011599996203184137488761962231365032494068145751953125f,0.10880000020265578886125013013952411711215972900390625f,0.1698000002324580037793566589243710041046142578125f,0.121799998876452431062489267787896096706390380859375f,-0.0465000029504298861837696676957421004772186279296875f,0.3573999888658523360618346487171947956085205078125f,-0.9245999853402373513944212390924803912639617919921875f,-0.194099995160102789526490596472285687923431396484375f,-0.422999999642371449226629920303821563720703125f,-0.133999995648860892316633908194489777088165283203125f,-0.0123999993711709777244323049671947956085205078125f,-0.61420001761913312066099024377763271331787109375f,0.8589999906569716348059273514081723988056182861328125f,0.2746999951332806944037656649015843868255615234375f,0.803200005817412687747491872869431972503662109375f,-0.224400013512373153190537777845747768878936767578125f,-0.0171000044509768966971563486367813311517238616943359375f,0.3252999966710803381175765025545842945575714111328125f,0.701600046271086075222456202027387917041778564453125f,0.36940001284182077956330658707884140312671661376953125f,0.97380006219148729229573291377164423465728759765625f,0.87759999876618355330037957173772156238555908203125f,0.3673999958693980882884488892159424722194671630859375f,1.2134000147491688093026596106938086450099945068359375f,0.721000012323260275337588609545491635799407958984375f,0.3455000021010636856999553856439888477325439453125f,1.2258000344604254028268996989936567842960357666015625f,-0.2241999950438737787550280700088478624820709228515625f,0.11260000007003545530182009315467439591884613037109375f,0.49050000441074370627347889239899814128875732421875f,0.9125999982446433289595688620465807616710662841796875f,0.18240000171810388795989865684532560408115386962890625f,0.5251000190317629545688760117627680301666259765625f,0.595200008410215364307305208058096468448638916015625f,0.20220000288188455517257580140721984207630157470703125f,0.76860003515481967184541645110584795475006103515625f,-0.3734000055104493842605961617664434015750885009765625f,-0.07480000164806842322118285437682061456143856048583984375f,0.21209999431818726822740472925943322479724884033203125f,-0.411800037768483395694829596322961151599884033203125f,-0.2035000077039004207080097330617718398571014404296875f,0.1675999914646144617336176452226936817169189453125f,-0.8596000347375873928967848769389092922210693359375f,-0.1839000235557559648214009939692914485931396484375f,-0.1903999831438063505828495181049220263957977294921875f,0.402199994611739997107946464893757365643978118896484375f,-0.109300002378225347143825274542905390262603759765625f,-0.0178000071763992906426210538484156131744384765625f,-0.641399996212124623440331561141647398471832275390625f,0.02630000720620184750941916718147695064544677734375f,-1.26539997509717938584117291611619293689727783203125f,1.4209999542087319301941761295893229544162750244140625f,-0.7077000014066694433267912245355546474456787109375f,0.2060000323653223830433489638380706310272216796875f,0.545799989533424234622316362219862639904022216796875f,0.06690000963807118949944197083823382854461669921875f,0.83999998420476895688580043497495353221893310546875f,-1.1233999657243491920866063082939945161342620849609375f,0.4890999975442884561971368384547531604766845703125f,-0.6271000237852337733102103811688721179962158203125f,0.260400010114908209146733497618697583675384521484375f,-0.319900017401576253206485489499755203723907470703125f,0.3944000467419623845444220933131873607635498046875f,-0.460600043112039170267735244124196469783782958984375f,0.2401000185847284029705406283028423786163330078125f,0.538399995377659479345311410725116729736328125f,-0.97459997856020930129261614638380706310272216796875f,0.339899996837973450425351984449662268161773681640625f,0.2590000027269123705764286569319665431976318359375f,-0.728999996319413146039778439444489777088165283203125f,0.188899998298287297160413800156675279140472412109375f,-0.802299985554814298893688828684389591217041015625f,0.7273999873757361900317164327134378254413604736328125f,-0.4244000098988414126921497881994582712650299072265625f,1.1758999783158305962871281735715456306934356689453125f,-1.494599963435530920463634174666367471218109130859375f,0.668000005155801712675156522891484200954437255859375f,0.4247999881625172946542079444043338298797607421875f,-0.832999995023012029804476696881465613842010498046875f,0.2666999991416929560728021897375583648681640625f,-0.78710001123100514774932889849878847599029541015625f,0.3997999994128942358173617321881465613842010498046875f,-0.4090000106617808217635001710732467472553253173828125f,-0.7672000783562662462600201251916587352752685546875f,-0.038599955955147979835828664363361895084381103515625f,-0.381300034475327009886314044706523418426513671875f,-0.58690001366138488947399309836328029632568359375f,0.2883999801784751237931914147338829934597015380859375f,-0.2158000153869392701011520330212078988552093505859375f,-0.214700006160140066668873259914107620716094970703125f,-0.05480001468658457497440394945442676544189453125f,-0.2942000232070685061813719585188664495944976806640625f,-0.1954000047877431089915489792474545538425445556640625f,-0.4017000244736674829937328468076884746551513671875f,-0.794399982368946044886115487315692007541656494140625f,-0.2135999953687190977547061265795491635799407958984375f,-1.0427999952554696250217602937482297420501708984375f,1.122000021517276735494306194595992565155029296875f,0.3399000020161271340413122743484564125537872314453125f,1.1147000265002251495616292231716215610504150390625f,0.172599995264410832618295899010263383388519287109375f,-0.0130000002756715249407903911560424603521823883056640625f,-0.4650999970525504512153247560490854084491729736328125f,1.001599998289346871160887531004846096038818359375f,0.3004000038564205166125020696199499070644378662109375f,1.2976000421285629471412903512828052043914794921875f,0.2200000070035457166994774524937383830547332763671875f,0.0368000002920627622415850055404007434844970703125f,0.260000008940696858417140902020037174224853515625f,-0.0960000141859056466131505658267997205257415771484375f,-0.1584000051379204077051099375239573419094085693359375f,-0.0976000129222871493084312533028423786163330078125f,0.314999994896352164541752927107154391705989837646484375f,0.0098999982349574355833254912795382551848888397216796875f,0.127200003142654782806175717269070446491241455078125f,0.535000009909271767583049950189888477325439453125f,0.3805999975517391220591889577917754650115966796875f,1.0253000283360478306349250487983226776123046875f,-0.4709999973848457699432401568628847599029541015625f,-0.33149999469891178183189595074509270489215850830078125f,-0.512800002072750782389221058110706508159637451171875f,-0.6382000235870481930078312871046364307403564453125f,-0.25700000577792525557896397003787569701671600341796875f,-0.513700028072297865122664006776176393032073974609375f,1.0142000084549185334736876029637642204761505126953125f,0.24699999883025880453857325846911408007144927978515625f,0.71330002395510661017397069372236728668212890625f,-0.73760001202821723342850646076840348541736602783203125f,-0.3205000014677643616067825860227458178997039794921875f,-0.670300020289420928065737825818359851837158203125f,-0.74780001514852045829684357158839702606201171875f,-0.240000003650784332620560235227458178997039794921875f,-0.25220003544092239877727479324676096439361572265625f,-0.4071999927490945747621253758552484214305877685546875f,-0.1110000040531158305157077847979962825775146484375f,-0.15230003286600091172431348240934312343597412109375f,0.0184000281602141324555077517288736999034881591796875f,-0.086500010550022210509268916212022304534912109375f,-0.5795999661922461854146604309789836406707763671875f,1.358399940168858233136006674612872302532196044921875f,-0.3731999924063682527730634319595992565155029296875f,1.058500018954276811200543306767940521240234375f,-1.579800002467631969693684368394315242767333984375f,0.62560002150535609644066425971686840057373046875f,0.6478999933511015996145943063311278820037841796875f,-0.204799993303418137458038472686894237995147705078125f,0.324900001904368185279281533439643681049346923828125f,0.729399959421158428085618652403354644775390625f,-1.572599952572584669496791320852935314178466796875f,0.463499990344047319013043306767940521240234375f,0.19100000166892971975585169275291264057159423828125f,-0.4566000084221359944791629459359683096408843994140625f,0.121500006973743523985831416212022304534912109375f,-0.03200002956390424202481881366111338138580322265625f,-0.4111999887704846923242030243272893130779266357421875f,-0.000800002944469468957322533242404460906982421875f,0.467999982506036527052373230617376975715160369873046875f,-0.713799986642599026964717268128879368305206298828125f,0.26910000197738404015268542934791184961795806884765625f,0.0545000161528577109493198804557323455810546875f,-0.072200015398859562765210284851491451263427734375f,0.053399994111060777868260629475116729736328125f,-0.32900001844763693270579096861183643341064453125f,-0.11400000414252275948712167519261129200458526611328125f,-0.17239999411851136823514707430149428546428680419921875f,-0.66030002967715262229830841533839702606201171875f,0.56960001462101927938164180886815302073955535888671875f,-0.36850001526623976655372416644240729510784149169921875f,1.1942999968647949327049673229339532554149627685546875f,-1.537599994817375748112908695475198328495025634765625f,0.683300013822317087175406413734890520572662353515625f,-0.697900019502639690216483359108678996562957763671875f,0.59679999835491182036406598854227922856807708740234375f,-0.39210000702142711048736600787378847599029541015625f,-1.2817000220894811945981928147375583648681640625f,0.519300029647350669392835698090493679046630859375f,-0.6628000024080276109117448868346400558948516845703125f,-0.14090000197142360516266990089206956326961517333984375f,-0.13530001909732813913933568983338773250579833984375f,-0.84520002793073667479717414607875980436801910400390625f,-0.161900007142126600712117578950710594654083251953125f,-0.3759000376224523876089733676053583621978759765625f,0.491399998894333389642952170106582343578338623046875f,0.289499994546174921339343200088478624820709228515625f,0.161799991798400100151411606930196285247802734375f,-0.2455999985575675059745748285422450862824916839599609375f,0.0554000011742115106017081416212022304534912109375f,-0.041000004127621725302788036060519516468048095703125f,0.375800015684962573914162931032478809356689453125f,0.018700008663535161446134225116111338138580322265625f,-0.276399971699714086526000755839049816131591796875f,0.494599995771050515713795903138816356658935546875f,0.0907000010490417452047040569595992565155029296875f,0.5894000190258026350420550443232059478759765625f,-0.32600000464916234488299551230738870799541473388671875f,-0.15130000180453062608165737401577644050121307373046875f,-0.16610000654458989544082214706577360630035400390625f,0.70380002247989192909471967141143977642059326171875f,0.20040000545829539380093820000183768570423126220703125f,0.13270001571178458021904589259065687656402587890625f,-0.42740003811419047252684322302229702472686767578125f,-0.14320001107007274132598695359774865210056304931640625f,0.19469997434616015397068622405640780925750732421875f,0.9900000146031378012168033819762058556079864501953125f,0.24330000082105396330689472961239516735076904296875f,0.823700030195713139846702688373625278472900390625f,0.08380000310838209287567224237136542797088623046875f,-0.1248999993711710221333532899734564125537872314453125f,-0.6099999940395353092981167719699442386627197265625f,-0.0346000041157008553227569791488349437713623046875f,-0.1730000000521541014819604242802597582340240478515625f,-0.5646999925255773433718786691315472126007080078125f,1.0405999919638038164748650160618126392364501953125f,0.2932000004716217578248915742733515799045562744140625f,0.709500018127262155331891335663385689258575439453125f,-0.1782000032097101893668877892196178436279296875f,-0.0054000029996037479662618352449499070644378662109375f,0.1490999865174291727498712134547531604766845703125f,-0.122399992889165976084342446483788080513477325439453125f,0.244700016069412384922543424181640148162841796875f,-1.02669999228715891348429067875258624553680419921875f,0.9281999843597412702678184359683655202388763427734375f,-0.560000006258487559307468472979962825775146484375f,-1.171300017189979225662455064593814313411712646484375f,1.359400019025802208716413588263094425201416015625f,-0.65660002645254156306009463150985538959503173828125f,0.6356000295519823595213892986066639423370361328125f,0.064800001052021993785956510691903531551361083984375f,0.321200003075599482826874009333550930023193359375f,-0.43439998300224502703059670238872058689594268798828125f,0.794999996088444849107190748327411711215972900390625f,-0.25370000655874613837426068130298517644405364990234375f,0.8622000044465067958299187012016773223876953125f,-0.610800004211068170434373314492404460906982421875f,0.4544000136613846052568987943232059478759765625f,0.42239996949434299722270225174725055694580078125f,-0.9951999727219345004414208233356475830078125f,0.2683999956011771104158469825051724910736083984375f,-0.397500012218952480935740823042578995227813720703125f,0.15859999343156838680357623161398805677890777587890625f,-0.21080000294446943343018574523739516735076904296875f,1.075700035619735928094087284989655017852783203125f,-0.76520001071989529606298674480058252811431884765625f,0.5777000243544581081778233055956661701202392578125f,-0.841100058102608016952217440120875835418701171875f,0.23760003907382465371256330399774014949798583984375f,-0.4297000312685970158099735272116959095001220703125f,1.07350000047683646897667131270281970500946044921875f,-1.5546000022530550577215535668074153363704681396484375f,0.6256000161409378534926872816868126392364501953125f,0.642199995505809084761494887061417102813720703125f,-0.41160001311600158402370652765966951847076416015625f,0.327600011730194040637798025272786617279052734375f,0.419099977278708735184409306384623050689697265625f,-0.377200003924965443502514972351491451263427734375f,0.21470000310540182653085139463655650615692138671875f,1.2238999760210520495462560575106181204319000244140625f,-1.398199947851896940420601822552271187305450439453125f,0.69219999725669600909583323300466872751712799072265625f,-0.423299994897842424279588158242404460906982421875f,0.2953999976724386300475089228712022304534912109375f,-0.22260000293850890074054404976777732372283935546875f,-0.2705999806761745229977123017306439578533172607421875f,-0.4503000291556122913760873416322283446788787841796875f,0.369200018960237574816574124270118772983551025390625f,0.23770000226348642069496008843998424708843231201171875f,0.72890001163482676105331847793422639369964599609375f,0.622800010901689571340966722345910966396331787109375f,0.11880000310838223498421939439140260219573974609375f,0.48780002624988583193044178187847137451171875f,-0.683399988076090370014981090207584202289581298828125f,-0.262799994249641788446325563199934549629688262939453125f,-0.941300017935037036664880361058749258518218994140625f,0.50839998308420131234441896594944410026073455810546875f,0.16689999216645945434578379718004725873470306396484375f,0.33049999576806943224482893128879368305206298828125f,-0.18699999558925639586703937311540357768535614013671875f,-0.032900001509487630801942259495262987911701202392578125f,-0.473000016778707621512012337916530668735504150390625f,0.14719999923110005379811582315596751868724822998046875f,0.0117999996960163056058945585391484200954437255859375f,0.193000005900859861185381305404007434844970703125f,-0.4890000207200646542560207308270037174224853515625f,-0.319200004338473064535719458945095539093017578125f,-0.532500021569430881385187603882513940334320068359375f,0.217599989229440904381362997810356318950653076171875f,-0.0582999996289610411537296386086381971836090087890625f,-0.2817000161290170723304981947876513004302978515625f,-1.0088000043749811851512276916764676570892333984375f,-0.2668000037193298812354669280466623604297637939453125f,-1.1384000262975693118505660095252096652984619140625f,-0.549800005634129007603405625559389591217041015625f,-0.2176000009082257602432974863404524512588977813720703125f,-0.438100014017522187970143932034261524677276611328125f,0.389999987930059521801240407512523233890533447265625f,-0.0176999982401728761516324084368534386157989501953125f,-0.1924999895691872353609142010100185871124267578125f,-0.08159999769330017471702376496978104114532470703125f,-0.07979999971091744637163856168626807630062103271484375f,0.11380000020265601534674715367145836353302001953125f,-0.3146000017315147001539799020974896848201751708984375f,-0.2415999975815414924085189340985380113124847412109375f,-0.2768999990582461379062806372530758380889892578125f,0.6331999867439266349578019799082539975643157958984375f,0.19539999476671210398848188560805283486843109130859375f,0.36400000280141764363861511810682713985443115234375f,0.65119999562501884593501699782791547477245330810546875f,-0.45030001470148572906282424810342490673065185546875f,0.0333000333428383754608148592524230480194091796875f,-0.166000016421079532591420502285473048686981201171875f,0.040100009942054981593173579312860965728759765625f,0.7121999934196470150027380441315472126007080078125f,-1.107999998301267385869550707866437733173370361328125f,0.4170000101923942992243610206060111522674560546875f,-0.4320999705135815105450092232786118984222412109375f,0.855799973216652798413406344479881227016448974609375f,-0.27519998665451961272765402100048959255218505859375f,0.599099981153010663348368325387127697467803955078125f,-0.57499997377395628550544870449812151491641998291015625f,0.33379999215602840223482417059130966663360595703125f,0.01900002869218564427455930854193866252899169921875f,0.50579998018294591854981945289182476699352264404296875f,-0.02249999089166510035653345767059363424777984619140625f,0.119799989727139344353190608671866357326507568359375f,-0.36919999549090853907529208299820311367511749267578125f,0.0803999999448656854639239099924452602863311767578125f,-0.3525000397861004497457315665087662637233734130859375f,-0.012599984759092297537108606775291264057159423828125f,-0.18900000820308926474666577632888220250606536865234375f,0.6168999862432489322827677824534475803375244140625f,-0.586199964183569566245068926946260035037994384765625f,0.3296000040411950493535186978988349437713623046875f,-0.87119998042583546293826657347381114959716796875f,1.6519999607205395619757837266661226749420166015625f,-0.53480000395774851540409144945442676544189453125f,-0.573700007992982818905147723853588104248046875f,0.50459999256730092564993128689820878207683563232421875f,-0.32040000359565012910678660773555748164653778076171875f,0.8434999644160274812065836158581078052520751953125f,-0.837599962109327744741449350840412080287933349609375f,0.45649999609589553273281126166693866252899169921875f,-0.192400006449222615856342599727213382720947265625f,-0.21460000508427601317862354335375130176544189453125f,-0.085299997043609465663394075818359851837158203125f,-0.25790002337694151179903201409615576267242431640625f,-0.1940000001937149942676796854357235133647918701171875f,-0.1278999992370604132929656771011650562286376953125f,0.78579998208284340677209911518730223178863525390625f,-0.7453999700307847309233011401374824345111846923828125f,0.435899994409084001745213754475116729736328125f,0.862799982738494453116118165780790150165557861328125f,-0.04580001486539853061685789725743234157562255859375f,0.137800010678172046851841514580883085727691650390625f,0.089000001274049278521260930574499070644378662109375f,-0.406500003397464837462393916212022304534912109375f,0.955999997198581308310849635745398700237274169921875f,0.2094999986812471348684994154609739780426025390625f,0.11370002602338757924371748231351375579833984375f,-0.5378000446155670744019516860134899616241455078125f,-0.32590001126602297087941906283958815038204193115234375f,-0.536200039695204022649477337836287915706634521484375f,0.82420001292526734459897852502763271331787109375f,0.296900002261996309727010157075710594654083251953125f,1.093600018763542180977310636080801486968994140625f,-0.1762000031501054220228752456023357808589935302734375f,0.08499999772757294014269291437813080847263336181640625f,0.39009998311996429976034050923772156238555908203125f,0.458000030636787958115974106476642191410064697265625f,0.1900000145286322439375226167612709105014801025390625f,-0.1901999590873710843652588664554059505462646484375f,0.7038000434905296032184196519665420055389404296875f,0.2952000152274967259558025034493766725063323974609375f,0.2823000483632096546671164105646312236785888671875f,-0.018600001552700806684015333303250372409820556640625f,-0.098999998293817004224592892569489777088165283203125f,0.010100001299381489161532954312860965728759765625f,-0.31839999008774722000225665397010743618011474609375f,-0.1867999949276446525203709825291298329830169677734375f,0.0520000110864646813979561557061970233917236328125f,0.260400001323223051485911128111183643341064453125f,-0.0140999984011054291244136038585565984249114990234375f,-0.2092999849438665904699519160203635692596435546875f,0.57699998612701863809348878930904902517795562744140625f,0.28729999524056903936752860317938029766082763671875f,0.3859999960064879331866904976777732372283935546875f,0.901799980846047066762594113242812454700469970703125f,0.1460999960392712626600086878170259296894073486328125f,-0.0351999936580664751772928866557776927947998046875f,-0.392000015109777610877728193372604437172412872314453125f,-0.0688000048965216048912907353951595723628997802734375f,-0.16320001997351685218973216251470148563385009765625f,-0.66420001769363867794027100899256765842437744140625f,-0.3684000018969177947525395211414434015750885009765625f,-0.8561000274062156023546776850707828998565673828125f,0.4197999986678360162528633736656047403812408447265625f,-0.5939000238537790465898069669492542743682861328125f,0.4987000367522238519768507103435695171356201171875f,0.274599982509017070952950234641321003437042236328125f,0.2306000128626823908462029066868126392364501953125f,1.5364999828338614662470718030817806720733642578125f,-1.271999981582164540583335110568441450595855712890625f,0.83110000505447345631182543002068996429443359375f,-0.43420006634593022454282618127763271331787109375f,0.06820004590153672108243654292891733348369598388671875f,-0.23370002442151338595266452102805487811565399169921875f,0.5598000096201900532832951284945011138916015625f,-1.096399988016486304331920109689235687255859375f,0.353200009393692226922212284989655017852783203125f,-0.6322999861508602492676800466142594814300537109375f,0.5546000058293338863535382188274525105953216552734375f,-0.3336000072523950255032332279370166361331939697265625f,0.8720000373125085246783783077262341976165771484375f,-0.109000014424324387363185451249592006206512451171875f,0.439900016283989003795795724727213382720947265625f,0.92670005427599022596041322685778141021728515625f,-0.4378000294536352754448671475984156131744384765625f,0.492000025987625377865697373636066913604736328125f,0.0030999836564067351218909607268869876861572265625f,-0.383199990692734804298424933222122490406036376953125f,0.018100001239776730699304607696831226348876953125f,-0.524600010299682395498166442848742008209228515625f,-0.11620001514554001431633878382854163646697998046875f,-0.258299996089935035570306354202330112457275390625f,0.622099988245963686495088040828704833984375f,-0.528999998629092971214049612171947956085205078125f,0.33140000448226913931648596189916133880615234375f,0.628999997437000413214036598219536244869232177734375f,-0.24359996444881015786876332640531472861766815185546875f,0.33599999004602398855467981775291264057159423828125f,1.5741999648690221391689192387275397777557373046875f,-1.347599950858950901277921730070374906063079833984375f,0.849999998211860230412639793939888477325439453125f,-0.5540000087618828228386291812057606875896453857421875f,0.651600008273124586821722914464771747589111328125f,-0.310600013309717315479474564199335873126983642578125f,-0.412300028097629223111653118394315242767333984375f,0.32440001209676250226721094804815948009490966796875f,-0.240700005221366808427774230949580669403076171875f,0.3741999826759094815997741534374654293060302734375f,-0.082199999976158011349980370141565799713134765625f,0.5832000153541565534709434359683655202388763427734375f,0.18870000408142806325173523873672820627689361572265625f,0.33610002293586749289033832610584795475006103515625f,-0.28660003084838425735370037727989256381988525390625f,-0.26300000988692051606676614028401672840118408203125f,-0.29710002907514621028894907794892787933349609375f,-0.616000010758638527619268643320538103580474853515625f,-0.0227000050947070430851226774393580853939056396484375f,0.2698999832034107981826309696771204471588134765625f,0.7578000097841022242306507905595935881137847900390625f,0.36000000003725285413480605711811222136020660400390625f,0.75810002329349490679533118964172899723052978515625f,0.5176000038325785368442666367627680301666259765625f,0.05960000009238719531623473812942393124103546142578125f,0.3032000125229359355216729454696178436279296875f,0.197000015780329551517979780328460037708282470703125f,0.1099000013828277388938658987171947956085205078125f,-0.21899999237060541190658113919198513031005859375f,0.2766000081986188519067582092247903347015380859375f,0.2336999999463558186363343338598497211933135986328125f,0.6438000064611433259642581106163561344146728515625f,0.40359998389482465430688762353383935987949371337890625f,0.1921999949097632498506982301478274166584014892578125f,0.2147999971032135135828866623342037200927734375f,0.2547999848842621428701704644481651484966278076171875f,-0.09370000251680610237059454448171891272068023681640625f,-0.37050002348423038966984677244909107685089111328125f,1.146799989041685563506689504720270633697509765625f,0.258699993836879638120507252097013406455516815185546875f,0.75190001284182006457967872847802937030792236328125f,-0.6204000095188619834374321726500056684017181396484375f,-0.168300004769861732967228817869909107685089111328125f,-0.62870002602338814767790609039366245269775390625f,1.0890000254884366626129121868871152400970458984375f,0.24480001094266778949304352863691747188568115234375f,0.719700043599308170172434984124265611171722412109375f,0.723999988496303092944117452134378254413604736328125f,0.2479999927133320902772339877628837712109088897705078125f,0.3347999910682430968478229260654188692569732666015625f,-0.23379999968111508934498488088138401508331298828125f,0.039400000138580804076582353445701301097869873046875f,0.452100004637241426053151371888816356658935546875f,-0.940999995037913095075055025517940521240234375f,-0.405100019478798145655673579312860965728759765625f,0.71930001534223553250058103003539144992828369140625f,-0.6500000053644179320855300829862244427204132080078125f,0.397700011837482492182971327565610408782958984375f,-0.183900051271915998540862346999347209930419921875f,-0.33619997290074810081250689108856022357940673828125f,-0.085000011920929097186672152020037174224853515625f,-1.2855000014305115740853580064140260219573974609375f,1.125399995511770168121756796608678996562957763671875f,-0.68980002005100260475956019945442676544189453125f,0.65370001678466760353103381930850446224212646484375f,-0.5173999990731477094385581949609331786632537841796875f,0.368400002753734412408448406495153903961181640625f,0.6823999900579449384707686476758681237697601318359375f,-1.0359999945163724799357396477716974914073944091796875f,0.394800009471178103837019079946912825107574462890625f,0.4610000359416002169155035517178475856781005859375f,0.140199970635772164229138070368207991123199462890625f,0.2215000141263008259784328402020037174224853515625f,-0.06959997841119758277272921986877918243408203125f,0.0627999960750340591175699955783784389495849609375f,-0.02340000007152553962441743351519107818603515625f,0.483399987554550047974544213502667844295501708984375f,-0.18019996922016179308201344611006788909435272216796875f,0.25609998806714973085263409302569925785064697265625f,0.762699981662631643075656029395759105682373046875f,-0.7739999633878473783710205680108629167079925537109375f,0.4091000062242151624758434991235844790935516357421875f,1.3986999705910676539133419282734394073486328125f,-1.798799963247776023678170531638897955417633056640625f,0.7983000028699633876527741449535824358463287353515625f,-0.58509999384880106987338876933790743350982666015625f,0.9627999861657621227806203023646958172321319580078125f,-0.349900005257129720348530099727213382720947265625f,1.3245000061690814785464453962049447000026702880859375f,-1.714599988394975849104184817406348884105682373046875f,0.75660002172142293286327685564174316823482513427734375f,0.9459999998062844639434842974878847599029541015625f,-0.707999974310398183519055237411521375179290771484375f,0.514400000920891553590763578540645539760589599609375f,-0.759700002062320578488652245141565799713134765625f,0.45759999675452700529376670601777732372283935546875f,-0.3923000114083290412736459984444081783294677734375f,-0.6829999820590018089205841533839702606201171875f,-0.087400016880035735766796278767287731170654296875f,0.0683999958992005385738366385339759290218353271484375f,-0.13769999637752772247978327868622727692127227783203125f,-0.56369998519420594362827614531852304935455322265625f,0.8452000055789945509587823835317976772785186767578125f,0.307399999818205760249156810459680855274200439453125f,0.5620000196546313109280390563071705400943756103515625f,-0.752800046664476862900983178406022489070892333984375f,-0.40550001495331533529764556078589521348476409912109375f,-1.10130006904602151251992836478166282176971435546875f,0.2304000002801416258080280385911464691162109375f,0.0875999972760677092509240537765435874462127685546875f,-0.3976000170946125678028693073429167270660400390625f,0.331000023946165899957350120530463755130767822265625f,0.289400002969801395380500252940692007541656494140625f,0.886100037539005569442451815120875835418701171875f,0.896600040832162381576608822797425091266632080078125f,0.290200013066828355068338396449689753353595733642578125f,0.478900043970347244481899906531907618045806884765625f,-0.8912000114202498490811876763473264873027801513671875f,-0.397000000469386549184491741470992565155029296875f,-1.28300002657622069790477326023392379283905029296875f,0.27360000460743894024062683456577360630035400390625f,0.1091999985456466593092272887588478624820709228515625f,0.7784000119924545657568160095252096652984619140625f,0.428200017663836529635545957717113196849822998046875f,0.3812000050500035097655882054823450744152069091796875f,1.0865000421404840125205737422220408916473388671875f,0.4008000193357468265986653932486660778522491455078125f,0.0222000051170587742177531254128552973270416259765625f,-0.0905999756097788377928736736066639423370361328125f,0.3708000151634218699570055832737125456333160400390625f,0.12180000520944600506112465154728852212429046630859375f,0.37040001801252397939379079616628587245941162109375f,-0.56540000571906556814383293385617434978485107421875f,0.00050000046193598990384998614899814128875732421875f,0.11839999887943264411660493351519107818603515625f,0.556599983313679214091962421662174165248870849609375f,0.25609999149441708965468933456577360630035400390625f,0.8921999993801108530533383600413799285888671875f,0.2557999910980461866216728594736196100711822509765625f,-0.0320000032261014144552291327272541821002960205078125f,0.3223000000834463918408800964243710041046142578125f,-0.73939999168217152014648263502749614417552947998046875f,-0.01360001667737975594718591310083866119384765625f,0.5870999728232622061341317021287977695465087890625f,-0.4547999888330698237126625826931558549404144287109375f,0.3027000004753468420659601179067976772785186767578125f,0.97380000757873030892142196535132825374603271484375f,-0.8353999891042709347033223821199499070644378662109375f,0.538900005450844599153015224146656692028045654296875f,-0.3787000427126887558415546664036810398101806640625f,0.483400035089254220821430862997658550739288330078125f,-0.2360000150799752560715205618180334568023681640625f,0.658600027704238755177357234060764312744140625f,0.0605999981105327378827496431767940521240234375f,0.320500010967254667093584430404007434844970703125f,-0.1704999725222595685636406415142118930816650390625f,0.059199963900447460929399312590248882770538330078125f,-0.068699997711181737258812063373625278472900390625f,1.1329000458776956605788654997013509273529052734375f,-0.980200022193789433089250451303087174892425537109375f,0.62340003002285993005671116407029330730438232421875f,-0.49980000958293668844589774380438029766082763671875f,0.9135999913454055931794073330820538103580474853515625f,-0.3194000025227665684468547624419443309307098388671875f,-0.1166000057399276812475363840349018573760986328125f,-0.524200003314017948241598787717521190643310546875f,-0.019100000002980177526978877722285687923431396484375f,-0.1806999822735786409566571819595992565155029296875f,0.134999990090727717273466623737476766109466552734375f,-0.072600008749962086085361079312860965728759765625f,0.77220001189708664668387427809648215770721435546875f,-0.7428000220775599071743044987670145928859710693359375f,0.4188000195741654607672899146564304828643798828125f,0.34420001314878501119665088481269776821136474609375f,-0.4830000038146973206920620214077644050121307373046875f,0.2023000099182130639974275254644453525543212890625f,-1.096600001418590153434706735424697399139404296875f,1.18719999975264034475230801035650074481964111328125f,-0.6006000212073328015094375587068498134613037109375f,0.2433999789118761469808305264450609683990478515625f,-0.580999969705939367514702098560519516468048095703125f,0.167299986076354656461262493394315242767333984375f,0.24069997094869588494248091592453420162200927734375f,-0.8573999822348354893364330564509145915508270263671875f,0.164999999105930328369140625f,-0.639099994564056306245447558467276394367218017578125f,-0.18510001053810132276566946529783308506011962890625f,0.51260003147423294223017364856787025928497314453125f,0.2905000085830689027233120214077644050121307373046875f,0.7166000368833547184976850985549390316009521484375f,0.0449999982863664715893037282512523233890533447265625f,-0.08459999953359365054694762875442393124103546142578125f,-0.54090000662803650044452297152020037174224853515625f,0.743800014880299453778889073873870074748992919921875f,0.1796000010237097654908211552537977695465087890625f,1.04190003422498733698375872336328029632568359375f,0.79960001477003128655951513792388141155242919921875f,0.12380000571608551496183281415142118930816650390625f,0.10680000967979452752842917107045650482177734375f,0.243400004169344708060407356242649257183074951171875f,-0.0715000009387731783050412559532560408115386962890625f,-0.3719999973773955304068294935859739780426025390625f,-0.846799985688924561344492758507840335369110107421875f,-0.236199994581937744442257098853588104248046875f,-1.00400000429153379855051753111183643341064453125f,-0.0628000077724457650418798948521725833415985107421875f,-0.05120000225603582588718154511298052966594696044921875f,0.16100000017881388458818037179298698902130126953125f,-0.0827999921262261295851203612983226776123046875f,-0.1154999949112534007866770480177365243434906005859375f,0.4345000262856490991225655307061970233917236328125f,0.778200013640522936242405194207094609737396240234375f,0.17310000161230565396408564993180334568023681640625f,0.9084000274896624205211992375552654266357421875f,0.580400015032291438643596848123706877231597900390625f,0.22180000394284726805693708229227922856807708740234375f,0.5204000232279302196047865436412394046783447265625f,-0.124000008612870915580828068414120934903621673583984375f,-0.13039999826848502895160208936431445181369781494140625f,-0.1540000041425226839919560006819665431976318359375f,0.78600001776218419990982511080801486968994140625f,0.187500005401670943872005636876565404236316680908203125f,-0.004099975341558259600560631952248513698577880859375f,0.4038000238209964454227929309126920998096466064453125f,0.23940000505596403801433780245133675634860992431640625f,0.57370001976490048178902725339867174625396728515625f,-0.988000023856759668205995694734156131744384765625f,-0.254300011914968593362829096804489381611347198486328125f,-0.65570004180371821433936929679475724697113037109375f,0.9014000430017714560193553552380762994289398193359375f,-0.65829997195005451882110492078936658799648284912109375f,-0.6380000175237654502780060283839702606201171875f,-0.8290000083148481024863940547220408916473388671875f,-0.974600007431208670283240280696190893650054931640625f,-0.45399998044967670995930575372767634689807891845703125f,0.42639998237267151548479660050361417233943939208984375f,-0.61699997435510189891516574789420701563358306884765625f,0.911500028133392703466597595252096652984619140625f,-0.6232000037312506179887350299395620822906494140625f,0.1424999844282865968381202037562616169452667236328125f,0.3440000096559521214345522821531631052494049072265625f,0.2781999729976061264125064553809352219104766845703125f,0.9406000003829593225646021892316639423370361328125f,0.3059999742507937270374895888380706310272216796875f,-0.4743000001430512924116555950604379177093505859375f,0.56339996009916115671245506746345199644565582275390625f,-0.35850002379715473210808340809307992458343505859375f,-0.1101000102400779834255217792815528810024261474609375f,0.1884999983906745502082458187942393124103546142578125f,0.84400002217292779960189363919198513031005859375f,0.20900000444054622761314021772705018520355224609375f,0.7434000041320916185583200785913504660129547119140625f,-0.251000010609626400537308654747903347015380859375f,-0.21819998838305476596133303246460855007171630859375f,0.8047999764278532808248201035894453525543212890625f,0.0330000133514403160006622783839702606201171875f,0.746100003713369375191177823580801486968994140625f,-0.2557999812632800740175298415124416351318359375f,-0.6679999904036524327466395334340631961822509765625f,0.14809999185204514304814438219182193279266357421875f,-0.68679997116029289916383504532859660685062408447265625f,0.442499989271163851611845529987476766109466552734375f,0.012900009593367567362065528868697583675384521484375f,0.77429999090433110353615120402537286281585693359375f,-0.36599999613687383970273003797046840190887451171875f,0.0296999989330768432438389936578460037708282470703125f,-0.313999997306615041825494927252293564379215240478515625f,-0.52300004261359589463609154336154460906982421875f,-0.3854000046014787184134320341399870812892913818359375f,0.3163999822981657938925081907655112445354461669921875f,-0.343500018954276953309090458787977695465087890625f,0.2613000022888183337954615126363933086395263671875f,0.8764999824240804482400335473357699811458587646484375f,0.70749998807907132913896930404007434844970703125f,0.600200009304284964883891007048077881336212158203125f,0.30839999090731140540810883976519107818603515625f,-0.6090000004917381914992802194319665431976318359375f,0.27080001464188097060059590148739516735076904296875f,0.49660002555847171379355131648480892181396484375f,-1.61890002772808117015301831997931003570556640625f,-0.200900013184547532318902085535228252410888671875f,-0.4453000073999167085503358975984156131744384765625f,0.90550000515580197912868243292905390262603759765625f,-0.1870999982297416597276651373249478638172149658203125f,-0.0153999855279920438988483510911464691162109375f,1.893400027489661852087010629475116729736328125f,0.2008000095009805363588384352624416351318359375f,-0.4144000164180994261187152005732059478759765625f,0.73390002161860490303979531745426356792449951171875f,0.4783000248491766637215505397762171924114227294921875f,-1.08079999012947070013979100622236728668212890625f,-0.6147999646186832478633732534945011138916015625f,0.8962000295996670473641643184237182140350341796875f,-0.37010003437995919739478267729282379150390625f,-1.9476999757766719767460017465054988861083984375f,0.288500011801719580262215458787977695465087890625f,-1.311100001478195054005482234060764312744140625f,0.35320004336833932256922707892954349517822265625f,0.1925999861001965740570085472427308559417724609375f,-0.218600000137090688667740323580801486968994140625f,0.9092000027179718557590604177676141262054443359375f,-0.024399998760223251537126998300664126873016357421875f,-1.058600003713369375191177823580801486968994140625f,1.5644000223040581687428129953332245349884033203125f,0.163400012588501564181342473602853715419769287109375f,0.6669000121712684858721331693232059478759765625f,-0.846000023722648819557434762828052043914794921875f,-0.15820001893043578178321695304475724697113037109375f,-0.2178999724335966003962994363973848521709442138671875f,-1.0072999766886241435059901050408370792865753173828125f,-0.17450001939386139415688603548915125429630279541015625f,0.4794000047206878889483050443232059478759765625f,0.481199988174438431087764911353588104248046875f,0.230600000345706490634256624616682529449462890625f,0.701599997842311751128363539464771747589111328125f,-0.313400032854080023980714031495153903961181640625f,-0.1164000059723857560811666189692914485931396484375f,0.1980999936401843797284527681767940521240234375f,0.37529999376535416732991734534152783453464508056640625f,-0.580500003293156652262041461654007434844970703125f,-0.29219999982714650599291417165659368038177490234375f,-0.1792999921336770885460509816766716539859771728515625f,-0.095500016927719144632646930404007434844970703125f,-1.02590002003908153938027680851519107818603515625f,-0.0447000187888739741737254007603041827678680419921875f,0.570499997630715238727816540631465613842010498046875f,0.50500000916421405872114291923935525119304656982421875f,0.211100001254677760442746148328296840190887451171875f,-0.26249999582767458150556194595992565155029296875f,1.07760001187920551046772743575274944305419921875f,-0.353699999685585186881553454441018402576446533203125f,-0.324500002965331102444679345353506505489349365234375f,0.63350001187622539955413003553985618054866790771484375f,0.076300009180605365965988085008575581014156341552734375f,-0.37150004065036767997298738919198513031005859375f,0.18880000362992266360606663511134684085845947265625f,0.7040999714300040235315236714086495339870452880859375f,-0.09350000762939458809341886080801486968994140625f,-0.8649999853968619589750232989899814128875732421875f,0.54070000149607633233017622842453420162200927734375f,0.717000005125999706478978623636066913604736328125f,0.50970001219511029688646885915659368038177490234375f,0.857199981014430623105226914049126207828521728515625f,0.18900000324845311894250698969699442386627197265625f,0.5178000028550624467271745743346400558948516845703125f,0.0102000025987625253520718615618534386157989501953125f,0.56149998894333830179448341368697583675384521484375f,1.0942000125527380216539086177363060414791107177734375f,0.472699993769824633460530094453133642673492431640625f,-0.2819999863505362469595638685859739780426025390625f,-0.633900017297267748261901942896656692028045654296875f,-0.3097999990031123918043931553256697952747344970703125f,0.39599998348951370719106535034370608627796173095703125f,-0.55199999672546995466149155618040822446346282958984375f,0.367799965974688947145665451898821629583835601806640625f,-0.6604999811649321372897247783839702606201171875f,0.2401999887406827127733777160756289958953857421875f,-0.481299986530840495646543786278925836086273193359375f,-0.3654999847412110369759830064140260219573974609375f,-0.343400015419721427178956218995153903961181640625f,-0.43469998436719203827038882081978954374790191650390625f,0.37299999174475662044869750388897955417633056640625f,0.0730000069439411536365014399052597582340240478515625f,-0.42910003032684329582480131648480892181396484375f,-2.29129996831417059865998453460633754730224609375f,0.383100011074542834421663428656756877899169921875f,-0.9171000266909601350562297739088535308837890625f,-1.01669999943971589573266101069748401641845703125f,0.880500028252601651956865680404007434844970703125f,-0.2766000046968457581897382624447345733642578125f,2.08890005693435654166023596189916133880615234375f,0.0818999984621999743694686912931501865386962890625f,-1.00330001933574664008119725622236728668212890625f,-0.701299942088127181705203838646411895751953125f,0.8707000174403187742200316279195249080657958984375f,0.808699988770484878841671161353588104248046875f,-1.5917000021219251948423334397375583648681640625f,-0.17989998528957329426702926866710186004638671875f,-1.27980000872612009743534144945442676544189453125f,0.4977000020027162463520653545856475830078125f,0.0496999844789502986941442941315472126007080078125f,-0.395100006961822458606548025272786617279052734375f,-0.370199978458881417964221327565610408782958984375f,0.4850000089406965031457730219699442386627197265625f,-1.74469999759197236244290252216160297393798828125f,0.81740001032352438414818607270717620849609375f,0.3581999775052064904912185738794505596160888671875f,-0.278699972677230789486202411353588104248046875f,-0.27450001347065011714221327565610408782958984375f,-0.6939000259399410452942902338691055774688720703125f,-0.31139998984187844399684763629920780658721923828125f,0.239399986466765302139947380055673420429229736328125f,0.35780000561177727202988307908526621758937835693359375f,-0.7062999981351196776557799239526502788066864013671875f,0.9394000063598155758626262468169443309307098388671875f,0.56800001189857718397746566552086733281612396240234375f,0.9455999773979186784345074556767940521240234375f,-0.855899988746642748083104379475116729736328125f,-0.139899982309341197606045170687139034271240234375f,-0.13990001568794241393334232270717620849609375f,-1.440499963581562070658037555404007434844970703125f,0.37470000891685462107716375612653791904449462890625f,0.17460001285150639205312472768127918243408203125f,1.133699971168488307426969186053611338138580322265625f,-0.5805000091604883838414252750226296484470367431640625f,-1.4102000258445741565083153545856475830078125f,0.01179999895095829032243273104541003704071044921875f,-0.172000011086463899800946819595992565155029296875f,-1.00399998506903642692122957669198513031005859375f,0.6221999951705334819251902445103041827678680419921875f,-0.47000001341104535867998492904007434844970703125f,-0.062200007873773444089238182641565799713134765625f,0.656200003634392903251182360691018402576446533203125f,0.086000025510788447036247816868126392364501953125f,1.19230003003478035594753237091936171054840087890625f,-0.17079999281167967950523234321735799312591552734375f,-0.39500000357627840230634319595992565155029296875f,0.11900001785159108891321011469699442386627197265625f,0.6675999917253852000698088886565528810024261474609375f,-0.3545000194311143104641814716160297393798828125f,-1.06539998970031746239328640513122081756591796875f,-0.28189999659955500277419560006819665431976318359375f,0.8805000097751616294772247783839702606201171875f,0.53189999458789838371330915833823382854461669921875f,0.8400999932155011062917537856264971196651458740234375f,-0.3415000004172323855300419381819665431976318359375f,0.0451000035345554550758606637828052043914794921875f,0.24869999741315840235955647585797123610973358154296875f,0.81850001657009130440201261080801486968994140625f,0.89749998673796671511126987752504646778106689453125f,1.0598999846935270863212963377009145915508270263671875f,1.118499980211257849305184208787977695465087890625f,-0.32680000580549251054662818205542862415313720703125f,0.397899970850348783102390370913781225681304931640625f,-0.2400000050291419828596417573862709105014801025390625f,0.309599984036386055574752163010998629033565521240234375f,0.110799995829164998895066673867404460906982421875f,-0.22699999946355819968601963410037569701671600341796875f,0.83029999337419868599141636877902783453464508056640625f,0.2355999993771314693358220893060206435620784759521484375f,-0.51250000298023223876953125f,-0.6874999794363976235445079510100185871124267578125f,-0.515699982608854856636071417597122490406036376953125f,-0.3800000047683713688684292719699442386627197265625f,-0.611199984747171409793509155861102044582366943359375f,0.2677999953672289823458640967146493494510650634765625f,0.625999996956437598782940767705440521240234375f,0.3714999938607215224095625671907328069210052490234375f,-0.1971999994359909802454922100878320634365081787109375f,0.716000002861022721845074556767940521240234375f,-0.73300001603364961511033470742404460906982421875f,0.1493999990582466352861956693232059478759765625f,0.266999994993209810445478069595992565155029296875f,0.70180000491142191521021231892518699169158935546875f,-0.3754999921917914207369904033839702606201171875f,0.794300013852119235480131465010344982147216796875f,-0.38230003077983898407410379149951040744781494140625f,-0.554999999701976776123046875f,0.109800003361701925541638047434389591217041015625f,0.13619999055862397341343239531852304935455322265625f,0.9107000105857849803214776329696178436279296875f,-0.60740000913143177285746787674725055694580078125f,-0.1661999997973442333432103623636066913604736328125f,-0.876300006163120315250125713646411895751953125f,-0.21149999207258218802962801419198513031005859375f,0.34990000168085089171654544770717620849609375f,0.0579999895095824058444122783839702606201171875f,0.81379999096393618174261064268648624420166015625f,-0.0277999869108196406841670977883040904998779296875f,-0.20560000965893276969609360094182193279266357421875f,-0.71540000430345518367403201409615576267242431640625f,-0.127800006133317989309716722345910966396331787109375f,-1.1152000389575960070942528545856475830078125f,-0.493099977397918110000318847596645355224609375f,0.8027000315070154101704247295856475830078125f,0.4594000101037323258168498796294443309307098388671875f,1.4608999769747257335694712310214526951313018798828125f,0.18290000784248106047158444198430515825748443603515625f,1.304200004208087904089552466757595539093017578125f,0.223999976277351464659659541212022304534912109375f,-0.8866000133395193216756524634547531604766845703125f,0.16830000354051577460268163122236728668212890625f,-0.419100036287307631255316664464771747589111328125f,-0.81430003799200090242038641008548438549041748046875f,-1.2215000178515911244403469027020037174224853515625f,0.4970000343322755753661112976260483264923095703125f,0.55600001627206818710646984982304275035858154296875f,0.660200007367134134028674452565610408782958984375f,-0.4745000092983246986477752216160297393798828125f,-0.66510002037286763965084901428781449794769287109375f,0.6303999913394449094994342885911464691162109375f,0.604999959021807143244586768560111522674560546875f,0.1768000157892704038431475055404007434844970703125f,-0.18810003640651729028832050971686840057373046875f,-0.8378999734207990268686216950300149619579315185546875f,-1.0329999743103979170655293273739516735076904296875f,0.350799983274936888477668617269955575466156005859375f,-0.3037999845564367529249238941702060401439666748046875f,0.7755000219941141637036707834340631961822509765625f,0.366000004351139107683366091805510222911834716796875f,0.2204999929368496314197045649052597582340240478515625f,0.2320000078082085082087360206060111522674560546875f,0.213199994492530908729577276972122490406036376953125f,0.3579999950975178979462043571402318775653839111328125f,-0.453500003457069311707527958787977695465087890625f,-0.55750000193715099072733210050500929355621337890625f,-0.526899998685717463331457111053168773651123046875f,0.19149999624490732230697176419198513031005859375f,0.1777999961495400071953554288484156131744384765625f,0.60209999212026588111257296986877918243408203125f,-0.011000000476837357155091012828052043914794921875f,0.41180000312328335354550290503539144992828369140625f,-0.17499998666346083808775802026502788066864013671875f,0.323999998927116283908844707184471189975738525390625f,-0.38340000044405453394347205176018178462982177734375f,0.29919999824762333329175589824444614350795745849609375f,-0.224000009655952680986956693232059478759765625f,0.22850000926852231941666104830801486968994140625f,0.728400007894634882177342660725116729736328125f,-0.58849998646974566174350229630363173782825469970703125f,0.74869998618140820578759075942798517644405364990234375f,-0.62329998349845416871772840750054456293582916259765625f,0.1955000091791152527775921043939888477325439453125f,-0.4369999948441982695612750831060111522674560546875f,-0.80289998645931481835447129924432374536991119384765625f,0.9460000162720678673622387577779591083526611328125f,-0.532000013917684366759885961073450744152069091796875f,0.1213999990060923295231987140141427516937255859375f,0.17299999427795409445707264239899814128875732421875f,0.6919000126183032506332892808131873607635498046875f,0.6899999960884450889153640673612244427204132080078125f,0.38600000435113912544693448580801486968994140625f,-0.607099997037649163900141502381302416324615478515625f,-0.26040000236630433949613916411180980503559112548828125f,-0.70249998912215261270830524154007434844970703125f,0.2581999827206136188806340214796364307403564453125f,-0.601299972449243380623329358058981597423553466796875f,0.051499962568282597885627183131873607635498046875f,-0.375100012475252153620886019780300557613372802734375f,0.0955000142641366789320045427302829921245574951171875f,-1.0599999915063385014235564085538499057292938232421875f,-0.31700001738220462410566824473789893090724945068359375f,-0.244700010649114840788342917221598327159881591796875f,0.0797000186398626464523431423003785312175750732421875f,0.342900017900765130551832271521561779081821441650390625f,1.46579999549388872992494725622236728668212890625f,0.1934999641180041862753569148480892181396484375f,-1.0029000069856639498766526230610907077789306640625f,-1.08830000793635850442342416499741375446319580078125f,-0.55419998841285700308390005375258624553680419921875f,0.440600003176927490500247586169280111789703369140625f,-0.70160001631975177360800444148480892181396484375f,-1.70709998139142982154226046986877918243408203125f,0.5559000084161755950162842054851353168487548828125f,0.0938000284962356989382215033401735126972198486328125f,0.9645999827325340003625342433224432170391082763671875f,0.20399999419599745831277459728880785405635833740234375f,-0.373800023448467300113406963646411895751953125f,1.260900022721290980598496389575302600860595703125f,0.35750002115964907289935581502504646778106689453125f,-0.23349997842311864815201261080801486968994140625f,0.29319996647834756231532082892954349517822265625f,0.852800017607211913173159700818359851837158203125f,0.116300008249282882388797588646411895751953125f,0.18699999320507032507521216757595539093017578125f,0.7376000105381006477500704932026565074920654296875f,1.27500002801418332865068805404007434844970703125f,0.86220000534057561480949516408145427703857421875f,-0.812200033354759654002918978221714496612548828125f,0.418699999201297856643577688373625278472900390625f,1.251700017023086530798536841757595539093017578125f,0.429700025904178772861996549181640148162841796875f,-0.25009999265670757040425087325274944305419921875f,-1.40889995977878612620770581997931003570556640625f,0.10070000790357624964599381200969219207763671875f,-0.8164000019490718074166579754091799259185791015625f,0.3148000295877455556592394714243710041046142578125f,0.52840001288652427291481217253021895885467529296875f,0.37040002181232001277066956390626728534698486328125f,1.03840002629756877894351418945007026195526123046875f,0.020400005942583288032210475648753345012664794921875f,1.16020003359317769309200230054557323455810546875f,-0.08069999832957985841375148083898238837718963623046875f,0.35099999094009415312456212632241658866405487060546875f,-0.68969999749884014494938355710473842918872833251953125f,0.177999980345368624679025515433750115334987640380859375f,-0.26900000691786407625016863676137290894985198974609375f,0.2066000097073613928078117396580637432634830474853515625f,0.09240000255629421399561351790907792747020721435546875f,0.228500005841254960614605806767940521240234375f,-0.52429999448060993216813585604541003704071044921875f,-0.8910999787166715702113606312195770442485809326171875f,0.231499991178512498635200245189480483531951904296875f,0.067899996182322563953448479878716170787811279296875f,0.79609998769462098078264489231514744460582733154296875f,-0.200000010430812835693359375f,-0.58149999967217436136479591368697583675384521484375f,0.6851999904543160102576848657918162643909454345703125f,-0.34249999374151218756168191248434595763683319091796875f,0.56359997948035600590088733952143229544162750244140625f,-0.276100004532933096523805716060451231896877288818359375f,-0.1989999908804893635760890902020037174224853515625f,0.864700015920400577584814527654089033603668212890625f,-0.0173999820858241616150507979909889400005340576171875f,-0.947500008940696858417140902020037174224853515625f,0.490899963861703980683159898035228252410888671875f,-0.0605000026226043541299759453977458178997039794921875f,-1.054999988675117350567234097979962825775146484375f,0.32689998147487653312737165833823382854461669921875f,-0.273599992798268853988474802463315427303314208984375f,0.116000026702881342544060316868126392364501953125f,-0.066799992096424176679647644050419330596923828125f,-0.9059999989122149433029562715091742575168609619140625f,-0.8809999963045118676063793827779591083526611328125f,0.72920000972151743923177491524256765842437744140625f,-0.61039998482018698044981874772929586470127105712890625f,0.060999967694282730690247262828052043914794921875f,-0.6657999813377859510410416987724602222442626953125f,0.4011999724909667275341007552924565970897674560546875f,-0.09000000834465016197327713598497211933135986328125f,0.5064000070154666133248611004091799259185791015625f,0.430799990725517389922316624506493099033832550048828125f,-0.307000005125999120281221621553413569927215576171875f,0.486400012752413513084093210636638104915618896484375f,-0.45180000722110225108707481922465376555919647216796875f,0.44749999821186037252118694595992565155029296875f,0.39050003033876468094831579946912825107574462890625f,0.7021000305652620454566203989088535308837890625f,0.6124999976158136405501863919198513031005859375f,0.57070001475810983038172707892954349517822265625f,0.38459999092817298560476046986877918243408203125f,0.5858999848723414061169023625552654266357421875f,0.352300010812282238248371868394315242767333984375f,-0.0349999852478504180908203125f,1.05019998576045026084102573804557323455810546875f,-0.49720001502633071055470281862653791904449462890625f,-0.3518999922782182210312384995631873607635498046875f,0.81270000274777398630021707504056394100189208984375f,0.2284999958574769340913235282641835510730743408203125f,-0.55000001195818182253560735261999070644378662109375f,0.588400028756260962126134472782723605632781982421875f,0.70260002521574504985579778804094530642032623291015625f,0.4383000018641354245119146071374416351318359375f,-0.39959998001307273085558335878886282444000244140625f,0.441200005869566869165510070160962641239166259765625f,-0.6644000071048736799639300443232059478759765625f,0.0632000141620636668449151329696178436279296875f,0.63300002199411409264939720742404460906982421875f,1.0394000145554542768877581693232059478759765625f,1.218699977743625595394405536353588104248046875f,-0.7919000106811520112159996642731130123138427734375f,-0.15620003705024743112517171539366245269775390625f,-1.310500002622603687996161170303821563720703125f,-0.5419000032305714142921715392731130123138427734375f,0.61140000939965233328621252439916133880615234375f,0.248099956983327984971765545196831226348876953125f,0.33270002867579506755646434612572193145751953125f,-1.2155999881267547380048199556767940521240234375f,-0.23279998631477383241872303187847137451171875f,1.001600031518936617658255272544920444488525390625f,-0.5119000184297561872881487943232059478759765625f,-1.036099980616569382618763484060764312744140625f,0.70430001415014231724853743799030780792236328125f,-0.296899972571432524404144714935682713985443115234375f,1.354999983608722313732641850947402417659759521484375f,0.4801999993950125666941630697692744433879852294921875f,0.582499988526105738628757535479962825775146484375f,0.70449997219443361018420546315610408782958984375f,-0.466500013083219755571917630732059478759765625f,0.69219998835325213804026134312152862548828125f,-0.3154000030368564466698444448411464691162109375f,-1.27250000119209261129071819595992565155029296875f,0.3817999861359595570320379920303821563720703125f,-0.738700000393390343589317126316018402576446533203125f,-0.750499980866909055521318805404007434844970703125f,0.3556999903202058277429387089796364307403564453125f,-0.4556999770209195776971000668709166347980499267578125f,0.6725000013411044363920154864899814128875732421875f,0.396399996882677019272023244411684572696685791015625f,-0.0475000048801301755219128608587197959423065185546875f,-0.043499989822506801573354096035473048686981201171875f,0.57839999329149727191889951427583582699298858642578125f,0.0702999988317489776790125688421539962291717529296875f,-0.5175000027194618379411394926137290894985198974609375f,0.658800009329616909070637120748870074748992919921875f,0.15250000461935997453366553600062616169452667236328125f,-0.9014999890550967620583833195269107818603515625f,-0.148499997645616499397647203295491635799407958984375f,-0.346099995480477762299642563448287546634674072265625f,-0.33850001239776617012466886080801486968994140625f,0.37840000051856037543984712101519107818603515625f,0.340299997229874140458605324965901672840118408203125f,0.262000007212161989400556194595992565155029296875f,0.17570000298619259382348900544457137584686279296875f,-0.79879998992085443632049646112136542797088623046875f,0.8339999947547909187051118351519107818603515625f,-0.84989998394846910656497129821218550205230712890625f,0.4524000073432918878069131096708588302135467529296875f,-0.8910000179111960960653959773480892181396484375f,0.12509997850060461388466137577779591083526611328125f,-0.5597999888330698059490941886906512081623077392578125f,-0.4120000391006470863430877216160297393798828125f,0.4539999899864195498366825631819665431976318359375f,0.715199975627661022059555762098170816898345947265625f,-0.53900000011920923270736238919198513031005859375f,-0.211599999481439482451605726964771747589111328125f,0.400800001193582833280970589839853346347808837890625f,-0.4174999928474425470170672269887290894985198974609375f,0.9412999751314521379441657700226642191410064697265625f,-0.115700004178285464906394963691127486526966094970703125f,0.1765000273883341463942997506819665431976318359375f,0.102199995505809937412777799181640148162841796875f,-0.44649997605383429100101011499646119773387908935546875f,-0.98550000590086028751102276146411895751953125f,0.527900005644559744411026258603669703006744384765625f,-0.79179998658299410863037337549030780792236328125f,-1.264099933618307858296248014084994792938232421875f,0.7235000050961968298679494182579219341278076171875f,-1.0961999737203118598927176208235323429107666015625f,-0.1361999643325813025285242474637925624847412109375f,0.64679999537467924852762735099531710147857666015625f,-0.87760002804696579659093913505785167217254638671875f,-0.51409999918341586777614793390966951847076416015625f,0.186900002038478785237884949310682713985443115234375f,0.763800016593933150943485088646411895751953125f,-0.06990001509189625039653037674725055694580078125f,-0.6975000113248821520528508699499070644378662109375f,0.7346000405490400453345500864088535308837890625f,1.072599991017579412755367229692637920379638671875f,-0.5894000138103958619240074767731130123138427734375f,0.10999999754130840301513671875f,0.9693999955564738257862700265832245349884033203125f,-0.3524000211268667470676518860273063182830810546875f,-0.75379998172521567312287515960633754730224609375f,-0.52659995909929335766719304956495761871337890625f,0.4680000061988831561166080064140260219573974609375f,1.3305000069439412158089908189140260219573974609375f,0.1472999956130980336865832214243710041046142578125f,-0.40230000097751617005314983543939888477325439453125f,0.4428999950647352079613483510911464691162109375f,1.55299996387958572086063213646411895751953125f,-0.33519998650550775209921994246542453765869140625f,1.02879998023509955373810953460633754730224609375f,0.93979995985031195004921755753457546234130859375f,-0.5941999997377394038267084397375583648681640625f,0.80969998403191567604153533466160297393798828125f,-0.623399994260072531915284343995153903961181640625f,0.2598000115573402268864811048842966556549072265625f,-0.84220001636743546669094939716160297393798828125f,-0.43579997195005404364565038122236728668212890625f,0.6414000196814537702039160649292171001434326171875f,-0.30009999321550129280922192265279591083526611328125f,0.571599997469782739045740527217276394367218017578125f,0.03219999215304836237550034638843499124050140380859375f,0.68219997181296321286936290562152862548828125f,0.269099966102839260884138639084994792938232421875f,-0.2446999920785426496649961336515843868255615234375f,0.345900009310245337701417156495153903961181640625f,0.73569999322593193635810848718392662703990936279296875f,-0.355500019162892755275606759823858737945556640625f,-0.2714999849200250281455737422220408916473388671875f,0.6400999657973651668640968637191690504550933837890625f,-0.03550001215934717180289226234890520572662353515625f,0.36159999516010277176292220246978104114532470703125f,0.6868999664619568346068945174920372664928436279296875f,0.408000002771615744023847582866437733173370361328125f,-0.038099999675154538891774791409261524677276611328125f,0.68920000443160500669392831696313805878162384033203125f,0.349499990820884676168134319595992565155029296875f,-0.42329999758005154575357664725743234157562255859375f,-0.398100005672871848361182856024242937564849853515625f,0.22299999204278009301560814492404460906982421875f,0.2263999985218045907231498858891427516937255859375f,-0.594200013595818976597229266189970076084136962890625f,0.3755000292211772006112369126640260219573974609375f,0.374200000184774506095664037275128066539764404296875f,-0.1704999888017775244719587135477922856807708740234375f,-0.05250002473592729757001507095992565155029296875f,0.02640000761151295449735698639415204524993896484375f,0.5410999689564113257489452735171653330326080322265625f,-0.45899999177455885757126452517695724964141845703125f,-0.3185999922394753269827560870908200740814208984375f,-0.9119999873563646719532016504672355949878692626953125f,0.10299999904632528568981797434389591217041015625f,0.625099996232986399036235525272786617279052734375f,-0.516799978722632058492081341682933270931243896484375f,0.012000022411346122908071265555918216705322265625f,0.0965999990344048597989967674948275089263916015625f,-0.7855999635398391856000444022356532514095306396484375f,-0.937499985247850275982273160479962825775146484375f,-0.4016999951183795047882085782475769519805908203125f,-0.5408999796569349260977332960464991629123687744140625f,-0.215000011026859283447265625f,0.15080000801086423933838887023739516735076904296875f,0.5533999954521655606498597990139387547969818115234375f,0.1770000069513917306807115892297588288784027099609375f,0.373499993883073366784941526930197142064571380615234375f,0.11519999723434448668513141456060111522674560546875f,-0.1859999824464324547079741023480892181396484375f,-0.0735000027120111809608715702779591083526611328125f,-0.4923999657690529829778824932873249053955078125f,-0.5069999651908876359129862976260483264923095703125f,-0.2685000027120114651779658743180334568023681640625f,-0.103100031936168790025476482696831226348876953125f,-1.780299982738494435352549771778285503387451171875f,-0.0908999863624568860132058034650981426239013671875f,-1.04900002038478845634017488919198513031005859375f,0.84800004792213456994431908242404460906982421875f,0.043999998927116479308097041212022304534912109375f,-0.99600002983212476692642667330801486968994140625f,-0.380399972862004887019793386571109294891357421875f,-0.1896000253498559828813085914589464664459228515625f,1.09350001507997518501724698580801486968994140625f,0.10949998515844328039747779257595539093017578125f,-0.2274999901652332567891789949499070644378662109375f,-0.4318000055074691090339911170303821563720703125f,-1.76299997997283952599900658242404460906982421875f,0.3743999919056886227508584852330386638641357421875f,-0.21530001790523556337575428187847137451171875f,-1.238200026679038501242757774889469146728515625f,-0.670400005793570841206019395031034946441650390625f,0.40460000165700904517507296986877918243408203125f,0.3744999780058861915676970966160297393798828125f,0.478700023937225438430687063373625278472900390625f,0.7871000240087511201636516489088535308837890625f,0.32980000634193373798552784137427806854248046875f,-0.4856000119686125771067963796667754650115966796875f,-0.9858000059247018498354009352624416351318359375f,0.035999991536140640846497262828052043914794921875f,0.91640003368854561216494403197430074214935302734375f,-0.503499988052993785458966158330440521240234375f,0.321100035098939695643593950080685317516326904296875f,-0.2901000295557085717490508613991551101207733154296875f,0.3560999733708796188835776774794794619083404541015625f,0.6782999686717994247686647213413380086421966552734375f,0.03990001304298640416590160384657792747020721435546875f,1.11369998877048459462457685731351375579833984375f,-0.13840006980895935839726007543504238128662109375f,-0.094199988412857038611036841757595539093017578125f,0.4722999670028684704448096454143524169921875f,1.40299999487400128828085144050419330596923828125f,0.2616000315189364044954345445148646831512451171875f,-0.42350000584125524483170011080801486968994140625f,-1.98999999463558197021484375f,0.2270000122785571505801271996460855007171630859375f,-0.31010003199577340637915767729282379150390625f,-0.52279997007250822438351178789162077009677886962890625f,0.2054999811649320662354512023739516735076904296875f,-0.9569999794960022398981891456060111522674560546875f,0.39770000051259979390039234203868545591831207275390625f,0.715000007152557515155422152020037174224853515625f,0.671000027000903997986824833787977695465087890625f,0.6129999972581863243448196953977458178997039794921875f,0.989000025302171792418448603712022304534912109375f,-0.0715999859213827249959649634547531604766845703125f,0.8330000006109472554527428656001575291156768798828125f,-0.512500010430812835693359375f,-0.23300000023841871410468229441903531551361083984375f,-0.7694999953657386537742013388196937739849090576171875f,-0.161499998927116195091002737171947956085205078125f,-0.72560000198483454170172990416176617145538330078125f,0.5360999870613216700121483881957828998565673828125f,1.04599997901916452747173025272786617279052734375f,-0.8434999983906745768535984097979962825775146484375f,0.51059999568909386624682156252674758434295654296875f,-0.930000003278255604755031527020037174224853515625f,0.2822999859273433997941538109444081783294677734375f,-0.4611999859392643674738110348698683083057403564453125f,0.051999998569488781186009873636066913604736328125f,-0.14349999243021027695021984982304275035858154296875f,-0.528200007940828442087877192534506320953369140625f,-0.4720000244975093295352053246460855007171630859375f,0.522999985039234172035094161401502788066864013671875f,0.5195999965161084421794157606200315058231353759765625f,0.75200001304224173281909315846860408782958984375f,0.1708000124812125708473331542336381971836090087890625f,0.3749999816156925902532748295925557613372802734375f,-0.31999998167157206463429020004696212708950042724609375f,0.30439999322816746474273941203136928379535675048828125f,-0.3841999662846331542453981455764733254909515380859375f,-0.700999990344048029555779066868126392364501953125f,-0.323500027000903855878277681767940521240234375f,-0.7703999667897825087692353918100707232952117919921875f,-0.700999983191490372291809762828052043914794921875f,0.74210001029968264418812395888380706310272216796875f,-0.639599959151447183813843366806395351886749267578125f,0.02699996697902662390333716757595539093017578125f,-0.89789999953508381480560274212621152400970458984375f,0.5999999816343188907552530508837662637233734130859375f,0.2629999877214430625826935283839702606201171875f,0.9199000082373611775210520136170089244842529296875f,-0.33540001439899214119577663950622081756591796875f,-0.09179999839216446133605131763033568859100341796875f,-0.019000005446374590434288620599545538425445556640625f,0.05639998362064346792976721189916133880615234375f,-1.99440001306533787328589824028313159942626953125f,0.12020000468492497702754917554557323455810546875f,-0.083899986898899214793345890939235687255859375f,-0.231400043821334833182845613919198513031005859375f,-0.81440003809928906974846540833823382854461669921875f,-0.2167999870300292286628973670303821563720703125f,1.49449999797344190710646216757595539093017578125f,-0.1201000173926356495712752803228795528411865234375f,0.175400007879733976778879878111183643341064453125f,1.728999971508979882628409541212022304534912109375f,0.0437999939441677810236797085963189601898193359375f,0.4508000020503999394350103102624416351318359375f,0.9592000241756437617368646897375583648681640625f,0.78740003148317327941185794770717620849609375f,-1.1192999757051467213386786170303821563720703125f,-0.646799993586540722390054725110530853271484375f,0.8472000187516215419236687012016773223876953125f,-0.06880000236332417529183658189140260219573974609375f,-0.118699998120963545744643852231092751026153564453125f,0.11710000459998848310050334475818090140819549560546875f,-1.0107999989211560176727289217524230480194091796875f,0.1986999911546707409115697373636066913604736328125f,0.662700005280971282672908273525536060333251953125f,-0.36009998431205758606665767729282379150390625f,-0.811400030708313124705455265939235687255859375f,-0.4818000248789786610359442420303821563720703125f,-0.80479999233484278420291957445442676544189453125f,-0.529600012683868470730885746888816356658935546875f,1.04100003772974059046418915386311709880828857421875f,0.736000011019408706403055475675500929355621337890625f,-0.648400022721290714144970479537732899188995361328125f,-0.5296000188678504638772892576525919139385223388671875f,0.133299999517202394372361595742404460906982421875f,0.4823000000834465339494272484444081783294677734375f,-0.500700019228458526043823439977131783962249755859375f,0.783500003792345633968352558440528810024261474609375f,-0.407599988782405997511659734300337731838226318359375f,-0.4294000012189147508223641125368885695934295654296875f,0.6647999845862386791850440204143524169921875f,-0.1223999989986419567600250957184471189975738525390625f,0.27400000976771110572371981106698513031005859375f,0.010400001621246435234979799133725464344024658203125f,0.2648000049635765851263613512855954468250274658203125f,-0.188500010013580521217591012828052043914794921875f,-1.0304000065386293982783172396011650562286376953125f,0.2727000002518293886311084861517883837223052978515625f,1.1320000066161153284838292165659368038177490234375f,-0.382900020098686066916116033098660409450531005859375f,0.2737999907031654078792826112476177513599395751953125f,0.310000020861625813495265902020037174224853515625f,0.79609999417662624665581461158581078052520751953125f,-0.07039999901354310196666119736619293689727783203125f,-0.1969999741315842101130328956060111522674560546875f,0.8995999819874764824589874478988349437713623046875f,-0.43179998255968110587588171256356872618198394775390625f,-1.37249999523162813375165569595992565155029296875f,0.688800006312131785080055124126374721527099609375f,-0.660099985764920571540415039635263383388519287109375f,-0.27850003981590276680435636080801486968994140625f,0.16269998278021802207149448804557323455810546875f,0.748099966520071379960654667229391634464263916015625f,-0.1020000025928020637167037421022541821002960205078125f,-0.009899999929964542477733857595012523233890533447265625f,0.045600000122189510154413483178359456360340118408203125f,-0.12150000041723263421999945421703159809112548828125f,0.528299979847669742838434103759936988353729248046875f,0.5640999946013092181118508960935287177562713623046875f,0.903499987959861527997418306767940521240234375f,-0.5243000178754328999275458045303821563720703125f,0.5036999796807766216488744248636066913604736328125f,-0.7665000379681590203517771442420780658721923828125f,0.229199978429079020969538760255090892314910888671875f,0.45709998679310093194771980051882565021514892578125f,0.1530000042617321032167865269002504646778106689453125f,-0.668400007559359021769296305137686431407928466796875f,-0.315000001192092871082905958246556110680103302001953125f,0.55550001499056822495958840590901672840118408203125f,0.0604000059425831015147423386224545538425445556640625f,-0.0730999970674515164859030846855603158473968505859375f,-0.0150000065565107920662057949812151491641998291015625f,-0.5156999841734768441625647028558887541294097900390625f,-0.412000001288950257194443338448763824999332427978515625f,-0.762499969303608082782375277020037174224853515625f,0.4610000165700913754562861868180334568023681640625f,-1.52579996807575213324525975622236728668212890625f,0.5862000078439706385324825532734394073486328125f,0.3871999829888341793093786691315472126007080078125f,-0.78230003502666978221213867072947323322296142578125f,-0.62710000895857742619909913628362119197845458984375f,0.279500009596347798179749588598497211933135986328125f,-0.337000025093555422017743694595992565155029296875f,-1.962499957680702067364109097979962825775146484375f,0.02750001072883634378740680404007434844970703125f,0.323699981915950729671749286353588104248046875f,-1.703999988198280135520690237171947956085205078125f,0.17340002331733739282526585157029330730438232421875f,-0.7364999908804890083047212101519107818603515625f,-1.09979993392229147275429568253457546234130859375f,0.79980000514984084247771534137427806854248046875f,-0.62330001926124101174764291499741375446319580078125f,-0.06459997870922062901399840484373271465301513671875f,0.355000017434358827728146934532560408115386962890625f,-0.783800016146898315128055401146411895751953125f,0.430100014263391727808993891812860965728759765625f,-0.2397000206887724971238640137016773223876953125f,-0.43960000553131095557546359486877918243408203125f,-0.64759997835159310852759517729282379150390625f,-0.6266000407576566288980757235549390316009521484375f,0.574799999487400015141247422434389591217041015625f,1.251899986839294598439664696343243122100830078125f,0.140700018334388943230806034989655017852783203125f,0.22399998015165323295150301419198513031005859375f,1.54390000239610714061200269497931003570556640625f,-0.3529000114560130185736852581612765789031982421875f,-0.51539998549073917644136599847115576267242431640625f,-0.382199973116815527873768587596714496612548828125f,0.03739999404400595750530555960722267627716064453125f,0.4207000010490418162589776329696178436279296875f,0.071000003159046087830574833787977695465087890625f,0.30740000436305958686489248066209256649017333984375f,-0.45050002467632310754197533242404460906982421875f,-1.96989999184608421955999801866710186004638671875f,0.44490003684759216184829710982739925384521484375f,-0.40649998492002481498275301419198513031005859375f,1.277099993014335410634885192848742008209228515625f,0.22689998445510806135416714823804795742034912109375f,0.875300025653839242067988379858434200286865234375f,-0.0652000033810733459205266626668162643909454345703125f,0.6194999902248383705227752216160297393798828125f,0.74159998100399970866192234097979962825775146484375f,0.9330999645456674596033508350956253707408905029296875f,0.216500002056359885926895003649406135082244873046875f,-0.085400003036856464433412838843651115894317626953125f,0.61910001162588546019804880415904335677623748779296875f,0.2244999777078628966364703956060111522674560546875f,-0.96789997181892406530323569313623011112213134765625f,0.57449998839944604878127165648038499057292938232421875f,-0.4580000231862069171029361314140260219573974609375f,-0.910299987208843308650330072850920259952545166015625f,0.0166000007107853786436635346035473048686981201171875f,-0.4975000104308122672591707669198513031005859375f,-0.1690999857723713972745827049948275089263916015625f,0.555499972298741706566715947701595723628997802734375f,-0.003500008225441053610893504810519516468048095703125f,0.208100011596083600551310155424289405345916748046875f,0.39110000140368927912248864231514744460582733154296875f,0.893000020056962995340654742904007434844970703125f,0.30070000835061083677146598347462713718414306640625f,0.550600000867247363345313715399242937564849853515625f,1.151500019192695845049456693232059478759765625f,-0.47079998789429655658977935672737658023834228515625f,0.5646999874964355736750576397753320634365081787109375f,-0.6174999940395355935152110760100185871124267578125f,0.60069999806880947090803601895458996295928955078125f,-0.66309998004287484807406372055993415415287017822265625f,0.28850003772974019966568448580801486968994140625f,0.65710001119375238687325690989382565021514892578125f,-0.3514999729618433210731609506183303892612457275390625f,0.334999984391033844843832412152551114559173583984375f,-0.0895999926418067360600616666488349437713623046875f,0.43379997528046398969081565155647695064544677734375f,-0.7154999865293500960206074523739516735076904296875f,0.059699996548891132164271766669116914272308349609375f,-0.3848999902814627116498513714759610593318939208984375f,-0.239000043630600345068160095252096652984619140625f,-0.8079999911487103503304751939140260219573974609375f,0.5699999926611780676211083118687383830547332763671875f,-0.0034999763369558678505200077779591083526611328125f,0.835399983143806590391022837138734757900238037109375f,0.031500000268220862409407345694489777088165283203125f,0.0015000039935113562705737422220408916473388671875f,0.323100016140938084419076403719373047351837158203125f,-0.948800018978118941959110088646411895751953125f,-0.15069996826648690557703957892954349517822265625f,0.9081000441551214663604696397669613361358642578125f,0.05319999913051720596968152676708996295928955078125f,-0.33070000550076361633955457364208996295928955078125f,0.24610001323148622542902330678771249949932098388671875f,-0.2745000119805336424860797706060111522674560546875f,-0.2048999928891657162921546841971576213836669921875f,0.27290000713467588155936027760617434978485107421875f,0.242499992400407649029148160479962825775146484375f,-1.3636000166773794006758180330507457256317138671875f,0.234600026690960472564029259956441819667816162109375f,0.02490001330375690713481162674725055694580078125f,-1.5473999691963200575628434307873249053955078125f,-0.1717999986529346045927013619802892208099365234375f,1.00820002813190257029418717138469219207763671875f,0.17440002442747282174195788684301078319549560546875f,0.170999998949467713060812457115389406681060791015625f,-1.33839997272789457127828427474014461040496826171875f,0.27070002846717766686879258486442267894744873046875f,0.608499993175267928080529600265435874462127685546875f,1.370799985659122199876946979202330112457275390625f,-0.748300058078765317759462050162255764007568359375f,-0.55010000427961358582251705229282379150390625f,-0.4100999981701374252907044137828052043914794921875f,0.2046000004649162207215340458787977695465087890625f,0.06920000092983258355161524377763271331787109375f,0.12040001272261147136077852337621152400970458984375f,-1.59880000884532957883266135468147695064544677734375f,-0.421400025790930055080707461456768214702606201171875f,0.95730002571344385842166957445442676544189453125f,0.320399974203109394466082449071109294891357421875f,-1.18380005056858106371464600670151412487030029296875f,0.82260001634955415283911861479282379150390625f,0.063299992066621513231439166702330112457275390625f,0.360900020784139741181206773035228252410888671875f,1.1650999972760676115512978867627680301666259765625f,-0.6653000381708142896286517498083412647247314453125f,-0.57450000989437111087454468361102044582366943359375f,-0.200799994748830812341111595742404460906982421875f,1.1736999926447868602963353623636066913604736328125f,-0.379300015044212290860059511032886803150177001953125f,-0.62070001051127920987937613972462713718414306640625f,0.1214999899491668866602367415907792747020721435546875f,-0.487500030398368977557765902020037174224853515625f,0.41370001261234268241651079733856022357940673828125f,0.526099996002018510665720896213315427303314208984375f,-0.3605000102408231210660005672252736985683441162109375f,-0.102700004200637351470248859186540357768535614013671875f,-0.034899999315291641011071988032199442386627197265625f,-0.15349999794363977656530551030300557613372802734375f,0.037400001531839421886616037227213382720947265625f,0.177500004023313395773442380232154391705989837646484375f,-0.48000003084540399100887952954508364200592041015625f,-0.6927000045359135160794039620668627321720123291015625f,0.004999999441206437467144496622495353221893310546875f,0.21549996566772477990525658242404460906982421875f,-0.88569998301863694933899751049466431140899658203125f,0.2816999704569582707591735015739686787128448486328125f,-0.9724999923631543197188875637948513031005859375f,-0.046599995011091355223697973997332155704498291015625f,-0.8029000042289491378966204138123430311679840087890625f,0.191999988436699187133172017638571560382843017578125f,0.612299991217255534792229809681884944438934326171875f,0.80279996612370052577745127564412541687488555908203125f,-0.2899999856948856091776178800500929355621337890625f,-0.85270001496672609420102162403054535388946533203125f,-0.75759997052848382370626723059103824198246002197265625f,0.21199999439716332716443503159098327159881591796875f,0.212499998062849027036236293497495353221893310546875f,0.250199994365870981294364128189045004546642303466796875f,0.472499994039535575751642682007513940334320068359375f,-1.016400007238984048996144338161684572696685791015625f,0.2736999808356166408174203752423636615276336669921875f,0.8420000256896020829344706726260483264923095703125f,-0.418700006353855069818337142351083457469940185546875f,-0.50260000319927922163287803414277732372283935546875f,-1.083500009864568625062020146287977695465087890625f,-0.00490000734329232301433876273222267627716064453125f,-0.688699995811283383062573193456046283245086669921875f,-0.10599999094009415756545422482304275035858154296875f,-0.776600008273124586821722914464771747589111328125f,-0.6161999842628838042202232827548868954181671142578125f,0.64000001087784763598165227449499070644378662109375f,0.5344999994635581952451275355997495353221893310546875f,0.0276000009268522195071682290290482342243194580078125f,0.276499998629093024504754794179461896419525146484375f,-0.14970000250935555641262908466160297393798828125f,-0.972600011283159204822368337772786617279052734375f,0.7542000387787819448703885427676141262054443359375f,0.650600024187565129096810778719373047351837158203125f,-0.13890000180006012442390783689916133880615234375f,-1.2608999669909479734997148625552654266357421875f,0.10509999682903270468159462325274944305419921875f,-0.336800002825260236249960144050419330596923828125f,-0.675700007605552599443399230949580669403076171875f,-0.3149000144958495894798033987171947956085205078125f,1.0858999923229220030407304875552654266357421875f,-0.5476999543190004260395653545856475830078125f,-0.106499991476535882384268916212022304534912109375f,0.283800012421608016666141338646411895751953125f,2.12399999356269830741439363919198513031005859375f,-0.0933999950051305205533935804851353168487548828125f,-0.8684999931752681590069187222979962825775146484375f,0.3585000028610227928993481327779591083526611328125f,0.6975000038743015551290227449499070644378662109375f,-1.3135999978832899603986561487545259296894073486328125f,0.0497000340998170475614870156277902424335479736328125f,0.63010000781863928498438554015592671930789947509765625f,-0.0397000065326693629685905762016773223876953125f,-0.818300054502486773344571702182292938232421875f,-0.732100028181076112332448246888816356658935546875f,0.597799988549947869387324317358434200286865234375f,-0.8980999723315239435805779066868126392364501953125f,-0.023300002944469877519395595300011336803436279296875f,-0.3722000490009786943801373126916587352752685546875f,-1.2852999839305869755889943917281925678253173828125f,-0.28509999384880035933065300923772156238555908203125f,0.7207000074565410585591962444595992565155029296875f,0.2062999802350997669009302626363933086395263671875f,-0.420900011992454636811089585535228252410888671875f,-1.3008000407934190434389165602624416351318359375f,-0.29279996724128665164244011975824832916259765625f,1.1502000446200373318106358055956661701202392578125f,0.510999994777142863000563011155463755130767822265625f,0.751999969363212894535308805643580853939056396484375f,-0.0539999961704016318009280439582653343677520751953125f,0.116400022959709303904674015939235687255859375f,-1.014300000739098095436929725110530853271484375f,-0.322900023376941902597536682151257991790771484375f,1.62579997701644884955385350622236728668212890625f,-0.65999999150633803512988606598810292780399322509765625f,-0.13700001004338258070447409409098327159881591796875f,0.8120000148117541005632347150822170078754425048828125f,0.44419999631047257793170501827262341976165771484375f,-0.028000007092952472476099501363933086395263671875f,-0.607399986928701451915912912227213382720947265625f,0.2937999857857824803630819587851874530315399169921875f,0.6664999959468840273757450631819665431976318359375f,-0.400100002193450876575298025272786617279052734375f,0.42809999099522821897068070029490627348423004150390625f,-0.66199999499320938411983661353588104248046875f,-0.5397999674499036615316072129644453525543212890625f,-0.677799992237984927356819753185845911502838134765625f,-0.099499990820884676168134319595992565155029296875f,1.04180000029206265566017464152537286281585693359375f,-0.549099989162385337948535379837267100811004638671875f,-0.27349999719858164581864912179298698902130126953125f,0.597199984031915764859377304674126207828521728515625f,0.425499992787837955088292574146180413663387298583984375f,0.14649998670816433143926360571640543639659881591796875f,0.49730000345483416079872540649375878274440765380859375f,0.81769997942745697994126885532750748097896575927734375f,0.99450000572204544369014911353588104248046875f,-0.6756000209093091513068429776467382907867431640625f,0.323900002098083117374471839866600930690765380859375f,-0.40749998703598944160830797045491635799407958984375f,-0.6022999832451343404926547009381465613842010498046875f,-0.2790999934092164291854487601085565984249114990234375f,0.65699999678134890501723930356092751026153564453125f,-0.70709997483491893177642850787378847599029541015625f,0.551400011411308899056393784121610224246978759765625f,0.01700001078844071145113048260100185871124267578125f,-0.174200004804134334079890322755090892314910888671875f,-0.472399992479383967758366225098143331706523895263671875f,-0.543500014185905655494934762828052043914794921875f,0.48920001121163370072508769226260483264923095703125f,0.756100008444487770020714378915727138519287109375f,-0.3119999851584436267870614756247960031032562255859375f,0.2813999887987972403635694718104787170886993408203125f,-0.48999997839331654869710064303944818675518035888671875f,0.34299999308586137658494408242404460906982421875f,-0.6715000059306621693622219027020037174224853515625f,0.15619997744560265573454671539366245269775390625f,-0.944999996423721455585109652020037174224853515625f,-0.426799970042704757133833481930196285247802734375f,0.60220000683069230262844939716160297393798828125f,-0.401500015616417016417472041212022304534912109375f,-0.27710001208782220061266343691386282444000244140625f,-0.4698000011265275333016688819043338298797607421875f,-1.5359999372959141084038492408581078052520751953125f,0.01040000013113040466805614414624869823455810546875f,0.1887000022374094587718218463123776018619537353515625f,-1.3295999818384649504565686584101058542728424072265625f,-0.41500001233071082129555406936560757458209991455078125f,-0.48589998740553852485390962101519107818603515625f,0.4021999984860418209109411691315472126007080078125f,-0.010000010430812888984064557007513940334320068359375f,0.83860002710819259164054528810083866119384765625f,0.7576999930620189616092829965054988861083984375f,-0.9795000218152996040998914395458996295928955078125f,0.479800000679492910649059922434389591217041015625f,0.0256999828696251597648370079696178436279296875f,-0.08210000255107896549588986090384423732757568359375f,0.7247999953523276417399756610393524169921875f,0.33299996145814692027897763182409107685089111328125f,0.052600018920004654177091651945374906063079833984375f,0.77129997993707632986115640960633754730224609375f,0.97289996556043689679427188821136951446533203125f,0.0507000239968302679471889859996736049652099609375f,0.238499997086822901337654911912977695465087890625f,-0.15300001443177446702748056850396096706390380859375f,0.074000012524426228566198915359564125537872314453125f,0.1333000087559224056121820467524230480194091796875f,0.9596000070214270039059556438587605953216552734375f,0.05100000345706945381607511080801486968994140625f,-0.109599966490268485586057067848742008209228515625f,-1.712599993848801460671893437393009662628173828125f,-0.061600026452541811750052147544920444488525390625f,0.6047999951660631978711535339243710041046142578125f,0.9956999721407893133573452359996736049652099609375f,0.18390001014471035745145854889415204524993896484375f,1.03560003469288375299584004096686840057373046875f,0.1998000038832419278378438320942223072052001953125f,0.24840001355707652663795670378021895885467529296875f,-0.6263000450551512443553292541764676570892333984375f,-1.1721999949097625659533150610513985157012939453125f,-0.18740000108480447948977598571218550205230712890625f,0.099699994164705429966488736681640148162841796875f,-0.8964999659210446392165749784908257424831390380859375f,-0.0804999921917914917912639793939888477325439453125f,-0.425300006580352718543736045830883085727691650390625f,-0.312899997262656714980266769998706877231597900390625f,0.34299997520446812160344052244909107685089111328125f,-0.72099996873736404978671998833306133747100830078125f,0.6025999706029896429981818073429167270660400390625f,0.41199998843669904946551696411916054785251617431640625f,-0.88449998607113966553328054942539893090724945068359375f,0.1765999843567611637951131342560984194278717041015625f,0.37099999913573267207311801030300557613372802734375f,0.3054999929368495958925677769002504646778106689453125f,0.28879998839348564043660871902829967439174652099609375f,0.6594999967813492958157439716160297393798828125f,-0.09369999145269414242420680238865315914154052734375f,-0.5337000089988108708638492316822521388530731201171875f,-0.247999990701675443460771930404007434844970703125f,-0.108100007349252624777591336169280111789703369140625f,-0.33759999027252207692839647279470227658748626708984375f,-0.608000001914799526048227562569081783294677734375f,0.052999987125396774700902824406512081623077392578125f,-0.597599978910386742114724256680347025394439697265625f,-0.6434999894499782158163725398480892181396484375f,0.38339999470710761642067154753021895885467529296875f,-0.7400999636739495901593954840791411697864532470703125f,-0.27300000862032192117112572304904460906982421875f,-0.1029000043034553613097159541212022304534912109375f,-0.16399999430775646924729471720638684928417205810546875f,-0.17099999630451190313351617078296840190887451171875f,0.501300003033876340197139143128879368305206298828125f,-0.271599997879564625247184039835701696574687957763671875f,0.1714999870061877373927927692420780658721923828125f,-0.90980001245141028931584514793939888477325439453125f,0.38469996395260130928051012233481742441654205322265625f,-0.69499998062849055457945723901502788066864013671875f,0.47329998714923871006021727225743234157562255859375f,-0.6505999735236169811969375587068498134613037109375f,-1.094500012651085540937856421805918216705322265625f,-0.01220000399947185343307864968664944171905517578125f,-0.8437000068381423911745287114172242581844329833984375f,0.71650000339746444666388924815692007541656494140625f,-0.56409998651742920827700800145976245403289794921875f,0.6971000101506705792786533493199385702610015869140625f,1.02350001731514961278435293934307992458343505859375f,0.647000034630298870297337998636066913604736328125f,-0.8221000371217730418038627249188721179962158203125f,-0.6029000050485127104593630065210163593292236328125f,0.33770001228451729247126422706060111522674560546875f,0.2542999885201453480476629920303821563720703125f,-0.457699999618530029010798898525536060333251953125f,-0.82270001034736583989115388249047100543975830078125f,-0.48090002443492441130956649431027472019195556640625f,-0.86910002764463367697089779539965093135833740234375f,-0.107700008410215541942989148083142936229705810546875f,1.2048000336110593622152009629644453525543212890625f,-0.2499000162839895011757107567973434925079345703125f,-0.79950002270936959547498190659098327159881591796875f,0.091500009972602214247672236524522304534912109375f,1.088100001817196460507375377346761524677276611328125f,-0.5791000270299615504399071141961030662059783935546875f,0.16039999409616001457834499888122081756591796875f,-0.7996999823182822098033284419216215610504150390625f,0.40550000761449300767935710609890520572662353515625f,0.87060001509785678308617207221686840057373046875f,-0.350699977952242392120751901529729366302490234375f,0.3039000049293036909148213453590869903564453125f,-0.9268000028252600941414129920303821563720703125f,1.080100024247169443469829275272786617279052734375f,0.15729998726844751644193820538930594921112060546875f,0.28030002326965330894381622783839702606201171875f,1.1128999670505521635277546010911464691162109375f,0.5237000129103659418206007103435695171356201171875f,0.75860001790672537680393361370079219341278076171875f,-1.550900038293004268297181624802760779857635498046875f,-0.24590001534521610526695667431340552866458892822265625f,0.245399997001886305270090815611183643341064453125f,0.6944999875426294266844706726260483264923095703125f,-0.504700011003017312560814389144070446491241455078125f,0.69129998731315112081574625335633754730224609375f,0.8809999564439063846066346741281449794769287109375f,-0.35339999746382222411966722575016319751739501953125f,-1.075700006115436480058633605949580669403076171875f,0.101000041604041967957527958787977695465087890625f,0.9086000345587734727814677171409130096435546875f,-0.41659999754428866935995756648480892181396484375f,0.705499993979930906107256305404007434844970703125f,-0.03670000867843636882525970577262341976165771484375f,-1.2100000083446502685546875f,-0.55509998494535672808325443838839419186115264892578125f,0.16950000169873202793269229005090892314910888671875f,-0.04500000104308110593365199747495353221893310546875f,0.631700012329220239593041696934960782527923583984375f,0.8934999918341635094520825077779591083526611328125f,-0.572199981796741585782228867174126207828521728515625f,0.0668999988347291907331282345694489777088165283203125f,0.502500006854533776134985600947402417659759521484375f,-0.404400011798739189572415853035636246204376220703125f,0.52230000813007304572010980336926877498626708984375f,0.15749999582767504335834019002504646778106689453125f,-0.6399000024259091645717489882372319698333740234375f,-0.6681000112608070917730174187454394996166229248046875f,0.68600002035126106392226574826054275035858154296875f,0.364000004440546032213887883699499070644378662109375f,-0.12900000043585890807662508450448513031005859375f,-0.646499996468424598106139455921947956085205078125f,-0.31889999673366542509711507591418921947479248046875f,-0.0672999979600310549443520358181558549404144287109375f,-1.04199999395012810055050067603588104248046875f,-0.25369999547600752265452683786861598491668701171875f,-0.6411999999463555699463768178247846662998199462890625f,0.4745000185370447098875956726260483264923095703125f,0.808900004780292558592691420926712453365325927734375f,0.4600999964192509228411154253990389406681060791015625f,-0.90099999749660497627701261080801486968994140625f,0.71439997149109846219516839482821524143218994140625f,-0.5055999930813908083138130677980370819568634033203125f,-0.2310000043138862313440995421842671930789947509765625f,-1.056300023336708360233160419738851487636566162109375f,-0.2382000068232417167024550508358515799045562744140625f,0.47650001040101042093510841368697583675384521484375f,0.1499000030219554968624606772209517657756805419921875f,-0.1902999914184213547940771604771725833415985107421875f,-0.0534999840110543800619780085980892181396484375f,0.213099990361929059190515545196831226348876953125f,-0.5800999709755185751447470465791411697864532470703125f,-0.3955000270605086853947796043939888477325439453125f,0.57250001206994038938091762247495353221893310546875f,0.57589998688399812731830706979962997138500213623046875f,0.3530000079870223572697796043939888477325439453125f,0.444099994117021612538565022987313568592071533203125f,0.1921999993056058286811094149015843868255615234375f,-0.37449997830390913122755591757595539093017578125f,0.020900028085709010383652639575302600860595703125f,0.559899979440867667790371342562139034271240234375f,0.44959996548444092212548639508895576000213623046875f,-0.006799991984665521016495404182933270931243896484375f,0.867400018370151570934467599727213382720947265625f,0.666499979257583419212096487171947956085205078125f,0.058300007712841050988572533242404460906982421875f,1.2629000028133390287621296010911464691162109375f,0.53379993016719851084417314268648624420166015625f,-0.8520000343322760016917527536861598491668701171875f,-0.17360003545284286019523278810083866119384765625f,-1.79349998140335031848735525272786617279052734375f,0.063300024700165380409089266322553157806396484375f,-0.02970000146627427284329314716160297393798828125f,0.95789999536275871605539578013122081756591796875f,0.0176999927639958087866034475155174732208251953125f,0.298000002622604398538896930404007434844970703125f,0.182300001275539358402966172434389591217041015625f,0.1937000144004823454224606393836438655853271484375f,-0.1706999936729669542501142132095992565155029296875f,0.37979999457001678564438407192938029766082763671875f,0.1798000002324580126611408559256233274936676025390625f,-1.539000037074089277666644193232059478759765625f,0.687000014662743296867120079696178436279296875f,0.270000006854534291278469027020037174224853515625f,-0.3069999775588510004808995290659368038177490234375f,-1.0171999698758131813747240812517702579498291015625f,0.01419999109506608903075175476260483264923095703125f,0.313800000575184867557254619896411895751953125f,-0.34229999665617949489160309894941747188568115234375f,-0.0611000028192998367870814035995863378047943115234375f,0.30620000601857899624747005873359739780426025390625f,-0.180500012680888222149633293156512081623077392578125f,-0.32310000980794428837583609492867253720760345458984375f,-1.263599987173080307911732234060764312744140625f,-0.2960999704837803392365458421409130096435546875f,0.506899997270107149915929767303168773651123046875f,0.273800014507770583804813213646411895751953125f,0.124599989438056724111447692848742008209228515625f,0.37300001037120811275826781638897955417633056640625f,-0.692400028502941466967968153767287731170654296875f,-0.929100027346610346512534306384623050689697265625f,-0.2757000073075293045121725299395620822906494140625f,0.7600000202655792236328125f,-0.8244999659731988561617299637873657047748565673828125f,-0.4084999798759820777149798232130706310272216796875f,0.127099989885091968488950442406348884105682373046875f,-0.48429996829181953810206096022739075124263763427734375f,-0.7229999978542327454533733543939888477325439453125f,0.20429998866915699551327634253539144992828369140625f,-0.75979999136626708011732489467249251902103424072265625f,0.179000041842460433372252737171947956085205078125f,-0.25250002682208982918155015795491635799407958984375f,-0.83279997085481927143746361252851784229278564453125f,0.055499973416328174380396376363933086395263671875f,-0.89789997807741173119211453013122081756591796875f,0.4187000015482305048664102287148125469684600830078125f,-0.00649997895956033744369051419198513031005859375f,0.52099999570846566854243064881302416324615478515625f,-0.1450999893039466304145435060490854084491729736328125f,-0.4760000061392783976543796597979962825775146484375f,0.1043000011861323628181708045303821563720703125f,-0.28999999605119219925342122223810292780399322509765625f,-0.111499997511506077074727727449499070644378662109375f,0.29629999178349974986446113689453341066837310791015625f,0.021899999132752423491865556570701301097869873046875f,0.78250000655651064107587444595992565155029296875f,0.7580000090301037829476626939140260219573974609375f,0.95050000757724006916760117746889591217041015625f,0.21449997860193281695728728664107620716094970703125f,-0.4757999931097032231264165602624416351318359375f,0.394499967612326585264526102037052623927593231201171875f,-0.150499996736645602624093953636474907398223876953125f,-0.26409999761879443536116696122917346656322479248046875f,-0.158099998892843707221089744052733294665813446044921875f,0.1980000004991888662431165357702411711215972900390625f,-0.265500003330409517499077765023685060441493988037109375f,-0.12440000140517948334917264219257049262523651123046875f,0.27299998533725755578416283242404460906982421875f,0.2618999980151652806625861558131873607635498046875f,0.86619997088909173044157796539366245269775390625f,-0.6930000041127204468693889793939888477325439453125f,0.139199986922740936989839610760100185871124267578125f,-0.3006000000476836131468871826655231416225433349609375f,0.8799999976158137116044599679298698902130126953125f,-0.4519000069558618548626327537931501865386962890625f,0.72280000441968395019642912302515469491481781005859375f,-0.300500000238418607523271930404007434844970703125f,0.7727000228643419177387841045856475830078125f,-0.151800019067525937543905456550419330596923828125f,0.1137000123143199203923359164036810398101806640625f,0.825700042474270201608987918007187545299530029296875f,-0.81880000169277167287873453460633754730224609375f,-0.83449994134902993891955702565610408782958984375f,0.74990000793933830891546676866710186004638671875f,-0.93500000953674344827959430404007434844970703125f,-0.432100035333633059053681790828704833984375f,0.8771000222206115637391121708787977695465087890625f,-0.5442999913513659748787176795303821563720703125f,0.768100012713670565744905616156756877899169921875f,-0.2777000128805635625894865370355546474456787109375f,-0.08189997432231876928199199028313159942626953125f,-0.83669998364448616712252260185778141021728515625f,-0.7341000589370736406635842286050319671630859375f,1.04240000436305990660912357270717620849609375f,-0.9668000317335128102058661170303821563720703125f,0.07760002245903052653375198133289813995361328125f,-0.1231000018358230363446637056767940521240234375f,1.362100001060962739529713871888816356658935546875f,-0.24030002267360739409696179791353642940521240234375f,1.1047000190496447658006218262016773223876953125f,0.4751000242471690171441878192126750946044921875f,-1.102700032401084939692736952565610408782958984375f,-0.4268000046133995084574053180404007434844970703125f,0.0659000097572803866796675720252096652984619140625f,0.0734999988377094126690280972979962825775146484375f,-1.3296000190913677130311043583787977695465087890625f,-0.1811999595642088678459913353435695171356201171875f,0.74300001692771910910551014239899814128875732421875f,0.205700001347064898027383605949580669403076171875f,-1.643300023508071916467088158242404460906982421875f,-0.1893000183224682331228905241005122661590576171875f,-0.413700000393390610042843036353588104248046875f,-1.37549998444318788415330345742404460906982421875f,0.45210000851154319434499484486877918243408203125f,1.417999977886676532534693251363933086395263671875f,0.137799953532219632279520737938582897186279296875f,-0.803800004673004053756812936626374721527099609375f,0.309000025451183546465472318232059478759765625f,1.325700010287761188010335899889469146728515625f,-0.0917000146389010950542797218076884746551513671875f,0.04239998946189871276146732270717620849609375f,0.7160000118017197934250361868180334568023681640625f,0.34589998740553795641972101293504238128662109375f,0.789799989689886405841434680041857063770294189453125f,1.648600055420399002059639315120875835418701171875f,0.58189996987059744437686958917765878140926361083984375f,-0.242399989145249239452795109173166565597057342529296875f,0.0083999969236554505869207787327468395233154296875f,-0.30189999399185207806795006035827100276947021484375f,0.808300014045834291920300529454834759235382080078125f,0.200300028038025601517801987938582897186279296875f,-0.5454999965503806169664358094451017677783966064453125f,-0.48659996684789696974604567003552801907062530517578125f,-0.88140000116676020525119383819401264190673828125f,-0.568499966949225221668484664405696094036102294921875f,0.2379000061661002973778522573411464691162109375f,0.13830003155469938747046398930251598358154296875f,-0.1908999989539383879133538357564248144626617431640625f,-0.159999999739229792350414527390967123210430145263671875f,0.210400013020634713001300042378716170787811279296875f,-0.41129999290108631981865983107127249240875244140625f,-0.1245999913379549628444920017500407993793487548828125f,-0.9554000257611274804503409541212022304534912109375f,-0.6646999655544760798875358887016773223876953125f,-0.3107999967977403965591065571061335504055023193359375f,0.0119999791979790160212360206060111522674560546875f,0.4689999864101412896388865192420780658721923828125f,-0.43430000137239677115985614364035427570343017578125f,-0.73510002871751822794976760633289813995361328125f,0.515599970245361749476842305739410221576690673828125f,-0.0941000041380524177014876840985380113124847412109375f,0.4498999939322469998614906216971576213836669921875f,-0.52529998057782634202084182106773369014263153076171875f,-0.1668000033095478062961092291516251862049102783203125f,-0.560400014808774216135134338401257991790771484375f,0.218599995089322207686421961625455878674983978271484375f,-0.0408999994195997640389350635814480483531951904296875f,0.276699998713284767148934406577609479427337646484375f,-0.394099983891099958643877698705182410776615142822265625f,-0.082000005517155027945364054176025092601776123046875f,0.539200022182613647458993000327609479427337646484375f,-0.53539997077584278173389975563623011112213134765625f,-0.747199991035461419386365378159098327159881591796875f,-0.509200024771690351599318091757595539093017578125f,-0.40379997591376337595647783018648624420166015625f,0.115099997797608377680944613530300557613372802734375f,-0.6275000005960464921628272350062616169452667236328125f,-0.313599988216161751353183717583306133747100830078125f,0.48819999911189082553164553246460855007171630859375f,-0.0799000040650366116778968716971576213836669921875f,0.1616000187039379198949973215349018573760986328125f,0.4021000074312091054906659337575547397136688232421875f,0.60079998878836615716636515571735799312591552734375f,0.2661999796807765772399534398573450744152069091796875f,-0.533199992071092143675059560337103903293609619140625f,-0.2137000030755995538811475853435695171356201171875f,-0.45640003368854564769208081997931003570556640625f,-0.3464000083565712539979131179279647767543792724609375f,-0.42550001037120832592108854441903531551361083984375f,-0.3204000124990942222069634226500056684017181396484375f,0.10589999995976684399323630714206956326961517333984375f,-0.68650003826618188895736238919198513031005859375f,-0.140800010991096513635056908242404460906982421875f,0.660400009593367531834928740863688290119171142578125f,-0.27920000182390225518247461877763271331787109375f,0.229400030946731448011632892303168773651123046875f,0.68559999807327987042526729055680334568023681640625f,0.5565000036954881323936206172220408916473388671875f,0.4653999941706654652762154000811278820037841796875f,-0.2845000110864639442098678046022541821002960205078125f,0.21279999734163279612175756483338773250579833984375f,0.000399985229968979894010772113688290119171142578125f,-0.32979999874234200429867769344127736985683441162109375f,0.490299975883960570399722200818359851837158203125f,0.176399963653087521464613018906675279140472412109375f,-0.5729999901801348105578881586552597582340240478515625f,-0.22009997656345348104878212325274944305419921875f,-0.08259997447729094943724703625775873661041259765625f,0.27429999075531963370622179354541003704071044921875f,0.35869999415352971094961276321555487811565399169921875f,0.422600001336634056858798658140585757791996002197265625f,0.06339999684914943145486176945269107818603515625f,-0.6304999799728392417819122783839702606201171875f,-0.53159999217987063957480131648480892181396484375f,0.20099998803436758265661410405300557613372802734375f,-0.2828999816536903466612784541212022304534912109375f,-0.21839998099803903386373349349014461040496826171875f,0.174599996888637587488801727886311709880828857421875f,0.45849999034404742559445367078296840190887451171875f,-0.8882000210911034088212545611895620822906494140625f,-0.0094000058382748274965479140519164502620697021484375f,0.047299993489682723435407751821912825107574462890625f,0.6780999978870152489207612234167754650115966796875f,-0.36599999794364013183667339035309851169586181640625f,0.75520000759065109008361105225048959255218505859375f,1.20680006272792894606027402915060520172119140625f,0.068100003847479939622644451446831226348876953125f,-0.2273000050753353484811469797932659275829792022705078125f,-0.9241000158727172841821584370336495339870452880859375f,0.653200003284215569010484614409506320953369140625f,-0.7708999998852608115385010023601353168487548828125f,-0.921700036990642956880037672817707061767578125f,0.1754000027388336047096117908949963748455047607421875f,-0.65720000697970359926358696611714549362659454345703125f,-0.5848000403165822813633667465182952582836151123046875f,-0.44219996615052270527712607872672379016876220703125f,-0.6262999985262751589942809005151502788066864013671875f,-0.3403000050902367235039491788484156131744384765625f,-0.719099988901614750602675485424697399139404296875f,0.368299982455372987288910735514946281909942626953125f,0.772299990248679790738606243394315242767333984375f,-0.381899970895052121022672508843243122100830078125f,-0.4598999901324509753663960509584285318851470947265625f,-1.2795000295639038512263141456060111522674560546875f,0.1993999998033046949785784818232059478759765625f,-0.313599998274445457280279470069217495620250701904296875f,-0.670400014734268356875190875143744051456451416015625f,0.10840001586675640510293305851519107818603515625f,-0.79630000083595486870535751222632825374603271484375f,-1.689500034332275912873910783673636615276336669921875f,-0.03550001171231276231310403090901672840118408203125f,0.49670001195669144511413151121814735233783721923828125f,0.97390003070831365761250708601437509059906005859375f,-0.45589997326806253052922102142474614083766937255859375f,0.425200002282112687890247570976498536765575408935546875f,0.2672000302441421837329471600241959095001220703125f,0.4055999838054180628432732191868126392364501953125f,-0.253899988277256571933548912056721746921539306640625f,0.254499991714954632016087998636066913604736328125f,0.160999977976084007735835257335565984249114990234375f,0.296700006406009020309966217610053718090057373046875f,0.744300016236305594929945073090493679046630859375f,-0.083099998557567555934610936674289405345916748046875f,0.28979999710321424633008291493752039968967437744140625f,-0.3981000021338463312758904066868126392364501953125f,0.2274000347614286710040687466971576213836669921875f,0.8465999956443905904990288036060519516468048095703125f,-0.78330001367330570616331897326745092868804931640625f,-0.1525999795436856487640397972427308559417724609375f,0.80120000735968355254357220474048517644405364990234375f,0.13860001667737975594718591310083866119384765625f,-0.2416000276446350625292325275950133800506591796875f,-0.5753000240519643426750917569734156131744384765625f,-0.7009000003695486924470969825051724910736083984375f,-0.14579998714923902980444836430251598358154296875f,0.6827999847128987109812214839621447026729583740234375f,-0.3082000132679940662683293339796364307403564453125f,-0.025799986553192155724900658242404460906982421875f,0.4980000091418623764383255547727458178997039794921875f,0.757300004255771597172497422434389591217041015625f,0.5551999993205072314594872295856475830078125f,-0.44089999958723780082436860539019107818603515625f,-0.49339999798536293695860877051018178462982177734375f,-0.17059999080896393053308202070184051990509033203125f,0.41929999209642410118448196953977458178997039794921875f,0.6018000126600266952436868450604379177093505859375f,0.4758000005602838200502446852624416351318359375f,-0.3843000086367129863873515205341391265392303466796875f,1.2077999824404717088555116788484156131744384765625f,0.8097999854803088481958184274844825267791748046875f,-0.6093999838963151116644212379469536244869232177734375f,-0.3302000055044889137434438453055918216705322265625f,-0.19240000175535654225456028143526054918766021728515625f,0.163600003974139707185742054207366891205310821533203125f,-0.36670000478550790479204124494572170078754425048828125f,-0.277800010715424983143151393960579298436641693115234375f,0.0488999987639486821677792249829508364200592041015625f,-0.424700000870227967197934049181640148162841796875f,-0.24439999011755020319469622336328029632568359375f,0.368899998857080912983974485541693866252899169921875f,-0.53369999681711188799226874834857881069183349609375f,-0.04539997777938875600511892116628587245941162109375f,0.68499998901039360621467722012312151491641998291015625f,0.808199989277124331010782043449580669403076171875f,0.25759997466355588624509209694224409759044647216796875f,-0.77209998489320275094627277212566696107387542724609375f,0.5197999962091444103862158954143524169921875f,0.4963999950945374717292679633828811347484588623046875f,-0.83349998304247829850055495626293122768402099609375f,0.40199998340755715986460927524603903293609619140625f,0.92879999364614462820100015960633754730224609375f,-0.50869999309182123425898680579848587512969970703125f,-0.4577999945476651788567323819734156131744384765625f,0.00659998621940616203573881648480892181396484375f,0.582200007426738608273808495141565799713134765625f,-0.7875000064447517100774120990536175668239593505859375f,-1.2939000709414489875825893250294029712677001953125f,-0.44409997414946555949200046597979962825775146484375f,-0.5672000029936430376409361997502855956554412841796875f,-0.2356000450491908537742347107268869876861572265625f,-0.2239000044822692103707595379091799259185791015625f,0.778899972295761244822642765939235687255859375f,1.4765000227689739631387055851519107818603515625f,-0.35439998728632904345658971578814089298248291015625f,-0.3969999809861184392190125436172820627689361572265625f,-1.3742000140428540788661848637275397777557373046875f,-0.173299990725517272238676014239899814128875732421875f,-0.29739999769479030788943418883718550205230712890625f,0.00339998189210888313027680851519107818603515625f,-0.37249997913837429308614446199499070644378662109375f,-0.8009999629631641138161057824618183076381683349609375f,-1.8474000213503831702155366656370460987091064453125f,-0.36049998354911849673953838646411895751953125f,0.5589000044077632178840531196328811347484588623046875f,-0.3386999512195583150742095313034951686859130859375f,0.030200000624358569467631241423077881336212158203125f,-0.39039998359084149281983400214812718331813812255859375f,-0.27919999385178051465317139445687644183635711669921875f,0.1421000052146613457093593524405150674283504486083984375f,-0.835699984415620544808689373894594609737396240234375f,-0.9193000244133173737282049842178821563720703125f,0.0801999884426596310049717430956661701202392578125f,0.529399967393279435867725624120794236660003662109375f,1.232199991631507174361104262061417102813720703125f,-0.44029998616576193626315216533839702606201171875f,0.17399998719245213241180181285017170011997222900390625f,0.74399998641014075673183469916693866252899169921875f,0.308599991252273275765816151761100627481937408447265625f,0.08370000175312153345430488116107881069183349609375f,-0.624299982056766733506947275600396096706390380859375f,-0.50699998635053589168819598853588104248046875f,-0.5038000038534402591494654188863933086395263671875f,-1.1379999710321424544190449523739516735076904296875f,-0.74759997894763952075436463928781449794769287109375f,0.5831999795168638645037617607158608734607696533203125f,0.8452000207781793506001122295856475830078125f,0.42099999451637248881752384477294981479644775390625f,-0.655000013299286365509033203125f,0.151800007891655042158163269050419330596923828125f,0.15400000488758092132002275320701301097869873046875f,-0.061600005069375018962318790727294981479644775390625f,-0.3527999970436095367176676518283784389495849609375f,-0.2397999929308891609025522484444081783294677734375f,0.2682999986231326960961496297386474907398223876953125f,0.02119999562501906353872982435859739780426025390625f,0.1802000049829484140673230285756289958953857421875f,0.12640000109225513202915180954732932150363922119140625f,0.0385999913454053711348024080507457256317138671875f,-0.1280000139474868348088421043939888477325439453125f,-0.2313999918907880815055477796704508364200592041015625f,0.292800000694394046973911827080883085727691650390625f,0.3304000119030476856352152026374824345111846923828125f,0.02780000315308572833572497984278015792369842529296875f,-0.519200014936924247876959270797669887542724609375f,-0.05719999014139176551907439716160297393798828125f,0.535700004588067546507090810337103903293609619140625f,-0.50569999212697147594752777877147309482097625732421875f,-0.62560001260191189231818498228676617145538330078125f,-0.114500002574175585579041580785997211933135986328125f,0.5143000087857245716804754920303821563720703125f,0.07319999449253078438459851895458996295928955078125f,-0.4841999971300363636572683390113525092601776123046875f,1.0993000189185142545511553180404007434844970703125f,0.70380001480579412742599743069149553775787353515625f,-0.5537999974086880161650014997576363384723663330078125f,-0.04030001268982896789339065435342490673065185546875f,0.070999989748000924549842238775454461574554443359375f,0.019299999770522102693348642787896096706390380859375f,0.56100000882148748360123136080801486968994140625f,0.319200004804134351843458716757595539093017578125f,-0.2739000007197260888602841077954508364200592041015625f,-0.640299994063377564401662311865948140621185302734375f,-0.58159999605417223023096084943972527980804443359375f,0.161000004798173945719241828555823303759098052978515625f,-0.082200036633015116649403353221714496612548828125f,0.465100007259845682483501150272786617279052734375f,0.30659998592138304474019605549983680248260498046875f,-0.58219997792243971534986712867976166307926177978515625f,0.23059997799396558804119194974191486835479736328125f,0.01670000748634326015462647774256765842437744140625f,-0.03609998896121979772289023458142764866352081298828125f,-0.87050001096725448945790049037896096706390380859375f,-0.16219999341964719263842198415659368038177490234375f,-0.22199998642504226165073077936540357768535614013671875f,-0.50280000598430607539057746180333197116851806640625f,0.1404999909996984541749043273739516735076904296875f,0.4619999922737478836864966069697402417659759521484375f,1.0644000223040581687428129953332245349884033203125f,-0.35049998712539665035592406638897955417633056640625f,-0.323899989506602370425980552681721746921539306640625f,-0.550300008070468749110659700818359851837158203125f,0.2815999900937082855989501695148646831512451171875f,-0.2372000065326689277611649231403134763240814208984375f,-0.3772000142812732548236454022116959095001220703125f,-0.34559999714195710485142853940487839281558990478515625f,0.29699999842047686282597851459286175668239593505859375f,0.14340000938475139236061295378021895885467529296875f,-0.437999988913535531764864572323858737945556640625f,-0.6025000058487055554223843500949442386627197265625f,-0.4649000404238705641546403057873249053955078125f,0.70439997910931706481818537213257513940334320068359375f,-0.5427999872274698045515606281696818768978118896484375f,-0.2840000110156832846541874459944665431976318359375f,0.409599977666139647425325165386311709880828857421875f,0.78169998271316298854571869014762341976165771484375f,0.7429000218868253568871296010911464691162109375f,-0.2505000090301034987305683898739516735076904296875f,0.76400001036375730922856064353254623711109161376953125f,0.09160003777742442565568126155994832515716552734375f,-0.2162999808311460991916419516201131045818328857421875f,-0.7612999994575975204469386881100945174694061279296875f,-1.1521000426352028256360426894389092922210693359375f,-0.032200005042553048184572617174126207828521728515625f,0.74789999167472120955579839574056677520275115966796875f,0.60630002613067635053312187665142118930816650390625f,0.495799969565868803300645595300011336803436279296875f,-0.231499994270503606230704463087022304534912109375f,0.306499998182058419615714228712022304534912109375f,-0.74709999301433516194492767681367695331573486328125f,-0.376299998824298376209895877764211036264896392822265625f,0.37189999963566677809723159953136928379535675048828125f,0.393000007949769469828282808521180413663387298583984375f,0.0380999988369643549646070823655463755130767822265625f,-0.356700009460747313738693264895118772983551025390625f,-0.190200004647672338276009895707829855382442474365234375f,0.2124000002838671019578242749048513360321521759033203125f,0.5853000027060506482712298748083412647247314453125f,0.096799993288517072187460144050419330596923828125f,-0.5338999893203377755668270765454508364200592041015625f,0.005199991795420633167168489308096468448638916015625f,0.2622000084698201671784545396803878247737884521484375f,0.2421000026255846204747967931325547397136688232421875f,-0.46080000294446943343018574523739516735076904296875f,0.1284000283837318789892378845252096652984619140625f,0.76409999735802414111418556785793043673038482666015625f,-0.32609999341294149122205681123887188732624053955078125f,-0.377200009177625028566893661263748072087764739990234375f,-0.0596000041715800676911385380662977695465087890625f,-0.24800000172853486901658470742404460906982421875f,-0.31280001522302658401031294488348066806793212890625f,-0.133300004173815267449754173867404460906982421875f,-1.121000007629394445984871708787977695465087890625f,-0.6849999973177904877275068429298698902130126953125f,0.60430000573098663352311632479541003704071044921875f,-1.1740999850273130533651055884547531604766845703125f,-0.7501999868035316154646352515555918216705322265625f,0.6274999871104955939671299347537569701671600341796875f,0.447400031185150481860546278767287731170654296875f,-0.0740000090599064463958711712621152400970458984375f,-0.60160002041757110191610991023480892181396484375f,-0.8593000019311904935648271930404007434844970703125f,-0.79380001838207281394943493069149553775787353515625f,0.14819999281615015007673719082958996295928955078125f,-0.0509000242114066026033469825051724910736083984375f,0.2533999988794324309537842054851353168487548828125f,0.4060000037923454652144528154167346656322479248046875f,-0.26320001073479648567854383145458996295928955078125f,-0.090400009071827103213081500143744051456451416015625f,0.1551999975323676839167319485568441450595855712890625f,-0.222600006291270080538424736005254089832305908203125f,-0.3696000268399719512757428674376569688320159912109375f,-0.165600006380677289286040831939317286014556884765625f,0.7671000004649162207215340458787977695465087890625f,-0.160399992829561188756315459613688290119171142578125f,-0.37609998655840792824989193832152523100376129150390625f,0.595899997072666831599008219200186431407928466796875f,0.52430002918168927550368607626296579837799072265625f,0.0025000052340329315114786368212662637233734130859375f,-0.316800007817148954192276733010658062994480133056640625f,-0.142800022040307894588551107517560012638568878173828125f,0.43999997034668947293312157853506505489349365234375f,0.9023999758273364424354667789884842932224273681640625f,0.8628000250577922969341670977883040904998779296875f,-0.52849998162686817604338784803985618054866790771484375f,-0.3111999892547727686320513384998776018619537353515625f,-0.531200005832314303688690415583550930023193359375f,-0.717199982988834250363652245141565799713134765625f,0.0411000011429190070799677414470352232456207275390625f,0.4442999989509583969038430950604379177093505859375f,0.39759999472424356614652651842334307730197906494140625f,-0.452099975598603764925798031981685198843479156494140625f,-0.34529999121352972224485711194574832916259765625f,0.33579999698400531116249112528748810291290283203125f,-0.73450000806897841609810484442277811467647552490234375f,-0.86090005460977625517671185662038624286651611328125f,0.19149999356269820083298327517695724964141845703125f,-0.7611999674245719660348186152987182140350341796875f,-0.07159998472928918999969027936458587646484375f,0.275099991017579981189555837772786617279052734375f,-0.5874999789893629031922728245262987911701202392578125f,0.2544999896287922780402368516661226749420166015625f,0.62139998809099150633983299485407769680023193359375f,0.5131999966531992640739190392196178436279296875f,0.4132000082015991893058526329696178436279296875f,0.675499990552663742704453397891484200954437255859375f,-0.6626000075206157635676618156139738857746124267578125f,-0.1294000380992894605469700763933360576629638671875f,-0.53729997043013622004536955500952899456024169921875f,0.348799981538951531234715730533935129642486572265625f,0.538000012457370502261255751363933086395263671875f,-0.1147999823510648109703424779581837356090545654296875f,-0.6187999964773653882588178021251223981380462646484375f,-0.7084000264465810658975897240452468395233154296875f,0.49400000570714419634299474637373350560665130615234375f,-0.298200006860494415406037660432048141956329345703125f,-0.1998000140160325344140801462344825267791748046875f,0.29239997560381869590173664619214832782745361328125f,0.6813000124588608485964869032613933086395263671875f,-1.27719999669790240659494884312152862548828125f,-0.973400006628036607025933335535228252410888671875f,0.52399999017268417134118863032199442386627197265625f,-0.74889998332262042168849802692420780658721923828125f,-0.606399991667270654716048738919198513031005859375f,0.25329999158233407197116093811928294599056243896484375f,0.812499995827674847959087855997495353221893310546875f,0.3059999781250954953293330618180334568023681640625f,-0.7331999925553798203026190094533376395702362060546875f,-0.086599992030858885527777601964771747589111328125f,-0.1414000055253505916397216424229554831981658935546875f,-0.11780000699013470910614387321402318775653839111328125f,-0.68250000752508643842020319425500929355621337890625f,-0.42800000821054007271726504768594168126583099365234375f,0.305599995018541792379807020552107132971286773681640625f,-0.95719998179674126159710567662841640412807464599609375f,-0.5661999837785962608194267886574380099773406982421875f,0.615699980485439279931370037957094609737396240234375f,-0.527100015068054261746510746888816356658935546875f,-0.226400003886222833671126863919198513031005859375f,0.357200003477931016249158346909098327159881591796875f,0.58100000196695322784989912179298698902130126953125f,-0.02160002406835559440878569148480892181396484375f,-0.74549999483674767031260444127838127315044403076171875f,0.8589999932646747993203462101519107818603515625f,0.76480001558065424660526332445442676544189453125f,-0.159099987523257713206703556352294981479644775390625f,-0.877200020539760760840408693184144794940948486328125f,-0.65760000755786851556194960721768438816070556640625f,0.390300018166005624475900503966840915381908416748046875f,-0.743600039625167852364029386080801486968994140625f,-0.2216000124454495789905195124447345733642578125f,0.611600015127658824809486759477294981479644775390625f,-0.7762999954342841846255396376363933086395263671875f,-0.20539997539520271629953640513122081756591796875f,0.7621999913707375728932902347878552973270416259765625f,-0.4771999951705334641616218505077995359897613525390625f,-0.47740001233518103429531720394152216613292694091796875f,-0.0305000042244791824685989922727458178997039794921875f,0.360600022697448618913540485664270818233489990234375f,0.0050000044703484292085704510100185871124267578125f,-0.443600003825128041956560309699852950870990753173828125f,-0.96710001149177582391303076292388141155242919921875f,0.95290001801252355306814934010617434978485107421875f,0.6166999916911120038776061846874654293060302734375f,-0.5134999869912864056686885305680334568023681640625f,0.4985000010728839470175444148480892181396484375f,0.427999978929758384538217796944081783294677734375f,-0.392599992060661318049596957280300557613372802734375f,0.157399980223178914684467599727213382720947265625f,0.213999998629093024504754794179461896419525146484375f,0.01730000511258833473249296730500645935535430908203125f,-0.73909998905062668228538313996978104114532470703125f,0.1703000015884637008412028080783784389495849609375f,-0.4890999842077494719205787987448275089263916015625f,-0.6539000138700006292680200203903950750827789306640625f,0.12300000742822891908190285903401672840118408203125f,-0.79590000115185954854268857161514461040496826171875f,-0.55470004251897375713298288246733136475086212158203125f,0.15049999759346233130230530150583945214748382568359375f,-0.12699999909102910322644675034098327159881591796875f,0.74259998464733367740109315491281449794769287109375f,-0.1101999899327754661726430640555918216705322265625f,-0.590099997015297361713237478397786617279052734375f,-0.408900019681453841258189640939235687255859375f,0.64459998049736011438426430686376988887786865234375f,0.3450999900117517338316019959165714681148529052734375f,0.0275000131130216374231167719699442386627197265625f,-0.61179999924898087471092367195524275302886962890625f,0.8050999749615790346268795474315993487834930419921875f,0.3547000068306918052485343650914728641510009765625f,0.407399979776144061105469518224708735942840576171875f,-0.2324999962374569850709349339012987911701202392578125f,0.5259000216782094838663397240452468395233154296875f,-0.3499999731779102063455866300500929355621337890625f,-0.849999986886978309286178046022541821002960205078125f,-0.621200000095367244057342759333550930023193359375f,-0.3878999916374681333763874135911464691162109375f,-0.25899999702721832051111050532199442386627197265625f,0.31659998115301135612753569148480892181396484375f,0.425699996839463412623416616042959503829479217529296875f,-0.84399996718764336289808625224395655095577239990234375f,-0.8000000023469322485425436752848327159881591796875f,0.36309997456669851345623101224191486835479736328125f,0.20310000094175340024094111868180334568023681640625f,-0.003300004285573976403611595742404460906982421875f,0.45319996871352241640806823852472007274627685546875f,0.23140000280588868264430857379920780658721923828125f,-0.637100007021426772979566521826200187206268310546875f,-0.04079999132156419960892890230752527713775634765625f,0.7652999838933347565017584201996214687824249267578125f,-0.28230000589489900875150851788930594921112060546875f,0.018799996626376991315510167623870074748992919921875f,0.306399990475177741444667844916693866252899169921875f,0.2543999997287986758465194725431501865386962890625f,0.3724000146448613879357480982434935867786407470703125f,0.065900003759563008376431980650522746145725250244140625f,0.5606999924063682527730634319595992565155029296875f,0.2327999884009361863945741788484156131744384765625f,-0.3848999907657504326863318055984564125537872314453125f,0.463999991774559195079064011224545538425445556640625f,-0.02040003172159199351654024212621152400970458984375f,-0.610799995270371542943621534504927694797515869140625f,0.4997000256061554512854172571678645908832550048828125f,0.111999999016523243966503287083469331264495849609375f,-0.544600011938810357747797752381302416324615478515625f,-0.24690001902580238635209752828814089298248291015625f,0.0571999922275541194949255441315472126007080078125f,0.34959999808073038973788015937316231429576873779296875f,-0.23309998916983598604701910517178475856781005859375f,-0.363000002324581405588332927436567842960357666015625f,-0.0312000065401196735592748154886066913604736328125f,0.2868999856472012055519371642731130123138427734375f,-0.1666000276446339967151288874447345733642578125f,-0.5986999815434217442344788651098497211933135986328125f,0.279200013446807560058005037717521190643310546875f,-0.1512000030755995538811475853435695171356201171875f,-0.49859999484717842932468556682579219341278076171875f,-0.929399980282783477747443612315692007541656494140625f,-0.56579997701644924035235817427746951580047607421875f,0.6188999852597714124158301274292171001434326171875f,-0.44760001173019414721920838928781449794769287109375f,-0.0651999954462052500048230285756289958953857421875f,0.45279999950230120919769660758902318775653839111328125f,0.16550000508129603105089699965901672840118408203125f,0.04339999522864823600087902377708815038204193115234375f,-0.209900002686679354457766066843760199844837188720703125f,-0.053799988877773596840370373683981597423553466796875f,-0.18079999072551711236656046821735799312591552734375f,-0.102699995408952193809426489679026417434215545654296875f,-0.4220999718546867285340340458787977695465087890625f,-0.67130001011192774740266031585633754730224609375f,-0.572799959194660868178061718936078250408172607421875f,-0.24159999970495693588645735871978104114532470703125f,0.192399987822771123546772287227213382720947265625f,-0.3495999664902693648826925709727220237255096435546875f,-0.60149997750669736973350154585205018520355224609375f,-0.684300003570317727508154348470270633697509765625f,-0.48599999444186636754494656997849233448505401611328125f,-0.104699997480213635725476706284098327159881591796875f,-0.4503000138074160219048280850984156131744384765625f,0.13439999503493282873023417778313159942626953125f,0.6551999882191419377619467923068441450595855712890625f,0.4788000332832336169985865126363933086395263671875f,0.517399998551606454810780633124522864818572998046875f,0.19659999977946274185569563996978104114532470703125f,-0.099399972385167956190343829803168773651123046875f,0.2801999894112350286690116263343952596187591552734375f,-0.04069999229460952161474551758146844804286956787109375f,-0.52030001514852042276970678358338773250579833984375f,-0.3755999671161180941680868272669613361358642578125f,-0.2888999975904822381522762952954508364200592041015625f,-0.2397000005722045301581601961515843868255615234375f,0.5048999817132948653153334817034192383289337158203125f,0.4535999837830664294102689382270909845829010009765625f,0.9684000061810014159391357679851353168487548828125f,0.610499976247549369645639671944081783294677734375f,-0.19330000666975966083782623172737658023834228515625f,-0.52010000278949775065484573133289813995361328125f,0.59439996240139070238228669040836393833160400390625f,0.288999988272786278997728004469536244869232177734375f,0.228199995088576912394273676909506320953369140625f,0.189899983948469408545633996254764497280120849609375f,0.13739998987168089428223538561724126338958740234375f,1.126199984151124766640350571833550930023193359375f,-0.3433999836802481553377219825051724910736083984375f,-0.4602000020027159354896184595418162643909454345703125f,-0.3066000258564951508333251695148646831512451171875f,0.069100001679360811834840205847285687923431396484375f,-0.25729998771548279190568564445129595696926116943359375f,-0.54970000392496565222444360188092105090618133544921875f,0.312999968498945957406931484001688659191131591796875f,0.19749999035149823356505294214002788066864013671875f,0.383899991816281982437430997379124164581298828125f,0.757499964088201860334947923547588288784027099609375f,-0.410500002212822401315150955269928090274333953857421875f,0.1283999839782712371061279554851353168487548828125f,0.40340001676082604120665564551018178462982177734375f,0.3796000095918774519532234990037977695465087890625f,0.63399999892711633719954988919198513031005859375f,0.3859999843835826283111600787378847599029541015625f,-0.479000007420778306510555921704508364200592041015625f,0.900600024485588068756669599679298698902130126953125f,0.75200001972913721459690350457094609737396240234375f,-0.333200014460086790801796041705529205501079559326171875f,-0.34700000900030136818941173260100185871124267578125f,-0.3890000108480453633319484652020037174224853515625f,0.06800000631064173628459457177086733281612396240234375f,-0.361499987006187240012877737171947956085205078125f,0.1572000345468520521308164461515843868255615234375f,0.693599987173080467783847780083306133747100830078125f,1.0924999773502346300801946199499070644378662109375f,0.90020000528097199321564403362572193145751953125f,-0.3076999731689691319758139798068441450595855712890625f,-0.96020000528097160241713936557061970233917236328125f,-0.698400000965595069146729656495153903961181640625f,0.42750000428408385655387746737687848508358001708984375f,0.244100001716613945745848468504846096038818359375f,-0.0294000145554544900505788973532617092132568359375f,-0.302100003817677542627961884136311709880828857421875f,-0.7502000294208528430317528545856475830078125f,-0.30680000491142234153585377498529851436614990234375f,0.525800016355514543420213158242404460906982421875f,0.7683000156104566880088668767712078988552093505859375f,0.47699999495595657439395154142403043806552886962890625f,-0.4794000161200762111235462725744582712650299072265625f,-0.25769999013766675677317152803880162537097930908203125f,-0.325399994580447493586206064719590358436107635498046875f,-0.053799996645003511641647264696075581014156341552734375f,-0.57029999227523830995778553187847137451171875f,-0.4817999825596803731286854599602520465850830078125f,0.258300012034177850495098027749918401241302490234375f,-0.6344999961853030612246584496460855007171630859375f,-0.6942000101685525947914356947876513004302978515625f,-0.0261000009566545454475772203295491635799407958984375f,-0.1547000256061554779307698481716215610504150390625f,0.319000015914440382402972318232059478759765625f,0.6287000144004821766685608963598497211933135986328125f,0.1678999891042707304222858510911464691162109375f,-0.2235000158250333157639033743180334568023681640625f,-0.640499990701675070425835656351409852504730224609375f,0.634499970182776706906224717386066913604736328125f,0.79530000211000384524595574475824832916259765625f,-0.14699998009204851001641145558096468448638916015625f,-0.4720000084787603356062390957958996295928955078125f,-1.352000051021576609855401329696178436279296875f,-0.576399986749887016657112326356582343578338623046875f,0.1110000020414592114548213430680334568023681640625f,-0.750600029104948163194421795196831226348876953125f,0.4344999863505361048510167165659368038177490234375f,0.3004999874234199896960717524052597582340240478515625f,0.8365000200867651614089481881819665431976318359375f,-0.665199983972311059687854140065610408782958984375f,0.2349000068590043621696850095759145915508270263671875f,0.6513000061631206705214935936965048313140869140625f,-0.6416999975025650115867392742075026035308837890625f,0.867099983850121436290692145121283829212188720703125f,0.221900038099288821058507892303168773651123046875f,0.396199983406066991165062063373625278472900390625f,-0.413499994404614046317192332935519516468048095703125f,0.326499997735023583800284541212022304534912109375f,0.321499999374150746689338120631873607635498046875f,0.3714999914020300497696780439582653343677520751953125f,0.3647000154733657240058164461515843868255615234375f,-0.18739999184608446824995553470216691493988037109375f,-0.83979998697042479438579221096006222069263458251953125f,-0.58460000791549671106395180686376988887786865234375f,-0.168699979662150501003026192847755737602710723876953125f,0.0312000058136880920045541643048636615276336669921875f,-0.709200011043995548476459589437581598758697509765625f,0.3756999708555646133589789315010420978069305419921875f,-0.196800001279264613618380508341942913830280303955078125f,-0.020400018962472898920168518088757991790771484375f,0.39249995753169120149550508358515799045562744140625f,0.28650000362098193562587766791693866252899169921875f,1.02690002617836029230602434836328029632568359375f,0.708699971634150216459602233953773975372314453125f,-0.5799000147938724314400360526633448898792266845703125f,-0.2163000517606743500209631747566163539886474609375f,-0.76869998117089277656077683786861598491668701171875f,0.4192999906808136945102205572766251862049102783203125f,0.7261000283002854160940842120908200740814208984375f,-0.5864999886453148292275727726519107818603515625f,0.71349998077005150065588168217800557613372802734375f,0.23740000823736184809575888721155934035778045654296875f,0.2076000085264444461330413105315528810024261474609375f,-0.08280000285506246282540843139940989203751087188720703125f,-0.298700001883506871536155813373625278472900390625f,0.1624000329732897540679914527572691440582275390625f,0.66680000573098663352311632479541003704071044921875f,1.2671000079154968176453621708787977695465087890625f,0.80860000594854408717537808115594089031219482421875f,-0.6738999895066022371992175976629368960857391357421875f,0.3796000159621240044316436978988349437713623046875f,-0.0028000140309334398125429288484156131744384765625f,-0.5735000093057751602287908099242486059665679931640625f,-0.1055999812722205888348980806767940521240234375f,-0.39160000946521744680239862645976245403289794921875f,-0.3872000058621167806194307559053413569927215576171875f,-1.429099996948242079497504164464771747589111328125f,-1.087200009512901743846668978221714496612548828125f,0.550799985584616624834097819984890520572662353515625f,-0.5898000355482100331983019714243710041046142578125f,-0.07180000640153849644775618799030780792236328125f,0.6157000142365693928780956412083469331264495849609375f,-0.13720000243484975488428290191222913563251495361328125f,-0.05079999847412108859856516573927365243434906005859375f,0.11650000064074993166851612613754696212708950042724609375f,-0.34650000205636022343469448969699442386627197265625f,-0.090799994301795639017882422194816172122955322265625f,0.251900001405179541080769922700710594654083251953125f,0.918599991941452032051529386080801486968994140625f,0.74340000573396736172071541659533977508544921875f,-0.298499984979629484627139390795491635799407958984375f,-1.0755000377893448870736392564140260219573974609375f,-0.42880000228881787194268326857127249240875244140625f,0.8392000199288129902441824015113525092601776123046875f,0.3022000104442239543089954167953692376613616943359375f,0.18400001122057452196401072797016240656375885009765625f,-0.086799999658763393739491220912896096706390380859375f,0.6876000129222870071998841012828052043914794921875f,0.305799995791911971565468775224871933460235595703125f,-0.535000004805624485015869140625f,0.1314999968037009470123166465782560408115386962890625f,0.03299999569356415474175037161330692470073699951171875f,-0.051900000287592451542195703950710594654083251953125f,-0.69709997304677973062325690989382565021514892578125f,0.0784999822974210559323182678781449794769287109375f,-0.17219999986439915584668369774590246379375457763671875f,-0.0075999981328844778971642881515435874462127685546875f,-0.25640001062899830941432810504920780658721923828125f,-0.59660000037550897644678116193972527980804443359375f,0.573499994777142863000563011155463755130767822265625f,1.117900025165081245859255432151257991790771484375f,-0.29350000420212740692704755929298698902130126953125f,0.66370001626312691289655276705161668360233306884765625f,-0.22869995121955799532997843925841152667999267578125f,0.3683999970912932298006126075051724910736083984375f,-0.4344999820664525724822624397347681224346160888671875f,-1.0025000146031377568078823969699442386627197265625f,0.642999978482723388850672563421539962291717529296875f,-0.5717999779403211135786477825604379177093505859375f,-0.72060001345872848332874127663671970367431640625f,0.6745999919712541981198228313587605953216552734375f,-0.82570000257641051888413130654953420162200927734375f,0.16989995757341347371038864366710186004638671875f,-0.479499970555306020969510427676141262054443359375f,-0.5855999804154041754600257263518869876861572265625f,-0.3743999787926668432191945612430572509765625f,-0.04039999886453149446907673336681909859180450439453125f,-0.05810000049471854277616245099125080741941928863525390625f,0.03949999786168334470204399622161872684955596923828125f,-0.2021999928236006827120263551478274166584014892578125f,-0.70979997273981598748804344722884707152843475341796875f,-0.77580000666975923451218477566726505756378173828125f,-0.520299991679191720095332129858434200286865234375f,0.565800013114511557432706467807292938232421875f,-0.062999948084353718513739295303821563720703125f,-0.2681999880850316486657902714796364307403564453125f,-0.71849998516589430597178989046369679272174835205078125f,-0.09089998725652659317120196647010743618011474609375f,-0.026800009568035687568254843426984734833240509033203125f,0.7165999991461633467082492643385194242000579833984375f,0.808600035490095958579104262753389775753021240234375f,0.09829999653697019112996713374741375446319580078125f,0.231899998164177001402919131578528322279453277587890625f,-0.298500002562999799948784129810519516468048095703125f,0.093499998874962131356625150147010572254657745361328125f,0.57109997797161360377771188723272643983364105224609375f,0.7603000101193782001018917071633040904998779296875f,0.54659996520876941161759532405994832515716552734375f,0.3183000030934810542504465047386474907398223876953125f,0.4098999906539919635406477027572691440582275390625f,0.369199986624717979566412395797669887542724609375f,-0.2359999986886978096123357317992486059665679931640625f,0.05019999805390806812965820427052676677703857421875f,0.2562000187218191893379071188974194228649139404296875f,0.204000007085502199632998099332326091825962066650390625f,0.091299983811378382370094186626374721527099609375f,0.44800001543760270550365021335892379283905029296875f,0.4697000026211142209575655215303413569927215576171875f,-0.53610001757144942757804528810083866119384765625f,-0.06979998458623892787500153644941747188568115234375f,0.706700007784366590612989966757595539093017578125f,-0.77999999880790671369368283194489777088165283203125f,-0.1629999874234204781942025874741375446319580078125f,0.80229997892379756319058969893376342952251434326171875f,-0.140800012257695339457086447509936988353729248046875f,0.0201999973833559831604134160443209111690521240234375f,0.159500002965331066917542557348497211933135986328125f,0.874800015580654122260284566436894237995147705078125f,0.56320000224113453413110619294457137584686279296875f,-0.5413000078395009406762028447701595723628997802734375f,1.1933000241041185063295415602624416351318359375f,0.8904000153303144315941608510911464691162109375f,-0.492000013694167126487855057348497211933135986328125f,-0.5241000011205674269376686424948275089263916015625f,-0.61720001577138905446417993516661226749420166015625f,-0.0628999986782670106322257197462022304534912109375f,0.45190002826452246864619155530817806720733642578125f,-0.0340000063776972893947458942420780658721923828125f,-0.67130001394897698130392882376327179372310638427734375f,0.25479998838603512556488794871256686747074127197265625f,0.29559999296963201942389787291176617145538330078125f,-0.00280000024735931052344284353239345364272594451904296875f,-0.72559998335838304939215959166176617145538330078125f,-0.50779998959302918848379704286344349384307861328125f,0.36469998641610146261626823616097681224346160888671875f,-0.464100041353702863489161245524883270263671875f,0.1350000107288362727331332280300557613372802734375f,0.807000023417174805473450760473497211933135986328125f,-0.0303999786734578947289264760911464691162109375f,-0.18100000345706934723466474679298698902130126953125f,-0.261200013096630545561538383481092751026153564453125f,0.20600000733137147079787609982304275035858154296875f,-0.0431000018358229652903901296667754650115966796875f,-0.35079996941685731570714779081754386425018310546875f,-0.7128999625429515152319481785525567829608917236328125f,-1.5244999905228606706941718584857881069183349609375f,-0.46250000096857479814360658565419726073741912841796875f,-0.1212999986380338679481383223901502788066864013671875f,-0.276500013753771867186515009962022304534912109375f,-0.769199976640939642180683222250081598758697509765625f,0.5707999696403744938066893155337311327457427978515625f,0.7215999960541719104867297573946416378021240234375f,-0.38029999600052821762119492632336914539337158203125f,0.48029997424483317214338740086532197892665863037109375f,1.22310000839233357083912778762169182300567626953125f,-0.47949996399879513120367846568115055561065673828125f,-0.343199997361004260909567165072076022624969482421875f,0.182399978733062795299701974727213382720947265625f,-0.19749999009072760625116416122182272374629974365234375f,-0.3368000080034134757767105838865973055362701416015625f,-0.400000026002526709589801612310111522674560546875f,-0.237399992293119144193269676179625093936920166015625f,-0.0848999947890638484437886290834285318851470947265625f,-0.9969000233471394523121489328332245349884033203125f,-0.53259998475909231530067700077779591083526611328125f,0.6067000004455447292883718546363525092601776123046875f,-0.2664999974370001467605106881819665431976318359375f,0.674599980050325509495223741396330296993255615234375f,-0.4254000025898216907904725303524173796176910400390625f,-0.0882000180363657904081264859996736049652099609375f,0.5133999811470513208178090280853211879730224609375f,-0.2110000027865168714669152905116789042949676513671875f,-0.7154000418543819961314511601813137531280517578125f,-0.32779997562319085346871361252851784229278564453125f,0.0182000001177191617074413443333469331264495849609375f,-0.24739999322444194973513731383718550205230712890625f,0.2656999906182289095113446819595992565155029296875f,-0.18519999533444642114687894718372263014316558837890625f,0.43959998764991770059396003489382565021514892578125f,-0.72409998487830140589949223794974386692047119140625f,0.247799976368248575653296938980929553508758544921875f,0.765000012218951752629436668939888477325439453125f,0.2980999970674513832591401296667754650115966796875f,-0.64849996955692790212566478658118285238742828369140625f,-0.94210001387596076227737285080365836620330810546875f,0.360699991214274984230314657907001674175262451171875f,0.53430001187771558601724564141477458178997039794921875f,-0.7689000357747080016679319669492542743682861328125f,-0.2908000112891198085662836092524230480194091796875f,0.5797000113382935193584444277803413569927215576171875f,-0.417100009331107191012932844387250952422618865966796875f,-0.1987999974831937688435345990001223981380462646484375f,0.320900005324184889321514901894261129200458526611328125f,-0.244399998313188593357381250825710594654083251953125f,-0.068999992370605500724423109204508364200592041015625f,0.220799997393786917587732432366465218365192413330078125f,-0.5625000277161600337194613530300557613372802734375f,-0.132999991297721553706878694356419146060943603515625f,0.645100019478797914729284457280300557613372802734375f,0.178399975037574876068902085535228252410888671875f,0.373399997985362830377198406495153903961181640625f,0.2797000071287154554511289461515843868255615234375f,-0.88889998332261999092196447236347012221813201904296875f,-0.7108000012308364201629728995612822473049163818359375f,0.376599977874755798001871198721346445381641387939453125f,-0.054400008520483655871657902025617659091949462890625f,0.1195999904811382297253885553800500929355621337890625f,0.0954999943897127945291458672727458178997039794921875f,-0.0735999835968019766596626141108572483062744140625f,-0.11719997524023018087291347910650074481964111328125f,0.0953000068411231637810487882234156131744384765625f,0.164499992161989183614423382095992565155029296875f,0.123399993515014738676427441532723605632781982421875f,-0.10689999525845050865058283307007513940334320068359375f,0.490700005444884457261878196732141077518463134765625f,0.5544000212609769828731032248470000922679901123046875f,0.100800000709295289880174095742404460906982421875f,1.1847000303745272020705670001916587352752685546875f,0.50780000091791155369946864084340631961822509765625f,-0.8657000086486339451852245474583469331264495849609375f,-1.061100006544589913204390541068278253078460693359375f,-0.76640000388622286919826365192420780658721923828125f,0.523099999935924930838382351794280111789703369140625f,0.09690001291632643898310561780817806720733642578125f,0.000600016438961237241755952709354460239410400390625f,-0.0047999966189264942073577913106419146060943603515625f,0.7456000042200088273602887056767940521240234375f,0.269199983048439062116585773765109479427337646484375f,-0.7086999986425042141746644119848497211933135986328125f,0.174999996647238642566435373737476766109466552734375f,0.566400023555755804949285447946749627590179443359375f,-0.23459998124241820960378390736877918243408203125f,-0.7668000110208985642401557925040833652019500732421875f,-0.4584000334501272533316296176053583621978759765625f,-0.029800001219660032791836101750959642231464385986328125f,-0.06500000117346647898752820537993102334439754486083984375f,0.30020000507608057827013681162497960031032562255859375f,-0.1178999983429906972531853170949034392833709716796875f,-0.1764999995976685998044786174432374536991119384765625f,-0.0461000136971474461233810870908200740814208984375f,-0.2895000015497206646841732435859739780426025390625f,0.311099986428022334816745342322974465787410736083984375f,0.996300026428699414537959455628879368305206298828125f,-0.53049997311830576762758937547914683818817138671875f,0.6375999792829156742612894959165714681148529052734375f,0.7616000282406805155233087134547531604766845703125f,0.51039999808221991894896518715540878474712371826171875f,-0.0279000011742114306656503686099313199520111083984375f,0.86129999621659525388395195477642118930816650390625f,-0.32599996657669605415463820463628508150577545166015625f,-0.74389997203499103051171914557926356792449951171875f,-1.1215000039190048397585997008718550205230712890625f,0.197599961364269827157613690360449254512786865234375f,0.8622999919995664175331739897956140339374542236328125f,1.4159000334501268270059881615452468395233154296875f,-0.0868999936938286143828236163244582712650299072265625f,-0.18129999550879005720815939639578573405742645263671875f,-0.44570000060200687386213758145458996295928955078125f,-0.56419999057352565596801241554203443229198455810546875f,0.8533000030562278848123014540760777890682220458984375f,0.788900046131015386663420940749347209930419921875f,0.2308999868094918639371826429851353168487548828125f,0.91590000603198984396868809199077077209949493408203125f,0.28590003881454517653537550359033048152923583984375f,0.440799989235400868636816085199825465679168701171875f,-0.1602000015556812595463043180643580853939056396484375f,0.8178000079214573503350038663484156131744384765625f,0.1342999748110773605702661370742134749889373779296875f,0.8213999918535350719395182750304229557514190673828125f,1.1826000324428083132488609408028423786163330078125f,0.21239999765753747595908862422220408916473388671875f,-0.135699993337690838490772193836164660751819610595703125f,-0.910100005173683168635534457280300557613372802734375f,-0.28129999446570852494886594286072067916393280029296875f,-0.255300000359117984505274989714962430298328399658203125f,-0.329600017154216828885182621888816356658935546875f,-0.183600017869472775799977171118371188640594482421875f,0.0995999993473291045376072361250407993793487548828125f,-0.12189999846220000989660547929815948009490966796875f,0.168600014591216762482872582040727138519287109375f,0.39959999111443755825945345350191928446292877197265625f,0.658699997711181595150264911353588104248046875f,0.56120000754594823177967555238865315914154052734375f,-0.1828999934628605661401934412424452602863311767578125f,-0.551099988663196427296497859060764312744140625f,-0.394599995398522196410340256989002227783203125f,0.35199998102337115568616354721598327159881591796875f,0.0480999988555907975751324556767940521240234375f,-0.3852000329971314585009167785756289958953857421875f,-0.5861999997228384273739720811136066913604736328125f,-0.48239999735951410997358834720216691493988037109375f,-0.220999996602535109246900901780463755130767822265625f,0.2773999945655465371174841493484564125537872314453125f,-0.11170001406148066802614948755945079028606414794921875f,0.120399993239343083839543169233365915715694427490234375f,0.181699998564273101919752662070095539093017578125f,1.091200006353854945473358384333550930023193359375f,0.79980001885891027768593630753457546234130859375f,-0.428999984435737058419135792064480483531951904296875f,-0.381099979424476487110950984060764312744140625f,-0.159199979472160801918789729825221002101898193359375f,0.3880999812349676947320631370530463755130767822265625f,0.61119999398589097694411975680850446224212646484375f,0.644800017070770170590776615426875650882720947265625f,0.000500007428228865791197677026502788066864013671875f,0.18390001178383830193752146442420780658721923828125f,-0.033799999010562853385408743633888661861419677734375f,-0.2296000017687678251832039677537977695465087890625f,-0.91530002088546780214528553187847137451171875f,-0.3007999943017960475799554842524230480194091796875f,0.73520000386536121794733844581060111522674560546875f,-0.2117999899730087509652776134316809475421905517578125f,-0.33400000004470331571582164542633108794689178466796875f,-0.11419999880641695799710078063071705400943756103515625f,0.3694000032305719827263601473532617092132568359375f,0.38040000966787346214914578013122081756591796875f,0.0260999981626868216011416734545491635799407958984375f,-0.7735999940276148123530219891108572483062744140625f,-0.0278999922335145811302936635911464691162109375f,-0.0089999820888042592059719027020037174224853515625f,-0.8770000027418134447287911825696937739849090576171875f,-1.032200045573711921775839073234237730503082275390625f,-0.4424999819695951686071566655300557613372802734375f,0.31419999083429583919269134639762341976165771484375f,0.5618000007390975270027411170303821563720703125f,-0.428999992817640229958442432689480483531951904296875f,0.401800001111626503558227341272868216037750244140625f,-0.056199980127811244301483384333550930023193359375f,0.2454999988973138869141621398739516735076904296875f,0.425399966789782535414587982813827693462371826171875f,1.208999976575373835885329754091799259185791015625f,0.7847999874174593770703722839243710041046142578125f,-0.5832999963507055074529716875986196100711822509765625f,-0.8741000210881235688020751695148646831512451171875f,-0.0838999909222126749597236994304694235324859619140625f,-0.76589997354745886770643892305088229477405548095703125f,-0.72030001298785162333615517127327620983123779296875f,-0.38259997118040944830141825150349177420139312744140625f,-0.683599988756328880157298044650815427303314208984375f,-0.97840002110078927444192231632769107818603515625f,-0.4471000117897983017201113398186862468719482421875f,0.802100009852647399810621209326200187206268310546875f,0.140500047922134996269960538484156131744384765625f,-0.05040000132322308701304791611619293689727783203125f,0.403599984416365853423513954112422652542591094970703125f,0.984799986374377933628920800401829183101654052734375f,-0.6345999955475323250908559202798642218112945556640625f,0.57409996993988787750140545540489256381988525390625f,0.236499983876942820870681316591799259185791015625f,0.2686999851942062367271546463598497211933135986328125f,0.35399999948590987752794490006635896861553192138671875f,0.39120001514554036958770666387863457202911376953125f,-0.37099998572468706470317556522786617279052734375f,-0.9531000056728717861886934770154766738414764404296875f,-0.6159000546097761485953014926053583621978759765625f,0.442199975799024425082706102330121211707592010498046875f,-0.12779999506920580731872405522153712809085845947265625f,0.047399996651709130901508615352213382720947265625f,-0.3539999879002575511321992962621152400970458984375f,0.588500005729496233897179990890435874462127685546875f,0.517300038826466135333248530514538288116455078125f,0.35059996786117597622478569974191486835479736328125f,0.220100020596385004267858676030300557613372802734375f,0.01790001264810570091867703013122081756591796875f,-0.00879998887777322380543409963138401508331298828125f,0.030699996317923083921641591587103903293609619140625f,0.21879998396039024299852826516143977642059326171875f,0.469200002419948081211487078689970076084136962890625f,0.3017999953746794972175848670303821563720703125f,0.44889999637976296664731989949359558522701263427734375f,0.472200008805095994279810156513121910393238067626953125f,0.0512999999977648002413843641988933086395263671875f,0.3396999981880188812510823481716215610504150390625f,0.204599992418289300388778428896330296993255615234375f,-0.2510999987214803663704287828295491635799407958984375f,0.9710999958157540135061935870908200740814208984375f,0.321599972808360945464301039464771747589111328125f,-0.8384999944791198256410780231817625463008880615234375f,-0.17979998965263366272893108543939888477325439453125f,-0.183199993002414629472696105949580669403076171875f,0.05149999754875900226380736057762987911701202392578125f,-0.52480000604391119356506578697008080780506134033203125f,-0.4792000017493960317693790784687735140323638916015625f,0.152200011450052297146129376415046863257884979248046875f,-0.52499999910593064811337171704508364200592041015625f,-0.5982000052213667373735006549395620822906494140625f,-0.01199999242275950894764946497161872684955596923828125f,-0.26870000963211071454139755587675608694553375244140625f,-0.1690000003427265529154510659282095730304718017578125f,0.17170000852942468583250956726260483264923095703125f,-0.12149999967217439689193270169198513031005859375f,-0.260200015711784526928340710583142936229705810546875f,-0.18170000376105310824215166576323099434375762939453125f,0.0913000037789348795058685936965048313140869140625f,0.37040002009868633336964194313623011112213134765625f,0.3849999941885471343994140625f,-0.6185999855339527897513107745908200740814208984375f,-0.633199999260902313125143336947076022624969482421875f,0.05329999605268243012545781311928294599056243896484375f,-1.0215000060796735059653883581631816923618316650390625f,-0.798400011843443113690454993047751486301422119140625f,0.44499999508261678382581294499686919152736663818359375f,0.6302000009596344654028143850155174732208251953125f,0.674200022387504649401535061770118772983551025390625f,-0.01389999434947963852238217441481538116931915283203125f,1.1980000178217888873177798814140260219573974609375f,1.5398000149846069462000741623342037200927734375f,0.0551999902307986900495961890555918216705322265625f,0.42880000132024276293662978787324391305446624755859375f,0.64760002007484462893671661731787025928497314453125f,-0.6893999625504021455668635098845697939395904541015625f,0.4231999760895970297269741422496736049652099609375f,0.378399997240304486467721289955079555511474609375f,-0.4472999860204758260096014055307023227214813232421875f,0.7708999835498631103547495513339526951313018798828125f,0.71290002197995772093008781666867434978485107421875f,-0.071899991309642974357529965345747768878936767578125f,-0.199299986433983000733149992811377160251140594482421875f,-0.641299994540214246541154352598823606967926025390625f,0.36999999687075568743921394343487918376922607421875f,0.6035999879926443600908214648370631039142608642578125f,0.0240000275373457583327763131819665431976318359375f,0.187399986481666669391188406734727323055267333984375f,0.12610000263303511314205707094515673816204071044921875f,0.38450000661611571928233388462103903293609619140625f,0.41839997425526409546847617093590088188648223876953125f,-0.0362000024423003718965219377423636615276336669921875f,0.48860000904053446646457814495079219341278076171875f,0.64469996570348797604310675524175167083740234375f,-0.2373000078320501859963087554206140339374542236328125f,0.1994999833703037239729383145458996295928955078125f,0.06529999268502006959380423722905106842517852783203125f,0.03229999587386844783765127431252039968967437744140625f,0.315100007818639227252788259647786617279052734375f,0.3776999978303907656851379215368069708347320556640625f,-0.33149999885261050014406691843760199844837188720703125f,-0.35910001408457770111226636799983680248260498046875f,-0.614599977070093839159881099476478993892669677734375f,0.5897000147655602209084690912277437746524810791015625f,0.7645000504255303752643158077262341976165771484375f,0.6135999721229079550965934686246328055858612060546875f,-0.3686999920114875095578099717386066913604736328125f,0.084899986034631780285053537227213382720947265625f,0.5473999957948916250671800298732705414295196533203125f,-0.08910000160485509468344389460980892181396484375f,0.8985000147074462262253291555680334568023681640625f,-0.6804999965131275363461327287950552999973297119140625f,0.425799984429776845473725188639946281909942626953125f,-0.014200012105703763154451735317707061767578125f,-0.1156999901711941447501885704696178436279296875f,0.25319999605715270973860242520458996295928955078125f,-0.011000002712011625050081420340575277805328369140625f,0.221400018191337810691265985951758921146392822265625f,0.237600006477534819726571413411875255405902862548828125f,1.0557999898314474052085643052123486995697021484375f,0.67179999358654018948300290503539144992828369140625f,-0.5271999882414937221852824222878552973270416259765625f,0.430800006148218894708179504959844052791595458984375f,0.4806000243365768920256186902406625449657440185546875f,0.01260000781863934715687491916469298303127288818359375f,0.42690000337958355913769992184825241565704345703125f,0.118799993348121635250436156638897955417633056640625f,-0.3050999988779425908802522826590575277805328369140625f,0.856900017088651821950406883843243122100830078125f,0.30699998713284715190496854120283387601375579833984375f,-0.78100000829994675743961352054611779749393463134765625f,-0.7438999898791311959911354279029183089733123779296875f,-0.327599981182813859703628622810356318950653076171875f,0.6182999875590204919006964701111428439617156982421875f,-1.022999984443187404536956819356419146060943603515625f,-0.67839999053478283741469567758031189441680908203125f,0.599499980054795766903907860978506505489349365234375f,0.395900013482570756195855210535228252410888671875f,0.18519999216794946761410756153054535388946533203125f,-0.409900012819468972935510464594699442386627197265625f,-0.918499999582767401307137333787977695465087890625f,-0.784200008678435978026755037717521190643310546875f,0.23100000133365394816564730717800557613372802734375f,-0.17559999192655073585456193541176617145538330078125f,0.05360002703368704768394081838778220117092132568359375f,0.415300003376603166227454266845597885549068450927734375f,0.631300005269050501510719186626374721527099609375f,0.218399990534782428852622615522705018520355224609375f,-0.5660999972686171499702822984545491635799407958984375f,0.1290000078678128492271071081631816923618316650390625f,-0.180200003045797085832191442023031413555145263671875f,-0.36309999215006827366636343867867253720760345458984375f,-0.678800030601024673160281963646411895751953125f,-0.26220001517534274881882083718664944171905517578125f,0.4767000048041343251981061257538385689258575439453125f,0.34840000394582748555194484652020037174224853515625f,0.14639999345541010455917785293422639369964599609375f,-0.33870000005811451426751546023297123610973358154296875f,-0.694299975109100131476225215010344982147216796875f,-0.2819000416755681470704075763933360576629638671875f,-0.4579000001311295253714206410222686827182769775390625f,-0.178299999964237176897086101234890520572662353515625f,-0.304700018453598175938168424181640148162841796875f,-0.22949999827146516651055208058096468448638916015625f,0.79609997480511685008508493410772643983364105224609375f,0.18490001509189557538093140465207397937774658203125f,-0.50430000573098598959376204220461659133434295654296875f,0.2901000068873165194105467890040017664432525634765625f,-0.01469998812973472013254649937152862548828125f,0.2345999936103815297627761538024060428142547607421875f,0.8573999934107063847221752439509145915508270263671875f,0.9282000476896765661649624234996736049652099609375f,0.259899997713416708311484626392484642565250396728515625f,0.084800004721432953314064207006595097482204437255859375f,-0.67400000498071310772729702875949442386627197265625f,-0.1017999981313943980154590462916530668735504150390625f,-0.01240000220239156902835020446218550205230712890625f,0.6903999897748231973082511103712022304534912109375f,0.313399997240304539758426471962593495845794677734375f,0.0248999964281917396391463626059703528881072998046875f,1.0544999945461750456843219581060111522674560546875f,0.00420001180768014847899394226260483264923095703125f,-0.61279998527169221578247970683150924742221832275390625f,-1.32520003538131714293513141456060111522674560546875f,0.59469998343586905065194514463655650615692138671875f,-0.7077999745056036129398080447572283446788787841796875f,-0.1489999944567674816653379821218550205230712890625f,-0.246200009520352036673784823506139218807220458984375f,0.8065000089481468847907308372668921947479248046875f,1.3889000586479909316039993427693843841552734375f,0.21239999005794540920533108874224126338958740234375f,0.182599995934963288402030912038753740489482879638671875f,-0.309800000083446658294406006461940705776214599609375f,0.4906999680429702248574130862834863364696502685546875f,0.0785999983862042395088565172045491635799407958984375f,0.04020000088512887259639683179557323455810546875f,-0.205099974179268063068093397305347025394439697265625f,-0.8692000036492941728027972203562967479228973388671875f,-0.7004000338077549514537167851813137531280517578125f,0.04900000071525578704267900320701301097869873046875f,-0.26829998793154963276919033887679688632488250732421875f,-0.74510000576972945651732516125775873661041259765625f,-0.3197999895036218731547705829143524169921875f,0.52050002322345978456041848403401672840118408203125f,-0.32619998519420645433086747289053164422512054443359375f,-0.3565999895721672086068565477035008370876312255859375f,0.02330000056028370958305373505936586298048496246337890625f,-0.6099999976158141379301014239899814128875732421875f,-0.60960000821352000599517850787378847599029541015625f,0.104699999342858784956433737534098327159881591796875f,0.5013000416278838855532740126363933086395263671875f,0.0174000183701519262058354797773063182830810546875f,-0.5499000048846005572755757384584285318851470947265625f,-0.059100025260448774133692495524883270263671875f,0.3044000133633613103256720933131873607635498046875f,0.499300014187395557740956064662896096706390380859375f,0.03180001713037494681657335604541003704071044921875f,-0.1442000021219254080051541677676141262054443359375f,-0.25480000906139610439282705556252039968967437744140625f,-0.47240000674724580420615893672220408916473388671875f,-0.35800000590085989671251809340901672840118408203125f,0.20110000136643646495571147170267067849636077880859375f,-0.146199998456239654842647723853588104248046875f,-0.1282000023901461727149353464483283460140228271484375f,0.00729999896585942220639253719127736985683441162109375f,0.5984999939203259344822072307579219341278076171875f,0.46599999511241918526138761080801486968994140625f,-0.3699999947845935555079677214962430298328399658203125f,0.1312999881327154838572823791764676570892333984375f,0.301999999463558044254796186578460037708282470703125f,0.20739999817907806001215931246406398713588714599609375f,-0.12129999334812158195973097463138401508331298828125f,-0.0591999930322171241670048402738757431507110595703125f,0.12549999498575925560572841277462430298328399658203125f,0.28380000221431256335336001939140260219573974609375f,0.15520000148117552019044751432375051081180572509765625f,-0.150699997323751444522343945209286175668239593505859375f,0.02739998856782932534770225174725055694580078125f,0.256399997627735132255111238919198513031005859375f,0.18189999790340660723586552194319665431976318359375f,-0.728800001692771814987281686626374721527099609375f,-0.49719999997615804687711715814657509326934814453125f,0.367599999138712885127233676030300557613372802734375f,0.814799994421004925015949993394315242767333984375f,0.75160001751184513096859518554992973804473876953125f,-0.1758999834194778788543089831364341080188751220703125f,-0.2086999929428099420647413353435695171356201171875f,1.42300003600120561486619408242404460906982421875f,0.37489996953159632209207074993173591792583465576171875f,-0.0559999989867210334892888567992486059665679931640625f,0.26239999009519809902712950133718550205230712890625f,0.586599982345104020708959069452248513698577880859375f,-0.1276999957069755442518044219468720257282257080078125f,0.4451000093460084627849937533028423786163330078125f,0.5008999558150772912767934030853211879730224609375f,0.6483000156104561373382466626935638487339019775390625f,0.5583000312566763767563315923325717449188232421875f,-0.63899998983740768920824848464690148830413818359375f,0.51419996658265620936845152755267918109893798828125f,0.79139999852180409334323485381901264190673828125f,0.302499990761280201923000277020037174224853515625f,-0.3314999890923500114325861432007513940334320068359375f,-0.5139000184893607325875564129091799259185791015625f,0.225599995875358505514896023669280111789703369140625f,-0.156700004766881395568134394125081598758697509765625f,0.235299995553493346278628450818359851837158203125f,0.09349999935925001892655927804298698902130126953125f,-0.28339998908191932347477859366335906088352203369140625f,-0.2866000018656251757676045599509961903095245361328125f,-0.3499999891221516890027487534098327159881591796875f,-0.661499996580183324823565271799452602863311767578125f,-1.627500045597553679499469581060111522674560546875f,-0.4531999715447432297565910630510188639163970947265625f,0.2754999983385204576080695915152318775653839111328125f,0.2567000213444234901771778822876513004302978515625f,0.0043999979406594835751320715644396841526031494140625f,0.2055999962478876341709366215582122094929218292236328125f,0.4340000030994415336493830182007513940334320068359375f,-0.00540000411719088191375703900121152400970458984375f,0.34469999919384708420011520502157509326934814453125f,0.25590001832544817528258818128961138427257537841796875f,-0.50199996787309686396838515065610408782958984375f,-0.471699983532726907498044965905137360095977783203125f,-0.797300003659724865201496868394315242767333984375f,0.23739999452829341208826008369214832782745361328125f,-0.27379999006986632803517522916081361472606658935546875f,0.30380000705719023557094260468147695064544677734375f,-0.648800000798701592685802097548730671405792236328125f,0.401100000397860867451527155935764312744140625f,-0.22769998412132252951778355054557323455810546875f,0.0797999874174593060160987079143524169921875f,0.6349000131174922234578161805984564125537872314453125f,-1.1395000176429750382567362976260483264923095703125f,-0.676800003123283477890481663052923977375030517578125f,0.65190000416189430865188114694319665431976318359375f,-0.64189999905824635106910136528313159942626953125f,-0.1147999887585644529508499545045197010040283203125f,0.663499982409179178688418687670491635799407958984375f,-0.7427000142216684253071434795856475830078125f,-0.227999984443187031502020545303821563720703125f,0.633300014828145574341533574624918401241302490234375f,0.793999993860721620109188734204508364200592041015625f,0.42019998024702065464452971355058252811431884765625f,-0.60199999763816602893484741798602044582366943359375f,1.12020003389120148540314403362572193145751953125f,0.8266000184059141275838555884547531604766845703125f,-0.437200021247565739912488425034098327159881591796875f,-0.8174000156879426270961630507372319698333740234375f,-0.473599991941451747834435082040727138519287109375f,0.6164000140190124543693173109204508364200592041015625f,0.49530002952814111694124221685342490673065185546875f,0.07539999625682813899629763909615576267242431640625f,-0.61720001595765350277389416078221984207630157470703125f,0.9758000172495844282138932612724602222442626953125f,0.334199983644484888145598233677446842193603515625f,-0.7968000137403608018615841501741670072078704833984375f,0.162899998790025701822514747618697583675384521484375f,0.149199998843669856540827822755090892314910888671875f,-0.069700000758469105033299229035037569701671600341796875f,-0.28070001073479655673281740746460855007171630859375f,0.062400001680850802898703477694652974605560302734375f,0.3395999965161085043519051396287977695465087890625f,0.0861000128030777744925217120908200740814208984375f,0.30820003263950379590596639900468289852142333984375f,0.339799999524653006943708533071912825107574462890625f,-0.47370000933408729082429999834857881069183349609375f,0.0198000164747238471818491234444081783294677734375f,0.66060000015944242424126287005492486059665679931640625f,0.314200017321109470458395662717521190643310546875f,-0.04799999070167526582508799037896096706390380859375f,-0.3569999946206807717175024663447402417659759521484375f,-0.00339998308420152994813179248012602329254150390625f,-0.28000001579523114969560992904007434844970703125f,-0.35930001552850010515527401366853155195713043212890625f,-0.076599997544288811468504718504846096038818359375f,-0.260299986910820013719103371840901672840118408203125f,-1.590900021529197516656495281495153903961181640625f,0.02090002391338430243195034563541412353515625f,-0.13680000461339947293026853003539144992828369140625f,-0.137899996480345787830401604878716170787811279296875f,-0.712499961219728472627821247442625463008880615234375f,-0.4559999904930591707596931883017532527446746826171875f,-0.0197000107049944972459343262016773223876953125f,0.0204999656677244956881622783839702606201171875f,0.32130000541806236213915326516143977642059326171875f,0.89409998845458016347720331395976245403289794921875f,0.58710000068843282150510276551358401775360107421875f,0.330000016689300623706770920762210153043270111083984375f,-0.013800027918815516159156686626374721527099609375f,0.79439996418953029433396295644342899322509765625f,0.447599989825487210026722095790319144725799560546875f,-0.212000011757015993651975804823450744152069091796875f,0.549000018373131837279288447462022304534912109375f,0.12370000255405899647342948810546658933162689208984375f,1.01070001237392403936610207892954349517822265625f,0.072899975991248311402159743010997772216796875f,0.2740000087618827961932765902020037174224853515625f,0.035300014925002898280581575818359851837158203125f,1.2482999567508699101381353102624416351318359375f,0.4505999893188477045669060316868126392364501953125f,0.601100011610984807930435636080801486968994140625f,-0.90529998571872738466481678187847137451171875f,-0.40619999905228620917796433786861598491668701171875f,-0.5506999870419502229879071819595992565155029296875f,-0.41829995824098631373999523930251598358154296875f,-0.38359999268651012727104898658581078052520751953125f,0.6198000030636787727189584984444081783294677734375f,0.5447999778807162130078722839243710041046142578125f,0.35789999923706050655169974561431445181369781494140625f,-0.32089999532178048458064267833833582699298858642578125f,-0.118499994646757755845101200975477695465087890625f,-0.1616000026293098745799881044149515219032764434814453125f,-0.50170001741424175012440400678315199911594390869140625f,1.026900005856901998413377441465854644775390625f,0.28659999432191252122237301591667346656322479248046875f,0.47030000598430632408053497783839702606201171875f,1.0094999759197236244290252216160297393798828125f,0.29629999215602875750619205064140260219573974609375f,-0.065099993848800608020610525272786617279052734375f,0.59989997821152218904217079398222267627716064453125f,0.50400000444054604553656417920137755572795867919921875f,-0.33379999379515645774318954863701947033405303955078125f,-0.72119999674260615307730404310859739780426025390625f,0.58120001842379576117991746286861598491668701171875f,0.09519999768137932250056110206060111522674560546875f,-0.98420000808238938816430163569748401641845703125f,-0.5906000012397765175364838796667754650115966796875f,0.3105999924480915552749138441868126392364501953125f,0.9086999977856871080206246915622614324092864990234375f,-0.53400001859664936176841365522705018520355224609375f,-0.48460002102851884586698361090384423732757568359375f,0.67899997895955976900950190611183643341064453125f,-0.042400002872943876042199917719699442386627197265625f,-0.225600001388788218292802412179298698902130126953125f,0.3391999942988153105005721954512409865856170654296875f,0.5368000189185142545511553180404007434844970703125f,0.1413999863028525538766189129091799259185791015625f,0.0543999728322024367344056372530758380889892578125f,0.788600007885694509468521573580801486968994140625f,0.367400022095441869396381662227213382720947265625f,-0.708899960970878595389876863919198513031005859375f,-0.38220001249313373392624271218664944171905517578125f,0.71100000286102282842648492078296840190887451171875f,1.1653000119447707305653239018283784389495849609375f,-0.406800004613399490693836924037896096706390380859375f,0.015999994516372684216776178800500929355621337890625f,0.596599992328882056114025544957257807254791259765625f,-0.75800000083446494869576781638897955417633056640625f,0.34379998410940171282845767564140260219573974609375f,1.480599981421231614575617641094140708446502685546875f,0.388199998664855883134805480949580669403076171875f,-0.195799988341331498986619408242404460906982421875f,-0.83429999388456277387149384594522416591644287109375f,0.167400001196563064098654649569652974605560302734375f,0.4735999878808854202105749209295026957988739013671875f,-0.1150999955810607655593003073590807616710662841796875f,0.138799990665912531540016061626374721527099609375f,-0.612999992787838010599443805404007434844970703125f,-0.378200008499621986857164301909506320953369140625f,0.2392000009298325124973416677676141262054443359375f,-0.3477999869108199249012614018283784389495849609375f,-0.66719999759197179400871391408145427703857421875f,-0.141399997925758302841359181911684572696685791015625f,-0.10990000406503674934555192521656863391399383544921875f,-0.678799987313151387269272163393907248973846435546875f,0.4392000115841625529355951584875583648681640625f,0.02170000085532663991472190900822170078754425048828125f,-1.040700032043456957353555480949580669403076171875f,0.8373000057458881428829045034945011138916015625f,0.3755000050067902606087955064140260219573974609375f,0.888899997329712032723136871936731040477752685546875f,-0.5669000051677226537094611558131873607635498046875f,-0.0897000016897917529234973699203692376613616943359375f,0.705200009155273477290393202565610408782958984375f,-1.073200005221366382102132774889469146728515625f,-0.58430001549124721549333116854541003704071044921875f,0.454900020158290896432617955724708735942840576171875f,-0.1946999986350537170665120356716215610504150390625f,-0.215500006273388944322277893661521375179290771484375f,0.305399987912178261240114807151257991790771484375f,0.894599980497361002562684006989002227783203125f,0.24469998552203175989916417165659368038177490234375f,-0.61229997475147257546268519945442676544189453125f,1.360699984359740710715414024889469146728515625f,0.582300003510713537480114609934389591217041015625f,1.2780000032186507041842560283839702606201171875f,0.0481999539613724437003838829696178436279296875f,0.20779999272227289708325770334340631961822509765625f,0.59679999984502796195329210604541003704071044921875f,-0.534800000381469686772106797434389591217041015625f,-0.193000002175569562723467242904007434844970703125f,1.56259999504089375221838054130785167217254638671875f,-0.739800013792514477017903118394315242767333984375f,-0.197900009071827032158807924133725464344024658203125f,-0.8984000054359437825723944115452468395233154296875f,0.35820000879764535284266457892954349517822265625f,0.0988000081002713415045946021564304828643798828125f,-0.129700022104382473031591871404089033603668212890625f,0.575299965117872513786778654321096837520599365234375f,0.3891999912440777809052860902738757431507110595703125f,-0.36299998891353624230760033242404460906982421875f,-0.26779998273849514589528553187847137451171875f,-0.3246999977409839033271055086515843868255615234375f,-0.72919999735355389702817774377763271331787109375f,0.04480002481937361835662159137427806854248046875f,-0.072499989122152275200505755492486059665679931640625f,-0.08460000046491611414012368186376988887786865234375f,-0.4599999845772984041758491002838127315044403076171875f,0.96100000837445254120439130929298698902130126953125f,0.39200002196431160683687267010100185871124267578125f,-0.87029995979070662315280060283839702606201171875f,0.717800020736456101388966999365948140621185302734375f,0.108400020784139794471911955042742192745208740234375f,-0.249500004529952690290883765555918216705322265625f,-0.76760003870129622782769729383289813995361328125f,-0.23620000918507599863005452789366245269775390625f,1.0754000126481049193216676940210163593292236328125f,0.57259999340176559456949689774774014949798583984375f,0.12240001285672175157515084720216691493988037109375f,-0.0324999818205835566686801030300557613372802734375f,0.68060001301169403831181625719182193279266357421875f,-0.14820000119805332161604383145458996295928955078125f,-1.26129998142719301768011064268648624420166015625f,-0.5268000005900859150642645545303821563720703125f,0.78040001190304764122629421763122081756591796875f,1.3002000020027164595148860826157033443450927734375f,0.360999998092651264158803314785473048686981201171875f,-0.051599992179870657338369710487313568592071533203125f,-0.1530999955773353349286480806767940521240234375f,-0.55900000607967381682783525320701301097869873046875f,0.80200001466274262185152110760100185871124267578125f,1.6268999919056899017277828534133732318878173828125f,-0.48459998779892909936961586936376988887786865234375f,0.81339998144507408284198390902020037174224853515625f,0.6168999910116195195541877183131873607635498046875f,0.0780000086575747264561186966602690517902374267578125f,-0.1136000004351138681357724635745398700237274169921875f,0.12299997449666288940051117606344632804393768310546875f,-0.139000000156462188982686711824499070644378662109375f,-0.1278000003591179822848289404646493494510650634765625f,0.6196999758921564183111740931053645908832550048828125f,0.4887999976694583637026880751363933086395263671875f,-0.4067999976098537473490068805404007434844970703125f,-1.095999971270561701430779066868126392364501953125f,0.739399997717141932440654272795654833316802978515625f,0.044800011855363752744096927926875650882720947265625f,-0.75549997133016599804022916941903531551361083984375f,-0.76759999589771010786165561512461863458156585693359375f,0.1525999834552408118693023197920410893857479095458984375f,0.7393999806739388613863184218644164502620697021484375f,-0.263599994623660904835560359060764312744140625f,0.1465999964267015176488939687260426580905914306640625f,-0.748099975609779477281335857696831226348876953125f,1.6176999775648113200077204965054988861083984375f,0.6724000013828277388938658987171947956085205078125f,-0.0078999905943870629698722041212022304534912109375f,1.110700006711482501486898399889469146728515625f,0.318400006031990034927048327517695724964141845703125f,1.090900036430358710504151531495153903961181640625f,-1.2043000287532805714363348670303821563720703125f,-0.5392000165760517660373807302676141262054443359375f,0.20950003135204298132521216757595539093017578125f,0.926099966311454636525013484060764312744140625f,0.2545999900341033850281746708787977695465087890625f,-1.331899992799758791761632892303168773651123046875f,0.78510001113414773499243892729282379150390625f,0.27770000066161149021581877605058252811431884765625f,1.380699990916252062334024230949580669403076171875f,-0.102900020694733029813505709171295166015625f,0.19390000597238543633693552692420780658721923828125f,-0.0279999862313271563607486314140260219573974609375f,0.48019998918771733542598667554557323455810546875f,0.0914999965429306172381984652020037174224853515625f,1.7947999872684476940776221454143524169921875f,-0.106600032114982923303614370524883270263671875f,0.15230000365972529152713832445442676544189453125f,0.49369995360374474557829671539366245269775390625f,-0.0761000175714485749267623759806156158447265625f,0.32710000001788142043324114638380706310272216796875f,0.2121999864161014937025129256653599441051483154296875f,0.04380000936687000745450859540142118930816650390625f,-0.07279999871999021099355786645901389420032501220703125f,0.769000000044703480028829289949499070644378662109375f,-0.242399999333918092503381558344699442386627197265625f,-0.2170000033378600978739569882236537523567676544189453125f,-1.17249996960163116455078125f,0.3323000111103056042338721454143524169921875f,0.021299998414516352340797311626374721527099609375f,-0.658899974977970259715220890939235687255859375f,1.05329997910261141669252538122236728668212890625f,0.353999995350837703966817571199499070644378662109375f,0.60129997406974455831374370973207987844944000244140625f,-0.95469998621121021642466075718402862548828125f,-0.2247999973081052649259703457573777996003627777099609375f,1.216200002479553177181514911353588104248046875f,-0.244599986755847709218869567848742008209228515625f,0.45399998909234984267868640017695724964141845703125f,-0.72679998017549518607438585604541003704071044921875f,-0.51220000861883097087456917506642639636993408203125f,-0.6702000200331212909077294170856475830078125f,0.44020000125765790244258823804557323455810546875f,0.93770001362562194202610044158063828945159912109375f,-0.1084000027537346255712691345252096652984619140625f,0.2632000079035758943746259319595992565155029296875f,0.43780000032186539016265669488348066806793212890625f,0.37100000107288355621903974679298698902130126953125f,-0.40400000488758092132002275320701301097869873046875f,-0.9408999985814094912939253845252096652984619140625f,0.062600005918741263855054057785309851169586181640625f,0.372600008302927054870679057785309851169586181640625f,0.4320999974846841951148235239088535308837890625f,-0.29199999827146516651055208058096468448638916015625f,-0.16080000413656225788372466922737658023834228515625f,0.3155999917030332113654367276467382907867431640625f,0.100800005775690149079082402749918401241302490234375f,0.375600009137392110147857238189317286014556884765625f,-0.0343999860197303775066757225431501865386962890625f,0.4160000110566617337326533743180334568023681640625f,0.2210000182092191067795283743180334568023681640625f,0.20240000823736181700951419770717620849609375f,-0.250000006072223168729351527872495353221893310546875f,-0.7312000046774744799193967992323450744152069091796875f,0.0905999704875053435415566127630881965160369873046875f,-0.32360000714063641424900197307579219341278076171875f,-0.23940000919103621157546513131819665431976318359375f,-0.423799986791610905356719740666449069976806640625f,-0.38499999895691860984925369848497211933135986328125f,0.62000001117587100196715255151502788066864013671875f,0.24790001711845466303429930121637880802154541015625f,-0.1454000010251998986632315791212022304534912109375f,0.32500000298023223876953125f,0.891699999290704425902731600217521190643310546875f,-0.11000000402331355786600397550500929355621337890625f,0.15859999685883519049411916057579219341278076171875f,-0.31259998222589491234657543827779591083526611328125f,0.1016000055909158650280232905060984194278717041015625f,-0.6977999919772146508734067538171075284481048583984375f,-0.4200000189244743609151555574499070644378662109375f,0.7510000322163108421591459773480892181396484375f,-0.33769998993277550169977985206060111522674560546875f,-0.3797999783277514751489434274844825267791748046875f,0.10220001487731877887199516408145427703857421875f,-0.29179999656677235719826057902537286281585693359375f,0.8748999984025953580157874966971576213836669921875f,-0.58850002729892736397232511080801486968994140625f,-0.0283000010073184427028536447323858737945556640625f,0.4969999973773955304068294935859739780426025390625f,0.06699998843669874304396216757595539093017578125f,0.0856000021338463312758904066868126392364501953125f,-1.02280000061988829429537872783839702606201171875f,0.1746000120878221650855266489088535308837890625f,-0.03330000085830686629151387023739516735076904296875f,0.5562999966263770801333521376363933086395263671875f,0.396099987471103531788685359060764312744140625f,0.210900001412630100361411678022705018520355224609375f,0.29980000455379496315799769945442676544189453125f,-0.5169999836683274452298064716160297393798828125f,-0.24019999902248390100112374057061970233917236328125f,-0.0426000021934509476295716012828052043914794921875f,0.4141999945223329149257551762275397777557373046875f,0.29120000575780868867781236986047588288784027099609375f,0.43530002446174620445162872783839702606201171875f,0.7868999951839441564516164362430572509765625f,0.2428000013649462829334879643283784389495849609375f,0.089100001195073108561928165727294981479644775390625f,-0.883299970124661992798564824624918401241302490234375f,-0.5858999993264675243409556060214526951313018798828125f,-0.682600001001357981067485525272786617279052734375f,-0.68339998159408565925332368351519107818603515625f,-0.196699996310472524640999836265109479427337646484375f,0.0752999785661696563465739018283784389495849609375f,-0.10010000100135840739312698133289813995361328125f,0.276800015342235639081991394050419330596923828125f,1.07220001308918000404446502216160297393798828125f,0.095999979317188177674324833787977695465087890625f,0.0517999937355517747761268765316344797611236572265625f,-0.4829000069856643762022940791212022304534912109375f,-0.07490000317096701110131107270717620849609375f,0.121600002759695069443068859982304275035858154296875f,-0.42559999364018441525558955618180334568023681640625f,-0.386799954694510006447671912610530853271484375f,-0.3874999901652336209423310720012523233890533447265625f,-0.74689999577999088842261699028313159942626953125f,0.272000003635883391694960664608515799045562744140625f,0.178200001980364231002340602572076022624969482421875f,0.3769999956265090990115140812122263014316558837890625f,-0.3120999871097505096173563288175500929355621337890625f,0.14420000044554470708391136213322170078754425048828125f,-0.12080000108182431528547340349177829921245574951171875f,0.006899999635666287378654715212178416550159454345703125f,-0.34760000115036948642455172375775873661041259765625f,-0.25360000877976407451797058456577360630035400390625f,-0.4965999862194063751985595445148646831512451171875f,0.18580000994801526559285775874741375446319580078125f,0.508800010186433837588992901146411895751953125f,0.40809999893009685223432825296185910701751708984375f,0.795200011390447603076836458058096468448638916015625f,0.341800019067525884253200274542905390262603759765625f,-0.5039999834299084824351666611619293689727783203125f,-0.168400002790987546319456669152714312076568603515625f,-0.4022000077620149216528488977928645908832550048828125f,0.1849999654851857133763814999838359653949737548828125f,-0.3044000114262104261797503568232059478759765625f,-0.24779999838471411521823029033839702606201171875f,0.61340000126361804433372526546008884906768798828125f,0.437800018203258645144160254858434200286865234375f,-0.583199998962879107011758605949580669403076171875f,-0.27150005465745863375559565611183643341064453125f,0.49819999717473972822290306794457137584686279296875f,-0.71859998985528950043999429908581078052520751953125f,-0.5425000214576716217607099679298698902130126953125f,-0.5060000311732295585898100398480892181396484375f,-0.39120001410245919259978109039366245269775390625f,-0.17859998598098769662101403810083866119384765625f,-0.010600012117624402208093670196831226348876953125f,-0.86760001068711289917700923979282379150390625f,-0.1369000160455708936524388263933360576629638671875f,0.49820000835061062360864525544457137584686279296875f,0.34159998636841759633853143895976245403289794921875f,-0.452099989140034352885777479968965053558349609375f,0.02159999903440468216331282746978104114532470703125f,0.09199999707937234205701315659098327159881591796875f,0.477999991446733218936060438863933086395263671875f,-0.243200013715028973138032597489655017852783203125f,-0.46120002468228360470448023988865315914154052734375f,0.10319998517930439874135117861442267894744873046875f,-0.40999998643994306490867529646493494510650634765625f,-0.0360999970078468290779483140795491635799407958984375f,-0.373600013920664775213253960828296840190887451171875f,0.504999979473650828509789789677597582340240478515625f,0.3545999977827072324743085118825547397136688232421875f,0.09050001342594626851933981015463359653949737548828125f,0.125899997724592704884116756147705018520355224609375f,-0.041800005190819555789261130485101602971553802490234375f,-0.78830000085830675971010350622236728668212890625f,-0.73509996941089639221900142729282379150390625f,-0.20289999268054970116281765513122081756591796875f,0.5659999948143958903301609097979962825775146484375f,0.6725999826729296415805947617627680301666259765625f,0.34330000145435335401344900674303062260150909423828125f,-0.3354999909996987383919986314140260219573974609375f,1.36969998298883410825510509312152862548828125f,0.537200002211332350299244353664107620716094970703125f,0.213199982497096041100803631707094609737396240234375f,-0.519599988581239902174502276466228067874908447265625f,-0.3507000016450880774954157459433190524578094482421875f,0.6977000294208526298689321265555918216705322265625f,-0.57189999816417724787243059836328029632568359375f,-0.346200005012750722244163625873625278472900390625f,-0.086200022149085953060421161353588104248046875f,0.2020000307559968177884002216160297393798828125f,-0.23289999685287465780447746510617434978485107421875f,-0.4001999906778337390278466045856475830078125f,0.1704000165224073271019733510911464691162109375f,-0.26899999967217436136479591368697583675384521484375f,-0.4257999960899354618959478102624416351318359375f,-1.0390000078678127692910493351519107818603515625f,-0.38880000646114343254566847463138401508331298828125f,-0.083899975872039789237533113919198513031005859375f,-0.64889998481273636343757971189916133880615234375f,-0.577000010788440764741835664608515799045562744140625f,-0.405300004792213286464175325818359851837158203125f,0.9376999692201621883214102126657962799072265625f,0.44339998695850368903847993351519107818603515625f,0.630800005626678483849900658242404460906982421875f,0.138799982917308994956329115666449069976806640625f,0.0003999914884567079553789881174452602863311767578125f,0.0772000026583672394053792231716215610504150390625f,-0.6788000072062008172224523150362074375152587890625f,-0.40100001627206804943881479630363173782825469970703125f,-0.53510004868507365927143837325274944305419921875f,-1.02620002214908634385892582940869033336639404296875f,0.403399995452165427423096843995153903961181640625f,-1.072200012940168534214535611681640148162841796875f,-0.75750003755092620849609375f,0.160399993723630895914311622618697583675384521484375f,-0.663599987918138456421957016573287546634674072265625f,-0.33620000754594769887262373231351375579833984375f,-0.7412000021815299266592091953498311340808868408203125f,0.2477999860167503509700281938421539962291717529296875f,0.82899998715519895853276466368697583675384521484375f,-0.231000007331371381980034129810519516468048095703125f,-0.079399999207258264988240625825710594654083251953125f,0.4940999870389701786876912592560984194278717041015625f,0.1295999959200619944255095106200315058231353759765625f,-0.2873999994456768458661599652259610593318939208984375f,0.23799999140948024933805982072954066097736358642578125f,0.590999987587332409333384930505417287349700927734375f,-0.601999992385506832448527347878552973270416259765625f,-0.866099992276728070095259681693278253078460693359375f,0.2320000078082085082087360206060111522674560546875f,-0.0573999992966651717551940237171947956085205078125f,0.163599985682964188526966609060764312744140625f,-0.689200009125471080295710635255090892314910888671875f,-0.303000019758939753700133223901502788066864013671875f,0.1657999931097027257465015281923115253448486328125f,0.0300000102818014369177035405300557613372802734375f,0.7834000145256521108194647240452468395233154296875f,-0.0393999567389489158131254953332245349884033203125f,0.3348000219881537287136552549782209098339080810546875f,-0.7494000093400476902871787387994118034839630126953125f,-0.665400043493509230074778315611183643341064453125f,0.4360000255107880917648799368180334568023681640625f,0.0839999887943267964374172152020037174224853515625f,0.261199955987930110268280259333550930023193359375f,0.72020000334382050510839690105058252811431884765625f,-0.03159998010993003703106296597979962825775146484375f,-0.4890999883055684449573163874447345733642578125f,-0.22660000179111960960653959773480892181396484375f,-0.419200005772709882734261555015109479427337646484375f,0.499499954871833740099873466533608734607696533203125f,-0.4102000104963778870370560980518348515033721923828125f,-0.1549999894201754546685378954862244427204132080078125f,0.1181000118196013914939612732268869876861572265625f,0.370800019037723860293453981284983456134796142578125f,-0.01860000550150875398003336158581078052520751953125f,-0.6869000005483627546709612943232059478759765625f,-0.240699979591369128684164024889469146728515625f,-0.5075000096857547049467029864899814128875732421875f,-0.2914999807476998938682299922220408916473388671875f,-0.2686999741673474773051566444337368011474609375f,-0.359099995160102825053627384477294981479644775390625f,0.72079997835755360569010008475743234157562255859375f,-0.84000000014901132772138225845992565155029296875f,-0.1648999998927116461544528647209517657756805419921875f,0.554499998867511667555163512588478624820709228515625f,-0.3537000035226345318051244248636066913604736328125f,-0.1925000015646218809450829212437383830547332763671875f,0.4086000212967395572860596075770445168018341064453125f,0.00499999620020386004171086824499070644378662109375f,-0.19330000827163462684410433212178759276866912841796875f,-0.7981999754562976168159593726159073412418365478515625f,0.36239999814182510817772708833217620849609375f,-0.115700005258619731396407814827398397028446197509765625f,0.33100000464916234932388761080801486968994140625f,0.313799998712539718326297588646411895751953125f,0.004499995738267870137860882095992565155029296875f,-0.0665000093579291018386356881819665431976318359375f,-1.2152999812483784580763312987983226776123046875f,-0.431200000840425534676114693866111338138580322265625f,-0.130099991464614817004985525272786617279052734375f,0.6018999927997583654359914362430572509765625f,0.55040001071095456808279777760617434978485107421875f,-0.63510001575350738534098127274774014949798583984375f,-0.1022999510586259930278174579143524169921875f,-0.3197999965071678385442055514431558549404144287109375f,0.5383999780178072569469804875552654266357421875f,0.72319999508857790715410374104976654052734375f,0.146399990773200983085189363919198513031005859375f,-0.306899979090690777638883446343243122100830078125f,0.993299991917609759184415452182292938232421875f,0.25780000091791155369946864084340631961822509765625f,0.60359998225569722052341603557579219341278076171875f,-0.59019998777210691542904896778054535388946533203125f,-0.424500001847743890781572417836287058889865875244140625f,-0.01689999026656163749748884583823382854461669921875f,-0.70349998036027017178639653138816356658935546875f,-0.2420999891400337222790994928800500929355621337890625f,-0.7153999977469442939082000521011650562286376953125f,0.48869999271929298600980473565869033336639404296875f,0.7307999956429001020552504996885545551776885986328125f,0.1911999960720536240188494048197753727436065673828125f,-0.3541999915421012445904125343076884746551513671875f,0.1827999991297719617477923748083412647247314453125f,0.27259999355077724203511024825274944305419921875f,0.4477999808013439775322694913484156131744384765625f,0.0528000074744225944556319518596865236759185791015625f,0.5598000203490258730454343094606883823871612548828125f,0.41920001620352298488114684005267918109893798828125f,-0.24599999734759325775712568429298698902130126953125f,0.04079998252987859785889668273739516735076904296875f,-0.2082000004529955816678921109996736049652099609375f,-0.5836000056505203925638625150895677506923675537109375f,-0.2268000329256059099947151480591855943202972412109375f,0.2281999746739860057687110383994877338409423828125f,-0.56480001900792131674933216345380060374736785888671875f,0.04060000705122945863223549167742021381855010986328125f,0.5652000120610001232535068993456661701202392578125f,0.296400003886222673799011317896656692028045654296875f,0.244199986624717535477202545735053718090057373046875f,0.1069999896287914253889539395458996295928955078125f,0.059399988180398821668859454803168773651123046875f,-0.6408000090539456294891351717524230480194091796875f,-0.821099991494417480453194002620875835418701171875f,-0.65979999412298173666613365639932453632354736328125f,-0.31180001295804959937640887801535427570343017578125f,0.50969997613429995908518321812152862548828125f,-0.451200005757808497719452134333550930023193359375f,-0.408399994409084143853760906495153903961181640625f,-0.083199991512298510087930480949580669403076171875f,0.6521999907374380622826492981403134763240814208984375f,-0.6779999850392341986804467524052597582340240478515625f,-1.1594999802410601574820248060859739780426025390625f,0.362000005424022663902405838598497211933135986328125f,0.149200007188320160622652110760100185871124267578125f,0.1010000025629995690223950077779591083526611328125f,-0.2706000038474798419230182844330556690692901611328125f,0.0866000000774860279051381439785473048686981201171875f,0.48429999138861912211240223768982104957103729248046875f,-0.0659999994337556739054662102716974914073944091796875f,-0.3309999851286409278117162102716974914073944091796875f,-0.3113999882400033669682670733891427516937255859375f,-0.3930000053048132713229279033839702606201171875f,0.23850000360608103466830698380363173782825469970703125f,-0.14989999199509629335125282523222267627716064453125f,1.1800999504864222444666665978729724884033203125f,0.3938999816834927170106084304279647767543792724609375f,0.644799984884262045170544297434389591217041015625f,0.502399963533878946009281207807362079620361328125f,0.1750999877393246695334028117940761148929595947265625f,0.51270000751614563938574065105058252811431884765625f,0.5146999922275536931692840880714356899261474609375f,0.33800000888109205110509947189711965620517730712890625f,-0.39450001883506757849318091757595539093017578125f,-0.3500999944448466294488753192126750946044921875f,-0.01350000107288362727331332280300557613372802734375f,0.05619997133612653072987086488865315914154052734375f,-1.000199997085331915513961575925350189208984375f,-0.3574000095784664043918610332184471189975738525390625f,0.478800008770823293247076435363851487636566162109375f,-0.75200000326335469935656874440610408782958984375f,-0.21749999575316902866717327924561686813831329345703125f,0.2507999898314476894256586092524230480194091796875f,0.619599967086315928099793381989002227783203125f,0.237699985164403937432098246063105762004852294921875f,-0.9818999874353409040850237943232059478759765625f,-0.442499984204768992412937222979962825775146484375f,-0.3030000032186508374110189834027551114559173583984375f,0.402599992060661548975986079312860965728759765625f,-1.148799962353706405338016338646411895751953125f,-0.47819999553561221006248160847462713718414306640625f,-0.27219998626708985511868377216160297393798828125f,-0.9625999604702002443445962853729724884033203125f,-0.39839998785257346725074967253021895885467529296875f,-1.14839996535182020664933588705025613307952880859375f,0.3726000139653677933893050067126750946044921875f,-0.1076999985754489852496362800593487918376922607421875f,0.2846000135779380713074715458787977695465087890625f,0.64739998618364325011498294770717620849609375f,0.192599995636939969045897669275291264057159423828125f,0.5035999935805797189658505885745398700237274169921875f,-0.29380000385344029467660220689140260219573974609375f,-0.08899999956041571014253577232011593878269195556640625f,-0.40329998133778577340535775874741375446319580078125f,-0.36089996415972791510284878313541412353515625f,-0.2007999865531922001338216432486660778522491455078125f,0.323399993515014472222901531495153903961181640625f,0.224200001972914009229498333297669887542724609375f,0.5370000130236149971096892841160297393798828125f,0.459000009208917703062979853712022304534912109375f,-0.604299963486194968709241948090493679046630859375f,-0.5860000165700913754562861868180334568023681640625f,0.5600000005960463766996326739899814128875732421875f,1.25100002193450876575298025272786617279052734375f,-0.3008000154614449694889799502561800181865692138671875f,0.9284000265955925090821665435214526951313018798828125f,0.8456000481784347044822425232268869876861572265625f,0.127000003188848520352394189103506505489349365234375f,-0.609000017553567918326962171704508364200592041015625f,-0.7525000140070918774881647550500929355621337890625f,0.7660000016689298973915356327779591083526611328125f,0.3086000205516814531847558100707828998565673828125f,-0.6365999763846392767163706594146788120269775390625f,-0.10579997493028603372522411518730223178863525390625f,0.91780000270605111012400811887346208095550537109375f,0.133900017595291132010970613919198513031005859375f,0.332400008386373624347243094234727323055267333984375f,-0.744200000482797552336933222250081598758697509765625f,-0.5330000282526015098483185283839702606201171875f,-0.2290000105500222815635424922220408916473388671875f,-0.2190000200867654456260424922220408916473388671875f,-0.2077000008106235640070735826157033443450927734375f,0.3484000251054766295055742375552654266357421875f,-0.05639999852180466177742346189916133880615234375f,0.23419996188879022014361908077262341976165771484375f,-0.4742000207856300608710853339289315044879913330078125f,0.5383999969050285994143223433638922870159149169921875f,0.7833000193171202329978086709161289036273956298828125f,-0.1053999995723364779820485637173987925052642822265625f,-0.403999984659254351981871877796947956085205078125f,-0.0514999998398122382781139094731770455837249755859375f,0.3170000158548358371035646996460855007171630859375f,-0.6889999858140942734507916611619293689727783203125f,-0.9699000144958489499913412146270275115966796875f,-0.16900000935792913736577247618697583675384521484375f,-0.79459999390840518884715493186376988887786865234375f,-0.0009000006675723426496915635652840137481689453125f,0.74340002570152297067807012354023754596710205078125f,0.46300000262260443406603371840901672840118408203125f,-0.6366999943733215872043729177676141262054443359375f,0.39100000703334814033951261080801486968994140625f,-0.4047999899506569132512368014431558549404144287109375f,-0.60209998466968528418874484486877918243408203125f,1.06569998972415902471766457892954349517822265625f,0.556900002187490361649224723805673420429229736328125f,1.31400002098083490409408113919198513031005859375f,-0.3520000367164612953274627216160297393798828125f,0.03299999189376823238717406638897955417633056640625f,0.86800002095103234722728302585892379283905029296875f,0.2959999606907359748220187611877918243408203125f,0.4262000068008899855698246028623543679714202880859375f,-0.8635000055432318077919262577779591083526611328125f,-0.31449997621774627987178973853588104248046875f,-0.271500004142522843864071546704508364200592041015625f,-0.520399991488456947763552307151257991790771484375f,1.2915999754905698182483320124447345733642578125f,0.52510000204443940674536861479282379150390625f,0.019799977135658508586857351474463939666748046875f,0.513600002074241501759388484060764312744140625f,0.57800001230835906795846312888897955417633056640625f,-0.4618999972701072209702033433131873607635498046875f,-0.0906999655842781749015557579696178436279296875f,-0.335999998539686206555643366300500929355621337890625f,-0.355400007581710752901926753111183643341064453125f,-0.5112000048637383997629513032734394073486328125f,-0.186400008505582803763900301419198513031005859375f,0.4783999887466432454630194115452468395233154296875f,0.49160001542568210197714506648480892181396484375f,0.0288000005006789905337427626363933086395263671875f,0.75069999983832225165514273612643592059612274169921875f,-0.243500043224543105679913423955440521240234375f,0.1179999926201999727037872389701078645884990692138671875f,-0.069699981424212520408900672919116914272308349609375f,-0.448699963773787846577079108101315796375274658203125f,-0.29959998794794084897574748538318090140819549560546875f,-1.05280000181198118980319122783839702606201171875f,-0.1113999377250678435302688740193843841552734375f,-0.23979998086094855835881389793939888477325439453125f,0.01340003941059109138222993351519107818603515625f,-0.8175999646425253786219400353729724884033203125f,-0.59259999817013753187211477779783308506011962890625f,-0.5492000128507612544126459397375583648681640625f,1.368999990582466352861956693232059478759765625f,0.624200000780820829504591529257595539093017578125f,-0.2855999809741973649579449556767940521240234375f,-0.4370000215172762381143911625258624553680419921875f,-0.082000002667307836379251284597557969391345977783203125f,0.269600012162327817843987531887250952422618865966796875f,0.268600014181435309268408673233352601528167724609375f,0.812800018203258645144160254858434200286865234375f,0.210600018823146939439538982696831226348876953125f,-0.984399981772899224097272963263094425201416015625f,-0.52779999138116817647414791281335055828094482421875f,0.76620000396966947420196447637863457202911376953125f,0.54930000163316794470347304013557732105255126953125f,-0.68320001609921465757224723347462713718414306640625f,-0.00940000308156019315219964482821524143218994140625f,0.9119999887347223221922831726260483264923095703125f,-0.53080001069307325423096699523739516735076904296875f,-0.46279999734163279612175756483338773250579833984375f,0.552199975836277445750965853221714496612548828125f,0.3599999865889547123742886469699442386627197265625f,-1.1318000064015389938276712200604379177093505859375f,-0.54200001913309137080432265065610408782958984375f,0.50140002743005762653183410293422639369964599609375f,0.21439998832941054018874638131819665431976318359375f,0.10909995552301410270956694148480892181396484375f,0.105600002133846293528307569431490264832973480224609375f,-0.026199998828768722436421967358910478651523590087890625f,-0.07280000099241734634603062659152783453464508056640625f,0.1008000099480153011199945467524230480194091796875f,0.14040001130700119347238796763122081756591796875f,0.50439997566342409385242717689834535121917724609375f,-0.1657999817848203605308299302123486995697021484375f,0.7915999993324280836759498924948275089263916015625f,0.3098999945282940871038590557873249053955078125f,0.69360003396272684739187752711586654186248779296875f,-0.18780000449419009811435898882336914539337158203125f,-0.25360004975795735759902527206577360630035400390625f,0.07200000117719174141939220135100185871124267578125f,0.024400000473856930938154619070701301097869873046875f,-0.54879999100118848076590438722632825374603271484375f,-0.49100001224875455818619229830801486968994140625f,0.2305999998986720811444683931767940521240234375f,0.441700015085935593361909923260100185871124267578125f,0.07259999511539927397052451851777732372283935546875f,-0.133400011917948813078282910282723605632781982421875f,-0.558499984197319054857189257745631039142608642578125f,0.096799992841482218608462062547914683818817138671875f,-0.24600000301003444036496148328296840190887451171875f,0.2850000079348682380242507861112244427204132080078125f,0.095799993817507811399991624057292938232421875f,0.122000005014240742173825537975062616169452667236328125f,-0.94659999873638156486777006648480892181396484375f,1.2602000055789943644413142465054988861083984375f,0.52170001001954080521727519226260483264923095703125f,0.3897999616384506538224741234444081783294677734375f,-0.17920000033378613579770899377763271331787109375f,0.29200001481175430484427124611102044582366943359375f,0.88769999544620503684200230054557323455810546875f,-0.9319000053167343367022112943232059478759765625f,-0.33600000092387194428056318429298698902130126953125f,0.536599998736381422759222914464771747589111328125f,-1.06379997606277498789495439268648624420166015625f,-0.550199994552135418501848107553087174892425537109375f,-0.41539996883869179100656765513122081756591796875f,-0.328599984788894516896107234060764312744140625f,-0.5919000129163265455645159818232059478759765625f,0.352599965238571400050204829312860965728759765625f,0.899599990630150614379090256989002227783203125f,0.27649999237060551848799150320701301097869873046875f,-0.043899998186528677024398348294198513031005859375f,0.13010000074058769570228832890279591083526611328125f,0.023199999707937235537968945209286175668239593505859375f,0.6968999892234801762924689683131873607635498046875f,0.292100001060962455312619567848742008209228515625f,0.0527000043869019219044957935693673789501190185546875f,0.23919995801448834527036524377763271331787109375f,0.3501999936580659777973778545856475830078125f,0.4489000095486641583875098149292171001434326171875f,0.0101999769687655117422764305956661701202392578125f,0.86260001888275184001031448133289813995361328125f,0.1092999982059001951029131305404007434844970703125f,-0.6944000120222568739336566068232059478759765625f,0.07600000666081907496618441655300557613372802734375f,0.116600006037950565396243973737000487744808197021484375f,0.32770000140666954990820158855058252811431884765625f,-0.518100008690357327623132732696831226348876953125f,-0.048299998176097869162504139239899814128875732421875f,-0.68839999455809586237364783301018178462982177734375f,-0.03700000326335395328669619630090892314910888671875f,0.010199994105100584906864469303400255739688873291015625f,-0.4310999779343607229975532391108572483062744140625f,-0.1475000250339508767183360760100185871124267578125f,-0.38999999403953555798807428800500929355621337890625f,0.66339999232292168329649939551018178462982177734375f,1.3157999647974971679786904132924973964691162109375f,0.1882000123739244745735277319909073412418365478515625f,0.4362000245332720016477878743899054825305938720703125f,0.22210001409947910389064418268389999866485595703125f,0.61640000731348987272895101341418921947479248046875f,0.56000000521540638231954289949499070644378662109375f,-0.5601999587893491394652301096357405185699462890625f,0.8282000147581101145988213829696178436279296875f,-0.07339999709129330085488618351519107818603515625f,-1.13269998203516042423188991961069405078887939453125f,0.743799990963935453436306488583795726299285888671875f,0.060800001454353047591894210199825465679168701171875f,-0.386499988794327009600237943232059478759765625f,0.090200003343820611689807265065610408782958984375f,0.16320000358223918368594240746460855007171630859375f,0.26910000536739797638574600568972527980804443359375f,-0.5612000130593777225129770158673636615276336669921875f,0.6770000130236148994100631171022541821002960205078125f,1.0402000248014928729389794170856475830078125f,-0.3520000161528589188719706726260483264923095703125f,0.9378000128388404021961832768283784389495849609375f,0.5516000455260279267122314195148646831512451171875f,-0.171200004416704132381710223853588104248046875f,-0.742000007659196825215985882095992565155029296875f,-0.0700000205636026606725863530300557613372802734375f,-0.735200026589632127382856197073124349117279052734375f,-0.140399990445375433267827247618697583675384521484375f,0.7104000209927558984190909541212022304534912109375f,0.007600001075863727084680476764333434402942657470703125f,0.401999989107251043396473733082530088722705841064453125f,0.262999999530613148834845560486428439617156982421875f,0.322199990439414829523911976139061152935028076171875f,-0.562399993038177559157020368729718029499053955078125f,-0.55570000015497189593816074193455278873443603515625f,0.8846000333964827433419486624188721179962158203125f,0.3004000020682813243411146686412394046783447265625f,-0.758099995577335761254289536736905574798583984375f,-0.4138000131666661474127977271564304828643798828125f,-0.1205999948322773462905388441868126392364501953125f,0.74109998091459328151131558115594089031219482421875f,-0.216600007081031886713162748492322862148284912109375f,-0.151400004631280893363509676419198513031005859375f,1.478799960565567062076297588646411895751953125f,0.0155999696493154260679148137569427490234375f,0.1512999952852727147956102271564304828643798828125f,0.32990001047253603161379942321218550205230712890625f,0.6081000039219848218863262445665895938873291015625f,0.32110001220703121571631299957516603171825408935546875f,-0.518699982213973953548702411353588104248046875f,1.25849996948242193184341886080801486968994140625f,0.642699994850158606851664444548077881336212158203125f,-1.1019999866485594708365169935859739780426025390625f,1.06660000350475314689902006648480892181396484375f,0.353199998515844271196328918449580669403076171875f,-0.1711999837040902860962887643836438655853271484375f,1.10299995344877288516727276146411895751953125f,0.313099983060360020914458800689317286014556884765625f,0.3784000134825706851415816345252096652984619140625f,0.2725999916136263578891885117627680301666259765625f,0.09909999694824211946553305097040720283985137939453125f,1.03330000845789893304527140571735799312591552734375f,-0.2499000256717209822454606182873249053955078125f,0.1506000042200088540056412966805510222911834716796875f,0.454600002551078574697385192848742008209228515625f,0.20219995646476718320627696812152862548828125f,0.46010000353455549060299745178781449794769287109375f,-0.085099989676475473743266775272786617279052734375f,-0.78909997012615207268027006648480892181396484375f,-0.547300001871585894974714392446912825107574462890625f,0.6125000289082525029016323969699442386627197265625f,-1.119499997079373088126885704696178436279296875f,-0.411699996161460912702523273765109479427337646484375f,0.3037999874994159466012888515251688659191131591796875f,0.3217999709740286817805099417455494403839111328125f,0.239399990825355069024027443447266705334186553955078125f,-0.51799998265504854089158470742404460906982421875f,0.02260000845193843588276649825274944305419921875f,-0.211300001096725420524080618633888661861419677734375f,-0.62470000057220431699533946812152862548828125f,1.4495000033378602211087127216160297393798828125f,0.57140000001192081668932587490417063236236572265625f,0.8004999948740003645752949523739516735076904296875f,-0.63050000202655809289353783242404460906982421875f,-0.30929999850392331239845589152537286281585693359375f,-0.4252000136256217643904165015555918216705322265625f,-0.601200016635656364627493530861102044582366943359375f,0.42000001564621936012144942651502788066864013671875f,-0.04320000447630878426252820645458996295928955078125f,0.1279999847412103264332472463138401508331298828125f,0.31040000940710310572967500775121152400970458984375f,-0.1812000004306435207990233493546838872134685516357421875f,-0.36120000953897835049843934029922820627689361572265625f,0.1862000005424022663902405838598497211933135986328125f,0.0074000089824199921650915712234564125537872314453125f,-0.0388999946475028224313064129091799259185791015625f,0.7266000276446344940950439195148646831512451171875f,-0.1254000016212462043085906771011650562286376953125f,-0.959500009149313104472867053118534386157989501953125f,0.43460001029968253760671359486877918243408203125f,0.393599982404708725880482234060764312744140625f,-0.5602999884009367548287627869285643100738525390625f,0.364399979612230862091593053264659829437732696533203125f,-0.8858000128537419914920292285387404263019561767578125f,-0.931000004537403835769282522960565984249114990234375f,-0.06239999899864212551392483874224126338958740234375f,0.4045999924182890339352525188587605953216552734375f,0.86079997336566460575113524100743234157562255859375f,0.284000008314848173540667630732059478759765625f,-0.391799984198808459723295527510344982147216796875f,-1.1339999864101404369876036071218550205230712890625f,-0.146599995756149237280396846472285687923431396484375f,0.198599997305870079600254030083306133747100830078125f,0.39829999176859853804444355773739516735076904296875f,0.3024000077903270522483580862171947956085205078125f,0.53900000473856923832727261469699442386627197265625f,-0.48999998293817004224592892569489777088165283203125f,-0.67580001754760754550943602225743234157562255859375f,0.23299999487400047115670531638897955417633056640625f,0.13130001689195669456466930569149553775787353515625f,0.4844000039756297670834328528144396841526031494140625f,-0.8900000114738941814351846915087662637233734130859375f,-0.949400025731325314382047508843243122100830078125f,0.17500000342726718116637130151502788066864013671875f,-0.24039999342560758321951652760617434978485107421875f,-0.900899986809491792882909066975116729736328125f,-0.75040000802874562424449322861619293689727783203125f,0.24739999408125878943565112422220408916473388671875f,0.943999995201826092028341008699499070644378662109375f,-0.022999993532895945946847859886474907398223876953125f,0.20089999932646751545917140902020037174224853515625f,0.36990001211166401162699912674725055694580078125f,0.5970999965906145234839641489088535308837890625f,0.07389999017715442874987274990417063236236572265625f,-0.31130000156238668562735938394325785338878631591796875f,0.29510001048222189723446717835031449794769287109375f,0.0181000000663101567877077968660159967839717864990234375f,0.03450001093745225233533346909098327159881591796875f,0.275499998301267368105982313863933086395263671875f,0.064700001168251031202771628159098327159881591796875f,-0.91440000829696632678178502828814089298248291015625f,0.895000021755695485126125277020037174224853515625f,0.263200002837181035175717624952085316181182861328125f,-0.584000013232230941184752737171947956085205078125f,0.872199971961975961676216684281826019287109375f,0.47609998836517331000095509807579219341278076171875f,-0.9561999859765173415127037515048868954181671142578125f,-0.1794000036403531339601613581180572509765625f,-0.366300014247000149669730717505444772541522979736328125f,1.0606999775052070589254071819595992565155029296875f,0.2722999650657182968416236690245568752288818359375f,0.13479998950362215293097278845380060374736785888671875f,-1.319899996614456227916889474727213382720947265625f,0.03890004560947346590182860381901264190673828125f,-0.022399980968236921086145230219699442386627197265625f,0.434599991524219575467213871888816356658935546875f,-0.038800004076957606002906686626374721527099609375f,0.0359999994337558693047185442992486059665679931640625f,-0.5698000209450722053361459984444081783294677734375f,0.7881999839127065143884465214796364307403564453125f,0.52710000538229950794999467689194716513156890869140625f,-0.1688000195741654607672899146564304828643798828125f,-0.81460000553131095557546359486877918243408203125f,-0.0555999936401843086741791921667754650115966796875f,-0.92409999888539307022483626496978104114532470703125f,-0.0076999744355674692997126840054988861083984375f,-0.3317000061452388681715319762588478624820709228515625f,-1.080500001430511503031084430404007434844970703125f,0.007300026905536327603840618394315242767333984375f,0.0160000115036964540848885008017532527446746826171875f,0.86849999034404756770300082280300557613372802734375f,-0.843900003588199609794173738919198513031005859375f,-0.184399998760223393645674150320701301097869873046875f,0.436399997329711908378158113919198513031005859375f,-0.0114000245988368220650954754091799259185791015625f,-0.09400000250339513030439775320701301097869873046875f,0.02599999481439585480302412179298698902130126953125f,0.716199990409612130548566710785962641239166259765625f,0.482199997144937331228220500634051859378814697265625f,0.233200003582239023813826861442066729068756103515625f,-0.489699976134300385410824674181640148162841796875f,-0.08719999237656583801481247064657509326934814453125f,0.61480000500082976344629059894941747188568115234375f,0.2296000117897989412085735239088535308837890625f,0.445399977928399692927996511571109294891357421875f,-0.587600005620718235377353266812860965728759765625f,-1.111299963843823235265517723746597766876220703125f,-0.73720001219511033241360564716160297393798828125f,-0.49660001810789111686972319148480892181396484375f,0.7221999728560444253844252671115100383758544921875f,0.11180000237822529385312009253539144992828369140625f,-0.01699999216198921914156017010100185871124267578125f,0.42309997695088430447185601224191486835479736328125f,0.039200003202259470214130487875081598758697509765625f,0.4079999986365436814850227165152318775653839111328125f,0.7295999890841542434571920239250175654888153076171875f,-0.2465999830901619560563631239347159862518310546875f,0.7702000158607962276846592430956661701202392578125f,0.202300011110306598993702209554612636566162109375f,0.48279998407959912043452277430333197116851806640625f,-0.24799998965859426647284635691903531551361083984375f,-0.890999963670969119533538105315528810024261474609375f,0.1404000056445600108645521686412394046783447265625f,0.722200018900633011753598111681640148162841796875f,-0.0201999488055708553702061180956661701202392578125f,-0.158600005203485494575943448580801486968994140625f,-0.2032000017940998048970868694595992565155029296875f,-0.24769999768137918039201395004056394100189208984375f,0.538400011694431412934136460535228252410888671875f,0.033400016462802994965386460535228252410888671875f,0.2055999997496602560431711026467382907867431640625f,-0.152400001457333456045262209954671561717987060546875f,-0.44999999471008766960267166723497211933135986328125f,-0.089800010104477134831313378526829183101654052734375f,0.16400000295042982401128028868697583675384521484375f,0.35899999579787245096440528868697583675384521484375f,-0.306799994480609683478178340010344982147216796875f,0.65160000156760222722596154198981821537017822265625f,-0.4699999999999999733546474089962430298328399658203125f,0.89940001485347753629184808232821524143218994140625f,-0.057800015819072569911440950818359851837158203125f,-0.112600011283159329167347095790319144725799560546875f,-0.4496999901413918365733479731716215610504150390625f,0.8332999725461007756166509352624416351318359375f,0.369999994039535540224505894002504646778106689453125f,0.190499985933303861429521930404007434844970703125f,0.33209998884201041846608859486877918243408203125f,0.362300006043910993724921354441903531551361083984375f,-1.19169995861053479302427149377763271331787109375f,0.1376999876976015002583153545856475830078125f,-0.1128000134348869920586366788484156131744384765625f,0.668999998927116479308097041212022304534912109375f,-1.3765999838352200868030195124447345733642578125f,-0.657700006026029626582385390065610408782958984375f,0.6130999946832655922435151296667754650115966796875f,0.188499995410442267029793583787977695465087890625f,-0.0401000002562998947297501217690296471118927001953125f,0.934299996045231839758571368292905390262603759765625f,0.412699970747530731074448340223170816898345947265625f,0.1854999914467334587442337578977458178997039794921875f,0.0618999707460403669756487943232059478759765625f,0.2069000118732446935609914362430572509765625f,0.426300021809339568790164776146411895751953125f,-0.92309996786117576306196497171185910701751708984375f,0.454899987375736003514248295687139034271240234375f,0.098699996668100364871634155861102044582366943359375f,-0.046099974656105047188248136080801486968994140625f,0.721899986541270521911428659223020076751708984375f,0.5378000076234339399405826043221168220043182373046875f,-0.3773000033617019965959116234444081783294677734375f,-0.399899987375736287731342599727213382720947265625f,-0.1493999960780143521077434343169443309307098388671875f,0.518600030684471136055435636080801486968994140625f,0.8325999962329859727105940692126750946044921875f,0.1494999972283839184683529310859739780426025390625f,-0.13679999522566799186051866854541003704071044921875f,-0.55399996949732344120320703950710594654083251953125f,-0.339199992287158968995441910010413266718387603759765625f,-0.371100022339820867500748136080801486968994140625f,0.471699981856346539643709547817707061767578125f,0.33739999944567689027508095023222267627716064453125f,-1.778299969267845170861619408242404460906982421875f,-1.30889998809099239451825269497931003570556640625f,0.1251999979794025108503774390555918216705322265625f,-0.1303999967038631524474112666212022304534912109375f,-0.50419999347925159582928245072253048419952392578125f,0.412399998104572063084560795687139034271240234375f,0.5935999910473821472578492830507457256317138671875f,-0.2497999677479274538427489460445940494537353515625f,0.231800001148879442069983269902877509593963623046875f,0.4381999963924287921912537058233283460140228271484375f,-0.4396999727256598600177994740079157054424285888671875f,-0.29659999933242797709453952847979962825775146484375f,0.28259999593496321068641918827779591083526611328125f,0.604899987822771212364614257239736616611480712890625f,-0.72040000862479214305267305462621152400970458984375f,0.07359997182488431377578308456577360630035400390625f,0.2777999982357020058998386957682669162750244140625f,0.076399998372793209711062445421703159809112548828125f,-0.277799995404481858685130646335892379283905029296875f,-0.35220000057220435252247625612653791904449462890625f,0.137600011208653671967994114311295561492443084716796875f,-0.6156000005692241483501447874004952609539031982421875f,-0.531200020174682041584901526221074163913726806640625f,-0.077399997657537245032699502189643681049346923828125f,-0.758999995797872362146563318674452602863311767578125f,-0.2979000120520591821104972041212022304534912109375f,0.179400009861588605186710765337920747697353363037109375f,-0.203000001654028816577834959389292635023593902587890625f,-0.326700009237229860303841633140109479427337646484375f,-0.264600008809566578094063515891321003437042236328125f,-0.304200010168552470446456936770118772983551025390625f,0.25609998791813826102270468254573643207550048828125f,0.53200000691413862341505591757595539093017578125f,0.427799998235702361171206575818359851837158203125f,-0.5903999980449672335680588730610907077789306640625f,-0.040000005215406364555974505492486059665679931640625f,-0.753799993199109952257686018128879368305206298828125f,-0.2360000155270096655613087932579219341278076171875f,0.41919999750256520254509950973442755639553070068359375f,-0.97300002333521862230014676242717541754245758056640625f,-0.99880002963244951530441539944149553775787353515625f,0.093800006759166887349010721663944423198699951171875f,0.802800025355816071481740436865948140621185302734375f,0.1821000441253186608037140103988349437713623046875f,-0.60620002512931847604704671539366245269775390625f,-0.11169999109506623113929890678264200687408447265625f,-0.5925000029802323098238048260100185871124267578125f,0.04710000731945029883718234486877918243408203125f,0.01970000444650654713996118516661226749420166015625f,-0.207599992060661264758891775272786617279052734375f,1.009599949204922353374058729968965053558349609375f,0.5617999846458434642926249580341391265392303466796875f,-0.524400003305077522242072518565692007541656494140625f,0.616599976496398749503669023397378623485565185546875f,0.43039999685287477770856412462308071553707122802734375f,0.62779998333454134495923426584340631961822509765625f,-0.181200009930133631996795884333550930023193359375f,0.020199999320507089350940077565610408782958984375f,0.0226999781608583361958153545856475830078125f,-0.98170000331401752191595733165740966796875f,-0.1987000060558319347592259873636066913604736328125f,-0.397099997484684053006276371888816356658935546875f,-0.1170999849677087922827922739088535308837890625f,-0.11719999535679814783861729665659368038177490234375f,-0.5654000008389352682769413149799220263957977294921875f,-0.2845999565437438150183879770338535308837890625f,-0.304799993042647876517747818070347420871257781982421875f,-0.3404999826550483277287639793939888477325439453125f,-0.6830999514698987695737741887569427490234375f,-0.498899992114305579349320396431721746921539306640625f,-0.3242000026807188017841099281213246285915374755859375f,0.090000017546117305755615234375f,-0.039799997811019449667213621069095097482204437255859375f,0.250800002944469468957322533242404460906982421875f,-0.584599997782706992666135192848742008209228515625f,-0.3218000062525272486624317025416530668735504150390625f,-0.582400029397010854381733224727213382720947265625f,1.03719997136592922970521613024175167083740234375f,0.532099997633695753762594904401339590549468994140625f,-0.2418999871373177512623442453332245349884033203125f,-0.619099964165688021466849022544920444488525390625f,-0.4933999939620494767922309620189480483531951904296875f,-1.010000004991888911121122873737476766109466552734375f,-0.1671999848514786890518735162913799285888671875f,-0.40030001350939270476914089158526621758937835693359375f,0.1900000181794165854398670489899814128875732421875f,0.728599987769126045122902723960578441619873046875f,0.5503000119447707394471080988296307623386383056640625f,0.36740001389980303514448678470216691493988037109375f,-0.0788000267267232601398063707165420055389404296875f,0.2094000117242338365741716188495047390460968017578125f,0.4126000246942045368570006758091039955615997314453125f,0.22630001480579409189886064268648624420166015625f,-0.35139999747872341373522431240417063236236572265625f,0.57259999310374265490963807678781449794769287109375f,0.16190002766847566562091742525808513164520263671875f,0.087000013172626733393144604633562266826629638671875f,0.581400025045871959861187860951758921146392822265625f,0.12750001646578379421725912834517657756805419921875f,-0.12240000331997880067547157523222267627716064453125f,-0.15720002039074909561122694867663085460662841796875f,-0.600000001341104649554836214520037174224853515625f,-0.77540001886934051267274981000809930264949798583984375f,0.2183999944463371478686752880093990825116634368896484375f,0.7919000082410871943494612423819489777088165283203125f,0.62399999564886077241254724867758341133594512939453125f,-0.30559998961687095953010384619119577109813690185546875f,-0.76759998796284190092364951851777732372283935546875f,0.6051999854624268149194676880142651498317718505859375f,-0.8050000034272672966295658625313080847263336181640625f,-1.127799997192621361818964942358434200286865234375f,-0.5504000169694425181887709186412394046783447265625f,0.3400000055134295706693592364899814128875732421875f,1.16280000389814386352327346685342490673065185546875f,0.064600005382299485745534184388816356658935546875f,-0.822000005871057481954267132095992565155029296875f,-0.1409000504374500195581276784650981426239013671875f,0.6036000145167113029032179838395677506923675537109375f,-0.0657999934077262427223331542336381971836090087890625f,-1.15340000167339962189316793228499591350555419921875f,-0.53560000615715974703334723017178475856781005859375f,0.11719999714493756215460962266661226749420166015625f,0.436600006037950461035279658972285687923431396484375f,-0.35899999907612778482501880716881714761257171630859375f,-0.35559999483227711092325762365362606942653656005859375f,0.01809999460876010601850794046185910701751708984375f,0.29260000874996183739540356327779591083526611328125f,0.00239999125003809155032286071218550205230712890625f,0.3462999814271920939745541545562446117401123046875f,-0.402800002855062455608958771335892379283905029296875f,0.202200001317262678668384978664107620716094970703125f,0.69219999252557773417038333718664944171905517578125f,0.3759999936223028527138012577779591083526611328125f,-0.624900011664629051466590681229718029499053955078125f,0.34810001360774034395717535517178475856781005859375f,0.622700003045796535161571227945387363433837890625f,0.3105000112652778199162639793939888477325439453125f,-0.0171000043392179890133775188587605953216552734375f,-0.09210000910758964209890109486877918243408203125f,0.26650000205636015238042091368697583675384521484375f,0.140699996280670092119180480949580669403076171875f,0.5553000058352939305450490792281925678253173828125f,0.40470001741051675470117743316222913563251495361328125f,-0.831300023150443934127906686626374721527099609375f,-0.344099996650218287186362431384623050689697265625f,-0.0567000030159949819363873757538385689258575439453125f,0.65929999636188141298731579809100367128849029541015625f,-0.914500013116747112462689983658492565155029296875f,-0.1896000006698071727040399991892627440392971038818359375f,-0.687499979808926742208541327272541821002960205078125f,0.6316999956399200755186029709875583648681640625f,0.08249999754130843410138140825438313186168670654296875f,-1.11349998006224648605666516232304275035858154296875f,0.21529999927878407106618396937847137451171875f,-0.2128000101566314139489577428321354091167449951171875f,1.2399000079393385220782874966971576213836669921875f,-0.46490002582073230996684287674725055694580078125f,-0.094500003039836855123212444595992565155029296875f,-0.059500014066696138570478069595992565155029296875f,-0.506099964523315293263294734060764312744140625f,-0.52339999917745583246642127051018178462982177734375f,-1.2349000229895115143818884462234564125537872314453125f,0.74030001574456694157788660959340631961822509765625f,0.29620001019090425042890046825050376355648040771484375f,0.31349999809265138850378207280300557613372802734375f,-0.665299994957447182741816504858434200286865234375f,-0.138099996322393447911736075184307992458343505859375f,-0.101599996724724750407631290727294981479644775390625f,-0.7805999609321361276670359075069427490234375f,-0.35009998796284202970952037503593601286411285400390625f,0.59360000743865981576163903810083866119384765625f,0.46139998421669048411786206997931003570556640625f,0.036699986028671194304706659750081598758697509765625f,0.7030999925971030961591168306767940521240234375f,-0.38730000306367884377323207445442676544189453125f,-0.078199997621774652856174725457094609737396240234375f,0.594600014472007387666963040828704833984375f,-1.3676000224590296738824690692126750946044921875f,0.003799999532103692789775095661752857267856597900390625f,-0.470399984112381763790011746095842681825160980224609375f,-0.294899999035894477827923765289597213268280029296875f,0.0659999985396861887920749722979962825775146484375f,-0.7798000106632709815812631859444081783294677734375f,-0.5182000254869461031148603069595992565155029296875f,-0.42140000895261753299791962490417063236236572265625f,-0.29259998282194121799193453625775873661041259765625f,-0.2798999930381764755793483345769345760345458984375f,0.6312000181257726438843747018836438655853271484375f,0.3852000069200993692675183410756289958953857421875f,-0.656799969148636364479898475110530853271484375f,-0.337000012427568496065077852108515799045562744140625f,-0.160999994069337848401346491300500929355621337890625f,0.4218999978661539529412038973532617092132568359375f,0.164000001460313793444356633699499070644378662109375f,-0.401800006550550481421169024542905390262603759765625f,-0.320400010859966499765505432151257991790771484375f,-0.057399999743700025334192105219699442386627197265625f,-0.092399999892711637272668667719699442386627197265625f,0.1848999867051841494713926294934935867786407470703125f,-0.51399999892711623061813952517695724964141845703125f,0.75100001657009141098342297482304275035858154296875f,1.453000011265277890970537555404007434844970703125f,0.279600003147125075741996624856255948543548583984375f,0.384199987220763983231108795735053718090057373046875f,-0.174199997502565651075201458297669887542724609375f,0.172399996316432879694957591709680855274200439453125f,-0.112799998086690944631982347345910966396331787109375f,-0.3781999911397695424142284537083469331264495849609375f,-0.0755999984830617588471568524255417287349700927734375f,0.0785999979019165184723760830820538103580474853515625f,-0.11779999752789727107682438145275227725505828857421875f,0.088800004822015665695289499126374721527099609375f,0.533799997669458292648414499126374721527099609375f,0.625199988591671029780627577565610408782958984375f,0.3060000043511390543926609097979962825775146484375f,-0.5212000039696693676205541123636066913604736328125f,-0.45020002435445771737931863754056394100189208984375f,-0.07599998095631566030760950525291264057159423828125f,0.87940002066493061505525474785827100276947021484375f,0.406000018358231073989372816868126392364501953125f,0.29360000773668293305718179908581078052520751953125f,-0.18109998091459278413140054908581078052520751953125f,-0.3325999837532640501791547649190761148929595947265625f,-0.35679996344894249205026426352560520172119140625f,-0.278299987484514732560825223117717541754245758056640625f,-0.562300003957748373295544297434389591217041015625f,-0.50629997189044928518342203460633754730224609375f,-0.439800002020597435858917378936894237995147705078125f,-0.53180000669956228875889792107045650482177734375f,-0.8527999606847771474349428899586200714111328125f,-0.28219997777342786804410934564657509326934814453125f,-0.61889998362064346792976721189916133880615234375f,1.1466999943733213740415521897375583648681640625f,0.54130000079870210782928552362136542797088623046875f,0.4193000081896780528722956660203635692596435546875f,-0.574099996054172834192286245524883270263671875f,-0.268699997562170000975356742856092751026153564453125f,-0.2996999955058097242499570711515843868255615234375f,-0.07449998873472196692091529257595539093017578125f,-0.1812000041186809795590306748636066913604736328125f,0.228399995005130751479782702517695724964141845703125f,-0.1345999964416026983826668583787977695465087890625f,-0.1064999997466802295065235739457421004772186279296875f,1.592000012874603243062665569595992565155029296875f,-0.092000015854836192374932579696178436279296875f,0.15550000545382491878854125388897955417633056640625f,-0.1450000163912772421781482989899814128875732421875f,0.670999966502190403616623370908200740814208984375f,0.3690999890506267977485777009860612452030181884765625f,-0.406099994027614652480906443088315427303314208984375f,0.1398999988496303359397643362171947956085205078125f,0.027299999561905874401190885691903531551361083984375f,-0.2001000091969966643290490537765435874462127685546875f,-0.06489999683797353924319395446218550205230712890625f,0.04880000292211772450201578976702876389026641845703125f,0.80469998507499695961087127216160297393798828125f,0.566699965763092450288240797817707061767578125f,0.30099999228119855843033292330801486968994140625f,-0.3910000052452087260235202847979962825775146484375f,0.03740001628398914590434287674725055694580078125f,-0.2030000020265578797307171043939888477325439453125f,0.397699989485740701411486952565610408782958984375f,0.55330000205039908678372739814221858978271484375f,0.521200017529726000731216117856092751026153564453125f,-0.855000019073486328125f,-0.085800026488303871019525104202330112457275390625f,0.506800008338689877973592956550419330596923828125f,-0.587599998468160578113383962772786617279052734375f,-0.400600014799833648027060917229391634464263916015625f,0.5887999920070168258234843960963189601898193359375f,0.1136000055015085052900758455507457256317138671875f,-0.165999986767769058815247262828052043914794921875f,0.5020000294148925235049318871460855007171630859375f,0.0216000182569029419710204820148646831512451171875f,0.0177999654531486584119193139486014842987060546875f,-0.6366000054419040932174311819835565984249114990234375f,0.4789999886453151223264512736932374536991119384765625f,1.0008999975383279235074951429851353168487548828125f,-0.2542000198543075129009594093076884746551513671875f,0.8050000055134294285608120844699442386627197265625f,0.232900034403801470261896611191332340240478515625f,0.71360002055168170187471332610584795475006103515625f,-0.63400000548362722696538185118697583675384521484375f,-1.38160001602172854973105131648480892181396484375f,-0.694200023728609227902097700280137360095977783203125f,0.045600008243322331935587499174289405345916748046875f,0.85810000928640395301272292272187769412994384765625f,-0.62820002921223672132100546150468289852142333984375f,0.03499997898936246798484717146493494510650634765625f,0.010700031447409941165460622869431972503662109375f,-0.0992000010788440800268972452613525092601776123046875f,0.2312000023305416096519593338598497211933135986328125f,0.35080000354051588118409199523739516735076904296875f,0.197600001597404428821391775272786617279052734375f,0.3674000091314315596946471487171947956085205078125f,0.565600004667043521067171241156756877899169921875f,0.5106000103294849878921013441868126392364501953125f,0.2964000171482563672498145024292171001434326171875f,-0.747499984502791647855701739899814128875732421875f,0.689400011724233596765998299815692007541656494140625f,0.164400005763769119226935799815692007541656494140625f,-0.54209999718666068702077609486877918243408203125f,-0.19920000301599483094605602673254907131195068359375f,-0.37279999018907528096633541281335055828094482421875f,-0.6187999781489370576537112356163561344146728515625f,-0.444400021559000268922545728855766355991363525390625f,-0.376200012165308184108880595886148512363433837890625f,0.3585999877691268267199120600707828998565673828125f,-0.6272000110030173658515195711515843868255615234375f,-0.56910000052452047736295526192407123744487762451171875f,-1.09590002778768536018105805851519107818603515625f,-0.2311999898135663755738278268836438655853271484375f,-1.124900029061735029500823657144792377948760986328125f,-0.4254999881871046607528796812403015792369842529296875f,0.5159999724626544992389653998543508350849151611328125f,-0.37290000931397049299675927613861858844757080078125f,-0.4627000478982932207827616366557776927947998046875f,-0.59979999173879594565050865639932453632354736328125f,-0.3139000096231698844206903231679461896419525146484375f,1.069100010396539790491488020052202045917510986328125f,-0.096000024765729052855789404929964803159236907958984375f,0.98130001473128858346939296097843907773494720458984375f,0.42449996876716600269219270558096468448638916015625f,-0.513599979871511624907043369603343307971954345703125f,0.34109998817890840872024682539631612598896026611328125f,0.398700014922023004970697002136148512363433837890625f,-0.07020000304579721017717020004056394100189208984375f,0.2567000058472157508759892152738757431507110595703125f,0.5389000059723851876469780108891427516937255859375f,-0.16800000306963891461009552585892379283905029296875f,0.6155000238195065254132032350753434002399444580078125f,1.0878999983429906706078327260911464691162109375f,-0.3155999838054182049518203712068498134613037109375f,0.8325000062957403912378140375949442386627197265625f,-0.148999981045722762473815237171947956085205078125f,0.573199986147880480302774230949580669403076171875f,-0.02019999760687340994991245679557323455810546875f,-0.47039997897148122518728996510617434978485107421875f,0.373599983000755475615051182103343307971954345703125f,-0.48240000041425201704470282493275590240955352783203125f,0.7407000108093024781652502497308887541294097900390625f,-0.25139998987913136918592726942733861505985260009765625f,0.69030002140700819079910388609278015792369842529296875f,-0.19260000239834196289478995822719298303127288818359375f,0.15359999711960548030020845544640906155109405517578125f,-0.217000010024756273541157725048833526670932769775390625f,0.57650000788643984694914479405269958078861236572265625f,-0.26759998926669370522546387292095459997653961181640625f,0.34070000170096736979985507787205278873443603515625f,1.3488000156998634082583521376363933086395263671875f,0.01020000244975083347753752605058252811431884765625f,1.0966000083103775164516946460935287177562713623046875f,-0.18560001703500716985217877663671970367431640625f,-0.5568000010371207952886152270366437733173370361328125f,0.34299999338388442726710536589962430298328399658203125f,0.73099998639523988686050870455801486968994140625f,0.2900999915391206940284973825328052043914794921875f,0.32340000558495507476663988200016319751739501953125f,0.95800001066923101689098984934389591217041015625f,0.6925000213086605072021484375f,-0.4079999953210353691446243828977458178997039794921875f,-0.90129998552501200403952452688827179372310638427734375f,-0.437000000506639618702564575869473628699779510498046875f,-0.06939998713731743151811315328814089298248291015625f,-0.7403999981939790586693561635911464691162109375f,-0.59010001269876966034644283354282379150390625f,-0.0854000017702578606559882246074266731739044189453125f,-0.25460000500977020276849316360312514007091522216796875f,-0.406300017040968020065605514901108108460903167724609375f,-0.178199990320205614580117980949580669403076171875f,0.24220001413226110997811701963655650615692138671875f,-0.275300002184510450575771756120957434177398681640625f,0.06719997687935830299466033466160297393798828125f,0.8874999710917474970983676030300557613372802734375f,0.43339998800158507918922623503021895885467529296875f,-0.69420000212192523036947022774256765842437744140625f,-0.95530000091791134053664791281335055828094482421875f,-0.622500000149011523120634592487476766109466552734375f,-0.20719999520778653590014073415659368038177490234375f,-0.6065999917328357415868822499760426580905914306640625f,-0.35880000154376034959824437464703805744647979736328125f,-0.657399988269805835017223216709680855274200439453125f,-1.3683999753355979844826606495189480483531951904296875f,-0.89090000357329868219125046380213461816310882568359375f,0.37899999535083761514897560118697583675384521484375f,0.751099989259242928341109291068278253078460693359375f,0.5217000095725061736828820357914082705974578857421875f,-0.1221999932333825977792685080203227698802947998046875f,0.17550000375881780190212566594709642231464385986328125f,0.230100013127177926985922340463730506598949432373046875f,0.33779999823570250327975372783839702606201171875f,0.3071999882042408813731526606716215610504150390625f,0.02749999500811117769671909627504646778106689453125f,0.3753999941706656073847625521011650562286376953125f,0.55899999222159379996810457669198513031005859375f,0.3715000031739474906089526484720408916473388671875f,-0.04140000060796733549750570091418921947479248046875f,0.683699994358420326534542255103588104248046875f,0.008400010129809398762290584272705018520355224609375f,-0.070199998724460588306328645558096468448638916015625f,-0.1281999963179230039855838185758329927921295166015625f,-0.2662000307917598007634296664036810398101806640625f,-0.68099999257922139150878138025291264057159423828125f,-0.3716000132277608170028315726085565984249114990234375f,0.0909000098317864768660001573152840137481689453125f,0.2102000006616113747526242150343023240566253662109375f,0.236900009265541999781135018565692007541656494140625f,-0.030999993026256333905621431767940521240234375f,0.8697999924838537566529339528642594814300537109375f,0.071999980650841788332172654918394982814788818359375f,0.428800027993321730690468029933981597423553466796875f,0.46499999642372102925946819595992565155029296875f,0.385000008381903124909939606368425302207469940185546875f,1.1288000091433527671824776916764676570892333984375f,0.00780001060366597442907732329331338405609130859375f,0.8997999988541003357767067427630536258220672607421875f,0.50850001418590551338638761080801486968994140625f,-0.71779997856616972740084747783839702606201171875f,0.29290003246665019531747020664624869823455810546875f,0.8254999951720236595065216533839702606201171875f,0.0292000058472154933042475022375583648681640625f,0.871700006555020667775579568115063011646270751953125f,0.04420000614523900139829493127763271331787109375f,0.325999994218349353758412689785473048686981201171875f,0.1361999989032744995398616083548404276371002197265625f,0.573800018531084532469321857206523418426513671875f,0.551399997478723147281698402366600930690765380859375f,0.713600006507336903638361036428250372409820556640625f,-0.4005000216960912240438119624741375446319580078125f,-0.3168000056564803657010997994802892208099365234375f,-0.4627000126942990565481750309118069708347320556640625f,0.111499965716153226491513805740396492183208465576171875f,-0.5311999786376955245970066243899054825305938720703125f,0.08609998860731682412961163208819925785064697265625f,-0.192999998152256324601694359444081783294677734375f,0.3841999817073347145424122572876513004302978515625f,-0.238800014992058162732746495748870074748992919921875f,-0.43920000271797210444901793380267918109893798828125f,-0.073000012308358730450663642841391265392303466796875f,-0.47060000425726145056870564076234586536884307861328125f,-0.29970000042319266952972611761651933193206787109375f,0.256999991416931139465162914348184131085872650146484375f,0.10179997645020488761247179354541003704071044921875f,1.08429996780753157281651510857045650482177734375f,0.51009998840987691437476314604282379150390625f,-0.059800011259317553680148193961940705776214599609375f,0.53709999822974197059011203236877918243408203125f,-0.089500005945563287923505413345992565155029296875f,-0.2864000010550018515687042963691055774688720703125f,-1.11820002295374809619943334837444126605987548828125f,-0.744200024697184492339374628500081598758697509765625f,-0.13820001460909825397038730443455278873443603515625f,0.31809997099041975587851993623189628124237060546875f,-0.217900020024180296474014539853669703006744384765625f,0.50919999213814737260008769226260483264923095703125f,1.18749998509883880615234375f,0.803599993729591233204700984060764312744140625f,-0.7502000015556813394823620910756289958953857421875f,-1.11759998371601110278561463928781449794769287109375f,-0.647099989736080072333379575866274535655975341796875f,0.015199993360042629575445971568115055561065673828125f,0.2856999862968923054040715214796364307403564453125f,-0.06440000852048388679804702405817806720733642578125f,-0.90639999881982813434433410293422639369964599609375f,-1.35189998192191129788852776982821524143218994140625f,-0.938799986419081466948455272358842194080352783203125f,-0.44199998351931579776419312111102044582366943359375f,-0.63829996360540430799801470129750669002532958984375f,0.010400027996301464128237057593651115894317626953125f,-0.04039999268054972336727814763435162603855133056640625f,-0.045399984596669884506781045274692587554454803466796875f,-0.2105999931931495650072605485547683201730251312255859375f,-0.2293999942526221413885423316969536244869232177734375f,-0.48889998948797586120207370186108164489269256591796875f,-0.5857000028558074777862429982633329927921295166015625f,0.5473999863326550752162802382372319698333740234375f,0.9717999688506129274401246220804750919342041015625f,0.63129998008608811943531691213138401508331298828125f,0.196999983817339074931851428118534386157989501953125f,0.884699967640638629262639369699172675609588623046875f,0.3640999858468771943620367892435751855373382568359375f,-0.2431999843597413002438401008475921116769313812255859375f,-0.882999967884272596307937419624067842960357666015625f,-0.2473999862208964284349121953709982335567474365234375f,-0.7020000084042550270169158466160297393798828125f,-0.0643999925017357810475004953332245349884033203125f,0.6171000094056129370301277958787977695465087890625f,-0.82239997470974923743369799922220408916473388671875f,0.3517000302478672235650947186513803899288177490234375f,1.0571000293731689367859871708787977695465087890625f,-0.138400006181001344884862191975116729736328125f,0.790900021640956740753836129442788660526275634765625f,0.04300001096725480920213158242404460906982421875f,0.8511999992012970750465683522634208202362060546875f,0.20659999426603281591496852342970669269561767578125f,1.0777000172019004509138540015555918216705322265625f,-0.02759998922944095767206817981787025928497314453125f,0.8443000142619012660105681788991205394268035888671875f,-0.436699997353553470702536287717521190643310546875f,-0.6101999766707422168110497295856475830078125f,-0.56149998693168168273359697195701301097869873046875f,0.7230000017285345137452168273739516735076904296875f,0.02459999018311531671088232542388141155242919921875f,0.8168000121757390541432641839492134749889373779296875f,0.50470000772476186767789840814657509326934814453125f,0.019000001013278922101790158194489777088165283203125f,0.36790000340938566747439608661807142198085784912109375f,1.1246999952077860740473624900914728641510009765625f,0.23019998024702115202444474562071263790130615234375f,1.1879000010252001029442681101500056684017181396484375f,-0.17650003290176385917220613919198513031005859375f,-0.4905999866366386186200543306767940521240234375f,-0.002900005868077482062972194398753345012664794921875f,0.345000001750886298168552457354962825775146484375f,0.22079998558461688684673163152183406054973602294921875f,0.286999990262091220216689180233515799045562744140625f,0.6271000140435993852605633946950547397136688232421875f,0.5459999828934669618973885008017532527446746826171875f,0.59169999401941897332335429382510483264923095703125f,-0.550399985826015125667254324071109294891357421875f,-0.2677999808013442617493637953884899616241455078125f,-0.632599984796345271576001323410309851169586181640625f,0.382300004255771597172497422434389591217041015625f,-0.4319999808371068894530253601260483264923095703125f,0.1737000134691595309988088047248311340808868408203125f,-0.48500000627711392642282817178056575357913970947265625f,0.21279999320656062256063023596652783453464508056640625f,-0.2494000066019594896715716458857059478759765625f,0.93500001728534698486328125f,0.29209999979436407357979987864382565021514892578125f,0.371999991714954347798993694595992565155029296875f,0.554999977648258369100631171022541821002960205078125f,-0.0584000096082684105880389324738644063472747802734375f,-0.44479999963641159155258719692938029766082763671875f,-0.957900000727176514914162908098660409450531005859375f,-0.3665999939680097963190519294585101306438446044921875f,-0.26680000163316730521501085604541003704071044921875f,-0.340999997496604922986307428800500929355621337890625f,-0.2541999954164027020198091122438199818134307861328125f,0.49640000060796740655177927692420780658721923828125f,0.80789999044537541550425885361619293689727783203125f,0.543799999681115142635690062888897955417633056640625f,-0.332000002294778884248671602108515799045562744140625f,-0.2466999937772751394504666677676141262054443359375f,-0.29139999688267703703559163841418921947479248046875f,-0.04279999734163286717603114084340631961822509765625f,-0.36479999203681945374455608543939888477325439453125f,-0.223000005602836726126270150416530668735504150390625f,-0.117600006514787747136097095790319144725799560546875f,0.084799999263882597233532578684389591217041015625f,0.144000004664063485648739515454508364200592041015625f,-0.2484000145256519687109175720252096652984619140625f,0.08359999089837089059074060060083866119384765625f,-0.320800016281008737450974876992404460906982421875f,0.2286000090405344575827939479495398700237274169921875f,-0.20839999271407727565730283458833582699298858642578125f,-0.274900003003329007622568269653129391372203826904296875f,0.34100000166893007502721957280300557613372802734375f,0.1111999963700771587582494248636066913604736328125f,0.47950000192224973005750143784098327159881591796875f,-0.22740000272393245950297568924725055694580078125f,-0.345899988895654875165064368047751486301422119140625f,0.2327000226408244554221482758293859660625457763671875f,-0.4950000029802321677152576739899814128875732421875f,-0.6616999986946581202573725022375583648681640625f,-0.64970001085400586049445337266661226749420166015625f,0.1239999990761280201923000277020037174224853515625f,0.156999996036291111778382401098497211933135986328125f,0.4318000075191259501394824837916530668735504150390625f,0.3909999995827673213710795607767067849636077880859375f,0.313800002586841486618141061626374721527099609375f,0.0508999961227178943090621032752096652984619140625f,0.091599987262487303496527601964771747589111328125f,-0.9328000035628674080356859121820889413356781005859375f,0.2554000084757799271528710960410535335540771484375f,0.4707999734401706604103310382924973964691162109375f,0.0857999852120878703232165207737125456333160400390625f,-0.0413000180840494834910714416764676570892333984375f,0.0817999983549118070413896930404007434844970703125f,0.1430999951303004724678658021730370819568634033203125f,0.3805999967694280172736398526467382907867431640625f,0.02239999333620090737895225174725055694580078125f,0.3826000003308058783346723430440761148929595947265625f,-0.562700010943412820552111952565610408782958984375f,-0.1841999937772751394504666677676141262054443359375f,-0.615900010427832711457085679285228252410888671875f,0.6995000054240225750845638685859739780426025390625f,-0.0574000031709669400470374966971576213836669921875f,0.635900008118152726410698960535228252410888671875f,-0.109199981856345829100973787717521190643310546875f,0.3291999934792517734649663907475769519805908203125f,-0.03839999418556683252745642676018178462982177734375f,0.1063999980002641176923816601629368960857391357421875f,-0.38719999401569349384288898363593034446239471435546875f,0.0920000126510860383177714538760483264923095703125f,1.09120003943443322214079671539366245269775390625f,0.00720000057220460121243377216160297393798828125f,0.8984000314384701368908281438052654266357421875f,-0.2801999908454717580497117523918859660625457763671875f,0.7593999734282494973314214803394861519336700439453125f,-0.23240002010241145313784727477468550205230712890625f,-1.22260001232624038181029391125775873661041259765625f,-0.127799993020296209778052798355929553508758544921875f,-1.03960000422745935111379367299377918243408203125f,-0.484300025773048758992445073090493679046630859375f,-0.53680000178217834871929881046526134014129638671875f,-0.3140999960169192917192049208097159862518310546875f,0.950300023418665151808681912370957434177398681640625f,0.2772000001251695433523991596302948892116546630859375f,0.908500015638768587677986943162977695465087890625f,-0.68580002112388616097859994624741375446319580078125f,-0.409599987947940835653071189881302416324615478515625f,-0.58460000180602078589942038888693787157535552978515625f,-0.31679998002648357413590929354541003704071044921875f,0.43299997505545650966496395994909107685089111328125f,-0.346200004081427703539475260186009109020233154296875f,0.3073000248193746841707252315245568752288818359375f,-0.02129999424219131132218763013952411711215972900390625f,-0.0009999993219971248237243344192393124103546142578125f,0.31009999852403991038585218120715580880641937255859375f,0.374100010228902102316084210542612709105014801025390625f,0.003799997520446851684283728900481946766376495361328125f,0.0658999935708941275702699158500763587653636932373046875f,-0.135700011628866246127245176467113196849822998046875f,0.38500000074505802416524602449499070644378662109375f,0.14720000027418134180834385915659368038177490234375f,0.556900001293420654491228560800664126873016357421875f,-0.386800003570318295942342956550419330596923828125f,-0.163899993082881056949418052681721746921539306640625f,-0.2737999957323073996207085656351409852504730224609375f,-0.024400014406442682712850000825710594654083251953125f,0.75319998480677607943789553246460855007171630859375f,0.219199997130036461356894506025128066539764404296875f,0.0674000142350792419421168233384378254413604736328125f,-0.22079998422488589238099621070432476699352264404296875f,-0.266499990675598319445072093003545887768268585205078125f,-0.15640000463128078678209931240417063236236572265625f,-0.572600009793042818984076802735216915607452392578125f,-0.5444000211864710170317493975744582712650299072265625f,0.216800013256072787726225215010344982147216796875f,0.389700023669004025350659503601491451263427734375f,-0.2215999898701905834741410217247903347015380859375f,0.37499999910593029284200383699499070644378662109375f,0.59809998931884766903976924368180334568023681640625f,-0.048700005459785256078930615331046283245086669921875f,0.30680000520944616937413229607045650482177734375f,-0.150200017201900237751033273525536060333251953125f,0.2710000003278256741623408743180334568023681640625f,0.3298000038087369745198884629644453525543212890625f,0.06249998763203645779640282853506505489349365234375f,-0.239500021591782630281386445858515799045562744140625f,0.01140000105500238447575611644424498081207275390625f,0.5664999943077564381610500277020037174224853515625f,0.32569999784529191089177402318455278873443603515625f,-0.26079999817609778034466216922737658023834228515625f,-0.305999998837709341614754521287977695465087890625f,-0.43680000714659694693864366854541003704071044921875f,0.13960001283288026030504624941386282444000244140625f,-0.206000001594424109185865745530463755130767822265625f,-0.1730000088065863916853004411677829921245574951171875f,0.21820001848340009331650435342453420162200927734375f,-0.18420000074356790431551189612946473062038421630859375f,-0.137900025928765657123875598699669353663921356201171875f,-0.5715999817490577949996577444835565984249114990234375f,-0.14650000304356236568992244428955018520355224609375f,0.3111000164724888605860542156733572483062744140625f,0.158800003220140706350349546482902951538562774658203125f,0.24050000692531459112188940707710571587085723876953125f,-1.3730000041127201626522946753539144992828369140625f,-0.17059999274611481467900375719182193279266357421875f,-1.1703999965921043635574960717349313199520111083984375f,0.8939000055998562732639811656554229557514190673828125f,-0.18780000002384167334668063631397671997547149658203125f,0.7895000118315220749565241931122727692127227783203125f,0.818400005733966651177979656495153903961181640625f,-0.54319998972415906024480136693455278873443603515625f,0.5696000175267459386674318011500872671604156494140625f,-0.021800011300295907279434004522045142948627471923828125f,0.5081999778032304515562600499833934009075164794921875f,-0.00120002905949956328868211130611598491668701171875f,0.3034000325560572974836759385652840137481689453125f,0.629199992585181888671286287717521190643310546875f,0.3322000112265348636952921879128552973270416259765625f,0.5511000056505199751200052560307085514068603515625f,0.49339996758699466994357862859033048152923583984375f,0.3638999752759934924739582129404880106449127197265625f,0.2989000059723849744841572828590869903564453125f,0.44999997660517720987627399154007434844970703125f,0.115099980922043432229884274420328438282012939453125f,-1.2158000342369081892002213862724602222442626953125f,0.16480000425577134848253990639932453632354736328125f,-0.9986000282257798676965876438771374523639678955078125f,-0.11389997825622533156320059788413345813751220703125f,0.785999990791081959429220660240389406681060791015625f,-0.14570001181513092536334852411528117954730987548828125f,0.3020999891400339976144095999188721179962158203125f,-0.56839996684193661025119581609033048152923583984375f,0.1735000052079553878314754911116324365139007568359375f,0.717200011152029137662111679674126207828521728515625f,0.2536000002861020785616119610494934022426605224609375f,0.646800001782178668463529902510344982147216796875f,-0.2808000055521724380724890579585917294025421142578125f,0.49400000458955706239549954261747188866138458251953125f,-0.2204000214397912049690830826875753700733184814453125f,-1.1890000337958337439658862422220408916473388671875f,0.3971999955803153792288640033802948892116546630859375f,0.604000012487173165709464228712022304534912109375f,0.683699996221065475765499286353588104248046875f,0.05659999441504481865194975398480892181396484375f,0.328599994474649381714925766573287546634674072265625f,0.2887999870896340581793992896564304828643798828125f,-0.04690001038312896497473047929815948009490966796875f,-0.3201999868035316776371246305643580853939056396484375f,-0.926799975109100504511161489062942564487457275390625f,-0.3070999875754119035065059506450779736042022705078125f,-0.1651999958932399703570581550593487918376922607421875f,-0.45069999546110628418915666770772077143192291259765625f,-0.393599998944997697680747705817339010536670684814453125f,-0.05340000215768803126792363400454632937908172607421875f,-0.100600008876621516140659196025808341801166534423828125f,-0.42570001766383647190394867720897309482097625732421875f,0.475599991405010358302973827449022792279720306396484375f,0.86509996818155077136225372669287025928497314453125f,0.268199985700845910940870453487150371074676513671875f,-0.1081999992609024019429853069595992565155029296875f,0.00780000374913214500338654033839702606201171875f,-0.307100006946921411099538090638816356658935546875f,0.152800012093782466848779222345910966396331787109375f,-0.47179998866915706656754991854541003704071044921875f,0.071000003680586676324537620530463755130767822265625f,-0.224000017255544747740714228712022304534912109375f,0.06299999293684965806505715590901672840118408203125f,0.364199999663233686675312128500081598758697509765625f,0.4764000126779077692873443083954043686389923095703125f,0.59580002171993218240686474018730223178863525390625f,0.144000003024935541162676599924452602863311767578125f,-0.2391999860286713186496854177676141262054443359375f,0.0508000282764431432269702781923115253448486328125f,-0.44839999589920065403703119955025613307952880859375f,0.09659998517632484293926609097979962825775146484375f,0.694699978071451340611019986681640148162841796875f,0.0814999891668559683921557734720408916473388671875f,-0.065999984011054113608452098560519516468048095703125f,-0.4612999787077309843397188160452060401439666748046875f,-0.54939999127238980936027701318380422890186309814453125f,0.0739999940097331165844707356882281601428985595703125f,-0.2931000055611132637523041921667754650115966796875f,0.03439999965429318962151228333823382854461669921875f,0.532600002193450716703182479250244796276092529296875f,-0.213800006386637875266387709416449069976806640625f,-0.155700004029274197137056034989655017852783203125f,0.6369999830722807843130794935859739780426025390625f,-0.291300037157535829379639835678972303867340087890625f,-0.083799983215332218833282240666449069976806640625f,0.47059997844100021069380090921185910701751708984375f,-0.1480000055655834589884989327401854097843170166015625f,-0.31540001130700101583670402760617434978485107421875f,0.18539999551177022141246197861619293689727783203125f,-0.0976000081166625843565043396665714681148529052734375f,0.2582999997407195991172557114623486995697021484375f,0.3869999859035015976616023181122727692127227783203125f,0.2830999896168708840349381716805510222911834716796875f,0.63320002552419918373516338760964572429656982421875f,0.49179999470412705253608010025345720350742340087890625f,0.52580000354051581457071051772800274193286895751953125f,0.0056999997451898654077240280457772314548492431640625f,0.1181999988883732921607361276983283460140228271484375f,-0.16510001102238902603858150541782379150390625f,0.702800018799305092898066504858434200286865234375f,0.2249999959766864066068592364899814128875732421875f,0.576600004734098892100746525102294981479644775390625f,-1.0970000101923940150072667165659368038177490234375f,0.28319999836683251714930520392954349517822265625f,-0.84380001376271263069384076516143977642059326171875f,-0.0901999906778332416479315725155174732208251953125f,-0.82239997664690012157961973571218550205230712890625f,-0.0473999590635296730312120416783727705478668212890625f,-0.19859998493790609330744700855575501918792724609375f,0.358799981725216099448516615666449069976806640625f,-0.2970000132471322107363675968372263014316558837890625f,1.231600013637542900823973468504846096038818359375f,0.1935999961137770952745995600707828998565673828125f,1.013199995535611908081818910432048141956329345703125f,0.73510001441240291342182899825274944305419921875f,-0.1667999993979929485021784785203635692596435546875f,0.492500012107193629873336249147541821002960205078125f,0.31529999618679249717700940891518257558345794677734375f,0.738999968826770992080810174229554831981658935546875f,0.3156999699428677530477216350845992565155029296875f,-0.628099996918439984483484295196831226348876953125f,0.156599986815452840716034188517369329929351806640625f,-0.47849999697506395701651626950479112565517425537109375f,-0.2329999698400495589112324523739516735076904296875f,-0.3759999960809945473982907060417346656322479248046875f,-0.0374000030219552304089347671833820641040802001953125f,0.615999978572130402199036325328052043914794921875f,0.0416999918401241842502713552676141262054443359375f,-0.333799990963935755416969186626374721527099609375f,-0.19839999232292182540504654753021895885467529296875f,-0.413899993231892526779347463161684572696685791015625f,-0.3804000126481057453275980151374824345111846923828125f,-0.238000000759959196017234717146493494510650634765625f,-0.1920000002086161838832367720897309482097625732421875f,0.13740000957846643103721362422220408916473388671875f,0.01440001157522186048254297929815948009490966796875f,0.350700009840726689702705698437057435512542724609375f,0.0784000236153603591304772635339759290218353271484375f,-0.45739997709393520608500693924725055694580078125f,-0.01239998797178287759379600174725055694580078125f,-0.16740000346899031047342987221782095730304718017578125f,-0.665800006446242154112269417964853346347808837890625f,-0.2338999999374149485475982146454043686389923095703125f,-0.185399992382526246359475408098660409450531005859375f,0.08680000386834141323788571753539144992828369140625f,-0.154499999240040875037038858863525092601776123046875f,0.57419999988675129998227930627763271331787109375f,0.561899981772899792531461571343243122100830078125f,0.48869999122917651135367123060859739780426025390625f,-0.01499998837709437538023848901502788066864013671875f,-0.686899974322319195607633446343243122100830078125f,-0.2832999996662142194736588862724602222442626953125f,0.25519999514818181296504917554557323455810546875f,-0.1864000029921530909859939129091799259185791015625f,0.14159999441504478312481296597979962825775146484375f,0.6263999839186669138513252619304694235324859619140625f,1.090499961793423011613413109444081783294677734375f,0.4883999810725452306314764427952468395233154296875f,-0.167200006830692249337744215154089033603668212890625f,0.12179999269247065996069068205542862415313720703125f,-0.224800010514259351879218229441903531551361083984375f,-0.20259999593496325065444807478343136608600616455078125f,-0.459899991808831731798790087850647978484630584716796875f,-0.29069999508857724546118106445646844804286956787109375f,0.2623999890148638325371166502009145915508270263671875f,0.16539999119043358177805203013122081756591796875f,0.193799992081522987064090557396411895751953125f,0.1398000066399571750253016944043338298797607421875f,0.8221000104486944071169318704050965607166290283203125f,0.92299998697638496736317392787896096706390380859375f,-0.2953999767363075790171933476813137531280517578125f,0.6711999939486379407327376611647196114063262939453125f,0.972999983698129344844574006856419146060943603515625f,0.0814000004589556791501081534079276025295257568359375f,0.849199982638656702960133770829997956752777099609375f,0.6392000160545110443166549885063432157039642333984375f,-0.26000000089406938297287297245929948985576629638671875f,0.5936000265121463836948123571346513926982879638671875f,-0.79260001918077449545307899825274944305419921875f,-0.24299998787045495873826439492404460906982421875f,-0.655200007776916226731600545463152229785919189453125f,-0.80660002719759926748110956395976245403289794921875f,0.27240000227689709078049418167211115360260009765625f,-0.602600030654669271967804888845421373844146728515625f,-0.318000005230307802861489108181558549404144287109375f,0.244999989271164064774666258017532527446746826171875f,-0.1522000028446315322838700012653134763240814208984375f,0.713799989771843002017703838646411895751953125f,-0.11019998486638105106294460711069405078887939453125f,0.5871999920412893647636565219727344810962677001953125f,-0.982500007003545672290556467487476766109466552734375f,0.0780000057518479561480262418626807630062103271484375f,-0.933900012044608818229107782826758921146392822265625f,-0.54449998039007141414913348853588104248046875f,0.5451999848663806602644399390555918216705322265625f,-0.352699999506771444401920234668068587779998779296875f,-0.9679999841451643050049824523739516735076904296875f,0.26239998707771317043579983874224126338958740234375f,-0.7887999920070167814145634110900573432445526123046875f,-0.28409998875260367157125074299983680248260498046875f,-0.0018000001430511503031084430404007434844970703125f,-0.4448999979555605666092787942034192383289337158203125f,0.8470000191330910865872283466160297393798828125f,-0.00359999551773082515637725009582936763763427734375f,0.6870000142157077771543072230997495353221893310546875f,0.1456999962434171180802877643145620822906494140625f,0.20199999059736739592807452936540357768535614013671875f,0.199499993652105356289894189103506505489349365234375f,-0.53819997765421856428247338044457137584686279296875f,0.10079998834133174767657692427746951580047607421875f,-0.4873999845445151635203728801570832729339599609375f,-0.23789999327659661076950214919634163379669189453125f,-0.136200004789233108937196448096074163913726806640625f,0.040799983423948305016892845742404460906982421875f,0.6912999832153321477790086646564304828643798828125f,0.50459999107718456201610024436376988887786865234375f,-0.6480000061988828718995137023739516735076904296875f,-1.121000007927417385644730529747903347015380859375f,-0.6640000130832195424090969027020037174224853515625f,-0.6236000211477279453475830450770445168018341064453125f,-0.6940000202357765601846040226519107818603515625f,-0.25940000248551342565406230278313159942626953125f,0.518000009328126953533910636906512081623077392578125f,0.55150000220537176431889747618697583675384521484375f,0.399999998211860496866165703977458178997039794921875f,0.054999984800815582275390625f,0.82599997678399095235590721131302416324615478515625f,0.349300000515580411075688971322961151599884033203125f,-0.31980000306367895035464243846945464611053466796875f,-0.1186999777436259506657734164036810398101806640625f,0.32590001631379106328267880599014461040496826171875f,0.412000004380941309278796325088478624820709228515625f,0.5062000035226343896965772728435695171356201171875f,0.01859999440014359350925587932579219341278076171875f,0.212200009810924683506527799181640148162841796875f,0.01569999896287921359316896996460855007171630859375f,0.227900002068281093414725546608678996562957763671875f,-0.16599999332427994858107922482304275035858154296875f,0.020200024354457557507203091518022119998931884765625f,-0.4369999991655351134767215626197867095470428466796875f,-0.44060000004768351544726101565174758434295654296875f,-0.751700006983428981044426109292544424533843994140625f,-0.2770999964043496799348531567375175654888153076171875f,0.1681999915495513420182760455645620822906494140625f,-0.03050000439211719704957204157835803925991058349609375f,-0.070300008983164963272116665393696166574954986572265625f,0.6073999966144558726455215946771204471588134765625f,0.79190000501871082860816386528313159942626953125f,0.476300003927946136172977276146411895751953125f,0.27499999195337299084940241300500929355621337890625f,-0.10750001609325376961123765795491635799407958984375f,-0.256300027024745968873276069643907248973846435546875f,0.01299999606609336666451781638897955417633056640625f,0.81870000456571556668450284632854163646697998046875f,0.428700006204843386825587003841064870357513427734375f,-0.076200016784667923275264911353588104248046875f,-0.38859999775290454682163954203133471310138702392578125f,0.14710001298189201435206996393389999866485595703125f,-0.789399970149994079093858090345747768878936767578125f,0.03070003502368934267252598147024400532245635986328125f,0.5610000141859057265492083388380706310272216796875f,0.20200000989437061349462965154089033603668212890625f,0.6730000193119047846579405813827179372310638427734375f,0.50320000000596021294541060342453420162200927734375f,-0.4051999918699262792642912245355546474456787109375f,0.5732000288397076648294614642509259283542633056640625f,-0.753500004351138841229840181767940521240234375f,0.10900000339746451771816282416693866252899169921875f,-0.7415000150948765078595670274808071553707122802734375f,0.57419998930692628391625476069748401641845703125f,-0.6105999897658822561652414151467382907867431640625f,0.3544000074401498778797758859582245349884033203125f,-0.127900001323222767268816824071109294891357421875f,-0.863999980896711150535338674671947956085205078125f,-0.04249996904283737109153662458993494510650634765625f,-0.014199982750415784948927466757595539093017578125f,0.44799998146295560985663541941903531551361083984375f,-0.1298000106632708483545002309256233274936676025390625f,-0.72710002192258826880788546986877918243408203125f,0.0264000105917450156312042963691055774688720703125f,-0.613700018759072207785720820538699626922607421875f,1.0354000275492669658206068561412394046783447265625f,0.2991999845385553413734669447876513004302978515625f,0.8494000049442054223192144490894861519336700439453125f,0.11549997634068109864102780193206854164600372314453125f,-0.08799998954683563301415460955468006432056427001953125f,0.2436999948613343125458641225122846662998199462890625f,-0.235299971767514881637595181018696166574954986572265625f,0.35359997972250012399086926961899735033512115478515625f,-0.20869998815581158879695067298598587512969970703125f,-0.6776999850153924143114636535756289958953857421875f,0.07479997549653116806211983202956616878509521484375f,-0.7219000001385802445241779423668049275875091552734375f,-0.2426999552130695292362361215054988861083984375f,0.775599983209371401926546241156756877899169921875f,-0.220899988411366710039374083862639963626861572265625f,0.5106999989628793201745793339796364307403564453125f,-0.77319997884631153084455945645458996295928955078125f,0.3105000165924429733621536797727458178997039794921875f,0.274100013935566266809473745524883270263671875f,-0.161800006625056003173313001752831041812896728515625f,-0.193200008201599160440053992715547792613506317138671875f,-0.2142000053629278255584722501225769519805908203125f,-0.1006000010162590996287690359167754650115966796875f,0.24139998600482925894539221189916133880615234375f,1.0797999892055987203320910339243710041046142578125f,0.62050000329315668778917824965901672840118408203125f,-0.3865999942660334198762939195148646831512451171875f,-0.616499970912933736855165989254601299762725830078125f,0.0785000310242176535524549763067625463008880615234375f,-0.29399999251961705937219448969699442386627197265625f,-0.95959999107718463307037382037378847599029541015625f,-0.530799995419382941719277368974871933460235595703125f,-0.01599999302625665364985252381302416324615478515625f,-0.6533999896407127749853316345252096652984619140625f,-0.45879999752044664518280114862136542797088623046875f,0.5278000012159349552121057058684527873992919921875f,0.515799977165460887817971524782478809356689453125f,-0.1563000260561704379824732313863933086395263671875f,-0.21919998364448556316119720577262341976165771484375f,0.18950003209710075680050067603588104248046875f,-0.27799998615682142144578392617404460906982421875f,0.021199999052286144785028909609536640346050262451171875f,0.101999998942017555680905616100062616169452667236328125f,0.0035999983489513331935683027040795423090457916259765625f,-0.30419999750256554449379109428264200687408447265625f,-0.281799983900785644408415464567951858043670654296875f,-0.4832999925136565178007685972261242568492889404296875f,-0.3535999922394754690913032391108572483062744140625f,-0.27049996879696891483035869896411895751953125f,0.131200029525160744015011005103588104248046875f,0.0366000118494032022908868384547531604766845703125f,0.519400029009580155303638093755580484867095947265625f,-0.2018999942153694160396071310970000922679901123046875f,0.240399999907612738070383784361183643341064453125f,0.404399999095499396162267657928168773651123046875f,0.5070000111982226886908620144822634756565093994140625f,-0.180800001305341684343375163734890520572662353515625f,-0.5403000019609927040420416233246214687824249267578125f,-0.06979999740123743467989925193251110613346099853515625f,0.292599999734759474989687078050337731838226318359375f,0.4068999878451229346154605082119815051555633544921875f,0.44729999460726965754275852304999716579914093017578125f,0.414599990779161320375578725361265242099761962890625f,0.3253999771833420151523341701249592006206512451171875f,0.1898000051125883214098166718031279742717742919921875f,-0.1000000014156101624251959947287105023860931396484375f,0.2046000121250750591883615925326012074947357177734375f,0.1804000013232229804316375521011650562286376953125f,-0.75019998993277514642841197201050817966461181640625f,-0.0169999819919464112416562784346751868724822998046875f,-0.3537000293016436813786640414036810398101806640625f,-0.5000000043213361777816317044198513031005859375f,-0.1551000002190467963458786471164785325527191162109375f,-0.050400007581711037119021057151257991790771484375f,0.5199999827146530861909923260100185871124267578125f,0.11439998497664927157302372506819665431976318359375f,-0.20899999505281474654339035623706877231597900390625f,0.757199967193603828974346470204181969165802001953125f,-0.08820001773834207359215042743016965687274932861328125f,0.074000022470951165587393916212022304534912109375f,0.8433999877035613934594948659650981426239013671875f,-0.077000018052757024378252026508562266826629638671875f,1.1280999970674514543134137056767940521240234375f,-0.0350000020861624960843982989899814128875732421875f,0.911299988430738228117888866108842194080352783203125f,0.1088000035554170796814332788926549255847930908203125f,-0.00319999985694885413778365546022541821002960205078125f,0.072000001698732385424506219351314939558506011962890625f,0.906800008636712728815609807497821748256683349609375f,0.2397999787747864264275676760007627308368682861328125f,0.8017999977961185464891968877054750919342041015625f,0.02009997358322124227925087325274944305419921875f,-0.6213999964356418104216572828590869903564453125f,0.0985000009238721219162471243180334568023681640625f,1.038400019145012009857964585535228252410888671875f,0.15619998772740384396229273988865315914154052734375f,0.76419999575167896210814433288760483264923095703125f,-0.64640002187937517508231621832237578928470611572265625f,-0.403599995219707297078315377802937291562557220458984375f,-0.59880000642389052956104933400638401508331298828125f,-0.3709000094592571628027144470252096652984619140625f,-0.022199997889995604083424041164107620716094970703125f,-0.2072999985933303701557406384381465613842010498046875f,-0.64030000929981445434435727293021045625209808349609375f,-0.249199979919195568900391890565515495836734771728515625f,-0.60170000085532693301360041004954837262630462646484375f,-0.3303999807596209592475133831612765789031982421875f,-0.126500001758337088375583334709517657756805419921875f,-0.75019998203516014001479561557061970233917236328125f,-1.06119996269345318040677739190869033336639404296875f,-0.785699976685643175500217694207094609737396240234375f,-0.2764000149130823036358606259454973042011260986328125f,-0.040300001141428953843615090590901672840118408203125f,-0.1256000020593404098434575644205324351787567138671875f,-0.02700001630187021106621614308096468448638916015625f,0.70519997637271902846123339259065687656402587890625f,0.463699988621473213612489416846074163913726806640625f,0.39499998927116397595682428800500929355621337890625f,1.179099982345104269398916585487313568592071533203125f,0.6328999891042708103583436241024173796176910400390625f,0.014799993526935839582847620476968586444854736328125f,0.6406999716937544775419155484996736049652099609375f,0.00939998810589326438957868958823382854461669921875f,-0.19640000552535052946723226341418921947479248046875f,-0.129300003942847130389282028772868216037750244140625f,-0.2386000105679035865335890775895677506923675537109375f,-0.5876000088989734582156643227790482342243194580078125f,-0.96420000361204127869996227673254907131195068359375f,-0.420799998846649980777101518469862639904022216796875f,-0.475400012350082334933176753111183643341064453125f,-0.52340001422762849614400693099014461040496826171875f,0.058100008168816774656306733959354460239410400390625f,0.2926000059187412460914856637828052043914794921875f,-0.06099999868869776520341474679298698902130126953125f,-0.164900010770559379835731306229718029499053955078125f,-0.147400008982419894465465404209680855274200439453125f,-0.85500001668930014631087033194489777088165283203125f,-0.364100012892484503623791169957257807254791259765625f,-0.248799994093179599463638851375435478985309600830078125f,-0.160999994032084980943864138680510222911834716796875f,-0.0847999947190285485731919834506697952747344970703125f,0.369999994933605247382502057007513940334320068359375f,0.521099980169534848783996494603343307971954345703125f,0.1253999903708697960524887093924917280673980712890625f,0.0913999930083750911080642254091799259185791015625f,0.9677000030457971746500334120355546474456787109375f,0.40129999476373168221243759035132825374603271484375f,-0.23359999417662624665581461158581078052520751953125f,-0.710499992489814768958922286401502788066864013671875f,-0.5553999937236306916332750915898941457271575927734375f,0.256400010144710588377847670926712453365325927734375f,0.1780999892815946505919555420405231416225433349609375f,1.0313999797463413443665558588691055774688720703125f,0.18419998707175277985470529529266059398651123046875f,1.0567999820753930517724938908941112458705902099609375f,0.6407000243693590224580702852108515799045562744140625f,-0.24660000469684562052208320892532356083393096923828125f,0.5567000304341320759959899078239686787128448486328125f,0.35620000545978580674955082940869033336639404296875f,-0.844199961441755686308852091315202414989471435546875f,0.20600003128498778348642872515483759343624114990234375f,0.0323000117063525493676934274844825267791748046875f,-0.563199976760149212395845097489655017852783203125f,-0.1690999814137816859016538728610612452030181884765625f,0.9180999908089635397345773526467382907867431640625f,-0.18259998192787207926812698133289813995361328125f,0.656899993768333967381067850510589778423309326171875f,0.5457000206440689726861137387459166347980499267578125f,0.04540000519752485086399929059552960097789764404296875f,0.4797000168144702758610264936578460037708282470703125f,0.29419999988675105129232179024256765842437744140625f,-0.06359999998807897014785339706577360630035400390625f,0.432600016126036823749245741055347025394439697265625f,-0.0212000260233882187321796664036810398101806640625f,-0.63659998249411575699241438996978104114532470703125f,0.1126000134065747726452855204115621745586395263671875f,-0.35239998171329478537927570869214832782745361328125f,0.671799983900785324664184372522868216037750244140625f,-0.31660001017302275005960154885542578995227813720703125f,-0.42719998745918275062649627216160297393798828125f,0.49819998778700824715315320645458996295928955078125f,-0.2201999934718010809575616804067976772785186767578125f,0.344199985097348726270638508140109479427337646484375f,-0.22039998500645185952606652790564112365245819091796875f,0.32979999632090295502706567276618443429470062255859375f,-0.08869999711513543161345296539366245269775390625f,0.243799990963935897525516338646411895751953125f,-0.1851000104635953658060998350265435874462127685546875f,0.6369000234961514905762669513933360576629638671875f,-0.66439997625947011528069197083823382854461669921875f,0.3827000305756926845646148649393580853939056396484375f,0.253699993240832810670326580293476581573486328125f,0.56779997320175201735992231988348066806793212890625f,0.319699980455637078335939804674126207828521728515625f,-0.78490002343654641236980751273222267627716064453125f,0.25850000494718539112426469728234224021434783935546875f,-0.13380000005364411919117628713138401508331298828125f,0.12820000343322757174746584496460855007171630859375f,-0.365700016546249617732655678992159664630889892578125f,0.1642000009298324181283845746293081901967525482177734375f,0.395400002235919145743281433169613592326641082763671875f,0.1084999986514448566676804830422042869031429290771484375f,-0.0216000066339969709616752879810519516468048095703125f,0.121499992072582330138175166212022304534912109375f,-0.05740000585317606152102598571218550205230712890625f,0.348599999690055728507331878063268959522247314453125f,1.0099000068962571408093253921833820641040802001953125f,0.49530000061988810333701849231147207319736480712890625f,0.0604000037074090556643568561412394046783447265625f,0.60769999887347214695410002605058252811431884765625f,0.487400003394484304664047158439643681049346923828125f,0.6386000041604045218690544061246328055858612060546875f,0.784799973075092083263371023349463939666748046875f,0.32749998837709437538023848901502788066864013671875f,-0.521199987280368759456905536353588104248046875f,-0.596399981012940560987090066191740334033966064453125f,-0.6287999887287616385123101281351409852504730224609375f,0.6027999897420404096237689373083412647247314453125f,0.89279999153017985946689805132336914539337158203125f,0.783200004103779878761315558222122490406036376953125f,-0.220599995875358612096306387684307992458343505859375f,-0.335899988299608320829747754032723605632781982421875f,-0.2236999959230423495881723283673636615276336669921875f,0.00979999725222580053696219692938029766082763671875f,0.497000000730156710204710179823450744152069091796875f,0.5523000182628632348524888584506697952747344970703125f,0.0046000049352645788758309208787977695465087890625f,-0.427299997103214312943464392446912825107574462890625f,0.3313000198721887112895956306601874530315399169921875f,0.6242000079333782647239559082663618028163909912109375f,0.85839999634623520563536658301018178462982177734375f,0.280099993252754142503135881270281970500946044921875f,0.46779999451041209823642930132336914539337158203125f,0.6852999928712844024403239018283784389495849609375f,0.679900005331635526317768380977213382720947265625f,-0.373199992853403106352061513462103903293609619140625f,-1.0083999842762947363183911875239573419094085693359375f,-0.41439999302327634733700278957257978618144989013671875f,-0.280000003427267163402802907512523233890533447265625f,0.0997999987050890435824612723081372678279876708984375f,1.02400003618001989025287912227213382720947265625f,0.224600005978345507173798978328704833984375f,0.823000016666948663868197400006465613842010498046875f,0.12120000693127519486580467855674214661121368408203125f,-0.00860000114291903372532033245079219341278076171875f,0.0270000000782311122549117499147541821002960205078125f,0.313700010526180239622817680356092751026153564453125f,0.0037999984145165033311286606476642191410064697265625f,0.239700008022785293615442014925065450370311737060546875f,0.0847000120460990046922233887016773223876953125f,-0.407199990290403324166845777654089033603668212890625f,-0.100699989239871445789731296827085316181182861328125f,0.0142999804735186586412964970804750919342041015625f,-0.71259996345043230547844359534792602062225341796875f,-0.0684999972730879758131550261168740689754486083984375f,-0.5042000178799035392529503951664082705974578857421875f,0.3138000026613469994885008418350480496883392333984375f,-0.5868000329256062297389462401042692363262176513671875f,1.0615999851018187438711493086884729564189910888671875f,0.19119999473094939634876254785922355949878692626953125f,0.9935999828517434462327173605444841086864471435546875f,-1.01399999862909329095828070421703159809112548828125f,-0.353600007438659158509608460008166730403900146484375f,-1.001999993130564092780332430265843868255615234375f,0.255700001496076634310838926467113196849822998046875f,0.0519999981224537055624068671022541821002960205078125f,0.275500001244246950360405890023685060441493988037109375f,-0.1652000318795445732433790908544324338436126708984375f,-0.83859998434185956117659088704385794699192047119140625f,-0.19179999995678664870268903541727922856807708740234375f,-1.183700044202804946280593867413699626922607421875f,-0.28639999807476979043485698639415204524993896484375f,-0.89710001801103356200428606825880706310272216796875f,-0.29139999688267703703559163841418921947479248046875f,0.33019999350905404611467020004056394100189208984375f,-0.423200021351874067221388031612150371074676513671875f,-0.914700009065866570523439804674126207828521728515625f,-0.3639999993741509509703746516606770455837249755859375f,-0.878899999378621288315116544254124164581298828125f,-0.1688000048220157367495630751363933086395263671875f,0.266399992263317120233523382921703159809112548828125f,0.0367999967902898905691699837916530668735504150390625f,0.4423000068634748149776214631856419146060943603515625f,-0.07219999632537366096585174091160297393798828125f,-0.331199998605251177963282316341064870357513427734375f,-0.5261999996483324171236972688348032534122467041015625f,-0.63640001521110545201764807643485255539417266845703125f,0.065399991041422023130280649638734757900238037109375f,0.58049997892975824242967064492404460906982421875f,0.32349999503791337929214932955801486968994140625f,-0.27780000225901613220003127935342490673065185546875f,-0.181399990773201125193736515939235687255859375f,0.0907000122249126405904462444595992565155029296875f,0.61819998748898541407470474950969219207763671875f,0.9548999511659150840614529442973434925079345703125f,0.59029997253119947941968348459340631961822509765625f,-0.0449999850988387350980701739899814128875732421875f,-0.87359998553395268316990041057579219341278076171875f,-0.529900004437565907977614187984727323055267333984375f,-0.2199999968707562203462657635100185871124267578125f,-0.226099991494417285053941668593324720859527587890625f,-0.4785999965608118689175398685620166361331939697265625f,-0.4801999968245624916818314886768348515033721923828125f,-0.45659999167695641819619822854292578995227813720703125f,-0.5533000005416572886218773419386707246303558349609375f,-0.2792000034630301996685375343076884746551513671875f,-0.2404999769926075003922960604541003704071044921875f,0.26940002707242971524692620732821524143218994140625f,0.402599983268976391315163709805347025394439697265625f,0.8359999583065513828472603563568554818630218505859375f,0.50409998368620889674929230750421993434429168701171875f,-0.23720002337098122779934783466160297393798828125f,0.10499998062849069668800439103506505489349365234375f,0.260399988731741860448209990863688290119171142578125f,0.21220000161528584925463292165659368038177490234375f,0.078899998819828009999355344916693866252899169921875f,0.2365000042170286587150940249557606875896453857421875f,-0.2182000135660171480367353069595992565155029296875f,0.13899998924136180988853084272705018520355224609375f,-0.470500022962689445904516105656512081623077392578125f,0.21000000007450569938782791723497211933135986328125f,-0.0301999979421496700382476774393580853939056396484375f,-0.02820000168234104709341636407771147787570953369140625f,0.00480000380873679688420452293939888477325439453125f,0.2988000039279459230101565481163561344146728515625f,0.36860001630485061951958414283581078052520751953125f,0.91239998573660852088096362422220408916473388671875f,-0.45860002278685574328420671008643694221973419189453125f,0.836600025111437162195215933024883270263671875f,0.467600000107287971928826664225198328495025634765625f,0.77820000775456410480046542943455278873443603515625f,0.2292999900102614674324286170303821563720703125f,-0.48799997863173505407985430792905390262603759765625f,0.1059000060692427691577677251189015805721282958984375f,0.08239998946189874828860411071218550205230712890625f,-0.481799991947412298287645171512849628925323486328125f,0.12700001112371694933500521074165590107440948486328125f,-0.637099999868869559804807067848742008209228515625f,-0.1031999734818942471292757545597851276397705078125f,-0.710099999921024238602740297210402786731719970703125f,-0.371799992096423892462553340010344982147216796875f,0.870199982482194656085994211025536060333251953125f,-0.188400018064677876594714689417742192745208740234375f,0.830900004690885207736528172972612082958221435546875f,0.3445999804973605140645531719201244413852691650390625f,0.7224999909102918227432610365212894976139068603515625f,1.0752000054113566296365434027393348515033721923828125f,0.1150000014901160216851394579862244427204132080078125f,0.98220000271424634519235041807405650615692138671875f,-0.292300015282631164836857351474463939666748046875f,-0.6938000040769569665144445025362074375152587890625f,-0.165499984741210415251089216326363384723663330078125f,-0.2874000033199788362026083632372319698333740234375f,-0.284199984389543747198558776290155947208404541015625f,-0.3855999951675534997974637008155696094036102294921875f,0.164200021344423685576430216315202414989471435546875f,-0.7227999743938446908231298948521725833415985107421875f,0.1530000481456521566059336691978387534618377685546875f,-0.646300010782480338633604333153925836086273193359375f,-0.0036000034153460536145985315670259296894073486328125f,-0.5565000085011124753009426058270037174224853515625f,1.361600049698353132043848745524883270263671875f,0.2269999978244303662222591810859739780426025390625f,1.103800030265748688407256850041449069976806640625f,-0.37819998909085994487355719684273935854434967041015625f,0.363199980783462772482295122244977392256259918212890625f,-0.3290000041052696388987897080369293689727783203125f,-0.156400033092499057829627417959272861480712890625f,-0.55239998737573614562279544770717620849609375f,-0.1608000103950502079896978102624416351318359375f,-0.9697999825000760409920985694043338298797607421875f,0.8069999868720767466356846853159368038177490234375f,0.25499999806284900927266789949499070644378662109375f,0.397599993848800625784178919275291264057159423828125f,0.3679000050485135009381565396324731409549713134765625f,-0.1440000063776967209605572861619293689727783203125f,0.39419998051524185456173654529266059398651123046875f,0.283999992370605358615875957184471189975738525390625f,0.0305999951675535175610320948180742561817169189453125f,0.42449998964741830764779706441913731396198272705078125f,0.496700003369897558069823162441025488078594207763671875f,-0.314199992287159002302132648765109479427337646484375f,-0.5484999836385251459347500713192857801914215087890625f,-0.30889998958110809201826896241982467472553253173828125f,-0.00819998391270626569848900544457137584686279296875f,-0.55239998439550408448894813773222267627716064453125f,0.142700010794400977687246268033050000667572021484375f,0.236799998205900141812207948532886803150177001953125f,0.266999995440244664024476151098497211933135986328125f,0.1814000003844500330529854181804694235324859619140625f,0.412000003337859965757417057830025441944599151611328125f,0.4088000079885123483336428762413561344146728515625f,0.175999998018145475953133427537977695465087890625f,0.387599997872114077068772530765272676944732666015625f,0.13889999881982806329006052692420780658721923828125f,-1.2516975456122736432007513940334320068359375e-08f,0.181000001668929988429823652040795423090457916259765625f,0.304700005228817349944847592269070446491241455078125f,0.1346999994546174139742333863978274166584014892578125f,0.023400006926059813139318066532723605632781982421875f,-0.314899998253583834895152904209680855274200439453125f,-0.2421000049352646232847519058850593864917755126953125f,0.0446000040411949072449715458787977695465087890625f,0.67860000222325300711645468254573643207550048828125f,0.480100021192431558603175290045328438282012939453125f,0.348799999457597653673701643128879368305206298828125f,0.11679999567270282767594835604541003704071044921875f,-0.077800007399916548678220351575873792171478271484375f,0.68620001469850555597673746888176538050174713134765625f,0.8557999960154294427638888009823858737945556640625f,0.32649999512732019724126075743697583675384521484375f,-0.348800022554397681773252770653925836086273193359375f,-0.05190001306533797986730860429815948009490966796875f,0.123199995610117962741014707717113196849822998046875f,-0.317200008320808191086825900129042565822601318359375f,-1.0558999792844054876894688277388922870159149169921875f,-0.458200013566017361199556034989655017852783203125f,-0.20220000012516958776132014463655650615692138671875f,-0.449600006686150965862225348246283829212188720703125f,0.391399987345934174953754336456768214702606201171875f,-0.6987999575853354183863075377303175628185272216796875f,0.2874000134527678884666102021583355963230133056640625f,-0.221500019397586511704645317877293564379215240478515625f,-0.71439997164011026509200519285514019429683685302734375f,-0.23129999198839090723822664585895836353302001953125f,0.24349999049305903753293023328296840190887451171875f,0.08279999451041231139925002935342490673065185546875f,0.30409999303668733272587587634916417300701141357421875f,-1.0434000277876851470182373304851353168487548828125f,-0.02779998705983199869251620839349925518035888671875f,-0.7684000162765387553776008644490502774715423583984375f,-0.31980000127553953603865011245943605899810791015625f,0.047199992525577716406814943184144794940948486328125f,-0.32100000494718539112426469728234224021434783935546875f,-0.4143999833002685040383994419244118034839630126953125f,0.2767999847203495011882523613166995346546173095703125f,-0.43840000308901050374288388411514461040496826171875f,-0.490499968349933368472193251363933086395263671875f,0.54539997986555110998097006813623011112213134765625f,-0.4640999956816431737394168521859683096408843994140625f,-0.0872999916270373876869825835456140339374542236328125f,0.0107999978035688837252337179961614310741424560546875f,-0.13149999840557569097398982194135896861553192138671875f,-0.122899997895955959137381796608678996562957763671875f,0.4387999940931794906617824381100945174694061279296875f,-0.064700011636316945295988034558831714093685150146484375f,0.16079995615482278736863008816726505756378173828125f,-0.748599985235929299420831739553250372409820556640625f,0.04259998300820566630164876187336631119251251220703125f,-0.327999983698129771170215462916530668735504150390625f,0.5923999742627146236628732367535121738910675048828125f,-0.3260000095665451791404620962566696107387542724609375f,-0.482000012055039395164612869848497211933135986328125f,0.365799999368190409398948759189806878566741943359375f,-0.5842000353887681907139040049514733254909515380859375f,0.351900017684698251940744739840738475322723388671875f,-0.6697999851822851180571660734131000936031341552734375f,0.337500044107437613405409138067625463008880615234375f,-0.495700019079446718706094543449580669403076171875f,-0.359700022402405839017092148424126207828521728515625f,-0.2756000156939031331404521552030928432941436767578125f,-0.03290000311136243027476666611619293689727783203125f,-0.151200004416704114618141829851083457469940185546875f,-0.1788000027358531696108912001363933086395263671875f,-0.272499996721744519589947230997495353221893310546875f,0.2780000035166738658887197743752039968967437744140625f,-0.254400012767315164552428541355766355991363525390625f,-0.024899995198846003319204100989736616611480712890625f,0.0834000099807978845234401887864805757999420166015625f,0.35940001202225690946079339482821524143218994140625f,0.3342000055491922694272943772375583648681640625f,0.58050001364946357540475219138897955417633056640625f,-0.28659999575614925060307314197416417300701141357421875f,-0.9202999967269598347030523655121214687824249267578125f,-0.26569999452978365628297297007520683109760284423828125f,0.33219998939633377244007306217099539935588836669921875f,0.8241999737352134758339161635376513004302978515625f,0.28409998875260367157125074299983680248260498046875f,0.6591999972045423650257589542889036238193511962890625f,1.04179996825456644415908158407546579837799072265625f,0.426999986127019059978238146868534386157989501953125f,-0.54660000126957886124046126496978104114532470703125f,-1.049800000679492750776944376411847770214080810546875f,-0.6929000022172926431807127300999127328395843505859375f,0.3026000151574610441684853867627680301666259765625f,0.10490001851916286312871307018212974071502685546875f,-0.255499995246529554293601904646493494510650634765625f,0.44839999992251378113650162049452774226665496826171875f,1.0782000024273987737188917890307493507862091064453125f,0.77520001861751086646989961081999354064464569091796875f,-0.134999998360872286440326206502504646778106689453125f,-0.5627999939143657481821492183371447026729583740234375f,-0.09069999717473986589055812146398238837718963623046875f,0.17199999946355803981390408807783387601375579833984375f,-0.20549999301135546403429543715901672840118408203125f,0.017199999901652329725720846909098327159881591796875f,-0.22599999600648867925656304578296840190887451171875f,-0.08169999571442598806925161625258624553680419921875f,-0.491000013515353384008221837575547397136688232421875f,-0.309999996274709754828791119507513940334320068359375f,-0.663399988001585061425657841027714312076568603515625f,-0.3727999939888715363878191055846400558948516845703125f,-0.2586000122070313267386154620908200740814208984375f,0.01779998732060172894620109218521974980831146240234375f,0.691700027155876195905648273765109479427337646484375f,-0.2150000041723247878877600669511593878269195556640625f,0.5965000316351656284297177990083582699298858642578125f,-0.5351000060677524317043207702226936817169189453125f,-0.5429999755024912388989832834340631961822509765625f,-0.3702999799817801029888642005971632897853851318359375f,0.2865000066012142188043299029232002794742584228515625f,-0.48420000510215699396354693817556835711002349853515625f,0.26850002454221300052239485012250952422618865966796875f,-1.084800036889314700516706579946912825107574462890625f,-0.2957999875962735725209995507611893117427825927734375f,-0.958200023289025093475856920122168958187103271484375f,-0.43310001630857597465507069500745274126529693603515625f,0.18159999743252985826558187909540720283985137939453125f,-0.1973000080369413655745347568881697952747344970703125f,0.177200003030896002798044719384051859378814697265625f,-0.106999996185302759243995751603506505489349365234375f,-0.0026000000700354686244963886565528810024261474609375f,-0.31700000944733641716766214813105762004852294921875f,0.140800003838538856371087604202330112457275390625f,-0.4702000218212607496326427281019277870655059814453125f,0.5316000017166135904744805884547531604766845703125f,0.3709999907910823679912937222979962825775146484375f,0.6266000005617737844687553661060519516468048095703125f,0.0245000048279759852221104665659368038177490234375f,0.75239996666312247697305792826227843761444091796875f,-0.0361000301256775291136591476970352232456207275390625f,-0.445000026859343478236041846685111522674560546875f,-0.63319999933540771497320065464009530842304229736328125f,-0.5466000113651154901361906013335101306438446044921875f,-0.4534000017106531998933860450051724910736083984375f,0.080599998110532755646318037179298698902130126953125f,-0.2771999971821903496760342022753320634365081787109375f,-0.645699985477328208816061305697076022624969482421875f,0.300599981719255726630990466219373047351837158203125f,-0.5168999965995546563846119170193560421466827392578125f,1.07810002478361166566855899873189628124237060546875f,0.307800005388259467764555665780790150165557861328125f,0.90410000641047927327775823869160376489162445068359375f,0.1669999991655348736685482435859739780426025390625f,0.20519998948574080799289731658063828945159912109375f,0.2557999981015921520111078280024230480194091796875f,1.2036000077366821869873092509806156158447265625f,0.661300002735853098556617624126374721527099609375f,0.282399991548061224388987966449349187314510345458984375f,0.186299992714822337802615948021411895751953125f,0.07999999344348918128844161401502788066864013671875f,0.4700000099837779998779296875f,0.35900000324845304788823341368697583675384521484375f,0.129599998900294277603961745626293122768402099609375f,0.30819999598264669060654341592453420162200927734375f,-0.03649999952316296258914007921703159809112548828125f,0.367900015330314911210507489158771932125091552734375f,0.0689999844729900502215969027020037174224853515625f,0.77259998580217370545142330229282379150390625f,0.473699988025426677040741196833550930023193359375f,-0.084400000995397572722822587820701301097869873046875f,-0.45890000299215305545885712490417063236236572265625f,-0.302800002408027513212118719820864498615264892578125f,0.241200003224611325691739693866111338138580322265625f,0.28719999803900719825833220966160297393798828125f,0.0499999933689831976835193927399814128875732421875f,-0.0487999960303306945519352666451595723628997802734375f,-0.15549999330937869235214066065964289009571075439453125f,-0.189999997243285145831492854995303787291049957275390625f,-0.81700000959634788699759155861102044582366943359375f,-1.2139999916255472811599247506819665431976318359375f,-0.81550000552833079581205311114899814128875732421875f,-0.089799994122981896538249202421866357326507568359375f,0.2700000014901160483304920489899814128875732421875f,0.1893000076681374821419012732803821563720703125f,0.1797999963581562443692973829456605017185211181640625f,0.33799998898804195501810454516089521348476409912109375f,0.227899997448921254328269014877150766551494598388671875f,0.04800000187754627223313264039461500942707061767578125f,0.042200004260241896769656477772514335811138153076171875f,0.1890000070482492355861126043237163685262203216552734375f,-0.47839999455809589790078462101519107818603515625f,-0.3579999881684781115609439439140260219573974609375f,-0.42299999241530894522611561114899814128875732421875f,0.31040000668764111679820416611619293689727783203125f,0.443400003349780913453059838502667844295501708984375f,0.0492000003337860203345144327613525092601776123046875f,-0.44440001083016422711580162285827100276947021484375f,-0.2863999855577947784013304044492542743682861328125f,0.02840001296103000782977687777020037174224853515625f,0.39420000510215746913900147774256765842437744140625f,-0.7857999955311412154657091377885080873966217041015625f,-0.3590999794766307484650269543635658919811248779296875f,0.2697999800413850568503448812407441437244415283203125f,-0.34949999346584037507312814341275952756404876708984375f,-0.51860000177621845551811929908581078052520751953125f,0.438799986195564262203561156638897955417633056640625f,-0.52420002350509176114456977302324958145618438720703125f,-1.204500016748905721897244802676141262054443359375f,0.0091999736607082382988664903677999973297119140625f,-1.00310001066476051079234821372665464878082275390625f,0.2680000213980680001668588374741375446319580078125f,-0.73179996959567095160537064657546579837799072265625f,0.1138000378653407640427985825226642191410064697265625f,-0.046299996626377293296172865666449069976806640625f,0.39619998430013669832305822637863457202911376953125f,0.0288999944239856088046281001879833638668060302734375f,-0.08519999246597276254533426254056394100189208984375f,0.215199998426437222320828368538059294223785400390625f,-0.13640000589787970586286292018485255539417266845703125f,0.161900000324845194654699298553168773651123046875f,0.2149999897181988917083117485162802040576934814453125f,0.154699992600083380267506072414107620716094970703125f,0.971000031173229416481262887828052043914794921875f,0.1669999933540822212307830341160297393798828125f,1.0360000332966450287131010554730892181396484375f,0.260599974566698033839884374174289405345916748046875f,-0.4585999774873259493546129306196235120296478271484375f,0.189599991859495220491993450195877812802791595458984375f,-0.185800004807114671478984746499918401241302490234375f,-0.1197999956130981757951303734444081783294677734375f,-0.214000001646578341674143075579195283353328704833984375f,-0.32860001067966226884209390846081078052520751953125f,-0.18079999750554553994419393347925506532192230224609375f,-0.27460000251382588043469468175317160785198211669921875f,1.0091999913930891352720209397375583648681640625f,-0.077999996364116697122881305404007434844970703125f,0.91759999682903270468159462325274944305419921875f,0.0254000001311300849238250521011650562286376953125f,0.273999988198280419737784541212022304534912109375f,-0.0762000173807144687287973283673636615276336669921875f,0.32600000852346422419714144780300557613372802734375f,-0.623199998068808991291689380886964499950408935546875f,0.35800003324449114483485345772351138293743133544921875f,-0.49440001991987259089000872336328029632568359375f,-0.15909999441504485417908654198981821537017822265625f,0.265799996536970173366398739744909107685089111328125f,0.5111999913036824327861040728748776018619537353515625f,-0.13430002070665369995339233355480246245861053466796875f,-0.0750000087916848912072964594699442386627197265625f,0.4153999742031100339545446331612765789031982421875f,-0.10030001343488681442295273882336914539337158203125f,-0.117399999445677138965038466267287731170654296875f,0.24560002418756443631764341262169182300567626953125f,-0.273499990195035902473819078295491635799407958984375f,-0.4935999914944171340636103195720352232456207275390625f,-0.9818999837100506056231097318232059478759765625f,-0.428999993115663613707511103712022304534912109375f,-0.3028000037491320739491129643283784389495849609375f,-0.2639000110387801356637282879091799259185791015625f,0.25880002017021208615688010468147695064544677734375f,0.70680000312328328249122932902537286281585693359375f,1.2716000020146367432971601374447345733642578125f,0.69140000016093239310066564939916133880615234375f,-0.282600002938508954031249231775291264057159423828125f,-0.927700006324052584005812605028040707111358642578125f,-0.4485000011473891934343782850191928446292877197265625f,0.11240000331997901383829230326227843761444091796875f,-0.3277000174999233905737128225155174732208251953125f,0.33719999788999555079271885915659368038177490234375f,-0.2174000015318393597141266582184471189975738525390625f,-0.278999998927116354963118283194489777088165283203125f,-0.1883000004112720393578683797386474907398223876953125f,-0.4218000075936316761726629920303821563720703125f,-0.26700000631809217566114966757595539093017578125f,-0.49510001120865343438026684452779591083526611328125f,0.1637999911129472963011721731163561344146728515625f,0.7690999950110910532430352759547531604766845703125f,0.58710001454651372654325314215384423732757568359375f,0.068999995201826092028341008699499070644378662109375f,0.7196999935686587690497617586515843868255615234375f,0.23490000041425229682090503047220408916473388671875f,0.44919998990297305141439210274256765842437744140625f,0.30159998890161521245545372948981821537017822265625f,0.545399996256828334395549973123706877231597900390625f,0.032800010007620006291517711360938847064971923828125f,-0.47740000212788569200483834720216691493988037109375f,-0.278200000676512626540670680697076022624969482421875f,-0.0923999957203864852317565237171947956085205078125f,0.1480000094771387608716395334340631961822509765625f,0.750699988234043047441446105949580669403076171875f,-1.56819999948441957116074263467453420162200927734375f,0.470800009202957170373338158242404460906982421875f,-0.47589998558014645624325567041523754596710205078125f,0.16939999022930873895376180371385999023914337158203125f,-0.23380000104084597278841783918323926627635955810546875f,-0.044300049614907521799977985210716724395751953125f,-0.14999998316168738909937019343487918376922607421875f,-0.031100011014938644393623690120875835418701171875f,0.0581000458687537246760257403366267681121826171875f,0.6721999625742440809261779577354900538921356201171875f,-0.0171999813124534828290279619977809488773345947265625f,0.61799999964237173344372422434389591217041015625f,-0.335400015777349125301043386571109294891357421875f,0.3159000099062918565095969825051724910736083984375f,0.557600039222836851848796868580393493175506591796875f,-0.36480002414882173322752123567624948918819427490234375f,0.301200021180510812968122991151176393032073974609375f,-0.4413000091433527671824776916764676570892333984375f,0.907000008180737449237085456843487918376922607421875f,-0.285200007367134134028674452565610408782958984375f,1.0021000040411944809193300898186862468719482421875f,-0.369799989578127874523261198191903531551361083984375f,0.517000002741813347029165015555918216705322265625f,-0.98549999934434850956677109934389591217041015625f,0.9368000009626147317476352327503263950347900390625f,-0.537300012898445089604138047434389591217041015625f,-0.0464999876022336167125104111619293689727783203125f,-0.332799996820092314209205142105929553508758544921875f,0.009700001764297638828793424181640148162841796875f,0.13500000879168538858721149154007434844970703125f,0.389999998435377914773880547727458178997039794921875f,0.034800004479289103898054236196912825107574462890625f,0.071299996421485989372968106181360781192779541015625f,-0.24119999525249002925164631960797123610973358154296875f,0.05609999654218544851147498775389976799488067626953125f,1.077100040567666727753248778753913938999176025390625f,-0.81100002469122400849954601653735153377056121826171875f,0.58460002629980467592218928984948433935642242431640625f,0.630700035619735643876992980949580669403076171875f,0.17699998486042023415620860760100185871124267578125f,0.305400010561942991671458003111183643341064453125f,0.221699952054024151948397047817707061767578125f,-0.1910000022649764872539890347979962825775146484375f,0.686200007322430582945571586606092751026153564453125f,0.2429000010997056924200165894944802857935428619384765625f,0.6038000159233807284664408143726177513599395751953125f,0.7712000225961208599301244248636066913604736328125f,0.1129000052347779448069786667474545538425445556640625f,0.2855000345110898507527963374741375446319580078125f,-0.814800006043910673980690262396819889545440673828125f,-0.252899998678266957341520537738688290119171142578125f,-0.65090001929402330205221005599014461040496826171875f,-0.5618000063270329746956122107803821563720703125f,0.0005000002384185631143509453977458178997039794921875f,0.1910000022649764872539890347979962825775146484375f,-0.120399999684095260210625610852730460464954376220703125f,0.08710000132173301157223477275692857801914215087890625f,0.1022999975502489888867785339243710041046142578125f,0.347000030532479630807074499898590147495269775390625f,0.180700011554360440158006895217113196849822998046875f,-0.098799962353705694795280578546226024627685546875f,0.77639999449849117496569306240417063236236572265625f,0.13829999851137397826050801086239516735076904296875f,-0.04330000294446989528296398930251598358154296875f,-0.822400020605325909883731583249755203723907470703125f,-0.2696000052332878649252734248875640332698822021484375f,-1.0964000324964526811299947439692914485931396484375f,-0.51880000392794600738710641962825320661067962646484375f,-0.11420000018477438619157737775822170078754425048828125f,-0.24920001106262201773233755375258624553680419921875f,-1.15340000044405410761783059570007026195526123046875f,-0.2766999957889317851567057005013339221477508544921875f,-0.884400017535686089331647963263094425201416015625f,0.6219999997615810460871443865471519529819488525390625f,0.14059999542832368746303473017178475856781005859375f,0.496400007462501235977470059879124164581298828125f,0.08860000132918355308930813407641835510730743408203125f,-0.092799997565150282863299935343093238770961761474609375f,-0.399699983733892150183919511619023978710174560546875f,0.33859999872148005550798188778571784496307373046875f,0.21489999534785741985842832946218550205230712890625f,0.72360000326633411305010668002068996429443359375f,0.47620000151097752194573331507854163646697998046875f,0.143899996435642218983730344916693866252899169921875f,0.573000001430511218813990126363933086395263671875f,-0.14040000430345511706065053658676333725452423095703125f,-0.216599999704957024704299328732304275035858154296875f,0.6878000071018934846733827725984156131744384765625f,-0.779399990043044166299068820080719888210296630859375f,0.393300006297230719809476795489899814128875732421875f,1.12530000777244509890806511975824832916259765625f,-1.3594000173866742642303506727330386638641357421875f,0.631400023555755751658580265939235687255859375f,-0.8797000014662736955273203420802019536495208740234375f,1.0431999946415422275691753384307958185672760009765625f,-0.502300007683038618466753177926875650882720947265625f,-1.14220000623464557065744884312152862548828125f,1.143399999549984613622655160725116729736328125f,-0.620100023055076832179111079312860965728759765625f,-0.3270999873518938283467605288024060428142547607421875f,0.58300000411272012712515788734890520572662353515625f,-0.187400007045269045846680455724708735942840576171875f,0.6120000399947169711367678246460855007171630859375f,0.050599972704052920136064130929298698902130126953125f,0.30410001542568210197714506648480892181396484375f,1.359099996352195915960692218504846096038818359375f,-1.10079996971488025536700661177746951580047607421875f,0.73090001080036159919472993351519107818603515625f,-0.5666000088691713898469970445148646831512451171875f,1.209399999207258158406830261810682713985443115234375f,-0.361500012636184919756487943232059478759765625f,-0.699199998843669678905143882730044424533843994140625f,0.78079999415278422691955029222299344837665557861328125f,-0.39120000784397124249380794935859739780426025390625f,-1.307599986398219282790478246170096099376678466796875f,1.802599980363249532189229284995235502719879150390625f,-0.75550001066923133663522094138897955417633056640625f,0.69499999284744173877470529987476766109466552734375f,-1.0181999968767161401927978658932261168956756591796875f,0.4047000071287154554511289461515843868255615234375f,0.48589998409002976842430143733508884906768798828125f,-0.39020000360459061727880225589615292847156524658203125f,0.25260000362768753046083247681963257491588592529296875f,-0.007599995934963743593471008352935314178466796875f,-0.19259999809563144168578219250775873661041259765625f,0.0204999954700468833834747783839702606201171875f,0.377800001513957539600596646778285503387451171875f,-0.72139999680817101790353262913413345813751220703125f,0.2339000059723854718640723149292171001434326171875f,-0.029600016558169794933519369806163012981414794921875f,0.2082000083506105880815084674395620822906494140625f,0.9573999960929153729694007779471576213836669921875f,0.1855999988183378857087291180505417287349700927734375f,0.0027000034928317262483687954954802989959716796875f,0.5962000199884176510067845811136066913604736328125f,0.23600000319629910183749643692863173782825469970703125f,0.04930000610351559231503415503539144992828369140625f,-0.917800038543343799091189794125966727733612060546875f,-0.1358000091657043384429925936274230480194091796875f,-0.19870003317594608915896969847381114959716796875f,0.408199991437792508719439865672029554843902587890625f,0.1929999969229101441925422477652318775653839111328125f,-0.0489000029921538015287296730093657970428466796875f,0.984800010514259582805607351474463939666748046875f,0.2041000057771802378425718416110612452030181884765625f,0.6183000303626062787998307612724602222442626953125f,-0.988400011396408206820751729537732899188995361328125f,-0.3379000046387314881712882197462022304534912109375f,-1.22110004230737700936515466310083866119384765625f,0.32699999938905222052198951132595539093017578125f,0.14969999848604198522394881365471519529819488525390625f,0.13540000251531569830376611207611858844757080078125f,-1.136200012686849003529232504661194980144500732421875f,-0.3263000075042248315781989731476642191410064697265625f,-1.3506000471353534209129065857268869876861572265625f,-0.709999992400407453629895826452411711215972900390625f,-0.3972999939367174437876428783056326210498809814453125f,-1.0276999987244597178914773394353687763214111328125f,0.0292000167995691561895199583887006156146526336669921875f,0.080000002793967739389557891627191565930843353271484375f,0.077000010117888706417943467386066913604736328125f,-0.30999999247491361042960988925187848508358001708984375f,0.043600000621378420806451003954862244427204132080078125f,-0.091800003272294983247547861537896096706390380859375f,0.82559999393820771018681625719182193279266357421875f,0.223500000588595870709696100675500929355621337890625f,0.793300014865398139818353229202330112457275390625f,0.8093999931722877239081981315393932163715362548828125f,0.21599999790638679808552069516736082732677459716796875f,0.27590000841617534632632668945007026195526123046875f,-0.85539999543726408237631630981923080980777740478515625f,-0.2917999960638582024330389685928821563720703125f,-0.462700007311254102315700720282620750367641448974609375f,-0.718000008136034040262529742904007434844970703125f,0.311199999946355720936708166846074163913726806640625f,1.62909999605417254997519194148480892181396484375f,-1.3213999689430000472611936856992542743682861328125f,0.87740001270771017516381107270717620849609375f,0.92730004687309275368534144945442676544189453125f,-0.3366000143080949413842972717247903347015380859375f,0.4828000217795374027218713308684527873992919921875f,-1.4397000372290609249148474191315472126007080078125f,1.542000031873583143493533498258329927921295166015625f,-0.795700041580200689850244089029729366302490234375f,0.6803000050902365813954020268283784389495849609375f,-0.056599980631470980796393632772378623485565185546875f,0.344199996161460575194723787717521190643310546875f,1.2286999899625783427836722694337368011474609375f,-1.607599975371361011866611079312860965728759765625f,0.701500012636184777647940791212022304534912109375f,-0.7440999886035921662141845445148646831512451171875f,1.346999979496002364243167903623543679714202880859375f,-0.45960000135898582129811984486877918243408203125f,0.42200000751018507116896216757595539093017578125f,-0.152399994304776242870502755977213382720947265625f,0.222200000572204459103886620141565799713134765625f,-0.9207999796986587881519881193526089191436767578125f,1.785199966463447029951794320368207991123199462890625f,-0.570200003790855447505236952565610408782958984375f,-0.3585000064373016215313327847979962825775146484375f,0.387599972242117285503582024830393493175506591796875f,-0.22009999146461467489643837325274944305419921875f,-0.03819997065067315400455072449403814971446990966796875f,0.2363999719977381841484742608372471295297145843505859375f,-0.0275999907940625011093516150140203535556793212890625f,-0.50799997181445366578600442153401672840118408203125f,0.91179997730702166958138832342228852212429046630859375f,-0.29690000149831174969250469075632281601428985595703125f,0.809899975752831124964359332807362079620361328125f,-1.26179995558857971360566807561554014682769775390625f,0.47959999718666068702077609486877918243408203125f,1.15509998908042899046222373726777732372283935546875f,-1.0021999695032837163211070219404064118862152099609375f,0.631000001966953050214215181767940521240234375f,-1.070099999678879587605706547037698328495025634765625f,0.87479997765719896829494928169879131019115447998046875f,-0.58690000224336957845849838122376240789890289306640625f,-0.3629999817609785850436310283839702606201171875f,0.3939999822378155158730805851519107818603515625f,0.6351999990224832526308773594792000949382781982421875f,0.3673999922186135247414995319559238851070404052734375f,0.9078000012159339604522756417281925678253173828125f,-0.9040000281333926412941082162433303892612457275390625f,-0.3239000086173415837720312993042171001434326171875f,-0.792500037252903410944782081060111522674560546875f,-0.2828000055372716925461418213672004640102386474609375f,-0.085600003549456626927849356434307992458343505859375f,-0.4046000162601472993628703989088535308837890625f,0.682200006458163255018689596909098327159881591796875f,0.1893000017821788727445664335391484200954437255859375f,0.7286000259160996250784592120908200740814208984375f,-0.17399999698996548858076494070701301097869873046875f,-0.05070000019222498099935592108522541821002960205078125f,-0.49030001015663149388501551584340631961822509765625f,-0.303199997472762827754877434927038848400115966796875f,-0.0381999978452920441185369782033376395702362060546875f,0.0225999950408937166912437533028423786163330078125f,0.0259999973475932844024782752967439591884613037109375f,-0.08120000091493129179553989160922355949878692626953125f,-0.33860000952482227631890054908581078052520751953125f,0.191800005134940221296346862800419330596923828125f,0.120200001294910929772186136688105762004852294921875f,-0.235100010240078205470126704312860965728759765625f,-0.09839999284446243610346982677583582699298858642578125f,0.138299999368190762449870589989586733281612396240234375f,0.646100005501508700689328179578296840190887451171875f,-0.0398000105887650335034777526743710041046142578125f,-0.0979000009506940660486407068674452602863311767578125f,0.3336000002861023716604904620908200740814208984375f,-0.4702000131040813268867850638343952596187591552734375f,-0.3308000055894255719834973206161521375179290771484375f,-0.7131000295519829279555779066868126392364501953125f,-0.487600003981590202073448381270281970500946044921875f,-0.076399999788403505363021395169198513031005859375f,-0.61420001761913312066099024377763271331787109375f,0.0105999896913766544770396649255417287349700927734375f,-0.1258000017896294675523449768661521375179290771484375f,-0.0621000022530555639832527958787977695465087890625f,0.34060001323521138516525752493180334568023681640625f,0.222500000298023248301859666753443889319896697998046875f,0.814600006872415516312457839376293122768402099609375f,0.76760002081990297284619373385794460773468017578125f,-0.241700011658668501013380591757595539093017578125f,0.32380001718997863946469806251116096973419189453125f,-0.2903999992370602800662027220823802053928375244140625f,0.1957999978780744498862986802123486995697021484375f,-0.90650002813339281004800795926712453365325927734375f,1.0088000052690506702646189296501688659191131591796875f,-0.5175000184774400935339144780300557613372802734375f,-0.17819999449253121071023997501470148563385009765625f,0.4595999910771848551149787454050965607166290283203125f,-0.1184000006675720584325972595252096652984619140625f,0.6147999893546103322705675964243710041046142578125f,-1.059199988785386015166523066000081598758697509765625f,0.371200003671646072689327411353588104248046875f,0.062500018775462962139499722979962825775146484375f,0.41039998567700397558155600563623011112213134765625f,0.0031000063061714655532341566868126392364501953125f,-0.5417999923944467610681385849602520465850830078125f,0.493999997138976798538578805164434015750885009765625f,-0.293600003266334397267200984060764312744140625f,0.32499999836087223314962102449499070644378662109375f,-0.2335999973058699996641962570720352232456207275390625f,0.166200006726384241773075700621120631694793701171875f,0.4587000313878064616801566444337368011474609375f,0.203200002166628479471910395659506320953369140625f,0.21730000872612009743534144945442676544189453125f,-0.70009998412579310667069876217283308506011962890625f,0.48259998494535694124607516641844995319843292236328125f,-0.35900000632181761073269399275886826217174530029296875f,-0.26720003603696795835276134312152862548828125f,-0.4917999778658153076094095013104379177093505859375f,-0.1063000085473060352114771376363933086395263671875f,-0.18430001832246833970430088811554014682769775390625f,-0.0138000047475095311000359288300387561321258544921875f,-0.1081999956846235733110006549395620822906494140625f,-0.3983999836802478711206276784650981426239013671875f,1.052599995338916460951850240235216915607452392578125f,-0.25930000506043437979997179354541003704071044921875f,0.11889996961355198124010712490417063236236572265625f,-0.5251999846428632157113725043018348515033721923828125f,0.08089999738931652473183930851519107818603515625f,-0.072599975520372339587993337772786617279052734375f,-0.20980000954568378546127860317938029766082763671875f,-0.00949999395012834924045819207094609737396240234375f,1.398900016999245821125441580079495906829833984375f,-0.263900012379884696400722532416693866252899169921875f,0.18479999807178970616661217718501575291156768798828125f,-0.076799999211728586789860173666966147720813751220703125f,-0.242399991101026390794004328199662268161773681640625f,0.121400002358853897899138019056408666074275970458984375f,0.0982000018872320745710879918988212011754512786865234375f,0.128100005002319772273722264799289405345916748046875f,-0.028000006645917618897101419861428439617156982421875f,-0.1102999989435076155785964147071354091167449951171875f,0.3683000005602838911045182612724602222442626953125f,-0.097399994155764613168457799474708735942840576171875f,-0.0456999992609024019429853069595992565155029296875f,-0.6006000083923339616376324556767940521240234375f,0.709400005838274783087626929045654833316802978515625f,0.16899999985843894290127309432136826217174530029296875f,-0.08429999477863336920790970907546579837799072265625f,-0.12659998845458042548983712549670599400997161865234375f,0.076500000230967979408802648322307504713535308837890625f,0.308600011461973355864074619603343307971954345703125f,-0.4086000115364789575522763698245398700237274169921875f,0.0131999976590275736043622600845992565155029296875f,0.008499981105327236718949279747903347015380859375f,0.511400039425492280997787020169198513031005859375f,0.33850000591576101527380160405300557613372802734375f,0.69080003364086195460913586430251598358154296875f,0.677200011599063511624763123109005391597747802734375f,0.3799999973922967289041707772412337362766265869140625f,0.9142000170230861755271689617075026035308837890625f,-0.23880002896189722605413408018648624420166015625f,-0.1008000080853701518890375155024230480194091796875f,0.3813999943494792432829854078590869903564453125f,0.181400001576542901915445327176712453365325927734375f,0.25780000095516442115695099346339702606201171875f,0.87450001555681211584669654257595539093017578125f,0.47460000813901448424303453066386282444000244140625f,0.0522000072404742443410441410378552973270416259765625f,-0.1308999705672260205346901784650981426239013671875f,-0.175999997496604887459170640795491635799407958984375f,0.0260000015199184364433904192992486059665679931640625f,-0.2403999998331070031554190791212022304534912109375f,-0.1412000014364718225579053978435695171356201171875f,0.0566000026106834308592397064785473048686981201171875f,0.3840000191926957739951831172220408916473388671875f,-0.195599982762336477293274583644233644008636474609375f,-0.2202000042378902122663930640555918216705322265625f,0.5381999815657732000317992060445249080657958984375f,-0.61179999474138002657497281688847579061985015869140625f,0.29190000447854391740776236474630422890186309814453125f,0.082100007170439137649253780182334594428539276123046875f,0.085400005942582790652295443578623235225677490234375f,0.04059999822229143351393076954991556704044342041015625f,-0.2655000324249261467457472463138401508331298828125f,-0.587399977391958483252665246254764497280120849609375f,-0.100600004816055133005647803656756877899169921875f,0.27640002340674385550300939939916133880615234375f,0.317599983343482161757265203050337731838226318359375f,0.112400010547041961928016462479718029499053955078125f,1.2551000159025189617523210472427308559417724609375f,-0.803199995461106208693990993197076022624969482421875f,0.6649000159859657088645690237171947956085205078125f,-0.48139998172074616178406358812935650348663330078125f,0.44979997046738906529839141512638889253139495849609375f,-0.25490000054463746170796412116033025085926055908203125f,-0.74289999715089738430151555803604423999786376953125f,0.9504000127226108762812373242923058569431304931640625f,-0.4147000160694123138682698481716215610504150390625f,0.270600062930583362685865722596645355224609375f,-0.033200052830576254336847341619431972503662109375f,0.153200021314621182000337284989655017852783203125f,0.3888000267267219811628820025362074375152587890625f,-0.332600011432170550307318990235216915607452392578125f,0.2305000038146971519381622783839702606201171875f,-0.6562000591039662822367972694337368011474609375f,-0.0899999700486660003662109375f,-0.318400027489662562629746389575302600860595703125f,-0.4158999967932697217065651784650981426239013671875f,0.260200010719894958555187258752994239330291748046875f,-0.2012000146985055692994137643836438655853271484375f,0.919899990355968810717968153767287731170654296875f,-0.75819998920261877373150127823464572429656982421875f,0.4930000067949293907076935283839702606201171875f,-0.1539999855160711916823856881819665431976318359375f,0.615999992430209974969557151780463755130767822265625f,-0.10920000286698350322467376827262341976165771484375f,-0.5528000150740144391647845623083412647247314453125f,0.36040000579357123200452406308613717555999755859375f,-0.285400016745925189098898044903762638568878173828125f,-0.3199000019788744708648664527572691440582275390625f,0.917100052320957814799839979968965053558349609375f,0.744400020366907000379796954803168773651123046875f,0.43250000294297930025777532136999070644378662109375f,1.31290004423856743187570828013122081756591796875f,0.34680000014305090161315092700533568859100341796875f,0.24389999572783704895329037753981538116931915283203125f,0.76069999866485549233630081289447844028472900390625f,-0.8846000029236076311889291901024989783763885498046875f,-0.302499998807907066744604662744677625596523284912109375f,-0.580200012880563331663097415002994239330291748046875f,-0.338000002920627762303951158173731528222560882568359375f,-0.02170000290423634847769562838948331773281097412109375f,-0.106900009638071225026578758843243122100830078125f,0.0518000020802020788579511645366437733173370361328125f,0.167500000633299339636295144373434595763683319091796875f,0.21160000238716580867048833169974386692047119140625f,0.8457999949723478128049691804335452616214752197265625f,0.349699995394050999752977304524392820894718170166015625f,0.91640001554638084169113199095590971410274505615234375f,0.1170000130981208030789275653660297393798828125f,0.129300003793835660559352618292905390262603759765625f,0.025600008988380551500085857696831226348876953125f,-0.513400006553530463548895568237639963626861572265625f,-0.17059999904185529917555186329991556704044342041015625f,0.16709999957084686883490576292388141155242919921875f,-0.088600013026594925946710645803250372409820556640625f,-0.240200000549852799025529748178087174892425537109375f,-0.734900025522708943981342599727213382720947265625f,0.609000039532781034523623020504601299762725830078125f,0.310800010432303042140489424127736128866672515869140625f,0.966900042420626437689179510925896465778350830078125f,0.20640004024505653745791278197430074214935302734375f,0.177600012624263836613636158290319144725799560546875f,-0.120799963903426288425180246122181415557861328125f,0.7090000021308657807139752549119293689727783203125f,0.2581999975845217232262029938283376395702362060546875f,0.2611000029683108181188799790106713771820068359375f,-0.2825999848335982722602466310490854084491729736328125f,0.08850000025331972164366334254737012088298797607421875f,0.21740000764131561794556546374224126338958740234375f,-0.31680001713037508892512050806544721126556396484375f,-0.107700007106363793152326024937792681157588958740234375f,0.068899972444772306090499114361591637134552001953125f,0.6782000309258702230863491422496736049652099609375f,0.5307000171422959766687199589796364307403564453125f,0.2001000182867045396051253192126750946044921875f,-0.455200013178586715412166086025536060333251953125f,0.14949999916553480261427466757595539093017578125f,-0.032299987566471344280216726474463939666748046875f,-0.10619999651908873516958919935859739780426025390625f,0.0051999969363212272810415015555918216705322265625f,-1.03340000647902474639749925700016319751739501953125f,0.9517999827831984394066466848016716539859771728515625f,-0.57430000655055035707619026652537286281585693359375f,-0.5184999915361407918368286118493415415287017822265625f,0.7043999872863293543190366108319722115993499755859375f,-0.293700006204843599988407731871120631694793701171875f,-0.13279998273849458190198902229894883930683135986328125f,0.49060000176131690619030223388108424842357635498046875f,-0.0831000060826541098180086919455789029598236083984375f,0.70219999211579509079683703021146357059478759765625f,-0.8327999775975942764461024125921539962291717529296875f,0.4097999943837520309131150497705675661563873291015625f,0.134000028431415785235003568232059478759765625f,0.27739998700320711577660404145717620849609375f,0.056300005865097091373172588646411895751953125f,-0.9631000262737270389834520756267011165618896484375f,0.324400005093216758922380904550664126873016357421875f,-0.4955000136494636109318889793939888477325439453125f,0.4740999811530119956159978755749762058258056640625f,-0.458799974200129856427565755438990890979766845703125f,0.2421000073194505830542766489088535308837890625f,0.2595000446140769412295412621460855007171630859375f,-0.501600031742453378313939538202248513698577880859375f,0.17370001931786571702787114190869033336639404296875f,0.377600060606003040675204829312860965728759765625f,0.35759995212554951393713054130785167217254638671875f,0.17240002164840717568949912674725055694580078125f,0.96750001639127702901532757095992565155029296875f,-0.5953999909669160928160636103712022304534912109375f,0.518800008249282740280250436626374721527099609375f,-0.70039996652901237439436954446136951446533203125f,0.8887999586284165420835279292077757418155670166015625f,-0.3859999999925494140740056536742486059665679931640625f,-0.568700016337633229568382375873625278472900390625f,0.20680001236200329373104978003539144992828369140625f,-0.29390000939965243986762288841418921947479248046875f,0.29080004258155867091772961430251598358154296875f,-0.5215000111460685872089015902020037174224853515625f,0.63120000084042526822258878382854163646697998046875f,0.204900001122057362490380683084367774426937103271484375f,0.005100024694204119413143416750244796276092529296875f,-0.076599991956353363775633624754846096038818359375f,0.0319000012934207433090705308131873607635498046875f,-0.38739998826980581725365482270717620849609375f,-0.20620000538527971656321824411861598491668701171875f,-0.115400002701580517427970562494010664522647857666015625f,-0.666500027686357743306189149734564125537872314453125f,-0.107999997407197678711554544861428439617156982421875f,-0.1340999957934021491468001840985380113124847412109375f,0.0033000130772595781536438153125345706939697265625f,-0.47199999439716311400161430356092751026153564453125f,-0.145399994841217961027979299615253694355487823486328125f,0.084600020581484525195037349476478993892669677734375f,-0.6258000252217057468584471280337311327457427978515625f,-0.1899000092059374422692741291029960848391056060791015625f,-0.2786000347822910061523771219071932137012481689453125f,0.136600010731816379205838529742322862148284912109375f,0.05450000542402266834329793709912337362766265869140625f,-0.23859997674226729458268891903571784496307373046875f,0.449399992874264686548713143565692007541656494140625f,0.148199998851120451348606366082094609737396240234375f,0.885300016415119017665347200818359851837158203125f,-0.6179999904036517222039037733338773250579833984375f,-0.1223999949008225396340776569559238851070404052734375f,0.0673999966144567252968045067973434925079345703125f,-0.23160001110434524917991438996978104114532470703125f,-0.06360000363886353369480275432579219341278076171875f,0.37259998013973216757221962325274944305419921875f,1.01099999265372719747801966150291264057159423828125f,0.370499994874000482258935562640544958412647247314453125f,1.090200010198353997026288197957910597324371337890625f,0.201600004696846024643264172482304275035858154296875f,0.11100000159442424685352079904987476766109466552734375f,-0.13899999862909329095828070421703159809112548828125f,-0.389600002774596276822194340638816356658935546875f,-0.091600001977384104545620857606991194188594818115234375f,-0.26680001094639305136979601229541003704071044921875f,0.12019999432861805388483844581060111522674560546875f,0.047799996708333514217059700968093238770961761474609375f,0.411099984043836297331608875538222491741180419921875f,-0.291599993968010284817182764527387917041778564453125f,0.030100010761618722909815915045328438282012939453125f,1.035299996596574345630870084278285503387451171875f,-0.52339998978972435139667140902020037174224853515625f,0.54420000018477399095218061120249330997467041015625f,0.126600017809867182450034306384623050689697265625f,0.4949999745935205197611139738000929355621337890625f,0.03410000216364839076277348794974386692047119140625f,0.1597000133872032989756917231716215610504150390625f,0.2943999986857175343857306870631873607635498046875f,0.053100007423758643909650345449335873126983642578125f,-0.1365000209808346909312604111619293689727783203125f,-0.331799997907876953462391611537896096706390380859375f,-0.05460000016689292579030734486877918243408203125f,-0.841000017613172445862801396287977695465087890625f,0.38079998595714581455240477225743234157562255859375f,-0.43820000954270366122500490746460855007171630859375f,-0.83620002267062698564359379815869033336639404296875f,0.7102000107198953582354761238093487918376922607421875f,-0.457900017192959918332917368388734757900238037109375f,0.3914000086545943446481032879091799259185791015625f,0.048199988904595425509569395217113196849822998046875f,0.1899000034689901639239906216971576213836669921875f,0.1040999695301056959806373924948275089263916015625f,-0.798599971601367197848730938858352601528167724609375f,0.10619998921751960807569048483856022357940673828125f,-1.09999998748302374451668583787977695465087890625f,0.8077999842286107679001361248083412647247314453125f,-0.587300002765655193570637493394315242767333984375f,-0.653400014376639859392525977455079555511474609375f,0.05600001671910259659625808126293122768402099609375f,-0.323400008416175666070557781495153903961181640625f,0.863999988347291747459166799671947956085205078125f,-1.19399996472895164600913631147705018520355224609375f,0.51059999662041644086230007815174758434295654296875f,0.4114000190854074645585569669492542743682861328125f,0.095200000214576530055410330533050000667572021484375f,0.2002000046849250480818227515555918216705322265625f,-0.46539999536275888925018762165564112365245819091796875f,0.60099998766183870824164614532492123544216156005859375f,-0.2661000034898519661652471768320538103580474853515625f,-0.100700001198053001871812739409506320953369140625f,-0.26519998434484026716972948634065687656402587890625f,-0.03030000017285328084426510031335055828094482421875f,-0.9443999683618553575570331304334104061126708984375f,0.222099974387883758453199334326200187206268310546875f,0.540399998492002708871950744651257991790771484375f,0.01610000322908167191826578346081078052520751953125f,-0.19029999734163283164889435283839702606201171875f,-0.978000028401613263895342242904007434844970703125f,-0.297000004976987863614112939103506505489349365234375f,-1.000000040531158873591266456060111522674560546875f,-0.67960003965497062239364822744391858577728271484375f,-0.336200012761354527501822531121433712542057037353515625f,-1.193200061100722297879883626592345535755157470703125f,0.468000013947487136789504802436567842960357666015625f,0.17250000599771742049171052713063545525074005126953125f,0.83950003433227582405606881366111338138580322265625f,0.742599995860456996155107844970189034938812255859375f,0.154699995505809706486388677149079740047454833984375f,-0.012599998617172758486049133352935314178466796875f,-0.27279999272227239970334267127327620983123779296875f,-0.3217999938100575274546599757741205394268035888671875f,-0.9334000102043145119523614994250237941741943359375f,0.55740002246797093476970985648222267627716064453125f,0.32100000409036877346835581192863173782825469970703125f,0.91890002593994157820134205394424498081207275390625f,-0.054400005465745859822845886810682713985443115234375f,-0.044599998900294313131098533631302416324615478515625f,0.253800014805793949790313490666449069976806640625f,0.1752000353813174537975783096044324338436126708984375f,0.1836000092267990879690842120908200740814208984375f,0.013600019955635644919311744160950183868408203125f,-0.72020002845227748533574185785255394876003265380859375f,-0.28960000730082402498055671458132565021514892578125f,-0.8169000277988616165458779505570419132709503173828125f,0.00699999812245366559437798059661872684955596923828125f,0.079999996796250361086322300252504646778106689453125f,0.39309998306035964787952252663671970367431640625f,0.6992000204503536764377713552676141262054443359375f,0.254800001424551059159284704946912825107574462890625f,1.005600013160705685777429607696831226348876953125f,0.004399993395805434914791476330719888210296630859375f,0.1827999977141618881404383500921539962291717529296875f,0.7445999888420100631947207148186862468719482421875f,0.701600022131204870134979501017369329929351806640625f,0.0964000083565712539979131179279647767543792724609375f,-0.1627999737977976479896824457682669162750244140625f,0.3223999922931195527553427382372319698333740234375f,-0.40050001022219650081979125388897955417633056640625f,1.09609998955726695157864014618098735809326171875f,-1.02639997065663379771649488247931003570556640625f,0.5919000109791756614185942453332245349884033203125f,-0.90200001943111374202999286353588104248046875f,1.388000017970799859767794259823858737945556640625f,-0.536000021338463028541809762828052043914794921875f,-0.212400028055906631152538466267287731170654296875f,0.66120002617239936881787798483856022357940673828125f,-0.16180001221299189495539394556544721126556396484375f,0.1594999946951869418398928246460855007171630859375f,-0.713999996542930670528903647209517657756805419921875f,0.1295000012516975829157672706060111522674560546875f,1.2693999990582458536891863332130014896392822265625f,-0.972199988129734737896114893374033272266387939453125f,0.680900010204315009332276531495153903961181640625f,0.322800000619889004838114487938582897186279296875f,-0.30100000628829004511999301030300557613372802734375f,0.154700019645691355663075228221714496612548828125f,0.197100031757354798855885746888816356658935546875f,-0.32120001507103435045564765459857881069183349609375f,0.1335000085234643307785518118180334568023681640625f,-0.2596000299692153845398934208787977695465087890625f,-0.194599988991022154749543915386311709880828857421875f,-0.116900011277198956349820946343243122100830078125f,0.22280006380081207595367231988348066806793212890625f,0.3851999572992326204001756195793859660625457763671875f,0.0962000215530398605778827914036810398101806640625f,-0.575300034799427262299786889343522489070892333984375f,0.76180001213848591579136382279102690517902374267578125f,-0.34060001884177359965377718253876082599163055419921875f,-0.30449998743087025587783500668592751026153564453125f,0.12900000462681038282397594230133108794689178466796875f,-0.15020000049397330688094598372117616236209869384765625f,0.41780002624988554771334747783839702606201171875f,-0.9562000103771683967579519958235323429107666015625f,0.273700017976761245108718867413699626922607421875f,-0.6063999943494788880116175278089940547943115234375f,0.396200020956992471354851659270934760570526123046875f,-0.30590001378059383796426118351519107818603515625f,1.3294000190258028482048757723532617092132568359375f,-1.054600007170438669135137388366274535655975341796875f,0.711300025534630009360625990666449069976806640625f,0.81659998800754607373164617456495761871337890625f,0.8513000156998629108784371055662631988525390625f,-0.2020000109374524566163700001197867095470428466796875f,-0.09320000421553852054667999027515179477632045745849609375f,-0.35340002201348574839556704318965785205364227294921875f,0.92100001113116736206620771554298698902130126953125f,0.17130000146925450366097720689140260219573974609375f,0.19840001199245449470254243351519107818603515625f,-0.7088000027358531962562437911401502788066864013671875f,-0.4066000017911195474340502187260426580905914306640625f,-0.9650000244379040026387883699499070644378662109375f,-0.832200014057755677043815012439154088497161865234375f,-0.1890000057443976810844787905807606875896453857421875f,-0.3547000247120859484084576251916587352752685546875f,-0.516000003159046372047669137828052043914794921875f,-0.0174000050336122757954626649734564125537872314453125f,0.2113999830245969491215873858891427516937255859375f,0.6426000056952236860752236680127680301666259765625f,0.454500000134110404559351081843487918376922607421875f,1.296000027298926937646683654747903347015380859375f,0.28940004219710857569225481711328029632568359375f,0.1483000124067068714595052369986660778522491455078125f,-0.1899999594688406290288185118697583675384521484375f,0.1078000028550624989076567317169974558055400848388671875f,0.04460000072568655327120978881794144399464130401611328125f,0.05930000316053629649815093216602690517902374267578125f,-0.1359999854266644714329004273167811334133148193359375f,0.0457000001922249765584638225845992565155029296875f,-0.287300005447864492680309922434389591217041015625f,-0.36139998891055569174568518064916133880615234375f,-0.289299996566772410488965761032886803150177001953125f,-0.875400004005431497944300645031034946441650390625f,0.6274000510782007378196567515260539948940277099609375f,0.3197000137597323377036673264228738844394683837890625f,0.3320000456571587932330658077262341976165771484375f,0.25159999843835822996851447896915487945079803466796875f,-0.03140000112950802613198675317107699811458587646484375f,0.16120000545978552253245652536861598491668701171875f,-0.55680002011060725219948608355480246245861053466796875f,-0.2319000051304698217791155911982059478759765625f,-0.4133000247001650251377213862724602222442626953125f,0.2807999940037728681119233442586846649646759033203125f,0.01800000180304051067281534415087662637233734130859375f,0.31560001465082176963505844469182193279266357421875f,0.349999996721744199845716138952411711215972900390625f,0.233500001072883378583355806767940521240234375f,-0.06700000307708986468924194923602044582366943359375f,0.2244000060617923342221047278144396841526031494140625f,-0.0512000016972422589134339432348497211933135986328125f,1.430000011026859141338718472979962825775146484375f,-1.39859999350607377976984935230575501918792724609375f,0.784700022625923310215512174181640148162841796875f,-0.39720000534057664509646201622672379016876220703125f,0.3145999752819543804349677884601987898349761962890625f,-0.2350999902725217083343522972427308559417724609375f,-1.1513000061631206705214935936965048313140869140625f,1.205599988648295539661603470449335873126983642578125f,-0.6395000158548356239407439716160297393798828125f,-1.06699998784065286372424452565610408782958984375f,0.95679998136758825921788229607045650482177734375f,-0.5738000097393989307192896376363933086395263671875f,0.02040001026391990990305203013122081756591796875f,-0.1297999876409772213037285837344825267791748046875f,0.04469998745918246640940196812152862548828125f,0.588600057208537918995716609060764312744140625f,0.08519994619786785960968700237572193145751953125f,0.288800026130676457114532240666449069976806640625f,0.13070000488609079258850442784023471176624298095703125f,-0.0776000006288290189448986211573355831205844879150390625f,0.070500002138316641975279708276502788066864013671875f,-0.05749995678663299969457511906512081623077392578125f,0.6345999636590484715981119734351523220539093017578125f,-0.06239998812079416978804147220216691493988037109375f,0.108400008714198747838963754475116729736328125f,0.00279996880590971386482124216854572296142578125f,0.03500001847744016458818805404007434844970703125f,0.74360007389783877584932270110584795475006103515625f,-0.1906000525742765905334863418829627335071563720703125f,0.3903000316143039327698716078884899616241455078125f,0.348399982413649222934282079222612082958221435546875f,-0.76599999444186661623490408601355738937854766845703125f,0.2118000045016408261489004871691577136516571044921875f,-0.597700026440620479917242846568115055561065673828125f,0.44720001189708702415970265064970590174198150634765625f,-0.32950001257658012576712280861102044582366943359375f,0.26819996923506295161132584325969219207763671875f,-0.6973999737411740174053420560085214674472808837890625f,0.1718999938055871812281338861794210970401763916015625f,-0.1074999883770946240701960050500929355621337890625f,0.4032000231027605963163296109996736049652099609375f,-0.876199988099932713936368600116111338138580322265625f,-0.2430999979615211525896256716805510222911834716796875f,-1.1002000073671336366487594204954802989959716796875f,-0.015199999916553519341277933563105762004852294921875f,-0.039700001801550366398174674031906761229038238525390625f,-0.470700024443865050471913491492159664630889892578125f,0.565599993640184539600568314199335873126983642578125f,0.0209000026419758899720591216464526951313018798828125f,-0.2932999940037728237030023592524230480194091796875f,1.0324000124096872621493048427510075271129608154296875f,0.2534000060319900882177535095252096652984619140625f,0.520000029206276082049953402020037174224853515625f,0.67339999612271750351055743522010743618011474609375f,0.24849999839067449247664853828609921038150787353515625f,0.14180001891851379269837707397527992725372314453125f,-0.060199999991059183201969062793068587779998779296875f,0.0113000015437603185119996851426549255847930908203125f,-0.2342000056982040945285916677676141262054443359375f,-0.900600028881430869631685709464363753795623779296875f,-0.344400008706748528286567534451023675501346588134765625f,-1.041500049740076594417814703774638473987579345703125f,-0.609400037279725470540370224625803530216217041015625f,-0.388700010600686152173466325621120631694793701171875f,-1.112400044894219064417484332807362079620361328125f,0.020000006407499437699470945517532527446746826171875f,0.15850000155717136607336215092800557613372802734375f,0.716300009739399001773563213646411895751953125f,-0.774400035566091826666479391860775649547576904296875f,-0.18830001140087848643389634162303991615772247314453125f,-0.16930004723072133998584831715561449527740478515625f,-0.1559999986886977385580621557892300188541412353515625f,-0.09899999710172414923992079138770350255072116851806640625f,-0.46939999231547091529392901065875776112079620361328125f,0.5293999971956011574292233490268699824810028076171875f,0.25689999592900270020123798531130887567996978759765625f,0.75100000792741727906332016573287546634674072265625f,0.87500003017485195044855572632513940334320068359375f,0.194200012589991199973837865400128066539764404296875f,0.046500033199787793591895024292171001434326171875f,-0.5556000073492530777485853832331486046314239501953125f,-0.059700006793439452934535438544116914272308349609375f,-0.16570002042055165247802506200969219207763671875f,-0.01340000118911266469012844027020037174224853515625f,0.067700006175041238520861952565610408782958984375f,-0.6717999676585204582579535781405866146087646484375f,1.441799944487214713007006139378063380718231201171875f,-0.424499993801116914937665569595992565155029296875f,0.3277000225663186938618309795856475830078125f,0.097999997407197891874375272891484200954437255859375f,0.150500012233853563969887545681558549404144287109375f,1.2128999807596212434646076872013509273529052734375f,-1.010599964731932143280346281244419515132904052734375f,0.6478000062823294769032145268283784389495849609375f,1.36260000100135858502881092135794460773468017578125f,-1.4549999776482585023273941260413266718387603759765625f,0.7603000143289566636894960538484156131744384765625f,0.993599995517730150140778278000652790069580078125f,-0.49919998491108419358397441101260483264923095703125f,0.5251999969363208009554000454954802989959716796875f,0.05520001019835518718537059612572193145751953125f,0.296000004425644558381236493005417287349700927734375f,0.010000000223517435671283237752504646778106689453125f,-0.7129000184595586375735365436412394046783447265625f,1.037600009420514002300706124515272676944732666015625f,-0.42670001299977311504108001827262341976165771484375f,-0.1216000455260282109293257235549390316009521484375f,0.244600025126338049830110321636311709880828857421875f,-0.096500012040138472002581693232059478759765625f,-0.5474999904632564806661321199499070644378662109375f,0.272999998301267421396687495871447026729583740234375f,-0.2716000073790549862451371154747903347015380859375f,-1.1811000303864478411242089350707828998565673828125f,0.989800027352571110128565123886801302433013916015625f,-0.6412000272154809721314450143836438655853271484375f,0.10099999500066036972611982491798698902130126953125f,0.1559999943673611166872206013067625463008880615234375f,0.0338000002399085897053510052501223981380462646484375f,0.29559999885559040677662778762169182300567626953125f,-0.4479999883919953962418958326452411711215972900390625f,0.1847999953150747387553565204143524169921875f,1.446500031411648734547270578332245349884033203125f,-1.12540000214278723689176331390626728534698486328125f,0.778000028848648383927866234444081783294677734375f,-0.84889998958110890470152298803441226482391357421875f,1.0511999784886840725306456079124473035335540771484375f,-0.4763000094413758489508836646564304828643798828125f,-0.247600020670891041163486079312860965728759765625f,0.31830003393888528506749935331754386425018310546875f,0.264400021708012022969569443375803530216217041015625f,0.1985000092312694253138261046842671930789947509765625f,0.0425000205636028027811335050500929355621337890625f,0.38800001417100438150242780466214753687381744384765625f,0.1499000038973987425361400482870521955192089080810546875f,0.321700017265230620022720131601090542972087860107421875f,0.148399998730420978887423189007677137851715087890625f,0.070299998570978627920879944213083945214748382568359375f,0.024299995225667725406992758507840335369110107421875f,0.90280003161430399938325308539788238704204559326171875f,0.245000010728836148388154470012523233890533447265625f,0.5036000387310988202216321951709687709808349609375f,0.460200009229779549713157393853180110454559326171875f,-0.00089999172687527106972993351519107818603515625f,-0.15219996719360295855949516408145427703857421875f,0.7505999958008524597374844233854673802852630615234375f,0.34049999759346238903390258201397955417633056640625f,1.18640003298073981596871817600913345813751220703125f,-0.42859999067485343715588896884582936763763427734375f,-0.070599998296797306096550528309307992458343505859375f,-0.672099996292590873281369567848742008209228515625f,0.688600003191828680115804672823287546634674072265625f,0.13099999798089267954992465092800557613372802734375f,0.7705000029206274803073029033839702606201171875f,-0.2855999957263468669310668701655231416225433349609375f,-0.08459999815523622235247103162691928446292877197265625f,-0.33380000318288782779063694761134684085845947265625f,0.315000000074505681624259523232467472553253173828125f,-0.0945000004321336906087935858522541821002960205078125f,-0.3275999950408934324741494492627680301666259765625f,-0.5764000253438952281470619709580205380916595458984375f,-0.3386000078856945538774425585870631039142608642578125f,-0.3340000221729280127647143672220408916473388671875f,0.845200022268295381167035884573124349117279052734375f,0.389300005619227895703460262666339986026287078857421875f,1.208500044435262932296382132335565984249114990234375f,0.498999999597668608686262814444489777088165283203125f,0.0607999985858798108750988831161521375179290771484375f,0.5391000055909156429834183654747903347015380859375f,-0.93380001160204406662757037338451482355594635009765625f,-0.2950000024959444910876982248737476766109466552734375f,-0.617900025463104185519114253111183643341064453125f,-0.210599998930096621307939130929298698902130126953125f,0.45820002012252825096538799698464572429656982421875f,0.3551000385522851132691357634030282497406005859375f,0.237799990189075050039946290780790150165557861328125f,0.16980000932216654518924769945442676544189453125f,0.39750001648440969059805638607940636575222015380859375f,-0.3778000068783760045931785498396493494510650634765625f,0.22360000913366684383021265603019855916500091552734375f,0.2171000071704387579529793583787977695465087890625f,-0.040799995940923761139629277749918401241302490234375f,0.111700001525878889196974341757595539093017578125f,1.146200028109551016797240663436241447925567626953125f,-1.197400008833408246999852053704671561717987060546875f,0.63990002552270919267130011576227843761444091796875f,0.9327999836325648885804184828884899616241455078125f,-0.940799989756941901219988722004927694797515869140625f,0.5068000084877013478035223670303821563720703125f,0.34239997579008285555346446926705539226531982421875f,-0.7957999779850244426171457234886474907398223876953125f,0.2328999853417272891675793289323337376117706298828125f,-0.2538999751269823690336124855093657970428466796875f,0.9759999628514055558525797096081078052520751953125f,-0.184499997600913179240933459368534386157989501953125f,0.6218999936938285344467658433131873607635498046875f,-1.340799985733628130191164018469862639904022216796875f,0.3877000118374827053457920555956661701202392578125f,-0.23439998981952658851923843030817806720733642578125f,0.4349999867379665818845069225062616169452667236328125f,-0.14409999769330017471702376496978104114532470703125f,0.8399999855458730735335848294198513031005859375f,-0.930999999657272869768576128990389406681060791015625f,0.455700012895464912077159169712103903293609619140625f,-0.64260005047321389071157682337798178195953369140625f,-0.0005999893188473048866171666304580867290496826171875f,-0.3313000136137009832282274146564304828643798828125f,0.4743000151932239560892412555404007434844970703125f,-0.808200004625320378437436374952085316181182861328125f,0.30040000504851338547496197861619293689727783203125f,0.4794999842643736798208919935859739780426025390625f,-1.106599985548853748440478739212267100811004638671875f,0.30080000637173665012369383475743234157562255859375f,-1.097700008857250164595598107553087174892425537109375f,1.11039999051988136358204428688623011112213134765625f,-0.61430001116991039822323728003539144992828369140625f,-0.006399986451864236869369051419198513031005859375f,0.509700009661913089331619630684144794940948486328125f,-0.33759999891519532067007958175963722169399261474609375f,0.0226000024169683566555022480315528810024261474609375f,-0.265800003540515916711228783242404460906982421875f,0.4941999926596878367490717209875583648681640625f,0.07639999878257508481027571178856305778026580810546875f,-0.01649999237060573165081223123706877231597900390625f,-0.24640000254511829069770101341418921947479248046875f,-0.2087000000581145098266233617323450744152069091796875f,-0.6721000129818914814450181438587605953216552734375f,0.82859997830688936204523997730575501918792724609375f,0.26159999646395437622459212434478104114532470703125f,0.96710001715421611834244686178863048553466796875f,-0.723400001785158952571919144247658550739288330078125f,-0.0748999993711709777244323049671947956085205078125f,0.16799999606609361535447533242404460906982421875f,-0.0727999901890756806466242778697051107883453369140625f,0.0922999988541006732845062288106419146060943603515625f,-0.1091000020146370985685280174948275089263916015625f,-0.30679999485313891316451417878852225840091705322265625f,-0.042699997979402548597960276310914196074008941650390625f,-0.622700002747774039590922257048077881336212158203125f,-0.37679999924898144314511228003539144992828369140625f,-0.326100001515448045807943344698287546634674072265625f,-0.828300026190280647142571979202330112457275390625f,0.71999997928738557195771363694802857935428619384765625f,0.1904999932721256339362980725127272307872772216796875f,0.574300000739096816460005356930196285247802734375f,-0.00580001881420599829652928747236728668212890625f,-0.17360000211149451132541798870079219341278076171875f,-0.67130003000497850962347001768648624420166015625f,0.0089999949783086119481367859407328069210052490234375f,-0.024600002141296872792963768006302416324615478515625f,0.258699999201297714535030536353588104248046875f,0.309400010755658261274447795585729181766510009765625f,0.23030000292956831486890223459340631961822509765625f,-0.044000002503395307940081693232059478759765625f,-0.0055999981105328000552390221855603158473968505859375f,0.0715999980658292844992729442310519516468048095703125f,0.4447999950170515859326769714243710041046142578125f,0.17540000139772882192801262135617434978485107421875f,-0.0763999995276332111160400017979554831981658935546875f,-0.2282999892354009574546580552123486995697021484375f,1.21940000211298471555210198857821524143218994140625f,-0.30970000548958775965502354665659368038177490234375f,-0.423399982786178075144789545447565615177154541015625f,0.96959999599456747620962460132432170212268829345703125f,-0.26060000749826439658818344469182193279266357421875f,0.8622999825000761120463721454143524169921875f,-0.75539997800290592522287624888122081756591796875f,0.4664000006079671578618217608891427516937255859375f,0.132899996256828245577708003111183643341064453125f,-0.00940000516772254712805079179815948009490966796875f,0.0524000076413155824184286757372319698333740234375f,0.673899949347973148405799292959272861480712890625f,-1.19299993728101316747824967023916542530059814453125f,0.41199998009204819027218036353588104248046875f,-1.3815999996304508812272615614347159862518310546875f,1.190999988928437058888221145025454461574554443359375f,-0.743300017547607438928025658242404460906982421875f,-0.09269996594190654803924189764074981212615966796875f,0.6047999711751941287474210184882394969463348388671875f,-0.07489999467730523718955737422220408916473388671875f,-0.08369997785538441092967332224361598491668701171875f,0.80179998095780613542871151366853155195713043212890625f,-0.09019999807253487045244355613249354064464569091796875f,0.063300003242492408617181354202330112457275390625f,-0.14240001837015103802741577965207397937774658203125f,0.01950001823902169917346327565610408782958984375f,0.792499964833259351593142127967439591884613037109375f,-1.06099995353817977417065776535309851169586181640625f,0.45739998975992168794846293167211115360260009765625f,0.547299980115891315790577209554612636566162109375f,-0.3903999623566871690627522184513509273529052734375f,0.27750000253319750953551192651502788066864013671875f,-0.15850001537799851547561047482304275035858154296875f,-0.2517999939590691571567049322766251862049102783203125f,-0.06340000379681587361346828402020037174224853515625f,0.4836000429034239544989759451709687709808349609375f,0.316199999275803111942195755545981228351593017578125f,0.229300008487701489912069519050419330596923828125f,-0.3603999915629625405699698603712022304534912109375f,0.0955999960988760388858054284355603158473968505859375f,-0.1758000012680887014226982500986196100711822509765625f,0.511099986279010209955231403000652790069580078125f,-0.59640000440776308465729016461409628391265869140625f,0.277900006985664305148020503111183643341064453125f,1.005099983417988340761439758352935314178466796875f,1.039600025200844157780011300928890705108642578125f,0.2983999975383280212071213099989108741283416748046875f,0.043899997739493823445400266791693866252899169921875f,0.3005000041127203758151154033839702606201171875f,0.2971999902904034485118245356716215610504150390625f,-0.043399997240304966084067928022705018520355224609375f,-0.446399987792968744315658113919198513031005859375f,0.26119999242126945571129681411548517644405364990234375f,-0.086599999071657662863543691855738870799541473388671875f,-0.254399993544816904744720886810682713985443115234375f,-0.48439999846220016532782892682007513940334320068359375f,-0.1527999998003244375155418310896493494510650634765625f,-0.496200007545947840981170884333550930023193359375f,-0.637600008302926735126447965740226209163665771484375f,-0.2920999973729252463527927830000407993793487548828125f,-0.2330999961733812853026392986066639423370361328125f,0.2302000001400708040222298222943209111690521240234375f,0.08539999808222054067385897724307142198085784912109375f,0.38770000200271592660783426254056394100189208984375f,0.3948000193059446605303719479707069694995880126953125f,0.20040000441521410579071016400121152400970458984375f,0.77540001842230588113835665353690274059772491455078125f,0.175200007367134258373653210583142936229705810546875f,0.18030000088065867291930999272153712809085845947265625f,0.81570000164508815743147351895458996295928955078125f,0.28360001339912432882783832610584795475006103515625f,0.10640000414699320174793939486335148103535175323486328125f,0.31820001442283418757739354987279511988162994384765625f,-0.2384000020086765037063969430164434015750885009765625f,0.01280000002384185098236457633902318775653839111328125f,0.18200000244379044289644298260100185871124267578125f,0.536600008645653492322935562697239220142364501953125f,0.02530000106692309458367162733338773250579833984375f,-0.1653999765872953275902546010911464691162109375f,0.178600003042816979359486140310764312744140625f,0.19999999705702065089241159512312151491641998291015625f,0.79449999976158114378677055356092751026153564453125f,0.829400020590424613686764132580719888210296630859375f,0.3841000062614678967065628967247903347015380859375f,1.2974000552505258188062953195185400545597076416015625f,-0.12740001725256444586875659297220408916473388671875f,-0.19870000411868093959100178835797123610973358154296875f,-0.81860003753900567602386217913590371608734130859375f,-0.73700004304945487820788230237667448818683624267578125f,0.437400010919571258227733778767287731170654296875f,0.29909998800754511449895289842970669269561767578125f,-0.6189999931156633383722009966732002794742584228515625f,0.1838000016927718860415552626363933086395263671875f,0.870799970161915126709573087282478809356689453125f,-0.616799970191717505940687260590493679046630859375f,0.45519999738335581440651367302052676677703857421875f,0.6877999707803130746697206632234156131744384765625f,-0.81539997841268785361279469725559465587139129638671875f,0.37570000211074938700761549625894986093044281005859375f,-0.443599992537498710731824758113361895084381103515625f,0.66379997963905357227787362717208452522754669189453125f,-0.26289999983310696762828229111619293689727783203125f,-0.8449000383377072154189590946771204471588134765625f,0.281000004500150613040432290290482342243194580078125f,-0.443000013351440458109209430404007434844970703125f,0.09629999871253946963634007261134684085845947265625f,-0.3363999971061943927708171031554229557514190673828125f,0.0715000007152557515155422152020037174224853515625f,0.06220002229064736098962384858168661594390869140625f,-0.4032000101387500645699901724583469331264495849609375f,0.0658000093892219251046071804012171924114227294921875f,-0.3506999846577638635380935738794505596160888671875f,-0.025199996638298305384751074598170816898345947265625f,-0.1575000008940694584680386469699442386627197265625f,0.2324000137135389110198957496322691440582275390625f,-0.33060000600814820614914424368180334568023681640625f,0.1383000083461405349538608788861893117427825927734375f,-0.56260000278949728880206748726777732372283935546875f,0.4885999986469744538197801375645212829113006591796875f,-0.30030001060366640075471877935342490673065185546875f,1.0673999945282925949641139595769345760345458984375f,-0.991400012156366639004545504576526582241058349609375f,0.5765000147223471316237919381819665431976318359375f,-0.338099986636638760728601482696831226348876953125f,0.0611999974876642482968236436136066913604736328125f,-0.15530000374913210947624975233338773250579833984375f,0.3846000021785496159054673626087605953216552734375f,-0.839400002858042615372369255055673420429229736328125f,0.258499999061226670704627395025454461574554443359375f,0.44639999107122463328778394497931003570556640625f,-0.15639997624456913882795561221428215503692626953125f,0.21340000588297858286068731104023754596710205078125f,-0.569000064194202348488715870189480483531951904296875f,0.93840002152919765876504243351519107818603515625f,0.0023999995946883956321471487171947956085205078125f,-0.023100001798570135580490614302107132971286773681640625f,-0.447300024372339510847496057976968586444854736328125f,-0.4064000125288960152403205938753671944141387939453125f,-0.3102999981984495558862136022071354091167449951171875f,-0.6365000135302540940074322861619293689727783203125f,-0.076800001633167358505716038052923977375030517578125f,-0.13650000313669441442954166632262058556079864501953125f,-0.58950002717971816679209950962103903293609619140625f,0.158400001114606947538732129032723605632781982421875f,-0.0584999975338578170891423724242486059665679931640625f,0.3321000237107281094495192519389092922210693359375f,-0.8910000243932010288716583090717904269695281982421875f,-0.3108000064648688276491839133086614310741424560546875f,-0.543500029925257255314363646903075277805328369140625f,0.785999998092651086523119374760426580905914306640625f,0.27869999722689386967289237873046658933162689208984375f,0.6039000120073552135835370791028253734111785888671875f,0.985399997970461516416662561823613941669464111328125f,0.394599997521936807221010212742839939892292022705078125f,1.196900031095742722442309968755580484867095947265625f,-0.8844000217080119075063748823595233261585235595703125f,-0.221100007475912629928416208713315427303314208984375f,-0.75710003741979647884363657794892787933349609375f,0.230200026813149438709160676808096468448638916015625f,0.27380000336915255587655337876640260219573974609375f,0.8397000259041789149705437012016773223876953125f,0.8758000168025492637724482847261242568492889404296875f,0.156800001633167263026535920289461500942707061767578125f,0.49670002618730091370480295154266059398651123046875f,-0.56920001195669167604052063325070776045322418212890625f,-0.2147000022485852088749425092828460037708282470703125f,-0.30750001966953277587890625f,-0.2106000004202126518748627859167754650115966796875f,-0.0327000014811753836330154854294960387051105499267578125f,0.2985999869495630232307803453295491635799407958984375f,0.083800017115474023654542179428972303867340087890625f,0.1529000075817108506015529201249592006206512451171875f,-0.0845999810934063845024866168387234210968017578125f,-0.5810000044256449225343885700567625463008880615234375f,-0.16240000216513872377532834434532560408115386962890625f,-0.45810001852512360898117549368180334568023681640625f,0.13859999425113134208231713273562490940093994140625f,-0.2127000112414361154833386535756289958953857421875f,0.335100022906065220240634516812860965728759765625f,0.1179999993443487937838654033839702606201171875f,0.1539000113368036437577757169492542743682861328125f,-0.13410003211498189301664751837961375713348388671875f,-0.1051999796509746065709123286069370806217193603515625f,-0.0773000027656554067334582214243710041046142578125f,0.345300013583898390834292513318359851837158203125f,-0.117600006812810686795955916750244796276092529296875f,0.165900015196204453360451225307770073413848876953125f,0.0710999678015706848555055330507457256317138671875f,-0.760199994701146852804640730028040707111358642578125f,0.07559999781250947847865973017178475856781005859375f,-1.0675000308640303359197787358425557613372802734375f,0.9126000062912702492923244790290482342243194580078125f,-0.58900001939758672708791209515766240656375885009765625f,0.8504999976307150433285642066039144992828369140625f,-0.847399981117248568551758580724708735942840576171875f,0.478200000974535743836213441682048141956329345703125f,0.7310999831497664303014971665106713771820068359375f,-1.088399974814057369343345271772705018520355224609375f,0.44149999192357025634692035964690148830413818359375f,-0.94210000553131134637396826292388141155242919921875f,1.3833999973893165158500551115139387547969818115234375f,-0.5506000167369844433551406837068498134613037109375f,-0.3106999584317211571260486380197107791900634765625f,0.237799961355328992596014359151013195514678955078125f,-0.1479999930858610923678497783839702606201171875f,1.14170000450611031084235946764238178730010986328125f,-1.508000009700655397182345041073858737945556640625f,0.6507000216871501319104709182283841073513031005859375f,-0.703300016355514312493824036209844052791595458984375f,0.49500000476837147100894753748434595763683319091796875f,-0.38240000883340830029055723571218550205230712890625f,-0.576399996361136146560966153629124164581298828125f,0.1771999987095593809272031649015843868255615234375f,-0.291400001278519393821397898136638104915618896484375f,0.1432000323414808207189707900397479534149169921875f,0.504799986895918539886451981146819889545440673828125f,0.0488000037789344531802271376363933086395263671875f,-0.63949999439716354032725575962103903293609619140625f,0.8167999857634307137033147228066809475421905517578125f,-0.367300003659725149418591172434389591217041015625f,0.286400029814242174097671522758901119232177734375f,1.27650003379583409923725412227213382720947265625f,0.1869999984949826110636195153347216546535491943359375f,0.12229999609738588517426904900275985710322856903076171875f,0.54639998812824452922853879499598406255245208740234375f,0.364200013521313703535042805015109479427337646484375f,0.224400002522766595092207353445701301097869873046875f,0.921300026428699680991485365666449069976806640625f,-0.155600014650822071615721142734400928020477294921875f,-0.1072000101834535978895246444153599441051483154296875f,0.3281999724388118266915626008994877338409423828125f,0.6400000058114532208719538175500929355621337890625f,0.073100006119906968393706847564317286014556884765625f,-0.093099989616870715281038428656756877899169921875f,-0.375200012135505911459176786593161523342132568359375f,-0.04630000597685579588613791202078573405742645263671875f,-0.01610002055168191503753405413590371608734130859375f,0.30180000915825377916235083830542862415313720703125f,0.16380000314563514240973063351702876389026641845703125f,0.50510001828670514356645071529783308506011962890625f,-0.095399995288252881220358858627150766551494598388671875f,0.02369999987185000822620395410922355949878692626953125f,0.023000000387430219461748492904007434844970703125f,-1.03100001426041121277421552804298698902130126953125f,-0.3085000036060809858184939002967439591884613037109375f,-1.2410000511407854872913958388380706310272216796875f,0.206200004938244862984220162616111338138580322265625f,-0.02409999597966672180149316773167811334133148193359375f,-0.28359997853040663784440766903571784496307373046875f,0.283199995982646834935536617194884456694126129150390625f,0.06989999843984839866806169084156863391399383544921875f,0.2477000042378900701578459120355546474456787109375f,-0.0238000003516673774850431755112367682158946990966796875f,-0.067399999818205824642092238718760199844837188720703125f,-0.04489999922215925920454537845216691493988037109375f,-0.25219999319612984578498071641661226749420166015625f,-0.07239999799281360282066088984720408916473388671875f,-0.70449999886751157873732154257595539093017578125f,0.4720000176429748339756997665972448885440826416015625f,0.247400002947449682011438198969699442386627197265625f,0.6758000258922578495912603102624416351318359375f,-0.34580000003874289404848241247236728668212890625f,-0.2710000002905725846602535966667346656322479248046875f,-0.9815000308156012209792606881819665431976318359375f,0.544200014266371656646015253500081598758697509765625f,0.512000028967857900852322927676141262054443359375f,-0.11099999105185265335649091866798698902130126953125f,-0.1635999941021204495683605273370631039142608642578125f,-0.0367999992862342306665368596441112458705902099609375f,-0.1076999921679497873583386535756289958953857421875f,-0.306400008282065261511206699651665985584259033203125f,-0.0207000007510185213277509319595992565155029296875f,-0.4664000181913383613618862000294029712677001953125f,-0.273799997818469531551954787573777139186859130859375f,-0.216100002670287949513294734060764312744140625f,1.17949999916553593948265188373625278472900390625f,-1.0521999738991265616050441167317330837249755859375f,0.637200014281273041660824674181640148162841796875f,-0.6393000028252604494127808720804750919342041015625f,0.637199999976158171222095916164107620716094970703125f,-0.3497000110030175079600667231716215610504150390625f,0.105700007009506435906587284989655017852783203125f,-0.21999999962747107673521895776502788066864013671875f,0.073900000309944147147689363919198513031005859375f,-0.1975999875903131641763366133091039955615997314453125f,0.298799986419081842203837595661752857267856597900390625f,-0.111799999099969848970204111537896096706390380859375f,-0.818999983131885045395392808131873607635498046875f,1.57499999307095972511660875170491635799407958984375f,-0.507500004172324992168796597979962825775146484375f,0.5827999901890752454391986248083412647247314453125f,-0.415400002142786828329690251848660409450531005859375f,0.30510000442862494907103609875775873661041259765625f,0.2982999892354010196271474342211149632930755615234375f,-0.449399987882375728798223235571640543639659881591796875f,0.174999998509883791797392404987476766109466552734375f,0.01929999179840080625325526852975599467754364013671875f,-0.2108000003367661301378888083490892313420772552490234375f,0.0177000019282103071560641183168627321720123291015625f,0.085700019228458046427476801909506320953369140625f,0.59639997579753423195825234870426356792449951171875f,0.00210000553131095557546359486877918243408203125f,0.25100002074241611893512526876293122768402099609375f,-0.3248000152826306941022949104080908000469207763671875f,0.1554000063896179284483878291212022304534912109375f,0.20430001355409643792881979607045650482177734375f,0.098399989864230263947320054285228252410888671875f,0.0769000175356868709286572993732988834381103515625f,0.5248000048518175475464886403642594814300537109375f,-0.0168999787926669142734681372530758380889892578125f,0.61000001817941684745250086052692495286464691162109375f,0.35570000533312562396304201683960855007171630859375f,0.7437000227451324718686009873636066913604736328125f,0.47079999370574920902754456619732081890106201171875f,0.189499997712671675031970153213478624820709228515625f,0.004300007593631249847021535970270633697509765625f,0.354400035864114837380611788830719888210296630859375f,0.200900005324184949273558231652714312076568603515625f,0.024300009977817893513929448090493679046630859375f,-0.90920000942349421535482179024256765842437744140625f,-0.316299999941885456866685899512958712875843048095703125f,-0.846700018364190842845573570230044424533843994140625f,-0.39920003311634122411533098784275352954864501953125f,-0.337300011147558809199153984081931412220001220703125f,-0.872300028097630075762936030514538288116455078125f,1.130800031405687633423440274782478809356689453125f,0.3602000091552735039357457935693673789501190185546875f,1.1260000487565999804928651428781449794769287109375f,-0.8290000292509798196505244050058536231517791748046875f,-0.21620001108497388653262305524549447000026702880859375f,-0.39110004260540076614915960817597806453704833984375f,-0.46560003551244744102177719469182193279266357421875f,-0.2679000054582953982418302985024638473987579345703125f,-0.1005000100731852086255457834340631961822509765625f,-0.2388000043749809453430543726426549255847930908203125f,-0.080100000964105133594017615905613638460636138916015625f,-0.344900010472536155958778181229718029499053955078125f,-0.124600001731514975489290009136311709880828857421875f,0.019299999770522102693348642787896096706390380859375f,-0.514800011706352478313419851474463939666748046875f,0.600200003418326133441951242275536060333251953125f,0.042999997667968248293846045271493494510650634765625f,0.009100002610683333159613539464771747589111328125f,0.67500001288950439715108586824499070644378662109375f,0.17820000317245721088710297408397309482097625732421875f,0.08730002005100256923242341144941747188568115234375f,0.143999987900257142570126234204508364200592041015625f,-0.0234000040203332648758305367664434015750885009765625f,0.5369999854564664332201573415659368038177490234375f,-0.395400012424588442883077732403762638568878173828125f,-0.02820000458806759535690389384399168193340301513671875f,0.19969999699592566599903875612653791904449462890625f,-0.6592000065922736240509038907475769519805908203125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_3d_2item_large_5o3i_uneven_filter_uneven_data_dilation_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,3,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3,4};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,5,14,20,12};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1},        // move_strides
                                                                  Strides{1,1,1},        // filter_dilation
                                                                  CoordinateDiff{0,0,0}, // below_pads
                                                                  CoordinateDiff{0,0,0}, // above_pads
                                                                  Strides{2,3,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0229999961778524220079589213128201663494110107421875f,1.0035999919787048639818749506957828998565673828125f,0.21589999534040682505064978613518178462982177734375f,0.266299998973309914873652815003879368305206298828125f,0.9888999852225186515397581388242542743682861328125f,-0.392199983845651356517691965564154088497161865234375f,-0.137999977625906711242720348309376277029514312744140625f,0.24879996235370693824506815872155129909515380859375f,-0.504899973070621843618255297769792377948760986328125f,0.07589996721446479188699640872073359787464141845703125f,-0.3658999761179086984697050866088829934597015380859375f,-1.0561000048682089857976507119019515812397003173828125f,0.83070000753104744628529942929162643849849700927734375f,0.142000030905008844950287993924575857818126678466796875f,0.36189997365176651555884745903313159942626953125f,-0.37670000778436640853641392823192290961742401123046875f,1.1936000144049525584222237739595584571361541748046875f,0.59930002350062150373588565344107337296009063720703125f,-0.2292999910160897769628718378953635692596435546875f,0.33000000413507224461540090487687848508358001708984375f,-1.6798000128984451606584116234444081783294677734375f,-0.02209999808073070948211125141824595630168914794921875f,-0.50500003308057817008602796704508364200592041015625f,-0.3495000182762744156406142792548052966594696044921875f,0.0226000394463550247081684574368409812450408935546875f,1.703400061315298419373220895067788660526275634765625f,2.00550005015730814506014212383888661861419677734375f,0.33880001324117181571438095488701947033405303955078125f,0.0578000424176454696834071000921539962291717529296875f,1.2010000144094223895052664374816231429576873779296875f,-0.0290000228062272658036135908332653343677520751953125f,0.317000018090009660909345257095992565155029296875f,0.857800015074014599036900108330883085727691650390625f,0.51900000101327936619100000825710594654083251953125f,0.09569991607516925569143495522439479827880859375f,-1.66130003708302975695687564439140260219573974609375f,0.462900001397728910745854591368697583675384521484375f,-0.69149998588860039916426103445701301097869873046875f,-1.037500040456652339315724020707421004772186279296875f,-0.294299996380507966620143633917905390262603759765625f,0.3837999825075268933716188257676549255847930908203125f,0.3868999865040176899810830946080386638641357421875f,0.2226000283077358421479630123940296471118927001953125f,-0.93989999676346780432822924922220408916473388671875f,-1.386500002354383642710899948724545538425445556640625f,1.11330001575946813119344369624741375446319580078125f,-0.92339999008774764632789811003021895885467529296875f,-0.985999996975063908166703185997903347015380859375f,0.180699969607591270914781489409506320953369140625f,-0.48400002813339237928147440470638684928417205810546875f,-0.3611999785631894166471056450973264873027801513671875f,0.025200023981929164929027820107876323163509368896484375f,0.20600001705437909205187452243990264832973480224609375f,0.0778999999448657387546290919999592006206512451171875f,0.05570000209212278008408247842453420162200927734375f,-0.29950000981986579784432933593052439391613006591796875f,-0.6300999938488001106406954932026565074920654296875f,-0.048800003853440188095191842876374721527099609375f,1.563300041687488484143386813229881227016448974609375f,0.0455999868974086286499414200079627335071563720703125f,1.0810999818459148258398272446356713771820068359375f,-0.900400011716782966431082968483678996562957763671875f,-0.27699996776133739562197888517403043806552886962890625f,0.644000035956501459821765820379368960857391357421875f,1.55730002459585659124741141567938029766082763671875f,0.807800012205541317911183796240948140621185302734375f,0.77500000264495572110945431631989777088165283203125f,0.1906000067532060970876273131580092012882232666015625f,0.911499994717539951949447640799917280673980712890625f,0.23280001343488709864004704286344349384307861328125f,0.90390006192624572367577684417483396828174591064453125f,1.047799975138902883742275662370957434177398681640625f,-0.3843000303924082317053034785203635692596435546875f,-0.79639998581856463744088614475913345813751220703125f,-0.342600012661516573064801605141838081181049346923828125f,-0.79449997178465192604335243231616914272308349609375f,-0.8154999719262121526952569183777086436748504638671875f,-0.210899990646541135586033988147391937673091888427734375f,-0.5963999935671691066119137758505530655384063720703125f,-0.012400018370150700519616293604485690593719482421875f,-1.5724999901652341183222461040713824331760406494140625f,-0.6512999888777730728151027506100945174694061279296875f,0.73289998746514317673472760361619293689727783203125f,0.512399963682890646765599740319885313510894775390625f,-0.7121999990075826847402140629128552973270416259765625f,-0.1339000082820656079007903827005065977573394775390625f,-0.378700018256157566742814424287644214928150177001953125f,0.086899989968538260409758322566631250083446502685546875f,-0.438400001114607196228689645067788660526275634765625f,0.345200018766522287450015937793068587779998779296875f,-0.803800023895501869475310741108842194080352783203125f,-0.4407000107720493442542419870733283460140228271484375f,1.5969999970048662785160331623046658933162689208984375f,-1.0051000229060649271417560157715342938899993896484375f,1.958300049436092482579852003254927694797515869140625f,-0.5779000421524056463340457412414252758026123046875f,0.139099954386800117589473302359692752361297607421875f,-0.37250000631436697151599446442560292780399322509765625f,-0.05599999220669238386705046650604344904422760009765625f,-0.6000000184774396672082730219699442386627197265625f,-1.352000060446561402471843393868766725063323974609375f,0.01599997849762413437701980001293122768402099609375f,0.66529998393058809025291111538535915315151214599609375f,0.290399994841218589414211237453855574131011962890625f,0.977100056158005880746486582211218774318695068359375f,-0.2607999932587148350648931227624416351318359375f,-0.4969999913051719175882681156508624553680419921875f,-0.585700064229965011719514222932048141956329345703125f,-0.0936999635688959831014699375373311340808868408203125f,-0.297400008963793205385428564113681204617023468017578125f,-0.2569000195287169940883131857845000922679901123046875f,0.9337000130593773317144723478122614324092864990234375f,0.50680002040863048051733130705542862415313720703125f,-0.36300000478327287822821745066903531551361083984375f,-0.48499998472630967416563407823559828102588653564453125f,-0.8057999805182214370091742239310406148433685302734375f,0.25289995431005962700510281138122081756591796875f,0.33660001706480979777325046597979962825775146484375f,0.6031000082433219944277880131267011165618896484375f,-0.6691999787271019073386923992075026035308837890625f,0.3033999828606843696121586617664434015750885009765625f,-0.515000012144446461803681813762523233890533447265625f,0.38289998102039068950119826695299707353115081787109375f,-0.2758000029817224696415678408811800181865692138671875f,0.837700025397539516092138001113198697566986083984375f,0.25030000348836178858391576795838773250579833984375f,0.9021000149190427919165813364088535308837890625f,-0.0283999931424856466577466562739573419094085693359375f,0.8716999714255333486789822927676141262054443359375f,-0.6288000137627121066685731420875526964664459228515625f,0.431599973851441998107247854932211339473724365234375f,-0.50230000921040751649115918553434312343597412109375f,0.159000005595385740519986939034424722194671630859375f,-0.6183000027954574928656938936910592019557952880859375f,-0.41380004051029739553513309147092513740062713623046875f,-1.151400018936396207891448284499347209930419921875f,-1.151800014895199897324573612422682344913482666015625f,0.217199995654821176316318087629042565822601318359375f,0.16630005105286882471915532732964493334293365478515625f,1.060700005519390032304727355949580669403076171875f,-0.28640000321567082863793984870426356792449951171875f,-0.72429998047351773493574000895023345947265625f,-0.1378999280095103419085944551625289022922515869140625f,-0.445899982599914090908299613147391937673091888427734375f,0.240900013855099626169931070762686431407928466796875f,0.6128000193208456014559715185896493494510650634765625f,-0.1044000024110081081829548566020093858242034912109375f,0.54439998773336384374488261528313159942626953125f,1.2620000316500661341478917165659368038177490234375f,-0.616100021147727883175093666068278253078460693359375f,1.122599981480836728309213867760263383388519287109375f,0.385199987101554786050883194548077881336212158203125f,0.5663999630570415444452692099730484187602996826171875f,-0.730499991595744635475284667336381971836090087890625f,0.0164999272525314655268857677583582699298858642578125f,0.4241000079192220884038988515385426580905914306640625f,0.7833000182926657117832291987724602222442626953125f,0.030999994255602292270168618415482342243194580078125f,-0.10729999069571505287967738695442676544189453125f,-0.5138000019162893838853278793976642191410064697265625f,-0.75460002266764636402740507037378847599029541015625f,-0.1723999884933233861517010154784657061100006103515625f,0.590999950259923689799279600265435874462127685546875f,-0.5629000064268705472159126657061278820037841796875f,1.6194000145554543479420317453332245349884033203125f,-0.10480001304745678591956448144628666341304779052734375f,1.0515000245571137771349867762182839214801788330078125f,-0.60820001550316804550533333895145915448665618896484375f,-1.8903999827712769654652902318048290908336639404296875f,-0.64829998983144765389852182124741375446319580078125f,0.179700022178887863777418942845542915165424346923828125f,-0.5585999830007555289057563641108572483062744140625f,0.32009997529685474404459455399774014949798583984375f,-0.16289999879002525773330489755608141422271728515625f,0.17219997315406754267996802809648215770721435546875f,-1.327200003105402092984377304674126207828521728515625f,-1.58719997684210500921153652598150074481964111328125f,-0.538300010395050154698992628254927694797515869140625f,-0.370899944117665025800079092732630670070648193359375f,-0.8659999726116656137975269302842207252979278564453125f,1.62319999449253060674891457892954349517822265625f,-0.819899982011318417818301895749755203723907470703125f,0.2206999927043913345414694049395620822906494140625f,-0.1673999720275405334035667692660354077816009521484375f,-1.4221999829888343214179258211515843868255615234375f,0.7827000002145769741446201805956661701202392578125f,0.01149998044967670551841365522705018520355224609375f,0.6609999861717223978985202847979962825775146484375f,1.8556999865949155292810246464796364307403564453125f,-0.72080000652074804889934966922737658023834228515625f,1.33619997461438178021353451185859739780426025390625f,0.3520000069141389076321502216160297393798828125f,0.18549999725818633322660389239899814128875732421875f,0.17049999759346246008817615802399814128875732421875f,0.0817000162780285865693485902738757431507110595703125f,-0.31029999201446745171750762892770580947399139404296875f,-1.5965000026524069909328318317420780658721923828125f,0.2562000086262816722637580824084579944610595703125f,0.4532000050723556938692127005197107791900634765625f,0.2254000054955482834628810451249592006206512451171875f,0.57070000611543658664004396996460855007171630859375f,-0.28669999631047227151015022172941826283931732177734375f,0.1219000120222570870964773348532617092132568359375f,0.0681999715819952001538695185445249080657958984375f,0.231300007168948607017000540508888661861419677734375f,0.382599984908103873948448381270281970500946044921875f,0.8289000383451583520155736550805158913135528564453125f,0.1784000546097754824614867175114341080188751220703125f,0.7260999377384773634247494555893354117870330810546875f,-0.05260002856850665153842783183790743350982666015625f,0.645699985328317627164551595342345535755157470703125f,0.67319999970793720223127820645458996295928955078125f,0.046000012323260397462121318312711082398891448974609375f,-0.0459999577105048018665911513380706310272216796875f,-0.5889999865218988883697193159605376422405242919921875f,-0.9602000101983545476969084120355546474456787109375f,-0.149400008744001500105014201835729181766510009765625f,-0.87649999862909311332259676419198513031005859375f,-0.20579999281167982161377949523739516735076904296875f,0.31290004960298478664526555803604423999786376953125f,-2.40919995533227915274210317875258624553680419921875f,0.005699980932474257855346877477131783962249755859375f,0.2903000347435469308265965082682669162750244140625f,0.1050999856531620313404573607840575277805328369140625f,0.550699994045495966332737225457094609737396240234375f,0.5109999779760838745090723023167811334133148193359375f,0.04969999103546118845997625612653791904449462890625f,0.2196999819457534641742313397116959095001220703125f,0.17860000937581066438042398658581078052520751953125f,0.35870000821650027678089145410922355949878692626953125f,0.251800012511015136595915464567951858043670654296875f,0.53100000487267973170446566655300557613372802734375f,-0.7339999984800815724383937777020037174224853515625f,-0.135499986268579675208201251734863035380840301513671875f,-1.10110000877976421662651773658581078052520751953125f,-0.17879999256581069477789469601702876389026641845703125f,0.70009997950643310105078853666782379150390625f,-0.1752999955534933762546501156975864432752132415771484375f,-0.167100003109872385920198212261311709880828857421875f,0.1860999992802738756125791041995398700237274169921875f,0.195799980220198843738899086019955575466156005859375f,1.848899949944020182357462545041926205158233642578125f,0.74859999305903890398639077830011956393718719482421875f,1.22569998540282210086616032640449702739715576171875f,0.2518000166460871991347403309191577136516571044921875f,0.8433000107675792111905366255086846649646759033203125f,0.0429999966248868492613155467552132904529571533203125f,0.758099988275766190071180972154252231121063232421875f,0.93800000277161588169150263638584874570369720458984375f,-0.2861000236809250196756693185307085514068603515625f,0.5429999791905284478588100682827644050121307373046875f,-0.571200029450654955809341117856092751026153564453125f,1.1528999964058395111266008825623430311679840087890625f,1.73650003416836273828494086046703159809112548828125f,0.3653999721914527043509224313311278820037841796875f,-0.80460001596212382679595975787378847599029541015625f,-0.6313000222563740493342265835963189601898193359375f,0.4567999947786329784094050410203635692596435546875f,1.307999972522258413931695031351409852504730224609375f,1.5159000319600106632123015515389852225780487060546875f,0.0835999776363369750953324910369701683521270751953125f,-0.253899985632300317917042775661684572696685791015625f,-1.72840001758039019108537104330025613307952880859375f,-0.27549999859929064083274852237082086503505706787109375f,-0.0003000542640683168116311207995750010013580322265625f,0.03540002553761023573741795189562253654003143310546875f,-0.50759999392330623635416486649774014949798583984375f,-0.69659999776780612279480919823981821537017822265625f,-0.968500003755092819801575387828052043914794921875f,-1.1240000102892515609909196427906863391399383544921875f,0.830800015461444996134332541259936988353729248046875f,0.33190002029240162073620012961328029632568359375f,-0.0478000070273874033688343843095935881137847900390625f,0.3221999936059118940789858243078924715518951416015625f,-0.5730000143200157936007599346339702606201171875f,0.0357000027626754867782210567384026944637298583984375f,0.171599982345103985181822281447239220142364501953125f,0.213899967974423788064086693339049816131591796875f,0.21769999533444650552382881869561970233917236328125f,0.055800023061037595795141896815039217472076416015625f,-0.16010001765340564627848607415216974914073944091796875f,1.507499983757734529632443809532560408115386962890625f,0.70629999096393569768537190611823461949825286865234375f,0.15459997282326209955272133811376988887786865234375f,-0.320200010719895455935102290823124349117279052734375f,-0.4521999807536598581236830796115100383758544921875f,-0.8334000070378184688024703064002096652984619140625f,-0.8810000056177378358057694640592671930789947509765625f,-1.1705000337660322884403285570442676544189453125f,-0.25140000507831583576034972793422639369964599609375f,0.261100020104645835772316786460578441619873046875f,-0.0124999870359896458893445014837197959423065185546875f,-1.1579000110089783248668027226813137531280517578125f,-1.0595999974846836977349084918387234210968017578125f,1.14849998065829250748492995626293122768402099609375f,1.39789998984932850589757435955107212066650390625f,-0.1915000209808344067141661071218550205230712890625f,0.4704999959170823586873666499741375446319580078125f,0.397699929583072986361003131605684757232666015625f,0.19859997495412784473955980502068996429443359375f,0.018900035923719915587071227491833269596099853515625f,-0.51320001825988292853963912421022541821002960205078125f,0.44450004416704214094124836265109479427337646484375f,-0.1114000076115131410148251234204508364200592041015625f,-0.080899977570771941515204161987639963626861572265625f,-1.8127999870225786072097662326996214687824249267578125f,0.07050003972649587780097135691903531551361083984375f,-0.0320999852657319895143928079050965607166290283203125f,-0.438700001734495526051205160911194980144500732421875f,-1.19839995570331847574152561719529330730438232421875f,-0.4246999804556370605723714106716215610504150390625f,-0.73210001912862032735773709646309725940227508544921875f,-0.486099989035725599251236417330801486968994140625f,0.1841000531256196115492684839409776031970977783203125f,-0.440099987031519479518948401164379902184009552001953125f,0.427700042384862921807098246063105762004852294921875f,-0.5659999907910824301637831013067625463008880615234375f,0.3313000399887553459876698980224318802356719970703125f,0.294399993507564239347829015969182364642620086669921875f,0.7778999560236936616774983122013509273529052734375f,-0.2407000122994185087321739047183655202388763427734375f,0.09610000982284550019585367408581078052520751953125f,-0.0671000077664849481351438953424803912639617919921875f,-0.818399964755773368096924968995153903961181640625f,0.102299975831806388981704003526829183101654052734375f,0.2296999685347078656860730916378088295459747314453125f,1.3358000111401082676820806227624416351318359375f,1.665199972796440164302111952565610408782958984375f,0.47570000566840153766179355443455278873443603515625f,-0.6023999814152716236748119627009145915508270263671875f,-1.267399938350916244189647841267287731170654296875f,0.45179996572136882360126719504478387534618377685546875f,0.74369998437464257534657008363865315914154052734375f,0.21159996312260620499046126496978104114532470703125f,0.819000036254524221845940701314248144626617431640625f,1.5750000365078449249267578125f,-0.11869998389035441022087979945354163646697998046875f,0.12099999563395957835609806352294981479644775390625f,0.52680000535845739051410419051535427570343017578125f,0.6652000136256213114194224544917233288288116455078125f,-0.8810999849379061998888573725707828998565673828125f,-0.128400001747905889715184457600116729736328125f,-2.26479995716810211803249330841936171054840087890625f,0.2856999957218764318866988105583004653453826904296875f,-0.06950002203881755491465810337103903293609619140625f,-0.0763000367105008958645839811651967465877532958984375f,-0.875799978730082084865671276929788291454315185546875f,0.9272000017270449756523476025904528796672821044921875f,0.771300004225969448867772371158935129642486572265625f,0.216499973297119563397927777259610593318939208984375f,0.7074999937042594222447178253787569701671600341796875f,-1.0428000055372710352941112432745285332202911376953125f,-0.169699994425475508830203352772514335811138153076171875f,-1.472799945932627974087836264516226947307586669921875f,-0.1940999881565570461816605529747903347015380859375f,-0.145000033751129020487269372097216546535491943359375f,0.61729998413920394995102469692938029766082763671875f,-0.73689996389150580480276175876497291028499603271484375f,0.50370000970661632511138350309920497238636016845703125f,-0.52969998105168425439615020877681672573089599609375f,-0.9156000231444831882754442631267011165618896484375f,-1.111000034302473071790018366300500929355621337890625f,0.4239999774694442891131984652020037174224853515625f,0.7818000312864779033361628535203635692596435546875f,0.193600043201445970453278278000652790069580078125f,0.40959999212026598769398333388380706310272216796875f,-0.8488000329852098957417183555662631988525390625f,-1.268700023192167236629757098853588104248046875f,-0.681099943363665971673981403000652790069580078125f,0.09789996541142453878592277760617434978485107421875f,1.4175000321865081076566639239899814128875732421875f,0.209600023561716142239674809388816356658935546875f,0.30959996663928013305167041835375130176544189453125f,-0.3570000322088600075431941149872727692127227783203125f,0.45199994984269142861421642010100185871124267578125f,0.9433000033915046600441201007924973964691162109375f,-0.1838000019907948257014140835963189601898193359375f,0.033400008304416584081764085567556321620941162109375f,-0.4956000170350076672320938087068498134613037109375f,0.793100001798569653743697926984168589115142822265625f,-0.372499977350235322859361986047588288784027099609375f,-0.2640999988108874152459293327410705387592315673828125f,0.0997000297784804700995664461515843868255615234375f,-0.0386000133991240002018230370595119893550872802734375f,-0.447900014734268214766643723123706877231597900390625f,-0.2423000103652466208359328447841107845306396484375f,0.474099990801513104798914355342276394367218017578125f,-0.46639999688267774757832739851437509059906005859375f,-0.5392999984666706492220100699341855943202972412109375f,-0.51250002264976490806702713598497211933135986328125f,-0.0872999842882153931356015164055861532688140869140625f,-1.0710999597549442086119597661308944225311279296875f,-0.0883999776452777386026582462363876402378082275390625f,-0.235799987745284767015618854202330112457275390625f,0.333499998241662876097279877285473048686981201171875f,0.31780002267360618617431100574322044849395751953125f,0.50370005515515714478169684298336505889892578125f,-0.122899994915723453914324636571109294891357421875f,-0.860800017175078036046897977939806878566741943359375f,-0.5687000070989132183285619248636066913604736328125f,-0.779900008833408708852630297769792377948760986328125f,-0.2140000085383653161130723674432374536991119384765625f,0.485900006851554433495721241342835128307342529296875f,-0.76860000814646500799653949798084795475006103515625f,0.40250002473592783047706689103506505489349365234375f,0.172100038537382982184453794616274535655975341796875f,-0.4971999972939489254031286691315472126007080078125f,-1.582000008106231536686436811578460037708282470703125f,0.42409997780025054936459127929992973804473876953125f,0.2778000357121228613976882115821354091167449951171875f,-1.2602000199586154138842175598256289958953857421875f,-0.8906000232934954130570304187131114304065704345703125f,-0.038499948024749652830678314785473048686981201171875f,0.70220000575035779188937112849089317023754119873046875f,1.79050001484155618669547038734890520572662353515625f,0.20809999684393398755588577841990627348423004150390625f,1.029700005042552657386067949119023978710174560546875f,0.6433999976500874851126354769803583621978759765625f,1.238199970799684468403256687452085316181182861328125f,-0.8981000022828578011058198171667754650115966796875f,-0.0083999814450741450144732880289666354656219482421875f,-1.089499969363212361628256985568441450595855712890625f,-1.0645000166743994185480914893560111522674560546875f,0.7903000383198257594585811602883040904998779296875f,0.241000019103288831701092931325547397136688232421875f,0.83539997923225162157478962399181909859180450439453125f,0.5162000113829972036683102487586438655853271484375f,0.2873000052988528008057755869231186807155609130859375f,-0.04719999539405106414591273278347216546535491943359375f,0.7373999544441698361652015591971576213836669921875f,-0.0126000028640034233973210575641132891178131103515625f,-0.236699998098611530394919100217521190643310546875f,0.02020002495050388091613058350048959255218505859375f,-0.7967999983921647544349298186716623604297637939453125f,-0.2116999757841224738541541228187270462512969970703125f,0.3888999885939063982931429563905112445354461669921875f,0.730199965122342309342684529838152229785919189453125f,0.2742999879986047773172685992904007434844970703125f,-0.57879998925030251566425931741832755506038665771484375f,0.496499985404312449421837527552270330488681793212890625f,-0.409900002686679254537693850579671561717987060546875f,-0.943899999937415135065066351671703159809112548828125f,-0.35009999571144601038241717105847783386707305908203125f,0.33670001020580564432549408593331463634967803955078125f,0.5561999607563021807976610944024287164211273193359375f,0.4930000130906699862265440970077179372310638427734375f,0.2870999967396257179785834523499943315982818603515625f,-0.73619997591823349569750689624925144016742706298828125f,0.24020001936256873964481428629369474947452545166015625f,0.02960000370591853169344176421873271465301513671875f,-1.26219997069537637202074620290659368038177490234375f,-0.20369999327808641087500518551678396761417388916015625f,-0.595299980242550486053687563980929553508758544921875f,-0.8660000223815440545394039872917346656322479248046875f,-0.07629997986257064379600478787324391305446624755859375f,0.45000000935047868022564898637938313186168670654296875f,0.26530001604259023650911331060342490673065185546875f,-0.213399982544034971798652122743078507483005523681640625f,-0.44509999690353863055491956401965580880641937255859375f,0.2515999928876759828000331253861077129840850830078125f,0.13349999006465085304995454862364567816257476806640625f,-0.7419999719336629606658561897347681224346160888671875f,-0.43359999909400925854896513556013815104961395263671875f,-0.055599988722800919305200295639224350452423095703125f,-0.440499998524785052467223067651502788066864013671875f,0.538500033520161736078080139122903347015380859375f,0.42360001570880412291586480932892300188541412353515625f,1.2028000031344594500382072510547004640102386474609375f,0.84660001170039134166245275991968810558319091796875f,0.91409998964667327214783654198981821537017822265625f,-0.072600001895427368481250596232712268829345703125f,-0.0679000151813031749270521686412394046783447265625f,-1.22139995292425140860359533689916133880615234375f,-1.02880000631213164297150797210633754730224609375f,-0.290999977827072431324495482840575277805328369140625f,-0.3287000155925750011220998203498311340808868408203125f,0.7231000124156476971393203712068498134613037109375f,0.48469998760819432703783604665659368038177490234375f,0.9508999726533888807722405545064248144626617431640625f,0.95179999060630759988299587348592467606067657470703125f,-0.87919995384216331757443185779266059398651123046875f,0.271599960291386111066458397544920444488525390625f,-0.599899980372190722022196496254764497280120849609375f,0.3403999940216539954462859896011650562286376953125f,-0.94999996647238749147845737752504646778106689453125f,0.02649997143447446745767592801712453365325927734375f,-0.33339999656975283226501005628961138427257537841796875f,0.3203999832183124230056137093924917280673980712890625f,0.376199986013770359960517453146167099475860595703125f,-0.8208999789118767154150191345252096652984619140625f,-0.06220000876784315124723434564657509326934814453125f,0.026300050121545570647185741108842194080352783203125f,-0.2041999638259408555995833012275397777557373046875f,-0.5442000070765613539691685218713246285915374755859375f,0.70950000259280177061782524106092751026153564453125f,0.0997999891310927811360897976555861532688140869140625f,0.218500013589859154450323330820538103580474853515625f,0.4075000189244748494132863925187848508358001708984375f,-0.482400013005733541149311349727213382720947265625f,-0.5964000148385764088487803746829740703105926513671875f,-0.340900010055303681610894273035228252410888671875f,-0.029900007231533631824049734859727323055267333984375f,-0.2315000083148479603778469027020037174224853515625f,0.037200007724762063077150742174126207828521728515625f,1.04740005830526428098892210982739925384521484375f,0.50490002023279689780110857100225985050201416015625f,0.29409997668266230075317935188650153577327728271484375f,-0.245600044602155787032415901194326579570770263671875f,0.534600028702616913989231761661358177661895751953125f,1.903300011438131544849738929769955575466156005859375f,1.85260001184195299828161296318285167217254638671875f,-0.437799983707070072824762974050827324390411376953125f,-0.25340003024637713924249737829086370766162872314453125f,-1.43520002189576612039445535629056394100189208984375f,-1.2425000226870179798055460196337662637233734130859375f,-0.4185000085234638067532841887441463768482208251953125f,-0.380100010463595427978589214035309851169586181640625f,0.37150003387033925239535392393008805811405181884765625f,-0.573100001612305565146243679919280111789703369140625f,-1.033699994432926150267348930356092751026153564453125f,0.219500010937452305626038651098497211933135986328125f,-0.373600002074241377414409726043231785297393798828125f,-1.489300001484155533404418747522868216037750244140625f,0.5655000173747541936108973459340631961822509765625f,-0.979300003495812365628125917282886803150177001953125f,0.71589999060928832808059496528585441410541534423828125f,0.726800008413195808287809995817951858043670654296875f,0.6442000115841626239898687344975769519805908203125f,-0.7503000348925590312632039058371447026729583740234375f,0.081800026071072284850060896133072674274444580078125f,-1.1813000373065474235545480041764676570892333984375f,0.1150000087544322813215558198862709105014801025390625f,-0.7260000171661376011655875117867253720760345458984375f,0.5785000246912244126207269800943322479724884033203125f,-0.2807000166952615671789317275397479534149169921875f,-0.568600014665722852669205167330801486968994140625f,-0.36859998300075513810725169605575501918792724609375f,0.04930002081841189376376632935716770589351654052734375f,0.1119999841153624942080568871460855007171630859375f,-0.581700004021822980604383701574988663196563720703125f,0.0533000033915040916099314927123486995697021484375f,-0.4510999348700044375704010235494934022426605224609375f,0.35770002875029993649746984374360181391239166259765625f,-0.669699977326393192100795204169116914272308349609375f,-0.3990999978423117600101477364660240709781646728515625f,1.0521000134289268945764206364401616156101226806640625f,1.97160000994950390662552308640442788600921630859375f,-0.21320000045299547508648174698464572429656982421875f,0.3137000006914145711078845124575309455394744873046875f,0.46990000972747747454150157864205539226531982421875f,1.02199994920939207787569102947600185871124267578125f,-0.69809998022913877235851032310165464878082275390625f,-0.184499996632336848989552890998311340808868408203125f,-0.04539999759793289513254421763122081756591796875f,-0.9916000227272505629372290059109218418598175048828125f,0.399999984949827247504572369507513940334320068359375f,-0.193599998646974302829448788543231785297393798828125f,-0.939299960654974341878187260590493679046630859375f,-0.7574000046610829262050401666783727705478668212890625f,-0.24479998920559875585922782192938029766082763671875f,-1.271399970358610520548836575471796095371246337890625f,0.26610001488923984425127855502068996429443359375f,-0.68979996715188018896469657192938029766082763671875f,-0.89699999722838352766984826303087174892425537109375f,0.9726000047266480930119314507464878261089324951171875f,-0.045199994738399862370670234668068587779998779296875f,0.3058999911308288854883130625239573419094085693359375f,-1.03359999499619004126316212932579219341278076171875f,0.920899978166819010994004202075302600860595703125f,-0.2685000104606147797170478952466510236263275146484375f,2.0335999926865095943639971665106713771820068359375f,-0.16270000248700411393798503922880627214908599853515625f,0.94749999664723916215081089831073768436908721923828125f,-0.355599983842670386291473505480098538100719451904296875f,1.44019998330175891254611997283063828945159912109375f,-1.070100021639465825984416369465179741382598876953125f,0.7955999966949216517519971603178419172763824462890625f,1.041099992090463555172163978568278253078460693359375f,0.6557000134915114131217706017196178436279296875f,-0.6775999531686311971867553438642062246799468994140625f,-0.5277999945104117074379246332682669162750244140625f,-0.799399997866153011472079015220515429973602294921875f,-0.502899940712751458704588003456592559814453125f,0.808799963396787280345279214088805019855499267578125f,0.216900001405180287150642470805905759334564208984375f,0.48840003836750967014523894249578006565570831298828125f,0.48850008645653753003301744683994911611080169677734375f,-1.14139994115233367466544223134405910968780517578125f,0.5161999971523882901891511210124008357524871826171875f,-0.50609999685883533260266631259582936763763427734375f,-0.33090001139640801142149939551018178462982177734375f,0.39810000436902015508167096413671970367431640625f,-0.1585000174641606474068566967616789042949676513671875f,-0.45089997876286513900367936003021895885467529296875f,1.985999942362308701149231637828052043914794921875f,-0.16230001386702053167709891567938029766082763671875f,0.3059999956339598536914081705617718398571014404296875f,-0.67070000946819785525576662621460855007171630859375f,0.49929994404017985498711595937493257224559783935546875f,0.05890002087354684334741250495426356792449951171875f,-0.6851999927639962351122449035756289958953857421875f,0.6461000144422051061354750345344655215740203857421875f,0.41060001301169446463745771325193345546722412109375f,-0.9132000312611456838141066327807493507862091064453125f,-0.57759999355077784599643564433790743350982666015625f,0.116900020888447642164464923553168773651123046875f,0.92320002161264458351297435001470148563385009765625f,0.05489999884963037146690112422220408916473388671875f,0.61540000162124641747141140513122081756591796875f,-0.22919999295771120717546409650822170078754425048828125f,0.12859996660947814461906091310083866119384765625f,0.00119999935030978832628534291870892047882080078125f,-0.57590004976689812554724312576581723988056182861328125f,0.8451999580442901560672908090054988861083984375f,0.2979999984875332241784917641780339181423187255859375f,-1.43899996092915483103524820762686431407928466796875f,-0.69239996591806463044349584379233419895172119140625f,0.02750000715255662697700245189480483531951904296875f,-0.691200055192410456328389045665971934795379638671875f,-0.84720000653266946955000094021670520305633544921875f,-1.092199978406727556290434222319163382053375244140625f,0.4297000205397605299140195711515843868255615234375f,0.8318000397056344308310826818342320621013641357421875f,0.44750001341104539420712171704508364200592041015625f,0.1879000294864174858133765155798755586147308349609375f,0.082699962365627488480868123588152229785919189453125f,0.86240001479387284888389331172220408916473388671875f,-0.43669996650814990601929821423254907131195068359375f,-0.80490003431439394177004942321218550205230712890625f,1.3537999835133547321674996055662631988525390625f,1.07310005071163150347501868964172899723052978515625f,-0.3317000091254715954391940613277256488800048828125f,1.645399987614154735382498984108678996562957763671875f,-0.72030000523924808675246822531335055828094482421875f,-0.625300040853023375575503450818359851837158203125f,-0.26079998863935482944498289725743234157562255859375f,0.14650001449882943393987488889251835644245147705078125f,-0.758400035499037006303524321992881596088409423828125f,-0.28609999022781806843340746127068996429443359375f,0.77820004236251183460382208068040199577808380126953125f,0.2149999834597107195577336824499070644378662109375f,0.51570001341700544283952467594644986093044281005859375f,-0.7691000165432690938160931182210333645343780517578125f,0.245200000102818116420877458949689753353595733642578125f,-1.110900001598894792920191321172751486301422119140625f,0.3736999847099180538378959681722335517406463623046875f,-0.8462999995321036639239764554076828062534332275390625f,0.4655999956145879803415255082654766738414764404296875f,-1.1534999752938752681785672393743880093097686767578125f,-2.24860002040267037415333106764592230319976806640625f,-0.781299998787044813752800109796226024627685546875f,0.99150002202391629424482744070701301097869873046875f,0.54150000697374256475313814007677137851715087890625f,0.897599991278350106682637488120235502719879150390625f,-0.29790004602670683286902431063936091959476470947265625f,-0.1642999977216124118939433174091391265392303466796875f,-0.0212999975204468672274060736526735126972198486328125f,0.175899970939755956322869678842835128307342529296875f,-0.7747000523164866425673835692577995359897613525390625f,0.21159996133446679067446893895976245403289794921875f,0.330099969485402322533218466560356318950653076171875f,-0.58999998413026322197083572973497211933135986328125f,-0.06879999446570883581131283790455199778079986572265625f,0.323999994754790687778722713119350373744964599609375f,0.49680001102089921261040217359550297260284423828125f,0.5045999835893508755901848417124710977077484130859375f,-0.0776999851271511499106736664543859660625457763671875f,-0.35870000095218423918908001724048517644405364990234375f,-0.44329999724477542333289648013305850327014923095703125f,1.0697999739319092871170369107858277857303619384765625f,-0.01969999289810697717939547146670520305633544921875f,-0.980899983233213479394407841027714312076568603515625f,0.46930003024339672634823728003539144992828369140625f,-0.11140001923441900100186785493860952556133270263671875f,-0.93769998505264517962842774068121798336505889892578125f,0.117400017327070005368483407437452115118503570556640625f,-0.149900031390040988643619357389979995787143707275390625f,-0.52249999850988348981672970694489777088165283203125f,0.589499987989663853937827298068441450595855712890625f,-0.32079999601542941167764411147800274193286895751953125f,-0.155400010785460285234194088843651115894317626953125f,0.7544000214844943652536812805919907987117767333984375f,0.426800006997585246182325136032886803150177001953125f,0.094299998168647147789300788645050488412380218505859375f,-1.44500000096857572629005517228506505489349365234375f,-1.1241000060938299842661081129335798323154449462890625f,-0.5255999925784766713121598513680510222911834716796875f,0.66000000180676565708637326679308898746967315673828125f,0.8141999623730777901897681658738292753696441650390625f,-0.9731999815657734753671093130833469331264495849609375f,-0.31409997455924809628413640893995761871337890625f,-0.630399979232251883587423435528762638568878173828125f,-0.01779999123215636469552691778517328202724456787109375f,1.0840000044032931381110529400757513940334320068359375f,0.731300029110908411666969186626374721527099609375f,0.453600004160404191022593067827983759343624114990234375f,1.0270999961435796077324766883975826203823089599609375f,-0.64259999010488400017493404448032379150390625f,-0.1601999930806460581411698740339488722383975982666015625f,0.285000002030283194898174770059995353221893310546875f,-0.714599991952627799918218443053774535655975341796875f,-0.27089997578263325106462389157968573272228240966796875f,0.28659998744875225806794105665176175534725189208984375f,0.026199997226893867452446329480153508484363555908203125f,-0.4395999954357743266797342585050500929355621337890625f,-0.19290001868307593557716472787433303892612457275390625f,0.6472999984815717144925883985706605017185211181640625f,-0.132200000702589692824773237589397467672824859619140625f,0.50649998760223358118537362315692007541656494140625f,-0.916300010074674986526588327251374721527099609375f,0.24310000734925285570398045820184051990509033203125f,-0.1031999985158444932409338434808887541294097900390625f,1.48549997371435171800158059340901672840118408203125f,0.133100004070997590677194466479704715311527252197265625f,0.655400019949674561559049834613688290119171142578125f,0.10040000199377541179046602337621152400970458984375f,-1.07100002908706670723404386080801486968994140625f,0.26300003156811035953666078057722188532352447509765625f,-0.573200025486946262986975852982141077518463134765625f,-0.62060000526309011537051674167742021381855010986328125f,-0.239199990312755128574195850887917913496494293212890625f,-1.5357999876521528648254388826899230480194091796875f,-0.695600002599507494238650906481780111789703369140625f,-0.28169996218681314648080160623067058622837066650390625f,-1.0045000020153818098833653493784368038177490234375f,0.860000036954879032435883345897309482097625732421875f,-0.0721999693915245099873345679952763020992279052734375f,1.205699975568056192543053839472122490406036376953125f,0.5013000072807074491976209174026735126972198486328125f,0.8925999948918814652643050067126750946044921875f,-0.1550999897882341382882032121415250003337860107421875f,-0.18350002402067122631024176371283829212188720703125f,-0.43460002356171489878988722921349108219146728515625f,-0.044799998518824768467538888216949999332427978515625f,0.5208000294685373177117071463726460933685302734375f,-0.4891000163197514183366365614347159862518310546875f,-0.4213000100374220124876956106163561344146728515625f,-0.6359000297248365018987215080414898693561553955078125f,-0.2939999709129337279733817922533489763736724853515625f,1.0532000002294779061884355542133562266826629638671875f,1.04710001272112140924264167551882565021514892578125f,-0.1722999930053947803543223926681093871593475341796875f,-0.2781999972864983572407027168083004653453826904296875f,0.7930999982595441366584054776467382907867431640625f,1.131000042796134419376130608725361526012420654296875f,0.783400023913383591889214585535228252410888671875f,1.432399987226724125122245823149569332599639892578125f,0.256200012426078149729846700211055576801300048828125f,-1.075700027871131947421190488967113196849822998046875f,-0.51750002440065168141103413290693424642086029052734375f,-1.3161999674618243272306017388473264873027801513671875f,-0.064299992245435699800282236537896096706390380859375f,0.0825000142678617987002098743687383830547332763671875f,0.2236000066936016938967668465920723974704742431640625f,-0.6030000113770366265697475682827644050121307373046875f,-0.331599998699128428736315754576935432851314544677734375f,0.406200021627545648783552678651176393032073974609375f,-0.1512000474810601957642575143836438655853271484375f,-0.121799997870624121532046046922914683818817138671875f,-1.629699982541799041513286283588968217372894287109375f,-0.54350001206248987895008895065984688699245452880859375f,-0.546499951466917988085469914949499070644378662109375f,0.089300007444620188579165187547914683818817138671875f,0.30079996416419707561118457306292839348316192626953125f,0.72500000692903998622540484575438313186168670654296875f,-0.170999997496604938529429773552692495286464691162109375f,0.16749999426305295369132863925187848508358001708984375f,0.4230999844014646793510792122106067836284637451171875f,0.907899993574620367553507094271481037139892578125f,-0.0694999951422214934382282081060111522674560546875f,1.22040002069473274559641140513122081756591796875f,0.50219998745918292826218021218664944171905517578125f,0.966399964845181091988024491001851856708526611328125f,1.0240999827921390963325620759860612452030181884765625f,-0.3154000253140925025263641146011650562286376953125f,-0.33010000070333500588048991630785167217254638671875f,-0.18899998968839692992105483426712453365325927734375f,0.481200022149085970823989555356092751026153564453125f,0.665400002813338886653582449071109294891357421875f,-0.697700008410215399834441996063105762004852294921875f,1.49090000185966431445194757543504238128662109375f,-0.175499996215104925312289196881465613842010498046875f,0.599800012898445089604138047434389591217041015625f,-0.356199966120720190598802901149610988795757293701171875f,-0.332200002807379046743108119699172675609588623046875f,-0.1095999769955877667371169081889092922210693359375f,-0.485599955344200306939228539704345166683197021484375f,-0.94800003112107500413685556850396096706390380859375f,0.41800003704428689843553001992404460906982421875f,0.337300004293024979773463201127015054225921630859375f,0.30179996616840387702040970907546579837799072265625f,-1.1255000256448983719792522606439888477325439453125f,0.56130000199079521649991875165142118930816650390625f,0.45959998422264991546626333729363977909088134765625f,0.7612000030383467841232913997373543679714202880859375f,0.1494999745786191880370097351260483264923095703125f,0.4213999834716322112626585294492542743682861328125f,0.5789000155091290622522137709893286228179931640625f,-0.3827000069946049354285833032918162643909454345703125f,0.90780001641511898213821041281335055828094482421875f,0.3892000120311973621056722549838013947010040283203125f,-0.20100001597404482112096957280300557613372802734375f,-0.3238000120490791555738496754202060401439666748046875f,-0.19589998382925966069478818099014461040496826171875f,0.899800013047457269976803218014538288116455078125f,0.2795000134706497885161979866097681224346160888671875f,-0.9966999752998353390154306907788850367069244384765625f,0.14520000617504125184353824806748889386653900146484375f,-0.89179998956322659164897004302474670112133026123046875f,-0.153599991084635234539490511451731435954570770263671875f,-0.644699969540536432788258025539107620716094970703125f,-1.117799992424249211353526334278285503387451171875f,-1.3071000105977059746464874478988349437713623046875f,-0.6647000364094974855788677814416587352752685546875f,0.955000017359852737541814349242486059665679931640625f,0.349299990457296427592837062547914683818817138671875f,0.5863999977767464510947093003778718411922454833984375f,0.25450001022964718888630386572913266718387603759765625f,-1.3790999833881854019779211739660240709781646728515625f,0.5153000048667191190787661980721168220043182373046875f,-0.31859999283552176141398604158894158899784088134765625f,1.0391999923244121806220618964289315044879913330078125f,0.228899981944262975730453035794198513031005859375f,1.0999999944493172865378483038512058556079864501953125f,0.34789998105764396996164578013122081756591796875f,-0.378100016885996037530048852204345166683197021484375f,0.96700002047419530981642310507595539093017578125f,0.235000007264316135291437603882513940334320068359375f,-0.12729999628365040731381441219127736985683441162109375f,-1.031599969753623557977562086307443678379058837890625f,-0.6115999888271093087865892812260426580905914306640625f,-0.4762999961420895100872030525351874530315399169921875f,-0.353699992607533875155212399477022700011730194091796875f,-0.866000012844800881595119790290482342243194580078125f,-0.328900000533461689400382965686731040477752685546875f,-0.50499996572732896993329632095992565155029296875f,-0.629000003397464535481731218169443309307098388671875f,-0.03389999855905745373974014000850729644298553466796875f,0.1948000013872981384110971703194081783294677734375f,-1.18010000590011454590921857743524014949798583984375f,-0.13730001051425855251864049932919442653656005859375f,-0.689199998098611654739897858235053718090057373046875f,-0.98070000075101848580061414395458996295928955078125f,-0.634000032007694169777778370189480483531951904296875f,0.19759999742507972086968948133289813995361328125f,0.574300027449428984738233339157886803150177001953125f,1.17860003720372841229391269735060632228851318359375f,-0.74470000219270604535637403387227095663547515869140625f,-0.9197999944396320071149375507957302033901214599609375f,-1.165799998176097584945409835199825465679168701171875f,0.2455999911069870478286247816868126392364501953125f,0.41489998156428331554934629821218550205230712890625f,-1.42050002601742786367822191095910966396331787109375f,-0.36590000483989737478651704805088229477405548095703125f,-1.341699990797042829626661841757595539093017578125f,0.49979998954087445639515863149426877498626708984375f,1.4396000225186345033989709918387234210968017578125f,0.71049998168647354734872578774229623377323150634765625f,2.75580003539174800408773080562241375446319580078125f,0.91249996358528717710356659154058434069156646728515625f,0.480599944708496540357600679271854460239410400390625f,-0.8784999786093832252475976929417811334133148193359375f,-0.157599984312057461721678919275291264057159423828125f,-1.2704999941661956341931727365590631961822509765625f,-0.32220002083778387369505935566849075257778167724609375f,-0.61459997304678015694889836595393717288970947265625f,0.1740000109970571973150299527333118021488189697265625f,1.35190001097917633643419321742840111255645751953125f,0.6643000045388938179513615978066809475421905517578125f,1.3659999790750438908304431606666184961795806884765625f,0.4601000092156228848949695020564831793308258056640625f,-0.6237999930500979228753521965700201690196990966796875f,-0.6434999890588224413789930622442625463008880615234375f,-0.21559994014501615566814507474191486835479736328125f,-0.5190000006407498034377567819319665431976318359375f,0.7305000075399874504000763408839702606201171875f,1.444999996423721455585109652020037174224853515625f,0.33960003838837149459095599013380706310272216796875f,-1.50020001913905165480400683009065687656402587890625f,-0.117999999716877912447898779646493494510650634765625f,0.8382999975800515812807134352624416351318359375f,0.2089999854415651725503266789019107818603515625f,-0.462199948343634758884945767931640148162841796875f,-0.1154999646991490269698488191352225840091705322265625f,-1.2195999983787542131352665819576941430568695068359375f,-0.9797000194408000961487914537428878247737884521484375f,0.113600021669268524959761634818278253078460693359375f,0.52200002390146238440138404257595539093017578125f,0.31939998534917835026902821482508443295955657958984375f,0.5872999870449306936137645607232116162776947021484375f,-0.53340004209279989755287942898576147854328155517578125f,0.1760000005885957730100699336617253720760345458984375f,1.28289998448491093796519635361619293689727783203125f,0.81889999449849160129133451846428215503692626953125f,-1.302400029769539724355809084954671561717987060546875f,-0.2370000110119578895506720073171891272068023681640625f,-1.2164000052645798088946094139828346669673919677734375f,0.08479996085613972223171685982379131019115447998046875f,0.52179997905790809209491953879478387534618377685546875f,1.1531999824970962098547033747308887541294097900390625f,0.3996999915197487762696937352302484214305877685546875f,0.2540999912112951175657826752285473048686981201171875f,-0.2444999974519015406571043058647774159908294677734375f,-0.00660002794265768244486025650985538959503173828125f,0.32700001030415304370535523048602044582366943359375f,-1.7391999894186849218158386065624654293060302734375f,-0.95730000466555387195199955385760404169559478759765625f,-0.555199969872087972788676779600791633129119873046875f,-0.25809998492300512129560274843242950737476348876953125f,0.227700002561510128629151950008235871791839599609375f,0.685300006897002322858725165133364498615264892578125f,0.080799968075751404938955602119676768779754638671875f,1.1339000150620943685453312355093657970428466796875f,0.724599994057416818549199888366274535655975341796875f,0.10000001147389436795265282853506505489349365234375f,-0.7236999842256306436638624290935695171356201171875f,-0.925699995610118175903835435747168958187103271484375f,-0.916599989460408881569719596882350742816925048828125f,1.35379999319911004107552798814140260219573974609375f,0.02239998931288678107875966816209256649017333984375f,-0.032700055348872325566844665445387363433837890625f,0.00770002122521429299695228110067546367645263671875f,1.66689998445510934033109151641838252544403076171875f,1.071199981915950782962454468361102044582366943359375f,0.76830000249743424234338817768730223178863525390625f,1.0506000003829598643534382063080556690692901611328125f,0.156999977558851089298741499078460037708282470703125f,0.7036999805375934169404672502423636615276336669921875f,0.7304999905526639025765689439140260219573974609375f,-1.162299996916949407221864021266810595989227294921875f,1.2645999974846839908337869928800500929355621337890625f,-0.94530000505298428237210828228853642940521240234375f,0.44349996948242154104491419275291264057159423828125f,-0.89909995045661894597088803493534214794635772705078125f,0.86190001276731464940894511528313159942626953125f,-0.43899999237060549628353101070388220250606536865234375f,0.670199996377527806856733150198124349117279052734375f,-0.1752000408947462783970649979892186820507049560546875f,0.199499993465840741446726269714417867362499237060546875f,0.40630002192109826442134590251953341066837310791015625f,-0.18839999273270369162247561689582653343677520751953125f,0.97600000010430765229330063448287546634674072265625f,0.05459998958706846483579511186690069735050201416015625f,1.53289995989799532338793142116628587245941162109375f,0.5459000099062920607906335135339759290218353271484375f,0.8837999793410293847273351275362074375152587890625f,0.604699989098310464186170065659098327159881591796875f,-0.079900021797417952740261171129532158374786376953125f,-1.14679996363520597668639311450533568859100341796875f,0.43720001875162128879281908666598610579967498779296875f,-0.160900061240791814043404883705079555511474609375f,-1.0404999850392340210447628123802132904529571533203125f,0.254199964496493624466211258550174534320831298828125f,0.37299998429417602352486937888897955417633056640625f,0.06979995999932331329773660399951040744781494140625f,-0.98040000758171064187962429059552960097789764404296875f,0.115600000122189072726541780866682529449462890625f,-0.6249000030219555856803026472334749996662139892578125f,0.529600010076164640082652113051153719425201416015625f,0.869600015068054066347258412861265242099761962890625f,0.38459998165220010690745766623876988887786865234375f,0.0194999992214144324265134855522774159908294677734375f,1.30299999843165270618783324607647955417633056640625f,-0.0746999752588575205436427495442330837249755859375f,-0.738299984448402835113256514887325465679168701171875f,0.4307999941527835829901960096322000026702880859375f,0.3043000369489192724614667895366437733173370361328125f,0.391099966460466408335605592583306133747100830078125f,-0.2252999702215199295807224189047701656818389892578125f,0.069900008088350062962490483187139034271240234375f,-0.981099977338314221952941807103343307971954345703125f,0.481399976468086521208533667959272861480712890625f,-0.571500002205371782082465870189480483531951904296875f,0.0943999910861248736626549771244754083454608917236328125f,-1.0108000137478112545608155414811335504055023193359375f,-0.404300011728704333791029057465493679046630859375f,0.66940000203847915827282122336328029632568359375f,-0.06249998815357626913424837766797281801700592041015625f,1.0924999988079069357382877569762058556079864501953125f,-0.154399994848668509206390808685682713985443115234375f,0.8324999994412063397675183296087197959423065185546875f,-0.1324000211641195579037599827643134631216526031494140625f,0.6857000181481243128445157708483748137950897216796875f,-0.36410000480860460481125073783914558589458465576171875f,0.29569992766082275892358666169457137584686279296875f,-0.318500011578202180118069009040482342243194580078125f,-0.7846000634968277598346730883349664509296417236328125f,-1.418800025385618557294264974188990890979766845703125f,-0.813600007624923904359093285165727138519287109375f,-0.0528999867945909585387198603712022304534912109375f,0.4390000755563374656986752597731538116931915283203125f,1.3125000205636023764554920489899814128875732421875f,0.6868000032350420891447129179141484200954437255859375f,-0.020000031366944337474222948003443889319896697998046875f,-0.089799981009960061495434047174057923257350921630859375f,0.0836000236809255614645053356071002781391143798828125f,-0.32439994072020095217112611862830817699432373046875f,0.8360999815851453131898551873746328055858612060546875f,-0.2562999914854771077443729154765605926513671875f,-0.047799995255470406618769629858434200286865234375f,0.98070000060200701597068473347462713718414306640625f,-0.9602999921262267424282299543847329914569854736328125f,0.779599973717332250799927351181395351886749267578125f,0.09790000240355734018038447175058536231517791748046875f,-0.2913000132784249185391445280401967465877532958984375f,-0.575399996666610125117813367978669703006744384765625f,0.5803999916002153813820996219874359667301177978515625f,-0.397199988949298532503462411114014685153961181640625f,-0.82080000279545739516606772667728364467620849609375f,0.194200000482797729972617162275128066539764404296875f,0.2153000036001202488478156737983226776123046875f,-0.27520000826120405434949134360067546367645263671875f,0.49240001360178009548462796374224126338958740234375f,-0.5019999974891542482424711124622263014316558837890625f,-0.4849000276088712979571937466971576213836669921875f,0.80690003329366477569095650324015878140926361083984375f,0.020499969542026263980005751363933086395263671875f,-0.4967000246599317936357920189038850367069244384765625f,-0.77219998999238015358059783466160297393798828125f,0.4112000108987092783507932836073450744152069091796875f,0.991299973492325126045443539624102413654327392578125f,1.28730000231862096171653320197947323322296142578125f,0.892400013303756178828507472644560039043426513671875f,0.68849997797608342153807825525291264057159423828125f,0.9510999889612197222277245600707828998565673828125f,-0.48219998403191599578576642670668661594390869140625f,0.23350000174343588099645785405300557613372802734375f,1.34590001705884976246352380258031189441680908203125f,0.775700012634694235913457305287010967731475830078125f,-0.72729998935461015463488365639932453632354736328125f,-0.093899995057284524335727837751619517803192138671875f,0.81339998531937585113382738200016319751739501953125f,-0.146899967467785330654805875383317470550537109375f,-0.5589999865591526173602687777020037174224853515625f,-0.30040001607537281103077475563623011112213134765625f,-0.758899970209598873083223224966786801815032958984375f,-0.570299981248378884401972754858434200286865234375f,-0.3071999849259856585348416047054342925548553466796875f,-1.134799992334842411167983300401829183101654052734375f,0.5189999685287478570216990192420780658721923828125f,0.1094999829232694565916972351260483264923095703125f,-0.219099986666440482707685077912174165248870849609375f,-1.00260001232624063050025142729282379150390625f,-0.369100013190507780791449476964771747589111328125f,-0.59630000880807598750976694645942188799381256103515625f,-0.5445000098198651272696224623359739780426025390625f,0.6961999999463557298184923638473264873027801513671875f,-0.1258000008583068929368664612411521375179290771484375f,0.2229999904036521485295452293939888477325439453125f,-0.07560001062750831835046483320184051990509033203125f,0.0720999804973601055024801098625175654888153076171875f,0.1138999898791313025725457919179461896419525146484375f,-0.73750000163912776685037897550500929355621337890625f,0.42800003086030524723781809370848350226879119873046875f,-0.5499000343889000053110294174985028803348541259765625f,-0.57040001454800381974763467951561324298381805419921875f,0.0505999851092704400201682801707647740840911865234375f,-1.3312999746471643192080591688863933086395263671875f,-0.8124999808892610086985541784088127315044403076171875f,-0.741399998410046112695681586046703159809112548828125f,-0.9891000201940538527622948095086030662059783935546875f,-0.93880001961141790189913081121630966663360595703125f,-1.5194000281900168491944214110844768583774566650390625f,-1.553100034916401028795007732696831226348876953125f,-1.6671000270262366083073857225826941430568695068359375f,0.501200013171136493639323816751129925251007080078125f,0.454699993754922981992194763734005391597747802734375f,-0.540000009331852215410663120565004646778106689453125f,-0.63489999210685521546793097513727843761444091796875f,0.1314000091016291893453171724104322493076324462890625f,0.673299981859326379662888939492404460906982421875f,0.682099993610381982733770200866274535655975341796875f,0.68589999966174364232074367464520037174224853515625f,0.9572999806001785128017900206032209098339080810546875f,1.0130999798193573990801041873055510222911834716796875f,0.20730000485181765412789900437928736209869384765625f,0.965600013756752417748430161736905574798583984375f,0.13989999344795922553430500556714832782745361328125f,-0.266300005827843910832797291732276789844036102294921875f,-0.18279999378398048559546396063524298369884490966796875f,1.05000000864267395428441886906512081623077392578125f,0.22680000792890808725132956169545650482177734375f,1.58990002343654612815271320869214832782745361328125f,0.5897000103324649966651804788853041827678680419921875f,-0.71560001554489094388600278762169182300567626953125f,0.4641000177912417701264757852186448872089385986328125f,-1.7245000677108770492651501626824028789997100830078125f,0.06829999880939741085938976539182476699352264404296875f,-0.76869996790885897208767119082040153443813323974609375f,0.09029999968856561309848984819836914539337158203125f,-0.9893999971210956090317267808131873607635498046875f,0.26090003039538889328952109281090088188648223876953125f,-0.58510001504570130492766111274249851703643798828125f,0.18399998760223379434819435118697583675384521484375f,-0.08079998238086771866761637284071184694766998291015625f,1.09169996993541662533289127168245613574981689453125f,0.148800003816187853544761310331523418426513671875f,1.12120003583952865255923825316131114959716796875f,-0.1091000354677447159446046498487703502178192138671875f,-1.417899993183464157908701963606290519237518310546875f,-1.516100003713368504776326517458073794841766357421875f,-0.794099983835220513128660968504846096038818359375f,0.7025000178441402898243950403411872684955596923828125f,1.396200020807981445614132098853588104248046875f,-0.054699995356798591927827146719209849834442138671875f,-0.148799989324808645818620789214037358760833740234375f,1.676800006401538922773397644050419330596923828125f,-0.483600028896332023720106008113361895084381103515625f,-0.10109999223947507829279857105575501918792724609375f,1.102999980829656667680183090851642191410064697265625f,-1.264300044604390382829706140910275280475616455078125f,0.62009997781142589534653097871341742575168609619140625f,0.67420002631768582990190452619572170078754425048828125f,-1.042899977034330571967757350648753345012664794921875f,-0.715799994003772699358023601234890520572662353515625f,-0.170799993258714977173440274782478809356689453125f,-0.140699992108344940078268336947076022624969482421875f,-0.101000023610890110603577340953052043914794921875f,0.2746000455409289831010255511500872671604156494140625f,0.393500018283724717349514321540482342243194580078125f,-0.3375000011175870451296532337437383830547332763671875f,-1.05060003454387196342167953844182193279266357421875f,0.215800007563829776557895456789992749691009521484375f,0.012799999316036869689838795238756574690341949462890625f,0.98750002410262804897200794584932737052440643310546875f,0.62960000597834564484145403184811584651470184326171875f,0.5127000102914873735926448716782033443450927734375f,-0.0706000008672471590642771843704394996166229248046875f,1.464499999128282770044506833073683083057403564453125f,-0.500200017946958563896941996063105762004852294921875f,-0.912299997997283451667271947371773421764373779296875f,0.683600012728571737596894308808259665966033935546875f,-0.100500049002468561099021826521493494510650634765625f,-2.39249998953193454553911578841507434844970703125f,-1.21439994854330901574712697765789926052093505859375f,-0.4057999812819061968838241227786056697368621826171875f,-0.4203999817095700475277908481075428426265716552734375f,1.06089997222498055862160981632769107818603515625f,0.11050003343075598394307235139422118663787841796875f,0.6746000418901445527808391489088535308837890625f,-1.297000012204051699882256798446178436279296875f,0.3508999908328058747741806655540131032466888427734375f,0.64910001900196057089686973995412699878215789794921875f,0.8749999887496231476546881822287105023860931396484375f,0.017700017276406132538113524788059294223785400390625f,-0.35940001075565752852725154298241250216960906982421875f,0.2789999658092854328828025245456956326961517333984375f,0.369600001712888481808505503067863173782825469970703125f,1.110999994534999313344769689138047397136688232421875f,-0.168700046344846033008479935233481228351593017578125f,1.0096999856896697878028135164640843868255615234375f,0.45590000111460682319375337101519107818603515625f,0.1626999960422514934776927475468255579471588134765625f,-0.37830001814365410428564473477308638393878936767578125f,0.399600001955032535505552004906348884105682373046875f,0.3462999954342842467980290166451595723628997802734375f,-0.7833999953776598079713266997714526951313018798828125f,0.50790000262707479805612820200622081756591796875f,-1.7891999909833078419296725769527256488800048828125f,0.67610001664012653321833568043075501918792724609375f,-0.1592999324917787618005604599602520465850830078125f,0.2769999897032968672050401437445543706417083740234375f,-0.53980000395774840882268108543939888477325439453125f,-0.4548000000089407190984047701931558549404144287109375f,0.6350999871432778665081286817439831793308258056640625f,-0.0695000237524510122710807991097681224346160888671875f,-0.746500009916722273572986523504368960857391357421875f,-0.45110001313835379477268361370079219341278076171875f,-0.9108000027954579191913353497511707246303558349609375f,0.5974000083491211032793444246635772287845611572265625f,1.589900005145370442960484069772064685821533203125f,1.63309999408721839841973633156158030033111572265625f,1.1739999845102442321831404115073382854461669921875f,1.4186999860882760771119137643836438655853271484375f,1.651199972677230842776907593361102044582366943359375f,0.44569999315142627693830945645458996295928955078125f,0.195999999582767259198590181767940521240234375f,0.25799998936057111365016680792905390262603759765625f,0.2826999924659727714271184595418162643909454345703125f,0.1881999853655695886800458538345992565155029296875f,0.439500004269182842353558271497604437172412872314453125f,-0.93899997631460407898629227929632179439067840576171875f,-1.7774999875202777133154086186550557613372802734375f,-0.01860001831650759385183846461586654186248779296875f,-0.472499917447567696626720135100185871124267578125f,-0.68309998499751056755258105113171041011810302734375f,-1.33550000083446551712995642446912825107574462890625f,-0.221299987834691957999666556133888661861419677734375f,0.1698000561118124807080675964243710041046142578125f,0.27739999657720282311146320353145711123943328857421875f,-1.17189998244345172651037501054815948009490966796875f,0.9121000111564991907897592682274989783763885498046875f,-0.8880999953538182278833801319706253707408905029296875f,0.43300001391023379415656791024957783520221710205078125f,-1.35340000707507090993431120296008884906768798828125f,0.47630002046823516348439397916081361472606658935546875f,-1.92200003917515260809523169882595539093017578125f,0.713500037357211791544386869645677506923675537109375f,-0.2017000202268366404467769825714640319347381591796875f,-0.523000022105872819366823023301549255847930908203125f,0.33080001438111061862201722760801203548908233642578125f,-0.54869998608827597053050340036861598491668701171875f,-0.04269998546242703696407261304557323455810546875f,1.26159998979568399590789340436458587646484375f,0.17560002496987603404221545133623294532299041748046875f,0.90079997805953038181314695975743234157562255859375f,0.8854000102266670335637854805099777877330780029296875f,0.17270000244975136638458934612572193145751953125f,-0.27319999452978382947776481159962713718414306640625f,-0.32419996814727891631946476991288363933563232421875f,0.5267000119566918048263914897688664495944976806640625f,-0.08550005596876086855218090931884944438934326171875f,0.867999980866908771304224501363933086395263671875f,0.7487000062048434489980763828498311340808868408203125f,-0.514499997340142289914410866913385689258575439453125f,-0.379399995556473523805607328540645539760589599609375f,0.0142999825224279231150603663991205394268035888671875f,0.07859999283552188131807270110584795475006103515625f,1.88369997580647474677562058786861598491668701171875f,1.90349998334050152237750808126293122768402099609375f,-0.728000025272368844753145822323858737945556640625f,-0.4073999894618987038796831257059238851070404052734375f,-0.9529000184595586286917523466399870812892913818359375f,0.17259999429583550156763749328092671930789947509765625f,-0.455999967098235980955678314785473048686981201171875f,0.140700018632411882890664855949580669403076171875f,-0.9118999908998610681720720094745047390460968017578125f,-0.21920000141412021577025370788760483264923095703125f,0.6131000143155456161281335880630649626255035400390625f,-1.2974999712407588869922392404987476766109466552734375f,0.2243000320315360340828192420303821563720703125f,0.367400027981400256749111576937139034271240234375f,-1.48700000363588369367562336265109479427337646484375f,-0.85880000340640538780689894338138401508331298828125f,-0.77649998939037312428723680568509735167026519775390625f,0.55439999749362434133814758752123452723026275634765625f,0.2645000063180924509964597746147774159908294677734375f,0.03519998713880767571282603967119939625263214111328125f,0.08710000065118073120373765050317160785198211669921875f,0.85170001747012147319537689327262341976165771484375f,0.21999999877065423703470514738000929355621337890625f,0.3425999916136259759724680407089181244373321533203125f,0.5297999836549165397769911578507162630558013916015625f,-0.28010003840327168944668301264755427837371826171875f,-0.6609000204488635343835767343989573419094085693359375f,0.0112000275135037963281092743272893130779266357421875f,-0.238299996723234652762357654864899814128875732421875f,0.3517000031650068248012530602863989770412445068359375f,-0.1891999942988153993184141654637642204761505126953125f,1.179599977442622016354789593606255948543548583984375f,0.434199998098611761321308222250081598758697509765625f,-0.36239998946189899697856162674725055694580078125f,0.6879000519871709684593952260911464691162109375f,0.7741999751508235050323492032475769519805908203125f,-0.0604000156283373001997460960410535335540771484375f,0.3349000310361385235324860332184471189975738525390625f,-0.6068999744713305943832892808131873607635498046875f,1.080399995660781709005959783098660409450531005859375f,1.22219998403191620894858715473674237728118896484375f,-0.399600008139014306607350590638816356658935546875f,-0.92179999708831328320002285181544721126556396484375f,0.773299984206258894658958524814806878566741943359375f,0.770399993127584448160405372618697583675384521484375f,-0.65629999651461812248953719972632825374603271484375f,-1.15429999552369100257465106551535427570343017578125f,-0.4650999822631483038293254139716736972332000732421875f,-2.2714000184893610168046507169492542743682861328125f,0.0283000099107624247807279971311800181865692138671875f,-0.40769995670318603941950641456060111522674560546875f,0.233900017967819895403636110131628811359405517578125f,1.542999996513128024844263563863933086395263671875f,0.367000018835067720601728069595992565155029296875f,0.28769999160915615821210167268873192369937896728515625f,0.49449996519088823543341959521058015525341033935546875f,0.2978000085175036559803629643283784389495849609375f,-1.2894999778568756454433241742663085460662841796875f,-0.12509998770207175766699947416782379150390625f,-0.9111000087797638258280130685307085514068603515625f,0.176199999983608801557011247496120631694793701171875f,0.870800018590687674446826349594630300998687744140625f,0.2540000166594982289325344027020037174224853515625f,0.6412999349355696931951342776301316916942596435546875f,0.498900009846687420012045777184539474546909332275390625f,-2.013499992765486634738181237480603158473968505859375f,0.5168000084877011346407016390003263950347900390625f,1.1799000004887580939083591147209517657756805419921875f,0.5691000114023676559327213908545672893524169921875f,-0.927700011539459357123860172578133642673492431640625f,0.066400042182206409080436060321517288684844970703125f,0.4296000260949133231491714468575082719326019287109375f,0.6087000249803065088372022728435695171356201171875f,0.113500003755092837565143781830556690692901611328125f,0.6048999873757363587856161757372319698333740234375f,-0.30570001863241191841780164395458996295928955078125f,0.561299998414515499689514399506151676177978515625f,0.11990000570416459169109657523222267627716064453125f,0.06300003018975264268419778090901672840118408203125f,-0.048000019013881622953476835391484200954437255859375f,0.5085000454783443757378336158581078052520751953125f,-0.414600031980872085757283684870344586670398712158203125f,-1.043999995201825736756973128649406135082244873046875f,0.0001999819234017596869534827419556677341461181640625f,0.3256999928534032306970402714796364307403564453125f,0.9475000087171794316276418612687848508358001708984375f,0.642299951356649412304022916941903531551361083984375f,-0.6512000453203918848288367371424101293087005615234375f,0.67350003430247351587922821636311709880828857421875f,1.346199997860193064980194321833550930023193359375f,0.1351999998420476689631186673068441450595855712890625f,0.58590000483989701507425706950016319751739501953125f,0.47960001689344633479805679598939605057239532470703125f,-0.7843999443709852581463337628520093858242034912109375f,0.000500010073184842251947657132404856383800506591796875f,0.637199956166743408658703629043884575366973876953125f,1.24860001533627507086521291057579219341278076171875f,0.18339998353123743157766512013040482997894287109375f,0.0616999801799653635470122026163153350353240966796875f,0.728100028806924814972489912179298698902130126953125f,0.89160002172142271970045612761168740689754486083984375f,-0.0628999970018870868671001517213881015777587890625f,-0.18509998284056783379725175109342671930789947509765625f,-0.6501000062167643367416758337640203535556793212890625f,-0.3064999820888036907717832946218550205230712890625f,-0.5020000084042550714258368316222913563251495361328125f,-0.90959999226927745752391274436376988887786865234375f,-0.729099989721178953772096065222285687923431396484375f,0.17389997840523729877304504043422639369964599609375f,0.6126000022679567447170256855315528810024261474609375f,0.85519997793734103819218717035255394876003265380859375f,-0.85730002686828365998650269830250181257724761962890625f,-0.072999998804181842615435016341507434844970703125f,0.4852000278934835986177631639293394982814788818359375f,0.65030000604018567411657159027527086436748504638671875f,0.5974000219836830272157612853334285318851470947265625f,-0.30250001668930082132646930404007434844970703125f,0.04789998705536124834480915524181909859180450439453125f,-0.43820000492334376662739714447525329887866973876953125f,-0.40190000129342084989048089482821524143218994140625f,1.079300000292062744478016611537896096706390380859375f,-0.10099998002499344096349886967800557613372802734375f,1.323600010195374210297813988290727138519287109375f,-0.623100014725327611131433513946831226348876953125f,-0.7710000034756958608994636961142532527446746826171875f,1.030599989728629761742695336579345166683197021484375f,0.17820001481398950460288688191212713718414306640625f,-1.293499994367361072278299616300500929355621337890625f,-0.643500006213784114805775971035473048686981201171875f,-0.179899989387392711392976707429625093936920166015625f,-0.282000000692904040366926210481324233114719390869140625f,1.1538000092923639261499602071125991642475128173828125f,0.183700002554059105275285901370807550847530364990234375f,0.84489996722191573752525073359720408916473388671875f,0.1538999968454242139870302708004601299762725830078125f,-0.592499975580721649492943470249883830547332763671875f,0.10859999713823198508322320776642300188541412353515625f,0.7157999941527839471433480866835452616214752197265625f,-0.2661999919369810729818937033996917307376861572265625f,-0.31369999940618864542329902178607881069183349609375f,0.0780999971233307110907162496005184948444366455078125f,0.6166000099122523891992386779747903347015380859375f,-0.1258999899387360787983425325364805757999420166015625f,-1.2293000143736600993094043587916530668735504150390625f,0.06570002585947509743391492520458996295928955078125f,1.120800026041269603638284024782478809356689453125f,0.4351999933600427805657773205894045531749725341796875f,0.53409995887577499384946122518158517777919769287109375f,-0.9690000243335958884927094914019107818603515625f,0.658800019574165229840900792623870074748992919921875f,-0.563800012905895986392579288803972303867340087890625f,-0.06250000713393089046832074018311686813831329345703125f,0.193499995131045654517265575123019516468048095703125f,0.81009999660551557099097408354282379150390625f,0.6478999978959560923641447516274638473987579345703125f,-0.2374000239580869031641441324609331786632537841796875f,-0.3491000247389077770776566467247903347015380859375f,0.635400046361982884235430901753716170787811279296875f,-0.5208000050306320627413469992461614310741424560546875f,-0.47089999407380889184793204549350775778293609619140625f,-1.57399999155104186598919113748706877231597900390625f,-1.04229998168051185558624638360925018787384033203125f,0.1568999978102743053653966853744350373744964599609375f,0.1501000065147877204907445047865621745586395263671875f,0.12119999944344161946219173842109739780426025390625f,0.0757999791771173381249582234886474907398223876953125f,-0.048799993757903614710613737770472653210163116455078125f,-0.846399986302852624930892488919198513031005859375f,0.15060000418275587552585648154490627348423004150390625f,-0.5927000226408247751663793678744696080684661865234375f,-0.308100015321373821297612494163331575691699981689453125f,-0.5634999831914893064777061226777732372283935546875f,-0.9033000013053416044073173907236196100711822509765625f,-0.73869999040961342728905947296880185604095458984375f,1.14149999274313440622563575743697583675384521484375f,1.095599976056814028879671241156756877899169921875f,0.6332999919176103276186040602624416351318359375f,0.7639999813437459152964947861619293689727783203125f,-0.03069997530728496570873176096938550472259521484375f,0.953600034633279136642158846370875835418701171875f,-0.2546999982997772971060612690052948892116546630859375f,-0.244800031971931897345484685502015054225921630859375f,0.7689000023961067853406348149292171001434326171875f,0.55009999966025358020260682678781449794769287109375f,0.027600006663799803163783508352935314178466796875f,0.58610002122223381348931070533581078052520751953125f,-0.630099963524937578540630056522786617279052734375f,-0.4309999871402983018242593971081078052520751953125f,-1.76929999358654033159155005705542862415313720703125f,-0.03239998849332348385132718249224126338958740234375f,-1.659900004921853966521894108154810965061187744140625f,1.058600031839311395032154905493371188640594482421875f,-0.1735000098273158375405955666792578995227813720703125f,-0.94990001766234666291666144388727843761444091796875f,-0.4039000077605248151257910649292171001434326171875f,-0.0344999729394912524327310165972448885440826416015625f,-1.548800012868642994590118178166449069976806640625f,0.6103999836280944446542662262800149619579315185546875f,0.61859998937100169058567189495079219341278076171875f,-0.756300023746490523990360088646411895751953125f,0.363299995568394873401985023519955575466156005859375f,-0.64380003898292781361334391476702876389026641845703125f,0.57790000385642048552625738011556677520275115966796875f,-0.026500019788742334991837878988008014857769012451171875f,-0.143000008843839176986278971526189707219600677490234375f,-0.393400011843442865000497477012686431407928466796875f,0.26930000681132082451796350142103619873523712158203125f,-0.740700007456541076322764638462103903293609619140625f,0.05359999376684421168448579919640906155109405517578125f,0.51689999723285418031792914916877634823322296142578125f,1.0794000117242337211109770578332245349884033203125f,-0.117599978798627768927786974018090404570102691650390625f,0.19789996388405572957935874001123011112213134765625f,-0.49230002861917032674909933120943605899810791015625f,1.40699995807558320137786722625605762004852294921875f,1.049100036808848557257078937254846096038818359375f,0.0066999993652103739805170334875583648681640625f,-0.910700002092122762320514084422029554843902587890625f,-0.36970002992749240178227410069666802883148193359375f,0.1157000121504072609468494192697107791900634765625f,0.67529998333454077652504565776325762271881103515625f,-1.111600073689222423212186185992322862148284912109375f,1.4552999623239044613143278183997608721256256103515625f,-0.525300002110004715660807050880976021289825439453125f,0.7921000065743921680905259563587605953216552734375f,0.3892999961197372016385997994802892208099365234375f,-0.3658999623715881721608411680790595710277557373046875f,0.276099961692094719722945228568278253078460693359375f,0.18560002925395924222584653762169182300567626953125f,0.484899977391958447725528458249755203723907470703125f,-0.44490002582073184811406463268212974071502685546875f,-0.12229999024868032364565806346945464611053466796875f,1.08469997762441661137700066319666802883148193359375f,-0.796499981939792434104674612171947956085205078125f,0.3147999979972837536479346454143524169921875f,-0.3348000156551602657373223337344825267791748046875f,0.8830000091418626073647146768053062260150909423828125f,0.40130000631213180728451561662950552999973297119140625f,-0.5455000089928507822634173862752504646778106689453125f,0.1000000159069898142405463659088127315044403076171875f,-0.0511000196576123499880850431509315967559814453125f,-1.1276000236138703503030455976841039955615997314453125f,0.38630000005364450998968095518648624420166015625f,0.3318000113934278516580889117904007434844970703125f,0.3389000012040137566060593599104322493076324462890625f,0.0556000104784966919790889505748054943978786468505859375f,1.765800022129714097474106893059797585010528564453125f,0.06489998864233481601360153945279307663440704345703125f,-0.0855999933794138190279454647679813206195831298828125f,0.5941999899774790261375301270163618028163909912109375f,0.4141999974653123306467250586138106882572174072265625f,1.2899000083863738641554164132685400545597076416015625f,1.938599989557264979822548411902971565723419189453125f,-1.24170000852942497004960387130267918109893798828125f,-0.47370000821649949074299001949839293956756591796875f,-0.01119999964833251482332343584857881069183349609375f,0.24400000116229036972725907617132179439067840576171875f,-0.2215999845802787415749435240286402404308319091796875f,-0.711699988710880138143011208740063011646270751953125f,-0.07950000255554900974175325245596468448638916015625f,0.29749999690800876805951702408492565155029296875f,-0.2498000022068620484816392490756697952747344970703125f,-0.3138999962493775885974400807754136621952056884765625f,-0.80289995006322900650275187217630445957183837890625f,-1.2653000016629694091108149223146028816699981689453125f,0.08730001260042197230859528644941747188568115234375f,-0.6794999832212926804686503601260483264923095703125f,0.2276999944776296747050992053118534386157989501953125f,0.48170000867843631997544662226573564112186431884765625f,0.48329999147057500774593563619419001042842864990234375f,1.039000011555850644384690895094536244869232177734375f,-0.218899988873302808745080483276979066431522369384765625f,-0.133600004830956020640542192268185317516326904296875f,-0.35850000669807169373370925313793122768402099609375f,0.8469000215589996827247887267731130123138427734375f,-0.7045000129863623339332434625248424708843231201171875f,2.13480004955530233701210818253457546234130859375f,-0.013100011372566111589321735664270818233489990234375f,0.3170000097453591347829160440596751868724822998046875f,0.16380002095252266247626948825200088322162628173828125f,-0.473000005900859665786128971376456320285797119140625f,-0.67500002712011364369715238353819586336612701416015625f,0.38570002433210592851509090905892662703990936279296875f,0.48220002344548651418421059133834205567836761474609375f,-0.34340000800639292055649320900556631386280059814453125f,-0.445399955576657902156512136571109294891357421875f,0.56720005552023611272716152598150074481964111328125f,0.0356999985158434895993195823393762111663818359375f,0.5476000438794506042228249498293735086917877197265625f,0.479700013983249906601713519194163382053375244140625f,0.2493000336334103561597430598339997231960296630859375f,1.2044000249862667484279654672718606889247894287109375f,0.1920999275237325054632719911751337349414825439453125f,-1.65170001169592151057941009639762341976165771484375f,0.72779998963028180725132187944836914539337158203125f,0.64939997432231899576748901381506584584712982177734375f,-0.46089999157786365469036127251456491649150848388671875f,-0.7220999764740465565182603313587605953216552734375f,0.21379996682405444374808212160132825374603271484375f,-0.762499995380640172015773714520037174224853515625f,0.6440000139772887877143148216418921947479248046875f,-0.51659999068975448466289890347979962825775146484375f,-0.009700016665458832676449674181640148162841796875f,1.19129999006986597720469944761134684085845947265625f,-0.8364999671876434117478993357508443295955657958984375f,-0.407599988521635481220073415897786617279052734375f,-0.369900017625093280315695665194652974605560302734375f,0.05180003206878946997449020273052155971527099609375f,0.2925999522745608150131602087640203535556793212890625f,-0.10360000069588382043406227239756844937801361083984375f,0.745700015801191273823178562452085316181182861328125f,0.434900002425909093517475412227213382720947265625f,1.443300024551152915819329791702330112457275390625f,-0.3924000385612256280865040025673806667327880859375f,-0.681999981135129385023674331023357808589935302734375f,0.78800005138665507598005888212355785071849822998046875f,0.55039997718334188192557121510617434978485107421875f,-0.0633999969795342455114450785913504660129547119140625f,0.7722000514596703002467847909429110586643218994140625f,1.17560005682110801927819920820184051990509033203125f,0.3325000053644178166223355219699442386627197265625f,-0.3356999758660792476661072214483283460140228271484375f,0.3954000928908587564336585273849777877330780029296875f,2.237299997252226280153308834997005760669708251953125f,0.5881000118568537882168811847805045545101165771484375f,-0.40000003919005422403643024154007434844970703125f,-0.2955999932676542929499419187777675688266754150390625f,1.954800038081407453915971927926875650882720947265625f,0.8909000250309702106932263632188551127910614013671875f,0.19510002350211141930458325077779591083526611328125f,-0.6333000203788277104877124656923115253448486328125f,0.88049998071789747910997903090901672840118408203125f,-0.403100035661458466762496755109168589115142822265625f,-1.148699998456239601551942541846074163913726806640625f,0.6086999610543253336203406433924101293087005615234375f,0.3110999849379065818055778436246328055858612060546875f,0.793999981194734250067313041654415428638458251953125f,-0.78260000383257821710003554471768438816070556640625f,-0.622400003767013654254469656734727323055267333984375f,0.352099977517127626924775540828704833984375f,0.389899985587597530667380851809866726398468017578125f,0.246800004017352581087152429972775280475616455078125f,0.1993999911792574408764266991056501865386962890625f,-0.3293000188067558564597447912092320621013641357421875f,-0.1963999997138976549848621289129368960857391357421875f,0.09089999653249980493541215764707885682582855224609375f,0.0786000057995319689752022895845584571361541748046875f,0.6066000064104795086450394592247903347015380859375f,-1.1455999852210283496134479719330556690692901611328125f,0.1839000121936200660144322682754136621952056884765625f,-1.0558000224649906062523996297386474907398223876953125f,0.206200009036064113576713907605153508484363555908203125f,0.81830000281408443729702639757306315004825592041015625f,-0.034600010001659187164335662600933574140071868896484375f,-1.209100039155781569633063554647378623485565185546875f,-1.619300020706653686630716038052923977375030517578125f,-1.0370000202879310347015007209847681224346160888671875f,-0.5517000179916620172804186950088478624820709228515625f,-0.43579998703747968580302085683797486126422882080078125f,0.6596999946862458497065517804003320634365081787109375f,0.63079999087452875983217381872236728668212890625f,0.6853000102683901584299519527121447026729583740234375f,0.4067999445989727291816961951553821563720703125f,0.134899984395503824696760375445592217147350311279296875f,0.4009000315688553417459161209990270435810089111328125f,-0.03569999289065572956047844854765571653842926025390625f,0.34130000104084612377874918820452876389026641845703125f,-0.04809999036192902366337875719182193279266357421875f,0.54780000211000423604446041281335055828094482421875f,-0.040700011554360315813028137199580669403076171875f,0.59440000419914718055025559806381352245807647705078125f,-0.3714999973624946161265825139707885682582855224609375f,-0.6722999727398157876478990147006697952747344970703125f,0.60339999954998491116242576026706956326961517333984375f,-1.2122999811217189947143424433306790888309478759765625f,-0.457200010835379366813668866598163731396198272705078125f,0.80759999327138076719023729310720227658748626708984375f,0.364499995980411828355727266171015799045562744140625f,0.8321000171542169976390823649126105010509490966796875f,0.16590000092834233935690235739457421004772186279296875f,-0.08159999635219572500233198297792114317417144775390625f,-0.10219999934285872722483645702595822513103485107421875f,-0.283900013720989274901285170926712453365325927734375f,0.2958000017151236615831066956161521375179290771484375f,0.6958000206768513162813860617461614310741424560546875f,-0.92870003272891066270489091039053164422512054443359375f,0.0804000017702577451927936635911464691162109375f,-0.6106999917358157947688823696807958185672760009765625f,0.081500009804964523851822377764619886875152587890625f,-0.39340000465363278436825567041523754596710205078125f,-0.49590000670254175130224894019193015992641448974609375f,0.21269999412372708746943317237310111522674560546875f,0.13620000665187836919045594186172820627689361572265625f,-0.1580999892443417653709047954180277884006500244140625f,-0.82560002813637256569023747942992486059665679931640625f,0.00949998627603054757173595135100185871124267578125f,0.0190000326037409461576999092358164489269256591796875f,-0.9335000014454124528384681980242021381855010986328125f,-0.355400023190677760709377253078855574131011962890625f,-1.44789997532069758534589709597639739513397216796875f,-0.12099998248368415421083454930339939892292022705078125f,-0.3331999678194517144902420113794505596160888671875f,0.33389999436810569033440287967096082866191864013671875f,-0.1704000023849308320667006455551018007099628448486328125f,0.05449999929592019842772288029664196074008941650390625f,0.8648999893128872518133221092284657061100006103515625f,0.0070999962180853248838729996350593864917755126953125f,-0.35040001346766935785836949435179121792316436767578125f,1.1197999901741741979321886901743710041046142578125f,-0.2595999527812005158722286068950779736042022705078125f,-0.6479000076562166920979279893799684941768646240234375f,-0.14269999537169975045713954386883415281772613525390625f,-0.338999963983893426444637952954508364200592041015625f,0.138800002437830638513105441234074532985687255859375f,0.994599975132941960254129298846237361431121826171875f,-0.734599998825788080836218796321190893650054931640625f,1.92270002539753903647579136304557323455810546875f,-0.43080001479089291560597985153435729444026947021484375f,2.333700032170117122376495899516157805919647216796875f,-0.6419000389933591232960452543920837342739105224609375f,0.030900004504621048084800349897705018520355224609375f,0.15150002489239022818168223238899372518062591552734375f,0.648100023144483561310380537179298698902130126953125f,-0.6570000300854450525633865254349075257778167724609375f,1.35150002664327661960896875825710594654083251953125f,0.08049999982863675906941125504090450704097747802734375f,1.5246000125348562193039470002986490726470947265625f,0.4084000149726865203092529554851353168487548828125f,0.789500031575560257124379859305918216705322265625f,0.68079998096525640161047476794919930398464202880859375f,0.2520000317618258378615792025811970233917236328125f,-0.658600019732117569759566322318278253078460693359375f,-0.789600016595423337406600694521330296993255615234375f,0.664900025969743513343246377189643681049346923828125f,0.956900022713840225918602300225757062435150146484375f,-0.276400015956163702668391124461777508258819580078125f,0.1969000313937672164144032649346627295017242431640625f,0.472600002156198240044204794685356318950653076171875f,-0.558000032797455691735422078636474907398223876953125f,-0.651899958825111980331712402403354644775390625f,0.17230000425577163269963421043939888477325439453125f,-0.321500008799135539305780184804461896419525146484375f,1.1210999929472797997931365898693911731243133544921875f,-0.05499998506158565447776709333993494510650634765625f,-0.179900013191998164874263466117554344236850738525390625f,-0.18839999604821233702978133806027472019195556640625f,0.054100017958879309531994294957257807254791259765625f,-0.7448999958693978129531387821771204471588134765625f,0.359899987375735808114995961659587919712066650390625f,0.614299981628358526819511098437942564487457275390625f,-1.4841000176981093972017333726398646831512451171875f,0.461799998839199421496459763147868216037750244140625f,-0.852499988190829949274984755902551114559173583984375f,0.705400022669136372854836736223660409450531005859375f,-0.458099989505111881982202248764224350452423095703125f,-0.0013000100746751730440564642776735126972198486328125f,-0.307800010305643301222744412370957434177398681640625f,0.6350000074505803837610073969699442386627197265625f,-1.199200012850761165594803969725035130977630615234375f,-0.0772999838784331760876966654905118048191070556640625f,-1.7102000557214027676167233948945067822933197021484375f,0.6873000507846478424056613221182487905025482177734375f,-1.001199997822940002123459635186009109020233154296875f,-0.362200014020502170097870475728996098041534423828125f,-0.064500006113201335811169201406300999224185943603515625f,-1.018899927443265962523355483426712453365325927734375f,-1.03209999945908759144685973296873271465301513671875f,1.149200059342384339089448985760100185871124267578125f,0.08180000364482420405209950331482104957103729248046875f,1.46100000017881459513091613189317286014556884765625f,-0.0375000042468311978183237442863173782825469970703125f,0.9241000569999204827809080597944557666778564453125f,0.21860003258436933037955895997583866119384765625f,1.021199971336126832710533562931232154369354248046875f,-0.94929998077154120750265064998529851436614990234375f,-0.6704000217378141002200209186412394046783447265625f,-0.1264000158071514334778839838691055774688720703125f,-0.449400007402896761732336017303168773651123046875f,-0.6669000053167337682680226862430572509765625f,1.53280002535581605371817204286344349384307861328125f,0.4527999949574468274704486248083412647247314453125f,-0.876200025129318049721405259333550930023193359375f,-0.76520001973509810255791308009065687656402587890625f,0.8319000216335059594285894490894861519336700439453125f,-0.62180002104193032863577172975055873394012451171875f,-0.966600015351176455880022331257350742816925048828125f,-0.4121000128328802247779094614088535308837890625f,0.1714999960586425231667817570269107818603515625f,-0.9788000757515431526911697801551781594753265380859375f,-0.4586999794200063984561666075023822486400604248046875f,-1.381699993330240516797857708297669887542724609375f,0.4999000129684809667907074981485493481159210205078125f,0.889000023812055673033682978712022304534912109375f,0.12129999815374592486705296323634684085845947265625f,-1.029400022900104350043193335295654833316802978515625f,-0.4888999974787233693263033273979090154170989990234375f,0.1930000531747939618298914865590631961822509765625f,-1.39780001134872389201291298377327620983123779296875f,-0.226500010997057188433245755732059478759765625f,-1.725800016653537216626546069164760410785675048828125f,-0.53520000941604362054704324691556394100189208984375f,-0.279999972879886538379423654987476766109466552734375f,-0.18590000774562354113328410676331259310245513916015625f,0.5926999510407444660131659475155174732208251953125f,0.435399996927380517064420928363688290119171142578125f,-1.0314999480396498388046211402979679405689239501953125f,-1.095600024820864160091105077299289405345916748046875f,0.84190001556128224446950980563997291028499603271484375f,-0.37779999700635669146464579171151854097843170166015625f,-0.211299981576203721456153061808436177670955657958984375f,-1.586800003719330387497166157118044793605804443359375f,0.9104000148832798178233360886224545538425445556640625f,1.08459999763369552283620578236877918243408203125f,0.71060001062750810518764410517178475856781005859375f,-0.4219000017404557212330473703332245349884033203125f,0.39390001103878002908231792389415204524993896484375f,-0.33699995476007416073116473853588104248046875f,-1.502899991339444962790139470598660409450531005859375f,-0.02770000349283208151973667554557323455810546875f,1.38390000343918817549138111644424498081207275390625f,0.9641999405801298106410968102863989770412445068359375f,-0.24260000692456984427991528718848712742328643798828125f,-2.646200013059377464941235302831046283245086669921875f,-0.79229998063743067859832081012427806854248046875f,-0.3465999887526034495266458179685287177562713623046875f,0.3065999890506267977485777009860612452030181884765625f,-1.0005000257194041068942169658839702606201171875f,1.710399999795854331097189060528762638568878173828125f,-0.00900004988908786884849178022705018520355224609375f,-0.3598999998182055293227676884271204471588134765625f,-0.57250004544854160570821477449499070644378662109375f,-0.51679996642917414018114641294232569634914398193359375f,0.3364000143170355894284284659079276025295257568359375f,0.3157000047743314663506453143781982362270355224609375f,-0.5872000032544135716960909121553413569927215576171875f,0.300299980801343568970196429290808737277984619140625f,0.381899989446997434328068266040645539760589599609375f,-0.53050003406405465966599876992404460906982421875f,0.416100005017220997416416139458306133747100830078125f,-1.07750005185604091906270696199499070644378662109375f,-0.0337999981164933682720175056601874530315399169921875f,-0.53969997613430020777514073415659368038177490234375f,-0.214900018966198036451942243729718029499053955078125f,0.917799962398409885366845628595910966396331787109375f,0.29059998678565046592581211370998062193393707275390625f,0.401299989566951731401189817916019819676876068115234375f,0.35570000313520377943632411188445985317230224609375f,0.449100003206730047367045699502341449260711669921875f,-0.0304000177145002936640594271011650562286376953125f,1.2242999968647954034395297640003263950347900390625f,0.2223000036597255757442326284945011138916015625f,-0.26270001779794682761348667554557323455810546875f,-0.07440001872777912694800761528313159942626953125f,1.6330999970674511256873984166304580867290496826171875f,-0.9594999811351296870043370290659368038177490234375f,0.11120000873804070096184659632854163646697998046875f,0.42890002370476754123274076846428215503692626953125f,1.15130001525282832375296493410132825374603271484375f,-0.387499985098838539698817839962430298328399658203125f,-0.2350999729126694859360213740728795528411865234375f,-0.79259997701048856555416932678781449794769287109375f,-0.372799980391561813775069822440855205059051513671875f,-0.4348999817132952472320539527572691440582275390625f,-0.7808999842390420553783769719302654266357421875f,-2.2328000256538391710137148038484156131744384765625f,0.7041000103965397993732722170534543693065643310546875f,-1.0936999756574632414185543893836438655853271484375f,-0.8794999779686329954841994549497030675411224365234375f,-1.0342999990999699466698302785516716539859771728515625f,0.0953000173836950903449860561522655189037322998046875f,0.10950000352412458948236917422036640346050262451171875f,0.035800016951560653666319922194816172122955322265625f,-0.20480000202059722713698874940746463835239410400390625f,-0.03610004096627239533745523658581078052520751953125f,-0.0387000055715442936588033262523822486400604248046875f,-1.5025999846100805967807900742627680301666259765625f,-0.241900011128187042430681685800664126873016357421875f,-0.106100011014938377940097780083306133747100830078125f,-0.39579998390823600828980488586239516735076904296875f,-0.9808000008583077633517177673638798296451568603515625f,-0.316499991960823623404763793587335385382175445556640625f,-1.54110002338290197343440013355575501918792724609375f,0.60639997199773798541855285293422639369964599609375f,-0.724500034183263696974108825088478624820709228515625f,-0.2511000357508663682892802171409130096435546875f,1.4399000146448610593097328091971576213836669921875f,0.80090002212524336044907613540999591350555419921875f,-1.133899964100122392807179494411684572696685791015625f,-1.3525000005960468030252741300500929355621337890625f,-0.2556000077962874428294526296667754650115966796875f,1.5083999561131002931091416030540131032466888427734375f,0.08450000475347019257554848081781528890132904052734375f,-0.19109997711479653759170105331577360630035400390625f,-0.2127999776721001268242616788484156131744384765625f,0.677800003302097220370114882825873792171478271484375f,-1.1659000156059866615265718792215920984745025634765625f,-0.4316999684453008967466303147375583648681640625f,-0.60310000049471812477719367961981333792209625244140625f,0.377700064140558566805339069105684757232666015625f,-0.2780000291094177899964279276900924742221832275390625f,-0.68770003642439814228737304802052676677703857421875f,-0.09290000188201685826783204902312718331813812255859375f,1.8665999664008616765187298369710333645343780517578125f,0.830499985076487146073986878036521375179290771484375f,1.2276999992460011501549388412968255579471588134765625f,0.10070001103281978060977053246460855007171630859375f,0.03249998379498719724978172962437383830547332763671875f,0.537700069057942098282865117653273046016693115234375f,-0.25759999928772436827983938201214186847209930419921875f,0.76010000308752001529910558019764721393585205078125f,0.1011000088542697294968775167944841086864471435546875f,0.03030001522302594452185076079331338405609130859375f,0.0392999969393013071083942122641019523143768310546875f,1.19109995982945005010833483538590371608734130859375f,0.8595999931260944748601104947738349437713623046875f,0.822399981564283510948598632239736616611480712890625f,0.1716000090181826198687531359610147774219512939453125f,0.29409999665021846482204637140966951847076416015625f,-2.08850001060962586763025683467276394367218017578125f,-1.397000041782856083472097452613525092601776123046875f,0.2316999964594845806686862488277256488800048828125f,0.35300000739097558977164226234890520572662353515625f,1.12280004740953387454283074475824832916259765625f,-1.1777999723076817417677375487983226776123046875f,-0.88299998980760552313995503936894237995147705078125f,-0.069799997103214383997737968456931412220001220703125f,1.59169997917413663657271172269247472286224365234375f,0.49030000583529531610338381142355501651763916015625f,1.05859993650913253304679528810083866119384765625f,0.7172999462902538425623788498342037200927734375f,-0.99710000314712576852116399095393717288970947265625f,-0.1250000152736905345562945512938313186168670654296875f,1.412899997746944080745379324071109294891357421875f,0.17299997263401767355617266730405390262603759765625f,1.054600009554624850949267056421376764774322509765625f,0.56869999432116802307035641206311993300914764404296875f,-0.38440005344748495730300419381819665431976318359375f,-0.37270001600980739997481805403367616236209869384765625f,1.485099980884790671353812285815365612506866455078125f,-1.050799964275956011050539018469862639904022216796875f,-0.2425999939978122910133606637828052043914794921875f,-0.154600036264956219866917308536358177661895751953125f,-0.5371000000923868977764641385874710977077484130859375f,0.65420001299977403874663650640286505222320556640625f,0.1324999968335032196620204558712430298328399658203125f,0.1087999808311455041121007525362074375152587890625f,0.5113000235974787255344153891201131045818328857421875f,-0.382299996805191000248669297434389591217041015625f,0.151199988248944094948456040583550930023193359375f,1.20899998208880443684165584272705018520355224609375f,0.46739998741298904860741458833217620849609375f,0.32319997765421870639102053246460855007171630859375f,0.66809997363537565462365819257684051990509033203125f,-0.852800023418664121521715060225687921047210693359375f,-1.14239999706149131242227667826227843761444091796875f,-0.6223999897599219455202046447084285318851470947265625f,-0.5968000214517115153967097285203635692596435546875f,-0.15719999535679818336575408466160297393798828125f,-0.21849997469782866943432964035309851169586181640625f,0.10310000705122934760993302916176617145538330078125f,0.05540002188682446870870990096591413021087646484375f,0.1361000522166495585452139494009315967559814453125f,-1.264200017246604357268324747565202414989471435546875f,-1.39060005372911721366335768834687769412994384765625f,-0.076999996297061112926485293428413569927215576171875f,0.1833000244021416591522211092524230480194091796875f,-0.8409999918341635183338667047792114317417144775390625f,-0.93849999630451197418778974679298698902130126953125f,-0.1239999969899654441718439557007513940334320068359375f,0.598399972243607525257402812712825834751129150390625f,0.4770999818384649149294318704050965607166290283203125f,0.7516999682217839762188305030576884746551513671875f,0.0147999647676950729646705440245568752288818359375f,0.07640002221465136411637786295614205300807952880859375f,-0.55119998601376984925792612557415850460529327392578125f,-0.36960002967119243155735830441699363291263580322265625f,-0.093600019396841815932930330745875835418701171875f,0.0351000094205142776360162315540947020053863525390625f,-0.93089999377578502848251673640334047377109527587890625f,0.16270001928806276936256836052052676677703857421875f,-0.2262999934971332560706969161401502788066864013671875f,-0.1339000098466869292934688928653486073017120361328125f,0.217399998290836504732936873551807366311550140380859375f,-0.7557999743342396214984546531923115253448486328125f,-0.728999999746680504841833680984564125537872314453125f,-0.377499979250134121144810706027783453464508056640625f,-0.5430999942362306764920276691555045545101165771484375f,-0.40159999352097397462557637481950223445892333984375f,0.17569999069273467551255407670396380126476287841796875f,0.743700007992982303761664297780953347682952880859375f,1.090699996131657911746515310369431972503662109375f,-0.7216000066339969265527543029747903347015380859375f,0.89370000903606428011016760137863457202911376953125f,1.223599978381395114723773076548241078853607177734375f,0.961900031840801972293775179423391819000244140625f,-0.37589999828338616083556189551018178462982177734375f,-1.169999983310699320782077847979962825775146484375f,0.31790000147223462789725090260617434978485107421875f,-0.029600018793344506917719627381302416324615478515625f,0.181600019150972180614900253203813917934894561767578125f,-0.619300019253790079432064885622821748256683349609375f,-0.0929999921545385799248606417677365243434906005859375f,0.2653000216864047633436030082521028816699981689453125f,0.7591999914117157910453670410788618028163909912109375f,-0.2130000050626694729771770653314888477325439453125f,-0.36079997493028681532223345129750669002532958984375f,0.60349997231364316263579894439317286014556884765625f,-0.99979998562932070882425250601954758167266845703125f,-1.085899983233213017541629596962593495845794677734375f,0.26659997712969829564144674804992973804473876953125f,-2.19620001335740067105462003382854163646697998046875f,0.27370000307559916308264291728846728801727294921875f,0.6881000083923343169090003357268869876861572265625f,-0.433800025981664560958961374126374721527099609375f,0.665199982929229438610718716518022119998931884765625f,-0.403300013487041031368107724119909107685089111328125f,0.545200060415268250579856612603180110454559326171875f,-0.188500019587576839175113718738430179655551910400390625f,0.647700017648935588709946387098170816898345947265625f,-0.0228999910041689513917617659899406135082244873046875f,0.1317999511182303962186779244802892208099365234375f,-0.5583000209376216549372884401236660778522491455078125f,-0.695999998241662698461595937260426580905914306640625f,0.11920002492070247424038598182960413396358489990234375f,-0.7565000481382018193698968389071524143218994140625f,0.15619999502897241594467914183042012155055999755859375f,-1.30240005029588967744302863138727843761444091796875f,-1.44000001847744041327814557007513940334320068359375f,-0.65710000080019170809464412741363048553466796875f,-0.468400030320883065115822319057770073413848876953125f,0.3209000328168276627849309079465456306934356689453125f,1.0226000057697302025871977093629539012908935546875f,-0.2364000630810865200004400321631692349910736083984375f,-2.34730002519190339427268554572947323322296142578125f,-0.59419996639639105495689364033751189708709716796875f,0.03890003815889409022332756649120710790157318115234375f,0.12349996907263882661709430976770818233489990234375f,-0.356600022205710132094935715940664522349834442138671875f,0.6851999802470203349002986215054988861083984375f,-0.243999982535839432529201076249592006206512451171875f,0.8447000415503975734310415646177716553211212158203125f,-1.042699994254112194624894982553087174892425537109375f,0.393800001394748466765349803608842194080352783203125f,1.40419999899268166387855671928264200687408447265625f,-0.346500019937753478416198049671947956085205078125f,-1.08329998096525681461343992850743234157562255859375f,-0.11819999680221038307337266815011389553546905517578125f,0.9304000108972190208334041017224080860614776611328125f,-0.8590000020191068674790813020081259310245513916015625f,-0.92980000049322786281891239923425018787384033203125f,1.529900034761428884166889474727213382720947265625f,-0.2334000078573825742722647191840223968029022216796875f,0.448199989873170778764688293449580669403076171875f,-0.191600003728270407155065413462580181658267974853515625f,1.839700031417608183659240239649079740047454833984375f,-0.4883000347211960967541699574212543666362762451171875f,0.72670002208948147881528711877763271331787109375f,0.0419999675750728496126384925446473062038421630859375f,-0.113599957370758009034261704073287546634674072265625f,-0.191299989436566786338289603008888661861419677734375f,-0.2807000079035759654288995079696178436279296875f,0.353500011838972783007051248205243609845638275146484375f,-0.1966999143168328689768031836138106882572174072265625f,0.946200009930134200430984492413699626922607421875f,1.2755000128298998873788150376640260219573974609375f,0.51989999125003816260459643672220408916473388671875f,-0.3826999984264374266018648995668627321720123291015625f,-0.85300002944469444088326781638897955417633056640625f,-1.762000016637145893838578558643348515033721923828125f,-0.71700004744529710620781770558096468448638916015625f,0.3259000391870738866373358177952468395233154296875f,0.9807000111073254089433248736895620822906494140625f,1.0442999523475762213564621561090461909770965576171875f,-0.0310000093989077907252749355393461883068084716796875f,1.35750000417232552507584841805510222911834716796875f,-0.3450999916136259226817628587014041841030120849609375f,-0.9293000269651414679827894360641948878765106201171875f,0.523000000983476542870675984886474907398223876953125f,1.29140000254511821964342743740417063236236572265625f,0.65409998204708141855689973453991115093231201171875f,1.2235999865770335048864581040106713771820068359375f,-1.45999998256564111898114788345992565155029296875f,0.28600000032782535441810978227294981479644775390625f,0.1464000338375563980974902733578346669673919677734375f,-1.489699968460202139652892583399079740047454833984375f,0.56019999628439476335728386402479372918605804443359375f,0.361799991798400721876305397017858922481536865234375f,-0.45080000644624218519851410746923647820949554443359375f,0.0739000096976756282174392254091799259185791015625f,-0.2978999968156219591008948555099777877330780029296875f,0.41519997667074193259395542554557323455810546875f,-0.540900012513995331886462736292742192745208740234375f,1.5991000412046911804964111070148646831512451171875f,-0.83450000810623148339573162957094609737396240234375f,0.5604000066876413388428090911475010216236114501953125f,-0.30550000318139802768513391129090450704097747802734375f,0.52810001703500741854213629267178475856781005859375f,-1.20040004848539894055647891946136951446533203125f,0.74360003027468890568485448966384865343570709228515625f,-0.393999984920025081436278924229554831981658935546875f,0.389000010922550654157703320379368960857391357421875f,0.25720001055598284978032097569666802883148193359375f,0.305599975274503332656195198069326579570770263671875f,-0.7196000117897991543713942519389092922210693359375f,0.4837000366404655693486347445286810398101806640625f,-0.847499961107969657092553461552597582340240478515625f,-0.8067999847576017291572725298465229570865631103515625f,-0.4264999628663062036793007791857235133647918701171875f,0.566100012691319154356506260228343307971954345703125f,-1.74390000865459438017524007591418921947479248046875f,0.02029998407959965334157459437847137451171875f,-0.712099961572886019922634659451432526111602783203125f,-0.579699988800287169254943364649079740047454833984375f,0.6647999971032145793969903024844825267791748046875f,-0.47950000527501135394459197414107620716094970703125f,-0.9723999720275404712310773902572691440582275390625f,0.73890001074075684073250158689916133880615234375f,0.372799998533725585048159700818359851837158203125f,-0.08189999712109585772168429684825241565704345703125f,-0.09679998896718000622740873950533568859100341796875f,-0.159300005358458118820408344618044793605804443359375f,-0.199699999976157283043676216038875281810760498046875f,-0.758500022232532433719143227790482342243194580078125f,0.25530002729296719099494339388911612331867218017578125f,-0.58560000809431045354358502663671970367431640625f,-0.14259997715949968011273085721768438816070556640625f,-0.40300006133317989309716722345910966396331787109375f,0.5238999826893209199596412872779183089733123779296875f,0.670200039106607192707087961025536060333251953125f,0.5965000108107922915934295815532095730304718017578125f,-0.817399976944923167820888920687139034271240234375f,0.246099996784329366761312485323287546634674072265625f,-0.784400005912780695638275574310682713985443115234375f,0.052399997173249779347514731853152625262737274169921875f,0.30479996964782485324718663832754828035831451416015625f,-0.048799977552891338206109139719046652317047119140625f,1.0386000451013448486747847709921188652515411376953125f,0.78600007021427220621490050689317286014556884765625f,0.8615000077188010862982991966418921947479248046875f,0.07929995454549843003633213811554014682769775390625f,-0.2039999722167851192722309860982932150363922119140625f,1.702299979296326259969873717636801302433013916015625f,0.9339999640956524462609422698733396828174591064453125f,0.26549999539554125504992043715901672840118408203125f,-0.605500009216367818254411758971400558948516845703125f,-0.6578999704778194512755362666212022304534912109375f,-1.149099992030858885527777601964771747589111328125f,-0.921499998480082016527603627764619886875152587890625f,-0.659199984240531389190209665684960782527923583984375f,0.6181999967277054253145252005197107791900634765625f,-1.459200046676397022338278475217521190643310546875f,-0.206900010383128662994067781255580484867095947265625f,1.386599996501207243682074476964771747589111328125f,-0.1878999986410143652193482921575196087360382080078125f,-0.843599959531426524250719012343324720859527587890625f,0.42919997164905099307219416004954837262630462646484375f,-0.1255000023990870960943766476702876389026641845703125f,0.139500014662742533033679137588478624820709228515625f,0.66750001445412632250508977449499070644378662109375f,0.7658000150889154866717944969423115253448486328125f,-1.24130001718997906579033951857127249240875244140625f,0.18470002158284248849895448074676096439361572265625f,-0.0208000077128414062599404132924973964691162109375f,0.23169995767921125473520760351675562560558319091796875f,0.68580003587603588499632678576745092868804931640625f,0.247600002454221534975431495695374906063079833984375f,-1.3282999884903432530336431227624416351318359375f,0.15480000615566924526689263075240887701511383056640625f,0.841099981659650719478804603568278253078460693359375f,-0.6670999975219362720935123434173874557018280029296875f,-1.61930004335641886115126908407546579837799072265625f,0.36759996747374490411175429471768438816070556640625f,0.1921999920040378118102353255380876362323760986328125f,0.95960005768537559589503871393389999866485595703125f,0.1783000100597735837482105125673115253448486328125f,-0.25320007913112618780360207892954349517822265625f,-0.703599982963501791033422705368138849735260009765625f,1.0569000069558622811882742098532617092132568359375f,1.260200012582540551875354140065610408782958984375f,1.55399996420741093317019476671703159809112548828125f,1.435199957671760895294710280722938477993011474609375f,0.183800015252828519152217268128879368305206298828125f,1.50990004012584666526208820869214832782745361328125f,-0.231900040185451405960748161305673420429229736328125f,-0.15709998332858088332386614638380706310272216796875f,0.63569999061822901609275504597462713718414306640625f,-0.204200025516748429055269298260100185871124267578125f,-0.534400020068883208068655221723020076751708984375f,1.535899984723329225744237191975116729736328125f,-0.774699981647729796208068364649079740047454833984375f,-1.686299992454051821511029629618860781192779541015625f,-0.127799974393844717468482485855929553508758544921875f,-0.3647999950170515148784033954143524169921875f,-0.7283000002622603830104708322323858737945556640625f,-0.50059999345391992253695434555993415415287017822265625f,0.5388000151410696769715968912350945174694061279296875f,-0.2027999964475630800819772048271261155605316162109375f,0.230499995395540946407919591365498490631580352783203125f,0.271499978959560461788669272209517657756805419921875f,-0.640400015106796782760056885308586061000823974609375f,-0.855899990832805546148165376507677137851715087890625f,-0.00400000183284276999984285794198513031005859375f,0.14869999189972826769690072978846728801727294921875f,0.491900012022256749588677848805673420429229736328125f,0.3602000009596351137730607661069370806217193603515625f,-0.1364000123798845276468227893928997218608856201171875f,-0.775699985514580969692133294302038848400115966796875f,0.135600003474950714377200711169280111789703369140625f,0.3428999801635743693850599811412394046783447265625f,0.2256999764621261395092233215109445154666900634765625f,-0.6923999983653421619322898550308309495449066162109375f,-1.69759995972514143858234092476777732372283935546875f,-0.83739995362460550865080222138203680515289306640625f,-0.7940999921426172836191881287959404289722442626953125f,-0.8443000466346755050750516602420248091220855712890625f,0.199499954871833029557137706433422863483428955078125f,0.17010002219826059199903056651237420737743377685546875f,0.131900015002489379156713766860775649547576904296875f,-0.58219999427646396128466221853159368038177490234375f,0.42449998530745514102591187111102044582366943359375f,0.1817000217914579440758870987338013947010040283203125f,0.648099959069491138308194422279484570026397705078125f,0.81279996515512475951936721685342490673065185546875f,-0.222299997401237181549049637396819889545440673828125f,-0.5311000247240063032450052560307085514068603515625f,0.91970002389252158980781359787215478718280792236328125f,0.6285000041648747259870333436992950737476348876953125f,-0.4068000302061438588907549274154007434844970703125f,0.9429000071719289000071739792474545538425445556640625f,-0.762699969667196331357672534068115055561065673828125f,-1.175399988806247630890311484108678996562957763671875f,0.81330003248602189547256102741812355816364288330078125f,-0.1402999982357027164425744558684527873992919921875f,-0.6558000042483211888821870161336846649646759033203125f,-0.150500016927718860415552626363933086395263671875f,-0.612000019319355548219618867733515799045562744140625f,0.14970000712871556203253931016661226749420166015625f,0.0755000128671527992452183752902783453464508056640625f,-0.086900008744001500105014201835729181766510009765625f,0.1187999952480196796944511561378021724522113800048828125f,-0.627899972601234868108122100238688290119171142578125f,0.48320000626444770119860550039447844028472900390625f,-0.0528000041589142821152336182422004640102386474609375f,-0.182999982208013278750513563863933086395263671875f,-0.644699999044835436734501854516565799713134765625f,-0.5346999809399244352192681617452763020992279052734375f,-0.476600000971555692874659371227608062326908111572265625f,0.29710001104474048627679394485312514007091522216796875f,0.29700000065565124174327138462103903293609619140625f,0.25509995797425510932754377790843136608600616455078125f,0.90719999472349888147704177754349075257778167724609375f,-0.355200056764483351745553818545886315405368804931640625f,0.239399991868436357034255479447892867028713226318359375f,0.140600017835945279021103715422214008867740631103515625f,-0.2291000107876961511266244997386820614337921142578125f,-0.814400024390220522718664142303168773651123046875f,-0.4516000327110294421117941965349018573760986328125f,-0.27330000525414932521783839547424577176570892333984375f,-0.082700016289205091624125998350791633129119873046875f,0.5620999811306590654425008324324153363704681396484375f,0.89480002718493290014833974055363796651363372802734375f,0.128800026577711168584983170148916542530059814453125f,-1.3470999617218968325005334918387234210968017578125f,-0.21499999731779073641746435896493494510650634765625f,-0.92129999036788934319019972463138401508331298828125f,-1.654400041824579670191042168880812823772430419921875f,0.761299972095340837796584310126490890979766845703125f,-0.267800018613040702319949559750966727733612060546875f,0.36460000270009051970276914289570413529872894287109375f,-0.56730001714527611422766995019628666341304779052734375f,0.9445000491403046094518458630773238837718963623046875f,-0.52450002211332336088389638462103903293609619140625f,0.13200001859292409545076907306793145835399627685546875f,0.695100007744133829845623040455393493175506591796875f,-1.85950003269314745324436444207094609737396240234375f,1.462299999748169820890097980736754834651947021484375f,0.055499987572431574989195723901502788066864013671875f,-0.35950005202740442644682161699165590107440948486328125f,-0.36160001134648922516134916804730892181396484375f,0.273200038823486313077637532842345535755157470703125f,-0.481900011686980656033796321935369633138179779052734375f,1.1334000474758447740697420158539898693561553955078125f,-0.130599996452778588373888624118990264832973480224609375f,-0.3096000013589861321605667399126105010509490966796875f,-0.0718999802269042476154936593957245349884033203125f,-1.690800006073713834808813771815039217472076416015625f,-0.2622999911427499331040280594606883823871612548828125f,-0.649199984836578369851167735760100185871124267578125f,-0.28849998602271054171097830476355738937854766845703125f,0.47240003803968466655760494177229702472686767578125f,-0.711399997329711819560316143906675279140472412109375f,-0.7747999556779860963473538504331372678279876708984375f,-0.835699969905615347443017526529729366302490234375f,-1.000800002646446529297463712282478809356689453125f,-0.4371000025510785036431116168387234210968017578125f,-0.74729998294711119655175934894941747188568115234375f,0.6090000427365307444915742962621152400970458984375f,0.07309999274611467257045660517178475856781005859375f,-0.4615999823451044647981689195148646831512451171875f,-0.219399996376037353940091634285636246204376220703125f,-0.699799976837635284709904226474463939666748046875f,0.173000003218650721947824422386474907398223876953125f,0.270400023078918305685647283098660409450531005859375f,0.76059997631758502478760419762693345546722412109375f,1.01189998177289997016714551136828958988189697265625f,0.7574999773502344435627264829236082732677459716796875f,1.89149996480345716776128028868697583675384521484375f,0.1056000041827565016916423701331950724124908447265625f,1.367999986827373337661128971376456320285797119140625f,1.1736999894037840608262968089547939598560333251953125f,-0.82240005130171756064783039619214832782745361328125f,-0.055499997854232763216941748396493494510650634765625f,-1.154999975264072276104343472979962825775146484375f,1.068999985925853213331038205069489777088165283203125f,-0.1581999902829529514036721593583934009075164794921875f,0.164100015127658149793887787382118403911590576171875f,-0.24979998406469838823085183321381919085979461669921875f,0.64479999963641176918827113695442676544189453125f,-0.332599973993003228400766602135263383388519287109375f,-0.18460002356171667514672662946395576000213623046875f,0.2312000098183748519886648864485323429107666015625f,-1.642799990934133091968760709278285503387451171875f,-0.755499997071921836067076583276502788066864013671875f,-1.03749996095895813397191886906512081623077392578125f,-0.199699988874793010751318433904089033603668212890625f,-0.186199976700544223007227628841064870357513427734375f,0.072399979366362166022241808605031110346317291259765625f,0.82449998156353798695050727474153973162174224853515625f,-0.6832000022225084290283803056809119880199432373046875f,-0.5263999489009376642201232243678532540798187255859375f,-1.2243999954819686504237097324221394956111907958984375f,0.289099981525540616900116219767369329929351806640625f,0.59499997176229957318582819425500929355621337890625f,-0.09960003771781966719345291494391858577728271484375f,-0.38209992506504075748807736090384423732757568359375f,-0.199399955844878906674466634285636246204376220703125f,-1.1864999801516535882228708942420780658721923828125f,-1.635400012648105860790792576153762638568878173828125f,-0.7829999823197721919854075167677365243434906005859375f,-0.495700008108466760869958989133010618388652801513671875f,-1.773200006357580349458658020012080669403076171875f,-0.17140001834034845895615717381588183343410491943359375f,-0.9068000095307822139290010454715229570865631103515625f,1.3781999977335328555483329182607121765613555908203125f,1.47499998010694977068624211824499070644378662109375f,1.255499981611966742178765343851409852504730224609375f,-0.2104999835491181414681705064140260219573974609375f,0.2925999981701379315524036428541876375675201416015625f,-0.51250000417232488558738623396493494510650634765625f,0.4219999688044191454849851652397774159908294677734375f,-0.49190000857636329012478881850256584584712982177734375f,-0.8570999948024742298713363197748549282550811767578125f,-0.6528000097654762523546878583147190511226654052734375f,-0.1913999768406160484346401062794029712677001953125f,0.1698000000461934444473399707931093871593475341796875f,-0.005500036448239598030340857803821563720703125f,0.5389999980702999682335985198733396828174591064453125f,-1.3258999710142607142415727139450609683990478515625f,-0.95579999173134577716837156913243234157562255859375f,-0.818399993366003997152802185155451297760009765625f,-0.25189998881369835004306878545321524143218994140625f,-0.441400000607966802590453880839049816131591796875f,-0.79609999000430065052569261752068996429443359375f,0.591200022298097760398150057881139218807220458984375f,-0.35280002788901310140090572531335055828094482421875f,-1.7869000267744066245967360373470000922679901123046875f,0.026000004332512371973962217452935874462127685546875f,0.1947000136479737353312202685629017651081085205078125f,-0.00380001597926005185712483580573461949825286865234375f,-0.191600036734342893396387808024883270263671875f,2.228600015783310794859062298201024532318115234375f,0.57670000867843640435239649377763271331787109375f,-0.22600000464916103481982645462267100811004638671875f,1.211699956971406866301776972250081598758697509765625f,-0.95869999070763611825896077789366245269775390625f,0.3274000151664020474839844609959982335567474365234375f,0.771199987280368759456905536353588104248046875f,-0.90780003343969595253071247498155571520328521728515625f,-0.2072000113569198997964804220828227698802947998046875f,0.523800004300474686402822044328786432743072509765625f,1.52540003136768920199983767815865576267242431640625f,-0.79569996975660384208595132804475724697113037109375f,1.148200037556886154987978443386964499950408935546875f,-0.48799994026124460244631109162583015859127044677734375f,-1.927399994081259393396976520307362079620361328125f,1.2043999930232753836634174149367026984691619873046875f,-0.27750000074505809521951960050500929355621337890625f,0.4859999720156193969700098023167811334133148193359375f,-0.211200004528462936814747763492050580680370330810546875f,-0.271199956155568078219886274382588453590869903564453125f,0.058300001286715252835080036675208248198032379150390625f,1.6506999971747393640697509908932261168956756591796875f,0.66060002115145355361391921178437769412994384765625f,0.24369998146891536094926777877844870090484619140625f,0.41590001098662643386205672868527472019195556640625f,-0.68030003340244338261300072190351784229278564453125f,0.0880999799311161790882351851905696094036102294921875f,-0.152000038802622583489210228435695171356201171875f,-0.716600007006526151798198043252341449260711669921875f,0.85129999364614494794523125165142118930816650390625f,1.446699973958730733869515461265109479427337646484375f,0.4561000107914205958792308592819608747959136962890625f,-1.6404999762475487301571774878539144992828369140625f,-0.731699988263844858238371671177446842193603515625f,0.5466999758958817068332791677676141262054443359375f,-0.243600033515691816177195505588315427303314208984375f,-1.383899969017505782176158390939235687255859375f,-0.8831000164389610773696404066868126392364501953125f,0.257900033807754169856707449071109294891357421875f,0.56570003517270084358870008145458996295928955078125f,-0.944800017741322495368194722686894237995147705078125f,-0.300699978734553174941623865379369817674160003662109375f,0.896500025749206397307489169179461896419525146484375f,0.9436999933525920258858832312398590147495269775390625f,0.099300005656480561100352133507840335369110107421875f,-0.0396999797105788143625204611453227698802947998046875f,0.95369998728036886603831590036861598491668701171875f,1.357199981647729369882426908588968217372894287109375f,-0.65669998736977586162311126827262341976165771484375f,-0.3150000231340526202217233731062151491641998291015625f,0.229900020456314191363844656734727323055267333984375f,1.6474999867752198046133571551763452589511871337890625f,0.145599994515627606528340720615233294665813446044921875f,0.8064999805241821473344998594257049262523651123046875f,0.41839999925196191821186175729962997138500213623046875f,0.325899987107515443085503648035228252410888671875f,-0.5247999880880118706016901342081837356090545654296875f,-0.53639999762773538094506875495426356792449951171875f,-0.0295000057965515427582658958272077143192291259765625f,-1.162799968433379405041705467738211154937744140625f,-0.7304999952465294210668389496277086436748504638671875f,-0.334099972361326269520986897987313568592071533203125f,0.17450000680237998107457997321034781634807586669921875f,-0.0776000170946122835857750033028423786163330078125f,-1.16709998899102185276888121734373271465301513671875f,-0.381999998942017526815106975845992565155029296875f,0.5702999715626244636723640724085271358489990234375f,0.269100009688734598256587560172192752361297607421875f,-0.9100999872922894695648210472427308559417724609375f,-1.43929996959567052527972919051535427570343017578125f,0.309000006675720140236762745189480483531951904296875f,-0.4928999952137467222001987465773709118366241455078125f,0.23759997239112795597293370519764721393585205078125f,-0.95529999130666265472200393560342490673065185546875f,0.452200027692317263472432387061417102813720703125f,-0.43410000015199157186174261369160376489162445068359375f,-0.37770004253387468029501405908376909792423248291015625f,-0.73579997764974847118679690538556315004825592041015625f,0.49129999394416790092776636811322532594203948974609375f,0.411699981260299718854867023765109479427337646484375f,0.736499995797872397673700106679461896419525146484375f,0.1247999852940446796623064074083231389522552490234375f,1.656799956929683848017020864062942564487457275390625f,0.4990999782472849144454585257335565984249114990234375f,1.623999981641769618789794549229554831981658935546875f,-0.0530000442713494113178285260801203548908233642578125f,1.593200006264448465032046442502178251743316650390625f,0.5943999926507472775227824968169443309307098388671875f,-0.6646000032216310415833504521287977695465087890625f,0.4977999987572430118376587415696121752262115478515625f,-0.08160000365376463005162577246665023267269134521484375f,0.890799973738193529015916283242404460906982421875f,0.0639000181540845257899263742729090154170989990234375f,0.50939999965429283435014440328814089298248291015625f,0.2427999984964726021274827871820889413356781005859375f,0.8729000109344726698878957904526032507419586181640625f,0.1543999945506453475019270626944489777088165283203125f,0.783500001817941438275738619267940521240234375f,0.6329000059425828883519216105923987925052642822265625f,0.44719999643713304049441603638115338981151580810546875f,-1.217099951067567520368584155221469700336456298828125f,0.30229998488426179648058678139932453632354736328125f,-0.50300000202655770209503316436894237995147705078125f,-0.539499975025653721871776724583469331264495849609375f,0.115299986258893927271174106863327324390411376953125f,1.2801999811783433269596343961893580853939056396484375f,0.62529997385367852569260094242054037749767303466796875f,0.84359997346401183193620454403571784496307373046875f,-1.39439997372627200178385464823804795742034912109375f,-0.3157999893099070476409906405024230480194091796875f,0.068100035437941297544739427394233644008636474609375f,0.3649999993667005782072010333649814128875732421875f,-0.1274000110685822306777481571771204471588134765625f,0.82579999981522522745081005268730223178863525390625f,0.1146000225186352139417067519389092922210693359375f,0.117799992666393460449825170144322328269481658935546875f,0.2745000047534704723517506863572634756565093994140625f,0.02289999856650803966573448633425869047641754150390625f,0.84500001136213553909470874714315868914127349853515625f,-0.5832999902412296933107427321374416351318359375f,-0.3193999957427384739361286847270093858242034912109375f,-0.18649998890608543433700106106698513031005859375f,0.120100014524161247120304096824838779866695404052734375f,-0.4872999933220445445414270579931326210498809814453125f,-0.90870000754594826286592024189303629100322723388671875f,0.26610001526176951802682424386148341000080108642578125f,-1.153800020617246513410236730123870074748992919921875f,0.50110003120601209758433469687588512897491455078125f,-1.3594000502623619919262409894145093858242034912109375f,-0.132499967068432977779224302139482460916042327880859375f,-0.1193000104062260646742288372479379177093505859375f,-0.484700033093989457455563751864247024059295654296875f,2.030900031363964242103747892542742192745208740234375f,-0.2465000091716647023787345460732467472553253173828125f,0.082500014752149741781295233522541821002960205078125f,0.90170002749115252260736497191828675568103790283203125f,-0.9421000320181256437734873543377034366130828857421875f,-0.6559000096827747317007606397964991629123687744140625f,-0.0376999992087483359881616706843487918376922607421875f,0.346400028584897656802610299564548768103122711181640625f,0.51990000188574148065612234859145246446132659912109375f,0.15970008107572797317885715528973378241062164306640625f,0.49570002064406881725489029122400097548961639404296875f,-1.0536000215947627367540917475707828998565673828125f,-1.389699986974894496682964017963968217372894287109375f,-1.148800003331900132508280876209028065204620361328125f,-1.69430001374036098837905228720046579837799072265625f,0.95360000803470601482558777206577360630035400390625f,-0.04000000929459901488627338039805181324481964111328125f,-0.2124000020533802768341047340072691440582275390625f,-0.39809998402893487234877056835102848708629608154296875f,0.139699990756064540420311459456570446491241455078125f,-0.97760000843331151809678658537450246512889862060546875f,0.48529999784454691535984238726086914539337158203125f,-0.70340000757798548658428217095206491649150848388671875f,0.47629998202323964306970083271153271198272705078125f,2.13979997594356508017199303139932453632354736328125f,0.319500001847743408944779730518348515033721923828125f,1.06880001152753845161669232766143977642059326171875f,-0.471100053483247460661687000538222491741180419921875f,-0.350300028932094331679536480805836617946624755859375f,0.4398999918460846370038552777259610593318939208984375f,-0.87689999324679401837556724785827100276947021484375f,0.5888000131294133243642363595427013933658599853515625f,-1.9524999581277366900167180574499070644378662109375f,0.6205000000521534442299298461875878274440765380859375f,-0.4571000167071819220154793583787977695465087890625f,-0.86090002339184279467332316926331259310245513916015625f,0.22890003368854561216494403197430074214935302734375f,-0.1287999952480196608206597375101409852504730224609375f,1.123199982720613387954244899447076022624969482421875f,-0.3785000102370974506271750215091742575168609619140625f,-0.1122000113010399591217947090626694262027740478515625f,-0.511399995690583697438569288351573050022125244140625f,-1.141200008924305731028425725526176393032073974609375f,-0.5356999782502658291605257545597851276397705078125f,-0.9652000304639332117773165009566582739353179931640625f,0.5047999912917617848506779409945011138916015625f,-0.2537000394716855122823062629322521388530731201171875f,-2.116299985450506682127524982206523418426513671875f,-1.5186999753221861464425046506221406161785125732421875f,-0.53379998738765710442066847463138401508331298828125f,-0.478099983656406823939732930739410221576690673828125f,-1.364399985498189860066986511810682713985443115234375f,-0.06099999392032628975357511080801486968994140625f,0.6789999800026420562204521047533489763736724853515625f,0.571499988198280295392805783194489777088165283203125f,0.55169997723698571689965319819748401641845703125f,-0.21109993710517915843638547812588512897491455078125f,0.89309995862245639131060670479200780391693115234375f,0.378799980682134052045739736058749258518218994140625f,-0.066499996095895852477042353712022304534912109375f,0.62200000005960465188081798260100185871124267578125f,0.22949999633431428236463034409098327159881591796875f,1.1870999703645708223120891489088535308837890625f,0.1681999941945075960347821819595992565155029296875f,-0.1515000106990338490930980697157792747020721435546875f,-0.6566000045105815363655210603610612452030181884765625f,0.76629999379515678636920483768335543572902679443359375f,0.359199987667798925627948847250081598758697509765625f,1.23640000880360556578807518235407769680023193359375f,0.1231000149860979053784149073180742561817169189453125f,-0.012500014752149457564200929482467472553253173828125f,-0.7848000080183143989387417605030350387096405029296875f,-0.6833000320017337259059786447323858737945556640625f,0.1735999967843293578795282883220352232456207275390625f,0.81409999426603274486069494741968810558319091796875f,0.2900999770849941317152342890040017664432525634765625f,0.5109999872893091765746476085041649639606475830078125f,-1.046099985776096641387766794650815427303314208984375f,-0.0492999653860917153025411607814021408557891845703125f,0.300299970296026064175975989201106131076812744140625f,-0.3003999826967724828818973037414252758026123046875f,0.55849997730553102570638657198287546634674072265625f,-1.0008000684723261652919745756662450730800628662109375f,-2.3855999656260014063491325941868126392364501953125f,-1.57509995748996711739664533524774014949798583984375f,-0.4724000027239316779059663531370460987091064453125f,-1.1150000280141838526759556771139614284038543701171875f,-0.585800055992603319054978783242404460906982421875f,-0.83569999726787214111567436702898703515529632568359375f,0.11659998646155045332051258810679428279399871826171875f,-0.00300001355633139343837001433712430298328399658203125f,0.223699996221065955381845924421213567256927490234375f,1.562099968054890819502134036156348884105682373046875f,0.92960000053942126729822348352172411978244781494140625f,0.524100022652745689555331409792415797710418701171875f,-0.090400010710954603609934565611183643341064453125f,0.06019996191859267042900683009065687656402587890625f,1.182499991953373186248654747032560408115386962890625f,1.51109998106360432501560353557579219341278076171875f,-0.78490001449584934078984588268212974071502685546875f,0.3455000139474868614541946953977458178997039794921875f,0.10939999749362427916565820851246826350688934326171875f,-0.738400012588500853638606713502667844295501708984375f,0.587000014998019015166619283263571560382843017578125f,0.0289000179119408839856220083674998022615909576416015625f,0.5567000172838569849176337811513803899288177490234375f,-0.65409999875500801636007963679730892181396484375f,0.1656000282481315810656496978481300175189971923828125f,-0.83580000652074826206217039725743234157562255859375f,-0.52240003450065930490353593995678238570690155029296875f,1.71030001462697978098503881483338773250579833984375f,-0.2431999802246689323936834625783376395702362060546875f,-0.235099985951185086463510742760263383388519287109375f,-0.0707000101387500468064217784558422863483428955078125f,-0.539200062993168582892167250975035130977630615234375f,0.141199990670382857782527707968256436288356781005859375f,0.68940004955455680413223262803512625396251678466796875f,1.354300010238587592681369642377831041812896728515625f,0.01939996776580754644925264074117876589298248291015625f,0.9583000457108017400287280906923115253448486328125f,1.47489998245835263190883779316209256649017333984375f,0.00199994671344772001475575962103903293609619140625f,0.46749998845159990157327456472557969391345977783203125f,0.278799996030329566565342247486114501953125f,-0.02249999776482614066708265454508364200592041015625f,1.077299985629320389080021413974463939666748046875f,-0.3780999808251859217733681362005881965160369873046875f,-0.8824000089265406909788680422934703528881072998046875f,0.56840000295862569412719267347711138427257537841796875f,0.3113999984659248099205797188915312290191650390625f,-0.2848999730147421249881745097809471189975738525390625f,-0.54459997259974546324201583047397434711456298828125f,-1.467799976778030757174065001890994608402252197265625f,-0.593999973893165389426940237171947956085205078125f,0.54519999857544831911582150496542453765869140625f,1.237099970662593140247054179781116545200347900390625f,0.8668000299267470953878955697291530668735504150390625f,-0.0678999823242423250491128783323802053928375244140625f,-0.587600013667345155710108883795328438282012939453125f,-0.07270000967681411463416907281498424708843231201171875f,-0.4640000043660401640721602234407328069210052490234375f,0.4254000112324949345321556393173523247241973876953125f,-0.44620002423524862678050340036861598491668701171875f,-0.4992000010043378122048807199462316930294036865234375f,0.172499977499246615053607456502504646778106689453125f,0.41010001847296972954381999443285167217254638671875f,0.1641999702334402400083490647375583648681640625f,-0.33530002386569968120255680332775227725505828857421875f,0.014400039663911012866037708590738475322723388671875f,1.014400031617283648444072241545654833316802978515625f,1.0125999743655318408031007493264041841030120849609375f,-0.918800002512335733939607962383888661861419677734375f,-1.443600017459690132426430864143185317516326904296875f,0.02289998463392262095794649212621152400970458984375f,-0.2546000702023512030081064949627034366130828857421875f,-1.9749000031709673663726789527572691440582275390625f,-1.928399985617399448045716781052760779857635498046875f,0.315200006920099529139633887098170816898345947265625f,0.1700000227615234837941216028411872684955596923828125f,-1.0759000006675716321069558034650981426239013671875f,0.863599994977563678588694529025815427303314208984375f,-0.105699971768260514437542951782234013080596923828125f,-1.46819998473227020241438367520458996295928955078125f,0.946299969431758203342042179428972303867340087890625f,0.38500000752508611867597210220992565155029296875f,0.76190000278353675611242579179815948009490966796875f,0.28110000743865981576163903810083866119384765625f,0.23759999653100916106041040620766580104827880859375f,1.175399988657236605149591923691332340240478515625f,0.0261999804258348234498043893836438655853271484375f,0.55069996722340608386048188549466431140899658203125f,0.828599991792440704330147127620875835418701171875f,-0.29469996585249891296598434564657509326934814453125f,0.228300001901387883407323897699825465679168701171875f,-0.9450999824494121792639589330065064132213592529296875f,0.41340000618100158913392760950955562293529510498046875f,-0.3825999903470276297667851395090110599994659423828125f,0.836099988327919874109284137375652790069580078125f,-1.11629998694062226860523878713138401508331298828125f,0.4516000063359737470847221629810519516468048095703125f,-1.302300011259317269463053889921866357326507568359375f,0.9372000205770139036332011528429575264453887939453125f,-0.5115000255256896277700207065208815038204193115234375f,0.12879998973458983702045088648446835577487945556640625f,-1.1569999882876869090608806800446473062038421630859375f,0.19810000228285762347013587714172899723052978515625f,0.85849998114258074810578591495868749916553497314453125f,0.401600004026293699865846065222285687923431396484375f,-0.6176000187337391533759500816813670098781585693359375f,0.477399965322018005053905653767287731170654296875f,-0.06610001656562047855913988314568996429443359375f,0.313199986371397898210489074699580669403076171875f,-0.2257000178873538320800662404508329927921295166015625f,1.324399993917346751715058417175896465778350830078125f,0.240500004187225613350165076553821563720703125f,-0.0219000275567176050373063844745047390460968017578125f,0.88920002469718439908064055998693220317363739013671875f,-0.4274999882280827723235461235162802040576934814453125f,-1.00159996565580389216165713150985538959503173828125f,1.0055999582499268729662844634731300175189971923828125f,0.244400018802284790808698744513094425201416015625f,0.052499971091747976714714241097681224346160888671875f,0.1425999965310094097503679222427308559417724609375f,0.1264999881237744983053516989457421004772186279296875f,-0.44889999323189211377638230260345153510570526123046875f,0.3783999761551621876520812293165363371372222900390625f,0.2911999715596440552189960726536810398101806640625f,-0.3414999993368983410846340120770037174224853515625f,0.18589998911917149371220148168504238128662109375f,-0.2072000030681493409900895130704157054424285888671875f,-0.36110001816749548453344687004573643207550048828125f,0.350500004895031580520736724793096072971820831298828125f,1.0796000276409085660844766607624478638172149658203125f,-0.8322000154361131052382916095666587352752685546875f,1.02390002677813196640954629401676356792449951171875f,-0.8112000120908022804400161476223729550838470458984375f,0.394599991524219984029286933946423232555389404296875f,-0.03599999116361118911555649901856668293476104736328125f,0.7026999995440241786326396322692744433879852294921875f,-0.6824000293970111652441801197710447013378143310546875f,-0.204199993404746038550001685507595539093017578125f,-1.433699966120720592499537815456278622150421142578125f,-0.64889999628663030950548318287474103271961212158203125f,-1.050399994841217932162180659361183643341064453125f,-0.670100008563697002728076768107712268829345703125f,0.12430001921653843766080171917565166950225830078125f,-0.4552999989062549435203663961146958172321319580078125f,-0.54499998107552638515471699065528810024261474609375f,-0.2796999967351554428063309387653134763240814208984375f,-1.5001000085264430961018433663411997258663177490234375f,-0.5249000142350792597056852173409424722194671630859375f,-0.73209995985925269135208282023086212575435638427734375f,0.38390003517866155835491781544988043606281280517578125f,0.35310001014322056622773970957496203482151031494140625f,-1.89780004530474588619881615159101784229278564453125f,-1.6433999962344765233268617521389387547969818115234375f,-0.62340000362917835641241026678471826016902923583984375f,-0.8720999898850916309811509563587605953216552734375f,1.097599977755546518665141775272786617279052734375f,-0.31820001624822591423935591592453420162200927734375f,1.72659994270801586679908723453991115093231201171875f,0.802299982574581793670631668646819889545440673828125f,-0.14630001383721802366011388585320673882961273193359375f,0.29330000871866968026324684615246951580047607421875f,-0.17409999456405655937629717300296761095523834228515625f,0.26639999129474156713826005216105841100215911865234375f,-0.9252000196792182773464219280867837369441986083984375f,-0.8052000352323054332970286850468255579471588134765625f,1.05760002642646444570573294186033308506011962890625f,-0.440300006803870047633608919568359851837158203125f,2.25460000582933428603382708388380706310272216796875f,-1.20039996608197707672616161289624869823455810546875f,0.81360001503825163382543905754573643207550048828125f,-0.2408000153496860917812227853573858737945556640625f,0.42790001629888985146266122683300636708736419677734375f,0.2445000214800236992829240989522077143192291259765625f,-0.60300001703947803122218829230405390262603759765625f,-0.202899976158886385935176122075063176453113555908203125f,-1.0073000091545283307681302176206372678279876708984375f,1.0528000548601148533833793408120982348918914794921875f,-0.00669998846873642861510234070010483264923095703125f,1.69499999210238438962505824747495353221893310546875f,0.7239999965801835468681701968307606875896453857421875f,1.0354999939799309771615298814140260219573974609375f,0.4171999762088063334886101074516773223876953125f,1.322799954463542260185704435571096837520599365234375f,0.7174999941512940626608951788512058556079864501953125f,-0.041699988561868739367355374270118772983551025390625f,1.4160999920904639992613738286308944225311279296875f,-0.790099998319149232628433310310356318950653076171875f,-0.17729998264908797267480622394941747188568115234375f,0.4154000170439484662665563519112765789031982421875f,0.886899970969558193445436700130812823772430419921875f,-1.148800012086331978622411043033935129642486572265625f,0.473000028364360947108480104361660778522491455078125f,1.3391000242173671797019096629810519516468048095703125f,0.68989997083544807310317992232739925384521484375f,-0.58550002542138113170722135691903531551361083984375f,-0.738900011485814189882148639298975467681884765625f,1.225600020015239710602372724679298698902130126953125f,1.50869999651908859306104204733856022357940673828125f,0.1331999703899027220899142776033841073513031005859375f,0.1552999729037285447930116788484156131744384765625f,0.57750001922249760699656917495303787291049957275390625f,1.9967000026434658099816488174838013947010040283203125f,-0.317599991986155405498948312015272676944732666015625f,0.4962000105634327695725005469284951686859130859375f,0.3837000371620060690247555612586438655853271484375f,0.5040999684870239860856599989347159862518310546875f,0.11310001591742036008980676342616789042949676513671875f,-1.6595000627934937131868764481623657047748565673828125f,0.1672000026956201867989193488028831779956817626953125f,0.7263999995276331222981980317854322493076324462890625f,0.3592000007808209272042176962713710963726043701171875f,0.386100004048645362164648986436077393591403961181640625f,1.213599966721236267375161332893185317516326904296875f,0.68500001359730988692575692766695283353328704833984375f,0.165700002837180893067170472932048141956329345703125f,0.072499992810189706204937465372495353221893310546875f,0.085499905318021429678765343851409852504730224609375f,0.3689999876767398045984691634657792747020721435546875f,-1.4096000098526466626225328582222573459148406982421875f,-0.810300032210350185124525523860938847064971923828125f,-0.00210002669096009952909298590384423732757568359375f,-0.8752999867618083129627848393283784389495849609375f,0.756399991071223798400069426861591637134552001953125f,1.3777000213742258694082920555956661701202392578125f,0.310600038641691611474016099236905574798583984375f,-1.5364000304102898297742285649292171001434326171875f,-1.7439000295162199183351958708954043686389923095703125f,-0.573299986404180117460782639682292938232421875f,0.2620999874264000606416402661125175654888153076171875f,0.83469999930560534462387067833333276212215423583984375f,-1.43910002432912609293680361588485538959503173828125f,-0.300399978412687840290118401753716170787811279296875f,-0.190199963706731534074378942023031413555145263671875f,-0.360899989640712792748900028527714312076568603515625f,0.1693000117659571035488852430717088282108306884765625f,1.19849999809265117534096134477294981479644775390625f,0.3574000111058353024162670408259145915508270263671875f,-0.4966000052928922769979180884547531604766845703125f,0.3166999990299341760646711918525397777557373046875f,0.49870001276135411671930341981351375579833984375f,-0.15810002042502180330529881757684051990509033203125f,1.59550003831088549333117043715901672840118408203125f,-0.32710003943145304905470993617200292646884918212890625f,0.34560000008493663159470088430680334568023681640625f,0.26350001094490294040184608093113638460636138916015625f,0.262600015157461397219407217562547884881496429443359375f,0.53009996628165279020095113082788884639739990234375f,-0.1542000088647010880293919399264268577098846435546875f,-0.36099999675154759160022877040319144725799560546875f,-0.2968999823316929020933230276568792760372161865234375f,-0.138999953329562497827964762109331786632537841796875f,-0.3930000087693332755378605725127272307872772216796875f,1.0424000007495284325642614930984564125537872314453125f,0.340000011622905773389646810755948536098003387451171875f,0.4104000384271146995018852976500056684017181396484375f,0.280900020113587334247284843513625673949718475341796875f,1.70769998948573942243456258438527584075927734375f,0.1844000009953976615406645578332245349884033203125f,-1.106700062918663629574211881845258176326751708984375f,-0.013299982008337707384271197952330112457275390625f,1.254400025582314004424233644385822117328643798828125f,0.059300017651915393201988990767858922481536865234375f,0.35409998197257497309919926919974386692047119140625f,-0.59839996199905931550944160335347987711429595947265625f,0.278200018967688256221748588359332643449306488037109375f,-1.415099978537857960958490366465412080287933349609375f,-1.4668999922037126992790945223532617092132568359375f,-0.568799959131330634676260160631500184535980224609375f,-0.3929000072464348125578226245124824345111846923828125f,0.4525999986171722611061341012828052043914794921875f,-0.548200034055114304720746076782234013080596923828125f,-1.03810002396404765789839075296185910701751708984375f,0.9754000286296011790199145252699963748455047607421875f,-0.46889997952282413962166174314916133880615234375f,0.59749998316168773104806177798309363424777984619140625f,-1.0741999852091073108795171719975769519805908203125f,0.7801999854996803929196858007344417273998260498046875f,-0.5995000031515953509142491384409368038177490234375f,0.45899997046589857685461311120889149606227874755859375f,-0.1154000263758001576519518494023941457271575927734375f,-1.4538999813109636516372802361729554831981658935546875f,-0.011200003224611176921854394095134921371936798095703125f,-1.1171999881297340895258685122826136648654937744140625f,0.02240000320821999846288008484407328069210052490234375f,-0.54580000294446995301456126981065608561038970947265625f,0.1432000039547682013196805428378866054117679595947265625f,-1.24399996871501183903774290229193866252899169921875f,-0.78519999537169937742220326981623657047748565673828125f,-0.881799992096424123388942462042905390262603759765625f,-0.3330999915912749020918681708280928432941436767578125f,-0.78959996311888147602076060138642787933349609375f,-0.539999990668147855643610455445013940334320068359375f,1.0693999916076661893526988933444954454898834228515625f,1.075900007633864507994303494342602789402008056640625f,0.565500027209520528259645288926549255847930908203125f,0.13769997577667236754450641456060111522674560546875f,1.0724999868124722723905506427399814128875732421875f,0.455199951413274295219935083878226578235626220703125f,0.217599994929134898935529918162501417100429534912109375f,-0.298200000490247862927617461537010967731475830078125f,-0.2647999927073716452952112376806326210498809814453125f,-0.029399996487796453692453724215738475322723388671875f,0.34119999838173387107786993510671891272068023681640625f,0.0215999904662371733365944237448275089263916015625f,-1.3476000312507150358243279697489924728870391845703125f,-0.29349998300522539551593581563793122768402099609375f,-0.77239997251182768156496649680775590240955352783203125f,-1.0492999903082846913093817420303821563720703125f,-0.6754999763965610082294688254478387534618377685546875f,-0.295799993705749830752438356284983456134796142578125f,-0.093799986977875227101009159014211036264896392822265625f,-0.6347999902114274117792547258432023227214813232421875f,0.0846999915197494124274868454449460841715335845947265625f,0.1127999957397580743645448819734156131744384765625f,0.90309999218732095460637765427236445248126983642578125f,1.083199982646107617512143406202085316181182861328125f,-0.2647000110402704109446858637966215610504150390625f,0.4307000055193899168415327949333004653453826904296875f,0.32480000097751560161896122735925018787384033203125f,0.0764999868571758501190060997032560408115386962890625f,0.58219996238797933063580103407730348408222198486328125f,0.2453999904826285050063461312674917280673980712890625f,-0.20820000607818356375133816982270218431949615478515625f,0.05049999502301216747213175040087662637233734130859375f,0.167299996283650387329799968938459642231464385986328125f,0.243999992780387364721406129319802857935428619384765625f,0.347199966150522509877873744699172675609588623046875f,0.23080000573843728606249214863055385649204254150390625f,-0.520399967497586768416795166558586061000823974609375f,1.045000000037251908224789076484739780426025390625f,-0.5940000086873766793615914139081723988056182861328125f,0.4993999988719817206828111011418513953685760498046875f,-0.37310001271367099207054707221686840057373046875f,0.070700044709443243817759139346890151500701904296875f,1.1194000290095804661660849887994118034839630126953125f,1.7997000218808647531432143296115100383758544921875f,-0.804699978369473711836690199561417102813720703125f,-1.7456000623345371280947802006267011165618896484375f,0.30169992985129301388269595918245613574981689453125f,2.6779000034093858317874037311412394046783447265625f,1.287600016796588864309569544275291264057159423828125f,2.054800015692413328594057020382024347782135009765625f,-0.60560001032948473920214382815174758434295654296875f,-0.0623999893501399061079837338184006512165069580078125f,0.579399941353500036456125599215738475322723388671875f,0.072699990379810675023009025608189404010772705078125f,0.419100008086860764677794577437452971935272216796875f,0.481699961739778803604394852300174534320831298828125f,-0.0788999797835947536128742285654880106449127197265625f,-1.25340002182722098922340592253021895885467529296875f,-1.0709000124394900677060604721191339194774627685546875f,1.36450004476308794920669242856092751026153564453125f,1.2130000151395796681441652253852225840091705322265625f,0.13240001032352444187978335321531631052494049072265625f,-0.243299980443715979649965674980194307863712310791015625f,0.476199995289742883386452376726083457469940185546875f,0.0797000048935416760542693737079389393329620361328125f,0.7409000072240827794445294784964062273502349853515625f,-0.31860005229115540004158901865594089031219482421875f,0.2844999924972650973131749196909368038177490234375f,0.04910002660155310394429761799983680248260498046875f,-0.0831000153586265444261016455129720270633697509765625f,0.93299997401237533267703838646411895751953125f,1.003500006288290169464971768320538103580474853515625f,-0.01969999699592595021613306016661226749420166015625f,-1.645999992132185951732026296667754650115966796875f,-1.17130000392794642039007158018648624420166015625f,0.0581999909162522754968449589796364307403564453125f,0.11020000736713431166435839259065687656402587890625f,0.726299943578244011632705223746597766876220703125f,0.692899991935491232908361780573613941669464111328125f,-0.026099986875056657709137653000652790069580078125f,-0.72910000261068363958116833600797690451145172119140625f,-0.33459998891651565600113826803863048553466796875f,-0.351800010871887280927694519050419330596923828125f,0.775299996820092207627794778090901672840118408203125f,0.45530001328587543785175739685655571520328521728515625f,-0.67609999298453349325654926360584795475006103515625f,-0.1260999640762812123995217916672118008136749267578125f,0.66780000479221346409985926584340631961822509765625f,-0.1650999631524092681189586073742248117923736572265625f,0.03750001080334186553955078125f,0.88449994827806988073604088640422560274600982666015625f,0.9332999851375820554721940425224602222442626953125f,0.19820004515647848819526188890449702739715576171875f,-0.639700016218423428426831378601491451263427734375f,0.49479997795522212555852092918939888477325439453125f,1.4123000053733587577653452171944081783294677734375f,-0.7576999797999857122476896620355546474456787109375f,-0.580200031805038651810946248588152229785919189453125f,-0.322800006543099871070268136463710106909275054931640625f,-0.34149999712035039589608231835882179439067840576171875f,1.145900031214952985436639210092835128307342529296875f,-1.20220001877024795788884148350916802883148193359375f,1.0232000172168016494111952852108515799045562744140625f,-1.4905000106692316563794520334340631961822509765625f,0.2634000098317861304764164742664434015750885009765625f,-1.03580000205039990390787352225743234157562255859375f,0.4487000155180691507439405540935695171356201171875f,0.457199975985288720181642929674126207828521728515625f,-0.41440000501871099292117150980629958212375640869140625f,-0.3390000025033952368858081172220408916473388671875f,-1.654700006905198250706234830431640148162841796875f,-1.12210001210644794156223724712617695331573486328125f,0.56490000432580733669141181962913833558559417724609375f,0.42610000855624619031658539825002662837505340576171875f,1.74509998297095325625605255481787025928497314453125f,1.06879996943175736845432766131125390529632568359375f,1.141099984788894516896107234060764312744140625f,1.993299978581070330818647562409751117229461669921875f,1.7441999707177282541437079999013803899288177490234375f,0.51649998369067962045164676965214312076568603515625f,-0.73789997409135121841927684727124869823455810546875f,-0.6327000374302269580795154979568906128406524658203125f,0.57749999651685346435669998754747211933135986328125f,0.20190002373829518678149952393141575157642364501953125f,0.056199978581815912548336200416088104248046875f,0.06470000435337464050888911515357904136180877685546875f,-0.170200001555681268428088515065610408782958984375f,-0.9669999850094317395132748060859739780426025390625f,-1.24409995194673594909318126155994832515716552734375f,-1.8108999662458895585359641700051724910736083984375f,0.66019996244013345876311404936132021248340606689453125f,-0.0068999973073602394180170449544675648212432861328125f,-0.342299964134395506842878376119188033044338226318359375f,-0.5261000080347060237073719690670259296894073486328125f,0.33389998902231454724898185304482467472553253173828125f,0.197800037127733308039978510350920259952545166015625f,0.45020001496672623630956877605058252811431884765625f,-0.822699995446205090132707482553087174892425537109375f,0.322299991664290297332939871921553276479244232177734375f,-0.6650000028312208488756596125313080847263336181640625f,-0.183500010460615481377999458345584571361541748046875f,0.3866000140845773369591142909484915435314178466796875f,-0.67640004531145170307837588552501983940601348876953125f,0.3888000402867799465411735582165420055389404296875f,-0.224699994388222579200231621143757365643978118896484375f,0.9985999654173853823380113681196235120296478271484375f,0.4826999771177769016361480680643580853939056396484375f,0.099900042547286016514362927409820258617401123046875f,1.3738000097021456635815184199600480496883392333984375f,0.0531999307155606171448880559182725846767425537109375f,0.0664999725893142823451853473670780658721923828125f,0.515200011427700221844361294643022119998931884765625f,0.750399976587295736152327663148753345012664794921875f,-0.12920008873492516698888721293769776821136474609375f,-1.9747000677764419140913787487079389393329620361328125f,-0.5519999795705066159001717096543870866298675537109375f,0.0710000196993352261642939993180334568023681640625f,-0.71709999286532433160346045042388141155242919921875f,-0.63750001534819578097312842146493494510650634765625f,0.5625999984681604448866210077540017664432525634765625f,0.68060000474154891403344436184852384030818939208984375f,-0.611199963252246458722538591246120631694793701171875f,-0.884999993927776618107827744097448885440826416015625f,-0.2499000051453712512028459968860261142253875732421875f,1.067900020992755827364817378111183643341064453125f,-0.2429999670088296426229135249741375446319580078125f,-1.01829999504685364541956005268730223178863525390625f,0.51350001835823011475667954073287546634674072265625f,1.10959999882578852492542864638380706310272216796875f,0.7589000221401451806713112091529183089733123779296875f,-0.363600012020766971687635304988361895084381103515625f,-1.57670001925826053224000133923254907131195068359375f,0.1333000095009804208956438742461614310741424560546875f,0.68240001524090754259077584720216691493988037109375f,0.35629999483823748818167587160132825374603271484375f,-1.0378000253558157250921567538171075284481048583984375f,0.46210003976672953118765008184709586203098297119140625f,-1.0502000414907934899844121900969184935092926025390625f,0.69650001673400419033299613147391937673091888427734375f,-0.1364999770224091246717534886556677520275115966796875f,0.836500014759600230007663412834517657756805419921875f,-0.52439998002201371196662194051896221935749053955078125f,-0.17099997767806041082394585828296840190887451171875f,0.4308999981343755170115628061466850340366363525390625f,-0.38709999107718484623319454840384423732757568359375f,-1.1875000591203577204879593409714289009571075439453125f,-0.988400045818090600135974455042742192745208740234375f,-0.56089999407380819462787258089520037174224853515625f,1.05540004390329134054127280251123011112213134765625f,0.4806999820128077249847819985006935894489288330078125f,0.5177000042751427333342917336267419159412384033203125f,0.30070005204826666389550382518791593611240386962890625f,1.95930001534223574566340175806544721126556396484375f,1.2477000074416400021704021128243766725063323974609375f,-0.193400008565186798392687705927528440952301025390625f,-1.9928000120937827688294419203884899616241455078125f,1.44640000418424552464102816884405910968780517578125f,1.26390003234744074944728708942420780658721923828125f,-0.70810001286268242637333969469182193279266357421875f,0.3945999959945680757300578989088535308837890625f,0.62589999329149692552931583122699521481990814208984375f,-0.976000001482665080487777231610380113124847412109375f,-0.569100034238398055919105900102294981479644775390625f,0.62240000626295799435183653258718550205230712890625f,0.265200003045797050305054654018022119998931884765625f,-0.039900012409687324321794221759773790836334228515625f,0.094000014908611984054687127354554831981658935546875f,1.04500002823770099524836041382513940334320068359375f,-0.65790001771450068446256409515626728534698486328125f,0.983099973747134203705400068429298698902130126953125f,-0.79040000203102767084573088141041807830333709716796875f,0.210499981462955343403109509381465613842010498046875f,-0.079400001889467386462229114840738475322723388671875f,0.9062999741256236863051753971376456320285797119140625f,-0.005100013779103240718626466332352720201015472412109375f,-0.5384999622739854441277884689043276011943817138671875f,0.33169999653398940520077076143934391438961029052734375f,-0.96760005486905653615536948564113117754459381103515625f,-0.2095000075101858261206189126824028789997100830078125f,0.24720005466342076516639281180687248706817626953125f,-0.18149999102950087337404738718760199844837188720703125f,-0.75970001215785776249589389408356510102748870849609375f,-1.8206999617472290342590213185758329927921295166015625f,-1.014199957716465316792664452805183827877044677734375f,-0.75970000500529977216501720249652862548828125f,-1.5808000168025493348267218607361428439617156982421875f,-1.4517999923571942577638083093916065990924835205078125f,-0.78869995250478430737217649948433972895145416259765625f,0.8907999873355028075394557163235731422901153564453125f,-0.1761000092267990257965948330820538103580474853515625f,-0.615400020620227028445015093893744051456451416015625f,1.10060001129806028785651506041176617145538330078125f,0.4277000055044890558519909973256289958953857421875f,0.39619995785057560144792887513176538050174713134765625f,-0.01699999074637903451190368286916054785251617431640625f,0.2473999696806077341904028799035586416721343994140625f,1.38639996629804418404319221735931932926177978515625f,1.5631999695703389097189983658608980476856231689453125f,-0.7314999837093056900272358689107932150363922119140625f,0.61859999687746169971802601139643229544162750244140625f,-1.1078999965734777077841499703936278820037841796875f,0.3969000054657458864681984778144396841526031494140625f,-0.317100018309056874699081163271330296993255615234375f,0.4920000171959397761156651540659368038177490234375f,-0.13519998918771747309364172906498424708843231201171875f,0.3017000066667796520647470970288850367069244384765625f,-0.09770000453591372313599805465855752117931842803955078125f,0.6923999912500382603042226037359796464443206787109375f,0.1670999938711524024359533768802066333591938018798828125f,0.190799997542798305261158020584844052791595458984375f,0.1455000071302057218503023250377736985683441162109375f,-0.312399996912479327448863841709680855274200439453125f,0.37950000769644953724224478719406761229038238525390625f,1.31470003961324710672897708718664944171905517578125f,0.30699999469518690631275603664107620716094970703125f,1.039100017623602933980464513297192752361297607421875f,0.852500000856816431138440748327411711215972900390625f,0.9363999703958632014888507910654880106449127197265625f,0.06799998090416192741969325652462430298328399658203125f,-0.013100004965066858186872877922724001109600067138671875f,0.351799975258112296305768040838302113115787506103515625f,0.0370000054612752149463261730488738976418972015380859375f,0.311799994853138695560801352257840335369110107421875f,-0.76010004246383899850769694239716045558452606201171875f,0.314199972915649272664495583740063011646270751953125f,-0.835800024737417768250224980874918401241302490234375f,-0.856799961996078440762403261032886803150177001953125f,-0.63010000249147424256079830229282379150390625f,0.094800016586482183100770271266810595989227294921875f,-0.8143999628111722177692399782245047390460968017578125f,0.05309999594986447579714194944244809448719024658203125f,0.4183999943345784178205803982564248144626617431640625f,-0.1667999984666703738866999628953635692596435546875f,0.309299987514317142878184085930115543305873870849609375f,0.83059999013841123716161973788985051214694976806640625f,0.170900022795796413532798396772705018520355224609375f,-0.646100015783309888917074204073287546634674072265625f,-1.8575000279024245219972044651512987911701202392578125f,0.41220001547336571068314015064970590174198150634765625f,0.7503999728992580831032910282374359667301177978515625f,0.19050000072270678597163851009099744260311126708984375f,0.98580000622272478949525975622236728668212890625f,-0.64530002531856300862500575021840631961822509765625f,-0.0488999810874464202470335294492542743682861328125f,0.21249997757375183038419663716922514140605926513671875f,0.88570003710985201195171612198464572429656982421875f,0.3462000534415252683828612134675495326519012451171875f,-0.420800027084350602990525658242404460906982421875f,-0.6980000229626897034762578186928294599056243896484375f,0.46300000049918843547658298120950348675251007080078125f,0.9232999860316517537484060085262171924114227294921875f,0.6149000086843965373617493241908960044384002685546875f,0.8626999902307990186756114781019277870655059814453125f,-0.149599929572641787700604254496283829212188720703125f,0.2203000259518628212163093849085271358489990234375f,0.347300006304681385671528914826922118663787841796875f,-0.27860007762312921641978391562588512897491455078125f,-0.3478000146642330481228100325097329914569854736328125f,2.2067000509977336974998252117075026035308837890625f,2.146000013738870393353863619267940521240234375f,-1.10649998283386263864258580724708735942840576171875f,-1.2868000053212040878491961848339997231960296630859375f,-0.035900012029707273342182816122658550739288330078125f,0.6152999710038307146220404320047236979007720947265625f,-0.402300024074315754063491112901829183101654052734375f,1.0733999564111236235675050920690409839153289794921875f,-0.1655999765038485005419488516054116189479827880859375f,-1.079900026416778846538591096759773790836334228515625f,-0.54719998343586961908613375271670520305633544921875f,1.369300035160780026899374206550419330596923828125f,-0.49109999387860359121305009466595947742462158203125f,-1.65829998983144744073570109321735799312591552734375f,0.51489997903108619681233903975225985050201416015625f,-0.0563999670803543295960480463691055774688720703125f,0.4454000032231213435096606190199963748455047607421875f,-1.4126999622166156456160024390555918216705322265625f,0.3013000072062010925577624220750294625759124755859375f,-0.74069998674392678594813332892954349517822265625f,-0.00299999010562890244813161189085803925991058349609375f,0.649799997401237394711870365426875650882720947265625f,-0.068700024868548315026828277041204273700714111328125f,0.10030004986822638191057421863661147654056549072265625f,0.8151000199630853426668863903614692389965057373046875f,0.57530001067817249094105136464349925518035888671875f,-0.269800006863474661766844064914039336144924163818359375f,-0.12430001470893603521261638888972811400890350341796875f,0.436399991667271169859532164991833269596099853515625f,-0.32789999327659591354944268459803424775600433349609375f,0.38739999601840935383734176866710186004638671875f,0.5305999904736877770261571640730835497379302978515625f,0.731099997901917486586853556218557059764862060546875f,0.42959999167323126290085610889946110546588897705078125f,1.33070002012252786016688332892954349517822265625f,0.702500015199184257852493828977458178997039794921875f,1.609100002908706361637314330437220633029937744140625f,0.3701000052109357785212750968639738857746124267578125f,1.40400003662705419316125698969699442386627197265625f,-0.64009997566938370283651238423772156238555908203125f,-0.206999989256262839631972383358515799045562744140625f,-0.801699969115853061651932875975035130977630615234375f,0.4282000097289682116752373985946178436279296875f,0.40719999952912377949587607872672379016876220703125f,-0.14120001037716889413786702789366245269775390625f,-1.6346000010609618158241573837585747241973876953125f,-0.8107000165462492358159352079383097589015960693359375f,1.6740999851763245231950349989347159862518310546875f,0.25990002678930768098553016898222267627716064453125f,0.12340002324283061430065799868316389620304107666015625f,0.762400024144351817767528700642287731170654296875f,-1.22070002503991137388084098347462713718414306640625f,0.49769999976754208947937740958877839148044586181640625f,-1.027300026458501935877620780956931412220001220703125f,0.21620003705024704032666704733856022357940673828125f,-0.8382000169932843913755959874833934009075164794921875f,0.2696000016942619037507711254875175654888153076171875f,-0.89080001516342210976517890230752527713775634765625f,-1.0669000128418206774227883215644396841526031494140625f,0.96830000115633030333128772326745092868804931640625f,0.380799978022277940681306063197553157806396484375f,0.29940000926554233284804240611265413463115692138671875f,0.7315000184476379008202684417483396828174591064453125f,-0.440299993057549021724383919718093238770961761474609375f,-1.1289000045195225485628043315955437719821929931640625f,-0.213199995349347692918939856099314056336879730224609375f,-1.095300019767881050114510799176059663295745849609375f,-0.3148000043302772166242675666580908000469207763671875f,-0.002000013619660734320859774015843868255615234375f,-0.2539000027313829122022070805542171001434326171875f,0.605800013971328876749566916259936988353729248046875f,0.7938999970689415430769031445379368960857391357421875f,0.7190999890878793099346921735559590160846710205078125f,0.8015999976187941911121015436947345733642578125f,0.647099973195791822178080110461451113224029541015625f,-0.29029998631477393900013339589349925518035888671875f,-1.81349998714029769786293400102294981479644775390625f,-0.28619998124539891559692250666557811200618743896484375f,0.4303999816164374436766593134962022304534912109375f,1.130499993570148742350056636496447026729583740234375f,0.453000005751848122681479935636161826550960540771484375f,-0.0663999611571437498014347511343657970428466796875f,0.182600010016560787562411860562860965728759765625f,-0.6697999927446243173534412562730722129344940185546875f,-1.732900037384033087306534071103669703006744384765625f,0.110500009477139116143007413484156131744384765625f,1.96420000912547099147786866524256765842437744140625f,0.3935000325143338528732783743180334568023681640625f,-0.26440002155899977598352279528626240789890289306640625f,-0.23300002221763105314522590560954995453357696533203125f,0.095199987399578134272815077565610408782958984375f,0.67769999436587102792373116244561970233917236328125f,0.94149997996538881128714137958013452589511871337890625f,-0.2315999985501167035550906803109683096408843994140625f,1.03990001267045695243496084003709256649017333984375f,0.1910000026002525164159351334092207252979278564453125f,-0.45909998346269187408097423030994832515716552734375f,-0.27469997136592905206953219021670520305633544921875f,-0.1401999616950748706045715152868069708347320556640625f,0.2582000140503046825557476040557958185672760009765625f,-0.634399952827394475463052003760822117328643798828125f,-0.0572999963209033236211098483181558549404144287109375f,0.37609997316598953176480790716595947742462158203125f,0.83499999295920124264824835336185060441493988037109375f,-0.55559999248534464033610902333748526871204376220703125f,-0.042799988475441974600244066095910966396331787109375f,-0.5454000090718273963119600011850707232952117919921875f,-0.058999983541667189168578033786616288125514984130859375f,-0.3746999426066874860907773836515843868255615234375f,-0.297100004301964926156642832211218774318695068359375f,0.80049994376301736309642365085892379283905029296875f,-0.399599974052608164498678888776339590549468994140625f,-1.353400017207861072421337667037732899188995361328125f,-0.5287999736785895521506972727365791797637939453125f,0.099200002568960332638425825280137360095977783203125f,0.62289997867345814341888399212621152400970458984375f,0.159200004208087886325984072755090892314910888671875f,0.2654999853372572715670685283839702606201171875f,0.727000014215708034726048936136066913604736328125f,-0.20800006908178403364217956550419330596923828125f,-0.60109999969005567521662669605575501918792724609375f,-0.72430002368688661817941465415060520172119140625f,-1.27460000746846180419424854335375130176544189453125f,0.41079998815506701959066049312241375446319580078125f,0.5164999523982405360555958395707421004772186279296875f,-0.332000033885240242170766578055918216705322265625f,-0.35549998407065885874800414967467077076435089111328125f,0.4315000129342079215888361432007513940334320068359375f,-1.0611999898135657804942866277997381985187530517578125f,-0.29229997922182082703557171043939888477325439453125f,-0.811400003550946546937439052271656692028045654296875f,0.083000004708767338712505079456605017185211181640625f,-0.363900035588443415690562687814235687255859375f,1.069500036269426690438422156148590147495269775390625f,-0.645700016359984640956781731802038848400115966796875f,-0.1302000072181226375533924510818906128406524658203125f,-0.97139996350407553649120018235407769680023193359375f,-0.32360000054687265702568765846081078052520751953125f,0.5748000070869920818950049579143524169921875f,0.09259999921321881988234281379845924675464630126953125f,0.696100010716915296171691807103343307971954345703125f,-0.15530002081096194732623416712158359587192535400390625f,1.17060001271367131181477816426195204257965087890625f,-0.202100017191469749633370156516321003437042236328125f,1.3087000425636770017945309518836438655853271484375f,0.1676000135555864023473304769140668213367462158203125f,0.69830000964999161539026317768730223178863525390625f,-0.761499998703599079163950591464526951313018798828125f,-0.64940000010132781227412124280817806720733642578125f,-0.8958000342369079049831270822323858737945556640625f,-1.7907999926626683162567132967524230480194091796875f,0.2801000240981570410525591796613298356533050537109375f,-0.13409998420774915661723980520037002861499786376953125f,0.27539997353255696310725397779606282711029052734375f,-0.6058999932169915059176901195314712822437286376953125f,0.05049999606609301139314993633888661861419677734375f,0.43749999575316900202182068824185989797115325927734375f,0.16939997350275515231032841256819665431976318359375f,-0.452599988782405926457386158290319144725799560546875f,0.591700001451373136518441242515109479427337646484375f,0.5971999807909131874339436762966215610504150390625f,1.53139996171593661955512288841418921947479248046875f,-0.23710005172491011027346985429176129400730133056640625f,1.4258999759316448585622083555790595710277557373046875f,0.464799982574582326577683488721959292888641357421875f,1.1381999607414010622363775837584398686885833740234375f,-0.3418000166088341895687108262791298329830169677734375f,0.66350000345706960924729855832993052899837493896484375f,-0.2687999998301269233280663684126920998096466064453125f,0.751800002452731153113063555792905390262603759765625f,-0.0962000156298277175892508239485323429107666015625f,-0.46450000434368854396183223798288963735103607177734375f,0.631099987024069886132338069728575646877288818359375f,-0.35990000998824844824497404260910116136074066162109375f,0.25720002568066124837287134141661226749420166015625f,0.0899999989941713263164047020836733281612396240234375f,0.536999991714954827415340332663618028163909912109375f,0.106299990926682852432350046001374721527099609375f,0.6263000247150658506001263958751223981380462646484375f,-0.20359999846071008544612368496018461883068084716796875f,0.69899999333918128030518346349708735942840576171875f,-0.083099995279312166562846186934621073305606842041015625f,-0.50559998276233730774009700326132588088512420654296875f,0.43890003085732498533388934447430074214935302734375f,0.568700038540363106420727490331046283245086669921875f,1.23809999930262559786342535517178475856781005859375f,0.6851000376582145889869934762828052043914794921875f,-1.086000056207179653711136779747903347015380859375f,-1.8571000149190421968370401373249478638172149658203125f,1.18209999536126897368149002431891858577728271484375f,1.099300024059414848665028330287896096706390380859375f,-0.4070999846324327986479829633026383817195892333984375f,-1.490200006175041114175883194548077881336212158203125f,-0.4189000271320342250191970379091799259185791015625f,-0.1611000100463630957392524578608572483062744140625f,0.56909999486207940577742192544974386692047119140625f,-0.8037999820232386571916549655725248157978057861328125f,0.936300007057190075698827058658935129642486572265625f,0.048400009496509799333807677612639963626861572265625f,1.0643999889254569524155158433131873607635498046875f,-1.0908999853938816659848498602514155209064483642578125f,0.982500008083879716735964393592439591884613037109375f,1.435199990156293292642430969863198697566986083984375f,0.5850999925449491545492719524190761148929595947265625f,0.279999977126717425335300504229962825775146484375f,0.9656999505341052891793651724583469331264495849609375f,-0.5036000047191973028049005733919329941272735595703125f,0.3700000150874256377164783771149814128875732421875f,1.5210999903023234569587884834618307650089263916015625f,0.142800001625716432851476156429271213710308074951171875f,1.5990999710202213890397615614347159862518310546875f,-0.04699998046457753986260286183096468448638916015625f,0.06770002252906548445565704241744242608547210693359375f,0.40109998914748434817312272571143694221973419189453125f,0.277099953079223804142117160154157318174839019775390625f,0.12039998869448885476796107241170830093324184417724609375f,0.140700001645088779156367309042252600193023681640625f,-0.0837000071361659880864181104698218405246734619140625f,0.3499999849125747797273788819438777863979339599609375f,-0.1560000457763663916921359486877918243408203125f,0.297899991041421785542553379855235107243061065673828125f,0.014499986089766547792123674298636615276336669921875f,-1.1273999737411737331882477519684471189975738525390625f,-1.3759999488070613349322002250119112432003021240234375f,0.0249000319674614889464692168985493481159210205078125f,0.324499983929097346457837147681857459247112274169921875f,-0.8822000298343601887296472341404296457767486572265625f,-0.0328999787665897880373222506023012101650238037109375f,0.686899996525049072459978560800664126873016357421875f,-0.19259998908042863519085585721768438816070556640625f,0.55710003291219489796048947027884423732757568359375f,0.621399980044364586007077377871610224246978759765625f,0.352900010636448657752595181591459549963474273681640625f,0.5510999673172829460554567049257457256317138671875f,0.447399964018166673707099789680796675384044647216796875f,-0.270400007916986595546404714696109294891357421875f,0.9195000084787607885772331428597681224346160888671875f,1.328899977175891589098455369821749627590179443359375f,-0.1213000097207725946901746283401735126972198486328125f,-0.4491000246457759192963976602186448872089385986328125f,0.9705000097006557524537129211239516735076904296875f,-0.60320004202723520592144268448464572429656982421875f,0.466200023713708500139318857691250741481781005859375f,0.1803999981567263599657735539949499070644378662109375f,-0.22700002609938330078165336090023629367351531982421875f,-0.34509999679178005038693299866281449794769287109375f,-0.55129995818138155527776689268648624420166015625f,-0.30569998752623828153218710212968289852142333984375f,0.0177999859981234909156455614720471203327178955078125f,-0.1270000017546115689270891380147077143192291259765625f,-1.0053000196933743470850686207995750010013580322265625f,-0.0125000050850212485187284983112476766109466552734375f,-0.902800034892558667110051828785799443721771240234375f,-0.21400000295042997944250373620889149606227874755859375f,-1.1417999958775941760080741005367599427700042724609375f,-0.1231999931141729565098330567707307636737823486328125f,-0.5555000160522760577208600807352922856807708740234375f,0.326300040361285681456138263456523418426513671875f,-0.3515000229179860991024497707257978618144989013671875f,0.3509999833405021973931070533581078052520751953125f,0.06300000604987143759672107989899814128875732421875f,0.7295000396966937472598146996460855007171630859375f,-0.03269999023079872557673297706060111522674560546875f,0.99660000678300786347563189337961375713348388671875f,0.533100016736984372300867107696831226348876953125f,-0.327600038477778909840054666346986778080463409423828125f,0.1077999806523322889884042297126143239438533782958984375f,0.2358999964579939767617133838939480483531951904296875f,0.411599985325336259478490319452248513698577880859375f,-0.6866999994769689141804747123387642204761505126953125f,0.3259999814033507359312125117867253720760345458984375f,0.1860000118389724121925610234029591083526611328125f,1.2107999711304902046293818784761242568492889404296875f,-0.0773999940067529035303550699609331786632537841796875f,0.30020003791451443930782261304557323455810546875f,-0.486800016534328250372709590010344982147216796875f,-1.645599992299079161739427945576608180999755859375f,0.22240000317096642046266197212389670312404632568359375f,-1.24019999574422801202899790951050817966461181640625f,-0.391900072297454382663772776140831410884857177734375f,-0.964199960622190044290391597314737737178802490234375f,0.21290000139772846665664474130608141422271728515625f,0.4083000162810090927223427570424973964691162109375f,-0.2420000497922305537912279760348610579967498779296875f,1.4849000674322254678116905779461376368999481201171875f,0.067899970813094956412214742158539593219757080078125f,0.030299994063377244657431219820864498615264892578125f,-0.05260002394914664591851760633289813995361328125f,1.1952000394046304876383146620355546474456787109375f,1.9308000449657445329876281903125345706939697265625f,-1.243400065338612581200550266657955944538116455078125f,0.957900008177757111837991033098660409450531005859375f,1.161800014746189102510243174037896096706390380859375f,0.48820001207590102421107758345897309482097625732421875f,-0.9428000100821256612704246435896493494510650634765625f,1.0622000042229888805422888253815472126007080078125f,-0.744700037527084912625241486239247024059295654296875f,0.797800015558302266782675360445864498615264892578125f,0.1845000546723607470767092308960855007171630859375f,0.809300018322467895615091038052923977375030517578125f,0.30699999707937219994846600457094609737396240234375f,-0.16290000072717614187922663404606282711029052734375f,-0.227299979892372494560959239606745541095733642578125f,0.124700010034441532980054034851491451263427734375f,-0.762600003981589669166396561195142567157745361328125f,0.721699982005357565384429108235053718090057373046875f,0.0602000081866982394984688653494231402873992919921875f,-0.056199999387562371566673391498625278472900390625f,-1.034400012022256287735899604740552604198455810546875f,-0.3623000047773123899474967402056790888309478759765625f,0.03399998141825211206423773546703159809112548828125f,0.031499999523163069170550443232059478759765625f,0.3186000002861018032263018540106713771820068359375f,-1.1707000147581094751103591988794505596160888671875f,1.640400024792551647578875417821109294891357421875f,1.13190002349615070897925761528313159942626953125f,-1.1225000341236592849014641615212894976139068603515625f,-0.8796000094801186275361715161125175654888153076171875f,-0.304899959287046495859385686344467103481292724609375f,0.47299998150020805542936841447954066097736358642578125f,1.40820001446008635781481643789447844028472900390625f,0.89409995872676351069685551919974386692047119140625f,-0.2098000500023378123160000541247427463531494140625f,1.381299977701901493531977394013665616512298583984375f,-0.839699997144936816084737074561417102813720703125f,-0.51689999697208488527166991843841969966888427734375f,1.0398000124514099606898298588930629193782806396484375f,0.6952000035673382427603428368456661701202392578125f,0.5824000068217516368207498089759610593318939208984375f,0.1760000080764292373913804112817160785198211669921875f,-0.192900013281405602327822634833864867687225341796875f,0.5275999998837705984300328054814599454402923583984375f,-1.33049996156990513185292002162896096706390380859375f,1.28399998994916675343347378657199442386627197265625f,0.8956000242620714146823956980369985103607177734375f,-0.0149999897927048930768023637938313186168670654296875f,-0.54880001070797479467699986344086937606334686279296875f,-0.031800008040666849495892165577970445156097412109375f,-0.5597000290334219751997579805902205407619476318359375f,-0.406000022828578721600933931767940521240234375f,0.1800000258535146002714100177399814128875732421875f,0.1684000140413639545755586368613876402378082275390625f,0.66559998738169723342394945575506426393985748291015625f,0.51959998597353662663778095520683564245700836181640625f,0.61119998608827563746359601282165385782718658447265625f,-0.4271000102251762964300496605574153363704681396484375f,0.75200000460445881600435313885100185871124267578125f,-0.469099961781501928470561324502341449260711669921875f,0.37540001287162338972080988241941668093204498291015625f,1.158499971866607580750496708787977695465087890625f,-1.0629999509155749759514719698927365243434906005859375f,0.34880001599788668098511834614328108727931976318359375f,-0.90280005363076998303739628681796602904796600341796875f,-0.44930001513734463269855723410728387534618377685546875f,1.7602000550508503096125423326157033443450927734375f,0.56120004435181680690902794594876468181610107421875f,-1.30620003779530602372460634796880185604095458984375f,-0.301599989348649177856032110867090523242950439453125f,-1.759200046378374349131945564295165240764617919921875f,-0.721200014474987494139668342540971934795379638671875f,-0.76830004615783753507685105432756245136260986328125f,-0.0661000503540041250971626141108572483062744140625f,-2.07440005225539270128365387790836393833160400390625f,-0.07499999947845947811941869076690636575222015380859375f,0.84570004264190856702043674886226654052734375f,-0.2934999998994169967403422560892067849636077880859375f,-1.237600034752488920020141449640505015850067138671875f,0.0454000063896182748379715121700428426265716552734375f,1.338800020691752745705116467433981597423553466796875f,-0.88030002207756030685459336382336914539337158203125f,0.6655000380128626158438009952078573405742645263671875f,-0.030000034868717051494968472979962825775146484375f,0.483999989911913619522465523914434015750885009765625f,-0.0577000255465514033659246706520207226276397705078125f,0.484400001963972925977941486053168773651123046875f,-0.19740000739917162864145439016283489763736724853515625f,-0.179600015328824691440701144529157318174839019775390625f,0.20470001634880918839343166837352328002452850341796875f,0.425800009277462532253366589429788291454315185546875f,-1.0514000240027900900940949213691055774688720703125f,-0.3324000068217518588653547340072691440582275390625f,0.732100009331106971188773968606255948543548583984375f,-0.110599983954430047816686055739410221576690673828125f,1.106999990448355841721195247373543679714202880859375f,-0.014400011649728039486717534600757062435150146484375f,-0.181399960970878737498424015939235687255859375f,-0.2129999752789739630998155917041003704071044921875f,0.04060000392198614971306369625381194055080413818359375f,0.0283999757081263570768214776762761175632476806640625f,0.8907000168070202672510049524134956300258636474609375f,-0.52510004704594681612661588587798178195953369140625f,-0.45510001024007795678016918827779591083526611328125f,-1.17410003241300575638206282746978104114532470703125f,-1.64210003592968067920310204499401152133941650390625f,0.0021000520229350616574492960353381931781768798828125f,-0.60170002089738883199743213481269776821136474609375f,-1.9632000533521172247475305994157679378986358642578125f,-1.1715999914348120825025034719146788120269775390625f,-0.011800000701844393091732854372821748256683349609375f,-0.735300020177662272402585585950873792171478271484375f,0.20680002581030143549156719018355943262577056884765625f,-0.524600016409158431685000323341228067874908447265625f,-0.025100021117925752633937008795328438282012939453125f,-1.1254000185340646833509481439250521361827850341796875f,0.1461000092267992211958471671096049249172210693359375f,-0.2071000035196544075688507291488349437713623046875f,0.5895999540477989686593218721100129187107086181640625f,0.410400025463104611844755709171295166015625f,1.26760004302263329378774869837798178195953369140625f,-0.259199996459483994470929246745072305202484130859375f,-0.18050002598017489408022129282471723854541778564453125f,0.54629999427944431200643293777829967439174652099609375f,0.80379997643530309847648140930687077343463897705078125f,-0.56199999920278731480038914014585316181182861328125f,0.276599972361326162939576533972285687923431396484375f,0.16750002771615957186668310896493494510650634765625f,-0.9616000121474261863596666444209404289722442626953125f,1.26619999860525123125398749834857881069183349609375f,-1.7237999825447791746313441763049922883510589599609375f,-0.08680000744462024186987036955542862415313720703125f,-0.253599995070695971577379168593324720859527587890625f,0.581100012207031113575794734060764312744140625f,0.34539999726265679935721664151060394942760467529296875f,-0.7994999943971645706142226117663085460662841796875f,-1.3011000169754023403356768540106713771820068359375f,-1.057699996936321884533072079648263752460479736328125f,0.55230002362728125575586091144941747188568115234375f,1.373999993264675367754534818232059478759765625f,-0.10230000559687635330874400096945464611053466796875f,1.2020000542998321435561592807061970233917236328125f,-1.081100050652027189102000193088315427303314208984375f,-0.9833999881505963713834717054851353168487548828125f,-1.6172000357389446634215346421115100383758544921875f,-2.01370002259612146389144982094876468181610107421875f,0.82210002385973979244226939044892787933349609375f,-1.22599997313320674408032573410309851169586181640625f,0.313500026851892155121959149255417287349700927734375f,-0.40459998966157417754629932460375130176544189453125f,-0.30599999481439599247067917531239800155162811279296875f,0.5201000176161529875429323510616086423397064208984375f,0.342800007921457705606371746398508548736572265625f,-1.0387000155180687865907884770422242581844329833984375f,0.319299974811077191816366394050419330596923828125f,-1.338900014913082525680465550976805388927459716796875f,0.23839999805986877845498383976519107818603515625f,-0.36849999831616886414309419706114567816257476806640625f,-0.1178000088155268798573160893283784389495849609375f,-0.0178999771833421306155287311412394046783447265625f,0.140999968141317655323518920340575277805328369140625f,0.921799976897240025408564179087989032268524169921875f,1.615999986767768792361721352790482342243194580078125f,-0.87370000039339057451570624834857881069183349609375f,0.7547999939739700181462467298842966556549072265625f,-0.14380001212358450857209390960633754730224609375f,-0.128600015187263494453873136080801486968994140625f,0.3139000289201732130095479078590869903564453125f,-0.68580000108182392892786083393730223178863525390625f,1.2372999954640864217481066589243710041046142578125f,0.8916000006735327332307861070148646831512451171875f,-0.471300016295909784957984811626374721527099609375f,0.5812000139534472253899366478435695171356201171875f,-0.11410003554224967814434421597979962825775146484375f,1.24500004827976251675636376603506505489349365234375f,2.4785000185072423306564814993180334568023681640625f,0.000599959814547190717348712496459484100341796875f,0.500999979317187982275072499760426580905914306640625f,0.450700016546249582205518890987150371074676513671875f,-0.280100018882751822246746087330393493175506591796875f,-0.5336000367939472965872482745908200740814208984375f,-0.11890001968145469390947255305945873260498046875f,0.2706999783992767305562665569595992565155029296875f,-0.1641000082731245424128019294585101306438446044921875f,-1.36989999847710119951216256595216691493988037109375f,1.132699992614984552119494765065610408782958984375f,0.213900018861889140708854029071517288684844970703125f,0.6353000147759912952238892103196121752262115478515625f,-0.6286999857157464788315337500534951686859130859375f,-0.58340002756416797780048000277020037174224853515625f,-0.47200001332163810108255574959912337362766265869140625f,0.3686999997973441356435841953498311340808868408203125f,1.197200006681681028197772320709191262722015380859375f,-0.4430999816447502848149042620207183063030242919921875f,-0.861699999067187416557089818525128066539764404296875f,0.42290000162124652405282176914624869823455810546875f,0.59640003003776076440090037067420780658721923828125f,0.3289999862611292513747685006819665431976318359375f,1.395199962738156163055691649788059294223785400390625f,-1.53439999637603730064938645227812230587005615234375f,0.90359998128712160081477122730575501918792724609375f,0.553199999856949009569007102982141077518463134765625f,-0.62539999342560748107899826209177263081073760986328125f,0.140599995875358541042032811674289405345916748046875f,-0.78800003883242564217681547233951278030872344970703125f,-0.31719995139837298125939923920668661594390869140625f,-0.139200010392069639664214264485053718090057373046875f,-0.8428000371277331037589419793221168220043182373046875f,1.07390002176761623076117757591418921947479248046875f,0.256599979811906298010626414907164871692657470703125f,1.0301000463008886942617436943692155182361602783203125f,-0.13959999301433612117762095294892787933349609375f,-0.9460999871730797483593278229818679392337799072265625f,0.32270000788867470031817674680496565997600555419921875f,0.5970000010281799607270158958272077143192291259765625f,0.7044999930188058367974690554547123610973358154296875f,-0.6635999768540254972748471118393354117870330810546875f,-0.142699994812905739394182091928087174892425537109375f,2.1259000203371041237687677494250237941741943359375f,-0.8527999514460571361951224389486014842987060546875f,-0.866599959248304685388575308024883270263671875f,-0.27199999409913999670607154257595539093017578125f,0.1954999790787699254224207834340631961822509765625f,0.3916999821543694082492947927676141262054443359375f,0.44349998319149042114162284633493982255458831787109375f,0.09859998829066807246590542490594089031219482421875f,0.0919000273704522196993593752267770469188690185546875f,1.0202999419838192363130247031222097575664520263671875f,-0.1526999808430673510883934795856475830078125f,0.5045999709606174832288161269389092922210693359375f,-0.223999991327524572426455051754601299762725830078125f,-0.03930000595450400258101808503852225840091705322265625f,0.50950003783404884671881518443115055561065673828125f,0.4059999964162706032055893956567160785198211669921875f,0.020199985909461926070207482553087174892425537109375f,-0.5750999866217372247234607129939831793308258056640625f,-1.13130000452399226418265243410132825374603271484375f,-0.591000013589859207741028512828052043914794921875f,0.777100048670173126907911864691413938999176025390625f,-0.1815000054463744572075256655807606875896453857421875f,1.4646000444978479748669997206889092922210693359375f,-0.3857999982878563827171092270873486995697021484375f,-0.07909995336234476592807141059893183410167694091796875f,0.2201999776393182006728466149070300161838531494140625f,0.510899997687340512442233375622890889644622802734375f,0.783900010368228095103404484689235687255859375f,-0.2904000192046165551573722041212022304534912109375f,-1.63909999128580086136253157746978104114532470703125f,0.44200000631809199802546572755090892314910888671875f,0.17760000822842147982782989856787025928497314453125f,-0.257099984669685088789492510841228067874908447265625f,1.1161000023350122756227165154996328055858612060546875f,-1.882499986290931293098083187942393124103546142578125f,-1.0965000112950800126299100156757049262523651123046875f,0.22999995790422023134169648983515799045562744140625f,-0.39960003093778961869020349695347249507904052734375f,-0.578100012564659149205681387684307992458343505859375f,-0.460799991247057949550480770994909107685089111328125f,0.00740000138282770336672911071218550205230712890625f,-1.0542999464988707103429987910203635692596435546875f,0.598500012248754043042708872235380113124847412109375f,-1.0960999962627884229959818185307085514068603515625f,-0.0781000098079445148613331184606067836284637451171875f,-0.43420000830590677054487969144247472286224365234375f,-0.133800010335445751508132161688990890979766845703125f,0.102100000241399424538712992216460406780242919921875f,0.9521999939411873725703117088414728641510009765625f,-1.003600028151273360066397799528203904628753662109375f,-0.22290001026391959015882093808613717555999755859375f,-0.748100023479759901334773530834354460239410400390625f,0.9655000133514402005374677173676900565624237060546875f,-0.93950000181049075909101020442903973162174224853515625f,-1.54889998868703759882237136480398476123809814453125f,-0.316900021708011792043180321343243122100830078125f,0.76019999514818170638363881153054535388946533203125f,-0.87069996182173536869441932140034623444080352783203125f,-0.0510999301016337170722181326709687709808349609375f,0.00929996318817238876164310568128712475299835205078125f,0.3724999405443671918192194425500929355621337890625f,-1.5364000213950870232793022296391427516937255859375f,-0.50039999685287472885875104111619293689727783203125f,0.27170001001954080521727519226260483264923095703125f,-1.8112999660789963485285625210963189601898193359375f,-0.98340000297725194133846571276080794632434844970703125f,0.76259994139671416490955380140803754329681396484375f,-0.6094999559521678822449075596523471176624298095703125f,-0.481199945855141919537345529533922672271728515625f,0.1561000091150404056605793812195770442485809326171875f,-0.03190004078149666355557201313786208629608154296875f,-0.1300000053644183584111715390463359653949737548828125f,0.6727000023007381201978205353952944278717041015625f,-0.73439999846219894408250183914788067340850830078125f,1.362599997872113721797404650715179741382598876953125f,-0.5920000073611735302847591810859739780426025390625f,0.041699979323148728127534923260100185871124267578125f,-0.59009998461008006387373825418762862682342529296875f,1.988499965012073378289869651780463755130767822265625f,-1.03579996882081104558892548084259033203125f,-0.548799977999925747695897371158935129642486572265625f,-1.446399935042858686529143596999347209930419921875f,1.31159991156458932692885355209000408649444580078125f,-0.86069997780322982094958206289447844028472900390625f,0.326600057744979022089637510362081229686737060546875f,0.35299999375641366583522540167905390262603759765625f,-1.1095000396966934186337994105997495353221893310546875f,0.5003999878004192769509472782374359667301177978515625f,0.64979999859332959744051549932919442653656005859375f,1.081499958321452847798127550049684941768646240234375f,0.8119999543130405061930332522024400532245635986328125f,1.0101999991342420770479293423704802989959716796875f,-0.48070001304447684820075892275781370699405670166015625f,0.87219997449517305820876345023862086236476898193359375f,-0.597600067311525595670218535815365612506866455078125f,1.1405999511346223140861866340856067836284637451171875f,1.19189997208714526522044252487830817699432373046875f,-0.99139999069869533254717453019111417233943939208984375f,1.6404999687969681332333493628539144992828369140625f,1.0870999894380568573382106478675268590450286865234375f,1.2611999685049060104802265414036810398101806640625f,1.7690999669224023449487503967247903347015380859375f,0.336599987858534177576075308024883270263671875f,-1.7443999254465107906497678413870744407176971435546875f,1.38339995752871036671649562777020037174224853515625f,0.684099961148202595495604327879846096038818359375f,1.9848999453544617921352255507372319698333740234375f,0.4508999769374730792748096064315177500247955322265625f,0.29350002551078802071060636080801486968994140625f,-0.0572999903604387572642053783056326210498809814453125f,-0.25949995997548125359344339813105762004852294921875f,0.28459998496770833043001402984373271465301513671875f,-0.304800002467632058511526338406838476657867431640625f,0.26169996859431332580925300135277211666107177734375f,-0.71289997658729564733448569313623011112213134765625f,0.11280002863407023738773204968310892581939697265625f,-0.712099972227215882725204210146330296993255615234375f,0.494399966722727679524496124940924346446990966796875f,-0.3336999562859539469883429774199612438678741455078125f,0.182599952796102638075126378680579364299774169921875f,-1.05679996691346200776706609758548438549041748046875f,-0.52939997778683933749022116899141110479831695556640625f,-0.93209998899102242120306982542388141155242919921875f,0.660299973760545722001324975281022489070892333984375f,0.40409997996091906458104858756996691226959228515625f,-0.2666999798819429390306368077290244400501251220703125f,0.99890005306005402729851994081400334835052490234375f,0.824599999794364268979052212671376764774322509765625f,0.0175999375224105136794605641625821590423583984375f,0.61730000969469578375736773523385636508464813232421875f,-1.1031000501155858728452585637569427490234375f,0.41149996130168464691934104848769493401050567626953125f,0.42379996876120618054528677021153271198272705078125f,-0.6994999931678174132088088299497030675411224365234375f,0.532299958062171896244763047434389591217041015625f,-0.34830005472600411575712087142164818942546844482421875f,0.5134000116944310576627685804851353168487548828125f,1.4862000183120371854528229960124008357524871826171875f,-0.794199984240531620116598787717521190643310546875f,-0.5094000337779522880055083078332245349884033203125f,0.301199984598159886672874563373625278472900390625f,-0.547499994449318183598052200977690517902374267578125f,1.75119996671676592114863524329848587512969970703125f,1.171900012730061835242167944670654833316802978515625f,-0.183499985575675594873246154747903347015380859375f,0.13340000618100145146627255599014461040496826171875f,0.49579999951720221673667765571735799312591552734375f,0.92109993978738913256165687926113605499267578125f,-0.2915999692320827563207785715349018573760986328125f,-0.2577000017046930935293858055956661701202392578125f,-0.160300052773951762219439842738211154937744140625f,0.766899995183944582777257892303168773651123046875f,-0.694299994778632356684511250932700932025909423828125f,1.182200009810923546638150583021342754364013671875f,0.62380002255439759295541080064140260219573974609375f,-0.13290002337694151179903201409615576267242431640625f,-1.1802999787151815525021447683684527873992919921875f,0.900600003921984804122757850564084947109222412109375f,0.926900026178359315309762678225524723529815673828125f,-0.417299997624754059888374513320741243660449981689453125f,0.15649998909235041111287500825710594654083251953125f,-0.37810000015944267293122038608998991549015045166015625f,-0.636200014512241285302707183291204273700714111328125f,-0.118099994198977709114473100271425209939479827880859375f,-0.70320002697706229799479160647024400532245635986328125f,-0.56679998311847479275371597395860590040683746337890625f,0.4554000330626968295888445936725474894046783447265625f,-0.42169997589588203990018655531457625329494476318359375f,-1.491199969100951872036375789321027696132659912109375f,1.9004999731555585196218771670828573405742645263671875f,-0.32220002858638707721183891408145427703857421875f,-0.1084000052869320551707232880289666354656219482421875f,-1.2908999818921085278589089284650981426239013671875f,-1.9349999874085190842976089697913266718387603759765625f,-0.16979998622536651708969657192938029766082763671875f,0.734699983808397849571747428853996098041534423828125f,-0.02899999080598403278230534851900301873683929443359375f,0.42850000587850789468546963689732365310192108154296875f,-1.3901999291360385146987255211570300161838531494140625f,0.13760001322031045756233424981473945081233978271484375f,0.85749993145465897015355949406512081623077392578125f,1.391299932849407117174678205628879368305206298828125f,0.3465999568641191519446920210612006485462188720703125f,-0.679199971947073155575935743399895727634429931640625f,-0.010700002092123295227565904497168958187103271484375f,-0.36049996373057435761211308999918401241302490234375f,-0.525099995636939542720256213215179741382598876953125f,1.073299987111985043242157189524732530117034912109375f,0.425200011688470880244494765065610408782958984375f,0.20230000511258772188938337421859614551067352294921875f,-0.4894999900758267852296512501197867095470428466796875f,0.7058999743670224535918578112614341080188751220703125f,1.566899965530633931365400712820701301097869873046875f,0.9531000128999351783676274862955324351787567138671875f,-0.1286000245936218533415740239433944225311279296875f,-0.64419996851980687591066043751197867095470428466796875f,-0.39439996258765486203401451348327100276947021484375f,0.062399964334816260258520514980773441493511199951171875f,0.7273999860718849408414143908885307610034942626953125f,0.1677999949201940677045996608285349793732166290283203125f,0.655899997575580595565725161577574908733367919921875f,-0.3756000020593404098434575644205324351787567138671875f,-0.3943000258848072814288343579391948878765106201171875f,-0.1267999975353479857886895842966623604297637939453125f,-0.5284000056408346690517419119714759290218353271484375f,1.202599941992759990938566261320374906063079833984375f,-0.192699994328618284811227567843161523342132568359375f,0.135200008745491206951783169643022119998931884765625f,0.240099976079166388398533626968855969607830047607421875f,0.4375000039488075032068081782199442386627197265625f,0.0710999791264537162049919061246328055858612060546875f,-0.320899989417195552476869124802760779857635498046875f,-0.51969997985959093256269625271670520305633544921875f,-0.16699999372661122887251394786289893090724945068359375f,-0.4705999854445457319940260276780463755130767822265625f,-1.738699984747171356502803973853588104248046875f,-0.18250000020489043350835345336236059665679931640625f,1.79609994172453912852915891562588512897491455078125f,0.108699995103478830316134917666204273700714111328125f,-0.0038999716065830636324562874506227672100067138671875f,1.0106999654352666340173527714796364307403564453125f,-0.349700003794580493998722658943734131753444671630859375f,-0.152799988587200896716922216000966727733612060546875f,-0.444099969492853130503817737917415797710418701171875f,0.225699993747472849037194464472122490406036376953125f,-0.751299973119795350129379585268907248973846435546875f,0.00779999275952591997196350348531268537044525146484375f,-0.20449998556822546191824585548602044582366943359375f,-0.2603999978028237460847549300524406135082244873046875f,0.50699998597800710609106999982031993567943572998046875f,0.76960001432299574020134969032369554042816162109375f,0.4670000021085145558430440360098145902156829833984375f,1.1108999784648414532739479909650981426239013671875f,1.1215999778747558490721303314785473048686981201171875f,-0.276500035732984539293966008699499070644378662109375f,0.6522000042229894045675564484554342925548553466796875f,-0.8930000477731230290601160959340631961822509765625f,-0.88569999952167244305911708579515106976032257080078125f,-0.686299993348121528669025792623870074748992919921875f,-0.547900015218556024620966127258725464344024658203125f,0.2335999979764220579880884542944841086864471435546875f,1.038099966967106269066789536736905574798583984375f,-0.52799995329976123770165941095910966396331787109375f,-0.7637999811291695806403367896564304828643798828125f,-0.48460000471174691227815856109373271465301513671875f,-0.7859999897480012265305049368180334568023681640625f,-0.137999974161386429472031522891484200954437255859375f,-0.152200015696883017568552531884051859378814697265625f,-1.9617999646037820316024635758367367088794708251953125f,0.428299981263279949672551083494909107685089111328125f,-1.31319996454119714002217733650468289852142333984375f,0.707999999046325712015459430404007434844970703125f,0.8472000048190349019705536193214356899261474609375f,-1.3478000174582007719692455793847329914569854736328125f,0.3919000410795216993165013263933360576629638671875f,1.05910000216364874603414136799983680248260498046875f,-0.09590001229047828701368416659533977508544921875f,-0.6599000323772425957713494426570832729339599609375f,0.91520003627538670798458042554557323455810546875f,1.156999982774257862416789066628552973270416259765625f,0.539400015896558837624752413830719888210296630859375f,1.4455000292956829976986909969127736985683441162109375f,0.7735000055432319499004734097979962825775146484375f,0.43150006106495908397846505977213382720947265625f,-1.008500014334916983216317021287977695465087890625f,0.104899992889165627474312714184634387493133544921875f,0.1612999806821340254003871450549922883510589599609375f,0.50640000045895583458133160092984326183795928955078125f,-1.2137999939441679320140110576176084578037261962890625f,0.6224999786913396615517513055237941443920135498046875f,-0.14059998827576603019906542613171041011810302734375f,1.276699991840124059905292597250081598758697509765625f,0.26159999076426021513697151021915487945079803466796875f,0.0361000183165074428615071155945770442485809326171875f,-1.009300027859211024150454250047914683818817138671875f,-0.86009999474287013310203064975212328135967254638671875f,1.3138000026613472215331057668663561344146728515625f,1.2823000018715855485851307093980722129344940185546875f,0.2546000088095663471676743938587605953216552734375f,-0.691200008291005918437122090836055576801300048828125f,0.7358000312566754796961276952060870826244354248046875f,0.718599981510639196358170011080801486968994140625f,-0.1993999643385411246754301828332245349884033203125f,-0.174400004422664789416330677340738475322723388671875f,0.96540000974237905051467123485053889453411102294921875f,-1.1429999933838843606537238883902318775653839111328125f,1.2124000116646287406041437861858867108821868896484375f,-0.463600009971857129897898630588315427303314208984375f,-0.3886000088542704844485342619009315967559814453125f,1.49670008243918406520833741524256765842437744140625f,-0.04089997712373616423064959235489368438720703125f,0.83190006447434416969599624280817806720733642578125f,1.392400046384335343674365503829903900623321533203125f,1.02920004250407171042525078519247472286224365234375f,-0.91529996709227479101400604122318327426910400390625f,-0.0172999599993222119564961758442223072052001953125f,1.126199973720310776315045586670748889446258544921875f,1.315199941205978717562175006605684757232666015625f,-0.0503999433577053679300661315210163593292236328125f,1.2950000190734858307450849679298698902130126953125f,-1.280700059461593820486768890987150371074676513671875f,-0.26339995991289610888230754426331259310245513916015625f,0.3420000182390214860106425476260483264923095703125f,-0.4308999663949013569919088695314712822437286376953125f,-1.2767000279009341756619733132538385689258575439453125f,-0.00449996049702161560190916134160943329334259033203125f,0.505600031936168203827719480614177882671356201171875f,-0.0160000235736366125394170012441463768482208251953125f,-0.958399971684813056782559215207584202289581298828125f,1.228499986022710377397970660240389406681060791015625f,0.370299954426287936115613774745725095272064208984375f,-0.463999984845518742559988822904415428638458251953125f,-0.1124999422579993080262283910997211933135986328125f,-0.5579000279963022279616779997013509273529052734375f,0.89219997285604524250857139122672379016876220703125f,-0.218300003838539424805276212282478809356689453125f,1.3356000013887887600816384292556904256343841552734375f,-0.6994000126183035348503835848532617092132568359375f,-0.100400000429153646308577663148753345012664794921875f,1.17379994655847585960373180569149553775787353515625f,1.512000001624226097618475250783376395702362060546875f,-1.03789996377229698509836453013122081756591796875f,0.649799991142750332784316924517042934894561767578125f,0.0220999733448028479187996708787977695465087890625f,1.0551999959677456075723966932855546474456787109375f,-0.15500000678002834320068359375f,-0.71370001469850574693509770440869033336639404296875f,-0.65370005418658205798010385478846728801727294921875f,-1.225500001877545486195231205783784389495849609375f,0.25929998062252934687421657145023345947265625f,-1.30080002037882724863493422162719070911407470703125f,-1.69499998107552496406924547045491635799407958984375f,-0.616800011020899319191812537610530853271484375f,-0.697500060126185150721767058712430298328399658203125f,-1.667599996605514167669070957344956696033477783203125f,-1.26040003149807500193446685443632304668426513671875f,-0.20689999294876937341314260265789926052093505859375f,0.8089999901726845354943407073733396828174591064453125f,0.906900016343593851075866041355766355991363525390625f,0.66899998007714744918672522544511593878269195556640625f,1.4449999649822711234037342364899814128875732421875f,-0.5854999650716783410331345294252969324588775634765625f,-0.597799972307682114802673822850920259952545166015625f,-0.55410002093911125964353914241655729711055755615234375f,0.628699972453713673559150265646167099475860595703125f,0.398099984848499344192163107436499558389186859130859375f,0.927499980852008132359287628787569701671600341796875f,0.14800001752376534813748776286956854164600372314453125f,0.070200009751320013862141422578133642673492431640625f,0.4051999563306571960907831453368999063968658447265625f,-1.247399990653991874722805732744745910167694091796875f,0.341600017064809691191840101964771747589111328125f,0.144199965912104488552358816377818584442138671875f,-0.40219996510744149276206371723674237728118896484375f,0.018199980485439137822822885937057435512542724609375f,1.286699956971407488026670762337744235992431640625f,-1.173599966236948954900753960828296840190887451171875f,0.31960001871883836344068186008371412754058837890625f,-1.000399995437263545028372391243465244770050048828125f,0.963400010055302846723179754917509853839874267578125f,0.955399986720085525604417853173799812793731689453125f,0.67900004169344985172074302681721746921539306640625f,-0.65230001721978236872701018000952899456024169921875f,-0.9062000093340873974057103623636066913604736328125f,-0.7733000321507450536273609031923115253448486328125f,-0.1099999845027923583984375f,0.5012000155925750988217259873636066913604736328125f,0.64900000756978926119700190611183643341064453125f,-1.133000026762485124010026993346400558948516845703125f,-2.1078000289320950599858406349085271358489990234375f,-1.478400024509429755426026531495153903961181640625f,-0.539600032651424754703839425928890705108642578125f,0.7226000419795521878540967009030282497406005859375f,-0.5970000284463166106974085778347216546535491943359375f,0.6035999664604665415623685476020909845829010009765625f,-0.6202000211507083804463036358356475830078125f,0.620300041225552600820947191095910966396331787109375f,0.161900024092197281078142623300664126873016357421875f,-0.05040005556345050763411563821136951446533203125f,-1.137500005811453274162658999557606875896453857421875f,0.04840000521242604492044847575016319751739501953125f,-0.5717999931395052470861628535203635692596435546875f,1.36000002764165461854872773983515799045562744140625f,0.122500013858080070150435858522541821002960205078125f,-1.265800009351969013238203842774964869022369384765625f,-0.43620000158548322133356123231351375579833984375f,-1.4329999696910380446723820568877272307872772216796875f,0.948400008714198605730416602455079555511474609375f,0.878399985244870062928157494752667844295501708984375f,-0.584099983239173781157660414464771747589111328125f,-0.158200039494037270060289301909506320953369140625f,-0.23239998677968998208598350174725055694580078125f,-0.86340004410445647664573698421008884906768798828125f,0.9239999959468843115928393672220408916473388671875f,-0.6084000087887044827539284597150981426239013671875f,0.570000011622905589092624722979962825775146484375f,-0.004999984651803845991935304482467472553253173828125f,-0.758299970236420595171011882484890520572662353515625f,-1.113800031122565314944949932396411895751953125f,-0.39229999687969741017923297476954758167266845703125f,1.51200006860494706728559322073124349117279052734375f,0.548200028467178857027874983032234013080596923828125f,0.9915000288784503457151231486932374536991119384765625f,0.0082999815613031824312884054961614310741424560546875f,-0.11960002214610609971856547417701222002506256103515625f,-0.3661000027820466851125047469395212829113006591796875f,1.734599992120264833062037723721005022525787353515625f,0.926900029084086529707064983085729181766510009765625f,-1.49240002820491834967242539278231561183929443359375f,-0.6555999891698365278358551222481764853000640869140625f,-0.039500011086464059673062365618534386157989501953125f,-0.59069996819198100634906722916639409959316253662109375f,-1.499300016683340341927532790577970445156097412109375f,0.297299986150860728884026684681884944438934326171875f,0.13219999192953102440384327564970590174198150634765625f,0.37560000578463093034997655195184051990509033203125f,0.2685000368729235642462072064517997205257415771484375f,0.090400000913441136418668975238688290119171142578125f,-0.5486999895900483981137085720547474920749664306640625f,-0.0268000043898819484411433222703635692596435546875f,0.8405000073164696772209936170838773250579833984375f,-0.2315999688595535843660400132648646831512451171875f,1.723799998116493537025917248683981597423553466796875f,0.065399995809793498580120285623706877231597900390625f,0.031999992161989343486538928118534386157989501953125f,0.142199969279765969787376889144070446491241455078125f,-1.2668000083386896648107722285203635692596435546875f,-0.5000999947056172789672245926340110599994659423828125f,-0.20420001568198220542882381778326816856861114501953125f,0.0332000080153347365552463088533841073513031005859375f,-0.138799980086088847741621066234074532985687255859375f,-1.048399993887543235615567027707584202289581298828125f,-0.49319997728168996076192343025468289852142333984375f,-1.2265000054836274756553393672220408916473388671875f,-0.47940003362894090255252876886515878140926361083984375f,-0.78430001254826808843034768869983963668346405029296875f,-0.02849997466057530015604015716235153377056121826171875f,0.98330001454874882416135051244054920971393585205078125f,-0.66270000364184322716454289547982625663280487060546875f,-0.25629999237954559365704199080937542021274566650390625f,0.217299993526935519838616528431884944438934326171875f,0.735799995642900217518445060704834759235382080078125f,0.6831000346183782045983434727531857788562774658203125f,-0.30060000280439902464735268949880264699459075927734375f,1.0876000544592743235483567332266829907894134521484375f,-0.211899989819527068135585068375803530216217041015625f,-0.04979996819496157467455077494378201663494110107421875f,0.02100001161545517192053011967800557613372802734375f,0.6540999878771602826788011952885426580905914306640625f,-0.2903999979145821530579496538848616182804107666015625f,-0.1548999957948924421913261539884842932224273681640625f,-0.2323000031754374195003265413106419146060943603515625f,-0.3359999962300059817010833285166881978511810302734375f,-0.044999985881149895394770510392845608294010162353515625f,-1.077300008651614238175397986196912825107574462890625f,0.3915999956443905194447552275960333645343780517578125f,-0.1403000145897268513550670832046307623386383056640625f,-0.1842999785363677034411011845804750919342041015625f,0.4337999791175126684805718468851409852504730224609375f,0.09640000386759661754609851413988508284091949462890625f,0.049399999766051860827786867957911454141139984130859375f,-0.08650001034513110642620858925511129200458526611328125f,0.833999979928135903861630140454508364200592041015625f,0.17119999644458283594161684959544800221920013427734375f,1.15479998078644285897098598070442676544189453125f,-0.3751999868035318375092401765868999063968658447265625f,-0.4839999934881922261098452509031631052494049072265625f,0.68439997283220288526450758581631816923618316650390625f,-0.03499997150152889258123423132929019629955291748046875f,0.4901000005543227899806879577226936817169189453125f,-1.09930001832246748705301797599531710147857666015625f,-0.08230001081228266457401332445442676544189453125f,-0.3955000431537631921941056134528480470180511474609375f,-0.59909999188184759333353213150985538959503173828125f,-0.8727999791622149672321029356680810451507568359375f,-1.954400008445978276228061076835729181766510009765625f,0.180600020760298018984713053214363753795623779296875f,-0.2128000301241872449509173748083412647247314453125f,0.71560005011558569520957462373189628124237060546875f,1.3548000042557719613256494994857348501682281494140625f,-1.00110005691051551224290960817597806453704833984375f,0.8828999707758422577086321325623430311679840087890625f,-0.22420001510456166737839112101937644183635711669921875f,0.44390000183731326277580819805734790861606597900390625f,0.307699967879057123543162788337212987244129180908203125f,-0.79139995867982493304992885896353982388973236083984375f,0.4846999777734282144336930286954157054424285888671875f,-0.4122999307930488299689386622048914432525634765625f,-0.1157000174403182146676272168406285345554351806640625f,0.812000000655650477909830442513339221477508544921875f,0.3621999990820885528819417231716215610504150390625f,0.232100014919041530703225362231023609638214111328125f,1.329199966508150421162781640305183827877044677734375f,-0.72669996330440056464539111402700655162334442138671875f,-0.2733999848723416281615072875865735113620758056640625f,0.1411000079974530085902273413012153469026088714599609375f,0.39970002888441058530588634312152862548828125f,0.79659997470825982812669963095686398446559906005859375f,-0.1342000628441570864168852494913153350353240966796875f,0.32069999341219646016298838731017895042896270751953125f,-0.476900020031631177719333436471060849726200103759765625f,0.74469999304711798071565453938092105090618133544921875f,0.051599979364871817466564607457257807254791259765625f,0.1932000119641425761596309484957600943744182586669921875f,-0.37930000073909742042133075301535427570343017578125f,-0.8396999778851874651763864676468074321746826171875f,-0.0748999612987051310852848473587073385715484619140625f,1.56850001791119542104979700525291264057159423828125f,0.4855999365687377888889386667869985103607177734375f,0.069899972921609876408410855219699442386627197265625f,0.662499970346689170952458880492486059665679931640625f,-1.211499986745417256628343238844536244869232177734375f,1.2887000110104676053879302344284951686859130859375f,0.910200017611682454798938124440610408782958984375f,0.419099988007545665169573112507350742816925048828125f,-0.5282999950841069658480364523711614310741424560546875f,0.56100000673532424144696051371283829212188720703125f,-0.1218999340146798004980155383236706256866455078125f,0.5090999938189988416326059450511820614337921142578125f,1.2183999690026035001011450731311924755573272705078125f,0.450699972736835707820546303992159664630889892578125f,0.57049997669458463178671081550419330596923828125f,-0.2023999670356515512281703195185400545597076416015625f,-0.4772999728888276393945488962344825267791748046875f,-0.4638999859675763115518520862678997218608856201171875f,0.0949999906867737831106524026836268603801727294921875f,-0.88649999266862844482517402866506017744541168212890625f,-0.0376999854251740540433956994093023240566253662109375f,0.6663000143587585188953426040825434029102325439453125f,-0.315100001187622158482781742350198328495025634765625f,-1.217700034785270446491267648525536060333251953125f,0.296600004845857245783236066927202045917510986328125f,-0.6546999956548216204055279376916587352752685546875f,0.2302000001028183806539573197369463741779327392578125f,0.726299993199109650277023320086300373077392578125f,-0.91970000243484928414972046084585599601268768310546875f,-0.1960000126957890387302541057579219341278076171875f,-0.24450001659989395452754479265422560274600982666015625f,-0.15809997993111579717151471413671970367431640625f,-0.2345999986395241876380168832838535308837890625f,-0.220799999368190835724590215249918401241302490234375f,-0.336400001427531680775473432731814682483673095703125f,1.34639999017715439322273596189916133880615234375f,0.584200025628506924846305992105044424533843994140625f,1.36859995439052628540821388014592230319976806640625f,0.512199996548890812420040674624033272266387939453125f,-0.59919999407529811463746227673254907131195068359375f,-0.0267999985039237831330183325917460024356842041015625f,-0.94039995661974007390426777419634163379669189453125f,-1.2471999692052602171088437899015843868255615234375f,0.531499996244907180198424612171947956085205078125f,1.441499994866550338201705017127096652984619140625f,0.2119999821782112547907672706060111522674560546875f,0.37220000325441304767082328908145427703857421875f,-0.1235999809145926775499901850707828998565673828125f,0.121800000143050368706099106930196285247802734375f,-0.058600015783309533645706324023194611072540283203125f,-0.60869996686577820810271077789366245269775390625f,-0.259200001972913707248835635255090892314910888671875f,-1.39739994043707849158408862422220408916473388671875f,0.35390001595616293883495018235407769680023193359375f,-0.20710000821352014810372565989382565021514892578125f,0.52280000181198094111323371180333197116851806640625f,-0.209100028538703242020346806384623050689697265625f,0.2021999879062175153876523836515843868255615234375f,0.168199986110627863755695443614968098700046539306640625f,1.1472999973267314910430059171631000936031341552734375f,0.33570000056475401972733152433647774159908294677734375f,0.9234000118806957591033324206364341080188751220703125f,0.22940000095814472302890862920321524143218994140625f,0.6678000158563259791577593205147422850131988525390625f,0.61709998992234460502004367299377918243408203125f,0.66920002164244696274408852332271635532379150390625f,-0.4053999904453748914789912305423058569431304931640625f,-1.049800011073052541377137458766810595989227294921875f,0.182199972818791577910957357744337059557437896728515625f,0.22149999525770536212831984812510199844837188720703125f,1.382799991790950144832095247693359851837158203125f,-0.307599990421533409090670829755254089832305908203125f,-0.41509997682422417053516028317972086369991302490234375f,-0.7650999822258948146469492712640203535556793212890625f,-0.22189999648779645813334582271636463701725006103515625f,-0.4555999893188478200301005927030928432941436767578125f,0.4790999984011055090604713768698275089263916015625f,-1.2755000348091112272186364862136542797088623046875f,-0.4974000459372998061979842532309703528881072998046875f,0.4991999769389625640769736492075026035308837890625f,0.669999998696148235666214532102458178997039794921875f,1.188799965780973355577998518128879368305206298828125f,0.30929997864812597274664085489348508417606353759765625f,-0.2397000136107203527302544898702763020992279052734375f,-0.10049998496472856146510821417905390262603759765625f,-0.40230000518709407852924186954624019563198089599609375f,0.522399997992813336367134979809634387493133544921875f,-0.3307999796986582641267204962787218391895294189453125f,1.4468999469414354219765073139569722115993499755859375f,-0.5218000069975854415815774700604379177093505859375f,0.9508999923229222162035512155853211879730224609375f,1.245000019669532331789696399937383830547332763671875f,0.530899970678984534089295266312547028064727783203125f,-0.19279999085962773452962437659152783453464508056640625f,0.73010000007003572619623810169287025928497314453125f,0.38969999043941516703171146218664944171905517578125f,0.7120000094845890803441079697222448885440826416015625f,-0.80550006073713387166890242951922118663787841796875f,-0.7215999936699859507172050143708474934101104736328125f,-0.76050000342354084903462307920563034713268280029296875f,-0.44530002520680478372838706491165794432163238525390625f,0.61370001959726205864598114203545264899730682373046875f,-0.155199981308728585105427555390633642673492431640625f,-0.66449999544024507258654921315610408782958984375f,-0.84360000233501153399373606589506380259990692138671875f,-0.35180000997781757376969835604541003704071044921875f,1.007099999235570475519807587261311709880828857421875f,-0.052800021183490031262408592738211154937744140625f,-1.846899990303814576009244774468243122100830078125f,-0.2180000008344649131686310283839702606201171875f,-0.29600003504753147343109276334871537983417510986328125f,-0.35829995615482357340653152277809567749500274658203125f,1.99510001672208314715817323303781449794769287109375f,-1.102199987459183017080022182199172675609588623046875f,0.07619993559196647225917331525124609470367431640625f,0.0090999626755721152449041255749762058258056640625f,-0.402299984250963138521228756872005760669708251953125f,0.19020000021457672545466266456060111522674560546875f,-0.2616999890089033442563959397375583648681640625f,-0.64380001212358450857209390960633754730224609375f,0.1932000305533403405888748238794505596160888671875f,-1.188899972891807266250907559879124164581298828125f,0.287300012600422149944279226474463939666748046875f,0.1386000448591999667513618987868539988994598388671875f,0.979799927160888817212480716989375650882720947265625f,0.05249999776482561220092293297057040035724639892578125f,1.04279995708987183178351187962107360363006591796875f,1.646599943900108886651878492557443678379058837890625f,-0.9806999643921852083394696819595992565155029296875f,0.4034999933242797709453952847979962825775146484375f,0.0269999916777012016755321610617102123796939849853515625f,1.3202000198096037753003884063218720257282257080078125f,-0.36099995983392008103152193143614567816257476806640625f,-0.6779999690949918278448649289202876389026641845703125f,0.5680999869719147721269791873055510222911834716796875f,0.8656999738916748565742409482481889426708221435546875f,0.39419999675750727607947965225321240723133087158203125f,0.98640001970008039933190957526676356792449951171875f,0.35240000697076301339194515094277448952198028564453125f,-0.768100046874583330946961723384447395801544189453125f,0.4205999942548575010192735135206021368503570556640625f,-0.6566000260055062653918866999447345733642578125f,0.8896000446841123565633324687951244413852691650390625f,-1.3862999541580676154950424461276270449161529541015625f,-1.1235000213012094061326706651016138494014739990234375f,0.3281999958336354605847873244783841073513031005859375f,1.1274999783188104363063075652462430298328399658203125f,0.5615000051856029461561092830379493534564971923828125f,0.8675000004842881029532009051763452589511871337890625f,-0.7728999845594162199091670117923058569431304931640625f,-1.41409998823106253240666774217970669269561767578125f,0.3729000032231206684940616469248197972774505615234375f,-0.66910002250373334931055069318972527980804443359375f,1.1530000163316724837159199523739516735076904296875f,-0.7606000162154433752448312588967382907867431640625f,-1.454700002360343802365605370141565799713134765625f,-0.1011999878764144877862918292521499097347259521484375f,0.6530999754607680340967590382206253707408905029296875f,1.2106999494165180397686754076858051121234893798828125f,0.13670002488344901170336243012570776045322418212890625f,0.0946999732658273529750658781267702579498291015625f,-0.3145000263601540613223050968372263014316558837890625f,-0.029899996931105565778352683992125093936920166015625f,-0.6382999683737755614032494122511707246303558349609375f,0.871199957235903266195009564398787915706634521484375f,-0.7687999715924263011146422286401502788066864013671875f,0.5308999960109590521284417263814248144626617431640625f,0.272599987441301205848276367760263383388519287109375f,0.529900003543496200819618024979718029499053955078125f,0.354399993880093244769113880465738475322723388671875f,-0.67099998032301655381814953216235153377056121826171875f,-1.63069999441802426787262447760440409183502197265625f,-1.2394999811723834692855916728149168193340301513671875f,-0.2961999902606020640405404265038669109344482421875f,-0.7361999479785563682554538900149054825305938720703125f,-1.52969999073743867512575889122672379016876220703125f,-0.299900003468990039579011863679625093936920166015625f,0.0495000103414060532713847351260483264923095703125f,0.453800015141069490454128754208795726299285888671875f,0.5387000063538547323105376563034951686859130859375f,0.9576999808430672889159041005768813192844390869140625f,-1.29589998174309695144756915397010743618011474609375f,-0.639000023551285156742096660309471189975738525390625f,0.22580000093281216155816082391538657248020172119140625f,0.3432000332728029068363184705958701670169830322265625f,-0.82499994248151864439932978712022304534912109375f,-1.2751000272646544342336483168764971196651458740234375f,1.14039999268054970116281765513122081756591796875f,0.73370005139261440518794188392348587512969970703125f,0.9477999696254730821465273038484156131744384765625f,0.68689999130964185525272114318795502185821533203125f,-1.181799964380264356123007019050419330596923828125f,0.74839997429251692295792963705025613307952880859375f,-0.721999991714954436616835664608515799045562744140625f,0.496399994163215563247604222851805388927459716796875f,1.953699988919496366435168965836055576801300048828125f,-1.07409997619837582760737859643995761871337890625f,-0.17690003601312564995851062121801078319549560546875f,-0.882699943888187732454753131605684757232666015625f,0.9647999562740334766886007855646312236785888671875f,-0.6902999800562863441655281349085271358489990234375f,-0.0389999868571762053903739797533489763736724853515625f,-0.27669994993060864629796924418769776821136474609375f,0.21670005286037896663486890247440896928310394287109375f,-1.2244999736100432574659180318121798336505889892578125f,-0.9258999741435045560677963294438086450099945068359375f,-0.9381999834656717940362113949959166347980499267578125f,-0.02769998054504441142853465862572193145751953125f,-0.8534999880716200859609443796216510236263275146484375f,-0.53750000439584277867055561728193424642086029052734375f,-1.265800019559263578372565461904741823673248291015625f,-0.8768000144481653546080224259640090167522430419921875f,-0.162900015218556404317240549062262289226055145263671875f,0.54810001301169375409472195315174758434295654296875f,-0.58279996765106911738030248670838773250579833984375f,0.53579998204559087238152415011427365243434906005859375f,0.4985000029169024404751553447567857801914215087890625f,1.9239999772831801738703916271333582699298858642578125f,-0.75650000067800304837106750710518099367618560791015625f,0.467999948531389264871904742904007434844970703125f,-0.01270002062916741891740457504056394100189208984375f,-1.11600000204145910487341097905300557613372802734375f,-0.39479999918937647152006320538930594921112060546875f,0.0873999772429463472889210606808774173259735107421875f,-0.74539999253153776948011000058613717555999755859375f,-0.412400006747245750915453754714690148830413818359375f,1.647299968343972853546119949896819889545440673828125f,-0.138300058078767218461280208430252969264984130859375f,0.301800009009241865243211577762849628925323486328125f,-1.852900034701824072413955946103669703006744384765625f,-1.0709999825954437113750827847979962825775146484375f,-0.986400007611512474881010348326526582241058349609375f,-1.2757000239968296906312161809182725846767425537109375f,-0.0964999966174364676163577314582653343677520751953125f,0.2334999572262164146962959421216510236263275146484375f,-0.2685000057294966158139004619442857801914215087890625f,-0.992300040204823208256357247591949999332427978515625f,-0.1791000167667868847587442360236309468746185302734375f,0.8102999910086392532093668705783784389495849609375f,1.29840004186928315021987145883031189441680908203125f,-0.9641000015676020051813566169585101306438446044921875f,0.93349997335672352249957839376293122768402099609375f,-1.439899998402595304725082314689643681049346923828125f,-0.462599987813830271221604562015272676944732666015625f,0.26679998539090143960805789902224205434322357177734375f,0.64650001010298741022097601671703159809112548828125f,0.58909999277591695854283670996665023267269134521484375f,-0.52330000659525399697713510249741375446319580078125f,0.59139999639838947143033465181360952556133270263671875f,0.570700030404329705646659931517206132411956787109375f,-1.029500001624226168672748826793394982814788818359375f,-0.5968000387370591131031005716067738831043243408203125f,-0.561799971905350137291179635212756693363189697265625f,0.13179999239444750713801113306544721126556396484375f,0.739699998784065115842167870141565799713134765625f,-0.03760000629127002724771955399774014949798583984375f,1.1319999891817567050367188130621798336505889892578125f,-0.9478000082194804054580572483246214687824249267578125f,-0.066099988774955154013923674938268959522247314453125f,-0.254199975970387459511812267010100185871124267578125f,0.4272999883860351122422116532106883823871612548828125f,0.680200006622075914464176094043068587779998779296875f,-0.404200038778782300141756422817707061767578125f,-0.4068999993562698591631487943232059478759765625f,-0.585700033831596300615274230949580669403076171875f,-0.683599995964765838607490877620875835418701171875f,-0.252000060260295644098960110568441450595855712890625f,0.877200034397840777700139369699172675609588623046875f,1.1240999763846399872591064195148646831512451171875f,1.39440003437399884234082492184825241565704345703125f,0.88490003237724312867840126273222267627716064453125f,-0.71020003120899222182060839259065687656402587890625f,0.281800018322467593634428340010344982147216796875f,-0.32769999678730954428829136304557323455810546875f,0.000799983796477388153789433999918401241302490234375f,0.54820003137290473915754773770458996295928955078125f,0.78199998337030418582571655861102044582366943359375f,0.4191999973535533996482627117075026035308837890625f,0.796999981582164895854702990618534386157989501953125f,-0.899100009167194347270424259477294981479644775390625f,0.9341000228762628143641677525010891258716583251953125f,-0.8694999927580351339884146000258624553680419921875f,0.155999992281198540666764529305510222911834716796875f,0.9813000197976828875567889554076828062534332275390625f,-0.144499982699751949866140421363525092601776123046875f,-0.62080002343356621707926024100743234157562255859375f,0.737200000870227967197934049181640148162841796875f,-1.22339999627172968388322260580025613307952880859375f,0.45379999155998262949651689268648624420166015625f,-0.27439998683929456291252790833823382854461669921875f,0.345699995684623839764526564977131783962249755859375f,1.5990000004172326697471362422220408916473388671875f,-0.84379999692440055270026277867145836353302001953125f,-0.26519999201893806883845172706060111522674560546875f,0.01119998281002043682974544935859739780426025390625f,0.5070000095218418767473167463322170078754425048828125f,-0.3903999834418296899229972041212022304534912109375f,-0.34379997688233832064952366636134684085845947265625f,0.1916999867364757026422239505336619913578033447265625f,-0.5880000117123120872975050588138401508331298828125f,-0.1977999800562864063380175139172933995723724365234375f,0.2815999993324280747941656954935751855373382568359375f,0.11960002318918705466188612263067625463008880615234375f,-0.2463000347733501627089935936965048313140869140625f,-0.36549995546042879368542344309389591217041015625f,0.1962000227451330403027895954437553882598876953125f,-0.9274999526143066219674437888897955417633056640625f,-0.6761000180184841834574172025895677506923675537109375f,-0.0380999722570172227875673343078233301639556884765625f,-1.5694000399619341123980120755732059478759765625f,-0.7276999847546221644734032452106475830078125f,-0.1664000015765429996150714941904880106449127197265625f,-0.69759998051226135284963447702466510236263275146484375f,-0.230900011694431361863877327778027392923831939697265625f,-0.65699999368935813048864247321034781634807586669921875f,0.56340000059306649671242439580964855849742889404296875f,-1.251900001777708215655593448900617659091949462890625f,-0.586800040972233372116306782118044793605804443359375f,-0.626299987462162643936380845843814313411712646484375f,1.2072000300019982699240017609554342925548553466796875f,-0.7370000209957361381185592108522541821002960205078125f,0.88269999943226584715461058294749818742275238037109375f,-0.0489999673739076602174691288382746279239654541015625f,0.9057999677032235741336307910387404263019561767578125f,0.0494999948441983139701960681122727692127227783203125f,0.272799980503320771418884760350920259952545166015625f,-0.33019999917149533974480846154619939625263214111328125f,-0.42140000656843223936220965697430074214935302734375f,-0.2513999902516601547830532581428997218608856201171875f,0.214400013959407775843146737315692007541656494140625f,-0.1319000175356865867115629953332245349884033203125f,-0.791800022718310625435833571827970445156097412109375f,0.3901000096812843320748243058915250003337860107421875f,-0.23780000315308569280858819183777086436748504638671875f,0.21719993917942037597867965814657509326934814453125f,-0.29099998110532698802899176371283829212188720703125f,-1.12490000674724566209761178470216691493988037109375f,1.3614000482916832623914160649292171001434326171875f,-0.6382000692963600840812432579696178436279296875f,1.5422000432640317058030632324516773223876953125f,1.41970006699413087147831902257166802883148193359375f,0.000299974095821387010118996840901672840118408203125f,-0.3378999596372249669684606487862765789031982421875f,-1.067799995255469536203918323735706508159637451171875f,-0.3137999757647520482350955717265605926513671875f,0.55790004036426577016527517116628587245941162109375f,-0.1062000085890293377133275498636066913604736328125f,-0.8857000228792435425617668443010188639163970947265625f,-0.1732000266790389986226728069595992565155029296875f,-1.0880000264644620333598368233651854097843170166015625f,1.6203000173091890445675744558684527873992919921875f,0.82520002439171147301522069028578698635101318359375f,0.22820002632513680662640354057657532393932342529296875f,0.102399974299967111424081167569966055452823638916015625f,-1.1358999849282211958012567265541292726993560791015625f,0.2329000160008665165634056393173523247241973876953125f,-0.313699974390864166906567334081046283245086669921875f,-0.477799996522068948223704865085892379283905029296875f,-0.45289996019601819199351666611619293689727783203125f,-0.4298000067144631675120081126806326210498809814453125f,1.01609996869564067623059600009582936763763427734375f,-1.0165999671459193809397447694209404289722442626953125f,-0.1724000010848047992340070777572691440582275390625f,-0.987999990217387935587112224311567842960357666015625f,-1.3940999970972536292634913479560054838657379150390625f,0.196799983900785679935552252572961151599884033203125f,-0.28909996830076034601830770043306984007358551025390625f,0.9697999819785361186319505577557720243930816650390625f,0.7115999713182450392423561424948275089263916015625f,0.105199995967745874025922603323124349117279052734375f,0.987299985778330668750868426286615431308746337890625f,1.474400018578767568300236234790645539760589599609375f,0.20559999535381745516815499286167323589324951171875f,-0.1626999957442283317732290015555918216705322265625f,0.280300025877356251413630161550827324390411376953125f,-2.00580010486841242567379595129750669002532958984375f,-0.868800007578731214863410059479065239429473876953125f,0.19350002372264951677749422742635942995548248291015625f,-0.06889996048659152716453490938874892890453338623046875f,0.08149999136477681371815151578630320727825164794921875f,-1.2305000388510531905694733723066747188568115234375f,-0.2358999988794321378549057044438086450099945068359375f,0.2803000030040740941927879248396493494510650634765625f,-1.70530003861784962282399646937847137451171875f,0.55580000950098007450606019119732081890106201171875f,1.332799989816546126775165248545818030834197998046875f,-0.8996000008374454015580568011500872671604156494140625f,0.9841000318169601523976552925887517631053924560546875f,-1.0865999760493638692793183508911170065402984619140625f,-0.8307000279456382418885596052859909832477569580078125f,-0.52659999165833004663994643124169670045375823974609375f,-0.2807000142924482855022461080807261168956756591796875f,0.3499999958090482810035837246687151491641998291015625f,1.28620000769495934633823708281852304935455322265625f,0.002700004014373202920751282363198697566986083984375f,0.6848000307053327961881450391956605017185211181640625f,-0.640599999898672223253015545196831226348876953125f,-1.2224000252246856046411949137109331786632537841796875f,-0.109699989023804622689794996404089033603668212890625f,1.045599973523616998960505952709354460239410400390625f,0.16530004413127874496325375730521045625209808349609375f,0.4397999937504533107812676462344825267791748046875f,0.26529997549280526225601306578028015792369842529296875f,-1.575399970813095240629309046198613941669464111328125f,0.72760000522956225221804515967960469424724578857421875f,-1.680600021654367282053499366156756877899169921875f,-0.99270003255009608977843527100048959255218505859375f,-0.019500025838614210016430661198683083057403564453125f,-0.372800039511918868129214388318359851837158203125f,1.1263000283658508582362856031977571547031402587890625f,-0.2919999788254499240025552353472448885440826416015625f,-0.2007000182598834836511514367884956300258636474609375f,0.5046000219598416602906354455626569688320159912109375f,1.90020002077817817820459822542034089565277099609375f,0.988499926269053474925385671667754650115966796875f,-0.25919999169111296311029946082271635532379150390625f,0.35119996850490498019325968925841152667999267578125f,-0.4328000007689001282784602153697051107883453369140625f,0.01529996597468874508507497012033127248287200927734375f,0.313199993896484674138491755002178251743316650390625f,0.265399996033310880960698341368697583675384521484375f,-1.0614000225871802474131300186854787170886993408203125f,0.2393000187695026870215997405466623604297637939453125f,0.4841000700011850454984596581198275089263916015625f,0.4199999823048710112516346271149814128875732421875f,-1.482100018122792572938806188176386058330535888671875f,1.26089997846484092036689617088995873928070068359375f,0.7156000346928834687787457369267940521240234375f,0.782900002962351404534047105698846280574798583984375f,0.3285999907493593052976166291045956313610076904296875f,0.8293999965250495876034619868732988834381103515625f,-0.618999986410141200821044549229554831981658935546875f,-0.41429997272491458915055773104541003704071044921875f,0.034600018569827195591415147646330296993255615234375f,-0.3892000089392066985993778871488757431507110595703125f,0.33420000286698281488639850067556835711002349853515625f,0.2837999890640381384088186678127385675907135009765625f,0.015200000736117491584309391328133642673492431640625f,-0.226699980962276725904303020797669887542724609375f,-0.52479998928010473946415004320442676544189453125f,0.27859997286796644999640193418599665164947509765625f,0.2188000231504438897189857016201131045818328857421875f,-1.4806000007927426764808842563070356845855712890625f,1.90150003230571673640270091709680855274200439453125f,1.713099995800852592964247378404252231121063232421875f,0.998900030782818415531210121116600930690765380859375f,0.4945000195056206138133347849361598491668701171875f,2.144700040954350850341825207578949630260467529296875f,1.215399980610609276254763244651257991790771484375f,0.5424000423610220167347506503574550151824951171875f,0.517400053015351080176742470939643681049346923828125f,0.3694000380247839387948260991834104061126708984375f,0.37970003715455558079838738194666802883148193359375f,0.26449997472762998285134017351083457469940185546875f,1.448299944978952424889939720742404460906982421875f,-0.675499987125396383902398156351409852504730224609375f,0.837399995571374677894027627189643681049346923828125f,-0.6027000200331202872661151559441350400447845458984375f,0.374200039523838512423026259057223796844482421875f,0.575599980676174904914432772784493863582611083984375f,1.136099977487325052294409033493138849735260009765625f,-0.88699996891617782779348999611102044582366943359375f,1.0018999507784851488167987554334104061126708984375f,0.9874000031709666558299431926570832729339599609375f,0.7756999745249739675045930198393762111663818359375f,-0.7374999728798865561429920489899814128875732421875f,0.0313999813854690668080138493678532540798187255859375f,0.69749998364597498579087186953984200954437255859375f,0.182100014024973055892786533149774186313152313232421875f,-0.3677000051319598838972524390555918216705322265625f,-0.6487999993830915190784480728325434029102325439453125f,-1.58669998617768204240974228014238178730010986328125f,0.1108999766767022610025605899863876402378082275390625f,1.6515000160634514259072602726519107818603515625f,-0.2171999875709413885260801180265843868255615234375f,0.5662000240862370414873794288723729550838470458984375f,-0.2211000017762183578184931320720352232456207275390625f,0.3295000025182959202396659748046658933162689208984375f,-0.4661999811708927854425610348698683083057403564453125f,-0.000599971623719053326340144849382340908050537109375f,-0.48989997683316488519267295487225055694580078125f,-0.464300004315375769436968766967765986919403076171875f,-0.5232000049605967095800451716058887541294097900390625f,1.69009993633031907478425637236796319484710693359375f,0.794200035090744105303883770829997956752777099609375f,-0.3458999729514129484186923946253955364227294921875f,0.3752999998748303145390536883496679365634918212890625f,-1.114199985879659404730546157225035130977630615234375f,-1.669699981126189225477673971909098327159881591796875f,-0.7520999854147432106543647023499943315982818603515625f,-1.12389997348785453112895993399433791637420654296875f,-1.0767999977961177915375401425990276038646697998046875f,-0.3982999665856363780136462082737125456333160400390625f,-0.13120000516176222316033772585797123610973358154296875f,-0.3119999860897657573133301411871798336505889892578125f,1.505499985188245570810749995871447026729583740234375f,-0.87199997577071286514183157123625278472900390625f,-0.75069998525381098630759879597462713718414306640625f,-1.1831999500870704622457196819595992565155029296875f,0.41599997074902061999779334655613638460636138916015625f,-0.55989997649788847411400638520717620849609375f,-0.3313000052690504571017982016201131045818328857421875f,0.3384000085651874822900708750239573419094085693359375f,0.17299998749792511176792686455883085727691650390625f,0.185599992746114883512831283951527439057826995849609375f,-0.5580000248253345063176311668939888477325439453125f,0.8286999976366751230472118550096638500690460205078125f,0.401799979579447352762144873850047588348388671875f,1.0247000036641960019068164911004714667797088623046875f,0.3930999477446077694509085631580092012882232666015625f,-0.5704999625012281416758241903153248131275177001953125f,-0.478799964290857804627421501209028065204620361328125f,0.40939999373108137969978770343004725873470306396484375f,0.482899985565244715957078369683586061000823974609375f,-1.003199972513318272149263066239655017852783203125f,0.166900024390220114156591080245561897754669189453125f,1.277699950332940037611706429743207991123199462890625f,1.585699985849856830100179649889469146728515625f,0.293600052998959970551595688448287546634674072265625f,1.4533999795079228789518310804851353168487548828125f,-0.56500000882893797182759954011999070644378662109375f,-0.4639999466240407599570971797220408916473388671875f,-1.0832999478474263366223340199212543666362762451171875f,0.3411000271081920942606302560307085514068603515625f,1.8378999822124837404402342144749127328395843505859375f,1.008799977105855560921554570086300373077392578125f,-0.25799996995180884962195477783097885549068450927734375f,-0.8239999752342701544449710127082653343677520751953125f,0.47539996600747169708256478770636022090911865234375f,1.15319998868107820300110688549466431140899658203125f,-0.2065000033974655480051296763122081756591796875f,-0.652300023329257072646214510314166545867919921875f,-0.7699999645352360033712102449499070644378662109375f,-1.38949998754262882272314527654089033603668212890625f,-0.80909993615150543888603351661004126071929931640625f,0.333899976766109940484739126986823976039886474609375f,0.79619996865391673424028340377844870090484619140625f,0.124199993181229151328892612582421861588954925537109375f,-0.53910001494139392352877848679781891405582427978515625f,0.0250999916136265266430882547865621745586395263671875f,0.415799954962730478058574590249918401241302490234375f,-0.497600060456991233337475932785309851169586181640625f,0.32349999283999186783233881214982829988002777099609375f,-0.6148000304818150851815516944043338298797607421875f,1.0424000124096863828526693396270275115966796875f,-0.471900009936094821938468157895840704441070556640625f,-0.35579999337047263363587035200907848775386810302734375f,-0.85070002429485302997136386693455278873443603515625f,0.45299996171891698804046200166339986026287078857421875f,-0.20750004887580875134744928800500929355621337890625f,-0.09850000528246172226687349393614567816257476806640625f,0.146600016468763527655028156004846096038818359375f,0.7516999975770712882905399965238757431507110595703125f,1.02299996984004959443836924037896096706390380859375f,-0.701999992460012212092124173068441450595855712890625f,0.97370003585815378954748666728846728801727294921875f,-0.1619999753981826717819103578221984207630157470703125f,0.80989997247457523599223350174725055694580078125f,1.2185999562159184339549256037571467459201812744140625f,0.25679998077154220226248071412555873394012451171875f,-0.5411000282257794058438093998120166361331939697265625f,0.63449999886751218269864693866111338138580322265625f,-0.76239998588561963543952515465207397937774658203125f,-0.22519998322725331973970241961069405078887939453125f,0.21669995384216367284579973784275352954864501953125f,0.2214000219166276650639701983891427516937255859375f,-0.29350000643730123073282811674289405345916748046875f,-1.3235999689936637224718651850707828998565673828125f,0.77569997318386985085680862539447844028472900390625f,-0.705099990421533373563534041750244796276092529296875f,0.0958999759316458977309594047255814075469970703125f,-0.00799998727440875967431566095910966396331787109375f,0.44269995998144207050017939764074981212615966796875f,-1.329299971681833003600559095502831041812896728515625f,0.5620000084973868492710380451171658933162689208984375f,0.2327000127315519417692257775343023240566253662109375f,0.3097000017642976832377144091879017651081085205078125f,0.5127999884754419479548914750921539962291717529296875f,-0.224099985884130159519145308877341449260711669921875f,-0.6860999992430210081550967515795491635799407958984375f,-1.061499975234270198853891997714526951313018798828125f,-0.41769996664971120736709053744561970233917236328125f,-0.39500000797212120318135930574499070644378662109375f,-0.3610000161789359562902745892642997205257415771484375f,0.47929998129308248877578080282546579837799072265625f,0.60119997774362599063380230290931649506092071533203125f,1.1508999826557932966153430243139155209064483642578125f,0.17229997341036806801639613695442676544189453125f,0.5032000026136633774598294621682725846767425537109375f,-0.77139998287558597667157300747931003570556640625f,-1.0009999974593519755927673031692393124103546142578125f,-0.6933999882996080277308692529913969337940216064453125f,-0.54350000490993266577532949668238870799541473388671875f,-0.2079999834001065028843413529102690517902374267578125f,1.153600000360607946703339621308259665966033935546875f,0.5100999647356569965950257028453052043914794921875f,-0.93330000488162034866235217123175971210002899169921875f,-0.85509995279610162555172792053781449794769287109375f,1.209699975370615021574849379248917102813720703125f,0.5027999839305881124573716078884899616241455078125f,0.69269999455213504546691183350048959255218505859375f,-0.26169998528361337886138926478452049195766448974609375f,-0.04679998412430297349828833830542862415313720703125f,0.2901999817371370138374686575843952596187591552734375f,-0.4596999718874694718095952339353971183300018310546875f,0.65729998104721321094956465458380989730358123779296875f,-0.0147000070914629077378776855766773223876953125f,0.40570000009909268356977918301708996295928955078125f,0.00369999409765042752695762828807346522808074951171875f,-1.4716999660611154165934522097813896834850311279296875f,0.43490001799762267875593124699662439525127410888671875f,-0.26920000445023173352154799431446008384227752685546875f,-0.085800000206381010769973727292381227016448974609375f,0.4677000120609999811449597473256289958953857421875f,0.15049999777972689063432198963710106909275054931640625f,-0.6078000217050314457623017005971632897853851318359375f,0.7165999959424136367402979885810054838657379150390625f,0.353500026181340576414413590100593864917755126953125f,0.23740000436306007980391541423159651458263397216796875f,1.242200007799266980867969323298893868923187255859375f,-0.4608999929562209718625354071264155209064483642578125f,0.52840001392960544990273774601519107818603515625f,0.282100013726949594428106138366274535655975341796875f,0.04289999163746849308864739214186556637287139892578125f,-0.17440000745877615173640151624567806720733642578125f,-0.417099975132942279998360390891321003437042236328125f,0.83979998775273567712673639107379131019115447998046875f,0.719399975961446447314528995775617659091949462890625f,-0.5549000030219553014632083431934006512165069580078125f,0.946999971151352593068395435693673789501190185546875f,1.4982000152051446750789409634307958185672760009765625f,-0.800699989724159788551105521037243306636810302734375f,-0.688500017352402071679762229905463755130767822265625f,0.2291000300288192903508388553746044635772705078125f,0.138200010753422919407285007764585316181182861328125f,-0.01359999849796300619431121958768926560878753662109375f,0.04019999505504989745219290853128768503665924072265625f,-0.4954999880939723322370582536677829921245574951171875f,0.75359996951520447083794351783581078052520751953125f,0.399600007766485187943317214376293122768402099609375f,0.689399971565604285927975070080719888210296630859375f,-0.2263000123471019531251613443600945174694061279296875f,-0.24289998500645193502123220241628587245941162109375f,0.507799980801344030822974673355929553508758544921875f,-1.310299941983819049795556566095910966396331787109375f,1.104699980604648690274416367174126207828521728515625f,0.5327999674275518682264873859821818768978118896484375f,-0.005900016909837546563721843995153903961181640625f,1.1523999991476532311907021721708588302135467529296875f,0.279199952761828018577006105260807089507579803466796875f,1.3521000138759611264305249278550036251544952392578125f,0.6879000050485133410660409936099313199520111083984375f,-0.0601999837487932898394404901409870944917201995849609375f,-0.727500007003545778871966831502504646778106689453125f,0.62929999667853053768595827932585962116718292236328125f,0.25139997825622462102046483778394758701324462890625f,0.6588000038906935973415102125727571547031402587890625f,0.741499985106289560832237839349545538425445556640625f,0.2191000495493409250258309839409776031970977783203125f,1.947699956703186074946643202565610408782958984375f,0.28140000463128078678209931240417063236236572265625f,1.431000018991528843770311141270212829113006591796875f,-0.459099970349669650460100456257350742816925048828125f,0.11010001009106673564019729383289813995361328125f,0.7694000093027948405932647801819257438182830810546875f,0.99049998652934956311355563229881227016448974609375f,-0.03020000300854430719255105941556394100189208984375f,1.21999999165534944722821819595992565155029296875f,-0.08339997999221093749611100065521895885467529296875f,-0.72759996091723433409015342476777732372283935546875f,-0.834199988934397840267820356530137360095977783203125f,-0.39929998703002933524430773104541003704071044921875f,-0.625999964714051060354904620908200740814208984375f,-1.328199977765976758092847376246936619281768798828125f,0.0323000242233268952674052343354560434818267822265625f,-1.27319999791979743264391800039447844028472900390625f,0.0312000089243056333287995585124008357524871826171875f,1.063599999094009262989857234060764312744140625f,-0.13760000129938187996003762236796319484710693359375f,-0.61880001137852591597265927703119814395904541015625f,-0.4972999786257741305917079444043338298797607421875f,-1.520899972951412326693798604537732899188995361328125f,-0.8457000009372832582954515601159073412418365478515625f,-0.449599982173740475577261577200260944664478302001953125f,-1.0085999756991863574029366645845584571361541748046875f,-1.6707000033587220411135376707534305751323699951171875f,-0.4969000069931148733104464554344303905963897705078125f,0.343200006189942730117081737262196838855743408203125f,-0.30489998748749513879374717362225055694580078125f,-0.207899978747963842806711909361183643341064453125f,-0.78020000822395019213217892684042453765869140625f,-0.3249999578297126934245397933409549295902252197265625f,-0.76420002715587609376513000825070776045322418212890625f,-0.26200002628564877937833443866111338138580322265625f,0.04180001161694502087584623950533568859100341796875f,-0.660400018236040775576611849828623235225677490234375f,0.49000001624226552365826137247495353221893310546875f,-0.2672000302255153059149961336515843868255615234375f,-0.86540000117421111980320347356610000133514404296875f,-2.04080000860691068709229512023739516735076904296875f,0.4871999989330770830520123126916587352752685546875f,-0.0977999963730569188413710435270331799983978271484375f,-1.22309997322559382837425800971686840057373046875f,-1.531000007778406502012558121350593864917755126953125f,1.479299956035614371785413823090493679046630859375f,-0.197799974542856915604716050438582897186279296875f,0.74660001065731051994589506648480892181396484375f,0.4337999967008825397130067358375526964664459228515625f,0.0263999499812717086655311504728160798549652099609375f,0.9076000464871529782584502754616551101207733154296875f,0.135699998553097334053063605097122490406036376953125f,-1.1424999853968618168664761469699442386627197265625f,0.2547000037014495177345452248118817806243896484375f,0.34049996238946800275471105123870074748992919921875f,1.156699954438209942964022047817707061767578125f,0.60210002281665830903278902042075060307979583740234375f,0.4825000338256357768074167324812151491641998291015625f,0.864000028729439151220503845252096652984619140625f,-0.069300018769502980120478241587989032268524169921875f,-0.8096000031471251023873492158600129187107086181640625f,0.6739000015020371048279912429279647767543792724609375f,-0.77160001550018797455976482524420134723186492919921875f,0.76770000993758469309824477022630162537097930908203125f,1.27550003845989667894400554359890520572662353515625f,0.3056999649137261965137213337584398686885833740234375f,-0.546099997007846393870522661018185317516326904296875f,0.40259999526441081485472750500775873661041259765625f,-1.012800013732910286989863379858434200286865234375f,-0.34550000433623784257264333064085803925991058349609375f,-0.254600023785233664508353967903531156480312347412109375f,0.6597999893546102612162940204143524169921875f,0.2972000031054020663390247136703692376613616943359375f,-0.173399999214708611106061653117649257183074951171875f,-0.06359997383654103497718779181013815104961395263671875f,-0.0968000215262174723562793587916530668735504150390625f,-0.41900000034272721904926584102213382720947265625f,0.69709997274875679096339808893389999866485595703125f,1.932599998021125475844428365235216915607452392578125f,-1.04799996149539964562791283242404460906982421875f,0.72060000094175347129521469469182193279266357421875f,0.2453999738305807643001799078774638473987579345703125f,0.125799986851215683802962530535296536982059478759765625f,-1.0904000416681167706656196969561278820037841796875f,0.802400012037157939204234935459680855274200439453125f,0.72479996394813017701608259812928736209869384765625f,-2.43030006216168370514196794829331338405609130859375f,-0.966699989902974010647085378877818584442138671875f,-1.345700031447410793816743534989655017852783203125f,-0.8078000980734838520902485470287501811981201171875f,1.31160003673434299997779817203991115093231201171875f,-0.01860004215836585927945634466595947742462158203125f,0.675600011819600609896951937116682529449462890625f,1.409599993461370104341767728328704833984375f,-2.4413000906527049238547988352365791797637939453125f,0.517899972005188669044173366273753345012664794921875f,0.307699970486760399079884109596605412662029266357421875f,0.441500020906328671799201401881873607635498046875f,1.41550005738437167934762328513897955417633056640625f,-0.91640000589787984353051797370426356792449951171875f,-0.0411000239044430060175727703608572483062744140625f,0.35819999508857680581286331289447844028472900390625f,-1.747599984088540114868237651535309851169586181640625f,1.61560001360774041501144893118180334568023681640625f,1.281699993702769102554839264485053718090057373046875f,-0.27500000007450609018633258529007434844970703125f,0.9888000304520130345764528101426549255847930908203125f,0.85989998055845529023599738138727843761444091796875f,-0.441000003233551540748891284238197840750217437744140625f,-0.97209995427131712375512506696395576000213623046875f,-0.08240004914104925859419381595216691493988037109375f,1.233299996984005275635354337282478809356689453125f,0.16539998873174255322737735696136951446533203125f,0.8749000171780583201552872196771204471588134765625f,0.357400003096461471585598701494745910167694091796875f,-0.2587000237882133291122954688034951686859130859375f,-0.959200033786892891640718517010100185871124267578125f,-0.93239996860027307690188536071218550205230712890625f,0.11939995793104163368525405530817806720733642578125f,-0.4419000309467318032830007723532617092132568359375f,0.885900062805415444699974614195525646209716796875f,1.21480002556443178463041476788930594921112060546875f,-1.62060002463459884580743164406158030033111572265625f,-1.587300010663271532251883400022052228450775146484375f,0.65700001436471922033888404257595539093017578125f,-0.4131000432610516526210631127469241619110107421875f,-0.404300014001131469143501817598007619380950927734375f,-0.7944000521063809827637669513933360576629638671875f,0.67550007966160929839816162711940705776214599609375f,0.08460001626014701514577609486877918243408203125f,-0.02619997506141569232340771122835576534271240234375f,0.8677000224918127724293981373193673789501190185546875f,0.333699971634149772370392383891157805919647216796875f,0.17240003103613854573694652572157792747020721435546875f,0.1449000162839890748500693007372319698333740234375f,-0.019499992311001079769994248636066913604736328125f,-0.17759999697804451646021561828092671930789947509765625f,0.9168999735027549835564286695444025099277496337890625f,0.317799982216954379765638805110938847064971923828125f,0.9052000294953586045920701508293859660625457763671875f,-0.102700043874979218827547811088152229785919189453125f,0.293000005304813182505085933371447026729583740234375f,0.447299994271993295313905036891810595989227294921875f,0.3699999958276745104512883699499070644378662109375f,-0.522400002127885176861354921129532158374786376953125f,-0.6913999977767470994649556814692914485931396484375f,-0.16220000325441308319796007708646357059478759765625f,1.300299992871284171513934779795818030834197998046875f,-2.03159998189806945134705529198981821537017822265625f,0.28569999449253113965596639900468289852142333984375f,-0.217599950560927180021053573000244796276092529296875f,-0.238799984407425114341094740666449069976806640625f,-1.447999990254639879339038088801316916942596435546875f,0.3981999840617174157841873238794505596160888671875f,-1.402800025504827186040301967295818030834197998046875f,0.4380000027157369135721864950028248131275177001953125f,-1.509400057619810553433126187883317470550537109375f,-0.93939998251795753247961329179815948009490966796875f,0.26889997929930642417417629985720850527286529541015625f,-0.315200000959634518693519567023031413555145263671875f,1.080800036025047372589824590249918401241302490234375f,0.284199997726082731475116816000081598758697509765625f,0.8278000204384323712503146452945657074451446533203125f,1.41600001396238805995153597905300557613372802734375f,-1.006200017417967274013790301978588104248046875f,-0.97450001043453815530170913916663266718387603759765625f,1.536900030779093651034372669528238475322723388671875f,-0.0355999957449735504599885871357400901615619659423828125f,0.949900018519163058527965404209680855274200439453125f,-1.29020000207722151941425181576050817966461181640625f,-0.81029999398887131434321418055333197116851806640625f,-0.11000001132488257393760022750939242541790008544921875f,-0.8047999984443190424343583799782209098339080810546875f,0.8281000031024217111763618959230370819568634033203125f,0.5886000055760145510674874458345584571361541748046875f,0.05150000537186871785166886184015311300754547119140625f,0.08459999402016415981364616527571342885494232177734375f,-0.39810001506060366249784010506118647754192352294921875f,-1.005600004480779130489054296049289405345916748046875f,-1.08199997781962142795464387745596468448638916015625f,0.203799970251321571623748241108842194080352783203125f,0.3707000056684019995145717984996736049652099609375f,0.413400002232193752860212043742649257183074951171875f,0.134700002434849697152685621404089033603668212890625f,0.03480001431405543854680217918939888477325439453125f,-0.119400016120076113423920105560682713985443115234375f,-0.05100000800192372452102063107304275035858154296875f,-0.85139999181628212010508605089853517711162567138671875f,-0.3845999945417044596496225494774989783763885498046875f,1.1068999916262922678100721896043978631496429443359375f,-0.011999988026917041139540742733515799045562744140625f,-0.222900058133900902390678311348892748355865478515625f,0.036300008547306195083592683658935129642486572265625f,-0.24700000099092722649629649822600185871124267578125f,0.645900007522106189838950740522705018520355224609375f,0.5495000200271606960455983426072634756565093994140625f,-1.3809000444769861104532537865452468395233154296875f,-1.48480000216960927872378306346945464611053466796875f,-0.979600013428926441605426589376293122768402099609375f,0.0365999957561495836699805295211263000965118408203125f,-0.17000002756714849283525836654007434844970703125f,0.80809998291134821357672990416176617145538330078125f,-0.847300038826465762298312256461940705776214599609375f,0.61720000580623757269194129548850469291210174560546875f,-0.11320000563114884783999514183960855007171630859375f,0.1705000006295740722084275375891593284904956817626953125f,0.2486999892920256360895336911198683083057403564453125f,-0.269299987626075765234645587042905390262603759765625f,0.556300011825561657730077058658935129642486572265625f,-0.75440002029240160741352383411140181124210357666015625f,0.9004000116050239199694260605610907077789306640625f,0.0297000060111287655928435924579389393329620361328125f,-0.5740000052601097468851776284282095730304718017578125f,-0.38979999635815676395367290751892141997814178466796875f,0.4761000434249644808204493529046885669231414794921875f,-0.354899989610910127080245501929312013089656829833984375f,-0.232999988168477945027490250140544958412647247314453125f,0.16919999411255115528973647087696008384227752685546875f,-0.2042000075608492171141961080138571560382843017578125f,-0.8238999842166900844375732049229554831981658935546875f,-0.9496000059783458180362458733725361526012420654296875f,1.245899999773502297983895914512686431407928466796875f,0.573799995583295530110490290098823606967926025390625f,-0.4030000014305110678236587773426435887813568115234375f,2.096799961772561449180329873342998325824737548828125f,-0.232599968963862036019918377860449254512786865234375f,1.0441999725431203582814987385063432157039642333984375f,0.25159997981190684868124662898480892181396484375f,-1.50000002145767208361348821199499070644378662109375f,-0.533100062036513833163553499616682529449462890625f,-0.79970005488693818307410765555687248706817626953125f,-0.9971001037672169164949309561052359640598297119140625f,0.2672000035151838037705829265178181231021881103515625f,-0.065900023950636654745949272182770073413848876953125f,-1.1265000373721125725978708942420780658721923828125f,-0.158700012612342788997921161353588104248046875f,-1.1871000390589241391126051894389092922210693359375f,-0.8435000717043887874524443759582936763763427734375f,1.4882000421762471287223661420284770429134368896484375f,0.62939999004304458818381817764020524919033050537109375f,0.6318000069975848731473888619802892208099365234375f,0.3583999603599306027490456472150981426239013671875f,-0.1109999855011698510764972525066696107387542724609375f,0.50149996748566605386798755716881714761257171630859375f,-0.24280000769793996795442581060342490673065185546875f,0.6567000224992636248089183936826884746551513671875f,0.9143000240594147953743231482803821563720703125f,-0.5530000365227467629125612802454270422458648681640625f,0.0952000022262335932055066223256289958953857421875f,0.36580000592470174325399057124741375446319580078125f,-1.1025000283122061706109207079862244427204132080078125f,0.498499971717596412901229996350593864917755126953125f,-0.1782000180363656482995793339796364307403564453125f,0.709700012493133414182011620141565799713134765625f,-0.7862000126868486926667856096173636615276336669921875f,0.7356000200524928089862441993318498134613037109375f,0.34520002010762684818701018230058252811431884765625f,-0.7270000025182958847125291867996565997600555419921875f,-0.48240003438889988984783485648222267627716064453125f,0.858000020205974323062037001363933086395263671875f,0.1498000020951035793359551462344825267791748046875f,0.58110000445842757699210778810083866119384765625f,-0.311999978750943984806553999078460037708282470703125f,-0.0396999924883243426876333614927716553211212158203125f,1.4060000378042456503635548870079219341278076171875f,0.7670999952122572462798189008026383817195892333984375f,0.2588000156626100167756021619425155222415924072265625f,0.3820999585554003541432166457525454461574554443359375f,1.70240003893375391186282286071218550205230712890625f,0.97319999851584437777773928246460855007171630859375f,1.15110004275441202281626829062588512897491455078125f,0.101200031238794618815290959901176393032073974609375f,-0.036499946475027744696717491024173796176910400390625f,1.052600090482832317917427644715644419193267822265625f,-0.31199998307228060667739555356092751026153564453125f,-1.1166999241702264544784384270315058529376983642578125f,0.5268999759614469979851492098532617092132568359375f,-0.3440999989040196549439087903010658919811248779296875f,-0.553899993455410299958430186961777508258819580078125f,-0.608699959862232020668670884333550930023193359375f,1.44049998340010620978546285186894237995147705078125f,-1.82949992823600826596930346568115055561065673828125f,0.55949999931454641455275123007595539093017578125f,-1.4740999978423108274228070513345301151275634765625f,-0.0860000202953816739182002493180334568023681640625f,-0.214899969494343423548343707807362079620361328125f,0.59129999901056340405602895771153271198272705078125f,0.5979000089228148073772217685473151504993438720703125f,0.091700035053491557590632510255090892314910888671875f,0.8109000148981808653303460232564248144626617431640625f,0.261899974024295101315829015220515429973602294921875f,-0.247399985215067397259502968154265545308589935302734375f,0.3366000376284130624071622150950133800506591796875f,0.5217999730974434147157126062666065990924835205078125f,0.45980000656545172432743129320442676544189453125f,-0.748300005514919153171149446279741823673248291015625f,0.9196000409588227508805857723928056657314300537109375f,-0.84090001325905350260114801130839623510837554931640625f,-0.5458999888956543866669335329788736999034881591796875f,0.2878000461429357503817527685896493494510650634765625f,0.7995999471187591467469246708787977695465087890625f,0.28999994944781104511122293843072839081287384033203125f,0.058799990963935844234811156638897955417633056640625f,-1.39749997545033721735308063216507434844970703125f,-0.1120999772191044741020959918387234210968017578125f,1.2605000173375009442366945222602225840091705322265625f,-0.287600065374373770055171917192637920379638671875f,-0.3768000378802416339141245771315880119800567626953125f,-1.7490999638304109975450728597934357821941375732421875f,-0.420499972783029196676807259791530668735504150390625f,0.027499981969594244901600177399814128875732421875f,0.730299983483552406227090614265762269496917724609375f,0.7582999768674374418964134747511707246303558349609375f,0.5678999853789807872317396686412394046783447265625f,-0.317300015133618718010666270856745541095733642578125f,-0.974799940627813388260847204946912825107574462890625f,-0.5745999824345116024915114394389092922210693359375f,0.7191999606966970048915754887275397777557373046875f,0.67489999721050220049534118516021408140659332275390625f,-0.77839998062550908031198559910990297794342041015625f,0.4402999921634787039437242128769867122173309326171875f,-0.822099976995587677919274938176386058330535888671875f,1.75089999679326968617942839046008884906768798828125f,0.2437000093340870865432634673197753727436065673828125f,-0.52519999738335609862360797706060111522674560546875f,0.224399980916082764093033574681612662971019744873046875f,0.638700007545948356124654310406185686588287353515625f,1.256700007523595896685719708329997956752777099609375f,0.63509999057054500326557899825274944305419921875f,1.136500018633901820663822945789434015750885009765625f,-0.697600008749961641996151229250244796276092529296875f,-0.33769997790008776661352385417558252811431884765625f,-0.215599953481554695855493264389224350452423095703125f,-0.28130000623762596578814054737449623644351959228515625f,0.9001999899886545808413984559592790901660919189453125f,-2.235000023618340048159325306187383830547332763671875f,1.12270000349283183282977915951050817966461181640625f,0.577900044424831893508098801248706877231597900390625f,0.508800003182888094244162857648916542530059814453125f,-0.3609000099807975203702881117351353168487548828125f,0.5514000244498244995838831528089940547943115234375f,1.1533999906837941740178621330414898693561553955078125f,0.5833999556660653951212225365452468395233154296875f,-0.817099996329843758502420314471237361431121826171875f,0.341599984133244216621960731572471559047698974609375f,0.9167999938100575008093073847703635692596435546875f,-1.556099945673346862662356215878389775753021240234375f,-0.9138999596297736616179463453590869903564453125f,0.4706999959826470014689903109683655202388763427734375f,-0.87900000101327879775681140017695724964141845703125f,-1.264799985331297005330952742951922118663787841796875f,0.653799985152482943107088431133888661861419677734375f,-1.37709996559619884948233448085375130176544189453125f,1.7545999492049215717770493938587605953216552734375f,0.49359999209046367951714273658581078052520751953125f,-0.459999982714652588811077293939888477325439453125f,0.3084999732822180096292186135542578995227813720703125f,-0.9262999694317581855784737854264676570892333984375f,-1.030600003400444553136594549869187176227569580078125f,0.231999976776540162148876333958469331264495849609375f,0.22649999639391893424544832669198513031005859375f,-0.38629998645633445431002428449573926627635955810546875f,0.483900019085407251395736238919198513031005859375f,0.3531999888673420517903878135257400572299957275390625f,0.0755000084340569088681149878539144992828369140625f,0.8776999952971937801748936180956661701202392578125f,1.6275999738812438888402311931713484227657318115234375f,-0.0319999758079649865294413757510483264923095703125f,-0.08209998735189483198837478994391858577728271484375f,0.55809999837130253030892390597728081047534942626953125f,1.571000013440846831969111008220352232456207275390625f,-0.366499991923570522800446269684471189975738525390625f,0.24149998044967713184405511128716170787811279296875f,-0.12369999007433685189738525878055952489376068115234375f,0.0554000394701947840303546399809420108795166015625f,0.50330004355013358008363866247236728668212890625f,1.343699936318397902823562617413699626922607421875f,-0.709500001884996844836450691218487918376922607421875f,-0.5710999369561680083506871596910059452056884765625f,0.0996000085487960262753404094837605953216552734375f,0.762100015366076632972180959768593311309814453125f,0.2286000538185237740407274031895212829113006591796875f,0.397300031673907039220239312271587550640106201171875f,-0.32090000990629174992818661849014461040496826171875f,-0.10150001993775337183478768565692007541656494140625f,-1.07079994035959291664994452730752527713775634765625f,0.3561000050544738115831933100707828998565673828125f,-0.3714999932646758651344498503021895885467529296875f,0.5515999671459201891821066965349018573760986328125f,-0.666199972081184199623749009333550930023193359375f,-1.3571999856710430520934096421115100383758544921875f,-0.53499995768070274237970807007513940334320068359375f,1.4681999596238142213877608810435049235820770263671875f,-0.46419999139309009450471421587280929088592529296875f,-0.31210000344514821080110777984373271465301513671875f,-0.45720001018345335364045922688092105090618133544921875f,-0.2426000159025196722950568073429167270660400390625f,-1.1025000143051142398320507709286175668239593505859375f,0.03379999811649359031662243069149553775787353515625f,-0.2223000378206376748124739606282673776149749755859375f,0.407100002104044733641785569489002227783203125f,-0.133699969398975770928927886416204273700714111328125f,-1.592999984368681953839086418156512081623077392578125f,-0.2246999933823941031363347065052948892116546630859375f,0.084899973741173528907211220939643681049346923828125f,-0.4778999802008272457243265307624824345111846923828125f,-0.279399966871738403284552987315692007541656494140625f,0.9711999892547726886959935654886066913604736328125f,-0.63180003232955872505272054695524275302886962890625f,-1.379300027002394291031350803677923977375030517578125f,-0.0557999555587773699016906903125345706939697265625f,0.9575000524148344904773466623737476766109466552734375f,-1.009499979794025392720868694595992565155029296875f,-2.284599984818696416510874769301153719425201416015625f,0.770399970328808247899132766178809106349945068359375f,0.9551999482467772306648612357093952596187591552734375f,-0.760599945807457711310917147784493863582611083984375f,0.30869999365061528973086524274549447000026702880859375f,-0.576399980640411424559488295926712453365325927734375f,-0.395499999344348651675318251363933086395263671875f,0.8694999774098395306509701185859739780426025390625f,-1.52139995530843652460362136480398476123809814453125f,0.7214000147640700078000008943490684032440185546875f,-0.77920001672506344903013086877763271331787109375f,-0.71339998889565503503717991407029330730438232421875f,0.0073000075340274861446232534945011138916015625f,0.1808999882251025592694304577889852225780487060546875f,0.370199980023503627535319537855684757232666015625f,-0.9088999673783779709168584304279647767543792724609375f,0.573699986162781616627626135596074163913726806640625f,1.126999963849783181757402417133562266826629638671875f,0.085800039750337564470328288734890520572662353515625f,-0.96140002385377965055113236303441226482391357421875f,-0.733300006744264987190717874909751117229461669921875f,-0.741000020146369564599808654747903347015380859375f,0.68199998929351479670657454335014335811138153076171875f,0.04719998582005491272184372064657509326934814453125f,0.24000002227723638004164286030572839081287384033203125f,-1.513100003176926833248217008076608180999755859375f,-0.83369999227225821680775652566808275878429412841796875f,-0.770599946850537786957602293114177882671356201171875f,0.55859996102154318986521275292034260928630828857421875f,-0.108800067704915903732398874126374721527099609375f,-1.411200002703071110232713181176222860813140869140625f,-0.5256000030279164292323912377469241619110107421875f,-0.31330002857446681563402535175555385649204254150390625f,0.833000025868415150398504920303821563720703125f,1.3646000090330840937014045266550965607166290283203125f,-0.04230000544786438609889955841936171054840087890625f,-1.24429999824315284939757475513033568859100341796875f,0.775099979841709085803813650272786617279052734375f,1.08519997775107679416350947576574981212615966796875f,1.10829993350505962013130556442774832248687744140625f,0.06850000971555691986480951527482829988002777099609375f,1.90549998742341841762026888318359851837158203125f,0.477600056135653261435436434112489223480224609375f,0.581400004184246199656627140939235687255859375f,0.3454999886155132315934679354541003704071044921875f,0.319500026732682851360323184053413569927215576171875f,0.93659994181394523593553458340466022491455078125f,-1.22499999061226816365888225845992565155029296875f,0.69739998499155131383986372384242713451385498046875f,0.672500020116566954442305359407328069210052490234375f,-0.6128000101566311030865108477883040904998779296875f,0.51309996413588532249150375719182193279266357421875f,-0.48289999715089759746433628606610000133514404296875f,-0.49499992102384648973156799911521375179290771484375f,-0.7819999762922529296105267349048517644405364990234375f,0.2002000364243980978784520630142651498317718505859375f,-0.343400001449882863457929715877980925142765045166015625f,0.099899992591142794395864257239736616611480712890625f,0.51529997797012339066924369035405106842517852783203125f,0.278899998670816717805109874461777508258819580078125f,-0.257300007496774341131384744585375301539897918701171875f,1.2415000643059619367392087951884604990482330322265625f,0.0022000023230910326077491845353506505489349365234375f,-0.6440000283941629266593054126133210957050323486328125f,0.3000999885961415092339166221790947020053863525390625f,-0.489600015515088582418457008316181600093841552734375f,-0.111999995365738680419553929823450744152069091796875f,-1.1842999770462512287849676795303821563720703125f,-1.2824999840930104699765479381312616169452667236328125f,-0.153799984705448533617300199694000184535980224609375f,0.10989999259114302532225337927229702472686767578125f,0.174699984776973682443212965154089033603668212890625f,1.254099956603349053096962961717508733272552490234375f,-1.4194999932050691437979139664093963801860809326171875f,-0.79529997528791451788521271737408824265003204345703125f,0.2696999704718583412699217660701833665370941162109375f,-0.9267000243619083477142339688725769519805908203125f,0.2686999986052516220524921664036810398101806640625f,-0.95039996380954983035138639024808071553707122802734375f,0.9766999489247802923586050383164547383785247802734375f,-0.34499998673796650194844914949499070644378662109375f,-0.5806999637961391957929890850209631025791168212890625f,1.117599932008982666076235545915551483631134033203125f,0.416500019341706551045945161604322493076324462890625f,0.070599947446586153176895095384679734706878662109375f,-0.01150004679709659693997991780634038150310516357421875f,0.15560001401752199257089159800671041011810302734375f,0.3594000270724293510937741302768699824810028076171875f,0.4543999852374198678006678164820186793804168701171875f,-0.09709999390840540200997565989382565021514892578125f,-0.54989999650269727471396663531777448952198028564453125f,-1.3387000325798990019166012643836438655853271484375f,0.07420000584721553327227638874319382011890411376953125f,-0.02760001739263628905973746441304683685302734375f,0.6473999941557646575773787844809703528881072998046875f,1.00689998922347978549396430025808513164520263671875f,-0.36849998870491962321693790727294981479644775390625f,0.12280001231729986699292567209340631961822509765625f,-0.5465000022053720929449127652333118021488189697265625f,0.495900011992454425868714906755485571920871734619140625f,-0.995999957673252112044792738743126392364501953125f,-0.14419996092021525324611275209463201463222503662109375f,0.54599998354539291955944690926116891205310821533203125f,0.4555999775841834686929132658406160771846771240234375f,-0.28389999481514094359368982622982002794742584228515625f,1.784199991691112874292457490810193121433258056640625f,-0.059100021684169945501707843504846096038818359375f,0.892599996009469265345614985562860965728759765625f,0.5580999690532681967169992276467382907867431640625f,1.296800014820695512440806851373054087162017822265625f,-1.4012999838113788797500092186965048313140869140625f,-0.050800006371736206034483984694816172122955322265625f,0.205199949550628701899768202565610408782958984375f,-1.50119999622106536918408892233856022357940673828125f,0.7190000196397310361362542607821524143218994140625f,-2.26570008047223137026549011352472007274627685546875f,-1.25940000144243224866613672929815948009490966796875f,-0.443600016230345062240303377620875835418701171875f,-0.85010005211234140887199828284792602062225341796875f,1.2542000425040722433323026052676141262054443359375f,1.4858000369191177725269881193526089191436767578125f,-1.399500032544136018941571819595992565155029296875f,-0.911399982279538445339994723326526582241058349609375f,1.866400066545606417633962337276898324489593505859375f,2.21910007909089390665258179069496691226959228515625f,-0.9691000539079317466217844412312842905521392822265625f,-0.51260002104341939599407851346768438816070556640625f,1.2768999965995544432217911889893002808094024658203125f,-1.5316000322639948816316746160737238824367523193359375f,-1.0324999951571225409452381427399814128875732421875f,0.582299999040365445779343644971959292888641357421875f,0.478699988994002456621501551126129925251007080078125f,-0.00710000396668863942295502056367695331573486328125f,-0.106700008156895620459181373007595539093017578125f,-0.4961999869823454645256788353435695171356201171875f,-0.8948000050753355250066078951931558549404144287109375f,-0.0260999903023240442667685101696406491100788116455078125f,0.039000040873885755132732811034657061100006103515625f,-0.9510999914944164856933639384806156158447265625f,0.0317999729484317317229624677565880119800567626953125f,0.93040003038048713079888329957611858844757080078125f,0.733600010344386266325500400853343307971954345703125f,0.72279998199343697962149235536344349384307861328125f,-0.657399988642335397770466443034820258617401123046875f,-1.1861000035643574079813333810307085514068603515625f,1.674600018271803936187325234641321003437042236328125f,0.447900000354647165323740409803576767444610595703125f,1.37499999552965146421001918497495353221893310546875f,-1.176700021083652902831317987875081598758697509765625f,1.012400022616982031564702992909587919712066650390625f,-1.173300010991095820855889542144723236560821533203125f,-0.3850000053644182518297611750313080847263336181640625f,0.65079998238086655959477866417728364467620849609375f,-0.0557000026881695475822198204696178436279296875f,-0.43310002195239061251186285517178475856781005859375f,-0.71180007345676532537481762119568884372711181640625f,-1.101700025665760307447271770797669887542724609375f,-1.45640004575848625023581917048431932926177978515625f,-1.7010000150799751139629734097979962825775146484375f,0.7272000495970250000254964106716215610504150390625f,1.6048999899089342324742801793036051094532012939453125f,1.592200011003016779653762569068931043148040771484375f,-0.26080003095865311735224167932756245136260986328125f,0.56870002550184761691554058415931649506092071533203125f,0.5208000001877548523765426580212078988552093505859375f,-0.1450000271201137280741022550500929355621337890625f,-0.066300029110908820229042248683981597423553466796875f,0.821300008547306337192139835678972303867340087890625f,-0.68820000760555277707908317097462713718414306640625f,-0.10899997694790375390994086046703159809112548828125f,0.1188999845147133971323682999354787170886993408203125f,0.243299985957145192827510982169769704341888427734375f,0.759799991068244029435163611196912825107574462890625f,-1.590100001671910678879839906585402786731719970703125f,1.116400004780291776995682084816507995128631591796875f,0.03280001499950879750855392558150924742221832275390625f,0.093599992388487152084053377620875835418701171875f,-0.826999994844198393906253841123543679714202880859375f,0.20750000610947605395040227449499070644378662109375f,-1.001600010657310857453694552532397210597991943359375f,-0.222699962961673492145564523525536060333251953125f,0.16480003219544858694689537514932453632354736328125f,1.2476999796509744555805809795856475830078125f,0.3904000078797338346703327260911464691162109375f,-0.7654000028133391975160293441149406135082244873046875f,0.166599964761733776441587906447239220142364501953125f,-0.09019997637271881529841266456060111522674560546875f,-0.20779997841715758255531909526325762271881103515625f,0.0958999774217605960302535095252096652984619140625f,0.1958999687790861088387828203849494457244873046875f,1.3196000004649164338843547739088535308837890625f,-2.3182000448584556551168134319595992565155029296875f,-1.23190003184080154596813372336328029632568359375f,0.8296999929726123212958555086515843868255615234375f,0.81339995611309934275823252392001450061798095703125f,-1.5965000610649588974609969227458350360393524169921875f,0.20410001229643714992789682582952082157135009765625f,-1.342100097620488430294471982051618397235870361328125f,-0.3059000032007694880320514130289666354656219482421875f,-0.3909999998062854142943933766218833625316619873046875f,0.659700006458163290545826384914107620716094970703125f,0.43030003235936220562507514841854572296142578125f,-0.340100018435716577869243337772786617279052734375f,0.53599999503791284638509750948287546634674072265625f,-0.61940002282559891710889132809825241565704345703125f,-0.86140000291764717843534526764415204524993896484375f,0.50590002264678435306421988570946268737316131591796875f,1.181900048604607622593221094575710594654083251953125f,-1.1464000042587521033254915892030112445354461669921875f,0.089000021204352908199552985024638473987579345703125f,0.64789998672008497493379763909615576267242431640625f,0.9405999905854465215071513739530928432941436767578125f,-0.0244000121712684148178595933131873607635498046875f,0.4665999783217902319165659719146788120269775390625f,-0.506699982750415500731833162717521190643310546875f,0.40919998863637463415443562553264200687408447265625f,1.4354999945759772117526154033839702606201171875f,1.22459996328651943287013637018389999866485595703125f,-1.0455999656260015484576797462068498134613037109375f,0.513999943271279402523532553459517657756805419921875f,0.634000005930662524633589782752096652984619140625f,1.57239999251663675750023685395717620849609375f,0.98629999662637768409467753372155129909515380859375f,0.2830000512003909562963599455542862415313720703125f,0.26639999941587433340828283689916133880615234375f,0.81860003351569243790208929567597806453704833984375f,2.07400004601478560317673327517695724964141845703125f,0.67419999899268123755291526322253048419952392578125f,-0.8823999837994573880450843716971576213836669921875f,0.465500033617019415288496020366437733173370361328125f,0.0347999659597875155014889969606883823871612548828125f,1.3946000029981131529410731673124246299266815185546875f,0.25719998492598517003671076963655650615692138671875f,0.13199999424815178628023204510100185871124267578125f,0.4683000027954580257727457137661986052989959716796875f,-1.0516000277191401846010876397485844790935516357421875f,0.578400022721290429927876175497658550739288330078125f,0.445800028201938314253993667080067098140716552734375f,0.168100027167797261284931664704345166683197021484375f,-1.6865000057816506018326663252082653343677520751953125f,0.241999973610043550564796532853506505489349365234375f,0.55889997877776576018504783860407769680023193359375f,0.87339998777806737706441708724014461040496826171875f,-0.5869999941736454562857261407771147787570953369140625f,0.61700001294910900018209076733910478651523590087890625f,0.59410002146065199468694117967970669269561767578125f,-0.74859999104738239594780679908581078052520751953125f,1.1834000098317869476005625983816571533679962158203125f,0.397199983286856905806416762061417102813720703125f,0.592799985346198266000783405615948140621185302734375f,-0.93590000066757195185118689551018178462982177734375f,-0.0717999536514286607058465961017645895481109619140625f,0.662899967348575813730349182151257991790771484375f,-1.437799970296025353633240229100920259952545166015625f,-0.0474999943375589150917903680237941443920135498046875f,-0.386199992570281092074679918368929065763950347900390625f,-0.819699981498718255323865378159098327159881591796875f,0.664999970719218236325787074747495353221893310546875f,-0.26990001091956994372367262258194386959075927734375f,0.4202999928712838340061352937482297420501708984375f,-0.3773000182628631904435678734444081783294677734375f,-0.236400006419419828063155364361591637134552001953125f,0.2766000252604481346452303114347159862518310546875f,-0.2194000266253954833928219159133732318878173828125f,-1.90700002956390424202481881366111338138580322265625f,0.172100015217065749340008551371283829212188720703125f,0.34909999337196406798966563655994832515716552734375f,-0.2948999517619608212726234341971576213836669921875f,0.420299997043610318314676987938582897186279296875f,-1.6277000785946853511632070876657962799072265625f,-0.356400033167004526291066213161684572696685791015625f,-0.1797999997854227149929329243605025112628936767578125f,0.09710000508427629739571784739382565021514892578125f,1.343400026744604236483837667037732899188995361328125f,-0.4110999986469749956086161546409130096435546875f,-0.921900013139843821363683673553168773651123046875f,0.3148999935597185384494878235273063182830810546875f,0.4931000212818383854340709149255417287349700927734375f,1.131099993059038855136577694793231785297393798828125f,0.5128999721169467118642160130548290908336639404296875f,0.3131999737054116383916380073060281574726104736328125f,0.66590003568530153899018841912038624286651611328125f,-0.36069999806880925774521529092453420162200927734375f,-0.5445999876499174607857867158600129187107086181640625f,0.23380000824928259817170328460633754730224609375f,-1.269000033646821901101020557689480483531951904296875f,-0.1408999792098999392919722595252096652984619140625f,-0.77299994339048883063014727667905390262603759765625f,0.80089997622966802026667210157029330730438232421875f,-0.3469999642223122737760832023923285305500030517578125f,-0.15729999412298223404604868846945464611053466796875f,-0.6297999571681025532399189614807255566120147705078125f,0.464099959397316297327051870524883270263671875f,-0.37560000943541549389692590921185910701751708984375f,1.1568000223085281863433237958815880119800567626953125f,0.6180000166296959474010463964077644050121307373046875f,0.7438000382006162425341244670562446117401123046875f,0.2724999879300591398845199364586733281612396240234375f,-0.342699982333183328364611952565610408782958984375f,-1.05480002765059399649771876283921301364898681640625f,-0.20509995674490966166558791883289813995361328125f,0.7001000491321096586005978679168038070201873779296875f,0.72030008473694351334160046462784521281719207763671875f,-0.78330004645586104317089848336763679981231689453125f,-1.26660002667605819937080013914965093135833740234375f,0.2765000245571140879974336712621152400970458984375f,-0.001499973036349189214888610877096652984619140625f,-0.576300009515881672683690339908935129642486572265625f,-0.64399999691545961599814518194762058556079864501953125f,0.12599998810887402811431456939317286014556884765625f,-0.3697999947190282465925292854080908000469207763671875f,0.90219999721944343917812147992663085460662841796875f,-0.614899958392978174259724255534820258617401123046875f,-0.60890002661049447141294876928441226482391357421875f,-0.885199992130696333703099298872984945774078369140625f,0.03900000511109880108051584102213382720947265625f,-0.409800035212934066208845251821912825107574462890625f,-0.2319999789744617846309893138823099434375762939453125f,-1.0590999808922412217526698441361077129840850830078125f,0.534399987956881705741807309095747768878936767578125f,0.16079997969865811313638914725743234157562255859375f,-0.927199970173835819053920204169116914272308349609375f,-0.07899999445676808562666337820701301097869873046875f,0.823399999848008601333049227832816541194915771484375f,-0.3171999706208712410671068937517702579498291015625f,0.8165999885290859960917941862135194242000579833984375f,-0.43100004220008802935382163923350162804126739501953125f,-0.79099998706579233154201347133493982255458831787109375f,1.1089000112622973137632698126253671944141387939453125f,0.3640000140517949400731367859407328069210052490234375f,-0.0406000033259399373264386667869985103607177734375f,0.46340005177855569939993074513040482997894287109375f,-0.38770001839399359511162401759065687656402587890625f,-0.7136000397741799616824209806509315967559814453125f,0.22349995875358530383891775272786617279052734375f,-1.0616000328600403435075349989347159862518310546875f,-0.51649999117851308483295724727213382720947265625f,0.5376000145614145964145791367627680301666259765625f,1.1791000099122523891992386779747903347015380859375f,1.411400010070204213974420781596563756465911865234375f,0.9772999911800024364083583350293338298797607421875f,1.48700002595037172881120568490587174892425537109375f,0.148699972379208844586173654533922672271728515625f,0.66509998744130083281334009370766580104827880859375f,0.9924999639391902661600397550500929355621337890625f,0.96379999740868793622894372674636542797088623046875f,0.38799999079480740338254918242455460131168365478515625f,-0.419900005406141196839797657958115451037883758544921875f,0.498600010493397416411198719288222491741180419921875f,0.479500030420720424473302045953460037708282470703125f,0.4463999886870373412506296517676673829555511474609375f,-1.01809999423623054326526471413671970367431640625f,0.1354000140264626228514543981873430311679840087890625f,-0.1421000191658732259014641385874710977077484130859375f,-0.2931000036053357238330363543354906141757965087890625f,0.016600015500188192163477651774883270263671875f,-0.99330001344978768873517083193291909992694854736328125f,-0.1590999982707199389420082979995640926063060760498046875f,-1.206000025175511414232687457115389406681060791015625f,-0.29510000368356703592809253677842207252979278564453125f,0.40259997907802380634478822685196064412593841552734375f,1.344099999779462262239349001902155578136444091796875f,1.445800008383393286948148670489899814128875732421875f,0.758499993622302692841685711755417287349700927734375f,0.278800015401840683981760093956836499273777008056640625f,-1.27370008562803338492130933445878326892852783203125f,0.64619997996017364272347549558617174625396728515625f,-1.07590003255605637377811945043504238128662109375f,-0.60290004081130010860078982659615576267242431640625f,-1.300499995712190326457857736386358737945556640625f,-0.4762000051990151749947699499898590147495269775390625f,0.559200017097591484116492210887372493743896484375f,1.1002000031761827170129208752769045531749725341796875f,0.167000023603439018415883765555918216705322265625f,-0.7634999739155172715499020341667346656322479248046875f,-0.392800016117095029954953133710660040378570556640625f,0.201299952109157853641363544738851487636566162109375f,0.09780001879930377839400534867309033870697021484375f,0.65469996184781276138409111808869056403636932373046875f,0.73279999853372512763627355525386519730091094970703125f,-0.4621999940901986292374203912913799285888671875f,0.704199998601525489760888376622460782527923583984375f,-0.70789999543726389585884817279293201863765716552734375f,1.34710000374317129256951375282369554042816162109375f,-0.04500003056600732076475424037198536098003387451171875f,-1.1565000086128705003574168586055748164653778076171875f,0.974999978989362858783351839520037174224853515625f,-1.27210000448822935226189656532369554042816162109375f,1.189799990695715070643245780956931412220001220703125f,0.307500042170286036480320035479962825775146484375f,-0.41540000951886180136085613412433303892612457275390625f,0.34810000749826475185955132474191486835479736328125f,-0.4049999793991450047769831144250929355621337890625f,-0.0906999928534037991312288795597851276397705078125f,-1.045499951288104245605836695176549255847930908203125f,-0.76519998874068306804474559612572193145751953125f,0.54179998602420076370123069864348508417606353759765625f,0.76390000299215277124176282086409628391265869140625f,0.58680000398010034867724016294232569634914398193359375f,0.590199994701146923858914306038059294223785400390625f,0.7965999745592478031852579078986309468746185302734375f,1.017799974095821458064392572850920259952545166015625f,-0.7153999936863779662843398909899406135082244873046875f,-1.37649995481967923893762417719699442386627197265625f,-0.296599998140334886187474694452248513698577880859375f,-1.6010999534964565071959441411308944225311279296875f,0.18019993789047017429538755095563828945159912109375f,0.0633000393033018582400472951121628284454345703125f,0.7375000369548796452789929389837197959423065185546875f,0.51179997093677531694311255705542862415313720703125f,0.9939999449476604187481143526383675634860992431640625f,0.0534999837875371753170838928781449794769287109375f,-0.7064999795183541930754245186108164489269256591796875f,-0.952799974095821511355097754858434200286865234375f,-0.13940000218749037941279311780817806720733642578125f,-0.31439999712109578666741072083823382854461669921875f,0.98439996344447155962598117184825241565704345703125f,0.19789997375607537577479888568632304668426513671875f,0.8277999949574468274704486248083412647247314453125f,0.921999983966351077668832658673636615276336669921875f,1.4590999331712719566667146864347159862518310546875f,-0.2275000010430812125150623614899814128875732421875f,0.72059996279478166769649760681204497814178466796875f,-0.0695000140666959254076573415659368038177490234375f,0.381799989712238385664022644050419330596923828125f,-0.889099967592954953943262808024883270263671875f,-0.26780001008212561686150365858338773250579833984375f,1.16020001111105042213011984131298959255218505859375f,0.22289997917637227597964511005557142198085784912109375f,0.629200013074279862479443181655369699001312255859375f,0.4281000009790061344716605162830092012882232666015625f,-0.345099972465634063922834684490226209163665771484375f,0.853199977728725134085152603802271187305450439453125f,0.64059994446635215581409283913671970367431640625f,0.002099991710483806173215270973742008209228515625f,1.02160003926754061609472046256996691226959228515625f,0.777899994617700318855213481583632528781890869140625f,-0.453400031065940734009700463502667844295501708984375f,0.10400003334879859284711756117758341133594512939453125f,-0.21729998428821650779951824006275273859500885009765625f,-0.41209999258592677762180755962617695331573486328125f,-0.690699994492530056078294364851899445056915283203125f,-1.8952999699234960306881703218095935881137847900390625f,0.568899939811231103448108115117065608501434326171875f,1.61029991531074045951754669658839702606201171875f,0.084300006401537785905020427890121936798095703125f,1.2729999932721256516998664665152318775653839111328125f,-0.94699997651576950374874286353588104248046875f,-0.77929996959567071623808942604227922856807708740234375f,0.0805999890208237701472171465866267681121826171875f,-1.088099968848377674390803804271854460239410400390625f,0.43670000614523851734105619470938108861446380615234375f,-0.93229999991580758678111351400730200111865997314453125f,-0.9066999942988160920975815315614454448223114013671875f,0.28539998709261471532272480544634163379669189453125f,-0.951799991351366170277970013557933270931243896484375f,1.948999939695001071271462933509610593318939208984375f,0.13170005367994264133812976069748401641845703125f,0.5866999752998349748622786137275397777557373046875f,-0.24300001692771910910551014239899814128875732421875f,1.068699968206882289223358384333550930023193359375f,-0.3298999663650992175689680152572691440582275390625f,-0.32119996775984793746516743340180255472660064697265625f,0.627800055605173046302525108330883085727691650390625f,-0.35150000112503743121550314754131250083446502685546875f,-0.935200026459246824828142052865587174892425537109375f,-0.12740000313371424578434698560158722102642059326171875f,-0.23450001809000997621268425064045004546642303466796875f,-0.670399973774701063433667513891123235225677490234375f,-1.89720002202987725326011059223674237728118896484375f,0.05069999903738453550516851464635692536830902099609375f,0.43830000234842270145918519119732081890106201171875f,0.72030001581907276975158538334653712809085845947265625f,1.070000001341103512686458998359739780426025390625f,-0.591399967154860650708769753691740334033966064453125f,-1.20829996539354311835268163122236728668212890625f,0.373899992114305579349320396431721746921539306640625f,0.2178999638654290915695810326724313199520111083984375f,-1.306100012356042672223566114553250372409820556640625f,-0.79669997209608478527798069990240037441253662109375f,-0.1461000210918490527234325782046653330326080322265625f,1.7029999802708619682789503713138401508331298828125f,1.7160000259950749512682932618190534412860870361328125f,0.93599999898672070486327356775291264057159423828125f,-0.72539992502927841400151010020636022090911865234375f,-0.19069999717473962164149270392954349517822265625f,-0.770599946105480881897165090776979923248291015625f,0.008699974316358716208696932881139218807220458984375f,0.26919998409152068319372119731269776821136474609375f,0.9661000006772575421365445436094887554645538330078125f,-0.16130000050067871963932475409819744527339935302734375f,-1.35340000722408237976424061344005167484283447265625f,0.28600002806261126675479999903473071753978729248046875f,1.859199975746870681092559607350267469882965087890625f,-0.1899999749660512549098712042905390262603759765625f,-0.49480001602768819424227331182919442653656005859375f,-0.75219993232488757683995572733692824840545654296875f,-1.8418999890744682801368981017731130123138427734375f,0.018300016783923102325903897735770442523062229156494140625f,-0.85739998894035807097679935395717620849609375f,-0.1975000104308128057173377101207734085619449615478515625f,1.5951999912738796805200536255142651498317718505859375f,-0.612099965372681964481671457178890705108642578125f,-0.61609993144273733634008749504573643207550048828125f,0.534499961316586169601805522688664495944976806640625f,1.247399970239400968097243094234727323055267333984375f,0.224399996711313942654442143975757062435150146484375f,-0.617500025928020335186374722979962825775146484375f,0.40619996995776885828632885022670961916446685791015625f,1.13470000414848382064292309223674237728118896484375f,-1.3758000061854718243381512365886010229587554931640625f,0.85839997712373783400607862859033048152923583984375f,1.3986999918624756222840233021997846662998199462890625f,-2.0553999067008508205844918848015367984771728515625f,-0.61389998812824497775864074355922639369964599609375f,0.1285999448537840095241335802711546421051025390625f,0.7636999841138718547739472342072986066341400146484375f,-0.6417000197052953325282942387275397777557373046875f,-0.0876000361680986383561275943065993487834930419921875f,0.3179999856352804243897480773739516735076904296875f,-0.228099985891580647745513488189317286014556884765625f,-1.00039995572566997594776694313623011112213134765625f,-0.3109999785535031424643648279015906155109405517578125f,0.50769998546242722792243284857249818742275238037109375f,-0.45819998815953777437925964477472007274627685546875f,-1.1956999766111373606491952159558422863483428955078125f,0.2432000188559289011180908346432261168956756591796875f,1.29699994462728529498463103664107620716094970703125f,1.1996999451398853153705204022116959095001220703125f,-0.98249995753169105938695793156512081623077392578125f,0.87149999594688409843001863919198513031005859375f,0.627100018346309351358058847836218774318695068359375f,0.5768999638915062178057269193232059478759765625f,-1.38449996444582978938342421315610408782958984375f,-0.2607999643877152440296640634187497198581695556640625f,-0.6725999781280760370094640165916644036769866943359375f,-0.7273999860346316292947221882059238851070404052734375f,0.7552999928712840205236034307745285332202911376953125f,-0.4455999683454636706159135428606532514095306396484375f,-1.024799987566471060063122422434389591217041015625f,-0.41039999241977920263479973073117434978485107421875f,-0.529799984958767566922688274644315242767333984375f,-0.7148000101789826477016731587355025112628936767578125f,-1.0089000189363945825249402332701720297336578369140625f,-0.0327000118374830284206922215162194333970546722412109375f,-0.70019998743683087294442657366744242608547210693359375f,-0.756900026141106962995763751678168773651123046875f,-0.0789000013530254395988094984204508364200592041015625f,0.0458000046581028552594716529711149632930755615234375f,-1.6506999046385300022876663206261582672595977783203125f,-0.63059999770075070291142083078739233314990997314453125f,-0.39889998928308489922756052692420780658721923828125f,-0.0555000051558014462216306128539144992828369140625f,-0.57980002452134993262689022230915725231170654296875f,0.4195999409720305717286237268126569688320159912109375f,-0.2317999889671804591984027865692041814327239990234375f,-0.1589999931901691088143024899181909859180450439453125f,0.18399995876848773690426241955719888210296630859375f,0.40240001039803008353601398994214832782745361328125f,1.144799953889847454746586663532070815563201904296875f,-0.7532999683737757745660701402812264859676361083984375f,-0.8862999863445757409152747641201131045818328857421875f,0.3701000015228983475168433869839645922183990478515625f,0.465799987298250339762262228759936988353729248046875f,0.8926999488055713527501211501657962799072265625f,-0.13839999575018924193869906957843340933322906494140625f,0.4816999601751579263009261921979486942291259765625f,0.1070000458061686199329187729745171964168548583984375f,0.9615999460607767002073842377285473048686981201171875f,-0.101599998661875912109309183506411500275135040283203125f,-0.010299994845688609235168087252532131969928741455078125f,0.67710000940561254623162312782369554042816162109375f,-0.5213000069081783482971559351426549255847930908203125f,-0.43549997675046248080121813472942449152469635009765625f,0.54590000130087090735031551957945339381694793701171875f,0.19860001816749528469330243751755915582180023193359375f,0.9483000111959876132772251366986893117427825927734375f,1.767499975711106063869237914332188665866851806640625f,0.041900009414553907038936131357331760227680206298828125f,-0.0416000004127621281213578185997903347015380859375f,0.6848999855130906144751179454033263027667999267578125f,0.64279999011456923074803171402891166508197784423828125f,-1.7677000148177146598982290015555918216705322265625f,0.24479999300539478923610658966936171054840087890625f,-0.55100000047683739268222780083306133747100830078125f,0.22279997901320436337613273281021974980831146240234375f,0.303700003094226123057097765922662802040576934814453125f,-0.7385999811008574500448276012321002781391143798828125f,0.39809996935129188244673059671185910701751708984375f,-1.7638000048220163762380252592265605926513671875f,0.8220000140294434487486796570010483264923095703125f,-0.352200002360344210927678432199172675609588623046875f,-0.0755999735236165815166486936504952609539031982421875f,1.1266000213116409423008690282586030662059783935546875f,-0.0724999360367650780290205148048698902130126953125f,0.40590002704262773125520880057592876255512237548828125f,0.259400038993359505212765725445933640003204345703125f,1.450100073458255423020091257058084011077880859375f,1.2368000537313523778948365361429750919342041015625f,-0.73030002190992193700225243446766398847103118896484375f,0.9583000343300398515822280387510545551776885986328125f,-1.0126000562846664276861474718316458165645599365234375f,-1.23690001217126965826764717348851263523101806640625f,-0.041399982279538338758584359311498701572418212890625f,-1.0225000025331976161169222905300557613372802734375f,-0.00160001706480983330038725398480892181396484375f,-0.44669998157694890128510678550810553133487701416015625f,-0.551800022792816147187977549037896096706390380859375f,-0.1337999923795460954778491213801316916942596435546875f,-0.79560001595467311208409455502987839281558990478515625f,0.39150002999603772391168376998393796384334564208984375f,1.4386000283375384167783295197295956313610076904296875f,-0.9476999956138438818697977694682776927947998046875f,-0.063300010916590654375113444984890520572662353515625f,-1.0710999995410441787413446945720352232456207275390625f,-0.18670004570782250397087409510277211666107177734375f,-0.3202999757349491716240663663484156131744384765625f,-0.1914999754577866308835609743255190551280975341796875f,-0.46380001827031358541120198424323461949825286865234375f,0.87660002168416983892029747948981821537017822265625f,-0.092100007095933467127224503201432526111602783203125f,0.73300001547485627018119203057722188532352447509765625f,-0.583300021738559326678341676597483456134796142578125f,0.0827999409779909267825814822572283446788787841796875f,-0.09549996637553015421673308082972653210163116455078125f,-0.250800015684962129824953080969862639904022216796875f,0.57020000066161191654146023211069405078887939453125f,-0.306400015136599090936897482606582343578338623046875f,-0.2577999942123891941037072683684527873992919921875f,-1.2572000036269421752166408623452298343181610107421875f,1.371900013959407527153189221280626952648162841796875f,0.96400001069903407113770299474708735942840576171875f,-0.0872999386906627705684513784945011138916015625f,1.22730000589489929296860282192938029766082763671875f,0.2625000131130210689889281638897955417633056640625f,-0.309900044894219295343873454839922487735748291015625f,-0.0428999655604371810113661922514438629150390625f,-1.74590005356669397684754585498012602329254150390625f,-1.08069999687671725041582249104976654052734375f,-0.792599993848799755369327613152563571929931640625f,-0.6117999730229382038260155241005122661590576171875f,0.53269999917149579715669460711069405078887939453125f,-0.2323999912500382958313593917409889400005340576171875f,-0.50880000169277195265493674014578573405742645263671875f,-0.4518999776750802777058879655669443309307098388671875f,-0.3473000147610907362150101107545197010040283203125f,-0.2898999759763487560348949045874178409576416015625f,1.298300034832954974461927122320048511028289794921875f,-1.2784999954849478598362111370079219341278076171875f,1.024999978765845032313563933712430298328399658203125f,-1.1182999741852288622112610028125345706939697265625f,0.082499983385205144514884523232467472553253173828125f,-0.9047999874919649965221424281480722129344940185546875f,-0.65680000252723669262877592700533568859100341796875f,-0.696399982428550945456890985951758921146392822265625f,-0.19479999710321427297543550594127736985683441162109375f,-0.44740001032352427756677570869214832782745361328125f,-0.013800037530064201973800663836300373077392578125f,-0.7426999674320224897883235826157033443450927734375f,-1.333200023400783695848303977982141077518463134765625f,0.172400002574921273890140582807362079620361328125f,0.416799975928664689917013674858026206493377685546875f,0.09410003747940010043748770840466022491455078125f,-0.040000019595026525820458118687383830547332763671875f,0.588999978363514031798331416212022304534912109375f,0.79279999346733109888418766786344349384307861328125f,1.0165000287666916545248341208207421004772186279296875f,-0.565600046390295485565502531244419515132904052734375f,0.703100005449354359399194436264224350452423095703125f,0.574700032907724978059604836744256317615509033203125f,-0.6321999572098253583618543416378088295459747314453125f,0.14750004783272796515802838257513940334320068359375f,-0.4342999985039224242200361914001405239105224609375f,-1.43710007944107065469552253489382565021514892578125f,-0.73869998579025253349072954733856022357940673828125f,0.20390000537633934385439715697430074214935302734375f,-0.0138999252080918012097754399292171001434326171875f,-0.749800066095591688508648076094686985015869140625f,-0.40329999312832931490646615202422253787517547607421875f,-0.052599997164308354147266300060437060892581939697265625f,-0.6712999963469805653204502959852106869220733642578125f,-1.295200023609400119539714069105684757232666015625f,-0.0035000122487540696880614632391370832920074462890625f,1.0596999324738971548498511765501461923122406005859375f,-0.27530000404715526673982139982399530708789825439453125f,1.015900042167306782658897645887918770313262939453125f,-1.07829997262060661711302600451745092868804931640625f,-0.1490000129342081702787936592358164489269256591796875f,-1.355900003647803941930760629475116729736328125f,-0.322799980652332063613130230805836617946624755859375f,-0.145000012665986777182780542716500349342823028564453125f,1.520199994105100760322102360078133642673492431640625f,-0.03969997557550708489060298234107904136180877685546875f,-0.33919995369315170119506319679203443229198455810546875f,0.5275999488472942200445459093316458165645599365234375f,-1.845199987399578134272815077565610408782958984375f,-0.480699987787008620188089480507187545299530029296875f,0.772599948921799839496316053555347025394439697265625f,-0.23879995162487066551193493069149553775787353515625f,-0.2214000119328500826298977699480019509792327880859375f,0.0273999733686451918401871807873249053955078125f,-0.601399974977970597223020376986823976039886474609375f,0.22209996246695506982860024436376988887786865234375f,1.3467000217914577575584189617075026035308837890625f,1.3806000338733195320628510671667754650115966796875f,-0.96610001760870201081843333668075501918792724609375f,-1.244299996287375531522911842330358922481536865234375f,2.281500095617027046301927839522249996662139892578125f,0.4768999703548846280654061047243885695934295654296875f,-1.383700030046701723307478459901176393032073974609375f,0.1813999705076216883981032879091799259185791015625f,-0.91140005231499809923434440861456096172332763671875f,-0.53870006655454627519929999834857881069183349609375f,0.937400029844045690197162912227213382720947265625f,0.65259999481737607229803188602090813219547271728515625f,-0.19440002230405806216140263131819665431976318359375f,-0.307699985685944366053945486783050000667572021484375f,-0.11329999150782821271121747486176900565624237060546875f,-0.28900001740455649734684584473143331706523895263671875f,-1.2643000385507947580521204145043157041072845458984375f,1.527500020358711463330791957559995353221893310546875f,2.037900066962837986039858151343651115894317626953125f,-0.187500017881393699070713410037569701671600341796875f,-0.261299963247775934860328561626374721527099609375f,0.29689995748400743735828655189834535121917724609375f,-0.2083000811755659764656911647762171924114227294921875f,-0.3616999924361703477870833012275397777557373046875f,0.39560000749826418342536271666176617145538330078125f,-1.08799999725818619111805674037896096706390380859375f,-0.35459999387115226365807529873563908040523529052734375f,1.3848999876178798018599991337396204471588134765625f,-0.7667999934747811163759934061090461909770965576171875f,-0.87530001451522110311742608246277086436748504638671875f,-0.750800073203444551239726933999918401241302490234375f,0.4746000183463099375558158499188721179962158203125f,0.30699999946355749358417597250081598758697509765625f,0.69159998115301224430595539161004126071929931640625f,-0.8316000137865537933379300739034079015254974365234375f,-1.143000034119933783216538358828984200954437255859375f,0.14060004177093554655897378324880264699459075927734375f,0.94439998847842243634431724785827100276947021484375f,1.916799986582993664541163525427691638469696044921875f,0.833200002315640286809639292187057435512542724609375f,-0.78579997850656457814011446316726505756378173828125f,-0.0207999263525007194175486802123486995697021484375f,-0.45720001979470215047740566660650074481964111328125f,-1.0124999871850004495854591368697583675384521484375f,-0.674900010025500929344843825674615800380706787109375f,-0.619399971938133120374914142303168773651123046875f,0.97499999687075611376485539949499070644378662109375f,0.790199991888552943919421522878110408782958984375f,0.82700000058114486733984449529089033603668212890625f,0.6466000036537646877832230529747903347015380859375f,-1.48579999445080712661138022667728364467620849609375f,0.0784000122159718149106311102514155209064483642578125f,1.503599972495436087882580977748148143291473388671875f,-1.585099995338916212261892724200151860713958740234375f,-0.242400018072128187185398928704671561717987060546875f,-0.7577999927967786764071433935896493494510650634765625f,0.55150001472234766453084375825710594654083251953125f,-0.666300003182887845554205341613851487636566162109375f,-0.380800008085369956489785181474871933460235595703125f,0.63329998016431920948576816954300738871097564697265625f,-0.230200010123848830545512100798077881336212158203125f,-0.45859999752938751527153726783581078052520751953125f,0.94019998538792126652197111980058252811431884765625f,-1.241599976906180291535974902217276394367218017578125f,-0.879200005772709847207124767010100185871124267578125f,0.2831999736309054327421108609996736049652099609375f,0.448199978995323267128014776972122490406036376953125f,-0.164999980032444426569782081060111522674560546875f,-0.78900000667572012247319435118697583675384521484375f,0.7148999906539916793235533987171947956085205078125f,0.170200007888674509359816511278040707111358642578125f,-1.002299996060132425412803058861754834651947021484375f,-0.01079999951720267414856380128185264766216278076171875f,0.5088999938279386014983174391090869903564453125f,2.162700003492831424267706097452901303768157958984375f,0.976999991342424767282182074268348515033721923828125f,0.32549999636411630632437663734890520572662353515625f,-0.547299970579147920801688087522052228450775146484375f,0.97550005701184339113041232849354855716228485107421875f,0.107100027436018319093591344426386058330535888671875f,0.775600020611286300464826126699335873126983642578125f,-0.210499955162405161246397256036289036273956298828125f,0.0650000216066839442419222905300557613372802734375f,-0.6650000222027305785132966775563545525074005126953125f,0.14600001470744583542682448751293122768402099609375f,0.14330000219941130268352935672737658023834228515625f,-1.2676000015974040469046713042189367115497589111328125f,0.33040000475048980632664097356610000133514404296875f,0.13550000381469740062811979441903531551361083984375f,-0.175899989417195090624090880737639963626861572265625f,-0.20060004422962685044495856345747597515583038330078125f,0.4448000252664090492515924779581837356090545654296875f,0.012900015702844047638109259423799812793731689453125f,0.072299982574582255523409912711940705776214599609375f,0.2774999696016309247426079309661872684955596923828125f,0.285100021192431274386080986005254089832305908203125f,0.4695999891400337578062362808850593864917755126953125f,0.07409999530911459686421949299983680248260498046875f,0.040799991396069490434683757484890520572662353515625f,-0.980999977603554729199686335050500929355621337890625f,-1.03479996670484553078495082445442676544189453125f,0.245999983489513684986604857840575277805328369140625f,-0.46219995445311046200487226087716408073902130126953125f,-0.508900007387996122787399144726805388927459716796875f,-1.4829999722242359627699670454603619873523712158203125f,0.166799984720349403488626194302923977375030517578125f,-0.450999957412481222718270146287977695465087890625f,-0.82769993815124021097062723129056394100189208984375f,0.2441999599516382346564569161273539066314697265625f,1.00110000743865956707168152206577360630035400390625f,-0.61579999232739179859663636307232081890106201171875f,-0.37819997696503981909899039237643592059612274169921875f,-1.14950003051385341024115405161865055561065673828125f,-0.2611000700049099787491968527319841086864471435546875f,0.0736999544978136356121467542834579944610595703125f,0.2171999667465680516897919005714356899261474609375f,-1.3043000287532802161649669869802892208099365234375f,-0.1924999598413712487143811813439242541790008544921875f,-1.2258000544279818999626741060637868940830230712890625f,0.67189997134208656603959752828814089298248291015625f,0.659999951571226706192874189582653343677520751953125f,-0.10129996488690462541626402526162564754486083984375f,0.1327999835580591092565327926422469317913055419921875f,-0.70609993151724470550334444851614534854888916015625f,-1.12989996800422698441934699076227843761444091796875f,0.185399999907613466376687938463874161243438720703125f,1.63039998046159695377355092205107212066650390625f,0.40079996956586771972297356114722788333892822265625f,-0.604699993792176204721044996404089033603668212890625f,0.543499987065792833362820601905696094036102294921875f,0.21320003368258422238312732588383369147777557373046875f,-0.037700013178587121753793098832829855382442474365234375f,1.8825000533647837652750922643463127315044403076171875f,-0.76739999560862737215671813828521408140659332275390625f,-0.45240000570416405434315265665645711123943328857421875f,-1.172899998957663836307574456441216170787811279296875f,-1.6122999959111208312378948903642594814300537109375f,-0.9213999947965145764783301274292171001434326171875f,-1.74369998310804330543533069430850446224212646484375f,-0.8544000014796850894782664909143932163715362548828125f,0.1979999860450625259744583672727458178997039794921875f,-0.06410000309497132509051198212546296417713165283203125f,-0.3879999689087270109411065277527086436748504638671875f,1.2296000482231377315400777661125175654888153076171875f,-2.12289998871311436090536517440341413021087646484375f,-1.1139000402078027551766581382253207266330718994140625f,-1.77849998147785637314655105001293122768402099609375f,-1.7784999958761036342735906146117486059665679931640625f,-0.2206000042200084720889208256267011165618896484375f,0.139199995379149843444110956625081598758697509765625f,1.064799933624268390985889709554612636566162109375f,0.524600001135468119173310697078704833984375f,-0.6566999451994905978580163719016127288341522216796875f,-0.69159999068975441804951742597040720283985137939453125f,0.4734999521970759683853202659520320594310760498046875f,-0.11900001095980439203003697912208735942840576171875f,-0.3313000184379522039535004296340048313140869140625f,-0.60839998423904051438881879221298731863498687744140625f,0.75479999486803972530424289288930594921112060546875f,-0.20079999264404169156961188491550274193286895751953125f,0.52599999868869762309486759477294981479644775390625f,0.0865999939680097696736993384547531604766845703125f,1.37939997059702879056430901982821524143218994140625f,-1.3142999473929410925165939261205494403839111328125f,-1.060999954767525732535204952000640332698822021484375f,-0.13759999537616973697140565491281449794769287109375f,0.8212999680161476856454783046501688659191131591796875f,0.076299988207221058900131538393907248973846435546875f,-0.439200044441223180768929523765109479427337646484375f,-0.15119996969699889266536274590180255472660064697265625f,-0.7022999760553243486782548643532209098339080810546875f,-0.139799989540874636251288620769628323614597320556640625f,-1.0015000375583766878406777323107235133647918701171875f,-0.475799980704486757954185804919688962399959564208984375f,0.1192000666439538836272049593389965593814849853515625f,0.79990001345276784405768921715207397937774658203125f,0.5511000057995314449499346665106713771820068359375f,0.018500021114945697231490839840262196958065032958984375f,0.997700042384863206024192550103180110454559326171875f,0.854200008268654276122333612875081598758697509765625f,0.1154000153303140763227929710410535335540771484375f,-0.8343000086367127199338256104965694248676300048828125f,-0.0193999966740598939196615901892073452472686767578125f,1.1828999653741714137566987119498662650585174560546875f,0.3642000006318084626144582216511480510234832763671875f,-0.191199957887828730918045039288699626922607421875f,0.5028000141799449096424723393283784389495849609375f,1.425900008825957687719210298382677137851715087890625f,0.3495999709606163463604389107786118984222412109375f,-0.5009000096455213935087158461101353168487548828125f,-0.592299970281124021909135990426875650882720947265625f,0.36339997116327271697855394450016319751739501953125f,-0.826499968379736049683970122714526951313018798828125f,0.14000000134110468508197300252504646778106689453125f,-0.50140003231018714924260848420090042054653167724609375f,0.83979998924285137462675265851430594921112060546875f,0.6179000186085701340488185451249592006206512451171875f,-0.1800999706402433009344576930743642151355743408203125f,-0.23950001391768438452345435507595539093017578125f,0.70219998984336751135515442001633346080780029296875f,0.2037999999046327115337362556601874530315399169921875f,-0.51929997183084442013978332397527992725372314453125f,-0.353200008052587666185218040482141077518463134765625f,0.1708000281646847307026604312341078184545040130615234375f,-0.67080001263022470681107733980752527713775634765625f,-0.6933000018641355399751091681537218391895294189453125f,1.50000006780028360964251987752504646778106689453125f,0.4477000089317562103730097078368999063968658447265625f,-0.3467000334143641726569740058039315044879913330078125f,-0.7874000004142516218053060583770275115966796875f,-0.64010000096410468728436171659268438816070556640625f,-0.39820002243220831150694039024529047310352325439453125f,-0.6369000588491566805515731175546534359455108642578125f,-0.38119998422563072804081230060546658933162689208984375f,-1.22020001451969140049413908855058252811431884765625f,-1.2048000104397538212452900552307255566120147705078125f,0.5641000353932366095932593452744185924530029296875f,2.12220001543611314076542839757166802883148193359375f,-1.5762999826192842789396308944560587406158447265625f,-0.9757000153541566245252170119783841073513031005859375f,-0.6511000293433664154463258455507457256317138671875f,-0.7362000073596839389011847742949612438678741455078125f,-0.3150999913156031784211563717690296471118927001953125f,0.84139998552054162672675374778918921947479248046875f,-0.3499999839067464701969356610788963735103607177734375f,-1.364200018438696648814811851480044424533843994140625f,-0.070999973952769579454979975707828998565673828125f,0.4484000305443995859633332656812854111194610595703125f,0.93310001822710031404994879267178475856781005859375f,-0.4801000170946121414772278512828052043914794921875f,-0.6508000559926028216750637511722743511199951171875f,0.7534000024557112595857688575051724910736083984375f,0.2056000106275082117690544691868126392364501953125f,0.25810001554489137021164424368180334568023681640625f,0.56460001570135298631925024892552755773067474365234375f,0.19629997505694607440318577573634684085845947265625f,-0.1202999900400636423825062593095935881137847900390625f,0.0956000047788025941741807400831021368503570556640625f,-0.551300013613700290449060048558749258518218994140625f,0.41670000171214349737880411339574493467807769775390625f,-0.45260004115998808682519438661984167993068695068359375f,0.7853999950274823671492185894749127328395843505859375f,0.62339999902844400736512397998012602329254150390625f,0.9415999728828674530944908838137052953243255615234375f,0.9564999781027438974234655688633210957050323486328125f,0.26319998614788031598976658642641268670558929443359375f,1.4654000233396888841497229805099777877330780029296875f,0.84959997461140179808580796816386282444000244140625f,-0.1060000300370150527751178515245555900037288665771484375f,-0.322599963077903506558641311130486428737640380859375f,-1.161699992734194442078887732350267469882965087890625f,1.599300004725157986484873617882840335369110107421875f,0.92019997003972431315332869417034089565277099609375f,-0.80270002666413764469410807578242383897304534912109375f,0.805499987125397609588617342524230480194091796875f,1.0804000111579892262625435250811278820037841796875f,0.0707999883413312769420144832110963761806488037109375f,0.4841999932929874628229072186513803899288177490234375f,-1.0496999749422071257498600971302948892116546630859375f,-0.4816000000402329561666192603297531604766845703125f,-0.1567000098891557779978711550938896834850311279296875f,-0.414300007444619922125639277510344982147216796875f,0.242300006118416177969265845604240894317626953125f,0.78179998211264578600321328849531710147857666015625f,0.5742000187367199970367437344975769519805908203125f,0.229399993097782139983564775320701301097869873046875f,-1.2110000034570689297908074877341277897357940673828125f,0.648899971625209293080160932731814682483673095703125f,0.02180003281384645941187727657961659133434295654296875f,0.0179999892488122981148990220390260219573974609375f,-0.8090999935209752802478533340035937726497650146484375f,1.2164999456927187981847282571834512054920196533203125f,0.2056000201456245068953876398154534399509429931640625f,0.35050000321865037999913283783826045691967010498046875f,1.3143999592721467006839475288870744407176971435546875f,0.12829998871386039116515576097299344837665557861328125f,0.0348000073850158742061466909945011138916015625f,-0.3160999854966999311756126189720816910266876220703125f,0.04890002895742640021126135252416133880615234375f,-0.04959999748468398028666825894106295891106128692626953125f,0.332799997676908765331660333686158992350101470947265625f,0.32319999426901357963970440323464572429656982421875f,-0.2004000085502857242403251802898012101650238037109375f,0.0538000021770595193704167513715219683945178985595703125f,0.123500014856457784873100536060519516468048095703125f,0.17370000791847706178572252611047588288784027099609375f,-1.1281000299990180391063177012256346642971038818359375f,0.392100029261410298886403325013816356658935546875f,0.433699992048740767813796992413699626922607421875f,-0.57159995947182185016544053723919205367565155029296875f,0.2002000009596347496199086890555918216705322265625f,0.11799999759346235794765789250959642231464385986328125f,0.06020000241249789940667369592119939625263214111328125f,-0.4235999811381103707930151358596049249172210693359375f,0.105299992126226538147193423355929553508758544921875f,-0.301899993470311100995928654810995794832706451416015625f,-0.331199990260600873881458028336055576801300048828125f,0.768799957473575990007930158753879368305206298828125f,1.03639993787407913572451434447430074214935302734375f,-0.8773999976947901568991028398158960044384002685546875f,-0.081600015425681959868597914464771747589111328125f,-1.4715999430060389130403564195148646831512451171875f,0.143599980169534846563550445353030227124691009521484375f,-0.903199960108101773670341572142206132411956787109375f,-0.33969999904483538788468877100967802107334136962890625f,-0.07199998690933007150505318350042216479778289794921875f,-0.42639997609555757573929213322116993367671966552734375f,0.1779999948367476658717123427777551114559173583984375f,-0.74849999019503599129166104830801486968994140625f,1.055699972513318041222873944207094609737396240234375f,0.5203000047922134996269960538484156131744384765625f,-0.8482000196009875647717990432283841073513031005859375f,-0.753699999946355614355297802831046283245086669921875f,0.72099995465576693920439765861374326050281524658203125f,-0.4589000751137728428830087068490684032440185546875f,0.63809999803602657220125138337607495486736297607421875f,-0.819299990308284264983740285970270633697509765625f,0.67769999734610253394606615984230302274227142333984375f,0.85440002796649938687778558232821524143218994140625f,-0.33139997237026597165510111153707839548587799072265625f,-2.220700046348571987664399784989655017852783203125f,0.199699970695375927931536352843977510929107666015625f,1.783299976420401389276548798079602420330047607421875f,1.048999966479838175104077890864573419094085693359375f,0.7587999537110334813405643217265605926513671875f,-0.35059999591261148399468083880492486059665679931640625f,0.85249999493360473223901863093487918376922607421875f,-0.333800020691751964108107131323777139186859130859375f,2.03169999169111203940474297269247472286224365234375f,2.5854000084385280189280820195563137531280517578125f,-1.08720003693103617337101240991614758968353271484375f,-2.555800018776953397292572844889946281909942626953125f,2.0423999697923651552855517365969717502593994140625f,1.6718999897822734990171511526568792760372161865234375f,-1.0701999307751659529230892076157033443450927734375f,0.1755999560892587130211950352531857788562774658203125f,0.46939999876022320268731391479377634823322296142578125f,0.4202999600514776279425177563098259270191192626953125f,-1.513599915349484348325859173201024532318115234375f,0.81249995559454024629530977108515799045562744140625f,0.369200015532970216014518882730044424533843994140625f,0.26599994638562218796096203732304275035858154296875f,-1.3639000390529627537716805818490684032440185546875f,-0.44729993630647602032013310235925018787384033203125f,1.41320002816915479826320733991451561450958251953125f,-0.3519000041246425070085024344734847545623779296875f,-1.80819999642968110009633164736442267894744873046875f,0.1264000375628473449296507169492542743682861328125f,-0.6218999974191188329086799058131873607635498046875f,-1.11539996202141100667404316482134163379669189453125f,-0.6552999574065214716256377869285643100738525390625f,-0.74629999006986580400990760608692653477191925048828125f,0.341399998223781420136901942896656692028045654296875f,0.54730000839084336217155168924364261329174041748046875f,-0.07640002243816923499508675377001054584980010986328125f,-0.1348999613732100311125350344809703528881072998046875f,-1.132600027823448929353844505385495722293853759765625f,0.427399988232553262879065414381329901516437530517578125f,-0.719700034528225618402075269841589033603668212890625f,-0.715599964284896916666411925689317286014556884765625f,0.74939995048046126324692295384011231362819671630859375f,0.6450999786496165455673690303228795528411865234375f,1.3375999532431361860318475009989924728870391845703125f,0.0193000025272371811269067620742134749889373779296875f,0.83020001749992378137221749057061970233917236328125f,-0.761299994093179943632776485173963010311126708984375f,-0.08940003273487118207185631035827100276947021484375f,0.1170999793052676096749564749188721179962158203125f,0.397599997499585189331128276535309851169586181640625f,0.20680002480447257084961165674030780792236328125f,1.066399975499510599519226161646656692028045654296875f,-0.01659997482001751567537439768784679472446441650390625f,1.187100028888881553257306222803890705108642578125f,0.99730004871711164238234914591885171830654144287109375f,1.502500054519624317350690034800209105014801025390625f,0.3218000599525872029715856115217320621013641357421875f,1.26530005336999895604321864084340631961822509765625f,0.36970001189708767697084113024175167083740234375f,-0.4049999694526196236665782635100185871124267578125f,0.30699997532367806485353867174126207828521728515625f,2.12960002028345929403485570219345390796661376953125f,1.0781999582082040234354280983097851276397705078125f,-0.69809997590780170639845891855657100677490234375f,-0.77529999514371106261734212239389307796955108642578125f,-1.458300054912269327900276039144955575466156005859375f,-0.38279997845441104953323474546778015792369842529296875f,-1.3562000037834047727614006362273357808589935302734375f,0.1056000049091876669127287868832354433834552764892578125f,-0.18799999286234392315009245066903531551361083984375f,0.96499999120831514431984032853506505489349365234375f,0.5078000210344786324867527582682669162750244140625f,1.049799975049495959211753870476968586444854736328125f,-1.24390000574886716577793777105398476123809814453125f,1.1295999720409513056296191280125640332698822021484375f,1.0489999886453145183651258776080794632434844970703125f,-0.07389999256133983340788518034969456493854522705078125f,2.00660002067834142280844389460980892181396484375f,-0.8802000003449623743989604918169789016246795654296875f,0.1074000150546432497122850691084749996662139892578125f,0.16809996469467869406599902504240162670612335205078125f,-1.05859996519386800883921750937588512897491455078125f,-0.126800024580955295050443965010344982147216796875f,0.0248000196039672715642154798842966556549072265625f,0.41249997615814226747943394002504646778106689453125f,-0.5089999632388357841961123995133675634860992431640625f,-0.65299998921155932851689840390463359653949737548828125f,0.59279999398887184280937390212784521281719207763671875f,0.49929998896717986411886158748529851436614990234375f,0.2426999850153919169315486215054988861083984375f,-1.1210000426471236067982317763380706310272216796875f,1.351400074070692625127776409499347209930419921875f,0.6756999893516297817086524446494877338409423828125f,-0.600000048875809710580142564140260219573974609375f,-0.84680002093017048281353709171526134014129638671875f,0.30019998322725172101854695938527584075927734375f,0.98059999632239414069090344128198921680450439453125f,-1.8124001088202017495376594524714164435863494873046875f,-1.326200046736001780800506821833550930023193359375f,0.8582000214636327228845402714796364307403564453125f,0.19229999449551116175172182920505292713642120361328125f,-0.830499992787837815200191471376456320285797119140625f,0.865699988234042816515056983917020261287689208984375f,0.2726999617695800015582108244416303932666778564453125f,0.6596000165954232219434061335050500929355621337890625f,0.1170000667423014562018579454161226749420166015625f,0.251099972905218848939767894989927299320697784423828125f,1.001600008198618496635390329174697399139404296875f,-0.30560004281401631498482629467616789042949676513671875f,0.04900002269446890323933985200710594654083251953125f,-0.5389000259399414627381474929279647767543792724609375f,-0.0750000235438348372696282240212894976139068603515625f,0.0013000003516676628123605041764676570892333984375f,-0.686999972939491332368788789608515799045562744140625f,0.50220000265836706176969528314657509326934814453125f,-1.243400004243850442975372061482630670070648193359375f,-0.86270000334382046514036801454494707286357879638671875f,-0.25660000529289306303581952306558378040790557861328125f,0.6673999932616954122721608655410818755626678466796875f,-0.71140000783503154480058583430945873260498046875f,-0.726999985307454466010312899015843868255615234375f,1.129600012907385320204411982558667659759521484375f,0.80120002289414404827994076185859739780426025390625f,0.3435999929845332534483759445720352232456207275390625f,-1.217900033137202076005678463843651115894317626953125f,-0.21170005308389772125110539491288363933563232421875f,1.0314999834299083403266195091418921947479248046875f,1.162300060284138680799514986574649810791015625f,0.6840000576376912277964947861619293689727783203125f,-0.2947000148773195604690045001916587352752685546875f,-1.27840000051855984253279530094005167484283447265625f,1.0657000523090369714651615140610374510288238525390625f,1.22000003837049053601049308781512081623077392578125f,0.3075000026822092280553988530300557613372802734375f,-1.00490003260076132818312544259242713451385498046875f,1.671000082433224509514957389910705387592315673828125f,0.6330999884247778819457153076655231416225433349609375f,1.2265000584572562924989824750809930264949798583984375f,0.2587000216275452402214796165935695171356201171875f,-0.804800065350533611052696869592182338237762451171875f,-0.945999992355704311108866022550500929355621337890625f,-0.502199990364908810391852966859005391597747802734375f,-0.10589999329149668572114251219318248331546783447265625f,0.3885999979019165717630812650895677506923675537109375f,0.15030000892728556660671301870024763047695159912109375f,-0.9112000041186807397508573558297939598560333251953125f,0.459100007863342318703558930792496539652347564697265625f,-0.2491000032812362707801412398112006485462188720703125f,0.384599986681342098648883620626293122768402099609375f,0.5001999876230953656630617842893116176128387451171875f,-0.829799957466125448490856797434389591217041015625f,0.228999979257583863301306337234564125537872314453125f,-0.5401000233530997007846963242627680301666259765625f,0.115400011530518487035124053363688290119171142578125f,0.511299985450506255801883526146411895751953125f,-0.446600010061263930083441664464771747589111328125f,-0.70269999440312413963027893260004930198192596435546875f,-0.59280000143945155155478232700261287391185760498046875f,0.7441000020891423982760670696734450757503509521484375f,-1.5691999977260826515390590429888106882572174072265625f,0.38050001588463810975326850893907248973846435546875f,-1.7911000737488276257636243826709687709808349609375f,-1.515600042292475801986029182444326579570770263671875f,-0.3357999712795010083254965138621628284454345703125f,-0.17850000695884293833159972564317286014556884765625f,-1.1483000358760351300446700406610034406185150146484375f,-0.30929997071325743274172737073968164622783660888671875f,0.431800005060434699544202885590493679046630859375f,0.088500055536627542096539400517940521240234375f,-1.087900045393407122418238941463641822338104248046875f,-1.54589997360333786247110765543766319751739501953125f,0.13949998957291154244586550703388638794422149658203125f,-0.055299987283348617239653322030790150165557861328125f,0.5265000123381617935791609852458350360393524169921875f,-0.7894000340759761424891394199221394956111907958984375f,0.55039996794462187068575076409615576267242431640625f,0.01730003703832683203955866702017374336719512939453125f,0.5116000112533571897444062415161170065402984619140625f,-0.8711999850079419349668796712649054825305938720703125f,-0.250800038632750244005364947952330112457275390625f,-1.2467999830439684405547495771315880119800567626953125f,-0.00669996691793228738021070967079140245914459228515625f,-0.50450000661611593688604671115172095596790313720703125f,-0.2973999733313916760124584470759145915508270263671875f,-0.96700000214576675716671161353588104248046875f,-0.5332000245928762982572379769408144056797027587890625f,1.47470003030002061450431938283145427703857421875f,-0.44159999754428813645290574640966951847076416015625f,-0.417300010663270715127737275906838476657867431640625f,0.991399968421459387712957322946749627590179443359375f,-0.08259997656345408056921542083728127181529998779296875f,-0.09419999943971613109994223123067058622837066650390625f,-0.3610000042766332573052068255492486059665679931640625f,0.20479998171776514492847809378872625529766082763671875f,0.680400002645701107439890620298683643341064453125f,0.6502000171646475923381558459368534386157989501953125f,0.28329997768700021509857833734713494777679443359375f,-0.01799998384714118770943969138897955417633056640625f,0.26299997729063040452501809340901672840118408203125f,0.5649000174760814285690457836608402431011199951171875f,1.1894999658614400228628937838948331773281097412109375f,-0.0270000081062320429481360406498424708843231201171875f,-0.0922000183045863508368711336515843868255615234375f,0.1494000031933186978250205356744118034839630126953125f,-0.4556000008299940784439741037203930318355560302734375f,0.868700019615888496815614416846074163913726806640625f,0.25499999955296548392880140454508364200592041015625f,-1.41560001092553111590177650214172899723052978515625f,0.6991000168040393258905851325835101306438446044921875f,-0.6646999858200552946385641917004249989986419677734375f,1.030599989691376894285212983959354460239410400390625f,-0.1322000077992679933913677814416587352752685546875f,0.57319997787773591113591464818455278873443603515625f,-0.2584000003322957095264200688689015805721282958984375f,0.0132999901667239794900154947754344902932643890380859375f,-0.29969999565482130510218894414720125496387481689453125f,0.353699983592331623771798376765218563377857208251953125f,0.1424999717622992267962445112061686813831329345703125f,-0.00640000172555466040336114019737578928470611572265625f,0.7296999942392112803446480029379017651081085205078125f,0.2842999921709299382399649402941577136516571044921875f,0.8290000229179856905403767086681909859180450439453125f,0.18839997175931966211237522657029330730438232421875f,0.4009999984651801963053685540216974914073944091796875f,-0.3681999917730689464434590263408608734607696533203125f,0.01820000313520420132107346944394521415233612060546875f,0.1450999943703414896134518130565993487834930419921875f,-0.45539997241497076618799155767192132771015167236328125f,0.19769998732507243488498716033063828945159912109375f,-0.781000000551343109833624112070538103580474853515625f,-0.721799980622530146234794301562942564487457275390625f,-1.1787999850779768973296768308500759303569793701171875f,0.7955000627115371703013124715653248131275177001953125f,-0.7093999803945416626760334111168049275875091552734375f,-0.595999980770051340783766136155463755130767822265625f,-0.0720000042319295374682042165659368038177490234375f,-1.554400036460160894336013370775617659091949462890625f,-1.445100046896934831153203049325384199619293212890625f,0.0574999958276745104512883699499070644378662109375f,0.84810005607605010169436354772187769412994384765625f,-1.009500024795531913923696265555918216705322265625f,-0.71640000120401392535995910293422639369964599609375f,0.5325999515295034569817289593629539012908935546875f,1.24480000947117819265486104995943605899810791015625f,0.0438000280678270403900143037390080280601978302001953125f,0.5435999827772379777712785653420723974704742431640625f,-0.400000006593763390849716188313323073089122772216796875f,-0.38230000470280600666228565387427806854248046875f,0.8579999869018790636943094796151854097843170166015625f,2.1275000386685132269803943927399814128875732421875f,-0.0501999982029202040934023898444138467311859130859375f,-2.30300006677210422623147678677923977375030517578125f,0.805699987339973944244775339029729366302490234375f,1.769400042197108557928686423110775649547576904296875f,0.5908999716103071619954789639450609683990478515625f,-1.215800024998187733871191085199825465679168701171875f,-0.50079999970346733562109875492751598358154296875f,-1.0779999767690895406957451996277086436748504638671875f,0.8978999723404645916247091008699499070644378662109375f,-1.55219996048808095423510167165659368038177490234375f,0.577500008940696307746520687942393124103546142578125f,-0.646000001445413030154441003105603158473968505859375f,-0.18959997304677944640616260585375130176544189453125f,0.442000057205557794759442913345992565155029296875f,0.521599945688247856878660968504846096038818359375f,-1.321200035038590403502212211606092751026153564453125f,0.3732000338315959453439063508994877338409423828125f,1.061199986088276592255397190456278622150421142578125f,-1.40420000987052873142602038569748401641845703125f,0.13989997426271383318407970364205539226531982421875f,0.0585000094175338603008640347979962825775146484375f,0.573100033426284571902442621649242937564849853515625f,1.1650999828219408271934298682026565074920654296875f,0.4163999781072131867176722153089940547943115234375f,0.57479998071193705300174769945442676544189453125f,-0.6414000560402879358434802270494401454925537109375f,-1.083700050237775602823830922716297209262847900390625f,0.715200029048323582259172326303087174892425537109375f,-0.26210004792511565341328605427406728267669677734375f,-0.619199980925022597233464693999849259853363037109375f,-0.920499952256680131768007413484156131744384765625f,0.200099996978044369910776367760263383388519287109375f,-0.0402999624729165617509352159686386585235595703125f,0.720799970012903745697485646815039217472076416015625f,0.37840000767111714452539672493003308773040771484375f,0.2180000194609164054782013408839702606201171875f,-0.89969997002482404724332809564657509326934814453125f,0.3084000055849553945108709740452468395233154296875f,-1.464499970555305452535321819595992565155029296875f,-1.1441999906480313509149482342763803899288177490234375f,-0.1601000255882727429934675456024706363677978515625f,0.46780000628232887294188913074322044849395751953125f,0.898300010097026646604945199214853346347808837890625f,0.68499999243766040990522014908492565155029296875f,0.8271000113800166531063950969837605953216552734375f,0.09210002274215234319143519314820878207683563232421875f,0.603900016216933455126536500756628811359405517578125f,1.19919998498558921795620335615240037441253662109375f,-0.23709996492564666681346352561376988887786865234375f,-1.9435000106096269600897130658267997205257415771484375f,0.083399965240061657567594011197797954082489013671875f,-0.2643000378802409233713888170314021408557891845703125f,0.28320000022947755535795977266388945281505584716796875f,-0.0955999430879941325400750429253093898296356201171875f,0.2232000019803643819926719515933655202388763427734375f,0.14610001384615856068194261752068996429443359375f,-0.4351999857231974022653275824268348515033721923828125f,1.151699967327714180242992370040155947208404541015625f,1.010700010660290359965074458159506320953369140625f,-1.3048999447584161970326022128574550151824951171875f,-1.6740999915093184302605777702410705387592315673828125f,1.309299950522184730061781010590493679046630859375f,0.2887999868288637639324178962851874530315399169921875f,0.4159000105023384907809713695314712822437286376953125f,-0.2738999785542486353762114958954043686389923095703125f,-0.1745000322461118802408464034670032560825347900390625f,0.52369999160170532803704190882854163646697998046875f,-0.605499997109174348253191055846400558948516845703125f,0.700099986658990314225547990645281970500946044921875f,0.35789997241497084612404933068319223821163177490234375f,-0.02049997770041322997514043890987522900104522705078125f,-1.21579998923540077981897411518730223178863525390625f,-0.1485999567747118277338813641108572483062744140625f,1.19349998587369920954870394780300557613372802734375f,-0.11230003435611646267489049932919442653656005859375f,-1.553599939228595072648886343813501298427581787109375f,0.3505999792978169438129043555818498134613037109375f,0.1574000021278851857431391181307844817638397216796875f,0.24559998298585394849169460940174758434295654296875f,-0.6318000150814648829822317566140554845333099365234375f,0.0859000063672661351432680021389387547969818115234375f,-0.8689000124916435030542061213054694235324859619140625f,0.702999965742230781273747197701595723628997802734375f,0.351499968193471790112170083375531248748302459716796875f,0.5058999999970195116105742272338829934597015380859375f,0.7461000195086000541522253115545026957988739013671875f,0.7212000028520824113087428486323915421962738037109375f,-0.91780004726052322183704745839349925518035888671875f,-0.6711999886959787442464175910572521388530731201171875f,0.31270003195405049467581193312071263790130615234375f,0.6681000074237586350278661484480835497379302978515625f,0.1991999633789067747358103588339872658252716064453125f,-1.092399998998641930114672504714690148830413818359375f,1.2282999719500542479266869122511707246303558349609375f,0.8955999736353756901507949805818498134613037109375f,0.563199996019899895571825254592113196849822998046875f,0.770799986962974248427826751139946281909942626953125f,-0.441099996858834497714951794478110969066619873046875f,0.29849998244643172196077784974477253854274749755859375f,0.4690999415159229357641379465349018573760986328125f,-0.740999989561737404386576599790714681148529052734375f,-0.0532999579429631609173156903125345706939697265625f,0.3066999699354173714027638197876513004302978515625f,0.44319999192208048288676991433021612465381622314453125f,0.37669998688548833598588316817767918109893798828125f,-0.4011999731242659184005105998949147760868072509765625f,0.5919999483898290559835686508449725806713104248046875f,-0.8826999612852934884443811824894510209560394287109375f,0.5954999723732472549642125159152783453464508056640625f,-0.85740001837015100694117109014769084751605987548828125f,1.4076999835625285673046391821117140352725982666015625f,0.83289998850822488662970499717630445957183837890625f,-0.6221999364972123913730683852918446063995361328125f,0.2369000071048734668011093162931501865386962890625f,0.84759996609538823530982654119725339114665985107421875f,-0.108500030875205322189458456705324351787567138671875f,0.154700005936622364544064112124033272266387939453125f,-0.531299994689226284805272371158935129642486572265625f,-0.5331999975845216344083610238158144056797027587890625f,-0.223299999219178868514745772699825465679168701171875f,-0.96560000764727593747238643118180334568023681640625f,0.41800005254149485978132361196912825107574462890625f,-1.16330001710057207020554415066726505756378173828125f,0.0932000056684021416231189505197107791900634765625f,-0.911200013692676780152623905451036989688873291015625f,-2.1362000218510619475864587002433836460113525390625f,0.6464999990388751172076808870770037174224853515625f,0.197799956512451302614863379858434200286865234375f,-0.4486000418230886577219962418894283473491668701171875f,0.2119999854564671437628931016661226749420166015625f,0.3438999969944356305262544992729090154170989990234375f,-0.227899989402294611551269554183818399906158447265625f,-0.48079995518624774053506598647800274193286895751953125f,-1.222100001358985110755384084768593311309814453125f,-0.5343000220477576167610322954715229570865631103515625f,-1.88289998493194499218361670500598847866058349609375f,-1.360200025807321022597307091928087174892425537109375f,1.10939995197057772458038016338832676410675048828125f,1.6899999250844126574833126142038963735103607177734375f,-0.10869995881915173185916501097381114959716796875f,0.6486999711871153095898989704437553882598876953125f,0.4399999761581423474154917130363173782825469970703125f,-0.6357999765694142269012445467524230480194091796875f,0.838500004947185129111630885745398700237274169921875f,-1.643599965268373264137835576548241078853607177734375f,0.438199978995322592112415804876945912837982177734375f,-1.489799985182283847962025902234017848968505859375f,-0.096999999612570331208871721173636615276336669921875f,-0.02120000367164553978227559127844870090484619140625f,0.88670001910924867161156726069748401641845703125f,-0.7615999807804827437252015442936681210994720458984375f,0.45910005103945739080018029198981821537017822265625f,-0.8438999993786211462065693922340869903564453125f,-1.115300006058811188580648376955650746822357177734375f,-0.42090001035332724743653898258344270288944244384765625f,-0.33399997337162457000658832839690148830413818359375f,-0.393600003340840054466553965539787895977497100830078125f,-0.3755000025480981218350962080876342952251434326171875f,-1.015599987158179562385384997469373047351837158203125f,0.07280002811253016847814478751388378441333770751953125f,-0.5365000090226528239867320735356770455837249755859375f,0.06919999064803139532386921928264200687408447265625f,-0.21579998264163735566256718811928294599056243896484375f,-2.107899975395202574190989253111183643341064453125f,-0.27470005227923355306529629160650074481964111328125f,-0.421100006544590232948621633113361895084381103515625f,-0.3194000079616898801759816706180572509765625f,-0.251900016045569774547630004235543310642242431640625f,0.58789995013773499454856619195197708904743194580078125f,-0.663399993664026244033493640017695724964141845703125f,-0.26890000358819943215848979889415204524993896484375f,-0.7825999736577273768745044435490854084491729736328125f,0.3245999792307616704789552386500872671604156494140625f,-0.0632999859571459211338151362724602222442626953125f,0.097800009188055536668571221525780856609344482421875f,0.060399994915723453914324636571109294891357421875f,0.8304999611973773454565161955542862415313720703125f,1.0451000280469646686043461158988066017627716064453125f,0.73919993692934582441012025810778141021728515625f,-0.1364999981820587127145927297533489763736724853515625f,0.403299956229330458512549739680252969264984130859375f,0.66140000120401343242093616936472244560718536376953125f,-0.579099987486004774694947627722285687923431396484375f,0.5769999727159740299242685068747960031032562255859375f,-0.2861000082582230152894453567569144070148468017578125f,0.2212000030755996160536369643523357808589935302734375f,-0.41709997289776812312567244589445181190967559814453125f,-0.90950000870227842852955291164107620716094970703125f,-1.398399970865250363516452125622890889644622802734375f,1.05970006285905782306144828908145427703857421875f,-1.052499956600367880810154019854962825775146484375f,-1.91799999610334648281195768504403531551361083984375f,-0.0167000302478665929584167315624654293060302734375f,0.8435000056922448141705217494745738804340362548828125f,0.2285000045746560515258494206136674620211124420166015625f,-0.39789997763037643352390659856610000133514404296875f,-0.46529997912496323664299779920838773250579833984375f,-1.128200003023445585625950116082094609737396240234375f,-0.41360000471919777798035511295893229544162750244140625f,0.1432000561833373097897492698393762111663818359375f,0.72530004606843012648909052586532197892665863037109375f,-2.59539991012811643855684451409615576267242431640625f,-0.8452000240564345734384232855518348515033721923828125f,0.787799945373833576667266243021003901958465576171875f,-0.21889997613281053645550855435431003570556640625f,-0.4204999732673169177132876939140260219573974609375f,0.892199998746812550365348215564154088497161865234375f,-0.798400025105475918962838477455079555511474609375f,-0.60790005466937990785680767658050172030925750732421875f,0.75379999461471947075352773026679642498493194580078125f,-0.096999985456466930600072373636066913604736328125f,0.8549999795854095197000788175500929355621337890625f,-1.67929996572136897015070644556544721126556396484375f,-0.59330005986690448338549686013720929622650146484375f,-0.18139996752738962726425597793422639369964599609375f,1.817799965192377964484649055521003901958465576171875f,0.2679999841451643494139034373802132904529571533203125f,-1.683899931764603508099753526039421558380126953125f,0.9288999660745262954009149325429461896419525146484375f,0.1461999951779839879151268178247846662998199462890625f,0.7345999863088135128919020644389092922210693359375f,1.2931000167183572813200953532941639423370361328125f,-0.76909995585829060704696757966303266584873199462890625f,0.317500037439166382657873555217520333826541900634765625f,0.03049999643862288944973215620848350226879119873046875f,0.6328999761402609447458189606550149619579315185546875f,0.0898000017598269695273671686663874424993991851806640625f,-0.12989994906112467543124466828885488212108612060546875f,-0.064499998532235591763850379720679484307765960693359375f,-0.28480000852122888677087075848248787224292755126953125f,0.7962000055156643885112544012372381985187530517578125f,0.6503000007688992667453931062482297420501708984375f,0.7011000118717551909952589994645677506923675537109375f,1.11460003404840879426274113939143717288970947265625f,-0.3511999913409351581350392734748311340808868408203125f,-0.465799974818528195186217999435029923915863037109375f,0.875899967773258847358874845667742192745208740234375f,-1.0795999666392805949044486624188721179962158203125f,-1.3219999797567723032187814169446937739849090576171875f,-0.19620002661943314326009613068890757858753204345703125f,1.3742999663174153557321233165566809475421905517578125f,0.57499998629093085344976543638040311634540557861328125f,-0.2880999929510057455672722426243126392364501953125f,0.597300006937980132448728909366764128208160400390625f,0.8854999783709647243057361265528015792369842529296875f,-0.19809999736547456716806436816114000976085662841796875f,-0.7043999552488333648625484784133732318878173828125f,0.54299996205419365224997818586416542530059814453125f,0.5114999695718294159263450637809000909328460693359375f,-0.51389997631907491371094920395989902317523956298828125f,0.8258999626323586529252906984766013920307159423828125f,0.094299982354789813054907199330045841634273529052734375f,1.0184999621994794694046504446305334568023681640625f,0.46630001868009551824201253111823461949825286865234375f,0.72989999130591665110756594003760255873203277587890625f,-0.788899994498490908512167152366600930690765380859375f,0.586999981433153461551910368143580853939056396484375f,0.215199989318102524737952307987143285572528839111328125f,0.520499998338520786234084880561567842960357666015625f,-0.482600044102966918568853316173772327601909637451171875f,-0.1050999904029073228617363611192558892071247100830078125f,-0.719700021284819246858432961744256317615509033203125f,0.294400015524029667890459904810995794832706451416015625f,-0.77520004715323498345469488413073122501373291015625f,0.52099999716132894267417441369616426527500152587890625f,-0.79260001113414713103111353120766580104827880859375f,-0.468400007149577246590155255034915171563625335693359375f,0.42130003358125722723315220719086937606334686279296875f,-0.142000001512467888264978910228819586336612701416015625f,-0.5321000096105039656180224483250640332698822021484375f,-0.8434999704509975604338478660793043673038482666015625f,0.2851999621793626094046203434118069708347320556640625f,1.4038999551966793166712932361406274139881134033203125f,-0.8249000265657902009053259462234564125537872314453125f,0.55100001426041078644857407198287546634674072265625f,-0.970599979819357638888277506339363753795623779296875f,-0.15080000544041372023684743908233940601348876953125f,0.5971999777734282588426140137016773223876953125f,0.707199913251400857916451059281826019287109375f,0.93970004259347916786282439716160297393798828125f,0.066599993520974454241923012887127697467803955078125f,0.357300018523634133060795647907070815563201904296875f,-0.91189999011754963031961551678250543773174285888671875f,-0.963100015321373792431813853909261524677276611328125f,0.266599999779462581983580093947239220142364501953125f,0.94130000847280026476937564439140260219573974609375f,-1.0383999947071076430660241385339759290218353271484375f,0.511400028324127564616219387971796095371246337890625f,0.2140000051110984014002269759657792747020721435546875f,-1.178000031530857416584012753446586430072784423828125f,0.899800039124489359210201655514538288116455078125f,1.0439000482544305015153440763242542743682861328125f,-0.2804000215142963270409381948411464691162109375f,-1.2695000544860963742621606797911226749420166015625f,-0.25060003346353820763425801487755961716175079345703125f,0.819200035649538360615906640305183827877044677734375f,1.3594000469654801133145838321070186793804168701171875f,-0.01119998869597882418247536406852304935455322265625f,-0.37020001854300499388727985206060111522674560546875f,0.4545000224858524173754403818747960031032562255859375f,-0.69900005003810006254383324630907736718654632568359375f,0.3867000697731981517790700308978557586669921875f,0.45849999179691069972619743566610850393772125244140625f,-0.8128000530719763361275909119285643100738525390625f,0.80850001329183596165961489532492123544216156005859375f,-0.645900010353326781142868640017695724964141845703125f,-1.230900037175416628087987191975116729736328125f,0.40440004502832938015899344463832676410675048828125f,-0.4461999805003404873104955186136066913604736328125f,1.6464000266849996290119406694429926574230194091796875f,-1.35549999181926228430938863311894237995147705078125f,1.387200033988058134326593062723986804485321044921875f,-0.36580001639276726876914835884235799312591552734375f,-0.9564999911785125874530422152020037174224853515625f,-1.282600003683565859091686434112489223480224609375f,-0.3537000220000745542847653268836438655853271484375f,1.6265999853253365614591530174948275089263916015625f,-1.495200023832917057831082274788059294223785400390625f,0.14489997552931299651390872895717620849609375f,-0.34280002084821459273911159471026621758937835693359375f,0.29080000618547163782068309956230223178863525390625f,-0.871199998903274153150277925305999815464019775390625f,-0.38300001901388192049324743493343703448772430419921875f,0.47660001430809473266236864219536073505878448486328125f,-0.8287999894365662978401587679400108754634857177734375f,-0.096200033958256880861625859324703924357891082763671875f,-0.09220001539885969155108114136965014040470123291015625f,-0.707499978914856608724903708207421004772186279296875f,-0.7496000472545627513198951419326476752758026123046875f,1.36590001176893682810487007373012602329254150390625f,0.410600032941997650226539917639456689357757568359375f,0.3425000125914807735938438781886361539363861083984375f,-0.48509998461008052572651649825274944305419921875f,-0.2103999828457834286865590911475010216236114501953125f,1.579100002983211847862321519642136991024017333984375f,-0.54139997784644300349299328445340506732463836669921875f,0.712699975366890559058674625703133642673492431640625f,-0.55329999758005055099374658311717212200164794921875f,1.490700044336914942277871887199580669403076171875f,-0.018300011438131758012559657800011336803436279296875f,0.8165000166594982289325344027020037174224853515625f,0.434199984240532188550787395797669887542724609375f,0.74059998633861567896019550971686840057373046875f,-0.9924999904260036753811391463386826217174530029296875f,-1.5740999418884522587092078538262285292148590087890625f,0.217900076238810580520066650933586061000823974609375f,-0.1638000064238911424041589270927943289279937744140625f,1.184300007407367072431725318892858922481536865234375f,-0.536300028738378120607421806198544800281524658203125f,-1.1798999861836427793804205066408030688762664794921875f,0.5929999568760404571321487310342490673065185546875f,1.115500046283007140601739592966623604297637939453125f,0.0784999793171881066200512577779591083526611328125f,1.96899997761845657606727399979718029499053955078125f,0.71399999402835956363588820750010199844837188720703125f,-0.482700036461651882380152756013558246195316314697265625f,0.52650003347918439278174673745525069534778594970703125f,1.273700062941015342943273935816250741481781005859375f,-0.3660000158995391217331416555680334568023681640625f,-1.63420004965662979401486154529266059398651123046875f,1.1506000162154437216344149419455789029598236083984375f,-0.5637000202491881939437234905199147760868072509765625f,-0.490200000363588461738117985078133642673492431640625f,1.58000001486390839744444747339002788066864013671875f,-1.212600013443827950965214768075384199619293212890625f,-0.330199971641599976379666259163059294223785400390625f,-0.098800020319223857967472213204018771648406982421875f,0.36250003371387806705428147324710153043270111083984375f,-0.1412999974459407948046418823651038110256195068359375f,-0.789500054746867352406525242258794605731964111328125f,-0.8885000305771828532641620768117718398571014404296875f,0.523100020164251500176533227204345166683197021484375f,0.6860000170916318307234860185417346656322479248046875f,0.481600006932020097139002245967276394367218017578125f,0.6347000329449781208523972964030690491199493408203125f,-0.1107000150561336937471423880197107791900634765625f,-0.4330000049322844102306362401577644050121307373046875f,0.5752999901890749612221043207682669162750244140625f,0.75359999708235225757135822277632541954517364501953125f,-0.91500003056600676121234982929308898746967315673828125f,-0.38229999812766812805620020299102179706096649169921875f,0.26309998142123180997486997512169182300567626953125f,0.61159999948143894954455390688963234424591064453125f,-1.46520002815425431919038601336069405078887939453125f,0.667200009959935780301520935609005391597747802734375f,0.591699984315038118865004435065202414989471435546875f,-0.639099958130717293869338391232304275035858154296875f,0.8596000316083429737545884563587605953216552734375f,-0.376499953627586592119769193232059478759765625f,0.571999976068735094258954632095992565155029296875f,-0.38959998362660375192945139133371412754058837890625f,0.8756000190094113122540875338017940521240234375f,-0.680099997648596410471100170980207622051239013671875f,0.40009999325275413806224378276965580880641937255859375f,-0.3192000031650066294020007262588478624820709228515625f,-0.9576000224217775524948592646978795528411865234375f,0.3936000152617695757584215243696235120296478271484375f,0.96960002035796666319811265566386282444000244140625f,0.114900013303756765026264474727213382720947265625f,-0.3907000514149669978536394410184584558010101318359375f,0.254700036334991164466146074119023978710174560546875f,1.0356000214867295827758653103956021368503570556640625f,0.907699989932775341827664306038059294223785400390625f,-0.84380003093630084887166731277829967439174652099609375f,0.389600001433491993640956252420437522232532501220703125f,-0.3466000178098678219384964904747903347015380859375f,-0.6293000240221624519421084187342785298824310302734375f,1.04040004721880041671511207823641598224639892578125f,-0.6128000236794355348735052757547236979007720947265625f,1.07170000599622738945981836877763271331787109375f,-1.2387000214040282397576220319024287164211273193359375f,0.6760000255107883049277006648480892181396484375f,-0.4537999499857414686943002379848621785640716552734375f,-0.439600031422079151610660119331441819667816162109375f,0.9627999659746888649891616296372376382350921630859375f,-0.4745000312775375927998311453848145902156829833984375f,1.0507999940410261974221839409437961876392364501953125f,-0.189100005888939382003854916547425091266632080078125f,1.038399982190131964898682781495153903961181640625f,0.778000003367662618103395288926549255847930908203125f,-0.70380001510381617890743655152618885040283203125f,1.37560000198483489697309778421185910701751708984375f,-0.91509997745752347242387259029783308506011962890625f,-0.80590002659559178965764658642001450061798095703125f,-1.3077999316275121799435510183684527873992919921875f,0.45520002852678309590572780507500283420085906982421875f,-0.319199988710880067088737632730044424533843994140625f,-1.3081999956846235289020796699333004653453826904296875f,0.48850001090765038380681062335497699677944183349609375f,0.433000001952052127052184005151502788066864013671875f,-0.6675999913156029919036882347427308559417724609375f,0.28170000078082058081463401322253048419952392578125f,-0.577700014966726183018863594043068587779998779296875f,0.833299986292421746014724703854881227016448974609375f,-0.49000000067055271557592277531512081623077392578125f,-0.747600011208653381089561662520281970500946044921875f,0.2553000086665153656184656938421539962291717529296875f,1.30420000696480276047850566101260483264923095703125f,-1.3277000266268854655749009907594881951808929443359375f,0.3716000383734705536653564195148646831512451171875f,0.061299975951015550634792816708795726299285888671875f,-1.156400007015466524507019130396656692028045654296875f,1.1108999972403046374580526389763690531253814697265625f,-0.90280005545616148765475372783839702606201171875f,1.1603000423431402143847890329197980463504791259765625f,-0.05300003123283403283494408242404460906982421875f,-1.2401999911621210959111749616567976772785186767578125f,0.9497000309705736498244732501916587352752685546875f,-1.02480002094507227639041957445442676544189453125f,-0.81020001630783067270158426254056394100189208984375f,0.771399952626229179486472276039421558380126953125f,-0.7447999813079835274010065404581837356090545654296875f,-0.4512000120162955596470055752433836460113525390625f,-0.9818999777495864833554151118732988834381103515625f,-0.0723000149846075856885363464243710041046142578125f,-0.3874000460863111783282874966971576213836669921875f,2.27249996364116668701171875f,0.144599981242418351712331059388816356658935546875f,0.7722000187516204761095650610513985157012939453125f,0.40350000703334831797519655083306133747100830078125f,-1.0849999880790708317590542719699442386627197265625f,-0.91189999026656121117184738977812230587005615234375f,0.1309000146001579878429765813052654266357421875f,-0.4688000098511577728999100145301781594753265380859375f,-0.4938000056415796645836735478951595723628997802734375f,0.665600033910572985718090421869419515132904052734375f,-0.340800008383393304711717064492404460906982421875f,-0.741200014922023253660654518171213567256927490234375f,-0.43680001429915360500189080994459800422191619873046875f,0.282100008437037474973152484380989335477352142333984375f,-0.15220000191330906869069394815596751868724822998046875f,0.6451999761119482368343369671492837369441986083984375f,0.317800018724799127056712677585892379283905029296875f,0.35020003247559083892070930232875980436801910400390625f,0.13899999505281446232629605219699442386627197265625f,-0.166899986243247866468664142303168773651123046875f,0.525299990300834185319445168715901672840118408203125f,-1.26070000879764609891253712703473865985870361328125f,1.64910001263171412944075200357474386692047119140625f,-0.70690000472068703629702213220298290252685546875f,1.27419998301118564398848320706747472286224365234375f,0.344000006526708812515380486729554831981658935546875f,-0.4131000145763159547840359664405696094036102294921875f,0.433600028449297791866001716698519885540008544921875f,-0.70049998712539685019606849891715683043003082275390625f,0.0539000023961069274491819669492542743682861328125f,-0.4966999830484386535545127117075026035308837890625f,0.9617000159800053182834744802676141262054443359375f,-1.109300011169910504804647644050419330596923828125f,-1.39349994444847169461354496888816356658935546875f,0.561100008779764181099380948580801486968994140625f,0.1149999853968617369304183739586733281612396240234375f,1.6026999102115642159560593427158892154693603515625f,1.2203000037491322782301494953571818768978118896484375f,-0.2689000101447103219243217608891427516937255859375f,-0.40409997906684846924463272443972527980804443359375f,0.034600007319451009379918104968965053558349609375f,0.368899992859363923258797512971796095371246337890625f,0.014600023785233562367835702389129437506198883056640625f,0.366000000514089929293248815156402997672557830810546875f,-0.25310001450181018878282657169620506465435028076171875f,0.12950001794099830210171830913168378174304962158203125f,0.5802999977886675875282662673271261155605316162109375f,-0.9772000137224789995116225327365100383758544921875f,0.65740001326650376878291126558906398713588714599609375f,0.7138999991923566934559630681178532540798187255859375f,-0.8839999965056772879279378685168921947479248046875f,-0.72719999319612937949131037385086528956890106201171875f,-0.055399998864531507791753028868697583675384521484375f,1.63050002873688892890413626446388661861419677734375f,-0.6037000127613547650895498009049333631992340087890625f,-0.444799947333336831434280611574649810791015625f,-0.698799966600538002836628947989083826541900634765625f,0.1964000381588935084664626629091799259185791015625f,-1.371899968436359973367189013515599071979522705078125f,0.4871999761343002166569249311578460037708282470703125f,-0.74840002674460404108458533301018178462982177734375f,1.87130001249611410685247392393648624420166015625f,-0.069399993544816851454015704803168773651123046875f,0.28339997682571382853211616748012602329254150390625f,-0.3541999811112883644881321743014268577098846435546875f,-1.35289999238252622859590701409615576267242431640625f,0.29750006314367094528705592892947606742382049560546875f,-1.0644000368326904659710407941020093858242034912109375f,-0.204300018471479827297798692598007619380950927734375f,-0.46829995138645141850020081619732081890106201171875f,1.3813000364124776719876308561651967465877532958984375f,-0.848499999433755647260113619267940521240234375f,0.085500020168721935220901286811567842960357666015625f,-0.066300024864077489183955549378879368305206298828125f,0.0464000058420002137182791557279415428638458251953125f,0.3787000234156849831634872316499240696430206298828125f,0.8754000246062876300356947467662394046783447265625f,1.6228000189483164916737223393283784389495849609375f,-1.2832999937057494310721494912286289036273956298828125f,-0.4248999942302706944730061877635307610034942626953125f,2.00990005554854889169291709549725055694580078125f,0.9847000535458325654047939679003320634365081787109375f,-0.882600011134147877100986079312860965728759765625f,-0.7382999479591842995063188936910592019557952880859375f,-0.116600012743473424592366427532397210597991943359375f,0.659700035068393031423283900949172675609588623046875f,1.6328000258028507740704071693471632897853851318359375f,1.2676000069618227339418581323116086423397064208984375f,-0.32130002203285723538783713593147695064544677734375f,-0.3692000301361086922469212368014268577098846435546875f,-0.29409999419152743627137169823981821537017822265625f,-0.937499987855553218452087094192393124103546142578125f,0.1559000415712599174611341368290595710277557373046875f,-0.59660002943277401499244660953991115093231201171875f,0.2803999852299685624501535130548290908336639404296875f,0.01280003906488413889519506483338773250579833984375f,-0.66899997746944439569460882921703159809112548828125f,-1.143099991703033335710415485664270818233489990234375f,-0.403399974963068785882569500245153903961181640625f,-0.307500027492642491466767751262523233890533447265625f,0.4488999802678821371415551766403950750827789306640625f,1.5004999883174898744897518554353155195713043212890625f,1.31440002673715383707531145773828029632568359375f,1.32789999789595558610244552255608141422271728515625f,0.720800006818771432648418340249918401241302490234375f,-0.1337000120162965544068356393836438655853271484375f,1.0114000124916435741084796973154880106449127197265625f,1.02560001837611114439141601906158030033111572265625f,0.98139999516904297127695144808967597782611846923828125f,-1.18889997050762108443677789182402193546295166015625f,0.84220001396462340625959086537477560341358184814453125f,0.5703999628782270292504108510911464691162109375f,0.86099999123811710166620514428359456360340118408203125f,0.08560008528232554425585476565174758434295654296875f,-1.026500063747167956762496032752096652984619140625f,1.08939999332130010856189983314834535121917724609375f,-0.80640005842447326500632698298431932926177978515625f,1.249800011855364711976790204062126576900482177734375f,-0.3139000266849993892037673504091799259185791015625f,-0.12480000343620811786848889823886565864086151123046875f,-1.951600012445450005316160968504846096038818359375f,1.275800061506032090363760289619676768779754638671875f,1.5564999911785122321816743351519107818603515625f,0.045600002431869679497822289704345166683197021484375f,-0.1549000094294541440831380896270275115966796875f,1.085700029584766301837817081832326948642730712890625f,0.14679996728599087330024985931231640279293060302734375f,0.1876000269293793820679638884030282497406005859375f,-0.564200035500526109188967893715016543865203857421875f,-0.713700006651877938423922387300990521907806396484375f,-0.13950002516806103702862174031906761229038238525390625f,1.4423999924421320173451022128574550151824951171875f,0.069799971622229062262476873002015054225921630859375f,-2.27029993952512842980695495498366653919219970703125f,-0.628199955749512373159859635052271187305450439453125f,0.6506999697566033802331730839796364307403564453125f,0.440699996355176093487671096227131783962249755859375f,-0.70339999917745554824932696647010743618011474609375f,-0.25270001792833252096670548780821263790130615234375f,-0.408999993860721389182799612171947956085205078125f,-1.076900002634525232991791199310682713985443115234375f,0.30729995731711401418806417495943605899810791015625f,-0.55790001216381757132012353395111858844757080078125f,-0.401299947452546490467284456826746463775634765625f,0.83619996671676577459919599277782253921031951904296875f,-0.74710000076293869852861462277360260486602783203125f,-1.014299980454891336734135620645247399806976318359375f,-0.316200012835860189142067611101083457469940185546875f,-0.48929998384490558382964309203089214861392974853515625f,-0.578800024640560906163955223746597766876220703125f,-0.091599953474104101047714721062220633029937744140625f,-1.203099983432889263923470934969373047351837158203125f,0.4910999959647650570104815415106713771820068359375f,-0.06690001507699439020626641649869270622730255126953125f,2.298799990665912673648563213646411895751953125f,0.06159994017481817962789136799983680248260498046875f,-0.0999999623000622506197032635100185871124267578125f,-1.33739996733367405568060348741710186004638671875f,0.88970000213682620682220658636651933193206787109375f,-1.020300010231136145222308186930604279041290283203125f,0.69799998578429178763826712383888661861419677734375f,0.973800005790592226873059189529158174991607666015625f,0.5762000085145240202422201036824844777584075927734375f,0.185300028932094296152399692800827324390411376953125f,-0.36969998351037547390518511747359298169612884521484375f,-0.6187999966263770801333521376363933086395263671875f,0.3612000114202498224358350853435695171356201171875f,0.165399953043462222268544792314060032367706298828125f,1.784599948161841087568291186471469700336456298828125f,0.1431999897241584829288285618531517684459686279296875f,0.222500022500752681064994931148248724639415740966796875f,0.2799999876320367064863603445701301097869873046875f,0.65649998946487908568769853445701301097869873046875f,-2.001999995589255121331007103435695171356201171875f,0.44510001498982310064178591346717439591884613037109375f,0.53840000603199111850472036167047917842864990234375f,-0.6550999624446027791435653853113763034343719482421875f,0.5226999879956242267553534475155174732208251953125f,-0.01859999724999095160882234267774038016796112060546875f,-0.047100029969215029268525540828704833984375f,-0.19250003315508390500099267228506505489349365234375f,0.35240000466108245547047772561199963092803955078125f,-0.4712000413902106554786541892099194228649139404296875f,0.6107000046253201475110472529195249080657958984375f,-0.3952999994836743358206376797170378267765045166015625f,-1.7158000041365628618450500653125345706939697265625f,-0.78700001198053382012176371063105762004852294921875f,-1.4546000063508746080742639605887234210968017578125f,-1.3871000311613084221562530728988349437713623046875f,1.506500021204352446346774740959517657756805419921875f,1.36990003476142874205834232270717620849609375f,-0.874700086402893983716921866289339959621429443359375f,0.310199990379810941476534935645759105682373046875f,-0.177800066334009354562795124365948140621185302734375f,-0.7553000009179111629009639727883040904998779296875f,-0.40189997819661993361251006717793643474578857421875f,0.1712999802350996247923831106163561344146728515625f,-0.281800011169910380459668886032886803150177001953125f,0.0081000038474794422427294193767011165618896484375f,1.9855999681591978145434040925465524196624755859375f,0.528200005891918511480298548121936619281768798828125f,-1.7061999845981592471844123792834579944610595703125f,-0.848499992728232843575142396730370819568634033203125f,1.0096000007629388761642985627986490726470947265625f,1.184000025302171632546333057689480483531951904296875f,0.54929997212886849666801936109550297260284423828125f,0.5257999528765679997377446852624416351318359375f,0.3298999998927119037261945777572691440582275390625f,-0.273000024080276570970227112411521375179290771484375f,-0.8249999982118598751412719138897955417633056640625f,-1.548800014451891637889957564766518771648406982421875f,1.179499997749924755652273233863525092601776123046875f,1.35650003033131394403199010412208735942840576171875f,-0.98269998248219536662872997112572193145751953125f,0.665599979893862769841916815494187176227569580078125f,-0.38680002882778674599961732383235357701778411865234375f,-1.528700037311017556618253365741111338138580322265625f,0.739200009088218035202544342610053718090057373046875f,-0.270800006092339839991467442814609967172145843505859375f,-0.362199996027350312743919857894070446491241455078125f,-0.7262000059068203228207494248636066913604736328125f,1.555000006407500023897227947600185871124267578125f,1.14670002335607978949383323197253048419952392578125f,2.443000051423907859060591363231651484966278076171875f,0.16140001029372275098694444750435650348663330078125f,-0.54740004832148603242103490629233419895172119140625f,-1.129100041651726282765366704552434384822845458984375f,-1.2217000201150778337932933936826884746551513671875f,-0.2302000043123959560631419662968255579471588134765625f,0.3683000008955594761772545098210684955120086669921875f,-0.916500018522143466981333403964526951313018798828125f,-0.341299991783499567787885098368860781192779541015625f,0.0252000242054464251850731670856475830078125f,0.073100018078088080386578440084122121334075927734375f,-0.250299973946809306557526042524841614067554473876953125f,-1.24630007128119491000006746617145836353302001953125f,0.008099973225593526393595311674289405345916748046875f,0.64170003460645741455437018885277211666107177734375f,0.583499983116984477504729511565528810024261474609375f,0.50580003229975700751452905024052597582340240478515625f,-0.83399999997019780284546186521765775978565216064453125f,0.1676000224962830298380822569015435874462127685546875f,-0.881300013948977056799094498273916542530059814453125f,0.036700011882185634704001131467521190643310546875f,-0.5174999871104954962675037677399814128875732421875f,1.447999960899352345222723670303821563720703125f,-1.9182000141620638711259516639984212815761566162109375f,0.79120000378340460400750089320354163646697998046875f,0.2460999953128395478785250816144980490207672119140625f,0.51479999265149223308668524623499251902103424072265625f,0.11499999485909950802664525326690636575222015380859375f,0.0885000241324302994172512626391835510730743408203125f,0.783599996970593792866566218435764312744140625f,0.10460001205056912443325245476444251835346221923828125f,0.87190000414326807032239230466075241565704345703125f,-0.81299999543279444225163388182409107685089111328125f,0.425599992895126177927522803656756877899169921875f,-0.44320004381537447812888785847462713718414306640625f,1.030600041472911954087976482696831226348876953125f,1.297000008124857828306630835868418216705322265625f,-0.65029999611228672673490791567019186913967132568359375f,-0.3761999863676723787619948780047707259654998779296875f,-0.087300020162761171604870469309389591217041015625f,0.9369999913051720863421678586746565997600555419921875f,0.56609998255371996034313042400754056870937347412109375f,1.06660001255720882085142875439487397670745849609375f,0.430199969331920062387553116423077881336212158203125f,-0.7889000314161178639693616787553764879703521728515625f,-1.350800005030631911751015650224871933460235595703125f,0.18150006179139099860009309850283898413181304931640625f,-0.09369996582269646268059659632854163646697998046875f,-0.5878000073254108226450398433371447026729583740234375f,-0.72780001991689202700541727608651854097843170166015625f,0.71349999703094368896927335299551486968994140625f,0.02669999342337237013822459630318917334079742431640625f,-0.73289999528825244823337925481610000133514404296875f,0.5104999851882456862739445568877272307872772216796875f,-0.7864999862238766237254594670957885682582855224609375f,0.02279999242424946004348385031335055828094482421875f,0.8614999912530187931025693615083582699298858642578125f,0.56769998255670106601655788836069405078887939453125f,-0.9336999879881735520115171311772428452968597412109375f,-0.93420003389865191589791493242955766618251800537109375f,0.7258999726533890139990035095252096652984619140625f,-0.0974000193014743498309826463810168206691741943359375f,-0.054599953675270374020556118921376764774322509765625f,-0.059800020628422156931947029079310595989227294921875f,-0.10770003210306189345146776759065687656402587890625f,-0.80430001247376237127895137746236287057399749755859375f,-0.173699999052286724321447763941250741481781005859375f,-0.04540000435933466693683158155181445181369781494140625f,-0.2013000072807076268333048574277199804782867431640625f,0.33479998819977030422023744904436171054840087890625f,0.4825000266730785636326572785037569701671600341796875f,-0.073699992942810155227562063373625278472900390625f,1.7742999710857869644087259075604379177093505859375f,-0.51180001891851389927978743799030780792236328125f,-0.43129996026754380267220767564140260219573974609375f,0.001600002312660109282660414464771747589111328125f,-0.238500024840235358425388767500407993793487548828125f,0.248299947363138873157595298835076391696929931640625f,1.269499967649579286188554760883562266826629638671875f,0.240600037226080143426543145324103534221649169921875f,0.1124999953806398167444058344699442386627197265625f,-1.91429990954399187330636777915060520172119140625f,-0.8471000147700313842591413049376569688320159912109375f,0.860199988815188110180542935268022119998931884765625f,1.279400046518445410725917099625803530216217041015625f,-0.9287000241607430428558700441499240696430206298828125f,-0.876900014778971836904020165093243122100830078125f,-0.355000001043081159224357179482467472553253173828125f,-0.325600015731156100518006724087172187864780426025390625f,-0.03510001091063041922524234905722551047801971435546875f,1.164500023081898927301836010883562266826629638671875f,-0.6248000198274852312607663407106883823871612548828125f,0.447400014235079357405311384354718029499053955078125f,-0.413200022953748913323579472489655017852783203125f,-0.3363999930083749756448696643928997218608856201171875f,0.33320002820640814977792842910275794565677642822265625f,-0.4322000346213588084509638065355829894542694091796875f,0.4787000092968345388300122067448683083057403564453125f,-0.12179999090433124564469835604541003704071044921875f,0.350299977523088390540806358330883085727691650390625f,0.1280000100731850665169986314140260219573974609375f,1.422300037783384762946070623002015054225921630859375f,-0.977500045597553768317311551072634756565093994140625f,0.5464000325709583716360384642030112445354461669921875f,-1.1830000649839644832894691717228852212429046630859375f,-0.19499998930841666577862270059995353221893310546875f,0.1326999619185915690877664019353687763214111328125f,-1.44169995920658156052240883582271635532379150390625f,0.587400030812621043452281810459680855274200439453125f,-0.03319996871352248746234181453473865985870361328125f,0.5304999915957449019288105773739516735076904296875f,-0.979700008022785340244809049181640148162841796875f,0.7802000134766104988415236221044324338436126708984375f,-0.4027999937653543582882775808684527873992919921875f,-0.80819999166130995771339939892641268670558929443359375f,0.54060000541210140312387011363171041011810302734375f,-0.615400033435225424227610346861183643341064453125f,1.55239996100068022855111848912201821804046630859375f,-1.077000043272972273911136653623543679714202880859375f,-0.4004000369369988820977823706925846636295318603515625f,-0.70780002960264687406066741459653712809085845947265625f,0.3593000071465966005490599854965694248676300048828125f,0.3341000079751015316986695324885658919811248779296875f,-0.4643000169813633615234493845491670072078704833984375f,-2.071299991373717119813591125421226024627685546875f,0.617099985787272320436613881611265242099761962890625f,0.9819999765157694238126850905246101319789886474609375f,-1.099399987733364003616998161305673420429229736328125f,-0.32919996416121666271692447480745613574981689453125f,-0.30650004042685008442958860541693866252899169921875f,-0.51930006306320508624452259027748368680477142333984375f,1.0744000438362359961530501095694489777088165283203125f,-0.60119999800920442822871336829848587512969970703125f,1.04830006068646941486122159403748810291290283203125f,-1.22940003392696350914548020227812230587005615234375f,0.4564000153973697404552467560279183089733123779296875f,0.47040001503229067481015590601600706577301025390625f,1.700800023657083936967637782800011336803436279296875f,-1.120299993765353718799815396778285503387451171875f,0.795000000223517577779830389772541821002960205078125f,-0.640800033044815364746682462282478809356689453125f,1.687299994905291811164715909399092197418212890625f,-0.3298999826073636398859889595769345760345458984375f,2.0649999186396588157776932348497211933135986328125f,-1.562399984246491957406988149159587919712066650390625f,0.9252999802052972455612689373083412647247314453125f,-0.3660999883651729902567240060307085514068603515625f,1.0132999296307563241725802072323858737945556640625f,-0.23989998797178291312093278975225985050201416015625f,0.3021000196874137344593691523186862468719482421875f,-0.00859998687505747483328377711586654186248779296875f,-0.8315999951601025230729646864347159862518310546875f,-0.3643999556958679164608838618732988834381103515625f,1.6726999379277227575357755995355546474456787109375f,-0.092900027102231330644599438528530299663543701171875f,-0.320000027157366417895900667645037174224853515625f,-0.13780003034770516023854725062847137451171875f,0.3915000329017639391082639122032560408115386962890625f,1.1312999848544598169297614731476642191410064697265625f,-0.550999984383583107927506716805510222911834716796875f,-0.27509998181611283740721773938275873661041259765625f,-0.6651999876230954011901985722943209111690521240234375f,0.3435999801322815461190884889219887554645538330078125f,-0.00639998891055582053155603716732002794742584228515625f,-0.2482000432938338274624356927233748137950897216796875f,-0.06179997883439103389946467359550297260284423828125f,1.4247000072404742976317493230453692376613616943359375f,0.008099976131319852612477916409261524677276611328125f,1.1430000440478327305982020334340631961822509765625f,-0.330300006915628774351034735445864498615264892578125f,-0.16760000189542711979129308019764721393585205078125f,-0.1206999901711947043025929815485142171382904052734375f,0.2628000009179114471180582768283784389495849609375f,-0.7067000165760510821399975611711852252483367919921875f,0.93430001131892215227026099455542862415313720703125f,-0.6015000167712571954581335376133210957050323486328125f,1.095100035721064291038828741875477135181427001953125f,-0.36019997506886713978246916667558252811431884765625f,-0.6820000110864636866381260915659368038177490234375f,-0.358899994871020755482504682731814682483673095703125f,0.58779999376535396748977291281335055828094482421875f,-1.51919993685483945000669336877763271331787109375f,-0.69649995884299276127649136469699442386627197265625f,0.078700007098913005165741196833550930023193359375f,1.13189997000098241386467634583823382854461669921875f,-0.297799988400935689014659146778285503387451171875f,1.017899989998340526398123984108678996562957763671875f,-1.0850999649405483893360724323429167270660400390625f,0.8268000304669147482172775198705494403839111328125f,-1.3332999647229915041179992840625345706939697265625f,-0.42479998026490140006217188783921301364898681640625f,0.0557999782085418782884289612411521375179290771484375f,-0.9165000219866634711962660730932839214801788330078125f,0.835199981811642455653554861783050000667572021484375f,0.473799991410970733340946026146411895751953125f,-0.13699997800588636920338103664107620716094970703125f,0.872000036455691240888654647278599441051483154296875f,-0.42090003263056274818154633976519107818603515625f,-0.482900007469952097238774513243697583675384521484375f,0.53160004470646438079484141780994832515716552734375f,-0.7069000483065848161601252286345697939395904541015625f,0.455300036233663885099076651385985314846038818359375f,0.5162000245332720727020614503999240696430206298828125f,0.140800007303059526719835048425011336803436279296875f,1.04859999939203252239394714706577360630035400390625f,-0.3275999980956318946567762395716272294521331787109375f,-0.3272999875664712732259431504644453525543212890625f,-0.3578999688014381508338601634022779762744903564453125f,-0.502200038719177399570980924181640148162841796875f,-0.077900028517843278308419030508957803249359130859375f,0.9233000518202789663746443693526089191436767578125f,-1.686500066876412073924029755289666354656219482421875f,0.8413000514626514103611043537966907024383544921875f,0.30280001678764822958811464559403248131275177001953125f,0.0045999977827069216118616168387234210968017578125f,1.530299978715181197230776888318359851837158203125f,0.56699998351931579776419312111102044582366943359375f,0.550699993300437284915460622869431972503662109375f,-1.55489998528957329426702926866710186004638671875f,-1.326399938321113580741439363919198513031005859375f,0.5346000253498548460129313752986490726470947265625f,2.3417999423265456471199286170303821563720703125f,0.221899989968537880713483900763094425201416015625f,-0.8528000053882589526210722397081553936004638671875f,-0.43089999828338587661846759147010743618011474609375f,-1.0247999863743775250668477383442223072052001953125f,1.67170001583099381292640828178264200687408447265625f,0.49310001703500672132207682807347737252712249755859375f,-0.3306999979197977612699332894408144056797027587890625f,-0.32380000541806186475923823309130966663360595703125f,-0.1636999875783917701710379333235323429107666015625f,-0.31910001006126431644105423401924781501293182373046875f,0.38420000860393044295193476500571705400943756103515625f,0.749600025610626108374390241806395351886749267578125f,-0.69480002295672915746393982772133313119411468505859375f,0.8125000279769303279664427464012987911701202392578125f,0.93410002403110237167993545881472527980804443359375f,0.14109998538494128439424457610584795475006103515625f,-2.096200005310773661904022446833550930023193359375f,-1.346199966157972216507232587900944054126739501953125f,-0.843800012570619539786775931133888661861419677734375f,0.0849999814108006734869604770210571587085723876953125f,-0.340600014054775357408288982696831226348876953125f,0.53679999995678695512424383196048438549041748046875f,0.6562000237137082248040087506524287164211273193359375f,-0.89380003849864042564377086819149553775787353515625f,0.3567000382572416139481674690614454448223114013671875f,-0.238400004728138326104414090877980925142765045166015625f,-0.449400036460161356188791614840738475322723388671875f,-0.4720999934241170148396804506774060428142547607421875f,0.893700016784666928515434847213327884674072265625f,-0.4852999107658870769910208764486014842987060546875f,-1.8053999844104051675230948603712022304534912109375f,-0.29130003350675082174348062835633754730224609375f,1.4719000026345250287107546682818792760372161865234375f,1.0718999692559236791566945612430572509765625f,0.4183999839410181831311774658388458192348480224609375f,-0.2618999963015313792169536100118421018123626708984375f,-0.8375000116601587496489855766412802040576934814453125f,-0.5694999851956958902832184321596287190914154052734375f,0.065799946692586264163082887534983456134796142578125f,1.009699973824620844453647805494256317615509033203125f,-0.7887999861955645730660080516827292740345001220703125f,0.4119000079989427742788166142418049275875091552734375f,-0.62160001075044279961190341055043973028659820556640625f,0.50290002678558265447605890585691668093204498291015625f,-0.21889999442398544449162045566481538116931915283203125f,0.0689999872855844298413785509183071553707122802734375f,0.116999990262091235759811524985707364976406097412109375f,-0.58039998474568144093410637651686556637287139892578125f,0.238599980877339579166118710418231785297393798828125f,-0.5451999940678475820021731124143116176128387451171875f,-0.665199992018937980020609757048077881336212158203125f,0.0017999859496951042814316679141484200954437255859375f,0.33570000401064759021352301715523935854434967041015625f,-2.10700002058595448062305877101607620716094970703125f,0.06820003789216311229637312862905673682689666748046875f,-0.526200028370320982418206767761148512363433837890625f,-0.19340001687258479012854195389081723988056182861328125f,1.118500012174248325891312560997903347015380859375f,-0.433600004085898493855211199843324720859527587890625f,0.062600002938509202721206747810356318950653076171875f,-0.86910000294595890490967349251150153577327728271484375f,0.65790000966787320901829616559552960097789764404296875f,0.31840001176893739653905868181027472019195556640625f,0.23149998976290220298324129544198513031005859375f,0.01160000827312479554365154399420134723186492919921875f,-0.96869992737770171942202068748883903026580810546875f,-1.15729999842569242218814906664192676544189453125f,-1.0255999941244713369314922601915895938873291015625f,-0.34349999952688825555213725237990729510784149169921875f,1.126999972138553740563793326145969331264495849609375f,-0.291399994423985730929160808955202810466289520263671875f,-0.1721999786674981436362941167317330837249755859375f,-0.565499987386167024538963232771493494510650634765625f,0.42969996331930193633752423920668661594390869140625f,1.5383000097990038312900651362724602222442626953125f,0.193399958199262922420302857062779366970062255859375f,-0.32729999155253153286793121878872625529766082763671875f,-0.15720000742673878590949243516661226749420166015625f,0.681600004361570199762354604899883270263671875f,-0.30479998406469821503605999168939888477325439453125f,0.5825999867707485346812745774514041841030120849609375f,-0.432299987268447427624096235376782715320587158203125f,-0.3295999869421117889345396179123781621456146240234375f,0.52510001329481570397916811998584307730197906494140625f,-0.56620002326667329128895289613865315914154052734375f,0.429000041246414554052535095252096652984619140625f,-0.25930000409185882670470846278476528823375701904296875f,-0.066900001591444102633232660082285292446613311767578125f,0.021899976259469600137208544765599071979522705078125f,0.2645000210702421750141866141348145902156829833984375f,-0.046700056883692997455881368296104483306407928466796875f,0.9533999808117747942759478974039666354656219482421875f,-1.5267000017866489969264875981025397777557373046875f,-0.55020001988410960347408718007500283420085906982421875f,0.776200058805943005069138962426222860813140869140625f,0.30779997543692605432852360536344349384307861328125f,0.785400014585256212029662492568604648113250732421875f,-0.97670000994503514135658406303264200687408447265625f,-0.20039994961619367330740715260617434978485107421875f,-1.532199999678134805236595639144070446491241455078125f,1.5621999841809266484915497130714356899261474609375f,-0.21570006894245796758013966609723865985870361328125f,1.390500002697109938054609301616437733173370361328125f,-1.22200003872811802096975952736102044582366943359375f,1.48990000466108352128458136576227843761444091796875f,0.2678000179797412894089347901172004640102386474609375f,-0.4235999608725311560419868328608572483062744140625f,0.7765000070668757192748898887657560408115386962890625f,-0.9803000201404097335711185223772190511226654052734375f,-0.924700032628327672767909461981616914272308349609375f,-0.8797999988541007621023481988231651484966278076171875f,0.01480000578314066839169527156627736985683441162109375f,0.312500003874302212381053323042578995227813720703125f,-0.153200010436772782185244068386964499950408935546875f,-0.34790002650618578883268128265626728534698486328125f,-0.143300001603364535185392014682292938232421875f,0.5520999908536674549708322956576012074947357177734375f,-0.2282000079035757522660787799395620822906494140625f,0.1143999840825795644150275620631873607635498046875f,-0.4360000238716608134126317963819019496440887451171875f,-1.50430001828521486828549313941039144992828369140625f,1.270200033108890291799752958468161523342132568359375f,-0.0851999779745934160413156632785103283822536468505859375f,0.257900041034818006124851308413781225681304931640625f,0.68970005175769355520998260544729419052600860595703125f,-0.3123999844700098282856970399734564125537872314453125f,0.89420001843869645341555951745249330997467041015625f,-0.6734000070750714161960104320314712822437286376953125f,0.64809993448257419146329993964172899723052978515625f,0.84209997856020901707552184234373271465301513671875f,-0.0465000026524062803900960716418921947479248046875f,0.80879997554123395087088965738075785338878631591796875f,0.06140002780258668746427019868860952556133270263671875f,0.1173000282466414023740952643493073992431163787841796875f,-0.580100035795569457519604839035309851169586181640625f,0.25049999956786639376815628565964289009571075439453125f,-0.792099970364570804548520754906348884105682373046875f,-0.4375000130012629551146119410987012088298797607421875f,-0.531800006401539349099039100110530853271484375f,-0.8410000243745747727786010727868415415287017822265625f,-0.5462000100791453149895460228435695171356201171875f,-0.0525000021606685240982415052712894976139068603515625f,-0.744499996334314406709609102108515799045562744140625f,1.033900008356571476042518042959272861480712890625f,-0.65379997218847307749456376768648624420166015625f,0.26239999359697119274414944811724126338958740234375f,0.98569998257160218457784139900468289852142333984375f,-0.0841999727480111648247884659213013947010040283203125f,-0.55539998493194531192784779705107212066650390625f,-2.273500041306018459863480529747903347015380859375f,-1.239000006005168241784986093989573419094085693359375f,-0.001299965333938501999000436626374721527099609375f,-0.35289996451735561322493595071136951446533203125f,0.686400028324127386980535447946749627590179443359375f,1.5118000612378121871870462200604379177093505859375f,1.093799970549344635628585820086300373077392578125f,-0.954900021797417952740261171129532158374786376953125f,-1.01400003558397333591756250825710594654083251953125f,0.896000041455030871162534822360612452030181884765625f,0.590000022053718709003078402020037174224853515625f,-0.749700017186999190244023338891565799713134765625f,-0.425300020959973767986639359151013195514678955078125f,0.38010001709461249674859573133289813995361328125f,-0.326999990224837944197133765555918216705322265625f,-0.44880000467300451560959118069149553775787353515625f,-0.107999973565340301462356364936567842960357666015625f,-2.004499973386526079366376507095992565155029296875f,-0.560300004941225449073272102396003901958465576171875f,1.0325000262260441985517900320701301097869873046875f,0.379299979430437250726981801562942564487457275390625f,-0.612800007772445809450800879858434200286865234375f,-0.780899983754753890252686687745153903961181640625f,0.98369996463060349611851052031852304935455322265625f,-0.4267000121615824870247024591662921011447906494140625f,0.2861999995738262470013069105334579944610595703125f,0.73149998288974149573959948611445724964141845703125f,0.1012000022187826697717127899522893130779266357421875f,-0.03550001899525534465595910660340450704097747802734375f,0.816100016267597538899281062185764312744140625f,-0.8511000078111885525089519433095119893550872802734375f,0.6662999921560288640876024146564304828643798828125f,-0.10169998222887510763712271000258624553680419921875f,1.36920003080666052852620850899256765842437744140625f,-0.5267000065363942606921909828088246285915374755859375f,0.318100010105967612172861436192761175334453582763671875f,-0.07170002976357903179405184346251189708709716796875f,0.966699966731667803543359696050174534320831298828125f,0.6073000139415267728537628499907441437244415283203125f,0.20509996847957367993586785814841277897357940673828125f,0.30879999729692941823344654039829038083553314208984375f,-0.24330002779215542663138194257044233381748199462890625f,0.117599998430907515256649276125244796276092529296875f,1.00360000147819494742407187004573643207550048828125f,-0.8715000123381617669338083942420780658721923828125f,-0.691100027368963143459268394508399069309234619140625f,-1.1447000653177508144864304995280690491199493408203125f,-0.1810000039227304124977990795741789042949676513671875f,-0.4276000052854420641068600161815993487834930419921875f,-0.551100026363133821405426715500652790069580078125f,0.76659997772574473007267670254805125296115875244140625f,-0.0296000052332876517624526968575082719326019287109375f,0.42049998727440840440294778090901672840118408203125f,0.6671999832496047222463175785378552973270416259765625f,0.3139999811947344898754863606882281601428985595703125f,-0.1562000355601309564690382103435695171356201171875f,-0.250000026300549738067502403282560408115386962890625f,1.634300039184093389366125848027877509593963623046875f,-0.21440001872777980640449868587893433868885040283203125f,0.077899966491758732001216003482113592326641082763671875f,0.0657000056125227605008376485784538090229034423828125f,-0.83559998782873190492637149873189628124237060546875f,0.4600999788172537297015196600113995373249053955078125f,-1.54490001777410501659915098571218550205230712890625f,-0.9136999972641464129452515408047474920749664306640625f,0.17579997509792377741177915595471858978271484375f,-1.88240002388358096396814289619214832782745361328125f,1.23269998238906364207423393963836133480072021484375f,-0.24819997314661723653017588731017895042896270751953125f,1.663299990129470273814149550162255764007568359375f,1.37700001084431988829237525351345539093017578125f,-0.9210999835968021187682097661308944225311279296875f,2.43310002359151855699792577070184051990509033203125f,-0.667000061154366274962512761703692376613616943359375f,-0.52829997969865782891929484321735799312591552734375f,-0.729699956017732187518731734598986804485321044921875f,-0.491600019001960930609129718504846096038818359375f,-1.16889999317601223083329387009143829345703125f,0.2419999894797795203515988760045729577541351318359375f,0.7459999625720079574620058338041417300701141357421875f,-0.6913000328920781978325749150826595723628997802734375f,1.64680001519322427583347234758548438549041748046875f,-1.678600012803077135004059528000652790069580078125f,-0.63790000333488006134530223789624869823455810546875f,0.443600030684470514330541845993138849735260009765625f,0.60759997149705924357476760633289813995361328125f,1.447699992316961559168930762098170816898345947265625f,-0.543199984657764645135102909989655017852783203125f,-1.058800011825562048528581726714037358760833740234375f,0.50910006255060535895751172574819065630435943603515625f,-1.5754000106737013897628685299423523247241973876953125f,0.36709999929144909724954004559549503028392791748046875f,-0.15999998979270479981806829528068192303180694580078125f,0.21459995963573508248600774095393717288970947265625f,-0.3985000413805241947784452349878847599029541015625f,0.2109000313639644019758634385652840137481689453125f,0.03319999980106957959691271753399632871150970458984375f,-1.4836000206261867884194316502544097602367401123046875f,0.35690002042278645699724393125507049262523651123046875f,0.977999962761998187232848067651502788066864013671875f,-1.313900028808415054726310700061731040477752685546875f,1.537200026462972601848377962596714496612548828125f,0.50880003020986974693329329966218210756778717041015625f,-0.36740001524090803997069087927229702472686767578125f,0.747199992879479690799371383036486804485321044921875f,-0.74679999729320367674034741867217235267162322998046875f,-0.88669998483657785914857640818809159100055694580078125f,-0.171300062191486635043702335678972303867340087890625f,0.241700041013956923308114710380323231220245361328125f,0.529900011068582532658410855219699442386627197265625f,0.693200044858456099206023282022215425968170166015625f,-1.39950005817413369868518202565610408782958984375f,-0.4140000041425224708291352726519107818603515625f,1.8112000172689566301897912126150913536548614501953125f,0.846599990093708232308244987507350742816925048828125f,0.10779994762763400739657981830532662570476531982421875f,-0.09549996805190985593725372382323257625102996826171875f,-1.539700007110088364470357191748917102813720703125f,-0.90590000226944678018980994238518178462982177734375f,-1.61170005986392528285477965255267918109893798828125f,-0.2759999811053270857286179307266138494014739990234375f,-1.3336000188380483510997009943821467459201812744140625f,0.2197000317156316828715034716879017651081085205078125f,1.71740001442134371245629154145717620849609375f,-0.1301999865800140820937258467893116176128387451171875f,-0.8872000337272887282580313694779761135578155517578125f,0.212299980711936786548221789416857063770294189453125f,0.180299963329733026196066703050746582448482513427734375f,1.1674999962002046149933676133514381945133209228515625f,1.018299980127066017843162626377306878566741943359375f,0.25989999387636764094366981225903145968914031982421875f,0.378000037193298776827532492461614310741424560546875f,-0.9788000239886345266171474577276967465877532958984375f,-0.6904000032603736514147385605610907077789306640625f,-1.110399999907612400562584298313595354557037353515625f,-0.34590001370608802755413080376456491649150848388671875f,-0.1899999885819851908053124134312383830547332763671875f,1.3084000214919446758443655198789201676845550537109375f,0.66520001703426212547043405720614828169345855712890625f,-1.61990002075433636719026253558695316314697265625f,0.44119999929442987873784431940293870866298675537109375f,0.1058999762296668478711580974049866199493408203125f,-0.170700024089962543172305231564678251743316650390625f,-0.573599984788894179388307748013176023960113525390625f,-0.85159999620318416191366850398480892181396484375f,-0.58769998240768905883868455930496565997600555419921875f,1.3665000212788580569167606881819665431976318359375f,0.4139000052645804839102083860780112445354461669921875f,-0.6564000009804964452797548801754601299762725830078125f,0.7511000365331763628518046971294097602367401123046875f,-1.803100014427304387254480388946831226348876953125f,-0.634500011384488526999803070793859660625457763671875f,-0.502500002086161856595936114899814128875732421875f,-0.61889996067285490966014549485407769680023193359375f,-0.43529994637966051840294312569312751293182373046875f,-0.227299957019090559384721927926875650882720947265625f,-0.07370000553429145728756566313677467405796051025390625f,-0.626999993801117483371854177676141262054443359375f,-0.1585999775618318619052615758846513926982879638671875f,-0.37510000785589203697867333175963722169399261474609375f,-0.49340001206696026958553602526080794632434844970703125f,0.51960002970844509917469622450880706310272216796875f,-1.2730999997869132389638480162830092012882232666015625f,-0.1143999546527876276513779885135591030120849609375f,-0.13269993554353742837292884360067546367645263671875f,0.4344999783039096286074709496460855007171630859375f,-0.731400003886222283000506649841554462909698486328125f,-0.515399995660782206385874815168790519237518310546875f,1.38190000878125385241901312838308513164520263671875f,-0.576500016957520955429572495631873607635498046875f,0.63830001488402487996864920205553062260150909423828125f,-0.038099979707598929934420084464363753795623779296875f,0.61570000145882330944147042828262783586978912353515625f,-0.768700027662514884241318213753402233123779296875f,-0.890399994077533118996825578506104648113250732421875f,1.267899983292817722713152761571109294891357421875f,0.441700017581880377548486649175174534320831298828125f,0.130800008532404365979573412914760410785675048828125f,0.13469998509362313132697863693465478718280792236328125f,0.0948999857366086274623739882372319698333740234375f,0.12690003429949303637158664059825241565704345703125f,-0.63000001344829803517910704613314010202884674072265625f,0.381099979349971196285196128883399069309234619140625f,-1.8383999709956351242823302527540363371372222900390625f,0.227899981560557574056247176486067473888397216796875f,0.7122000050425523198782684630714356899261474609375f,1.65369993959665340099718378041870892047882080078125f,0.402599974477290789565131490235216915607452392578125f,0.0008999549210084722972169402055442333221435546875f,-1.2137000026285651443913593539036810398101806640625f,-0.20309998261332484759122962714172899723052978515625f,-0.757300015431642492558239609934389591217041015625f,-0.346499997660516978470468529849313199520111083984375f,0.18229999427199317096892627887427806854248046875f,0.944700024488569045644226207514293491840362548828125f,-0.499300001372396717869150961632840335369110107421875f,0.20319999337494376590029787621460855007171630859375f,-0.7709000319227576891734088349039666354656219482421875f,-0.292699977862835136743768771339091472327709197998046875f,0.6378000055745243201954508549533784389495849609375f,-0.256499991923570480611971333928522653877735137939453125f,0.0333000112518668789363118776236660778522491455078125f,-0.265399997150897792863588620093651115894317626953125f,-0.0739000290319327124421988628455437719821929931640625f,0.403799942162632152786727601778693497180938720703125f,0.173800013762713145837324191234074532985687255859375f,-0.49810001018047334042648799368180334568023681640625f,0.1289999816417690681191743351519107818603515625f,-0.4474000116273766369801023756735958158969879150390625f,0.4703000335514548879700669203884899616241455078125f,-0.07750000260770295579249022921430878341197967529296875f,0.5847000229239467383735018302104435861110687255859375f,0.094299997125566026312526446417905390262603759765625f,0.4348000131219624808665003001806326210498809814453125f,-0.66730003539919879873565378147759474813938140869140625f,-0.3327999901890752454391986248083412647247314453125f,1.120799927693606701240014444920234382152557373046875f,-1.425400048187375201536042368388734757900238037109375f,1.038499982669951027247634556260891258716583251953125f,-1.071000006586313446632630075328052043914794921875f,0.5785999981999392360876299790106713771820068359375f,0.89260000919699677979224361479282379150390625f,-1.08959997662305863030951513792388141155242919921875f,1.0942000165015459689499266460188664495944976806640625f,-0.56819997936785160419503881712444126605987548828125f,0.922300000046193613201239713816903531551361083984375f,-1.409599988655745761434445739723742008209228515625f,-0.9293999854981895847316764047718606889247894287109375f,1.2714000160306699882539760437794029712677001953125f,-0.383199997323751428979221600457094609737396240234375f,0.41830001724958432163248289725743234157562255859375f,-0.1834999909400938378212231327779591083526611328125f,-0.19529999659657459876171969881397671997547149658203125f,0.638099986264109464428884166409261524677276611328125f,-0.92299998325109466890125986537896096706390380859375f,-0.15990000767856871188854483989416621625423431396484375f,-0.0777000117629766950955172433168627321720123291015625f,-0.033000008098781119514342208276502788066864013671875f,0.24049999345839001563263082061894237995147705078125f,-0.789699997666478026303593651391565799713134765625f,0.275499962873756221171106517431326210498809814453125f,1.11030004383325575645358185283839702606201171875f,-0.13430001094639276715270170825533568859100341796875f,0.514600023710727327852509915828704833984375f,0.37819996677637046644804286188445985317230224609375f,-1.122000002890824799095526032033376395702362060546875f,0.8095999953612682542569700672174803912639617919921875f,-0.2020000146627420889444692875258624553680419921875f,0.7725000347569590530838468112051486968994140625f,-0.17770000438690214394910071860067546367645263671875f,0.2445000391379001936087433932698331773281097412109375f,0.35069999359846093511805520392954349517822265625f,0.24289995334148528627338237129151821136474609375f,0.8430000026226043274846233543939888477325439453125f,-1.3638999514341350050017354078590869903564453125f,0.4957000272750848868241746458807028830051422119140625f,-1.3953999598979951457522474811412394046783447265625f,-0.327599977904558414820712641812860965728759765625f,-0.610300017607212197390254004858434200286865234375f,1.188799972635508073182109001209028065204620361328125f,1.3878999482750895566596227581612765789031982421875f,-1.5751999902307982637239547329954802989959716796875f,-0.385899970865249741791558335535228252410888671875f,1.5045000197291376053954081726260483264923095703125f,0.6488000199094419162548774693277664482593536376953125f,0.3748000123396517668794558630906976759433746337890625f,-0.6271000006139277882510896233725361526012420654296875f,0.6506000139802694537394245344330556690692901611328125f,-0.2451999989107249700026613936643116176128387451171875f,0.58250000197440365834467002059682272374629974365234375f,0.3582000075682998385673272423446178436279296875f,-0.0314999804869289246056496267556212842464447021484375f,0.491300001618265813618791071348823606967926025390625f,-0.9411000407055023497804313592496328055858612060546875f,1.0052000242426990528343822006718255579471588134765625f,0.08989999219998712209900304515031166374683380126953125f,0.298600004905461791082643685513176023960113525390625f,-0.63970001323819136729298406862653791904449462890625f,-0.16550000314414514690497526316903531551361083984375f,-0.4693000239849089982868690640316344797611236572265625f,-0.49640001021921642543333064168109558522701263427734375f,0.00719996864646665901688038502470590174198150634765625f,-1.1586999922722573952427183030522428452968597412109375f,-0.1404000308275232811183741432614624500274658203125f,-1.4275000286102297319956733190338127315044403076171875f,0.5833000192612411982651110520237125456333160400390625f,0.344100031295419217514108822797425091266632080078125f,0.37500000100582842055274568338063545525074005126953125f,0.5196999776616701982590029729180969297885894775390625f,-0.43699998769164072331960824158159084618091583251953125f,0.47839997265338940479750817758031189441680908203125f,-0.15880001696646228737108685891143977642059326171875f,0.932299976390600360076632568961940705776214599609375f,-0.646100061567127514905450880178250372409820556640625f,1.128300000038743089447734746499918401241302490234375f,-0.82080002425313036695797563879750669002532958984375f,0.017800017905234000181735609658062458038330078125f,-0.7813000321283938287564296842901967465877532958984375f,0.8757000240713355143640228561707772314548492431640625f,-0.06720000573173157132345068021095357835292816162109375f,-1.0346999894708390765885042128502391278743743896484375f,-0.23009996792077969107737089871079660952091217041015625f,-0.4442000137448312013788154217763803899288177490234375f,0.0461999813571569717396414489485323429107666015625f,0.44170003751218356313756885356269776821136474609375f,0.7987999968498942227057568743475712835788726806640625f,0.91960001989230499841454502529813908040523529052734375f,0.044899987841397592802650251542218029499053955078125f,0.569699989042430576802189534646458923816680908203125f,-1.8012000025168057870672555509372614324092864990234375f,0.563299994823336813709602211019955575466156005859375f,-0.189600002588331761899098637513816356658935546875f,-0.1233000173240903496463261035387404263019561767578125f,0.35629999513626042784153469256125390529632568359375f,-0.488000040508806964822952068061567842960357666015625f,0.83060004802942355439654420479200780391693115234375f,-1.0309000567704444950578590578516013920307159423828125f,0.166299987424910256095245131291449069976806640625f,-0.1716999397046852227077806674060411751270294189453125f,-0.2913000188104810650457920928602106869220733642578125f,-0.951999986462295222366947200498543679714202880859375f,-0.15629997487068170158863722463138401508331298828125f,1.0194999996498221861429556156508624553680419921875f,-0.159499993503093406843618140555918216705322265625f,-0.4349999955296510734115145169198513031005859375f,1.675400009667872947005662354058586061000823974609375f,0.050700000452994942179429926909506320953369140625f,1.36819999553561189031825051642954349517822265625f,-2.0748000033617017834330908954143524169921875f,-0.58970005213022158585545184905640780925750732421875f,0.6862000204354525845218404356273822486400604248046875f,-1.2958999928258359002342103849514387547969818115234375f,0.9818999941036097300894880390842445194721221923828125f,-1.62219996257424359242804712266661226749420166015625f,-0.1163000291109081985041484585963189601898193359375f,1.1880000164806840956543965148739516735076904296875f,0.05649997642636339634236719575710594654083251953125f,1.1554000370860102453463014171575196087360382080078125f,-0.88640003443360360080305326846428215503692626953125f,-0.0411999853804705384874296214547939598560333251953125f,0.89999999105930328369140625f,-0.017700053709746033092642392148263752460479736328125f,-0.1280999746784565740398420530254952609539031982421875f,0.254500041726977155320810197736136615276336669921875f,-0.65520003990754516198791179704130627214908599853515625f,-0.39480002183914220115212856399011798202991485595703125f,0.5313000061444939081667371283401735126972198486328125f,0.7727000137001278634585332838469184935092926025390625f,1.4322000349193813040216127774328924715518951416015625f,-0.75410003926754054504044688655994832515716552734375f,-0.6796000165209175047920098222675733268260955810546875f,0.555199997383355903224355643033050000667572021484375f,-0.16539999566078211756803284515626728534698486328125f,-0.3568999848276366293475803104229271411895751953125f,-0.515199975031614076925734480028040707111358642578125f,-0.506200037795305757271080437931232154369354248046875f,-0.0264999958723783901604065249557606875896453857421875f,1.19120002918988543427758486359380185604095458984375f,-0.98830001613199713883517460999428294599056243896484375f,1.07800002914667114595204111537896096706390380859375f,0.267299968492986206314299124642275273799896240234375f,0.97520001690387747572685839259065687656402587890625f,-0.2117000212699178263164867530576884746551513671875f,-0.440799998995661912459809173014946281909942626953125f,1.360899998953938538903685184777714312076568603515625f,-1.99970000832080785357902641408145427703857421875f,-0.364300006848572888173976025427691638469696044921875f,0.70690002392455963953210584804764948785305023193359375f,1.037799963553249771308628623955883085727691650390625f,1.531599982102960666452418081462383270263671875f,-0.76280001319274270965564710422768257558345794677734375f,0.180700021165609570061860722489655017852783203125f,0.9182000247418884431027663595159538090229034423828125f,1.0481999927416441042993255905457772314548492431640625f,-0.13249998085200775932435135473497211933135986328125f,0.211000011429190781342413174570538103580474853515625f,0.2516999992534516561448754146113060414791107177734375f,-1.814700019682943921139894882799126207828521728515625f,0.9799999977275728468839588458649814128875732421875f,0.04990001199990512503745776484720408916473388671875f,-0.417900031796097692904368159361183643341064453125f,-0.89129997576475139542395709213451482355594635009765625f,0.4950999947428711411845370093942619860172271728515625f,0.35949998709559427556570199158159084618091583251953125f,-0.339700026686489575666882956284098327159881591796875f,0.36420000361204163397133015678264200687408447265625f,-1.0986000170499086170394775763270445168018341064453125f,-0.1341000113278629779500761287636123597621917724609375f,-0.96280001417994498513763801383902318775653839111328125f,0.1567000336378809866033634534687735140323638916015625f,1.5180000389441854480310212238691747188568115234375f,-0.2466999791741368852626692387275397777557373046875f,0.865299962212144446738193437340669333934783935546875f,0.6259000361323359129528398625552654266357421875f,1.0685999808400865607183050087769515812397003173828125f,-0.099400013065338299611539696343243122100830078125f,-0.63159996446371025058397208340466022491455078125f,0.39269997845888138243708453956060111522674560546875f,-0.9066999679982661319854742032475769519805908203125f,0.1016999888226390869050419496488757431507110595703125f,-0.2673000089496377906783664002432487905025482177734375f,-1.3477999964103102303880632462096400558948516845703125f,-0.2651999747708437826787530866567976772785186767578125f,1.66600006201863237720317556522786617279052734375f,0.039699950504303416209950228221714496612548828125f,0.021100011014938413467234568088315427303314208984375f,-0.746000016570089741207993938587605953216552734375f,0.449300005060434326509266611537896096706390380859375f,-0.135200005951523483105347622768022119998931884765625f,0.3516000248134136807465210949885658919811248779296875f,-0.34610003291964519522849741406389512121677398681640625f,-1.1412999781116841546690920949913561344146728515625f,0.4540999778002501319207340202410705387592315673828125f,0.91539999797046167628877810784615576267242431640625f,0.223000003367662458231279742904007434844970703125f,-0.9750000212714076130993134938762523233890533447265625f,0.84670000800788403072516530301072634756565093994140625f,-0.84850000498443822749550236039794981479644775390625f,0.410299995031952580148981724050827324390411376953125f,0.826000004574656276901123419520445168018341064453125f,0.827500003874301892636822230997495353221893310546875f,-0.09740000838637374869222185225225985050201416015625f,-0.2212999987870454265959097028826363384723663330078125f,-0.344399953460693186713115210295654833316802978515625f,-1.08499999292194804212385861319489777088165283203125f,2.2907000119268889903878516633994877338409423828125f,-0.40510000927150269234289226005785167217254638671875f,-1.0740999909877775309041680884547531604766845703125f,-1.361400016105175136971183746936731040477752685546875f,2.6444999896287928464744254597462713718414306640625f,0.860099971199036250624203603365458548069000244140625f,-1.2315999996304523023127330816350877285003662109375f,0.96749999321997137702311420071055181324481964111328125f,-1.394299997684359748717497495817951858043670654296875f,0.2380999845691025296190446169930510222911834716796875f,-0.6173000115945936894235046565881930291652679443359375f,0.090800007861852716217754277749918401241302490234375f,0.64750001356005615349431536742486059665679931640625f,0.1584999869726593235697009731666184961795806884765625f,-0.773399967288970202616837923415005207061767578125f,1.000300019171834531306330973166041076183319091796875f,-0.1142999827831985282244886548141948878765106201171875f,1.1891000034675001106876379708410240709781646728515625f,-0.0898000273525714209910120189306326210498809814453125f,-0.7478000372767450443234338308684527873992919921875f,-0.013099988164007481117323550279252231121063232421875f,1.15670003635734364166864907019771635532379150390625f,-0.0412999711081382392396932345945970155298709869384765625f,-0.594700001391768484637623259914107620716094970703125f,-0.59629998448193166638020557002164423465728759765625f,-0.418399999177456294319199514575302600860595703125f,1.673500063583254426902158229495398700237274169921875f,2.429099989795684422233534860424697399139404296875f,0.33809998194277302019372655195184051990509033203125f,-0.8602000112414358579115969405393116176128387451171875f,0.17669998915791496063576460073818452656269073486328125f,0.487600005620718146559511296800337731838226318359375f,-1.086500009953975887100341424229554831981658935546875f,0.420200007590651569699957690318115055561065673828125f,-0.196800003048777316649875501752831041812896728515625f,-0.48280000933706757049179714158526621758937835693359375f,0.840400021290779619675959111191332340240478515625f,-0.245699995312094632282651218702085316181182861328125f,-2.4764999747872362689804504043422639369964599609375f,0.06890003108084119531895339605398476123809814453125f,0.4261999790847303959395730998949147760868072509765625f,0.82659998294115089034761467701173387467861175537109375f,-0.340999976038932839372819216805510222911834716796875f,0.3295999993845819542315211947425268590450286865234375f,-0.7767000374376766824724427351611666381359100341796875f,-1.16229999995306076954193486017175018787384033203125f,1.4382000093936913032166557968594133853912353515625f,0.5141999779261643421790495267487131059169769287109375f,-1.046200012314319138795326580293476581573486328125f,0.707100001675635692066634874208830296993255615234375f,-2.13860000833273034004378132522106170654296875f,0.42489996720328881973927082071895711123943328857421875f,0.19719995244145405166591444867663085460662841796875f,-1.546700045465678297063050194992683827877044677734375f,1.28490002947151626955246683792211115360260009765625f,-0.1623999715805053423167692017159424722194671630859375f,-0.746899997121095449159611234790645539760589599609375f,1.483399980625510039544678875245153903961181640625f,0.156999941572546930501630413345992565155029296875f,-0.354199972021579156944426358677446842193603515625f,-0.17720000690519821517909804242663085460662841796875f,1.3501000358700760983765576384030282497406005859375f,0.19210000568032281620389767340384423732757568359375f,0.634300053936243557473062537610530853271484375f,-1.571300043565035498005499903229065239429473876953125f,0.8243999975681308711727979243732988834381103515625f,0.6503999590411779774257183817098848521709442138671875f,0.6371999919667838963022177267703227698802947998046875f,-0.586599952617288256107030974817462265491485595703125f,-0.5360999632567171602204325608909130096435546875f,0.8993999778985983795109859784133732318878173828125f,0.76389998488724319969378484529443085193634033203125f,-0.7102000371694572322667227126657962799072265625f,1.31450003075599664015271628159098327159881591796875f,-1.614499989926814205176697214483283460140228271484375f,-0.14460000523328708776915618727798573672771453857421875f,0.90440005717277482943927680025808513164520263671875f,-0.5320999579221010744589648311375640332698822021484375f,0.29699994939565643736756328507908619940280914306640625f,0.414800022677332569376318360809818841516971588134765625f,-0.753400037324428950569199514575302600860595703125f,-0.81900001930445409659142796954256482422351837158203125f,0.34190001217126830823644922929815948009490966796875f,0.6192000214934350044160282777738757431507110595703125f,-2.102900019353628469076511464663781225681304931640625f,-1.4889000383451584941241208071005530655384063720703125f,-0.19040003798007898438982010702602565288543701171875f,-0.69829995594993177210341173122287727892398834228515625f,-0.855600029551982999009851482696831226348876953125f,-0.74540000761896363368208540123305283486843109130859375f,-0.5999000435531147701340159983374178409576416015625f,-0.3910000272989275771351458388380706310272216796875f,-1.3189000213950861084555299385101534426212310791015625f,1.0803999850809571370291450875811278820037841796875f,0.30549998652935095311278246299480088055133819580078125f,-0.743800001692771939332260444643907248973846435546875f,-2.82080004809498774420717381872236728668212890625f,0.681900071924925299526876187883317470550537109375f,0.507400091162324873295119687099941074848175048828125f,0.67700003939867059443713515065610408782958984375f,-2.0967000495821235261928450199775397777557373046875f,-0.6926000337839131049122443073429167270660400390625f,-1.142600053751468891505282954312860965728759765625f,-0.5516000239938498861391735772485844790935516357421875f,0.270100033485890289597364244400523602962493896484375f,0.04509999980926548968085398882976733148097991943359375f,0.2174999690055852141767900320701301097869873046875f,0.5424999746680263523757048460538499057292938232421875f,0.75040001518130328150846253265626728534698486328125f,0.590800038036704666666310004075057804584503173828125f,-1.2875000399351126390001809340901672840118408203125f,-1.1769999838918450141278526643873192369937896728515625f,1.4985000249147422124451622948981821537017822265625f,0.41989999795556087747172568924725055694580078125f,-0.08209997751712716507199729676358401775360107421875f,-1.54540004243180195686591105186380445957183837890625f,1.7618000168323515453039362910203635692596435546875f,0.02359999309629190467063608593889512121677398681640625f,-0.1307999642014501251452429642085917294025421142578125f,0.54790001248046771298305657182936556637287139892578125f,-0.282100015440583273829133759136311709880828857421875f,0.2207000050537286650609303251258097589015960693359375f,0.357899980051815447268381831236183643341064453125f,-0.5495000170096755454096637549810111522674560546875f,0.31550002661347409382841533442842774093151092529296875f,0.4407000173844399792955073280609212815761566162109375f,-1.342100018849224252193153006373904645442962646484375f,1.6762000181630252182429785534623079001903533935546875f,0.91559998395442931951038190163671970367431640625f,-0.433700003820657542519256821833550930023193359375f,0.55440000099539743505516753430129028856754302978515625f,-0.477700019586086543910141699598170816898345947265625f,0.19870001820027793915102165556163527071475982666015625f,-1.1883000013053415244712596177123486995697021484375f,-0.340499989397823998871217554551549255847930908203125f,0.18479998682141307586590528444503434002399444580078125f,-0.3364000103123483409461869086953811347484588623046875f,-0.9614000220097602689151017330004833638668060302734375f,0.0163000064425171675708270413451828062534332275390625f,-0.16909997183978553447758486072416417300701141357421875f,0.398899965739250017549011317896656692028045654296875f,0.12509997291266905516948781951214186847209930419921875f,-0.1876000030875208945957410833216272294521331787109375f,0.30129998828172654956603082609944976866245269775390625f,-0.04749998636543750762939453125f,-0.8597999959856270635327746276743710041046142578125f,0.2634000373989342502767385667539201676845550537109375f,-0.84539998679459105179745392888435162603855133056640625f,0.9650999679580329893013868058915250003337860107421875f,0.8475999804750082855520076918764971196651458740234375f,-1.275399972005188065082847970188595354557037353515625f,-0.46960001771301029815930405675317160785198211669921875f,-0.088300004155188738241832879793946631252765655517578125f,-0.06759999172538515288533034208739991299808025360107421875f,-0.06030001887381075675875763408839702606201171875f,-0.8683000145114958545633498943061567842960357666015625f,0.7750000213831664819252864617737941443920135498046875f,-0.143899992300569989911451784792006947100162506103515625f,0.31769997957646811670429087826050817966461181640625f,-0.12390001193285027358825800547492690384387969970703125f,0.4558000299900777374517701900913380086421966552734375f,1.844100021591037918966549113974906504154205322265625f,0.7299000329732887593081613886170089244842529296875f,1.63320006229281489140703342854976654052734375f,-0.151899993973225233556689772740355692803859710693359375f,-0.3573999983839690752773776694084517657756805419921875f,-0.33430001549124710447102870602975599467754364013671875f,0.7920000012516974496890043155872263014316558837890625f,-0.66380004028677941363412173814140260219573974609375f,-1.24620000464022151476228827959857881069183349609375f,0.82470003074705566792346189686213620007038116455078125f,1.6790000555515292024466589282383210957050323486328125f,2.0590000074952836683905843528918921947479248046875f,0.4645999334841963701592248980887234210968017578125f,0.5259999853521595269967292551882565021514892578125f,0.1742000192955137638506357689038850367069244384765625f,0.14729999302402141392320800150628201663494110107421875f,-0.0888000012457370591079097721376456320285797119140625f,-0.120399978822469666539518584613688290119171142578125f,-0.2788999908477067801726434481679461896419525146484375f,1.47469996935427172246590998838655650615692138671875f,0.75249998629093173274640093950438313186168670654296875f,-1.275600014501809820188782396144233644008636474609375f,-0.354799984586238625894338838406838476657867431640625f,-1.174899978062511340937135173589922487735748291015625f,0.441800004538893009708999670692719519138336181640625f,0.31079997582435670011591355432756245136260986328125f,0.1470999856010077877499497844837605953216552734375f,-0.8121999874591825374636755441315472126007080078125f,0.205799986180662752843772977939806878566741943359375f,-1.67569997683465476967512586270458996295928955078125f,-0.254100030066072957168898938107304275035858154296875f,-0.8551000286802652450290906926966272294521331787109375f,-0.40320000689774759816685900659649632871150970458984375f,0.707200021731853922801747103221714496612548828125f,0.185699995367973880178169565624557435512542724609375f,-0.6528000050902367235039491788484156131744384765625f,0.383899998838454603600212067249231040477752685546875f,-1.726600016394257863794337026774883270263671875f,0.67080003535449517215738524100743234157562255859375f,-0.58870002110600605504941995604895055294036865234375f,1.623300020527839393480462604202330112457275390625f,-1.30050002784282003887028622557409107685089111328125f,1.467400047762692505415316190919838845729827880859375f,0.5359000451847915069691907774540595710277557373046875f,1.654700068446994354331991416984237730503082275390625f,-0.7453000547856103441546338217449374496936798095703125f,-0.589000034093857038897112943232059478759765625f,0.1495000214427715246046091124298982322216033935546875f,1.1345000551939019572955658077262341976165771484375f,-0.739600006313621438636118909926153719425201416015625f,0.419099996203184499421467990032397210597991943359375f,-0.4851999784588816311270420555956661701202392578125f,0.9986000425308940720015016268007457256317138671875f,-0.29360000773668348816869411166408099234104156494140625f,1.3288000256091354600584963918663561344146728515625f,-1.71700002606213075750929419882595539093017578125f,-0.473599990078806598603478050790727138519287109375f,1.34520003359317730229349763249047100543975830078125f,0.6643000225692985427627945682615973055362701416015625f,-0.8585000508427622634144427138380706310272216796875f,0.673200005519390298758253265987150371074676513671875f,-0.66269999276399571552786937900236807763576507568359375f,-0.08639999345540994024617020841105841100215911865234375f,-0.83840000327527519630166352726519107818603515625f,0.337800009318440952466033877499285154044628143310546875f,-0.2586999958112830011458527224021963775157928466796875f,0.18070002541244056804004003424779511988162994384765625f,-1.4880000253468754767283144246903248131275177001953125f,-0.7831000159919263570174052802030928432941436767578125f,0.1581999848067765501724579735309816896915435791015625f,-0.292499963734298429329783175489865243434906005859375f,1.656899982890486011655184483970515429973602294921875f,0.031899992799758525308106982265599071979522705078125f,-0.29449995326995814792780947755090892314910888671875f,0.11060004713535231957166615757159888744354248046875f,-0.12379997949004195589850496617145836353302001953125f,-1.47429996780753125307228401652537286281585693359375f,0.3028000153720373788246433832682669162750244140625f,-0.24650002396106618363091911305673420429229736328125f,0.767199977922439213529059998109005391597747802734375f,0.44159996610283958062836973113007843494415283203125f,-1.19390001759529074121246594586409628391265869140625f,-0.76329999489784228217104100622236728668212890625f,0.7230999735236167680341168306767940521240234375f,0.859300034117698174895849660970270633697509765625f,-0.09820001028776204066161881200969219207763671875f,-0.0957000164717438650541225797496736049652099609375f,0.844300013665854720557035761885344982147216796875f,-0.9106999965041870925830380656407214701175689697265625f,-0.219599990481138096498625600361265242099761962890625f,0.114399996599555020537763994070701301097869873046875f,-1.2355999979615208683725313676404766738414764404296875f,0.422900002366304317291678671608678996562957763671875f,1.5021999797850842384150382713414728641510009765625f,-1.5284999897480009423134106327779591083526611328125f,-0.29409998100399958875783568146289326250553131103515625f,-0.463400031661987343856168308775522746145725250244140625f,0.57290000244081029645570879438309930264949798583984375f,1.87790003522336501617928661289624869823455810546875f,-0.257099995622038335341130732558667659759521484375f,-0.19060003387331991842046363672125153243541717529296875f,0.0620000017732380559465354963322170078754425048828125f,0.100299989034235370599645875699934549629688262939453125f,0.65310002150535506615369740757159888744354248046875f,1.51030001950711056935006126877851784229278564453125f,-1.4770000241994853951155164395458996295928955078125f,0.5198999994829296422693687418359331786632537841796875f,-0.701200032877922208029985995381139218807220458984375f,0.7567000068902969278639147887588478624820709228515625f,-0.3260000042766331151966596735292114317417144775390625f,0.941399992859363266006766934879124164581298828125f,1.1129000059425830926329581416212022304534912109375f,-0.7138000242680302420694715692661702632904052734375f,0.530100003422796728358434847905300557613372802734375f,-0.522600008936226601008456782437860965728759765625f,-0.25399999102950143736734389676712453365325927734375f,-0.3499999357759957518965165945701301097869873046875f,-0.4783999979853632567028398625552654266357421875f,-0.261200004267692520443233661353588104248046875f,0.46290000095069416818915897238184697926044464111328125f,1.395299942952394456341380646335892379283905029296875f,-0.29439998818039914141309054684825241565704345703125f,-0.292599992991983637313779809119296260178089141845703125f,1.3076999790176746518710615418967790901660919189453125f,-0.170599986636638778492169876699335873126983642578125f,-1.21260000968128434095660850289277732372283935546875f,0.2532000024646521296745049767196178436279296875f,0.1787999982282519884080329575226642191410064697265625f,-1.123500009641051011755052968510426580905914306640625f,0.6276000270411372650158909891615621745586395263671875f,-0.8588999880164858158337892746203579008579254150390625f,0.879600014695525178609614158631302416324615478515625f,0.406999997675419056264445316628552973270416259765625f,-0.4956000099197030994702117823180742561817169189453125f,-0.28550003309547899998932507514837197959423065185546875f,-0.0160999944001429806661462862393818795680999755859375f,-0.430099994184077072389982276945374906063079833984375f,0.5447000390172007655564812012016773223876953125f,0.53040002475529934855558167328126728534698486328125f,0.222700029122830045480441185645759105682373046875f,0.2015999567523587554518371689482592046260833740234375f,-0.1982999734401698077590481261722743511199951171875f,-0.22530003757476801951042943983338773250579833984375f,0.12969999997615833109421146218664944171905517578125f,0.2888000004634256878688347569550387561321258544921875f,0.3306000217288735942844368764781393110752105712890625f,-1.1777999841913577849794592111720703542232513427734375f,0.27510002067089089905493892729282379150390625f,-0.21229999762475504354597433120943605899810791015625f,-0.80619997282624211720758466981351375579833984375f,-0.004399985647200566063474980182945728302001953125f,1.27430006734728795692035419051535427570343017578125f,-1.140400019502639139545863145031034946441650390625f,1.6482000205695630512536808964796364307403564453125f,-1.648800015699862342444248497486114501953125f,-0.2149999864399436688700006925500929355621337890625f,1.311299993199110502928306232206523418426513671875f,-0.8654000097051264273062542997649870812892913818359375f,0.142400024330616048473530099727213382720947265625f,1.036000000625848738167178453295491635799407958984375f,-0.3046000028491022693089007589151151478290557861328125f,-1.202000032320618583270288581843487918376922607421875f,0.15050001007318469792295445586205460131168365478515625f,0.7511000322118401850701729927095584571361541748046875f,-0.178300010171532630209867420489899814128875732421875f,0.1027999902635812201623366490821354091167449951171875f,0.34920000957250607598325586877763271331787109375f,-0.1732000015705821294176303126732818782329559326171875f,0.269399989074468049210508979740552604198455810546875f,0.7905000152885917064082832439453341066837310791015625f,0.30020000312030303835086897379369474947452545166015625f,-0.113200019228457904318929649889469146728515625f,0.3705000433400276182993593465653248131275177001953125f,0.71900004124641458957967188325710594654083251953125f,-0.1503999843731520513756549917161464691162109375f,1.197000016599893523761011238093487918376922607421875f,-0.7145000163018699890216112180496565997600555419921875f,0.566100022637844091377701261080801486968994140625f,-0.896100021743774899363188524148426949977874755859375f,0.7740000013113021992694484652020037174224853515625f,0.14070002254396685170689806909649632871150970458984375f,0.9618000344157220382612649700604379177093505859375f,-1.5273999981045722762473815237171947956085205078125f,0.72850001150369703140086130588315427303314208984375f,0.87199998068809492224318091757595539093017578125f,0.1651000261843209582224289988516829907894134521484375f,-1.2342999833047390012552568805404007434844970703125f,2.82099999809265167272087637684307992458343505859375f,-0.7141999770879738917983559076674282550811767578125f,-0.50229998786449403525011803139932453632354736328125f,-2.3780999690532684809340935316868126392364501953125f,1.73310002239942573254438684671185910701751708984375f,0.5255999970674505306078572175465524196624755859375f,0.5645000027418134447287911825696937739849090576171875f,-0.21119998482167734454861829362926073372364044189453125f,-0.5222000056013464952542335595353506505489349365234375f,0.5599000172525647744947718820185400545597076416015625f,-0.08700001350790287357739316576044075191020965576171875f,-0.17969999535679814783861729665659368038177490234375f,-0.6232999887883661216392283677123486995697021484375f,0.4148999917715787688621276174671947956085205078125f,0.306600006671249813994251098847598768770694732666015625f,0.32609999838620440382186416172771714627742767333984375f,-0.9590000201426447379304818241507746279239654541015625f,0.1488999975532293296165420315446681343019008636474609375f,1.24310001727715135899643428274430334568023681640625f,-1.19290001303926107567576764267869293689727783203125f,0.0866000049948691952295121154747903347015380859375f,-0.6367000190719964702878996831714175641536712646484375f,-0.286300004225969129123541279113851487636566162109375f,-0.272800006841122932854659666190855205059051513671875f,0.036700015085935344671952407225035130977630615234375f,0.34739998700320706692679095795028842985630035400390625f,-0.286200008365511848751339130103588104248046875f,0.7149000128194689107630210855859331786632537841796875f,0.52630001137852688852802884866832755506038665771484375f,0.07559999695569263877814591978676617145538330078125f,-0.23900000566989187955613260783138684928417205810546875f,0.235400031851976887509181324276141822338104248046875f,-1.6620000258386138369814943871460855007171630859375f,0.3285000067353252006796537898480892181396484375f,0.16289997785389420670298932236619293689727783203125f,0.120999984681606331804459841805510222911834716796875f,0.163700017753243276530383809586055576801300048828125f,-0.460000007823109680060724713257513940334320068359375f,-0.6704000038564206231939124336349777877330780029296875f,0.889799988907575833962937394971959292888641357421875f,0.47309999546557668370638793930993415415287017822265625f,0.05019998952299342676042215316556394100189208984375f,0.6391999907039107586825821272213943302631378173828125f,-0.7031999942131335057382557351957075297832489013671875f,-0.245399999739974672419151602298370562493801116943359375f,0.1722999971218406312090110077406279742717742919921875f,1.01420000800788390193929444649256765842437744140625f,0.025700004252791341929906820951146073639392852783203125f,-1.1025000176578760857637462322600185871124267578125f,0.6347000003486871211322295494028367102146148681640625f,0.6010000264421109239521001654793508350849151611328125f,-0.337499993890523819484172918237163685262203216552734375f,0.08769996284991476986903080614865757524967193603515625f,-0.9696999978899956129652082381653599441051483154296875f,-0.19630000057518515177434892393648624420166015625f,-0.7669000414706770296646709539345465600490570068359375f,0.235300003842264293663077978635556064546108245849609375f,-0.10200000143796206231172618572600185871124267578125f,-0.228900020128488534965072176419198513031005859375f,-0.558299991582333898776369096594862639904022216796875f,0.9562000228196383400103286476223729550838470458984375f,-0.49810000187307590380214605829678475856781005859375f,-0.20340000737309527067964154412038624286651611328125f,-0.08790000922083862633371609263122081756591796875f,-1.4483000064462416833777069768984802067279815673828125f,1.0799000065237278622731764698983170092105865478515625f,-0.80050001871585774182449313229881227016448974609375f,-1.0834000074475993447009614101261831820011138916015625f,0.2047000019319356045599533899803645908832550048828125f,-0.2866999953046442950466143884113989770412445068359375f,-0.24980001483559599595452027642750181257724761962890625f,0.60280000166296954233757787733338773250579833984375f,0.21399998857080948511111273546703159809112548828125f,0.02400000771880161920535101671703159809112548828125f,-1.05199998843669906278819325962103903293609619140625f,-0.92459994607567796975899909739382565021514892578125f,-1.034599962392449779002845389186404645442962646484375f,0.78899995645880682815231921267695724964141845703125f,-0.5329000084757797850443239440210163593292236328125f,-0.42930000528395151349059233325533568859100341796875f,0.2388999954670669723100218106992542743682861328125f,-0.88279999227523830995778553187847137451171875f,0.6588999958023424863284844832378439605236053466796875f,-0.9385999847143882135469539207406342029571533203125f,1.162400031706690928245961913489736616611480712890625f,-0.5171999943882223949032095333677716553211212158203125f,-0.88990002854019412126262977835722267627716064453125f,0.238499996826052385046068593510426580905914306640625f,0.1361000032663344239125535750645212829113006591796875f,-0.709999998211860106067661035922355949878692626953125f,1.3302000228643418022755895435693673789501190185546875f,1.0531000099197032060516221463331021368503570556640625f,0.433800022554397202156906132586300373077392578125f,0.357799988102912802645505507825873792171478271484375f,0.68440003832280726925318958819843828678131103515625f,-0.5353000500172380160535112736397422850131988525390625f,-0.2198000088006260011042058977182023227214813232421875f,-0.81570002969652433133518343311152420938014984130859375f,0.468700026023388005569358938373625278472900390625f,0.81320002194792040395299181909649632871150970458984375f,0.2589000262379652639310734230093657970428466796875f,-0.129900036102533977810935539309866726398468017578125f,-0.1676000225335358972955646095215342938899993896484375f,-0.52779998974204023198808499728329479694366455078125f,1.007000042378901838446836336515843868255615234375f,-1.116399929380417432867034221999347209930419921875f,2.12770000617504084772235728451050817966461181640625f,-1.18070000142156938949256073101423680782318115234375f,-0.003500000961124793974477142910473048686981201171875f,-1.35699998407810884515356519841589033603668212890625f,0.06470002426505061521311290562152862548828125f,0.6860000183582304345009106327779591083526611328125f,0.62350002640485779892287609982304275035858154296875f,-1.37530001581907157515161088667809963226318359375f,1.5116999894186857744671215186826884746551513671875f,1.1421000209912659517641486672800965607166290283203125f,-0.1793999783083795040994345981744118034839630126953125f,-0.033900017967820217368313251427025534212589263916015625f,-1.2459000234290957820348921814002096652984619140625f,0.475999988034367493838772134040482342243194580078125f,1.391700030992925274375693334150128066539764404296875f,-0.4246000041902064925380955173750407993793487548828125f,-0.228399998804926784856661470257677137851715087890625f,0.645799987372755790460132629959844052791595458984375f,-0.33419998926967331431825414256309159100055694580078125f,0.6475000080466266627610139039461500942707061767578125f,-0.8233000307723881139310151411336846649646759033203125f,0.6058999808490279637140929480665363371372222900390625f,0.337000000860542048286561112035997211933135986328125f,0.83229998758509726730636657521245069801807403564453125f,-1.6577000312089911204793679644353687763214111328125f,0.4494999519288540579253776741097681224346160888671875f,0.191900036311150046230977750383317470550537109375f,-0.299200027900934362179441450280137360095977783203125f,-1.288199959921836779130899230949580669403076171875f,-1.096900002411008046010465477593243122100830078125f,0.7449000165820134355953996418975293636322021484375f,0.459899986481667522042471318854950368404388427734375f,1.070300023567675840041601986740715801715850830078125f,-0.54440002435296774851991585819632746279239654541015625f,1.872000010490418642206122967763803899288177490234375f,0.245200026310235130733161668104003183543682098388671875f,-0.61609997569918650839326801360584795475006103515625f,1.27130001080110677236234550946392118930816650390625f,-0.7865000150203709239349336712621152400970458984375f,-1.368300035261362790350858631427399814128875732421875f,1.5614000326454646749851917775231413543224334716796875f,1.5659000722117728532367664229241199791431427001953125f,-0.229299989861250441691709056613035500049591064453125f,0.8522999897085128395701758563518524169921875f,1.3137000664427882501428257455700077116489410400390625f,1.63250004410743709826192571199499070644378662109375f,0.59329996010362950897842893027700483798980712890625f,-0.45470001070499410644742965814657509326934814453125f,0.9668000353097918608824556940817274153232574462890625f,-0.2568999808788294814121400122530758380889892578125f,-1.50450000616908052819553631707094609737396240234375f,1.5706000206112857853213427006267011165618896484375f,2.7646000733315947428536674124188721179962158203125f,-0.715700046497584008164949409547261893749237060546875f,0.72540000292509798640594453900121152400970458984375f,0.255599964173137461642681955709122121334075927734375f,1.280800055582821617150557358399964869022369384765625f,0.55190000732839106234450810006819665431976318359375f,0.848199974226950370592703620786778628826141357421875f,0.7341000298798079359841040059109218418598175048828125f,-0.8896000474035741234501983853988349437713623046875f,0.4812999787449836475161646376363933086395263671875f,0.177000035524368826145291677676141262054443359375f,-0.1186999837785958078484327415935695171356201171875f,-1.0686000479698185561261425391421653330326080322265625f,-0.2563000307500369334690049072378315031528472900390625f,-0.48289995490610515549434467175160534679889678955078125f,-0.1694000116497276497984358911708113737404346466064453125f,0.4925999741047615287925509619526565074920654296875f,1.9488000529900195711974220103002153337001800537109375f,-0.34540000087618860646898610866628587245941162109375f,0.19029999873861680459441458879155106842517852783203125f,-1.32110000609755484646257173153571784496307373046875f,-0.6252000153765084888846104149706661701202392578125f,0.006399997329712192595252417959272861480712890625f,-1.5917000188857315379209467209875583648681640625f,0.7571000246047976389718314749188721179962158203125f,0.8816000451721250907866078705410473048686981201171875f,-0.59640002646148193576891571865417063236236572265625f,0.016799991239607070525607923627831041812896728515625f,-1.1506999811932445698658966648508794605731964111328125f,-0.279600045764447191487533928011544048786163330078125f,-0.434499964147805339820251901983283460140228271484375f,0.78300003615021740444035458494909107685089111328125f,-1.44680004909336634710825819638557732105255126953125f,1.2294000437617302878834379953332245349884033203125f,-0.723800030899047985855077058658935129642486572265625f,1.30820004843473469691161881200969219207763671875f,1.04960004487037661391468645888380706310272216796875f,-1.4602999711155888462599250487983226776123046875f,-0.2148999676316984963619916015886701643466949462890625f,1.1484000069260595910947131415014155209064483642578125f,-0.0545000591240840659423838587827049195766448974609375f,1.53289999998211801113257024553604423999786376953125f,-0.146000008672475534154955312260426580905914306640625f,0.046100008258223912349649253883399069309234619140625f,0.182399997657537227269131108187139034271240234375f,1.231600041204691020624295560992322862148284912109375f,-0.0601000147104269188957914593629539012908935546875f,0.26450001883506768507459128159098327159881591796875f,-0.00310006471872359412600417272187769412994384765625f,-0.0404000124990939735170059066149406135082244873046875f,-0.642699959236383566718586735078133642673492431640625f,-0.525499957323074529114137476426549255847930908203125f,-0.3171999540060761457738180979504249989986419677734375f,0.4474000075668096432224274394684471189975738525390625f,-0.39110001131296190379771360312588512897491455078125f,0.864399993675201816500930362963117659091949462890625f,-0.023099966669082760972742107696831226348876953125f,0.17099997749179607353653409518301486968994140625f,0.6310000162720683647421537898480892181396484375f,1.2092000178612767324892729448038153350353240966796875f,-1.311300008547306106265750713646411895751953125f,0.3853999747060239400298087275587022304534912109375f,1.494600009852647293229210845311172306537628173828125f,0.82229997301921209373887222682242281734943389892578125f,0.1224000298440461875770779442973434925079345703125f,-0.65120001857280751522694117738865315914154052734375f,1.0477000195860857179042113784817047417163848876953125f,0.620000015795231007587062777020037174224853515625f,-0.216500004366040332826059966464526951313018798828125f,1.074999991953372813213718472979962825775146484375f,0.53509998848438211638267603120766580104827880859375f,-0.03299998384714086796520859934389591217041015625f,0.90129998910129049960460179136134684085845947265625f,2.2552000060260297686909325420856475830078125f,0.37529994824230661354391713757649995386600494384765625f,-0.18659998592138204998036599135957658290863037109375f,1.49910002444088519268916570581495761871337890625f,0.4678999873161318046044243601500056684017181396484375f,-1.2942999971628186273164828890003263950347900390625f,-0.153999994382262084258172762929461896419525146484375f,-0.0020999679803850312964641489088535308837890625f,0.39370000933408721977002642233856022357940673828125f,0.223299943935870714994962327182292938232421875f,1.35420000592172140585489614750258624553680419921875f,-2.131099960947036908720519932103343307971954345703125f,0.16540002002418041637810119937057606875896453857421875f,0.877999985337257538020594438421539962291717529296875f,0.767099992790818863142021655221469700336456298828125f,-1.39910005163550454909682230209000408649444580078125f,-0.19529998678043458681941046961583197116851806640625f,0.06900000190734889571331223123706877231597900390625f,1.700000042114407250437579932622611522674560546875f,-0.07370000337362370146365719847381114959716796875f,0.10759997933879505982446289635845459997653961181640625f,-0.99719996838569624486581233213655650615692138671875f,0.684200027081370176773589264485053718090057373046875f,-0.3078000461429351020115063874982297420501708984375f,-1.29349999140575544487319348263554275035858154296875f,-0.7776000070363282556940021095215342938899993896484375f,-0.89130000759735683590889721017447300255298614501953125f,1.50499996826052662157735539949499070644378662109375f,0.67109998717308005922177471802569925785064697265625f,1.3364000380098823850261169354780577123165130615234375f,-1.2633000670194629577736122882924973964691162109375f,-0.154399993246792543999390545650385320186614990234375f,1.00489998171329464327072855667211115360260009765625f,-0.091200001138448616444520666846074163913726806640625f,-1.1636999613523482111077100853435695171356201171875f,-1.441599980407954006977888639084994792938232421875f,-0.5274999913573263921051648139837197959423065185546875f,-0.3080999907344577071199864803929813206195831298828125f,0.217899999013543066439524409361183643341064453125f,1.097299991142749231443076496361754834651947021484375f,-0.8141999855816368647509762013214640319347381591796875f,-0.43490000257492056334740482270717620849609375f,-0.177800024312734894493814863380976021289825439453125f,-0.2066999511599529881777925766073167324066162109375f,-0.350800012928246918164632006664760410785675048828125f,0.2118999543547630537432269193232059478759765625f,0.828299997654557529358498868532478809356689453125f,0.718000024229287436838831126806326210498809814453125f,-1.057899954235554051962253652163781225681304931640625f,-0.5076999682515861422871239483356475830078125f,-0.822399995571374997638258719234727323055267333984375f,0.0206000213563442002850933931767940521240234375f,0.3957999760851260706573384595685638487339019775390625f,0.60550001722574275930810472345910966396331787109375f,-0.2484999778270715609096441767178475856781005859375f,0.41889997617006269337025514687411487102508544921875f,-1.38920003471821562612831257865764200687408447265625f,-0.7964999911785124453444950631819665431976318359375f,-0.234299974196404026116624663700349628925323486328125f,0.0876999847173696167601519846357405185699462890625f,-0.3838999642677605184104550062329508364200592041015625f,-0.48240000823736206569947171374224126338958740234375f,-0.0533999991774560811563787865452468395233154296875f,-0.6068999418377867272056391811929643154144287109375f,0.41669999496936771521404807572253048419952392578125f,-0.083099971064924726960043699364177882671356201171875f,-1.8622000622630121569045513751916587352752685546875f,-1.553000069975853048021008362411521375179290771484375f,0.2478000003218649993641520268283784389495849609375f,-0.295299968880414098748588003218173980712890625f,-0.68959993072748115849890382378362119197845458984375f,-0.08929994150698217225681219133548438549041748046875f,-0.39940000323057223141631766338832676410675048828125f,-1.31559996450841421022914801142178475856781005859375f,0.197100031757354798855885746888816356658935546875f,-0.5258000043600803419252542880713008344173431396484375f,0.826600002536177047574028620147146284580230712890625f,0.871000000774859728380761225707828998565673828125f,-0.769200029838085885813825370860286056995391845703125f,1.21600005143880895985830647987313568592071533203125f,-0.16819998450875228712675379938445985317230224609375f,-0.0180000887513163121411707834340631961822509765625f,0.4358999864369632604166326927952468395233154296875f,-0.21879995967149756808112215367145836353302001953125f,-1.47489996509850040951050686999224126338958740234375f,0.79329996688365955748167834826745092868804931640625f,0.4939000121563672518476550976629368960857391357421875f,-0.39589995685815893011749722063541412353515625f,1.212000009708106507133607010473497211933135986328125f,0.515800002348422825804163949214853346347808837890625f,0.95009999520853138932352521806024014949798583984375f,-0.186599991285800737017552819452248513698577880859375f,-0.981199986386298572682562735280953347682952880859375f,1.48949997487664287376674110419116914272308349609375f,0.1090000509135429229701230724458582699298858642578125f,0.21599998300522482708174720755778253078460693359375f,0.575099971571563894912060277420096099376678466796875f,0.47720005782991725684638595339492894709110260009765625f,0.04500003151595655115357885733828879892826080322265625f,-0.39140002027750053770205340697430074214935302734375f,1.13239998745024195159203372895717620849609375f,0.200099971497059048175515272305347025394439697265625f,-1.33820001609921401808378504938445985317230224609375f,-1.018599996039270205727689244667999446392059326171875f,0.00240000257492045676599445869214832782745361328125f,1.57710002341270527637107079499401152133941650390625f,0.38559998365640613116056556464172899723052978515625f,0.520699956755339865566156731802038848400115966796875f,-0.00229996849298430561248096637427806854248046875f,-0.51829999813884508608197165813180617988109588623046875f,-1.552300057937205490787846429157070815563201904296875f,0.6877000411182641048668529037968255579471588134765625f,-0.2255999924108385012999633545405231416225433349609375f,1.0035999797224999241507248370908200740814208984375f,0.52749998219311267089182138079195283353328704833984375f,0.691999977558851231407288651098497211933135986328125f,0.228999986112117692726997120189480483531951904296875f,-0.76030000881552695091158966533839702606201171875f,-0.677199997368454820190208920394070446491241455078125f,0.1116999713510284930606530906516127288341522216796875f,0.6656999839127071272315561145660467445850372314453125f,1.2953999886199822899612854598672129213809967041015625f,0.52479998197853472419183162855915725231170654296875f,-1.18189998445510813240844072424806654453277587890625f,-1.270699994958192302618726898799650371074676513671875f,-0.55670003460645745008150697685778141021728515625f,-1.004700047864765632965600161696784198284149169921875f,0.53279997171163540059524166281335055828094482421875f,-1.4206000463344157225265007582493126392364501953125f,0.88950002390146210018428973853588104248046875f,1.8305000064596532727279054597602225840091705322265625f,-0.1483999786138536780555341465515084564685821533203125f,1.29660000033825628662498274934478104114532470703125f,-0.70519999775588504409284951179870404303073883056640625f,0.9042999981500206008178111005690880119800567626953125f,0.2063999798953535247392210294492542743682861328125f,-0.03900002608448238650140638128505088388919830322265625f,-0.05670001093223697719025722108199261128902435302734375f,1.766899942899495812298482633195817470550537109375f,-0.7883999987304211032324019470252096652984619140625f,-0.3358000011563309072926131193526089191436767578125f,0.18160003107190103577295303693972527980804443359375f,0.0239999756813045195258382591418921947479248046875f,-0.55289998105764315283749965601600706577301025390625f,0.481699931192398622670225449837744235992431640625f,1.536100028896331792793716886080801486968994140625f,0.97779996962547244265806511975824832916259765625f,0.281600007230043303252386976964771747589111328125f,-0.7603000029295678974250449755345471203327178955078125f,-0.8449000131919977008010391728021204471588134765625f,0.437399986109137106637945180409587919712066650390625f,-0.5616000014558435449174567111185751855373382568359375f,-0.581599995830654581396856883657164871692657470703125f,1.6671000336572527888989725397550500929355621337890625f,-0.282200005117058783099537322414107620716094970703125f,-1.084800039757788159278106832061894237995147705078125f,0.088300007861852325419249609694816172122955322265625f,0.7752000196233392870226452942006289958953857421875f,-0.6763000180840494923728556386777199804782867431640625f,-1.766800043691695076830683319712989032268524169921875f,-0.683499998502432593028288465575315058231353759765625f,0.3221000010609618158241573837585747241973876953125f,-0.36860000561326788925953223952092230319976806640625f,-0.08849997663497877198324204073287546634674072265625f,0.1238999557927245032118435119627974927425384521484375f,-0.2379000501990322646861386601813137531280517578125f,-0.0442999763384455125247995965764857828617095947265625f,-0.562900007134676361175706915673799812793731689453125f,0.049200010503828384145208474365063011646270751953125f,-1.559900061210990340754278804524801671504974365234375f,1.27560004840195251318846203503198921680450439453125f,-0.2889999998956918059178633484407328069210052490234375f,1.15109998821616255071376144769601523876190185546875f,-1.37499997794628203706679414608515799045562744140625f,0.43660004373788741105499866534955799579620361328125f,0.601300011527538202926734811626374721527099609375f,-1.24630003134608191572851865203119814395904541015625f,0.56929995707869540666479224455542862415313720703125f,1.199300015044212130987943965010344982147216796875f,-2.068299998548627005590105909504927694797515869140625f,-0.782999980717898669269061429076828062534332275390625f,-0.3421000130563973673503141981200315058231353759765625f,1.968599968695639912397155057988129556179046630859375f,-1.0117000181034203620811240398325026035308837890625f,-1.011900007030368531246722341165877878665924072265625f,-0.049799997475743484898202950716949999332427978515625f,-0.37640001103877995802804434788413345813751220703125f,1.0636000296413901100578414116171188652515411376953125f,-0.718900019532442424718965412466786801815032958984375f,1.167800003711878975565241489675827324390411376953125f,0.15760001985132721102900177356787025928497314453125f,-1.678600006768107277821400202810764312744140625f,0.504799992185831047919464253936894237995147705078125f,0.5931000246718529655964857738581486046314239501953125f,-0.479300015789270561494817002312629483640193939208984375f,0.71350002046301963520846811661613173782825469970703125f,0.06610001120120312378958260524086654186248779296875f,0.069500005573034595585113493143580853939056396484375f,1.1374000263050201287029494778835214674472808837890625f,0.33739997783899244865324362763203680515289306640625f,-0.7515000153183937214862453402020037174224853515625f,0.5884000030517579915567694115452468395233154296875f,-2.00550002717226760751145775429904460906982421875f,0.633900025641917608254516380839049816131591796875f,-0.1694000151515007157598802223219536244869232177734375f,-0.471600018852949887104841764084994792938232421875f,1.2523000057086346448187441637855954468250274658203125f,-0.47689999641329039459236582843004725873470306396484375f,-0.3574999797344222685069325962103903293609619140625f,0.31420000808239034739699491183273494243621826171875f,-0.29190000650882641508587767020799219608306884765625f,1.767200006085634100827519432641565799713134765625f,0.08750002786517097064233894343487918376922607421875f,-0.0231000395357607857249604421667754650115966796875f,-0.41449997055530474199258605949580669403076171875f,-0.4635999993920325579210839350707828998565673828125f,-1.56919997112751019585630274377763271331787109375f,0.06660003286004012590382217240403406322002410888671875f,0.11510002603530811171594905317761003971099853515625f,1.10649999380484143074454550514928996562957763671875f,0.1061999895900485046951189360697753727436065673828125f,-0.01689998706281148344032772001810371875762939453125f,-0.58339999589920044087421047152020037174224853515625f,0.720899992434679948161146967322565615177154541015625f,-0.60839999850690329452618243522010743618011474609375f,-0.989800001685321007016682415269315242767333984375f,-0.4428000172346830964897890225984156131744384765625f,1.7663999955788252105293167915078811347484588623046875f,-1.3762999810546634460450832193600945174694061279296875f,-0.7783000015661121206989037091261707246303558349609375f,-0.036900012245773883279298388515599071979522705078125f,0.6678999909669156576086379573098383843898773193359375f,0.70400003092736029852716228560893796384334564208984375f,-0.39049998772144345338119819643907248973846435546875f,0.50159997988641225052930394667782820761203765869140625f,-0.67720001785755101764152641408145427703857421875f,0.9474000248149041514267310049035586416721343994140625f,0.258500010907649624414261779747903347015380859375f,0.470099983045458191810439529945142567157745361328125f,-1.116899997568129521141599980182945728302001953125f,-0.184200016352534579056055008550174534320831298828125f,-0.2256999846577638635380935738794505596160888671875f,1.17260002175122490797321006539277732372283935546875f,-0.95450001883506718769467624952085316181182861328125f,0.469599999570846193819306790828704833984375f,-0.1331999563455585899873767630197107791900634765625f,-1.0363999818325044799394163419492542743682861328125f,-0.1186000266611575426622948725707828998565673828125f,1.325799975228310501051964820362627506256103515625f,-0.04629995922446283884710283018648624420166015625f,-0.38779998750686672792653553187847137451171875f,0.298700031834840729061397723853588104248046875f,-0.327100027883052035804212209768593311309814453125f,-1.0732999768674371665611033677123486995697021484375f,1.27599996158480788466249578050337731838226318359375f,-0.5623999913990491705817476031370460987091064453125f,-0.0453999955862760540270528508699499070644378662109375f,-1.1579000295236705486701112022274173796176910400390625f,-0.186400006791949568452082530711777508258819580078125f,0.66319998897910081847584251590888015925884246826171875f,-0.4437999984145164500404234786401502788066864013671875f,-0.3373999719902871508026009905734099447727203369140625f,-0.282300035473704635791136752231977880001068115234375f,-0.3945999907419082131099230537074618041515350341796875f,0.9178999910414216145682075875811278820037841796875f,0.7743999913468953000261763008893467485904693603515625f,0.1977000198841096789692528545856475830078125f,-0.43139999211430524184152091038413345813751220703125f,0.505400013840198614190057924133725464344024658203125f,-0.53049998891353578045482208835892379283905029296875f,1.243100011372565649736543491599150002002716064453125f,-0.8125000175833694271432250388897955417633056640625f,1.513800074857474253775535544264130294322967529296875f,0.06759997268915096668706610216759145259857177734375f,-0.5423000079438085041516615092405118048191070556640625f,1.0601000235021107442889842786826193332672119140625f,-0.0442000172093515164561949859489686787128448486328125f,0.0227000110179191860737546448945067822933197021484375f,-0.8383000091284511512412791489623486995697021484375f,-0.48629999692440062375453635468147695064544677734375f,-0.1346999847769740910052860272116959095001220703125f,-0.17649994990229611602217119070701301097869873046875f,-0.2632000368118294630903619690798223018646240234375f,-0.376600003504752312011305548367090523242950439453125f,1.22659996953010619336055242456495761871337890625f,1.52979997400641476446025990298949182033538818359375f,-0.118300009799003902344338712282478809356689453125f,0.49790002561211543774533083706046454608440399169921875f,-0.89919995473623348658520626486279070377349853515625f,0.3485999765187504095820258953608572483062744140625f,-0.4496000095546239805344157502986490726470947265625f,-1.99349997350573548970942283631302416324615478515625f,-0.7991999912440774789246233922312967479228973388671875f,-1.0511000426799064211280665404046885669231414794921875f,1.053000022143125935514262891956605017185211181640625f,0.20909999065250151062400618684478104114532470703125f,0.6178000090390440846022102050483226776123046875f,-0.27769997603744267511416410343372263014316558837890625f,0.1523999877482655751492757190135307610034942626953125f,-0.72860001965761178599478853357140906155109405517578125f,0.6970999991238111537228405722999013960361480712890625f,-0.263699959936737560184383255545981228351593017578125f,-0.193800032687187151481111868633888661861419677734375f,0.081199927489458811891154255135916173458099365234375f,-1.120000063627958564183018097537569701671600341796875f,0.49540003231763929392172940424643456935882568359375f,-0.189900016433000917714934985269792377948760986328125f,0.66780001850128201112966053187847137451171875f,-0.112299956423043578723763857851736247539520263671875f,0.8293000010371205377168735140003263950347900390625f,-1.5705000489279630215833094553090631961822509765625f,0.54190002498626643756551857222802937030792236328125f,0.664100037144125021626450688927434384822845458984375f,0.43159997042417419521598276332952082157135009765625f,1.030299994026124377199948867200873792171478271484375f,0.6474000431060797922100391588173806667327880859375f,0.566799992394447116339506465010344982147216796875f,-0.32010002901554113208248963928781449794769287109375f,-0.106300036859511948250656132586300373077392578125f,-0.0422999954640861375310123548842966556549072265625f,0.6518000281572344789537964970804750919342041015625f,1.228099964135885624472166455234400928020477294921875f,-1.4533000059247012103469387511722743511199951171875f,-0.427599978351592913128342843265272676944732666015625f,-1.3793999296188363956616740324534475803375244140625f,1.0198999981045719920302872196771204471588134765625f,0.2146000177502624950420795357786118984222412109375f,0.8372000042974949707286214106716215610504150390625f,-0.9894000033423306916802175692282617092132568359375f,-0.57279998005628574020420273882336914539337158203125f,-0.0126000349760053698133788202540017664432525634765625f,0.471299991001188356420925629208795726299285888671875f,0.27669999966323377549315409851260483264923095703125f,-0.10070003405511374072744956720271147787570953369140625f,0.3197999996364109254187724218354560434818267822265625f,0.5950000704824931485603656255989335477352142333984375f,-0.47570003781765712869145090735401026904582977294921875f,-1.157600050771236954716414402355439960956573486328125f,0.6149000163212419156621990623534657061100006103515625f,-0.1516999913185832493667248854762874543666839599609375f,0.458300025147199807662445891764946281909942626953125f,-0.63470002530813207641813278314657509326934814453125f,0.9498000051125881082469959437730722129344940185546875f,0.40319997929334672193135702400468289852142333984375f,-0.9361999823629851835704585027997381985187530517578125f,-0.32770000677108779285617856658063828945159912109375f,-0.256599938610195810184677611687220633029937744140625f,-1.01080006947815537188262169365771114826202392578125f,0.093899999341368722838296889676712453365325927734375f,0.81080002361983105618747913467814214527606964111328125f,-0.367000009521841974446942913345992565155029296875f,0.02970002143382988180064785410650074481964111328125f,0.4285999761834740073851435226970352232456207275390625f,-0.0699999947845935110990467364899814128875732421875f,1.93609991981983231568165138014592230319976806640625f,0.520299946379662259232645737938582897186279296875f,0.424699985969066773350277799181640148162841796875f,-2.145799972993134741017229316639713943004608154296875f,-0.33019996817708108238775821519084274768829345703125f,-0.2424000058531763368563360927510075271129608154296875f,1.876799973171948909822503992472775280475616455078125f,0.408099960224330260416536475531756877899169921875f,-0.057099999794364197924778636661358177661895751953125f,-0.26599997454881663117021162179298698902130126953125f,0.87109999574124774568417706177569925785064697265625f,-0.55619998321980201883008021468413062393665313720703125f,0.7580000120848418010410796341602690517902374267578125f,0.981399981310963287484128159121610224246978759765625f,0.23450004491210041379645190318115055561065673828125f,-0.571700007560849154941706729005090892314910888671875f,-0.5481999934494492521253050654195249080657958984375f,-0.298900001390278446944392953810165636241436004638671875f,-0.57550001573562614698431616488960571587085723876953125f,0.252100015217065820394282127381302416324615478515625f,-0.5294000150397419890424544064444489777088165283203125f,-0.093999989427626218230216181837022304534912109375f,-0.270399975320696484004656667821109294891357421875f,0.7473000081300735786271616234444081783294677734375f,-0.6066000135630371659090087632648646831512451171875f,0.9242999976471064638161578841391019523143768310546875f,0.00969996898174256549918226255613262765109539031982421875f,0.67839998366162201914875140573712997138500213623046875f,0.2290999871134757892576772064785473048686981201171875f,1.277700040745734622049667450482957065105438232421875f,-0.29999997623264829105238504780572839081287384033203125f,-0.8833000007838018863282059101038612425327301025390625f,0.14640004404485207345487651764415204524993896484375f,-1.2943000309139491843524183423141948878765106201171875f,-0.45339997999221093305521890215459279716014862060546875f,0.7840000188946729675620872512809000909328460693359375f,0.856800049018860310212630793103016912937164306640625f,0.07949999711662503187881156918592751026153564453125f,0.6197000182487073516313103027641773223876953125f,1.3164999955557288302543383906595408916473388671875f,-0.05740001654475934689259020160534419119358062744140625f,-0.295899997389316826712502006557770073413848876953125f,-0.08090002296343457022231859809835441410541534423828125f,-0.1051000431716442307106262887828052043914794921875f,-0.7895000154264273373172500214423052966594696044921875f,0.12030002595186228830925756483338773250579833984375f,-0.1709999744929373566293406838667578995227813720703125f,1.275500043898821100896157076931558549404144287109375f,-1.234500071138144416948989601223729550838470458984375f,0.642200032516569763174629770219326019287109375f,-0.9504000607974834036184574870276264846324920654296875f,-0.2436000641934576993463679173146374523639678955078125f,0.1622999496057626611644764125230722129344940185546875f,-0.379600057238339783083347356296144425868988037109375f,0.87270006905794250684493817971087992191314697265625f,0.2013999641001220641811642053653486073017120361328125f,0.46930001504421259284072220907546579837799072265625f,-0.936600020939110766704516208847053349018096923828125f,-0.6149000077903268302037531611858867108821868896484375f,-0.0991999985456468724720480167889036238193511962890625f,-0.5064000218421227383913674202631227672100067138671875f,-0.099699975463747259052382787558599375188350677490234375f,-0.766400033204257535857095717801712453365325927734375f,0.9715000596120955123069506953470408916473388671875f,0.11780001097619552385964425411657430231571197509765625f,0.198199925053119674345225575962103903293609619140625f,1.7344999911189085128881970376824028789997100830078125f,-0.51440001708865157326044936780817806720733642578125f,-0.036100001478195142823324204073287546634674072265625f,1.37660001333951953483847319148480892181396484375f,0.37419999778196266770891043051960878074169158935546875f,0.324699979859590204256392098614014685153961181640625f,-0.50000000528991239701070981027442030608654022216796875f,0.8522999521017080581941627315245568752288818359375f,-0.539700005694478956996817942126654088497161865234375f,1.42829999728202761133388776215724647045135498046875f,-1.21680003620386134599584693205542862415313720703125f,1.3386999723792083472062586224637925624847412109375f,-0.14979997480735163239984331084997393190860748291015625f,0.2790000010132782648497595801018178462982177734375f,0.30089997986182559497336796994204632937908172607421875f,-0.139599997186660829129323246888816356658935546875f,-0.227300059092045092512535120476968586444854736328125f,0.1084999688863759192969382638693787157535552978515625f,-0.8554999798238274166806149878539144992828369140625f,0.55789995058476815614767474471591413021087646484375f,-1.3310000368356700306549100787378847599029541015625f,-1.3544000503927460243858149624429643154144287109375f,-1.183299986255168079196664621122181415557861328125f,1.16619998213946818310660091810859739780426025390625f,0.7883000023484232343662370112724602222442626953125f,-0.15770004685521055165509096696041524410247802734375f,-0.06219999312162372007151134312152862548828125f,1.2313000107079747902361077649402432143688201904296875f,0.1617999859869476697582513224915601313114166259765625f,0.37499998733401351813654400757513940334320068359375f,-0.3628000160425894904392407624982297420501708984375f,0.2974000431060783711245676386170089244842529296875f,-1.1924999802559614892061290447600185871124267578125f,2.26669997738599793279945515678264200687408447265625f,0.0068999705225234464478489826433360576629638671875f,-0.5347000315666203817954738042317330837249755859375f,0.16940001083016387184443374280817806720733642578125f,0.62099995547533115569649453391321003437042236328125f,1.1726999616950755100930336993769742548465728759765625f,-1.35359998806714987296118124504573643207550048828125f,1.1966999973163006121268381320987828075885772705078125f,0.37990004081502615207455164636485278606414794921875f,1.39510003613084609952466053073294460773468017578125f,-1.178899988612533089593625845736823976039886474609375f,0.302200005862116594101962618879042565822601318359375f,-0.1429000539988274365299503187998197972774505615234375f,-2.353700014251470573611868530861102044582366943359375f,1.802700035381317178462268202565610408782958984375f,-0.05619999221637794750705552360159344971179962158203125f,0.27930000163316659467227509594522416591644287109375f,-0.168100030669570355001951611484400928020477294921875f,0.3349000392317780239181956858374178409576416015625f,0.2547000010192397301267419607029296457767486572265625f,1.2034000228516765762520890348241664469242095947265625f,0.8449999653361734974765795413986779749393463134765625f,-0.69950001540779993547403137199580669403076171875f,-0.4302999411642549620893305473146028816699981689453125f,-1.8381000247836121186395530457957647740840911865234375f,0.72769999477565239232035310124047100543975830078125f,1.764999994561076590571246924810111522674560546875f,0.43479993541240613552645299932919442653656005859375f,-0.57850007162988159592487136251293122768402099609375f,-0.2019999660849567391096570645458996295928955078125f,0.251400013571977165582893576356582343578338623046875f,0.1435999915316703567924605522421188652515411376953125f,-0.61870001233294658060657411624561063945293426513671875f,0.91039999700188600773032021606923080980777740478515625f,0.404599939668179420237947852001525461673736572265625f,1.201399984589219815944716174271889030933380126953125f,0.771300017711519902974259821348823606967926025390625f,-0.391299970325828194717132646474055945873260498046875f,-0.7343999776005745161455706693232059478759765625f,-0.469700041475892504649891634471714496612548828125f,1.1113999618649490486177455750294029712677001953125f,1.193599978977442521710372602683492004871368408203125f,0.31799997431039717099565677926875650882720947265625f,0.7655999919638036832481020610430277884006500244140625f,0.05369997938275350435333166387863457202911376953125f,-0.2752000042378901500939036850468255579471588134765625f,0.822600030542910420905400314950384199619293212890625f,0.72800003235042110016905780867091380059719085693359375f,-1.9609000622652477119345348910428583621978759765625f,-1.030700040220469571039529910194687545299530029296875f,1.61289999446868836940893743303604423999786376953125f,-0.5075000542588536944776933523826301097869873046875f,-0.906099951112270929343139869160950183868408203125f,0.783299970012902857519065946689806878566741943359375f,-1.49550003778934392784094598027877509593963623046875f,0.9721000208795071984013702603988349437713623046875f,0.06310001114904906227565106746624223887920379638671875f,0.35530000360012081728200428187847137451171875f,0.0667999877937137664929423408466391265392303466796875f,-2.21070003977343443324343752465210855007171630859375f,-1.0155000138171021983879427352803759276866912841796875f,0.2349000217601660001065511096385307610034942626953125f,-0.580200046780706912841196754015982151031494140625f,-0.7299000323772428799884437466971576213836669921875f,-0.014599981391430372212880683946423232555389404296875f,0.97420006768703526489616706385277211666107177734375f,-1.42850004234910077371978331939317286014556884765625f,0.12300000716745851381261900314711965620517730712890625f,-0.06339999798536322117570307455025613307952880859375f,-0.3505999908834696032755573469330556690692901611328125f,1.281299977701902292892555124126374721527099609375f,0.38889997665435060980598791502416133880615234375f,-0.6570999577358360710377382929436862468719482421875f,-0.93289998150467834392429722356610000133514404296875f,1.28700006622076035256441173260100185871124267578125f,-0.812499995827674847959087855997495353221893310546875f,-0.607900042152405006845583557151257991790771484375f,1.12909999977946284843710600398480892181396484375f,-0.3168999668717393802808146574534475803375244140625f,0.4805000252909958913249965917202644050121307373046875f,-0.0524000371456148084092774297459982335567474365234375f,-0.50780001937672525347267082906910218298435211181640625f,-0.86339997943341728614541352726519107818603515625f,0.12679998585656304666002824887982569634914398193359375f,1.9314999688267704414101899601519107818603515625f,-0.77950001434609317296775543582043610513210296630859375f,0.403200028765201778924165409989655017852783203125f,-0.07589999321699114620543014098075218498706817626953125f,0.76910002406835520361028102342970669269561767578125f,-0.2566000029832122830697471727035008370876312255859375f,-0.3986000166773795427843651850707828998565673828125f,-0.11069996532350767637353783356957137584686279296875f,-0.4685999899297959014887737794197164475917816162109375f,0.64889999688267696598131806240417063236236572265625f,0.83779999354183676274487879709340631961822509765625f,0.668699988174438431087764911353588104248046875f,-0.803100034767389114875868472154252231121063232421875f,-1.1925000011920925402364446199499070644378662109375f,1.075799983423948003036230147699825465679168701171875f,0.449699948418139427985806833021342754364013671875f,-0.7139000283986323136531382260727696120738983154296875f,-1.13820003174543415980224381200969219207763671875f,0.4231000004947187420611953712068498134613037109375f,0.49530000246390704088383927228278480470180511474609375f,-0.1650999524235723381337948012514971196651458740234375f,-0.8559999736547470039482732317992486059665679931640625f,-1.23389996939003498965803373721428215503692626953125f,-0.16280000382363812860830876161344349384307861328125f,2.183099955344200537865617661736905574798583984375f,0.128599976295233009437879445613361895084381103515625f,-0.75899999415874486174971025320701301097869873046875f,0.2555999709159129107405306058353744447231292724609375f,1.45729999084472705561665861750952899456024169921875f,0.37769996102452285668960030307061970233917236328125f,-1.1264999893903731020827763131819665431976318359375f,0.4945000276826323482026737110572867095470428466796875f,0.7342999872535467265066699837916530668735504150390625f,0.61250000108033397783202644859557040035724639892578125f,-1.143200000080466072205354066682048141956329345703125f,-0.347900018217414508381324367292108945548534393310546875f,0.716099980169534688911880948580801486968994140625f,-0.76519998374879349967159214429557323455810546875f,-0.0010000398159022427790887377341277897357940673828125f,-0.7612999646447595747389414100325666368007659912109375f,-0.099099982121586993599748893757350742816925048828125f,-1.092699998612701950406744799693115055561065673828125f,0.27069996431768006406315407730289734899997711181640625f,-0.875700030106306481769706806517206132411956787109375f,0.6152000142216678124640338864992372691631317138671875f,-0.914700004148482737065251058083958923816680908203125f,-0.397000025987625182466445039608515799045562744140625f,0.15179999284148237848057760857045650482177734375f,-0.600999989822506019976344759925268590450286865234375f,-1.841899974024295172370102591230534017086029052734375f,-0.13769995774626675455465374398045241832733154296875f,-0.644100015425682403957807764527387917041778564453125f,-1.560600017556547758346141563379205763339996337890625f,-1.3590000163614757155983170378021895885467529296875f,0.125900055243075126298890609177760779857635498046875f,0.1284999991357320681117926142178475856781005859375f,0.1888000016182658225005752683500759303569793701171875f,-0.5859999946653842162191949682892300188541412353515625f,-0.2650999590545897177662482135929167270660400390625f,-0.15599998438358309016393832280300557613372802734375f,1.31269995789527893492731891456060111522674560546875f,-0.674700014951824744713348991353996098041534423828125f,0.0240999967992305830222221629810519516468048095703125f,-1.1077999977886678895089289653697051107883453369140625f,1.527300023701787523577877436764538288116455078125f,-0.319100014308094870330023695714771747589111328125f,-0.24290001533031502223280995167442597448825836181640625f,1.13439997868090802768392677535302937030792236328125f,0.53630004699230227060979814268648624420166015625f,1.2021000238969927753629463040851987898349761962890625f,-0.068999988384545130060132578364573419094085693359375f,0.592399994975327803814479921129532158374786376953125f,-0.27970001638606156735278318592463620007038116455078125f,-0.576200027886033527835252243676222860813140869140625f,-0.14269997860789285226701395004056394100189208984375f,0.648099990101159040278844258864410221576690673828125f,1.206700009796022765584666558424942195415496826171875f,0.5348999739274378217857019990333355963230133056640625f,-0.2003999859377738612664643369498662650585174560546875f,0.815499979376792083485270268283784389495849609375f,-1.801500022768974584863599375239573419094085693359375f,1.58930001325607150874930084683001041412353515625f,-1.28950003194808981987762308563105762004852294921875f,1.0543999894097446645702120804344303905963897705078125f,1.070899977943300829252848416217602789402008056640625f,-0.0903999960519377143697283827350474894046783447265625f,0.5124999864399428783912071594386361539363861083984375f,-1.08279992395341384536777695757336914539337158203125f,0.58280001265257563858313005766831338405609130859375f,-1.292599967958032447512550788815133273601531982421875f,-0.38509998932257294423919802284217439591884613037109375f,0.501199983387440095583542642998509109020233154296875f,-0.72400002668052909626794644282199442386627197265625f,0.311099997603892841624428911018185317516326904296875f,-0.2979000127412378962077355026849545538425445556640625f,0.1389999975487587191569360811627120710909366607666015625f,-0.196900035547465268592759457533247768878936767578125f,0.22330000339150390953335545418667607009410858154296875f,0.774600031943619260488276268006302416324615478515625f,-0.678400015196203742817715465207584202289581298828125f,0.75320002261847307511999360940535552799701690673828125f,0.1236000133246182297597215438145212829113006591796875f,-1.284600061820448058114152445341460406780242919921875f,-0.6612999854505059449394366311025805771350860595703125f,-0.539800020796060930905468921991996467113494873046875f,0.42480002228617674830957184894941747188568115234375f,-0.71190001973360761411413477617315948009490966796875f,0.9460000390335920883444487117230892181396484375f,-0.21829998904913605617394978253287263214588165283203125f,1.080200066468865127689014116185717284679412841796875f,0.08249998152256043937313734204508364200592041015625f,-1.279700034211576298304180454579181969165802001953125f,0.18089999828338587661846759147010743618011474609375f,-1.787500048764050664118485656217671930789947509765625f,1.10990006396770635177517760894261300563812255859375f,0.1804000280335535943976310591096989810466766357421875f,-0.999900007864833018089711913489736616611480712890625f,0.397299996507166408576949834241531789302825927734375f,-0.40660000194013090624167716669035144150257110595703125f,0.746899965493380335601614206098020076751708984375f,-1.1308999701760706901865205509238876402378082275390625f,1.588299998604506324539897832437418401241302490234375f,0.81529998482465781961536777089349925518035888671875f,0.46749995797872667679939695517532527446746826171875f,-0.121699984538555572299856066820211708545684814453125f,-0.59789998150467926762985371169634163379669189453125f,0.693099978739023736551416732254438102245330810546875f,-0.34710000091195081228789831584435887634754180908203125f,0.208400025068223637703113126917742192745208740234375f,0.5391999869972465830869623459875583648681640625f,-0.1386000023722646634638522300519980490207672119140625f,0.996500000268220720300860193674452602863311767578125f,-0.4733999948933724777333509337040595710277557373046875f,1.202799999055267132774815763696096837520599365234375f,0.2084999827817084483427834129543043673038482666015625f,0.112500004395842179150122319697402417659759521484375f,-0.031799985241889761056199859012849628925323486328125f,-0.265600004369020314953786510159261524677276611328125f,0.784799985145032241717899523791857063770294189453125f,0.58630002128779901138244667890830896794795989990234375f,-0.141800022345781595589642165577970445156097412109375f,-0.95029998914599378423417874728329479694366455078125f,-0.2154999808669089134127716533839702606201171875f,-0.6883000192239887571332701554638333618640899658203125f,0.659200007486344219387319753877818584442138671875f,-0.2995999917104839038728414379875175654888153076171875f,-0.03169999943971646416684961877763271331787109375f,-1.077799996447563746215791979921050369739532470703125f,-0.0025000146031377568078823969699442386627197265625f,-0.2804000055328013107924789437674917280673980712890625f,-0.6130000061988827297909665503539144992828369140625f,1.810299915683270199906473862938582897186279296875f,-0.3453000232696528115639011957682669162750244140625f,0.7269999788999559342528300476260483264923095703125f,-1.15539996407032052871954874717630445957183837890625f,0.6205999744176864396649762056767940521240234375f,-0.60920005382895414669519595918245613574981689453125f,1.7277999428033847806318590301088988780975341796875f,-1.64409996789097778702171126496978104114532470703125f,-0.3456999733328824930822520400397479534149169921875f,0.26239999542236347451762412674725055694580078125f,0.803899966931343357146033667959272861480712890625f,-0.63260001158118228659077431075274944305419921875f,-0.45560001723989829169880749759613536298274993896484375f,0.29079996362402982246209148797788657248020172119140625f,-1.00930001282766479420160976587794721126556396484375f,0.205900052374601738591763933072797954082489013671875f,-0.516100040742755172828992726863361895084381103515625f,0.985500022105873174638190903351642191410064697265625f,0.743900010442733350402022551861591637134552001953125f,0.02669992418885158702579474265803582966327667236328125f,0.4008000644862652617206322247511707246303558349609375f,0.5577000510275367251011857661069370806217193603515625f,-0.157000030457973505093605126603506505489349365234375f,-1.0161000293061139831962691459921188652515411376953125f,1.4048000221557914724002102957456372678279876708984375f,-1.1291000296004110037273449052008800208568572998046875f,0.99260004581660055134051390268723480403423309326171875f,0.071799949777125338101768647902645170688629150390625f,-0.57069999933540760395089819212444126605987548828125f,0.9881999819010485186510095445555634796619415283203125f,-1.6270000313147907622379761960473842918872833251953125f,-0.2742999688506113642461059498600661754608154296875f,-0.9104999724104996250417798364651389420032501220703125f,-0.32460005768537580905785944196395576000213623046875f,-0.5852999937653537987358731697895564138889312744140625f,0.8844000303506849292034530662931501865386962890625f,0.9462999739021065170874180694227106869220733642578125f,-0.615700000099092203953432544949464499950408935546875f,1.1051999533131720010459275727043859660625457763671875f,1.8067999695956711292410545866005122661590576171875f,0.34709998884201009872185750282369554042816162109375f,0.3196999902158967898913033423013985157012939453125f,-0.10639992744326765006235291366465389728546142578125f,-2.418500029087066405253381162765435874462127685546875f,-0.288799982023238754891281132586300373077392578125f,0.31009996196031597293085724231787025928497314453125f,2.094099965953827080511473468504846096038818359375f,0.07069997079968448616682508145458996295928955078125f,-0.9439999892413613036268316136556677520275115966796875f,0.9071000278085470380773358556325547397136688232421875f,-0.10230002273321137096218080841936171054840087890625f,-1.8573000319905579313939369967556558549404144287109375f,0.69979999654442048839797507753246463835239410400390625f,0.934100041409582360074637108482420444488525390625f,0.00010000740885740544428017528844065964221954345703125f,-1.37030002952814111694124221685342490673065185546875f,1.12930003918409394003674606210552155971527099609375f,0.12670002663433532763548328148317523300647735595703125f,-0.9987999967753877772480564090074039995670318603515625f,2.04180000364482427954726517782546579837799072265625f,0.01859997927546508389440305109019391238689422607421875f,-0.82220004736185092752975833718664944171905517578125f,0.173900006121396888403296543401665985584259033203125f,0.698200028429925456663340810337103903293609619140625f,0.2233000074893236597262102804961614310741424560546875f,0.22340001538246856416236596487578935921192169189453125f,-0.7006000533007090691484108901931904256343841552734375f,0.863700080859661767362922546453773975372314453125f,1.1250999953761693372911167898564599454402923583984375f,0.0268000096797944564741555950604379177093505859375f,0.28179994325786805742239948813221417367458343505859375f,0.00999998360872300651180921704508364200592041015625f,-0.7078000131368629155304006417281925678253173828125f,-0.45520001764893525120214690105058252811431884765625f,1.48669995667338383782407618127763271331787109375f,0.07170000480413440957505599726573564112186431884765625f,-0.177600000144541692037591928965412080287933349609375f,1.5783000317595894390620969716110266745090484619140625f,-0.991199999927729447080082536558620631694793701171875f,-1.2727999681353576733044974389486014842987060546875f,-0.0873999680042256699152858345769345760345458984375f,1.085999976634978825273947222740389406681060791015625f,-0.7575000074505791047840830287896096706390380859375f,1.39680001623630456464297822094522416591644287109375f,-0.4675000101141633113144280287087894976139068603515625f,0.0889000060096385436025911985780112445354461669921875f,0.16800000649690616238984830488334409892559051513671875f,1.4616999986574048531196012845612131059169769287109375f,-0.655999987065791767548716961755417287349700927734375f,-0.6882999642014497876374434781610034406185150146484375f,1.127699975478649196958258471568115055561065673828125f,0.610799968522786951297121049719862639904022216796875f,-0.115499958068132624333657076931558549404144287109375f,-0.8257000246301295920403617856209166347980499267578125f,0.647600012400746383178784526535309851169586181640625f,1.5208000157967209720055734578636474907398223876953125f,-0.6110000293105841695506796895642764866352081298828125f,0.159500016301870051194100597058422863483428955078125f,0.8359000070005653260096778467413969337940216064453125f,-1.18010002186298290638433172716759145259857177734375f,-0.70390001260340195887721392864477820694446563720703125f,0.4304999647736540424602935672737658023834228515625f,-0.0063999960631135888178278037230484187602996826171875f,-0.2832000076055525283891256549395620822906494140625f,-1.5390000147968529997655195984407328069210052490234375f,0.62020001153945969463165965862572193145751953125f,-0.0925999780163164754043236825964413583278656005859375f,-0.459100031220913251672754995524883270263671875f,-1.49779998737648156748036853969097137451171875f,-0.2816999621123079666773492135689593851566314697265625f,-0.388999961525202220258279339759610593318939208984375f,0.100899965575337535739208760787732899188995361328125f,0.705999945566059050605645097675733268260955810546875f,-1.16730002183914205460268931346945464611053466796875f,1.9325999902725214951715315692126750946044921875f,-0.732900030827522641629911959171295166015625f,-0.2159999718666085755103267729282379150390625f,-0.00060003238320316398812792613171041011810302734375f,0.5969999911189081132079081726260483264923095703125f,-0.38540002859234778753716454957611858844757080078125f,0.252000009968876614863120266818441450595855712890625f,-0.91349999567121253907231448465608991682529449462890625f,0.3672999882370235891215770607232116162776947021484375f,1.125600004070996806859739081119187176227569580078125f,-0.32320000306069829321131692267954349517822265625f,-0.91159998566061251068504134309478104114532470703125f,1.260199980917573014949084608815610408782958984375f,-0.0338999961376187375350355068803764879703521728515625f,0.71959999130070162465244720806367695331573486328125f,-0.4291000080496074620128865717560984194278717041015625f,0.947600003311037664133209545980207622051239013671875f,-0.263000001840293151644800673238933086395263671875f,-0.8652000115767120025367376001668162643909454345703125f,-0.17019998203516006896052203956060111522674560546875f,1.0290999932229514257642222219146788120269775390625f,-0.620500010222195808040623887791298329830169677734375f,-0.37610000825822342829241051731514744460582733154296875f,0.751200002777576258949920884333550930023193359375f,-0.558599998684227383449751869193278253078460693359375f,0.248499983489512743517479975707828998565673828125f,-0.51509999019801622210934510803781449794769287109375f,0.854999997466802330592372527462430298328399658203125f,-0.4551999658301462137188764245365746319293975830078125f,0.5156000019103281761800872118328697979450225830078125f,-0.031499982982874374926041127764619886875152587890625f,-0.813399969524145394217384819057770073413848876953125f,0.149999977573752829584918799810111522674560546875f,0.877699984642863473283114217338152229785919189453125f,0.893899940481782895318474402301944792270660400390625f,-0.2404000285923475477289912305423058569431304931640625f,0.1299999779462819304853837820701301097869873046875f,-0.248100000569224210522634166409261524677276611328125f,-0.404099981600047453156321353162638843059539794921875f,0.768199945393205130272917813272215425968170166015625f,0.373300025817751457424265026929788291454315185546875f,-0.5750999928429723073719515014090575277805328369140625f,-0.303899981385469697414691836456768214702606201171875f,-0.02930003523528645903439837638870812952518463134765625f,-0.4706000001966950119225430171354673802852630615234375f,-0.8677999652482564219013738693320192396640777587890625f,-0.1047000397250059389175902424540254287421703338623046875f,0.7921000143043699370792865011026151478290557861328125f,-0.47279998624026753351046181705896742641925811767578125f,1.176399970805644290550162622821517288684844970703125f,-0.9643999695539469119154318832443095743656158447265625f,-0.739699997331201952732726567774079740047454833984375f,0.0957999893471599417438255841261707246303558349609375f,0.16779999436140047297527644332149066030979156494140625f,-0.7870999936103821870148067318950779736042022705078125f,0.3092999633371835699335861136205494403839111328125f,0.9524999704211953854837702238000929355621337890625f,-0.951599993520975129257521984982304275035858154296875f,0.643299982529878189296823620679788291454315185546875f,-0.759800005820393753452890450716949999332427978515625f,0.4551999613970512115201927372254431247711181640625f,1.2176000129222881440682613174431025981903076171875f,-0.3764000142425292239067857735790312290191650390625f,-0.0937000388383875115749788164976052939891815185546875f,1.303599990451334900143365302938036620616912841796875f,-0.8066999933302396730283589931786991655826568603515625f,0.255500003144145171329881804922479204833507537841796875f,-0.08319999724924487249921867260127328336238861083984375f,1.3905000472143296263993761385791003704071044921875f,0.16260001718774408363543670930084772408008575439453125f,0.691400013851374506401725739124231040477752685546875f,-0.2312000158905984648072262643836438655853271484375f,-0.7739999990575017196903218064107932150363922119140625f,0.394200000333785993689161841757595539093017578125f,0.4613000120863315345332011929713189601898193359375f,-2.2139999630525704077399495872668921947479248046875f,-0.456300000724195964352247756323777139186859130859375f,1.265599950315058119798550251289270818233489990234375f,0.27150000138580787645281588993384502828121185302734375f,1.9689000344149771848378804861567914485931396484375f,0.3578999698631462056397367632598616182804107666015625f,0.1738999909035865432116452211630530655384063720703125f,-2.54050005198270145712058365461416542530059814453125f,-1.013199973705411149893507172237150371074676513671875f,-1.546300004076957890220000990666449069976806640625f,0.216500062853096864046165137551724910736083984375f,-0.90160001065731076863585258251987397670745849609375f,-1.460099970714747730227145439130254089832305908203125f,0.67749998122453625182259884240920655429363250732421875f,1.1085000297203659869182956754229962825775146484375f,-0.39510003393292425499794262577779591083526611328125f,-1.3473000193431976523328330586082302033901214599609375f,0.2503000012531872986443204354145564138889312744140625f,1.009400008110702628982835449278354644775390625f,-0.30520004029870018147363452953868545591831207275390625f,0.8675000274181368098425082280300557613372802734375f,-1.2787999967753886920718287001363933086395263671875f,0.044400021111965060072179767303168773651123046875f,-1.085799986255169269355747019289992749691009521484375f,-0.2180999527364975154597459550132043659687042236328125f,0.139700020688771520127602343563921749591827392578125f,-0.5707000015333292264330111720482818782329559326171875f,-1.0255000182874500769258929722127504646778106689453125f,-0.99409999203085863683782008592970669269561767578125f,-0.21649998164176931236823975268634967505931854248046875f,-0.703600083546340737683522092993371188640594482421875f,-1.69010004412159364761691904277540743350982666015625f,0.62710005183666905903550059520057402551174163818359375f,0.19269999462664111344878392628743313252925872802734375f,-0.243400010204315009332276531495153903961181640625f,-1.2376000398747628583606683605466969311237335205078125f,1.012100062304736258766979517531581223011016845703125f,1.27850005562976054562795980018563568592071533203125f,-0.4817000156819819522979742032475769519805908203125f,-0.58359999149441676991045824252068996429443359375f,-0.42339999165236952283208893277333118021488189697265625f,-0.58459997900724403052663546986877918243408203125f,-1.4942000268951065589106974584865383803844451904296875f,0.33370003049373686554446294394438154995441436767578125f,0.32630001812130171590098370870691724121570587158203125f,-1.359600009368360229444760989281348884105682373046875f,-1.369400024688245398607477909536100924015045166015625f,1.4461000493109228415278266766108572483062744140625f,1.929300061535835109083336647017858922481536865234375f,-0.168700003075599624935421161353588104248046875f,-0.371100025469064842553734706598334014415740966796875f,-0.53909998972117889604049878471414558589458465576171875f,-0.16870000545978580674955082940869033336639404296875f,-0.0906999882340431273775038789608515799045562744140625f,0.3110000336319209690572051840717904269695281982421875f,-1.0206999656587836255994261591695249080657958984375f,0.3656000069394707008285649862955324351787567138671875f,-0.5202000268876554311958670950843952596187591552734375f,0.408700032281875014206207197275944054126739501953125f,0.07940005955696094464002499080379493534564971923828125f,-0.816300058615209156442915627849288284778594970703125f,0.44719999818801792201838907203637063503265380859375f,-0.1657999816358088907009005197323858737945556640625f,-0.24200001969188489336914926752797327935695648193359375f,1.05690000233650227556836398434825241565704345703125f,-0.3199999893270428774627589518786408007144927978515625f,-0.4198000444889071758325371774844825267791748046875f,0.00259996225833825089779338668449781835079193115234375f,0.8757000385999678115922506549395620822906494140625f,0.34870002153441337355843643308617174625396728515625f,0.15589998371750102279520433512516319751739501953125f,-0.7387000128917395880279173070448450744152069091796875f,-0.38949999905377608033774095019907690584659576416015625f,0.159800010793656355190250906161963939666748046875f,-1.1441000681757937940119518316350877285003662109375f,-0.12469998254179959218390649766661226749420166015625f,0.818800058987737333637824121979065239429473876953125f,0.5959999811053275919903171597979962825775146484375f,-0.376500082150102333145014199544675648212432861328125f,0.3075000283122069077990090590901672840118408203125f,1.112800039586425260296209671651013195514678955078125f,-1.8010000459253792115532633033581078052520751953125f,-0.76140001591891059984362755130860023200511932373046875f,-0.24740005279183463482439719882677309215068817138671875f,-0.3078000231578943424182170929270796477794647216796875f,-1.645100028978287998171481376630254089832305908203125f,-1.745900029277801746019349593552760779857635498046875f,0.41440000054836279019809808232821524143218994140625f,-0.6883999595403675897387074655853211879730224609375f,-1.535099993774295068504898154060356318950653076171875f,0.4317999807715420246267967741005122661590576171875f,1.055800010916591258336438841070048511028289794921875f,0.081299968612194017936189993633888661861419677734375f,-0.56340006153881605488464856534847058355808258056640625f,0.6305000685602430987586330957128666341304779052734375f,-0.81419999839663514951126899177324958145618438720703125f,0.5810999989822505096270788271795026957988739013671875f,0.0095999836266040805554666803800500929355621337890625f,0.46900003319978811333612611633725464344024658203125f,-1.2038000345312063199543217706377618014812469482421875f,0.82360004901290029266647252370603382587432861328125f,0.409500022970140375999648085780791006982326507568359375f,-0.734900050333143095571131198084913194179534912109375f,-0.982100032837688985409840825013816356658935546875f,0.991800023575127198682821472175419330596923828125f,0.49780001293197295719750172793283127248287200927734375f,-0.074199999476969313860763577395118772983551025390625f,-0.399300009307265002522768782000639475882053375244140625f,1.196700018624961447955001858645118772983551025390625f,0.37760001687109434609368463497958146035671234130859375f,0.11790000249668997067686859736568294465541839599609375f,1.75840005967617063475927352556027472019195556640625f,-0.2854000403642660277370168842026032507419586181640625f,-1.10420005762875117483190479106269776821136474609375f,-0.655000002011655535483214407577179372310638427734375f,2.00930006406903327587087915162555873394012451171875f,1.4003999879494311020522445687674917280673980712890625f,0.79249996036291048273625392539543099701404571533203125f,-0.46840004190653450333314822273678146302700042724609375f,0.555900005771219785088987919152714312076568603515625f,1.239799970486014313308942291769199073314666748046875f,-1.2915000051297249594739469102933071553707122802734375f,0.68160006350055335477833295954042114317417144775390625f,-0.4215999827176328818012507326784543693065643310546875f,1.2560999964117993688006436059367842972278594970703125f,1.0582999971330158128779430626309476792812347412109375f,-0.6984000389635554029155173338949680328369140625f,0.2931999909907585522006456812960095703601837158203125f,-0.7526999900072810500972764202742837369441986083984375f,0.2787999801605938188231448293663561344146728515625f,0.5179999616816644003591818545828573405742645263671875f,1.1215999748200180530233183162636123597621917724609375f,0.025899994558096051111562019286793656647205352783203125f,-0.205700012709200186211688787807361222803592681884765625f,-0.17490000283569173689102171920239925384521484375f,0.2021999942392109783639853048953227698802947998046875f,-0.96659999516010319808856365852989256381988525390625f,1.0905000050440427639131257819826714694499969482421875f,-0.77460000009238694218538512359373271465301513671875f,0.35559999317452295475305845684488303959369659423828125f,-0.4794999805018307359461005034972913563251495361328125f,0.16719999122172601868641095279599539935588836669921875f,-0.5180999829486012675516803938080556690692901611328125f,-0.2494999919570982382932555765364668332040309906005859375f,-0.3868000289767976607180344217340461909770965576171875f,-1.299400013270229248263376575778238475322723388671875f,0.79330001188516607868450591922737658023834228515625f,0.502200035031140412655759064364247024059295654296875f,0.610300044429302079862509344820864498615264892578125f,0.60310005130767851966311354772187769412994384765625f,-0.259799984996021748884231783449649810791015625f,0.8812999947637318864934741213801316916942596435546875f,-0.21079999564290030633628703071735799312591552734375f,0.3478000081822276712273378507234156131744384765625f,-0.4226999840840697242327905769343487918376922607421875f,-0.1635999859064820594056754998746328055858612060546875f,-0.891300022405386638268964816234074532985687255859375f,0.52449999730288965338331763632595539093017578125f,-0.74419999616146004228767196764238178730010986328125f,-0.0381999910652630614293912003631703555583953857421875f,-1.1769000160455700410011559142731130123138427734375f,0.123700013878941916800613398663699626922607421875f,-0.43589996401071606779709100010222755372524261474609375f,0.08020004000067670890672388850362040102481842041015625f,-0.08300001450628047283686328228213824331760406494140625f,0.53359999168068161790046133319265209138393402099609375f,0.040999993845820537075042011565528810024261474609375f,0.022899998715519842562571284361183643341064453125f,0.137800032433867070125188547535799443721771240234375f,-0.403499971121549894093050170340575277805328369140625f,1.427399952208995959068715819739736616611480712890625f,-0.60519998495951288219885100261308252811431884765625f,0.2168000056192276314703804018790833652019500732421875f,-0.314299994257092263350017447010031901299953460693359375f,-1.49479998577833139705717258038930594921112060546875f,-1.1027999789759512783149375536595471203327178955078125f,0.746399965590238689827629059436731040477752685546875f,-1.165100000833720006454541362472809851169586181640625f,-1.70770000319480885764278355054557323455810546875f,1.0593000216938552870971079755690880119800567626953125f,0.4371000076174738069312297739088535308837890625f,-0.906900019826740066974934961763210594654083251953125f,-1.430800032150745693115823087282478809356689453125f,-1.076900050355494631304509312030859291553497314453125f,-1.310100007371603592076780842035077512264251708984375f,-0.822000004306435716472378771868534386157989501953125f,-1.46920000092983205064456342370249330997467041015625f,0.212100008511542537092964266776107251644134521484375f,-0.541299989101290623949580549378879368305206298828125f,1.4560000602304938865927397273480892181396484375f,0.498900035402178421151120346621610224246978759765625f,-0.5240999495625491277905894094146788120269775390625f,-1.64370001738071547237041158950887620449066162109375f,-0.2611000759840020890578671242110431194305419921875f,0.96330001069307424899079705937765538692474365234375f,-1.37460000515878189020213540061376988887786865234375f,1.555799960699677964015563702560029923915863037109375f,0.69879997293353091070144955665455199778079986572265625f,0.02950000810623176761282593361102044582366943359375f,0.23730000760853275920680971466936171054840087890625f,-0.0849999897181987762451171875f,-0.648899955829977503896088819601573050022125244140625f,-0.8999000785708428651332724257372319698333740234375f,0.79659999754428856277854720246978104114532470703125f,1.2677001045227047626440253225155174732208251953125f,-0.78199997532367770958217079169116914272308349609375f,-2.24180000699758519289161995402537286281585693359375f,-0.054199974685162022325357611407525837421417236328125f,1.0294000142574315503907200763933360576629638671875f,0.7957000245556233863908346393145620822906494140625f,0.09739997158050517356286945869214832782745361328125f,0.69239998581111394937437353291898034512996673583984375f,-0.5192999795794488449018899700604379177093505859375f,-0.390400041425973398645510314963757991790771484375f,0.090200010496378268953776569105684757232666015625f,0.4066000092417001443578783437260426580905914306640625f,-1.303700028631090912512036084081046283245086669921875f,-1.0360000044070181157707111196941696107387542724609375f,-0.243300054837763735093858485925011336803436279296875f,0.4287000003933911784770316444337368011474609375f,-1.004399983561039544355253383400849997997283935546875f,0.832099992418289247098073246888816356658935546875f,-0.83170000301599589231926756838220171630382537841796875f,0.25160000290870687678079775650985538959503173828125f,0.67099993990361728180005229660309851169586181640625f,1.01399991831183466928223424474708735942840576171875f,-2.03669999839663518059751368127763271331787109375f,1.1771000612616528968601414817385375499725341796875f,-0.048800032836199047636682735173963010311126708984375f,0.82189997268319103795874980278313159942626953125f,-0.921999997973441676180073045543394982814788818359375f,0.2982000254496937063919403954059816896915435791015625f,0.84460001188293121199279767097323201596736907958984375f,-0.95550003436207819884629088846850208938121795654296875f,-1.0934000024743377732505678068264387547969818115234375f,0.60669998662471746886382106822566129267215728759765625f,-0.794999988898634768474948941729962825775146484375f,1.06209999673962585120534640736877918243408203125f,-1.467700001928209818657933283247984945774078369140625f,0.5677000027477741017634116360568441450595855712890625f,1.15569998819678954049550156923942267894744873046875f,-0.69319999631792283967257617405266501009464263916015625f,-0.30629999264031593231294436918688006699085235595703125f,0.2584000289052730270356050823465920984745025634765625f,-1.74960007504522874199892612523399293422698974609375f,0.2089000176697968225170143341529183089733123779296875f,-0.21840001039057976850443765215459279716014862060546875f,0.4636000195644800481176162065821699798107147216796875f,0.71910001512765830966600333340466022491455078125f,0.2805000006854536653833065429353155195713043212890625f,0.478000033020972381336832768283784389495849609375f,1.4749999921768901511853755437186919152736663818359375f,-0.16560001502335008893851409084163606166839599609375f,0.155299998086691370957623803406022489070892333984375f,-0.18130000038892102320886579036596231162548065185546875f,0.091100007587672227060693330713547766208648681640625f,-0.68439999358206982105201632293756119906902313232421875f,1.238399957491456149227815330959856510162353515625f,0.2600999815367164291757262617466039955615997314453125f,1.017499954868107625571838070754893124103546142578125f,0.4279999868273733909518341533839702606201171875f,-0.42069997839927708582763443700969219207763671875f,-0.468099972331524671886882060789503157138824462890625f,-1.2301999435901649349034414626657962799072265625f,2.2020999541223051920724174124188721179962158203125f,0.220699974077940286321108942502178251743316650390625f,-1.911999956585467064229533207253552973270416259765625f,0.7914999952018264561814930857508443295955657958984375f,-0.3348000104770065821213620438356883823871612548828125f,0.16250000439584244560364822973497211933135986328125f,0.7545000492334372754754667766974307596683502197265625f,0.56039999487847091952374967149808071553707122802734375f,-1.2942000282734635874248851905576884746551513671875f,0.749700033764541640124434707104228436946868896484375f,1.21150004582852144352500545210205018520355224609375f,-0.1376000167220829961678418840165250003337860107421875f,-0.79430003702342555360615961035364307463169097900390625f,-0.2098999936342236249942061476758681237697601318359375f,1.3184000279739500616216218986664898693561553955078125f,0.4667000023081897541743501278688199818134307861328125f,0.73680003203153621171850318205542862415313720703125f,-0.8062000194296239374835977287148125469684600830078125f,0.50890000604689067387198520009405910968780517578125f,-0.4890000387877233567479606790584512054920196533203125f,0.62160005018264097298441583916428498923778533935546875f,0.023499984458089262506774730354663915932178497314453125f,0.023099967861174519612177391536533832550048828125f,-0.004100011551380067231775683467276394367218017578125f,-0.1958999902367590806306907325051724910736083984375f,-0.223200023847818673772280817502178251743316650390625f,-0.8036000509500507149596160161308944225311279296875f,0.368199987041951004584916518069803714752197265625f,1.21230000127553960709292368846945464611053466796875f,-1.549999982267618037212741910479962825775146484375f,0.5454000333607191830509464125498197972774505615234375f,-0.151299985450506380146862284163944423198699951171875f,-0.405000038705766396418539443402551114559173583984375f,-0.969599980124830285177495170501060783863067626953125f,0.21309997903704580579642424709163606166839599609375f,0.983600041711330419502701261080801486968994140625f,-1.163300020527839873096809242269955575466156005859375f,-0.91299999472498871710968160186894237995147705078125f,0.2611000023722649387991623370908200740814208984375f,-0.2251999860584736889990153940743766725063323974609375f,-0.0597000500440605375018776612705551087856292724609375f,0.483000029295683308561137891956605017185211181640625f,1.1906000787630686144069613874307833611965179443359375f,-0.612900047144294024548116794903762638568878173828125f,-1.18390003290623457843366850283928215503692626953125f,0.8490000371485952879169190055108629167079925537109375f,0.158899985446035840741529909792006947100162506103515625f,0.50400000999122818168274307026877067983150482177734375f,-0.9639000283986332018315579261980019509792327880859375f,0.4713000094041228660302067510201595723628997802734375f,0.25030005147010137012131281153415329754352569580078125f,-1.2691000543177128889738014549948275089263916015625f,-0.23369996552467231509808698319830000400543212890625f,0.7681000410631302344199866638518869876861572265625f,-0.8522000709056865019874749123118817806243896484375f,-0.4252000253230339144039362508920021355152130126953125f,0.3209000268936161859301137155853211879730224609375f,-0.77640003331601636915593189769424498081207275390625f,-0.03310000295341009035610113642178475856781005859375f,0.534600040586293179245558349066413938999176025390625f,0.63870004736930219291224375410820357501506805419921875f,-0.41540001980066332265550954616628587245941162109375f,0.87330002887248969756228689220733940601348876953125f,-1.227799975660443632108353995135985314846038818359375f,-1.92170000718831968100630547269247472286224365234375f,-0.41400000131130099134679767303168773651123046875f,0.62490001822113949714321279316209256649017333984375f,2.222299956274031984548855689354240894317626953125f,-0.81700004670023940178680277313105762004852294921875f,-0.836900010383129000501867267303168773651123046875f,-0.237800001215934753151515224089962430298328399658203125f,0.870700011777877147522985978866927325725555419921875f,0.92430001984983700680231777369044721126556396484375f,-0.716100034186243572520425004768185317516326904296875f,1.189400015598535365057841772795654833316802978515625f,-0.1447999968051909558397483124281279742717742919921875f,-1.26450006115436597298185006366111338138580322265625f,0.33450001555681219134186221708660013973712921142578125f,1.3729000377938154198176334830350242555141448974609375f,-0.514699994239210756319380379864014685153961181640625f,0.2609999609887598825963550552842207252979278564453125f,-0.30529997443109790911108802902163006365299224853515625f,1.2060000225678084717628735234029591083526611328125f,-0.4207000169560313196370771038345992565155029296875f,-1.0794000180758533957714462303556501865386962890625f,0.6942000139310950945770173348137177526950836181640625f,-1.3661000145725907817251254527946002781391143798828125f,-0.827600014933943750605749301030300557613372802734375f,0.77980004255175661143084653303958475589752197265625f,-0.128500003419816710703571516205556690692901611328125f,0.171399984961748241829582184436731040477752685546875f,-0.8945000208467248636878821343998424708843231201171875f,-0.5571000054568049364434045855887234210968017578125f,-0.28029996400028422964822993890265934169292449951171875f,-0.50700006245821704897736026396160013973712921142578125f,-0.9565999887526033251816670599509961903095245361328125f,0.336200026805698382048603889415971934795379638671875f,-0.8719999946951861868882360795396380126476287841796875f,-0.08609996359199245663518240689882077276706695556640625f,-0.17039996034502991051340359263122081756591796875f,0.162600004875659731595760604250244796276092529296875f,-0.0291999963104725424045682302676141262054443359375f,0.4526000039070847691391463740728795528411865234375f,-0.54979999956190539478484424762427806854248046875f,-1.4054000022172925543628707600873894989490509033203125f,0.140900024062394546575660569942556321620941162109375f,0.345099954845011769322127292980439960956573486328125f,1.880199948880076465940192065318115055561065673828125f,-1.11019999648928635593847502605058252811431884765625f,-0.819199949781597158704471439705230295658111572265625f,-0.2277999594740569566653221045271493494510650634765625f,-0.2523000097319483270297268973081372678279876708984375f,-0.11009997317343922507149045486585237085819244384765625f,-0.8412000180512666513976682836073450744152069091796875f,0.4606000086531043535842400160618126392364501953125f,-0.6789000126406550261748407137929461896419525146484375f,0.255500004187226126273202453376143239438533782958984375f,-0.406199997077881957974199167438200674951076507568359375f,-0.0394999960362911461952961644783499650657176971435546875f,0.066200001883506665034673233094508759677410125732421875f,0.94930001269727959378741388718481175601482391357421875f,0.2640000033974645443635154151706956326961517333984375f,-1.36480002545267353752933559007942676544189453125f,0.9561000211477279631111514390795491635799407958984375f,-0.31720000146627480575034496723674237728118896484375f,0.94709999446719894677926276926882565021514892578125f,-0.4545000084787609306857802948798052966594696044921875f,1.1132999935567375882072838066960684955120086669921875f,-0.0582000050723550099718295314232818782329559326171875f,-0.032500017508864498250176211513462476432323455810546875f,0.3796999966233964851625160008552484214305877685546875f,-0.39990000242590861834202087266021408140659332275390625f,0.1824999877065416331589631226961500942707061767578125f,-0.903400014190375966194324064417742192745208740234375f,1.24790003052949938222582204616628587245941162109375f,-0.748900005003809265957670504576526582241058349609375f,0.640700027424097040551487225457094609737396240234375f,-1.090099984908103891712016775272786617279052734375f,0.5474000263795251175480416350183077156543731689453125f,0.2727000011831519632465870017767883837223052978515625f,0.6066999818935989718937662473763339221477508544921875f,0.0593999827042226424822501940070651471614837646484375f,0.67089997351020580484970423640334047377109527587890625f,-0.582699972014129485842204303480684757232666015625f,0.247999994352459562918511437601409852504730224609375f,0.24260002350211085087039464269764721393585205078125f,0.12429997913241397355932349455542862415313720703125f,1.0971000012844793758404193795286118984222412109375f,-0.7192999986156820568794501014053821563720703125f,0.397499989271163922666119105997495353221893310546875f,0.2233999700829383261435623353463597595691680908203125f,0.10350002230703780892184795447974465787410736083984375f,0.2244999991282821127924762549810111522674560546875f,0.38629999878704524007844156585633754730224609375f,0.0280000635311015155792802033829502761363983154296875f,-0.803600035974383342107785210828296840190887451171875f,-0.9783000665724281841306719798012636601924896240234375f,0.859500042378902318063182974583469331264495849609375f,1.9365000135302541384163532711681909859180450439453125f,0.482100057983398944116970596951432526111602783203125f,0.8135999923139809464345262313145212829113006591796875f,0.10060001185685407687486758732120506465435028076171875f,1.28060002746582046739831639570184051990509033203125f,-1.4798999945655457732840432072407566010951995849609375f,0.657300016623735050558252623886801302433013916015625f,0.09060004344731620307129560387693345546722412109375f,-0.1823000352501882304068203666247427463531494140625f,-0.70580008072853173128891057785949669778347015380859375f,1.1369000753521927293832050054334104061126708984375f,1.3154000308275224373488754281424917280673980712890625f,-1.0538000076532352267122405464760959148406982421875f,0.447100023636220811340535874478518962860107421875f,0.738899978144467173279963390086777508258819580078125f,0.44159999452680320786157608381472527980804443359375f,1.9223999796271329376651237907935865223407745361328125f,-1.8922999400317674911775611690245568752288818359375f,-1.0296999814987177757075187400914728641510009765625f,-0.8353999262213704923851764760911464691162109375f,1.82820000894665657398263647337444126605987548828125f,-1.0211000142931947465285702492110431194305419921875f,-0.00369997267723132239325423142872750759124755859375f,-0.45859994246959701058585778810083866119384765625f,0.33230001707077061467998646548949182033538818359375f,0.187100011193752635563214425928890705108642578125f,-1.2904000217378139847568263576249592006206512451171875f,0.676099993878602756325335576548241078853607177734375f,-0.600899970231949964727391488850116729736328125f,0.0142999747365717411184959928505122661590576171875f,0.747800001960992499761005092295818030834197998046875f,-0.367999990701675105952972444356419146060943603515625f,-0.598900009362399732282256081816740334033966064453125f,0.10790000512301956003824443541816435754299163818359375f,-0.1660999990567566531041165944770909845829010009765625f,0.3372999873429538553182283067144453525543212890625f,0.375700003749877442515980874304659664630889892578125f,-1.0230999863386152082256330686504952609539031982421875f,0.30440005419254223539837767020799219608306884765625f,0.128600006172060243869736950728110969066619873046875f,1.6312999659299851629157274146564304828643798828125f,-0.0731000232934950755492309326655231416225433349609375f,-0.63959995829463078820253940648399293422698974609375f,-0.847599976861477699685565312393009662628173828125f,-0.28489996711015663777288864366710186004638671875f,-0.212499998435377701611059819697402417659759521484375f,-1.097600006961821694773107083165086805820465087890625f,-0.852700001928209605495112555217929184436798095703125f,0.80760002558827448382317015784792602062225341796875f,0.02559998512789596336602926385239697992801666259765625f,0.09150001256167883401104745644261129200458526611328125f,-0.07280001002624636452509321316028945147991180419921875f,0.9910999898180363754107702334295026957988739013671875f,0.085099976302682733830806682817637920379638671875f,-1.284999955892562439885296043939888477325439453125f,0.6750000051781537280248812749050557613372802734375f,0.457999976694583921243975055404007434844970703125f,-0.42749998310580827887150690003181807696819305419921875f,-2.03729999386221205526226185611449182033538818359375f,-0.96350000002980229485416430179611779749393463134765625f,0.51039997334629350067558561931946314871311187744140625f,0.908800019425154204100181232206523418426513671875f,-0.09740003591626866796815420457278378307819366455078125f,0.2520000345557933396634098244248889386653900146484375f,0.558599972011147638539796389522962272167205810546875f,0.789700038048625430064930696971714496612548828125f,0.5129000139147041892329070833511650562286376953125f,0.1230999997496606823688125587068498134613037109375f,-1.322599986025690288471423627925105392932891845703125f,-1.383600010567903382252552546560764312744140625f,-1.443499974139033970033096920815296471118927001953125f,0.67370005165338486818882302031852304935455322265625f,1.058200101967156303572892284137196838855743408203125f,0.88510000852644454827355957604595459997653961181640625f,0.21699995863437637755311016007908619940280914306640625f,0.95509999332726025489392895906348712742328643798828125f,-0.21700000717490919299734741798602044582366943359375f,0.2268000211909415586575278211967088282108306884765625f,-0.904000012338162406422270578332245349884033203125f,0.596199998679756593134015929535962641239166259765625f,1.588699984598159975490716533386148512363433837890625f,-0.30600000621378409260131547853234224021434783935546875f,-0.4234999993592494238470180789590813219547271728515625f,-0.01469999371767016782541759312152862548828125f,-1.2653999979332091196937426502699963748455047607421875f,-0.900000027380883249605858509312383830547332763671875f,0.83950000613182762521091717644594609737396240234375f,0.3394000180572269709244892510469071567058563232421875f,-0.70890001755803788707765988874598406255245208740234375f,-1.330999991610645860617978541995398700237274169921875f,-0.203200017067790117408776495722122490406036376953125f,-0.6878000047735870481346864835359156131744384765625f,-0.0465000350996855882357294831308536231517791748046875f,-1.047400047408789891534297566977329552173614501953125f,1.09220003454685166133231177809648215770721435546875f,1.026400004761665929464697910589165985584259033203125f,-0.39649998641014150280170724727213382720947265625f,2.15099999690055820877887526876293122768402099609375f,0.111899977339804390652489018975757062435150146484375f,-1.910999984681605923242386779747903347015380859375f,-1.8226999796882263904507226470741443336009979248046875f,0.572700004051625466416908238898031413555145263671875f,-0.0639000213578342357578776500304229557514190673828125f,1.086900014741718489830191174405626952648162841796875f,-0.7578000703945761973301387115498073399066925048828125f,0.544800017890334498105175953241996467113494873046875f,1.3841000180333846714120227261446416378021240234375f,-1.83800003510713594323533470742404460906982421875f,-0.303599981585145339835207778378389775753021240234375f,0.007700019139051494931891284068115055561065673828125f,1.0848999434918169537667154145310632884502410888671875f,0.4036999705165629226399914841749705374240875244140625f,0.6162000022932885290316562532098032534122467041015625f,-0.076599983909726443442878007772378623485565185546875f,-0.61319998853206592048792344940011389553546905517578125f,-0.0112999935716388277828770014821202494204044342041015625f,0.45059999654591109674584004096686840057373046875f,0.4262999999791385885572481129202060401439666748046875f,-0.0093000192165373363195612910203635692596435546875f,-0.372800048825145058373209394630976021289825439453125f,-0.6049999317526815190149136469699442386627197265625f,-1.198100004592537626280090989894233644008636474609375f,0.8686000259161001935126478201709687709808349609375f,0.05440004419013877434707637803512625396251678466796875f,0.037399973368644534588156602694652974605560302734375f,0.20749999800696949936451574103557504713535308837890625f,-0.0779999571740628283578189439140260219573974609375f,1.3716000329717996653045020138961263000965118408203125f,-0.62360001399517006603900881600566208362579345703125f,1.176200050982833200663435491151176393032073974609375f,-0.81570001267194758298728629597462713718414306640625f,0.1554000092767180429831341825774870812892913818359375f,0.442000007510184644843320711515843868255615234375f,0.125900003610551092236846670857630670070648193359375f,0.5958999907582995803068115492351353168487548828125f,-1.0339000283241279731782924500294029712677001953125f,0.3140000199377535050615506406757049262523651123046875f,-0.1110999967098234009199586580507457256317138671875f,1.1421000827193268367665268669952638447284698486328125f,-0.2973000123023989971215996774844825267791748046875f,0.98050007839500974426982793374918401241302490234375f,0.1828999768853183383043869980610907077789306640625f,-0.359500006392597715176862038788385689258575439453125f,-1.2278000521779066644967315369285643100738525390625f,-1.955800010171532488101320268469862639904022216796875f,0.3764999681562184452587871419382281601428985595703125f,0.29679998255968076392719012801535427570343017578125f,0.1768999883294102293262994862743653357028961181640625f,-0.345599990510940369148329409654252231121063232421875f,-0.6410999938786028362613933495595119893550872802734375f,-0.7884000026047228715242454200051724910736083984375f,-0.4116000171393157103238991112448275089263916015625f,-1.02899999564886091008020230219699442386627197265625f,-0.489399986168742362480088559095747768878936767578125f,0.42150002485513748951007073628716170787811279296875f,-0.261600020119547860275588391232304275035858154296875f,2.14740000764131622190689085982739925384521484375f,0.217999999959021639739376041688956320285797119140625f,0.4966000023126611040424904786050319671630859375f,-0.367499964460729966475582841667346656322479248046875f,1.30730002429783365158755259471945464611053466796875f,-0.345699962101131408331866623484529554843902587890625f,0.0411999630659825033518472992000170052051544189453125f,-0.6739999628663060349254010361619293689727783203125f,0.4482000025391579267619590609683655202388763427734375f,0.578400021678209252939950602012686431407928466796875f,0.212299971845746338061644564731977880001068115234375f,0.08760000409334922633064479668973945081233978271484375f,0.449900007715820837717046742909587919712066650390625f,-0.38380000333190000816330211819149553775787353515625f,0.2938000284403565753876819144352339208126068115234375f,-1.067900046622753507108427584171295166015625f,-0.336299999904631885527805934543721377849578857421875f,-2.39280003340244373788436860195361077785491943359375f,-0.1375999612152571938139544727164320647716522216796875f,-0.1682999921411276744720453280024230480194091796875f,0.9065000191926955164234414041857235133647918701171875f,-1.020200024652481385345481612603180110454559326171875f,0.13730000574588807626952302598510868847370147705078125f,-0.02600003035366516002113712602294981479644775390625f,-0.415800050106644281111556438190746121108531951904296875f,-0.5040000113695859429441270549432374536991119384765625f,0.4559999980926505713796359486877918243408203125f,-0.664800004628301355324992982787080109119415283203125f,1.733799990516901257109338985173963010311126708984375f,-0.4643000285297631535286200232803821563720703125f,-0.0547000109285118440993755939416587352752685546875f,-0.8056999810069800371792325677233748137950897216796875f,-0.1992999803245065759682574935141019523143768310546875f,-0.77250000551342956622846713798935525119304656982421875f,0.0687000220745799250465779550722800195217132568359375f,-0.474499990299344143185322764111333526670932769775390625f,0.94739998722672513764564428129233419895172119140625f,-0.108699997506290502169434830648242495954036712646484375f,-0.21680003218054810787407404859550297260284423828125f,0.6320000028908250300219151540659368038177490234375f,-1.37320006154775597906336770392954349517822265625f,0.08709997334480246156118710132432170212268829345703125f,1.61990001755058710131152110989205539226531982421875f,-0.0904000015653669830584249211824499070644378662109375f,-1.2132999836847184749188954810961149632930755615234375f,-1.3168000009253619797533474411466158926486968994140625f,0.605100003683567866374914956395514309406280517578125f,-0.2497999674499034039598654999281279742717742919921875f,1.1260000232942406483971353736706078052520751953125f,-1.11980001006722407197457869187928736209869384765625f,0.91210005820691641975628272120957262814044952392578125f,0.0768000394821175547122038551606237888336181640625f,-0.07300003018975331769979675300419330596923828125f,0.177400012409687946046688011847436428070068359375f,-0.797999995283781959898306013201363384723663330078125f,-2.641300015960633462697160211973823606967926025390625f,-0.57159997135549789337716219961293973028659820556640625f,1.731900025209783589019707505940459668636322021484375f,0.071600001046060679499305479112081229686737060546875f,-1.70880002315044432492641135468147695064544677734375f,0.301300058875978660211103488109074532985687255859375f,-0.51590000402033331372564362027333118021488189697265625f,-1.18709997364282582310579527984373271465301513671875f,-1.5274999801814548305856078513897955417633056640625f,-0.5376999617695812361262142076157033443450927734375f,-0.425899955964088139381829023477621376514434814453125f,2.3826999531269077436945735826157033443450927734375f,-0.1156000500410794551431337140456889756023883819580078125f,-1.0246999690562486051703672274015843868255615234375f,-0.3810999796479943579896598748746328055858612060546875f,0.793999989986418963638925561099313199520111083984375f,-0.3721999928236012777915675542317330837249755859375f,0.332300008092820231553332632756792008876800537109375f,0.22949998592212772496878869787906296551227569580078125f,1.06589996915161666635185611085034906864166259765625f,-0.33710000063255396440808908664621412754058837890625f,0.1103000206992026388519434476620517671108245849609375f,0.1151000099048020697267702416866086423397064208984375f,0.904199950806051422347309198812581598758697509765625f,-1.108700008179247742390316489036194980144500732421875f,0.9793000600270931332858026507892645895481109619140625f,-0.130000003390014218229708831131574697792530059814453125f,0.04509998714327867475049060885794460773468017578125f,-0.54279999885037522044939350962522439658641815185546875f,1.07070001161023942159999933210201561450958251953125f,0.4211000169754033350955069181509315967559814453125f,0.2034999865815045883010725447093136608600616455078125f,0.134499981842935767417657189071178436279296875f,0.14609997484237091658343388189678080379962921142578125f,-1.02559998969137655677741349791176617145538330078125f,-0.107700013122708160295815105200745165348052978515625f,0.0697000211358069776679258211515843868255615234375f,-0.49990004226788986585461316280998289585113525390625f,-0.8821000247351828971176246341201476752758026123046875f,0.1292999976098536674129491075291298329830169677734375f,1.37820002906322525149107605102472007274627685546875f,1.4970000132657592661900025632348842918872833251953125f,-0.4138999462187291644710285254404880106449127197265625f,0.42109999521970664648762294746120460331439971923828125f,-0.2203999888807537110846368477723444811999797821044921875f,0.514999951049686099935343008837662637233734130859375f,0.6125999537646775738863880178541876375675201416015625f,0.30830000283271108418858830191311426460742950439453125f,-0.8097999853685495796895565945305861532688140869140625f,1.38780000101104317167255430831573903560638427734375f,0.81069999680221027649196230413508601486682891845703125f,-0.64450001689791758252567888121120631694793701171875f,0.2270000251680610592330822328221984207630157470703125f,-0.73610003936439716287765122615383006632328033447265625f,-0.9117000366367402630629612758639268577098846435546875f,0.65980003489628391566412801694241352379322052001953125f,-0.04090001135915521945918271740083582699298858642578125f,-1.148000016033649206548261645366437733173370361328125f,-0.37140002171173713829688267651363275945186614990234375f,-1.16020001720190091276663224562071263790130615234375f,-1.8271000275105235832739936085999943315982818603515625f,0.750799993705749901806711932295002043247222900390625f,0.05060005205273665485066203473252244293689727783203125f,-0.8975000228174028205074819197761826217174530029296875f,0.39650000630319104377718986143008805811405181884765625f,0.781400023406744193010808885446749627590179443359375f,0.8494999950677151190348013187758624553680419921875f,-0.1122000171124941658717943937517702579498291015625f,-0.4541000358402731418294706600136123597621917724609375f,-1.2478000138446685429727267546695657074451446533203125f,-0.21370001987665876441724321921356022357940673828125f,-0.96779999611228684219810247668647207319736480712890625f,-0.3961999721556905829089600956649519503116607666015625f,0.852800010901688665398978628218173980712890625f,-1.0287999724864957951098176636151038110256195068359375f,1.621200075569749987636214427766390144824981689453125f,-0.2534999797642225605187604742241092026233673095703125f,1.0070000135451551148690896297921426594257354736328125f,0.0261999540507797945565471309237182140350341796875f,-0.04080000361502189587525890601682476699352264404296875f,-0.64900002470612516702885841368697583675384521484375f,-0.1699999912828207282444736847537569701671600341796875f,-0.05090004276335235999795258976519107818603515625f,-1.0061000456228856858587050737696699798107147216796875f,-1.215100029090046884761022738530300557613372802734375f,0.7749999924004076223837955694762058556079864501953125f,-1.437600011730193472203609417192637920379638671875f,-0.6617000060707329822662359219975769519805908203125f,0.7381000060081479574591867276467382907867431640625f,1.628400048798322785614800523035228252410888671875f,0.902900017565489321214045048691332340240478515625f,0.818000066176058027167528052814304828643798828125f,-0.335200066450239031468072425923310220241546630859375f,-0.71820001453459225704278878765762783586978912353515625f,1.0465000100284811423989594914019107818603515625f,0.697100025685131630126534219016321003437042236328125f,0.0227000103101133721139603949268348515033721923828125f,1.061200080561639058629452847526408731937408447265625f,-1.7665000194907189978721362422220408916473388671875f,0.815100038254261249903720454312860965728759765625f,-0.828600033366679866730919457040727138519287109375f,-1.466500070527196886160936628584749996662139892578125f,0.5422000287353998526640452837455086410045623779296875f,1.1259000473827127652981516803265549242496490478515625f,1.612100017005205554454505545436404645442962646484375f,0.7948000122651459609102175818406976759433746337890625f,0.67670001668781054604551172815263271331787109375f,1.22820004660934234408387055736966431140899658203125f,-0.6404000163733962747159011996700428426265716552734375f,0.068499998390674221582230529747903347015380859375f,-0.1300999760419132567079714135616086423397064208984375f,0.0669999796450131412939299480058252811431884765625f,-1.0066000398635870372032741215662099421024322509765625f,-1.404600039356946883373211676371283829212188720703125f,-0.95810000243186976831566425971686840057373046875f,0.92750002510845686032325829728506505489349365234375f,-0.5053999611645940692739031874225474894046783447265625f,0.56739998551309067664760732441209256649017333984375f,0.288599978716671312639618918183259665966033935546875f,-0.0772000057131052574987961634178645908832550048828125f,-0.05910004560053305766587072866968810558319091796875f,0.4343000472307212422862221501418389379978179931640625f,0.58830000428557394087647480773739516735076904296875f,0.27470000303089658899580172146670520305633544921875f,0.8447999979972831141594724613241851329803466796875f,-1.17289997457563810456804276327602565288543701171875f,-0.06890007541179699046551831997931003570556640625f,0.303599977971612589033867379839648492634296417236328125f,-0.4731999676704405910498962839483283460140228271484375f,-0.58970001290291573781132683507166802883148193359375f,-0.1931999922201039776581410478684119880199432373046875f,-0.4448999642044293434395285657956264913082122802734375f,-0.1109000119552019381075069759390316903591156005859375f,0.6602000005125991943799590444541536271572113037109375f,-1.254900003319978196714146179147064685821533203125f,-1.753600060039758989915981146623380482196807861328125f,1.47920001180767979320762606221251189708709716796875f,0.3361999848961827463966756113222800195217132568359375f,-0.13280001052916023951411261805333197116851806640625f,-0.7656000419199464612773908811504952609539031982421875f,0.28580001568496271602271008305251598358154296875f,0.4995000482648619399839162724674679338932037353515625f,1.1936000308334826502942860315670259296894073486328125f,0.9662000125378364945305520450347103178501129150390625f,0.09920002659708193615273330578929744660854339599609375f,-0.3219000237941747055714358793920837342739105224609375f,0.0952000593721873400454569491557776927947998046875f,0.477900049304962326601753375143744051456451416015625f,1.714900042062998064551493371254764497280120849609375f,1.2547999902486797196843326673842966556549072265625f,-0.76429999168664231046221857468481175601482391357421875f,0.405600019698590108507829654627130366861820220947265625f,-0.31370000449120938323943619252531789243221282958984375f,-0.50730003107786192373396261245943605899810791015625f,0.32170001314878415854536797269247472286224365234375f,0.461600008794665062072937189441290684044361114501953125f,-0.277799967092275057467531951260752975940704345703125f,0.3784000368028881400306318028015084564685821533203125f,0.455300003898143845759705072850920259952545166015625f,-0.326799980622529684382016057497821748256683349609375f,1.06349999213218637805766775272786617279052734375f,0.3901000372856859854664435260929167270660400390625f,-0.51609999890774493902512176646268926560878753662109375f,0.52610003725960918785631292848847806453704833984375f,-0.457400017494708155396665461012162268161773681640625f,-0.2502000145196916491840966045856475830078125f,0.04010002216100705396684134029783308506011962890625f,0.66010001314580402098641798147582449018955230712890625f,-0.856100021743773975657632036018185317516326904296875f,-0.784900010100006539914829772897064685821533203125f,1.1314999699443575309487641789019107818603515625f,-0.495100055688620699356761178933084011077880859375f,1.485000055432319499004734097979962825775146484375f,-1.267900053179264574509943486191332340240478515625f,-0.7291000256702309112455395734286867082118988037109375f,-0.5124000302165752085414851535460911691188812255859375f,0.12130000351816420944839336470977286808192729949951171875f,-0.26040000951886188573780600563623011112213134765625f,-1.660700051265955767831883349572308361530303955078125f,-0.4992000176191329074981695157475769519805908203125f,0.352299995017051781331929305451922118663787841796875f,0.31129998366236666385020726011134684085845947265625f,2.051900018951297255398458219133317470550537109375f,-0.022100020283460253267548978328704833984375f,0.853500015527009825433424339280463755130767822265625f,-0.22720004751086353422806496382690966129302978515625f,1.40619996660500845564456540159881114959716796875f,-0.00099998985975979781670730517362244427204132080078125f,1.3797999865978958222711980852182023227214813232421875f,-0.081599999853968707697049467242322862148284912109375f,1.2041999970555306020969510427676141262054443359375f,-0.02470000161528584925463292165659368038177490234375f,-0.6231000188231465841681711026467382907867431640625f,-1.121100003415346435531319002620875835418701171875f,-1.08420002864599229752684550476260483264923095703125f,0.72600003482401376242449941855738870799541473388671875f,-0.2217999956727023658231701119802892208099365234375f,-0.21019999961853008674239617903367616236209869384765625f,0.375899994036555273879685046267695724964141845703125f,-0.3688999885007733237074489807127974927425384521484375f,0.1575999890804291592161234802915714681148529052734375f,-1.022100066775084581394139604526571929454803466796875f,1.1450000494718555188455866300500929355621337890625f,-0.501200004416703759346773949800990521907806396484375f,1.1465999999284743893213089904747903347015380859375f,1.0843000339686874600175769955967552959918975830078125f,0.75439996865987790641838728333823382854461669921875f,0.2563000289618979632422224312904290854930877685546875f,0.15579996673464702183764529763720929622650146484375f,-0.6752000480845576912969363547745160758495330810546875f,0.516300001245736606136915725073777139186859130859375f,-0.6467000328928238506165371290990151464939117431640625f,-0.26690002308636973094024824604275636374950408935546875f,-0.869600003147124933633449472836218774318695068359375f,-0.160300004866719358886939517105929553508758544921875f,0.402600028270482912517991280765272676944732666015625f,1.132300015655160141392343575716949999332427978515625f,1.03010000576972959862587231327779591083526611328125f,0.198099949979782419262619441724382340908050537109375f,-0.6938000332832334748900393606163561344146728515625f,0.95500000387430261650223428659955970942974090576171875f,-0.20649999067559787757630829219124279916286468505859375f,1.18039995453357704491281765513122081756591796875f,0.8325999828219412535190713242627680301666259765625f,-0.13320000984072688510195803246460855007171630859375f,0.813500011056661254116306736250407993793487548828125f,0.794599968278408397281964425928890705108642578125f,-0.214099979737401024948439953732304275035858154296875f,1.84819998793602024989013443700969219207763671875f,-0.2920999898850917819714823053800500929355621337890625f,1.3190999993324279415674027404747903347015380859375f,0.3102000039398671304979870910756289958953857421875f,-0.3790000016093255652549487422220408916473388671875f,0.3111000108659267571198370205820538103580474853515625f,1.27820004038810708379969582892954349517822265625f,-0.4769999791979782077788740934920497238636016845703125f,0.03400000578165052189660855219699442386627197265625f,-0.1136999831080434120167410583235323429107666015625f,-1.16150000175833678639492063666693866252899169921875f,-0.68459999971985752154068904928863048553466796875f,0.367699943590164224360705702565610408782958984375f,-1.103600069576502473722712238668464124202728271484375f,1.893000031530856830386255751363933086395263671875f,-0.741200028407573707767141968361102044582366943359375f,1.3225000202655792236328125f,0.591300024789572109540358724188990890979766845703125f,-0.575900007298588700876962320762686431407928466796875f,-0.5737999844074253008585628776927478611469268798828125f,0.69369999927580305865149057353846728801727294921875f,-0.60200002092123039432181030861102044582366943359375f,-0.3697000010192397212449577637016773223876953125f,-0.4238000070571903421523529686965048313140869140625f,0.072299978029727984818464392446912825107574462890625f,-0.92150000384449892720795105560682713985443115234375f,0.155799986478686758317735439050011336803436279296875f,-0.16549999025464057211820545489899814128875732421875f,1.1200000052154066576548530065338127315044403076171875f,-0.774000026941299434923848821199499070644378662109375f,0.0076999999165536792133934795856475830078125f,0.4676000122517347534767395700328052043914794921875f,-0.340700006711482927812539855949580669403076171875f,0.4131999729603526017029935246682725846767425537109375f,0.5160999904513356995039430330507457256317138671875f,0.20170001523494729411822845577262341976165771484375f,1.31819997825026558047056823852472007274627685546875f,-0.5857000070095061960984139659558422863483428955078125f,-0.41240001553893090857627612422220408916473388671875f,-0.432500003054737902630222379229962825775146484375f,-0.1887000194668768671135694603435695171356201171875f,-0.13660001646876285263942918390966951847076416015625f,-0.01040000236630422847383670159615576267242431640625f,-0.930000029802322547567428046022541821002960205078125f,0.120200008261203805659533827565610408782958984375f,1.0842000591933729225502247572876513004302978515625f,-2.24719999207854304046350080170668661594390869140625f,-1.4727000404477121886515078585944138467311859130859375f,0.54519999991655421212044529966078698635101318359375f,-0.1143000022292135486878805750166065990924835205078125f,0.68769999619126309653438511304557323455810546875f,1.06510002722740182434790767729282379150390625f,-0.04919998707175299301752602332271635532379150390625f,-0.733800018084048932820451227598823606967926025390625f,0.670499999195337093027546870871447026729583740234375f,0.7058000103950501369354242342524230480194091796875f,0.62970003909170646494430911843664944171905517578125f,-0.332200002360343749074900188134051859378814697265625f,-1.01509999764859770721159293316304683685302734375f,-0.0566999860286718782020898288465104997158050537109375f,0.521000030502676736432476900517940521240234375f,0.8063000219583511540832887476426549255847930908203125f,0.18420000212192544353229095577262341976165771484375f,-0.2588999900281425681214386713691055774688720703125f,-0.2748999857366081212006747591658495366573333740234375f,-0.415299989295006444223190555931068956851959228515625f,-0.440399996033310703325014401343651115894317626953125f,-0.65079999892115569792849782970733940601348876953125f,-0.40570000879764567258689567097462713718414306640625f,-0.59150002261996259544929444018634967505931854248046875f,-1.34739999154806167069864386576227843761444091796875f,0.85399998473375993146561313551501370966434478759765625f,-0.3378999989017843486038827904849313199520111083984375f,-0.22409999091327204023826880074921064078807830810546875f,1.14139994934797339709575680899433791637420654296875f,0.84289998921602915515904896892607212066650390625f,0.283899987718462920582851438666693866252899169921875f,-1.07930000535845760367692491854541003704071044921875f,0.2380999769508840291365459052030928432941436767578125f,0.372199953559042118200750337564386427402496337890625f,-0.296999993428588293653547225403599441051483154296875f,0.527399958951770830850591664784587919712066650390625f,-0.277699991795420597640031701303087174892425537109375f,1.2699999904632568448192841970012523233890533447265625f,-0.035900033524632224413153380737639963626861572265625f,0.6733999366298324407154041182366199791431427001953125f,0.3041000075280668735189237850136123597621917724609375f,0.028399982115626887235748654347844421863555908203125f,-1.3385000131428237413189208382391370832920074462890625f,-1.85889998723417448900363524444401264190673828125f,-0.3067999188572161362031920361914671957492828369140625f,-0.601300020393729095502521886373870074748992919921875f,0.193500007331370849072982309735380113124847412109375f,-0.1882999871492383459070651952060870826244354248046875f,0.971100046181678777656998136080801486968994140625f,-0.62560001092553108037463971413671970367431640625f,-0.917899977630376451287474992568604648113250732421875f,-0.65440001120269275514118589853751473128795623779296875f,0.7493000152677298242309689157991670072078704833984375f,-0.226699982824921264512596508211572654545307159423828125f,1.426200007545948000853286430356092751026153564453125f,1.17700002174079454420052570640109479427337646484375f,0.9114999867826696355876947563956491649150848388671875f,0.373700033921003649250991429653367958962917327880859375f,-0.20590000670254227088662446476519107818603515625f,-0.27750001192092899060526178800500929355621337890625f,0.78739997903108616128520225174725055694580078125f,-0.64479999229758966361458760729874484241008758544921875f,0.121699961143731716362026418210007250308990478515625f,0.97930005320981183825068683290737681090831756591796875f,-1.1014999966174363610349473674432374536991119384765625f,1.154100024627149334577325134887360036373138427734375f,-1.29180002435743812583268663729541003704071044921875f,0.831400039071590324368798974319361150264739990234375f,-0.8441000174373385345205633711884729564189910888671875f,-1.861900036534667179921598290093243122100830078125f,0.2937000331014398835094425521674565970897674560546875f,1.27620001469850574693509770440869033336639404296875f,-0.6169000038266181373813878963119350373744964599609375f,-0.4680000310093159754387670545838773250579833984375f,0.248800009143352429674678205628879368305206298828125f,-1.6175000086054203141117113773361779749393463134765625f,0.3998000311523679517478058187407441437244415283203125f,0.5061000278532510332496485716546885669231414794921875f,0.7485000273734334630404418930993415415287017822265625f,-0.5648999786213033669213245957507751882076263427734375f,0.99029997484087939341179662733338773250579833984375f,0.16120000411868051770625243079848587512969970703125f,0.58149996599555020537763994070701301097869873046875f,-0.7838000098884110311558970352052710950374603271484375f,1.13519997801184668873020200408063828945159912109375f,0.912300005298852578761170661891810595989227294921875f,-0.2959999887049207245581783354282379150390625f,-1.448799974647163590901755014783702790737152099609375f,1.2276000153064732245411505573429167270660400390625f,0.339899991846083882052198532619513571262359619140625f,0.5099000079393380957526460406370460987091064453125f,0.370200047376751939509631483815610408782958984375f,-1.8170000340342529199233467807061970233917236328125f,0.8141999969065187858774379492388106882572174072265625f,-1.44160001304149698597711903857998549938201904296875f,0.765199988554417842578914132900536060333251953125f,-1.06109999372959151742179528810083866119384765625f,-0.09430000684857320347731501897214911878108978271484375f,-0.5203000721454618116013079998083412647247314453125f,0.38440002817139007351698865022626705467700958251953125f,-0.025499986529350593400522484444081783294677734375f,1.100699992480873579125955075141973793506622314453125f,-1.8029999966621392815113722463138401508331298828125f,0.6858000019945207359484129483462311327457427978515625f,1.5001000220865012835247398470528423786163330078125f,-0.04369996612072046815455905743874609470367431640625f,0.690000013262033196070888152462430298328399658203125f,0.44790002345144674933408168726600706577301025390625f,0.53440000911653051662852931258385069668292999267578125f,0.4343999950349328731391551627893932163715362548828125f,0.840099996232986701016898223315365612506866455078125f,-0.8372000013545146668292318281601183116436004638671875f,-0.532299962532520876123953712522052228450775146484375f,-1.1693999385595332807241675254772417247295379638671875f,1.756699968370795783556559399585239589214324951171875f,-0.40259997622817689233443161356262862682342529296875f,-0.9817999996960169450943567426293157041072845458984375f,0.191899989521503666622948003350757062435150146484375f,-1.34189998967051504763503544381819665431976318359375f,0.35869998906850764086584604228846728801727294921875f,0.5368999852001667960621489328332245349884033203125f,0.789100021684169039559719749377109110355377197265625f,1.08629997889399465549331580405123531818389892578125f,0.199799985554814707455761890741996467113494873046875f,-1.013399960061907467689934492227621376514434814453125f,0.7600999814808364618556879577226936817169189453125f,-0.4989999939352269819892171653918921947479248046875f,-0.654200024175644490043168843840248882770538330078125f,1.036499969087541384027417734614573419094085693359375f,-1.12150000399351146285198410623706877231597900390625f,1.397100005233287589589963317848742008209228515625f,0.9378999995723358296118021826259791851043701171875f,-0.519399932599068137051290250383317470550537109375f,0.14929997175633979988873534239246509969234466552734375f,0.8310999669075016527131083421409130096435546875f,-0.833300000318140110522335817222483456134796142578125f,1.266300014358758829757789499126374721527099609375f,-0.62130003628209296007156581254093907773494720458984375f,1.33300001782178778597653945325873792171478271484375f,0.64790001868307622867604322891565971076488494873046875f,0.08989999847710183900062475004233419895172119140625f,-0.3599000098764892907610146721708588302135467529296875f,-0.91910001177489786261531889977050013840198516845703125f,-2.29519993159473045807317248545587062835693359375f,0.18029998556971538192783555132336914539337158203125f,0.6028999746501442213997279395698569715023040771484375f,-0.11889999285936359019189012542483396828174591064453125f,-0.571199987019598065529635277925990521907806396484375f,-0.617200022849440532723974683904089033603668212890625f,-0.5865999764218934320325615772162564098834991455078125f,-0.629400018429756347160264340345747768878936767578125f,-1.3668000251770024977560069601167924702167510986328125f,0.47700001719594009585989624611102044582366943359375f,1.0502000099003321320623172141495160758495330810546875f,-1.335499960005282815700411447323858737945556640625f,-0.311000010386109249083119721035473048686981201171875f,0.986200001883506871536155813373625278472900390625f,0.337000025093555422017743694595992565155029296875f,0.07189996150732014257300761528313159942626953125f,0.78389999576508895273718735552392899990081787109375f,-0.231300050866604323118735919706523418426513671875f,0.67709999100267825866694693104363977909088134765625f,-0.5751999957442279765018611215054988861083984375f,-0.713100019568205123476900553214363753795623779296875f,-1.24360004647970168178972016903571784496307373046875f,-0.611900011686981049052747039240784943103790283203125f,0.2246999779224392312926283921115100383758544921875f,0.38990003951117435665452148896292783319950103759765625f,0.264800027203559551480793743394315242767333984375f,0.95259998267292900209213257767260074615478515625f,-1.433699984896183110549827688373625278472900390625f,-0.6807000658877198073781755738309584558010101318359375f,1.3550000539422040191084306570701301097869873046875f,-0.3761000194899739579312836212920956313610076904296875f,0.53290002814531334252023953013122081756591796875f,-0.63640001994222383796540043476852588355541229248046875f,0.872200026723742638523617642931640148162841796875f,0.67500002525746871651080027731950394809246063232421875f,0.530099968740344262840835654060356318950653076171875f,-0.6297000271335242427284129007603041827678680419921875f,-1.292599962556362225285511158290319144725799560546875f,-0.8347999611914165285497801960445940494537353515625f,1.782399993783235547795129605219699442386627197265625f,1.463500024430452572943295308505184948444366455078125f,-1.0372000479578975529193485272116959095001220703125f,-0.602699970076978619459850960993207991123199462890625f,-0.461800043058395726092157929087989032268524169921875f,0.3979999460726972859703209906001575291156768798828125f,-0.665300027218461043077013528090901672840118408203125f,0.2022999974757428987004459486342966556549072265625f,2.2245000429004431197199664893560111522674560546875f,-0.415599990361929183535494303214363753795623779296875f,-0.165000052005053188253214102587662637233734130859375f,-0.37940003135651390042681896375142969191074371337890625f,-1.7850000463426116681375788175500929355621337890625f,0.071800000962614962674024354782886803150177001953125f,-0.4765000175163149886969904400757513940334320068359375f,-0.7476999982774252817563365169917233288288116455078125f,-0.615800005999207034079745426424778997898101806640625f,1.20830001195967273730502711259759962558746337890625f,0.4248000151336182028671828447841107845306396484375f,1.179600010560452716390500427223742008209228515625f,-0.57409994777440953583891314337961375713348388671875f,-0.1828000389531256875130793559947051107883453369140625f,-0.299700055855513181057858673739247024059295654296875f,-1.093400023056566805479405957157723605632781982421875f,0.7023000285446647783516027629957534372806549072265625f,-0.484100007211416938357473327414481900632381439208984375f,0.3160000107958913506678300109342671930789947509765625f,-0.026599964463709990791784321118029765784740447998046875f,-0.557600015977024465740896630450151860713958740234375f,-0.39269998745545742213636231099371798336505889892578125f,-0.703599960723519490812805088353343307971954345703125f,0.1785000665634866034992001004866324365139007568359375f,-0.49580002402961209639897788292728364467620849609375f,0.8724999909847970247511739216861315071582794189453125f,0.98320002466738198876328169717453420162200927734375f,0.502699982519447363671361017622984945774078369140625f,-0.48940002494901435614593765421886928379535675048828125f,0.91319999572187671166290101609774865210056304931640625f,-0.67840000890046392445498213419341482222080230712890625f,0.18489994326830017090657065637060441076755523681640625f,0.3688000153273345294024920804076828062534332275390625f,1.107800009746850111724825183046050369739532470703125f,-1.18670001821517967499630685779266059398651123046875f,0.00869999724552006359346023600664921104907989501953125f,0.47849999175965773900998101453296840190887451171875f,0.28759999857991902505460757311084307730197906494140625f,-0.21449999463930746745887745419167913496494293212890625f,0.109700015175342446838158139144070446491241455078125f,1.287399990318715214954181647044606506824493408203125f,-0.32469998272806421812930466330726630985736846923828125f,1.1747999892801048726909129982232116162776947021484375f,-0.0136000012546781956501718013896606862545013427734375f,-0.52780001995414493887182061371277086436748504638671875f,-1.0186000601142648491759246098808944225311279296875f,1.415500036597251209968817420303821563720703125f,-0.197000010993331697850550199291319586336612701416015625f,0.52540003902316134709593598017818294465541839599609375f,0.5458000063903629683892404500511474907398223876953125f,-1.20970001035109131493072709417901933193206787109375f,0.6552999998748301191398013543221168220043182373046875f,-0.12700000479072370840327721452922560274600982666015625f,0.8521999962694939423357709529227577149868011474609375f,-0.8382999817475678128175786696374416351318359375f,-0.698300049696862767945049199624918401241302490234375f,0.50549997397512191898982791826711036264896392822265625f,-0.75379999765083194329662319432827644050121307373046875f,-1.1988999837696550088139701983891427516937255859375f,1.1526000070177015999206560081802308559417724609375f,-1.17660001514628564933673260384239256381988525390625f,-0.9769999646693470207736709198798052966594696044921875f,1.1955999782733621739083673674031160771846771240234375f,-0.2908999684810640307119911085464991629123687744140625f,-0.10929997719526307609072546256356872618198394775390625f,-0.47769995826780820014079154134378768503665924072265625f,0.357900002217292456663244593073613941669464111328125f,1.18029997662901831034787392127327620983123779296875f,0.37089998666048040742992952800705097615718841552734375f,0.18229998516365852889720144958118908107280731201171875f,-1.170300023716688286867793067358434200286865234375f,0.24930000178217870399066669051535427570343017578125f,1.019599985563754529493962763808667659759521484375f,0.66240003647506195960659169941209256649017333984375f,0.4336999714851383913583049434237182140350341796875f,0.578999969273805259462051253649406135082244873046875f,-0.749700020539761258220323725254274904727935791015625f,-0.548600034335256392381552359438501298427581787109375f,-0.40340001080036191893896102556027472019195556640625f,-0.390000003650784687891928115277551114559173583984375f,-0.3492000197798017513406421130639500916004180908203125f,0.370200045141577838148094770076568238437175750732421875f,0.67810001263916408476006836281158030033111572265625f,1.77129999200701693240489476011134684085845947265625f,-0.630699964690208059181486532907001674175262451171875f,0.34880002553463018699630993069149553775787353515625f,-1.293399985989928335783361035282723605632781982421875f,0.221299999755620202535055796033702790737152099609375f,-0.837399995124339824315029545687139034271240234375f,1.51250002235174196840716831502504646778106689453125f,0.0485999831497669898539015775895677506923675537109375f,0.377299953144788702275036484934389591217041015625f,1.05899998015165319742436622618697583675384521484375f,-0.8292000670909882575898564027738757431507110595703125f,-1.035999942287803232687792842625640332698822021484375f,-0.354999999105930275078435442992486059665679931640625f,0.601999992758035062934141024015843868255615234375f,0.790300020736455888226146271335892379283905029296875f,-0.058399992918968024468995281495153903961181640625f,-0.242899986869096640162979383603669703006744384765625f,-0.83889999449849117496569306240417063236236572265625f,0.1140000169575217103812292407383210957050323486328125f,0.952400005331635046701421742909587919712066650390625f,0.146900002783536987038814913830719888210296630859375f,1.4155000155866144240235371398739516735076904296875f,-0.23430000833868991350072974455542862415313720703125f,-0.367099983030557464047660687356255948543548583984375f,0.20060000168681124677050320315174758434295654296875f,0.147000000134110297977940717828460037708282470703125f,-1.36470004944801370783125094021670520305633544921875f,0.40550001513958022769656963646411895751953125f,0.8404000312745572021100315396324731409549713134765625f,-0.98920004265308403290646310779266059398651123046875f,0.36650000086426703926889558715629391372203826904296875f,-0.757900000429153397618620147113688290119171142578125f,-1.232400057709217566781489949789829552173614501953125f,0.285400013020634890636983982403762638568878173828125f,0.3195000061690806969494360600947402417659759521484375f,1.4263000484079129126513407754828222095966339111328125f,0.41519999231696136376967842807061970233917236328125f,0.2318999843060967069874322987743653357028961181640625f,0.17789998791217787044161013909615576267242431640625f,0.8082000245928764314840009319595992565155029296875f,1.4626000124752522868476489747990854084491729736328125f,0.48900003707408945530232813325710594654083251953125f,-0.59050002869963658103102943641715683043003082275390625f,-0.69960000627636897974070961936376988887786865234375f,-1.102000007510185231041077713598497211933135986328125f,-1.1745000174939630444015392640721984207630157470703125f,-0.779300016310811027864247080287896096706390380859375f,1.1693999781966208928452033433131873607635498046875f,0.6410999879926440048194535847869701683521270751953125f,0.4757999835729602722267372882924973964691162109375f,0.2216999946713447133816998757538385689258575439453125f,0.8853999733090400781065909541212022304534912109375f,0.4967999761521812995823665914940647780895233154296875f,-0.340899976527690995453667710535228252410888671875f,0.5846000307887793123740038936375640332698822021484375f,0.4939999810457234019622774212621152400970458984375f,0.0035999930590389084272828768007457256317138671875f,1.1602999797582622676372920977883040904998779296875f,1.0091000474631786065771166249760426580905914306640625f,-0.4200000330805782056131647550500929355621337890625f,0.203200046646596188537614580127410590648651123046875f,-1.27010004227757544725818661390803754329681396484375f,-0.80470001517534228696604259312152862548828125f,-0.44510000755786904846900142729282379150390625f,-0.05069995902776636143016730784438550472259521484375f,-0.52290004945397416946661905967630445957183837890625f,-0.285500015586614974694157353951595723628997802734375f,-0.7806999688625335664937665569595992565155029296875f,-0.2736000122070318951728040701709687709808349609375f,-0.26630001510381617890743655152618885040283203125f,-1.4520000008046611839063189108856022357940673828125f,-1.087699998724459771182182521442882716655731201171875f,-0.77589996475577294177128351293504238128662109375f,-0.4579999872744078270869749758276157081127166748046875f,-0.4834999693334101067421215702779591083526611328125f,-1.092299992111324780097447728621773421764373779296875f,0.582100032055377969442133689881302416324615478515625f,1.3527999973416322543329215477569960057735443115234375f,-0.02760003616809836302081748726777732372283935546875f,-0.27629994968771942609464531415142118930816650390625f,0.0421000289261339588620103313587605953216552734375f,0.3978999814301726889453902913373894989490509033203125f,-0.434400009042024581873420174815692007541656494140625f,-0.65280000367462687194120007916353642940521240234375f,-0.234799987715482938455124894971959292888641357421875f,-1.326199998605250840455482830293476581573486328125f,0.0589000193834301466466740748728625476360321044921875f,1.0393999454140665505974538973532617092132568359375f,0.196699991914629723765983726480044424533843994140625f,0.8376999683260919482563622295856475830078125f,-0.4989999690502882057074884869507513940334320068359375f,-0.7590999970972536203817071509547531604766845703125f,-0.01769998061955002643941270434879697859287261962890625f,-0.46880003849863971510103510809130966663360595703125f,0.484000016584992476254001303459517657756805419921875f,0.9827999821797013257906883154646493494510650634765625f,1.132600034081935991281397946295328438282012939453125f,-0.050200015413761178706408827565610408782958984375f,0.3509000026047228715242454200051724910736083984375f,0.33970000101923947255500024766661226749420166015625f,-2.103700008738039084477122742100618779659271240234375f,2.36699997860193178667032043449580669403076171875f,0.5705999785900106591185476645478047430515289306640625f,0.69599999958276814737700988189317286014556884765625f,0.02310000444352677817505536950193345546722412109375f,1.099999969974159608199215654167346656322479248046875f,-0.236199973571299182140137418173253536224365234375f,0.734799979780614176405606485786847770214080810546875f,0.84679997734427470135187832056544721126556396484375f,0.85389999680817130212062693317420780658721923828125f,-0.798500005096196563414423508220352232456207275390625f,0.367199979412556221092245323234237730503082275390625f,-0.14679998166561158967624578508548438549041748046875f,1.384799974900483565676267971866764128208160400390625f,0.78979997035562998775048981769941747188568115234375f,0.834099989646673645182772816042415797710418701171875f,0.5301000278979548507862773476517759263515472412109375f,1.687600008079410685013499460183084011077880859375f,-0.83280001149773585922275742632336914539337158203125f,0.696200047779083064369842759333550930023193359375f,-0.665399992904066817089869800838641822338104248046875f,-0.671299959373474308677032240666449069976806640625f,1.0676000117301942538006187533028423786163330078125f,0.138400006031989875054932781495153903961181640625f,-1.0593000308394433517378274700604379177093505859375f,1.3664999786615368293496430851519107818603515625f,0.0750999752223486805036145597114227712154388427734375f,-0.5851000367641443489219454932026565074920654296875f,0.378600026512145859669544734060764312744140625f,-0.6640000287294380854064002051018178462982177734375f,-0.2452000119119883692064831848256289958953857421875f,-0.29479998652338945674955539288930594921112060546875f,0.2965999552249893866928687202744185924530029296875f,0.6553000021100050531686065369285643100738525390625f,0.401200030195713708280891296453773975372314453125f,-0.86989998171329574461196898482739925384521484375f,-0.5678000478565703446065526804886758327484130859375f,1.116800033521652579793226323090493679046630859375f,-0.502599985951184269339364618645049631595611572265625f,1.1881000161409378534926872816868126392364501953125f,0.82600005412101840107652606093324720859527587890625f,-0.339900044596195272106342599727213382720947265625f,-0.83390000731348923324048882932402193546295166015625f,-1.13900000213086638467530065099708735942840576171875f,0.62340001154541990757707026205025613307952880859375f,0.516100004011393220793024738668464124202728271484375f,-1.3015000385642050417800419381819665431976318359375f,-0.273299989458918535234488444984890520572662353515625f,-0.030399956470727573787371511571109294891357421875f,-0.49700002017617261884652180015109479427337646484375f,-0.4025999865472318361980796908028423786163330078125f,0.29349996739625883179769516573287546634674072265625f,0.11979999114274964000514955841936171054840087890625f,1.1041000075280671399724496950511820614337921142578125f,0.89010002722740200198359161731787025928497314453125f,-0.326199979084730529166336054913699626922607421875f,1.542900029337406220264483636128716170787811279296875f,1.19199999693036051695571586606092751026153564453125f,-0.688100041174889209827369995764456689357757568359375f,0.14939997357726131355093457386828958988189697265625f,-0.89260002670586136019892364856787025928497314453125f,-1.4115000063031908350552612319006584584712982177734375f,0.373699989180266756161330476970761083066463470458984375f,-0.9119999881386753326495409055496565997600555419921875f,-0.8123999995946877827890375556307844817638397216796875f,-0.117099967682361860710216205916367471218109130859375f,0.659699995058775190415190081694163382053375244140625f,0.055000019520520027072052471339702606201171875f,-1.147800050613284827960569600691087543964385986328125f,-0.225899957379697813308894183137454092502593994140625f,-0.8894999970793715515782196234795264899730682373046875f,0.56389996871948167012078556581400334835052490234375f,1.4760000539347533088374575527268461883068084716796875f,-0.5747000102579591374052370156277902424335479736328125f,0.73300004140287688958466105759725905954837799072265625f,-0.964399999766051951866074887220747768878936767578125f,0.64560000936090933265631974791176617145538330078125f,-0.538500002637505303937359713017940521240234375f,0.1972999655127509743834934852202422916889190673828125f,-0.31050001603364929536610361537896096706390380859375f,-1.8836000380605464332717247089021839201450347900390625f,-0.007799999651312727877439101575873792171478271484375f,1.0845000697970397229852324016974307596683502197265625f,-0.1387000046402218078611667806399054825305938720703125f,0.7028999992370603688840446920949034392833709716796875f,-0.974600034514069957225501639186404645442962646484375f,-0.27600000956654557882075096131302416324615478515625f,-0.4144999935030939663960225516348145902156829833984375f,0.1033999748885623404248690349049866199493408203125f,-1.6656000042945147132655847599380649626255035400390625f,-0.0741000064849852702053567554685287177562713623046875f,-0.90859997942447723318082353216595947742462158203125f,0.93419999377727502842816420525196008384227752685546875f,-0.49619999551326021691721734896418638527393341064453125f,-0.1945999946720899931307258157175965607166290283203125f,0.43139999278485785527692542018485255539417266845703125f,0.6073000016108149878846234059892594814300537109375f,0.9040999753788117043740157896536402404308319091796875f,0.48770001280605779214738504379056394100189208984375f,0.950199965569377358320934945368207991123199462890625f,-0.49420002190321687951524154414073564112186431884765625f,-0.4542000169113278307264636168838478624820709228515625f,0.070400003707408842501536128111183643341064453125f,0.67750004619359938118350328295491635799407958984375f,0.02710001590624477874058584347949363291263580322265625f,-0.69159996908307064256149487846414558589458465576171875f,0.4271000185884534783298249749350361526012420654296875f,-0.490800036267190709082797184237278997898101806640625f,-0.99180001228749692376851498920586891472339630126953125f,0.5354000018447633291174270198098383843898773193359375f,-0.5822000063836572092412779966252855956554412841796875f,-0.2838000000908971198754215947701595723628997802734375f,-0.7125000038743014574293965779361315071582794189453125f,0.7415999810785052215322821211884729564189910888671875f,1.3449000460490585506789784631109796464443206787109375f,-0.4570000342577695828794048793497495353221893310546875f,-0.91590002085864574699058948681340552866458892822265625f,-0.34829999396652022713993801517062820494174957275390625f,0.1884000031262633712003662367351353168487548828125f,0.16189998344928024920363895944319665431976318359375f,-0.6866000011950725845366605426534079015254974365234375f,0.43529998907148825804114267157274298369884490966796875f,-0.5653999957352875416205506553524173796176910400390625f,-0.18019998117834301609718750114552676677703857421875f,1.197500059828162211061908237752504646778106689453125f,0.489200018140673709154953030520118772983551025390625f,0.30109998227432355211163894637138582766056060791015625f,-0.5016000187039377777864501695148646831512451171875f,0.580099997127055733159295414225198328495025634765625f,0.916699978876113874548536841757595539093017578125f,-0.64729998473525041102760724243125878274440765380859375f,1.18619995867013994228500450844876468181610107421875f,-0.189700034695864161449208040721714496612548828125f,-0.99809999781250979822289082221686840057373046875f,0.6132000030606981066938487856532447040081024169921875f,-1.3498000191569321515316914883442223072052001953125f,-0.083500012770295561903566294859047047793865203857421875f,-0.119800018337368641141438274644315242767333984375f,0.53030002401471154627188298036344349384307861328125f,-0.1195999810561539644648831881568185053765773773193359375f,0.27940003431811921696947820237255655229091644287109375f,0.01199998618289842278183954249470843933522701263427734375f,-0.1495999322175987078509251659852452576160430908203125f,-0.1643999874539672223505704096169210970401763916015625f,1.29370000285208153201210734550841152667999267578125f,1.2510000165700905228050032746978104114532470703125f,1.1277999844148751673600372669170610606670379638671875f,-0.281000053599476995458417150075547397136688232421875f,0.90559997531175617790921705818618647754192352294921875f,-0.61409998681545285847960258251987397670745849609375f,-0.7787999510288237114963294516201131045818328857421875f,-0.8169000828027730420899388263933360576629638671875f,0.455000017434358472456779054482467472553253173828125f,-0.888400032630562552782294005737639963626861572265625f,1.27530001224279398996941381483338773250579833984375f,0.8573000319719312756205908954143524169921875f,0.416200014922022631935760728083550930023193359375f,0.5071999963253734922119519978878088295459747314453125f,-0.0923000092476609079739091612282209098339080810546875f,-1.0469000147417186763476593114319257438182830810546875f,-0.1648000180020929850144284500856883823871612548828125f,0.0521000257596373472779305302537977695465087890625f,-0.303899995094537356266073402366600930690765380859375f,-0.463699985492228794470292996265925467014312744140625f,0.285100017839670538677410149830393493175506591796875f,-0.960599967339633931118214604794047772884368896484375f,0.8590000059306619473176169776706956326961517333984375f,-0.5583999836802480132291748304851353168487548828125f,-0.41810003907009996026999942841939628124237060546875f,1.1439999818652861751644422838580794632434844970703125f,-0.5499000071756541263567896749009378254413604736328125f,0.6522000130891800750987385981716215610504150390625f,-0.72020001366287456079362527816556394100189208984375f,1.464500016152859185325496582663618028163909912109375f,0.331600032450258819238797514117322862148284912109375f,0.8458999899387353860191751664387993514537811279296875f,-0.3805999969184394871035692631267011165618896484375f,0.411099968248605396325956462533213198184967041015625f,-0.086199931102991289577630595886148512363433837890625f,0.31019997473359151030081193312071263790130615234375f,0.744500013954937145399526343680918216705322265625f,-0.57699999566376247717158776140422560274600982666015625f,-1.3089000354021791405756403037230484187602996826171875f,-0.0278000084802510760706439896239317022264003753662109375f,0.5847999781042336753245081126806326210498809814453125f,-1.0698000145748256528577257995493710041046142578125f,-0.9773999995201820834012096383958123624324798583984375f,-1.68950001317262632483107154257595539093017578125f,0.12499998249113541959331996622495353221893310546875f,-1.397900004005432350595583557151257991790771484375f,-0.01650001502037046208215542719699442386627197265625f,-0.252199964362383344251838934724219143390655517578125f,-1.19070001252293611315735688549466431140899658203125f,1.38489999542236308371911945869214832782745361328125f,-0.2406999845087529621423527714796364307403564453125f,-0.63780003191232648163122576079331338405609130859375f,0.0790999937444929468455256937886588275432586669921875f,0.3917000375866894756882174988277256488800048828125f,0.54210002005994351037543310667388141155242919921875f,0.204400009191036957645337679423391819000244140625f,-1.2326000111155208660562720979214645922183990478515625f,1.369499993801117199154759873636066913604736328125f,-0.53629999863803401449757757291081361472606658935546875f,0.959600013428926867931068045436404645442962646484375f,0.52559999870657880816082752062357030808925628662109375f,-0.231499972850084390074698603712022304534912109375f,-0.1238000202819702355583331154775805771350860595703125f,-0.66500005751848245694191064103506505489349365234375f,0.459900047148019408549401987329474650323390960693359375f,-0.43509998729228982483618892729282379150390625f,-0.2281000255472960258629200325231067836284637451171875f,0.292599930444359390691033695475198328495025634765625f,0.024299945642054066041737314662896096706390380859375f,-0.758100010776519894761804607696831226348876953125f,-0.595099998170137478581409595790319144725799560546875f,-0.66070001550316792560124667943455278873443603515625f,-0.5384999571144584162851742803468368947505950927734375f,-0.416799989265203674193571714567951858043670654296875f,0.33549997602403180962937767617404460906982421875f,1.308600007289647759733952625538222491741180419921875f,-0.71760000826567360565633180158329196274280548095703125f,-0.3865999873369929673572187311947345733642578125f,0.537799997006356722550890481215901672840118408203125f,-0.2692000158682461563586230113287456333637237548828125f,0.2240000112205737803350302783655934035778045654296875f,1.4694999782294033696672386213322170078754425048828125f,-0.005499991670251114239675871431245468556880950927734375f,0.48599997197836619644562006214982829988002777099609375f,0.5219000126555564111896501344745047390460968017578125f,-0.1563000119745729943332435141201131045818328857421875f,-0.007200023519992715392845639144070446491241455078125f,0.4160000007748596573264876496978104114532470703125f,0.9331000260874631191398975715856067836284637451171875f,-1.0135000223815442410568721243180334568023681640625f,0.78710000990852713176337829281692393124103546142578125f,-0.8847999524369833945769414640381000936031341552734375f,1.009799990677088477042389058624394237995147705078125f,-0.5647999763160942432449473926681093871593475341796875f,0.6034999993406235940796022987342439591884613037109375f,-0.61039999048262816305765454671927727758884429931640625f,0.47950002472102681849719374440610408782958984375f,-0.611400003476440634386790407006628811359405517578125f,-0.02209999390840533539659418238443322479724884033203125f,0.2249000034689903060325377737171947956085205078125f,-0.01309999827817087680870145049993880093097686767578125f,-0.7321000359296803150499499679426662623882293701171875f,0.3156000074796379006869528893730603158473968505859375f,0.38709995989650447523189313869806937873363494873046875f,-0.36139997784644439793311221365001983940601348876953125f,0.25370000173449513969359259135671891272068023681640625f,-0.11699998608976625025235307475668378174304962158203125f,0.44060000369846819001651283542742021381855010986328125f,-0.0559999965652817621730719110928475856781005859375f,0.381500001609325511964243560214526951313018798828125f,1.6623000228822231161274203259381465613842010498046875f,-0.756899987956880959671934760990552604198455810546875f,0.004800028023124180975855779252015054225921630859375f,0.74819999646693435835231866803951561450958251953125f,-0.79769997914806001215737296661245636641979217529296875f,1.22190003839731264889678641338832676410675048828125f,0.4283000101715324081652624954585917294025421142578125f,0.19329998685121463353198123513720929622650146484375f,-1.0646000558972363858600829189526848495006561279296875f,-1.5165000159144401692401515902020037174224853515625f,-0.638100038082897835778339867829345166683197021484375f,-2.031800049894302961916991989710368216037750244140625f,0.32010001610741090072309589231736026704311370849609375f,0.315399991562963055713453286443836987018585205078125f,0.6260999939531080027421694467193447053432464599609375f,0.97430000997781718297119368799030780792236328125f,-0.254399990341067638865979461115784943103790283203125f,-0.11299995136260942985018118633888661861419677734375f,-0.303600037892907959058419464781763963401317596435546875f,0.77839997384548187397967922152020037174224853515625f,-2.03320005521476332432939670979976654052734375f,2.2983000431776048344545415602624416351318359375f,-0.507799966477601838477085038903169333934783935546875f,-0.5914000219352544274187266637454740703105926513671875f,0.533100010441243110648201763979159295558929443359375f,-0.7491999841474006327501911073341034352779388427734375f,0.450899978669732692804217322191107086837291717529296875f,-0.61669998602867082126977038569748401641845703125f,-0.03920002356097118667577205997076816856861114501953125f,-0.240700006413459899334839064977131783962249755859375f,0.80529998799115365404333033438888378441333770751953125f,0.195900003498792774081493917037732899188995361328125f,1.68079999310970240600227043614722788333892822265625f,-1.823000038683414647522340601426549255847930908203125f,-0.608800060328840597634325604303739964962005615234375f,0.241100017347931849798214898328296840190887451171875f,0.80640002845451252966313404613174498081207275390625f,0.0990000203847885007490958741982467472553253173828125f,0.948200042437016055174581197206862270832061767578125f,-0.02820003107488133764491067267954349517822265625f,-0.5676999958559874048802385004819370806217193603515625f,-0.4773999684512606478392626740969717502593994140625f,-1.132300011706351750007115697371773421764373779296875f,0.786400016626715991918672443716786801815032958984375f,0.739199999141693098181349341757595539093017578125f,-1.499200029018521451718015669030137360095977783203125f,0.746800033223629089462747288052923977375030517578125f,1.78260002540201067944281021482311189174652099609375f,0.13240000631883719339754179600276984274387359619140625f,0.8814999885708092453029394164332188665866851806640625f,0.24569998789876701383860790883773006498813629150390625f,-1.528699992141871799589125657803378999233245849609375f,0.10429996855258938381894040503539144992828369140625f,-0.688400000369548070722203192417509853839874267578125f,-0.02369999080076812258965901492047123610973358154296875f,0.4904000075072048048241413198411464691162109375f,0.00420000253170715875938867611694149672985076904296875f,-0.862699996414780567732805138803087174892425537109375f,1.1890000177957116278548710397444665431976318359375f,-0.41439994329065099609721301021636463701725006103515625f,0.6251999879024923156833892790018580853939056396484375f,-0.49710001763850464318039712452446110546588897705078125f,-1.4485000412128867797179054832668043673038482666015625f,1.574900004716962342854458256624639034271240234375f,-0.97509999768584965096351879765279591083526611328125f,0.7617999730974436278785333342966623604297637939453125f,0.5261999578133227384313386210124008357524871826171875f,-0.467399966737627625690265631419606506824493408203125f,-0.5508999998480084148155810908065177500247955322265625f,0.992399982979893735546284005977213382720947265625f,-0.52509994277507043847208478837274014949798583984375f,-0.58370001283586059326324857465806417167186737060546875f,0.132799981267004596485747924816678278148174285888671875f,0.8182000180177393389158169156871736049652099609375f,1.2322999779552221699674419141956605017185211181640625f,-0.592700035399943647718146166880615055561065673828125f,-0.6498000672131780675044865347445011138916015625f,-1.63410000126957921651182914501987397670745849609375f,-0.299100010806321758849435354932211339473724365234375f,-0.515899988895654360021580941975116729736328125f,-0.6337000254645943631004456619848497211933135986328125f,-0.1295000124275684783015094581060111522674560546875f,0.1829000153303150799644072321825660765171051025390625f,-0.3518999771162867329366008561919443309307098388671875f,-1.033499984011053474119989914470352232456207275390625f,-0.538800004374980545662765507586300373077392578125f,0.0858999710887662359937166911549866199493408203125f,-1.03219997941255581253017226117663085460662841796875f,-0.0614999937117095551997181246406398713588714599609375f,0.9495999897360807295854101539589464664459228515625f,-0.47119998474717139647083286035922355949878692626953125f,0.9625000301003456826265392010100185871124267578125f,-0.51700000905618070934366414803662337362766265869140625f,0.5556000060081480729223812886630184948444366455078125f,-0.23579997558221166276126723460038192570209503173828125f,-0.5205000468418001791093274732702411711215972900390625f,-0.68580003384575338731821148030576296150684356689453125f,0.286300006759166336678390507586300373077392578125f,-0.6861000064142042109693875318043865263462066650390625f,0.30450001436472096116858665482141077518463134765625f,0.266900023347139303542263633062248118221759796142578125f,-0.30490003394186515617292343449662439525127410888671875f,-0.5223000130474562130444837748655118048191070556640625f,-0.8822000016897917351599289759178645908832550048828125f,1.476499981045722886818793995189480483531951904296875f,-0.73629999010711888463021068673697300255298614501953125f,0.6999999707564719120256313544814474880695343017578125f,0.02550001085549580270850356100709177553653717041015625f,0.4844999765530226820686721111997030675411224365234375f,0.34990000108480412421840810566209256649017333984375f,-0.30190003575235646593455385300330817699432373046875f,-0.099200055170058920595010931720025837421417236328125f,-1.35769998248219447845031027100048959255218505859375f,1.8667000281989576393470997572876513004302978515625f,-1.19510000524818860156983646447770297527313232421875f,0.316500017926097054754563941969536244869232177734375f,-0.19759999615848045095845009200274944305419921875f,-0.58859998858869122528858497389592230319976806640625f,-1.23100002320110757381144139799289405345916748046875f,-1.022200016218423712643925682641565799713134765625f,-0.115800025296211916980837486335076391696929931640625f,0.047699977341294275134941926808096468448638916015625f,0.29200000765919664758030194207094609737396240234375f,-0.844200034680962829725103802047669887542724609375f,-0.5751000000327823880041933080065064132213592529296875f,-0.3246000209167601013859894010238349437713623046875f,-1.307200017689913895679865163401700556278228759765625f,0.1663999902330337565814488698379136621952056884765625f,-1.6428000110693272706186007781070657074451446533203125f,-0.8828000140681862006886149174533784389495849609375f,0.5867000788629059826462253113277256488800048828125f,-0.70239999333620151134027764783240854740142822265625f,0.859899998998641112990526380599476397037506103515625f,-1.110600015470386114913026176509447395801544189453125f,-0.38790000728368756455211041611619293689727783203125f,-1.435200001257658453113208452123217284679412841796875f,-0.1249000033199791914739762432873249053955078125f,-0.216000048309564096626900209230370819568634033203125f,-0.3673000036597249273739862474030815064907073974609375f,-0.186599987709521908385568167432211339473724365234375f,0.5117000124037265340604108132538385689258575439453125f,0.9481000143527982260138742276467382907867431640625f,0.4593000008881087392609288144740276038646697998046875f,-0.100100028493999904100064668455161154270172119140625f,-0.70179999723732466865300239078351296484470367431640625f,-0.47760002610981455717364951851777732372283935546875f,0.133000012643635423525978467296226881444454193115234375f,-0.509200001749396502503941519535146653652191162109375f,0.63370002676844616740226001638802699744701385498046875f,-0.385700002539157704717354135937057435512542724609375f,1.459200058969855717805330641567707061767578125f,-0.9429000064641233080919846543110907077789306640625f,-0.721200051206351222532475730986334383487701416015625f,0.81860002323389124967434327118098735809326171875f,-1.21270001302957464162091127946041524410247802734375f,1.168899990028142266140775973326526582241058349609375f,0.80650003863871144726971351701649837195873260498046875f,0.332099981093406437793191798846237361431121826171875f,-0.510100024135411000969497763435356318950653076171875f,0.7804000187948350042432821283000521361827850341796875f,0.647400023287534764904194162227213382720947265625f,-1.2664000053763400099882119320682249963283538818359375f,-0.75640003756284812652666005305945873260498046875f,-1.1953000216305260661187048754072748124599456787109375f,-1.30650001963973050322920244070701301097869873046875f,0.476599969604611484186307279742322862148284912109375f,0.32710002743601862551514614096959121525287628173828125f,1.4316000149786474171520467280060984194278717041015625f,1.0800000055506824292450573921087197959423065185546875f,0.7330000035539268044004757030052132904529571533203125f,0.37419998427778444760605225383187644183635711669921875f,0.63990000019073478565445611820905469357967376708984375f,-0.2165000215768813074390664041857235133647918701171875f,-1.4274999952316278495345613919198513031005859375f,-0.29130000810027123492318423814140260219573974609375f,-1.411199995326995804134639911353588104248046875f,0.25440001179873927839025782304815948009490966796875f,-0.55499999865889559913512130151502788066864013671875f,0.7507000191539521249950439596432261168956756591796875f,0.5983999520897871349234264926053583621978759765625f,-0.880200010421872125476738801808096468448638916015625f,0.045299982440472064126879558898508548736572265625f,0.724200022685528299604129642830230295658111572265625f,0.0901000337839121812066878192126750946044921875f,-0.2752999984592197080246478435583412647247314453125f,-0.59170001761913226800970733165740966796875f,0.9463000523567206556663222727365791797637939453125f,-0.045800019335747066406838712282478809356689453125f,0.2888999968826764241924820453277789056301116943359375f,0.01560001524686871476887972676195204257965087890625f,-0.451399997776746442212925103376619517803192138671875f,-0.78950000438094125598809114308096468448638916015625f,-0.239199975597857328324380432604812085628509521484375f,0.1854999373555177299977003713138401508331298828125f,0.74610000699162437598488395451568067073822021484375f,-1.1053999870181083764464347041212022304534912109375f,-1.25800006327033120356873041600920259952545166015625f,0.14090001027882070872010444873012602329254150390625f,1.099899998104572063084560795687139034271240234375f,0.8615000002682204893744710716418921947479248046875f,0.6221999746441839729271805481403134763240814208984375f,1.163999996840953343735236558131873607635498046875f,-0.29400001323223090565761594916693866252899169921875f,-0.28339999344050870178080003825016319751739501953125f,1.156600051337480561386428234982304275035858154296875f,-0.1911999845236536099690738410572521388530731201171875f,-0.9393000372469426650923196575604379177093505859375f,-0.208399981817602775180375829222612082958221435546875f,0.9543000451445582399401246220804750919342041015625f,0.0720999860107902623695963484351523220539093017578125f,0.437499998658895439263005755492486059665679931640625f,1.341299964663386301566561087383888661861419677734375f,-0.24870000870078812216235064624925144016742706298828125f,0.7985999845281244180483781747170723974704742431640625f,-0.4758999968677757319568399907439015805721282958984375f,0.48300001893937594132921731215901672840118408203125f,0.28749997854232756111514390795491635799407958984375f,-0.10749996423721341898271930404007434844970703125f,-0.20490001188814677135496822302229702472686767578125f,-0.4047999851077788147080127600929699838161468505859375f,0.183500023759901598197075145435519516468048095703125f,-0.84520000263601513523781250114552676677703857421875f,-1.185900015568732701609633295447565615177154541015625f,0.783900010293721916099229929386638104915618896484375f,-0.4545000085532662215115351500571705400943756103515625f,0.4780000039637088971034017959027551114559173583984375f,0.1888999991923572263630148881929926574230194091796875f,0.4456999922946096592824005711008794605731964111328125f,-0.37340003896355700163667279412038624286651611328125f,0.01760003497600592936578323133289813995361328125f,0.9576000492438681011009293797542341053485870361328125f,-0.5868000436544422715456903461017645895481109619140625f,-0.37080002091899533223084972632932476699352264404296875f,-0.11529999249875533262610360907274298369884490966796875f,-1.00319998838305490806988018448464572429656982421875f,0.085699992853403017534219543449580669403076171875f,-0.573499998316168824175065310555510222911834716796875f,-0.67600002476573006759963391232304275035858154296875f,-0.9025000280886892323195525023038499057292938232421875f,-0.96280003984719508824952072245650924742221832275390625f,-1.13010001873373955305623894673772156238555908203125f,-1.255100052261352683302675359300337731838226318359375f,-0.0272999936759468209146461958880536258220672607421875f,0.03409998703896999217022312222979962825775146484375f,-0.43580002253949656765286135851056315004825592041015625f,1.1930999776959421154742813087068498134613037109375f,-0.389500028222799610233550993143580853939056396484375f,0.2805999878287313009650461026467382907867431640625f,-0.406700002866983822968904860317707061767578125f,-2.229100024366378107743003056384623050689697265625f,-1.0405000115819273975859005076927132904529571533203125f,-1.15249998683109833308435554499737918376922607421875f,0.09559998190552010388643111582496203482151031494140625f,0.372300035138427798386828726506792008876800537109375f,1.3891999896049498364192231747438199818134307861328125f,0.1091000023126600382283868384547531604766845703125f,-0.307600004726648723618609437835402786731719970703125f,0.56590003315210424261749722063541412353515625f,-1.531100043499469709473714829073287546634674072265625f,1.963400016462802710748292156495153903961181640625f,0.8394000245392323478199614328332245349884033203125f,-0.4142000059962267499713561846874654293060302734375f,2.572000117704273947083493112586438655853271484375f,-1.857300083716214356144291741657070815563201904296875f,-0.01300004766136531042519663969869725406169891357421875f,-0.21239994455352440017037451980286277830600738525390625f,-0.440699988383054463980670334422029554843902587890625f,-0.97199999707937223547560279257595539093017578125f,-0.36469997985958979569431903655640780925750732421875f,0.416300004822015790040268257143907248973846435546875f,0.631600040459633138567596688517369329929351806640625f,0.96969998291432801806877250783145427703857421875f,1.3243999998033046949785784818232059478759765625f,-0.425200007814168667863441442023031413555145263671875f,0.4424999896809456156887563338386826217174530029296875f,-1.8040000324547287302578979506506584584712982177734375f,-1.306099989818035655986250276328064501285552978515625f,-1.334099992384761179664565133862197399139404296875f,-0.90249998196959513308001987752504646778106689453125f,0.47500001378357392667339809122495353221893310546875f,1.210700037258863659417329472489655017852783203125f,-0.045699993747473133254288768512196838855743408203125f,-0.207899999609589103410911548053263686597347259521484375f,0.4010000095665446906423312611877918243408203125f,-0.90040002889037129563121197861619293689727783203125f,-0.312499980628490714451572785037569701671600341796875f,1.297300048176944553546263705356977880001068115234375f,1.1659999878481033253052601139643229544162750244140625f,-0.22800002013146858370618019762332551181316375732421875f,1.1343999907508486302276651258580386638641357421875f,-0.77359999412074653690751802059821784496307373046875f,-1.38899999691545961155725308344699442386627197265625f,-0.85250002846121775146315258098184131085872650146484375f,-0.787400028353929304358871377189643681049346923828125f,-1.00450007173419120221069533727131783962249755859375f,0.635600055405497688099103470449335873126983642578125f,1.373000032499432254695648225606419146060943603515625f,-0.350100007036328264575786306522786617279052734375f,-0.6352000264406203466904798915493302047252655029296875f,0.1490000285059214224503421064582653343677520751953125f,-1.45090004550144069384032263769768178462982177734375f,1.273200014944374292014117600047029554843902587890625f,0.227799988792091401279549245373345911502838134765625f,0.866399992170185040407659471384249627590179443359375f,1.8813000599563125536661800651927478611469268798828125f,0.706100045883655713652160557103343307971954345703125f,0.082100004041194996062813515891321003437042236328125f,-0.7763999799698588777374652636353857815265655517578125f,0.11430001668334011100114366854541003704071044921875f,-0.5296000253498556187281565144076012074947357177734375f,-2.123500042721629288422491299570538103580474853515625f,0.1970000294893987646815958214574493467807769775390625f,-0.08659995239377005216141469645663164556026458740234375f,0.14010000271499156720977907752967439591884613037109375f,1.180400019428133884247245077858678996562957763671875f,0.413700015480817029356330749578773975372314453125f,0.4117000074863437220074047218076884746551513671875f,-1.3389999825358387397500337101519107818603515625f,-0.3881999782502649765092428424395620822906494140625f,-0.434699999082088339719120995141565799713134765625f,-0.102600016275048222524901575525291264057159423828125f,-0.5726999861329795660225272513343952596187591552734375f,0.89249999251216671769526556090568192303180694580078125f,-0.9146999473750592191123587326728738844394683837890625f,-0.592400021499395190716086290194652974605560302734375f,-0.520500006198883369279428734444081783294677734375f,0.118800006908178712450308012194000184535980224609375f,-1.0757999963879587568271745112724602222442626953125f,0.07060003364980216300494930692366324365139007568359375f,0.12929998077154181146397604607045650482177734375f,0.605699995088576859103568494901992380619049072265625f,0.32470001502633127898889142670668661594390869140625f,-0.0670000130608678912125242277397774159908294677734375f,-0.2113999830245971711661923109204508364200592041015625f,0.909100051896274408136378042399883270263671875f,-2.02750001620501318910783083993010222911834716796875f,-0.18690004093051104661071804002858698368072509765625f,-0.155799944829940528734368854202330112457275390625f,1.114600064837933057759755683946423232555389404296875f,-1.0365000311136238764220252051018178462982177734375f,-0.1519000211119656551517209663870744407176971435546875f,0.77119998318254889824174824752844870090484619140625f,0.072599957489967170687350517255254089832305908203125f,-0.219900012558698509934629328199662268161773681640625f,-0.1291999949693678484408110307413153350353240966796875f,-1.1668000099033115635194235437666065990924835205078125f,0.0528999944686887602074421010911464691162109375f,0.1889000430762763915737423303653486073017120361328125f,1.6100000493228439069071100675500929355621337890625f,-0.02129993188083201260951682343147695064544677734375f,0.916999986052511761869254769408144056797027587890625f,0.795100006514787072120498123695142567157745361328125f,0.366599978470802501107073112507350742816925048828125f,0.550100016200541830357906292192637920379638671875f,0.0026000009268525303696151240728795528411865234375f,0.1698000154316421372868717298842966556549072265625f,-0.09090002197623248125779582551331259310245513916015625f,-0.68549996164441129309352618292905390262603759765625f,-0.72520000766515746448703794158063828945159912109375f,-0.814599994504451530019650817848742008209228515625f,0.590699986892938344595904709422029554843902587890625f,-0.3777000033438202564184393850155174732208251953125f,-0.43990003058910398525682694526039995253086090087890625f,-0.37090000320076921269674130599014461040496826171875f,0.3514000090271236498296048011980019509792327880859375f,-0.072499972134828727376998358522541821002960205078125f,1.269399996487795778676854752120561897754669189453125f,-1.5240000134557487587727564459783025085926055908203125f,1.0547999560505143534783201175741851329803466796875f,-0.599900006821751929919628310017287731170654296875f,1.1394999914169308130595936745521612465381622314453125f,0.359699935230612499736935205874033272266387939453125f,0.01360001380890574207427334840758703649044036865234375f,-1.297400013005733487858606167719699442386627197265625f,1.0847999564975501218810904902056790888309478759765625f,0.909300005805492528310196576057933270931243896484375f,-0.1437000205844629618923136149533092975616455078125f,0.0836999893665311134327566833235323429107666015625f,-0.6475000044703487223074489520513452589511871337890625f,0.5107999995544554305837436913861893117427825927734375f,-0.57530002978891159326479964875034056603908538818359375f,-0.40799999811500331503566485480405390262603759765625f,0.690500002697110204508135211654007434844970703125f,-0.68129997889399618316019768826663494110107421875f,-1.034099970424174674832329401397146284580230712890625f,2.15010002782344855631890823133289813995361328125f,0.021500020161270949614618075429461896419525146484375f,0.8338999962121247389035261221579276025295257568359375f,-0.5094000158220530760644351175869815051555633544921875f,-1.06790004632473056744856876321136951446533203125f,-0.12330001548007130107720286105177365243434906005859375f,0.734100029954314781122093336307443678379058837890625f,-0.6746000149190425343448396233725361526012420654296875f,0.561699993628263616329832075280137360095977783203125f,-0.2018999804504211237343724860693328082561492919921875f,-0.1800000539422033085656948969699442386627197265625f,1.29470004043281061711923030088655650615692138671875f,-0.61669997053146374810239649377763271331787109375f,0.673899988128245031049345925566740334033966064453125f,-0.9950000362843274270829851957387290894985198974609375f,0.0314999906942243779184309460106305778026580810546875f,1.431400009399652173414096978376619517803192138671875f,-0.3581000142037871203370968942181207239627838134765625f,-0.43300001107901342489725493578589521348476409912109375f,0.8351999857604501809049679650343023240566253662109375f,-0.41520000554174163465148694740491919219493865966796875f,-0.839100008906423777688132759067229926586151123046875f,0.426600003281235817809147192747332155704498291015625f,-1.6496999551236621872618570705526508390903472900390625f,-0.16929996952116521669040594133548438549041748046875f,0.39460000851154308776358448085375130176544189453125f,1.209500006690621631832982529886066913604736328125f,0.1029000166714186814687082005548290908336639404296875f,0.037100024865567515774955609231255948543548583984375f,-0.683600016453862036058808371308259665966033935546875f,-0.764900020381807710379007403389550745487213134765625f,1.50910003367960410258774572866968810558319091796875f,0.50479999531507502297245082445442676544189453125f,0.3112000126868483818043387145735323429107666015625f,0.9447000318646434635638797772116959095001220703125f,-1.346800029274822119162990929908119142055511474609375f,0.27169996740221957765015758923254907131195068359375f,-0.032300001350045182135772847686894237995147705078125f,-1.5784000425398350131445113220252096652984619140625f,1.3930000382736322794841044014901854097843170166015625f,1.370099985988438628936592067475430667400360107421875f,-0.248600001496822375912643110495992004871368408203125f,0.200900045743583888224748079665005207061767578125f,0.5500999903656544898211677718791179358959197998046875f,-0.315700000751018450273477355949580669403076171875f,0.08929996186569244542852175072766840457916259765625f,0.9909000552058220279150191345252096652984619140625f,-0.90620003814920746521011096774600446224212646484375f,1.6000000214576726165205400320701301097869873046875f,0.12150001123920051160354205421754159033298492431640625f,-1.0855999762058257118724213796667754650115966796875f,0.7339999940097332586930178877082653343677520751953125f,-0.6779000121638178999461388229974545538425445556640625f,-1.059499988101423095798736540018580853939056396484375f,-0.25569999791979824976806412450969219207763671875f,-0.538300005291402872131811818690039217472076416015625f,0.37459998668134186772249449859373271465301513671875f,-0.95549996697157713043679905240423977375030517578125f,-0.31160003330707564117574293049983680248260498046875f,0.6657999327972523584406872032559476792812347412109375f,-1.209699995952844719937502304674126207828521728515625f,-1.6708000033542516060691696111462078988552093505859375f,1.0955999889463183816218361243954859673976898193359375f,-0.1394999643340708583849618662497960031032562255859375f,0.02990000853538532510356162674725055694580078125f,-0.364000026196241943665654616779647767543792724609375f,-0.87380004326701143924083226011134684085845947265625f,0.808600004011392314851036644540727138519287109375f,0.390900012886524539368338082567788660526275634765625f,-0.22180001564025875193664205653476528823375701904296875f,-2.09980001587867715073798535740934312343597412109375f,-0.468500053003430227960279808030463755130767822265625f,-0.328599936807155046381012653000652790069580078125f,0.156599976235627824650009642937220633029937744140625f,-1.71860003724098220345695153810083866119384765625f,-0.131200044053793707377053578966297209262847900390625f,0.998300020527839393480462604202330112457275390625f,0.08269999056607468812529759816243313252925872802734375f,-0.571700001823902681508116074837744235992431640625f,-0.40810003491640112205374180120998062193393707275390625f,1.086500000566243517852171862614341080188751220703125f,-1.57150001990050025568734781700186431407928466796875f,1.718899981087446349192759953439235687255859375f,-0.967799986035376758053416779148392379283905029296875f,1.667900063014030820340849459171295166015625f,-0.91610004443079284808248985427781008183956146240234375f,0.3744999642968167563594761304557323455810546875f,-1.14800003007799400478461393504403531551361083984375f,-1.0250000218302013355042845432762987911701202392578125f,-1.4513000068336732173435166259878315031528472900390625f,0.90630000482201567013618159762700088322162628173828125f,0.845600003921985088339852154604159295558929443359375f,-0.7581000100314618794783427802030928432941436767578125f,-0.3107001067727803711449041657033376395702362060546875f,-0.176500000640749998837009115959517657756805419921875f,0.3932999942645432067678257226361893117427825927734375f,-2.08850002856552574570514480001293122768402099609375f,0.1426999879211179322879843311966396868228912353515625f,0.74710000001788134937896757037378847599029541015625f,0.1892999646037807348619708136538974940776824951171875f,-0.7828000517122450929718979750759899616241455078125f,0.999100019150972240566943582962267100811004638671875f,0.66170000346302959570721213822253048419952392578125f,-0.11419997798204395422771995072253048419952392578125f,0.6059000012636186482950506615452468395233154296875f,-0.6083000194102525792771984924911521375179290771484375f,-0.518800002810359206506518603418953716754913330078125f,-0.32780000680387022526929285959340631961822509765625f,-0.71810001669973144711178747456870041787624359130859375f,0.8641000176608562011182357309735380113124847412109375f,0.02610001978799714184020785978646017611026763916015625f,-1.25920003179386252867288931156508624553680419921875f,0.1695000227838752149267520508146844804286956787109375f,0.68050001085549605139846107704215683043003082275390625f,-0.5165999754905694629769641323946416378021240234375f,0.10749999446794389346138132168562151491641998291015625f,0.069299988520145294756957810022868216037750244140625f,-0.745700013435631969826999920769594609737396240234375f,0.384499991342425406770644258358515799045562744140625f,0.236600020641088537587393147987313568592071533203125f,-1.080899998469650569177247234620153903961181640625f,-0.13240002529919170370931169600225985050201416015625f,0.277299982947111001152507014921866357326507568359375f,-0.64889999712482071547725581694976426661014556884765625f,-0.1582999739058317256734653710736893117427825927734375f,0.2818000434309244628394708342966623604297637939453125f,-0.11570002138912671707515755770145915448665618896484375f,0.8181000084295872998296772493631578981876373291015625f,-0.08479999121725578792307942421757616102695465087890625f,1.2204999732300638726201214012689888477325439453125f,-0.4067999993234872047054295762791298329830169677734375f,-0.8735999984234570359120652938145212829113006591796875f,0.71839999664425846503945649601519107818603515625f,0.573800029353052298830561994691379368305206298828125f,0.265699977784603802444252096393029205501079559326171875f,0.38259994463771584971567563115968368947505950927734375f,0.0956000084854664589073536262731067836284637451171875f,0.00709999105855825540789538763419841416180133819580078125f,-0.27849999250471590972466628954862244427204132080078125f,-0.3133000083833932247756592914811335504055023193359375f,-0.3763000256091355577581225588801316916942596435546875f,1.2355000232607127319539586096652783453464508056640625f,-0.492399988418817269320015839184634387493133544921875f,-0.058400018250942320463536816532723605632781982421875f,0.7743000242084259188146688757115043699741363525390625f,-0.189199949222802865644865732974722050130367279052734375f,2.406399998223781810935406610951758921146392822265625f,0.6544000435382122393690451644943095743656158447265625f,1.078400050195307269262912086560390889644622802734375f,-0.34960000527054069241472689100191928446292877197265625f,0.3583999951727687705016478503239341080188751220703125f,0.20230001667961483580171488938503898680210113525390625f,-0.4233999870330096282344811697839759290218353271484375f,-0.369600020469725176752717743511311709880828857421875f,0.47409998480379567098452753270976245403289794921875f,0.5068000430583958770824892781092785298824310302734375f,0.0707000280946499248813097437960095703601837158203125f,1.640299995758383477806319206138141453266143798828125f,-1.4441999779075391785454485216177999973297119140625f,1.5972000121392315730162181353080086410045623779296875f,-0.8605000178962944890059816316352225840091705322265625f,0.411699992492049915426832740195095539093017578125f,-0.22150000272691239278088914943509735167026519775390625f,-0.59210000854879607512515349299064837396144866943359375f,0.176400012007355666554531126166693866252899169921875f,0.8486000029683113954348527840920723974704742431640625f,0.07439997953772536920524771630880422890186309814453125f,-1.2818000058054923595562968330341391265392303466796875f,-0.7297999896526332630486422203830443322658538818359375f,-0.59729990542382027651768794385134242475032806396484375f,0.67160001669228019949287045164965093135833740234375f,-1.477399990169704135922756904619745910167694091796875f,-0.353999978363514156143310174229554831981658935546875f,0.4144000234961513484677197993732988834381103515625f,0.41230001707076979755584034137427806854248046875f,0.1673000242978333051979689116706140339374542236328125f,-0.1126999956324692409026511086267419159412384033203125f,-0.8349000400140884625699300158885307610034942626953125f,0.4164000174090265460336013347841799259185791015625f,1.034200050625205591359190293587744235992431640625f,-0.3230000050067900474459747783839702606201171875f,0.170699996057152247885824181139469146728515625f,0.7349000344634060155613042297773063182830810546875f,-0.9730000267252327805778122638002969324588775634765625f,0.33589997794330184177624687436036765575408935546875f,0.025900001338123868066531940712593495845794677734375f,0.839199993777274499962004483677446842193603515625f,-0.5721000334709884782569133676588535308837890625f,1.5790000322312121028289766400121152400970458984375f,-0.1885999796107407089351681861444376409053802490234375f,0.925900047904253842290245302137918770313262939453125f,0.229999991636722800336656291619874536991119384765625f,-0.5471000305652626849450825829990208148956298828125f,-0.07390001929029832439255187637172639369964599609375f,1.0581000536918647725315167917869985103607177734375f,-0.5191000096514819528437101325835101306438446044921875f,0.3213999819815160918778929044492542743682861328125f,-1.52589999485611915730487453402020037174224853515625f,-0.2134999786466362792225481825880706310272216796875f,-0.7942000194445253402619755433988757431507110595703125f,-0.4489000279143458005393085841205902397632598876953125f,-0.4252000314325095065015602813218720257282257080078125f,-1.2942000027552242880091171173262409865856170654296875f,-0.53499998170882445691631801309995353221893310546875f,0.9927000072926277507434633662342093884944915771484375f,-0.57179999429434591462495518499054014682769775390625f,-0.756600039863586371069459346472285687923431396484375f,0.0389999545589088114638798288069665431976318359375f,-0.9686000027820467206396415349445305764675140380859375f,-0.9310000158250328894382619182579219341278076171875f,-0.3478999830320479436096547942725010216236114501953125f,0.0139000010177496857721735068480484187602996826171875f,-1.1972999665185806161815662562730722129344940185546875f,-1.2720000341087580597587702868622727692127227783203125f,-0.625500012159347473783554960391484200954437255859375f,-0.3813000303030011917115871256100945174694061279296875f,-0.6263999857068061061227126629091799259185791015625f,0.063199976760148768306635247427038848400115966796875f,0.205699979665874277401371728046797215938568115234375f,0.9075000353902578087428310027462430298328399658203125f,-0.03650003536045520746711190440692007541656494140625f,-0.3674999982118603014669133699499070644378662109375f,0.05040000363290297880070056635304354131221771240234375f,0.50400002157688117421230344916693866252899169921875f,-1.46970000653266996692991597228683531284332275390625f,1.354800024968385141477256183861754834651947021484375f,-0.22390001200735609732106468072743155062198638916015625f,-0.89970003693103794972785181016661226749420166015625f,-1.6909000291287892281388849369250237941741943359375f,0.8020000489354135453368144226260483264923095703125f,0.946399976691603139755670781596563756465911865234375f,-0.0637000784754757631844768184237182140350341796875f,0.503099985742569089097742107696831226348876953125f,-0.10559998335838305383305169016239233314990997314453125f,0.2751999609500170862474988098256289958953857421875f,-0.064900007082522026546911320110666565597057342529296875f,-0.757499998882412928224994175252504646778106689453125f,0.3159999935477968424635264454991556704044342041015625f,-0.52399997717142099418197176419198513031005859375f,0.735200049984455095142266145558096468448638916015625f,1.64299996745586440738406963646411895751953125f,-0.079500009968876794719250256093800999224185943603515625f,1.02319999203383904529118808568455278873443603515625f,-0.2086000166028740832047105868696235120296478271484375f,-0.1188000294089324171409316477365791797637939453125f,-0.70500001482665519692005773322307504713535308837890625f,0.7887000111967328308537616976536810398101806640625f,0.14010000789314513980343690491281449794769287109375f,-0.4200000119581817070724127916037105023860931396484375f,0.8147000551104550680747706792317330837249755859375f,-0.72490004325509094229573747725225985050201416015625f,-0.90080004645586075895380417932756245136260986328125f,-0.062199967193603100668042316101491451263427734375f,-1.3261000334411858769811942693195305764675140380859375f,0.5854999911487102082219280418939888477325439453125f,0.421200008663535019337587073096074163913726806640625f,1.2082000295102595099905329334433190524578094482421875f,0.42659999229162881562160691828466951847076416015625f,0.3487000087007878779132852287148125469684600830078125f,0.63930000979155299223322117541101761162281036376953125f,-0.22070000320971006058101693270145915448665618896484375f,0.680599992150068278107255537179298698902130126953125f,1.4475999881118528644918796999263577163219451904296875f,-0.9082000558853149385640790569595992565155029296875f,-0.179299978610873100848266403772868216037750244140625f,0.10730003823041911470426157393376342952251434326171875f,0.045000008791685086606548793497495353221893310546875f,-0.217099990332126235870191521826200187206268310546875f,-0.4041000023871659241336828927160240709781646728515625f,-0.66459999137520797685141360489069484174251556396484375f,-0.32950002598762484495864555356092751026153564453125f,0.723899995318055378135113642201758921146392822265625f,0.534299993586540011847318965010344982147216796875f,0.315900026446580994843316148035228252410888671875f,0.68509995033740977987690712325274944305419921875f,0.010499990478157972262351904646493494510650634765625f,-0.5805000500082968528658966533839702606201171875f,-1.1597000211358070576039835941628552973270416259765625f,-0.1954000054955482568175284541212022304534912109375f,0.750300013062358051030287242610938847064971923828125f,1.106800001744926209568120611947961151599884033203125f,1.117000008627771823199736900278367102146148681640625f,-0.382999992340803174784014117904007434844970703125f,-0.42609997376203434527042190893553197383880615234375f,0.548699995326995093591904151253402233123779296875f,-1.35180006675124264603482515667565166950225830078125f,1.724500049829483572239041677676141262054443359375f,-0.682499974966047862068307949812151491641998291015625f,0.372499999105929902043499168939888477325439453125f,1.9103000523269173527296516112983226776123046875f,-0.86600000777840602239621148328296840190887451171875f,-1.0384000248447058378786778121138922870159149169921875f,1.17120000188350648073765114531852304935455322265625f,0.3390999998539692139587486963137052953243255615234375f,-0.7005999925971024833160072375903837382793426513671875f,-0.61299997773766534692185814492404460906982421875f,1.6054000264316794499563911813311278820037841796875f,0.40590001422762878036110123503021895885467529296875f,2.021300038722157577097959801903925836086273193359375f,0.9976000194787975061672113952226936817169189453125f,0.16550001767277677799938828684389591217041015625f,0.485900006479024870742478015017695724964141845703125f,-0.830800016131997498547434588545002043247222900390625f,-1.3611000236809258634451680336496792733669281005859375f,-0.8428000256538386025795261957682669162750244140625f,-1.034200003984570681581089957035146653652191162109375f,0.1483999550700190184215898625552654266357421875f,0.7497000056385993982388527001603506505489349365234375f,-0.31220001055598256556322667165659368038177490234375f,-1.631100013697146966507034449023194611072540283203125f,-1.252900004303455716581083834171295166015625f,0.6367000379592184788890563140739686787128448486328125f,-1.3301999733924869673273860826157033443450927734375f,-1.257200031790137728648915071971714496612548828125f,-0.8170000200271605450552669935859739780426025390625f,-0.662900032764673685647949241683818399906158447265625f,-0.5083000199317933009979242342524230480194091796875f,0.820900047308207536644886204157955944538116455078125f,-0.0522000631570822548610522062517702579498291015625f,-0.8395000109374517460736342400196008384227752685546875f,0.8002999992787831473606274812482297420501708984375f,0.600600022846460745995500474236905574798583984375f,1.34619998415112451795039305579848587512969970703125f,0.948599983745812114221962474402971565723419189453125f,-0.8075999628543844011119290371425449848175048828125f,0.4679000208437447128062558476813137531280517578125f,0.5865999498605720674504482303746044635772705078125f,1.1063000598073016167433024747879244387149810791015625f,0.6502000212252145860958307821420021355152130126953125f,0.8187000401794914949960002559237182140350341796875f,-0.286800013628601302428933195187710225582122802734375f,-0.0751999699652190489729264299967326223850250244140625f,1.4646000331729656096513281227089464664459228515625f,2.21510002707838982161092644673772156238555908203125f,0.77190000635981537158158971578814089298248291015625f,-0.23900005942583124607381250825710594654083251953125f,-0.07339999113082829040877186344005167484283447265625f,1.6809000778555880373232866986654698848724365234375f,0.4065999913603073334655846338137052953243255615234375f,0.3165000045895578484334009772283025085926055908203125f,-0.0594000148907303238132726619369350373744964599609375f,0.67320000157058235146223523770458996295928955078125f,0.5057000168442726817374932579696178436279296875f,0.165100004428625102281813497029361315071582794189453125f,0.291700000482797872081164314295165240764617919921875f,0.099199978056550008886915748007595539093017578125f,-1.008599981808662615634375470108352601528167724609375f,-0.37830000625997783902931814736803062260150909423828125f,-0.1320999954357742200983238944900222122669219970703125f,0.09200000721216194943252730809035710990428924560546875f,0.04519999958127751682468442595563828945159912109375f,-0.38950000983849164537531351015786640346050262451171875f,0.21240000747367704381218800335773266851902008056640625f,-0.161400004780292594119828208931721746921539306640625f,0.03050002259016082462039776146411895751953125f,-0.85660002906024457214329004273167811334133148193359375f,0.422900011605024328531499122618697583675384521484375f,0.36280001276433448964553463156335055828094482421875f,-0.97289999141395089310435650986619293689727783203125f,1.31720004676580426661303135915659368038177490234375f,-1.3440000225454566962213220904232002794742584228515625f,-0.774600029857456906512425121036358177661895751953125f,0.9530999931186439066976845424505881965160369873046875f,0.3606000199221079949296608901931904256343841552734375f,-1.137400041280687279510175358154810965061187744140625f,-1.5615000358819965953216524212621152400970458984375f,-0.170500028327107866488177023711614310741424560546875f,-0.239100001567602138408119571977294981479644775390625f,-0.19769998941123478886083830730058252811431884765625f,0.598800017040967968995346382143907248973846435546875f,-0.435499983549118230286012476426549255847930908203125f,0.1427999910086390489283303395495750010013580322265625f,-0.84880001078248046297858309117145836353302001953125f,0.21550001798942697295302650672965683043003082275390625f,1.071800007724017067545219106250442564487457275390625f,0.280999992225319183969389769117697142064571380615234375f,-0.05730002543404710824148651227005757391452789306640625f,-0.02400000216811892794765981307136826217174530029296875f,-0.391499991141259773286265044589526951313018798828125f,-0.554600005233287784989215651876293122768402099609375f,0.136599997767806069504104016232304275035858154296875f,-0.05120000441670402580029985983856022357940673828125f,-0.3321999792635441650645589106716215610504150390625f,0.70430001467168334983171007479541003704071044921875f,0.405300008293985936091985422535799443721771240234375f,1.060500015810131646531999649596400558948516845703125f,1.9136999588191514476420707069337368011474609375f,0.3544000249862675477885431973845697939395904541015625f,0.70370002389997299285795406831311993300914764404296875f,0.06819999669045195833660955031518824398517608642578125f,-0.00859999985769388519685207938891835510730743408203125f,0.69130001689195641478846710015204735100269317626953125f,-0.245300003898143881286841860855929553508758544921875f,-0.3496000184953214073857452603988349437713623046875f,0.970000005885958582751982248737476766109466552734375f,0.34320001147985446099397677244269289076328277587890625f,-0.384399979686736781303579846280626952648162841796875f,-1.53330005614161546390050716581754386425018310546875f,-0.049499962583184231590394119848497211933135986328125f,-1.027999985337256116935122918221168220043182373046875f,0.1415999994814394202791163479560054838657379150390625f,0.22960000437647110071992528901319019496440887451171875f,0.086600001884251931461022877556388266384601593017578125f,-0.24820000909566874991440954545396380126476287841796875f,-1.04120002132952205187166327959857881069183349609375f,0.038300016057491337306828427244909107685089111328125f,0.8978000091135502902517373513546772301197052001953125f,-0.91320005663037340237764283301657997071743011474609375f,0.20619997252821864464067402877844870090484619140625f,-1.2204999898076049902329032192938029766082763671875f,0.0514000389784576849905306517030112445354461669921875f,-0.531999987244606842295979731716215610504150390625f,-0.0930000212490558197941936668939888477325439453125f,0.164400007998942943032716357265599071979522705078125f,0.432399978733062795299701974727213382720947265625f,-0.4976999760746954049039914025343023240566253662109375f,0.055799952057004276184670743532478809356689453125f,0.231100019992888261466390531495562754571437835693359375f,-0.136300006088614811261550130438990890979766845703125f,-0.091900012320280222155588489840738475322723388671875f,0.2421000055313111687382843228988349437713623046875f,0.473500034675002456907577652600593864917755126953125f,0.26139996693134293082039221189916133880615234375f,-0.8834000034987925520368889920064248144626617431640625f,0.359399946382641832798299219575710594654083251953125f,-0.58440002260208157025545006035827100276947021484375f,1.286000021934508907861527404747903347015380859375f,-0.83730002385079849158699971667374484241008758544921875f,2.07870001969039464029265218414366245269775390625f,0.185100000070035353161301827640272676944732666015625f,-0.0897999775454396687024427592405118048191070556640625f,-0.04980000351071378617007212596945464611053466796875f,0.3971000037431711149338298127986490726470947265625f,-0.80240002894997619620198747725225985050201416015625f,1.0486000256180769696356946951709687709808349609375f,-0.140100002826749892026469979100511409342288970947265625f,1.5619999961853026082536644025822170078754425048828125f,0.509499997675419535880791954696178436279296875f,2.00610000863075210730812614201568067073822021484375f,1.0482999961271877165103205697960220277309417724609375f,1.695800022651254845840185225824825465679168701171875f,-0.33030004990547967569369802731671370565891265869140625f,-0.3953000193208457968552238526172004640102386474609375f,0.31800003406405519257305058999918401241302490234375f,1.1362000320583580670330547945923171937465667724609375f,0.0528999819517128599954958190210163593292236328125f,1.40210002162456515151234270888380706310272216796875f,0.784800012898445142894843229441903531551361083984375f,-1.2948999764978881277244227021583355963230133056640625f,-0.0323000077575443800270704741706140339374542236328125f,-0.7289999750107529763454294879920780658721923828125f,-0.112499995268881269883110007867799140512943267822265625f,-1.19659998115301124954612532746978104114532470703125f,-0.710599998259544118894837083644233644008636474609375f,-0.4677000045359136493061669170856475830078125f,-0.247900005570054038361860193617758341133594512939453125f,-0.731999966233968901718753841123543679714202880859375f,0.5900999759674065980874502201913855969905853271484375f,0.212900023674964966602374261128716170787811279296875f,-0.758400018548966325937499277642928063869476318359375f,-0.069499999631196185401194043151917867362499237060546875f,-2.008899935974181261144622112624347209930419921875f,1.7602999748967587123615885502658784389495849609375f,0.4636000509500497468451385429943911731243133544921875f,-0.6027999745428560540716489413171075284481048583984375f,1.086200028668343531279560920665971934795379638671875f,0.488500007182359308188779323245398700237274169921875f,-1.4290999688223007257903418576461263000965118408203125f,0.6340999934464690568347577936947345733642578125f,1.5620999419778576200457109734998084604740142822265625f,0.770700036513805475379967901972122490406036376953125f,0.46329999698400481378257609321735799312591552734375f,0.38029999838471439943532459437847137451171875f,1.5281000500410806264284246935858391225337982177734375f,0.76090004179477688239785493351519107818603515625f,-0.13839999880492737105441847234033048152923583984375f,1.16460002833008768874378802138380706310272216796875f,-0.548000018045306180880515967146493494510650634765625f,-1.0915000260472300652736521442420780658721923828125f,0.38020000855922742033499162062071263790130615234375f,2.148900062894821161307845613919198513031005859375f,-0.9300999932527542757298988362890668213367462158203125f,0.8702999623611564761205272588995285332202911376953125f,0.7039000481054193958385667428956367075443267822265625f,-0.7128000159680834002529081772081553936004638671875f,0.81480002394393036269093499868176877498626708984375f,-0.53949999797344272423060829169116914272308349609375f,-0.187300032698363150274190047639422118663787841796875f,0.99090002376437258391206341912038624286651611328125f,0.19180004158690644455731444395496509969234466552734375f,-0.7819000276684757722023277892731130123138427734375f,0.83010002068951693754428333704709075391292572021484375f,-0.195299992573261693706854202901013195514678955078125f,1.212000028390437744718610701966099441051483154296875f,-2.034500053852796863651519743143580853939056396484375f,0.66780002274811300910783984363661147654056549072265625f,0.628300022390484880219219121499918401241302490234375f,-0.04079994777291962027021554604289121925830841064453125f,0.2331000092864037309681179976905696094036102294921875f,0.8768000164225693282560314401052892208099365234375f,-0.804800030183792092230987691436894237995147705078125f,-1.7711000412270418991056430968455970287322998046875f,0.9597000232964756349929302814416587352752685546875f,1.0427000839218500960470237259869463741779327392578125f,-0.9299999918788668384905804487061686813831329345703125f,-0.7653999996840952224630427735974080860614776611328125f,1.523500052444636931880950214690528810024261474609375f,0.5779000306785118112884447327814996242523193359375f,2.183800034624338248789854333153925836086273193359375f,0.00669999608695481807529858997440896928310394287109375f,-0.09599997127056081325235936674289405345916748046875f,0.09700001034140637301561582717113196849822998046875f,-0.34769999708533294580092842807061970233917236328125f,0.344000006377697342685451076249592006206512451171875f,-0.526200001585483079225014080293476581573486328125f,0.64880000206530130668625133694149553775787353515625f,0.5480000714659691851693423814140260219573974609375f,-0.6945000140666959254076573415659368038177490234375f,-0.1291999895304447587562890475965104997158050537109375f,-0.785599976373463793066775906481780111789703369140625f,0.372399958914516826524732096004299819469451904296875f,0.085900014358014420423614865285344421863555908203125f,-0.12640000761151259922598910634405910968780517578125f,-0.199799996171891702800849088816903531551361083984375f,0.67379998455643619337251948309130966663360595703125f,-0.21569998048543903568230462042265571653842926025390625f,0.815600045049190924828508286736905574798583984375f,-0.5523999967448409709191992078558541834354400634765625f,1.167899988508224851102568209171295166015625f,0.2600000308826564410225046231062151491641998291015625f,-0.6236000131756067599297921333345584571361541748046875f,-0.195400031684339448023735030801617540419101715087890625f,-0.128399952238798409354103569057770073413848876953125f,-1.324400025619566267920390600920654833316802978515625f,0.353799986121058562815733239403925836086273193359375f,-0.7791000377029178025622968561947345733642578125f,0.2466000236213208296476295799948275089263916015625f,-0.3982000262320035677277019203756935894489288330078125f,-0.79630000884532936566984062665142118930816650390625f,1.680799989570677777095397686935029923915863037109375f,-1.2189000135719771922282461673603393137454986572265625f,-1.562700002226233397806254288298077881336212158203125f,0.4944000230491163438983903688495047390460968017578125f,0.9368999893911180709693553581018932163715362548828125f,0.00830000234842298567627949523739516735076904296875f,-0.4384000120669606648249327918165363371372222900390625f,1.03650000309944090304270503111183643341064453125f,-0.03829995988011347662904881872236728668212890625f,0.40189998892545730768688372336328029632568359375f,1.22970003134310257308925429242663085460662841796875f,1.32200005072355253332716529257595539093017578125f,-1.15069999099075825910176718025468289852142333984375f,1.41589995790123968077978133806027472019195556640625f,1.191800048944353651592109599732793867588043212890625f,0.8667999907553198823961793095804750919342041015625f,0.252400048097968454641204516519792377948760986328125f,-0.40840003196001095631118005258031189441680908203125f,0.164099994321912301398214140135678462684154510498046875f,0.3581000179290771967544060316868126392364501953125f,1.02800002694875036723942685057409107685089111328125f,-1.40620001529455151967340498231351375579833984375f,0.6195999908722933380289532578899525105953216552734375f,0.1306000021338462602216168306767940521240234375f,0.707999988522379997135658413753844797611236572265625f,-0.0303999390363696164740758831612765789031982421875f,1.29039999914392833346710176556371152400970458984375f,-1.56289999357461883988662521005608141422271728515625f,1.584500031780451845264678922831080853939056396484375f,0.6848000018715858505657934074406512081623077392578125f,-0.21249999005347552927247534171328879892826080322265625f,-0.635199989858269642439836388803087174892425537109375f,-1.74399999333918032107249018736183643341064453125f,1.31780000747442205266679593478329479694366455078125f,-0.328599954055249998674526068498380482196807861328125f,0.15469998544752616709274661843664944171905517578125f,-1.14829998644143405073236863245256245136260986328125f,-0.35809998134672704761527484151883982121944427490234375f,-1.1181999815657739372198875571484677493572235107421875f,1.367499979063868220663380270707421004772186279296875f,-1.4333999795451755066011401140713132917881011962890625f,-0.11159998204708099223125827847979962825775146484375f,-0.2933000070422890903643065030337311327457427978515625f,1.0389000095486640162789626629091799259185791015625f,-0.482600000628829572946187909110449254512786865234375f,0.04879996145963705345138805569149553775787353515625f,-0.1742999906063074089246356379589997231960296630859375f,0.233500034153461655250794137828052043914794921875f,-0.176000035271048016483064202475361526012420654296875f,-2.121699978503585271027986891567707061767578125f,-0.9062000167846679943295384873636066913604736328125f,-0.814100027346610577438923428417183458805084228515625f,-0.003799998340010546371559030376374721527099609375f,-0.35780000061988825876824193983338773250579833984375f,0.137899961332230869270887296806904487311840057373046875f,-0.2040000057816513390207546763122081756591796875f,0.86540001788213893885171046349569223821163177490234375f,-1.60990002075433746853150296374224126338958740234375f,-0.5703000013649457411446519472519867122173309326171875f,0.8863999424934387860730566899292171001434326171875f,-1.440499997798353692957107341499067842960357666015625f,0.9748999992221591970320559994434006512165069580078125f,-0.7368000054515897456752782090916298329830169677734375f,0.6476000231295824249855286325328052043914794921875f,0.6017000229276716627424548278213478624820709228515625f,-0.0710999911963938746595204065670259296894073486328125f,1.006500063039361680949923538719303905963897705078125f,0.014799996656179370546624340931884944438934326171875f,-1.8884000119552017959989598239189945161342620849609375f,1.308500016719102543305552899255417287349700927734375f,1.752999935418368071537997820996679365634918212890625f,-0.1778999760657544726427659043110907077789306640625f,0.2994999700710179180163095224997960031032562255859375f,-0.5664000102937218894538773383828811347484588623046875f,0.54879996686130783078993999879457987844944000244140625f,-0.73169999943971664180253355880267918109893798828125f,0.049400001032650298071757788420654833316802978515625f,1.12649997419118808039684154209680855274200439453125f,0.0962999960303305702069565086276270449161529541015625f,-0.96110000028610187428057543002068996429443359375f,0.507300005596876157909491666941903531551361083984375f,-1.530500049412250263003443251363933086395263671875f,0.070199975851178209040881483815610408782958984375f,-0.30889994517564733911285657086409628391265869140625f,-0.3943000222712746971609476531739346683025360107421875f,-1.06449996191263185352227083058096468448638916015625f,0.3289999789595603463254747111932374536991119384765625f,-0.7256999497890479489115023170597851276397705078125f,0.50870005120635131135031770099885761737823486328125f,0.03759999831914928591913849231787025928497314453125f,-0.7993000318825249639331786966067738831043243408203125f,0.2926000132203103731853843783028423786163330078125f,-0.23249999780207863953052083161310292780399322509765625f,-0.3383000100970274814926597173325717449188232421875f,-1.2064000122122460201268268065177835524082183837890625f,2.02770000885725121264613335370086133480072021484375f,-0.9700999692060052748132648048340342938899993896484375f,0.67799997162818836926589938229881227016448974609375f,-0.1917999812185759989091593524790368974208831787109375f,-0.0761000455856315483060825499705970287322998046875f,-0.2096999980762606075046505793579854071140289306640625f,-0.039499972864985632980960872373543679714202880859375f,1.631400038158893561757167844916693866252899169921875f,-0.320700045454502369324245591997168958187103271484375f,-0.9489000404685736800303175186854787170886993408203125f,0.60230001185536341523629744187928736209869384765625f,1.85089994989186568119521325570531189441680908203125f,-1.39930003150999482386396266520023345947265625f,0.17249995958060104772613385648583061993122100830078125f,-0.8755999734491108910106049734167754650115966796875f,-1.033400007820129307134493501507677137851715087890625f,0.8305999849230059073335041830432601273059844970703125f,-1.040499987050889973971834479016251862049102783203125f,2.14049998380988881763187237083911895751953125f,0.242300021168589285736061356146819889545440673828125f,0.43039998716712002391204805462621152400970458984375f,0.15729999233484281973005636245943605899810791015625f,-0.88299999219179081677566500729881227016448974609375f,0.273799993571639532774497638456523418426513671875f,0.9635000247657306005066857323981821537017822265625f,-0.0677000037163499879255823543644510209560394287109375f,0.978499974362551316886538188555277884006500244140625f,0.87250003401190101559592449120827950537204742431640625f,-0.5174999751150604065941251974436454474925994873046875f,-0.7270000327676535700760496183647774159908294677734375f,-1.1416999795094138203666034314665012061595916748046875f,0.75949998405948282442778918266412802040576934814453125f,-1.52340002182722056289776446647010743618011474609375f,-0.352099992939829409266394577571190893650054931640625f,0.2376999911248685037890027160756289958953857421875f,0.746300034531205636056938601541332900524139404296875f,-0.508299975973368844961441936902701854705810546875f,1.159100019486249077971251608687452971935272216796875f,-1.47810003085583385740164885646663606166839599609375f,-0.558400022125244532844590139575302600860595703125f,0.565099975036084334334418599610216915607452392578125f,-0.5996999896757302384031618203152902424335479736328125f,0.07070004541724894675525092679890803992748260498046875f,0.3904000167086716377440325231873430311679840087890625f,1.58159998644292354441631687222979962825775146484375f,0.16679998092055381420095727662555873394012451171875f,0.1078999882102014140627943561412394046783447265625f,-0.245299973388016123720944960950873792171478271484375f,-0.62379997241199003354950036737136542797088623046875f,-0.18410001363754313619125468903803266584873199462890625f,1.2285000290125613897629364146268926560878753662109375f,1.2653000395491726948193900170736014842987060546875f,-0.0180000237077477809322090251953341066837310791015625f,0.26759995782524337304408845739089883863925933837890625f,-1.82070001177787776924787976895458996295928955078125f,0.087799973276256881860035719000734388828277587890625f,-2.00840005550384592680757123162038624286651611328125f,0.1489999964684243227708293488831259310245513916015625f,-0.5595000420808791119497982435859739780426025390625f,0.27630002121329244602065955405123531818389892578125f,2.44270000259876240988887730054557323455810546875f,-0.300699977132678153424194533727131783962249755859375f,0.870599998110532791173454825184307992458343505859375f,-1.0166999885991214735980747718713246285915374755859375f,0.7574999916180968906331827383837662637233734130859375f,-0.654000007160008056672495513339526951313018798828125f,0.583900015173851816285832683206535875797271728515625f,-0.088699998381734257435482504661194980144500732421875f,0.295999979354441278278642357690841890871524810791015625f,-1.52470000578761055720633521559648215770721435546875f,-0.15540002613365633266084842034615576267242431640625f,0.7624000563681132547344532213173806667327880859375f,0.6795000353753566368908423100947402417659759521484375f,0.24370000516176215654695624834857881069183349609375f,-0.749999963417648185526331872097216546535491943359375f,1.11610009714365077115871827118098735809326171875f,-0.38199999760091341016732258140109479427337646484375f,1.1760000019669529791599416057579219341278076171875f,1.14110000416040424653374429908581078052520751953125f,0.68669999683201321527548088852199725806713104248046875f,0.78930005348920762031639242195524275302886962890625f,0.824400020739436190098103907075710594654083251953125f,0.13189997372627271232659040833823382854461669921875f,-1.36300000120699404959623279864899814128875732421875f,0.221299997520446822818485088646411895751953125f,1.9203000308692452779979475963045842945575714111328125f,0.682500028461217933539728619507513940334320068359375f,-0.222600015082955682288456955575384199619293212890625f,1.310599983432889192869197358959354460239410400390625f,1.13710003455132291350082596181891858577728271484375f,0.7370999877989294901681205374188721179962158203125f,-0.5427000766575356127674467643373645842075347900390625f,0.64370007966756848105660537839867174625396728515625f,-1.47280003712773321922213654033839702606201171875f,0.7412000208079820851025942829437553882598876953125f,1.22580004943609299772333542932756245136260986328125f,-0.16920005457401288140317774377763271331787109375f,-1.15840006571114173539172043092548847198486328125f,-0.494899962043761743046843548654578626155853271484375f,-0.30119997580647372981133003122522495687007904052734375f,-0.8255000483691699031396638019941747188568115234375f,-0.898100033910572914663816845859400928020477294921875f,-0.96990002962052856982921866801916621625423431396484375f,-1.41230001632571156022777358884923160076141357421875f,0.66530000933706734400630011805333197116851806640625f,0.0439999940097326458499082946218550205230712890625f,1.02750002153217767641990576521493494510650634765625f,0.63309998052716220939828417613171041011810302734375f,-0.935800052192807374495941985514946281909942626953125f,-1.950100041085481183955607775715179741382598876953125f,0.14110000736415440059090542490594089031219482421875f,-0.1458999778315421824714803733513690531253814697265625f,-1.2264000157698988768828485262929461896419525146484375f,-0.49360001112669726897053124048397876322269439697265625f,1.32229999516606344656111104995943605899810791015625f,0.05870002959966702515970382592058740556240081787109375f,0.09499998867511738609437088598497211933135986328125f,-0.9865000055581329974074833444319665431976318359375f,0.36280000002384138468869423377327620983123779296875f,-0.545599986338614950653891355614177882671356201171875f,-0.138899986600875990916392765939235687255859375f,1.213200015577674317768241962767206132411956787109375f,-0.6805999464035039636655710637569427490234375f,-0.142000002145767467709447373636066913604736328125f,0.162700001034140395717031424283050000667572021484375f,-0.58500004053115894908643213057075627148151397705078125f,1.80650004452466905746632619411684572696685791015625f,0.3041000286877160174725531760486774146556854248046875f,-0.195400037235021528658762690611183643341064453125f,-0.541800045144558595211492502130568027496337890625f,1.825900024658441367364503093995153903961181640625f,0.51189998996853791624062068876810371875762939453125f,0.925500037193298652482553734444081783294677734375f,-0.7447000433385380091522165457718074321746826171875f,0.111900038099289389492696500383317470550537109375f,0.753800015699862768769889953546226024627685546875f,-0.930500005453824829970699283876456320285797119140625f,-0.79080001238808017571813024915172718465328216552734375f,1.323200039568543928680810495279729366302490234375f,0.6372999908447258832211446133442223072052001953125f,1.02860002286136165139396325685083866119384765625f,0.76490001062154799882364386576227843761444091796875f,0.618599991047382058440007313038222491741180419921875f,-0.6150000499188905678238370455801486968994140625f,-0.478600021669268738122582362848334014415740966796875f,-0.4152000264406208174250423326157033443450927734375f,1.064500026583671488111804137588478624820709228515625f,0.58619998936653150423126135137863457202911376953125f,0.046399991145729568842170920106582343578338623046875f,0.37550002340972443715116924067842774093151092529296875f,0.2298999987006187240012877737171947956085205078125f,0.8085999966353174528421732247807085514068603515625f,0.2471999785184859632636289461515843868255615234375f,-0.3206999792933462600785787799395620822906494140625f,1.184099976682663424298880272544920444488525390625f,0.89770001466870308348688922706060111522674560546875f,0.983300001454352923246915452182292938232421875f,0.135600010553002192636995459906756877899169921875f,1.126899926936626883389180875383317470550537109375f,0.50510000070333516131171336382976733148097991943359375f,1.31869998400211319022901079733856022357940673828125f,0.9389000599145898018349498670431785285472869873046875f,0.2451999733924861146761031704954802989959716796875f,-0.55190000367760649879755874280817806720733642578125f,-0.189899979031086019176655099727213382720947265625f,0.01100002238154473843678715638816356658935546875f,-0.009600031757354798855885746888816356658935546875f,-1.5416000692188749354727406171150505542755126953125f,1.38570004158020054774169693700969219207763671875f,-0.743600008928775313421510873013176023960113525390625f,0.364800028693675670865559368394315242767333984375f,-0.4913999834716313852567282083327881991863250732421875f,0.1790999970972535493274335749447345733642578125f,-1.720300074678660084970260868431068956851959228515625f,0.7960000096410515535438889855868183076381683349609375f,-0.18870000188350655179192472132854163646697998046875f,0.5372000034034247750724944125977344810962677001953125f,-0.034300015193224453469156287610530853271484375f,0.009400006583332842780009741545654833316802978515625f,-0.495899996644258322930909343995153903961181640625f,2.04100001791119556315834415727294981479644775390625f,-0.4165999978423116090198163874447345733642578125f,-0.45599997961521143707841474679298698902130126953125f,-0.753599979722500368239934687153436243534088134765625f,0.954600030639767282991670072078704833984375f,-1.31980003793239664133807309553958475589752197265625f,-0.13449999111890775793654029257595539093017578125f,0.942299998295306640017088284366764128208160400390625f,0.1328999753952029294623571331612765789031982421875f,-0.286399989059567428029140501166693866252899169921875f,0.7663999559044842868615887709893286228179931640625f,-0.237200003924965319157536214333958923816680908203125f,-0.123099960112572404113961965776979923248291015625f,-0.5983000173240899943749582234886474907398223876953125f,-1.921099954167008849736930642393417656421661376953125f,-0.51510000122487531459825049751088954508304595947265625f,-0.88930000625252692003641641349531710147857666015625f,0.576899976333975939013498646090738475322723388671875f,-1.0902000130295750324194159475155174732208251953125f,-0.98720002784132976358932864968664944171905517578125f,-0.55010000427961358582251705229282379150390625f,0.715900080388786275165102779283188283443450927734375f,-1.0167000855684289462033120798878371715545654296875f,-1.489400065517425986172384000383317470550537109375f,1.3011000202536582293078026850707828998565673828125f,1.814000015467403859048545200494118034839630126953125f,-0.988499983936547810259298785240389406681060791015625f,0.8061000274062160020349665501271374523639678955078125f,-0.546500001683831282406345053459517657756805419921875f,0.876000029087066423016949556767940521240234375f,-0.46930000610351552126076057902537286281585693359375f,0.059600046509505233416348346509039402008056640625f,0.4519000333309179939789146374096162617206573486328125f,0.246200020956993004261903479346074163913726806640625f,-0.32229999024867961310292230336926877498626708984375f,0.62670001448988887915447776322253048419952392578125f,0.210700019302964225431651357212103903293609619140625f,1.2544999973773958146239237976260483264923095703125f,-0.036300000128150156086803690413944423198699951171875f,0.346799990308284566964402984012849628925323486328125f,-1.2036999956995246829904999685822986066341400146484375f,-1.0221999993801116346503476961515843868255615234375f,1.0081999900221827459745327359996736049652099609375f,0.0005000033676623161227325908839702606201171875f,-1.208799991559981634736686828546226024627685546875f,-0.8657000240713357275268435842008329927921295166015625f,-0.4891000011205672848291214904747903347015380859375f,-0.07299998049437927960525485104881227016448974609375f,-0.962099964702129994975621229968965053558349609375f,0.3618000024527312508126897228066809475421905517578125f,-1.566399947559833805144080542959272861480712890625f,-0.49599998982250692591833285405300557613372802734375f,-0.0811999798297886599129924434237182140350341796875f,0.228500037878751616204908714280463755130767822265625f,1.2850000500679019665994928800500929355621337890625f,0.63990001300573329245935383369214832782745361328125f,-0.8483000130772584412852665991522371768951416015625f,1.450300068271161091359999772976152598857879638671875f,-0.2552000329971324532607468427158892154693603515625f,-0.770600055480004098029667147784493863582611083984375f,1.2842000316262254244747964548878371715545654296875f,1.52990003073811475786669689114205539226531982421875f,-1.7175000026822093701639460050500929355621337890625f,0.23349999496340689830731207621283829212188720703125f,0.3549999642372139163626343361102044582366943359375f,-0.46340002376437183784219087101519107818603515625f,-0.87090003553629014021453258465044200420379638671875f,0.22249999940395337461040980997495353221893310546875f,-0.559399992203712059790632338263094425201416015625f,0.43200003090500871394397108815610408782958984375f,-0.12499999731779087852601151098497211933135986328125f,0.366099970930813700675798827433027327060699462890625f,0.3623999981790781976798143659834749996662139892578125f,0.640999984681606793657238085870631039142608642578125f,0.129000031337142218035296536982059478759765625f,1.1865999883055682317944956594146788120269775390625f,1.2150999654620882051148100799764506518840789794921875f,0.34799998846650215256204319302923977375030517578125f,1.5448999732196331269307165712234564125537872314453125f,-0.17270001720190031324619894803618080914020538330078125f,0.88359999670982336539282187004573643207550048828125f,-0.81459998109340625571661576032056473195552825927734375f,-0.2996999880552291273261289461515843868255615234375f,1.063200017216801906982936998247168958187103271484375f,0.39640002564191778589020032086409628391265869140625f,-0.25120003313869243566358591124298982322216033935546875f,1.4598000057831399534080674129654653370380401611328125f,-0.0527999865382910993361065266071818768978118896484375f,-0.53929998084604680030906820320524275302886962890625f,0.636099996411800372442257867078296840190887451171875f,-0.55519997104555418587779058725573122501373291015625f,-0.7819000167906284826102591978269629180431365966796875f,0.04679999150037783550715175806544721126556396484375f,0.3690999978423119554094000704935751855373382568359375f,0.5400999709010125826580406283028423786163330078125f,0.325099983417987292710904512205161154270172119140625f,-0.9340999747455125401529585360549390316009521484375f,-0.93820001550316778349269952741451561450958251953125f,0.4507999988466500074224541094736196100711822509765625f,-0.4003000337749724746316815071622841060161590576171875f,0.7216999979868534698113080594339407980442047119140625f,0.538700033473968886710281367413699626922607421875f,-0.937199972483515608701054588891565799713134765625f,-0.0042999564826491365465699345804750919342041015625f,-0.282199988278746705105959335924126207828521728515625f,0.237899988210200863392174142063595354557037353515625f,-0.47960001357793735632384368727798573672771453857421875f,-1.698900013720988422250002258806489408016204833984375f,-0.49639997602105034868458233177079819142818450927734375f,-0.6641999817818404761027295535313896834850311279296875f,-0.860300048229099001417807812686078250408172607421875f,0.2685999976783992604367767853545956313610076904296875f,0.66210001115649974590127158080576919019222259521484375f,0.1561999869823456066342259873636066913604736328125f,-1.9912999835133557979816032457165420055389404296875f,-0.86580005204379606453812812105752527713775634765625f,-1.639100012743472944976019789464771747589111328125f,-0.1630999932676555630450820899568498134613037109375f,-0.097000038504599928046445711515843868255615234375f,1.49970001063048830047819137689657509326934814453125f,1.39919996933937085259458399377763271331787109375f,0.299200003686547255643546350256656296551227569580078125f,0.847899979567527939394722125143744051456451416015625f,0.23709999040663254366023693364695645868778228759765625f,0.2985999959647653856364968305570073425769805908203125f,-0.086000003755092369051027390014496631920337677001953125f,0.31240004958808498258804320357739925384521484375f,0.812300031040609216148595805861987173557281494140625f,0.54470003059804472655969220795668661594390869140625f,0.72280004061087999023271777332411147654056549072265625f,0.54479999641403542565143425235874019563198089599609375f,1.31309997659698129979233272024430334568023681640625f,-0.184200005307048275682291205157525837421417236328125f,-0.0769000297546378330793004352017305791378021240234375f,-0.4522000114127997516533241650904528796672821044921875f,-0.92710001950114939717195738921873271465301513671875f,-0.8846000091448427138374199785175733268260955810546875f,0.30170002672746754068811014803941361606121063232421875f,-0.02309997613132031002436406197375617921352386474609375f,-0.147499999925493785468688656692393124103546142578125f,0.66390001893639549734871252439916133880615234375f,-0.70120002159029282129409921253682114183902740478515625f,0.09029997746720896945049616988399066030979156494140625f,0.76850004125013959832557475237990729510784149169921875f,-0.036000043820589411325272521935403347015380859375f,0.719600018718838274622839890071190893650054931640625f,0.7768000019311909198904686491005122661590576171875f,-0.1148000317484143550927910837344825267791748046875f,0.0289999710619444073245176696218550205230712890625f,0.4348000046283002628655367516330443322658538818359375f,-0.2836000108100471539529507936094887554645538330078125f,0.3360000084489580540747510895016603171825408935546875f,-0.664100027644634938184253769577480852603912353515625f,0.165899992285668762548311860882677137851715087890625f,-0.907400028689206106236042614909820258617401123046875f,1.0631000055611128374266627361066639423370361328125f,-1.7928000103801500220157549847499467432498931884765625f,0.53049998584017188374417628438095562160015106201171875f,0.27050003451109017049702742951922118663787841796875f,-0.2863000192947691147082878160290420055389404296875f,0.8701000550925730436802041367627680301666259765625f,-1.33730006585344707303875111392699182033538818359375f,-0.33449998255073942754478366623516194522380828857421875f,1.03970005399286780090051252045668661594390869140625f,0.0754000175654889748244613656424917280673980712890625f,0.54900002077594367566604205421754159033298492431640625f,0.520600012955814595017045576241798698902130126953125f,-1.538399971014261513602150444057770073413848876953125f,1.4980000186972313169775361529900692403316497802734375f,-0.2880000120475891822735547975753434002399444580078125f,0.2275999734342100122574947818065993487834930419921875f,0.683000014059244708874985008151270449161529541015625f,-1.0736000026702878784590211580507457256317138671875f,1.825000002719461722477944931597448885440826416015625f,0.239299962443114022647705496638081967830657958984375f,-0.20790001458525664279619604712934233248233795166015625f,-0.37640002772808056619169292389415204524993896484375f,-0.3929000018447634801077583688311278820037841796875f,-0.5723000522375105703076769714243710041046142578125f,1.067900012648105079193783240043558180332183837890625f,-1.5430000285506242363453566213138401508331298828125f,0.741300005641579939918983654933981597423553466796875f,1.094600017452240336979230050928890705108642578125f,0.739500002965330249793396433233283460140228271484375f,0.5519999786019322840502354665659368038177490234375f,-0.68780000985860834106233596685342490673065185546875f,-1.402300005894899559422128731966949999332427978515625f,-0.13139999539256042027091098134405910968780517578125f,-0.2267000055491923404815679532475769519805908203125f,-0.21810002059266064833309428649954497814178466796875f,-0.4645999566555030213521604309789836406707763671875f,0.02009999511540006000842595312860794365406036376953125f,-1.020399992978571646062846411950886249542236328125f,0.78610001364126735889925612355000339448451995849609375f,-0.08419994132518837659517885185778141021728515625f,0.55769997367188339776333805275498889386653900146484375f,-0.348399995899200121129979379475116729736328125f,0.2358000027209519178228447344736196100711822509765625f,1.3314999975860113412551299916231073439121246337890625f,1.160799952280521463165996465249918401241302490234375f,-0.03219998723566508402882391237653791904449462890625f,1.45730000965744199703522099298425018787384033203125f,-0.2349999953061345792093561613000929355621337890625f,0.0306999792933464465960469169658608734607696533203125f,1.1742999437421559605354559607803821563720703125f,-0.3097999493822458649816553588607348501682281494140625f,-0.698799996775388621017555124126374721527099609375f,-0.1055999768391260307254242434282787144184112548828125f,0.6523999799996615944763789229909889400005340576171875f,0.441700009572505880584003534750081598758697509765625f,1.364800009899586274997318469104357063770294189453125f,0.19850004884973204699605275891372002661228179931640625f,1.974500029042363991038655512966215610504150390625f,-0.04360001161098481237132773458142764866352081298828125f,0.0584000196665523940708908412489108741283416748046875f,0.145199954691529686812145882868207991123199462890625f,-2.011600013637543593603140834602527320384979248046875f,0.527400027534365545278660647454671561717987060546875f,1.18029996813535742461453992291353642940521240234375f,0.5481000137567517693781837806454859673976898193359375f,0.16969997255802127256174571812152862548828125f,0.456500017702580418443858434329740703105926513671875f,-1.5750000078789885282759541951236315071582794189453125f,1.6522000196084378753624832825153134763240814208984375f,-1.4817000041522090381107545908889733254909515380859375f,-0.325699999111889848535383862326852977275848388671875f,0.6041999400585893464921127815614454448223114013671875f,-0.8803999881356958656652977879275567829608917236328125f,0.73360001511275729768613018677569925785064697265625f,0.883200005277245736579061485826969146728515625f,-0.42519994865655963423023422365076839923858642578125f,0.8015000103265055297896424235659651458263397216796875f,-1.1120000334382051931925161625258624553680419921875f,-1.009500002741813506901280561578460037708282470703125f,0.986100004309415911762926043593324720859527587890625f,-1.2317999409668143329099621041677892208099365234375f,1.0777000116139647811763779827742837369441986083984375f,0.8723999301552776142187894947710447013378143310546875f,0.2887000198394067407292595817125402390956878662109375f,-0.370599991032481312913660076446831226348876953125f,-0.43899999810755208073942412738688290119171142578125f,-0.0261999287560581439748830234748311340808868408203125f,-0.82349999891221603576241250266321003437042236328125f,-0.08980002854466440087577439044252969324588775634765625f,1.386100010865927156800125885638408362865447998046875f,2.0720999839246285745275599765591323375701904296875f,1.18489999765753761806763577624224126338958740234375f,-0.563200006860494539751016418449580669403076171875f,-0.0130000063478945548922638408839702606201171875f,0.94980002120584272606862441534758545458316802978515625f,-0.13729997968748197667565591473248787224292755126953125f,-1.489400061959774035358350374735891819000244140625f,-1.4576000568993396910855153691954910755157470703125f,0.988500022158026236951400278485380113124847412109375f,-1.181400046205521192632659221999347209930419921875f,0.175900026297568956579198129475116729736328125f,0.93010003437995880659627800923772156238555908203125f,-0.5447000117480753633714130046428181231021881103515625f,0.3932000177383425665311733609996736049652099609375f,0.6607000278711321339386586259934119880199432373046875f,-1.328399996495246426775338477455079555511474609375f,-0.47160003109052806014034331383300013840198516845703125f,0.4068999654933804777101613581180572509765625f,-1.4958000406630340695102177051012404263019561767578125f,1.3964000300377608088098213556804694235324859619140625f,-0.18380001996532124852734568776213563978672027587890625f,-0.5095999816894529743649400188587605953216552734375f,0.1660000013709075350476496168994344770908355712890625f,0.66710000374317335314344745711423456668853759765625f,-0.179900025261939877641026441779104061424732208251953125f,-0.56109998620450529660530492037651129066944122314453125f,-0.564800000381469491372854463406838476657867431640625f,0.8995000035613778255338957023923285305500030517578125f,1.6514000200726091094338698894716799259185791015625f,0.00019998752996315538599247929596458561718463897705078125f,0.77599994932860028029608656652271747589111328125f,-0.9587999999605119061385494205751456320285797119140625f,-0.7401999753668910830839422487770207226276397705078125f,0.348899967527388721322267883806489408016204833984375f,-0.67450001555681193821101260255090892314910888671875f,-0.1216000274956225979394730529747903347015380859375f,-0.994999955371022704042616169317625463008880615234375f,0.32790003710463644370776137293432839214801788330078125f,-0.28870002349019063814239416387863457202911376953125f,-0.773500012844801521083581974380649626255035400390625f,-1.2248999831289051343219398404471576213836669921875f,0.822499955967069151796522419317625463008880615234375f,1.307499986961483617875501295202411711215972900390625f,-0.9502000002145774004702616366557776927947998046875f,0.084900009727478131793532156734727323055267333984375f,1.0304999731183048794491696753539144992828369140625f,0.89650001055002181971076424815692007541656494140625f,-0.6807000159502027969438131549395620822906494140625f,-0.818400018474459312045610204222612082958221435546875f,1.688299994469434128774310011067427694797515869140625f,-0.22429999118372823119216263876296579837799072265625f,-0.1405999942362310406451797462068498134613037109375f,0.4781000010348857021114099552505649626255035400390625f,-0.449399976184964078385064567555673420429229736328125f,0.500900015661865705141053695115260779857635498046875f,-1.4613000171899788171003820025362074375152587890625f,0.42359999551773019899059136150754056870937347412109375f,0.277200023221968905318135512061417102813720703125f,-0.1734000194430349583996076034964062273502349853515625f,-0.1532000251889229502921807579696178436279296875f,0.06700003360584416700618248796672560274600982666015625f,-1.108599997287243343890850155730731785297393798828125f,-1.1175000013783578101111970681813545525074005126953125f,0.0098000058203935314082855256856419146060943603515625f,0.3318999913468960727414014399982988834381103515625f,-0.0257999797731642832587795055587776005268096923828125f,-0.1654999729320409951327519593178294599056243896484375f,1.363199969011545942265684061567299067974090576171875f,0.6036000601887703265191476020845584571361541748046875f,-0.8141999801427128868880345180514268577098846435546875f,0.475800010544061624528922038734890520572662353515625f,0.51219998015761358800546076963655650615692138671875f,0.0299999608471994694269824321963824331760406494140625f,0.9065000126361848487022143672220408916473388671875f,0.050600003549456484819302204414270818233489990234375f,0.114400008818507092911431755055673420429229736328125f,-1.0638999878674744170581334401504136621952056884765625f,-0.353800045576690980198009128798730671405792236328125f,0.31469997553825379554837127216160297393798828125f,-1.423399979954957483840871645952574908733367919921875f,-0.51990005055665944322385030318400822579860687255859375f,-1.00789998403787617320404024212621152400970458984375f,0.144299992394447684773695073090493679046630859375f,-1.928100016066431887651333454414270818233489990234375f,-0.2310000443980093631779482166166417300701141357421875f,0.082799993765353629981973426765762269496917724609375f,0.15029999663382753727347562744398601353168487548828125f,-0.60729999211132490444242648663930594921112060546875f,-0.120800013133138539611621808944619260728359222412109375f,-0.278699988844990809155888200621120631694793701171875f,-0.90649999982118545371889695161371491849422454833984375f,0.310799992066622454700564048835076391696929931640625f,-0.79569999059960261345025855916901491582393646240234375f,0.445299997192620811148344728280790150165557861328125f,-1.100799996648728296122499159537255764007568359375f,0.418699934008717189470871744561009109020233154296875f,-0.83769998337626472295625035258126445114612579345703125f,-0.526499976798892266316443055984564125537872314453125f,0.2365000124871727837927437576581723988056182861328125f,0.2600999828964472015968567575328052043914794921875f,0.31869996794611232804328437850927002727985382080078125f,0.3907000099897379730151669718907214701175689697265625f,1.55309995370507269996096511022187769412994384765625f,-0.20790003134906243076329701580107212066650390625f,0.9343999929115176517058216631994582712650299072265625f,-0.6659999942928547245202253179741092026233673095703125f,1.0098999396175154519283978515886701643466949462890625f,0.1950000149756669909351103342487476766109466552734375f,0.759799998779595142650578054599463939666748046875f,2.082299957838653625685765291564166545867919921875f,-0.16329995898604376947105265571735799312591552734375f,1.39630001331567843436687326175160706043243408203125f,0.76530001928359292495684940149658359587192535400390625f,-0.12929997859224717782211655503488145768642425537109375f,0.19800000636652137497861758674844168126583099365234375f,0.1337000133015209257791866548359394073486328125f,-0.45329997239708941236813188879750669002532958984375f,1.860399968689680605393732548691332340240478515625f,1.212300034169852214205320706241764128208160400390625f,0.45909997563958171440390287898480892181396484375f,0.3304000170439482797490882148849777877330780029296875f,-1.784400025880336304595630281255580484867095947265625f,-1.321200002181529331579667996265925467014312744140625f,-1.807400040721893308415246792719699442386627197265625f,-0.97100000287964938205931275660987012088298797607421875f,-0.4501000086568296421063450907240621745586395263671875f,0.723300010748952448835780160152353346347808837890625f,-0.094700029871613244925043773037032224237918853759765625f,1.0981999847695231853350605888408608734607696533203125f,1.145900004355608903239271967322565615177154541015625f,0.19429995868056970653725556985591538250446319580078125f,0.6130000070184464799893930830876342952251434326171875f,-0.44870002192556890374675049315555952489376068115234375f,-0.129399947090447664521661863545887172222137451171875f,0.645400001025199454574021729058586061000823974609375f,0.127300023180246579812546769971959292888641357421875f,0.09979999009966811218674820338492281734943389892578125f,-0.057700012917816234647716555627994239330291748046875f,-0.071200013860315269909051494323648512363433837890625f,0.8646999808654192154477868825779296457767486572265625f,-0.38079998707473217134378273840411566197872161865234375f,1.19809997829198788821258858661167323589324951171875f,-0.1036000253945592586291013503796420991420745849609375f,-0.22530000747442269215525811887346208095550537109375f,-0.31200000505149372287405640236102044582366943359375f,-0.66709999351724957872278309878311119973659515380859375f,-0.31170002797544060779699748309212736785411834716796875f,-0.876900002485513585526177848805673420429229736328125f,0.187500030584634103103525148981134407222270965576171875f,-1.266200052137673282004470820538699626922607421875f,0.623800042317063319075032268301583826541900634765625f,-0.55300002544000859128203728687367402017116546630859375f,-1.4694000540435314672293998228269629180431365966796875f,2.34680003113746682430473811109550297260284423828125f,-0.0436000230848794245730459806509315967559814453125f,0.5434000496923925282999334740452468395233154296875f,0.378600038731099264310842045233584940433502197265625f,-1.061000038791447774855214447597973048686981201171875f,-0.6317999998450272158834195579402148723602294921875f,-0.6489000019490713810910165193490684032440185546875f,0.15000003119930671591220061600324697792530059814453125f,-0.9192000025130802054462719752336852252483367919921875f,-0.444499957386403410364295041290461085736751556396484375f,0.12190002718418868621341744074015878140926361083984375f,-0.51600000151991931574002592242322862148284912109375f,0.52829999080002298939007232547737658023834228515625f,-0.708600002223252811717202348518185317516326904296875f,-0.82070004284679909378752427073777653276920318603515625f,0.893800043863058224502537996158935129642486572265625f,0.83799996283650468598125371499918401241302490234375f,-0.38550002579390962864636094309389591217041015625f,-0.265699973332882422027978464029729366302490234375f,-0.091400051048398101016800865181721746921539306640625f,-0.704799979519844299602482351474463939666748046875f,-0.8881000070512288235846654060878790915012359619140625f,-0.3256999855518341036031415569595992565155029296875f,0.822800010082125776733619204605929553508758544921875f,0.7371000108957286300892519648186862468719482421875f,-0.1321000061273576164921905728988349437713623046875f,0.920799967479705827599900658242404460906982421875f,0.4102000038653610403116545057855546474456787109375f,-0.17700003135204323001516968361102044582366943359375f,0.227700009751320209261393756605684757232666015625f,-0.26659999590516125778094647102989256381988525390625f,0.8871000006139269089544541202485561370849609375f,-1.3632999364852906865053228102624416351318359375f,1.190400060182810637599004621733911335468292236328125f,-0.2402000413417813007299628225155174732208251953125f,-0.253600051248074720433578477241098880767822265625f,0.309600066030025544705495121888816356658935546875f,-1.02170000897645874005092991865240037441253662109375f,1.305999980211257849305184208787977695465087890625f,-0.284600001060962171095525263808667659759521484375f,-1.0172999805629248104565931498655118048191070556640625f,-0.4032999858081343091953385737724602222442626953125f,-1.479100009316205888154627245967276394367218017578125f,-0.62079996658563540989916873513720929622650146484375f,-1.990599981346726554676251907949335873126983642578125f,-0.05119999726414814489316995604895055294036865234375f,-0.622100013354420777744735460146330296993255615234375f,-1.85630001942515310275894080405123531818389892578125f,-0.1040999978423113869752114624134264886379241943359375f,-0.344699972856044922764340299181640148162841796875f,0.494599972972273871363313446636311709880828857421875f,-0.95330001412034004459883362869732081890106201171875f,1.34609999009743308562292440910823643207550048828125f,0.60780004580765989441459851150284521281719207763671875f,2.033599973724782739026295530493371188640594482421875f,-0.6699000063002105154197352021583355963230133056640625f,0.251000020816921409760880123940296471118927001953125f,-1.23230002653300818593606891226954758167266845703125f,-1.1144999707043172776366191101260483264923095703125f,1.120200021225213671272058491013012826442718505859375f,0.5829999737143527482885474455542862415313720703125f,-0.848100022697448441277856545639224350452423095703125f,1.183000008583068307643770822323858737945556640625f,0.42570000812709363202657186775468289852142333984375f,-0.51760001064985983187938245464465580880641937255859375f,-1.38400005253404412286499791662208735942840576171875f,0.50789999724403023240171251018182374536991119384765625f,-1.358200043815374069566814796417020261287689208984375f,1.706099985682964170763398215058259665966033935546875f,1.752500004023312296652648001327179372310638427734375f,0.091800013032556027070540949353016912937164306640625f,0.112200028288365505346746431314386427402496337890625f,-0.5606999992609029703771739150397479534149169921875f,-0.7811000136971468776891924790106713771820068359375f,1.2668999859452245715374374412931501865386962890625f,-0.347300047767163277967483736574649810791015625f,0.2463000054180619624588644001050852239131927490234375f,-0.93490000365525471881511521132779307663440704345703125f,0.444600040008873598207372879187460057437419891357421875f,0.75200002077221828056252661554026417434215545654296875f,0.312899983292818095748089035623706877231597900390625f,0.346999978899956484923450261703692376613616943359375f,-0.8487000814557077443822663553874008357524871826171875f,-2.222200034844874938499970085103996098041534423828125f,-0.688000000610947015644569546566344797611236572265625f,-0.36729997981786777216939299250952899456024169921875f,-0.589600040400028557741052281926386058330535888671875f,-0.19020002629160881468806110206060111522674560546875f,0.5401999707102778103262608055956661701202392578125f,-0.5257000002294776042077728561707772314548492431640625f,0.5940000093579289597300885361619293689727783203125f,0.44959999122619620504082149636815302073955535888671875f,-0.536199975769221737209591083228588104248046875f,-0.95590004486814184048171227914281189441680908203125f,-1.5573000265702605648954204298206605017185211181640625f,0.3171999702483410121800488923327066004276275634765625f,-0.8606000125646584564265140215866267681121826171875f,0.273100031265616660647310709464363753795623779296875f,0.69479997802972803810916957445442676544189453125f,-1.36030000784695204174568061716854572296142578125f,0.3056000215053558122235699556767940521240234375f,0.45419999854564618857466484769247472286224365234375f,-0.67230000291466662787343011586926877498626708984375f,-0.6737000261351469010406844972749240696430206298828125f,0.3622000021740792163882360910065472126007080078125f,0.930700037910789479411732827429659664630889892578125f,-0.46099999345466502465029634549864567816257476806640625f,1.700900102144480730004261204157955944538116455078125f,-1.4263000195741657449843842186965048313140869140625f,-1.298200019079446487779705421417020261287689208984375f,1.4221000082135208941735982079990208148956298828125f,0.23499995872378232064647818333469331264495849609375f,0.0253000234186646633105510773020796477794647216796875f,-0.266800011914968937531966730603016912937164306640625f,-1.42630001718997956317025455064140260219573974609375f,0.18020003383532212648532322418759576976299285888671875f,0.955300028354674868324991621193476021289825439453125f,1.4610000317133966518667875789105892181396484375f,-1.01990002347379959957152095739729702472686767578125f,0.1297000121951104034678792231716215610504150390625f,0.780500006869434326262080503511242568492889404296875f,0.334599955761431200329525381675921380519866943359375f,1.5825999966055144252408126703812740743160247802734375f,0.49210000940561382520854749600403010845184326171875f,-0.45969999394118821189891832545981742441654205322265625f,-0.0635000135898593498495756648480892181396484375f,-0.574700005638599353829931715154089033603668212890625f,-0.3044999977499245336076683088322170078754425048828125f,0.722099969768524641011708808946423232555389404296875f,-0.493000024825334559608336348901502788066864013671875f,0.155800041985511228403993300162255764007568359375f,0.393600000211595801857811238733120262622833251953125f,0.58079998178482128423638641834259033203125f,-0.2667000037610531837373173402738757431507110595703125f,-0.2543000019311900672391857369802892208099365234375f,-0.281500000491738067154301461414434015750885009765625f,1.114500027496367895452067386941052973270416259765625f,1.3516000521570443737573441467247903347015380859375f,-1.176700015234947382936070425785146653652191162109375f,-0.56100000844895792084798813448287546634674072265625f,-0.01489998476803311433513954398222267627716064453125f,0.892800017011165181202159146778285503387451171875f,-0.29650002336502012667551753111183643341064453125f,-0.7347000278413293727908239816315472126007080078125f,0.661800021004676608527006465010344982147216796875f,0.111600028091670022689640973112545907497406005859375f,0.342599953019618652660938096232712268829345703125f,0.388299998474121110803025658242404460906982421875f,0.794100012706220326208494952879846096038818359375f,0.5198000014245514055488683879957534372806549072265625f,0.132099990332126271397328309831209480762481689453125f,-0.498600008481741241439522127620875835418701171875f,-1.76969999237656594459622283466160297393798828125f,-0.53859999909400979589690905413590371608734130859375f,-0.453300064188242313178989206789992749691009521484375f,-1.4211000220417968620267856749705970287322998046875f,-0.751500039309263012654582780669443309307098388671875f,0.1327000206291675254988149390555918216705322265625f,0.342599990049004876624394455575384199619293212890625f,-0.76420003207325937211180644226260483264923095703125f,0.277200034993887012291224891669116914272308349609375f,0.5380999908834696032755573469330556690692901611328125f,-0.160699988085031719720063847489655017852783203125f,-0.9295000109374520480542969380621798336505889892578125f,0.045999978721142031901081281830556690692901611328125f,-0.345100008302926308800806509680114686489105224609375f,-1.33689994347691598619576325290836393833160400390625f,-0.218700008216500318969366389865172095596790313720703125f,-1.193599961692095146048586684628389775753021240234375f,-1.0057000186324118740088806589483283460140228271484375f,-1.339599961721896992372649037861265242099761962890625f,0.22750000447034846029481514051440171897411346435546875f,0.277000056684017170738343338598497211933135986328125f,0.27460001059770544618032772632432170212268829345703125f,-1.088799985972046524551615220843814313411712646484375f,0.02800000105798261529344017617404460906982421875f,-0.029399992278218434194059227593243122100830078125f,0.9368000134050842309108020344865508377552032470703125f,-0.66040000489950134721084396005608141422271728515625f,1.08679999902546331469466167618520557880401611328125f,-1.061000001221894972758263975265435874462127685546875f,1.01020004439652044680997278192080557346343994140625f,-0.1805000544041401866479645832441747188568115234375f,1.0908999888584005599767579042236320674419403076171875f,-0.490499986156821332627941956161521375179290771484375f,-0.08809999304413818066450403421185910701751708984375f,0.050500023186206444147927641097339801490306854248046875f,0.5242000217914579707212396897375583648681640625f,-1.428199976909160451299385385937057435512542724609375f,0.2683999984323970355859501069062389433383941650390625f,-0.69349999727308730967934025102294981479644775390625f,0.205699995684624159508757657022215425968170166015625f,-1.076500018447636985996496150619350373744964599609375f,1.05020001630783088586440499057061970233917236328125f,-0.477800012764334425252599203304271213710308074951171875f,-0.0830000293701884661601297921151854097843170166015625f,1.29219997181296353261359399766661226749420166015625f,0.56890003555119061928735391120426356792449951171875f,1.2851999826312070496214801096357405185699462890625f,-0.24720001860260942816438500813092105090618133544921875f,0.26670002860873864847235381603240966796875f,0.0732000066556033335274378259782679378986358642578125f,0.2627999908968805531372936457046307623386383056640625f,0.49790002546310407893770388909615576267242431640625f,1.51319993160963139899877205607481300830841064453125f,0.1176999869525433839445582862026640214025974273681640625f,1.0321999850004910381784384298953227698802947998046875f,0.54480003267973697855808268286637030541896820068359375f,0.0515999853253361617788641524384729564189910888671875f,-0.308199948596955264346064495839527808129787445068359375f,0.6628999897748233394167982623912394046783447265625f,-1.0228999286428095416567884967662394046783447265625f,0.2886000014781948674880140970344655215740203857421875f,0.59890003584921391865947271071490831673145294189453125f,0.350099998393654576744893347495235502719879150390625f,-0.62430000342130664847672960604541003704071044921875f,0.87040003105104002401049001491628587245941162109375f,-0.5724999153986569666585637605749070644378662109375f,0.479499983891844727690312311096931807696819305419921875f,0.5578999852672221759775084137800149619579315185546875f,0.88180001962631970879868958945735357701778411865234375f,-0.82549999248981420496562577682198025286197662353515625f,2.14830002110525963843201679992489516735076904296875f,-0.25950000231340542011793104393291287124156951904296875f,-0.393900027001649277735850773751735687255859375f,-0.45060003513991819801276506041176617145538330078125f,-1.987100000762939799869855050928890705108642578125f,-0.01010001664757709249897743575274944305419921875f,-0.55179995990991681509285626816563308238983154296875f,-1.1787999917834992569254382033250294625759124755859375f,-0.95770000516921227617928025210858322679996490478515625f,0.07879996812790623916811227900325320661067962646484375f,-0.88809999174028553259319096468971110880374908447265625f,1.4381000284530218724654559991904534399509429931640625f,-1.385100008824468265089535634615458548069000244140625f,-0.385400027754157659121148071790230460464954376220703125f,-0.4749000568896535323659691130160354077816009521484375f,-1.4318000009439881825556994954240508377552032470703125f,0.6026999614715577990864403545856475830078125f,-0.6734999879971146352630739784217439591884613037109375f,0.36379997040033362765143465367145836353302001953125f,-0.7038000020280479329670697552501223981380462646484375f,0.44719996737986800194875058878096751868724822998046875f,0.430300026641040833652596120373345911502838134765625f,-0.05850001247227198941658343755989335477352142333984375f,0.12770004691109082539668406752753071486949920654296875f,0.5611000439278786000585341753321699798107147216796875f,2.0652999890342353950245524174533784389495849609375f,0.6182000090397894442872939180233515799045562744140625f,1.171099946828186855185549575253389775753021240234375f,-0.302100002700090186635861755348742008209228515625f,0.1589000047057869124955686856992542743682861328125f,-0.452900003707408682629420582088641822338104248046875f,-0.32370003287792104007536408971645869314670562744140625f,0.39880000996291598003296030583442188799381256103515625f,-0.52759997186958840220682986910105682909488677978515625f,-1.82910000395178773402449223794974386692047119140625f,-0.292000007361173263831233271048404276371002197265625f,-0.63949998292326881710323505103588104248046875f,-0.668500056356192029483054284355603158473968505859375f,-1.979900042025745765528199626714922487735748291015625f,0.34440001839250322657193237319006584584712982177734375f,-0.3500999950408936189916175862890668213367462158203125f,-0.0096999841064207004137642798013985157012939453125f,-0.91209999673962638411239822744391858577728271484375f,0.64810004601776682875424739904701709747314453125f,1.90780008451342641961900881142355501651763916015625f,-0.3688999909222123729790610013878904283046722412109375f,-0.1282000031352037439091873238794505596160888671875f,-1.329700022104382650667275811429135501384735107421875f,1.82939999905824635106910136528313159942626953125f,-1.02740001205578490584002793184481561183929443359375f,0.46800003313273219607282271681469865143299102783203125f,-0.6974000076413149340481822946458123624324798583984375f,0.725699985738099240251131050172261893749237060546875f,0.3029000572212046105136096230125986039638519287109375f,-0.72280001909732760623228386975824832916259765625f,0.6611000240720807941130487961345352232456207275390625f,0.2655000202059749625504991854541003704071044921875f,0.9084000267818570506506148376502096652984619140625f,0.53599996508657932192676298654987476766109466552734375f,1.4202999520421031309780346418847329914569854736328125f,-1.2553000001728535917067119953571818768978118896484375f,-0.20970003748983234714842183166183531284332275390625f,-0.25290001160502406651886531108175404369831085205078125f,1.2497999855920678413667701533995568752288818359375f,-0.760600018823147205893064892734400928020477294921875f,-1.845300042901932169314704879070632159709930419921875f,0.87980002258420009209061163346632383763790130615234375f,0.5899000574484478676140497555024921894073486328125f,-0.8338000151038169605044458876363933086395263671875f,-1.4306000084668397676068707369267940521240234375f,-0.314200027081370958370598600595258176326751708984375f,-0.2607999942272900550932490659761242568492889404296875f,-0.09549999189376734420875436626374721527099609375f,-0.154100004100799825579315438517369329929351806640625f,0.4428999566197386883459330420009791851043701171875f,0.204999990686773880810278569697402417659759521484375f,0.17670000584721545777711071423254907131195068359375f,-0.31940001112818716677566044381819665431976318359375f,-1.175100046412647092353154221200384199619293212890625f,0.02629999815374584049010309172444976866245269775390625f,-0.6238000195741648656877487155725248157978057861328125f,-0.8319000660389670454009092281921766698360443115234375f,1.815100006589293268888241073000244796276092529296875f,0.315800032169371969192894766820245422422885894775390625f,-0.6987999614596365205443362356163561344146728515625f,0.928399997016787548176353084272705018520355224609375f,-1.2679000072836874579707000521011650562286376953125f,-0.56310002629235389459694260949618183076381683349609375f,0.7512000132083898051860160194337368011474609375f,-0.1466999969996513886183464592249947600066661834716796875f,-0.78700002617389042125495279833558015525341033935546875f,-1.1041000646181411415369666428887285292148590087890625f,1.169800014127790444007359837996773421764373779296875f,-1.768000031512230396657514575053937733173370361328125f,-1.1233999918758870162349694510339759290218353271484375f,-1.01799998887628273536165579571388661861419677734375f,-0.340300014403462913747944185161031782627105712890625f,-0.2985000052452095875565873939194716513156890869140625f,1.453000022366643495530524887726642191410064697265625f,2.2498000441536305515910498797893524169921875f,-1.13160003546774401428365308674983680248260498046875f,-0.4278000010296703070622470477246679365634918212890625f,0.690800020900368405563085616449825465679168701171875f,1.911999999538063210735572283738292753696441650390625f,-0.61340002644658131458754724008031189441680908203125f,0.68729991145878932545798534192726947367191314697265625f,-0.32570001192689002067481851554475724697113037109375f,1.3643000573635106764669444601167924702167510986328125f,0.112600013592839776066512058605439960956573486328125f,-0.143800050345064267531824953039176762104034423828125f,-0.14120001223981404336882405914366245269775390625f,0.395600024485588175338079963694326579570770263671875f,1.521899987025558953490644853445701301097869873046875f,0.21970000440925330220665045999339781701564788818359375f,0.3392999776422980229284576125792227685451507568359375f,0.58990001796037005998840641041169874370098114013671875f,0.57319999935403453950044649900519289076328277587890625f,-0.26459999329373229404183121005189605057239532470703125f,0.04710002609491414915510176797397434711456298828125f,-0.0486000345215209517135690475697629153728485107421875f,0.33809999513029964646193548105657100677490234375f,-0.0025000032410023020901235213386826217174530029296875f,0.58970005421638571618814239627681672573089599609375f,1.13020001457557039969969991943798959255218505859375f,-0.63989998845607132249568849147181026637554168701171875f,0.8660000051707033019710024746018461883068084716796875f,-0.954400003193318635652531156665645539760589599609375f,-0.17849999941512939560794848148361779749393463134765625f,0.617199990476667181837910902686417102813720703125f,-0.278500008560717215999602558440528810024261474609375f,-0.3997999882370228963424096946255303919315338134765625f,0.61800005850195904866239970942842774093151092529296875f,0.189399980655311883648295179227716289460659027099609375f,-1.30410001162588695677868599887005984783172607421875f,-0.4643000259220595449249913144740276038646697998046875f,-0.56529994798153637702853302471339702606201171875f,-0.13029997141361271406623245638911612331867218017578125f,-0.3977999921634793878411073819734156131744384765625f,1.094400000473857215155248923110775649547576904296875f,1.70589998813197052385248753125779330730438232421875f,-1.074899972083419807944437707192264497280120849609375f,0.4146000029236078798788867061375640332698822021484375f,-0.1986999917507168422758923043147660791873931884765625f,-0.5294000084459782318191400918294675648212432861328125f,0.88970003752708404221039018011651933193206787109375f,-0.489399986727535374342323848395608365535736083984375f,0.162700009006261581134822336025536060333251953125f,-0.001099992686510287143164532608352601528167724609375f,0.232700003939867006153008333058096468448638916015625f,-0.10880000746697160440845664197695441544055938720703125f,-0.4412999834761024420259900580276735126972198486328125f,-0.376500014871358479329188639894709922373294830322265625f,-1.052300023031234932346933419466949999332427978515625f,1.1535999878436322685359982642694376409053802490234375f,0.329000010736286707668796225334517657756805419921875f,-0.0267999882593745741843349605915136635303497314453125f,0.52699998217821164558927193866111338138580322265625f,0.309600011994689394700941420524031855165958404541015625f,-0.4939000101447106327867686559329740703105926513671875f,0.0844999935403467627992313282447867095470428466796875f,0.1267000008925800447201481802039779722690582275390625f,-0.61559999209418847510022487767855636775493621826171875f,0.7972999727398157876478990147006697952747344970703125f,-0.402199990495294590342467699883854947984218597412109375f,0.170100011357664726574512314982712268829345703125f,-0.799900033401697907464722447912208735942840576171875f,-0.02840001482367515706073390902020037174224853515625f,-0.34990003528296942381103917796281166374683380126953125f,-0.39219998485147922195892533636651933193206787109375f,-1.2220999896615740709648889605887234210968017578125f,0.65010001355558622027075443838839419186115264892578125f,-0.215499994315206944150986601016484200954437255859375f,0.1596999868631362728965683572823763824999332427978515625f,0.499099967704713931571092189187766052782535552978515625f,-0.457299990136921241656864367541857063770294189453125f,-0.16660001233369092776825937107787467539310455322265625f,-0.12869998623728751141470638685859739780426025390625f,1.03320002999454718661809238255955278873443603515625f,-1.1325000251457095767904093008837662637233734130859375f,0.89869998985081878117853193543851375579833984375f,-0.92479998410195118907495270832441747188568115234375f,-0.12230001796483946918669971637427806854248046875f,-1.325099990868567889634732637205161154270172119140625f,0.49499999158084440065152875831699930131435394287109375f,-0.73639996782541228270702049485407769680023193359375f,-0.70300003264844423966906106215901672840118408203125f,0.877000047445296360137945157475769519805908203125f,0.548100017705559938718806733959354460239410400390625f,-1.9772000309705735077159260981716215610504150390625f,-1.372900025761128350865192260243929922580718994140625f,-0.6937000228196392725976693327538669109344482421875f,-0.371300014805793499039765492852893657982349395751953125f,0.128399966245889896043763656052760779857635498046875f,-1.72290002013593923635426108376123011112213134765625f,2.484999983161686909483023555367253720760345458984375f,-0.0175999458484346060771486008889041841030120849609375f,0.110700013715028688920938293449580669403076171875f,-0.6652000220261513607766801214893348515033721923828125f,1.521599989795685559101912076584994792938232421875f,-0.814900019096584049549392148037441074848175048828125f,0.0860999841928471454366444959305226802825927734375f,0.39149997799098457118560645540128462016582489013671875f,0.1943000205948949010714699170421226881444454193115234375f,0.10010000442862498903906498526339419186115264892578125f,0.64529997152537144078365827226662077009677886962890625f,0.891199945333600584973510194686241447925567626953125f,-1.471099985906481943942480938858352601528167724609375f,-1.8144000481948250325103799696080386638641357421875f,-0.429899989238381241563047296949662268161773681640625f,1.832499986328184338191249480587430298328399658203125f,-0.897700001704691885606735013425350189208984375f,-1.1204999751672148899928060927777551114559173583984375f,0.34939999101161856032149444217793643474578857421875f,-0.4662999949872503879788609992829151451587677001953125f,1.033600038433075241073311190120875835418701171875f,0.0451000270783907808436197228729724884033203125f,0.2517000074863435798988575697876513004302978515625f,-0.288700007843971651055881011416204273700714111328125f,-0.02199999350309411738635390065610408782958984375f,-0.04459996686279765754079562611877918243408203125f,-0.316400037264823907889876863919198513031005859375f,0.167500014752149706254158445517532527446746826171875f,0.420000021383166721733459780807606875896453857421875f,-0.033700023937225598302802609396167099475860595703125f,-2.1108000030189746354380986304022371768951416015625f,0.01299994655698488710271476520574651658535003662109375f,-0.905300010678172117906115090590901672840118408203125f,-1.251299991708993974981467545148916542530059814453125f,0.604899965992569121908672968856990337371826171875f,-1.15250002875924106859883977449499070644378662109375f,0.4978999723404637922641313707572408020496368408203125f,-0.223100002580881362490572428214363753795623779296875f,-1.305100009271501715346630589920096099376678466796875f,-1.052199995356797757040112628601491451263427734375f,-0.1943000406742101393575694601167924702167510986328125f,-0.966700030359625817055757579510100185871124267578125f,0.168899977734685347030563207226805388927459716796875f,-1.0616999975770708974920353284687735140323638916015625f,0.6146000097408894635719889265601523220539093017578125f,0.49000001057982445207272803600062616169452667236328125f,1.77660000480860436056218532030470669269561767578125f,-0.1958000333614648980073980055749416351318359375f,-0.5184999855384229139332319391542114317417144775390625f,-0.528300025594234501369328427244909107685089111328125f,-1.9669999568462372963040252216160297393798828125f,0.77059999475777107846852231887169182300567626953125f,-1.1903999655604362573058097041212022304534912109375f,-1.295999979130923573933387160650454461574554443359375f,-0.3721000218853347529801567361573688685894012451171875f,0.8960999933570621411860201988019980490207672119140625f,-0.8175000263750551443564518194762058556079864501953125f,-1.509800087590516870506007762742228806018829345703125f,-0.0031999643921849507677279689232818782329559326171875f,-0.339899988511949135983059022692032158374786376953125f,-0.57690003524944233337379273507394827902317047119140625f,1.54690005300045019254184808232821524143218994140625f,-1.39780003042221068199069122783839702606201171875f,0.921300033134221596498036888078786432743072509765625f,-1.5558000086069103673480640281923115253448486328125f,0.135100001299381489161532954312860965728759765625f,0.4152999616533519233740889831096865236759185791015625f,0.9377000187665218700061586787342093884944915771484375f,-0.05570000656247142689636575596523471176624298095703125f,-1.213400001394003169252755469642579555511474609375f,-1.244899998123198070487660515937022864818572998046875f,0.52620005427971516809293461847119033336639404296875f,-0.82369999245852210112417424170416779816150665283203125f,1.3151000069618223875522744492627680301666259765625f,-0.537100006723404188591075580916367471218109130859375f,0.678599980914592837422105731093324720859527587890625f,0.944800002616643208597224656841717660427093505859375f,1.7596999972194431460792429788853041827678680419921875f,-0.322499988451599772787403708207421004772186279296875f,-1.0280000181943174109022720585926435887813568115234375f,-1.065199986207485238765002577565610408782958984375f,-0.0856999988883730967614837936707772314548492431640625f,-1.2606999840617181263269230839796364307403564453125f,-1.0941000448182227611226835506386123597621917724609375f,-1.3735999964490537283978710547671653330326080322265625f,0.5903000030785798823984578120871447026729583740234375f,-1.0156999819010490426762771676294505596160888671875f,-1.027700007367133849811580148525536060333251953125f,0.3001000346779818528375471942126750946044921875f,1.50920005308389715281691678683273494243621826171875f,0.78240003148317338599326831172220408916473388671875f,-0.7109000038713215730012962012551724910736083984375f,-0.759500032246112510847524390555918216705322265625f,-0.17990002090334922257852667826227843761444091796875f,0.49420002521872508083333741524256765842437744140625f,1.8731000477313990426608825146104209125041961669921875f,-0.8863999905496833253693012011353857815265655517578125f,0.2264000404685744261001900667906738817691802978515625f,-0.5388000126078724694167476627626456320285797119140625f,1.107200011748074697237598229548893868923187255859375f,0.004300014150143027791273198090493679046630859375f,-1.4208000881791129632603087884490378201007843017578125f,0.5043000655591498571084230206906795501708984375f,-0.1017000127017499977455372572876513004302978515625f,0.012800001439452035612021063570864498615264892578125f,0.829300005805492457255923000047914683818817138671875f,0.378600033143162928439551251358352601528167724609375f,1.0062000152200460956208871721173636615276336669921875f,-1.01870003831684652340072716469876468181610107421875f,0.0347999737828963429109307980979792773723602294921875f,0.4052000115767115939746645381092093884944915771484375f,0.33979995865821788214589105336926877498626708984375f,-0.37550003287196265233660597004927694797515869140625f,-1.4958000686585908312054016278125345706939697265625f,0.82090003196001060103981217253021895885467529296875f,-0.02649999073147757400192858767695724964141845703125f,-0.764500015556812240191675300593487918376922607421875f,0.693800035965443040453237699694000184535980224609375f,-0.3438000005006784931538277305662631988525390625f,-0.99400004750490200677859320421703159809112548828125f,1.42249999284744177430184208787977695465087890625f,0.78880000526905025282076167059130966663360595703125f,-0.8460999910473816498779342509806156158447265625f,-0.64580002473741815460783755042939446866512298583984375f,-0.5115000046268101119295579337631352245807647705078125f,-0.064600016558170381131276371888816356658935546875f,-1.10030001544654343348383918055333197116851806640625f,-1.1318000527441502978120979605591855943202972412109375f,1.21320002883970712304062544717453420162200927734375f,1.621600066089630587384817772544920444488525390625f,-2.1341000192999839413232621154747903347015380859375f,-0.91810004490017949052571566426195204257965087890625f,1.114900028204917958873920724727213382720947265625f,-0.270499997109174383780327843851409852504730224609375f,-0.425600004518031482803053222596645355224609375f,1.37990006262660092062333205831237137317657470703125f,-1.1992000488370664346149396806140430271625518798828125f,-0.3724000093549488799027358254534192383289337158203125f,0.3344000107929108178694832531618885695934295654296875f,1.842100090020895919451504596509039402008056640625f,0.19329998461604080972620067768730223178863525390625f,-0.69100004547834448231924397987313568592071533203125f,-0.54519996199309783690978292725048959255218505859375f,-0.19579995600581145964724782970733940601348876953125f,-1.050100047045946727308773915865458548069000244140625f,-0.68980002630949055486553334048949182033538818359375f,-0.416200020360946609798702411353588104248046875f,0.5206000278383500212697754250257275998592376708984375f,-0.869000012934207699544231218169443309307098388671875f,0.7429000517636541456312215814250521361827850341796875f,0.25210003909617706430168482256704010069370269775390625f,1.658000029891729010245171593851409852504730224609375f,0.040800006222724505278165452182292938232421875f,0.28180000804066640540668231551535427570343017578125f,0.110399998492002993089045048691332340240478515625f,-0.43909998793303961495126941372291184961795806884765625f,0.20589995939135530278463193099014461040496826171875f,0.83890001323670115862540797024848870933055877685546875f,1.2021999796733264798120899286004714667797088623046875f,-0.9135999962627885384591763795469887554645538330078125f,1.31459998414814460687694008811376988887786865234375f,0.97129997792541977741365144538576714694499969482421875f,-0.3768999597564341996758230379782617092132568359375f,0.81480001686587966158725748755387030541896820068359375f,-0.23449999544024446862522381707094609737396240234375f,-0.8682000470936304115099346745410002768039703369140625f,1.201600023919343840361761976964771747589111328125f,0.323400022050738922274604192352853715419769287109375f,0.733399982413649897949881051317788660526275634765625f,-0.64830000905394502552780977566726505756378173828125f,0.1808000031679861674405174198909662663936614990234375f,-0.7373000289171931509457635911530815064907073974609375f,1.3393999805435539141029721577069722115993499755859375f,-0.480400015069544272794388461989001370966434478759765625f,-0.0469999553188689134231026400811970233917236328125f,0.086099996877460949207261364790610969066619873046875f,-0.81160005521178302245743907405994832515716552734375f,-0.114299977865814472721694983192719519138336181640625f,1.3886000369429591927428191411308944225311279296875f,-0.0487000179022556434205171171925030648708343505859375f,0.084099950009584478749502522987313568592071533203125f,-1.6015000103265040198863289333530701696872711181640625f,-0.0695000024437899543983121475321240723133087158203125f,-0.5324999912828209946979995947913266718387603759765625f,-0.89570003930777264411489113626885227859020233154296875f,-1.119199998564272657830542812007479369640350341796875f,-0.051099983689934180830505283665843307971954345703125f,0.6098000040881330630071488485555164515972137451171875f,-0.25460002423226857359850328066386282444000244140625f,0.615000013560056846273482733522541821002960205078125f,0.2000999977231021631496332702226936817169189453125f,-0.1884999958574773426533965903217904269695281982421875f,0.4501999880701303968777438058168627321720123291015625f,-0.16209998709112405368415466000442393124103546142578125f,0.3416999819308521590954796920414082705974578857421875f,-0.200199995930492813389633965925895608961582183837890625f,-0.5658999799549579989843550720252096652984619140625f,-0.8565000191181899591441606389707885682582855224609375f,-0.5118999866902829154469145578332245349884033203125f,0.061400007238983977941870762151665985584259033203125f,-0.1645000227466228981398899122723378241062164306640625f,-0.777100011938810286693524176371283829212188720703125f,0.10410000499486926628378569148480892181396484375f,0.0142999895259732223706805598339997231960296630859375f,-0.1755999931931493396319865496479906141757965087890625f,-0.52030000937432030472251653918647207319736480712890625f,0.6819999909326437403933596215210855007171630859375f,0.00579999754279858503736022612429223954677581787109375f,0.3067999907925724745183515551616437733173370361328125f,0.994700003403424570791457881568931043148040771484375f,-0.172299982425570430422112622181884944438934326171875f,0.153000005304813502249317025416530668735504150390625f,1.152200033988058702760781670804135501384735107421875f,0.485299990636109956909649554290808737277984619140625f,1.06390002050101717401275891461409628391265869140625f,0.6530999993026258554351670682081021368503570556640625f,0.377700011315941441836230296758003532886505126953125f,0.2081999909535046100472754915244877338409423828125f,0.347799991641938532893618685193359851837158203125f,0.4740999859586356723895050890860147774219512939453125f,0.1627999912321567155260026993346400558948516845703125f,0.22569999389648476295633372501470148563385009765625f,-1.0547000270962716417244564581778831779956817626953125f,0.24640000269412976052763042389415204524993896484375f,-0.96059997330009938565353877493180334568023681640625f,-0.533599996300041734542674021213315427303314208984375f,-0.305199993434548400017547464813105762004852294921875f,-0.6517999900847673000470194892841391265392303466796875f,-1.53409999471306779383894536294974386692047119140625f,0.1691000194489962193955534530687145888805389404296875f,-0.4374999639391905503771340590901672840118408203125f,0.112599992805719306687706193770281970500946044921875f,-0.0560999847888954406016637221910059452056884765625f,-0.307400011068583278728283403324894607067108154296875f,-1.377400017476080762435231008566915988922119140625f,0.017400017252564570213735350989736616611480712890625f,-1.8958000352054842352345076506026089191436767578125f,0.035100004912913096433157988940365612506866455078125f,0.7659999512657524878278536561992950737476348876953125f,0.1673999925911429098590588182560168206691741943359375f,1.1948999899089329801427084021270275115966796875f,1.683200042027234832886506410432048141956329345703125f,-0.78969999625086817474084455170668661594390869140625f,-1.986100101166965004040321218781173229217529296875f,1.484800054323674345368999638594686985015869140625f,2.032800090920925839554911362938582897186279296875f,-1.215700050520897690375932143069803714752197265625f,-0.65250001057982398577905769343487918376922607421875f,-1.001300000835954495670421238173730671405792236328125f,0.71620001764148455425385009220917709171772003173828125f,0.8891999765664342358917338060564361512660980224609375f,1.2261000056505209077073459411622025072574615478515625f,0.677700010868907742889177825418300926685333251953125f,-0.27700004014372847649383402313105762004852294921875f,0.540400014957785668201495354878716170787811279296875f,0.58349999794363949234821120626293122768402099609375f,0.3490999751925463190360687804059125483036041259765625f,1.29660002436637977751843209262005984783172607421875f,1.117300022733211495307159566436894237995147705078125f,-0.51940000621080439913157533737830817699432373046875f,0.4680000455379484947115997783839702606201171875f,0.764199982154369461539999974775128066539764404296875f,-0.397199960041046296055355924181640148162841796875f,0.2089999871551990739959592247032560408115386962890625f,-0.0345000021457675387637209496460855007171630859375f,-0.76340000670254237746803482878021895885467529296875f,0.4731999748229984703584705130197107791900634765625f,0.6128000075489286047059067641384899616241455078125f,-0.351899992054701016286344383843243122100830078125f,-1.28780000441968400792802640353329479694366455078125f,-1.44990001136660584535320595023222267627716064453125f,-0.19540000236630439278684434611932374536991119384765625f,0.0322000151753429886269941562204621732234954833984375f,0.7048000297367578159679624150157906115055084228515625f,1.013800028887390514142907704808749258518218994140625f,1.23739997560381898011883095023222267627716064453125f,0.0755999538540843563083626577281393110752105712890625f,0.2133000296175480148264114177436567842960357666015625f,-1.207500006556511351618610206060111522674560546875f,-0.20160002019405454110057007710565812885761260986328125f,0.037299979445337694272666340111754834651947021484375f,0.0164999317601319805959292352781631052494049072265625f,-0.365599998781084511989547536359168589115142822265625f,1.793199966739118078606907147332094609737396240234375f,0.91219999312162425297856316319666802883148193359375f,0.5794000195845960821117159866844303905963897705078125f,0.44070000313520463208760702400468289852142333984375f,0.749500016674399471838796671363525092601776123046875f,0.7144000087812536481379765973542816936969757080078125f,-0.758099973374604996223524722154252231121063232421875f,0.8997000038504605612388331792317330837249755859375f,0.746299975392222147974052859353832900524139404296875f,-1.64399997553229315627731921267695724964141845703125f,-0.753400037398934241394954369752667844295501708984375f,-1.254399988068640059424296850920654833316802978515625f,-0.791300043006240461096467697643674910068511962890625f,-0.418699965822696640316280536353588104248046875f,0.234599993070215173673886965843848884105682373046875f,0.2828000187993053859969450058997608721256256103515625f,0.658400023466348738310216504032723605632781982421875f,-0.2371999801203605873212154619977809488773345947265625f,-0.03140002009123576964810808931360952556133270263671875f,0.5367000185132031475632174988277256488800048828125f,-1.00620002410486364397002034820616245269775390625f,0.4732999826788901742702364572323858737945556640625f,-1.5369999964460721031400680658407509326934814453125f,2.0419999662712218224669413757510483264923095703125f,0.169300077349693101069760814425535500049591064453125f,0.25659999657571297415614708370412699878215789794921875f,0.1199999938160181134350068532512523233890533447265625f,-0.05350001452118158251636259592487476766109466552734375f,-0.96369999033510700403581950013176538050174713134765625f,-0.16139999319463960159026783003355376422405242919921875f,0.9095000050514940870272084794123657047748565673828125f,-0.313000008955598207904813534696586430072784423828125f,0.3168999973818655835344770821393467485904693603515625f,0.16909995991885684585298577076173387467861175537109375f,0.373399907237291994732686362112872302532196044921875f,-1.11579998096525656592348241247236728668212890625f,-0.54870000553429021383777808296144939959049224853515625f,-0.38200005072355214252866062452085316181182861328125f,0.9783999971657990624152034797589294612407684326171875f,-0.13309997967034592480928267832496203482151031494140625f,0.10949997238069786309466735474416054785251617431640625f,-1.6609999806582924630760089712566696107387542724609375f,-0.0067000138938426712087448322563432157039642333984375f,-0.0095000163391232295140298447222448885440826416015625f,0.151099987210332908915688676643185317516326904296875f,-0.4449000508919365159954395494423806667327880859375f,0.6326000024169677882213136399514041841030120849609375f,0.5802000043869011136621338664554059505462646484375f,-0.7728999874651432122618643916212022304534912109375f,-0.72729998875856427531516601447947323322296142578125f,-0.56210002099126554764296770372311584651470184326171875f,0.77229997672587635815233397806878201663494110107421875f,-0.9854000019192696857572855151374824345111846923828125f,0.944099999891221397518847879837267100811004638671875f,-0.1825999959535897776419233196065761148929595947265625f,0.7773000193804495694394063320942223072052001953125f,0.14849998820200538407476642532856203615665435791015625f,-0.178899970805644681348667290876619517803192138671875f,-0.5545000420250001393895900037023238837718963623046875f,1.0385000150799754692343412898480892181396484375f,0.859400046853721732986741699278354644775390625f,0.840700022357701914899053008412010967731475830078125f,1.811799983751773535090023869997821748256683349609375f,-0.45350001224875413630144294074852950870990753173828125f,-0.634600012497603671590695739723742008209228515625f,-0.2256000203877686072218011759105138480663299560546875f,-0.3448000158041717444490359412156976759433746337890625f,-0.8690999773159624464113903741235844790935516357421875f,-0.20979999646991565054321426941896788775920867919921875f,0.2881999941572548351587101933546364307403564453125f,1.933800049190222747341749709448777139186859130859375f,-1.302000033512711762995195385883562266826629638671875f,-2.32190001313984328845663185347802937030792236328125f,-0.6102000009596346696838509160443209111690521240234375f,0.759700011822582066400855183019302785396575927734375f,0.24790002427101143211984890513122081756591796875f,-0.284500021293759619567254048888571560382843017578125f,-0.07969999908208880157189923920668661594390869140625f,-0.5039999904334544478246016296907328069210052490234375f,0.92830004094243090406735063879750669002532958984375f,-1.154900019860267246230023374664597213268280029296875f,0.900900038516521561859917710535228252410888671875f,-0.87030002572834508356436344911344349384307861328125f,-0.5787000382423403976872577914036810398101806640625f,0.8245999774426220341183579876087605953216552734375f,-0.500400009109079224600691304658539593219757080078125f,-0.36979999786689898844116441978258080780506134033203125f,0.9314999789595592716295868740417063236236572265625f,-0.18480000483319181103780692865257151424884796142578125f,0.054600011193752351346120121888816356658935546875f,0.343700020919740778513329360066563822329044342041015625f,-0.47740003401637043367600199417211115360260009765625f,0.65980003212094306963564349644002504646778106689453125f,-1.1483000387072568315716125653125345706939697265625f,-0.3636000243701043022070962251746095716953277587890625f,-0.41420004384517739026705385185778141021728515625f,-1.5171000127024942827347331331111490726470947265625f,0.475799985882640275036692401045002043247222900390625f,1.5034000127002602464898473044740967452526092529296875f,-1.0196999749422073211491124311578460037708282470703125f,-0.0486999984189873114104329943074844777584075927734375f,-0.2621000046372412572992516288650222122669219970703125f,-0.05590000133812356164497714416938833892345428466796875f,0.145300005313754088120958840590901672840118408203125f,-0.4352999891459938819338049142970703542232513427734375f,0.724800036963821003865859893267042934894561767578125f,1.166099995182454218678458346403203904628753662109375f,0.679900022467970988060415038489736616611480712890625f,0.9034999585673215705128313857130706310272216796875f,0.4445999898105856829033655230887234210968017578125f,-0.50580000264644553453763364814221858978271484375f,-0.348499992132186520166214904747903347015380859375f,0.718300005403161634376374422572553157806396484375f,-1.1294999847859139574524078852846287190914154052734375f,-0.347000036045908399895409957025549374520778656005859375f,-0.720299967539310248554329518810845911502838134765625f,-0.316000002041458838419885069015435874462127685546875f,0.206400000607967371024642488919198513031005859375f,-0.425000002980232327587373220012523233890533447265625f,-0.04170001553297009166954012471251189708709716796875f,0.62780001529753182154536261805333197116851806640625f,-0.75390003279447537209989604889415204524993896484375f,0.234100025856494919906936047482304275035858154296875f,-1.10850001567602163277115323580801486968994140625f,-0.5738999875880772538749852174078114330768585205078125f,-0.1698000636368994786806752017582766711711883544921875f,-0.483999974489212003714300180945429019629955291748046875f,0.2624999959021803164205266511999070644378662109375f,-0.0624999937415122719386317839962430298328399658203125f,-0.254100023025274790455796392052434384822845458984375f,0.54530000043362336636931786415516398847103118896484375f,-1.291099986204505167819434063858352601528167724609375f,0.21969998846501137546027848657104186713695526123046875f,0.1472000361859799877350951646803878247737884521484375f,1.02580002008080573006054692086763679981231689453125f,0.588300007191300711184567262534983456134796142578125f,1.673499971829355725816412814310751855373382568359375f,0.2046000084370376281839298826525919139385223388671875f,0.37539998143017327958403939192066900432109832763671875f,-0.4735999882906669622428807997494004666805267333984375f,0.861899991831183598378629540093243122100830078125f,-0.19309997657835487050448364243493415415287017822265625f,-0.5487999742746356712785882336902432143688201904296875f,0.922699995148181795201480781543068587779998779296875f,0.1578999876886604258885427043423987925052642822265625f,-0.77430001102089818232343532145023345947265625f,-1.0122000203907492998922634797054342925548553466796875f,0.07040001041293209027571720071136951446533203125f,-1.686200013804435737796438843361102044582366943359375f,-0.0630000497102738421517642564140260219573974609375f,1.4017999967157834806386063064564950764179229736328125f,-1.3373999808564784874675979153835214674472808837890625f,-0.9772000158086411314428687546751461923122406005859375f,-0.53139999796301129464382029254920780658721923828125f,0.2097999999344357657804494010633789002895355224609375f,-0.5996999773263933519729107501916587352752685546875f,0.34290001095309896594898191324318759143352508544921875f,1.56100004160404193243039117078296840190887451171875f,-0.2350999910175797236178141247364692389965057373046875f,-0.0944999791979785896955945645458996295928955078125f,0.36129999267756895964254226782941259443759918212890625f,-1.2757999847650527414089083322323858737945556640625f,-0.6494000239990647127541478766943328082561492919921875f,-0.103300027456878495968339848332107067108154296875f,-0.34659994401186622337007747773895971477031707763671875f,0.627099975281953714301153013366274535655975341796875f,-0.602399993038177594684157156734727323055267333984375f,0.34319997392892798160346501390449702739715576171875f,-0.0145999694891270070939981451374478638172149658203125f,-1.229400039365888375186841585673391819000244140625f,-0.33390005604028727415055755045614205300807952880859375f,-0.48410000290870680572652418049983680248260498046875f,-0.0042000295400625997643828668515197932720184326171875f,1.140300022077560537780982485855929553508758544921875f,1.2134000157177446954648303290014155209064483642578125f,0.51069993306249406028740622787154279649257659912109375f,-0.57930005818307250819998444058001041412353515625f,-0.69439994887858613736852930742315948009490966796875f,1.80770000095963556674405481317080557346343994140625f,-0.37859998851418463772233735653571784496307373046875f,-0.2444999966882159814218766769045032560825347900390625f,-0.0939000056743619637700248858891427516937255859375f,0.2359000113964080380668519865139387547969818115234375f,0.0695999914124614260657608610927127301692962646484375f,0.25240000231415005149671060280525125563144683837890625f,0.227899989290536009178822496323846280574798583984375f,0.07639996749013688859264448183239437639713287353515625f,-0.7181000060640283511048664877307601273059844970703125f,1.2393000576429076264872719548293389379978179931640625f,-0.21999999247491353049355211624060757458209991455078125f,0.162000006839633226007890698383562266826629638671875f,0.3789000162541871219445965834893286228179931640625f,0.3107000141248106306335330373258329927921295166015625f,-0.7257000067114827146497191279195249080657958984375f,0.87870000436082429384754277634783647954463958740234375f,-1.458499948993325023849365607020445168018341064453125f,-0.06320001613646686333680690950131975114345550537109375f,0.20069998629689234093120830948464572429656982421875f,0.04680004043206598129955864351359196007251739501953125f,-0.458400004187971443769811230595223605632781982421875f,0.08680000338405380322370774592855013906955718994140625f,-0.0787000294320285842530893205548636615276336669921875f,0.26730001161321970126749647533870302140712738037109375f,0.3535000557601453152756221243180334568023681640625f,1.459300045070052842532959402888081967830657958984375f,0.7324000200092790624495364681934006512165069580078125f,-0.6966999515324834302276713060564361512660980224609375f,0.87759996974617238141291863939841277897357940673828125f,1.4637999731570483508136248929076828062534332275390625f,-0.0254000047877429580012176302261650562286376953125f,-0.1360000196993349508289838922792114317417144775390625f,-0.098299976122378673881740951401297934353351593017578125f,0.36510001729950403159818961285054683685302734375f,0.420899995303153584558231159462593495845794677734375f,-0.7586999859765171105863146294723264873027801513671875f,-0.5732000118523836729167442172183655202388763427734375f,-0.852099987240135359201076425961218774318695068359375f,-0.918900045385957486843153674271889030933380126953125f,0.35780001026838992306267073217895813286304473876953125f,0.03949999393150205317937206928036175668239593505859375f,-0.03339999761283396484401464476832188665866851806640625f,-1.2437000225961210020386715768836438655853271484375f,-0.26190003929138239158191936439834535121917724609375f,-2.0555000127553935129753881483338773250579833984375f,0.065599979335070113251049406244419515132904052734375f,0.11870005850791987001002780743874609470367431640625f,-0.55600000679120398228150179420481435954570770263671875f,0.169600000558048302767844006666564382612705230712890625f,0.8371000086791813288300545536912977695465087890625f,-0.79579996557980814397836866191937588155269622802734375f,-0.3312000121280551656610668942448683083057403564453125f,-1.174299995113909034216703730635344982147216796875f,0.033900003290176616843609735951758921146392822265625f,-0.177900004564225611147776362486183643341064453125f,-0.490799977761507133511287293003988452255725860595703125f,0.6912000081047409150158955526421777904033660888671875f,0.66379997129440315717374687665142118930816650390625f,1.249100021386146952551143840537406504154205322265625f,1.519900025671720555919819162227213382720947265625f,1.2277000036045910835724725984618999063968658447265625f,0.18919999269694109944595084016327746212482452392578125f,0.22490002880096471304938177127041853964328765869140625f,0.81340000018328362241248896680190227925777435302734375f,-0.9636000492364169556225306223495863378047943115234375f,-0.99810000418275601763440363356494344770908355712890625f,0.033699993985891130154897155080107040703296661376953125f,-0.6068000246554612342464452012791298329830169677734375f,-0.33400000365823501180528865006635896861553192138671875f,-0.9232000375382602985752100721583701670169830322265625f,0.63380001331567736855276962160132825374603271484375f,0.46319998590573663310721030939021147787570953369140625f,-0.50899998879432661880173327517695724964141845703125f,-0.00539999763518576869625320568957249633967876434326171875f,-0.15019999604225109379740388249047100543975830078125f,-1.78469999856054784004300017841160297393798828125f,-0.951000010684132757177167150075547397136688232421875f,-0.173400013072788627965792329632677137851715087890625f,-0.577799993467330796903524969820864498615264892578125f,0.6280000149533153663838902502902783453464508056640625f,-1.0719000241294496245103573528467677533626556396484375f,-0.468300029990076804864429504959844052791595458984375f,-0.5422000583887101043956135981716215610504150390625f,0.467600031101703894620413848315365612506866455078125f,1.33089999053478180712772882543504238128662109375f,-0.382200027468801106778073517489247024059295654296875f,-0.142499948367475592903019787627272307872772216796875f,-0.11899997776746740196784912768634967505931854248046875f,-0.918100001463293846626356753404252231121063232421875f,-0.3904000107109546480188555506174452602863311767578125f,-0.67719999673515562932379907579161226749420166015625f,0.900200008857249489579999135457910597324371337890625f,0.8010000160112975553516889704042114317417144775390625f,-0.3011999790847301738949681748636066913604736328125f,1.476500052720308264753157345694489777088165283203125f,-0.55380002017021201510260652867145836353302001953125f,-0.693800026726723029213417248683981597423553466796875f,-0.69999998405575691862168241641484200954437255859375f,-1.5358000215709211033754399977624416351318359375f,-0.807000001996755589317444901098497211933135986328125f,1.052900001472234947641481994651257991790771484375f,0.21229999132901433700482130007003434002399444580078125f,-0.47240002697557204047740242458530701696872711181640625f,-0.9696000147327776819139444341999478638172149658203125f,-0.1934999896734959090593974906369112432003021240234375f,0.37949998545646701497702224514796398580074310302734375f,-0.120999995708465757360272618825547397136688232421875f,0.73280001831501717646233373670838773250579833984375f,-0.745900028383732038861353430547751486301422119140625f,-0.5950000108964743628092719518463127315044403076171875f,-0.95229997773170538266640505753457546234130859375f,-0.4841000090926887988729276912636123597621917724609375f,-0.6965999734044074909178334564785473048686981201171875f,-1.0649999951198696468424031991162337362766265869140625f,0.59809998205453229758177258190698921680450439453125f,0.04009997093826600522703529350110329687595367431640625f,0.5078999617233870278454332947148941457271575927734375f,-0.6888000320166351997386300354264676570892333984375f,1.14419995622634917964433043380267918109893798828125f,-0.87340001623928531504503780524828471243381500244140625f,0.27900003454089183918540584272705018520355224609375f,0.3646999922275540040317309831152670085430145263671875f,-0.4163000444591045123843287001363933086395263671875f,-0.752500007674097748378017058712430298328399658203125f,0.43439999961704034436849042322137393057346343994140625f,-0.47349994500726511059696122174500487744808197021484375f,-0.6144999783784153013499462758773006498813629150390625f,-0.44809999948889023269060771781369112432003021240234375f,-0.1257999845787889370285483892075717449188232421875f,-1.2098999966144556861280534576508216559886932373046875f,-0.75670001642703965671898913569748401641845703125f,0.267899997448921123321952109108678996562957763671875f,-1.1275000365078451380895785405300557613372802734375f,0.443499940276146364936948884860612452030181884765625f,-0.512000013731420011708905803970992565155029296875f,0.1004000020124018177636315840572933666408061981201171875f,0.373700010060518839072329910777625627815723419189453125f,-0.805900018846988697163169490522705018520355224609375f,0.415000028163194922825596222537569701671600341796875f,1.1949000154085458458297352990484796464443206787109375f,-0.2022000014662739264537094641127623617649078369140625f,0.74480004407912492325749553856439888477325439453125f,-0.5189000261634590938086830647080205380916595458984375f,0.13440005838349566857914396678097546100616455078125f,-0.138900036966801199156407164991833269596099853515625f,0.6316000091485676204428045821259729564189910888671875f,-0.40730001565515949746298929312615655362606048583984375f,-0.37660001350715754941944624079042114317417144775390625f,-0.4362000291153791398102157472749240696430206298828125f,1.6685000254362822857956416555680334568023681640625f,0.83480002467036218405382896889932453632354736328125f,-0.726099985608458897701211753883399069309234619140625f,-0.103300005328655242209379139239899814128875732421875f,-1.548600007215142237981808648328296840190887451171875f,-0.57549995985627244721172246499918401241302490234375f,-0.462199991258978482022712341859005391597747802734375f,-0.2424999928474420585189363919198513031005859375f,-1.02369997081458574683665574411861598491668701171875f,0.7260999975480135759653421700932085514068603515625f,-0.18619999910816520394263307025539688766002655029296875f,0.24250001592561620800125865571317262947559356689453125f,0.151099970483779433294557748013176023960113525390625f,0.6859999942183490073688290067366324365139007568359375f,0.396900038546323941091031883843243122100830078125f,0.4086000264376405954891424698871560394763946533203125f,-0.295500007018446364526198522071354091167449951171875f,-1.533500026032328822367389875580556690692901611328125f,-0.90570000149607654549299695645458996295928955078125f,-2.10330007119178841179518713033758103847503662109375f,0.050699966478347402443205282907001674175262451171875f,-0.04159999299943450967731450873543508350849151611328125f,0.41030001755133305163525392345036379992961883544921875f,-0.04479999118000288493846028359257616102695465087890625f,0.7163000377535826412866981627303175628185272216796875f,0.029199998694658280129488048260100185871124267578125f,0.47270001353248947140173186198808252811431884765625f,-0.28260001068711293470414602779783308506011962890625f,-0.1253000219844292162374443933003931306302547454833984375f,-0.353300005924701121529096781159751117229461669921875f,-0.0140999705918138096283342974857077933847904205322265625f,0.6211999796807763374317801208235323429107666015625f,0.07090002680048380911959071681849309243261814117431640625f,1.6023999677434563881917028993484564125537872314453125f,0.22170001150965734648679017482209019362926483154296875f,1.3465999817863101295500882770284079015254974365234375f,0.578700010228156980218727767351083457469940185546875f,0.650299993765354855668192612938582897186279296875f,-0.5813000134646892913536930791451595723628997802734375f,-1.491400004780292221084891934879124164581298828125f,-0.1577999610573057598372770371497608721256256103515625f,-2.13299998205900198655626809340901672840118408203125f,-0.9944000175356866311204839803394861519336700439453125f,-1.06779997826814732064804047695361077785491943359375f,0.2082000019989909134210392949171364307403564453125f,-0.47009995890557776387908006654470227658748626708984375f,-0.79460000717043877127565565388067625463008880615234375f,-1.095800019484758802690294032800011336803436279296875f,0.28890001893639549734871252439916133880615234375f,-0.53960001521706590921212409739382565021514892578125f,1.081500004738569220563704220694489777088165283203125f,0.14340000737309477329972651205025613307952880859375f,0.52269997026324244160377929802052676677703857421875f,-0.380399997895955355176056400523521006107330322265625f,-0.602800010901688665398978628218173980712890625f,-0.3855000017285343805184538723551668226718902587890625f,-0.41310000615715980476494451067992486059665679931640625f,0.4881999985158440580335081904195249080657958984375f,0.25139999034479243444906160220853053033351898193359375f,0.33180000118613239834530759253539144992828369140625f,-0.81250004207715409432211117746192030608654022216796875f,0.58810002374053027551781269721686840057373046875f,1.52040001831054638614659779705107212066650390625f,-0.836499997287988517058465731679461896419525146484375f,0.8665000631324948354716752874082885682582855224609375f,0.027199977922439444455449120141565799713134765625f,0.19700001458823657163321740881656296551227569580078125f,0.2107999914705753763399798117461614310741424560546875f,-1.299100041186064924403353870729915797710418701171875f,0.526499955415726361707129399292171001434326171875f,-0.540900002753734288063469648477621376514434814453125f,-0.2561000296413897814318261225707828998565673828125f,-0.03259999548792846812972356929094530642032623291015625f,1.69929995797276500724137804354541003704071044921875f,0.00820001371502865339380150544457137584686279296875f,1.8835000076293937354421359486877918243408203125f,0.46120000829100593620069048483856022357940673828125f,0.049800030481815582561466726474463939666748046875f,-0.515600005039572817366888557444326579570770263671875f,-1.4674999809265134587121792719699442386627197265625f,0.6741999942243097621030756272375583648681640625f,-0.22360001116395000764214273658581078052520751953125f,0.204900017029046477290421535144560039043426513671875f,0.45510001977682090767984846024774014949798583984375f,-1.654399974918365767706518454360775649547576904296875f,-0.30239995295405375230757272220216691493988037109375f,-0.25680000506043398900146712549030780792236328125f,-1.48150001323223090565761594916693866252899169921875f,0.327699996787309988377501213108189404010772705078125f,-1.07090004045367237495156587101519107818603515625f,0.71829999400377264606731841922737658023834228515625f,0.081999998420477115956828129128552973270416259765625f,-1.4096000176012515314738493543700315058231353759765625f,0.18630000020265580218392642564140260219573974609375f,-0.97210002182945609572328748981817625463008880615234375f,0.0757000141620636224359941479633562266826629638671875f,-0.62870002406760983060252101495279930531978607177734375f,0.300499990850686682364312218851409852504730224609375f,1.1033000040620570825211643750662915408611297607421875f,-0.77049997952580451254789295489899814128875732421875f,-0.105699999875575378904812851033057086169719696044921875f,-0.3875000047683716530855235760100185871124267578125f,0.32400002725794951885518457856960594654083251953125f,0.07479996744990291546173466485925018787384033203125f,-0.5080000067017971243643614798202179372310638427734375f,-0.31690000963807118949944197083823382854461669921875f,-0.25920002648532403100034571252763271331787109375f,0.0750000196695328469331798260100185871124267578125f,0.026800009605289165648400739883072674274444580078125f,1.94699993017315886589813089813105762004852294921875f,-0.07930001467168334983171007479541003704071044921875f,1.6422999701321128185327324899844825267791748046875f,0.10969999699592591912988837066222913563251495361328125f,-0.73429999179840077516701057902537286281585693359375f,-0.04410001244544969889460617196164093911647796630859375f,-0.77250004902481972379746366641484200954437255859375f,1.065299975958466927039580696146003901958465576171875f,-0.2861999933898449199887181748636066913604736328125f,-0.10279999614953982955967148882336914539337158203125f,0.3804000338077546672366224811412394046783447265625f,-1.8551999626636510587474049316369928419589996337890625f,-0.24220000176429845595293954829685389995574951171875f,0.332000024795532144850085387588478624820709228515625f,-0.60449998337030397266289583058096468448638916015625f,0.601999988436698885152509319595992565155029296875f,-0.03250000536441799425801946199499070644378662109375f,-0.125999992281198291976807013270445168018341064453125f,-0.384500035077333546240652140113525092601776123046875f,0.74259995793700195321207502274774014949798583984375f,0.653500002264976398436147064785473048686981201171875f,-0.27900000762566923295793230863637290894985198974609375f,-0.48290000221729290075245444313623011112213134765625f,-0.66610002988353411712552087919902987778186798095703125f,0.064500004976987934668386515113525092601776123046875f,0.1180999868601559921188481894205324351787567138671875f,0.3508999807745218912913287567789666354656219482421875f,0.214499997824430355120028934834408573806285858154296875f,-0.364899989685416314966204254233161918818950653076171875f,-0.0822000206328931948274174601465347222983837127685546875f,0.30520000159293447783426245223381556570529937744140625f,-0.3404999960102139677786681204452179372310638427734375f,0.254000037670135014877814683131873607635498046875f,0.29929995603561376782408842700533568859100341796875f,-0.61539997986555050601964467205107212066650390625f,0.7881999986648562384061733609996736049652099609375f,-1.2686999458551415642659776494838297367095947265625f,-0.30470005145966982951222234987653791904449462890625f,-0.92029998601675000458044451079331338405609130859375f,0.9516999972790483042217601905576884746551513671875f,0.0473000271290537899204764471505768597126007080078125f,0.3029000093698499540550983510911464691162109375f,-0.1360999984979631705073188641108572483062744140625f,0.65760006850361829577877870178781449794769287109375f,0.6507999715775250049176747779711149632930755615234375f,1.239700036633015312048655687249265611171722412109375f,0.5884000204861166150038798150490038096904754638671875f,0.62079998744726150317063684269669465720653533935546875f,-1.0705000237822535069653895334340631961822509765625f,1.2306000075727698828131906338967382907867431640625f,0.808100027018785471710771162179298698902130126953125f,1.2039000275418165752938648438430391252040863037109375f,0.7191000266760589898495936722611077129840850830078125f,-0.1918999735027551167831916245631873607635498046875f,-1.3845000312030315559042037421022541821002960205078125f,-1.6248000033989546952994942330406047403812408447265625f,-1.4376000263519588262539627976366318762302398681640625f,-0.453999996319412790768410559394396841526031494140625f,0.75470003236755778264210903216735459864139556884765625f,-1.5109999918341632252349882037378847599029541015625f,-0.4882000236243009272385506847058422863483428955078125f,-0.3119000209629536612965239328332245349884033203125f,-0.5332000146090984937785606234683655202388763427734375f,0.475000017806888052973590674810111522674560546875f,0.96470003797411951751428205170668661594390869140625f,0.552300047692657170017582757282070815563201904296875f,0.940100036372989666944022246752865612506866455078125f,1.889500030159950938468682579696178436279296875f,0.263300062102079124315423541702330112457275390625f,1.0170000668168073243435856056748889386653900146484375f,0.8075999745517961070362389364163391292095184326171875f,-2.099000057935715091161910095252096652984619140625f,0.01170000808239013423417418380267918109893798828125f,0.29329999325871458637493560672737658023834228515625f,0.6584000274896624205211992375552654266357421875f,0.964000039085746163181056544999592006206512451171875f,0.094699993419646855130622498109005391597747802734375f,-0.6117000396728518030187160547939129173755645751953125f,-0.4700000046938657138895223397412337362766265869140625f,1.060000026896596114767135077272541821002960205078125f,-1.310800023806094838363378585199825465679168701171875f,0.12749999314546567319439418497495353221893310546875f,0.3294000263273719752987744868732988834381103515625f,0.75230003592073912965787485518376342952251434326171875f,-0.935299997341632494141094866790808737277984619140625f,-0.4076000151574608043603120677289552986621856689453125f,0.13400000116229104918375014676712453365325927734375f,2.488499963708221685010357759892940521240234375f,0.25930003203153528801294669392518699169158935546875f,0.1866000327110295398114203635486774146556854248046875f,0.0042999715328218002241555950604379177093505859375f,0.6494999937638628040303956368006765842437744140625f,0.65220001938492055959528670427971519529819488525390625f,1.2323000233292571437004880863241851329803466796875f,-0.060700007419288215526620433593052439391613006591796875f,0.1758000169515607780112986802123486995697021484375f,0.67310001012459375058227806221111677587032318115234375f,0.4731000012397768017535781837068498134613037109375f,-1.212799999129771322259330190718173980712890625f,-1.19439996329546005426891497336328029632568359375f,0.6656999859616157255715052087907679378986358642578125f,1.00039996168613498639388126321136951446533203125f,0.269600029317289602293072903194115497171878814697265625f,-1.0959999521970757996314205229282379150390625f,0.10140000030994489321756191202439367771148681640625f,-0.105499996364116999103544003446586430072784423828125f,0.0501000297605997957361978478729724884033203125f,-0.700600039237738059227922349236905574798583984375f,-0.4983000002622599566848293761722743511199951171875f,-0.752099982285499013556773206801153719425201416015625f,1.3765000175535675452920258976519107818603515625f,-0.455299994063377511110957129858434200286865234375f,-0.2462999793410300952700708876363933086395263671875f,0.421399989879130743020141380839049816131591796875f,1.63129994089603425067025455064140260219573974609375f,-1.3320999906301498327820809208787977695465087890625f,-0.55000001065432979618918807318550534546375274658203125f,-0.0247000330567364034806132622179575264453887939453125f,1.191000012397765761562595798750407993793487548828125f,0.2160000138133764124859226285479962825775146484375f,1.05010000174641593417845797375775873661041259765625f,-0.8360999752521516281689173410995863378047943115234375f,-1.723199988830088624780501049826852977275848388671875f,-0.183399995638430457489675973192788660526275634765625f,0.6274999764561648429861406839336268603801727294921875f,-0.3986999769613148725255769022624008357524871826171875f,0.473699994656443745810747714131139218807220458984375f,1.3673000364422798202923559074406512081623077392578125f,-0.2814999907314772453759132986306212842464447021484375f,-0.553400019741058457611870835535228252410888671875f,-0.7269000320084393340636097491369582712650299072265625f,0.3413999746799465384583527338691055774688720703125f,0.81379999081492371271195906956563703715801239013671875f,-1.431599949264526827619192772544920444488525390625f,0.04289998321831289818106824895949102938175201416015625f,-0.7201000421285623787071017432026565074920654296875f,-0.28200002030655768603395472382544539868831634521484375f,-0.99010004689693520418813932337798178195953369140625f,-0.54470005546435762422419202266610227525234222412109375f,-0.857199967789650685290325782261788845062255859375f,0.62999997515231331846052853506989777088165283203125f,0.65640001804232550597362205735407769680023193359375f,0.09679999567270325400158981210552155971527099609375f,1.447200037229059876864312172983773052692413330078125f,0.82589993703961450677297762013040482997894287109375f,-1.47799995419382934613850011373870074748992919921875f,0.86499996960163105796937088598497211933135986328125f,0.69949997942149533258771043620072305202484130859375f,0.344599965447188072431572436471469700336456298828125f,0.1227999994277941819831312386668287217617034912109375f,0.949999980852008985010570540907792747020721435546875f,0.241900074160099620712571777403354644775390625f,-0.7879000175654891080512243206612765789031982421875f,-1.80289999230802067131662624888122081756591796875f,-0.20270006965398845721892939764074981212615966796875f,0.9976999793902041613335995862144045531749725341796875f,1.064600053513050426090558175928890705108642578125f,-0.9915999771296970521916591678746044635772705078125f,-0.9139000572323805471341984230093657970428466796875f,-1.545400029449165213435435362043790519237518310546875f,-0.70610001131296140641779857105575501918792724609375f,-0.148000016666949729682301040156744420528411865234375f,1.2442000331163409754253734718076884746551513671875f,0.8171000331729654231338599856826476752758026123046875f,0.48310004134252704233176700654439628124237060546875f,0.6991999416232115294178584008477628231048583984375f,-0.73210000631362237566435169355827383697032928466796875f,-0.297499965429306456599078956060111522674560546875f,0.26190000041797778518315453766263090074062347412109375f,1.08849998259544378242935636080801486968994140625f,1.0050000339373943436527270023361779749393463134765625f,-0.514000004589558301404395024292171001434326171875f,-0.4852000350832941233392148205894045531749725341796875f,0.015099967622756338414546917192637920379638671875f,-0.75280004707425884902249890728853642940521240234375f,-1.776099985980987838729561190120875835418701171875f,-0.39799997654557284221255031297914683818817138671875f,0.34629992688894173369362761150114238262176513671875f,0.50010000427961376345820099231787025928497314453125f,-0.04629993433952250825313967652618885040283203125f,-0.90829996032714888087866711430251598358154296875f,1.0790999939680094854566050344146788120269775390625f,0.936399953222274206154907005839049816131591796875f,-0.7873000033617021387044587754644453525543212890625f,-0.7851000340819354050836409442126750946044921875f,0.3776000129222865098199690692126750946044921875f,0.57609998300075648813844964024610817432403564453125f,-0.830799982976913042875821702182292938232421875f,1.1714000114858149625973737784079276025295257568359375f,0.4462000003188848484825257401098497211933135986328125f,-0.360200017574429320887929861783050000667572021484375f,0.0011000143676997975461517853545956313610076904296875f,-1.428900013273954439085855483426712453365325927734375f,-0.46310002042502185215511190108372829854488372802734375f,0.67110001395791751832575755543075501918792724609375f,-0.27360004826784167786257739862776361405849456787109375f,0.5236000194713472666308007319457828998565673828125f,-0.03659998122751739746405519326799549162387847900390625f,-0.091699975672363986944901625975035130977630615234375f,-1.174500035114585561046851580613292753696441650390625f,0.1636000092826785934363442720496095716953277587890625f,-0.18130000377893384921890174155123531818389892578125f,0.3194000069372353589614021984743885695934295654296875f,-0.2855999674141411759364928002469241619110107421875f,0.055499996941537066419414259144105017185211181640625f,0.484199987705051793085431199870072305202484130859375f,-0.265499999642372142005797286401502788066864013671875f,0.407200009289383491051239616353996098041534423828125f,-0.6176000149525702198616272653453052043914794921875f,0.84820002429485352735127889900468289852142333984375f,0.59800003877654683837050697547965683043003082275390625f,0.5812999568402776873199400142766535282135009765625f,-0.603599964225292939801192915183492004871368408203125f,-1.3762000322818759290299794884049333631992340087890625f,-0.373300020900368512144495980464853346347808837890625f,-0.656600017958879789148340933024883270263671875f,0.42149999922513980976646053022705018520355224609375f,1.08080000506788476144492960884235799312591552734375f,-1.15530001701116535883784308680333197116851806640625f,-0.226000000290572433669922247645445168018341064453125f,-0.98019996672421683125975278016994707286357879638671875f,0.9059000028282415684088846319355070590972900390625f,-1.0154999793022876364290141282253898680210113525390625f,-0.8076000437676906784645325387828052043914794921875f,0.590499975949525524043792756856419146060943603515625f,-0.000700046963244826958572275543701834976673126220703125f,0.1293000132746995323174132863641716539859771728515625f,-0.0263999881096186328477415372617542743682861328125f,0.49169999927207841405873978146701119840145111083984375f,0.3524999879300594329833984375f,-0.45010000126212845739104295716970227658748626708984375f,0.82449999458342748681616285466589033603668212890625f,-0.21559998946785940976411666269996203482151031494140625f,-0.35339997444152826400198819101206026971340179443359375f,-1.14610001172274333924860911793075501918792724609375f,1.12250000659376336642480964655987918376922607421875f,0.107300013066083332891054169522249139845371246337890625f,-0.1668999902851882044529929771670140326023101806640625f,-0.4299000134713948373388348045409657061100006103515625f,1.0392999847203496788239363013417460024356842041015625f,0.0498000142022967662303045699445647187530994415283203125f,-0.569699957992136685192008371814154088497161865234375f,0.349299990662187642698199852020479738712310791015625f,-0.0055999832466249177542749748681671917438507080078125f,0.50989998905211653124069925979711115360260009765625f,-0.651300019611418701259708541329018771648406982421875f,-0.4984000193126500999341033093514852225780487060546875f,-0.1319000176474449670394051281618885695934295654296875f,-1.13109999432563856913702693418599665164947509765625f,-1.0008000040061768576293843580060638487339019775390625f,-1.2753999763637782205449866523849777877330780029296875f,0.8233000014729794369117144015035592019557952880859375f,1.04139996329918549378135139704681932926177978515625f,-0.703099980489910070247105977614410221576690673828125f,0.33639998339712651187483061221428215503692626953125f,-0.35600000889599370257343480261624790728092193603515625f,0.22419999061077888757864684521337039768695831298828125f,-0.889400033442675930217546920175664126873016357421875f,0.404400007775425951450642969575710594654083251953125f,0.3680999918334182385848407648154534399509429931640625f,0.70570003874898024065487334155477583408355712890625f,0.174899971953034860661091443034820258617401123046875f,0.62669999139308885105492663569748401641845703125f,-1.315599968494475913161068092449568212032318115234375f,-1.300900002604722605070719509967602789402008056640625f,-1.2968000039428464464918988596764393150806427001953125f,-0.388900041139125907108109458931721746921539306640625f,0.908499964080750288530907710082828998565673828125f,0.798300042581558511045614068279974162578582763671875f,0.9412999891757964920913082096376456320285797119140625f,0.629300021600723180625891473027877509593963623046875f,0.5692999815911050642824875467340461909770965576171875f,1.23420002670884088047387194819748401641845703125f,0.377599977755547211444309141370467841625213623046875f,0.64310001137256644909712122171185910701751708984375f,0.21640000343918774916573966038413345813751220703125f,1.33639998049139929747752830735407769680023193359375f,0.6443999740242958296221331693232059478759765625f,1.3938000241190200423346823299652896821498870849609375f,1.69569997504651581721191178075969219207763671875f,0.58039996205866317691146605284302495419979095458984375f,0.07579999206662169086712310672737658023834228515625f,0.1021999937176705230967854731716215610504150390625f,-1.6809000006675720584325972595252096652984619140625f,0.232499984502792411689142682007513940334320068359375f,-0.1621999992497265008495332949678413569927215576171875f,-0.296600017511844615825111759477294981479644775390625f,-0.6725000154040754463125040274462662637233734130859375f,0.251700028347969340103418289800174534320831298828125f,-0.253500014707446152950609530307701788842678070068359375f,-0.7310000123977662411789424368180334568023681640625f,-0.2865000155977905915705150619032792747020721435546875f,0.301400005376339041873734458931721746921539306640625f,0.113299993724375713810559318517334759235382080078125f,0.335200030612945898411680900608189404010772705078125f,1.6299000417463480605562153868959285318851470947265625f,-1.32619996381103977256543657858856022357940673828125f,-0.315899973323940985725499785985448397696018218994140625f,0.365900012066960211853938744752667844295501708984375f,-0.2871000153660776543773636149126105010509490966796875f,0.1693000323295592579597723670303821563720703125f,-0.8202000167548657572069714660756289958953857421875f,-0.190099989825487369898837641812860965728759765625f,0.24200000683963296399525688684661872684955596923828125f,1.3640999976187941911121015436947345733642578125f,1.3876999999910590855023428957792930305004119873046875f,1.947400029471516358370308807934634387493133544921875f,0.004400037205219309299764063325710594654083251953125f,0.47909999076426057484923148876987397670745849609375f,-1.0255000114887955486864257181878201663494110107421875f,-1.41910003665983719400855989078991115093231201171875f,0.046400005525350618285074233426712453365325927734375f,0.4459000438809397337536211125552654266357421875f,-0.2637999966263766538077106815762817859649658203125f,-0.12369999487996086173779985983856022357940673828125f,-1.3953000177562235872841256423271261155605316162109375f,-0.785500023409724246192809005151502788066864013671875f,0.0778999826222659397245706713874824345111846923828125f,0.27560000325143363397728535346686840057373046875f,0.46629999945759781354581718915142118930816650390625f,-0.3847999886095518906614643128705210983753204345703125f,0.34579998897463026796827989528537727892398834228515625f,0.183699984747171196630688427831046283245086669921875f,-0.436700045856834140334257199356216005980968475341796875f,0.2625000257790091051646186315338127315044403076171875f,1.088800007746368425642913280171342194080352783203125f,-0.841500025004148444196516720694489777088165283203125f,-1.51770000503882673825728488736785948276519775390625f,0.621800014746188622893896535970270633697509765625f,1.2504999607317142107376639614813029766082763671875f,-0.89249999538063917725594365037977695465087890625f,0.107500028777867295826098370525869540870189666748046875f,2.458800009441375777896610088646411895751953125f,-0.010300022226572114192322260350920259952545166015625f,1.37509996568560577401285627274774014949798583984375f,1.0775999805867668701608863557339645922183990478515625f,1.9227999860167503953789491788484156131744384765625f,-1.36669998498558964428184481221251189708709716796875f,-0.00199998351931629514410815318115055561065673828125f,0.070499964699149320068727320176549255847930908203125f,1.2718999695539476846306570223532617092132568359375f,-0.49599995592236556518628276535309851169586181640625f,1.527899976587296038132990361191332340240478515625f,-0.96100004644691949806656339205801486968994140625f,0.3469000352680691179330096929334104061126708984375f,-0.4907999720990654513030904126935638487339019775390625f,0.340000024288892976898068809532560408115386962890625f,-0.27630000377893448870736392564140260219573974609375f,0.4716000321149831364664350985549390316009521484375f,0.4496000304162499627835813953424803912639617919921875f,0.004600008660554433248535133316181600093841552734375f,-1.71209998347759206893670125282369554042816162109375f,-0.34899998253583941476563268224708735942840576171875f,-0.32509997119903655260486630140803754329681396484375f,-1.112199985224008091933001196593977510929107666015625f,-0.8115999926269044095761273638345301151275634765625f,-0.974300009456276594477230901247821748256683349609375f,-0.23159998288527117171753388902288861572742462158203125f,-0.1238000176370144256310368291451595723628997802734375f,0.45489999767616329240382810894516296684741973876953125f,0.185699979442358387160538768512196838855743408203125f,0.5080000404529270152664821580401621758937835693359375f,-0.1738000106334689487397326956852339208126068115234375f,-0.0110999960951509157869310229216353036463260650634765625f,-0.28690000039935059650275661624618805944919586181640625f,0.340100013294815095576950625400058925151824951171875f,-0.2813000394672153792186009013676084578037261962890625f,-0.988300010991097099832813910325057804584503173828125f,-1.0413999540790916942256671973154880106449127197265625f,-1.005499988615512929612805237411521375179290771484375f,-1.9894999932050705382380328956060111522674560546875f,-0.54649997001886418956928537227213382720947265625f,0.14129995862841671083742767223156988620758056640625f,-0.250600001388787685385750592104159295558929443359375f,-0.3958999660968789413573176716454327106475830078125f,-0.69859997480511726308805009466595947742462158203125f,-1.7277999673902986188522845623083412647247314453125f,-1.321899997866154308212571777403354644775390625f,-1.795299983558058531940559987560845911502838134765625f,0.064399982666969446398752552340738475322723388671875f,-0.3132000112935902080835148808546364307403564453125f,0.0225000099837782130407504155300557613372802734375f,-0.145699991512298510087930480949580669403076171875f,-1.2641999757468702636487023482914082705974578857421875f,-1.7360999953687183960937545634806156158447265625f,0.68579998208284376204346699523739516735076904296875f,-0.79840000826716472914768019109033048152923583984375f,0.1709999930262566802952051148167811334133148193359375f,-0.5985999942511313065551803447306156158447265625f,0.06500001251697573589893863754696212708950042724609375f,-0.990699986967443546603817594586871564388275146484375f,-1.63980002437233896017687584389932453632354736328125f,-0.020499975204467890677051400416530668735504150390625f,1.2395000206604598336213030052022077143192291259765625f,-0.85690000934004739718830023775808513164520263671875f,-1.140799994990974397524041705764830112457275390625f,1.41959999063014929987502910080365836620330810546875f,1.435700024145841968703507518512196838855743408203125f,-0.96680001593828190920021370402537286281585693359375f,-0.006699994969368183728164467538590542972087860107421875f,0.46940000770092016324497308232821524143218994140625f,0.272399990952014281475612733629532158374786376953125f,-0.008800010335445307418922311626374721527099609375f,-0.84410000099018234198666732481797225773334503173828125f,1.42850002297759015590372655424289405345916748046875f,-0.2997999856293207532331734910258091986179351806640625f,-1.2545999569535251083607363398186862468719482421875f,-0.169700023445486625206513053853996098041534423828125f,0.1901999829292293497928767465054988861083984375f,0.8028000334024429918144960538484156131744384765625f,1.006299974274636355175971402786672115325927734375f,0.21150000958144676843630804796703159809112548828125f,-1.5160000278949734564548634807579219341278076171875f,-1.339500004157424317696722937398590147495269775390625f,-1.177500028014183186542140902020037174224853515625f,0.101899990564584985719420728855766355991363525390625f,0.53989997634887654243129873066209256649017333984375f,1.54419999958872811163246296928264200687408447265625f,0.19659998413324419885839233756996691226959228515625f,-1.64460000866055455759351389133371412754058837890625f,0.05849998661875677186117172823287546634674072265625f,0.72740002135038306363412630162201821804046630859375f,-1.0405000374913218053052332834340631961822509765625f,1.06990001121759359392626720364205539226531982421875f,0.73780003489255907567212489084340631961822509765625f,-0.3800999861001965740570085472427308559417724609375f,-0.402099969768523468616194804781116545200347900390625f,0.15119998340606688458365169935859739780426025390625f,0.56829994661807947675669083764660172164440155029296875f,-0.45660004684850630241044200374744832515716552734375f,1.2707000444859273269315735888085328042507171630859375f,0.3017000092372297270770786781213246285915374755859375f,-0.95890001387000101118474049144424498081207275390625f,-0.39890001781880857212314595017232932150363922119140625f,-0.05689998952150332467425641880254261195659637451171875f,-0.41980000121965976855875624096370302140712738037109375f,0.53250000435858935166066885358304716646671295166015625f,-0.27130000331327275286952271926566027104854583740234375f,-0.0268999738007786870497284326120279729366302490234375f,0.79110001794397877272757568789529614150524139404296875f,-1.21969999818801877466967198415659368038177490234375f,0.05290000124871685471816817880608141422271728515625f,1.412199984255433360402776088449172675609588623046875f,0.58739999840259571328715537674725055694580078125f,0.6492000124037264896514898282475769519805908203125f,0.57990001956224446200138800122658722102642059326171875f,-1.2708000233590601713018486407236196100711822509765625f,0.042799983446300260414574267997522838413715362548828125f,-0.49310002605021008292851547594182193279266357421875f,1.340899983456731003883533048792742192745208740234375f,0.52390000850558227085684848134405910968780517578125f,-1.0236999885469675319882298936136066913604736328125f,-0.190900061240792950911782099865376949310302734375f,-1.204299998056888920672236054087989032268524169921875f,0.064200039151311560914336951100267469882965087890625f,1.090300020587444240760532920830883085727691650390625f,0.227200017857551284095052324119023978710174560546875f,0.75319999635517564939846124616451561450958251953125f,1.024700016851723294308840195299126207828521728515625f,0.1268000109836460165269045319291763007640838623046875f,1.1833000184044240032932293615886010229587554931640625f,0.381299948421121026598257230944000184535980224609375f,0.71049996127188175254474344910704530775547027587890625f,0.695000020563602216583376502967439591884613037109375f,0.478699971485137876214821517351083457469940185546875f,-0.8689000165894624760909437100053764879703521728515625f,1.1240000208318232655102519856882281601428985595703125f,0.94709998211786106114828953650430776178836822509765625f,-0.70019999717846470588256124756298959255218505859375f,-0.09029998465701949417194782654405571520328521728515625f,0.3745999929398298178284676396287977695465087890625f,-0.1632000137895346369987237267196178436279296875f,0.2459000093102454709281801115139387547969818115234375f,0.261899995779990957256444517042837105691432952880859375f,-0.8424000069707633375770683414884842932224273681640625f,-1.3744000662252313649247525972896255552768707275390625f,-1.98419997738599729331099297269247472286224365234375f,0.16749998308718228035019137678318656980991363525390625f,-0.16080003244876850398981105172424577176570892333984375f,-0.43809997354224350374352070502936840057373046875f,0.6752999872460960784081862584571354091167449951171875f,-0.1462999943912028921744195031351409852504730224609375f,0.64960000087469804697093422873876988887786865234375f,0.136099983485042930198005706188268959522247314453125f,1.3923999643161895090059942958760075271129608154296875f,0.0333999841272831776706198070314712822437286376953125f,-0.381499988757073804634956104564480483531951904296875f,0.222600003292411141586626399657689034938812255859375f,0.2564999569058414863320649601519107818603515625f,-0.2859999912753703465995158694568090140819549560546875f,1.215600029384344527016992287826724350452423095703125f,-0.15789998759552859031174421033938415348529815673828125f,-0.6805999904550608103903641676879487931728363037109375f,-0.61949997740983964167327258110162802040576934814453125f,-0.5853999792322517325970920865074731409549713134765625f,-0.82169998003095379157656452662195079028606414794921875f,-0.37059997385889265064662367876735515892505645751953125f,1.262899999535083583879213620093651115894317626953125f,0.3619000071421265563031965939444489777088165283203125f,-0.606100015597045516102525652968324720859527587890625f,-0.714800024521351051731699044466949999332427978515625f,-0.0446999927304683186690681395702995359897613525390625f,0.78519999142289176319309262908063828945159912109375f,1.123700028500705716538732303888536989688873291015625f,-0.513499991238117292624565379810519516468048095703125f,-0.16049998820573119218124702456407248973846435546875f,0.272599982374906790738577910815365612506866455078125f,-1.30870004656836425027677250909619033336639404296875f,0.698200023102760525262056034989655017852783203125f,0.147899988321959885428924508232739754021167755126953125f,-0.6171000165581703722494921748875640332698822021484375f,0.6230000304505234343110942063503898680210113525390625f,-0.0216000192999843410035509805311448872089385986328125f,-0.7486000106237826745569918784894980490207672119140625f,0.04599997905641772799611999289481900632381439208984375f,-0.742100008884073236714584709261544048786163330078125f,-0.71540001516267626158196435426361858844757080078125f,-0.87640000999569878104011877439916133880615234375f,0.0625000131130217795316639239899814128875732421875f,0.62299997192621248132127220742404460906982421875f,-1.250400012499093715945264193578623235225677490234375f,0.17640001655221038134868649649433791637420654296875f,-1.679699950653315010384858396719209849834442138671875f,1.1576999784588810626928534475155174732208251953125f,-0.5567999774187806227843111628317274153232574462890625f,-1.12890003927648141512918300577439367771148681640625f,1.026800032553076835739602756802923977375030517578125f,-1.382099997633695398491227024351246654987335205078125f,-0.35010005434751523267777884029783308506011962890625f,-0.09410001248270294382791689713485538959503173828125f,-0.8368000104993582599632873098016716539859771728515625f,0.36340000867694610686697842538706026971340179443359375f,0.89990003304779542236957468048785813152790069580078125f,0.2472000117480754877163917626603506505489349365234375f,-0.3761000044956801513507116396795026957988739013671875f,1.1998999840229747082531730484333820641040802001953125f,0.5282000098779795482784038540557958185672760009765625f,2.05050000888109185126495503936894237995147705078125f,0.139700018304586226491892375634051859378814697265625f,0.66989998618364410276626585982739925384521484375f,0.07710001203194261076845350544317625463008880615234375f,-1.49729998793900076492491280077956616878509521484375f,-0.557500048950314575080255963257513940334320068359375f,-0.440800025668740769191344952560029923915863037109375f,-0.0264000135533513091701252051279880106449127197265625f,-1.480500017523765254878753694356419146060943603515625f,-0.2698000141464174106431528343819081783294677734375f,0.1367000021219251237880598637275397777557373046875f,-0.333500028192997177711731637828052043914794921875f,1.62940003489554019466822865069843828678131103515625f,-0.6645000062435866272636530993622727692127227783203125f,-0.20860002293586799027025335817597806453704833984375f,0.4757999633073808354311040602624416351318359375f,-1.4993000112444168081538009573705494403839111328125f,0.57430000208020270946462915162555873394012451171875f,1.2111000543400649576142313890159130096435546875f,0.8142999798029659785925105097703635692596435546875f,-0.84150001129508000818901791717507876455783843994140625f,-0.2730000121966005277585054500377736985683441162109375f,0.264200017619133031843148273765109479427337646484375f,0.1925999861001965740570085472427308559417724609375f,-0.032000016488134885861427392228506505489349365234375f,0.876700031626224784986334270797669887542724609375f,0.742299980562925565408249894971959292888641357421875f,0.751699994522332826107913206215016543865203857421875f,2.1443999916076652567653582082130014896392822265625f,0.083400025403499444820454300497658550739288330078125f,-0.1667999669879680624262618948705494403839111328125f,-0.82209998288154562118279500282369554042816162109375f,0.43069999520033619422321180536528117954730987548828125f,1.17060002061128631822839452070184051990509033203125f,-0.550899966357647929982022105832584202289581298828125f,-1.2431000276707118157304421401931904256343841552734375f,-0.62030001909732845888356678187847137451171875f,-0.856600019188225036970152359572239220142364501953125f,-1.0065000297725195110842832946218550205230712890625f,-0.472500009164213974344193047727458178997039794921875f,-0.94009997224211705457719290279783308506011962890625f,0.55959998509809361078026768154813908040523529052734375f,0.35900001591443952975168940611183643341064453125f,0.01139999250546086528856903896667063236236572265625f,0.5993999925017359231560476473532617092132568359375f,0.01770001424029465919574022336746565997600555419921875f,-0.788600037837028366993763484060764312744140625f,-0.067699999320506520916751469485461711883544921875f,0.3470000426769264123549874057061970233917236328125f,0.138400009906291643346776254475116729736328125f,-0.0682999849140646375644791987724602222442626953125f,0.5827999752879140515915423748083412647247314453125f,-0.694600009405613949553526254021562635898590087890625f,-0.263300026041268342424928050604648888111114501953125f,0.1018999826669695352165945223532617092132568359375f,0.66210000970363669381413274095393717288970947265625f,0.41970002113580751057497764122672379016876220703125f,-0.31019996355772061491506974562071263790130615234375f,-0.9625000485777857051061801030300557613372802734375f,1.0998000100299714087981328702881000936031341552734375f,-0.02629998049587001229809857250074855983257293701171875f,-0.7479999942034478710439771020901389420032501220703125f,-0.39869998865872624538297941398923285305500030517578125f,-0.59290000359565009357964981973054818809032440185546875f,-1.4147999906957153815056926760007627308368682861328125f,-0.18110000319182850692101283129886724054813385009765625f,-0.8740000059306616275733858856256119906902313232421875f,-0.7993000478267671127241555950604379177093505859375f,-1.28370004450082841884750450844876468181610107421875f,0.67699999972432822747947511743404902517795562744140625f,-0.531999992590397763336795833311043679714202880859375f,-0.1558000125557190695957388015813194215297698974609375f,-0.16509997902214568199497080058790743350982666015625f,0.6847000027328729032660703524015843868255615234375f,0.4269000259920954221115607651881873607635498046875f,0.6973000164933502720288061027531512081623077392578125f,1.2330000305995347797960448588128201663494110107421875f,0.56460002587139623830836399065447039902210235595703125f,0.277500028610230486947330064140260219573974609375f,0.12020002971887644438453435213887132704257965087890625f,1.593200017179548400036992461537010967731475830078125f,-0.5891000099495051767206632575835101306438446044921875f,-0.2597000107236204780036814554478041827678680419921875f,0.19699999175220772595906737478799186646938323974609375f,-0.49079997649490803013350159744732081890106201171875f,0.31389997542500491789496663841418921947479248046875f,0.2442000084921718272568114116438664495944976806640625f,-0.1904000254631041944008984501124359667301177978515625f,0.94080001128912005281534902678686194121837615966796875f,-0.52570001531690346840974825681769289076328277587890625f,-0.7760000433921820928162560448981821537017822265625f,-1.775800001454353349572556908242404460906982421875f,0.830300007213651714010893556405790150165557861328125f,0.379000012785196904729900779784657061100006103515625f,0.1954000184036790432884345136699266731739044189453125f,0.45180000640153838986634582397527992725372314453125f,-0.78169998174458676931664058429305441677570343017578125f,-1.6173000427007675483537241234444081783294677734375f,0.570599985444545154678053222596645355224609375f,0.16610000475645048112482982105575501918792724609375f,-0.04859998612999982814386612517409957945346832275390625f,1.71250008100643835717846741317771375179290771484375f,-1.155300014105438588529750632005743682384490966796875f,-0.4007000517502434799865795866935513913631439208984375f,-0.08210002009719585824853993472061119973659515380859375f,-0.022800004792213002247081021778285503387451171875f,0.7908000072285530368532135980785824358463287353515625f,0.9490999976560470230424471083097159862518310546875f,-1.1388999961376189418160720379091799259185791015625f,-0.0905000023432072087103961166576482355594635009765625f,-1.4389000339865685162976660649292171001434326171875f,-0.79320004370361607559658523314283229410648345947265625f,1.8432000277034934487829787030932493507862091064453125f,0.1148000226214531682700226156157441437244415283203125f,1.257100000781565807272954771178774535655975341796875f,0.3451000197768208099802222932339645922183990478515625f,0.4201999811410903618025258765555918216705322265625f,-0.80670000784024598061705546570010483264923095703125f,-0.52639999673366588694989331997931003570556640625f,0.577899951553344948251833557151257991790771484375f,-1.03670000331401812587728272774256765842437744140625f,0.72999997586011833305974505492486059665679931640625f,-0.53930006272792940791305227321572601795196533203125f,-1.74580001367330428507784745306707918643951416015625f,2.39160005396381070141842428711242973804473876953125f,-1.1146999789282674431234454459627158939838409423828125f,0.081599956417083063797690556384623050689697265625f,-1.1581999545760457426268885683384723961353302001953125f,-1.48130002255439752190113722463138401508331298828125f,0.06720001696348187891771885915659368038177490234375f,-0.1276999915719034817129795555956661701202392578125f,1.1155000463575130975613092232379131019115447998046875f,0.9209000439181931341181552852503955364227294921875f,-1.1741000207901002294619274834985844790935516357421875f,-1.4855000047087665304701431523426435887813568115234375f,0.816299986717104797406818761373870074748992919921875f,-0.251000018320978401931142798275686800479888916015625f,0.22880000985115767075939174901577644050121307373046875f,-0.3351999439813189152204131460166536271572113037109375f,-1.4481000233493741546908495365642011165618896484375f,-1.030299982757121757259710648213513195514678955078125f,-0.38340002050474286665604495283332653343677520751953125f,-0.655200004014372172633784430217929184436798095703125f,-0.28929999943524642436187832572613842785358428955078125f,-1.226299998787045097969894413836300373077392578125f,0.79610002235844723461610783488140441477298736572265625f,1.94919997112751008927489237976260483264923095703125f,-0.087899973458051672281499122618697583675384521484375f,0.207000006653367929487785659148357808589935302734375f,0.392999974291770914902599542983807623386383056640625f,-0.283699998512119133664555192808620631694793701171875f,0.56860002163201583957885532072396017611026763916015625f,-0.6568000511795284435123676303192041814327239990234375f,-0.7669000351563102224616841340321116149425506591796875f,0.4527999730527394461887524812482297420501708984375f,1.241200016337632661134193767793476581573486328125f,-0.7446000296711918764458459918387234210968017578125f,-0.8495999807208785004064566237502731382846832275390625f,1.84820002258121984795025127823464572429656982421875f,0.2388999950200314525972089541028253734111785888671875f,-0.445100002379715586897646062425337731838226318359375f,-0.186599990764259704434380182647146284580230712890625f,-0.58629999739006100067939541986561380326747894287109375f,0.64499999772757210525497839626041240990161895751953125f,-0.1516000554911796083246144917211495339870452880859375f,-0.3209999920390540495418463251553475856781005859375f,1.8964000737726696144136440125294029712677001953125f,0.656200003373622831048805892351083457469940185546875f,0.1255999972164627553894433731329627335071563720703125f,0.57320000775456403374619185342453420162200927734375f,-0.07189999019205550734312737404252402484416961669921875f,-0.63110001045614472214850820819265209138393402099609375f,0.3477999451130628916217801815946586430072784423828125f,0.9698000299230218335111430860706605017185211181640625f,1.0934999858364469194071944002644158899784088134765625f,-0.590000032186509315579314716160297393798828125f,0.46700000009685727064834281918592751026153564453125f,1.21980004746913994750912024755962193012237548828125f,0.86070001043677313301571984993643127381801605224609375f,-1.0361000172175465205981481631170026957988739013671875f,-0.4897000044465062984500036691315472126007080078125f,0.29439999566823227272749363692128099501132965087890625f,-0.235899989901482864951276496867649257183074951171875f,0.040500008732080539086695125661208294332027435302734375f,0.819900007492303739553562991204671561717987060546875f,-1.18380000156238640585115717840380966663360595703125f,-0.859600047850609172428448800928890705108642578125f,-0.1419000099174685392000583306071348488330841064453125f,0.065699948298931332146821659989655017852783203125f,-0.93500000182539222404187739812186919152736663818359375f,-0.62550000366568525578259141184389591217041015625f,0.21269999365806580016169391456060111522674560546875f,0.812000011384486963805784398573450744152069091796875f,-0.59810001748204211224901882815174758434295654296875f,-0.607500038407743137014449530397541821002960205078125f,-1.600599979037046427521318037179298698902130126953125f,-0.579099992217122494508885210962034761905670166015625f,-0.06469991701394384175927143587614409625530242919921875f,-0.458800050270557591147735365666449069976806640625f,0.2200000351667394937749122618697583675384521484375f,0.307200029778481820130764390341937541961669921875f,0.5538999880909916129212433588691055774688720703125f,0.29490000719428000053312644013203680515289306640625f,-1.0408999969422811915364945889450609683990478515625f,1.4586000059112909088554488334921188652515411376953125f,-0.0483000004112723591020994717837311327457427978515625f,0.0605999883502723601935713304555974900722503662109375f,-0.42080000391304483997600982547737658023834228515625f,0.70889998626559957983772619627416133880615234375f,0.5047999924838535434901132248342037200927734375f,-0.053200006189942694589944949257187545299530029296875f,0.2271000104486946558068893864401616156101226806640625f,-0.4838999979630111969441941255354322493076324462890625f,-0.1511999901860949790943777770735323429107666015625f,0.87279998009353876309290853896527551114559173583984375f,-1.0446000436410312062207594863139092922210693359375f,0.064099992477893608100458777698804624378681182861328125f,-0.3120000091306864842266577397822402417659759521484375f,-1.4384999707490209175375639460980892181396484375f,0.26790005438998376607884210898191668093204498291015625f,1.11200004342198344176040336606092751026153564453125f,0.755100023502110584416868732660077512264251708984375f,1.05999998882412871381575314444489777088165283203125f,1.5191000274956227844569411900010891258716583251953125f,-0.11019998888969428918471749057061970233917236328125f,1.6869000108301632767648925437242724001407623291015625f,0.35540000847578046005992291611619293689727783203125f,-0.7917999833047393298812721695867367088794708251953125f,0.6378000107154255804431386422947980463504791259765625f,0.1695000135451552036869315998046658933162689208984375f,1.055900008583813498574954792275093495845794677734375f,0.24920000748634407727877260185778141021728515625f,-0.06999997599050422625310829971567727625370025634765625f,-0.0618000198125840949359144360641948878765106201171875f,0.1439000147268178209092326369500369764864444732666015625f,-0.6438000271737578383834943451802246272563934326171875f,-0.72950001414120213549807658637291751801967620849609375f,0.27360001649111531651925588448648341000080108642578125f,-0.3873000049449502046883253569831140339374542236328125f,0.775499934673309798682794280466623604297637939453125f,0.51340000208318270491503199082217179238796234130859375f,-0.05750005766749399782611362752504646778106689453125f,-0.2083999665439133508471059030853211879730224609375f,0.17060001584291484277855488471686840057373046875f,-0.738600013175606751048007936333306133747100830078125f,-0.73719998142421250264533227891661226749420166015625f,-0.778999993525445244557658952544443309307098388671875f,-1.14280001753270621378533178358338773250579833984375f,-0.221899999244511203499996554455719888210296630859375f,-1.0178999747619033033885216354974545538425445556640625f,-0.693800023299455670411362007143907248973846435546875f,0.4378000169366591531883159404969774186611175537109375f,0.176599989423155800949416516232304275035858154296875f,0.45669997701346842777780921096564270555973052978515625f,0.119500007957219711585139521048404276371002197265625f,-0.4966999980986113172320983721874654293060302734375f,0.3247000247120859217631050341879017651081085205078125f,-0.22899996979534620322738192044198513031005859375f,-0.1680000029951334017397357456502504646778106689453125f,-1.047300026793778204847740198601968586444854736328125f,0.120899979358911335847182044744840823113918304443359375f,0.110400010524690284086091196513734757900238037109375f,-1.2910999908983702422204942195094190537929534912109375f,-0.69889998645186413028795868740417063236236572265625f,-0.20029998767450429841829873112146742641925811767578125f,-0.3081000206112864958640784607268869876861572265625f,-0.35110000935718399528440158974262885749340057373046875f,-0.0636000011801716169657083810307085514068603515625f,-1.30340004982277779532751083024777472019195556640625f,0.672900028443336850614286959171295166015625f,0.6767000066108999067893137180362828075885772705078125f,-1.1803000083684924703675278578884899616241455078125f,-0.2714000048175453372323318035341799259185791015625f,0.7350999975740908354282510117627680301666259765625f,0.8620000214055181242400749397347681224346160888671875f,-1.1027000365734096476444392465054988861083984375f,-0.93199999730288940469336012029089033603668212890625f,-1.873599997305870346053779940120875835418701171875f,-1.15139997199773791436427927692420780658721923828125f,-0.9603999952137467488455513375811278820037841796875f,0.10509999504089317934329983472707681357860565185546875f,1.42179997108578692888158911955542862415313720703125f,0.372299975049495746048933142446912825107574462890625f,0.5463000085473055378315621055662631988525390625f,-0.3557000130444765151338515352108515799045562744140625f,0.3145999962925909443356431438587605953216552734375f,-0.5468000073701144714277688763104379177093505859375f,1.1719000313937666390984304598532617092132568359375f,0.30469999535679814783861729665659368038177490234375f,0.0675999881863589280328596942126750946044921875f,-0.560199983525276667961634302628226578235626220703125f,-0.57949998649954803653372437111102044582366943359375f,-1.552099976772070011321602578391321003437042236328125f,-0.170199931371211476971438969485461711883544921875f,1.0234999900460248767330995178781449794769287109375f,0.024899996167421445392164969234727323055267333984375f,-1.1260000123977658148533009807579219341278076171875f,-0.01060002433657647458176143118180334568023681640625f,1.0373000035107136085343881859444081783294677734375f,0.09809999464601271146335648154490627348423004150390625f,0.213800001674145201402410521041019819676876068115234375f,0.3416000009715556284817239429685287177562713623046875f,0.25430000450164069736302963065099902451038360595703125f,0.0847000092893839262586652694153599441051483154296875f,0.675700024387985376250753688509576022624969482421875f,0.168100010553002388036247793934307992458343505859375f,-0.57830001799464270106909680180251598358154296875f,1.01700003239512515307296780520118772983551025390625f,1.18160003800094148829202822525985538959503173828125f,-0.49750002533197423826294425452942959964275360107421875f,0.568800021585821991010334386373870074748992919921875f,0.074299933758378511328146487358026206493377685546875f,0.2592999963432551790987190543091855943202972412109375f,1.02319997877180579592959475121460855007171630859375f,0.808100008392333979401200849679298698902130126953125f,0.70889999081045385054267171653918921947479248046875f,0.9235000107958912796135564349242486059665679931640625f,0.0573000145748252531774369344930164515972137451171875f,1.315700027573108332745732695912010967731475830078125f,0.117300040912628134037731797434389591217041015625f,-0.3953999924570321677919082503649406135082244873046875f,-0.3870000098943713329191496086423285305500030517578125f,-0.6750999849453569456869672649190761148929595947265625f,-0.142099994839727628015424443219671957194805145263671875f,-0.4562999993085863348341035816702060401439666748046875f,-0.682300001648068477066999548696912825107574462890625f,0.13419999310672270365074609799194149672985076904296875f,-0.1593000042781234082411856434191577136516571044921875f,0.54019996799081593241709242647630162537097930908203125f,0.08989997191578158464153602835722267627716064453125f,0.5519999899268146492659070645458996295928955078125f,1.50780002863407158741892999387346208095550537109375f,0.644700029703975108219538014964200556278228759765625f,0.22370000221878283408472043447545729577541351318359375f,-0.23650000451505182041955777094699442386627197265625f,0.4718000174283982772749368450604379177093505859375f,0.5065000288411976026026195540907792747020721435546875f,-0.64150001502037046208215542719699442386627197265625f,-1.771100060859323033213286180398426949977874755859375f,0.38340003702640590432793032960034906864166259765625f,-0.15710001314953025985943213527207262814044952392578125f,-0.13220000411123067340923853407730348408222198486328125f,0.203100021617114989691543769367854110896587371826171875f,0.6452999931320546611601685071946121752262115478515625f,0.158799961347877494066693770946585573256015777587890625f,0.49310001092553168433596511022187769412994384765625f,0.37679999967738975197306672271224670112133026123046875f,0.4859999814033511000843645888380706310272216796875f,-0.0055999807506796894784883988904766738414764404296875f,-0.5462999653339384309447268606163561344146728515625f,0.3687999993085855354735258515574969351291656494140625f,-1.9170999992728230409966272418387234210968017578125f,0.71319999493956576230857535847462713718414306640625f,-0.3738000085286807827600341624929569661617279052734375f,1.175799988844245635988272624672390520572662353515625f,0.599899998160452252449204024742357432842254638671875f,-0.6323000062674284382779887891956605017185211181640625f,0.7901999689221383960102684795856475830078125f,-0.66519999805390828129247893230058252811431884765625f,-1.58990000674724551998906463268212974071502685546875f,1.16300000619888255215528261032886803150177001953125f,-0.4134000094592575891283559030853211879730224609375f,0.67699999529123289221388404257595539093017578125f,1.72090007994175042682627463364042341709136962890625f,0.076899959942699158688128591165877878665924072265625f,0.84029998283162721239847314791404642164707183837890625f,0.417300008372217534624581958269118331372737884521484375f,0.942599976097792335849590017460286617279052734375f,-0.32539996943473870061325214919634163379669189453125f,0.73060004162192360155358983320184051990509033203125f,0.0177000259935855552839711890555918216705322265625f,-0.6861000090777877868930545446346513926982879638671875f,-1.145300014925002773935602817800827324390411376953125f,0.39350001873075957092851240304298698902130126953125f,1.3107000218734146113064298333483748137950897216796875f,-0.6885000252872706116846757140592671930789947509765625f,0.008000012475997186900400492959306575357913970947265625f,1.2631000084668395633258342058979906141757965087890625f,0.56690001114681354277280433962005190551280975341796875f,-0.423000013276934760941827562419348396360874176025390625f,0.0160000010356304844805208631441928446292877197265625f,-0.484199999234825317895314356064773164689540863037109375f,-0.1252999696068461676912164648456382565200328826904296875f,0.134099970573186677569310631952248513698577880859375f,1.164499981284141671977749865618534386157989501953125f,0.645700031596422530100198855507187545299530029296875f,0.09570002764761464941756230473401956260204315185546875f,-0.51580002157092064152266175369732081890106201171875f,-0.21150003550946738783977707498706877231597900390625f,0.632899966715276374173981821513734757900238037109375f,0.48409999661296632122997607439174316823482513427734375f,0.882300019343197572396775285596959292888641357421875f,-0.034900015259534455935863661579787731170654296875f,-1.9920999725624917164878979747300036251544952392578125f,-0.08909999381899780246385489590466022491455078125f,-0.8522000419229269763121692449203692376613616943359375f,-0.079400034597516100376424219575710594654083251953125f,0.27189997730255111019204150579753331840038299560546875f,0.797600006589293197833967496990226209163665771484375f,-0.05150004221498993839389868298894725739955902099609375f,-1.196400001055001549588041598326526582241058349609375f,-0.4597999995246524473913041219930164515972137451171875f,0.47329998603165124304581468095420859754085540771484375f,-0.5858000363044444380733466459787450730800628662109375f,1.5066999771066011959419483901001513004302978515625f,0.6701000013552611545009085602941922843456268310546875f,0.2281999854028224916646649944595992565155029296875f,-0.4954000101149083068463596646324731409549713134765625f,-0.6041999941311775312868803666788153350353240966796875f,0.36870000873804098517894090036861598491668701171875f,-0.95630002549737724848455400206148624420166015625f,0.59870002408623712586432930038427002727985382080078125f,-0.0335999963372939358663415987393818795680999755859375f,0.08999998915940465860074937154422514140605926513671875f,-0.5078000044569359605617364650242961943149566650390625f,-0.7931000086903576828945006127469241619110107421875f,0.04599995573610049515167474964982829988002777099609375f,-0.42439998564720138318762110429815948009490966796875f,0.1229000073395668246600820339153870008885860443115234375f,-1.9485999984979631705073188641108572483062744140625f,0.160000007301568825113236016477458178997039794921875f,-1.04799999055266379599515857989899814128875732421875f,1.1744999908953903666741780398297123610973358154296875f,-0.0648999802231788880391150087234564125537872314453125f,-0.851500031299888604507941636256873607635498046875f,0.261299986419081253785634544328786432743072509765625f,0.168799974200129820900428967433981597423553466796875f,0.237599969410897671195925795473158359527587890625f,-1.20889999382793877913400137913413345813751220703125f,-0.8047000294804576014939812012016773223876953125f,-0.8679999865293506200458750754478387534618377685546875f,0.289199996459483799071676912717521190643310546875f,-0.2434000225722787735804786279913969337940216064453125f,-0.18710000895857792357901416835375130176544189453125f,0.5298999789565801421531432424671947956085205078125f,1.44010001772791174090571075794287025928497314453125f,-0.2687999798253175587348096087225712835788726806640625f,1.08269999727159671465415158309042453765869140625f,0.43800003063678705217398601234890520572662353515625f,0.277299989503621446829129126854240894317626953125f,-0.20190001276731539547881766338832676410675048828125f,0.03730000455379478552231375942938029766082763671875f,-0.79719999341964697947560125612653791904449462890625f,1.159700019049644037494317672098986804485321044921875f,-0.7155000418871637624107506781001575291156768798828125f,-0.01879997114539133651334168462199158966541290283203125f,0.1077999906733635437916518640122376382350921630859375f,-0.608999989613890679862606702954508364200592041015625f,-0.94180003484413010728104609370348043739795684814453125f,0.123400024956465070857802857062779366970062255859375f,-0.232900055377186332439265470384270884096622467041015625f,0.48740002194643061717016507827793247997760772705078125f,0.82640001929029816007954423184855841100215911865234375f,-0.96460003242790737232326137018389999866485595703125f,-0.76910000853389448582930754128028638660907745361328125f,0.014499982178211467953587998636066913604736328125f,-0.643200012224912853753266972489655017852783203125f,-1.224199976342916773575097977300174534320831298828125f,0.15620001514554093802189527195878326892852783203125f,-1.643299985659123052528229891322553157806396484375f,0.9064999680817127369891750277020037174224853515625f,-0.254600007468461786430680149351246654987335205078125f,0.96909999575614858002836626837961375713348388671875f,-2.10460001029968157837402031873352825641632080078125f,-0.08109997584819694793623057194054126739501953125f,0.95830002559423377306302427314221858978271484375f,-0.1840999903917310831502618384547531604766845703125f,0.03369999324083305936028409632854163646697998046875f,0.9772000199809667275729907487402670085430145263671875f,-0.45959998131692392231428812010562978684902191162109375f,0.4923000108867883195529202566831372678279876708984375f,-0.1475999979838726883230037856264971196651458740234375f,0.143599965752661429263525860733352601528167724609375f,1.24170001806616792094928314327262341976165771484375f,2.172099985712765857215345022268593311309814453125f,0.012800014626979105969439842738211154937744140625f,-0.0869999672770498388985060955747030675411224365234375f,-1.406700020748376633861198570230044424533843994140625f,0.18539999446868904442453640513122081756591796875f,-0.415600012154877074266323688789270818233489990234375f,-1.4177000037908555896137841045856475830078125f,0.439999986551701749437626176586491055786609649658203125f,-0.512600009196996442284444128745235502719879150390625f,-0.34990000302195534143123722969903610646724700927734375f,0.3428000024080279928284653578884899616241455078125f,-0.9613999983914187641431681186077184975147247314453125f,-0.964399955844879031019445392303168773651123046875f,1.323899997720867194317406756454147398471832275390625f,0.072400012856721929210834787227213382720947265625f,0.30800001316517600091771100778714753687381744384765625f,0.393400003573298295833637894247658550739288330078125f,1.92480001319646820689968080841936171054840087890625f,-0.3777999913439151757899026051745750010013580322265625f,0.1590999666988850425042301139910705387592315673828125f,-0.52070002831816697419498041199403814971446990966796875f,-0.5557999970957634428003757420810870826244354248046875f,0.354700055706501427010834959219209849834442138671875f,1.4469000006973740113380699767731130123138427734375f,-0.21039999238252704572005313821136951446533203125f,0.20799997252225921329227276146411895751953125f,-0.82460002683997135619620166835375130176544189453125f,0.76369999547600730949170610983856022357940673828125f,0.4056999902456992490584752886206842958927154541015625f,-0.6578000040099023504325259636971168220043182373046875f,0.59240000201612696972830462982528842985630035400390625f,0.9389000064939254652784939025877974927425384521484375f,-0.962400014235079037661080292309634387493133544921875f,0.2738000195741658870929313707165420055389404296875f,0.61730001450032023768699218635447323322296142578125f,-0.4523000319719319151090530795045197010040283203125f,1.6743000194773076660936794723966158926486968994140625f,-0.651599945092200538709903412382118403911590576171875f,-0.3127000123217700267019836246618069708347320556640625f,1.4595000148117538429914930020458996295928955078125f,-1.272100024046003863276155243511311709880828857421875f,-0.47730002176463581786691747765871696174144744873046875f,0.118299974706768562526804089429788291454315185546875f,0.51739998888447813474300573943764902651309967041015625f,0.3420999876499166703069931827485561370849609375f,-0.557200018956512277412684852606616914272308349609375f,-1.93280006380081204042653553187847137451171875f,0.07369996166899717149334492205525748431682586669921875f,-2.281299987983704369298720848746597766876220703125f,0.047299974900484098583319791941903531551361083984375f,1.8144999741315839258959385915659368038177490234375f,0.39530002615675308152987099674646742641925811767578125f,-0.017899996219575609046614772523753345012664794921875f,0.35479998611360785698565223356126807630062103271484375f,1.458599997305868978259013601928018033504486083984375f,-0.4378000098586076749285211917595006525516510009765625f,-0.273399938082693694241243065334856510162353515625f,1.435300021481513255139361717738211154937744140625f,-0.759200009274483278431944199837744235992431640625f,1.5664999572038649233718388131819665431976318359375f,-1.20239997470974824267386793508194386959075927734375f,-0.62390000180006044416813892894424498081207275390625f,2.648000047326088068899707650416530668735504150390625f,0.36319998093247374271186345140449702739715576171875f,-0.560699973891674918746730327256955206394195556640625f,0.332500014305114888202297152020037174224853515625f,0.3032000540226695850520854946807958185672760009765625f,0.593100033128262094095362044754438102245330810546875f,-0.490499999344348847074570585391484200954437255859375f,-0.401900007700919825737173596280626952648162841796875f,-0.2907999602526423199577720879460684955120086669921875f,0.054900022244453783315520922769792377948760986328125f,0.7476999987617134468820268011768348515033721923828125f,1.1532000010490417452047040569595992565155029296875f,-0.7253999607175585229157377398223616182804107666015625f,-0.1761000202536582293078026850707828998565673828125f,-0.5513000096648930092868567953701131045818328857421875f,-0.7545000471845273448678881322848610579967498779296875f,0.55209998824596340227799373678863048553466796875f,0.54579999901428799535807456777547486126422882080078125f,-1.0414000231087214842773391865193843841552734375f,-0.8481000470049682160578186085331253707408905029296875f,-1.11330003587603609815914751379750669002532958984375f,-0.26500000797212119874046720724436454474925994873046875f,-0.95820005558729093309011659584939479827880859375f,-0.5021000122368339813050397424376569688320159912109375f,0.103699964928626542359779705293476581573486328125f,-0.4360000156573951013427858924842439591884613037109375f,-0.3546000021040445204789648414589464664459228515625f,-0.284899993932246076155934133566915988922119140625f,1.680600029402970818637186312116682529449462890625f,0.68379997010231097220867013675160706043243408203125f,-1.414500014811754358134976428118534386157989501953125f,0.40610002874732042954519783961586654186248779296875f,1.055800045561791744574975382420234382152557373046875f,0.589599999719857326141436715261079370975494384765625f,-0.2328000363826756569096687599085271358489990234375f,-1.46319999061822869634852395392954349517822265625f,0.72719998269081020936255299602635204792022705078125f,0.75499999374151283149103619507513940334320068359375f,0.93920005412697804558774805627763271331787109375f,0.51290001780763250227579419515677727758884429931640625f,-1.2334000354617835615300691642914898693561553955078125f,0.044299985241889938691883799037896096706390380859375f,-0.090900008453429048671523560187779366970062255859375f,-0.9369000131398441677532673566020093858242034912109375f,0.49999998580664428704523061242070980370044708251953125f,-0.031999999687075675325331758358515799045562744140625f,0.3319999892562626175873674583272077143192291259765625f,1.06410002470165476751162714208476245403289794921875f,-0.93179999723732487293403892181231640279293060302734375f,-0.6858000361368061792433081791386939585208892822265625f,0.916900012767314809281060661305673420429229736328125f,0.90380002294555328301584040673333220183849334716796875f,0.970800016355514827637307462282478809356689453125f,-0.02639998926445830118581170609104447066783905029296875f,-0.38339997421801097488014420378021895885467529296875f,0.123099991647154016760623562731780111789703369140625f,-0.21010001113414755735675498726777732372283935546875f,-0.59399999888986343421493074856698513031005859375f,1.339299999956786724197854709927923977375030517578125f,0.18540002255737786374112374687683768570423126220703125f,-0.2663000205799936903616753625101409852504730224609375f,-0.53580002306103724496466611526557244360446929931640625f,-0.8314999725520613171880768277333118021488189697265625f,-0.72120002429112817221579234683304093778133392333984375f,-0.14179999254345876380511981551535427570343017578125f,-0.36550001364946371751329934340901672840118408203125f,-0.892000030681491029582730334368534386157989501953125f,-0.33629998962283114138926976011134684085845947265625f,-0.692000027999281730473057905328460037708282470703125f,-0.8060999631077054861094666193821467459201812744140625f,1.380300021369755381073218813980929553508758544921875f,-0.027200005266070359510877096909098327159881591796875f,-0.7479000379987066349229962725075893104076385498046875f,0.81689999164491933214549135300330817699432373046875f,0.126800018545985604401238333593937568366527557373046875f,0.068300022930652126706974058834020979702472686767578125f,-0.106299996700882803946086596624809317290782928466796875f,-0.042300007124245142531293595311581157147884368896484375f,-0.11219999040216210861586887403973378241062164306640625f,0.3901000137418508817432893920340575277805328369140625f,-1.70230000924766056158432547817938029766082763671875f,-0.4650000335276127660932843355112709105014801025390625f,0.8197999905467037162765109314932487905025482177734375f,1.0707000266045334502251762387459166347980499267578125f,1.068499968923628085093469053390435874462127685546875f,-0.7632000271260748203161483615986071527004241943359375f,0.105900035946070669723440005327574908733367919921875f,0.464999995045363601064991598832421004772186279296875f,-0.17390001064762461435719842484104447066783905029296875f,-0.270700003097951391595188397332094609737396240234375f,0.74240001382529730022952207946218550205230712890625f,0.62720001977607608978360076434910297393798828125f,0.175599976578354688427907603909261524677276611328125f,-0.73450001788511887212962392368353903293609619140625f,-0.22929998591244160621727132820524275302886962890625f,0.3580000238195066852853187810978852212429046630859375f,-0.129699972185493006548995253979228436946868896484375f,0.80069998313039481008246411875006742775440216064453125f,0.6453000167131417441623852937482297420501708984375f,-0.2437000349268318988293913207598961889743804931640625f,-0.207900013691186991149351115382160060107707977294921875f,-0.161399991760402927720718935233890078961849212646484375f,1.193500021710991898515885623055510222911834716796875f,-0.54470000861883161036303135915659368038177490234375f,-0.4117000198543070421663969682413153350353240966796875f,0.4604000167459247894186091798474080860614776611328125f,0.417300067511201522307828781777061522006988525390625f,-0.06160002257824004345820867456495761871337890625f,-1.0717000343084348568112318389466963708400726318359375f,0.6314999943822627859191243260283954441547393798828125f,0.06140004531145226707167239510454237461090087890625f,0.5634000353872783417585878851241432130336761474609375f,0.58059997754693082328003583825193345546722412109375f,0.3089999873042099665099158301018178462982177734375f,0.1608000038385386520900510731735266745090484619140625f,-0.42409999404251597088233438626048155128955841064453125f,1.1181000207602977969401081281830556690692901611328125f,0.4522000327214603654368829666054807603359222412109375f,-0.83730003625601501227038170327432453632354736328125f,0.13849997395276947287356961169280111789703369140625f,0.56410001732557990550276372232474386692047119140625f,-0.0675999935507771709808366722427308559417724609375f,0.537900048299133626272805486223660409450531005859375f,-0.34530001125559184327329376174020580947399139404296875f,-1.44300000202655809289353783242404460906982421875f,-0.0134000221252439377650489404913969337940216064453125f,-1.94660002794265807324336492456495761871337890625f,0.07979995813667717374784160710987634956836700439453125f,-1.034200000035762290195862078689970076084136962890625f,0.49030001365840447657973300010780803859233856201171875f,0.59899998819827970919504878111183643341064453125f,-0.450500017505139294105021008363110013306140899658203125f,-1.566800044250488355146444519050419330596923828125f,-0.0373000278741126845005737777682952582836151123046875f,-0.911499991588294644628831520094536244869232177734375f,1.6305999625712634948371260179555974900722503662109375f,0.016100055718421657502403832040727138519287109375f,1.700599976801871804354959749616682529449462890625f,-0.0809999732449646625553896228666417300701141357421875f,1.2144999563992022739711273970897309482097625732421875f,0.07870002945065557309334280944312922656536102294921875f,-0.9648000251546504646427138141007162630558013916015625f,0.29050001275539383271961924037896096706390380859375f,1.51570002548694571231635563890449702739715576171875f,-0.65409999396800966309228897443972527980804443359375f,0.7195999638080596838562996708787977695465087890625f,0.296600003392994526763004614622332155704498291015625f,0.7841000244036318012064157301210798323154449462890625f,1.0969999988302590043787176909972913563251495361328125f,-0.00399998976290227847840696995262987911701202392578125f,0.3529999807924031340888859631377272307872772216796875f,0.55539998586326788654332631267607212066650390625f,1.1643000326275825262456464770366437733173370361328125f,0.4156000457942490289298120842431671917438507080078125f,-0.38029998486191030071523755395901389420032501220703125f,-0.2764000331670046772813975621829740703105926513671875f,0.1455999762803319441673011169768869876861572265625f,1.156800022867322308428583710337989032268524169921875f,-1.51370001183003211053801351226866245269775390625f,-0.600100034379959179631214283290319144725799560546875f,0.45560001003146144427091712714172899723052978515625f,1.0161000065445897622140591920469887554645538330078125f,0.402800003451108334928676413255743682384490966796875f,1.517900013989209373477251574513502418994903564453125f,1.93090000573396647354229571647010743618011474609375f,1.1432000061154365955218281669658608734607696533203125f,-0.66210002534985523681143604335375130176544189453125f,1.08630003596544355559672112576663494110107421875f,2.02070000164508822848574709496460855007171630859375f,0.22249995723366655653308043838478624820709228515625f,-0.86400000965595236124272560118697583675384521484375f,-1.12560004653930700868613712373189628124237060546875f,-0.69660001725107434378259085860918276011943817138671875f,0.040600017929077136802362701928359456360340118408203125f,-0.687000036697835270160794607363641262054443359375f,0.01360001775771413345950122675276361405849456787109375f,0.79170002026408925477340972065576352179050445556640625f,-0.696000031471252444958963678800500929355621337890625f,-1.0138000064052636872702350956387817859649658203125f,0.785800025445222871667283470742404460906982421875f,0.64889999772086726093078823396353982388973236083984375f,-0.6204999950230121186223186668939888477325439453125f,-0.44260001892000444545516302241594530642032623291015625f,0.6932999811887743391025651362724602222442626953125f,0.03329999240189795983724252437241375446319580078125f,-0.025300010901689429232419570325873792171478271484375f,0.18290000076442936016718476821552030742168426513671875f,-0.2360999849751592716273762562195770442485809326171875f,-1.753799969282745951915103432838805019855499267578125f,-0.58039999074339920781540058669634163379669189453125f,0.164899988716840528724105752189643681049346923828125f,1.870699977803230495965181034989655017852783203125f,0.394699992823600798175220916164107620716094970703125f,1.400700011405348721638119968702085316181182861328125f,0.7627000060260293867742120710317976772785186767578125f,-0.974100007975101878088253215537406504154205322265625f,-0.3985000153779985065938262778217904269695281982421875f,-0.6060999910846351923510155756957828998565673828125f,0.259999992996454099003500459730275906622409820556640625f,0.800300031353533558586832441505976021289825439453125f,-0.3213999682724477668926965634454973042011260986328125f,-0.674900002574921220599435400799848139286041259765625f,-2.11089998859763117167176460498012602329254150390625f,-1.24989997888207415854822102119214832782745361328125f,0.4944000005483626392077667333069257438182830810546875f,-0.2455000021755695538416830459027551114559173583984375f,-0.08939998996853848467480929684825241565704345703125f,-0.1513999914065002894147937695379368960857391357421875f,-0.502300007105618728786566862254403531551361083984375f,0.0560999905258416919906494513270445168018341064453125f,1.2894000478595497583000906161032617092132568359375f,-0.208200046497584168037064955569803714752197265625f,0.0959000039644541946159961298690177500247955322265625f,-0.29020001392364491721309605054557323455810546875f,-1.717400034966319655183042414137162268161773681640625f,0.2896000159621234804063760748249478638172149658203125f,0.855500012867153269979780816356651484966278076171875f,-0.77030002416372322926463311887346208095550537109375f,0.851400038531422875820453555206768214702606201171875f,-1.031299968463182725741944523178972303867340087890625f,0.308099999526142698247355156127014197409152984619140625f,-0.39080002753138476379035637364722788333892822265625f,-1.39920001769363810950608240091241896152496337890625f,-0.314300031733512952314413269050419330596923828125f,-1.23709999815523641331083126715384423732757568359375f,-0.53590001914501073088104021735489368438720703125f,0.217499970421195509828748981817625463008880615234375f,-0.3536000145912172598627876141108572483062744140625f,0.095900026074052124869240287807770073413848876953125f,-0.11370005508065172961096322978846728801727294921875f,-0.5524999970197674858951586429611779749393463134765625f,-0.155799990017712219891876657129614613950252532958984375f,-0.47159999389350393261821636770037002861499786376953125f,-0.280499999791383292091495604836381971836090087890625f,-0.128599981659650808296646573580801486968994140625f,-0.9826000437676913890072682988829910755157470703125f,-0.54399997612834027904682443477213382720947265625f,-1.75980000112652756882880566990934312343597412109375f,-0.27010002067089100563634929130785167217254638671875f,0.3395000249445434548078992520458996295928955078125f,-0.0085999856829646059708238681196235120296478271484375f,-0.5841000404596332629125754465349018573760986328125f,-0.07179999276697657251133932732045650482177734375f,0.408600008183717999799000608618371188640594482421875f,0.468299989421664786259924539990606717765331268310546875f,-0.8350000244379043312648036589962430298328399658203125f,0.0797000152498480163298921752357273362576961517333984375f,0.182699975031614059162166086025536060333251953125f,0.125900033971667768550872779087512753903865814208984375f,-1.2314000338375574727933781105093657970428466796875f,-1.6144000553473836223616899587796069681644439697265625f,1.235700035768747984121773697552271187305450439453125f,0.8035000091195099169993909526965580880641937255859375f,-1.3570000414848326641958919935859739780426025390625f,-0.660499998040497526829994967556558549404144287109375f,-0.6225000080466269736234607989899814128875732421875f,0.052000007063150111008553722058422863483428955078125f,0.0822999887585638134623877704143524169921875f,-0.9653000274419785142754335538484156131744384765625f,-0.98479998756647102453598563442938029766082763671875f,-0.35250000238418532916284675593487918376922607421875f,-2.28749997243285196901751987752504646778106689453125f,-0.295300001290441205270553837181068956851959228515625f,-1.3939999840259549301890729111619293689727783203125f,-0.2021999856710436915818718262016773223876953125f,-1.1026999748826025182779630995355546474456787109375f,0.298800022554397859408936710678972303867340087890625f,0.545300061714649952904210294946096837520599365234375f,-0.068500008076429086401049062260426580905914306640625f,-0.92490000793933813127978282864205539226531982421875f,0.2685000112056734611343244978343136608600616455078125f,0.0177000090062620074604637920856475830078125f,1.19899996003508579889285101671703159809112548828125f,-0.62140006512999601540059302351437509059906005859375f,-0.78510002886652952014401307678781449794769287109375f,0.89580002678632819623771865735761821269989013671875f,1.381500011891126700191989584709517657756805419921875f,-0.157900002068280809197631242568604648113250732421875f,0.427699991571902859988085765508003532886505126953125f,0.087599987534433376712428298560553230345249176025390625f,0.55210001247897755316529355695820413529872894287109375f,0.7061000144422056035153900666045956313610076904296875f,-0.2154999912790952210084327589356689713895320892333984375f,0.19960001372694957666453774436376988887786865234375f,-0.75980002869367579965143022491247393190860748291015625f,0.0912000106006860544738401586073450744152069091796875f,0.225299984787404650177222720230929553508758544921875f,-0.271199997040629359190688774106092751026153564453125f,0.803000015363096952825117114116437733173370361328125f,-0.47480000716149783901443015565746463835239410400390625f,-1.5090000180937350737764290897757746279239654541015625f,-0.43910001097396011981999208728666417300701141357421875f,0.102200015622377460289271766669116914272308349609375f,0.1640999832391738522119339904747903347015380859375f,0.51009998200237749443886059452779591083526611328125f,0.422300010365247668886468090931884944438934326171875f,0.21969998835325255104322650367976166307926177978515625f,-0.08390000209808334830086096189916133880615234375f,0.1378000119447708726738710538484156131744384765625f,0.570699967558681908741391453077085316181182861328125f,-0.57179999837353856495525405989610590040683746337890625f,-0.512499978430569047560538820107467472553253173828125f,1.0235999862603843979513840167783200740814208984375f,-0.28850002884492320642806362229748629033565521240234375f,-0.03689999339580563031404381035827100276947021484375f,0.24980000306367866613754813442938029766082763671875f,0.2949999985098841204234076940338127315044403076171875f,0.3882000118523836196260390352108515799045562744140625f,0.3149000056296584748594113989383913576602935791015625f,1.29130001964867080488374995184130966663360595703125f,-0.36279999421238906531783641185029409825801849365234375f,0.0372999963954090407725061595556326210498809814453125f,0.7835000115409493925966444294317625463008880615234375f,-0.230199990398436415972582835820503532886505126953125f,-0.19140003093183022286893901764415204524993896484375f,0.29260002115517858012339047490968368947505950927734375f,-0.116700005064904799301217508400441147387027740478515625f,-0.01829999940544357883709380985237658023834228515625f,-0.0422999974012374935217195570658077485859394073486328125f,0.114400000399351053914642761810682713985443115234375f,1.009400039421767036884602930513210594654083251953125f,-0.3752000203311445236664667390868999063968658447265625f,-0.52930000807791943717717231265851296484470367431640625f,0.8392000059962274605140919447876513004302978515625f,-0.01279999695047695507099660972016863524913787841796875f,0.43520000319480889316992033855058252811431884765625f,1.4056000143714246775061837979592382907867431640625f,-0.6332000093005596408346491443808190524578094482421875f,-0.051500000435859061287402482776087708771228790283203125f,-0.37909998713210224519087887529167346656322479248046875f,0.7350000070780515759594209157512523233890533447265625f,0.1777999907106160293324137455783784389495849609375f,-0.4798000050753354894794711071881465613842010498046875f,-1.138499981179833309141713471035473048686981201171875f,-0.510799972695112014520191223709844052791595458984375f,0.085499995097517711428736220113933086395263671875f,0.1045999806463715486870569293387234210968017578125f,2.440399952074886247288532103993929922580718994140625f,-0.6456000251561400116173672358854673802852630615234375f,-0.54030000760480756394343870852026157081127166748046875f,0.258699996537715193323236917422036640346050262451171875f,0.3083000034473837303039545076899230480194091796875f,-0.10460001491904247217235024436376988887786865234375f,-0.015300017532706267076036965590901672840118408203125f,1.1827000028222800498411970693268813192844390869140625f,0.1055999776214360252879487234167754650115966796875f,-1.50460000731945076068996058893389999866485595703125f,-0.708700009706616729232564466656185686588287353515625f,-1.2652999409407377218172996435896493494510650634765625f,-0.75919996255934218964966930798254907131195068359375f,-0.64280000851750340729040544829331338405609130859375f,-0.84900001099705679763474108767695724964141845703125f,0.28960005451887915750575075435335747897624969482421875f,-0.4090000473558905724758005817420780658721923828125f,0.0723000338159508204594061453462927602231502532958984375f,-0.5897000127911571354388797772116959095001220703125f,-1.3860000396855178816934994756593368947505950927734375f,0.94120003328770429629201998977805487811565399169921875f,1.4113000657491394651543714644503779709339141845703125f,-0.894800054472685513218266351032070815563201904296875f,-0.311800016571581461999329576428863219916820526123046875f,0.19110001131296172616202966310083866119384765625f,-0.756600013861060460840235464274883270263671875f,-0.592499988004565825150393720832653343677520751953125f,-1.52369997774362531117731123231351375579833984375f,-0.824399992799758507544538588263094425201416015625f,-0.872900006501376335421582552953623235225677490234375f,-1.458100011968612275126133681624196469783782958984375f,0.2859000079318880782608403023914434015750885009765625f,-0.2826999967500569699296875114669091999530792236328125f,-1.001600004920363051752474348177202045917510986328125f,-0.2239000161051745152462899568490684032440185546875f,-0.9280000052303076785165103501640260219573974609375f,-0.6478000140309330134869014727883040904998779296875f,0.8724999952316283557962606209912337362766265869140625f,-1.33700000192224965900322786183096468448638916015625f,-0.7219999613165857255125956726260483264923095703125f,-1.934300021004677461178289377130568027496337890625f,0.1517999952256678941608925015316344797611236572265625f,-1.554500021331012504788304795511066913604736328125f,-1.0086000397741796685835424796096049249172210693359375f,-0.0064999950900672676112890258082188665866851806640625f,0.415000047832727592123092108522541821002960205078125f,-0.858300035279989881331630385830067098140716552734375f,-0.630900010353327100887099732062779366970062255859375f,-1.031699996161460575194723787717521190643310546875f,0.30129996302425876564257123391143977642059326171875f,0.06070000272542219743598934655892662703990936279296875f,-0.86220002521499961023465630205464549362659454345703125f,-0.092499985396862050013311318252817727625370025634765625f,0.196599991043657329470306649454869329929351806640625f,-0.86930001165419812370060981265851296484470367431640625f,-1.057400024740398070122182616614736616611480712890625f,0.05430001444816578981544807902537286281585693359375f,-0.412800040145218982701180721051059663295745849609375f,0.2085000316202647585583918043994344770908355712890625f,0.14439998218268090823812599410302937030792236328125f,0.5241999986201528738405386320664547383785247802734375f,1.429200037176907134295333889895118772983551025390625f,0.3118999982386820857271914064767770469188690185546875f,1.1631000180222088591364126841654069721698760986328125f,1.7475999199576686127244329327368177473545074462890625f,-1.73360000833273009135382380918599665164947509765625f,-0.5330000270605079748520438442938029766082763671875f,-1.73860004081726149394171443418599665164947509765625f,0.62990002909898823446610549581237137317657470703125f,0.679899982011317405294903437606990337371826171875f,0.207999990105630416792337200604379177093505859375f,-1.0036000065445893625337703269906342029571533203125f,-0.0331999655842789564985650940798223018646240234375f,0.69769997294545138544208384701050817966461181640625f,-0.3656999627530570506905860383994877338409423828125f,-1.568200049068032786436788228456862270832061767578125f,-0.104000081628562224267398050869815051555633544921875f,0.7036000008448954901041361154057085514068603515625f,-0.98290000624060613887422732659615576267242431640625f,0.2042000394865868262428421076037921011447906494140625f,0.811300041627884382933189044706523418426513671875f,0.230899967475235445846237780642695724964141845703125f,-0.4115000450462107384197452120133675634860992431640625f,1.1082000274613499346543221690808422863483428955078125f,-0.43059996979832615426886377463233657181262969970703125f,-0.03360001868903683686085059889592230319976806640625f,1.3668000403203073300772985021467320621013641357421875f,0.8380999861896043512388132512569427490234375f,-0.168999985944479735877621351392008364200592041015625f,-1.7897000124931334852362851961515843868255615234375f,-0.31300003099069029222079052487970329821109771728515625f,0.979300030541420341023695073090493679046630859375f,0.93120004299208580800240042663062922656536102294921875f,-0.29939997886717240049847532645799219608306884765625f,-0.2295000095777213200420874272822402417659759521484375f,0.89910007115602486038596907746978104114532470703125f,1.0490000371485954655526029455359093844890594482421875f,0.292800006878376262164920262875966727733612060546875f,-0.034100033754110636863288164022378623485565185546875f,-0.22450001361966176460782662616111338138580322265625f,0.52730002630948913378006182028912007808685302734375f,-1.97350003549456598506139926030300557613372802734375f,0.143799980384111680820069523178972303867340087890625f,2.3846000666260724898393164039589464664459228515625f,0.297999957919120816995928180404007434844970703125f,-2.05640006863176960649752800236456096172332763671875f,-0.73929998017549447553165009594522416591644287109375f,0.3839000232949854041208936905604787170886993408203125f,0.279899995943904134065860489499755203723907470703125f,-1.475500051945447754775386783876456320285797119140625f,-2.000900069361926103539417454157955944538116455078125f,0.852500027008354255286803891067393124103546142578125f,0.789300043803454087765203439630568027496337890625f,-0.371899968287348059448049752973020076751708984375f,-0.58500002756714852836239515454508364200592041015625f,-1.0773000050008298966730535539682023227214813232421875f,0.34740000406503668273217044770717620849609375f,-0.368900021506845643415317681501619517803192138671875f,-0.38850002908706660065263349679298698902130126953125f,1.470999995931983495012218554620631039142608642578125f,1.0219000104576345666629322295193560421466827392578125f,0.1363999682165684124157678525079973042011260986328125f,0.137399967724829874526903950027190148830413818359375f,-0.29390002415180216388534972793422639369964599609375f,-0.13430000720247636802895385699230246245861053466796875f,-0.8313000064611433259642581106163561344146728515625f,0.52910001214742674591207105549983680248260498046875f,0.129899998998642018932514474727213382720947265625f,0.48890000859871485250351952345226891338825225830078125f,-0.97080002112388541490872739814221858978271484375f,-1.14530001328587527353874975233338773250579833984375f,0.09540000065267140172409199294634163379669189453125f,0.30880002555325669622021678151213563978672027587890625f,-0.5790999541260306582302064271061681210994720458984375f,0.8123000242233278100911775254644453525543212890625f,0.456400036519766683085208569536916911602020263671875f,1.271399990326166129506191282416693866252899169921875f,0.6321000283300879374337455374188721179962158203125f,0.832899980014561780450321748503483831882476806640625f,1.15920003371238777845064760185778141021728515625f,-0.817500013411045500788532081060111522674560546875f,-2.0291000416517253057691050344146788120269775390625f,-0.366899954950810069931321777403354644775390625f,0.9467000361710784517299543949775397777557373046875f,0.70929998241066893882589283748529851436614990234375f,-0.526000001836568031876595341600477695465087890625f,-0.14430000878572446509906512801535427570343017578125f,1.292699985499680348510764815728180110454559326171875f,0.05310000809431070223354254267178475856781005859375f,0.450900001040100306681779329665005207061767578125f,0.277700036871433297847033827565610408782958984375f,-0.447000006951391082310465208138339221477508544921875f,-0.603700053441524886466140742413699626922607421875f,1.71450000248104306166396781918592751026153564453125f,-0.270099966132641089444632598315365612506866455078125f,-0.6253000078842043674143269527121447026729583740234375f,0.120300017607212428316643126890994608402252197265625f,0.7592000252559778505911936008487828075885772705078125f,-0.350500008825212649998803726703044958412647247314453125f,-1.1045999915242195044129402958787977695465087890625f,0.016699993441999183563240194416721351444721221923828125f,0.717100031757354372530244290828704833984375f,0.77810000833645454054732226722990162670612335205078125f,-0.82120002930164304189020185731351375579833984375f,-0.3240000138655305672585882348357699811458587646484375f,-1.12829997344017129989879322238266468048095703125f,1.2800999828591936857691280238213948905467987060546875f,1.0859999895989886908864718861877918243408203125f,0.0410999995596703637801283548469655215740203857421875f,-0.8099000004887582093715536757372319698333740234375f,1.01360000833272945186536162509582936763763427734375f,-0.9209999802112580624680049368180334568023681640625f,0.0938999907732007699223686358891427516937255859375f,1.0862000456929212788281802204437553882598876953125f,1.2111999753594400175416012643836438655853271484375f,-1.21060000749826457422386738471686840057373046875f,1.650699983465671039084554649889469146728515625f,-0.00669997008442901886837717029266059398651123046875f,-0.20480002571344346762316490639932453632354736328125f,0.190700049179793662545989718637429177761077880859375f,1.71930000297427110211856415844522416591644287109375f,-0.657899988061188878418761305510997772216796875f,0.92640001729726773049833354889415204524993896484375f,0.93079999664872892228828504812554456293582916259765625f,-0.70670003073215514888261168380267918109893798828125f,1.1035000350102774913096936870715580880641937255859375f,0.90150008088350386259435254032723605632781982421875f,-0.36939996076226189192226456725620664656162261962890625f,-1.86930003083944296093932280200533568859100341796875f,-0.0997000227376811931634392749401740729808807373046875f,1.2792999766737216393863718622014857828617095947265625f,0.34709999558478543679740369043429382145404815673828125f,0.2262000298976898360336917903623543679714202880859375f,0.040199967096746558326003651018254458904266357421875f,0.1772000104069710868515130641753785312175750732421875f,-0.6820000143647195756102519226260483264923095703125f,-0.4858000320017341611134042977937497198581695556640625f,0.97300002676248453781226999126374721527099609375f,0.6303000012904405036096022740821354091167449951171875f,-1.2441999893069262128619811846874654293060302734375f,-0.17999998947605455601461699188803322613239288330078125f,0.22379994141757553194338470348156988620758056640625f,-0.94049999314174048681280737582710571587085723876953125f,-0.121699992957711611296645060065202414989471435546875f,0.151699992547928985686667147092521190643310546875f,1.5568000037193296947179987910203635692596435546875f,-0.233200005221366524210679926909506320953369140625f,-1.850100077891351446623957599513232707977294921875f,0.1819000136613855289624552824534475803375244140625f,1.36590006027221733120313729159533977508544921875f,0.9928999929785735645282329642213881015777587890625f,-1.61360004260539913190086735994555056095123291015625f,-0.23529996187687007846989217796362936496734619140625f,0.90659998241960959575180822866968810558319091796875f,-0.48369997491240557252467624493874609470367431640625f,0.4333999909818173623676784700364805757999420166015625f,-1.4296999842554327653232348893652670085430145263671875f,0.344400017386675472153001464903354644775390625f,-1.72920003490448070948559688986279070377349853515625f,-0.71720003275871369030625146478996612131595611572265625f,0.01929998285770384569559610099531710147857666015625f,-0.9129000028878448969038572613499127328395843505859375f,1.10760003408193608009923991630785167217254638671875f,-0.71499996442347768521585749113000929355621337890625f,-1.07330002470016427906784883816726505756378173828125f,0.86739996759444526475135717191733419895172119140625f,1.52130001450777019300630854559130966663360595703125f,0.10160003628730807090363441602676175534725189208984375f,0.3906999828696253729276577359996736049652099609375f,0.254699961158632692814762776833958923816680908203125f,1.0254000212162737160070946629275567829608917236328125f,0.5745999754309654150574715458787977695465087890625f,-0.8522000279158359337117190079879947006702423095703125f,1.28979999293088809508844860829412937164306640625f,-0.1091000008970499646210328137385658919811248779296875f,-1.5359000337481489850688376463949680328369140625f,-0.53560000367984184066472153062932193279266357421875f,-0.0498000179648396823495204444043338298797607421875f,-0.9732000050909814792277074957382865250110626220703125f,-0.1047000300765024149995952029712498188018798828125f,-0.3113999736554913244646058956277556717395782470703125f,-1.409899991846085498536922386847436428070068359375f,-0.716400035476684848845252417959272861480712890625f,-0.42140008226633174359676559106446802616119384765625f,-1.00409998890161400453280293731950223445892333984375f,1.8764000649809844389892532490193843841552734375f,0.4036000271081920942606302560307085514068603515625f,-1.3298000513434402591883554123342037200927734375f,-0.96139999822378197080752215697430074214935302734375f,0.15349999988079066071122724679298698902130126953125f,-0.52249992877244988420670779305510222911834716796875f,0.62570001326799395968691896996460855007171630859375f,1.6427999478697774549118548748083412647247314453125f,-0.12469998179674224303425944526679813861846923828125f,-0.4065999792531134193751540806260891258716583251953125f,-0.4956999751582740199040699735633097589015960693359375f,-1.0517000198543069444667708012275397777557373046875f,-0.951599960291386270938573943567462265491485595703125f,-0.142999997705220760479960517841391265392303466796875f,0.045499997593462904177386008086614310741424560546875f,0.7892999961197375569099676795303821563720703125f,-0.29089995343089014578907836039434187114238739013671875f,-1.091000008560716327821182858315296471118927001953125f,-1.371000032514334332489625012385658919811248779296875f,-0.65780004912316869525312768018920905888080596923828125f,1.307599986062944363851556772715412080287933349609375f,1.1569000091165302368523271070444025099277496337890625f,0.9540999935209750759668168029747903347015380859375f,0.17030001922771320632676861350773833692073822021484375f,0.4086999918997289427124997018836438655853271484375f,0.1516000024616716290726259330767788924276828765869140625f,0.7009999966025350914833325077779591083526611328125f,0.1475000010058280519587015078286640346050262451171875f,0.775600044341385963519996948889456689357757568359375f,0.97530006314888684215702596702612936496734619140625f,-0.0265000095441936256435155883082188665866851806640625f,-0.797400034090876896897270853514783084392547607421875f,-0.8586999950289726513119603623636066913604736328125f,-0.452799980317056149914378693210892379283905029296875f,-0.36380003060102428236177729559130966663360595703125f,0.70919999914169284949139182572253048419952392578125f,0.781099979126453458633250193088315427303314208984375f,-0.338500007927417190245478195720352232456207275390625f,-0.0238000083982949917071891832165420055389404296875f,-0.789099945539236014013795283972285687923431396484375f,0.064600033806265333424789787386544048786163330078125f,0.201799973842500701692870279657654464244842529296875f,-0.73500007063150507580218118164339102804660797119140625f,0.16970000761300407798870537590119056403636932373046875f,-0.5416999738097185712604186846874654293060302734375f,1.171600007975101664925432487507350742816925048828125f,-0.6694000169768925534441450508893467485904693603515625f,0.32650000429153447356611650320701301097869873046875f,-0.05340003937333881456339668147847987711429595947265625f,1.71960004129409771422842823085375130176544189453125f,0.396199990633129939254786222591064870357513427734375f,-0.9987999996066090346857890835963189601898193359375f,0.9804000065386293538693962545949034392833709716796875f,-0.3938999828010796289134987091529183089733123779296875f,1.2956999919593334968936915174708701670169830322265625f,-0.55309998823851369564863489358685910701751708984375f,0.5556000050768254983069027730380184948444366455078125f,-0.22409997901096989636471334961242973804473876953125f,-0.52859998648390182918177515603019855916500091552734375f,-1.834199990070610741810241961502470076084136962890625f,-0.922299992483854413904964530956931412220001220703125f,0.00250005766749428204320793156512081623077392578125f,0.084700056600570672316052878159098327159881591796875f,0.622500066161155718447162144002504646778106689453125f,-1.58579999787807413014206758816726505756378173828125f,-0.37259995227455977584440915961749851703643798828125f,1.4399000036180016337539200321771204471588134765625f,0.7966999879658214211985978181473910808563232421875f,0.65650001472234720267806551419198513031005859375f,-0.078099974119663517768685778719373047351837158203125f,1.1822000661373148755473039273056201636791229248046875f,0.2997999693870543325147082214243710041046142578125f,-0.7219000314123931172360926211695186793804168701171875f,0.363500031620264341114534545340575277805328369140625f,0.41490000646784874760442107799462974071502685546875f,-0.7247000037014483808661680086515843868255615234375f,-0.652500022593886175314992215135134756565093994140625f,1.5108000315546996628057740963413380086421966552734375f,0.1137999882072206758731880427149008028209209442138671875f,-0.349999996349215580782043844010331667959690093994140625f,0.64870001158788814787925502969301305711269378662109375f,0.8301000159025182512095852871425449848175048828125f,-0.4760999821811913168545515873120166361331939697265625f,-0.02949999052286056411276149447076022624969482421875f,-0.4204000140637160765066937528899870812892913818359375f,0.004399999356270001271695946343243122100830078125f,0.526500032827258213075083403964526951313018798828125f,0.153600014591216194048683973960578441619873046875f,-0.101900004497171181583325960673391819000244140625f,1.070100026631355394357569821295328438282012939453125f,0.93719999911934148695280555330100469291210174560546875f,2.557699997383355405844440610962919890880584716796875f,-0.8214000362589957138226282040704973042011260986328125f,-0.8740000501871112437157762542483396828174591064453125f,0.10500003889203046725242529646493494510650634765625f,-0.423500044882298087856042911880649626255035400390625f,-0.207399998849630673447563822264783084392547607421875f,0.0384000447750094675569698665640316903591156005859375f,-0.1200000366568563237024136469699442386627197265625f,-0.90530003712773332580354690435342490673065185546875f,-0.96719997643232336059782028314657509326934814453125f,0.7560000607520349635848333491594530642032623291015625f,-0.5956999976962802367808080816757865250110626220703125f,-0.30119999327808633093894741250551305711269378662109375f,-0.167599987590312249352564322180114686489105224609375f,0.20469997911453230443612483213655650615692138671875f,0.2688999893948433861368130237678997218608856201171875f,-0.537100006872415658421004991396330296993255615234375f,0.45500004921108538891161288120201788842678070068359375f,-0.718100029700994735293306803214363753795623779296875f,-0.04259996464252413517215245519764721393585205078125f,0.523900003588199325577079434879124164581298828125f,-0.42870000354126081010264215365168638527393341064453125f,0.691400015211105500867461159941740334033966064453125f,-0.180100012866407421796566268312744796276092529296875f,-0.374700009363889474656161837629042565822601318359375f,-0.5332999820083370590140248168609105050563812255859375f,0.7843000506579880237723045866005122661590576171875f,-1.141500009134411630640215662424452602863311767578125f,-0.0457000062644485893770252005197107791900634765625f,0.0029000186085703649752076671575196087360382080078125f,-0.8152999621748922010056048748083412647247314453125f,0.538500004500150897257526594330556690692901611328125f,-1.308899999713897699393783113919198513031005859375f,-0.74750001542270183563232421875f,-0.82509995808601388489478267729282379150390625f,-0.50449996876716607374646628159098327159881591796875f,0.65009999362528336774857962154783308506011962890625f,1.2634999983161694370181749036419205367565155029296875f,-0.62709998586177828627796770888380706310272216796875f,-0.719400015896558553407658109790645539760589599609375f,-1.131700017470121277796124559245072305202484130859375f,0.52879998470544808952809034963138401508331298828125f,-1.2574000587522988769961784782935865223407745361328125f,0.21989996055364624538697171374224126338958740234375f,-0.18860004990696932480886971461586654186248779296875f,0.21149999207258218802962801419198513031005859375f,0.93800000798702232174264281638897955417633056640625f,-1.0700999786496165899762900153291411697864532470703125f,-0.841500001385808271692212656489573419094085693359375f,0.695800019522010870787198655307292938232421875f,-0.575800025892257760773418340249918401241302490234375f,-0.8301000201679766821172279378515668213367462158203125f,0.51040000206828128881397788063623011112213134765625f,0.79890002405867022172714086991618387401103973388671875f,-0.6132000052213673058076892630197107791900634765625f,0.911900029270351186738707838230766355991363525390625f,-0.1530000200569627821778340148739516735076904296875f,-1.103300008569657375545602917554788291454315185546875f,0.5977000048339362603400104489992372691631317138671875f,1.613100033258647147960118672926910221576690673828125f,-0.49320002906322546465389677905477583408355712890625f,0.157500017229468103341361029379186220467090606689453125f,0.07520001034736623068965855054557323455810546875f,-0.4721999928981064353905594543903134763240814208984375f,0.425099985504150623683017329312860965728759765625f,-0.14880000646114321938284774660132825374603271484375f,1.208799968910216904305343632586300373077392578125f,-1.34459998519122603255482317763380706310272216796875f,-0.05339998695850312060429132543504238128662109375f,0.584699976804852639133969205431640148162841796875f,1.1476999423980718262328082346357405185699462890625f,-0.922000016823410373234537473763339221477508544921875f,1.014699941116571579868832486681640148162841796875f,-0.3941000356167554397046615122235380113124847412109375f,-0.450700006562471333637631687452085316181182861328125f,-1.032899975097179900984656342188827693462371826171875f,-0.257899970328808070263448826153762638568878173828125f,-0.64789999998211822429539097356610000133514404296875f,-1.38850002506375336253086061333306133747100830078125f,-0.2300999682188038519825568073429167270660400390625f,-1.3077000100493432199755261535756289958953857421875f,-1.37769998903870583006892047706060111522674560546875f,-0.8868999866902829154469145578332245349884033203125f,-0.3701999786078931098387556630768813192844390869140625f,-1.283700044053793565268506426946260035037994384765625f,-1.46130001957416499891451167059130966663360595703125f,0.211700014340877817886621414800174534320831298828125f,-0.44730004385560828694679003092460334300994873046875f,0.11980000723600436884908049250952899456024169921875f,0.445299988568573557046192945563234388828277587890625f,-0.60160001721382183603736848453991115093231201171875f,-0.25109998477026795882238729973323643207550048828125f,-0.2594999888837339341307597351260483264923095703125f,-0.296500009153037868969704504706896841526031494140625f,-0.347800012689828630385591168305836617946624755859375f,-1.03640001712962970259468420408666133880615234375f,0.377600018733739606346944128745235502719879150390625f,-0.02680000768676349398589309203089214861392974853515625f,0.403699998903274348549530259333550930023193359375f,-0.3097000148028141719436234780005179345607757568359375f,1.054199996161461427846006699837744235992431640625f,-0.225700020942091850173483180697076022624969482421875f,1.4084999614357940345144015736877918243408203125f,0.6065999756395819009213710160111077129840850830078125f,-0.2020999241709713345471755019389092922210693359375f,-0.467499975487589924938447438762523233890533447265625f,-1.74429998457133716982525584171526134014129638671875f,0.922299977284669836308239609934389591217041015625f,-0.470000014081597417003877126262523233890533447265625f,0.3167999891906980725053699643467552959918975830078125f,-0.00909998323917360352197647443972527980804443359375f,-0.262699952828884253364094547578133642673492431640625f,0.554400075352192711619636611430905759334564208984375f,-0.9961000062465661386568172019906342029571533203125f,-1.0608000074148176139487986802123486995697021484375f,1.2408999562621119139294023625552654266357421875f,0.63619997014403306678786975680850446224212646484375f,0.5362000088870522152006969918147660791873931884765625f,0.819399991235136848644060592050664126873016357421875f,0.1914999760538342865601180164958350360393524169921875f,1.014100008347630588190213529742322862148284912109375f,-0.5749999822676183924841097905300557613372802734375f,0.36500002905726436353006647550500929355621337890625f,-0.16110003152266172410378430868149735033512115478515625f,0.10070003114938774757547435001470148563385009765625f,0.94480004789754790106570681018638424575328826904296875f,-0.65700005519390192176842901972122490406036376953125f,-0.7357000205881896182091850278084166347980499267578125f,0.37150004109740297764119532075710594654083251953125f,-0.8267000032767650719023322380962781608104705810546875f,0.24379998098015764895762913511134684085845947265625f,0.8868999880313868100500940272468142211437225341796875f,-1.163800000649689803111641595023684203624725341796875f,0.133100077738613709055215394982951693236827850341796875f,0.886400011336803572703502140939235687255859375f,0.3106999876379961111894090208807028830051422119140625f,-0.5825000128149984135461636469699442386627197265625f,-0.278000007838010265714956403826363384723663330078125f,-0.8885000261068345395187861868180334568023681640625f,0.315699992033839027527619691682048141956329345703125f,-1.63779993535280254945973865687847137451171875f,-0.8118000125855207027569804267841391265392303466796875f,-0.8969999788999558631985564716160297393798828125f,-1.1072999442040922790653212359757162630558013916015625f,-0.4726999492526058332941829576157033443450927734375f,-1.37390002623796458891547445091418921947479248046875f,0.1225000104308122672591707669198513031005859375f,0.9368000098288060684126321575604379177093505859375f,1.6341000556588181069628262775950133800506591796875f,0.42040000191926996109259562217630445957183837890625f,-0.39820005826950133354102945304475724697113037109375f,1.11339993569850914667540564551018178462982177734375f,0.950699994045496321604105105507187545299530029296875f,-0.4901999911248680064090876840054988861083984375f,0.85249999657273267672508154646493494510650634765625f,0.15749999091029120990015144343487918376922607421875f,-1.170400008624791876599147144588641822338104248046875f,-0.504499969214200039147044662968255579471588134765625f,0.939800008092820160499059056746773421764373779296875f,1.034299990233778832049438278772868216037750244140625f,-0.7418999935448169491536418718169443309307098388671875f,0.33260000284537627468495202265330590307712554931640625f,-0.264899988269805763962949640699662268161773681640625f,-0.3514999992810192708247996051795780658721923828125f,1.226599985623359589936853808467276394367218017578125f,0.61559999704882473192668612682609818875789642333984375f,-0.1965999927014110415512959662009961903095245361328125f,0.6560000292174519298527002320042811334133148193359375f,-0.34889999710619434836189611814916133880615234375f,-1.34790002633854744118480084580369293689727783203125f,2.365699965360760881338819672237150371074676513671875f,0.441699992659688067675460843020118772983551025390625f,1.50529998452663438257559391786344349384307861328125f,0.5554999942779537125403521713451482355594635009765625f,-0.71899996569752744335346505977213382720947265625f,0.0259999758154150217848155079991556704044342041015625f,0.5681000000476839062457656837068498134613037109375f,1.0523000291407111905783722249907441437244415283203125f,1.746100017683207550334145707893185317516326904296875f,-0.26240000581592359818472459664917550981044769287109375f,-0.9685999716386197722073347904370166361331939697265625f,-0.3609000116199252428117461022338829934597015380859375f,-0.2345999918594958710826858805376105010509490966796875f,0.6666000052928922059436445124447345733642578125f,0.010499992340803121493308935896493494510650634765625f,0.8395999518871313238577158699627034366130828857421875f,0.1563999807894236315775060575106181204319000244140625f,-1.104899997359513719175083679147064685821533203125f,0.3178999895513061613172567376750521361827850341796875f,0.15369998102188053845651438678032718598842620849609375f,-0.3347000278785829951999630793579854071140289306640625f,1.25700001537054806277637908351607620716094970703125f,0.509100021572411520764944725669920444488525390625f,-0.846100002819299756851023630588315427303314208984375f,0.190399987316131724668366587138734757900238037109375f,0.151500003416091322261394225279218517243862152099609375f,-0.76050002795457860571559649542905390262603759765625f,0.01580000385716584521134109309059567749500274658203125f,0.448699993240833094887420884333550930023193359375f,-0.2460000177435579526985520715243183076381683349609375f,0.94150002127885823455244462820701301097869873046875f,0.2628999929971993498867277594399638473987579345703125f,-0.4891000020146372140317225785111077129840850830078125f,-0.84730001394152576477125649034860543906688690185546875f,0.190700012001394803018428092400426976382732391357421875f,-0.15860004700124263887772713133017532527446746826171875f,-0.83000001370906861808407484204508364200592041015625f,0.44839997533559738940311945043504238128662109375f,-1.2569999726414682328368144226260483264923095703125f,-0.26080002127289791946651575926807709038257598876953125f,-0.22639998544603556762666585200349800288677215576171875f,-0.501200022670626577081520736101083457469940185546875f,-0.435100009830296230450841221681912429630756378173828125f,0.7875000335648658644771558101638220250606536865234375f,0.4955000215470788393901102608651854097843170166015625f,0.43320003360807934900122972976532764732837677001953125f,0.1174999800696963081492185665410943329334259033203125f,-1.0785000348053876972898024177993647754192352294921875f,0.304399992538989028201257269756752066314220428466796875f,0.1171999839574098256633760684053413569927215576171875f,-0.4078000364571814362335544501547701656818389892578125f,0.71180001616179910950421572124469093978404998779296875f,-0.133399985058605341503579211348551325500011444091796875f,-0.49230002046078435995468680630438029766082763671875f,0.84850001636520000491969994982355274260044097900390625f,-1.1623999478504061233508082295884378254413604736328125f,-1.293300014716386225899213968659751117229461669921875f,-0.588600002335011307508239042363129556179046630859375f,-0.29450003090500853630828714813105762004852294921875f,-1.159000018931926501153384378994815051555633544921875f,0.14690004631355468855957724372274242341518402099609375f,0.46290002270638996861862324294634163379669189453125f,-0.1631999970257280718755055204383097589015960693359375f,0.57200002181529985278984895558096468448638916015625f,-0.00619999946206795105041464921669103205204010009765625f,1.7789000056743609068377054427401162683963775634765625f,0.219699993009864924520258000484318472445011138916015625f,-0.5561000153362751774466232745908200740814208984375f,0.7442000070579346537869014355237595736980438232421875f,-0.2875000035762784733606167719699442386627197265625f,0.3416000101171434710778385124285705387592315673828125f,-0.0547000303745281968303970643319189548492431640625f,-1.1256999870233241889394548707059584558010101318359375f,-0.957400069406630027657456594170071184635162353515625f,-0.66009997850060386781478882767260074615478515625f,-0.31240003705248320375886805777554400265216827392578125f,-0.43859998896121876743592338243615813553333282470703125f,1.3488000463217495461520911703701131045818328857421875f,1.263200026623159555327902126009576022624969482421875f,-0.097099983030556558105672593228518962860107421875f,0.6117999554395670003259510849602520465850830078125f,-0.01170000584721631042839362635277211666107177734375f,-1.429200023132562780148191450280137360095977783203125f,-0.59610005959272438502694058115594089031219482421875f,-0.719899989014864072345289969234727323055267333984375f,-0.8100999746263024547943132347427308559417724609375f,-0.506199991452693343063629072275944054126739501953125f,-0.58780004744678659989176594535820186138153076171875f,0.04809996994733722885939641855657100677490234375f,-2.1232999880433087724895813153125345706939697265625f,0.45950002747774210121178839472122490406036376953125f,-0.7061000134363764058065271456143818795680999755859375f,0.26789996109008740177159779705107212066650390625f,-0.9596999617919326208692609725403599441051483154296875f,-0.90190000948905879596395607222802937030792236328125f,0.551099995815753640471257313038222491741180419921875f,0.7369999941363925000104018181446008384227752685546875f,0.81260001396536818418780967476777732372283935546875f,0.19770002830326571796604184783063828945159912109375f,1.33599998557567545276469900272786617279052734375f,-0.06439998752847364560381038245395757257938385009765625f,2.29429997302293742222900618799030780792236328125f,-0.26289998442903195208231181823066435754299163818359375f,0.35409998264312747551230131648480892181396484375f,0.3619000015914435319785980027518235146999359130859375f,0.081500000715255538352721487171947956085205078125f,0.29069999968931081735235011365148238837718963623046875f,-0.366799959015846610554945073090493679046630859375f,0.16959999092817401589172732201404869556427001953125f,0.104099995160103819813457448617555201053619384765625f,-1.373199962455034839337031371542252600193023681640625f,0.142499957680701783147014793939888477325439453125f,-1.9718999385595328277531734784133732318878173828125f,1.5867000182151809184460944379679858684539794921875f,0.1379999951720236595065216533839702606201171875f,-0.0104000060915954151141704642213881015777587890625f,-0.53459999003410363371813218691386282444000244140625f,-0.18959999063014976172780734486877918243408203125f,0.7875999545097354115341659053228795528411865234375f,-0.44820000522136727028055247501470148563385009765625f,0.087400015445798950874944921451970003545284271240234375f,0.58420002126991688040646977242431603372097015380859375f,-0.6168000065132976938997444449341855943202972412109375f,-0.5212999207794661860049245660775341093540191650390625f,-1.7010000148564567989950546689215116202831268310546875f,-1.192800007176398668917727263760752975940704345703125f,1.0441999536931507730486146101611666381359100341796875f,-0.4865000580474745373038558682310394942760467529296875f,0.55640002743005734231473979889415204524993896484375f,-1.461399944430589403765452516381628811359405517578125f,-1.20950000870227736271544927149079740047454833984375f,-1.0159000404536724371240552500239573419094085693359375f,0.398800018568337133473278299788944423198699951171875f,0.07110001621171802010934470672509633004665374755859375f,0.876599989274143620576751345652155578136444091796875f,-1.0010999884396785120088679832406342029571533203125f,0.18450002546608512687953407294116914272308349609375f,0.847199980232120619660918237059377133846282958984375f,0.07440003178492238244956524795270524919033050537109375f,0.80790005019903077254639356397092342376708984375f,0.29809996478781053408368961754604242742061614990234375f,1.0222000288844110826858013751916587352752685546875f,0.80880004758834889155849623421090655028820037841796875f,-0.739800006341933880094074993394315242767333984375f,1.0085999875828621785700534019269980490207672119140625f,-0.378100020760298305422253406504751183092594146728515625f,1.2924999910779304546082357774139381945133209228515625f,-0.804400006583331883547316465410403907299041748046875f,0.315699996280669470394286690861918032169342041015625f,2.0818000069975841626046531018801033496856689453125f,-0.8928000419706105805772722305846400558948516845703125f,-0.0771999907374389948699899832718074321746826171875f,-1.1228999885082231458000023849308490753173828125f,0.58720003007650500848058072733692824840545654296875f,0.44429997659921571795393901993520557880401611328125f,-1.3868000576615333585550615680404007434844970703125f,-0.554199972021579778669320148765109479427337646484375f,0.206700026857852936501558360760100185871124267578125f,0.06920000465512288201352930627763271331787109375f,-1.140800021012127185571216614334844052791595458984375f,-0.7643000450327994688137778211967088282108306884765625f,0.942200007575750841937178847729228436946868896484375f,0.12800001564249396945882608633837662637233734130859375f,-0.5292000937268139804103839196613989770412445068359375f,1.89790003350973091045261753606610000133514404296875f,0.58059999915361348854503376060165464878082275390625f,-0.58459998019933667734449045383371412754058837890625f,-0.12639998891055570506836147615103982388973236083984375f,0.524999986961483866565458811237476766109466552734375f,1.1811000074386601710330069181509315967559814453125f,0.82970004318952561561673064716160297393798828125f,-0.11170002000331891167661524377763271331787109375f,-1.135000029448419933686409422080032527446746826171875f,-0.9422000160694123938043276211828924715518951416015625f,-0.0529999977983532932768184764427132904529571533203125f,1.4319999839663495322383823804557323455810546875f,-1.08140002532526846579230550560168921947479248046875f,-1.5976999891877170512088923715054988861083984375f,0.120999993044883236148478999893995933234691619873046875f,-0.2927000207781791374372915015555918216705322265625f,-0.211800002825260236249960144050419330596923828125f,1.037799995851516943190517849870957434177398681640625f,-0.36030001477599160608633610536344349384307861328125f,-1.45210005949214160381188776227645576000213623046875f,0.31360004059374346763178209585021249949932098388671875f,0.323200002315640222416703863927978090941905975341796875f,-1.418400011638552005166502567590214312076568603515625f,1.0304000656962397641791540081612765789031982421875f,0.805300010752677764003237825818359851837158203125f,-0.7887999587774263687833808944560587406158447265625f,1.30269994180202441924620870850048959255218505859375f,1.52069998137950701533327446668408811092376708984375f,-1.015500025179237209016491760849021375179290771484375f,0.839199977423251031183326631435193121433258056640625f,1.4953000221520660328877738720620982348918914794921875f,0.730800003726779667800883544259704649448394775390625f,-0.9460999730914831928885178058408200740814208984375f,-2.1109999833777539635093489778228104114532470703125f,0.429999965727329680476032081060111522674560546875f,-0.5499000290989879413672269947710447013378143310546875f,-0.67710002520084433541569524095393717288970947265625f,-1.369699996548891629544186798739247024059295654296875f,-0.15889999881982763696441907086409628391265869140625f,0.2249000152036548794143300256109796464443206787109375f,-0.32770001641958945715060735892620868980884552001953125f,0.36219998708665368525316807790659368038177490234375f,-0.22859999540597175649025984967011027038097381591796875f,-0.1317999463498598089472579886205494403839111328125f,1.1866000058516856796586580458097159862518310546875f,2.479099977874756888240881380625069141387939453125f,-0.19259994761794752005101827307953499257564544677734375f,-0.181499942302703942687003291212022304534912109375f,0.59129998839348563155482452202704735100269317626953125f,0.7426999999165531107792048715054988861083984375f,-0.50650001693889501463985425289138220250606536865234375f,0.09230003852844248513065394945442676544189453125f,-0.2554999951906504751519833007478155195713043212890625f,1.06250004313886137197187053970992565155029296875f,0.31529999465942371017490586382336914539337158203125f,-0.107700010719895100663734410773031413555145263671875f,-1.2980999993771311640244903173879720270633697509765625f,1.365000005662441395770656527020037174224853515625f,0.405099967920779846508594346232712268829345703125f,-0.5088000180840506203594486578367650508880615234375f,-1.2024000222444524155207545845769345760345458984375f,0.14419999198913568960733755375258624553680419921875f,0.406999998569489207511651329696178436279296875f,0.6312000215530400026864299434237182140350341796875f,-0.9171000430077322906896597487502731382846832275390625f,0.322599973806739104276175567065365612506866455078125f,0.3722999982953070219338087554206140339374542236328125f,0.37170001273900232785507569133187644183635711669921875f,-0.685599982538819485711201195954345166683197021484375f,-1.25949998262524598402478659409098327159881591796875f,0.127899995213747619260402643703855574131011962890625f,-0.022899986533820193557176025933586061000823974609375f,0.4873999792546033216211753824609331786632537841796875f,-1.1471999908864500827831989226979203522205352783203125f,0.071000012472272278074569840100593864917755126953125f,0.02389999177902979443643971535493619740009307861328125f,0.00089999811574814625458884620456956326961517333984375f,0.51349997775256639442886807955801486968994140625f,-0.467599995860457529062159665045328438282012939453125f,0.09880001361370016610408129054121673107147216796875f,0.7307000031724566468938064645044505596160888671875f,0.9316999584615235363571628113277256488800048828125f,-0.8290999834068120133423462903010658919811248779296875f,0.3813999781817205558809291687794029712677001953125f,-0.342699983935057961303982665413059294223785400390625f,0.71090001407861702631407752051018178462982177734375f,-0.64239999700561123407993591172271408140659332275390625f,0.041399966521562081567253699176944792270660400390625f,-1.0031999575376513433866421109996736049652099609375f,-0.37400001144409156239589719916693866252899169921875f,0.1210000208169217383868954129866324365139007568359375f,0.4271999833613637775897586834616959095001220703125f,-0.7917999854654074187720880217966623604297637939453125f,0.087099960678816756853848346509039402008056640625f,-1.0405999589204792954433287377469241619110107421875f,0.253699950399993934269105011480860412120819091796875f,-1.426399977809190477984202516381628811359405517578125f,0.65079999631345319954789374605752527713775634765625f,-0.4770999612003563594697652661125175654888153076171875f,0.0488999588474623436695765121839940547943115234375f,-0.782399987897276272263979990384541451930999755859375f,0.645799997505544620679529543849639594554901123046875f,-0.39579998476505284799031869624741375446319580078125f,-0.69859997566193243745402696731616742908954620361328125f,-0.7683000246629116958274607895873486995697021484375f,0.39549996041506574329105205833911895751953125f,0.94059998501613650834229929387220181524753570556640625f,0.70659998927414324754181507159955799579620361328125f,-0.6646999879062178706590202637016773223876953125f,-1.137799988140165918792945376480929553508758544921875f,-0.34289999130219162015720257841167040169239044189453125f,0.1952000101611017779390522264293394982814788818359375f,-0.13080002738237350712324769119732081890106201171875f,0.45890000850558276823676351341418921947479248046875f,1.27919999735355371939249380375258624553680419921875f,0.45990001635849486749663128648535348474979400634765625f,0.62410000935345888439798045510542578995227813720703125f,1.23299998818710410120047527016140520572662353515625f,0.78560000984519706701547647753614000976085662841796875f,0.6399000207543374951768555547459982335567474365234375f,0.0041999789878719440139320795424282550811767578125f,-0.6323000069379809406910908364807255566120147705078125f,-0.806100023867190262905069175758399069309234619140625f,-0.8319000043109061603985310284770093858242034912109375f,0.68620001935511876212103743455372750759124755859375f,-0.27280002205893449929163807610166259109973907470703125f,0.4551000191807752504047357433591969311237335205078125f,-0.3358000454314062910299298891914077103137969970703125f,-0.0488999854832884162103567859958275221288204193115234375f,1.474700051552802815280074355541728436946868896484375f,-0.605500060494990055559583197464235126972198486328125f,0.76610002362504603379278478314517997205257415771484375f,-0.14789999387264263219776694313623011112213134765625f,-0.6955000110231341903244128843653015792369842529296875f,0.873500009864569104678366784355603158473968505859375f,0.13010000046119063465965837167459540069103240966796875f,0.16820004212036676616293107144883833825588226318359375f,0.471400000458955137361272136331535875797271728515625f,1.072600025290250780329870394780300557613372802734375f,0.54989999026283575833673467059270478785037994384765625f,1.2605000003874298197814596278476528823375701904296875f,-0.54620001624450054134030096975038759410381317138671875f,0.88149994377419382818317217243020422756671905517578125f,0.346599999406933978463030143757350742816925048828125f,-0.23539999446868886678885246510617434978485107421875f,0.9454000277727840906294431988499127328395843505859375f,-0.6860000001788133516811285517178475856781005859375f,-0.1268000434309246582387231683242134749889373779296875f,0.82399998657777917543398871202953159809112548828125f,-0.0399999776482580227110474879737012088298797607421875f,1.0369000509887940086883872936596162617206573486328125f,-0.07249999083578562153462598871556110680103302001953125f,0.86300005229935106143557277391664683818817138671875f,-0.38549995501339395786999375559389591217041015625f,-1.04140003866180830272014645743183791637420654296875f,0.55799995702505089667511128936894237995147705078125f,-1.147200050938129489708217079169116914272308349609375f,0.249399998909234366095688528730534017086029052734375f,0.6778000131368637770634677508496679365634918212890625f,1.54600000673532544936961130588315427303314208984375f,-0.547499985694885449305502334027551114559173583984375f,-0.188000005546957338342650700724334456026554107666015625f,0.32880000988841029396780868410132825374603271484375f,-0.52070000667422922102645088671124540269374847412109375f,0.666599997991323078849745797924697399139404296875f,1.6349000098206103448461590232909657061100006103515625f,-0.74099999615550071752068106434307992458343505859375f,0.02930001746565114029152709917980246245861053466796875f,1.15169998841285714519244720577262341976165771484375f,0.956100003974138967777207653853110969066619873046875f,-0.355700003284215693355463372427038848400115966796875f,-1.157700015600025889028756864718161523342132568359375f,1.5919999662339687773737750831060111522674560546875f,1.347299980860948931393750171991996467113494873046875f,0.18630000988841022291353510809130966663360595703125f,1.42099998103082203471103639458306133747100830078125f,0.372399945354462857949329190887510776519775390625f,0.271899993246794036139135641860775649547576904296875f,-0.4990000358819965953216524212621152400970458984375f,0.0790000031739470642833111924119293689727783203125f,0.030099989080429878640643437393009662628173828125f,0.032300014463066961667436771676875650882720947265625f,0.20890000209808334830086096189916133880615234375f,0.99549998116493210176258799037896096706390380859375f,-0.058999985665082466113062764634378254413604736328125f,0.80659995984435095550679761799983680248260498046875f,1.280499971181153995303247938863933086395263671875f,0.56680002979636157078857650049030780792236328125f,2.00809999930262517153778389911167323589324951171875f,1.295299964112043511477168067358434200286865234375f,0.3087999532639980060366724501363933086395263671875f,-0.700899987107515443085503648035228252410888671875f,-0.196400016403198041103905779891647398471832275390625f,0.5011999935388562477101004333235323429107666015625f,-0.74050000098347634747142365085892379283905029296875f,-0.586200008887052259609617976821027696132659912109375f,-0.03009999731332058114929850489716045558452606201171875f,0.63760001281052824939621359590091742575168609619140625f,-0.0573999994084239961722460066084749996662139892578125f,0.224800007310509197822057103621773421764373779296875f,-0.628299993705749404426796900224871933460235595703125f,0.142800009784102233112434987560845911502838134765625f,-0.08260001053810039906011297716759145259857177734375f,-0.29159998240098372601636356193921528756618499755859375f,1.5294000039756294739845543517731130123138427734375f,1.31210002790167923336639432818628847599029541015625f,-0.63610000326633464595715850009582936763763427734375f,-0.53099997266754439539937493464094586670398712158203125f,1.8070999748162923648209243765450082719326019287109375f,-0.18120002831444181889963829235057346522808074951171875f,1.999499990019947492925211918191052973270416259765625f,0.53460000120997441808157191189820878207683563232421875f,-0.2240999158114203027025723713450133800506591796875f,0.493900076529383225132363577358773909509181976318359375f,-1.846999964222311607642268427298404276371002197265625f,0.327799971190094741047005300060845911502838134765625f,0.456599948370456587554144789464771747589111328125f,1.0021000150680545726089576419326476752758026123046875f,0.7421000249028200101975016877986490726470947265625f,0.094599975430965432821039939881302416324615478515625f,-0.86489999938979755800261273179785348474979400634765625f,-1.011899991104751261872252143803052604198455810546875f,-1.452400003152340079992654864327050745487213134765625f,1.1969999398589135086723445056122727692127227783203125f,-0.138100038120150703235822220449335873126983642578125f,0.7433000006347889598856681914185173809528350830078125f,0.27449999223649523827361917938105762004852294921875f,-0.95679997764229707257754853344522416591644287109375f,0.42700003656745000313321725116111338138580322265625f,1.03319996007084835554223900544457137584686279296875f,-0.51010005792379420341831064433790743350982666015625f,0.95479998965263401800029896548949182033538818359375f,0.4976000187337394908837495677289552986621856689453125f,0.1253999753952025619785359822344616986811161041259765625f,-0.1058999756336209685514404554851353168487548828125f,0.61470001990646128131601244604098610579967498779296875f,-0.27800001484155656417129875990212894976139068603515625f,-1.5150000461749735247707349117263220250606536865234375f,-0.012599990067630850720803437070571817457675933837890625f,0.9653000292301185947252406549523584544658660888671875f,-0.379600043380261098491246229968965053558349609375f,0.076600007956475479886648827232420444488525390625f,-0.27589997622966766499530422152020037174224853515625f,-0.103599993487447417095381752005778253078460693359375f,0.6929999817609786560979046043939888477325439453125f,1.029400009340047716932531329803168773651123046875f,1.28409999083876602554710188996978104114532470703125f,-0.5657000166021288745099582229158841073513031005859375f,-0.6118999752163885030853407442918978631496429443359375f,0.84830001588985337601656055994681082665920257568359375f,-1.01260001320168324667747583589516580104827880859375f,-1.513299989458917860218889472889713943004608154296875f,-0.7898000909805293900944889173842966556549072265625f,-0.896899989372491379668872468755580484867095947265625f,-0.17199999946355859492541640065610408782958984375f,0.9060000424981111688538248927216045558452606201171875f,-0.255899988001585310115615357062779366970062255859375f,-0.1642000228345394940987489462713710963726043701171875f,0.06489998543858543911255765124224126338958740234375f,1.122700024503469506953479140065610408782958984375f,0.748800014079362252772398278466425836086273193359375f,0.4688999968081706271050279610790312290191650390625f,1.139100029488652854325891894404776394367218017578125f,-0.2323999288141731511814214172773063182830810546875f,-0.1075999951899060391724560759030282497406005859375f,-0.4123000161767009785762638784945011138916015625f,-1.400200004833937139636645952123217284679412841796875f,0.28710001626014669540154500282369554042816162109375f,0.2048000127680602577839863442932255566120147705078125f,-0.143300010097026753186355563229881227016448974609375f,-0.1880000216960906467278391573927365243434906005859375f,-0.98590001068860255717396512409322895109653472900390625f,0.5742000346809621458277206329512409865856170654296875f,1.84659999858736956213078883592970669269561767578125f,-0.60260000070333408217493342817761003971099853515625f,-0.5120999842040243610341576641076244413852691650390625f,1.110000013560056952854893097537569701671600341796875f,1.0239000179864461426149091494153253734111785888671875f,-0.518800011825561568912235088646411895751953125f,-1.05400000106915836539656083914451301097869873046875f,-0.8754999481216072521050364230177365243434906005859375f,-0.74209997505843627862986977561376988887786865234375f,-0.272200026053190047292673625634051859378814697265625f,1.792299964618683016936984131461940705776214599609375f,1.4067999697446822437996161170303821563720703125f,0.101700057330727400284331451985053718090057373046875f,0.006500012636184493430846487171947956085205078125f,0.985500010967254258531511368346400558948516845703125f,0.18660000827312472893027006648480892181396484375f,-0.2288000033318997594733446021564304828643798828125f,-0.58579999221563383571265148930251598358154296875f,1.41729999121725569466434535570442676544189453125f,0.26740001198127882897637164205661974847316741943359375f,-0.757699969965219821688151569105684757232666015625f,-0.3209999928027393867324690290843136608600616455078125f,-0.20930002242028677983398665674030780792236328125f,0.55340000863969318611879089075955562293529510498046875f,0.97109999372959077135192273999564349651336669921875f,0.07749997526407259584857456502504646778106689453125f,-0.10439998534917815931066797929815948009490966796875f,0.43680004216432610775200373609550297260284423828125f,0.991099998348951682913821059628389775753021240234375f,0.1437000023305419205144062289036810398101806640625f,-0.3441000428438183433854646864347159862518310546875f,0.8375000083446499932193773929611779749393463134765625f,0.06369997930824790322024142597001628018915653228759765625f,0.01280000456869601066500763408839702606201171875f,-0.4844000026717776297147111108643002808094024658203125f,-0.2262999873131519290581081804702989757061004638671875f,0.14289997742548565806686156065552495419979095458984375f,-0.0479999804198740775973419658839702606201171875f,-0.7676000402659184373987955041229724884033203125f,0.3895000164508826134834862386924214661121368408203125f,0.3922000249542294536553299622028134763240814208984375f,-0.6815000149458649847389324349933303892612457275390625f,-0.8907000331610444021634975797496736049652099609375f,0.339100013935566213518768563517369329929351806640625f,-0.38690000524222833977461277754628099501132965087890625f,0.581699965912104044463148966315202414989471435546875f,1.4134999650120736447433955618180334568023681640625f,0.66379997114539168734381746617145836353302001953125f,0.4254999872744082978215374168939888477325439453125f,-0.9667999773442748079332886845804750919342041015625f,0.62020001325309215278736019172356463968753814697265625f,-0.554999962002038937924908168497495353221893310546875f,0.34989999132454407959613718048785813152790069580078125f,-1.22679995320439427786141095566563308238983154296875f,0.0753000155955547700870056360145099461078643798828125f,0.9526000001817938045434175364789552986621856689453125f,-0.273399975410104190132187795825302600860595703125f,0.42900001010298705050871603816631250083446502685546875f,0.474099981078505816611823320272378623485565185546875f,0.50979998540580251376042042465996928513050079345703125f,1.276499955564737387447848959709517657756805419921875f,1.60739997843503967800415921374224126338958740234375f,0.344099984282255189071975109982304275035858154296875f,1.4251000220865017720228706821217201650142669677734375f,1.522099970290065495959197505726478993892669677734375f,1.012999976918101285860984717146493494510650634765625f,-0.04230000984370718697391566820442676544189453125f,1.0952999905616043196943110160646028816699981689453125f,0.21059999052956701603989131399430334568023681640625f,0.10710002371072757654246743186376988887786865234375f,1.179100040142983640834017933229915797710418701171875f,-1.08130003298521071286586447968147695064544677734375f,-0.0317999752208582286971960684240912087261676788330078125f,0.178599986297637347565370191659894771873950958251953125f,-0.55730001254454253345471670400002039968967437744140625f,0.857100028777122435741375738871283829212188720703125f,1.6451000363357366484962085451115854084491729736328125f,-0.64150003319978765592423997077275998890399932861328125f,-0.073399989379942243150622971370466984808444976806640625f,-0.546800004464387701119676421512849628925323486328125f,-0.557799966049194129169563893810845911502838134765625f,0.876299958628415698314029214088805019855499267578125f,-0.4884999621063474961601968971081078052520751953125f,0.77529993907809302555733665940351784229278564453125f,-0.5143000657081613358201366281718946993350982666015625f,-0.7594000395148976512160743368440307676792144775390625f,1.0200000751018531985891968361102044582366943359375f,0.37850002640485680416304603568278253078460693359375f,-0.042100042709709573074405852821655571460723876953125f,0.549999935030938136293343632132746279239654541015625f,-1.4476999722748986609843768746941350400447845458984375f,0.85020004920214475863105008102138526737689971923828125f,0.916500034876168268027640806394629180431365966796875f,-1.3606000430747868801262256965856067836284637451171875f,-0.465199990752339420652106127818115055561065673828125f,-1.164200011453777161563039044267497956752777099609375f,-0.028800014097988491101887120748870074748992919921875f,0.7235000120997430173019893118180334568023681640625f,0.9208999447137120597517423448152840137481689453125f,1.23789999983310661235691441106610000133514404296875f,0.9371000220715988149322583922185003757476806640625f,-0.40559998041540412661021264284499920904636383056640625f,0.46799998038262202726400573737919330596923828125f,0.06420000191703445491242518983199261128902435302734375f,0.0060000379905111866918332452769391238689422607421875f,1.2687000347591936844082738389261066913604736328125f,0.5281000197172163179715198566555045545101165771484375f,-0.52479999892860629273627637303434312343597412109375f,0.421699984836578778413240797817707061767578125f,-1.04379999118745292463472651434130966663360595703125f,-0.200900014078616351298478548415005207061767578125f,-1.2499000036552541104128977167420089244842529296875f,-0.826099984602630232899400652968324720859527587890625f,0.43049998688325263884735250030644237995147705078125f,0.7984999874755744908583210417418740689754486083984375f,-0.0367000015258784895166854767012409865856170654296875f,-0.92580005899146289305434720517951063811779022216796875f,-0.50779999991208291110211803243146277964115142822265625f,-1.0834000607937575022532428192789666354656219482421875f,-0.63129995812550154798969970215694047510623931884765625f,0.2917999840497973451647339970804750919342041015625f,-0.8499999621510507807897738530300557613372802734375f,-0.50249999731779126932451617904007434844970703125f,0.3470000176429746119310948415659368038177490234375f,0.614899977801739883176423973054625093936920166015625f,0.210499980494380789508568341261707246303558349609375f,0.32510001858472781233189152771956287324428558349609375f,1.056700018811226104986644713790155947208404541015625f,0.0097999950170517546865767144481651484966278076171875f,-0.4498000085771085565511384629644453525543212890625f,0.05650000112503850147049888619221746921539306640625f,1.11670000439435224137696422985754907131195068359375f,-0.32620005910396654424943108097068034112453460693359375f,0.19130002620518216094325225640204735100269317626953125f,-0.152400009317695595001396213774569332599639892578125f,0.952900006687640743763267892063595354557037353515625f,-0.059600042821467358322706786566413938999176025390625f,-0.591400042927265889858290393021889030933380126953125f,0.31470003745257890681585877246106974780559539794921875f,-0.60879999653324501363016452160081826150417327880859375f,-1.4320000218153001725340800476260483264923095703125f,-0.68300002860650443903267614587093703448772430419921875f,0.577699960540235313288803808973170816898345947265625f,-0.08709997105374911452457808991312049329280853271484375f,-0.1822999793335797757976024513482116162776947021484375f,-1.02360002710819220084204062004573643207550048828125f,1.0152000145196922176182852126657962799072265625f,0.6774000553250318912290595108061097562313079833984375f,-0.785899996346234530619767610915005207061767578125f,-0.156400016701221833415047512971796095371246337890625f,0.616800018918514325605428894050419330596923828125f,0.35350000107288348516476617078296840190887451171875f,-1.005500033617019450815632808371447026729583740234375f,-0.75899995854496982161663254373706877231597900390625f,1.651900026960670953002363603445701301097869873046875f,-0.875400016149879167670633250963874161243438720703125f,-0.934499999314545970463541380013339221477508544921875f,-0.48599997100979075437265919390483759343624114990234375f,2.20730007771849745523695673909969627857208251953125f,0.63829997850656372548883155104704201221466064453125f,-1.789600054556131691896325719426386058330535888671875f,-0.06569999046921726204573133145458996295928955078125f,1.19770003195405028151299120509065687656402587890625f,-1.023200020942092169917714272742159664630889892578125f,0.94389995854943886666177377264830283820629119873046875f,1.95620006685256964118480027536861598491668701171875f,-0.6627999445915209175694826626568101346492767333984375f,-1.3942000432491301609161382657475769519805908203125f,-0.6774000396415600366850640057236887514591217041015625f,-0.531400013422965944442921681911684572696685791015625f,0.0441999903127550108905552406213246285915374755859375f,0.87320002150088538162009399457019753754138946533203125f,-0.523900009660422938395640812814235687255859375f,-0.83230002176463624419255893371882848441600799560546875f,0.116500007979572384186894851154647767543792724609375f,-0.194100017064809560185523196196299977600574493408203125f,1.0382999957919114564219853491522371768951416015625f,-0.127100017359108363734776503406465053558349609375f,0.723500013589859491958122816868126392364501953125f,1.252800063316524603607149401796050369739532470703125f,-0.146999983072280571150258765555918216705322265625f,-0.787400025336444375767541714594699442386627197265625f,1.274400015859305401733081453130580484867095947265625f,-0.0189999948665497964128690000507049262523651123046875f,-0.7713000372692950801223332746303640305995941162109375f,-0.8647000465795404711144556131330318748950958251953125f,-0.0144000288605692361443288973532617092132568359375f,-0.7594000068813564485736833375995047390460968017578125f,0.94020002226829557656628821860067546367645263671875f,-0.47909996744394245382636654539965093135833740234375f,0.605800014269352260498635587282478809356689453125f,0.250499959707259023389269714243710041046142578125f,-1.56700006845593531323856950621120631694793701171875f,-1.53000000149011494698925162083469331264495849609375f,1.32200001574307623997128757764585316181182861328125f,-0.148399986138939343760512201697565615177154541015625f,-1.25460006662607259642072676797397434711456298828125f,-0.57739996655136305303557264778646640479564666748046875f,1.01740004101991754481559837586246430873870849609375f,0.96630003075003667589726319420151412487030029296875f,-1.45390004449188747770449481322430074214935302734375f,-0.6514000582754622126202548315632157027721405029296875f,0.1980000570863484909978069481439888477325439453125f,0.544100031891466162647930104867555201053619384765625f,-0.053200017477572192348134194617159664630889892578125f,-0.03929999623149649234932212493731640279293060302734375f,-1.125099931114912354956913986825384199619293212890625f,-1.11990001188814591870368531090207397937774658203125f,-0.87570004780143440026307644075131975114345550537109375f,-0.430100018845498810460270533440052531659603118896484375f,1.1793999759241937663745147801819257438182830810546875f,0.719100042359531510527403952437452971935272216796875f,-0.1304000221848493623610920622013509273529052734375f,-0.6949000488244003737037246537511236965656280517578125f,-0.46890002146959286477567729889415204524993896484375f,0.644600010597705885828645477886311709880828857421875f,0.243900046913325230235614071716554462909698486328125f,0.5797999871194361443116349619231186807155609130859375f,0.7893000194028021443415354951866902410984039306640625f,1.180100029797852112523059986415319144725799560546875f,-0.10689997223615677057750872336328029632568359375f,-0.617700009099393820832801793585531413555145263671875f,-0.4126000486850738280253381162765435874462127685546875f,-0.61350003549456655349558786838315427303314208984375f,-0.823100048848987686511691208579577505588531494140625f,0.955200015711784811145435014623217284679412841796875f,1.1052000523686411614932012525969184935092926025390625f,0.058799925473332681491456241928972303867340087890625f,-0.237999982319772929173495867871679365634918212890625f,-0.051799997386335672189261458697728812694549560546875f,0.82059999647140513534537831219495274126529693603515625f,0.12119995401352678943140972478431649506092071533203125f,-1.419700005396455022577129057026468217372894287109375f,-1.133199927511812088454234981327317655086517333984375f,-1.094200043025612689717718239990063011646270751953125f,0.71380003633797228790314193247468210756778717041015625f,1.57890001483857655983911172370426356792449951171875f,-0.1335999791264530500711771310307085514068603515625f,1.1493999902665616996699782248470000922679901123046875f,0.29009999559968646654084523106575943529605865478515625f,0.06270001142770009749938253662548959255218505859375f,-0.42139999721795307063842983552603982388973236083984375f,-0.080199994738400059990368617945932783186435699462890625f,-0.16349999880045640310299859265796840190887451171875f,-0.32880000679641929739460692871944047510623931884765625f,0.4261999859392637812760540327872149646282196044921875f,-1.53220000314265547558534308336675167083740234375f,-0.39210002099126606278645112979575060307979583740234375f,-1.5640999969482418663346834364347159862518310546875f,-0.103299999684840049241074666497297585010528564453125f,-0.269100001083314221972386803827248513698577880859375f,0.33280000702738765649968399884528480470180511474609375f,-0.461199994730949303090028479346074163913726806640625f,0.2068999951839446682644307884402223862707614898681640625f,-0.566699989381432178703335011960007250308990478515625f,-0.2579999999590214532219079046626575291156768798828125f,0.546299999830127003264124141423963010311126708984375f,0.3464000208921731438493907262454740703105926513671875f,0.0754999594092371495435145334340631961822509765625f,1.028900005786120619433177125756628811359405517578125f,-1.06509997507333675770269110216759145259857177734375f,0.70090000185966427892481078743003308773040771484375f,0.432000013507902291820528262178413569927215576171875f,1.001399981683492423911729929386638104915618896484375f,-0.1259999993592499922812066870392300188541412353515625f,0.43679996672719745287594150795484893023967742919921875f,-0.23400000213086580291843574741506017744541168212890625f,-0.2511999929800630138032602189923636615276336669921875f,1.2553000053882594766463398627820424735546112060546875f,-0.8327000040143730519304199333419091999530792236328125f,-0.4105000418126589778466950519941747188568115234375f,1.156700011286139417876484003500081598758697509765625f,-1.21449996392428882785452515236102044582366943359375f,-0.1627000179097051191234868383617140352725982666015625f,-0.369999986365437738555783653282560408115386962890625f,-0.165100002193451000920276783290319144725799560546875f,0.100699992518127334761857127887196838855743408203125f,2.184700041997432951035307269194163382053375244140625f,0.515600040541590143305938909179531037807464599609375f,-2.0920000668168068358454547706060111522674560546875f,-0.0104000782504682920404093238175846636295318603515625f,2.0844000229001036217368891811929643154144287109375f,1.5045000383555884315711637100321240723133087158203125f,-0.52419999202638845048340954235754907131195068359375f,0.317499971091747656970483149052597582340240478515625f,-0.55170000383555872769392180998693220317363739013671875f,-1.681800012511015296468031010590493679046630859375f,-0.60060004283264289970389882000745274126529693603515625f,0.64639997125267978361762288841418921947479248046875f,-0.205699988289922419681943210889585316181182861328125f,-2.340000008940695153114575077779591083526611328125f,-0.0091999808132643412506013191887177526950836181640625f,0.802699976968764161711078486405313014984130859375f,-1.154000050187111270361128845252096652984619140625f,-1.186499996542931256726660649292171001434326171875f,-0.7675999464631078073040271192439831793308258056640625f,0.28529998482465723785850286731147207319736480712890625f,0.9979999718144541009934300745953805744647979736328125f,-0.230800016504526528393625994795002043247222900390625f,-0.56709997468590689351231048931367695331573486328125f,1.682500083148480651829004273167811334133148193359375f,0.406900033479927980550883148680441081523895263671875f,-2.2887000689387324570134296664036810398101806640625f,-0.54100003385543882217234568088315427303314208984375f,1.4558000339388836863463438930921256542205810546875f,1.187900017416477993492662790231406688690185546875f,-0.139299997460842206464803894050419330596923828125f,-0.9354000442385679292556233122013509273529052734375f,-0.0181000539153810535708544193767011165618896484375f,-0.094600017154217397319371229968965053558349609375f,0.8455999915540222122700697582331486046314239501953125f,0.37040000162124631089000104111619293689727783203125f,-0.1133000116989020256141884601674973964691162109375f,-0.07789998224973704310514222015626728534698486328125f,-0.0974000282049178878196471487171947956085205078125f,-0.4041000140845778521025977170211263000965118408203125f,-0.525099963338673259016786687425337731838226318359375f,0.020999990567565074428557636565528810024261474609375f,-0.02549998883903004109896528461831621825695037841796875f,0.08929999582171443961442491854541003704071044921875f,-0.222099969917536554930848069489002227783203125f,-0.788999995946884524755660095252096652984619140625f,-1.1747999588072304266717083010007627308368682861328125f,-1.1626999619930984408711083233356475830078125f,0.2949999707192187958781914858263917267322540283203125f,0.316700048390031962952662070165388286113739013671875f,-0.16119998117089284761505041387863457202911376953125f,-0.059500038802624555245301962713710963726043701171875f,-0.019299997013807246304395448532886803150177001953125f,-1.068899985483287906617988483048975467681884765625f,-0.62349999257922117390506855372223071753978729248046875f,0.7665000204592942179004921854357235133647918701171875f,-0.51760002612844147051873733289539813995361328125f,0.63260003117620999812942272910731844604015350341796875f,0.0675999812945719258383547867197194136679172515869140625f,-0.404299989805370574291742968853213824331760406494140625f,-0.43390002342537048463100290973670780658721923828125f,-0.3780999952793120399974213796667754650115966796875f,-0.366500020403414783487505701486952602863311767578125f,-0.05120000277757608131423694430850446224212646484375f,-0.427000000245868982506891597950016148388385772705078125f,-0.9434999846071006057712793335667811334133148193359375f,-0.3445000285021964092635471388348378241062164306640625f,0.6765999768689265891907780314795672893524169921875f,1.40569999596401995489713954157195985317230224609375f,2.64750002671033168866188134416006505489349365234375f,0.87159997711107128903762486515915952622890472412109375f,0.68000001998618220255821142927743494510650634765625f,0.016200004938244916274925344623625278472900390625f,-1.0847999783277515462032170034945011138916015625f,-1.43379997472167008965016066213138401508331298828125f,-0.510199993956089059565783827565610408782958984375f,0.5023999860346315404768802181934006512165069580078125f,-0.161699979062378762506568818935193121433258056640625f,-0.39389997635632767458702119256486184895038604736328125f,-0.172299996469914784569255061796866357326507568359375f,-0.7277000029340385811593705511768348515033721923828125f,0.059500018145889566056894182111136615276336669921875f,1.332600001075864071253818110562860965728759765625f,1.648900029125064836676983759389258921146392822265625f,0.88329999117255209029053730773739516735076904296875f,1.56820001058578473163152011693455278873443603515625f,-0.55379995847940488573613038170151412487030029296875f,0.45910003837347002075830459943972527980804443359375f,0.46749997571110790683945879209204576909542083740234375f,-0.3096999939039346561031607052427716553211212158203125f,-1.180899977310002402219879513722844421863555908203125f,-0.8430000069439411714000698339077644050121307373046875f,-0.912999994389712465903130578226409852504730224609375f,0.172000000208615944075063453055918216705322265625f,-0.426700028981268297822992963119759224355220794677734375f,-0.2695000056475398420019473633146844804286956787109375f,-0.28509999131560281870889639321831054985523223876953125f,0.32340001325905287643536212272010743618011474609375f,0.7407000446721923037074475359986536204814910888671875f,-0.6570000164508822404485499646398238837718963623046875f,-0.27039998936504072712949664492043666541576385498046875f,1.06780003474354767689646905637346208095550537109375f,0.5204000150881709085837201200774870812892913818359375f,-0.9356000185251236445083122816868126392364501953125f,-0.021400029516219998271253643906675279140472412109375f,-1.48159997206330285024478143895976245403289794921875f,0.434000040873885328807091354974545538425445556640625f,-1.101900022117793032094823502120561897754669189453125f,-0.48240002123862468774717626729398034512996673583984375f,-0.8081000099197035435594216323806904256343841552734375f,0.03170004079043931000114753260277211666107177734375f,-0.1017999977588655013960305950604379177093505859375f,-0.433300016728043857483498868532478809356689453125f,0.4157000149071219397001186734996736049652099609375f,0.5913000115275384160895555396564304828643798828125f,0.14649998044967649235559292719699442386627197265625f,0.7204999834001066805200252929353155195713043212890625f,0.9031999974727629165727194049395620822906494140625f,0.39040000028014099076045795300160534679889678955078125f,0.6381999912515279316238547835382632911205291748046875f,0.33940000520497581870671410797513090074062347412109375f,1.0481000241503115599783768630004487931728363037109375f,-0.18870001350641274484587484039366245269775390625f,-1.1322000758230696959572014748118817806243896484375f,-0.6886999941349027931636328503373079001903533935546875f,-0.018899982726573671953929078881628811359405517578125f,-0.32780001060366625864617162733338773250579833984375f,0.1467999985784291805401835517841391265392303466796875f,-0.4934000197969374834627842574263922870159149169921875f,0.71060000913739163053151060012169182300567626953125f,0.82160000631734730891508888817043043673038482666015625f,-0.59830000428557372771365407970733940601348876953125f,-0.704600036581606303798253065906465053558349609375f,0.9485000500977045501116435843869112432003021240234375f,0.25920000269934517778125382392317987978458404541015625f,-0.4715000232905151023032885859720408916473388671875f,-0.35130000854730603077058503913576714694499969482421875f,-0.10910000283420107081155947525985538959503173828125f,-0.78720003570169228002129102605977095663547515869140625f,-1.2686999768495557106007254333235323429107666015625f,-0.4814000370413065166275146111729554831981658935546875f,0.4137999352335928193724612356163561344146728515625f,1.431200039732456463781318234396167099475860595703125f,1.35700000929832498286486952565610408782958984375f,0.450299996224045262493973496020771563053131103515625f,-0.4278999920845034665717321331612765789031982421875f,0.24110000572502610083347462932579219341278076171875f,0.173099984587728794593886050279252231121063232421875f,0.11159996509700997879832584658288396894931793212890625f,-0.9943000095680349925686414280789904296398162841796875f,0.3226000161260371701388294241041876375675201416015625f,0.59870001626312696618725794905913062393665313720703125f,-1.4671999965488904393851044005714356899261474609375f,0.2380000093281271489331629709340631961822509765625f,0.70699999067187224710551163298077881336212158203125f,-0.55590004209280063918185987859033048152923583984375f,0.6013000021398067218569849501363933086395263671875f,1.88800007995963170515096862800419330596923828125f,0.14370001946687727123475042390055023133754730224609375f,0.3334000063672661884339731841464526951313018798828125f,1.2033000138595699723964571603573858737945556640625f,1.21360002885907913849905526149086654186248779296875f,0.88419997977018294221807082067243754863739013671875f,0.4464000080585481811112913419492542743682861328125f,1.1190000589601700742292678114608861505985260009765625f,-0.38440003064870864601942912486265413463115692138671875f,-0.217300033387542834706351868589990772306919097900390625f,0.1646000117152934816289189257076941430568695068359375f,0.065700026325136218208200489243608899414539337158203125f,0.8422999864488838195342168546631000936031341552734375f,0.914300006904452455813725464395247399806976318359375f,-0.356999987170099952749069416313432157039642333984375f,-0.386299963117390288136476783620310015976428985595703125f,1.0351999838232988082609153934754431247711181640625f,-0.37900003632903056516312290114001370966434478759765625f,-0.26709999912381210407374965143389999866485595703125f,-0.8134000150471927259587801017914898693561553955078125f,-0.79210000255107804179033337277360260486602783203125f,-0.033099965849519019656099771964363753795623779296875f,-0.713100002804398780398287271964363753795623779296875f,0.08659998927414402913882440770976245403289794921875f,0.916599987895786671998621386592276394367218017578125f,1.278300010953843379724048645584844052791595458984375f,0.7884000226840373048986521098413504660129547119140625f,0.1726999738208954138229955788119696080684661865234375f,-0.4067999876260757208257246020366437733173370361328125f,-1.01430003141686331247228736174292862415313720703125f,-0.37710002654194862969916357542388141155242919921875f,0.87450004580616980121021697414107620716094970703125f,-0.3291999990671872211578374844975769519805908203125f,-0.52980003852844281819756133700138889253139495849609375f,-1.003600003191828182735889640753157436847686767578125f,-0.93529997916221574438822017327765934169292449951171875f,-0.4993999888882039162041337476694025099277496337890625f,0.69819995548874158597385530811152420938014984130859375f,-0.63220002907067485153902453021146357059478759765625f,0.31210001313090340868683369990321807563304901123046875f,1.294599990835040070891182040213607251644134521484375f,-0.49409997765123858659563893525046296417713165283203125f,-0.407300022956729179668400320224463939666748046875f,-0.7086000164911150367430536789470352232456207275390625f,0.0249000042140483657249205862171947956085205078125f,0.062300004125387019315862602297784178517758846282958984375f,-0.56510000904798507459503298377967439591884613037109375f,0.1309999980367718974694213329712511040270328521728515625f,-0.238599986986815781886406284684198908507823944091796875f,-0.2962999932922423251824284307076595723628997802734375f,-0.04469999524503942556208357927971519529819488525390625f,-0.20039999048262802094910739469924010336399078369140625f,0.00219999505877499501593774766661226749420166015625f,0.6942999936424187890082748708664439618587493896484375f,-0.4177000196233391360323139451793394982814788818359375f,-0.8148000048518178051182303534005768597126007080078125f,0.3193000180244442987032016390003263950347900390625f,-0.4491999852091073108795171719975769519805908203125f,0.6699999871850017552077360960538499057292938232421875f,-0.048799994316697181684361339648603461682796478271484375f,0.02650000376999361861862780642695724964141845703125f,-0.00049999826401470048864439377211965620517730712890625f,-0.7467999599471693006336181497317738831043243408203125f,0.77470002202987675588019556016661226749420166015625f,-0.559099996575712854252060424187220633029937744140625f,-0.7401999843448405780321763813844881951808929443359375f,-0.8074000114038586417564147268421947956085205078125f,-0.89690000591278096209180148434825241565704345703125f,-0.626499998182058703832808532752096652984619140625f,-0.670099973061681009056655966560356318950653076171875f,-1.002000002108514475906986262998543679714202880859375f,0.0729999909624456932988323387689888477325439453125f,-0.507199992898106355454501681379042565822601318359375f,0.399999979138374539555655928779742680490016937255859375f,0.2908999966815113413787230456364341080188751220703125f,0.3533999962717293552572073167539201676845550537109375f,0.35469998321235163274423030088655650615692138671875f,1.1108000005602836068874239572323858737945556640625f,0.057699976521730367284845897302147932350635528564453125f,1.18490000281706375773183026467449963092803955078125f,-0.94540000799149226384798794242669828236103057861328125f,-0.744099971038848284621280981809832155704498291015625f,-0.067500020563602713963291535037569701671600341796875f,0.222700029886514438981492958191665820777416229248046875f,0.2225999615132803199912814307026565074920654296875f,-1.12380004429146662658922650734893977642059326171875f,0.20250004887580974610727935214526951313018798828125f,0.78620004066377935370013574356562457978725433349609375f,-0.9638000618934636776202751207165420055389404296875f,-1.532900054650754295693104722886346280574798583984375f,0.030599994683265574479946735664270818233489990234375f,-0.607199976991117384983454030589200556278228759765625f,-0.9349999598041180970398045246838591992855072021484375f,-0.19400001874566097370689021772705018520355224609375f,-2.53189993855953243695466881035827100276947021484375f,-1.199600001582502795116624838556163012981414794921875f,-1.05230000857710859207827525096945464611053466796875f,0.636000030353665035676158368005417287349700927734375f,-0.132099922680854575673947692848742008209228515625f,-1.781699994596838809712835427490063011646270751953125f,-1.432100050234794252901338040828704833984375f,1.162799977672099860370735768810845911502838134765625f,1.1728000075861813034094893737346865236759185791015625f,1.069199986177682859533888404257595539093017578125f,-0.3871000028118485314365671001723967492580413818359375f,0.9604999719262130586372450125054456293582916259765625f,0.37779999242424988636912530637346208095550537109375f,-1.086000016570090931367076336755417287349700927734375f,-0.87150007075071389550657841027714312076568603515625f,0.70089997608065512935127117088995873928070068359375f,0.608900035178661536150457322946749627590179443359375f,-0.0690999251246449119889803114347159862518310546875f,-0.46749998509883905484230126603506505489349365234375f,-1.061600026750564751409910968504846096038818359375f,1.2805000507533552678296473459340631961822509765625f,-0.22999998895451334579576041505788452923297882080078125f,0.390199993509053655316165531985461711883544921875f,0.115999960672110546955337895269622094929218292236328125f,-0.44369998966455437283684659632854163646697998046875f,0.5939000370226807934415091949631460011005401611328125f,-0.673800014805794322825249764719046652317047119140625f,-0.5248999986074858181694935410632751882076263427734375f,0.54790001175403568733912607058300636708736419677734375f,0.316399988612532434562041316894465126097202301025390625f,-1.2235000115782022067634216000442393124103546142578125f,-0.5895000131167476009608208187273703515529632568359375f,-0.34010003199577365506911519332788884639739990234375f,-0.31700003701448498105719409068115055561065673828125f,-0.294799983394145925785778672434389591217041015625f,-0.2239000126779071564442347153089940547943115234375f,0.07999999016523329231631578295491635799407958984375f,-0.588700032579900334184230814571492373943328857421875f,-0.129599980050324692371077617281116545200347900390625f,-0.49520000915527351281752999057061970233917236328125f,0.93850001418590611734771300689317286014556884765625f,0.5097999883115296171354202670045197010040283203125f,-1.045700005221366524210679926909506320953369140625f,-0.1945000505745415608771509141661226749420166015625f,-0.332600012773275555133523084805347025394439697265625f,-0.92029998527169232236389007084653712809085845947265625f,-0.75520001142770032398487956015742383897304534912109375f,-0.20460003745704813393757603989797644317150115966796875f,0.6520999951377508541128236174699850380420684814453125f,0.5016000017911188546548828526283614337444305419921875f,-1.591200013208389663077468867413699626922607421875f,-0.7006999923691152876159549123258329927921295166015625f,0.855399989104271174511495701153762638568878173828125f,0.47980004091262795640204785740934312343597412109375f,-0.03759996631890472063020069981575943529605865478515625f,1.1681999966904523802213589078746736049652099609375f,0.6498000035293396425828404971980489790439605712890625f,-0.269900003729760751269850516109727323055267333984375f,0.28069999935403444624171243049204349517822265625f,0.221400006121397208147527635446749627590179443359375f,0.05560001681149040475560241247876547276973724365234375f,-1.319899981713295034069233224727213382720947265625f,0.55839998453706451986278125332319177687168121337890625f,0.391199928569793797805687063373625278472900390625f,-1.348500020779669572590364623465575277805328369140625f,-1.668599993543326664990900098928250372409820556640625f,-0.24260000929012848214227915377705357968807220458984375f,0.6391000155374422231346898115589283406734466552734375f,0.44089998323321299977806120296008884906768798828125f,-0.21859999339431457343607689836062490940093994140625f,0.162899986124038331780639055068604648113250732421875f,0.210900065003336134594746908987872302532196044921875f,0.54689999380558618202741172353853471577167510986328125f,-1.2128000521779060960625429288484156131744384765625f,-0.5303000161170956516798469237983226776123046875f,0.26979997311234438228666476788930594921112060546875f,0.76840001288652448607763290056027472019195556640625f,0.087000019691885199790704064071178436279296875f,-0.460799996909499132158316569984890520572662353515625f,0.70780000181198088338163643129519186913967132568359375f,0.23230005104541806559836913947947323322296142578125f,-1.433499997049569429918847163207828998565673828125f,-0.797900034441053396250254081678576767444610595703125f,-0.27749997869133979921940635904320515692234039306640625f,0.921199998679757214858909719623625278472900390625f,-0.0411999975249168759461326771997846662998199462890625f,0.38240001278221613656427280147909186780452728271484375f,-1.05149998875707328949147267849184572696685791015625f,-0.2808000174358487033288156453636474907398223876953125f,-0.29640007403343948677587604834116064012050628662109375f,-0.23250001370906847597552769002504646778106689453125f,1.34899999482929633387584544834680855274200439453125f,1.450800010152905716864779606112278997898101806640625f,-0.200600018934905310885596918524242937564849853515625f,-0.988300005943327786184227079502306878566741943359375f,-0.1496000450566408179753352669649757444858551025390625f,-0.37930001306980865027895788443856872618198394775390625f,-0.4420999895870689755383864394389092922210693359375f,0.22079999100491409791402475093491375446319580078125f,0.681800006699562199941055951057933270931243896484375f,0.67550003020837923184416240474092774093151092529296875f,-0.7668999930977822288014067453332245349884033203125f,-1.3286000432386992908817546776845119893550872802734375f,0.93700005280971510046583716757595539093017578125f,0.095599963986874314514352590776979923248291015625f,-1.365700030292570144041519597521983087062835693359375f,-0.214299965050817053935361400363035500049591064453125f,1.238699995028972100641340148285962641239166259765625f,0.35639997695237457531192148962873034179210662841796875f,0.3591000350579618416446692208410240709781646728515625f,-0.4968999929487705191633040158194489777088165283203125f,1.075300028634071480837519629858434200286865234375f,-0.4388000465825203644243401868152432143688201904296875f,-2.20810003283023892350911410176195204257965087890625f,-0.59199999201297703876889499952085316181182861328125f,1.8835000824332244206971154198981821537017822265625f,-0.02380000132024229220206734680687077343463897705078125f,-1.1740000279471274335918451470206491649150848388671875f,-0.51060004802942327017944990075193345546722412109375f,0.7202999891087416006740795637597329914569854736328125f,0.587700009006261847588348246063105762004852294921875f,1.45820000425279072686635117861442267894744873046875f,0.02640003771185917003094800747931003570556640625f,1.8234000458180907866534425920690409839153289794921875f,-0.19380000795125962298470767564140260219573974609375f,0.134100036324560800693461715127341449260711669921875f,-0.97790001652240743368338371510617434978485107421875f,-1.43490001252144594445780967362225055694580078125f,0.473200029864162541315408816444687545299530029296875f,-0.307499986290931559551609097979962825775146484375f,-0.60040001665279252307527713128365576267242431640625f,-0.1019000164926051610336799058131873607635498046875f,-0.3321000007070600279490690809325315058231353759765625f,-0.48239997962713232482201419770717620849609375f,-0.1978000123545526900414870397071354091167449951171875f,-0.08509999414682312135482789017260074615478515625f,0.518600026325881646727111728978343307971954345703125f,0.2719000163435936201494769193232059478759765625f,0.128100011298060645348328989712172187864780426025390625f,0.685999982595443924537903512828052043914794921875f,-0.37560003920048423697863881898229010403156280517578125f,-1.149700039017201191882122657261788845062255859375f,-0.8180999499797820817548199556767940521240234375f,-0.14649999520182621637331976671703159809112548828125f,-0.076399994051456587840220890939235687255859375f,-1.0169000158965584290626793517731130123138427734375f,0.449000018745661311214689703774638473987579345703125f,-0.3686000443935402159922887221910059452056884765625f,0.2155000121593477757642176584340631961822509765625f,-0.2289000545501718164587146020494401454925537109375f,-0.62919999198913512117314894567243754863739013671875f,0.6048000096201899822290215524844825267791748046875f,-0.352699983748793410853750174283050000667572021484375f,-0.59509999012351055824865397880785167217254638671875f,-0.4827000365734104292414485826157033443450927734375f,-1.609200002382695782188193334150128066539764404296875f,-0.0763000120490786581939346433500759303569793701171875f,0.01540001905560550721219215120072476565837860107421875f,0.5420999801993369171526637728675268590450286865234375f,0.4325999818533661223085573510616086423397064208984375f,0.0379000051230191647988476688624359667301177978515625f,0.4443000119149684845609726835391484200954437255859375f,0.3033000007838009270955126339686103165149688720703125f,-1.0421999823182821476308390629128552973270416259765625f,0.438400034530461446546922843481297604739665985107421875f,1.527400013899804065431453636847436428070068359375f,0.4641000391930341084645306182210333645343780517578125f,1.29980000358521952108503683120943605899810791015625f,-0.95419999584481107746114503243006765842437744140625f,0.352899974128604743128789777983911335468292236328125f,-0.154399977246671815667156124618486501276493072509765625f,0.20950001548230634540459504933096468448638916015625f,-0.37400000032410007921868100311257876455783843994140625f,1.04859998270273191423029857105575501918792724609375f,0.39870001196041660040947363086161203682422637939453125f,0.54530000479221296671994423377327620983123779296875f,-1.006799975928664547808466522837989032268524169921875f,0.43149992173910067805309154209680855274200439453125f,-1.4596999847769733804625502671115100383758544921875f,-0.18210004181563821390454904758371412754058837890625f,-0.67609992801547047491794728557579219341278076171875f,0.4214999674856658717914115186431445181369781494140625f,-0.9625999718695881224306276635616086423397064208984375f,-0.73700001890957356209810313885100185871124267578125f,-0.638399946576357013583447042037732899188995361328125f,0.71909997876882503220485887140966951847076416015625f,0.0585000009983780433486799665843136608600616455078125f,0.1292000146389014059167266168515197932720184326171875f,0.087100006127357243457254298846237361431121826171875f,2.116399995541572653934281333931721746921539306640625f,1.26690007728934350694771637790836393833160400390625f,1.1199000362515458828482906028511933982372283935546875f,-0.8070000104904178073184084496460855007171630859375f,-1.7953000170484190167741189725347794592380523681640625f,0.29209998846948204143103566821082495152950286865234375f,0.50490001524090732942795511917211115360260009765625f,0.3675000320002437259603311758837662637233734130859375f,-1.085500027507543041593862653826363384723663330078125f,-0.485600062781573083015018710284493863582611083984375f,0.127999974776058678127554912862251512706279754638671875f,-1.1082000582508750863297564137610606849193572998046875f,-0.324700028586388356188763282261788845062255859375f,-0.467800011665374437797026985208503901958465576171875f,0.52969999401569367591946502216160297393798828125f,1.1741000509090719905458399807685054838657379150390625f,1.20519996027946518779572215862572193145751953125f,-1.5337000073969362379244785188348032534122467041015625f,-0.306999939262866394784623480518348515033721923828125f,0.86130002808645433898249166304594837129116058349609375f,0.470499997407198833343500155024230480194091796875f,-0.55769998125284880874374948689364828169345855712890625f,0.822199988427758210463025534409098327159881591796875f,-0.70229996104240388632433678139932453632354736328125f,-0.298799973082541026059288924443535506725311279296875f,-0.2087999380648142278005252592265605926513671875f,0.6173999790310862323394758277572691440582275390625f,-0.71009999504089460486966345342807471752166748046875f,-0.2337000296369191598699899259372614324092864990234375f,0.5348999940812591002980980192660354077816009521484375f,1.194800039571524141734926161007024347782135009765625f,-1.1463000060513615085966421247576363384723663330078125f,-0.58290006286501938603805683669634163379669189453125f,-1.18909995790720035557797018554992973804473876953125f,0.267899992494285310584700710023753345012664794921875f,-0.1298999891638761283729763817973434925079345703125f,-0.5134000004068017819491842601564712822437286376953125f,0.42319998428523508682275178216514177620410919189453125f,-1.1601999550640587077765530921169556677341461181640625f,-1.4136999996110788391234791561146266758441925048828125f,-0.641700003239511485020329928374849259853363037109375f,0.9095999896056949918232703566900454461574554443359375f,-1.6657000471681360220799206217634491622447967529296875f,-0.38860003101974793793260687380097806453704833984375f,-0.1083000049933791064660226766136474907398223876953125f,0.34089999098181689163311602897010743618011474609375f,0.0785000171288847692352419471717439591884613037109375f,0.610500026110559534942012760438956320285797119140625f,1.6546999661505221723700742586515843868255615234375f,0.5370000170469275690976473924820311367511749267578125f,-0.027299985331297182966636682976968586444854736328125f,0.10439999868571747665413340655504725873470306396484375f,1.4027000087641188219578225471195764839649200439453125f,0.14730005462169726282439796705148182809352874755859375f,0.57100001860037441492323750935611315071582794189453125f,-0.38230000975057476519936017211875878274440765380859375f,-0.2194000199198722356186408433131873607635498046875f,0.47969999848604150116671007708646357059478759765625f,0.5751000372298065155263202541391365230083465576171875f,0.00550001811981104538062936626374721527099609375f,1.1069999781921502357562303586746565997600555419921875f,-0.0825000114738940748537743274937383830547332763671875f,-0.81110003083348214403258680249564349651336669921875f,-0.6987999840721486055628020039875991642475128173828125f,1.2286000324912371706176372754271142184734344482421875f,-1.0118999888882049820182373878196813166141510009765625f,-0.0528000007688997907706607293221168220043182373046875f,0.59080000910982466422183279064483940601348876953125f,0.50060000973343843355678473017178475856781005859375f,-0.205100016051531053307144247810356318950653076171875f,-0.0378999626733353967011908025597222149372100830078125f,-0.146000029738992787020634978034649975597858428955078125f,-0.222900051558763578896105173043906688690185546875f,0.42320003016218554758864911491400562226772308349609375f,-0.255000010542571098337560897562070749700069427490234375f,-0.353700030344725302455799464951269328594207763671875f,-0.381699982005358595671395960380323231220245361328125f,0.73630004311800067995363860973156988620758056640625f,0.6159999910891056362771678323042578995227813720703125f,1.236500019639730663101317986729554831981658935546875f,0.6560000397600240784612424249644391238689422607421875f,-0.5220000189468265716641326434910297393798828125f,-0.34639998237267211056433779958751983940601348876953125f,-0.8359000109493734953503008000552654266357421875f,0.73119997193217312059232426690869033336639404296875f,-1.57259994825124760353673991630785167217254638671875f,0.087000007212161722947030284558422863483428955078125f,0.954799997699260938333054582471959292888641357421875f,-0.014499999612569869356093477108515799045562744140625f,-0.506499983578920787152810589759610593318939208984375f,-0.43879996756911321398320069420151412487030029296875f,-0.4045999675333502576535238404176197946071624755859375f,0.47459999361038196497020180686376988887786865234375f,-0.268999992966652001769034541212022304534912109375f,0.807399988567829129948449917719699442386627197265625f,0.47890003577470796614079517894424498081207275390625f,-0.5969000142946843112667920649982988834381103515625f,1.215599992001056950385873278719373047351837158203125f,-0.7095000043064352279742479367996565997600555419921875f,1.393199967372417358291158961947076022624969482421875f,0.2884999969005590525483739838819019496440887451171875f,0.0692000180661676012050520512275397777557373046875f,-0.07580001840442462945901525017688982188701629638671875f,-1.5143000068485736875345537555404007434844970703125f,-0.06709998522847848700934036969556473195552825927734375f,0.9578000172719358662476452082046307623386383056640625f,-0.40150004810094908069828534280532039701938629150390625f,0.95509998889416503065064034672104753553867340087890625f,-0.21790001473426823253021211712621152400970458984375f,0.156300009106100201705658037099055945873260498046875f,-1.1553999861240384916527546010911464691162109375f,-1.90429999228268886923842728720046579837799072265625f,-1.10429999492764441271219766349531710147857666015625f,-1.417399934811890194197303571854718029499053955078125f,-0.3039000028431413369389701983891427516937255859375f,0.0181999605923890417358279592008329927921295166015625f,-0.11459998280704020157116929112817160785198211669921875f,-0.1139999944567680056906056051957421004772186279296875f,-1.1096000517994164535906520541175268590450286865234375f,-0.0103999578118327828946121371700428426265716552734375f,-0.06250000134110411664778439444489777088165283203125f,0.1985999794244766025741455450770445168018341064453125f,1.187099985861778339568672890891321003437042236328125f,1.4763999529242521902006046730093657970428466796875f,-0.073400027042627158380128093995153903961181640625f,1.688499969482422091715534406830556690692901611328125f,-0.00429998598694769640360391349531710147857666015625f,-1.281299999904632613834110088646411895751953125f,0.521999999202787723362462202203460037708282470703125f,0.7337000110104678896050245384685695171356201171875f,-1.278299976606666721323790625319816172122955322265625f,-0.34630002762079215017365640960633754730224609375f,-0.8456000187486407693171486243954859673976898193359375f,0.4266000306248667328645751695148646831512451171875f,0.86380001447051757423878370900638401508331298828125f,0.24149999445676773035529549815692007541656494140625f,0.5680000170394781111582460653153248131275177001953125f,-1.0267000134468080574379200697876513004302978515625f,0.266900003118812845226415220167837105691432952880859375f,-0.247699991460144375299279317914624698460102081298828125f,-0.822300004106760074051862829946912825107574462890625f,0.64199998430162708018542616628110408782958984375f,-1.0117000211581583801745409800787456333637237548828125f,-0.0561999946191897858938091303571127355098724365234375f,-2.1875999568939210604412437533028423786163330078125f,-0.879299931523203071037642075680196285247802734375f,1.0183999914288524024641446885652840137481689453125f,0.658899999862908369863134794286452233791351318359375f,1.37459999375939379007149909739382565021514892578125f,0.3902999746173616557598506915383040904998779296875f,0.74659998674094663062561494371038861572742462158203125f,0.5241999879658225669487592313089407980442047119140625f,-0.7370999735683205766889614096726290881633758544921875f,-0.4854999893233171714967966181575320661067962646484375f,-2.75269994090795488972389648552052676677703857421875f,-0.21720000802278516260912510915659368038177490234375f,-0.25139999986290906264230216038413345813751220703125f,0.3529999854862690966683658189140260219573974609375f,-0.017400001978874257702045724727213382720947265625f,0.63279999510645890126170343137346208095550537109375f,0.129599995845556037465939880348742008209228515625f,-0.7179999880194662953414308503852225840091705322265625f,-0.18210000519603486424102811724878847599029541015625f,-0.2885999883279206645880776704871095716953277587890625f,0.28039999323934328145924155251123011112213134765625f,-0.092700021336972582286506394666503183543682098388671875f,-0.3489000088036053881523912423290312290191650390625f,0.54310000820606962879111279107746668159961700439453125f,-0.0904000010997055569728075852253823541104793548583984375f,-0.64840000966414823579953008447773754596710205078125f,0.7353000189855694035401256769546307623386383056640625f,0.2707999710559843009605174302123486995697021484375f,0.4372000012427570414530464404379017651081085205078125f,-1.542299977582693326638718644971959292888641357421875f,0.411800009717046666679607369587756693363189697265625f,-1.292799993616342124624907228280790150165557861328125f,0.33610000833272912768023843455011956393718719482421875f,-0.3131000393867491293775628946605138480663299560546875f,0.16520002129972033433347178288386203348636627197265625f,-1.24730002066567546847863923176191747188568115234375f,-0.42890000118538751383567841912736184895038604736328125f,0.376600004734097992820096578725497238337993621826171875f,1.786399961790442691977887079701758921146392822265625f,0.5601999679908151730245435828692279756069183349609375f,0.29709998787343483428458057460375130176544189453125f,-0.83070007041394777758114287280477583408355712890625f,0.2675999748870723227156531720538623631000518798828125f,1.126899954354763533359573557390831410884857177734375f,0.34629998086839908211231886525638401508331298828125f,-0.631600028277933489562201430089771747589111328125f,-0.500899950897693457818604656495153903961181640625f,1.189099999611824554079930749139748513698577880859375f,-1.09659996079430044346736394800245761871337890625f,-0.52319999715611320567631992162205278873443603515625f,0.4325999811828136198954553037765435874462127685546875f,0.259900005927681920780969448969699442386627197265625f,0.52180001407563614268525498118833638727664947509765625f,-0.362799945709109561420291356625966727733612060546875f,-0.74179998978674455134552090385113842785358428955078125f,-1.1093999432906513913366097767720930278301239013671875f,0.7892999653860928166437815889366902410984039306640625f,0.2443999984249474177744332337169907987117767333984375f,0.326900001293420672254796954803168773651123046875f,-1.36240001430958468375820302753709256649017333984375f,0.298100007647275955235954825184307992458343505859375f,-0.9119000460155313181331848682020790874958038330078125f,0.431500000603497024798116399324499070644378662109375f,-1.045199998128413909626033273525536060333251953125f,0.736800017651915606364809718797914683818817138671875f,-0.1848999864816665006372886637109331786632537841796875f,-0.20859998735934459634933091365383006632328033447265625f,-0.43260000770688034066324689774774014949798583984375f,-0.0726999966010450915376850389293394982814788818359375f,-0.07800004151463557633405798696912825107574462890625f,-0.777900032317638601142562038148753345012664794921875f,0.503199940717965699121805300819687545299530029296875f,-0.03380001042857727622248376064817421138286590576171875f,-1.16200008171796920208862502477131783962249755859375f,-1.05150003364682209650027289171703159809112548828125f,-1.1657999983623614781436117482371628284454345703125f,1.150800002944469380139480563229881227016448974609375f,1.035200025472045037844281978323124349117279052734375f,0.0426000449597829788928038396988995373249053955078125f,-0.93330001084208469297465171621297486126422882080078125f,-0.68199997502565334883684045053087174892425537109375f,-1.0659000428751121969384030307992361485958099365234375f,-1.97650002887845044341474931570701301097869873046875f,-1.1635999581716955919574729705345816910266876220703125f,1.3352999988131222242060403004870750010013580322265625f,0.101000001277774753560834142263047397136688232421875f,0.206699961441755508673168151290155947208404541015625f,0.71720001174807546107103917165659368038177490234375f,-0.588100009584426874909013349679298698902130126953125f,0.068699997711181737258812063373625278472900390625f,0.66689998758435198311644853674806654453277587890625f,-0.2286999927937982679537753938348032534122467041015625f,1.037000002443790425132874588598497211933135986328125f,0.54809996815919870272182379267178475856781005859375f,-0.96369999309182130531326038180850446224212646484375f,-0.5066000255584715006307305884547531604766845703125f,-0.674999995902179961149158771149814128875732421875f,0.906799994629621242125949720502831041812896728515625f,-0.4209999938458202084490267225191928446292877197265625f,0.0213000055670737875601616906351409852504730224609375f,-0.481699997986853312159638562661712057888507843017578125f,-0.006700020450353338929971869220025837421417236328125f,-0.3896000077292323116040506647550500929355621337890625f,0.23529999823570246775261693983338773250579833984375f,-0.664699990886449487703657723614014685153961181640625f,0.276600025111437108904510751017369329929351806640625f,-0.092199991519749335822098146309144794940948486328125f,-0.0040999882683154798002078678109683096408843994140625f,-1.4149999500438570354532430428662337362766265869140625f,-0.56169994624257146842438714884337969124317169189453125f,-0.77089996106773650552668186719529330730438232421875f,0.323700003224610899366098237805999815464019775390625f,-0.84689997342824963055818443535827100276947021484375f,-0.1767000089392058992388001570361666381359100341796875f,-1.12519997309446306843483398552052676677703857421875f,0.26640004099011438398747486644424498081207275390625f,-1.03439993766546312059517731540836393833160400390625f,0.1445000231936577339553195997723378241062164306640625f,0.2160000155270100918869502493180334568023681640625f,-0.6280000255703921396843725233338773250579833984375f,-0.14069997847378290511954901376157067716121673583984375f,0.5778999738678340936104405045625753700733184814453125f,-0.337399968115985993133421061429544351994991302490234375f,-0.52479997903555641869388637132942676544189453125f,0.4667999886691571731489602825604379177093505859375f,-0.48649999490380313460491379373706877231597900390625f,-0.3169999945461745571861911230371333658695220947265625f,-0.1614000232577323945548641859204508364200592041015625f,-0.064299979132414364357828162610530853271484375f,-0.5387999723374843341616724501363933086395263671875f,0.2191999997377396258713133647688664495944976806640625f,0.86470001279115660253182795713655650615692138671875f,-0.45990001699179428040764605611911974847316741943359375f,0.216900014592707190974607556199771352112293243408203125f,0.39910000339299378513402416501776315271854400634765625f,0.27870000959485796698800186277367174625396728515625f,0.05089999917745602342478150603710673749446868896484375f,0.1196000134289267879950102724251337349414825439453125f,0.14700000546127578449073780575417913496494293212890625f,-0.461600013935566266809473745524883270263671875f,0.5620000005066392301245059570646844804286956787109375f,-1.347099994876980399993726678076200187206268310546875f,0.216000023126602436196463941087131388485431671142578125f,-0.49049999915808495387636867235414683818817138671875f,-1.38980000731050967743840374168939888477325439453125f,-1.0519000227510926226415222117793746292591094970703125f,0.023899967154860146667516573870670981705188751220703125f,-0.71780006823390696268916144617833197116851806640625f,-0.515900006330013649602506120572797954082489013671875f,0.00830006921738457581483316971571184694766998291015625f,0.6916000333070757566389374915161170065402984619140625f,-0.6883000383347279704793209020863287150859832763671875f,-0.3162000239372255716574500183924101293087005615234375f,0.1650000076740985033296738038188777863979339599609375f,0.87750003501772899650035242302692495286464691162109375f,0.5556000195682049280776482191868126392364501953125f,0.547499998472630977630615234375f,0.4026000271901482019387685795663855969905853271484375f,1.0205999795213340775035248952917754650115966796875f,1.0417000344201918782260918305837549269199371337890625f,-1.14240001598596574439170581172220408916473388671875f,0.601299985301494643863406963646411895751953125f,1.116299993944168011950068830628879368305206298828125f,-0.2103000064313411687777488623396493494510650634765625f,-0.66870001522004596239412421709857881069183349609375f,-0.2344999849349260756525836768560111522674560546875f,1.0816999853581190649265408865176141262054443359375f,-0.3877999920517203324976662770495750010013580322265625f,-0.6701000385522843938446158063015900552272796630859375f,-0.28649998644739393949265604533138684928417205810546875f,-0.4462000045657157354384025893523357808589935302734375f,0.44060001301169382514899552916176617145538330078125f,-0.041100007624924383975439923233352601528167724609375f,0.38549999472498885921822875388897955417633056640625f,-0.10209999685138460012723271574941463768482208251953125f,0.12500000700354585436713250601314939558506011962890625f,0.232599996680021126049808799507445655763149261474609375f,0.59700000583380441465664034694782458245754241943359375f,-0.07860001619309198162000029697082936763763427734375f,0.10850000539422033352110474879737012088298797607421875f,-0.4952000292718412577386288830894045531749725341796875f,-0.5003000033393505407985912825097329914569854736328125f,-0.167000019282102840634252061136066913604736328125f,-0.58970000411123013606129461550153791904449462890625f,-0.5910000099390735339710545304114930331707000732421875f,-0.62320000548213716484724500332958996295928955078125f,-0.14419993536472353046207217630580998957157135009765625f,-0.1453999902218577611190397647078498266637325286865234375f,1.46009999787211341981674195267260074615478515625f,0.30879999062865959302826013299636542797088623046875f,1.9032999329090127815788946463726460933685302734375f,0.770900008360296595810723374597728252410888671875f,0.0291000114023685085840043029747903347015380859375f,-0.67889997795820189452342674485407769680023193359375f,-0.232199944096803445603427462629042565822601318359375f,-0.6829999871253971122087023104541003704071044921875f,-1.92140002668499931814949377439916133880615234375f,0.5251999590873719370165417785756289958953857421875f,0.76139999211430531289579448639415204524993896484375f,0.49810000422000921815879337373189628124237060546875f,-0.043899994051456392440968556911684572696685791015625f,0.253499966204166593541913243825547397136688232421875f,1.2108999988794326707619575245189480483531951904296875f,0.6900999913342296121498975480790250003337860107421875f,1.4371000020295383592383586801588535308837890625f,0.0798999519109724332110999966971576213836669921875f,1.0353999968901284045585953208501450717449188232421875f,1.4037999815762045585643136291764676570892333984375f,-0.8195000198781492439792373261298052966594696044921875f,0.6765999564170837032861527404747903347015380859375f,0.41000002026557902379266806747182272374629974365234375f,0.15250000849366163180320654646493494510650634765625f,-1.5189999779537315394151164582581259310245513916015625f,-0.8660999990940094761526779620908200740814208984375f,-0.4054999985992910893628504709340631961822509765625f,0.6491000125944614840278745759860612452030181884765625f,-0.465700023251771622057049171417020261287689208984375f,-0.18669999888092292383845460790325887501239776611328125f,0.3622999921858305327759808278642594814300537109375f,-0.41930000226646679806208339869044721126556396484375f,-0.13679999403357445686424398445524275302886962890625f,0.2774999864026905793679134148987941443920135498046875f,0.20919997828006753337604095577262341976165771484375f,-0.1379000061661004306046152123599313199520111083984375f,0.27250000357627879310484786401502788066864013671875f,-0.166300026093423536366344706038944423198699951171875f,0.43729998399019276433818959048949182033538818359375f,-1.5120999938338997026221477426588535308837890625f,-0.957699997048079953998467317433096468448638916015625f,0.5281999984785910040585577007732354104518890380859375f,0.0460999825723467093752105938619934022426605224609375f,0.482900008364021748885619444990879856050014495849609375f,-1.14619999903365954452283403952606022357940673828125f,-0.1327999800749124492682540221721865236759185791015625f,1.2829000601828097760659375126124359667301177978515625f,-0.371399988538026804008040926419198513031005859375f,-1.85830001710057235442263845470733940601348876953125f,-0.0405000494122500498406225233338773250579833984375f,0.7664999858140946198403753442107699811458587646484375f,0.10790000095069418595272736638435162603855133056640625f,0.0109999736919998980511081754229962825775146484375f,2.10800000560283695705265927244909107685089111328125f,0.238599991606175565461711585157900117337703704833984375f,0.986900001032650298071757788420654833316802978515625f,-1.003100007293373341354936201241798698902130126953125f,-0.721599980165809284926581312902271747589111328125f,1.1600000086426742740286499611102044582366943359375f,1.24359998836517338105522867408581078052520751953125f,-0.0590000137910251520878546216408722102642059326171875f,1.11049993854761108735829111537896096706390380859375f,1.064500033140182377877636099583469331264495849609375f,-0.34899999386072166895900181771139614284038543701171875f,1.0941000152952968793584886952885426580905914306640625f,0.23610001079142095559149083783267997205257415771484375f,0.1379000016398730377176207184675149619579315185546875f,-0.189699987012147908149728436910663731396198272705078125f,-0.1782999473631388109851059198263101279735565185546875f,0.025300011292844981625194122898392379283905029296875f,1.056499936938285255649816463119350373744964599609375f,0.12629999774396394318642933285445906221866607666015625f,-0.021400006270408056252563255839049816131591796875f,0.22809999729096919196535964147187769412994384765625f,-0.778300033938884805451152715249918401241302490234375f,-0.36139999017715496165692456997931003570556640625f,-0.3803999561727042788561448105610907077789306640625f,-0.8374999961256985869795244070701301097869873046875f,-0.206700024026632345197640461265109479427337646484375f,-0.179300048422813773640882573090493679046630859375f,-0.1148999788820741496664368241908960044384002685546875f,-0.55310001003146158637946427916176617145538330078125f,0.7741999752998351969068835387588478624820709228515625f,0.733599966907501510604561190120875835418701171875f,-0.65650000339746472644009145369636826217174530029296875f,-0.583499973729252996434979650075547397136688232421875f,-0.0153000082194802988766468843095935881137847900390625f,0.6983999352514747016584806260652840137481689453125f,0.334300020930171637445482701878063380718231201171875f,-0.635100013071298263866992783732712268829345703125f,-1.4876000142633916123457993307965807616710662841796875f,-0.9981000007927416373121332071605138480663299560546875f,-0.17599998687952733700257113014231435954570770263671875f,-0.62689999160766607388950433232821524143218994140625f,0.32350000435113879238002709826105274260044097900390625f,0.959399988180398732851017484790645539760589599609375f,0.305799989570677277495036605614586733281612396240234375f,-0.4682999930351972039943575509823858737945556640625f,-0.2354999910742046065337262916727922856807708740234375f,-0.49239999400675316110209678299725055694580078125f,-0.953099955344200555629186055739410221576690673828125f,-1.28100001768767857157627076958306133747100830078125f,-0.205799985510110250430670930654741823673248291015625f,-0.196000013813376394722354234545491635799407958984375f,0.221600001418590153434706735424697399139404296875f,0.11309998950511224169446222731494344770908355712890625f,0.4285999509632596460306785957072861492633819580078125f,0.362099988767504765707627711890381760895252227783203125f,-0.621899969777464978193393108085729181766510009765625f,-0.274499996744096252943023728221305646002292633056640625f,0.50300001901388202707465779894846491515636444091796875f,-1.042099978411197724881276371888816356658935546875f,0.05270000677108743758481068653054535388946533203125f,-0.0286999957740305955411486138473264873027801513671875f,-0.83469997311681465301802518297336064279079437255859375f,1.4610999781951303777560724483919329941272735595703125f,1.6424999945424494374179857913986779749393463134765625f,0.6116999825455253780859266043989919126033782958984375f,-0.1013000112295150412222710656351409852504730224609375f,-0.763000006794928964382052072323858737945556640625f,-0.320699999186396578210178631707094609737396240234375f,-0.58539998493194556061780531308613717555999755859375f,-0.9339000206500294165579134642030112445354461669921875f,-1.616099979126453423106113405083306133747100830078125f,-0.0527000208526846591894354787655174732208251953125f,0.8956999870419501963425545909558422863483428955078125f,-0.7035999885141845933134163715294562280178070068359375f,0.652899987427890682312181525048799812793731689453125f,0.1015000207386911323936828921432606875896453857421875f,-0.253900008971244428579439045279286801815032958984375f,0.50450000598281607988582209145533852279186248779296875f,-1.19080004422068608249674070975743234157562255859375f,0.82629999576955992512949933370691724121570587158203125f,0.47820001103281961629676288794144056737422943115234375f,-0.364099993073940364496365873492322862148284912109375f,-0.4313999921515583313436081880354322493076324462890625f,1.07449997241795092151050994289107620716094970703125f,0.5901000045031306040499430309864692389965057373046875f,0.3692000158496193673585139549686573445796966552734375f,0.4334999876432121812541709005017764866352081298828125f,0.1121999195098879464893570911954157054424285888671875f,0.3955999843083322087267106326180510222911834716796875f,-1.1231000184506172434595328013529069721698760986328125f,-0.17370000959485765168466286922921426594257354736328125f,0.157600004205107335764068920980207622051239013671875f,0.155500019833445801697990873435628600418567657470703125f,0.32439998568445427284956394942128099501132965087890625f,-0.2547999952778219334703635468031279742717742919921875f,0.2081000151909886408674310587230138480663299560546875f,-0.009499964520335080209179068333469331264495849609375f,-0.51519999306201924582637730054557323455810546875f,-1.12719995259046523727874955511651933193206787109375f,-0.249299992841482076499914910527877509593963623046875f,-0.0068999653816224082447661203332245349884033203125f,0.30290000028014174571211469810805283486843109130859375f,-1.974900014868378406163174076937139034271240234375f,-0.04530000598430650171621891786344349384307861328125f,0.646099996188282954534543023328296840190887451171875f,0.861200013804435560160754903336055576801300048828125f,0.4567999977588652615878572760266251862049102783203125f,1.71539998645931479615001080674119293689727783203125f,0.81409994173944000284137700873543508350849151611328125f,1.91240000261217346633202396333217620849609375f,-0.4044000482320786460377348703332245349884033203125f,0.2351000149711970355070889127091504633426666259765625f,-1.24990001598596567333743223571218550205230712890625f,0.6353000008061526759917114759446121752262115478515625f,-1.7463000332087279531378953834064304828643798828125f,-0.96669999504387360556023622848442755639553070068359375f,-0.301700004729628545874220435507595539093017578125f,-0.16879999487549046666146068673697300255298614501953125f,0.44760000603049976408698285013088025152683258056640625f,-0.953800049935281446522594706038944423198699951171875f,0.073400015326589296282833174700499512255191802978515625f,-2.00269999842643731113867033855058252811431884765625f,-0.8026999937325716150127163928118534386157989501953125f,-2.0412999918580059244277435936965048313140869140625f,-0.7092000127762556616062283865176141262054443359375f,-0.9162999996811145297925804698024876415729522705078125f,-0.9002000097513198628718100735568441450595855712890625f,-0.29820000045299532853704249646398238837718963623046875f,-0.2077000066965819791153791129545425064861774444580078125f,-0.6400999950408934324741494492627680301666259765625f,-0.013900003010780093148923697299323976039886474609375f,0.6543000316962601647929886894416995346546173095703125f,-0.21800000796839602568155669359839521348476409912109375f,-0.2660000253617766929892241023480892181396484375f,-1.522300020051002178433918743394315242767333984375f,-1.060099998430907408675238912110216915607452392578125f,0.1191999888971448928742802308988757431507110595703125f,0.2125999926753341373597550045815296471118927001953125f,-0.859199984166025831910928900470025837421417236328125f,-0.338300021123886462959262644289992749691009521484375f,0.321500040054321090110533987171947956085205078125f,-0.58379997323155397026539503713138401508331298828125f,0.3618999830581250964911532719270326197147369384765625f,0.35610002621412306655912516362150199711322784423828125f,0.27800001389160755582707906796713359653949737548828125f,0.3430999822035427637700877312454394996166229248046875f,-0.09130002911090873141120027867145836353302001953125f,-0.91049997639656099490679252994596026837825775146484375f,0.01100003296136853325748461429611779749393463134765625f,0.3876999961167573172104994227993302047252655029296875f,0.32460003704726669848668052509310655295848846435546875f,0.925499966077506286410425673238933086395263671875f,1.1010999798715115360892013995908200740814208984375f,0.114300025475025268661966038052923977375030517578125f,-0.194599999552220126464163740820367820560932159423828125f,0.0505999893188471550065088422343251295387744903564453125f,-0.522900009425729450640574214048683643341064453125f,-0.240900000965594163204741562367416918277740478515625f,-0.90169999839663506069342702176072634756565093994140625f,-0.216299942106754350401587316810037009418010711669921875f,-0.4320999889165160201542903450899757444858551025390625f,0.495899993552267659424614976160228252410888671875f,0.153400009291619066065237575458013452589511871337890625f,-0.44780000628232929926753058680333197116851806640625f,-0.0115999783962965619110718762385658919811248779296875f,-0.834899961298705139967069044359959661960601806640625f,0.8496000341415399503830485627986490726470947265625f,0.89210001067221167403431536513380706310272216796875f,-1.095100001224875274630221611005254089832305908203125f,0.3300999991759656637668740586377680301666259765625f,-0.918899991182982933679568304796703159809112548828125f,-0.27090000536143765685892503825016319751739501953125f,-0.797299992707371618649858646676875650882720947265625f,0.890200000922381828871721154428087174892425537109375f,-0.2795999845206740275216361624188721179962158203125f,-0.294699988427758352571572686429135501384735107421875f,-1.083099977621436504904295361484400928020477294921875f,-0.5677999825522304089275849037221632897853851318359375f,1.0654999634698036903301954225753434002399444580078125f,0.7143999882921583388650788037921302020549774169921875f,0.71569997515827399325871738255955278873443603515625f,0.9102000152647491404422908090054988861083984375f,0.40320005618333798480534824193455278873443603515625f,0.223199979069828913225137512199580669403076171875f,-0.023799963471293761330116467433981597423553466796875f,-1.0146999823182827338285960649955086410045623779296875f,-0.02159998688995823368230730920913629233837127685546875f,-1.157299972814321176173280036891810595989227294921875f,-1.76110001429319407151297127711586654186248779296875f,1.17829995317459168546747605432756245136260986328125f,0.371300014284253021568105168626061640679836273193359375f,-0.3352000372439629671816874179057776927947998046875f,0.072400030589104158451618786784820258617401123046875f,0.24609998061656934709162669605575501918792724609375f,-1.43560001718401952786052788724191486835479736328125f,0.882800019134581059887523224460892379283905029296875f,-0.29079997701644888508099029422737658023834228515625f,-0.536800012958049688194250848027877509593963623046875f,-0.5536999990522861736508275498636066913604736328125f,0.00449998966604425731929950416088104248046875f,-0.0582000022411346407125165569595992565155029296875f,-0.23859998724758602062223644679761491715908050537109375f,0.61789998385161160054934725849307142198085784912109375f,-0.962700033891201290003891699598170816898345947265625f,-1.469100004175304174935945411561988294124603271484375f,-0.7556000159919253889029278070665895938873291015625f,-1.3326000483497983939429332167492248117923736572265625f,0.62039995527863478486096937558613717555999755859375f,0.6767999834537505865483808520366437733173370361328125f,-0.40669999109506616008502533077262341976165771484375f,0.03199999827146526865107034609536640346050262451171875f,0.22179997719526323152194891008548438549041748046875f,1.46540001037567879649259339203126728534698486328125f,0.3882000193774697294202269404195249080657958984375f,0.2115000143498178830636646807761280797421932220458984375f,0.148499975740910006294370759860612452030181884765625f,0.778500000774859923780013559735380113124847412109375f,0.957699992763996643674317965633235871791839599609375f,-0.0466000133395194637841996154747903347015380859375f,-0.39809999900460280031211368623189628124237060546875f,0.51610003947615634700696318759582936763763427734375f,-0.426599976682663140081785968504846096038818359375f,-1.02759999310374272596391165279783308506011962890625f,-0.76679998732805199068707224796526134014129638671875f,-0.917100000166893192243833254906348884105682373046875f,0.3796000105977055394390617948374710977077484130859375f,-0.37210000746846194630279569537378847599029541015625f,0.844000025041401702452503741369582712650299072265625f,-1.679900013713538697857075021602213382720947265625f,0.030900015997141760948352384730242192745208740234375f,-1.05120000693127479962640791200101375579833984375f,0.33400000585615641224279670495889149606227874755859375f,0.35540001235008233937406885161180980503559112548828125f,-0.023400025366246968161476615932770073413848876953125f,-0.68330001009702723280270220129750669002532958984375f,0.9443000144854185595733042646315880119800567626953125f,-0.46519995625615084833270884701050817966461181640625f,-0.5622000060111285169028860764228738844394683837890625f,-0.965699999260902330888711730949580669403076171875f,0.7250000155903399967627365185762755572795867919921875f,0.1954999831952151101432946234126575291156768798828125f,-0.253699961128831308343478667666204273700714111328125f,-0.186200008328258392875653726150630973279476165771484375f,0.153900013367086696547403334989212453365325927734375f,0.816900053261220282507792944670654833316802978515625f,0.331399984365702504618411694536916911602020263671875f,-0.943100056225061766923545292229391634464263916015625f,0.3919999490976326939062346355058252811431884765625f,-0.1949000479862090795535323195508681237697601318359375f,-0.487099997037649057318731138366274535655975341796875f,0.85069999613165869778441674498026259243488311767578125f,0.2457999871678648862172167355311103165149688720703125f,-0.6245999897547063195446526151499710977077484130859375f,0.35869998457953322645863636353169567883014678955078125f,-0.71159998782128097705168556785793043673038482666015625f,-0.87780000056400886432328434239025227725505828857421875f,0.38059998505339021068749616460991092026233673095703125f,1.2305999900639064126295352252782322466373443603515625f,-0.415600010776520090161056941724382340908050537109375f,0.135599990510939960586256347596645355224609375f,0.94250006705522526573304276098497211933135986328125f,0.47430001444816571876117450301535427570343017578125f,-0.672299978029727629547096512396819889545440673828125f,-0.6034000378087165383789169936790131032466888427734375f,-1.0879000108599665708197790081612765789031982421875f,0.21100002755969793355461661121807992458343505859375f,0.452999995023012136385887060896493494510650634765625f,0.044699985112249762675418196522514335811138153076171875f,0.5196000061273571279940597378299571573734283447265625f,0.04440001332610832296410308117629028856754302978515625f,-0.2148000187098977331601190599030815064907073974609375f,-0.077300005187094011915860392036847770214080810546875f,0.19139999680817110228048250064603053033351898193359375f,0.036400006121396988323368759665754623711109161376953125f,-1.345100011767446090971134253777563571929931640625f,-2.518300004136561742740241243154741823673248291015625f,-0.95749999798834384279189180233515799045562744140625f,0.21820000149607654549299695645458996295928955078125f,0.9854000048995017468911328251124359667301177978515625f,0.18929996527433434749809748609550297260284423828125f,0.16990000052601017532794003273011185228824615478515625f,0.7620999843716618205036184008349664509296417236328125f,0.01599994787573855159479307985748164355754852294921875f,-0.812800004047155244535360907320864498615264892578125f,-0.583599974395335063803713637753389775753021240234375f,0.00130000020265563787091878111823461949825286865234375f,0.56710000687241546302175265736877918243408203125f,-0.348299970012902804228360764682292938232421875f,-0.398700025576352867773266552831046283245086669921875f,0.0279000071346757749779499135911464691162109375f,-0.0490999793648716487126648644334636628627777099609375f,-0.406699978578090703962288898765109479427337646484375f,0.384599996888637551961664939881302416324615478515625f,0.711799989563226542799156959517858922481536865234375f,-0.4643999774515628065074679398094303905963897705078125f,-0.2929999961405993502694400376640260219573974609375f,0.7200999824494125345353268130565993487834930419921875f,-0.3016999926596881653750870100338943302631378173828125f,0.75610001578330976457209544605575501918792724609375f,0.60520000267326790055477658825111575424671173095703125f,-0.04659999963045091675439834943972527980804443359375f,0.27189999248310925406002525051007978618144989013671875f,-0.5519999809861186879089700596523471176624298095703125f,-0.42370000505000360302432227399549447000026702880859375f,0.85519999887347208922250274554244242608547210693359375f,-0.356300022479892231075382369454018771648406982421875f,1.2113000320911411211710628776927478611469268798828125f,0.2143000063270333044318505244518746621906757354736328125f,0.0313000114530324058392807273776270449161529541015625f,0.509600005978345649282346130348742008209228515625f,-0.127199946816265896387676548329181969165802001953125f,1.364999980106949895031220876262523233890533447265625f,0.25689999600350954533922731570783071219921112060546875f,0.790999993938953149807957743178121745586395263671875f,1.340100010463594504273032725905068218708038330078125f,0.788199983894080080659705345169641077518463134765625f,0.265499959632754300997703467146493494510650634765625f,-0.7958000453382734207252724445424973964691162109375f,-0.85360000263303448253537908385624177753925323486328125f,0.0152000018537041814425947450217790901660919189453125f,-0.84490001319199847795715641041169874370098114013671875f,0.98919998483657833876492304625571705400943756103515625f,0.646599994713068060292471272987313568592071533203125f,0.5193999492883689672595437514246441423892974853515625f,0.11499999590218035194766343920491635799407958984375f,-1.21239995567351588334759071585722267627716064453125f,-0.84590000359192474288505536605953238904476165771484375f,0.571900013381987815108686845633201301097869873046875f,-0.35779998733922868670020989156910218298435211181640625f,-0.0156000056356191407758160494267940521240234375f,1.3177000138491390401895841932855546474456787109375f,0.2865999994814389939534748918958939611911773681640625f,0.78830001620650291815906030024052597582340240478515625f,-0.5616000057771801667882982656010426580905914306640625f,0.36179999503940341032404148791101761162281036376953125f,-0.1652000136256219775532372295856475830078125f,0.32080002788528805268697396968491375446319580078125f,-0.58510003482699435295444345683790743350982666015625f,0.292000015556812542172337998636066913604736328125f,-0.7055000235587358670130697646527551114559173583984375f,0.620999993026256191797074279747903347015380859375f,-0.660900027564168102145458760787732899188995361328125f,0.63640003137886547168733386570238508284091949462890625f,-0.049599994951486647831728760138503275811672210693359375f,-0.18720000124275681940844151540659368038177490234375f,-0.80130002464056027111638513815705664455890655517578125f,0.03239998229071527902078742044977843761444091796875f,-0.50710000093057761016979156920569948852062225341796875f,-1.8150000363215799126237470773048698902130126953125f,-2.43830002231597919859495959826745092868804931640625f,-2.609000014238059161897353988024406135082244873046875f,-1.7938000034809107319233589805662631988525390625f,-0.7519999916031963049789510478149168193340301513671875f,-1.9416000295817854581770234290161170065402984619140625f,-1.4015999935582275082168735025334171950817108154296875f,-0.05629998217225062884239150662324391305446624755859375f,0.8502000081866982750256056533544324338436126708984375f,-1.33569998435974124362246584496460855007171630859375f,0.43230000136867141602436959146871231496334075927734375f,1.215199979110807237958624682505615055561065673828125f,1.17120000335499607757583362399600446224212646484375f,0.58699998486042037626475575962103903293609619140625f,1.71629998768568015066193765960633754730224609375f,-0.154499971598386576232542211073450744152069091796875f,0.64019996594190597960505328956060111522674560546875f,0.0933000042110680993800997384823858737945556640625f,0.955800022241473090645058618974871933460235595703125f,-0.1834999966770407553440236370079219341278076171875f,-0.69210000113546865652125461565447039902210235595703125f,-0.231999992907046870271869920543394982814788818359375f,-0.58069999298378827123912060415023006498813629150390625f,-0.309900010174512796634616051960620097815990447998046875f,-0.838300025165826578898986554122529923915863037109375f,0.0313000067219137978469234440126456320285797119140625f,-0.788899988090991488576264600851573050022125244140625f,0.283300018590688817976541713505866937339305877685546875f,-0.79930000561922798230085618342855013906955718994140625f,0.3311000136971473661873233140795491635799407958984375f,0.184399963276833223080330981247243471443653106689453125f,0.106100014591217373105536125876824371516704559326171875f,-0.485799996536969924676441223709844052791595458984375f,-0.4820000280365348555022819709847681224346160888671875f,-0.216699990703910660982955960207618772983551025390625f,-0.82069999423175998831680999501259066164493560791015625f,-0.5299000460118066246906209926237352192401885986328125f,-1.467100007021427732212259797961451113224029541015625f,-1.2134000237271178601616838932386599481105804443359375f,-0.4912999672897162550810890024877153337001800537109375f,0.55379999239817190304080440910183824598789215087890625f,0.42089998790845284393213887597084976732730865478515625f,-0.6110000254362824012588362165843136608600616455078125f,-0.3322999677479270275171074899844825267791748046875f,-0.958499996937810738728558135335333645343780517578125f,-1.797899976736307081637278315611183643341064453125f,-0.00920000459924319446969320779317058622837066650390625f,0.527700029513984869566911584115587174892425537109375f,-0.71280000175610169765860746338148601353168487548828125f,0.806399970060586834819105206406675279140472412109375f,0.312100047552585468935149037861265242099761962890625f,-0.4114999762214726075626458623446524143218994140625f,-0.4105999831162392421646245566080324351787567138671875f,0.8946999703042208818004610293428413569927215576171875f,0.6467999928973615642036065764841623604297637939453125f,0.65379996980428689568043409963138401508331298828125f,0.3698999992966651717551940237171947956085205078125f,-0.8272999690890312507463022484444081783294677734375f,0.686200017827748975918211726821027696132659912109375f,-1.0933000384092335099950332732987590134143829345703125f,0.658000027805567100358530296944081783294677734375f,-0.5082999998897312909917900469736196100711822509765625f,0.5047999988913536295598305514431558549404144287109375f,-0.37400000838935387736938764646765775978565216064453125f,0.8969999996870756664435475613572634756565093994140625f,-0.5845000072121619982823403915972448885440826416015625f,0.64979997821599233542855245104874484241008758544921875f,-0.04720001264214512826100644815596751868724822998046875f,0.8466000175863503951489974497235380113124847412109375f,0.06159998428225514632838866191377746872603893280029296875f,0.484299993847310705774589223437942564487457275390625f,-0.59240007216334333861595951020717620849609375f,0.013699981729686339093632341246120631694793701171875f,-0.993399954921006411723283235915005207061767578125f,-0.40590002551525827811929048039019107818603515625f,0.8999999726004903610743212993838824331760406494140625f,0.20209998042285415298380257809185422956943511962890625f,0.030000006966293124577305206912569701671600341796875f,0.5005000044293701488840042657102458178997039794921875f,-0.125099995767325833373462273812037892639636993408203125f,0.13540000944435603980053883788059465587139129638671875f,0.176800013218820328830815924447961151599884033203125f,-0.19430000446438810968174948357045650482177734375f,0.004500008180737591345632608863525092601776123046875f,-0.34120001410245937023546503041870892047882080078125f,0.591900015300512283289435799815692007541656494140625f,0.245499986529349456532145268283784389495849609375f,0.6797000279903411268378476961515843868255615234375f,-1.09260001024007902259427282842807471752166748046875f,-1.791600018703937369224377107457257807254791259765625f,0.1384999897480003738792220246978104114532470703125f,1.12030001105070109446160131483338773250579833984375f,0.76840001154541948125142880599014461040496826171875f,0.144599999459087857900385643006302416324615478515625f,-0.89449999305605876198654868858284316956996917724609375f,-0.31520000874549125580159625314990989863872528076171875f,-0.78640001491308186842843497288413345813751220703125f,-0.51110001231878943173114748788066208362579345703125f,-0.096700001823901704511854404699988663196563720703125f,0.40229995966404719087705643687513656914234161376953125f,0.187999976545572877739687100984156131744384765625f,-0.9677000249877576454338168332469649612903594970703125f,-0.1570999977827067795033144648186862468719482421875f,-0.8370999819129705254994178176275454461574554443359375f,0.18100007796287531647294599679298698902130126953125f,0.5306000030279156565171660986379720270633697509765625f,-0.12420000431984679067909382865764200687408447265625f,-0.10209998392462704686067809234373271465301513671875f,-0.1462000106006862143459557046298868954181671142578125f,0.49750000953674344827959430404007434844970703125f,-0.2296000146955254894720610536751337349414825439453125f,0.2836999992012980698063984164036810398101806640625f,-0.29689999220371277033336809836328029632568359375f,-0.2205999051272868882733746431767940521240234375f,-0.418899987606704105047583652776665985584259033203125f,0.9748000076085319154373109995503909885883331298828125f,0.3952000167921185802555328336893580853939056396484375f,-0.164600018178671814172986387347918935120105743408203125f,0.6141999556303030516346552758477628231048583984375f,-0.3569000123202801244559623228269629180431365966796875f,-1.2518000001430509282585035180090926587581634521484375f,-0.34370000836551184431044703160296194255352020263671875f,0.734900001308322003978901193477213382720947265625f,-0.12130001018643365995330896112136542797088623046875f,0.664899987375736856165531207807362079620361328125f,0.2127000034928319960325637794085196219384670257568359375f,0.758099968140573121644365528482012450695037841796875f,0.8102999789759517401677157977246679365634918212890625f,1.474599919868261554967148185824044048786163330078125f,0.822200012418627945720572824939154088497161865234375f,0.01420001940727200206993074971251189708709716796875f,-0.28130000497102736201071593313827179372310638427734375f,-1.438800001841783693379284159163944423198699951171875f,-0.1414000083193183154861571892979554831981658935546875f,0.0984999880343675471294773160479962825775146484375f,-0.0094999713003638408537199211423285305500030517578125f,-0.27789999819397959157640798366628587245941162109375f,-1.060999977603554800253959911060519516468048095703125f,0.7051000216580922685949417427764274179935455322265625f,-0.86389995050281342958697905487497337162494659423828125f,0.74759999304786273199852075777016580104827880859375f,0.58040001264810570091867703013122081756591796875f,-0.17830001486539825972243988871923647820949554443359375f,-0.72619999048411809638992053805850446224212646484375f,-0.297400008311867747323731236974708735942840576171875f,-0.1591000197470189281290231519960798323154449462890625f,0.75409999884814027382162748835980892181396484375f,0.173499992430210081550967515795491635799407958984375f,0.31229998607635522933634319997509010136127471923828125f,1.8435999549679455977724273907369934022426605224609375f,0.8026999880142511312186570648918859660625457763671875f,1.211599950829148841791038648807443678379058837890625f,0.329199979118258045929223953862674534320831298828125f,0.7535999944746496481684516766108572483062744140625f,-0.76810001710951336661992172594182193279266357421875f,-0.961700007486343100282510931720025837421417236328125f,0.159900020232795814223436536849476397037506103515625f,0.8794000300526618740803996843169443309307098388671875f,-0.64579998312592579168267548084259033203125f,-0.1479000135421753014952628291212022304534912109375f,-1.00000004425644828387476081843487918376922607421875f,-1.48499999642372149111224644002504646778106689453125f,0.6286000503540041250971626141108572483062744140625f,-0.03830000756382911930586487869732081890106201171875f,0.2167999658703809728876876761205494403839111328125f,-0.2269000056147572053077965392731130123138427734375f,-0.57450001935660832685925925034098327159881591796875f,-0.995100018063187263805957627482712268829345703125f,-0.858299973067640831203561901929788291454315185546875f,0.701999992907047509760332104633562266826629638671875f,-0.599199986326694133964565480710007250308990478515625f,0.27780000028461149241820749011822044849395751953125f,0.9905999961733815695197336026467382907867431640625f,-0.5883000107675788736827371394610963761806488037109375f,1.24160002958178505849673456395976245403289794921875f,0.5606000307068226273798927650204859673976898193359375f,0.2961999729752538002003348083235323429107666015625f,-1.5808999906092882081765083057689480483531951904296875f,-0.371000000514090100267594607430510222911834716796875f,-0.85299993407726304450733323392341844737529754638671875f,-0.0367000002592803575840463281565462239086627960205078125f,-0.048899992412329229551914977491833269596099853515625f,-0.3341999986201524830420339640113525092601776123046875f,0.8654000031113631141721498352126218378543853759765625f,-0.402999988019466126587531107361428439617156982421875f,0.2154000099658954781034481129609048366546630859375f,0.493100021952390665802568037179298698902130126953125f,0.12219999535679892943562663276679813861846923828125f,-0.2842000217542051121455415341188199818134307861328125f,1.1773000452339648091992785339243710041046142578125f,2.13750001072883577535321819595992565155029296875f,1.16730002660751353005252894945442676544189453125f,-0.32709998675584728289322811178863048553466796875f,-0.124700017075240865427332437320728786289691925048828125f,-0.2810999410539860576818682602606713771820068359375f,-0.44839996049031605185319904194329865276813507080078125f,-1.510899977701157137488507942180149257183074951171875f,-1.8065000334978105200889331172220408916473388671875f,0.9370999867558484908158789039589464664459228515625f,0.7849999964982270483915272052399814128875732421875f,1.179499960348009413024783498258329927921295166015625f,0.280400012387335362262774651753716170787811279296875f,0.95309995787739776318403528421185910701751708984375f,1.2494000229001043233978407442918978631496429443359375f,-0.1649000155389310773301758672459982335567474365234375f,1.4316000024616715169401004459359683096408843994140625f,0.0316999866247174022504395907162688672542572021484375f,0.7715000091530381798321513997507281601428985595703125f,0.2551000121586024516062707334640435874462127685546875f,-1.5128999997213481432112303082249127328395843505859375f,0.54889998257756200672474733437411487102508544921875f,-1.4963000155135990354438035865314304828643798828125f,-0.05069997713267770933498468366451561450958251953125f,-0.6153000037491320739491129643283784389495849609375f,0.914500006690621258798046255833469331264495849609375f,-0.260900049841404690909030250622890889644622802734375f,0.545000023506581943166793280397541821002960205078125f,-1.176699997334927250136615839437581598758697509765625f,-0.58309998740032320529991238799993880093097686767578125f,-0.102999978482724408035409169315244071185588836669921875f,-0.60829997249022105254567804877297021448612213134765625f,0.44930002145171243910226621665060520172119140625f,-0.0472000227376820902236431720666587352752685546875f,1.320100010053813388566368303145281970500946044921875f,0.7251000058442353957133263975265435874462127685546875f,1.145800008159875371660518794669769704341888427734375f,-0.2184999925792217023712282752967439591884613037109375f,-1.487800002184510361757929786108434200286865234375f,0.31250005193054608554348305915482342243194580078125f,0.69110001250505470604679203461273573338985443115234375f,0.47610003701746439475073202629573643207550048828125f,-0.416100000062584740589954890310764312744140625f,-0.097199977251887670348651226959191262722015380859375f,0.46119999525248989158399126608856022357940673828125f,0.205700018408894624855065558222122490406036376953125f,-0.972500013522803907761726804892532527446746826171875f,0.49570001043677347496441143448464572429656982421875f,-0.0303999618351458167353484896011650562286376953125f,0.8134999532401561150862789872917346656322479248046875f,0.0019999606087803822873638637247495353221893310546875f,0.0300999857276682547535529010929167270660400390625f,0.82570001237392443016460674698464572429656982421875f,-0.34589998822510226172965985824703238904476165771484375f,0.0217999843105676216481469964492134749889373779296875f,-0.837799996522068823878726107068359851837158203125f,1.302299980898201425816296250559389591217041015625f,0.6630999705433844315649594136630184948444366455078125f,0.0426999939188359434183439589105546474456787109375f,0.3572999771356581977244104564306326210498809814453125f,-0.64469999572932668030489367083646357059478759765625f,0.204700002919137258317050509504042565822601318359375f,-0.97789994872212471221928353770636022090911865234375f,0.2032000291004776304504275685758329927921295166015625f,-0.714499997526407693015926270163618028163909912109375f,0.19870000531077380845346169735421426594257354736328125f,-0.64659999024271985756939784550922922790050506591796875f,0.44239999624192716254356128047220408916473388671875f,0.00329994866699012678878943916060961782932281494140625f,0.6393000028997659622831406522891484200954437255859375f,0.0350000185519458995031527592800557613372802734375f,-0.07020000114589930451103327868622727692127227783203125f,1.69429997659921571795393901993520557880401611328125f,-0.63210005232095767269129282794892787933349609375f,-1.0755000424087051147381544069503434002399444580078125f,-0.5631999564945697756002118694595992565155029296875f,-0.4435000037550924645302075077779591083526611328125f,-0.12590003717541708994076543604023754596710205078125f,0.3575000238418579723287393790087662637233734130859375f,0.19839999843239741750267057796008884906768798828125f,-1.1862000261723995020446409398573450744152069091796875f,0.2213999831363562265096334158442914485931396484375f,0.522199989619851034916564458399079740047454833984375f,0.6626999936953186232102552821743302047252655029296875f,-0.20769999276399608856280565305496565997600555419921875f,0.370700029957294674431977909989655017852783203125f,0.591099976257979609073345272918231785297393798828125f,-0.1073999893128874116854376552510075271129608154296875f,-0.6911000046446920119791457182145677506923675537109375f,0.59270000930428512475600655307061970233917236328125f,-0.00429999921172841137462228289223276078701019287109375f,1.15900003349781055561606990522705018520355224609375f,-0.833100007237494111222986248321831226348876953125f,-0.103400018064677912121851477422751486301422119140625f,-0.169099998773634629944950802382663823664188385009765625f,0.0973000121906399950688637545681558549404144287109375f,-0.682500001974403858184814453125f,0.042999995432794424488065487821586430072784423828125f,-0.8158999695241444527482599369250237941741943359375f,-0.70150000529736278309655972407199442386627197265625f,1.0025000253319744647484412780613638460636138916015625f,-0.036600004249811191048280534232617355883121490478515625f,0.860500000983476009963624164811335504055023193359375f,-0.012899991935491517125456084613688290119171142578125f,-0.10839999340325545684748931307694874703884124755859375f,0.166399989655613922412413785423268564045429229736328125f,0.511899997419118957253658663830719888210296630859375f,0.2909000010028481852941695251502096652984619140625f,1.1377000260680911747357413332792930305004119873046875f,-1.4134999983906748610706927138380706310272216796875f,-1.467500001043081425677883089520037174224853515625f,0.137300044786929476003933814354240894317626953125f,0.0786000274062157444632248370908200740814208984375f,0.20000002264976490806702713598497211933135986328125f,-1.059300000180304124342001159675419330596923828125f,0.0495000024065370691772614009096287190914154052734375f,0.4899000218532980621688466271734796464443206787109375f,-0.1694000427931552366089817951433360576629638671875f,0.3689999963194130483401522724307142198085784912109375f,-0.0641000155746933586442537489347159862518310546875f,0.873000001728534424927374857361428439617156982421875f,-0.7818999710440639461239697993732988834381103515625f,1.13480001006722464040876729995943605899810791015625f,0.61779999316930744868159308680333197116851806640625f,-1.4404999802708626788216861314140260219573974609375f,0.70839998993873631860651585157029330730438232421875f,-0.309399991365521653818859704188071191310882568359375f,-1.6700000133551655689956305650412105023860931396484375f,-0.182300012730807037275582160873454995453357696533203125f,0.4964000075556338487103857914917171001434326171875f,-0.202699978812784120663081921520642936229705810546875f,-0.1977000241495668053648415707357344217598438262939453125f,0.308899969334154922417923216926283203065395355224609375f,0.5557999934077262338405489572323858737945556640625f,-0.1631000117823482586931760351944831199944019317626953125f,0.3179000098913908889386448208824731409549713134765625f,-0.21930000640153879398752678753226064145565032958984375f,-0.60940001574754720792270745732821524143218994140625f,0.09090000729858893624424354129587300121784210205078125f,-0.16450000616908067030408346909098327159881591796875f,0.3071000189051036333154343083151616156101226806640625f,-0.91669997634291622290447776322253048419952392578125f,0.0420000268071893945176498164073564112186431884765625f,-0.006900040036440291402186630875803530216217041015625f,0.1597000071287154876475966602811240591108798980712890625f,0.45890001334846031166847524218610487878322601318359375f,-0.252400010398030172353855959954671561717987060546875f,0.22540003049224643927317401903565041720867156982421875f,-0.08039998433589889970107833505608141422271728515625f,0.64220002802759434956669792882166802883148193359375f,-0.237699978458881133747127023525536060333251953125f,-0.91660000592619128578775189453153870999813079833984375f,-0.691099958711863138205444556660950183868408203125f,0.7789000679239632685124661293230019509792327880859375f,1.2929000630885365552558141644112765789031982421875f,1.2314000020980826466399093988002277910709381103515625f,0.3307999905206260304879606337635777890682220458984375f,-0.07570001334249976121526515271398238837718963623046875f,-0.4394999986439940276028437438071705400943756103515625f,0.033999989315867562567063941969536244869232177734375f,-0.253899999266862241853459636331535875797271728515625f,-1.193000024378299439575812357361428439617156982421875f,1.365099964195490400697963195852935314178466796875f,0.27839999217391042662939071306027472019195556640625f,-1.365399989551306259016882904688827693462371826171875f,-0.3588999897301192731902119703590869903564453125f,0.20560001174509512367194474791176617145538330078125f,-1.164000021949410434984883977449499070644378662109375f,0.7686999878950413478406744616222567856311798095703125f,-0.3996000231146812353699715458787977695465087890625f,-0.0258000043600798978360444380086846649646759033203125f,1.0816999991416931248267019327613525092601776123046875f,0.290900007186830122929421804656158201396465301513671875f,0.997400012409686897996152765699662268161773681640625f,-0.2706000195682050080137059921980835497379302978515625f,0.1706000235915183793622418306767940521240234375f,0.5749000047728418660852867105859331786632537841796875f,-0.33679999058768184116985366927110590040683746337890625f,-0.023799994689226611210841610954958014190196990966796875f,-0.923899983639269617441414084169082343578338623046875f,0.1472000060111281760644175164998159743845462799072265625f,0.633799936388432794132086200988851487636566162109375f,-0.5728000349670647750599528080783784389495849609375f,-0.3843999892234801762924689683131873607635498046875f,1.1797999800413847548696821831981651484966278076171875f,0.475800001566111685491478056064806878566741943359375f,0.220799969863891387689136536209844052791595458984375f,0.6697000129401680101892679886077530682086944580078125f,2.05919996930211812724564879317767918109893798828125f,1.770299976573138867053103240323252975940704345703125f,1.102499987222253619023604187532328069210052490234375f,1.618799944360554743383318054839037358760833740234375f,-0.032200018379092032461130656884051859378814697265625f,-1.351499972403049643077110886224545538425445556640625f,0.0708999902367593026752956575364805757999420166015625f,1.2294999974519014163121255478472448885440826416015625f,-0.16419999489486236665669594003702513873577117919921875f,0.7198999977320432730465427084709517657756805419921875f,-1.37419996024966284409174477332271635532379150390625f,-0.404799986821413160242855155956931412220001220703125f,-0.5483999828793109099223102020914666354656219482421875f,-1.126099967894703457460536810685880482196807861328125f,-0.508799972020835156172324786894023418426513671875f,-2.051399932230264511190398479811847209930419921875f,1.3327999732762576545752608581096865236759185791015625f,0.30770000744164038852801468237885273993015289306640625f,0.209600015664100747247999834144138731062412261962890625f,0.71339996334016309020853441325016319751739501953125f,-0.61600001575052676372479254496283829212188720703125f,0.0897999999344343269314094868605025112628936767578125f,0.4261000205144285235547840784420259296894073486328125f,-1.33069999344944900343534754938445985317230224609375f,1.0194000262156130975910173219745047390460968017578125f,-0.34380001525282910534997427021153271198272705078125f,-0.3863000292971736637781532408553175628185272216796875f,-0.9138999876439566349972665193490684032440185546875f,0.480400031349063283414579927921295166015625f,-1.19400000984221588140599124017171561717987060546875f,0.00030001388192213429562116289162077009677886962890625f,0.63220004996955359022337006535963155329227447509765625f,-0.841299976360797341357056211563758552074432373046875f,-0.674499975472689694555583628243766725063323974609375f,-0.3320999959945687418638726740027777850627899169921875f,-0.6847000535085789874045758551801554858684539794921875f,1.130700012671947973785790964029729366302490234375f,0.5225999916881314266703384419088251888751983642578125f,-0.82780003980994276702176648541353642940521240234375f,-0.2145000170841806497179504731320776045322418212890625f,1.0283000117361549285988076007924973964691162109375f,-0.115799990651011430742300944984890520572662353515625f,-0.405500008955597568416351350606419146060943603515625f,0.61829994564950518931567557956441305577754974365234375f,-0.0670999675333501244267608853988349437713623046875f,-0.144900022840499520526691412669606506824493408203125f,0.85939997536539980416137041174806654453277587890625f,-0.2752999350547791213017490008496679365634918212890625f,0.250199984866380731318713515065610408782958984375f,0.47819998659491513848251997842453420162200927734375f,-0.80229998637437827113672028644941747188568115234375f,-0.77099997708201417623286033631302416324615478515625f,-0.56069997437596352796163046150468289852142333984375f,0.013900049930811064768931828439235687255859375f,-0.2766000136002901843568224649061448872089385986328125f,1.48380003477334998507330965367145836353302001953125f,-0.60190002109333884394715141752385534346103668212890625f,-0.308399996308982016213207089094794355332851409912109375f,0.1650999969780446996470146814317558892071247100830078125f,-0.62849998456984790262680462546995840966701507568359375f,0.6392999975353472752459538241964764893054962158203125f,-1.30539999700188591447158614755608141422271728515625f,0.2039000301867728293103709802380762994289398193359375f,0.224700010555982654381068641669116914272308349609375f,0.34250000860542062497415827238000929355621337890625f,1.32620004301071237051701245945878326892852783203125f,0.5464000009059901685759541578590869903564453125f,-1.0767000381827351862540353977237828075885772705078125f,-1.78669998835697807493261279887519776821136474609375f,-1.602399987487494126270348715479485690593719482421875f,-1.1422999786071479366711400871281512081623077392578125f,-1.4555000319778927941882784580229781568050384521484375f,0.2936999603092669275383741478435695171356201171875f,1.6496000276967894837554240439203567802906036376953125f,1.9740000522360201529181722435168921947479248046875f,2.18760004261285079252274954342283308506011962890625f,1.61600001318007713280167081393301486968994140625f,-0.80079999504685428490802223677746951580047607421875f,0.9828999904453752378685749135911464691162109375f,1.2209000127375120303696576229413039982318878173828125f,0.061499982610345005351604186216718517243862152099609375f,0.46399998586997359684147568259504623711109161376953125f,0.78320000516548715641107492047012783586978912353515625f,0.7161000202722844765190757243544794619083404541015625f,-1.42660001944899494930041328188963234424591064453125f,-1.03229999024868046575420521548949182033538818359375f,-0.59429998963773267295351843131356872618198394775390625f,-1.3731999938592311938378998092957772314548492431640625f,-0.391400007350742651368591396021656692028045654296875f,0.0834000073730951640982311801053583621978759765625f,0.3523000120043757732446465524844825267791748046875f,0.5207000248908997264152276329696178436279296875f,-0.5838000165939332131159744676551781594753265380859375f,0.83419998506009573890906949600321240723133087158203125f,0.040899996234476765355481120423064567148685455322265625f,1.4954999965131279271446373968501575291156768798828125f,-0.04050000742822879029603200251585803925991058349609375f,0.6951000129967928042873381855315528810024261474609375f,0.00229998577833168127426688442938029766082763671875f,-0.79459996742159100246993830296560190618038177490234375f,0.85700000620633343118015545769594609737396240234375f,-1.35790001518130143409734955639578402042388916015625f,-0.032499963268637799274074495770037174224853515625f,-0.1489999393597243315667810747982002794742584228515625f,-0.1973999858111141758598705564509145915508270263671875f,-0.487099982732534630969212230411358177661895751953125f,0.42749997474253220719475621081073768436908721923828125f,0.00050003253668473579551800867193378508090972900390625f,0.635699975381791571038547772332094609737396240234375f,-0.867800012615322913234194857068359851837158203125f,-0.4167000052884224370330912279314361512660980224609375f,0.29770003966540159456855008102138526737689971923828125f,2.10389997944831907972229601000435650348663330078125f,0.337500008419155950178947023232467472553253173828125f,-0.35370001879632473329451158861047588288784027099609375f,1.199599972078204235259590859641321003437042236328125f,-0.072799994063377226893862825818359851837158203125f,0.34339995283484470167678637153585441410541534423828125f,-0.3393999936938285610921184343169443309307098388671875f,1.316299963843822862230581449694000184535980224609375f,0.50859998624175783987766408245079219341278076171875f,0.851299997110664730115558995748870074748992919921875f,-1.227799995441734903778296938980929553508758544921875f,-0.45429999123960751461481777369044721126556396484375f,0.22009999369978938688063863082788884639739990234375f,0.2233999456822868268091042409650981426239013671875f,-0.19510000167190977293785181245766580104827880859375f,-0.7239000207245351870000149574480019509792327880859375f,0.49930001392662515247167220877599902451038360595703125f,-0.974200011360645046210038344725035130977630615234375f,0.2775000182539228976708045593113638460636138916015625f,-1.0785000314712529512206629078718833625316619873046875f,0.48090000726133563802022763411514461040496826171875f,0.122000013656914430004718497002613730728626251220703125f,-0.1598999960556624910790191052001318894326686859130859375f,-0.046899997307360052900548907928168773651123046875f,0.37330000808537011636190072749741375446319580078125f,-0.59090007938295752154544970835559070110321044921875f,-1.717200009512901193176048764144070446491241455078125f,-0.937999966152012643050284168566577136516571044921875f,0.4367999955236909759292984745115973055362701416015625f,0.66339999466985499765314671094529330730438232421875f,0.388100033128262023041088468744419515132904052734375f,0.06359996664672984412192136005614884197711944580078125f,0.41399996592104415515933624192257411777973175048828125f,0.7333000083088878628956308602937497198581695556640625f,0.5669000182062384762815554495318792760372161865234375f,0.682200020204484669505973215564154088497161865234375f,0.7658000089049339376146008362411521375179290771484375f,0.6024000113666065914230784983374178409576416015625f,-0.5521999965488912920363873126916587352752685546875f,0.3234999866187575623399652613443322479724884033203125f,-0.139100014084577505713014033972285687923431396484375f,-0.7075999831199644152235350702540017664432525634765625f,0.6863000106334684602416018606163561344146728515625f,-0.85060000213384601153165931464172899723052978515625f,-0.57020000755339861342463336768560111522674560546875f,0.329500017903745057168407583958469331264495849609375f,-0.476799992655217774739639935432933270931243896484375f,-0.7514999998584388496425390258082188665866851806640625f,-0.5247000018388032760441319624078460037708282470703125f,-0.021900016194581706230337658780626952648162841796875f,1.040300035041571202754084879416041076183319091796875f,-0.99400002127885844771526535623706877231597900390625f,0.05919998807758075631824112861067987978458404541015625f,-1.30080001203417783273152963374741375446319580078125f,0.032599986360967003751198944883071817457675933837890625f,-0.90419999724179500599774428337696008384227752685546875f,-0.0317999930649993378661832821308053098618984222412109375f,-0.064499983128160298662123750546015799045562744140625f,-0.2241999998122453929827457841383875347673892974853515625f,-0.2620999920457600662615504916175268590450286865234375f,-0.222799990561604432937059527830570004880428314208984375f,0.7750000001676380367854335418087430298328399658203125f,0.9910000286400315605561672782641835510730743408203125f,0.288300009556859748727219994179904460906982421875f,1.277799963068961375256549217738211154937744140625f,1.21530000076889965754389777430333197116851806640625f,0.569799968101829534816715749911963939666748046875f,0.250699984117597363120211184650543145835399627685546875f,1.2188999645471578769928555630031041800975799560546875f,0.3190999997980892288751419982872903347015380859375f,-0.087699972125888425722450847388245165348052978515625f,-0.16420000163763737166533474010066129267215728759765625f,-0.5765000485479827574408773216418921947479248046875f,0.087599997946619989619421176030300557613372802734375f,0.251000033557415402896140221855603158473968505859375f,-0.44290002218484847418267236207611858844757080078125f,0.629599993312358829911090651876293122768402099609375f,0.37300001662969606286424095742404460906982421875f,-1.068000016555190168077160706161521375179290771484375f,-0.32869996992051608852847266462049447000026702880859375f,0.581700018028914911383253638632595539093017578125f,0.159800018449127723130231970571912825107574462890625f,-0.357600019031762972332444405765272676944732666015625f,-0.643200009691714758019998043891973793506622314453125f,-0.0072999914407729793452972444356419146060943603515625f,-1.54380003946721533480967991636134684085845947265625f,0.07300002229213731208545823392341844737529754638671875f,-0.74450001734495163674409923260100185871124267578125f,1.2691999949321151053283074361388571560382843017578125f,1.072799990338086928431948763318359851837158203125f,0.79389997598379835608284338377416133880615234375f,-0.00130002963244901792450036737136542797088623046875f,0.78470001111477627464552142555476166307926177978515625f,0.4981999989256262750814130413345992565155029296875f,0.2504000231906766682499210219248197972774505615234375f,-0.984200006890297185435656501795165240764617919921875f,0.20390001435428850573572390203480608761310577392578125f,0.039899996558576622529557198504335246980190277099609375f,0.04859998493790573803607912850566208362579345703125f,0.434499997638166046698415811988525092601776123046875f,-0.005800052118301035619651884189806878566741943359375f,0.47829999869763850650628000948927365243434906005859375f,-0.888699982176720748583420572686009109020233154296875f,-0.216499997027218338274678899324499070644378662109375f,-0.29030002535581633793526634690351784229278564453125f,-0.03199999406188724915267584947287105023860931396484375f,-1.15819997601509072637782082892954349517822265625f,-0.8343000027880069779939731233753263950347900390625f,0.28170000465512234910647748620249330997467041015625f,0.525000032689422635456821808475069701671600341796875f,0.1371000031098723592748456212575547397136688232421875f,0.21909998467341129657626197513309307396411895751953125f,-0.636400005376339006346597670926712453365325927734375f,-0.73229995076060294678654827293939888477325439453125f,-0.365399997374415363982080862115253694355487823486328125f,-0.038899986340105829896174327586777508258819580078125f,-0.2196000294476745384741889210999943315982818603515625f,1.1618999724224210456924311074544675648212432861328125f,1.23240000883340794501918935566209256649017333984375f,0.1589000327944756207898535649292171001434326171875f,0.76730004881024405083422834650264121592044830322265625f,1.6201999984450636826949221358518116176128387451171875f,-0.0019999815076590099494069363572634756565093994140625f,0.55489997502639865079032688299776054918766021728515625f,0.8354999836608769658852224893053062260150909423828125f,-0.983100028806924708391079548164270818233489990234375f,0.42929998643398281643612790503539144992828369140625f,-0.228700019578635505013153306208550930023193359375f,-1.099199987705052450337461777962744235992431640625f,-0.1948999675199389880475564496009610593318939208984375f,-0.1050999877765772527737908603739924728870391845703125f,-0.3970999545693398857792999478988349437713623046875f,-0.0420999599337574803570305448374710977077484130859375f,-0.061399987588077797884267283734516240656375885009765625f,-0.24280003787279136329146922435029409825801849365234375f,0.0245000041015446534675703560424153693020343780517578125f,-1.6069999928325413574015101403347216546535491943359375f,-0.20809999848306193204194869394996203482151031494140625f,-0.39439998628050088047558574544382281601428985595703125f,-0.0093999991700053353582688941969536244869232177734375f,0.325100024843215928971318362528108991682529449462890625f,0.58380000173002510877040549530647695064544677734375f,0.7101999881446356965852828579954802989959716796875f,-0.01709999841600673420316525152884423732757568359375f,0.8218999709695575806023271070444025099277496337890625f,-0.1203000497564676862793930922634899616241455078125f,-0.158599995890259748421158292330801486968994140625f,0.0723999905422327838522278398158960044384002685546875f,-0.1699000005632640419861445479909889400005340576171875f,0.157900020545721275766481994651257991790771484375f,-1.1772000349938871455179878466879017651081085205078125f,0.2457000461995608731058382545597851276397705078125f,0.06290000966787356873055614414624869823455810546875f,0.6116000167667863252063398249447345733642578125f,0.52940000397562980705146173932007513940334320068359375f,0.2061000228613618645567839848808944225311279296875f,-1.227999993532896017001121435896493494510650634765625f,0.055699994604289759791981850867159664630889892578125f,-0.225200012060999765761692970045260153710842132568359375f,0.86040000672489413968690996625809930264949798583984375f,0.6611999879136678526236892139422707259654998779296875f,0.806099988961219704464156166068278253078460693359375f,0.5688999910712240204446743518928997218608856201171875f,-0.11990003148317374126463619177229702472686767578125f,0.35699999544024441089362653656280599534511566162109375f,0.681600027793646034979246906004846096038818359375f,0.9802000226408242866682485328055918216705322265625f,0.93070002258121942162460982217453420162200927734375f,-0.792600005508959259969969934900291264057159423828125f,0.19489999214410780314921112221782095730304718017578125f,0.7500000002235174267894990407512523233890533447265625f,0.16620000255405897870986109410296194255352020263671875f,-0.240200028824806732785646090633235871791839599609375f,0.44310002437382955964295661033247597515583038330078125f,-0.892299997550248580324705471866764128208160400390625f,0.224600003370642564703985044616274535655975341796875f,0.5232000096917144293939827548456378281116485595703125f,0.8081000246345999560304562692181207239627838134765625f,0.3424999880045651590165789457387290894985198974609375f,-0.1952999998003244197519734370871447026729583740234375f,0.61290001249909364933188271606923080980777740478515625f,1.46469998731017181370361868175677955150604248046875f,1.16819998786151391101384433568455278873443603515625f,0.47399998670816412271733497618697583675384521484375f,0.478500045776367510796944770845584571361541748046875f,-1.8742000027179717136505132657475769519805908203125f,-0.80480006192326580816143177798949182033538818359375f,0.56489996800422659362084232270717620849609375f,0.695700028616190291330667605507187545299530029296875f,0.000900006926059404577245004475116729736328125f,1.85709997677207017119371812441386282444000244140625f,1.1631999912142756414823452359996736049652099609375f,0.440299977560341115889741558930836617946624755859375f,1.730700015056132912150133051909506320953369140625f,-0.51719998649060749951189563944353722035884857177734375f,-0.339799978178739525702667378936894237995147705078125f,0.1360000414922837297382329779793508350849151611328125f,-1.74159998100399970866192234097979962825775146484375f,-2.059900029695034273657938683754764497280120849609375f,-1.1823000142395496681047006859444081783294677734375f,0.0102000087827445184984753723256289958953857421875f,-1.03429997406601881237975248950533568859100341796875f,0.790600045868754097710962014389224350452423095703125f,-0.36559998171925567334028528421185910701751708984375f,0.5445999837011095134897686875774525105953216552734375f,0.3007999871119857449031087526236660778522491455078125f,-0.07770000513195984837011565105058252811431884765625f,-0.33779999637305735404879669658839702606201171875f,-0.8447999702811248567968505085445940494537353515625f,-1.273200009095668772118870037957094609737396240234375f,-0.42309998425245165520891532651148736476898193359375f,0.1409999855756758346814194737817160785198211669921875f,0.05200000452995201527528479346074163913726806640625f,0.7746000099271537209233429166488349437713623046875f,-0.856800036054849112332476579467765986919403076171875f,-0.10270001969784481321568136991118080914020538330078125f,-1.6963999575622377147254837836953811347484588623046875f,-0.736400008133053507464182985131628811359405517578125f,0.36170000465512242016075106221251189708709716796875f,0.8432999653935429051898609031923115253448486328125f,-0.03379998172521592181283267564140260219573974609375f,0.485599987679720346278600118239410221576690673828125f,-0.7262999959558247287105814393726177513599395751953125f,0.6903999920845029780736012980923987925052642822265625f,0.47200000698864441606161790332407690584659576416015625f,0.012499993462115721598593154340051114559173583984375f,0.2082999861434104982294002184062264859676361083984375f,0.04049998092278872530158650988596491515636444091796875f,-1.245300026398897585977465496398508548736572265625f,-1.098200028690695884137085158727131783962249755859375f,-0.8172000259414315959105579167953692376613616943359375f,1.53850000688433663498244641232304275035858154296875f,0.132900003521144671747578058784711174666881561279296875f,1.557799972605705018935395855805836617946624755859375f,-0.25870000490099187562265115047921426594257354736328125f,0.5609999833405019398213653403217904269695281982421875f,0.04310000608265396326856944142491556704044342041015625f,0.29250000288709987472657303442247211933135986328125f,0.378500011019408333368119201622903347015380859375f,0.42870000052377565946670756602543406188488006591796875f,0.9021999596685168487653072588727809488773345947265625f,-0.3363000262796880246440878181601874530315399169921875f,2.153699972677230789486202411353588104248046875f,0.767700007814168916553398958058096468448638916015625f,1.0740999593228102160225034822360612452030181884765625f,0.489800000120699241534794055041857063770294189453125f,0.1373000107377766454419543151743710041046142578125f,0.1522999960601330027287758639431558549404144287109375f,0.579599980143457482739677288918755948543548583984375f,0.033500001389533295981237870364566333591938018798828125f,-0.27280001641511886223412375329644419252872467041015625f,0.286900006490200976116966558038257062435150146484375f,-1.1116999805152421032516940613277256488800048828125f,0.73869999517798401456047940882854163646697998046875f,-0.358899970656632927301643576356582343578338623046875f,-1.174999992251397173959048814140260219573974609375f,0.24259998058676723875493053128593601286411285400390625f,0.02509998081028452787677451851777732372283935546875f,0.642000001251697316462241360568441450595855712890625f,0.443100002059340081217442275374196469783782958984375f,0.2811000169754027666613183100707828998565673828125f,-1.2634000033497816417593639926053583621978759765625f,-0.828199981677531837931383051909506320953369140625f,0.50660000946521677178679965436458587646484375f,1.644899983352422712101770230219699442386627197265625f,-0.50190002155899993141474624280817806720733642578125f,-0.25910001419633577057766160578466951847076416015625f,-2.025799990874528333506532362662255764007568359375f,0.50709999096542579533064554198062978684902191162109375f,0.172299990323185880924938828684389591217041015625f,-0.236299999830126505884209109353832900524139404296875f,0.077100005009770544717184748151339590549468994140625f,0.0367999650880699302746279499842785298824310302734375f,-0.6856000338733192478457567631267011165618896484375f,-0.20110000334083988349220817326568067073822021484375f,-0.8756000279128546281981471111066639423370361328125f,0.750799982194602311125208871089853346347808837890625f,-0.152200022178888560997478407443850301206111907958984375f,-1.1913999527752405782621281105093657970428466796875f,-0.1553000057980418180392234717146493494510650634765625f,0.2249999863654383869260300343739800155162811279296875f,0.13129998403489595570903247789829038083553314208984375f,0.10369999949932118266104907888802699744701385498046875f,-0.5146000107467176842845901774126105010509490966796875f,-0.4152000067710872599491267465054988861083984375f,-0.149599996665120471561749582178890705108642578125f,-1.2613999570965770402608541189692914485931396484375f,-0.35770001548826702020278389682061970233917236328125f,-0.4017000183641901145392694161273539066314697265625f,0.0358000062227248339041807412286289036273956298828125f,-1.874499998494982389019014590303413569927215576171875f,-0.11250001125037645266502295271493494510650634765625f,0.35719998090267157664357000612653791904449462890625f,0.9024999868869780783597889239899814128875732421875f,-0.5279000098168846744073334775748662650585174560546875f,0.6260999974548813185037943185307085514068603515625f,-0.19430001310706135342343259253539144992828369140625f,-0.0589999838769443840646999888122081756591796875f,1.390799999368190764670316639239899814128875732421875f,-0.5621000016570096846635351539589464664459228515625f,-0.33009998595118528186276307678781449794769287109375f,-0.79589999230429520071794513569329865276813507080078125f,-1.180899986530095535641748938360251486301422119140625f,-1.3545999950446188098140964939375407993793487548828125f,0.32979997996687959727069028303958475589752197265625f,-0.08820002831816697863587251049466431140899658203125f,-0.7133999783344566747445014698314480483531951904296875f,-0.41219997704699651297488571799476630985736846923828125f,0.67649997644498949256330888601951301097869873046875f,-0.755699966496974351315429885289631783962249755859375f,-0.7703999630272393428498389766900800168514251708984375f,-0.70099998624622816834062177804298698902130126953125f,0.818899986526370415873543606721796095371246337890625f,1.3099000510968270560852033668197691440582275390625f,0.000399979194998234532931746798567473888397216796875f,1.266900032734870951145467188325710594654083251953125f,-0.0032999998152256626582357057486660778522491455078125f,0.24399997597932809867415926419198513031005859375f,-0.047100010597705743720098325866274535655975341796875f,0.0063000180840493413825242896564304828643798828125f,-0.2060000049471855110283513567992486059665679931640625f,-0.0667000109136106633656027042889036238193511962890625f,-0.53210000940561297255726458388380706310272216796875f,-0.08979998443722703171943066990934312343597412109375f,0.25750000774860382080078125f,0.621900043500960730824544953065924346446990966796875f,0.63859996377825734015232228557579219341278076171875f,0.01879998723864539922345784361823461949825286865234375f,0.519799977731704831995784843456931412220001220703125f,-0.9631000446394086811352508448180742561817169189453125f,-0.9997000161811711027581850430578924715518951416015625f,-0.9659999722018834944492482463829219341278076171875f,-0.233299988676607394921802551834844052791595458984375f,-0.77630003492236177020657805769587866961956024169921875f,-1.83590000036954936746269595460034906864166259765625f,-0.74690000490695229728999038343317806720733642578125f,-0.20500000864267331479595668497495353221893310546875f,0.21110000358298375289223258732818067073822021484375f,-0.08689999071359633120437138131819665431976318359375f,0.883300007265805930956048541702330112457275390625f,1.000299982440471691091943284845910966396331787109375f,0.516399989432096528929605483426712453365325927734375f,-0.37300000776350505926615142016089521348476409912109375f,0.9618999936565757469253412637044675648212432861328125f,0.775400003297626927434293975238688290119171142578125f,0.0871999916687606901888329957728274166584014892578125f,-0.3747999753475195205965064815245568752288818359375f,0.25610002681016919012790822307579219341278076171875f,0.7833999923229215678333048344939015805721282958984375f,-1.6303999888621272251754135140799917280673980712890625f,0.22870000676363655411904574066284112632274627685546875f,-0.036999993894248905945687511120922863483428955078125f,-0.2813000213995577869496855782927013933658599853515625f,1.2189000091016293225720801274292171001434326171875f,-0.18020001321583978270979287117370404303073883056640625f,0.203100021021068111171103964807116426527500152587890625f,0.18610002539455905434806481935083866119384765625f,0.433899998633563654948375187814235687255859375f,-0.50930001187771534176818022388033568859100341796875f,-0.13750001475214990165341077954508364200592041015625f,0.0673999864071601617609985623857937753200531005859375f,-0.41840001014843586357727645008708350360393524169921875f,0.2444999951049682540560326060585794039070606231689453125f,-0.2653000630557531547282223982620052993297576904296875f,-1.098000027284026014484652478131465613842010498046875f,-1.2924999712035056820269574018311686813831329345703125f,-0.42600000360608103466830698380363173782825469970703125f,-0.31559996722787619916061885305680334568023681640625f,1.2370000174194568653973647087696008384227752685546875f,1.6104000200614330129411655434523709118366241455078125f,0.1073000294387345121549515170045197010040283203125f,0.5453000341102478554233812246820889413356781005859375f,-0.1820000265836714259393147585797123610973358154296875f,-1.025099990738183475258438193122856318950653076171875f,1.0618999804504216655232085031457245349884033203125f,0.1944000292889768999327060328141669742763042449951171875f,0.0424999814480546689310358488000929355621337890625f,1.819799966481328024059394010691903531551361083984375f,-0.5423999932989478356404333680984564125537872314453125f,0.3817999894514680914170412506791763007640838623046875f,-0.9361999788984662895785504588275216519832611083984375f,-0.5114999895766374482519722732831723988056182861328125f,-0.451999986499547645735219703055918216705322265625f,-1.159100026899576363348387531004846096038818359375f,0.9145000125020743553250213153660297393798828125f,-0.137599990086257395471847075896221213042736053466796875f,0.46680001452267172457055721679353155195713043212890625f,-0.6170000141039491126093707862310111522674560546875f,0.351900018392503621811329139745794236660003662109375f,0.4396000188678501618966265596100129187107086181640625f,0.0079999806061381573130120159476064145565032958984375f,0.5665000256001955225571009577834047377109527587890625f,0.26020001537650860878869707448757253587245941162109375f,0.455900004988908591485596843995153903961181640625f,-0.3118000133305789400850471793091855943202972412109375f,-1.0583000280529262226991704665124416351318359375f,0.806300013315677688297000713646411895751953125f,0.5142999901965261688729924571816809475421905517578125f,0.6435000030100346268824296203092671930789947509765625f,0.250599985593557283980459260419593192636966705322265625f,0.60850001500546913035805118852294981479644775390625f,-0.684700032572448602508075055084191262722015380859375f,-0.311700036096573374067020267830230295658111572265625f,-0.4112999595224844373575479039573110640048980712890625f,1.2749000288009642023467904436984099447727203369140625f,0.22640000939965176929291601481963880360126495361328125f,-0.5579999962151049874847785758902318775653839111328125f,-0.6673000246703626459066072129644453525543212890625f,0.61029997946023961663541967936907894909381866455078125f,-0.8108999809607861930516037318739108741283416748046875f,0.378699984709918613390300379251129925251007080078125f,-0.3172999961718916406283597098081372678279876708984375f,0.927899992457031697057345809298567473888397216796875f,0.988400020113587185477399543742649257183074951171875f,-0.81930003203153578539286172599531710147857666015625f,1.0976999873995776368929000454954802989959716796875f,-0.069499992758036199802518240176141262054443359375f,0.2980999909579755691169111742055974900722503662109375f,0.130100003013014831054761089035309851169586181640625f,-0.774500010564923346834120820858515799045562744140625f,-0.1688999951690437484330686856992542743682861328125f,-1.675000021904706759556802353472448885440826416015625f,-0.45219998306333997195594065487966872751712799072265625f,-0.2276000161260365306503672400140203535556793212890625f,-0.7221000321671369182041644307901151478290557861328125f,0.00909998890161478612981227342970669269561767578125f,0.3544000314310193910927182514569722115993499755859375f,0.8565000078678135508880586712621152400970458984375f,-0.168000005267560315047603580751456320285797119140625f,1.892800011795758852173321429290808737277984619140625f,-1.156400009958446162272593937814235687255859375f,0.41849995562434205709223533631302416324615478515625f,-0.8535000001601869001888189814053475856781005859375f,0.917799975138902990323686026385985314846038818359375f,-0.234499945148825883478593823383562266826629638671875f,0.46649999244511153317915841398644261062145233154296875f,-2.281500003993510716782111558131873607635498046875f,-1.0875000169500717017712076994939707219600677490234375f,0.75249999523162802717024533194489777088165283203125f,1.798300049138070288989865730400197207927703857421875f,1.16200000452995322319793558563105762004852294921875f,0.3990000265687694280813957448117434978485107421875f,-0.605399991935491765815413600648753345012664794921875f,-0.30880003589093707461188387242145836353302001953125f,-2.243999973837286443512084588292054831981658935546875f,-1.771599980780482308517775891232304275035858154296875f,0.3745000087767837992913655398297123610973358154296875f,-1.139499991789460153768231975845992565155029296875f,0.214200030732154989010496137780137360095977783203125f,0.18889999032616611174262288841418921947479248046875f,-0.69670001836419093166341554024256765842437744140625f,0.02910001408457763005799279198981821537017822265625f,-0.3112999766588213645945870666764676570892333984375f,0.85989997232556358852662015124224126338958740234375f,0.1760000317692753668552541057579219341278076171875f,-0.840200018095969891618324254523031413555145263671875f,0.362499999366700353942150059083360247313976287841796875f,0.161300022926926800437286146916449069976806640625f,0.7453000306829815624354296232922933995723724365234375f,-0.9711999844491483457886715768836438655853271484375f,0.577099995584785308011532833916135132312774658203125f,-0.762399990803003024808504051179625093936920166015625f,0.23569999903738503288508354671648703515529632568359375f,-0.5214999857395887783440002749557606875896453857421875f,-0.026900001479685242689043889185995794832706451416015625f,-0.3634999920763075653695750588667578995227813720703125f,0.218600018987059330211053520542918704450130462646484375f,0.533599981603771755800380560685880482196807861328125f,0.17600000692159001314252009251504205167293548583984375f,0.08490004072189372141821195327793247997760772705078125f,2.1495000268816948363337360206060111522674560546875f,1.4915999952718614895985638213460333645343780517578125f,1.1480999761313199769574566744267940521240234375f,0.202399958355725662073609782964922487735748291015625f,0.245499996811151532938310992904007434844970703125f,0.513400009272992452480366409872658550739288330078125f,0.219100006000697622443595946606365032494068145751953125f,0.12269998834952711153079007999622263014316558837890625f,0.4993999910488720050949495998793281614780426025390625f,-0.53280005646198969060378658468835055828094482421875f,-0.180099992209672876697368337772786617279052734375f,0.46199999223649490520671179183409549295902252197265625f,0.07889999852180462625028667389415204524993896484375f,-1.40899996480345723881555386469699442386627197265625f,0.521899967840314449318839251645840704441070556640625f,-0.40519998315274763367455079787760041654109954833984375f,-0.5868999885529280202689506040769629180431365966796875f,-0.34879999617934220879078566213138401508331298828125f,0.09880000536218280782208012169576250016689300537109375f,-0.53709998457655327985804660784197039902210235595703125f,1.069999980162828823182508131139911711215972900390625f,0.76640001552775505189174509723670780658721923828125f,-0.22630001480579409189886064268648624420166015625f,0.41209997632503547748683558893389999866485595703125f,-0.190500008210539473196831750101409852504730224609375f,1.851899994289874396002915091230534017086029052734375f,-0.1531000040337442413829194265417754650115966796875f,-0.03190000747740262543317157906130887567996978759765625f,-0.436200004528462692565682345957611687481403350830078125f,-1.0077000186175109064379284973256289958953857421875f,0.1629000024780629107823415324673987925052642822265625f,0.9689000015020372558183225919492542743682861328125f,-0.43340002447217695902281775488518178462982177734375f,0.19100001463294002945758620626293122768402099609375f,-0.52789998847097141521089724847115576267242431640625f,-0.2460999970450987728298741785692982375621795654296875f,-0.5427000140726558896631104289554059505462646484375f,-0.356099959717692371441444265656173229217529296875f,0.0209999705255030644224234492867253720760345458984375f,-0.21950000173598527286600301522412337362766265869140625f,-0.70989997023940054621249373667524196207523345947265625f,0.30210001246035111943655238064820878207683563232421875f,-0.0779000078797338346703327260911464691162109375f,-0.7937000004306427047851002498646266758441925048828125f,-1.1107000010490413188790626008994877338409423828125f,0.18060005093514863716563922935165464878082275390625f,-0.129499987989664333554173936136066913604736328125f,-0.6424000289499758320488354002009145915508270263671875f,-0.1886999746143818157406712998636066913604736328125f,1.70859995409250320363980790716595947742462158203125f,-0.2044999947696926057005839538760483264923095703125f,-0.06419999593794312175987215596251189708709716796875f,-0.147099989065527569920277528581209480762481689453125f,-1.435100012996792795405553988530300557613372802734375f,1.2775999524235732707211354863829910755157470703125f,1.25969998701214791481106658466160297393798828125f,-1.20039996272921545283907107659615576267242431640625f,0.0827999635159972768860825453884899616241455078125f,1.2449999749660491943359375f,-0.9162999960303306323794458876363933086395263671875f,0.75799999967962483626848779749707318842411041259765625f,-1.07260001158118267738927897880785167217254638671875f,0.7633000333428385797418513902812264859676361083984375f,-1.38030004234313974365022659185342490673065185546875f,0.13330000562667831953689301371923647820949554443359375f,-0.3578000269949435097061041233246214687824249267578125f,-0.453500000625848220803248977972543798387050628662109375f,0.142299981456994739659194237901829183101654052734375f,-0.193799981613457017459722919738851487636566162109375f,0.64209996216893205911446784739382565021514892578125f,-0.8200000268593428121022270715911872684955596923828125f,0.2744000022247432557520596674294210970401763916015625f,0.25180004723072091366020686109550297260284423828125f,0.0352000016301869944612690233043394982814788818359375f,1.2476000079303977141620407564914785325527191162109375f,0.887400011031329594146654926589690148830413818359375f,0.53300000470876718328128163193468935787677764892578125f,-0.2043000056564807931369642801655572839081287384033203125f,1.08030003846883726481564735877327620983123779296875f,-0.1737999857485295063241892421501688659191131591796875f,1.22779997960925069122595232329331338405609130859375f,0.39669999295771118941189570250571705400943756103515625f,-0.0723000108122817675138094273279421031475067138671875f,-0.55969996421337242509252973832190036773681640625f,0.188999976724385898574354314405354671180248260498046875f,-0.62599998799711453756344781140796840190887451171875f,-0.353099986785650354903509651194326579570770263671875f,-1.1075999666169284996186661373940296471118927001953125f,-0.3917999554023143815584262483753263950347900390625f,-0.59449999752640803052372575621120631694793701171875f,-1.0564999959468845958099336712621152400970458984375f,-0.1087000000953679546000785194337368011474609375f,-0.416299983699619791099877375017968006432056427001953125f,0.7193999901920562489721078236470930278301239013671875f,0.2735999787352978440679862615070305764675140380859375f,-0.7289000003099435076592271798290312290191650390625f,0.2751999882563946897562345839105546474456787109375f,-1.127000019840896261058560412493534386157989501953125f,0.86709998012483169294029039519955404102802276611328125f,-0.5918000467464328284705743499216623604297637939453125f,1.3033999760806567191906424341141246259212493896484375f,1.2602999526381497563676248319097794592380523681640625f,-0.05280004748404021341912084608338773250579833984375f,1.5142999618470671663317261845804750919342041015625f,-0.5603999832928179269941892925999127328395843505859375f,0.49559996506720771503040623429114930331707000732421875f,-0.20149998022615933646051189498393796384334564208984375f,-0.771599999444186668284828556352294981479644775390625f,-1.245200016792118002939560028607957065105438232421875f,-0.39910000275969503835682417047792114317417144775390625f,-0.8434000144138931176485129981301724910736083984375f,-0.12000000201165705870920419329195283353328704833984375f,0.24020000267326846454807309783063828945159912109375f,-0.33330000819712901627411838489933870732784271240234375f,0.6515000062286855353477221797220408916473388671875f,-0.335599988536536830707746048574335873126983642578125f,-0.099600027305633387353367425021133385598659515380859375f,0.1886000047937035350997803107020445168018341064453125f,-1.28720000746399154678556442377157509326934814453125f,-0.21289998066648874175399441810441203415393829345703125f,-0.79329998970106208044938966850168071687221527099609375f,0.418799991112947633808971659163944423198699951171875f,-0.1440000266432763798007954392232932150363922119140625f,0.778200005966425134573682953487150371074676513671875f,-0.0337999622046949443898711251677013933658599853515625f,-0.70669998632669450699950175476260483264923095703125f,-1.315999983042478671535491230315528810024261474609375f,0.976699962186813763764803297817707061767578125f,-0.6490999818980693891745659129810519516468048095703125f,0.39639994156211688647317714639939367771148681640625f,0.6404999852627515366521038231439888477325439453125f,0.1401000058256090807784488561082980595529079437255859375f,0.02980000589489950613142354995943605899810791015625f,0.80579999608993524429223498373175971210002899169921875f,0.464899987152218752139987145710620097815990447998046875f,0.754999982863664431675942978472448885440826416015625f,1.0511999811708931940046340969274751842021942138671875f,1.2744999672770500609431110206060111522674560546875f,-0.745199998836219013043091763393022119998931884765625f,1.688499935805797491639168583787977695465087890625f,-1.2398999852895735696023393757059238851070404052734375f,0.781199969138204775020994929946027696132659912109375f,-0.7413999793365599888517181170755065977573394775390625f,0.21660000475272533915216399691416881978511810302734375f,0.256599991695582563266953002312220633029937744140625f,0.58739999421164412751750205643475055694580078125f,0.198699967610836747411440228461287915706634521484375f,0.4826999869525432362848960110568441450595855712890625f,0.9213999649941919667384127023979090154170989990234375f,0.041300002326071061364842762486659921705722808837890625f,0.937999973416328458597490680404007434844970703125f,0.0259999843835829747007437617867253720760345458984375f,1.6530999685689806710797711275517940521240234375f,0.4587999868661165603356266728951595723628997802734375f,1.431699944007397196088504642830230295658111572265625f,0.4043999662756920798756254953332245349884033203125f,1.128099964284896561395044045639224350452423095703125f,0.8127999650061130676448328813421539962291717529296875f,-0.01799999271333219130752922865212894976139068603515625f,0.07119999823272227246206966810859739780426025390625f,-0.11280003489255907567212489084340631961822509765625f,0.42259997686147698914282955229282379150390625f,1.215800006818771095140618854202330112457275390625f,0.356500014424323552475470933131873607635498046875f,0.266300014060735890097930678166449069976806640625f,-0.1017000098705285182631996576674282550811767578125f,-0.315100020223856081003077633795328438282012939453125f,-0.23739999244213105811240893672220408916473388671875f,-0.9775000014901162881386653680237941443920135498046875f,0.3154000405132768580784841105923987925052642822265625f,-0.4176000035718081537794432733790017664432525634765625f,-0.5466000077515840160913285217247903347015380859375f,-0.2527000050574546019532817808794789016246795654296875f,-0.24100000852346425972427823580801486968994140625f,-0.400599997067451862875486767734400928020477294921875f,0.43430007360577604913487448357045650482177734375f,-0.9025999471336605406435182885616086423397064208984375f,-0.718899954861402790129432105459272861480712890625f,-0.31399999952316293150289538971264846622943878173828125f,-0.1361000243887302563194907634169794619083404541015625f,1.224799969163537838312549865804612636566162109375f,0.42759998524338005410072582890279591083526611328125f,-0.124000013232231420801099375239573419094085693359375f,0.37509999794661996741496068352716974914073944091796875f,0.320100017392636271296169070410542190074920654296875f,0.15429999448060993660902795454603619873523712158203125f,1.1249000289499750948607470490969717502593994140625f,-0.10189996623843899925532241468317806720733642578125f,-0.22419998155832399078235539491288363933563232421875f,-0.4113999890595672059845355761353857815265655517578125f,-0.20869999964833230166050270781852304935455322265625f,0.56060000660419451179450334166176617145538330078125f,-0.62879999737143510429859816213138401508331298828125f,-0.90060001435279790626964313560165464878082275390625f,1.061799982261657504523100215010344982147216796875f,0.60609994761049801326180386240594089031219482421875f,0.20689998609423643216587151982821524143218994140625f,0.237700011464953231410390799283050000667572021484375f,-0.23290002278089527720794649212621152400970458984375f,-0.4497999896526341245817093295045197010040283203125f,0.3351000214159487455845010117627680301666259765625f,-0.0466999946713446689727788907475769519805908203125f,-0.13200000371038900226494661183096468448638916015625f,-0.5235999794244765581652245600707828998565673828125f,-1.112999974906444311528730395366437733173370361328125f,-1.66399998015165362375000768224708735942840576171875f,-0.25359998694956320530735638385522179305553436279296875f,0.5230999855190512359826016108854673802852630615234375f,0.1027999961867929190617587664746679365634918212890625f,-0.7065000073462723850781230794382281601428985595703125f,-0.1643000151559714794302635709755122661590576171875f,-1.42199998426437357323948162957094609737396240234375f,-0.5090999932602049415919509556260891258716583251953125f,0.72850005874037737640946943429298698902130126953125f,-0.5290999748572706717908431528485380113124847412109375f,-0.2399000026121733686323977963184006512165069580078125f,-0.2436999814689162491276874789036810398101806640625f,-0.3198000185236333781091389028006233274936676025390625f,0.8896999368697413590467704125330783426761627197265625f,0.45409998215883973227136038985918276011943817138671875f,-0.97880000154376001209044488859944976866245269775390625f,0.0288999944239854145155987907855887897312641143798828125f,-0.575099943035840777127987166750244796276092529296875f,-0.31389999282211056286229222678230144083499908447265625f,1.590200008261202668791156611405313014984130859375f,0.428600026512146126123070644098334014415740966796875f,1.175199995781481820955605144263245165348052978515625f,-0.64880000050067909711515312665142118930816650390625f,0.177300020349025810872944930451922118663787841796875f,0.324900021648407033580951974727213382720947265625f,0.852799994584918064077783128595910966396331787109375f,-0.03740001367628498218920185536262579262256622314453125f,-0.326200017492473126612395617485162802040576934814453125f,-0.069999984242021806579714393592439591884613037109375f,-0.6623000093221662609721533954143524169921875f,-0.4096999946117403368361920001916587352752685546875f,0.72029999436140013102658485877327620983123779296875f,-0.4118999665737153037525786203332245349884033203125f,-1.418099981123209119004968670196831226348876953125f,-1.734399996674060862034139063325710594654083251953125f,-0.0628999970018859766440755265648476779460906982421875f,-0.2850999869942665299049622262828052043914794921875f,1.76560002195239018618622139911167323589324951171875f,0.50079995459020099080049703843542374670505523681640625f,0.1556000059336424268252585534355603158473968505859375f,-0.345800004844367681045014251139946281909942626953125f,-0.09430001441091295788510251441039144992828369140625f,-1.01770000557899464865840855054557323455810546875f,1.5298999710589644696057121109333820641040802001953125f,0.99029999063611029441744904033839702606201171875f,-1.05100000122189474183187485323287546634674072265625f,-0.2840000065267085371800703796907328069210052490234375f,0.010499971777199856859397186781279742717742919921875f,0.86950000069290400706023547172662802040576934814453125f,-0.343700063611566353216630886890925467014312744140625f,-0.057000012427568247375120336073450744152069091796875f,-1.02719999952912299789886674261651933193206787109375f,-0.36210000866055491730577386988443322479724884033203125f,-0.771999980539083363595409537083469331264495849609375f,0.20759999463111167283813074391218833625316619873046875f,0.45859997301697763560923704062588512897491455078125f,0.0005000203177332185333625602652318775653839111328125f,0.32460003458857489277988861431367695331573486328125f,-0.2215999902427203682719891730812378227710723876953125f,0.70319997787773569353220182165387086570262908935546875f,-0.26339999001324176930438625277020037174224853515625f,-0.2205999632418151890078661381267011165618896484375f,-1.11509999474287013754292274825274944305419921875f,1.349399979537725169365103283780626952648162841796875f,0.6118999883294105046616095933131873607635498046875f,-0.056299994391202812238361730123870074748992919921875f,-0.8675999966800211904427442277665250003337860107421875f,-0.30850001828372508594355849709245376288890838623046875f,-0.344899985811114362377338693477213382720947265625f,-0.195399971520900717081303810118697583675384521484375f,-1.0323999702394004440719754711608402431011199951171875f,-0.4056000000849369069300109913456253707408905029296875f,-1.72520000595152378508601032081060111522674560546875f,-0.548899971140920595047418828471563756465911865234375f,-2.1670999952495098028748543583787977695465087890625f,0.2778999961078165714667420616024173796176910400390625f,-0.3001000211179256638160950387828052043914794921875f,-0.0431999858498575139975628189858980476856231689453125f,-1.6582000016450881840768261099583469331264495849609375f,-0.34960002892613417646572315788944251835346221923828125f,0.05290001197755334061412213486619293689727783203125f,0.17539997468739787489511172680067829787731170654296875f,-0.24759999384880071460202088928781449794769287109375f,-1.3031999959826467971879537799395620822906494140625f,-0.6470999930888414741758651871350593864917755126953125f,-0.89159997489452447183566619060002267360687255859375f,-0.2219000088185072439017631040769629180431365966796875f,-0.2921000481486313304912982857786118984222412109375f,0.1015999900192021687672649932210333645343780517578125f,-1.031899971565604534617932586115784943103790283203125f,-0.51349999667704093742059967553359456360340118408203125f,0.674900013005733878657110835774801671504974365234375f,0.33180001146793347555075115451472811400890350341796875f,0.9548999952733510099278646521270275115966796875f,-0.7709999973103405235264062866917811334133148193359375f,0.6778000085361302051722987016546539962291717529296875f,0.459099939355254615946932972292415797710418701171875f,1.1618000018566849718126832158304750919342041015625f,-1.1322000142812722600638153380714356899261474609375f,-1.659600012236833510570477301371283829212188720703125f,-0.4517999546945092870231519555090926587581634521484375f,-0.30240004213750404371552349402918480336666107177734375f,1.34939998244345193967319573857821524143218994140625f,0.929900036475062474750075125484727323055267333984375f,0.20319998428523522449040683568455278873443603515625f,-0.3043000128462913789206822912092320621013641357421875f,1.22919997619390475307454835274256765842437744140625f,-0.519500014960765810201337444595992565155029296875f,0.6537999775528913204425407457165420055389404296875f,-0.42739999922215954342163968249224126338958740234375f,-1.255000010952353139970227857702411711215972900390625f,-0.21909999508559696579368392121978104114532470703125f,-0.98179999798238271058181680928100831806659698486328125f,-0.384899997955560735363178537227213382720947265625f,-1.014699991184472960270568364649079740047454833984375f,-0.09429999880194639416686186450533568859100341796875f,-1.119899972325563819453009273274801671504974365234375f,-0.351800007444619478036429427447728812694549560546875f,0.1302999825894826102512524812482297420501708984375f,-0.41750000037252876783355759471305646002292633056640625f,0.9376000095322727378999161373940296471118927001953125f,-0.0660000296086078463986268616281449794769287109375f,0.9129000183477999907921685007750056684017181396484375f,0.103799973082542240643277864364790730178356170654296875f,-1.27429999988228104257359518669545650482177734375f,0.628400015307963411004266163217835128307342529296875f,-1.858799970698356673892703838646411895751953125f,0.360899999214708333550305496828514151275157928466796875f,-0.129299965870380229038261177265667356550693511962890625f,-0.69610001719892056204486152637400664389133453369140625f,-1.36949997852742821891069979756139218807220458984375f,-0.98530000382363802646779049609904177486896514892578125f,-0.80409999605417237233950800145976245403289794921875f,-0.6875000451132653456198795538512058556079864501953125f,-1.309599986495076695547368217376060783863067626953125f,-0.8821000261321658708624227074324153363704681396484375f,-1.394400015598535436112115348805673420429229736328125f,-0.2865999744474887478418168029747903347015380859375f,-0.17249999515712222120100705069489777088165283203125f,0.56429999993816004622004811608348973095417022705078125f,0.5298999679483478164598864168510772287845611572265625f,-0.5474000021092597645377963999635539948940277099609375f,0.811199948388338754057258483953773975372314453125f,0.398800001208484022896527676493860781192779541015625f,0.9359000059947371052970765958889387547969818115234375f,0.402599961662293281960955937393009662628173828125f,1.62619997684955563954645185731351375579833984375f,0.44779998642653284779413525029667653143405914306640625f,0.09059998317211892526756855659186840057373046875f,0.2530999839544296747817497816868126392364501953125f,-0.58909996714591983391073881648480892181396484375f,0.233799994745105710336474658106453716754913330078125f,-0.1056999725691980807074088488661800511181354522705078125f,-0.7532999854542314910332834188011474907398223876953125f,0.31679999386593682419999140620348043739795684814453125f,0.579500002071261288705272818333469331264495849609375f,0.132299995110183699065231621716520749032497406005859375f,0.0749999966844916432506806813762523233890533447265625f,-0.0582000034332275095749764659558422863483428955078125f,0.31559997933506955813953709366614930331707000732421875f,-0.9316999811112882667885060072876513004302978515625f,0.7320999898105855496766025680699385702610015869140625f,-1.966699951308965577112530809245072305202484130859375f,0.79199999499320938856072871203650720417499542236328125f,-0.8656999476656317416001229503308422863483428955078125f,0.40990001416057297856099239652394317090511322021484375f,-0.57629998695924899987375056298333220183849334716796875f,0.98569995971694612801883295105653814971446990966796875f,0.1882999690443269980022478193859569728374481201171875f,0.576799966317415258032497149542905390262603759765625f,0.073900000533461740470642098443931899964809417724609375f,0.0239999735951428316838018872658722102642059326171875f,-0.987600022608041694383018693770281970500946044921875f,0.45349996978044515572037198580801486968994140625f,0.997699971343577463045448894263245165348052978515625f,1.481199958744645162056485787616111338138580322265625f,1.6128999755814674710308054272900335490703582763671875f,1.9291999877423042786261930814362131059169769287109375f,0.3550999835669996773646062138141132891178131103515625f,-0.007399994360655692826611584678175859153270721435546875f,0.564900000432878801603919782792218029499053955078125f,-0.3081000042200088273602887056767940521240234375f,-0.03350000256299967560380537179298698902130126953125f,0.5900999813318257292138468983466736972332000732421875f,0.0955000072419640577692234728601761162281036376953125f,-0.57889999494552579761830202187411487102508544921875f,-1.27519994835853633929900752264074981212615966796875f,-0.696099990488588726833540931693278253078460693359375f,-0.9890999387219554517258757186937145888805389404296875f,0.0822999913290143325639292015694081783294677734375f,-0.537399998104572507173770645749755203723907470703125f,0.12220003119409117431359845795668661594390869140625f,-0.180300042641162949763611322850920259952545166015625f,0.1268999998033046860967942848219536244869232177734375f,0.9866000148296354410604180884547531604766845703125f,-0.1542000011160971073564951439038850367069244384765625f,-0.07179998062252934687421657145023345947265625f,-0.17359996847212233461732466821558773517608642578125f,-0.94350001925230042587600109982304275035858154296875f,-0.123599991382658647154357822728343307971954345703125f,-0.0216000072300435164152077049948275089263916015625f,0.190200001890957093308998082648031413555145263671875f,0.6749000044748176829756403094506822526454925537109375f,1.459700008097291057396205360419116914272308349609375f,1.416799986582994552719583225552923977375030517578125f,0.707999987199902758305825045681558549404144287109375f,0.4405000078752634440348856514901854097843170166015625f,0.166599996054171750614614211372099816799163818359375f,0.7045000420063745050214265575050376355648040771484375f,0.3619001076877115252727890037931501865386962890625f,0.695900020784139261564860134967602789402008056640625f,-0.32459996500015275699979611090384423732757568359375f,0.5219999686926597437519603772670961916446685791015625f,0.23549996641278259090768187888897955417633056640625f,-0.61969999699592559494476518011651933193206787109375f,0.1774000001907347634499956257059238851070404052734375f,-1.369599991002678596174746417091228067874908447265625f,-0.06589999507963650859920790026080794632434844970703125f,0.5583999965697525880159446387551724910736083984375f,-1.09990000421404854336060452624224126338958740234375f,-0.539699985969066542423888677149079740047454833984375f,0.18540001085996582474990646005608141422271728515625f,-1.68700000065565092199904029257595539093017578125f,0.126300025236606217049484257586300373077392578125f,-0.48649998179078135507324986974708735942840576171875f,0.018099978888034939927820232696831226348876953125f,0.23329998729824996672732595470733940601348876953125f,0.0087999868661163826999427328701131045818328857421875f,0.660400007134675615105834367568604648113250732421875f,0.075400008028745357790967318578623235225677490234375f,-0.00150000317394705096063489691005088388919830322265625f,-0.319300001372397002086245265672914683818817138671875f,0.96699998761713512607229858986102044582366943359375f,-0.478500000551342508092744765235693193972110748291015625f,0.562900016671419312075386187643744051456451416015625f,0.512100022965670031993568045436404645442962646484375f,-0.4942000033140185877300609718076884746551513671875f,0.31469999844878848715978847394580952823162078857421875f,-0.43550000657141285653750628625857643783092498779296875f,-0.386000042051076519555863342247903347015380859375f,0.67530001328587452302798510572756640613079071044921875f,1.2000999731361863265277634127414785325527191162109375f,0.64689994973540365652553418840398080646991729736328125f,-0.22120000292658803520140509135671891272068023681640625f,-0.19509998814910634035157954713213257491588592529296875f,-0.880599972331524316615514180739410221576690673828125f,0.11380007757693544334642865578643977642059326171875f,1.3397000428915024627940510981716215610504150390625f,0.7994999952539794119132920968695543706417083740234375f,0.461899983896314925146953100920654833316802978515625f,-0.713500016421079408246441744267940521240234375f,-0.895999969184398725730034129810519516468048095703125f,0.37260000711083396396361422375775873661041259765625f,0.490399993798136701883549903868697583675384521484375f,-0.7625999638229610244621881065540947020053863525390625f,0.099999995753169201861965120770037174224853515625f,0.59809997508823886658291257845121435821056365966796875f,-1.142999984741211338956645704456605017185211181640625f,0.210099988782406210674480462330393493175506591796875f,0.1622000039994713205260268296115100383758544921875f,-0.70329998759627354587564695975743234157562255859375f,-0.32130001059621537962129877996630966663360595703125f,1.193200003284216048626831252477131783962249755859375f,0.21819999732375150447438727496773935854434967041015625f,1.53839999843239727539412342594005167484283447265625f,-0.600200028415024622319151603733189404010772705078125f,-0.0739999800026414078502057236619293689727783203125f,-0.1813000036299230455227871061651967465877532958984375f,0.1715999795138836159225093069835565984249114990234375f,0.05989999832808978741383043598034419119358062744140625f,1.339400004273652911024328204803168773651123046875f,-0.192400000861286724074261655914597213268280029296875f,-0.56799999025464043000965830287896096706390380859375f,0.38979998626261913585722140851430594921112060546875f,0.01930004466027035459063654343481175601482391357421875f,1.433699976402521780727283839951269328594207763671875f,0.3708999982833851571939476343686692416667938232421875f,-0.7588000412181024056934575128252618014812469482421875f,-1.472900055563449495110717180068604648113250732421875f,-0.861500015281140729683784229564480483531951904296875f,-0.5311000106796621711424677414470352232456207275390625f,0.049399970000982396101107951835729181766510009765625f,-0.099899971841276080652960445149801671504974365234375f,-0.30999997511506105496437157853506505489349365234375f,-1.3107999973565334084213418464059941470623016357421875f,-0.31159999083876588343855473794974386692047119140625f,0.924599994802475233512950580916367471218109130859375f,0.094900010472536155958778181229718029499053955078125f,0.986299972635507504747920393128879368305206298828125f,0.43080001032054393572678918644669465720653533935546875f,0.1992000307694075100783948073512874543666839599609375f,0.196799994890391960478126520683872513473033905029296875f,0.4752999874696135496066062842146493494510650634765625f,-0.3744999966323376838772674091160297393798828125f,1.371799981814623148324017165577970445156097412109375f,0.6921999750912188886786680086515843868255615234375f,0.00840000603198998163634314551018178462982177734375f,-1.11240000436305930264779817662201821804046630859375f,0.6065999862194058067643709364347159862518310546875f,-0.7542999964177605765058842735015787184238433837890625f,0.845200018393993612875192411593161523342132568359375f,-0.2681000313028698567308083511306904256343841552734375f,1.61910001423358895777937505044974386692047119140625f,-0.5831999999687074165422018268145620822906494140625f,0.542900002217292954043159625143744051456451416015625f,-0.5114000149503350467483642205479554831981658935546875f,0.866100024724006711807078318088315427303314208984375f,0.43000001285225153413449561412562616169452667236328125f,0.199800000530481636218382845981977880001068115234375f,0.598100008764863133592371013946831226348876953125f,-0.18799998176097931779082728098728694021701812744140625f,-1.19419995317161120595983447856269776821136474609375f,-0.0950999684795730271247293785563670098781585693359375f,-0.53720003484487488520926490309648215770721435546875f,0.039700022737681361917339017963968217372894287109375f,0.0899000094294546414630531216971576213836669921875f,-0.0047000051915639939892344045802019536495208740234375f,-1.20019999417960665510918261134065687656402587890625f,-0.40389997408390054811633262943360023200511932373046875f,0.799700001689791495351755656884051859378814697265625f,0.39270003798902120006886207193019799888134002685546875f,1.47470002098679575652795392670668661594390869140625f,-0.428400034567713472455352530232630670070648193359375f,0.06010000599324694103842148251715116202831268310546875f,0.057200005713105017690622844384051859378814697265625f,-0.53469998313784561361217129160650074481964111328125f,1.19899997344613051808437376166693866252899169921875f,0.348300003093481080895799095742404460906982421875f,-0.41229999516606330445256389793939888477325439453125f,-0.4379999804198744239869256489328108727931976318359375f,0.690600001314282341269290554919280111789703369140625f,-0.11080003982484309954514856144669465720653533935546875f,0.27339999358952038477355017676018178462982177734375f,0.245899979805946689026541207567788660526275634765625f,0.22150003323703992830218112430884502828121185302734375f,-0.0395999954730275049996635061688721179962158203125f,0.380700035154074634125009879426215775310993194580078125f,-0.40229999985992936917256201923009939491748809814453125f,1.0157999854356052349402261825161986052989959716796875f,0.3742999961569906108849181691766716539859771728515625f,0.123599978977441349314858598518185317516326904296875f,0.1726000078186393782431196086690761148929595947265625f,-0.660199977564811302244152102502994239330291748046875f,0.23580000715404747513304073436302132904529571533203125f,-0.0868999973446131779297729735844768583774566650390625f,0.141900004087388509343981013444135896861553192138671875f,-0.25500000610947637369463336654007434844970703125f,-0.22339995328187978174128147657029330730438232421875f,-0.123299979847669494148476587724871933460235595703125f,0.5489999835044141462958577903918921947479248046875f,0.3732000082388517991915932725532911717891693115234375f,0.22730000574588782313867341144941747188568115234375f,-0.279200002419948578591402110760100185871124267578125f,0.3702999609828001581490752869285643100738525390625f,0.13020000259876240988887730054557323455810546875f,-0.8612999285280711347922988352365791797637939453125f,-0.589800003026426100660728479851968586444854736328125f,1.323799977552891249388267169706523418426513671875f,-1.42509998908042945231500198133289813995361328125f,-0.78969997225999843948329726117663085460662841796875f,-0.42719998865127539744435125612653791904449462890625f,-1.595000025331974047304584019002504646778106689453125f,0.92390000764876578642770255100913345813751220703125f,1.233200050669908787170925279497168958187103271484375f,-0.46339996069520761334814551446470431983470916748046875f,-0.9250000257417558824357683988637290894985198974609375f,0.053199989686906423713708136347122490406036376953125f,0.7242000290930270534062174192513339221477508544921875f,-0.9787999971106646768248538137413561344146728515625f,-0.537700029346347196934630119358189404010772705078125f,-0.249300014448165629943332533002831041812896728515625f,0.83789998612403893130107235265313647687435150146484375f,1.86159995224475860453594577847979962825775146484375f,-0.6274000257462264418251152164884842932224273681640625f,0.557499975226819710627523818402551114559173583984375f,1.378800014917552996251970398589037358760833740234375f,-0.02999997552484245932902240383555181324481964111328125f,-0.354500008851289738487366776098497211933135986328125f,-0.705800035093724797974346074624918401241302490234375f,0.8951000126242638632589887492940761148929595947265625f,-0.302800010566413480006531244725920259952545166015625f,-1.35059999811053277340988643118180334568023681640625f,-0.1148999803721906243225703292409889400005340576171875f,0.4542999822616575755773737910203635692596435546875f,-0.22299998459219949609178001992404460906982421875f,-0.45619996369928095258927669419790618121623992919921875f,0.21810000801980489182341216292115859687328338623046875f,-0.308600008071960030253677587097627110779285430908203125f,-0.156099992686510147255063429838628508150577545166015625f,-1.08789999789595537293962479452602565288543701171875f,-0.1730000196471810358644916050252504646778106689453125f,0.03209997822493282360056809920934028923511505126953125f,-0.53150000242888928436713058545137755572795867919921875f,-0.267300031077861210970780803108937107026576995849609375f,1.2766000116258862107088134507648646831512451171875f,-0.229200005586445321181798817633534781634807586669921875f,0.1093000188067556610604924571816809475421905517578125f,0.11769999686181564779730024383752606809139251708984375f,-1.081000006809830438214703463017940521240234375f,-1.117700014817714748716070971568115055561065673828125f,0.08960003846287722950592069537378847599029541015625f,0.253799992230534510184725149883888661861419677734375f,0.9258000097990031207473293761722743511199951171875f,0.48900002783536944406250768224708735942840576171875f,-1.47049999070167558556931908242404460906982421875f,-1.589600051873922392786653290386311709880828857421875f,2.061800041344761691419762428267858922481536865234375f,-0.0269999894052745104122692509918124414980411529541015625f,-0.198699982251226980878300309996120631694793701171875f,0.73210000445097633825497496218304149806499481201171875f,0.2882000270515678863603170611895620822906494140625f,0.31189998568445442828078739694319665431976318359375f,1.344400030797719747255314359790645539760589599609375f,-0.11910001050829865931746098794974386692047119140625f,0.326599995160103073743584900512360036373138427734375f,1.1658000134125352520442220338736660778522491455078125f,-1.301100007289647919606068171560764312744140625f,-0.70370002423524891099759770440869033336639404296875f,0.5286000152245164951381184437195770442485809326171875f,-0.439099980929494204673346757772378623485565185546875f,-0.22779999458491795305548066608025692403316497802734375f,0.241400011932849878348861238919198513031005859375f,-0.0750999824494120726825485689914785325527191162109375f,0.3072000313431024753896281254128552973270416259765625f,-0.1012999915972352393822575322701595723628997802734375f,-0.43150000682473166335739733767695724964141845703125f,0.0458999961972235137608322474989108741283416748046875f,-1.360999992877245379219175447360612452030181884765625f,-0.63599999887496216910420798740233294665813446044921875f,-0.09899994409084200697179767303168773651123046875f,-0.5386999747261398407971455526421777904033660888671875f,-0.48270000051260009144016294158063828945159912109375f,-1.0982999755263325170062671531923115253448486328125f,0.84070001356601675723823063890449702739715576171875f,-0.4123999405860896416697869426570832729339599609375f,-1.3754999836981294247806317798676900565624237060546875f,-0.638099987679719760080843116156756877899169921875f,-0.33910001274347312261170372948981821537017822265625f,0.97089997995495735949589288793504238128662109375f,1.4638000288128854009528367896564304828643798828125f,0.44129997576475066267676083953119814395904541015625f,-0.406500001162290569567403508699499070644378662109375f,-1.4683000520810487454781423366512171924114227294921875f,0.4153999875396490182311026728712022304534912109375f,-1.82290005332827576012277859263122081756591796875f,2.0429999945014714057833771221339702606201171875f,0.0441999468386171656675287522375583648681640625f,0.2700000058487059817480258061550557613372802734375f,-0.594300044697523066616895448532886803150177001953125f,-1.520300020326673884341062148450873792171478271484375f,0.04750001400709180643389117904007434844970703125f,-0.22530000050812926115639811541768722236156463623046875f,0.207300015580654584113062810502015054225921630859375f,-0.95750000294297932246223581387312151491641998291015625f,-0.61650000797957194009768500109203159809112548828125f,-1.745700011926889505531335089472122490406036376953125f,-0.08439999388009311598324302394757978618144989013671875f,-1.1071000597715379143437530728988349437713623046875f,-0.0836999833315612562500973581336438655853271484375f,-1.006399963504076122688957184436731040477752685546875f,-0.6097000049680472155699817449203692376613616943359375f,0.9651000261843207805867450588266365230083465576171875f,0.4297999896898863081418085130280815064907073974609375f,0.98419998826384524903687633923254907131195068359375f,0.2656999975100158284391227425658144056797027587890625f,-0.131799987179041178109173415577970445156097412109375f,0.48729995808079851205008026227005757391452789306640625f,-0.337800026100874450918354341411031782627105712890625f,-1.7375999872922900379990096553228795528411865234375f,-2.142899981802701692146229106583632528781890869140625f,0.1035000102370968733112022164277732372283935546875f,0.64150003185868254007573341368697583675384521484375f,0.732200002136826011422954252338968217372894287109375f,-1.030399971818923621214025843073613941669464111328125f,0.3057999730676410621299510239623486995697021484375f,1.218499960839748208485389113775454461574554443359375f,0.835199963632226261012192480848170816898345947265625f,-0.094099998587369615421494017937220633029937744140625f,0.79150000304356205038658345074509270489215850830078125f,0.7100999432966113023013576821540482342243194580078125f,-0.10380003983974450676441847463138401508331298828125f,-0.741500035323202411063903127796947956085205078125f,0.69769998687803802539519892889074981212615966796875f,-0.07710001640915808973630873879301361739635467529296875f,1.7554000073581936813837955924100242555141448974609375f,-0.369800002951919726257301590521819889545440673828125f,0.6135999622136356634882758953608572483062744140625f,-0.0256999914005404672678878341685049235820770263671875f,0.398999994531273660669512537424452602863311767578125f,-0.3872999981462952323596482528955675661563873291015625f,-0.13550002592429510883675902732647955417633056640625f,-0.44430000148415427219106277334503829479217529296875f,-0.6541999962732198969916908026789315044879913330078125f,0.3359000804632896741708236731938086450099945068359375f,-0.14139998518526564197372863418422639369964599609375f,1.629699970322846969139618522603996098041534423828125f,0.6318000047624108272970033794990740716457366943359375f,0.4048000052243467816737165776430629193782806396484375f,0.0548999888285994774861364930984564125537872314453125f,0.18889996648430928960493702106759883463382720947265625f,0.0080999971792102554157821714397869072854518890380859375f,-1.558999938130379181444595815264619886875152587890625f,0.86120003958344426564508466981351375579833984375f,0.163800021213293067745553344138897955417633056640625f,0.1682999943017957633628611802123486995697021484375f,-0.566000041157007416359192575328052043914794921875f,0.600799980890752038931168499402701854705810546875f,1.620700020569562749273018198437057435512542724609375f,-0.27669996814727770839681397774256765842437744140625f,-0.593399982190132124770798327517695724964141845703125f,-0.686999976515770161000773441628552973270416259765625f,-0.6991000002264977641885934644960798323154449462890625f,0.4223000090241428861048689213930629193782806396484375f,0.21170002834796919355397903927951119840145111083984375f,1.470600018227100402867790762684307992458343505859375f,0.4687000081047412169965582506847567856311798095703125f,0.208099986189604013731013765209354460239410400390625f,-1.775899972392619208250152951222844421863555908203125f,-0.7269999747276312263011277536861598491668701171875f,0.93070000898391025412337285160901956260204315185546875f,0.0461000212967395128771386225707828998565673828125f,0.8158999836429956520333917069365270435810089111328125f,0.313799968314171007222057596663944423198699951171875f,-0.46510001277327506219450015123584307730197906494140625f,-0.0991000242173669665390889349509961903095245361328125f,0.13619999305456875760000912123359739780426025390625f,0.55750000286847323227590322858304716646671295166015625f,-0.08029999063611104048732158844359219074249267578125f,-0.397499985806643696406581511837430298328399658203125f,0.2181000061571598536147575941868126392364501953125f,0.32360000322908177849967614747583866119384765625f,0.3447999640226360185124576673842966556549072265625f,0.2554000152558086877974119488499127328395843505859375f,-0.545199991720914933779340572073124349117279052734375f,-0.36479999248385441834585662945755757391452789306640625f,-0.358799980160593889877418405376374721527099609375f,-0.29870000735968371685657984926365315914154052734375f,0.91110000434666849056242199367261491715908050537109375f,-0.761699983756244147770075869630090892314910888671875f,-0.42969999148249637954677382367663085460662841796875f,1.03319999896287928464744254597462713718414306640625f,-0.2460999669075010221064303550519980490207672119140625f,1.0112000062048434045891553978435695171356201171875f,-0.09260000502467136129780556075274944305419921875f,0.705199975627660791133166640065610408782958984375f,0.650900004839896961783551887492649257183074951171875f,-0.0417999876260761737967186491005122661590576171875f,-0.4617999844223259486852839472703635692596435546875f,-0.77630003320872820182785289944149553775787353515625f,-0.36360000516623258715043220945517532527446746826171875f,-0.05070000376850369860903811058960855007171630859375f,-0.53170001057833438107280699114198796451091766357421875f,1.97140000954866412286037302692420780658721923828125f,-0.1724999916926022169860743815661408007144927978515625f,1.4546000114917756462773468228988349437713623046875f,-0.03550000292062749362997919888584874570369720458984375f,0.7355999839544296037274762056767940521240234375f,0.7857999918058518051822147754137404263019561767578125f,0.1275999805867673586590171908028423786163330078125f,-0.5317000197425485463753602743963710963726043701171875f,0.056899987882375990810857047108584083616733551025390625f,0.56690000591278078001522544582257978618144989013671875f,-0.087399980930984055849108926850021816790103912353515625f,1.81349999853968579799357030424289405345916748046875f,0.8297000230357050032381494020228274166584014892578125f,1.094299995225667121445667362422682344913482666015625f,0.590300003823637897681919639580883085727691650390625f,0.4180999712884430064008256522356532514095306396484375f,0.651699986214935744754939150880090892314910888671875f,-0.09820000849962173816720678587444126605987548828125f,-0.6254000172674658575289186046575196087360382080078125f,0.23840001046508529913836582636577077209949493408203125f,0.052300011035800064718159774201922118663787841796875f,-0.292500002626329635990742872309056110680103302001953125f,0.13300003242492675070707264239899814128875732421875f,-0.742700045365095373739450224093161523342132568359375f,1.5983000083088878540138466632924973964691162109375f,-0.04510000979304340518183380481787025928497314453125f,1.3834000349402426621736594825051724910736083984375f,0.767700020033121433016276569105684757232666015625f,0.0543999835610389226303595933131873607635498046875f,-0.58860001667737993358286985312588512897491455078125f,-0.0586999930918226109355373409925960004329681396484375f,0.62730004173219244822945483974763192236423492431640625f,0.852799995926022180725567523040808737277984619140625f,0.2945999964416021743573992353049106895923614501953125f,-0.246600014084577878747950308024883270263671875f,0.141199989962577543423094539321027696132659912109375f,1.304000006526708776988243698724545538425445556640625f,0.8021999727070336927425842077354900538921356201171875f,0.083799959596991602239768326398916542530059814453125f,-0.67709999040663249036953175163944251835346221923828125f,-0.258499967023730459203534337575547397136688232421875f,1.16939996389150646649568443535827100276947021484375f,-0.69659998085498830988626650650985538959503173828125f,-1.3518999853491777685121633112430572509765625f,0.5353999493181706981204115436412394046783447265625f,0.2449999882280824436975308344699442386627197265625f,0.3211000309824942888781151850707828998565673828125f,0.60939999309778170033524702375871129333972930908203125f,-1.563899976617097564712821622379124164581298828125f,0.55160001713931550160197048171539790928363800048828125f,-0.245899986250698143752657642835401929914951324462890625f,-0.6073000217273829548503272235393524169921875f,0.291199970181285738846099775400944054126739501953125f,0.0044999960735438992998069807072170078754425048828125f,0.5436000086307526402151779620908200740814208984375f,-0.52020002662688480388197831416619010269641876220703125f,-0.344199979881942563775254484426113776862621307373046875f,0.30840000733584116421326370982569642364978790283203125f,-1.073399994073807928174346670857630670070648193359375f,-0.3355000059008599322396548814140260219573974609375f,-1.12550004434585471102536757825873792171478271484375f,0.34260000956952529893584369347081519663333892822265625f,0.6404000326529153408472438968601636588573455810546875f,-0.6737999815389505986473750454024411737918853759765625f,-1.351500017888845217584048441494815051555633544921875f,-0.528000038944184790778990645776502788066864013671875f,0.02640000817030685453801197581924498081207275390625f,-1.152100004562735424684660756611265242099761962890625f,-1.04530001593083188282662376877851784229278564453125f,0.6290999822705980459858210451784543693065643310546875f,0.499700006532668883352243938134051859378814697265625f,0.98519995774626778484162059612572193145751953125f,0.1156000262737268968749049236066639423370361328125f,-0.55330002376884213077801177860237658023834228515625f,0.2584999976828694645547557229292578995227813720703125f,0.79110001686364428419295791172771714627742767333984375f,0.283199952098727447680204249991220422089099884033203125f,-0.395399995996057962432956855991506017744541168212890625f,0.7345000033006072737151725959847681224346160888671875f,0.197800024275481600710691054700873792171478271484375f,-0.12999998573213833452655308065004646778106689453125f,-0.2542000159427519889732138835825026035308837890625f,-1.2405000370442866852727092918939888477325439453125f,-0.153499999433755807132229165290482342243194580078125f,-0.1131000077962875938197839786880649626255035400390625f,0.183500015527010063021151609063963405787944793701171875f,0.003699985194206334426780813373625278472900390625f,0.4979000127971173750296429716399870812892913818359375f,-1.299900040274858614708364257239736616611480712890625f,-0.1079999900683759339603540183816221542656421661376953125f,-0.0604000236004591517513517828774638473987579345703125f,-0.68510002193748942289630576851777732372283935546875f,0.99870000247955292849155739531852304935455322265625f,0.326800008301436850643995057907886803150177001953125f,0.492699962812662040079203507048077881336212158203125f,-0.52289998370260004634246797650121152400970458984375f,-0.0599999916926028387109681716538034379482269287109375f,-0.157199996772408923106922884471714496612548828125f,-1.04990003695935030236796592362225055694580078125f,-0.208699996742605975441620103083550930023193359375f,-0.34189999864846498667247942648828029632568359375f,0.109400021633506039364647222100757062435150146484375f,0.2055000006854534877476226029102690517902374267578125f,-1.699000008314847764978594568674452602863311767578125f,-1.8283000187024482929842861267388798296451568603515625f,-0.4310000299811367341362711158581078052520751953125f,-1.1169999933913350442793444017297588288784027099609375f,-0.236099991159141209262628535725525580346584320068359375f,-0.05380000828653574984627994126640260219573974609375f,0.86199998776614605855428408176521770656108856201171875f,0.738500019624829473485760900075547397136688232421875f,-0.000700020346045437946713718702085316181182861328125f,-0.11860002017915227678912515330011956393718719482421875f,0.3106999951630833312066215512459166347980499267578125f,-0.36130002352297285739268772886134684085845947265625f,0.6424000228404995738173965946771204471588134765625f,1.384499989330768787709757816628552973270416259765625f,-1.21100001761317255244421176030300557613372802734375f,1.148300030176341746113166664144955575466156005859375f,-0.267200021098554341136832590564154088497161865234375f,2.1334000569939615132852850365452468395233154296875f,-0.653200032900274063507595201372168958187103271484375f,0.74430003590583826422744095907546579837799072265625f,-0.135300035674870589019747058046050369739532470703125f,-0.2359999617338177646530539277591742575168609619140625f,0.38330001523792756046304930350743234157562255859375f,-0.2758000071167945321803927072323858737945556640625f,-0.194499995850026419219602757948450744152069091796875f,-0.107700015860796138866817273083142936229705810546875f,0.13030000695288201928434546061907894909381866455078125f,-1.31060001278817583880709207733161747455596923828125f,-0.774800030630827141209238106966949999332427978515625f,-1.039299974736570764122234322712756693363189697265625f,-0.9896000004649163628300811978988349437713623046875f,-2.08930001154243871752669292618520557880401611328125f,-1.578400029873848087191845479537732899188995361328125f,-0.66949997305124941959775242139585316181182861328125f,1.2937000100791455903248561298823915421962738037109375f,1.9838000541076066252088594410452060401439666748046875f,0.872700031656027164217448444105684757232666015625f,0.65289999934881903786987322746426798403263092041015625f,0.410700003582239236976647589472122490406036376953125f,0.9849999960884445737718806412885896861553192138671875f,0.07949999022483833499563843361102044582366943359375f,2.333700003075599216373348099295981228351593017578125f,0.9701999739885334150812923326157033443450927734375f,1.1869999918639653202490080730058252811431884765625f,-0.86099999973177876455565638025291264057159423828125f,0.678299997431039525253027022699825465679168701171875f,-1.05180000312328303380127181299030780792236328125f,-1.1191000149786469730628368779434822499752044677734375f,0.314800008279085385964890519971959292888641357421875f,-1.129500015109777155686288097058422863483428955078125f,-0.586299980384111574238659159163944423198699951171875f,-0.89110003606751586602285897242836654186248779296875f,0.142300006230175579702290633576922118663787841796875f,-0.17440000084638551669513617525808513164520263671875f,0.735700005966425152337251347489655017852783203125f,-1.270599970468878847640326057444326579570770263671875f,0.5590000097304581583301796854357235133647918701171875f,-0.89250002317130583406878940877504646778106689453125f,0.408299987819790766163663420229568146169185638427734375f,-0.79599996463954469927415402707993052899837493896484375f,-0.326699980291724256797891712267301045358180999755859375f,-0.324300010350345946363859184202738106250762939453125f,-0.4830000025480984948700324821402318775653839111328125f,-0.0808000712662941822372886235825717449188232421875f,-1.20110001384615916464326801360584795475006103515625f,-1.6251999917581663179788620254839770495891571044921875f,-0.1381999755680560770798592784558422863483428955078125f,0.65760006198436027347042909241281449794769287109375f,1.670100014710426794550812701345421373844146728515625f,0.59949998068809506879262016809661872684955596923828125f,0.1734999755918982256019944543368183076381683349609375f,0.5064999970272179297126058372668921947479248046875f,0.24200002628564754036943895698641426861286163330078125f,0.27919996945113056963094777529477141797542572021484375f,-0.894099981749057359792232091422192752361297607421875f,-1.2673999870218335406235610207659192383289337158203125f,1.129499937921762953152438058168627321720123291015625f,1.1228000076234341264580507413484156131744384765625f,-1.121900004944205164747472736053168773651123046875f,-0.198400006106495663260602668742649257183074951171875f,0.74439997201263974346829854766838252544403076171875f,-1.008800018009543553176854402408935129642486572265625f,-0.6645999771445989523499520146287977695465087890625f,-0.333699980202317281197110787616111338138580322265625f,0.5433999886721370131681396742351353168487548828125f,-1.06929996847808350679542854777537286281585693359375f,-0.27889999598860715224191153538413345813751220703125f,-0.55970004575997600593240122179850004613399505615234375f,-0.414500006727873737677469989648670889437198638916015625f,-0.834499954164028334702152278623543679714202880859375f,0.38790002035945614355938459993922151625156402587890625f,0.533800015699863905638267169706523418426513671875f,-0.383700008812546844438884363626129925251007080078125f,0.597799985420703894334337746840901672840118408203125f,0.05309998674839722099250138853676617145538330078125f,-0.3271999879062175153876523836515843868255615234375f,-0.0454000095188618058017482326249592006206512451171875f,-0.53879998634457582085133253713138401508331298828125f,0.3209000042811036568224380971514619886875152587890625f,0.433200014124810905968843144364655017852783203125f,-0.92379995222091704221156760468147695064544677734375f,-0.3123999960556631538821648064185865223407745361328125f,1.0760999989449970826171920634806156158447265625f,1.819600010560453728913898885366506874561309814453125f,0.3671000204324723625859405728988349437713623046875f,-0.70570001233667112305880664280266501009464263916015625f,-0.957300005894899275205034427926875650882720947265625f,-1.21670000230818953212974520283751189708709716796875f,-0.556799993959069094984215553267858922481536865234375f,0.43000003740191505841039543156512081623077392578125f,0.351800002191960725639319207402877509593963623046875f,-0.1270000380948184126594924236997030675411224365234375f,-0.39709995486736371361757846898399293422698974609375f,-0.737199967938661604449634978664107620716094970703125f,-0.805700006264448376214204472489655017852783203125f,-0.300799965840578664710847078822553157806396484375f,0.92799997982382809169621395994909107685089111328125f,0.0990999765336511018176679499447345733642578125f,0.43449998113513021991138884914107620716094970703125f,0.756799993139505300376868035527877509593963623046875f,-0.718399998432397435266238971962593495845794677734375f,-0.458800011304020927127567119896411895751953125f,0.52819999222010360906409687231644056737422943115234375f,-0.3152999642610547770260609468095935881137847900390625f,0.9391000212371347632966944729560054838657379150390625f,-1.1212999870896336318537578335963189601898193359375f,0.51009999891519530734740328625775873661041259765625f,-0.8201000066637995633556101893191225826740264892578125f,0.37400000477582195923531571679632179439067840576171875f,0.21979999248385428955998577293939888477325439453125f,-0.68850002703815693649858076241798698902130126953125f,-1.01490001091957093848350268672220408916473388671875f,0.64930002446919699021776750669232569634914398193359375f,0.2283000149399041500686280414811335504055023193359375f,-0.090100010351836790079005368170328438282012939453125f,0.12990004865676230139115432393737137317657470703125f,2.00900002619624107325080331065692007541656494140625f,1.203199999745190762467927925172261893749237060546875f,1.270300003004074529400213577901013195514678955078125f,0.7771999874219293058530411144602112472057342529296875f,1.2020999624669552741096367753925733268260955810546875f,0.41769997324347485356810238954494707286357879638671875f,-1.36989999870061840425705668167211115360260009765625f,-0.6636999818786974980611148566822521388530731201171875f,0.534400004981458121022797058685682713985443115234375f,-0.02410002421736745503721977001987397670745849609375f,-0.5674000081628560732127652954659424722194671630859375f,0.393400020374357506369733528117649257183074951171875f,-0.059000033348798552879088674671947956085205078125f,-0.96869998385310207567044926690869033336639404296875f,-0.182699996936320996354652379523031413555145263671875f,-0.77780001343488702758577346685342490673065185546875f,0.12159999575614932609823881648480892181396484375f,0.1168000032722951164743108165566809475421905517578125f,-0.220399994766711682103732528048567473888397216796875f,-0.97690002126097663648351954179815948009490966796875f,0.836699988822638740515458266600035130977630615234375f,0.7394000288978219703750482949544675648212432861328125f,-0.14100000001117585224363892848487012088298797607421875f,-1.2914000473976141591947452980093657970428466796875f,0.8254000237867236133837423039949499070644378662109375f,-0.38940001530051215894445704179815948009490966796875f,0.07790000001937169571419872227124869823455810546875f,0.5479000232279300774962393916212022304534912109375f,-0.5759000293523077740331927998340688645839691162109375f,-0.60090000185966463419617866748012602329254150390625f,0.9407000454172498304927785284235142171382904052734375f,-0.09070001684427264621035646996460855007171630859375f,-0.339500011533498735616376507095992565155029296875f,-1.067500010281801525735545510542578995227813720703125f,-0.295400005756318417837746892473660409450531005859375f,-0.015800002087652753601787480874918401241302490234375f,0.64710001074671652521175246874918229877948760986328125f,0.2253000227481126438444647419601096771657466888427734375f,0.57599997112154976974807141232304275035858154296875f,0.766300009180605368186434134258888661861419677734375f,0.04030001000761895824098246521316468715667724609375f,-0.26599998900294286041656732777482829988002777099609375f,0.5548999978065491944789755507372319698333740234375f,0.883700009632110816681915821391157805919647216796875f,0.07799997818469961430309922434389591217041015625f,-0.1050999945193529239162444355315528810024261474609375f,0.051900006657839004020615902845747768878936767578125f,0.718600020141899609171787233208306133747100830078125f,0.519999984353780586587845391477458178997039794921875f,1.3734999955594540121950331013067625463008880615234375f,-0.541500026866793771063157691969536244869232177734375f,-0.126900025656819792629903531633317470550537109375f,1.364900021201372215529090681229718029499053955078125f,1.13839998725652602473701335838995873928070068359375f,0.47990002015829080761477598571218550205230712890625f,-0.25410001780986846142695867456495761871337890625f,1.30100001358985917221389172482304275035858154296875f,0.96779997141361295831529787392355501651763916015625f,-0.10240004027485838378197513520717620849609375f,1.7076000264450907906166321481578052043914794921875f,0.28550000564008970460605496555217541754245758056640625f,1.0078999821007250670135135806049220263957977294921875f,-1.57010002722740171776649731327779591083526611328125f,-1.60430001720488046856871733325533568859100341796875f,-1.3554000008016817702127809752710163593292236328125f,-0.69959997319579070307327128830365836620330810546875f,-0.723400010353326905487847398035228252410888671875f,-1.094099988901614750602675485424697399139404296875f,-0.50519995063096267973179465116118080914020538330078125f,0.153200016695260732291217209422029554843902587890625f,-0.8061000047937039170165007817558944225311279296875f,0.81490000879615553941448524710722267627716064453125f,0.180599959293008538452340872026979923248291015625f,0.020699976201355330118758502067066729068756103515625f,-0.12639995843768048189303954131901264190673828125f,-0.1927000085964800657478690482093952596187591552734375f,0.7766000066339966423356599989347159862518310546875f,0.56629999487549032011202143621630966663360595703125f,0.5250000233948224348523581284098327159881591796875f,0.1620000114217401421257136462372727692127227783203125f,0.546999956846236923269088947563432157039642333984375f,0.26289998664557934215935119937057606875896453857421875f,-0.9380999759078019195612796465866267681121826171875f,-0.76889998689889882399484122288413345813751220703125f,0.12259999384880071460202088928781449794769287109375f,0.7216999705314635082942231747438199818134307861328125f,-0.37499997891485692402824270175187848508358001708984375f,-1.13290000244081046076871643890626728534698486328125f,-0.3280999833583830849192963796667754650115966796875f,1.156899951374530388648054213263094425201416015625f,0.394100020492076819067506221472285687923431396484375f,0.097099985265732158268292550928890705108642578125f,-1.0517000085294245792510992032475769519805908203125f,-0.86300001484155597353264965931884944438934326171875f,-1.280000026822090131162212855997495353221893310546875f,-0.4663000098884106936480975491576828062534332275390625f,-1.171299985897541251489428759668953716754913330078125f,-0.1060999827027320208117089350707828998565673828125f,-0.452000019840895550515824652393348515033721923828125f,-0.4354999964386223609835724346339702606201171875f,-0.404300016422569630236694138147868216037750244140625f,2.18689997700452831708162193535827100276947021484375f,0.552500010840594857342011891887523233890533447265625f,1.57149998879432661880173327517695724964141845703125f,0.64289999722540336346554568081046454608440399169921875f,1.0567999777913090753145297640003263950347900390625f,-0.7539999805614356898075811841408722102642059326171875f,-0.545800003316998516567082333494909107685089111328125f,0.2258000135987996426223389789811335504055023193359375f,-0.139600006201863247046190963374101556837558746337890625f,0.81780001254081735595491409185342490673065185546875f,0.12199999104440184538589164731092751026153564453125f,1.8315999798119069197355202049948275089263916015625f,0.62379999368339777987557681626640260219573974609375f,0.89419995667338358913411866524256765842437744140625f,-0.00219999978989360300829503103159368038177490234375f,0.027999999493360849811551815946586430072784423828125f,-0.26899999106675365201368776979506947100162506103515625f,0.20550004911422714570790049037896096706390380859375f,-0.9069999935030939042235331726260483264923095703125f,-0.5821999995291236018601921387016773223876953125f,0.17219999684691433827765649766661226749420166015625f,-1.21559998931884738482267493964172899723052978515625f,1.523199968042970198212060495279729366302490234375f,0.7513999944612381742814477547653950750827789306640625f,-0.01560001699775437344897000002674758434295654296875f,-1.224099993222952154070526376017369329929351806640625f,0.06860001153647932170542844687588512897491455078125f,-0.45250000707805160260477350675500929355621337890625f,0.453799968686700250231069730944000184535980224609375f,-0.012899983441829743213702386128716170787811279296875f,-0.47339999187588688300820649601519107818603515625f,0.2626999738395212613539797530393116176128387451171875f,2.1133999979853630435400191345252096652984619140625f,-0.941599979364871497722333515412174165248870849609375f,1.782199982467293608578984276391565799713134765625f,1.0679999890998004286046807465027086436748504638671875f,1.26479997858852133418849916779436171054840087890625f,-0.070800000970065557481802898109890520572662353515625f,0.89559998387992367785415126490988768637180328369140625f,-0.66590000435560947611435267390334047377109527587890625f,1.048399986138939699031880081747658550739288330078125f,0.63969998406916872557559372580726630985736846923828125f,0.0442999933257699485267266936716623604297637939453125f,1.6016999828249216530906551270163618028163909912109375f,0.368500011280179062822526248055510222911834716796875f,1.136299975951015728270476756733842194080352783203125f,0.74719998216927052681057830341160297393798828125f,0.818199945914745985220406510052271187305450439453125f,0.338199999186396649264452207717113196849822998046875f,0.2688000103354450942561015835963189601898193359375f,-0.278999997027218338274678899324499070644378662109375f,1.111099981808662651161512258113361895084381103515625f,0.54409999951869247869495893610292114317417144775390625f,0.7653999929785726408226764760911464691162109375f,0.06670001687407545176711209933273494243621826171875f,0.366599997395276933076502245967276394367218017578125f,-0.1614999670386314534198390902020037174224853515625f,-0.936299984072148649971722988993860781192779541015625f,-0.6281000039592392614196114664082415401935577392578125f,0.0856999784924090679538721815333701670169830322265625f,1.2035000084675846476756078118341974914073944091796875f,0.500800016579031836982949243974871933460235595703125f,-1.041399987122417325480228100786916911602020263671875f,-0.02399999218434079040207507205195724964141845703125f,0.0461999764025211590023900498636066913604736328125f,-0.56449998724460581644990497807157225906848907470703125f,0.3922999877154824677205624539055861532688140869140625f,1.3386000037133694018365304145845584571361541748046875f,0.6104999971091746857609905418939888477325439453125f,-0.6254000265061854246795292056049220263957977294921875f,0.419100021572411496340038183916476555168628692626953125f,-0.1686999927379189134768466828973032534122467041015625f,-0.057300001294165792131707348744384944438934326171875f,-0.05490002149939510189824432018212974071502685546875f,-1.6217999938845630847339407409890554845333099365234375f,-0.19880000202804792852617765674949623644351959228515625f,0.199599984222650572718293915386311709880828857421875f,0.0333000083088876852599469202687032520771026611328125f,0.50380001286864295462208929166081361472606658935546875f,-0.17739999244213056073249390465207397937774658203125f,-0.662900001397729088381538531393744051456451416015625f,0.36490004413053334086924905932391993701457977294921875f,0.00270001960471277779873844337998889386653900146484375f,-0.6809999953359360258531296494766138494014739990234375f,-0.05340001841858044162592022985336370766162872314453125f,-0.48889997531324647095374302807613275945186614990234375f,-0.3686999991267920773196919981273822486400604248046875f,0.29919998371899136913043548702262341976165771484375f,0.399699996474385255140759909409098327159881591796875f,0.2931000013142821547518224178929813206195831298828125f,0.2308999903857706925691672950051724910736083984375f,0.538599982404708743644050628063268959522247314453125f,-0.89179998375177405023350729607045650482177734375f,-0.346600005441903835645689468947239220142364501953125f,0.094800025415420208219074993394315242767333984375f,0.86020000408887842269223256153054535388946533203125f,-0.872699962067604584348146090633235871791839599609375f,-0.4289999941587445686508317521656863391399383544921875f,0.56260000129938081414593398221768438816070556640625f,0.112599999436735931368502861005254089832305908203125f,-1.1868999862432474401430226862430572509765625f,-0.62220000325441304767082328908145427703857421875f,-0.801300003778934399889521955628879368305206298828125f,0.577499980628490394707341692992486059665679931640625f,-1.37380001331567758171559034963138401508331298828125f,0.2024000008612871770452557029784657061100006103515625f,-1.2830000041127203047608418273739516735076904296875f,1.1663999669313434015549546529655344784259796142578125f,-0.383900008952617444180077654891647398471832275390625f,0.1692000015258787737337797807413153350353240966796875f,-0.26500002130866018301702524695429019629955291748046875f,0.3054000109344718882908864543423987925052642822265625f,-0.783899983620643947546113849966786801815032958984375f,-0.4573999812662601360813141582184471189975738525390625f,-0.19640001398175954694380607179482467472553253173828125f,-1.2551000087499613044883517432026565074920654296875f,-0.1518999996542931274490229043294675648212432861328125f,-1.472499970644712607992232733522541821002960205078125f,-0.12140000451952215332340756503981538116931915283203125f,-0.332400020158290399052702923654578626155853271484375f,0.08650000246614186760751863403129391372203826904296875f,-1.0953999772578482119200771194300614297389984130859375f,-0.243999997064471063623614099924452602863311767578125f,-0.05869999696612371309356603887863457202911376953125f,-0.3474999944120644723710711332387290894985198974609375f,0.518299985808134078268949451739899814128875732421875f,0.0443000546813010487312567420303821563720703125f,0.407700010943413015951364286593161523342132568359375f,0.7482999326109887050506586092524230480194091796875f,0.5416999882638453556182867032475769519805908203125f,0.725200005578994222332767094485461711883544921875f,1.0300000068545340781156482989899814128875732421875f,0.152100015068054261746510746888816356658935546875f,-0.790399995362758289729754324071109294891357421875f,-0.1488999979257581429692436358891427516937255859375f,0.98589997533559792231017127051018178462982177734375f,1.1479999731183045952320753713138401508331298828125f,-0.9653999993860720163496580425999127328395843505859375f,-0.2377000003635882929842182420543394982814788818359375f,-1.6433999853938814350584607382188551127910614013671875f,-0.48010002558827391538898154976777732372283935546875f,-0.3964000171855091014805339000304229557514190673828125f,-1.1940000071227547806529400986619293689727783203125f,0.581299999383091847704463361878879368305206298828125f,-0.667500007450580579160259730997495353221893310546875f,0.4002000053554775060860038138343952596187591552734375f,0.17609999328255643291640808456577360630035400390625f,0.0551999919816845707742913873516954481601715087890625f,1.57660003941655180170755556900985538959503173828125f,-1.186900002075731652695367301930673420429229736328125f,-0.7541999964594843230969445357914082705974578857421875f,0.63539997211694743128873597015626728534698486328125f,0.603999978736042830718133700429461896419525146484375f,-0.429900006002187584641660578199662268161773681640625f,0.57379999316185703595039058200200088322162628173828125f,-0.2875000365078452801981256925500929355621337890625f,0.6656000108137727355739343693130649626255035400390625f,0.30030001417994522938670343137346208095550537109375f,0.3218999871000647328145305436919443309307098388671875f,1.241800007295609020729898475110530853271484375f,0.040200007590651676281368054333142936229705810546875f,0.201300049525498625513364459038712084293365478515625f,-0.30069999546110592891778878765762783586978912353515625f,-0.057099990741908579483521180009120143949985504150390625f,-0.296300023336708573395981147768907248973846435546875f,-0.44869999338984456471735029481351375579833984375f,1.3100999512314803752133229863829910755157470703125f,1.07689994556307766515601542778313159942626953125f,1.3583999687790875299242543405853211879730224609375f,1.099400016790628598073453758843243122100830078125f,-0.9136999828100211828996179974637925624847412109375f,-0.6545000145137311875487284851260483264923095703125f,-0.28209993728399229695469330181367695331573486328125f,-0.840700008946657195707530263462103903293609619140625f,1.384299995300173780066188555792905390262603759765625f,-0.226499990656972016722647822462022304534912109375f,-0.20820001930296416770005407670396380126476287841796875f,0.021299970698356762710545808658935129642486572265625f,-0.17729997475147296626118986750952899456024169921875f,-0.57369996567368541917630864190869033336639404296875f,-0.6532999948978426285606246892712078988552093505859375f,0.055999979168176228228048785240389406681060791015625f,0.53640002504587158682625158689916133880615234375f,0.34410000797510154058045372948981821537017822265625f,-0.33590003076791763447772609652020037174224853515625f,-0.002300011408328916928667240426875650882720947265625f,-0.5263000194251541419276918531977571547031402587890625f,-0.314700009475648467827113563544116914272308349609375f,-1.0201000199258327683082825387828052043914794921875f,-0.05439999563097941415179548130254261195659637451171875f,0.510400035744905888890343703678809106349945068359375f,-0.21599999514967205271886996342800557613372802734375f,0.74149998298287389530969448969699442386627197265625f,-0.169799996544420128685715098981745541095733642578125f,0.663200042474269668701936097932048141956329345703125f,-0.2233000051051383660905003125662915408611297607421875f,-0.01180001951456066677792478003539144992828369140625f,-0.26489998949915161130519436483155004680156707763671875f,0.23199997744709144331665129357133992016315460205078125f,-0.037900010189413801953151050838641822338104248046875f,0.2930999830976130926529776843381114304065704345703125f,-0.06119999063313041887113286065869033336639404296875f,1.421400085321069983734787456342019140720367431640625f,1.56140004158616108043133863247931003570556640625f,1.962200051012635615421686452464200556278228759765625f,0.4883000391542910989528536447323858737945556640625f,0.92320005409717520450385563890449702739715576171875f,0.859999997913837521679170095012523233890533447265625f,0.91729995117038487517646672131377272307872772216796875f,0.061299990367889023445968632586300373077392578125f,0.0469999790862203337127311897347681224346160888671875f,-0.2025999893784520278217087252414785325527191162109375f,-1.37390000574886794737494710716418921947479248046875f,-1.334400002261995954455642277025617659091949462890625f,-0.290999993100762299746975259040482342243194580078125f,-0.460399998939036603218255550018511712551116943359375f,-1.276200045990943721108124009333550930023193359375f,-0.758999969124794393593447239254601299762725830078125f,-0.30239999020695673692671334720216691493988037109375f,0.25450001317262671562957621063105762004852294921875f,-1.19539999491572412892992360866628587245941162109375f,-0.673399992173910444392959107062779366970062255859375f,-0.30329998785704359587356293559423647820949554443359375f,0.02520001627057799620246214544749818742275238037109375f,-0.48470001085400582496731658466160297393798828125f,0.439499989181756756106977945819380693137645721435546875f,0.2706999971747396926957662799395620822906494140625f,0.8013000149548055173198690681601874530315399169921875f,-1.6293999863922596915699614328332245349884033203125f,0.2697000001996754381394794108928181231021881103515625f,-1.271000003606081119045256855315528810024261474609375f,0.57520000952780225844662709278054535388946533203125f,-0.8678999902963635548758247750811278820037841796875f,0.1849000109195710894738340357434935867786407470703125f,-0.81109998735934463187646770165883935987949371337890625f,0.2496000272125005192691560296225361526012420654296875f,1.1963999974787233870898717214004136621952056884765625f,0.50589995931685027841240298585034906864166259765625f,0.25330002660006289971761361812241375446319580078125f,1.63259999638199815308325923979282379150390625f,1.107899972228705731680520329973660409450531005859375f,0.25079999445080769504556883475743234157562255859375f,-0.447900066776573790860993540263734757900238037109375f,-0.509100025707483805348374517052434384822845458984375f,-1.154600055971741312532685697078704833984375f,-0.5614999801516535882228708942420780658721923828125f,0.077299998630583122150028430041857063770294189453125f,-0.2825999825239182694502915182965807616710662841796875f,-1.151399989357590580851820050156675279140472412109375f,-0.18430002227127484371038690369459800422191619873046875f,0.476799981591105481726344805792905390262603759765625f,-0.10289999491572388023996609263122081756591796875f,-1.41479996685385689403346987091936171054840087890625f,0.918299995940923707848924095742404460906982421875f,-0.70319997079968477038391938549466431140899658203125f,0.78459997766613920333611531532369554042816162109375f,0.760600046390294881604177135159261524677276611328125f,-1.270399973011017369373121255193836987018585205078125f,0.551499992072582045921080862171947956085205078125f,0.8224000178858641429968656666460447013378143310546875f,-1.599199991393089437252683637780137360095977783203125f,0.98980000101476839358127790546859614551067352294921875f,-1.0713999968081713287659795241779647767543792724609375f,0.100000002831220680121759869507513940334320068359375f,-0.4385999974548815405483992435620166361331939697265625f,0.37509999261945481396907098314841277897357940673828125f,-1.378299976643920121688324798014946281909942626953125f,0.771900014480948026829310038010589778423309326171875f,-0.4742999892652031146411673034890554845333099365234375f,0.29110000110566625064478785134269855916500091552734375f,-0.7088999852970248038985801031230948865413665771484375f,0.96880000087320805590707095689140260219573974609375f,0.0854999779611824717306944876327179372310638427734375f,1.7507999498590829379196520676487125456333160400390625f,1.5238999557554719910257290393928997218608856201171875f,0.5510999822184441399031129549257457256317138671875f,-0.732400045564770341144367193919606506824493408203125f,-0.4742000053256749669827740945038385689258575439453125f,0.022299999673664583355048307566903531551361083984375f,1.449999979659914561835876156692393124103546142578125f,0.899199977274239614644102402962744235992431640625f,0.42520001064538970325656919158063828945159912109375f,0.83759998222589526761794331832788884639739990234375f,0.26589998598992803380269833724014461040496826171875f,-0.628899967005848825607472463161684572696685791015625f,-0.52419999780058834648599486172315664589405059814453125f,-0.8333999794706705710467531389440409839153289794921875f,0.8566999853208658866066116388537921011447906494140625f,-0.1136999723419547247971195247373543679714202880859375f,-0.4992999962687492132573652270366437733173370361328125f,-0.42299997937679323367632377994596026837825775146484375f,0.1111999722301961757153776488848961889743804931640625f,-0.34649996972084073920683522374019958078861236572265625f,0.350599983507394963311298852204345166683197021484375f,0.69720001655370011928170015380601398646831512451171875f,-0.62040002523958703850581741789937950670719146728515625f,-1.312900019949674312869092318578623235225677490234375f,0.80790003267154142374550929162069223821163177490234375f,-0.27100001038610965764519278309307992458343505859375f,-0.38910000577718040215557948613422922790050506591796875f,-0.082600031735002854560434570885263383388519287109375f,-0.497800009225308370819362835391075350344181060791015625f,0.9410000088959928898901807770016603171825408935546875f,0.3429000040799377035938277913373894989490509033203125f,0.41720000634640463932356624354724772274494171142578125f,0.130199985685944541469183377557783387601375579833984375f,-0.51000001609325396056959789348184131085872650146484375f,-1.63389999047517786578964660293422639369964599609375f,-1.2329999920427792137189726418000645935535430908203125f,0.46520002634748836012335004852502606809139251708984375f,0.10239999709874314515190008023637346923351287841796875f,-0.2871000008188187013757897148025222122669219970703125f,0.44750001586973697786930870279320515692234039306640625f,-0.765999980993568474474386675865389406681060791015625f,-0.287299945805967293921412419877015054225921630859375f,0.8704999779984354812967239922727458178997039794921875f,0.235899979321658737063671651412732899188995361328125f,0.1827999679863456794493004053947515785694122314453125f,1.139400010271370167203031087410636246204376220703125f,-0.06400001293420753967211567214690148830413818359375f,-0.7174000217974185744651549612171947956085205078125f,0.5903999883592125907938452655798755586147308349609375f,0.5327000026173893676428860999294556677341461181640625f,-0.49599999706819619671449572706478647887706756591796875f,-0.62590003510788061458214315280201844871044158935546875f,-0.209099984617531209352137011592276394367218017578125f,0.490699973668158317963161607622168958187103271484375f,0.32999998938292274264227899038814939558506011962890625f,0.3659000094220041798820375333889387547969818115234375f,-0.7346999735265971054332112544216215610504150390625f,0.5654999898076056297213654033839702606201171875f,-0.45760000133663403243389211638714186847209930419921875f,1.642399995869398221515211844234727323055267333984375f,-0.51700001458823674482800925034098327159881591796875f,1.174400001889467137772271598805673420429229736328125f,-0.3212000115692614787832326328498311340808868408203125f,0.9934999819993972636211765347979962825775146484375f,-0.3340000028759237959974370824056677520275115966796875f,0.069600021773576603578703725361265242099761962890625f,-0.20780001127421854345556084808777086436748504638671875f,0.2535000094920396573883181190467439591884613037109375f,0.8428000188738102860241951930220238864421844482421875f,-0.0157000086858871679140747801284305751323699951171875f,-0.6578000077724457383965273038484156131744384765625f,-1.065299986016750022344012904795818030834197998046875f,-0.834800003510713484189409427926875650882720947265625f,-0.2900999578624967600859463345841504633426666259765625f,-0.218400010502338037809977322467602789402008056640625f,-1.289799980562924996974061286891810595989227294921875f,-0.83399997471272957483279242296703159809112548828125f,-0.1997999879390006672252866337657906115055084228515625f,-0.728600009599328135578844012343324720859527587890625f,-0.615699995312094738864061582717113196849822998046875f,1.793499950557947197893327029305510222911834716796875f,-0.08270001437067975302852573804557323455810546875f,0.6516999837934971395725369802676141262054443359375f,1.4936999816179277189576168893836438655853271484375f,-0.6985999800205231480276779620908200740814208984375f,0.914599963361025114494395893416367471218109130859375f,0.834499964892864820598106234683655202388763427734375f,1.088299957048893151778656829264946281909942626953125f,-0.4193999653071169841922483101370744407176971435546875f,0.12459999979436420236567073516198433935642242431640625f,-0.0530999983340503245443642299505881965160369873046875f,0.4659000104278323561857177992351353168487548828125f,0.37179999299049359962054950301535427570343017578125f,-1.17649998134374644820354660623706877231597900390625f,0.3548000038459890870257140704779885709285736083984375f,0.8062000149965289352849140414036810398101806640625f,-0.3847000029191371961445611304952763020992279052734375f,1.63950001510977738661267721909098327159881591796875f,-0.34500000637024630378846268285997211933135986328125f,1.74610002517104145880466603557579219341278076171875f,0.18459997528195326577105106480303220450878143310546875f,0.06050000083446516185858854441903531551361083984375f,-0.0754999634698025889889549944200552999973297119140625f,-1.7097000366330146192694883211515843868255615234375f,0.6443000240221621321978773266891948878765106201171875f,0.2187000617861747375769709833548404276371002197265625f,1.7659999717175960398662937222979962825775146484375f,1.8387999841839082204586475199903361499309539794921875f,0.783900004333257793831535309436731040477752685546875f,0.8614000597283244520241396458004601299762725830078125f,0.81720000399947156921598434564657509326934814453125f,-0.210099990049004592407300151535309851169586181640625f,0.20509994124770170031979432678781449794769287109375f,-0.0640999848037957509205853057210333645343780517578125f,0.4629000674843783968981369980610907077789306640625f,0.5664999872669576053141327065532095730304718017578125f,0.757700007665157659886290275608189404010772705078125f,-0.6517000223875049069732767748064361512660980224609375f,-0.096300006014108152641028937068767845630645751953125f,-0.482299983096122986125919851474463939666748046875f,-0.76159998621940605545432845246978104114532470703125f,0.296299970996380057641772509668953716754913330078125f,0.22940000397562965162023829179815948009490966796875f,0.891399991518258882905456630396656692028045654296875f,-0.86820000581741307854599654092453420162200927734375f,0.475199982631206552241565077565610408782958984375f,0.20260001143217021279951950418762862682342529296875f,-2.09449998411536153497536361101083457469940185546875f,-0.22980000336170203212304841144941747188568115234375f,1.259600008064508447347407127381302416324615478515625f,0.39160000309497122739088581511168740689754486083984375f,0.4002000237584113495614701605518348515033721923828125f,0.360699999372660951024727182812057435512542724609375f,-0.985399983888864738901247619651257991790771484375f,-0.0064999838024382139423096305108629167079925537109375f,0.46189995852708864987334891338832676410675048828125f,-0.50119998880773708460623083738028071820735931396484375f,1.32690000144243214208472636528313159942626953125f,-0.2168000122502447002403869191766716539859771728515625f,0.431199994879960968319210223853588104248046875f,0.316599997209012384846715804087580181658267974853515625f,0.3706999778777360088355408151983283460140228271484375f,0.2675999810338023365829940303228795528411865234375f,-0.1919000158593056060141179841593839228153228759765625f,0.2082999986976385997650140780024230480194091796875f,0.2067999967902898195148964077816344797611236572265625f,0.69349999913573245891029728227294981479644775390625f,0.03919999396353990306351988692767918109893798828125f,0.86219997568726558512253177468664944171905517578125f,-0.2426000176534055530197520056390203535556793212890625f,0.61789999785870330928361227051937021315097808837890625f,-0.1814999965056778297167738855932839214801788330078125f,0.47079997709095433577886069542728364467620849609375f,0.81410001721382219130873636459000408649444580078125f,0.099899995422363385699782156734727323055267333984375f,-1.02019998531341560266127999057061970233917236328125f,-0.377700012731552181577399096568115055561065673828125f,-0.192799961280822884646113379858434200286865234375f,-0.13450000974535925024611060507595539093017578125f,0.2566000103592873671232155174948275089263916015625f,0.2284999961555005398849971243180334568023681640625f,-1.400400005197525388211943209171295166015625f,-0.468200004737079122918430584832094609737396240234375f,0.4061000143304458465109973985818214714527130126953125f,-0.5779999967180191600135685803252272307872772216796875f,0.706900010383128662994067781255580484867095947265625f,0.68729998965263361387911800193251110613346099853515625f,-0.5731999937474727691011366914608515799045562744140625f,1.15199998378008583443943280144594609737396240234375f,0.35419999467134477555418925476260483264923095703125f,0.00670002261102187190999757149256765842437744140625f,-0.6461999883234501584894360348698683083057403564453125f,0.1927999892950058580254335538484156131744384765625f,-0.709599974834918878485723325866274535655975341796875f,-0.0522000399112700907977568931528367102146148681640625f,1.0489000169992450661737848349730484187602996826171875f,0.44770000859647984814415622167871333658695220947265625f,-0.21800000452250234417306273826397955417633056640625f,-0.676599997991323309776134919957257807254791259765625f,-0.028799994465708689261873587383888661861419677734375f,-0.81979999218583099462875907192938029766082763671875f,-0.1761999887332319492116994297248311340808868408203125f,-1.005699977207183959393432814977131783962249755859375f,-0.4610000058785079790624195084092207252979278564453125f,0.09460003116130888400903131696395576000213623046875f,-0.417700005280971620180707759573124349117279052734375f,0.234500015463679878369163134266273118555545806884765625f,0.46020001526474951791811918155872263014316558837890625f,-0.441499985180795473382886484614573419094085693359375f,0.0117000096842649892181498216814361512660980224609375f,0.4803999882102012453088946131174452602863311767578125f,-0.6119000340759743750140842166729271411895751953125f,-1.00380000854730599968434034963138401508331298828125f,-0.40570002354979539660462251049466431140899658203125f,1.0175000093877315521240234375f,0.43279997200965869552646836382336914539337158203125f,-0.407800024163722962811107208835892379283905029296875f,0.081999971151351047637945157475769519805908203125f,2.00630006815195116587347001768648624420166015625f,-0.4056000083923354537773775518871843814849853515625f,0.360699995237589110530507241492159664630889892578125f,-0.2130999659612775598560574508155696094036102294921875f,-0.2256999613001944293699807531083934009075164794921875f,0.10110001131296098009215711499564349651336669921875f,-1.108500004835426988591962071950547397136688232421875f,-0.215699976462125686538229274447076022624969482421875f,-0.78349997752904865233603004526230506598949432373046875f,0.215800028872489946252244408242404460906982421875f,-1.76940003217607699781410701689310371875762939453125f,-1.58120000963211015942988524329848587512969970703125f,0.4367000088274488600603717713966034352779388427734375f,-0.9669999894797793871248359209857881069183349609375f,-0.6030000099241732414157013408839702606201171875f,-0.228999984063207817630569707034737803041934967041015625f,0.422699976670742660900259579648263752460479736328125f,-0.530700014050304869073215741082094609737396240234375f,0.3410000554621219759354744383017532527446746826171875f,-0.483700001622736353024123445720761083066463470458984375f,0.7949999924004078621919688885100185871124267578125f,-0.307800014589727055636103614233434200286865234375f,-0.377699961173533438341110013425350189208984375f,0.027499978318810125443860670202411711215972900390625f,0.028600006842612657464997028000652790069580078125f,-0.65120002036094692954293350339867174625396728515625f,-0.68550000008940603635210209176875650882720947265625f,0.4790000069364903634294705625507049262523651123046875f,0.3309999584555622931247853557579219341278076171875f,0.926800014094264046349280761205591261386871337890625f,1.3034000096455213846269316491088829934597015380859375f,0.38950003170594527102110760097275488078594207763671875f,-0.823899966484308077241394130396656692028045654296875f,-1.2444000294566155417896879953332245349884033203125f,0.83920000368654701361492698197253048419952392578125f,0.23599996881187035313587330165319144725799560546875f,-0.7354000015094876996357697862549684941768646240234375f,1.78489999363422402467449501273222267627716064453125f,2.07989998722672453368431888520717620849609375f,0.4552000331088901230458532154443673789501190185546875f,0.78480000388324222537761443163617514073848724365234375f,0.4443999824993307701248568264418281614780426025390625f,0.31450000128895083673086219278047792613506317138671875f,-0.60990000453069781460868625799776054918766021728515625f,-1.2851999862074852121196499865618534386157989501953125f,-2.03060000362396220197069851565174758434295654296875f,-0.4039999933391806852256422644131816923618316650390625f,1.3101999981284138119264071065117605030536651611328125f,-1.0671999948725103823932158775278367102146148681640625f,-0.0455000342130660317963020133902318775653839111328125f,0.4217000574424856207400580387911759316921234130859375f,0.79250001288950466804550387678318656980991363525390625f,-0.029300043263285446837329573099850676953792572021484375f,0.472299991533905283436212130254716612398624420166015625f,-0.67240001032352447740692014122032560408115386962890625f,-0.4462000011570754764989032992161810398101806640625f,-0.28769994344115357165492241620086133480072021484375f,-0.6358999979481099185107950688689015805721282958984375f,1.878499932266772365352380802505649626255035400390625f,0.6921999945744869986441472065052948892116546630859375f,-0.4315000125616794690586175420321524143218994140625f,0.47559999490678317446423761793994344770908355712890625f,-0.8019000530004500859604377183131873607635498046875f,-0.19149998074770024913959787227213382720947265625f,1.3642000231325628334388966322876513004302978515625f,0.22779994995593977336056923377327620983123779296875f,-0.81280002282261865076407048036344349384307861328125f,-1.0653999956607815846609810250811278820037841796875f,0.362700008149445085603446159439045004546642303466796875f,1.0045000083297492832201669443747960031032562255859375f,0.2122000000692902677013051970789092592895030975341796875f,0.002400014439970288293579869787208735942840576171875f,-0.84409999873638152934063327847979962825775146484375f,-1.0208999872565271971325273625552654266357421875f,-0.460099988633393852666131351725198328495025634765625f,-0.838900019383430173292026665876619517803192138671875f,-0.63659998487830160573963667047792114317417144775390625f,-0.39119999234676328114801435731351375579833984375f,-0.3802000143706802948173617551219649612903594970703125f,-0.83859998702406901571748676360584795475006103515625f,-0.043999985516071260516213214941672049462795257568359375f,1.194300026816129456364024008507840335369110107421875f,0.2156999958522618499046075157821178436279296875f,-0.1436000090219080593811895596445538103580474853515625f,0.606899965865910218099088524468243122100830078125f,0.23109997316598906991202966310083866119384765625f,-0.72749999884515992309985676911310292780399322509765625f,-0.6341000112533572430351114235236309468746185302734375f,0.38009998893141716536092644673772156238555908203125f,-0.0448000018343328587633322968031279742717742919921875f,-0.05719996704459173741952326963655650615692138671875f,-0.0847999872311948621472765807993710041046142578125f,0.84839998846724584691258996826945804059505462646484375f,0.4077000072367489291735864753718487918376922607421875f,-0.395600015842914654040640698440256528556346893310546875f,0.02440000354722136888252492781248292885720729827880859375f,0.19160002153515875988887273706495761871337890625f,-0.019399996636808136685203862725757062435150146484375f,0.69670002641081740790696130716241896152496337890625f,-0.3485999729424715809500412433408200740814208984375f,-0.41770000319480882211564676254056394100189208984375f,-1.768199980112909575069579659611918032169342041015625f,-1.135899996495246977445958691532723605632781982421875f,-0.66069992207288752439353629597462713718414306640625f,-0.35159998621940591334578130044974386692047119140625f,0.0512999945402139445604916545562446117401123046875f,-0.71890004471540436270515783689916133880615234375f,0.77699999678134901159864966757595539093017578125f,1.0226999997675416675946280520292930305004119873046875f,-0.98849994847178468404536033631302416324615478515625f,0.459699996101856189767431715154089033603668212890625f,-0.82720002240240564361783981439657509326934814453125f,1.28280000855475595500365670886822044849395751953125f,-0.891600021609664228350311532267369329929351806640625f,1.156099978753924784058426666888408362865447998046875f,0.880500015735626195834129248396493494510650634765625f,0.6661999851197004662850531531148590147495269775390625f,1.4134999890029433800009428523480892181396484375f,-0.9565999963149434126563619429362006485462188720703125f,-0.72539996854066846054820416611619293689727783203125f,0.61189999745637180250668052394757978618144989013671875f,-1.302699953871965465879156909068115055561065673828125f,0.32090001549422719762105771224014461040496826171875f,-0.29539998552799229258880586712621152400970458984375f,-1.472799987059831838820400662370957434177398681640625f,-0.50939997421056004700545827290625311434268951416015625f,-0.35989996666312240591878435225225985050201416015625f,-0.28729998469799777893740611034445464611053466796875f,0.20570001520514491488711428246460855007171630859375f,-0.2630999940499663125592633150517940521240234375f,-0.999099948370456569790576395462267100811004638671875f,-0.14779999726712678143059065405395813286304473876953125f,-0.503899994498491654582039700471796095371246337890625f,-0.0765999853253365170502320324885658919811248779296875f,-1.0460000106096269956168498538318090140819549560546875f,0.6211999755829575864396474571549333631992340087890625f,0.3891000021077692938575864900485612452030181884765625f,-0.6402999987013642257949186387122608721256256103515625f,-0.173499985016882352084621743415482342243194580078125f,0.19720001323819058569597473251633346080780029296875f,0.70570001147985461642520021996460855007171630859375f,-0.3201000090479855231251349323429167270660400390625f,0.00499998509883869957093338598497211933135986328125f,0.011300030004978367514922865666449069976806640625f,0.6229999738633633654671939439140260219573974609375f,-0.5621999806046478198595650610513985157012939453125f,-0.14290003395676631470223583164624869823455810546875f,-0.371399997925758562633546944198315031826496124267578125f,-0.0247000236876304679611848769127391278743743896484375f,-0.67690000991746757108558085747063159942626953125f,0.3217999770089987610077741919667460024356842041015625f,-0.2382000232517713644853074583807028830051422119140625f,0.1552999841913582645958058492396958172321319580078125f,-0.051100007885694243014995663543231785297393798828125f,0.660500010482966359859346994198858737945556640625f,-1.00730001066327101710839997394941747188568115234375f,0.0697000417366619995362952977302484214305877685546875f,-2.1967999739170078754568748991005122661590576171875f,-0.6269000181689861062039881289820186793804168701171875f,-0.031300019108504051335017948076711036264896392822265625f,-0.18820000905841538507701216076384298503398895263671875f,0.2362999940373000651305801511625759303569793701171875f,-0.469799971100687940861462266184389591217041015625f,-0.34920000800788419947906504603452049195766448974609375f,-0.03630001853108399956227003713138401508331298828125f,0.181600005255639462831140917842276394367218017578125f,-1.76689998996853869783763002487830817699432373046875f,-0.5234999850913879537728234936366789042949676513671875f,-1.728699986833333923641475848853588104248046875f,-0.50019999995380626245378152816556394100189208984375f,-1.0896999534845352997081135981716215610504150390625f,0.67419995130896648305451890337280929088592529296875f,0.2493999989837407671444680090644396841526031494140625f,-0.8448999982535827513174808700568974018096923828125f,-0.356300019723177374686429175199009478092193603515625f,0.57769999917149572610242103110067546367645263671875f,0.92780001522302624206162136033526621758937835693359375f,0.727700000493973764292832129285670816898345947265625f,-0.5389000033274295997642866495880298316478729248046875f,0.21020004255250113178732362939626909792423248291015625f,0.56110002911984935280997888185083866119384765625f,-0.806000006660818169024196322425268590450286865234375f,0.187100011380016528761416338966228067874908447265625f,0.57959998317956973323816782794892787933349609375f,1.1670000305324794709349589538760483264923095703125f,-1.49769995580911707833138279966078698635101318359375f,-0.4345999919712542070016070283600129187107086181640625f,0.58869993453025859508898065541870892047882080078125f,-1.5394000150024897966005710259196348488330841064453125f,0.111300035220384785361602553166449069976806640625f,1.10919998036623024262325998279266059398651123046875f,-0.22239999005794519604251036071218550205230712890625f,1.687599953764677085388257182785309851169586181640625f,-0.736399986898899072684798738919198513031005859375f,0.49599998229742059407954002381302416324615478515625f,0.7923000176668164584725673194043338298797607421875f,-1.0582000263810160678445981830009259283542633056640625f,-0.0480000191628935368726160959340631961822509765625f,-0.0524999827146519493226151098497211933135986328125f,-0.16419995637536022314861838822253048419952392578125f,-0.304100034052133150197505528922192752361297607421875f,-1.7653999685406684960753409541212022304534912109375f,-0.136299989324808024093726999126374721527099609375f,0.567100036823749320546994567848742008209228515625f,-0.7137000414460901165369932641624473035335540771484375f,0.0298000171080234910192530151107348501682281494140625f,-0.9414000482544311321220220634131692349910736083984375f,-0.6497000282883647059861687012016773223876953125f,0.5976999877721074216907481968519277870655059814453125f,-0.3275000008195636524988003657199442386627197265625f,0.0435999434381724171316818683408200740814208984375f,-0.918699975731968709879993184586055576801300048828125f,-0.63059999028742286242277259589172899723052978515625f,-0.6022999970287088444820255972445011138916015625f,0.98490000250041465079675617744214832782745361328125f,-0.115099970230459813791412670980207622051239013671875f,0.5587000113457440964026545771048404276371002197265625f,-1.0793000357568269809149796856218017637729644775390625f,-0.685999975740909651023002879810519516468048095703125f,0.5525999889314174851051575387828052043914794921875f,0.2262000149220229072710708351223729550838470458984375f,0.2858000050306319650417208322323858737945556640625f,0.007400006076693887990813891519792377948760986328125f,-0.57749999180436173418229373055510222911834716796875f,-0.46220000541508188263151168939657509326934814453125f,0.6010000064373013373142384807579219341278076171875f,0.179800016698241194035290391184389591217041015625f,-0.582700007069110625934627023525536060333251953125f,-0.1226000334113839240757215520716272294521331787109375f,0.40010002093166097125731539563275873661041259765625f,0.93540002188682525030571923707611858844757080078125f,0.2441000330090528080972944735549390316009521484375f,-0.6948000273525707370936288498342037200927734375f,-0.050799976420403680776871624402701854705810546875f,1.30440008563399434393659248598851263523101806640625f,0.088900038456916519180595059879124164581298828125f,1.0890000264942643060095406326581723988056182861328125f,0.25749998539686291820771657512523233890533447265625f,-0.437400018593669503985665869549848139286041259765625f,-0.2885000235363841536440077106817625463008880615234375f,-1.1818000232025978046834779888740740716457366943359375f,0.103199989314376683324780969996936619281768798828125f,-0.500400034105776825299471965990960597991943359375f,-1.6020000161528589188719706726260483264923095703125f,-0.576800015714764580110340830287896096706390380859375f,-0.62089995954036769632011782960034906864166259765625f,-1.62650003156065992016010568477213382720947265625f,0.455900007522106687218865772592835128307342529296875f,0.1147000115990639557139729731716215610504150390625f,0.906099983894824934083089829073287546634674072265625f,0.7823000126749277693960493706981651484966278076171875f,0.0533999862134455771656149636328336782753467559814453125f,1.269999979287386171478146934532560408115386962890625f,-0.658999971695244379787936850334517657756805419921875f,-0.808099988201259833431322476826608180999755859375f,0.3061000381350513777078958810307085514068603515625f,2.001199983629585066324807485216297209262847900390625f,-0.03969998388290463253724738024175167083740234375f,-0.101399970656633087173759122379124164581298828125f,0.45379993195533785410589189268648624420166015625f,-1.18939998564720195162180971237830817699432373046875f,-0.48519999082684517333063922706060111522674560546875f,-0.9023999986261130867859492354909889400005340576171875f,0.30970001007169489781745141954161226749420166015625f,0.172999982953072173330610894481651484966278076171875f,-0.2306999678939585241010945537709631025791168212890625f,2.6713999847382314811738979187794029712677001953125f,-1.31610003355294491456106698024086654186248779296875f,-1.13979999766200723598785771173425018787384033203125f,-0.92780001300647807482846474158577620983123779296875f,1.826700036096572166144369475659914314746856689453125f,0.4915999741494665453700463331188075244426727294921875f,1.49920002357959702976586413569748401641845703125f,0.148499991685152821219162433408200740814208984375f,-0.9347000379741203790473491608281619846820831298828125f,-0.06220003879368307675434834891348145902156829833984375f,2.419500054001808120318628425593487918376922607421875f,0.8890999938935042479215553612448275089263916015625f,0.0866000190019615700975919025950133800506591796875f,-0.15299999796599139667563349576084874570369720458984375f,0.50450002548471051255774000310339033603668212890625f,0.2469999985508624096297580763348378241062164306640625f,-1.1678999978959556660385032955673523247241973876953125f,0.971899963518977560994471787125803530216217041015625f,-0.2185999963372953214246763309347443282604217529296875f,0.57149998641014077005451099466881714761257171630859375f,1.0188000378280877011860638958751223981380462646484375f,-0.2491000197842716534779583525960333645343780517578125f,-1.7528000312417741923809444415383040904998779296875f,0.76420001959353722753576221293769776821136474609375f,0.97800001253187662797472512465901672840118408203125f,0.04640001072213062460747323711984790861606597900390625f,0.050399982883036098524343060489627532660961151123046875f,0.57309998162612341143784533414873294532299041748046875f,-0.8917999728739267606414387046243064105510711669921875f,-0.2163000168919559929037177425925619900226593017578125f,-0.816600016394258165774999724817462265491485595703125f,-1.012299986225366321690444237901829183101654052734375f,-0.07480003368556520371157603221945464611053466796875f,0.59950000691413907194515786613919772207736968994140625f,0.2508000214219094914369634352624416351318359375f,-1.0450000085681676598170497527462430298328399658203125f,0.3463999786287548232621702481992542743682861328125f,0.701199957813323226929469456081278622150421142578125f,1.378500038139522043678653062670491635799407958984375f,0.784100057540090045193892365205101668834686279296875f,-0.0765000582337389545273254043422639369964599609375f,-0.86019998024702015726461468148045241832733154296875f,-0.66819998272061287281076147337444126605987548828125f,-0.87129999647736600110192739521153271198272705078125f,1.26510005166530703490934683941304683685302734375f,0.49199997577071119536640253500081598758697509765625f,-0.779800020349025402310871868394315242767333984375f,-0.0245999337077140722840340458787977695465087890625f,-2.912500128298999069187402710667811334133148193359375f,1.0077999935045840462777277934947051107883453369140625f,0.0602999927036462768370483900071121752262115478515625f,-0.4651999981842929177133783014141954481601715087890625f,-0.0214999878257506882306415718630887567996978759765625f,-0.896000010907650246139155569835565984249114990234375f,0.5313999942749736415947836576378904283046722412109375f,0.02830000048577774318658839547424577176570892333984375f,-0.24970004006028201359868035069666802883148193359375f,0.6310000456273552327246534332516603171825408935546875f,1.4481000292539596419061354026780463755130767822265625f,-1.65019999045431564610453278874047100543975830078125f,0.955600042292476192784533850499428808689117431640625f,-0.74319998560771349804809915440273471176624298095703125f,-0.467600008191168814430938027726369909942150115966796875f,-1.119500011216848722739314325735904276371002197265625f,-0.2217999997705224490829323258367367088794708251953125f,0.14030002952814069061560076079331338405609130859375f,-1.2481999865204091992865187421557493507862091064453125f,1.0601999818861482793863615370355546474456787109375f,-0.29040002471804537975685889250598847866058349609375f,-1.21579997254610105983374523930251598358154296875f,-0.5678000569462777757934190958621911704540252685546875f,-0.1574000070452683530675130896270275115966796875f,0.644799985703825573324365905136801302433013916015625f,-0.17219996242523194496243377216160297393798828125f,-0.7247999930798993961644782757502980530261993408203125f,-0.17539996787011624679308852137182839214801788330078125f,-0.3490999956071372256616314189159311354160308837890625f,0.57409999583065474126897242967970669269561767578125f,0.41819997884631110451891800039447844028472900390625f,1.372099995547532369499776905286125838756561279296875f,-0.8942000301361081593398694167262874543666839599609375f,0.36290003917217195095190618303604423999786376953125f,-0.1623999857366076327025439240969717502593994140625f,1.041600002759694110210375583847053349018096923828125f,-0.615400003409385387698193881078623235225677490234375f,-0.892899972265958741246549834613688290119171142578125f,-0.3703000091135504323602845033747144043445587158203125f,-0.5259999831542374604254064252017997205257415771484375f,-0.19399998678267049712076186551712453365325927734375f,-0.129299991388619084364819400434498675167560577392578125f,-0.31549998742341944790723573532886803150177001953125f,-0.387999980568885849407934074406512081623077392578125f,0.00110002181828050549228237287024967372417449951171875f,-0.7617999852418899653372363900416530668735504150390625f,-0.33180002294182830979707432561554014682769775390625f,-0.245899972951412859600850424612872302532196044921875f,0.04530000993311411594532955859904177486896514892578125f,0.161800013852118951263037160970270633697509765625f,1.274799970728159426158754286007024347782135009765625f,0.17819999419450727184965899141388945281505584716796875f,1.413399949705600899818591642542742192745208740234375f,-0.0155000046342610442451359631377272307872772216796875f,1.594399960464239374147155103855766355991363525390625f,-0.695199980098009095996758333058096468448638916015625f,1.554599986234307618104821813176386058330535888671875f,0.3724999912083148956298828125f,1.889399957036972832469245986430905759334564208984375f,-0.312699985760450882565919528133235871791839599609375f,0.14499996729195141487167575178318656980991363525390625f,-0.22269997726678791849508343148045241832733154296875f,0.7325999435573802287535727373324334621429443359375f,0.6898999825701117583065524740959517657756805419921875f,0.1451999918326729588358148248516954481601715087890625f,-0.033099986226857058824180057854391634464263916015625f,-1.5000000502169132943208751385100185871124267578125f,-1.899600043231249202335675363428890705108642578125f,2.00340006399750780730073529412038624286651611328125f,1.0690000184476371458686116966418921947479248046875f,-0.125700011628866459290065904497168958187103271484375f,-0.658699997040629536826372714131139218807220458984375f,0.836000032588839925296042565605603158473968505859375f,-1.213700019615888692214866750873625278472900390625f,-1.0526999668359753314916815725155174732208251953125f,-0.3717000171720983292544815412838943302631378173828125f,0.282899973160028128660314905573613941669464111328125f,-1.169500011198222555464099059463478624820709228515625f,0.229600018569827479808509451686404645442962646484375f,-0.02199999886751202726742349113919772207736968994140625f,-0.5107000172168019158647211952484212815761566162109375f,0.6036000294178724967508742338395677506923675537109375f,-0.207999990403652024184566471376456320285797119140625f,-0.019899990803003253514447123961872421205043792724609375f,0.708399990683794111845372754032723605632781982421875f,0.48759999593496328174069276428781449794769287109375f,1.532399954742192704770786804147064685821533203125f,-0.6869999757334586121260144864208996295928955078125f,-0.694800018560886467611226180451922118663787841796875f,0.0724999978393319199909683447913266718387603759765625f,-0.461400006940960505374960121116600930690765380859375f,1.17670001538395840867678998620249330997467041015625f,-0.4868000097542999338173785872641019523143768310546875f,-1.37280000330209706049799933680333197116851806640625f,0.041000010609626880153655292815528810024261474609375f,0.46519999417960633536495151929557323455810546875f,-0.656000011056661502806264252285473048686981201171875f,0.5863999978512521860096740056178532540798187255859375f,-1.6514000635653738324748474042280577123165130615234375f,-0.6232999874100089154893566956161521375179290771484375f,0.3745000353753571431525415391661226749420166015625f,1.089400037205219717861837125383317470550537109375f,-0.650700040239096555438891300582326948642730712890625f,-0.2547000031054011959241734075476415455341339111328125f,0.63410001855492570399519536294974386692047119140625f,0.7071999867141249840329919607029296457767486572265625f,-0.7556000191211704741789390027406625449657440185546875f,-0.5184000141903755132233300173538736999034881591796875f,0.50060002314448315274830747512169182300567626953125f,0.5450999791711568231988849220215342938899993896484375f,-1.485799957644939883749657383305020630359649658203125f,0.88319997344464074018333121784962713718414306640625f,0.2406999547064305744470402714796364307403564453125f,1.026099994027614314973106957040727138519287109375f,1.079599988618493267011899661156348884105682373046875f,0.438999994605779431111614030669443309307098388671875f,0.9433999529838568154360700646066106855869293212890625f,-1.001899946233630434022643385105766355991363525390625f,0.56700001540780053943535676808096468448638916015625f,-2.152299916636945198433750192634761333465576171875f,-0.813100006306171518843939338694326579570770263671875f,0.63650000272691187319651362486183643341064453125f,0.637399985736607721520385894109494984149932861328125f,0.213400027787685686586627298311213962733745574951171875f,0.54370004017949202790305207599885761737823486328125f,-0.6656999860361221266202846891246736049652099609375f,-1.39469996957778885615653052809648215770721435546875f,0.091499992184341127909874558099545538425445556640625f,-0.014600015813112321438893559388816356658935546875f,-0.402599981369078374626724325935356318950653076171875f,-0.27089997369647011993265550700016319751739501953125f,-0.60100000744312997991158908916986547410488128662109375f,0.66980003539919863442264613695442676544189453125f,0.189200007653981483457528156577609479427337646484375f,-0.146299972188473237366679313709028065204620361328125f,0.40539997640103120346566356602124869823455810546875f,-1.5914999595880505722789166611619293689727783203125f,0.43690000956356545014358516709762625396251678466796875f,0.32479992796182699521523318253457546234130859375f,-0.6984000459298489449366797998663969337940216064453125f,0.4651000057697292078273676452226936817169189453125f,1.1056000049650667182987717751529999077320098876953125f,0.03839999440908403727235054248012602329254150390625f,-0.863299953174590850579761536209844052791595458984375f,-0.37179999597072566075439681299030780792236328125f,0.647999976471065775029956057551316916942596435546875f,-1.375699988234042603352236255886964499950408935546875f,0.970799977463484342621313771815039217472076416015625f,-0.232799993616342959512621746398508548736572265625f,0.9130999616026880261188125587068498134613037109375f,1.3365999772787091615100507624447345733642578125f,0.517099979603290904606183175928890705108642578125f,-0.19969998373389241663744542165659368038177490234375f,-0.51669996844530174939791322685778141021728515625f,1.2642999752581118855232489295303821563720703125f,-1.7244999593794343439867589040659368038177490234375f,-0.199400015002489272575303402845747768878936767578125f,-0.592599989750981048786115934490226209163665771484375f,-1.004699999622255202069709412171505391597747802734375f,-0.85269999980479493917329136820626445114612579345703125f,-0.261900001088530454129710278721177019178867340087890625f,-0.29590000619962808503515816482831723988056182861328125f,-0.51780003401711616195512988269911147654056549072265625f,-0.1452999980680644842578885800321586430072784423828125f,0.83910002689957696730971292709000408649444580078125f,-0.3744999707043177306076131571899168193340301513671875f,-0.90279995755553255065706252935342490673065185546875f,0.055900013743341414595988680957816541194915771484375f,-0.2229999999403953214738294263952411711215972900390625f,-1.421500005222856799491637502796947956085205078125f,-0.7405999950557944355722383988904766738414764404296875f,0.28419997369796046182699456039699725806713104248046875f,-0.1648999859228734710114849804085679352283477783203125f,1.0617999714955688173034786814241670072078704833984375f,0.17359999432563777421734130257391370832920074462890625f,-1.40959996542856114132291622809134423732757568359375f,-1.19159991260767039733536876155994832515716552734375f,0.8320000297129155342190642841160297393798828125f,-0.3597000509381285127119554090313613414764404296875f,-0.6740000744760035189528935006819665431976318359375f,1.270499972969293622782060992904007434844970703125f,0.307300000232458625504250449012033641338348388671875f,0.637800000843405268113883721525780856609344482421875f,0.507799971953779571975928774918429553508758544921875f,0.7013999895997347078235861772554926574230194091796875f,0.290899975559115564482937088541802950203418731689453125f,0.66339998129606281285219893106841482222080230712890625f,-0.64379997384622711376067627497832290828227996826171875f,-1.38059996920228034156252761022187769412994384765625f,0.58979998086094820308744601788930594921112060546875f,0.54319995753765137891377889900468289852142333984375f,1.1835999791264537606139128911308944225311279296875f,0.0645000165998936836331267841160297393798828125f,0.2274999979138374595066807160037569701671600341796875f,0.849400042271614807987134554423391819000244140625f,-0.7918000005900858173646383875166065990924835205078125f,0.090600043857098189192811332759447395801544189453125f,0.012499973177909795563067518742172978818416595458984375f,0.2567999990999705328675872806343249976634979248046875f,-0.160799981263280045151731201258371584117412567138671875f,-0.2470999712586403607161855688900686800479888916015625f,0.2901999797254803947765822158544324338436126708984375f,0.23389997609555746027609757220488972961902618408203125f,0.08780000259429210363038009745650924742221832275390625f,-0.219699992972612445640834266669116914272308349609375f,0.69799994547665189514873418374918401241302490234375f,-0.819600020134448215003430959768593311309814453125f,0.154700022737682019169369596056640148162841796875f,-0.328300033640861244066400104202330112457275390625f,-1.171400009623169591321811822126619517803192138671875f,-0.230800003391503860683542370679788291454315185546875f,-1.0826999681770803629632382580894045531749725341796875f,0.279899986332655004162006662227213382720947265625f,1.2068999582663177960739631089381873607635498046875f,-0.08800001402199253419667002162896096706390380859375f,-0.48509999489188204702116991029470227658748626708984375f,-0.239200022089480768272551358677446842193603515625f,1.335299921047688798125818721018731594085693359375f,0.87319997497200940728134810342453420162200927734375f,-0.5444999973773949619726408855058252811431884765625f,1.3694999805390828395701419140095822513103485107421875f,-0.958399988001584546282174414955079555511474609375f,-0.706699992287159073356406224775128066539764404296875f,-0.463799957138300467107683289214037358760833740234375f,-0.089099993073940453314207843504846096038818359375f,0.5608999695241454475080900010652840137481689453125f,-0.215499982878565976562867945176549255847930908203125f,0.270300002706050701561935056815855205059051513671875f,0.3625999878883361393633322222740389406681060791015625f,-0.07749997958540955078632350705447606742382049560546875f,-0.191000013366342091813976367120631039142608642578125f,-0.14969999599009753410427947528660297393798828125f,-1.71759998857751394751858242671005427837371826171875f,-0.287500000558793544769287109375f,0.26730002461448354456052811656263656914234161376953125f,0.74300000208243677235486757126636803150177001953125f,0.6824000083118673121163055839133448898792266845703125f,0.83920000927448301641931038830080069601535797119140625f,0.2456999765366314480985465706908144056797027587890625f,0.0505000263899564316716350731439888477325439453125f,-1.5529000340685252812278349665575660765171051025390625f,-1.387099975728988798806540216901339590549468994140625f,-0.608799938511849436650891220779158174991607666015625f,-0.487099996329843687448146738461218774318695068359375f,1.35789993039369694116658138227649033069610595703125f,0.4524000046238303429646521180984564125537872314453125f,0.78310001465082113014659626060165464878082275390625f,1.02940001341924070032973759225569665431976318359375f,0.324799980860948522831677109934389591217041015625f,-0.4388000219210981267536908490001223981380462646484375f,-0.4976999869525429165406649190117605030536651611328125f,-0.28750001657754220563134595067822374403476715087890625f,1.0094999937638633458192316538770683109760284423828125f,-0.762799946081639035355692612938582897186279296875f,-0.6481999819010495489379763967008329927921295166015625f,0.30239997269809304469845301355235278606414794921875f,0.988799974926561020538429147563874721527099609375f,-0.4631999984599648456651266315020620822906494140625f,-0.51779997588396098340268736137659288942813873291015625f,-0.00529999516233881085014445488923229277133941650390625f,-0.76670000197291343635441762671689502894878387451171875f,0.559099997320771535669337026774883270263671875f,-1.12419999377727464207055163569748401641845703125f,0.8594000141084185973028297667042352259159088134765625f,0.6909000045046214122379524269490502774715423583984375f,0.22690000285804234891884334501810371875762939453125f,0.311600014419853632574586299597285687923431396484375f,1.25220001172944872536163529730401933193206787109375f,1.0805000038146967966667943983338773250579833984375f,0.9408000079363587619951658780337311327457427978515625f,0.295000019669533486421642010100185871124267578125f,0.975499981313943553828949006856419146060943603515625f,1.93260001761615285431616939604282379150390625f,-0.108699969026446296993526630103588104248046875f,-0.427699979948997555112555346568115055561065673828125f,-0.006100015410781145508423151113674975931644439697265625f,1.26390001401901219679757559788413345813751220703125f,-0.51729998819977029977934535054373554885387420654296875f,0.3952000176861878433243191466317512094974517822265625f,1.58809995944201975959231276647187769412994384765625f,-0.0802000073671340629744008765555918216705322265625f,0.7789999723285436772357570589520037174224853515625f,1.03259995391368875061743892729282379150390625f,0.77450003079324958310536430872161872684955596923828125f,-1.20079998439252388919840086600743234157562255859375f,-1.1501000189200041301518240288714878261089324951171875f,0.4343999960780138280824758112430572509765625f,0.2609000217899681839384129489189945161342620849609375f,-0.33779997439384512603055554791353642940521240234375f,0.45759998146221025905333590344525873661041259765625f,-0.8755999878287312743196935116429813206195831298828125f,2.348599917975813244908067645155824720859527587890625f,-0.7438999988198276014372822828590869903564453125f,1.9696999466300013637010124512016773223876953125f,0.94950000095367403929458305356092751026153564453125f,0.48119997357130017689996748231351375579833984375f,-1.0440999950855971434293678612448275089263916015625f,-0.332199962425231642981771074119023978710174560546875f,1.5789999789595603463254747111932374536991119384765625f,-0.659700004670023876229834058904089033603668212890625f,-0.034900010025501249089074917719699442386627197265625f,-0.28140000347644067435481929351226426661014556884765625f,0.64850004518032167144525601543136872351169586181640625f,-0.459799996414035849756629659168538637459278106689453125f,1.311200030195712518121808898285962641239166259765625f,-0.657799981360137397956577842705883085727691650390625f,-0.4957999815426770151560731392237357795238494873046875f,-0.5509000070005651839011306947213597595691680908203125f,0.7031999839499591953284607370733283460140228271484375f,-0.0007000024832784779160732568925595842301845550537109375f,-0.277100020730495888443556395941413938999176025390625f,0.466499995686113611004230961043504066765308380126953125f,0.24239999169707271420293182018212974071502685546875f,0.0497999781042339773051708107232116162776947021484375f,-0.74999999023973895617700691218487918376922607421875f,0.72580002501681428750401892102672718465328216552734375f,-2.0473999596595771066631641588173806667327880859375f,-0.74390004534870346475372571148909628391265869140625f,-0.30070003181993865126742093707434833049774169921875f,-0.46989998402297550317285868004546500742435455322265625f,-1.16879995218366428133549561607651412487030029296875f,-0.68719998183399477742483441033982671797275543212890625f,0.187400012409686400616237733629532158374786376953125f,1.21689995506256831703240095521323382854461669921875f,-0.0243999972701072209702033433131873607635498046875f,0.18289997807741242841217399472952820360660552978515625f,0.278899992263316798268846241626306436955928802490234375f,-0.16559998825714050685320444245007820427417755126953125f,0.186299990069866083786109811626374721527099609375f,0.0911000077366832528014128911308944225311279296875f,1.699399930289388294113450683653354644775390625f,-0.5622999653823679278019653793307952582836151123046875f,-0.9724999752640723915675380339962430298328399658203125f,0.5390000055022532432502657684381119906902313232421875f,0.097100019687414107494305426371283829212188720703125f,1.495899934357404870155505705042742192745208740234375f,0.6509999654591089868205244783894158899784088134765625f,1.1508999937012784897660822025500237941741943359375f,1.6425000267848368640244416383211500942707061767578125f,0.4254000152185561933748658702825196087360382080078125f,1.2106000249698756210392502907779999077320098876953125f,0.3904999929741024278229133415152318775653839111328125f,0.699300002489983807407725180382840335369110107421875f,0.428899989357590938343633979457081295549869537353515625f,0.81690000494420555998686950260889716446399688720703125f,0.5261999804630874688626818169723264873027801513671875f,0.813499996341765285734481949475593864917755126953125f,0.51750000685453390047996435896493494510650634765625f,-0.33369999882876866248437863760045729577541351318359375f,0.416600007490813606381152567337267100811004638671875f,0.3636000131011003499992284559994004666805267333984375f,-0.9266999820053574143940977592137642204761505126953125f,0.213199994269012815806263461126945912837982177734375f,-0.58620000844001740603061989531852304935455322265625f,0.4067999649763105463051715560141019523143768310546875f,0.030500037714838779123738277121447026729583740234375f,0.55199997726082872251396338469930924475193023681640625f,1.017299970132112374443522639921866357326507568359375f,0.9490000122636550461407978218630887567996978759765625f,-0.0411000229358666757661922019906342029571533203125f,-0.39039993203282374878426708164624869823455810546875f,0.19609999466091376785215061318012885749340057373046875f,-0.00630004252195437430827951175160706043243408203125f,-0.9384000410497197464110286091454327106475830078125f,1.29490001315474501097924076020717620849609375f,0.5984000826239590509203480905853211879730224609375f,-0.912300005149841553020451101474463939666748046875f,-1.0515999796628943130372135783545672893524169921875f,-0.1320000207722191731818384141661226749420166015625f,0.27609996452331575511607297812588512897491455078125f,-0.246900017535687243963593573425896465778350830078125f,0.707000000357628000102749865618534386157989501953125f,0.914200041908026062031922265305183827877044677734375f,-0.305400030082464635228234328678809106349945068359375f,-0.1786000124305484604292360018007457256317138671875f,0.025200019362568770731058975798077881336212158203125f,-0.2827999951064585903992565363296307623386383056640625f,-0.450599982389807252047830843366682529449462890625f,0.561899985721706851649059899500571191310882568359375f,-0.706300042968988517344541833153925836086273193359375f,0.281800029647350402939309788052923977375030517578125f,0.64360000006258477611709167831577360630035400390625f,-0.3476999689221378364578640685067512094974517822265625f,0.298499982744455660821358833345584571361541748046875f,-0.53200003768503700829484159839921630918979644775390625f,0.29400001146271836915957464952953159809112548828125f,-0.290399991860985640101944227353669703006744384765625f,-0.3340999738514423000879105529747903347015380859375f,0.1850000204145902937824530454236082732677459716796875f,-1.07980001379251433490935596637427806854248046875f,-1.082399979254603739065032641519792377948760986328125f,1.2882999841690061515464549302123486995697021484375f,0.549399987137317413754544759285636246204376220703125f,-0.54199997770786279005506003159098327159881591796875f,1.1934000280857082199048591064638458192348480224609375f,0.444299976897239989881427391082979738712310791015625f,0.76760000807940975686705087355221621692180633544921875f,-0.118899970805645072147171958931721746921539306640625f,1.039600045019388740996646447456441819667816162109375f,0.36830004351288103681127950039808638393878936767578125f,0.021199995885789579830316142761148512363433837890625f,0.58340001186206869032702115873689763247966766357421875f,-0.685500002026557364587233678321354091167449951171875f,-0.817000002779067369118592978338710963726043701171875f,2.28000001236796290271513498737476766109466552734375f,1.394900021648407317798046278767287731170654296875f,-0.13739998886585258475179216475225985050201416015625f,-1.0465999912858006126725740614347159862518310546875f,-1.2651000355720523060654159053228795528411865234375f,-0.04439998244345166877877773004001937806606292724609375f,0.01929999604523169320913211777224205434322357177734375f,0.14150001379102461473991070306510664522647857666015625f,0.77769999967440972543641919401125051081180572509765625f,-0.45439997171461621316979062612517736852169036865234375f,-0.44729998127073056224389802082441747188568115234375f,0.03970000453963873354723546071909368038177490234375f,-0.67909998592138254291938892492908053100109100341796875f,0.6535000025630000042298206608393229544162750244140625f,0.1508999538779249860454001463949680328369140625f,-0.711200027886033314672431515646167099475860595703125f,2.036600025931000867984721480752341449260711669921875f,-0.27500001616775993085184381925500929355621337890625f,-0.6557999937802552192778193784761242568492889404296875f,-0.33220000699833052149045897749601863324642181396484375f,0.072199966672063353723132195227663032710552215576171875f,-0.4910999702043834513887077264371328055858612060546875f,0.105999990195036364326597322360612452030181884765625f,1.13659997195154449656229189713485538959503173828125f,-0.04539997390508732078018283573328517377376556396484375f,-1.74239999393224653800871237763203680515289306640625f,-1.2793999886274338706471098703332245349884033203125f,0.7791999928832051836025129887275397777557373046875f,-0.758000035330653521015165097196586430072784423828125f,-0.2918999463081366485539547284133732318878173828125f,-0.8964000210225577802702900953590869903564453125f,-0.0178999160885808805687702260911464691162109375f,-0.2573999913990496768434468322084285318851470947265625f,0.75020003895759668210985182668082416057586669921875f,0.4435000149309633599159496952779591083526611328125f,-0.34680000997781679217268901993520557880401611328125f,0.487400012111663283320694972644560039043426513671875f,0.7649999868869787889025246840901672840118408203125f,-0.593900014615058591260776665876619517803192138671875f,-0.95939998780786961418698410852812230587005615234375f,0.344200034978986213474172473070211708545684814453125f,-1.4767999767482284934061453895992599427700042724609375f,0.148500016793608580201180302537977695465087890625f,-0.944399967467784318131407417240552604198455810546875f,-0.1319000138103962882496489328332245349884033203125f,-0.6613000320166351198025722624151967465877532958984375f,-1.083699993091821855983880595886148512363433837890625f,0.71929997496008912793996614709612913429737091064453125f,0.7154999743849039806065093216602690517902374267578125f,0.00159998882710932210926557672792114317417144775390625f,-0.1762999923050403427993160221376456320285797119140625f,-0.319700000870227707405746286895009689033031463623046875f,-0.21100001396238809991956486555864103138446807861328125f,-0.188399972653389369270371389575302600860595703125f,1.92700000832974893683058326132595539093017578125f,-0.2066000249624249107682771864347159862518310546875f,0.78960000806450825194815479335375130176544189453125f,0.55969998909831009115123379160650074481964111328125f,0.1798000283956522338257855153642594814300537109375f,0.2225999679207806281056036823429167270660400390625f,-0.6831999887555839645614241817384026944637298583984375f,-0.6404999756515028508374598459340631961822509765625f,0.0729999944269656975137650078977458178997039794921875f,0.435499992117285739112730880151502788066864013671875f,0.51240001531541368162692151599912904202938079833984375f,-0.6337000123143194940666944603435695171356201171875f,1.609600008064507647986829397268593311309814453125f,0.5159999741017811114573987651965580880641937255859375f,-1.077999971702694015363022117526270449161529541015625f,-0.754199987481534162014895628090016543865203857421875f,0.195699972587822568215187857276760041713714599609375f,0.89739996461421345230746737797744572162628173828125f,-0.911100047075749319702708817203529179096221923828125f,-1.135500036001205526048352112411521375179290771484375f,0.71260002603530925302521836783853359520435333251953125f,-0.5977000148922204658674672828055918216705322265625f,-0.687300034281612237663239284302107989788055419921875f,0.047999988466500997930097582866437733173370361328125f,-0.818200006934999723995360909611918032169342041015625f,0.7033000036150216516261934884823858737945556640625f,0.7779000125363462192495944691472686827182769775390625f,0.574199993684143095151739544235169887542724609375f,0.566600017325580296301268390379846096038818359375f,0.25449995466694230739079785053036175668239593505859375f,-0.0094000207394362433888090890832245349884033203125f,-0.52810001956820495916389290869119577109813690185546875f,-0.6196000174522404702059930059476755559444427490234375f,0.2514000013530253152538307404029183089733123779296875f,0.80720004669129874486088738194666802883148193359375f,0.5535999955177306475206933100707828998565673828125f,-0.7715999682635068523950394592247903347015380859375f,-0.3816999919891357340162585387588478624820709228515625f,-0.6393999690696599014216872092220000922679901123046875f,0.864799959552288388664464946486987173557281494140625f,-0.30019999887347303957341182467644102871417999267578125f,-0.56269997387677406219808062814990989863872528076171875f,-0.180399994990229739499909555888734757900238037109375f,0.517400012782216034423754535964690148830413818359375f,-0.651200019392371043380762785091064870357513427734375f,-0.1927000006616115257429555640555918216705322265625f,-1.2256000139057638964601437692181207239627838134765625f,0.66679999772161246962554059791727922856807708740234375f,-0.547900044126809149247492314316332340240478515625f,-0.75379998990222862875754117339965887367725372314453125f,-1.52260007222890880740351349231787025928497314453125f,-1.66539998046159798406051777419634163379669189453125f,-2.65860008045434970114229145110584795475006103515625f,-1.0317999497771257466638417099602520465850830078125f,0.17099996516108451061199957621283829212188720703125f,-0.231200009408592865867149157566018402576446533203125f,-0.93560003588497675508506290498189628124237060546875f,0.543000020727514964846704970113933086395263671875f,-0.320800035130977878594649155274964869022369384765625f,0.14149998145550479744514404956134967505931854248046875f,1.3776999848477540222546622317167930305004119873046875f,-1.2277999976955358274466334478347562253475189208984375f,0.9557999881923198159938692697323858737945556640625f,-1.78849997916817660126298505929298698902130126953125f,1.7194000405579805690337025225744582712650299072265625f,-1.815600016438960739861840920639224350452423095703125f,-1.28580002075135713113240853999741375446319580078125f,-0.294000008612871399638066804982372559607028961181640625f,0.893000031456351539560500896186567842960357666015625f,-0.67630000623762620559631386640830896794795989990234375f,-0.4849999965727327388975709254737012088298797607421875f,-0.35679999289736141765416732596349902451038360595703125f,-1.512800027255714052643043032730929553508758544921875f,-0.36429996026381850082742630547727458178997039794921875f,-0.27580003140568720709779881872236728668212890625f,0.08200002509355552859915405861102044582366943359375f,0.3261999718576664708535872705397196114063262939453125f,0.79759999891519584025445510633289813995361328125f,1.56940000390112377459672643453814089298248291015625f,-0.2891999847620727592811817885376513004302978515625f,0.525599983581900520590579617419280111789703369140625f,0.370600005561113388097282950184307992458343505859375f,1.8009999970495702559247774843242950737476348876953125f,-0.42269996834471845748026908040628768503665924072265625f,0.48150001097843053710079175289138220250606536865234375f,1.0833000585816809913097813478088937699794769287109375f,0.93899997240304866608084921608678996562957763671875f,0.762100021922588410916432621888816356658935546875f,-0.71299997222423439069416417623870074748992919921875f,0.06530004476457784701182163189514540135860443115234375f,1.590100021825730625124606376630254089832305908203125f,0.50299999943748152642086779451346956193447113037109375f,-0.4730000295937060172946075908839702606201171875f,-0.43539998746514374516891621169634163379669189453125f,-1.2512000389873978445365310108172707259654998779296875f,-0.23949997875094408694707226459286175668239593505859375f,0.151000016830861394367957473150454461574554443359375f,-0.3494999984018500871485457537346519529819488525390625f,-0.8245000206977126655516485698171891272068023681640625f,-0.9126999967873101748949693501344881951808929443359375f,0.32060000429451396275482011333224363625049591064453125f,-0.73939997920244915352583348067128099501132965087890625f,-1.021500006992369424807520772446878254413604736328125f,0.1579999963827434239505009827553294599056243896484375f,1.1767999972373248684931468233116902410984039306640625f,-0.946599982792139638121398093062452971935272216796875f,0.19990003975331749330734965042211115360260009765625f,0.17049998796358689645558115444146096706390380859375f,0.966799993623792541797001831582747399806976318359375f,-0.92550001085549549184605666596326045691967010498046875f,1.3191999793976545429785574015113525092601776123046875f,-0.7032999639034276384563781903125345706939697265625f,0.676799957972765486857724681613035500049591064453125f,0.57079996712580338691367387582431547343730926513671875f,-0.2578000025942923656430139089934527873992919921875f,-0.60050000200793129945253667756333015859127044677734375f,-0.9826000377327199775123744984739460051059722900390625f,-0.16249995723366783551000480656512081623077392578125f,1.230999960169196327797180856578052043914794921875f,-0.98039999387264220587212548707611858844757080078125f,-0.9093000527441505997927606586017645895481109619140625f,-0.23320000917017491559590780525468289852142333984375f,-0.7258000288724897330894236802123486995697021484375f,0.3324000372201201258803848759271204471588134765625f,-1.1694000255823133738175556572969071567058563232421875f,-0.6363999905496837694585110511980019509792327880859375f,-0.4562000205844640721153382401098497211933135986328125f,0.2836000163421037445488082084921188652515411376953125f,-0.1150000309944154697205931370262987911701202392578125f,-0.6899999829381704419262177907512523233890533447265625f,-0.08040001167953025884571616188623011112213134765625f,-1.154799956125021953567966193077154457569122314453125f,-1.87829999445080719766565380268730223178863525390625f,0.6065000030994411872597993351519107818603515625f,0.523500022977589907213769038207828998565673828125f,-0.6365999725103375084245271864347159862518310546875f,0.42630002724826254745238429677556268870830535888671875f,-1.4648999937646085545139840178308077156543731689453125f,0.024099985474348051273096871227608062326908111572265625f,-0.5262000089056779739138391960295848548412322998046875f,-0.808999996766447626583840246894396841526031494140625f,-0.0526999930433931229600830192794091999530792236328125f,-0.8483999902367589385221435804851353168487548828125f,0.1339999867081640705368528188046184368431568145751953125f,0.2609000040948392662443211520439945161342620849609375f,-0.1641000040635464951588318172071012668311595916748046875f,0.08200001347065022372362363967113196849822998046875f,-0.71210001648366461068917487864382565021514892578125f,1.27540003961920778152716593467630445957183837890625f,-0.14870001115947972802899812450050376355648040771484375f,-0.93730002109408427912740080500952899456024169921875f,-0.8857000063762063835071103312657214701175689697265625f,-0.23250001572072509503641413175500929355621337890625f,-0.2078000167876485060336477772580110467970371246337890625f,0.582399999054521000374506911612115800380706787109375f,-0.51129997159242612791985038711572997272014617919921875f,-0.0484000115454195078967813969939015805721282958984375f,-0.65260000353455505095467970022582449018955230712890625f,-0.23039999148845691223641551914624869823455810546875f,0.18750000741332828457785808495827950537204742431640625f,-1.69289994916915897960052461712621152400970458984375f,-0.33990003248900091392670219647698104381561279296875f,0.04569994055032822188877617008984088897705078125f,-0.91980003931075327017197196255438029766082763671875f,0.146600035393237959624457289464771747589111328125f,0.2948000586077565987608295472455210983753204345703125f,-0.3591999795466660483356235999963246285915374755859375f,-0.680299995478987451491548199555836617946624755859375f,-0.0804999981522562801927733744378201663494110107421875f,-0.6595999971121540017549023104947991669178009033203125f,0.557500012516974674525727095897309482097625732421875f,-1.15100001090764880729011565563268959522247314453125f,-0.49399999102950165053016462479718029499053955078125f,0.35809994461536565069081916590221226215362548828125f,1.5755999556422235485797500587068498134613037109375f,-0.894999985694884259146419935859739780426025390625f,0.35999999165534990464010434152442030608654022216796875f,-0.94519997018873791461146538495086133480072021484375f,-1.091700003686546960324221799965016543865203857421875f,-0.2036000125050543285709636620595119893550872802734375f,0.9003000015884639051222393391071818768978118896484375f,0.356099998982250698364993013456114567816257476806640625f,-0.892900010785460107598510148818604648113250732421875f,0.95279999283403149501481266270275227725505828857421875f,-0.7512000304937362926693822373636066913604736328125f,-0.183299974818527555697755815344862639904022216796875f,-0.925400008196383350167479875381104648113250732421875f,-0.85249996393919047488196838457952253520488739013671875f,-0.95809995810091563850363627352635376155376434326171875f,-0.77319998966827940822810205645509995520114898681640625f,1.4660000214874744361992497942992486059665679931640625f,0.94700002017617190830378604005090892314910888671875f,-0.595299997788667933917849950375966727733612060546875f,-1.58069998260885480334536623558960855007171630859375f,1.27219996272325541752934441319666802883148193359375f,0.06349999339878553694660467954236082732677459716796875f,1.2647999556779865315547795034945011138916015625f,0.1665000048503275653644095655181445181369781494140625f,-0.0348999658249324884451425532461144030094146728515625f,-0.683399952760339157720181901822797954082489013671875f,0.61579997783601303495970569201745092868804931640625f,0.967399979198723780626778534497134387493133544921875f,-2.07559994416832882535572935012169182300567626953125f,-0.832699968922138822335909935645759105682373046875f,-0.294099989870190814400530143757350742816925048828125f,-0.11559998000562199393925766344182193279266357421875f,-0.252299986076355065023335555451922118663787841796875f,0.72269997883141012806618164177052676677703857421875f,-0.4843999947369095337990074767731130123138427734375f,-0.6469999428391464135756905307061970233917236328125f,-0.0937999928079544798009692385676316916942596435546875f,-1.225399924433232978771002308349125087261199951171875f,0.897399948185682916346195270307362079620361328125f,-1.32659994395598790362100771744735538959503173828125f,-1.1363999925613403885193974929279647767543792724609375f,0.0967000044316057572046929635689593851566314697265625f,-0.512100011491775308769547336851246654987335205078125f,0.553899942195415828649629474966786801815032958984375f,0.9838999897301208275024464455782435834407806396484375f,-0.25239998018592657569314496868173591792583465576171875f,0.735400033509730377545565715990960597991943359375f,-1.082899979493021191956358961760997772216796875f,-1.30450002554059008019748944207094609737396240234375f,0.239200000333785300909994475659914314746856689453125f,-0.1705999912559992282012899522669613361358642578125f,-0.7127999882146707211205693965894170105457305908203125f,-0.69719998880028644094863921054638922214508056640625f,0.7788999534457925477681783377192914485931396484375f,0.635799948108196844032136141322553157806396484375f,-0.77589997641593211330501844713580794632434844970703125f,-0.3323999869287010966445450321771204471588134765625f,0.15660001162588610412740308674983680248260498046875f,-2.17019996176958063216488881153054535388946533203125f,-0.565200054603815171816449947073124349117279052734375f,0.0877000231623643600187278934754431247711181640625f,-0.32869994678646374808295149705372750759124755859375f,0.2348999635338788749550076317973434925079345703125f,0.7056999643176793401977420217008329927921295166015625f,-0.43390003428459067436051554977893829345703125f,-0.002400007641314427786483065574429929256439208984375f,1.1987999930500981005110361365950666368007659912109375f,-0.109100037106870662029223240097053349018096923828125f,-1.054399961805344343446222410420887172222137451171875f,-0.38269998024702100991589759360067546367645263671875f,-0.072300041955708937990721096866764128208160400390625f,0.041500023514032591265277005732059478759765625f,1.06649999877810497395103084272705018520355224609375f,0.9476999826312049179932728293351829051971435546875f,-0.2016000187039370672437144094146788120269775390625f,-0.4006999781012527961365776718594133853912353515625f,1.2322999857783312194214886403642594814300537109375f,0.467200009103118762965323185198940336704254150390625f,0.7708999996244898511577048338949680328369140625f,1.29620002371370812710438258363865315914154052734375f,0.454099961539356777873166493009193800389766693115234375f,0.338700010228157155633965658125816844403743743896484375f,1.126300013297051094696143991313874721527099609375f,-0.48509995642825998185543312501977197825908660888671875f,-1.70880001525282931851279499824158847332000732421875f,-1.091000005282461327027476727380417287349700927734375f,-0.8880000078007579400463100682827644050121307373046875f,-0.3735000146329403580836014953092671930789947509765625f,-0.4259000375106933855562374446890316903591156005859375f,1.473599968695641138083374244160950183868408203125f,0.4232000377058984241784855839796364307403564453125f,0.5492999923944470452852328890003263950347900390625f,0.30350001586228525507493714030715636909008026123046875f,1.2325999871805308760741581863840110599994659423828125f,-1.871899944967031270692814359790645539760589599609375f,-0.727999983269721706591326437774114310741424560546875f,0.3086999804258336865814271732233464717864990234375f,-0.6663999832108615795078776500304229557514190673828125f,0.7092999926924707931874536370742134749889373779296875f,-0.2532000092074271346831437767832539975643157958984375f,0.1524000282049178256471577697084285318851470947265625f,0.1646999600410463671096295001916587352752685546875f,-2.0606000162899480443456923239864408969879150390625f,1.380499987870454248195528634823858737945556640625f,-0.3753999841310078355860468946048058569431304931640625f,1.0580999743618069164341477517154999077320098876953125f,-0.5000000303983682670150301419198513031005859375f,0.61480002901032448914264705308596603572368621826171875f,2.13959998124241845829374142340384423732757568359375f,-1.1407999835729594195754543761722743511199951171875f,0.48879998653084022475212577774072997272014617919921875f,-1.60669998092502286368699060403741896152496337890625f,-0.7114999957978722644469371516606770455837249755859375f,0.4710000036060810746363358703092671930789947509765625f,-0.9061000425308947114899638108909130096435546875f,0.84249998725950714373311711824499070644378662109375f,0.01599997365474681298991299627232365310192108154296875f,0.943699978376925319167867201031185686588287353515625f,-0.1986000281512729781496773284743539988994598388671875f,1.0050999861747025310165781775140203535556793212890625f,-0.043000022105872393041181567241437733173370361328125f,0.9060000109449026783892122693941928446292877197265625f,-0.90430003821551852016824568636366166174411773681640625f,-0.606899976408481478529211017303168773651123046875f,-0.70139999196529334568595004384405910968780517578125f,1.2341999298885470981446133009740151464939117431640625f,0.979799972069263613860812256461940705776214599609375f,-0.16310001800358275847457889540237374603748321533203125f,1.35499997861683407762711794930510222911834716796875f,1.2035000102743504957203413141542114317417144775390625f,0.9675999618113044764555752408341504633426666259765625f,-0.654999984875321406008197300252504646778106689453125f,0.690800004583596916063470416702330112457275390625f,0.891499964989721949137901901849545538425445556640625f,0.61399998112022846630253525290754623711109161376953125f,0.178099981383979422133734260569326579570770263671875f,-0.5355999586224555741864605806767940521240234375f,1.29899993537366498230767319910228252410888671875f,0.481499992668628973291333750239573419094085693359375f,1.118799941566587019536882507964037358760833740234375f,-0.796999985828995338721369989798404276371002197265625f,0.668199965509772564331569810747168958187103271484375f,-0.730899963265657870437053134082816541194915771484375f,0.5524999837577342365335653084912337362766265869140625f,-1.6892999475419523403019184115692041814327239990234375f,1.502099956618249354534100348246283829212188720703125f,0.32830000130534198188314576327684335410594940185546875f,-0.32489998614639026275341393557027913630008697509765625f,-0.32280000136494635398776154033839702606201171875f,0.378599956402183135395489443908445537090301513671875f,0.1377999535322194102349158129072748124599456787109375f,-0.396199992942810441665102416664012707769870758056640625f,-0.242599985802173900850675636320374906063079833984375f,-1.056900007104873306928993770270608365535736083984375f,-0.0127999776720992830547629637294448912143707275390625f,-0.6073000027284030100105383098707534372806549072265625f,-0.81230000101476917961917934007942676544189453125f,0.01029999689459815126468811286031268537044525146484375f,-0.4066000224292281028937168230186216533184051513671875f,-0.1926000001817941009729651113957515917718410491943359375f,-0.28880002158582229743188918291707523167133331298828125f,-0.2708000299528243726143728054012171924114227294921875f,-0.2672000183045870613796068937517702579498291015625f,-0.0318000058427454490583841106854379177093505859375f,-1.034799996507167474391053474391810595989227294921875f,-0.812300014723837282559770756051875650882720947265625f,-0.8760999751031395277323099435307085514068603515625f,0.4919000155612824887185752231744118034839630126953125f,0.300399969322979298880227361223660409450531005859375f,-0.88380000642389067166959648602642118930816650390625f,0.63519999499917023655370940105058252811431884765625f,0.4384999922811985140214119383017532527446746826171875f,-0.6007999784320592340236544259823858737945556640625f,0.80529997677803066835622303187847137451171875f,0.2701000568807117030445397176663391292095184326171875f,0.25930000953078202741153290844522416591644287109375f,0.5568999755144119490068987943232059478759765625f,-0.789599984967707779759393815766088664531707763671875f,0.37259999802112542255372318322770297527313232421875f,-0.01370000218153055726588718243874609470367431640625f,0.82409998983293775154379545710980892181396484375f,-0.1456999931514262325293884714483283460140228271484375f,-0.16749999269843096616483535399311222136020660400390625f,-0.5922999858900903635827717153006233274936676025390625f,-0.0515000068992375048537724069319665431976318359375f,-0.305199997271596912273849966368288733065128326416015625f,-0.93550004017353050045358031638897955417633056640625f,-0.38050000593811306170977104557096026837825775146484375f,-0.03629998872876161186695753713138401508331298828125f,-0.26779999383985997329915562659152783453464508056640625f,-0.3946999897129830348063705969252623617649078369140625f,-0.643600009264051919899429776705801486968994140625f,0.339400041414797792871382853263639844954013824462890625f,-0.43889999611899266351855430912110023200511932373046875f,0.2887000144004818746878981983172707259654998779296875f,-0.434599989289045474105677158149774186313152313232421875f,1.1128000216305249381321118562482297420501708984375f,1.13269998397231130837781165610067546367645263671875f,0.027200009587406537292508801328949630260467529296875f,-1.0573999924421304541510835406370460987091064453125f,0.520600029700994060277707831119187176227569580078125f,1.48619998072385772758252642233856022357940673828125f,0.7888000338792797716536142615950666368007659912109375f,-0.56309998747482914449591362426872365176677703857421875f,-0.363799980123340471749315838678739964962005615234375f,0.75619999545738048052356816697283647954463958740234375f,-0.079400010830163569863771044765599071979522705078125f,-0.25400000363960850702227389774634502828121185302734375f,-1.088600011499225583833094788133166730403900146484375f,0.642400014793872653484640977694652974605560302734375f,-0.11939999712109550245031641679815948009490966796875f,-0.124899999520182714007887625484727323055267333984375f,0.378200010287761845262366477982141077518463134765625f,1.067400028800964406627826974727213382720947265625f,1.69300003615021754654890173696912825107574462890625f,1.35850006544589962942382044275291264057159423828125f,-1.206699972319601243953002267517149448394775390625f,-0.9354999766945848449495315435342490673065185546875f,0.904900053834916118233877568854950368404388427734375f,1.566100005799531569294913424528203904628753662109375f,0.611800012212991628501868035527877509593963623046875f,-0.20340000349879261420937837101519107818603515625f,-0.5281000126391648397117251079180277884006500244140625f,0.28280000501573088200757410959340631961822509765625f,-0.1959999931007618823031179999816231429576873779296875f,0.449400018876791040867146875825710594654083251953125f,-0.786800048944353846991361933760344982147216796875f,-0.42879997166693240018275901093147695064544677734375f,-0.0263000187546020836037996559753082692623138427734375f,-0.1887999610871080591323334374465048313140869140625f,-0.0627999774485825668079996830783784389495849609375f,0.711800016236305399530692739062942564487457275390625f,-0.43300002549588711975303567669470794498920440673828125f,0.078199993821978619479295957717113196849822998046875f,0.58640003204941759662460754043422639369964599609375f,0.224899990132450877666769883944652974605560302734375f,1.1034000366538763149293345122714526951313018798828125f,-0.4608000017523767866123307612724602222442626953125f,0.55200002367794542834644744289107620716094970703125f,-1.10339998934269001296115675359033048152923583984375f,0.6815999888271104811821032853913493454456329345703125f,1.1385999803185455903076217509806156158447265625f,0.256599972435832324180182695272378623485565185546875f,-0.525000016018748016932704558712430298328399658203125f,-0.7830000190883874555680677076452411711215972900390625f,1.823400007820129786750840139575302600860595703125f,-0.1538000289618974836258757932228036224842071533203125f,-0.4574999994039530282208261269261129200458526611328125f,-0.7345000016987324187311969581060111522674560546875f,-1.01980002347826914643746931687928736209869384765625f,-0.76880000690817862363246604218147695064544677734375f,-1.0936000733017916530798174790106713771820068359375f,1.5948000198647367398052665521390736103057861328125f,0.88819994172379457264554503126419149339199066162109375f,0.41670002117678439645942489732988178730010986328125f,1.3510999386511743924899064950295723974704742431640625f,2.477000054150819696729968200088478624820709228515625f,-0.169100007081031566968931656447239220142364501953125f,-0.850400000950693790713330599828623235225677490234375f,-0.49970002873539909327149644013843499124050140380859375f,0.6108999967932697838790545574738644063472747802734375f,0.07069996506273745762172211470897309482097625732421875f,-0.086999981358647726636945662903599441051483154296875f,1.247099978858232649514548029401339590549468994140625f,-1.136099956811964517555679776705801486968994140625f,1.864999950379133242250873081502504646778106689453125f,0.3659999971613283609173095101141370832920074462890625f,1.0781999875076113681870992877520620822906494140625f,-1.5162000313132997231235776780522428452968597412109375f,-0.7795999962925908022270959918387234210968017578125f,0.46700000669062102787165713380090892314910888671875f,0.5339999914765358735024847192107699811458587646484375f,-1.615399988806248021688816152163781225681304931640625f,0.842499992251396268017060720012523233890533447265625f,-0.197599985876679706819913917570374906063079833984375f,0.54869999055862450632048421539366245269775390625f,-0.18379997498244138309786421814351342618465423583984375f,-0.1510000163279467955135260126553475856781005859375f,0.4254000049367545610579099957249127328395843505859375f,-0.45919998897165059670300024663447402417659759521484375f,0.62080001531243311774233006872236728668212890625f,0.823099972257017586940719411359168589115142822265625f,-1.500799993482230920704978416324593126773834228515625f,-0.75599994839727902018466920708306133747100830078125f,0.441400016552210505693665254511870443820953369140625f,0.19300002095103341304138666600920259952545166015625f,2.0462000471830368297787572373636066913604736328125f,1.249700008171796383749097003601491451263427734375f,-0.19680001429915439103979224455542862415313720703125f,-0.4791999657630920950168729177676141262054443359375f,-0.303100004816055701439836411736905574798583984375f,-0.4097000431895256866710042231716215610504150390625f,0.810300079298020392570833791978657245635986328125f,0.318400033301114715467150517724803648889064788818359375f,0.6089999970644714988310397529858164489269256591796875f,-0.94149998722970440478974296638625673949718475341796875f,-0.65899999251961727253501521772705018520355224609375f,0.56480000425577159273160532393376342952251434326171875f,-1.17830005033016238513710050028748810291290283203125f,-0.0179999738633636052753672629478387534618377685546875f,0.6265000542849303855064135859720408916473388671875f,-0.709200022760033466084905739990063011646270751953125f,0.005899983829259713985493362997658550739288330078125f,0.6541000204175713150789306382648646831512451171875f,-0.713799995806813303289573013898916542530059814453125f,-1.31779996180236391722928601666353642940521240234375f,-0.2027999917536968954578924240195192396640777587890625f,1.282599963897466555096116280765272676944732666015625f,0.87880000690817805519827743410132825374603271484375f,-0.6785000030100345469463718472979962825775146484375f,0.7955999839544301011073912377469241619110107421875f,-1.09839998487234158375258630258031189441680908203125f,-1.480800038483739200501076993532478809356689453125f,0.7994999800920492560862840036861598491668701171875f,0.554000027090311153443735747714526951313018798828125f,-0.2499999943375588173921642010100185871124267578125f,0.803600026735663330867964759818278253078460693359375f,-0.70830005390644146245904266834259033203125f,0.5257000134170055627436113354633562266826629638671875f,-0.32600005561113487573265956598334014415740966796875f,-1.2222000725448136648765284917317330837249755859375f,-1.03700002054870221712690181448124349117279052734375f,1.1183000200435524451592073091887868940830230712890625f,0.193500047340988690081076128990389406681060791015625f,-0.91810005533099303676181079936213791370391845703125f,-0.4420000233054164340273928246460855007171630859375f,0.6861000712156306580169484732323326170444488525390625f,1.16669997403323577600531280040740966796875f,1.1299000507056717879095231182873249053955078125f,0.67619998839795503187133363098837435245513916015625f,0.00249996006488739652695585391484200954437255859375f,0.16320000939369183612370761693455278873443603515625f,0.96150002012401858397794285338022746145725250244140625f,-0.87379996941313164970921434360207058489322662353515625f,-0.976300001879037093743818331859074532985687255859375f,-0.5785999810077251392925745676620863378047943115234375f,0.222400051078201155263514010584913194179534912109375f,0.143499990120530274140264737070538103580474853515625f,-0.3691000209391119835089511980186216533184051513671875f,1.4410000099763269343355887031066231429576873779296875f,-1.0955000051558017037933723258902318775653839111328125f,0.66240003178119621907171676866710186004638671875f,-1.30240002492666206990179489366710186004638671875f,-0.48659998607039423035303116193972527980804443359375f,0.0089999526217583447618153513758443295955657958984375f,-0.148699956993758763967861113997059874236583709716796875f,0.60599995376169690786127830506302416324615478515625f,-0.64060001289993484530072009874857030808925628662109375f,0.319000023290514800322625887929461896419525146484375f,1.1405999943852425104751091566868126392364501953125f,0.102000020772219368581090748193673789501190185546875f,0.06189999101161902661516478474368341267108917236328125f,-0.41419997969567734497076116895186714828014373779296875f,0.3085999581158158644456079855444841086864471435546875f,-1.71330001397132747342766379006206989288330078125f,-1.836699982899428817972875549457967281341552734375f,-1.08500000268220819776843200088478624820709228515625f,-0.80640001543462336286438585375435650348663330078125f,-0.90160000566542120026269913068972527980804443359375f,-1.460100006663798577477564322180114686489105224609375f,0.2523999888658519097361931926570832729339599609375f,-0.8107000406116242618992373536457307636737823486328125f,-0.8858000311821701000525308700161986052989959716796875f,-1.1214000403568153974021015528705902397632598876953125f,1.9634000556528565795133545179851353168487548828125f,0.1710000107586377993129644892178475856781005859375f,0.9861999927937981080816598478122614324092864990234375f,-0.9499999768286946366657730322913266718387603759765625f,0.967399994304776189579797573969699442386627197265625f,-0.035099973434209508216241601985529996454715728759765625f,0.01869999987185000378531185560859739780426025390625f,0.00679997004270582738882922058110125362873077392578125f,0.2994999841898680070784166673547588288784027099609375f,0.3692999798029659164200211307615973055362701416015625f,0.161000021077692512250223444425500929355621337890625f,0.51660001054555149124780655256472527980804443359375f,-1.060499991223216031954734717146493494510650634765625f,-1.1848999340295798265998428178136236965656280517578125f,-1.362800027441977590569877065718173980712890625f,1.352599996083974520644233052735216915607452392578125f,-0.720800004062056132170255295932292938232421875f,0.087899999386072291684968149638734757900238037109375f,-1.0855999765038493176660949757206253707408905029296875f,1.00699997733533397337168935337103903293609619140625f,-1.665999984309077586175362739595584571361541748046875f,-0.2696999729678035695457083420478738844394683837890625f,0.00500000208616280250595309553318656980991363525390625f,0.5635000106468794900393959323992021381855010986328125f,-0.785799960252643092672997227055020630359649658203125f,1.07960008629560633153232629410922527313232421875f,-1.0091000637054448052509769695461727678775787353515625f,-1.1350999862492070491271078935824334621429443359375f,1.17880004803538351865199729218147695064544677734375f,0.81060001226663569440233914065174758434295654296875f,1.267400049811602258387210895307362079620361328125f,1.3493000593006616583124923636205494403839111328125f,-0.68210002408325731693139459821395576000213623046875f,-0.7386000150755045456918423951719887554645538330078125f,0.0836000166401269506621929394896142184734344482421875f,0.2008999623343346030424072523601353168487548828125f,1.1874000462353233587009526672773063182830810546875f,-0.7733000082343810532847783179022371768951416015625f,0.1450000455230480156387784518301486968994140625f,-0.60590001735687259998286435802583582699298858642578125f,-0.92740002366006368816897520446218550205230712890625f,0.5284000195920468545551784700364805757999420166015625f,-0.066799992468953295343681020312942564487457275390625f,0.05600000964105145140337072007241658866405487060546875f,0.045300007474422532283142572850920259952545166015625f,-0.2048000291407108541807957635683123953640460968017578125f,0.0106999991118907900045087444595992565155029296875f,0.8881000091746447111518136807717382907867431640625f,-1.265399999907612649252541814348660409450531005859375f,-0.644900036847591451305561349727213382720947265625f,0.54199998001754234877580529428087174892425537109375f,0.34060002090930918683397976565174758434295654296875f,0.598200026604533530161234011757187545299530029296875f,-0.23620001663565659555388265289366245269775390625f,0.221999991044401934203733617323450744152069091796875f,-0.90960001283287983397940479335375130176544189453125f,-0.398199988383054925833448578487150371074676513671875f,1.02959999230653043156280546099878847599029541015625f,0.00160000469684629109679008251987397670745849609375f,-0.1208999918013813346107099278015084564685821533203125f,0.44340001817643670545265877080964855849742889404296875f,0.32509999459385863218585654976777732372283935546875f,0.746000033110381099987762354430742561817169189453125f,-0.2735999529004100594420378911308944225311279296875f,0.6082000029861927004048993694595992565155029296875f,0.26700003343820544188247367856092751026153564453125f,-1.054300048720837157389951244113035500049591064453125f,-0.29399995798617595266222224381635896861553192138671875f,0.6494000433519488968414634655346162617206573486328125f,1.071300028496235512420753366313874721527099609375f,0.0407999881923195584221275566960684955120086669921875f,0.782099994280934129875504368101246654987335205078125f,0.43589998412728281351746772998012602329254150390625f,-0.2181000085599718030238136634579859673976898193359375f,0.468299983721971901928782244795002043247222900390625f,-1.454999955240637188325081297080032527446746826171875f,-0.31329998521208768380574838374741375446319580078125f,-0.0985000156760218459339739638380706310272216796875f,-0.8778999852113427149191693388274870812892913818359375f,-0.04669999109506572931849177621188573539257049560546875f,0.018099992150068355822867260940256528556346893310546875f,0.16919998820796633420826537985703907907009124755859375f,1.2834999600201848490854672490968368947505950927734375f,-0.62729996543824739774208865128457546234130859375f,-0.089499983072280908658058251603506505489349365234375f,0.6380000127740199644676977186463773250579833984375f,0.313800013315677528424885167623870074748992919921875f,0.5167000172466043039776195655576884746551513671875f,-0.0018000004410740899629672640003263950347900390625f,0.8643999875470995686299602311919443309307098388671875f,-0.1322000301137566946607648787903599441051483154296875f,0.2926000087127089699379212106578052043914794921875f,0.2734000167980792372901532871765084564685821533203125f,0.4778999980077145437462604604661464691162109375f,0.6210000135898585682525663287378847599029541015625f,-0.059299989935755359482527637737803161144256591796875f,0.964600003892183366360768559388816356658935546875f,-0.378099989691376148215340435854159295558929443359375f,0.2277999961495407177380911889486014842987060546875f,-0.866499989837408168824595122714526951313018798828125f,0.4204000034093855475703094271011650562286376953125f,1.394599977517128053250416996888816356658935546875f,0.3165999731063842137501751494710333645343780517578125f,0.0440999673321844642970290806260891258716583251953125f,0.1067000134468079064475887207663618028163909912109375f,1.156900005689263100094876790535636246204376220703125f,-0.09810003640651654421844796161167323589324951171875f,-0.04270001891553432127324185785255394876003265380859375f,-1.57799995255470282273790871840901672840118408203125f,-0.711199993482976910996740116388536989688873291015625f,-1.25980002392530376909007827634923160076141357421875f,0.0705000202059746783334048814140260219573974609375f,-1.09589995760321645690282821306027472019195556640625f,1.664999943375587321270359097979962825775146484375f,-0.29289997558146729783601358576561324298381805419921875f,0.94929997358173157095961869345046579837799072265625f,-0.2554999978542327188080207633902318775653839111328125f,0.9058999707534904910488648965838365256786346435546875f,0.441999966830015633689754395163618028163909912109375f,-0.1833999644577506415998868760652840137481689453125f,-0.5469000020384786608929061912931501865386962890625f,-0.837499959245324721024417158332653343677520751953125f,0.01160005118846829663681319289025850594043731689453125f,0.5445999929212035350900578123400919139385223388671875f,0.477899974128603854950370077858678996562957763671875f,1.1707999943017959321167609232361428439617156982421875f,-0.136500022508203866511422575058531947433948516845703125f,0.1468000087484717386399069027902442030608654022216796875f,-1.2259000185489659084936420185840688645839691162109375f,0.1353999779284005555712866453177412040531635284423828125f,0.35849995324015626607661033631302416324615478515625f,-0.62349997831136061421375416102819144725799560546875f,0.95560009519159827551249009047751314938068389892578125f,1.3281000026553868575973638144205324351787567138671875f,-0.1863999822050335097856077481992542743682861328125f,0.58629999273344868271351515431888401508331298828125f,0.5635999862790104231180521310307085514068603515625f,-0.21169999940246331249227296211756765842437744140625f,0.5267999772325160368069418836967088282108306884765625f,0.0592999811813237354130023959442041814327239990234375f,1.3348999386489399654465159983374178409576416015625f,-0.1376000082656745338027803882141597568988800048828125f,0.058500060975550383091103867627680301666259765625f,0.266200063462555203130932568456046283245086669921875f,-1.76659997109472666210194802260957658290863037109375f,0.29660000603794955953418366334517486393451690673828125f,0.6528999542355544694061109112226404249668121337890625f,0.85079996906295407566034327828674577176570892333984375f,-0.66569997079968457054377495296648703515529632568359375f,1.27399995727837112013958176248706877231597900390625f,-0.4547999845489860692993033808306790888309478759765625f,0.229399989894032874104823349625803530216217041015625f,0.66799997147917788975490793745848350226879119873046875f,-0.7739999707266689288331917850882746279239654541015625f,-0.5291999589085580879554981947876513004302978515625f,-1.300399947082997442748819594271481037139892578125f,0.081399942567944805205115699209272861480712890625f,-0.624199991095065964685772996745072305202484130859375f,-0.2291999904245136665537074804888106882572174072265625f,-0.42510000539720060874770979353343136608600616455078125f,-0.2037999996066093277846675846376456320285797119140625f,0.94240001270770978880619850315270014107227325439453125f,-0.7552000656306756543045821672421880066394805908203125f,0.6898000067144636204830021597445011138916015625f,-0.68760003795623791944535696529783308506011962890625f,0.44010000245422131737171866916469298303127288818359375f,0.13809998343288876210266380439861677587032318115234375f,-0.506399991667270565898206768906675279140472412109375f,-0.50829998983144708546433321316726505756378173828125f,-0.27000001933425632483221079382929019629955291748046875f,-0.68029998169541361363599207834340631961822509765625f,-0.182100003854930303504033872741274535655975341796875f,0.2236000114619731693466064825770445168018341064453125f,0.517899978971481100842311207088641822338104248046875f,1.275399984186886825909823528490960597991943359375f,0.0852999869480728367676647394546307623386383056640625f,0.79179998911619264845285215415060520172119140625f,0.3083999909818169182784686199738644063472747802734375f,-0.18529997670352482685984796262346208095550537109375f,1.289200011807679846498331244220025837421417236328125f,-0.852799995255470122401675325818359851837158203125f,-0.9381999789208166351528461746056564152240753173828125f,0.2316999997377393594177874547312967479228973388671875f,0.0355999879404904628898975715856067836284637451171875f,-0.4626999997675416143039228700217790901660919189453125f,1.6479999676793803953245287630124948918819427490234375f,-0.41719998000860147868706917506642639636993408203125f,0.96929997600317019390558925806544721126556396484375f,-0.975199997532367302000011477502994239330291748046875f,0.65229999397396998261910994187928736209869384765625f,-0.44389998384416085031034526764415204524993896484375f,0.231000006139277846983759445720352232456207275390625f,0.5896999850749968796748134991503320634365081787109375f,-0.97179999712556586199951880189473740756511688232421875f,0.3339000248968597706067384933703579008579254150390625f,-0.0602000198468566616316621775695239193737506866455078125f,2.03740004489421888678180039278231561183929443359375f,-0.554099984282255153544838321977294981479644775390625f,-0.8627999982357028585511216078884899616241455078125f,0.3483999626696110407664264130289666354656219482421875f,0.094599972301721901857263219426386058330535888671875f,0.9229000010997057135142540573724545538425445556640625f,0.18800001916289321712838500388897955417633056640625f,0.9231999559730297288950850997935049235820770263671875f,2.229999976325780064456694162799976766109466552734375f,1.32569999426901308225978937116451561450958251953125f,0.0856000106275078831430391801404766738414764404296875f,0.894800002989172948986151823191903531551361083984375f,-1.617399974262713868711216491647064685821533203125f,-0.987799942281842646707445965148508548736572265625f,0.7987999560952179223249913775362074375152587890625f,-1.2493000139638776868622471738490276038646697998046875f,1.047299983692170144422561861574649810791015625f,-0.602399974262714632544657433754764497280120849609375f,1.7664999408125883206821526982821524143218994140625f,2.65929998814761603398437728174030780792236328125f,-0.87249998778104809193933988353819586336612701416015625f,-0.44559998149573853964966474450193345546722412109375f,1.0189999755322933783219241377082653343677520751953125f,-0.8016999932557344887840145020163618028163909912109375f,-0.03650003334879858840622546267695724964141845703125f,-0.9510999418735508470490458421409130096435546875f,1.8981999913632865428780860383994877338409423828125f,-1.0751000018954268711013355641625821590423583984375f,-1.60749999180436109469383154646493494510650634765625f,-0.0289999813437464837306833942420780658721923828125f,-0.0634999934732913828838718472979962825775146484375f,0.1918999808788299787920550443232059478759765625f,-0.808999970372766608051051662187092006206512451171875f,-0.5396999819457526381683010185952298343181610107421875f,-0.8891000000774855749341440969146788120269775390625f,-1.020299976181984646927958237938582897186279296875f,-0.5757999830514197725506164715625345706939697265625f,-0.31609998870044930807665650718263350427150726318359375f,-0.9515999986618755013267900721984915435314178466796875f,-0.2226999989107250887965250285560614429414272308349609375f,0.1497000019319355834657159221023903228342533111572265625f,-0.412899979194999044995739723162841983139514923095703125f,0.11410003136992397099191975939902476966381072998046875f,0.8955999700777239613813662799657322466373443603515625f,-0.3221000932991504583924324833787977695465087890625f,-1.4447999873429535622193498056731186807155609130859375f,-0.5498999575734142553073979797773063182830810546875f,-0.1760000143349167434081437022541649639606475830078125f,-0.40339995656013538649631300359033048152923583984375f,0.3777999950692059183410265177371911704540252685546875f,0.5486999668657783768566105209174565970897674560546875f,0.237099986718595168166956455024774186313152313232421875f,-0.3602999696440998267377153752022422850131988525390625f,0.12580000663250678893945178060675971210002899169921875f,0.029099980333448183245081963832490146160125732421875f,0.4229999668411907887133338590501807630062103271484375f,0.91209999986886902689775524777360260486602783203125f,-0.820799962487816170408905236399732530117034912109375f,0.09360001086592628638527457951568067073822021484375f,-0.8234999799132349806995989638380706310272216796875f,-0.12750001356005624675304943593800999224185943603515625f,0.49759997306168113340163472457788884639739990234375f,0.625599989691376645595255467924289405345916748046875f,-0.0323000097692009990879569159005768597126007080078125f,-0.241999996948987217137982952408492565155029296875f,-0.35159997675716925247257904629805125296115875244140625f,-0.215700028765201057279199403637903742492198944091796875f,1.24349996178969757210097668576054275035858154296875f,-1.05309993716478356162724594469182193279266357421875f,0.68279995897114320602128145765163935720920562744140625f,0.838100008094312176609719244879670441150665283203125f,0.898399979172646911962374360882677137851715087890625f,0.02709999338686508441043088168953545391559600830078125f,-0.0944000026345250820014598502893932163715362548828125f,0.094400011575222375626026405370794236660003662109375f,-0.675000008344650570535350198042578995227813720703125f,1.2378000357866287828301210538484156131744384765625f,0.6350999808102846255764006855315528810024261474609375f,-0.62209999524950898575070823426358401775360107421875f,0.308999982312321730404391928459517657756805419921875f,1.464100016170740037324549120967276394367218017578125f,1.89979997668862399251565875601954758167266845703125f,0.108799950730801953113768831826746463775634765625f,-1.641599973590671890377734598587267100811004638671875f,0.17599997060000927007195059559307992458343505859375f,0.2756999632373453579248234746046364307403564453125f,-0.0803999879121786165114826872013509273529052734375f,0.94850001236051351138911513771745376288890838623046875f,-1.1735999588608745369811003911308944225311279296875f,-1.038000013425945500245006769546307623386383056640625f,1.410299916875363379631380666978657245635986328125f,0.3618999997660521383835430242470465600490570068359375f,-0.29509999444484691366596962325274944305419921875f,-0.00930005125403265964223464834503829479217529296875f,-0.5656999868929393215921663795597851276397705078125f,0.0209999963045124360405679908581078052520751953125f,-0.77269996378123817493133174139074981212615966796875f,0.075199998128414602405200639623217284679412841796875f,0.9944000108301633833463029077393002808094024658203125f,-0.928100001314282163633606614894233644008636474609375f,-0.440899996346235667488144827075302600860595703125f,0.6158999419569983757583031547255814075469970703125f,0.9386999936878677175400298438034951686859130859375f,-0.22430002547502425613856757991015911102294921875f,-1.28119997252821971045477766892872750759124755859375f,-0.01779999343007798717763989770901389420032501220703125f,-0.97010001299679249342489129048772156238555908203125f,-0.98329999046474636514858502778224647045135498046875f,-0.80810000183582264554615903762169182300567626953125f,-0.62390000954866431381873326245113275945186614990234375f,-0.6521999841809271725168173361453227698802947998046875f,0.369299959462881688398994128874619491398334503173828125f,0.048999970763922373606646942789666354656219482421875f,-0.155900018958747443864609749653027392923831939697265625f,1.18830002067685125410889668273739516735076904296875f,0.6282999647229912110191207830212078988552093505859375f,0.5626999935090548632388163241557776927947998046875f,0.0578000169739123492718135821633040904998779296875f,0.42799999748170447499973079175106249749660491943359375f,0.18959999439269281662490129747311584651470184326171875f,-1.406399962684512505717293606721796095371246337890625f,0.126099972271919824606811744160950183868408203125f,-0.32179995376318737410059611647739075124263763427734375f,-0.171699995993822795981031958945095539093017578125f,-0.3105999875679612554080222253105603158473968505859375f,-0.88299998418241754105650898054591380059719085693359375f,0.3451999783098698149075289620668627321720123291015625f,0.5618999773025514787860856813495047390460968017578125f,0.2251999698162077123697599745355546474456787109375f,-0.19170003835037430217624887518468312919139862060546875f,0.364300028753280713544882019050419330596923828125f,-0.55550003689527560624128454946912825107574462890625f,0.3441999945223326307086608721874654293060302734375f,-0.1474000073432915058901926386170089244842529296875f,-1.1326000413835048963306917357840575277805328369140625f,-1.6747000049680469402346716378815472126007080078125f,-0.03340000633001327656756984652020037174224853515625f,0.26330002358257775796346322749741375446319580078125f,-1.26250003010034550499085526098497211933135986328125f,-0.603799999308586077262361868633888661861419677734375f,1.44010002007484416708393837325274944305419921875f,-0.0292000201709574636055322116590104997158050537109375f,-0.84060003312826214738606722676195204257965087890625f,0.6768999919056895020474939883570186793804168701171875f,0.7929999883174898567261834614328108727931976318359375f,-0.185900012886524024224854656495153903961181640625f,0.3683000005602836690599133362411521375179290771484375f,-0.543900001241266917162420213571749627590179443359375f,0.8034000043928621881406115790014155209064483642578125f,-0.5688999947592614514491060617729090154170989990234375f,0.2967999952256674678352510454715229570865631103515625f,-0.8643999951094389899708403390832245349884033203125f,-1.171199994805455002477856396581046283245086669921875f,0.61900001924857483981412542561884038150310516357421875f,-0.52939997104406355532546513131819665431976318359375f,0.27319997094869652443094310001470148563385009765625f,-1.5121999928236011800919413872179575264453887939453125f,-0.6038999773621558375680251629091799259185791015625f,-0.543200004029273930683530124952085316181182861328125f,0.62130001391172384739292056110571138560771942138671875f,-0.41580002060234566574337122801807709038257598876953125f,-0.5538999905496833076057328071328811347484588623046875f,1.0625999627053739349236138878040947020053863525390625f,0.289700007426738626037376889144070446491241455078125f,0.43079998051822154803147668644669465720653533935546875f,0.21419998817071295160729960116441361606121063232421875f,0.01249998351558978537667599084670655429363250732421875f,0.6574000156693153851250599473132751882076263427734375f,-1.54960006677508399519638260244391858577728271484375f,-1.834500002592802214707035091123543679714202880859375f,-0.7044000103086229813698082580231130123138427734375f,-1.0725000129267574511260363578912802040576934814453125f,-0.7771999566510310319955578961526043713092803955078125f,-0.334900039418041917116397598874755203723907470703125f,0.724299984049796563567724660970270633697509765625f,0.9044999625086784789118610206060111522674560546875f,1.90210004993677106455152170383371412754058837890625f,0.54810001628994908795533547163358889520168304443359375f,0.9114999883845447126162753193057142198085784912109375f,-1.1357000021480023743691845083958469331264495849609375f,0.515100004354119622718144455575384199619293212890625f,-1.0895000085532664524379242720897309482097625732421875f,0.0716999948948620069444359614863060414791107177734375f,0.1702000052064655821748573316654074005782604217529296875f,0.37699998478591389972081060477648861706256866455078125f,0.6538999983355402445539539257879368960857391357421875f,-0.67169994035661151343674646341241896152496337890625f,1.293400001934170706618942858767695724964141845703125f,-0.90720003119409131642214560997672379016876220703125f,-0.84459997377321105460623584804125130176544189453125f,-0.5110000279322266525383611224242486059665679931640625f,-0.22179998647123555510773940113722346723079681396484375f,-0.70800000552833086686632668715901672840118408203125f,0.840200009378790912961676440318115055561065673828125f,-0.263799989324807970803021817118860781192779541015625f,0.39110000158995372743220286793075501918792724609375f,-0.51600000338256302168105094096972607076168060302734375f,0.8875999800652263349576287509989924728870391845703125f,1.03439999078810185295651535852812230587005615234375f,-0.7059999882578846808200978557579219341278076171875f,1.22460002557337244155633015907369554042816162109375f,1.2703999895140529741155432930099777877330780029296875f,1.023700029115378828947768852231092751026153564453125f,0.686800022457540126907815647427923977375030517578125f,0.032100000464916345066512803896330296993255615234375f,-0.435999996230006292563530223560519516468048095703125f,-1.68600007781386462824002592242322862148284912109375f,-2.29710007601380450381611808552406728267669677734375f,0.31180006168484819539798991172574460506439208984375f,-0.61710002654194884286198430345393717288970947265625f,0.4561999954760072029102957458235323429107666015625f,0.346699991020560371879355443525128066539764404296875f,0.695700017067789833191682191682048141956329345703125f,1.22600000703334810481237582280300557613372802734375f,-0.33560000697672354164069474791176617145538330078125f,0.162100004450976886705149127010372467339038848876953125f,0.6988999898046248659966295235790312290191650390625f,-0.6839000176697973554240661542280577123165130615234375f,0.645500016704202206341278724721632897853851318359375f,-0.17939999980330478823731255033635534346103668212890625f,0.017899975693225211870185376028530299663543701171875f,0.8313000149548053219206167341326363384723663330078125f,0.2220000219643118999357511711423285305500030517578125f,0.361200023713708240347131095404620282351970672607421875f,-0.846999985456466930600072373636066913604736328125f,-0.857700005541742083181588895968161523342132568359375f,-0.145999987885355952954569147550500929355621337890625f,-0.3462000057578087375276254533673636615276336669921875f,-1.58980004016757003881821219692938029766082763671875f,0.028200015428662350558397520217113196849822998046875f,-1.263000007018446613216156038106419146060943603515625f,1.1722000239670276044989805086515843868255615234375f,-1.463800007653235812909997548558749258518218994140625f,0.1270999849677085791199715458787977695465087890625f,0.2397999877154826098291096059256233274936676025390625f,-0.9400999866217372158416765159927308559417724609375f,-0.111599993669987185285208397544920444488525390625f,0.517199957507849195081917059724219143390655517578125f,-0.168799999010562640222588015603832900524139404296875f,1.543200016471743207802092001657001674175262451171875f,0.21179999254345904802221411955542862415313720703125f,0.7139000584244730163163694669492542743682861328125f,-1.3810000402629380999997010803781449794769287109375f,-0.10990001449584951842552982270717620849609375f,0.05860005378127208786054325173608958721160888671875f,-0.7544000383973124002068288973532617092132568359375f,-1.48990001434683794201419004821218550205230712890625f,0.350899981296063145919106318615376949310302734375f,-0.608300034460425020910179227939806878566741943359375f,0.95419999884367001641294336877763271331787109375f,0.7570000044554472395930133643560111522674560546875f,-0.45599998296797306096550528309307992458343505859375f,-1.0664999883472920938487504827207885682582855224609375f,-0.67939998505115450910807339823804795742034912109375f,0.435900005212426666645342265837825834751129150390625f,-0.6056000384926800705898131127469241619110107421875f,-0.62610002532005371023160478216595947742462158203125f,1.0872999944955112905375926857232116162776947021484375f,2.013899994200468057670150301419198513031005859375f,-0.37019999105036216491271261475048959255218505859375f,-0.74350000796467041741522052689106203615665435791015625f,-0.432699981811642597762102013803087174892425537109375f,1.0579999617934230382587657004478387534618377685546875f,-0.580699978175759579102077623247168958187103271484375f,2.46679998658299393099468943546526134014129638671875f,-0.56340002957582413500858820043504238128662109375f,0.145800006595254050267840284504927694797515869140625f,1.51460000910758907366471248678863048553466796875f,-0.92719998120069480052052313112653791904449462890625f,0.446699964869022636548834270797669887542724609375f,0.0529999848902225068059124168939888477325439453125f,-0.5664999756813049458514797152020037174224853515625f,1.519599958257377370074436839786358177661895751953125f,-1.46410003353059270381208989419974386692047119140625f,2.21520006205439567992243610206060111522674560546875f,1.5581999870419505072050014859996736049652099609375f,1.279100008869170412850735374377109110355377197265625f,0.6165999845802776491154872928746044635772705078125f,-0.596199995625018797085203914321027696132659912109375f,-0.5296999937921762491299659814103506505489349365234375f,0.8113000050082799230466434892150573432445526123046875f,0.66089998675361283364537712259334512054920196533203125f,0.13910003012195260030381405158550478518009185791015625f,0.1240000065453348288002644039806909859180450439453125f,-1.3243000540852545299230769160203635692596435546875f,0.54629998790919820361722258894587866961956024169921875f,-0.921299999010563031021092683658935129642486572265625f,1.58719998544752538549573728232644498348236083984375f,-0.10630000504553294149445719085633754730224609375f,0.449400007700920145481404688325710594654083251953125f,0.46180000401735277204551266549970023334026336669921875f,0.1116000000029802041723314687260426580905914306640625f,-0.408699989105761052332610461235162802040576934814453125f,0.5363999959141014794994362091529183089733123779296875f,0.6343999473884698314662955453968606889247894287109375f,-0.80919999394491293198683479204191826283931732177734375f,0.359900031483173510338247069739736616611480712890625f,0.939199977236986338624546988285146653652191162109375f,-0.278199978995323338182288352982141077518463134765625f,-0.7134999954104426223011614638380706310272216796875f,-0.828999978139996596127048178459517657756805419921875f,-0.543499994143843423444195650517940521240234375f,-0.39879998746216294147615144538576714694499969482421875f,0.4779999895840882917497083326452411711215972900390625f,0.323199996466934480476851376806735061109066009521484375f,-0.545199991441517983759013077360577881336212158203125f,-1.0652000124707814432412078531342558562755584716796875f,-0.13459999878853567967240678626694716513156890869140625f,-1.801000043094158176160135553800500929355621337890625f,0.4012999842584132892397974501363933086395263671875f,-0.879899977988003456630394794046878814697265625f,-1.1412999462604513922769911005161702632904052734375f,0.2458000097990034049644236802123486995697021484375f,-0.280500022813676697097662326996214687824249267578125f,-0.188800023224950042077807665918953716754913330078125f,1.10790003000795866938688050140626728534698486328125f,0.837199998933076727780644432641565799713134765625f,0.5773000441908830993043011403642594814300537109375f,-2.5138000268757334509928114130161702632904052734375f,-1.519999962449073649395359097979962825775146484375f,0.69630002747178121325077881920151412487030029296875f,-0.2816999831229447526226294939988292753696441650390625f,-0.61980002221167129761170144774951040744781494140625f,-0.5335000195503230013827078437316231429576873779296875f,-0.250300018873810703468052452080883085727691650390625f,0.632299986970424665599921354441903531551361083984375f,0.5287000342935328500715286281774751842021942138671875f,-0.10199998702108870052285283236415125429630279541015625f,-0.61670004280209500535647748620249330997467041015625f,-0.9307999950468541783266118727624416351318359375f,1.169300011691450702500105762737803161144256591796875f,-0.624199994000792290904655601480044424533843994140625f,-0.119900064712762599583584233187139034271240234375f,-0.48589996371269261743464085157029330730438232421875f,0.10280002975165825063186275656335055828094482421875f,1.011899947500229046681852196343243122100830078125f,-0.165999964788556386707796264090575277805328369140625f,-0.23059999118149310248782057897187769412994384765625f,1.2873999959439037521491400184459052979946136474609375f,-0.595199982184171805243977360078133642673492431640625f,0.05560002225041404955163670820184051990509033203125f,-0.2417999944806101808580933720804750919342041015625f,0.166400019606947946471109389676712453365325927734375f,1.193899975275993341483626863919198513031005859375f,0.058400010949373637458847952075302600860595703125f,-0.3548999711334703821563607561984099447727203369140625f,0.2318000338196757326159058720804750919342041015625f,-1.027799989145993464489947655238211154937744140625f,2.179300046932696854895539217977784574031829833984375f,1.3975000053644190955992598901502788066864013671875f,-0.9153999929785723299602295810473151504993438720703125f,-0.5001999646008012945230802870355546474456787109375f,-1.172600003981589367185733863152563571929931640625f,-0.60709998049735958147721248678863048553466796875f,1.15510003870129640546338123385794460773468017578125f,1.2439000301122664637887282879091799259185791015625f,-2.423500077068805325097855529747903347015380859375f,0.780399980014562455465920720598660409450531005859375f,-1.03899996703863184421834375825710594654083251953125f,0.57849997603893399400476482696831226348876953125f,1.1904000201731914643232812522910535335540771484375f,-0.146200027513504249299103321391157805919647216796875f,-0.3919000147044655602002194427768699824810028076171875f,-0.755199981364607086931073354207910597324371337890625f,0.991799994629621206598812932497821748256683349609375f,-1.41889995292425208361919430899433791637420654296875f,0.8835999521553521418582022306509315967559814453125f,-0.92409997549057010246542631648480892181396484375f,0.60450001168250988570207482553087174892425537109375f,0.5256999781385063297278747995733283460140228271484375f,0.824800007943809365684728618361987173557281494140625f,0.83549999994039492179354056133888661861419677734375f,0.8042999891534445833229938216391019523143768310546875f,0.706600009353458791139246386592276394367218017578125f,0.35740002604425036292212780608679167926311492919921875f,-0.617799991604685683199704726575873792171478271484375f,-0.3888999871969218702361104078590869903564453125f,-1.1149999696016319461477905861102044582366943359375f,-0.7465999634020035191639408367336727678775787353515625f,0.318100009249150605938893932034261524677276611328125f,-0.062100013019145305914747723363689146935939788818359375f,0.5306000030651689680638583013205789029598236083984375f,0.124099994452297135438811892527155578136444091796875f,-0.049700010034441799433579944889061152935028076171875f,-0.3201999695181854121983633376657962799072265625f,-0.61959999904930551650750203407369554042816162109375f,-1.098499991759657401502181528485380113124847412109375f,0.50179996326267783945951350688119418919086456298828125f,-1.338399963414669713301918818615376949310302734375f,-0.079100039342045125323465981637127697467803955078125f,0.404400010941922349871902042650617659091949462890625f,0.32400001028925184964890604533138684928417205810546875f,1.69159996278732993602034184732474386692047119140625f,0.7728999572157864150767636601813137531280517578125f,-0.044799978737533496797595944372005760669708251953125f,0.4657999913215633558394301871885545551776885986328125f,-1.1303999707758427550885471646324731409549713134765625f,1.0909999574124820131970636793994344770908355712890625f,-0.66749997988343301447144995108828879892826080322265625f,0.93629993989020732225725396347115747630596160888671875f,0.9988000036299222728075619670562446117401123046875f,0.160099977457523579005282954312860965728759765625f,1.117499946169556057640193102997727692127227783203125f,0.326099984062462766587486839853227138519287109375f,0.17429998539090207909652008311240933835506439208984375f,0.218400005193799484626282492172322236001491546630859375f,-0.8657999915823335612685696105472743511199951171875f,0.3328999954931439742011889393324963748455047607421875f,-0.8016000263035294448599188399384729564189910888671875f,0.2437999992713331121052533490001223981380462646484375f,0.44120000054240282594264499493874609470367431640625f,-0.69589995350539746166163013185723684728145599365234375f,-1.501799957972764776314988921512849628925323486328125f,-1.2107999627113346097218027352937497198581695556640625f,-0.97459997397810216313018827349878847599029541015625f,-1.2371999492749574667271872385754249989986419677734375f,-1.1686999708890921656490036184550262987613677978515625f,0.25489998720809825538680115641909651458263397216796875f,0.3957999655053028309481533142388798296451568603515625f,-0.14719999826252483376975987994228489696979522705078125f,0.93249997630715331098372189444489777088165283203125f,0.78969997706562289341292171229724772274494171142578125f,-1.37279992969036168659613394993357360363006591796875f,-0.065300002631544729059243081792374141514301300048828125f,-0.2592999962687496662283592741005122661590576171875f,0.1973999870777134457711099457810632884502410888671875f,-0.585799973253906092196530153159983456134796142578125f,-1.20079998741000881778973052860237658023834228515625f,0.518600004905461986481896019540727138519287109375f,0.48929998777508731944152486903476528823375701904296875f,-0.169300002676248506627842971283826045691967010498046875f,0.8296999853543933767241469467990100383758544921875f,-0.445499951362610335792169280466623604297637939453125f,0.58419997842907900320597036625258624553680419921875f,-1.233599964523315772879641372128389775753021240234375f,0.6628999882102011298457000521011650562286376953125f,1.683399962818622697113823960535228252410888671875f,-0.4092999971628188404793036170303821563720703125f,0.517999963320792122800639845081605017185211181640625f,-1.2508999359965329301047631815890781581401824951171875f,0.59549997073411986558966191296349279582500457763671875f,-0.713299996089934662535370080149732530117034912109375f,-0.3016999957889314742942588054575026035308837890625f,-0.405800023955106947681770179769955575466156005859375f,-0.155999998763203251428421935997903347015380859375f,0.5738000018790361256293408587225712835788726806640625f,-0.5274000343143949720570162753574550151824951171875f,-0.4206000003084533478414641649578697979450225830078125f,0.23009997388124414641197290620766580104827880859375f,-0.9518999942526225055416944087482988834381103515625f,0.3336000308334821085054500144906342029571533203125f,-0.16849999868869769414914117078296840190887451171875f,0.404299978536367010661933818482793867588043212890625f,-0.2076999945521358359457053666119463741779327392578125f,0.004000009357928657749425838119350373744964599609375f,0.2046000081762671118923435642500407993793487548828125f,0.493999995052814444562727658194489777088165283203125f,-0.558799983140826128646949655376374721527099609375f,0.681299968612194106754031963646411895751953125f,-0.1131000424787400593373831725330092012882232666015625f,-0.619799941372871643352482351474463939666748046875f,0.5301000211924311589228864249889738857746124267578125f,-0.347299985778331432584309368394315242767333984375f,-0.323199943828583524663144999067299067974090576171875f,-0.3034999837875371753170838928781449794769287109375f,-0.959399948841333394256025712820701301097869873046875f,0.4356000284343950479382101548253558576107025146484375f,-1.1321999760597947215501335449516773223876953125f,-0.189299994927644821274270725552923977375030517578125f,0.022600001522898427452901159995235502719879150390625f,-0.15950000184774371092544242856092751026153564453125f,-0.8375999789476384904673977871425449848175048828125f,0.321399983769655506193885230459272861480712890625f,-1.22239995846748428220962523482739925384521484375f,1.1574999250471602607603927026502788066864013671875f,-0.877199940669537880921780015341937541961669921875f,-0.50569998823404294086003574193455278873443603515625f,0.4181999925181270061358418388408608734607696533203125f,0.380900001710653857145416623097844421863555908203125f,-0.3922999783277514307400224424782209098339080810546875f,-0.11889996462166319002307091068360023200511932373046875f,-0.012700004275143006449155791415250860154628753662109375f,0.038000000536441813636656661401502788066864013671875f,1.0032999675914648918251259601674973964691162109375f,0.270399950659274690423217180068604648113250732421875f,-0.141599997618794215537008085448178462684154510498046875f,-0.0929000248670579509280287311412394046783447265625f,-0.8606999705389146715361903261509723961353302001953125f,-0.786699983383715828466620223480276763439178466796875f,1.86310000138878795183927650214172899723052978515625f,-0.4237000075459480541439916123636066913604736328125f,-2.058499935731292307394824092625640332698822021484375f,0.05760000778138607557821160298772156238555908203125f,0.52709997811317454985413633039570413529872894287109375f,0.980899965500831694242833691532723605632781982421875f,0.46959999405741703615291271489695645868778228759765625f,0.446799967062473601941974266082979738712310791015625f,-0.7580000173375009975273997042677365243434906005859375f,-0.96799993348121660119431908242404460906982421875f,-0.75589995846003343871188917546533048152923583984375f,-0.407499991208315037738429964520037174224853515625f,-0.145599991106987403099992661736905574798583984375f,0.31429999179840084622128415503539144992828369140625f,-1.3700999121904384736581050674431025981903076171875f,-0.1513999996393919911241709996829740703105926513671875f,-0.274999995380639905562247804482467472553253173828125f,0.089400010383128947211162085295654833316802978515625f,0.49779996150434069335233289166353642940521240234375f,-0.288099991106987030065056387684307992458343505859375f,-0.894899965173005629281988149159587919712066650390625f,-0.39590001124739604421165495296008884906768798828125f,-0.735099941992761074516238295473158359527587890625f,0.9763999697625640461495777344680391252040863037109375f,0.50199998594075456725960293624666519463062286376953125f,-0.37600001761317169979292884818278253078460693359375f,1.0934000002950423624525910781812854111194610595703125f,0.5427000178351997217163216191693209111690521240234375f,0.41030002475976967701853936887346208095550537109375f,0.1248000097319483803204320793156512081623077392578125f,0.52569997851103555941421063835150562226772308349609375f,-0.0171000158131118240589785273186862468719482421875f,0.80959998712837710765910514965071342885494232177734375f,0.66329996286034553776289612869732081890106201171875f,-0.20339996948689265110488122445531189441680908203125f,-1.12980000708699268585633035399951040744781494140625f,-1.11290000784248110932139752549119293689727783203125f,1.00169999735355386150104095577262341976165771484375f,-0.32759995424896504179201883744099177420139312744140625f,0.3550999887824060063934439313015900552272796630859375f,1.6098999485582112800585719014634378254413604736328125f,0.2762999781489370310083586446125991642475128173828125f,0.365300006617605588221664447701186873018741607666015625f,0.60369995360374506532252780743874609470367431640625f,-0.9938000033691525292312007877626456320285797119140625f,-0.669299938154221507602414931170642375946044921875f,0.91379996090084336657355379429645836353302001953125f,-0.11829998327493740362115204334259033203125f,-0.4028999810576436857445514760911464691162109375f,-0.96109996988773360726554528810083866119384765625f,0.63019999514818092478662947542034089565277099609375f,-1.440699982832371883745281593292020261287689208984375f,1.13689998881369813688024805742315948009490966796875f,0.6208000029444693534941279722261242568492889404296875f,0.5513999764308332718343308442854322493076324462890625f,-0.4931999819755557012967983609996736049652099609375f,0.39569999210834527758606782299466431140899658203125f,-1.5266999501168729835853810072876513004302978515625f,0.881099979424476487110950984060764312744140625f,1.86549996492266689784855770994909107685089111328125f,0.1987999900698660393771888266201131045818328857421875f,0.843600006768106869259327140753157436847686767578125f,0.596199993799626515311729235691018402576446533203125f,0.81330000562667859043131102225743234157562255859375f,0.4770000061690808923486883941222913563251495361328125f,0.235600013160705668013861213694326579570770263671875f,0.65519998725056627364438099903054535388946533203125f,0.81909999866187543915430069318972527980804443359375f,0.26730001140832915229594846096006222069263458251953125f,-0.073900045683980675192970011266879737377166748046875f,-0.27609999667257068001191555595141835510730743408203125f,-0.97590002033710465667581956950016319751739501953125f,0.52920002693235879132060972551698796451091766357421875f,0.411400005674361413099404671811498701572418212890625f,0.164000001683831442278460599482059478759765625f,0.0339999950528149241790742962621152400970458984375f,-1.7924999329075219645091010534088127315044403076171875f,0.894599993014335126417790888808667659759521484375f,1.307499972246587649493676508427597582340240478515625f,0.056399985408783326334969387971796095371246337890625f,-0.7813999901771546685580460689379833638668060302734375f,-0.496699958908558780734665560885332524776458740234375f,-1.116599998810887228728461195714771747589111328125f,0.267000024795532198140790569595992565155029296875f,-0.658399992546438994622803875245153903961181640625f,0.785299984377622717346412173355929553508758544921875f,-0.584799991217255676900776961701922118663787841796875f,-0.4341000242173667089673472219146788120269775390625f,-0.6511999580740930326783200143836438655853271484375f,-0.1935999751031403093293192796409130096435546875f,0.0508999753355983131086759385652840137481689453125f,-0.1798000238507981851654449201305396854877471923828125f,-0.4387999651104212972541063209064304828643798828125f,-0.483899977660179114735683469916693866252899169921875f,-0.250499995693564070364800500101409852504730224609375f,-0.323099987530708343541618887684307992458343505859375f,-0.546899968175590611707548305275849997997283935546875f,-0.0912999436154964261191935293027199804782867431640625f,-0.08219999246969822426223117872723378241062164306640625f,1.5826000291645521134142882146988995373249053955078125f,0.766399996249378379076233613886870443820953369140625f,-0.41620004234015883781694356002844870090484619140625f,0.1638000020653007648974153198651038110256195068359375f,-1.439800036963820861757312741247005760669708251953125f,-0.6378999938726426233159827461349777877330780029296875f,0.04710000921934831552562172873876988887786865234375f,-0.364099989870190710039565828992635942995548248291015625f,-1.18900001598894622389934738748706877231597900390625f,0.09540000475049037476082958164624869823455810546875f,-0.4819999523758884407698133145458996295928955078125f,0.167099982583522432832978665828704833984375f,-1.272999998226761686481722790631465613842010498046875f,0.3397999965816733691781337256543338298797607421875f,0.908600016379356834050895486143417656421661376953125f,-0.798299998027086399332574728759936988353729248046875f,1.148599986204505096765160487848334014415740966796875f,-0.1310000109076501217941768118180334568023681640625f,0.098200017738342193496237086947076022624969482421875f,-0.13630002031922305860689448309130966663360595703125f,-0.0393999537587166326346732603269629180431365966796875f,-1.122299902629851686697293189354240894317626953125f,-0.97550005902350012121360123273916542530059814453125f,-0.3791999932184808574930912072886712849140167236328125f,-0.172500055767596638389704821747727692127227783203125f,-1.0945000088140373062373100765398703515529632568359375f,0.8804000171929600160325435354025103151798248291015625f,-0.1472999410375950279927792507805861532688140869140625f,0.8470999913752077503659165813587605953216552734375f,0.54660005238056186271933256648480892181396484375f,-0.55400002701580586261798089253716170787811279296875f,0.14590000387132062709127922062180005013942718505859375f,-2.16000006064772609448709772550500929355621337890625f,-1.42449997837841468850683668279089033603668212890625f,0.4786000084072352667163841033470816910266876220703125f,-1.12380003246367010660833329893648624420166015625f,-0.3752000048339361182314632969792000949382781982421875f,-0.5474000048473477431087985678459517657756805419921875f,-0.3180000257566570898148938795202411711215972900390625f,-0.060199969853460544300105539150536060333251953125f,-0.18309998648762704220871455618180334568023681640625f,-0.3593999972701080736214862554334104061126708984375f,0.95750001348555058289235830670804716646671295166015625f,-0.72780000758618135225930245724157430231571197509765625f,1.0045000095404683637667631046497263014316558837890625f,0.36479997236728678444706019945442676544189453125f,0.2121999733775850049966038568527437746524810791015625f,0.2591000278308985826924981665797531604766845703125f,-0.49579997865557690062132678576745092868804931640625f,0.2388000114902855131049363990314304828643798828125f,-0.02819994837343708926624685773276723921298980712890625f,-0.8214000051900745003052861648029647767543792724609375f,0.31679999135136593935158089152537286281585693359375f,-0.39079997701644941798804211430251598358154296875f,-0.5992000234305858707983816202613525092601776123046875f,0.40080000169649754315770451285061426460742950439453125f,0.0139999777302148853408425566158257424831390380859375f,-0.28780001278296063471628940533264540135860443115234375f,0.00100002495199447150042715293238870799541473388671875f,-0.79129999426081709668068242535809986293315887451171875f,-0.33240002582073291392816827283240854740142822265625f,-0.761600038820504199321703708847053349018096923828125f,1.905299998086690038689994253218173980712890625f,0.296700007337332039014654583297669887542724609375f,0.120999971568583220005166367627680301666259765625f,0.2597000113010405897284726961515843868255615234375f,1.40090001474916814316884483559988439083099365234375f,0.6432999599918722832825324076111428439617156982421875f,0.9136000243887296701217337613343261182308197021484375f,0.51939999795928593062654954337631352245807647705078125f,0.16469999179914562859039506292901933193206787109375f,0.10639997659847189037662928967620246112346649169921875f,0.5231000132352117137912728139781393110752105712890625f,0.48379997226297855039689466138952411711215972900390625f,-1.108799990218877429271060464088805019855499267578125f,0.663499996453523532835561127285473048686981201171875f,0.334200034904479803543608795735053718090057373046875f,0.1404999806433913089875886726076714694499969482421875f,-1.2725999816298487754551160833216272294521331787109375f,1.3995999652981756522507339468575082719326019287109375f,0.403699990782141693301809937111102044582366943359375f,-0.3683999900691212747716463127289898693561553955078125f,-1.41479997478872565608298828010447323322296142578125f,0.51159996884092728830495389047428034245967864990234375f,-1.1007999757498507786834807120612822473049163818359375f,0.168000019162893199364816609886474907398223876953125f,-0.19379997203946175421407360772718675434589385986328125f,-1.0265999897956845643420820124447345733642578125f,-0.017199979710579516023472024244256317615509033203125f,0.19430000073909781121983542107045650482177734375f,0.1686000347077840633147616244968958199024200439453125f,0.1196999813869599194049442303366959095001220703125f,-0.597599981853365935791089214035309851169586181640625f,-0.96060000585913574155938476906158030033111572265625f,1.170700026381015224075099467881955206394195556640625f,1.7818999834120285985505915959947742521762847900390625f,-0.23240000540614147439555381424725055694580078125f,-0.341200010079145243935272446833550930023193359375f,-0.3423999921441076565997718716971576213836669921875f,-0.5336000697255141034247571951709687709808349609375f,1.353300063890220172879708115942776203155517578125f,1.95290000877380265364990918897092342376708984375f,-0.2141999879658218475242392742075026035308837890625f,0.9864000193834296936756800278089940547943115234375f,0.1962999793410293847273351275362074375152587890625f,0.7443999971956021255437008221633732318878173828125f,-0.2281999878615135202153396676294505596160888671875f,0.8891999717980623163526843200088478624820709228515625f,-0.0805999957263470179213982191868126392364501953125f,0.07929997130930421800343310678726993501186370849609375f,-0.313700023564696284239516899106092751026153564453125f,1.043800018605589574605119196348823606967926025390625f,-0.409799990397691882293429443961940705776214599609375f,0.814300051403045177522699304972775280475616455078125f,0.13490001807212781415046265465207397937774658203125f,-0.608499988034367333966656588017940521240234375f,-0.856099995666741886424233598518185317516326904296875f,0.68169995399117500056718199630267918109893798828125f,0.353400015941261802510098277707584202289581298828125f,-0.6093999883294107799969197003520093858242034912109375f,-0.5505999899148943388382804187131114304065704345703125f,0.538499988704919996251874181325547397136688232421875f,0.196000020518899642496535307145677506923675537109375f,0.3462000132083886683176388032734394073486328125f,1.173799945515395126705016082269139587879180908203125f,-1.15029999048709896669606678187847137451171875f,1.09759994109869030154413849231787025928497314453125f,0.0098999769449239494178982567973434925079345703125f,-0.134599997633695789289731692406348884105682373046875f,0.18049999427795437867416694643907248973846435546875f,-0.2573000239253051546484130085445940494537353515625f,-0.35620000873803991936483726021833717823028564453125f,1.0529000114560116418971347229671664535999298095703125f,1.256499963834882382940349998534657061100006103515625f,-1.2992000255912541373248814124963246285915374755859375f,-0.58279999652206893046013647108338773250579833984375f,-0.95669999675750627687875748961232602596282958984375f,-0.99150003096461336582478907075710594654083251953125f,2.10920006753802358190341692534275352954864501953125f,1.2057999886393542254836575011722743511199951171875f,0.7944000160455697567840616102330386638641357421875f,0.084800018188357473292171562206931412220001220703125f,-1.2711999600112438013610471898573450744152069091796875f,0.2560999525278810917683358638896606862545013427734375f,-0.0152000161588190518813235030393116176128387451171875f,0.612099973791837559389250600361265242099761962890625f,0.9898999919950963732873105982434935867786407470703125f,-0.75540000415444363834893692910554818809032440185546875f,-0.541400012007355879717351854196749627590179443359375f,0.449300019365548752858785519492812454700469970703125f,-1.3843999870628120874016531161032617092132568359375f,1.29500001028180111717347244848497211933135986328125f,-0.0702999908596273481720118070370517671108245849609375f,-0.59379998977184289543629347463138401508331298828125f,1.1616000090181819448531541638658381998538970947265625f,0.85069996200800002128517007804475724697113037109375f,-0.2009999761879448509915846443618647754192352294921875f,-0.678699980351328946426292532123625278472900390625f,-0.4654000007271765770866522871074266731739044189453125f,0.494800006490945687431803889921866357326507568359375f,0.81599999310076309910755298915319144725799560546875f,0.65160000037550869222968685789965093135833740234375f,0.1549999786168338999914340092800557613372802734375f,-0.6621000189423558168755334918387234210968017578125f,0.529800021317601288473042586701922118663787841796875f,-0.95360001399517013709328239201568067073822021484375f,-0.086899995183944866994352196343243122100830078125f,-0.73960001804828578997330623678863048553466796875f,-0.0279000271022329382475390957552008330821990966796875f,-0.43180004462301668777257646070211194455623626708984375f,-1.202100048446655744527333808946423232555389404296875f,-0.451599977651238049247695016674697399139404296875f,-0.921700004208087619872458162717521190643310546875f,-1.4340000057816499889895567321218550205230712890625f,1.2924999950826165928674527094699442386627197265625f,0.2483999738454816252897217054851353168487548828125f,-0.65190003824830089484976269886828958988189697265625f,-0.4533000209003688052433744815061800181865692138671875f,0.0048000166982411496263694061781279742717742919921875f,0.41339995648562943841852757032029330730438232421875f,0.5341999790996310171209415784687735140323638916015625f,0.096899990080296927175140808685682713985443115234375f,0.94620001566708122897608745915931649506092071533203125f,-1.2058000099852679287693035803386010229587554931640625f,-0.169200015905499601132078169030137360095977783203125f,-0.206500024929642478355162893421947956085205078125f,-0.3954000069111582860159614938311278820037841796875f,0.509600019910930956967831662041135132312774658203125f,0.141500053316355067778431475744582712650299072265625f,0.97980000008344614315092258038930594921112060546875f,-1.5443999842315918602508872936596162617206573486328125f,0.584899974262715005579593707807362079620361328125f,-0.480399996331334566690429710433818399906158447265625f,-0.83649996256828307306108172269887290894985198974609375f,-0.01700001615285895439910746063105762004852294921875f,0.3697000075012446540512200954253785312175750732421875f,-0.697399978807568654559645437984727323055267333984375f,1.06089997920989986823769868351519107818603515625f,-0.2488000162214039079344729543663561344146728515625f,-1.2295999500989918118420973769389092922210693359375f,0.762199999827146701392166505684144794940948486328125f,0.8710000211894508570509287892491556704044342041015625f,-1.41920005740523347270709564327262341976165771484375f,0.3233999996244900643205255619250237941741943359375f,-0.6940000142753124379169094027020037174224853515625f,-0.851500003397465121679488220252096652984619140625f,-0.439200027602911546864561387337744235992431640625f,0.550899977123737727424668264575302600860595703125f,-0.95460001670718153121697469032369554042816162109375f,-0.6813000075042252579038404292077757418155670166015625f,0.9794000068068502429241561912931501865386962890625f,-0.00900003632903167982703962479718029499053955078125f,0.805700038897991799302644722047261893749237060546875f,0.262000030830502605994070108863525092601776123046875f,0.767800012689829003420527442358434200286865234375f,-1.12229999896586019048072557779960334300994873046875f,-0.940699992629885795025757033727131783962249755859375f,-0.091300011825560911660204510553739964962005615234375f,0.93509999556243439400304851005785167217254638671875f,-0.0968000145971779080156238705967552959918975830078125f,-0.31790000653862993118536905967630445957183837890625f,-0.3770999944299460171492910376400686800479888916015625f,-1.1001000038325781105186251807026565074920654296875f,0.72390000686645483707337689338601194322109222412109375f,0.1531999952375888707223339224583469331264495849609375f,-0.7084000078201293515434144865139387547969818115234375f,1.1689999452829364923189814362558536231517791748046875f,0.238900034731626131900839027366600930690765380859375f,0.624400016120076006842509741545654833316802978515625f,0.92419995235204766004244447685778141021728515625f,-0.05580000152885933317747912951745092868804931640625f,-0.4307999594330791381935341632924973964691162109375f,0.870999979764223386524690795340575277805328369140625f,0.5047999914407730326360024264431558549404144287109375f,-0.211599988305568309510107383175636641681194305419921875f,0.115400007134675686160107943578623235225677490234375f,-1.4335999828517433929420121785369701683521270751953125f,0.22269999399334172718312174765742383897304534912109375f,0.0474000076040621554085419120383448898792266845703125f,1.24909998407736377856735998648218810558319091796875f,0.798400030469894606000025305547751486301422119140625f,0.461599990615248589875818652217276394367218017578125f,-0.888600033813715217689832570613361895084381103515625f,-1.7703999865710724481715487854671664535999298095703125f,-0.3563000227034098799094863352365791797637939453125f,-0.14660006504654887748984037898480892181396484375f,1.7734000672757630479026147440890781581401824951171875f,1.2976999932855359798367089751991443336009979248046875f,-1.3304000495284802330075990539626218378543853759765625f,-0.504600005680322372114687823341228067874908447265625f,-0.931700028795003465376112217199988663196563720703125f,0.1723000081300742891698973835445940494537353515625f,-0.4663999564260248309466305727255530655384063720703125f,1.32999994166195367739646826521493494510650634765625f,0.400399998790025524186830807593651115894317626953125f,-0.011899991980194979390716980560682713985443115234375f,0.55149997001886408298787500825710594654083251953125f,0.518299948108197128249230445362627506256103515625f,0.5277000169038768007112594204954802989959716796875f,-0.8447999825000760409920985694043338298797607421875f,-0.26500001639127734875955866300500929355621337890625f,-0.21459995781034291173483552483958192169666290283203125f,0.0304000256121153000776757835410535335540771484375f,-0.59189997059702881276876951233134604990482330322265625f,-1.4866000140473254464978936084662564098834991455078125f,0.3038999730408196153774724734830670058727264404296875f,1.07860000240951858785365402582101523876190185546875f,0.44439999831318832690385534078814089298248291015625f,-0.345999984271823990411576232872903347015380859375f,0.59059998347014197594972984006744809448719024658203125f,1.21800001122802559194724381086416542530059814453125f,-0.5206999535888439112341075087897479534149169921875f,-0.9494999736845499693771444071899168193340301513671875f,-0.66590000532418447409810369208571501076221466064453125f,-1.50709993579387724338403131696395576000213623046875f,0.327699974472821065063499190728180110454559326171875f,0.956199951778352374986980066751129925251007080078125f,0.05029996560215976852958874587784521281719207763671875f,-0.1523999756038183495121529631433077156543731689453125f,1.0494000322505829814190292381681501865386962890625f,0.6964999412596226679994515507132746279239654541015625f,-0.30570001662075496629000781467766501009464263916015625f,-0.952799959604442303628957233740948140621185302734375f,0.076400010032951826133285067044198513031005859375f,0.7600999934017647063910771976225078105926513671875f,0.11790000668764122337961453013122081756591796875f,0.11769997667074250102814403362572193145751953125f,1.24409995403289830306903240852989256381988525390625f,1.436599973069131674918708085897378623485565185546875f,0.5276999480605135683930484447046183049678802490234375f,-1.376699973474443172705150573165155947208404541015625f,0.7741999775350090207126640962087549269199371337890625f,0.8074000031709667180024325716658495366573333740234375f,0.2807999807044867512928476571687497198581695556640625f,-1.0211999842628838308655758737586438655853271484375f,-0.2247999605953701074412265370483510196208953857421875f,1.77569996856451073341531810001470148563385009765625f,-0.3553000114232300887806559330783784389495849609375f,-0.0975000122562053039843021906563080847263336181640625f,0.037899996368587041128961345748393796384334564208984375f,0.909700002732872547994702472351491451263427734375f,-0.069499994695186140258869045283063314855098724365234375f,-0.60070003524720672061221193871460855007171630859375f,-0.6042000084921717029118326536263339221477508544921875f,-0.4905999857425686894174532426404766738414764404296875f,0.54859996202737082438005700169014744460582733154296875f,1.68580002797842087858271042932756245136260986328125f,0.756500005371868677883639975334517657756805419921875f,-0.560900040602684502033525859587825834751129150390625f,1.3722999995991582711241107972455210983753204345703125f,0.2735999907493591454255010830820538103580474853515625f,0.224300015453993972780466492622508667409420013427734375f,-1.28669998155832221442551599466241896152496337890625f,-0.53970002281218809603302588584483601152896881103515625f,-0.5609999918341639357777239638380706310272216796875f,0.16760000848919121008151478235959075391292572021484375f,0.8390999768316744766849524239660240709781646728515625f,1.2206999931886794996671596891246736049652099609375f,-0.4846000138759603004245946067385375499725341796875f,-0.139300002527237065663712201057933270931243896484375f,-0.14980004329681406716190394945442676544189453125f,0.46949998705834072865172856836579740047454833984375f,0.4502999905616047460199524721247144043445587158203125f,-0.0375999909058217784973976449691690504550933837890625f,-1.07240001240968663154262685566209256649017333984375f,0.687000013843178436445668921805918216705322265625f,0.46690002185702306203296529929502867162227630615234375f,0.60119998303353749946609241305850446224212646484375f,0.9409000072240825129910035684588365256786346435546875f,0.3997999429374938795689331527682952582836151123046875f,-1.16150000146031384673506181570701301097869873046875f,0.791099998199939591358997859060764312744140625f,0.65640001223087285353585684788413345813751220703125f,1.2222000101089476320481708171428181231021881103515625f,0.79689999279975864965308574028313159942626953125f,1.523000019758939505010175707866437733173370361328125f,0.94809998216628965650443205959163606166839599609375f,1.23970002985298588527030005934648215770721435546875f,0.0179999975562090686054261823301203548908233642578125f,-0.20969998101443010352085138947586528956890106201171875f,-1.19000002101063717674378494848497211933135986328125f,-0.6772000208005308774517061465303413569927215576171875f,0.217500020712614539064588825567625463008880615234375f,-0.31070001434832816844533454059273935854434967041015625f,-0.46189999667406045347206600126810371875762939453125f,0.7522000152125956073945189928053878247737884521484375f,0.5583999950051303784448464284650981426239013671875f,-1.0459000055849549948305821089888922870159149169921875f,0.45960005470514175840435200370848178863525390625f,1.5293999828904869531243093661032617092132568359375f,-0.0095000228583810297777745290659368038177490234375f,-0.5850000045448540131332038072287105023860931396484375f,0.54349998199939708598549259477294981479644775390625f,-0.63939999037831984907143123564310371875762939453125f,-0.093499981999396908349808654747903347015380859375f,0.540200002859533068289010770968161523342132568359375f,0.51659997370243093683939150650985538959503173828125f,0.7244999786391852492073439861997030675411224365234375f,-0.90919999660849537548301668721251189708709716796875f,1.018499986171722326844246708787977695465087890625f,-1.659999967068433335271038231439888477325439453125f,1.93439994362592670995582011528313159942626953125f,0.290199970859289724245400066138245165348052978515625f,0.10799999964237283478496465249918401241302490234375f,1.014000006228684913622828389634378254413604736328125f,-1.4851999908268442851522195269353687763214111328125f,-0.313199996578693795612480244017206132411956787109375f,-0.79760000755786819581771851517260074615478515625f,0.65889998734593380191881806240417063236236572265625f,-0.384100011644512406849827357291360385715961456298828125f,1.42060000422000864972460476565174758434295654296875f,-0.0992999989882112910066780386841855943202972412109375f,-1.19320002347528930641828992520458996295928955078125f,-1.0734999898225068282187066870392300188541412353515625f,-0.518399993515014312350785985472612082958221435546875f,-0.5085000064373019768027006648480892181396484375f,-0.82939997454583569425068390046362765133380889892578125f,-0.63889997624456906777368203620426356792449951171875f,-0.00619997357130142034975506248883903026580810546875f,0.121399975052476405412704707487137056887149810791015625f,0.576499979555606945069712310214526951313018798828125f,1.416599995774775688772706416784785687923431396484375f,-1.9357999463200581402588795754127204418182373046875f,-0.89060004910975631986502776271663606166839599609375f,-0.37180001944005436342877146671526134014129638671875f,-1.0803000139191742956512598539120517671108245849609375f,-0.5368999400496488050293919513933360576629638671875f,0.604500014998019086220892859273590147495269775390625f,-0.041300026950239487888438816298730671405792236328125f,-0.35250000953674287540451359745929948985576629638671875f,0.377999993681907398013208876363933086395263671875f,1.4581999493792654476465031621046364307403564453125f,0.724199960696697786488584824837744235992431640625f,-0.19650000024959457523010541990515775978565216064453125f,-0.45789996019601897359052600222639739513397216796875f,0.8275999527215969209237300674431025981903076171875f,1.21949997033178814120901733986102044582366943359375f,-0.417400008535384170471616016584448516368865966796875f,-0.143699981319904690479916098411194980144500732421875f,0.095899959763884989882853915332816541194915771484375f,0.581999980166554298222081342828460037708282470703125f,-0.2591999933302400194179426762275397777557373046875f,0.5236999816179277456029694803874008357524871826171875f,-0.164300017651914931349210746702738106250762939453125f,0.267400001680850873952977053704671561717987060546875f,0.04429996907412982187768335506916628219187259674072265625f,0.044600008958578296613950442406348884105682373046875f,-0.517899959748983729213023252668790519237518310546875f,-0.9010000183582320687492028810083866119384765625f,-0.04249999899417133963908099758555181324481964111328125f,0.950299980354308626573356377775780856609344482421875f,0.4375000059977172117697818976012058556079864501953125f,0.41830000622272489607667012023739516735076904296875f,0.0810000091567635305267458534217439591884613037109375f,-0.581900003454089098653412293060682713985443115234375f,-0.31240000283569113737058842161786742508411407470703125f,0.73610001880079511948906656471081078052520751953125f,0.4159000104650856233234890169114805757999420166015625f,0.089499990820885333420164897688664495944976806640625f,0.83939997257143261766287878344883210957050323486328125f,0.61640000060796662495476994081400334835052490234375f,-1.1599999648705130450565548017038963735103607177734375f,-0.1967999289155013542540473281405866146087646484375f,0.697300024782121052879801936796866357326507568359375f,-0.63049998444318777757189309340901672840118408203125f,0.04029998352080610413139538650284521281719207763671875f,0.425899983978271112761149197467602789402008056640625f,0.1304000015839938964035127355600707232952117919921875f,-1.510999960690736276802681459230370819568634033203125f,0.74549994111806172636391920605092309415340423583984375f,0.114199972766638069288092083297669887542724609375f,-0.284499970704317650671555384178645908832550048828125f,1.26789994916916004541462825727649033069610595703125f,-1.20699993479251954653364009573124349117279052734375f,-1.5852999748408791447218391112983226776123046875f,0.4468999895215031159523277892731130123138427734375f,-0.230199998873472200244805208058096468448638916015625f,0.9838999608218674808313153334893286228179931640625f,1.1362999971479175176369835753575898706912994384765625f,-0.45890001468956409524935224908404052257537841796875f,-0.9095000163018702732387055220897309482097625732421875f,-0.72059996402412684890492755585000850260257720947265625f,-1.388800026726723757519721402786672115325927734375f,-0.8009999866187571537778921992867253720760345458984375f,-1.1464000056743619548882406888878904283046722412109375f,-0.43389999587684879855942199355922639369964599609375f,-0.765799985361098833891446702182292938232421875f,0.38929998367726825758694531032233498990535736083984375f,0.50899999743700075072183608426712453365325927734375f,-0.666499995574355352800921536982059478759765625f,0.362100045950711102449304235051386058330535888671875f,0.043799999755620933061805999386706389486789703369140625f,-0.2484000126816333919865797952297725714743137359619140625f,-1.58200000859051925772291724570095539093017578125f,-0.2073999507188801771917496807873249053955078125f,0.0211999967798590649437073807348497211933135986328125f,0.28870001127123856576872640289366245269775390625f,0.50310002616196858316044426828739233314990997314453125f,0.6053999679446224746470761601813137531280517578125f,0.3013999804541469540453135778079740703105926513671875f,-0.1767999891161924352900314261205494403839111328125f,1.158399989640713112493131120572797954082489013671875f,-1.23359999119639329734354760148562490940093994140625f,-1.317300021280348065744192354031838476657867431640625f,0.24789999463632683696090452940552495419979095458984375f,-0.3832999678894879469481793421437032520771026611328125f,0.24209997602701260888125034398399293422698974609375f,-0.1051000060677527159214150742627680301666259765625f,0.1795999924182893892066203989088535308837890625f,0.06269998978376367659848256153054535388946533203125f,-0.191199989217520016637763546896167099475860595703125f,0.55459997081160583576320277643389999866485595703125f,-0.1074999581277378268850952736102044582366943359375f,-0.3971000150680534801495014107786118984222412109375f,0.2256999654352664919088056194595992565155029296875f,0.278799990330636848767653646063990890979766845703125f,0.06420000743046416769033157834201119840145111083984375f,-0.658900007164478385135453208931721746921539306640625f,-1.94500004574656504274798862752504646778106689453125f,0.2236999875783920455063480403623543679714202880859375f,-1.3154000548183919505618177936412394046783447265625f,0.7424000041767950808235809745383448898792266845703125f,0.31650000131130173741667022113688290119171142578125f,-0.68379999681264092004084886866621673107147216796875f,-0.495800032523274314399941431474871933460235595703125f,-0.3852000185802582077343458877294324338436126708984375f,-0.310599972853064176803172813379205763339996337890625f,-0.3851000026032327383518349961377680301666259765625f,1.361300012719631080670978917623870074748992919921875f,-0.3411999594151975401246090768836438655853271484375f,0.1394000047206886971906669714371673762798309326171875f,0.2467999754071232842989047640003263950347900390625f,-1.214799990844726895744543071486987173557281494140625f,-0.6053000003218647062652735257870517671108245849609375f,0.1856000097334382648028849871479906141757965087890625f,0.208700007396936282333399503841064870357513427734375f,-0.301200011718272708804988724295981228351593017578125f,-0.888599968099594406112373690120875835418701171875f,0.96240001449584955395266661071218550205230712890625f,1.2145999664902689119116985239088535308837890625f,-1.06679995432198015947733438224531710147857666015625f,0.8248999981045717078131929156370460987091064453125f,1.196999987840651868964414461515843868255615234375f,0.35930003381967434705757113988511264324188232421875f,-1.58410001535117572757371817715466022491455078125f,-1.1786999930545698500594653523876331746578216552734375f,0.447000022299588462004749089828692376613616943359375f,0.24780003872960831845517759575159288942813873291015625f,-1.686000051587820625087488224380649626255035400390625f,-1.11399998991191306885184530983678996562957763671875f,0.84669998744428109915816094144247472286224365234375f,0.4110000020787125674104345307569019496440887451171875f,-1.1434000677600517636989252423518337309360504150390625f,0.631700014191865388823998728184960782527923583984375f,-0.06480000626742832281479422817938029766082763671875f,-0.644499972268939291808464986388571560382843017578125f,1.208199990096688214435971531202085316181182861328125f,-0.81429998390078572878536533607984893023967742919921875f,0.1086000001370905909681141565670259296894073486328125f,-0.043499976336955514799598176978179253637790679931640625f,0.7833000051796437102069603497511707246303558349609375f,-0.99970003577619792611841376128722913563251495361328125f,-0.8630999848484994796393721117055974900722503662109375f,0.3716999939262866092803960782475769519805908203125f,1.002900028070807803004527158918790519237518310546875f,-1.185299983334541451540644629858434200286865234375f,-0.3762000101536511209587843040935695171356201171875f,-0.76689998117685309608759780530817806720733642578125f,-1.063800006535648545735739389783702790737152099609375f,1.2867999998077745971158947213552892208099365234375f,0.004599997186660598202934124856255948543548583984375f,-1.003100010702013378249830566346645355224609375f,-0.26379998910129109912503508894587866961956024169921875f,1.1013000001281498807514935833751223981380462646484375f,1.0959999949634069427162330612190999090671539306640625f,-0.95459998954981539753816832671873271465301513671875f,-0.180800010320544490838301499024964869022369384765625f,0.879099990168213718533252176712267100811004638671875f,-0.7545000143647191404028262695646844804286956787109375f,-0.2924999944120647565881654372788034379482269287109375f,1.39780005486011571491644644993357360363006591796875f,0.27590000513792034553262055851519107818603515625f,0.21709999852776540318899378689820878207683563232421875f,-0.333499971419572549535814687260426580905914306640625f,-0.975400016671418956804018307593651115894317626953125f,0.253400035312771887419103222782723605632781982421875f,-0.4085000003278249636196051142178475856781005859375f,-0.5538999621629709935177743318490684032440185546875f,-0.65240001568794259156902626273222267627716064453125f,0.369100018145144037617910726112313568592071533203125f,0.45660001758635038182632115422165952622890472412109375f,-0.387199991296231671444871835774392820894718170166015625f,-0.9726000474184754995832236090791411697864532470703125f,-0.01259998192787159521088824476464651525020599365234375f,0.1228000260263686360673318631597794592380523681640625f,-0.2412999844074246169611797085963189601898193359375f,1.51290000698566462489225159515626728534698486328125f,-0.8391999957144260502417409952613525092601776123046875f,-0.17880002911090908668256815872155129909515380859375f,-0.9261999980092046058643973083235323429107666015625f,-0.675900001785158632827688052202574908733367919921875f,-0.67389998660087579995803253041231073439121246337890625f,0.2011999728262426068159385295075480826199054718017578125f,0.249199998322129356864707006025128066539764404296875f,-0.06400001673400357304899443988688290119171142578125f,-0.674300005507469268906106663052923977375030517578125f,-0.3568000110954046011357831957866437733173370361328125f,-0.028100003549456520346438992419280111789703369140625f,-0.4168000055819749416485819892841391265392303466796875f,0.089400012245774096442119116545654833316802978515625f,0.48799998831748947480946299037896096706390380859375f,-0.25510001024007777914448524825274944305419921875f,0.427400024703145309246110628009773790836334228515625f,-0.644700009997189571464559776359237730503082275390625f,-1.4763000703871256291677127592265605926513671875f,-1.84400002902746251720600412227213382720947265625f,-0.5040000789463530139755675918422639369964599609375f,-0.8271000353336344090848797350190579891204833984375f,-1.373400045669078917143224316532723605632781982421875f,-1.58739999855160807129550448735244572162628173828125f,0.04590002391338376952489852556027472019195556640625f,0.35599998751282679876339898328296840190887451171875f,1.237300061178208121503985239542089402675628662109375f,1.55790001473426809042166496510617434978485107421875f,-0.6679000002801416258080280385911464691162109375f,-1.4924000133410098012376465703709982335567474365234375f,0.051699990499019410350456382730044424533843994140625f,-1.041000032663345731265280846855603158473968505859375f,0.907300006192922836589787038974463939666748046875f,-0.742099992567301303125759659451432526111602783203125f,-0.005699996727705158860999290482141077518463134765625f,-0.57119998005330552270919497459544800221920013427734375f,-0.3211999883979554937241118750534951686859130859375f,1.0417000005573033849515240945038385689258575439453125f,-0.3261000212222338046075265083345584571361541748046875f,0.9410000220090151135110545510542578995227813720703125f,0.0038000099629160732916943743475712835788726806640625f,0.1784000327795747242731749793165363371372222900390625f,-0.672400015687942609332594656734727323055267333984375f,1.1420000116825104186091266456060111522674560546875f,-0.947400035208463719982319162227213382720947265625f,-0.5941999419212340427520757657475769519805908203125f,0.424600005978346128898692768416367471218109130859375f,0.8459999884068967190842158743180334568023681640625f,0.20060004035532497113081262796185910701751708984375f,0.591999983370303795027211890555918216705322265625f,0.4452000299423934936982050203368999063968658447265625f,-0.95840000692605986643002324854023754596710205078125f,0.0512000018090008612858810010948218405246734619140625f,1.5613000305078938989566950112930499017238616943359375f,0.98549995791911992881750848027877509593963623046875f,0.2702000309109688913622449035756289958953857421875f,-0.3979999738633619443817224237136542797088623046875f,-0.973799960938097175500161029049195349216461181640625f,1.9756000440806149587302797954180277884006500244140625f,1.50870002088248700289341286406852304935455322265625f,-0.09609998940825459357029103557579219341278076171875f,-0.451599978992342609984689261182211339473724365234375f,0.060100003907084253995662948000244796276092529296875f,1.30369997617900335029617053805850446224212646484375f,-1.779200010168551226996669356594793498516082763671875f,0.1223999876178805401583105094687198288738727569580078125f,-0.8539000245243302256170636610477231442928314208984375f,-1.04539999804496819280075214919634163379669189453125f,-0.39480000030696427160137318423949182033538818359375f,0.100200036871433029173061868277727626264095306396484375f,-0.25979999583661506079579339711926877498626708984375f,0.85810001614093789346071616819244809448719024658203125f,-0.1621999949097638893391604142379947006702423095703125f,0.311599986591935163016131582480738870799541473388671875f,0.99019997108280666253676827182061970233917236328125f,-0.19589998420178933447033386983093805611133575439453125f,0.427600000330806029325003692065365612506866455078125f,-1.2372999963954089963635851745493710041046142578125f,0.5590000045150518293013419679482467472553253173828125f,-0.56109995588064265348293702118098735809326171875f,0.07340001206696111779592683888040482997894287109375f,0.96499996848404379079511272720992565155029296875f,-0.6157999677032233165618890780024230480194091796875f,-1.027300001573562493462077327421866357326507568359375f,-0.0993999679148200865341777898720465600490570068359375f,0.42569997474849241569927471573464572429656982421875f,0.22999998040497349194311027531512081623077392578125f,-0.748199955898523949571199409547261893749237060546875f,0.351599990615249158310007260297425091266632080078125f,-0.0574999867007133236285199018311686813831329345703125f,-0.347899986273049410812063797493465244770050048828125f,-0.00889999211430581027570951846428215503692626953125f,-0.5951000221610076579281667363829910755157470703125f,0.0431999319076528109917489928193390369415283203125f,1.23820000864863377643132480443455278873443603515625f,0.81499997138976976174262745189480483531951904296875f,-0.099200020897388441198927466757595539093017578125f,-0.554799967300892316046656560502015054225921630859375f,-1.13510003959536565076859915279783308506011962890625f,-0.4039999838769439133301375477458350360393524169921875f,2.0051000635862354926075568073429167270660400390625f,-0.038000004447996449385982487001456320285797119140625f,0.2943000000685442874015507186413742601871490478515625f,0.150600018599628793225519984844140708446502685546875f,-0.094899974337220083242527834954671561717987060546875f,0.213399979656934579708149612997658550739288330078125f,-0.34420000279247719987552045495249330997467041015625f,0.5398000213921070322697914889431558549404144287109375f,0.0856999915868044137567949292133562266826629638671875f,-0.7874999827891588921602306072600185871124267578125f,0.59340002696812177429563917030463926494121551513671875f,-0.3515999939680101160632830215035937726497650146484375f,-1.59770004149079358768403835711069405078887939453125f,0.1516000084593891461537396025960333645343780517578125f,0.6699999640136959921932202632888220250606536865234375f,-0.36669999005198494756996296928264200687408447265625f,0.096999966308474849796539274393580853939056396484375f,1.1042999953746797103804055950604379177093505859375f,-0.21580001136362536584556437446735799312591552734375f,-1.12949998009204843896213787957094609737396240234375f,-0.1482000033587216325514646086958236992359161376953125f,-0.098399993216991443745200740522705018520355224609375f,0.41560000489056114769681471443618647754192352294921875f,0.5668000037193297035997829880216158926486968994140625f,0.36660001795887975362120414501987397670745849609375f,-0.5157000252634293957498812233097851276397705078125f,1.54870005016326928171110921539366245269775390625f,0.65589996758699253831537134828977286815643310546875f,0.6432000363647940588407436734996736049652099609375f,0.20810000585913623893929980113171041011810302734375f,0.14950001883506747191177055356092751026153564453125f,-0.22020000960230756703595034196041524410247802734375f,-0.10460003756880809078211314044892787933349609375f,0.3783000130772595781536438153125345706939697265625f,1.0560000495016581556484425163944251835346221923828125f,0.5813000503450631573088003278826363384723663330078125f,-0.562200029294193104334453892079181969165802001953125f,0.426900001628696290634223942106473259627819061279296875f,-0.412000003412365867205835456843487918376922607421875f,1.193900031453370758072196622379124164581298828125f,0.1322000167399643988375146363978274166584014892578125f,0.80120001194179091275060500265681184828281402587890625f,0.339099981674551909094361690222285687923431396484375f,0.2958999766767027583824756220565177500247955322265625f,0.2161000235319143758516702291672118008136749267578125f,0.17820001729130741097151258145458996295928955078125f,-0.7515000447481876566513392390334047377109527587890625f,-0.5507000075310470865730394507409073412418365478515625f,0.5633000184416772082585112002561800181865692138671875f,0.4197000082463022696543930578627623617649078369140625f,-0.987599997201561219384302603430114686489105224609375f,-0.26080000175237660897664682124741375446319580078125f,-0.8455999811977148450381491784355603158473968505859375f,-0.130600024932622460482889437116682529449462890625f,-1.235599961677193991960166385979391634464263916015625f,0.1869999911189079710993610206060111522674560546875f,1.1516000198960301137418582584359683096408843994140625f,-0.838199971544742794549165409989655017852783203125f,-0.123999996915459487212274325429461896419525146484375f,0.0283999798431990857494611191214062273502349853515625f,-0.23220001185983429881076745004975236952304840087890625f,-0.81899994170665646464613018906675279140472412109375f,0.1193999968230716746120378957130014896392822265625f,-0.7202000532627099715909935184754431247711181640625f,-0.88590002212524421310035904753021895885467529296875f,-1.1331000569701199509609068627469241619110107421875f,0.29250002339482339408505140454508364200592041015625f,0.2746999752402299321829559630714356899261474609375f,1.452600049579144236844285842380486428737640380859375f,0.6387000214040277068505702118272893130779266357421875f,-0.12419995995163812807504655211232602596282958984375f,0.597100036674737655317812823341228067874908447265625f,0.83729995225071807141148383379913866519927978515625f,0.094499987542630226045048402738757431507110595703125f,-1.036599967294931534667057348997332155704498291015625f,-0.027899986198545168036844188463874161243438720703125f,0.533599977040291495455903714173473417758941650390625f,0.343899976915121197151847809436731040477752685546875f,-0.4222000206887714934822497525601647794246673583984375f,-0.4998999526560314876633128733374178409576416015625f,0.307999972373247832280185320996679365634918212890625f,0.053000019460915925861854702816344797611236572265625f,-1.376399957692623132743392488919198513031005859375f,0.6926999939560882779687744914554059505462646484375f,0.5541999931812284785337396897375583648681640625f,-0.13269999648928632041133823804557323455810546875f,-0.786499990358948242175074483384378254413604736328125f,-0.19270001336485165222001114671002142131328582763671875f,-0.225700018557906556537773212767206132411956787109375f,-0.45860000240951770411612642419640906155109405517578125f,-0.46660000484585761881817234097979962825775146484375f,0.0641999610692263189548611990176141262054443359375f,0.241199983406067186564314397401176393032073974609375f,-0.374199971909821460513967394945211708545684814453125f,-0.19979996283054379802024413947947323322296142578125f,0.05019998624473831494441355971503071486949920654296875f,-0.024800012004375648899667794466949999332427978515625f,0.26790002315342376082440978279919363558292388916015625f,0.966899977600574001002087243250571191310882568359375f,2.00719998742192995422328749555163085460662841796875f,0.283899978852271583917854513856582343578338623046875f,-1.613600011201202111266184147098101675510406494140625f,1.07210000582933417945241671986877918243408203125f,0.12530003824532087719489936716854572296142578125f,0.0452000067710880415461360826157033443450927734375f,-1.11529999562799897461218279204331338405609130859375f,0.5416000106573104488916214904747903347015380859375f,0.98060003744959800542346783913671970367431640625f,1.12419993417263075485834633582271635532379150390625f,0.8692999872162932017971570530789904296398162841796875f,0.6906000305205577394218607878428883850574493408203125f,-0.018400029203295531488038250245153903961181640625f,0.930099991017579785790303503745235502719879150390625f,-1.841099966982005842197622769162990152835845947265625f,0.41049999710917450812530660186894237995147705078125f,-1.6189000089526164316566791967488825321197509765625f,-0.527400007343292287487201974727213382720947265625f,-1.4181999438285828318839776329696178436279296875f,0.445299968284369018789448091411031782627105712890625f,-0.6671999835848811954974735272116959095001220703125f,0.18529999942779529220615586382336914539337158203125f,-0.447099976027011736245952988610952161252498626708984375f,-0.490099971720576288447546176030300557613372802734375f,-0.411800019402801920076484520905069075524806976318359375f,0.49799998466670558627811260521411895751953125f,-0.6573999858856203193369083237485028803348541259765625f,-1.3794999721944325443701018230058252811431884765625f,-1.077399987971782824303090819739736616611480712890625f,0.096700037437677632823351814295165240764617919921875f,-0.583999978736043257043775156489573419094085693359375f,0.2635000267028804188385038287378847599029541015625f,-0.3075999820023771480492769114789552986621856689453125f,-1.21250002324581185320084841805510222911834716796875f,-0.133499996975064316728776248055510222911834716796875f,0.194999980032444231170529747032560408115386962890625f,0.1377000114277004971796714016818441450595855712890625f,-1.271699956524372510102693922817707061767578125f,-2.0593999660521742356422691955231130123138427734375f,-0.2817000056982035260944030596874654293060302734375f,0.463799960975349367942044409574009478092193603515625f,-1.0202999562144281497921838308684527873992919921875f,0.827399993745982609283373676589690148830413818359375f,-1.44059998991489379704944440163671970367431640625f,-0.428200042660534574423536469112150371074676513671875f,0.221599963569641289495848468504846096038818359375f,-0.22389997590929267001769176204106770455837249755859375f,1.566799962815642377478297930792905390262603759765625f,-1.10199998195469373030164206284098327159881591796875f,-0.838799970698357100218345294706523418426513671875f,-0.70009997633993581445110976346768438816070556640625f,0.49599999213218648463907811674289405345916748046875f,-0.2387000132083887393719123792834579944610595703125f,-0.06220000683069226710131260915659368038177490234375f,0.479899993634223420713169616647064685821533203125f,-0.04440002796649888949787055025808513164520263671875f,1.160599992299079730173616553656756877899169921875f,1.779099969530105962434163302532397210597991943359375f,0.00440000870674861488396345521323382854461669921875f,-0.65560001398026912511340924538671970367431640625f,-1.105799963344634040396385898930020630359649658203125f,-2.0616999757841227847166010178625583648681640625f,-0.5014999990016220099420252154232002794742584228515625f,0.2682000097289682916112951716058887541294097900390625f,-0.07809998328387746102663413694244809448719024658203125f,-0.62900000697374347513601833270513452589511871337890625f,-0.90979996517747674378284727936261333525180816650390625f,-0.8129999893605714955668872789829038083553314208984375f,-0.413199996578693162785356207677978090941905975341796875f,-0.238700014549493744198116473853588104248046875f,-0.7626000034973026142637309021665714681148529052734375f,-1.13269995491504715801056590862572193145751953125f,-0.0379000228181480824929394657374359667301177978515625f,0.56989997500777267447347185225225985050201416015625f,0.1513000177115200184374543823651038110256195068359375f,-0.5280999732255937662017686307081021368503570556640625f,0.825699952732026787316499394364655017852783203125f,-0.50309998447597070736492241849191486835479736328125f,-0.1388999925613403352286923109204508364200592041015625f,-0.3607999813377860132135310777812264859676361083984375f,-0.498800013352930449173072702251374721527099609375f,-0.43569996707439440086773174698464572429656982421875f,0.19540001473426826805734890513122081756591796875f,-0.24649997776746779276635379574145190417766571044921875f,0.31920000636875567323613722692243754863739013671875f,1.25529996839612767445260033127851784229278564453125f,0.0002999964848158231944807994295842945575714111328125f,-1.2452999690294268564372259788797236979007720947265625f,0.8533999802902341169641431406489573419094085693359375f,0.79699998791515846985333837437792681157588958740234375f,-0.076299981054664289814581934479065239429473876953125f,0.34589997801780736352839085157029330730438232421875f,-0.0955000021755691985703151658526621758937835693359375f,-0.534599982285499653045235390891321003437042236328125f,0.8482000084251166693860568557283841073513031005859375f,0.498599975401162520682873946498148143291473388671875f,-0.66259999649375644903415150110959075391292572021484375f,0.00710001732185527867358132425579242408275604248046875f,-0.27400000924617062825205948684015311300754547119140625f,-0.782100002383440795483693364076316356658935546875f,0.420299953588098684775786750833503901958465576171875f,1.128599987806380600119382506818510591983795166015625f,0.082400006859004004677871080275508575141429901123046875f,-0.16079997813403601458759339948301203548908233642578125f,0.29469999360591192516523051381227560341358184814453125f,-0.63540000825226228275965922875911928713321685791015625f,0.0798999829798934246838371109333820641040802001953125f,-0.32949997860193303012010801467113196849822998046875f,1.422199984255433147239955360419116914272308349609375f,-1.295799989980458644112104593659751117229461669921875f,-1.3298999992594122687705748830921947956085205078125f,1.50379994514286519091683658189140260219573974609375f,1.383299999424070225728655714192427694797515869140625f,-0.332699990528821931690117708058096468448638916015625f,-0.9766999846503134907749199555837549269199371337890625f,-0.04070000425279107769682696016388945281505584716796875f,-0.365500010594725477375277478131465613842010498046875f,0.377199970844388499102706191479228436946868896484375f,1.32839998081177412814213312231004238128662109375f,-0.38149997612833974613977261469699442386627197265625f,0.21169998096227704564853411284275352954864501953125f,0.5842999759100375722908893294516019523143768310546875f,0.224299989507347419248617370612919330596923828125f,1.054700008171796543621212549624033272266387939453125f,-0.0455000419802955580195202855975367128849029541015625f,-0.6862999699532992270434306192328222095966339111328125f,-0.71639998201876853300973380100913345813751220703125f,-0.22600001120567314583098550428985618054866790771484375f,0.29779998411685271175741718252538703382015228271484375f,-0.149199985507130872264269783045165240764617919921875f,0.54529997890144599193007479698280803859233856201171875f,1.143199997919797983314538214472122490406036376953125f,1.2619999802783132825112488717422820627689361572265625f,-0.088500020593404560287353888270445168018341064453125f,0.99809997829198859875532434671185910701751708984375f,1.13699997428059607074146697414107620716094970703125f,0.7462999899953601801172453633625991642475128173828125f,-0.4549999836832281374654485261999070644378662109375f,0.53199999141693121718077463810914196074008941650390625f,1.0120999778151509307377864388399757444858551025390625f,-0.3291999659121038757092492232914082705974578857421875f,-1.11009997060298903903685641125775873661041259765625f,0.6167000103920698972359559775213710963726043701171875f,0.19029999287128429585891353781335055828094482421875f,-0.2409000175431368351297578556113876402378082275390625f,1.4524999347329146104357278090901672840118408203125f,0.171900002336502044641974862315692007541656494140625f,-0.28859996154308387161790960817597806453704833984375f,0.0852999929457907146712614121497608721256256103515625f,1.13119993244409666743877096450887620449066162109375f,0.8969999717473984279791920926072634756565093994140625f,-0.9066999587595467868794685273314826190471649169921875f,-0.9827999752879146289075151798897422850131988525390625f,0.44719995229244347001440473832190036773681640625f,-0.15190000904202438647416784078814089298248291015625f,-0.9090999689340595324438254465349018573760986328125f,0.80319996647834823733091980102472007274627685546875f,-0.04729999114274985316797028644941747188568115234375f,0.3372999816060069377954278024844825267791748046875f,-1.766999931365252241022290036198683083057403564453125f,0.9650999486982829722592214238829910755157470703125f,0.75719997434616093112680346166598610579967498779296875f,0.405399976363778336008181213401257991790771484375f,-0.8940999906897542093275887964409776031970977783203125f,-0.3511999436944732710941252662451006472110748291015625f,0.213599974805116943343819002620875835418701171875f,-0.9467999696701772105456029748893342912197113037109375f,-0.60799994525313394433396751992404460906982421875f,0.223299976196885519019730281797819770872592926025390625f,-0.43690003169178925901405818876810371875762939453125f,0.9825999830454585914907283950014971196651458740234375f,0.26119997729659072405183906084857881069183349609375f,1.266100008518993735862068206188268959522247314453125f,-0.45130003417730346626512982766143977642059326171875f,-0.24789999301582510415897786515415646135807037353515625f,-0.4967999974608421354105303180404007434844970703125f,-0.976500021204352197656817224924452602863311767578125f,-0.3618000199615953871301599065191112458705902099609375f,-0.178699946488440009062514945981092751026153564453125f,0.513899985110759960349469110951758921146392822265625f,-0.7283999909818168472241950439638458192348480224609375f,1.851699958908557874792677466757595539093017578125f,0.0720999981552365998282994041801430284976959228515625f,1.158700005161762192074093036353588104248046875f,0.3387999970361590396095152755151502788066864013671875f,0.26859997525215195679493263014592230319976806640625f,-0.606899999877810625292795521090738475322723388671875f,-0.1358000106930727923781887511722743511199951171875f,1.7015999490410089745040522757335565984249114990234375f,-0.383000000089406711367701063863933086395263671875f,-0.8706999716937549038675570045597851276397705078125f,0.815499953597784266179360201931558549404144287109375f,-1.045400001174210391496899319463409483432769775390625f,-0.47679996482729958273694137460552155971527099609375f,-0.3572000118970866111567374900914728641510009765625f,-1.604499977409839406305991360568441450595855712890625f,0.655099990272521637280078721232712268829345703125f,-1.0243999825179574969524765037931501865386962890625f,0.13420000838041357127394803683273494243621826171875f,0.60090000200867610402610807796008884906768798828125f,-0.5797000044465061563414565171115100383758544921875f,0.646499986857175912291495478712022304534912109375f,0.30000000823289163509599575263564474880695343017578125f,-0.0254999995306131044259245754801668226718902587890625f,-0.16359997938722392607502342798397876322269439697265625f,-1.515799985212087364061517291702330112457275390625f,-0.8868999750301240769800870111794210970401763916015625f,-0.74410000410080012755997813655994832515716552734375f,0.28399995306879322054527392538147978484630584716796875f,1.3609999980926517082480131648480892181396484375f,1.062799995702504940453536619315855205059051513671875f,0.10939996031522714048378475126810371875762939453125f,0.7391000034302470655944716781959868967533111572265625f,-0.463100031638145726020638903719373047351837158203125f,1.27799999792873819615124375559389591217041015625f,-0.16220000846981985631600764463655650615692138671875f,-0.432499999701976722832341692992486059665679931640625f,-1.343200006935000079266728789662010967731475830078125f,0.44710001611113536768016274436376988887786865234375f,0.682799971115589432457682050880976021289825439453125f,-0.37630000184178324929007430910132825374603271484375f,-1.27980001006722421408312584389932453632354736328125f,0.85779998378157618077466395334340631961822509765625f,-0.073000012606382558288942163926549255847930908203125f,-1.1496999823927875894469252671115100383758544921875f,-0.244399986839294758311780242365784943103790283203125f,-0.60650000004470339121098731993697583675384521484375f,-0.65650003886222929594396191532723605632781982421875f,0.3894999975264081815140571052324958145618438720703125f,-0.7953000183522700439198160893283784389495849609375f,-0.25440002960562768663521637790836393833160400390625f,0.1525000418722637363089233986102044582366943359375f,1.23589998934268852082141165737994015216827392578125f,0.519900046980382057881797663867473602294921875f,0.2189999759793277434027913841418921947479248046875f,0.297400009876489068716409747139550745487213134765625f,1.39559998335838297833788601565174758434295654296875f,-1.4942000077843662353416220867075026035308837890625f,0.8664000063076622293323225676431320607662200927734375f,1.4225000340491538164400253663188777863979339599609375f,0.067199989396333759117396766669116914272308349609375f,-0.7384000013381240012932948957313783466815948486328125f,0.44279999659657420796321503075887449085712432861328125f,0.0380999984085563792035600272356532514095306396484375f,1.10600002454221257863764549256302416324615478515625f,-1.2597000126421458165992817157530225813388824462890625f,1.2696000016570085922040789228049106895923614501953125f,0.58040001264810570091867703013122081756591796875f,0.3498999992966655980808354797773063182830810546875f,0.1806999765366315013892517526983283460140228271484375f,0.0441999621123081443130331535940058529376983642578125f,-0.0935999853104358958688635539147071540355682373046875f,-0.4271999713659286879163801131653599441051483154296875f,1.176000021114945948141894405125640332698822021484375f,1.070800000709294597101006729644723236560821533203125f,0.3462000369012361300491420479374937713146209716796875f,0.00989998230934130418745553470216691493988037109375f,0.7532000065624717688450573405134491622447967529296875f,0.1436999843001363075245535583235323429107666015625f,0.214600000464916007558713317848742008209228515625f,-0.56090000483989665980288918945007026195526123046875f,-0.0259999925047156299484640840091742575168609619140625f,-0.4163000169292094820860938853002153337001800537109375f,-0.3617999856889244636448665914940647780895233154296875f,0.9869000234961509132602941463119350373744964599609375f,1.6022000166654581931879874900914728641510009765625f,0.12269999723434477090222571860067546367645263671875f,1.48110006346702594015596332610584795475006103515625f,0.887200013983250102000965853221714496612548828125f,1.50870001380443508054440826526843011379241943359375f,1.2200000226497653699198053800500929355621337890625f,0.81670000838041278967693870072253048419952392578125f,-0.16609997853040692206150197307579219341278076171875f,-0.979400007402896566333083683275617659091949462890625f,0.16220001558512453510019213354098610579967498779296875f,0.4979000095933674430170867708511650562286376953125f,-0.1817999963432550547537402962916530668735504150390625f,-2.1071999972939483569689400610513985157012939453125f,1.00769998754859013700979630812071263790130615234375f,-0.19580001493990462080319048254750669002532958984375f,1.1354999976307151854371113586239516735076904296875f,-0.04479999538958079341455231769941747188568115234375f,0.0644999987930059415219830043497495353221893310546875f,-0.36880003067553079887375133694149553775787353515625f,-0.780799998027086328278301152749918401241302490234375f,0.4678000436842439935247739413171075284481048583984375f,-0.2372999919250606826182092845556326210498809814453125f,0.780999989375472214447881924570538103580474853515625f,0.1280999903619288726730474081705324351787567138671875f,0.438800057423114342469716575578786432743072509765625f,0.009299971830845077391813902067951858043670654296875f,-0.63219995475113410776657474343664944171905517578125f,-0.509100009763240990423582843504846096038818359375f,-0.04580000331699896065629218355752527713775634765625f,-1.188299988341331214769525104202330112457275390625f,-0.32540001659691331070689557236619293689727783203125f,-0.27579999489784245980672494624741375446319580078125f,0.7461999668657781636937897928874008357524871826171875f,-0.8233000148653975003298910451121628284454345703125f,1.971400031006335762384651388856582343578338623046875f,0.560599999898672596287951819249428808689117431640625f,0.0851000093460074769069478861638344824314117431640625f,-0.11000000536441778553609083246556110680103302001953125f,1.551500019043683398223265612614341080188751220703125f,0.592799978342652966745163212181068956851959228515625f,-0.06910002868771603079522947155055589973926544189453125f,-0.35649999050796044031130804796703159809112548828125f,1.084100022354721470918548220652155578136444091796875f,0.512899976587295025609591903048567473888397216796875f,1.047800022673607056589162311865948140621185302734375f,0.7102999909341338735657700453884899616241455078125f,-0.836099990525841718636002042330801486968994140625f,0.913199991884827255717027583159506320953369140625f,0.5230000254213809096626164318877272307872772216796875f,0.4818999741360550093105530322645790874958038330078125f,-0.29760002640783789207290510603343136608600616455078125f,0.588699991042911818794891587458550930023193359375f,-0.06489996100068096129831474172533489763736724853515625f,-0.062300001946091809745809086962253786623477935791015625f,-0.4370000136196612317007748060859739780426025390625f,0.53769999499917020546746471154619939625263214111328125f,-0.1948000026911496096460041371756233274936676025390625f,-0.0327000154137611076521352515555918216705322265625f,-0.545599992969632463513107722974382340908050537109375f,-0.734799989727139557516011336701922118663787841796875f,-1.2471999645859008776227483394904993474483489990234375f,0.60469996831119121605269128849613480269908905029296875f,-0.66839998338222539331354710157029330730438232421875f,-1.1331999698311092661384691382409073412418365478515625f,0.23119998593926405217047204132541082799434661865234375f,0.179600000315904839709446605411358177661895751953125f,-0.6327999793857335131264107985771261155605316162109375f,0.016199997636675789181026630103588104248046875f,-0.5575999919861553966171641150140203535556793212890625f,-1.0962000352993610530205614850274287164211273193359375f,-0.061600030475855049871825031004846096038818359375f,0.0140999963521952764722300344146788120269775390625f,0.9573000593155633897168854673509486019611358642578125f,0.4190000719055533284773673585732467472553253173828125f,-1.82320003539621833255068850121460855007171630859375f,-0.3681999947160481401198239836958236992359161376953125f,-0.004199999439715773608128301930264569818973541259765625f,-0.0599999824911352508394202232011593878269195556640625f,0.48870002296864978319490546709857881069183349609375f,0.0935999980881809801047666041995398700237274169921875f,-0.1844999952912331764309783466160297393798828125f,-0.176600012631714431421414701617322862148284912109375f,0.32000001177191794710097383358515799045562744140625f,0.0236000502794981859011613778420723974704742431640625f,-0.70330002421587745065068020267062820494174957275390625f,-0.53629999202564360150091715695452876389026641845703125f,1.704200016650557980568692073575221002101898193359375f,-0.54519999021217191437216342819738201797008514404296875f,-0.9367000005573034027150924885063432157039642333984375f,0.180399996256828121232729245093651115894317626953125f,-0.200000026561319987905562811647541821002960205078125f,0.1029999899193645429562593562877736985683441162109375f,-0.1803999797910453839477895598975010216236114501953125f,0.4073000070497394542456959243281744420528411865234375f,0.8125999939978125752304549678228795528411865234375f,-0.951599967741966423773192218504846096038818359375f,-0.113400006739794889654149301350116729736328125f,0.158099967302381905209784918042714707553386688232421875f,-0.166599991658329837918017801712267100811004638671875f,0.47010000755786884862885699476464651525020599365234375f,1.040000010803341812248845599242486059665679931640625f,0.0381999933376906408710738105583004653453826904296875f,-0.65819997608959679435969292171648703515529632568359375f,0.0920999887302518249754257340100593864917755126953125f,-0.2985999714523559500634064534096978604793548583984375f,0.087700016605854358431315631605684757232666015625f,-0.64280003042221078857210159185342490673065185546875f,-0.55499999999999971578290569595992565155029296875f,-2.216300029110907843232780578546226024627685546875f,-1.682600001001357981067485525272786617279052734375f,0.65629998440742465248831649660132825374603271484375f,-0.182500003576278491124185165972448885440826416015625f,-1.947000006616114387014704334433190524578094482421875f,-0.976399968868494116946976646431721746921539306640625f,0.670199983972310509017233925987966358661651611328125f,-0.070700028094649258747494968702085316181182861328125f,0.27310005145668991843876938219182193279266357421875f,-0.5112000042676922983986287363222800195217132568359375f,0.159099974223971596387627869262360036373138427734375f,-0.8878000139564277137793624206096865236759185791015625f,0.92919999973773936829957165173254907131195068359375f,-0.6297000010937450209524968158802948892116546630859375f,-0.447200016218423090919031892553903162479400634765625f,0.018500003308057788586893366300500929355621337890625f,1.436599990689753525430205627344548702239990234375f,-0.9075999917626382806901119693065993487834930419921875f,-1.255199985387921213231265937793068587779998779296875f,0.3515999927759170251562181874760426580905914306640625f,0.367799969476461097173114467295818030834197998046875f,0.09229999471902861074568136245943605899810791015625f,-0.217600004354119302973913363530300557613372802734375f,0.1888000195741650344416484585963189601898193359375f,-0.35640000209808331277372417389415204524993896484375f,1.101699988710880262487989966757595539093017578125f,-0.440999975591898074611663105315528810024261474609375f,-0.578799977254867759057788134668953716754913330078125f,-0.23120002185106247605261842181789688766002655029296875f,0.671199995699524709635852559586055576801300048828125f,0.8265999827921388654061729539535008370876312255859375f,-0.253299996611475730645679504959844052791595458984375f,-1.761399968868494259055523798451758921146392822265625f,-0.72609998031854683375740933115594089031219482421875f,0.25500002458691550799585456843487918376922607421875f,0.94979998413920352362538324086926877498626708984375f,-1.24250001117587061116864788345992565155029296875f,-0.8743000436544416942297175410203635692596435546875f,-0.172299998295307066342729740426875650882720947265625f,-0.6081000548839572417136878357268869876861572265625f,0.06870001007914527946240923483856022357940673828125f,-1.387900043642520842013254878111183643341064453125f,1.01440007162690282171979561098851263523101806640625f,1.7223000167727473552758965524844825267791748046875f,0.16980003107786245664101443253457546234130859375f,-0.3307999898314477604799321852624416351318359375f,-0.102100025051832243860872040386311709880828857421875f,0.4132999812632800473721772505086846649646759033203125f,0.443299998623132740505070614744909107685089111328125f,0.58229997855126836014960645115934312343597412109375f,0.129699982318281836768392167869023978710174560546875f,0.499499985381960609487350666313432157039642333984375f,-0.80160000961422905874087518895976245403289794921875f,1.0401000039815908593254789593629539012908935546875f,0.507299977880716124190030313911847770214080810546875f,0.68920003877878155407188387471251189708709716796875f,-0.713499985054135255069240884040482342243194580078125f,0.65640000045895607883039701846428215503692626953125f,-0.13959995158910754042835833388380706310272216796875f,0.46089997667670257186500748503021895885467529296875f,-0.675199975925683926192277795053087174892425537109375f,0.861100002372264583527794457040727138519287109375f,-1.167400002947449610957164622959680855274200439453125f,0.0912000117182731884213353623636066913604736328125f,-0.6233999474704265963964644470252096652984619140625f,0.467899974501133186777224182151257991790771484375f,0.488799987238646149734222490224055945873260498046875f,-1.2711999771475799292375086224637925624847412109375f,-0.100800006893277060981972681474871933460235595703125f,0.10799995277821949901664311255444772541522979736328125f,1.12350002864003162272865665727294981479644775390625f,0.07559997791945960443626972846686840057373046875f,0.07140003726482380130846649990417063236236572265625f,0.0515999668478972495222478755749762058258056640625f,0.072699980992077417596419763867743313312530517578125f,1.0710999391913418321564677171409130096435546875f,0.29720005466342147570912857190705835819244384765625f,-0.135899978315830338715386460535228252410888671875f,-1.4322000713527207604869317947304807603359222412109375f,-1.4208000083088876408510259352624416351318359375f,-0.44870001246333135469512853887863457202911376953125f,-0.47120000918507543019586591981351375579833984375f,0.438599969142675405464615323580801486968994140625f,-1.27789995185136806554737631813623011112213134765625f,-0.4319999858289957916923640368622727692127227783203125f,-0.2513999720722441821862958022393286228179931640625f,0.412699976372719046224801786593161523342132568359375f,-0.817299947705865559299809319782070815563201904296875f,0.73729996804595021586692382697947323322296142578125f,-1.2231999447226531430032991920597851276397705078125f,-0.1390999821960929860864553120336495339870452880859375f,-0.244699975277483883218110349844209849834442138671875f,1.1894999209344394586196358432061970233917236328125f,0.03629998500347164647195086217834614217281341552734375f,0.942499998807907246600734652020037174224853515625f,0.8354999861940743954846766428090631961822509765625f,0.136799977046251353129946437547914683818817138671875f,0.5301999974951148164592495959368534386157989501953125f,-0.0507999980270863460418695467524230480194091796875f,0.237999983288347982668398117311880923807621002197265625f,1.249199995565414500475753811770118772983551025390625f,-0.43910001449435898113193843528279103338718414306640625f,-0.6095000027231864958565665801870636641979217529296875f,0.437999968051910160138362471116124652326107025146484375f,-0.316400021730363079086600919254124164581298828125f,1.08770001668035920516786063672043383121490478515625f,-0.42729999978542299032824303139932453632354736328125f,-0.51790000784248124698905257901060394942760467529296875f,0.231899985051155166360103976330719888210296630859375f,0.77629997278451912112728905412950552999973297119140625f,0.752799955916405139078051433898508548736572265625f,0.62800001048296660854930451023392379283905029296875f,-1.392399984097480558631332314689643681049346923828125f,-1.16500000007450577044210149324499070644378662109375f,-0.02670001411735967167260241694748401641845703125f,0.15699997968226686584358731124666519463062286376953125f,0.5650999704912307297632878544391132891178131103515625f,0.434999991953372244779529864899814128875732421875f,-1.0484999803602697010518340903217904269695281982421875f,1.03659997712969786931580529198981821537017822265625f,-0.462599973583221579787050359300337731838226318359375f,1.340899962371587594844868362997658550739288330078125f,0.7681999836146833349204143814858980476856231689453125f,-0.1366999942243101173744435072876513004302978515625f,-0.78339997980594677784438317758031189441680908203125f,-0.393299988490342311564518240629695355892181396484375f,0.29340001981556351751123656868003308773040771484375f,1.0395999403201046096256732198526151478290557861328125f,-0.0816999922871588513118012997438199818134307861328125f,-0.46059995705783396413579566797125153243541717529296875f,0.8116000419497482187836112643708474934101104736328125f,-0.77789999774694396084129266455420292913913726806640625f,-0.15889999300837498452665386139415204524993896484375f,-1.16089999772459240290345405810512602329254150390625f,0.16850003743171715342441530083306133747100830078125f,-0.90260001776516440852304867803468368947505950927734375f,-0.2966999928832052546567865647375583648681640625f,0.2293999466061595882138135493732988834381103515625f,1.4303000404059904582254603155888617038726806640625f,-0.4991000118866566470643419961561448872089385986328125f,1.121700087505579812585665422375313937664031982421875f,0.6394000393286340955256719098542816936969757080078125f,-1.44649995824694688195677372277714312076568603515625f,-0.226399982540309796519295559846796095371246337890625f,-0.9951000144124027002590082702226936817169189453125f,-1.3175999938860536442319926209165714681148529052734375f,-0.59339997682571432591203119955025613307952880859375f,0.503200031857192531248301747837103903293609619140625f,0.504799954485893209721325547434389591217041015625f,-1.47560000064372953687552580959163606166839599609375f,-0.3634999951124182615558311226777732372283935546875f,0.84130002966970263145185526809655129909515380859375f,-0.95630003507137217866329592652618885040283203125f,-0.6745999303177014905230635122279636561870574951171875f,0.4813999669313435703088543959893286228179931640625f,-0.6286999607563021896794452914036810398101806640625f,1.28039997211694700496309451409615576267242431640625f,-1.25779995248913767369458582834340631961822509765625f,1.09019998412132235188209961052052676677703857421875f,-1.36819997959136951948266869294457137584686279296875f,1.21490000168085021670094647561199963092803955078125f,-1.29250002115964690574401174671947956085205078125f,-0.572699974137545808616778231225907802581787109375f,-1.2458000032424916270201720180921256542205810546875f,0.762699977564810893682079040445387363433837890625f,0.59119998973906007222467451356351375579833984375f,-0.66530004657134489409742172938422299921512603759765625f,0.45010000547170658791173991630785167217254638671875f,0.54749999806284888048679704297683201730251312255859375f,0.2946999825604257328137691729352809488773345947265625f,0.5957000146090987158231655484996736049652099609375f,-0.8345999522969129724714321127976290881633758544921875f,-0.098699981208146159161742616561241447925567626953125f,-0.5564999886453152466714300317107699811458587646484375f,-1.822499995753168899881302422727458178997039794921875f,0.5665000051483506382510313414968550205230712890625f,1.134200011062620916391097125597298145294189453125f,-0.8864999785497789108745791963883675634860992431640625f,1.4506999852351836199915169345331378281116485595703125f,1.28969997248351564422819137689657509326934814453125f,0.6521000234872094125648800400085747241973876953125f,1.481600017120689560812252238974906504154205322265625f,-0.7685000073313705826194563996978104114532470703125f,-2.485299967315793168154414161108434200286865234375f,-1.05349991509318474669498755247332155704498291015625f,-0.242099983850121436290692145121283829212188720703125f,2.18659994203746332885884839924983680248260498046875f,0.6497000057876112233401499906904064118862152099609375f,1.1054000040054319242699421010911464691162109375f,0.6440000000819552372632870174129493534564971923828125f,-0.8269999586530027979591750408872030675411224365234375f,0.53549999040365203750724276687833480536937713623046875f,0.3097000157713887258381646461202763020992279052734375f,-0.33480003325715601558698608641861937940120697021484375f,0.49969994305372278375898531521670520305633544921875f,-1.9191998996019368206589206238277256488800048828125f,-0.445499996960162902848168187119881622493267059326171875f,-0.247899993984401323388055971008725464344024658203125f,1.53629997367858894818937187665142118930816650390625f,-0.26640000377446459989272398161119781434535980224609375f,-1.021900029158591127753652472165413200855255126953125f,-0.62959999867677696183676516739069484174251556396484375f,0.48790006048083345291388468467630445957183837890625f,-0.04650001397728920071727998220012523233890533447265625f,0.380599965029954301343195766094140708446502685546875f,-0.20229998037666074850449149380438029766082763671875f,-0.750800009147078117877072145347483456134796142578125f,-0.26200000594556327460082911784411408007144927978515625f,0.16810000159367921579445237512118183076381683349609375f,0.51679999837353796099392866381094790995121002197265625f,0.3256999710977073192452735383994877338409423828125f,-0.33099999034404714581825146524352021515369415283203125f,-1.804999991655348967611871557892300188541412353515625f,-0.606399977809190193767108212341554462909698486328125f,-0.20610000267028727449769576196558773517608642578125f,0.303099994310736864377986421459354460239410400390625f,-0.721699985730648307935553020797669887542724609375f,-1.848900001055002295657914146431721746921539306640625f,0.32520001768618811421873715516994707286357879638671875f,0.9749000184074041452930714513058774173259735107421875f,0.23210000940561259508143621133058331906795501708984375f,0.298700002833455879880375505308620631694793701171875f,-0.92989997813701652518147966475225985050201416015625f,-1.06140002940446098733673352398909628391265869140625f,1.1333999365925802038645997527055442333221435546875f,-0.99619999711513429474507574923336505889892578125f,0.43509996047019949827472373726777732372283935546875f,-0.88249999046325644513899533194489777088165283203125f,-0.40330000219941108952070862869732081890106201171875f,-0.3413999553084376969991353689692914485931396484375f,0.237499993722886160174567748981644399464130401611328125f,0.0113999944426115273898858504253439605236053466796875f,-0.0995999918222427726988144058850593864917755126953125f,0.46789998107627017720488993290928192436695098876953125f,-1.04519997637271888635268624057061970233917236328125f,0.51789997509718066481809728429652750492095947265625f,1.343899979150295020957628366886638104915618896484375f,0.9769000139966601548024982548668049275875091552734375f,1.469400003081558825357433306635357439517974853515625f,0.9470000180527573530042673155548982322216033935546875f,0.0853999448478219846947467885911464691162109375f,-0.16019999887347224909461829156498424708843231201171875f,-0.101899971640109665571571895270608365535736083984375f,-0.45010001020282486283718981212587095797061920166015625f,-0.81910000737905530598936820751987397670745849609375f,0.494299956072867363587874933728016912937164306640625f,1.3171999515473842290447237246553413569927215576171875f,0.64730000015795197132462135414243675768375396728515625f,0.21830001471638649235273987869732081890106201171875f,1.049699993903935091310586358304135501384735107421875f,-0.807899986124038349544207449071109294891357421875f,-0.682100032725930560673077707178890705108642578125f,-0.679299966168404267818914377130568027496337890625f,0.06140002001672961728928612501476891338825225830078125f,-0.74700001019239348210021489649079740047454833984375f,0.85729995739161990453425232772133313119411468505859375f,1.234599917018414316771668381989002227783203125f,0.1289999893158670918325015009031631052494049072265625f,-0.1283999570071703288931530551053583621978759765625f,-0.4052999579653147055324780012597329914569854736328125f,0.80569998316764834811465334496460855007171630859375f,0.63559997814297641838265917613171041011810302734375f,-1.1990000370740878654629568700329400599002838134765625f,1.586899993172287803844255904550664126873016357421875f,-0.469099959099292806996572835487313568592071533203125f,1.3087999608635900727904299856163561344146728515625f,0.37559998946785899676115150214172899723052978515625f,0.79139995322227552065896816202439367771148681640625f,1.6662999698042870733161180396564304828643798828125f,-0.3083999982833862674169722595252096652984619140625f,-1.508899953669309734749504059436731040477752685546875f,0.10700001704692763127013677149079740047454833984375f,0.381899979537725808853565467870794236660003662109375f,-0.1317000065550208154352418432608828879892826080322265625f,0.66030000540688649213194594267406500875949859619140625f,0.7265999715976418915630574701935984194278717041015625f,-0.13500000976025983145945019714417867362499237060546875f,-0.22460000463724150154831704639946110546588897705078125f,0.61340003032088308287939071306027472019195556640625f,0.65459998031109567495633427824941463768482208251953125f,0.32619999189972848085972145781852304935455322265625f,0.9541000087201594315189367989660240709781646728515625f,1.024199989306925573373519000597298145294189453125f,1.1144999774470931708236776103149168193340301513671875f,0.1457999302268024877093921531923115253448486328125f,0.648300002422928844936222958494909107685089111328125f,-0.26469998537302041885510561769478954374790191650390625f,-0.7727999989621336940359697109670378267765045166015625f,0.55669996449649372660672952406457625329494476318359375f,0.5746999805301433728033089209930039942264556884765625f,-0.388099983954429461618929053656756877899169921875f,0.000799985398352465182369996909983456134796142578125f,1.11229996253252050308901743846945464611053466796875f,0.9839999822005627283516560055431909859180450439453125f,0.8285999791264533342882714350707828998565673828125f,-0.0455999698355800120452840928919613361358642578125f,1.23219997255802127256174571812152862548828125f,-0.8977999770201743334752109149121679365634918212890625f,-0.7484999730587009736382242408581078052520751953125f,0.38849995743110887769233841027016751468181610107421875f,0.16480002392530368471312840483733452856540679931640625f,-0.25830002756863723600844195971149019896984100341796875f,-0.427999997109175023268790027941577136516571044921875f,0.62159995790720046215938054956495761871337890625f,0.2385999843418602495148661546409130096435546875f,0.64610000028610148348207076196558773517608642578125f,-0.104500024199484897735601407475769519805908203125f,-0.1189999735951428050384492962621152400970458984375f,-0.3316999723196030203098416677676141262054443359375f,0.3552000211879609015142023054067976772785186767578125f,0.740899985766410029697226491407491266727447509765625f,0.2817000293910505437366964542889036238193511962890625f,-0.826300035965443324670332003734074532985687255859375f,-0.66069997087419063408475494725280441343784332275390625f,-0.1917999894328424048683956470995326526463031768798828125f,0.13199999827146535746891231610788963735103607177734375f,-0.40760003691315682683438126332475803792476654052734375f,1.0868000120826060150847069962765090167522430419921875f,0.823199996615945561728722168481908738613128662109375f,-0.105599941597878721477599128775182180106639862060546875f,-0.17739997627437109617432042796281166374683380126953125f,-2.7020999113559724946753703989088535308837890625f,-0.023299965393543065061976449214853346347808837890625f,1.014999945163726380314983543939888477325439453125f,1.231400002396106696522792844916693866252899169921875f,0.704899972474574809666592045687139034271240234375f,0.01209997870922041585117767681367695331573486328125f,-0.6130999353766437565127489506267011165618896484375f,-1.17559995757937496563272361527197062969207763671875f,-0.7695999982297418995358384563587605953216552734375f,-0.56600000405311590156998136080801486968994140625f,1.3139999381303795189523953013122081756591796875f,-0.5174000082373619857634139407309703528881072998046875f,-0.4830000554472209106648961096652783453464508056640625f,0.31999999489635211347149379434995353221893310546875f,-0.6413000107079742662108401418663561344146728515625f,1.903900011895597099709220856311731040477752685546875f,-0.8492000017493968044846042175777256488800048828125f,-0.413000027395785007655604204046539962291717529296875f,0.54170001754462671073042656644247472286224365234375f,-0.2555999807506796894784883988904766738414764404296875f,0.738699986982345624397794381366111338138580322265625f,9.9988335370948533409318770281970500946044921875e-05f,-0.174200010615587597140319076061132363975048065185546875f,-0.737899987837672188817350615863688290119171142578125f,1.6939000162914392699775589790078811347484588623046875f,-0.09130001838207224551524632261134684085845947265625f,0.336300009627640117404467900996678508818149566650390625f,-0.45530006380081200489939874387346208095550537109375f,-1.3883999789118763867890038454788736999034881591796875f,0.4801000132203103731853843783028423786163330078125f,-1.7125999900490038729827801944338716566562652587890625f,-0.0635000115036969958737245178781449794769287109375f,0.28190001891404381506589515993255190551280975341796875f,-0.088800013837218472190215834416449069976806640625f,0.251400024859607551519502521841786801815032958984375f,-0.0564999824613334755696314459783025085926055908203125f,1.82529994786977756149326523882336914539337158203125f,0.120799996238947215942971524782478809356689453125f,0.089699988800287400181332486681640148162841796875f,0.36359997406005906128712013014592230319976806640625f,0.80830000786185163264008224359713494777679443359375f,0.083499972015619761123161879368126392364501953125f,-0.3500999843120577992294784053228795528411865234375f,-1.72859999551773046988500937004573643207550048828125f,-0.0319999899268150755915485206060111522674560546875f,0.0283999970912931498645548344939015805721282958984375f,1.4822000098854299920958510483615100383758544921875f,-0.203299996499717305908916387124918401241302490234375f,0.546000011578202215645205797045491635799407958984375f,0.2257999857336283611175531405024230480194091796875f,-0.969400021260977240444844937883317470550537109375f,-0.522699944037198438451241599977947771549224853515625f,-0.189200039374828321570021216757595539093017578125f,0.111099989184737069081165827810764312744140625f,0.7582000239968298416215475299395620822906494140625f,-0.5470000109374524299710174091160297393798828125f,0.6712000059068203849932388038723729550838470458984375f,0.716999978080391731083409467828460037708282470703125f,-0.3216999914675950922315905700088478624820709228515625f,0.29499995749443730375105587881989777088165283203125f,0.7589000615164642749022050338680855929851531982421875f,-1.2521000225186345033989709918387234210968017578125f,0.2162000348895787293912462700973264873027801513671875f,-0.084200027155876711049131699837744235992431640625f,0.40219998347312202469083786127157509326934814453125f,0.72000002413988095639751918497495353221893310546875f,-0.3774000029847022208429052625433541834354400634765625f,0.759000026717782105833975947462022304534912109375f,0.009299993325769750907028310393798165023326873779296875f,-0.1323000050753349032817141051054932177066802978515625f,-0.091099980281293735373537856503389775753021240234375f,-1.690100020968912986063514836132526397705078125f,1.733699985790252640072139911353588104248046875f,-1.096599949562548914627768681384623050689697265625f,0.256199960756301692299530259333550930023193359375f,1.6256000048160554882770156837068498134613037109375f,0.585099944972991892200298025272786617279052734375f,0.48859998046755759304460298153571784496307373046875f,-0.74589994851350827076430505258031189441680908203125f,-0.0371000019550324910966310199000872671604156494140625f,-0.3030999831348655249030343838967382907867431640625f,-0.828399980960786042061272382852621376514434814453125f,0.17139999669641259316676951129920780658721923828125f,-1.697399980670212471522972919046878814697265625f,0.37270002485737230291107380253379233181476593017578125f,-1.155700008723139493582721115672029554843902587890625f,0.62830003436729331411925159045495092868804931640625f,0.5077000086709853032829187213792465627193450927734375f,-0.208399989789724016109317972222925163805484771728515625f,0.6312000284075740541567256514099426567554473876953125f,0.8211000216692685427233300288207828998565673828125f,-1.827000024646520781601566341123543679714202880859375f,1.1462000197276480673025389478425495326519012451171875f,1.285700010436773066402338372427038848400115966796875f,-0.580699990059434956179984510526992380619049072265625f,-0.49530002557933328066752665108651854097843170166015625f,-1.946100056910515352370794062153436243534088134765625f,0.54450004614144542625098210919531993567943572998046875f,0.000599970468878385787547813379205763339996337890625f,0.24779999540448205408438298036344349384307861328125f,-0.0650000266358252698495334698236547410488128662109375f,0.3275000175088635945286341666360385715961456298828125f,-1.2224000486940147514047794174985028803348541259765625f,0.4514000570088629871179364272393286228179931640625f,1.032699961434303759233443997800350189208984375f,-0.70980002567619016051736480221734382212162017822265625f,-0.48680002659261234487786396130104549229145050048828125f,-0.3193000028252601651956865680404007434844970703125f,-0.88469997244626252808075150824151933193206787109375f,-0.94109998830929464386230165473534725606441497802734375f,-1.444299951323122055413250564015470445156097412109375f,2.13379994640946346606824590708129107952117919921875f,0.09679999030828501105361283407546579837799072265625f,-0.15630000355541806555947914603166282176971435546875f,-0.59599994251132049072339214035309851169586181640625f,-1.4690999912858018205952248536050319671630859375f,0.055099983268977137385036257910542190074920654296875f,1.205499962724745355302502503036521375179290771484375f,0.318499938413501215705991853610612452030181884765625f,-2.218099999451637049929786371649242937564849853515625f,0.64900002070143847365812916905269958078861236572265625f,1.052800034259259831515009864233434200286865234375f,0.4304000126667322234652601764537394046783447265625f,-0.66520005151182459712799754925072193145751953125f,1.0241000220566987000125891427160240709781646728515625f,-1.526500005260109649185551461414434015750885009765625f,1.0329999871253965348927295053726993501186370849609375f,1.561400006121397066038980483426712453365325927734375f,-0.1771999755382538399572922571678645908832550048828125f,0.2377000202938914785733004464418627321720123291015625f,0.651800019142032116548080011853016912937164306640625f,-1.08970005365759181614748740685172379016876220703125f,-0.01420003069490238800653969519771635532379150390625f,-0.26430003832727722024031891123740933835506439208984375f,-0.8250000100582834949847210737061686813831329345703125f,0.19090000662803685571589085157029330730438232421875f,-1.04079999307244985828901917557232081890106201171875f,1.18130003119707094327850427362136542797088623046875f,0.17000000663101655362652309122495353221893310546875f,-0.13519998620748463480367718148045241832733154296875f,-1.0271000240460044228285596545902080833911895751953125f,0.11299998980760594946559649542905390262603759765625f,-0.5766999939635393257475470818462781608104705810546875f,0.216700020301342277662115520797669887542724609375f,-0.64549998440593492343708703629090450704097747802734375f,-0.3652000589065256264120762352831661701202392578125f,1.24960000744983545928334933705627918243408203125f,-1.094500055640935887169007401098497211933135986328125f,0.48719999759197207822580821812152862548828125f,0.35840002212524435520890619955025613307952880859375f,-0.313199995386600704705415409989655017852783203125f,0.286700040119886256917425271240063011646270751953125f,-0.61220000101923943702786345966160297393798828125f,-1.23690004510283468874831669381819665431976318359375f,1.7116999907970438243864919058978557586669921875f,-1.3139000192344187922799392254091799259185791015625f,0.749900001904368007643597593414597213268280029296875f,0.2747999954640867326105535539682023227214813232421875f,-0.91159999944418712569671470191678963601589202880859375f,-0.20779999335557197692736508543021045625209808349609375f,-0.603299997170269275414966614334844052791595458984375f,0.6674999848380680500525841125636361539363861083984375f,-0.89209999487698021791715063955052755773067474365234375f,-0.640700012373924376873901564977131783962249755859375f,-0.5154000206947326745421378291212022304534912109375f,0.2927000024870041183788771377294324338436126708984375f,-1.065000023022294239893881240277551114559173583984375f,-0.0634999796897173229837108010542578995227813720703125f,0.20980005167871718096961330957128666341304779052734375f,-0.8017000176936388555759549490176141262054443359375f,0.145600023442506998350154390209354460239410400390625f,-0.87219998883753990792655486075091175734996795654296875f,1.04609998732209152905170412850566208362579345703125f,-1.39400001204013701538997338502667844295501708984375f,0.9921000076547261681270128974574618041515350341796875f,-0.931900018280744202314735957770608365535736083984375f,1.2192000451490285772848665146739222109317779541015625f,-1.35480001289844498302272768341936171054840087890625f,1.2483000153124326203624150366522371768951416015625f,-0.213900006717442803250150973326526582241058349609375f,0.2080000279545775043743560672737658023834228515625f,-0.348399970269203052009032717251102440059185028076171875f,0.264900001829861952984401796129532158374786376953125f,-0.765999952085316682115490038995631039142608642578125f,0.333099974790215913600377461989410221576690673828125f,-0.7687999954342841224530502586276270449161529541015625f,1.30209992014765862933245443855412304401397705078125f,-1.09779998840093551137897520675323903560638427734375f,0.99669995145797773972162758582271635532379150390625f,-0.7964999142885209604258989202207885682582855224609375f,0.76739996040463420712285369518212974071502685546875f,0.73719999625086796157802382367663085460662841796875f,-0.7034000040203334247479460827889852225780487060546875f,1.000400000578164583231455253553576767444610595703125f,-1.0045999876499169811694400777923874557018280029296875f,0.1545000347793104022997567881247960031032562255859375f,0.3700999797672037683327062040916644036769866943359375f,-1.320600058348477379155383459874428808689117431640625f,0.650300035861134606562927729100920259952545166015625f,1.6846000471800568476510306936688721179962158203125f,-0.19499999940395351671895696199499070644378662109375f,-1.4701000262588264089202994000515900552272796630859375f,1.00800000813603407578966653090901672840118408203125f,0.7787000068753957560119260961073450744152069091796875f,0.53709999282807086018465270171873271465301513671875f,-1.37270001221001081148642697371542453765869140625f,-0.0021000279948115707639999527600593864917755126953125f,0.0930999749392269482228812194080092012882232666015625f,-0.49390003823340011823717077277251519262790679931640625f,0.967700063395501075547144864685833454132080078125f,-0.69850001388788240941352114532492123544216156005859375f,-0.2659000164628029239111128845252096652984619140625f,-0.71620000996738664156282538897357881069183349609375f,0.313600028449297241195381502620875835418701171875f,-0.022500009462237624546787628787569701671600341796875f,-0.050200019735097800577250382048077881336212158203125f,0.916100009822844896234528278000652790069580078125f,-0.3145000225603584720346361791598610579967498779296875f,0.92649995236098892092968526412732899188995361328125f,0.097599978612362914276445735595189034938812255859375f,1.80449995122104933642503965529613196849822998046875f,-0.206199995774030586659364416846074163913726806640625f,-1.5483999857664105803678467054851353168487548828125f,0.7660000050961972561935908743180334568023681640625f,0.246699991542101315644686110317707061767578125f,-0.913300007712841033225004139239899814128875732421875f,-0.392500020712613917339695035479962825775146484375f,-0.06130000698268389669465250335633754730224609375f,0.6756999754562975635252541906083934009075164794921875f,-0.5717000230208040267854130433988757431507110595703125f,0.19529999894350746902915716418647207319736480712890625f,0.747000046402216621999059498193673789501190185546875f,-0.996600043737888796613333397544920444488525390625f,-0.8427000194743279326559104447369463741779327392578125f,-0.278899999713897506214976829141960479319095611572265625f,0.50720000511705887191737929242663085460662841796875f,-0.510100013406574959162753657437860965728759765625f,-1.006000003829598643534382063080556690692901611328125f,0.0953999920472503593060764615074731409549713134765625f,-0.1523999797761437235976700321771204471588134765625f,0.97680000096261510034167940830229781568050384521484375f,-1.2327000107944012796679089660756289958953857421875f,-0.1069999946951865066324671715847216546535491943359375f,0.5630999905109401737490770756267011165618896484375f,-0.389999984279274958254291050252504646778106689453125f,0.7283000109910953145941903130733408033847808837890625f,-0.267500012367964667969744141373666934669017791748046875f,0.29299997431039859208112829946912825107574462890625f,0.331599967965483966025885820272378623485565185546875f,-1.1832999579429621661574856261722743511199951171875f,-0.747600003981589988910627653240226209163665771484375f,-0.10309998157024491405309163383208215236663818359375f,-0.86170003065764932959069710705080069601535797119140625f,-0.37000002101063689252669064444489777088165283203125f,-0.823199995665996997473712326609529554843902587890625f,0.52919999020099606212852449971251189708709716796875f,-0.6422999998226763818109930070932023227214813232421875f,1.3261000178694732909434605971910059452056884765625f,-0.04619996651187541214511611542548052966594696044921875f,-0.0829999972581862976994671043939888477325439453125f,-0.04430000528395183767571552380104549229145050048828125f,0.2595000449120998808894000831060111522674560546875f,1.7168000333726407546919290325604379177093505859375f,-0.932300018113850992307334308861754834651947021484375f,-0.4826000007033346417273378392565064132213592529296875f,0.1905000108927492608046350142103619873523712158203125f,0.77120004330873559439396558445878326892852783203125f,0.48480001118481208521870939875952899456024169921875f,-0.3164999756813049458514797152020037174224853515625f,-0.749300014075637843546928706928156316280364990234375f,-2.5547000387191776127338016522116959095001220703125f,0.824000033944844556543785074609331786632537841796875f,1.05770000200271585555356068653054535388946533203125f,0.984300068464875721474527381360530853271484375f,-0.621799998950958165977453973027877509593963623046875f,0.076200010228156589420223099295981228351593017578125f,0.3513000248081983301062791724689304828643798828125f,-0.8369000337779528564396969159133732318878173828125f,-0.21219999036490921895392602891661226749420166015625f,-0.08600000792741724353618337772786617279052734375f,-0.855000014379620143500915219192393124103546142578125f,2.2597999783277504803891133633442223072052001953125f,0.210300001662969027194094451260752975940704345703125f,-1.3659999617338183242054583388380706310272216796875f,-0.85939998199641731702058677910827100276947021484375f,-0.1823000289916993921224275254644453525543212890625f,0.19129998992085450737477003713138401508331298828125f,-0.774099972957372983728419058024883270263671875f,0.36480001287981878022037562914192676544189453125f,0.5652999747291207466304285844671539962291717529296875f,-0.7028999966107309926854895820724777877330780029296875f,1.065999970935285201534270527190528810024261474609375f,-0.39640000364407879107631060833227820694446563720703125f,0.22890001183971708798026156728155910968780517578125f,1.05589999891668551157408728613518178462982177734375f,-1.357100010746717710929942768416367471218109130859375f,-1.164500023305416132046730126603506505489349365234375f,0.9091999794721612460079995798878371715545654296875f,-0.61749998368322867037250034627504646778106689453125f,1.1455999976634974046874049236066639423370361328125f,-0.806300011005997685487045600893907248973846435546875f,-0.403200008480995908399791005649603903293609619140625f,-1.55260003508776467384677744121290743350982666015625f,0.26379998127818116149256866265204735100269317626953125f,-0.185700041095912371513776406573015265166759490966796875f,0.2749999801255766929131141296238638460636138916015625f,-1.4594999969862396671516080459696240723133087158203125f,-0.4657999776124952528988387712161056697368621826171875f,1.5258000591218472408172601717524230480194091796875f,-0.9131000386416925351795725873671472072601318359375f,-2.2733000208258626884116893052123486995697021484375f,-0.509500014439225434870195385883562266826629638671875f,0.0896999691307545088392316756653599441051483154296875f,0.9261000328451398644347136723808944225311279296875f,-1.1535000080019235380035524940467439591884613037109375f,0.30230003102198221842655811997246928513050079345703125f,-0.71869999892190106205447364118299447000026702880859375f,0.59599996592477022527845065269502811133861541748046875f,0.53589996527731476039235758435097523033618927001953125f,-1.09089996654391274688578050700016319751739501953125f,-0.82429997138380972643290078849531710147857666015625f,2.014099976384638779336455627344548702239990234375f,0.056900016567110522913708337000571191310882568359375f,0.296500018298626155655028924229554831981658935546875f,-0.669999972768127616262745505082421004772186279296875f,1.9148000090241428239323795423842966556549072265625f,1.3444000352866947167740363511256873607635498046875f,0.617500042319298447779374328092671930789947509765625f,1.4401000362053515413407467349315993487834930419921875f,-1.12809997337460554689414493623189628124237060546875f,0.49409999746978250367845930668408982455730438232421875f,-0.13780001179575851466552194324322044849395751953125f,-1.1095999980993564992814981451374478638172149658203125f,0.32330002067685104094607595470733940601348876953125f,0.225400002962350798352275660363375209271907806396484375f,0.10049999077618121390287342364899814128875732421875f,-0.616400012230872818008720059879124164581298828125f,2.80690005165934675090966265997849404811859130859375f,-0.35329998357295977484682225622236728668212890625f,0.28410000052452044627671057241968810558319091796875f,-0.89710005917549207055117221898399293422698974609375f,0.0962000229686494900960269660572521388530731201171875f,0.03640003711581218936998993740417063236236572265625f,-1.04579998491406467309161598677746951580047607421875f,0.74679997466206504697083801147527992725372314453125f,0.50069998823404315846374856846523471176624298095703125f,-0.426600006857514646441131844767369329929351806640625f,0.6439000441938642627093258852255530655384063720703125f,-0.1122999951660632600436429129331372678279876708984375f,-0.973000063717364582771551795303821563720703125f,-0.40570004292130423806383987539447844028472900390625f,0.925500002026557577750054406351409852504730224609375f,0.9513000029593701167840436028200201690196990966796875f,-0.637900015479325954714795443578623235225677490234375f,0.30249999791383752612006219351314939558506011962890625f,0.341200033473967767605472545255906879901885986328125f,-0.37829996964037493345500706709572114050388336181640625f,-0.0377000163078304861841161255142651498317718505859375f,0.262299973857402834997998297694721259176731109619140625f,-0.47130000050067888395233239862136542797088623046875f,1.3418000004410741698990250370115973055362701416015625f,-1.31490000264942619168095916393212974071502685546875f,-0.6579999962709839333996342247701250016689300537109375f,0.6225999698393043235711274974164552986621856689453125f,-0.0376999852202837271164526100619696080684661865234375f,-0.8978999759726222773537074317573569715023040771484375f,-0.6751000189200044854231919089215807616710662841796875f,-0.1314999853670590912457782906130887567996978759765625f,-0.02429998920932396888616722208098508417606353759765625f,-0.7634000220507386469392940853140316903591156005859375f,-0.768800035574287665696147087146528065204620361328125f,0.548800015699863141804826227598823606967926025390625f,-0.0830000118613245518872645334340631961822509765625f,-1.276999985978006346698521156213246285915374755859375f,-0.1440999897956845021695926334359683096408843994140625f,-0.1878999703101815743622182708350010216236114501953125f,-1.11709995479285684183423654758371412754058837890625f,-1.04039999344423517158020331407897174358367919921875f,1.9008000179760153880437201223685406148433685302734375f,-0.0527000078141692807065510351094417273998260498046875f,0.455700021016598011414089341997168958187103271484375f,-0.51110004916191187351870439670165069401264190673828125f,0.002000008702278233130300577613525092601776123046875f,-0.940800010879338177716135760420002043247222900390625f,0.754399985051155130832967188325710594654083251953125f,0.319700005191564162743134147603996098041534423828125f,-0.124799990397691740184882291941903531551361083984375f,0.55169997086674038566656008697464130818843841552734375f,-0.6936999949172133472785617414047010242938995361328125f,-0.33909995233789158586290568564436398446559906005859375f,0.26940005225539154221081616924493573606014251708984375f,-0.0441000434398659546531007435987703502178192138671875f,0.95220001189708669553368736160336993634700775146484375f,-0.1266999792486427356408285049838013947010040283203125f,-0.0933999969422816267439202420064248144626617431640625f,1.19830001330077617893721253494732081890106201171875f,-0.5620000134892759735549816468846984207630157470703125f,0.62959995738193441638941294513642787933349609375f,-2.1593000284552577028307496220804750919342041015625f,1.5218999883294106467701567453332245349884033203125f,-0.90310000865310424700282965204678475856781005859375f,2.23580003900527923832441956619732081890106201171875f,0.3192000009857116626932338476763106882572174072265625f,-0.4393999673187725107936785207130014896392822265625f,1.177400028894096895015763948322273790836334228515625f,1.336700019183755028251425756025128066539764404296875f,0.76570001205727467041839418016024865210056304931640625f,-0.022499990165233185734905418939888477325439453125f,0.06749999847263088437188116586185060441493988037109375f,-1.34160007758215105155841229134239256381988525390625f,0.82760003538578796877800414222292602062225341796875f,0.615699953216314543880116616492159664630889892578125f,0.34640002206564002307231930899433791637420654296875f,0.86699997696280473036267721909098327159881591796875f,0.0421000102996824665524400188587605953216552734375f,1.7490000164359802425906309508718550205230712890625f,0.86289999960958940761202029534615576267242431640625f,0.443700002107024371600374479385209269821643829345703125f,-0.5545000207722186047476498060859739780426025390625f,1.746100028896332201355789948138408362865447998046875f,0.364200013595819382938856278997263871133327484130859375f,0.940500000406056191337711425148881971836090087890625f,0.13430003111884014632693151725106872618198394775390625f,0.197999941825866443423365126363933086395263671875f,0.4277999775230885148857851163484156131744384765625f,0.679099955523014386926661245524883270263671875f,0.7847999722182752435628572129644453525543212890625f,-1.6115000215768822133810544983134604990482330322265625f,0.7288000113785262357168903690762817859649658203125f,0.73869999696612342887647173483856022357940673828125f,0.1627999955534935594414491788484156131744384765625f,-1.51120002155303989610501957940869033336639404296875f,0.0351000212296843638881682636565528810024261474609375f,0.59590000552907540498637217751820571720600128173828125f,0.186399987755715479398332945493166334927082061767578125f,0.74170002447366822906360539491288363933563232421875f,0.8010000275410715797619332079193554818630218505859375f,1.6817999800637355445331877490389160811901092529296875f,1.08429998077154099433982992195524275302886962890625f,-1.136399976263195554793128394521772861480712890625f,-0.431900029996782397478938264612224884331226348876953125f,1.1187999738276008798720795311965048313140869140625f,-1.3549000094108281100346857783733867108821868896484375f,1.387999991595744386785327151301316916942596435546875f,-0.7426999957256017470541564762243069708347320556640625f,0.9958000152938064619689839673810638487339019775390625f,0.5849000080510970978053819635533727705478668212890625f,1.4520999915242189803876726728049106895923614501953125f,-0.4845000721812251498477053246460855007171630859375f,1.0233999655567107556208839014288969337940216064453125f,-1.72020002260357074419516720809042453765869140625f,-0.71000001752749064554137703453307040035724639892578125f,0.23629997989982332029512690496630966663360595703125f,-0.644700060772895877647670204169116914272308349609375f,-0.2207000353217125621085870079696178436279296875f,2.08769999097585667868770542554557323455810546875f,-0.3332000112935897817578734247945249080657958984375f,-0.65629998369961928261773209669627249240875244140625f,-0.51220000377595409357667222138843499124050140380859375f,-0.26399998074770092415519684436731040477752685546875f,0.0553000033020965009455949257244355976581573486328125f,0.87329997850656582158990204334259033203125f,-0.068800040510296256446309826060314662754535675048828125f,-0.00539997500404698627107791253365576267242431640625f,0.24569998024329520180941699436516501009464263916015625f,1.01310002880692451299182721413671970367431640625f,-0.32779999555349359496858596685342490673065185546875f,-0.758800033581256361259192999568767845630645751953125f,-1.1591999954164029507097666282788850367069244384765625f,0.39569999654144094591856628539972007274627685546875f,0.5470000121295452988334773181122727692127227783203125f,1.058400012029707237815046028117649257183074951171875f,0.89959999109581101350840981467626988887786865234375f,0.61570002392232392463000678617390803992748260498046875f,1.04710000201091180116463874583132565021514892578125f,0.1051000183239578777971701128990389406681060791015625f,0.120900001375377375012476477422751486301422119140625f,-0.22569997594058477385914329715888015925884246826171875f,0.0575000177882614149638129674713127315044403076171875f,0.3321999820575121109555993825779296457767486572265625f,-0.488699958744645446273580091656185686588287353515625f,-0.290900053678452508165719336830079555511474609375f,-0.150799988378583993409165486809797585010528564453125f,-1.296500015914440862019318956299684941768646240234375f,1.50390001223087210746598429977893829345703125f,-1.8676000245451931380813448413391597568988800048828125f,1.13379997263550791330999345518648624420166015625f,-0.087599946388602578650761643075384199619293212890625f,-0.301000031545758162110359990037977695465087890625f,0.5386999983072282560669918893836438655853271484375f,-1.287899998641012899724955786950886249542236328125f,1.847899986869097066488620839663781225681304931640625f,0.45369999188110232690718248704797588288784027099609375f,0.83770000446140746586110026328242383897304534912109375f,0.0263000110805038378458675651927478611469268798828125f,0.6461000335529447635707356312195770442485809326171875f,0.502399988046288381582371584954671561717987060546875f,-1.914400007253885149793859454803168773651123046875f,1.265599978590011165380246893619187176227569580078125f,0.35999998271465283306014271147432737052440643310546875f,-0.1636999704420565304729962008423171937465667724609375f,-1.013500006474554737678772653453052043914794921875f,-0.43340002547800515753095851323450915515422821044921875f,0.886400014466047547756488711456768214702606201171875f,0.2056000061385334642949374028830789029598236083984375f,0.22450000989437146614591256366111338138580322265625f,-1.6827000420868400976104339861194603145122528076171875f,-0.54140005007982328066873378702439367771148681640625f,-0.78039997830092833197568324976600706577301025390625f,1.0315000028014171817858368740417063236236572265625f,0.13069999661594611239934238255955278873443603515625f,-0.1943999731302259448284530662931501865386962890625f,0.08819999315142634799258303246460855007171630859375f,-0.59660001125335693217266452847979962825775146484375f,1.8510000253990295249195696669630706310272216796875f,-0.850900035983324531940752422087825834751129150390625f,0.9806000004574659811851233826018869876861572265625f,-0.006600010210275453204076256952248513698577880859375f,-1.04070000954270280857372199534438550472259521484375f,-1.01700005593895870248388746404089033603668212890625f,0.951899997568130373792882892303168773651123046875f,-0.14050000977516230449282375047914683818817138671875f,0.5466000315189347702471422962844371795654296875f,-0.738299988937378248721188356284983456134796142578125f,-1.226199959564208796791717759333550930023193359375f,-0.2572000066816804153546627276227809488773345947265625f,0.26139996633529705150067456997931003570556640625f,0.5344000400364397052044296287931501865386962890625f,-0.106500006973743399640852658194489777088165283203125f,-0.61250000404193982728173750729183666408061981201171875f,1.13660000080391743182417485513724386692047119140625f,-0.07360002502203000229741292059770785272121429443359375f,0.57030001086443660174296610421151854097843170166015625f,-1.3807000046439468210479617482633329927921295166015625f,0.11689999460652489293721600915887393057346343994140625f,2.019900041317939543006332314689643681049346923828125f,-0.392200006085634100827519432641565799713134765625f,-1.1816999807760113494481402085511945188045501708984375f,1.262999973490833927058929475606419146060943603515625f,0.557899990147352031755190182593651115894317626953125f,-0.713000011600553929014267851016484200954437255859375f,0.32700000680237983896603282119031064212322235107421875f,-0.0383999801039693888782267094939015805721282958984375f,-0.15859998411834175247037137523875571787357330322265625f,0.187300019138306073074318192084319889545440673828125f,0.311100031746179339631197535709361545741558074951171875f,0.519299995225667831988403122522868216037750244140625f,-0.642800043684244482022904776385985314846038818359375f,-0.6957999918058506150231323772459290921688079833984375f,0.92619999711513401052798144519329071044921875f,-0.7137000414088376931687207616050727665424346923828125f,0.5147000136852266649611920001916587352752685546875f,0.461899950182437901702314775320701301097869873046875f,-0.47400000308081491340317370486445724964141845703125f,0.37930000569373312213627968958462588489055633544921875f,-1.305199977136403344246673441375605762004852294921875f,0.66289997980967185764455962271313183009624481201171875f,0.38620002209320691832772354246117174625396728515625f,-0.2912000357463948585490243203821592032909393310546875f,0.4701999479115013258478938951157033443450927734375f,0.2901000039815908593254789593629539012908935546875f,0.08170000584721481828864853014238178730010986328125f,0.644900012111662590541527606546878814697265625f,-1.31520003895759618472993679461069405078887939453125f,0.831499979034065805905129309394396841526031494140625f,-0.529599987426399909651308917091228067874908447265625f,0.06249998614191998314026932348497211933135986328125f,-0.8202000139236449438584486415493302047252655029296875f,-0.35940003571510370505848186439834535121917724609375f,0.08040002803355472682511617676937021315097808837890625f,-0.51530000501573081095330053358338773250579833984375f,0.72929999299049363958857838952098973095417022705078125f,0.188500003308057717532619790290482342243194580078125f,-0.332300009359419334931118328313459642231464385986328125f,0.1180000153630971215790168571402318775653839111328125f,-0.2597000050425528616671044801478274166584014892578125f,-0.8358999708279968299251549979089759290218353271484375f,-1.0751000412344939860531667363829910755157470703125f,0.237800034035742680060820930521003901958465576171875f,-0.62760004063844743171785012236796319484710693359375f,1.628000044830143711038772380561567842960357666015625f,-0.03000001072883673458591147209517657756805419921875f,-0.134200025479495221869541410342208109796047210693359375f,0.1512999844074247590697268606163561344146728515625f,-0.0385999981626869992368256134795956313610076904296875f,0.157000006765127153585126507095992565155029296875f,-0.64100000602751949774216200239607132971286773681640625f,-0.27370000247955328376292527536861598491668701171875f,1.324699985074996533285229816101491451263427734375f,-2.6449000168800349541697869426570832729339599609375f,0.34809999617338149846546002663671970367431640625f,-0.074399995929002926686735008843243122100830078125f,1.77929998553991186582834416185505688190460205078125f,1.111299980235099571501677928608842194080352783203125f,-0.81999997869133967043353550252504646778106689453125f,0.048299985510110499120628446689806878566741943359375f,0.4939000195324414477227037423290312290191650390625f,-0.1922000157713890100552589501603506505489349365234375f,-0.386599985027313408636473468504846096038818359375f,-0.45789998977482326836252468638122081756591796875f,-0.70509998870790013825171627104282379150390625f,0.53919999847114030711026089193182997405529022216796875f,-0.032400010696053194170218603176181204617023468017578125f,-1.561099975401162520682873946498148143291473388671875f,0.8671999829329546383149818211677484214305877685546875f,1.245700004457682030789555938099510967731475830078125f,-0.6715999968178569812238265512860380113124847412109375f,0.27160000443607568154646969560417346656322479248046875f,-0.323600004458427292775013484060764312744140625f,0.54680002037137775960928820495610125362873077392578125f,0.1288000180095434377136598413926549255847930908203125f,-1.1800000219419601865666891171713359653949737548828125f,0.0789999915510417594077807734720408916473388671875f,0.172299998742342808100147522054612636566162109375f,-0.18340000089108932090908865575329400599002838134765625f,1.0348000233665095581869763918803073465824127197265625f,0.51330000598058111993537977468804456293582916259765625f,-0.096700016371161101602638154872693121433258056640625f,0.282400002686679751917608882649801671504974365234375f,0.230299992424249477807052244315855205059051513671875f,-0.948600038731098660349516649148426949977874755859375f,0.32890001521110523441393524990417063236236572265625f,-1.259100048133731331034823597292415797710418701171875f,1.171300060924888253310882646474055945873260498046875f,-0.4705000002384185364689983543939888477325439453125f,1.481199975955486802803307000431232154369354248046875f,0.19759994556903937024117112741805613040924072265625f,-0.30789996831715116254457598188309930264949798583984375f,-1.4366999317139395042630667376215569674968719482421875f,0.7003999767176802837553850622498430311679840087890625f,0.2127000437632202700655170701793394982814788818359375f,0.418599990376829822480431175790727138519287109375f,-0.258299996238946949489445614744909107685089111328125f,0.773700006353854607965558898285962641239166259765625f,-0.5113999853342772183850684086792171001434326171875f,0.0394999832957980689940313823171891272068023681640625f,-0.20970001001581539679108345808344893157482147216796875f,1.07850001731514932856725863530300557613372802734375f,0.6809000029213725380117239183164201676845550537109375f,0.48900000358372930353567653583013452589511871337890625f,0.5129000179380180934884947419050149619579315185546875f,-0.6731000088766216915558970868005417287349700927734375f,0.304600007095932934220172683126293122768402099609375f,0.8647000021740793851421358340303413569927215576171875f,-1.11080000883042817605428353999741375446319580078125f,-0.91070002056956311786706237398902885615825653076171875f,0.1792999865829945083106622405466623604297637939453125f,0.44039999398440110578434314447804354131221771240234375f,-0.089899994267522931323810553294606506824493408203125f,0.72659999288767551650636278282036073505878448486328125f,-1.962100012311339458648262734641321003437042236328125f,0.23479999203681956032596644945442676544189453125f,-2.234900031185150570678388248779810965061187744140625f,0.10849998736381483155355454073287546634674072265625f,-0.1704999722987408095065120505751110613346099853515625f,-1.147700037616491641756510944105684757232666015625f,-0.5815000206083060785289262639707885682582855224609375f,-0.251299998563527804407158328103832900524139404296875f,1.351200041595101541958001689636148512363433837890625f,-1.19120002185106255154778409632854163646697998046875f,0.99889996853321838532480114736245013773441314697265625f,-0.369400014313310265379186603240668773651123046875f,0.61600002973899259384182869325741194188594818115234375f,-1.484600031012296650345660964376293122768402099609375f,0.27140000150203746898114331997931003570556640625f,-1.89590003508925430963927283301018178462982177734375f,0.672600040340423532825298025272786617279052734375f,-0.145500004857778808542434489936567842960357666015625f,0.0405999897658819719481471111066639423370361328125f,-1.103299993482231844410534904454834759235382080078125f,1.043399978688359652778672170825302600860595703125f,-0.632399969643353987436285024159587919712066650390625f,0.49969998786896441256288881049840711057186126708984375f,0.58950002492591757619067038831417448818683624267578125f,-0.15190000278353699147970701233134604990482330322265625f,0.3688999742701646322728947779978625476360321044921875f,-0.18650000734627258935915961046703159809112548828125f,-1.50809998494163099991283161216415464878082275390625f,0.57249999063089507256307797433692030608654022216796875f,1.0236000033035870249165100176469422876834869384765625f,-0.349899982439726198180096616852097213268280029296875f,0.10319996882528048587346347630955278873443603515625f,0.941500020608305732139342580921947956085205078125f,-0.856399992412328447954905641381628811359405517578125f,-0.2186999894782901687761977882473729550838470458984375f,-0.91890004102736788649252730465377680957317352294921875f,0.337000021424145035364716704862075857818126678466796875f,-1.29670003311634030040977449971251189708709716796875f,-0.0178999947294595784796911175362765789031982421875f,-0.24309998726993808926266638081870041787624359130859375f,1.0336999948427074702550498841446824371814727783203125f,-0.32050001577287845488939410643070004880428314208984375f,-0.5558000198572872196933758459636010229587554931640625f,0.5565000218749052152134026982821524143218994140625f,-1.0138999849617482684749347754404880106449127197265625f,-0.37749997258186329673890213598497211933135986328125f,-0.148199994529784273566974661662243306636810302734375f,-1.2813000256277622401768212512251920998096466064453125f,0.844900018835812449680133795482106506824493408203125f,-1.12500000998377824856788720353506505489349365234375f,1.76369994257688489369684248231351375579833984375f,-1.37819996061101601725340515258722007274627685546875f,0.937799961914122182093933588475920259952545166015625f,1.13849995027855044327225186862051486968994140625f,0.74899995971843658093547446696902625262737274169921875f,-0.89839999738931641815042894450016319751739501953125f,0.56900000771880154815107744070701301097869873046875f,-1.79469996764063832728197667165659368038177490234375f,0.341300008994340320356286611058749258518218994140625f,0.97930000845044862245458716643042862415313720703125f,-0.84140000571161499554051488303230144083499908447265625f,0.23610000028610234057424577258643694221973419189453125f,-0.509200009386241436715181407635100185871124267578125f,-0.3866999998494982104801920286263339221477508544921875f,-0.4185000036433337289309974948992021381855010986328125f,0.72999999146908500780028816734557040035724639892578125f,-0.5732999954566360134577962526236660778522491455078125f,0.3414999994859098109145634225569665431976318359375f,-0.2815000127479436731192663501133210957050323486328125f,-0.5280999960243699664630412371479906141757965087890625f,0.424400017908215687612027977593243122100830078125f,-1.2983000151634218610752213862724602222442626953125f,0.7470000254660842387011143728159368038177490234375f,0.0156999536633488645520628779195249080657958984375f,0.566299966824054035186009059543721377849578857421875f,-1.03239996144771595254496787674725055694580078125f,-0.5198999697178605661207484445185400545597076416015625f,0.632299981214850870259169823839329183101654052734375f,1.120300027535110931609096951433457434177398681640625f,-1.01660000082254331488229581736959517002105712890625f,-0.143699994581937218196543426529387943446636199951171875f,0.712300011762231566336822652374394237995147705078125f,0.8180000078380109673759079669252969324588775634765625f,-0.322599993029236475905463521485216915607452392578125f,0.773900037041306720908551142201758921146392822265625f,0.0412000069499018906071796664036810398101806640625f,0.83720004805102998801658031879924237728118896484375f,-0.4612000227451318323801388032734394073486328125f,-0.058699971503763936908626419608481228351593017578125f,1.58099996829032907186274314881302416324615478515625f,1.4325000426359475458326642183237709105014801025390625f,-0.7924000158369552959669590563862584531307220458984375f,0.19419995058253369801803955851937644183635711669921875f,-0.9375000112876297198027941703912802040576934814453125f,0.245800000709295307643742489744909107685089111328125f,0.599800039646030125339848382282070815563201904296875f,0.06040001041293141526011822861619293689727783203125f,0.997400071567298152075409234385006129741668701171875f,0.3352999916791912227154170977883040904998779296875f,0.00519999395609005432561389170587062835693359375f,0.56279998780488948995071041281335055828094482421875f,1.53889999785125297648846753872931003570556640625f,-2.3140000454187390488414166611619293689727783203125f,-0.521800017279386185720113644492812454700469970703125f,0.735300002259016149963599673355929553508758544921875f,-1.16540006180331179308495848090387880802154541015625f,0.6370999964416028671365666014025919139385223388671875f,-0.57110003180205826112114664283581078052520751953125f,0.25670000011026861130858378601260483264923095703125f,-0.88050000143051132539540049037896096706390380859375f,-0.49099999839067454132646162179298698902130126953125f,-0.17529999674558638389498810283839702606201171875f,0.22380001458227649635546185891143977642059326171875f,0.459199963974953107026522047817707061767578125f,-0.318700033101439128557785807061009109020233154296875f,-0.2308999511957157135810803083586506545543670654296875f,-0.1696000077664845395730708332848735153675079345703125f,0.395200006845593365678581676547764800488948822021484375f,0.417299980786442592517460070666857063770294189453125f,-0.280099940279125991793307548505254089832305908203125f,0.91900001138821263424460994428955018520355224609375f,-0.4090999989598989028394271372235380113124847412109375f,-0.954200012254715179693675963790155947208404541015625f,-0.7054999957680703204232486314140260219573974609375f,0.07939999900236693886057537383749149739742279052734375f,-1.774199966023862540254185660160146653652191162109375f,0.26129998206049165343500817471067421138286590576171875f,-0.15800002652034106365164234375697560608386993408203125f,0.2692000032022594524505620938725769519805908203125f,-1.329999988898635354672705943812616169452667236328125f,0.91599999064207082710709073580801486968994140625f,0.29649999482929700889144442044198513031005859375f,-0.2770000151097777862929660841473378241062164306640625f,-1.79230003063082765635272153303958475589752197265625f,-0.21679997846186171983617896330542862415313720703125f,0.2604000439405431688300041059846989810466766357421875f,0.2859000346422195804052535095252096652984619140625f,-0.726099958171696702180497595691122114658355712890625f,-0.2576999820165337684585438182693906128406524658203125f,0.1708000606678414801109511245158500969409942626953125f,0.3564999947175386818543074696208350360393524169921875f,0.013600018391013879437423383933492004871368408203125f,-0.2457999832749369062412370112724602222442626953125f,0.129699973005056090613607011619023978710174560546875f,1.227799981173872456707840683520771563053131103515625f,1.186299996998906092215975149883888661861419677734375f,0.5243000166833400310650858955341391265392303466796875f,-1.0503000021100046268429650808684527873992919921875f,-0.483999985218048323076800443232059478759765625f,-0.442899944251776089831906801919103600084781646728515625f,-0.61369996861666464837981038726866245269775390625f,-0.5892999642685066152836270703119225800037384033203125f,-1.0667999504476792793639106093905866146087646484375f,-0.8057999881178137258075366844423115253448486328125f,-0.248399986623227153614834605832584202289581298828125f,1.138300019112228778084272562409751117229461669921875f,0.7119000190816822115635886802920140326023101806640625f,-0.83959999718666100676500718691386282444000244140625f,0.248500002227723282288707196130417287349700927734375f,-0.09740003342032377275927501614205539226531982421875f,0.2381999997079373709851779494783841073513031005859375f,0.9686000217437733539327382459305226802825927734375f,-0.693800053008646333552178475656546652317047119140625f,0.2044999777078633229621118516661226749420166015625f,0.11020003184229165693608365472755394876003265380859375f,-0.1521999752402312111598803312517702579498291015625f,2.49040001093447216362619656138122081756591796875f,-0.171500009730456592915714963965001516044139862060546875f,-1.31550002214312566906073698191903531551361083984375f,0.968900013720988884102780502871610224246978759765625f,1.419200025516747842857512296177446842193603515625f,0.726299990218877145053966160048730671405792236328125f,-0.5500000512599942936731167719699442386627197265625f,-1.48280008406639129958648481988348066806793212890625f,-2.1580000115633009016846699523739516735076904296875f,1.280200083288550505500325016328133642673492431640625f,0.2765000135302546624416208942420780658721923828125f,0.474100052306056340967188589274883270263671875f,-0.110999962776899663285945507595897652208805084228515625f,1.509099938088656500667639193125069141387939453125f,0.98199993941187901924649850116111338138580322265625f,-1.794399974620341620124008841230534017086029052734375f,-0.21739998677968952467409735618275590240955352783203125f,-0.3320999712586409913228635559789836406707763671875f,-0.61829999102354094020483898930251598358154296875f,2.491600018405915051289412076584994792938232421875f,0.125499963060021979543989800731651484966278076171875f,-1.3651000311017043031824869103729724884033203125f,-1.0594999949187045018561548204161226749420166015625f,0.0427000160098085501658715656958520412445068359375f,-1.1810999892592430882132248370908200740814208984375f,-1.716200068640709286427181723411194980144500732421875f,-1.69040006837844902776168964919634163379669189453125f,1.21779999207034705932528595440089702606201171875f,-0.677000013973563685709677884005941450595855712890625f,0.1751999769128851358601650645141489803791046142578125f,0.13989995183646597443072323585511185228824615478515625f,1.574700051236153086620106478221714496612548828125f,1.2538000540144746963733268785290420055389404296875f,-1.2904000389859078268273151479661464691162109375f,0.60100001692399385166964975724113173782825469970703125f,-1.947800020028651157844024055521003901958465576171875f,-0.043299977612495155199212604202330112457275390625f,-1.85930007502138661124035934335552155971527099609375f,2.00560008736848960353427173686213791370391845703125f,0.848199975884706081075137262814678251743316650390625f,0.5284999932125213906175531519693322479724884033203125f,0.8226999272547654840082032023929059505462646484375f,-0.4299999815039330375299186925985850393772125244140625f,0.9661000762075193648570348159410059452056884765625f,-2.7140000475049017580886356881819665431976318359375f,-0.670799978469311941609021232579834759235382080078125f,0.29299999725818637319463277890463359653949737548828125f,-0.94149999106675330562410408674622885882854461669921875f,-0.63079994341433032190025187446735799312591552734375f,-0.225300006282329601248193284845910966396331787109375f,0.662700041192770594733474354143254458904266357421875f,-0.416100008444488356218471380998380482196807861328125f,-0.327700019921362439845324843190610408782958984375f,-0.24159999342784221898483565382775850594043731689453125f,-0.03630001584887432297676923553808592259883880615234375f,0.96569996647834699388113222084939479827880859375f,-0.678800036188960564942362907459028065204620361328125f,-0.2851999664634463638179795452742837369441986083984375f,-0.159099962675571138248642455437220633029937744140625f,-0.4437999919697637185578287244425155222415924072265625f,-1.7145999830491831517065293155610561370849609375f,1.1623999630123373894008409479283727705478668212890625f,-0.575000006109476213822517820517532527446746826171875f,-0.1945000343322760283371053446899168193340301513671875f,-0.9878000224314629207356119877658784389495849609375f,-0.674199971425533561841803020797669887542724609375f,-0.3806000013142825100231902979430742561817169189453125f,-1.192700012657045949282519359258003532886505126953125f,-1.9588000130362808182127309919451363384723663330078125f,0.5931999925181272725893677488784305751323699951171875f,0.214900025913864478610548758297227323055267333984375f,-1.6290000048875814542270745732821524143218994140625f,1.7813999860048290724279240748728625476360321044921875f,-0.8766000827789299787440313593833707273006439208984375f,-1.46119999391138488675778717151843011379241943359375f,-0.6058000069677820231817122476059012115001678466796875f,1.400500050231814785917094923206605017185211181640625f,-1.1421000165581702834316502048750407993793487548828125f,-0.01619997386932414684679315541870892047882080078125f,-1.7507000201597815447485118056647479534149169921875f,1.1817000279754397151776856844662688672542572021484375f,0.8775000305101274822305867928662337362766265869140625f,-0.89970000094473334684153087437152862548828125f,-1.443499985575675825799635276780463755130767822265625f,-0.362199952217937326537366971024312078952789306640625f,1.821799958345294623285326451878063380718231201171875f,0.5807999744832523791870926288538612425327301025390625f,-0.33670002983808444696478545665740966796875f,0.146799990010262060291523766863974742591381072998046875f,0.59769997152984188915070262737572193145751953125f,0.98949998851120479770315796486102044582366943359375f,0.01160002496242473757348534491029568016529083251953125f,-0.03709996157288586449141121192951686680316925048828125f,-0.39790001126974861023199991905130445957183837890625f,-0.8332000018313527878177637830958701670169830322265625f,0.3806000146508221604335631127469241619110107421875f,-0.50590001614615343061842622773838229477405548095703125f,1.358300073873996982598555405274964869022369384765625f,0.411100023476034692748726229183375835418701171875f,-0.1530000088810918867920918273739516735076904296875f,0.7436000278718772893427058079396374523639678955078125f,0.1231999855518335351689529488794505596160888671875f,-0.26090002240464127414298900475841946899890899658203125f,0.54800003294646781881738206720910966396331787109375f,0.458000022124499306386269381619058549404144287109375f,-0.6439000181540845968441999502829276025295257568359375f,-0.04559999652728430330483888610615395009517669677734375f,-1.033000005900860163166044003446586430072784423828125f,0.1321999716639510324966977350413799285888671875f,1.54550001029670180940911450306884944438934326171875f,-0.87650007447600497556550180888734757900238037109375f,0.52960004338026112069570672247209586203098297119140625f,-1.033900031006336206473861238919198513031005859375f,-0.0242000121057035499916310072876513004302978515625f,-0.05980001245141064458721302798949182033538818359375f,-0.24509999448210029182604330344474874436855316162109375f,-1.576100025580823515980455340468324720859527587890625f,-0.07269997596293686914492582218372263014316558837890625f,0.60830000197589395583008808898739516735076904296875f,0.310400030902028500889855422428809106349945068359375f,0.57799999196827389624786519561894237995147705078125f,-0.2758999750375750181774492375552654266357421875f,-0.35300003838539206757474175901734270155429840087890625f,-0.0895000058710575530085407081060111522674560546875f,-0.5196000128328801537236358853988349437713623046875f,0.066100045585633093736532828188501298427581787109375f,-0.71420000294148888730916269196313805878162384033203125f,1.1451999793529505922151656704954802989959716796875f,0.35249999113380869886213986319489777088165283203125f,-0.70499997861683372235575006925500929355621337890625f,-0.4406999831676483569964375419658608734607696533203125f,-0.6464999922215934891056576816481538116931915283203125f,-0.15959999433681326674872025250806473195552825927734375f,0.50679998762607514350975179695524275302886962890625f,-0.72470001903101810558638362635974772274494171142578125f,0.6029999597072599470948262023739516735076904296875f,-1.3640000125803053432349543072632513940334320068359375f,1.7860000367984181224301210022531449794769287109375f,1.1422000624679033986552667556679807603359222412109375f,-0.31390001951381540923335933257476426661014556884765625f,0.933200017775595025426582651562057435512542724609375f,0.301600008943677000416982991737313568592071533203125f,0.963399999177455779175716088502667844295501708984375f,-0.02510001605153089343502870178781449794769287109375f,0.20470000459551851434980562771670520305633544921875f,0.92269995580911601251727915951050817966461181640625f,0.199100007081032259748099022544920444488525390625f,-0.360300027292967062209072537370957434177398681640625f,-1.159799954485894346589702763594686985015869140625f,0.02690000585690188739107497895020060241222381591796875f,-0.35029998188167843498064257801161147654056549072265625f,1.654700049727410249289505372871644794940948486328125f,0.24819997677877492225917421819758601486682891845703125f,0.26590003190413125100377556009334512054920196533203125f,-1.196099974991380765487747339648194611072540283203125f,1.682300009005516461257911942084319889545440673828125f,-0.5976000198513269356936916665290482342243194580078125f,-0.15600000155717130834176487041986547410488128662109375f,-0.4390000190436842419927643277333118021488189697265625f,-0.0175000175461175189184359624050557613372802734375f,-0.384000018969178569250289001502096652984619140625f,1.330300013397633929201901992200873792171478271484375f,-0.5427000417143112986906317019020207226276397705078125f,-2.25590002499371689026474996353499591350555419921875f,0.770499987944960107455472098081372678279876708984375f,0.4240000117421149905538868551957421004772186279296875f,-0.5336999942094085014332449645735323429107666015625f,0.90410002878084794186719364006421528756618499755859375f,0.9125000289455058588572455846588127315044403076171875f,-0.253200015801191558040272866492159664630889892578125f,0.2925000150129200005011398388887755572795867919921875f,-0.74930002577304843924821398104541003704071044921875f,1.13450006368756373120731950621120631694793701171875f,-1.0919000741600992654412038973532617092132568359375f,-0.05909994516670658004642291416530497372150421142578125f,0.20010003736019132958290356327779591083526611328125f,0.7546000147327773799332817361573688685894012451171875f,-0.434499990075825959223720929003320634365081787109375f,1.37210002207160020049059312441386282444000244140625f,0.19259999314099551792622833090717904269695281982421875f,0.863199967670440937439479966997168958187103271484375f,0.218700008961558722830886836163699626922607421875f,-0.416300005343556822623440893949009478092193603515625f,-0.6334999964535228400563937611877918243408203125f,0.019999992549418976750530418939888477325439453125f,0.146199994283915390980155279976315796375274658203125f,1.8727999526381484685089162667281925678253173828125f,-1.28029992000460612899814805132336914539337158203125f,-0.55120001738071522368045407347381114959716796875f,0.567800024089217370004689655615948140621185302734375f,0.466899971491099297082882912945933640003204345703125f,1.2904999938495453903897214331664144992828369140625f,-0.4277000096768137638036932912655174732208251953125f,-0.0229000057935717649115758831612765789031982421875f,-0.6150000052899120550620182257262058556079864501953125f,-0.335899991391598984336042121867649257183074951171875f,0.3846000003159044666745103313587605953216552734375f,-0.5171999989330766656081550536327995359897613525390625f,-0.016599974596500199908177819452248513698577880859375f,-0.177899988415092469296041599591262638568878173828125f,-0.29059999695569238564729630525107495486736297607421875f,-0.27719999481663093465755309807718731462955474853515625f,-0.03050000858306856077462043685954995453357696533203125f,1.73020002569556208271706054802052676677703857421875f,1.4939000095486638652886313138878904283046722412109375f,-0.36990002447962755383059629821218550205230712890625f,-0.1026000101655719642934627700014971196651458740234375f,1.2578000166386360536563415735145099461078643798828125f,-0.5190000166967509986903905883082188665866851806640625f,0.756300050270557022713546757586300373077392578125f,-0.2400000520050529217996881925500929355621337890625f,0.124200026522576667531438943115063011646270751953125f,1.0686999734967950193009755821549333631992340087890625f,-0.19489997906833911311963447587913833558559417724609375f,1.584899974914639297907115178531967103481292724609375f,-1.200400058394670121941771867568604648113250732421875f,1.2595999831795694490210735239088535308837890625f,0.67939996135830871271110709130880422890186309814453125f,0.671399974381923669852767488919198513031005859375f,1.26239999311268302761845916393212974071502685546875f,-0.552799949210882601846606121398508548736572265625f,0.46960002132654121709265382378362119197845458984375f,-2.661199931848049260452171438373625278472900390625f,1.659299989861249269296195052447728812694549560546875f,-1.005099983157216936291433739825151860713958740234375f,-0.3781999487645932855883756928960792720317840576171875f,-0.562099992399662795605763676576316356658935546875f,0.758399943968653733605833622277714312076568603515625f,-1.104500002965331351134636861388571560382843017578125f,-1.38470001592040059534838292165659368038177490234375f,0.324299988371133718345618035527877509593963623046875f,1.036100012356043098549207570613361895084381103515625f,-0.7806999927043920539659893620410002768039703369140625f,0.56370000989288104875640783575363457202911376953125f,-0.01170001218020910727091177250258624553680419921875f,-1.2720999999433757476907658201525919139385223388671875f,1.258700014102459352471896636416204273700714111328125f,-0.1704000035956501069023261152324266731739044189453125f,0.183800011155009546115479679428972303867340087890625f,-0.9803000000983475015203794100671075284481048583984375f,-0.58860000535249756836719825514592230319976806640625f,0.413099992746114974551119303214363753795623779296875f,0.607200041326880768366436313954181969165802001953125f,-0.6276000209689147624203542363829910755157470703125f,-0.3999999807029954723702758201397955417633056640625f,-1.05660002168416955470320317544974386692047119140625f,0.05749997306615017844677595348912291228771209716796875f,0.98550003570318267520633526146411895751953125f,0.0006000015005473979812222751206718385219573974609375f,0.0082000165462492446977194049395620822906494140625f,-0.3260000382140286756538216650369577109813690185546875f,-0.4935000150799755402886148658581078052520751953125f,-0.2561000105679038796324675786308944225311279296875f,-0.17190001284181999352540515246801078319549560546875f,0.7925999706774948805332314805127680301666259765625f,-0.01509999593496313963214561226777732372283935546875f,-0.9097999416708955067178976605646312236785888671875f,-0.412599974775314848329799133352935314178466796875f,1.2980999301612374807746164151467382907867431640625f,0.19809996100664140072922236868180334568023681640625f,1.125099983716012275181128643453121185302734375f,-0.97499998345971139457333265454508364200592041015625f,-1.739399983859062359670133446343243122100830078125f,0.13219999788999547973844528314657509326934814453125f,0.85450000140070869747432880103588104248046875f,1.9424000617325309914207309702760539948940277099609375f,-0.570600004369020030736692206119187176227569580078125f,0.2633999659851189445447516845888458192348480224609375f,0.472900044517964579515734158121631480753421783447265625f,0.5687000468663876429076253771199844777584075927734375f,-0.018700003727525338348414152278564870357513427734375f,-2.62120005854517312826601482811383903026580810546875f,1.064500003561378083105637415428645908832550048828125f,-0.260200000773369966022841026642709039151668548583984375f,-0.56780003008693513688598386579542420804500579833984375f,0.19930001608729364104277692604227922856807708740234375f,0.1078000203639268850253074560896493494510650634765625f,0.47900000220537197748171820421703159809112548828125f,-1.6033000711545357663823097027488984167575836181640625f,0.1844000380620371970508131198585033416748046875f,-1.38890002025887415726401741267181932926177978515625f,-0.77750001689419168116046421346254646778106689453125f,-0.78960003172010206906605844778823666274547576904296875f,0.4289000500798234583044177270494401454925537109375f,1.3994000129163264301013214208069257438182830810546875f,-0.0208999874800444018774214782752096652984619140625f,1.7824000360280274346536089069559238851070404052734375f,0.27690002308636951777742751801270060241222381591796875f,-1.62919992880821240532895899377763271331787109375f,1.78750002786517114827802288345992565155029296875f,-0.29780001045465365194786500069312751293182373046875f,-1.5796000068724154186128316723625175654888153076171875f,-0.5319000054657447851269580496591515839099884033203125f,-0.4528999954558912133251169507275335490703582763671875f,1.0493999637424951121289495858945883810520172119140625f,-1.12690001619458168846676926477812230587005615234375f,-0.43110004439353843963544932194054126739501953125f,1.2692000821411613120659467313089407980442047119140625f,-0.540200002449751526256704892148263752460479736328125f,2.005200028713047810668967940728180110454559326171875f,-0.180899986138940038760125617045559920370578765869140625f,-1.282300009582936439755940227769315242767333984375f,1.03649999803304648793300657416693866252899169921875f,0.6238000103354455205817430396564304828643798828125f,-0.06359998158514479360547966280137188732624053955078125f,-0.1844999948069452333498929874622263014316558837890625f,-0.07329999415278454222288928576745092868804931640625f,1.2039000028431414701657331534079276025295257568359375f,0.1499000195249913236494876400684006512165069580078125f,-1.17940003001540905103183831670321524143218994140625f,0.67190001854151493176914300420321524143218994140625f,0.16580004206001763833455697749741375446319580078125f,-1.4638999991923571375451729181804694235324859619140625f,1.3076000408619645742902548590791411697864532470703125f,-0.83699999402463465258961150539107620716094970703125f,0.653100014352798741157357653719373047351837158203125f,-0.921000009566545596584319355315528810024261474609375f,0.97060000153779935061493233661167323589324951171875f,-0.830700008760392183404519528266973793506622314453125f,0.9913000078767535416801592873525805771350860595703125f,-1.7743000256612895260133200281416065990924835205078125f,0.6185000573247674271470941675943322479724884033203125f,-0.238900016254186997599617825471796095371246337890625f,0.2382999910235401586078296531923115253448486328125f,-0.15369999383687993343983180238865315914154052734375f,-0.0566999650925392728595397784374654293060302734375f,-0.10610000430941557425512655754573643207550048828125f,0.13200000505149400709115070640109479427337646484375f,-1.073399974739551065994191958452574908733367919921875f,1.433999969273805685787692709709517657756805419921875f,-0.806699951457977793012332767830230295658111572265625f,1.2913999758347867174990142302704043686389923095703125f,-1.0400999359577898228934600410866551101207733154296875f,-0.7417999698564410326895313119166530668735504150390625f,-1.8447999780669805947042050320305861532688140869140625f,1.8426999784588815600727684795856475830078125f,-0.187000027850270811313748708926141262054443359375f,-0.788600067378581126575909365783445537090301513671875f,0.5702000549763439618544680342893116176128387451171875f,-0.4841000019028782741514760346035473048686981201171875f,0.058799981650710098080026000388897955417633056640625f,0.40470001908689756664472270131227560341358184814453125f,0.172299996581673386941702119656838476657867431640625f,-1.3531000191211701721982763046980835497379302978515625f,0.5017999941825868059908088980591855943202972412109375f,0.0904999883919954672961694086552597582340240478515625f,-0.19809999088346963436180203643743880093097686767578125f,-0.29729999587386846116032756981439888477325439453125f,-0.049799992185830976865190677926875650882720947265625f,0.01970002650022539825158673920668661594390869140625f,-0.72059998872280051074312723358161747455596923828125f,-0.4901000114321707457065713242627680301666259765625f,0.85250000782310930702578843920491635799407958984375f,-0.5421000246047977810803786269389092922210693359375f,0.4975000309199100190227227358263917267322540283203125f,0.56110001719892022009616994182579219341278076171875f,0.35759999068230374774657320813275873661041259765625f,0.1531000253051522097536008004681207239627838134765625f,0.34069999970793707344540734993643127381801605224609375f,-0.248299985659122146586241797194816172122955322265625f,2.2921000067234018615636159665882587432861328125f,-1.54000005096197156717607867904007434844970703125f,-0.58029995174408011138211804791353642940521240234375f,-0.0070999877989290638424790813587605953216552734375f,0.34039995036125159089124281308613717555999755859375f,-0.477400008982419965519738980219699442386627197265625f,1.023200004327296852579820551909506320953369140625f,0.2027999895930288065670765718095935881137847900390625f,0.05219999409019937530729293939657509326934814453125f,0.3280000338777899759890033237752504646778106689453125f,-1.09889998388141396645778513629920780658721923828125f,0.802900022482872177675972125143744051456451416015625f,-0.713299987819790981546930197509936988353729248046875f,-0.825200013439357338285162768443115055561065673828125f,1.1751000119537113874912392930127680301666259765625f,0.65250001206994057145749366100062616169452667236328125f,0.7669000096380711450905209858319722115993499755859375f,-1.5068000386066737750212496393942274153232574462890625f,0.56459998763129093379831147103686816990375518798828125f,0.104800025564432630620359532258589752018451690673828125f,-0.59670004095807727129141539990087039768695831298828125f,-0.1257000048488379206901299767196178436279296875f,-0.395000005289912081707370816729962825775146484375f,1.163900019234418881097781195421703159809112548828125f,-0.6916999733999376065440856109489686787128448486328125f,-0.82779998186305137064522341461270116269588470458984375f,-0.79069999337494334401554851865512318909168243408203125f,0.987600007557869474794642883352935314178466796875f,0.397700011762977201357216472388245165348052978515625f,-1.1247000013172632204572209957404993474483489990234375f,-0.961499953590333689135150052607059478759765625f,0.18170003784745869523931105504743754863739013671875f,-0.79809997411966360214563565023127011954784393310546875f,0.368199993449449980431609219522215425968170166015625f,-0.357800015297531803781794224050827324390411376953125f,-0.070999969407916641017664005630649626255035400390625f,0.73770001831948761594759389481623657047748565673828125f,-1.65030002774000195131520740687847137451171875f,-0.1743000166088346514214890703442506492137908935546875f,-0.46470000683069212499276545713655650615692138671875f,-2.06440002319812787590080915833823382854461669921875f,0.786500008575618636541548767127096652984619140625f,-0.42999998250976190217187422604183666408061981201171875f,0.35749998036771979403880550307803787291049957275390625f,-0.99270003133937745332104896078817546367645263671875f,1.31620000471472753389434728887863457202911376953125f,0.8926000029012559622287881211377680301666259765625f,-0.602499972730875743565093216602690517902374267578125f,0.66110003027468966507740333327092230319976806640625f,-0.89470000258386139790900415391661226749420166015625f,0.64360008283853620270065221120603382587432861328125f,-1.6940000166594983976864341457257978618144989013671875f,1.1002000073671336366487594204954802989959716796875f,0.2542999839380382098852351191453635692596435546875f,-0.66490001121759434443703185024787671864032745361328125f,-0.47920003211051243052764903040952049195766448974609375f,0.807400021015107771660268554114736616611480712890625f,1.68569998860657133121776496409438550472259521484375f,-0.147300021243095269340983577421866357326507568359375f,0.13849997164308991415282434900291264057159423828125f,0.95030003736987733731211847043596208095550537109375f,-0.34740004787445055711714303470216691493988037109375f,0.7140000196397309206730596997658722102642059326171875f,0.4470000265836713282396885915659368038177490234375f,-0.18259998282194178642612314433790743350982666015625f,0.3119999791234728814259824503096751868724822998046875f,0.0200999873667953021794119194964878261089324951171875f,0.2061000120207669983329878959921188652515411376953125f,-0.1267999936610457734076362612540833652019500732421875f,-0.45320002101659806470479452400468289852142333984375f,0.206599987187981337655173774692229926586151123046875f,-0.14889998227953871179352063336409628391265869140625f,-0.81880002125054629491529567530960775911808013916015625f,0.900899977142363805882041560835205018520355224609375f,-0.6493000272445381693131594147416763007640838623046875f,0.5309000270426273981883014130289666354656219482421875f,-0.471899997214227706621159086353145539760589599609375f,-0.027800010454654522362716306815855205059051513671875f,-0.17629998379275202413651868482702411711215972900390625f,-0.2139000232204797402602025613305158913135528564453125f,0.274999993070959902752292691729962825775146484375f,-1.199199969637394502797178574837744235992431640625f,0.5880999837867915136513374818605370819568634033203125f,-0.2924000247776508132346862112171947956085205078125f,0.1073000076271595215615661800256930291652679443359375f,-0.87720004805102969047680971925728954374790191650390625f,-0.1819999974146477850212022531195543706417083740234375f,-0.2896000342160458540519130110624246299266815185546875f,0.008100016885995930948638488189317286014556884765625f,0.1856000047788024520656335880630649626255035400390625f,0.02170001840144397675658183288760483264923095703125f,-0.6665000003054739607932788203470408916473388671875f,-0.479100010284781330227588114212267100811004638671875f,0.8327000402241946375170300598256289958953857421875f,1.4334000134825704009244873304851353168487548828125f,0.782500039488077003824173516477458178997039794921875f,-1.587199986825883257779423729516565799713134765625f,1.41409999303668687531398973078466951847076416015625f,0.31490001173913484411315266697783954441547393798828125f,0.6157999481081961601347529722261242568492889404296875f,1.039699976581335061354138815659098327159881591796875f,-0.33180001899302002943414890978601761162281036376953125f,-0.0576999967873095542802985846719820983707904815673828125f,0.50339999925196199370702743181027472019195556640625f,0.0985999997645616854669015083345584571361541748046875f,-1.9479999474883076260312009253539144992828369140625f,-0.559299995300173602430504615767858922481536865234375f,-0.7285000126957894561741113648167811334133148193359375f,0.32440002172663817692210841414635069668292999267578125f,-0.147300004367530323889923238311894237995147705078125f,-0.1567999965667723483164763820241205394268035888671875f,0.036299971070885561630348092876374721527099609375f,-1.5783000512987381824814292485825717449188232421875f,0.41060001047849681299339863471686840057373046875f,-0.73199999538436488766279808260151185095310211181640625f,-1.21810006039738727423582531628198921680450439453125f,-1.262400043851137798611716789309866726398468017578125f,0.08179999045729591244935363647527992725372314453125f,-0.647400020977854762094239049474708735942840576171875f,0.040400027325749210405092526343651115894317626953125f,0.992400018034875319727916576084680855274200439453125f,-0.36049999580532288145917618749081157147884368896484375f,-0.555700017961859860093909446732141077518463134765625f,-1.7932001025259507454023832906386815011501312255859375f,-0.0403999914884565214379108510911464691162109375f,-0.5067000071883196454791686846874654293060302734375f,-0.666099991196393403924957965500652790069580078125f,-1.61630004803538351865199729218147695064544677734375f,0.34860003873109857153167467913590371608734130859375f,0.6181000287324192044025039649568498134613037109375f,0.81750000365078445696553899324499070644378662109375f,-0.114600015515089859174935327246203087270259857177734375f,-0.3667999600589275654982657215441577136516571044921875f,0.4072000159949065167808157639228738844394683837890625f,-0.3390000024288892799262384869507513940334320068359375f,1.41710003347098822956695585162378847599029541015625f,0.347100024083256908369321536156348884105682373046875f,0.609399955546855220944735265220515429973602294921875f,0.19489998052120183213986592818400822579860687255859375f,0.2157000150561341200727838440798223018646240234375f,-0.155699990022181822268976247869431972503662109375f,-1.0837999754667280161157805196125991642475128173828125f,-1.1412000117927789677452210526098497211933135986328125f,0.90200000676512726016653687111102044582366943359375f,-0.484299996156990264495334486127831041812896728515625f,0.755599987605214185037993956939317286014556884765625f,0.5938999930456281273194463210529647767543792724609375f,0.41480002899169843288973424932919442653656005859375f,1.8925999944820997011873942028614692389965057373046875f,0.9416000601291667493342174566350877285003662109375f,1.46600005440041503135262246360071003437042236328125f,-2.2499000308871277553635081858374178409576416015625f,-0.4471000351846223797025459134601987898349761962890625f,0.622599983716010996204204275272786617279052734375f,-0.8748000143513081638957373797893524169921875f,0.05030005122795699890048126690089702606201171875f,-0.358600035303831443656008559628389775753021240234375f,0.6526999922424554512190297828055918216705322265625f,0.87439996612668036135573856881819665431976318359375f,2.5833000190377237714756120112724602222442626953125f,0.6254000272512438840522008831612765789031982421875f,-0.216099992835521614864546791068278253078460693359375f,-1.583900079584122266851409221999347209930419921875f,-1.4805000242292880585637249168939888477325439453125f,1.30390008226633113963544019497931003570556640625f,0.0302000338166952264629117053118534386157989501953125f,2.509100002908706272819472360424697399139404296875f,1.5385999572031199278399071772582828998565673828125f,-1.6880999961733824665799374997732229530811309814453125f,-0.2547999655872583701921030296944081783294677734375f,-0.042599988707900049433874301030300557613372802734375f,-1.5937000436067574327125839772634208202362060546875f,0.533899964845180985406614126986823976039886474609375f,1.6757999898314466236115549691021442413330078125f,1.1688999980747691864735315903089940547943115234375f,-0.51770002048015495432764510042034089565277099609375f,0.8839000146895654719259027842781506478786468505859375f,-0.8218999910116195906084612943232059478759765625f,-0.250699989053607463773687413777224719524383544921875f,-1.04410000272244207764060774934478104114532470703125f,1.293700053292513363345506149926222860813140869140625f,-1.452800019395350972217784146778285503387451171875f,-0.20139999576509015621894604919361881911754608154296875f,-0.4670000295639036291817092205747030675411224365234375f,-0.97370000603720530119034037852543406188488006591796875f,0.6271999847769738067881917231716215610504150390625f,-0.0960999838761986691082483957870863378047943115234375f,-0.5498000016853212823519925223081372678279876708984375f,0.03229999157115803765094597110874019563198089599609375f,0.08399996448680757676896746488637290894985198974609375f,-1.13050002937018856385975595912896096706390380859375f,-0.04579997075796171657202648930251598358154296875f,-0.91559997903704637423061285517178475856781005859375f,-0.91599998885393230096951810992322862148284912109375f,-0.3005000166296962760270616854541003704071044921875f,-1.058399985766410367205025977455079555511474609375f,-0.324699974197148577559346449561417102813720703125f,-0.01839998889565475082008561003021895885467529296875f,-0.9181999935984610505812497649458236992359161376953125f,0.1723000078134233614690629110555164515972137451171875f,0.4396000052146613601422586725675500929355621337890625f,-1.488600047131627857055491404025815427303314208984375f,0.2243999931350353360670624169870279729366302490234375f,-1.28610010079443526365139405243098735809326171875f,0.136099974358081521330632313038222491741180419921875f,-0.1340999146938326447298095445148646831512451171875f,-0.9924999474734068627412852947600185871124267578125f,0.5098000006794924932052026633755303919315338134765625f,-0.6938999821305273929539225719054229557514190673828125f,0.491499987080693312435641928459517657756805419921875f,-0.4567999985039232768713191035203635692596435546875f,-0.6559999676197769691299299665843136608600616455078125f,-0.410699979926645808436802553842426277697086334228515625f,1.24969998734742393509122848627157509326934814453125f,0.53399999997019753639193595518008805811405181884765625f,1.0923000140905383403833184274844825267791748046875f,-0.4684999668002121087084788086940534412860870361328125f,-0.46089996967315727260938729159533977508544921875f,0.8130000121779736854676912116701714694499969482421875f,-0.68929998777508671992109157145023345947265625f,-0.387299971864372150065491950954310595989227294921875f,0.992400016805529805452579239499755203723907470703125f,0.7102000272415580628404541130294091999530792236328125f,-0.41339999381303726977421320043504238128662109375f,0.0762000229313974930533959195599891245365142822265625f,1.6291999979496001316192632657475769519805908203125f,0.58420004192665209163948247805819846689701080322265625f,-1.1061000131011002878267390769906342029571533203125f,2.0534999995827671881443166057579219341278076171875f,-0.058800020244717199346951019833795726299285888671875f,-0.017499964237213561091266456060111522674560546875f,-0.0215999523192647036040625607711263000965118408203125f,0.5864999854415646751704116468317806720733642578125f,1.4365000468343500639178955680108629167079925537109375f,0.40889997095465613341502830735407769680023193359375f,-0.646700025218724050546370563097298145294189453125f,0.2143999984622002585865629953332245349884033203125f,0.99220002545714436337220831774175167083740234375f,-0.6405999702453613053876324556767940521240234375f,-0.710100005695223135404603453935123980045318603515625f,-0.088999983057380216422416197019629180431365966796875f,0.23089993599653357847500956268049776554107666015625f,1.6885999725699429863112754901521839201450347900390625f,-0.7719999857544903587580620296648703515529632568359375f,-0.370899983978272229645511970375082455575466156005859375f,-0.792999973863363294412920367904007434844970703125f,-0.733899999862909435677238434436731040477752685546875f,0.710499967157840472964380751363933086395263671875f,1.2561000034153462223684982745908200740814208984375f,0.2383999303340917919058483676053583621978759765625f,0.151899979910254945281167238135822117328643798828125f,-0.57849998915195488535800905083306133747100830078125f,-1.25850000608339929186740846489556133747100830078125f,0.2751000150829547852282530584488995373249053955078125f,0.33660000812411272619328883592970669269561767578125f,-0.77300000172853433610953288734890520572662353515625f,-1.507600005210935734112354111857712268829345703125f,1.263599981808662509052965106093324720859527587890625f,-0.765399982864409356153601038386113941669464111328125f,0.5312000281095503595452100853435695171356201171875f,0.989500032190233635986942317686043679714202880859375f,-0.79650002008676512588181140017695724964141845703125f,-0.683700009240954642564247478730976581573486328125f,2.2122000306725499996218786691315472126007080078125f,-1.1660999704837802237733512811246328055858612060546875f,0.1893000728793443609987434683716855943202972412109375f,-1.5711000083327288923129572140169329941272735595703125f,2.521899971044064159286790527403354644775390625f,0.20209997751712904801024706102907657623291015625f,0.30430001564025854765560552550596185028553009033203125f,-2.202799925406278003237048324081115424633026123046875f,0.2089999984614550054828896463732235133647918701171875f,0.632399975249916312947107144282199442386627197265625f,-1.089899969792366363208202528767287731170654296875f,-0.21089998219013228464291387354023754596710205078125f,-0.01210002110302443867340116412378847599029541015625f,-0.7097000254757703796570922349928878247737884521484375f,0.59390006021261265711785881649120710790157318115234375f,-0.6448000272035596669439883044105954468250274658203125f,1.2637000022932884935045194652047939598560333251953125f,0.356000023201108017900651248055510222911834716796875f,-0.54530002479702233131320099346339702606201171875f,0.67410002119988188251653582483413629233837127685546875f,0.7964000187501312222337901403079740703105926513671875f,0.1803000060960651129704501727246679365634918212890625f,-0.459800006081163614712892240277142263948917388916015625f,-0.02409999001920215544458869771915487945079803466796875f,0.71840002201348529542457299612578935921192169189453125f,0.634900015315413401850719310459680855274200439453125f,0.338199987339972807376398122869431972503662109375f,-0.584900053983927303846712675294838845729827880859375f,0.68359998985528935833144714706577360630035400390625f,-1.414100010955333441842185493442229926586151123046875f,0.7213000236719846469668482313863933086395263671875f,-1.340300013956427616079736253595910966396331787109375f,0.01929998654574138772233027339098043739795684814453125f,-0.5689999974370001378787264911807142198085784912109375f,-0.44599999451637284408889172482304275035858154296875f,-0.10340002637207579283540326287038624286651611328125f,-0.744300022941827510436496595502831041812896728515625f,-0.15980000341758093806987517382367514073848724365234375f,1.881900015449523788646501998300664126873016357421875f,-0.819900004363059764500576420687139034271240234375f,0.52729994897246346141628237091936171054840087890625f,0.036800043356418399298490840010344982147216796875f,-0.5212000115692616564189165728748776018619537353515625f,-0.832299965363741289792187671991996467113494873046875f,-0.48890001614242795113796091754920780658721923828125f,0.606200015369057876313263477641157805919647216796875f,-1.5396999765813357274879535907530225813388824462890625f,0.0543000025272366571016391390003263950347900390625f,0.493999958097934399603445854154415428638458251953125f,0.6665000326409937780880454738507978618144989013671875f,0.2094000216893852073241077960119582712650299072265625f,0.38589997861385383348675759407342411577701568603515625f,-0.0352000129923226712236328239669091999530792236328125f,0.183600014293193947167992519098334014415740966796875f,-0.221100004868209187858241193680441938340663909912109375f,0.160200005877017714883692178773344494402408599853515625f,-1.043000002697109795946062149596400558948516845703125f,0.1332000017195940078096327852108515799045562744140625f,0.7284999789819115445510533390915952622890472412109375f,-0.911899969330429716052321964525617659091949462890625f,0.9415000644177202726581299430108629167079925537109375f,-0.533300006110966062777833940344862639904022216796875f,-0.06160001903921352717219406258664093911647796630859375f,0.158799992174654625021190668121562339365482330322265625f,0.678900003737211488186176211456768214702606201171875f,-1.5369000136241315335183799106744118034839630126953125f,0.501100016453861929477398007293231785297393798828125f,-0.7432000071957707287850780630833469331264495849609375f,0.7460000398904085283646736570517532527446746826171875f,0.89309998160749604068087137420661747455596923828125f,-0.665699973332881000942506943829357624053955078125f,0.67520001470595669701424412778578698635101318359375f,-1.420700074511767496687753009609878063201904296875f,-0.995299972195922588724670276860706508159637451171875f,-0.4673000051684679245767028987756930291652679443359375f,1.998599951205403346676803266745992004871368408203125f,0.071699959728122042434961258550174534320831298828125f,-0.173099990361928579574168907129205763339996337890625f,0.026200014027953244521995657123625278472900390625f,-1.227699963557720597151501351618207991123199462890625f,0.691700055579840711317274326574988663196563720703125f,-0.91399998667091164605835729162208735942840576171875f,0.24269999827742605447156165610067546367645263671875f,-1.20279999406337623213403276167809963226318359375f,-0.422900005793571676093733913148753345012664794921875f,-0.8565000296235087962060106292483396828174591064453125f,0.005700001905858453898900961576146073639392852783203125f,0.21749999068677416946826497223810292780399322509765625f,0.4740000219121578428627117318683303892612457275390625f,0.573900004631280769018530918401665985584259033203125f,-0.8684000404536729167404018880915828049182891845703125f,0.7775000366568571852354807560914196074008941650390625f,0.34740003029108101895161553329671733081340789794921875f,0.2147000107049941153292138551478274166584014892578125f,0.683899993008374895708811891381628811359405517578125f,0.19120003779530569953948315742309205234050750732421875f,-0.989099985884129839774914216832257807254791259765625f,0.6187000085890292933044065648573450744152069091796875f,-0.1440000128597020978560294679482467472553253173828125f,0.29590001597851534054228750392212532460689544677734375f,0.78480000902414293051378990639932453632354736328125f,-1.1008000239923598773117419113987125456333160400390625f,1.4383000705957424969483327004127204418182373046875f,-1.4352000492766505690411804607720114290714263916015625f,1.03460003324747074060496743186376988887786865234375f,-0.8422000343605879901787147900904528796672821044921875f,-0.20459998332858031488967753830365836620330810546875f,-0.531200013580918284361587211606092751026153564453125f,0.078199999111890239333888530381955206394195556640625f,-0.336700000259281040371206472627818584442138671875f,0.1857999895706774484693823978886939585208892822265625f,0.660899979396164294342952416627667844295501708984375f,0.613500002637505037483833802980370819568634033203125f,0.782400024554133377563402973464690148830413818359375f,0.203099988722801327867273357696831226348876953125f,-1.673700003373621569835449918173253536224365234375f,1.487100039952993224545707562356255948543548583984375f,-1.227699969071149865840197890065610408782958984375f,-0.937200020241737430382045204169116914272308349609375f,-0.7431999904692165870301323593594133853912353515625f,0.46909998420774901006780055467970669269561767578125f,0.791500037744641726789041058509610593318939208984375f,0.30049998556077472944281225863960571587085723876953125f,-0.4331000062316656684657800724380649626255035400390625f,0.5883000387445094236937848108937032520771026611328125f,-1.142300016847252575047377831651829183101654052734375f,-1.847000061154365990745418457663618028163909912109375f,0.03849997752904865677692214376293122768402099609375f,0.790800051745772947242585360072553157806396484375f,-0.872800008070468980037048822850920259952545166015625f,1.019099989721178989299232853227294981479644775390625f,-0.2615999686360357401326837134547531604766845703125f,0.886599974820017511234482299187220633029937744140625f,0.5322999923720954651429337900481186807155609130859375f,-0.934800010961293725841869672876782715320587158203125f,1.4228000403314826627365619060583412647247314453125f,-1.85080005897283594862301470129750669002532958984375f,0.4481000001594426240814073025831021368503570556640625f,0.2023000096201901243375687045045197010040283203125f,1.8342000442549586392004812296363525092601776123046875f,0.70660000469684547397264395840466022491455078125f,-0.605000037141144364483125173137523233890533447265625f,0.0229999948740005066838421043939888477325439453125f,-0.0158999964579942254516708999290131032466888427734375f,-1.164600028888880700606023310683667659759521484375f,-1.149199977833032182417127842199988663196563720703125f,-1.17789996354877946060923932236619293689727783203125f,1.294399972906708828901400920585729181766510009765625f,-1.22500001922249790453633977449499070644378662109375f,1.058699954795837783194656367413699626922607421875f,0.1727000121355057871141980285756289958953857421875f,1.099299969148635369720068410970270633697509765625f,2.5460000287890434123028171597979962825775146484375f,-1.516499965548515405089347041212022304534912109375f,-1.2605000026226047538102648104541003704071044921875f,-0.4597999410748485615840763784945011138916015625f,-0.28560001226663589424248357317992486059665679931640625f,-0.145199999618530029010798898525536060333251953125f,0.1504000020310283236568693610024638473987579345703125f,-0.020500017598271913499274887726642191410064697265625f,0.283600017459690378895942330927937291562557220458984375f,0.47849998527765313927062607035622932016849517822265625f,-0.9707000004902479606272436285507865250110626220703125f,0.68329999392926676904380656196735799312591552734375f,0.450300004624575311851231163018383085727691650390625f,-0.2588999982610344918754208265454508364200592041015625f,0.624000024911016470952063173172064125537872314453125f,0.23419999191463014565073308403952978551387786865234375f,-0.3542000296145679794079796920414082705974578857421875f,-0.255000027827918529510498046875f,-0.0955999922990802275535315857268869876861572265625f,1.0713000249758366511088070183177478611469268798828125f,0.274300001186132735853107078582979738712310791015625f,-0.1368000124737622780202173089492134749889373779296875f,0.591500017553567403183478745631873607635498046875f,0.1065000052601098590177031155690201558172702789306640625f,-0.3530999978497624258721998558030463755130767822265625f,0.22619999689161784939273047712049447000026702880859375f,0.31909997731596229098016692660166881978511810302734375f,-0.55890002718791442770651656246627680957317352294921875f,1.554600062751770650493199354968965053558349609375f,-1.7794999980479477841299740248359739780426025390625f,1.21440003497004500587763686780817806720733642578125f,-0.15569998495578829533769749104976654052734375f,0.4930999879032376131959836129681207239627838134765625f,0.252799961429835295945167672471143305301666259765625f,-0.41099998371303081423633329904987476766109466552734375f,-0.622599971571565102834711069590412080287933349609375f,0.12910000730454906925359637170913629233837127685546875f,-0.160799997244775116911341683589853346347808837890625f,0.4482999940223988399878862765035592019557952880859375f,1.337899972172826412730728407041169703006744384765625f,1.00359997346401197404475169605575501918792724609375f,-0.38289999335110191935882539837621152400970458984375f,-0.09409996487349214788764584227465093135833740234375f,-0.722700026813150042670486072893254458904266357421875f,0.42279998463839285616217011920525692403316497802734375f,-0.10499998532235654824518178429570980370044708251953125f,-0.106600032562017776882612452027387917041778564453125f,-0.091099996262788973666602032608352601528167724609375f,0.9343000123247502397560992903891019523143768310546875f,-0.611500045642257017419751718989573419094085693359375f,-1.46910000298321197220730027765966951847076416015625f,0.52369998817443796923498666728846728801727294921875f,2.015699997733533255228621783317066729068756103515625f,0.1045000378340483759842527433647774159908294677734375f,0.0466999493718134317532530985772609710693359375f,-0.39659998666644025178129595587961375713348388671875f,0.193699994581937318116615642793476581573486328125f,-0.56450000665336830252272193320095539093017578125f,-0.840500008434057033213093745871447026729583740234375f,1.1146000602558261416419327360927127301692962646484375f,-0.65400006926059717216048738919198513031005859375f,-0.1794999644085761847378535094321705400943756103515625f,0.6199000040650357590266139595769345760345458984375f,0.02320003155916960935201132087968289852142333984375f,-0.63640005633831009390632971189916133880615234375f,-0.247599979692697758082431391812860965728759765625f,1.1226999777138235714346592430956661701202392578125f,0.16390003130435815137388999573886394500732421875f,0.35629998366236659279593368410132825374603271484375f,-1.1194999462664139144862929242663085460662841796875f,-1.73209999979436357619988484657369554042816162109375f,0.3259000059574841401399680762551724910736083984375f,0.83499997302889905625988831161521375179290771484375f,2.177499931454658366192234097979962825775146484375f,-1.3526999844193454691776423715054988861083984375f,-1.032299938392640115125686861574649810791015625f,0.835099993103742832545322016812860965728759765625f,0.9752000063985584166204034772817976772785186767578125f,-1.4842999898612496689764839175040833652019500732421875f,0.906100011536479232887586476863361895084381103515625f,0.6956999805226917743539161165244877338409423828125f,0.61290001823604089992159060784615576267242431640625f,-0.5415000080540774973769657663069665431976318359375f,0.8986000002115963614102156498120166361331939697265625f,-0.67850003270059822124693482692237012088298797607421875f,-0.1918999843806032945536799161345697939395904541015625f,1.8032000078290699729421930896933190524578094482421875f,-0.732500011324882738250607872032560408115386962890625f,-0.64260002909004698690154100404470227658748626708984375f,1.055599983060359736697364496649242937564849853515625f,-0.501000032961368191308793029747903347015380859375f,-0.584400026103854219883260157075710594654083251953125f,1.3469000595569615708058108793920837342739105224609375f,-0.941599993073940044752134781447239220142364501953125f,-1.61040000743269917649058697861619293689727783203125f,-0.78139999524354930571234945091418921947479248046875f,-0.235799954366683550688321702182292938232421875f,-0.4633000099480149014397056816960684955120086669921875f,0.387699998724460481724918281543068587779998779296875f,-0.41849998408555955098364620425854809582233428955078125f,0.18219996898174262156544500612653791904449462890625f,0.05800000759586654286437124028452672064304351806640625f,0.62240004377663193935177332605235278606414794921875f,0.16440000334233051404453362920321524143218994140625f,-0.35519997700601779744289387963362969458103179931640625f,-0.457199985838681044469922198913991451263427734375f,0.2555999716982249037044994111056439578533172607421875f,-0.780699959437548773877324492787010967731475830078125f,1.19839999619722448187530972063541412353515625f,0.1288000319793820569458375757676549255847930908203125f,1.0379999741986394301562768305302597582340240478515625f,-0.259800020479411253315760177429183386266231536865234375f,0.46730000334307575382553068266133777797222137451171875f,-0.01439997849464447199352434836328029632568359375f,0.17309996067136435016209361492656171321868896484375f,0.476099971713125125205579024623148143291473388671875f,-0.36920000674128450324218420064426027238368988037109375f,0.1747000326097014610837732107029296457767486572265625f,0.5646000442370782135270701473928056657314300537109375f,-0.1712999743491412374396531959064304828643798828125f,0.73829998111426764495490715489722788333892822265625f,-1.7110000517368320060995756648480892181396484375f,-0.734500043291598903039130163961090147495269775390625f,-0.88359997189938910278073080917238257825374603271484375f,0.163399991559237012239691466675139963626861572265625f,0.347499960288406128938731853850185871124267578125f,0.30580001158714231745960887565161101520061492919921875f,0.9784000106513495165216909299488179385662078857421875f,0.16169994929730846511262143394560553133487701416015625f,0.79959999241828916272112337537691928446292877197265625f,-0.51120003045648321204907915671356022357940673828125f,-0.9876000024542210820044374486315064132213592529296875f,0.604900045788287599179966491647064685821533203125f,-0.9804999831020833056527408189140260219573974609375f,1.5290999817490575907186212134547531604766845703125f,0.1559000159412627095623093964604777283966541290283203125f,-0.076200034833699969993858758243732154369354248046875f,-0.755800003316998481039945545489899814128875732421875f,-0.21980000386461651107339321242761798202991485595703125f,-0.92540001492053303167040212429128587245941162109375f,-0.27239997828602735552294689114205539226531982421875f,-1.0712999856926497255216190751525573432445526123046875f,-1.011300000202655535730400515603832900524139404296875f,-0.8088999970689416674218819025554694235324859619140625f,1.3914999965429313277809342253021895885467529296875f,0.24830003542900069835042131671798415482044219970703125f,-0.91379999416768509234998418833129107952117919921875f,0.7821000146768983807277209052699618041515350341796875f,-0.5322000301882618966686777639552019536495208740234375f,0.93080000659525341522027019891538657248020172119140625f,-1.387100009685009904814023684593848884105682373046875f,-0.86799997315555810217801990802399814128875732421875f,-0.856299971294403494681546362699009478092193603515625f,0.125599989467858552671941652079112827777862548828125f,-0.07409998510181925457374063626048155128955841064453125f,0.1344000302016734593735236558131873607635498046875f,0.512000036418437165508521502488292753696441650390625f,-2.17449999797344162288936786353588104248046875f,-0.0242999568179260716505041273194365203380584716796875f,-1.380699974189698142623683452256955206394195556640625f,-1.162999962128699049657143405056558549404144287109375f,-0.116699988040328161975622833779198117554187774658203125f,1.15169998027309805621598570724017918109893798828125f,0.02149999373033650851283482552389614284038543701171875f,0.132000023305415492558267942513339221477508544921875f,-0.48789998366534781393255570947076193988323211669921875f,-0.75419997425675422419999449630267918109893798828125f,-0.852599977159500088674803919275291264057159423828125f,0.88120000158548261737223583622835576534271240234375f,-1.2865999736279249976433902702410705387592315673828125f,-0.04929998762607656903611541565624065697193145751953125f,-0.663799961087107703860965557396411895751953125f,0.37570003286302000589103045058436691761016845703125f,0.83800002363324133103361646135454066097736358642578125f,-0.9061999594897027776596587500534951686859130859375f,0.29980000971332232406751927555887959897518157958984375f,-0.5866999855071306502196648580138571560382843017578125f,-0.027600006216764672029029270561295561492443084716796875f,-0.531600019001960077957846806384623050689697265625f,0.42570003783628396210048094872036017477512359619140625f,-0.4913000502705582306361975497566163539886474609375f,0.54299996019154750381829899197327904403209686279296875f,0.798299988639353585995195317082107067108154296875f,1.1716999475836746835710755476611666381359100341796875f,-0.8063999517321587262586035649292171001434326171875f,-0.7480999738216400629653435316868126392364501953125f,-0.474599953079223979557355050928890705108642578125f,0.6382999403595912557562996880733408033847808837890625f,-0.904400018653273907176526336115784943103790283203125f,0.00180002525150779746354601229541003704071044921875f,0.5035000547170642715144595058518461883068084716796875f,0.0415999844312668898282936424948275089263916015625f,0.756799972948432042585409362800419330596923828125f,0.880399975991249306161989807151257991790771484375f,0.799099957758187873224642316927202045917510986328125f,-0.1994999914169302002164840814657509326934814453125f,-0.42590000834166996668272986426018178462982177734375f,-0.63040002002418038529185650986619293689727783203125f,-0.301499924868345647865908176754601299762725830078125f,-0.2993999792397030734747431779396720230579376220703125f,0.528199992182850852628916982212103903293609619140625f,0.7282000129699708335095920119783841073513031005859375f,-0.92600001031160417142018559388816356658935546875f,-0.43829998945891901485083508305251598358154296875f,-1.2211999223113050305755677982233464717864990234375f,-0.855199995893239250932538197957910597324371337890625f,0.206499992370606122449316899292171001434326171875f,0.41289997710883685755334226996637880802154541015625f,0.26409998592138261841455459943972527980804443359375f,-0.388500022605061889890976090100593864917755126953125f,-0.33329991919994395033199907629750669002532958984375f,1.948100004629790493737573342514224350452423095703125f,-0.920299996298550748718980685225687921047210693359375f,-0.439799992930888894449026338406838476657867431640625f,0.5110000080019236889938838430680334568023681640625f,-0.323499994032084625672496258630417287349700927734375f,-0.24610000103116025371718933456577360630035400390625f,1.116100025003403661827405812800861895084381103515625f,-1.5860000231266031534005378489382565021514892578125f,0.176899948375671467459824270918034017086029052734375f,0.41169998953044417699942414401448331773281097412109375f,2.21590003166198723505431189551018178462982177734375f,1.154800025936960405914533112081699073314666748046875f,-0.82889999434947991829858437995426356792449951171875f,-0.8831999629020697017978136500460095703601837158203125f,0.89039998843371837011773095582611858844757080078125f,-0.5271000108212227086568191225524060428142547607421875f,0.470299998682737196986636263318359851837158203125f,1.2469000104576350995699840495944954454898834228515625f,0.145299984526634062831362825818359851837158203125f,-0.40070002995729492312193542602472007274627685546875f,-0.0538999880909916129212433588691055774688720703125f,0.677600076587498989511004765518009662628173828125f,-0.363499983713030605514404669520445168018341064453125f,-0.43560002329349511995815191767178475856781005859375f,0.8425999763399361075499882645090110599994659423828125f,-0.1543999618053446543086693054647184908390045166015625f,-0.238599953645468543239616110440692864358425140380859375f,0.616899973726272587981611650320701301097869873046875f,0.7310999927610157822499559188145212829113006591796875f,-1.1529000145852563719017780385911464691162109375f,0.2953999883592128838927237666212022304534912109375f,-0.5402999419093124089386037667281925678253173828125f,-0.48899999535083704671478699310682713985443115234375f,-0.27200000303983717930833563514170236885547637939453125f,1.049800024875253701139854456414468586444854736328125f,-1.39899998104572365065223493729718029499053955078125f,-0.94480003947839197309122027945704758167266845703125f,-0.35170002205222772317938506603240966796875f,-1.473900006605684875893302887561731040477752685546875f,1.37120003328770412309722814825363457202911376953125f,-0.0151999980539081480657159772817976772785186767578125f,-0.3648999742627143660911315237171947956085205078125f,0.04050001083686949332474114271462894976139068603515625f,-0.9276000248432154648980940692126750946044921875f,-1.241500011090189037332720545236952602863311767578125f,1.6292000036120413142270990647375583648681640625f,-0.22120003526210840760768405743874609470367431640625f,-2.0845000347048046052123027038760483264923095703125f,0.4672000592827796339179258211515843868255615234375f,-0.135799983833731374716080608777701854705810546875f,1.728100028061866577644423159654252231121063232421875f,0.23169999016374365208292829265701584517955780029296875f,-0.298500024728477253432856741710565984249114990234375f,-0.6548000132709741460956820446881465613842010498046875f,-0.365199990379810657259440631605684757232666015625f,-0.127400018295646066945892016519792377948760986328125f,1.2753999891042706593680122750811278820037841796875f,1.459600034365058807139803320751525461673736572265625f,1.1372999843627209504148822816205210983753204345703125f,0.7328999989390378999587483122013509273529052734375f,0.85240000905692614452391353552229702472686767578125f,-1.0687999852269898237722145495354197919368743896484375f,-0.5967999551415450465441381311393342912197113037109375f,0.7881999793305991541814137235633097589015960693359375f,0.424799973782896689300514481146819889545440673828125f,0.5936999607563020475708981393836438655853271484375f,1.057599986621737375713792062015272676944732666015625f,-1.1622999360084538267301468295045197010040283203125f,-0.829399969181418672548034010105766355991363525390625f,0.414899965620040944713764474727213382720947265625f,0.3623999855503436950954210260533727705478668212890625f,1.0769000144809488528352403591270558536052703857421875f,1.0802000024497508956500269050593487918376922607421875f,-0.6576999698162087071295900386758148670196533203125f,1.3238999926172194676610160968266427516937255859375f,-0.11999999821185980408699833787977695465087890625f,-0.5283999879084524398109579124138690531253814697265625f,0.14990000160634497916589680244214832782745361328125f,1.04049999785423263887196299037896096706390380859375f,-1.1392999872535465311074176497641019523143768310546875f,0.1864000091016297933066425684955902397632598876953125f,-0.64320004932880436854247818700969219207763671875f,2.01900004154443735160384676419198513031005859375f,1.0159999597221609501929151520016603171825408935546875f,0.77719997114241134994472304242663085460662841796875f,0.476400004631281515088403466506861150264739990234375f,0.038500070512295536673263995908200740814208984375f,-0.407099989885092217178907958441413938999176025390625f,1.6624999889731402191728193429298698902130126953125f,1.0691999819308517505334066299838013947010040283203125f,0.2802000207781796259354223366244696080684661865234375f,-0.46359999544322416653585605672560632228851318359375f,-0.443900021842122072257552645169198513031005859375f,-1.594800020721554023594990212586708366870880126953125f,0.213200006860495339111594148562289774417877197265625f,-0.124599992939830261917677489691413938999176025390625f,0.9558000105440616067653536447323858737945556640625f,-0.3400000843405717176892721909098327159881591796875f,1.448699970442057338715358127956278622150421142578125f,-0.38439996508359985938341196742840111255645751953125f,0.0961000446915620809562597059994004666805267333984375f,-0.624500038355588760197178999078460037708282470703125f,0.8196999907374384886082907542004249989986419677734375f,-0.036899990192055920346092534600757062435150146484375f,0.1450999921351669996738564805127680301666259765625f,0.74180000997781814220388696412555873394012451171875f,1.7846000280134379778473885380662977695465087890625f,-0.08500001275911894804693247351679019629955291748046875f,1.116500000398605862983458791859447956085205078125f,-0.968100025901198701916428035474382340908050537109375f,-0.5612000487476589416502292806399054825305938720703125f,0.61270006272494725862998166121542453765869140625f,-0.0579000105060638592391342172049917280673980712890625f,-0.0734999884814013793032927424064837396144866943359375f,0.22010004669204408234151060241856612265110015869140625f,-1.59750004909932652452653201180510222911834716796875f,-0.24370002721548111868088426490430720150470733642578125f,0.5310000119879842994663476929417811334133148193359375f,0.69009996210932822435779598890803754329681396484375f,0.440199989634752597567057819105684757232666015625f,-1.683999976277352317310942453332245349884033203125f,0.190000021159649090662924209027551114559173583984375f,-0.3658999579012398584154652780853211879730224609375f,-0.3816999695628879862852045334875583648681640625f,-0.86080001926124083411195897497236728668212890625f,-0.18509997179508186349039533524774014949798583984375f,0.494699963915348206455746549181640148162841796875f,0.386700020003318822858773273765109479427337646484375f,0.4804999564290053370996247394941747188568115234375f,0.04179998464584411266287133912555873394012451171875f,0.3449999976158135694959128159098327159881591796875f,0.2855999781429765516094221311504952609539031982421875f,-0.297099993759393943282276495665428228676319122314453125f,0.55379999975562055336553157758316956460475921630859375f,-0.1293000042781231595512281273840926587581634521484375f,0.5262000183492900884374421366374008357524871826171875f,0.549299984683095932069818445597775280475616455078125f,-0.74100002439320089564489535405300557613372802734375f,0.9203999853789799345804567565210163593292236328125f,-0.73089997684434038216494400330702774226665496826171875f,0.636699971332401304380255169235169887542724609375f,-0.42360001451671125405340490033267997205257415771484375f,-0.8782999646857374553832187302759848535060882568359375f,-0.21800001640617860942938932566903531551361083984375f,-0.325799989011883450729101241449825465679168701171875f,0.247699983674288137791563713108189404010772705078125f,-1.45770001343935717841304722242057323455810546875f,-0.1326000427991160801610703856567852199077606201171875f,-1.2671999777734275216545256625977344810962677001953125f,-0.338000001430511787248178734444081783294677734375f,0.245900046749413458968547274707816541194915771484375f,0.37120000959485766056644706623046658933162689208984375f,0.535699994418024516562581993639469146728515625f,0.45670003218501842301435544868581928312778472900390625f,0.234400057172774012315130676142871379852294921875f,-0.3751000217139723869053113958216272294521331787109375f,0.91350002245604944306478500948287546634674072265625f,0.6715999505311256445594381148112006485462188720703125f,0.0342999761149294091211459090118296444416046142578125f,-0.8107000333473080022628209917456842958927154541015625f,-0.2375999794319281210874805765342898666858673095703125f,-1.313099988424777819773225928656756877899169921875f,0.96920000033378528314642608165740966796875f,-0.41399998035654428374385815914138220250606536865234375f,1.123299966473878530592855895520187914371490478515625f,-0.19009996603950873872435067823971621692180633544921875f,-0.235999981570989170176488869401509873569011688232421875f,-0.4308999700829383439071307293488644063472747802734375f,0.682099997168034377637013676576316356658935546875f,-0.066000030353665639637483764090575277805328369140625f,0.52380002896189681305116891962825320661067962646484375f,1.0031000170350072853153733376529999077320098876953125f,0.099399998909234899002740348805673420429229736328125f,-1.686300000761449258135371565003879368305206298828125f,1.262699971678852950418558975798077881336212158203125f,-0.9136999753221868303398878197185695171356201171875f,0.085999966427683982583829447321477346122264862060546875f,-0.0183999974638224816914089387864805757999420166015625f,-0.9519000029139217389229088439606130123138427734375f,1.022600000368057759914108828525058925151824951171875f,-0.64710002870261662533124535912065766751766204833984375f,0.96190002617835990150751968030817806720733642578125f,-0.7524000076040622264628154880483634769916534423828125f,-0.208800016146898581581581311183981597423553466796875f,-1.10710000560581622863765005604363977909088134765625f,0.2696000174522392711651264107786118984222412109375f,0.8883000127047300242821847859886474907398223876953125f,-0.01809998492300568528889925801195204257965087890625f,1.1630999732255933309943429776467382907867431640625f,0.2490000140517951709995259079732932150363922119140625f,-0.86899995874986102339931903770775534212589263916015625f,0.89259996969029276048246401842334307730197906494140625f,-0.31329996708855067932830706922686658799648284912109375f,0.3669999675750730272483224325696937739849090576171875f,-1.3008999342083935601976918405853211879730224609375f,0.45669996188879036225216623279266059398651123046875f,-0.3570999892890451832272447063587605953216552734375f,-1.298399990981817353485894273035228252410888671875f,0.050800007414817827111619408242404460906982421875f,0.9417000217914577309130663707037456333637237548828125f,-0.56470001405775516190033158636651933193206787109375f,0.482300025080144134648207909776829183101654052734375f,0.280700003805756548302952069207094609737396240234375f,0.38719996689557978353235512258834205567836761474609375f,-0.5548999919205905850816407109959982335567474365234375f,-0.080899985803663643224581392132677137851715087890625f,-0.2258000136733055995819086092524230480194091796875f,-0.5054000317960978261311311143799684941768646240234375f,-0.105999990642070773816385553800500929355621337890625f,0.647599997425079454416163571295328438282012939453125f,-0.5788999898791313825086035649292171001434326171875f,-0.3489999662190672591322027074056677520275115966796875f,-0.363800011825560876133067722548730671405792236328125f,0.02419999169111208825455605619936250150203704833984375f,0.2481999709486959471149702949333004653453826904296875f,-0.5229000101149079426932075875811278820037841796875f,1.56970003722906170651185675524175167083740234375f,0.3927999962240453779571680570370517671108245849609375f,0.05309999662041686718794153421185910701751708984375f,0.319500041186810079807401052676141262054443359375f,0.1735999796107403625455845030955970287322998046875f,-1.0761999786376943433197084232233464717864990234375f,1.32590002927780137298441331950016319751739501953125f,-0.8590000007152553962441743351519107818603515625f,1.38539998333007030595354081015102565288543701171875f,0.0372000029563910317165209562517702579498291015625f,-1.98709994339346973646343030850403010845184326171875f,0.621799984049796972129797723027877509593963623046875f,0.47570000790357536146757411188445985317230224609375f,1.1130999946087598573285504244267940521240234375f,-0.00259998535513883410885682678781449794769287109375f,0.26320005171298976875959851895458996295928955078125f,-1.85929999984502813958897604607045650482177734375f,-0.4337999236106870881712893606163561344146728515625f,0.29849996054172578396901371888816356658935546875f,-0.79499996691942254045670779305510222911834716796875f,1.4588999335527432776871137320995330810546875f,1.282399965023994337087742678704671561717987060546875f,-0.4646999788165093736580502081778831779956817626953125f,-0.89359999253749844427829884807579219341278076171875f,-0.7122000039994713649349478146177716553211212158203125f,0.78720000027418113308641522962716408073902130126953125f,-0.35790001041293140193744193311431445181369781494140625f,0.0032999878570436624869444131036289036273956298828125f,0.003400001934170671091806070762686431407928466796875f,-0.4942000271186230975217767991125583648681640625f,-0.507999986454844787431284203194081783294677734375f,0.4568000287160278727327522574341855943202972412109375f,-0.416099997827410472694964482798241078853607177734375f,-0.596400010666251478852473155711777508258819580078125f,-0.173899999378621661350052818306721746921539306640625f,-0.49780000449419059549427402089349925518035888671875f,0.8594999778568748194373938531498424708843231201171875f,-0.2048000187098981683675447129644453525543212890625f,0.626099997231364113758900202810764312744140625f,0.398799992603063646612326920148916542530059814453125f,-0.241899986429512436902911076685995794832706451416015625f,-0.04329996926784485111738831619732081890106201171875f,-0.0184000060319904346073371925740502774715423583984375f,-0.415299980652331868213877896778285503387451171875f,0.637099982434511158402301589376293122768402099609375f,-0.3168999882549041746671036889893002808094024658203125f,-0.98139993534088176829754956997931003570556640625f,0.535699971544742137297134831896983087062835693359375f,-0.209900004959106212254482670687139034271240234375f,0.70859999238848647706845440552569925785064697265625f,0.35630000422596896925142573309130966663360595703125f,-0.09010001299679260000630165450274944305419921875f,0.27010001269876937612934852950274944305419921875f,0.03600005412101747737096957280300557613372802734375f,-1.55349998661875776662100179237313568592071533203125f,1.28660003136992440175845331395976245403289794921875f,-0.062000034928320957305913907475769519805908203125f,-0.8932000007137654851163688363158144056797027587890625f,-0.4578999945431949658569692473975010216236114501953125f,0.3767000329673291236787235902738757431507110595703125f,-0.194099997991323380830408495967276394367218017578125f,1.408600011163950060932847918593324720859527587890625f,0.0322000225886702740041300785378552973270416259765625f,-0.37720002031624344507321211494854651391506195068359375f,0.358300012443959925434455726644955575466156005859375f,-0.39969999710768444600716975401155650615692138671875f,-0.08480000179708024887759165721945464611053466796875f,1.5579999806433912912240202786051668226718902587890625f,-0.079999969601631359950033584027551114559173583984375f,-2.06290002084374446411629833164624869823455810546875f,0.0916999989554285743764694416313432157039642333984375f,1.01650000250339456187020914512686431407928466796875f,-0.62630002069175183532223627480561845004558563232421875f,0.2459999969750636950038824579678475856781005859375f,-1.643300048653781875174217930180020630359649658203125f,0.07199999294430037277692235875292681157588958740234375f,-0.1681999908417461941922965706908144056797027587890625f,0.7174999961256975922196943429298698902130126953125f,-0.68460001536607728578331943936063908040523529052734375f,-0.152299985778331148367215064354240894317626953125f,-1.36970000608563413635465622064657509326934814453125f,-0.0153999616861337784712304710410535335540771484375f,-0.1138999931573874135892765480093657970428466796875f,2.14619996403455726152742499834857881069183349609375f,-0.5466999538421628557216536137275397777557373046875f,-0.329500012874603953605401329696178436279296875f,-1.093899934595823797423008727491833269596099853515625f,-1.048100040243566066777702872059307992458343505859375f,0.856099997827410863493469150853343307971954345703125f,1.1113999813854693599068923504091799259185791015625f,-0.5084999706745154668396935448981821537017822265625f,0.13330000413656239999227182124741375446319580078125f,-0.6592999822616576466316473670303821563720703125f,-1.574099978470801630692221806384623050689697265625f,-1.1046000100761654838521508281701244413852691650390625f,-0.00490002868920580425537991686724126338958740234375f,-0.654800005447864652552425468456931412220001220703125f,-0.219399957223236352632511625415645539760589599609375f,-0.94200002412497951809200458228588104248046875f,0.8839999779164788851204548336681909859180450439453125f,-0.40609999700784715770396360312588512897491455078125f,-0.12549998712539645051577963386080227792263031005859375f,-0.6114000067919491687717936656554229557514190673828125f,-0.00530002550482799872355599291040562093257904052734375f,-0.2292999945551155160927692122641019523143768310546875f,0.09350001720339073951748787294491194188594818115234375f,0.370200027111172946803208105848170816898345947265625f,-0.7656000008672469991921616383478976786136627197265625f,0.8711000302374360959589694175519980490207672119140625f,-0.2191000198960306732942626695148646831512451171875f,0.0782999702736735248009836141136474907398223876953125f,-0.0907000102877617564445245079696178436279296875f,0.167200026164949111517898927559144794940948486328125f,-0.3644000006973744376637114328332245349884033203125f,-0.06880002147406383272709717857651412487030029296875f,0.0784000211566697746690124404267407953739166259765625f,0.00119999573677775916991095073171891272068023681640625f,0.85580001546144490731649057124741375446319580078125f,-0.08379999543428429120695000165142118930816650390625f,-1.10589996050894256995889008976519107818603515625f,1.13899997612834003035686691873706877231597900390625f,1.0432999864041807569492448237724602222442626953125f,-0.218900034284590816469062701798975467681884765625f,-0.16409999337196357060975060448981821537017822265625f,0.07590004000663697070194757543504238128662109375f,-0.17319997005462661743280250448151491582393646240234375f,-0.23530001134872424728428086382336914539337158203125f,1.318100000271201555079869649489410221576690673828125f,-0.30270005013346601430157534196041524410247802734375f,0.918500009566546093964234387385658919811248779296875f,-0.96870001378580938844464753856300376355648040771484375f,-0.28709997274875609374333862433559261262416839599609375f,-0.6356000469677149933289683758630417287349700927734375f,1.88010004514604833758539825794287025928497314453125f,-0.88679999939799269981222096248529851436614990234375f,-1.0921000164464118586948870870401151478290557861328125f,0.744800036740303372795324321486987173557281494140625f,-1.1057000591635708364179890850209631025791168212890625f,0.41490001337826232230554523994214832782745361328125f,0.9241000434026125365249981769011355936527252197265625f,-0.55600002361088984859094352941610850393772125244140625f,-0.997300017778575575988497803336940705776214599609375f,0.477600014263391159374805283732712268829345703125f,-0.16000004440545989581323738093487918376922607421875f,0.824599992045760732395365266711451113224029541015625f,0.666300020021200811726203028229065239429473876953125f,-0.6879000203967091664480904000811278820037841796875f,-0.105399996182323096860500299953855574131011962890625f,-0.910799982976913558019305128254927694797515869140625f,0.7419000139594071896453897352330386638641357421875f,-0.31369998094737550076160914613865315914154052734375f,-0.323600061082839118853371473960578441619873046875f,0.43059994908571308513955955277197062969207763671875f,0.6371000129818913393364709918387234210968017578125f,-0.1214000099770726759373928871355019509792327880859375f,0.8434999608397486525745989638380706310272216796875f,-0.9079000052161514400239639144274406135082244873046875f,1.6856999753445389700345913297496736049652099609375f,-0.217099977144598665912411661338410340249538421630859375f,-0.6559000246584425486418012951617129147052764892578125f,-0.9540000385642055658053095612558536231517791748046875f,1.268900044976175589539479915401898324489593505859375f,0.3363000343076887332927071838639676570892333984375f,-0.6190999907642602551050003967247903347015380859375f,0.2726999934159215488449490294442512094974517822265625f,1.2931000318244108004961390179232694208621978759765625f,1.4327000359773638393789951805956661701202392578125f,-0.04720002977848025693674571812152862548828125f,-1.09519999485015784301822350244037806987762451171875f,-0.592300026868282980530011627706699073314666748046875f,-2.042000027962028507744207672658376395702362060546875f,-0.0848000126004213872210613089919206686317920684814453125f,1.946600060799717812898279589717276394367218017578125f,0.06570000347048010613804081003763712942600250244140625f,-0.979600025014579767201894355821423232555389404296875f,1.272000012539326263549810391850769519805908203125f,1.178300028015673017733888627844862639904022216796875f,-0.38820000436455048831163594513782300055027008056640625f,0.14850003772974051940991557785309851169586181640625f,0.76949999678134872738155536353588104248046875f,-0.843799996998905843526017633848823606967926025390625f,2.045799957942962432611011536209844052791595458984375f,0.0779999775141479168016900302973226644098758697509765625f,-1.0926000002562987756249412996112369000911712646484375f,-0.1046000069469206206207445575273595750331878662109375f,1.1347999645069235530314699644804932177066802978515625f,0.71989999147355498987366217988892458379268646240234375f,-1.805199973392486612056018202565610408782958984375f,-0.39509997619092462883827465702779591083526611328125f,1.653499990865586966037881211377680301666259765625f,-1.269700042053312216694394010119140148162841796875f,0.76360000203698830123499874389381147921085357666015625f,0.2193999857962130040078818637994118034839630126953125f,-0.8478000267714265181240307356347329914569854736328125f,-0.42580003343597072618109677932807244360446929931640625f,0.9317000174701208781158356941887177526950836181640625f,-0.413000006739050018467196423443965613842010498046875f,1.215799996611476085917047385009936988353729248046875f,-0.514699990905076010250240869936533272266387939453125f,-0.0362000042676928757146015414036810398101806640625f,-0.0327000170528892741828030921169556677341461181640625f,0.5350000479817396126236417330801486968994140625f,-0.19800003063678739412267759689711965620517730712890625f,0.47359998430460692464549765645642764866352081298828125f,-0.2264000070527194541369908620254136621952056884765625f,-0.2609000388890507782235772538115270435810089111328125f,0.2863999875694513974622168461792171001434326171875f,0.213199998664856504859699271037243306636810302734375f,-1.396700046378374526767629504320211708545684814453125f,-0.486900007030367287796934760990552604198455810546875f,0.0877999799072739506300422362983226776123046875f,0.5932999642759557712423656994360499083995819091796875f,1.2446000368237495337098152958787977695465087890625f,1.076000053077936779999390637385658919811248779296875f,-1.1838000046730041692200074976426549255847930908203125f,0.1163999950572851549424058248405344784259796142578125f,-0.0052000210762022103239132775343023240566253662109375f,-0.261899987509846443600736165535636246204376220703125f,0.121199990633130028072628192603588104248046875f,0.6337000138044361907674328904249705374240875244140625f,-0.8168000500619412296288146535516716539859771728515625f,0.99260000226795686018022024654783308506011962890625f,0.0094999972283840161679790980997495353221893310546875f,-0.7016000336050982610913706594146788120269775390625f,0.902199994760751966538236956694163382053375244140625f,0.12599997067451429444417954073287546634674072265625f,-0.07690000941455366056942466457257978618144989013671875f,-0.12359997942447620289385668002068996429443359375f,-0.06960000620186346242945774065447039902210235595703125f,-0.125100026333331726391406846232712268829345703125f,-1.85350003352016212687658480717800557613372802734375f,1.21359999462366108247124429908581078052520751953125f,0.69870002874284986571495892349048517644405364990234375f,-0.31869998593926407437493253382854163646697998046875f,0.958200044988841259652190274209715425968170166015625f,-1.1792000357985497061008572927676141262054443359375f,-0.02309999160990106048529924009926617145538330078125f,0.12590001497268676899921047152020037174224853515625f,1.255700040909648595999215103802271187305450439453125f,-0.6489000708669430128594513007556088268756866455078125f,0.172000040292740408176541677676141262054443359375f,-0.76120005984902405771208577789366245269775390625f,1.8952000526666648028140116366557776927947998046875f,1.409100015723705912051855193567462265491485595703125f,-1.174499968618153644683843594975769519805908203125f,0.02330006143152807407403770412201993167400360107421875f,-0.95100002312660247838493887684307992458343505859375f,-0.216499961450695277420663842349313199520111083984375f,1.39440002349615088661494155530817806720733642578125f,-0.0584000096827750336814233378390781581401824951171875f,1.0068000121384859113504717242904007434844970703125f,0.7186999485373490870188106782734394073486328125f,1.8239000132739544568494238774292171001434326171875f,-0.3262999954342848951682753977365791797637939453125f,0.209700034099817411714639092679135501384735107421875f,-0.059100083673000458617252661497332155704498291015625f,0.05749997146427665573042986579821445047855377197265625f,0.1094999997615815345852752216160297393798828125f,-0.300400004526972796980999191873706877231597900390625f,-1.011799978089332086028662160970270633697509765625f,-0.4120999726742502478060714565799571573734283447265625f,0.503699992644786931350608938373625278472900390625f,-0.37399999538809058918786831782199442386627197265625f,0.28540004215240610818682398530654609203338623046875f,0.1575000450201342605538457064540125429630279541015625f,1.14759996747374515280171181075274944305419921875f,0.848000037360936431696245563216507434844970703125f,-0.97350002603232876907668469357304275035858154296875f,-0.9631000044248996250217942360904999077320098876953125f,0.4377999901890756717648400808684527873992919921875f,0.2682000274240967652161771184182725846767425537109375f,0.403499968662858865542375497170723974704742431640625f,-0.366499993246048205719489487819373607635498046875f,0.60650002515315915019300518906675279140472412109375f,1.1697999927446238732642314062104560434818267822265625f,0.563199989053607463773687413777224719524383544921875f,-0.3295000742673881433120186557061970233917236328125f,0.912799999800325334575745728216134011745452880859375f,-2.75090000625550690216414295719005167484283447265625f,-1.2171999771028747527878977052750997245311737060546875f,0.22040001607537273997650117962621152400970458984375f,0.3367000240266317945270202471874654293060302734375f,-1.34429997265040857001849872176535427570343017578125f,-0.14649998953938503376548396772705018520355224609375f,1.09509995316863051328937217476777732372283935546875f,0.3216999818563477386845761429867707192897796630859375f,-0.84509997730851227348836118835606612265110015869140625f,0.1562000105261791560451456462033092975616455078125f,0.474499995887278647188622926478274166584014892578125f,0.7694999562501914081025233826949261128902435302734375f,-0.623800034177302453741731369518674910068511962890625f,0.166300035220384945233718099188990890979766845703125f,-0.90039998455941638866306675481610000133514404296875f,0.3007999783575534546997687357361428439617156982421875f,-0.5527999874323603624048928395495750010013580322265625f,-0.10509996121525855272693661390803754329681396484375f,-0.3773999665141112291877334428136236965656280517578125f,0.3994999306201947320005274377763271331787109375f,-1.024499994825571302925482086720876395702362060546875f,0.58439994466900824221511356881819665431976318359375f,0.139399989577382033001384797898936085402965545654296875f,0.096200023937225154213592759333550930023193359375f,1.202299979314953137787824744009412825107574462890625f,-1.6284000242114071710375355905853211879730224609375f,-0.9853000431440770423563435542746447026729583740234375f,0.71109998762011539241711943759582936763763427734375f,-0.410799993575364286169104843793320469558238983154296875f,1.319600012013316447934130337671376764774322509765625f,0.6597000372476877760874458545004017651081085205078125f,-1.403700032132864095046898000873625278472900390625f,-1.027800045397878392350321519188582897186279296875f,1.207999972596763171850398066453635692596435546875f,0.356200021255016707755203242413699626922607421875f,-0.2524000159114603292209721985273063182830810546875f,-1.213700024235247809656357276253402233123779296875f,-0.823900015360116366736065174336545169353485107421875f,2.16790004155635873672736124717630445957183837890625f,0.2689999770224098529780576427583582699298858642578125f,-0.425399990892409984866162631078623235225677490234375f,0.921400024598837408262852477491833269596099853515625f,-0.9428000106036662497643874303321354091167449951171875f,-0.0103000043079257697087314227246679365634918212890625f,-0.4314000281378629342299291238305158913135528564453125f,-0.99719997211098654332772639463655650615692138671875f,-0.8168999664992089293491517310030758380889892578125f,0.9283999961972231318441117764450609683990478515625f,0.635499989658594000019320446881465613842010498046875f,-1.06029997349977467280268683680333197116851806640625f,0.0288000156998637901750726086902432143688201904296875f,0.32619997297525404889029232435859739780426025390625f,-0.0117000114351508699428450199775397777557373046875f,0.280699976909160486826522173942066729068756103515625f,-0.253000010743737124840890828636474907398223876953125f,-0.69149999319016919319125236143008805811405181884765625f,0.7915000118166208853409671064582653343677520751953125f,-0.7899000603914263507476789527572691440582275390625f,-0.71990001902207734207905787116033025085926055908203125f,-0.31880003328323436306845906074158847332000732421875f,-0.7475000186450782013736215958488173782825469970703125f,-0.46970000221133201279144486761651933193206787109375f,-0.96489999205097543022446870963904075324535369873046875f,0.52480001766681727559671344351954758167266845703125f,0.204000045120716622104595217024325393140316009521484375f,-0.17870001685917358269506394208292476832866668701171875f,0.2582999986417588988985016840160824358463287353515625f,0.7331000015377997502952212016680277884006500244140625f,-1.382100030155479775118010365986265242099761962890625f,-1.223900007387995092500432292581535875797271728515625f,-1.04389998987913035222163671278394758701324462890625f,-0.84739998677968930707038452965207397937774658203125f,-0.54039995825886677494054310955107212066650390625f,-1.460799982008337938310660319984890520572662353515625f,-0.14129999066591292233852072968147695064544677734375f,1.697500075101852967662807714077644050121307373046875f,0.9841999878168099513686684076674282550811767578125f,-0.668800053288042928301138090319000184535980224609375f,-0.9715000103265045705569491474307142198085784912109375f,0.2811000013850628587164237615070305764675140380859375f,-0.0022000055640936100331828129128552973270416259765625f,0.6127000246524811899462292785756289958953857421875f,-1.073599990004300508417145465500652790069580078125f,0.371300001394749390470906291739083826541900634765625f,-0.90880000281035833165077519879559986293315887451171875f,0.805199996042251342487361398525536060333251953125f,-0.052499984502792695906236986047588288784027099609375f,-1.654099986815452894006739370524883270263671875f,-1.006299994987249757372183012194000184535980224609375f,1.167999994874000524447410498396493494510650634765625f,0.74350000956654571648840601483243517577648162841796875f,0.038999997772276306928773692561662755906581878662109375f,-0.403100039591640868508193307206965982913970947265625f,-0.38249999046325644513899533194489777088165283203125f,-1.7919000029698011200451901459018699824810028076171875f,-0.528500021338462744324715458787977695465087890625f,-1.39740001129209989727542051696218550205230712890625f,-0.532100040102005067410573246888816356658935546875f,0.22900002103671457387434884367394261062145233154296875f,-0.8382999905019996589317088364623486995697021484375f,0.20440001755431336238899575619143433868885040283203125f,0.6583999356240026745723525891662575304508209228515625f,-0.48310001828298021475660561918630264699459075927734375f,0.555500044792891056744110755971632897853851318359375f,0.42119996895194145025698162498883903026580810546875f,1.16610004171133052608411162509582936763763427734375f,0.1227999559164043574810420977883040904998779296875f,0.11279993505477958759541934341541491448879241943359375f,0.24580000801086487882685105432756245136260986328125f,-0.9658999839037658574625311302952468395233154296875f,-0.86199996235966747093470985419116914272308349609375f,-0.6016999907970426164638411137275397777557373046875f,1.737899940228462725144709111191332340240478515625f,-0.5783000099480144484687116346322000026702880859375f,-0.969299995970725802862943965010344982147216796875f,2.0545000356733797985953060560859739780426025390625f,-1.09250001855194600608456312329508364200592041015625f,0.16200001496076577023330855809035710990428924560546875f,0.9700999847590925373452819258091039955615997314453125f,-1.8781000278756021515391694265417754650115966796875f,1.310399991413950804286514539853669703006744384765625f,1.734100041502714351082659050007350742816925048828125f,-0.49599998851865534366112342468113638460636138916015625f,-1.1219000234961509221420783433131873607635498046875f,0.4523999950870869479757629960658960044384002685546875f,1.1347000195339322914378499262966215610504150390625f,-0.6766999941498041959420106650213710963726043701171875f,-1.3885000304654244729363199439831078052520751953125f,-0.7287000016972420279870448212022893130779266357421875f,-0.1379000084757802113699654000811278820037841796875f,-0.81520000468492537226694594210130162537097930908203125f,1.1474000378161672841770268860273063182830810546875f,0.10990001218616907152636485989205539226531982421875f,-0.3268000336706642361406238705967552959918975830078125f,-1.10409999530911395737575730890966951847076416015625f,0.50929999436885065033919772758963517844676971435546875f,-0.031499991066753718627069247304461896419525146484375f,-0.9007000196009879999792246962897479534149169921875f,-0.51390001189559664229733471074723638594150543212890625f,0.2810000167936090864628795316093601286411285400390625f,0.1704999769926072161752017564140260219573974609375f,-0.919700012195109994905806161114014685153961181640625f,1.2203999435067185430625613662414252758026123046875f,-0.6528999721169468362091947710723616182804107666015625f,0.19089997898638255691139420378021895885467529296875f,-0.005900027601420554379529903599177487194538116455078125f,-0.171999968804418923440380240208469331264495849609375f,-0.539300027970969875212858823942951858043670654296875f,0.318100029477476287098625107319094240665435791015625f,-0.3610000062882903204553031173418276011943817138671875f,-0.47999996952712553621722690877504646778106689453125f,-0.477699995632469676110076761688105762004852294921875f,0.8273000308170919137040755231282673776149749755859375f,-0.42990000786483284489492007196531631052494049072265625f,0.203700014661252704062377461013966239988803863525390625f,1.06240002786964193859375882311724126338958740234375f,-0.2791999934792517290560454057413153350353240966796875f,-0.701100030870736024013467613258399069309234619140625f,1.020900036132335930716408256557770073413848876953125f,-0.24900002038478863397585882921703159809112548828125f,0.481599997767805820814146500197239220142364501953125f,1.1890000420287256677198683973983861505985260009765625f,-0.673000005677342638676918795681558549404144287109375f,-1.65970005868673364801679781521670520305633544921875f,0.1843999955192205941756355969118885695934295654296875f,0.03580001825541290205734412666060961782932281494140625f,0.08239999698698508012739694095216691493988037109375f,-0.3985000188797711562216363745392300188541412353515625f,-0.8707000432193285899273860195535235106945037841796875f,0.310100025290251490872606154880486428737640380859375f,-0.453999994382261906622488822904415428638458251953125f,0.257899991041421916548870285623706877231597900390625f,0.3920000502392653185523840875248424708843231201171875f,-0.5266000499963769954092640546150505542755126953125f,-1.2422000197947025146305577436578460037708282470703125f,0.2581000307440763918975790147669613361358642578125f,0.201399988240004379491665531531907618045806884765625f,-0.73489999441653497402882067035534419119358062744140625f,-0.004900003692507315378179555409587919712066650390625f,0.684500001028179649864569000783376395702362060546875f,0.592799967539310301845034700818359851837158203125f,-0.861500042364001128447625887929461896419525146484375f,0.064299988967180699006576105603016912937164306640625f,-0.947600009867548553899041507975198328495025634765625f,0.881100014516710938750065906788222491741180419921875f,-0.18580001531243350854083473677746951580047607421875f,0.440300024163722270031939842738211154937744140625f,-0.2238000082492828113345240126363933086395263671875f,-0.3465000022053714712200189751456491649150848388671875f,0.10260001195371160065406002104282379150390625f,-0.3482000125229358644673993694595992565155029296875f,1.7015000030249354256994820389081723988056182861328125f,0.004800021690130495954917932976968586444854736328125f,-1.15209999670237284163931690272875130176544189453125f,0.4889999918490650632207916714833118021488189697265625f,-1.245200031246245231386637897230684757232666015625f,0.0184999860972163698846770785166881978511810302734375f,1.041700021158158850909103421145118772983551025390625f,-0.7413000244170431241030883029452525079250335693359375f,-0.89769999734610295138992341890116222202777862548828125f,-0.67420000938624158326462065815576352179050445556640625f,0.583399998879432502008057781495153903961181640625f,-1.3163000368222597113998517670552246272563934326171875f,0.409400011277199382675462402403354644775390625f,-2.176400026722252523114775613066740334033966064453125f,0.241200015890598251644405536353588104248046875f,-0.667300024782121692368264120887033641338348388671875f,0.19870001976490048178902725339867174625396728515625f,1.554200042355061039955899104825221002101898193359375f,-0.61830003304481540027381925028748810291290283203125f,0.85020004003793037128389187273569405078887939453125f,0.20990000615119885907233765465207397937774658203125f,0.68119999566227296128317902912385761737823486328125f,1.0204999954700468833834747783839702606201171875f,0.101400003588199894011268042959272861480712890625f,-0.10140001431703549172880229889415204524993896484375f,-0.26029998393058662031762651167809963226318359375f,-1.09069999106526349663681685342453420162200927734375f,-0.7989000190854067540158212068490684032440185546875f,0.503100022995472517806092582759447395801544189453125f,-0.7191000087201597779085204820148646831512451171875f,0.536700026410818598066043705330230295658111572265625f,0.0006000018358225389647486736066639423370361328125f,0.347300020498037209648600764921866357326507568359375f,-0.16420000122785582963302886128076352179050445556640625f,0.7339999547824256342920534734730608761310577392578125f,-0.164300004538894039995966522837989032268524169921875f,-2.142300005857646016949047407251782715320587158203125f,1.091399978777765955584300172631628811359405517578125f,0.73660001177489731194469868569285608828067779541015625f,0.1604999850392341276261731763952411711215972900390625f,-1.0913999776229257321347176912240684032440185546875f,0.62929997384250124792259839523467235267162322998046875f,0.6161999743536112905673007844598032534122467041015625f,-0.116999999687075639798194970353506505489349365234375f,-0.4051000041306018761844143227790482342243194580078125f,0.5362000229127710237975179552449844777584075927734375f,-0.88370004360675746823972076526843011379241943359375f,-0.76679996762126645393209400936029851436614990234375f,0.481900032287835955457921954803168773651123046875f,0.2933000196337698373127977902186103165149688720703125f,-0.063700016337632892060582889826036989688873291015625f,0.4006000393867499287381406247732229530811309814453125f,0.369399999207256968247747863642871379852294921875f,0.165800001752376691133150643508997745811939239501953125f,-0.413100002878903804770516217104159295558929443359375f,0.7730000158101313356695527545525692403316497802734375f,-0.0107000144600863933419532258994877338409423828125f,0.591600041223316441829638279159553349018096923828125f,-0.33250004753470374652124519343487918376922607421875f,1.78359994604587512867510668002068996429443359375f,-0.738099951916932450757258266094140708446502685546875f,-0.6142000231325628334388966322876513004302978515625f,0.731299982097745715492465023999102413654327392578125f,0.46959997729361102614120682119391858577728271484375f,-0.297899988210201360772089174133725464344024658203125f,-0.62309996279478152558795045479200780391693115234375f,-0.797599963673949474696200923062860965728759765625f,0.8036999825865027702320730895735323429107666015625f,-0.04410001542568142696154609438963234424591064453125f,0.9479000255748630809904398120124824345111846923828125f,-0.0355999796703450055446182886953465640544891357421875f,0.139199997726082269622338571934960782527923583984375f,-1.435000018365681473397899026167578995227813720703125f,-0.081099999913572418108742567710578441619873046875f,2.026000017799436836440918341395445168018341064453125f,-0.8358999867349858892140446187113411724567413330078125f,0.103800005678832574229630836271098814904689788818359375f,0.43729999479353420799299101418000645935535430908203125f,-1.217400028651952847980055594234727323055267333984375f,0.495599993714689990298438715399242937564849853515625f,0.87990004057288206951170650427229702472686767578125f,-0.126899962252377651594770213705487549304962158203125f,-0.241099981510640049009452923201024532318115234375f,-0.07320000607818410998106628539972007274627685546875f,0.215100054347516334019019268453121185302734375f,1.581399963764845484348597892676480114459991455078125f,-0.349500006616114688995367032475769519805908203125f,-0.6693000118404626164192450232803821563720703125f,0.01489998677968973339602598571218550205230712890625f,-1.0066999980613589027456100666313432157039642333984375f,0.112300015282631004964741805451922118663787841796875f,0.5529000356703999141672056794050149619579315185546875f,-0.663800031942129553641507300199009478092193603515625f,0.3175999944075946768151652577216736972332000732421875f,-1.2649999237060551848799150320701301097869873046875f,-0.82239994788765979905065250932238996028900146484375f,0.635300035339593449634776334278285503387451171875f,0.2391999285101906735917509649880230426788330078125f,1.058399991875887291570279558072797954082489013671875f,-0.07520002644061918317675008438527584075927734375f,-0.6576000160515305115183082307339645922183990478515625f,0.4690999865174294569669655174948275089263916015625f,-1.2324999912083143271956942044198513031005859375f,-0.19929999716281887600644040503539144992828369140625f,1.0668000208184122978849472929141484200954437255859375f,0.078700004155933811489376239478588104248046875f,-0.40799999304860834481445408528088591992855072021484375f,0.6783999922484162947711183733190409839153289794921875f,-0.3356999745622275543865953295608051121234893798828125f,0.6320999965161087086329416706576012074947357177734375f,-0.01769999052882187395852042754995636641979217529296875f,0.3858999932169913105184377855039201676845550537109375f,0.4686999634757633703685542059247381985187530517578125f,-0.124699982765317241018010463449172675609588623046875f,0.0236000373154883202886367143946699798107147216796875f,-0.08360004726201342162283935977029614150524139404296875f,-0.231900012525171173027871418526046909391880035400390625f,-0.075500013947487065735231226426549255847930908203125f,0.5697000461697587780918183852918446063995361328125f,0.4265999879330391042486780861509032547473907470703125f,0.61810001882314669074958146666176617145538330078125f,0.123099999302625828789814477204345166683197021484375f,0.80349999153613982372235113871283829212188720703125f,-0.281300006572902105972389108501374721527099609375f,-0.66780001045465464670769506483338773250579833984375f,-0.096699956487118488013265960034914314746856689453125f,-0.3691000473886736354955928618437610566616058349609375f,0.29090000770837054488993089762516319751739501953125f,0.3303000212207438668343684184947051107883453369140625f,-0.416000036314129406633810503990389406681060791015625f,0.925000043660402671008569086552597582340240478515625f,-0.3955000562667834174135350622236728668212890625f,0.3604000005781653470648961956612765789031982421875f,-0.655599993789195423232740722596645355224609375f,0.907399975305795525315488703199662268161773681640625f,-0.24979996595978803952675662003457546234130859375f,-1.63140001953244162535838768235407769680023193359375f,-0.790299964857101855386645183898508548736572265625f,0.775500012569129371087228719261474907398223876953125f,1.9274999622255555298266926911310292780399322509765625f,-0.83550001510232696499969051728839986026287078857421875f,1.6191000307738783181576991410111077129840850830078125f,-0.6366000394165520770428656760486774146556854248046875f,1.234600035482645186135641779401339590549468994140625f,-0.2470000183507801150284421964897774159908294677734375f,0.567200021210312765873595708399079740047454833984375f,0.5601000112459064705916489401715807616710662841796875f,-0.423899998745322026394433123641647398471832275390625f,-0.3871000274732712131964262880501337349414825439453125f,0.063099985891580112618015618863864801824092864990234375f,0.2271000362649566728379113556002266705036163330078125f,0.33469999580383291704066550664720125496387481689453125f,0.41639997132718509220694613759405910968780517578125f,1.3072999793708321991658749539055861532688140869140625f,-1.060299961429833626169738636235706508159637451171875f,-0.62469998887479238902642464381642639636993408203125f,1.44540000713467531312517166952602565288543701171875f,-0.0556999764621257664742870474583469331264495849609375f,-1.518199994790553564172341793891973793506622314453125f,-0.713399994111060475887597931432537734508514404296875f,1.7149000057041643430011390591971576213836669921875f,0.831299994838237576999517841613851487636566162109375f,-1.55449996995925854292863732553087174892425537109375f,-0.9657000028371809374760914579383097589015960693359375f,1.0453999849319455250906685250811278820037841796875f,-1.44330003913566518036759589449502527713775634765625f,0.9850000155717140160049893893301486968994140625f,-0.6632999739430840424603275096160359680652618408203125f,0.0020000285953292173957152044749818742275238037109375f,2.100900067033619311729353285045363008975982666015625f,-1.15200002330541639850025603664107620716094970703125f,0.6594000192865732135061307417345233261585235595703125f,-1.56220002351999287526496118516661226749420166015625f,1.0997000058248633802548965832102112472057342529296875f,-0.26610001310110131811370592913590371608734130859375f,-0.0752000250250102197924206848256289958953857421875f,-0.11369997297525369361892444430850446224212646484375f,-0.697399952246248400200556716299615800380706787109375f,0.23270006130933751364864292554557323455810546875f,0.5175999795436851957930457501788623631000518798828125f,1.1976000427246098478661906483466736972332000732421875f,-0.8950000330060721154268321697600185871124267578125f,-0.53339994679987245973507015150971710681915283203125f,1.10110003455877336620005735312588512897491455078125f,0.422399996837973024099710528389550745487213134765625f,0.55240003178119678750590537674725055694580078125f,-0.3953999683916566976193962545949034392833709716796875f,1.09060003134012273307007490075193345546722412109375f,-0.217099998378753600292156988871283829212188720703125f,-0.7812000190943473310056788250221870839595794677734375f,-0.222399968451262086688302588299848139286041259765625f,-1.138299982455373449141688979580067098140716552734375f,-0.355700025710464107220332152792252600193023681640625f,0.603200003805756335140131341177038848400115966796875f,-0.5249000124096863117983957636170089244842529296875f,0.199100011029840207044117050827480852603912353515625f,-1.28569998406171759341987126390449702739715576171875f,0.2685000337064271658249481333768926560878753662109375f,-0.23759999414682386742470043827779591083526611328125f,-0.234399984753131729320330123300664126873016357421875f,0.36559996726512888898241726565174758434295654296875f,0.484900011031329736255202078609727323055267333984375f,0.932599998691677978257530412520281970500946044921875f,0.97210000931248063960055105781066231429576873779296875f,-1.7016999536186456776221120890113525092601776123046875f,0.874500028520822869637640906148590147495269775390625f,-0.3581000408768648668456080486066639423370361328125f,-0.6396999744765461404227835373603738844394683837890625f,0.0196999829888344635264729731716215610504150390625f,-0.7961000184468927098890844717971049249172210693359375f,0.6621999865651133632127312012016773223876953125f,-0.4952000107757748015302468047593720257282257080078125f,1.7951000096440310471734846942126750946044921875f,0.3578000124663116565670861746184527873992919921875f,0.59480001446306707268973923419252969324588775634765625f,-0.5131000268697738508905104026780463755130767822265625f,0.1404000502735385236263709884951822459697723388671875f,-0.50569998592436338213929047924466431140899658203125f,-0.6853999605685476836924863164313137531280517578125f,-0.96740000108480472817973350174725055694580078125f,-0.1726000004798180498255533166229724884033203125f,0.158000002026557950784990680404007434844970703125f,-0.0212000402539975763005486442125402390956878662109375f,-1.25670003013610820374879040173254907131195068359375f,1.64790002054572060075088302255608141422271728515625f,-0.29360000993460422247238739146268926560878753662109375f,0.34860000639557853219230310060083866119384765625f,-0.134700003403425139225646489649079740047454833984375f,-0.657800020512938399264157851575873792171478271484375f,0.323999984845519062304219914949499070644378662109375f,0.3183000375151632255210643052123486995697021484375f,-1.4705999609321356302871208754368126392364501953125f,-0.06269995759725599526746009360067546367645263671875f,-0.9587999770313502256868787299026735126972198486328125f,0.86909999873638099643358145840466022491455078125f,-0.202400007790327407519725966267287731170654296875f,0.10680001057386423468642533407546579837799072265625f,-0.11199999024346463105672455640160478651523590087890625f,-0.6018000077426430838301030235015787184238433837890625f,-0.75870000447258367781699917031801305711269378662109375f,0.522899998380243591356020260718651115894317626953125f,-0.52209999886304159222305543153197504580020904541015625f,-1.12030001816600499608966856612823903560638427734375f,-1.4046000033706425025314956656075082719326019287109375f,1.0757000086486339096580877594533376395702362060546875f,0.3122000040739771886677544898702763020992279052734375f,-0.49370001291036569313064319430850446224212646484375f,-0.222099994225054775398575657163746654987335205078125f,-0.4316000297307969191251686424948275089263916015625f,0.5285000162720674410365973017178475856781005859375f,-0.4429000031858691954766982235014438629150390625f,-0.978600026959181246155594635638408362865447998046875f,-1.13399996521323931375491156359203159809112548828125f,-2.002699978905915667581894012982957065105438232421875f,-0.89409999273866491709128467846312560141086578369140625f,1.0340999949738378660413218312896788120269775390625f,0.93220000005066339099357719533145427703857421875f,-1.578500008746981375651330381515435874462127685546875f,-0.970099989005923202256553850020281970500946044921875f,-1.019100008273124302604628610424697399139404296875f,-0.467900001621246008909338343073613941669464111328125f,0.165199991720914596271541086025536060333251953125f,0.0626000698447224390719156872364692389965057373046875f,-1.3102999787896874028803040346247144043445587158203125f,1.6147999826490877506302012989181093871593475341796875f,-0.847600001969933458667583181522786617279052734375f,-0.5259999471679337457175051895319484174251556396484375f,-0.7005999554187065658794608680182136595249176025390625f,0.2419000060617921832317733787931501865386962890625f,-0.844300010350346408216637428267858922481536865234375f,0.83270000461041870920553265023045241832733154296875f,-0.300199986878037616833125866833142936229705810546875f,-0.13769997786283472152035756153054535388946533203125f,-1.290100033597648465644169846200384199619293212890625f,0.7157999859571457790252679842524230480194091796875f,-0.3140999829784030250579007770284079015254974365234375f,0.232299988590925376996665363549254834651947021484375f,0.35079999044612020231426186001044698059558868408203125f,1.439400009321421425312337305513210594654083251953125f,-0.1608999739572400944354058083263225853443145751953125f,-0.79579997440874572500746353398426435887813568115234375f,1.046700020804256059392400857177563011646270751953125f,-0.5968000119522212099099078841391019523143768310546875f,-0.38219999874681231943895909353159368038177490234375f,-0.2015999844312670319368407945148646831512451171875f,-0.108799980458616829537277226336300373077392578125f,1.61900005212426290057692312984727323055267333984375f,0.8384999882578849650371921597979962825775146484375f,-0.4027000568576155625777346358518116176128387451171875f,-1.5984000256270169959549320992664434015750885009765625f,0.1655000433214020016947642943705432116985321044921875f,0.1038999833784994830665482368203811347484588623046875f,0.88830000395029795612344969413243234157562255859375f,0.8526000104635949039533215909614227712154388427734375f,-1.0481000035867094055674897390417754650115966796875f,-0.141499982610344687827819143421947956085205078125f,0.1049000347614283956687586396583355963230133056640625f,0.3757000074937939526620311880833469331264495849609375f,-1.55359999998807918331067412509582936763763427734375f,-1.2584000195175411018766453707939945161342620849609375f,1.32609999920576893117640793207101523876190185546875f,0.41609997573643975332657873877906240522861480712890625f,0.45229997246041897529522657350753434002399444580078125f,-0.20280002114623807418780643274658359587192535400390625f,-1.17760000323653191145467644673772156238555908203125f,0.75710000337064287112553984115947969257831573486328125f,0.628799984891712515633344082743860781192779541015625f,0.336200013245642193027151733986102044582366943359375f,-0.2623000073850152435994687039055861532688140869140625f,0.1783000039502977696059815571061335504055023193359375f,-0.3015999754905696050855112844146788120269775390625f,-0.25789999549314346349859761176048777997493743896484375f,0.6873000060439107272713954444043338298797607421875f,0.0391000072300433654248763559735380113124847412109375f,-1.295899982637166658605565316975116729736328125f,-0.92469999196678343000854738420457579195499420166015625f,1.5839999634623527224874806051957421004772186279296875f,-0.1519999964088205590684310664073564112186431884765625f,-0.851399970582127796348004267201758921146392822265625f,0.67199998247623426994579176607658155262470245361328125f,-0.2843000096797947406912498991005122661590576171875f,0.8230000179708003571477092918939888477325439453125f,-0.522700033891200899205387031543068587779998779296875f,1.115599990808963326571756624616682529449462890625f,0.0883999891191724618266789548215456306934356689453125f,0.1935000222325324870098484097979962825775146484375f,-0.7496999982625236480515695802750997245311737060546875f,1.573400010204314636297340257442556321620941162109375f,0.325799965021013715471553950919769704341888427734375f,-0.266399986898898877285546404891647398471832275390625f,0.870499995097517409448073522071354091167449951171875f,0.4009000032007694613866988220252096652984619140625f,-2.18920000711381401714561434346251189708709716796875f,0.408200001347065022372362363967113196849822998046875f,0.15259998610019653852987175923772156238555908203125f,1.3557000036567445899748918236582539975643157958984375f,0.24179994679689453818127731210552155971527099609375f,0.7917999783128502055973285678192041814327239990234375f,-0.63299999696016318040392434340901672840118408203125f,0.5286000046074388336592164705507457256317138671875f,0.6049999742209914987967067645513452589511871337890625f,0.0786000035643581451694217321346513926982879638671875f,-1.16339998837411418008969121729023754596710205078125f,0.2399999935925005711823132514837197959423065185546875f,1.91999997168779401590654742904007434844970703125f,1.3646000154592095032768384044175036251544952392578125f,-0.68750000700354529925562019343487918376922607421875f,0.64450002621114277356895172488293610513210296630859375f,-0.934499992012977287458852515555918216705322265625f,-0.10029998143464313731243464644649066030979156494140625f,-1.68110002561807636567436929908581078052520751953125f,-1.1224000332526866685611821594648063182830810546875f,-0.99549999666214006310838158242404460906982421875f,-0.2813999817207453180145648730103857815265655517578125f,0.2333999485135080931286211125552654266357421875f,-0.38739999337345343288774301981902681291103363037109375f,-0.1230999690532694756939235958270728588104248046875f,-1.50850000613927814896442214376293122768402099609375f,-1.07669996010065016633916457067243754863739013671875f,-0.2090999707221978809457141323946416378021240234375f,-0.9494999898523091008684104963322170078754425048828125f,0.9791000258564945823991365614347159862518310546875f,-1.680200042310357577690638208878226578235626220703125f,1.7762000585079196213200702914036810398101806640625f,-0.131799939867853321828761181677691638469696044921875f,-0.4087999623537061921751956106163561344146728515625f,0.42830002663731558953941203071735799312591552734375f,0.6493999346852294962673113332130014896392822265625f,-0.3927000301659102632356734829954802989959716796875f,-0.624100039118527849524298289907164871692657470703125f,0.82789996064305260148330489755608141422271728515625f,0.6983000280529261249995442994986660778522491455078125f,-0.1421000079154968176453621708787977695465087890625f,0.759100025334954242595131290727294981479644775390625f,-0.27559999140501023617844111868180334568023681640625f,0.766100040742755172828992726863361895084381103515625f,-0.5778000506877898345692301518283784389495849609375f,-0.33380002151131638044034843915142118930816650390625f,-0.7918999173998830798382186912931501865386962890625f,-1.6158999804019931190168790635652840137481689453125f,1.102199983882905076626457230304367840290069580078125f,0.7742999873839317981349950059666298329830169677734375f,-1.491799996604024958202217021607793867588043212890625f,0.04740000652372822198543644844903610646724700927734375f,-0.1260000153779987641655679908581078052520751953125f,0.192699986021219960008465932332910597324371337890625f,-0.63889995679855271504266056581400334835052490234375f,-0.09199997489526767768808213077136315405368804931640625f,-1.0736000542283061776061003911308944225311279296875f,-0.87800004004314591288249403078225441277027130126953125f,1.003900029814242866876838888856582343578338623046875f,-0.49650002783536917316808967370889149606227874755859375f,-0.5625999981701372831821572617627680301666259765625f,0.1317999618470668821146318805404007434844970703125f,0.662600025364756817225497798062860965728759765625f,-1.40749996833503221438377295271493494510650634765625f,0.7587000270664692180844212998636066913604736328125f,0.16399996837973684904454785282723605632781982421875f,-0.6276999505937101986319248680956661701202392578125f,-1.44079998357295924193977043614722788333892822265625f,1.936800008636712533416357473470270633697509765625f,0.60749995365738929109511445858515799045562744140625f,-0.45890002713203426054633382591418921947479248046875f,-1.23709996082782680559830623678863048553466796875f,0.0118999731302260602916476273094303905963897705078125f,0.8571000151053074223028716005501337349414825439453125f,-0.1359000383302573222721321144490502774715423583984375f,0.598100022101402117868929053656756877899169921875f,-1.39760004242658641970820099231787025928497314453125f,-0.5620000259131197051232220474048517644405364990234375f,0.11810000235736362039773439391865395009517669677734375f,-0.9092000322222710817499091717763803899288177490234375f,0.11010002201199586835400623385794460773468017578125f,-1.019300024282932870534068570123054087162017822265625f,0.50490002369731656894913385258405469357967376708984375f,-0.3922000100344413819897226858302019536495208740234375f,-0.62490002064257854641482481383718550205230712890625f,-0.926900034895539182144830192555673420429229736328125f,0.23490000987648951280561959720216691493988037109375f,0.305000036843121602458950292202644050121307373046875f,-0.3058999836057421095603103822213597595691680908203125f,1.48530003079473971183688263408839702606201171875f,-0.21260000725984529168499648221768438816070556640625f,1.82130005827993191047653454006649553775787353515625f,-0.35539999297857338689254902419634163379669189453125f,0.71710000161975617771759061724878847599029541015625f,-0.3492000098705290156431146897375583648681640625f,1.1610999993175266098432985018007457256317138671875f,0.35650000369548795475793667719699442386627197265625f,-1.133999975606798660265894795884378254413604736328125f,-1.47310002672076212348883927916176617145538330078125f,-0.55939993826150935518626283737830817699432373046875f,0.209100053945184161108272746787406504154205322265625f,-0.55650002858042757480916407075710594654083251953125f,1.033500042796136053624422856955789029598236083984375f,-0.3452000288248058268436579965054988861083984375f,1.2760000776648521281231296597979962825775146484375f,-1.146799982261658357174383127130568027496337890625f,-0.041299998265504900274436295148916542530059814453125f,0.4377999782681474272294508409686386585235595703125f,0.771200017231702616982147446833550930023193359375f,-0.2635999856457111878427213014219887554645538330078125f,-0.8431000063061713234446870046667754650115966796875f,0.026599982568621560741206621969467960298061370849609375f,0.1589000060096379396412658024928532540798187255859375f,-0.6997000163674353956366758211515843868255615234375f,-0.439099985362588984827425520052202045917510986328125f,0.78759997499883116489627354894764721393585205078125f,0.776300021027028552822457641013897955417633056640625f,0.320599996396899289408111144439317286014556884765625f,-0.02739998581111413589184166994527913630008697509765625f,0.7182999977290631665738374067586846649646759033203125f,-2.00720003391355295008224857156164944171905517578125f,-0.547400020307302614952504882239736616611480712890625f,-0.4607000038802624430900323204696178436279296875f,1.7024000336065887584169331603334285318851470947265625f,-0.4196000243812798569109645541175268590450286865234375f,0.0669000018894678749603599499096162617206573486328125f,0.358699998009203824267387972213327884674072265625f,0.5656000203877689092024638739530928432941436767578125f,-0.434900020754337646167186903767287731170654296875f,-0.0988000002026556689571634706226177513599395751953125f,-0.0745000125765802323485331726260483264923095703125f,-0.9173000388264662685600114855333231389522552490234375f,0.10680005885362664486137873609550297260284423828125f,-0.8907000066742296606747686382732354104518890380859375f,-0.92619997059106828896091201386298052966594696044921875f,0.8498999812662599850909828091971576213836669921875f,1.6881999577611690455114512587897479534149169921875f,-0.3342000372886655412685286137275397777557373046875f,0.1972000224769114851142148836515843868255615234375f,-0.41909996416568873200958478264510631561279296875f,-0.509800020200014358806583913974463939666748046875f,0.84509999861717233216040767729282379150390625f,-0.449600016260147672397806672961451113224029541015625f,1.163099967563152148386507178656756877899169921875f,-1.93859997331500100159473731764592230319976806640625f,-0.473499984607100632416631924570538103580474853515625f,0.0451999944776295681236888412968255579471588134765625f,0.68900001092255092061122923041693866252899169921875f,-1.57559998291134828463100348017178475856781005859375f,1.2885999862417583106122265235171653330326080322265625f,1.43690000445991739042028711992315948009490966796875f,-0.00569999095350526285841397111653350293636322021484375f,-1.1270000101551413962397418799810111522674560546875f,0.85900000801682485640498043721891008317470550537109375f,-0.40239999620467437058124460236285813152790069580078125f,0.3629999811649324836793084614328108727931976318359375f,0.47400000492483373992769202232011593878269195556640625f,-1.14320002675354448484767999616451561450958251953125f,0.147300017517804970879069514921866357326507568359375f,-0.315600004331767269860620217514224350452423095703125f,1.084400023868679507899059899500571191310882568359375f,0.649299984906613669721764381392858922481536865234375f,0.02279998568147389992333273767144419252872467041015625f,-0.9543000149697065115361738207866437733173370361328125f,0.1044000307977199781817034818232059478759765625f,0.02619999350160362450168349823798052966594696044921875f,0.27910000827312497762022758251987397670745849609375f,0.4947999874919648544135952761280350387096405029296875f,0.20270000677108779285617856658063828945159912109375f,-1.32669996337890694348971010185778141021728515625f,0.348199992778897016165728928172029554843902587890625f,-0.263899983173609076203547374461777508258819580078125f,0.145899977049231299730536193237639963626861572265625f,-0.26969999379217579615897193434648215770721435546875f,0.6395999838873740461764327847049571573734283447265625f,-0.209699967417121602153429194004274904727935791015625f,-1.13480000075399889425398214370943605899810791015625f,-0.273400000071525095535207583452574908733367919921875f,0.8554000056445598687560050166212022304534912109375f,0.349699956166744652108491209219209849834442138671875f,0.158499996155500699757112670340575277805328369140625f,0.445300013248622672534793309750966727733612060546875f,0.5691999659121032006936502511962316930294036865234375f,0.22550003704428700501694038393907248973846435546875f,-0.4778999757304782658451358656748197972774505615234375f,0.276799970694630970147187554175616241991519927978515625f,-0.762000037554651843407782507711090147495269775390625f,0.5259000145442784823757165213464759290218353271484375f,0.5191999999985099645272157431463710963726043701171875f,-1.0750000109896069577786192894564010202884674072265625f,0.16780002315789477318475064748781733214855194091796875f,-0.0189000231459737211281435520504601299762725830078125f,-0.5584999902695415574527260105242021381855010986328125f,-1.2999000496253376724098416161723434925079345703125f,-0.4485999847888937352990978979505598545074462890625f,-0.779799993303418315093722412711940705776214599609375f,-0.2494000297173861735444688747520558536052703857421875f,0.3538000207476319491917138293501920998096466064453125f,0.4875000127963724505519849117263220250606536865234375f,0.423500035084784176575567471445538103580474853515625f,-0.6895000272542242125695111099048517644405364990234375f,0.074600029112399379727094128611497581005096435546875f,0.2647999809354539824113317081355489790439605712890625f,-0.058400008863210839393786955042742192745208740234375f,0.5439000129759303803211878403089940547943115234375f,-0.3335999893710013264325198178994469344615936279296875f,-1.84590001050233798451927214046008884906768798828125f,-0.9424000052571290808600679156370460987091064453125f,0.38420000402182352683411181715200655162334442138671875f,1.758799958181380684862915586563758552074432373046875f,-1.33180001413151583022909107967279851436614990234375f,-0.293399978613853473774497615522705018520355224609375f,0.5064000077232722052400504253455437719821929931640625f,-0.2640999818235643115116317858337424695491790771484375f,0.9924999952688813298351533376262523233890533447265625f,0.3858999896034596144289707808638922870159149169921875f,0.173299980741739023670788810704834759235382080078125f,-0.293800008919835153875510513898916542530059814453125f,0.948999989688396272669024256174452602863311767578125f,-0.644799999040365445779343644971959292888641357421875f,-0.5790000057071444938827653459156863391399383544921875f,0.08420001016855227504720460274256765842437744140625f,1.681099988067149553216950153000652790069580078125f,-0.64579998849034325747453522126306779682636260986328125f,-0.8119999902993433327225147877470590174198150634765625f,0.62079999243915029438767305691726505756378173828125f,0.5619000019639728282783153190393932163715362548828125f,-0.71179998278319800419922103174030780792236328125f,-0.4307999631583687705216334506985731422901153564453125f,0.32220001577138912551845351117663085460662841796875f,-0.057900030529498991427317378111183643341064453125f,-0.85579999486058933033660878209047950804233551025390625f,0.7468000126600262689180453890003263950347900390625f,-0.6365000036954879814032892682007513940334320068359375f,-0.067200029480456890951245441101491451263427734375f,-1.364899984768032314974561813869513571262359619140625f,0.54680000759363078799424329190514981746673583984375f,-0.213799989995360650851807804428972303867340087890625f,-0.621799993735551392859406405477784574031829833984375f,-0.36640003614723692937360510768485255539417266845703125f,1.05339998695850312060429132543504238128662109375f,0.28370002036094632558160810731351375579833984375f,0.6293999752163887961842192453332245349884033203125f,0.6883000053286547625930325011722743511199951171875f,-0.23829999728202810871380279422737658023834228515625f,-0.916499956309795749120894470252096652984619140625f,1.1096999508023268621315082782530225813388824462890625f,-0.0816999946713448110813260427676141262054443359375f,-1.314199957641959404242015807540155947208404541015625f,-0.211199944216013957287714220001362264156341552734375f,0.446199990633129761619102282566018402576446533203125f,-0.370400020471214563855255619273521006107330322265625f,1.29679996229410132713155689998529851436614990234375f,0.621700020152330612432933776290155947208404541015625f,-0.4265999730318783544902316862135194242000579833984375f,-0.4736000606358050646349511225707828998565673828125f,-0.0641000145688652711584154531010426580905914306640625f,-0.496700001302361915378469348070211708545684814453125f,1.232300024595856857700937325716949999332427978515625f,-0.639599997186660829129323246888816356658935546875f,-0.3181999938219786105975117607158608734607696533203125f,0.203500014632940651182479996350593864917755126953125f,0.4140999739632011955592361118760891258716583251953125f,0.237300015953182619199424152611754834651947021484375f,-1.001099991978704917272580132703296840190887451171875f,0.564200028534234121480039902962744235992431640625f,-0.01180003024339715267387873609550297260284423828125f,1.6902000147804614460511629658867605030536651611328125f,0.826000017166137467938824556767940521240234375f,0.049200014676153980275330468430183827877044677734375f,0.711900010383129000501867267303168773651123046875f,-0.984300031100213690393729848437942564487457275390625f,0.8182000225067138643453290569595992565155029296875f,-1.064100047984719576987799882772378623485565185546875f,-0.5163999937534331508004470379091799259185791015625f,0.9217000414982436939936860653688199818134307861328125f,0.0981000048160556303855628357268869876861572265625f,-0.69090000454187272538320030434988439083099365234375f,1.2740000196397307519191599567420780658721923828125f,0.80429995469450954459489366854541003704071044921875f,1.31530002222657227406443780637346208095550537109375f,-1.43599994117021623196706059388816356658935546875f,1.89330001993179308783510350622236728668212890625f,-0.9761999613523482111077100853435695171356201171875f,1.01459997572898874551583503489382565021514892578125f,-2.1610999862790105652265992830507457256317138671875f,-1.18539994872212428589364208164624869823455810546875f,-0.03289997479915740541400737129151821136474609375f,0.92039998359084140844288413063623011112213134765625f,-1.249400006397067830476999006350524723529815673828125f,0.24560001930743469156226410632370971143245697021484375f,-0.380000018738210432189816856407560408115386962890625f,-1.1374999716505402869159979672986082732677459716796875f,0.525000001564622120753256240277551114559173583984375f,0.0886999473825087481060336358495987951755523681640625f,-0.349099989274143762685298497672192752361297607421875f,1.8130000413656242042037547435029409825801849365234375f,-0.785500003218651432490560182486660778522491455078125f,-0.28120002281963862866831505016307346522808074951171875f,0.5008000394523148823822111808112822473049163818359375f,-0.42519999976754185855298828755621798336505889892578125f,-0.910499987050890968731664543156512081623077392578125f,-1.10199997219443357465706867515109479427337646484375f,-0.231599962675571813264241427532397210597991943359375f,0.316999995291233016558862800593487918376922607421875f,0.9677999825149772306076556560583412647247314453125f,-0.776199973496794815019939051126129925251007080078125f,-1.09589997645318515395729264128021895885467529296875f,-0.7861999708890916593873043893836438655853271484375f,-0.56780000292956867014027011464349925518035888671875f,0.07430002845525696120176917247590608894824981689453125f,0.21930000014305084388155364649719558656215667724609375f,-0.31170000074356796204710917663760483264923095703125f,-0.2886000198438762875952079411945305764675140380859375f,-0.535899987554550261137364941532723605632781982421875f,-0.5194000149466095894723594028619118034839630126953125f,0.7434999931752681590069187222979962825775146484375f,-1.0130000118613242676701702293939888477325439453125f,1.405800005924701334691917509189806878566741943359375f,0.425499997574835919778024617698974907398223876953125f,-0.8680999730765819322186871431767940521240234375f,0.799800024111569474172256377642042934894561767578125f,0.27769998684078434081357045215554535388946533203125f,-1.0222000088982283738658907168428413569927215576171875f,0.53190000276491022912495054697501473128795623779296875f,-0.88060002746582011212694851565174758434295654296875f,-0.722500037550926066387546597979962825775146484375f,0.5027999976396557713087531737983226776123046875f,-0.162400040945410939485782364499755203723907470703125f,0.081699985730647739501364412717521190643310546875f,0.944500001177191794710097383358515799045562744140625f,-0.00750000685453411364278508699499070644378662109375f,0.88150004020333394549879812984727323055267333984375f,0.3517999724268907613122792099602520465850830078125f,-1.548900053227692641399926287704147398471832275390625f,-0.80090003333836856658223268823348917067050933837890625f,0.75480000975057437440085550406365655362606048583984375f,-0.4602000340402128131245262920856475830078125f,0.320899985766410988929919767542742192745208740234375f,0.95479998644888386394313783966936171054840087890625f,0.34230000500082979897342738695442676544189453125f,-0.066700012180209267143027318525128066539764404296875f,0.017699974882602997894309737603180110454559326171875f,0.090199973988533077573492846568115055561065673828125f,-1.05550001528859116461944722686894237995147705078125f,0.3112999698042866469904765835963189601898193359375f,1.6764000241518026701470489570056088268756866455078125f,0.137399990281462824892599883241928182542324066162109375f,-0.467600012456625524492892509442754089832305908203125f,0.23219994301646840195729737388319335877895355224609375f,0.25920002834796918023130274377763271331787109375f,0.8149999961070719667333150937338359653949737548828125f,0.77740002239346495116478763520717620849609375f,1.725900025720149333352537723840214312076568603515625f,0.335000012218952036846530972979962825775146484375f,0.9468000116355719253391498568817041814327239990234375f,0.24710004576444699608828159398399293422698974609375f,1.6523000756323347193443851210759021341800689697265625f,-0.610500009775162055802866234444081783294677734375f,0.835200035176426158756157747120596468448638916015625f,0.7102000276327142813670434406958520412445068359375f,1.281500065460801973671323139569722115993499755859375f,1.073800039467215583499637432396411895751953125f,0.074999965578317784320461214520037174224853515625f,1.557300018486380555060577535186894237995147705078125f,-1.700800050330162349609963712282478809356689453125f,1.049200058671832636036924668587744235992431640625f,0.44209997632503483799837340484373271465301513671875f,0.1553000532209889428258975385688245296478271484375f,-0.91960003890991259822840220294892787933349609375f,1.7949000845313083374321649898774921894073486328125f,2.4468000531911844319665760849602520465850830078125f,0.496300012794137046512332744896411895751953125f,-0.6367999857634296656527794766589067876338958740234375f,-0.516199992644787553075502728461287915706634521484375f,-0.25390001230537950771548594275373034179210662841796875f,-0.00420000092983219275311057572253048419952392578125f,-0.19410002906024459434775053523480892181396484375f,-0.71449997442960722082716529257595539093017578125f,-0.64400000324845318999678056570701301097869873046875f,0.77699994462728483313185279257595539093017578125f,1.1945999804973599367485803668387234210968017578125f,-2.613900008058546831080093397758901119232177734375f,-0.4229999883919948189259230275638401508331298828125f,0.57749997407197906085229988093487918376922607421875f,-0.56790003961920809683050492822076193988323211669921875f,-0.480999995000659819055499610840342938899993896484375f,-1.2276999985381957802843544413917697966098785400390625f,0.74829999398514635000623229643679223954677581787109375f,0.1261000106796625885863250005058944225311279296875f,-0.16779998959302933059234419488348066806793212890625f,2.68790001506954379095759577467106282711029052734375f,-1.759499992832540993248358063283376395702362060546875f,-0.5415000108852978666362787407706491649150848388671875f,1.2783999911680818950543425671639852225780487060546875f,-0.0289999978095298871494378545321524143218994140625f,-0.97000002413988095639751918497495353221893310546875f,-1.510299999651313118675943769630976021289825439453125f,-0.728199970147759056970926394569687545299530029296875f,0.9378999995909629294743581340298987925052642822265625f,0.85250001857057322585120573421590961515903472900390625f,1.5944000083342195761559878519619815051555633544921875f,2.094899994751810101689670773339457809925079345703125f,-1.0736999658226960008278183522634208202362060546875f,-1.34719996529370522608815008425153791904449462890625f,0.3331000013142826343681690559606067836284637451171875f,-0.6998000260487209356341509192134253680706024169921875f,-1.035599972108006028292948030866682529449462890625f,0.267599986845253656753129689604975283145904541015625f,0.6574999856203798120901637958013452589511871337890625f,-0.473599978679419553184715141469496302306652069091796875f,-0.54319997664839103634903949568979442119598388671875f,-0.85239999836534252608544193208217620849609375f,-0.11329999549388858337550800570170395076274871826171875f,-0.953500041007995235986527404747903347015380859375f,0.752499998733402009065684978850185871124267578125f,-0.00279995651245201759849123845924623310565948486328125f,-0.4699999684840445723921220633201301097869873046875f,1.466199990521371177010223618708550930023193359375f,0.1072000280275944295027557018329389393329620361328125f,-1.70529998869896015634139985195361077785491943359375f,1.504200053903460787552148758550174534320831298828125f,-0.31150003826618188895736238919198513031005859375f,0.5282999982133504968118131728260777890682220458984375f,-0.69169997649192804800577505375258624553680419921875f,0.896999999091029120990015144343487918376922607421875f,-0.1219000295870011907339858225896023213863372802734375f,-0.7175000282935799500450002597062848508358001708984375f,0.160500004205852808691901145721203647553920745849609375f,-0.375400010338425682565599572626524604856967926025390625f,-0.7663000016927714597159138065762817859649658203125f,-0.156699965744464986272532769362442195415496826171875f,0.87949995446205075921852767351083457469940185546875f,-1.0957000001549719314652975299395620822906494140625f,0.076799999174475441776621664757840335369110107421875f,1.07510001113414777051957571529783308506011962890625f,0.31309997464120353782845995738171041011810302734375f,-0.8433000247001647409206270822323858737945556640625f,-0.6337999963842328998708808285300619900226593017578125f,0.150400010897219493788412592039094306528568267822265625f,-0.434000026512891157182139068027026951313018798828125f,0.3382000045694411038965654370258562266826629638671875f,0.06740002485215779159943849663250148296356201171875f,-0.6179999993443487937838654033839702606201171875f,-0.701099996933340907645515471813268959522247314453125f,-0.102400033494829845182039207429625093936920166015625f,-0.388599983596801479279747582040727138519287109375f,0.1784000344187024467146329698152840137481689453125f,-0.08660000380277654841165713150985538959503173828125f,0.6677999851971863076727231600671075284481048583984375f,0.1241000248506669567660765096661634743213653564453125f,-0.8203000013276930957317745196633040904998779296875f,0.36740000445619214186621093176654540002346038818359375f,0.34090000700056555249517487027333118021488189697265625f,-0.2993000365018847475084839970804750919342041015625f,-0.055000015869736795792732664267532527446746826171875f,1.072300008130074644441265263594686985015869140625f,-0.547400026286393615038150528562255203723907470703125f,0.177299965587258245847124271676875650882720947265625f,-0.43740003990232967368001482100225985050201416015625f,0.70080001784562995226224302314221858978271484375f,-0.7636000127471980203353041360969655215740203857421875f,1.05349995726346978841547752381302416324615478515625f,-0.68459998969882729369373919325880706310272216796875f,1.7956000033259389425666086026467382907867431640625f,0.67630000083595509519085453575826250016689300537109375f,-1.38459997245073385130353926797397434711456298828125f,0.00059996463879941064334389011492021381855010986328125f,1.0874999798834323883056640625f,1.97160002771914033559141898876987397670745849609375f,-0.04060009407401121706016056123189628124237060546875f,0.22710001700520621170653612352907657623291015625f,0.25629998433292033865882331156171858310699462890625f,-0.89080001188516622079305307124741375446319580078125f,0.25739999355971843186807745951227843761444091796875f,-0.5432000201225282154382512089796364307403564453125f,0.2423999675571917311600600442034192383289337158203125f,-1.0416000550627710907747314195148646831512451171875f,-0.110600028508901271351305695134215056896209716796875f,1.69950001212954493468032524106092751026153564453125f,0.16769995804429005392677254349109716713428497314453125f,0.5802000384360554985363478408544324338436126708984375f,0.3250999528706071117767351097427308559417724609375f,1.155699986818433000479444672237150371074676513671875f,0.30929998285770476940115258912555873394012451171875f,-0.44220006189048266520558172487653791904449462890625f,0.39500000387430223014462171704508364200592041015625f,-0.8682000068604944775785270394408144056797027587890625f,1.293099984550476477807023911736905574798583984375f,0.557899966752529063995780234108678996562957763671875f,-1.092499982714652428938961747917346656322479248046875f,0.85079998506307619177135848076432012021541595458984375f,0.2394999970793723065298763685859739780426025390625f,0.423600016323477268631592096426174975931644439697265625f,-0.9939999785311517399577496689744293689727783203125f,-0.180299981099366568582098580009187571704387664794921875f,0.7056999909535053294717954486259259283542633056640625f,0.26660004742592591231442611388047225773334503173828125f,-0.52410001542568185328718755044974386692047119140625f,-0.5646000053264199625147057304275222122669219970703125f,1.5327999818444251189930582768283784389495849609375f,-0.13340002739653045171763778853346593677997589111328125f,0.484600018942355603712712763808667659759521484375f,0.70049997030571109490892922622151672840118408203125f,-0.5699999645352367139139460050500929355621337890625f,0.8600999858394260400018538348376750946044921875f,-1.028599992556123776665799596230499446392059326171875f,0.521000004444271080927819639327935874462127685546875f,1.1116000398263341519822233749437145888805389404296875f,0.1075000096112486147603704011999070644378662109375f,-1.09150003133714168512824471690692007541656494140625f,0.70740000346899023497826419770717620849609375f,0.612199923607705809303070054738782346248626708984375f,0.556400001800060550749549292959272861480712890625f,0.6041999747782949015117992530576884746551513671875f,1.1677000052809720642699176096357405185699462890625f,-0.23080003453493080911584911518730223178863525390625f,-0.2459999797269707411118133677518926560878753662109375f,0.3756999717123805648810730417608283460140228271484375f,1.040799996164440965884523393469862639904022216796875f,-0.73710002195984092310254709445871412754058837890625f,-0.4659000013008711693629493311163969337940216064453125f,0.55550000381469732957384621840901672840118408203125f,0.661600016692280856744901029742322862148284912109375f,0.1595999986767768774598152958787977695465087890625f,0.5209000059574844243570623802952468395233154296875f,-0.30340004596710201667519868351519107818603515625f,1.35609996869564053412204884807579219341278076171875f,0.046299981427193159788657794706523418426513671875f,0.5994999620430163655981914416770450770854949951171875f,-1.257399971804022964505520576494745910167694091796875f,0.354999975860118777148954904987476766109466552734375f,-0.38170000614523846849124311120249330997467041015625f,0.1703000058911741110279081112821586430072784423828125f,-0.276000051699579085351388130220584571361541748046875f,-0.6347000355526806192330013800528831779956817626953125f,0.22770001550689322744602804959868080914020538330078125f,0.02719998062327466659127139791962690651416778564453125f,-1.68260005367696319211745503707788884639739990234375f,-0.71070004301443756578038346560788340866565704345703125f,0.9935000165328380372642413931316696107387542724609375f,1.006900031095742775733015150763094425201416015625f,0.348699995326996248223849761416204273700714111328125f,-0.9535999464929094671106213354505598545074462890625f,0.859900062477588544851414553704671561717987060546875f,-0.06880002598166523597456034622155129909515380859375f,0.66659996878504745865257063996978104114532470703125f,0.269600001880526018993577963556163012981414794921875f,-1.202400038188696118623965958249755203723907470703125f,0.1683999896407122331964956174488179385662078857421875f,-0.7497999798178671682080675964243710041046142578125f,-0.826799979281425567734231663052923977375030517578125f,-0.4921999829888348276796250502229668200016021728515625f,1.52110002055168092027770398999564349651336669921875f,0.13600004264712362012090807183994911611080169677734375f,0.947899986422061857638254878111183643341064453125f,-0.6801999863564969217577527160756289958953857421875f,1.35889997348785396269477132591418921947479248046875f,0.0519999822527170696417897488572634756565093994140625f,-0.4452999833345412383778239018283784389495849609375f,0.453400004914403353950547170825302600860595703125f,-0.4277999583005911432564971619285643100738525390625f,-0.35590000990629200305903623302583582699298858642578125f,1.09490000726878644599082690547220408916473388671875f,1.297100027771294517009437186061404645442962646484375f,0.39280002088546783767242231988348066806793212890625f,-0.5447000326097015676651835747179575264453887939453125f,0.82979999129176196248636188101954758167266845703125f,-0.64770003716945634408830301254056394100189208984375f,1.558599989110231121003380394540727138519287109375f,-0.732200022625922652963481596088968217372894287109375f,0.9172000184535971101240647840313613414764404296875f,1.36830003200173333510747397667728364467620849609375f,-1.721500001609324925766486558131873607635498046875f,-0.55720002657473111096209095194353722035884857177734375f,0.93480000969469490001984013360925018787384033203125f,-1.28720004777163321563193676411174237728118896484375f,0.712700039032101440028554861783050000667572021484375f,-0.19790002412200013992560343467630445957183837890625f,0.99060000071823639533619143549003638327121734619140625f,-0.75540000653862904300694935955107212066650390625f,-0.59100005173683090031744313819217495620250701904296875f,-0.5701999765217298232755638309754431247711181640625f,-0.454799990099667983400877346866764128208160400390625f,-0.71760003497600610700146717135794460773468017578125f,-0.7762000134319066990684632401098497211933135986328125f,-0.2336999786749476282210480349021963775157928466796875f,0.42240001080781208742109811282716691493988037109375f,-0.57000004295259809605767031825962476432323455810546875f,1.123100019717217179504586965776979923248291015625f,1.2326000200003388584946151240728795528411865234375f,1.904999975562095215764202293939888477325439453125f,-0.532900031498074522318120216368697583675384521484375f,-0.192600005769730131532924133352935314178466796875f,-0.70570001855790642775190235624904744327068328857421875f,-1.77159997698068583105168727342970669269561767578125f,-0.20540000072717656820486809010617434978485107421875f,-0.88369997960627078015249935560859739780426025390625f,0.171500029735267178754298811327316798269748687744140625f,0.896299968835711613479588777408935129642486572265625f,1.67160004001259832051573539501987397670745849609375f,-0.462300024819374044682263047434389591217041015625f,-1.8573000723540786793819279409945011138916015625f,0.495300045919417897266612271778285503387451171875f,-1.01520000751614514200582561898045241832733154296875f,0.668899969315528863944564363919198513031005859375f,1.7252000544548036220504627635818906128406524658203125f,-1.03830001814365413537188942427746951580047607421875f,0.93250001899898016244350174019928090274333953857421875f,-0.3673000001579520557015712256543338298797607421875f,0.2408999931052324683378174086101353168487548828125f,0.2174999993667008357789427464012987911701202392578125f,-1.9721000387608995652044541202485561370849609375f,0.022600018212199923794969436130486428737640380859375f,-1.6711999789357179224680294282734394073486328125f,0.078900048142672485340654020546935498714447021484375f,1.1986000586986538252176615060307085514068603515625f,0.1277999863892791410080462810583412647247314453125f,1.06559999021291762488772292272187769412994384765625f,-0.679200011882186149847484557540155947208404541015625f,0.4355999919638036121938284850330092012882232666015625f,0.29940000505596386926043805942754261195659637451171875f,-0.84700001469999530723242742169531993567943572998046875f,0.28149999438226291914588728104718029499053955078125f,-2.1602999909341331630230342852883040904998779296875f,-0.89170000469237564733049339338322170078754425048828125f,1.00629998572990242422520168474875390529632568359375f,-0.83979999747574307633612988865934312343597412109375f,-1.2413000054925682480444493194227106869220733642578125f,-0.837900011381506271135322094778530299663543701171875f,-0.593200052756071460891007518512196838855743408203125f,-1.5665999643146992781339577049948275089263916015625f,0.73849999134987565074794702013605274260044097900390625f,-0.6929999852254982162236274234601296484470367431640625f,-0.794399997046590033988877621595747768878936767578125f,-0.07210004270970760131831411854363977909088134765625f,0.39439998177289936620582011528313159942626953125f,0.85590002629756967156282598807592876255512237548828125f,0.2996000382579868670518408180214464664459228515625f,-0.710799997375160419466055827797390520572662353515625f,0.18760002380013529599267485536984167993068695068359375f,0.429199956077337407833738325280137360095977783203125f,1.647600039893388323974932063720189034938812255859375f,-1.202699955138564202883344478323124349117279052734375f,0.8289000335395331209298319663503207266330718994140625f,-1.4386999879509201338834145644796080887317657470703125f,-0.2318999939173453928020762759842909872531890869140625f,0.0612999725982550369707269055652432143688201904296875f,1.332200058388710584011960236239247024059295654296875f,-0.29860004457980426906260618125088512897491455078125f,-0.43170001769363874899454458500258624553680419921875f,0.56139997754841974852979546994902193546295166015625f,1.7935000016689297552829884807579219341278076171875f,-0.217400019040704162165411617024801671504974365234375f,0.228499981552362729786409545340575277805328369140625f,-1.7638000329107057506661249135504476726055145263671875f,-0.261000015452504674495770586872822605073451995849609375f,0.356500008761881925778425284079276025295257568359375f,0.6553000029295679151886133695370517671108245849609375f,-1.37420002160519327816246004658751189708709716796875f,-1.029599992418288589846042668796144425868988037109375f,-1.568499987624585312318004071130417287349700927734375f,-0.9475000289641324346945339129888452589511871337890625f,-0.13149998901784398785963503542006947100162506103515625f,0.1595000024437904784235797706060111522674560546875f,-0.3968999591231341383945618872530758380889892578125f,-0.175000018998980433337919748737476766109466552734375f,-1.296799973618984580525648198090493679046630859375f,-0.5880000234097246814002346582128666341304779052734375f,-0.54880005056858038869904703460633754730224609375f,-0.47040002531409275121632163063623011112213134765625f,0.636499988794327009600237943232059478759765625f,1.3974000446707013356473225940135307610034942626953125f,-1.467100002849101247903718103771097958087921142578125f,1.08230005162283848818560727522708475589752197265625f,-0.596700029819458688251643252442590892314910888671875f,0.127199976581334528447086995583958923816680908203125f,-0.3404999808669089134127716533839702606201171875f,0.578300012630224014031909973709844052791595458984375f,-0.0933000133752823757049554842524230480194091796875f,0.990700031447410811580311928992159664630889892578125f,-0.42599998363852531468864981434307992458343505859375f,0.69510007207989854549623487400822341442108154296875f,0.0448999593615528880263809696771204471588134765625f,-1.402600065672398255145481016370467841625213623046875f,-0.86410001382380752232847953564487397670745849609375f,-0.2150000218302015042581842863000929355621337890625f,-1.89160007387548745327876531518995761871337890625f,-0.948500019103288405375451475265435874462127685546875f,-1.11929999381005718106507629272527992725372314453125f,0.84730004016757032303530650096945464611053466796875f,0.80500001341104532315284814103506505489349365234375f,0.14370001961588929617619214695878326892852783203125f,-1.241800017800926969613328765262849628925323486328125f,-1.01150000414252350111610212479718029499053955078125f,-0.745700061622262655447457291302271187305450439453125f,-1.648700044351816274001976125873625278472900390625f,0.0837000227451323297600538353435695171356201171875f,0.71029999648481567664504154890892095863819122314453125f,1.5765999606266618560113101921160705387592315673828125f,0.86449998590350107807722679353901185095310211181640625f,1.1733999573796989324137030052952468395233154296875f,0.383400020411610586990036608767695724964141845703125f,1.7594999400079256446360886911861598491668701171875f,1.008500021934509494059284406830556690692901611328125f,1.68099998676776873907101617078296840190887451171875f,0.85329999392926669798953298595733940601348876953125f,1.8658999549210069091031982679851353168487548828125f,-0.2790999999284744514937983694835565984249114990234375f,-0.82400001889467233695540926419198513031005859375f,-1.60010002827048225526596070267260074615478515625f,0.9038000053435555347647323287674225866794586181640625f,-0.922199996027349921945415189838968217372894287109375f,-0.2080999836191537166740772590856067836284637451171875f,-0.8131999988138669532844460263731889426708221435546875f,-0.21870000858902916007764360983856022357940673828125f,-1.3833000230610366099170960296760313212871551513671875f,0.1015999890506265046496991999447345733642578125f,-1.2565000419914726936809756807633675634860992431640625f,-0.04419996024966277303747119731269776821136474609375f,-1.190699989426134752790176207781769335269927978515625f,0.1707999887138613726023095296113751828670501708984375f,-1.0042999714583149550861662646639160811901092529296875f,0.449200001153350125804308845545165240764617919921875f,-0.7100999784260986213979549575014971196651458740234375f,0.35300001078098952600470283869071863591670989990234375f,-0.8373999970614913745947660572710447013378143310546875f,-0.41659997988641250810104565971414558589458465576171875f,-1.080999985873699387184387887828052043914794921875f,0.57090003531277222492690270883031189441680908203125f,-0.9237999574363222610173806970124132931232452392578125f,0.23660003718137756489880985100171528756618499755859375f,0.2182000329375259894959526718594133853912353515625f,2.0920999885439872656434090458787977695465087890625f,0.2866000058144331230636225882335565984249114990234375f,1.642899965634942560654963017441332340240478515625f,-0.8906000209465620987003831032779999077320098876953125f,0.449300010946392713861996526247821748256683349609375f,0.76330001076757891809165812446735799312591552734375f,-0.92410000588893836948045645840466022491455078125f,-1.1415000329017639391082639122032560408115386962890625f,-0.4670000256896020829344706726260483264923095703125f,0.6660999968588352526666085395845584571361541748046875f,-0.9289999728500841769118778756819665431976318359375f,-1.002400014793872973228872069739736616611480712890625f,0.82140003637075409415047033689916133880615234375f,-1.348000024601816893010663989116437733173370361328125f,0.4150000362843275780733165447600185871124267578125f,0.008899979709088068347000444191507995128631591796875f,0.6540000159516938804671326579409651458263397216796875f,-0.3237000099301337030510694603435695171356201171875f,0.3689000152111052699410720379091799259185791015625f,1.069700055557490259161568246781826019287109375f,-0.40180001400113152243420699960552155971527099609375f,0.1708999996244904284736776389763690531253814697265625f,-0.9450999913156028497951410827226936817169189453125f,1.309100049325824244306204491294920444488525390625f,0.66589998688399720805364268017001450061798095703125f,-0.9706000193819408483619781691231764853000640869140625f,-0.787099984855950562945281490101478993892669677734375f,-0.10859999279826937357285032703657634556293487548828125f,-0.6095000349283223872731696246773935854434967041015625f,-0.256599995458125729186349417432211339473724365234375f,0.9353000278890137852982888944097794592380523681640625f,0.644900017923116575246922366204671561717987060546875f,-1.51240002976954013291788214701227843761444091796875f,-0.339900000265240809227407225989736616611480712890625f,-0.31919998356997947297486462048254907131195068359375f,0.3754000173419715924438833098975010216236114501953125f,-0.7241999662846330121368509935564361512660980224609375f,1.24090004168301870635104933171533048152923583984375f,0.1949000425100322342331082836608402431011199951171875f,0.17149999482929700889144442044198513031005859375f,1.66200001019239351762735168449580669403076171875f,0.6633000251471996566721145427436567842960357666015625f,0.677700011315942152378966056858189404010772705078125f,-0.1306999934494495752002052313400781713426113128662109375f,0.2459000019341706089193166917539201676845550537109375f,0.114000013045966852587298490107059478759765625f,-0.3835999803185463630228468900895677506923675537109375f,0.506100006768107402166378960828296840190887451171875f,-0.283699986796081216056109042256139218807220458984375f,-0.0336000451758493579035302900592796504497528076171875f,-0.7596999701365827473154013205203227698802947998046875f,0.8460999927982693069594688495271839201450347900390625f,0.477100030043721812234025492216460406780242919921875f,0.43280007885098503717102857990539632737636566162109375f,0.51519997942745643371154073975048959255218505859375f,0.9695000435709939523576395004056394100189208984375f,0.049499971449375124166181194595992565155029296875f,0.02959994268566423780697505208081565797328948974609375f,-0.408599986577034446355582986143417656421661376953125f,-0.447600044885276826533981875400058925151824951171875f,-1.498200026641785864711664544302038848400115966796875f,-0.30660001773536205149639499722979962825775146484375f,-0.02399998894333776888743159361183643341064453125f,-0.5306000201642511182598127561504952609539031982421875f,-0.220500009477138547708818805404007434844970703125f,1.291400017818809420333536763791926205158233642578125f,-1.0909999593496326752983804908581078052520751953125f,-0.549500001847743835270421186578460037708282470703125f,0.38750000029802311729554276098497211933135986328125f,0.0515000254511834842929829392232932150363922119140625f,1.130100060456991517554570236825384199619293212890625f,-0.71139999710619405970390971560846082866191864013671875f,-0.29590002601817266825179331135586835443973541259765625f,-0.4493000033281744354241027394891716539859771728515625f,1.1397000692665582022300441167317330837249755859375f,0.227500047422946494179996079765260219573974609375f,-0.7625000202655787262528974679298698902130126953125f,-0.57270000068023818595719376389752142131328582763671875f,-0.0244000601530078853329541743732988834381103515625f,-0.02099998207390318949450147556490264832973480224609375f,0.38820002936124797798811414395458996295928955078125f,0.65160002146065298944677124381996691226959228515625f,-0.838400042539834355892480743932537734508514404296875f,0.62400001600757193376267650819499976933002471923828125f,1.02869999622106522707554177031852304935455322265625f,0.50339998744279146336566554964520037174224853515625f,-1.335400024345517078216971640358678996562957763671875f,0.60479994964301664328587548880022950470447540283203125f,0.033900027206540617186192321241833269596099853515625f,1.4659000208958989919239002119866199791431427001953125f,0.511199992197752361988705160911194980144500732421875f,-0.5607999974310395874255164017085917294025421142578125f,-0.9855999873071912276145667419768869876861572265625f,-0.19999999195337225810220616040169261395931243896484375f,0.728799994391203131982592822168953716754913330078125f,-1.1979999882802367938694487747852690517902374267578125f,0.417299986970424807708468506461940705776214599609375f,-0.18870002417936915239948802991420961916446685791015625f,-1.402100028479099425027243341901339590549468994140625f,0.13010000100135787892696725975838489830493927001953125f,-0.265300031912326483851671810043626464903354644775390625f,0.55190000701174068975518594015738926827907562255859375f,0.35779999667108064453913129909778945147991180419921875f,-1.510100020223855477041752237710170447826385498046875f,-0.63490001598596546017461150768212974071502685546875f,0.8664000000119216338134719990193843841552734375f,-1.3476999826312070496214801096357405185699462890625f,0.683199995610117571942510039662010967731475830078125f,0.053999996319413323675462379469536244869232177734375f,-0.720200006007403192853644213755615055561065673828125f,-0.17810002023875759480375791099504567682743072509765625f,0.37769999326690972818454383741482160985469818115234375f,-0.92619996377378743801500604604370892047882080078125f,-0.233000046394765403245941115528694353997707366943359375f,0.3601999927081169783349423596519045531749725341796875f,-0.1817999467968929305783376548788510262966156005859375f,-0.0725000078976147932507956284098327159881591796875f,-0.763200014311075758399738333537243306636810302734375f,0.1985999803185460876875367830507457256317138671875f,0.27050002570077802399595157112344168126583099365234375f,-0.1161999980837105095332617565873079001903533935546875f,1.132899981486052087831239987281151115894317626953125f,0.016000006195157734367739976732991635799407958984375f,-0.31499996270984464974418415295076556503772735595703125f,0.31989998877271974553337940960773266851902008056640625f,1.1091999986760310292766007478348910808563232421875f,0.042299973261356260678667240426875650882720947265625f,-0.652300094407795683082440518774092197418212890625f,-0.180599962049722506662874366156756877899169921875f,1.7655000167787076037484439439140260219573974609375f,0.237199984255433538038460028474219143390655517578125f,-1.4115999756768344841617590645910240709781646728515625f,-0.1425999932527535207782420911826193332672119140625f,0.2539999963194130572219364694319665431976318359375f,2.204700039613246786984745995141565799713134765625f,0.601300047886372368566298973746597766876220703125f,0.91240003148317327941185794770717620849609375f,-0.486600005888938813569666308467276394367218017578125f,0.32110001935958809582416506600566208362579345703125f,-0.1582000054448837733644950276357121765613555908203125f,-1.05219999490976334755032439716160297393798828125f,0.3219000044226645318445889643044210970401763916015625f,0.44029999473392944508987056906335055828094482421875f,-0.37610002189278624040724707811023108661174774169921875f,-1.052599982821941893007533508352935314178466796875f,1.4735000509358950893101791734807193279266357421875f,-1.2944000029325479772523976862430572509765625f,1.1173000175550578116911992765381000936031341552734375f,-1.67710007705688557422263329499401152133941650390625f,-0.46529999451041159641562217075261287391185760498046875f,0.9385999667584901118289053556509315967559814453125f,-0.098999976817518764438119660553638823330402374267578125f,-0.35299999934434911352809649542905390262603759765625f,-0.049400009489059371059482828059117309749126434326171875f,0.451199996817111870228700354346074163913726806640625f,0.0012999998860059591709870119302649982273578643798828125f,0.4693999998033049347867518008570186793804168701171875f,0.35560002288371350687157246284186840057373046875f,-0.18339998323321271556096689892001450061798095703125f,-0.407799951222538314965504469000734388828277587890625f,1.8377000291973353274244118438218720257282257080078125f,0.6162999572873120168736704727052710950374603271484375f,-0.54689998221993418869857350728125311434268951416015625f,-1.02329998033195668227790520177222788333892822265625f,0.40290000050365926131945570887182839214801788330078125f,0.80860000736415382771582471832516603171825408935546875f,-0.5113999963611364218962762606679461896419525146484375f,0.180300024871527908576496201931149698793888092041015625f,1.5341999899029730158872553147375583648681640625f,-0.71000000931322571062764836824499070644378662109375f,-1.2070000022202720391106822717119939625263214111328125f,0.5273999619320035581182537498534657061100006103515625f,-0.8612999535061425238069432452903129160404205322265625f,-0.2899999727122477555241175650735385715961456298828125f,-0.35589998733103289652035527979023754596710205078125f,0.48780001775622405801868808339349925518035888671875f,0.5146000325024129296025421353988349437713623046875f,-0.3979000186085703827387760611600242555141448974609375f,-0.641400050303340574231469872756861150264739990234375f,0.634300006252527026617826777510344982147216796875f,0.3228999923825259799059494980610907077789306640625f,-0.8334000159226355730623936324263922870159149169921875f,0.4615000007897618328200906034908257424831390380859375f,-0.967999988242983278041720041073858737945556640625f,0.945799969435483500745931451092474162578582763671875f,0.23130004583746222035500750280334614217281341552734375f,-0.2104000217564403474312939579249359667301177978515625f,0.11760000016316751736411561068962328135967254638671875f,0.578600020253658531288465383113361895084381103515625f,-0.7794000313751403563600206325645558536052703857421875f,1.02260002417266360197345420601777732372283935546875f,-1.0464000241331754548212984445854090154170989990234375f,-0.307299978923798011720691647497005760669708251953125f,0.3341999556675556704021801124326884746551513671875f,0.1784999796152113571423569737817160785198211669921875f,0.63949995539337389782730269871535710990428924560546875f,-0.7970000026673071946703430512570776045322418212890625f,-1.96860002239569897852788926684297621250152587890625f,0.496000028267502557355328463017940521240234375f,0.0885000212267044172875785079668276011943817138671875f,-0.30630001294314845861066487486823461949825286865234375f,0.87079998010843928657465085052535869181156158447265625f,-0.75549999643862264520066673867404460906982421875f,0.69469998619258410688104277141974307596683502197265625f,-0.900800019186734690634921207674778997898101806640625f,0.67709999960809985619647477506077848374843597412109375f,-1.268800026931613533776044278056360781192779541015625f,1.002800004941224898402651888318359851837158203125f,0.54349999094009415756545422482304275035858154296875f,1.6775000398606063622963802117737941443920135498046875f,1.141500025451183120139830862171947956085205078125f,-0.4936999694734804933204941335134208202362060546875f,2.214300025996565945973770794807933270931243896484375f,0.77410003554224982025289136799983680248260498046875f,0.51220001867711584253584078396670520305633544921875f,-0.3029999741613862429545633858651854097843170166015625f,-0.18580002760589120480716474048676900565624237060546875f,-0.1087999830104411369546824062126688659191131591796875f,0.08530004020109760443091317938524298369884490966796875f,0.5986000063583254426902158229495398700237274169921875f,0.00350006604194685877473602886311709880828857421875f,1.307700041993708151011333029600791633129119873046875f,-1.0116000172138210899674959364347159862518310546875f,0.99430001953318691843008991781971417367458343505859375f,-0.1531999895751479101591030484996736049652099609375f,0.93270001621469866837088602551375515758991241455078125f,-1.4379000200986862267882315791212022304534912109375f,1.31840002141743894981118501164019107818603515625f,0.8277000064171848858762814415968023240566253662109375f,-0.5422000234454869005418231608928181231021881103515625f,-0.0712000023864209907742406358011066913604736328125f,0.3804000341802833418114460073411464691162109375f,-0.63180002443194371863910419051535427570343017578125f,-0.70309996909052074887114258672227151691913604736328125f,0.448800003927946278281524428166449069976806640625f,0.2423000311896210678952456873957999050617218017578125f,-0.328799965110421144043328922634827904403209686279296875f,-0.7196999730423090380071471372502855956554412841796875f,1.141599993148445957302783426712267100811004638671875f,-0.83079997858107079711231790497549809515476226806640625f,0.00520001405403014427264452024246565997600555419921875f,-0.38269999896660433780226640010369010269641876220703125f,0.74759999219104589229800694738514721393585205078125f,-0.2872999547094099437316572220879606902599334716796875f,1.3263999880909924655725262709893286228179931640625f,0.76499999986961386699846343617537058889865875244140625f,0.6355999721080056730215801508165895938873291015625f,-0.69979999803453718509871350761386565864086151123046875f,-0.40970004823729400111886889135348610579967498779296875f,0.6450000066310168644889699862687848508358001708984375f,-1.02350004234909963685140610323287546634674072265625f,0.38880000519454416263442908530123531818389892578125f,0.03050000273436292985707041225396096706390380859375f,0.9623000520884998909565410940558649599552154541015625f,-1.2728000221520658907792267200420610606670379638671875f,1.257200008860974715929614831111393868923187255859375f,-1.6512999909639365370139785227365791797637939453125f,0.69149997001886343017673652866506017744541168212890625f,-1.23639999554157231642648184788413345813751220703125f,-0.187900019018351904787067496727104298770427703857421875f,-0.01030000462457503207502895747893489897251129150390625f,-0.08380001121088920257307108840905129909515380859375f,-0.25959996485114178454978173249401152133941650390625f,0.686899994830042270876901966403238475322723388671875f,-0.314900002146511870382283859726157970726490020751953125f,-0.70180000893473570844349751496338285505771636962890625f,0.41540003710463668795682679046876728534698486328125f,0.47510006809383664716506245895288884639739990234375f,0.8425000199675556356027072979486547410488128662109375f,0.63190004575475910986170902106096036732196807861328125f,0.7851999743796886921387567781493999063968658447265625f,0.21749999213963755462231119963689707219600677490234375f,0.371199997282772919948712342375074513256549835205078125f,0.01000002045184344989792180058429948985576629638671875f,0.035499991297721411598331542336381971836090087890625f,0.5992000096097580552623185212723910808563232421875f,-0.5003999792695044135371063021011650562286376953125f,0.0447000167399647096999615314416587352752685546875f,1.991300026093423714002028646063990890979766845703125f,0.58430003929585205835195438339724205434322357177734375f,1.6520999818384647372937479303800500929355621337890625f,0.7063000222936268723827879512100480496883392333984375f,1.4140000095069407137771122506819665431976318359375f,0.1210000151544809998682694640592671930789947509765625f,0.8171000035569073727259592487826012074947357177734375f,0.236700021009147665296268314705230295658111572265625f,0.4100000126659875121504228445701301097869873046875f,-0.1395000077337023025592088742996565997600555419921875f,0.84429998289495722385566978118731640279293060302734375f,-0.943900054363906892973545836866833269596099853515625f,-1.3036000121325252987247722558095119893550872802734375f,-1.113000017523765539095847998396493494510650634765625f,-1.78960004233718006361186780850403010845184326171875f,-1.3519000208884477398640910905669443309307098388671875f,-0.742199992078542702955701315659098327159881591796875f,0.0031000504881146806468450449756346642971038818359375f,1.06050001424550988105011128936894237995147705078125f,1.060299991679190867444049217738211154937744140625f,0.846100052738189223333620248013176023960113525390625f,1.763400002008676192843950047972612082958221435546875f,-0.40749999050050977889014802713063545525074005126953125f,-0.0324999825656409058183271554298698902130126953125f,-0.41319999777078653124817719799466431140899658203125f,-0.2974000111430878945384392864070832729339599609375f,-0.061299952220917219847251544706523418426513671875f,-0.08620002289414419038848791387863457202911376953125f,-0.848399924373627101203965139575302600860595703125f,0.6498000119298694698954932391643524169921875f,-0.2436000083327289900125833810307085514068603515625f,0.8389000356256965940104919354780577123165130615234375f,0.71410001289248459244163313996978104114532470703125f,-0.2193999985367061089647222615894861519336700439453125f,0.29749998755753004342494705269928090274333953857421875f,0.113699971038102365383792857755906879901885986328125f,0.39420005174279282300631166435778141021728515625f,0.70820002191066766528138032299466431140899658203125f,-0.9723000096201896980119272484444081783294677734375f,0.18239996263980717827735134051181375980377197265625f,0.058700014400482558585281367413699626922607421875f,0.560700026232004322679358665482141077518463134765625f,-0.9099999833106995339448985760100185871124267578125f,-0.8950000183284283483686749605112709105014801025390625f,-0.39620002214908556226191649329848587512969970703125f,-0.381900036758185290608480499940924346446990966796875f,0.25700000937283007385047994830529205501079559326171875f,-0.533200037109852420513789184042252600193023681640625f,-0.66799999934434872717048392587457783520221710205078125f,-0.78080003051161828153681199182756245136260986328125f,-2.07490003371834763612469032523222267627716064453125f,0.32690001276731539547881766338832676410675048828125f,0.576499992445111075767272268421947956085205078125f,-0.402700038361549328413957482553087174892425537109375f,-0.0946999913334849452439812012016773223876953125f,0.0922999856293194032019755468354560434818267822265625f,0.97900003953278069701582353445701301097869873046875f,0.35700001734495234728683499270118772983551025390625f,0.128599997007846855723300905083306133747100830078125f,-0.368500034824013888989924225825234316289424896240234375f,-1.26439996959120026787104507093317806720733642578125f,0.08579998942166555675470362984924577176570892333984375f,-0.021300010652094858443206248921342194080352783203125f,0.50259999060779814161747935941093601286411285400390625f,-0.116600017772614972244582531857304275035858154296875f,-0.992299994942545282583523658104240894317626953125f,-0.19779998244047192201833240687847137451171875f,-0.516600042247772339720768286497332155704498291015625f,-0.998099930310250016418649465776979923248291015625f,-0.13429999358654054475437078508548438549041748046875f,0.30980000149905695394636495620943605899810791015625f,1.325200044564157852988728336640633642673492431640625f,-1.393700020286441354500084344181232154369354248046875f,0.95900002971664066730994591125636361539363861083984375f,-1.1206999979756773200278985314071178436279296875f,-1.015099999101459093964194835280068218708038330078125f,0.51249998917803141207372163989930413663387298583984375f,1.018400034195185099861191702075302600860595703125f,-0.64190001083016401395298089482821524143218994140625f,0.354099947625396538342101848684251308441162109375f,0.16330005211830211919732391834259033203125f,1.511700068581105060872005196870304644107818603515625f,-1.0947000075757500336948169206152670085430145263671875f,0.35100002603232904885288689911249093711376190185546875f,-1.1482000065997242099768982370733283460140228271484375f,-0.6112999843701716340405027949600480496883392333984375f,-0.8040999894604083930715887618134729564189910888671875f,0.28850004506856186115015816540108062326908111572265625f,1.1005000294633209279027141747064888477325439453125f,0.3415999564170828506348698283545672893524169921875f,-1.87060001696050193231712910346686840057373046875f,0.26370001901984263525946516892872750759124755859375f,-0.311700021269917915134328723070211708545684814453125f,-0.85200005593895955513517037616111338138580322265625f,0.348500044584273638292870600707828998565673828125f,1.637900018534064638942027158918790519237518310546875f,1.255800036025046750864930800162255764007568359375f,0.16630000400245170677493433686322532594203948974609375f,1.35680001832246777127011228003539144992828369140625f,0.051100001105666148504269585828296840190887451171875f,-0.16909999039173140289449293049983680248260498046875f,-0.762699992838502094372188366833142936229705810546875f,-0.2721999883532522090945349191315472126007080078125f,0.3690999965012071726278009009547531604766845703125f,0.02390000030994432478337330394424498081207275390625f,0.42020000475943064532913240327616222202777862548828125f,-0.27769996623992909690059605054557323455810546875f,-0.786600017362832915068793226964771747589111328125f,-0.9301999903798101598795255995355546474456787109375f,-1.1318000525951392720713784001418389379978179931640625f,0.33549998474121078828602549037896096706390380859375f,-0.826500016659498015769713674671947956085205078125f,0.64260000174641618286841548979282379150390625f,1.233400006031989182275765415397472679615020751953125f,-0.45820000775456382058337112539447844028472900390625f,-0.598400007671117339924649058957584202289581298828125f,0.4906000036239621664435617276467382907867431640625f,0.186000007107853804200203740037977695465087890625f,-1.153599999019502941877135526738129556179046630859375f,-2.32540004394054466985153339919634163379669189453125f,-0.66970001241862764373991012689657509326934814453125f,0.0065000284314162826149186003021895885467529296875f,0.72290001719296004267789612640626728534698486328125f,0.563499984085559457724912135745398700237274169921875f,0.41479998928010408665301156361238099634647369384765625f,0.38489997634887718191976091475225985050201416015625f,-0.524400023868679898697564567555673420429229736328125f,-0.5139000024706124047924049591529183089733123779296875f,0.1197999889075755941547640759381465613842010498046875f,-0.0712000046402214703533672945923171937465667724609375f,0.7772999717712405498559746774844825267791748046875f,-1.069199987220763592432604127679951488971710205078125f,-0.745800007340311932324539156979881227016448974609375f,0.600199984959512988780261366628110408782958984375f,-1.420499942552299277309657554724253714084625244140625f,-0.69199998005479568252695798946660943329334259033203125f,0.418700004230439848385003642761148512363433837890625f,0.005000016987323940842458114275359548628330230712890625f,0.481100008481740726296038701548241078853607177734375f,-0.53510003773272085680901000159792602062225341796875f,-0.703500014632940207093270146287977695465087890625f,-0.090900007820129413715903865522705018520355224609375f,0.9157000195264810571416091988794505596160888671875f,0.43770000483393667334297560955747030675411224365234375f,0.106299978893995561435303898178972303867340087890625f,0.808300024327635480148046553949825465679168701171875f,-0.80690000192672017842454579295008443295955657958984375f,0.5345000036731359926989171071909368038177490234375f,0.30599999736621985579887450512615032494068145751953125f,-0.91310006896555540034654541159397922456264495849609375f,1.417800048303604842914182881941087543964385986328125f,0.12640000880360624524456625294988043606281280517578125f,0.440100025215744761197811385500244796276092529296875f,1.025200054901838964127591680153273046016693115234375f,0.7142000231325624781675287522375583648681640625f,1.316600012892484183879560077912174165248870849609375f,-0.5080999735236162440088492076029069721698760986328125f,0.4950999964192508429050576523877680301666259765625f,1.0844999973773952195443825985421426594257354736328125f,0.57439995774477703438520848067128099501132965087890625f,0.2719999927394087801957311967271380126476287841796875f,-0.391300025236606785483672865666449069976806640625f,0.50649999296665182413335060118697583675384521484375f,0.307500016987324098494127611047588288784027099609375f,-0.635200027483702189812220240128226578235626220703125f,0.6563999950572843022911229127203114330768585205078125f,-0.04840000465363247794670087387203238904476165771484375f,-1.033400018753856119957390546915121376514434814453125f,-0.22439997275769762996588951864396221935749053955078125f,0.211100001925230262855848195613361895084381103515625f,0.5945000125765798060228917165659368038177490234375f,0.873300015908479831949762228759936988353729248046875f,1.269700029778480399045292870141565799713134765625f,0.647100019836425843777760746888816356658935546875f,1.524699998933076727780644432641565799713134765625f,0.50600001321733023562643438708619214594364166259765625f,0.77440000934004782351394169381819665431976318359375f,0.9535000334084038353665846443618647754192352294921875f,0.50129997732937325860547161937574855983257293701171875f,1.202599999418108911442004682612605392932891845703125f,0.579600040493160495458369041443802416324615478515625f,-0.192400031632185442020954724284820258617401123046875f,0.8686000521421437525759756681509315967559814453125f,-0.510400005346536289607684011571109294891357421875f,0.5422999988913534963330675964243710041046142578125f,-1.9010000958442698770767265159520320594310760498046875f,0.977700065630674686190104694105684757232666015625f,-0.05739999281466001690432676696218550205230712890625f,-0.513900016924738967105668052681721746921539306640625f,-1.3974000563681130415716324932873249053955078125f,-0.478099993230402364741138399040210060775279998779296875f,-0.598900015136599517262538938666693866252899169921875f,-0.982800032098591458407099707983434200286865234375f,-2.049000039011240392738955051754601299762725830078125f,-0.85920004205703737198973612976260483264923095703125f,-1.84200004133582151411019367515109479427337646484375f,0.711699973958730414125284369220025837421417236328125f,-1.191800027337670098148691977257840335369110107421875f,-0.386699990275502170078425479005090892314910888671875f,-0.4713999942749740323932883256929926574230194091796875f,0.57270003016591086719699887908063828945159912109375f,-1.014400008147954945769697587820701301097869873046875f,-0.2966999745175247227280124207027256488800048828125f,0.279199995472282136432795596192590892314910888671875f,0.8077000184312461694702278691693209111690521240234375f,0.2207999999642371591335177072323858737945556640625f,1.336700002717972068921881145797669887542724609375f,0.7956999630510803722671653304132632911205291748046875f,0.43329999076277003755564010134548880159854888916015625f,-0.9799000052385034020829834844334982335567474365234375f,0.08309996316731049770254458053386770188808441162109375f,-0.45010001865923410235836854553781449794769287109375f,-0.42209999718666058043936573085375130176544189453125f,1.50389999159276488427394724567420780658721923828125f,-0.3959999902322888232220066129229962825775146484375f,0.260000024940818974528156104497611522674560546875f,-0.93420001149103060189560210346826352179050445556640625f,-0.83710006200671305265359478653408586978912353515625f,-0.300199956107139342975642648525536060333251953125f,-0.35769997564628663866415081429295241832733154296875f,-0.71110000594854361199992354158894158899784088134765625f,-0.1631999811001112465902451731381006538867950439453125f,0.515499966580420743156309981714002788066864013671875f,1.16050002303719512752877562888897955417633056640625f,-0.2828999886572365340953183476813137531280517578125f,-0.6799999916553491896564764829236082732677459716796875f,-0.227100003501027547514468096778728067874908447265625f,-0.946300034233182874032763720606453716754913330078125f,-1.046700032334030083802645094692707061767578125f,-1.28159999650120681735643302090466022491455078125f,-1.54089998092353308578594806022010743618011474609375f,-0.73429995670616621250559319378226064145565032958984375f,0.165799990986287060223958178539760410785675048828125f,0.59070000604093042539943780866451561450958251953125f,-0.92930002640634778998673937167041003704071044921875f,-0.580399983590840662373011582531034946441650390625f,-0.82230000216960863479442878087866120040416717529296875f,-1.57500000536441842058366091805510222911834716796875f,-0.22119996463060420666124628041870892047882080078125f,0.77690002725869444777373473698389716446399688720703125f,1.6162999930873509502049500952125526964664459228515625f,0.4058999996989964831328734362614341080188751220703125f,1.96809997732341290799240596243180334568023681640625f,0.6112000102281569535733751763473264873027801513671875f,1.2187999810546636947350407353951595723628997802734375f,0.07550002329796556832519627278088591992855072021484375f,1.0830999841034414377105576932081021368503570556640625f,-0.2633999911308284591626716064638458192348480224609375f,0.5480000143200156603739969796151854097843170166015625f,0.56939999127239016019075279473327100276947021484375f,1.005199981662630914769351875293068587779998779296875f,-2.22090001128464908930482124560512602329254150390625f,0.10570000127255951838378678075969219207763671875f,1.5439000407665977032678483737981878221035003662109375f,-0.828099993193149419568044322659261524677276611328125f,-1.4111000153176476157312890791217796504497528076171875f,0.239300033428519853817562079711933620274066925048828125f,-0.35150000116229096480680027525522746145725250244140625f,0.7786999987356366670354645975749008357524871826171875f,-0.77820002211555927562613987902295775711536407470703125f,-0.8646000027187168424092078566900454461574554443359375f,0.624200007784365684671001872629858553409576416015625f,0.774600013428926814640362863428890705108642578125f,-0.1528000037491319407223500093095935881137847900390625f,0.771400042107701633398164631216786801815032958984375f,-0.5271000045627352026400558315799571573734283447265625f,0.814500042080879893546807579696178436279296875f,-0.699600002029537648695622920058667659759521484375f,0.35799999397993165217712885350920259952545166015625f,0.7629999851137394539790648195776157081127166748046875f,0.766600010359287153960394789464771747589111328125f,-0.60200001063943009427248398424126207828521728515625f,-0.2243000558733942995104371220804750919342041015625f,-0.09549999869242331573815363299218006432056427001953125f,-0.895900024360418267832528727012686431407928466796875f,0.129900008311867820598450862235040403902530670166015625f,0.212999987497925591384273502626456320285797119140625f,0.5559000368215147869221937071415595710277557373046875f,-0.710600025975704596703508286736905574798583984375f,0.8549000272922218091053991884109564125537872314453125f,-1.37020002137422647336961745168082416057586669921875f,-0.011400048720092537113401931492262519896030426025390625f,-0.0194999860525140178424408077262341976165771484375f,0.20039998159781113873378899370436556637287139892578125f,-0.673700005459785256078930615331046283245086669921875f,-0.28950002529844665044578277957043610513210296630859375f,0.9727000066965818536601773303118534386157989501953125f,0.16119999935030959736792510739178396761417388916015625f,-0.1990999913603064275235965396859683096408843994140625f,-0.57070000458806691145952072474756278097629547119140625f,0.1936000256180765433100532391108572483062744140625f,-0.50810001126080717170907519175671041011810302734375f,1.15660004127919702199278617626987397670745849609375f,1.72960005816966155833824814180843532085418701171875f,-0.384500019505619849979893842828460037708282470703125f,-1.3119000221923002857948858945746906101703643798828125f,0.8537000009149309232014957160572521388530731201171875f,0.01309998736306983158073080630856566131114959716796875f,0.574100000524520481803847360424697399139404296875f,0.269200039151311187879400677047669887542724609375f,1.0313000154763460880502634609001688659191131591796875f,0.676899997195601788035901336115784943103790283203125f,-1.442299989950657224113683696486987173557281494140625f,-0.245600021430850024017900068429298698902130126953125f,-0.0971999680876738381130053312517702579498291015625f,-1.142999994762241566803595560486428439617156982421875f,-0.89149999194219742282285778856021352112293243408203125f,-0.59760000906661125963381664405460469424724578857421875f,-0.27269996819570696544587917742319405078887939453125f,-1.3506000150978569873672086032456718385219573974609375f,-1.813300003540515348277040175162255764007568359375f,0.5210999971009793174658852876746095716953277587890625f,-0.2053000318378208621794556165696121752262115478515625f,-1.0142000087529421392673611990176141262054443359375f,1.13899997366964811362777254544198513031005859375f,-1.146200008719414409341652572038583457469940185546875f,2.0441000190019611437719504465349018573760986328125f,-1.1703999987713988861770531002548523247241973876953125f,0.9309999543949967204525819397531449794769287109375f,-0.1123999864816671578893192418036051094532012939453125f,-0.3058999798804518110983963197213597595691680908203125f,0.4464000038862232511149841229780577123165130615234375f,1.2676000036835670670143372262828052043914794921875f,0.408300021161139614633839300950057804584503173828125f,0.54520000550448888265719915580120868980884552001953125f,0.06399996860325363190469261098769493401050567626953125f,-0.1522000065326689632883017111453227698802947998046875f,0.5417999831929802834196152616641484200954437255859375f,-1.862300008428096731449841172434389591217041015625f,0.30869993156865327410542931829695589840412139892578125f,-0.284399986708909591204275102427345700562000274658203125f,-0.9440999748200178398604975882335565984249114990234375f,0.0272999855734408214402719750069081783294677734375f,-0.18879999945759762258745695362449623644351959228515625f,0.990800003167987330954247227055020630359649658203125f,1.3547000031054012847420153775601647794246673583984375f,0.2296000155895949745854522916488349437713623046875f,-0.88670001061558689769981356221251189708709716796875f,1.00479999486803972530424289288930594921112060546875f,-0.9645000058710575530085407081060111522674560546875f,0.2705999599635600105784760671667754650115966796875f,1.0362000219255680377727912855334579944610595703125f,-1.7202000598192217495352451805956661701202392578125f,-0.460700036066770568510264638462103903293609619140625f,0.4586000488638877214953026850707828998565673828125f,-0.423000012978911765770817510201595723628997802734375f,-0.06289993858933495829433013568632304668426513671875f,0.3986999783210458669913123230799101293087005615234375f,-0.56310003581047052279018316767178475856781005859375f,0.202300032530725371060498218866996467113494873046875f,0.4707999856591218446055790991522371768951416015625f,0.4141999796956767898592488563735969364643096923828125f,-1.08999996602535276224443805404007434844970703125f,-0.62460003307983225084853984299115836620330810546875f,-0.2984000382184976984945024014450609683990478515625f,-0.35409999042898443466498292764299549162387847900390625f,0.8838000562310224239581657457165420055389404296875f,0.08229999950602684410938536530011333525180816650390625f,-0.466099980393052060190228758074226789176464080810546875f,0.25310003309100814750109975648229010403156280517578125f,0.938799970027804686623085217433981597423553466796875f,-0.3381000393122430836001512943767011165618896484375f,-0.89269993509650225060880757155246101319789886474609375f,0.682100046360492040520284717786125838756561279296875f,0.5375999996602534025669228867627680301666259765625f,1.00109996187835914582819896168075501918792724609375f,0.916299972710012955445790794328786432743072509765625f,0.4993000303179024879085545762791298329830169677734375f,-1.5873000251919027192570865736342966556549072265625f,0.2376000111714005047502951128990389406681060791015625f,-1.848899977064132116311157005839049816131591796875f,-0.37940002960562768663521637790836393833160400390625f,-1.0314000462055208373612913419492542743682861328125f,0.4935000059530134652874266976141370832920074462890625f,0.4078999716512863304984648493700660765171051025390625f,-0.42960000029727873904761281664832495152950286865234375f,0.615199928428232123422958466107957065105438232421875f,0.734300024655461403000344944302923977375030517578125f,0.0143000618338593454836882301606237888336181640625f,-0.631700021046400106428109211265109479427337646484375f,-0.395799988155066895245681735104881227016448974609375f,-0.87570003066510027078805933342664502561092376708984375f,1.3496000102996816849554306827485561370849609375f,-0.6085000284910204726429583388380706310272216796875f,1.12970007209777900669678274425677955150604248046875f,0.29809998678564986196448671762482263147830963134765625f,0.2134000867031515813465603059739805757999420166015625f,0.48959998429715645418269787114695645868778228759765625f,-0.6578999799214300114869047320098616182804107666015625f,-0.32620001438185564079930145453545264899730682373046875f,-0.1674000141978257971686616656370460987091064453125f,-0.44650001909956404944068708573468029499053955078125f,0.19849997853487633125979527903837151825428009033203125f,-1.10340004062131047390948879183270037174224853515625f,-0.82360000497996832535818612086586654186248779296875f,-0.9699000480234634125054071773774921894073486328125f,-0.347100023412704405956219488871283829212188720703125f,0.08700004178285547506987995802774094045162200927734375f,-0.99230003282874867931440121537889353930950164794921875f,0.93319997934922493953990851878188550472259521484375f,0.1805000167414545497734934542677365243434906005859375f,1.57590003900080954934992405469529330730438232421875f,-0.057300056484341777007784912711940705776214599609375f,-0.849100019784272408429615097702480852603912353515625f,0.39200001488626001755477545884787105023860931396484375f,0.22539994656145501128463592976913787424564361572265625f,-0.8916000041753042726355715785757638514041900634765625f,1.508099991479515455949922397849150002002716064453125f,-0.911200023043155393764891414321027696132659912109375f,0.303999963760375191412776985089294612407684326171875f,-0.363600005725026154124179811333306133747100830078125f,0.60029999831020841583040237310342490673065185546875f,-0.534799982798099815539671908481977880001068115234375f,-0.2365000165477388893719989937380887567996978759765625f,-1.6050000061094760184232654864899814128875732421875f,0.1549999888241291312596104035037569701671600341796875f,-0.0128999756559726730387183124548755586147308349609375f,-0.927800025430321806396705142105929553508758544921875f,0.5297000477343789537343354822951368987560272216796875f,-0.58070005327761220481619375277659855782985687255859375f,0.64490002157390069470466187340207397937774658203125f,-0.30920000694617610914605165817192755639553070068359375f,0.966100047224760505315543923643417656421661376953125f,-0.77500000737607432910181159968487918376922607421875f,1.394199961665272713418062267010100185871124267578125f,-1.22019999263361089703039397136308252811431884765625f,1.6633000124812122866302388501935638487339019775390625f,-0.7902000192321840810194544246769510209560394287109375f,1.2156000364437697935926507852855138480663299560546875f,-1.153300002665071932739238036447204649448394775390625f,-0.5451999903053048601719865473569370806217193603515625f,1.0170000001341090722917215316556394100189208984375f,-0.1251000369131565204128264667815528810024261474609375f,-0.8634000128492720360640078069991432130336761474609375f,-0.82890002549290642264168127439916133880615234375f,0.379699967640638735844049733714200556278228759765625f,-0.4427999958515165257466605908120982348918914794921875f,-0.170500046208500233291260883561335504055023193359375f,-1.02969998846501109568407628103159368038177490234375f,1.26380000664740776983308023773133754730224609375f,-1.5309000214919450399975175969302654266357421875f,0.4719999073371294873169290440273471176624298095703125f,-0.436300025832654370105956331826746463775634765625f,0.462000037536025587314725271426141262054443359375f,0.85699999395012849134900534409098327159881591796875f,-0.59210001722872274143583126715384423732757568359375f,0.11270001064538970325656919158063828945159912109375f,-0.555600011596083742659857307444326579570770263671875f,-0.3842000410139565502731784363277256488800048828125f,0.74790002307891878530199392116628587245941162109375f,-0.2916999689295887154827369158738292753696441650390625f,-0.4455000176355243812764683752902783453464508056640625f,-0.193599996598064816311079994193278253078460693359375f,-0.89490002142489000203084970053168945014476776123046875f,-1.40300006252527253991502220742404460906982421875f,0.194200008622556818949078660807572305202484130859375f,0.18279998750686576869384225574322044849395751953125f,-0.0645000130049886433170058808173052966594696044921875f,-0.44060000913739205685715205618180334568023681640625f,0.27530002075508275272142100220662541687488555908203125f,-1.04700003045797362943858388462103903293609619140625f,-0.341400014521926753463532122623291797935962677001953125f,0.172099971780180993619069340638816356658935546875f,-0.45640001012608444330709289715741761028766632080078125f,0.68010001947879761274862175923772156238555908203125f,-0.5837000279046595796472729489323683083057403564453125f,1.64740003468692375321325016557238996028900146484375f,1.25230004899650904093277858919464051723480224609375f,-0.93189997849464478729686334190773777663707733154296875f,0.766500017590820537094487008289434015750885009765625f,0.295199990677833756791414998588152229785919189453125f,-0.3373000259742140460872406038106419146060943603515625f,0.67160003248751165561003517723293043673038482666015625f,0.96460002043247261571679018743452616035938262939453125f,0.2466999783918261412996031367583782412111759185791015625f,-0.3763000032946470785333303865627385675907135009765625f,-0.6165000014230610503318530390970408916473388671875f,0.789899977876245440455704738269560039043426513671875f,0.57039996719956409521046225563623011112213134765625f,0.266999987617134504347404799773357808589935302734375f,-1.560700065124034807695352355949580669403076171875f,-0.28980003174841417745710714370943605899810791015625f,-0.89220002381801588597909358213655650615692138671875f,0.70600002430006914000415463306126184761524200439453125f,0.9698000578254468706518309772945940494537353515625f,-0.44120000020712579757997673368663527071475982666015625f,0.77050001638755183375195656481082551181316375732421875f,-0.17079997716546058139641672823927365243434906005859375f,1.023099999824166328465935293934307992458343505859375f,0.03260003333687766513548922375775873661041259765625f,1.850300031912325948724173940718173980712890625f,-0.06580007119178876706655501038767397403717041015625f,-0.32500006288290084199843477108515799045562744140625f,-0.632300028246640888340834862901829183101654052734375f,1.377600008079409743544374578050337731838226318359375f,0.534499979421496629328203198383562266826629638671875f,0.442600044326484809431576650240458548069000244140625f,1.187300055888295791106656906777061522006988525390625f,1.33620003827959354936183444806374609470367431640625f,0.06839999731481072320349312576581723988056182861328125f,0.332699986021220528442654540413059294223785400390625f,1.067400048395991785099568005534820258617401123046875f,1.035599987977743108302774999174289405345916748046875f,0.435000022128224639317295441287569701671600341796875f,0.313500008225440662812388836755417287349700927734375f,0.5957000342786316071652663595159538090229034423828125f,0.4567000116586683589048334397375583648681640625f,-0.782499996945261511172020618687383830547332763671875f,1.095700006115436053732992149889469146728515625f,1.4376999759256836597387518850155174732208251953125f,-0.34069999508857762293700943700969219207763671875f,-0.371100013250112770180066945613361895084381103515625f,0.232199938732386090833870184724219143390655517578125f,-1.03330002477467086663409645552746951580047607421875f,0.359199979323148621546124559245072305202484130859375f,-0.786300041255355353086997638456523418426513671875f,0.288100006380677342576746013946831226348876953125f,-0.047000004976987863614112939103506505489349365234375f,-0.94930002577304861688389792107045650482177734375f,-0.256900040781498528730253383400849997997283935546875f,0.0746999943137165445250502671115100383758544921875f,-0.64780002163052596841907870839349925518035888671875f,-0.7130999770998958098289222107268869876861572265625f,0.413500080868603792083604275831021368503570556640625f,-0.549400064325333392645234198425896465778350830078125f,-0.230900010707229430551024051965214312076568603515625f,0.44820001945197607273740914024529047310352325439453125f,-0.030800002795457803728140788734890520572662353515625f,0.473600021222234268680750801649992354214191436767578125f,-0.2094000108301639073715705308131873607635498046875f,-0.0558999646067610544974968433962203562259674072265625f,-0.202499973624944207273301799432374536991119384765625f,-1.4919999912679184905783813519519753754138946533203125f,-0.67739993715882196312350060907192528247833251953125f,0.729600037941337209446146516711451113224029541015625f,0.922700039851665376744449531543068587779998779296875f,0.214900003916024928685146733187139034271240234375f,0.53640001826584338129322304666857235133647918701171875f,1.100299989891052376833613379858434200286865234375f,0.00940001559853553825263361432007513940334320068359375f,1.038599994102120671612965452368371188640594482421875f,-0.7249000115156174484098983157309703528881072998046875f,-0.94709997856020944340116329840384423732757568359375f,0.7482000057429072370496214716695249080657958984375f,-1.237599993252754160266704275272786617279052734375f,-0.34120000344812873027677824211423285305500030517578125f,-1.186699992696940597625143709592521190643310546875f,0.374100005721300643557469811639748513698577880859375f,-0.4823999922558666053618026126059703528881072998046875f,-0.9095999954730273895364689451525919139385223388671875f,-0.9782000156521792888497657258994877338409423828125f,-1.65630005474090591377489545266143977642059326171875f,1.400100001448393083336441122810356318950653076171875f,0.427400001978873955721383026684634387493133544921875f,0.955599995353817899257364842924289405345916748046875f,0.090500003367662174014185438863933086395263671875f,-0.0660999689936634382547708810307085514068603515625f,0.366500033684075188222806218618643470108509063720703125f,-0.617900015218555864748850581236183643341064453125f,1.033900009232012973114933629403822124004364013671875f,-1.61439997624084252692000518436543643474578857421875f,-0.33359999276101515164327793172560632228851318359375f,-0.235200012135506231203407878638245165348052978515625f,-0.16670000391006489781631216828827746212482452392578125f,0.418599998944997775396359429578296840190887451171875f,1.0749000048100949555873739882372319698333740234375f,1.023999982684850262870668302639387547969818115234375f,0.4697999637246130788525988464243710041046142578125f,1.1507000004529945869080620468594133853912353515625f,1.92629999016299802150342657114379107952117919921875f,0.441500026959924962710601903381757438182830810546875f,0.6092999781824648408701250446029007434844970703125f,0.31050001964718043634405830744071863591670989990234375f,0.1881999906182283410771560738794505596160888671875f,0.9150999674737452238559853867627680301666259765625f,0.26240001039803029225794261947157792747020721435546875f,0.49119999249577528388499558786861598491668701171875f,0.3991999996632336067392543554888106882572174072265625f,0.3817999992489813365637019160203635692596435546875f,-0.2732000285416844587160767332534305751323699951171875f,0.332099984967708650174245121888816356658935546875f,-0.767300043520331431778913611196912825107574462890625f,-0.903199992108344673624742426909506320953369140625f,1.00070002444386485507266115746460855007171630859375f,-0.6732999835729600590639165602624416351318359375f,0.3864999952018255413577207946218550205230712890625f,0.1223000413596633251245293649844825267791748046875f,-0.7771999973684540208296311902813613414764404296875f,0.753699938479065689733715771581046283245086669921875f,-0.41690000248551317696410478674806654453277587890625f,0.0182000114798537282467805198393762111663818359375f,0.00559997426867386849380636704154312610626220703125f,-0.90289996362328484025283614755608141422271728515625f,0.727000079780818708741207956336438655853271484375f,-0.01600005114078584256276371888816356658935546875f,0.37599997887015224051765471813268959522247314453125f,-0.987999956727026784619738464243710041046142578125f,-0.24709996559619806788532514474354684352874755859375f,-0.8851999983519309722623802372254431247711181640625f,-0.1339999654740096080018929569632746279239654541015625f,1.2156999928534037991312288795597851276397705078125f,-0.44540001816153473868098444654606282711029052734375f,-0.356800006103515698896444519050419330596923828125f,-0.300999970972537500557564271730370819568634033203125f,-1.8840000000447034711470450929482467472553253173828125f,0.68290004125833458914485163404606282711029052734375f,0.035599988648294722537457346334122121334075927734375f,-0.65000005498528512504208265454508364200592041015625f,-0.187900002887845030130620216368697583675384521484375f,0.74900002217292804829185115522705018520355224609375f,-1.77519996802806900859650340862572193145751953125f,-1.423600022861361225068321800790727138519287109375f,-0.18280004025697760283719617291353642940521240234375f,0.561800009884685369598855686490423977375030517578125f,0.07029997661039277812022874059039168059825897216796875f,-0.136100052067637644626074688858352601528167724609375f,0.57769998766034813542091796989552676677703857421875f,1.208600032994151529663895416888408362865447998046875f,-0.851299967122077561043624882586300373077392578125f,-0.12150000291317708533966879258514381945133209228515625f,-0.3877000231996181600635509312269277870655059814453125f,-0.5039000040352343834371140474104322493076324462890625f,-0.509799978923798136065670405514538288116455078125f,0.3560999976038929926147602600394748151302337646484375f,1.50620004122257267198392582940869033336639404296875f,0.365200016009807892913840987603180110454559326171875f,-0.11849996780604110657719729715608991682529449462890625f,-0.54239999132454330688091204137890599668025970458984375f,0.00550003207102378599557823690702207386493682861328125f,0.01940000412464115697730449028313159942626953125f,-1.299000017404555507027907879091799259185791015625f,0.188500006288289778666467100265435874462127685546875f,-0.4383000211983925087366742445738054811954498291015625f,-0.0932999785810701975918846073909662663936614990234375f,1.249100051337480810076385751017369329929351806640625f,0.7915000111460681608832601341418921947479248046875f,-0.212800031539798428781296024681068956851959228515625f,-1.599900028800964602027079308754764497280120849609375f,-0.60190000416189415322065769942128099501132965087890625f,-0.8303999718934296225825164583511650562286376953125f,-1.5824000240884730228430044007836841046810150146484375f,-0.03020001228451807406827356317080557346343994140625f,0.866800060939789229763619005098007619380950927734375f,1.6518000604182490054228082954068668186664581298828125f,1.15580005293786580722326107206754386425018310546875f,1.7229000379055736669187126608449034392833709716796875f,-0.329000004515051625020305436919443309307098388671875f,-0.090800029468536269661171900224871933460235595703125f,-0.6934999994337562867485758033581078052520751953125f,-0.104799987491965229668977599430945701897144317626953125f,-0.274400026476383285256588351330719888210296630859375f,-0.39959999636709697679037844864069484174251556396484375f,-0.435499990254640589881773848901502788066864013671875f,-1.116800001931190333692711647017858922481536865234375f,-1.1530999850720167199114030154305510222911834716796875f,1.031900004534423320734504159190692007541656494140625f,-1.1706999836146831928118672294658608734607696533203125f,-1.060900014898180643285741098225116729736328125f,0.139400029307604089723326978855766355991363525390625f,0.068100006902218179760666316724382340908050537109375f,0.847700004759430303380440818727947771549224853515625f,0.49930002987086841148567373238620348274707794189453125f,0.584099986368417756210646984982304275035858154296875f,-0.556600013488530720451308297924697399139404296875f,1.624299981814622650944102133507840335369110107421875f,1.00929999906271650189637512085027992725372314453125f,0.7912000191688537409362425023573450744152069091796875f,0.2714999994859095266974691185168921947479248046875f,-0.73989996319860207307783639407716691493988037109375f,0.57280003802180345928718452341854572296142578125f,-1.276199994134902038211976105230860412120819091796875f,-0.079099938535689862106892178417183458805084228515625f,0.8216000092417006239742249817936681210994720458984375f,-0.12340001698434388544001194532029330730438232421875f,0.52769997063577189777561216033063828945159912109375f,0.7033999967187642976540473682689480483531951904296875f,1.1881999759405850181082087146933190524578094482421875f,0.7735999737620355976019936861121095716953277587890625f,-0.265100025141239203918530620285309851169586181640625f,-0.306600009465217482329535414464771747589111328125f,0.44430003612935553558571655230480246245861053466796875f,1.9280000154376022436508719692938029766082763671875f,0.85450001227855665320021216757595539093017578125f,1.069400008296966131382532694260589778423309326171875f,-0.04249998569488422361928314785473048686981201171875f,0.1575000004470348269336454904987476766109466552734375f,-0.18630000020265580218392642564140260219573974609375f,-0.37270002711117289351250292384065687656402587890625f,0.0323000045537948921037241234444081783294677734375f,0.169300002899765988928493243292905390262603759765625f,-0.3824999742954968695585193927399814128875732421875f,0.176600006745755544468323705586954019963741302490234375f,-0.530900018623471137146907494752667844295501708984375f,0.164499995440244628497339363093487918376922607421875f,0.443999960184097375304190791212022304534912109375f,-0.3614999965429299688679520841105841100215911865234375f,-0.33719999550581025715700889122672379016876220703125f,-0.1134000088632109992659025010652840137481689453125f,-0.86210001357793775156324045383371412754058837890625f,0.8157000537991532240766900940798223018646240234375f,-1.1203000354886061273873565369285643100738525390625f,-0.37430002554953156757022725287242792546749114990234375f,0.28070000291168728523416575626470148563385009765625f,0.3961000145539641525971319424570538103580474853515625f,-0.3937999998673797907855487210326828062534332275390625f,-0.2907999874472613210940608041710220277309417724609375f,-1.07020009200572996377331946860067546367645263671875f,-1.4409000364303583552327836514450609683990478515625f,-0.3217000043570994449737554532475769519805908203125f,0.354800021541118670853620642446912825107574462890625f,-0.46189998311400370933910153325996361672878265380859375f,0.055100012922287167071999647305347025394439697265625f,-1.154600021326542158561778705916367471218109130859375f,0.3425000061467289302896688241162337362766265869140625f,0.81750001575797792696675969636999070644378662109375f,0.51720000388771325550152369032730348408222198486328125f,0.4463000191271306871243496061651967465877532958984375f,0.871800027784705111599805604782886803150177001953125f,0.33769995789527929019868679461069405078887939453125f,-0.6817000442177059138515460290363989770412445068359375f,-1.1799000019788739024306778446771204471588134765625f,-0.3415000098794696015147565049119293689727783203125f,-0.658199979293346615349946659989655017852783203125f,1.206600029134750950987609030562452971935272216796875f,0.33560004694908840416900375203113071620464324951171875f,0.16869999121055012647474313780548982322216033935546875f,0.76360001064240867751919950023875571787357330322265625f,0.56580000421106813046634442798676900565624237060546875f,0.809999985992908566601045095012523233890533447265625f,-0.697100021997094199122102509136311709880828857421875f,-0.60460000430196558784956550880451686680316925048828125f,0.30019999188855184257818109472282230854034423828125f,-0.8168000140383842033742212151992134749889373779296875f,-0.5531000948563213892583689812454394996166229248046875f,0.786899986615777091714107882580719888210296630859375f,0.74879994434192853614007390206097625195980072021484375f,-1.866799976692348206341875993530265986919403076171875f,-0.10570000749379371285385786904953420162200927734375f,0.4042999986715614113563788123428821563720703125f,0.34670001910924852506212801017682068049907684326171875f,-0.0028000093370674772330630730721168220043182373046875f,-2.16779996910393180087339715100824832916259765625f,0.78430004087910132337668756008497439324855804443359375f,-0.36810002955198317664553542272187769412994384765625f,-0.25269997935295140933931179461069405078887939453125f,-0.0507000141620632671646262679132632911205291748046875f,-1.624900018668174794811420724727213382720947265625f,0.680900002753734856497658256557770073413848876953125f,-0.11879999264031637640215421924949623644351959228515625f,0.12849999522417776542937417616485618054866790771484375f,-0.4487000270664693868383210428874008357524871826171875f,0.5677000191390513261779915410443209111690521240234375f,1.567300002020596938479002346866764128208160400390625f,-0.60659998808205173315144520529429428279399871826171875f,0.00589999625310266839761652590823359787464141845703125f,-0.721100014367699326811589344288222491741180419921875f,1.31840000931024636798838400864042341709136962890625f,-0.6306999798148869018632467486895620822906494140625f,0.5611000068426132969534592120908200740814208984375f,-0.45620001020953082626618879658053629100322723388671875f,0.95110001805573662903015019765007309615612030029296875f,0.2878000206805768623752328494447283446788787841796875f,-0.469499986983835437825973713188432157039642333984375f,0.9687999971106642238538597666774876415729522705078125f,-0.565099965387582781062292269780300557613372802734375f,0.34179998794272570261654209389234893023967742919921875f,0.352500007580965613041001915917149744927883148193359375f,0.137199948269128402245087272603996098041534423828125f,-0.4701000130712980062952510706963948905467987060546875f,-0.676100022637843522943512653000652790069580078125f,-0.25249999850988391614237116300500929355621337890625f,1.18290001544207346029224936501123011112213134765625f,-1.0461000068426133946530853791045956313610076904296875f,-0.836000015452504019464186058030463755130767822265625f,0.246900005242228548496541407075710594654083251953125f,0.20590007182061620394364354069693945348262786865234375f,-0.1976000052481889923683411325328052043914794921875f,0.4056999948278068313101130115683190524578094482421875f,-0.60960003632083525904050702592940069735050201416015625f,0.63600001811608664059605189322610385715961456298828125f,-0.621100017366558176945545710623264312744140625f,1.067200018304586439654713103664107620716094970703125f,0.15260001053810157145562698133289813995361328125f,0.1580999852955342621640966171980835497379302978515625f,0.0047000175967812918287336287903599441051483154296875f,0.39249997939914404998518193679046817123889923095703125f,-0.57139999647289574369324327562935650348663330078125f,-1.18899999184906501881187068647705018520355224609375f,0.8041999833464625879742015968076884746551513671875f,-1.0573000144630668728495948016643524169921875f,0.4624000405356287757285826955921947956085205078125f,0.6773000049822031076729444976081140339374542236328125f,-1.4993999851256603061955274824867956340312957763671875f,-0.0045000068582592422927746156346984207630157470703125f,0.697500014007091717616049209027551114559173583984375f,-0.5212000045843423468028277056873776018619537353515625f,1.1052000136256214801733221975155174732208251953125f,-0.66500004224479258851943086483515799045562744140625f,0.711100004160404086661628753063268959522247314453125f,0.0264000081889335103113580771605484187602996826171875f,0.4141000091671944716154030174948275089263916015625f,0.9523000473387543962644485873170197010040283203125f,-1.198499991927296282057113785413093864917755126953125f,-0.8172999814942480956148074255906976759433746337890625f,1.51049997082725173669359719497151672840118408203125f,0.45040000791698719950773011078126728534698486328125f,-0.411400019308924669303451082669198513031005859375f,0.579299989190697584007239129277877509593963623046875f,-1.0700999969407913869900994541239924728870391845703125f,-0.0725999755203734498110179629293270409107208251953125f,-0.106400029404461360371669798041693866252899169921875f,-0.771400036258995225324497369001619517803192138671875f,-0.719699976506829930400499506504274904727935791015625f,-0.5092000151231881321933769868337549269199371337890625f,0.2176000227198005010365022826590575277805328369140625f,-0.3322000066444277255328643150278367102146148681640625f,-0.572200010891258603606956967269070446491241455078125f,0.53300000649690637555266903291339986026287078857421875f,-1.562500057742000070248877818812616169452667236328125f,0.945000001117586752030774732702411711215972900390625f,-0.74189998520016597893800280871801078319549560546875f,-0.246000051885844062127262077410705387592315673828125f,-1.60230004612803522690001045702956616878509521484375f,-0.6752000236094002350029086301219649612903594970703125f,-0.084300026145577522385110569302923977375030517578125f,0.035900015345216029771790999802760779857635498046875f,0.13789998716711959758640659856610000133514404296875f,-0.1566999996632336689117437344975769519805908203125f,-0.419200020599365341666953099775128066539764404296875f,-0.47190001626908806287019615410827100276947021484375f,0.143200004848837547655193702667020261287689208984375f,-1.129100047928840222510871171834878623485565185546875f,1.8714000643849377780725262709893286228179931640625f,-1.42490004534125347390727256424725055694580078125f,1.66290002635717382162283684010617434978485107421875f,-0.124899990206956690297346312945592217147350311279296875f,0.2206000460922723505063913762569427490234375f,-0.395900009720027146187248945352621376514434814453125f,-0.0867999653860923314763198277432820759713649749755859375f,0.256399988165497916270396672189235687255859375f,0.4787000044539571064206029404886066913604736328125f,-0.110300025355816178063150800880976021289825439453125f,0.4221000030726191898367005705949850380420684814453125f,1.2072000185653564141574634049902670085430145263671875f,-1.44460007005333945784286697744391858577728271484375f,0.2061999521136270718812966151745058596134185791015625f,-0.8641000122964381802148636779747903347015380859375f,-0.17990000145733364700362244548159651458263397216796875f,-0.95470000846982028264164910069666802883148193359375f,1.417599975296855330242351556080393493175506591796875f,-0.97339999560117718147012055851519107818603515625f,0.050500022292137014545687634381465613842010498046875f,0.899999982044100921285689764772541821002960205078125f,-0.4286999811708931407139289149199612438678741455078125f,2.92700000303983642879757098853588104248046875f,1.326600065419078600115199151332490146160125732421875f,0.424100052976609021015974576584994792938232421875f,-0.38090004119873110965954765561036765575408935546875f,-0.5644000068068510955754391034133732318878173828125f,0.3307999846160409873618846177123486995697021484375f,0.248100013458728341220194124616682529449462890625f,0.174100048692524378424195674597285687923431396484375f,0.392900021290779388749569989158771932125091552734375f,1.4199000144585962601695428020320832729339599609375f,-1.897799987581372693767889359151013195514678955078125f,1.924200033228100181759145925752818584442138671875f,-1.5650000041723242105717872618697583675384521484375f,0.59069998575672444385276094180881045758724212646484375f,-1.02380005831718534636820550076663494110107421875f,-1.0867000286459920221915353977237828075885772705078125f,1.981099999094009689315498690120875835418701171875f,0.7684000412359832665742942481301724910736083984375f,-0.38249995589256258199384319595992565155029296875f,1.6098999922931189754393699331558309495449066162109375f,-0.8608000226885084149586191415437497198581695556640625f,-1.809899950346351094054853092529810965061187744140625f,1.2717000091999766109296388094662688672542572021484375f,-0.246899982592463373975988361053168773651123046875f,0.528600026810169598689981285133399069309234619140625f,-0.1179999274089926775133108094451017677783966064453125f,1.62119998534321752003961591981351375579833984375f,-0.364800068926812048886176853557117283344268798828125f,-0.7680999968439337077796835728804580867290496826171875f,-2.5643999441474676359575823880732059478759765625f,-0.225400009146332624965225477353669703006744384765625f,-0.37649993377178925246795415660017170011997222900390625f,1.1649000771552326938262922340072691440582275390625f,-1.283300009500979665943987129139713943004608154296875f,-1.2989000149130820460641189129091799259185791015625f,-0.493600020998716360054459073580801486968994140625f,-1.446899998611211213273008979740552604198455810546875f,0.38979998540580285126821991070755757391452789306640625f,-0.5297999777317048408775690404581837356090545654296875f,-0.66719999785274242132260269499965943396091461181640625f,0.04619999005571007177906039942172355949878692626953125f,0.46509998611882286123631047303206287324428558349609375f,-1.03160002770051395515338299446739256381988525390625f,1.016300019574165158786627216613851487636566162109375f,1.39629999483823841188723235973156988620758056640625f,-0.95849999824166320916418726483243517577648162841796875f,0.2017999557375897978772627539001405239105224609375f,-0.735900000648945340486761779175139963626861572265625f,0.11650001576542923231727399979718029499053955078125f,-0.51869999888464801696130734853795729577541351318359375f,0.4197999911427496844140705434256233274936676025390625f,-0.046799991649389249825929937287583015859127044677734375f,-0.0166999979868531767124295583926141262054443359375f,0.0891999831601972825723123605712316930294036865234375f,-0.5342999790579077146190911662415601313114166259765625f,-1.23010005330443394910844290279783308506011962890625f,-1.4242000167995687576194541179575026035308837890625f,0.0781999392092231904172194845159538090229034423828125f,0.1971000433057543688164514605887234210968017578125f,0.2204999583289029008170700763002969324588775634765625f,0.1059999716430903848873867900692857801914215087890625f,0.6288999896183609106259382315329276025295257568359375f,-0.43280000568628285151362433680333197116851806640625f,-0.2078000195257363458267718669958412647247314453125f,-0.12949998283013719468925728506292216479778289794921875f,-0.630499968424440115910556414746679365634918212890625f,0.9638999673597516970602328001405112445354461669921875f,1.3298999769449231234119679356808774173259735107421875f,0.4536000490129001860850621596910059452056884765625f,0.62170000219643117844725566101260483264923095703125f,0.9892000277519219508803871576674282550811767578125f,-0.491600014755129155474833169137127697467803955078125f,-0.205999971494078781830694424570538103580474853515625f,-0.1821999823927875628015726761077530682086944580078125f,1.0245000171214346806891626329161226749420166015625f,0.07870002229809802685167596791870892047882080078125f,0.3230000094771394714143752935342490673065185546875f,1.272999989584088442740039681666530668735504150390625f,0.73700006473064405554396216757595539093017578125f,-0.457400003618001704808193608187139034271240234375f,-0.4292999685525895614546243450604379177093505859375f,0.3139000092692680876638178233406506478786468505859375f,0.3929999307990073020846466533839702606201171875f,-0.5766999814279378799852793235913850367069244384765625f,0.98709999137520743062168548931367695331573486328125f,1.0219000429980458211076665975269861519336700439453125f,-1.29120000963211101208116815541870892047882080078125f,0.8648000121161343312081726253381930291652679443359375f,-1.2062000284075740097478046664036810398101806640625f,0.19160002505555795426772647260804660618305206298828125f,0.37079998178482043158510350622236728668212890625f,0.737800013546645150341873886645771563053131103515625f,-0.069700005862116665156236194889061152935028076171875f,0.300799976010620806476936195394955575466156005859375f,0.2802000344872481729652236026595346629619598388671875f,0.161400004072488112427663509151898324489593505859375f,-0.9719999786764381699555315208272077143192291259765625f,-0.89939997912794289458560115235741250216960906982421875f,0.29000002630054932950542934122495353221893310546875f,-0.442700002524256230884702745242975652217864990234375f,1.6024000119626524707427961402572691440582275390625f,0.3475000049546361768904034761362709105014801025390625f,-0.707099997410177483203597148531116545200347900390625f,-1.71279996567666525919548803358338773250579833984375f,-1.0925000254064800575548588312813080847263336181640625f,-0.0128000309437517056920796676422469317913055419921875f,-0.509199989902973992883516984875313937664031982421875f,-0.998300017100572034678407362662255764007568359375f,-2.126000032663345251648934208787977695465087890625f,0.8664999784380196246047489694319665431976318359375f,-0.77609998344779018708550211158581078052520751953125f,0.295800002869963662988084251992404460906982421875f,1.19640002146959290030281408689916133880615234375f,-0.4707000057429076012027735487208701670169830322265625f,-0.075900032258033434118260629475116729736328125f,1.312900029784441091607050111633725464344024658203125f,0.4440000266805295137118037018808536231517791748046875f,0.720200035306811425783735103323124349117279052734375f,0.813900052445382105048565790639258921146392822265625f,2.41550002199411295578101999126374721527099609375f,0.714000009730458184975532276439480483531951904296875f,0.36059995670393119038266149800620041787624359130859375f,-0.041899988683313349469017339288257062435150146484375f,0.6674000034876168552244735110434703528881072998046875f,-1.1066999944105742059718977543525397777557373046875f,-0.41519999142289154558937980254995636641979217529296875f,0.562499995529651020120809334912337362766265869140625f,-0.228900018489361034568219110951758921146392822265625f,0.79800000046193553249196384058450348675251007080078125f,-0.558900003737210937515555997379124164581298828125f,-0.2642000024199493424248430528677999973297119140625f,-1.229899999073147842665321149979718029499053955078125f,-0.71469998924732180967112071812152862548828125f,0.36279999737888568578370040995650924742221832275390625f,-0.5764999771341681178427052145707421004772186279296875f,-0.4782000103622674469505682282033376395702362060546875f,0.18180000275075425264503792277537286281585693359375f,1.115799994748830403779038533684797585010528564453125f,0.96069999553561213900820803246460855007171630859375f,-0.5090000190436843041652537067420780658721923828125f,0.960600033035129197145352009101770818233489990234375f,-0.652500036954880346939944502082653343677520751953125f,1.3373000260114675796785377315245568752288818359375f,1.40570002757310863472639539395458996295928955078125f,0.2807999853610994023256353102624416351318359375f,-0.066900022006035175792248992365784943103790283203125f,0.4234000323325395331863774117664434015750885009765625f,0.7964000310808423410691148092155344784259796142578125f,-0.1167000047668815820856025311513803899288177490234375f,-0.8673000353991990873936401840182952582836151123046875f,0.03110004916191133617076047812588512897491455078125f,-0.638699983853100672348546140710823237895965576171875f,1.38870005839616084131193929351866245269775390625f,1.015100001597403878150771561195142567157745361328125f,0.64159998562335918137478074640966951847076416015625f,0.3779999637305735404879669658839702606201171875f,-0.7814000500798226056531348149292171001434326171875f,-0.7228000126153226734260215380345471203327178955078125f,0.030699994269013597403272797237150371074676513671875f,-0.4075000155717136696154057062813080847263336181640625f,-0.69620003246814043951218309302930720150470733642578125f,-1.5996000131309029423931633573374710977077484130859375f,0.07349998900294263393107030424289405345916748046875f,0.015500006645917441261417479836381971836090087890625f,-1.1301999924287198240335783339105546474456787109375f,-0.77470001251175968359774515192839317023754119873046875f,0.5945000068023797989980039346846751868724822998046875f,-0.051199984672665344032083112324471585452556610107421875f,0.253100005859136611974236075184307992458343505859375f,0.3559000016361475449144791127764619886875152587890625f,-0.541599986591935422808319344767369329929351806640625f,0.181699987407028140662390569559647701680660247802734375f,-0.722300023254752066037553959176875650882720947265625f,0.5073000246703625037980600609444081783294677734375f,0.105299954724312527787333237938582897186279296875f,-0.2033000039130445912860523094423115253448486328125f,-1.61829997988492291227657915442250669002532958984375f,0.593300002925842395740119172842241823673248291015625f,-0.02599999501928629275226967365597374737262725830078125f,-0.69940003236234193906284417607821524143218994140625f,1.610600014054775375171857376699335873126983642578125f,-0.09070001758933099456072568500530906021595001220703125f,0.30460002534985530786570961936376988887786865234375f,0.1365000160261988781940090120770037174224853515625f,0.0445999860480428278464160030125640332698822021484375f,-0.10059995132088506153422713396139442920684814453125f,-0.596500022768974957898535649292171001434326171875f,-0.0860000243186960222629977579344995319843292236328125f,-1.178299986981600877555820261477492749691009521484375f,-0.193899991797656323644361009428394027054309844970703125f,-0.57560001453906328716669804634875617921352386474609375f,-0.714400019174814548961194304865784943103790283203125f,0.463699983629584533417755665141157805919647216796875f,-0.175800006334483338577001632074825465679168701171875f,0.375700004439056101102067941610584966838359832763671875f,1.01450003585964498853400073130615055561065673828125f,1.535700002911686734563545542187057435512542724609375f,0.24879998008608783521822260809130966663360595703125f,0.1442000376611947132232671719975769519805908203125f,-0.272900020769239137763406688463874161243438720703125f,0.0424000010102982827220330364070832729339599609375f,-0.43930000845044847590514791590976528823375701904296875f,-0.27179999321401115963681149878539144992828369140625f,-1.2401000480890276289613893823116086423397064208984375f,-0.480699994492530979783850852982141077518463134765625f,0.236300019797683003019983516423963010311126708984375f,-1.223200007307529091349351801909506320953369140625f,-0.8195000388026241200378763096523471176624298095703125f,-1.4252000264406206042622216045856475830078125f,0.249600006574392185854094350361265242099761962890625f,-0.463200016993283725241781212389469146728515625f,-0.13140002929270278020368323268485255539417266845703125f,-0.379399989819526606282806824310682713985443115234375f,0.41380001469403515645950619727955199778079986572265625f,-0.5171000224813819823310723222675733268260955810546875f,0.84490002038180822552249082946218550205230712890625f,0.67700001697242218057226637029089033603668212890625f,0.100799992066621602049281136714853346347808837890625f,0.841800027263165162594305002130568027496337890625f,0.833300043326616890482227972825057804584503173828125f,0.43289999461770012345596114755608141422271728515625f,1.0018000554263599699567066636518575251102447509765625f,-1.1304000080287455176630828646011650562286376953125f,-0.383500001296400938599617802537977695465087890625f,-0.3203999831810591114589215067098848521709442138671875f,0.45140001569539311976342332854983396828174591064453125f,-0.6228999987155197093358083293423987925052642822265625f,0.43090002912879032948012536508031189441680908203125f,-0.4332000306278473988896848823060281574726104736328125f,-0.0556000171840196344419382512569427490234375f,0.910300015893578784442752294125966727733612060546875f,1.2880000397264961264909288729541003704071044921875f,0.319399975588917417468337589525617659091949462890625f,-0.573700041446090214236619431176222860813140869140625f,-1.0421000211030246873633586801588535308837890625f,0.5857000233262776855980291657033376395702362060546875f,-1.461999992758034938589162265998311340808868408203125f,0.3176000057697294653991093582590110599994659423828125f,1.351999968171119892446085941628552973270416259765625f,1.068400011470914012789990010787732899188995361328125f,2.317099984669685142080197692848742008209228515625f,0.3665000267177820347797023714520037174224853515625f,0.752400008535385467212108778767287731170654296875f,-0.710500011935830233511524056666530668735504150390625f,-0.532200032162666758495106478221714496612548828125f,0.2976000332623716104762934264726936817169189453125f,0.595599979186058448021867661736905574798583984375f,0.541700010969489387235853428137488663196563720703125f,-2.002400016283988559706585874664597213268280029296875f,0.144700016888976268347732911934144794940948486328125f,-0.7772000128656628703538444824516773223876953125f,-1.95900003230571684298411128111183643341064453125f,0.1851000114321710299236656283028423786163330078125f,-1.073600055569411182432304485701024532318115234375f,0.6131999813795090403800713829696178436279296875f,-0.5200000001490110435042879544198513031005859375f,1.713899993753432937637626309879124164581298828125f,1.09650003185868261113000698969699442386627197265625f,-0.3808000154614443744094387511722743511199951171875f,-2.2970000564604990955785979167558252811431884765625f,-0.334599990406632574746481623151339590549468994140625f,-0.643999984622001253598000403144396841526031494140625f,-0.063999982535838828567875680164434015750885009765625f,-1.78260002678036766354807696188800036907196044921875f,-0.661400017371774229246739196241833269596099853515625f,0.4288000374555593907643924467265605926513671875f,1.893300059494376963442618944100104272365570068359375f,1.5405000228881835511174358543939888477325439453125f,1.31349997805058915645304296049289405345916748046875f,-0.90900006717443471160322587820701301097869873046875f,0.348699947419762512623719885596074163913726806640625f,-1.538199990953504237012339217471890151500701904296875f,-0.33219997270703327529872694867663085460662841796875f,-0.712000025466084540681777070858515799045562744140625f,-1.150899986362457383393120835535228252410888671875f,-0.8461000002861016611177547019906342029571533203125f,-0.978899983024597464265070811961777508258819580078125f,0.3376999914973974892262731373193673789501190185546875f,-0.8318999662011865581234815181232988834381103515625f,-0.0471000035196542654603035771287977695465087890625f,-0.07030000378638501246086889295838773250579833984375f,-1.147500017266720906405907953740097582340240478515625f,0.1743999857589602076046730871894396841526031494140625f,-0.48720002948045770807539156521670520305633544921875f,0.458900027653575737218716312781907618045806884765625f,1.0176000263333317974456804222427308559417724609375f,-1.204700008916855580309857032261788845062255859375f,1.177000020399689095285111761768348515033721923828125f,-0.689900003468989719834780771634541451930999755859375f,0.50450000624358615208819855979527346789836883544921875f,-0.534899993261694906010461636469699442386627197265625f,0.207200009773671645074699654287542216479778289794921875f,-0.466299990554154053512547761783935129642486572265625f,0.5393999987788493655216370825655758380889892578125f,0.64019999103173674814826199508388526737689971923828125f,-0.15679997749328489220488336286507546901702880859375f,0.8968000237613917846601907513104379177093505859375f,1.78810004013180812165728639229200780391693115234375f,-0.435699956569075563805881756707094609737396240234375f,0.8626999964892856365139550689491443336009979248046875f,-0.835899990311264939890634195762686431407928466796875f,0.08070000194311155450321848547901026904582977294921875f,0.352600006067752824723271487528108991682529449462890625f,0.21219999563619496019128973784972913563251495361328125f,-0.391999987263232707590532299946062266826629638671875f,0.272800014906376564471912615772453136742115020751953125f,-0.4140000518634916470972484603407792747020721435546875f,0.21380002195835157152714600670151412487030029296875f,0.6976999957442284738817761535756289958953857421875f,0.00559999937713140383266363642178475856781005859375f,0.97910001304149574252733145840466022491455078125f,0.01780001589357826929926886805333197116851806640625f,-0.5910999822556970428877320955507457256317138671875f,0.89850002551078811396934042932116426527500152587890625f,0.71940000755190869341504367184825241565704345703125f,-1.0085999850124114374239070457406342029571533203125f,1.3889000020980830640837666578590869903564453125f,1.30850001381337666117588014458306133747100830078125f,-0.01239995012283401365493773482739925384521484375f,-0.954099977949261823795268355752341449260711669921875f,-0.21740001017451238141120484215207397937774658203125f,-1.91720005421638450826549160410650074481964111328125f,2.115599983358382729647928499616682529449462890625f,0.2587999785959713694438733000424690544605255126953125f,-0.0453999904453752378685749135911464691162109375f,0.30800001983344582612289741518907248973846435546875f,-1.409800016251206766781933765741996467113494873046875f,-0.96620006417036119472641075844876468181610107421875f,1.294299991202354505048788269050419330596923828125f,-0.356600020939112027917872183024883270263671875f,0.072200009512901175412480370141565799713134765625f,-0.284300041270256542702554725110530853271484375f,0.279499999314547498130423264228738844394683837890625f,-0.020899987703561606622315593995153903961181640625f,1.1356999655842781038472821819595992565155029296875f,0.20299996425211475070682354271411895751953125f,-0.09070001431107499456629739142954349517822265625f,-0.9830000235587357249045226126327179372310638427734375f,-0.330699971470236331327896550646983087062835693359375f,0.04380002627968782036305128713138401508331298828125f,-0.569199991318582565469341716379858553409576416015625f,-1.069100019896030318022894789464771747589111328125f,1.3033999991029503462414140813052654266357421875f,0.0158000010073191532455894048325717449188232421875f,-0.768999968156218738357665642979554831981658935546875f,0.470299979758262320927997279795818030834197998046875f,-1.445999969631433401673348271287977695465087890625f,-1.2568000116169455449011138625792227685451507568359375f,-1.3916999866247170558608559076674282550811767578125f,-0.08689999637603762483450964282383210957050323486328125f,-0.38789997360706340856495444313623011112213134765625f,1.1237000162631269883917184415622614324092864990234375f,0.35049999606609372193588569643907248973846435546875f,-1.153799993422627512273948013898916542530059814453125f,1.09299996328353898888963158242404460906982421875f,1.30100002461671859776970450184307992458343505859375f,0.57649997478723502553066282416693866252899169921875f,2.05529998378157596761184322531335055828094482421875f,-1.794500002220272616426655076793394982814788818359375f,0.152699967618286525095072647673077881336212158203125f,0.129700004521011991176493438615580089390277862548828125f,-0.3770999878734355714726689257076941430568695068359375f,-0.734500008031725659662924954318441450595855712890625f,0.7599999885633585972044556910987012088298797607421875f,0.5564000293672080044160566103528253734111785888671875f,-0.1336000287473202430277297025895677506923675537109375f,0.77309998201727891942169890171498991549015045166015625f,-1.0289999825358382423701186780817806720733642578125f,1.00330000503063132555325864814221858978271484375f,0.7289000549227007130781430532806552946567535400390625f,0.8097000014290214320311633855453692376613616943359375f,0.23980001222789315562522460822947323322296142578125f,0.04670002160519448608511083875782787799835205078125f,-0.567700042608379806807761269737966358661651611328125f,-0.2892999719426032623204037008690647780895233154296875f,-0.09439997715353865004317412967793643474578857421875f,-0.24540001235008190860753529705107212066650390625f,0.86030001654550425360667986751650460064411163330078125f,-0.7187999985262749635950285664875991642475128173828125f,0.97000002115964878424136941248434595763683319091796875f,-0.445400018310547096689333557151257991790771484375f,0.933600021147727776593683302053250372409820556640625f,-0.7067000182896858717640498070977628231048583984375f,-0.155799997598677464338834397494792938232421875f,0.6605000111162666609487814639578573405742645263671875f,-0.316999956846236552454598722761147655546665191650390625f,0.2735999968960880490698173161945305764675140380859375f,0.1285000167563554729355246308841742575168609619140625f,-1.00670002954006321260749245993793010711669921875f,0.2029999974630773973416353328502736985683441162109375f,-0.88839999358951982078025366718065924942493438720703125f,0.3344999889954920480050759579171426594257354736328125f,-0.2350999702304594762836131849326193332672119140625f,-0.6240999646499745256988944674958474934101104736328125f,0.64000002671033107137787965257302857935428619384765625f,0.588499977752567016153761869645677506923675537109375f,-0.32110001392066467307273569531389512121677398681640625f,0.99779995882213190494525179019547067582607269287109375f,-0.91819997530728503232211323847877793014049530029296875f,1.190499991372228283381673463736660778522491455078125f,0.3465000405013558548716900986619293689727783203125f,-0.26350001163408187654368930452619679272174835205078125f,1.0493999674119056653154302694019861519336700439453125f,-0.825999962963164247042868737480603158473968505859375f,-0.4240000130645931175621399233932606875896453857421875f,0.3500999760046601405605315449065528810024261474609375f,-1.50979999332204517514810504508204758167266845703125f,0.0538999960631139085620588957681320607662200927734375f,-1.204399977004527944046685661305673420429229736328125f,-1.20190001202225626997233121073804795742034912109375f,-0.63650000712275467407152973464690148830413818359375f,0.212200011301041602251871154294349253177642822265625f,-0.7137999510288235427424297085963189601898193359375f,-0.8180999960243695579009681750903837382793426513671875f,-0.5028999611273408110179161667474545538425445556640625f,-0.4762000133946535651574549774522893130779266357421875f,1.050500001132487870592058243346400558948516845703125f,-1.095000004041939423160556543734855949878692626953125f,0.93679996987506808370227417981368489563465118408203125f,-1.089499987263232494427711571916006505489349365234375f,1.323999991625547156814945992664434015750885009765625f,-0.74049997390061561564067460494698025286197662353515625f,0.6819999921619892546686969581060111522674560546875f,-0.41109996996223896470468162078759633004665374755859375f,-0.4623000286936758129741065204143524169921875f,-0.9130999990604815597095011980854906141757965087890625f,0.476299964365363592833091388456523418426513671875f,-0.57749999642372162877990149354445748031139373779296875f,-0.841899975030124370078965512220747768878936767578125f,0.81069996089041251874363069873652420938014984130859375f,-0.241200014046579147564131062608794309198856353759765625f,0.25719998637884955439147915967623703181743621826171875f,-1.03759999854266649066403260803781449794769287109375f,0.9429999222308407258452689347905106842517852783203125f,-0.836099965864419925054562554578296840190887451171875f,1.2374999710917482520500243481365032494068145751953125f,0.0032999986976387507553454270237125456333160400390625f,1.1599999438226229475645823185914196074008941650390625f,-0.1139999962821601486862022056811838410794734954833984375f,-0.839800005298852791923991389921866357326507568359375f,0.55490005398392572288912560907192528247833251953125f,-0.766999979272484200265580511768348515033721923828125f,-0.2462999968498944536321459963801316916942596435546875f,0.2787000122398139989599030741374008357524871826171875f,-0.342099987053871235076485390891321003437042236328125f,-0.335900023689866156217931347782723605632781982421875f,0.1844000342249874080380322993732988834381103515625f,-1.142800015968085336481863123481161892414093017578125f,0.536299917054176233932594186626374721527099609375f,0.30190001783370856713872854015789926052093505859375f,1.1283999583482735573625177494250237941741943359375f,0.054700025755166858942857288639061152935028076171875f,1.16009999101758065620515480986796319484710693359375f,0.157400021294504310898076937519363127648830413818359375f,-0.7233000507026916547914652255713008344173431396484375f,-0.28769999984204741583226905277115292847156524658203125f,0.41009996739923959552243104553781449794769287109375f,-0.715300032005460106887539950548671185970306396484375f,1.2904000457286830538805588730610907077789306640625f,0.56419997056871584728554580578929744660854339599609375f,-0.987799962472916348588114487938582897186279296875f,1.2972999802649021194866918449406512081623077392578125f,-1.7406999819755544223198739928193390369415283203125f,-0.14090002760142084081707025688956491649150848388671875f,-1.456000005915760286967497449950315058231353759765625f,-1.2548999858483662617203435729607008397579193115234375f,0.89230002869367552875701221637427806854248046875f,-0.1313000315695997954890117398463189601898193359375f,0.2933000068187714415302025372511707246303558349609375f,0.4798000359579914775309816832304932177066802978515625f,-1.251800026145577504621542175300419330596923828125f,-1.0576999965637938760920633285422809422016143798828125f,0.399400018876790330324411115725524723529815673828125f,-0.778599968099595418635772148263640701770782470703125f,1.38829997530281534778850982547737658023834228515625f,0.4761000093013050271650854483596049249172210693359375f,0.67509999295473033242842575418762862682342529296875f,0.0247000412523747936432982896803878247737884521484375f,1.57060003104209844337901813560165464878082275390625f,-0.6413999937534324846666322628152556717395782470703125f,-0.657099977442622051881926381611265242099761962890625f,0.8784000090867281063111704497714526951313018798828125f,-0.01109998307526094407648997730575501918792724609375f,-0.004800005820393860034300814731977880001068115234375f,1.32400000861287114872766323969699442386627197265625f,-1.2928000179797418667249075951986014842987060546875f,-0.4943000701785091877127342741005122661590576171875f,-0.5593000220105055486641276729642413556575775146484375f,0.10530004025697703440300756483338773250579833984375f,0.355600002394617253909103737896657548844814300537109375f,0.07119994347095559561466870945878326892852783203125f,-0.086399973264336737965862766941427253186702728271484375f,-0.3112000142514705913754369248636066913604736328125f,0.334800006602704536273762414566590450704097747802734375f,-0.12940000039935117825962151982821524143218994140625f,-0.04300003512576300312986177232232876121997833251953125f,1.00960002803206361221555198426358401775360107421875f,-0.600200038976222316478015272878110408782958984375f,0.2245999998688699150761749478988349437713623046875f,-0.11869999758079641249963742666295729577541351318359375f,-0.753299980667233359810097681474871933460235595703125f,0.849299997088313940452053429908119142055511474609375f,0.1366000138610601322142201752285473048686981201171875f,-1.11849999861419124869144070544280111789703369140625f,0.5482000087231397866815996167133562266826629638671875f,-0.78049997438490381629350167713710106909275054931640625f,0.3445999872028828736603145443950779736042022705078125f,0.5523000374481072949350846101879142224788665771484375f,-1.100099996083975018024148084805347025394439697265625f,0.27539994510859155951720822486095130443572998046875f,0.10730001081228213166696150437928736209869384765625f,0.265600018227100331813517186674289405345916748046875f,1.16250002294778820299825383699499070644378662109375f,0.1991999965339898981397936950088478624820709228515625f,-1.3242000051021580731003268738277256488800048828125f,-0.439500016972421914118740460253320634365081787109375f,-0.39719998771995335129503246207605116069316864013671875f,0.9839000281378627033035400017979554831981658935546875f,0.3391000069320202481293335949885658919811248779296875f,0.665200002971291670661457828828133642673492431640625f,-0.0928999762892726010932165081612765789031982421875f,-1.0861000113874670347513529122807085514068603515625f,-0.5186000479698176235388018540106713771820068359375f,0.455200009006261563371253942023031413555145263671875f,0.14709998228549903132034160080365836620330810546875f,-1.0501000021748236878949001038563437759876251220703125f,1.354499955654144827121854177676141262054443359375f,-0.3607999655425551122078786647762171924114227294921875f,0.955400051353871848647258957498706877231597900390625f,0.72040001398921038600065003265626728534698486328125f,-0.816500004403292845012174439034424722194671630859375f,-0.47990001313611851951890230338904075324535369873046875f,1.0491999699354170871856695157475769519805908203125f,-0.50020000246837792445830928045324981212615966796875f,0.7363000493764868537027723505161702632904052734375f,0.08079999188035758006520836715935729444026947021484375f,0.1062000038951633751338476940873079001903533935546875f,-0.227000079594552151007746942923404276371002197265625f,-0.09019999417960633536495151929557323455810546875f,-0.521099999615549425158178564743138849735260009765625f,-1.3411999903351066176782069305772893130779266357421875f,-0.236299956728517612791762303459108807146549224853515625f,0.5463999844402074312910144726629368960857391357421875f,0.6369999969676138906748974477523006498813629150390625f,-0.273800001320242625268974734353832900524139404296875f,-0.02350002048164601120561201241798698902130126953125f,-1.061699973586202272457512663095258176326751708984375f,1.2979999942034481374975030121277086436748504638671875f,1.1365999883800743663897492297110147774219512939453125f,0.7861000574320555767116047718445770442485809326171875f,1.748099995279312590668041593744419515132904052734375f,0.287000007733702044987467161263339221477508544921875f,1.40320003293752648687586770392954349517822265625f,0.02540001868307673049685035948641598224639892578125f,0.006500019490719211034956970252096652984619140625f,0.317700011688470507209558491013012826442718505859375f,1.5297000061601397913335631528752855956554412841796875f,-0.5237999983400107861797323494101874530315399169921875f,-0.346800013256072237055605000932700932025909423828125f,-0.2517000416100028115096165493014268577098846435546875f,-0.4005999864131213339391024419455789029598236083984375f,1.72470000422298852527092094533145427703857421875f,-1.57229998153150152262469418928958475589752197265625f,0.35509999003037895359824460683739744126796722412109375f,0.4288999845147145606460981070995330810546875f,0.4609000364489850287696981467888690531253814697265625f,-0.3475999974250800761410573613829910755157470703125f,0.6611999860137698359352498300722800195217132568359375f,0.0335000217109919784519433960667811334133148193359375f,-0.67280004264116322065802933138911612331867218017578125f,0.6959000071495768935392334242351353168487548828125f,-1.2579000161871312091221852824673987925052642822265625f,0.0790000299215318779744166022283025085926055908203125f,-1.4684999346882108284262358210980892181396484375f,1.118100009435414765590621755109168589115142822265625f,0.3149999752268193287108033473487012088298797607421875f,0.086199983853102235542564812931232154369354248046875f,-1.768399982748924958997349676792509853839874267578125f,0.2516000240683553546006123724509961903095245361328125f,0.0609000248074524908048488214262761175632476806640625f,-0.164800001722574140927690677926875650882720947265625f,0.7654999928623436034058613586239516735076904296875f,-0.138200017216802084618620938272215425968170166015625f,0.34819992736279925527043133115512318909168243408203125f,2.15920003676712557449945961707271635532379150390625f,0.02990002466589236629346260087913833558559417724609375f,0.65330002859309310725421937604551203548908233642578125f,-0.446100010269880886681903575663454830646514892578125f,0.371099955508112255841979276738129556179046630859375f,0.78960001365244369964102588710375130176544189453125f,-0.3065000091344123323011672255233861505985260009765625f,-0.13809999483227652916639272007159888744354248046875f,-0.571199956583976042878703083260916173458099365234375f,-1.1457000349119310644852021141559816896915435791015625f,-0.604900026118755373971680455724708735942840576171875f,0.97840002905428435919787943930714391171932220458984375f,0.13309997829198849661480608119745738804340362548828125f,0.1174999974668029523172663175500929355621337890625f,-0.481300014805793097139030578546226024627685546875f,1.334300042238832251229041503393091261386871337890625f,0.469900020530820139441630090004764497280120849609375f,-0.2316000166922809189173904087510891258716583251953125f,-0.4315999929249283439958162489347159862518310546875f,0.27840005341768314650607862859033048152923583984375f,-0.2224999450892208852081921577337197959423065185546875f,0.997800015372037218952527837245725095272064208984375f,0.1130000254958865024690339851076714694499969482421875f,0.317599973583221117934272115235216915607452392578125f,0.799200031924247156212004483677446842193603515625f,0.0252000070691107413978215845418162643909454345703125f,1.21370000642836117776823812164366245269775390625f,-0.437699974770843269755715709834476001560688018798828125f,0.1866000119984148053475792039535008370876312255859375f,-0.03179999526292032197005710258963517844676971435546875f,-0.6193999396026141912585671889246441423892974853515625f,0.19359997070729750845430316985584795475006103515625f,-0.3521999469280245875779655762016773223876953125f,-0.301600016915797686234412822159356437623500823974609375f,0.00170002361684973291033884379430674016475677490234375f,0.66260002352073799070097948060720227658748626708984375f,0.360099996232985830602046917192637920379638671875f,-0.77190004145205060037682187612517736852169036865234375f,1.00630002218186831441926187835633754730224609375f,-0.6973000478416684355664756367332302033901214599609375f,1.397400018705428070830976139404810965061187744140625f,0.70329998383372949177783084451220929622650146484375f,-0.80039998563975078837984256097115576267242431640625f,0.5972999954640858533139180508442223072052001953125f,-1.47299997282028272138632019050419330596923828125f,0.53720000856295169189280613863957114517688751220703125f,-0.20669998520910703998509916345938108861446380615234375f,-0.62460003103092365250859074876643717288970947265625f,-0.102000053182244254657007331843487918376922607421875f,-0.8524000129684801141394245860283263027667999267578125f,0.21449999666959040922620260971598327159881591796875f,-0.23830002287477325406683803521445952355861663818359375f,-0.2008999976873393489285035684588365256786346435546875f,-0.71550003555417074352362760691903531551361083984375f,-1.69730002012550773571319950860925018787384033203125f,-0.53869997148513704132710699923336505889892578125f,-1.37659995693862402532658961717970669269561767578125f,0.10260001884549796447032576907076872885227203369140625f,0.218400015717744366838815039955079555511474609375f,-0.44440001250654470599243950346135534346103668212890625f,-0.49680000640153865187897963551222346723079681396484375f,-0.693200019694119706770152333774603903293609619140625f,-1.19129996525943315788254039944149553775787353515625f,-0.229499985456466770727956827613525092601776123046875f,-1.31219997181296310628795254160650074481964111328125f,-0.5713000063121314031633346530725248157978057861328125f,-0.8273999650239947101226789527572691440582275390625f,0.56189998967051568712349762790836393833160400390625f,-1.111299911689759056798720848746597766876220703125f,0.0670999893635515487488873986876569688320159912109375f,-1.04279995673596914684821967966854572296142578125f,-0.587199993903934736039218478254042565822601318359375f,0.3808000002436335851285775788710452616214752197265625f,0.599600015906244454555462652933783829212188720703125f,0.35609998462125691531809934531338512897491455078125f,0.9877000109434124208718230875092558562755584716796875f,-0.11520000110864625497697488754056394100189208984375f,0.56810000138878802289355007815174758434295654296875f,-0.49169997798204434502622461877763271331787109375f,1.02459999942183355159386337618343532085418701171875f,-1.895799965691566058012540452182292938232421875f,0.01639998235404382942448364701704122126102447509765625f,-0.8685999693661923259924151352606713771820068359375f,1.457500022016464669150082045234739780426025390625f,-0.27880001309216007054914143736823461949825286865234375f,0.892999993011355019945085587096400558948516845703125f,0.1907999576076859771234239815385080873966217041015625f,0.5196999598920346574715267706778831779956817626953125f,0.228799992677569452581565201398916542530059814453125f,0.261499974936246548651297416654415428638458251953125f,-1.644399993991851882668697726330719888210296630859375f,-0.0267000132977965876079906593076884746551513671875f,-0.385299953979254095060014151385985314846038818359375f,1.3747000061228866751861232842202298343181610107421875f,0.2037000513926141564979843678884208202362060546875f,1.31879999785572277204437341424636542797088623046875f,-0.82649996674060854928711705724708735942840576171875f,-0.3668999796122316414681563401245512068271636962890625f,0.164799983021617801881575360312126576900482177734375f,-0.4243000015586613216100886347703635692596435546875f,-0.0232999905765058912265885737724602222442626953125f,0.708900021171569694189429355901665985584259033203125f,-0.78580001814365374457338475622236728668212890625f,0.2104999711066479761711889295838773250579833984375f,0.43199992287159005854846327565610408782958984375f,0.4128999771088359693749225698411464691162109375f,1.378599955582618719063248136080801486968994140625f,-0.35359998776912693330132242408581078052520751953125f,-0.6112999875366682545063667930662631988525390625f,0.45110002502203006002901020110584795475006103515625f,-0.741800052744150395511724127572961151599884033203125f,-0.386500014573335715084567709709517657756805419921875f,0.940599996694922335649380329414270818233489990234375f,-0.09890002862215130363665593904443085193634033203125f,-0.2623999718785281487498650676570832729339599609375f,0.42640007228255427662588772363960742950439453125f,-1.399900023436547513711047940887510776519775390625f,-0.67619996984600927447672802372835576534271240234375f,-0.23990000972747793639427982270717620849609375f,-1.101000023722648268886814548750407993793487548828125f,0.87269997845888080956200383297982625663280487060546875f,1.059099995011091532859381914022378623485565185546875f,0.14910001497864744379739931900985538959503173828125f,0.744600008511543176581426450866274535655975341796875f,0.8186000030428168816598599732969887554645538330078125f,0.344100002908707125470755272544920444488525390625f,0.8120000370889908225535691599361598491668701171875f,0.1109000629544265592585361446253955364227294921875f,-0.64239996860027304137474857270717620849609375f,-0.53849996918439924087351755588315427303314208984375f,0.1988999938279381041184024070389568805694580078125f,0.544000003695487510668726827134378254413604736328125f,0.4493999793887137883530158433131873607635498046875f,0.8309000105023383042635032325051724910736083984375f,0.7708000041365625776279557612724602222442626953125f,-0.69629998314082619526033113288576714694499969482421875f,-0.07099999108910548528683648328296840190887451171875f,-0.110599981644749600917521092924289405345916748046875f,0.45789999819397930735931367962621152400970458984375f,1.10000000655651053449446408194489777088165283203125f,0.17129996592998519844286420266143977642059326171875f,-0.555100039818883228548429542570374906063079833984375f,-0.528500047564506303388043306767940521240234375f,0.45260001858472842517500112080597318708896636962890625f,-0.3443999898195269082634695223532617092132568359375f,-1.988700034070014499576473099295981228351593017578125f,0.784700000274181519444027799181640148162841796875f,0.709399989074467551830593947670422494411468505859375f,-0.37510000144839228397586339269764721393585205078125f,-0.0959000120669604161349752757814712822437286376953125f,0.820299986389279478515845767105929553508758544921875f,-1.2269000351190566533432502183131873607635498046875f,0.5362000273644920156357329688034951686859130859375f,0.1505000580549253896833761245943605899810791015625f,-0.51610002114772868253567139618098735809326171875f,0.8797000320509074100527868722565472126007080078125f,0.17949996794760192386775088380090892314910888671875f,0.030199985313414945409249412477947771549224853515625f,1.081500011146067752321187072084285318851470947265625f,0.9034999765604734278667820035479962825775146484375f,-0.18759999705255037127926698303781449794769287109375f,-0.2843000099033117233915390897891484200954437255859375f,0.71269998434484038707381614585756324231624603271484375f,-0.39289997733235271226703844149596989154815673828125f,1.052100028181076396549542550928890705108642578125f,1.31560005384087563840012080618180334568023681640625f,-0.50029998512268125665514162392355501651763916015625f,0.0978999807596199200787623340147547423839569091796875f,0.837099987351894725406964425928890705108642578125f,-1.100700008797645512714780124952085316181182861328125f,-0.7021999952077866424815510981716215610504150390625f,-0.21769999265223749507214279219624586403369903564453125f,-0.4769999764040113721108582467422820627689361572265625f,0.218400013855099661697067858767695724964141845703125f,0.66110000900328191875132688437588512897491455078125f,-0.160399997299909724546296274638734757900238037109375f,-0.1260999895572663120901779620908200740814208984375f,-0.516500008389353837401358759962022304534912109375f,-0.585400019502639423762957449071109294891357421875f,1.293000007167459219914462664746679365634918212890625f,1.271200009632110550228389911353588104248046875f,0.05850002253055652801094765891321003437042236328125f,1.488599970185756404816856957040727138519287109375f,0.9774999798834309583384083452983759343624114990234375f,-0.0742999385267488765549614981864579021930694580078125f,-0.1269000013679264515786826450494118034839630126953125f,0.2468999875470998528470545352320186793804168701171875f,-0.9143000387370587844770852825604379177093505859375f,1.5733999988794327151708785095252096652984619140625f,0.88660001408457755900371921597979962825775146484375f,-0.3911999691009526713969535194337368011474609375f,-0.00309998142123202313769070315174758434295654296875f,1.186000008150935425277339163585565984249114990234375f,1.5013999973297114109982430818490684032440185546875f,0.9943999708950512772531737937242724001407623291015625f,-0.564400071254372637241658594575710594654083251953125f,-0.870300038915873042100201928406022489070892333984375f,-0.9640000249296429846168621224933303892612457275390625f,0.07349997890740633810224835542612709105014801025390625f,0.37880000273585334724657514016143977642059326171875f,1.0771999806046486103383585941628552973270416259765625f,-0.09980000514984066484203140134923160076141357421875f,-1.4957999913215633824847827781923115253448486328125f,-0.5537999870896346266135878977365791797637939453125f,1.113999961897730539561734985909424722194671630859375f,0.69050001297891139273588123614899814128875732421875f,1.9762999684631825658698289771564304828643798828125f,0.3838000172644857599379974999465048313140869140625f,1.003099986338614968417459749616682529449462890625f,-0.19649997560679910435510464594699442386627197265625f,0.501200002479553319290062063373625278472900390625f,0.735500000387430130643906522891484200954437255859375f,1.012199979785084469341427393374033272266387939453125f,-0.38769999172091473838008823804557323455810546875f,-0.4752000334441657258821578579954802989959716796875f,-0.9454000209927555520295072710723616182804107666015625f,0.167599987441300779522634911700151860713958740234375f,-1.049900033718347724942532295244745910167694091796875f,0.507400024554133022292035093414597213268280029296875f,-2.0894000437617297194492493872530758380889892578125f,1.75640000090599102122723706997931003570556640625f,0.535399985677003087403136305510997772216796875f,0.0368000096797942433113348670303821563720703125f,-1.452600042873620989070104769780300557613372802734375f,0.935500035852193878582738761906512081623077392578125f,-0.400700005668401360026109614409506320953369140625f,0.388299986031651389595253931474871933460235595703125f,1.3487000488221649874276408809237182140350341796875f,0.08059999468326584093347264570184051990509033203125f,-1.228399981594085144109840257442556321620941162109375f,0.7052000366479160842203555148444138467311859130859375f,0.01719999595284349425128311850130558013916015625f,1.274099996575712712143513272167183458805084228515625f,-0.11080002619028039845261446316726505756378173828125f,0.23250000797212155845272718579508364200592041015625f,-0.77659999203085894325937488247291184961795806884765625f,0.68940000494420505816606237203814089298248291015625f,0.980500033468008069803545367904007434844970703125f,0.09690003824830117906685700290836393833160400390625f,1.209900031334161685236949779209680855274200439453125f,1.171700000073015335289028371335007250308990478515625f,0.13710000018551971034952430272824130952358245849609375f,0.98940000226199564803408748048241250216960906982421875f,-0.8921000119388107219009498294326476752758026123046875f,1.117899951255321600029901674133725464344024658203125f,0.2099999912083145847674359174561686813831329345703125f,-0.723099982985853539929621547344140708446502685546875f,0.62660002265274528099325834773480892181396484375f,-1.3123999836504456339980606571771204471588134765625f,-0.762500015720724899637161797727458178997039794921875f,-0.2914000264614824420306149477255530655384063720703125f,-0.04160000026375032522452102057286538183689117431640625f,0.539199998992681006626526141189970076084136962890625f,1.00959999510049680537804306368343532085418701171875f,-0.072600002416967956975213382975198328495025634765625f,0.2325999446004634041429426360991783440113067626953125f,-0.1762000044167042478449047848698683083057403564453125f,1.7088999801933770772421894434955902397632598876953125f,-1.03269997726678841587499846355058252811431884765625f,-1.4728999628782275532756784741650335490703582763671875f,1.35719996950328347651293370290659368038177490234375f,1.581599982047081187630510612507350742816925048828125f,-0.76009999146461471042357516125775873661041259765625f,-0.345000025145709710017172255902551114559173583984375f,0.4839999682307247752532930462621152400970458984375f,-0.17130000713169568626881300588138401508331298828125f,-0.86690002796649867633504982222802937030792236328125f,1.50109999648630587643083345028571784496307373046875f,-0.9814999862238760197641340710106305778026580810546875f,-0.5987000341072683529120013190549798309803009033203125f,0.12149996711313681974075961989001370966434478759765625f,1.298800035257638096908294755849055945873260498046875f,0.2449000502586364813595309897209517657756805419921875f,-1.0266999934792513382575407376862131059169769287109375f,1.1243999941036102452329714651568792760372161865234375f,1.1890000577494508338105561051634140312671661376953125f,0.2315000126361841381594786071218550205230712890625f,-0.071599999723583440669472111039794981479644775390625f,-0.75429996736049620409403360099531710147857666015625f,-0.427500038929284176258960314953583292663097381591796875f,0.00440001284182045537818339653313159942626953125f,0.22490003017932258533306821846053935587406158447265625f,0.19190001451820182243324097726144827902317047119140625f,0.2914000177256763635114111821167171001434326171875f,-0.203100037561357193993671899079345166683197021484375f,0.3898000360883775439191367695457302033901214599609375f,0.097600005695223757129497244022786617279052734375f,0.51460000456273535807127927910187281668186187744140625f,-0.18269995133876815174289731658063828945159912109375f,1.016399978367984235916310353786684572696685791015625f,0.259099982419610075368154866737313568592071533203125f,0.93600003754347482765041377206216566264629364013671875f,0.2678999457791448879362405932624824345111846923828125f,-0.9877000523686410016210857065743766725063323974609375f,-0.49049997270852296882281962098204530775547027587890625f,0.36290003820359795216887732749455608427524566650390625f,0.33209999636709708337178881265572272241115570068359375f,-0.0724000310361379018075922431307844817638397216796875f,-0.76229994871169337589122960707754828035831451416015625f,-0.396499964803456617090660074609331786632537841796875f,-1.73530006055980923207471278146840631961822509765625f,0.4299000051453705228965418427833355963230133056640625f,0.348799983736872487583013935363851487636566162109375f,-0.67570002176165644414140842854976654052734375f,-0.123000057607889790034505494986660778522491455078125f,0.1472999920368189830099936443730257451534271240234375f,-0.6782000368118286104390790569595992565155029296875f,0.849500008776784110153812434873543679714202880859375f,-0.445999991759657543610728680505417287349700927734375f,0.11539999893903729599742291611619293689727783203125f,1.2195999906301491222393451607786118984222412109375f,0.356100010046362935867136911838315427303314208984375f,-0.19689997041076401362857950516627170145511627197265625f,0.29209998869299880208671993386815302073955535888671875f,-0.2378999831065533587803884074673987925052642822265625f,-0.068400031159073615416588154403143562376499176025390625f,0.02020002815425403497329170932061970233917236328125f,-0.435700019023567308718014601254253648221492767333984375f,0.005000002980232398641646796022541821002960205078125f,0.51970002439543616645778456586413085460662841796875f,-0.984800014612078999931554790236987173557281494140625f,0.8712000231176617592865341066499240696430206298828125f,-0.1806999935984610061723287799395620822906494140625f,-0.40620003498271162190036420724936760962009429931640625f,0.12709997043907594882483635956305079162120819091796875f,0.012600021602212907456674884087988175451755523681640625f,-0.0285000143349172230244903403217904269695281982421875f,0.39320001494437362143941072645247913897037506103515625f,0.06969999781548974482348057790659368038177490234375f,1.280000012107193274601968369097448885440826416015625f,-0.3476000204473737031918290085741318762302398681640625f,-1.06559998056441518343717689276672899723052978515625f,0.818799997669458434756961651146411895751953125f,-0.388899996435642325565140708931721746921539306640625f,-0.081599969679116757248493740917183458805084228515625f,1.87390006404966147357527006533928215503692626953125f,1.157900023041666059953058720566332340240478515625f,1.096000039815903104312155846855603158473968505859375f,0.42120001529455208810759359039366245269775390625f,0.15870000858902955087614827789366245269775390625f,-0.4927000282287590238183838664554059505462646484375f,0.708199994045495717642779709422029554843902587890625f,0.93870004047751454123726944089867174625396728515625f,-0.13330000614821901905315826297737658023834228515625f,-0.16569995709061569044706629938445985317230224609375f,-1.0570000315755605502232583603472448885440826416015625f,-0.02119999204874023490674517233856022357940673828125f,0.4863000172644855734205293629202060401439666748046875f,-0.746599964314698549827653550892136991024017333984375f,0.10449998463690268746262290733284316956996917724609375f,0.73470000712871552650540252216160297393798828125f,-0.040599989579618078749945198069326579570770263671875f,0.6527000759124756967821667785756289958953857421875f,0.03819998849481354152857193184900097548961639404296875f,0.5214000366687780996244327980093657970428466796875f,-0.5162000103957951058220032791723497211933135986328125f,0.29439996091127351718341742525808513164520263671875f,0.135600025919825117881600817781873047351837158203125f,-0.31650001353025469796875768224708735942840576171875f,0.008299996406585019581569895308348350226879119873046875f,0.3683999888584017501358403023914434015750885009765625f,0.2626999958932401124656053070793859660625457763671875f,0.0370999925673007879822762333787977695465087890625f,-0.8343000277847050227819636347703635692596435546875f,-0.757600006812811255230144524830393493175506591796875f,1.5223000152081247460245094771380536258220672607421875f,-0.95440003474652723713944624250871129333972930908203125f,0.30669999418705717886268757865764200687408447265625f,0.047299995389580740123847135691903531551361083984375f,-1.330000036023557097308867014362476766109466552734375f,0.2797000311940911476682458669529296457767486572265625f,0.13639999453574425114510404455359093844890594482421875f,0.04560001895353149592438057879917323589324951171875f,-0.1507999533981074780086828468483872711658477783203125f,0.734299995076657552317556110210716724395751953125f,0.460299973127245465320811490528285503387451171875f,-0.107699989262223017050246198778040707111358642578125f,-0.988900028771162009633144407416693866252899169921875f,-1.51630001398622926700454627280123531818389892578125f,0.1016999864757066607268143343389965593814849853515625f,-0.311299957510829283791053967433981597423553466796875f,-0.081599994564056199664037194452248513698577880859375f,-0.084600003519653910188935697078704833984375f,1.120300006952880789157234175945632159709930419921875f,-1.0843000285297630380654254622641019523143768310546875f,0.867700011111050439893688235315494239330291748046875f,-1.1311000098228465304828205262310802936553955078125f,-0.67610003968104737115396574154146946966648101806640625f,0.430799995642900057646329514682292938232421875f,-0.6211999839834863035292755739646963775157928466796875f,-0.21510002126693716917316123726777732372283935546875f,-0.586999980390071396385565094533376395702362060546875f,0.45489996383190156592490893672220408916473388671875f,0.7322000089354803176178165813325904309749603271484375f,-1.20199996042251644468024096568115055561065673828125f,0.90329998180344739378000440410687588155269622802734375f,0.3878000375188890291866528059472329914569854736328125f,0.58850004477053985407764002957264892756938934326171875f,0.788400024807452748376590534462593495845794677734375f,2.377499988637863825857721167267300188541412353515625f,-0.5896000179737803659207884265924803912639617919921875f,-0.434800070528685633775012320256792008876800537109375f,0.426100008481741010513133005588315427303314208984375f,1.209499959304929372905235140933655202388763427734375f,-0.511999983966350047381865806528367102146148681640625f,-1.73449993252009182498341033351607620716094970703125f,0.7852999896675345592456096710520796477794647216796875f,-0.4832999641641981813933170997188426554203033447265625f,-1.7206000155076377478025051459553651511669158935546875f,-0.2329000148087738697455506553524173796176910400390625f,0.59579998103976183898566887364722788333892822265625f,0.585200036349891927756061704712919890880584716796875f,0.1240999951601029493986061424948275089263916015625f,-1.02320001736581378537493947078473865985870361328125f,-0.31810004445314365995045591262169182300567626953125f,0.2846999978154898247595383509178645908832550048828125f,0.020099993997812592994023361825384199619293212890625f,0.65290001421272780834925697490689344704151153564453125f,0.4872000370055442619587893204879947006702423095703125f,-1.1793000242456794790513185944291763007640838623046875f,0.1337999710336326142368079672451131045818328857421875f,0.185100008358805856456541505394852720201015472412109375f,0.06690003288388179925050280871801078319549560546875f,0.00049993998184738597245768687571398913860321044921875f,-0.2672999860763543011898946133442223072052001953125f,-0.075600004424899669430715221096761524677276611328125f,0.32400004541873972385701563325710594654083251953125f,0.1527000043682755769935965872718952596187591552734375f,-0.170699983912706354516330975457094609737396240234375f,0.2792000118263067154344980735913850367069244384765625f,-0.3321000078409912514842972086626105010509490966796875f,-0.210600010143220550684617364822770468890666961669921875f,0.25399997996538836719793152951751835644245147705078125f,-0.00250000808388073370025495023583061993122100830078125f,-0.856199964407086344664321586606092751026153564453125f,0.1253000441685310217820870093419216573238372802734375f,0.880799938720464137276167093659751117229461669921875f,-1.290000040195882657911852220422588288784027099609375f,-0.1471000016570094270917934409226290881633758544921875f,0.68379999524801970967047282101702876389026641845703125f,-0.134400002858042544318095679045654833316802978515625f,0.3939000131621959166494661985780112445354461669921875f,-0.0941000050693749923169661997235380113124847412109375f,0.51739996625333983804040371978771872818470001220703125f,0.2705999791860584924307886467431671917438507080078125f,-0.693600040444732446331954633933492004871368408203125f,-1.90580003542900033863816133816726505756378173828125f,-0.3292000415354972009396306020789779722690582275390625f,-1.44440000725388539848381697083823382854461669921875f,0.121099982330203470581864166888408362865447998046875f,-1.43139997304081934004216236644424498081207275390625f,-0.5696999996781342279206228340626694262027740478515625f,1.051800035607815431148992502130568027496337890625f,0.0800000089034440531321479284088127315044403076171875f,-0.203800016295910157992921085678972303867340087890625f,0.2713999971434473135190046377829276025295257568359375f,0.610500007502734032271973774186335504055023193359375f,-1.38620005260333467589362044236622750759124755859375f,0.63200003314018271538543558563105762004852294921875f,-1.05970000476315551196648812037892639636993408203125f,0.6579999957680708888574372394941747188568115234375f,-0.87479998799487923566431391009245999157428741455078125f,0.67979997087717070058943136245943605899810791015625f,0.5348000542491677666845362182357348501682281494140625f,-0.1961999893665318239754924434237182140350341796875f,0.3436999925144015310962686271523125469684600830078125f,-1.0373000452339653509881145510007627308368682861328125f,-0.1773999980673193477276328167135943658649921417236328125f,-0.43690000092089231742420452064834535121917724609375f,0.7674999544396985395877663904684595763683319091796875f,-0.15910004269480726435403994400985538959503173828125f,0.40089993446916316610639796635950915515422821044921875f,-0.42480001669824096754979336765245534479618072509765625f,0.0043999767437583603424400280346162617206573486328125f,-0.049900005406140923724933600169606506824493408203125f,-0.38449999171495330418935054694884456694126129150390625f,-0.945799966436624561794133114744909107685089111328125f,0.375999992281198291976807013270445168018341064453125f,1.498000020541251142702776633086614310741424560546875f,0.360700007586926274516514467904926277697086334228515625f,-1.67800007027387731994849673355929553508758544921875f,0.2324999858438971944707418515463359653949737548828125f,-2.828400003647804084039307781495153903961181640625f,0.375899983680248350736974316532723605632781982421875f,-1.7721999996781345743102065171115100383758544921875f,-0.964100050964951993393015072797425091266632080078125f,0.7462000313878061064087887643836438655853271484375f,-0.320099994221329620103233537520281970500946044921875f,-0.9064000286966555908207965330802835524082183837890625f,-0.24600004890561155690420491737313568592071533203125f,-0.12420000852942525426669817534275352954864501953125f,-0.59049994309246478518815592906321398913860321044921875f,0.32780003846883831730707470342167653143405914306640625f,0.21690005627870601045970033737830817699432373046875f,-0.08170000331401805482300915173254907131195068359375f,0.36949998411536155717982410351396538317203521728515625f,-1.1864000316768883447338112091529183089733123779296875f,-0.529000007718801068534730802639387547969818115234375f,-0.4663999683469535195712296626879833638668060302734375f,-0.0136999923467637785279293893836438655853271484375f,-0.9295000007301563726969106937758624553680419921875f,-0.8370999967396262064767142874188721179962158203125f,1.037800035786629049283646963885985314846038818359375f,2.0655000394284730447225228999741375446319580078125f,0.5322999789237972123601139173842966556549072265625f,-0.5462000236392015040109981782734394073486328125f,-0.46959996887445498714441782794892787933349609375f,0.1838999904751779101985675879404880106449127197265625f,-0.8033999804764988539318437688052654266357421875f,-0.415500002697110126792523487893049605190753936767578125f,0.3588999841049311800844634490204043686389923095703125f,0.395600015246868164098259512684307992458343505859375f,0.29990004467070097149417051696218550205230712890625f,0.3847999980717897727799936546944081783294677734375f,0.512800008442997778956851107068359851837158203125f,0.141300017897785179510350417331210337579250335693359375f,0.568600035750865373529450153000652790069580078125f,0.988400008416175257508484719437547028064727783203125f,-0.9848999718785282908584122196771204471588134765625f,0.47790000005662391924232679230044595897197723388671875f,-0.1099000033944843579547523404471576213836669921875f,0.377300039906800999123248629985027946531772613525390625f,0.252799980093538323444590787403285503387451171875f,-0.5384000460416087374682092558941803872585296630859375f,0.6600000097975138313444176674238406121730804443359375f,-0.40170001575648794922557272002450190484523773193359375f,-0.24529995673596882710398858762346208095550537109375f,-0.0351000241354118003300754935480654239654541015625f,-1.199699977847933585195505656884051859378814697265625f,-0.508499990940094459546116922865621745586395263671875f,-2.02279999257326092987341326079331338405609130859375f,-0.0232999591723082044580905858310870826244354248046875f,0.80720001931041540643718690262176096439361572265625f,-0.46880002937167930543438387758214958012104034423828125f,-0.8982000532776119872124809262459166347980499267578125f,-0.5501000214904545604355234900140203535556793212890625f,0.89719999517053372617425566204474307596683502197265625f,-0.7421000361904501740895057082525454461574554443359375f,-0.9641000013068323770681899986811913549900054931640625f,0.12290000687390534306331346670049242675304412841796875f,-1.1774000259324901573876331895007751882076263427734375f,0.44379999498724920226067069961572997272014617919921875f,0.188299985510111067554817054769955575466156005859375f,-0.7305000310838227761678353999741375446319580078125f,0.15449996325373671623992777313105762004852294921875f,0.096399988612532350185091445382568053901195526123046875f,-0.872100001209973552107612704276107251644134521484375f,-0.1132999891608952314214775469736196100711822509765625f,0.271200014698505409427298218361102044582366943359375f,-0.008000008508563194453699907171539962291717529296875f,1.309699999380111279378979816101491451263427734375f,1.008900022065639223711741578881628811359405517578125f,-1.4319000413030387175439273050869815051555633544921875f,-0.06789997859895169352029142828541807830333709716796875f,-0.1409000045046211457844265169114805757999420166015625f,-0.21260000323653205356322359875775873661041259765625f,0.426100008407235275598168300348334014415740966796875f,0.73360005802810313024764354850049130618572235107421875f,-0.917400038039684506685489395749755203723907470703125f,0.828700023564696408584495657123625278472900390625f,-0.3636000329941513342646430828608572483062744140625f,-1.05580004258155835117349852225743234157562255859375f,1.2935000311732292033184421597979962825775146484375f,-0.9597000381231310939256218262016773223876953125f,-0.8352000154137613208149559795856475830078125f,-0.55889999740421780760613046368234790861606597900390625f,0.19479999497979860745289215628872625529766082763671875f,0.111700023579597740308599895797669887542724609375f,-1.162000042304396796311038997373543679714202880859375f,-0.334900000339746650723782295244745910167694091796875f,-0.881100016677379471730091609060764312744140625f,-0.3771000037431715412594712688587605953216552734375f,0.169399993768334866661717796887387521564960479736328125f,-1.180700021463632509721719543449580669403076171875f,-0.454200054872036407260793566820211708545684814453125f,-0.164799995911121044400715618394315242767333984375f,1.533600006917118729887761219288222491741180419921875f,-0.308199999335407870404424102162010967731475830078125f,0.71980000693798018573943409137427806854248046875f,-0.70289999424517102255549616529606282711029052734375f,0.058399992918968024468995281495153903961181640625f,1.1336000092267994876493730771471746265888214111328125f,-0.0608999729514128063101452426053583621978759765625f,0.4566000219076873367640700962510891258716583251953125f,-0.6388999961376189418160720379091799259185791015625f,-0.3288999894320963068850005583954043686389923095703125f,0.195699988830089210978258051909506320953369140625f,0.277199993196129756967138746404089033603668212890625f,1.0792000447764997783650642304564826190471649169921875f,-1.4921000792920597888269185204990208148956298828125f,2.5162000570178033598267575143836438655853271484375f,0.68779999540448155670446794829331338405609130859375f,0.892700053858757058833361952565610408782958984375f,-0.83290004319548671674056095071136951446533203125f,0.1853000480055815302193877869285643100738525390625f,-0.057199994462728387389915951644070446491241455078125f,-0.63160003911852857783060244400985538959503173828125f,0.4636000141441822819388107745908200740814208984375f,-0.5981000271677974211570472107268869876861572265625f,-0.609000016510486741339036598219536244869232177734375f,0.2808999985069033922258086022338829934597015380859375f,-0.25089999440908439254371842253021895885467529296875f,1.032000003710388913447104641818441450595855712890625f,-0.75440003362894092475698926136828958988189697265625f,-0.80360000751316551514946695533581078052520751953125f,-1.20130001510381756446577128372155129909515380859375f,0.28310001398026951591191391344182193279266357421875f,1.0578000247597696414914025808684527873992919921875f,1.1578000533699996932313069919473491609096527099609375f,0.692200007873773781597037668689154088497161865234375f,-0.22999999068677379199243659968487918376922607421875f,0.365700021165609623352565904497168958187103271484375f,0.3771000303044915735739550655125640332698822021484375f,-0.391699982005357494330155532225035130977630615234375f,1.10950000691413919184924452565610408782958984375f,0.910899972504377597459779281052760779857635498046875f,-1.085899969524145802779457881115376949310302734375f,-1.0645999987512821949309227420599199831485748291015625f,-0.8523999934852124482631552382372319698333740234375f,0.4395999846696856394601127249188721179962158203125f,-0.87089993316531266742686057114042341709136962890625f,-0.15499998666346093134649208877817727625370025634765625f,0.54189997745156315289705162285827100276947021484375f,0.4392000214934348445439127317513339221477508544921875f,-1.273300018274038603038889050367288291454315185546875f,-0.8266000695168980172411465900950133800506591796875f,0.4869000181317328923569220933131873607635498046875f,-0.1892000009298324680884206827613525092601776123046875f,-0.83389999032616657359540113247931003570556640625f,1.4420000549703841929982672809273935854434967041015625f,-1.055100050994754123934171730070374906063079833984375f,0.1886999832570543933485396337346173822879791259765625f,-0.6388999737858771510445876629091799259185791015625f,0.936899999728797894249510136432945728302001953125f,-0.845199993881582845034472484258003532886505126953125f,-1.489200005698202655679551753564737737178802490234375f,-1.008200037594139786278901738114655017852783203125f,-0.90809999602436963783702594810165464878082275390625f,-1.52549997781216983838703526998870074748992919921875f,-0.133799942162632579112369057838805019855499267578125f,0.201900042048097194680167376645840704441070556640625f,1.2415000106245275279803763623931445181369781494140625f,-1.005599987232684622284750730614177882671356201171875f,0.127999994799494754005309005151502788066864013671875f,0.11250001050531832635925866270554251968860626220703125f,-0.2249000005632633136798403938882984220981597900390625f,-0.313299990874528422324374332674778997898101806640625f,-0.6655999851465228545066565857268869876861572265625f,-1.1420999940574161346518167192698456346988677978515625f,0.8374999622255561160244496932136826217174530029296875f,-0.10249998629093237667575522209517657756805419921875f,0.27630000020265566007537927362136542797088623046875f,0.1182999555587773699016906903125345706939697265625f,-0.938499990418553142745849982020445168018341064453125f,0.2076999450802805569082920555956661701202392578125f,-0.062200006681680353182173348614014685153961181640625f,0.07950002047419513218073916505090892314910888671875f,0.9261000160813335213561003911308944225311279296875f,0.000800028797983909356617004959844052791595458984375f,-0.55870002214908598414666585085797123610973358154296875f,-0.4695000706538561274783205590210855007171630859375f,0.352100003296136332409105307306163012981414794921875f,-0.949899935370684289637210895307362079620361328125f,-0.74319999553561189031825051642954349517822265625f,0.282899994170666246873224736191332340240478515625f,0.410500032722950880526013861526735126972198486328125f,0.4192999858379362621008112910203635692596435546875f,0.0050999844610688516155505567439831793308258056640625f,0.12180002070665363334001085604541003704071044921875f,-0.06540000713467619686269927115063183009624481201171875f,-0.607399995720386609576735281734727323055267333984375f,-0.411800025437772720948714777478016912937164306640625f,-0.2457999973565333229341689502689405344426631927490234375f,-0.88519998844265845860945773893035948276519775390625f,-0.35059999043643530480807157800882123410701751708984375f,0.8619000409305090926181946997530758380889892578125f,1.5799000451177354076293113394058309495449066162109375f,0.496999994546174495013701744028367102146148681640625f,-0.6993000074446200642341864295303821563720703125f,-0.8123000026166440346031549779581837356090545654296875f,-0.859800007757544282327444307156838476657867431640625f,-0.69060000436902047038500995768117718398571014404296875f,0.47960003667473838362411697744391858577728271484375f,0.6137000303447248672483738118899054825305938720703125f,-0.42630001368820646945323460386134684085845947265625f,-0.72240002567172023617558807018212974071502685546875f,-1.7635000186562539425949580618180334568023681640625f,-0.3637000147730109489430105895735323429107666015625f,-1.425499985039234474015756859444081783294677734375f,-0.7587999969989061010977593468851409852504730224609375f,-0.952199967193603669102230924181640148162841796875f,0.98580004027187850823565895552746951580047607421875f,0.254700021433830414707699674181640148162841796875f,-0.851200036230683299010024711606092751026153564453125f,0.6081000160664313813896342253428883850574493408203125f,0.29240000067502258662699432534282095730304718017578125f,0.1400000301375980971130275065661408007144927978515625f,1.378700057166815096110212834901176393032073974609375f,-1.1376000159025192459694153512828052043914794921875f,1.256100062870979972018403714173473417758941650390625f,-0.9153000304222107530449648038484156131744384765625f,0.810500000238417950271241352311335504055023193359375f,0.17189998371005099642161439987830817699432373046875f,0.35359999730586988420100169605575501918792724609375f,-1.1948999909520150453090536757372319698333740234375f,-0.6128000398471957765877959900535643100738525390625f,0.6008000053286550734554793962161056697368621826171875f,0.5324000204563141824820604597334749996662139892578125f,-0.02889994670301654355881737501476891338825225830078125f,1.617100055748224018969949611346237361431121826171875f,-0.401899961060285360048283109790645539760589599609375f,0.960100005173682546910640667192637920379638671875f,-0.1616000039517881958772704820148646831512451171875f,0.99769998367428769370235386304557323455810546875f,0.2362999758392572147158716688863933086395263671875f,-1.27440005970597258766474624280817806720733642578125f,-0.7505999814957373672541507403366267681121826171875f,0.47840005639791538527560987859033048152923583984375f,-0.4831000067532063013686638441868126392364501953125f,-0.58370002458915137832917707783053629100322723388671875f,-1.6003999888062470091654176940210163593292236328125f,-0.96770000356733820723320604884065687656402587890625f,-2.427999979972838673347723670303821563720703125f,-0.442100001806020603822844350361265242099761962890625f,-1.6918999957799911726397112943232059478759765625f,0.781000028416514613383014875580556690692901611328125f,-0.223900013422966281950721167959272861480712890625f,-0.8488000144332648044809275234001688659191131591796875f,-0.428700017604231486956223307061009109020233154296875f,0.4064999848455190800677883089520037174224853515625f,-0.29069998942613661796485757804475724697113037109375f,0.708600010940432678552269862848334014415740966796875f,0.282099965223670035019409851884120143949985504150390625f,0.43179993234276725644349426147527992725372314453125f,-1.235499972075224217604727527941577136516571044921875f,-0.017300007720292054358424138627015054225921630859375f,0.4537999532639978017556359191075898706912994384765625f,-0.0389000169992457234258154130657203495502471923828125f,-1.253900001353025484007730483426712453365325927734375f,0.2521000139504663284384378130198456346988677978515625f,1.00739995131492587887578338268212974071502685546875f,-0.5608000311821699224168469299911521375179290771484375f,-1.4350999920606615223306334883091039955615997314453125f,1.1212999994575982842803796302177943289279937744140625f,-2.13420002849698153823965185438282787799835205078125f,0.1174000033944853083056614195811562240123748779296875f,-0.4649999626725909163127425927086733281612396240234375f,0.236700029987097215755653678570524789392948150634765625f,1.479000060617924550143698070314712822437286376953125f,-0.4326000235021119078027140858466736972332000732421875f,0.10150003558397369118893038830719888210296630859375f,-0.8217000106155871730351236692513339221477508544921875f,0.813000003516673785952662001363933086395263671875f,-0.30510000666379966105523635633289813995361328125f,-0.14949999052286155887259155861102044582366943359375f,0.8489000185638666540199892551754601299762725830078125f,1.626399999080599290124382605426944792270660400390625f,0.0258000549681487445496941290912218391895294189453125f,-0.2397000066071750534746342964353971183300018310546875f,0.323199980168789036127918734564445912837982177734375f,0.0320999796777954315984970889985561370849609375f,0.0896000025138261602108968872926197946071624755859375f,-0.13430001623630527518571398104541003704071044921875f,-0.27220001752227585001264742459170520305633544921875f,0.846099987471102821245949598960578441619873046875f,0.4335000222325329222172740628593601286411285400390625f,-0.81989999125003709679049279657192528247833251953125f,0.196199979252367706639148536851280368864536285400390625f,-1.000800026115774787882628515944816172122955322265625f,-0.87499999072402590005737010869779624044895172119140625f,-0.4328999891787768117268342393799684941768646240234375f,0.499500039286911334812657514703460037708282470703125f,0.12170000234544264827718507149256765842437744140625f,-1.6850000316649680343061845633201301097869873046875f,0.608999994978308922810583680984564125537872314453125f,0.00599996061623031540221973045845516026020050048828125f,-0.4797999585092067587055453259381465613842010498046875f,-1.7093000071465969114115068805404007434844970703125f,0.755299980354309230534681773860938847064971923828125f,-0.32229994655102511824651401184382848441600799560546875f,1.1008000096127392719580484481411986052989959716796875f,0.316300012794136886640217198873870074748992919921875f,0.5391000110298393988017551237135194242000579833984375f,-1.004600000092386924421816729591228067874908447265625f,0.369000027090311544242240415769629180431365966796875f,-0.76480003785788941428336329408921301364898681640625f,-1.653899979746341841746470890939235687255859375f,-0.72440000524222813993446834501810371875762939453125f,-0.7035000237226487485031611868180334568023681640625f,0.08300000180304056840441262465901672840118408203125f,-0.47290003201961550161058767116628587245941162109375f,0.4608000019013875903084453966584987938404083251953125f,0.4405000380501153944834413778153248131275177001953125f,0.8110000085793434010383862187154591083526611328125f,-1.3926000205963851019674848430440761148929595947265625f,-0.54490000339448385613394520987640134990215301513671875f,0.7564999525099995736354685504920780658721923828125f,-1.1442999795608221891285438687191344797611236572265625f,0.325199962365626760174563969485461711883544921875f,0.0481000309117141089387814645306207239627838134765625f,-0.767400002276897197361904545687139034271240234375f,-0.02660001563057345919105500797741115093231201171875f,-0.50409998413324341726138300145976245403289794921875f,0.00239997579008299766201162128709256649017333984375f,-1.030200001890957395289660780690610408782958984375f,-0.6876999667242171820902285617194138467311859130859375f,1.0582999990701675852022844992461614310741424560546875f,1.230699948671460930427201674319803714752197265625f,-0.221699984762072421773382302490063011646270751953125f,0.647199947114289297900313613354228436946868896484375f,-0.2701999755531545588382869027554988861083984375f,-0.203800028328597004900757383438758552074432373046875f,0.3146000187188386920666971491300500929355621337890625f,1.283300026674568439233325989334844052791595458984375f,0.29789998686909680003509492962621152400970458984375f,1.1698000512689343821648435550741851329803466796875f,0.1786000107169147810282083810307085514068603515625f,-1.00530001134872382095863940776325762271881103515625f,-1.4028000348925591111992616788484156131744384765625f,0.396299981501698539432254619896411895751953125f,-1.924599964217841741032088975771330296993255615234375f,-1.0016000158354632088020252922433428466320037841796875f,-1.1170000134706496996983560165972448885440826416015625f,0.13190001015961183572500203808885999023914337158203125f,-0.026800003793835625032215830287896096706390380859375f,-0.2758000071167945321803927072323858737945556640625f,-1.165599993789195654159129844629205763339996337890625f,-0.6167000290185213895455262900213710963726043701171875f,1.5559999894499778605450046597979962825775146484375f,-1.137900004433840539519451340311206877231597900390625f,0.514399982666968735856016792240552604198455810546875f,-1.2345999965906135908966234637773595750331878662109375f,0.39659996412843578905693675551447086036205291748046875f,-0.6578999793440105658959282664000056684017181396484375f,0.5352000088199968530489059048704802989959716796875f,-0.25150000272691241942624174043885432183742523193359375f,-0.021399997925758196259948817896656692028045654296875f,-0.711600020585209325219011589069850742816925048828125f,0.055999994292855514999018851085565984249114990234375f,-0.7226000031061470263438195615890435874462127685546875f,-1.2543999792397020343059921287931501865386962890625f,1.2897999534055595294290696983807720243930816650390625f,-0.5313999736368659743135367534705437719821929931640625f,1.009699957135320236289999229484237730503082275390625f,0.05219995989203496389308156722108833491802215576171875f,-0.25220002467483215724541878444142639636993408203125f,-0.493299973216652976049090284504927694797515869140625f,1.4210999901533127864894368030945770442485809326171875f,-0.494500004306435758660853707624482922255992889404296875f,0.55529998732060192878634552471339702606201171875f,0.4452999727547168884456141313421539962291717529296875f,0.42399995999783268718630324656260199844837188720703125f,-1.3648000264957549365618660885957069694995880126953125f,-0.54490002298951178971719855326227843761444091796875f,0.6966999961614614989002802758477628231048583984375f,0.1744999999105927290798945250571705400943756103515625f,-0.3025000401772561797741900591063313186168670654296875f,0.983799997911602108757733731181360781192779541015625f,-0.03109995919614993997726060115383006632328033447265625f,-0.3688999845519658205006408024928532540798187255859375f,1.51280001283884057983186721685342490673065185546875f,-0.0157000096172092984403434456908144056797027587890625f,-0.7398999870404601164608493490959517657756805419921875f,-0.1239000178188083278740805326378904283046722412109375f,1.1163000259444120221274943105527199804782867431640625f,0.2740000133439897123110995380557142198085784912109375f,0.0870000733733178321926970966160297393798828125f,0.18639997691512111277489793792483396828174591064453125f,-0.9918000472866002059646461930242367088794708251953125f,0.32920002037584794596369874852825887501239776611328125f,-0.324799992167204454318607531604357063770294189453125f,0.5611000148706140056020785777946002781391143798828125f,0.1283999689280979072947275199112482368946075439453125f,-0.365600001165270693803677204414270818233489990234375f,0.1486999825492503202184479960124008357524871826171875f,-0.875299986407905183938282789313234388828277587890625f,-0.38500002864748250619442160314065404236316680908203125f,0.037200030300021058593529232894070446491241455078125f,-1.3344000126928092786471324870944954454898834228515625f,0.88899999110400618196337063636747188866138458251953125f,1.303600013138353830299820401705801486968994140625f,-0.83539995652660703484571058652363717555999755859375f,-0.02210002948492750807218953923438675701618194580078125f,-0.2146000161856409516047961005824618041515350341796875f,-0.12799994767457245625763562202337197959423065185546875f,-0.00170001992881285701741944649256765842437744140625f,0.42479999348968300321160995736136101186275482177734375f,0.90659996122270880558602357268682681024074554443359375f,0.62100002729892711528236759477294981479644775390625f,0.491100027182698628536172691383399069309234619140625f,0.509200002121925621167974895797669887542724609375f,1.14349999660253498490192214376293122768402099609375f,0.4416000261545178773303632624447345733642578125f,-0.514900001531839013324542975169606506824493408203125f,-0.149300004166363908808534688432700932025909423828125f,1.169999991878867273698006101767532527446746826171875f,0.14800001126527728700921215931884944438934326171875f,-0.6917999827086926956098977825604379177093505859375f,1.044299976971745369525024216272868216037750244140625f,0.512199970620870193016571647603996098041534423828125f,-0.5831000085413453604132882901467382907867431640625f,0.60350000638142209385961223233607597649097442626953125f,-0.1100000044703488555342119070701301097869873046875f,0.0237999951362610762117810736526735126972198486328125f,0.7948000075340271308732553734444081783294677734375f,-0.1456999858871099728929721095482818782329559326171875f,0.660000001937150582165259038447402417659759521484375f,0.31060000936090925716115407340112142264842987060546875f,-0.130599991852045516083080656244419515132904052734375f,0.14549998481571668751399784014211036264896392822265625f,0.371800017204880983712200759327970445156097412109375f,-0.98300001420825677822534771621576510369777679443359375f,0.390000005364418367292955736047588288784027099609375f,-0.7945999734565611927195050157024525105953216552734375f,0.4788000130921601371625229148776270449161529541015625f,-0.051099981212616274461879584123380482196807861328125f,-0.6174000138252970781849171544308774173259735107421875f,-0.4956000266462556869129230108228512108325958251953125f,-0.335699979144334914593628127477131783962249755859375f,-0.173399979582429530911014126104419119656085968017578125f,-0.1929999953955408020789263900951482355594635009765625f,0.5024999988079070778468349089962430298328399658203125f,-0.48109999060034791540374499163590371608734130859375f,0.3128999850064515531045117313624359667301177978515625f,1.3103000071763997169682625099085271358489990234375f,0.124700002360344175400541644194163382053375244140625f,-0.33019993420243221038390402100048959255218505859375f,0.378399999624491112371060808072797954082489013671875f,-0.2285999591588971924238649080507457256317138671875f,-0.139000012263655658983907414949499070644378662109375f,-0.803700012016296483352562063373625278472900390625f,-0.39260004525780711759352925582788884639739990234375f,-0.542699962961673776362658827565610408782958984375f,-0.888100033575296432530876700184307992458343505859375f,-0.4024000350594523212066633277572691440582275390625f,0.376700009050964457202326229889877140522003173828125f,-0.27619999473094924979932329733856022357940673828125f,0.09110001004636314458906554136774502694606781005859375f,-0.59650004169344850168954508262686431407928466796875f,-0.593200000117719117298520359327085316181182861328125f,1.3699999812245362562634909409098327159881591796875f,-0.75100001275166883285550056825741194188594818115234375f,1.4762000209569929864983350853435695171356201171875f,-1.4926000211551782026475621023564599454402923583984375f,2.234200017917157055080679128877818584442138671875f,0.13900000865012318573832317269989289343357086181640625f,-0.7937999811291707175087140058167278766632080078125f,-0.11140004186555707565986494955723173916339874267578125f,-0.677400004065037197875653873779810965061187744140625f,-0.9002000137746331009935829570167697966098785400390625f,1.4184999795407062794794228466344065964221954345703125f,-0.7794000208884475089377019685343839228153228759765625f,-0.541199989143013926451430961606092751026153564453125f,-0.9702000465199354106715645684744231402873992919921875f,-0.536699995192885470629562405520118772983551025390625f,1.567099970886111304224641571636311709880828857421875f,-1.8154000330626960391100510605610907077789306640625f,-0.9720999623924500010474503142177127301692962646484375f,-0.0337999730825426780711495666764676570892333984375f,0.8658999941483134232811380570637993514537811279296875f,1.111800025959313042989151654182933270931243896484375f,-0.66020000472217754694526092862361110746860504150390625f,-0.4691999885618687304855711772688664495944976806640625f,-0.3122000281393528808848714106716215610504150390625f,-0.7486999795317661465787750785239040851593017578125f,-0.094099980035423858026888410677202045917510986328125f,-0.6531999572396269826413117698393762111663818359375f,-0.263400024583936076538748238817788660526275634765625f,0.10169994102716373163275420665740966796875f,0.974900023883580768568890562164597213268280029296875f,0.01559997292757042686162094469182193279266357421875f,0.3947999948680400716938265759381465613842010498046875f,1.19040001801252426361088510020636022090911865234375f,0.4925000195950273162992516517988406121730804443359375f,-0.316899984976649395918002483085729181766510009765625f,1.18930003292560559913226825301535427570343017578125f,0.50350002975761898316164888456114567816257476806640625f,-0.53440000710487378654534040833823382854461669921875f,-0.083399996644259122291487074107863008975982666015625f,1.78860005363225926799941589706577360630035400390625f,1.002399995645880448336129120434634387493133544921875f,-0.52300005120039028128076097345910966396331787109375f,-1.082000035151839512082005967386066913604736328125f,0.0870000063180922378336390465847216546535491943359375f,-2.08349999012053022084955955506302416324615478515625f,0.4405000305622817080575259751640260219573974609375f,-0.538499960318207460119310781010426580905914306640625f,1.157900029411912168342269069398753345012664794921875f,0.5689999957978724154372685006819665431976318359375f,-0.02299996276199767208936464157886803150177001953125f,0.2759999662041661139255666057579219341278076171875f,0.36319998435974110151391869294457137584686279296875f,0.0993999802827829626039601862430572509765625f,-0.2136999819904563668870878245797939598560333251953125f,0.3680999842524528276044293306767940521240234375f,0.29620001961588826588922529481351375579833984375f,-0.3121000364884733979664588332525454461574554443359375f,1.54580003408789590224614585167728364467620849609375f,-1.488500013440847258294752464280463755130767822265625f,-1.65120007191896522869001273647882044315338134765625f,0.328800019499659423871662511373870074748992919921875f,-0.023899981087445620886455799336545169353485107421875f,-0.105800058972835842041604337282478809356689453125f,-1.05120003019571317537383947637863457202911376953125f,0.4328000295281402287628225167281925678253173828125f,0.68830003721714039244261584826745092868804931640625f,-0.6446000284790986967209391877986490726470947265625f,-1.25570005275607154970884948852472007274627685546875f,-0.30180004279762562280353677124367095530033111572265625f,-0.49400006330013279676194315470638684928417205810546875f,-0.18199999894201734917942303582094609737396240234375f,0.134000035434961972669043461792171001434326171875f,1.64260008377730937212390927015803754329681396484375f,1.256600027942657238355650406447239220142364501953125f,1.39590004708469006544646617840044200420379638671875f,0.8229000469207772283652957412414252758026123046875f,0.93560004378259176149867926142178475856781005859375f,0.427599976414441140804001406650058925151824951171875f,0.19859999358057933704913011752068996429443359375f,0.2372000135362153372398097417317330837249755859375f,0.2000999928057194399144691487890668213367462158203125f,0.55010000062882891125326523251715116202831268310546875f,0.626999999538063956805444831843487918376922607421875f,-0.218899978405236783629561614361591637134552001953125f,-0.12669999750256533133097036625258624553680419921875f,-0.395600022846460230852017048164270818233489990234375f,-1.6862999408960348546315799467265605926513671875f,-0.572200017410516625915306576644070446491241455078125f,-0.2594999759197236244290252216160297393798828125f,-0.47010003482699413979162272880785167217254638671875f,-0.1741999809622765127414822927676141262054443359375f,1.5130999841034407094042535391054116189479827880859375f,-0.12849996713548872406818190938793122768402099609375f,0.00510001292228701164077619978343136608600616455078125f,-0.887600020968914105168323658290319144725799560546875f,-1.228800033134221703079447252093814313411712646484375f,0.635599979111552304544829894439317286014556884765625f,1.62270002092718979014307478792034089565277099609375f,-1.714000000715255822569815791212022304534912109375f,1.1781000297009942556769601651467382907867431640625f,-1.196599995160102736235785414464771747589111328125f,0.186099981212615173120639155968092381954193115234375f,-0.17599996680021234851665212772786617279052734375f,0.296800048273802463683068708633072674274444580078125f,-0.36499999083578582581566251974436454474925994873046875f,-0.18179998196363433837774437051848508417606353759765625f,0.460900018176436443440024959272705018520355224609375f,0.978500001072883041075556320720352232456207275390625f,-0.05649997456371746995529292689752764999866485595703125f,1.1706000100314621903407896752469241619110107421875f,-0.689799986225366534853264965931884944438934326171875f,-0.03120001842379593881560140289366245269775390625f,-1.636400015211105785084555463981814682483673095703125f,-0.465899990236759098394259126507677137851715087890625f,-1.481900029456615808243213905370794236660003662109375f,-0.80279999540448176986728867632336914539337158203125f,-0.433100000643729998728304053656756877899169921875f,-0.5675000341236593470739535405300557613372802734375f,-1.527599974328279319735202079755254089832305908203125f,-0.26950000888854230840507852917653508484363555908203125f,-1.7889999403283003420739305511233396828174591064453125f,-0.964700006085634331753908554674126207828521728515625f,-0.7377999856442212234242106205783784389495849609375f,0.6756999824225904394126018814858980476856231689453125f,-0.681499967932701178341403647209517657756805419921875f,-0.64440000561475729856653060778626240789890289306640625f,-0.833099990957975489180853401194326579570770263671875f,-0.4859000128865240686337756415014155209064483642578125f,0.858000005006790189554521930404007434844970703125f,1.189500034108757819950596967828460037708282470703125f,-1.271599993371963943644686878542415797710418701171875f,1.3637000420421359070388689360697753727436065673828125f,-1.1057000254869453481632035618531517684459686279296875f,1.139599998602270236602862496511079370975494384765625f,-0.3600999796554444909446601741365157067775726318359375f,0.29120001760423186443205167961423285305500030517578125f,-1.082599976563453747502308033290319144725799560546875f,0.5297000002741807378470184630714356899261474609375f,-0.407000005871057002337920494028367102146148681640625f,1.01260003333687809146113067981787025928497314453125f,-0.051100025767087942085709073580801486968994140625f,0.831500013530254822313736440264619886875152587890625f,-1.39619998795092126187000758363865315914154052734375f,1.179799986448883064582560109556652605533599853515625f,0.33249999731779045220037005492486059665679931640625f,0.542599976190925037400347719085402786731719970703125f,-1.295500017710029982964670125511474907398223876953125f,-0.53080000301897545256224475451745092868804931640625f,-0.4924999776482585911452360960538499057292938232421875f,1.1827000227898356587985517762717790901660919189453125f,-0.150699978101253406759241215695510618388652801513671875f,0.361900004571676259246260087820701301097869873046875f,-0.8800000043213367373340361154987476766109466552734375f,0.8724000284284354478359091444872319698333740234375f,-0.1240000144243236235297445091418921947479248046875f,-0.59950004377588650417152393856667913496494293212890625f,0.27190000302568051449014774334500543773174285888671875f,0.63550003749132166763757822991465218365192413330078125f,-0.0116999897912142269973401198512874543666839599609375f,0.41690001202225690501990129632758907973766326904296875f,1.0541999896794556068613246679888106882572174072265625f,-0.725600003437697704811171206529252231121063232421875f,-0.2306999865949157513256295715109445154666900634765625f,0.3460999978274106325670800288207828998565673828125f,0.96600001727789663874546022270806133747100830078125f,0.7029000241220000333441930706612765789031982421875f,-1.289599988543987052480588317848742008209228515625f,-0.44239999776929561647875743801705539226531982421875f,0.17879997341781794784054682168061845004558563232421875f,0.69050002229213724991296885491465218365192413330078125f,-0.51790003390088645840449999013799242675304412841796875f,-1.495100043208896778423877549357712268829345703125f,0.9410000074803821501490119771915487945079803466796875f,0.0860999820694321460479159213718958199024200439453125f,0.477099996329844344700177316553890705108642578125f,0.2252000116884709246534157500718720257282257080078125f,0.1175999746263023393311186737264506518840789794921875f,-1.48930004812479044318251908407546579837799072265625f,-1.2290999979913228568051408728933893144130706787109375f,0.35710006860047627252896518257330171763896942138671875f,-0.5624000106215476524340601827134378254413604736328125f,-0.65660003513246800732616748064174316823482513427734375f,0.80469999481663168072742564618238247931003570556640625f,-0.2804999917447561497141350628226064145565032958984375f,0.44380001912713085143735725068836472928524017333984375f,0.803400009310245355464985550497658550739288330078125f,0.08329999124705789181888349048676900565624237060546875f,0.297499969378113515716677284217439591884613037109375f,0.8303000074744222303024798748083412647247314453125f,-0.506899982666969339817342188325710594654083251953125f,1.1311999845981599577271481393836438655853271484375f,0.574100011849403291108728808467276394367218017578125f,0.157800002408027495448550325818359851837158203125f,-0.86309997158646556414396400214172899723052978515625f,0.25979998384118108134543945197947323322296142578125f,0.63639997900128353336413056240417063236236572265625f,0.067200030225514684190102343563921749591827392578125f,0.2487999996811149916453587138676084578037261962890625f,0.473599968397617754334305573138408362865447998046875f,-1.23530001663863675531729313661344349384307861328125f,0.24429999313950556683039394556544721126556396484375f,-0.9129000510931017942084508831612765789031982421875f,1.755400006091594633517161128111183643341064453125f,-0.5893000174283979930578425410203635692596435546875f,1.1227000040888786003279165015555918216705322265625f,-0.1841000195980069253920419214409776031970977783203125f,1.17009998610019660958414533524774014949798583984375f,0.25019999328553621520399019573233090341091156005859375f,-0.033599970930815015179859983618371188640594482421875f,-1.0780000155121083338372045545838773250579833984375f,0.5024000238090753356345885549671947956085205078125f,0.81580002306103693854311131872236728668212890625f,0.517799981695413524818150108330883085727691650390625f,-0.3525000050663944506368352449499070644378662109375f,0.2931999823480848643697527222684584558010101318359375f,-0.6118000126600264820808661170303821563720703125f,1.343899979746341788455765708931721746921539306640625f,0.1590999793648719684568959564785473048686981201171875f,-0.54940001053213993742474485770799219608306884765625f,-0.975399993872641868364326001028530299663543701171875f,0.607199982243776137380564250634051859378814697265625f,-0.22230000663995763687807993846945464611053466796875f,-0.291499992966652354819956372011802159249782562255859375f,1.4289000032901757464287584298290312290191650390625f,0.02840001467466368723080449854023754596710205078125f,-0.485699975642561287969556360621936619281768798828125f,0.32940001723766254571046374621801078319549560546875f,-0.019299976524710604763868104782886803150177001953125f,-0.293900002694130080271861515939235687255859375f,-0.9386000038623809604843017950770445168018341064453125f,-0.3244000237941744302361257723532617092132568359375f,0.167900018087029589963776743388734757900238037109375f,-1.4889999899864196919452297152020037174224853515625f,0.167799984601139318129980892990715801715850830078125f,-0.421900019025802208716413588263094425201416015625f,-0.19000001877546335293800439103506505489349365234375f,-1.07999999105930388765273164608515799045562744140625f,-0.30880003954172252633725292980670928955078125f,0.355099980884791222024432499893009662628173828125f,-0.30370002214908620175037867738865315914154052734375f,1.1863999958395954337220246088691055774688720703125f,0.080599991703033335710415485664270818233489990234375f,1.5333000187397001212730174302123486995697021484375f,-0.08760000949502000366919673979282379150390625f,0.351300003182888787023330223746597766876220703125f,1.664600055301189485135182621888816356658935546875f,-2.200500020503998399590273038484156131744384765625f,-0.934899974709747727530384509009309113025665283203125f,0.315100055316090532642192556522786617279052734375f,1.40230001647472324322052372735925018787384033203125f,1.2221000301927329445561554166488349437713623046875f,0.8870000011026863973029321641661226749420166015625f,1.88430001958906689907280451734550297260284423828125f,1.0354000051975251750491224811412394046783447265625f,0.79499998994171594546287451521493494510650634765625f,0.64269996802806783620098940446041524410247802734375f,0.3691999870717526111008055522688664495944976806640625f,-0.06319998301863538614497883827425539493560791015625f,-1.023000000461935510287503348081372678279876708984375f,0.75640004918575254322377077187411487102508544921875f,0.7185999609097837304005906844395212829113006591796875f,-0.5566000082731246134670755054685287177562713623046875f,-0.51299998824298409516586616518907248973846435546875f,0.523599976444244052942167400033213198184967041015625f,-1.47809996845722224634300800971686840057373046875f,-0.4101999411314725119126478602993302047252655029296875f,-0.0589000211715688948288516257889568805694580078125f,-0.870700035694242036043988264282234013080596923828125f,0.7307999609231945470355640281923115253448486328125f,0.17250000096857542875028457274311222136020660400390625f,0.02179998703002983262422276311554014682769775390625f,-0.372200006011128348148986333399079740047454833984375f,-0.78179999336302241630392018123529851436614990234375f,-1.3901999672085045833824779037968255579471588134765625f,0.35770003031492247913547544158063828945159912109375f,0.4245999877244239772977607572101987898349761962890625f,0.6777999971181152272237113720620982348918914794921875f,0.43199999398738120337526424918905831873416900634765625f,-1.010899986436962993963106782757677137851715087890625f,-1.06419996978640529761150901322253048419952392578125f,0.92209999800622455268239718861877918243408203125f,1.110799979363382927743941763765178620815277099609375f,0.63380003332048617803451406871317885816097259521484375f,-0.4526999860212208570686698294593952596187591552734375f,-0.29860001220703136226575225009582936763763427734375f,0.1464000036627053358273542471579276025295257568359375f,-0.04470002143383045023483646218664944171905517578125f,0.5336999965190887262878050023573450744152069091796875f,-0.40600002893805520187697766232304275035858154296875f,0.83850000636279542476358983549289405345916748046875f,0.84430000729560816807861556299030780792236328125f,-0.82460000597834604008085079840384423732757568359375f,1.14930000178217905926203457056544721126556396484375f,0.49549999131634836491144824321963824331760406494140625f,-1.50800003592669984442409258917905390262603759765625f,0.24759998963922258408132393014966510236263275146484375f,-0.79769997309446250000064537744037806987762451171875f,0.1252999924056232838864843870396725833415985107421875f,0.11469999490976334755032439716160297393798828125f,-0.159100004771352054877553428013925440609455108642578125f,0.40080002514719925699182567768730223178863525390625f,0.7021000051774084482758553349412977695465087890625f,-0.46589998159408541056336616748012602329254150390625f,0.447299997587502051743513220571912825107574462890625f,-0.19550000813603407578966653090901672840118408203125f,-0.3840000074766576343421320416382513940334320068359375f,-0.33610004498958667706887126769288443028926849365234375f,-0.421999998606741311135692740208469331264495849609375f,0.26509995138049102791910627274774014949798583984375f,-0.03429999258071181333917820666101761162281036376953125f,0.3128000033766031862114687100984156131744384765625f,0.32790001562833770432092705959803424775600433349609375f,-0.606900000324845478871793602593243122100830078125f,-0.35139995318502226240298114134930074214935302734375f,-0.072600016051530769090049943770281970500946044921875f,0.35980002459585680441023214370943605899810791015625f,0.037900012648105718682245424133725464344024658203125f,0.12079998627379529008152303504175506532192230224609375f,0.8472000138528645862834309809841215610504150390625f,0.7729999941661953588578626295202411711215972900390625f,0.7707999560430636165619944222271442413330078125f,-0.6156999751955272159875676152296364307403564453125f,1.00859999626278895590303363860584795475006103515625f,-0.017999986007809720689465393661521375179290771484375f,1.137000003784894630598500953055918216705322265625f,-0.64239999177157880438926440547220408916473388671875f,0.88830001039505024351683459826745092868804931640625f,0.7940000272393230584810908112558536231517791748046875f,0.9139000792860993982458239770494401454925537109375f,-1.2114000297397371497254425776191055774688720703125f,1.9591000446319579708642777404747903347015380859375f,0.445300036680699395930105310981161892414093017578125f,0.384700049299002255764889923739247024059295654296875f,1.1337000133387744593704837825498543679714202880859375f,-1.70489998439550305420198128558695316314697265625f,0.52219998822286706197104422244592569768428802490234375f,-0.882100033843517650211651925928890705108642578125f,1.134000038508326202446596653317101299762725830078125f,-1.81590002212524392888326474349014461040496826171875f,0.6000000014156103844698009197600185871124267578125f,-0.663300063443184484412995516322553157806396484375f,1.23590008815601581915188944549299776554107666015625f,-0.7237000340700152634099140414036810398101806640625f,1.77450005213916295332410300034098327159881591796875f,-0.937199984888732462451343963039107620716094970703125f,-0.424699948753416212099409676739014685153961181640625f,1.021199981245398458185036361101083457469940185546875f,1.1156000359967348511958107337704859673976898193359375f,0.44550001193583010916654529864899814128875732421875f,-1.243899983434378686553145598736591637134552001953125f,1.05450001600384712929781017010100185871124267578125f,-0.15579995734691565179019789866288192570209503173828125f,1.1961999932408335922673359164036810398101806640625f,-1.72930004480928101173731192830018699169158935546875f,-0.2935999182552091468778598937205970287322998046875f,-0.9496999426066865535034366985200904309749603271484375f,-1.660700034688414206129891681484878063201904296875f,-0.03680000815242501222002147187595255672931671142578125f,-1.014699984385817987941891260561533272266387939453125f,0.13750001268461342629478849630686454474925994873046875f,1.01550000986829491722573948209173977375030517578125f,-0.1218999798357487396316400918294675648212432861328125f,2.360100026035308218297359417192637920379638671875f,-1.3568999977543947377256472464068792760372161865234375f,-1.241600006484984586307973586372099816799163818359375f,1.473300017920136983917700490565039217472076416015625f,-0.8143999609485270685382829469745047390460968017578125f,-0.320699986371398182427583378739655017852783203125f,0.581099933379888522466671929578296840190887451171875f,-1.1255000080056494216051987677928991615772247314453125f,1.755500051535666017343828571029007434844970703125f,1.1735000334084035866766271283267997205257415771484375f,-1.0077999683588745316598078716197051107883453369140625f,0.61509998330622916995480409241281449794769287109375f,-0.0395000274777412840876422706060111522674560546875f,-0.5211999961093065625306053334497846662998199462890625f,-0.3232000107347958728354342383681796491146087646484375f,-0.45269999540895222711611722843372263014316558837890625f,0.3660999753639100351421120649320073425769805908203125f,0.7812999993272118359755040728487074375152587890625f,0.604300001521408614024721828172914683818817138671875f,-0.18579999141469560886008594025042839348316192626953125f,-1.558500028397888481634936397313140332698822021484375f,-1.7974999839067458129449050829862244427204132080078125f,0.2364000059537587628000210315803997218608856201171875f,1.2162999898463482395527535118162631988525390625f,-0.056699970456957515807516756467521190643310546875f,-0.457500043697654401597674223012290894985198974609375f,0.83319998856931898334465813604765571653842926025390625f,-0.1626000193297862939090236977790482342243194580078125f,0.03529998940676459806553566522779874503612518310546875f,-2.25489996662587000031408024369738996028900146484375f,1.4601000540494926127621511113829910755157470703125f,-0.42350000509619700750363335828296840190887451171875f,0.60920000331401791271446199971251189708709716796875f,-0.1375999819278719282777956323116086423397064208984375f,0.5845999807953834448426277958787977695465087890625f,-0.3244000264391309062972368337796069681644439697265625f,-1.039500027421861982901418741676025092601776123046875f,0.319700014821440114953787769991322420537471771240234375f,-0.15570000779181702998954506256268359720706939697265625f,-0.48280002200305460746676544658839702606201171875f,-0.1854999859333030798325125942938029766082763671875f,0.5051999563306566187748103402554988861083984375f,-0.1046000064253806982605965458787977695465087890625f,1.0428000291183590064747477299533784389495849609375f,-1.48540002188682596084845499717630445957183837890625f,-0.253100051084161392633831155762891285121440887451171875f,0.5942000054001805864345442387275397777557373046875f,-0.46870000844001757922541173684294335544109344482421875f,0.035200019567460272273962118561030365526676177978515625f,0.2061000063955784611380295245908200740814208984375f,-0.2275000002048910285878946524462662637233734130859375f,-0.816100025301426779122948573785834014415740966796875f,1.1686000199556350054308495600707828998565673828125f,-0.6743999980337915811645643771043978631496429443359375f,-0.500700006487965421086983042187057435512542724609375f,-1.726099998721480677232875677873380482196807861328125f,1.316200035187602424002761836163699626922607421875f,1.273800019425153084995372410048730671405792236328125f,-0.1611000087052587570468631383846513926982879638671875f,-0.41750002287328291661339108031825162470340728759765625f,-0.0595000205859542996567057571155601181089878082275390625f,-0.46680000343993288680621844832785427570343017578125f,0.33080000223666417014101170934736728668212890625f,1.63499998856335881924906061613000929355621337890625f,-0.947600016126036504005014649010263383388519287109375f,-0.017700020666421067971896263770759105682373046875f,0.601899973800778642640807447605766355991363525390625f,0.515100003609060497211658002925105392932891845703125f,0.53479994934499242020820020115934312343597412109375f,0.1909999887794249229244769594515673816204071044921875f,-0.43840004149675326772239714046008884906768798828125f,-0.0070000285208224877209204350947402417659759521484375f,-0.49050001633167283898728783242404460906982421875f,1.550200015562772204447128387982957065105438232421875f,0.3667999953746789998376698349602520465850830078125f,-0.509299989190698187968564525363035500049591064453125f,0.7456999556005001750236260704696178436279296875f,0.7984000156432389250227288357564248144626617431640625f,-0.61029996783733420073758679791353642940521240234375f,0.46689995137452999784954954520799219608306884765625f,0.0186000786662106065705302171409130096435546875f,-0.484700023687630487945199320165556855499744415283203125f,0.57590003046989490798068800359033048152923583984375f,0.1590000168457625928652987568057142198085784912109375f,-0.757799995851517138589770183898508548736572265625f,0.60509997160881690003719768355949781835079193115234375f,-0.7518000034213070392752342741005122661590576171875f,0.23669998310431850097046435621450655162334442138671875f,0.52100001627206804499792269780300557613372802734375f,0.812900008382647865090575578506104648113250732421875f,0.351400028920173357338541109129437245428562164306640625f,0.53870004375576829858118799165822565555572509765625f,0.07029996277093886192233185283839702606201171875f,1.52579999217838047087525410461239516735076904296875f,-0.888600029566884330733955721370875835418701171875f,0.34450000847876049991924674031906761229038238525390625f,0.1050000114738944834158473895513452589511871337890625f,-0.2321999155610809939531691270531155169010162353515625f,0.722299972963333036801714115426875650882720947265625f,-0.7087000119045379076254675965174101293087005615234375f,-0.23299998667836241406092767647351138293743133544921875f,-1.10269996851235685397796260076574981212615966796875f,0.534300031528621932608302813605405390262603759765625f,-0.036099993785470463336650936980731785297393798828125f,-0.14019997935295036572966864696354605257511138916015625f,0.1822999828726057369721047507482580840587615966796875f,0.6404000425621869663217466950300149619579315185546875f,-0.05009999809563137063150861649774014949798583984375f,-0.3927000112414358312662443495355546474456787109375f,1.228700023713707789596583097591064870357513427734375f,0.7955999738216394945311549236066639423370361328125f,0.86640002601444621177506633102893829345703125f,1.5858999908328055283845969825051724910736083984375f,0.752500022873281881885532129672355949878692626953125f,-0.274500005330890528565390695803216658532619476318359375f,0.09340000521242652897768721231841482222080230712890625f,-0.047300028451532138973334440379403531551361083984375f,0.03510002909004705795581458005472086369991302490234375f,0.4611999942466609159197332701296545565128326416015625f,0.4038000132225454397172370590851642191410064697265625f,-0.762400021350384093921093153767287731170654296875f,0.78569999354258113211102454442880116403102874755859375f,-0.2294999538660054128058618516661226749420166015625f,1.207299974025040523173402107204310595989227294921875f,-0.4620999420523634881874386337585747241973876953125f,-0.07490005139485045315694833334418945014476776123046875f,-0.008199988830089710578619133229949511587619781494140625f,-0.4185000105909997270003941594040952622890472412109375f,0.74690000006407519794748850472387857735157012939453125f,0.2314999933391812536598308724933303892612457275390625f,-1.02569997225254727624133010976947844028472900390625f,-0.943100011111795755169850963284261524677276611328125f,1.144100017437339023018694206257350742816925048828125f,-0.6268999528273948573797724748146720230579376220703125f,-0.22619999912679222831002334714867174625396728515625f,-1.1784999228045354158211921458132565021514892578125f,0.39439995286464668566850377828814089298248291015625f,-0.0678999733462937182792984458501450717449188232421875f,0.2670000009536739327131726895458996295928955078125f,-0.392099985712766940793017056421376764774322509765625f,0.577600004652142562378003276535309851169586181640625f,-0.3926999944776294881876310682855546474456787109375f,-0.4682000065624709606026954133994877338409423828125f,-0.27950001484900777182218689631554298102855682373046875f,-0.69570003131702506937728003322263248264789581298828125f,1.166300034158676890427841499331407248973846435546875f,-0.163400001431256214345921762287616729736328125f,-1.1865000375583771852205927643808536231517791748046875f,-0.117900028592348604661310673691332340240478515625f,0.4356000297755004968536240994581021368503570556640625f,0.76329998804330756456693052314221858978271484375f,-1.240699990618228554239976801909506320953369140625f,-0.6456000015377991729792483965866267681121826171875f,1.597799982291459475192141326260752975940704345703125f,0.8107999928116793597610012511722743511199951171875f,0.30370001559257531198454671539366245269775390625f,0.059699969875811831343526137061417102813720703125f,-1.4103999762892731606456209192401729524135589599609375f,-0.416599972063302903535486620967276394367218017578125f,-0.72219997553825354685841375612653791904449462890625f,-0.24939997521638890276562960934825241565704345703125f,0.3733999926209445874292214284650981426239013671875f,-0.934900011217594251178297781734727323055267333984375f,0.60479999907761772259817689700867049396038055419921875f,-0.26209999539852102401482625282369554042816162109375f,0.286000014614312902949677663855254650115966796875f,0.147299997401237892091785397497005760669708251953125f,-0.248899962311983025120554202658240683376789093017578125f,0.61029998944401686600258472026325762271881103515625f,-0.22530004085302390848255527089349925518035888671875f,0.6080000176727775595963976229541003704071044921875f,1.4747000751711436539181931948405690491199493408203125f,-0.7400000435113913699325394190964289009571075439453125f,-0.9921000248283144973271419075899757444858551025390625f,1.86800000530481380422997972345910966396331787109375f,-0.72949998836219354103604928241111338138580322265625f,0.19200001347064965528943503159098327159881591796875f,0.9723000253409141979688001811155118048191070556640625f,0.3427000404477116290991034475155174732208251953125f,0.928099962273240208787683513946831226348876953125f,-0.8228999694347383098147474811412394046783447265625f,-0.056399988463521122383781403186731040477752685546875f,0.230199993062019103717830148525536060333251953125f,-0.13500000018626412412459103506989777088165283203125f,0.519600000166893227770970042911358177661895751953125f,-0.4701000165358184545993935898877680301666259765625f,-0.953800033879280473314565824694000184535980224609375f,-0.5984000124394892594636985450051724910736083984375f,-1.5090000086128705358845536466105841100215911865234375f,-0.1371000353336331301079553668387234210968017578125f,-1.1790000202357762137950203396030701696872711181640625f,1.25529998407959997308580568642355501651763916015625f,1.3406999900221823196488912799395620822906494140625f,-0.7078000195443632236447228933684527873992919921875f,0.661499993413686926402306198724545538425445556640625f,0.3993000247299673954870513625792227685451507568359375f,-1.106900053335726941128314138040877878665924072265625f,-0.56310001673698462099082462373189628124237060546875f,-0.1407000090956689319909855839796364307403564453125f,-0.2665000047571959296277555040433071553707122802734375f,0.9043999963760374072307968162931501865386962890625f,0.12979996240213431679677569263731129467487335205078125f,-0.86649998521804771911547504714690148830413818359375f,-0.6795999950818714108180529365199618041515350341796875f,-0.097100007319450565290708254906348884105682373046875f,0.32800000940263307480648791170096956193447113037109375f,-0.267100008958578161166741438137250952422618865966796875f,0.486399998167901748136188189164386130869388580322265625f,0.21499999035149841564162898066570051014423370361328125f,-0.74020002394467676598566185930394567549228668212890625f,1.1566999848365782099790521897375583648681640625f,0.285899959018826255174872130737639963626861572265625f,1.978899986004829969488127971999347209930419921875f,0.431300005865097091373172588646411895751953125f,0.236600039267539585807753610424697399139404296875f,0.15090000588297858286068731104023754596710205078125f,-1.38119999294281026180897242738865315914154052734375f,-0.5340999943032860297620345590985380113124847412109375f,-1.048700006353854963236926778336055576801300048828125f,-0.4414999874904756982374465223983861505985260009765625f,0.1961999738320712172168214237899519503116607666015625f,0.51069993153512438510688298265449702739715576171875f,-0.50660000633597324082302293390966951847076416015625f,-1.184299992096423892462553340010344982147216796875f,-0.60500000357627836677920640795491635799407958984375f,1.281499974191188329086799058131873607635498046875f,0.5142000106155872884983182302676141262054443359375f,0.47130000199079535860846590367145836353302001953125f,1.078600006842612923918522938038222491741180419921875f,0.3883000066697597230103156107361428439617156982421875f,0.2319999904856084871340726749622263014316558837890625f,-0.9784999925792217112530124722979962825775146484375f,-1.3019000027462832580482654520892538130283355712890625f,-1.70660000588893812079049894236959517002105712890625f,0.328300006967783275513284024782478809356689453125f,-0.53060001453906313617636669732746668159961700439453125f,0.746300005175918546029834033106453716754913330078125f,-0.26290000037327398985098625416867434978485107421875f,-0.950199982780218110889336458058096468448638916015625f,-1.203399993105231491341555738472379744052886962890625f,0.6366000023126598961198396864347159862518310546875f,0.57809998672977147560203547982382588088512420654296875f,-0.0016999866992233325646566299838013947010040283203125f,-1.053200051992387198396272651734761893749237060546875f,-0.007499950900673457709899594192393124103546142578125f,0.24419999090880117886825928508187644183635711669921875f,-1.872099996888637374325980999856255948543548583984375f,-0.7522999805070458290146007129806093871593475341796875f,-0.7659000147864223340121725414064712822437286376953125f,0.4494000069558621301979428608319722115993499755859375f,-0.66459997371733248172631647321395576000213623046875f,1.1368000196635730247862738906405866146087646484375f,-0.947700017127394378491089810268022119998931884765625f,-0.876599987411498915435004164464771747589111328125f,-0.087100063310562969576267278171144425868988037109375f,1.4698999949380751051108973115333355963230133056640625f,0.6029999991580841101068699572351761162281036376953125f,0.339699995133280641113060482894070446491241455078125f,0.3824999972991645069697597136837430298328399658203125f,-0.32619996625110558863269716312061063945293426513671875f,0.7920000516735015150260323935071937739849090576171875f,1.0388999810874464113652493324480019509792327880859375f,0.245300016191601688575474327080883085727691650390625f,0.3895000083297486259681363662821240723133087158203125f,1.75699999357759839568871029769070446491241455078125f,0.08020005654096540315123320397105999290943145751953125f,0.7920000413730738930695451927022077143192291259765625f,0.29379997013956316020966141877579502761363983154296875f,0.634400048492849588654962644795887172222137451171875f,-0.39749997884035082051923382095992565155029296875f,0.044900021350384122786891794021357782185077667236328125f,0.037199994928389601422935584196238778531551361083984375f,0.042200020260364012880671680250088684260845184326171875f,0.08210002317056097620451282637077383697032928466796875f,0.6170000170469283062857357435859739780426025390625f,0.68530000004246804934382453211583197116851806640625f,-0.6236000230848788294935047815670259296894073486328125f,-1.1899000307753684335665411708760075271129608154296875f,0.543999997586011918571102796704508364200592041015625f,1.430000024735927244279309888952411711215972900390625f,-0.15810000824332259838911340921185910701751708984375f,0.72330005506128125514209159518941305577754974365234375f,-0.84060001643896076206630141314235515892505645751953125f,1.412200035012513676946355190011672675609588623046875f,0.2877000013694159807897676728316582739353179931640625f,0.9986999766260382660476579985697753727436065673828125f,-0.0868999898195266240463752183131873607635498046875f,-0.7819999989420176600418699308647774159908294677734375f,-1.463499982893466722089215181767940521240234375f,0.87800000753998741487293955287896096706390380859375f,-0.43290004706978724158261684351600706577301025390625f,0.4458000248491769124115080558112822473049163818359375f,0.2000999607682221181903514661826193332672119140625f,-0.945200021001696644162848315318115055561065673828125f,1.0834000380136075136761064641177654266357421875f,0.45159997495040282711187273889663629233837127685546875f,0.67899999611452199754779712748131714761257171630859375f,0.94900000235438408680010979878716170787811279296875f,1.6246000131309035197091361624188721179962158203125f,1.2129000075817106818476531771011650562286376953125f,0.6720999940574170494755890103988349437713623046875f,-0.4278999913394454512882703056675381958484649658203125f,0.3647000226259233812697857501916587352752685546875f,0.42009998699426631674214149825274944305419921875f,-0.31510003154873889030795908183790743350982666015625f,-0.42069998391270591042712112539447844028472900390625f,0.3099999850988393035322587820701301097869873046875f,0.09790001704394779569184947831672616302967071533203125f,0.3700999799162147940734257645090110599994659423828125f,0.0383999929189684230390611219263519160449504852294921875f,0.22789999424517137782686404534615576267242431640625f,0.2052999814160167968424275386496447026729583740234375f,0.071500019043683860076043856679461896419525146484375f,0.72229999963641133842173758239368908107280731201171875f,0.056499998927116212854571131174452602863311767578125f,0.10139999403283017631594020713237114250659942626953125f,0.200199979203939282257351806038059294223785400390625f,1.3791000073790546931462586144334636628627777099609375f,-0.13779999525547026451022247783839702606201171875f,0.021099993357062363230625123833306133747100830078125f,1.271699998359382188795052570640109479427337646484375f,-0.30270000077336989274812140138237737119197845458984375f,1.5474999428167943715806131876888684928417205810546875f,0.3510999783813952834776728195720352232456207275390625f,2.290699931162595959222016972489655017852783203125f,-0.36840003084242345376964067327207885682582855224609375f,1.9310999858319757294111695955507457256317138671875f,-0.1404000107109546480188555506174452602863311767578125f,2.259899948483706566548789851367473602294921875f,-0.45779999264776682910138561055646277964115142822265625f,-0.240199998575448159243705958942882716655731201171875f,-0.73239997828602787510732241571531631052494049072265625f,-0.9236999919369817124703558874898590147495269775390625f,-0.99419995313435849393357557346462272107601165771484375f,-0.612399975715578026580487858154810965061187744140625f,-1.107299982947111072206780590931884944438934326171875f,-0.9346999780714508876400259396177716553211212158203125f,-0.741199995177983961269774226821027696132659912109375f,-0.804000005036592302332110193674452602863311767578125f,-1.389799948674440788209949459997005760669708251953125f,-1.1459999945163727996799707398167811334133148193359375f,0.301399999266862561597690728376619517803192138671875f,0.30429998643398281643612790503539144992828369140625f,-0.3312999811105419301071606241748668253421783447265625f,0.452700024056434002783788628221373073756694793701171875f,1.2978000093370678502679993471247144043445587158203125f,-0.67389998917132587497036411150475032627582550048828125f,-0.2653999924384058406445774380699731409549713134765625f,0.5663999970316886400922840039129368960857391357421875f,-0.78030002222657213195589065435342490673065185546875f,-0.047299999785423540998863245476968586444854736328125f,-0.31590002929642813089827768635586835443973541259765625f,0.51370002542734172212846033289679326117038726806640625f,0.65710000722631800584849770530126988887786865234375f,1.7573999870777132770172102027572691440582275390625f,1.0369999732002621595228220030548982322216033935546875f,1.992999949276447413382129525416530668735504150390625f,0.7334000322952867190196002411539666354656219482421875f,0.1593999898567799444748516179970465600490570068359375f,-0.6415000249296429757350779254920780658721923828125f,-1.4384000051379199902612526784650981426239013671875f,1.1513000059396032437319945529452525079250335693359375f,-0.12710002296566980106717892340384423732757568359375f,0.12549998213350743725413849460892379283905029296875f,-0.21330000443458541070640421821735799312591552734375f,0.361000010982155838945573123055510222911834716796875f,-1.163000000387430787895937100984156131744384765625f,-0.6885999922394752115195615260745398700237274169921875f,-0.4052000012204057810549784335307776927947998046875f,1.1747999921858305327759808278642594814300537109375f,-0.0841000288739801593607126051210798323154449462890625f,0.49630002285242102999518465367145836353302001953125f,0.838100039833784560272533781244419515132904052734375f,0.0249999687075612708753169499686919152736663818359375f,0.29249997943639716169172970694489777088165283203125f,0.573200009840726831811252850457094609737396240234375f,1.73150004236400167911824610200710594654083251953125f,0.14579999310970326309444544676807709038257598876953125f,-0.96880000124573706354880187063827179372310638427734375f,-0.26069999590814119105885993121773935854434967041015625f,0.97560000366121524262297270979615859687328338623046875f,0.2568000029370189896127385509316809475421905517578125f,-0.1121000032961365633354944293387234210968017578125f,0.36300001499056833154099876992404460906982421875f,-0.105599984550476033717814061674289405345916748046875f,-0.280100000796466852559518656562431715428829193115234375f,-0.167200002211332243717833989649079740047454833984375f,0.4903000172905624953756387185421772301197052001953125f,-0.61200000866502535235014192949165590107440948486328125f,-1.426800016366689849434123971150256693363189697265625f,0.6067999736189844117717484550666995346546173095703125f,0.30000002376735313003308647239464335143566131591796875f,-1.399899990318715925496917407144792377948760986328125f,0.3513999940514569431115887709893286228179931640625f,-1.2599999758601203581065419712103903293609619140625f,0.037699956032632986335784153197892010211944580078125f,-0.421399988389014712453217725851573050022125244140625f,-1.6872000297784810474155392512329854071140289306640625f,0.61299999450147169000047142617404460906982421875f,-1.22329996449947220327203467604704201221466064453125f,0.45980003554821013977971233543939888477325439453125f,1.2183999428510663420865967054851353168487548828125f,0.2670999967768785676724974109674803912639617919921875f,-0.17739999456554667034424710436724126338958740234375f,-0.0301000029385090073219544137828052043914794921875f,-0.34529997729957095931041521907900460064411163330078125f,0.02730002630949091013690122053958475589752197265625f,-1.9445000334009521214539972788770683109760284423828125f,-0.2886000834345825438731480971910059452056884765625f,1.2178999537885186743579879475873894989490509033203125f,1.00310003372430767143441698863171041011810302734375f,0.96490000913143114669168198815896175801753997802734375f,-0.303899989432096173658237603376619517803192138671875f,1.2525000227615243897361096969689242541790008544921875f,-0.273899997106194614815422028186731040477752685546875f,-0.6043000254750250377355769160203635692596435546875f,0.27959999666511958782422198055428452789783477783203125f,1.2984000444769865367788952426053583621978759765625f,0.539900054915249949516464766929857432842254638671875f,0.984699972558022551538670086301863193511962890625f,-1.240699988811462706195243299589492380619049072265625f,-0.47429997957944891595616354607045650482177734375f,0.050299987879395491319201028090901672840118408203125f,-0.3038999974787230939909932203590869903564453125f,-0.275100001727789589267558767460286617279052734375f,-0.31339999843239763066549130599014461040496826171875f,0.878500013440846938550521372235380113124847412109375f,-0.00250001922249776242779262247495353221893310546875f,-1.430200005429983267646321110078133642673492431640625f,0.64250001281499979910449837916530668735504150390625f,-1.66389997259378485949810055899433791637420654296875f,-0.431500060915947614148535649292171001434326171875f,0.3347000333547587302973624900914728641510009765625f,0.039800032866001089359997422434389591217041015625f,-0.5657999913215636667018770822323858737945556640625f,-1.15200002047419491901791843702085316181182861328125f,1.25380001033544541400033267564140260219573974609375f,0.322499999105929635589973258902318775653839111328125f,-1.383000011786818195247406038106419146060943603515625f,-0.5462000145681200624636630891473032534122467041015625f,0.498400000537186915749998661340214312076568603515625f,-0.26080002304241067800916198393679223954677581787109375f,0.822900004750489966198756519588641822338104248046875f,-0.36179999246895311326710498178726993501186370849609375f,-0.8597000013545150753913048902177251875400543212890625f,1.025000031888484652853321676957421004772186279296875f,0.431400064384937831363231452996842563152313232421875f,-2.066100027555226148479050607420504093170166015625f,-0.5018000996828089199652822571806609630584716796875f,-0.7986000375017530128474163575447164475917816162109375f,0.3428000067852436938409255162696354091167449951171875f,1.1974999954365193044480975004262290894985198974609375f,-0.60779999037533982697567580544273369014263153076171875f,1.384999983664601419519613045849837362766265869140625f,0.05099999582022374244871798509848304092884063720703125f,-0.64240002177879274025684708249173127114772796630859375f,-0.7768000303365292236179584506317041814327239990234375f,0.17110004424452807114676033961586654186248779296875f,1.057400045546144529140519807697273790836334228515625f,1.02089995656013599045763839967548847198486328125f,-0.856299973473697573211893541156314313411712646484375f,1.317700021076202432368518202565610408782958984375f,-1.1291999980241056444896230459562502801418304443359375f,0.4353999773137269269085436462773941457271575927734375f,-1.129499954760074587056806194595992565155029296875f,0.839799994607269173485519786481745541095733642578125f,-0.400699984750896354146476596724824048578739166259765625f,-1.5320000201761718727766492520458996295928955078125f,0.6121000144533812470371003655600361526012420654296875f,1.449800052237512293373811189667321741580963134765625f,-1.282899990743399332160379344713874161243438720703125f,-0.108500070810318760550217120908200740814208984375f,-0.5350999786496171140015576384030282497406005859375f,1.92210007511973390847970222239382565021514892578125f,0.10269995908737161727231068653054535388946533203125f,0.198800010447204300589874037541449069976806640625f,-0.1789999884963027199091811780817806720733642578125f,-0.20119999376237363009067848906852304935455322265625f,-0.863600051695109005578387950663454830646514892578125f,0.6272000092148781735801321701728738844394683837890625f,-0.772799994808435197768403668305836617946624755859375f,-0.06400000946968809056869531559641472995281219482421875f,0.949999986439942656346602234407328069210052490234375f,-0.51710000955462465110912262389319948852062225341796875f,-0.5431000067532061326147641011630184948444366455078125f,0.698999997921287974378401486319489777088165283203125f,0.25660002915337642992454902923782356083393096923828125f,0.738800043416024010412002098746597766876220703125f,0.3456000365741556112908483555656857788562774658203125f,0.6507000178873543205781970755197107791900634765625f,0.1060000120438637782882551618968136608600616455078125f,-1.4197999708771700255738323903642594814300537109375f,-0.0671999940715723997897157460101880133152008056640625f,0.85339999023675972011915291659533977508544921875f,0.26520001520887015455940627361997030675411224365234375f,-0.3418999774515629752613676828332245349884033203125f,-0.149900010491162360981576284757466055452823638916015625f,-0.0427000024497505847875800100155174732208251953125f,-0.727499965131283232722125831060111522674560546875f,-1.15439999786615299370851062121801078319549560546875f,-0.7423999790310862323394758277572691440582275390625f,-0.4815000354349621147775906138122081756591796875f,1.777100007617473664822682621888816356658935546875f,-1.036299992156028082490593078546226024627685546875f,-0.079800005894899772584949459997005760669708251953125f,1.648499997645615611219227503170259296894073486328125f,0.05379997531771696372970836819149553775787353515625f,-0.80100000531971460304703214205801486968994140625f,-0.414599996441602502983414524351246654987335205078125f,-0.3574000237345702490898702308186329901218414306640625f,-1.20820000522136705711773174698464572429656982421875f,0.1567000384435051074660805170424282550811767578125f,-0.2508999828606841564493379337363876402378082275390625f,-0.11010000953227183639882014176691882312297821044921875f,0.976399996864051100686765494174323976039886474609375f,0.6651999862819908404532043277868069708347320556640625f,0.756000017650424904758210686850361526012420654296875f,-0.682199998672306406888310448266565799713134765625f,-0.7178000467389826777520056566572748124599456787109375f,-0.32940001489073000850993366839247755706310272216796875f,-0.3885000080764295926627482913318090140819549560546875f,-0.85630004948824700594656178509467281401157379150390625f,0.10719997319132090751736541278660297393798828125f,-0.24680003165900732398085892782546579837799072265625f,0.030699990171193458632359352122875861823558807373046875f,-0.2335000372640780863520149068790487945079803466796875f,-1.40629999101981439935116213746368885040283203125f,-0.06519997408166500196813331058365292847156524658203125f,0.6568000157147648732092193313292227685451507568359375f,-0.64200001324713174000180515577085316181182861328125f,0.2386000005841253379657018740545026957988739013671875f,-0.49559999088346984308373066596686840057373046875f,0.05359999905675665032855903291419963352382183074951171875f,0.31399997568130455505297504714690148830413818359375f,0.27910000946521762443808256648480892181396484375f,0.50109996415078672526988157187588512897491455078125f,-0.26809999423623054326526471413671970367431640625f,-0.517299982369690436456721727154217660427093505859375f,0.09870003917366165779867515084333717823028564453125f,1.5202000592231754705352386736194603145122528076171875f,0.95639997626319495083180299843661487102508544921875f,-0.37370002237260369071236709714867174625396728515625f,0.16879997338056540456818765960633754730224609375f,-0.004799992036819578089534843456931412220001220703125f,-1.0953000087410214025140930971247144043445587158203125f,0.1532000068232421963188016889034770429134368896484375f,-0.474599991319328562422441564194741658866405487060546875f,-1.881900000529735716980894721928052604198455810546875f,0.6505999820359047447482225834392011165618896484375f,-1.9544000062853097432480353745631873607635498046875f,-0.24730002779960624792465750942938029766082763671875f,-1.4696000186070801163396026822738349437713623046875f,0.18649999594688448922852330724708735942840576171875f,0.6730000003501774852310290953028015792369842529296875f,-1.4699000384867197954719131303136236965656280517578125f,-0.2390000096186997158298481735982932150363922119140625f,0.346400004407763528746500014676712453365325927734375f,1.251200020733475692935598999611102044582366943359375f,0.3849000058718023975501409950084052979946136474609375f,1.3799999801069497973315947092487476766109466552734375f,0.4803000298634170217582095574471168220043182373046875f,0.922900002515316231210817932151257991790771484375f,0.5346000173032281477247806833474896848201751708984375f,1.1476000358700755299423690303228795528411865234375f,0.07860001753419687542390192902530543506145477294921875f,-1.057799956512450734180674771778285503387451171875f,1.051200010488927194529651387711055576801300048828125f,1.10369999353885628323723722132854163646697998046875f,-0.17579999988973160629512904051807709038257598876953125f,-0.6708999914288522603555975365452468395233154296875f,0.253699997860193260379446655861102044582366943359375f,2.0839000379912562266326858662068843841552734375f,-0.7608000003553936352318487479351460933685302734375f,0.3663000025868416997809617896564304828643798828125f,-1.222200022625923310215512174181640148162841796875f,0.08950005072355349255985856871120631694793701171875f,0.81629999394416774549654292059130966663360595703125f,-1.343100033202767829010326749994419515132904052734375f,-0.452700004386902055131258748588152229785919189453125f,0.1130000061988833959247813254478387534618377685546875f,1.222500018402933985584013498737476766109466552734375f,-0.90830000473260863458335734321735799312591552734375f,0.056299983029067746098661473297397606074810028076171875f,0.54929999489039194049411207743105478584766387939453125f,0.37290003043636710255981370210065506398677825927734375f,-0.14550002232939018842472478354466147720813751220703125f,-1.06020004581213100891545764170587062835693359375f,-0.595300016806274090441775115323252975940704345703125f,-0.22000001171603766891848863451741635799407958984375f,0.00650001604482441930343838976114057004451751708984375f,-0.77289997962340695192295925153302960097789764404296875f,-1.486500024892389859587638056837022304534912109375f,0.7017999886691566047147716744802892208099365234375f,0.132599972353875994457439446705393493175506591796875f,-0.3337999902561306075909897117526270449161529541015625f,0.9381000620365147479873257907456718385219573974609375f,0.7101000449225314259393826432642526924610137939453125f,1.0494000330328940862045783433131873607635498046875f,0.919900013620405854197770167957060039043426513671875f,1.0962999919511375868097502461750991642475128173828125f,-0.50269999237284046511575752447242848575115203857421875f,-0.0959999632984402939683832300943322479724884033203125f,0.62969999822527167321339902628096751868724822998046875f,1.508699981319903571375107276253402233123779296875f,-0.03630000772774277795207353847217746078968048095703125f,-0.0381999910652641716524158255197107791900634765625f,-0.55739997776448746424904356899787671864032745361328125f,0.5278000492721803826867699171998538076877593994140625f,-0.37760000094547907512065876289852894842624664306640625f,-0.3794000673055659689225649344734847545623779296875f,-0.69719998269081084885101518011651933193206787109375f,-0.7979000021241604567734384545474313199520111083984375f,0.4090999986618755190903584662009961903095245361328125f,0.4145999951563774654772487338050268590450286865234375f,-0.424900038654356382306076511667924933135509490966796875f,1.30820000000595992872831629938445985317230224609375f,-0.699299978238343999947801421512849628925323486328125f,-1.3651999750316132775651567499153316020965576171875f,1.173800000649690478127240567118860781192779541015625f,0.3849000054061415543316115872585214674472808837890625f,-0.30810001662522579213288054233998991549015045166015625f,-0.413799993832409285232643014751374721527099609375f,-0.68530001336038115500315370809403248131275177001953125f,-0.44179998963773259301746065830229781568050384521484375f,-0.935000019036233087632581373327411711215972900390625f,0.174000017441808818574600081774406135082244873046875f,0.0301000155672432878617428286815993487834930419921875f,-0.7267999721288680436970253140316344797611236572265625f,-1.073600022377073859303209246718324720859527587890625f,0.1952999800190335921712403433048166334629058837890625f,-1.113300019969046150691838192869909107685089111328125f,-0.4001000597864383667712218084488995373249053955078125f,0.27669996624737958068607213135692290961742401123046875f,0.55869999398589076378129902877844870090484619140625f,1.1661999634385100677036462002433836460113525390625f,0.10430003545880328630346411955542862415313720703125f,0.6412000179022552259766598581336438655853271484375f,0.18440004569888046592751607022364623844623565673828125f,0.223000048443675158438281869166530668735504150390625f,-0.99769999797940212005187277100048959255218505859375f,0.07610002770423829332457898999564349651336669921875f,0.448899955308437625944861792959272861480712890625f,-1.18660003930479351907933960319496691226959228515625f,-0.98490001879855970656763020087964832782745361328125f,0.941300031197071174204893395653925836086273193359375f,-0.418900016142428111010076463571749627590179443359375f,0.61190003020167349490066044381819665431976318359375f,1.26960003434643198261255747638642787933349609375f,-0.66300001722574197771109538734890520572662353515625f,-1.066300005101412207153543931781314313411712646484375f,0.8378000040471551557175189373083412647247314453125f,0.283699982418864904420985340038896538317203521728515625f,-0.3419999812841414410513607435859739780426025390625f,-0.076500000845640736546471316614770330488681793212890625f,-0.76330001346841436227208532727672718465328216552734375f,0.7338999892085789067408541086479090154170989990234375f,-0.8602000070691104838260798715054988861083984375f,1.5699999956786634403016478245262987911701202392578125f,0.7514000238537779097214297507889568805694580078125f,-0.464099972137808958194682418252341449260711669921875f,-0.94719994633197757138987071812152862548828125f,-0.314300006401538656319871734012849628925323486328125f,-0.002600031101703592639751150272786617279052734375f,-0.8800000210106373454976846915087662637233734130859375f,1.3258000120341772998244778136722743511199951171875f,0.05290000989139065357136360034928657114505767822265625f,-0.41130005343705489107009043436846695840358734130859375f,-0.020700036513805475379967901972122490406036376953125f,0.6034999943673613476136097233393229544162750244140625f,0.19259994363188770449824005481787025928497314453125f,1.2703999733090398649437702260911464691162109375f,1.62780002639889698201614010031335055828094482421875f,0.870299989742040480678042513318359851837158203125f,-1.279799982798099478031872422434389591217041015625f,0.0518000198125836419649203890003263950347900390625f,-0.633700001734494922089879764826036989688873291015625f,-0.5248999894618986417071937466971576213836669921875f,-0.743899981981515967532914146431721746921539306640625f,0.3055000364482405306176815429353155195713043212890625f,-0.1204000135421754436038099811412394046783447265625f,-0.043100005486607584348490718184621073305606842041015625f,0.24749999267980460349036775369313545525074005126953125f,0.229699989414960004108223756702500395476818084716796875f,0.335699993002414431852997722671716473996639251708984375f,-0.2894000062480570623080211589694954454898834228515625f,-1.093000010240822827967122066183947026729583740234375f,0.23410002585649447581772619741968810558319091796875f,0.84320001844614800301513923841412179172039031982421875f,-0.41770000542998264592142731999047100543975830078125f,0.51770001722052683135188999585807323455810546875f,0.6479000212162731031639850698411464691162109375f,0.5662999966263770890151363346376456320285797119140625f,-0.34679999969601638110106023304979316890239715576171875f,0.20939999144002785502749475199379958212375640869140625f,0.448899945231527652822478557936847209930419921875f,-0.62310002442970857661208583522238768637180328369140625f,-0.16420000817552204974703045081696473062038421630859375f,1.408699987280369381181799326441250741481781005859375f,0.36600001046061425569178027217276394367218017578125f,0.40020003851056085153459207504056394100189208984375f,-1.051699962782858932541785179637372493743896484375f,-0.307499959692358881824247873737476766109466552734375f,1.0933000239551071697263751048012636601924896240234375f,0.8053000046804548173184912229771725833415985107421875f,1.623600008761137747370639772270806133747100830078125f,-0.276299973361939177340929063575458712875843048095703125f,0.3692000065177680756534073225338943302631378173828125f,0.604600010243803165366216489928774535655975341796875f,0.22440001557990907343764774850569665431976318359375f,-0.38330006985068276748052085167728364467620849609375f,0.30899995452165551768075602012686431407928466796875f,1.8873000206470482709164571133442223072052001953125f,1.32649998596310592091640501166693866252899169921875f,-0.5468999980151654227711333078332245349884033203125f,0.351699992883205414528902110760100185871124267578125f,-0.295200031562895259806822423342964611947536468505859375f,1.41810003581047094911582462373189628124237060546875f,1.4370000331588090869416873829322867095470428466796875f,-0.97810002195239054145758927916176617145538330078125f,-0.69640000247061240035151286065229214727878570556640625f,0.0220000168979168631011589241097681224346160888671875f,0.2848000046096743620438473953981883823871612548828125f,-0.3763000016927715574155399735900573432445526123046875f,-0.1322999899692835212849928439027280546724796295166015625f,0.43800000336766242714503505339962430298328399658203125f,-0.5044000390306120351624485920183360576629638671875f,0.321900001740456076504415250383317470550537109375f,0.401599991509318243743109633214771747589111328125f,1.9569000139594070475368425832130014896392822265625f,-0.4307999931097032941806901362724602222442626953125f,1.63180003352165226004899523104541003704071044921875f,0.0308000045835974400887380397762171924114227294921875f,-0.2267999563336378088251876761205494403839111328125f,0.0909000317364927479246716757188551127910614013671875f,-0.30989999616742114341150227119214832782745361328125f,0.1551999965637922418437710803118534386157989501953125f,-0.248999985814094770830706693232059478759765625f,-0.06209999286532441598041032193577848374843597412109375f,0.58089995521903059483292963705025613307952880859375f,-1.194200022983550990574030947755090892314910888671875f,0.543500010460615357033020700328052043914794921875f,0.1659000352382662413219804875552654266357421875f,-0.947500007748603767510076067992486059665679931640625f,0.125299978864192862459958632825873792171478271484375f,0.0944000060617919967143052417668513953685760498046875f,-0.0511999938368796758680900893523357808589935302734375f,-0.33080000428557365665938050369732081890106201171875f,0.2660999992430210792093703275895677506923675537109375f,0.395100013890862467036413363530300557613372802734375f,-0.46129998962283147445617714765830896794795989990234375f,0.33549999338388403202770859934389591217041015625f,-0.253100019884854587903788569747121073305606842041015625f,-0.664599983030557783791891779401339590549468994140625f,-0.11020001055225747688126602952252142131328582763671875f,0.04029998240321897018390018274658359587192535400390625f,0.26800004477426508930903992222738452255725860595703125f,-0.5675000160560015327604332924238406121730804443359375f,0.2156000205926600354899846934131346642971038818359375f,0.1513000100374224388133370666764676570892333984375f,-0.65009999694079134702207056761835701763629913330078125f,0.239799999114870932004350834176875650882720947265625f,0.31700001112371722911120741628110408782958984375f,0.6185999967098236851370529620908200740814208984375f,0.548699970963597127848743184586055576801300048828125f,-0.50869999629557138831614793161861598491668701171875f,-1.3259999806582924986031457592616789042949676513671875f,-1.3964999607987704788314431425533257424831390380859375f,-0.3778999814860519723680454262648709118366241455078125f,-0.0794999943785370533078094013035297393798828125f,0.05220001465380175176278498838655650615692138671875f,-0.475099977159499697876299251220189034938812255859375f,0.79200001332163838529965005363919772207736968994140625f,-0.889800008130073205592225349391810595989227294921875f,-0.78450000456720558883461080768029205501079559326171875f,-0.23699998128414190290413898765109479427337646484375f,0.8150999595761303506691319853416644036769866943359375f,0.36259998811185401024204111308790743350982666015625f,0.086900000846385605512978145270608365535736083984375f,0.62089999118670780031692402189946733415126800537109375f,-0.6443000025458631707664380883215926587581634521484375f,-1.5048000427566474712648414424620568752288818359375f,0.280299992275237830341438893810845911502838134765625f,-0.5019000138849019077014190770569257438182830810546875f,0.6831000044435258100605778963654302060604095458984375f,0.76690000889301257469554684575996361672878265380859375f,0.7651000119537114674272970660240389406681060791015625f,-0.9117000020474199661890679635689593851566314697265625f,0.70140004717409676349149094676249660551548004150390625f,0.124399976650625132545968654085299931466579437255859375f,0.34590001661181446479531587101519107818603515625f,1.1392000179171564155922169447876513004302978515625f,1.39750005193054693819476597127504646778106689453125f,0.3979999856352804954440216533839702606201171875f,-0.7460999908611174458172854428994469344615936279296875f,0.88000002622604378554882487151189707219600677490234375f,1.147800016564131109220170401385985314846038818359375f,-0.3970000162273648047772667268873192369937896728515625f,-0.322900004638731308315158230470842681825160980224609375f,-0.02989999139904997438321743175038136541843414306640625f,0.772000020194798963757421006448566913604736328125f,-1.2749000081256027794296414867858402431011199951171875f,0.9670000148117541272085873060859739780426025390625f,-0.407700010049342864704158273525536060333251953125f,-0.6332999851375815669740632074535824358463287353515625f,0.3664000107407563433525865548290312290191650390625f,0.942299989503621926445475764921866357326507568359375f,1.48720002024173769683557111420668661594390869140625f,-0.77470001472830762878629684564657509326934814453125f,0.996299977031350092460115774883888661861419677734375f,-0.0628000329554081471172821693471632897853851318359375f,-0.2095000305138530194426493835635483264923095703125f,0.72049997548386446322155052257585339248180389404296875f,-0.0510000759325925923093336677993647754192352294921875f,0.2969999759197232691576573415659368038177490234375f,0.3486999783396722563111325143836438655853271484375f,-0.37510000100135787448607516125775873661041259765625f,-0.89220003633499178619103986420668661594390869140625f,-0.671300001990795092154939993633888661861419677734375f,-0.25139999397695067528957224567420780658721923828125f,-0.37089998099803900277748880398576147854328155517578125f,-0.654900018444657394667274274979718029499053955078125f,-1.39940000025033928210405065328814089298248291015625f,0.39020000230073959013310513910255394876003265380859375f,-0.1469000023923814068904647456292877905070781707763671875f,0.472700012489408238902655057245283387601375579833984375f,0.094000006452203355156171937778708525002002716064453125f,0.444900020456314049255297504714690148830413818359375f,0.8255999773606654823510098140104673802852630615234375f,-0.030400004005431913167711854839581064879894256591796875f,-0.1827000072926285023644510374651872552931308746337890625f,-0.377599976395815606355910176716861315071582794189453125f,0.5988000300608578019279093496152199804782867431640625f,0.161799972277879344773054981487803161144256591796875f,-0.40760001409575330466594778044964186847209930419921875f,-0.1086999837040900640516838393523357808589935302734375f,0.58100001334771544936330656128120608627796173095703125f,1.46169999945834216958928664098493754863739013671875f,1.4875999797672030400264020499889738857746124267578125f,2.1963999758720404997802688740193843841552734375f,-0.30339998755454999912473112999578006565570831298828125f,1.0501999572992328779719173326157033443450927734375f,1.11569999927952867579961093724705278873443603515625f,0.2266999949321146790026659800787456333637237548828125f,0.43429999470412727902157712378539144992828369140625f,-0.7450000351667398490462801419198513031005859375f,-0.201800022271275025786962942220270633697509765625f,0.716200002200156671250397266703657805919647216796875f,-0.897300010439753492619274766184389591217041015625f,-0.02680000947490379648030511816614307463169097900390625f,-0.063999975979327938802043718169443309307098388671875f,0.5371000128328802247779094614088535308837890625f,0.1229000421896568884250200426322408020496368408203125f,0.43680002825036634561683968058787286281585693359375f,0.65799998032674211767556471386342309415340423583984375f,0.16500001968815947162028123784693889319896697998046875f,0.56590001422762892246964838705025613307952880859375f,0.4826999771177769016361480680643580853939056396484375f,-0.8503999876141552505259824101813137531280517578125f,0.28129995475411373462293340708129107952117919921875f,-0.284099996892362594014258547758799977600574493408203125f,1.0166000163942576772768688897485844790935516357421875f,1.0049000220209360900724959719809703528881072998046875f,0.062599992135166981910288086510263383388519287109375f,0.63120000527352060348817985868663527071475982666015625f,0.09020000677108785946956004409003071486949920654296875f,0.32490001553893066432721070668776519596576690673828125f,0.07819999728649851267192616433021612465381622314453125f,0.266600031481683374945390596622019074857234954833984375f,-0.03240000227689765477379069125163368880748748779296875f,-0.6989000052459533041115946616628207266330718994140625f,2.073300009649991171301053327624686062335968017578125f,1.6006000656127934433925474877469241619110107421875f,0.3715998971104621517724808654747903347015380859375f,-2.3795000563859947106948311557061970233917236328125f,0.153800020468235487669517169706523418426513671875f,0.2891000123523175258100081919110380113124847412109375f,0.8280000256448987183688359436928294599056243896484375f,-0.2885000346563761919327362193143926560878753662109375f,1.6088000005193052732721525899250991642475128173828125f,0.3168000535450878896170934240217320621013641357421875f,-0.1082000245183707409779572117258794605731964111328125f,0.0902999709106973025285469702794216573238372802734375f,-0.4376999914973971339549052572692744433879852294921875f,0.56290002777278436152386120738810859620571136474609375f,-0.466400012156367171911597324651665985584259033203125f,1.304200029204785504788333128090016543865203857421875f,0.352100025200844157780011300928890705108642578125f,0.16830007197409901920082120341248810291290283203125f,-0.70280001254081714279209336382336914539337158203125f,-0.39510001608878286827319925578194670379161834716796875f,-0.34910004791021354009217247948981821537017822265625f,1.227400019599496960864826178294606506824493408203125f,0.5457000704139478575172006458160467445850372314453125f,-0.648299989105015850299196245032362639904022216796875f,0.38060000304654229008605170747614465653896331787109375f,-0.81450000870227778904109072755090892314910888671875f,0.00189999628290526967333562424755655229091644287109375f,-0.0403999932765959357539031771011650562286376953125f,0.181600002387165948558589434469467960298061370849609375f,0.83290001026391990990305203013122081756591796875f,1.236900036422908133459941382170654833316802978515625f,0.0271999502062794107359877671115100383758544921875f,-1.210300030142814176059573583188466727733612060546875f,0.02710001156628027985107109998352825641632080078125f,0.253199990841746380709764707717113196849822998046875f,-0.2065999991275371261423288160585798323154449462890625f,1.127999993830919311932348136906512081623077392578125f,-0.897200028958916817600766080431640148162841796875f,0.2355000102967024222522240961552597582340240478515625f,-0.26990001360177995337608081172220408916473388671875f,0.2295000322461131503359865746460855007171630859375f,0.8190999716162679078479413874447345733642578125f,-0.21309998395442963925461299368180334568023681640625f,-2.05450007069110984758708582376129925251007080078125f,-0.13049994927644714692860361537896096706390380859375f,0.75080006470978322141718308557756245136260986328125f,1.02319997661113770703877889900468289852142333984375f,0.5782000122062862512706260531558655202388763427734375f,0.79890002057552322867195471189916133880615234375f,-0.3928999889925126609568906133063137531280517578125f,-1.5471000278085471624223146136500872671604156494140625f,-1.033900003737211026333397967391647398471832275390625f,0.2008999778687952098010782719939015805721282958984375f,0.9314999526962639553318012985982932150363922119140625f,0.554500004231929466413930640555918216705322265625f,1.612899967534839884564235035213641822338104248046875f,0.167200037042796345598816287747467868030071258544921875f,1.5967999548807743082079468877054750919342041015625f,1.371899996059386950264524784870445728302001953125f,0.794200046899913747466825952869839966297149658203125f,0.65280003042221090847618825137033127248287200927734375f,-0.56759996650516963967447736649774014949798583984375f,0.37149999639391906303131918321014381945133209228515625f,-0.338500024393200593664232656010426580905914306640625f,-0.66670000519528949123326810877188108861446380615234375f,-1.8802000103473659464725642465054988861083984375f,-0.884500026211142209575655215303413569927215576171875f,-1.302899983143806394991770503111183643341064453125f,-0.5813000102050598361103084243950434029102325439453125f,0.0142999622195965070403644858743064105510711669921875f,-0.36539999472945938752133088200935162603855133056640625f,0.10000005245208765103370751603506505489349365234375f,0.99739996532201757872826419770717620849609375f,-0.99480002906620512703739223070442676544189453125f,0.6178999785244467801703649456612765789031982421875f,0.5042000165760518459734385032788850367069244384765625f,0.6183999929934735906300602437113411724567413330078125f,-1.10389998749494555596584177692420780658721923828125f,-0.40530000061988857851247303187847137451171875f,-0.44889998483136206441912463560584001243114471435546875f,0.19330002000629853409208180892164818942546844482421875f,0.3683999958991999168489428484463132917881011962890625f,0.45379999400004711329614792703068815171718597412109375f,0.533399988821148696160889812745153903961181640625f,0.1763999977767463089861621483578346669673919677734375f,-1.310399956135452015359987854026257991790771484375f,-0.564600011566280368668913069996051490306854248046875f,1.30590000491440338947768395883031189441680908203125f,-0.057699970710277437291324531543068587779998779296875f,0.397399991846084876812028596759773790836334228515625f,-0.61610003470778540446417537168599665164947509765625f,-2.01180002372413913036552912672050297260284423828125f,-0.0709999757409089937709723017178475856781005859375f,0.9234999744743108696098943255492486059665679931640625f,0.71469998924732180967112071812152862548828125f,1.1120000077709553121252383789396844804286956787109375f,1.36500000916421360130925677367486059665679931640625f,0.51269998143911355015234221355058252811431884765625f,-0.56880001555085180076076767363701947033405303955078125f,-1.082099940264225068631276371888816356658935546875f,0.64830001065582099073481003870256245136260986328125f,0.8161999983072281139584447373636066913604736328125f,0.6592000197239216152667040660162456333637237548828125f,1.45500000655651007264168583787977695465087890625f,-0.9090000056512650417062104679644107818603515625f,-0.702300027799605430800511385314166545867919921875f,-0.33450000387802736323550334418541751801967620849609375f,-0.114499969363212272810415015555918216705322265625f,-0.583100029551982590447778420639224350452423095703125f,-0.5658000211238862764417945072636939585208892822265625f,1.367299982723594542477485447307117283344268798828125f,-0.38689995271563493162148006376810371875762939453125f,-0.67819997959137001686258372501470148563385009765625f,0.002699936735628849504564641392789781093597412109375f,-1.1519000024855134967083358787931501865386962890625f,0.52870003079176086657753330655395984649658203125f,0.2940000197887417954234479111619293689727783203125f,-0.744100038373471051045271451584994792938232421875f,-1.247400006449222331639248295687139034271240234375f,-0.8635000123977665253960367408581078052520751953125f,1.7500999986171716926719454932026565074920654296875f,-0.266199962395428979533562596770934760570526123046875f,0.012300003510713697352230155956931412220001220703125f,-0.78680003750771343451475559049868024885654449462890625f,0.53670001435950387413953421855694614350795745849609375f,-0.36110000125467711651339186573750339448451995849609375f,-1.0344999934285876719286534353159368038177490234375f,-0.2775000436976562401270030022715218365192413330078125f,0.166699993777275068396193091757595539093017578125f,1.6422000304117805313808275968767702579498291015625f,0.484400006248056680391300687915645539760589599609375f,1.1336000356391071619555077631957828998565673828125f,-1.26790003079026991628097675857134163379669189453125f,-0.7147000427424909929641216876916587352752685546875f,0.79520001528337569407511864483240060508251190185546875f,0.5345999964229755097022689369623549282550811767578125f,0.18859997750595136611906355028622783720493316650390625f,0.55320004249289589370874864471261389553546905517578125f,0.71130003322735457782499679524335078895092010498046875f,0.381499960184097375304190791212022304534912109375f,-1.590300040480495002981342622661031782627105712890625f,-0.969900051040947897007526989909820258617401123046875f,-0.41020002191439297600794589016004465520381927490234375f,-1.6231000317499046925462380386306904256343841552734375f,0.4040000644922248085322280530817806720733642578125f,1.5218000548303134422667426406405866146087646484375f,-0.1154000603318223738824599422514438629150390625f,0.267400008740275918484030626132152974605560302734375f,-1.17690000691860863213378252112306654453277587890625f,-0.30009997611641792136794038015068508684635162353515625f,-1.228299993612618212779352688812650740146636962890625f,-1.6463000857353211614508836646564304828643798828125f,0.195000047981739754732188885100185871124267578125f,-0.079200004468858775652506665210239589214324951171875f,1.360700022581219581496725368197076022624969482421875f,-0.43289999953508340180263758156797848641872406005859375f,0.5870999902576199414028224055073224008083343505859375f,-2.193999987304210197436304952134378254413604736328125f,-0.1618000289022918369852277464815415441989898681640625f,-0.1994999726042152587979217059910297393798828125f,0.79650003029406146737301241955719888210296630859375f,-0.862599981555342676386999301030300557613372802734375f,-0.561100012803077419221153832040727138519287109375f,-0.51770000498294821422717859604745171964168548583984375f,0.52199999361485194260268372090649791061878204345703125f,0.131699954885244441271652249270118772983551025390625f,0.4869999942854044139295410786871798336505889892578125f,0.6100000346079468283022606556187383830547332763671875f,0.86879999595582402260873777777305804193019866943359375f,0.503899937054514079903810852556489408016204833984375f,-0.955600025565922717163402921869419515132904052734375f,-0.45880002516210094398729779641143977642059326171875f,0.0592999903082847001911659390316344797611236572265625f,-1.0831999935984608640637816279195249080657958984375f,-0.41080001212730954396334936973289586603641510009765625f,-0.571000007443129842243934035650454461574554443359375f,0.08689999889060862070522262001759372651576995849609375f,-0.673100034469366281797420015209354460239410400390625f,-1.0513000346988443656215395094477571547031402587890625f,-0.499600004339217917959103942848742008209228515625f,-1.09740000361800138506396251614205539226531982421875f,-0.1567000110626219910869849627488292753696441650390625f,0.2019000244647262132247078625368885695934295654296875f,0.774900049066543861187028596759773790836334228515625f,0.020099958682060492520804473315365612506866455078125f,-0.3426999865800139932758838767767883837223052978515625f,0.5316999834954738712866628702613525092601776123046875f,1.269200020487606384023138161865063011646270751953125f,-0.798600006991625033236914532608352601528167724609375f,-1.224000033386051899952917665359564125537872314453125f,-0.940299953867495208470472789485938847064971923828125f,0.26800001126527916994746192358434200286865234375f,-0.5960000407099723673809421597979962825775146484375f,-0.603800078471005807756455396884121000766754150390625f,-0.1998000044047838485994361690245568752288818359375f,0.666400016961991692454603253281675279140472412109375f,-0.352900009965896543917551753111183643341064453125f,0.3665000035092236263523091110982932150363922119140625f,0.434800034356117581779699321486987173557281494140625f,-0.85900002176314604884765913084265775978565216064453125f,-0.45700003932416477514522057390422560274600982666015625f,0.9958999929562211139710825591464526951313018798828125f,0.6462000155553220270832071037148125469684600830078125f,1.21880001305490726082325636525638401508331298828125f,0.9270000654011976148893836580100469291210174560546875f,0.1730000719130035946591306128539144992828369140625f,-0.327800029826164074364669431815855205059051513671875f,0.059799983692169433879826101474463939666748046875f,-0.1970000002458693888485186107573099434375762939453125f,1.091100039923190934132435359060764312744140625f,0.34910003028959091242455770043306984007358551025390625f,0.040499979674816444230600609444081783294677734375f,-0.84020001494809992692580635775811970233917236328125f,-0.784600021177530582150438931421376764774322509765625f,-0.239100022261589995054009705199860036373138427734375f,-1.05600000867247612035271231434307992458343505859375f,0.3875999375224115084392906283028423786163330078125f,-1.13490002045631488414301202283240854740142822265625f,-1.15179996706247234072861829190514981746673583984375f,-0.697100061410666160810478686471469700336456298828125f,-0.275299989891051755108719589770771563053131103515625f,0.88070003785491035586119323852472007274627685546875f,-0.36249998658895510317279331502504646778106689453125f,-0.8167000480547550456122962714289315044879913330078125f,0.5211000131011009894876906400895677506923675537109375f,-0.02979999501705210551705249599763192236423492431640625f,0.0524000035807492547945685146260075271129608154296875f,-1.05430002554953095028622556128539144992828369140625f,-0.76060000734925292675825403421185910701751708984375f,-1.3891000001147391085254412246285937726497650146484375f,-0.19679998881816818112611144897527992725372314453125f,-0.427199998150765924975758025539107620716094970703125f,0.97040002874136011001837687217630445957183837890625f,-0.3738000095345082041120576832327060401439666748046875f,-0.0709000115267930386409034326788969337940216064453125f,0.96130002626106236807146387945977039635181427001953125f,-0.30590001536384237024179810759960673749446868896484375f,0.7915999828666449023018003572360612452030181884765625f,-0.037999992862344011967934420681558549404144287109375f,-0.35329999765455699645144704845733940601348876953125f,-0.9341000194489963437405322110862471163272857666015625f,-0.8979000467717650479926305706612765789031982421875f,-1.823200023400783464921914855949580669403076171875f,0.21130001704096823544887229218147695064544677734375f,0.45340004134774236632665633806027472019195556640625f,-1.3684000013008705831651923290337435901165008544921875f,-0.85920000355616199527730714180506765842437744140625f,0.55710000994578001698442903943941928446292877197265625f,-1.106600026024132876756311816279776394367218017578125f,-0.7718000070348385310836647477117367088794708251953125f,-0.520299997732788455095942481420934200286865234375f,1.2469000112213191044929772033356130123138427734375f,-0.4990000038444997176867445887182839214801788330078125f,-0.315200013551116320353884248106623999774456024169921875f,0.1381000084668400074150440559606067836284637451171875f,-0.41450002427399212923120330742676742374897003173828125f,0.2098000066399574592423959984444081783294677734375f,0.19599999565258585665361579231102950870990753173828125f,-0.27420000018477441727782206726260483264923095703125f,0.0456999685086303397696383399306796491146087646484375f,0.50319998450875313977803671150468289852142333984375f,0.125099982393532649371792331294273026287555694580078125f,-0.7613000183820728405947875216952525079250335693359375f,0.25700001779198677898108371664420701563358306884765625f,1.074900002872943627352242401684634387493133544921875f,-0.20830001255571872764704721703310497105121612548828125f,0.34550003287196151546822875388897955417633056640625f,-0.2427000296443704296933674413594417273998260498046875f,1.2076000185847277634820784442126750946044921875f,1.438199982497095685829435751657001674175262451171875f,1.617000026285648317525556194595992565155029296875f,-0.6934000165373090940335032428265549242496490478515625f,0.2372999869704255360147726605646312236785888671875f,-0.4602000066965814539798884652554988861083984375f,0.683599949622153868489249362028203904628753662109375f,0.6904000214397909562791255666525103151798248291015625f,-0.959100015351176171662928027217276394367218017578125f,-0.307500012889504237278970322222448885440826416015625f,-0.18139999561607800249163346961722709238529205322265625f,-1.0723000155806534650082539883442223072052001953125f,-0.203799996626376600517005499568767845630645751953125f,0.07589996520280894998222720460034906864166259765625f,-0.211800038290023362463898592977784574031829833984375f,0.15620005359053568128047118079848587512969970703125f,1.16750004932284401348852043156512081623077392578125f,-0.609899988269805959362201974727213382720947265625f,-0.8983999916523697226722333653015084564685821533203125f,0.8775999949291344304214135263464413583278656005859375f,-0.32939999428987487561926172929815948009490966796875f,0.475700023121386816882250059279613196849822998046875f,-0.2837999853014945284002124026301316916942596435546875f,0.63930003860667372617143655588733963668346405029296875f,-0.760600028508901626622673575184307992458343505859375f,-1.3149000113852318261109530794783495366573333740234375f,0.65460000791549699528104611090384423732757568359375f,-0.952600003311037113462589331902563571929931640625f,-0.63449999201297802020604876815923489630222320556640625f,-0.4687000339955085248533350750221870839595794677734375f,-0.129099993018060577032457558743772096931934356689453125f,-0.56320004362911058048979384693666361272335052490234375f,1.6868999615073203557358283433131873607635498046875f,0.30739999564588060820824466645717620849609375f,0.6787000310152766502369559020735323429107666015625f,-0.7279999734908340069949872486176900565624237060546875f,-1.950999962404370347002213748055510222911834716796875f,-1.040099986621737304659518486005254089832305908203125f,-1.5636999896645544794182569603435695171356201171875f,-0.4862000131152564019743067547096870839595794677734375f,-0.4508999958992008227909309425740502774715423583984375f,-0.94639997760429983575392043348983861505985260009765625f,0.33599996175244528640035923672257922589778900146484375f,1.752000008329748670377057351288385689258575439453125f,-1.08430000044107455181574550806544721126556396484375f,0.4898999933362009340243048427510075271129608154296875f,-0.48749999620020378898743729223497211933135986328125f,1.034899998030066381460301272454671561717987060546875f,-0.3327000294953588177548908788594417273998260498046875f,-0.531299984109402156917667525704018771648406982421875f,0.3540000352859493659707368351519107818603515625f,0.02670000301599506631333724726573564112186431884765625f,0.318300010264664035020132359932176768779754638671875f,-1.1358999554425484035391491488553583621978759765625f,-0.20099997382238521392849861513241194188594818115234375f,0.033399961999058813688634472782723605632781982421875f,0.73670001553297070895354181629954837262630462646484375f,1.5227999988690006372138441292918287217617034912109375f,-0.6838000107079740264026668228325434029102325439453125f,-1.257299989335983969596099996124394237995147705078125f,0.8411999932408329438970895353122614324092864990234375f,-0.4965000410042701428636746641132049262523651123046875f,0.225500027209520670368192440946586430072784423828125f,1.4342000378288333184428893218864686787128448486328125f,-1.00590002436041903166596966912038624286651611328125f,-1.43350005812570469032607434201054275035858154296875f,1.265400044089556530479967477731406688690185546875f,-0.01890003553256480728350652498193085193634033203125f,0.86239998618364399618485549581237137317657470703125f,-0.8204999977052216397765960209653712809085845947265625f,0.001500028505920791843664119369350373744964599609375f,0.3897999911427501018579278024844825267791748046875f,0.46600002551078745227641775272786617279052734375f,1.582100001657008814248683847836218774318695068359375f,1.59989995861649614283805931336246430873870849609375f,-0.08310000816881579766004506382159888744354248046875f,-0.961700024771690920033506699837744235992431640625f,-0.149899988269806438978548612794838845729827880859375f,-0.31800000262260397221325547434389591217041015625f,-0.535499991521239504521645358181558549404144287109375f,-0.7389999961704010189578184508718550205230712890625f,0.73429999507665666413913641008548438549041748046875f,-0.514499990299343235022888620733283460140228271484375f,-1.061400012528896041885673184879124164581298828125f,-0.13170000703930906382765897433273494243621826171875f,0.1333000258922572012210139291710220277309417724609375f,1.069300035607815058114056228077970445156097412109375f,1.818800001394748733218875713646411895751953125f,0.72699997338652533329650395899079740047454833984375f,-0.97809999900460287136638726224191486835479736328125f,0.88540005183816017364506478770636022090911865234375f,-0.232299979668855627323864609934389591217041015625f,-0.3316000611722476065779119380749762058258056640625f,-0.1516999960497017740923553219545283354818820953369140625f,-0.359600008511543389744247178896330296993255615234375f,-0.2870999994218348394525719413650222122669219970703125f,1.2289000115230679366362664950429461896419525146484375f,-0.30690000365897984746510473996750079095363616943359375f,0.50500001065432986724346164919552393257617950439453125f,0.26709996965676541247347586249816231429576873779296875f,0.03490002894997612514771390124224126338958740234375f,-0.1792000179544093185768360854126513004302978515625f,-0.9404000392466784763456644213874824345111846923828125f,-0.78380003078728943677333518280647695064544677734375f,-2.395000007748603110258045489899814128875732421875f,0.534200030285120419648592360317707061767578125f,-1.39930002696514055315901714493520557880401611328125f,0.3687999857112764789945913435076363384723663330078125f,-0.4087000602774317759013911199872381985187530517578125f,1.6231999960012739592229991103522479534149169921875f,1.1330999640800059058420856672455556690692901611328125f,-1.079300041791797060142243935842998325824737548828125f,0.02149998700618738212142488919198513031005859375f,-1.195000012740492945084724851767532527446746826171875f,0.018899999602139505583409118116833269596099853515625f,-0.35370001793950789359399777822545729577541351318359375f,-0.06140004799366050036724118399433791637420654296875f,0.4264999722540370186152358655817806720733642578125f,-0.898900048887730118707395376986823976039886474609375f,-0.4262999959558249063462653793976642191410064697265625f,0.43960001251623059825845984960324130952358245849609375f,0.38379999582543911085252830162062309682369232177734375f,0.9224999893642953363581682424410246312618255615234375f,0.294300016981363654622327885590493679046630859375f,1.23310004802942341228799705277197062969207763671875f,-0.51039997181892393651736483661807142198085784912109375f,-0.8139999984428285539905800760607235133647918701171875f,0.23859998616725197617682852069265209138393402099609375f,0.292300003957748355531975903431884944438934326171875f,1.1202000229947264298147047156817279756069183349609375f,-0.274100041502714386609795838012360036373138427734375f,-0.130100011152774808209642287692986428737640380859375f,0.29399999564886114544748352273018099367618560791015625f,0.337300034970789919253064681470277719199657440185546875f,-0.433700014400482114496071517351083457469940185546875f,-0.488199966590106004815652340766973793506622314453125f,-0.00530001283884062868168030036031268537044525146484375f,-0.93480001757368424986083255134872160851955413818359375f,0.91800000504404299928040700251585803925991058349609375f,-0.38340002009496088053452922395081259310245513916015625f,-0.69320000171959339496652319212444126605987548828125f,-0.518900007686018849284437237656675279140472412109375f,-1.646200006875395249750226867035962641239166259765625f,-0.50609998195767413875501006259582936763763427734375f,-1.324700004446506707012076731189154088497161865234375f,-0.51409998368620868358647157947416417300701141357421875f,-1.626299936202169060805999833974055945873260498046875f,0.3240000180751085867569827314582653343677520751953125f,-2.146399968719482131973563809879124164581298828125f,-0.5957000243321062260548615086008794605731964111328125f,-1.001200015667080389647480842540971934795379638671875f,0.209399994066358008382167099625803530216217041015625f,-0.041500020161270523288976619369350373744964599609375f,1.3942000072628257800744222549838013947010040283203125f,0.3731999737054110255485284142196178436279296875f,-0.36890002519488351850895924144424498081207275390625f,-0.6814000064194198902356447433703579008579254150390625f,0.90989997806251066148064410299411974847316741943359375f,-0.43910001527667053000669739049044437706470489501953125f,0.609599974909424968672055911156348884105682373046875f,0.04389996756464220606375192801351658999919891357421875f,-0.2074999820068479383650128511362709105014801025390625f,1.1969000297918916952966128519619815051555633544921875f,0.83819995962381366183535646996460855007171630859375f,-0.9096999995291223939375413465313613414764404296875f,-0.42359996662810417422662112585385330021381378173828125f,1.1861000083327291054757779420469887554645538330078125f,-0.377900050236285256488599770818836987018585205078125f,-0.10539997539520218339248458505608141422271728515625f,-0.01400000505521903360062196952640078961849212646484375f,-0.1462000068008901809690769368899054825305938720703125f,0.6704000139147046066767643424100242555141448974609375f,-0.453000000089406829051341674130526371300220489501953125f,-0.6764000159189105243484618767979554831981658935546875f,0.10509999187439678092204076165216974914073944091796875f,-0.9477000423848631616152715650969184935092926025390625f,0.959300057959556085052099660970270633697509765625f,0.27559998030364507570766363642178475856781005859375f,0.07670000271797139390628217370249330997467041015625f,0.3116999893814320632401404509437270462512969970703125f,0.88120000233054174287872228887863457202911376953125f,0.620400004526973081198093495913781225681304931640625f,0.9033999902367586543050492764450609683990478515625f,-0.5507000006392599456006564651033841073513031005859375f,-0.476299980384111254494428067118860781192779541015625f,-0.5866000055536628732255621798685751855373382568359375f,-0.4696000136896965049260188607149757444858551025390625f,-0.00670002372860911687979523776448331773281097412109375f,-0.4786000038623807739668336580507457256317138671875f,-0.11869997223019534970944732776843011379241943359375f,1.111900045549869542327314775320701301097869873046875f,0.0807000038802623276268377594533376395702362060546875f,0.7868000403761870042984583051293157041072845458984375f,-0.1603000018119811187489176518283784389495849609375f,0.35279999272227247075761624728329479694366455078125f,0.873600016230344778023209073580801486968994140625f,0.3333000089421869205352777498774230480194091796875f,-0.604199985879659617893366885255090892314910888671875f,-0.1492999818518756516283474411466158926486968994140625f,-0.0566999998867512289280057302676141262054443359375f,0.625500026613474258141422978951595723628997802734375f,0.42900000144168692894908190282876603305339813232421875f,-0.42760001649856604899468948133289813995361328125f,-0.43720002048388140192258788374601863324642181396484375f,0.65730001483559608033147014793939888477325439453125f,0.347899997840076469213244081402081064879894256591796875f,0.498600010567903539904222043332993052899837493896484375f,0.02329999566152700374299655550203169696033000946044921875f,-0.283300014511495834579335451053339056670665740966796875f,0.0181999960199002164262793712623533792793750762939453125f,0.0920000011771915371383556703221984207630157470703125f,0.8592000308066614078228440121165476739406585693359375f,-0.054299977083503758734650546102784574031829833984375f,-1.020100008302927019343542269780300557613372802734375f,0.0688000308804215254809832913451828062534332275390625f,0.9789000316023841463675125851295888423919677734375f,1.212399987673759227391201420687139034271240234375f,-0.49950000894442181387233858913532458245754241943359375f,-1.329999954923988259025691149872727692127227783203125f,1.001199975433945343894492907566018402576446533203125f,-0.52479999190643422668500761574250645935535430908203125f,-0.64849998430907707103187931352294981479644775390625f,0.2751000118792052973049067077226936817169189453125f,1.4323000406146049190425628694356419146060943603515625f,-0.27820001432970198340655088031780906021595001220703125f,0.284800047618151364048344476032070815563201904296875f,-0.708500013887881863183793029747903347015380859375f,-0.80970000146627463255555312571232207119464874267578125f,0.82820000849962216449284824193455278873443603515625f,-1.211300029073655526445918440003879368305206298828125f,-0.15960003149658474885796977105201222002506256103515625f,-0.49959999767094842582082492299377918243408203125f,0.62580002030432246851177069402183406054973602294921875f,-0.26800004129111731909773652660078369081020355224609375f,-0.320000035613775768439381863572634756565093994140625f,0.073000004224478665104669516949797980487346649169921875f,0.077899986086785333316839796680142171680927276611328125f,-1.571600033009051600174643681384623050689697265625f,0.0838000141352417404760899444227106869220733642578125f,0.5976000090479853810165877803228795528411865234375f,0.3505000145435335223709216734278015792369842529296875f,-0.088400004653632624496140124392695724964141845703125f,-0.483500016551464906200408222503028810024261474609375f,0.9703000141426920688303425777121447026729583740234375f,0.49280002584010385024981815149658359587192535400390625f,0.2227000146687032611225731670856475830078125f,0.159200002308189869637544688885100185871124267578125f,0.12380005816817263308848851011134684085845947265625f,1.4596000211216502151501117623411118984222412109375f,-0.3537999930500976830671788775362074375152587890625f,0.2426999650478354197957742144353687763214111328125f,1.995000030994415141094577847979962825775146484375f,-0.62920003073215458044842307572253048419952392578125f,-0.570800026525557502310448398930020630359649658203125f,-0.949599978783726594855352232116274535655975341796875f,1.387900051689148650524430195218883454799652099609375f,-0.23170001534670625620293549218331463634967803955078125f,-0.91780000970959729755804801243357360363006591796875f,-0.16090001318454749679176529753021895885467529296875f,-0.114299989712237870520539217977784574031829833984375f,1.192200049112737136880468824529089033603668212890625f,-1.2099999919533728220955026699812151491641998291015625f,0.21899998251721253250678955737384967505931854248046875f,-1.6781999919965866485682681741309352219104766845703125f,-0.6603000217050312148359125785646028816699981689453125f,-0.159500004269182316107844599173404276371002197265625f,0.04609998717308039228868210557266138494014739990234375f,0.211499989613890992945499647248652763664722442626953125f,0.4303999838516114895270447959774173796176910400390625f,2.205899983866511870900239955517463386058807373046875f,0.44130008543729815073675126768648624420166015625f,1.0781000378593807997873454951331950724124908447265625f,-0.22050000530481350669020912391715683043003082275390625f,1.8905999726295472651571571987005881965160369873046875f,0.398400032984465080065916708917939104139804840087890625f,-0.15090000044405460499774562777020037174224853515625f,0.01079998835995776840235294002923183143138885498046875f,-0.75550001439452163509713500388897955417633056640625f,-0.37150001231953500013815983038512058556079864501953125f,-0.691600049549341289178983060992322862148284912109375f,-0.9507000294730068024051661268458701670169830322265625f,-1.3429999549388895729862269945442676544189453125f,0.339099969176202886700366434524767100811004638671875f,0.754000020980836183071005507372319698333740234375f,-1.7462999511033299171458565979264676570892333984375f,1.542100008288026291580763427191413938999176025390625f,0.62079997506067108314908864485914818942546844482421875f,-0.1929000043779609541161335073411464691162109375f,-0.06440001604557021863683985429815948009490966796875f,-0.5711000072523952919567591379745863378047943115234375f,-0.526399981608987044268133104196749627590179443359375f,-0.39790000534653746644409011423704214394092559814453125f,1.0771999784812333889050250945729203522205352783203125f,-0.39909994807243265313445590436458587646484375f,-0.08429998703002894444580306299030780792236328125f,0.393699997785688093898670558701269328594207763671875f,-0.02669999936521072481099281503702513873577117919921875f,0.0157000123739241548292966399458236992359161376953125f,-0.029799995426833536527055912301875650882720947265625f,-1.60640001558363376688021162408404052257537841796875f,0.152199980083107533346264972351491451263427734375f,-1.3618000302061445960788432785193435847759246826171875f,0.080599984252452738786587360664270818233489990234375f,-0.1252999857187264698410444907494820654392242431640625f,0.4423000118553641613061699899844825267791748046875f,0.70659997400045337911933529539965093135833740234375f,0.9748000523865225641628740049782209098339080810546875f,-0.441000016942620032267541319015435874462127685546875f,0.48720001640468846293430260629975236952304840087890625f,0.538500000998377803540506647550500929355621337890625f,-0.007699991273879991382500520558096468448638916015625f,0.9555999924480911289492723881267011165618896484375f,0.06680001614317311597890380880926386453211307525634765625f,-0.715600058460235999291398911736905574798583984375f,-1.0050000544823713966025025001727044582366943359375f,-0.8526000376582150153126349323429167270660400390625f,-0.46090001286789761270057397268828935921192169189453125f,0.1800000241398809208703823969699442386627197265625f,0.2337999980047344994460445377626456320285797119140625f,0.12230002005100271134097056346945464611053466796875f,-0.062999993160367306899161121691577136516571044921875f,0.33059998313486571586139461942366324365139007568359375f,-0.2030999883130191197011527037830092012882232666015625f,1.510500039502978619765372059191577136516571044921875f,-0.0878000041216620008555082677048631012439727783203125f,-0.61540003961920763941861878265626728534698486328125f,-0.55840000543594303650252186344005167484283447265625f,0.566100035154819547500437693088315427303314208984375f,-0.32730001632571248393333007697947323322296142578125f,0.529100029358267942569682418252341449260711669921875f,-1.345000011920928439934641573927365243434906005859375f,-0.8935000222325319985117175747291184961795806884765625f,1.6172000346213575294740394383552484214305877685546875f,0.722499995976686459897564418497495353221893310546875f,0.672900016447901538896303463843651115894317626953125f,0.97560002619922159272647377292742021381855010986328125f,-1.31529997856616898133097492973320186138153076171875f,-0.7412000115320089843606865542824380099773406982421875f,-0.6402000610113152134772462886758148670196533203125f,0.567800052475929906137253055931068956851959228515625f,-0.38839998695850308507715453743003308773040771484375f,0.4755999826878305469790575443767011165618896484375f,1.9109000236153601548494407325051724910736083984375f,-0.01829999486058986324366060216561891138553619384765625f,-0.80230002316162007947042411615257151424884796142578125f,1.10950000512599888935483249952085316181182861328125f,0.6617000054374333473106162273325026035308837890625f,-0.1297999231934543473698795423842966556549072265625f,1.2818000395379964828634911100380122661590576171875f,0.359399966573715090589757892303168773651123046875f,-0.74640003646388708435921444106497801840305328369140625f,-2.2281000209093093644696637056767940521240234375f,-0.0350000078417356252913350544986315071582794189453125f,0.535700006413460272369775339029729366302490234375f,0.57340000813677927116174259936087764799594879150390625f,-0.5802999266542501199950265799998305737972259521484375f,-0.86799998414516377209793063229881227016448974609375f,0.85790002881586513439060581731610000133514404296875f,1.1075999900490047789247682885616086423397064208984375f,-1.28209997513294116089355156873352825641632080078125f,-0.03870002512931858262845707940869033336639404296875f,-0.849000071942807466029989882372319698333740234375f,2.4026000176906592287195962853729724884033203125f,-0.1410999499201768259126765769906342029571533203125f,0.6750999583840364692832736182026565074920654296875f,1.78790006808042445385353858000598847866058349609375f,0.579399927234650391483228304423391819000244140625f,-0.9944999981038280534306750269024632871150970458984375f,-0.980800019857287264102296830969862639904022216796875f,1.4041999909460549655904060273314826190471649169921875f,0.4835000144839280977748785517178475856781005859375f,-0.07149998190253992191856013960205018520355224609375f,0.4789000140935184557378079261980019509792327880859375f,-0.362700013439356816480341194619541056454181671142578125f,0.4757999874472618184739758362411521375179290771484375f,0.8446999876827001596524269189103506505489349365234375f,0.181500018335878643416236855046008713543415069580078125f,-1.266099974544346196125843562185764312744140625f,0.03419996712282291628781649706070311367511749267578125f,-2.04870004152059603796942610642872750759124755859375f,-0.70800002342835066659887388595961965620517730712890625f,-0.294000006079674136572066345252096652984619140625f,-0.30800000051781528664918141657835803925991058349609375f,0.55679995774924773144221035181544721126556396484375f,-0.14629998662397303288429384338087402284145355224609375f,0.15269997421205072640759681235067546367645263671875f,-0.8093000005900858884189119635266251862049102783203125f,-0.235799985882640061873871673014946281909942626953125f,0.225500003386288894180466968464315868914127349853515625f,1.1245999694705013371986979109351523220539093017578125f,0.392900022631883505397354383603669703006744384765625f,-0.66879998679161190011654980480670928955078125f,-0.551299984332918935336920185363851487636566162109375f,0.3937999951362609607485865126363933086395263671875f,-0.129499939262867513889432302676141262054443359375f,0.94719998172223629495647401199676096439361572265625f,-0.59589999113082914306005477556027472019195556640625f,-0.295500010222196518583359647891484200954437255859375f,-0.1982999690443281171070566415437497198581695556640625f,0.10290001393333103596461342021939344704151153564453125f,-0.28449998694658307218929849113919772207736968994140625f,1.552799996522068237680969104985706508159637451171875f,0.39099993522837783732626348864869214594364166259765625f,-0.2543999755144117358440780662931501865386962890625f,1.322900012499093946871653315611183643341064453125f,-0.664099939616023871025163316517136991024017333984375f,-1.62130002940893103158259691554121673107147216796875f,0.536599985213577657106043261592276394367218017578125f,0.2616999840915197328428121181786991655826568603515625f,0.98979998298436366876984493501367978751659393310546875f,0.838400034418702144733970271772705018520355224609375f,-0.760999919191002849316873835050500929355621337890625f,-1.3483000312566755241050486802123486995697021484375f,-0.7255999972537152675755578457028605043888092041015625f,0.371799980175495647927164100110530853271484375f,0.30760005606114848575316500500775873661041259765625f,0.57919999813586431347545158132561482489109039306640625f,-1.1010999639272682770751998759806156158447265625f,0.214300017651916530070366206928156316280364990234375f,-0.81209998615980083513932186178863048553466796875f,-0.04320000516548727631516157998703420162200927734375f,0.76900002962350821889003782416693866252899169921875f,0.2662000020883976691976613437873311340808868408203125f,0.63799999487400071984666283242404460906982421875f,1.06200002637878032629714653012342751026153564453125f,-0.088800016593932440400749328546226024627685546875f,-0.849599997726827371735680571873672306537628173828125f,-0.023599992537498337696888484060764312744140625f,0.554799969238043644281788147054612636566162109375f,-1.12419998632669404514672351069748401641845703125f,1.0013999785542484488587433588691055774688720703125f,-0.095500021025539449937014069291763007640838623046875f,-0.679600001955031896017089820816181600093841552734375f,0.1194000148534775096464954913244582712650299072265625f,0.094400034001469901312475485610775649547576904296875f,0.590499962091445951273271930404007434844970703125f,0.7236999896645546215268041123636066913604736328125f,-1.0176999927639958087866034475155174732208251953125f,0.3414000307083124852169930818490684032440185546875f,1.38879999573230605847129481844604015350341796875f,0.4383999773472544436714315452263690531253814697265625f,-0.4097000238180161790779720831778831779956817626953125f,-0.4937000112712379706891852038097567856311798095703125f,0.31959998176395831848140005604363977909088134765625f,1.04540002516508057084365646005608141422271728515625f,-1.291999990187585378720314110978506505489349365234375f,0.66529997901320481190623468137346208095550537109375f,0.42649999009817829431767677306197583675384521484375f,-0.07059999706745134773200334166176617145538330078125f,0.08540005999654509682983416496426798403263092041015625f,-0.17429999686479558107521370402537286281585693359375f,-0.9785000029355288564403281270642764866352081298828125f,0.32070001559630056942040710055152885615825653076171875f,-0.71439996957257412280029029716388322412967681884765625f,0.3584000379018486004412125112139619886875152587890625f,0.9827999508500102621155747328884899616241455078125f,-0.72580004243254681028929553576745092868804931640625f,-0.34370001752972612063530277737299911677837371826171875f,0.980199991367011858045543704065494239330291748046875f,0.19209999845325953504726612663944251835346221923828125f,0.54590000020191087326537626722711138427257537841796875f,-0.2467999645292764387960460226167924702167510986328125f,0.322400018444656932814496030914597213268280029296875f,-0.7033999938875432622609196187113411724567413330078125f,-0.411200029096753016943921466008760035037994384765625f,-0.4085000166632235529817762653692625463008880615234375f,-0.363299987745284713724913672194816172122955322265625f,0.097400023380667055672432752544409595429897308349609375f,-0.15760000159740439329425498726777732372283935546875f,-0.14619997953176488980631120284670032560825347900390625f,0.7204999980777502255335775771527551114559173583984375f,0.344200014750659588624870366402319632470607757568359375f,0.055199992689490162689480712288059294223785400390625f,0.0822000017456712139818364448728971183300018310546875f,0.8721000080645087138009330374188721179962158203125f,-0.10860000622794030444850932326517067849636077880859375f,0.1487000008776788728681594875524751842021942138671875f,-0.03589997801780686614847581950016319751739501953125f,-0.9354999647736548240573029033839702606201171875f,0.5294000309467314924205538773094303905963897705078125f,-1.424099982643127759729395620524883270263671875f,0.131199997487664088424708097591064870357513427734375f,-1.145599983842670699374366449774242937564849853515625f,0.38700001745671064323772725401795469224452972412109375f,-0.0889000148758294361783782733255065977573394775390625f,0.0048000157296657075534085379331372678279876708984375f,0.90659997295737326794551336206495761871337890625f,1.22540000184476394196053661289624869823455810546875f,-1.3793000492610041352037342221592552959918975830078125f,0.44439996523261005023641700972802937030792236328125f,-0.443500020146370133033997262828052043914794921875f,0.507799972307682256911220974870957434177398681640625f,-0.087600024545192667346782400272786617279052734375f,0.56680000267624863763415987705229781568050384521484375f,0.8605000051185489606808687312877736985683441162109375f,-0.09790001447349794272412282225559465587139129638671875f,-0.28220003227442536086755353608168661594390869140625f,-0.622399959510565370379708838299848139286041259765625f,0.01910002749562256241233626496978104114532470703125f,0.422299989950656762260905452421866357326507568359375f,1.010100002193450308141109417192637920379638671875f,1.4762000115878881612019313251948915421962738037109375f,-0.7470000041015445457759369674022309482097625732421875f,0.6099000357486306089782601702609099447727203369140625f,0.2395000009536742968663247665972448885440826416015625f,-0.53009999653100969840835432478343136608600616455078125f,0.53809999431073674003300766344182193279266357421875f,0.27259998451694811283374519916833378374576568603515625f,-0.60500002238899430739849094607052393257617950439453125f,-1.3835000094361600275760793010704219341278076171875f,0.47960000627636834025224743527360260486602783203125f,-0.283400043992698191797074969144887290894985198974609375f,-0.4375000520050524865922625394887290894985198974609375f,0.33420002050623309752808154371450655162334442138671875f,0.50099996495619458780623745042248629033565521240234375f,0.100199999991059218729105850798077881336212158203125f,-1.050599975311756306695087914704345166683197021484375f,-0.22200000300258404539732737248414196074008941650390625f,-0.969899979031085823777402765699662268161773681640625f,-1.41449997506290703341846892726607620716094970703125f,-0.9861000084444875302125410598819144070148468017578125f,-0.29360004551112606208107536076568067073822021484375f,0.57969996182918492877433891408145427703857421875f,1.137000019729137445523292626603506505489349365234375f,1.345500002715735998748414203873835504055023193359375f,0.55129999997913803344573580034193582832813262939453125f,-0.50060004223659648747712935801246203482151031494140625f,0.769599974089861138537571605411358177661895751953125f,-0.2196000224441287951293588776024989783763885498046875f,0.0199999956041574389331572092487476766109466552734375f,0.36480000004619361764213181231752969324588775634765625f,0.101800043654441285667644478962756693363189697265625f,-0.3968999651208517942535536349168978631496429443359375f,-1.4790000038444990337893614196218550205230712890625f,-0.39069998093247448878173599950969219207763671875f,-0.600900027638674227858928134082816541194915771484375f,-0.094299992990493575195642961261910386383533477783203125f,0.06519998862892373292510228566243313252925872802734375f,-0.35289999629408119741214022724307142198085784912109375f,0.8990000524967916017970992470509372651576995849609375f,-0.6248000390499834910684739952557720243930816650390625f,-0.09239999808594578922793516539968550205230712890625f,0.4367000116586685631858699707663618028163909912109375f,-0.4617000090137126644407317144214175641536712646484375f,-0.317500025480985659243060581502504646778106689453125f,-1.105000032447278623948250242392532527446746826171875f,0.96340002823472026260986922352458350360393524169921875f,0.23420001134201928749689614051021635532379150390625f,0.6804000252395869807742201373912394046783447265625f,0.4221999944254755110506494020228274166584014892578125f,0.2947000043347476339050672322628088295459747314453125f,0.44280002602636869823982124216854572296142578125f,-1.2102000142216677858186812954954802989959716796875f,-0.577400014868378885779520715004764497280120849609375f,-2.3167000170230860334186218096874654293060302734375f,-0.234299977493285904728281821007840335369110107421875f,-2.01260000413060158308553582173772156238555908203125f,0.4584999978691339794778514260542578995227813720703125f,-0.03780003906488449416656294488348066806793212890625f,0.256499995723367124611513645504601299762725830078125f,-0.2230999987438320175670014577917754650115966796875f,0.393999992817640476427953899474232457578182220458984375f,0.456100001217424999566674159723334014415740966796875f,1.231900009712576515852333614020608365535736083984375f,-0.03420005181729879328855759013094939291477203369140625f,0.84179999537467953274472165503539144992828369140625f,0.64310001852512410636109052575193345546722412109375f,0.6606999998569482723809187518781982362270355224609375f,0.1986999989032742774952566833235323429107666015625f,-0.847400010025501249089074917719699442386627197265625f,0.67170003170073044884702540002763271331787109375f,1.054400007253885274138838212820701301097869873046875f,0.4632000189304352755215177239733748137950897216796875f,0.71010000213757162867977967835031449794769287109375f,-0.2142000206738712275722491540363989770412445068359375f,-0.32170002363547711698998909923830069601535797119140625f,0.86050004255771650463202604441903531551361083984375f,1.87760004073157915627234615385532379150390625f,-1.287500031292438507080078125f,-0.50180000057145990322027273577987216413021087646484375f,-0.563500018209218911380276040290482342243194580078125f,0.185499997109174252774010938082938082516193389892578125f,0.4737999982655054509450565092265605926513671875f,0.20830003606230007573429929834674112498760223388671875f,-0.613700038242340095706595093361102044582366943359375f,-0.35880000251233579167120524289202876389026641845703125f,1.77199994418025053022347492515109479427337646484375f,0.9180999937891962670022394377156160771846771240234375f,2.9222999947190277936215352383442223072052001953125f,-1.438600016602874287485747117898426949977874755859375f,0.55270001630783127666290965862572193145751953125f,-0.59519997562766047138893554802052676677703857421875f,-1.67299996149539964562791283242404460906982421875f,-0.88929994590282479549614436109550297260284423828125f,-0.1871000207304955864628936978988349437713623046875f,0.75569998815953720594507103669457137584686279296875f,1.0578000510603191575143000591197051107883453369140625f,-1.227799962994456262066478302585892379283905029296875f,-1.2007000041037798609977471642196178436279296875f,-1.23950002002716086479949808563105762004852294921875f,-0.45089998978972456455949213705025613307952880859375f,0.56100005024671606435049397987313568592071533203125f,-1.04509999757409044462974634370766580104827880859375f,0.2946999746441836265375968650914728641510009765625f,-0.994100016468763669763575308024883270263671875f,-1.714099986070394709969377800007350742816925048828125f,-1.57379998694062184227959733107127249240875244140625f,-0.5518999808043238974875066560343839228153228759765625f,0.8533000465303663961691427175537683069705963134765625f,-0.74229999868646190375187643439858220517635345458984375f,-0.716900027221441860092454589903354644775390625f,-0.168900028454512185493996412333217449486255645751953125f,-0.3801999808430671645709253425593487918376922607421875f,0.000599996713548878179977918989607132971286773681640625f,0.1171999908119440991782767014228738844394683837890625f,-0.58130000200942111288071600938565097749233245849609375f,0.1156000013887879263041469357631285674870014190673828125f,1.1293999972142276799758064953493885695934295654296875f,-0.7711000007145101253769325921894051134586334228515625f,0.27380007036849907109399282489903271198272705078125f,-0.342399950197339375534966166014783084392547607421875f,0.036399977511167858068574787466786801815032958984375f,-0.04129999811649298635529703460633754730224609375f,0.4620999914497136984437020146287977695465087890625f,-0.535500011414289378564035359886474907398223876953125f,-0.5285999967098236052009951890795491635799407958984375f,-0.74879998537600034325123488088138401508331298828125f,0.216999989461154008107968138574506156146526336669921875f,0.3532999845415354389643880494986660778522491455078125f,0.42250004993751610893326642326428554952144622802734375f,-0.38820000635758000750996643546386621892452239990234375f,-0.022700008708238073040774906985461711883544921875f,0.834200041684508786232754573575221002101898193359375f,0.1093000165343287477526246220804750919342041015625f,0.55739998782277044853117331513203680515289306640625f,0.583199966776370981591526287957094609737396240234375f,0.6925999808102847321578110495465807616710662841796875f,0.2271000059038399410127340161125175654888153076171875f,0.63019999827742567699573328354745171964168548583984375f,-0.42019999686181541687091112180496565997600555419921875f,0.095699997603147846092497275094501674175262451171875f,0.87489999966919362872630472338641993701457977294921875f,-0.51500000907108145487001138462801463901996612548828125f,1.5995000073611738145018534851260483264923095703125f,0.360400003707409322117882766178809106349945068359375f,0.63490003099888492332780742799513973295688629150390625f,-0.378300011587142659408300460199825465679168701171875f,-0.9089000017069281955883752743829973042011260986328125f,0.43630001063346901535311417319462634623050689697265625f,-0.154200003295391796509505866197287105023860931396484375f,0.5410999985724684879784263102919794619083404541015625f,1.107300016232579675801162011339329183101654052734375f,-0.8662999990105628711489771376363933086395263671875f,0.3662000096507365132225686465972103178501129150390625f,0.64589998442530660582860946306027472019195556640625f,0.2714000060468906294630642150877974927425384521484375f,0.13730000231862060644516532192938029766082763671875f,-0.13119997826516627270621029310859739780426025390625f,-0.0190999063491830156635842286050319671630859375f,0.06420001463890034898440717370249330997467041015625f,-0.68779995770454416259553909185342490673065185546875f,-0.2552000268876553068508883370668627321720123291015625f,1.503899978479742660653073471621610224246978759765625f,1.033499987065791714258011779747903347015380859375f,1.14150003647804254569564363919198513031005859375f,-1.210699992182850959210327346227131783962249755859375f,-0.608699951815605544425125117413699626922607421875f,0.6961000223398201569580123759806156158447265625f,-1.353800006312130932428772212006151676177978515625f,0.61829997001290326608113900874741375446319580078125f,-0.91260002365112313782447017729282379150390625f,-0.24940000174045540148881627828814089298248291015625f,0.2984000242859128348982267198152840137481689453125f,-1.421799989414214593352880910970270633697509765625f,-0.198899996212124730021741925156675279140472412109375f,0.7954999579191213143758432124741375446319580078125f,-0.84010005151629396635826196870766580104827880859375f,-0.3568000043153762845804521930404007434844970703125f,-0.02960001283287994056081515736877918243408203125f,-0.36290001279711692205864892457611858844757080078125f,0.332299995464086617147358992951922118663787841796875f,-0.7335000405609612972313016143743880093097686767578125f,0.410900037324429145968451848602853715419769287109375f,-0.42100000410899507397033403321984224021434783935546875f,-0.50309999974966057578740219469182193279266357421875f,0.77669999061077754642923309802426956593990325927734375f,-0.0189999986290925182430555651080794632434844970703125f,-0.30699999564513624772388311612303368747234344482421875f,0.61959999107718477517892097239382565021514892578125f,0.5241000000029796268563586636446416378021240234375f,-0.59200000467896440881077069207094609737396240234375f,0.73440003465339509514109295196249149739742279052734375f,0.589399941390753578929206923930905759334564208984375f,-1.1588000618934639618373694247566163539886474609375f,-1.88839998040199219531132257543504238128662109375f,0.766800014373659966082641403772868216037750244140625f,0.329000052571296830450364723219536244869232177734375f,-0.2170000116825108182894155106623657047748565673828125f,-0.646200014176964376844125581555999815464019775390625f,-1.1890000156164166611461041611619293689727783203125f,0.08100006157159800324052412179298698902130126953125f,0.097099992269278345702332444489002227783203125f,0.6249000100255006628913179156370460987091064453125f,0.310299980652331885977446290780790150165557861328125f,-0.0770000075101844316804999834857881069183349609375f,-0.8280000240802765087977377334027551114559173583984375f,-0.50080003162920549986125706709572114050388336181640625f,-0.99119996589720293922454175117309205234050750732421875f,-0.95830001359879946054576294045546092092990875244140625f,-0.20029996757656298722594101491267792880535125732421875f,0.1550999971643094443862764819641597568988800048828125f,-0.457600035944580429969619217445142567157745361328125f,0.02329997850656440050443052314221858978271484375f,-1.095200001872330464181004572310484945774078369140625f,0.349899988288432439720310185293783433735370635986328125f,-0.2446000157199799129870143588050268590450286865234375f,0.19050003307685348108435618996736593544483184814453125f,0.4443000382900237354988348670303821563720703125f,0.569600003892183348597200165386311709880828857421875f,-0.4073000057086346714640967547893524169921875f,0.34939999954253397884684773089247755706310272216796875f,0.480500007838011722327564712031744420528411865234375f,0.20850003869831590375127916558994911611080169677734375f,1.47990001211166344319281051866710186004638671875f,-0.2665999865733088558528152134385891258716583251953125f,1.3579999512694775898324905938352458178997039794921875f,-0.3537999796763064974669532603002153337001800537109375f,-0.6601000383287669581733325685490854084491729736328125f,-0.343699995923042678214187617413699626922607421875f,0.2162000113829967151701794136897660791873931884765625f,0.6602999960005282442665475173271261155605316162109375f,-0.15460002739876455013501299617928452789783477783203125f,-0.4592000051394105053503835733863525092601776123046875f,0.175799971316755065942061264649964869022369384765625f,-2.05899996007233898609456446138210594654083251953125f,-0.1382000394940376963859307579696178436279296875f,-1.10539989880323474835677188821136951446533203125f,0.75270001260116803365463056252337992191314697265625f,0.791100012728570778364201032673008739948272705078125f,-0.34249998604878795216421849545440636575222015380859375f,0.089700011040270200002311185016878880560398101806640625f,-1.040199996284395300705227782600559294223785400390625f,0.03569998583123112911863472618279047310352325439453125f,-0.25410003744214815224466974541428498923778533935546875f,0.8100999972760671852256564307026565074920654296875f,0.4340000025779009451554202314582653343677520751953125f,-0.170699977356195464750499013462103903293609619140625f,-0.2668000472307199277821609939564950764179229736328125f,0.69649998164176896153776397113688290119171142578125f,1.0976999787196521918275493590044789016246795654296875f,-0.26940000956356568995175848613143898546695709228515625f,0.779699990811943965951513746404089033603668212890625f,0.23239999229311958384158742774161510169506072998046875f,0.0269999890327461411487774967099539935588836669921875f,-0.6827999844148748831429429628769867122173309326171875f,-0.37810000109076469243518658913671970367431640625f,-0.59810001843199123161554098260239697992801666259765625f,-0.25479997043013558055690737091936171054840087890625f,0.3454000217750670120864242562674917280673980712890625f,1.159899969792366647425296832807362079620361328125f,-1.123399978651106412286253544152714312076568603515625f,-0.807400024032592256162388366647064685821533203125f,-0.8048999927029012457779799660784192383289337158203125f,0.421299943876266791420448498683981597423553466796875f,-0.68760000023767331445156969493837095797061920166015625f,0.291300003946572960700223120511509478092193603515625f,-0.2093000126414004125052770177717320621013641357421875f,0.4143999945878977797519837622530758380889892578125f,-0.56679998881816828770752181299030780792236328125f,0.18559999617338274191524760681204497814178466796875f,-1.877099953079224281538017748971469700336456298828125f,-0.4204000376820555828771830420009791851043701171875f,-0.6275999086141599292432147194631397724151611328125f,-1.1241000369206062270421853099833242595195770263671875f,-0.0871999455869210127190171988331712782382965087890625f,1.170399987167119792985658932593651115894317626953125f,0.25859999060034777329519783961586654186248779296875f,1.4479999850019804608081130936625413596630096435546875f,0.5503000236421817792376032230095006525516510009765625f,-0.3024999862909314440884145369636826217174530029296875f,-0.3894000093400478146321574968169443309307098388671875f,0.9912999866798519743582573937601409852504730224609375f,0.2902999788641924538978855707682669162750244140625f,1.9132999998897315396817475630086846649646759033203125f,0.65000006526708631326982867904007434844970703125f,0.231999998867511880717984240618534386157989501953125f,-0.2833999780178064753499711514450609683990478515625f,-0.2714999933391817332761775105609558522701263427734375f,-1.006999999463557227130650062463246285915374755859375f,-0.00749996758997450996275802026502788066864013671875f,0.223399988485872835752843457157723605632781982421875f,-0.513000004038214552082308728131465613842010498046875f,-0.14629999856352782217072672210633754730224609375f,-1.426300055336952254947391338646411895751953125f,-0.644700044735521782257592349196784198284149169921875f,0.128300009799001912824678584001958370208740234375f,0.543499983638524586382345660240389406681060791015625f,0.81230000813007308124724659137427806854248046875f,0.374800015971809674653059119009412825107574462890625f,-0.816399988687038558055064640939235687255859375f,-0.508099998110532968809138765209354460239410400390625f,0.68060000280439769682061523781158030033111572265625f,0.99839999217391106611785289715044200420379638671875f,-0.0251999856114382136951235224842093884944915771484375f,0.8405999981850380553538570893579162657260894775390625f,-1.08620005880594394653826384455896914005279541015625f,-0.251599995458125391678549931384623050689697265625f,0.3739000476956366725289626629091799259185791015625f,0.0599000115156176349273664527572691440582275390625f,-0.1306999563455573110104523948393762111663818359375f,1.173299961817264147612149827182292938232421875f,-0.687400026118755391735248849727213382720947265625f,1.31640003249645243244003722793422639369964599609375f,0.59759998140633074914518374498584307730197906494140625f,-1.49950000333786004347302878159098327159881591796875f,0.83829999549388867219334997571422718465328216552734375f,0.5835999925374977248537788909743539988994598388671875f,-0.86589998509585874852945153179462067782878875732421875f,0.168999985888600434691397822462022304534912109375f,0.7383999914661052432762744501815177500247955322265625f,-0.52700001779198668572234964813105762004852294921875f,1.33810000530034312049565414781682193279266357421875f,-0.260999948918819058008011779747903347015380859375f,0.92729999121725492639001231509610079228878021240234375f,0.5345000097639862612908245864673517644405364990234375f,-1.3455999446153636966982958256267011165618896484375f,-0.19620000598132547597884922652156092226505279541015625f,0.10469999267458973690736456774175167083740234375f,0.34170001318603748785562856937758624553680419921875f,0.31629998783469215339891889016143977642059326171875f,0.0682999962575732977310138949178508482873439788818359375f,-1.6708000097990041155071594403125345706939697265625f,-0.28760004421472584734686961382976733148097991943359375f,-0.7004999865293513039432582445442676544189453125f,0.0886999924212694690339731096173636615276336669921875f,0.536199973087013059824812444276176393032073974609375f,0.00139996901750549795906408689916133880615234375f,-2.19850000613927765158450711169280111789703369140625f,0.78709999748468373326204527984373271465301513671875f,1.111299992677568848620239805313758552074432373046875f,0.484699982392787998008998329169116914272308349609375f,0.5691000091671940541715457584359683096408843994140625f,0.85269997152984178256929226336069405078887939453125f,-1.78050002661347495092059034504927694797515869140625f,0.015200007814169413933313990128226578235626220703125f,-0.138999968826770015084548504091799259185791015625f,-0.246099992239475540145576815120875835418701171875f,-0.108900038941204702069853738066740334033966064453125f,-1.01860004230737732910938575514592230319976806640625f,-0.54359997856765984725058160620392300188541412353515625f,-0.19709998109340620686680267681367695331573486328125f,0.12559996376335647028099629096686840057373046875f,0.92169998442679546002409551874734461307525634765625f,-1.1724000155389313615472701712860725820064544677734375f,0.01119998988807202611184266061172820627689361572265625f,-0.9177999847128985866362427259446121752262115478515625f,-0.879200036990642530554396216757595539093017578125f,0.56169998040348279033651124336756765842437744140625f,0.459200020897387872764738858677446842193603515625f,-0.0080000171512366602399879411677829921245574951171875f,-0.06870002626553184388313866293174214661121368408203125f,-0.9692999903082846202551081660203635692596435546875f,-0.24339999765008624166284789680503308773040771484375f,-0.196999994136392242438660105108283460140228271484375f,0.509400021149218229510324817965738475322723388671875f,1.0753000176444651447837941304896958172321319580078125f,0.07240000391602519069778054472408257424831390380859375f,-0.940600001239776162265115999616682529449462890625f,-0.3610000101625916446579367402591742575168609619140625f,-0.23339998517036519842804409563541412353515625f,0.639899988642334438537773166899569332599639892578125f,0.564699964809417309652417316101491451263427734375f,0.909199985432625368275694199837744235992431640625f,0.5563000586152089255165265058167278766632080078125f,0.764199975895881067344816983677446842193603515625f,1.073300019522010817496493473299778997898101806640625f,-1.391100021967292210689493003883399069309234619140625f,0.30020000170469274269891002404619939625263214111328125f,0.35849998047202868978189371773623861372470855712890625f,-0.72860001548528696702078377711586654186248779296875f,0.50019998203516014001479561557061970233917236328125f,0.8580000073909754831902318983338773250579833984375f,-1.72759997686147670492573524825274944305419921875f,-1.44790003589391602645264356397092342376708984375f,0.2242000427275903096102638301090337336063385009765625f,-0.0359000131845472747471603724989108741283416748046875f,-0.1908000147163873005951018058112822473049163818359375f,0.0894000509515404662153059689444489777088165283203125f,-1.58109999819993873870771494694054126739501953125f,0.412799989555776125627062356215901672840118408203125f,0.92039999566078112280820278101600706577301025390625f,0.0990000054836277509906494742608629167079925537109375f,-0.8620000192075967238025668848422355949878692626953125f,0.3453000167131416997534643087419681251049041748046875f,-0.7122999942719936417034887199406512081623077392578125f,-0.35479996752440901008895934864995069801807403564453125f,0.9612000132828952825292390116374008357524871826171875f,0.568000005379318384512998818536289036273956298828125f,0.67309998931884706951933594609727151691913604736328125f,-1.527499992847442200627483543939888477325439453125f,0.15440000432953271403135886430391110479831695556640625f,0.22600000047683721504654386080801486968994140625f,0.5264000017814336462862456755829043686389923095703125f,-0.81049999718368059831163918715901672840118408203125f,0.257399995776265821945116840652190148830413818359375f,-0.905800020602346211973099343595094978809356689453125f,-0.1763000389643021303953673850628547370433807373046875f,0.1644999571442600228010633145458996295928955078125f,0.51199997755885107153517310507595539093017578125f,-0.0039999816417697342529891102458350360393524169921875f,1.4981999982923268621703982717008329927921295166015625f,-1.609299990308283412332457373850047588348388671875f,1.008199962902068591574789024889469146728515625f,1.2712000251293176233957638032734394073486328125f,0.6877999940633774400566835538484156131744384765625f,-1.1713999968081711955392165691591799259185791015625f,0.41809998440146411979867480113171041011810302734375f,-0.6184000489473346906521555865765549242496490478515625f,-0.0980999922245733291248370733228512108325958251953125f,0.95270001362562251046028904966078698635101318359375f,-0.6844999846555293032679401221685111522674560546875f,0.8393000092886395702151958175818435847759246826171875f,-0.04950001866743003464677030933671630918979644775390625f,-0.002699970039725219095316788298077881336212158203125f,-0.618800011900067392645041763898916542530059814453125f,-1.4614000351414091483093216083943843841552734375f,-0.7471000005394217158283254320849664509296417236328125f,0.85799995072931023454287924323580227792263031005859375f,0.9690999938189974738378396068583242595195770263671875f,1.266600046755373742968231454142369329929351806640625f,0.14000003624707513250058354969951324164867401123046875f,-0.7407000292494891890981989490683190524578094482421875f,-0.8364000058233733714274649173603393137454986572265625f,0.67279997200965890868928909185342490673065185546875f,1.7628000447273262096814505639486014842987060546875f,0.568700026768446331715267660911194980144500732421875f,0.56579999616444087706668142345733940601348876953125f,-1.21680000640153895830053443205542862415313720703125f,0.144600037941337244973283304716460406780242919921875f,-0.17779997141361203460974138579331338405609130859375f,0.663500018209218556108908160240389406681060791015625f,0.036699995565414145204385931720025837421417236328125f,0.048699991452693769389270528336055576801300048828125f,-0.4664000169992441602317967408453114330768585205078125f,0.8665999872624876587678954820148646831512451171875f,0.9178000652723021790535540276323445141315460205078125f,-0.061100021296739914777873536877450533211231231689453125f,0.06870001464262609491839839392923749983310699462890625f,0.13650002168864006080184481106698513031005859375f,-1.0107000295288866009713046878459863364696502685546875f,1.01040003056675242731898833881132304668426513671875f,-0.010699977169930097176120398216880857944488525390625f,0.302999979972838673347723670303821563720703125f,1.3159000144511467933483572778641246259212493896484375f,-0.71930001698136347698664394556544721126556396484375f,0.542700020927191051356430762098170816898345947265625f,0.216800043654441054741255356930196285247802734375f,0.099000004626810689245530738844536244869232177734375f,0.519800007534027219691097343456931412220001220703125f,0.72659998148828730535342401708476245403289794921875f,-0.600799967442453120014533851644955575466156005859375f,1.2988999945357437848514337019878439605236053466796875f,-0.100599945881963304117334701004438102245330810546875f,0.84139996007680917244897500495426356792449951171875f,0.86009999146461524333062698133289813995361328125f,-0.39869999730139926708005759792285971343517303466796875f,-0.06730003256797745248007913687615655362606048583984375f,-1.297799993672221763318930243258364498615264892578125f,-0.00600002118945131002192283631302416324615478515625f,0.0046000187933448177801665224251337349414825439453125f,-0.91950002982467449186287922202609479427337646484375f,0.796299946036935324400474200956523418426513671875f,0.05899996867775947340106768024270422756671905517578125f,0.1101999408707026173459553319844417273998260498046875f,1.529599985750019985886183349066413938999176025390625f,0.4139000201284890323449872084893286228179931640625f,1.5822999760925764434205120778642594814300537109375f,0.596900003491341646366663553635589778423309326171875f,-0.210700052215904543029267870224430225789546966552734375f,-0.543999977227300757220973537187092006206512451171875f,0.2675999949850140779972207383252680301666259765625f,0.278500004164874415124586448655463755130767822265625f,-0.483099996862560221444482522201724350452423095703125f,-0.4490999792531129575223758365609683096408843994140625f,0.1646999977409839832631632816628552973270416259765625f,-1.342700002319364926961497985757887363433837890625f,0.546700022536515728432959804194979369640350341796875f,0.00909999780505998945301371350069530308246612548828125f,0.137200003105401702185872636619023978710174560546875f,-0.73760001791417495464742160038440488278865814208984375f,-0.426299986493587745872702043925528414547443389892578125f,0.051199974726140871916779673256314708851277828216552734375f,-0.8392999777354301915721634941291995346546173095703125f,-0.085699965379387066377603332512080669403076171875f,-1.5834999550655488587125319099868647754192352294921875f,-0.3770999924927951330033693011500872671604156494140625f,-1.7430999709904195782428359962068498134613037109375f,1.42849997201561951243320436333306133747100830078125f,-0.2154999784827236197770616854541003704071044921875f,-0.260300022673606523682110491790808737277984619140625f,0.057900000429153664072146057151257991790771484375f,-0.244200021381675735909766444819979369640350341796875f,0.050200010049343379847641699598170816898345947265625f,-0.6190999889388681953761306431260891258716583251953125f,0.102499999552965126436987475244677625596523284912109375f,1.23269997899904826255124135059304535388946533203125f,0.5722999911241231085767822150955908000469207763671875f,0.264199996757507715727797403815202414989471435546875f,0.4453000055372715593193788663484156131744384765625f,0.3075999518275265298683507353416644036769866943359375f,1.35269997428655575077982575749047100543975830078125f,0.30020000706911087462458453956060111522674560546875f,1.59259997477531367593428512918762862682342529296875f,0.3275000034272668170132192244636826217174530029296875f,0.96489999516159219439259686623699963092803955078125f,-0.158299986702203965283075604020268656313419342041015625f,0.873100025677681745861491435789503157138824462890625f,0.0133000069491566730306431054486893117427825927734375f,-0.80130000780224808210050468915142118930816650390625f,0.4918000295542184208130720435292460024356842041015625f,-0.678600017124414645053320782608352601528167724609375f,-1.0641000402361158183595080117811448872089385986328125f,0.48670000942349478378901039832271635532379150390625f,-0.700000033080578010213912421022541821002960205078125f,0.08809997531175550733451018459163606166839599609375f,0.91159998288527133158964943504543043673038482666015625f,-0.38009999876618394409888423979282379150390625f,0.249800008521229077729230994009412825107574462890625f,-0.5899999603629115796366022550500929355621337890625f,-0.441600001940131381417131706257350742816925048828125f,0.81880000087320770063570307684130966663360595703125f,-0.4147999950915572942022890856605954468250274658203125f,-0.584499970927834677780765559873543679714202880859375f,-0.751500010475516955210650849039666354656219482421875f,-0.7371000076174727411171261337585747241973876953125f,0.096499970465898865512599513749592006206512451171875f,1.554099906945229037091849022544920444488525390625f,-0.23079998998045869740280977566726505756378173828125f,1.813799927783012577720000990666449069976806640625f,0.147299993228912295961663403431884944438934326171875f,0.8449000530898571614812908592284657061100006103515625f,-0.6488999748289578928250875833327881991863250732421875f,-0.799199977013468565445464264485053718090057373046875f,0.5329999910742044821887475336552597582340240478515625f,-0.6152000312834982054255306138657033443450927734375f,-0.530199974286556408031856335583142936229705810546875f,0.3311000540792949919932652846910059452056884765625f,-1.3303999585568906383059584186412394046783447265625f,-0.256999974429607647152806748636066913604736328125f,-0.51829996956586921186271865735761821269989013671875f,-0.8148999934852128035345231182873249053955078125f,0.72889999300837526874374816543422639369964599609375f,0.82319998689293871763084098347462713718414306640625f,-0.09849998738244136742281398255727253854274749755859375f,-0.082699978309869859316449947073124349117279052734375f,-0.0175999990642065895496415350862662307918071746826171875f,-0.62089999560117703492068130799452774226665496826171875f,0.93880002233087989083060165285132825374603271484375f,-0.2769000129908321117255809440393932163715362548828125f,-0.819200010969490133305725976242683827877044677734375f,0.712899991041421987603143861633725464344024658203125f,-0.85280002477839567109896279362146742641925811767578125f,-0.44720000504255352780091925524175167083740234375f,-0.57200006778538270424405709491111338138580322265625f,-0.012700021374225656245471327565610408782958984375f,-0.3738000101864333846179988540825434029102325439453125f,0.9489000289201729998467271798290312290191650390625f,-0.0720999829560520222315744831576012074947357177734375f,0.168500006735325058571106637828052043914794921875f,-0.52659998271763353017149711376987397670745849609375f,-0.456499984100461109193247466464526951313018798828125f,-0.5982000161737206500589536517509259283542633056640625f,-0.096100002074241075433747028000652790069580078125f,0.43289997949302261304183048196136951446533203125f,0.362999955534935025980303180404007434844970703125f,-0.1057000117033723984860671407659538090229034423828125f,-0.9910999989449980063227485516108572483062744140625f,-0.03499999359250072217264460050500929355621337890625f,-0.20399997717142159814329716027714312076568603515625f,-0.30519999589323987265743198804557323455810546875f,-0.619000007197260782021430713939480483531951904296875f,0.09409999590516104905901784150046296417713165283203125f,-0.9563000157743688500744383418350480496883392333984375f,-0.631699966955185043815390599775128066539764404296875f,-0.0638999545633789356458009933703579008579254150390625f,0.0853000083684918308790656737983226776123046875f,0.415500014245509419197333045303821563720703125f,-0.540800001528859208832500371499918401241302490234375f,-0.060500008136033844863277408876456320285797119140625f,0.384100041186064722342763388951425440609455108642578125f,-0.9651000199258330525253768428228795528411865234375f,-1.0136999926261598403698371839709579944610595703125f,0.9837000164121392220550887941499240696430206298828125f,1.0166000222243367634433752755285240709781646728515625f,0.047500004842877530109035433270037174224853515625f,0.409600025666504830024194916404667310416698455810546875f,-0.32590000707507105204285835498012602329254150390625f,-1.3435999942883842805940730613656342029571533203125f,0.78249999657273239250798724242486059665679931640625f,-0.0011000309079884917906611008220352232456207275390625f,0.1948000245213510339681306504644453525543212890625f,-0.4704999796003097589647268250701017677783966064453125f,-1.82179995305538255934152402915060520172119140625f,0.561600035840272848730592158972285687923431396484375f,-1.004099995122849531270503575797192752361297607421875f,-0.0789999889805914623508442673482932150363922119140625f,0.2625999978348609875666852531139738857746124267578125f,-0.35989998163878933468140530749224126338958740234375f,-1.1182999409556391157138932612724602222442626953125f,1.1610999766677618794119553058408200740814208984375f,-0.441200030493735795289467205293476581573486328125f,-0.619700056004524046926462688134051859378814697265625f,-0.6881999791443345060315550654195249080657958984375f,0.999899966067075762765625768224708735942840576171875f,-0.739500043272972806818188473698683083057403564453125f,-0.57940000859499019014720033737830817699432373046875f,0.1869999916031954700912365296971984207630157470703125f,0.14220000239759700288999511030851863324642181396484375f,-0.3053000024452803273788958904333412647247314453125f,0.899699990886449807447888815659098327159881591796875f,-0.296800002527236816973754685022868216037750244140625f,-0.30550000426173173906363444984890520572662353515625f,0.698599997007846695851185359060764312744140625f,-0.13049999770522102693348642787896096706390380859375f,0.239799989801645185849565677926875650882720947265625f,1.11960004138723068223271184251643717288970947265625f,-0.05460003309845973262781626544892787933349609375f,0.19469999153837524996646379804587922990322113037109375f,-0.20389997617006283547880229889415204524993896484375f,0.26940003442987847659395583832520060241222381591796875f,-0.118900024747848664929961159941740334033966064453125f,0.00560000222697853988762517474242486059665679931640625f,0.389900000637769750255756662227213382720947265625f,-0.424900015874207170885057394116302020847797393798828125f,-0.3694999920129777848387675476260483264923095703125f,-0.165000010896474369470610099597251974046230316162109375f,0.99799999196827382519359161960892379283905029296875f,-0.5382000160247086828491092092008329927921295166015625f,1.595899951121211746141170806367881596088409423828125f,-0.316800001260638008915293539757840335369110107421875f,0.34780003399848880007994011975824832916259765625f,-0.44799999077618135601142057566903531551361083984375f,0.02310000004768397730003925971686840057373046875f,-0.175099999362230196453538155765272676944732666015625f,0.58979999315440689855449818423949182033538818359375f,0.218399963042139766411509071986074559390544891357421875f,-0.297000008031725659662924954318441450595855712890625f,0.0644999897778035791162665191222913563251495361328125f,-1.279600014099478766382844696636311709880828857421875f,-0.9625999559253453075058359900140203535556793212890625f,-1.61680002346336859631037441431544721126556396484375f,0.15139996305704084278431764687411487102508544921875f,-0.86320000835061083677146598347462713718414306640625f,-0.86039998731613120952488316106610000133514404296875f,-1.2493000286042690305521318805404007434844970703125f,-0.4028999876141552416441982131800614297389984130859375f,-0.560799995419383190409234885009936988353729248046875f,0.5968999945133928175522441961220465600490570068359375f,0.355400023302436363081824310938827693462371826171875f,0.84859996824860584041516631259582936763763427734375f,0.6405000051558011886498888998175971210002899169921875f,0.78750000296160649337906534128705970942974090576171875f,-0.872100016688555523813874970073811709880828857421875f,0.11239998929426053830837872737902216613292694091796875f,-0.1423999808937306266187761139008216559886932373046875f,0.14330002401098596021000730615924112498760223388671875f,0.25030001462697981651217560283839702606201171875f,-0.6895000023692841040201528812758624553680419921875f,0.11919998617768268189820446423254907131195068359375f,-0.58360003435388219106272345015895552933216094970703125f,-0.54850000166893020381309042932116426527500152587890625f,0.9848000451594598469995389677933417260646820068359375f,0.939799986523390362691543487017042934894561767578125f,-1.03600003892183334386345450184307992458343505859375f,-0.23480001319646870427959584048949182033538818359375f,1.40810002992451277492591543705202639102935791015625f,-1.0777999951064582973003780352883040904998779296875f,-1.1305000365972517339940850433777086436748504638671875f,-1.3919999154955158271462778429850004613399505615234375f,2.0917999894142145222986073349602520465850830078125f,0.4212999721884731485488373436965048313140869140625f,-2.5383000166535367725373362191021442413330078125f,-0.0448999477386475831508505507372319698333740234375f,0.75439996687173760392397525720298290252685546875f,-0.901799985539913251386678894050419330596923828125f,-1.040799996201693833342005746089853346347808837890625f,0.02879999573230751508390312665142118930816650390625f,-1.8796000041157003845881945380824618041515350341796875f,0.29229995694458488220135450319503434002399444580078125f,0.4023000393480062886197856641956605017185211181640625f,0.37660005692541698163466890036943368613719940185546875f,-1.3802000220820307418989614234305918216705322265625f,-0.32110005311071920797161283189780078828334808349609375f,1.2097999809727075781751182148582302033901214599609375f,-0.22369997975528266298539392664679326117038726806640625f,-0.911500025972724170486571892979554831981658935546875f,1.234700015864520938890791512676514685153961181640625f,1.7354999766945837791354279033839702606201171875f,0.162400029993056749244573211399256251752376556396484375f,1.0771999210000036129031286691315472126007080078125f,-1.2574000237345701602720282608061097562313079833984375f,-0.12039995412379500550059674424119293689727783203125f,1.0017000314772135372010097853490151464939117431640625f,0.4625000007823116821015219102264381945133209228515625f,-0.4987999997369942928315822427975945174694061279296875f,-0.6825999396085737913608681992627680301666259765625f,0.8300000226311390338906903707538731396198272705078125f,1.142199957060813897413709128159098327159881591796875f,-0.82119998072385858023380933445878326892852783203125f,0.78229996849298544248085818253457546234130859375f,-1.60469996004104675790813416824676096439361572265625f,-0.1426000526711350691044799532392062246799468994140625f,0.4307000232517721460823167944909073412418365478515625f,1.095000083930791401343185498262755572795867919921875f,-0.7372000317528846213832594003179110586643218994140625f,-0.47140002996325536255284305298118852078914642333984375f,0.71160000600069761578225779885542578995227813720703125f,-0.3757999666228883928198456487734802067279815673828125f,-1.1040999802589424039211962735862471163272857666015625f,0.13639996886849414803322133593610487878322601318359375f,1.39220004300326127832931888406164944171905517578125f,-0.286400017446279686605947745192679576575756072998046875f,0.15080001069307347183467982176807709038257598876953125f,-0.06359998549669965139941041343263350427150726318359375f,-0.7675000137090683960394699170137755572795867919921875f,-0.79150002876669256490771431344910524785518646240234375f,0.2055000246018174880902051882003434002399444580078125f,-0.815299994398653637972529395483434200286865234375f,0.936500004068017144476243629469536244869232177734375f,-0.426899974918364844000961966230534017086029052734375f,-0.8527000195860860998209318495355546474456787109375f,-0.1849000275716192742692101091961376368999481201171875f,-0.32980001058876540209752192822634242475032806396484375f,0.3047000047817822743212445857352577149868011474609375f,-0.149200009013712442396126789390109479427337646484375f,-0.048199985179305127047655332717113196849822998046875f,0.70100001284480095709028546480112709105014801025390625f,0.90110002122223331610939567326568067073822021484375f,-0.5335000166818499867105174416792578995227813720703125f,1.45109998985528942938572072307579219341278076171875f,-0.05049999288097029026545214946963824331760406494140625f,-0.194199953246116052696379483677446842193603515625f,1.09470003834664897368611491401679813861846923828125f,-0.788500011801719580262215458787977695465087890625f,-0.526500010326504064295249918359331786632537841796875f,-0.19069998413622368804709594769519753754138946533203125f,0.07929999347478167148750571868731640279293060302734375f,0.2607999887883656331410975326434709131717681884765625f,-0.8365000447481867329457827509031631052494049072265625f,-0.0958000183671714466981939040124416351318359375f,-1.2047999863743772408497534343041479587554931640625f,1.25560001994073378028815568541176617145538330078125f,0.7604000021427872457735475109075196087360382080078125f,0.02820003710985208300598969799466431140899658203125f,-0.28590003970861488369337166659533977508544921875f,-2.1477001015424743712856070487760007381439208984375f,-0.0583000087931753174785853843786753714084625244140625f,-1.6123999890148634772657487701508216559886932373046875f,-1.362900055861473447293974459171295166015625f,-0.9248000046283006980729624046944081783294677734375f,0.51820001267194726324305520392954349517822265625f,-0.7815000016838309360167613704106770455837249755859375f,-0.579200027751922252861049855710007250308990478515625f,0.93990001289397451245122283580712974071502685546875f,0.1977000087082387835835106670856475830078125f,1.653900050936639498644353807321749627590179443359375f,0.6063000341773037149550873436965048313140869140625f,-0.10390000466853333005445847447845153510570526123046875f,0.798100017295777508508081155014224350452423095703125f,-0.72590001556873318122597993351519107818603515625f,0.19499999644234755624694344078307040035724639892578125f,0.26969998924732152545402641408145427703857421875f,0.8384000050447875107550999018712900578975677490234375f,-0.2877000315070148417362361215054988861083984375f,-0.5685000044815244901741380090243183076381683349609375f,-0.7293000326275826949995462200604379177093505859375f,-0.938500033780932607641034337575547397136688232421875f,-0.3492000042080878330352788907475769519805908203125f,0.564300016776472457280533490120433270931243896484375f,-0.3597000053405761121894101961515843868255615234375f,0.6321999768421047161126580249401740729808807373046875f,-0.47900000280141785680143584613688290119171142578125f,0.7273999870777121401488329865969717502593994140625f,1.616600048208237438984724576584994792938232421875f,-0.9363000079512602269460330717265605926513671875f,-1.63530004368424375371660062228329479694366455078125f,-0.76509997864961665214877939433790743350982666015625f,-0.963700041222572689747494223411194980144500732421875f,0.852000036269426885837674490176141262054443359375f,0.77250002592802058387633223901502788066864013671875f,0.36410000022649757767112532746978104114532470703125f,-0.7185999984979627441816774080507457256317138671875f,1.090600020816177018190273884101770818233489990234375f,-0.77509998926669421148716310199233703315258026123046875f,-1.33230004270076829442359667154960334300994873046875f,-0.6353999939098946558857505806372500956058502197265625f,-0.4171000211775297561445086103049106895923614501953125f,0.6794000425323840630653648986481130123138427734375f,0.76120005291998449337143028969876468181610107421875f,-0.14040001085996534069266772348782978951930999755859375f,0.0455999768018722217988170086755417287349700927734375f,0.49259998665899029646197959664277732372283935546875f,1.151799986955522658860218143672682344913482666015625f,-0.7783999479547147881675073222140781581401824951171875f,0.682600045797974619432579856947995722293853759765625f,0.8864000321984288888188530108891427516937255859375f,0.989300001651792992873879484250210225582122802734375f,0.064199950265884098143942537717521190643310546875f,-1.3756000177055600630637854919768869876861572265625f,0.56049998816847779181671285186894237995147705078125f,0.1270000142529605913210843937122263014316558837890625f,-0.077700012433528087285594665445387363433837890625f,-0.83079997902810520660210613641538657248020172119140625f,1.09099996829032885869992242078296840190887451171875f,0.502999999716877255195868201553821563720703125f,-0.262299991738796478557560476474463939666748046875f,-0.22190001479759813296510628788382746279239654541015625f,-0.01080008072853200218332858639769256114959716796875f,-0.73749997645616449659655700088478624820709228515625f,-1.845500063195824491657504040631465613842010498046875f,-0.773499921798706413511581558850593864917755126953125f,1.3931000124156482922188615702907554805278778076171875f,-0.347299976614117600348663472686894237995147705078125f,-0.851800017651914931349210746702738106250762939453125f,-0.51769994373917693764042269322089850902557373046875f,1.4966999913930878474133123745559714734554290771484375f,0.590599975833297374805397339514456689357757568359375f,-0.64239998260736452806440865970216691493988037109375f,0.5640000296607616814270613758708350360393524169921875f,0.63419995998889355792726973959361203014850616455078125f,0.68679999462962104672669738647527992725372314453125f,0.0336999714478845024956399356597103178501129150390625f,-0.4377000088572504665762608055956661701202392578125f,0.912700037728249480295517059857957065105438232421875f,0.2728000578403477760502937599085271358489990234375f,-1.01049998842924804165477326023392379283905029296875f,-0.328600013250112343854425489553250372409820556640625f,-0.2743000134423374536396522671566344797611236572265625f,0.2829999961405993413876558406627736985683441162109375f,0.84130003890842264269167571910656988620758056640625f,-0.6445000260621311927167198518873192369937896728515625f,-0.00869999890327477487517171539366245269775390625f,-0.028599985068291200462908818735741078853607177734375f,-0.3414000462055213347412063740193843841552734375f,-0.7727000430740422753927987287170253694057464599609375f,-0.532499994337559456880626385100185871124267578125f,1.05609999905675611131528057740069925785064697265625f,0.70099999824166303596939542330801486968994140625f,0.19010001547411070621507178657338954508304595947265625f,0.4149999842792748694364490802399814128875732421875f,-0.60610001138746760762643361886148341000080108642578125f,-0.610299998347461070125063997693359851837158203125f,-0.17649998316913728668708927216357551515102386474609375f,-0.26659996103644356679751581395976245403289794921875f,-0.822899953788520210906654028804041445255279541015625f,-0.9422000270962715973155354731716215610504150390625f,1.70369997774362591513863662839867174625396728515625f,1.32100005084276261868581059388816356658935546875f,-0.2539000393509862618657280108891427516937255859375f,0.0878999847829344815863805706612765789031982421875f,-0.90899998313188579146526535623706877231597900390625f,-0.709400020739435976935283179045654833316802978515625f,0.141800046932697210166907098027877509593963623046875f,-0.2830000218823551794145032545202411711215972900390625f,-1.101700017693639122029480859055183827877044677734375f,0.3598000128611922310284398918156512081623077392578125f,0.62300002855062519557804989744909107685089111328125f,0.16910001292973764197569153111544437706470489501953125f,0.833400042092800941162522576632909476757049560546875f,-0.4919999911561607763843539942172355949878692626953125f,0.421099982255697113942005671560764312744140625f,1.127399982011317636221292559639550745487213134765625f,0.81930002994537343141701057902537286281585693359375f,-0.77439996083676720939337201343732886016368865966796875f,-1.20190004931241301022737388848327100276947021484375f,-0.469600029708445332321531395791680552065372467041015625f,-0.45500003669411004381117891171015799045562744140625f,0.07939992514848626115053775720298290252685546875f,-0.80820002135187463121468454119167290627956390380859375f,0.0878999739050865258604972041212022304534912109375f,-0.6922999940484759751058163601555861532688140869140625f,0.0301999796509750950690431636758148670196533203125f,-0.4201999896161254799409334736992605030536651611328125f,0.0068000362038613815229837200604379177093505859375f,0.3823999997623264146540122965234331786632537841796875f,0.749299996156990832929523094207979738712310791015625f,0.351800035849958891986943854135461151599884033203125f,0.374700002658367115060400465154089033603668212890625f,-0.60819995049685238175385393333272077143192291259765625f,0.412699974882603015657878131605684757232666015625f,-0.5381999709486962046867120079696178436279296875f,0.4483000239551078180966214858926832675933837890625f,-0.483099982166290686791398911736905574798583984375f,0.755999991238117008407471075770445168018341064453125f,0.12829998849034307539795918273739516735076904296875f,-0.12370000009536763485584742738865315914154052734375f,-1.07949996280670124093603590154089033603668212890625f,0.137499972768127864952703021117486059665679931640625f,0.534200005847214942633627288159914314746856689453125f,0.10080000586882242874509074681554920971393585205078125f,-0.51049999338388385439202465931884944438934326171875f,-0.92040004267394615045105865647201426327228546142578125f,1.13079999966621347340378633816726505756378173828125f,-0.4455000173747538649848820568877272307872772216796875f,1.1195000146627425152701107435859739780426025390625f,1.6771000213637947684475193455000407993793487548828125f,0.65050002735853240665164776146411895751953125f,-0.1116999665454032619749114019214175641536712646484375f,-0.7938000050455331635390621158876456320285797119140625f,0.60190000676959820591349625829025171697139739990234375f,1.4208000387072570180890807023388333618640899658203125f,0.8559999707862731010976631296216510236263275146484375f,-1.4146000140249725962604543383349664509296417236328125f,0.75939999071359576277018277323804795742034912109375f,0.20620000337362309750233180238865315914154052734375f,0.7418999515235429331738714608945883810520172119140625f,0.832399982830882123607807443477213382720947265625f,0.1940000112205744198234924624557606875896453857421875f,1.7062999938696616908373471233062446117401123046875f,-0.1394999991655350157770953956060111522674560546875f,-0.5778000177562230277317212312482297420501708984375f,-0.0152000176489348604036422329954802989959716796875f,0.58100000941753382477372724679298698902130126953125f,0.7245000003576276270678135915659368038177490234375f,0.19890000001192031930941084283404052257537841796875f,0.025199982556701083780126282363198697566986083984375f,0.2870000189840791193773839040659368038177490234375f,1.275499999549239316110060826758854091167449951171875f,-0.410900021827220685022297175237326882779598236083984375f,-0.6358000054590409177990295575000345706939697265625f,0.55440000721663229565905339768505655229091644287109375f,1.11490004103854278838525715400464832782745361328125f,-0.1623000077575442734456601101555861532688140869140625f,-1.3246999927490943349539520568214356899261474609375f,1.0050000144541260116426428794511593878269195556640625f,1.695900034604967299145528158987872302532196044921875f,0.7884000003695490477184648625552654266357421875f,1.503799978074431553665135652408935129642486572265625f,0.3072999673008922627559513784945011138916015625f,-0.8475000018253926459266267556813545525074005126953125f,-1.137100008213519419797421505791135132312774658203125f,0.7606000158801677901720950103481300175189971923828125f,-0.0244999908208856087554750047274865210056304931640625f,-0.60369998500794197493490855777054093778133392333984375f,-0.3211999569192529602190688819973729550838470458984375f,0.89229998857229997355489103938452899456024169921875f,0.915999999731778924427771926275454461574554443359375f,-1.0608000140085815932167179198586381971836090087890625f,-0.071300035816431517332603107206523418426513671875f,0.84029997260570521433464818983338773250579833984375f,1.575799969267845046516640650224871933460235595703125f,1.1400000210106371323348639634787105023860931396484375f,0.56770000002831277896575556951574981212615966796875f,-0.648600049757957375362593666068278253078460693359375f,-0.422400024852156885657450402504764497280120849609375f,0.55140001167207974841488749007112346589565277099609375f,-0.22110001090317954464126160019077360630035400390625f,-0.990200010794401119795793420053087174892425537109375f,0.250999990083276447450089108315296471118927001953125f,0.400100027972460026148837641812860965728759765625f,-0.2860000039786101400096640645642764866352081298828125f,0.2303999995164572045514006504163262434303760528564453125f,0.242299997624754848146721997181884944438934326171875f,-0.39310002303272517654164630585000850260257720947265625f,0.031200017827749615406673910911194980144500732421875f,-0.1559000166118149621752309030853211879730224609375f,0.4041000181078910902243706004810519516468048095703125f,1.17290002177506647029758823919110000133514404296875f,-0.3617000245481729603369558390113525092601776123046875f,0.026100019322335576976712445684825070202350616455078125f,0.567300006341934448528263601474463939666748046875f,-0.3473000237390404532078491683932952582836151123046875f,-0.631900061978399651962945426930673420429229736328125f,-0.2528999999448655611189451519749127328395843505859375f,0.1205999722942709961870377810555510222911834716796875f,1.3042000241756437350915120987338013947010040283203125f,0.305999985799193741087265152600593864917755126953125f,0.5167999866575001544077849757741205394268035888671875f,-0.329500019133091459622164620668627321720123291015625f,-0.53729999158978480888748663346632383763790130615234375f,-0.01030000017285370716990655637346208095550537109375f,0.875300033104419838991816504858434200286865234375f,0.5728000295281407971970111248083412647247314453125f,-0.385200021597743802459490325418300926685333251953125f,0.6344000233098865404457455952069722115993499755859375f,-0.18759998494535701230034874242846854031085968017578125f,-0.0005999872699380404128532973118126392364501953125f,0.4971000021785492162251784975524060428142547607421875f,0.3987000272899867692188990986323915421962738037109375f,0.61869999920129803427926162839867174625396728515625f,1.5050000412017110473783532142988406121730804443359375f,-0.104799997773766140340967467636801302433013916015625f,-0.64570002675354454257927727667265571653842926025390625f,-0.225800020378828403266879831789992749691009521484375f,-0.62950002241134672686939666164107620716094970703125f,0.42260002603530910647577911731787025928497314453125f,0.35239999985545844563006312455399893224239349365234375f,0.94190006305873463787747823516838252544403076171875f,-0.28920001910924941768143980880267918109893798828125f,-0.1281999599032095371597961275256238877773284912109375f,0.61089999083280499547754516243003308773040771484375f,-0.251000023908913350023652810705243609845638275146484375f,0.9841000271975997026885352170211263000965118408203125f,-0.1399000229895119573608752716609160415828227996826171875f,-0.512700007963180492964738732553087174892425537109375f,-0.55890002733692567549184104791493155062198638916015625f,-0.273400007075071282969247477012686431407928466796875f,0.5395000232309106191763703463948331773281097412109375f,0.4407999806299807143972202538861893117427825927734375f,-0.028999996393918703319059204659424722194671630859375f,-0.8974999734759325775712568429298698902130126953125f,0.7668999927997592891415479243732988834381103515625f,-1.14480003376007033466521534137427806854248046875f,-0.035600014799833434864240189199335873126983642578125f,-0.2090999945640561463733320124447345733642578125f,-1.165100029835105299724773431080393493175506591796875f,-0.09069997944235819176128643448464572429656982421875f,-0.0660000023394824442135586650692857801914215087890625f,0.66690001246929142553199199028313159942626953125f,-0.7998000000089404704084472541580908000469207763671875f,-1.3204000575378540549564831962925381958484649658203125f,1.017699989336728894073758056038059294223785400390625f,-0.89650005629658746642007827176712453365325927734375f,-0.0187000325798987176995069603435695171356201171875f,-0.3868000646650795459891014616005122661590576171875f,0.160099985504150055248828721232712268829345703125f,0.95750000953674341275245751603506505489349365234375f,-0.1390999905407424019898598999134264886379241943359375f,0.059900025671721035536165800294838845729827880859375f,-0.96160001915097215174910161294974386692047119140625f,-0.72030001268982868367629635031335055828094482421875f,0.208300018888711857556472750729881227016448974609375f,0.214200016035884954757051445994875393807888031005859375f,0.2967000253118576846844689498539082705974578857421875f,0.16900001502037031997360827517695724964141845703125f,-0.221700034271181178890941509962431155145168304443359375f,0.37759999414682388074737673377967439591884613037109375f,-0.316800024394691182028083176192012615501880645751953125f,-0.6185999981254342028336168368696235120296478271484375f,-0.319000002112239278151406551842228509485721588134765625f,0.22029999883174899988347306134528480470180511474609375f,0.44719999146386990585000376086100004613399505615234375f,-0.2075000116974112795986684432136826217174530029296875f,1.139400002187489047145163567620329558849334716796875f,1.02390001642182415508841586415655910968780517578125f,1.7152999635905021857951169295120052993297576904296875f,1.4213000133156779014598214416764676570892333984375f,-0.224300036054849716293801975552923977375030517578125f,-0.33439999339580506187985520227812230587005615234375f,0.457799978268146556814599534845910966396331787109375f,-0.095099997648597334176656659110449254512786865234375f,-0.2191999998122451387416731449775397777557373046875f,-0.42999995768070231605406661401502788066864013671875f,-0.252799984228611496206440278911031782627105712890625f,0.744699951472879373426394522539339959621429443359375f,-0.659299993512034721021564109832979738712310791015625f,-0.29460000219717630276505815345444716513156890869140625f,0.5943999890744695591138224699534475803375244140625f,0.4349999924004084306261574965901672840118408203125f,0.806500007271766516936395419179461896419525146484375f,1.87360001883804816458223285735584795475006103515625f,0.18509995689392144679885632285731844604015350341796875f,0.35219997412264358871425429242663085460662841796875f,0.347599993252754035921725517255254089832305908203125f,1.0720999837011093713812215355574153363704681396484375f,-0.91829996107220657197700575125054456293582916259765625f,-0.882499992027878210620883692172355949878692626953125f,0.010699986445903864051842901972122490406036376953125f,-0.58300000362843296120018976580468006432056427001953125f,-0.8223999947145574918039301337557844817638397216796875f,0.999399971640109630044435107265599071979522705078125f,0.911699992976338080552523024380207061767578125f,-1.00420000331401837456724024377763271331787109375f,-0.65300001281127340035936867934651672840118408203125f,-0.064799992334843015129308696486987173557281494140625f,0.106600002759695000609241333222598768770694732666015625f,0.4541000084221362698144730529747903347015380859375f,0.265400003297626974063661009495262987911701202392578125f,0.346099993729591215441132590058259665966033935546875f,-0.82110002887770539015122039927518926560878753662109375f,-1.074199994149804826548688652110286056995391845703125f,-0.040799997058510228953309706412255764007568359375f,0.2538999675273903022798549500294029712677001953125f,-1.23050001223385319093495127162896096706390380859375f,0.1726999388217933528721914626657962799072265625f,0.65749999023973959566546909627504646778106689453125f,-0.08920003401041043389341211877763271331787109375f,0.8390999916583299356176439687260426580905914306640625f,0.0048000047028065040422006859444081783294677734375f,-0.63470002687275417496692853092099539935588836669921875f,0.20840000059306618140908540226519107818603515625f,1.1097999961346387554073089631856419146060943603515625f,0.6572000052288180516057991553680039942264556884765625f,-0.9783999622225767467398327426053583621978759765625f,0.4747999875664701274757817373028956353664398193359375f,0.358200050073861575583578087389469146728515625f,-0.590199969369173071953582621063105762004852294921875f,-0.4879000358939169501582000521011650562286376953125f,-0.774500017642974825093915569595992565155029296875f,0.48659997638463980962342247948981821537017822265625f,0.689599979305267396512135746888816356658935546875f,-0.2628999938726421792267728960723616182804107666015625f,0.323399995526671535372997823287732899188995361328125f,0.2394999835193154513746094380621798336505889892578125f,0.1591999977260822873859069659374654293060302734375f,-0.48920004945173867216823282433324493467807769775390625f,-0.5855000020265581639478114084340631961822509765625f,-0.40540002998933222233546302959439344704151153564453125f,-0.2069999969303608633452995491097681224346160888671875f,0.24810000578463065057377434641239233314990997314453125f,-0.587600001150369255498162601725198328495025634765625f,-1.165500010799616514844956327578984200954437255859375f,1.33870001887083045488679999834857881069183349609375f,1.001900014983862607920173104503192007541656494140625f,-0.98710001596212382235506765937316231429576873779296875f,-0.202500000838190252760995235803420655429363250732421875f,0.0482999800339347640232290359563194215297698974609375f,-0.015400027772784152801932577858678996562957763671875f,-0.961700048315524469444426358677446842193603515625f,-0.2334000023812052848626308332313783466815948486328125f,-1.041600000822544558332083397544920444488525390625f,-0.267099970886111481860325511661358177661895751953125f,-1.30999995276331926419288720353506505489349365234375f,0.2250999716460702071429977877414785325527191162109375f,0.7857000055193896770333594758994877338409423828125f,0.7844000129908323959426752480794675648212432861328125f,0.8728000266969206943912240603822283446788787841796875f,0.050100000312179371331211541473749093711376190185546875f,1.09680002148896438285419208114035427570343017578125f,0.7394999972283837763598057790659368038177490234375f,1.13650000846385967889773382921703159809112548828125f,0.3428999730110167121210906771011650562286376953125f,0.7907999929606912559165721177123486995697021484375f,0.7241999828994274412963250142638571560382843017578125f,0.04419998841285721624672078178264200687408447265625f,-1.5347000142812721179552681860513985157012939453125f,0.3681000118196013914939612732268869876861572265625f,1.153000001505017024783228407613933086395263671875f,-0.05000003363937166600550199291319586336612701416015625f,-0.03840000025779011227911041714833118021488189697265625f,-0.4165999957561492550439652404747903347015380859375f,0.0186000139392914487501684561721049249172210693359375f,-0.62410001110434532023418796597979962825775146484375f,-0.152299972497671465276880553574301302433013916015625f,-0.2356000374495987870204771752469241619110107421875f,-1.6233000721976169611338036702363751828670501708984375f,1.09850003862381040420359568088315427303314208984375f,0.40680000461339915762692953649093396961688995361328125f,-1.132500023692846635725572923547588288784027099609375f,-0.4952000359959904951523412819369696080684661865234375f,0.6555999985575677868610000587068498134613037109375f,-0.30709998656958259477534056713921017944812774658203125f,0.900799981784821124364270872320048511028289794921875f,-0.2309999594986431592502640341990627348423004150390625f,-1.038499963298440409431577791110612452030181884765625f,0.8224999792873859405517578125f,0.618500006884336261947510138270445168018341064453125f,-0.52449995401501592340309798601083457469940185546875f,-0.820199995744228971261691185645759105682373046875f,0.19539997435212086429601185955107212066650390625f,1.343899954265356466720504613476805388927459716796875f,0.9261000643610959315310537931509315967559814453125f,-1.57370001946687665395074873231351375579833984375f,-0.51650003930926313699956153868697583675384521484375f,0.54320003688633533567298172783921472728252410888671875f,0.31469995523542115822834830396459437906742095947265625f,1.7999000238090749892450048719183541834354400634765625f,0.84650000522285717696746587535017170011997222900390625f,-0.8902000388085846704910863991244696080684661865234375f,1.70280007579624648172966772108338773250579833984375f,0.60450004833936787917991750873625278472900390625f,1.676500018522144586086142226122319698333740234375f,-0.591400002098083632517955265939235687255859375f,0.97490001404881443392014261917211115360260009765625f,0.975600041472912238305070786736905574798583984375f,-0.31169999157935379674455589338322170078754425048828125f,-0.8496999969959253551365918610827066004276275634765625f,0.88360001094043294500579577288590371608734130859375f,-0.2884999981112776890057602940942160785198211669921875f,-1.286800034937262804390911696827970445156097412109375f,-0.58290004323273958419804330333136022090911865234375f,-0.9125000476837161755838678800500929355621337890625f,-0.025800017864257451805087839602492749691009521484375f,-1.9896000076174740200940505019389092922210693359375f,-0.016500013250857481494904277496971189975738525390625f,0.399199987816810875074224895797669887542724609375f,0.327100018160045191706331024761311709880828857421875f,-0.9578000199913976331345111248083412647247314453125f,-1.09150000768154864516645830008201301097869873046875f,1.017100019314884917775998474098742008209228515625f,-0.3126999759256836597387518850155174732208251953125f,-2.171200074005128044518642127513885498046875f,1.2380000430792572618798885741853155195713043212890625f,0.24150001978874247043904688325710594654083251953125f,1.84279999406337768874664106988348066806793212890625f,-1.29989998886585222948042428470216691493988037109375f,0.51800003123283300254797723027877509593963623046875f,1.13800004553794931183574590249918401241302490234375f,0.9897999595522879445752550964243710041046142578125f,-0.82460002728700754204282930004410445690155029296875f,-0.2762999905541538847586480187601409852504730224609375f,0.4140999944522978370997634556260891258716583251953125f,0.4017000017493963515136101705138571560382843017578125f,0.904600044721364238142768954276107251644134521484375f,-1.5114999879002566363084270051331259310245513916015625f,0.587599993364513739635413003270514309406280517578125f,0.9779999933838841030819821753539144992828369140625f,0.230199985723197553255658931448124349117279052734375f,0.59460001894235592345694385585375130176544189453125f,-1.276600020082294673073874946567229926586151123046875f,-0.64530004621744208037625867291353642940521240234375f,0.10879992428123930192640500536072067916393280029296875f,0.2230999916285274498051194314029999077320098876953125f,0.82469999304711816279223057790659368038177490234375f,-0.18269999494329092204480957661871798336505889892578125f,-1.582699975851177942587355573778040707111358642578125f,0.07479999158978467566072367844753898680210113525390625f,0.39599997186660829129323246888816356658935546875f,-0.76079997832030021864824220756418071687221527099609375f,1.17120002274513268503142171539366245269775390625f,0.45540003798007988589091610265313647687435150146484375f,1.20070000194311177210693131200969219207763671875f,0.13360002502202983354351317757391370832920074462890625f,0.5037000024795528219101470313034951686859130859375f,-1.003099994944035788790870356024242937564849853515625f,-0.054699997517466680818642998929135501384735107421875f,1.1154000072836876000792472041212022304534912109375f,-1.4680000480711454802218440818251110613346099853515625f,-0.2941999849110839004850959099712781608104705810546875f,0.465999976634979162781746708787977695465087890625f,-0.8159999381899840642518029198981821537017822265625f,1.2044000187277799085450169513933360576629638671875f,-0.9761999848961835368754691444337368011474609375f,0.16479995418786952399159417836926877498626708984375f,-1.2158000026464463871889165602624416351318359375f,-1.34590001527071034814753147657029330730438232421875f,-0.735100003087520548206157400272786617279052734375f,0.298699962283671016649577723001129925251007080078125f,0.2689000093251461276366853780928067862987518310546875f,-0.67720001815557484547980493516661226749420166015625f,0.80430000226646691796617005820735357701778411865234375f,0.4052999885499477539241297563421539962291717529296875f,0.53399999527633112972324624934117309749126434326171875f,-0.326100048640370232533314265310764312744140625f,0.086500003769993671909332988434471189975738525390625f,-0.05259999165087947403662838041782379150390625f,-0.0384000134825708272501287865452468395233154296875f,0.10520000665932893735288189418497495353221893310546875f,0.131099993245304080602409158018417656421661376953125f,0.24270001563727827686989257927052676677703857421875f,-0.3804999983571468735732423738227225840091705322265625f,0.703799957026542077898056959384121000766754150390625f,-1.4393999882921573174598961486481130123138427734375f,0.0297000102207067850912380890804342925548553466796875f,0.50550001648068432213989353840588591992855072021484375f,0.5133000061482189124717478989623486995697021484375f,-0.6933000024415552076106905587948858737945556640625f,0.2173999545931817323207724257372319698333740234375f,-0.465000002682209423454651187057606875896453857421875f,0.1775999929547307232269304222427308559417724609375f,-0.9122000151753422159117690171115100383758544921875f,0.19899998067319391026330777094699442386627197265625f,-1.61079994155168559899493629927746951580047607421875f,0.168099983544648612365790540934540331363677978515625f,-0.0052999773740773914454393889172933995723724365234375f,1.2815999982893462316724253469146788120269775390625f,-1.174099998401105349188355830847285687923431396484375f,-0.8455000157356256096363722463138401508331298828125f,-1.027599943333864729311244445852935314178466796875f,0.1824999534338711537628796577337197959423065185546875f,0.0712000091850757410583128148573450744152069091796875f,0.60140000269413018685327187995426356792449951171875f,-0.0644000205531718439289079469745047390460968017578125f,0.690099981108307058974560277420096099376678466796875f,0.92050003927946111303981524542905390262603759765625f,-0.9098999661415809736553228503908030688762664794921875f,-1.0963000218652183548329048790037631988525390625f,-0.4615000070109969154685813919059000909328460693359375f,1.2226000101655725149640829840791411697864532470703125f,0.87490002760887175536907989226165227591991424560546875f,-0.9850999983564019402138001169078052043914794921875f,-0.32939997018724653798926738090813159942626953125f,-0.2552000204429030194575034329318441450595855712890625f,0.19429997812658561517906719018355943262577056884765625f,-0.28410000691339387657308179768733680248260498046875f,0.236700009274482425780661287717521190643310546875f,0.3352999865755432740144215131294913589954376220703125f,0.9283000087372956787845623694011010229587554931640625f,0.0101000210247936816898572942591272294521331787109375f,0.77009997766241455874336452325223945081233978271484375f,-0.4837000142142178216175807392573915421962738037109375f,-1.280699957947433187399610687862150371074676513671875f,0.5992999965108933846380523391417227685451507568359375f,-0.014900024032592273925956760649569332599639892578125f,-0.0177999851971863964905651300796307623386383056640625f,-0.7911999837040895044992794282734394073486328125f,-0.27360002394169591344308400948648341000080108642578125f,0.0581999915122981548165626008994877338409423828125f,-1.79019999336004165257918430143035948276519775390625f,-0.1608000202111898868650996519136242568492889404296875f,0.2216000140473244339744951503234915435314178466796875f,0.5052000040516253509537136778817512094974517822265625f,0.64169998697862118408608012032345868647098541259765625f,-1.039799988721310963768473811796866357326507568359375f,0.3838000159978864900267581106163561344146728515625f,0.405199985574185905790045580943115055561065673828125f,0.35029998717159005483523515067645348608493804931640625f,-0.725899991354345797134328677202574908733367919921875f,0.342900000131130422431624538148753345012664794921875f,-0.7910999748423697131016751882270909845829010009765625f,-0.76710003086328537591498388792388141155242919921875f,-0.918399995601177021598005012492649257183074951171875f,-0.01790001771450011602837548707611858844757080078125f,-0.0031999559730297999493586758035235106945037841796875f,-0.410899985021352609493305862997658550739288330078125f,-1.04560001636445409900488812127150595188140869140625f,0.6329999670833348357490422131377272307872772216796875f,0.7631999838382002021575090111582539975643157958984375f,-0.974599987277388724038473810651339590549468994140625f,-1.2998999749332666997503338279784657061100006103515625f,0.309100017809868177209864370524883270263671875f,-0.16589998338222500251504243351519107818603515625f,0.8204999996609985135620490837027318775653839111328125f,-0.546800011467933444464506465010344982147216796875f,-0.39460001447200754309818648835062049329280853271484375f,-0.6475999764144422243816734408028423786163330078125f,0.9109999950006602009722200818941928446292877197265625f,0.711399978777766506254920386709272861480712890625f,-0.4411999859020110381635504381847567856311798095703125f,-1.10179995931386986995903498609550297260284423828125f,0.491999988101422702779785822713165543973445892333984375f,0.58709996798038588394774706102907657623291015625f,0.835299975138902528470907782320864498615264892578125f,0.4666000008225434925179797573946416378021240234375f,0.69779996277093925272083652089349925518035888671875f,-0.50889998302459726886581847793422639369964599609375f,0.5546999794125557770030354731716215610504150390625f,0.75020000274777398630021707504056394100189208984375f,-0.4481000185251229339655765215866267681121826171875f,0.107200026872753984008568295394070446491241455078125f,0.245600014799832511158683701069094240665435791015625f,-1.02689999972879864031938268453814089298248291015625f,-0.5326000097185372705865802345215342938899993896484375f,0.95049999040365218405668201739899814128875732421875f,0.064400039142370690825600831885822117328643798828125f,0.4310000582933428692200550358393229544162750244140625f,-0.1688000235974786988890627981163561344146728515625f,0.2786000155597927463446694673621095716953277587890625f,0.26579999251365649115541600622236728668212890625f,0.0651000219747428499061925322166644036769866943359375f,0.12780000211000430709873398882336914539337158203125f,0.204300010946392607280586162232793867588043212890625f,-0.2573999989986415215525994426570832729339599609375f,1.3825999736577274656923464135616086423397064208984375f,1.6681000209093088670897486736066639423370361328125f,-0.5232999789908532317639355824212543666362762451171875f,-0.80620002861246486958179957582615315914154052734375f,-0.1529000618219367169103861670009791851043701171875f,-0.20059998442009108199357569901621900498867034912109375f,-1.52660002675056460930136381648480892181396484375f,-0.4114000297956168505919549716054461896419525146484375f,-1.311500023066997755449847318232059478759765625f,0.2916999921567737796834762775688432157039642333984375f,-1.729100024366378107743003056384623050689697265625f,-0.7719000217452645085103313249419443309307098388671875f,0.917200001168250622640698566101491451263427734375f,0.13119999802783122166971452315920032560825347900390625f,-0.149999986886978575739703956060111522674560546875f,-0.5237999230146410667430245666764676570892333984375f,0.1842999710857871065172730595804750919342041015625f,2.028699984747170947940730911795981228351593017578125f,-0.13739998141527198782796403975225985050201416015625f,1.24339999887943264411660493351519107818603515625f,0.865800049436092677979104337282478809356689453125f,0.5114999691247934521243223571218550205230712890625f,-1.2009000411987305056982222595252096652984619140625f,-1.15050001245737032462557181133888661861419677734375f,1.1327000073671324997803822043351829051971435546875f,-0.591499984323977656686111004091799259185791015625f,-0.93060000615715932070770577411167323589324951171875f,-0.34579995909780192331339776501408778131008148193359375f,0.51440001530051271405596935437642969191074371337890625f,-0.66390001558363465061773922570864669978618621826171875f,0.89249994821846545534071992733515799045562744140625f,0.918400007224082770562745281495153903961181640625f,0.8304999995306119320304105713148601353168487548828125f,0.91030001611709554509843655978329479694366455078125f,-0.1811999584838737131775587840820662677288055419921875f,-0.4204000144362449731261222041212022304534912109375f,-1.4598000327169902146096092110383324325084686279296875f,-0.098799989324807935275885029113851487636566162109375f,-0.87439999823868264083870371905504725873470306396484375f,-0.393099989095329505062181851826608180999755859375f,-0.22970000981092442149389398764469660818576812744140625f,0.62419999489486155397344191442243754863739013671875f,0.92060001167058924664843289065174758434295654296875f,-0.0043000213027000189214277270366437733173370361328125f,-1.123199989426136635728425972047261893749237060546875f,0.307100016409158182995042807306163012981414794921875f,1.49410002570748279282497605890966951847076416015625f,-0.4309999846816061630505600987817160785198211669921875f,-0.96750002831220616172913651098497211933135986328125f,0.346300012570619486496070749126374721527099609375f,0.991199997115135289504905813373625278472900390625f,-0.169399988105892962408915991545654833316802978515625f,-0.188499990623444080295456615203875117003917694091796875f,0.512199999082088464064099753159098327159881591796875f,0.02699999741464853553196689972537569701671600341796875f,-0.1567000125527382436985135427676141262054443359375f,0.103099966408311427557009665179066359996795654296875f,-0.90840001750588417195331203402020037174224853515625f,-0.94940000071600050279840843359124846756458282470703125f,-0.071200029003620546319552886416204273700714111328125f,0.285599975404888350993815038236789405345916748046875f,-0.51619998936653122001416704733856022357940673828125f,0.80360001969486438699874497615383006632328033447265625f,-2.1239999827966098422393770306371152400970458984375f,-0.6187999975949527442509179309126920998096466064453125f,-0.31829999869763891950924517004750669002532958984375f,-1.115300043684244624131451928406022489070892333984375f,0.793400014302135137000959730357863008975982666015625f,-1.3827000228643420154384102715994231402873992919921875f,0.3197000065326687234801283921115100383758544921875f,-0.4988000189781185422788212235900573432445526123046875f,0.2395000113844876210578149766661226749420166015625f,-0.5879999832510947044283966533839702606201171875f,0.582699969816207641315486398525536060333251953125f,-1.1167999805480246511280029153567738831043243408203125f,1.265600008578598334452181006781756877899169921875f,-0.9737999964773649264060395580600015819072723388671875f,0.102099998900294419712508897646330296993255615234375f,-0.3992999662429090879101067912415601313114166259765625f,-0.415599978068470488068442136864177882671356201171875f,-0.97550002005696256901501328684389591217041015625f,-1.5958000117361539338389775366522371768951416015625f,-0.5960999807655813498286079266108572483062744140625f,-0.46660000514388144665645086206495761871337890625f,-0.889200008082390080943469001795165240764617919921875f,-1.08380000854730607073861392564140260219573974609375f,0.1328000280380248199207926518283784389495849609375f,1.0871000299319624193827849012450315058231353759765625f,0.735099991464614799241417131270281970500946044921875f,-0.388700012612342771234352767351083457469940185546875f,-0.91090002193897989801740777693339623510837554931640625f,-0.216599964314699189316115734982304275035858154296875f,-0.67600002087280131046753695045481435954570770263671875f,-0.232900008922815260348215815611183643341064453125f,-0.31100000515207593121402851465973071753978729248046875f,0.100900011545419499014997199992649257183074951171875f,0.226899994792789494457707633046084083616733551025390625f,0.14730002034902600627219726447947323322296142578125f,-0.278500027522444792982270200809580273926258087158203125f,0.385399987688660683460284417378716170787811279296875f,-0.752399999073147807138184361974708735942840576171875f,1.23269997823536403558364327182061970233917236328125f,0.6298999625653027845118003824609331786632537841796875f,0.350900054982303810646726560662500560283660888671875f,0.018900002470612520255599520169198513031005859375f,-0.605699970054626390947305480949580669403076171875f,0.99690000449717075525768450461328029632568359375f,0.5654999823570250327975372783839702606201171875f,1.3800000176578759436551990802399814128875732421875f,0.2437999885797506038898063707165420055389404296875f,-0.7628000100076197664833443923271261155605316162109375f,0.5305000250861049426731597122852690517902374267578125f,-0.600399967944622137139276674133725464344024658203125f,-0.50679999261796471188290524878539144992828369140625f,0.5251999523073440645504206258920021355152130126953125f,0.5218000140756371418859771438292227685451507568359375f,0.3018999989092350233477191068232059478759765625f,1.31499999135732625887840185896493494510650634765625f,-0.0715000118911266469012844027020037174224853515625f,0.68419998036622953208052422269247472286224365234375f,1.116100007587670805975221810513176023960113525390625f,1.30460000851154322987213163287378847599029541015625f,-0.46059994312524832338340274873189628124237060546875f,0.5087000230431559799626484164036810398101806640625f,-1.020300001439451431650695667485706508159637451171875f,0.1128999996840949204823800755548290908336639404296875f,0.3928999802194537149802044950774870812892913818359375f,-0.671699986922740688299882094725035130977630615234375f,-0.8609999946281308158546607955940999090671539306640625f,0.138100004741549486908525068429298698902130126953125f,-0.364500002201645967314647123203030787408351898193359375f,0.61020000811219177183630790750612504780292510986328125f,0.144599977945536528611825133339152671396732330322265625f,-0.757899991190433386378799696103669703006744384765625f,-0.71190001664161683958553794582257978618144989013671875f,-0.7598999453544621474065934307873249053955078125f,0.920499980494380309892221703194081783294677734375f,-0.468900005078315640361097393906675279140472412109375f,-0.73139998682439311128433701014728285372257232666015625f,-1.310699981267750668934013447142206132411956787109375f,0.9522000132754444567950713462778367102146148681640625f,-0.5846999886140225033415163125027902424335479736328125f,0.18589997421801129906526739432592876255512237548828125f,0.060799976867437433014629277749918401241302490234375f,0.05099999354779705118545507502858527004718780517578125f,-0.329300004389881717514754200237803161144256591796875f,0.6313999880164862243958623366779647767543792724609375f,0.4252000264406206042622216045856475830078125f,0.65160000514388105585794619400985538959503173828125f,1.7899000065237278267460396818933077156543731689453125f,1.040599986115097852490407603909261524677276611328125f,1.11759999597221604261676475289277732372283935546875f,0.06759998904317632284488581717596389353275299072265625f,0.45209998992234456949290688498876988887786865234375f,0.2205000114142894318547405418939888477325439453125f,1.4934000070750708122346850359463132917881011962890625f,0.82469999766647805738983834089594893157482147216796875f,0.54590000245571157488910785104962997138500213623046875f,0.24769997875690474842258481658063828945159912109375f,-0.34400000894814730667548019482637755572795867919921875f,-1.3569000453822319673946594775770790874958038330078125f,1.638400036877393883827380705042742192745208740234375f,0.90130000329464632358167364145629107952117919921875f,-0.8074000148311257785138650433509610593318939208984375f,0.68479999477490716142114024478360079228878021240234375f,0.16659999866187613637436015778803266584873199462890625f,-0.4406999780640006303400468823383562266826629638671875f,0.24149998760223356786269732765504159033298492431640625f,0.372199998690932953859800136342528276145458221435546875f,-0.67509999962300037967821708662086166441440582275390625f,-0.7682000025019048994323611623258329927921295166015625f,-1.777500002607702800361266781692393124103546142578125f,0.605099976116418414306963313720189034938812255859375f,0.3555999589204787980634137056767940521240234375f,0.7412000010639430147563189166248776018619537353515625f,0.786200014009326597630433752783574163913726806640625f,-1.153899990419299026456201318069361150264739990234375f,-0.41859999914988910152402468156651593744754791259765625f,0.091499978289008243592661528964526951313018798828125f,0.8545000364184376362430839435546658933162689208984375f,0.084200015570223218919210239619133062660694122314453125f,-0.420799971726536714555777507484890520572662353515625f,-0.32239999169707278525720539619214832782745361328125f,0.18239998465627405010991424205712974071502685546875f,-1.2678000131368634129103156737983226776123046875f,0.912400041317940502239025590824894607067108154296875f,1.2523000027656554511423792064306326210498809814453125f,0.4860999894268809295994060448720119893550872802734375f,-0.59889999768361423804208243382163345813751220703125f,-0.5166999879844478282819864034536294639110565185546875f,-0.595000008121132406557762806187383830547332763671875f,-0.72530001205652971929538352924282662570476531982421875f,0.6883999827489255540768908758764155209064483642578125f,-0.177900019316375168632049508232739754021167755126953125f,0.1004999880194663575139202293939888477325439453125f,-0.01610000445842718619360312004573643207550048828125f,0.76570004374086853005110242520458996295928955078125f,-0.277700012284517239180559045053087174892425537109375f,0.479100019691138967470322995723108761012554168701171875f,0.286299986046552934482178898178972303867340087890625f,-0.404899982886761711231571325697586871683597564697265625f,0.066900007104873537855382892303168773651123046875f,0.7018000236868857655281317420303821563720703125f,0.3651999647498131995604353505768813192844390869140625f,-0.8892000024944539671167831329512409865856170654296875f,0.4850000222772357094669359867111779749393463134765625f,1.0325999871805302543492643962963484227657318115234375f,0.39260001426339119490194207173772156238555908203125f,0.2801000125497576931365983909927308559417724609375f,-0.002299971771240194584606797434389591217041015625f,-0.9167999804735185165327493450604379177093505859375f,-1.0664999718070031775596362422220408916473388671875f,0.859299986881017385798031682497821748256683349609375f,-0.4548999642789368635931168682873249053955078125f,-0.72719997121691692942846430014469660818576812744140625f,-0.521799994033574687790633106487803161144256591796875f,0.65349998147785692825806336259120143949985504150390625f,0.63859998046755794831597086158581078052520751953125f,-0.219499942690134286493730542133562266826629638671875f,-1.04990001673102373302981504821218550205230712890625f,0.25999999105930360343563734204508364200592041015625f,-1.248700025278329572842039851821027696132659912109375f,0.007999999493360387958773571881465613842010498046875f,-0.62560000243186930646288601565174758434295654296875f,0.14599995025992473784981484641321003437042236328125f,-0.49490002120137166485847046715207397937774658203125f,-0.149799974453449724620668348507024347782135009765625f,-0.1654000237494710479069226494175381958484649658203125f,-0.264199998024106097460617093020118772983551025390625f,0.16930002316534487061261415874469093978404998779296875f,1.587999980419873669035268903826363384723663330078125f,1.098700002181529189471120844245888292789459228515625f,1.028599984509497744511463679373264312744140625f,-1.4887000018835063741562407813034951686859130859375f,0.100800032933056338269039997612708248198032379150390625f,-0.4402000364243985330858777160756289958953857421875f,-0.134199968985467765758556879518437199294567108154296875f,-0.4560000286400320845814349013380706310272216796875f,-1.34740004634708210318194687715731561183929443359375f,0.56410004343986575037206421256996691226959228515625f,-0.72390003251507917259033320078742690384387969970703125f,0.366899981474876568654508446343243122100830078125f,0.79199999603629056554865428552147932350635528564453125f,0.66809995832443203056527636363171041011810302734375f,-0.9877999910831454233317572288797236979007720947265625f,0.3494000336289406760670317453332245349884033203125f,-1.407499997317791073925263845012523233890533447265625f,-1.4689999569058418416034328402020037174224853515625f,-0.348599982181191148100651844288222491741180419921875f,-1.6473999882698056040908340946771204471588134765625f,-0.065099997574091350571734437835402786731719970703125f,-1.737099956059456218326886300928890705108642578125f,-0.001499991178512960487978489254601299762725830078125f,0.71549995791912124332156963646411895751953125f,-0.201000005468725984059119582525454461574554443359375f,1.056900010979175075220837243250571191310882568359375f,0.22749998524785031150940994848497211933135986328125f,-1.2479000305294987160920072710723616182804107666015625f,-0.172499999254941638326954489457421004772186279296875f,-1.27090002108216282294961274601519107818603515625f,0.18559998618960360516894070315174758434295654296875f,-1.17020004119277043486135880812071263790130615234375f,0.47659996193051323842837518895976245403289794921875f,1.126400044119358678784692529006861150264739990234375f,-1.2476999519348144218611196265555918216705322265625f,0.35490004206299818889647212927229702472686767578125f,1.04940000233650199135126968030817806720733642578125f,0.0197000210613011983440401309053413569927215576171875f,-0.5010999949216838889043401650269515812397003173828125f,-0.470800002124905692113543409504927694797515869140625f,-0.78900002600997642954183675101376138627529144287109375f,0.0861000052034857077387641766108572483062744140625f,0.52249999370425914690940771833993494510650634765625f,-1.00809999259710281194202252663671970367431640625f,-0.0081999758474531336815971371834166347980499267578125f,-0.038300034236907976037400658242404460906982421875f,0.48580003026947393696133303819806315004825592041015625f,0.419100001567601854191025267937220633029937744140625f,0.01890001316219580562716373606235720217227935791015625f,-0.11759996106624637235427144332788884639739990234375f,0.093200019377469356385290666366927325725555419921875f,0.763099987381696376331774445134215056896209716796875f,-0.635100020000338272296858121990226209163665771484375f,0.678599974060059007996414948138408362865447998046875f,-0.6407999901294709754751011132611893117427825927734375f,-0.6292999698191887869569427493843249976634979248046875f,0.3156000137567520624770622816868126392364501953125f,0.4164999655485157603607149212621152400970458984375f,-0.840600040094554135094995217514224350452423095703125f,-0.361999960571527612529507678118534386157989501953125f,0.654000030033290880027152525144629180431365966796875f,0.89620001410245908601837072637863457202911376953125f,-0.87189997145384567911463591372012160718441009521484375f,-0.907500019818544334526677630492486059665679931640625f,-1.213100023442506891768744026194326579570770263671875f,0.260899989789724173760987468995153903961181640625f,-0.396900022341311053963863741955719888210296630859375f,0.404600000911951251936216067406348884105682373046875f,0.801300039318203261018425109796226024627685546875f,-1.1822999996364114139169032569043338298797607421875f,-1.364099980910867149219711791374720633029937744140625f,0.112999987851828465057479888855596072971820831298828125f,-0.1061000373341141056471315096132457256317138671875f,-0.50320000529587283200072533873026259243488311767578125f,-0.070599977546930148264436866156756877899169921875f,0.6577000093415381609673886487144045531749725341796875f,0.56570001207590081548914895392954349517822265625f,0.016099996691197215881174997775815427303314208984375f,0.4714000067174435315564551274292171001434326171875f,-0.1543000031605361588304958786466158926486968994140625f,-0.7443999626994131091350936912931501865386962890625f,0.765699997156113365548435467644594609737396240234375f,1.2072000007398424603621833739452995359897613525390625f,-0.211499990303069040908923170718480832874774932861328125f,-0.5410999909728764212246687748120166361331939697265625f,-0.03310001643896054446258858661167323589324951171875f,0.40459999912381139353101389133371412754058837890625f,0.003699955093861007071609492413699626922607421875f,0.52990001196265179572719716816209256649017333984375f,-0.903200002688169245601557122427038848400115966796875f,-0.248099979335069775743249920196831226348876953125f,0.3371999817967417101272076251916587352752685546875f,-0.8411000445425518279307652846910059452056884765625f,-0.415700028020142386964153047301806509494781494140625f,1.1639000049479302223431886886828579008579254150390625f,0.1934000083416702597816083653015084564685821533203125f,-0.8041999864943323306221145685412921011447906494140625f,0.5444000524416570119257130500045605003833770751953125f,0.782199990327655303445908430148847401142120361328125f,-0.316599988007545629642436324502341449260711669921875f,0.4126999827429653766586170604568906128406524658203125f,-0.7811000168263912968313888995908200740814208984375f,1.516200008849799107935041320160962641239166259765625f,0.60070004426240952710713827400468289852142333984375f,-0.0533999467253689630297230905853211879730224609375f,-0.91050003987550720552235361537896096706390380859375f,2.41130006364435001131596436607651412487030029296875f,0.4755000577569017394807815435342490673065185546875f,-0.4046999575078482624945763745927251875400543212890625f,0.3385000120252374955498453346081078052520751953125f,0.849500027775764721127416123636066913604736328125f,0.80940001684650741964333064970560371875762939453125f,0.1280000213980674317326702293939888477325439453125f,-0.39170000508353097767866302092443220317363739013671875f,0.821300028812885329898563213646411895751953125f,0.521600016990304116148990942747332155704498291015625f,-0.3989000316023822989563996088691055774688720703125f,-0.40080002402961290020044771154061891138553619384765625f,-0.692999935865401539558661170303821563720703125f,1.258100043857098615518452788819558918476104736328125f,1.206200029301644605084220529533922672271728515625f,-0.5720000053867708711408113231300376355648040771484375f,0.07859997271895391435236888355575501918792724609375f,-0.0397000354409211553274872130714356899261474609375f,-1.679100044333935226603671253542415797710418701171875f,0.08830003200173397459593616076745092868804931640625f,0.363800014209747502036407240666449069976806640625f,0.58720000116825143976484469021670520305633544921875f,-1.18030007393360225620426717796362936496734619140625f,2.09920000599622813552969091688282787799835205078125f,0.0340000635981567711496609263122081756591796875f,-0.0852999872088435751038559828884899616241455078125f,0.233799960565566067316467524506151676177978515625f,-0.4574000163957474551779114335658960044384002685546875f,0.12549995147436787679140479667694307863712310791015625f,1.1727000065103165837854248820804059505462646484375f,0.6340000149086113534480091402656398713588714599609375f,0.96890004473403135154541132578742690384387969970703125f,1.03410000253617884169443641440011560916900634765625f,-1.80820003114938732124983289395458996295928955078125f,1.077400023399292638970337065984494984149932861328125f,0.068900032533705246606814398546703159809112548828125f,-0.9576000412717451393262990677612833678722381591796875f,1.0229999785944816625971043322351761162281036376953125f,1.375799983088672462372414884157478809356689453125f,-0.8664999967105677658452123068855144083499908447265625f,-0.84480002869367609719120082445442676544189453125f,-0.501700040231645605359744877205230295658111572265625f,0.5706000015377992173881693815928883850574493408203125f,0.670599995297938011873384311911650002002716064453125f,-1.6563999868988990016305251629091799259185791015625f,-0.2096999962322410038240150242927484214305877685546875f,0.081399988053739047444423704291693866252899169921875f,0.43120000732243013441546963804285041987895965576171875f,0.334299948883056696757876125047914683818817138671875f,-1.4000000078231094047254146062186919152736663818359375f,-1.191600014531612572454832843504846096038818359375f,0.7643999953702087513107699123793281614780426025390625f,0.50389999786987826890793940037838183343410491943359375f,-0.61610001228153643548779427874251268804073333740234375f,0.816900041601062110174780173110775649547576904296875f,0.51350000511482385423533969515119679272174835205078125f,-3.03400001591443935211600546608678996562957763671875f,0.62220004050731514411154421395622193813323974609375f,1.108699978302419619780039283796213567256927490234375f,0.020099991315603471520034872810356318950653076171875f,-0.02949996709078472889586919336579740047454833984375f,1.33150004649907227616267846315167844295501708984375f,0.5263000589132307283080081106163561344146728515625f,-0.32599999511241861682719900272786617279052734375f,1.080000038668514239503792850882746279239654541015625f,0.25009999712705610619423168827779591083526611328125f,-0.01650000801682482975962784621515311300754547119140625f,-1.47820002787113224940185318700969219207763671875f,0.36379999852627487033629449797444976866245269775390625f,1.057600045574456970598475891165435314178466796875f,-0.5040999990344048331536441764910705387592315673828125f,0.058400000071525515199510891761747188866138458251953125f,-0.20999996889382632758724867017008364200592041015625f,-0.952200019645690787228886620141565799713134765625f,-0.30180000353306457583357769181020557880401611328125f,-0.59940002100020672859415071798139251768589019775390625f,0.46939999905824691950328997336328029632568359375f,0.7974000385984780781001290961285121738910675048828125f,-1.1318999984622006849122044513933360576629638671875f,-0.13790003097653424912749642317066900432109832763671875f,0.4513999822609122691829952600528486073017120361328125f,-0.24300000297650703462437604684964753687381744384765625f,0.38100000295415503703821968883858062326908111572265625f,0.22330000400617711076023397254175506532192230224609375f,-0.893800023150443934127906686626374721527099609375f,-0.116400017483532958184611061369650997221469879150390625f,0.85119997029304528268767171539366245269775390625f,-0.03139994875192630985338837490417063236236572265625f,0.8163000317931174976138208876363933086395263671875f,1.19689998892545634845419044722802937030792236328125f,-1.54820004754066520291644337703473865985870361328125f,-0.02550000113248795940990021335892379283905029296875f,0.932100030770153242798414794378913938999176025390625f,-0.192200005452334732325425648014061152935028076171875f,0.59210001871883888302505738465697504580020904541015625f,0.294799996153265075893301627729670144617557525634765625f,-0.938800076496601576536704669706523418426513671875f,-1.31279998631477301529457690776325762271881103515625f,0.589300030243396832929647644050419330596923828125f,0.2767000062942501781293458407162688672542572021484375f,-1.59329998506307557448735678917728364467620849609375f,-1.370300052252412026376759968115948140621185302734375f,-1.20740000581592443751333121326752007007598876953125f,-0.23469996421337047110000639804638922214508056640625f,-0.62029999443590588814601005651638843119144439697265625f,0.538999988794326867491690791212022304534912109375f,-1.1117000275284050214708031489863060414791107177734375f,0.2409999659061430321571606327779591083526611328125f,0.6014000212088219665673705094377510249614715576171875f,-0.713299957644939031098374471184797585010528564453125f,-0.60950001626461702830539479691651649773120880126953125f,1.109100015947222406254013549187220633029937744140625f,0.19169998036622981629761852673254907131195068359375f,0.81870001035854167437122441697283647954463958740234375f,0.4831000223994248443659671465866267681121826171875f,-0.20040002002418066950895081390626728534698486328125f,1.5294999994635585238711428246460855007171630859375f,0.095100031511485827451224395190365612506866455078125f,0.84129998023509955373810953460633754730224609375f,-0.7175000221468508243560791015625f,-1.239599953079224547991543659009039402008056640625f,0.92479997717291106962278490755124948918819427490234375f,0.841799990904330996954740840010344982147216796875f,0.381499994605778880440993816591799259185791015625f,0.0032999820828434334174517061910592019557952880859375f,-0.577200056973099950852201800444163382053375244140625f,-0.19590001199245410390403776546008884906768798828125f,-1.74299999368190672299760990426875650882720947265625f,0.78489998856782872138637685566209256649017333984375f,-1.1330999791860580927504997816868126392364501953125f,-0.32580000383854024192942233639769256114959716796875f,1.3962999790430075108815799467265605926513671875f,-0.00709995531439755467317809234373271465301513671875f,0.19169995712041920654655768885277211666107177734375f,0.770200040745735226010992846568115055561065673828125f,0.6988000103540714658123533808975480496883392333984375f,-0.32629993422776515377137229734216816723346710205078125f,-0.867200011077522869840095154359005391597747802734375f,0.2643000028625135389148681497317738831043243408203125f,0.4994999991282822460192392099997960031032562255859375f,-0.407499950192868443110683074337430298328399658203125f,-0.30350001329183573517411787179298698902130126953125f,-1.0838000473275772872483457831549458205699920654296875f,0.3763999749779696202267587068490684032440185546875f,-0.3104999965503818515344391926191747188568115234375f,-0.4817000009298322282802473637275397777557373046875f,-0.2155999916657809212239271801081486046314239501953125f,0.09019999313652515837702594581060111522674560546875f,-0.30820000427141758247984171248390339314937591552734375f,0.56550000288337454179554697475396096706390380859375f,-0.335700000005960230708978997427038848400115966796875f,-0.32330000337287767564475871040485799312591552734375f,0.629699999082088623936215299181640148162841796875f,0.78410002704858783317831694148480892181396484375f,0.545700001645087695578695274889469146728515625f,0.098999996822327407386410413892008364200592041015625f,-0.3260000016689303947714506648480892181396484375f,0.61069997953548982305704839745885692536830902099609375f,-1.668400006031991011923309997655451297760009765625f,0.58829999549388911628255982577684335410594940185546875f,0.399399973912536954134822053674724884331226348876953125f,0.7255999708227818256744967584381811320781707763671875f,-0.659700002621114389711465264554135501384735107421875f,-0.53599995260685695530611383219365961849689483642578125f,1.2493000081896781239265692420303821563720703125f,0.619399979388713717298742267303168773651123046875f,0.193400050437450232720948406495153903961181640625f,0.29349999362230239086102301371283829212188720703125f,-0.24279997677803066835622303187847137451171875f,1.2914999130964286422340592253021895885467529296875f,-1.2160999964118008875857412931509315967559814453125f,0.2100999906450506937716227184864692389965057373046875f,0.49699999879300538196957859327085316181182861328125f,-0.9125999939233064850441223825328052043914794921875f,0.654399961246550088134199540945701301097869873046875f,0.31280000404715480044615105725824832916259765625f,0.36050002277642489989517571302712894976139068603515625f,0.8191000340148810732188167094136588275432586669921875f,-1.149900008684396457425691551179625093936920166015625f,-0.37610000842586155389568602913641370832920074462890625f,0.88489999810457309337152764783240854740142822265625f,-0.22239999575763957917473589986911974847316741943359375f,-1.3781000333145267955359258849057368934154510498046875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_3d_1item_large_5o3i_padded_uneven_filter_uneven_data_dilation_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{1,3,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3,4};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{1,5,17,23,17};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{1,1,1},        // move_strides
                                                                  Strides{1,1,1},        // filter_dilation
                                                                  CoordinateDiff{2,1,2}, // below_pads
                                                                  CoordinateDiff{1,2,3}, // above_pads
                                                                  Strides{2,3,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.5656999876379966707418134319595992565155029296875f,-0.0684999778643256007626405335031449794769287109375f,-1.710999955624342039328666942310519516468048095703125f,0.15210003194361920719757108599878847599029541015625f,-0.5616999671041968955620404813089407980442047119140625f,0.65789998672008509483788429861306212842464447021484375f,-0.812700007776915978041643029428087174892425537109375f,0.048600002223253113697865046560764312744140625f,-0.3836000059112907312197648934670723974704742431640625f,-0.63199999920278759901748344418592751026153564453125f,1.162599990868568244906100517255254089832305908203125f,-0.55250004932284380032569970353506505489349365234375f,2.106699999812245760466566935065202414989471435546875f,-0.4690000086873766793615914139081723988056182861328125f,0.77880001894086614555590131203643977642059326171875f,0.273900006866455214549205265939235687255859375f,0.02429999895095846795811667107045650482177734375f,0.0859999942183495846848018118180334568023681640625f,-0.79339996859282291286064037194591946899890899658203125f,-0.48729999565035075903551842202432453632354736328125f,-0.187100024232268236090703794616274535655975341796875f,-0.14130000441223378171429203575826250016689300537109375f,-1.266199983294308051284815519466064870357513427734375f,-0.60209999931007629481172216401319019496440887451171875f,-0.184999981224537257684659152801032178103923797607421875f,-0.6687999927148222667483423720113933086395263671875f,1.53749999374151258280107867904007434844970703125f,0.1396999943137167132789500101353041827678680419921875f,1.429400014443695798860289869480766355991363525390625f,0.9277000234976411352505465401918627321720123291015625f,0.478000012755393388630409390316344797611236572265625f,0.7793000088229773414383316776365973055362701416015625f,0.5281999789953228940930785029195249080657958984375f,0.369800001722574211981964253936894237995147705078125f,0.8351999948501589443594639305956661701202392578125f,0.22829997016191472258839212372549809515476226806640625f,-0.80940000881850693303931620903313159942626953125f,0.26659999158382419182089506648480892181396484375f,1.902300011631845588766509536071680486202239990234375f,0.8500000070035456101180670884787105023860931396484375f,-0.41119998575299998577747828676365315914154052734375f,-0.168800001618266026781611799378879368305206298828125f,0.0060999936923387121368023144896142184734344482421875f,-0.77600001135468499313674328732304275035858154296875f,1.0348000260114673665157170034945011138916015625f,1.2540000349879267815822458942420780658721923828125f,0.327599966728687519434970454312860965728759765625f,-0.7988000297069550725836961646564304828643798828125f,-0.64250001773238185620584772550500929355621337890625f,0.59090001474916942214576920378021895885467529296875f,-0.1841000342011458457136541255749762058258056640625f,-0.736199994879961128191325769876129925251007080078125f,-0.0238000069826844740106253084377385675907135009765625f,-1.09899999572336692921226131147705018520355224609375f,-1.282000009149312891310046325088478624820709228515625f,0.149700048181415024117768552969209849834442138671875f,0.1771999947234987882183077090303413569927215576171875f,-1.043999983765184769168854472809471189975738525390625f,0.333099996285139976492928326479159295558929443359375f,1.1791000204548238716739660958410240709781646728515625f,-0.293099989020824569507794876699335873126983642578125f,1.383400034642219278424590811482630670070648193359375f,-0.0423999589517709551955704228021204471588134765625f,0.7178000177562235961659098393283784389495849609375f,-0.617300008837878610989946537301875650882720947265625f,-1.325100029015541025501079275272786617279052734375f,-0.811900012320279973465630973805673420429229736328125f,-0.259099984729289634088900129427202045917510986328125f,0.13859999432563796517570153810083866119384765625f,0.1172000122696170709701846135430969297885894775390625f,0.041099996709823560792074204073287546634674072265625f,0.42990002731084764331370706713641993701457977294921875f,0.76210000903308383612966281361877918243408203125f,0.11830000164061760248301879983046092092990875244140625f,0.28690000919103619825278883581631816923618316650390625f,1.03530000106692288142085089930333197116851806640625f,0.60990002589523817366767843850539065897464752197265625f,0.94100002029538121206542200525291264057159423828125f,0.103300006707012670403855736367404460906982421875f,0.64899995809793420420419352012686431407928466796875f,0.0216999906480312976242430522688664495944976806640625f,-0.8556999714702355763762398055405355989933013916015625f,-0.891500031113624746836876511224545538425445556640625f,0.6932000077545641403276022174395620822906494140625f,0.1505000024735927421914993828977458178997039794921875f,0.05629999126195883718537515960633754730224609375f,0.618400032258033860443902085535228252410888671875f,1.09270000170469216982382931746542453765869140625f,-0.1170999963670968924134285771287977695465087890625f,0.369100004622339827875521223177202045917510986328125f,0.0716999827876687589878201833926141262054443359375f,0.54149995832145203511487352443509735167026519775390625f,0.6171000145837661765568782357149757444858551025390625f,-0.199999991916119501667026270297355949878692626953125f,0.9314999911785122321816743351519107818603515625f,1.032999978929758366774649402941577136516571044921875f,-0.2502999776720999935974987238296307623386383056640625f,-0.40940000844597879137154450290836393833160400390625f,0.778900016701221442616542844916693866252899169921875f,-0.1271999901413920497361687012016773223876953125f,-0.2764999841749666842360966256819665431976318359375f,0.5686000098228447541259811259806156158447265625f,0.0355000051558018725472720689140260219573974609375f,-0.49090001582950293368412530981004238128662109375f,1.415900023913383876106308889575302600860595703125f,0.55269999037981054623713816909003071486949920654296875f,-0.069100041502713926977463643197552300989627838134765625f,-1.39319999128878180272295139729976654052734375f,-0.72309997672736636697976564391865395009517669677734375f,-0.538999977245926853441915227449499070644378662109375f,-0.22319995444566043119039022712968289852142333984375f,-0.630699988718330661896516176057048141956329345703125f,0.198599967652559161734870940563268959522247314453125f,-1.219700013759732026841220431379042565822601318359375f,-1.92140001491308165526561424485407769680023193359375f,0.66139997877776668389060432673431932926177978515625f,0.1785999777853490844137240856071002781391143798828125f,0.55559998127222076647058202070184051990509033203125f,0.69699997651577039192716256366111338138580322265625f,0.026600000822543989897894789464771747589111328125f,1.4153999752461903227640505065210163593292236328125f,1.138300010357796931970142395584844052791595458984375f,-0.45439999931901653429378029613872058689594268798828125f,-0.2383000149771572484524995161336846649646759033203125f,0.352100027994811437537236997741274535655975341796875f,0.11760000319927954581800122468848712742328643798828125f,1.059299988967179473320356919430196285247802734375f,0.2328000091508031221820829159696586430072784423828125f,0.3181999567925937189016849515610374510288238525390625f,-0.111100016677379953566884296378702856600284576416015625f,-0.58709996753335058627953912946395576000213623046875f,0.054400003193318724470373126678168773651123046875f,0.46669996531605750789140074630267918109893798828125f,0.66689999935626953497802560377749614417552947998046875f,-0.11659999009370825007181338150985538959503173828125f,0.0851000033855437987284631162765435874462127685546875f,-0.542499999925494247321466900757513940334320068359375f,-0.50080000309348093878725194372236728668212890625f,0.3773999795898796971727051641209982335567474365234375f,-0.2602000187106432438355341218993999063968658447265625f,0.5341000341266397555273215402849018573760986328125f,0.2221000441253189183754557234351523220539093017578125f,-1.3677000200331210777449086890555918216705322265625f,-1.07190005151033407315708245732821524143218994140625f,1.0062999842584137155654389061965048313140869140625f,0.318499995782971279112416596035473048686981201171875f,0.582100018942355745821259915828704833984375f,1.813100007647275635491723733139224350452423095703125f,-0.134400002336501955824132892303168773651123046875f,-0.049099985772370846603962490917183458805084228515625f,-0.2237000051617616946941780042834579944610595703125f,0.9125999847590922087192666367627680301666259765625f,-0.3981000057101251599078750587068498134613037109375f,0.479499997228384045033777738353819586336612701416015625f,-0.03530000047087711312343571989913471043109893798828125f,1.428600008183717573473359152558259665966033935546875f,0.195900007000565590242757707528653554618358612060546875f,0.29750000633299322760905170071055181324481964111328125f,0.95630001551359899991666679852642118930816650390625f,0.401300000947713897403446026146411895751953125f,-0.66989999981820613772498518301290459930896759033203125f,0.68509999459385839681857532923459075391292572021484375f,-0.8344000031188125721115511623793281614780426025390625f,-1.656099996113777006456757590058259665966033935546875f,0.57830000544041393339966816711239516735076904296875f,-1.67580004377365110457276387023739516735076904296875f,-0.122899960978328781635582345188595354557037353515625f,-2.04570002980828302696636455948464572429656982421875f,0.6216999884128568965024896897375583648681640625f,-0.8599000088334083358176940237171947956085205078125f,-0.1149999980628491069722940665087662637233734130859375f,0.7195999968327579932036996979149989783763885498046875f,0.36900001509487656559116430798894725739955902099609375f,-0.335900004877149882531739422120153903961181640625f,-0.092300004348904007844822672268492169678211212158203125f,0.35389999554157203220938754384405910968780517578125f,-0.3257999966114757395274637019610963761806488037109375f,-0.2842999973863353790903829576564021408557891845703125f,-0.06270001757442944523290861980058252811431884765625f,-0.84520000125765815113254575408063828945159912109375f,0.23309998373091200818407742190174758434295654296875f,-0.9877000620543956443952993140555918216705322265625f,0.0830999978497624081086314618005417287349700927734375f,-1.545000009536743323934615546022541821002960205078125f,-0.03280002610087395797933140784152783453464508056640625f,-0.291000020742415710373052206705324351787567138671875f,0.03479998995065691769212889994378201663494110107421875f,-0.17490002254247638546758025768212974071502685546875f,-0.419800035846233488001644218456931412220001220703125f,-0.749400000846385250241610265220515429973602294921875f,1.0690000153928995718644046064582653343677520751953125f,-0.368899998410045615315766553976573050022125244140625f,-0.071900024837255660514756527845747768878936767578125f,-1.12780000628232901505043628276325762271881103515625f,-1.109500022560358178935757678118534386157989501953125f,0.01720003842115414016689101117663085460662841796875f,0.616900001293421151871143592870794236660003662109375f,0.6492000164270397277732627117075026035308837890625f,0.07639998973011952188016948639415204524993896484375f,0.681400062596797084779609576798975467681884765625f,0.92210001909136762865415448686690069735050201416015625f,-0.630900005119294338129520838265307247638702392578125f,-0.2545999935731291241580720452475361526012420654296875f,-0.47379999277070139473977405941695906221866607666015625f,-0.20369997789263738496856603887863457202911376953125f,-0.8124999807029966714111424153088591992855072021484375f,-1.3715999597698453893457326557836495339870452880859375f,0.1298000371500851457540193223394453525543212890625f,-0.7392999826714399258520415969542227685451507568359375f,0.44659999724626497918933409891906194388866424560546875f,-1.058600033217668823226631502620875835418701171875f,-0.22360001887530100983525471747270785272121429443359375f,-0.5214000000119209943250098149292171001434326171875f,0.84389999710619445494330648216418921947479248046875f,0.5450000199675562040368959060288034379482269287109375f,0.4063999754250053886295290794805623590946197509765625f,-0.5643999908626076145168326547718606889247894287109375f,0.3571999905884266635069934636703692376613616943359375f,-1.0657000254310664555390530949807725846767425537109375f,0.1883000082716346224032122336211614310741424560546875f,-0.55520001491084702838207931563374586403369903564453125f,-0.61620000128745999745660810731351375579833984375f,-0.73599996821582303052622364702983759343624114990234375f,-0.6347000019133088866141179096302948892116546630859375f,-0.450000030025840214165100405807606875896453857421875f,-0.37970000824630256719416365740471519529819488525390625f,-1.16960001402497226763443904928863048553466796875f,-0.4868999988719819871363370111794210970401763916015625f,-0.635199972051382122373297534068115055561065673828125f,0.29180001292079682517766059390851296484470367431640625f,0.071399996733665371806409893906675279140472412109375f,0.79400001014024024215132158133201301097869873046875f,-1.008299979549646341325797038734890520572662353515625f,-0.0868000061035159031774810500792227685451507568359375f,-0.5625000139884652750055238357163034379482269287109375f,-0.160099994836002623710413672597496770322322845458984375f,-0.5641999952673912854805848837713710963726043701171875f,-0.2465000000633299492847072542645037174224853515625f,0.0637000155925750988217259873636066913604736328125f,-0.11570000030398386314800518448464572429656982421875f,-1.5739999500513075503249638131819665431976318359375f,-1.09149999579787237991013171267695724964141845703125f,0.046600024664402273089081063517369329929351806640625f,1.0465999966502190776651559644960798323154449462890625f,0.0257999694168571380714638507924973964691162109375f,0.711399998074770056888382896431721746921539306640625f,-0.27760001083612451111548580229282379150390625f,0.2410999844908712219648805330507457256317138671875f,-1.2002999743938449483948716078884899616241455078125f,-0.9805000076889991422746106763952411711215972900390625f,-0.2757999640524388951234868727624416351318359375f,1.1986000230848790071291887215920723974704742431640625f,-0.19119999145269428453275395440869033336639404296875f,-0.8010000016689300395000827847979962825775146484375f,-0.38750000730156930472958265454508364200592041015625f,-0.24670000405907632767821269226260483264923095703125f,1.0727999907106158250513772145495750010013580322265625f,-1.06160000454783443046835600398480892181396484375f,-0.345999981403350920228234599562711082398891448974609375f,-0.408399982488155455229161816532723605632781982421875f,-0.21599995853006870305534903309307992458343505859375f,-0.25639996290802979928002969245426356792449951171875f,-0.70479998473525018454211021889932453632354736328125f,-0.90639997095465663079494333942420780658721923828125f,0.882199969466030520237609380274079740047454833984375f,-1.539500033736228790104405561578460037708282470703125f,0.219099982419610039841018078732304275035858154296875f,-0.94580001829266535651186131872236728668212890625f,1.0370000111609698478787322528660297393798828125f,0.4134000134825708272501287865452468395233154296875f,-0.8116000139355659115381058654747903347015380859375f,-0.12509998982548697910033297375775873661041259765625f,-0.23029998899698256309420685283839702606201171875f,-1.4175999758183959187363143428228795528411865234375f,-0.675299971152842015698070099460892379283905029296875f,-0.62369995568990788115115719847381114959716796875f,-0.25769999093860374017594949691556394100189208984375f,0.4533000016033650325653070467524230480194091796875f,-0.464400009451806372595683569670654833316802978515625f,-0.49480000410675994970688407192938029766082763671875f,-0.316600013227760879175320951617322862148284912109375f,-1.743900004035233930466120000346563756465911865234375f,-0.41249999571591622338218030563439242541790008544921875f,-1.126099967652559374897691668593324720859527587890625f,0.026200006726384117428096942603588104248046875f,0.3084999980926512730405875117867253720760345458984375f,-0.402399997843802037511551361603778786957263946533203125f,-0.0635999664604665060352317595970816910266876220703125f,-0.268299995754659181823598146365839056670665740966796875f,-0.031899999058246475414080123300664126873016357421875f,-0.0395000095963478070615337855997495353221893310546875f,0.450800009351969066528909024782478809356689453125f,-0.3408000057756901401972982057486660778522491455078125f,-0.3387000178277492778988744248636066913604736328125f,1.52280002759099009068677332834340631961822509765625f,0.83749995812773736503231702954508364200592041015625f,-0.4506000301480295178180313087068498134613037109375f,-0.672600026929377481366145730135031044483184814453125f,-0.0441999980986118590209343892638571560382843017578125f,-0.160299990934133607112244135350920259952545166015625f,0.5969999887347217093491735795396380126476287841796875f,0.588500042349100027649910771287977695465087890625f,-0.7337999577343463641909693251363933086395263671875f,-1.36259995510578146848956748726777732372283935546875f,-0.579799997997283877992913403431884944438934326171875f,0.82239999035596866860942100174725055694580078125f,0.091900003230571680745697449310682713985443115234375f,-0.116099983336031442604507901705801486968994140625f,-0.381599997246265587591551593504846096038818359375f,-0.7227000080749392818546539274393580853939056396484375f,-0.5542999947786331205179521930404007434844970703125f,-0.303299950306117782616865952149964869022369384765625f,0.93670003162622439418782960274256765842437744140625f,-1.2886999803513290441259186991374008357524871826171875f,-1.104199997353554341117387593840248882770538330078125f,0.033599987694621080702717108579236082732677459716796875f,-0.0182999956429004129176973947323858737945556640625f,0.8211999809101222513874063224648125469684600830078125f,0.504899984097480825084858224727213382720947265625f,0.94049999316036736463075840219971723854541778564453125f,-0.24080002470016470539349029422737658023834228515625f,-0.00339999679327007697793305851519107818603515625f,1.194799974453449209477184922434389591217041015625f,0.13040000683665287084522788063623011112213134765625f,-0.478600002223253273569980592583306133747100830078125f,-0.05930000807791946382252490366226993501186370849609375f,0.644600019687414427238536518416367471218109130859375f,0.099899993708729706298754535964690148830413818359375f,1.2161999699950210018783991472446359694004058837890625f,-0.239699997591972080446254267371841706335544586181640625f,-0.2267000022709364515094421221874654293060302734375f,-0.301100008556246745428097710828296840190887451171875f,0.22880000079870210782928552362136542797088623046875f,0.3831000099942087899762555025517940521240234375f,-0.3971000254988670263855965458787977695465087890625f,0.14740002067983137834517037845216691493988037109375f,-0.51939998430609790602829889394342899322509765625f,-0.293699980090558643297526941751129925251007080078125f,0.30399998849630360808760087820701301097869873046875f,0.1459999969750642723198552630492486059665679931640625f,0.274000003099441613585440791212022304534912109375f,0.04879998992085443632049646112136542797088623046875f,0.3170000195801259135208738371147774159908294677734375f,0.07240000130832180413875676094903610646724700927734375f,-0.171200005534291488373810352641157805919647216796875f,-0.4536999802768229006488809318398125469684600830078125f,0.19630000780224765577486323309130966663360595703125f,-0.045400014063715854462088827858678996562957763671875f,1.331699988785385979639386277995072305202484130859375f,0.6671999918922779659880006875027902424335479736328125f,-0.7740999940425155045886640436947345733642578125f,-0.2624000059276816454456593419308774173259735107421875f,0.0584999821111560347475233356817625463008880615234375f,0.751800025773048385957508799037896096706390380859375f,-1.00709998884201024083040465484373271465301513671875f,-1.264700024563074176597865516669116914272308349609375f,0.27779998065233257875661365687847137451171875f,-0.030999999731779137590592654305510222911834716796875f,0.32330000115633017454541686674929223954677581787109375f,-0.3770000103414059555717585681122727692127227783203125f,-0.171400024561583930182706581035745330154895782470703125f,0.230599990883469718738751907949335873126983642578125f,0.47109998285174381482676153609645552933216094970703125f,1.4306999976217749104279164384934119880199432373046875f,-0.43859997990876475881805163226090371608734130859375f,0.66710002214610586435128425364382565021514892578125f,-0.238099990101158731636843413070891983807086944580078125f,-0.9462999655574562130055937814177013933658599853515625f,-0.48779997763484750361584474376286379992961883544921875f,-0.5562999744608998486938844507676549255847930908203125f,0.67430001001507078317587229321361519396305084228515625f,0.53370003049373604397942472132854163646697998046875f,0.01389998689889893057625158689916133880615234375f,0.699999947249889231670749722979962825775146484375f,0.44950000244379051395071655861102044582366943359375f,-0.378100005188584109561134027899242937564849853515625f,-0.02590000863969332822733804277959279716014862060546875f,0.6921999934196474413283795001916587352752685546875f,0.4087000073596833704669961662148125469684600830078125f,0.864300008189677893000180119997821748256683349609375f,0.4127000134393572494673207984305918216705322265625f,0.157200016553699750687655978254042565822601318359375f,-0.097399999073147558448226845939643681049346923828125f,-0.10279999730437994198695150771527551114559173583984375f,-0.17679999448060990108189116654102690517902374267578125f,-0.220299962472915888955782293123775161802768707275390625f,-0.291200001473724601197545780451036989688873291015625f,0.652599966579675339062305283732712268829345703125f,-0.008499999769032040575211794930510222911834716796875f,-0.38240000779032623512421196210198104381561279296875f,-0.2184000145629048095230473336414434015750885009765625f,0.3512999974086878918200227417401038110256195068359375f,-1.132099969284236617994565676781348884105682373046875f,-0.0852999673157939231060709062148816883563995361328125f,-0.0883000087186693871643683451111428439617156982421875f,-0.45700004889816059350238219849416054785251617431640625f,-1.504900000861287168163471505977213382720947265625f,-1.247399987785518415961405480629764497280120849609375f,-1.246999986425042283855191271868534386157989501953125f,-0.41119998489618314607696447637863457202911376953125f,0.91949996221065521950777110760100185871124267578125f,0.55299996726960021220520502538420259952545166015625f,2.09890003074556563689156973850913345813751220703125f,1.068200010436773705890800556517206132411956787109375f,1.324200024883449344770269817672669887542724609375f,1.3565000559613109576417855350882746279239654541015625f,0.6642000128507614675754666677676141262054443359375f,0.1871999984860419630194883211515843868255615234375f,0.008999991178512800615862943232059478759765625f,-0.691499955825508383355781916179694235324859619140625f,0.0229999961778524220079589213128201663494110107421875f,1.0035999919787048639818749506957828998565673828125f,0.21589999534040682505064978613518178462982177734375f,0.266299998973309914873652815003879368305206298828125f,0.9888999852225186515397581388242542743682861328125f,-0.392199983845651356517691965564154088497161865234375f,-0.137999977625906711242720348309376277029514312744140625f,0.24879996235370693824506815872155129909515380859375f,-0.504899973070621843618255297769792377948760986328125f,0.07589996721446479188699640872073359787464141845703125f,-0.3658999761179086984697050866088829934597015380859375f,-1.0561000048682089857976507119019515812397003173828125f,-0.7174000442609191896536913191084749996662139892578125f,0.11400000950694089141279619070701301097869873046875f,-0.4024000055551528731712096487171947956085205078125f,0.71400000935792906631149890017695724964141845703125f,-0.20720000619739320502077362107229419052600860595703125f,0.83070000753104744628529942929162643849849700927734375f,0.142000030905008844950287993924575857818126678466796875f,0.36189997365176651555884745903313159942626953125f,-0.37670000778436640853641392823192290961742401123046875f,1.1936000144049525584222237739595584571361541748046875f,0.59930002350062150373588565344107337296009063720703125f,-0.2292999910160897769628718378953635692596435546875f,0.33000000413507224461540090487687848508358001708984375f,-1.6798000128984451606584116234444081783294677734375f,-0.02209999808073070948211125141824595630168914794921875f,-0.50500003308057817008602796704508364200592041015625f,-0.3495000182762744156406142792548052966594696044921875f,-0.269099996501207083809958930942229926586151123046875f,0.20580002119839224672404043303686194121837615966796875f,0.2319999947696922415474318768247030675411224365234375f,0.7983000035405154459766663421760313212871551513671875f,-0.0409000225722795374139195700990967452526092529296875f,0.0226000394463550247081684574368409812450408935546875f,1.703400061315298419373220895067788660526275634765625f,2.00550005015730814506014212383888661861419677734375f,0.33880001324117181571438095488701947033405303955078125f,0.0578000424176454696834071000921539962291717529296875f,1.2010000144094223895052664374816231429576873779296875f,-0.0290000228062272658036135908332653343677520751953125f,0.317000018090009660909345257095992565155029296875f,0.857800015074014599036900108330883085727691650390625f,0.51900000101327936619100000825710594654083251953125f,0.09569991607516925569143495522439479827880859375f,-1.66130003708302975695687564439140260219573974609375f,0.084600040064752857205121472361497581005096435546875f,0.834100025334954420230815230752341449260711669921875f,0.7282000009000297868766438114107586443424224853515625f,-0.175099997276067842477687008795328438282012939453125f,-0.50970002344548692718717575189657509326934814453125f,0.462900001397728910745854591368697583675384521484375f,-0.69149998588860039916426103445701301097869873046875f,-1.037500040456652339315724020707421004772186279296875f,-0.294299996380507966620143633917905390262603759765625f,0.3837999825075268933716188257676549255847930908203125f,0.3868999865040176899810830946080386638641357421875f,0.2226000283077358421479630123940296471118927001953125f,-0.93989999676346780432822924922220408916473388671875f,-1.386500002354383642710899948724545538425445556640625f,1.11330001575946813119344369624741375446319580078125f,-0.92339999008774764632789811003021895885467529296875f,-0.985999996975063908166703185997903347015380859375f,-0.12229999956190573673353583217249251902103424072265625f,0.3821000042647122452166286166175268590450286865234375f,-0.355899994334578195775975473225116729736328125f,-0.6568999866902824891212731017731130123138427734375f,0.753700024794042189313358903746120631694793701171875f,0.180699969607591270914781489409506320953369140625f,-0.48400002813339237928147440470638684928417205810546875f,-0.3611999785631894166471056450973264873027801513671875f,0.025200023981929164929027820107876323163509368896484375f,0.20600001705437909205187452243990264832973480224609375f,0.0778999999448657387546290919999592006206512451171875f,0.05570000209212278008408247842453420162200927734375f,-0.29950000981986579784432933593052439391613006591796875f,-0.6300999938488001106406954932026565074920654296875f,-0.048800003853440188095191842876374721527099609375f,1.563300041687488484143386813229881227016448974609375f,0.0455999868974086286499414200079627335071563720703125f,0.9277000017046930224751122295856475830078125f,-0.053400005771219449801634482355439104139804840087890625f,-0.456499986857175965582200660719536244869232177734375f,1.01300005757063704692200190038420259952545166015625f,2.6097000194966799568874193937517702579498291015625f,1.0810999818459148258398272446356713771820068359375f,-0.900400011716782966431082968483678996562957763671875f,-0.27699996776133739562197888517403043806552886962890625f,0.644000035956501459821765820379368960857391357421875f,1.55730002459585659124741141567938029766082763671875f,0.807800012205541317911183796240948140621185302734375f,0.77500000264495572110945431631989777088165283203125f,0.1906000067532060970876273131580092012882232666015625f,0.911499994717539951949447640799917280673980712890625f,0.23280001343488709864004704286344349384307861328125f,0.90390006192624572367577684417483396828174591064453125f,1.047799975138902883742275662370957434177398681640625f,-0.316699991616606002509115569409914314746856689453125f,-0.161099984341859681080677546560764312744140625f,-0.254999983310699729344150910037569701671600341796875f,-0.6516999951183795047882085782475769519805908203125f,0.8473999773919587141790543682873249053955078125f,-0.3843000303924082317053034785203635692596435546875f,-0.79639998581856463744088614475913345813751220703125f,-0.342600012661516573064801605141838081181049346923828125f,-0.79449997178465192604335243231616914272308349609375f,-0.8154999719262121526952569183777086436748504638671875f,-0.210899990646541135586033988147391937673091888427734375f,-0.5963999935671691066119137758505530655384063720703125f,-0.012400018370150700519616293604485690593719482421875f,-1.5724999901652341183222461040713824331760406494140625f,-0.6512999888777730728151027506100945174694061279296875f,0.73289998746514317673472760361619293689727783203125f,0.512399963682890646765599740319885313510894775390625f,0.6439999724030494032689375671907328069210052490234375f,-0.243899988538026857298746108426712453365325927734375f,-0.14990001285672160946660369518212974071502685546875f,-0.6364999981075525337104181744507513940334320068359375f,0.69030002889484187722501928874407894909381866455078125f,-0.7121999990075826847402140629128552973270416259765625f,-0.1339000082820656079007903827005065977573394775390625f,-0.378700018256157566742814424287644214928150177001953125f,0.086899989968538260409758322566631250083446502685546875f,-0.438400001114607196228689645067788660526275634765625f,0.345200018766522287450015937793068587779998779296875f,-0.803800023895501869475310741108842194080352783203125f,-0.4407000107720493442542419870733283460140228271484375f,1.5969999970048662785160331623046658933162689208984375f,-1.0051000229060649271417560157715342938899993896484375f,1.958300049436092482579852003254927694797515869140625f,-0.5779000421524056463340457412414252758026123046875f,1.950200018095970211362555346568115055561065673828125f,-0.564000008650123785258756470284424722194671630859375f,0.9235000040531158305157077847979962825775146484375f,-0.32800000299513332180367797263897955417633056640625f,0.925400011437385927592913503758609294891357421875f,0.139099954386800117589473302359692752361297607421875f,-0.37250000631436697151599446442560292780399322509765625f,-0.05599999220669238386705046650604344904422760009765625f,-0.6000000184774396672082730219699442386627197265625f,-1.352000060446561402471843393868766725063323974609375f,0.01599997849762413437701980001293122768402099609375f,0.66529998393058809025291111538535915315151214599609375f,0.290399994841218589414211237453855574131011962890625f,0.977100056158005880746486582211218774318695068359375f,-0.2607999932587148350648931227624416351318359375f,-0.4969999913051719175882681156508624553680419921875f,-0.585700064229965011719514222932048141956329345703125f,-0.0744999950304627844843707862310111522674560546875f,-0.276400013422966051024332045926712453365325927734375f,-0.497199983584880822462537253159098327159881591796875f,-0.059499995104968561587810427226941101253032684326171875f,0.53679999781474407871684206838835962116718292236328125f,-0.0936999635688959831014699375373311340808868408203125f,-0.297400008963793205385428564113681204617023468017578125f,-0.2569000195287169940883131857845000922679901123046875f,0.9337000130593773317144723478122614324092864990234375f,0.50680002040863048051733130705542862415313720703125f,-0.36300000478327287822821745066903531551361083984375f,-0.48499998472630967416563407823559828102588653564453125f,-0.8057999805182214370091742239310406148433685302734375f,0.25289995431005962700510281138122081756591796875f,0.33660001706480979777325046597979962825775146484375f,0.6031000082433219944277880131267011165618896484375f,-0.6691999787271019073386923992075026035308837890625f,1.250899977570771870460930585977621376514434814453125f,-0.033799989920854756064727553166449069976806640625f,0.388199995982647205750026841997168958187103271484375f,-0.48730001572966585854373988695442676544189453125f,-0.15539998664557919116901985034928657114505767822265625f,0.3033999828606843696121586617664434015750885009765625f,-0.515000012144446461803681813762523233890533447265625f,0.38289998102039068950119826695299707353115081787109375f,-0.2758000029817224696415678408811800181865692138671875f,0.837700025397539516092138001113198697566986083984375f,0.25030000348836178858391576795838773250579833984375f,0.9021000149190427919165813364088535308837890625f,-0.0283999931424856466577466562739573419094085693359375f,0.8716999714255333486789822927676141262054443359375f,-0.6288000137627121066685731420875526964664459228515625f,0.431599973851441998107247854932211339473724365234375f,-0.50230000921040751649115918553434312343597412109375f,1.0170000299923120934408871107734739780426025390625f,-0.156000015042722484093218326961505226790904998779296875f,0.30900001578405511537539496202953159809112548828125f,-0.61420004805475547637882982598966918885707855224609375f,-0.9779999967366459490136776366853155195713043212890625f,0.159000005595385740519986939034424722194671630859375f,-0.6183000027954574928656938936910592019557952880859375f,-0.41380004051029739553513309147092513740062713623046875f,-1.151400018936396207891448284499347209930419921875f,-1.151800014895199897324573612422682344913482666015625f,0.217199995654821176316318087629042565822601318359375f,0.16630005105286882471915532732964493334293365478515625f,1.060700005519390032304727355949580669403076171875f,-0.28640000321567082863793984870426356792449951171875f,-0.72429998047351773493574000895023345947265625f,-0.1378999280095103419085944551625289022922515869140625f,-0.445899982599914090908299613147391937673091888427734375f,-0.5837999888963996131252542909351177513599395751953125f,-0.65360001306384807318039520396268926560878753662109375f,-0.44539999681562159050685067995800636708736419677734375f,-0.6056999794423578720170553424395620822906494140625f,-0.30029998657554435315120144878164865076541900634765625f,0.240900013855099626169931070762686431407928466796875f,0.6128000193208456014559715185896493494510650634765625f,-0.1044000024110081081829548566020093858242034912109375f,0.54439998773336384374488261528313159942626953125f,1.2620000316500661341478917165659368038177490234375f,-0.616100021147727883175093666068278253078460693359375f,1.122599981480836728309213867760263383388519287109375f,0.385199987101554786050883194548077881336212158203125f,0.5663999630570415444452692099730484187602996826171875f,-0.730499991595744635475284667336381971836090087890625f,0.0164999272525314655268857677583582699298858642578125f,0.4241000079192220884038988515385426580905914306640625f,0.55260000258460639788182788834092207252979278564453125f,-0.103300001249462258812172876787371933460235595703125f,0.53410000091567610969178758750786073505878448486328125f,0.3350999987661840151531578158028423786163330078125f,-0.4204000073209404053642401777324266731739044189453125f,0.7833000182926657117832291987724602222442626953125f,0.030999994255602292270168618415482342243194580078125f,-0.10729999069571505287967738695442676544189453125f,-0.5138000019162893838853278793976642191410064697265625f,-0.75460002266764636402740507037378847599029541015625f,-0.1723999884933233861517010154784657061100006103515625f,0.590999950259923689799279600265435874462127685546875f,-0.5629000064268705472159126657061278820037841796875f,1.6194000145554543479420317453332245349884033203125f,-0.10480001304745678591956448144628666341304779052734375f,1.0515000245571137771349867762182839214801788330078125f,-0.60820001550316804550533333895145915448665618896484375f,-0.8480000203549862280993920649052597582340240478515625f,0.42840001974105856863417329805088229477405548095703125f,-0.2785000080764294949631221243180334568023681640625f,-0.6772000090658660820253089696052484214305877685546875f,-1.7437999738276008798720795311965048313140869140625f,-1.8903999827712769654652902318048290908336639404296875f,-0.64829998983144765389852182124741375446319580078125f,0.179700022178887863777418942845542915165424346923828125f,-0.5585999830007555289057563641108572483062744140625f,0.32009997529685474404459455399774014949798583984375f,-0.16289999879002525773330489755608141422271728515625f,0.17219997315406754267996802809648215770721435546875f,-1.327200003105402092984377304674126207828521728515625f,-1.58719997684210500921153652598150074481964111328125f,-0.538300010395050154698992628254927694797515869140625f,-0.370899944117665025800079092732630670070648193359375f,-0.8659999726116656137975269302842207252979278564453125f,-0.8487000098556281013628677101223729550838470458984375f,-0.1093999872863291589197842768044210970401763916015625f,0.3034000049889085115495390709838829934597015380859375f,0.10940000099539748390498061780817806720733642578125f,0.66059997829198824348395646666176617145538330078125f,1.62319999449253060674891457892954349517822265625f,-0.819899982011318417818301895749755203723907470703125f,0.2206999927043913345414694049395620822906494140625f,-0.1673999720275405334035667692660354077816009521484375f,-1.4221999829888343214179258211515843868255615234375f,0.7827000002145769741446201805956661701202392578125f,0.01149998044967670551841365522705018520355224609375f,0.6609999861717223978985202847979962825775146484375f,1.8556999865949155292810246464796364307403564453125f,-0.72080000652074804889934966922737658023834228515625f,1.33619997461438178021353451185859739780426025390625f,0.3520000069141389076321502216160297393798828125f,-0.8876000163495538775038085077540017664432525634765625f,0.1282999950468539651637911447323858737945556640625f,0.1437000045657159663647917113848961889743804931640625f,0.232100003594160053665973464376293122768402099609375f,-0.2723000095456840075058835282106883823871612548828125f,0.18549999725818633322660389239899814128875732421875f,0.17049999759346246008817615802399814128875732421875f,0.0817000162780285865693485902738757431507110595703125f,-0.31029999201446745171750762892770580947399139404296875f,-1.5965000026524069909328318317420780658721923828125f,0.2562000086262816722637580824084579944610595703125f,0.4532000050723556938692127005197107791900634765625f,0.2254000054955482834628810451249592006206512451171875f,0.57070000611543658664004396996460855007171630859375f,-0.28669999631047227151015022172941826283931732177734375f,0.1219000120222570870964773348532617092132568359375f,0.0681999715819952001538695185445249080657958984375f,0.23860003604888913031345509807579219341278076171875f,0.325199998873471951554847692023031413555145263671875f,-0.9509999891519544945595043827779591083526611328125f,-0.167100011752546018239939940031035803258419036865234375f,-0.282300006937980629828643941436894237995147705078125f,0.231300007168948607017000540508888661861419677734375f,0.382599984908103873948448381270281970500946044921875f,0.8289000383451583520155736550805158913135528564453125f,0.1784000546097754824614867175114341080188751220703125f,0.7260999377384773634247494555893354117870330810546875f,-0.05260002856850665153842783183790743350982666015625f,0.645699985328317627164551595342345535755157470703125f,0.67319999970793720223127820645458996295928955078125f,0.046000012323260397462121318312711082398891448974609375f,-0.0459999577105048018665911513380706310272216796875f,-0.5889999865218988883697193159605376422405242919921875f,-0.9602000101983545476969084120355546474456787109375f,-1.418399987033009512771286608767695724964141845703125f,0.30099999153614032110226617078296840190887451171875f,0.26319997869730027417745077400468289852142333984375f,0.32220000846981999842455479665659368038177490234375f,-0.4832999868512153351929327982361428439617156982421875f,-0.149400008744001500105014201835729181766510009765625f,-0.87649999862909311332259676419198513031005859375f,-0.20579999281167982161377949523739516735076904296875f,0.31290004960298478664526555803604423999786376953125f,-2.40919995533227915274210317875258624553680419921875f,0.005699980932474257855346877477131783962249755859375f,0.2903000347435469308265965082682669162750244140625f,0.1050999856531620313404573607840575277805328369140625f,0.550699994045495966332737225457094609737396240234375f,0.5109999779760838745090723023167811334133148193359375f,0.04969999103546118845997625612653791904449462890625f,0.2196999819457534641742313397116959095001220703125f,1.3988999888360496015593525953590869903564453125f,-0.116800003719329748008703973027877509593963623046875f,-0.53719998388290424173874271218664944171905517578125f,0.808999999225139543312934620189480483531951904296875f,-0.42240001982301478289372198560158722102642059326171875f,0.17860000937581066438042398658581078052520751953125f,0.35870000821650027678089145410922355949878692626953125f,0.251800012511015136595915464567951858043670654296875f,0.53100000487267973170446566655300557613372802734375f,-0.7339999984800815724383937777020037174224853515625f,-0.135499986268579675208201251734863035380840301513671875f,-1.10110000877976421662651773658581078052520751953125f,-0.17879999256581069477789469601702876389026641845703125f,0.70009997950643310105078853666782379150390625f,-0.1752999955534933762546501156975864432752132415771484375f,-0.167100003109872385920198212261311709880828857421875f,0.1860999992802738756125791041995398700237274169921875f,-0.12370004077553797827704329392872750759124755859375f,0.313799998489022069492193622863851487636566162109375f,-0.8364999792575833481578229111619293689727783203125f,0.1164999925568697136668561142869293689727783203125f,-0.4416000110298394787378128967247903347015380859375f,-0.031499968826770530228031930164434015750885009765625f,0.93989999966919413054711185395717620849609375f,0.286700015048683365392889754730276763439178466796875f,0.741300004523992583926883526146411895751953125f,-0.15370000456571586422427344587049447000026702880859375f,-0.0573999591752889504192580716335214674472808837890625f,0.853099996583163378005565391504205763339996337890625f,-0.11739999031871561907536261060158722102642059326171875f,-0.116600001157820154507049892345094121992588043212890625f,-0.492799985942244778147625083875027485191822052001953125f,0.428000006683170231003288108695414848625659942626953125f,0.2032999786555773624741050298325717449188232421875f,-0.93850005855411378075103812079760245978832244873046875f,-0.51550003615021733338608100893907248973846435546875f,-0.17440001589655818037272183573804795742034912109375f,0.699999995976686495424701206502504646778106689453125f,-0.7418999904155729741006553012994118034839630126953125f,-0.103500015303492620688530223560519516468048095703125f,0.19519997272193456705480230084504000842571258544921875f,0.192500003203749603386540911742486059665679931640625f,0.1774999831616879131246378165087662637233734130859375f,-0.0291000042498110733646399239660240709781646728515625f,-0.62299996723234707385330466422601602971553802490234375f,-1.4688999876812098843714693430229090154170989990234375f,0.449399989148974443597950312323519028723239898681640625f,0.3025999943703412409234942970215342938899993896484375f,-0.6677999686196451900599413420422933995723724365234375f,-0.385299991045892020746777006934280507266521453857421875f,1.329999988228082852259603896527551114559173583984375f,0.3426999917209148094343618140555918216705322265625f,-0.241400005823373842162027358426712453365325927734375f,0.08080003200173369037884185672737658023834228515625f,0.76239999944567671263939701020717620849609375f,-1.533099978068471092029767532949335873126983642578125f,0.90249999374151190778547970694489777088165283203125f,-0.165600020499527378348147976794280111789703369140625f,1.25709999256730053929231871734373271465301513671875f,0.3280999882012605173287056459230370819568634033203125f,2.0794999780058862626219706726260483264923095703125f,-0.1502000006988641889194013856467790901660919189453125f,1.288699993538856336527942403336055576801300048828125f,-0.256300037232040978096847538836300373077392578125f,1.480399990892410588827488027163781225681304931640625f,0.32980000578314072612329255207441747188568115234375f,0.3606999670743942232320478069595992565155029296875f,0.4317000420570373808715203267638571560382843017578125f,-0.857099987649917682830391640891321003437042236328125f,-0.284000008314848173540667630732059478759765625f,-1.563200006562471600091157597489655017852783203125f,-0.338499993920326147645027958787977695465087890625f,0.73740000734329225196006518672220408916473388671875f,0.195799980220198843738899086019955575466156005859375f,1.848899949944020182357462545041926205158233642578125f,0.74859999305903890398639077830011956393718719482421875f,1.22569998540282210086616032640449702739715576171875f,0.2518000166460871991347403309191577136516571044921875f,0.8433000107675792111905366255086846649646759033203125f,0.0429999966248868492613155467552132904529571533203125f,0.758099988275766190071180972154252231121063232421875f,0.93800000277161588169150263638584874570369720458984375f,-0.2861000236809250196756693185307085514068603515625f,0.5429999791905284478588100682827644050121307373046875f,-0.571200029450654955809341117856092751026153564453125f,-0.33210002557337270356896397061063908040523529052734375f,-0.702399997806548270773419062606990337371826171875f,-0.442400011664628944885180317214690148830413818359375f,-0.4331999760150910816491887089796364307403564453125f,-1.13970000921487812917121118516661226749420166015625f,1.1528999964058395111266008825623430311679840087890625f,1.73650003416836273828494086046703159809112548828125f,0.3653999721914527043509224313311278820037841796875f,-0.80460001596212382679595975787378847599029541015625f,-0.6313000222563740493342265835963189601898193359375f,0.4567999947786329784094050410203635692596435546875f,1.307999972522258413931695031351409852504730224609375f,1.5159000319600106632123015515389852225780487060546875f,0.0835999776363369750953324910369701683521270751953125f,-0.253899985632300317917042775661684572696685791015625f,-1.72840001758039019108537104330025613307952880859375f,-0.27549999859929064083274852237082086503505706787109375f,0.2135999976783988785200563143007457256317138671875f,-0.3870000219643117134182830341160297393798828125f,-1.040199988368153416473660399788059294223785400390625f,0.517799977821111756526306635350920259952545166015625f,-0.66400002306699723586547179365879856050014495849609375f,-0.0003000542640683168116311207995750010013580322265625f,0.03540002553761023573741795189562253654003143310546875f,-0.50759999392330623635416486649774014949798583984375f,-0.69659999776780612279480919823981821537017822265625f,-0.968500003755092819801575387828052043914794921875f,-1.1240000102892515609909196427906863391399383544921875f,0.830800015461444996134332541259936988353729248046875f,0.33190002029240162073620012961328029632568359375f,-0.0478000070273874033688343843095935881137847900390625f,0.3221999936059118940789858243078924715518951416015625f,-0.5730000143200157936007599346339702606201171875f,0.0357000027626754867782210567384026944637298583984375f,0.12739999296367177095135048148222267627716064453125f,0.61219998909831030431405451963655650615692138671875f,-1.300600028434395927234845657949335873126983642578125f,0.2220000027418134180834385915659368038177490234375f,-0.32539998724162533250137130380608141422271728515625f,0.171599982345103985181822281447239220142364501953125f,0.213899967974423788064086693339049816131591796875f,0.21769999533444650552382881869561970233917236328125f,0.055800023061037595795141896815039217472076416015625f,-0.16010001765340564627848607415216974914073944091796875f,1.507499983757734529632443809532560408115386962890625f,0.70629999096393569768537190611823461949825286865234375f,0.15459997282326209955272133811376988887786865234375f,-0.320200010719895455935102290823124349117279052734375f,-0.4521999807536598581236830796115100383758544921875f,-0.8334000070378184688024703064002096652984619140625f,-0.8810000056177378358057694640592671930789947509765625f,-0.21100001012533897704059882016736082732677459716796875f,-1.29839999470710676376938863540999591350555419921875f,-0.51500000990927219390869140625f,-0.32380001122951540537542314268648624420166015625f,-0.2783999957501885891275605899863876402378082275390625f,-1.1705000337660322884403285570442676544189453125f,-0.25140000507831583576034972793422639369964599609375f,0.261100020104645835772316786460578441619873046875f,-0.0124999870359896458893445014837197959423065185546875f,-1.1579000110089783248668027226813137531280517578125f,-1.0595999974846836977349084918387234210968017578125f,1.14849998065829250748492995626293122768402099609375f,1.39789998984932850589757435955107212066650390625f,-0.1915000209808344067141661071218550205230712890625f,0.4704999959170823586873666499741375446319580078125f,0.397699929583072986361003131605684757232666015625f,0.19859997495412784473955980502068996429443359375f,-0.24169995056390813914504178683273494243621826171875f,0.964700030225515536841385255684144794940948486328125f,-0.404400034821033926846212125383317470550537109375f,-0.14299999696016296724110361537896096706390380859375f,-0.7508000270843506740447992342524230480194091796875f,0.018900035923719915587071227491833269596099853515625f,-0.51320001825988292853963912421022541821002960205078125f,0.44450004416704214094124836265109479427337646484375f,-0.1114000076115131410148251234204508364200592041015625f,-0.080899977570771941515204161987639963626861572265625f,-1.8127999870225786072097662326996214687824249267578125f,0.07050003972649587780097135691903531551361083984375f,-0.0320999852657319895143928079050965607166290283203125f,-0.438700001734495526051205160911194980144500732421875f,-1.19839995570331847574152561719529330730438232421875f,-0.4246999804556370605723714106716215610504150390625f,-0.73210001912862032735773709646309725940227508544921875f,-1.928899961268901730448988018906675279140472412109375f,-0.2290999996304510233358087134547531604766845703125f,-0.7386000131011005720438333810307085514068603515625f,-0.83800001275539415246385033242404460906982421875f,0.658900066620111690696148798451758921146392822265625f,-0.486099989035725599251236417330801486968994140625f,0.1841000531256196115492684839409776031970977783203125f,-0.440099987031519479518948401164379902184009552001953125f,0.427700042384862921807098246063105762004852294921875f,-0.5659999907910824301637831013067625463008880615234375f,0.3313000399887553459876698980224318802356719970703125f,0.294399993507564239347829015969182364642620086669921875f,0.7778999560236936616774983122013509273529052734375f,-0.2407000122994185087321739047183655202388763427734375f,0.09610000982284550019585367408581078052520751953125f,-0.0671000077664849481351438953424803912639617919921875f,-0.818399964755773368096924968995153903961181640625f,-0.3429000072464349901935065645375289022922515869140625f,0.3103999910414216856224811635911464691162109375f,0.116999999277293653676679241470992565155029296875f,0.42490000443756559267427519444026984274387359619140625f,-0.024499977000057293619050824418081901967525482177734375f,0.102299975831806388981704003526829183101654052734375f,0.2296999685347078656860730916378088295459747314453125f,1.3358000111401082676820806227624416351318359375f,1.665199972796440164302111952565610408782958984375f,0.47570000566840153766179355443455278873443603515625f,-0.6023999814152716236748119627009145915508270263671875f,-1.267399938350916244189647841267287731170654296875f,0.45179996572136882360126719504478387534618377685546875f,0.74369998437464257534657008363865315914154052734375f,0.21159996312260620499046126496978104114532470703125f,0.819000036254524221845940701314248144626617431640625f,1.5750000365078449249267578125f,-0.3535999505162248368606014992110431194305419921875f,-0.29159999113678924942405501496978104114532470703125f,-0.769400014853477642873258446343243122100830078125f,-0.39610000617206087003552283931639976799488067626953125f,-0.5504999978169797802962648347602225840091705322265625f,-0.11869998389035441022087979945354163646697998046875f,0.12099999563395957835609806352294981479644775390625f,0.52680000535845739051410419051535427570343017578125f,0.6652000136256213114194224544917233288288116455078125f,-0.8810999849379061998888573725707828998565673828125f,-0.128400001747905889715184457600116729736328125f,-2.26479995716810211803249330841936171054840087890625f,0.2856999957218764318866988105583004653453826904296875f,-0.06950002203881755491465810337103903293609619140625f,-0.0763000367105008958645839811651967465877532958984375f,-0.875799978730082084865671276929788291454315185546875f,0.9272000017270449756523476025904528796672821044921875f,-0.98199996832013081160539513803087174892425537109375f,-0.475700004625320360673867980949580669403076171875f,-0.803600008630752427052357234060764312744140625f,-0.018800005157292076773956068791449069976806640625f,0.631600006932019564231950425892136991024017333984375f,0.771300004225969448867772371158935129642486572265625f,0.216499973297119563397927777259610593318939208984375f,0.7074999937042594222447178253787569701671600341796875f,-1.0428000055372710352941112432745285332202911376953125f,-0.169699994425475508830203352772514335811138153076171875f,-1.472799945932627974087836264516226947307586669921875f,-0.1940999881565570461816605529747903347015380859375f,-0.145000033751129020487269372097216546535491943359375f,0.61729998413920394995102469692938029766082763671875f,-0.73689996389150580480276175876497291028499603271484375f,0.50370000970661632511138350309920497238636016845703125f,-0.52969998105168425439615020877681672573089599609375f,-0.4166999882265927102054092756588943302631378173828125f,-0.32699998143315323062552124611102044582366943359375f,-0.3042999996587634203848438119166530668735504150390625f,-0.4376000056207178801059853867627680301666259765625f,-0.41119998310804373176097215036861598491668701171875f,-0.9156000231444831882754442631267011165618896484375f,-1.111000034302473071790018366300500929355621337890625f,0.4239999774694442891131984652020037174224853515625f,0.7818000312864779033361628535203635692596435546875f,0.193600043201445970453278278000652790069580078125f,0.40959999212026598769398333388380706310272216796875f,-0.8488000329852098957417183555662631988525390625f,-1.268700023192167236629757098853588104248046875f,-0.681099943363665971673981403000652790069580078125f,0.09789996541142453878592277760617434978485107421875f,1.4175000321865081076566639239899814128875732421875f,0.209600023561716142239674809388816356658935546875f,-0.6248999864816671134803982567973434925079345703125f,0.30149999654293058171106167719699442386627197265625f,0.894499984860420482846166123636066913604736328125f,0.126199998903274490658077411353588104248046875f,-0.00069999769628015240385821016388945281505584716796875f,0.30959996663928013305167041835375130176544189453125f,-0.3570000322088600075431941149872727692127227783203125f,0.45199994984269142861421642010100185871124267578125f,0.9433000033915046600441201007924973964691162109375f,-0.1838000019907948257014140835963189601898193359375f,0.033400008304416584081764085567556321620941162109375f,-0.4956000170350076672320938087068498134613037109375f,0.793100001798569653743697926984168589115142822265625f,-0.372499977350235322859361986047588288784027099609375f,-0.2640999988108874152459293327410705387592315673828125f,0.0997000297784804700995664461515843868255615234375f,-0.0386000133991240002018230370595119893550872802734375f,-0.721699975150823291869528475217521190643310546875f,-0.008799991559982345279422588646411895751953125f,1.039699995803832877072636620141565799713134765625f,-0.517700007814168916553398958058096468448638916015625f,0.1392999733209612234219321180717088282108306884765625f,-0.447900014734268214766643723123706877231597900390625f,-0.2423000103652466208359328447841107845306396484375f,0.474099990801513104798914355342276394367218017578125f,-0.46639999688267774757832739851437509059906005859375f,-0.5392999984666706492220100699341855943202972412109375f,-0.51250002264976490806702713598497211933135986328125f,-0.0872999842882153931356015164055861532688140869140625f,-1.0710999597549442086119597661308944225311279296875f,-0.0883999776452777386026582462363876402378082275390625f,-0.235799987745284767015618854202330112457275390625f,0.333499998241662876097279877285473048686981201171875f,0.31780002267360618617431100574322044849395751953125f,0.2711000117599964287506963955820538103580474853515625f,0.74819997079968469932964580948464572429656982421875f,0.3915999955698847845297905223560519516468048095703125f,-0.0469999863505368153937524766661226749420166015625f,0.47320001729130722889493654292891733348369598388671875f,0.50370005515515714478169684298336505889892578125f,-0.122899994915723453914324636571109294891357421875f,-0.860800017175078036046897977939806878566741943359375f,-0.5687000070989132183285619248636066913604736328125f,-0.779900008833408708852630297769792377948760986328125f,-0.2140000085383653161130723674432374536991119384765625f,0.485900006851554433495721241342835128307342529296875f,-0.76860000814646500799653949798084795475006103515625f,0.40250002473592783047706689103506505489349365234375f,0.172100038537382982184453794616274535655975341796875f,-0.4971999972939489254031286691315472126007080078125f,-1.582000008106231536686436811578460037708282470703125f,0.896999961614608931625980403623543679714202880859375f,0.9493999959290027046421300838119350373744964599609375f,0.172899983888864738901247619651257991790771484375f,0.33679998777508757257237448357045650482177734375f,1.3736999691754581132130397236323915421962738037109375f,0.42409997780025054936459127929992973804473876953125f,0.2778000357121228613976882115821354091167449951171875f,-1.2602000199586154138842175598256289958953857421875f,-0.8906000232934954130570304187131114304065704345703125f,-0.038499948024749652830678314785473048686981201171875f,0.70220000575035779188937112849089317023754119873046875f,1.79050001484155618669547038734890520572662353515625f,0.20809999684393398755588577841990627348423004150390625f,1.029700005042552657386067949119023978710174560546875f,0.6433999976500874851126354769803583621978759765625f,1.238199970799684468403256687452085316181182861328125f,-0.8981000022828578011058198171667754650115966796875f,0.341399943089485180536257757921703159809112548828125f,0.131000009268522177308113896287977695465087890625f,-0.0706999976217748127282902714796364307403564453125f,0.553299997878074378832025104202330112457275390625f,-0.8783000113636261207972211195738054811954498291015625f,-0.0083999814450741450144732880289666354656219482421875f,-1.089499969363212361628256985568441450595855712890625f,-1.0645000166743994185480914893560111522674560546875f,0.7903000383198257594585811602883040904998779296875f,0.241000019103288831701092931325547397136688232421875f,0.83539997923225162157478962399181909859180450439453125f,0.5162000113829972036683102487586438655853271484375f,0.2873000052988528008057755869231186807155609130859375f,-0.04719999539405106414591273278347216546535491943359375f,0.7373999544441698361652015591971576213836669921875f,-0.0126000028640034233973210575641132891178131103515625f,-0.236699998098611530394919100217521190643310546875f,0.0982999799966812748408528932486660778522491455078125f,0.449599968129396732052782681421376764774322509765625f,0.28669999374002219649781864063697867095470428466796875f,0.18560002448558776677600690163671970367431640625f,0.18010001351833349048092713928781449794769287109375f,0.02020002495050388091613058350048959255218505859375f,-0.7967999983921647544349298186716623604297637939453125f,-0.2116999757841224738541541228187270462512969970703125f,0.3888999885939063982931429563905112445354461669921875f,0.730199965122342309342684529838152229785919189453125f,0.2742999879986047773172685992904007434844970703125f,-0.57879998925030251566425931741832755506038665771484375f,0.496499985404312449421837527552270330488681793212890625f,-0.409900002686679254537693850579671561717987060546875f,-0.943899999937415135065066351671703159809112548828125f,-0.35009999571144601038241717105847783386707305908203125f,0.33670001020580564432549408593331463634967803955078125f,0.6818999873794613364452743553556501865386962890625f,-0.2601000068500638473523167704115621745586395263671875f,0.43939999609664026625210908605367876589298248291015625f,0.154599977666139754006735529401339590549468994140625f,1.673399976900219687792059630737639963626861572265625f,0.5561999607563021807976610944024287164211273193359375f,0.4930000130906699862265440970077179372310638427734375f,0.2870999967396257179785834523499943315982818603515625f,-0.73619997591823349569750689624925144016742706298828125f,0.24020001936256873964481428629369474947452545166015625f,0.02960000370591853169344176421873271465301513671875f,-1.26219997069537637202074620290659368038177490234375f,-0.20369999327808641087500518551678396761417388916015625f,-0.595299980242550486053687563980929553508758544921875f,-0.8660000223815440545394039872917346656322479248046875f,-0.07629997986257064379600478787324391305446624755859375f,0.45000000935047868022564898637938313186168670654296875f,1.517700017481297347643476314260624349117279052734375f,-0.6665000188574194961432795025757513940334320068359375f,0.73680003063455246081758787113358266651630401611328125f,0.6373999978065487681533340946771204471588134765625f,-0.598000018566847035828004663926549255847930908203125f,0.26530001604259023650911331060342490673065185546875f,-0.213399982544034971798652122743078507483005523681640625f,-0.44509999690353863055491956401965580880641937255859375f,0.2515999928876759828000331253861077129840850830078125f,0.13349999006465085304995454862364567816257476806640625f,-0.7419999719336629606658561897347681224346160888671875f,-0.43359999909400925854896513556013815104961395263671875f,-0.055599988722800919305200295639224350452423095703125f,-0.440499998524785052467223067651502788066864013671875f,0.538500033520161736078080139122903347015380859375f,0.42360001570880412291586480932892300188541412353515625f,1.2028000031344594500382072510547004640102386474609375f,0.203299996518343684126506332177086733281612396240234375f,0.6396000204324721050141988598625175654888153076171875f,0.146200005850940950580252319923602044582366943359375f,0.3575000001490113987756558344699442386627197265625f,0.870900026148557770966363023035228252410888671875f,0.84660001170039134166245275991968810558319091796875f,0.91409998964667327214783654198981821537017822265625f,-0.072600001895427368481250596232712268829345703125f,-0.0679000151813031749270521686412394046783447265625f,-1.22139995292425140860359533689916133880615234375f,-1.02880000631213164297150797210633754730224609375f,-0.290999977827072431324495482840575277805328369140625f,-0.3287000155925750011220998203498311340808868408203125f,0.7231000124156476971393203712068498134613037109375f,0.48469998760819432703783604665659368038177490234375f,0.9508999726533888807722405545064248144626617431640625f,0.95179999060630759988299587348592467606067657470703125f,0.44360001347363020585135018336586654186248779296875f,0.45330002112388623203287352225743234157562255859375f,-1.23470001487731906308908946812152862548828125f,-0.1516999988436698032501226407475769519805908203125f,-0.067799997565149983103083286550827324390411376953125f,0.43620002423524883994332412839867174625396728515625f,0.820200029532610841442874516360461711883544921875f,-0.73600000107288376938186047482304275035858154296875f,0.3480999880522490652623446294455789029598236083984375f,-1.2219999848604203851465399566222913563251495361328125f,0.734799980376660499814533977769315242767333984375f,-0.072200004446506316213572063134051859378814697265625f,0.003700012984871658971997021581046283245086669921875f,0.893900044119357861660546404891647398471832275390625f,-1.0161999910056593865448348878999240696430206298828125f,0.589100045078992717861865457962267100811004638671875f,-0.5533999842762946652641176115139387547969818115234375f,-1.1039999937862159384849292109720408916473388671875f,-0.0233999833822249314607688575051724910736083984375f,-1.6723000200510025337052866234444081783294677734375f,0.266900003081560033280084098805673420429229736328125f,0.5595999857127658128064240372623316943645477294921875f,0.6965000117793678224842324198107235133647918701171875f,-1.05360000758767125006443166057579219341278076171875f,-0.5419999977126714885145020161871798336505889892578125f,-1.206199991154670581039454191341064870357513427734375f,-0.2051999906405808093978748729568906128406524658203125f,-0.6920999636590483561349174124188721179962158203125f,0.2177000174999239590079014305956661701202392578125f,0.8329000233769416894347159541212022304534912109375f,0.1772999982208014468909595962031744420528411865234375f,1.0759000020086764148885549730039201676845550537109375f,0.3324000016063453077919120914884842932224273681640625f,-0.56300002080202116161444791941903531551361083984375f,-0.22540000285059225149097983376123011112213134765625f,-0.622200002509355698521176236681640148162841796875f,-0.320800002311170118218797142617404460906982421875f,-1.0641000428810720723760141481761820614337921142578125f,0.84659995731115333938987532746978104114532470703125f,1.701799973246454822373152637737803161144256591796875f,1.813399966096878035415329577517695724964141845703125f,0.70759998636096810287199332378804683685302734375f,0.69360002695918065995783763355575501918792724609375f,0.0730999722570182530745341864530928432941436767578125f,0.34769997287094600579848702182061970233917236328125f,0.1473999798133973637703775239060632884502410888671875f,1.8768999864667659238648411701433360576629638671875f,0.6160999939531082159049901747494004666805267333984375f,0.4836999615013599651547337998636066913604736328125f,0.376600003243982683898138930089771747589111328125f,-0.814900006896257078636836013174615800380706787109375f,-0.253800031197072062383313095779158174991607666015625f,-0.7840000288784505499961596797220408916473388671875f,-0.395199992167949432086970773525536060333251953125f,0.0076000208199022978305947617627680301666259765625f,-1.08699997726082830951099822414107620716094970703125f,-0.87919995384216331757443185779266059398651123046875f,0.271599960291386111066458397544920444488525390625f,-0.599899980372190722022196496254764497280120849609375f,0.3403999940216539954462859896011650562286376953125f,-0.94999996647238749147845737752504646778106689453125f,0.02649997143447446745767592801712453365325927734375f,-0.33339999656975283226501005628961138427257537841796875f,0.3203999832183124230056137093924917280673980712890625f,0.376199986013770359960517453146167099475860595703125f,-0.8208999789118767154150191345252096652984619140625f,-0.06220000876784315124723434564657509326934814453125f,0.026300050121545570647185741108842194080352783203125f,-0.690100011283159009423116003745235502719879150390625f,0.1047999963581560667336134429206140339374542236328125f,0.94160001557469374944275841698981821537017822265625f,0.21239999870061865294701419770717620849609375f,0.08419998450130206091301943160942755639553070068359375f,-0.2041999638259408555995833012275397777557373046875f,-0.5442000070765613539691685218713246285915374755859375f,0.70950000259280177061782524106092751026153564453125f,0.0997999891310927811360897976555861532688140869140625f,0.218500013589859154450323330820538103580474853515625f,0.4075000189244748494132863925187848508358001708984375f,-0.482400013005733541149311349727213382720947265625f,-0.5964000148385764088487803746829740703105926513671875f,-0.340900010055303681610894273035228252410888671875f,-0.029900007231533631824049734859727323055267333984375f,-0.2315000083148479603778469027020037174224853515625f,0.037200007724762063077150742174126207828521728515625f,0.06009997321069204812005182247958146035671234130859375f,-0.466800018471479394310819088786956854164600372314453125f,0.8120000171214343254177947528660297393798828125f,-0.7377000089317558018109366457792930305004119873046875f,0.114599981465935751856477509136311709880828857421875f,1.04740005830526428098892210982739925384521484375f,0.50490002023279689780110857100225985050201416015625f,0.29409997668266230075317935188650153577327728271484375f,-0.245600044602155787032415901194326579570770263671875f,0.534600028702616913989231761661358177661895751953125f,1.903300011438131544849738929769955575466156005859375f,1.85260001184195299828161296318285167217254638671875f,-0.437799983707070072824762974050827324390411376953125f,-0.25340003024637713924249737829086370766162872314453125f,-1.43520002189576612039445535629056394100189208984375f,-1.2425000226870179798055460196337662637233734130859375f,-0.4185000085234638067532841887441463768482208251953125f,-0.4172000378251083940739363242755644023418426513671875f,-1.494200032967329061506234211265109479427337646484375f,-1.103599977487325300984366549528203904628753662109375f,0.793499983340501646722486839280463755130767822265625f,-1.4218999965250487260703948777518235146999359130859375f,-0.380100010463595427978589214035309851169586181640625f,0.37150003387033925239535392393008805811405181884765625f,-0.573100001612305565146243679919280111789703369140625f,-1.033699994432926150267348930356092751026153564453125f,0.219500010937452305626038651098497211933135986328125f,-0.373600002074241377414409726043231785297393798828125f,-1.489300001484155533404418747522868216037750244140625f,0.5655000173747541936108973459340631961822509765625f,-0.979300003495812365628125917282886803150177001953125f,0.71589999060928832808059496528585441410541534423828125f,0.726800008413195808287809995817951858043670654296875f,0.6442000115841626239898687344975769519805908203125f,0.4073999911755326053253156715072691440582275390625f,-0.26139999740421782092880675918422639369964599609375f,1.221000000253319672793850259040482342243194580078125f,-1.2030000208020208418702168273739516735076904296875f,0.4928000364571814007064176621497608721256256103515625f,-0.7503000348925590312632039058371447026729583740234375f,0.081800026071072284850060896133072674274444580078125f,-1.1813000373065474235545480041764676570892333984375f,0.1150000087544322813215558198862709105014801025390625f,-0.7260000171661376011655875117867253720760345458984375f,0.5785000246912244126207269800943322479724884033203125f,-0.2807000166952615671789317275397479534149169921875f,-0.568600014665722852669205167330801486968994140625f,-0.36859998300075513810725169605575501918792724609375f,0.04930002081841189376376632935716770589351654052734375f,0.1119999841153624942080568871460855007171630859375f,-0.581700004021822980604383701574988663196563720703125f,0.9964999719560143631724713486619293689727783203125f,-0.18900000451505183374223406644887290894985198974609375f,0.73950002211332321877534923260100185871124267578125f,-0.2779999924153084833733373670838773250579833984375f,-0.172399962639808723707801618729718029499053955078125f,0.0533000033915040916099314927123486995697021484375f,-0.4510999348700044375704010235494934022426605224609375f,0.35770002875029993649746984374360181391239166259765625f,-0.669699977326393192100795204169116914272308349609375f,-0.3990999978423117600101477364660240709781646728515625f,1.0521000134289268945764206364401616156101226806640625f,1.97160000994950390662552308640442788600921630859375f,-0.21320000045299547508648174698464572429656982421875f,0.3137000006914145711078845124575309455394744873046875f,0.46990000972747747454150157864205539226531982421875f,1.02199994920939207787569102947600185871124267578125f,-0.69809998022913877235851032310165464878082275390625f,0.54390009102076286584548370228731073439121246337890625f,0.4187000085890293377133275498636066913604736328125f,0.22280000076889994176099207834340631961822509765625f,-1.22780000926256160909133541281335055828094482421875f,0.653099980825185610910921241156756877899169921875f,-0.184499996632336848989552890998311340808868408203125f,-0.04539999759793289513254421763122081756591796875f,-0.9916000227272505629372290059109218418598175048828125f,0.399999984949827247504572369507513940334320068359375f,-0.193599998646974302829448788543231785297393798828125f,-0.939299960654974341878187260590493679046630859375f,-0.7574000046610829262050401666783727705478668212890625f,-0.24479998920559875585922782192938029766082763671875f,-1.271399970358610520548836575471796095371246337890625f,0.26610001488923984425127855502068996429443359375f,-0.68979996715188018896469657192938029766082763671875f,-0.89699999722838352766984826303087174892425537109375f,0.061899939304590478883483228855766355991363525390625f,0.259600008958578154505403290386311709880828857421875f,-0.0021999923765658735419492586515843868255615234375f,-0.003099993044137772102430972154252231121063232421875f,0.4666000111043453468795405569835565984249114990234375f,0.9726000047266480930119314507464878261089324951171875f,-0.045199994738399862370670234668068587779998779296875f,0.3058999911308288854883130625239573419094085693359375f,-1.03359999499619004126316212932579219341278076171875f,0.920899978166819010994004202075302600860595703125f,-0.2685000104606147797170478952466510236263275146484375f,2.0335999926865095943639971665106713771820068359375f,-0.16270000248700411393798503922880627214908599853515625f,0.94749999664723916215081089831073768436908721923828125f,-0.355599983842670386291473505480098538100719451904296875f,1.44019998330175891254611997283063828945159912109375f,-1.070100021639465825984416369465179741382598876953125f,0.45779998199343641118730374728329479694366455078125f,0.0780000017657875854837357110227458178997039794921875f,0.3640000123381614827167140902020037174224853515625f,0.7549000439628965608562793931923806667327880859375f,1.59199999033659711500376943149603903293609619140625f,0.7955999966949216517519971603178419172763824462890625f,1.041099992090463555172163978568278253078460693359375f,0.6557000134915114131217706017196178436279296875f,-0.6775999531686311971867553438642062246799468994140625f,-0.5277999945104117074379246332682669162750244140625f,-0.799399997866153011472079015220515429973602294921875f,-0.502899940712751458704588003456592559814453125f,0.808799963396787280345279214088805019855499267578125f,0.216900001405180287150642470805905759334564208984375f,0.48840003836750967014523894249578006565570831298828125f,0.48850008645653753003301744683994911611080169677734375f,-1.14139994115233367466544223134405910968780517578125f,-0.8303999545708304896862728128326125442981719970703125f,-0.37660000126957893229473484097979962825775146484375f,-0.04179999462962147305233884253539144992828369140625f,-0.4770000114217400888350084642297588288784027099609375f,-0.044199990610778616684228836675174534320831298828125f,0.5161999971523882901891511210124008357524871826171875f,-0.50609999685883533260266631259582936763763427734375f,-0.33090001139640801142149939551018178462982177734375f,0.39810000436902015508167096413671970367431640625f,-0.1585000174641606474068566967616789042949676513671875f,-0.45089997876286513900367936003021895885467529296875f,1.985999942362308701149231637828052043914794921875f,-0.16230001386702053167709891567938029766082763671875f,0.3059999956339598536914081705617718398571014404296875f,-0.67070000946819785525576662621460855007171630859375f,0.49929994404017985498711595937493257224559783935546875f,0.05890002087354684334741250495426356792449951171875f,-0.2788000604033467055842265835963189601898193359375f,0.11529999898076059849927332834340631961822509765625f,0.48630000839829445880013736314140260219573974609375f,-0.7426000056207182620227058578166179358959197998046875f,0.32219998980611530559059474398964084684848785400390625f,-0.6851999927639962351122449035756289958953857421875f,0.6461000144422051061354750345344655215740203857421875f,0.41060001301169446463745771325193345546722412109375f,-0.9132000312611456838141066327807493507862091064453125f,-0.57759999355077784599643564433790743350982666015625f,0.116900020888447642164464923553168773651123046875f,0.92320002161264458351297435001470148563385009765625f,0.05489999884963037146690112422220408916473388671875f,0.61540000162124641747141140513122081756591796875f,-0.22919999295771120717546409650822170078754425048828125f,0.12859996660947814461906091310083866119384765625f,0.00119999935030978832628534291870892047882080078125f,0.86150000861287079345629535964690148830413818359375f,0.17220000358968956533090022276155650615692138671875f,-0.5830000029206274803073029033839702606201171875f,-0.6083000328585510541046232901862822473049163818359375f,-0.5336000434994694341384047220344655215740203857421875f,-0.57590004976689812554724312576581723988056182861328125f,0.8451999580442901560672908090054988861083984375f,0.2979999984875332241784917641780339181423187255859375f,-1.43899996092915483103524820762686431407928466796875f,-0.69239996591806463044349584379233419895172119140625f,0.02750000715255662697700245189480483531951904296875f,-0.691200055192410456328389045665971934795379638671875f,-0.84720000653266946955000094021670520305633544921875f,-1.092199978406727556290434222319163382053375244140625f,0.4297000205397605299140195711515843868255615234375f,0.8318000397056344308310826818342320621013641357421875f,0.44750001341104539420712171704508364200592041015625f,0.1086000390291206318948979969718493521213531494140625f,0.85929999552369107362892464152537286281585693359375f,0.50919997515082382477658029529266059398651123046875f,-0.49879999454021461957609062665142118930816650390625f,0.6700999920606616200302596553228795528411865234375f,0.1879000294864174858133765155798755586147308349609375f,0.082699962365627488480868123588152229785919189453125f,0.86240001479387284888389331172220408916473388671875f,-0.43669996650814990601929821423254907131195068359375f,-0.80490003431439394177004942321218550205230712890625f,1.3537999835133547321674996055662631988525390625f,1.07310005071163150347501868964172899723052978515625f,-0.3317000091254715954391940613277256488800048828125f,1.645399987614154735382498984108678996562957763671875f,-0.72030000523924808675246822531335055828094482421875f,-0.625300040853023375575503450818359851837158203125f,-0.26079998863935482944498289725743234157562255859375f,0.54419997902512573517697092029266059398651123046875f,0.04639999762773516778224802692420780658721923828125f,-0.80939998698830617485100447083823382854461669921875f,0.7291999767899515205726856947876513004302978515625f,-0.5226000060304991645665495525463484227657318115234375f,0.14650001449882943393987488889251835644245147705078125f,-0.758400035499037006303524321992881596088409423828125f,-0.28609999022781806843340746127068996429443359375f,0.77820004236251183460382208068040199577808380126953125f,0.2149999834597107195577336824499070644378662109375f,0.51570001341700544283952467594644986093044281005859375f,-0.7691000165432690938160931182210333645343780517578125f,0.245200000102818116420877458949689753353595733642578125f,-1.110900001598894792920191321172751486301422119140625f,0.3736999847099180538378959681722335517406463623046875f,-0.8462999995321036639239764554076828062534332275390625f,0.4655999956145879803415255082654766738414764404296875f,-0.482599958533048489783823242760263383388519287109375f,-0.2028000027805567651029150511021725833415985107421875f,0.35349999898672113118891502381302416324615478515625f,-0.2086999862000340488776828351547010242938995361328125f,-0.6716999495208260118062071342137642204761505126953125f,-1.1534999752938752681785672393743880093097686767578125f,-2.24860002040267037415333106764592230319976806640625f,-0.781299998787044813752800109796226024627685546875f,0.99150002202391629424482744070701301097869873046875f,0.54150000697374256475313814007677137851715087890625f,0.897599991278350106682637488120235502719879150390625f,-0.29790004602670683286902431063936091959476470947265625f,-0.1642999977216124118939433174091391265392303466796875f,-0.0212999975204468672274060736526735126972198486328125f,0.175899970939755956322869678842835128307342529296875f,-0.7747000523164866425673835692577995359897613525390625f,0.21159996133446679067446893895976245403289794921875f,0.9758999996244910324350030350615270435810089111328125f,0.566199991452693840443544104346074163913726806640625f,0.196599987858534053231096550007350742816925048828125f,1.0351999643027784969717686180956661701202392578125f,0.3801000138908623426914346055127680301666259765625f,0.330099969485402322533218466560356318950653076171875f,-0.58999998413026322197083572973497211933135986328125f,-0.06879999446570883581131283790455199778079986572265625f,0.323999994754790687778722713119350373744964599609375f,0.49680001102089921261040217359550297260284423828125f,0.5045999835893508755901848417124710977077484130859375f,-0.0776999851271511499106736664543859660625457763671875f,-0.35870000095218423918908001724048517644405364990234375f,-0.44329999724477542333289648013305850327014923095703125f,1.0697999739319092871170369107858277857303619384765625f,-0.01969999289810697717939547146670520305633544921875f,-0.980899983233213479394407841027714312076568603515625f,-0.1728999671995641307375990436412394046783447265625f,0.1289000016510486457121942294179461896419525146484375f,-0.309699992823600833702357704169116914272308349609375f,1.0011999932408333080502416123636066913604736328125f,-0.24560000977069107452877005925984121859073638916015625f,0.46930003024339672634823728003539144992828369140625f,-0.11140001923441900100186785493860952556133270263671875f,-0.93769998505264517962842774068121798336505889892578125f,0.117400017327070005368483407437452115118503570556640625f,-0.149900031390040988643619357389979995787143707275390625f,-0.52249999850988348981672970694489777088165283203125f,0.589499987989663853937827298068441450595855712890625f,-0.32079999601542941167764411147800274193286895751953125f,-0.155400010785460285234194088843651115894317626953125f,0.7544000214844943652536812805919907987117767333984375f,0.426800006997585246182325136032886803150177001953125f,0.094299998168647147789300788645050488412380218505859375f,-1.3986000267915426054088356977445073425769805908203125f,0.4662000310152774051886126471799798309803009033203125f,-0.651500028450042290018018320552073419094085693359375f,-0.013899983024596718195198263856582343578338623046875f,-0.2782999914705754918031743727624416351318359375f,-1.44500000096857572629005517228506505489349365234375f,-1.1241000060938299842661081129335798323154449462890625f,-0.5255999925784766713121598513680510222911834716796875f,0.66000000180676565708637326679308898746967315673828125f,0.8141999623730777901897681658738292753696441650390625f,-0.9731999815657734753671093130833469331264495849609375f,-0.31409997455924809628413640893995761871337890625f,-0.630399979232251883587423435528762638568878173828125f,-0.01779999123215636469552691778517328202724456787109375f,1.0840000044032931381110529400757513940334320068359375f,0.731300029110908411666969186626374721527099609375f,0.453600004160404191022593067827983759343624114990234375f,0.18609998264685267965745651963516138494014739990234375f,0.71270003012865801306219282196252606809139251708984375f,0.70760000275224388399664121607202105224132537841796875f,1.079499965637922276329163651098497211933135986328125f,0.46370000143647160939508466981351375579833984375f,1.0270999961435796077324766883975826203823089599609375f,-0.64259999010488400017493404448032379150390625f,-0.1601999930806460581411698740339488722383975982666015625f,0.285000002030283194898174770059995353221893310546875f,-0.714599991952627799918218443053774535655975341796875f,-0.27089997578263325106462389157968573272228240966796875f,0.28659998744875225806794105665176175534725189208984375f,0.026199997226893867452446329480153508484363555908203125f,-0.4395999954357743266797342585050500929355621337890625f,-0.19290001868307593557716472787433303892612457275390625f,0.6472999984815717144925883985706605017185211181640625f,-0.132200000702589692824773237589397467672824859619140625f,-1.282100006555765503435395658016204833984375f,0.168399998823553165294697464560158550739288330078125f,-0.59820000516548710312036973846261389553546905517578125f,0.0713000115275386203705920706852339208126068115234375f,0.06600000893324631245917544219992123544216156005859375f,0.50649998760223358118537362315692007541656494140625f,-0.916300010074674986526588327251374721527099609375f,0.24310000734925285570398045820184051990509033203125f,-0.1031999985158444932409338434808887541294097900390625f,1.48549997371435171800158059340901672840118408203125f,0.133100004070997590677194466479704715311527252197265625f,0.655400019949674561559049834613688290119171142578125f,0.10040000199377541179046602337621152400970458984375f,-1.07100002908706670723404386080801486968994140625f,0.26300003156811035953666078057722188532352447509765625f,-0.573200025486946262986975852982141077518463134765625f,-0.62060000526309011537051674167742021381855010986328125f,0.5698000053733587311199926261906512081623077392578125f,-0.39300002255290789054953393133473582565784454345703125f,1.21800003123283406836208087042905390262603759765625f,0.9678000538915403350159749606973491609096527099609375f,1.3886000311315063182604490066296420991420745849609375f,0.5871999987095588569019355418276973068714141845703125f,-1.41959997647404723153385930345393717288970947265625f,-0.91510001757889991580441346741281449794769287109375f,-0.61810001137256609382575334166176617145538330078125f,-1.0991999878913161214910587659687735140323638916015625f,-0.230499999642372443986459984444081783294677734375f,-0.7565999738886957093342289226711727678775787353515625f,0.29360000222325322027927541057579219341278076171875f,0.597700013849138844790331859258003532886505126953125f,1.220900034791231103525888102012686431407928466796875f,1.412299988833069175342416201601736247539520263671875f,-0.310899971312284417734872477012686431407928466796875f,0.555400019428134328336454927921295166015625f,-0.6657000187814237079919621464796364307403564453125f,-0.46919999720454175218264936120249330997467041015625f,-0.17380000288486474602223097463138401508331298828125f,0.33420000435709962260943939327262341976165771484375f,-0.2868000262200831596004491075291298329830169677734375f,0.1205000291466706840992628713138401508331298828125f,-0.08219998492598534767239470966160297393798828125f,0.79759996166229285563531448133289813995361328125f,1.8191999645709990840458658567513339221477508544921875f,-0.3435999938786036267401868826709687709808349609375f,1.41069999449253113965596639900468289852142333984375f,-0.9029999901056291466971970294252969324588775634765625f,-1.037000012874603083190550023573450744152069091796875f,-0.0733999852448705691898567238240502774715423583984375f,-1.36419998736977543529746981221251189708709716796875f,-0.17729998771548249880680714340996928513050079345703125f,-0.59550000634789501674504208494909107685089111328125f,0.10800000113248842126267845742404460906982421875f,0.893400008863210359777440316975116729736328125f,-0.19340000185966488288613618351519107818603515625f,1.222700006435811559413195936940610408782958984375f,-0.480200008223950813857072716928087174892425537109375f,0.50030001075267793719802966734278015792369842529296875f,-1.411100027834624182077050136285834014415740966796875f,0.3397000254943962627152131972252391278743743896484375f,-0.45069997633174063711436474477523006498813629150390625f,-0.9388000100746756171332663143402896821498870849609375f,0.2953000606715676656932600963045842945575714111328125f,0.29350001567602135299495103026856668293476104736328125f,2.07820000492334333586086358991451561450958251953125f,0.4982000034332274562842712839483283460140228271484375f,1.846500006377696845305536044179461896419525146484375f,-0.569399987323582212894734766450710594654083251953125f,0.480299983185529466567231793305836617946624755859375f,0.5474999862909317727144298260100185871124267578125f,-1.2615999966502187135120038874447345733642578125f,0.341900000399351089441779549815692007541656494140625f,-1.0056000304833059288966978783719241619110107421875f,-0.239199990312755128574195850887917913496494293212890625f,-1.5357999876521528648254388826899230480194091796875f,-0.695600002599507494238650906481780111789703369140625f,-0.28169996218681314648080160623067058622837066650390625f,-1.0045000020153818098833653493784368038177490234375f,0.860000036954879032435883345897309482097625732421875f,-0.0721999693915245099873345679952763020992279052734375f,1.205699975568056192543053839472122490406036376953125f,0.5013000072807074491976209174026735126972198486328125f,0.8925999948918814652643050067126750946044921875f,-0.1550999897882341382882032121415250003337860107421875f,-0.18350002402067122631024176371283829212188720703125f,0.0889999894648789702245039734407328069210052490234375f,-1.044199968296289693370226814295165240764617919921875f,-0.39449999361485243998259875297662802040576934814453125f,0.11759997686147727335992385633289813995361328125f,0.36329999921917899285972453071735799312591552734375f,-0.43460002356171489878988722921349108219146728515625f,-0.044799998518824768467538888216949999332427978515625f,0.5208000294685373177117071463726460933685302734375f,-0.4891000163197514183366365614347159862518310546875f,-0.4213000100374220124876956106163561344146728515625f,-0.6359000297248365018987215080414898693561553955078125f,-0.2939999709129337279733817922533489763736724853515625f,1.0532000002294779061884355542133562266826629638671875f,1.04710001272112140924264167551882565021514892578125f,-0.1722999930053947803543223926681093871593475341796875f,-0.2781999972864983572407027168083004653453826904296875f,0.7930999982595441366584054776467382907867431640625f,0.950899996346235010236114248982630670070648193359375f,0.3259000189960004068012722200364805757999420166015625f,-0.6623000294387342279378572129644453525543212890625f,-0.215999979615211668004803868825547397136688232421875f,-0.2756999730348592692052989150397479534149169921875f,1.131000042796134419376130608725361526012420654296875f,0.783400023913383591889214585535228252410888671875f,1.432399987226724125122245823149569332599639892578125f,0.256200012426078149729846700211055576801300048828125f,-1.075700027871131947421190488967113196849822998046875f,-0.51750002440065168141103413290693424642086029052734375f,-1.3161999674618243272306017388473264873027801513671875f,-0.064299992245435699800282236537896096706390380859375f,0.0825000142678617987002098743687383830547332763671875f,0.2236000066936016938967668465920723974704742431640625f,-0.6030000113770366265697475682827644050121307373046875f,-0.331599998699128428736315754576935432851314544677734375f,0.2220999997198582764923457943950779736042022705078125f,-0.55029999778866756088291367632336914539337158203125f,-0.82930000744461995765277606551535427570343017578125f,-0.2834999963045117254978322307579219341278076171875f,1.6522999676734204665962124636280350387096405029296875f,0.406200021627545648783552678651176393032073974609375f,-0.1512000474810601957642575143836438655853271484375f,-0.121799997870624121532046046922914683818817138671875f,-1.629699982541799041513286283588968217372894287109375f,-0.54350001206248987895008895065984688699245452880859375f,-0.546499951466917988085469914949499070644378662109375f,0.089300007444620188579165187547914683818817138671875f,0.30079996416419707561118457306292839348316192626953125f,0.72500000692903998622540484575438313186168670654296875f,-0.170999997496604938529429773552692495286464691162109375f,0.16749999426305295369132863925187848508358001708984375f,0.4230999844014646793510792122106067836284637451171875f,-0.059900010956824012442467619621311314404010772705078125f,0.07120000396966918998487017233856022357940673828125f,-0.1063000003516674230041871851426549255847930908203125f,-0.921800004315375787200537160970270633697509765625f,-1.53830004064440739597330320975743234157562255859375f,0.907899993574620367553507094271481037139892578125f,-0.0694999951422214934382282081060111522674560546875f,1.22040002069473274559641140513122081756591796875f,0.50219998745918292826218021218664944171905517578125f,0.966399964845181091988024491001851856708526611328125f,1.0240999827921390963325620759860612452030181884765625f,-0.3154000253140925025263641146011650562286376953125f,-0.33010000070333500588048991630785167217254638671875f,-0.18899998968839692992105483426712453365325927734375f,0.481200022149085970823989555356092751026153564453125f,0.665400002813338886653582449071109294891357421875f,-0.697700008410215399834441996063105762004852294921875f,-0.30560002120733287256371113471686840057373046875f,0.847100015366077485623463871888816356658935546875f,0.5346000100016592426754868938587605953216552734375f,0.7892000094234941087734114262275397777557373046875f,-0.768799995173514272295278715319000184535980224609375f,1.49090000185966431445194757543504238128662109375f,-0.175499996215104925312289196881465613842010498046875f,0.599800012898445089604138047434389591217041015625f,-0.356199966120720190598802901149610988795757293701171875f,-0.332200002807379046743108119699172675609588623046875f,-0.1095999769955877667371169081889092922210693359375f,-0.485599955344200306939228539704345166683197021484375f,-0.94800003112107500413685556850396096706390380859375f,0.41800003704428689843553001992404460906982421875f,0.337300004293024979773463201127015054225921630859375f,0.30179996616840387702040970907546579837799072265625f,-1.1255000256448983719792522606439888477325439453125f,-1.094000022321939269431823049671947956085205078125f,0.614000006675720300108878291212022304534912109375f,0.3302000034928322946825574035756289958953857421875f,-0.193399979805946031774510629475116729736328125f,0.7353999929785732803111386601813137531280517578125f,0.56130000199079521649991875165142118930816650390625f,0.45959998422264991546626333729363977909088134765625f,0.7612000030383467841232913997373543679714202880859375f,0.1494999745786191880370097351260483264923095703125f,0.4213999834716322112626585294492542743682861328125f,0.5789000155091290622522137709893286228179931640625f,-0.3827000069946049354285833032918162643909454345703125f,0.90780001641511898213821041281335055828094482421875f,0.3892000120311973621056722549838013947010040283203125f,-0.20100001597404482112096957280300557613372802734375f,-0.3238000120490791555738496754202060401439666748046875f,-0.19589998382925966069478818099014461040496826171875f,0.33659999944418661499412337434478104114532470703125f,0.319399982070922572319204846280626952648162841796875f,0.34830000324249266174803096873802132904529571533203125f,-0.543899998000263895647776735131628811359405517578125f,0.078699972192942890814038037206046283245086669921875f,0.899800013047457269976803218014538288116455078125f,0.2795000134706497885161979866097681224346160888671875f,-0.9966999752998353390154306907788850367069244384765625f,0.14520000617504125184353824806748889386653900146484375f,-0.89179998956322659164897004302474670112133026123046875f,-0.153599991084635234539490511451731435954570770263671875f,-0.644699969540536432788258025539107620716094970703125f,-1.117799992424249211353526334278285503387451171875f,-1.3071000105977059746464874478988349437713623046875f,-0.6647000364094974855788677814416587352752685546875f,0.955000017359852737541814349242486059665679931640625f,0.349299990457296427592837062547914683818817138671875f,0.1134000135943293852136548593989573419094085693359375f,0.3589000039421021970298397718579508364200592041015625f,-0.753199984136224021114003335242159664630889892578125f,0.5595000036358832584681977095897309482097625732421875f,0.07150001349300116881835265303379856050014495849609375f,0.5863999977767464510947093003778718411922454833984375f,0.25450001022964718888630386572913266718387603759765625f,-1.3790999833881854019779211739660240709781646728515625f,0.5153000048667191190787661980721168220043182373046875f,-0.31859999283552176141398604158894158899784088134765625f,1.0391999923244121806220618964289315044879913330078125f,0.228899981944262975730453035794198513031005859375f,1.0999999944493172865378483038512058556079864501953125f,0.34789998105764396996164578013122081756591796875f,-0.378100016885996037530048852204345166683197021484375f,0.96700002047419530981642310507595539093017578125f,0.235000007264316135291437603882513940334320068359375f,-1.141600012594461421855385196977294981479644775390625f,0.48759999492913486118794708090717904269695281982421875f,-1.12800000016391255286407613311894237995147705078125f,0.0570000160038472980517099131247960031032562255859375f,-0.34310001398026912511340924538671970367431640625f,-0.12729999628365040731381441219127736985683441162109375f,-1.031599969753623557977562086307443678379058837890625f,-0.6115999888271093087865892812260426580905914306640625f,-0.4762999961420895100872030525351874530315399169921875f,-0.353699992607533875155212399477022700011730194091796875f,-0.866000012844800881595119790290482342243194580078125f,-0.328900000533461689400382965686731040477752685546875f,-0.50499996572732896993329632095992565155029296875f,-0.629000003397464535481731218169443309307098388671875f,-0.03389999855905745373974014000850729644298553466796875f,0.1948000013872981384110971703194081783294677734375f,-1.18010000590011454590921857743524014949798583984375f,-0.0609000080063939464025679626502096652984619140625f,-0.68659999057799581123617826960980892181396484375f,-0.217299998760968449129649116002838127315044403076171875f,0.856199998903274916983718867413699626922607421875f,1.68269998516440377755998270004056394100189208984375f,-0.13730001051425855251864049932919442653656005859375f,-0.689199998098611654739897858235053718090057373046875f,-0.98070000075101848580061414395458996295928955078125f,-0.634000032007694169777778370189480483531951904296875f,0.19759999742507972086968948133289813995361328125f,0.574300027449428984738233339157886803150177001953125f,1.17860003720372841229391269735060632228851318359375f,-0.74470000219270604535637403387227095663547515869140625f,-0.9197999944396320071149375507957302033901214599609375f,-1.165799998176097584945409835199825465679168701171875f,0.2455999911069870478286247816868126392364501953125f,0.41489998156428331554934629821218550205230712890625f,0.4343999805808067549151019193232059478759765625f,0.69090000051856037543984712101519107818603515625f,0.088900004482269423533580265939235687255859375f,-0.63779999972581880030020329286344349384307861328125f,0.62310003532618285504440791555680334568023681640625f,-1.42050002601742786367822191095910966396331787109375f,-0.36590000483989737478651704805088229477405548095703125f,-1.341699990797042829626661841757595539093017578125f,0.49979998954087445639515863149426877498626708984375f,1.4396000225186345033989709918387234210968017578125f,0.71049998168647354734872578774229623377323150634765625f,2.75580003539174800408773080562241375446319580078125f,0.91249996358528717710356659154058434069156646728515625f,0.480599944708496540357600679271854460239410400390625f,-0.8784999786093832252475976929417811334133148193359375f,-0.157599984312057461721678919275291264057159423828125f,-1.2704999941661956341931727365590631961822509765625f,-0.40419994877576836955768158077262341976165771484375f,0.189500009298324556539228069595992565155029296875f,-0.113699984598159886672874563373625278472900390625f,0.33809999498128906481042577070184051990509033203125f,-0.912400055027007272911987456609494984149932861328125f,-0.32220002083778387369505935566849075257778167724609375f,-0.61459997304678015694889836595393717288970947265625f,0.1740000109970571973150299527333118021488189697265625f,1.35190001097917633643419321742840111255645751953125f,0.6643000045388938179513615978066809475421905517578125f,1.3659999790750438908304431606666184961795806884765625f,0.4601000092156228848949695020564831793308258056640625f,-0.6237999930500979228753521965700201690196990966796875f,-0.6434999890588224413789930622442625463008880615234375f,-0.21559994014501615566814507474191486835479736328125f,-0.5190000006407498034377567819319665431976318359375f,0.7305000075399874504000763408839702606201171875f,0.463300002758205042852068800129927694797515869140625f,0.237599985802173563342876150272786617279052734375f,0.18770000073611736723933063331060111522674560546875f,-0.0652999918282031188709879643283784389495849609375f,0.68130002374649034635467614862136542797088623046875f,1.444999996423721455585109652020037174224853515625f,0.33960003838837149459095599013380706310272216796875f,-1.50020001913905165480400683009065687656402587890625f,-0.117999999716877912447898779646493494510650634765625f,0.8382999975800515812807134352624416351318359375f,0.2089999854415651725503266789019107818603515625f,-0.462199948343634758884945767931640148162841796875f,-0.1154999646991490269698488191352225840091705322265625f,-1.2195999983787542131352665819576941430568695068359375f,-0.9797000194408000961487914537428878247737884521484375f,0.113600021669268524959761634818278253078460693359375f,0.52200002390146238440138404257595539093017578125f,1.0839000470995898695036885328590869903564453125f,-0.40659999322295181656272688996978104114532470703125f,-0.276399986600875280373657005839049816131591796875f,0.12889999539256091765082601341418921947479248046875f,-0.9574999808892601382837028722860850393772125244140625f,0.31939998534917835026902821482508443295955657958984375f,0.5872999870449306936137645607232116162776947021484375f,-0.53340004209279989755287942898576147854328155517578125f,0.1760000005885957730100699336617253720760345458984375f,1.28289998448491093796519635361619293689727783203125f,0.81889999449849160129133451846428215503692626953125f,-1.302400029769539724355809084954671561717987060546875f,-0.2370000110119578895506720073171891272068023681640625f,-1.2164000052645798088946094139828346669673919677734375f,0.08479996085613972223171685982379131019115447998046875f,0.52179997905790809209491953879478387534618377685546875f,1.1531999824970962098547033747308887541294097900390625f,1.3234999861717227087609671798418276011943817138671875f,0.64180001653432849906266710604541003704071044921875f,-0.34160003628730795099954775650985538959503173828125f,0.030400012499093964635221709613688290119171142578125f,0.533599962362646618174721879768185317516326904296875f,0.3996999915197487762696937352302484214305877685546875f,0.2540999912112951175657826752285473048686981201171875f,-0.2444999974519015406571043058647774159908294677734375f,-0.00660002794265768244486025650985538959503173828125f,0.32700001030415304370535523048602044582366943359375f,-1.7391999894186849218158386065624654293060302734375f,-0.95730000466555387195199955385760404169559478759765625f,-0.555199969872087972788676779600791633129119873046875f,-0.25809998492300512129560274843242950737476348876953125f,0.227700002561510128629151950008235871791839599609375f,0.685300006897002322858725165133364498615264892578125f,0.080799968075751404938955602119676768779754638671875f,0.606300023932754950095613821758888661861419677734375f,-0.808000007987022428324053180404007434844970703125f,-0.252800003376603132920763528090901672840118408203125f,-0.382700004088878387165095773525536060333251953125f,-0.7580000205039978400378686274052597582340240478515625f,1.1339000150620943685453312355093657970428466796875f,0.724599994057416818549199888366274535655975341796875f,0.10000001147389436795265282853506505489349365234375f,-0.7236999842256306436638624290935695171356201171875f,-0.925699995610118175903835435747168958187103271484375f,-0.916599989460408881569719596882350742816925048828125f,1.35379999319911004107552798814140260219573974609375f,0.02239998931288678107875966816209256649017333984375f,-0.032700055348872325566844665445387363433837890625f,0.00770002122521429299695228110067546367645263671875f,1.66689998445510934033109151641838252544403076171875f,1.071199981915950782962454468361102044582366943359375f,0.54039997181892385214041496510617434978485107421875f,0.561700004953145981545503673260100185871124267578125f,-0.64379999662637743540472001768648624420166015625f,0.4937000079929827478508741478435695171356201171875f,-0.1948999916970730605925155032309703528881072998046875f,0.76830000249743424234338817768730223178863525390625f,1.0506000003829598643534382063080556690692901611328125f,0.156999977558851089298741499078460037708282470703125f,0.7036999805375934169404672502423636615276336669921875f,0.7304999905526639025765689439140260219573974609375f,-1.162299996916949407221864021266810595989227294921875f,1.2645999974846839908337869928800500929355621337890625f,-0.94530000505298428237210828228853642940521240234375f,0.44349996948242154104491419275291264057159423828125f,-0.89909995045661894597088803493534214794635772705078125f,0.86190001276731464940894511528313159942626953125f,-0.43899999237060549628353101070388220250606536865234375f,-0.8585999986469745604011905015795491635799407958984375f,0.14419999377727510392332987976260483264923095703125f,-0.98939999071359618909582422929815948009490966796875f,0.22180001176595709466710104607045650482177734375f,0.040299977970123190829099257825873792171478271484375f,0.670199996377527806856733150198124349117279052734375f,-0.1752000408947462783970649979892186820507049560546875f,0.199499993465840741446726269714417867362499237060546875f,0.40630002192109826442134590251953341066837310791015625f,-0.18839999273270369162247561689582653343677520751953125f,0.97600000010430765229330063448287546634674072265625f,0.05459998958706846483579511186690069735050201416015625f,1.53289995989799532338793142116628587245941162109375f,0.5459000099062920607906335135339759290218353271484375f,0.8837999793410293847273351275362074375152587890625f,0.604699989098310464186170065659098327159881591796875f,-0.079900021797417952740261171129532158374786376953125f,0.2055999806761741321992076336755417287349700927734375f,-0.2869999926090240904841266456060111522674560546875f,-0.0992999984294176130106279742904007434844970703125f,0.001700035649538023108107154257595539093017578125f,-1.0281999600708484621236493694595992565155029296875f,-1.14679996363520597668639311450533568859100341796875f,0.43720001875162128879281908666598610579967498779296875f,-0.160900061240791814043404883705079555511474609375f,-1.0404999850392340210447628123802132904529571533203125f,0.254199964496493624466211258550174534320831298828125f,0.37299998429417602352486937888897955417633056640625f,0.06979995999932331329773660399951040744781494140625f,-0.98040000758171064187962429059552960097789764404296875f,0.115600000122189072726541780866682529449462890625f,-0.6249000030219555856803026472334749996662139892578125f,0.529600010076164640082652113051153719425201416015625f,0.869600015068054066347258412861265242099761962890625f,-0.0244999975264068581282117520458996295928955078125f,-0.531699999141693080417780947755090892314910888671875f,-0.50470001815557452573557384312152862548828125f,-0.128900018638372415580306551419198513031005859375f,0.1682000139757995338385399008984677493572235107421875f,-1.473200034427642801659885662957094609737396240234375f,-0.4588999793738126609099481356679461896419525146484375f,0.7348999810427425671832679654471576213836669921875f,-1.15959999141246061782339893397875130176544189453125f,0.881800005283951637835571091272868216037750244140625f,0.56349999667704064876261327299289405345916748046875f,1.826799988967179988463840345502831041812896728515625f,-0.2420000100806356524429929777397774159908294677734375f,1.30680003225505370068049160181544721126556396484375f,-1.029500023715197887241856733453460037708282470703125f,0.58780003988444828877391046262346208095550537109375f,0.73530002155601981161936464559403248131275177001953125f,0.2212999847054479829466799856163561344146728515625f,-0.56779999495744704063326935283839702606201171875f,-0.388700012910366154983421438373625278472900390625f,-0.094299982112646230092423138557933270931243896484375f,-1.1944999674260625699417914802324958145618438720703125f,-0.839699999491870574530594240059144794940948486328125f,1.3409999704509976137245530480868183076381683349609375f,-0.1010000110194086975212712786742486059665679931640625f,1.16590000156164119715640481445007026195526123046875f,0.29019999123662720830196803945000283420085906982421875f,1.4323999621927747671890074343536980450153350830078125f,0.228899998559057793467985675306408666074275970458984375f,0.6569000581413508843553472615894861519336700439453125f,-0.38089999344050884388934719027020037174224853515625f,0.64730002765059413416537381635862402617931365966796875f,0.41950001473724840561629889634787105023860931396484375f,0.0341999677002429169903052752488292753696441650390625f,-0.356800002489984058318128745668218471109867095947265625f,-0.19059999244809144869350348017178475856781005859375f,-0.2630000027343630808474017612752504646778106689453125f,0.343100013943016701745136742829345166683197021484375f,-0.94069997132122507021989576969644986093044281005859375f,-1.2609000064790250039692409700364805757999420166015625f,-1.7334000239506366369823808781802654266357421875f,-1.230899995098262866832783402060158550739288330078125f,-1.8294999757334593493141028375248424708843231201171875f,-0.7211000131383540345808569327346049249172210693359375f,0.55020003329515443368791238754056394100189208984375f,0.3572999891683450446322467541904188692569732666015625f,1.09679996765851939244384993799030780792236328125f,0.02389994145035789774311751898494549095630645751953125f,-0.1819999848604201275747982435859739780426025390625f,-0.2776000297605996092187297108466736972332000732421875f,0.4545999858617779665337366168387234210968017578125f,0.485199959646166245619269830058328807353973388671875f,-1.185599990361928757209852847154252231121063232421875f,-0.926999957889318881854023857158608734607696533203125f,0.38899998939037327971846025320701301097869873046875f,0.700900009123980982650436999392695724964141845703125f,0.38459998165220010690745766623876988887786865234375f,0.0194999992214144324265134855522774159908294677734375f,1.30299999843165270618783324607647955417633056640625f,-0.0746999752588575205436427495442330837249755859375f,-0.738299984448402835113256514887325465679168701171875f,0.4307999941527835829901960096322000026702880859375f,0.3043000369489192724614667895366437733173370361328125f,0.391099966460466408335605592583306133747100830078125f,-0.2252999702215199295807224189047701656818389892578125f,0.069900008088350062962490483187139034271240234375f,-0.981099977338314221952941807103343307971954345703125f,0.481399976468086521208533667959272861480712890625f,-0.297699985462427374471872099093161523342132568359375f,-0.18089999038577087020485123503021895885467529296875f,1.035699970799684788147487779497168958187103271484375f,-0.7669999744296074339899860206060111522674560546875f,0.75020002845227706789188459879369474947452545166015625f,-0.571500002205371782082465870189480483531951904296875f,0.0943999910861248736626549771244754083454608917236328125f,-1.0108000137478112545608155414811335504055023193359375f,-0.404300011728704333791029057465493679046630859375f,0.66940000203847915827282122336328029632568359375f,-0.06249998815357626913424837766797281801700592041015625f,1.0924999988079069357382877569762058556079864501953125f,-0.154399994848668509206390808685682713985443115234375f,0.8324999994412063397675183296087197959423065185546875f,-0.1324000211641195579037599827643134631216526031494140625f,0.6857000181481243128445157708483748137950897216796875f,-0.36410000480860460481125073783914558589458465576171875f,-0.6790000062286853932391750277020037174224853515625f,0.017400000786781222306132121957489289343357086181640625f,0.558000015884637878826879386906512081623077392578125f,0.799000009097158514492775793769396841526031494140625f,1.42070000440180255196764846914447844028472900390625f,0.29569992766082275892358666169457137584686279296875f,-0.318500011578202180118069009040482342243194580078125f,-0.7846000634968277598346730883349664509296417236328125f,-1.418800025385618557294264974188990890979766845703125f,-0.813600007624923904359093285165727138519287109375f,-0.0528999867945909585387198603712022304534912109375f,0.4390000755563374656986752597731538116931915283203125f,1.3125000205636023764554920489899814128875732421875f,0.6868000032350420891447129179141484200954437255859375f,-0.020000031366944337474222948003443889319896697998046875f,-0.089799981009960061495434047174057923257350921630859375f,0.0836000236809255614645053356071002781391143798828125f,0.78220001744776990193486199132166802883148193359375f,0.1135000016689300395000827847979962825775146484375f,0.216999997377395725806081827613525092601776123046875f,-0.994899961447715863727125906734727323055267333984375f,0.97959998370110934473586894455365836620330810546875f,-0.32439994072020095217112611862830817699432373046875f,0.8360999815851453131898551873746328055858612060546875f,-0.2562999914854771077443729154765605926513671875f,-0.047799995255470406618769629858434200286865234375f,0.98070000060200701597068473347462713718414306640625f,-0.9602999921262267424282299543847329914569854736328125f,0.779599973717332250799927351181395351886749267578125f,0.09790000240355734018038447175058536231517791748046875f,-0.2913000132784249185391445280401967465877532958984375f,-0.575399996666610125117813367978669703006744384765625f,0.5803999916002153813820996219874359667301177978515625f,-0.397199988949298532503462411114014685153961181640625f,-1.513300009202957152609769764239899814128875732421875f,0.2525000010430813457418253165087662637233734130859375f,0.217300012600421865727184922434389591217041015625f,-0.823799977850914189048125990666449069976806640625f,1.045700022618472058155703052761964499950408935546875f,-0.82080000279545739516606772667728364467620849609375f,0.194200000482797729972617162275128066539764404296875f,0.2153000036001202488478156737983226776123046875f,-0.27520000826120405434949134360067546367645263671875f,0.49240001360178009548462796374224126338958740234375f,-0.5019999974891542482424711124622263014316558837890625f,-0.4849000276088712979571937466971576213836669921875f,0.80690003329366477569095650324015878140926361083984375f,0.020499969542026263980005751363933086395263671875f,-0.4967000246599317936357920189038850367069244384765625f,-0.77219998999238015358059783466160297393798828125f,0.4112000108987092783507932836073450744152069091796875f,1.282100032055378147077817629906348884105682373046875f,-0.3102000108316540494257651516818441450595855712890625f,0.13150000131130212821517488919198513031005859375f,-0.895300043758750607736374149681068956851959228515625f,1.3255999675631517931151392986066639423370361328125f,0.991299973492325126045443539624102413654327392578125f,1.28730000231862096171653320197947323322296142578125f,0.892400013303756178828507472644560039043426513671875f,0.68849997797608342153807825525291264057159423828125f,0.9510999889612197222277245600707828998565673828125f,-0.48219998403191599578576642670668661594390869140625f,0.23350000174343588099645785405300557613372802734375f,1.34590001705884976246352380258031189441680908203125f,0.775700012634694235913457305287010967731475830078125f,-0.72729998935461015463488365639932453632354736328125f,-0.093899995057284524335727837751619517803192138671875f,0.81339998531937585113382738200016319751739501953125f,0.86710003868639518032068735919892787933349609375f,0.403300011587143014679668340249918401241302490234375f,-0.056599996650219086546940161497332155704498291015625f,0.1375000460445878758264370844699442386627197265625f,-0.2712000337719917553158666123636066913604736328125f,-0.146899967467785330654805875383317470550537109375f,-0.5589999865591526173602687777020037174224853515625f,-0.30040001607537281103077475563623011112213134765625f,-0.758899970209598873083223224966786801815032958984375f,-0.570299981248378884401972754858434200286865234375f,-0.3071999849259856585348416047054342925548553466796875f,-1.134799992334842411167983300401829183101654052734375f,0.5189999685287478570216990192420780658721923828125f,0.1094999829232694565916972351260483264923095703125f,-0.219099986666440482707685077912174165248870849609375f,-1.00260001232624063050025142729282379150390625f,-0.369100013190507780791449476964771747589111328125f,0.9435000073313712931621921597979962825775146484375f,-0.05589999142885204719277680851519107818603515625f,-0.364300015342235550264149424037896096706390380859375f,-0.2874999844282866146016885977587662637233734130859375f,-0.1504999940544369252393153146840631961822509765625f,-0.59630000880807598750976694645942188799381256103515625f,-0.5445000098198651272696224623359739780426025390625f,0.6961999999463557298184923638473264873027801513671875f,-0.1258000008583068929368664612411521375179290771484375f,0.2229999904036521485295452293939888477325439453125f,-0.07560001062750831835046483320184051990509033203125f,0.0720999804973601055024801098625175654888153076171875f,0.1138999898791313025725457919179461896419525146484375f,-0.73750000163912776685037897550500929355621337890625f,0.42800003086030524723781809370848350226879119873046875f,-0.5499000343889000053110294174985028803348541259765625f,-0.57040001454800381974763467951561324298381805419921875f,1.190700001272559038767440142692066729068756103515625f,-0.484200013223290259833930804234114475548267364501953125f,0.909500006709247799108197796158492565155029296875f,-0.530500021398067733713332927436567842960357666015625f,-0.2653999804243449833762724665575660765171051025390625f,0.0505999851092704400201682801707647740840911865234375f,-1.3312999746471643192080591688863933086395263671875f,-0.8124999808892610086985541784088127315044403076171875f,-0.741399998410046112695681586046703159809112548828125f,-0.9891000201940538527622948095086030662059783935546875f,-0.93880001961141790189913081121630966663360595703125f,-1.5194000281900168491944214110844768583774566650390625f,-1.553100034916401028795007732696831226348876953125f,-1.6671000270262366083073857225826941430568695068359375f,0.501200013171136493639323816751129925251007080078125f,0.454699993754922981992194763734005391597747802734375f,-0.540000009331852215410663120565004646778106689453125f,-0.29229998255595546208240875785122625529766082763671875f,-0.75490000017210834304393074489780701696872711181640625f,-0.686199973534047824585968555766157805919647216796875f,0.360600026385485994406820964286453090608119964599609375f,0.11639998708516363645770752555108629167079925537109375f,-0.63489999210685521546793097513727843761444091796875f,0.1314000091016291893453171724104322493076324462890625f,0.673299981859326379662888939492404460906982421875f,0.682099993610381982733770200866274535655975341796875f,0.68589999966174364232074367464520037174224853515625f,0.9572999806001785128017900206032209098339080810546875f,1.0130999798193573990801041873055510222911834716796875f,0.20730000485181765412789900437928736209869384765625f,0.965600013756752417748430161736905574798583984375f,0.13989999344795922553430500556714832782745361328125f,-0.266300005827843910832797291732276789844036102294921875f,-0.18279999378398048559546396063524298369884490966796875f,0.64679997743740680338220272460603155195713043212890625f,-0.086299992211908149730703598834224976599216461181640625f,0.751299982786923781219456941471435129642486572265625f,0.869200020301341691464358518715016543865203857421875f,-1.1287000331014400700269106891937553882598876953125f,1.05000000864267395428441886906512081623077392578125f,0.22680000792890808725132956169545650482177734375f,1.58990002343654612815271320869214832782745361328125f,0.5897000103324649966651804788853041827678680419921875f,-0.71560001554489094388600278762169182300567626953125f,0.4641000177912417701264757852186448872089385986328125f,-1.7245000677108770492651501626824028789997100830078125f,0.06829999880939741085938976539182476699352264404296875f,-0.76869996790885897208767119082040153443813323974609375f,0.09029999968856561309848984819836914539337158203125f,-0.9893999971210956090317267808131873607635498046875f,0.26090003039538889328952109281090088188648223876953125f,0.54489997873306261766401803470216691493988037109375f,-0.1998000029146671518986977389431558549404144287109375f,0.3109999921321868754375827847979962825775146484375f,0.935200001518427637137165220337919890880584716796875f,-0.954100025260448791897260889527387917041778564453125f,-0.58510001504570130492766111274249851703643798828125f,0.18399998760223379434819435118697583675384521484375f,-0.08079998238086771866761637284071184694766998291015625f,1.09169996993541662533289127168245613574981689453125f,0.148800003816187853544761310331523418426513671875f,1.12120003583952865255923825316131114959716796875f,-0.1091000354677447159446046498487703502178192138671875f,-1.417899993183464157908701963606290519237518310546875f,-1.516100003713368504776326517458073794841766357421875f,-0.794099983835220513128660968504846096038818359375f,0.7025000178441402898243950403411872684955596923828125f,1.396200020807981445614132098853588104248046875f,1.46960000892132480743157429969869554042816162109375f,0.30970000265836716835110564716160297393798828125f,0.0465999984383582699365433654747903347015380859375f,0.05750000596046422884910498396493494510650634765625f,0.11440000516772297345369224785827100276947021484375f,-0.054699995356798591927827146719209849834442138671875f,-0.148799989324808645818620789214037358760833740234375f,1.676800006401538922773397644050419330596923828125f,-0.483600028896332023720106008113361895084381103515625f,-0.10109999223947507829279857105575501918792724609375f,1.102999980829656667680183090851642191410064697265625f,-1.264300044604390382829706140910275280475616455078125f,0.62009997781142589534653097871341742575168609619140625f,0.67420002631768582990190452619572170078754425048828125f,-1.042899977034330571967757350648753345012664794921875f,-0.715799994003772699358023601234890520572662353515625f,-0.170799993258714977173440274782478809356689453125f,-0.49829999713301642572105265571735799312591552734375f,0.0629000014722347344786612666212022304534912109375f,-0.152700007069110910151721327565610408782958984375f,-0.4748999647259710688018685686984099447727203369140625f,0.00629998932480757556362505056313239037990570068359375f,-0.140699992108344940078268336947076022624969482421875f,-0.101000023610890110603577340953052043914794921875f,0.2746000455409289831010255511500872671604156494140625f,0.393500018283724717349514321540482342243194580078125f,-0.3375000011175870451296532337437383830547332763671875f,-1.05060003454387196342167953844182193279266357421875f,0.215800007563829776557895456789992749691009521484375f,0.012799999316036869689838795238756574690341949462890625f,0.98750002410262804897200794584932737052440643310546875f,0.62960000597834564484145403184811584651470184326171875f,0.5127000102914873735926448716782033443450927734375f,-0.0706000008672471590642771843704394996166229248046875f,-0.03369998340606716880074600339867174625396728515625f,-0.282000030308962423841734334928332827985286712646484375f,0.189000029474497122095044687739573419094085693359375f,0.5549000399395829230542176446761004626750946044921875f,1.1509000110238793990191652483190409839153289794921875f,1.464499999128282770044506833073683083057403564453125f,-0.500200017946958563896941996063105762004852294921875f,-0.912299997997283451667271947371773421764373779296875f,0.683600012728571737596894308808259665966033935546875f,-0.100500049002468561099021826521493494510650634765625f,-2.39249998953193454553911578841507434844970703125f,-1.21439994854330901574712697765789926052093505859375f,-0.4057999812819061968838241227786056697368621826171875f,-0.4203999817095700475277908481075428426265716552734375f,1.06089997222498055862160981632769107818603515625f,0.11050003343075598394307235139422118663787841796875f,0.6746000418901445527808391489088535308837890625f,-0.5003999981194737767253855054150335490703582763671875f,-0.94660002928376218989114931900985538959503173828125f,-0.764800002765654962644248371361754834651947021484375f,-0.349199979621172218458013958297669887542724609375f,-0.17430000513493926206365358666516840457916259765625f,-1.297000012204051699882256798446178436279296875f,0.3508999908328058747741806655540131032466888427734375f,0.64910001900196057089686973995412699878215789794921875f,0.8749999887496231476546881822287105023860931396484375f,0.017700017276406132538113524788059294223785400390625f,-0.35940001075565752852725154298241250216960906982421875f,0.2789999658092854328828025245456956326961517333984375f,0.369600001712888481808505503067863173782825469970703125f,1.110999994534999313344769689138047397136688232421875f,-0.168700046344846033008479935233481228351593017578125f,1.0096999856896697878028135164640843868255615234375f,0.45590000111460682319375337101519107818603515625f,-0.0033000193357468621258021812536753714084625244140625f,-0.285200001555681037501699393033050000667572021484375f,0.612800004643201390308604459278285503387451171875f,0.00490000659823408568627201020717620849609375f,0.43360000576227886170954661793075501918792724609375f,0.1626999960422514934776927475468255579471588134765625f,-0.37830001814365410428564473477308638393878936767578125f,0.399600001955032535505552004906348884105682373046875f,0.3462999954342842467980290166451595723628997802734375f,-0.7833999953776598079713266997714526951313018798828125f,0.50790000262707479805612820200622081756591796875f,-1.7891999909833078419296725769527256488800048828125f,0.67610001664012653321833568043075501918792724609375f,-0.1592999324917787618005604599602520465850830078125f,0.2769999897032968672050401437445543706417083740234375f,-0.53980000395774840882268108543939888477325439453125f,-0.4548000000089407190984047701931558549404144287109375f,0.84819998912811289670798942097462713718414306640625f,-0.340200003232061620739301588400849141180515289306640625f,0.832500000149011487593497804482467472553253173828125f,-0.370800025407970412771874180180020630359649658203125f,1.077799971860647243460107347345910966396331787109375f,0.6350999871432778665081286817439831793308258056640625f,-0.0695000237524510122710807991097681224346160888671875f,-0.746500009916722273572986523504368960857391357421875f,-0.45110001313835379477268361370079219341278076171875f,-0.9108000027954579191913353497511707246303558349609375f,0.5974000083491211032793444246635772287845611572265625f,1.589900005145370442960484069772064685821533203125f,1.63309999408721839841973633156158030033111572265625f,1.1739999845102442321831404115073382854461669921875f,1.4186999860882760771119137643836438655853271484375f,1.651199972677230842776907593361102044582366943359375f,0.44569999315142627693830945645458996295928955078125f,0.72890000790953657361370687794988043606281280517578125f,-0.28609999268650998516250183456577360630035400390625f,-0.298999979555607087178259462234564125537872314453125f,0.5471000202834606085389168583787977695465087890625f,-0.3693999996542931540943754953332245349884033203125f,0.195999999582767259198590181767940521240234375f,0.25799998936057111365016680792905390262603759765625f,0.2826999924659727714271184595418162643909454345703125f,0.1881999853655695886800458538345992565155029296875f,0.439500004269182842353558271497604437172412872314453125f,-0.93899997631460407898629227929632179439067840576171875f,-1.7774999875202777133154086186550557613372802734375f,-0.01860001831650759385183846461586654186248779296875f,-0.472499917447567696626720135100185871124267578125f,-0.68309998499751056755258105113171041011810302734375f,-1.33550000083446551712995642446912825107574462890625f,-0.221299987834691957999666556133888661861419677734375f,-0.4475000062584875148985474879737012088298797607421875f,0.06410000693202011490257063996978104114532470703125f,0.458499987214803894630676950328052043914794921875f,0.6155000076889993732009997984278015792369842529296875f,-0.43770004983544386067961795561132021248340606689453125f,0.1698000561118124807080675964243710041046142578125f,0.27739999657720282311146320353145711123943328857421875f,-1.17189998244345172651037501054815948009490966796875f,0.9121000111564991907897592682274989783763885498046875f,-0.8880999953538182278833801319706253707408905029296875f,0.43300001391023379415656791024957783520221710205078125f,-1.35340000707507090993431120296008884906768798828125f,0.47630002046823516348439397916081361472606658935546875f,-1.92200003917515260809523169882595539093017578125f,0.713500037357211791544386869645677506923675537109375f,-0.2017000202268366404467769825714640319347381591796875f,-0.523000022105872819366823023301549255847930908203125f,-0.610699987637996599687539855949580669403076171875f,0.234600001470744334852724932716228067874908447265625f,0.0090000117421151770713549922220408916473388671875f,-0.95490000190436763460866131936199963092803955078125f,-2.00440001589655825142699541174806654453277587890625f,-0.722399954183399639617846332839690148830413818359375f,0.4444999830722806688498849325696937739849090576171875f,0.5191999496698385119231033968389965593814849853515625f,1.143899971997738074236394822946749627590179443359375f,1.2115999712809912836064540897496044635772705078125f,0.7145999833285803237714617353049106895923614501953125f,-0.2574000586777918098135842228657566010951995849609375f,-0.3728999737560749760945100206299684941768646240234375f,0.1428000019982456070266607639496214687824249267578125f,1.249300036054850071565169855603016912937164306640625f,1.084800017555057394247342017479240894317626953125f,-0.7818999944388866918387748228269629180431365966796875f,-0.61440002357065648386225120702874846756458282470703125f,-0.23830001382231724704752195975743234157562255859375f,0.10639998853802712375227201846428215503692626953125f,0.51209999524950955418489684234373271465301513671875f,-1.0314999797046180418647054466418921947479248046875f,0.7555999903619283752931323761004023253917694091796875f,0.27789999841749679632130209938623011112213134765625f,-1.207899948051572192042613096418790519237518310546875f,-0.30219999021589760701544946641661226749420166015625f,-0.55189997096955789590566610058885999023914337158203125f,0.7622999735593793957377783954143524169921875f,-0.7848999534010887568769021527259610593318939208984375f,-0.14100001262128358803238370455801486968994140625f,-0.945900005957483802632168590207584202289581298828125f,0.01509999884068946585102821700274944305419921875f,-0.22180001079738109748262786524719558656215667724609375f,0.211700009870529282096640599775128066539764404296875f,-0.6900999871432784704694540778291411697864532470703125f,0.098199992406368341590905401972122490406036376953125f,0.28520001973509767623227162403054535388946533203125f,-0.2472999933779238546094347839243710041046142578125f,0.961099961133301317062205271213315427303314208984375f,-0.2543999935448166827001159617793746292591094970703125f,0.897299984660744343045735149644315242767333984375f,0.16739997545480722607180723571218550205230712890625f,0.5540000147223469451063238011556677520275115966796875f,1.6486000195085999120436781595344655215740203857421875f,-0.333600000397861418122147370013408362865447998046875f,2.04039999923706005802159779705107212066650390625f,1.7420999942809343163929725051275454461574554443359375f,1.39880000422596939557706718915142118930816650390625f,-0.9052000274091955844824042287655174732208251953125f,0.2590000153183940057033396442420780658721923828125f,0.67320002198517325808779787621460855007171630859375f,-0.02950004595518151973010390065610408782958984375f,-0.31159999337196353508261381648480892181396484375f,0.025699986743926928056680480949580669403076171875f,0.16539998850822446030406354111619293689727783203125f,-0.66960000493526461440296770888380706310272216796875f,0.33080001438111061862201722760801203548908233642578125f,-0.54869998608827597053050340036861598491668701171875f,-0.04269998546242703696407261304557323455810546875f,1.26159998979568399590789340436458587646484375f,0.17560002496987603404221545133623294532299041748046875f,0.90079997805953038181314695975743234157562255859375f,0.8854000102266670335637854805099777877330780029296875f,0.17270000244975136638458934612572193145751953125f,-0.27319999452978382947776481159962713718414306640625f,-0.32419996814727891631946476991288363933563232421875f,0.5267000119566918048263914897688664495944976806640625f,-0.08550005596876086855218090931884944438934326171875f,0.1392999745130538702397871020366437733173370361328125f,0.46599997529387504613396231434307992458343505859375f,0.1656999921455979407625136445858515799045562744140625f,-0.2212000179767610319458981393836438655853271484375f,-0.014400019621849224904508446343243122100830078125f,0.867999980866908771304224501363933086395263671875f,0.7487000062048434489980763828498311340808868408203125f,-0.514499997340142289914410866913385689258575439453125f,-0.379399995556473523805607328540645539760589599609375f,0.0142999825224279231150603663991205394268035888671875f,0.07859999283552188131807270110584795475006103515625f,1.88369997580647474677562058786861598491668701171875f,1.90349998334050152237750808126293122768402099609375f,-0.728000025272368844753145822323858737945556640625f,-0.4073999894618987038796831257059238851070404052734375f,-0.9529000184595586286917523466399870812892913818359375f,0.17259999429583550156763749328092671930789947509765625f,-0.28470000511705872980883214040659368038177490234375f,-0.690200004386901877495574808563105762004852294921875f,0.1163000213623046619204615126363933086395263671875f,-0.022399990355968402155895091709680855274200439453125f,-1.701000004090368999953852835460565984249114990234375f,-0.455999967098235980955678314785473048686981201171875f,0.140700018632411882890664855949580669403076171875f,-0.9118999908998610681720720094745047390460968017578125f,-0.21920000141412021577025370788760483264923095703125f,0.6131000143155456161281335880630649626255035400390625f,-1.2974999712407588869922392404987476766109466552734375f,0.2243000320315360340828192420303821563720703125f,0.367400027981400256749111576937139034271240234375f,-1.48700000363588369367562336265109479427337646484375f,-0.85880000340640538780689894338138401508331298828125f,-0.77649998939037312428723680568509735167026519775390625f,0.55439999749362434133814758752123452723026275634765625f,0.69590000878870483802529633976519107818603515625f,-0.2610000025629997111309421597979962825775146484375f,0.3831000221014022599774762056767940521240234375f,-0.478000009775161771585771930404007434844970703125f,0.259500017344951316999868140555918216705322265625f,0.2645000063180924509964597746147774159908294677734375f,0.03519998713880767571282603967119939625263214111328125f,0.08710000065118073120373765050317160785198211669921875f,0.85170001747012147319537689327262341976165771484375f,0.21999999877065423703470514738000929355621337890625f,0.3425999916136259759724680407089181244373321533203125f,0.5297999836549165397769911578507162630558013916015625f,-0.28010003840327168944668301264755427837371826171875f,-0.6609000204488635343835767343989573419094085693359375f,0.0112000275135037963281092743272893130779266357421875f,-0.238299996723234652762357654864899814128875732421875f,0.3517000031650068248012530602863989770412445068359375f,-0.133699989441037059290096067343256436288356781005859375f,-0.048799975019693686562050061183981597423553466796875f,-0.10909999199360609356546092385542578995227813720703125f,0.425399999758601321531159555888734757900238037109375f,-0.41040000344663851716831004523555748164653778076171875f,-0.1891999942988153993184141654637642204761505126953125f,1.179599977442622016354789593606255948543548583984375f,0.434199998098611761321308222250081598758697509765625f,-0.36239998946189899697856162674725055694580078125f,0.6879000519871709684593952260911464691162109375f,0.7741999751508235050323492032475769519805908203125f,-0.0604000156283373001997460960410535335540771484375f,0.3349000310361385235324860332184471189975738525390625f,-0.6068999744713305943832892808131873607635498046875f,1.080399995660781709005959783098660409450531005859375f,1.22219998403191620894858715473674237728118896484375f,-0.399600008139014306607350590638816356658935546875f,-0.455499958738684096459792272071354091167449951171875f,0.355100002677738668666052035405300557613372802734375f,-0.81379999439120265236624618410132825374603271484375f,-0.376199996519088808444308824618929065763950347900390625f,-0.31139999289661635106796211402979679405689239501953125f,-0.92179999708831328320002285181544721126556396484375f,0.773299984206258894658958524814806878566741943359375f,0.770399993127584448160405372618697583675384521484375f,-0.65629999651461812248953719972632825374603271484375f,-1.15429999552369100257465106551535427570343017578125f,-0.4650999822631483038293254139716736972332000732421875f,-2.2714000184893610168046507169492542743682861328125f,0.0283000099107624247807279971311800181865692138671875f,-0.40769995670318603941950641456060111522674560546875f,0.233900017967819895403636110131628811359405517578125f,1.542999996513128024844263563863933086395263671875f,0.367000018835067720601728069595992565155029296875f,-1.6773000187098983104760918649844825267791748046875f,0.3788000096648931336318355533876456320285797119140625f,-1.1846000098526479060723204383975826203823089599609375f,0.0936000007331372341212727405945770442485809326171875f,-0.2267999966412783674485353913041763007640838623046875f,0.28769999160915615821210167268873192369937896728515625f,0.49449996519088823543341959521058015525341033935546875f,0.2978000085175036559803629643283784389495849609375f,-1.2894999778568756454433241742663085460662841796875f,-0.12509998770207175766699947416782379150390625f,-0.9111000087797638258280130685307085514068603515625f,0.176199999983608801557011247496120631694793701171875f,0.870800018590687674446826349594630300998687744140625f,0.2540000166594982289325344027020037174224853515625f,0.6412999349355696931951342776301316916942596435546875f,0.498900009846687420012045777184539474546909332275390625f,-2.013499992765486634738181237480603158473968505859375f,-0.54270000040084120929151367818121798336505889892578125f,-0.6550000049918889288846912677399814128875732421875f,-0.23050000288337468834498622527462430298328399658203125f,0.10959999510049822646351458388380706310272216796875f,-0.3643000269651412992288896930404007434844970703125f,0.5168000084877011346407016390003263950347900390625f,1.1799000004887580939083591147209517657756805419921875f,0.5691000114023676559327213908545672893524169921875f,-0.927700011539459357123860172578133642673492431640625f,0.066400042182206409080436060321517288684844970703125f,0.4296000260949133231491714468575082719326019287109375f,0.6087000249803065088372022728435695171356201171875f,0.113500003755092837565143781830556690692901611328125f,0.6048999873757363587856161757372319698333740234375f,-0.30570001863241191841780164395458996295928955078125f,0.561299998414515499689514399506151676177978515625f,0.11990000570416459169109657523222267627716064453125f,0.269700049746036452091857427149079740047454833984375f,-0.4186000144422055146975480965920723974704742431640625f,0.3591000277936462481420676340349018573760986328125f,0.063200008201599100488010662957094609737396240234375f,-0.3649000101000072771029181240010075271129608154296875f,0.06300003018975264268419778090901672840118408203125f,-0.048000019013881622953476835391484200954437255859375f,0.5085000454783443757378336158581078052520751953125f,-0.414600031980872085757283684870344586670398712158203125f,-1.043999995201825736756973128649406135082244873046875f,0.0001999819234017596869534827419556677341461181640625f,0.3256999928534032306970402714796364307403564453125f,0.9475000087171794316276418612687848508358001708984375f,0.642299951356649412304022916941903531551361083984375f,-0.6512000453203918848288367371424101293087005615234375f,0.67350003430247351587922821636311709880828857421875f,1.346199997860193064980194321833550930023193359375f,0.857500028312206286074115269002504646778106689453125f,0.42740000600218763793236576020717620849609375f,0.495500004559755069521997938863933086395263671875f,0.303100005114078641099695232696831226348876953125f,0.4002000407457355635187923326157033443450927734375f,0.1351999998420476689631186673068441450595855712890625f,0.58590000483989701507425706950016319751739501953125f,0.47960001689344633479805679598939605057239532470703125f,-0.7843999443709852581463337628520093858242034912109375f,0.000500010073184842251947657132404856383800506591796875f,0.637199956166743408658703629043884575366973876953125f,1.24860001533627507086521291057579219341278076171875f,0.18339998353123743157766512013040482997894287109375f,0.0616999801799653635470122026163153350353240966796875f,0.728100028806924814972489912179298698902130126953125f,0.89160002172142271970045612761168740689754486083984375f,-0.0628999970018870868671001517213881015777587890625f,0.42400002478063125721519099897705018520355224609375f,-0.2344000018894671910629767808131873607635498046875f,-0.0134999992474913454998386441729962825775146484375f,0.376699993795901466597797480062581598758697509765625f,1.1381000115588306709213384237955324351787567138671875f,-0.18509998284056783379725175109342671930789947509765625f,-0.6501000062167643367416758337640203535556793212890625f,-0.3064999820888036907717832946218550205230712890625f,-0.5020000084042550714258368316222913563251495361328125f,-0.90959999226927745752391274436376988887786865234375f,-0.729099989721178953772096065222285687923431396484375f,0.17389997840523729877304504043422639369964599609375f,0.6126000022679567447170256855315528810024261474609375f,0.85519997793734103819218717035255394876003265380859375f,-0.85730002686828365998650269830250181257724761962890625f,-0.072999998804181842615435016341507434844970703125f,0.4852000278934835986177631639293394982814788818359375f,-0.58779997824952034957135538206784985959529876708984375f,-0.73010000502467153893348950077779591083526611328125f,0.780299982142448556032832129858434200286865234375f,-0.1372999964512884307765716584981419146060943603515625f,1.345599999786913247845632213284261524677276611328125f,0.65030000604018567411657159027527086436748504638671875f,0.5974000219836830272157612853334285318851470947265625f,-0.30250001668930082132646930404007434844970703125f,0.04789998705536124834480915524181909859180450439453125f,-0.43820000492334376662739714447525329887866973876953125f,-0.40190000129342084989048089482821524143218994140625f,1.079300000292062744478016611537896096706390380859375f,-0.10099998002499344096349886967800557613372802734375f,1.323600010195374210297813988290727138519287109375f,-0.623100014725327611131433513946831226348876953125f,-0.7710000034756958608994636961142532527446746826171875f,1.030599989728629761742695336579345166683197021484375f,0.67180001051798476918719416062231175601482391357421875f,-0.41700000184774399514253673260100185871124267578125f,1.25389998421669002226508382591418921947479248046875f,0.371299994968622926183599020077963359653949737548828125f,0.0728999616861340793416701444584759883582592010498046875f,0.17820001481398950460288688191212713718414306640625f,-1.293499994367361072278299616300500929355621337890625f,-0.643500006213784114805775971035473048686981201171875f,-0.179899989387392711392976707429625093936920166015625f,-0.282000000692904040366926210481324233114719390869140625f,1.1538000092923639261499602071125991642475128173828125f,0.183700002554059105275285901370807550847530364990234375f,0.84489996722191573752525073359720408916473388671875f,0.1538999968454242139870302708004601299762725830078125f,-0.592499975580721649492943470249883830547332763671875f,0.10859999713823198508322320776642300188541412353515625f,0.7157999941527839471433480866835452616214752197265625f,0.0630000112093984654393352684564888477325439453125f,0.4917999946296216506880227825604379177093505859375f,0.12839999675601720952045070589520037174224853515625f,0.835400011456012947519411682151257991790771484375f,0.958100016178190738713738028309307992458343505859375f,-0.2661999919369810729818937033996917307376861572265625f,-0.31369999940618864542329902178607881069183349609375f,0.0780999971233307110907162496005184948444366455078125f,0.6166000099122523891992386779747903347015380859375f,-0.1258999899387360787983425325364805757999420166015625f,-1.2293000143736600993094043587916530668735504150390625f,0.06570002585947509743391492520458996295928955078125f,1.120800026041269603638284024782478809356689453125f,0.4351999933600427805657773205894045531749725341796875f,0.53409995887577499384946122518158517777919769287109375f,-0.9690000243335958884927094914019107818603515625f,0.658800019574165229840900792623870074748992919921875f,-0.048899975574016707469127140939235687255859375f,0.3570000170469285194485564716160297393798828125f,-1.308900018191337721873424015939235687255859375f,-0.54980000455379496315799769945442676544189453125f,0.7243000242084265405395626657991670072078704833984375f,-0.563800012905895986392579288803972303867340087890625f,-0.06250000713393089046832074018311686813831329345703125f,0.193499995131045654517265575123019516468048095703125f,0.81009999660551557099097408354282379150390625f,0.6478999978959560923641447516274638473987579345703125f,-0.2374000239580869031641441324609331786632537841796875f,-0.3491000247389077770776566467247903347015380859375f,0.635400046361982884235430901753716170787811279296875f,-0.5208000050306320627413469992461614310741424560546875f,-0.47089999407380889184793204549350775778293609619140625f,-1.57399999155104186598919113748706877231597900390625f,-1.04229998168051185558624638360925018787384033203125f,-1.500499974012374782006418172386474907398223876953125f,0.1707999990701676296112054842524230480194091796875f,-1.7713000338792799226439456106163561344146728515625f,0.4517000068902969900364041677676141262054443359375f,-0.1136000342793763717708088734070770442485809326171875f,0.1568999978102743053653966853744350373744964599609375f,0.1501000065147877204907445047865621745586395263671875f,0.12119999944344161946219173842109739780426025390625f,0.0757999791771173381249582234886474907398223876953125f,-0.048799993757903614710613737770472653210163116455078125f,-0.846399986302852624930892488919198513031005859375f,0.15060000418275587552585648154490627348423004150390625f,-0.5927000226408247751663793678744696080684661865234375f,-0.308100015321373821297612494163331575691699981689453125f,-0.5634999831914893064777061226777732372283935546875f,-0.9033000013053416044073173907236196100711822509765625f,-0.73869999040961342728905947296880185604095458984375f,-0.370200000102818282954331152723170816898345947265625f,-0.8483999975383280656160422950051724910736083984375f,-0.387300005485117448955634245066903531551361083984375f,0.4686999918997287739585999588598497211933135986328125f,0.75490000637471699196368035700288601219654083251953125f,1.14149999274313440622563575743697583675384521484375f,1.095599976056814028879671241156756877899169921875f,0.6332999919176103276186040602624416351318359375f,0.7639999813437459152964947861619293689727783203125f,-0.03069997530728496570873176096938550472259521484375f,0.953600034633279136642158846370875835418701171875f,-0.2546999982997772971060612690052948892116546630859375f,-0.244800031971931897345484685502015054225921630859375f,0.7689000023961067853406348149292171001434326171875f,0.55009999966025358020260682678781449794769287109375f,0.027600006663799803163783508352935314178466796875f,0.58610002122223381348931070533581078052520751953125f,-0.7423999881953005086643315735273063182830810546875f,-0.5649000003024935434581266235909424722194671630859375f,0.68370000665187813382317472132854163646697998046875f,-0.3408999908328056438477915435214526951313018798828125f,-0.99659997467100591705957413068972527980804443359375f,-0.630099963524937578540630056522786617279052734375f,-0.4309999871402983018242593971081078052520751953125f,-1.76929999358654033159155005705542862415313720703125f,-0.03239998849332348385132718249224126338958740234375f,-1.659900004921853966521894108154810965061187744140625f,1.058600031839311395032154905493371188640594482421875f,-0.1735000098273158375405955666792578995227813720703125f,-0.94990001766234666291666144388727843761444091796875f,-0.4039000077605248151257910649292171001434326171875f,-0.0344999729394912524327310165972448885440826416015625f,-1.548800012868642994590118178166449069976806640625f,0.6103999836280944446542662262800149619579315185546875f,0.080800012183189606762567791520268656313419342041015625f,0.1354999901801348105578881586552597582340240478515625f,0.9930999899148942322568700546980835497379302978515625f,0.132100002476573052945241215638816356658935546875f,0.21919999541640233786665703519247472286224365234375f,0.61859998937100169058567189495079219341278076171875f,-0.756300023746490523990360088646411895751953125f,0.363299995568394873401985023519955575466156005859375f,-0.64380003898292781361334391476702876389026641845703125f,0.57790000385642048552625738011556677520275115966796875f,-0.026500019788742334991837878988008014857769012451171875f,-0.143000008843839176986278971526189707219600677490234375f,-0.393400011843442865000497477012686431407928466796875f,0.26930000681132082451796350142103619873523712158203125f,-0.740700007456541076322764638462103903293609619140625f,0.05359999376684421168448579919640906155109405517578125f,0.51689999723285418031792914916877634823322296142578125f,0.27740001121759405577904544770717620849609375f,0.121200005534291221920284442603588104248046875f,0.08910000153034924874617672685417346656322479248046875f,-0.801000008746981517759877533535473048686981201171875f,-1.096300013725459709945653230533935129642486572265625f,1.0794000117242337211109770578332245349884033203125f,-0.117599978798627768927786974018090404570102691650390625f,0.19789996388405572957935874001123011112213134765625f,-0.49230002861917032674909933120943605899810791015625f,1.40699995807558320137786722625605762004852294921875f,1.049100036808848557257078937254846096038818359375f,0.0066999993652103739805170334875583648681640625f,-0.910700002092122762320514084422029554843902587890625f,-0.36970002992749240178227410069666802883148193359375f,0.1157000121504072609468494192697107791900634765625f,0.67529998333454077652504565776325762271881103515625f,-1.111600073689222423212186185992322862148284912109375f,-0.4709000149726865203092529554851353168487548828125f,0.53229999814629547216782157192938029766082763671875f,0.601900063133239626722570392303168773651123046875f,0.8148000109612940633496691589243710041046142578125f,-0.56900000656396143572379742181510664522647857666015625f,1.5064000152111052255321510529029183089733123779296875f,-0.53949999231100065344435279257595539093017578125f,0.799099997432529907115394962602294981479644775390625f,-1.4178000076979400123633467956096865236759185791015625f,0.864799993564188351768962093046866357326507568359375f,0.59300000601261892985149870582972653210163116455078125f,0.8145000072866679374783416278660297393798828125f,0.27589997995495796345721828402020037174224853515625f,1.314499980837106729580909814103506505489349365234375f,0.05320002135187384961767520508146844804286956787109375f,1.140300043386221151564541287370957434177398681640625f,-0.9463000281795868051659681441378779709339141845703125f,-0.78309996905326872962405104772187769412994384765625f,1.038900003886222833671126863919198513031005859375f,0.43090004358291622565957368351519107818603515625f,0.0564000123798849006817590634454973042011260986328125f,0.81900001151859758152795620844699442386627197265625f,0.22179999749809498066355217815726064145565032958984375f,1.34129998749941581337452589650638401508331298828125f,0.26899999129027130084779173557762987911701202392578125f,1.2420999857872729865704286567051894962787628173828125f,0.76129998928755515663624464650638401508331298828125f,1.040999968290328592246396510745398700237274169921875f,0.32830000123083580287897120797424577176570892333984375f,0.7070000162273653021571817589574493467807769775390625f,-0.0405000067576766209498373427777551114559173583984375f,0.25660002675056414744858557241968810558319091796875f,-0.45790000687390541855847914121113717555999755859375f,-0.38429996885061168399033704190514981746673583984375f,0.4163000075787308684738263764302246272563934326171875f,-0.06559999557733586783569990075193345546722412109375f,0.2579000127226114269518575383699499070644378662109375f,0.21820000056475397087751844082958996295928955078125f,0.150200012433529206390403487603180110454559326171875f,-1.05389997970908932956035641836933791637420654296875f,-0.708400002008676921150254202075302600860595703125f,-0.839899970909952831021882957429625093936920166015625f,0.482300012004375222574026338406838476657867431640625f,-0.3546000430822375815154146039276383817195892333984375f,1.850799971354007311674649827182292938232421875f,0.673499954767525554899521011975593864917755126953125f,-0.9429999890625477831918033189140260219573974609375f,-1.4767000126272435966967577769537456333637237548828125f,1.06510000517368386141470182337798178195953369140625f,0.70680001765191402540722265257500112056732177734375f,-0.24140005186796198444199035293422639369964599609375f,-0.0577000052064651214323021122254431247711181640625f,0.55939998728632911451086329179815948009490966796875f,0.40339997906088864709772678907029330730438232421875f,0.179899977093935348193554091267287731170654296875f,0.768799985897540949508766061626374721527099609375f,1.4552999623239044613143278183997608721256256103515625f,-0.525300002110004715660807050880976021289825439453125f,0.7921000065743921680905259563587605953216552734375f,0.3892999961197372016385997994802892208099365234375f,-0.3658999623715881721608411680790595710277557373046875f,0.276099961692094719722945228568278253078460693359375f,0.18560002925395924222584653762169182300567626953125f,0.484899977391958447725528458249755203723907470703125f,-0.44490002582073184811406463268212974071502685546875f,-0.12229999024868032364565806346945464611053466796875f,1.08469997762441661137700066319666802883148193359375f,-0.796499981939792434104674612171947956085205078125f,-0.42709998615980193648056228994391858577728271484375f,0.185499998152256040384600055404007434844970703125f,-0.31189997715353978691155134583823382854461669921875f,-0.93720001249313344970914840814657509326934814453125f,0.26750003732740934569989121882827021181583404541015625f,0.3147999979972837536479346454143524169921875f,-0.3348000156551602657373223337344825267791748046875f,0.8830000091418626073647146768053062260150909423828125f,0.40130000631213180728451561662950552999973297119140625f,-0.5455000089928507822634173862752504646778106689453125f,0.1000000159069898142405463659088127315044403076171875f,-0.0511000196576123499880850431509315967559814453125f,-1.1276000236138703503030455976841039955615997314453125f,0.38630000005364450998968095518648624420166015625f,0.3318000113934278516580889117904007434844970703125f,0.3389000012040137566060593599104322493076324462890625f,0.0556000104784966919790889505748054943978786468505859375f,-0.6627000077396629773573977217893116176128387451171875f,0.28440001120269309264898538458510302007198333740234375f,-0.49550001811981214672186979441903531551361083984375f,0.1066000060379506120256110079935751855373382568359375f,1.08910004642009727859885970246978104114532470703125f,1.765800022129714097474106893059797585010528564453125f,0.06489998864233481601360153945279307663440704345703125f,-0.0855999933794138190279454647679813206195831298828125f,0.5941999899774790261375301270163618028163909912109375f,0.4141999974653123306467250586138106882572174072265625f,1.2899000083863738641554164132685400545597076416015625f,1.938599989557264979822548411902971565723419189453125f,-1.24170000852942497004960387130267918109893798828125f,-0.47370000821649949074299001949839293956756591796875f,-0.01119999964833251482332343584857881069183349609375f,0.24400000116229036972725907617132179439067840576171875f,-0.2215999845802787415749435240286402404308319091796875f,-0.58990002135038366315455959920655004680156707763671875f,-0.096599985772371166348193582962267100811004638671875f,0.112200004595517821570638261619023978710174560546875f,-0.891799999547004507149949859012849628925323486328125f,-0.3523999820113186132175542297773063182830810546875f,-0.711699988710880138143011208740063011646270751953125f,-0.07950000255554900974175325245596468448638916015625f,0.29749999690800876805951702408492565155029296875f,-0.2498000022068620484816392490756697952747344970703125f,-0.3138999962493775885974400807754136621952056884765625f,-0.80289995006322900650275187217630445957183837890625f,-1.2653000016629694091108149223146028816699981689453125f,0.08730001260042197230859528644941747188568115234375f,-0.6794999832212926804686503601260483264923095703125f,0.2276999944776296747050992053118534386157989501953125f,0.48170000867843631997544662226573564112186431884765625f,0.48329999147057500774593563619419001042842864990234375f,-0.1706999833166598090627985584433190524578094482421875f,-0.041700005400180817360933360760100185871124267578125f,-0.047900010710954621373502959613688290119171142578125f,0.5230000025480985303971692701452411711215972900390625f,0.019699987086653492074361793129355646669864654541015625f,1.039000011555850644384690895094536244869232177734375f,-0.218899988873302808745080483276979066431522369384765625f,-0.133600004830956020640542192268185317516326904296875f,-0.35850000669807169373370925313793122768402099609375f,0.8469000215589996827247887267731130123138427734375f,-0.7045000129863623339332434625248424708843231201171875f,2.13480004955530233701210818253457546234130859375f,-0.013100011372566111589321735664270818233489990234375f,0.3170000097453591347829160440596751868724822998046875f,0.16380002095252266247626948825200088322162628173828125f,-0.473000005900859665786128971376456320285797119140625f,-0.67500002712011364369715238353819586336612701416015625f,1.1963000220328570133432322109001688659191131591796875f,-0.449400019584596355226580044472939334809780120849609375f,0.92950001507252466570463411699165590107440948486328125f,0.230799997542798340788294808589853346347808837890625f,0.325199963073432574134358219453133642673492431640625f,0.38570002433210592851509090905892662703990936279296875f,0.48220002344548651418421059133834205567836761474609375f,-0.34340000800639292055649320900556631386280059814453125f,-0.445399955576657902156512136571109294891357421875f,0.56720005552023611272716152598150074481964111328125f,0.0356999985158434895993195823393762111663818359375f,0.5476000438794506042228249498293735086917877197265625f,0.479700013983249906601713519194163382053375244140625f,0.2493000336334103561597430598339997231960296630859375f,1.2044000249862667484279654672718606889247894287109375f,0.1920999275237325054632719911751337349414825439453125f,-1.65170001169592151057941009639762341976165771484375f,-0.865499977104365658675533268251456320285797119140625f,-0.7175000176206232094244796826387755572795867919921875f,-0.62539999066889262469004506783676333725452423095703125f,0.070600001798570122257814318800228647887706756591796875f,-0.73449999190121884051762890521786175668239593505859375f,0.72779998963028180725132187944836914539337158203125f,0.64939997432231899576748901381506584584712982177734375f,-0.46089999157786365469036127251456491649150848388671875f,-0.7220999764740465565182603313587605953216552734375f,0.21379996682405444374808212160132825374603271484375f,-0.762499995380640172015773714520037174224853515625f,0.6440000139772887877143148216418921947479248046875f,-0.51659999068975448466289890347979962825775146484375f,-0.009700016665458832676449674181640148162841796875f,1.19129999006986597720469944761134684085845947265625f,-0.8364999671876434117478993357508443295955657958984375f,-0.407599988521635481220073415897786617279052734375f,1.2377999848619098965940565904020331799983978271484375f,-0.04989999799281352732549521533655934035778045654296875f,0.75289999741166813151949099847115576267242431640625f,0.39019999753236778161635811557061970233917236328125f,0.00920000770986095783854352703201584517955780029296875f,-0.369900017625093280315695665194652974605560302734375f,0.05180003206878946997449020273052155971527099609375f,0.2925999522745608150131602087640203535556793212890625f,-0.10360000069588382043406227239756844937801361083984375f,0.745700015801191273823178562452085316181182861328125f,0.434900002425909093517475412227213382720947265625f,1.443300024551152915819329791702330112457275390625f,-0.3924000385612256280865040025673806667327880859375f,-0.681999981135129385023674331023357808589935302734375f,0.78800005138665507598005888212355785071849822998046875f,0.55039997718334188192557121510617434978485107421875f,-0.0633999969795342455114450785913504660129547119140625f,0.160799988490342826708001666702330112457275390625f,-0.02040000557005416936817709938623011112213134765625f,-0.38750000953674312853536321199499070644378662109375f,0.14619999931305638352085907172295264899730682373046875f,0.2186000133991239380293336580507457256317138671875f,0.7722000514596703002467847909429110586643218994140625f,1.17560005682110801927819920820184051990509033203125f,0.3325000053644178166223355219699442386627197265625f,-0.3356999758660792476661072214483283460140228271484375f,0.3954000928908587564336585273849777877330780029296875f,2.237299997252226280153308834997005760669708251953125f,0.5881000118568537882168811847805045545101165771484375f,-0.40000003919005422403643024154007434844970703125f,-0.2955999932676542929499419187777675688266754150390625f,1.954800038081407453915971927926875650882720947265625f,0.8909000250309702106932263632188551127910614013671875f,0.19510002350211141930458325077779591083526611328125f,-0.9293000011861332065876695196493528783321380615234375f,-0.300200007963180581782580702565610408782958984375f,-0.2822000010192393659735898836515843868255615234375f,0.410900003349781162143017354537732899188995361328125f,-1.032100002402067229212434540386311709880828857421875f,-0.6333000203788277104877124656923115253448486328125f,0.88049998071789747910997903090901672840118408203125f,-0.403100035661458466762496755109168589115142822265625f,-1.148699998456239601551942541846074163913726806640625f,0.6086999610543253336203406433924101293087005615234375f,0.3110999849379065818055778436246328055858612060546875f,0.793999981194734250067313041654415428638458251953125f,-0.78260000383257821710003554471768438816070556640625f,-0.622400003767013654254469656734727323055267333984375f,0.352099977517127626924775540828704833984375f,0.389899985587597530667380851809866726398468017578125f,0.246800004017352581087152429972775280475616455078125f,0.58959997677207010013944454840384423732757568359375f,-0.2245000040829181475743325790972448885440826416015625f,-0.00190000829696668205315290833823382854461669921875f,0.43689999784752731049053409151383675634860992431640625f,-0.24759999708980295896054713011835701763629913330078125f,0.1993999911792574408764266991056501865386962890625f,-0.3293000188067558564597447912092320621013641357421875f,-0.1963999997138976549848621289129368960857391357421875f,0.09089999653249980493541215764707885682582855224609375f,0.0786000057995319689752022895845584571361541748046875f,0.6066000064104795086450394592247903347015380859375f,-1.1455999852210283496134479719330556690692901611328125f,0.1839000121936200660144322682754136621952056884765625f,-1.0558000224649906062523996297386474907398223876953125f,0.206200009036064113576713907605153508484363555908203125f,0.81830000281408443729702639757306315004825592041015625f,-0.034600010001659187164335662600933574140071868896484375f,-0.43180000284388686537795365438796579837799072265625f,0.5322000114127991565737829660065472126007080078125f,-1.093999998480081448093415019684471189975738525390625f,0.67820001649036998347952476251521147787570953369140625f,0.0658000178828835602384828007416217587888240814208984375f,-1.209100039155781569633063554647378623485565185546875f,-1.619300020706653686630716038052923977375030517578125f,-1.0370000202879310347015007209847681224346160888671875f,-0.5517000179916620172804186950088478624820709228515625f,-0.43579998703747968580302085683797486126422882080078125f,0.6596999946862458497065517804003320634365081787109375f,0.63079999087452875983217381872236728668212890625f,0.6853000102683901584299519527121447026729583740234375f,0.4067999445989727291816961951553821563720703125f,0.134899984395503824696760375445592217147350311279296875f,0.4009000315688553417459161209990270435810089111328125f,-0.03569999289065572956047844854765571653842926025390625f,-0.83940000125616831549990592975518666207790374755859375f,0.50969999550580968872282028314657509326934814453125f,0.553199975568056334651600991492159664630889892578125f,0.20099999177828442586957180537865497171878814697265625f,1.159499998625367567228749976493418216705322265625f,0.34130000104084612377874918820452876389026641845703125f,-0.04809999036192902366337875719182193279266357421875f,0.54780000211000423604446041281335055828094482421875f,-0.040700011554360315813028137199580669403076171875f,0.59440000419914718055025559806381352245807647705078125f,-0.3714999973624946161265825139707885682582855224609375f,-0.6722999727398157876478990147006697952747344970703125f,0.60339999954998491116242576026706956326961517333984375f,-1.2122999811217189947143424433306790888309478759765625f,-0.457200010835379366813668866598163731396198272705078125f,0.80759999327138076719023729310720227658748626708984375f,0.364499995980411828355727266171015799045562744140625f,0.00669998802170168605840672171325422823429107666015625f,-0.006000006884336439583194078295491635799407958984375f,-0.6529999787807465594369205064140260219573974609375f,0.93480002958774566224064983543939888477325439453125f,-0.4911000300325458756134366922196932137012481689453125f,0.8321000171542169976390823649126105010509490966796875f,0.16590000092834233935690235739457421004772186279296875f,-0.08159999635219572500233198297792114317417144775390625f,-0.10219999934285872722483645702595822513103485107421875f,-0.283900013720989274901285170926712453365325927734375f,0.2958000017151236615831066956161521375179290771484375f,0.6958000206768513162813860617461614310741424560546875f,-0.92870003272891066270489091039053164422512054443359375f,0.0804000017702577451927936635911464691162109375f,-0.6106999917358157947688823696807958185672760009765625f,0.081500009804964523851822377764619886875152587890625f,-0.39340000465363278436825567041523754596710205078125f,0.8266999811112878404628645512275397777557373046875f,-0.48120002185106292014182827188051305711269378662109375f,1.28050003689527525096991666941903531551361083984375f,0.983200031261146190075805861852131783962249755859375f,-0.05080002211108813892082025631680153310298919677734375f,-0.49590000670254175130224894019193015992641448974609375f,0.21269999412372708746943317237310111522674560546875f,0.13620000665187836919045594186172820627689361572265625f,-0.1580999892443417653709047954180277884006500244140625f,-0.82560002813637256569023747942992486059665679931640625f,0.00949998627603054757173595135100185871124267578125f,0.0190000326037409461576999092358164489269256591796875f,-0.9335000014454124528384681980242021381855010986328125f,-0.355400023190677760709377253078855574131011962890625f,-1.44789997532069758534589709597639739513397216796875f,-0.12099998248368415421083454930339939892292022705078125f,-0.3331999678194517144902420113794505596160888671875f,0.60910002175867550278098860871978104114532470703125f,-0.92980001841187487343631801195442676544189453125f,-0.68979998413920373678820396889932453632354736328125f,0.20470000623464557065744884312152862548828125f,0.054799996022880381740804978107917122542858123779296875f,0.33389999436810569033440287967096082866191864013671875f,-0.1704000023849308320667006455551018007099628448486328125f,0.05449999929592019842772288029664196074008941650390625f,0.8648999893128872518133221092284657061100006103515625f,0.0070999962180853248838729996350593864917755126953125f,-0.35040001346766935785836949435179121792316436767578125f,1.1197999901741741979321886901743710041046142578125f,-0.2595999527812005158722286068950779736042022705078125f,-0.6479000076562166920979279893799684941768646240234375f,-0.14269999537169975045713954386883415281772613525390625f,-0.338999963983893426444637952954508364200592041015625f,0.138800002437830638513105441234074532985687255859375f,-0.180800038260221729302656967774964869022369384765625f,0.02650000727176671233564775320701301097869873046875f,1.0647000072777270673896055086515843868255615234375f,0.05690001351237317095410617184825241565704345703125f,-0.1381999972864986769849338088533841073513031005859375f,0.994599975132941960254129298846237361431121826171875f,-0.734599998825788080836218796321190893650054931640625f,1.92270002539753903647579136304557323455810546875f,-0.43080001479089291560597985153435729444026947021484375f,2.333700032170117122376495899516157805919647216796875f,-0.6419000389933591232960452543920837342739105224609375f,0.030900004504621048084800349897705018520355224609375f,0.15150002489239022818168223238899372518062591552734375f,0.648100023144483561310380537179298698902130126953125f,-0.6570000300854450525633865254349075257778167724609375f,1.35150002664327661960896875825710594654083251953125f,0.08049999982863675906941125504090450704097747802734375f,0.4842999915748833483775115382741205394268035888671875f,0.223800004076957825827065562407369725406169891357421875f,-0.79600000066310172552874746543238870799541473388671875f,0.98560003752410441091313941797125153243541717529296875f,1.3383000048071149734596474445424973964691162109375f,1.5246000125348562193039470002986490726470947265625f,0.4084000149726865203092529554851353168487548828125f,0.789500031575560257124379859305918216705322265625f,0.68079998096525640161047476794919930398464202880859375f,0.2520000317618258378615792025811970233917236328125f,-0.658600019732117569759566322318278253078460693359375f,-0.789600016595423337406600694521330296993255615234375f,0.664900025969743513343246377189643681049346923828125f,0.956900022713840225918602300225757062435150146484375f,-0.276400015956163702668391124461777508258819580078125f,0.1969000313937672164144032649346627295017242431640625f,0.472600002156198240044204794685356318950653076171875f,-0.58639999665915976123642394668422639369964599609375f,0.0943000071093440528358087249216623604297637939453125f,0.08379999923408021356152630687574855983257293701171875f,-0.384399991533160179102424081065692007541656494140625f,-0.50699997942149621632523803782532922923564910888671875f,-0.558000032797455691735422078636474907398223876953125f,-0.651899958825111980331712402403354644775390625f,0.17230000425577163269963421043939888477325439453125f,-0.321500008799135539305780184804461896419525146484375f,1.1210999929472797997931365898693911731243133544921875f,-0.05499998506158565447776709333993494510650634765625f,-0.179900013191998164874263466117554344236850738525390625f,-0.18839999604821233702978133806027472019195556640625f,0.054100017958879309531994294957257807254791259765625f,-0.7448999958693978129531387821771204471588134765625f,0.359899987375735808114995961659587919712066650390625f,0.614299981628358526819511098437942564487457275390625f,0.5853999815419317354070471992599777877330780029296875f,-0.15849999887496235562167612442863173782825469970703125f,-0.3645000017359851796072689467109739780426025390625f,-0.5122000025838613357365147749078460037708282470703125f,0.625900012625753898731773006147705018520355224609375f,-1.4841000176981093972017333726398646831512451171875f,0.461799998839199421496459763147868216037750244140625f,-0.852499988190829949274984755902551114559173583984375f,0.705400022669136372854836736223660409450531005859375f,-0.458099989505111881982202248764224350452423095703125f,-0.0013000100746751730440564642776735126972198486328125f,-0.307800010305643301222744412370957434177398681640625f,0.6350000074505803837610073969699442386627197265625f,-1.199200012850761165594803969725035130977630615234375f,-0.0772999838784331760876966654905118048191070556640625f,-1.7102000557214027676167233948945067822933197021484375f,0.6873000507846478424056613221182487905025482177734375f,1.38649998223781611983440598123706877231597900390625f,-0.4242000071883198497602052157162688672542572021484375f,0.0309999688863751288181447307579219341278076171875f,-0.7057000282064083140909360736259259283542633056640625f,0.6601000000700356640237487226841039955615997314453125f,0.955900004504620870449116409872658550739288330078125f,0.93049998168647307661416334667592309415340423583984375f,0.85529997994452688914179816492833197116851806640625f,1.94199998769164050571589541505090892314910888671875f,2.0437000059813268393327234662137925624847412109375f,0.715199999693036492232067757868207991123199462890625f,-0.19810002578943997075811012109625153243541717529296875f,0.0681999686390160064775045611895620822906494140625f,-0.393400025627016702856053598225116729736328125f,-0.604699966225027640831513053853996098041534423828125f,0.00809995597749779694396465856698341667652130126953125f,0.91489998409748007901498567662201821804046630859375f,0.4960000146701932788317890299367718398571014404296875f,-0.05849997335672352249957839376293122768402099609375f,-0.45299997431039873418967545148916542530059814453125f,0.433700020547211295696143906752695329487323760986328125f,-0.60640000377446401369496697952854447066783905029296875f,-1.023700002069771297641409546486102044582366943359375f,-0.26589998952895399497720063664019107818603515625f,-0.8838999748662115241160108780604787170886993408203125f,-0.7234000079318880782608403023914434015750885009765625f,-1.086799971644580420360171046922914683818817138671875f,0.57799999688565761868375147969345562160015106201171875f,-0.49790001033842568034515352337621152400970458984375f,0.289199974182248187304367093020118772983551025390625f,0.00840001504719289915357194331591017544269561767578125f,0.2053000434607261670549860355095006525516510009765625f,-1.422200006011127726424092543311417102813720703125f,0.053999974936246086798519172589294612407684326171875f,1.5513000195741657449843842186965048313140869140625f,-0.25649999281764035430342119070701301097869873046875f,-0.15290004900693876521700076409615576267242431640625f,-0.34410000991225242472637546597979962825775146484375f,-0.3874999675154688905109878760413266718387603759765625f,-1.912799997043609590008372833835892379283905029296875f,-0.56569996912330411387159756486653350293636322021484375f,-0.84539996146261719989212224390939809381961822509765625f,0.475800004173815072050501839839853346347808837890625f,-1.716699959802627351024284507730044424533843994140625f,-1.8667000090882186480456539356964640319347381591796875f,0.04690006238818167361159794381819665431976318359375f,-0.5896999948352574794085967369028367102146148681640625f,0.69980004404187212685428676195442676544189453125f,-0.8116000325620174038476761779747903347015380859375f,0.2239000374138360172082684584893286228179931640625f,-0.3454000006899240471369694205350242555141448974609375f,1.78719999192953071798228847910650074481964111328125f,0.3163000055670739385504930396564304828643798828125f,0.40050001558661474376776823191903531551361083984375f,-0.38039999476671226830148953013122081756591796875f,-1.1185999536268418363960108763421885669231414794921875f,-1.001199997822940002123459635186009109020233154296875f,-0.362200014020502170097870475728996098041534423828125f,-0.064500006113201335811169201406300999224185943603515625f,-1.018899927443265962523355483426712453365325927734375f,-1.03209999945908759144685973296873271465301513671875f,1.149200059342384339089448985760100185871124267578125f,0.08180000364482420405209950331482104957103729248046875f,1.46100000017881459513091613189317286014556884765625f,-0.0375000042468311978183237442863173782825469970703125f,0.9241000569999204827809080597944557666778564453125f,0.21860003258436933037955895997583866119384765625f,1.021199971336126832710533562931232154369354248046875f,0.8522000005722045745670811811578460037708282470703125f,0.3441999970555300603081150256912223994731903076171875f,0.274400007775425891498599639817257411777973175048828125f,0.450999990642070969215637887828052043914794921875f,1.19950003612041466993787253159098327159881591796875f,-0.94929998077154120750265064998529851436614990234375f,-0.6704000217378141002200209186412394046783447265625f,-0.1264000158071514334778839838691055774688720703125f,-0.449400007402896761732336017303168773651123046875f,-0.6669000053167337682680226862430572509765625f,1.53280002535581605371817204286344349384307861328125f,0.4527999949574468274704486248083412647247314453125f,-0.876200025129318049721405259333550930023193359375f,-0.76520001973509810255791308009065687656402587890625f,0.8319000216335059594285894490894861519336700439453125f,-0.62180002104193032863577172975055873394012451171875f,-0.966600015351176455880022331257350742816925048828125f,0.1679999844804405118026124910102225840091705322265625f,0.909600020432472344822372178896330296993255615234375f,-0.1323000447869304707637638784945011138916015625f,-0.4731999907672406191494474114733748137950897216796875f,0.93020001939982244199001115703140385448932647705078125f,-0.4121000128328802247779094614088535308837890625f,0.1714999960586425231667817570269107818603515625f,-0.9788000757515431526911697801551781594753265380859375f,-0.4586999794200063984561666075023822486400604248046875f,-1.381699993330240516797857708297669887542724609375f,0.4999000129684809667907074981485493481159210205078125f,0.889000023812055673033682978712022304534912109375f,0.12129999815374592486705296323634684085845947265625f,-1.029400022900104350043193335295654833316802978515625f,-0.4888999974787233693263033273979090154170989990234375f,0.1930000531747939618298914865590631961822509765625f,-1.39780001134872389201291298377327620983123779296875f,-0.6710000220462679632049685096717439591884613037109375f,-0.431600002014636885405707289464771747589111328125f,-0.34750001162290544698407757095992565155029296875f,-0.07940000174045547254308985429815948009490966796875f,-0.3891000589370730011751220445148646831512451171875f,-0.226500010997057188433245755732059478759765625f,-1.725800016653537216626546069164760410785675048828125f,-0.53520000941604362054704324691556394100189208984375f,-0.279999972879886538379423654987476766109466552734375f,-0.18590000774562354113328410676331259310245513916015625f,0.5926999510407444660131659475155174732208251953125f,0.435399996927380517064420928363688290119171142578125f,-1.0314999480396498388046211402979679405689239501953125f,-1.095600024820864160091105077299289405345916748046875f,0.84190001556128224446950980563997291028499603271484375f,-0.37779999700635669146464579171151854097843170166015625f,-0.211299981576203721456153061808436177670955657958984375f,-0.38880002441704297755364905242458917200565338134765625f,0.60959998392462733107777239638380706310272216796875f,0.2162999960675835442458492252626456320285797119140625f,0.514599990034103171865353942848742008209228515625f,-0.5797000354409220790330437012016773223876953125f,-1.586800003719330387497166157118044793605804443359375f,0.9104000148832798178233360886224545538425445556640625f,1.08459999763369552283620578236877918243408203125f,0.71060001062750810518764410517178475856781005859375f,-0.4219000017404557212330473703332245349884033203125f,0.39390001103878002908231792389415204524993896484375f,-0.33699995476007416073116473853588104248046875f,-1.502899991339444962790139470598660409450531005859375f,-0.02770000349283208151973667554557323455810546875f,1.38390000343918817549138111644424498081207275390625f,0.9641999405801298106410968102863989770412445068359375f,-0.24260000692456984427991528718848712742328643798828125f,-1.3710000002905717852996758665540255606174468994140625f,-0.6558000173427165346851097638136707246303558349609375f,1.000800001379848591653853873140178620815277099609375f,-0.8080000006854533012301544658839702606201171875f,-0.56120004178136628780748651479370892047882080078125f,-2.646200013059377464941235302831046283245086669921875f,-0.79229998063743067859832081012427806854248046875f,-0.3465999887526034495266458179685287177562713623046875f,0.3065999890506267977485777009860612452030181884765625f,-1.0005000257194041068942169658839702606201171875f,1.710399999795854331097189060528762638568878173828125f,-0.00900004988908786884849178022705018520355224609375f,-0.3598999998182055293227676884271204471588134765625f,-0.57250004544854160570821477449499070644378662109375f,-0.51679996642917414018114641294232569634914398193359375f,0.3364000143170355894284284659079276025295257568359375f,0.3157000047743314663506453143781982362270355224609375f,-0.0307999820083384445723595490562729537487030029296875f,-0.09489997467249644547138132111285813152790069580078125f,1.45539999364912464585586349130608141422271728515625f,-0.4502000306129456674852917785756289958953857421875f,-1.23599995175003929404056179919280111789703369140625f,-0.5872000032544135716960909121553413569927215576171875f,0.300299980801343568970196429290808737277984619140625f,0.381899989446997434328068266040645539760589599609375f,-0.53050003406405465966599876992404460906982421875f,0.416100005017220997416416139458306133747100830078125f,-1.07750005185604091906270696199499070644378662109375f,-0.0337999981164933682720175056601874530315399169921875f,-0.53969997613430020777514073415659368038177490234375f,-0.214900018966198036451942243729718029499053955078125f,0.917799962398409885366845628595910966396331787109375f,0.29059998678565046592581211370998062193393707275390625f,0.401299989566951731401189817916019819676876068115234375f,-0.07079998111426799578538293644669465720653533935546875f,0.454199973176419913301060660160146653652191162109375f,0.197699995315819998520368017125292681157588958740234375f,-0.2221000040411951204077922739088535308837890625f,-0.334900007045269010319543667719699442386627197265625f,0.35570000313520377943632411188445985317230224609375f,0.449100003206730047367045699502341449260711669921875f,-0.0304000177145002936640594271011650562286376953125f,1.2242999968647954034395297640003263950347900390625f,0.2223000036597255757442326284945011138916015625f,-0.26270001779794682761348667554557323455810546875f,-0.07440001872777912694800761528313159942626953125f,1.6330999970674511256873984166304580867290496826171875f,-0.9594999811351296870043370290659368038177490234375f,0.11120000873804070096184659632854163646697998046875f,0.42890002370476754123274076846428215503692626953125f,1.15130001525282832375296493410132825374603271484375f,-0.9064999870061871689586041611619293689727783203125f,0.23480003197193166641909556346945464611053466796875f,-1.2097000232219699000779655762016773223876953125f,0.055900002008676619169591504032723605632781982421875f,-0.81340001538246864409842373788706026971340179443359375f,-0.387499985098838539698817839962430298328399658203125f,-0.2350999729126694859360213740728795528411865234375f,-0.79259997701048856555416932678781449794769287109375f,-0.372799980391561813775069822440855205059051513671875f,-0.4348999817132952472320539527572691440582275390625f,-0.7808999842390420553783769719302654266357421875f,-2.2328000256538391710137148038484156131744384765625f,0.7041000103965397993732722170534543693065643310546875f,-1.0936999756574632414185543893836438655853271484375f,-0.8794999779686329954841994549497030675411224365234375f,-1.0342999990999699466698302785516716539859771728515625f,0.0953000173836950903449860561522655189037322998046875f,-2.565300047111511272390771409845910966396331787109375f,-0.2299999988079068913293667719699442386627197265625f,-1.5449000264167782603408340946771204471588134765625f,-0.28450000333786018558157593361102044582366943359375f,-0.199899972623586830167141670244745910167694091796875f,0.10950000352412458948236917422036640346050262451171875f,0.035800016951560653666319922194816172122955322265625f,-0.20480000202059722713698874940746463835239410400390625f,-0.03610004096627239533745523658581078052520751953125f,-0.0387000055715442936588033262523822486400604248046875f,-1.5025999846100805967807900742627680301666259765625f,-0.241900011128187042430681685800664126873016357421875f,-0.106100011014938377940097780083306133747100830078125f,-0.39579998390823600828980488586239516735076904296875f,-0.9808000008583077633517177673638798296451568603515625f,-0.316499991960823623404763793587335385382175445556640625f,-1.54110002338290197343440013355575501918792724609375f,-0.78190001392215524589346387074328958988189697265625f,-0.560799986404180828003518399782478809356689453125f,-0.34909999951869252754477201960980892181396484375f,0.08229998846054087380252894945442676544189453125f,0.765899994558095986718626591027714312076568603515625f,0.60639997199773798541855285293422639369964599609375f,-0.724500034183263696974108825088478624820709228515625f,-0.2511000357508663682892802171409130096435546875f,1.4399000146448610593097328091971576213836669921875f,0.80090002212524336044907613540999591350555419921875f,-1.133899964100122392807179494411684572696685791015625f,-1.3525000005960468030252741300500929355621337890625f,-0.2556000077962874428294526296667754650115966796875f,1.5083999561131002931091416030540131032466888427734375f,0.08450000475347019257554848081781528890132904052734375f,-0.19109997711479653759170105331577360630035400390625f,-0.2127999776721001268242616788484156131744384765625f,0.38849999451637273750748136080801486968994140625f,-0.50530000240802763755709747783839702606201171875f,-0.48660002079009956332811270840466022491455078125f,0.0905000050067901185002483543939888477325439453125f,0.06610001485198735426962457495392300188541412353515625f,0.677800003302097220370114882825873792171478271484375f,-1.1659000156059866615265718792215920984745025634765625f,-0.4316999684453008967466303147375583648681640625f,-0.60310000049471812477719367961981333792209625244140625f,0.377700064140558566805339069105684757232666015625f,-0.2780000291094177899964279276900924742221832275390625f,-0.68770003642439814228737304802052676677703857421875f,-0.09290000188201685826783204902312718331813812255859375f,1.8665999664008616765187298369710333645343780517578125f,0.830499985076487146073986878036521375179290771484375f,1.2276999992460011501549388412968255579471588134765625f,0.10070001103281978060977053246460855007171630859375f,1.14329998342394834054402963374741375446319580078125f,-0.6693999874353409040850237943232059478759765625f,-0.35620001022815728219939046539366245269775390625f,0.24339999411106116866676529753021895885467529296875f,0.646099974209070726516301874653436243534088134765625f,0.03249998379498719724978172962437383830547332763671875f,0.537700069057942098282865117653273046016693115234375f,-0.25759999928772436827983938201214186847209930419921875f,0.76010000308752001529910558019764721393585205078125f,0.1011000088542697294968775167944841086864471435546875f,0.03030001522302594452185076079331338405609130859375f,0.0392999969393013071083942122641019523143768310546875f,1.19109995982945005010833483538590371608734130859375f,0.8595999931260944748601104947738349437713623046875f,0.822399981564283510948598632239736616611480712890625f,0.1716000090181826198687531359610147774219512939453125f,0.29409999665021846482204637140966951847076416015625f,-0.4687999941676854742667046593851409852504730224609375f,-0.0969999988675116497915951185859739780426025390625f,-0.241500003919005390429219914949499070644378662109375f,0.414199988412857322828131145797669887542724609375f,-0.03060000526309014645676143118180334568023681640625f,-2.08850001060962586763025683467276394367218017578125f,-1.397000041782856083472097452613525092601776123046875f,0.2316999964594845806686862488277256488800048828125f,0.35300000739097558977164226234890520572662353515625f,1.12280004740953387454283074475824832916259765625f,-1.1777999723076817417677375487983226776123046875f,-0.88299998980760552313995503936894237995147705078125f,-0.069799997103214383997737968456931412220001220703125f,1.59169997917413663657271172269247472286224365234375f,0.49030000583529531610338381142355501651763916015625f,1.05859993650913253304679528810083866119384765625f,0.7172999462902538425623788498342037200927734375f,0.5536000366449354004316774080507457256317138671875f,1.0447000409543516497024029376916587352752685546875f,-0.50520000259876240988887730054557323455810546875f,-0.6037999921560288640876024146564304828643798828125f,0.82520001571178391852612321599735878407955169677734375f,-0.99710000314712576852116399095393717288970947265625f,-0.1250000152736905345562945512938313186168670654296875f,1.412899997746944080745379324071109294891357421875f,0.17299997263401767355617266730405390262603759765625f,1.054600009554624850949267056421376764774322509765625f,0.56869999432116802307035641206311993300914764404296875f,-0.38440005344748495730300419381819665431976318359375f,-0.37270001600980739997481805403367616236209869384765625f,1.485099980884790671353812285815365612506866455078125f,-1.050799964275956011050539018469862639904022216796875f,-0.2425999939978122910133606637828052043914794921875f,-0.154600036264956219866917308536358177661895751953125f,-1.527400007343292287487201974727213382720947265625f,0.51370002393722558053923421539366245269775390625f,-1.0130000175237654502780060283839702606201171875f,-0.2980000121593473494385762023739516735076904296875f,-0.55450001436471918481174725457094609737396240234375f,-0.5371000000923868977764641385874710977077484130859375f,0.65420001299977403874663650640286505222320556640625f,0.1324999968335032196620204558712430298328399658203125f,0.1087999808311455041121007525362074375152587890625f,0.5113000235974787255344153891201131045818328857421875f,-0.382299996805191000248669297434389591217041015625f,0.151199988248944094948456040583550930023193359375f,1.20899998208880443684165584272705018520355224609375f,0.46739998741298904860741458833217620849609375f,0.32319997765421870639102053246460855007171630859375f,0.66809997363537565462365819257684051990509033203125f,-0.852800023418664121521715060225687921047210693359375f,0.32139999669641239332662507877103053033351898193359375f,-0.2312000154435640553174380329437553882598876953125f,0.039000003360211810132796017569489777088165283203125f,0.412999995097517835773714978131465613842010498046875f,0.966800015603005657993662680382840335369110107421875f,-1.14239999706149131242227667826227843761444091796875f,-0.6223999897599219455202046447084285318851470947265625f,-0.5968000214517115153967097285203635692596435546875f,-0.15719999535679818336575408466160297393798828125f,-0.21849997469782866943432964035309851169586181640625f,0.10310000705122934760993302916176617145538330078125f,0.05540002188682446870870990096591413021087646484375f,0.1361000522166495585452139494009315967559814453125f,-1.264200017246604357268324747565202414989471435546875f,-1.39060005372911721366335768834687769412994384765625f,-0.076999996297061112926485293428413569927215576171875f,0.1833000244021416591522211092524230480194091796875f,0.56970000087022754087229259312152862548828125f,0.37819999419450756050764539395458996295928955078125f,0.4156999590277674627714077359996736049652099609375f,-0.474199997949600160485061906001647002995014190673828125f,0.345800001342594587327994304359890520572662353515625f,-0.8409999918341635183338667047792114317417144775390625f,-0.93849999630451197418778974679298698902130126953125f,-0.1239999969899654441718439557007513940334320068359375f,0.598399972243607525257402812712825834751129150390625f,0.4770999818384649149294318704050965607166290283203125f,0.7516999682217839762188305030576884746551513671875f,0.0147999647676950729646705440245568752288818359375f,0.07640002221465136411637786295614205300807952880859375f,-0.55119998601376984925792612557415850460529327392578125f,-0.36960002967119243155735830441699363291263580322265625f,-0.093600019396841815932930330745875835418701171875f,0.0351000094205142776360162315540947020053863525390625f,0.51050001215934770470994408242404460906982421875f,-0.746799991798400952802694519050419330596923828125f,0.5421000028491018696286118938587605953216552734375f,-0.1288000095903873987168708481476642191410064697265625f,-0.12239998655617245759685829398222267627716064453125f,-0.93089999377578502848251673640334047377109527587890625f,0.16270001928806276936256836052052676677703857421875f,-0.2262999934971332560706969161401502788066864013671875f,-0.1339000098466869292934688928653486073017120361328125f,0.217399998290836504732936873551807366311550140380859375f,-0.7557999743342396214984546531923115253448486328125f,-0.728999999746680504841833680984564125537872314453125f,-0.377499979250134121144810706027783453464508056640625f,-0.5430999942362306764920276691555045545101165771484375f,-0.40159999352097397462557637481950223445892333984375f,0.17569999069273467551255407670396380126476287841796875f,0.743700007992982303761664297780953347682952880859375f,0.07529999790042640750442615171778015792369842529296875f,0.961999973982572864628082243143580853939056396484375f,0.30369999312907458577370789498672820627689361572265625f,-0.210399980163574529257175527163781225681304931640625f,-0.2906999834656713854741383329383097589015960693359375f,1.090699996131657911746515310369431972503662109375f,-0.7216000066339969265527543029747903347015380859375f,0.89370000903606428011016760137863457202911376953125f,1.223599978381395114723773076548241078853607177734375f,0.961900031840801972293775179423391819000244140625f,-0.37589999828338616083556189551018178462982177734375f,-1.169999983310699320782077847979962825775146484375f,0.31790000147223462789725090260617434978485107421875f,-0.029600018793344506917719627381302416324615478515625f,0.181600019150972180614900253203813917934894561767578125f,-0.619300019253790079432064885622821748256683349609375f,-0.0929999921545385799248606417677365243434906005859375f,0.0401999609872703000945648454944603145122528076171875f,-0.49579999936819074690674824523739516735076904296875f,0.20679999582171415539733061450533568859100341796875f,0.555999982297420647370245205820538103580474853515625f,0.3656999779149889828744335318333469331264495849609375f,1.979899991250038571166669498779810965061187744140625f,-0.33310000567287245676340035061002708971500396728515625f,1.2893000333726405415291083045303821563720703125f,1.094500029042362765352436326793394982814788818359375f,0.6071000119388099136585879023186862468719482421875f,-0.160999979987740626796721699065528810024261474609375f,-1.7613999834716320691541113774292171001434326171875f,-1.17239999016970397605064135859720408916473388671875f,-0.54059997069239618117109102968242950737476348876953125f,-0.328899992263317120233523382921703159809112548828125f,0.0168999976798891449192296931869350373744964599609375f,-0.41239998573660852088096362422220408916473388671875f,1.164599992679059337064018109231255948543548583984375f,-0.5480000082850455811467327293939888477325439453125f,0.49080001143813145603189695975743234157562255859375f,0.578900007015466400162040372379124164581298828125f,1.2356000083923337484748117276467382907867431640625f,0.637400014719367025151086636469699442386627197265625f,0.897800008368491386789855823735706508159637451171875f,0.88470002411603954062258026169729419052600860595703125f,-0.40590004417896263788634314551018178462982177734375f,0.0394000086694956674904233295819722115993499755859375f,-1.1214999452829363946193552692420780658721923828125f,-1.01989999877512449444338926696218550205230712890625f,0.4986000106424091082857330547994934022426605224609375f,-0.1200000017136335639378330597537569701671600341796875f,0.832599991837143615924787809490226209163665771484375f,-0.22519999630302189874697660343372263014316558837890625f,1.1153999885827305948993171114125289022922515869140625f,-0.106800003048777625291876347546349279582500457763671875f,0.2956000179290769747098011066555045545101165771484375f,-0.004599995547533042294929828130989335477352142333984375f,-0.6406000062689181095976209689979441463947296142578125f,-1.9575999881677332847829120510141365230083465576171875f,-0.3024999792315068436465708145988173782825469970703125f,0.22710001173391913820154286440811119973659515380859375f,0.34759996631890555107702311943285167217254638671875f,-2.221399984961748064193898244411684572696685791015625f,-1.17059996305555191753455801517702639102935791015625f,0.917100029373168812441008412861265242099761962890625f,1.9231000355496996601090131662203930318355560302734375f,0.65830004377365103351849029422737658023834228515625f,1.0198000004559764075651173698133789002895355224609375f,0.9370000392496589114443850121460855007171630859375f,0.3867000182151780762751513975672423839569091796875f,1.685899986660480021072316958452574908733367919921875f,1.1712999895110736137127105394029058516025543212890625f,0.4154000206947328077689007841399870812892913818359375f,0.0908000047326085191201627822010777890682220458984375f,-0.15670000405907646978675984428264200687408447265625f,-0.79639996132478163559653694392181932926177978515625f,0.2653000216864047633436030082521028816699981689453125f,0.7591999914117157910453670410788618028163909912109375f,-0.2130000050626694729771770653314888477325439453125f,-0.36079997493028681532223345129750669002532958984375f,0.60349997231364316263579894439317286014556884765625f,-0.99979998562932070882425250601954758167266845703125f,-1.085899983233213017541629596962593495845794677734375f,0.26659997712969829564144674804992973804473876953125f,-2.19620001335740067105462003382854163646697998046875f,0.27370000307559916308264291728846728801727294921875f,0.6881000083923343169090003357268869876861572265625f,-0.433800025981664560958961374126374721527099609375f,-0.84309998067617453187949649873189628124237060546875f,-0.01459999450445170765533475787378847599029541015625f,-0.3770000183880328759045141850947402417659759521484375f,0.12100000852346415314286787179298698902130126953125f,-0.399500029750168739184346122783608734607696533203125f,0.665199982929229438610718716518022119998931884765625f,-0.403300013487041031368107724119909107685089111328125f,0.545200060415268250579856612603180110454559326171875f,-0.188500019587576839175113718738430179655551910400390625f,0.647700017648935588709946387098170816898345947265625f,-0.0228999910041689513917617659899406135082244873046875f,0.1317999511182303962186779244802892208099365234375f,-0.5583000209376216549372884401236660778522491455078125f,-0.695999998241662698461595937260426580905914306640625f,0.11920002492070247424038598182960413396358489990234375f,-0.7565000481382018193698968389071524143218994140625f,0.15619999502897241594467914183042012155055999755859375f,-0.39519999384433024403051604167558252811431884765625f,-0.0167999958962201656476054267841391265392303466796875f,0.440500015735626249124834430404007434844970703125f,0.5463000038534400193412920998525805771350860595703125f,-0.52470003588795588456150653655640780925750732421875f,-1.30240005029588967744302863138727843761444091796875f,-1.44000001847744041327814557007513940334320068359375f,-0.65710000080019170809464412741363048553466796875f,-0.468400030320883065115822319057770073413848876953125f,0.3209000328168276627849309079465456306934356689453125f,1.0226000057697302025871977093629539012908935546875f,-0.2364000630810865200004400321631692349910736083984375f,-2.34730002519190339427268554572947323322296142578125f,-0.59419996639639105495689364033751189708709716796875f,0.03890003815889409022332756649120710790157318115234375f,0.12349996907263882661709430976770818233489990234375f,-0.356600022205710132094935715940664522349834442138671875f,1.1191000368833547984337428715662099421024322509765625f,0.75440000606179236086745731881819665431976318359375f,0.51019999112486846826186592807061970233917236328125f,-0.4252999889969826252666962318471632897853851318359375f,1.276699989157915826609723808360286056995391845703125f,0.6851999802470203349002986215054988861083984375f,-0.243999982535839432529201076249592006206512451171875f,0.8447000415503975734310415646177716553211212158203125f,-1.042699994254112194624894982553087174892425537109375f,0.393800001394748466765349803608842194080352783203125f,1.40419999899268166387855671928264200687408447265625f,-0.346500019937753478416198049671947956085205078125f,-1.08329998096525681461343992850743234157562255859375f,-0.11819999680221038307337266815011389553546905517578125f,0.9304000108972190208334041017224080860614776611328125f,-0.8590000020191068674790813020081259310245513916015625f,-0.92980000049322786281891239923425018787384033203125f,-0.5462000013247132468308109309873543679714202880859375f,0.313300006222724913840238514239899814128875732421875f,-0.30329998372197142231243560672737658023834228515625f,0.39979995329380013657782910740934312343597412109375f,-0.394899997098743238410634148749522864818572998046875f,1.529900034761428884166889474727213382720947265625f,-0.2334000078573825742722647191840223968029022216796875f,0.448199989873170778764688293449580669403076171875f,-0.191600003728270407155065413462580181658267974853515625f,1.839700031417608183659240239649079740047454833984375f,-0.4883000347211960967541699574212543666362762451171875f,0.72670002208948147881528711877763271331787109375f,0.0419999675750728496126384925446473062038421630859375f,-0.113599957370758009034261704073287546634674072265625f,-0.191299989436566786338289603008888661861419677734375f,-0.2807000079035759654288995079696178436279296875f,0.353500011838972783007051248205243609845638275146484375f,0.11540002006143323942666256698430515825748443603515625f,0.427199999231099469820804870323627255856990814208984375f,-1.156000005748122561044510803185403347015380859375f,-0.270500031605363400188934974721632897853851318359375f,-1.2927000018537047054678623680956661701202392578125f,-0.1966999143168328689768031836138106882572174072265625f,0.946200009930134200430984492413699626922607421875f,1.2755000128298998873788150376640260219573974609375f,0.51989999125003816260459643672220408916473388671875f,-0.3826999984264374266018648995668627321720123291015625f,-0.85300002944469444088326781638897955417633056640625f,-1.762000016637145893838578558643348515033721923828125f,-0.71700004744529710620781770558096468448638916015625f,0.3259000391870738866373358177952468395233154296875f,0.9807000111073254089433248736895620822906494140625f,1.0442999523475762213564621561090461909770965576171875f,-0.0310000093989077907252749355393461883068084716796875f,-0.7306000318802896575931526967906393110752105712890625f,0.46599997531250092475119117807480506598949432373046875f,0.40779995789080913937141303904354572296142578125f,-0.44690003973841641027320292778313159942626953125f,-0.841999989628792189222394881653599441051483154296875f,1.35750000417232552507584841805510222911834716796875f,-0.3450999916136259226817628587014041841030120849609375f,-0.9293000269651414679827894360641948878765106201171875f,0.523000000983476542870675984886474907398223876953125f,1.29140000254511821964342743740417063236236572265625f,0.65409998204708141855689973453991115093231201171875f,1.2235999865770335048864581040106713771820068359375f,-1.45999998256564111898114788345992565155029296875f,0.28600000032782535441810978227294981479644775390625f,0.1464000338375563980974902733578346669673919677734375f,-1.489699968460202139652892583399079740047454833984375f,0.56019999628439476335728386402479372918605804443359375f,0.86670004792436927676391178465564735233783721923828125f,-0.102700015916675158056392547223367728292942047119140625f,-0.70649997804686481828184696496464312076568603515625f,-0.438300013822316980593996049719862639904022216796875f,0.16030001801699433894299318126286379992961883544921875f,0.361799991798400721876305397017858922481536865234375f,-0.45080000644624218519851410746923647820949554443359375f,0.0739000096976756282174392254091799259185791015625f,-0.2978999968156219591008948555099777877330780029296875f,0.41519997667074193259395542554557323455810546875f,-0.540900012513995331886462736292742192745208740234375f,1.5991000412046911804964111070148646831512451171875f,-0.83450000810623148339573162957094609737396240234375f,0.5604000066876413388428090911475010216236114501953125f,-0.30550000318139802768513391129090450704097747802734375f,0.52810001703500741854213629267178475856781005859375f,-1.20040004848539894055647891946136951446533203125f,1.429800040018558870968945484491996467113494873046875f,-0.24960001499354855791779073115321807563304901123046875f,1.03950003582239158816946655861102044582366943359375f,-0.260599998110532471429223733139224350452423095703125f,0.382600023800134803053651921800337731838226318359375f,0.74360003027468890568485448966384865343570709228515625f,-0.393999984920025081436278924229554831981658935546875f,0.389000010922550654157703320379368960857391357421875f,0.25720001055598284978032097569666802883148193359375f,0.305599975274503332656195198069326579570770263671875f,-0.7196000117897991543713942519389092922210693359375f,0.4837000366404655693486347445286810398101806640625f,-0.847499961107969657092553461552597582340240478515625f,-0.8067999847576017291572725298465229570865631103515625f,-0.4264999628663062036793007791857235133647918701171875f,0.566100012691319154356506260228343307971954345703125f,-1.74390000865459438017524007591418921947479248046875f,-0.5956999629765735715380969850230030715465545654296875f,-0.6076000035345554550758606637828052043914794921875f,-0.33579998819231970941245890571735799312591552734375f,-0.322000006467103805363194624078460037708282470703125f,-0.2519999961853027770075641456060111522674560546875f,0.02029998407959965334157459437847137451171875f,-0.712099961572886019922634659451432526111602783203125f,-0.579699988800287169254943364649079740047454833984375f,0.6647999971032145793969903024844825267791748046875f,-0.47950000527501135394459197414107620716094970703125f,-0.9723999720275404712310773902572691440582275390625f,0.73890001074075684073250158689916133880615234375f,0.372799998533725585048159700818359851837158203125f,-0.08189999712109585772168429684825241565704345703125f,-0.09679998896718000622740873950533568859100341796875f,-0.159300005358458118820408344618044793605804443359375f,-0.199699999976157283043676216038875281810760498046875f,0.431899985051154455817368216230534017086029052734375f,0.1363000058650971624274461646564304828643798828125f,0.83210000746846191077565890736877918243408203125f,-0.018099995130300694512470727204345166683197021484375f,-0.1535000008493663248287930400692857801914215087890625f,-0.758500022232532433719143227790482342243194580078125f,0.25530002729296719099494339388911612331867218017578125f,-0.58560000809431045354358502663671970367431640625f,-0.14259997715949968011273085721768438816070556640625f,-0.40300006133317989309716722345910966396331787109375f,0.5238999826893209199596412872779183089733123779296875f,0.670200039106607192707087961025536060333251953125f,0.5965000108107922915934295815532095730304718017578125f,-0.817399976944923167820888920687139034271240234375f,0.246099996784329366761312485323287546634674072265625f,-0.784400005912780695638275574310682713985443115234375f,0.052399997173249779347514731853152625262737274169921875f,-1.721899983412027435036861788830719888210296630859375f,0.28020000144392209762145284912548959255218505859375f,-0.018499990046024095136090181767940521240234375f,0.5455000008717174608818822889588773250579833984375f,0.247599992507696153865026644780300557613372802734375f,0.30479996964782485324718663832754828035831451416015625f,-0.048799977552891338206109139719046652317047119140625f,1.0386000451013448486747847709921188652515411376953125f,0.78600007021427220621490050689317286014556884765625f,0.8615000077188010862982991966418921947479248046875f,0.07929995454549843003633213811554014682769775390625f,-0.2039999722167851192722309860982932150363922119140625f,1.702299979296326259969873717636801302433013916015625f,0.9339999640956524462609422698733396828174591064453125f,0.26549999539554125504992043715901672840118408203125f,-0.605500009216367818254411758971400558948516845703125f,-0.6578999704778194512755362666212022304534912109375f,-0.123300029021501611481426152749918401241302490234375f,-0.1295000197291376053954081726260483264923095703125f,0.2061999798297886599129924434237182140350341796875f,0.030800004136562364465135033242404460906982421875f,0.10720000668168072621710962266661226749420166015625f,-1.149099992030858885527777601964771747589111328125f,-0.921499998480082016527603627764619886875152587890625f,-0.659199984240531389190209665684960782527923583984375f,0.6181999967277054253145252005197107791900634765625f,-1.459200046676397022338278475217521190643310546875f,-0.206900010383128662994067781255580484867095947265625f,1.386599996501207243682074476964771747589111328125f,-0.1878999986410143652193482921575196087360382080078125f,-0.843599959531426524250719012343324720859527587890625f,0.42919997164905099307219416004954837262630462646484375f,-0.1255000023990870960943766476702876389026641845703125f,0.139500014662742533033679137588478624820709228515625f,-1.15069997243881250170716157299466431140899658203125f,0.13669998871088040459653711877763271331787109375f,0.2183000249981876805804859031923115253448486328125f,0.28859998866319624966081391903571784496307373046875f,-0.14840001020431525802223404753021895885467529296875f,0.66750001445412632250508977449499070644378662109375f,0.7658000150889154866717944969423115253448486328125f,-1.24130001718997906579033951857127249240875244140625f,0.18470002158284248849895448074676096439361572265625f,-0.0208000077128414062599404132924973964691162109375f,0.23169995767921125473520760351675562560558319091796875f,0.68580003587603588499632678576745092868804931640625f,0.247600002454221534975431495695374906063079833984375f,-1.3282999884903432530336431227624416351318359375f,0.15480000615566924526689263075240887701511383056640625f,0.841099981659650719478804603568278253078460693359375f,-0.6670999975219362720935123434173874557018280029296875f,1.7189000165522099194959082524292171001434326171875f,-0.568200031819940054589324063272215425968170166015625f,1.047500032782554768573390902020037174224853515625f,-0.075800037142635279252544933115132153034210205078125f,-1.21120004509687451133004287839867174625396728515625f,-1.61930004335641886115126908407546579837799072265625f,0.36759996747374490411175429471768438816070556640625f,0.1921999920040378118102353255380876362323760986328125f,0.95960005768537559589503871393389999866485595703125f,0.1783000100597735837482105125673115253448486328125f,-0.25320007913112618780360207892954349517822265625f,-0.703599982963501791033422705368138849735260009765625f,1.0569000069558622811882742098532617092132568359375f,1.260200012582540551875354140065610408782958984375f,1.55399996420741093317019476671703159809112548828125f,1.435199957671760895294710280722938477993011474609375f,0.183800015252828519152217268128879368305206298828125f,0.4949000833019621126140918931923806667327880859375f,-0.1981000028789043909682732191868126392364501953125f,-0.4275999966800210216888444847427308559417724609375f,-0.3539999995231628560077297152020037174224853515625f,0.66870000456571521141313496627844870090484619140625f,1.50990004012584666526208820869214832782745361328125f,-0.231900040185451405960748161305673420429229736328125f,-0.15709998332858088332386614638380706310272216796875f,0.63569999061822901609275504597462713718414306640625f,-0.204200025516748429055269298260100185871124267578125f,-0.534400020068883208068655221723020076751708984375f,1.535899984723329225744237191975116729736328125f,-0.774699981647729796208068364649079740047454833984375f,-1.686299992454051821511029629618860781192779541015625f,-0.127799974393844717468482485855929553508758544921875f,-0.3647999950170515148784033954143524169921875f,-0.7283000002622603830104708322323858737945556640625f,0.370599975758791000401970450184307992458343505859375f,0.0280000084340574773023035959340631961822509765625f,0.3957999935567375615619312156923115253448486328125f,0.1425999898999930071141761800390668213367462158203125f,-0.691700010764598527401858518715016543865203857421875f,-0.50059999345391992253695434555993415415287017822265625f,0.5388000151410696769715968912350945174694061279296875f,-0.2027999964475630800819772048271261155605316162109375f,0.230499995395540946407919591365498490631580352783203125f,0.271499978959560461788669272209517657756805419921875f,-0.640400015106796782760056885308586061000823974609375f,-0.855899990832805546148165376507677137851715087890625f,-0.00400000183284276999984285794198513031005859375f,0.14869999189972826769690072978846728801727294921875f,0.491900012022256749588677848805673420429229736328125f,0.3602000009596351137730607661069370806217193603515625f,-0.1364000123798845276468227893928997218608856201171875f,-0.836799993735552138929278953582979738712310791015625f,0.3708000160574911330257918962161056697368621826171875f,-0.342500005066394663799655972979962825775146484375f,0.372800012205541264620478614233434200286865234375f,-1.6015999592110514715415092723560519516468048095703125f,-0.775699985514580969692133294302038848400115966796875f,0.135600003474950714377200711169280111789703369140625f,0.3428999801635743693850599811412394046783447265625f,0.2256999764621261395092233215109445154666900634765625f,-0.6923999983653421619322898550308309495449066162109375f,-1.69759995972514143858234092476777732372283935546875f,-0.83739995362460550865080222138203680515289306640625f,-0.7940999921426172836191881287959404289722442626953125f,-0.8443000466346755050750516602420248091220855712890625f,0.199499954871833029557137706433422863483428955078125f,0.17010002219826059199903056651237420737743377685546875f,0.131900015002489379156713766860775649547576904296875f,0.5187000161141150922361475750221870839595794677734375f,1.119399993842840057567400435800664126873016357421875f,0.912999966710806187819571277941577136516571044921875f,-0.391800012548267739820317956400685943663120269775390625f,0.23450001142174003998519538072287105023860931396484375f,-0.58219999427646396128466221853159368038177490234375f,0.42449998530745514102591187111102044582366943359375f,0.1817000217914579440758870987338013947010040283203125f,0.648099959069491138308194422279484570026397705078125f,0.81279996515512475951936721685342490673065185546875f,-0.222299997401237181549049637396819889545440673828125f,-0.5311000247240063032450052560307085514068603515625f,0.91970002389252158980781359787215478718280792236328125f,0.6285000041648747259870333436992950737476348876953125f,-0.4068000302061438588907549274154007434844970703125f,0.9429000071719289000071739792474545538425445556640625f,-0.762699969667196331357672534068115055561065673828125f,-1.115900007373094471319063814007677137851715087890625f,0.31309999289512635556320674368180334568023681640625f,-0.767499971538782244095955320517532527446746826171875f,0.20740001062154789224223350174725055694580078125f,0.496100010493397913791113751358352601528167724609375f,-1.175399988806247630890311484108678996562957763671875f,0.81330003248602189547256102741812355816364288330078125f,-0.1402999982357027164425744558684527873992919921875f,-0.6558000042483211888821870161336846649646759033203125f,-0.150500016927718860415552626363933086395263671875f,-0.612000019319355548219618867733515799045562744140625f,0.14970000712871556203253931016661226749420166015625f,0.0755000128671527992452183752902783453464508056640625f,-0.086900008744001500105014201835729181766510009765625f,0.1187999952480196796944511561378021724522113800048828125f,-0.627899972601234868108122100238688290119171142578125f,0.48320000626444770119860550039447844028472900390625f,-1.4176000015228982231718646289664320647716522216796875f,0.39960001722872273699493916865321807563304901123046875f,-0.5470000109374524299710174091160297393798828125f,0.2210000395551332541543843035469762980937957763671875f,1.17050002661347374299793955287896096706390380859375f,0.4145000067278743483001335334847681224346160888671875f,1.758400021529197498892926887492649257183074951171875f,0.829299993474780894331388481077738106250762939453125f,-0.31549997341632884939599534845910966396331787109375f,-0.647100031086802029989257789566181600093841552734375f,-2.0573999879717828065395224257372319698333740234375f,-0.509299962666630801066958156297914683818817138671875f,0.36970002076327812545741835492663085460662841796875f,1.0487000066146252574839081717072986066341400146484375f,-0.02479999527782206225623440332128666341304779052734375f,0.2211999912291766623440025796298868954181671142578125f,0.387599992619454603026696304368670098483562469482421875f,0.384199953097105695309920747604337520897388458251953125f,0.3086000154852868160304524280945770442485809326171875f,0.4411999974131581847558436493272893130779266357421875f,0.74709998511672015553131132037378847599029541015625f,-0.1940000069737435328676156132132746279239654541015625f,-0.826299998712539451872771678608842194080352783203125f,-0.09909999888539378076757202506996691226959228515625f,0.048800012570619610841049507143907248973846435546875f,0.486099998050927961656952902558259665966033935546875f,-0.56470000221133265227990705170668661594390869140625f,-0.7171999809026718963878010981716215610504150390625f,0.2843999914586543553696174058131873607635498046875f,-0.139199991616606677524714541505090892314910888671875f,-0.76889999323189262003808153167483396828174591064453125f,-0.014799994719028486400702604441903531551361083984375f,-0.5851999661654236906116466343519277870655059814453125f,0.39859998136162777537805368410772643983364105224609375f,-1.0209999628514054847983061335980892181396484375f,0.08859999656081196661716603557579219341278076171875f,-0.2977999980866907758780826043221168220043182373046875f,-0.5971000100016592426754868938587605953216552734375f,0.848699995066226176021473293076269328594207763671875f,0.45309999110698750968140302575193345546722412109375f,0.69429998255968083942235580252599902451038360595703125f,1.23369998087286969479237086488865315914154052734375f,-0.044600008176267247339552568519138731062412261962890625f,0.242199998318403830754874661579378880560398101806640625f,-0.02910000210776930718026278555043973028659820556640625f,0.35600000341981641316380091666360385715961456298828125f,-0.3627000034555794005797224599518813192844390869140625f,0.910499993942677665614837678731419146060943603515625f,0.5931000267952679649852143484167754650115966796875f,-0.58860003236085212119377274575526826083660125732421875f,-0.020399978151917519397784417378716170787811279296875f,-0.4766999976515771120233466717763803899288177490234375f,-0.80650001490861178421454269482637755572795867919921875f,0.41670003930032273320449576203827746212482452392578125f,0.0309999837875363226658009807579219341278076171875f,0.97419997723698603664388429024256765842437744140625f,-0.0528000041589142821152336182422004640102386474609375f,-0.182999982208013278750513563863933086395263671875f,-0.644699999044835436734501854516565799713134765625f,-0.5346999809399244352192681617452763020992279052734375f,-0.476600000971555692874659371227608062326908111572265625f,0.29710001104474048627679394485312514007091522216796875f,0.29700000065565124174327138462103903293609619140625f,0.25509995797425510932754377790843136608600616455078125f,0.90719999472349888147704177754349075257778167724609375f,-0.355200056764483351745553818545886315405368804931640625f,0.239399991868436357034255479447892867028713226318359375f,0.140600017835945279021103715422214008867740631103515625f,-0.40230001345723864769610145231126807630062103271484375f,0.775400016410648706965957899228669703006744384765625f,0.185800005868822337706802727552712894976139068603515625f,-0.389899979887902981001701618879451416432857513427734375f,-0.26319996143057899917749864471261389553546905517578125f,-0.2291000107876961511266244997386820614337921142578125f,-0.814400024390220522718664142303168773651123046875f,-0.4516000327110294421117941965349018573760986328125f,-0.27330000525414932521783839547424577176570892333984375f,-0.082700016289205091624125998350791633129119873046875f,0.5620999811306590654425008324324153363704681396484375f,0.89480002718493290014833974055363796651363372802734375f,0.128800026577711168584983170148916542530059814453125f,-1.3470999617218968325005334918387234210968017578125f,-0.21499999731779073641746435896493494510650634765625f,-0.92129999036788934319019972463138401508331298828125f,-1.654400041824579670191042168880812823772430419921875f,0.35250005424022656796978480997495353221893310546875f,0.431800034564733703490446714567951858043670654296875f,1.224500013768672346259336336515843868255615234375f,0.414499981470406175798615322491968981921672821044921875f,0.808300002590567334692650547367520630359649658203125f,0.761299972095340837796584310126490890979766845703125f,-0.267800018613040702319949559750966727733612060546875f,0.36460000270009051970276914289570413529872894287109375f,-0.56730001714527611422766995019628666341304779052734375f,0.9445000491403046094518458630773238837718963623046875f,-0.52450002211332336088389638462103903293609619140625f,0.13200001859292409545076907306793145835399627685546875f,0.695100007744133829845623040455393493175506591796875f,-1.85950003269314745324436444207094609737396240234375f,1.462299999748169820890097980736754834651947021484375f,0.055499987572431574989195723901502788066864013671875f,-0.35950005202740442644682161699165590107440948486328125f,1.01250002071261402392110539949499070644378662109375f,0.35030000091791180238942615687847137451171875f,1.356700009572506360200350172817707061767578125f,0.09940000187084052640784648247063159942626953125f,-0.1440999944150449518787127090035937726497650146484375f,-0.36160001134648922516134916804730892181396484375f,0.273200038823486313077637532842345535755157470703125f,-0.481900011686980656033796321935369633138179779052734375f,1.1334000474758447740697420158539898693561553955078125f,-0.130599996452778588373888624118990264832973480224609375f,-0.3096000013589861321605667399126105010509490966796875f,-0.0718999802269042476154936593957245349884033203125f,-1.690800006073713834808813771815039217472076416015625f,-0.2622999911427499331040280594606883823871612548828125f,-0.649199984836578369851167735760100185871124267578125f,-0.28849998602271054171097830476355738937854766845703125f,0.47240003803968466655760494177229702472686767578125f,0.5131000200897453122905744749004952609539031982421875f,0.7845999995708456964393917587585747241973876953125f,0.510400012946128800450651397113688290119171142578125f,-0.1397000253081319698367224191315472126007080078125f,0.238699980872869144121750650811009109020233154296875f,-0.711399997329711819560316143906675279140472412109375f,-0.7747999556779860963473538504331372678279876708984375f,-0.835699969905615347443017526529729366302490234375f,-1.000800002646446529297463712282478809356689453125f,-0.4371000025510785036431116168387234210968017578125f,-0.74729998294711119655175934894941747188568115234375f,0.6090000427365307444915742962621152400970458984375f,0.07309999274611467257045660517178475856781005859375f,-0.4615999823451044647981689195148646831512451171875f,-0.219399996376037353940091634285636246204376220703125f,-0.699799976837635284709904226474463939666748046875f,0.173000003218650721947824422386474907398223876953125f,-1.0138999696135524430928853689692914485931396484375f,-0.7429999974071979096379436668939888477325439453125f,0.2394999935030934778978917165659368038177490234375f,-0.144699979710579018643556992174126207828521728515625f,-0.30610001712441459176261560060083866119384765625f,0.270400023078918305685647283098660409450531005859375f,0.76059997631758502478760419762693345546722412109375f,1.01189998177289997016714551136828958988189697265625f,0.7574999773502344435627264829236082732677459716796875f,1.89149996480345716776128028868697583675384521484375f,0.1056000041827565016916423701331950724124908447265625f,1.367999986827373337661128971376456320285797119140625f,1.1736999894037840608262968089547939598560333251953125f,-0.82240005130171756064783039619214832782745361328125f,-0.055499997854232763216941748396493494510650634765625f,-1.154999975264072276104343472979962825775146484375f,1.068999985925853213331038205069489777088165283203125f,-0.04889998794198024967272431240417063236236572265625f,-0.6943000084877013478035223670303821563720703125f,0.6257000084996224842370793339796364307403564453125f,-0.54380000318288779226350015960633754730224609375f,0.5765999990344052861246382235549390316009521484375f,-0.1581999902829529514036721593583934009075164794921875f,0.164100015127658149793887787382118403911590576171875f,-0.24979998406469838823085183321381919085979461669921875f,0.64479999963641176918827113695442676544189453125f,-0.332599973993003228400766602135263383388519287109375f,-0.18460002356171667514672662946395576000213623046875f,0.2312000098183748519886648864485323429107666015625f,-1.642799990934133091968760709278285503387451171875f,-0.755499997071921836067076583276502788066864013671875f,-1.03749996095895813397191886906512081623077392578125f,-0.199699988874793010751318433904089033603668212890625f,-0.186199976700544223007227628841064870357513427734375f,-0.57759999194890265794555261891218833625316619873046875f,0.308200022506713633418939934927038848400115966796875f,-0.04239999482631684468714183822157792747020721435546875f,-0.04770000200271606871638141456060111522674560546875f,0.5693000129207970161360208294354379177093505859375f,0.072399979366362166022241808605031110346317291259765625f,0.82449998156353798695050727474153973162174224853515625f,-0.6832000022225084290283803056809119880199432373046875f,-0.5263999489009376642201232243678532540798187255859375f,-1.2243999954819686504237097324221394956111907958984375f,0.289099981525540616900116219767369329929351806640625f,0.59499997176229957318582819425500929355621337890625f,-0.09960003771781966719345291494391858577728271484375f,-0.38209992506504075748807736090384423732757568359375f,-0.199399955844878906674466634285636246204376220703125f,-1.1864999801516535882228708942420780658721923828125f,-1.635400012648105860790792576153762638568878173828125f,-0.4391999818563459001552473637275397777557373046875f,0.31560001092553147117314438219182193279266357421875f,1.1438999725937843976453223149292171001434326171875f,-0.13589999798536300801288234652020037174224853515625f,-0.738299967107176602354456917964853346347808837890625f,-0.7829999823197721919854075167677365243434906005859375f,-0.495700008108466760869958989133010618388652801513671875f,-1.773200006357580349458658020012080669403076171875f,-0.17140001834034845895615717381588183343410491943359375f,-0.9068000095307822139290010454715229570865631103515625f,1.3781999977335328555483329182607121765613555908203125f,1.47499998010694977068624211824499070644378662109375f,1.255499981611966742178765343851409852504730224609375f,-0.2104999835491181414681705064140260219573974609375f,0.2925999981701379315524036428541876375675201416015625f,-0.51250000417232488558738623396493494510650634765625f,0.4219999688044191454849851652397774159908294677734375f,0.0961999400436877039055616478435695171356201171875f,-0.042899992680549559054270503111183643341064453125f,1.3544999803900719115290485206060111522674560546875f,-0.34379999901056290667611392564140260219573974609375f,-0.2470999914310876288681129153701476752758026123046875f,-0.49190000857636329012478881850256584584712982177734375f,-0.8570999948024742298713363197748549282550811767578125f,-0.6528000097654762523546878583147190511226654052734375f,-0.1913999768406160484346401062794029712677001953125f,0.1698000000461934444473399707931093871593475341796875f,-0.005500036448239598030340857803821563720703125f,0.5389999980702999682335985198733396828174591064453125f,-1.3258999710142607142415727139450609683990478515625f,-0.95579999173134577716837156913243234157562255859375f,-0.818399993366003997152802185155451297760009765625f,-0.25189998881369835004306878545321524143218994140625f,-0.441400000607966802590453880839049816131591796875f,-0.0519000088185073149560366800869815051555633544921875f,0.947399967706203494088867955724708735942840576171875f,0.48599999578297137237115066454862244427204132080078125f,0.71799999308586137658494408242404460906982421875f,0.6346000080645082253028022023499943315982818603515625f,-0.79609999000430065052569261752068996429443359375f,0.591200022298097760398150057881139218807220458984375f,-0.35280002788901310140090572531335055828094482421875f,-1.7869000267744066245967360373470000922679901123046875f,0.026000004332512371973962217452935874462127685546875f,0.1947000136479737353312202685629017651081085205078125f,-0.00380001597926005185712483580573461949825286865234375f,-0.191600036734342893396387808024883270263671875f,2.228600015783310794859062298201024532318115234375f,0.57670000867843640435239649377763271331787109375f,-0.22600000464916103481982645462267100811004638671875f,1.211699956971406866301776972250081598758697509765625f,-0.6149000231385226555858025676570832729339599609375f,-0.96390000180006030205959177692420780658721923828125f,-0.12989991316795368447856162674725055694580078125f,-0.636399991369247519656937583931721746921539306640625f,0.151600015127658860336623547482304275035858154296875f,-0.95869999070763611825896077789366245269775390625f,0.3274000151664020474839844609959982335567474365234375f,0.771199987280368759456905536353588104248046875f,-0.90780003343969595253071247498155571520328521728515625f,-0.2072000113569198997964804220828227698802947998046875f,0.523800004300474686402822044328786432743072509765625f,1.52540003136768920199983767815865576267242431640625f,-0.79569996975660384208595132804475724697113037109375f,1.148200037556886154987978443386964499950408935546875f,-0.48799994026124460244631109162583015859127044677734375f,-1.927399994081259393396976520307362079620361328125f,1.2043999930232753836634174149367026984691619873046875f,-0.0729000065386298246039586956612765789031982421875f,0.7081999950885773387199151329696178436279296875f,-0.0130999577283859025556012056767940521240234375f,0.1053999980449678020022474811412394046783447265625f,0.047699990528821789581570556038059294223785400390625f,-0.27750000074505809521951960050500929355621337890625f,0.4859999720156193969700098023167811334133148193359375f,-0.211200004528462936814747763492050580680370330810546875f,-0.271199956155568078219886274382588453590869903564453125f,0.058300001286715252835080036675208248198032379150390625f,1.6506999971747393640697509908932261168956756591796875f,0.66060002115145355361391921178437769412994384765625f,0.24369998146891536094926777877844870090484619140625f,0.41590001098662643386205672868527472019195556640625f,-0.68030003340244338261300072190351784229278564453125f,0.0880999799311161790882351851905696094036102294921875f,-0.152000038802622583489210228435695171356201171875f,0.3110000123232605773182513075880706310272216796875f,-1.001399945622683418378073838539421558380126953125f,-0.34559997959583999005417354055680334568023681640625f,0.5799000125959514750917378478334285318851470947265625f,-0.4815999918632207776880704841460101306438446044921875f,-0.716600007006526151798198043252341449260711669921875f,0.85129999364614494794523125165142118930816650390625f,1.446699973958730733869515461265109479427337646484375f,0.4561000107914205958792308592819608747959136962890625f,-1.6404999762475487301571774878539144992828369140625f,-0.731699988263844858238371671177446842193603515625f,0.5466999758958817068332791677676141262054443359375f,-0.243600033515691816177195505588315427303314208984375f,-1.383899969017505782176158390939235687255859375f,-0.8831000164389610773696404066868126392364501953125f,0.257900033807754169856707449071109294891357421875f,0.56570003517270084358870008145458996295928955078125f,-0.9254999719262126944840929354541003704071044921875f,-1.30280000464320178110710912733338773250579833984375f,0.09249998629093170166015625f,-0.50610001131296156184902201857767067849636077880859375f,-0.02690001135170450030642541605629958212375640869140625f,-0.944800017741322495368194722686894237995147705078125f,-0.300699978734553174941623865379369817674160003662109375f,0.896500025749206397307489169179461896419525146484375f,0.9436999933525920258858832312398590147495269775390625f,0.099300005656480561100352133507840335369110107421875f,-0.0396999797105788143625204611453227698802947998046875f,0.95369998728036886603831590036861598491668701171875f,1.357199981647729369882426908588968217372894287109375f,-0.65669998736977586162311126827262341976165771484375f,-0.3150000231340526202217233731062151491641998291015625f,0.229900020456314191363844656734727323055267333984375f,1.6474999867752198046133571551763452589511871337890625f,0.54449997875094435784149027313105762004852294921875f,-0.351700011062622053259474341757595539093017578125f,0.539899965620040944713764474727213382720947265625f,0.14159999374449239173401338121038861572742462158203125f,-0.0794999810978769261282650404609739780426025390625f,0.145599994515627606528340720615233294665813446044921875f,0.8064999805241821473344998594257049262523651123046875f,0.41839999925196191821186175729962997138500213623046875f,0.325899987107515443085503648035228252410888671875f,-0.5247999880880118706016901342081837356090545654296875f,-0.53639999762773538094506875495426356792449951171875f,-0.0295000057965515427582658958272077143192291259765625f,-1.162799968433379405041705467738211154937744140625f,-0.7304999952465294210668389496277086436748504638671875f,-0.334099972361326269520986897987313568592071533203125f,0.17450000680237998107457997321034781634807586669921875f,-0.0776000170946122835857750033028423786163330078125f,-0.3767999944061042327803079388104379177093505859375f,-0.2749999856948850407434292719699442386627197265625f,-0.262600004279613497004675082280300557613372802734375f,-0.32339999619722359369689002051018178462982177734375f,-1.0622000148773196315232780762016773223876953125f,-1.16709998899102185276888121734373271465301513671875f,-0.381999998942017526815106975845992565155029296875f,0.5702999715626244636723640724085271358489990234375f,0.269100009688734598256587560172192752361297607421875f,-0.9100999872922894695648210472427308559417724609375f,-1.43929996959567052527972919051535427570343017578125f,0.309000006675720140236762745189480483531951904296875f,-0.4928999952137467222001987465773709118366241455078125f,0.23759997239112795597293370519764721393585205078125f,-0.95529999130666265472200393560342490673065185546875f,0.452200027692317263472432387061417102813720703125f,-0.43410000015199157186174261369160376489162445068359375f,0.545999971717595844467041388270445168018341064453125f,-0.101600006894767336262930257362313568592071533203125f,0.463600002148746970220827279263176023960113525390625f,0.1523999943792819777854674612171947956085205078125f,0.5610999784559009828655007368070073425769805908203125f,-0.37770004253387468029501405908376909792423248291015625f,-0.73579997764974847118679690538556315004825592041015625f,0.49129999394416790092776636811322532594203948974609375f,0.411699981260299718854867023765109479427337646484375f,0.736499995797872397673700106679461896419525146484375f,0.1247999852940446796623064074083231389522552490234375f,1.656799956929683848017020864062942564487457275390625f,0.4990999782472849144454585257335565984249114990234375f,1.623999981641769618789794549229554831981658935546875f,-0.0530000442713494113178285260801203548908233642578125f,1.593200006264448465032046442502178251743316650390625f,0.5943999926507472775227824968169443309307098388671875f,1.2407999623388052956585170250036753714084625244140625f,-0.3562000158160925078476566341123543679714202880859375f,0.6704000261336566790504321033949963748455047607421875f,-0.18030000687837599571139435283839702606201171875f,-0.7757999948233360587579454659135080873966217041015625f,-0.6646000032216310415833504521287977695465087890625f,0.4977999987572430118376587415696121752262115478515625f,-0.08160000365376463005162577246665023267269134521484375f,0.890799973738193529015916283242404460906982421875f,0.0639000181540845257899263742729090154170989990234375f,0.50939999965429283435014440328814089298248291015625f,0.2427999984964726021274827871820889413356781005859375f,0.8729000109344726698878957904526032507419586181640625f,0.1543999945506453475019270626944489777088165283203125f,0.783500001817941438275738619267940521240234375f,0.6329000059425828883519216105923987925052642822265625f,0.44719999643713304049441603638115338981151580810546875f,0.1046999860435724460927531254128552973270416259765625f,0.51400001360476010869859919694135896861553192138671875f,0.1682000048115849522023523832103819586336612701416015625f,-0.0188999927848580995259908377192914485931396484375f,-0.34959998064637165526846729335375130176544189453125f,-1.217099951067567520368584155221469700336456298828125f,0.30229998488426179648058678139932453632354736328125f,-0.50300000202655770209503316436894237995147705078125f,-0.539499975025653721871776724583469331264495849609375f,0.115299986258893927271174106863327324390411376953125f,1.2801999811783433269596343961893580853939056396484375f,0.62529997385367852569260094242054037749767303466796875f,0.84359997346401183193620454403571784496307373046875f,-1.39439997372627200178385464823804795742034912109375f,-0.3157999893099070476409906405024230480194091796875f,0.068100035437941297544739427394233644008636474609375f,0.3649999993667005782072010333649814128875732421875f,-0.439599990779161231557736755348742008209228515625f,0.489400015151500777932369601330719888210296630859375f,-0.3534000036478044393106756615452468395233154296875f,-0.14909999672472462606265253270976245403289794921875f,0.473800005045533101366572736878879368305206298828125f,-1.481699955853819528073245237465016543865203857421875f,-0.48899999676644789747825825543259270489215850830078125f,-0.59399999490380306355064021772705018520355224609375f,-1.15380000258684134450959390960633754730224609375f,-0.8519000099547204829519841950968839228153228759765625f,0.5005000464692705985925158529425971210002899169921875f,-0.79710001776889016777971619376330636441707611083984375f,0.83700001168250981464780124952085316181182861328125f,-1.28859998419284860204925280413590371608734130859375f,-0.6946000006884334165846439645974896848201751708984375f,-0.8778999837398526739917770100873894989490509033203125f,0.14440000852048406887462306258385069668292999267578125f,-1.6740000288784504522965335127082653343677520751953125f,0.06760000249147424256079830229282379150390625f,-0.5918000001430510081945612910203635692596435546875f,-0.1820999936848879396933398311375640332698822021484375f,-1.454899922257662847613346457364968955516815185546875f,-1.039099981860816424017457393347285687923431396484375f,-0.0768999999523156674285928602330386638641357421875f,-0.5883000105440618909824479487724602222442626953125f,-0.255299993523211143919837695648311637341976165771484375f,0.331499998908489967863744141141069121658802032470703125f,-0.71050005760788892406054628736455924808979034423828125f,0.131799988948554158696424565277993679046630859375f,-0.853099995428323154555982910096645355224609375f,-0.826399991853535187402712836046703159809112548828125f,-0.96759997458905078193680537879117764532566070556640625f,-0.1924999849125743622835216228850185871124267578125f,-1.074000006452202615747637537424452602863311767578125f,-0.2502000080749393617907117004506289958953857421875f,-0.069199999439716552984691588790155947208404541015625f,-0.00839999791085721536632036077207885682582855224609375f,1.263900059057772917725515071651898324489593505859375f,0.16439999443888719810047405189834535121917724609375f,-0.950900043247640436305800903937779366970062255859375f,-1.1587999994575977069644068251363933086395263671875f,-1.257800046366453816659713993431068956851959228515625f,-0.8659000060878694338128980234614573419094085693359375f,-0.44700000797584660272576684292289428412914276123046875f,0.5428999644242227251567101120599545538425445556640625f,-0.08980001926869218831228636190644465386867523193359375f,1.543999986484646758100325314444489777088165283203125f,1.1532000019431112303180952949333004653453826904296875f,0.023599975401162820443090595290414057672023773193359375f,-0.195400010859965667098236963283852674067020416259765625f,-0.7219000193424525146923542706645093858242034912109375f,-0.0338999999187883371831730983103625476360321044921875f,0.89380003177449129925236093185958452522754669189453125f,0.822400001345574338529331726022064685821533203125f,-0.70280002356767656834790614084340631961822509765625f,1.363899989134073731378293814486823976039886474609375f,-0.1274000110685822306777481571771204471588134765625f,0.82579999981522522745081005268730223178863525390625f,0.1146000225186352139417067519389092922210693359375f,0.117799992666393460449825170144322328269481658935546875f,0.2745000047534704723517506863572634756565093994140625f,0.02289999856650803966573448633425869047641754150390625f,0.84500001136213553909470874714315868914127349853515625f,-0.5832999902412296933107427321374416351318359375f,-0.3193999957427384739361286847270093858242034912109375f,-0.18649998890608543433700106106698513031005859375f,0.120100014524161247120304096824838779866695404052734375f,-0.4872999933220445445414270579931326210498809814453125f,-0.44999999379739163973823679043562151491641998291015625f,0.287599998822063052106301483945571817457675933837890625f,-0.59680000448301395721273365779779851436614990234375f,0.0408999947071075897753189565264619886875152587890625f,-0.03600001962482946016308460457366891205310821533203125f,-0.90870000754594826286592024189303629100322723388671875f,0.26610001526176951802682424386148341000080108642578125f,-1.153800020617246513410236730123870074748992919921875f,0.50110003120601209758433469687588512897491455078125f,-1.3594000502623619919262409894145093858242034912109375f,-0.132499967068432977779224302139482460916042327880859375f,-0.1193000104062260646742288372479379177093505859375f,-0.484700033093989457455563751864247024059295654296875f,2.030900031363964242103747892542742192745208740234375f,-0.2465000091716647023787345460732467472553253173828125f,0.082500014752149741781295233522541821002960205078125f,0.90170002749115252260736497191828675568103790283203125f,-0.37630001480579355899180882261134684085845947265625f,0.067200000013411198551693814806640148162841796875f,-0.6975000032782556758093051030300557613372802734375f,-0.6179999980777497459172309390851296484470367431640625f,-0.85480001925006465768319685594178736209869384765625f,-0.9421000320181256437734873543377034366130828857421875f,-0.6559000096827747317007606397964991629123687744140625f,-0.0376999992087483359881616706843487918376922607421875f,0.346400028584897656802610299564548768103122711181640625f,0.51990000188574148065612234859145246446132659912109375f,0.15970008107572797317885715528973378241062164306640625f,0.49570002064406881725489029122400097548961639404296875f,-1.0536000215947627367540917475707828998565673828125f,-1.389699986974894496682964017963968217372894287109375f,-1.148800003331900132508280876209028065204620361328125f,-1.69430001374036098837905228720046579837799072265625f,0.95360000803470601482558777206577360630035400390625f,0.510300060559810031435290511581115424633026123046875f,0.9016000395655634491731689195148646831512451171875f,0.3988000019907946835928669315762817859649658203125f,0.51909998195394868591989734341041184961795806884765625f,0.32140001172795906736467941300361417233943939208984375f,-0.04000000929459901488627338039805181324481964111328125f,-0.2124000020533802768341047340072691440582275390625f,-0.39809998402893487234877056835102848708629608154296875f,0.139699990756064540420311459456570446491241455078125f,-0.97760000843331151809678658537450246512889862060546875f,0.48529999784454691535984238726086914539337158203125f,-0.70340000757798548658428217095206491649150848388671875f,0.47629998202323964306970083271153271198272705078125f,2.13979997594356508017199303139932453632354736328125f,0.319500001847743408944779730518348515033721923828125f,1.06880001152753845161669232766143977642059326171875f,-0.471100053483247460661687000538222491741180419921875f,0.47119997431635862739085496286861598491668701171875f,-0.053799990218877713488154768128879368305206298828125f,-0.949400022304057955579992267303168773651123046875f,-0.89290000057816509837493867962621152400970458984375f,0.35860002412796021520335898458142764866352081298828125f,-0.350300028932094331679536480805836617946624755859375f,0.4398999918460846370038552777259610593318939208984375f,-0.87689999324679401837556724785827100276947021484375f,0.5888000131294133243642363595427013933658599853515625f,-1.9524999581277366900167180574499070644378662109375f,0.6205000000521534442299298461875878274440765380859375f,-0.4571000167071819220154793583787977695465087890625f,-0.86090002339184279467332316926331259310245513916015625f,0.22890003368854561216494403197430074214935302734375f,-0.1287999952480196608206597375101409852504730224609375f,1.123199982720613387954244899447076022624969482421875f,-0.3785000102370974506271750215091742575168609619140625f,-0.519500013470649779634413789608515799045562744140625f,0.523200019191205623059204299352131783962249755859375f,-0.7305000091791153948861392564140260219573974609375f,0.0972999852195377545882593040005303919315338134765625f,0.23670001374483096157064210274256765842437744140625f,-0.1122000113010399591217947090626694262027740478515625f,-0.511399995690583697438569288351573050022125244140625f,-1.141200008924305731028425725526176393032073974609375f,-0.5356999782502658291605257545597851276397705078125f,-0.9652000304639332117773165009566582739353179931640625f,0.5047999912917617848506779409945011138916015625f,-0.2537000394716855122823062629322521388530731201171875f,-2.116299985450506682127524982206523418426513671875f,-1.5186999753221861464425046506221406161785125732421875f,-0.53379998738765710442066847463138401508331298828125f,-0.478099983656406823939732930739410221576690673828125f,-1.364399985498189860066986511810682713985443115234375f,-1.309299981553852632032430847175419330596923828125f,0.308800015997886756480284020653925836086273193359375f,0.521199994432925972631664990331046283245086669921875f,-0.976999975919723429029772887588478624820709228515625f,0.935999955028295804737581420340575277805328369140625f,-0.06099999392032628975357511080801486968994140625f,0.6789999800026420562204521047533489763736724853515625f,0.571499988198280295392805783194489777088165283203125f,0.55169997723698571689965319819748401641845703125f,-0.21109993710517915843638547812588512897491455078125f,0.89309995862245639131060670479200780391693115234375f,0.378799980682134052045739736058749258518218994140625f,-0.066499996095895852477042353712022304534912109375f,0.62200000005960465188081798260100185871124267578125f,0.22949999633431428236463034409098327159881591796875f,1.1870999703645708223120891489088535308837890625f,0.1681999941945075960347821819595992565155029296875f,0.172099982285499830680919330916367471218109130859375f,0.091399994796514505424056551419198513031005859375f,-0.331399994647502804667738018906675279140472412109375f,-0.490499994724988841454660359886474907398223876953125f,-0.383300003149360524190569776692427694797515869140625f,-0.1515000106990338490930980697157792747020721435546875f,-0.6566000045105815363655210603610612452030181884765625f,0.76629999379515678636920483768335543572902679443359375f,0.359199987667798925627948847250081598758697509765625f,1.23640000880360556578807518235407769680023193359375f,0.1231000149860979053784149073180742561817169189453125f,-0.012500014752149457564200929482467472553253173828125f,-0.7848000080183143989387417605030350387096405029296875f,-0.6833000320017337259059786447323858737945556640625f,0.1735999967843293578795282883220352232456207275390625f,0.81409999426603274486069494741968810558319091796875f,0.2900999770849941317152342890040017664432525634765625f,-0.114799954485893085376346789416857063770294189453125f,0.2484000015243885695070957808638922870159149169921875f,-0.787499990463256693828952847979962825775146484375f,0.03810000962168003102448210483998991549015045166015625f,-0.56499998016282904078622095767059363424777984619140625f,0.5109999872893091765746476085041649639606475830078125f,-1.046099985776096641387766794650815427303314208984375f,-0.0492999653860917153025411607814021408557891845703125f,0.300299970296026064175975989201106131076812744140625f,-0.3003999826967724828818973037414252758026123046875f,0.55849997730553102570638657198287546634674072265625f,-1.0008000684723261652919745756662450730800628662109375f,-2.3855999656260014063491325941868126392364501953125f,-1.57509995748996711739664533524774014949798583984375f,-0.4724000027239316779059663531370460987091064453125f,-1.1150000280141838526759556771139614284038543701171875f,-0.585800055992603319054978783242404460906982421875f,-0.5344999670907848443590637543820776045322418212890625f,1.084199999439716233240460496745072305202484130859375f,0.661199970293045335978376897401176393032073974609375f,-0.593799992454052016910281963646411895751953125f,-0.0791999737724662367099881521426141262054443359375f,-0.83569999726787214111567436702898703515529632568359375f,0.11659998646155045332051258810679428279399871826171875f,-0.00300001355633139343837001433712430298328399658203125f,0.223699996221065955381845924421213567256927490234375f,1.562099968054890819502134036156348884105682373046875f,0.92960000053942126729822348352172411978244781494140625f,0.524100022652745689555331409792415797710418701171875f,-0.090400010710954603609934565611183643341064453125f,0.06019996191859267042900683009065687656402587890625f,1.182499991953373186248654747032560408115386962890625f,1.51109998106360432501560353557579219341278076171875f,-0.78490001449584934078984588268212974071502685546875f,1.00100000956654522354938308126293122768402099609375f,0.05700000050663955875052124611102044582366943359375f,-1.032599981331825400587831609300337731838226318359375f,0.559200006890296918982130591757595539093017578125f,-0.43720002009272584952981333117350004613399505615234375f,0.3455000139474868614541946953977458178997039794921875f,0.10939999749362427916565820851246826350688934326171875f,-0.738400012588500853638606713502667844295501708984375f,0.587000014998019015166619283263571560382843017578125f,0.0289000179119408839856220083674998022615909576416015625f,0.5567000172838569849176337811513803899288177490234375f,-0.65409999875500801636007963679730892181396484375f,0.1656000282481315810656496978481300175189971923828125f,-0.83580000652074826206217039725743234157562255859375f,-0.52240003450065930490353593995678238570690155029296875f,1.71030001462697978098503881483338773250579833984375f,-0.2431999802246689323936834625783376395702362060546875f,1.1125999999582774080408853478729724884033203125f,-0.16500000502914158317935289232991635799407958984375f,-0.184000029027462375097456970252096652984619140625f,0.737600009718537119596248885500244796276092529296875f,-0.558800004971027330924471243633888661861419677734375f,-0.235099985951185086463510742760263383388519287109375f,-0.0707000101387500468064217784558422863483428955078125f,-0.539200062993168582892167250975035130977630615234375f,0.141199990670382857782527707968256436288356781005859375f,0.68940004955455680413223262803512625396251678466796875f,1.354300010238587592681369642377831041812896728515625f,0.01939996776580754644925264074117876589298248291015625f,0.9583000457108017400287280906923115253448486328125f,1.47489998245835263190883779316209256649017333984375f,0.00199994671344772001475575962103903293609619140625f,0.46749998845159990157327456472557969391345977783203125f,0.278799996030329566565342247486114501953125f,-1.2254000395447006699356506942422129213809967041015625f,-1.202899970775842319881121511571109294891357421875f,-0.880399956768751934532701852731406688690185546875f,-0.208599995219707512461582155083306133747100830078125f,-0.23529999272227275497471055132336914539337158203125f,-0.02249999776482614066708265454508364200592041015625f,1.077299985629320389080021413974463939666748046875f,-0.3780999808251859217733681362005881965160369873046875f,-0.8824000089265406909788680422934703528881072998046875f,0.56840000295862569412719267347711138427257537841796875f,0.3113999984659248099205797188915312290191650390625f,-0.2848999730147421249881745097809471189975738525390625f,-0.54459997259974546324201583047397434711456298828125f,-1.467799976778030757174065001890994608402252197265625f,-0.593999973893165389426940237171947956085205078125f,0.54519999857544831911582150496542453765869140625f,1.237099970662593140247054179781116545200347900390625f,1.0427000355303288614550183410756289958953857421875f,-0.41199999916553498024995860760100185871124267578125f,0.6557999443829061192445806227624416351318359375f,0.73829999895840880963504559986176900565624237060546875f,0.0086999825119971063713819603435695171356201171875f,0.8668000299267470953878955697291530668735504150390625f,-0.0678999823242423250491128783323802053928375244140625f,-0.587600013667345155710108883795328438282012939453125f,-0.07270000967681411463416907281498424708843231201171875f,-0.4640000043660401640721602234407328069210052490234375f,0.4254000112324949345321556393173523247241973876953125f,-0.44620002423524862678050340036861598491668701171875f,-0.4992000010043378122048807199462316930294036865234375f,0.172499977499246615053607456502504646778106689453125f,0.41010001847296972954381999443285167217254638671875f,0.1641999702334402400083490647375583648681640625f,-0.33530002386569968120255680332775227725505828857421875f,-0.49119999979734441097889430238865315914154052734375f,0.4404000041171907131598572959774173796176910400390625f,-0.7964999837279318484206669381819665431976318359375f,0.30240001362040658250407432205975055694580078125f,0.00849996640905736899895828173612244427204132080078125f,0.014400039663911012866037708590738475322723388671875f,1.014400031617283648444072241545654833316802978515625f,1.0125999743655318408031007493264041841030120849609375f,-0.918800002512335733939607962383888661861419677734375f,-1.443600017459690132426430864143185317516326904296875f,0.02289998463392262095794649212621152400970458984375f,-0.2546000702023512030081064949627034366130828857421875f,-1.9749000031709673663726789527572691440582275390625f,-1.928399985617399448045716781052760779857635498046875f,0.315200006920099529139633887098170816898345947265625f,0.1700000227615234837941216028411872684955596923828125f,-1.0759000006675716321069558034650981426239013671875f,-1.7969000154122711787607613587169907987117767333984375f,-0.3902000133275986826220105285756289958953857421875f,0.03279999078512219057301990687847137451171875f,0.1274999802745877541809704780462197959423065185546875f,-0.7540999972648918525663930267910473048686981201171875f,0.863599994977563678588694529025815427303314208984375f,-0.105699971768260514437542951782234013080596923828125f,-1.46819998473227020241438367520458996295928955078125f,0.946299969431758203342042179428972303867340087890625f,0.38500000752508611867597210220992565155029296875f,0.76190000278353675611242579179815948009490966796875f,0.28110000743865981576163903810083866119384765625f,0.23759999653100916106041040620766580104827880859375f,1.175399988657236605149591923691332340240478515625f,0.0261999804258348234498043893836438655853271484375f,0.55069996722340608386048188549466431140899658203125f,0.828599991792440704330147127620875835418701171875f,0.36289998448491100901946992962621152400970458984375f,-0.12400000563263890995813198969699442386627197265625f,0.0772000214338302015448789461515843868255615234375f,0.1823999772429463206435684696771204471588134765625f,-0.038200014422833827865133571322076022624969482421875f,-0.29469996585249891296598434564657509326934814453125f,0.228300001901387883407323897699825465679168701171875f,-0.9450999824494121792639589330065064132213592529296875f,0.41340000618100158913392760950955562293529510498046875f,-0.3825999903470276297667851395090110599994659423828125f,0.836099988327919874109284137375652790069580078125f,-1.11629998694062226860523878713138401508331298828125f,0.4516000063359737470847221629810519516468048095703125f,-1.302300011259317269463053889921866357326507568359375f,0.9372000205770139036332011528429575264453887939453125f,-0.5115000255256896277700207065208815038204193115234375f,0.12879998973458983702045088648446835577487945556640625f,-1.0806999921083448867875631549395620822906494140625f,0.300000023245811930916460141816060058772563934326171875f,-0.51600002487748886093044120570993982255458831787109375f,-1.10910005532354194457411722396500408649444580078125f,-1.5590000292509795798423510859720408916473388671875f,-1.1569999882876869090608806800446473062038421630859375f,0.19810000228285762347013587714172899723052978515625f,0.85849998114258074810578591495868749916553497314453125f,0.401600004026293699865846065222285687923431396484375f,-0.6176000187337391533759500816813670098781585693359375f,0.477399965322018005053905653767287731170654296875f,-0.06610001656562047855913988314568996429443359375f,0.313199986371397898210489074699580669403076171875f,-0.2257000178873538320800662404508329927921295166015625f,1.324399993917346751715058417175896465778350830078125f,0.240500004187225613350165076553821563720703125f,-0.0219000275567176050373063844745047390460968017578125f,-0.1738000136509534332418525082175619900226593017578125f,0.57640002530664224966727715582237578928470611572265625f,0.4972000048190350351973165743402205407619476318359375f,0.41919998856186868607665019226260483264923095703125f,-0.170200004759430978396039790823124349117279052734375f,0.88920002469718439908064055998693220317363739013671875f,-0.4274999882280827723235461235162802040576934814453125f,-1.00159996565580389216165713150985538959503173828125f,1.0055999582499268729662844634731300175189971923828125f,0.244400018802284790808698744513094425201416015625f,0.052499971091747976714714241097681224346160888671875f,0.1425999965310094097503679222427308559417724609375f,0.1264999881237744983053516989457421004772186279296875f,-0.44889999323189211377638230260345153510570526123046875f,0.3783999761551621876520812293165363371372222900390625f,0.2911999715596440552189960726536810398101806640625f,-0.3414999993368983410846340120770037174224853515625f,-0.51929998736530524894305926864035427570343017578125f,0.09380000068694338555275180624448694288730621337890625f,-0.51620000855177605725288003668538294732570648193359375f,0.42409996751844925011454279228928498923778533935546875f,-0.68940000740289608671673704520799219608306884765625f,0.18589998911917149371220148168504238128662109375f,-0.2072000030681493409900895130704157054424285888671875f,-0.36110001816749548453344687004573643207550048828125f,0.350500004895031580520736724793096072971820831298828125f,1.0796000276409085660844766607624478638172149658203125f,-0.8322000154361131052382916095666587352752685546875f,1.02390002677813196640954629401676356792449951171875f,-0.8112000120908022804400161476223729550838470458984375f,0.394599991524219984029286933946423232555389404296875f,-0.03599999116361118911555649901856668293476104736328125f,0.7026999995440241786326396322692744433879852294921875f,-0.6824000293970111652441801197710447013378143310546875f,1.2498000305190686276546330191195011138916015625f,-0.217200006607174866957166159409098327159881591796875f,0.542000008404254884908368694595992565155029296875f,-0.873100006753205537535222902079112827777862548828125f,-2.3378999482750888461168869980610907077789306640625f,-1.181299956467749101562958458089269697666168212890625f,-0.737000001251697511861493694595992565155029296875f,0.08090000521242701747581804738729260861873626708984375f,0.8368999956682328100754375554970465600490570068359375f,0.97820001287683766566516396778752095997333526611328125f,-0.1736000142559415426735114351686206646263599395751953125f,-0.1239000373020766598841646555229090154170989990234375f,-1.79579998834133203189367122831754386425018310546875f,-1.38729999009966764589307786081917583942413330078125f,-0.269599959747492956552150644711218774318695068359375f,0.2586000250592823679340881426469422876834869384765625f,-0.8652000164195896569907517914543859660625457763671875f,-0.3059999659806482696922103059478104114532470703125f,-0.045399994468688920079557647113688290119171142578125f,-0.111999992311000884370741914608515799045562744140625f,0.308699981394410460922017591656185686588287353515625f,-0.361399958363175954900725628249347209930419921875f,0.666799965348840117940198979340493679046630859375f,0.12779998095035605132352429791353642940521240234375f,-0.5198000014245511835042634629644453525543212890625f,-0.2758999782972038072870191172114573419094085693359375f,0.61340000053718679584591200182330794632434844970703125f,-0.10819996366575423962785862386226654052734375f,-0.0829999649040404907651691246428526937961578369140625f,0.377599980288743974909948519780300557613372802734375f,0.45980000038146950913642285740934312343597412109375f,0.082300004740059495844661796581931412220001220703125f,0.473699975322186883630593001726083457469940185546875f,-0.48599999656528203306748991963104344904422760009765625f,0.659999989755451377249073630082421004772186279296875f,0.072400004959106478708008580724708735942840576171875f,0.203199994790554061552256825962103903293609619140625f,0.517600001448392799119346818770281970500946044921875f,0.7749000118136404324786781216971576213836669921875f,0.988099996173381622810438784654252231121063232421875f,-0.19879996410459310762774975955835543572902679443359375f,-1.468099978590011733814435501699335873126983642578125f,-0.04669995145797789515285103334463201463222503662109375f,-1.6855999581754212979234353042556904256343841552734375f,0.53899998052418229832483120844699442386627197265625f,-0.423499976411461975800420987070538103580474853515625f,-0.130599993528425439848206224269233644008636474609375f,0.69900001561641733616170313325710594654083251953125f,0.5463000039651986217137391577125526964664459228515625f,-1.636400021842122409765352131216786801815032958984375f,-0.000900020895898023809422738850116729736328125f,-1.0251999894112344691166072152554988861083984375f,-0.616699980515241552581073847250081598758697509765625f,-0.007200028884410958340822617174126207828521728515625f,0.25730001155734072426639613695442676544189453125f,-0.16130002419352518217010583612136542797088623046875f,-0.204199993404746038550001685507595539093017578125f,-1.433699966120720592499537815456278622150421142578125f,-0.64889999628663030950548318287474103271961212158203125f,-1.050399994841217932162180659361183643341064453125f,-0.670100008563697002728076768107712268829345703125f,0.12430001921653843766080171917565166950225830078125f,-0.4552999989062549435203663961146958172321319580078125f,-0.54499998107552638515471699065528810024261474609375f,-0.2796999967351554428063309387653134763240814208984375f,-1.5001000085264430961018433663411997258663177490234375f,-0.5249000142350792597056852173409424722194671630859375f,-0.73209995985925269135208282023086212575435638427734375f,-0.6344999909698965101512158071273006498813629150390625f,0.56979997057914744118534144945442676544189453125f,0.1348999917343258747592926738434471189975738525390625f,-0.3398000014245508015875429919105954468250274658203125f,0.17770000647306438690264940305496565997600555419921875f,0.38390003517866155835491781544988043606281280517578125f,0.35310001014322056622773970957496203482151031494140625f,-1.89780004530474588619881615159101784229278564453125f,-1.6433999962344765233268617521389387547969818115234375f,-0.62340000362917835641241026678471826016902923583984375f,-0.8720999898850916309811509563587605953216552734375f,1.097599977755546518665141775272786617279052734375f,-0.31820001624822591423935591592453420162200927734375f,1.72659994270801586679908723453991115093231201171875f,0.802299982574581793670631668646819889545440673828125f,-0.14630001383721802366011388585320673882961273193359375f,0.29330000871866968026324684615246951580047607421875f,0.833699974949657640621580867446027696132659912109375f,-0.02360001369714748165051787509582936763763427734375f,-0.786699998545646650427443091757595539093017578125f,0.360800010395049941536171900224871933460235595703125f,0.26680002614557796203342832086491398513317108154296875f,-0.17409999456405655937629717300296761095523834228515625f,0.26639999129474156713826005216105841100215911865234375f,-0.9252000196792182773464219280867837369441986083984375f,-0.8052000352323054332970286850468255579471588134765625f,1.05760002642646444570573294186033308506011962890625f,-0.440300006803870047633608919568359851837158203125f,2.25460000582933428603382708388380706310272216796875f,-1.20039996608197707672616161289624869823455810546875f,0.81360001503825163382543905754573643207550048828125f,-0.2408000153496860917812227853573858737945556640625f,0.42790001629888985146266122683300636708736419677734375f,0.2445000214800236992829240989522077143192291259765625f,0.5952999955162407985653771902434527873992919921875f,-0.641200003075599767043968313373625278472900390625f,-0.87229996521472941140018519945442676544189453125f,0.09460001663267636473619859316386282444000244140625f,-0.804000004701316495214769020094536244869232177734375f,-0.60300001703947803122218829230405390262603759765625f,-0.202899976158886385935176122075063176453113555908203125f,-1.0073000091545283307681302176206372678279876708984375f,1.0528000548601148533833793408120982348918914794921875f,-0.00669998846873642861510234070010483264923095703125f,1.69499999210238438962505824747495353221893310546875f,0.7239999965801835468681701968307606875896453857421875f,1.0354999939799309771615298814140260219573974609375f,0.4171999762088063334886101074516773223876953125f,1.322799954463542260185704435571096837520599365234375f,0.7174999941512940626608951788512058556079864501953125f,-0.041699988561868739367355374270118772983551025390625f,-0.36290000706017000453584842034615576267242431640625f,-0.10639997959733005217231038841418921947479248046875f,-0.2296999972566962089359776655328460037708282470703125f,-0.465299999725818480555972200818359851837158203125f,-0.0891999994397163487036550577613525092601776123046875f,1.4160999920904639992613738286308944225311279296875f,-0.790099998319149232628433310310356318950653076171875f,-0.17729998264908797267480622394941747188568115234375f,0.4154000170439484662665563519112765789031982421875f,0.886899970969558193445436700130812823772430419921875f,-1.148800012086331978622411043033935129642486572265625f,0.473000028364360947108480104361660778522491455078125f,1.3391000242173671797019096629810519516468048095703125f,0.68989997083544807310317992232739925384521484375f,-0.58550002542138113170722135691903531551361083984375f,-0.738900011485814189882148639298975467681884765625f,1.225600020015239710602372724679298698902130126953125f,-0.4155000151395800145337489084340631961822509765625f,-0.74940000337958334597487919381819665431976318359375f,-0.767799986910819853846987825818359851837158203125f,0.53910000097155563736350813996978104114532470703125f,-0.4566999913930891441538051367388106882572174072265625f,1.50869999651908859306104204733856022357940673828125f,0.1331999703899027220899142776033841073513031005859375f,0.1552999729037285447930116788484156131744384765625f,0.57750001922249760699656917495303787291049957275390625f,1.9967000026434658099816488174838013947010040283203125f,-0.317599991986155405498948312015272676944732666015625f,0.4962000105634327695725005469284951686859130859375f,0.3837000371620060690247555612586438655853271484375f,0.5040999684870239860856599989347159862518310546875f,0.11310001591742036008980676342616789042949676513671875f,-1.6595000627934937131868764481623657047748565673828125f,0.1672000026956201867989193488028831779956817626953125f,-0.2899000012338159848468421841971576213836669921875f,0.248899997329711908378158113919198513031005859375f,-0.79600002312660222969498136080801486968994140625f,0.0828000122427940965508241788484156131744384765625f,0.866199965226650103744532316341064870357513427734375f,0.7263999995276331222981980317854322493076324462890625f,0.3592000007808209272042176962713710963726043701171875f,0.386100004048645362164648986436077393591403961181640625f,1.213599966721236267375161332893185317516326904296875f,0.68500001359730988692575692766695283353328704833984375f,0.165700002837180893067170472932048141956329345703125f,0.072499992810189706204937465372495353221893310546875f,0.085499905318021429678765343851409852504730224609375f,0.3689999876767398045984691634657792747020721435546875f,-1.4096000098526466626225328582222573459148406982421875f,-0.810300032210350185124525523860938847064971923828125f,-0.00210002669096009952909298590384423732757568359375f,-0.0867999961197376546095938465441577136516571044921875f,-1.02539998239874829977225090260617434978485107421875f,-0.4787000019207596768211487869848497211933135986328125f,0.46380000199079507439137159963138401508331298828125f,-0.57460000247657294636383085162378847599029541015625f,-0.8752999867618083129627848393283784389495849609375f,0.756399991071223798400069426861591637134552001953125f,1.3777000213742258694082920555956661701202392578125f,0.310600038641691611474016099236905574798583984375f,-1.5364000304102898297742285649292171001434326171875f,-1.7439000295162199183351958708954043686389923095703125f,-0.573299986404180117460782639682292938232421875f,0.2620999874264000606416402661125175654888153076171875f,0.83469999930560534462387067833333276212215423583984375f,-1.43910002432912609293680361588485538959503173828125f,-0.300399978412687840290118401753716170787811279296875f,-0.190199963706731534074378942023031413555145263671875f,0.97490000615119942750652626273222267627716064453125f,-0.254300023239851125111954388557933270931243896484375f,1.12410000022649825268672429956495761871337890625f,-0.37020000006556508242994141255621798336505889892578125f,0.47919998703449928623143705408438108861446380615234375f,-0.360899989640712792748900028527714312076568603515625f,0.1693000117659571035488852430717088282108306884765625f,1.19849999809265117534096134477294981479644775390625f,0.3574000111058353024162670408259145915508270263671875f,-0.4966000052928922769979180884547531604766845703125f,0.3166999990299341760646711918525397777557373046875f,0.49870001276135411671930341981351375579833984375f,-0.15810002042502180330529881757684051990509033203125f,1.59550003831088549333117043715901672840118408203125f,-0.32710003943145304905470993617200292646884918212890625f,0.34560000008493663159470088430680334568023681640625f,0.26350001094490294040184608093113638460636138916015625f,2.301500031411647828605282484204508364200592041015625f,0.2077000088572500402506193495355546474456787109375f,1.4466999934792514892478720867075026035308837890625f,0.18479999606013297608342327293939888477325439453125f,0.0109000189960000160027675519813783466815948486328125f,0.262600015157461397219407217562547884881496429443359375f,0.53009996628165279020095113082788884639739990234375f,-0.1542000088647010880293919399264268577098846435546875f,-0.36099999675154759160022877040319144725799560546875f,-0.2968999823316929020933230276568792760372161865234375f,-0.138999953329562497827964762109331786632537841796875f,-0.3930000087693332755378605725127272307872772216796875f,1.0424000007495284325642614930984564125537872314453125f,0.340000011622905773389646810755948536098003387451171875f,0.4104000384271146995018852976500056684017181396484375f,0.280900020113587334247284843513625673949718475341796875f,1.70769998948573942243456258438527584075927734375f,0.9621000222206117502565803079050965607166290283203125f,0.50919996933937117233881508582271635532379150390625f,0.3158999927699566168115552500239573419094085693359375f,0.32900001040101045646224520169198513031005859375f,1.208100038194656722367881229729391634464263916015625f,0.1844000009953976615406645578332245349884033203125f,-1.106700062918663629574211881845258176326751708984375f,-0.013299982008337707384271197952330112457275390625f,1.254400025582314004424233644385822117328643798828125f,0.059300017651915393201988990767858922481536865234375f,0.35409998197257497309919926919974386692047119140625f,-0.59839996199905931550944160335347987711429595947265625f,0.278200018967688256221748588359332643449306488037109375f,-1.415099978537857960958490366465412080287933349609375f,-1.4668999922037126992790945223532617092132568359375f,-0.568799959131330634676260160631500184535980224609375f,-0.3929000072464348125578226245124824345111846923828125f,0.96569995699748456541300356548163108527660369873046875f,1.0761000186145306400931076495908200740814208984375f,0.38899996435642236747298738919198513031005859375f,-0.0338000069081783038882349501363933086395263671875f,1.213500034227967372402190449065528810024261474609375f,0.4525999986171722611061341012828052043914794921875f,-0.548200034055114304720746076782234013080596923828125f,-1.03810002396404765789839075296185910701751708984375f,0.9754000286296011790199145252699963748455047607421875f,-0.46889997952282413962166174314916133880615234375f,0.59749998316168773104806177798309363424777984619140625f,-1.0741999852091073108795171719975769519805908203125f,0.7801999854996803929196858007344417273998260498046875f,-0.5995000031515953509142491384409368038177490234375f,0.45899997046589857685461311120889149606227874755859375f,-0.1154000263758001576519518494023941457271575927734375f,-1.4538999813109636516372802361729554831981658935546875f,0.31209998375698899675256825503311119973659515380859375f,-0.66989998707771292174584232270717620849609375f,0.30040001890659340233469265513122081756591796875f,0.482900004750490108307303671608678996562957763671875f,-0.1388000362634664863747957497253082692623138427734375f,-0.011200003224611176921854394095134921371936798095703125f,-1.1171999881297340895258685122826136648654937744140625f,0.02240000320821999846288008484407328069210052490234375f,-0.54580000294446995301456126981065608561038970947265625f,0.1432000039547682013196805428378866054117679595947265625f,-1.24399996871501183903774290229193866252899169921875f,-0.78519999537169937742220326981623657047748565673828125f,-0.881799992096424123388942462042905390262603759765625f,-0.3330999915912749020918681708280928432941436767578125f,-0.78959996311888147602076060138642787933349609375f,-0.539999990668147855643610455445013940334320068359375f,1.0693999916076661893526988933444954454898834228515625f,0.182599985634535499912090017460286617279052734375f,1.2011999778926369941700613708235323429107666015625f,0.461800001521408542970448252162896096706390380859375f,0.34050000686943515670890292312833480536937713623046875f,0.5956000137940049565798972253105603158473968505859375f,1.075900007633864507994303494342602789402008056640625f,0.565500027209520528259645288926549255847930908203125f,0.13769997577667236754450641456060111522674560546875f,1.0724999868124722723905506427399814128875732421875f,0.455199951413274295219935083878226578235626220703125f,0.217599994929134898935529918162501417100429534912109375f,-0.298200000490247862927617461537010967731475830078125f,-0.2647999927073716452952112376806326210498809814453125f,-0.029399996487796453692453724215738475322723388671875f,0.34119999838173387107786993510671891272068023681640625f,0.0215999904662371733365944237448275089263916015625f,-1.3476000312507150358243279697489924728870391845703125f,0.56030001328587530906588654033839702606201171875f,0.7080000009834765961613811668939888477325439453125f,0.36490002805590560086557161412201821804046630859375f,-0.17730000269114964961403302368125878274440765380859375f,-0.77739999449104046913561205656151287257671356201171875f,-0.29349998300522539551593581563793122768402099609375f,-0.77239997251182768156496649680775590240955352783203125f,-1.0492999903082846913093817420303821563720703125f,-0.6754999763965610082294688254478387534618377685546875f,-0.295799993705749830752438356284983456134796142578125f,-0.093799986977875227101009159014211036264896392822265625f,-0.6347999902114274117792547258432023227214813232421875f,0.0846999915197494124274868454449460841715335845947265625f,0.1127999957397580743645448819734156131744384765625f,0.90309999218732095460637765427236445248126983642578125f,1.083199982646107617512143406202085316181182861328125f,-0.2647000110402704109446858637966215610504150390625f,0.13750000208616253161153508699499070644378662109375f,0.5964999843239784382831203402020037174224853515625f,0.15910002958178548482237602001987397670745849609375f,0.213300006707012546058876978349871933460235595703125f,0.3431000061571591874809428190928883850574493408203125f,0.4307000055193899168415327949333004653453826904296875f,0.32480000097751560161896122735925018787384033203125f,0.0764999868571758501190060997032560408115386962890625f,0.58219996238797933063580103407730348408222198486328125f,0.2453999904826285050063461312674917280673980712890625f,-0.20820000607818356375133816982270218431949615478515625f,0.05049999502301216747213175040087662637233734130859375f,0.167299996283650387329799968938459642231464385986328125f,0.243999992780387364721406129319802857935428619384765625f,0.347199966150522509877873744699172675609588623046875f,0.23080000573843728606249214863055385649204254150390625f,-0.520399967497586768416795166558586061000823974609375f,0.320299994882941252427599465590901672840118408203125f,0.1542000065922735085877093297312967479228973388671875f,0.255200007441639897809437798059661872684955596923828125f,-0.8469000059127811397274854243732988834381103515625f,-0.451300000500679032722217698392341844737529754638671875f,1.045000000037251908224789076484739780426025390625f,-0.5940000086873766793615914139081723988056182861328125f,0.4993999988719817206828111011418513953685760498046875f,-0.37310001271367099207054707221686840057373046875f,0.070700044709443243817759139346890151500701904296875f,1.1194000290095804661660849887994118034839630126953125f,1.7997000218808647531432143296115100383758544921875f,-0.804699978369473711836690199561417102813720703125f,-1.7456000623345371280947802006267011165618896484375f,0.30169992985129301388269595918245613574981689453125f,2.6779000034093858317874037311412394046783447265625f,1.287600016796588864309569544275291264057159423828125f,-1.3663999767661092477055717608891427516937255859375f,-0.89009998848438254270831748726777732372283935546875f,0.1278000146269793191322605707682669162750244140625f,0.8793000001430513190570081860641948878765106201171875f,-0.10890003294348715723316445291857235133647918701171875f,2.054800015692413328594057020382024347782135009765625f,-0.60560001032948473920214382815174758434295654296875f,-0.0623999893501399061079837338184006512165069580078125f,0.579399941353500036456125599215738475322723388671875f,0.072699990379810675023009025608189404010772705078125f,0.419100008086860764677794577437452971935272216796875f,0.481699961739778803604394852300174534320831298828125f,-0.0788999797835947536128742285654880106449127197265625f,-1.25340002182722098922340592253021895885467529296875f,-1.0709000124394900677060604721191339194774627685546875f,1.36450004476308794920669242856092751026153564453125f,1.2130000151395796681441652253852225840091705322265625f,-0.68419998349547395122272064327262341976165771484375f,-0.6442000116586683589048334397375583648681640625f,0.5182000251889231634550014859996736049652099609375f,0.05939999846220000989660547929815948009490966796875f,0.99920002465993162932278437438071705400943756103515625f,0.13240001032352444187978335321531631052494049072265625f,-0.243299980443715979649965674980194307863712310791015625f,0.476199995289742883386452376726083457469940185546875f,0.0797000048935416760542693737079389393329620361328125f,0.7409000072240827794445294784964062273502349853515625f,-0.31860005229115540004158901865594089031219482421875f,0.2844999924972650973131749196909368038177490234375f,0.04910002660155310394429761799983680248260498046875f,-0.0831000153586265444261016455129720270633697509765625f,0.93299997401237533267703838646411895751953125f,1.003500006288290169464971768320538103580474853515625f,-0.01969999699592595021613306016661226749420166015625f,-0.461199984300136645032353044371120631694793701171875f,0.132600044214724821500794860185123980045318603515625f,-0.12269998598396780753461143831373192369937896728515625f,-0.1851999802470203349002986215054988861083984375f,-0.8194000008463859785479144193232059478759765625f,-1.645999992132185951732026296667754650115966796875f,-1.17130000392794642039007158018648624420166015625f,0.0581999909162522754968449589796364307403564453125f,0.11020000736713431166435839259065687656402587890625f,0.726299943578244011632705223746597766876220703125f,0.692899991935491232908361780573613941669464111328125f,-0.026099986875056657709137653000652790069580078125f,-0.72910000261068363958116833600797690451145172119140625f,-0.33459998891651565600113826803863048553466796875f,-0.351800010871887280927694519050419330596923828125f,0.775299996820092207627794778090901672840118408203125f,0.45530001328587543785175739685655571520328521728515625f,1.100000032801180527286533106234855949878692626953125f,0.1265000029876828246955255963257513940334320068359375f,-0.13289998584464213227107620696187950670719146728515625f,0.09579998610615714227378703071735799312591552734375f,1.489300008264184072004354675300419330596923828125f,-0.51180007300972985007092574960552155971527099609375f,-0.5416000158727169999650641329935751855373382568359375f,0.71919997261762613760538442875258624553680419921875f,-1.107599955813586944941562251187860965728759765625f,0.461100007140636147795476063038222491741180419921875f,-0.020899991577863819003368917037732899188995361328125f,-0.1452999858677390676575669203884899616241455078125f,0.0986000038251280130907616694457828998565673828125f,-0.46760003713667419589228302356787025928497314453125f,-0.5033000250354409121911203328636474907398223876953125f,0.868699991527199788521329537616111338138580322265625f,0.84829999493509544716829395838431082665920257568359375f,0.7714000060282650839127427389030344784259796142578125f,0.2599999957904219893833897003787569701671600341796875f,-0.24609999322667708288037147212889976799488067626953125f,-0.0066999955654143406036382657475769519805908203125f,-1.33680002517700113884302481892518699169158935546875f,-0.8715000188946724346550354312057606875896453857421875f,0.4695000085532665679011188331060111522674560546875f,-0.30629999845176936190682681626640260219573974609375f,0.95409998726248712586084366193972527980804443359375f,0.1495999940201642175452434457838535308837890625f,0.01909996058940865992781255044974386692047119140625f,0.2273000085771084144425913109444081783294677734375f,-0.3748000037342309465060452566831372678279876708984375f,-0.16240000026524070708688896047533489763736724853515625f,0.94689998534917851902292795784887857735157012939453125f,1.1179000113070010069549198306049220263957977294921875f,0.034599999887495791472957762380247004330158233642578125f,0.481400009455531796564997648602002300322055816650390625f,-0.2121999919295312342359949298042920418083667755126953125f,-0.037199996642023329396220532316874596290290355682373046875f,-0.265900010912120510209177837168681435286998748779296875f,-0.513899975275993625700721167959272861480712890625f,-0.6119000313937659196739105027518235146999359130859375f,-1.376299986940622499531627909163944423198699951171875f,-0.43520001001209107638345585655770264565944671630859375f,-1.367199998262524918146709751454181969165802001953125f,-0.3384000175058847759146374301053583621978759765625f,-0.73080002477467065347127572749741375446319580078125f,-1.248500002115964235827050288207828998565673828125f,-0.46229995374083454606761733884923160076141357421875f,0.068299979065356897223182386369444429874420166015625f,-1.3486000411152845313011994221596978604793548583984375f,-1.2520000100806345511017525495844893157482147216796875f,-1.402699964526294973410358579712919890880584716796875f,-0.30179992731362637048420083374367095530033111572265625f,0.675199979874491429399085973273031413555145263671875f,0.5917999631881720734583041121368296444416046142578125f,0.53240001479387277782961973571218550205230712890625f,0.5340999933719634551465560434735380113124847412109375f,-0.67609999298453349325654926360584795475006103515625f,-0.1260999640762812123995217916672118008136749267578125f,0.66780000479221346409985926584340631961822509765625f,-0.1650999631524092681189586073742248117923736572265625f,0.03750001080334186553955078125f,0.88449994827806988073604088640422560274600982666015625f,0.9332999851375820554721940425224602222442626953125f,0.19820004515647848819526188890449702739715576171875f,-0.639700016218423428426831378601491451263427734375f,0.49479997795522212555852092918939888477325439453125f,1.4123000053733587577653452171944081783294677734375f,-0.7576999797999857122476896620355546474456787109375f,0.101799965721368845805727687547914683818817138671875f,0.2700999943703414896134518130565993487834930419921875f,-0.3520999617964033490125075331889092922210693359375f,0.09100000256299978218521573580801486968994140625f,-0.5852000271856787616542305840994231402873992919921875f,-0.580200031805038651810946248588152229785919189453125f,-0.322800006543099871070268136463710106909275054931640625f,-0.34149999712035039589608231835882179439067840576171875f,1.145900031214952985436639210092835128307342529296875f,-1.20220001877024795788884148350916802883148193359375f,1.0232000172168016494111952852108515799045562744140625f,-1.4905000106692316563794520334340631961822509765625f,0.2634000098317861304764164742664434015750885009765625f,-1.03580000205039990390787352225743234157562255859375f,0.4487000155180691507439405540935695171356201171875f,0.457199975985288720181642929674126207828521728515625f,-0.41440000501871099292117150980629958212375640869140625f,-1.1828000028550624822543113623396493494510650634765625f,0.0960000164955856494231056785793043673038482666015625f,0.4664999780654905947585575631819665431976318359375f,-0.59970002568066138604052639493602328002452850341796875f,-1.3130000059381121690904592469451017677783966064453125f,-0.3390000025033952368858081172220408916473388671875f,-1.654700006905198250706234830431640148162841796875f,-1.12210001210644794156223724712617695331573486328125f,0.56490000432580733669141181962913833558559417724609375f,0.42610000855624619031658539825002662837505340576171875f,1.74509998297095325625605255481787025928497314453125f,1.06879996943175736845432766131125390529632568359375f,1.141099984788894516896107234060764312744140625f,1.993299978581070330818647562409751117229461669921875f,1.7441999707177282541437079999013803899288177490234375f,0.51649998369067962045164676965214312076568603515625f,-0.73789997409135121841927684727124869823455810546875f,0.0496999825417991925036176326102577149868011474609375f,-0.10759999340176573667804404976777732372283935546875f,0.15100000807642910416461745626293122768402099609375f,0.5079000210672617310336818263749592006206512451171875f,-0.226699988859891288228709527174942195415496826171875f,-0.6327000374302269580795154979568906128406524658203125f,0.57749999651685346435669998754747211933135986328125f,0.20190002373829518678149952393141575157642364501953125f,0.056199978581815912548336200416088104248046875f,0.06470000435337464050888911515357904136180877685546875f,-0.170200001555681268428088515065610408782958984375f,-0.9669999850094317395132748060859739780426025390625f,-1.24409995194673594909318126155994832515716552734375f,-1.8108999662458895585359641700051724910736083984375f,0.66019996244013345876311404936132021248340606689453125f,-0.0068999973073602394180170449544675648212432861328125f,-0.342299964134395506842878376119188033044338226318359375f,-1.2974999896809455979251879398361779749393463134765625f,0.22009999414682379637042686226777732372283935546875f,0.36189999652504933891350447083823382854461669921875f,-0.60689997536540030154128544381819665431976318359375f,0.5936000123932954064542855121544562280178070068359375f,-0.5261000080347060237073719690670259296894073486328125f,0.33389998902231454724898185304482467472553253173828125f,0.197800037127733308039978510350920259952545166015625f,0.45020001496672623630956877605058252811431884765625f,-0.822699995446205090132707482553087174892425537109375f,0.322299991664290297332939871921553276479244232177734375f,-0.6650000028312208488756596125313080847263336181640625f,-0.183500010460615481377999458345584571361541748046875f,0.3866000140845773369591142909484915435314178466796875f,-0.67640004531145170307837588552501983940601348876953125f,0.3888000402867799465411735582165420055389404296875f,-0.224699994388222579200231621143757365643978118896484375f,0.463299994599819076057656275224871933460235595703125f,-0.1608000148281456542775913476361893117427825927734375f,0.3230000178217888873177798814140260219573974609375f,0.075600009323656625070952941314317286014556884765625f,1.651699994522333181379281086265109479427337646484375f,0.9985999654173853823380113681196235120296478271484375f,0.4826999771177769016361480680643580853939056396484375f,0.099900042547286016514362927409820258617401123046875f,1.3738000097021456635815184199600480496883392333984375f,0.0531999307155606171448880559182725846767425537109375f,0.0664999725893142823451853473670780658721923828125f,0.515200011427700221844361294643022119998931884765625f,0.750399976587295736152327663148753345012664794921875f,-0.12920008873492516698888721293769776821136474609375f,-1.9747000677764419140913787487079389393329620361328125f,-0.5519999795705066159001717096543870866298675537109375f,0.0710000196993352261642939993180334568023681640625f,-0.4104000315353275585295023120124824345111846923828125f,-1.20560001897215851585087875719182193279266357421875f,-0.75379998634457567874278538511134684085845947265625f,-0.00489997560381905117310452624224126338958740234375f,0.23529998393058804140309803187847137451171875f,-0.71709999286532433160346045042388141155242919921875f,-0.63750001534819578097312842146493494510650634765625f,0.5625999984681604448866210077540017664432525634765625f,0.68060000474154891403344436184852384030818939208984375f,-0.611199963252246458722538591246120631694793701171875f,-0.884999993927776618107827744097448885440826416015625f,-0.2499000051453712512028459968860261142253875732421875f,1.067900020992755827364817378111183643341064453125f,-0.2429999670088296426229135249741375446319580078125f,-1.01829999504685364541956005268730223178863525390625f,0.51350001835823011475667954073287546634674072265625f,1.10959999882578852492542864638380706310272216796875f,0.0551000051736831863991028512828052043914794921875f,-0.21070000119805332161604383145458996295928955078125f,0.92070000030398357893091088044457137584686279296875f,0.7181999900221824884027910229633562266826629638671875f,0.1995000019967558824163234021398238837718963623046875f,0.7589000221401451806713112091529183089733123779296875f,-0.363600012020766971687635304988361895084381103515625f,-1.57670001925826053224000133923254907131195068359375f,0.1333000095009804208956438742461614310741424560546875f,0.68240001524090754259077584720216691493988037109375f,0.35629999483823748818167587160132825374603271484375f,-1.0378000253558157250921567538171075284481048583984375f,0.46210003976672953118765008184709586203098297119140625f,-1.0502000414907934899844121900969184935092926025390625f,0.69650001673400419033299613147391937673091888427734375f,-0.1364999770224091246717534886556677520275115966796875f,0.836500014759600230007663412834517657756805419921875f,-0.96950000542402303693734211265109479427337646484375f,0.24180000260174250303890630675596185028553009033203125f,-0.98500000983476621030376918497495353221893310546875f,0.1818000014841556488676133085391484200954437255859375f,0.1357999878197906351573465144610963761806488037109375f,-0.52439998002201371196662194051896221935749053955078125f,-0.17099997767806041082394585828296840190887451171875f,0.4308999981343755170115628061466850340366363525390625f,-0.38709999107718484623319454840384423732757568359375f,-1.1875000591203577204879593409714289009571075439453125f,-0.988400045818090600135974455042742192745208740234375f,-0.56089999407380819462787258089520037174224853515625f,1.05540004390329134054127280251123011112213134765625f,0.4806999820128077249847819985006935894489288330078125f,0.5177000042751427333342917336267419159412384033203125f,0.30070005204826666389550382518791593611240386962890625f,1.95930001534223574566340175806544721126556396484375f,0.52559996000081188327612835564650595188140869140625f,0.54889998242855053689481792389415204524993896484375f,0.3085999716758731636900847661308944225311279296875f,0.25999998606741436812939127776189707219600677490234375f,-0.2595999960690742458524482572101987898349761962890625f,1.2477000074416400021704021128243766725063323974609375f,-0.193400008565186798392687705927528440952301025390625f,-1.9928000120937827688294419203884899616241455078125f,1.44640000418424552464102816884405910968780517578125f,1.26390003234744074944728708942420780658721923828125f,-0.70810001286268242637333969469182193279266357421875f,0.3945999959945680757300578989088535308837890625f,0.62589999329149692552931583122699521481990814208984375f,-0.976000001482665080487777231610380113124847412109375f,-0.569100034238398055919105900102294981479644775390625f,0.62240000626295799435183653258718550205230712890625f,0.265200003045797050305054654018022119998931884765625f,-0.585499994277954183274914612411521375179290771484375f,-0.131300007504224769405709594138897955417633056640625f,-0.76729998607635518936831431346945464611053466796875f,0.5740000133812428462221078007132746279239654541015625f,-0.52400002150237579012781452547642402350902557373046875f,-0.039900012409687324321794221759773790836334228515625f,0.094000014908611984054687127354554831981658935546875f,1.04500002823770099524836041382513940334320068359375f,-0.65790001771450068446256409515626728534698486328125f,0.983099973747134203705400068429298698902130126953125f,-0.79040000203102767084573088141041807830333709716796875f,0.210499981462955343403109509381465613842010498046875f,-0.079400001889467386462229114840738475322723388671875f,0.9062999741256236863051753971376456320285797119140625f,-0.005100013779103240718626466332352720201015472412109375f,-0.5384999622739854441277884689043276011943817138671875f,0.33169999653398940520077076143934391438961029052734375f,-0.91059996881112414968129087355919182300567626953125f,0.1914000131249427028024001629091799259185791015625f,0.0374999940395355224609375f,0.579200029800832627557838350185193121433258056640625f,-0.3370000018477437020436582315596751868724822998046875f,-0.96760005486905653615536948564113117754459381103515625f,-0.2095000075101858261206189126824028789997100830078125f,0.24720005466342076516639281180687248706817626953125f,-0.18149999102950087337404738718760199844837188720703125f,-0.75970001215785776249589389408356510102748870849609375f,-1.8206999617472290342590213185758329927921295166015625f,-1.014199957716465316792664452805183827877044677734375f,-0.75970000500529977216501720249652862548828125f,-1.5808000168025493348267218607361428439617156982421875f,-1.4517999923571942577638083093916065990924835205078125f,-0.78869995250478430737217649948433972895145416259765625f,0.8907999873355028075394557163235731422901153564453125f,1.8905000081360350261405756100430153310298919677734375f,1.4337000166356563823910619248636066913604736328125f,0.9995999761760234747498543583787977695465087890625f,0.44229999457001678564438407192938029766082763671875f,0.71230002392530433752426688442938029766082763671875f,-0.1761000092267990257965948330820538103580474853515625f,-0.615400020620227028445015093893744051456451416015625f,1.10060001129806028785651506041176617145538330078125f,0.4277000055044890558519909973256289958953857421875f,0.39619995785057560144792887513176538050174713134765625f,-0.01699999074637903451190368286916054785251617431640625f,0.2473999696806077341904028799035586416721343994140625f,1.38639996629804418404319221735931932926177978515625f,1.5631999695703389097189983658608980476856231689453125f,-0.7314999837093056900272358689107932150363922119140625f,0.61859999687746169971802601139643229544162750244140625f,-1.1078999965734777077841499703936278820037841796875f,-1.554599963193387335280704064643941819667816162109375f,0.4163999995648861585095801274292171001434326171875f,-0.8777999879539013505791444913484156131744384765625f,0.1199000168800357091214436877635307610034942626953125f,-0.15499999903142447355008926024311222136020660400390625f,0.3969000054657458864681984778144396841526031494140625f,-0.317100018309056874699081163271330296993255615234375f,0.4920000171959397761156651540659368038177490234375f,-0.13519998918771747309364172906498424708843231201171875f,0.3017000066667796520647470970288850367069244384765625f,-0.09770000453591372313599805465855752117931842803955078125f,0.6923999912500382603042226037359796464443206787109375f,0.1670999938711524024359533768802066333591938018798828125f,0.190799997542798305261158020584844052791595458984375f,0.1455000071302057218503023250377736985683441162109375f,-0.312399996912479327448863841709680855274200439453125f,0.37950000769644953724224478719406761229038238525390625f,0.3122999820530416581476629289682023227214813232421875f,-0.31799999528378186663957194468821398913860321044921875f,0.1974999777972694658956243074499070644378662109375f,0.790700051116944191420543575077317655086517333984375f,0.9613000383868814946453085212851874530315399169921875f,1.31470003961324710672897708718664944171905517578125f,0.30699999469518690631275603664107620716094970703125f,1.039100017623602933980464513297192752361297607421875f,0.852500000856816431138440748327411711215972900390625f,0.9363999703958632014888507910654880106449127197265625f,0.06799998090416192741969325652462430298328399658203125f,-0.013100004965066858186872877922724001109600067138671875f,0.351799975258112296305768040838302113115787506103515625f,0.0370000054612752149463261730488738976418972015380859375f,0.311799994853138695560801352257840335369110107421875f,-0.76010004246383899850769694239716045558452606201171875f,0.314199972915649272664495583740063011646270751953125f,0.34170002413839117849647664115764200687408447265625f,0.2090000311136248623000710722408257424831390380859375f,-0.1563999714016921505077561960206367075443267822265625f,-0.1407000001177192149981465263408608734607696533203125f,-0.6051999901190396879968602661392651498317718505859375f,-0.835800024737417768250224980874918401241302490234375f,-0.856799961996078440762403261032886803150177001953125f,-0.63010000249147424256079830229282379150390625f,0.094800016586482183100770271266810595989227294921875f,-0.8143999628111722177692399782245047390460968017578125f,0.05309999594986447579714194944244809448719024658203125f,0.4183999943345784178205803982564248144626617431640625f,-0.1667999984666703738866999628953635692596435546875f,0.309299987514317142878184085930115543305873870849609375f,0.83059999013841123716161973788985051214694976806640625f,0.170900022795796413532798396772705018520355224609375f,-0.646100015783309888917074204073287546634674072265625f,0.6979999766945841344067957834340631961822509765625f,-0.0733999875545503499552069115452468395233154296875f,-0.1998000351011748332297202068730257451534271240234375f,-0.71370002080798133903272173483856022357940673828125f,0.54640003204941811620898306500748731195926666259765625f,-1.8575000279024245219972044651512987911701202392578125f,0.41220001547336571068314015064970590174198150634765625f,0.7503999728992580831032910282374359667301177978515625f,0.19050000072270678597163851009099744260311126708984375f,0.98580000622272478949525975622236728668212890625f,-0.64530002531856300862500575021840631961822509765625f,-0.0488999810874464202470335294492542743682861328125f,0.21249997757375183038419663716922514140605926513671875f,0.88570003710985201195171612198464572429656982421875f,0.3462000534415252683828612134675495326519012451171875f,-0.420800027084350602990525658242404460906982421875f,-0.6980000229626897034762578186928294599056243896484375f,0.242199998188019183231745046214200556278228759765625f,0.501000020630657960651888060965575277805328369140625f,-0.6985000171661379653187395888380706310272216796875f,-0.51870000110119551806064919219352304935455322265625f,0.58410004917681235259152572325547225773334503173828125f,0.46300000049918843547658298120950348675251007080078125f,0.9232999860316517537484060085262171924114227294921875f,0.6149000086843965373617493241908960044384002685546875f,0.8626999902307990186756114781019277870655059814453125f,-0.149599929572641787700604254496283829212188720703125f,0.2203000259518628212163093849085271358489990234375f,0.347300006304681385671528914826922118663787841796875f,-0.27860007762312921641978391562588512897491455078125f,-0.3478000146642330481228100325097329914569854736328125f,2.2067000509977336974998252117075026035308837890625f,2.146000013738870393353863619267940521240234375f,-1.10649998283386263864258580724708735942840576171875f,-1.3956999947905532888370316868531517684459686279296875f,0.100900034344196587454689506557770073413848876953125f,0.391000010013579757384150070720352232456207275390625f,0.2997999892055991377759482929832302033901214599609375f,0.17199999726563675039869849570095539093017578125f,-1.2868000053212040878491961848339997231960296630859375f,-0.035900012029707273342182816122658550739288330078125f,0.6152999710038307146220404320047236979007720947265625f,-0.402300024074315754063491112901829183101654052734375f,1.0733999564111236235675050920690409839153289794921875f,-0.1655999765038485005419488516054116189479827880859375f,-1.079900026416778846538591096759773790836334228515625f,-0.54719998343586961908613375271670520305633544921875f,1.369300035160780026899374206550419330596923828125f,-0.49109999387860359121305009466595947742462158203125f,-1.65829998983144744073570109321735799312591552734375f,0.51489997903108619681233903975225985050201416015625f,0.24589998829960801884908505599014461040496826171875f,0.04909999694824218607891452847979962825775146484375f,-0.182100013279914652031266086851246654987335205078125f,0.64220000176429703486746802809648215770721435546875f,-0.3530000483319171156182392223854549229145050048828125f,-0.0563999670803543295960480463691055774688720703125f,0.4454000032231213435096606190199963748455047607421875f,-1.4126999622166156456160024390555918216705322265625f,0.3013000072062010925577624220750294625759124755859375f,-0.74069998674392678594813332892954349517822265625f,-0.00299999010562890244813161189085803925991058349609375f,0.649799997401237394711870365426875650882720947265625f,-0.068700024868548315026828277041204273700714111328125f,0.10030004986822638191057421863661147654056549072265625f,0.8151000199630853426668863903614692389965057373046875f,0.57530001067817249094105136464349925518035888671875f,-0.269800006863474661766844064914039336144924163818359375f,0.0036999920301138133904572669052868150174617767333984375f,-0.177000003151595419748076665200642310082912445068359375f,0.23499999718740571807984451879747211933135986328125f,-1.19970006200224243997354278690181672573089599609375f,-2.31530008376836793360098454286344349384307861328125f,-2.23880003112256442676653023227117955684661865234375f,-0.293099996024370312852624920196831226348876953125f,0.4393000102758410907455299820867367088794708251953125f,0.88049999889731367375134141184389591217041015625f,1.3551999545052648699083874817006289958953857421875f,0.55020000706911087462458453956060111522674560546875f,-0.029100023919343964706740734982304275035858154296875f,-0.5925000257790091762188922075438313186168670654296875f,-0.165299976889788258205271631595678627490997314453125f,2.249099990540742499689486066927202045917510986328125f,1.006200012128056098248407579376362264156341552734375f,0.415800027121603632540569606135250069200992584228515625f,-0.11639997913166910681326271514990366995334625244140625f,-0.2344999919757246031881692260867566801607608795166015625f,-0.25159998558610696672843687338172458112239837646484375f,0.79000000223517385933291734545491635799407958984375f,0.2083999853193763129866056260652840137481689453125f,1.4864000228106970524777352693490684032440185546875f,-0.554299996566772534833944519050419330596923828125f,-0.68049996075034169251694038393907248973846435546875f,-0.25789998195171381922818909515626728534698486328125f,-1.290799951833486947094797869795002043247222900390625f,0.69839999247193329523497595801018178462982177734375f,0.2614000099956985678772980463691055774688720703125f,-0.73680000438988191291400653426535427570343017578125f,0.4185000187307590380214605829678475856781005859375f,0.07439995666444332300670794211328029632568359375f,0.36029999965131265238227342706522904336452484130859375f,-0.2270999927721917277523289158125407993793487548828125f,-0.28850003838166571323853304420481435954570770263671875f,-0.06159999570026997162131010554730892181396484375f,0.1957999860502780686832835499444627203047275543212890625f,-0.196299995285272643741336651146411895751953125f,0.0849000602424145878188710412359796464443206787109375f,0.573400030767917456842042156495153903961181640625f,-0.2336000132501130366335928556509315967559814453125f,0.278900004035234072574667152366600930690765380859375f,-0.6383999768257140328131526985089294612407684326171875f,0.902499998807906766984388013952411711215972900390625f,-0.46460000687241620465073310697334818542003631591796875f,0.38320000730752923345789895392954349517822265625f,-1.3644000139966607765273920449544675648212432861328125f,0.0413000480353833410163133521564304828643798828125f,0.388499991573393987920326253515668213367462158203125f,1.1513000170409684042027720352052710950374603271484375f,-0.175900047941505821569307954632677137851715087890625f,2.089300000441074889323544994113035500049591064453125f,-0.3171000049352645788758309208787977695465087890625f,1.7642000068902969900364041677676141262054443359375f,0.24220000087022786061652368516661226749420166015625f,1.046599971467256029455938914907164871692657470703125f,-0.12430001470893603521261638888972811400890350341796875f,0.436399991667271169859532164991833269596099853515625f,-0.32789999327659591354944268459803424775600433349609375f,0.38739999601840935383734176866710186004638671875f,0.5305999904736877770261571640730835497379302978515625f,0.731099997901917486586853556218557059764862060546875f,0.42959999167323126290085610889946110546588897705078125f,1.33070002012252786016688332892954349517822265625f,0.702500015199184257852493828977458178997039794921875f,1.609100002908706361637314330437220633029937744140625f,0.3701000052109357785212750968639738857746124267578125f,1.40400003662705419316125698969699442386627197265625f,0.27330001132637260496949238586239516735076904296875f,0.92399999430775636710677645169198513031005859375f,0.38660000197738408900249851285479962825775146484375f,0.1839000126779080090955176274292171001434326171875f,0.4046000010609624997215405528550036251544952392578125f,-0.64009997566938370283651238423772156238555908203125f,-0.206999989256262839631972383358515799045562744140625f,-0.801699969115853061651932875975035130977630615234375f,0.4282000097289682116752373985946178436279296875f,0.40719999952912377949587607872672379016876220703125f,-0.14120001037716889413786702789366245269775390625f,-1.6346000010609618158241573837585747241973876953125f,-0.8107000165462492358159352079383097589015960693359375f,1.6740999851763245231950349989347159862518310546875f,0.25990002678930768098553016898222267627716064453125f,0.12340002324283061430065799868316389620304107666015625f,0.762400024144351817767528700642287731170654296875f,0.656899953646957524000526973395608365535736083984375f,-0.57000002160668383766051192651502788066864013671875f,-0.46080002946853682033179211430251598358154296875f,-0.261300007355213192994369819643907248973846435546875f,0.575500006757676541013779569766484200954437255859375f,-1.22070002503991137388084098347462713718414306640625f,0.49769999976754208947937740958877839148044586181640625f,-1.027300026458501935877620780956931412220001220703125f,0.21620003705024704032666704733856022357940673828125f,-0.8382000169932843913755959874833934009075164794921875f,0.2696000016942619037507711254875175654888153076171875f,-0.89080001516342210976517890230752527713775634765625f,-1.0669000128418206774227883215644396841526031494140625f,0.96830000115633030333128772326745092868804931640625f,0.380799978022277940681306063197553157806396484375f,0.29940000926554233284804240611265413463115692138671875f,0.7315000184476379008202684417483396828174591064453125f,1.27549997173994800192531329230405390262603759765625f,-0.93170002775192273247739649377763271331787109375f,-0.26139999673366531851570471189916133880615234375f,-0.088799996775388745362533882143907248973846435546875f,-1.3625000134110452076896535800187848508358001708984375f,-0.440299993057549021724383919718093238770961761474609375f,-1.1289000045195225485628043315955437719821929931640625f,-0.213199995349347692918939856099314056336879730224609375f,-1.095300019767881050114510799176059663295745849609375f,-0.3148000043302772166242675666580908000469207763671875f,-0.002000013619660734320859774015843868255615234375f,-0.2539000027313829122022070805542171001434326171875f,0.605800013971328876749566916259936988353729248046875f,0.7938999970689415430769031445379368960857391357421875f,0.7190999890878793099346921735559590160846710205078125f,0.8015999976187941911121015436947345733642578125f,0.647099973195791822178080110461451113224029541015625f,-0.2939000319004059225136415989254601299762725830078125f,0.071400020724535551153167034499347209930419921875f,-0.22650000566989181294275113032199442386627197265625f,0.6998999751567840377219908987171947956085205078125f,1.9681000101804730917365304776467382907867431640625f,-0.29029998631477393900013339589349925518035888671875f,-1.81349998714029769786293400102294981479644775390625f,-0.28619998124539891559692250666557811200618743896484375f,0.4303999816164374436766593134962022304534912109375f,1.130499993570148742350056636496447026729583740234375f,0.453000005751848122681479935636161826550960540771484375f,-0.0663999611571437498014347511343657970428466796875f,0.182600010016560787562411860562860965728759765625f,-0.6697999927446243173534412562730722129344940185546875f,-1.732900037384033087306534071103669703006744384765625f,0.110500009477139116143007413484156131744384765625f,1.96420000912547099147786866524256765842437744140625f,0.7849999904632571912088678800500929355621337890625f,-0.53500001221895221448221491300500929355621337890625f,-1.227900017714500080501238699071109294891357421875f,-0.57089998174309730671893703402020037174224853515625f,0.61639998905956761010571653969236649572849273681640625f,0.3935000325143338528732783743180334568023681640625f,-0.26440002155899977598352279528626240789890289306640625f,-0.23300002221763105314522590560954995453357696533203125f,0.095199987399578134272815077565610408782958984375f,0.67769999436587102792373116244561970233917236328125f,0.94149997996538881128714137958013452589511871337890625f,-0.2315999985501167035550906803109683096408843994140625f,1.03990001267045695243496084003709256649017333984375f,0.1910000026002525164159351334092207252979278564453125f,-0.45909998346269187408097423030994832515716552734375f,-0.27469997136592905206953219021670520305633544921875f,-0.1401999616950748706045715152868069708347320556640625f,0.5168000457406041192598422640003263950347900390625f,-0.11480000216960917214237269945442676544189453125f,-1.371900036311149762013883446343243122100830078125f,0.0776999909758568918505261535756289958953857421875f,0.78589996475577361678688248503021895885467529296875f,0.2582000140503046825557476040557958185672760009765625f,-0.634399952827394475463052003760822117328643798828125f,-0.0572999963209033236211098483181558549404144287109375f,0.37609997316598953176480790716595947742462158203125f,0.83499999295920124264824835336185060441493988037109375f,-0.55559999248534464033610902333748526871204376220703125f,-0.042799988475441974600244066095910966396331787109375f,-0.5454000090718273963119600011850707232952117919921875f,-0.058999983541667189168578033786616288125514984130859375f,-0.3746999426066874860907773836515843868255615234375f,-0.297100004301964926156642832211218774318695068359375f,0.80049994376301736309642365085892379283905029296875f,0.1249999913945794016711943186237476766109466552734375f,-0.99399999609589517746144338161684572696685791015625f,-0.5571000114545225923023963332525454461574554443359375f,-0.40709999681413167049726098412065766751766204833984375f,-0.138899990214407187405498689258820377290248870849609375f,-0.399599974052608164498678888776339590549468994140625f,-1.353400017207861072421337667037732899188995361328125f,-0.5287999736785895521506972727365791797637939453125f,0.099200002568960332638425825280137360095977783203125f,0.62289997867345814341888399212621152400970458984375f,0.159200004208087886325984072755090892314910888671875f,0.2654999853372572715670685283839702606201171875f,0.727000014215708034726048936136066913604736328125f,-0.20800006908178403364217956550419330596923828125f,-0.60109999969005567521662669605575501918792724609375f,-0.72430002368688661817941465415060520172119140625f,-1.27460000746846180419424854335375130176544189453125f,1.59620001291036572865777998231351375579833984375f,0.6694999856054781872671810560859739780426025390625f,0.614899998402595571178608224727213382720947265625f,0.31530000143945213775253932908526621758937835693359375f,1.0363999957650904537587166487355716526508331298828125f,0.41079998815506701959066049312241375446319580078125f,0.5164999523982405360555958395707421004772186279296875f,-0.332000033885240242170766578055918216705322265625f,-0.35549998407065885874800414967467077076435089111328125f,0.4315000129342079215888361432007513940334320068359375f,-1.0611999898135657804942866277997381985187530517578125f,-0.29229997922182082703557171043939888477325439453125f,-0.811400003550946546937439052271656692028045654296875f,0.083000004708767338712505079456605017185211181640625f,-0.363900035588443415690562687814235687255859375f,1.069500036269426690438422156148590147495269775390625f,-0.645700016359984640956781731802038848400115966796875f,1.65360002382993709346692412509582936763763427734375f,0.812499996423721171368015347979962825775146484375f,0.44709998347759238868093234486877918243408203125f,-0.095699991698563025011026184074580669403076171875f,-0.834100010806321900957982506952248513698577880859375f,-0.1302000072181226375533924510818906128406524658203125f,-0.97139996350407553649120018235407769680023193359375f,-0.32360000054687265702568765846081078052520751953125f,0.5748000070869920818950049579143524169921875f,0.09259999921321881988234281379845924675464630126953125f,0.696100010716915296171691807103343307971954345703125f,-0.15530002081096194732623416712158359587192535400390625f,1.17060001271367131181477816426195204257965087890625f,-0.202100017191469749633370156516321003437042236328125f,1.3087000425636770017945309518836438655853271484375f,0.1676000135555864023473304769140668213367462158203125f,0.69830000964999161539026317768730223178863525390625f,0.9699999985471368102452061066287569701671600341796875f,0.15919996397495328466220598784275352954864501953125f,0.3156999901711941003412675854633562266826629638671875f,-0.28910000641047961522644982323981821537017822265625f,-0.2082999709069728311305880197323858737945556640625f,-0.761499998703599079163950591464526951313018798828125f,-0.64940000010132781227412124280817806720733642578125f,-0.8958000342369079049831270822323858737945556640625f,-1.7907999926626683162567132967524230480194091796875f,0.2801000240981570410525591796613298356533050537109375f,-0.13409998420774915661723980520037002861499786376953125f,0.27539997353255696310725397779606282711029052734375f,-0.6058999932169915059176901195314712822437286376953125f,0.05049999606609301139314993633888661861419677734375f,0.43749999575316900202182068824185989797115325927734375f,0.16939997350275515231032841256819665431976318359375f,-0.452599988782405926457386158290319144725799560546875f,-0.273099965998530702648849910474382340908050537109375f,-0.09649999371170991935287020169198513031005859375f,-0.1241999952673911167266851407475769519805908203125f,0.163100004145503074681755606434307992458343505859375f,0.3903000018119815450745591078884899616241455078125f,0.591700001451373136518441242515109479427337646484375f,0.5971999807909131874339436762966215610504150390625f,1.53139996171593661955512288841418921947479248046875f,-0.23710005172491011027346985429176129400730133056640625f,1.4258999759316448585622083555790595710277557373046875f,0.464799982574582326577683488721959292888641357421875f,1.1381999607414010622363775837584398686885833740234375f,-0.3418000166088341895687108262791298329830169677734375f,0.66350000345706960924729855832993052899837493896484375f,-0.2687999998301269233280663684126920998096466064453125f,0.751800002452731153113063555792905390262603759765625f,-0.0962000156298277175892508239485323429107666015625f,-0.238099994310736473579481753404252231121063232421875f,0.477700002598762551997424452565610408782958984375f,-0.2096000254988670263855965458787977695465087890625f,-0.1982000026136636616769237662083469331264495849609375f,0.31649999967217412599751469315378926694393157958984375f,-0.46450000434368854396183223798288963735103607177734375f,0.631099987024069886132338069728575646877288818359375f,-0.35990000998824844824497404260910116136074066162109375f,0.25720002568066124837287134141661226749420166015625f,0.0899999989941713263164047020836733281612396240234375f,0.536999991714954827415340332663618028163909912109375f,0.106299990926682852432350046001374721527099609375f,0.6263000247150658506001263958751223981380462646484375f,-0.20359999846071008544612368496018461883068084716796875f,0.69899999333918128030518346349708735942840576171875f,-0.083099995279312166562846186934621073305606842041015625f,-0.50559998276233730774009700326132588088512420654296875f,-0.0287999864190817966846935860303346998989582061767578125f,-0.454400004869699447596076424815692007541656494140625f,-0.10800000258535147334981729727587662637233734130859375f,-0.12989999274015406882654133369214832782745361328125f,-1.0708000258922580893994336292962543666362762451171875f,0.43890003085732498533388934447430074214935302734375f,0.568700038540363106420727490331046283245086669921875f,1.23809999930262559786342535517178475856781005859375f,0.6851000376582145889869934762828052043914794921875f,-1.086000056207179653711136779747903347015380859375f,-1.8571000149190421968370401373249478638172149658203125f,1.18209999536126897368149002431891858577728271484375f,1.099300024059414848665028330287896096706390380859375f,-0.4070999846324327986479829633026383817195892333984375f,-1.490200006175041114175883194548077881336212158203125f,-0.4189000271320342250191970379091799259185791015625f,-0.1611000100463630957392524578608572483062744140625f,1.30660000484585747670962518895976245403289794921875f,0.453400021044910062073540757410228252410888671875f,-0.4683000318340961865004601349937729537487030029296875f,0.280500001281499766747629109886474907398223876953125f,-0.18310002031326322313731225221999920904636383056640625f,0.56909999486207940577742192544974386692047119140625f,-0.8037999820232386571916549655725248157978057861328125f,0.936300007057190075698827058658935129642486572265625f,0.048400009496509799333807677612639963626861572265625f,1.0643999889254569524155158433131873607635498046875f,-1.0908999853938816659848498602514155209064483642578125f,0.982500008083879716735964393592439591884613037109375f,1.435199990156293292642430969863198697566986083984375f,0.5850999925449491545492719524190761148929595947265625f,0.279999977126717425335300504229962825775146484375f,0.9656999505341052891793651724583469331264495849609375f,-0.5036000047191973028049005733919329941272735595703125f,0.8327999859049917130704443479771725833415985107421875f,-0.27930000723972903120539967858348973095417022705078125f,-0.66730001880302991512650123695493675768375396728515625f,0.2982000153541566334070012089796364307403564453125f,0.475400013542175425840241587138734757900238037109375f,0.3700000150874256377164783771149814128875732421875f,1.5210999903023234569587884834618307650089263916015625f,0.142800001625716432851476156429271213710308074951171875f,1.5990999710202213890397615614347159862518310546875f,-0.04699998046457753986260286183096468448638916015625f,0.06770002252906548445565704241744242608547210693359375f,0.40109998914748434817312272571143694221973419189453125f,0.277099953079223804142117160154157318174839019775390625f,0.12039998869448885476796107241170830093324184417724609375f,0.140700001645088779156367309042252600193023681640625f,-0.0837000071361659880864181104698218405246734619140625f,0.3499999849125747797273788819438777863979339599609375f,0.06519998201653331992844186970614828169345855712890625f,0.0742000002965330640591901101288385689258575439453125f,-0.028800002139806768486351984392968006432056427001953125f,0.2802999913811685672726525808684527873992919921875f,0.841800019365548823913059095502831041812896728515625f,-0.1560000457763663916921359486877918243408203125f,0.297899991041421785542553379855235107243061065673828125f,0.014499986089766547792123674298636615276336669921875f,-1.1273999737411737331882477519684471189975738525390625f,-1.3759999488070613349322002250119112432003021240234375f,0.0249000319674614889464692168985493481159210205078125f,0.324499983929097346457837147681857459247112274169921875f,-0.8822000298343601887296472341404296457767486572265625f,-0.0328999787665897880373222506023012101650238037109375f,0.686899996525049072459978560800664126873016357421875f,-0.19259998908042863519085585721768438816070556640625f,0.55710003291219489796048947027884423732757568359375f,-0.64799998530000468832668047980405390262603759765625f,-0.41710000718906525829510201219818554818630218505859375f,-1.1274000225238502981284227644209749996662139892578125f,-0.3097000007212162842051839106716215610504150390625f,-0.40309998063892127362350947805680334568023681640625f,0.621399980044364586007077377871610224246978759765625f,0.352900010636448657752595181591459549963474273681640625f,0.5510999673172829460554567049257457256317138671875f,0.447399964018166673707099789680796675384044647216796875f,-0.270400007916986595546404714696109294891357421875f,0.9195000084787607885772331428597681224346160888671875f,1.328899977175891589098455369821749627590179443359375f,-0.1213000097207725946901746283401735126972198486328125f,-0.4491000246457759192963976602186448872089385986328125f,0.9705000097006557524537129211239516735076904296875f,-0.60320004202723520592144268448464572429656982421875f,0.466200023713708500139318857691250741481781005859375f,-1.0490000105127690321893396685482002794742584228515625f,-0.374900012614578059810810373164713382720947265625f,-1.234800023869425000810906567494384944438934326171875f,-0.1718000093817710904886553180404007434844970703125f,1.38209993832707400684967069537378847599029541015625f,0.1803999981567263599657735539949499070644378662109375f,-0.22700002609938330078165336090023629367351531982421875f,-0.34509999679178005038693299866281449794769287109375f,-0.55129995818138155527776689268648624420166015625f,-0.30569998752623828153218710212968289852142333984375f,0.0177999859981234909156455614720471203327178955078125f,-0.1270000017546115689270891380147077143192291259765625f,-1.0053000196933743470850686207995750010013580322265625f,-0.0125000050850212485187284983112476766109466552734375f,-0.902800034892558667110051828785799443721771240234375f,-0.21400000295042997944250373620889149606227874755859375f,-1.1417999958775941760080741005367599427700042724609375f,-0.86300001072511089983407828185590915381908416748046875f,-0.6951999859094615263899186174967326223850250244140625f,-0.470400007749348869623418067931197583675384521484375f,-0.3935999998390673493275926375645212829113006591796875f,-0.65010002122968424398408160413964651525020599365234375f,-0.1231999931141729565098330567707307636737823486328125f,-0.5555000160522760577208600807352922856807708740234375f,0.326300040361285681456138263456523418426513671875f,-0.3515000229179860991024497707257978618144989013671875f,0.3509999833405021973931070533581078052520751953125f,0.06300000604987143759672107989899814128875732421875f,0.7295000396966937472598146996460855007171630859375f,-0.03269999023079872557673297706060111522674560546875f,0.99660000678300786347563189337961375713348388671875f,0.533100016736984372300867107696831226348876953125f,-0.327600038477778909840054666346986778080463409423828125f,0.1077999806523322889884042297126143239438533782958984375f,0.8174000108264383168688027581083588302135467529296875f,0.66040002292990696020069663063623011112213134765625f,-0.789699994611741118433201336301863193511962890625f,0.3330000026226043186028391573927365243434906005859375f,0.5092000027552241459005699653062038123607635498046875f,0.53140000977218126987366986213601194322109222412109375f,0.245000010803341827791967943994677625596523284912109375f,0.08490001635849442340742143642273731529712677001953125f,-0.606900011575222553261710345395840704441070556640625f,0.2208000001132486289634471177123486995697021484375f,0.7679999990463259873507695374428294599056243896484375f,1.158700002032518217021106465836055576801300048828125f,0.07359998717308091631394972864654846489429473876953125f,0.3829999735653404346891193199553526937961578369140625f,-0.49340000808089978789894303190521895885467529296875f,-0.22959999960809918118087580296560190618038177490234375f,-0.47890000733211601779970578718348406255245208740234375f,-0.51740000508949146595938373138778842985630035400390625f,0.3086999792337419279419918893836438655853271484375f,-1.2278999974489206437056054710410535335540771484375f,-0.04859999380409707470107605331577360630035400390625f,-0.3596999966792762126743809858453460037708282470703125f,0.1503000139564278914150463606347329914569854736328125f,-0.35079999314695609058389891288243234157562255859375f,-0.118700002349168054482930756421410478651523590087890625f,0.63739998722672464026572924922220408916473388671875f,0.7904000155165793461975454192725010216236114501953125f,0.5581999688625334243852194049395620822906494140625f,0.8803000056117771254804438285646028816699981689453125f,0.052400007045269259009501183754764497280120849609375f,-0.0987000171199443609992840720224194228649139404296875f,0.557700000438094090071672326303087174892425537109375f,0.12900000300630931615586405314388684928417205810546875f,-0.5839999947920435641179892627405934035778045654296875f,0.066199983331560796617765163318836130201816558837890625f,-0.45499997645616563346493421704508364200592041015625f,-0.1399999938160181311985752472537569701671600341796875f,1.119200036282837373846632544882595539093017578125f,0.147799998980760793898525662370957434177398681640625f,0.2829999287128439622307496392750181257724761962890625f,-0.170900006628036837952322457567788660526275634765625f,-0.362400033122300957444394953199662268161773681640625f,1.10229994792938246206404073745943605899810791015625f,0.805199962812661595989993656985461711883544921875f,0.797900002962350196611396313528530299663543701171875f,1.25190003213882494748077078838832676410675048828125f,1.844100048208236586333441664464771747589111328125f,1.9990000607296829482351085971458815038204193115234375f,1.00860000714063691162891700514592230319976806640625f,-0.012800007884204323005405967705883085727691650390625f,0.88690001544952412615430148434825241565704345703125f,0.31950000315159510222429162240587174892425537109375f,1.0167000156819820944065213552676141262054443359375f,0.79559997993111597480719865416176617145538330078125f,-0.31169998856186875713092376827262341976165771484375f,0.310500037938356232558589908876456320285797119140625f,-1.631699983793497121808968586265109479427337646484375f,0.70359998419284774939796989201568067073822021484375f,0.2590000530183313998122685006819665431976318359375f,-0.376999998569488514732483963598497211933135986328125f,0.22880001212358447304495712160132825374603271484375f,-1.1712999905169014791539439102052710950374603271484375f,-1.134299977791309199659508522017858922481536865234375f,-0.733700015294551644018383740331046283245086669921875f,-1.1044000158965587843340472318232059478759765625f,0.59769997220039439156380467466078698635101318359375f,-0.6610000313222406109758821912691928446292877197265625f,0.086799985837936244337242897017858922481536865234375f,-0.480699991512298030471583842881955206394195556640625f,0.02039999640583989304332135361619293689727783203125f,-1.10459999346137038855886203236877918243408203125f,0.3423000039577483999408968884381465613842010498046875f,-0.6201000173553825600691880026715807616710662841796875f,1.0007000259339811076841897374833934009075164794921875f,-0.2638000221446159532234787548077292740345001220703125f,1.0777000238329172976392555938218720257282257080078125f,-0.71060001994073385134242926142178475856781005859375f,0.45400003018230183915449060805258341133594512939453125f,0.0236999823629850947526165327872149646282196044921875f,0.56400002142787020176228907075710594654083251953125f,0.6588000225543975130193530276301316916942596435546875f,-0.2358000074148176583577196652186103165149688720703125f,0.20909999009370816569486350999795831739902496337890625f,0.717099988543986821554199195816181600093841552734375f,-0.3673000231429937034732802203507162630558013916015625f,-1.6921000199854374645980215063900686800479888916015625f,0.418800028999150253383731978829018771648406982421875f,-0.1370000107884408180325408466160297393798828125f,-0.2508000042483209401922295000986196100711822509765625f,-1.2129999929368497912918201109278015792369842529296875f,-1.0277999942496414131909432398970238864421844482421875f,-0.8162000177159904890089592299773357808589935302734375f,-0.93389998715966904274665694174473173916339874267578125f,-0.9375999916508794829184125774190761148929595947265625f,-0.20570001460909892454509417802910320460796356201171875f,-0.5201000494301322163437362178228795528411865234375f,-0.7529000012859692336775196963571943342685699462890625f,1.706800003570318136070227410527877509593963623046875f,1.6676000185102226591737917260616086423397064208984375f,1.282499977983535099923528832732699811458587646484375f,0.1606000260874622664886146594653837382793426513671875f,-0.5410000236108896132236623088829219341278076171875f,-0.2746000491544606791904925557901151478290557861328125f,0.060200002151727716181994765065610408782958984375f,0.42260000323653201803608681075274944305419921875f,0.064699997591972202570786976139061152935028076171875f,0.3352999986827374101494569913484156131744384765625f,0.9323999998927112731195165906683541834354400634765625f,0.15819998849481375913228475837968289852142333984375f,1.35379998712688642825696661020629107952117919921875f,-0.32759995290786070309962951796478591859340667724609375f,-0.083599984677136252031459662248380482196807861328125f,0.13579996986389186730548317427746951580047607421875f,1.02130002210736225976006608107127249240875244140625f,-0.66180000878572453615333870402537286281585693359375f,-0.61320000864863377643132480443455278873443603515625f,-0.12260000495016531996128605896956287324428558349609375f,0.460299954240024344898074559750966727733612060546875f,0.047500025816261892686043211142532527446746826171875f,-1.6281999947533007944144856082857586443424224853515625f,0.21189999652504898364213659078814089298248291015625f,0.09450001749396363948108046315610408782958984375f,-0.08949999395012842029473176808096468448638916015625f,-0.1843999852374194059478895724168978631496429443359375f,0.30539995930194852036265729111619293689727783203125f,0.4263999930828810125404970676754601299762725830078125f,0.13610002591609937638850169605575501918792724609375f,0.1519000096380709319277002578019164502620697021484375f,0.5164999814927579979695337897283025085926055908203125f,-0.48270000665932843997296686211484484374523162841796875f,0.418600001068413052340844160426058806478977203369140625f,-0.0302999960377812183054402339621447026729583740234375f,0.427599997834861245138426966150291264057159423828125f,0.121599999928474478139150960487313568592071533203125f,-0.60259999995827673302528637577779591083526611328125f,-0.3305999918520450275849498211755417287349700927734375f,-0.36780005024075546060657870839349925518035888671875f,-0.154800012749433779646324182976968586444854736328125f,0.709500027179718273373509873636066913604736328125f,1.5206000411003828265421589094330556690692901611328125f,0.76120003660321255978260523988865315914154052734375f,-0.50330002119839167828985182495671324431896209716796875f,-0.3665000160261986383858356930431909859180450439453125f,-0.0313999605238441947818728294805623590946197509765625f,1.441399986563622892532521291286684572696685791015625f,0.5423999991849068891269780579023063182830810546875f,-0.57399997068941577271772302992758341133594512939453125f,-0.81309996592402489223871953072375617921352386474609375f,-0.401299998340010954933632092433981597423553466796875f,-1.3743000493541364104288504677242599427700042724609375f,-0.801500017553568255834761657752096652984619140625f,0.46030003183782053355344032752327620983123779296875f,0.72960002177357718977646072744391858577728271484375f,0.4988999400347464696636734515777789056301116943359375f,-1.14820003815293336657532563549466431140899658203125f,-0.62139999747872298740958285634405910968780517578125f,-0.80219999416470511022225764463655650615692138671875f,1.522099967384338725651105050928890705108642578125f,-0.2637000224471091058831007103435695171356201171875f,-0.75309999699294560837614653792115859687328338623046875f,1.0210999865397811792178117684670723974704742431640625f,0.103399981333315571507824870423064567148685455322265625f,-0.0601000181004403000173397231264971196651458740234375f,0.028600006805360234096724525443278253078460693359375f,0.387899969248473919236630536033771932125091552734375f,1.05549999789148518658521425095386803150177001953125f,1.10000001262873414731302545988000929355621337890625f,-1.618299973589182538802333510830067098140716552734375f,-0.04789999074339856832693840260617434978485107421875f,0.909000015914440240294425166212022304534912109375f,-1.50460000389218340188790534739382565021514892578125f,-0.04370000396966933209341732435859739780426025390625f,1.08290002263188345210664920159615576267242431640625f,-0.111300004225969306759225219138897955417633056640625f,0.01429999034553730563601448011468164622783660888671875f,0.2224999581277369298248913764837197959423065185546875f,-0.01050000541657158947828065720386803150177001953125f,0.061400021171569729716566143906675279140472412109375f,0.0318000047996640500258536121691577136516571044921875f,-0.630899998581409438003220202517695724964141845703125f,-0.022800007101893615679699678366887383162975311279296875f,-0.591900022602081410383334514335729181766510009765625f,0.07610001026988033601128336158581078052520751953125f,-0.82480001721978180029282157192938029766082763671875f,0.92710004047453453690508240470080636441707611083984375f,-0.82860003291964545724113122560083866119384765625f,0.97850002666562818642859156170743517577648162841796875f,-0.4042000331163411175339206238277256488800048828125f,-0.3450000098347663080033953519887290894985198974609375f,0.0300000029802323098238048260100185871124267578125f,-0.7452000242054463985397205760818906128406524658203125f,-0.583700033362209325105141033418476581573486328125f,-1.4210000371709468058867287254543043673038482666015625f,-1.15200001063942902845838034409098327159881591796875f,-0.8130999446526170348903406193130649626255035400390625f,0.352600014561414543123873954755254089832305908203125f,0.70079999392926672907577767546172253787517547607421875f,0.189199991840124592812344417325221002101898193359375f,1.2045000207722187379744127611047588288784027099609375f,0.3407000065624714579826104454696178436279296875f,0.805699989165365781929040167597122490406036376953125f,0.88570001490712169101016115746460855007171630859375f,0.127999991185962613826632150448858737945556640625f,1.91069998868107848721820118953473865985870361328125f,1.238799998302756666390678219613619148731231689453125f,0.20749998569488514732483963598497211933135986328125f,-0.22540000221729261653536013909615576267242431640625f,0.2876000076696276774868010761565528810024261474609375f,1.0585999743953351526215556077659130096435546875f,1.0998999629750849660325684453709982335567474365234375f,-0.62259999936223042737992727779783308506011962890625f,1.04449998694658319209338515065610408782958984375f,-0.4195999813914301768136283499188721179962158203125f,-0.9497999862253665437350491629331372678279876708984375f,-0.70809999304413739906749469810165464878082275390625f,-1.256500016659498175641829220694489777088165283203125f,0.12220000534057628982509413617663085460662841796875f,-1.325499985784291734347561941831372678279876708984375f,0.63719997658133475937347611761651933193206787109375f,0.038100007200241758908987321774475276470184326171875f,-0.36649999967217450347334306570701301097869873046875f,-0.14180002532601321263427962549030780792236328125f,-0.155899992769956696747613023035228252410888671875f,-0.29829999966621389972942779422737658023834228515625f,-0.33190001328885554432446269856882281601428985595703125f,0.149399991607666093873518775581032969057559967041015625f,-1.034400012841821148157350762630812823772430419921875f,0.6321000041157003312974893560749478638172149658203125f,-1.51640001357197728992787233437411487102508544921875f,0.9004000102266671579087642385275103151798248291015625f,-1.13530000941157371840972700738348066806793212890625f,0.20579999396651971199645458909799344837665557861328125f,-0.8728999928295610999384734896011650562286376953125f,0.294000006861985074824161756623652763664722442626953125f,-1.2025999972760679224137447818065993487834930419921875f,0.19000001739710559167662040636059828102588653564453125f,-0.85330000950098006118338389569544233381748199462890625f,0.179500005647540261666250671623856760561466217041015625f,0.436099997380375636879534795298241078853607177734375f,-0.2298000174432994757722781287156976759433746337890625f,0.1745000197291375343411345966160297393798828125f,-0.39290001484602699033388262250809930264949798583984375f,-0.70650001144409124709255820562248118221759796142578125f,-0.73150003185868339716790842430782504379749298095703125f,-0.394400043314695469831576701835729181766510009765625f,-1.074600061373411907794661601656116545200347900390625f,0.279899988940358834810240296064876019954681396484375f,0.102800021332502211635073763318359851837158203125f,0.17319998473227027346865725121460855007171630859375f,-0.225200015115737617321656216518022119998931884765625f,-0.11819998339116555285954746068455278873443603515625f,0.3647000159204003555402096026227809488773345947265625f,0.316100023457407619531522868783213198184967041015625f,0.2487999732315546719263466002303175628185272216796875f,-0.08189998311400381592051189727499149739742279052734375f,-0.24089998267442014334704936118214391171932220458984375f,-0.525700013118982401039147589472122490406036376953125f,-0.481599995905160671583189468947239220142364501953125f,-0.102200001391768324765507713891565799713134765625f,0.479900007417798146747145437984727323055267333984375f,-0.15829999705851072189233263998175971210002899169921875f,1.09749995887279538919756305404007434844970703125f,0.104400032138824538918697726330719888210296630859375f,-0.0565000251531595498732940541231073439121246337890625f,0.19029997655451325044850818812847137451171875f,0.15559999237358590473689901045872829854488372802734375f,-0.3847999934524295451154785041580908000469207763671875f,-0.375099992880225219238354839035309851169586181640625f,-0.76129999372065049190183572136447764933109283447265625f,0.5978999818027019852451076076249592006206512451171875f,-0.8642999773442745503615469715441577136516571044921875f,0.29189998870193978319775851559825241565704345703125f,0.75019997689425943487862014080747030675411224365234375f,-0.175399996703863081393137690611183643341064453125f,0.340000000447034711470450929482467472553253173828125f,0.34970001294016839210598845966160297393798828125f,0.41280000229626889751699536645901389420032501220703125f,-0.3832999689698215473043774181860499083995819091796875f,0.00699998616427166542308668795158155262470245361328125f,-1.519599981242418351712331059388816356658935546875f,0.57150001453608256785088315155007876455783843994140625f,-0.965499969840049487856958876363933086395263671875f,0.89300001260638184330531430532573722302913665771484375f,-0.5836000228986144566789562304620631039142608642578125f,-0.29220001863986244661219870977220125496387481689453125f,0.04579997355192855224004233605228364467620849609375f,0.6322000008702277629168975181528367102146148681640625f,-0.8636999717831610468010694603435695171356201171875f,0.05629999459609369427681713204947300255298614501953125f,-1.02650001401454193938889147830195724964141845703125f,-0.0816000013813375779658798592208768241107463836669921875f,0.5305000214911996803124338839552365243434906005859375f,-0.24700000941008337651538795398664660751819610595703125f,0.547500003129243406618797962437383830547332763671875f,0.57300005030632117364319810803863219916820526123046875f,0.7862000398814681378922841759049333631992340087890625f,1.5685000199973586632040678523480892181396484375f,0.661200014400481705933998455293476581573486328125f,-0.342800075386464175863920900155790150165557861328125f,0.93699996392428885894076984186540357768535614013671875f,1.2026999827429649680965439983992837369441986083984375f,-0.078900027206539380397742888817447237670421600341796875f,-1.242100029745698552829935579211451113224029541015625f,-0.23730004448890706925112681346945464611053466796875f,-0.51450002263486371845147004933096468448638916015625f,0.29849999589473041883280757247121073305606842041015625f,0.934500012446195515991576030501164495944976806640625f,0.12000000417232514760002004550187848508358001708984375f,0.114000003769993807356541992703569121658802032470703125f,0.632100015217065713812871763366274535655975341796875f,-0.957300024521350767514604740426875650882720947265625f,-0.04889999300837555296084246947430074214935302734375f,-0.800099986845254296241591873695142567157745361328125f,-2.36050000098347734223125371499918401241302490234375f,1.4322999810099599216073329444043338298797607421875f,-0.422799927902222094644457683898508548736572265625f,-0.632600006403028825019418945885263383388519287109375f,-0.0712000049009919866449536129948683083057403564453125f,0.89179999854117664170871648821048438549041748046875f,0.244199973325431585191580552418599836528301239013671875f,-0.028800007206201794218713985173963010311126708984375f,0.5478000398099422962872040443471632897853851318359375f,-0.5238000061072403124029506216174922883510589599609375f,-1.3155000159218908084568511185352690517902374267578125f,0.1830000070556998259529990491500939242541790008544921875f,0.14980001010447752118892594808130525052547454833984375f,-0.56089997429251692295792963705025613307952880859375f,0.300499999232589853903618859476409852504730224609375f,-0.563199967372417287236885385937057435512542724609375f,-0.0778000004708772063821697884122841060161590576171875f,-0.6215999916210763753809942500083707273006439208984375f,0.32829998878836630371580440623802132904529571533203125f,0.23499999586492759107159145059995353221893310546875f,0.022899995344132284547100653071538545191287994384765625f,-1.0212000015854838519402392194024287164211273193359375f,0.13530000054538249276703254508902318775653839111328125f,0.68630001480579372330481646713451482355594635009765625f,-0.6820000274777415771865207716473378241062164306640625f,0.630300028038025317300707683898508548736572265625f,-0.7833000320203604704971667160862125456333160400390625f,0.0902999925732610453366078218095935881137847900390625f,-0.03420000059455630125881953063071705400943756103515625f,0.61950001853704461662886160411289893090724945068359375f,0.9948000226959583525143671067780815064907073974609375f,0.96949998828768713110548560507595539093017578125f,0.515799944271147836616364656947553157806396484375f,-0.53809999505579531042798180351383052766323089599609375f,-0.6016000425085412661729833416757173836231231689453125f,-0.7805000104457133769386700805625878274440765380859375f,-1.1209999636337160122678824336617253720760345458984375f,-0.38949999836459736624050265163532458245754241943359375f,-0.84370005679428661249374954422819428145885467529296875f,-0.260900030600279553283371569705195724964141845703125f,-0.20979999844431862499050112091936171054840087890625f,-0.914200012999773381494605928310193121433258056640625f,-0.988400003126262749475472446647472679615020751953125f,-1.808199998199194080683582797064445912837982177734375f,-0.9643000064760449685508092443342320621013641357421875f,-0.799900019357353553317580008297227323055267333984375f,-0.3279999912604687484218857207451947033405303955078125f,-0.919499987095594661923314561136066913604736328125f,0.63389999420046816425156066543422639369964599609375f,-0.876599940472841954175464707077480852603912353515625f,0.81689997137933989534985812497325241565704345703125f,-0.2618999698147181920404591437545605003833770751953125f,0.2935999649330980343364672080497257411479949951171875f,1.50029995684772732289502528146840631961822509765625f,0.635200022678076958726478551398031413555145263671875f,-0.28349998749420024513057114745606668293476104736328125f,-0.0504000059612095541172749335601110942661762237548828125f,1.0433000087372956699027781723998486995697021484375f,0.169499993205070698110148441628552973270416259765625f,0.469500004827975825349994920543394982814788818359375f,0.21070001941472327189330826513469219207763671875f,0.05859997456297272311331880700890906155109405517578125f,0.0633999991215764835406076826984644867479801177978515625f,0.85520001729503281051592011863249354064464569091796875f,-0.095600004220008916178130675689317286014556884765625f,0.317800018166005726616418769481242634356021881103515625f,0.13589999526590113010371396740083582699298858642578125f,-0.1501000121027231681836155985365621745586395263671875f,-0.330400010096281893101632931575295515358448028564453125f,0.599000025972724170486571892979554831981658935546875f,0.680700032788515319026600991492159664630889892578125f,0.2935999889612193047838673010119237005710601806640625f,0.25889998674988756732773254043422639369964599609375f,-0.34540002184957263597908649899181909859180450439453125f,-0.8494999963343143889460407081060111522674560546875f,0.159300014541298384784795416635461151599884033203125f,0.28210000892132514049848168724565766751766204833984375f,-0.26030000445693757260556822075159288942813873291015625f,0.4651999746032060567557664398918859660625457763671875f,-0.47160000518113320833180068802903406322002410888671875f,1.04050000917911500408763458835892379283905029296875f,-0.66950002837181121773113545714295469224452972412109375f,0.0431999948836858660428106304607354104518890380859375f,-0.6689000324778262562830377646605484187602996826171875f,-0.54600000947341309487370608621858991682529449462890625f,-0.08149998201429864419509385697892867028713226318359375f,1.631700001674889932701262296177446842193603515625f,0.88540000370740923330004079616628587245941162109375f,1.28969997404813785379928958718664944171905517578125f,-0.0942000039100643338230156587087549269199371337890625f,-0.37899999564886055480883442214690148830413818359375f,0.44540004215240502905004404965438880026340484619140625f,0.64430001549124715776173388803726993501186370849609375f,0.3937999913550911390558439961750991642475128173828125f,0.05719999494701644149330377331352792680263519287109375f,0.6237000348523265014222261015675030648708343505859375f,-0.47099997484683964188434401876293122768402099609375f,-0.483599959158897974020874244160950183868408203125f,0.313900008431077015558230414171703159809112548828125f,0.44739999559000065421088265793514437973499298095703125f,0.9164000198118389395318672541179694235324859619140625f,-0.7027999922566114943123238845146261155605316162109375f,-0.0971000173591078930002140623400919139385223388671875f,-0.1166000105082993787419809450511820614337921142578125f,0.41760003691315628060465314774774014949798583984375f,0.1295999670863157149369726539589464664459228515625f,0.352599967175721840106916715740226209163665771484375f,-0.95539998627304978384700007154606282711029052734375f,-1.1342000293910501351746233922312967479228973388671875f,0.419800004907697255074339182101539336144924163818359375f,-0.11429998285770393007254597250721417367458343505859375f,-0.4192000132232898135242749049211852252483367919921875f,-0.23550004181265815628165682937833480536937713623046875f,0.029400016790628313856359454803168773651123046875f,0.62859997465610550904102638014592230319976806640625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.57699999380111677282911841757595539093017578125f,0.40199997666478193281136555015109479427337646484375f,-0.285599977137148908212793685379438102245330810546875f,-0.52479997497498998004772374770254828035831451416015625f,0.08830002995282359989914766629226505756378173828125f,-0.211400013981759560266482367296703159809112548828125f,-0.0507000106230377500793338185758329927921295166015625f,-0.86759997313618697489800979383289813995361328125f,-0.60119998992532497350538278624298982322216033935546875f,-1.74099994385242506922395477886311709880828857421875f,-1.264799990174174215695757084176875650882720947265625f,-0.0020000045299528590447835085797123610973358154296875f,-0.08910005875080795334497452131472527980804443359375f,1.8127999916791910361979489607620052993297576904296875f,0.8154000316843390550047843134962022304534912109375f,0.367400010621548034350780653767287731170654296875f,0.5352999957397581720641710489871911704540252685546875f,0.007899988508224708994021057151257991790771484375f,-0.23809999930262559786342535517178475856781005859375f,-0.945399931250513692049253222648985683917999267578125f,-0.39050000724196409773725235936581157147884368896484375f,0.09630001029819201807669060144689865410327911376953125f,-0.1543999799475077594479444087482988834381103515625f,0.6578999809086323313778166266274638473987579345703125f,0.22459998854398743883820088740321807563304901123046875f,-0.5731999519869692694129525989410467445850372314453125f,0.251499974489212352324329913244582712650299072265625f,-0.50709998347759199788242767681367695331573486328125f,0.514199996422231020432036530110053718090057373046875f,0.5344999599382287414073289255611598491668701171875f,0.4631000198662282851813642992055974900722503662109375f,-0.1416000225409865809211851228610612452030181884765625f,-0.9167999974608420643562567420303821563720703125f,-0.205400001919269215022723074071109294891357421875f,0.62459999230653029389515040747937746345996856689453125f,0.63039995967447659541704751973156817257404327392578125f,-0.12069999646693407857611646250006742775440216064453125f,-1.573100031489133687756520885159261524677276611328125f,0.3679999543428420505364329073927365243434906005859375f,0.591899959607422143648136625415645539760589599609375f,0.3015000131949781003726229755557142198085784912109375f,-0.3282000199735171097614738755510188639163970947265625f,-0.1768999827042219141759460399043746292591094970703125f,-1.33090004894733482387891854159533977508544921875f,-1.1548999846562744142630663191084749996662139892578125f,0.36820007019043021756488087703473865985870361328125f,-0.12220001163631677432164224228472448885440826416015625f,0.33240000786483303585328030749224126338958740234375f,-0.59700000650435669502513746920158155262470245361328125f,0.3169000290095811411816839608945883810520172119140625f,-0.2353000113487244693288857888546772301197052001953125f,0.4142000205993650041591536137275397777557373046875f,-0.20699999029934446070910780690610408782958984375f,-1.1842000155329703847684186257538385689258575439453125f,1.841099961245060256942451815120875835418701171875f,0.6938000240445136856948238346376456320285797119140625f,0.329500020734965648472325483453460037708282470703125f,-0.54509998773932455407020825077779591083526611328125f,-0.4890999792158599035474253469146788120269775390625f,-0.26260002670586146678033401258289813995361328125f,0.22540001384019792141089055803604423999786376953125f,0.5566000275328750745984507375396788120269775390625f,-0.59079997820854135426316133816726505756378173828125f,0.924800028693675724156264550401829183101654052734375f,-0.262800035786629138101488933898508548736572265625f,-0.900300007511674937887846681405790150165557861328125f,-0.6587999899208540899309127780725248157978057861328125f,-0.753700000952183923885741023696027696132659912109375f,-0.36109998270273280240871827118098735809326171875f,-1.1513999825030563073369194171391427516937255859375f,-0.672899967646598096138177425018511712551116943359375f,1.1410999999880788724482272300519980490207672119140625f,-0.740000018551945526468216485227458178997039794921875f,-1.458199974264204445972836765577085316181182861328125f,-0.4482999740362167973017903932486660778522491455078125f,-0.91020001496672564567091967546730302274227142333984375f,0.6151000110968942724554153755889274179935455322265625f,0.862000000357628248792707381653599441051483154296875f,-0.57969996868372053455686909728683531284332275390625f,0.30529996858239183410432815435342490673065185546875f,-0.05759999951124239458977172034792602062225341796875f,1.06290000877380297339414028101600706577301025390625f,-1.13829997992217446523000035085715353488922119140625f,0.1455999875307079083341932346229441463947296142578125f,-0.7762999919325108688639147658250294625759124755859375f,-1.067999991707503593119099605246447026729583740234375f,-0.897699993807076879193118656985461711883544921875f,0.341299971406161528619804812478832900524139404296875f,-0.265300008554756328038592982920818030834197998046875f,-0.127799993653595456155613874216214753687381744384765625f,-0.50949999540299206746141180701670236885547637939453125f,-0.33760000454038363582043302812962792813777923583984375f,0.23409997254759051799055669107474386692047119140625f,-0.642699965345860046994630465633235871791839599609375f,-0.320699972736836258491166518069803714752197265625f,-0.31210001052320002212780991612817160785198211669921875f,0.162600003534555614947976209805347025394439697265625f,0.494599999310076032799088352476246654987335205078125f,0.28229998324513427832016532192938029766082763671875f,0.5968999947369100222971383118419907987117767333984375f,-0.565600003623961899990035817609168589115142822265625f,0.057099999012052926605775837742839939892292022705078125f,-1.05880001122951483694123453460633754730224609375f,0.236500031411647881895987666212022304534912109375f,0.35280002472251670297964665223844349384307861328125f,0.817099965745210710110768559388816356658935546875f,0.8635999719366436178091817055246792733669281005859375f,0.47219996592700541615528209149488247931003570556640625f,-0.63630000646114359685867611915455199778079986572265625f,-0.5369999343454845419643106652074493467807769775390625f,-0.817599935920537035372035461477935314178466796875f,0.44850002908706532167570912861265242099761962890625f,1.156300001730025339696794617339037358760833740234375f,0.46479998577833203654563476447947323322296142578125f,-1.668799978186189747475509648211300373077392578125f,0.1626000334858890283840082702226936817169189453125f,0.02559999058544681904692197349504567682743072509765625f,0.625799969267845757059376410325057804584503173828125f,0.93699996750056779859505695640109479427337646484375f,-1.21139997050762104890964110381901264190673828125f,-0.171399988985061924040564917959272861480712890625f,0.433599947535991958602608065120875835418701171875f,-0.067899978077412104227050804183818399906158447265625f,-0.064300021116434624701696520787663757801055908203125f,0.9501999575600030834010567559744231402873992919921875f,-1.48929994761645811962580410181544721126556396484375f,-0.5208999984323974263844547749613411724567413330078125f,-1.1261999239131823546955502024502493441104888916015625f,1.18419998438954365838071680627763271331787109375f,-0.92120001738071355390502503723837435245513916015625f,0.2697999754965305641007944359444081783294677734375f,-0.95229998562931950090160171384923160076141357421875f,-1.6973999589145185584726505112485028803348541259765625f,0.6827999530851851517354589304886758327484130859375f,-0.023999990284443217802845538244582712650299072265625f,0.555499958962202722290157907991670072078704833984375f,-0.653999982498586263091056025587022304534912109375f,-0.64849998201802383501757276462740264832973480224609375f,-0.18249999590218057843316046273685060441493988037109375f,-0.469800028134882807773209378865431062877178192138671875f,0.0557999931283294781092507719222339801490306854248046875f,1.229400043612719262142718434915877878665924072265625f,-0.4466999728411435999220202575088478624820709228515625f,-1.227799989369511735048945411108434200286865234375f,-0.0749000308498737055185046074257115833461284637451171875f,-0.3112000203609459614284560302621684968471527099609375f,0.941699976529180471374047556309960782527923583984375f,1.083200013267993977450487363967113196849822998046875f,1.376000017762183613712068108725361526012420654296875f,0.266900016939640050139814775320701301097869873046875f,1.2578999889180069171601417110650800168514251708984375f,0.751299987089634413450767169706523418426513671875f,0.43460000288635480814036782248876988887786865234375f,-0.4600000026077031289872820707387290894985198974609375f,-0.43920000405907566598529001566930674016475677490234375f,0.13699997197091551281999954881030134856700897216796875f,0.6172000017270438121386177954263985157012939453125f,0.78510002063363770741233338412712328135967254638671875f,-1.779799975049494609180555926286615431308746337890625f,-0.486300005082786312993192723297397606074810028076171875f,0.6291999440073965388364740647375583648681640625f,0.533099980191886313463101032539270818233489990234375f,1.7080999656260009711417069411254487931728363037109375f,-0.17559995392858962492965702040237374603748321533203125f,1.1556000104784960313963892986066639423370361328125f,0.7145999879106883501123093083151616156101226806640625f,1.4387999441742902462237907457165420055389404296875f,-0.47409996494799866439961988362483680248260498046875f,0.3481999864459037752340009319595992565155029296875f,0.118699974577129108155304493266157805919647216796875f,-0.041999992199241997781200552708469331264495849609375f,0.2986999702185395566544912071549333631992340087890625f,0.67149999540671656550472334856749512255191802978515625f,0.452900019055605007611831069880281575024127960205078125f,0.2214999892227347277895432853256352245807647705078125f,-1.152100015664100585155438238871283829212188720703125f,0.549800018560886005758447936386801302433013916015625f,1.824199992436170258969241331215016543865203857421875f,0.10719998246729378621466821641661226749420166015625f,0.3140000320076936635160791411180980503559112548828125f,-0.6742000021964311695654714640113525092601776123046875f,-1.27229998339414596131291546043939888477325439453125f,0.1292999959707259449714911170303821563720703125f,-0.3355000176727767069451147108338773250579833984375f,0.0219000068068506692497976473532617092132568359375f,-1.29759995301961961189363137236796319484710693359375f,-0.01319999508857705450282082892954349517822265625f,-0.18000000189989773247134507982991635799407958984375f,0.5606999962061640641053372746682725846767425537109375f,-0.147700003604590845984745328678400255739688873291015625f,0.977600032778084493401138388435356318950653076171875f,0.2918000016704198618100463136215694248676300048828125f,1.092800018650292859234696152270771563053131103515625f,0.66780001030564317687776565435342490673065185546875f,-0.597800055009127362382059800438582897186279296875f,0.6405999964341522190380828760680742561817169189453125f,-0.35450002583861373040008402313105762004852294921875f,1.02599996434152185287302927463315427303314208984375f,-0.373900054401158588035514185321517288684844970703125f,0.621599971430003339634140502312220633029937744140625f,1.03990001898482464781636736006475985050201416015625f,0.5959000097200271017783279603463597595691680908203125f,0.34530000151395767726825170029769651591777801513671875f,0.178500009547919169516916326756472699344158172607421875f,0.9275999950408930772027815692126750946044921875f,-0.4700000099837782219225346125313080847263336181640625f,-1.140199967767298705467737818253226578235626220703125f,-0.25129997628629230366215097092208452522754669189453125f,0.378600046814977719833450464648194611072540283203125f,-0.454099998736381849084864370524883270263671875f,-0.5143000313609837892414589077816344797611236572265625f,0.8284999870657916432037382037378847599029541015625f,-0.78509999355077753069309665079344995319843292236328125f,1.11439997193813411513474420644342899322509765625f,0.989799978812038627751235253526829183101654052734375f,1.603799966228007800594923537573777139186859130859375f,1.1096000181227925196481010061688721179962158203125f,-0.0234999886490403486760669693467207252979278564453125f,-0.064200016054510644636366123449988663196563720703125f,-0.050199998649954835627795546315610408782958984375f,-0.31559999432936314267550415024743415415287017822265625f,-0.20219997047185955807435675524175167083740234375f,0.9139999815300108298998793543432839214801788330078125f,-0.850899936592580008465347418677993118762969970703125f,-0.01990000659823409900894830570905469357967376708984375f,-0.18949997547268970787825992374564521014690399169921875f,-0.327100008660554664174924255348742008209228515625f,0.627900001025199827608958003111183643341064453125f,-0.05619999055862390235915881930850446224212646484375f,0.4540999897956847775049027404747903347015380859375f,-1.2877999806523321435491880038171075284481048583984375f,1.1985999458968645825507337576709687709808349609375f,0.0762000672996034911221840957296080887317657470703125f,0.181699955332279561304176240810193121433258056640625f,-0.7883999983765186403417146721039898693561553955078125f,0.05809999155402223447453025073627941310405731201171875f,-0.144800012153386725710646487641497515141963958740234375f,0.13970000517293812425378973784972913563251495361328125f,0.6791999900892375929828403968713246285915374755859375f,1.4901999786078921061971414019353687763214111328125f,0.6838000154018402110267516036401502788066864013671875f,-0.48390002146959343320986590697430074214935302734375f,-0.0869999923482536186014613122097216546535491943359375f,0.53530001969337437373042121180333197116851806640625f,-0.870899998655914941991795785725116729736328125f,0.732800044280291107412494966411031782627105712890625f,0.438099957989156241211503584054298698902130126953125f,1.3538999994158746886796507169492542743682861328125f,1.3613999914065002538876569815329276025295257568359375f,-0.2012000427126887647233388634049333631992340087890625f,0.76389999904334560110186203019111417233943939208984375f,0.8215000028014183275359982872032560408115386962890625f,-0.55389999680817114668940348565229214727878570556640625f,-0.02179998226165746899596342700533568859100341796875f,-0.29950001078844079582808035411289893090724945068359375f,1.067399984097480825084858224727213382720947265625f,-0.7154999635815624259294054354541003704071044921875f,-0.4335000071823598144504785523167811334133148193359375f,0.41889997736215622836652983096428215503692626953125f,-0.4052999685078864100518103441572748124599456787109375f,-0.44909999575614900635400772443972527980804443359375f,0.453700014847516597260579374051303602755069732666015625f,-0.3614999998211860798846828402020037174224853515625f,-0.19969995937049400680507460492663085460662841796875f,0.1019999905228612391283604665659368038177490234375f,-0.22950003209710079232763746404089033603668212890625f,0.93329996762871747506551400874741375446319580078125f,-0.584100005106627850892664355342276394367218017578125f,-0.5304999729692934096192402648739516735076904296875f,1.213699994060397191475431100116111338138580322265625f,-0.7888000022888181916869143606163561344146728515625f,-0.99489999281466001690432676696218550205230712890625f,0.5592999733209609303230536170303821563720703125f,-0.8563000054180616160692807170562446117401123046875f,1.0280999902129170919806711026467382907867431640625f,0.763099967712164595212698259274475276470184326171875f,-0.184999971836806054170665447600185871124267578125f,0.32309997471570994775902363471686840057373046875f,-1.32269994046092076445120255812071263790130615234375f,0.85129999230503994311902715708129107952117919921875f,-0.356800006103515698896444519050419330596923828125f,-0.895899966990948204426103984587825834751129150390625f,1.114299957525730633278726600110530853271484375f,-0.36849998870491962321693790727294981479644775390625f,0.5554999820590023063004991854541003704071044921875f,-0.725899988895654768583654004032723605632781982421875f,-0.1303000046432014613628780352883040904998779296875f,1.069599979007243906181656711851246654987335205078125f,-0.28049999800324432186471312888897955417633056640625f,0.26940000293254851015944950631819665431976318359375f,-0.31570000894665728452537223347462713718414306640625f,0.2359999949261546436929393166792578995227813720703125f,-0.0943000238358975284569396535516716539859771728515625f,-0.714799983319640119816540391184389591217041015625f,0.06510005226135273659338054130785167217254638671875f,-1.15299997557699729355817908071912825107574462890625f,0.36300002080202009580034427926875650882720947265625f,-0.1273999991104004525510617895633913576602935791015625f,-0.0234000203371049764200506615452468395233154296875f,0.27750000253319739851320946399937383830547332763671875f,-0.0861000271081930890204603201709687709808349609375f,0.71200000017136300112241542592528276145458221435546875f,0.80389998153448072315541139687411487102508544921875f,0.09010004000514726385517860762774944305419921875f,-0.9541999857306482368812794447876513004302978515625f,-0.263199997323751322397811236442066729068756103515625f,0.017799994361400450770815950818359851837158203125f,0.84659997787475571584536737645976245403289794921875f,0.427200017298757839245837431008112616837024688720703125f,-0.3153999952137471751711927936412394046783447265625f,-1.3894999927952884632986751967109739780426025390625f,-0.51679996944665873570556868799030780792236328125f,1.4389000179305668769558224084903486073017120361328125f,-0.74089997891187753253916525864042341709136962890625f,-1.627299970988929533888267542351968586444854736328125f,0.0150000137090682272855701739899814128875732421875f,-0.5091000000029801686451946807210333645343780517578125f,0.771900028562544804344724980182945728302001953125f,1.5127999770015481129092904666322283446788787841796875f,-0.8979999498724939854810145334340631961822509765625f,-0.8893999823316931507832805436919443309307098388671875f,0.2919999878406525084528766456060111522674560546875f,0.75079998156130323128110148900304920971393585205078125f,0.3811000001370903333963724435307085514068603515625f,0.3807000000059606037439152714796364307403564453125f,-0.16759999161362681974196675582788884639739990234375f,-0.2232999937057495998260492342524230480194091796875f,-1.18979995239973046494696973240934312343597412109375f,0.824499950438738249403058944153599441051483154296875f,-1.26979993973374494231620701611973345279693603515625f,1.010299960237742045165987292421050369739532470703125f,-0.879399950629473181606954312883317470550537109375f,-0.67319999046921719099145775544457137584686279296875f,0.27789999476671223277435274212621152400970458984375f,-0.287799979013204421107730013318359851837158203125f,0.9889999490082264088641750277020037174224853515625f,-1.10440000442266406111002652323804795742034912109375f,0.8966999943733213740415521897375583648681640625f,0.73119998042583489450407796539366245269775390625f,0.3535999773383142752436469891108572483062744140625f,0.77099998866766694671781579017988406121730804443359375f,-0.212000032022595430447609032853506505489349365234375f,0.112600007744133367992844796390272676944732666015625f,-0.014400012543797746644713697605766355991363525390625f,-0.07809999498128894490633911118493415415287017822265625f,-0.54500001385807994580545710050500929355621337890625f,-0.3685999929100274297155692693195305764675140380859375f,-1.0335999964490534264172083567245863378047943115234375f,-0.471500010363757493525582731308531947433948516845703125f,0.9696999951332803124870451938477344810962677001953125f,0.143999993786215807478612305203569121658802032470703125f,-0.1242999956727028898484377350541763007640838623046875f,-0.337800004643201479126446429290808737277984619140625f,1.13030001745820118941310283844359219074249267578125f,-0.1604999896958474447927756045828573405742645263671875f,-0.7210999773383139910265526850707828998565673828125f,-0.172599996158480539776292062015272676944732666015625f,-0.0560000004768372861008174368180334568023681640625f,0.918899985110759320861006926861591637134552001953125f,0.4917000032022595945591092458926141262054443359375f,-0.383099969053268818441893017734400928020477294921875f,-0.66410001326501333362983814367908053100109100341796875f,-0.37189998564720117002480037626810371875762939453125f,0.142199980418384164249090417797560803592205047607421875f,0.8623999581694608007609303967910818755626678466796875f,1.29069999233186205600532048265449702739715576171875f,-0.6359999646022920227750319099868647754192352294921875f,-0.73669996941387683175861411655205301940441131591796875f,0.187699958342314676684736696188338100910186767578125f,-0.254199996124208293934998437180183827877044677734375f,-1.6072999327301982219751153024844825267791748046875f,-0.29439995856434142407209719749516807496547698974609375f,0.0699999612569808249418201739899814128875732421875f,0.22639995385557443174917580108740366995334625244140625f,0.6449999830126760258508511469699442386627197265625f,-0.3000999714225531800337876120465807616710662841796875f,-0.14859999514520172425591226783581078052520751953125f,-0.58299998876452452378771340590901672840118408203125f,-0.025200010570883168981026756227947771549224853515625f,-0.5266999967575072272296665687463246285915374755859375f,0.66439997223615687715891908737830817699432373046875f,1.411899969814717881178012248710729181766510009765625f,-0.596899984939396777150477646500803530216217041015625f,-0.33220000001341087880746272276155650615692138671875f,-0.414999972097576108609473521937616169452667236328125f,0.63859996586442024923968574512400664389133453369140625f,-0.7829999582543971659021053710603155195713043212890625f,-1.10759997060298953641677144332788884639739990234375f,0.55480000842809573669001110829412937164306640625f,1.003999992668628493674987112171947956085205078125f,0.5353999750971798476939511601813137531280517578125f,-0.474499999836087038573850804823450744152069091796875f,-0.2297999851077798805221164002432487905025482177734375f,0.1140999971717595595777083872235380113124847412109375f,-0.11070000779181687899921371354139409959316253662109375f,-0.42810000306516837742520920073729939758777618408203125f,0.3135999995410443386134602405945770442485809326171875f,-1.1313000031828881475348680396564304828643798828125f,-0.6573999728471042747202091049985028803348541259765625f,-0.082399998514354588774466492395731620490550994873046875f,-1.52020000632405238860656027100048959255218505859375f,0.722300007086992490457078019971959292888641357421875f,0.0203999661564824297244058470823802053928375244140625f,0.5951000121399754316797725550713948905467987060546875f,0.94460004300773181551420520918327383697032928466796875f,0.841100015000999068348619402968324720859527587890625f,-0.211900021633506074891784010105766355991363525390625f,0.90280001358389849741570287733338773250579833984375f,0.4444999952912329632681576185859739780426025390625f,1.196799960729479916921036419807933270931243896484375f,0.2358999964579939767617133838939480483531951904296875f,0.411599985325336259478490319452248513698577880859375f,-0.6866999994769689141804747123387642204761505126953125f,0.3259999814033507359312125117867253720760345458984375f,0.1860000118389724121925610234029591083526611328125f,1.2107999711304902046293818784761242568492889404296875f,-0.0773999940067529035303550699609331786632537841796875f,0.30020003791451443930782261304557323455810546875f,-0.486800016534328250372709590010344982147216796875f,-1.645599992299079161739427945576608180999755859375f,0.22240000317096642046266197212389670312404632568359375f,-1.24019999574422801202899790951050817966461181640625f,0.7800999946683646601996997560490854084491729736328125f,-0.155399980983137897538881588843651115894317626953125f,-0.573300000858306901818650658242404460906982421875f,-0.3304000129461286405785358510911464691162109375f,0.476400000458955474869071622379124164581298828125f,-0.391900072297454382663772776140831410884857177734375f,-0.964199960622190044290391597314737737178802490234375f,0.21290000139772846665664474130608141422271728515625f,0.4083000162810090927223427570424973964691162109375f,-0.2420000497922305537912279760348610579967498779296875f,1.4849000674322254678116905779461376368999481201171875f,0.067899970813094956412214742158539593219757080078125f,0.030299994063377244657431219820864498615264892578125f,-0.05260002394914664591851760633289813995361328125f,1.1952000394046304876383146620355546474456787109375f,1.9308000449657445329876281903125345706939697265625f,-1.243400065338612581200550266657955944538116455078125f,-0.489699991482495544659059305558912456035614013671875f,1.161800060492754749219557197648100554943084716796875f,1.274200031253695630795164106530137360095977783203125f,-0.299800024670362486034491666941903531551361083984375f,-1.36550002720951990653475149883888661861419677734375f,0.957900008177757111837991033098660409450531005859375f,1.161800014746189102510243174037896096706390380859375f,0.48820001207590102421107758345897309482097625732421875f,-0.9428000100821256612704246435896493494510650634765625f,1.0622000042229888805422888253815472126007080078125f,-0.744700037527084912625241486239247024059295654296875f,0.797800015558302266782675360445864498615264892578125f,0.1845000546723607470767092308960855007171630859375f,0.809300018322467895615091038052923977375030517578125f,0.30699999707937219994846600457094609737396240234375f,-0.16290000072717614187922663404606282711029052734375f,-0.227299979892372494560959239606745541095733642578125f,-0.517100014919042560990192214376293122768402099609375f,-0.7688000283658507072459542541764676570892333984375f,0.35990000056326376665083444095216691493988037109375f,0.735699994790554256951509159989655017852783203125f,0.46300002363324210818973369896411895751953125f,0.124700010034441532980054034851491451263427734375f,-0.762600003981589669166396561195142567157745361328125f,0.721699982005357565384429108235053718090057373046875f,0.0602000081866982394984688653494231402873992919921875f,-0.056199999387562371566673391498625278472900390625f,-1.034400012022256287735899604740552604198455810546875f,-0.3623000047773123899474967402056790888309478759765625f,0.03399998141825211206423773546703159809112548828125f,0.031499999523163069170550443232059478759765625f,0.3186000002861018032263018540106713771820068359375f,-1.1707000147581094751103591988794505596160888671875f,1.640400024792551647578875417821109294891357421875f,0.681900006136297864856032902025617659091949462890625f,0.4924000171035529671570429854909889400005340576171875f,-0.343200017514825095332753335242159664630889892578125f,-0.15410003122091353588984929956495761871337890625f,1.523900069749355878911956096999347209930419921875f,1.13190002349615070897925761528313159942626953125f,-1.1225000341236592849014641615212894976139068603515625f,-0.8796000094801186275361715161125175654888153076171875f,-0.304899959287046495859385686344467103481292724609375f,0.47299998150020805542936841447954066097736358642578125f,1.40820001446008635781481643789447844028472900390625f,0.89409995872676351069685551919974386692047119140625f,-0.2098000500023378123160000541247427463531494140625f,1.381299977701901493531977394013665616512298583984375f,-0.839699997144936816084737074561417102813720703125f,-0.51689999697208488527166991843841969966888427734375f,1.0398000124514099606898298588930629193782806396484375f,-0.08530003459453627812081322190351784229278564453125f,-0.59819998823404230137157355784438550472259521484375f,-0.64829997865557675851277963374741375446319580078125f,1.02270001682937117237770507927052676677703857421875f,0.0972999889820815866414704942144453525543212890625f,0.6952000035673382427603428368456661701202392578125f,0.5824000068217516368207498089759610593318939208984375f,0.1760000080764292373913804112817160785198211669921875f,-0.192900013281405602327822634833864867687225341796875f,0.5275999998837705984300328054814599454402923583984375f,-1.33049996156990513185292002162896096706390380859375f,1.28399998994916675343347378657199442386627197265625f,0.8956000242620714146823956980369985103607177734375f,-0.0149999897927048930768023637938313186168670654296875f,-0.54880001070797479467699986344086937606334686279296875f,-0.031800008040666849495892165577970445156097412109375f,-0.5597000290334219751997579805902205407619476318359375f,0.814300005209446009502016750047914683818817138671875f,0.726299989473819351815109257586300373077392578125f,-0.00489998305439964809693265124224126338958740234375f,0.40199999707937195125850848853588104248046875f,-1.5679999687224626114812053856439888477325439453125f,-0.406000022828578721600933931767940521240234375f,0.1800000258535146002714100177399814128875732421875f,0.1684000140413639545755586368613876402378082275390625f,0.66559998738169723342394945575506426393985748291015625f,0.51959998597353662663778095520683564245700836181640625f,0.61119998608827563746359601282165385782718658447265625f,-0.4271000102251762964300496605574153363704681396484375f,0.75200000460445881600435313885100185871124267578125f,-0.469099961781501928470561324502341449260711669921875f,0.37540001287162338972080988241941668093204498291015625f,1.158499971866607580750496708787977695465087890625f,-1.0629999509155749759514719698927365243434906005859375f,-0.40019999172091491601577217807061970233917236328125f,-0.586199988323449883154125927831046283245086669921875f,0.06690000069737411791948034078814089298248291015625f,0.2215000064522028022651056744507513940334320068359375f,0.8437000509828334049444720221799798309803009033203125f,0.34880001599788668098511834614328108727931976318359375f,-0.90280005363076998303739628681796602904796600341796875f,-0.44930001513734463269855723410728387534618377685546875f,1.7602000550508503096125423326157033443450927734375f,0.56120004435181680690902794594876468181610107421875f,-1.30620003779530602372460634796880185604095458984375f,-0.301599989348649177856032110867090523242950439453125f,-1.759200046378374349131945564295165240764617919921875f,-0.721200014474987494139668342540971934795379638671875f,-0.76830004615783753507685105432756245136260986328125f,-0.0661000503540041250971626141108572483062744140625f,-2.07440005225539270128365387790836393833160400390625f,-1.5790000308156013630878078402020037174224853515625f,0.0165999936699865457967462134547531604766845703125f,-0.2721000320553770279730088077485561370849609375f,0.397500008791685122133685581502504646778106689453125f,0.10729997553378278762892250597360543906688690185546875f,-0.07499999947845947811941869076690636575222015380859375f,0.84570004264190856702043674886226654052734375f,-0.2934999998994169967403422560892067849636077880859375f,-1.237600034752488920020141449640505015850067138671875f,0.0454000063896182748379715121700428426265716552734375f,1.338800020691752745705116467433981597423553466796875f,-0.88030002207756030685459336382336914539337158203125f,0.6655000380128626158438009952078573405742645263671875f,-0.030000034868717051494968472979962825775146484375f,0.483999989911913619522465523914434015750885009765625f,-0.0577000255465514033659246706520207226276397705078125f,0.484400001963972925977941486053168773651123046875f,-0.4896000271379941093385923522873781621456146240234375f,-0.35630001696646207420826613088138401508331298828125f,0.499399980729818171454326147795654833316802978515625f,-0.4290000027641653890242423585732467472553253173828125f,-0.87820001751482468232978817468392662703990936279296875f,-0.19740000739917162864145439016283489763736724853515625f,-0.179600015328824691440701144529157318174839019775390625f,0.20470001634880918839343166837352328002452850341796875f,0.425800009277462532253366589429788291454315185546875f,-1.0514000240027900900940949213691055774688720703125f,-0.3324000068217518588653547340072691440582275390625f,0.732100009331106971188773968606255948543548583984375f,-0.110599983954430047816686055739410221576690673828125f,1.106999990448355841721195247373543679714202880859375f,-0.014400011649728039486717534600757062435150146484375f,-0.181399960970878737498424015939235687255859375f,-0.2129999752789739630998155917041003704071044921875f,1.2522999729633337295808814815245568752288818359375f,0.10700002025067778532729789731092751026153564453125f,-0.2480999809741977202293128357268869876861572265625f,0.80510002812147174466872456832788884639739990234375f,-0.69870002915263196285877711488865315914154052734375f,0.04060000392198614971306369625381194055080413818359375f,0.0283999757081263570768214776762761175632476806640625f,0.8907000168070202672510049524134956300258636474609375f,-0.52510004704594681612661588587798178195953369140625f,-0.45510001024007795678016918827779591083526611328125f,-1.17410003241300575638206282746978104114532470703125f,-1.64210003592968067920310204499401152133941650390625f,0.0021000520229350616574492960353381931781768798828125f,-0.60170002089738883199743213481269776821136474609375f,-1.9632000533521172247475305994157679378986358642578125f,-1.1715999914348120825025034719146788120269775390625f,-0.011800000701844393091732854372821748256683349609375f,0.16640000375583741121232606019475497305393218994140625f,-0.47530002414509675556786305605783127248287200927734375f,-0.5362000062607232830913517318549565970897674560546875f,-0.5128999823242423872216022573411464691162109375f,-0.3062999741256235974873334271251223981380462646484375f,-0.735300020177662272402585585950873792171478271484375f,0.20680002581030143549156719018355943262577056884765625f,-0.524600016409158431685000323341228067874908447265625f,-0.025100021117925752633937008795328438282012939453125f,-1.1254000185340646833509481439250521361827850341796875f,0.1461000092267992211958471671096049249172210693359375f,-0.2071000035196544075688507291488349437713623046875f,0.5895999540477989686593218721100129187107086181640625f,0.410400025463104611844755709171295166015625f,1.26760004302263329378774869837798178195953369140625f,-0.259199996459483994470929246745072305202484130859375f,-0.18050002598017489408022129282471723854541778564453125f,-0.19820000015497252210394663052284158766269683837890625f,-0.069899999408423951763325021602213382720947265625f,-0.342899997616559204516306635923683643341064453125f,-0.74920000182390200649251710274256765842437744140625f,1.2567999566316603310411892380216158926486968994140625f,0.54629999427944431200643293777829967439174652099609375f,0.80379997643530309847648140930687077343463897705078125f,-0.56199999920278731480038914014585316181182861328125f,0.276599972361326162939576533972285687923431396484375f,0.16750002771615957186668310896493494510650634765625f,-0.9616000121474261863596666444209404289722442626953125f,1.26619999860525123125398749834857881069183349609375f,-1.7237999825447791746313441763049922883510589599609375f,-0.08680000744462024186987036955542862415313720703125f,-0.253599995070695971577379168593324720859527587890625f,0.581100012207031113575794734060764312744140625f,0.34539999726265679935721664151060394942760467529296875f,-0.2632999875403942979801286128349602222442626953125f,0.18039999368637804622039766400121152400970458984375f,0.26069999885112060677982981360401026904582977294921875f,0.3413000046730036984854450565762817859649658203125f,-0.4797000364840027231139174546115100383758544921875f,-0.7994999943971645706142226117663085460662841796875f,-1.3011000169754023403356768540106713771820068359375f,-1.057699996936321884533072079648263752460479736328125f,0.55230002362728125575586091144941747188568115234375f,1.373999993264675367754534818232059478759765625f,-0.10230000559687635330874400096945464611053466796875f,1.2020000542998321435561592807061970233917236328125f,-1.081100050652027189102000193088315427303314208984375f,-0.9833999881505963713834717054851353168487548828125f,-1.6172000357389446634215346421115100383758544921875f,-2.01370002259612146389144982094876468181610107421875f,0.82210002385973979244226939044892787933349609375f,0.91060001375675181378710476565174758434295654296875f,0.06650001159310381382283594575710594654083251953125f,0.3155000510513783495980533189140260219573974609375f,-0.86059998820126093477256290498189628124237060546875f,-0.175100015008449627629261158290319144725799560546875f,-1.22599997313320674408032573410309851169586181640625f,0.313500026851892155121959149255417287349700927734375f,-0.40459998966157417754629932460375130176544189453125f,-0.30599999481439599247067917531239800155162811279296875f,0.5201000176161529875429323510616086423397064208984375f,0.342800007921457705606371746398508548736572265625f,-1.0387000155180687865907884770422242581844329833984375f,0.319299974811077191816366394050419330596923828125f,-1.338900014913082525680465550976805388927459716796875f,0.23839999805986877845498383976519107818603515625f,-0.36849999831616886414309419706114567816257476806640625f,-0.1178000088155268798573160893283784389495849609375f,0.43150004459917556953740813696640543639659881591796875f,-0.24600000002980237923111417330801486968994140625f,-0.6881000024318695285074909406830556690692901611328125f,-0.3843999665737154458611257723532617092132568359375f,0.313799942684174215656867090729065239429473876953125f,-0.0178999771833421306155287311412394046783447265625f,0.140999968141317655323518920340575277805328369140625f,0.921799976897240025408564179087989032268524169921875f,1.615999986767768792361721352790482342243194580078125f,-0.87370000039339057451570624834857881069183349609375f,0.7547999939739700181462467298842966556549072265625f,-0.14380001212358450857209390960633754730224609375f,-0.128600015187263494453873136080801486968994140625f,0.3139000289201732130095479078590869903564453125f,-0.68580000108182392892786083393730223178863525390625f,1.2372999954640864217481066589243710041046142578125f,0.8916000006735327332307861070148646831512451171875f,-1.35229998711943633082910309894941747188568115234375f,0.4485999911963940434134201495908200740814208984375f,0.36119998251199714189851874834857881069183349609375f,-0.129500003635883320640687088598497211933135986328125f,-1.097700033146143283602214069105684757232666015625f,-0.471300016295909784957984811626374721527099609375f,0.5812000139534472253899366478435695171356201171875f,-0.11410003554224967814434421597979962825775146484375f,1.24500004827976251675636376603506505489349365234375f,2.4785000185072423306564814993180334568023681640625f,0.000599959814547190717348712496459484100341796875f,0.500999979317187982275072499760426580905914306640625f,0.450700016546249582205518890987150371074676513671875f,-0.280100018882751822246746087330393493175506591796875f,-0.5336000367939472965872482745908200740814208984375f,-0.11890001968145469390947255305945873260498046875f,0.2706999783992767305562665569595992565155029296875f,-1.06419999437332268854561334592290222644805908203125f,-0.247000006318092601986791123636066913604736328125f,-0.10499999642372159769365680404007434844970703125f,-0.1157000048488379118083457797183655202388763427734375f,0.67260002491772186150598145104595459997653961181640625f,-0.1641000082731245424128019294585101306438446044921875f,-1.36989999847710119951216256595216691493988037109375f,1.132699992614984552119494765065610408782958984375f,0.213900018861889140708854029071517288684844970703125f,0.6353000147759912952238892103196121752262115478515625f,-0.6286999857157464788315337500534951686859130859375f,-0.58340002756416797780048000277020037174224853515625f,-0.47200001332163810108255574959912337362766265869140625f,0.3686999997973441356435841953498311340808868408203125f,1.197200006681681028197772320709191262722015380859375f,-0.4430999816447502848149042620207183063030242919921875f,-0.861699999067187416557089818525128066539764404296875f,-0.298299980443716084010929989744909107685089111328125f,0.7472000048935412142014911296428181231021881103515625f,-0.125999994814395943620866091805510222911834716796875f,0.024500000357627893521339501603506505489349365234375f,0.01759999094307407307979929100838489830493927001953125f,0.42290000162124652405282176914624869823455810546875f,0.59640003003776076440090037067420780658721923828125f,0.3289999862611292513747685006819665431976318359375f,1.395199962738156163055691649788059294223785400390625f,-1.53439999637603730064938645227812230587005615234375f,0.90359998128712160081477122730575501918792724609375f,0.553199999856949009569007102982141077518463134765625f,-0.62539999342560748107899826209177263081073760986328125f,0.140599995875358541042032811674289405345916748046875f,-0.78800003883242564217681547233951278030872344970703125f,-0.31719995139837298125939923920668661594390869140625f,-0.139200010392069639664214264485053718090057373046875f,0.07309999229908026308066837373189628124237060546875f,-0.5587999967753887187171812911401502788066864013671875f,0.0720000039339065978083453956060111522674560546875f,-0.56200000810623151892286841757595539093017578125f,-0.95520004134178204679983537062071263790130615234375f,-0.8428000371277331037589419793221168220043182373046875f,1.07390002176761623076117757591418921947479248046875f,0.256599979811906298010626414907164871692657470703125f,1.0301000463008886942617436943692155182361602783203125f,-0.13959999301433612117762095294892787933349609375f,-0.9460999871730797483593278229818679392337799072265625f,0.32270000788867470031817674680496565997600555419921875f,0.5970000010281799607270158958272077143192291259765625f,0.7044999930188058367974690554547123610973358154296875f,-0.6635999768540254972748471118393354117870330810546875f,-0.142699994812905739394182091928087174892425537109375f,2.1259000203371041237687677494250237941741943359375f,1.17950003165006744865195287275128066539764404296875f,-0.902900043344498470787584665231406688690185546875f,-0.73909998860359138461717520840466022491455078125f,-0.1935000183582304966734000117867253720760345458984375f,0.344300024804473192574505446827970445156097412109375f,0.349199994373321498386530947755090892314910888671875f,-0.8154000205457208050319195535848848521709442138671875f,0.63680000312328310929643748750095255672931671142578125f,-0.34860002092421071839822843685396946966648101806640625f,0.5842000251442194258544304830138571560382843017578125f,1.224000017330050038566469083889387547969818115234375f,-0.0257999834612012701740013653761707246303558349609375f,-0.6962999993830916167780742398463189601898193359375f,-0.0884000001087783815467702197565813548862934112548828125f,0.01439999555647319962048413799493573606014251708984375f,0.010900010092556644547556743418681435286998748779296875f,-1.0624999899417157944725431661936454474925994873046875f,0.11139998354613833697612790274433791637420654296875f,0.7524000026494264137255640889634378254413604736328125f,-0.794000020682811946670653924229554831981658935546875f,0.4904999921917916338998111314140260219573974609375f,-0.00919999385178105200111531303264200687408447265625f,0.2994000136613845786115462033194489777088165283203125f,-0.389399978196620699666397058535949327051639556884765625f,-0.1312999955832960807811105041764676570892333984375f,-0.90359998158514465149693251078133471310138702392578125f,-0.31470000995993618886359399766661226749420166015625f,0.2947999697595837798047568867332302033901214599609375f,0.06110000132918347315325036106514744460582733154296875f,0.69299998131394391354120898540713824331760406494140625f,-0.2798999988123773707826558165834285318851470947265625f,0.73199998553097278541912373839295469224452972412109375f,0.3669999785274267178891705043497495353221893310546875f,0.1008000055521718341111636618734337389469146728515625f,-1.22500000864267288847031522891484200954437255859375f,-0.3213999927848580906442066407180391252040863037109375f,0.5292000152349469743739973637275397777557373046875f,-0.19799999725818651086228783242404460906982421875f,0.145199954319000568148112506605684757232666015625f,-0.7634999860599640530978149399743415415287017822265625f,-0.7410999382972729421226176782511174678802490234375f,-0.433200008574127881644244553172029554843902587890625f,-0.6060999678015708269640526850707828998565673828125f,0.810199965159595247854440458468161523342132568359375f,-0.93539997956752785057688015513122081756591796875f,-0.4016999971300363458936999450088478624820709228515625f,0.3696999864161016891017652596929110586643218994140625f,-0.0439000098839399388594983975053764879703521728515625f,1.95219992576837597653138800524175167083740234375f,1.187099973717332002109969835146330296993255615234375f,0.77119996135234902823185620945878326892852783203125f,1.36910001516491153239485356607474386692047119140625f,-0.71200000333786039874439666164107620716094970703125f,-1.4288000048220157456313472721376456320285797119140625f,0.32960002162456447649674373678863048553466796875f,1.98439995405674007855623131035827100276947021484375f,-0.8527999514460571361951224389486014842987060546875f,-0.866599959248304685388575308024883270263671875f,-0.27199999409913999670607154257595539093017578125f,0.1954999790787699254224207834340631961822509765625f,0.3916999821543694082492947927676141262054443359375f,0.44349998319149042114162284633493982255458831787109375f,0.09859998829066807246590542490594089031219482421875f,0.0919000273704522196993593752267770469188690185546875f,1.0202999419838192363130247031222097575664520263671875f,-0.1526999808430673510883934795856475830078125f,0.5045999709606174832288161269389092922210693359375f,-0.223999991327524572426455051754601299762725830078125f,-0.4660999816596511635680144536308944225311279296875f,-0.23679998703002969051567561109550297260284423828125f,-0.37999997004866603589334772550500929355621337890625f,0.34380000530630316912805710671818815171718597412109375f,-1.507800014552473921725095351575873792171478271484375f,-0.03930000595450400258101808503852225840091705322265625f,0.50950003783404884671881518443115055561065673828125f,0.4059999964162706032055893956567160785198211669921875f,0.020199985909461926070207482553087174892425537109375f,-0.5750999866217372247234607129939831793308258056640625f,-1.13130000452399226418265243410132825374603271484375f,-0.591000013589859207741028512828052043914794921875f,0.777100048670173126907911864691413938999176025390625f,-0.1815000054463744572075256655807606875896453857421875f,1.4646000444978479748669997206889092922210693359375f,-0.3857999982878563827171092270873486995697021484375f,-0.07909995336234476592807141059893183410167694091796875f,0.6885999685466288600110829065670259296894073486328125f,0.839399988627433923937815052340738475322723388671875f,0.487800009486079211296072344339336268603801727294921875f,-0.1109999969005581732517384807579219341278076171875f,1.529899943268298923015890977694652974605560302734375f,0.2201999776393182006728466149070300161838531494140625f,0.510899997687340512442233375622890889644622802734375f,0.783900010368228095103404484689235687255859375f,-0.2904000192046165551573722041212022304534912109375f,-1.63909999128580086136253157746978104114532470703125f,0.44200000631809199802546572755090892314910888671875f,0.17760000822842147982782989856787025928497314453125f,-0.257099984669685088789492510841228067874908447265625f,1.1161000023350122756227165154996328055858612060546875f,-1.882499986290931293098083187942393124103546142578125f,-1.0965000112950800126299100156757049262523651123046875f,0.22999995790422023134169648983515799045562744140625f,0.6353999996468424615869707849924452602863311767578125f,0.045599975311755969187288428656756877899169921875f,-0.5394000200688837676210596328019164502620697021484375f,1.017899956619739754160036682151257991790771484375f,1.1057000049978491507118860681657679378986358642578125f,-0.39960003093778961869020349695347249507904052734375f,-0.578100012564659149205681387684307992458343505859375f,-0.460799991247057949550480770994909107685089111328125f,0.00740000138282770336672911071218550205230712890625f,-1.0542999464988707103429987910203635692596435546875f,0.598500012248754043042708872235380113124847412109375f,-1.0960999962627884229959818185307085514068603515625f,-0.0781000098079445148613331184606067836284637451171875f,-0.43420000830590677054487969144247472286224365234375f,-0.133800010335445751508132161688990890979766845703125f,0.102100000241399424538712992216460406780242919921875f,0.9521999939411873725703117088414728641510009765625f,0.873199956643581298720846461947076022624969482421875f,-0.5983999726533892893343136165640316903591156005859375f,0.369000004217028720887583403964526951313018798828125f,-1.0464000075370070153013557501253671944141387939453125f,0.8131000316381455927938759487005881965160369873046875f,-1.003600028151273360066397799528203904628753662109375f,-0.22290001026391959015882093808613717555999755859375f,-0.748100023479759901334773530834354460239410400390625f,0.9655000133514402005374677173676900565624237060546875f,-0.93950000181049075909101020442903973162174224853515625f,-1.54889998868703759882237136480398476123809814453125f,-0.316900021708011792043180321343243122100830078125f,0.76019999514818170638363881153054535388946533203125f,-0.87069996182173536869441932140034623444080352783203125f,-0.0510999301016337170722181326709687709808349609375f,0.00929996318817238876164310568128712475299835205078125f,0.3724999405443671918192194425500929355621337890625f,0.20129997915476505188081546293688006699085235595703125f,0.70390003160238290291772500495426356792449951171875f,0.09200000430643562321364470335538499057292938232421875f,0.491999996185302990170384873636066913604736328125f,-2.1970999880969515061224228702485561370849609375f,-1.5364000213950870232793022296391427516937255859375f,-0.50039999685287472885875104111619293689727783203125f,0.27170001001954080521727519226260483264923095703125f,-1.8112999660789963485285625210963189601898193359375f,-0.98340000297725194133846571276080794632434844970703125f,0.76259994139671416490955380140803754329681396484375f,-0.6094999559521678822449075596523471176624298095703125f,-0.481199945855141919537345529533922672271728515625f,0.1561000091150404056605793812195770442485809326171875f,-0.03190004078149666355557201313786208629608154296875f,-0.1300000053644183584111715390463359653949737548828125f,0.6727000023007381201978205353952944278717041015625f,-0.70539994205385436742972160573117434978485107421875f,-0.29159998979568424459785092039965093135833740234375f,-0.51580000849515161842617771981167607009410858154296875f,-1.025199991869925497667281888425350189208984375f,0.636899970000981863194056131760589778423309326171875f,-0.73439999846219894408250183914788067340850830078125f,1.362599997872113721797404650715179741382598876953125f,-0.5920000073611735302847591810859739780426025390625f,0.041699979323148728127534923260100185871124267578125f,-0.59009998461008006387373825418762862682342529296875f,1.988499965012073378289869651780463755130767822265625f,-1.03579996882081104558892548084259033203125f,-0.548799977999925747695897371158935129642486572265625f,-1.446399935042858686529143596999347209930419921875f,1.31159991156458932692885355209000408649444580078125f,-0.86069997780322982094958206289447844028472900390625f,0.326600057744979022089637510362081229686737060546875f,-1.2007999796986581486635259352624416351318359375f,-0.66640001729726794366115427692420780658721923828125f,-0.58039999208450243628476528101600706577301025390625f,-0.1794000069558621124343744668294675648212432861328125f,-0.3716999949321146967662343740812502801418304443359375f,0.35299999375641366583522540167905390262603759765625f,-1.1095000396966934186337994105997495353221893310546875f,0.5003999878004192769509472782374359667301177978515625f,0.64979999859332959744051549932919442653656005859375f,1.081499958321452847798127550049684941768646240234375f,0.8119999543130405061930332522024400532245635986328125f,1.0101999991342420770479293423704802989959716796875f,-0.48070001304447684820075892275781370699405670166015625f,0.87219997449517305820876345023862086236476898193359375f,-0.597600067311525595670218535815365612506866455078125f,1.1405999511346223140861866340856067836284637451171875f,1.19189997208714526522044252487830817699432373046875f,0.2160999961510297051603401996544562280178070068359375f,0.5397000235199929107920979731716215610504150390625f,-0.0200999992504715674357385069015435874462127685546875f,-0.10059999565184085668079205788671970367431640625f,0.2448999811172496432476464178762398660182952880859375f,-0.99139999069869533254717453019111417233943939208984375f,1.6404999687969681332333493628539144992828369140625f,1.0870999894380568573382106478675268590450286865234375f,1.2611999685049060104802265414036810398101806640625f,1.7690999669224023449487503967247903347015380859375f,0.336599987858534177576075308024883270263671875f,-1.7443999254465107906497678413870744407176971435546875f,1.38339995752871036671649562777020037174224853515625f,0.684099961148202595495604327879846096038818359375f,1.9848999453544617921352255507372319698333740234375f,0.4508999769374730792748096064315177500247955322265625f,0.29350002551078802071060636080801486968994140625f,0.9785000308752060949046835958142764866352081298828125f,-0.6641999982476232133876692387275397777557373046875f,-1.0871999973312018727966687947628088295459747314453125f,-0.75250000003725237007756732054986059665679931640625f,-0.6515000151693819407938690346782095730304718017578125f,-0.0572999903604387572642053783056326210498809814453125f,-0.25949995997548125359344339813105762004852294921875f,0.28459998496770833043001402984373271465301513671875f,-0.304800002467632058511526338406838476657867431640625f,0.26169996859431332580925300135277211666107177734375f,-0.71289997658729564733448569313623011112213134765625f,0.11280002863407023738773204968310892581939697265625f,-0.712099972227215882725204210146330296993255615234375f,0.494399966722727679524496124940924346446990966796875f,-0.3336999562859539469883429774199612438678741455078125f,0.182599952796102638075126378680579364299774169921875f,-1.05679996691346200776706609758548438549041748046875f,-0.309700006532668936642949120141565799713134765625f,0.293599977040290838203873136080801486968994140625f,-0.3212999900698658706232890835963189601898193359375f,-0.53939998270422240267407687497325241565704345703125f,0.62420005323290883580966692534275352954864501953125f,-0.52939997778683933749022116899141110479831695556640625f,-0.93209998899102242120306982542388141155242919921875f,0.660299973760545722001324975281022489070892333984375f,0.40409997996091906458104858756996691226959228515625f,-0.2666999798819429390306368077290244400501251220703125f,0.99890005306005402729851994081400334835052490234375f,0.824599999794364268979052212671376764774322509765625f,0.0175999375224105136794605641625821590423583984375f,0.61730000969469578375736773523385636508464813232421875f,-1.1031000501155858728452585637569427490234375f,0.41149996130168464691934104848769493401050567626953125f,0.42379996876120618054528677021153271198272705078125f,0.024600014583766149911525644711218774318695068359375f,-0.689399994289875195363492821343243122100830078125f,-0.2100000000372528319303455646149814128875732421875f,0.3856000179290770546458588796667754650115966796875f,-1.5596999668955806583880985272116959095001220703125f,-0.6994999931678174132088088299497030675411224365234375f,0.532299958062171896244763047434389591217041015625f,-0.34830005472600411575712087142164818942546844482421875f,0.5134000116944310576627685804851353168487548828125f,1.4862000183120371854528229960124008357524871826171875f,-0.794199984240531620116598787717521190643310546875f,-0.5094000337779522880055083078332245349884033203125f,0.301199984598159886672874563373625278472900390625f,-0.547499994449318183598052200977690517902374267578125f,1.75119996671676592114863524329848587512969970703125f,1.171900012730061835242167944670654833316802978515625f,-0.183499985575675594873246154747903347015380859375f,-1.1055999786645174243204792219330556690692901611328125f,0.348600002968311173390247859060764312744140625f,0.8462999903678893875991207096376456320285797119140625f,-0.20939998147487681734446596237830817699432373046875f,0.324699991929530806800130449119023978710174560546875f,0.13340000618100145146627255599014461040496826171875f,0.49579999951720221673667765571735799312591552734375f,0.92109993978738913256165687926113605499267578125f,-0.2915999692320827563207785715349018573760986328125f,-0.2577000017046930935293858055956661701202392578125f,-0.160300052773951762219439842738211154937744140625f,0.766899995183944582777257892303168773651123046875f,-0.694299994778632356684511250932700932025909423828125f,1.182200009810923546638150583021342754364013671875f,0.62380002255439759295541080064140260219573974609375f,-0.13290002337694151179903201409615576267242431640625f,-1.1802999787151815525021447683684527873992919921875f,-0.371499973297119367998675443232059478759765625f,0.1287999981164933416266649146564304828643798828125f,0.026999992907047243306806194595992565155029296875f,0.706199964518845479943820464541204273700714111328125f,0.00839998032748778822309532188228331506252288818359375f,0.900600003921984804122757850564084947109222412109375f,0.926900026178359315309762678225524723529815673828125f,-0.417299997624754059888374513320741243660449981689453125f,0.15649998909235041111287500825710594654083251953125f,-0.37810000015944267293122038608998991549015045166015625f,-0.636200014512241285302707183291204273700714111328125f,-0.118099994198977709114473100271425209939479827880859375f,-0.70320002697706229799479160647024400532245635986328125f,-0.56679998311847479275371597395860590040683746337890625f,0.4554000330626968295888445936725474894046783447265625f,-0.42169997589588203990018655531457625329494476318359375f,-1.491199969100951872036375789321027696132659912109375f,-0.1437999959185718434895306927501223981380462646484375f,0.21160000037550918960960188996978104114532470703125f,0.0887000031501055108407172156148590147495269775390625f,-0.33240001866817436848577926866710186004638671875f,1.05499999321997162127217961824499070644378662109375f,1.9004999731555585196218771670828573405742645263671875f,-0.32220002858638707721183891408145427703857421875f,-0.1084000052869320551707232880289666354656219482421875f,-1.2908999818921085278589089284650981426239013671875f,-1.9349999874085190842976089697913266718387603759765625f,-0.16979998622536651708969657192938029766082763671875f,0.734699983808397849571747428853996098041534423828125f,-0.02899999080598403278230534851900301873683929443359375f,0.42850000587850789468546963689732365310192108154296875f,-1.3901999291360385146987255211570300161838531494140625f,0.13760001322031045756233424981473945081233978271484375f,0.85749993145465897015355949406512081623077392578125f,-0.198100027950107726582018585759215056896209716796875f,0.20040001115798933284395388909615576267242431640625f,0.293899996733665513914957045926712453365325927734375f,1.03239993045330180621022009290754795074462890625f,-0.55739995675385056728146082605235278606414794921875f,1.391299932849407117174678205628879368305206298828125f,0.3465999568641191519446920210612006485462188720703125f,-0.679199971947073155575935743399895727634429931640625f,-0.010700002092123295227565904497168958187103271484375f,-0.36049996373057435761211308999918401241302490234375f,-0.525099995636939542720256213215179741382598876953125f,1.073299987111985043242157189524732530117034912109375f,0.425200011688470880244494765065610408782958984375f,0.20230000511258772188938337421859614551067352294921875f,-0.4894999900758267852296512501197867095470428466796875f,0.7058999743670224535918578112614341080188751220703125f,1.566899965530633931365400712820701301097869873046875f,-0.149000007420777791367072495631873607635498046875f,-0.720799986851215379601853783242404460906982421875f,-0.24829998759627347482137338374741375446319580078125f,0.95699997767060984710241200446034781634807586669921875f,0.12559998797774341028343769721686840057373046875f,0.9531000128999351783676274862955324351787567138671875f,-0.1286000245936218533415740239433944225311279296875f,-0.64419996851980687591066043751197867095470428466796875f,-0.39439996258765486203401451348327100276947021484375f,0.062399964334816260258520514980773441493511199951171875f,0.7273999860718849408414143908885307610034942626953125f,0.1677999949201940677045996608285349793732166290283203125f,0.655899997575580595565725161577574908733367919921875f,-0.3756000020593404098434575644205324351787567138671875f,-0.3943000258848072814288343579391948878765106201171875f,-0.1267999975353479857886895842966623604297637939453125f,-0.5284000056408346690517419119714759290218353271484375f,-0.92450000613182725661687300089397467672824859619140625f,-0.54789999804496714030932480454794131219387054443359375f,-0.408800019667297842573816524236463010311126708984375f,-0.2261999989032741353867095313034951686859130859375f,0.70019998263120708514861689764074981212615966796875f,1.202599941992759990938566261320374906063079833984375f,-0.192699994328618284811227567843161523342132568359375f,0.135200008745491206951783169643022119998931884765625f,0.240099976079166388398533626968855969607830047607421875f,0.4375000039488075032068081782199442386627197265625f,0.0710999791264537162049919061246328055858612060546875f,-0.320899989417195552476869124802760779857635498046875f,-0.51969997985959093256269625271670520305633544921875f,-0.16699999372661122887251394786289893090724945068359375f,-0.4705999854445457319940260276780463755130767822265625f,-1.738699984747171356502803973853588104248046875f,-0.18250000020489043350835345336236059665679931640625f,1.0131999835774301299551325428183190524578094482421875f,-0.341600002685189085838146638707257807254791259765625f,-0.0895999976895749394856238723150454461574554443359375f,1.031899935281277436160962679423391819000244140625f,-1.4769999741315842367583854866097681224346160888671875f,1.79609994172453912852915891562588512897491455078125f,0.108699995103478830316134917666204273700714111328125f,-0.0038999716065830636324562874506227672100067138671875f,1.0106999654352666340173527714796364307403564453125f,-0.349700003794580493998722658943734131753444671630859375f,-0.152799988587200896716922216000966727733612060546875f,-0.444099969492853130503817737917415797710418701171875f,0.225699993747472849037194464472122490406036376953125f,-0.751299973119795350129379585268907248973846435546875f,0.00779999275952591997196350348531268537044525146484375f,-0.20449998556822546191824585548602044582366943359375f,-0.2603999978028237460847549300524406135082244873046875f,-0.17740000663548649573186821726267226040363311767578125f,0.879200003463029400307959804194979369640350341796875f,-0.25680000846907502509708365323604084551334381103515625f,-0.265200005690753304321560790413059294223785400390625f,-0.4212000065028664863575613708235323429107666015625f,0.50699998597800710609106999982031993567943572998046875f,0.76960001432299574020134969032369554042816162109375f,0.4670000021085145558430440360098145902156829833984375f,1.1108999784648414532739479909650981426239013671875f,1.1215999778747558490721303314785473048686981201171875f,-0.276500035732984539293966008699499070644378662109375f,0.6522000042229894045675564484554342925548553466796875f,-0.8930000477731230290601160959340631961822509765625f,-0.88569999952167244305911708579515106976032257080078125f,-0.686299993348121528669025792623870074748992919921875f,-0.547900015218556024620966127258725464344024658203125f,0.2335999979764220579880884542944841086864471435546875f,0.36449999853223513657241028340649791061878204345703125f,1.27400002127885780822680317214690148830413818359375f,0.34840001642555018523950138842337764799594879150390625f,-0.7042000122547147356044661137275397777557373046875f,0.75310001167058970850121113471686840057373046875f,0.6873000223606824388156155691831372678279876708984375f,0.554099974000454409406302147544920444488525390625f,1.262299976912140575535659081651829183101654052734375f,-0.3575999729871757182309011113829910755157470703125f,-0.88279997189790027078970524598844349384307861328125f,0.579799951207638830652513206587173044681549072265625f,-0.3236000379115344660618802663520909845829010009765625f,0.32239998767375954713543251273222267627716064453125f,0.422300014500319453869536800993955694139003753662109375f,0.080699996876716806326612640987150371074676513671875f,0.23180001109540437909117827075533568859100341796875f,0.8103000183522690580417702221893705427646636962890625f,0.55370003727376460034292904310859739780426025390625f,-0.47539998165369024007986809010617434978485107421875f,-1.1632999870747326998099424599786289036273956298828125f,-0.64769999485015805618104423047043383121490478515625f,-0.414499995440244628497339363093487918376922607421875f,0.19229997788071617748073549591936171054840087890625f,-0.19800001931190447379549368633888661861419677734375f,0.178500012695788967675980529747903347015380859375f,-0.6261999753594398310241331273573450744152069091796875f,0.94579997358918177496889256872236728668212890625f,-0.8604999935328958571290058898739516735076904296875f,1.156499983429908784415829359204508364200592041015625f,0.066200015443563575701091394876129925251007080078125f,-0.54759996806979227557121703284792602062225341796875f,0.88889998533427705407206076415604911744594573974609375f,-0.9898999718785290724554215557873249053955078125f,0.6820999997198582409652090063900686800479888916015625f,-0.8379999808669096328372916104854084551334381103515625f,-0.77520002584457348149271638249047100543975830078125f,-0.39059997143745395220548743964172899723052978515625f,0.19350000628829011617426658631302416324615478515625f,0.111300035294890076187357408343814313411712646484375f,0.1925999601721766207873542953166179358959197998046875f,-1.31159999888539235968210050486959517002105712890625f,1.58949998001754266852003638632595539093017578125f,0.03529999011456919078000282752327620983123779296875f,0.479700024041533001906145727843977510929107666015625f,-0.50100000032782521230956263025291264057159423828125f,0.2022000189006332160346346427104435861110687255859375f,0.308799994689226142696725219138897955417633056640625f,0.15809997486472138206181625719182193279266357421875f,1.23160000007748582362410161294974386692047119140625f,0.3709999796152116946501564598293043673038482666015625f,0.77420000785887221894654430798254907131195068359375f,-0.037900013244151598001963066053576767444610595703125f,-0.87440000859498956398141444879001937806606292724609375f,-0.4680000343620778213704625159152783453464508056640625f,-1.0413000037789341689631328335963189601898193359375f,-0.115799988117814667276661566575057804584503173828125f,1.038099966967106269066789536736905574798583984375f,-0.52799995329976123770165941095910966396331787109375f,-0.7637999811291695806403367896564304828643798828125f,-0.48460000471174691227815856109373271465301513671875f,-0.7859999897480012265305049368180334568023681640625f,-0.137999974161386429472031522891484200954437255859375f,-0.152200015696883017568552531884051859378814697265625f,-1.9617999646037820316024635758367367088794708251953125f,0.428299981263279949672551083494909107685089111328125f,-1.31319996454119714002217733650468289852142333984375f,0.707999999046325712015459430404007434844970703125f,0.8472000048190349019705536193214356899261474609375f,-0.594200000333785727235635931720025837421417236328125f,0.87680000252723655496112087348592467606067657470703125f,0.29520000647306421370785756153054535388946533203125f,-0.133199988532066271318399230949580669403076171875f,-1.880600046092272048525728678214363753795623779296875f,-1.3478000174582007719692455793847329914569854736328125f,0.3919000410795216993165013263933360576629638671875f,1.05910000216364874603414136799983680248260498046875f,-0.09590001229047828701368416659533977508544921875f,-0.6599000323772425957713494426570832729339599609375f,0.91520003627538670798458042554557323455810546875f,1.156999982774257862416789066628552973270416259765625f,0.539400015896558837624752413830719888210296630859375f,1.4455000292956829976986909969127736985683441162109375f,0.7735000055432319499004734097979962825775146484375f,0.43150006106495908397846505977213382720947265625f,-1.008500014334916983216317021287977695465087890625f,-0.049299929437040201918307502637617290019989013671875f,-1.099600056716800278167056603706441819667816162109375f,-1.30040001316964559663347245077602565288543701171875f,-0.15000000879168506884298039949499070644378662109375f,0.94590001884698882150814824854023754596710205078125f,0.104899992889165627474312714184634387493133544921875f,0.1612999806821340254003871450549922883510589599609375f,0.50640000045895583458133160092984326183795928955078125f,-1.2137999939441679320140110576176084578037261962890625f,0.6224999786913396615517513055237941443920135498046875f,-0.14059998827576603019906542613171041011810302734375f,1.276699991840124059905292597250081598758697509765625f,0.26159999076426021513697151021915487945079803466796875f,0.0361000183165074428615071155945770442485809326171875f,-1.009300027859211024150454250047914683818817138671875f,-0.86009999474287013310203064975212328135967254638671875f,1.3138000026613472215331057668663561344146728515625f,-1.52020002107620211262428711052052676677703857421875f,-0.137600001597404375530686593265272676944732666015625f,-0.63380002233088028606999841940705664455890655517578125f,-0.277499992549419260967624722979962825775146484375f,-0.447199963319301563302587965154089033603668212890625f,1.2823000018715855485851307093980722129344940185546875f,0.2546000088095663471676743938587605953216552734375f,-0.691200008291005918437122090836055576801300048828125f,0.7358000312566754796961276952060870826244354248046875f,0.718599981510639196358170011080801486968994140625f,-0.1993999643385411246754301828332245349884033203125f,-0.174400004422664789416330677340738475322723388671875f,0.96540000974237905051467123485053889453411102294921875f,-1.1429999933838843606537238883902318775653839111328125f,1.2124000116646287406041437861858867108821868896484375f,-0.463600009971857129897898630588315427303314208984375f,-0.3886000088542704844485342619009315967559814453125f,0.253799990591407009787872084416449069976806640625f,0.339999999701976474142384176957421004772186279296875f,0.518400015717744633292340949992649257183074951171875f,1.037800027441978301112612825818359851837158203125f,1.729200090336801221013729445985518395900726318359375f,1.49670008243918406520833741524256765842437744140625f,-0.04089997712373616423064959235489368438720703125f,0.83190006447434416969599624280817806720733642578125f,1.392400046384335343674365503829903900623321533203125f,1.02920004250407171042525078519247472286224365234375f,-0.91529996709227479101400604122318327426910400390625f,-0.0172999599993222119564961758442223072052001953125f,1.126199973720310776315045586670748889446258544921875f,1.315199941205978717562175006605684757232666015625f,-0.0503999433577053679300661315210163593292236328125f,1.2950000190734858307450849679298698902130126953125f,-1.280700059461593820486768890987150371074676513671875f,-0.40550000768899874259432181133888661861419677734375f,0.8491000446319585392984663485549390316009521484375f,0.7589000012040134635071808588691055774688720703125f,0.02900004020333302179324164171703159809112548828125f,-1.41480005596280111745954854995943605899810791015625f,-0.26339995991289610888230754426331259310245513916015625f,0.3420000182390214860106425476260483264923095703125f,-0.4308999663949013569919088695314712822437286376953125f,-1.2767000279009341756619733132538385689258575439453125f,-0.00449996049702161560190916134160943329334259033203125f,0.505600031936168203827719480614177882671356201171875f,-0.0160000235736366125394170012441463768482208251953125f,-0.958399971684813056782559215207584202289581298828125f,1.228499986022710377397970660240389406681060791015625f,0.370299954426287936115613774745725095272064208984375f,-0.463999984845518742559988822904415428638458251953125f,-0.1124999422579993080262283910997211933135986328125f,-0.00820001364052291847883680020458996295928955078125f,-0.7912000401049856801449777776724658906459808349609375f,0.541800006401538691847008522017858922481536865234375f,-0.857599990272522205714267329312860965728759765625f,1.32260001098513679806956133688800036907196044921875f,-0.5579000279963022279616779997013509273529052734375f,0.89219997285604524250857139122672379016876220703125f,-0.218300003838539424805276212282478809356689453125f,1.3356000013887887600816384292556904256343841552734375f,-0.6994000126183035348503835848532617092132568359375f,-0.100400000429153646308577663148753345012664794921875f,1.17379994655847585960373180569149553775787353515625f,1.512000001624226097618475250783376395702362060546875f,-1.03789996377229698509836453013122081756591796875f,0.649799991142750332784316924517042934894561767578125f,0.0220999733448028479187996708787977695465087890625f,1.0551999959677456075723966932855546474456787109375f,0.74220000474452962890836715814657509326934814453125f,0.4442000252932312154285909855389036238193511962890625f,-0.386400011336803572703502140939235687255859375f,-0.10470001718699972759196725746733136475086212158203125f,0.71330007223486990852734379586763679981231689453125f,-0.15500000678002834320068359375f,-0.71370001469850574693509770440869033336639404296875f,-0.65370005418658205798010385478846728801727294921875f,-1.225500001877545486195231205783784389495849609375f,0.25929998062252934687421657145023345947265625f,-1.30080002037882724863493422162719070911407470703125f,-1.69499998107552496406924547045491635799407958984375f,-0.616800011020899319191812537610530853271484375f,-0.697500060126185150721767058712430298328399658203125f,-1.667599996605514167669070957344956696033477783203125f,-1.26040003149807500193446685443632304668426513671875f,-0.20689999294876937341314260265789926052093505859375f,0.3038000180840487729483356815762817859649658203125f,0.04990000436306019082621787674725055694580078125f,0.125600025677680804392366553656756877899169921875f,0.2119000032678244327399852409143932163715362548828125f,-0.7538000355184084622095497252303175628185272216796875f,0.8089999901726845354943407073733396828174591064453125f,0.906900016343593851075866041355766355991363525390625f,0.66899998007714744918672522544511593878269195556640625f,1.4449999649822711234037342364899814128875732421875f,-0.5854999650716783410331345294252969324588775634765625f,-0.597799972307682114802673822850920259952545166015625f,-0.55410002093911125964353914241655729711055755615234375f,0.628699972453713673559150265646167099475860595703125f,0.398099984848499344192163107436499558389186859130859375f,0.927499980852008132359287628787569701671600341796875f,0.14800001752376534813748776286956854164600372314453125f,0.070200009751320013862141422578133642673492431640625f,0.268200043591857006930467832717113196849822998046875f,-0.543700014474988169155267314636148512363433837890625f,-0.4639000026941300092175879399292171001434326171875f,0.43130000884532904148471743610571138560771942138671875f,-0.92959996865093696971626968661439605057239532470703125f,0.4051999563306571960907831453368999063968658447265625f,-1.247399990653991874722805732744745910167694091796875f,0.341600017064809691191840101964771747589111328125f,0.144199965912104488552358816377818584442138671875f,-0.40219996510744149276206371723674237728118896484375f,0.018199980485439137822822885937057435512542724609375f,1.286699956971407488026670762337744235992431640625f,-1.173599966236948954900753960828296840190887451171875f,0.31960001871883836344068186008371412754058837890625f,-1.000399995437263545028372391243465244770050048828125f,0.963400010055302846723179754917509853839874267578125f,0.955399986720085525604417853173799812793731689453125f,-0.8045999945044517431824715458787977695465087890625f,0.608000000759959302598645081161521375179290771484375f,0.260099986696243234973735525272786617279052734375f,0.71360003142952965760059669264592230319976806640625f,0.96079997537731998136223410256206989288330078125f,0.67900004169344985172074302681721746921539306640625f,-0.65230001721978236872701018000952899456024169921875f,-0.9062000093340873974057103623636066913604736328125f,-0.7733000321507450536273609031923115253448486328125f,-0.1099999845027923583984375f,0.5012000155925750988217259873636066913604736328125f,0.64900000756978926119700190611183643341064453125f,-1.133000026762485124010026993346400558948516845703125f,-2.1078000289320950599858406349085271358489990234375f,-1.478400024509429755426026531495153903961181640625f,-0.539600032651424754703839425928890705108642578125f,0.7226000419795521878540967009030282497406005859375f,-0.2935000088214874125469577847979962825775146484375f,0.438500010609626400537308654747903347015380859375f,0.4899999797344207763671875f,-0.11789999595880484850596303658676333725452423095703125f,-0.642499988600611526834427422727458178997039794921875f,-0.5970000284463166106974085778347216546535491943359375f,0.6035999664604665415623685476020909845829010009765625f,-0.6202000211507083804463036358356475830078125f,0.620300041225552600820947191095910966396331787109375f,0.161900024092197281078142623300664126873016357421875f,-0.05040005556345050763411563821136951446533203125f,-1.137500005811453274162658999557606875896453857421875f,0.04840000521242604492044847575016319751739501953125f,-0.5717999931395052470861628535203635692596435546875f,1.36000002764165461854872773983515799045562744140625f,0.122500013858080070150435858522541821002960205078125f,-1.265800009351969013238203842774964869022369384765625f,0.945999999806284908032694147550500929355621337890625f,0.380500002622604416302465324406512081623077392578125f,0.5181999994099140138814618694595992565155029296875f,-0.7839000034391876425843292963691055774688720703125f,0.634000013306736054374823652324266731739044189453125f,-0.43620000158548322133356123231351375579833984375f,-1.4329999696910380446723820568877272307872772216796875f,0.948400008714198605730416602455079555511474609375f,0.878399985244870062928157494752667844295501708984375f,-0.584099983239173781157660414464771747589111328125f,-0.158200039494037270060289301909506320953369140625f,-0.23239998677968998208598350174725055694580078125f,-0.86340004410445647664573698421008884906768798828125f,0.9239999959468843115928393672220408916473388671875f,-0.6084000087887044827539284597150981426239013671875f,0.570000011622905589092624722979962825775146484375f,-0.004999984651803845991935304482467472553253173828125f,0.0941999752998352590793729177676141262054443359375f,-0.495199990975856874086957759573124349117279052734375f,-0.36089999113082882331582368351519107818603515625f,-0.6157999993681908534881586092524230480194091796875f,-1.1220000241994849687898749834857881069183349609375f,-0.758299970236420595171011882484890520572662353515625f,-1.113800031122565314944949932396411895751953125f,-0.39229999687969741017923297476954758167266845703125f,1.51200006860494706728559322073124349117279052734375f,0.548200028467178857027874983032234013080596923828125f,0.9915000288784503457151231486932374536991119384765625f,0.0082999815613031824312884054961614310741424560546875f,-0.11960002214610609971856547417701222002506256103515625f,-0.3661000027820466851125047469395212829113006591796875f,1.734599992120264833062037723721005022525787353515625f,0.926900029084086529707064983085729181766510009765625f,-1.49240002820491834967242539278231561183929443359375f,-1.033999995350837419749723267159424722194671630859375f,0.6323000102162357194401920423842966556549072265625f,0.510699963498115749871431034989655017852783203125f,-0.07389999777674649550363028538413345813751220703125f,1.5203000088900324726637336425483226776123046875f,-0.6555999891698365278358551222481764853000640869140625f,-0.039500011086464059673062365618534386157989501953125f,-0.59069996819198100634906722916639409959316253662109375f,-1.499300016683340341927532790577970445156097412109375f,0.297299986150860728884026684681884944438934326171875f,0.13219999192953102440384327564970590174198150634765625f,0.37560000578463093034997655195184051990509033203125f,0.2685000368729235642462072064517997205257415771484375f,0.090400000913441136418668975238688290119171142578125f,-0.5486999895900483981137085720547474920749664306640625f,-0.0268000043898819484411433222703635692596435546875f,0.8405000073164696772209936170838773250579833984375f,-0.4020000302344561848855164498672820627689361572265625f,-0.27380000824928274472114253512700088322162628173828125f,0.7252999916791915691050007808371447026729583740234375f,0.3180999967694280172736398526467382907867431640625f,-0.9018000285297631535286200232803821563720703125f,-0.2315999688595535843660400132648646831512451171875f,1.723799998116493537025917248683981597423553466796875f,0.065399995809793498580120285623706877231597900390625f,0.031999992161989343486538928118534386157989501953125f,0.142199969279765969787376889144070446491241455078125f,-1.2668000083386896648107722285203635692596435546875f,-0.5000999947056172789672245926340110599994659423828125f,-0.20420001568198220542882381778326816856861114501953125f,0.0332000080153347365552463088533841073513031005859375f,-0.138799980086088847741621066234074532985687255859375f,-1.048399993887543235615567027707584202289581298828125f,-0.49319997728168996076192343025468289852142333984375f,1.276199986684322329466567680356092751026153564453125f,-0.04099998646974578608848105432116426527500152587890625f,-0.47099998944997789607214144780300557613372802734375f,-0.7576000057697296341530091012828052043914794921875f,-0.4275999950408928551581766441813670098781585693359375f,-1.2265000054836274756553393672220408916473388671875f,-0.47940003362894090255252876886515878140926361083984375f,-0.78430001254826808843034768869983963668346405029296875f,-0.02849997466057530015604015716235153377056121826171875f,0.98330001454874882416135051244054920971393585205078125f,-0.66270000364184322716454289547982625663280487060546875f,-0.25629999237954559365704199080937542021274566650390625f,0.217299993526935519838616528431884944438934326171875f,0.735799995642900217518445060704834759235382080078125f,0.6831000346183782045983434727531857788562774658203125f,-0.30060000280439902464735268949880264699459075927734375f,1.0876000544592743235483567332266829907894134521484375f,0.63799999597296108699850947232334874570369720458984375f,0.52630003743693298812189595992094837129116058349609375f,0.7567000427089636271915651377639733254909515380859375f,-0.18020001824498166342891636304557323455810546875f,0.7210999630331986764986140769906342029571533203125f,-0.211899989819527068135585068375803530216217041015625f,-0.04979996819496157467455077494378201663494110107421875f,0.02100001161545517192053011967800557613372802734375f,0.6540999878771602826788011952885426580905914306640625f,-0.2903999979145821530579496538848616182804107666015625f,-0.1548999957948924421913261539884842932224273681640625f,-0.2323000031754374195003265413106419146060943603515625f,-0.3359999962300059817010833285166881978511810302734375f,-0.044999985881149895394770510392845608294010162353515625f,-1.077300008651614238175397986196912825107574462890625f,0.3915999956443905194447552275960333645343780517578125f,-0.1403000145897268513550670832046307623386383056640625f,1.08400003889948148838584529585205018520355224609375f,0.0582000042900443492754902763408608734607696533203125f,-0.0261999991826713241049873204246978275477886199951171875f,0.5471000016570091162293465458787977695465087890625f,-1.4470000042319295374682042165659368038177490234375f,-0.1842999785363677034411011845804750919342041015625f,0.4337999791175126684805718468851409852504730224609375f,0.09640000386759661754609851413988508284091949462890625f,0.049399999766051860827786867957911454141139984130859375f,-0.08650001034513110642620858925511129200458526611328125f,0.833999979928135903861630140454508364200592041015625f,0.17119999644458283594161684959544800221920013427734375f,1.15479998078644285897098598070442676544189453125f,-0.3751999868035318375092401765868999063968658447265625f,-0.4839999934881922261098452509031631052494049072265625f,0.68439997283220288526450758581631816923618316650390625f,-0.03499997150152889258123423132929019629955291748046875f,-0.8938000059954822607011237778351642191410064697265625f,-0.0370000022575258658008579004672355949878692626953125f,-0.07500001106411235962667660714942030608654022216796875f,-0.45610003470778526235562821966595947742462158203125f,0.42930004335641935853118411614559590816497802734375f,0.4901000005543227899806879577226936817169189453125f,-1.09930001832246748705301797599531710147857666015625f,-0.08230001081228266457401332445442676544189453125f,-0.3955000431537631921941056134528480470180511474609375f,-0.59909999188184759333353213150985538959503173828125f,-0.8727999791622149672321029356680810451507568359375f,-1.954400008445978276228061076835729181766510009765625f,0.180600020760298018984713053214363753795623779296875f,-0.2128000301241872449509173748083412647247314453125f,0.71560005011558569520957462373189628124237060546875f,1.3548000042557719613256494994857348501682281494140625f,-1.00110005691051551224290960817597806453704833984375f,0.577599980810284296950385396485216915607452392578125f,0.0830999899148938681037179776467382907867431640625f,-0.0766000175118453086042791255749762058258056640625f,0.58280000576078860863304953454644419252872467041015625f,0.369299995821714244215172584517858922481536865234375f,0.5496000096291304704010372006450779736042022705078125f,0.93429998777508682650250193546526134014129638671875f,-0.631699985507130801209996207035146653652191162109375f,-0.13699999953806329955341425375081598758697509765625f,-1.18160000678300836085554692544974386692047119140625f,0.079799966108798248143330056336708366870880126953125f,-0.162899994021654226372675111633725464344024658203125f,-0.15879997464716411048613053935696370899677276611328125f,1.1105000169277194910222306134528480470180511474609375f,0.04530000628232944137607773882336914539337158203125f,0.632699998724460588306328645558096468448638916015625f,0.76500004403293164756405531079508364200592041015625f,-0.31890001185834437880117775421240366995334625244140625f,-0.943100022846460550596248140209354460239410400390625f,-0.10380000064969063799935611314140260219573974609375f,0.6910000027120108967437772662378847599029541015625f,-0.918199996727704803589631410432048141956329345703125f,-0.24990001240968684470544758369214832782745361328125f,0.62719995571970965642094597569666802883148193359375f,-0.361399975574016707469127140939235687255859375f,-0.2666000214606523144311722717247903347015380859375f,0.37500000640749941993590255151502788066864013671875f,-1.21660000723004291245388230890966951847076416015625f,0.833199987488984827876947747427038848400115966796875f,0.01000000558793556759695775326690636575222015380859375f,-0.07070000998973835493188744294457137584686279296875f,0.2443999922037125571705473703332245349884033203125f,-0.3008000089794398146381126935011707246303558349609375f,0.79519996549487093062680287403054535388946533203125f,0.07760002461969861542456783354282379150390625f,0.800400004452467062066034486633725464344024658203125f,0.0842999920964242477339212200604379177093505859375f,-0.0172000041484832166815976961515843868255615234375f,0.600300016899406596593280482920818030834197998046875f,1.48910000011473897529867826960980892181396484375f,-0.1877000084288418246813989753718487918376922607421875f,-0.5697999967306852653337045921944081783294677734375f,-1.020500012978910575611735112033784389495849609375f,0.204599983980506383574038409278728067874908447265625f,-1.9633999452352526304821367375552654266357421875f,-2.07849996821582383432769347564317286014556884765625f,-0.655499964475632079796696416451595723628997802734375f,0.1696999972566957115560626334627158939838409423828125f,-0.826599966251850393206268563517369329929351806640625f,0.75410000112056707610719286094536073505878448486328125f,0.37229998518228502035753990639932453632354736328125f,0.6421999898433685682874738631653599441051483154296875f,0.04579998863935497155353004927746951580047607421875f,-0.485999996192753247470363930915482342243194580078125f,0.26639997944831872445092812995426356792449951171875f,-0.833599978940188890419449307955801486968994140625f,0.8828999707758422577086321325623430311679840087890625f,-0.22420001510456166737839112101937644183635711669921875f,0.44390000183731326277580819805734790861606597900390625f,0.307699967879057123543162788337212987244129180908203125f,-0.79139995867982493304992885896353982388973236083984375f,0.4846999777734282144336930286954157054424285888671875f,-0.4122999307930488299689386622048914432525634765625f,-0.1157000174403182146676272168406285345554351806640625f,0.812000000655650477909830442513339221477508544921875f,0.3621999990820885528819417231716215610504150390625f,0.232100014919041530703225362231023609638214111328125f,1.329199966508150421162781640305183827877044677734375f,0.30879996294975331494470083271153271198272705078125f,-0.746399977064132968962439917959272861480712890625f,0.21889999881982813434433410293422639369964599609375f,-1.015199967357516275257012239308096468448638916015625f,0.26840000849068079702419709064997732639312744140625f,-0.72669996330440056464539111402700655162334442138671875f,-0.2733999848723416281615072875865735113620758056640625f,0.1411000079974530085902273413012153469026088714599609375f,0.39970002888441058530588634312152862548828125f,0.79659997470825982812669963095686398446559906005859375f,-0.1342000628441570864168852494913153350353240966796875f,0.32069999341219646016298838731017895042896270751953125f,-0.476900020031631177719333436471060849726200103759765625f,0.74469999304711798071565453938092105090618133544921875f,0.051599979364871817466564607457257807254791259765625f,0.1932000119641425761596309484957600943744182586669921875f,-0.37930000073909742042133075301535427570343017578125f,-0.480200004871189911614948186979745514690876007080078125f,0.668500032514333764055436404305510222911834716796875f,-0.026499997437000211153446116441045887768268585205078125f,-0.4429999924898151419938585604541003704071044921875f,-1.68709991701841399702743728994391858577728271484375f,-0.8396999778851874651763864676468074321746826171875f,-0.0748999612987051310852848473587073385715484619140625f,1.56850001791119542104979700525291264057159423828125f,0.4855999365687377888889386667869985103607177734375f,0.069899972921609876408410855219699442386627197265625f,0.662499970346689170952458880492486059665679931640625f,-1.211499986745417256628343238844536244869232177734375f,1.2887000110104676053879302344284951686859130859375f,0.910200017611682454798938124440610408782958984375f,0.419099988007545665169573112507350742816925048828125f,-0.5282999950841069658480364523711614310741424560546875f,0.56100000673532424144696051371283829212188720703125f,1.0843000075191262521201451818342320621013641357421875f,-0.655999987065791767548716961755417287349700927734375f,-1.019499995999038066685216108453460037708282470703125f,-1.353099919283391017188478144817054271697998046875f,-0.50150003439187906817409157156362198293209075927734375f,-0.1218999340146798004980155383236706256866455078125f,0.5090999938189988416326059450511820614337921142578125f,1.2183999690026035001011450731311924755573272705078125f,0.450699972736835707820546303992159664630889892578125f,0.57049997669458463178671081550419330596923828125f,-0.2023999670356515512281703195185400545597076416015625f,-0.4772999728888276393945488962344825267791748046875f,-0.4638999859675763115518520862678997218608856201171875f,0.0949999906867737831106524026836268603801727294921875f,-0.88649999266862844482517402866506017744541168212890625f,-0.0376999854251740540433956994093023240566253662109375f,0.6663000143587585188953426040825434029102325439453125f,-0.94850000002980205948688308126293122768402099609375f,-0.051200028109550377308778479346074163913726806640625f,-0.586299983811378044862294700578786432743072509765625f,-0.712199947710335568018535923329181969165802001953125f,-1.65260001813769452638780421693809330463409423828125f,-0.315100001187622158482781742350198328495025634765625f,-1.217700034785270446491267648525536060333251953125f,0.296600004845857245783236066927202045917510986328125f,-0.6546999956548216204055279376916587352752685546875f,0.2302000001028183806539573197369463741779327392578125f,0.726299993199109650277023320086300373077392578125f,-0.91970000243484928414972046084585599601268768310546875f,-0.1960000126957890387302541057579219341278076171875f,-0.24450001659989395452754479265422560274600982666015625f,-0.15809997993111579717151471413671970367431640625f,-0.2345999986395241876380168832838535308837890625f,-0.220799999368190835724590215249918401241302490234375f,0.4647999937132001324613383985706605017185211181640625f,-0.1096000165581703100770027958787977695465087890625f,0.1843000037565827309293808866641484200954437255859375f,-0.40580000741481825343726086430251598358154296875f,0.146299999010562231660514953546226024627685546875f,-0.336400001427531680775473432731814682483673095703125f,1.34639999017715439322273596189916133880615234375f,0.584200025628506924846305992105044424533843994140625f,1.36859995439052628540821388014592230319976806640625f,0.512199996548890812420040674624033272266387939453125f,-0.59919999407529811463746227673254907131195068359375f,-0.0267999985039237831330183325917460024356842041015625f,-0.94039995661974007390426777419634163379669189453125f,-1.2471999692052602171088437899015843868255615234375f,0.531499996244907180198424612171947956085205078125f,1.441499994866550338201705017127096652984619140625f,0.2119999821782112547907672706060111522674560546875f,-1.755400007842480736286461251438595354557037353515625f,0.69860000416040435311515466310083866119384765625f,0.988599997827410703621353604830801486968994140625f,0.35850004696845871876575984060764312744140625f,1.106599975788594036885115201584994792938232421875f,0.37220000325441304767082328908145427703857421875f,-0.1235999809145926775499901850707828998565673828125f,0.121800000143050368706099106930196285247802734375f,-0.058600015783309533645706324023194611072540283203125f,-0.60869996686577820810271077789366245269775390625f,-0.259200001972913707248835635255090892314910888671875f,-1.39739994043707849158408862422220408916473388671875f,0.35390001595616293883495018235407769680023193359375f,-0.20710000821352014810372565989382565021514892578125f,0.52280000181198094111323371180333197116851806640625f,-0.209100028538703242020346806384623050689697265625f,0.2021999879062175153876523836515843868255615234375f,-0.388099999451636534786302945576608180999755859375f,-0.89119998370409003740633124834857881069183349609375f,0.1602999904870987535332460538484156131744384765625f,-0.250199970263242921220125936088152229785919189453125f,-0.53220000213682749912180725004873238503932952880859375f,0.168199986110627863755695443614968098700046539306640625f,1.1472999973267314910430059171631000936031341552734375f,0.33570000056475401972733152433647774159908294677734375f,0.9234000118806957591033324206364341080188751220703125f,0.22940000095814472302890862920321524143218994140625f,0.6678000158563259791577593205147422850131988525390625f,0.61709998992234460502004367299377918243408203125f,0.66920002164244696274408852332271635532379150390625f,-0.4053999904453748914789912305423058569431304931640625f,-1.049800011073052541377137458766810595989227294921875f,0.182199972818791577910957357744337059557437896728515625f,0.22149999525770536212831984812510199844837188720703125f,1.0316999808132651939018842313089407980442047119140625f,0.850799984280765198008111838134936988353729248046875f,0.4251000046707689161706866798340342938899993896484375f,-0.3870000099688770678341143138823099434375762939453125f,0.66279999994933536111574312599259428679943084716796875f,1.382799991790950144832095247693359851837158203125f,-0.307599990421533409090670829755254089832305908203125f,-0.41509997682422417053516028317972086369991302490234375f,-0.7650999822258948146469492712640203535556793212890625f,-0.22189999648779645813334582271636463701725006103515625f,-0.4555999893188478200301005927030928432941436767578125f,0.4790999984011055090604713768698275089263916015625f,-1.2755000348091112272186364862136542797088623046875f,-0.4974000459372998061979842532309703528881072998046875f,0.4991999769389625640769736492075026035308837890625f,0.669999998696148235666214532102458178997039794921875f,1.188799965780973355577998518128879368305206298828125f,0.44920000171214347073345152239198796451091766357421875f,0.090199996452033637250877973201568238437175750732421875f,-0.334799998816847743654534497181884944438934326171875f,0.459900019003450122312415260239504277706146240234375f,1.00109998270273248266448717913590371608734130859375f,0.30929997864812597274664085489348508417606353759765625f,-0.2397000136107203527302544898702763020992279052734375f,-0.10049998496472856146510821417905390262603759765625f,-0.40230000518709407852924186954624019563198089599609375f,0.522399997992813336367134979809634387493133544921875f,-0.3307999796986582641267204962787218391895294189453125f,1.4468999469414354219765073139569722115993499755859375f,-0.5218000069975854415815774700604379177093505859375f,0.9508999923229222162035512155853211879730224609375f,1.245000019669532331789696399937383830547332763671875f,0.530899970678984534089295266312547028064727783203125f,-0.19279999085962773452962437659152783453464508056640625f,0.30099998004362016335022644852870143949985504150390625f,-0.9279999787807466926636834614328108727931976318359375f,0.052700006528943854622326625758432783186435699462890625f,0.6948000049263234334517846946255303919315338134765625f,1.10510001500844889932295700418762862682342529296875f,0.73010000007003572619623810169287025928497314453125f,0.38969999043941516703171146218664944171905517578125f,0.7120000094845890803441079697222448885440826416015625f,-0.80550006073713387166890242951922118663787841796875f,-0.7215999936699859507172050143708474934101104736328125f,-0.76050000342354084903462307920563034713268280029296875f,-0.44530002520680478372838706491165794432163238525390625f,0.61370001959726205864598114203545264899730682373046875f,-0.155199981308728585105427555390633642673492431640625f,-0.66449999544024507258654921315610408782958984375f,-0.84360000233501153399373606589506380259990692138671875f,-0.35180000997781757376969835604541003704071044921875f,-0.06160001699030404065382526823668740689754486083984375f,0.1950999872922896116733681992627680301666259765625f,0.1127000012576580001422144050593487918376922607421875f,0.12220003275871249570627696812152862548828125f,0.797200004148482577193135512061417102813720703125f,1.007099999235570475519807587261311709880828857421875f,-0.052800021183490031262408592738211154937744140625f,-1.846899990303814576009244774468243122100830078125f,-0.2180000008344649131686310283839702606201171875f,-0.29600003504753147343109276334871537983417510986328125f,-0.35829995615482357340653152277809567749500274658203125f,1.99510001672208314715817323303781449794769287109375f,-1.102199987459183017080022182199172675609588623046875f,0.07619993559196647225917331525124609470367431640625f,0.0090999626755721152449041255749762058258056640625f,-0.402299984250963138521228756872005760669708251953125f,0.19020000021457672545466266456060111522674560546875f,-0.7587999714806679119050158988102339208126068115234375f,0.2024000138998029996173499966971576213836669921875f,0.24890001144856199744026525877416133880615234375f,-0.2364999762773525304737631813623011112213134765625f,-0.7702999921262261295851203612983226776123046875f,-0.2616999890089033442563959397375583648681640625f,-0.64380001212358450857209390960633754730224609375f,0.1932000305533403405888748238794505596160888671875f,-1.188899972891807266250907559879124164581298828125f,0.287300012600422149944279226474463939666748046875f,0.1386000448591999667513618987868539988994598388671875f,0.979799927160888817212480716989375650882720947265625f,0.05249999776482561220092293297057040035724639892578125f,1.04279995708987183178351187962107360363006591796875f,1.646599943900108886651878492557443678379058837890625f,-0.9806999643921852083394696819595992565155029296875f,0.4034999933242797709453952847979962825775146484375f,-1.05629995967149792335249003372155129909515380859375f,-0.6127999875068663726551676518283784389495849609375f,-0.1262999891757967674266183166764676570892333984375f,-0.7337999654084448319935063409502618014812469482421875f,-0.914899982085824348132518935017287731170654296875f,0.0269999916777012016755321610617102123796939849853515625f,1.3202000198096037753003884063218720257282257080078125f,-0.36099995983392008103152193143614567816257476806640625f,-0.6779999690949918278448649289202876389026641845703125f,0.5680999869719147721269791873055510222911834716796875f,0.8656999738916748565742409482481889426708221435546875f,0.39419999675750727607947965225321240723133087158203125f,0.98640001970008039933190957526676356792449951171875f,0.35240000697076301339194515094277448952198028564453125f,-0.768100046874583330946961723384447395801544189453125f,0.4205999942548575010192735135206021368503570556640625f,-0.6566000260055062653918866999447345733642578125f,0.7571000109702341607231801390298642218112945556640625f,0.150199997532367124364327537477947771549224853515625f,0.300400018906593568868146348904701881110668182373046875f,-0.7582000016450880508500631549395620822906494140625f,0.9964000286221494917526797507889568805694580078125f,0.8896000446841123565633324687951244413852691650390625f,-1.3862999541580676154950424461276270449161529541015625f,-1.1235000213012094061326706651016138494014739990234375f,0.3281999958336354605847873244783841073513031005859375f,1.1274999783188104363063075652462430298328399658203125f,0.5615000051856029461561092830379493534564971923828125f,0.8675000004842881029532009051763452589511871337890625f,-0.7728999845594162199091670117923058569431304931640625f,-1.41409998823106253240666774217970669269561767578125f,0.3729000032231206684940616469248197972774505615234375f,-0.66910002250373334931055069318972527980804443359375f,1.1530000163316724837159199523739516735076904296875f,1.93549998496472852593797142617404460906982421875f,0.360799994599819040530519487219862639904022216796875f,0.2007999969094991232765323729836381971836090087890625f,-0.69249996602535279777157484204508364200592041015625f,1.08269997771382353590752245509065687656402587890625f,-0.7606000162154433752448312588967382907867431640625f,-1.454700002360343802365605370141565799713134765625f,-0.1011999878764144877862918292521499097347259521484375f,0.6530999754607680340967590382206253707408905029296875f,1.2106999494165180397686754076858051121234893798828125f,0.13670002488344901170336243012570776045322418212890625f,0.0946999732658273529750658781267702579498291015625f,-0.3145000263601540613223050968372263014316558837890625f,-0.029899996931105565778352683992125093936920166015625f,-0.6382999683737755614032494122511707246303558349609375f,0.871199957235903266195009564398787915706634521484375f,-0.7687999715924263011146422286401502788066864013671875f,1.446299939852952576302413945086300373077392578125f,-0.318400013184546804012597931432537734508514404296875f,0.521199996966123624275724068866111338138580322265625f,-0.001199983517825575773940727231092751026153564453125f,-1.3964000235557556539589540989254601299762725830078125f,0.5308999960109590521284417263814248144626617431640625f,0.272599987441301205848276367760263383388519287109375f,0.529900003543496200819618024979718029499053955078125f,0.354399993880093244769113880465738475322723388671875f,-0.67099998032301655381814953216235153377056121826171875f,-1.63069999441802426787262447760440409183502197265625f,-1.2394999811723834692855916728149168193340301513671875f,-0.2961999902606020640405404265038669109344482421875f,-0.7361999479785563682554538900149054825305938720703125f,-1.52969999073743867512575889122672379016876220703125f,-0.299900003468990039579011863679625093936920166015625f,0.0495000103414060532713847351260483264923095703125f,0.4311999748006465349448035340174101293087005615234375f,0.799399993097782424200659079360775649547576904296875f,0.273099999786913294474999247540836222469806671142578125f,-0.43440001008510620295055559836328029632568359375f,1.2650999728381631115325944847427308559417724609375f,0.453800015141069490454128754208795726299285888671875f,0.5387000063538547323105376563034951686859130859375f,0.9576999808430672889159041005768813192844390869140625f,-1.29589998174309695144756915397010743618011474609375f,-0.639000023551285156742096660309471189975738525390625f,0.22580000093281216155816082391538657248020172119140625f,0.3432000332728029068363184705958701670169830322265625f,-0.82499994248151864439932978712022304534912109375f,-1.2751000272646544342336483168764971196651458740234375f,1.14039999268054970116281765513122081756591796875f,0.73370005139261440518794188392348587512969970703125f,0.9477999696254730821465273038484156131744384765625f,0.37699999499321001916740669912542216479778289794921875f,-0.118199996876716451055244760937057435512542724609375f,-0.47899999624490730010251127168885432183742523193359375f,0.5812000126123422205637325532734394073486328125f,0.35090001065135023594621088705025613307952880859375f,0.68689999130964185525272114318795502185821533203125f,-1.181799964380264356123007019050419330596923828125f,0.74839997429251692295792963705025613307952880859375f,-0.721999991714954436616835664608515799045562744140625f,0.496399994163215563247604222851805388927459716796875f,1.953699988919496366435168965836055576801300048828125f,-1.07409997619837582760737859643995761871337890625f,-0.17690003601312564995851062121801078319549560546875f,-0.882699943888187732454753131605684757232666015625f,0.9647999562740334766886007855646312236785888671875f,-0.6902999800562863441655281349085271358489990234375f,-0.0389999868571762053903739797533489763736724853515625f,-0.7166999969065184217242858721874654293060302734375f,-0.761599983537197378069549813517369329929351806640625f,-0.226899989074468511063287223805673420429229736328125f,1.032599994966387324524248469970189034938812255859375f,0.61929998598694790956642464152537286281585693359375f,-0.27669994993060864629796924418769776821136474609375f,0.21670005286037896663486890247440896928310394287109375f,-1.2244999736100432574659180318121798336505889892578125f,-0.9258999741435045560677963294438086450099945068359375f,-0.9381999834656717940362113949959166347980499267578125f,-0.02769998054504441142853465862572193145751953125f,-0.8534999880716200859609443796216510236263275146484375f,-0.53750000439584277867055561728193424642086029052734375f,-1.265800019559263578372565461904741823673248291015625f,-0.8768000144481653546080224259640090167522430419921875f,-0.162900015218556404317240549062262289226055145263671875f,0.54810001301169375409472195315174758434295654296875f,-0.3170999746859075596461252644076012074947357177734375f,0.2048000266075131747811610694043338298797607421875f,-0.146599996761977546810840067337267100811004638671875f,1.1523999829053881427398664527572691440582275390625f,-0.2034999843835834099081694148480892181396484375f,-1.0704999551624059250798381981439888477325439453125f,-1.3666999594300985876316190115176141262054443359375f,-0.3360000087097285703663374079042114317417144775390625f,-0.74189999682307217199195292778313159942626953125f,-1.1372999734103681479524539099656976759433746337890625f,-1.177799968135357033816035254858434200286865234375f,0.52759994568079715548947206116281449794769287109375f,-1.03480000641643954395476612262427806854248046875f,0.0211999873548744943718702415935695171356201171875f,-0.080099998170137354236430837772786617279052734375f,-0.6016999776467681915192997621488757431507110595703125f,-0.20069996737241790896177917602472007274627685546875f,1.068000001616776106772022103541530668735504150390625f,-0.6405999943852425104751091566868126392364501953125f,-0.91489999445378777931381364396656863391399383544921875f,1.07199994909763329786756003159098327159881591796875f,0.00539998031258637212204121169634163379669189453125f,-0.45109995960593263220061999163590371608734130859375f,0.26209995926320583947699560667388141155242919921875f,-0.979799962234496835122854463406838476657867431640625f,0.9559999718666076784501228758017532527446746826171875f,-0.665399962207675610414980837958864867687225341796875f,-0.5221999993801116346503476961515843868255615234375f,0.33149999549984965341309361974708735942840576171875f,1.185699981305003092302285949699580669403076171875f,-0.281100007736682311332288008998148143291473388671875f,0.9174000025749202702485263216658495366573333740234375f,-0.206500002279878192013029547524638473987579345703125f,-0.328700002032518145966832889826036989688873291015625f,-0.0859999884068969322470366023480892181396484375f,0.639199967551231651441412395797669887542724609375f,-0.2455999875307082191966401296667754650115966796875f,-0.12870000844001772133395888886298052966594696044921875f,1.0115000060796737191282090861932374536991119384765625f,-0.6798999923676254386606387924985028803348541259765625f,0.05219999673515551830149661327595822513103485107421875f,-1.1491000009343028676056519543635658919811248779296875f,-0.3374000184819105907507719166460447013378143310546875f,-0.25799996121600254905814608719083480536937713623046875f,-1.61159998547434835103331352001987397670745849609375f,-0.024400016045570627198912916355766355991363525390625f,-0.266499997958540735254473474924452602863311767578125f,0.464800005820393824507164026726968586444854736328125f,-0.0135999661624432999218470285995863378047943115234375f,0.2737000091850756433586866478435695171356201171875f,1.228399995601177518977920044562779366970062255859375f,-0.9787000038951634284245528760948218405246734619140625f,-0.26350000472366819082026268006302416324615478515625f,-0.370899994111061115376060115522705018520355224609375f,0.539100000077485930205511976964771747589111328125f,-0.0954000363409519547275294826249592006206512451171875f,-0.58279996765106911738030248670838773250579833984375f,0.53579998204559087238152415011427365243434906005859375f,0.4985000029169024404751553447567857801914215087890625f,1.9239999772831801738703916271333582699298858642578125f,-0.75650000067800304837106750710518099367618560791015625f,0.467999948531389264871904742904007434844970703125f,-0.01270002062916741891740457504056394100189208984375f,-1.11600000204145910487341097905300557613372802734375f,-0.39479999918937647152006320538930594921112060546875f,0.0873999772429463472889210606808774173259735107421875f,-0.74539999253153776948011000058613717555999755859375f,-0.412400006747245750915453754714690148830413818359375f,0.1906000119686126481610699556767940521240234375f,0.3467999932140113372724954388104379177093505859375f,0.33269999574422826071895542554557323455810546875f,0.2263999797463416285836501629091799259185791015625f,2.7868000531911842898580289329402148723602294921875f,1.647299968343972853546119949896819889545440673828125f,-0.138300058078767218461280208430252969264984130859375f,0.301800009009241865243211577762849628925323486328125f,-1.852900034701824072413955946103669703006744384765625f,-1.0709999825954437113750827847979962825775146484375f,-0.986400007611512474881010348326526582241058349609375f,-1.2757000239968296906312161809182725846767425537109375f,-0.0964999966174364676163577314582653343677520751953125f,0.2334999572262164146962959421216510236263275146484375f,-0.2685000057294966158139004619442857801914215087890625f,-0.992300040204823208256357247591949999332427978515625f,-0.1791000167667868847587442360236309468746185302734375f,0.883399994260072762841673466027714312076568603515625f,0.60920002506971382416622873279266059398651123046875f,0.64380001316666568556001948309130966663360595703125f,0.655500000983476382998560438863933086395263671875f,-1.479400004050135830624412847100757062435150146484375f,0.8102999910086392532093668705783784389495849609375f,1.29840004186928315021987145883031189441680908203125f,-0.9641000015676020051813566169585101306438446044921875f,0.93349997335672352249957839376293122768402099609375f,-1.439899998402595304725082314689643681049346923828125f,-0.462599987813830271221604562015272676944732666015625f,0.26679998539090143960805789902224205434322357177734375f,0.64650001010298741022097601671703159809112548828125f,0.58909999277591695854283670996665023267269134521484375f,-0.52330000659525399697713510249741375446319580078125f,0.59139999639838947143033465181360952556133270263671875f,0.570700030404329705646659931517206132411956787109375f,0.4843000065505502771401324935141019523143768310546875f,-0.80140002601444748631109860070864669978618621826171875f,0.0075000001490115320024187894887290894985198974609375f,0.1211999905586242931576634873636066913604736328125f,-0.0650000347942114586885509197600185871124267578125f,-1.029500001624226168672748826793394982814788818359375f,-0.5968000387370591131031005716067738831043243408203125f,-0.561799971905350137291179635212756693363189697265625f,0.13179999239444750713801113306544721126556396484375f,0.739699998784065115842167870141565799713134765625f,-0.03760000629127002724771955399774014949798583984375f,1.1319999891817567050367188130621798336505889892578125f,-0.9478000082194804054580572483246214687824249267578125f,-0.066099988774955154013923674938268959522247314453125f,-0.254199975970387459511812267010100185871124267578125f,0.4272999883860351122422116532106883823871612548828125f,0.680200006622075914464176094043068587779998779296875f,-0.424899980670213839317739257239736616611480712890625f,0.64600001068413270832735406656865961849689483642578125f,-0.06660001468062404228476225398480892181396484375f,0.413400004243850816010308335535228252410888671875f,0.045599983656405385090693016536533832550048828125f,-0.404200038778782300141756422817707061767578125f,-0.4068999993562698591631487943232059478759765625f,-0.585700033831596300615274230949580669403076171875f,-0.683599995964765838607490877620875835418701171875f,-0.252000060260295644098960110568441450595855712890625f,0.877200034397840777700139369699172675609588623046875f,1.1240999763846399872591064195148646831512451171875f,1.39440003437399884234082492184825241565704345703125f,0.88490003237724312867840126273222267627716064453125f,-0.71020003120899222182060839259065687656402587890625f,0.281800018322467593634428340010344982147216796875f,-0.32769999678730954428829136304557323455810546875f,-0.5969000133633617366513135493732988834381103515625f,0.1862000176787379501774921664036810398101806640625f,-0.006700026857853202955084270797669887542724609375f,0.3092000015258786760341536137275397777557373046875f,0.31430001936554941011081609758548438549041748046875f,0.000799983796477388153789433999918401241302490234375f,0.54820003137290473915754773770458996295928955078125f,0.78199998337030418582571655861102044582366943359375f,0.4191999973535533996482627117075026035308837890625f,0.796999981582164895854702990618534386157989501953125f,-0.899100009167194347270424259477294981479644775390625f,0.9341000228762628143641677525010891258716583251953125f,-0.8694999927580351339884146000258624553680419921875f,0.155999992281198540666764529305510222911834716796875f,0.9813000197976828875567889554076828062534332275390625f,-0.144499982699751949866140421363525092601776123046875f,-0.62080002343356621707926024100743234157562255859375f,-0.0140000233650210414992898222408257424831390380859375f,0.216299994987249277755836374126374721527099609375f,0.772700015115737937065887308563105762004852294921875f,-0.847100000762939231435666442848742008209228515625f,-1.23260001009106634484169262577779591083526611328125f,0.737200000870227967197934049181640148162841796875f,-1.22339999627172968388322260580025613307952880859375f,0.45379999155998262949651689268648624420166015625f,-0.27439998683929456291252790833823382854461669921875f,0.345699995684623839764526564977131783962249755859375f,1.5990000004172326697471362422220408916473388671875f,-0.84379999692440055270026277867145836353302001953125f,-0.26519999201893806883845172706060111522674560546875f,0.01119998281002043682974544935859739780426025390625f,0.5070000095218418767473167463322170078754425048828125f,-0.3903999834418296899229972041212022304534912109375f,-0.34379997688233832064952366636134684085845947265625f,1.146099983894824703156700707040727138519287109375f,-0.454999990910291529644382535479962825775146484375f,-0.4454999942779540589299358543939888477325439453125f,0.0448999956458806526171656514634378254413604736328125f,-0.9465000423640017590543038750183768570423126220703125f,0.1916999867364757026422239505336619913578033447265625f,-0.5880000117123120872975050588138401508331298828125f,-0.1977999800562864063380175139172933995723724365234375f,0.2815999993324280747941656954935751855373382568359375f,0.11960002318918705466188612263067625463008880615234375f,-0.2463000347733501627089935936965048313140869140625f,-0.36549995546042879368542344309389591217041015625f,0.1962000227451330403027895954437553882598876953125f,-0.9274999526143066219674437888897955417633056640625f,-0.6761000180184841834574172025895677506923675537109375f,-0.0380999722570172227875673343078233301639556884765625f,-1.5694000399619341123980120755732059478759765625f,-0.85790001637339596829434640312683768570423126220703125f,-0.35600001012533877275956228913855738937854766845703125f,-0.58350000839307891720153520509484224021434783935546875f,0.2064999986290929623322654151706956326961517333984375f,1.453200019638240636510317926877178251743316650390625f,-0.7276999847546221644734032452106475830078125f,-0.1664000015765429996150714941904880106449127197265625f,-0.69759998051226135284963447702466510236263275146484375f,-0.230900011694431361863877327778027392923831939697265625f,-0.65699999368935813048864247321034781634807586669921875f,0.56340000059306649671242439580964855849742889404296875f,-1.251900001777708215655593448900617659091949462890625f,-0.586800040972233372116306782118044793605804443359375f,-0.626299987462162643936380845843814313411712646484375f,1.2072000300019982699240017609554342925548553466796875f,-0.7370000209957361381185592108522541821002960205078125f,0.88269999943226584715461058294749818742275238037109375f,-0.57050000701844683081986886463710106909275054931640625f,-0.3731000085227191842562888268730603158473968505859375f,-0.0101000003494322587727083373465575277805328369140625f,-0.3732999986603855457900635883561335504055023193359375f,-0.15360004066824883839359472403884865343570709228515625f,-0.0489999673739076602174691288382746279239654541015625f,0.9057999677032235741336307910387404263019561767578125f,0.0494999948441983139701960681122727692127227783203125f,0.272799980503320771418884760350920259952545166015625f,-0.33019999917149533974480846154619939625263214111328125f,-0.42140000656843223936220965697430074214935302734375f,-0.2513999902516601547830532581428997218608856201171875f,0.214400013959407775843146737315692007541656494140625f,-0.1319000175356865867115629953332245349884033203125f,-0.791800022718310625435833571827970445156097412109375f,0.3901000096812843320748243058915250003337860107421875f,-0.23780000315308569280858819183777086436748504638671875f,0.56140000792816280306141152323107235133647918701171875f,0.305000001005828413891407535629696212708950042724609375f,0.08070000170096747194037334338645450770854949951171875f,-0.5806000185251232181826708256267011165618896484375f,-0.8000000645220293193915495066903531551361083984375f,0.21719993917942037597867965814657509326934814453125f,-0.29099998110532698802899176371283829212188720703125f,-1.12490000674724566209761178470216691493988037109375f,1.3614000482916832623914160649292171001434326171875f,-0.6382000692963600840812432579696178436279296875f,1.5422000432640317058030632324516773223876953125f,1.41970006699413087147831902257166802883148193359375f,0.000299974095821387010118996840901672840118408203125f,-0.3378999596372249669684606487862765789031982421875f,-1.067799995255469536203918323735706508159637451171875f,-0.3137999757647520482350955717265605926513671875f,0.55790004036426577016527517116628587245941162109375f,0.8596999826908113817580670001916587352752685546875f,0.363800007057189844772437936626374721527099609375f,0.249199974703788740271193091757595539093017578125f,-0.1503000417470932159602625688421539962291717529296875f,0.53180000185668507839409357984550297260284423828125f,-0.1062000085890293377133275498636066913604736328125f,-0.8857000228792435425617668443010188639163970947265625f,-0.1732000266790389986226728069595992565155029296875f,-1.0880000264644620333598368233651854097843170166015625f,1.6203000173091890445675744558684527873992919921875f,0.82520002439171147301522069028578698635101318359375f,0.22820002632513680662640354057657532393932342529296875f,0.102399974299967111424081167569966055452823638916015625f,-1.1358999849282211958012567265541292726993560791015625f,0.2329000160008665165634056393173523247241973876953125f,-0.313699974390864166906567334081046283245086669921875f,-0.477799996522068948223704865085892379283905029296875f,0.66789998418688778514251680462621152400970458984375f,-0.17579999825060355078676366247236728668212890625f,0.514300001186132504926717956550419330596923828125f,0.94139998838901561839520581997931003570556640625f,0.73779999607503388148188605555333197116851806640625f,-0.45289996019601819199351666611619293689727783203125f,-0.4298000067144631675120081126806326210498809814453125f,1.01609996869564067623059600009582936763763427734375f,-1.0165999671459193809397447694209404289722442626953125f,-0.1724000010848047992340070777572691440582275390625f,-0.987999990217387935587112224311567842960357666015625f,-1.3940999970972536292634913479560054838657379150390625f,0.196799983900785679935552252572961151599884033203125f,-0.28909996830076034601830770043306984007358551025390625f,0.9697999819785361186319505577557720243930816650390625f,0.7115999713182450392423561424948275089263916015625f,0.105199995967745874025922603323124349117279052734375f,0.2109999831914901591289890347979962825775146484375f,-0.0439999990016222142230617464520037174224853515625f,-0.320999995708465490906746708787977695465087890625f,-0.09410003345608775049413452506996691226959228515625f,-0.28710002698898395845361619649338535964488983154296875f,0.987299985778330668750868426286615431308746337890625f,1.474400018578767568300236234790645539760589599609375f,0.20559999535381745516815499286167323589324951171875f,-0.1626999957442283317732290015555918216705322265625f,0.280300025877356251413630161550827324390411376953125f,-2.00580010486841242567379595129750669002532958984375f,-0.868800007578731214863410059479065239429473876953125f,0.19350002372264951677749422742635942995548248291015625f,-0.06889996048659152716453490938874892890453338623046875f,0.08149999136477681371815151578630320727825164794921875f,-1.2305000388510531905694733723066747188568115234375f,-0.2358999988794321378549057044438086450099945068359375f,-0.43370000396966990052760593243874609470367431640625f,-0.96860004886388839651090165716595947742462158203125f,-1.10340002644658152775036796811036765575408935546875f,0.69930000021755633898834503270336426794528961181640625f,0.93000003926456020764135246281512081623077392578125f,0.2803000030040740941927879248396493494510650634765625f,-1.70530003861784962282399646937847137451171875f,0.55580000950098007450606019119732081890106201171875f,1.332799989816546126775165248545818030834197998046875f,-0.8996000008374454015580568011500872671604156494140625f,0.9841000318169601523976552925887517631053924560546875f,-1.0865999760493638692793183508911170065402984619140625f,-0.8307000279456382418885596052859909832477569580078125f,-0.52659999165833004663994643124169670045375823974609375f,-0.2807000142924482855022461080807261168956756591796875f,0.3499999958090482810035837246687151491641998291015625f,1.28620000769495934633823708281852304935455322265625f,0.10270001332759814971495870850048959255218505859375f,0.332800005388259823035923545830883085727691650390625f,-0.38639999569058414152777913841418921947479248046875f,0.1917000129997727952968489262275397777557373046875f,-0.096799991872906776535501194302923977375030517578125f,0.002700004014373202920751282363198697566986083984375f,0.6848000307053327961881450391956605017185211181640625f,-0.640599999898672223253015545196831226348876953125f,-1.2224000252246856046411949137109331786632537841796875f,-0.109699989023804622689794996404089033603668212890625f,1.045599973523616998960505952709354460239410400390625f,0.16530004413127874496325375730521045625209808349609375f,0.4397999937504533107812676462344825267791748046875f,0.26529997549280526225601306578028015792369842529296875f,-1.575399970813095240629309046198613941669464111328125f,0.72760000522956225221804515967960469424724578857421875f,-1.680600021654367282053499366156756877899169921875f,-0.37759997611641882286903637577779591083526611328125f,-0.1300000064074993133544921875f,0.34560000824332259838911340921185910701751708984375f,-0.180300003302097167079409700818359851837158203125f,-0.87790002725124427485070555121637880802154541015625f,-0.99270003255009608977843527100048959255218505859375f,-0.019500025838614210016430661198683083057403564453125f,-0.372800039511918868129214388318359851837158203125f,1.1263000283658508582362856031977571547031402587890625f,-0.2919999788254499240025552353472448885440826416015625f,-0.2007000182598834836511514367884956300258636474609375f,0.5046000219598416602906354455626569688320159912109375f,1.90020002077817817820459822542034089565277099609375f,0.988499926269053474925385671667754650115966796875f,-0.25919999169111296311029946082271635532379150390625f,0.35119996850490498019325968925841152667999267578125f,-0.4328000007689001282784602153697051107883453369140625f,0.4201999730944638855589801096357405185699462890625f,0.14240001002550162212401119177229702472686767578125f,-0.01490000182986239707361164619214832782745361328125f,0.176099994102120405159439542330801486968994140625f,0.948700032058358289077659719623625278472900390625f,0.01529996597468874508507497012033127248287200927734375f,0.313199993896484674138491755002178251743316650390625f,0.265399996033310880960698341368697583675384521484375f,-1.0614000225871802474131300186854787170886993408203125f,0.2393000187695026870215997405466623604297637939453125f,0.4841000700011850454984596581198275089263916015625f,0.4199999823048710112516346271149814128875732421875f,-1.482100018122792572938806188176386058330535888671875f,1.26089997846484092036689617088995873928070068359375f,0.7156000346928834687787457369267940521240234375f,0.782900002962351404534047105698846280574798583984375f,0.3285999907493593052976166291045956313610076904296875f,0.379599983924627348841340790386311709880828857421875f,-0.5669000132143497960868216978269629180431365966796875f,0.173099998408555943996134374174289405345916748046875f,-0.126600009763240706206488539464771747589111328125f,-1.5786000046819452791169169358909130096435546875f,0.8293999965250495876034619868732988834381103515625f,-0.618999986410141200821044549229554831981658935546875f,-0.41429997272491458915055773104541003704071044921875f,0.034600018569827195591415147646330296993255615234375f,-0.3892000089392066985993778871488757431507110595703125f,0.33420000286698281488639850067556835711002349853515625f,0.2837999890640381384088186678127385675907135009765625f,0.015200000736117491584309391328133642673492431640625f,-0.226699980962276725904303020797669887542724609375f,-0.52479998928010473946415004320442676544189453125f,0.27859997286796644999640193418599665164947509765625f,0.2188000231504438897189857016201131045818328857421875f,0.03570000715851762151942239142954349517822265625f,0.3958000050306320627413469992461614310741424560546875f,-0.08790000266432773656788413063623011112213134765625f,0.22089999962449002879338877392001450061798095703125f,-1.336900071775913634297694443375803530216217041015625f,-1.4806000007927426764808842563070356845855712890625f,1.90150003230571673640270091709680855274200439453125f,1.713099995800852592964247378404252231121063232421875f,0.998900030782818415531210121116600930690765380859375f,0.4945000195056206138133347849361598491668701171875f,2.144700040954350850341825207578949630260467529296875f,1.215399980610609276254763244651257991790771484375f,0.5424000423610220167347506503574550151824951171875f,0.517400053015351080176742470939643681049346923828125f,0.3694000380247839387948260991834104061126708984375f,0.37970003715455558079838738194666802883148193359375f,0.26449997472762998285134017351083457469940185546875f,0.60229998741745927048896191990934312343597412109375f,0.015099989676475189526172471232712268829345703125f,0.16440001485347721654761699028313159942626953125f,-1.0525999937742955836483815801329910755157470703125f,-0.529999956339597844134914339520037174224853515625f,-0.3957999952703714630075637614936567842960357666015625f,-1.0593000140756370086592141888104379177093505859375f,0.7677000038653611913019858548068441450595855712890625f,-0.785699981081485976375233803992159664630889892578125f,0.512699971306324275843735449598170816898345947265625f,-0.6275000349432229285184803302399814128875732421875f,0.7417000097215173948228539302363060414791107177734375f,-0.111000004649162153924635276780463755130767822265625f,-0.134099954032897705769045160195673815906047821044921875f,-0.034399984380602877109822657075710594654083251953125f,0.0299999903142451618265340584912337362766265869140625f,0.1929000160008669251254787013749592006206512451171875f,0.1271000114172695649727984346100129187107086181640625f,-0.3221000035941601336020312373875640332698822021484375f,-0.541200006800889976688040405861102044582366943359375f,-0.40749995008111028482744586654007434844970703125f,1.1417999859869478740392878535203635692596435546875f,0.80639998570680671008403805899433791637420654296875f,1.01100002230703811534340275102294981479644775390625f,-0.99710000098645679145192843861877918243408203125f,-0.039199977385998341361528218840248882770538330078125f,0.287899961984158014871582054183818399906158447265625f,-0.19999998249113559722900390625f,-1.29609997525215181468638547812588512897491455078125f,0.598199997472763644879023559042252600193023681640625f,-0.287000014737248676510716904886066913604736328125f,-0.509200010392069746245624628500081598758697509765625f,0.418399977719783766616501452517695724964141845703125f,0.6845999850422141985717416901024989783763885498046875f,-0.9016999747037885981626459397375583648681640625f,0.4567999958217143774419355395366437733173370361328125f,0.2960999913454056553518967120908200740814208984375f,0.190799993109703081017869408242404460906982421875f,0.388599998199939733467545011080801486968994140625f,0.2161999845609070547425289987586438655853271484375f,0.408499990046024219481068939785473048686981201171875f,0.098899969501793361104091673041693866252899169921875f,-1.2382999364852906865053228102624416351318359375f,-0.0506999920338397913610606337897479534149169921875f,0.183100006008147797587071181624196469783782958984375f,0.0244999908208842764878454545396380126476287841796875f,-0.3851999847173690483259633765555918216705322265625f,1.736999963589012541120837340713478624820709228515625f,0.5629999635815625680379525874741375446319580078125f,-0.9677999674648047889746749206096865236759185791015625f,1.183899985408782384865844505839049816131591796875f,1.0529999903291464846688541001640260219573974609375f,-0.18719999282360078041165252216160297393798828125f,-0.14090001012980923889017503825016319751739501953125f,0.1437999718904503509975256747566163539886474609375f,-1.3340999392807491030765731920837424695491790771484375f,1.448299944978952424889939720742404460906982421875f,-0.675499987125396383902398156351409852504730224609375f,0.837399995571374677894027627189643681049346923828125f,-0.6027000200331202872661151559441350400447845458984375f,0.374200039523838512423026259057223796844482421875f,0.575599980676174904914432772784493863582611083984375f,1.136099977487325052294409033493138849735260009765625f,-0.88699996891617782779348999611102044582366943359375f,1.0018999507784851488167987554334104061126708984375f,0.9874000031709666558299431926570832729339599609375f,0.7756999745249739675045930198393762111663818359375f,-0.7374999728798865561429920489899814128875732421875f,-0.098799983811378666587188490666449069976806640625f,-0.0472000127911567091132383211515843868255615234375f,-0.357699990677833312702205148525536060333251953125f,-0.97080000108182396001410552344168536365032196044921875f,-0.7842999953746794261633112910203635692596435546875f,0.0313999813854690668080138493678532540798187255859375f,0.69749998364597498579087186953984200954437255859375f,0.182100014024973055892786533149774186313152313232421875f,-0.3677000051319598838972524390555918216705322265625f,-0.6487999993830915190784480728325434029102325439453125f,-1.58669998617768204240974228014238178730010986328125f,0.1108999766767022610025605899863876402378082275390625f,1.6515000160634514259072602726519107818603515625f,-0.2171999875709413885260801180265843868255615234375f,0.5662000240862370414873794288723729550838470458984375f,-0.2211000017762183578184931320720352232456207275390625f,0.3295000025182959202396659748046658933162689208984375f,-0.36059999542832354979537967665237374603748321533203125f,-0.38480001200437552455468903644941747188568115234375f,-0.19510000793039805611073234103969298303127288818359375f,-0.0780000020265578797307171043939888477325439453125f,0.59299998116493224387113514239899814128875732421875f,-0.4661999811708927854425610348698683083057403564453125f,-0.000599971623719053326340144849382340908050537109375f,-0.48989997683316488519267295487225055694580078125f,-0.464300004315375769436968766967765986919403076171875f,-0.5232000049605967095800451716058887541294097900390625f,1.69009993633031907478425637236796319484710693359375f,0.794200035090744105303883770829997956752777099609375f,-0.3458999729514129484186923946253955364227294921875f,0.3752999998748303145390536883496679365634918212890625f,-1.114199985879659404730546157225035130977630615234375f,-1.669699981126189225477673971909098327159881591796875f,-0.7520999854147432106543647023499943315982818603515625f,0.3099999710917471507087839199812151491641998291015625f,-0.17719999759197246902431288617663085460662841796875f,-0.10869998072385811394013899189303629100322723388671875f,-0.9393999764084810522035695612430572509765625f,1.534699974942207223449486264144070446491241455078125f,-1.12389997348785453112895993399433791637420654296875f,-1.0767999977961177915375401425990276038646697998046875f,-0.3982999665856363780136462082737125456333160400390625f,-0.13120000516176222316033772585797123610973358154296875f,-0.3119999860897657573133301411871798336505889892578125f,1.505499985188245570810749995871447026729583740234375f,-0.87199997577071286514183157123625278472900390625f,-0.75069998525381098630759879597462713718414306640625f,-1.1831999500870704622457196819595992565155029296875f,0.41599997074902061999779334655613638460636138916015625f,-0.55989997649788847411400638520717620849609375f,-0.3313000052690504571017982016201131045818328857421875f,0.3327999985337257715656278378446586430072784423828125f,0.7463999940514565167859473149292171001434326171875f,0.11389999703168829370270032086409628391265869140625f,0.54719999118447304464751823616097681224346160888671875f,-0.1710000272244214247763238745392300188541412353515625f,0.3384000085651874822900708750239573419094085693359375f,0.17299998749792511176792686455883085727691650390625f,0.185599992746114883512831283951527439057826995849609375f,-0.5580000248253345063176311668939888477325439453125f,0.8286999976366751230472118550096638500690460205078125f,0.401799979579447352762144873850047588348388671875f,1.0247000036641960019068164911004714667797088623046875f,0.3930999477446077694509085631580092012882232666015625f,-0.5704999625012281416758241903153248131275177001953125f,-0.478799964290857804627421501209028065204620361328125f,0.40939999373108137969978770343004725873470306396484375f,0.482899985565244715957078369683586061000823974609375f,1.07770000516921271582759800367057323455810546875f,0.89439999466240382997028746103751473128795623779296875f,0.39590001176893718781713005228084512054920196533203125f,0.3872000031799076591454422668903134763240814208984375f,0.60759998297095296759806615227716974914073944091796875f,-1.003199972513318272149263066239655017852783203125f,0.166900024390220114156591080245561897754669189453125f,1.277699950332940037611706429743207991123199462890625f,1.585699985849856830100179649889469146728515625f,0.293600052998959970551595688448287546634674072265625f,1.4533999795079228789518310804851353168487548828125f,-0.56500000882893797182759954011999070644378662109375f,-0.4639999466240407599570971797220408916473388671875f,-1.0832999478474263366223340199212543666362762451171875f,0.3411000271081920942606302560307085514068603515625f,1.8378999822124837404402342144749127328395843505859375f,1.008799977105855560921554570086300373077392578125f,0.0451000249922273166447439507464878261089324951171875f,-0.01499999336898338864187962826690636575222015380859375f,-0.472599993029236831176831401535309851169586181640625f,0.071199999983608375231369791436009109020233154296875f,-0.54809998648762692230462789666489697992801666259765625f,-0.25799996995180884962195477783097885549068450927734375f,-0.8239999752342701544449710127082653343677520751953125f,0.47539996600747169708256478770636022090911865234375f,1.15319998868107820300110688549466431140899658203125f,-0.2065000033974655480051296763122081756591796875f,-0.652300023329257072646214510314166545867919921875f,-0.7699999645352360033712102449499070644378662109375f,-1.38949998754262882272314527654089033603668212890625f,-0.80909993615150543888603351661004126071929931640625f,0.333899976766109940484739126986823976039886474609375f,0.79619996865391673424028340377844870090484619140625f,0.124199993181229151328892612582421861588954925537109375f,0.489000003211199629760130846989341080188751220703125f,-0.86239999512433984651949003819026984274387359619140625f,-0.02569998208731416600159036534023471176624298095703125f,0.31199999029934433192323695038794539868831634521484375f,-0.715300051730871633282049515401013195514678955078125f,-0.53910001494139392352877848679781891405582427978515625f,0.0250999916136265266430882547865621745586395263671875f,0.415799954962730478058574590249918401241302490234375f,-0.497600060456991233337475932785309851169586181640625f,0.32349999283999186783233881214982829988002777099609375f,-0.6148000304818150851815516944043338298797607421875f,1.0424000124096863828526693396270275115966796875f,-0.471900009936094821938468157895840704441070556640625f,-0.35579999337047263363587035200907848775386810302734375f,-0.85070002429485302997136386693455278873443603515625f,0.45299996171891698804046200166339986026287078857421875f,-0.20750004887580875134744928800500929355621337890625f,0.5811000058367847831419794601970352232456207275390625f,-0.44200002613663702533131072414107620716094970703125f,0.0469000018149614561480120755732059478759765625f,-0.2418000132560731429975930950604379177093505859375f,0.54069997512102130343691896996460855007171630859375f,-0.09850000528246172226687349393614567816257476806640625f,0.146600016468763527655028156004846096038818359375f,0.7516999975770712882905399965238757431507110595703125f,1.02299996984004959443836924037896096706390380859375f,-0.701999992460012212092124173068441450595855712890625f,0.97370003585815378954748666728846728801727294921875f,-0.1619999753981826717819103578221984207630157470703125f,0.80989997247457523599223350174725055694580078125f,1.2185999562159184339549256037571467459201812744140625f,0.25679998077154220226248071412555873394012451171875f,-0.5411000282257794058438093998120166361331939697265625f,0.63449999886751218269864693866111338138580322265625f,1.0503999936491246192105109003023244440555572509765625f,0.44359999551773032777646221802569925785064697265625f,0.639700006234645623948154025129042565822601318359375f,0.30909999575614932609823881648480892181396484375f,0.18210001640915862708425265736877918243408203125f,-0.76239998588561963543952515465207397937774658203125f,-0.22519998322725331973970241961069405078887939453125f,0.21669995384216367284579973784275352954864501953125f,0.2214000219166276650639701983891427516937255859375f,-0.29350000643730123073282811674289405345916748046875f,-1.3235999689936637224718651850707828998565673828125f,0.77569997318386985085680862539447844028472900390625f,-0.705099990421533373563534041750244796276092529296875f,0.0958999759316458977309594047255814075469970703125f,-0.00799998727440875967431566095910966396331787109375f,0.44269995998144207050017939764074981212615966796875f,-1.329299971681833003600559095502831041812896728515625f,1.4000999787986270206374683766625821590423583984375f,0.10000001341104525209857456502504646778106689453125f,0.49269999350905369084330231999047100543975830078125f,0.20999998636543770746953896377817727625370025634765625f,0.72890000299215340628933290645363740622997283935546875f,0.5620000084973868492710380451171658933162689208984375f,0.2327000127315519417692257775343023240566253662109375f,0.3097000017642976832377144091879017651081085205078125f,0.5127999884754419479548914750921539962291717529296875f,-0.224099985884130159519145308877341449260711669921875f,-0.6860999992430210081550967515795491635799407958984375f,-1.061499975234270198853891997714526951313018798828125f,-0.41769996664971120736709053744561970233917236328125f,-0.39500000797212120318135930574499070644378662109375f,-0.3610000161789359562902745892642997205257415771484375f,0.47929998129308248877578080282546579837799072265625f,0.60119997774362599063380230290931649506092071533203125f,-0.458099973504990654049606746411882340908050537109375f,-1.007999978780746541673352112411521375179290771484375f,-0.434400003789365218853646410934743471443653106689453125f,-0.4393999979406595368658372535719536244869232177734375f,-0.20839996989667408655577673926018178462982177734375f,1.1508999826557932966153430243139155209064483642578125f,0.17229997341036806801639613695442676544189453125f,0.5032000026136633774598294621682725846767425537109375f,-0.77139998287558597667157300747931003570556640625f,-1.0009999974593519755927673031692393124103546142578125f,-0.6933999882996080277308692529913969337940216064453125f,-0.54350000490993266577532949668238870799541473388671875f,-0.2079999834001065028843413529102690517902374267578125f,1.153600000360607946703339621308259665966033935546875f,0.5100999647356569965950257028453052043914794921875f,-0.93330000488162034866235217123175971210002899169921875f,-0.85509995279610162555172792053781449794769287109375f,0.11730000162944198560666109187877736985683441162109375f,-0.040600010180473322662919599679298698902130126953125f,0.3992999844223261707298888723016716539859771728515625f,-0.12170000212192511046538356822566129267215728759765625f,-1.37039997539520275182667319313623011112213134765625f,1.209699975370615021574849379248917102813720703125f,0.5027999839305881124573716078884899616241455078125f,0.69269999455213504546691183350048959255218505859375f,-0.26169998528361337886138926478452049195766448974609375f,-0.04679998412430297349828833830542862415313720703125f,0.2901999817371370138374686575843952596187591552734375f,-0.4596999718874694718095952339353971183300018310546875f,0.65729998104721321094956465458380989730358123779296875f,-0.0147000070914629077378776855766773223876953125f,0.40570000009909268356977918301708996295928955078125f,0.00369999409765042752695762828807346522808074951171875f,-1.4716999660611154165934522097813896834850311279296875f,0.34160000262930967362962064726161770522594451904296875f,1.149599976921081623260079140891321003437042236328125f,0.138199982124567188890296165482141077518463134765625f,0.71400000406801655827848662738688290119171142578125f,0.63329998910501539288731009946786798536777496337890625f,0.43490001799762267875593124699662439525127410888671875f,-0.26920000445023173352154799431446008384227752685546875f,-0.085800000206381010769973727292381227016448974609375f,0.4677000120609999811449597473256289958953857421875f,0.15049999777972689063432198963710106909275054931640625f,-0.6078000217050314457623017005971632897853851318359375f,0.7165999959424136367402979885810054838657379150390625f,0.353500026181340576414413590100593864917755126953125f,0.23740000436306007980391541423159651458263397216796875f,1.242200007799266980867969323298893868923187255859375f,-0.4608999929562209718625354071264155209064483642578125f,0.52840001392960544990273774601519107818603515625f,0.43629997699409683864502085270942188799381256103515625f,1.32100002253055581746821189881302416324615478515625f,0.43910001408457766114423748149420134723186492919921875f,-0.06659998860359195305136381648480892181396484375f,0.37110002770423855533721280153258703649044036865234375f,0.282100013726949594428106138366274535655975341796875f,0.04289999163746849308864739214186556637287139892578125f,-0.17440000745877615173640151624567806720733642578125f,-0.417099975132942279998360390891321003437042236328125f,0.83979998775273567712673639107379131019115447998046875f,0.719399975961446447314528995775617659091949462890625f,-0.5549000030219553014632083431934006512165069580078125f,0.946999971151352593068395435693673789501190185546875f,1.4982000152051446750789409634307958185672760009765625f,-0.800699989724159788551105521037243306636810302734375f,-0.688500017352402071679762229905463755130767822265625f,0.2291000300288192903508388553746044635772705078125f,-0.28019992053061726355878136018873192369937896728515625f,-0.36160000290870630834660914842970669269561767578125f,-1.09270001161396468347675181576050817966461181640625f,-0.145299991083145840775614487938582897186279296875f,-1.413999994214623523447471598046831786632537841796875f,0.138200010753422919407285007764585316181182861328125f,-0.01359999849796300619431121958768926560878753662109375f,0.04019999505504989745219290853128768503665924072265625f,-0.4954999880939723322370582536677829921245574951171875f,0.75359996951520447083794351783581078052520751953125f,0.399600007766485187943317214376293122768402099609375f,0.689399971565604285927975070080719888210296630859375f,-0.2263000123471019531251613443600945174694061279296875f,-0.24289998500645193502123220241628587245941162109375f,0.507799980801344030822974673355929553508758544921875f,-1.310299941983819049795556566095910966396331787109375f,1.104699980604648690274416367174126207828521728515625f,-0.9142999938845637331041871220804750919342041015625f,-0.88720000981092450587084385915659368038177490234375f,-0.24149998551607065877533386810682713985443115234375f,-0.2280000013187526430868956595077179372310638427734375f,-0.709400004944205075929630766040645539760589599609375f,0.5327999674275518682264873859821818768978118896484375f,-0.005900016909837546563721843995153903961181640625f,1.1523999991476532311907021721708588302135467529296875f,0.279199952761828018577006105260807089507579803466796875f,1.3521000138759611264305249278550036251544952392578125f,0.6879000050485133410660409936099313199520111083984375f,-0.0601999837487932898394404901409870944917201995849609375f,-0.727500007003545778871966831502504646778106689453125f,0.62929999667853053768595827932585962116718292236328125f,0.25139997825622462102046483778394758701324462890625f,0.6588000038906935973415102125727571547031402587890625f,0.741499985106289560832237839349545538425445556640625f,0.09950000449270006430690926890747505240142345428466796875f,-0.81250000216066819991311831472557969391345977783203125f,-0.169100003989040903462637288612313568592071533203125f,-0.415800005775690095788377220742404460906982421875f,1.476399971401691768591035724966786801815032958984375f,0.2191000495493409250258309839409776031970977783203125f,1.947699956703186074946643202565610408782958984375f,0.28140000463128078678209931240417063236236572265625f,1.431000018991528843770311141270212829113006591796875f,-0.459099970349669650460100456257350742816925048828125f,0.11010001009106673564019729383289813995361328125f,0.7694000093027948405932647801819257438182830810546875f,0.99049998652934956311355563229881227016448974609375f,-0.03020000300854430719255105941556394100189208984375f,1.21999999165534944722821819595992565155029296875f,-0.08339997999221093749611100065521895885467529296875f,-0.72759996091723433409015342476777732372283935546875f,-0.4769999996870755154532162123359739780426025390625f,0.34359998098909871888650968685396946966648101806640625f,0.7236999827727677025990260517573915421962738037109375f,-0.8038999747544526286446853191591799259185791015625f,0.318599995517730771865672068088315427303314208984375f,-0.834199988934397840267820356530137360095977783203125f,-0.39929998703002933524430773104541003704071044921875f,-0.625999964714051060354904620908200740814208984375f,-1.328199977765976758092847376246936619281768798828125f,0.0323000242233268952674052343354560434818267822265625f,-1.27319999791979743264391800039447844028472900390625f,0.0312000089243056333287995585124008357524871826171875f,1.063599999094009262989857234060764312744140625f,-0.13760000129938187996003762236796319484710693359375f,-0.61880001137852591597265927703119814395904541015625f,-0.4972999786257741305917079444043338298797607421875f,-1.520899972951412326693798604537732899188995361328125f,0.397799988661706738213297285255976021289825439453125f,0.55840000066757189411958961500204168260097503662109375f,0.41429997246414440592587880018982104957103729248046875f,-0.4841999827131626688014875981025397777557373046875f,-1.51020002308785894484799428028054535388946533203125f,-0.8457000009372832582954515601159073412418365478515625f,-0.449599982173740475577261577200260944664478302001953125f,-1.0085999756991863574029366645845584571361541748046875f,-1.6707000033587220411135376707534305751323699951171875f,-0.4969000069931148733104464554344303905963897705078125f,0.343200006189942730117081737262196838855743408203125f,-0.30489998748749513879374717362225055694580078125f,-0.207899978747963842806711909361183643341064453125f,-0.78020000822395019213217892684042453765869140625f,-0.3249999578297126934245397933409549295902252197265625f,-0.76420002715587609376513000825070776045322418212890625f,-0.26200002628564877937833443866111338138580322265625f,0.7847999672636394308256058138795197010040283203125f,-0.2515000674724578999530422152020037174224853515625f,0.3144999961853025549629592205747030675411224365234375f,-0.6302000224918129500650820773444138467311859130859375f,0.17439999585449650343349503600620664656162261962890625f,0.3324000149428842920684701311984099447727203369140625f,-0.659599982546269725247611859231255948543548583984375f,-0.53029999104589220504379909471026621758937835693359375f,0.683399981445073745334184422972612082958221435546875f,-0.2534000129982827420604962753714062273502349853515625f,-0.519499952971935297085792626603506505489349365234375f,0.499000013902783479124991572462022304534912109375f,-1.211300011527537634492546203546226024627685546875f,-0.39400004955381129345681756603880785405635833740234375f,-0.3168000455915913615712042883387766778469085693359375f,1.378500000774860012597855529747903347015380859375f,0.020399954235553963144411682151257991790771484375f,-1.87750002902001167370826806291006505489349365234375f,1.0320000051259992090990635915659368038177490234375f,1.3555000244155521738065317549626342952251434326171875f,0.542300011520087554828251086291857063770294189453125f,1.250100007408857294421977712772786617279052734375f,-0.21349999999254987148589179923874326050281524658203125f,0.075599988014996011287394139799289405345916748046875f,-0.7992999646782876776995863110641948878765106201171875f,1.0747999883860348546704699401743710041046142578125f,-0.8875999885216352414119000968639738857746124267578125f,-1.122999981760978815970020150416530668735504150390625f,0.089799994942545868781280660186894237995147705078125f,0.02469999476075146471742982612340711057186126708984375f,0.47349997089803341765446020872332155704498291015625f,1.3052999810993666240932498112670145928859710693359375f,-0.1866000016421083262940783242811448872089385986328125f,0.19290000519752492635916496510617434978485107421875f,0.352100010895728843252072692848742008209228515625f,-0.9743999498844146955889300443232059478759765625f,0.5638999901771537537342737778089940547943115234375f,-0.2196999955803151660660432753502391278743743896484375f,0.26429998159110557054418677580542862415313720703125f,-0.433400011694431430697704854537732899188995361328125f,0.11770000073611708302223632927052676677703857421875f,-0.0111999928683046423572022831649519503116607666015625f,-0.93719999148249666376386812771670520305633544921875f,0.76830001434385764014223241247236728668212890625f,0.469900003916025266192946219234727323055267333984375f,-0.0153000152975319991810465580783784389495849609375f,0.3957999827533964509740371795487590134143829345703125f,0.1046000564932825227515422739088535308837890625f,0.548500024467706293052060573245398700237274169921875f,-0.815700039494037465459541635937057435512542724609375f,0.4052000268876554400776512920856475830078125f,0.834199973288178409092097354005090892314910888671875f,-0.3100000153481961007173595135100185871124267578125f,0.34509999042153349790851279976777732372283935546875f,0.34239998648166647399193607270717620849609375f,-1.2434000025302172076635542907752096652984619140625f,0.04180001161694502087584623950533568859100341796875f,-0.660400018236040775576611849828623235225677490234375f,0.49000001624226552365826137247495353221893310546875f,-0.2672000302255153059149961336515843868255615234375f,-0.86540000117421111980320347356610000133514404296875f,-2.04080000860691068709229512023739516735076904296875f,0.4871999989330770830520123126916587352752685546875f,-0.0977999963730569188413710435270331799983978271484375f,-1.22309997322559382837425800971686840057373046875f,-1.531000007778406502012558121350593864917755126953125f,1.479299956035614371785413823090493679046630859375f,-0.197799974542856915604716050438582897186279296875f,-0.016499973297118941673033987171947956085205078125f,0.1262000157415865686516553978435695171356201171875f,-0.272399997210502675670795724727213382720947265625f,0.5644000130653381575029925443232059478759765625f,-0.465199980247020139501046287477947771549224853515625f,0.74660001065731051994589506648480892181396484375f,0.4337999967008825397130067358375526964664459228515625f,0.0263999499812717086655311504728160798549652099609375f,0.9076000464871529782584502754616551101207733154296875f,0.135699998553097334053063605097122490406036376953125f,-1.1424999853968618168664761469699442386627197265625f,0.2547000037014495177345452248118817806243896484375f,0.34049996238946800275471105123870074748992919921875f,1.156699954438209942964022047817707061767578125f,0.60210002281665830903278902042075060307979583740234375f,0.4825000338256357768074167324812151491641998291015625f,0.864000028729439151220503845252096652984619140625f,0.470700006189942232737166705192066729068756103515625f,-0.249900004065036540623623295687139034271240234375f,-0.0990999742984772780118873924948275089263916015625f,0.356000024914741697301678868825547397136688232421875f,0.386000015377998106913537412765435874462127685546875f,-0.069300018769502980120478241587989032268524169921875f,-0.8096000031471251023873492158600129187107086181640625f,0.6739000015020371048279912429279647767543792724609375f,-0.77160001550018797455976482524420134723186492919921875f,0.76770000993758469309824477022630162537097930908203125f,1.27550003845989667894400554359890520572662353515625f,0.3056999649137261965137213337584398686885833740234375f,-0.546099997007846393870522661018185317516326904296875f,0.40259999526441081485472750500775873661041259765625f,-1.012800013732910286989863379858434200286865234375f,-0.34550000433623784257264333064085803925991058349609375f,-0.254600023785233664508353967903531156480312347412109375f,0.04080002164542662068669187647174112498760223388671875f,0.2614000208735467456477863379404880106449127197265625f,-0.4428999931275843238154266146011650562286376953125f,-0.410600006902217984361413982696831226348876953125f,0.212599974477291286945046522305347025394439697265625f,0.6597999893546102612162940204143524169921875f,0.2972000031054020663390247136703692376613616943359375f,-0.173399999214708611106061653117649257183074951171875f,-0.06359997383654103497718779181013815104961395263671875f,-0.0968000215262174723562793587916530668735504150390625f,-0.41900000034272721904926584102213382720947265625f,0.69709997274875679096339808893389999866485595703125f,1.932599998021125475844428365235216915607452392578125f,-1.04799996149539964562791283242404460906982421875f,0.72060000094175347129521469469182193279266357421875f,0.2453999738305807643001799078774638473987579345703125f,0.125799986851215683802962530535296536982059478759765625f,-0.70209998861849332030260484316386282444000244140625f,-0.0510000201463698399351187617867253720760345458984375f,0.2609999912381173459152705618180334568023681640625f,-0.1746000064998864953480506301275454461574554443359375f,-0.5122000165909532665153847119654528796672821044921875f,-1.0904000416681167706656196969561278820037841796875f,0.802400012037157939204234935459680855274200439453125f,0.72479996394813017701608259812928736209869384765625f,-2.43030006216168370514196794829331338405609130859375f,-0.966699989902974010647085378877818584442138671875f,-1.345700031447410793816743534989655017852783203125f,-0.8078000980734838520902485470287501811981201171875f,1.31160003673434299997779817203991115093231201171875f,-0.01860004215836585927945634466595947742462158203125f,0.675600011819600609896951937116682529449462890625f,1.409599993461370104341767728328704833984375f,-2.4413000906527049238547988352365791797637939453125f,-1.6317000157564882645289117135689593851566314697265625f,-0.2710000240951779826303891240968368947505950927734375f,-0.4775000261887909491775872083962894976139068603515625f,-0.3398000099927187545034712456981651484966278076171875f,-0.542799986724555427741734092705883085727691650390625f,0.517899972005188669044173366273753345012664794921875f,0.307699970486760399079884109596605412662029266357421875f,0.441500020906328671799201401881873607635498046875f,1.41550005738437167934762328513897955417633056640625f,-0.91640000589787984353051797370426356792449951171875f,-0.0411000239044430060175727703608572483062744140625f,0.35819999508857680581286331289447844028472900390625f,-1.747599984088540114868237651535309851169586181640625f,1.61560001360774041501144893118180334568023681640625f,1.281699993702769102554839264485053718090057373046875f,-0.27500000007450609018633258529007434844970703125f,0.9888000304520130345764528101426549255847930908203125f,-1.3183000124067068004052316609886474907398223876953125f,-0.458900006158649897969326048041693866252899169921875f,-0.032500004880130273221539027872495353221893310546875f,0.3296000025138259292845077652600593864917755126953125f,-0.35139998086392909559805275421240366995334625244140625f,0.85989998055845529023599738138727843761444091796875f,-0.441000003233551540748891284238197840750217437744140625f,-0.97209995427131712375512506696395576000213623046875f,-0.08240004914104925859419381595216691493988037109375f,1.233299996984005275635354337282478809356689453125f,0.16539998873174255322737735696136951446533203125f,0.8749000171780583201552872196771204471588134765625f,0.357400003096461471585598701494745910167694091796875f,-0.2587000237882133291122954688034951686859130859375f,-0.959200033786892891640718517010100185871124267578125f,-0.93239996860027307690188536071218550205230712890625f,0.11939995793104163368525405530817806720733642578125f,1.0353000104919078960818978885072283446788787841796875f,0.38259999943673606015437371752341277897357940673828125f,0.0825000043585897291364972261362709105014801025390625f,-0.38000001102685931897440241300500929355621337890625f,-0.30999998345971047086777616641484200954437255859375f,-0.4419000309467318032830007723532617092132568359375f,0.885900062805415444699974614195525646209716796875f,1.21480002556443178463041476788930594921112060546875f,-1.62060002463459884580743164406158030033111572265625f,-1.587300010663271532251883400022052228450775146484375f,0.65700001436471922033888404257595539093017578125f,-0.4131000432610516526210631127469241619110107421875f,-0.404300014001131469143501817598007619380950927734375f,-0.7944000521063809827637669513933360576629638671875f,0.67550007966160929839816162711940705776214599609375f,0.08460001626014701514577609486877918243408203125f,-0.02619997506141569232340771122835576534271240234375f,-0.963600015485286842675805019098334014415740966796875f,-0.633100032830238745873430161736905574798583984375f,-0.65590001527070995734902680851519107818603515625f,0.049699997740983992144947478664107620716094970703125f,-0.1949999962002035847064007612061686813831329345703125f,0.8677000224918127724293981373193673789501190185546875f,0.333699971634149772370392383891157805919647216796875f,0.17240003103613854573694652572157792747020721435546875f,0.1449000162839890748500693007372319698333740234375f,-0.019499992311001079769994248636066913604736328125f,-0.17759999697804451646021561828092671930789947509765625f,0.9168999735027549835564286695444025099277496337890625f,0.317799982216954379765638805110938847064971923828125f,0.9052000294953586045920701508293859660625457763671875f,-0.102700043874979218827547811088152229785919189453125f,0.293000005304813182505085933371447026729583740234375f,0.447299994271993295313905036891810595989227294921875f,0.2697000183790922989146565669216215610504150390625f,0.60780002624988582748954968337784521281719207763671875f,-0.1752999994277952833243716668221168220043182373046875f,0.32319999270439137006860619294457137584686279296875f,0.1426000131458049491328665681066922843456268310546875f,0.3699999958276745104512883699499070644378662109375f,-0.522400002127885176861354921129532158374786376953125f,-0.6913999977767470994649556814692914485931396484375f,-0.16220000325441308319796007708646357059478759765625f,1.300299992871284171513934779795818030834197998046875f,-2.03159998189806945134705529198981821537017822265625f,0.28569999449253113965596639900468289852142333984375f,-0.217599950560927180021053573000244796276092529296875f,-0.238799984407425114341094740666449069976806640625f,-1.447999990254639879339038088801316916942596435546875f,0.3981999840617174157841873238794505596160888671875f,-1.402800025504827186040301967295818030834197998046875f,-0.225199997234344362340152656543068587779998779296875f,-0.42860001548528681158956032959395088255405426025390625f,0.16740001062154785671509671374224126338958740234375f,-0.54570001414343705103959791813394986093044281005859375f,0.90510001964643616023664662861847318708896636962890625f,0.4380000027157369135721864950028248131275177001953125f,-1.509400057619810553433126187883317470550537109375f,-0.93939998251795753247961329179815948009490966796875f,0.26889997929930642417417629985720850527286529541015625f,-0.315200000959634518693519567023031413555145263671875f,1.080800036025047372589824590249918401241302490234375f,0.284199997726082731475116816000081598758697509765625f,0.8278000204384323712503146452945657074451446533203125f,1.41600001396238805995153597905300557613372802734375f,-1.006200017417967274013790301978588104248046875f,-0.97450001043453815530170913916663266718387603759765625f,1.536900030779093651034372669528238475322723388671875f,0.2479999992512167406033540828502736985683441162109375f,0.01650000220537162221035032416693866252899169921875f,0.27100001820921892914384443429298698902130126953125f,0.25759998922944082888619732329971157014369964599609375f,0.2138999813482163059319418607628904283046722412109375f,-0.0355999957449735504599885871357400901615619659423828125f,0.949900018519163058527965404209680855274200439453125f,-1.29020000207722151941425181576050817966461181640625f,-0.81029999398887131434321418055333197116851806640625f,-0.11000001132488257393760022750939242541790008544921875f,-0.8047999984443190424343583799782209098339080810546875f,0.8281000031024217111763618959230370819568634033203125f,0.5886000055760145510674874458345584571361541748046875f,0.05150000537186871785166886184015311300754547119140625f,0.08459999402016415981364616527571342885494232177734375f,-0.39810001506060366249784010506118647754192352294921875f,-1.005600004480779130489054296049289405345916748046875f,0.88370001732483505207937923842109739780426025390625f,0.593900005972385347519093556911684572696685791015625f,-0.24220000087022786061652368516661226749420166015625f,0.674199984296411258810621802695095539093017578125f,-0.6724000201210380556204881941084749996662139892578125f,-1.08199997781962142795464387745596468448638916015625f,0.203799970251321571623748241108842194080352783203125f,0.3707000056684019995145717984996736049652099609375f,0.413400002232193752860212043742649257183074951171875f,0.134700002434849697152685621404089033603668212890625f,0.03480001431405543854680217918939888477325439453125f,-0.119400016120076113423920105560682713985443115234375f,-0.05100000800192372452102063107304275035858154296875f,-0.85139999181628212010508605089853517711162567138671875f,-0.3845999945417044596496225494774989783763885498046875f,1.1068999916262922678100721896043978631496429443359375f,-0.011999988026917041139540742733515799045562744140625f,-0.97939999820142975561765297243255190551280975341796875f,-0.392399998253583959240131662227213382720947265625f,0.09689999980330465945144169381819665431976318359375f,-0.716600038969517072473536245524883270263671875f,0.20899998261034424817950139185995794832706451416015625f,-0.222900058133900902390678311348892748355865478515625f,0.036300008547306195083592683658935129642486572265625f,-0.24700000099092722649629649822600185871124267578125f,0.645900007522106189838950740522705018520355224609375f,0.5495000200271606960455983426072634756565093994140625f,-1.3809000444769861104532537865452468395233154296875f,-1.48480000216960927872378306346945464611053466796875f,-0.979600013428926441605426589376293122768402099609375f,0.0365999957561495836699805295211263000965118408203125f,-0.17000002756714849283525836654007434844970703125f,0.80809998291134821357672990416176617145538330078125f,-0.847300038826465762298312256461940705776214599609375f,0.8429000332117073668314333190210163593292236328125f,-0.1877000091552734062361196265555918216705322265625f,-0.3903000056862833133664025808684527873992919921875f,-0.051100029492378684636832986143417656421661376953125f,-0.32480000481456505756483466029749251902103424072265625f,0.61720000580623757269194129548850469291210174560546875f,-0.11320000563114884783999514183960855007171630859375f,0.1705000006295740722084275375891593284904956817626953125f,0.2486999892920256360895336911198683083057403564453125f,-0.269299987626075765234645587042905390262603759765625f,0.556300011825561657730077058658935129642486572265625f,-0.75440002029240160741352383411140181124210357666015625f,0.9004000116050239199694260605610907077789306640625f,0.0297000060111287655928435924579389393329620361328125f,-0.5740000052601097468851776284282095730304718017578125f,-0.38979999635815676395367290751892141997814178466796875f,0.4761000434249644808204493529046885669231414794921875f,-0.860699997547268669251252504182048141956329345703125f,-0.8492000335633758112408031593076884746551513671875f,-0.1987000059068204649292965768836438655853271484375f,1.034200008380413038366896216757595539093017578125f,-0.124399991719424729552656572195701301097869873046875f,-0.354899989610910127080245501929312013089656829833984375f,-0.232999988168477945027490250140544958412647247314453125f,0.16919999411255115528973647087696008384227752685546875f,-0.2042000075608492171141961080138571560382843017578125f,-0.8238999842166900844375732049229554831981658935546875f,-0.9496000059783458180362458733725361526012420654296875f,1.245899999773502297983895914512686431407928466796875f,0.573799995583295530110490290098823606967926025390625f,-0.4030000014305110678236587773426435887813568115234375f,2.096799961772561449180329873342998325824737548828125f,-0.232599968963862036019918377860449254512786865234375f,1.0441999725431203582814987385063432157039642333984375f,0.2835999898552898912384989671409130096435546875f,0.75860001503825191804253336158581078052520751953125f,0.1739999962449072512526981881819665431976318359375f,-0.3113000307500366492519106031977571547031402587890625f,-0.604499978899955436872915015555918216705322265625f,0.25159997981190684868124662898480892181396484375f,-1.50000002145767208361348821199499070644378662109375f,-0.533100062036513833163553499616682529449462890625f,-0.79970005488693818307410765555687248706817626953125f,-0.9971001037672169164949309561052359640598297119140625f,0.2672000035151838037705829265178181231021881103515625f,-0.065900023950636654745949272182770073413848876953125f,-1.1265000373721125725978708942420780658721923828125f,-0.158700012612342788997921161353588104248046875f,-1.1871000390589241391126051894389092922210693359375f,-0.8435000717043887874524443759582936763763427734375f,1.4882000421762471287223661420284770429134368896484375f,0.2638999982982872527514928151504136621952056884765625f,-0.3969999995380637525244083008146844804286956787109375f,-0.3029999823197722097489759107702411711215972900390625f,0.423200001347064702628131271922029554843902587890625f,0.502900014436244990889690598123706877231597900390625f,0.62939999004304458818381817764020524919033050537109375f,0.6318000069975848731473888619802892208099365234375f,0.3583999603599306027490456472150981426239013671875f,-0.1109999855011698510764972525066696107387542724609375f,0.50149996748566605386798755716881714761257171630859375f,-0.24280000769793996795442581060342490673065185546875f,0.6567000224992636248089183936826884746551513671875f,0.9143000240594147953743231482803821563720703125f,-0.5530000365227467629125612802454270422458648681640625f,0.0952000022262335932055066223256289958953857421875f,0.36580000592470174325399057124741375446319580078125f,-1.1025000283122061706109207079862244427204132080078125f,0.6564000130504370478234932306804694235324859619140625f,0.72560000649243583392689060929114930331707000732421875f,-0.1642999925062060828651055999216623604297637939453125f,0.6435999990940091119995258850394748151302337646484375f,-0.3639999826848507868959359257132746279239654541015625f,0.498499971717596412901229996350593864917755126953125f,-0.1782000180363656482995793339796364307403564453125f,0.709700012493133414182011620141565799713134765625f,-0.7862000126868486926667856096173636615276336669921875f,0.7356000200524928089862441993318498134613037109375f,0.34520002010762684818701018230058252811431884765625f,-0.7270000025182958847125291867996565997600555419921875f,-0.48240003438889988984783485648222267627716064453125f,0.858000020205974323062037001363933086395263671875f,0.1498000020951035793359551462344825267791748046875f,0.58110000445842757699210778810083866119384765625f,-0.311999978750943984806553999078460037708282470703125f,-0.6361000059112906779290597114595584571361541748046875f,-0.527599999809265085559673025272786617279052734375f,0.1176000017836689170991348873940296471118927001953125f,0.0894000056892634287208920795819722115993499755859375f,-0.0574000075668099629666585315135307610034942626953125f,-0.0396999924883243426876333614927716553211212158203125f,1.4060000378042456503635548870079219341278076171875f,0.7670999952122572462798189008026383817195892333984375f,0.2588000156626100167756021619425155222415924072265625f,0.3820999585554003541432166457525454461574554443359375f,1.70240003893375391186282286071218550205230712890625f,0.97319999851584437777773928246460855007171630859375f,1.15110004275441202281626829062588512897491455078125f,0.101200031238794618815290959901176393032073974609375f,-0.036499946475027744696717491024173796176910400390625f,1.052600090482832317917427644715644419193267822265625f,-0.31199998307228060667739555356092751026153564453125f,-0.8629000215888016356302614440210163593292236328125f,-0.4326000051736826890191878192126750946044921875f,-0.686400027132034296073470613919198513031005859375f,0.3818000122874976032250060598016716539859771728515625f,0.4333000455245379356483681476674973964691162109375f,0.29419999590069068062803125940263271331787109375f,-0.988499995410442533483319493825547397136688232421875f,1.1191999894559383488257253702613525092601776123046875f,0.056300006237626210037205964908935129642486572265625f,0.9578999780401584729361275094561278820037841796875f,-0.6485000056922434197304028202779591083526611328125f,0.985200011688470933535199947073124349117279052734375f,-0.6380000427812337893129779331502504646778106689453125f,0.072600017616152978661148154060356318950653076171875f,0.95050000582635352230909120407886803150177001953125f,-0.496099980542063168087452140753157436847686767578125f,-0.4547000359624624454824015629128552973270416259765625f,-0.53010003914833081495316946529783308506011962890625f,0.604200030210614524861512109055183827877044677734375f,0.5705000200569632085034754709340631961822509765625f,-0.5435000108703971211099315041792578995227813720703125f,-0.39319999859035037470533779924153350293636322021484375f,0.2929999944642187603704996945452876389026641845703125f,0.245000032633542641491430913447402417659759521484375f,-0.3592000199660656534206282231025397777557373046875f,-0.61419997209608556687499003601260483264923095703125f,0.548599998125433696571917607798241078853607177734375f,-0.146999974429607604964331812880118377506732940673828125f,-1.133000003367662156250617044861428439617156982421875f,-0.1987999584048985024509192953701131045818328857421875f,-0.340400010785460782614109120913781225681304931640625f,0.7081999871909616661724840014358051121234893798828125f,-0.707800024089217050260458563570864498615264892578125f,-0.9316000126689669702528817651909776031970977783203125f,1.285499992787837442165255197323858737945556640625f,-0.696000018581747870172193870530463755130767822265625f,-0.256199990558624079994842759333550930023193359375f,1.187199996399879609043637174181640148162841796875f,-0.43120001492202231219152963603846728801727294921875f,-1.12919997783303305283197914832271635532379150390625f,0.19629999718517066042977603501640260219573974609375f,0.3453000203266736178875362384133040904998779296875f,-0.915600025677679951741083641536533832550048828125f,-0.9622999935269358484646318174782209098339080810546875f,-0.56380001480579355899180882261134684085845947265625f,-1.589299980138242585070429413462989032268524169921875f,-0.6300999554038053673821195843629539012908935546875f,0.8085000057294966513410372499492950737476348876953125f,-1.65939996806383049943178775720298290252685546875f,-2.2964999818652866991897099069319665431976318359375f,1.0415999965012066041936122928746044635772705078125f,0.894399981623887896375890704803168773651123046875f,0.4418000087857247848432962200604379177093505859375f,0.7047000099971889586214501832728274166584014892578125f,0.3067999757051467213386786170303821563720703125f,0.52369994488656590458930395470815710723400115966796875f,-1.1166999241702264544784384270315058529376983642578125f,0.5268999759614469979851492098532617092132568359375f,-0.3440999989040196549439087903010658919811248779296875f,-0.553899993455410299958430186961777508258819580078125f,-0.608699959862232020668670884333550930023193359375f,1.44049998340010620978546285186894237995147705078125f,-1.82949992823600826596930346568115055561065673828125f,0.55949999931454641455275123007595539093017578125f,-1.4740999978423108274228070513345301151275634765625f,-0.0860000202953816739182002493180334568023681640625f,-0.214899969494343423548343707807362079620361328125f,0.59129999901056340405602895771153271198272705078125f,0.29869998251199714189851874834857881069183349609375f,-0.447999993383884298481234509381465613842010498046875f,0.08119999249577514177644843584857881069183349609375f,0.051000001184642262952451119417673908174037933349609375f,1.475500033468008620474165581981651484966278076171875f,0.5979000089228148073772217685473151504993438720703125f,0.091700035053491557590632510255090892314910888671875f,0.8109000148981808653303460232564248144626617431640625f,0.261899974024295101315829015220515429973602294921875f,-0.247399985215067397259502968154265545308589935302734375f,0.3366000376284130624071622150950133800506591796875f,0.5217999730974434147157126062666065990924835205078125f,0.45980000656545172432743129320442676544189453125f,-0.748300005514919153171149446279741823673248291015625f,0.9196000409588227508805857723928056657314300537109375f,-0.84090001325905350260114801130839623510837554931640625f,-0.5458999888956543866669335329788736999034881591796875f,-0.59990000797659182030230340387788601219654083251953125f,0.4222000366330149745408562012016773223876953125f,-0.0660999996155500202377197638270445168018341064453125f,0.23020000885725000472348256153054535388946533203125f,-0.756699938344956191116352783865295350551605224609375f,0.2878000461429357503817527685896493494510650634765625f,0.7995999471187591467469246708787977695465087890625f,0.28999994944781104511122293843072839081287384033203125f,0.058799990963935844234811156638897955417633056640625f,-1.39749997545033721735308063216507434844970703125f,-0.1120999772191044741020959918387234210968017578125f,1.2605000173375009442366945222602225840091705322265625f,-0.287600065374373770055171917192637920379638671875f,-0.3768000378802416339141245771315880119800567626953125f,-1.7490999638304109975450728597934357821941375732421875f,-0.420499972783029196676807259791530668735504150390625f,0.027499981969594244901600177399814128875732421875f,-0.919499979905783693112653054413385689258575439453125f,-0.341199991750716691285560955293476581573486328125f,-0.41170001441538295328115282245562411844730377197265625f,-0.5618999720871453718018528888933360576629638671875f,-1.117599972838163591148941122810356318950653076171875f,0.730299983483552406227090614265762269496917724609375f,0.7582999768674374418964134747511707246303558349609375f,0.5678999853789807872317396686412394046783447265625f,-0.317300015133618718010666270856745541095733642578125f,-0.974799940627813388260847204946912825107574462890625f,-0.5745999824345116024915114394389092922210693359375f,0.7191999606966970048915754887275397777557373046875f,0.67489999721050220049534118516021408140659332275390625f,-0.77839998062550908031198559910990297794342041015625f,0.4402999921634787039437242128769867122173309326171875f,-0.822099976995587677919274938176386058330535888671875f,1.75089999679326968617942839046008884906768798828125f,-1.2688999874576920579016814372153021395206451416015625f,-0.3640000159144403113486987422220408916473388671875f,-0.644699993121623293745869887061417102813720703125f,0.75659994863122737029925701790489256381988525390625f,1.09539997286200563308966593467630445957183837890625f,0.2437000093340870865432634673197753727436065673828125f,-0.52519999738335609862360797706060111522674560546875f,0.224399980916082764093033574681612662971019744873046875f,0.638700007545948356124654310406185686588287353515625f,1.256700007523595896685719708329997956752777099609375f,0.63509999057054500326557899825274944305419921875f,1.136500018633901820663822945789434015750885009765625f,-0.697600008749961641996151229250244796276092529296875f,-0.33769997790008776661352385417558252811431884765625f,-0.215599953481554695855493264389224350452423095703125f,-0.28130000623762596578814054737449623644351959228515625f,0.9001999899886545808413984559592790901660919189453125f,-0.314799950350821422517810788122005760669708251953125f,-1.150300000508130526810646188096143305301666259765625f,-0.353299994879215706333752677892334759235382080078125f,1.57660002377033237053183256648480892181396484375f,-0.24280003191232690795686721685342490673065185546875f,-2.235000023618340048159325306187383830547332763671875f,1.12270000349283183282977915951050817966461181640625f,0.577900044424831893508098801248706877231597900390625f,0.508800003182888094244162857648916542530059814453125f,-0.3609000099807975203702881117351353168487548828125f,0.5514000244498244995838831528089940547943115234375f,1.1533999906837941740178621330414898693561553955078125f,0.5833999556660653951212225365452468395233154296875f,-0.817099996329843758502420314471237361431121826171875f,0.341599984133244216621960731572471559047698974609375f,0.9167999938100575008093073847703635692596435546875f,-1.556099945673346862662356215878389775753021240234375f,-1.3880999941617255810655251480056904256343841552734375f,0.289000008873640801265736399727757088840007781982421875f,0.84820000920742710803779118577949702739715576171875f,-0.027900037384032572163050645031034946441650390625f,-0.313099971139431776379069560789503157138824462890625f,-0.9138999596297736616179463453590869903564453125f,0.4706999959826470014689903109683655202388763427734375f,-0.87900000101327879775681140017695724964141845703125f,-1.264799985331297005330952742951922118663787841796875f,0.653799985152482943107088431133888661861419677734375f,-1.37709996559619884948233448085375130176544189453125f,1.7545999492049215717770493938587605953216552734375f,0.49359999209046367951714273658581078052520751953125f,-0.459999982714652588811077293939888477325439453125f,0.3084999732822180096292186135542578995227813720703125f,-0.9262999694317581855784737854264676570892333984375f,-1.030600003400444553136594549869187176227569580078125f,0.2608000639088434713386277508107014000415802001953125f,0.9255999782919890339627499997732229530811309814453125f,0.353099998315423102557275569779449142515659332275390625f,-0.35160000898092957921647894181660376489162445068359375f,-0.453700004863738737270750789321027696132659912109375f,0.231999976776540162148876333958469331264495849609375f,0.22649999639391893424544832669198513031005859375f,-0.38629998645633445431002428449573926627635955810546875f,0.483900019085407251395736238919198513031005859375f,0.3531999888673420517903878135257400572299957275390625f,0.0755000084340569088681149878539144992828369140625f,0.8776999952971937801748936180956661701202392578125f,1.6275999738812438888402311931713484227657318115234375f,-0.0319999758079649865294413757510483264923095703125f,-0.08209998735189483198837478994391858577728271484375f,0.55809999837130253030892390597728081047534942626953125f,1.571000013440846831969111008220352232456207275390625f,0.66100001366436467176157520952983759343624114990234375f,1.15220003454685215871222681016661226749420166015625f,0.23890001081526290871437367968610487878322601318359375f,0.1609999897480012265305049368180334568023681640625f,0.45579997582435627379027209826745092868804931640625f,-0.366499991923570522800446269684471189975738525390625f,0.24149998044967713184405511128716170787811279296875f,-0.12369999007433685189738525878055952489376068115234375f,0.0554000394701947840303546399809420108795166015625f,0.50330004355013358008363866247236728668212890625f,1.343699936318397902823562617413699626922607421875f,-0.709500001884996844836450691218487918376922607421875f,-0.5710999369561680083506871596910059452056884765625f,0.0996000085487960262753404094837605953216552734375f,0.762100015366076632972180959768593311309814453125f,0.2286000538185237740407274031895212829113006591796875f,0.397300031673907039220239312271587550640106201171875f,-0.1381999700173730527552606872632168233394622802734375f,-0.62919999377727453548914127168245613574981689453125f,-1.3007000006765123689689289676607586443424224853515625f,-0.1983000071167939637462040991522371768951416015625f,0.813099996471404740105981545639224350452423095703125f,-0.32090000990629174992818661849014461040496826171875f,-0.10150001993775337183478768565692007541656494140625f,-1.07079994035959291664994452730752527713775634765625f,0.3561000050544738115831933100707828998565673828125f,-0.3714999932646758651344498503021895885467529296875f,0.5515999671459201891821066965349018573760986328125f,-0.666199972081184199623749009333550930023193359375f,-1.3571999856710430520934096421115100383758544921875f,-0.53499995768070274237970807007513940334320068359375f,1.4681999596238142213877608810435049235820770263671875f,-0.46419999139309009450471421587280929088592529296875f,-0.31210000344514821080110777984373271465301513671875f,-0.794599989289044739138034856296144425868988037109375f,-0.44560003253221491803515164065174758434295654296875f,-0.43609998627901003231954746297560632228851318359375f,-0.262199991631507867140271628159098327159881591796875f,-0.147199978518485874445786976139061152935028076171875f,-0.45720001018345335364045922688092105090618133544921875f,-0.2426000159025196722950568073429167270660400390625f,-1.1025000143051142398320507709286175668239593505859375f,0.03379999811649359031662243069149553775787353515625f,-0.2223000378206376748124739606282673776149749755859375f,0.407100002104044733641785569489002227783203125f,-0.133699969398975770928927886416204273700714111328125f,-1.592999984368681953839086418156512081623077392578125f,-0.2246999933823941031363347065052948892116546630859375f,0.084899973741173528907211220939643681049346923828125f,-0.4778999802008272457243265307624824345111846923828125f,-0.279399966871738403284552987315692007541656494140625f,-0.8697999766141180089107365347445011138916015625f,0.18980002809762908100310596637427806854248046875f,-0.19069999527484160495305332005955278873443603515625f,-0.682199986267089997227230924181640148162841796875f,0.0149999949336052651460704510100185871124267578125f,0.9711999892547726886959935654886066913604736328125f,-0.63180003232955872505272054695524275302886962890625f,-1.379300027002394291031350803677923977375030517578125f,-0.0557999555587773699016906903125345706939697265625f,0.9575000524148344904773466623737476766109466552734375f,-1.009499979794025392720868694595992565155029296875f,-2.284599984818696416510874769301153719425201416015625f,0.770399970328808247899132766178809106349945068359375f,0.9551999482467772306648612357093952596187591552734375f,-0.760599945807457711310917147784493863582611083984375f,0.30869999365061528973086524274549447000026702880859375f,-0.576399980640411424559488295926712453365325927734375f,0.961499994270503588467136069084517657756805419921875f,-0.708999986708164442461566068232059478759765625f,-0.9721999873101712097422932856716215610504150390625f,-0.2956999766111376715116421109996736049652099609375f,0.167100004339217900195535548846237361431121826171875f,-0.395499999344348651675318251363933086395263671875f,0.8694999774098395306509701185859739780426025390625f,-1.52139995530843652460362136480398476123809814453125f,0.7214000147640700078000008943490684032440185546875f,-0.77920001672506344903013086877763271331787109375f,-0.71339998889565503503717991407029330730438232421875f,0.0073000075340274861446232534945011138916015625f,0.1808999882251025592694304577889852225780487060546875f,0.370199980023503627535319537855684757232666015625f,-0.9088999673783779709168584304279647767543792724609375f,0.573699986162781616627626135596074163913726806640625f,1.126999963849783181757402417133562266826629638671875f,-0.111299998265505184491530599188990890979766845703125f,0.707199962127209147411122103221714496612548828125f,-0.31789997986555107445383328013122081756591796875f,0.505199989038705776778215295053087174892425537109375f,1.76730001602768904689355622394941747188568115234375f,0.085800039750337564470328288734890520572662353515625f,-0.96140002385377965055113236303441226482391357421875f,-0.733300006744264987190717874909751117229461669921875f,-0.741000020146369564599808654747903347015380859375f,0.68199998929351479670657454335014335811138153076171875f,0.04719998582005491272184372064657509326934814453125f,0.24000002227723638004164286030572839081287384033203125f,-1.513100003176926833248217008076608180999755859375f,-0.83369999227225821680775652566808275878429412841796875f,-0.770599946850537786957602293114177882671356201171875f,0.55859996102154318986521275292034260928630828857421875f,-0.108800067704915903732398874126374721527099609375f,1.1049999945238224707821927950135432183742523193359375f,0.887500007152556946721233543939888477325439453125f,0.429200019332766746771312682540155947208404541015625f,1.0540000233650204108926118351519107818603515625f,-1.6030999744176863686107026296667754650115966796875f,-1.411200002703071110232713181176222860813140869140625f,-0.5256000030279164292323912377469241619110107421875f,-0.31330002857446681563402535175555385649204254150390625f,0.833000025868415150398504920303821563720703125f,1.3646000090330840937014045266550965607166290283203125f,-0.04230000544786438609889955841936171054840087890625f,-1.24429999824315284939757475513033568859100341796875f,0.775099979841709085803813650272786617279052734375f,1.08519997775107679416350947576574981212615966796875f,1.10829993350505962013130556442774832248687744140625f,0.06850000971555691986480951527482829988002777099609375f,1.90549998742341841762026888318359851837158203125f,0.315899994930625371836185877327807247638702392578125f,0.230800021719932857422463712282478809356689453125f,-0.057399987711012290247936107334680855274200439453125f,-0.109399988031388062381665804423391819000244140625f,-1.40549999815225579169464253936894237995147705078125f,0.477600056135653261435436434112489223480224609375f,0.581400004184246199656627140939235687255859375f,0.3454999886155132315934679354541003704071044921875f,0.319500026732682851360323184053413569927215576171875f,0.93659994181394523593553458340466022491455078125f,-1.22499999061226816365888225845992565155029296875f,0.69739998499155131383986372384242713451385498046875f,0.672500020116566954442305359407328069210052490234375f,-0.6128000101566311030865108477883040904998779296875f,0.51309996413588532249150375719182193279266357421875f,-0.48289999715089759746433628606610000133514404296875f,-0.49499992102384648973156799911521375179290771484375f,-0.3089000050783159423417600919492542743682861328125f,-1.348000019013881001228583045303821563720703125f,-0.150399987316131245052019949071109294891357421875f,0.141599979811906973026225387002341449260711669921875f,1.571400009772181416423109112656675279140472412109375f,-0.7819999762922529296105267349048517644405364990234375f,0.2002000364243980978784520630142651498317718505859375f,-0.343400001449882863457929715877980925142765045166015625f,0.099899992591142794395864257239736616611480712890625f,0.51529997797012339066924369035405106842517852783203125f,0.278899998670816717805109874461777508258819580078125f,-0.257300007496774341131384744585375301539897918701171875f,1.2415000643059619367392087951884604990482330322265625f,0.0022000023230910326077491845353506505489349365234375f,-0.6440000283941629266593054126133210957050323486328125f,0.3000999885961415092339166221790947020053863525390625f,-0.489600015515088582418457008316181600093841552734375f,-0.8067999798029663605092309808242134749889373779296875f,-0.2945999703645707512578155728988349437713623046875f,-0.3819000039756297315562960648094303905963897705078125f,0.4024000042885542693937850344809703528881072998046875f,-1.3087999511033299171458565979264676570892333984375f,-0.111999995365738680419553929823450744152069091796875f,-1.1842999770462512287849676795303821563720703125f,-1.2824999840930104699765479381312616169452667236328125f,-0.153799984705448533617300199694000184535980224609375f,0.10989999259114302532225337927229702472686767578125f,0.174699984776973682443212965154089033603668212890625f,1.254099956603349053096962961717508733272552490234375f,-1.4194999932050691437979139664093963801860809326171875f,-0.79529997528791451788521271737408824265003204345703125f,0.2696999704718583412699217660701833665370941162109375f,-0.9267000243619083477142339688725769519805908203125f,0.2686999986052516220524921664036810398101806640625f,0.49420001430362459071687908362946473062038421630859375f,-0.387199984478950653965512174181640148162841796875f,-0.1485000015944242246490603065467439591884613037109375f,-0.434000005148350798123146887519396841526031494140625f,-0.24590000081658380803872887554462067782878875732421875f,-0.95039996380954983035138639024808071553707122802734375f,0.9766999489247802923586050383164547383785247802734375f,-0.34499998673796650194844914949499070644378662109375f,-0.5806999637961391957929890850209631025791168212890625f,1.117599932008982666076235545915551483631134033203125f,0.416500019341706551045945161604322493076324462890625f,0.070599947446586153176895095384679734706878662109375f,-0.01150004679709659693997991780634038150310516357421875f,0.15560001401752199257089159800671041011810302734375f,0.3594000270724293510937741302768699824810028076171875f,0.4543999852374198678006678164820186793804168701171875f,-0.09709999390840540200997565989382565021514892578125f,-0.7573999721765520831695539527572691440582275390625f,0.47439998058080679044223870732821524143218994140625f,-0.5548999867796897689231627737171947956085205078125f,-0.052799995627999418701392642105929553508758544921875f,-1.247900017267465688775018861633725464344024658203125f,-0.54989999650269727471396663531777448952198028564453125f,-1.3387000325798990019166012643836438655853271484375f,0.07420000584721553327227638874319382011890411376953125f,-0.02760001739263628905973746441304683685302734375f,0.6473999941557646575773787844809703528881072998046875f,1.00689998922347978549396430025808513164520263671875f,-0.36849998870491962321693790727294981479644775390625f,0.12280001231729986699292567209340631961822509765625f,-0.5465000022053720929449127652333118021488189697265625f,0.495900011992454425868714906755485571920871734619140625f,-0.995999957673252112044792738743126392364501953125f,-0.14419996092021525324611275209463201463222503662109375f,-1.0787999915599819633627021175925619900226593017578125f,-0.41759999027252181491576266125775873661041259765625f,-0.31980000984370715588767097870004363358020782470703125f,-1.11420001076459840305687976069748401641845703125f,0.30799999338388506231467545148916542530059814453125f,0.9366999547734859987713207374326884746551513671875f,0.47359999402761498998870592913590371608734130859375f,-0.0988999994531278847631483586155809462070465087890625f,1.1036000136971466645263717509806156158447265625f,2.420099995972215811690375630860216915607452392578125f,-0.476000004351138983338387333787977695465087890625f,-0.953799969729781427218995304428972303867340087890625f,-0.48579997880458858361407692427746951580047607421875f,-0.773999963983893479735343134962022304534912109375f,-1.2399999454990033864731913126888684928417205810546875f,-0.8354999847784638777881127680302597582340240478515625f,-1.2683999637871981036596480407752096652984619140625f,-0.1154000075444578943262285974924452602863311767578125f,-0.35080000085830675971010350622236728668212890625f,-0.2291999958634376444166491637588478624820709228515625f,0.438499968439341358816818683408200740814208984375f,-0.218300001901388096570144625729881227016448974609375f,0.42829996226430022687736709485761821269989013671875f,0.01329999057650610438940930180251598358154296875f,0.53380000705718888553974466049112379550933837890625f,-0.274799991887807237844754126854240894317626953125f,0.82629998023509898530392092652618885040283203125f,0.3759999743998054810845133033581078052520751953125f,-0.94619994674921059640837484039366245269775390625f,0.82309997978210447389102455417742021381855010986328125f,-1.34049995709955727107853817869909107685089111328125f,-0.101000017948448428395380460642627440392971038818359375f,-0.944799945433438370656631377642042934894561767578125f,0.7179999571740631747474026269628666341304779052734375f,-0.46299998824298349564543286760454066097736358642578125f,0.867199985671043283019798764144070446491241455078125f,-0.12940000606179236086745731881819665431976318359375f,-0.86829999020397696796180753153748810291290283203125f,-0.53689999123513754142322795814834535121917724609375f,-0.54919999105781325265951409164699725806713104248046875f,0.6044999863691631336592990919598378241062164306640625f,-0.68419999744668569263694735127501189708709716796875f,-0.993900018191337775164129197946749627590179443359375f,-0.3140999716162677923847468264284543693065643310546875f,-1.06619998243749147803782761911861598491668701171875f,0.847599992656707712512798025272786617279052734375f,-0.315100005397199733891966388910077512264251708984375f,0.976800049987435414777792175300419330596923828125f,-1.833500020593405555047183952410705387592315673828125f,0.348600001627206612653253614553250372409820556640625f,0.81370001618862186631986332940869033336639404296875f,0.70179999641776069640997093301848508417606353759765625f,0.188500001445412568301662759040482342243194580078125f,0.5326000188827515469114359802915714681148529052734375f,-0.4759999650120736447433955618180334568023681640625f,1.869799960446357989241050745476968586444854736328125f,0.54599998354539291955944690926116891205310821533203125f,0.4555999775841834686929132658406160771846771240234375f,-0.28389999481514094359368982622982002794742584228515625f,1.784199991691112874292457490810193121433258056640625f,-0.059100021684169945501707843504846096038818359375f,0.892599996009469265345614985562860965728759765625f,0.5580999690532681967169992276467382907867431640625f,1.296800014820695512440806851373054087162017822265625f,-1.4012999838113788797500092186965048313140869140625f,-0.050800006371736206034483984694816172122955322265625f,0.205199949550628701899768202565610408782958984375f,-1.50119999622106536918408892233856022357940673828125f,0.3100000192224978690092029864899814128875732421875f,-0.345400000056624190136744800838641822338104248046875f,-0.32250000268220890831116776098497211933135986328125f,-0.01220000981092450587084385915659368038177490234375f,0.82039995945095878226993590942583978176116943359375f,0.7190000196397310361362542607821524143218994140625f,-2.26570008047223137026549011352472007274627685546875f,-1.25940000144243224866613672929815948009490966796875f,-0.443600016230345062240303377620875835418701171875f,-0.85010005211234140887199828284792602062225341796875f,1.2542000425040722433323026052676141262054443359375f,1.4858000369191177725269881193526089191436767578125f,-1.399500032544136018941571819595992565155029296875f,-0.911399982279538445339994723326526582241058349609375f,1.866400066545606417633962337276898324489593505859375f,2.21910007909089390665258179069496691226959228515625f,-0.9691000539079317466217844412312842905521392822265625f,-0.028599982888996233754141940153203904628753662109375f,1.0585000481605533195761381648480892181396484375f,1.048500002264976416199715458787977695465087890625f,-0.0608000071167946742889398592524230480194091796875f,-0.219699980604648015258817395078949630260467529296875f,-0.51260002104341939599407851346768438816070556640625f,1.2768999965995544432217911889893002808094024658203125f,-1.5316000322639948816316746160737238824367523193359375f,-1.0324999951571225409452381427399814128875732421875f,0.582299999040365445779343644971959292888641357421875f,0.478699988994002456621501551126129925251007080078125f,-0.00710000396668863942295502056367695331573486328125f,-0.106700008156895620459181373007595539093017578125f,-0.4961999869823454645256788353435695171356201171875f,-0.8948000050753355250066078951931558549404144287109375f,-0.0260999903023240442667685101696406491100788116455078125f,0.039000040873885755132732811034657061100006103515625f,-0.342900024010241055716363689498393796384334564208984375f,-0.7034000377714637597392766110715456306934356689453125f,0.598099999302625473518446597154252231121063232421875f,0.32329999802708631051473275874741375446319580078125f,0.19780000702738664841717763920314610004425048828125f,-0.9510999914944164856933639384806156158447265625f,0.0317999729484317317229624677565880119800567626953125f,0.93040003038048713079888329957611858844757080078125f,0.733600010344386266325500400853343307971954345703125f,0.72279998199343697962149235536344349384307861328125f,-0.657399988642335397770466443034820258617401123046875f,-1.1861000035643574079813333810307085514068603515625f,1.674600018271803936187325234641321003437042236328125f,0.447900000354647165323740409803576767444610595703125f,1.37499999552965146421001918497495353221893310546875f,-1.176700021083652902831317987875081598758697509765625f,1.012400022616982031564702992909587919712066650390625f,0.8958999943345784533477171862614341080188751220703125f,0.3530000281035901021908784969127736985683441162109375f,-0.4542000080823900276527638197876513004302978515625f,0.5090000501871116966867703013122081756591796875f,-1.0118000318825242089815219515003263950347900390625f,-1.173300010991095820855889542144723236560821533203125f,-0.3850000053644182518297611750313080847263336181640625f,0.65079998238086655959477866417728364467620849609375f,-0.0557000026881695475822198204696178436279296875f,-0.43310002195239061251186285517178475856781005859375f,-0.71180007345676532537481762119568884372711181640625f,-1.101700025665760307447271770797669887542724609375f,-1.45640004575848625023581917048431932926177978515625f,-1.7010000150799751139629734097979962825775146484375f,0.7272000495970250000254964106716215610504150390625f,1.6048999899089342324742801793036051094532012939453125f,1.592200011003016779653762569068931043148040771484375f,0.15859997610896858777351781100151129066944122314453125f,-0.17830003464669108570461730778333730995655059814453125f,0.01079997297450863147361133087542839348316192626953125f,-1.094399997866152940417805439210496842861175537109375f,-1.2139000138700009046033301274292171001434326171875f,-0.26080003095865311735224167932756245136260986328125f,0.56870002550184761691554058415931649506092071533203125f,0.5208000001877548523765426580212078988552093505859375f,-0.1450000271201137280741022550500929355621337890625f,-0.066300029110908820229042248683981597423553466796875f,0.821300008547306337192139835678972303867340087890625f,-0.68820000760555277707908317097462713718414306640625f,-0.10899997694790375390994086046703159809112548828125f,0.1188999845147133971323682999354787170886993408203125f,0.243299985957145192827510982169769704341888427734375f,0.759799991068244029435163611196912825107574462890625f,-1.590100001671910678879839906585402786731719970703125f,0.2458999746277925613213710676063783466815948486328125f,0.8429000337518754992771619072300381958484649658203125f,-0.1175000029243526977751344020362012088298797607421875f,-0.215100007557868622143359971232712268829345703125f,1.20440000337958341702915276982821524143218994140625f,1.116400004780291776995682084816507995128631591796875f,0.03280001499950879750855392558150924742221832275390625f,0.093599992388487152084053377620875835418701171875f,-0.826999994844198393906253841123543679714202880859375f,0.20750000610947605395040227449499070644378662109375f,-1.001600010657310857453694552532397210597991943359375f,-0.222699962961673492145564523525536060333251953125f,0.16480003219544858694689537514932453632354736328125f,1.2476999796509744555805809795856475830078125f,0.3904000078797338346703327260911464691162109375f,-0.7654000028133391975160293441149406135082244873046875f,0.166599964761733776441587906447239220142364501953125f,-0.39309999634101966403676442496362142264842987060546875f,-0.639200015644729280239744184655137360095977783203125f,0.05190001192912496730258453681017272174358367919921875f,0.381900012171268343763586017303168773651123046875f,-0.659699990439414296616860156063921749591827392578125f,-0.09019997637271881529841266456060111522674560546875f,-0.20779997841715758255531909526325762271881103515625f,0.0958999774217605960302535095252096652984619140625f,0.1958999687790861088387828203849494457244873046875f,1.3196000004649164338843547739088535308837890625f,-2.3182000448584556551168134319595992565155029296875f,-1.23190003184080154596813372336328029632568359375f,0.8296999929726123212958555086515843868255615234375f,0.81339995611309934275823252392001450061798095703125f,-1.5965000610649588974609969227458350360393524169921875f,0.20410001229643714992789682582952082157135009765625f,-1.342100097620488430294471982051618397235870361328125f,-0.08620003123879538264873190200887620449066162109375f,0.1171000126838688260022536269389092922210693359375f,0.0339000149130814776299303048290312290191650390625f,-0.0109999786466359328329644995392300188541412353515625f,0.34570000283718105293928601895458996295928955078125f,-0.3059000032007694880320514130289666354656219482421875f,-0.3909999998062854142943933766218833625316619873046875f,0.659700006458163290545826384914107620716094970703125f,0.43030003235936220562507514841854572296142578125f,-0.340100018435716577869243337772786617279052734375f,0.53599999503791284638509750948287546634674072265625f,-0.61940002282559891710889132809825241565704345703125f,-0.86140000291764717843534526764415204524993896484375f,0.50590002264678435306421988570946268737316131591796875f,1.181900048604607622593221094575710594654083251953125f,-1.1464000042587521033254915892030112445354461669921875f,0.089000021204352908199552985024638473987579345703125f,-0.60839999969899682952245711931027472019195556640625f,-0.926600036436319829391550229047425091266632080078125f,-0.24689999443888677177483259583823382854461669921875f,-0.3969999955892564713622050476260483264923095703125f,-0.09820000343322732305750832892954349517822265625f,0.64789998672008497493379763909615576267242431640625f,0.9405999905854465215071513739530928432941436767578125f,-0.0244000121712684148178595933131873607635498046875f,0.4665999783217902319165659719146788120269775390625f,-0.506699982750415500731833162717521190643310546875f,0.40919998863637463415443562553264200687408447265625f,1.4354999945759772117526154033839702606201171875f,1.22459996328651943287013637018389999866485595703125f,-1.0455999656260015484576797462068498134613037109375f,0.513999943271279402523532553459517657756805419921875f,0.634000005930662524633589782752096652984619140625f,1.57239999251663675750023685395717620849609375f,0.10810002999901779929814438219182193279266357421875f,0.839000012189150101704626649734564125537872314453125f,0.1529999826550483277287639793939888477325439453125f,-0.3545000069141384102522351895458996295928955078125f,0.553300014269351603246605009189806878566741943359375f,0.98629999662637768409467753372155129909515380859375f,0.2830000512003909562963599455542862415313720703125f,0.26639999941587433340828283689916133880615234375f,0.81860003351569243790208929567597806453704833984375f,2.07400004601478560317673327517695724964141845703125f,0.67419999899268123755291526322253048419952392578125f,-0.8823999837994573880450843716971576213836669921875f,0.465500033617019415288496020366437733173370361328125f,0.0347999659597875155014889969606883823871612548828125f,1.3946000029981131529410731673124246299266815185546875f,0.25719998492598517003671076963655650615692138671875f,0.13199999424815178628023204510100185871124267578125f,0.79650004079937897216723285964690148830413818359375f,0.13399998462200102267161128111183643341064453125f,0.3065000060796734260293305851519107818603515625f,0.5122999840646984548442333107232116162776947021484375f,0.2433999986559154393717108177952468395233154296875f,0.4683000027954580257727457137661986052989959716796875f,-1.0516000277191401846010876397485844790935516357421875f,0.578400022721290429927876175497658550739288330078125f,0.445800028201938314253993667080067098140716552734375f,0.168100027167797261284931664704345166683197021484375f,-1.6865000057816506018326663252082653343677520751953125f,0.241999973610043550564796532853506505489349365234375f,0.55889997877776576018504783860407769680023193359375f,0.87339998777806737706441708724014461040496826171875f,-0.5869999941736454562857261407771147787570953369140625f,0.61700001294910900018209076733910478651523590087890625f,0.59410002146065199468694117967970669269561767578125f,-0.4245999803483488932442924124188721179962158203125f,-0.372299998891353567387341172434389591217041015625f,-0.5155000094771384766545452293939888477325439453125f,0.40869997506141686471892171539366245269775390625f,0.3320000189095733134081456228159368038177490234375f,-0.74859999104738239594780679908581078052520751953125f,1.1834000098317869476005625983816571533679962158203125f,0.397199983286856905806416762061417102813720703125f,0.592799985346198266000783405615948140621185302734375f,-0.93590000066757195185118689551018178462982177734375f,-0.0717999536514286607058465961017645895481109619140625f,0.662899967348575813730349182151257991790771484375f,-1.437799970296025353633240229100920259952545166015625f,-0.0474999943375589150917903680237941443920135498046875f,-0.386199992570281092074679918368929065763950347900390625f,-0.819699981498718255323865378159098327159881591796875f,0.664999970719218236325787074747495353221893310546875f,-0.60319999017119396711450463044457137584686279296875f,0.488199992704391405595742980949580669403076171875f,0.26549999248981492883103783242404460906982421875f,0.31820002369880739934160374104976654052734375f,0.92879998679160991059688967652618885040283203125f,-0.26990001091956994372367262258194386959075927734375f,0.4202999928712838340061352937482297420501708984375f,-0.3773000182628631904435678734444081783294677734375f,-0.236400006419419828063155364361591637134552001953125f,0.2766000252604481346452303114347159862518310546875f,-0.2194000266253954833928219159133732318878173828125f,-1.90700002956390424202481881366111338138580322265625f,0.172100015217065749340008551371283829212188720703125f,0.34909999337196406798966563655994832515716552734375f,-0.2948999517619608212726234341971576213836669921875f,0.420299997043610318314676987938582897186279296875f,-1.6277000785946853511632070876657962799072265625f,-0.955700072574614978293539024889469146728515625f,0.1927000097513200671528466045856475830078125f,-0.09470003514289970780737348832190036773681640625f,-0.717000012725591329143526309053413569927215576171875f,-1.50460001670718224175971045042388141155242919921875f,-0.356400033167004526291066213161684572696685791015625f,-0.1797999997854227149929329243605025112628936767578125f,0.09710000508427629739571784739382565021514892578125f,1.343400026744604236483837667037732899188995361328125f,-0.4110999986469749956086161546409130096435546875f,-0.921900013139843821363683673553168773651123046875f,0.3148999935597185384494878235273063182830810546875f,0.4931000212818383854340709149255417287349700927734375f,1.131099993059038855136577694793231785297393798828125f,0.5128999721169467118642160130548290908336639404296875f,0.3131999737054116383916380073060281574726104736328125f,0.66590003568530153899018841912038624286651611328125f,0.279099957683681676456899367622099816799163818359375f,-0.53160002444088494399920818977989256381988525390625f,0.55660002049207690788534819148480892181396484375f,-0.075000007152556946721233543939888477325439453125f,0.65499998226761757535996366641484200954437255859375f,-0.36069999806880925774521529092453420162200927734375f,-0.5445999876499174607857867158600129187107086181640625f,0.23380000824928259817170328460633754730224609375f,-1.269000033646821901101020557689480483531951904296875f,-0.1408999792098999392919722595252096652984619140625f,-0.77299994339048883063014727667905390262603759765625f,0.80089997622966802026667210157029330730438232421875f,-0.3469999642223122737760832023923285305500030517578125f,-0.15729999412298223404604868846945464611053466796875f,-0.6297999571681025532399189614807255566120147705078125f,0.464099959397316297327051870524883270263671875f,-0.37560000943541549389692590921185910701751708984375f,0.7982000496268266687138748238794505596160888671875f,0.22820001021325619916524374275468289852142333984375f,-0.29880000586509680715607828460633754730224609375f,0.0690000024288892621626700929482467472553253173828125f,0.7417999961942436382145160678192041814327239990234375f,1.1568000223085281863433237958815880119800567626953125f,0.6180000166296959474010463964077644050121307373046875f,0.7438000382006162425341244670562446117401123046875f,0.2724999879300591398845199364586733281612396240234375f,-0.342699982333183328364611952565610408782958984375f,-1.05480002765059399649771876283921301364898681640625f,-0.20509995674490966166558791883289813995361328125f,0.7001000491321096586005978679168038070201873779296875f,0.72030008473694351334160046462784521281719207763671875f,-0.78330004645586104317089848336763679981231689453125f,-1.26660002667605819937080013914965093135833740234375f,0.2765000245571140879974336712621152400970458984375f,0.128399992918968308686089585535228252410888671875f,1.0769000258803362868320618872530758380889892578125f,1.253099993193149686021570232696831226348876953125f,-0.2025999957114457128426465715165250003337860107421875f,-0.853599995778500986176595688448287546634674072265625f,-0.001499973036349189214888610877096652984619140625f,-0.576300009515881672683690339908935129642486572265625f,-0.64399999691545961599814518194762058556079864501953125f,0.12599998810887402811431456939317286014556884765625f,-0.3697999947190282465925292854080908000469207763671875f,0.90219999721944343917812147992663085460662841796875f,-0.614899958392978174259724255534820258617401123046875f,-0.60890002661049447141294876928441226482391357421875f,-0.885199992130696333703099298872984945774078369140625f,0.03900000511109880108051584102213382720947265625f,-0.409800035212934066208845251821912825107574462890625f,-0.2319999789744617846309893138823099434375762939453125f,1.1214000129014254358850166681804694235324859619140625f,-0.288400007671117730723153727012686431407928466796875f,0.415099988335370984060546106775291264057159423828125f,0.2092000010415910882244361346238292753696441650390625f,2.096999976590276038024285298888571560382843017578125f,-1.0590999808922412217526698441361077129840850830078125f,0.534399987956881705741807309095747768878936767578125f,0.16079997969865811313638914725743234157562255859375f,-0.927199970173835819053920204169116914272308349609375f,-0.07899999445676808562666337820701301097869873046875f,0.823399999848008601333049227832816541194915771484375f,-0.3171999706208712410671068937517702579498291015625f,0.8165999885290859960917941862135194242000579833984375f,-0.43100004220008802935382163923350162804126739501953125f,-0.79099998706579233154201347133493982255458831787109375f,1.1089000112622973137632698126253671944141387939453125f,0.3640000140517949400731367859407328069210052490234375f,-0.3013000127196310273802737356163561344146728515625f,0.083800011974572985451459317118860781192779541015625f,-0.3804000004291534509093253291212022304534912109375f,-0.680800024104118772783067470300011336803436279296875f,0.795400064951182539374485713778994977474212646484375f,-0.0406000033259399373264386667869985103607177734375f,0.46340005177855569939993074513040482997894287109375f,-0.38770001839399359511162401759065687656402587890625f,-0.7136000397741799616824209806509315967559814453125f,0.22349995875358530383891775272786617279052734375f,-1.0616000328600403435075349989347159862518310546875f,-0.51649999117851308483295724727213382720947265625f,0.5376000145614145964145791367627680301666259765625f,1.1791000099122523891992386779747903347015380859375f,1.411400010070204213974420781596563756465911865234375f,0.9772999911800024364083583350293338298797607421875f,1.48700002595037172881120568490587174892425537109375f,1.4100000107288355177814764829236082732677459716796875f,0.18520001764893567752778835711069405078887939453125f,0.3918000342667102842142412555404007434844970703125f,0.6282999951958654349937205552123486995697021484375f,0.17699997122585731101906958429026417434215545654296875f,1.021800038588046799503672446007840335369110107421875f,0.068099998483061252585457623354159295558929443359375f,0.5980999940872191444896088796667754650115966796875f,1.6539000194579365210501009642030112445354461669921875f,-0.331199982511997337297771082376129925251007080078125f,-0.49969998254179959218390649766661226749420166015625f,-0.555099979692697420574631905765272676944732666015625f,-0.577499982565641278853263429482467472553253173828125f,0.2609999951124191142071140347979962825775146484375f,-1.3022999885350465820721410636906512081623077392578125f,0.69239999810457197870761092417524196207523345947265625f,-0.524200010056793619384052362875081598758697509765625f,0.222800018277764355634218418344971723854541778564453125f,0.087400003767013512145922504714690148830413818359375f,-0.2725999963819980553836330727790482342243194580078125f,0.420800001752376306995984123204834759235382080078125f,-1.120999977156519111787247311440296471118927001953125f,-0.35690000293254886543081738636828958988189697265625f,-1.600400019577145283022900912328623235225677490234375f,0.42179998226165782426733130705542862415313720703125f,-0.4998000527590511499198555611656047403812408447265625f,-0.7199999523162841086332264239899814128875732421875f,1.80819996930956872205342733650468289852142333984375f,0.421199989366531468704124563373625278472900390625f,1.06679999313950535366757321753539144992828369140625f,-0.071000012472271833985359990037977695465087890625f,1.0833999961972236025786742175114341080188751220703125f,-0.5972999997854231413185743804206140339374542236328125f,0.58240000734329233633701505823410116136074066162109375f,-0.756800000552833029843213807907886803150177001953125f,0.028399994707107634184239941532723605632781982421875f,0.10859998866319653387790822307579219341278076171875f,-0.274999980330466797795452293939888477325439453125f,-1.753699948835372168787216651253402233123779296875f,-0.034800011222063886862088111229240894317626953125f,-1.435300017905234426507377065718173980712890625f,0.341599950195849100254008590127341449260711669921875f,-0.53810000896975418438472615889622829854488372802734375f,-0.18670000171214351514237250739824958145618438720703125f,-0.49900001509487557083133424384868703782558441162109375f,-1.0832000025391581576883481830009259283542633056640625f,1.916199970255791473761064480640925467014312744140625f,1.004100006149708956826316352817229926586151123046875f,1.12350000569224306445903494022786617279052734375f,0.816699999290704692356257510255090892314910888671875f,-0.4866000087946649177439439881709404289722442626953125f,-0.38390000295490034343259821980609558522701263427734375f,-0.5053999918982385874954843529849313199520111083984375f,-0.3578999895140528852977013229974545538425445556640625f,-1.186499998033045955025954754091799259185791015625f,-1.52819999210834556180316212703473865985870361328125f,0.148699972379208844586173654533922672271728515625f,0.66509998744130083281334009370766580104827880859375f,0.9924999639391902661600397550500929355621337890625f,0.96379999740868793622894372674636542797088623046875f,0.38799999079480740338254918242455460131168365478515625f,-0.419900005406141196839797657958115451037883758544921875f,0.498600010493397416411198719288222491741180419921875f,0.479500030420720424473302045953460037708282470703125f,0.4463999886870373412506296517676673829555511474609375f,-1.01809999423623054326526471413671970367431640625f,0.1354000140264626228514543981873430311679840087890625f,-0.1421000191658732259014641385874710977077484130859375f,-0.875799965412169090228644563467241823673248291015625f,0.6448000027656550781074429323780350387096405029296875f,-0.522199996455758874702723915106616914272308349609375f,0.15719999867230638468384995576343499124050140380859375f,0.18840004375055519059145581195480190217494964599609375f,-0.2931000036053357238330363543354906141757965087890625f,0.016600015500188192163477651774883270263671875f,-0.99330001344978768873517083193291909992694854736328125f,-0.1590999982707199389420082979995640926063060760498046875f,-1.206000025175511414232687457115389406681060791015625f,-0.29510000368356703592809253677842207252979278564453125f,0.40259997907802380634478822685196064412593841552734375f,1.344099999779462262239349001902155578136444091796875f,1.445800008383393286948148670489899814128875732421875f,0.758499993622302692841685711755417287349700927734375f,0.278800015401840683981760093956836499273777008056640625f,-1.27370008562803338492130933445878326892852783203125f,-0.19040000981688509629208283513435162603855133056640625f,-0.9057000099897383194047506549395620822906494140625f,-0.137400003953278115886860177852213382720947265625f,-0.063200004364549755564439692534506320953369140625f,-0.53390002340674291847477661576704122126102447509765625f,0.64619997996017364272347549558617174625396728515625f,-1.07590003255605637377811945043504238128662109375f,-0.60290004081130010860078982659615576267242431640625f,-1.300499995712190326457857736386358737945556640625f,-0.4762000051990151749947699499898590147495269775390625f,0.559200017097591484116492210887372493743896484375f,1.1002000031761827170129208752769045531749725341796875f,0.167000023603439018415883765555918216705322265625f,-0.7634999739155172715499020341667346656322479248046875f,-0.392800016117095029954953133710660040378570556640625f,0.201299952109157853641363544738851487636566162109375f,0.09780001879930377839400534867309033870697021484375f,-0.777900000764429666588739564758725464344024658203125f,0.7502000011086469299925738596357405185699462890625f,1.4276999989107252986286766827106475830078125f,0.713999958302826254197270827717147767543792724609375f,-0.01690000784993206384143604736891575157642364501953125f,0.65469996184781276138409111808869056403636932373046875f,0.73279999853372512763627355525386519730091094970703125f,-0.4621999940901986292374203912913799285888671875f,0.704199998601525489760888376622460782527923583984375f,-0.70789999543726389585884817279293201863765716552734375f,1.34710000374317129256951375282369554042816162109375f,-0.04500003056600732076475424037198536098003387451171875f,-1.1565000086128705003574168586055748164653778076171875f,0.974999978989362858783351839520037174224853515625f,-1.27210000448822935226189656532369554042816162109375f,1.189799990695715070643245780956931412220001220703125f,0.307500042170286036480320035479962825775146484375f,0.52559997084140785972294906969182193279266357421875f,-0.1232000037312506179887350299395620822906494140625f,0.2623999674081805011383039527572691440582275390625f,-0.88499998636543730778925009872182272374629974365234375f,0.676700000184773831080065065179951488971710205078125f,-0.41540000951886180136085613412433303892612457275390625f,0.34810000749826475185955132474191486835479736328125f,-0.4049999793991450047769831144250929355621337890625f,-0.0906999928534037991312288795597851276397705078125f,-1.045499951288104245605836695176549255847930908203125f,-0.76519998874068306804474559612572193145751953125f,0.54179998602420076370123069864348508417606353759765625f,0.76390000299215277124176282086409628391265869140625f,0.58680000398010034867724016294232569634914398193359375f,0.590199994701146923858914306038059294223785400390625f,0.7965999745592478031852579078986309468746185302734375f,1.017799974095821458064392572850920259952545166015625f,-0.469099987970292620076406819862313568592071533203125f,-0.551899984157085299329992267303168773651123046875f,-0.40900001036375766005903642508201301097869873046875f,0.1294000017404552949074059142731130123138427734375f,-1.52019993494749172668889514170587062835693359375f,-0.7153999936863779662843398909899406135082244873046875f,-1.37649995481967923893762417719699442386627197265625f,-0.296599998140334886187474694452248513698577880859375f,-1.6010999534964565071959441411308944225311279296875f,0.18019993789047017429538755095563828945159912109375f,0.0633000393033018582400472951121628284454345703125f,0.7375000369548796452789929389837197959423065185546875f,0.51179997093677531694311255705542862415313720703125f,0.9939999449476604187481143526383675634860992431640625f,0.0534999837875371753170838928781449794769287109375f,-0.7064999795183541930754245186108164489269256591796875f,-0.952799974095821511355097754858434200286865234375f,0.076299990069866208131088569643907248973846435546875f,-0.4574000016808508206622718716971576213836669921875f,-0.315699995572864988702121991082094609737396240234375f,-0.97829994572401091090796398930251598358154296875f,0.161000006288289920775014252285473048686981201171875f,-0.13940000218749037941279311780817806720733642578125f,-0.31439999712109578666741072083823382854461669921875f,0.98439996344447155962598117184825241565704345703125f,0.19789997375607537577479888568632304668426513671875f,0.8277999949574468274704486248083412647247314453125f,0.921999983966351077668832658673636615276336669921875f,1.4590999331712719566667146864347159862518310546875f,-0.2275000010430812125150623614899814128875732421875f,0.72059996279478166769649760681204497814178466796875f,-0.0695000140666959254076573415659368038177490234375f,0.381799989712238385664022644050419330596923828125f,-0.889099967592954953943262808024883270263671875f,0.20170000122785580742856836877763271331787109375f,1.17279997588396067698113256483338773250579833984375f,-0.12400000846385950126204988919198513031005859375f,-0.46259999183714395343258729553781449794769287109375f,0.36120003859624272291029001280548982322216033935546875f,-0.26780001008212561686150365858338773250579833984375f,1.16020001111105042213011984131298959255218505859375f,0.22289997917637227597964511005557142198085784912109375f,0.629200013074279862479443181655369699001312255859375f,0.4281000009790061344716605162830092012882232666015625f,-0.345099972465634063922834684490226209163665771484375f,0.853199977728725134085152603802271187305450439453125f,0.64059994446635215581409283913671970367431640625f,0.002099991710483806173215270973742008209228515625f,1.02160003926754061609472046256996691226959228515625f,0.777899994617700318855213481583632528781890869140625f,-0.453400031065940734009700463502667844295501708984375f,-0.0985999655291440735283003959921188652515411376953125f,-0.9104999790787697833138736314140260219573974609375f,-0.2873999982163308875016127785784192383289337158203125f,0.177399998402595571178608224727213382720947265625f,-0.09460000493526432574498130634310655295848846435546875f,0.10400003334879859284711756117758341133594512939453125f,-0.21729998428821650779951824006275273859500885009765625f,-0.41209999258592677762180755962617695331573486328125f,-0.690699994492530056078294364851899445056915283203125f,-1.8952999699234960306881703218095935881137847900390625f,0.568899939811231103448108115117065608501434326171875f,1.61029991531074045951754669658839702606201171875f,0.084300006401537785905020427890121936798095703125f,1.2729999932721256516998664665152318775653839111328125f,-0.94699997651576950374874286353588104248046875f,-0.77929996959567071623808942604227922856807708740234375f,0.0805999890208237701472171465866267681121826171875f,-0.7706000252306457465323319411254487931728363037109375f,0.419399998462200329640836571343243122100830078125f,1.0562999784842135309048671842901967465877532958984375f,-0.5670999825835227881043465458787977695465087890625f,1.0776999595157803479850144867668859660625457763671875f,-1.088099968848377674390803804271854460239410400390625f,0.43670000614523851734105619470938108861446380615234375f,-0.93229999991580758678111351400730200111865997314453125f,-0.9066999942988160920975815315614454448223114013671875f,0.28539998709261471532272480544634163379669189453125f,-0.951799991351366170277970013557933270931243896484375f,1.948999939695001071271462933509610593318939208984375f,0.13170005367994264133812976069748401641845703125f,0.5866999752998349748622786137275397777557373046875f,-0.24300001692771910910551014239899814128875732421875f,1.068699968206882289223358384333550930023193359375f,-0.3298999663650992175689680152572691440582275390625f,0.615399990296364052255739807151257991790771484375f,0.23999998420476931215716831502504646778106689453125f,0.0407999811887744812111122882924973964691162109375f,0.46319998957514763038290084296022541821002960205078125f,0.9302999812483785380123890718095935881137847900390625f,-0.32119996775984793746516743340180255472660064697265625f,0.627800055605173046302525108330883085727691650390625f,-0.35150000112503743121550314754131250083446502685546875f,-0.935200026459246824828142052865587174892425537109375f,-0.12740000313371424578434698560158722102642059326171875f,-0.23450001809000997621268425064045004546642303466796875f,-0.670399973774701063433667513891123235225677490234375f,-1.89720002202987725326011059223674237728118896484375f,0.05069999903738453550516851464635692536830902099609375f,0.43830000234842270145918519119732081890106201171875f,0.72030001581907276975158538334653712809085845947265625f,1.070000001341103512686458998359739780426025390625f,0.9940999906897551863238504665787331759929656982421875f,-0.184000066578388299376456416212022304534912109375f,0.24279999402612428838210689718835055828094482421875f,0.3108000074148176139487986802123486995697021484375f,-0.47049995568394731293437871499918401241302490234375f,-0.591399967154860650708769753691740334033966064453125f,-1.20829996539354311835268163122236728668212890625f,0.373899992114305579349320396431721746921539306640625f,0.2178999638654290915695810326724313199520111083984375f,-1.306100012356042672223566114553250372409820556640625f,-0.79669997209608478527798069990240037441253662109375f,-0.1461000210918490527234325782046653330326080322265625f,1.7029999802708619682789503713138401508331298828125f,1.7160000259950749512682932618190534412860870361328125f,0.93599999898672070486327356775291264057159423828125f,-0.72539992502927841400151010020636022090911865234375f,-0.19069999717473962164149270392954349517822265625f,0.426599920728802484148900475702248513698577880859375f,0.446199995625018885903045884333550930023193359375f,0.3679000396937126549090635307948105037212371826171875f,-0.394999984204769560847125831060111522674560546875f,-0.83849998751282672770912540727294981479644775390625f,-0.770599946105480881897165090776979923248291015625f,0.008699974316358716208696932881139218807220458984375f,0.26919998409152068319372119731269776821136474609375f,0.9661000006772575421365445436094887554645538330078125f,-0.16130000050067871963932475409819744527339935302734375f,-1.35340000722408237976424061344005167484283447265625f,0.28600002806261126675479999903473071753978729248046875f,1.859199975746870681092559607350267469882965087890625f,-0.1899999749660512549098712042905390262603759765625f,-0.49480001602768819424227331182919442653656005859375f,-0.75219993232488757683995572733692824840545654296875f,-1.8418999890744682801368981017731130123138427734375f,1.2397999795198444417110295034945011138916015625f,0.085600026273727536363367107696831226348876953125f,0.799799986970423759657933260314166545867919921875f,0.7739999812692401892633142779232002794742584228515625f,0.2003999775186182663588851937674917280673980712890625f,0.018300016783923102325903897735770442523062229156494140625f,-0.85739998894035807097679935395717620849609375f,-0.1975000104308128057173377101207734085619449615478515625f,1.5951999912738796805200536255142651498317718505859375f,-0.612099965372681964481671457178890705108642578125f,-0.61609993144273733634008749504573643207550048828125f,0.534499961316586169601805522688664495944976806640625f,1.247399970239400968097243094234727323055267333984375f,0.224399996711313942654442143975757062435150146484375f,-0.617500025928020335186374722979962825775146484375f,0.40619996995776885828632885022670961916446685791015625f,1.13470000414848382064292309223674237728118896484375f,-0.12909997746497425819001136915176175534725189208984375f,-0.6004999731183051636662639793939888477325439453125f,-0.2776000000700353798066544186440296471118927001953125f,0.52920001258999127546900353991077281534671783447265625f,0.053599983932078265613796475008712150156497955322265625f,-1.3758000061854718243381512365886010229587554931640625f,0.85839997712373783400607862859033048152923583984375f,1.3986999918624756222840233021997846662998199462890625f,-2.0553999067008508205844918848015367984771728515625f,-0.61389998812824497775864074355922639369964599609375f,0.1285999448537840095241335802711546421051025390625f,0.7636999841138718547739472342072986066341400146484375f,-0.6417000197052953325282942387275397777557373046875f,-0.0876000361680986383561275943065993487834930419921875f,0.3179999856352804243897480773739516735076904296875f,-0.228099985891580647745513488189317286014556884765625f,-1.00039995572566997594776694313623011112213134765625f,0.6964999862238767658340066191158257424831390380859375f,-0.2774000243306158353107093716971576213836669921875f,-0.2203000236794350197300218496820889413356781005859375f,0.1325999728195370597205737794865854084491729736328125f,-1.0280999881267547380048199556767940521240234375f,-0.3109999785535031424643648279015906155109405517578125f,0.50769998546242722792243284857249818742275238037109375f,-0.45819998815953777437925964477472007274627685546875f,-1.1956999766111373606491952159558422863483428955078125f,0.2432000188559289011180908346432261168956756591796875f,1.29699994462728529498463103664107620716094970703125f,1.1996999451398853153705204022116959095001220703125f,-0.98249995753169105938695793156512081623077392578125f,0.87149999594688409843001863919198513031005859375f,0.627100018346309351358058847836218774318695068359375f,0.5768999638915062178057269193232059478759765625f,-1.38449996444582978938342421315610408782958984375f,0.97989996770620368948812028975225985050201416015625f,1.2495999724507331762879402958787977695465087890625f,0.2845999796032909756604567519389092922210693359375f,0.5741999909833073179044049538788385689258575439453125f,0.597499994859099547994674139772541821002960205078125f,-0.2607999643877152440296640634187497198581695556640625f,-0.6725999781280760370094640165916644036769866943359375f,-0.7273999860346316292947221882059238851070404052734375f,0.7552999928712840205236034307745285332202911376953125f,-0.4455999683454636706159135428606532514095306396484375f,-1.024799987566471060063122422434389591217041015625f,-0.41039999241977920263479973073117434978485107421875f,-0.529799984958767566922688274644315242767333984375f,-0.7148000101789826477016731587355025112628936767578125f,-1.0089000189363945825249402332701720297336578369140625f,-0.0327000118374830284206922215162194333970546722412109375f,-0.70019998743683087294442657366744242608547210693359375f,-0.5025000010058278121505281887948513031005859375f,-0.48380000579059079246491137382690794765949249267578125f,-0.27460001577585957388549786628573201596736907958984375f,0.844400010681152224378820392303168773651123046875f,-0.83279999495744760906745796091854572296142578125f,-0.756900026141106962995763751678168773651123046875f,-0.0789000013530254395988094984204508364200592041015625f,0.0458000046581028552594716529711149632930755615234375f,-1.6506999046385300022876663206261582672595977783203125f,-0.63059999770075070291142083078739233314990997314453125f,-0.39889998928308489922756052692420780658721923828125f,-0.0555000051558014462216306128539144992828369140625f,-0.57980002452134993262689022230915725231170654296875f,0.4195999409720305717286237268126569688320159912109375f,-0.2317999889671804591984027865692041814327239990234375f,-0.1589999931901691088143024899181909859180450439453125f,0.18399995876848773690426241955719888210296630859375f,0.833399941062927585022634957567788660526275634765625f,-0.09320000425279129085964768819394521415233612060546875f,0.1687999999046325694251891036401502788066864013671875f,0.96879997040033316579865640960633754730224609375f,-0.13439998177289991243554823086014948785305023193359375f,0.40240001039803008353601398994214832782745361328125f,1.144799953889847454746586663532070815563201904296875f,-0.7532999683737757745660701402812264859676361083984375f,-0.8862999863445757409152747641201131045818328857421875f,0.3701000015228983475168433869839645922183990478515625f,0.465799987298250339762262228759936988353729248046875f,0.8926999488055713527501211501657962799072265625f,-0.13839999575018924193869906957843340933322906494140625f,0.4816999601751579263009261921979486942291259765625f,0.1070000458061686199329187729745171964168548583984375f,0.9615999460607767002073842377285473048686981201171875f,-0.101599998661875912109309183506411500275135040283203125f,0.37349996944517016572717693634331226348876953125f,0.52960001149177504675691352531430311501026153564453125f,-0.11380000977665194472621124077704735100269317626953125f,0.81539995371401374768538516946136951446533203125f,1.132299978923798189356375587522052228450775146484375f,-0.010299994845688609235168087252532131969928741455078125f,0.67710000940561254623162312782369554042816162109375f,-0.5213000069081783482971559351426549255847930908203125f,-0.43549997675046248080121813472942449152469635009765625f,0.54590000130087090735031551957945339381694793701171875f,0.19860001816749528469330243751755915582180023193359375f,0.9483000111959876132772251366986893117427825927734375f,1.767499975711106063869237914332188665866851806640625f,0.041900009414553907038936131357331760227680206298828125f,-0.0416000004127621281213578185997903347015380859375f,0.6848999855130906144751179454033263027667999267578125f,0.64279999011456923074803171402891166508197784423828125f,0.652800007437169593771386644220910966396331787109375f,0.5056000039219856745376091566868126392364501953125f,0.1604000034838914956480948603712022304534912109375f,1.10279995859861390528067204286344349384307861328125f,-0.3784000200390806867289938963949680328369140625f,-0.41889996886849445445477613247931003570556640625f,-0.830399963176251532104288344271481037139892578125f,-0.379599977591633663820402944111265242099761962890625f,0.4964999774508183083554513359558768570423126220703125f,-1.1679999692440030667484052173676900565624237060546875f,1.52309996376335643475385950296185910701751708984375f,1.3391999821357425393131279633962549269199371337890625f,-0.828099953854084969151472250814549624919891357421875f,-0.12749999828636671139747704728506505489349365234375f,0.4598000032126901004403407569043338298797607421875f,-0.25669995451271543362992133552324958145618438720703125f,0.75139999338090379676913244111347012221813201904296875f,-0.2433999927699563858851661279913969337940216064453125f,-0.066799992394447116339506465010344982147216796875f,-0.26499999567866316940722981598810292780399322509765625f,0.831599939131737908581953888642601668834686279296875f,0.0537999955832957343915268211276270449161529541015625f,-0.168799968090654228802804937004111707210540771484375f,0.71120000084042533927686235983856022357940673828125f,-0.8855999845504758383185617276467382907867431640625f,0.6015999760679899832638284351560287177562713623046875f,-0.32029999048709900666409566838410682976245880126953125f,-0.584100015872716316067680963897146284580230712890625f,0.10070002882108008979145097328000701963901519775390625f,-0.861299989771842522401357200578786432743072509765625f,-0.132099971109628899768040355411358177661895751953125f,1.0845999785602093989922423133975826203823089599609375f,0.024099989236890717592132205027155578136444091796875f,-0.51979999710321400652190959590370766818523406982421875f,-0.27339999146610460822870436459197662770748138427734375f,-0.53920000271797174917765005375258624553680419921875f,-0.2007999931097028678550486802123486995697021484375f,0.407499995604157394524236224242486059665679931640625f,-1.3800000568479300344648663667612709105014801025390625f,0.625300028708577482206010245135985314846038818359375f,0.27979997482597784852487166062928736209869384765625f,-1.2295999896615743551819832646287977695465087890625f,0.3875000188499690967347532932762987911701202392578125f,-1.0823000076644120337476806525955908000469207763671875f,-0.7701000183612112248709991035866551101207733154296875f,0.178199982161820258408368999880622141063213348388671875f,1.3474000466823579102992880507372319698333740234375f,0.35879995058178881794930248361197300255298614501953125f,0.3886000075876712145372948725707828998565673828125f,0.467499982006847669691040891848388127982616424560546875f,-0.74850001530349252742979615504737012088298797607421875f,0.50530002151876718397005561200785450637340545654296875f,-0.2947000005163251667994472882128320634365081787109375f,0.1771999988771975065304786767228506505489349365234375f,0.75589999917745576141214769450016319751739501953125f,0.5662000550061463410855822075973264873027801513671875f,-1.7677000148177146598982290015555918216705322265625f,0.24479999300539478923610658966936171054840087890625f,-0.55100000047683739268222780083306133747100830078125f,0.22279997901320436337613273281021974980831146240234375f,0.303700003094226123057097765922662802040576934814453125f,-0.7385999811008574500448276012321002781391143798828125f,0.39809996935129188244673059671185910701751708984375f,-1.7638000048220163762380252592265605926513671875f,0.8220000140294434487486796570010483264923095703125f,-0.352200002360344210927678432199172675609588623046875f,-0.0755999735236165815166486936504952609539031982421875f,1.1266000213116409423008690282586030662059783935546875f,-0.24760001029595735122512678572093136608600616455078125f,0.174999999813735540588055528132827021181583404541015625f,-0.22650001328811064649215722965891472995281219482421875f,-0.008299990390241263060744358881493099033832550048828125f,0.098500020593405235302952860365621745586395263671875f,-0.0724999360367650780290205148048698902130126953125f,0.40590002704262773125520880057592876255512237548828125f,0.259400038993359505212765725445933640003204345703125f,1.450100073458255423020091257058084011077880859375f,1.2368000537313523778948365361429750919342041015625f,-0.73030002190992193700225243446766398847103118896484375f,0.9583000343300398515822280387510545551776885986328125f,-1.0126000562846664276861474718316458165645599365234375f,-1.23690001217126965826764717348851263523101806640625f,-0.041399982279538338758584359311498701572418212890625f,-1.0225000025331976161169222905300557613372802734375f,-0.00160001706480983330038725398480892181396484375f,-1.08890002355575621351135851000435650348663330078125f,0.276200048375130791100673377513885498046875f,0.2573000087261192447840585373342037200927734375f,-0.10069999696984888171158445402397774159908294677734375f,0.271499979816376801888822001274093054234981536865234375f,-0.44669998157694890128510678550810553133487701416015625f,-0.551800022792816147187977549037896096706390380859375f,-0.1337999923795460954778491213801316916942596435546875f,-0.79560001595467311208409455502987839281558990478515625f,0.39150002999603772391168376998393796384334564208984375f,1.4386000283375384167783295197295956313610076904296875f,-0.9476999956138438818697977694682776927947998046875f,-0.063300010916590654375113444984890520572662353515625f,-1.0710999995410441787413446945720352232456207275390625f,-0.18670004570782250397087409510277211666107177734375f,-0.3202999757349491716240663663484156131744384765625f,-0.1914999754577866308835609743255190551280975341796875f,0.224300005656480561100352133507840335369110107421875f,0.6854999964386221389389675096026621758937835693359375f,0.719400037056207697361287500825710594654083251953125f,-0.308199988979101224817469528716173954308032989501953125f,0.086600029879808193555845718947239220142364501953125f,-0.46380001827031358541120198424323461949825286865234375f,0.87660002168416983892029747948981821537017822265625f,-0.092100007095933467127224503201432526111602783203125f,0.73300001547485627018119203057722188532352447509765625f,-0.583300021738559326678341676597483456134796142578125f,0.0827999409779909267825814822572283446788787841796875f,-0.09549996637553015421673308082972653210163116455078125f,-0.250800015684962129824953080969862639904022216796875f,0.57020000066161191654146023211069405078887939453125f,-0.306400015136599090936897482606582343578338623046875f,-0.2577999942123891941037072683684527873992919921875f,-1.2572000036269421752166408623452298343181610107421875f,0.87880000884532893934419917059130966663360595703125f,-0.8149999936670060396437520466861315071582794189453125f,-0.4539000086545943446481032879091799259185791015625f,0.634700009214878235752621549181640148162841796875f,1.55490000495910596356452515465207397937774658203125f,1.371900013959407527153189221280626952648162841796875f,0.96400001069903407113770299474708735942840576171875f,-0.0872999386906627705684513784945011138916015625f,1.22730000589489929296860282192938029766082763671875f,0.2625000131130210689889281638897955417633056640625f,-0.309900044894219295343873454839922487735748291015625f,-0.0428999655604371810113661922514438629150390625f,-1.74590005356669397684754585498012602329254150390625f,-1.08069999687671725041582249104976654052734375f,-0.792599993848799755369327613152563571929931640625f,-0.6117999730229382038260155241005122661590576171875f,0.53269999917149579715669460711069405078887939453125f,-0.0800999852061270445346963242627680301666259765625f,0.1430000178217891715348741854541003704071044921875f,0.4155000434517859275729279033839702606201171875f,0.219900016060471603651649274979718029499053955078125f,-0.822200003179907046302332673803903162479400634765625f,-0.2323999912500382958313593917409889400005340576171875f,-0.50880000169277195265493674014578573405742645263671875f,-0.4518999776750802777058879655669443309307098388671875f,-0.3473000147610907362150101107545197010040283203125f,-0.2898999759763487560348949045874178409576416015625f,1.298300034832954974461927122320048511028289794921875f,-1.2784999954849478598362111370079219341278076171875f,1.024999978765845032313563933712430298328399658203125f,-1.1182999741852288622112610028125345706939697265625f,0.082499983385205144514884523232467472553253173828125f,-0.9047999874919649965221424281480722129344940185546875f,-0.65680000252723669262877592700533568859100341796875f,0.0524000280559062669993863892159424722194671630859375f,0.15050000463426105312692016013897955417633056640625f,-0.433799994689226142696725219138897955417633056640625f,-0.428700008440017654720577411353588104248046875f,0.559799987044930613677706787711940705776214599609375f,-0.696399982428550945456890985951758921146392822265625f,-0.19479999710321427297543550594127736985683441162109375f,-0.44740001032352427756677570869214832782745361328125f,-0.013800037530064201973800663836300373077392578125f,-0.7426999674320224897883235826157033443450927734375f,-1.333200023400783695848303977982141077518463134765625f,0.172400002574921273890140582807362079620361328125f,0.416799975928664689917013674858026206493377685546875f,0.09410003747940010043748770840466022491455078125f,-0.040000019595026525820458118687383830547332763671875f,0.588999978363514031798331416212022304534912109375f,0.79279999346733109888418766786344349384307861328125f,-0.7188999866008760619706663419492542743682861328125f,0.03579999273717415775308836600743234157562255859375f,0.1970999852657318029969246708787977695465087890625f,0.40029999942779515009760871180333197116851806640625f,-0.39990000026524052945120502045028842985630035400390625f,1.0165000287666916545248341208207421004772186279296875f,-0.565600046390295485565502531244419515132904052734375f,0.703100005449354359399194436264224350452423095703125f,0.574700032907724978059604836744256317615509033203125f,-0.6321999572098253583618543416378088295459747314453125f,0.14750004783272796515802838257513940334320068359375f,-0.4342999985039224242200361914001405239105224609375f,-1.43710007944107065469552253489382565021514892578125f,-0.73869998579025253349072954733856022357940673828125f,0.20390000537633934385439715697430074214935302734375f,-0.0138999252080918012097754399292171001434326171875f,-0.749800066095591688508648076094686985015869140625f,-1.923400028681754747594823129475116729736328125f,0.5376000257372854918003213242627680301666259765625f,0.618899987494945236221610684879124164581298828125f,0.0559000132590530274256934717413969337940216064453125f,0.28600001459568724637705372515483759343624114990234375f,-0.40329999312832931490646615202422253787517547607421875f,-0.052599997164308354147266300060437060892581939697265625f,-0.6712999963469805653204502959852106869220733642578125f,-1.295200023609400119539714069105684757232666015625f,-0.0035000122487540696880614632391370832920074462890625f,1.0596999324738971548498511765501461923122406005859375f,-0.27530000404715526673982139982399530708789825439453125f,1.015900042167306782658897645887918770313262939453125f,-1.07829997262060661711302600451745092868804931640625f,-0.1490000129342081702787936592358164489269256591796875f,-1.355900003647803941930760629475116729736328125f,-0.322799980652332063613130230805836617946624755859375f,-0.0175000293552878272151929195388220250606536865234375f,0.503699990633129868200512646581046283245086669921875f,0.613800007504224698351436018128879368305206298828125f,-0.26229998905658735708357198745943605899810791015625f,1.3391999827131628730825241291313432157039642333984375f,-0.145000012665986777182780542716500349342823028564453125f,1.520199994105100760322102360078133642673492431640625f,-0.03969997557550708489060298234107904136180877685546875f,-0.33919995369315170119506319679203443229198455810546875f,0.5275999488472942200445459093316458165645599365234375f,-1.845199987399578134272815077565610408782958984375f,-0.480699987787008620188089480507187545299530029296875f,0.772599948921799839496316053555347025394439697265625f,-0.23879995162487066551193493069149553775787353515625f,-0.2214000119328500826298977699480019509792327880859375f,0.0273999733686451918401871807873249053955078125f,-0.601399974977970597223020376986823976039886474609375f,0.9153000119447707305653239018283784389495849609375f,-0.629399981400370567286017831065692007541656494140625f,-0.4328999950647354211241690791212022304534912109375f,-0.2740000162124633931171047152020037174224853515625f,0.5305999770998948150690921465866267681121826171875f,0.22209996246695506982860024436376988887786865234375f,1.3467000217914577575584189617075026035308837890625f,1.3806000338733195320628510671667754650115966796875f,-0.96610001760870201081843333668075501918792724609375f,-1.244299996287375531522911842330358922481536865234375f,2.281500095617027046301927839522249996662139892578125f,0.4768999703548846280654061047243885695934295654296875f,-1.383700030046701723307478459901176393032073974609375f,0.1813999705076216883981032879091799259185791015625f,-0.91140005231499809923434440861456096172332763671875f,-0.53870006655454627519929999834857881069183349609375f,0.937400029844045690197162912227213382720947265625f,0.1805999452114104286692963796667754650115966796875f,-1.4173000349521629459559335373342037200927734375f,-1.4921999773263934230271843262016773223876953125f,-0.20420001285076150310260345577262341976165771484375f,-1.0533999772727486998746826429851353168487548828125f,0.65259999481737607229803188602090813219547271728515625f,-0.19440002230405806216140263131819665431976318359375f,-0.307699985685944366053945486783050000667572021484375f,-0.11329999150782821271121747486176900565624237060546875f,-0.28900001740455649734684584473143331706523895263671875f,-1.2643000385507947580521204145043157041072845458984375f,1.527500020358711463330791957559995353221893310546875f,2.037900066962837986039858151343651115894317626953125f,-0.187500017881393699070713410037569701671600341796875f,-0.261299963247775934860328561626374721527099609375f,0.29689995748400743735828655189834535121917724609375f,-0.2083000811755659764656911647762171924114227294921875f,-0.019899995943904347228681217529810965061187744140625f,0.707300045531988441638304720981977880001068115234375f,-0.72859997510313956325944673153571784496307373046875f,0.6403999983429908837706534541212022304534912109375f,1.01579999355673766814334157970733940601348876953125f,-0.3616999924361703477870833012275397777557373046875f,0.39560000749826418342536271666176617145538330078125f,-1.08799999725818619111805674037896096706390380859375f,-0.35459999387115226365807529873563908040523529052734375f,1.3848999876178798018599991337396204471588134765625f,-0.7667999934747811163759934061090461909770965576171875f,-0.87530001451522110311742608246277086436748504638671875f,-0.750800073203444551239726933999918401241302490234375f,0.4746000183463099375558158499188721179962158203125f,0.30699999946355749358417597250081598758697509765625f,0.69159998115301224430595539161004126071929931640625f,-0.8316000137865537933379300739034079015254974365234375f,-0.7938999573946006194091751240193843841552734375f,-0.309200044888257696840128119220025837421417236328125f,0.581099993133545211776436190120875835418701171875f,-0.25290000337213280889869793099933303892612457275390625f,0.00010000871270909872379206717596389353275299072265625f,-1.143000034119933783216538358828984200954437255859375f,0.14060004177093554655897378324880264699459075927734375f,0.94439998847842243634431724785827100276947021484375f,1.916799986582993664541163525427691638469696044921875f,0.833200002315640286809639292187057435512542724609375f,-0.78579997850656457814011446316726505756378173828125f,-0.0207999263525007194175486802123486995697021484375f,-0.45720001979470215047740566660650074481964111328125f,-1.0124999871850004495854591368697583675384521484375f,-0.674900010025500929344843825674615800380706787109375f,-0.619399971938133120374914142303168773651123046875f,0.97499999687075611376485539949499070644378662109375f,0.128800029706955143637969740666449069976806640625f,-0.632200019049644623692074674181640148162841796875f,-0.393299963903427141076463158242404460906982421875f,-0.47430001411288980062153086691978387534618377685546875f,-1.1907000202901663055143899327958934009075164794921875f,0.790199991888552943919421522878110408782958984375f,0.82700000058114486733984449529089033603668212890625f,0.6466000036537646877832230529747903347015380859375f,-1.48579999445080712661138022667728364467620849609375f,0.0784000122159718149106311102514155209064483642578125f,1.503599972495436087882580977748148143291473388671875f,-1.585099995338916212261892724200151860713958740234375f,-0.242400018072128187185398928704671561717987060546875f,-0.7577999927967786764071433935896493494510650634765625f,0.55150001472234766453084375825710594654083251953125f,-0.666300003182887845554205341613851487636566162109375f,-0.380800008085369956489785181474871933460235595703125f,-0.0739999797046184681903469027020037174224853515625f,0.486500000491738138208575037424452602863311767578125f,-0.88080000458359730686197508475743234157562255859375f,0.45540000931397084382723505768808536231517791748046875f,-0.054599960269033853688114277247223071753978729248046875f,0.63329998016431920948576816954300738871097564697265625f,-0.230200010123848830545512100798077881336212158203125f,-0.45859999752938751527153726783581078052520751953125f,0.94019998538792126652197111980058252811431884765625f,-1.241599976906180291535974902217276394367218017578125f,-0.879200005772709847207124767010100185871124267578125f,0.2831999736309054327421108609996736049652099609375f,0.448199978995323267128014776972122490406036376953125f,-0.164999980032444426569782081060111522674560546875f,-0.78900000667572012247319435118697583675384521484375f,0.7148999906539916793235533987171947956085205078125f,0.170200007888674509359816511278040707111358642578125f,-1.170099983119964548450298025272786617279052734375f,-0.074600000986456915796907196636311709880828857421875f,0.54269999663829793234981480054557323455810546875f,0.36860004171133109451830023317597806453704833984375f,-0.41560006084442224771891005730140022933483123779296875f,-1.002299996060132425412803058861754834651947021484375f,-0.01079999951720267414856380128185264766216278076171875f,0.5088999938279386014983174391090869903564453125f,2.162700003492831424267706097452901303768157958984375f,0.976999991342424767282182074268348515033721923828125f,0.32549999636411630632437663734890520572662353515625f,-0.547299970579147920801688087522052228450775146484375f,0.97550005701184339113041232849354855716228485107421875f,0.107100027436018319093591344426386058330535888671875f,0.775600020611286300464826126699335873126983642578125f,-0.210499955162405161246397256036289036273956298828125f,0.0650000216066839442419222905300557613372802734375f,-0.4261000025212762221116236105444841086864471435546875f,0.3614000289201741988875937749980948865413665771484375f,0.5016000285014514670223206849186681210994720458984375f,-0.77110000550150825660011832951568067073822021484375f,-0.4640999870389699299977337432210333645343780517578125f,-0.6650000222027305785132966775563545525074005126953125f,0.14600001470744583542682448751293122768402099609375f,0.14330000219941130268352935672737658023834228515625f,-1.2676000015974040469046713042189367115497589111328125f,0.33040000475048980632664097356610000133514404296875f,0.13550000381469740062811979441903531551361083984375f,-0.175899989417195090624090880737639963626861572265625f,-0.20060004422962685044495856345747597515583038330078125f,0.4448000252664090492515924779581837356090545654296875f,0.012900015702844047638109259423799812793731689453125f,0.072299982574582255523409912711940705776214599609375f,0.2774999696016309247426079309661872684955596923828125f,0.453700015965104175297284427870181389153003692626953125f,0.1478000052019954324578066007234156131744384765625f,0.09060000150054692280576773555367253720760345458984375f,-0.66230000306367831086618025437928736209869384765625f,0.5141999908715482181520428639487363398075103759765625f,0.285100021192431274386080986005254089832305908203125f,0.4695999891400337578062362808850593864917755126953125f,0.07409999530911459686421949299983680248260498046875f,0.040799991396069490434683757484890520572662353515625f,-0.980999977603554729199686335050500929355621337890625f,-1.03479996670484553078495082445442676544189453125f,0.245999983489513684986604857840575277805328369140625f,-0.46219995445311046200487226087716408073902130126953125f,-0.508900007387996122787399144726805388927459716796875f,-1.4829999722242359627699670454603619873523712158203125f,0.166799984720349403488626194302923977375030517578125f,-0.450999957412481222718270146287977695465087890625f,-0.2082999977663162471941404874087311327457427978515625f,-0.15200000117719170145136331484536640346050262451171875f,-0.1134000055477024648808992424164898693561553955078125f,0.315500014916061832792593122576363384723663330078125f,-1.259300000813601716487255544052459299564361572265625f,-0.82769993815124021097062723129056394100189208984375f,0.2441999599516382346564569161273539066314697265625f,1.00110000743865956707168152206577360630035400390625f,-0.61579999232739179859663636307232081890106201171875f,-0.37819997696503981909899039237643592059612274169921875f,-1.14950003051385341024115405161865055561065673828125f,-0.2611000700049099787491968527319841086864471435546875f,0.0736999544978136356121467542834579944610595703125f,0.2171999667465680516897919005714356899261474609375f,-1.3043000287532802161649669869802892208099365234375f,-0.1924999598413712487143811813439242541790008544921875f,-1.2258000544279818999626741060637868940830230712890625f,-0.8494000412657862764120864085271023213863372802734375f,0.2254000138401988095893102581612765789031982421875f,0.1215999960541722657580976374447345733642578125f,-0.9481999647647143536488556492258794605731964111328125f,-0.07359998419284874415779995615594089031219482421875f,-0.94469997337758526700923766838968731462955474853515625f,-0.32989997590184128029022758710198104381561279296875f,-0.353499996751546863293924616300500929355621337890625f,-0.04069999356120879152598490691161714494228363037109375f,-0.249000036031007787595825675452942959964275360107421875f,1.0470999936476343972202585064223967492580413818359375f,0.0516999742381273053037915587992756627500057220458984375f,-0.5604000007271761063520898460410535335540771484375f,-0.418899976095557402544500291696749627590179443359375f,0.83500002980232235216817571199499070644378662109375f,-1.20130000072419651502286797040142118930816650390625f,0.0618000033095480461042825481854379177093505859375f,-0.1500000046566130063041555331437848508358001708984375f,-0.4002000142216683986617908885818906128406524658203125f,0.2224000058531760970481627737171947956085205078125f,-0.4524999720603233299698331393301486968994140625f,1.35939994839429889594839551136828958988189697265625f,0.287099977070093270725692491396330296993255615234375f,1.41699999454617486804863801808096468448638916015625f,-0.34119998281002050788401902536861598491668701171875f,-0.40779999156743318433626654950785450637340545654296875f,1.0708999632470312945997648057527840137481689453125f,-0.2262000254645946117904031780199147760868072509765625f,-0.05039997312277544327940859147929586470127105712890625f,1.389999957829714194446069086552597582340240478515625f,-0.657899960941076500375856994651257991790771484375f,0.0517999552905561433391312675666995346546173095703125f,0.7757999893471601016159411301487125456333160400390625f,0.42280000173747545044733442409778945147991180419921875f,0.5150000124797224909656279123737476766109466552734375f,0.2432000061154366843396701369783841073513031005859375f,-0.13500000536441802978515625f,-0.576400008952617781687877140939235687255859375f,0.2346000132799144211048769648186862468719482421875f,1.825699982459842996007637339062057435512542724609375f,0.19909995909929367741142414161004126071929931640625f,-0.341199976253510062207396913436241447925567626953125f,0.172599976265430665733902060310356318950653076171875f,0.00680002938657919830944820205331780016422271728515625f,-2.090299946379662099360530191916041076183319091796875f,-1.4236000021487473787829003413207828998565673828125f,0.55429995365142925578538779518567025661468505859375f,1.38210001477003086023387368186376988887786865234375f,-0.12180002435743819688696021330542862415313720703125f,-0.412600032629072632772704309900291264057159423828125f,-0.05589998040199350981538373162038624286651611328125f,-0.4656999590650199305486012235633097589015960693359375f,-0.4545999821364878901164274793700315058231353759765625f,-0.4548000051125882237101905047893524169921875f,0.501600006335973347404433297924697399139404296875f,0.347999995172024068068594715441577136516571044921875f,0.67189997134208656603959752828814089298248291015625f,0.659999951571226706192874189582653343677520751953125f,-0.10129996488690462541626402526162564754486083984375f,0.1327999835580591092565327926422469317913055419921875f,-0.70609993151724470550334444851614534854888916015625f,-1.12989996800422698441934699076227843761444091796875f,0.185399999907613466376687938463874161243438720703125f,1.63039998046159695377355092205107212066650390625f,0.40079996956586771972297356114722788333892822265625f,-0.604699993792176204721044996404089033603668212890625f,0.543499987065792833362820601905696094036102294921875f,0.21320003368258422238312732588383369147777557373046875f,-0.608699977147579840419666652451269328594207763671875f,0.2311999524116521431693627164349891245365142822265625f,-0.5304999851137395250333383955876342952251434326171875f,0.216600007192790655619063500125776045024394989013671875f,0.0420000023320319382236220917548052966594696044921875f,-0.037700013178587121753793098832829855382442474365234375f,1.8825000533647837652750922643463127315044403076171875f,-0.76739999560862737215671813828521408140659332275390625f,-0.45240000570416405434315265665645711123943328857421875f,-1.172899998957663836307574456441216170787811279296875f,-1.6122999959111208312378948903642594814300537109375f,-0.9213999947965145764783301274292171001434326171875f,-1.74369998310804330543533069430850446224212646484375f,-0.8544000014796850894782664909143932163715362548828125f,0.1979999860450625259744583672727458178997039794921875f,-0.06410000309497132509051198212546296417713165283203125f,-0.3879999689087270109411065277527086436748504638671875f,-0.49620000855177603948931164268287830054759979248046875f,0.6413999973297119794324316899292171001434326171875f,-0.0609000090867282128925808137864805757999420166015625f,0.00619997670054406313511208281852304935455322265625f,0.5090999987363808987339552913908846676349639892578125f,1.2296000482231377315400777661125175654888153076171875f,-2.12289998871311436090536517440341413021087646484375f,-1.1139000402078027551766581382253207266330718994140625f,-1.77849998147785637314655105001293122768402099609375f,-1.7784999958761036342735906146117486059665679931640625f,-0.2206000042200084720889208256267011165618896484375f,0.139199995379149843444110956625081598758697509765625f,1.064799933624268390985889709554612636566162109375f,0.524600001135468119173310697078704833984375f,-0.6566999451994905978580163719016127288341522216796875f,-0.69159999068975441804951742597040720283985137939453125f,0.4734999521970759683853202659520320594310760498046875f,1.542199969652294999633568295394070446491241455078125f,-0.7851999829292299892813389305956661701202392578125f,-1.2164999718815090457013639024808071553707122802734375f,0.710900002381205098345162696205079555511474609375f,0.87270001224726423405542163891368545591831207275390625f,-0.11900001095980439203003697912208735942840576171875f,-0.3313000184379522039535004296340048313140869140625f,-0.60839998423904051438881879221298731863498687744140625f,0.75479999486803972530424289288930594921112060546875f,-0.20079999264404169156961188491550274193286895751953125f,0.52599999868869762309486759477294981479644775390625f,0.0865999939680097696736993384547531604766845703125f,1.37939997059702879056430901982821524143218994140625f,-1.3142999473929410925165939261205494403839111328125f,-1.060999954767525732535204952000640332698822021484375f,-0.13759999537616973697140565491281449794769287109375f,0.8212999680161476856454783046501688659191131591796875f,-0.3238000038906927446902273004525341093540191650390625f,0.2248000018715856640483252704143524169921875f,-0.5322999705791477964567093295045197010040283203125f,-0.8027999626591804105402161439997144043445587158203125f,-1.3458999899387362741975948665640316903591156005859375f,0.076299988207221058900131538393907248973846435546875f,-0.439200044441223180768929523765109479427337646484375f,-0.15119996969699889266536274590180255472660064697265625f,-0.7022999760553243486782548643532209098339080810546875f,-0.139799989540874636251288620769628323614597320556640625f,-1.0015000375583766878406777323107235133647918701171875f,-0.475799980704486757954185804919688962399959564208984375f,0.1192000666439538836272049593389965593814849853515625f,0.79990001345276784405768921715207397937774658203125f,0.5511000057995314449499346665106713771820068359375f,0.018500021114945697231490839840262196958065032958984375f,0.997700042384863206024192550103180110454559326171875f,0.377999993570148740129610587246133945882320404052734375f,0.4286999992012976434807569603435695171356201171875f,0.2379000084385276547749299425049684941768646240234375f,-0.952400004363060048717670724727213382720947265625f,0.6365000080168243812295258976519107818603515625f,0.854200008268654276122333612875081598758697509765625f,0.1154000153303140763227929710410535335540771484375f,-0.8343000086367127199338256104965694248676300048828125f,-0.0193999966740598939196615901892073452472686767578125f,1.1828999653741714137566987119498662650585174560546875f,0.3642000006318084626144582216511480510234832763671875f,-0.191199957887828730918045039288699626922607421875f,0.5028000141799449096424723393283784389495849609375f,1.425900008825957687719210298382677137851715087890625f,0.3495999709606163463604389107786118984222412109375f,-0.5009000096455213935087158461101353168487548828125f,-0.592299970281124021909135990426875650882720947265625f,0.2389999990016224984401560504920780658721923828125f,-0.1362000135064125316830541123636066913604736328125f,-0.5652000037536024645845600389293394982814788818359375f,-0.226599965953827364728567772544920444488525390625f,0.72149998492002520578125768224708735942840576171875f,0.36339997116327271697855394450016319751739501953125f,-0.826499968379736049683970122714526951313018798828125f,0.14000000134110468508197300252504646778106689453125f,-0.50140003231018714924260848420090042054653167724609375f,0.83979998924285137462675265851430594921112060546875f,0.6179000186085701340488185451249592006206512451171875f,-0.1800999706402433009344576930743642151355743408203125f,-0.23950001391768438452345435507595539093017578125f,0.70219998984336751135515442001633346080780029296875f,0.2037999999046327115337362556601874530315399169921875f,-0.51929997183084442013978332397527992725372314453125f,-0.353200008052587666185218040482141077518463134765625f,0.136199985939264411882732019876129925251007080078125f,0.0519999949932099525540252216160297393798828125f,0.3182999999642373012420648592524230480194091796875f,0.75959997453689587221248302739695645868778228759765625f,0.2754999591112137213855248774052597582340240478515625f,0.1708000281646847307026604312341078184545040130615234375f,-0.67080001263022470681107733980752527713775634765625f,-0.6933000018641355399751091681537218391895294189453125f,1.50000006780028360964251987752504646778106689453125f,0.4477000089317562103730097078368999063968658447265625f,-0.3467000334143641726569740058039315044879913330078125f,-0.7874000004142516218053060583770275115966796875f,-0.64010000096410468728436171659268438816070556640625f,-0.39820002243220831150694039024529047310352325439453125f,-0.6369000588491566805515731175546534359455108642578125f,-0.38119998422563072804081230060546658933162689208984375f,-1.22020001451969140049413908855058252811431884765625f,-0.45599998121708640308469284718739800155162811279296875f,-1.08470000921487841338830548920668661594390869140625f,-0.22749999977648249771533528473810292780399322509765625f,0.427599999660253526911901644780300557613372802734375f,-0.2515000020563598059908372306381352245807647705078125f,-1.2048000104397538212452900552307255566120147705078125f,0.5641000353932366095932593452744185924530029296875f,2.12220001543611314076542839757166802883148193359375f,-1.5762999826192842789396308944560587406158447265625f,-0.9757000153541566245252170119783841073513031005859375f,-0.6511000293433664154463258455507457256317138671875f,-0.7362000073596839389011847742949612438678741455078125f,-0.3150999913156031784211563717690296471118927001953125f,0.84139998552054162672675374778918921947479248046875f,-0.3499999839067464701969356610788963735103607177734375f,-1.364200018438696648814811851480044424533843994140625f,-0.070999973952769579454979975707828998565673828125f,0.55490001371353814274556270902394317090511322021484375f,0.57879999334812115563408951857127249240875244140625f,1.202899987949430649081250521703623235225677490234375f,0.2845999761015178819434368051588535308837890625f,-1.4469999820291994385712541770772077143192291259765625f,0.4484000305443995859633332656812854111194610595703125f,0.93310001822710031404994879267178475856781005859375f,-0.4801000170946121414772278512828052043914794921875f,-0.6508000559926028216750637511722743511199951171875f,0.7534000024557112595857688575051724910736083984375f,0.2056000106275082117690544691868126392364501953125f,0.25810001554489137021164424368180334568023681640625f,0.56460001570135298631925024892552755773067474365234375f,0.19629997505694607440318577573634684085845947265625f,-0.1202999900400636423825062593095935881137847900390625f,0.0956000047788025941741807400831021368503570556640625f,-0.551300013613700290449060048558749258518218994140625f,0.6631000000476827693773884675465524196624755859375f,0.44480000872612013296247823745943605899810791015625f,0.41509998729228936298341068322770297527313232421875f,0.3539999978095291766067020944319665431976318359375f,0.531300032538176036922550338204018771648406982421875f,0.41670000171214349737880411339574493467807769775390625f,-0.45260004115998808682519438661984167993068695068359375f,0.7853999950274823671492185894749127328395843505859375f,0.62339999902844400736512397998012602329254150390625f,0.9415999728828674530944908838137052953243255615234375f,0.9564999781027438974234655688633210957050323486328125f,0.26319998614788031598976658642641268670558929443359375f,1.4654000233396888841497229805099777877330780029296875f,0.84959997461140179808580796816386282444000244140625f,-0.1060000300370150527751178515245555900037288665771484375f,-0.322599963077903506558641311130486428737640380859375f,-1.161699992734194442078887732350267469882965087890625f,-1.1037999814830719014224769125576131045818328857421875f,-0.00289997479915626854563015513122081756591796875f,-0.284800004106759985234020859934389591217041015625f,-0.5082000016450880508500631549395620822906494140625f,0.2552999997258194042615286889486014842987060546875f,1.599300004725157986484873617882840335369110107421875f,0.92019997003972431315332869417034089565277099609375f,-0.80270002666413764469410807578242383897304534912109375f,0.805499987125397609588617342524230480194091796875f,1.0804000111579892262625435250811278820037841796875f,0.0707999883413312769420144832110963761806488037109375f,0.4841999932929874628229072186513803899288177490234375f,-1.0496999749422071257498600971302948892116546630859375f,-0.4816000000402329561666192603297531604766845703125f,-0.1567000098891557779978711550938896834850311279296875f,-0.414300007444619922125639277510344982147216796875f,0.242300006118416177969265845604240894317626953125f,-0.412199987924843913589256771956570446491241455078125f,-0.313599969291686875294544734060764312744140625f,-0.091599986703693847545082462602294981479644775390625f,0.13789998850822460241261069313623011112213134765625f,-0.898100003772973831672743472154252231121063232421875f,0.78179998211264578600321328849531710147857666015625f,0.5742000187367199970367437344975769519805908203125f,0.229399993097782139983564775320701301097869873046875f,-1.2110000034570689297908074877341277897357940673828125f,0.648899971625209293080160932731814682483673095703125f,0.02180003281384645941187727657961659133434295654296875f,0.0179999892488122981148990220390260219573974609375f,-0.8090999935209752802478533340035937726497650146484375f,1.2164999456927187981847282571834512054920196533203125f,0.2056000201456245068953876398154534399509429931640625f,0.35050000321865037999913283783826045691967010498046875f,1.3143999592721467006839475288870744407176971435546875f,-1.678099951265007216960611913236789405345916748046875f,-0.098400021827220740533448406495153903961181640625f,-0.77519998352527608176387730054557323455810546875f,-0.138600000509619736277500123833306133747100830078125f,-0.7069000196591022078251853599795140326023101806640625f,0.12829998871386039116515576097299344837665557861328125f,0.0348000073850158742061466909945011138916015625f,-0.3160999854966999311756126189720816910266876220703125f,0.04890002895742640021126135252416133880615234375f,-0.04959999748468398028666825894106295891106128692626953125f,0.332799997676908765331660333686158992350101470947265625f,0.32319999426901357963970440323464572429656982421875f,-0.2004000085502857242403251802898012101650238037109375f,0.0538000021770595193704167513715219683945178985595703125f,0.123500014856457784873100536060519516468048095703125f,0.17370000791847706178572252611047588288784027099609375f,-1.1281000299990180391063177012256346642971038818359375f,0.50119996522665088978243375095189549028873443603515625f,-0.01110003753900468126403211499564349651336669921875f,0.20939999022930833039168874165625311434268951416015625f,-0.4884000113219020278165771742351353168487548828125f,1.1880000008344648865232784373802132904529571533203125f,0.392100029261410298886403325013816356658935546875f,0.433699992048740767813796992413699626922607421875f,-0.57159995947182185016544053723919205367565155029296875f,0.2002000009596347496199086890555918216705322265625f,0.11799999759346235794765789250959642231464385986328125f,0.06020000241249789940667369592119939625263214111328125f,-0.4235999811381103707930151358596049249172210693359375f,0.105299992126226538147193423355929553508758544921875f,-0.301899993470311100995928654810995794832706451416015625f,-0.331199990260600873881458028336055576801300048828125f,0.768799957473575990007930158753879368305206298828125f,1.03639993787407913572451434447430074214935302734375f,-0.20910001173764480864036841012421064078807830810546875f,0.6149999925494196162389926030300557613372802734375f,0.81089999779909849308978664339520037174224853515625f,-0.509999997429549267735637840814888477325439453125f,0.3825999904215338087709596948116086423397064208984375f,-0.8773999976947901568991028398158960044384002685546875f,-0.081600015425681959868597914464771747589111328125f,-1.4715999430060389130403564195148646831512451171875f,0.143599980169534846563550445353030227124691009521484375f,-0.903199960108101773670341572142206132411956787109375f,-0.33969999904483538788468877100967802107334136962890625f,-0.07199998690933007150505318350042216479778289794921875f,-0.42639997609555757573929213322116993367671966552734375f,0.1779999948367476658717123427777551114559173583984375f,-0.74849999019503599129166104830801486968994140625f,1.055699972513318041222873944207094609737396240234375f,0.5203000047922134996269960538484156131744384765625f,0.6061999670147901753125552204437553882598876953125f,-0.841599952542782414610655905562452971935272216796875f,0.17399998596310606302495216368697583675384521484375f,-0.779400002709030648162524812505580484867095947265625f,-1.107499998174607203083041895297355949878692626953125f,-0.8482000196009875647717990432283841073513031005859375f,-0.753699999946355614355297802831046283245086669921875f,0.72099995465576693920439765861374326050281524658203125f,-0.4589000751137728428830087068490684032440185546875f,0.63809999803602657220125138337607495486736297607421875f,-0.819299990308284264983740285970270633697509765625f,0.67769999734610253394606615984230302274227142333984375f,0.85440002796649938687778558232821524143218994140625f,-0.33139997237026597165510111153707839548587799072265625f,-2.220700046348571987664399784989655017852783203125f,0.199699970695375927931536352843977510929107666015625f,1.783299976420401389276548798079602420330047607421875f,-0.217199980492889466177075519226491451263427734375f,-0.50259999772310237631245399825274944305419921875f,-0.37190001399666083869988142396323382854461669921875f,-0.92020002137422540755551381153054535388946533203125f,-0.2890000107362862280524495872668921947479248046875f,1.048999966479838175104077890864573419094085693359375f,0.7587999537110334813405643217265605926513671875f,-0.35059999591261148399468083880492486059665679931640625f,0.85249999493360473223901863093487918376922607421875f,-0.333800020691751964108107131323777139186859130859375f,2.03169999169111203940474297269247472286224365234375f,2.5854000084385280189280820195563137531280517578125f,-1.08720003693103617337101240991614758968353271484375f,-2.555800018776953397292572844889946281909942626953125f,2.0423999697923651552855517365969717502593994140625f,1.6718999897822734990171511526568792760372161865234375f,-1.0701999307751659529230892076157033443450927734375f,-0.0373999946773053704163203292409889400005340576171875f,-0.4681999989628788938489378779195249080657958984375f,-0.3889999807476998139321722192107699811458587646484375f,0.2856999815285217181326515856198966503143310546875f,1.1076000070735805280719432630576193332672119140625f,0.1755999560892587130211950352531857788562774658203125f,0.46939999876022320268731391479377634823322296142578125f,0.4202999600514776279425177563098259270191192626953125f,-1.513599915349484348325859173201024532318115234375f,0.81249995559454024629530977108515799045562744140625f,0.369200015532970216014518882730044424533843994140625f,0.26599994638562218796096203732304275035858154296875f,-1.3639000390529627537716805818490684032440185546875f,-0.44729993630647602032013310235925018787384033203125f,1.41320002816915479826320733991451561450958251953125f,-0.3519000041246425070085024344734847545623779296875f,-1.80819999642968110009633164736442267894744873046875f,-0.07689997909069035131324199028313159942626953125f,1.19679997719526287625058103003539144992828369140625f,-0.054300011467933728681600769050419330596923828125f,1.0866000114396208164890822445158846676349639892578125f,1.2197000197947023281130896066315472126007080078125f,0.1264000375628473449296507169492542743682861328125f,-0.6218999974191188329086799058131873607635498046875f,-1.11539996202141100667404316482134163379669189453125f,-0.6552999574065214716256377869285643100738525390625f,-0.74629999006986580400990760608692653477191925048828125f,0.341399998223781420136901942896656692028045654296875f,0.54730000839084336217155168924364261329174041748046875f,-0.07640002243816923499508675377001054584980010986328125f,-0.1348999613732100311125350344809703528881072998046875f,-1.132600027823448929353844505385495722293853759765625f,0.427399988232553262879065414381329901516437530517578125f,-0.719700034528225618402075269841589033603668212890625f,0.4764999867081642914712347192107699811458587646484375f,0.18859998657703413993402818960021249949932098388671875f,0.151599999313801470091078726909472607076168060302734375f,1.029600020432472007314572692848742008209228515625f,-0.9144999935030941884406274766661226749420166015625f,-0.33350002480298268636715874890796840190887451171875f,-0.6980999836564070193389852647669613361358642578125f,-0.482700011166929954242732492275536060333251953125f,-0.1348000003814693315007389173842966556549072265625f,-0.0910999870240691844713865066296420991420745849609375f,-0.01279995800256816806950155296362936496734619140625f,0.09470002638846608533640392124652862548828125f,0.16800001282989907025466891354881227016448974609375f,-0.561599998587369864111451533972285687923431396484375f,0.57240001330375633870062301866710186004638671875f,-0.452100015626847540062271946226246654987335205078125f,1.4070999780200426076959274723776616156101226806640625f,1.646299988020956561740604229271411895751953125f,0.148999997511506165892569697462022304534912109375f,0.0721000024951994333832772099412977695465087890625f,1.406599974894522375734595698304474353790283203125f,-0.87929998956322652503558856551535427570343017578125f,1.1520999945044518852910186978988349437713623046875f,1.13089998606443398188048377051018178462982177734375f,-0.5599999740719798779764460050500929355621337890625f,1.092299928259850450018575429567135870456695556640625f,-1.1181999482989315453096423880197107791900634765625f,-0.28999999091029149411724574747495353221893310546875f,0.28560001107454180413469657651148736476898193359375f,-0.1921000073939556074265055940486490726470947265625f,-0.120099974179268098595230185310356318950653076171875f,-0.3832999963879581972747701001935638487339019775390625f,0.483099964657426550473928728024475276470184326171875f,-0.6516999756909906960089529093238525092601776123046875f,0.71639995471239126256790541447116993367671966552734375f,-0.346399987047910806747808010186417959630489349365234375f,0.09699999750778076634816216028411872684955596923828125f,0.29009999444484702024737998726777732372283935546875f,1.268300010469555871850388939492404460906982421875f,-0.582199977773428134497635255684144794940948486328125f,0.53330000145435263458892904964159242808818817138671875f,-1.2512000012129542714234275990747846662998199462890625f,0.1757000167697666537236500516883097589015960693359375f,-0.541700015011429503175577337970025837421417236328125f,-1.7656000291049471773163759280578233301639556884765625f,-0.08619998295903175122845141231664456427097320556640625f,1.7834000265955918251847833744250237941741943359375f,-1.25139999978840332772733745514415204524993896484375f,0.068799962428212513287917317938990890979766845703125f,-1.1331000109255313645917340181767940521240234375f,-0.2265999424844977738757734186947345733642578125f,-0.19579999981522588914373272928060032427310943603515625f,-0.46250002339482332303077782853506505489349365234375f,0.24729999613463871099838797817938029766082763671875f,-0.061500005185604056379133908194489777088165283203125f,-0.11180003732144816464000314226723276078701019287109375f,-0.715599964284896916666411925689317286014556884765625f,0.74939995048046126324692295384011231362819671630859375f,0.6450999786496165455673690303228795528411865234375f,1.3375999532431361860318475009989924728870391845703125f,0.0193000025272371811269067620742134749889373779296875f,0.83020001749992378137221749057061970233917236328125f,-0.761299994093179943632776485173963010311126708984375f,-0.08940003273487118207185631035827100276947021484375f,0.1170999793052676096749564749188721179962158203125f,0.397599997499585189331128276535309851169586181640625f,0.20680002480447257084961165674030780792236328125f,1.066399975499510599519226161646656692028045654296875f,0.3964000018745661435559668461792171001434326171875f,0.2852000174254178954669214363093487918376922607421875f,-0.245100005248188868023362374515272676944732666015625f,-0.0094000027835370314477358988369815051555633544921875f,-1.3400000700354584015627779081114567816257476806640625f,-0.01659997482001751567537439768784679472446441650390625f,1.187100028888881553257306222803890705108642578125f,0.99730004871711164238234914591885171830654144287109375f,1.502500054519624317350690034800209105014801025390625f,0.3218000599525872029715856115217320621013641357421875f,1.26530005336999895604321864084340631961822509765625f,0.36970001189708767697084113024175167083740234375f,-0.4049999694526196236665782635100185871124267578125f,0.30699997532367806485353867174126207828521728515625f,2.12960002028345929403485570219345390796661376953125f,1.0781999582082040234354280983097851276397705078125f,-0.69809997590780170639845891855657100677490234375f,0.3584000269681215655737105407752096652984619140625f,0.4025000035762786865234375f,0.486500016212463748388472595252096652984619140625f,-0.152799994286894946782240367610938847064971923828125f,1.2085000609010465577597415176569484174251556396484375f,-0.77529999514371106261734212239389307796955108642578125f,-1.458300054912269327900276039144955575466156005859375f,-0.38279997845441104953323474546778015792369842529296875f,-1.3562000037834047727614006362273357808589935302734375f,0.1056000049091876669127287868832354433834552764892578125f,-0.18799999286234392315009245066903531551361083984375f,0.96499999120831514431984032853506505489349365234375f,0.5078000210344786324867527582682669162750244140625f,1.049799975049495959211753870476968586444854736328125f,-1.24390000574886716577793777105398476123809814453125f,1.1295999720409513056296191280125640332698822021484375f,1.0489999886453145183651258776080794632434844970703125f,-0.105599969016015482470294273298350162804126739501953125f,-0.756599993520974845040427680942229926586151123046875f,-0.27990001255869856322533451020717620849609375f,-0.2842999958217145017869142975541763007640838623046875f,0.7291999865502114541726541574462316930294036865234375f,-0.07389999256133983340788518034969456493854522705078125f,2.00660002067834142280844389460980892181396484375f,-0.8802000003449623743989604918169789016246795654296875f,0.1074000150546432497122850691084749996662139892578125f,0.16809996469467869406599902504240162670612335205078125f,-1.05859996519386800883921750937588512897491455078125f,-0.126800024580955295050443965010344982147216796875f,0.0248000196039672715642154798842966556549072265625f,0.41249997615814226747943394002504646778106689453125f,-0.5089999632388357841961123995133675634860992431640625f,-0.65299998921155932851689840390463359653949737548828125f,0.59279999398887184280937390212784521281719207763671875f,-0.4073000004559756970223816097131930291652679443359375f,0.713799986642599026964717268128879368305206298828125f,0.115799990129470842248338158242404460906982421875f,-0.062900018012523872812380432151257991790771484375f,0.28150000682473130808602945762686431407928466796875f,0.49929998896717986411886158748529851436614990234375f,0.2426999850153919169315486215054988861083984375f,-1.1210000426471236067982317763380706310272216796875f,1.351400074070692625127776409499347209930419921875f,0.6756999893516297817086524446494877338409423828125f,-0.600000048875809710580142564140260219573974609375f,-0.84680002093017048281353709171526134014129638671875f,0.30019998322725172101854695938527584075927734375f,0.98059999632239414069090344128198921680450439453125f,-1.8124001088202017495376594524714164435863494873046875f,-1.326200046736001780800506821833550930023193359375f,0.8582000214636327228845402714796364307403564453125f,1.27210003011822703200550677138380706310272216796875f,-0.971500041842460859697894193232059478759765625f,-1.018999992966652001769034541212022304534912109375f,0.756099999615549744902409656788222491741180419921875f,0.866900013139844105580777977593243122100830078125f,0.19229999449551116175172182920505292713642120361328125f,-0.830499992787837815200191471376456320285797119140625f,0.865699988234042816515056983917020261287689208984375f,0.2726999617695800015582108244416303932666778564453125f,0.6596000165954232219434061335050500929355621337890625f,0.1170000667423014562018579454161226749420166015625f,0.251099972905218848939767894989927299320697784423828125f,1.001600008198618496635390329174697399139404296875f,-0.30560004281401631498482629467616789042949676513671875f,0.04900002269446890323933985200710594654083251953125f,-0.5389000259399414627381474929279647767543792724609375f,-0.0750000235438348372696282240212894976139068603515625f,-0.37149999855458748498904242296703159809112548828125f,0.015100002864003148062010950525291264057159423828125f,-0.7919000047206878889483050443232059478759765625f,-0.10660000261068347526816069148480892181396484375f,-1.2393999951094389899708403390832245349884033203125f,0.0013000003516676628123605041764676570892333984375f,-0.686999972939491332368788789608515799045562744140625f,0.50220000265836706176969528314657509326934814453125f,-1.243400004243850442975372061482630670070648193359375f,-0.86270000334382046514036801454494707286357879638671875f,-0.25660000529289306303581952306558378040790557861328125f,0.6673999932616954122721608655410818755626678466796875f,-0.71140000783503154480058583430945873260498046875f,-0.726999985307454466010312899015843868255615234375f,1.129600012907385320204411982558667659759521484375f,0.80120002289414404827994076185859739780426025390625f,0.3435999929845332534483759445720352232456207275390625f,-0.83279999272227289708325770334340631961822509765625f,0.273999990656971892377669064444489777088165283203125f,0.573300000858306901818650658242404460906982421875f,-0.278600001180171918946371079073287546634674072265625f,0.446000025361776852861339648370631039142608642578125f,-1.217900033137202076005678463843651115894317626953125f,-0.21170005308389772125110539491288363933563232421875f,1.0314999834299083403266195091418921947479248046875f,1.162300060284138680799514986574649810791015625f,0.6840000576376912277964947861619293689727783203125f,-0.2947000148773195604690045001916587352752685546875f,-1.27840000051855984253279530094005167484283447265625f,1.0657000523090369714651615140610374510288238525390625f,1.22000003837049053601049308781512081623077392578125f,0.3075000026822092280553988530300557613372802734375f,-1.00490003260076132818312544259242713451385498046875f,1.671000082433224509514957389910705387592315673828125f,0.32669999526739079698245404870249330997467041015625f,-0.1304000218868264227012332412414252758026123046875f,-0.0476000275254246929534929222427308559417724609375f,-0.43340000692605962218095783100579865276813507080078125f,-1.192800020065904131882916772156022489070892333984375f,0.6330999884247778819457153076655231416225433349609375f,1.2265000584572562924989824750809930264949798583984375f,0.2587000216275452402214796165935695171356201171875f,-0.804800065350533611052696869592182338237762451171875f,-0.945999992355704311108866022550500929355621337890625f,-0.502199990364908810391852966859005391597747802734375f,-0.10589999329149668572114251219318248331546783447265625f,0.3885999979019165717630812650895677506923675537109375f,0.15030000892728556660671301870024763047695159912109375f,-0.9112000041186807397508573558297939598560333251953125f,0.459100007863342318703558930792496539652347564697265625f,-0.2491000032812362707801412398112006485462188720703125f,1.21790001637339617701627503265626728534698486328125f,0.8748000234037640598927509927307255566120147705078125f,0.204099992030859045399893147987313568592071533203125f,0.396200000765919657652602836606092751026153564453125f,-0.6103999998331068876922245181049220263957977294921875f,0.384599986681342098648883620626293122768402099609375f,0.5001999876230953656630617842893116176128387451171875f,-0.829799957466125448490856797434389591217041015625f,0.228999979257583863301306337234564125537872314453125f,-0.5401000233530997007846963242627680301666259765625f,0.115400011530518487035124053363688290119171142578125f,0.511299985450506255801883526146411895751953125f,-0.446600010061263930083441664464771747589111328125f,-0.70269999440312413963027893260004930198192596435546875f,-0.59280000143945155155478232700261287391185760498046875f,0.7441000020891423982760670696734450757503509521484375f,-1.5691999977260826515390590429888106882572174072265625f,-0.53410002853870341965603074640966951847076416015625f,-0.7816000019401314613531894792686216533184051513671875f,-0.125999980509281073182137333787977695465087890625f,-0.65980002705454854350364257697947323322296142578125f,0.20980001856088659195620493846945464611053466796875f,0.38050001588463810975326850893907248973846435546875f,-1.7911000737488276257636243826709687709808349609375f,-1.515600042292475801986029182444326579570770263671875f,-0.3357999712795010083254965138621628284454345703125f,-0.17850000695884293833159972564317286014556884765625f,-1.1483000358760351300446700406610034406185150146484375f,-0.30929997071325743274172737073968164622783660888671875f,0.431800005060434699544202885590493679046630859375f,0.088500055536627542096539400517940521240234375f,-1.087900045393407122418238941463641822338104248046875f,-1.54589997360333786247110765543766319751739501953125f,0.13949998957291154244586550703388638794422149658203125f,0.6179999522753052421109032366075552999973297119140625f,1.52490005800724048867778037674725055694580078125f,1.602099992418288820772431790828704833984375f,0.23529998571872690060757804531021974980831146240234375f,0.66339998897016005940940885921008884906768798828125f,-0.055299987283348617239653322030790150165557861328125f,0.5265000123381617935791609852458350360393524169921875f,-0.7894000340759761424891394199221394956111907958984375f,0.55039996794462187068575076409615576267242431640625f,0.01730003703832683203955866702017374336719512939453125f,0.5116000112533571897444062415161170065402984619140625f,-0.8711999850079419349668796712649054825305938720703125f,-0.250800038632750244005364947952330112457275390625f,-1.2467999830439684405547495771315880119800567626953125f,-0.00669996691793228738021070967079140245914459228515625f,-0.50450000661611593688604671115172095596790313720703125f,-0.2973999733313916760124584470759145915508270263671875f,1.003499995093792396261278554447926580905914306640625f,-0.58900001837313187280642523546703159809112548828125f,0.895800006222724931603806908242404460906982421875f,0.702200001019239294919316307641565799713134765625f,-0.25900000138580825392864426248706877231597900390625f,-0.96700000214576675716671161353588104248046875f,-0.5332000245928762982572379769408144056797027587890625f,1.47470003030002061450431938283145427703857421875f,-0.44159999754428813645290574640966951847076416015625f,-0.417300010663270715127737275906838476657867431640625f,0.991399968421459387712957322946749627590179443359375f,-0.08259997656345408056921542083728127181529998779296875f,-0.09419999943971613109994223123067058622837066650390625f,-0.3610000042766332573052068255492486059665679931640625f,0.20479998171776514492847809378872625529766082763671875f,0.680400002645701107439890620298683643341064453125f,0.6502000171646475923381558459368534386157989501953125f,0.34269999756962066239651676369248889386653900146484375f,0.15300001506507410198310026316903531551361083984375f,-0.6918000108718871388191473670303821563720703125f,-0.37150003461539815585723545154905878007411956787109375f,-0.463899986600875280373657005839049816131591796875f,0.28329997768700021509857833734713494777679443359375f,-0.01799998384714118770943969138897955417633056640625f,0.26299997729063040452501809340901672840118408203125f,0.5649000174760814285690457836608402431011199951171875f,1.1894999658614400228628937838948331773281097412109375f,-0.0270000081062320429481360406498424708843231201171875f,-0.0922000183045863508368711336515843868255615234375f,0.1494000031933186978250205356744118034839630126953125f,-0.4556000008299940784439741037203930318355560302734375f,0.868700019615888496815614416846074163913726806640625f,0.25499999955296548392880140454508364200592041015625f,-1.41560001092553111590177650214172899723052978515625f,-1.57310000943541528073410518118180334568023681640625f,0.05920002551674929947012060438282787799835205078125f,-0.1297000016152853874018546775914728641510009765625f,0.40660000421255781954954500179155729711055755615234375f,0.42400001479685311966960625795763917267322540283203125f,0.6991000168040393258905851325835101306438446044921875f,-0.6646999858200552946385641917004249989986419677734375f,1.030599989691376894285212983959354460239410400390625f,-0.1322000077992679933913677814416587352752685546875f,0.57319997787773591113591464818455278873443603515625f,-0.2584000003322957095264200688689015805721282958984375f,0.0132999901667239794900154947754344902932643890380859375f,-0.29969999565482130510218894414720125496387481689453125f,0.353699983592331623771798376765218563377857208251953125f,0.1424999717622992267962445112061686813831329345703125f,-0.00640000172555466040336114019737578928470611572265625f,0.7296999942392112803446480029379017651081085205078125f,-0.39030000151395760621397812428767792880535125732421875f,0.172700001034140626643420546315610408782958984375f,0.6170000101923942548154400355997495353221893310546875f,0.58130001197457270567525711157941259443759918212890625f,-0.09059999885559033572235421161167323589324951171875f,0.2842999921709299382399649402941577136516571044921875f,0.8290000229179856905403767086681909859180450439453125f,0.18839997175931966211237522657029330730438232421875f,0.4009999984651801963053685540216974914073944091796875f,-0.3681999917730689464434590263408608734607696533203125f,0.01820000313520420132107346944394521415233612060546875f,0.1450999943703414896134518130565993487834930419921875f,-0.45539997241497076618799155767192132771015167236328125f,0.19769998732507243488498716033063828945159912109375f,-0.781000000551343109833624112070538103580474853515625f,-0.721799980622530146234794301562942564487457275390625f,-1.1787999850779768973296768308500759303569793701171875f,1.266599984282255064726996351964771747589111328125f,-0.3633999962717293641389915137551724910736083984375f,-0.34109999581575412008760395110584795475006103515625f,0.06830001397132878793172494624741375446319580078125f,0.6037000526219618024015289847739040851593017578125f,0.7955000627115371703013124715653248131275177001953125f,-0.7093999803945416626760334111168049275875091552734375f,-0.595999980770051340783766136155463755130767822265625f,-0.0720000042319295374682042165659368038177490234375f,-1.554400036460160894336013370775617659091949462890625f,-1.445100046896934831153203049325384199619293212890625f,0.0574999958276745104512883699499070644378662109375f,0.84810005607605010169436354772187769412994384765625f,-1.009500024795531913923696265555918216705322265625f,-0.71640000120401392535995910293422639369964599609375f,0.5325999515295034569817289593629539012908935546875f,1.24480000947117819265486104995943605899810791015625f,0.518999964654444312373016146011650562286376953125f,-0.0399999758601179422612403868697583675384521484375f,0.2265000600218769477578462101519107818603515625f,0.6503000239402059179383286391384899616241455078125f,0.6525999968290328023812207902665250003337860107421875f,0.0438000280678270403900143037390080280601978302001953125f,0.5435999827772379777712785653420723974704742431640625f,-0.400000006593763390849716188313323073089122772216796875f,-0.38230000470280600666228565387427806854248046875f,0.8579999869018790636943094796151854097843170166015625f,2.1275000386685132269803943927399814128875732421875f,-0.0501999982029202040934023898444138467311859130859375f,-2.30300006677210422623147678677923977375030517578125f,0.805699987339973944244775339029729366302490234375f,1.769400042197108557928686423110775649547576904296875f,0.5908999716103071619954789639450609683990478515625f,-1.215800024998187733871191085199825465679168701171875f,0.5400000528991222292773954904987476766109466552734375f,0.176900010681152242142388786305673420429229736328125f,-0.575899982190132053716524751507677137851715087890625f,-0.49430001549124735760187832056544721126556396484375f,-0.3253999980449673312676850400748662650585174560546875f,-0.50079999970346733562109875492751598358154296875f,-1.0779999767690895406957451996277086436748504638671875f,0.8978999723404645916247091008699499070644378662109375f,-1.55219996048808095423510167165659368038177490234375f,0.577500008940696307746520687942393124103546142578125f,-0.646000001445413030154441003105603158473968505859375f,-0.18959997304677944640616260585375130176544189453125f,0.442000057205557794759442913345992565155029296875f,0.521599945688247856878660968504846096038818359375f,-1.321200035038590403502212211606092751026153564453125f,0.3732000338315959453439063508994877338409423828125f,1.061199986088276592255397190456278622150421142578125f,-1.2216999803662300649875760427676141262054443359375f,0.065799982082844099551266481284983456134796142578125f,0.3002999761819840074394960538484156131744384765625f,0.164400028860569591415696777403354644775390625f,-0.58240009183287799743311552447266876697540283203125f,-1.40420000987052873142602038569748401641845703125f,0.13989997426271383318407970364205539226531982421875f,0.0585000094175338603008640347979962825775146484375f,0.573100033426284571902442621649242937564849853515625f,1.1650999828219408271934298682026565074920654296875f,0.4163999781072131867176722153089940547943115234375f,0.57479998071193705300174769945442676544189453125f,-0.6414000560402879358434802270494401454925537109375f,-1.083700050237775602823830922716297209262847900390625f,0.715200029048323582259172326303087174892425537109375f,-0.26210004792511565341328605427406728267669677734375f,-0.619199980925022597233464693999849259853363037109375f,-0.66710001326128887111366339013329707086086273193359375f,-0.17769999930188029590993892270489595830440521240234375f,-0.2667999991558491768017802314716391265392303466796875f,-1.038900027728080655009534893906675279140472412109375f,-0.3030999980360267187506906338967382907867431640625f,-1.03960001916587341241893227561376988887786865234375f,-0.29509996806979188477271236479282379150390625f,0.025799989011883184275575331412255764007568359375f,-0.113000001132488758770477943471632897853851318359375f,0.51100000151991853414301658631302416324615478515625f,0.0914000206500293899125608731992542743682861328125f,-0.1879000262081624850196703846449963748455047607421875f,-0.321099979275464963990316391573287546634674072265625f,0.152399992293119124209255232926807366311550140380859375f,-0.2039000255674121575566459796391427516937255859375f,0.77789999931156617041239087484427727758884429931640625f,1.04219999517053363291552159353159368038177490234375f,0.49430002994537358684823402654728852212429046630859375f,-0.0116999997749924755652273233863525092601776123046875f,0.0410000046305358523124340308640967123210430145263671875f,-0.507299989950656282644558814354240894317626953125f,0.38920000770986007410101592540740966796875f,0.010100023651123279933017329312860965728759765625f,0.5877999891459939618698626873083412647247314453125f,0.1912000096321104791741163353435695171356201171875f,0.506799993884563537704934788052923977375030517578125f,-0.1360000177621838446384572307579219341278076171875f,-0.31840003218352830316462132032029330730438232421875f,-0.2672999819040295932381923194043338298797607421875f,0.119199993926286662571101260255090892314910888671875f,0.5766000125199557402311256737448275089263916015625f,-1.7943999730557198546421204810030758380889892578125f,0.468899998670816220425194842391647398471832275390625f,-1.143399994669854091711158616817556321620941162109375f,-0.1081999959267676558738457970321178436279296875f,-0.0056000019475817008896001425455324351787567138671875f,0.00809999963790175581124231030116789042949676513671875f,-0.78159999217987063957480131648480892181396484375f,-0.5621999862670890024674008600413799285888671875f,0.944600029708444477449802434421144425868988037109375f,-1.837099957251549398051793104968965053558349609375f,-1.1978000144779683555640303893596865236759185791015625f,0.36579997105598494044897961430251598358154296875f,-0.5052999739468102546879890724085271358489990234375f,1.11399996137618995106777219916693866252899169921875f,0.4657999616682528820632569477311335504055023193359375f,0.50410003420114435357390902936458587646484375f,-0.58999996900558471235598290149937383830547332763671875f,-1.165099943482875488598438096232712268829345703125f,-0.0391999920263883527837833753437735140323638916015625f,-1.189300001335143353031753576942719519138336181640625f,-0.463200041542946916450773642282001674175262451171875f,0.24039999327659611338958711712621152400970458984375f,0.957399989871680734410119839594699442386627197265625f,-0.8523999635338785907379133277572691440582275390625f,-0.08600003474950757009764856775291264057159423828125f,-0.920499952256680131768007413484156131744384765625f,0.200099996978044369910776367760263383388519287109375f,-0.0402999624729165617509352159686386585235595703125f,0.720799970012903745697485646815039217472076416015625f,0.37840000767111714452539672493003308773040771484375f,0.2180000194609164054782013408839702606201171875f,-0.89969997002482404724332809564657509326934814453125f,0.3084000055849553945108709740452468395233154296875f,-1.464499970555305452535321819595992565155029296875f,-1.1441999906480313509149482342763803899288177490234375f,-0.1601000255882727429934675456024706363677978515625f,0.46780000628232887294188913074322044849395751953125f,0.284999970048666728672515091602690517902374267578125f,1.00079999281167975055950591922737658023834228515625f,0.14759997566938398705360668827779591083526611328125f,0.52199999734014246310920270843780599534511566162109375f,0.9604000040054316844617687820573337376117706298828125f,0.898300010097026646604945199214853346347808837890625f,0.68499999243766040990522014908492565155029296875f,0.8271000113800166531063950969837605953216552734375f,0.09210002274215234319143519314820878207683563232421875f,0.603900016216933455126536500756628811359405517578125f,1.19919998498558921795620335615240037441253662109375f,-0.23709996492564666681346352561376988887786865234375f,-1.9435000106096269600897130658267997205257415771484375f,0.083399965240061657567594011197797954082489013671875f,-0.2643000378802409233713888170314021408557891845703125f,0.28320000022947755535795977266388945281505584716796875f,-0.0955999430879941325400750429253093898296356201171875f,-0.90830000775009356317468700581230223178863525390625f,0.1201000263333327211512369103729724884033203125f,-0.310300015409290974588429889990948140621185302734375f,0.672400003170967153209858224727213382720947265625f,0.29090001273751209254214700195007026195526123046875f,0.2232000019803643819926719515933655202388763427734375f,0.14610001384615856068194261752068996429443359375f,-0.4351999857231974022653275824268348515033721923828125f,1.151699967327714180242992370040155947208404541015625f,1.010700010660290359965074458159506320953369140625f,-1.3048999447584161970326022128574550151824951171875f,-1.6740999915093184302605777702410705387592315673828125f,1.309299950522184730061781010590493679046630859375f,0.2887999868288637639324178962851874530315399169921875f,0.4159000105023384907809713695314712822437286376953125f,-0.2738999785542486353762114958954043686389923095703125f,-0.1745000322461118802408464034670032560825347900390625f,1.790399963548779727062765232403762638568878173828125f,-0.898600014889240128468372859060764312744140625f,-1.1446000167444345674283567859674803912639617919921875f,0.28819998525381063103623091592453420162200927734375f,-0.348400005733966455778727322467602789402008056640625f,0.52369999160170532803704190882854163646697998046875f,-0.605499997109174348253191055846400558948516845703125f,0.700099986658990314225547990645281970500946044921875f,0.35789997241497084612404933068319223821163177490234375f,-0.02049997770041322997514043890987522900104522705078125f,-1.21579998923540077981897411518730223178863525390625f,-0.1485999567747118277338813641108572483062744140625f,1.19349998587369920954870394780300557613372802734375f,-0.11230003435611646267489049932919442653656005859375f,-1.553599939228595072648886343813501298427581787109375f,0.3505999792978169438129043555818498134613037109375f,0.1574000021278851857431391181307844817638397216796875f,-0.2235000229403365512581558505189605057239532470703125f,0.970399999237059773804503493010997772216796875f,-0.45749999612569780538251507095992565155029296875f,-0.5148000088378783534182048242655582726001739501953125f,1.11829999996423712360638091922737658023834228515625f,0.24559998298585394849169460940174758434295654296875f,-0.6318000150814648829822317566140554845333099365234375f,0.0859000063672661351432680021389387547969818115234375f,-0.8689000124916435030542061213054694235324859619140625f,0.702999965742230781273747197701595723628997802734375f,0.351499968193471790112170083375531248748302459716796875f,0.5058999999970195116105742272338829934597015380859375f,0.7461000195086000541522253115545026957988739013671875f,0.7212000028520824113087428486323915421962738037109375f,-0.91780004726052322183704745839349925518035888671875f,-0.6711999886959787442464175910572521388530731201171875f,0.31270003195405049467581193312071263790130615234375f,-0.0989000134974722389102907982305623590946197509765625f,0.89020002286434163352168980054557323455810546875f,0.226100011014938484521508144098334014415740966796875f,-0.570599995875358700914148357696831226348876953125f,-0.821999923169613566642510704696178436279296875f,0.6681000074237586350278661484480835497379302978515625f,0.1991999633789067747358103588339872658252716064453125f,-1.092399998998641930114672504714690148830413818359375f,1.2282999719500542479266869122511707246303558349609375f,0.8955999736353756901507949805818498134613037109375f,0.563199996019899895571825254592113196849822998046875f,0.770799986962974248427826751139946281909942626953125f,-0.441099996858834497714951794478110969066619873046875f,0.29849998244643172196077784974477253854274749755859375f,0.4690999415159229357641379465349018573760986328125f,-0.740999989561737404386576599790714681148529052734375f,-0.0532999579429631609173156903125345706939697265625f,1.44750004954636057874495236319489777088165283203125f,-0.578400010800361741303277085535228252410888671875f,-1.2601000131458042208265624140040017664432525634765625f,-1.433699955391883662514374009333550930023193359375f,-1.2247999659597876842553887399844825267791748046875f,0.3066999699354173714027638197876513004302978515625f,0.44319999192208048288676991433021612465381622314453125f,0.37669998688548833598588316817767918109893798828125f,-0.4011999731242659184005105998949147760868072509765625f,0.5919999483898290559835686508449725806713104248046875f,-0.8826999612852934884443811824894510209560394287109375f,0.5954999723732472549642125159152783453464508056640625f,-0.85740001837015100694117109014769084751605987548828125f,1.4076999835625285673046391821117140352725982666015625f,0.83289998850822488662970499717630445957183837890625f,-0.6221999364972123913730683852918446063995361328125f,0.2369000071048734668011093162931501865386962890625f,-0.5336999834060680569791657035239040851593017578125f,0.0103999882102012719542472041212022304534912109375f,-0.07249998331069917867353069595992565155029296875f,0.08640000451952223325946533805108629167079925537109375f,0.4489000215068463806034060326055623590946197509765625f,0.84759996609538823530982654119725339114665985107421875f,-0.108500030875205322189458456705324351787567138671875f,0.154700005936622364544064112124033272266387939453125f,-0.531299994689226284805272371158935129642486572265625f,-0.5331999975845216344083610238158144056797027587890625f,-0.223299999219178868514745772699825465679168701171875f,-0.96560000764727593747238643118180334568023681640625f,0.41800005254149485978132361196912825107574462890625f,-1.16330001710057207020554415066726505756378173828125f,0.0932000056684021416231189505197107791900634765625f,-0.911200013692676780152623905451036989688873291015625f,-2.1362000218510619475864587002433836460113525390625f,0.19909996986538158747492843758664093911647796630859375f,-0.2311000163793561767988649080507457256317138671875f,0.2299000040650367449046598267159424722194671630859375f,0.2205999871581791893504487234167754650115966796875f,-0.1975000086426732526234673059661872684955596923828125f,0.6464999990388751172076808870770037174224853515625f,0.197799956512451302614863379858434200286865234375f,-0.4486000418230886577219962418894283473491668701171875f,0.2119999854564671437628931016661226749420166015625f,0.3438999969944356305262544992729090154170989990234375f,-0.227899989402294611551269554183818399906158447265625f,-0.48079995518624774053506598647800274193286895751953125f,-1.222100001358985110755384084768593311309814453125f,-0.5343000220477576167610322954715229570865631103515625f,-1.88289998493194499218361670500598847866058349609375f,-1.360200025807321022597307091928087174892425537109375f,1.10939995197057772458038016338832676410675048828125f,0.3128000139191744466415912029333412647247314453125f,0.8719999824762343365591732435859739780426025390625f,1.204899967892468115593374022864736616611480712890625f,0.58549999126046881059437509975396096706390380859375f,-0.3505999784410003261569954702281393110752105712890625f,1.6899999250844126574833126142038963735103607177734375f,-0.10869995881915173185916501097381114959716796875f,0.6486999711871153095898989704437553882598876953125f,0.4399999761581423474154917130363173782825469970703125f,-0.6357999765694142269012445467524230480194091796875f,0.838500004947185129111630885745398700237274169921875f,-1.643599965268373264137835576548241078853607177734375f,0.438199978995322592112415804876945912837982177734375f,-1.489799985182283847962025902234017848968505859375f,-0.096999999612570331208871721173636615276336669921875f,-0.02120000367164553978227559127844870090484619140625f,0.88670001910924867161156726069748401641845703125f,0.69619996135234796241775256930850446224212646484375f,-0.889599959635734904850323800928890705108642578125f,0.3684999855756760922531611868180334568023681640625f,-0.22620000419318653239741934157791547477245330810546875f,0.58320002682805149873956906958483159542083740234375f,-0.7615999807804827437252015442936681210994720458984375f,0.45910005103945739080018029198981821537017822265625f,-0.8438999993786211462065693922340869903564453125f,-1.115300006058811188580648376955650746822357177734375f,-0.42090001035332724743653898258344270288944244384765625f,-0.33399997337162457000658832839690148830413818359375f,-0.393600003340840054466553965539787895977497100830078125f,-0.3755000025480981218350962080876342952251434326171875f,-1.015599987158179562385384997469373047351837158203125f,0.07280002811253016847814478751388378441333770751953125f,-0.5365000090226528239867320735356770455837249755859375f,0.06919999064803139532386921928264200687408447265625f,0.466199993538856161112704512561322189867496490478515625f,0.1158999994754790208162376075051724910736083984375f,0.2198000098437071780921314712031744420528411865234375f,0.11859999402761456366306447307579219341278076171875f,-1.43649997031688680948491310118697583675384521484375f,-0.21579998264163735566256718811928294599056243896484375f,-2.107899975395202574190989253111183643341064453125f,-0.27470005227923355306529629160650074481964111328125f,-0.421100006544590232948621633113361895084381103515625f,-0.3194000079616898801759816706180572509765625f,-0.251900016045569774547630004235543310642242431640625f,0.58789995013773499454856619195197708904743194580078125f,-0.663399993664026244033493640017695724964141845703125f,-0.26890000358819943215848979889415204524993896484375f,-0.7825999736577273768745044435490854084491729736328125f,0.3245999792307616704789552386500872671604156494140625f,-0.0632999859571459211338151362724602222442626953125f,-0.3963999770641326580999930229154415428638458251953125f,0.30759998967647561585181392729282379150390625f,0.23359998758286248943250029697082936763763427734375f,0.056699997353554021373156501795165240764617919921875f,0.0039000062704082072428946048603393137454986572265625f,0.097800009188055536668571221525780856609344482421875f,0.060399994915723453914324636571109294891357421875f,0.8304999611973773454565161955542862415313720703125f,1.0451000280469646686043461158988066017627716064453125f,0.73919993692934582441012025810778141021728515625f,-0.1364999981820587127145927297533489763736724853515625f,0.403299956229330458512549739680252969264984130859375f,0.66140000120401343242093616936472244560718536376953125f,-0.579099987486004774694947627722285687923431396484375f,0.5769999727159740299242685068747960031032562255859375f,-0.2861000082582230152894453567569144070148468017578125f,0.2212000030755996160536369643523357808589935302734375f,0.510899976304173275565290168742649257183074951171875f,-0.35440000025033935315832422929815948009490966796875f,0.3229999915957453282544520334340631961822509765625f,0.36000001128762948443551294985809363424777984619140625f,-1.0575000709295283574107315871515311300754547119140625f,-0.41709997289776812312567244589445181190967559814453125f,-0.90950000870227842852955291164107620716094970703125f,-1.398399970865250363516452125622890889644622802734375f,1.05970006285905782306144828908145427703857421875f,-1.052499956600367880810154019854962825775146484375f,-1.91799999610334648281195768504403531551361083984375f,-0.0167000302478665929584167315624654293060302734375f,0.8435000056922448141705217494745738804340362548828125f,0.2285000045746560515258494206136674620211124420166015625f,-0.39789997763037643352390659856610000133514404296875f,-0.46529997912496323664299779920838773250579833984375f,-1.128200003023445585625950116082094609737396240234375f,-0.44290001514405030746956981602124869823455810546875f,0.5655000318847600571103839683928526937961578369140625f,0.045000002235174189901822927595276269130408763885498046875f,-0.2076000123262407015545250033028423786163330078125f,0.44470002128481755931943553150631487369537353515625f,-0.41360000471919777798035511295893229544162750244140625f,0.1432000561833373097897492698393762111663818359375f,0.72530004606843012648909052586532197892665863037109375f,-2.59539991012811643855684451409615576267242431640625f,-0.8452000240564345734384232855518348515033721923828125f,0.787799945373833576667266243021003901958465576171875f,-0.21889997613281053645550855435431003570556640625f,-0.4204999732673169177132876939140260219573974609375f,0.892199998746812550365348215564154088497161865234375f,-0.798400025105475918962838477455079555511474609375f,-0.60790005466937990785680767658050172030925750732421875f,0.75379999461471947075352773026679642498493194580078125f,1.06100001120567366541536102886311709880828857421875f,-0.373000001020728999545639226198545657098293304443359375f,-0.6315000154115255615039359327056445181369781494140625f,0.670199998426436849285892094485461711883544921875f,-0.034299986433982354583349660970270633697509765625f,-0.096999985456466930600072373636066913604736328125f,0.8549999795854095197000788175500929355621337890625f,-1.67929996572136897015070644556544721126556396484375f,-0.59330005986690448338549686013720929622650146484375f,-0.18139996752738962726425597793422639369964599609375f,1.817799965192377964484649055521003901958465576171875f,0.2679999841451643494139034373802132904529571533203125f,-1.683899931764603508099753526039421558380126953125f,0.9288999660745262954009149325429461896419525146484375f,0.1461999951779839879151268178247846662998199462890625f,0.7345999863088135128919020644389092922210693359375f,1.2931000167183572813200953532941639423370361328125f,-0.4731000263482327827802009778679348528385162353515625f,-0.283200015950203110026706099233706481754779815673828125f,-0.428799997967480861493783095284015871584415435791015625f,-0.85799996913224507721906775259412825107574462890625f,0.355400042748450939456006381078623235225677490234375f,-0.76909995585829060704696757966303266584873199462890625f,0.317500037439166382657873555217520333826541900634765625f,0.03049999643862288944973215620848350226879119873046875f,0.6328999761402609447458189606550149619579315185546875f,0.0898000017598269695273671686663874424993991851806640625f,-0.12989994906112467543124466828885488212108612060546875f,-0.064499998532235591763850379720679484307765960693359375f,-0.28480000852122888677087075848248787224292755126953125f,0.7962000055156643885112544012372381985187530517578125f,0.6503000007688992667453931062482297420501708984375f,0.7011000118717551909952589994645677506923675537109375f,1.11460003404840879426274113939143717288970947265625f,0.16959999655336133184135860574315302073955535888671875f,0.82210002656057501457809166822698898613452911376953125f,0.1034000026606023414643686919589526951313018798828125f,-0.48159996535778049064902006648480892181396484375f,-1.21119992439746937407107907347381114959716796875f,-0.3511999913409351581350392734748311340808868408203125f,-0.465799974818528195186217999435029923915863037109375f,0.875899967773258847358874845667742192745208740234375f,-1.0795999666392805949044486624188721179962158203125f,-1.3219999797567723032187814169446937739849090576171875f,-0.19620002661943314326009613068890757858753204345703125f,1.3742999663174153557321233165566809475421905517578125f,0.57499998629093085344976543638040311634540557861328125f,-0.2880999929510057455672722426243126392364501953125f,0.597300006937980132448728909366764128208160400390625f,0.8854999783709647243057361265528015792369842529296875f,-0.19809999736547456716806436816114000976085662841796875f,0.49590003095418222489598747415584512054920196533203125f,-0.73080001158714280595773971072048880159854888916015625f,-1.35400001429393856966498788096942007541656494140625f,-1.352799942207337124955301987938582897186279296875f,-0.186700027751922181806776279699988663196563720703125f,-0.7043999552488333648625484784133732318878173828125f,0.54299996205419365224997818586416542530059814453125f,0.5114999695718294159263450637809000909328460693359375f,-0.51389997631907491371094920395989902317523956298828125f,0.8258999626323586529252906984766013920307159423828125f,0.094299982354789813054907199330045841634273529052734375f,1.0184999621994794694046504446305334568023681640625f,0.46630001868009551824201253111823461949825286865234375f,0.72989999130591665110756594003760255873203277587890625f,-0.788899994498490908512167152366600930690765380859375f,0.586999981433153461551910368143580853939056396484375f,0.215199989318102524737952307987143285572528839111328125f,-0.118100005803257079861623424221761524677276611328125f,0.16879999006986590170953377310070209205150604248046875f,-0.26779998778626279420933542496641166508197784423828125f,0.06660000629872087074545561335980892181396484375f,-0.1775000047869983443860064653563313186168670654296875f,0.520499998338520786234084880561567842960357666015625f,-0.482600044102966918568853316173772327601909637451171875f,-0.1050999904029073228617363611192558892071247100830078125f,-0.719700021284819246858432961744256317615509033203125f,0.294400015524029667890459904810995794832706451416015625f,-0.77520004715323498345469488413073122501373291015625f,0.52099999716132894267417441369616426527500152587890625f,-0.79260001113414713103111353120766580104827880859375f,-0.468400007149577246590155255034915171563625335693359375f,0.42130003358125722723315220719086937606334686279296875f,-0.142000001512467888264978910228819586336612701416015625f,-0.5321000096105039656180224483250640332698822021484375f,0.53009995413720634171994561256724409759044647216796875f,0.98109999581575468852179255918599665164947509765625f,0.38659999787956456085424861157662235200405120849609375f,0.117799992126226271693667513318359851837158203125f,-0.005000021383165798027903292677365243434906005859375f,0.07119997584372705912159062791033647954463958740234375f,0.626099994884430799402252887375652790069580078125f,0.402200030374526562582104816101491451263427734375f,0.355000003874301750528275078977458178997039794921875f,-0.823000019125640580597291773301549255847930908203125f,0.8313000255346292277636166545562446117401123046875f,0.748000015586614441787105533876456320285797119140625f,0.97589996997117989252501502051018178462982177734375f,1.1520999891400343084768564949627034366130828857421875f,-0.66129999267756878200685832780436612665653228759765625f,-0.684100014233588904488669868442229926586151123046875f,1.243199966441094428404312566271983087062835693359375f,0.5782000082202258806063355223159305751323699951171875f,-0.04700000900030065764667597250081598758697509765625f,-0.4909000088259574123839001913438551127910614013671875f,0.56469999632537337674875743687152862548828125f,-0.20720001107752283875385046485462225973606109619140625f,1.2283999922670416626857559094787575304508209228515625f,0.185000023990869622014798778764088638126850128173828125f,0.061400017725675326563106182220508344471454620361328125f,-0.301800011169909954134027429972775280475616455078125f,0.080800020825862350903889819164760410785675048828125f,-1.187099963957070958286976747331209480762481689453125f,1.1304999594092368653264202293939888477325439453125f,0.5417000249207013506946850611711852252483367919921875f,-0.52430002920031437696479770238511264324188232421875f,-0.15100001915916816397356114975991658866405487060546875f,-0.4955999918892974864803591117379255592823028564453125f,0.7845000174939629200565605060546658933162689208984375f,-0.233900043113528577354287563139223493635654449462890625f,-1.01759997984170968976513904635794460773468017578125f,-0.18280000419616637685749083175323903560638427734375f,0.6848999880462882661191770239383913576602935791015625f,-0.3007000003784898733982799967634491622447967529296875f,0.8517000143408781642762050978490151464939117431640625f,-1.112199999678134876290869215154089033603668212890625f,-0.34580002835094969526608110754750669002532958984375f,0.22030001641511898213821041281335055828094482421875f,0.0083999729514128151919294396066106855869293212890625f,0.14600000375509269989748872831114567816257476806640625f,1.2729999917447563095862506088451482355594635009765625f,-0.6644999830722804201599274165346287190914154052734375f,1.51560001710951297582141705788671970367431640625f,-1.320300035265087768010516811045818030834197998046875f,1.350000051334500739130817237310111522674560546875f,-0.69050002385675934846176460268907248973846435546875f,0.480000017359852648723972379229962825775146484375f,0.76840001132190227650653469027020037174224853515625f,0.1818000072956085233499834430404007434844970703125f,0.74339999053478234003478064551018178462982177734375f,1.0628000096350904701836270760395564138889312744140625f,-1.3949000207543367224616304156370460987091064453125f,0.634199974405765143359303692704997956752777099609375f,-0.434399984157085583547086571343243122100830078125f,-1.19879998410940213915409913170151412487030029296875f,0.898099961900711729612112321774475276470184326171875f,-0.16199998903274515082983953107031993567943572998046875f,0.2921000219225877714279704377986490726470947265625f,0.0058000023484237051007994523388333618640899658203125f,0.1990999966502187135120038874447345733642578125f,0.020800055620074697770860439050011336803436279296875f,-1.30810002195239061251186285517178475856781005859375f,-0.550399968466162459179713550838641822338104248046875f,-0.24220002098679582758222750271670520305633544921875f,-0.686999998643994302938153850845992565155029296875f,-0.1812000003933906810971166123636066913604736328125f,0.11590000230669961212015550700016319751739501953125f,-1.09930005855560342098442561109550297260284423828125f,-0.52370000933408800136703575844876468181610107421875f,-0.8363000516116623206386293531977571547031402587890625f,-1.0277000095278030489254206258920021355152130126953125f,-1.7181000421434642344564736049505881965160369873046875f,-0.4846000084742903002421599012450315058231353759765625f,1.07929996587037901889516433584503829479217529296875f,-0.2136000512480737967280219891108572483062744140625f,-0.68969997330307908356417101458646357059478759765625f,-0.35540000802874605057013468467630445957183837890625f,0.8671000313103200962672190144076012074947357177734375f,-0.53870001596510352914748409602907486259937286376953125f,0.51910002749562256241233626496978104114532470703125f,0.978700035932661194237880408763885498046875f,0.33610001459121718880851403810083866119384765625f,0.58140004799366096222001942805945873260498046875f,-0.54710000411570092193613845665822736918926239013671875f,0.541500003769993742963606564444489777088165283203125f,-1.017200001540780274211783762439154088497161865234375f,0.0603000050157309619436318826046772301197052001953125f,-0.7407000062644482074603047294658608734607696533203125f,1.2202000223800533174056681673391722142696380615234375f,-0.630700015391409518628051955602131783962249755859375f,-1.769200003165006140903869891189970076084136962890625f,0.347999984368681847257676054141484200954437255859375f,0.921999994769692410301331619848497211933135986328125f,0.32640000261962420768924175717984326183795928955078125f,-0.7600999992877242039668317374889738857746124267578125f,1.283499987699091793302841324475593864917755126953125f,0.5096999846279619550415418416378088295459747314453125f,0.4482000179246072857353055951534770429134368896484375f,-0.452900010934472074808354591368697583675384521484375f,-0.54970002411603946512741458718664944171905517578125f,-0.07779998169541357810885529033839702606201171875f,0.247599962630868031254749439540319144725799560546875f,0.15030002520680429967114832834340631961822509765625f,-0.21220001271665100972541040391661226749420166015625f,0.4756000060454013134147999153356067836284637451171875f,0.27159996140897246785783636369160376489162445068359375f,0.3059000071123245678705870886915363371372222900390625f,0.617800008517503496108247418305836617946624755859375f,-0.1394999881386755902212826185859739780426025390625f,-1.285199975702166153013195071252994239330291748046875f,0.9221999831378457912478552316315472126007080078125f,-1.161199944141507511830013754661194980144500732421875f,-0.01470002173185325222704022962716408073902130126953125f,0.65880005198419044898372476382064633071422576904296875f,-1.2190999994069338097091304007335565984249114990234375f,0.566000006660818844039795294520445168018341064453125f,0.254099992775916661003066110424697399139404296875f,-0.3313000285148621770758836646564304828643798828125f,2.06030003831982622131135940435342490673065185546875f,1.88320000641345952629990279092453420162200927734375f,-1.1368000119149679338903524694615043699741363525390625f,-1.71769997391402728936782295932061970233917236328125f,-0.10479998294711112549748577293939888477325439453125f,-0.196300030824541948959449655376374721527099609375f,-0.5333000055521719406925740258884616196155548095703125f,-0.6427999972298741315768211279646493494510650634765625f,0.2626000207826499899255168202216736972332000732421875f,-0.220799982567131625188494581379927694797515869140625f,-1.44010002096891387424193453625775873661041259765625f,-1.61860002889633136646807543002068996429443359375f,1.086300011527537634492546203546226024627685546875f,2.2780000115633018964445000165142118930816650390625f,-0.574700043487549550036419532261788845062255859375f,-0.60130000050067877737092203460633754730224609375f,0.666400000086426747003542914171703159809112548828125f,-0.468500020891427393365802345215342938899993896484375f,0.392800022673606807899204795830883085727691650390625f,-0.046699995639920555134949609055183827877044677734375f,-0.477100004674494204692791754496283829212188720703125f,0.7465999769061806290437743882648646831512451171875f,0.47209999662786739094144650152884423732757568359375f,0.3962999976694583370573354841326363384723663330078125f,-0.0255999857425691923484833978363894857466220855712890625f,0.1904999644756311116822189433150924742221832275390625f,-0.7958999923601746129264711271389387547969818115234375f,-0.0743999667972328193599196310970000922679901123046875f,0.15899999244511153762005051248706877231597900390625f,0.5166000340521339495580832590349018573760986328125f,0.272499988898635248091295579797588288784027099609375f,-0.527200011748075514361744353664107620716094970703125f,-0.795300023716688286867793067358434200286865234375f,0.8893999808788297656292343162931501865386962890625f,0.017999998003244588318239038926549255847930908203125f,-1.60059997560977951280847264570184051990509033203125f,-0.5261999888449906404019884575973264873027801513671875f,1.3030999696493150707965469337068498134613037109375f,-1.483199966180324569364756825962103903293609619140625f,0.1112000080674884205933494740747846662998199462890625f,-0.2938000094413755203248683756100945174694061279296875f,-0.1493999848648908113091238192282617092132568359375f,-0.6534000310659406896007794784964062273502349853515625f,-0.3816000001519916917658292732085101306438446044921875f,0.3323999811917544011663494529784657061100006103515625f,1.2226999793529511606493542785756289958953857421875f,0.5223999830543988309727865271270275115966796875f,-1.3610999895572657436559893540106713771820068359375f,0.714200002270936717962968032225035130977630615234375f,0.53520000379085530539668980054557323455810546875f,0.23530001112520693151708428558777086436748504638671875f,-0.30730001870989820389468150096945464611053466796875f,-0.4344999475330109106607778812758624553680419921875f,-1.2661000353038307508768411935307085514068603515625f,-1.821599980854987421707846806384623050689697265625f,0.1414000307083134178043337669805623590946197509765625f,0.4066999922871588069028803147375583648681640625f,0.34879999662637661828057389357127249240875244140625f,0.90150000324845347421387486974708735942840576171875f,1.658500074982643734955445324885658919811248779296875f,1.9108000381112102417091591632924973964691162109375f,2.276700099426509193989431878435425460338592529296875f,0.95080005390644117824194836430251598358154296875f,-0.41689997074603990512287055025808513164520263671875f,-0.7487999778509131232340223505161702632904052734375f,-0.1148000093221668294063420034945011138916015625f,-0.328200041580199552981866872869431972503662109375f,-0.2443999915704131442595326006994582712650299072265625f,0.795899987480044757148789358325302600860595703125f,-1.2800999969407909073737528160563670098781585693359375f,0.112899999013543084203092803363688290119171142578125f,-0.5494000109791759012267675643670372664928436279296875f,0.067099998304247510105824403581209480762481689453125f,0.5606000122666359164469440656830556690692901611328125f,-0.5723999635338776759141410366282798349857330322265625f,0.73819999784529244379882584325969219207763671875f,-1.106400042331218802615921958931721746921539306640625f,0.767400020083785161517653250484727323055267333984375f,-1.53840004797875895548031621729023754596710205078125f,0.431100017645955180256578387343324720859527587890625f,0.9319000220805406797808245755732059478759765625f,0.191199975955486767276170212426222860813140869140625f,0.44019999641478069207778389682061970233917236328125f,0.67349999019503581365597710828296840190887451171875f,-0.50559999289512636000409884218242950737476348876953125f,-0.84879998679161106078794318818836472928524017333984375f,-0.1727000029712912887447373577742837369441986083984375f,-0.268599989184737264480418161838315427303314208984375f,0.8736000074386591762731768540106713771820068359375f,0.245799999591708395740852211019955575466156005859375f,-0.131100009077787404976334073580801486968994140625f,1.2258000311076637789398091626935638487339019775390625f,-0.06030001626610737019973385031335055828094482421875f,1.463400051033497017982654142542742192745208740234375f,-0.8885000183582310029350992408581078052520751953125f,0.0130000244528059027970812167041003704071044921875f,-0.520600028359890387719133286736905574798583984375f,-2.02819997743070157270039999275468289852142333984375f,0.9234999665021899062367083388380706310272216796875f,-0.60039998128116156550504456390626728534698486328125f,-0.3545999745368959565894328989088535308837890625f,0.302900016000867022825104868388734757900238037109375f,1.050500006571411848454999926616437733173370361328125f,0.61920005584061232895010107313282787799835205078125f,1.75390001655220917342603570432402193546295166015625f,0.810400049901009111863459111191332340240478515625f,-0.08530001090168948252312475233338773250579833984375f,-0.6122999313890928885228959188680164515972137451171875f,0.5789000299632551804762670144555158913135528564453125f,0.13430002174973498796362036955542862415313720703125f,0.49430001638531717578217694608611054718494415283203125f,0.340400029486418009838644138653762638568878173828125f,1.036200028258562166882938981871120631694793701171875f,1.4225000149011606254134676419198513031005859375f,-0.393000013425945926570648225606419146060943603515625f,-0.36229996797144448095195912173949182033538818359375f,-0.5471000201344491387089874478988349437713623046875f,-0.662900012052059395273317932151257991790771484375f,-0.045899987331032565673893941493588499724864959716796875f,-0.6359999837130307920318728065467439591884613037109375f,-0.19920000391006442708174972722190432250499725341796875f,-0.6926000382542607525238054222427308559417724609375f,-0.52959999144971359186229165061376988887786865234375f,0.292900021514296948765831984928809106349945068359375f,-0.568500006884336439583194078295491635799407958984375f,0.064899965471029108510236937945592217147350311279296875f,0.26520001660585401648262404705747030675411224365234375f,-0.421899985498189966648396875825710594654083251953125f,0.31440000755190833370278369329753331840038299560546875f,-0.042900020620226797518625971861183643341064453125f,-0.045899975857138841650595395549316890537738800048828125f,0.212200012195110421231447617174126207828521728515625f,-0.73429999969601633669213924804353155195713043212890625f,0.2003999952882526969233367708511650562286376953125f,-0.114799997103214312943464392446912825107574462890625f,-0.40770000326931477019343219581060111522674560546875f,0.7036000060230487296308865552418865263462066650390625f,-0.716700012105702999321010793210007250308990478515625f,0.1634000325560567290494873304851353168487548828125f,-0.9443999913096430276482351473532617092132568359375f,-0.832999970808625533891245140694081783294677734375f,0.5614999873042105793530254231882281601428985595703125f,-0.347799956214427885559103970081196166574954986572265625f,-0.449099999332428279075202226522378623485565185546875f,0.582800006282329530193919708835892379283905029296875f,-0.135600006976723808094220657949335873126983642578125f,0.4637999929010871991152953341952525079250335693359375f,-0.766899986839294722784643454360775649547576904296875f,-0.353600018167495644405562416068278253078460693359375f,0.414499992460012567363492053118534386157989501953125f,-0.1209999963790178156841648160479962825775146484375f,0.1404000057935714806944815791212022304534912109375f,-0.30670001374483124578773640678264200687408447265625f,-0.187500029355287978205524268560111522674560546875f,-1.0583000680625449518856839858926832675933837890625f,0.798900007164478065391222116886638104915618896484375f,1.3869999805390822444906007149256765842437744140625f,3.3496000782489776526062996708787977695465087890625f,2.2761000151872625707483166479505598545074462890625f,-1.17440000762641449938428195309825241565704345703125f,-0.8765999838724731763051067900960333645343780517578125f,1.1882000042900437986048700622632168233394622802734375f,0.347800016526879307576791688916273415088653564453125f,0.2799000009357925922159893161733634769916534423828125f,-0.77769998084306646290997377946041524410247802734375f,-0.655800042358041679335656226612627506256103515625f,0.31910000246167202764269177350797690451145172119140625f,0.49620003142505930693317850455059669911861419677734375f,0.46930001416876898634455983483348973095417022705078125f,-0.03710000813901448424303453066386282444000244140625f,-0.3038999734133478458630861496203579008579254150390625f,0.80370000598132562696918057554285041987895965576171875f,-0.1524999472498891339711235559661872684955596923828125f,1.4494000296056268606292860567918978631496429443359375f,-1.817500052750111283472733703092671930789947509765625f,0.527400021350383330087652211659587919712066650390625f,1.0154000245690346648785862271324731409549713134765625f,-0.5619000141084187216478085247217677533626556396484375f,-0.283900001129507639774374183616600930690765380859375f,0.402699985872209398340970665230997838079929351806640625f,-0.3636000223398212494174686071346513926982879638671875f,-0.28170001896023766363441609428264200687408447265625f,1.038300046046078950467972390470094978809356689453125f,0.0773999971546233123120828167884610593318939208984375f,-0.6169000349886719636316456671920605003833770751953125f,0.1610000011101363204257808092734194360673427581787109375f,0.08000000476837154650411321199499070644378662109375f,-0.6482000102132554619771553916507400572299957275390625f,0.65850001761317233928139103227294981479644775390625f,0.9028000047922135617994854328571818768978118896484375f,-0.63400000011920898401740487315692007541656494140625f,0.728799992454052247836671085678972303867340087890625f,-1.6514999989271164082538234652020037174224853515625f,-1.1579999914467336008527809099177829921245574951171875f,1.34049996257573411639896221458911895751953125f,-0.394399963369966410908773468690924346446990966796875f,-1.1723999892383811793905579179408960044384002685546875f,-0.4789999756813052567139266102458350360393524169921875f,0.27949999722838381188694256707094609737396240234375f,0.02280001771897044449133318266831338405609130859375f,0.097000014644116172579657586538814939558506011962890625f,0.4410000214502216575596094116917811334133148193359375f,-0.1371000101320447583930217660963535308837890625f,0.0586999866843225248658200143836438655853271484375f,2.160900011396406750208143421332351863384246826171875f,1.830199972200393432331111398525536060333251953125f,0.0785999741345637659151179832406342029571533203125f,0.7638000026240934214882827291148714721202850341796875f,0.195799984802305981901326958904974162578582763671875f,-0.9483999527975921495226430124603211879730224609375f,0.13000003241002622278443823233828879892826080322265625f,0.204400059221685470589591204770840704441070556640625f,-1.3572000788033020679534956798306666314601898193359375f,-1.1223000098064535023922871914692223072052001953125f,-0.8106000550329692799778058542869985103607177734375f,-0.534900040572882318201664020307362079620361328125f,-0.88610003880560406219757396684144623577594757080078125f,0.1709000471964479128672564911539666354656219482421875f,-0.220100015753507793903054334805347025394439697265625f,-0.18789997953027448573948277044110000133514404296875f,0.653899983248114491374280987656675279140472412109375f,-1.086699995937943530321945218020118772983551025390625f,0.8297999808609481942056618208880536258220672607421875f,0.7277999816954137113356182453571818768978118896484375f,-0.9194999756217006048331086276448331773281097412109375f,-0.82699999864399387217162029628525488078594207763671875f,-0.6592999840870497063605171206290833652019500732421875f,0.8559999777153126654383186178165487945079803466796875f,-0.53319998888596875641354699837393127381801605224609375f,0.73370003574639586219063858152367174625396728515625f,-1.0403000515632332412252480935421772301197052001953125f,0.601999993428588009436452921363525092601776123046875f,-0.3632999952703717116975212775287218391895294189453125f,0.48210001734048135713095462051569484174251556396484375f,-0.98449998614564504517687737461528740823268890380859375f,-0.6472000187888742317454671137966215610504150390625f,-0.6756000198103488951772988002630881965160369873046875f,0.3417999885201457033190308720804750919342041015625f,0.560200002226233007007749620242975652217864990234375f,-1.0636999860882756507862723083235323429107666015625f,0.2227999740958217511632710738922469317913055419921875f,-0.601999965377152168599650394753552973270416259765625f,-0.14059998745620239102294135591364465653896331787109375f,0.3583999824136496759052761262864805757999420166015625f,-0.42459999729841968463261991928447969257831573486328125f,-0.6625000071525573019926014239899814128875732421875f,-0.69360002219080996166411523518036119639873504638671875f,0.9813000166311862670909249573014676570892333984375f,-0.04900001107156271018538973294198513031005859375f,-0.1807999634563923763153070467524230480194091796875f,1.2419999899640676854772891601896844804286956787109375f,-0.297600003236531851502633116979268379509449005126953125f,0.768200006562471227056221323437057435512542724609375f,0.41999999608844496012949321084306575357913970947265625f,0.1704000122010709272757367216399870812892913818359375f,-0.21559998931884738482267493964172899723052978515625f,-0.73550000042468288707908641299582086503505706787109375f,-0.98720005060285387354923614111612550914287567138671875f,-0.93060002398267360756989319270360283553600311279296875f,0.494500023901462526509931194595992565155029296875f,-0.28950005593895955513517037616111338138580322265625f,0.392399994677304686518937160144560039043426513671875f,-0.726900051212311382187181152403354644775390625f,-0.9677000218212603588341380600468255579471588134765625f,-0.3342999941825866017097723670303821563720703125f,1.1119000566512351468873021076433360576629638671875f,0.9939000206500292478040137211792171001434326171875f,-1.434700037489831547787844101549126207828521728515625f,-0.29780000164434328180362854254781268537044525146484375f,0.0300000315904629388796820421703159809112548828125f,-0.1914999795556067141433231881819665431976318359375f,-0.1741999966830014567875650755013339221477508544921875f,0.385899980252980778772098346962593495845794677734375f,-0.64470000731498000590136143728159368038177490234375f,0.4645000527538363233048812617198564112186431884765625f,0.0165999916397037150517235204461030662059783935546875f,-0.46200001801550427682485633340547792613506317138671875f,0.6541999766409392069732575691887177526950836181640625f,0.25020000945329723407439814764074981212615966796875f,0.777200006756186834167010601959191262722015380859375f,0.790700022804737390202944880002178251743316650390625f,0.449200016799569112890821998007595539093017578125f,-0.776300038237870193569278853829018771648406982421875f,0.3233000008396802460453045569011010229587554931640625f,1.1039000131063170240253157317056320607662200927734375f,-0.38419999608695487580689587048254907131195068359375f,-0.619100030848384275117268771282397210597991943359375f,0.1910000098645685540077465702779591083526611328125f,-0.4380999997496604070335024516680277884006500244140625f,-1.0350000316649674569902117582387290894985198974609375f,0.25760000031217933358362870421842671930789947509765625f,-0.096200019057095353947062221777741797268390655517578125f,0.41860000592991741807935568431275896728038787841796875f,-1.12140000917613491537849768064916133880615234375f,0.5422000028073785671267614816315472126007080078125f,-1.83979997862577437928166546043939888477325439453125f,-0.05179999567270243687744368799030780792236328125f,0.262200035440921741525244215154089033603668212890625f,0.0829000068366525511009967885911464691162109375f,0.3476000159397722999443658409290947020053863525390625f,-0.362100013522058949977378006224171258509159088134765625f,-0.1461999792709944845370273469598032534122467041015625f,0.6122000061042605079109080179478041827678680419921875f,0.431400014391541741787250430206768214702606201171875f,-0.0765000105500224236720896442420780658721923828125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3904999945759772828068889793939888477325439453125f,1.364799987827240546067741888691671192646026611328125f,0.9626999937325713130320536947692744433879852294921875f,-0.2577000056907532421490714114042930305004119873046875f,1.248699995699524833980831317603588104248046875f,0.68349997399002304643289562591235153377056121826171875f,0.120699976201354974847390622016973793506622314453125f,1.7059999895617357079657949725515209138393402099609375f,-0.508200006785988644963936167187057435512542724609375f,0.052500046491623653110991654102690517902374267578125f,-1.4016000067830081121655894094146788120269775390625f,-1.804100036399066464554152844357304275035858154296875f,-1.742300044190883578920647778431884944438934326171875f,-0.99899998912960175090347547666169703006744384765625f,-0.499199972803890279493543857824988663196563720703125f,-0.55059998455047587384569851565174758434295654296875f,1.45690004823207885920055559836328029632568359375f,0.469599994131922660045574957621283829212188720703125f,-1.364899986481665994375589434639550745487213134765625f,0.46499999016523363426500736750313080847263336181640625f,-0.210799995084106850384841891354881227016448974609375f,0.62389998857527950715251563451602123677730560302734375f,0.07109998657703398006191264357767067849636077880859375f,0.60179998054802397167151184476097114384174346923828125f,-0.82159998838007464172505933674983680248260498046875f,-0.2646000018060206127046285473625175654888153076171875f,0.16159999068975405833725744741968810558319091796875f,-0.604600008660554966155586953391321003437042236328125f,1.6703000413745641150597975865821354091167449951171875f,-0.7273999843209981719382994924671947956085205078125f,0.5563999969944364298868322293856181204319000244140625f,-0.6278000135838979200997300722519867122173309326171875f,-0.949599992418288962880978942848742008209228515625f,-0.202899967050552021419207449071109294891357421875f,0.24029998452663381414140530978329479694366455078125f,-1.5774999843537802490800459054298698902130126953125f,0.66900002500414912365300779129029251635074615478515625f,0.112799974617361353779187993495725095272064208984375f,-0.09460000899583109745805131751694716513156890869140625f,-1.91640000783503072767643971019424498081207275390625f,0.1042999902337788942219276577816344797611236572265625f,-1.2796000554874542576300200380501337349414825439453125f,-1.000800040774792787345859323977492749691009521484375f,0.63129999930858549106460486655123531818389892578125f,0.06439998756572595794978042249567806720733642578125f,-0.0569000178337102369141575763933360576629638671875f,-0.3654000327646735879483230746700428426265716552734375f,1.3409000741302976589253148631541989743709564208984375f,1.9156000776268549135039620523457415401935577392578125f,-0.845100011357664993028038225020281970500946044921875f,0.6205000087879597447937385368277318775653839111328125f,0.138400013184547532318902085535228252410888671875f,0.350100043097139046466281797620467841625213623046875f,0.61299999003112315421049061114899814128875732421875f,-1.1537000077694645749915025589871220290660858154296875f,-0.14190000337958341702915276982821524143218994140625f,-0.28439999902099355466589258867315948009490966796875f,1.419100037442147854704899145872332155704498291015625f,-1.271400045795739064402596341096796095371246337890625f,-0.197599993178247945735392931965179741382598876953125f,0.67170002797544015038511133752763271331787109375f,-1.1613000195741651765501956106163561344146728515625f,-0.361799992394447489374442739062942564487457275390625f,0.92860002323389068124015466310083866119384765625f,2.2825000766664746976175592863000929355621337890625f,-0.948700053292513612035463665961287915706634521484375f,-0.252500000968575388782255686237476766109466552734375f,0.187099993163347022573361755348742008209228515625f,0.472400008982420072101149344234727323055267333984375f,-1.2745000167489057840697341816849075257778167724609375f,0.422099998527764752598301356556476093828678131103515625f,0.905400014585257650878702406771481037139892578125f,-0.029800007161497443775033389101736247539520263671875f,-1.31379999424219118253631677362136542797088623046875f,0.4634000172078611701209638340515084564685821533203125f,0.20940003012716790209424289059825241565704345703125f,-0.3854999871999023053348309986176900565624237060546875f,-0.19799997073411912396068146335892379283905029296875f,-1.489900027012824867966855890699662268161773681640625f,0.00260001947879828776422073133289813995361328125f,-0.3241999872207639299404036137275397777557373046875f,-0.5276000391483315343776894223992712795734405517578125f,0.2220999755799761832264493932598270475864410400390625f,-0.0610999830752607664408060372807085514068603515625f,0.339000016883015842239501580479554831981658935546875f,1.092200041997433146434559603221714496612548828125f,-1.329400035417080516708665527403354644775390625f,-1.348500027056783512335869090748019516468048095703125f,0.57839999433457911504063986285473220050334930419921875f,-0.07269995504543158570953664820990525186061859130859375f,-0.1090000178515910800314259176957421004772186279296875f,1.250800005626678146342101172194816172122955322265625f,-1.610199981997908036390754205058328807353973388671875f,0.657699960502981628707175332237966358661651611328125f,0.74540000072717749191042457823641598224639892578125f,0.3938000323705381777728007364203222095966339111328125f,-0.451999998718500162198097314103506505489349365234375f,1.374900019729882405528087474522180855274200439453125f,-1.958300016355514205912413672194816172122955322265625f,-1.184899992330371798487931300769560039043426513671875f,0.51329996017813694919595945975743234157562255859375f,-0.757599984721839536661036618170328438282012939453125f,-1.1469000312447548139971331693232059478759765625f,-2.59050001625716586062253554700873792171478271484375f,0.176400032794475691844127140939235687255859375f,0.90680005136579300284438431845046579837799072265625f,0.3034000165373085255993146347464062273502349853515625f,0.3866999985456460731114702866761945188045501708984375f,-0.37120001879632458230418023958918638527393341064453125f,0.13180000368207644978468806584714911878108978271484375f,0.494800023739040195636107455356977880001068115234375f,0.51050005775690154852242130800732411444187164306640625f,0.552399988269806296870001460774801671504974365234375f,0.004899979552626998469122554524801671504974365234375f,-0.34549996075034083986565747181884944438934326171875f,-1.5853999791204917979570154784596525132656097412109375f,1.27020004670619979236789731658063828945159912109375f,-0.115500001281499731220492321881465613842010498046875f,-0.0743999811768532026690081693232059478759765625f,-0.51480001088678850607038839370943605899810791015625f,1.19440002230405895033982233144342899322509765625f,-0.101899987360834387573049752973020076751708984375f,-0.82010002186298402993003264782601036131381988525390625f,-0.60479998778998866892919750171131454408168792724609375f,-0.0924999884888528800530593798612244427204132080078125f,0.1961000138089061817225910999695770442485809326171875f,-1.1100999588683244656550641593639738857746124267578125f,-0.14769999440312353566895353651489131152629852294921875f,-0.22609999283552273396935561322607100009918212890625f,-0.53059997330009966987063307897187769412994384765625f,0.2910999473869804177184050786308944225311279296875f,1.4256000057101256839331426817807368934154510498046875f,-0.364900012707710441617336982744745910167694091796875f,0.2431000232934952265395622816868126392364501953125f,0.092699993211031284090495319105684757232666015625f,-0.59249998942017523262393297045491635799407958984375f,0.09800000832229860403543852953589521348476409912109375f,0.14219996041357507721158981439657509326934814453125f,-0.80720002970397519703737998497672379016876220703125f,0.91130002667084319956103399817948229610919952392578125f,-0.2600000035949048271533001752686686813831329345703125f,-0.790800022614002617871165057295002043247222900390625f,-0.2424000227846210275828298108535818755626678466796875f,2.03630003820061755703818562324158847332000732421875f,0.0838000246591870112666811110102571547031402587890625f,-0.080399947902560331414179017883725464344024658203125f,0.5961000161185860957147042427095584571361541748046875f,-1.073100009286402922725756070576608180999755859375f,-0.379900008293241331358558454667218029499053955078125f,0.014600027287006156484494567848742008209228515625f,0.414700005061179322041198247461579740047454833984375f,-1.1749999809265130323865378159098327159881591796875f,-0.1872999820344147803297119025955908000469207763671875f,-0.2985000014454126660012889260542578995227813720703125f,-1.008700048542768623605070388293825089931488037109375f,-0.92510001653581841463136470338213257491588592529296875f,0.23710001389458812415256261374452151358127593994140625f,-0.203900015471875750705521568306721746921539306640625f,0.35279998735785422780963926925323903560638427734375f,0.4097000320136547912852620356716215610504150390625f,0.08460001737773392704866637359373271465301513671875f,-1.658100074404478707634780221269465982913970947265625f,-0.33949998299777472965388369630090892314910888671875f,0.5151000214159484613674067077226936817169189453125f,-0.566500028207899131160729666589759290218353271484375f,2.691500055551529158037737943232059478759765625f,-0.1316999788761135103953847647062502801418304443359375f,0.53790007493496005963606876321136951446533203125f,-0.0259999772310262056151941578718833625316619873046875f,1.582700000214575908330516540445387363433837890625f,-0.43800000359117985393453409415087662637233734130859375f,1.11740002943426386394776272936724126338958740234375f,-0.421499991066753787460896774064167402684688568115234375f,0.20660000102743492078616327489726245403289794921875f,-0.36610002328977009344868065454647876322269439697265625f,-0.78230000038146929597360212937928736209869384765625f,-0.18499999612569784090965185896493494510650634765625f,1.5743000022292130690715339369489811360836029052734375f,-0.2978000226736077227229770869598723948001861572265625f,0.19369997707307362588835530914366245269775390625f,1.05229999755024916652246247394941747188568115234375f,-0.97089998874664384942434480763040482997894287109375f,1.0556000706791877075119145956705324351787567138671875f,0.11629990125894540398121534963138401508331298828125f,1.4121999883532525554841186021803878247737884521484375f,-1.21579997999668165675757336430251598358154296875f,0.3155000294446939079762159963138401508331298828125f,0.2709999748468394642486600787378847599029541015625f,0.86350006168335646794531612613354809582233428955078125f,1.629199989400059056521286038332618772983551025390625f,0.12080002835094882929212189992540515959262847900390625f,0.3050999689638611567232828747364692389965057373046875f,0.426600020790100842305037076584994792938232421875f,0.21580000141710053984667183613055385649204254150390625f,0.09610001190900785417170482105575501918792724609375f,-0.090400005346536527195411281354608945548534393310546875f,-1.128200055587292194303472570027224719524383544921875f,-1.82720002666786296430245783994905650615692138671875f,0.790000032335519630777298516477458178997039794921875f,-0.2852999769642945437908565509133040904998779296875f,-0.632600018770992367223016117350198328495025634765625f,0.07620000303834661981028375521418638527393341064453125f,0.23230000237449999200833872237126342952251434326171875f,0.242199998933076809937148254903149791061878204345703125f,0.93949999707937248416556030861102044582366943359375f,1.30949998217821050872089472250081598758697509765625f,-0.6989000573068862021131053552380762994289398193359375f,0.44279999346733045495483338527265004813671112060546875f,1.191900015859305828058722909190692007541656494140625f,0.285199956628679363035416827187873423099517822265625f,0.22089997652769088887225734652020037174224853515625f,-0.7030000371187921093252271020901389420032501220703125f,-0.97770000021457637018329478451050817966461181640625f,-1.99620001492202181481161460396833717823028564453125f,1.18170003609657303655922078178264200687408447265625f,0.287500022351742767767746045137755572795867919921875f,0.59579999608993450266325453412719070911407470703125f,-0.00280002634301812491912642144598066806793212890625f,1.3507000302366913846441320856683887541294097900390625f,0.383099992466718053396590448755887337028980255126953125f,-0.0686999817296858328319331121747381985187530517578125f,-0.3044000003248452657089728745631873607635498046875f,0.08740000413954251978765341846155934035778045654296875f,0.78009996322691399495141695297206752002239227294921875f,-0.4891999892324208065730317684938199818134307861328125f,0.408400000257790107838218318647705018520355224609375f,0.24669999020099542264006231562234461307525634765625f,1.0119000109046691893155411889893002808094024658203125f,0.9844999681711197325739703956060111522674560546875f,0.7214999748617412222984057734720408916473388671875f,0.273699985492230180028627728461287915706634521484375f,0.0775999983191493214462752803228795528411865234375f,-0.232600007408856779278494286700151860713958740234375f,-0.01280002416372294504753881483338773250579833984375f,-0.62930000222921389507746425806544721126556396484375f,0.7271000355944039483802043832838535308837890625f,-0.0702000307247043764391492004506289958953857421875f,0.34489999236762536316547311798785813152790069580078125f,-0.4863000186800956470278833876363933086395263671875f,0.3899999877810476078821011469699442386627197265625f,-0.01240002962052833002104534898535348474979400634765625f,1.088599998348950492754738661460578441619873046875f,0.63569998037368014021097906152135692536830902099609375f,-0.038900003141165040432269961456768214702606201171875f,0.96709999450445172097801105337566696107387542724609375f,0.5943000463366505670137485140003263950347900390625f,0.735599995391071015404804711579345166683197021484375f,0.21370002363920281851505933445878326892852783203125f,-0.234199988785385837530839125975035130977630615234375f,0.2483000035405158456569552072323858737945556640625f,-0.0368999911792575741031896541244350373744964599609375f,-0.29910003584027311518411806900985538959503173828125f,-0.2839000015392898479404948375304229557514190673828125f,0.85150001785159101785893653868697583675384521484375f,0.876500009320676287671858517569489777088165283203125f,1.0690999880075455763517311424948275089263916015625f,0.55879991683065899366056328290142118930816650390625f,0.4645000015497213752269090036861598491668701171875f,0.73249997384846210479736328125f,-1.182699987101554217616694586467929184436798095703125f,1.548800049152970093047088084858842194080352783203125f,0.9510999852359294237658104975707828998565673828125f,0.08109999946653889679737403639592230319976806640625f,0.350699971544742084006429649889469146728515625f,-0.1573000539511444362261727292207069694995880126953125f,0.9437999888777728330069294315762817859649658203125f,-0.18180002547502471799134582397527992725372314453125f,-0.5243999789416786683204918517731130123138427734375f,1.0430000366717582238607064937241375446319580078125f,1.5975000150501728057861328125f,0.38009997723400601454812886004219762980937957763671875f,-1.4076000376582147310955406283028423786163330078125f,0.27580001359879968703125996398739516735076904296875f,-0.130700020718574450029336730949580669403076171875f,0.9338999807149175325093892752192914485931396484375f,0.43689996359348359789009919040836393833160400390625f,0.11129996123611984870649393997155129909515380859375f,-1.59659997593760483169944563996978104114532470703125f,-0.6670000007301570832396464538760483264923095703125f,0.60599999451637298619743887684307992458343505859375f,0.9409999765604732946400190485292114317417144775390625f,0.22469998269081070674246802809648215770721435546875f,1.4547000081717964548033705796115100383758544921875f,0.467899946188926829648835337138734757900238037109375f,0.77259997946918035349739284356473945081233978271484375f,-1.787399982905387485487835874664597213268280029296875f,-0.97639999606311300706096290014102123677730560302734375f,0.96779999316930798158864490687847137451171875f,0.38539997599124919958057944313623011112213134765625f,-0.35870001812577267941151148988865315914154052734375f,0.567700016158819042999539306038059294223785400390625f,0.134399997549504046645552080008201301097869873046875f,-1.08569997810125329351649270392954349517822265625f,1.0937000282771887071930905221961438655853271484375f,0.17249995425343467303491706843487918376922607421875f,0.38269996309205811524378759713727049529552459716796875f,0.3788000575721266471873605041764676570892333984375f,1.5597000021927061030879713143804110586643218994140625f,-0.944900009131431684039625906734727323055267333984375f,-0.3639000107780099302345888645504601299762725830078125f,0.364299966615438730599407790577970445156097412109375f,-1.2686000198438762698316395471920259296894073486328125f,-0.3003000167131428810307625099085271358489990234375f,-0.19180002035275134364411542264861054718494415283203125f,0.90079997612237949766722522326745092868804931640625f,-0.5531999937847256187950506500783376395702362060546875f,0.056900002932548598977291476330719888210296630859375f,0.2516000108808278401539837432210333645343780517578125f,0.0709000049889088046484175720252096652984619140625f,0.581200045394897557571312063373625278472900390625f,-0.465900002604723084687066148035228252410888671875f,1.072600010985134577623512086574919521808624267578125f,1.0757000076055529547147671109996736049652099609375f,0.51429999686479543896666655200533568859100341796875f,-1.045400040364265148440381381078623235225677490234375f,-0.877700019511580276088125174283050000667572021484375f,0.065400014287233521059761187643744051456451416015625f,-0.767300050151348056459710278431884944438934326171875f,0.95029999868273762331227771937847137451171875f,0.25389995582997837431094012572430074214935302734375f,-0.30509994199275869419807349913753569126129150390625f,-0.229599992865324065149934540386311709880828857421875f,-0.865399998342990528499285574071109294891357421875f,0.1696000051587820411924667496350593864917755126953125f,0.02069998868107791878401258145458996295928955078125f,0.04339999127984039972716345801018178462982177734375f,-0.843000004708767125549684351426549255847930908203125f,-0.00969999908208862837710739768226630985736846923828125f,-1.28939992544651005346167949028313159942626953125f,-1.385099960917234085400195908732712268829345703125f,0.68939998326301488162926034419797360897064208984375f,1.000699983093142009238363243639469146728515625f,0.17579999892115605319986570975743234157562255859375f,0.3262000035226346739136715768836438655853271484375f,0.27440003198981255394528488977812230587005615234375f,-0.17079997694194304358461522497236728668212890625f,-0.199800044488906536344074993394315242767333984375f,0.340699983912707171640477099572308361530303955078125f,0.7417999950766560601778110140003263950347900390625f,-0.7423999942302703658469908987171947956085205078125f,-0.66200001764297500272959950962103903293609619140625f,0.01460001663267629368192501715384423732757568359375f,-0.046099985999613846132660910370759665966033935546875f,-1.1846000108212229040560714565799571573734283447265625f,-0.187799990189075227675630230805836617946624755859375f,-0.55020004596114180372978808009065687656402587890625f,-0.9860000060461462378924579752492718398571014404296875f,1.3989999708756801766185162705369293689727783203125f,0.164499993763863816553794094943441450595855712890625f,-0.67319997500926287425926375362905673682689666748046875f,0.63409997960701591779297814355231821537017822265625f,0.5478999913021918022337786169373430311679840087890625f,0.92019999809116093114624845838989131152629852294921875f,-1.18519996713399944354705439764074981212615966796875f,-0.535900037827342856644463608972728252410888671875f,1.64180001742839909439908296917565166950225830078125f,-0.11399996191635697329047616221942007541656494140625f,-0.0809999879598617411602390347979962825775146484375f,0.4217000306576490498144949015113525092601776123046875f,0.4772000118970869397827527791378088295459747314453125f,-0.7894999993145461747445779110421426594257354736328125f,-0.009699996921419984374779232894070446491241455078125f,-0.777900005123018711827853621798567473888397216796875f,-0.5023999812662598429824356571771204471588134765625f,0.68750000607222272464014167780987918376922607421875f,-0.38200002419948575482777641809661872684955596923828125f,0.16439998676478884132023949860013090074062347412109375f,0.680299985942244500591868927585892379283905029296875f,0.99730000667721008245081293352995999157428741455078125f,-0.381700008156895531641339402995072305202484130859375f,-0.4731000068277122494464492774568498134613037109375f,0.79700000691413830367082482553087174892425537109375f,0.325800009873508766844452111399732530117034912109375f,-1.541100038731098464950264315120875835418701171875f,0.5610000025629997555398631448042578995227813720703125f,-0.714000004738569504780798524734564125537872314453125f,-0.221099997082352839328223126358352601528167724609375f,0.018000007763504743962812426616437733173370361328125f,0.114800016698241247325995573191903531551361083984375f,0.219100001567602564733761028037406504154205322265625f,-0.748600000360607698013382105273194611072540283203125f,0.0976000237256291480747449895716272294521331787109375f,-0.95180000230371941682960823527537286281585693359375f,-0.41319999080449332229392211957019753754138946533203125f,0.7206999868184327251441345651983283460140228271484375f,-0.31620001462399915936885008704848587512969970703125f,-0.0388000178605316659030677328701131045818328857421875f,-0.4157999633073803380511890281923115253448486328125f,0.011099992761015808895308509818278253078460693359375f,1.0856999882340421237358896178193390369415283203125f,-0.3487999963283545667991347727365791797637939453125f,0.33310000302791564763538190163671970367431640625f,-0.5126999825567006840998374173068441450595855712890625f,0.26159998256862149190737909520976245403289794921875f,-0.7913000372320408981607897658250294625759124755859375f,0.215499989248812140463229525266797281801700592041015625f,-0.2047000170379877914683675044216215610504150390625f,-0.768599990451335646213237851043231785297393798828125f,-0.6693999872118234772955247535719536244869232177734375f,0.11519999250322576767047166867996565997600555419921875f,-0.520000014901160767522014793939888477325439453125f,0.640299989891051968271540317800827324390411376953125f,1.0465000337213279379966479609720408916473388671875f,0.106200010488926910312557083670981228351593017578125f,1.377800072406233145017040442326106131076812744140625f,0.17830001136362572111693225451745092868804931640625f,-0.0315000087618830804103708942420780658721923828125f,-0.07520000271052129647841866244561970233917236328125f,0.18180001325607264561767806299030780792236328125f,-0.31089996740073022607475650147534906864166259765625f,-0.8434999704509975604338478660793043673038482666015625f,0.2851999621793626094046203434118069708347320556640625f,1.4038999551966793166712932361406274139881134033203125f,-0.8249000265657902009053259462234564125537872314453125f,0.55100001426041078644857407198287546634674072265625f,-0.970599979819357638888277506339363753795623779296875f,-0.15080000544041372023684743908233940601348876953125f,0.5971999777734282588426140137016773223876953125f,0.707199913251400857916451059281826019287109375f,0.93970004259347916786282439716160297393798828125f,0.066599993520974454241923012887127697467803955078125f,0.357300018523634133060795647907070815563201904296875f,-1.782999981388448507146904375986196100711822509765625f,0.89800003004074113732713158242404460906982421875f,-0.79109998300075545785148278810083866119384765625f,0.013100000047683746373650137684307992458343505859375f,0.4341999767899509254931444957037456333637237548828125f,-0.91189999011754963031961551678250543773174285888671875f,-0.963100015321373792431813853909261524677276611328125f,0.266599999779462581983580093947239220142364501953125f,0.94130000847280026476937564439140260219573974609375f,-1.0383999947071076430660241385339759290218353271484375f,0.511400028324127564616219387971796095371246337890625f,0.2140000051110984014002269759657792747020721435546875f,-1.178000031530857416584012753446586430072784423828125f,0.899800039124489359210201655514538288116455078125f,1.0439000482544305015153440763242542743682861328125f,-0.2804000215142963270409381948411464691162109375f,-1.2695000544860963742621606797911226749420166015625f,-0.132300001722574389617648193961940705776214599609375f,0.379100021386146845969733476522378623485565185546875f,-0.67670003676712564555373319308273494243621826171875f,0.65739999899864187682396732270717620849609375f,-0.2013999858558168654099063132889568805694580078125f,-0.25060003346353820763425801487755961716175079345703125f,0.819200035649538360615906640305183827877044677734375f,1.3594000469654801133145838321070186793804168701171875f,-0.01119998869597882418247536406852304935455322265625f,-0.37020001854300499388727985206060111522674560546875f,0.4545000224858524173754403818747960031032562255859375f,-0.69900005003810006254383324630907736718654632568359375f,0.3867000697731981517790700308978557586669921875f,0.45849999179691069972619743566610850393772125244140625f,-0.8128000530719763361275909119285643100738525390625f,0.80850001329183596165961489532492123544216156005859375f,-0.645900010353326781142868640017695724964141845703125f,0.76250003058463367011654554517008364200592041015625f,-0.683700040775538031567748475936241447925567626953125f,1.064400033405423329213590477593243122100830078125f,-0.1317999655723571805765459430404007434844970703125f,1.339300012063980194199075413052923977375030517578125f,-1.230900037175416628087987191975116729736328125f,0.40440004502832938015899344463832676410675048828125f,-0.4461999805003404873104955186136066913604736328125f,1.6464000266849996290119406694429926574230194091796875f,-1.35549999181926228430938863311894237995147705078125f,1.387200033988058134326593062723986804485321044921875f,-0.36580001639276726876914835884235799312591552734375f,-0.9564999911785125874530422152020037174224853515625f,-1.282600003683565859091686434112489223480224609375f,-0.3537000220000745542847653268836438655853271484375f,1.6265999853253365614591530174948275089263916015625f,-1.495200023832917057831082274788059294223785400390625f,-0.092700004535914093395376767148263752460479736328125f,0.828300024327635942000824798014946281909942626953125f,0.997399981489776710219530286849476397037506103515625f,0.040400000652670797762766596861183643341064453125f,0.611300032575428531345096416771411895751953125f,0.14489997552931299651390872895717620849609375f,-0.34280002084821459273911159471026621758937835693359375f,0.29080000618547163782068309956230223178863525390625f,-0.871199998903274153150277925305999815464019775390625f,-0.38300001901388192049324743493343703448772430419921875f,0.47660001430809473266236864219536073505878448486328125f,-0.8287999894365662978401587679400108754634857177734375f,-0.096200033958256880861625859324703924357891082763671875f,-0.09220001539885969155108114136965014040470123291015625f,-0.707499978914856608724903708207421004772186279296875f,-0.7496000472545627513198951419326476752758026123046875f,1.36590001176893682810487007373012602329254150390625f,-0.4810000088214874125469577847979962825775146484375f,-0.23379999632835346545789434458129107952117919921875f,0.6414999978840345562502989196218550205230712890625f,0.7123000063419340222026221454143524169921875f,0.56649998134374612845931551419198513031005859375f,0.410600032941997650226539917639456689357757568359375f,0.3425000125914807735938438781886361539363861083984375f,-0.48509998461008052572651649825274944305419921875f,-0.2103999828457834286865590911475010216236114501953125f,1.579100002983211847862321519642136991024017333984375f,-0.54139997784644300349299328445340506732463836669921875f,0.712699975366890559058674625703133642673492431640625f,-0.55329999758005055099374658311717212200164794921875f,1.490700044336914942277871887199580669403076171875f,-0.018300011438131758012559657800011336803436279296875f,0.8165000166594982289325344027020037174224853515625f,0.434199984240532188550787395797669887542724609375f,-1.007800004456937070784761090180836617946624755859375f,0.61920000718831946784348474466241896152496337890625f,-0.63579998234361401188152740360237658023834228515625f,-1.14960002877712241797780734486877918243408203125f,1.82169998677373001783053041435778141021728515625f,0.74059998633861567896019550971686840057373046875f,-0.9924999904260036753811391463386826217174530029296875f,-1.5740999418884522587092078538262285292148590087890625f,0.217900076238810580520066650933586061000823974609375f,-0.1638000064238911424041589270927943289279937744140625f,1.184300007407367072431725318892858922481536865234375f,-0.536300028738378120607421806198544800281524658203125f,-1.1798999861836427793804205066408030688762664794921875f,0.5929999568760404571321487310342490673065185546875f,1.115500046283007140601739592966623604297637939453125f,0.0784999793171881066200512577779591083526611328125f,1.96899997761845657606727399979718029499053955078125f,-0.69630000586509677162894149660132825374603271484375f,-0.57049997863173462775421285186894237995147705078125f,-0.84249994531273930675752126262523233890533447265625f,-0.0130000008717179316164447300252504646778106689453125f,-0.164099976217001231049152920604683458805084228515625f,0.71399999402835956363588820750010199844837188720703125f,-0.482700036461651882380152756013558246195316314697265625f,0.52650003347918439278174673745525069534778594970703125f,1.273700062941015342943273935816250741481781005859375f,-0.3660000158995391217331416555680334568023681640625f,-1.63420004965662979401486154529266059398651123046875f,1.1506000162154437216344149419455789029598236083984375f,-0.5637000202491881939437234905199147760868072509765625f,-0.490200000363588461738117985078133642673492431640625f,1.58000001486390839744444747339002788066864013671875f,-1.212600013443827950965214768075384199619293212890625f,-0.330199971641599976379666259163059294223785400390625f,-0.475900013706087587905813052202574908733367919921875f,1.063499987959861670105965458787977695465087890625f,-0.7917999876260761737967186491005122661590576171875f,-0.25850000412762152990353570203296840190887451171875f,0.35130000403970440547851694645942188799381256103515625f,-0.098800020319223857967472213204018771648406982421875f,0.36250003371387806705428147324710153043270111083984375f,-0.1412999974459407948046418823651038110256195068359375f,-0.789500054746867352406525242258794605731964111328125f,-0.8885000305771828532641620768117718398571014404296875f,0.523100020164251500176533227204345166683197021484375f,0.6860000170916318307234860185417346656322479248046875f,0.481600006932020097139002245967276394367218017578125f,0.6347000329449781208523972964030690491199493408203125f,-0.1107000150561336937471423880197107791900634765625f,-0.4330000049322844102306362401577644050121307373046875f,0.5752999901890749612221043207682669162750244140625f,-0.4085000383257868517006272668368183076381683349609375f,-0.0777000130295757429621517076157033443450927734375f,-0.6488000029966232151679150774725712835788726806640625f,-0.1380000153630971393425852511427365243434906005859375f,0.19780001788660828143662229194887913763523101806640625f,0.75359999708235225757135822277632541954517364501953125f,-0.91500003056600676121234982929308898746967315673828125f,-0.38229999812766812805620020299102179706096649169921875f,0.26309998142123180997486997512169182300567626953125f,0.61159999948143894954455390688963234424591064453125f,-1.46520002815425431919038601336069405078887939453125f,0.667200009959935780301520935609005391597747802734375f,0.591699984315038118865004435065202414989471435546875f,-0.639099958130717293869338391232304275035858154296875f,0.8596000316083429737545884563587605953216552734375f,-0.376499953627586592119769193232059478759765625f,0.571999976068735094258954632095992565155029296875f,-0.76479998637437773822966846637427806854248046875f,0.78819999359846093511805520392954349517822265625f,-1.007899942910672308471475844271481037139892578125f,-0.2727999962240457154649675430846400558948516845703125f,-0.7592000242874028526074425826664082705974578857421875f,-0.38959998362660375192945139133371412754058837890625f,0.8756000190094113122540875338017940521240234375f,-0.680099997648596410471100170980207622051239013671875f,0.40009999325275413806224378276965580880641937255859375f,-0.3192000031650066294020007262588478624820709228515625f,-0.9576000224217775524948592646978795528411865234375f,0.3936000152617695757584215243696235120296478271484375f,0.96960002035796666319811265566386282444000244140625f,0.114900013303756765026264474727213382720947265625f,-0.3907000514149669978536394410184584558010101318359375f,0.254700036334991164466146074119023978710174560546875f,1.0356000214867295827758653103956021368503570556640625f,-0.83510002435892871641698320672730915248394012451171875f,-0.0889999926872551139211964255082421004772186279296875f,-0.07400000287592389813795534792006947100162506103515625f,0.281499989092349522934455308131873607635498046875f,-0.7354999945759768120723265383276157081127166748046875f,0.907699989932775341827664306038059294223785400390625f,-0.84380003093630084887166731277829967439174652099609375f,0.389600001433491993640956252420437522232532501220703125f,-0.3466000178098678219384964904747903347015380859375f,-0.6293000240221624519421084187342785298824310302734375f,1.04040004721880041671511207823641598224639892578125f,-0.6128000236794355348735052757547236979007720947265625f,1.07170000599622738945981836877763271331787109375f,-1.2387000214040282397576220319024287164211273193359375f,0.6760000255107883049277006648480892181396484375f,-0.4537999499857414686943002379848621785640716552734375f,-0.439600031422079151610660119331441819667816162109375f,-0.258099989914894134557243887684307992458343505859375f,-0.02669999895542851664487216112320311367511749267578125f,-0.349100018201023598596322017328930087387561798095703125f,0.620000008940696289982952293939888477325439453125f,-0.6927000100865965404040025532594881951808929443359375f,0.9627999659746888649891616296372376382350921630859375f,-0.4745000312775375927998311453848145902156829833984375f,1.0507999940410261974221839409437961876392364501953125f,-0.189100005888939382003854916547425091266632080078125f,1.038399982190131964898682781495153903961181640625f,0.778000003367662618103395288926549255847930908203125f,-0.70380001510381617890743655152618885040283203125f,1.37560000198483489697309778421185910701751708984375f,-0.91509997745752347242387259029783308506011962890625f,-0.80590002659559178965764658642001450061798095703125f,-1.3077999316275121799435510183684527873992919921875f,0.45520002852678309590572780507500283420085906982421875f,0.284100012333690532528862604522146284580230712890625f,-0.1583000006161630057732736531761474907398223876953125f,-0.053800015271454615373158958391286432743072509765625f,-0.128299993556737934596867489744909107685089111328125f,0.7487000024050474156211976151098497211933135986328125f,-0.319199988710880067088737632730044424533843994140625f,-1.3081999956846235289020796699333004653453826904296875f,0.48850001090765038380681062335497699677944183349609375f,0.433000001952052127052184005151502788066864013671875f,-0.6675999913156029919036882347427308559417724609375f,0.28170000078082058081463401322253048419952392578125f,-0.577700014966726183018863594043068587779998779296875f,0.833299986292421746014724703854881227016448974609375f,-0.49000000067055271557592277531512081623077392578125f,-0.747600011208653381089561662520281970500946044921875f,0.2553000086665153656184656938421539962291717529296875f,1.30420000696480276047850566101260483264923095703125f,-0.4057999880433083017550188742461614310741424560546875f,-0.8467000028669833255889898282475769519805908203125f,0.19919998066425304017457165173254907131195068359375f,-0.17679998762607596063389792107045650482177734375f,-0.3856000000476829114859356195665895938873291015625f,-1.3277000266268854655749009907594881951808929443359375f,0.3716000383734705536653564195148646831512451171875f,0.061299975951015550634792816708795726299285888671875f,-1.156400007015466524507019130396656692028045654296875f,1.1108999972403046374580526389763690531253814697265625f,-0.90280005545616148765475372783839702606201171875f,1.1603000423431402143847890329197980463504791259765625f,-0.05300003123283403283494408242404460906982421875f,-1.2401999911621210959111749616567976772785186767578125f,0.9497000309705736498244732501916587352752685546875f,-1.02480002094507227639041957445442676544189453125f,-0.81020001630783067270158426254056394100189208984375f,-0.302499986812472254626982248737476766109466552734375f,-0.38760001858472836744340384029783308506011962890625f,0.7769000265508891889254527995944954454898834228515625f,0.891599995160103020452879718504846096038818359375f,-0.34589998964071266840392127051018178462982177734375f,0.771399952626229179486472276039421558380126953125f,-0.7447999813079835274010065404581837356090545654296875f,-0.4512000120162955596470055752433836460113525390625f,-0.9818999777495864833554151118732988834381103515625f,-0.0723000149846075856885363464243710041046142578125f,-0.3874000460863111783282874966971576213836669921875f,2.27249996364116668701171875f,0.144599981242418351712331059388816356658935546875f,0.7722000187516204761095650610513985157012939453125f,0.40350000703334831797519655083306133747100830078125f,-1.0849999880790708317590542719699442386627197265625f,-0.91189999026656121117184738977812230587005615234375f,-0.481099995666742330513443448580801486968994140625f,-0.4131000028789042488597260671667754650115966796875f,1.4552999647080895329054328612983226776123046875f,0.05390000202357769776284612817107699811458587646484375f,-0.467500012703240042100105711142532527446746826171875f,0.1309000146001579878429765813052654266357421875f,-0.4688000098511577728999100145301781594753265380859375f,-0.4938000056415796645836735478951595723628997802734375f,0.665600033910572985718090421869419515132904052734375f,-0.340800008383393304711717064492404460906982421875f,-0.741200014922023253660654518171213567256927490234375f,-0.43680001429915360500189080994459800422191619873046875f,0.282100008437037474973152484380989335477352142333984375f,-0.15220000191330906869069394815596751868724822998046875f,0.6451999761119482368343369671492837369441986083984375f,0.317800018724799127056712677585892379283905029296875f,0.35020003247559083892070930232875980436801910400390625f,0.0125999948918822379795301458216272294521331787109375f,-0.69760000070333472166339561226777732372283935546875f,0.9181999959826470103507745079696178436279296875f,-0.2945000151097776353026347351260483264923095703125f,0.279700020390749148901932130684144794940948486328125f,0.13899999505281446232629605219699442386627197265625f,-0.166899986243247866468664142303168773651123046875f,0.525299990300834185319445168715901672840118408203125f,-1.26070000879764609891253712703473865985870361328125f,1.64910001263171412944075200357474386692047119140625f,-0.70690000472068703629702213220298290252685546875f,1.27419998301118564398848320706747472286224365234375f,0.344000006526708812515380486729554831981658935546875f,-0.4131000145763159547840359664405696094036102294921875f,0.433600028449297791866001716698519885540008544921875f,-0.70049998712539685019606849891715683043003082275390625f,0.0539000023961069274491819669492542743682861328125f,-1.49850003664940611969313977169804275035858154296875f,0.50340001646280274627542894450016319751739501953125f,0.0261999994993207252491629333235323429107666015625f,0.1530000050067901185002483543939888477325439453125f,-0.5041000052928925612150123924948275089263916015625f,-0.4966999830484386535545127117075026035308837890625f,0.9617000159800053182834744802676141262054443359375f,-1.109300011169910504804647644050419330596923828125f,-1.39349994444847169461354496888816356658935546875f,0.561100008779764181099380948580801486968994140625f,0.1149999853968617369304183739586733281612396240234375f,1.6026999102115642159560593427158892154693603515625f,1.2203000037491322782301494953571818768978118896484375f,-0.2689000101447103219243217608891427516937255859375f,-0.40409997906684846924463272443972527980804443359375f,0.034600007319451009379918104968965053558349609375f,0.368899992859363923258797512971796095371246337890625f,-0.233600035452841581218308419920504093170166015625f,-0.767999993979930906107256305404007434844970703125f,-0.056599994564056288481879164464771747589111328125f,0.174999999329447764040423862752504646778106689453125f,-0.18340002510547659397843744955025613307952880859375f,0.014600023785233562367835702389129437506198883056640625f,0.366000000514089929293248815156402997672557830810546875f,-0.25310001450181018878282657169620506465435028076171875f,0.12950001794099830210171830913168378174304962158203125f,0.5802999977886675875282662673271261155605316162109375f,-0.9772000137224789995116225327365100383758544921875f,0.65740001326650376878291126558906398713588714599609375f,0.7138999991923566934559630681178532540798187255859375f,-0.8839999965056772879279378685168921947479248046875f,-0.72719999319612937949131037385086528956890106201171875f,-0.055399998864531507791753028868697583675384521484375f,1.63050002873688892890413626446388661861419677734375f,-0.5414999917000529183752632889081723988056182861328125f,-1.20540002665519718760833711712621152400970458984375f,1.08069999777078606939539895392954349517822265625f,-0.39850001478195196114029386080801486968994140625f,0.4075000302493574366735629155300557613372802734375f,-0.39220000552684053385377183076343499124050140380859375f,0.0709999978691342459313773360918276011943817138671875f,-0.05539999849200250015002211512182839214801788330078125f,0.518500008970498971194729165290482342243194580078125f,-0.741300009590387443125791833153925836086273193359375f,0.13460003376901141791677218861877918243408203125f,0.9650000138953325201640609520836733281612396240234375f,-0.77470001528710052962622967243078164756298065185546875f,0.399999995268881303189800746622495353221893310546875f,-0.2013999899536373927588783772080205380916595458984375f,0.619299987812339747250689470092765986919403076171875f,-0.79230004806518561366601716144941747188568115234375f,-0.4990000061914316997757623539655469357967376708984375f,0.47130000646114300622002701857127249240875244140625f,-0.20549999666214002758124479441903531551361083984375f,0.0870000158548354107779232435859739780426025390625f,-0.23920000301599497749549527725321240723133087158203125f,-1.52009995957612975558959078625775873661041259765625f,1.3105000097006562764789805441978387534618377685546875f,-0.06510002648234358701984092476777732372283935546875f,0.550000007450580863377354035037569701671600341796875f,-0.53990000197887422217490893672220408916473388671875f,-1.6489999536275867786372373302583582699298858642578125f,-0.340499971628189346262161052436567842960357666015625f,0.57239998867958785666587573359720408916473388671875f,0.876899948543310880921808347920887172222137451171875f,-0.65109996538013259481658678851090371608734130859375f,-0.3341999966457489534832347999326884746551513671875f,0.807699923175573264444437882048077881336212158203125f,1.185599982762337134545305161736905574798583984375f,-1.260200016009807910677409381605684757232666015625f,0.268700023937224585779404151253402233123779296875f,0.1128000134348869920586366788484156131744384765625f,0.28660003047585380642203745082952082157135009765625f,-2.789500025689601869771649944595992565155029296875f,-1.1048999472916125608179527262109331786632537841796875f,0.129699984776973309408276691101491451263427734375f,-0.335200016531348676807056108373217284679412841796875f,-0.5977999910831446328529636957682669162750244140625f,0.098000034511090294842006187536753714084625244140625f,-1.449800005894898990987940123886801302433013916015625f,-2.23860002375543132302482263185083866119384765625f,-1.189399997717142554165548062883317470550537109375f,-0.874399962624907711727928472100757062435150146484375f,0.196200032877921870522186509333550930023193359375f,0.7581000447511667683642144766054116189479827880859375f,2.084300022196769130999882690957747399806976318359375f,0.671200001734494566818511884775944054126739501953125f,0.0439999606311314295226111426018178462982177734375f,-0.62459999584555614404735024436376988887786865234375f,1.36499999225139578840071408194489777088165283203125f,-0.6037000127613547650895498009049333631992340087890625f,-0.444799947333336831434280611574649810791015625f,-0.698799966600538002836628947989083826541900634765625f,0.1964000381588935084664626629091799259185791015625f,-1.371899968436359973367189013515599071979522705078125f,0.4871999761343002166569249311578460037708282470703125f,-0.74840002674460404108458533301018178462982177734375f,1.87130001249611410685247392393648624420166015625f,-0.069399993544816851454015704803168773651123046875f,0.28339997682571382853211616748012602329254150390625f,-0.3541999811112883644881321743014268577098846435546875f,-1.35289999238252622859590701409615576267242431640625f,-0.05799998444318799073471382143907248973846435546875f,-0.24800000396370958100078496499918401241302490234375f,1.12879999811649245344824521453119814395904541015625f,-0.91350000166892986186439884477294981479644775390625f,1.1189000050038107048067104187794029712677001953125f,0.29750006314367094528705592892947606742382049560546875f,-1.0644000368326904659710407941020093858242034912109375f,-0.204300018471479827297798692598007619380950927734375f,-0.46829995138645141850020081619732081890106201171875f,1.3813000364124776719876308561651967465877532958984375f,-0.848499999433755647260113619267940521240234375f,0.085500020168721935220901286811567842960357666015625f,-0.066300024864077489183955549378879368305206298828125f,0.0464000058420002137182791557279415428638458251953125f,0.3787000234156849831634872316499240696430206298828125f,0.8754000246062876300356947467662394046783447265625f,1.6228000189483164916737223393283784389495849609375f,-0.39080000352188959400479006944806315004825592041015625f,0.311400044640899320569360497756861150264739990234375f,0.38820002477914117289259365861653350293636322021484375f,0.096000016570091162293465458787977695465087890625f,0.23470004847943837233970043598674237728118896484375f,-1.2832999937057494310721494912286289036273956298828125f,-0.4248999942302706944730061877635307610034942626953125f,2.00990005554854889169291709549725055694580078125f,0.9847000535458325654047939679003320634365081787109375f,-0.882600011134147877100986079312860965728759765625f,-0.7382999479591842995063188936910592019557952880859375f,-0.116600012743473424592366427532397210597991943359375f,0.659700035068393031423283900949172675609588623046875f,1.6328000258028507740704071693471632897853851318359375f,1.2676000069618227339418581323116086423397064208984375f,-0.32130002203285723538783713593147695064544677734375f,-0.3692000301361086922469212368014268577098846435546875f,-0.47490001874268106263343725004233419895172119140625f,0.8085999997645612058505548702669329941272735595703125f,0.18619998593926379015783822978846728801727294921875f,0.1776000038325786789528137887828052043914794921875f,0.640099955850839119619877237710170447826385498046875f,-0.29409999419152743627137169823981821537017822265625f,-0.937499987855553218452087094192393124103546142578125f,0.1559000415712599174611341368290595710277557373046875f,-0.59660002943277401499244660953991115093231201171875f,0.2803999852299685624501535130548290908336639404296875f,0.01280003906488413889519506483338773250579833984375f,-0.66899997746944439569460882921703159809112548828125f,-1.143099991703033335710415485664270818233489990234375f,-0.403399974963068785882569500245153903961181640625f,-0.307500027492642491466767751262523233890533447265625f,0.4488999802678821371415551766403950750827789306640625f,1.5004999883174898744897518554353155195713043212890625f,0.447399985140561951002524665454984642565250396728515625f,-0.404600023263693486796910292468965053558349609375f,0.740399995362758023276228414033539593219757080078125f,1.0572000399112706503501613042317330837249755859375f,0.03959998556375499134674100787378847599029541015625f,1.31440002673715383707531145773828029632568359375f,1.32789999789595558610244552255608141422271728515625f,0.720800006818771432648418340249918401241302490234375f,-0.1337000120162965544068356393836438655853271484375f,1.0114000124916435741084796973154880106449127197265625f,1.02560001837611114439141601906158030033111572265625f,0.98139999516904297127695144808967597782611846923828125f,-1.18889997050762108443677789182402193546295166015625f,0.84220001396462340625959086537477560341358184814453125f,0.5703999628782270292504108510911464691162109375f,0.86099999123811710166620514428359456360340118408203125f,0.08560008528232554425585476565174758434295654296875f,1.468900025437027689889646353549323976039886474609375f,-1.31160003181695916651960942544974386692047119140625f,0.370600009156018372902252622225205413997173309326171875f,0.490599975609779193064241553656756877899169921875f,0.93010000129938230628567907842807471752166748046875f,-1.026500063747167956762496032752096652984619140625f,1.08939999332130010856189983314834535121917724609375f,-0.80640005842447326500632698298431932926177978515625f,1.249800011855364711976790204062126576900482177734375f,-0.3139000266849993892037673504091799259185791015625f,-0.12480000343620811786848889823886565864086151123046875f,-1.951600012445450005316160968504846096038818359375f,1.275800061506032090363760289619676768779754638671875f,1.5564999911785122321816743351519107818603515625f,0.045600002431869679497822289704345166683197021484375f,-0.1549000094294541440831380896270275115966796875f,1.085700029584766301837817081832326948642730712890625f,0.35739996651410965711193057359196245670318603515625f,-0.17919999221265303646077882149256765842437744140625f,1.069900041019916869799999403767287731170654296875f,-0.82480001513361944631697042495943605899810791015625f,0.078799997222424256193562541739083826541900634765625f,0.14679996728599087330024985931231640279293060302734375f,0.1876000269293793820679638884030282497406005859375f,-0.564200035500526109188967893715016543865203857421875f,-0.713700006651877938423922387300990521907806396484375f,-0.13950002516806103702862174031906761229038238525390625f,1.4423999924421320173451022128574550151824951171875f,0.069799971622229062262476873002015054225921630859375f,-2.27029993952512842980695495498366653919219970703125f,-0.628199955749512373159859635052271187305450439453125f,0.6506999697566033802331730839796364307403564453125f,0.440699996355176093487671096227131783962249755859375f,-0.70339999917745554824932696647010743618011474609375f,0.80469996384084208074227717588655650615692138671875f,-0.7681000107765196815989838796667754650115966796875f,0.46320003472566639857177506200969219207763671875f,0.6493000297591093872284773169667460024356842041015625f,-1.593400027154386666694563245982863008975982666015625f,-0.25270001792833252096670548780821263790130615234375f,-0.408999993860721389182799612171947956085205078125f,-1.076900002634525232991791199310682713985443115234375f,0.30729995731711401418806417495943605899810791015625f,-0.55790001216381757132012353395111858844757080078125f,-0.401299947452546490467284456826746463775634765625f,0.83619996671676577459919599277782253921031951904296875f,-0.74710000076293869852861462277360260486602783203125f,-1.014299980454891336734135620645247399806976318359375f,-0.316200012835860189142067611101083457469940185546875f,-0.48929998384490558382964309203089214861392974853515625f,-0.578800024640560906163955223746597766876220703125f,0.04349998646974551075317094728234224021434783935546875f,0.1605000088810921710091861314140260219573974609375f,0.89580001153126354029865296979551203548908233642578125f,-0.27910001423358921091022466498543508350849151611328125f,-1.53120001510828718238599321921356022357940673828125f,-0.091599953474104101047714721062220633029937744140625f,-1.203099983432889263923470934969373047351837158203125f,0.4910999959647650570104815415106713771820068359375f,-0.06690001507699439020626641649869270622730255126953125f,2.298799990665912673648563213646411895751953125f,0.06159994017481817962789136799983680248260498046875f,-0.0999999623000622506197032635100185871124267578125f,-1.33739996733367405568060348741710186004638671875f,0.88970000213682620682220658636651933193206787109375f,-1.020300010231136145222308186930604279041290283203125f,0.69799998578429178763826712383888661861419677734375f,0.973800005790592226873059189529158174991607666015625f,2.3579000442385673608214347041212022304534912109375f,0.450499986156821297100805168156512081623077392578125f,-0.4967000033140180903501459397375583648681640625f,-0.034400002038478927346432101330719888210296630859375f,0.4532999807044870710370787492138333618640899658203125f,0.5762000085145240202422201036824844777584075927734375f,0.185300028932094296152399692800827324390411376953125f,-0.36969998351037547390518511747359298169612884521484375f,-0.6187999966263770801333521376363933086395263671875f,0.3612000114202498224358350853435695171356201171875f,0.165399953043462222268544792314060032367706298828125f,1.784599948161841087568291186471469700336456298828125f,0.1431999897241584829288285618531517684459686279296875f,0.222500022500752681064994931148248724639415740966796875f,0.2799999876320367064863603445701301097869873046875f,0.65649998946487908568769853445701301097869873046875f,-2.001999995589255121331007103435695171356201171875f,-0.074800004479288251246771324076689779758453369140625f,0.240799975228309648400681908242404460906982421875f,0.697499994039535664569484652020037174224853515625f,0.2883999953031537444303467054851353168487548828125f,0.47280004130005881535225853440351784229278564453125f,0.44510001498982310064178591346717439591884613037109375f,0.53840000603199111850472036167047917842864990234375f,-0.6550999624446027791435653853113763034343719482421875f,0.5226999879956242267553534475155174732208251953125f,-0.01859999724999095160882234267774038016796112060546875f,-0.047100029969215029268525540828704833984375f,-0.19250003315508390500099267228506505489349365234375f,0.35240000466108245547047772561199963092803955078125f,-0.4712000413902106554786541892099194228649139404296875f,0.6107000046253201475110472529195249080657958984375f,-0.3952999994836743358206376797170378267765045166015625f,-1.7158000041365628618450500653125345706939697265625f,0.26489998905211653568159135829773731529712677001953125f,-0.1860000183582304345009106327779591083526611328125f,-0.67050000381469720966975955889211036264896392822265625f,0.484200015085935575598341529257595539093017578125f,0.60310001092553078283486911459476687014102935791015625f,-0.78700001198053382012176371063105762004852294921875f,-1.4546000063508746080742639605887234210968017578125f,-1.3871000311613084221562530728988349437713623046875f,1.506500021204352446346774740959517657756805419921875f,1.36990003476142874205834232270717620849609375f,-0.874700086402893983716921866289339959621429443359375f,0.310199990379810941476534935645759105682373046875f,-0.177800066334009354562795124365948140621185302734375f,-0.7553000009179111629009639727883040904998779296875f,-0.40189997819661993361251006717793643474578857421875f,0.1712999802350996247923831106163561344146728515625f,-0.281800011169910380459668886032886803150177001953125f,0.5069000363111495488510627183131873607635498046875f,-0.600199979278445372443684391328133642673492431640625f,0.1532000028371811595206963829696178436279296875f,-0.49019999932050695168328502404619939625263214111328125f,0.28960000016689235735611873678863048553466796875f,0.0081000038474794422427294193767011165618896484375f,1.9855999681591978145434040925465524196624755859375f,0.528200005891918511480298548121936619281768798828125f,-1.7061999845981592471844123792834579944610595703125f,-0.848499992728232843575142396730370819568634033203125f,1.0096000007629388761642985627986490726470947265625f,1.184000025302171632546333057689480483531951904296875f,0.54929997212886849666801936109550297260284423828125f,0.5257999528765679997377446852624416351318359375f,0.3298999998927119037261945777572691440582275390625f,-0.273000024080276570970227112411521375179290771484375f,-0.8249999982118598751412719138897955417633056640625f,0.438600002595782800796087030903436243534088134765625f,0.073900022959708877579032559879124164581298828125f,-0.7232999805927278202943853102624416351318359375f,-0.9985999797224991425537155009806156158447265625f,0.687999962240457563211748492904007434844970703125f,-1.548800014451891637889957564766518771648406982421875f,1.179499997749924755652273233863525092601776123046875f,1.35650003033131394403199010412208735942840576171875f,-0.98269998248219536662872997112572193145751953125f,0.665599979893862769841916815494187176227569580078125f,-0.38680002882778674599961732383235357701778411865234375f,-1.528700037311017556618253365741111338138580322265625f,0.739200009088218035202544342610053718090057373046875f,-0.270800006092339839991467442814609967172145843505859375f,-0.362199996027350312743919857894070446491241455078125f,-0.7262000059068203228207494248636066913604736328125f,1.555000006407500023897227947600185871124267578125f,0.3757000424928962356574402292608283460140228271484375f,-1.139100002163648372999205093947239220142364501953125f,-0.28550000359117977399847632113960571587085723876953125f,-0.2288000031828882896434151916764676570892333984375f,-0.51910006050169510416480989078991115093231201171875f,1.14670002335607978949383323197253048419952392578125f,2.443000051423907859060591363231651484966278076171875f,0.16140001029372275098694444750435650348663330078125f,-0.54740004832148603242103490629233419895172119140625f,-1.129100041651726282765366704552434384822845458984375f,-1.2217000201150778337932933936826884746551513671875f,-0.2302000043123959560631419662968255579471588134765625f,0.3683000008955594761772545098210684955120086669921875f,-0.916500018522143466981333403964526951313018798828125f,-0.341299991783499567787885098368860781192779541015625f,0.0252000242054464251850731670856475830078125f,0.073100018078088080386578440084122121334075927734375f,-2.090000003278255302774368828977458178997039794921875f,-0.4288999765425922650052825702005065977573394775390625f,1.1690000121891497286696903756819665431976318359375f,0.51499998494982701657818324747495353221893310546875f,0.0746000296711926136339343429426662623882293701171875f,-0.250299973946809306557526042524841614067554473876953125f,-1.24630007128119491000006746617145836353302001953125f,0.008099973225593526393595311674289405345916748046875f,0.64170003460645741455437018885277211666107177734375f,0.583499983116984477504729511565528810024261474609375f,0.50580003229975700751452905024052597582340240478515625f,-0.83399999997019780284546186521765775978565216064453125f,0.1676000224962830298380822569015435874462127685546875f,-0.881300013948977056799094498273916542530059814453125f,0.036700011882185634704001131467521190643310546875f,-0.5174999871104954962675037677399814128875732421875f,1.447999960899352345222723670303821563720703125f,-0.1428999903708698671067622854025103151798248291015625f,-0.76579997805953059497596768778748810291290283203125f,-0.08159997310638456013975883251987397670745849609375f,-1.02789998970031692948623458505608141422271728515625f,-0.318100058758258530389184670639224350452423095703125f,-1.9182000141620638711259516639984212815761566162109375f,0.79120000378340460400750089320354163646697998046875f,0.2460999953128395478785250816144980490207672119140625f,0.51479999265149223308668524623499251902103424072265625f,0.11499999485909950802664525326690636575222015380859375f,0.0885000241324302994172512626391835510730743408203125f,0.783599996970593792866566218435764312744140625f,0.10460001205056912443325245476444251835346221923828125f,0.87190000414326807032239230466075241565704345703125f,-0.81299999543279444225163388182409107685089111328125f,0.425599992895126177927522803656756877899169921875f,-0.44320004381537447812888785847462713718414306640625f,-0.63449999301880577462497967644594609737396240234375f,0.36440000436678510187249457885627634823322296142578125f,0.13600000174343584991021316454862244427204132080078125f,-0.1911999869823457487427731393836438655853271484375f,-1.127300053131580881426998530514538288116455078125f,1.030600041472911954087976482696831226348876953125f,1.297000008124857828306630835868418216705322265625f,-0.65029999611228672673490791567019186913967132568359375f,-0.3761999863676723787619948780047707259654998779296875f,-0.087300020162761171604870469309389591217041015625f,0.9369999913051720863421678586746565997600555419921875f,0.56609998255371996034313042400754056870937347412109375f,1.06660001255720882085142875439487397670745849609375f,0.430199969331920062387553116423077881336212158203125f,-0.7889000314161178639693616787553764879703521728515625f,-1.350800005030631911751015650224871933460235595703125f,0.18150006179139099860009309850283898413181304931640625f,0.3061000058554113056885626065195538103580474853515625f,-0.248400009328871906877367337074247188866138458251953125f,-0.8535000438764697161531103120069019496440887451171875f,0.69879999766945832817555128713138401508331298828125f,0.027700002896785758110809183563105762004852294921875f,-0.09369996582269646268059659632854163646697998046875f,-0.5878000073254108226450398433371447026729583740234375f,-0.72780001991689202700541727608651854097843170166015625f,0.71349999703094368896927335299551486968994140625f,0.02669999342337237013822459630318917334079742431640625f,-0.73289999528825244823337925481610000133514404296875f,0.5104999851882456862739445568877272307872772216796875f,-0.7864999862238766237254594670957885682582855224609375f,0.02279999242424946004348385031335055828094482421875f,0.8614999912530187931025693615083582699298858642578125f,0.56769998255670106601655788836069405078887939453125f,-0.9336999879881735520115171311772428452968597412109375f,-1.5063000031456350580327807620051316916942596435546875f,0.740700033496320742187890573404729366302490234375f,-0.28460001186430450292874638762441463768482208251953125f,0.5263000285148624612929779686965048313140869140625f,-0.98000003978610106258884115959517657756805419921875f,-0.93420003389865191589791493242955766618251800537109375f,0.7258999726533890139990035095252096652984619140625f,-0.0974000193014743498309826463810168206691741943359375f,-0.054599953675270374020556118921376764774322509765625f,-0.059800020628422156931947029079310595989227294921875f,-0.10770003210306189345146776759065687656402587890625f,-0.80430001247376237127895137746236287057399749755859375f,-0.173699999052286724321447763941250741481781005859375f,-0.04540000435933466693683158155181445181369781494140625f,-0.2013000072807076268333048574277199804782867431640625f,0.33479998819977030422023744904436171054840087890625f,0.4825000266730785636326572785037569701671600341796875f,0.7137999918580055780381599106476642191410064697265625f,-0.833199998962879107011758605949580669403076171875f,0.88150000052899113445192824656260199844837188720703125f,-0.841100024873018270454849698580801486968994140625f,-1.51320002243220752546903895563445985317230224609375f,0.9237000445008280991032734164036810398101806640625f,0.896599978843330536193434454617090523242950439453125f,1.484100003802776512884520343504846096038818359375f,-0.567800051209329748047593966475687921047210693359375f,0.24509998520612708006183311226777732372283935546875f,-0.6983000545024875549415810382924973964691162109375f,-0.519599984669685266425176450866274535655975341796875f,0.50460005023479492791693701292388141155242919921875f,-0.367300014537573105144474538974463939666748046875f,-0.239599990481138558351403844426386058330535888671875f,-0.1235000325143340571543149053468368947505950927734375f,0.5284999951869246981317473910166881978511810302734375f,0.96029999771416196807649612310342490673065185546875f,0.4965999956816435911832741112448275089263916015625f,0.78860004439354014493801514618098735809326171875f,0.00139999472200857955073161065229214727878570556640625f,0.2235000150799759666142563219182193279266357421875f,-9.9985280632264306177603430114686489105224609375e-05f,-0.8882999945998193425111821852624416351318359375f,0.031700014117360009180401902995072305202484130859375f,0.755299972009658926452857485855929553508758544921875f,1.528999960780143663185981495189480483531951904296875f,0.53319997437596367007017761352472007274627685546875f,0.362599989676475775723929473315365612506866455078125f,-0.7720999812424178543324160273186862468719482421875f,-1.212200027096271615079103867174126207828521728515625f,0.1055000501573089000117988689453341066837310791015625f,-0.4918999930232763606596790850744582712650299072265625f,-0.338799980682134904697022648178972303867340087890625f,0.41880001316666604083138736314140260219573974609375f,-0.767900046026706917245974182151257991790771484375f,1.0819000091910364602654226473532617092132568359375f,-0.64270002048015584250606480054557323455810546875f,0.5483000309586518739024540991522371768951416015625f,1.313599982888996375862689092173241078853607177734375f,0.0148000057458883560457252315245568752288818359375f,0.1252999949574473692592846418847329914569854736328125f,0.139500012576579734968618140555918216705322265625f,-0.1040999970600011703680820573936216533184051513671875f,0.038200000900030506301163768512196838855743408203125f,0.05739999475181090105024850345216691493988037109375f,1.217899996405839235791290775523521006107330322265625f,-0.378799978446959784150749328546226024627685546875f,1.41820002667903821702566347084939479827880859375f,-0.700000014603137987734271519002504646778106689453125f,-0.5681000328302392432533451938070356845855712890625f,0.286000010460615516905136246350593864917755126953125f,-0.688800029855966666758604333153925836086273193359375f,0.13519999663829818103977231658063828945159912109375f,0.3193999933958053816240862943232059478759765625f,-0.600000004470349068697032635100185871124267578125f,-0.073699992942810155227562063373625278472900390625f,1.7742999710857869644087259075604379177093505859375f,-0.51180001891851389927978743799030780792236328125f,-0.43129996026754380267220767564140260219573974609375f,0.001600002312660109282660414464771747589111328125f,-0.238500024840235358425388767500407993793487548828125f,0.248299947363138873157595298835076391696929931640625f,1.269499967649579286188554760883562266826629638671875f,0.240600037226080143426543145324103534221649169921875f,0.1124999953806398167444058344699442386627197265625f,-1.91429990954399187330636777915060520172119140625f,-0.8471000147700313842591413049376569688320159912109375f,-0.3487999328494080231166662997566163539886474609375f,0.4007999738872047856830249656923115253448486328125f,0.7615999877095216419320422573946416378021240234375f,0.110099996978044512019323519780300557613372802734375f,-1.86940003601312643155551995732821524143218994140625f,0.860199988815188110180542935268022119998931884765625f,1.279400046518445410725917099625803530216217041015625f,-0.9287000241607430428558700441499240696430206298828125f,-0.876900014778971836904020165093243122100830078125f,-0.355000001043081159224357179482467472553253173828125f,-0.325600015731156100518006724087172187864780426025390625f,-0.03510001091063041922524234905722551047801971435546875f,1.164500023081898927301836010883562266826629638671875f,-0.6248000198274852312607663407106883823871612548828125f,0.447400014235079357405311384354718029499053955078125f,-0.413200022953748913323579472489655017852783203125f,-0.3363999930083749756448696643928997218608856201171875f,0.59009999176263783216001002074335701763629913330078125f,0.00939997722804530866369532304815948009490966796875f,-0.251799985539913340204520864062942564487457275390625f,-0.9443999939918512609438039362430572509765625f,1.06720003678202601804514415562152862548828125f,0.33320002820640814977792842910275794565677642822265625f,-0.4322000346213588084509638065355829894542694091796875f,0.4787000092968345388300122067448683083057403564453125f,-0.12179999090433124564469835604541003704071044921875f,0.350299977523088390540806358330883085727691650390625f,0.1280000100731850665169986314140260219573974609375f,1.422300037783384762946070623002015054225921630859375f,-0.977500045597553768317311551072634756565093994140625f,0.5464000325709583716360384642030112445354461669921875f,-1.1830000649839644832894691717228852212429046630859375f,-0.19499998930841666577862270059995353221893310546875f,0.1326999619185915690877664019353687763214111328125f,0.01270003024041699291046825237572193145751953125f,0.002999995023011958750203120871447026729583740234375f,-0.9760000157505273055136285620392300188541412353515625f,0.4217999941825869569811402470804750919342041015625f,-0.64919997917413663213181962419184856116771697998046875f,-1.44169995920658156052240883582271635532379150390625f,0.587400030812621043452281810459680855274200439453125f,-0.03319996871352248746234181453473865985870361328125f,0.5304999915957449019288105773739516735076904296875f,-0.979700008022785340244809049181640148162841796875f,0.7802000134766104988415236221044324338436126708984375f,-0.4027999937653543582882775808684527873992919921875f,-0.80819999166130995771339939892641268670558929443359375f,0.54060000541210140312387011363171041011810302734375f,-0.615400033435225424227610346861183643341064453125f,1.55239996100068022855111848912201821804046630859375f,-1.077000043272972273911136653623543679714202880859375f,1.6757000289142123250485383323393762111663818359375f,-0.24260002447068718112177521106787025928497314453125f,-0.4411999697715049961743716266937553882598876953125f,-0.3448000008285045936418100609444081783294677734375f,-0.00589999187588663431824897998012602329254150390625f,-0.4004000369369988820977823706925846636295318603515625f,-0.70780002960264687406066741459653712809085845947265625f,0.3593000071465966005490599854965694248676300048828125f,0.3341000079751015316986695324885658919811248779296875f,-0.4643000169813633615234493845491670072078704833984375f,-2.071299991373717119813591125421226024627685546875f,0.617099985787272320436613881611265242099761962890625f,0.9819999765157694238126850905246101319789886474609375f,-1.099399987733364003616998161305673420429229736328125f,-0.32919996416121666271692447480745613574981689453125f,-0.30650004042685008442958860541693866252899169921875f,-0.51930006306320508624452259027748368680477142333984375f,-0.7704999722987413424135638706502504646778106689453125f,0.90600003981590315760286102886311709880828857421875f,-0.97950004148483316157580702565610408782958984375f,0.7570000379085541908352752216160297393798828125f,-1.3808000595688820766326898592524230480194091796875f,1.0744000438362359961530501095694489777088165283203125f,-0.60119999800920442822871336829848587512969970703125f,1.04830006068646941486122159403748810291290283203125f,-1.22940003392696350914548020227812230587005615234375f,0.4564000153973697404552467560279183089733123779296875f,0.47040001503229067481015590601600706577301025390625f,1.700800023657083936967637782800011336803436279296875f,-1.120299993765353718799815396778285503387451171875f,0.795000000223517577779830389772541821002960205078125f,-0.640800033044815364746682462282478809356689453125f,1.687299994905291811164715909399092197418212890625f,-0.3298999826073636398859889595769345760345458984375f,0.7858000494733463625607328140176832675933837890625f,-0.56010003974437783114126432337798178195953369140625f,0.39170000689029682572339652324444614350795745849609375f,0.095999971866607580750496708787977695465087890625f,-0.17279998184442479924882718478329479694366455078125f,2.0649999186396588157776932348497211933135986328125f,-1.562399984246491957406988149159587919712066650390625f,0.9252999802052972455612689373083412647247314453125f,-0.3660999883651729902567240060307085514068603515625f,1.0132999296307563241725802072323858737945556640625f,-0.23989998797178291312093278975225985050201416015625f,0.3021000196874137344593691523186862468719482421875f,-0.00859998687505747483328377711586654186248779296875f,-0.8315999951601025230729646864347159862518310546875f,-0.3643999556958679164608838618732988834381103515625f,1.6726999379277227575357755995355546474456787109375f,-0.092900027102231330644599438528530299663543701171875f,-0.5345999953985209884876894648186862468719482421875f,1.0756000375986101147418594337068498134613037109375f,0.1490999906897538807015735073946416378021240234375f,-0.0205000005736947488177435161560424603521823883056640625f,0.12040005068332015891741093582822941243648529052734375f,-0.320000027157366417895900667645037174224853515625f,-0.13780003034770516023854725062847137451171875f,0.3915000329017639391082639122032560408115386962890625f,1.1312999848544598169297614731476642191410064697265625f,-0.550999984383583107927506716805510222911834716796875f,-0.27509998181611283740721773938275873661041259765625f,-0.6651999876230954011901985722943209111690521240234375f,0.3435999801322815461190884889219887554645538330078125f,-0.00639998891055582053155603716732002794742584228515625f,-0.2482000432938338274624356927233748137950897216796875f,-0.06179997883439103389946467359550297260284423828125f,1.4247000072404742976317493230453692376613616943359375f,-0.974599985340237839892552074161358177661895751953125f,-0.2229999868273733198975605773739516735076904296875f,-0.342200027394294892246762174181640148162841796875f,0.697900025798380507779938852763734757900238037109375f,0.54809998764246625757579067794722504913806915283203125f,0.008099976131319852612477916409261524677276611328125f,1.1430000440478327305982020334340631961822509765625f,-0.330300006915628774351034735445864498615264892578125f,-0.16760000189542711979129308019764721393585205078125f,-0.1206999901711947043025929815485142171382904052734375f,0.2628000009179114471180582768283784389495849609375f,-0.7067000165760510821399975611711852252483367919921875f,0.93430001131892215227026099455542862415313720703125f,-0.6015000167712571954581335376133210957050323486328125f,1.095100035721064291038828741875477135181427001953125f,-0.36019997506886713978246916667558252811431884765625f,-0.6820000110864636866381260915659368038177490234375f,-0.36759996762275637394168370519764721393585205078125f,-0.5043000272631648073229371220804750919342041015625f,0.4033000073030590382217042133561335504055023193359375f,-0.471800008040666352115977133507840335369110107421875f,2.154700016926228922642394536524079740047454833984375f,-0.358899994871020755482504682731814682483673095703125f,0.58779999376535396748977291281335055828094482421875f,-1.51919993685483945000669336877763271331787109375f,-0.69649995884299276127649136469699442386627197265625f,0.078700007098913005165741196833550930023193359375f,1.13189997000098241386467634583823382854461669921875f,-0.297799988400935689014659146778285503387451171875f,1.017899989998340526398123984108678996562957763671875f,-1.0850999649405483893360724323429167270660400390625f,0.8268000304669147482172775198705494403839111328125f,-1.3332999647229915041179992840625345706939697265625f,-0.42479998026490140006217188783921301364898681640625f,-0.856499976873397628196471487171947956085205078125f,0.074300007295608594404257019050419330596923828125f,0.952399967408180003758388920687139034271240234375f,-0.2066000020891427535474349497235380113124847412109375f,0.07260001254975778639533245950588025152683258056640625f,0.0557999782085418782884289612411521375179290771484375f,-0.9165000219866634711962660730932839214801788330078125f,0.835199981811642455653554861783050000667572021484375f,0.473799991410970733340946026146411895751953125f,-0.13699997800588636920338103664107620716094970703125f,0.872000036455691240888654647278599441051483154296875f,-0.42090003263056274818154633976519107818603515625f,-0.482900007469952097238774513243697583675384521484375f,0.53160004470646438079484141780994832515716552734375f,-0.7069000483065848161601252286345697939395904541015625f,0.455300036233663885099076651385985314846038818359375f,0.5162000245332720727020614503999240696430206298828125f,-0.757900004750490463578671551658771932125091552734375f,0.0888999991178511805856032879091799259185791015625f,0.328100003623962521714929607696831226348876953125f,0.08030000509023693666676990687847137451171875f,-0.77970004646778168222454041824676096439361572265625f,0.140800007303059526719835048425011336803436279296875f,1.04859999939203252239394714706577360630035400390625f,-0.3275999980956318946567762395716272294521331787109375f,-0.3272999875664712732259431504644453525543212890625f,-0.3578999688014381508338601634022779762744903564453125f,-0.502200038719177399570980924181640148162841796875f,-0.077900028517843278308419030508957803249359130859375f,0.9233000518202789663746443693526089191436767578125f,-1.686500066876412073924029755289666354656219482421875f,0.8413000514626514103611043537966907024383544921875f,0.30280001678764822958811464559403248131275177001953125f,0.0045999977827069216118616168387234210968017578125f,-0.325600009733438167103258820134215056896209716796875f,0.2892000128507614675754666677676141262054443359375f,0.205699997211992613443953814567066729068756103515625f,0.341400005972385400809798738919198513031005859375f,-0.530200052517652675732051648083142936229705810546875f,1.530299978715181197230776888318359851837158203125f,0.56699998351931579776419312111102044582366943359375f,0.550699993300437284915460622869431972503662109375f,-1.55489998528957329426702926866710186004638671875f,-1.326399938321113580741439363919198513031005859375f,0.5346000253498548460129313752986490726470947265625f,2.3417999423265456471199286170303821563720703125f,0.221899989968537880713483900763094425201416015625f,-0.8528000053882589526210722397081553936004638671875f,-0.43089999828338587661846759147010743618011474609375f,-1.0247999863743775250668477383442223072052001953125f,1.67170001583099381292640828178264200687408447265625f,-0.9683999851703646299938554875552654266357421875f,0.03520000379085530539668980054557323455810546875f,-0.323599989557266098927357234060764312744140625f,0.237599995562434163076659388025291264057159423828125f,0.6785999953687193997353688246221281588077545166015625f,0.49310001703500672132207682807347737252712249755859375f,-0.3306999979197977612699332894408144056797027587890625f,-0.32380000541806186475923823309130966663360595703125f,-0.1636999875783917701710379333235323429107666015625f,-0.31910001006126431644105423401924781501293182373046875f,0.38420000860393044295193476500571705400943756103515625f,0.749600025610626108374390241806395351886749267578125f,-0.69480002295672915746393982772133313119411468505859375f,0.8125000279769303279664427464012987911701202392578125f,0.93410002403110237167993545881472527980804443359375f,0.14109998538494128439424457610584795475006103515625f,-2.096200005310773661904022446833550930023193359375f,1.11299997460842092777966172434389591217041015625f,0.860900003796815749268489526002667844295501708984375f,-0.55399999698996538199935457669198513031005859375f,-0.86249998986721010396649944595992565155029296875f,0.65910000901818222018846427090466022491455078125f,-1.346199966157972216507232587900944054126739501953125f,-0.843800012570619539786775931133888661861419677734375f,0.0849999814108006734869604770210571587085723876953125f,-0.340600014054775357408288982696831226348876953125f,0.53679999995678695512424383196048438549041748046875f,0.6562000237137082248040087506524287164211273193359375f,-0.89380003849864042564377086819149553775787353515625f,0.3567000382572416139481674690614454448223114013671875f,-0.238400004728138326104414090877980925142765045166015625f,-0.449400036460161356188791614840738475322723388671875f,-0.4720999934241170148396804506774060428142547607421875f,0.893700016784666928515434847213327884674072265625f,0.5206000017985699113154396400204859673976898193359375f,-0.115800003689527475359000163734890520572662353515625f,0.042299989503622015263317734934389591217041015625f,0.1429999829530714805514435283839702606201171875f,-0.815099985206126920189717566245235502719879150390625f,-0.4852999107658870769910208764486014842987060546875f,-1.8053999844104051675230948603712022304534912109375f,-0.29130003350675082174348062835633754730224609375f,1.4719000026345250287107546682818792760372161865234375f,1.0718999692559236791566945612430572509765625f,0.4183999839410181831311774658388458192348480224609375f,-0.2618999963015313792169536100118421018123626708984375f,-0.8375000116601587496489855766412802040576934814453125f,-0.5694999851956958902832184321596287190914154052734375f,0.065799946692586264163082887534983456134796142578125f,1.009699973824620844453647805494256317615509033203125f,-0.7887999861955645730660080516827292740345001220703125f,-0.2762999647378929779506506747566163539886474609375f,0.8007000020921228866654928424395620822906494140625f,-0.5022999884605408027482553734444081783294677734375f,0.26009999504089353905555981327779591083526611328125f,0.99620002088248782445845108668436296284198760986328125f,0.4119000079989427742788166142418049275875091552734375f,-0.62160001075044279961190341055043973028659820556640625f,0.50290002678558265447605890585691668093204498291015625f,-0.21889999442398544449162045566481538116931915283203125f,0.0689999872855844298413785509183071553707122802734375f,0.116999990262091235759811524985707364976406097412109375f,-0.58039998474568144093410637651686556637287139892578125f,0.238599980877339579166118710418231785297393798828125f,-0.5451999940678475820021731124143116176128387451171875f,-0.665199992018937980020609757048077881336212158203125f,0.0017999859496951042814316679141484200954437255859375f,0.33570000401064759021352301715523935854434967041015625f,0.33020001198649417961661356457625515758991241455078125f,-0.4886000115923583297927734747645445168018341064453125f,0.34049999658763374643655197360203601419925689697265625f,-0.75659998681545204135545645840466022491455078125f,0.95820000432729646178131588385440409183502197265625f,-2.10700002058595448062305877101607620716094970703125f,0.06820003789216311229637312862905673682689666748046875f,-0.526200028370320982418206767761148512363433837890625f,-0.19340001687258479012854195389081723988056182861328125f,1.118500012174248325891312560997903347015380859375f,-0.433600004085898493855211199843324720859527587890625f,0.062600002938509202721206747810356318950653076171875f,-0.86910000294595890490967349251150153577327728271484375f,0.65790000966787320901829616559552960097789764404296875f,0.31840001176893739653905868181027472019195556640625f,0.23149998976290220298324129544198513031005859375f,0.01160000827312479554365154399420134723186492919921875f,-0.0647000012427569881623412584303878247737884521484375f,-0.10830000640898951314028408887679688632488250732421875f,0.8293000396125023154780819822917692363262176513671875f,0.1342000003337862068519825697876513004302978515625f,0.349500018388032795968456412083469331264495849609375f,-0.96869992737770171942202068748883903026580810546875f,-1.15729999842569242218814906664192676544189453125f,-1.0255999941244713369314922601915895938873291015625f,-0.34349999952688825555213725237990729510784149169921875f,1.126999972138553740563793326145969331264495849609375f,-0.291399994423985730929160808955202810466289520263671875f,-0.1721999786674981436362941167317330837249755859375f,-0.565499987386167024538963232771493494510650634765625f,0.42969996331930193633752423920668661594390869140625f,1.5383000097990038312900651362724602222442626953125f,0.193399958199262922420302857062779366970062255859375f,-0.32729999155253153286793121878872625529766082763671875f,-1.0968999941036106093861235422082245349884033203125f,-0.1788000011898576158131390911876223981380462646484375f,0.869699998206644675491361340391449630260467529296875f,0.1167000012278556209111002317513339221477508544921875f,0.665900029203295940050111312302760779857635498046875f,-0.15720000742673878590949243516661226749420166015625f,0.681600004361570199762354604899883270263671875f,-0.30479998406469821503605999168939888477325439453125f,0.5825999867707485346812745774514041841030120849609375f,-0.432299987268447427624096235376782715320587158203125f,-0.3295999869421117889345396179123781621456146240234375f,0.52510001329481570397916811998584307730197906494140625f,-0.56620002326667329128895289613865315914154052734375f,0.429000041246414554052535095252096652984619140625f,-0.25930000409185882670470846278476528823375701904296875f,-0.066900001591444102633232660082285292446613311767578125f,0.021899976259469600137208544765599071979522705078125f,-0.567299995166063109053311563911847770214080810546875f,0.66050003302097337609666283242404460906982421875f,-1.083700028407573512367889634333550930023193359375f,0.4698000066399572460795752704143524169921875f,0.43020000051259987827734221355058252811431884765625f,0.1025000109896065936254672124050557613372802734375f,-0.364999998211860798846828402020037174224853515625f,-0.356600019188225925148572059697471559047698974609375f,-0.09650001800060170609185661305673420429229736328125f,-0.8131000059336421781353010374004952609539031982421875f,1.13880002553462933434502701857127249240875244140625f,-1.631700041610002926972811110317707061767578125f,0.363100049370527866443580933264456689357757568359375f,0.068799982395768122245272024883888661861419677734375f,-0.274999993443489021416326067992486059665679931640625f,1.4936000220045446695849022944457828998565673828125f,-0.4613999874204390838627887205802835524082183837890625f,1.0813000266522170278449266334064304828643798828125f,-0.6570000289678574745266814716160297393798828125f,-0.08130002713650508194831445507588796317577362060546875f,-0.821600017511844527007269789464771747589111328125f,1.09630001584887537546819658018648624420166015625f,-0.153599987918138225495567894540727138519287109375f,-1.28219996495842991635072394274175167083740234375f,-0.141999972343445080014134873636066913604736328125f,0.6827999426543716054993637953884899616241455078125f,1.32829994572401144381501580937765538692474365234375f,0.6784000218272205895431170574738644063472747802734375f,-1.16799995538592416011169916600920259952545166015625f,-0.2824000006377698213100302382372319698333740234375f,-1.757199982392787518392651691101491451263427734375f,0.07650002612173534277673070391756482422351837158203125f,0.396499996989965186600102242664434015750885009765625f,-0.649199985879659546839093309245072305202484130859375f,0.378299993631244557690251895110122859477996826171875f,0.7720000143647194335017047706060111522674560546875f,0.2445999581456188565198317519389092922210693359375f,-0.1062000011384487407894994248636066913604736328125f,-0.13200004293769629359900363851920701563358306884765625f,2.006600029470026580469266264117322862148284912109375f,0.93319997767284512679708541327272541821002960205078125f,0.0224000156693164864663003754685632884502410888671875f,2.366900028357654672817034224863164126873016357421875f,0.4331999694958328372962341745733283460140228271484375f,0.087499919310210838574448644067160785198211669921875f,-1.79320005826950090721538799698464572429656982421875f,-1.036900002485513727634725000825710594654083251953125f,-1.09780000419616641238462761975824832916259765625f,0.632099994578956714263995309011079370975494384765625f,-0.810800012779235856896775658242404460906982421875f,-0.661500000566243695487855802639387547969818115234375f,-0.6151000302076337078460710472427308559417724609375f,0.6238999796718360801861535946954973042011260986328125f,0.506500012636184493430846487171947956085205078125f,0.45779999942779536770132153833401389420032501220703125f,-0.57559998306035986548323535316740162670612335205078125f,0.2645000210702421750141866141348145902156829833984375f,-0.046700056883692997455881368296104483306407928466796875f,0.9533999808117747942759478974039666354656219482421875f,-1.5267000017866489969264875981025397777557373046875f,-0.55020001988410960347408718007500283420085906982421875f,0.776200058805943005069138962426222860813140869140625f,0.30779997543692605432852360536344349384307861328125f,0.785400014585256212029662492568604648113250732421875f,-0.97670000994503514135658406303264200687408447265625f,-0.20039994961619367330740715260617434978485107421875f,-1.532199999678134805236595639144070446491241455078125f,1.5621999841809266484915497130714356899261474609375f,0.1719999953657385116656541867996565997600555419921875f,-0.5926000239491457932672346942126750946044921875f,0.700099984014034504298251704312860965728759765625f,1.25209999390840476252151347580365836620330810546875f,-1.5453000132858758508547225574147887527942657470703125f,-0.21570006894245796758013966609723865985870361328125f,1.390500002697109938054609301616437733173370361328125f,-1.22200003872811802096975952736102044582366943359375f,1.48990000466108352128458136576227843761444091796875f,0.2678000179797412894089347901172004640102386474609375f,-0.4235999608725311560419868328608572483062744140625f,0.7765000070668757192748898887657560408115386962890625f,-0.9803000201404097335711185223772190511226654052734375f,-0.924700032628327672767909461981616914272308349609375f,-0.8797999988541007621023481988231651484966278076171875f,0.01480000578314066839169527156627736985683441162109375f,0.312500003874302212381053323042578995227813720703125f,0.30520002357214714994171345097129233181476593017578125f,-0.594700007128715402160423764144070446491241455078125f,0.81870000730380410036701732678920961916446685791015625f,-0.5742000095725057207118879887275397777557373046875f,-0.8688000455766928808998272870667278766632080078125f,-0.153200010436772782185244068386964499950408935546875f,-0.34790002650618578883268128265626728534698486328125f,-0.143300001603364535185392014682292938232421875f,0.5520999908536674549708322956576012074947357177734375f,-0.2282000079035757522660787799395620822906494140625f,0.1143999840825795644150275620631873607635498046875f,-0.4360000238716608134126317963819019496440887451171875f,-1.50430001828521486828549313941039144992828369140625f,1.270200033108890291799752958468161523342132568359375f,-0.0851999779745934160413156632785103283822536468505859375f,0.257900041034818006124851308413781225681304931640625f,0.68970005175769355520998260544729419052600860595703125f,0.657399990355968188993074363679625093936920166015625f,0.148699995923042393997093313373625278472900390625f,0.1567000159800056025005687843076884746551513671875f,-0.57539999431967725485037590260617434978485107421875f,0.0129000677824018250561266540898941457271575927734375f,-0.3123999844700098282856970399734564125537872314453125f,0.89420001843869645341555951745249330997467041015625f,-0.6734000070750714161960104320314712822437286376953125f,0.64809993448257419146329993964172899723052978515625f,0.84209997856020901707552184234373271465301513671875f,-0.0465000026524062803900960716418921947479248046875f,0.80879997554123395087088965738075785338878631591796875f,0.06140002780258668746427019868860952556133270263671875f,0.1173000282466414023740952643493073992431163787841796875f,-0.580100035795569457519604839035309851169586181640625f,0.25049999956786639376815628565964289009571075439453125f,-0.792099970364570804548520754906348884105682373046875f,-0.7862999820232392522711961646564304828643798828125f,-0.16690001872777937563796513131819665431976318359375f,0.674800011855364090251896413974463939666748046875f,-0.224100042545796185322615201584994792938232421875f,0.685200013327598611567736952565610408782958984375f,-0.4375000130012629551146119410987012088298797607421875f,-0.531800006401539349099039100110530853271484375f,-0.8410000243745747727786010727868415415287017822265625f,-0.5462000100791453149895460228435695171356201171875f,-0.0525000021606685240982415052712894976139068603515625f,-0.744499996334314406709609102108515799045562744140625f,1.033900008356571476042518042959272861480712890625f,-0.65379997218847307749456376768648624420166015625f,0.26239999359697119274414944811724126338958740234375f,0.98569998257160218457784139900468289852142333984375f,-0.0841999727480111648247884659213013947010040283203125f,-0.55539998493194531192784779705107212066650390625f,0.7767000263176859764513437767163850367069244384765625f,-0.8939000357747080016679319669492542743682861328125f,-0.8580000133141876261788638657890260219573974609375f,-0.105799992513656349046868854202330112457275390625f,-1.0773000099182130639974275254644453525543212890625f,-2.273500041306018459863480529747903347015380859375f,-1.239000006005168241784986093989573419094085693359375f,-0.001299965333938501999000436626374721527099609375f,-0.35289996451735561322493595071136951446533203125f,0.686400028324127386980535447946749627590179443359375f,1.5118000612378121871870462200604379177093505859375f,1.093799970549344635628585820086300373077392578125f,-0.954900021797417952740261171129532158374786376953125f,-1.01400003558397333591756250825710594654083251953125f,0.896000041455030871162534822360612452030181884765625f,0.590000022053718709003078402020037174224853515625f,-0.749700017186999190244023338891565799713134765625f,-0.958099999153613879343538428656756877899169921875f,-0.5090999871134755938584248724509961903095245361328125f,1.296000041604042252174622262828052043914794921875f,-0.5451999850153921300943693495355546474456787109375f,1.9009000039458268105363458744250237941741943359375f,-0.425300020959973767986639359151013195514678955078125f,0.38010001709461249674859573133289813995361328125f,-0.326999990224837944197133765555918216705322265625f,-0.44880000467300451560959118069149553775787353515625f,-0.107999973565340301462356364936567842960357666015625f,-2.004499973386526079366376507095992565155029296875f,-0.560300004941225449073272102396003901958465576171875f,1.0325000262260441985517900320701301097869873046875f,0.379299979430437250726981801562942564487457275390625f,-0.612800007772445809450800879858434200286865234375f,-0.780899983754753890252686687745153903961181640625f,0.98369996463060349611851052031852304935455322265625f,1.2540000075697899095672482872032560408115386962890625f,-0.83969998924732180967112071812152862548828125f,-0.4728999795675274953055122750811278820037841796875f,-0.37790002743750850111581485180067829787731170654296875f,1.4656999881222845250050568211008794605731964111328125f,-0.4267000121615824870247024591662921011447906494140625f,0.2861999995738262470013069105334579944610595703125f,0.73149998288974149573959948611445724964141845703125f,0.1012000022187826697717127899522893130779266357421875f,-0.03550001899525534465595910660340450704097747802734375f,0.816100016267597538899281062185764312744140625f,-0.8511000078111885525089519433095119893550872802734375f,0.6662999921560288640876024146564304828643798828125f,-0.10169998222887510763712271000258624553680419921875f,1.36920003080666052852620850899256765842437744140625f,-0.5267000065363942606921909828088246285915374755859375f,0.318100010105967612172861436192761175334453582763671875f,-0.137499996554106329593736290917149744927883148193359375f,-0.56989999048635364697901195540907792747020721435546875f,0.20260000249147414042028003677842207252979278564453125f,-0.1866999969810245740831078364863060414791107177734375f,0.0718999869510532629846011332119815051555633544921875f,-0.07170002976357903179405184346251189708709716796875f,0.966699966731667803543359696050174534320831298828125f,0.6073000139415267728537628499907441437244415283203125f,0.20509996847957367993586785814841277897357940673828125f,0.30879999729692941823344654039829038083553314208984375f,-0.24330002779215542663138194257044233381748199462890625f,0.117599998430907515256649276125244796276092529296875f,1.00360000147819494742407187004573643207550048828125f,-0.8715000123381617669338083942420780658721923828125f,-0.691100027368963143459268394508399069309234619140625f,-1.1447000653177508144864304995280690491199493408203125f,-0.1810000039227304124977990795741789042949676513671875f,0.84080001825541261784024982262053526937961578369140625f,0.35289999485984446803144010118558071553707122802734375f,0.91760001728087647876463961438275873661041259765625f,-0.18439998989403239004758461305755190551280975341796875f,0.63470001171082213176077857497148215770721435546875f,-0.4276000052854420641068600161815993487834930419921875f,-0.551100026363133821405426715500652790069580078125f,0.76659997772574473007267670254805125296115875244140625f,-0.0296000052332876517624526968575082719326019287109375f,0.42049998727440840440294778090901672840118408203125f,0.6671999832496047222463175785378552973270416259765625f,0.3139999811947344898754863606882281601428985595703125f,-0.1562000355601309564690382103435695171356201171875f,-0.250000026300549738067502403282560408115386962890625f,1.634300039184093389366125848027877509593963623046875f,-0.21440001872777980640449868587893433868885040283203125f,0.077899966491758732001216003482113592326641082763671875f,0.77829999828785645377138280309736728668212890625f,-0.823100013328343482754689830471761524677276611328125f,0.58610000015571717124629458339768461883068084716796875f,0.1018999761104577572723428602330386638641357421875f,-0.3599999833106988234021628159098327159881591796875f,0.0657000056125227605008376485784538090229034423828125f,-0.83559998782873190492637149873189628124237060546875f,0.4600999788172537297015196600113995373249053955078125f,-1.54490001777410501659915098571218550205230712890625f,-0.9136999972641464129452515408047474920749664306640625f,0.17579997509792377741177915595471858978271484375f,-1.88240002388358096396814289619214832782745361328125f,1.23269998238906364207423393963836133480072021484375f,-0.24819997314661723653017588731017895042896270751953125f,1.663299990129470273814149550162255764007568359375f,1.37700001084431988829237525351345539093017578125f,-0.9210999835968021187682097661308944225311279296875f,0.66069998363330950663652174625894986093044281005859375f,-0.829100002610683617376707843504846096038818359375f,-0.2425000141374766826629638671875f,-0.142799972009658659999331575818359851837158203125f,0.5535000401139258396909781367867253720760345458984375f,2.43310002359151855699792577070184051990509033203125f,-0.667000061154366274962512761703692376613616943359375f,-0.52829997969865782891929484321735799312591552734375f,-0.729699956017732187518731734598986804485321044921875f,-0.491600019001960930609129718504846096038818359375f,-1.16889999317601223083329387009143829345703125f,0.2419999894797795203515988760045729577541351318359375f,0.7459999625720079574620058338041417300701141357421875f,-0.6913000328920781978325749150826595723628997802734375f,1.64680001519322427583347234758548438549041748046875f,-1.678600012803077135004059528000652790069580078125f,-0.63790000333488006134530223789624869823455810546875f,-0.033900023257732669890174292959272861480712890625f,-0.2768999911606311314926642808131873607635498046875f,0.92250001430511474609375f,0.5718000144481660829143265800666995346546173095703125f,-1.45710001000166045059813768602907657623291015625f,0.443600030684470514330541845993138849735260009765625f,0.60759997149705924357476760633289813995361328125f,1.447699992316961559168930762098170816898345947265625f,-0.543199984657764645135102909989655017852783203125f,-1.058800011825562048528581726714037358760833740234375f,0.50910006255060535895751172574819065630435943603515625f,-1.5754000106737013897628685299423523247241973876953125f,0.36709999929144909724954004559549503028392791748046875f,-0.15999998979270479981806829528068192303180694580078125f,0.21459995963573508248600774095393717288970947265625f,-0.3985000413805241947784452349878847599029541015625f,0.2109000313639644019758634385652840137481689453125f,0.4165999994814395535058793029747903347015380859375f,-0.6242999858379363331550848670303821563720703125f,-0.043599991941452032051529386080801486968994140625f,0.63090000931024547980996430851519107818603515625f,0.2508999878525732807332815355039201676845550537109375f,0.03319999980106957959691271753399632871150970458984375f,-1.4836000206261867884194316502544097602367401123046875f,0.35690002042278645699724393125507049262523651123046875f,0.977999962761998187232848067651502788066864013671875f,-1.313900028808415054726310700061731040477752685546875f,1.537200026462972601848377962596714496612548828125f,0.50880003020986974693329329966218210756778717041015625f,-0.36740001524090803997069087927229702472686767578125f,0.747199992879479690799371383036486804485321044921875f,-0.74679999729320367674034741867217235267162322998046875f,-0.88669998483657785914857640818809159100055694580078125f,-0.171300062191486635043702335678972303867340087890625f,-2.16050003924220845874515362083911895751953125f,0.50199999335408218570364624611102044582366943359375f,-0.18720000083297494430922824903973378241062164306640625f,-0.7773000151336191265727393329143524169921875f,-0.5375999707519998338511868496425449848175048828125f,0.241700041013956923308114710380323231220245361328125f,0.529900011068582532658410855219699442386627197265625f,0.693200044858456099206023282022215425968170166015625f,-1.39950005817413369868518202565610408782958984375f,-0.4140000041425224708291352726519107818603515625f,1.8112000172689566301897912126150913536548614501953125f,0.846599990093708232308244987507350742816925048828125f,0.10779994762763400739657981830532662570476531982421875f,-0.09549996805190985593725372382323257625102996826171875f,-1.539700007110088364470357191748917102813720703125f,-0.90590000226944678018980994238518178462982177734375f,-1.61170005986392528285477965255267918109893798828125f,0.99540004781484636708910329616628587245941162109375f,0.40599999928474428401159457280300557613372802734375f,0.01580003632307125371880829334259033203125f,-0.36479998868405816292437293668626807630062103271484375f,0.4162999921560286420429974896251223981380462646484375f,-0.2759999811053270857286179307266138494014739990234375f,-1.3336000188380483510997009943821467459201812744140625f,0.2197000317156316828715034716879017651081085205078125f,1.71740001442134371245629154145717620849609375f,-0.1301999865800140820937258467893116176128387451171875f,-0.8872000337272887282580313694779761135578155517578125f,0.212299980711936786548221789416857063770294189453125f,0.180299963329733026196066703050746582448482513427734375f,1.1674999962002046149933676133514381945133209228515625f,1.018299980127066017843162626377306878566741943359375f,0.25989999387636764094366981225903145968914031982421875f,0.378000037193298776827532492461614310741424560546875f,-0.612799965304135607624402837245725095272064208984375f,-0.144200044590235165742342360317707061767578125f,0.09199999693036087222708374611102044582366943359375f,-0.521200027811527633048171992413699626922607421875f,0.3187000185728070533741629333235323429107666015625f,-0.9788000239886345266171474577276967465877532958984375f,-0.6904000032603736514147385605610907077789306640625f,-1.110399999907612400562584298313595354557037353515625f,-0.34590001370608802755413080376456491649150848388671875f,-0.1899999885819851908053124134312383830547332763671875f,1.3084000214919446758443655198789201676845550537109375f,0.66520001703426212547043405720614828169345855712890625f,-1.61990002075433636719026253558695316314697265625f,0.44119999929442987873784431940293870866298675537109375f,0.1058999762296668478711580974049866199493408203125f,-0.170700024089962543172305231564678251743316650390625f,-0.573599984788894179388307748013176023960113525390625f,0.2513000215113165847213849701802246272563934326171875f,-0.828299992364645021325486595742404460906982421875f,0.332000001847743975158522289348184131085872650146484375f,-0.269099993073940169097113539464771747589111328125f,-1.436400017222762670598967815749347209930419921875f,-0.85159999620318416191366850398480892181396484375f,-0.58769998240768905883868455930496565997600555419921875f,1.3665000212788580569167606881819665431976318359375f,0.4139000052645804839102083860780112445354461669921875f,-0.6564000009804964452797548801754601299762725830078125f,0.7511000365331763628518046971294097602367401123046875f,-1.803100014427304387254480388946831226348876953125f,-0.634500011384488526999803070793859660625457763671875f,-0.502500002086161856595936114899814128875732421875f,-0.61889996067285490966014549485407769680023193359375f,-0.43529994637966051840294312569312751293182373046875f,-0.227299957019090559384721927926875650882720947265625f,1.2207999966859812701613918761722743511199951171875f,0.16209999718666079360218645888380706310272216796875f,0.900200009751319640827205148525536060333251953125f,0.0132000041037798609977471642196178436279296875f,1.384400005614757400707048873300664126873016357421875f,-0.07370000553429145728756566313677467405796051025390625f,-0.626999993801117483371854177676141262054443359375f,-0.1585999775618318619052615758846513926982879638671875f,-0.37510000785589203697867333175963722169399261474609375f,-0.49340001206696026958553602526080794632434844970703125f,0.51960002970844509917469622450880706310272216796875f,-1.2730999997869132389638480162830092012882232666015625f,-0.1143999546527876276513779885135591030120849609375f,-0.13269993554353742837292884360067546367645263671875f,0.4344999783039096286074709496460855007171630859375f,-0.731400003886222283000506649841554462909698486328125f,-0.515399995660782206385874815168790519237518310546875f,0.362200022476911076552141821593977510929107666015625f,-0.7475000007450578465295620844699442386627197265625f,0.56790000504851345652923555462621152400970458984375f,-0.9626000227570532530307900742627680301666259765625f,0.7976000202238555658595942077226936817169189453125f,1.38190000878125385241901312838308513164520263671875f,-0.576500016957520955429572495631873607635498046875f,0.63830001488402487996864920205553062260150909423828125f,-0.038099979707598929934420084464363753795623779296875f,0.61570000145882330944147042828262783586978912353515625f,-0.768700027662514884241318213753402233123779296875f,-0.890399994077533118996825578506104648113250732421875f,1.267899983292817722713152761571109294891357421875f,0.441700017581880377548486649175174534320831298828125f,0.130800008532404365979573412914760410785675048828125f,0.13469998509362313132697863693465478718280792236328125f,0.0948999857366086274623739882372319698333740234375f,-0.178100008336454729285236453506513498723506927490234375f,0.50980001379251493887068136245943605899810791015625f,0.7815000073835254301712893720832653343677520751953125f,1.2106999907672406635583683964796364307403564453125f,2.042199998262524740511025811429135501384735107421875f,-0.6720999393701543311863133567385375499725341796875f,0.675100049430131576855274033732712268829345703125f,-0.09099999652802903682413671049289405345916748046875f,-0.162099983477593079239653661716147325932979583740234375f,-1.89650000280141828312707730219699442386627197265625f,-0.37949996958673037905640512690297327935695648193359375f,0.164899965918063884373623295687139034271240234375f,1.121500025078655315979858642094768583774566650390625f,-1.62500005848705786348773472127504646778106689453125f,-0.846600039640069024216018078732304275035858154296875f,-1.3098000319719318440547795034945011138916015625f,0.85570001751482449581232003765762783586978912353515625f,1.16200002121925383136158416164107620716094970703125f,0.381999991044402076312280769343487918376922607421875f,-0.899800010216235790494465618394315242767333984375f,0.48460002318918704578010192562942393124103546142578125f,-0.83120006893873199516065142233856022357940673828125f,0.376599977204204350300642545334994792938232421875f,-0.7853999834418303738203803732176311314105987548828125f,-0.7896999687582255678108822394278831779956817626953125f,0.833700005310773928357548356871120631694793701171875f,-0.37830000495612614574980625548050738871097564697265625f,-0.42179994977712720327645001816563308238983154296875f,-1.191099970185757417340255415183492004871368408203125f,0.2600000002980238367200627180864103138446807861328125f,-1.1226000491321086460771994097740389406681060791015625f,1.311700032073260135945247384370304644107818603515625f,0.09419998371899163114306929855956695973873138427734375f,-1.2057999802947048095802529132924973964691162109375f,-0.685499977216124545265074630151502788066864013671875f,0.5742999760031697320528110140003263950347900390625f,0.4684999909400939799297702847979962825775146484375f,-0.06409999843835834099081694148480892181396484375f,-0.122300008204579313542126328684389591217041015625f,-0.915299994324147547786196810193359851837158203125f,-0.2395000130608677668675454697222448885440826416015625f,-0.75760002394914660595048871982726268470287322998046875f,-1.308500019215047327492129625170491635799407958984375f,0.405499998617916801446625640892307274043560028076171875f,-0.68679999269247016258077564998529851436614990234375f,0.730999982297420469734561265795491635799407958984375f,0.5581000313401229817600324167869985103607177734375f,-0.54120005310624907934879956883378326892852783203125f,-0.0941999752998352590793729177676141262054443359375f,-0.192300004479288855208096720161847770214080810546875f,0.1353999914884571609263730351813137531280517578125f,0.77250001400709111809561591144301928579807281494140625f,-0.081900007402897045949430321343243122100830078125f,-0.82269999950677130673426518114865757524967193603515625f,-0.055799995195865648156541283242404460906982421875f,-0.5455999991536140125703013836755417287349700927734375f,0.12690003429949303637158664059825241565704345703125f,-0.63000001344829803517910704613314010202884674072265625f,0.381099979349971196285196128883399069309234619140625f,-1.8383999709956351242823302527540363371372222900390625f,0.227899981560557574056247176486067473888397216796875f,0.7122000050425523198782684630714356899261474609375f,1.65369993959665340099718378041870892047882080078125f,0.402599974477290789565131490235216915607452392578125f,0.0008999549210084722972169402055442333221435546875f,-1.2137000026285651443913593539036810398101806640625f,-0.20309998261332484759122962714172899723052978515625f,-0.757300015431642492558239609934389591217041015625f,1.620999954879284388198357191868126392364501953125f,0.0536999989032747038208981393836438655853271484375f,-0.4729999787807468436540148104541003704071044921875f,-0.10689999585449694308181278756819665431976318359375f,0.924200035575032163848163691000081598758697509765625f,-0.346499997660516978470468529849313199520111083984375f,0.18229999427199317096892627887427806854248046875f,0.944700024488569045644226207514293491840362548828125f,-0.499300001372396717869150961632840335369110107421875f,0.20319999337494376590029787621460855007171630859375f,-0.7709000319227576891734088349039666354656219482421875f,-0.292699977862835136743768771339091472327709197998046875f,0.6378000055745243201954508549533784389495849609375f,-0.256499991923570480611971333928522653877735137939453125f,0.0333000112518668789363118776236660778522491455078125f,-0.265399997150897792863588620093651115894317626953125f,-0.0739000290319327124421988628455437719821929931640625f,-0.09959999577105060897252997165196575224399566650390625f,0.299300028604269297005657790577970445156097412109375f,-0.7333000381112100285463384352624416351318359375f,0.96849997842311807971782400272786617279052734375f,-0.636299994093179055454356785048730671405792236328125f,0.403799942162632152786727601778693497180938720703125f,0.173800013762713145837324191234074532985687255859375f,-0.49810001018047334042648799368180334568023681640625f,0.1289999816417690681191743351519107818603515625f,-0.4474000116273766369801023756735958158969879150390625f,0.4703000335514548879700669203884899616241455078125f,-0.07750000260770295579249022921430878341197967529296875f,0.5847000229239467383735018302104435861110687255859375f,0.094299997125566026312526446417905390262603759765625f,0.4348000131219624808665003001806326210498809814453125f,-0.66730003539919879873565378147759474813938140869140625f,-0.3327999901890752454391986248083412647247314453125f,1.0290000454559926623687715618871152400970458984375f,-0.669300036054850000510896279592998325824737548828125f,0.623800004971027277633766061626374721527099609375f,-0.8019999932050705382380328956060111522674560546875f,1.387100005605816033238397722016088664531707763671875f,1.120799927693606701240014444920234382152557373046875f,-1.425400048187375201536042368388734757900238037109375f,1.038499982669951027247634556260891258716583251953125f,-1.071000006586313446632630075328052043914794921875f,0.5785999981999392360876299790106713771820068359375f,0.89260000919699677979224361479282379150390625f,-1.08959997662305863030951513792388141155242919921875f,1.0942000165015459689499266460188664495944976806640625f,-0.56819997936785160419503881712444126605987548828125f,0.922300000046193613201239713816903531551361083984375f,-1.409599988655745761434445739723742008209228515625f,-0.9293999854981895847316764047718606889247894287109375f,-0.46159997981190681315410984097979962825775146484375f,0.64110002084970485469739287509582936763763427734375f,0.84699996265768984216038006707094609737396240234375f,-0.03249999418854709887227727449499070644378662109375f,-1.531900025470555259943239434505812823772430419921875f,1.2714000160306699882539760437794029712677001953125f,-0.383199997323751428979221600457094609737396240234375f,0.41830001724958432163248289725743234157562255859375f,-0.1834999909400938378212231327779591083526611328125f,-0.19529999659657459876171969881397671997547149658203125f,0.638099986264109464428884166409261524677276611328125f,-0.92299998325109466890125986537896096706390380859375f,-0.15990000767856871188854483989416621625423431396484375f,-0.0777000117629766950955172433168627321720123291015625f,-0.033000008098781119514342208276502788066864013671875f,0.24049999345839001563263082061894237995147705078125f,-0.789699997666478026303593651391565799713134765625f,-0.302600036615133571871183448820374906063079833984375f,0.810600012862682461900476482696831226348876953125f,-0.1764999899864196919452297152020037174224853515625f,-0.6848000468730930379024357534945011138916015625f,0.992800054264069586906771291978657245635986328125f,0.275499962873756221171106517431326210498809814453125f,1.11030004383325575645358185283839702606201171875f,-0.13430001094639276715270170825533568859100341796875f,0.514600023710727327852509915828704833984375f,0.37819996677637046644804286188445985317230224609375f,-1.122000002890824799095526032033376395702362060546875f,0.8095999953612682542569700672174803912639617919921875f,-0.2020000146627420889444692875258624553680419921875f,0.7725000347569590530838468112051486968994140625f,-0.17770000438690214394910071860067546367645263671875f,0.2445000391379001936087433932698331773281097412109375f,0.35069999359846093511805520392954349517822265625f,0.4725000023841856577888620449812151491641998291015625f,0.3288000124216079456118677626363933086395263671875f,-0.505800008420646207696336205117404460906982421875f,0.16659999754428866935995756648480892181396484375f,0.1852999819934364467144405352883040904998779296875f,0.24289995334148528627338237129151821136474609375f,0.8430000026226043274846233543939888477325439453125f,-1.3638999514341350050017354078590869903564453125f,0.4957000272750848868241746458807028830051422119140625f,-1.3953999598979951457522474811412394046783447265625f,-0.327599977904558414820712641812860965728759765625f,-0.610300017607212197390254004858434200286865234375f,1.188799972635508073182109001209028065204620361328125f,1.3878999482750895566596227581612765789031982421875f,-1.5751999902307982637239547329954802989959716796875f,-0.385899970865249741791558335535228252410888671875f,1.5045000197291376053954081726260483264923095703125f,0.311299975466727829598312382586300373077392578125f,0.470300017011165749636347754858434200286865234375f,-1.23769996713399876853145542554557323455810546875f,0.0551000040188431849941252949065528810024261474609375f,-0.38940003165453707101306690674391575157642364501953125f,0.6488000199094419162548774693277664482593536376953125f,0.3748000123396517668794558630906976759433746337890625f,-0.6271000006139277882510896233725361526012420654296875f,0.6506000139802694537394245344330556690692901611328125f,-0.2451999989107249700026613936643116176128387451171875f,0.58250000197440365834467002059682272374629974365234375f,0.3582000075682998385673272423446178436279296875f,-0.0314999804869289246056496267556212842464447021484375f,0.491300001618265813618791071348823606967926025390625f,-0.9411000407055023497804313592496328055858612060546875f,1.0052000242426990528343822006718255579471588134765625f,0.08989999219998712209900304515031166374683380126953125f,-0.22359999415799969302298677575890906155109405517578125f,0.59960000262558466044282567963819019496440887451171875f,-0.6338999943681062898548361772554926574230194091796875f,-0.62150003193318870131633957498706877231597900390625f,0.1737000314623127206203889727476052939891815185546875f,0.298600004905461791082643685513176023960113525390625f,-0.63970001323819136729298406862653791904449462890625f,-0.16550000314414514690497526316903531551361083984375f,-0.4693000239849089982868690640316344797611236572265625f,-0.49640001021921642543333064168109558522701263427734375f,0.00719996864646665901688038502470590174198150634765625f,-1.1586999922722573952427183030522428452968597412109375f,-0.1404000308275232811183741432614624500274658203125f,-1.4275000286102297319956733190338127315044403076171875f,0.5833000192612411982651110520237125456333160400390625f,0.344100031295419217514108822797425091266632080078125f,0.37500000100582842055274568338063545525074005126953125f,-0.381400001800060284296023382921703159809112548828125f,-0.14880000895708811459172693503205664455890655517578125f,-0.64320000738203508827695031868643127381801605224609375f,0.0829999959915876939220424901577644050121307373046875f,-1.4597999910309911175687602735706605017185211181640625f,0.5196999776616701982590029729180969297885894775390625f,-0.43699998769164072331960824158159084618091583251953125f,0.47839997265338940479750817758031189441680908203125f,-0.15880001696646228737108685891143977642059326171875f,0.932299976390600360076632568961940705776214599609375f,-0.646100061567127514905450880178250372409820556640625f,1.128300000038743089447734746499918401241302490234375f,-0.82080002425313036695797563879750669002532958984375f,0.017800017905234000181735609658062458038330078125f,-0.7813000321283938287564296842901967465877532958984375f,0.8757000240713355143640228561707772314548492431640625f,-0.06720000573173157132345068021095357835292816162109375f,-0.32389997833073136401793590266606770455837249755859375f,0.40789999988898639760037667656433768570423126220703125f,-0.56489998147115139115470583419664762914180755615234375f,0.102499998584389739875177838257513940334320068359375f,1.282399988530575907219599685049615800380706787109375f,-1.0346999894708390765885042128502391278743743896484375f,-0.23009996792077969107737089871079660952091217041015625f,-0.4442000137448312013788154217763803899288177490234375f,0.0461999813571569717396414489485323429107666015625f,0.44170003751218356313756885356269776821136474609375f,0.7987999968498942227057568743475712835788726806640625f,0.91960001989230499841454502529813908040523529052734375f,0.044899987841397592802650251542218029499053955078125f,0.569699989042430576802189534646458923816680908203125f,-1.8012000025168057870672555509372614324092864990234375f,0.563299994823336813709602211019955575466156005859375f,-0.189600002588331761899098637513816356658935546875f,-0.553200003358721659196817199699580669403076171875f,0.656900003230571627454992267303168773651123046875f,-0.40949998962879163855177466757595539093017578125f,0.5106999897241593089347588829696178436279296875f,-0.5235000097155566578521757037378847599029541015625f,-0.1233000173240903496463261035387404263019561767578125f,0.35629999513626042784153469256125390529632568359375f,-0.488000040508806964822952068061567842960357666015625f,0.83060004802942355439654420479200780391693115234375f,-1.0309000567704444950578590578516013920307159423828125f,0.166299987424910256095245131291449069976806640625f,-0.1716999397046852227077806674060411751270294189453125f,-0.2913000188104810650457920928602106869220733642578125f,-0.951999986462295222366947200498543679714202880859375f,-0.15629997487068170158863722463138401508331298828125f,1.0194999996498221861429556156508624553680419921875f,-0.159499993503093406843618140555918216705322265625f,1.34200000616908088346690419712103903293609619140625f,-0.048299995195865363939446979202330112457275390625f,-0.0892000101685523905103991637588478624820709228515625f,-0.294399954056739687757726642303168773651123046875f,-0.293200005668401875169593040482141077518463134765625f,-0.4349999955296510734115145169198513031005859375f,1.675400009667872947005662354058586061000823974609375f,0.050700000452994942179429926909506320953369140625f,1.36819999553561189031825051642954349517822265625f,-2.0748000033617017834330908954143524169921875f,-0.58970005213022158585545184905640780925750732421875f,0.6862000204354525845218404356273822486400604248046875f,-1.2958999928258359002342103849514387547969818115234375f,0.9818999941036097300894880390842445194721221923828125f,-1.62219996257424359242804712266661226749420166015625f,-0.1163000291109081985041484585963189601898193359375f,1.1880000164806840956543965148739516735076904296875f,0.3310999803185463719046310870908200740814208984375f,0.566699999737739545935255591757595539093017578125f,-0.484799996209144268277668743394315242767333984375f,0.0375000040978192838991844837437383830547332763671875f,-0.3706999979570511083437622801284305751323699951171875f,0.05649997642636339634236719575710594654083251953125f,1.1554000370860102453463014171575196087360382080078125f,-0.88640003443360360080305326846428215503692626953125f,-0.0411999853804705384874296214547939598560333251953125f,0.89999999105930328369140625f,-0.017700053709746033092642392148263752460479736328125f,-0.1280999746784565740398420530254952609539031982421875f,0.254500041726977155320810197736136615276336669921875f,-0.65520003990754516198791179704130627214908599853515625f,-0.39480002183914220115212856399011798202991485595703125f,0.5313000061444939081667371283401735126972198486328125f,0.7727000137001278634585332838469184935092926025390625f,0.668400015270709690895500898477621376514434814453125f,-0.15799999368190764670316639239899814128875732421875f,0.2338000057160858347060639061965048313140869140625f,0.3404999957680701072604279033839702606201171875f,0.094700012046099235618612510734237730503082275390625f,1.4322000349193813040216127774328924715518951416015625f,-0.75410003926754054504044688655994832515716552734375f,-0.6796000165209175047920098222675733268260955810546875f,0.555199997383355903224355643033050000667572021484375f,-0.16539999566078211756803284515626728534698486328125f,-0.3568999848276366293475803104229271411895751953125f,-0.515199975031614076925734480028040707111358642578125f,-0.506200037795305757271080437931232154369354248046875f,-0.0264999958723783901604065249557606875896453857421875f,1.19120002918988543427758486359380185604095458984375f,-0.98830001613199713883517460999428294599056243896484375f,1.07800002914667114595204111537896096706390380859375f,-1.2560999941766259890840728985494934022426605224609375f,0.34620001708269132478790197637863457202911376953125f,-0.8962000370502476442879924434237182140350341796875f,-0.72700001496076538387569598853588104248046875f,-0.193699986907839072358683552010916173458099365234375f,0.267299968492986206314299124642275273799896240234375f,0.97520001690387747572685839259065687656402587890625f,-0.2117000212699178263164867530576884746551513671875f,-0.440799998995661912459809173014946281909942626953125f,1.360899998953938538903685184777714312076568603515625f,-1.99970000832080785357902641408145427703857421875f,-0.364300006848572888173976025427691638469696044921875f,0.70690002392455963953210584804764948785305023193359375f,1.037799963553249771308628623955883085727691650390625f,1.531599982102960666452418081462383270263671875f,-0.76280001319274270965564710422768257558345794677734375f,0.180700021165609570061860722489655017852783203125f,-0.79449993628263460010430208058096468448638916015625f,-0.48120000203251844794749558786861598491668701171875f,-0.68860000252127573361349277547560632228851318359375f,0.11780000218451025517651942209340631961822509765625f,-0.8879000105619432314796313221449963748455047607421875f,0.9182000247418884431027663595159538090229034423828125f,1.0481999927416441042993255905457772314548492431640625f,-0.13249998085200775932435135473497211933135986328125f,0.211000011429190781342413174570538103580474853515625f,0.2516999992534516561448754146113060414791107177734375f,-1.814700019682943921139894882799126207828521728515625f,0.9799999977275728468839588458649814128875732421875f,0.04990001199990512503745776484720408916473388671875f,-0.417900031796097692904368159361183643341064453125f,-0.89129997576475139542395709213451482355594635009765625f,0.4950999947428711411845370093942619860172271728515625f,0.35949998709559427556570199158159084618091583251953125f,-0.6853000049412252270286671773646958172321319580078125f,0.6786000065445900730765060870908200740814208984375f,-0.7719000030815603707878835848532617092132568359375f,-0.6647000363349917506639030762016773223876953125f,1.355300044727325570192988379858434200286865234375f,-0.339700026686489575666882956284098327159881591796875f,0.36420000361204163397133015678264200687408447265625f,-1.0986000170499086170394775763270445168018341064453125f,-0.1341000113278629779500761287636123597621917724609375f,-0.96280001417994498513763801383902318775653839111328125f,0.1567000336378809866033634534687735140323638916015625f,1.5180000389441854480310212238691747188568115234375f,-0.2466999791741368852626692387275397777557373046875f,0.865299962212144446738193437340669333934783935546875f,0.6259000361323359129528398625552654266357421875f,1.0685999808400865607183050087769515812397003173828125f,-0.099400013065338299611539696343243122100830078125f,0.7150000181049109837516652987687848508358001708984375f,-0.3462000179767610319458981393836438655853271484375f,-0.172399997322261133714249581316835246980190277099609375f,0.03640000269412979605476721189916133880615234375f,0.4544999711513515450178601895458996295928955078125f,-0.63159996446371025058397208340466022491455078125f,0.39269997845888138243708453956060111522674560546875f,-0.9066999679982661319854742032475769519805908203125f,0.1016999888226390869050419496488757431507110595703125f,-0.2673000089496377906783664002432487905025482177734375f,-1.3477999964103102303880632462096400558948516845703125f,-0.2651999747708437826787530866567976772785186767578125f,1.66600006201863237720317556522786617279052734375f,0.039699950504303416209950228221714496612548828125f,0.021100011014938413467234568088315427303314208984375f,-0.746000016570089741207993938587605953216552734375f,0.449300005060434326509266611537896096706390380859375f,-0.33759997447729173103425637236796319484710693359375f,0.66230000440478331569238434894941747188568115234375f,-0.18149999132752459019002344575710594654083251953125f,-0.0729999924898148133678432714077644050121307373046875f,-0.8906999863341449330533805550658144056797027587890625f,-0.135200005951523483105347622768022119998931884765625f,0.3516000248134136807465210949885658919811248779296875f,-0.34610003291964519522849741406389512121677398681640625f,-1.1412999781116841546690920949913561344146728515625f,0.4540999778002501319207340202410705387592315673828125f,0.91539999797046167628877810784615576267242431640625f,0.223000003367662458231279742904007434844970703125f,-0.9750000212714076130993134938762523233890533447265625f,0.84670000800788403072516530301072634756565093994140625f,-0.84850000498443822749550236039794981479644775390625f,0.410299995031952580148981724050827324390411376953125f,0.826000004574656276901123419520445168018341064453125f,-0.2252999913811681853559321098146028816699981689453125f,-0.51829999281167982161377949523739516735076904296875f,-0.02390002206563934805672033689916133880615234375f,-0.56719998060464860145657439716160297393798828125f,-0.22619998072385794074534715036861598491668701171875f,-0.465200023832917253230334608815610408782958984375f,-1.0279000271022322721137243206612765789031982421875f,1.0536999946936962402332937926985323429107666015625f,0.031599997097253140765360512887127697467803955078125f,1.0041999948576094681129688979126513004302978515625f,-0.32670000987052905117025147774256765842437744140625f,-0.2361000174596910028412821702659130096435546875f,-0.36199999424815132442745380103588104248046875f,0.54510002588629713482504257626715116202831268310546875f,0.281500008612871166491231633699499070644378662109375f,0.8531000326812272493981481602531857788562774658203125f,-0.5645000000596045452994076185859739780426025390625f,-0.15290000530928382627138262250809930264949798583984375f,-0.37740001538991929663779956172220408916473388671875f,0.5122999969169494960397059912793338298797607421875f,1.145399991190434008103693486191332340240478515625f,-0.92139999047517751051827872288413345813751220703125f,-0.9751000118792045867621709476225078105926513671875f,-0.087599981927871883868874647305347025394439697265625f,0.59039995468258865685129421763122081756591796875f,0.4134999859482046957737111370079219341278076171875f,0.4307999878197907861476778634823858737945556640625f,0.172099996441602343111298978328704833984375f,0.2113999831736084189515167963691055774688720703125f,-1.527199996250867553015950761619023978710174560546875f,0.50079995838999735724428319372236728668212890625f,0.508200002762675406842163283727131783962249755859375f,0.82319999561011769628748879767954349517822265625f,-0.950199992913007829287153072073124349117279052734375f,-0.2835999926865095943639971665106713771820068359375f,-0.197400001680851033825092599727213382720947265625f,1.074799954485893493938419851474463939666748046875f,0.101499996542930404075377737171947956085205078125f,-1.5101000585943460396975979165290482342243194580078125f,0.97069999985694810362701900885440409183502197265625f,0.1216999682962889295367858721874654293060302734375f,-0.399199988412856754393942537717521190643310546875f,-0.1643000367254028315500136159243993461132049560546875f,-0.45609998389482431235819603898562490940093994140625f,-2.103200038674474026123561998247168958187103271484375f,-1.93710002043247175862461517681367695331573486328125f,0.49750003375113127646045541041530668735504150390625f,0.9806999906182287674027975299395620822906494140625f,-1.571300080966949952454569938709028065204620361328125f,-2.2172000011682513331834343262016773223876953125f,0.260300064992903923410949573735706508159637451171875f,1.07530003995895384605319122783839702606201171875f,-0.261099982181191681007703664363361895084381103515625f,0.30040000400543220848703640513122081756591796875f,0.3403999830693005268500428428524173796176910400390625f,0.480000018179417065056213687057606875896453857421875f,0.827500003874301892636822230997495353221893310546875f,-0.09740000838637374869222185225225985050201416015625f,-0.2212999987870454265959097028826363384723663330078125f,-0.344399953460693186713115210295654833316802978515625f,-1.08499999292194804212385861319489777088165283203125f,2.2907000119268889903878516633994877338409423828125f,-0.40510000927150269234289226005785167217254638671875f,-1.0740999909877775309041680884547531604766845703125f,-1.361400016105175136971183746936731040477752685546875f,2.6444999896287928464744254597462713718414306640625f,0.860099971199036250624203603365458548069000244140625f,-1.2315999996304523023127330816350877285003662109375f,-0.643199961039424561448640815797261893749237060546875f,-0.1473999930381779677190934307873249053955078125f,0.024600025200843589345822692848742008209228515625f,0.92820002816915536669739594799466431140899658203125f,0.41230001021623596813014955841936171054840087890625f,0.96749999321997137702311420071055181324481964111328125f,-1.394299997684359748717497495817951858043670654296875f,0.2380999845691025296190446169930510222911834716796875f,-0.6173000115945936894235046565881930291652679443359375f,0.090800007861852716217754277749918401241302490234375f,0.64750001356005615349431536742486059665679931640625f,0.1584999869726593235697009731666184961795806884765625f,-0.773399967288970202616837923415005207061767578125f,1.000300019171834531306330973166041076183319091796875f,-0.1142999827831985282244886548141948878765106201171875f,1.1891000034675001106876379708410240709781646728515625f,-0.0898000273525714209910120189306326210498809814453125f,-0.375800009463727435754520911359577439725399017333984375f,0.86630000973939846886651139357127249240875244140625f,0.139499988287687226584665722839417867362499237060546875f,-0.088400016164779771088433335535228252410888671875f,-0.329100031518936742003234030562452971935272216796875f,-0.7478000372767450443234338308684527873992919921875f,-0.013099988164007481117323550279252231121063232421875f,1.15670003635734364166864907019771635532379150390625f,-0.0412999711081382392396932345945970155298709869384765625f,-0.594700001391768484637623259914107620716094970703125f,-0.59629998448193166638020557002164423465728759765625f,-0.418399999177456294319199514575302600860595703125f,1.673500063583254426902158229495398700237274169921875f,2.429099989795684422233534860424697399139404296875f,0.33809998194277302019372655195184051990509033203125f,-0.8602000112414358579115969405393116176128387451171875f,0.17669998915791496063576460073818452656269073486328125f,0.1940999788433313000268753967247903347015380859375f,0.1536999800533058735396707561449147760868072509765625f,-1.015000022947788238525390625f,-0.661600014829635707513943998492322862148284912109375f,1.00440003184080151044099693535827100276947021484375f,0.487600005620718146559511296800337731838226318359375f,-1.086500009953975887100341424229554831981658935546875f,0.420200007590651569699957690318115055561065673828125f,-0.196800003048777316649875501752831041812896728515625f,-0.48280000933706757049179714158526621758937835693359375f,0.840400021290779619675959111191332340240478515625f,-0.245699995312094632282651218702085316181182861328125f,-2.4764999747872362689804504043422639369964599609375f,0.06890003108084119531895339605398476123809814453125f,0.4261999790847303959395730998949147760868072509765625f,0.82659998294115089034761467701173387467861175537109375f,-0.340999976038932839372819216805510222911834716796875f,-0.5017999974608422508737248790566809475421905517578125f,0.7354000004291538772349667851813137531280517578125f,-0.1172000318646428951296911691315472126007080078125f,-0.11200001440197249191754735875292681157588958740234375f,-0.8560999663487078858992163077346049249172210693359375f,0.3295999993845819542315211947425268590450286865234375f,-0.7767000374376766824724427351611666381359100341796875f,-1.16229999995306076954193486017175018787384033203125f,1.4382000093936913032166557968594133853912353515625f,0.5141999779261643421790495267487131059169769287109375f,-1.046200012314319138795326580293476581573486328125f,0.707100001675635692066634874208830296993255615234375f,-2.13860000833273034004378132522106170654296875f,0.42489996720328881973927082071895711123943328857421875f,0.19719995244145405166591444867663085460662841796875f,-1.546700045465678297063050194992683827877044677734375f,1.28490002947151626955246683792211115360260009765625f,0.35259998535513858985979140925337560474872589111328125f,-0.5870999870911239870707731824950315058231353759765625f,0.318399998544156570545737849897705018520355224609375f,0.2630000035911799205479155716602690517902374267578125f,-0.3329999875351787874677711442927829921245574951171875f,-0.1623999715805053423167692017159424722194671630859375f,-0.746899997121095449159611234790645539760589599609375f,1.483399980625510039544678875245153903961181640625f,0.156999941572546930501630413345992565155029296875f,-0.354199972021579156944426358677446842193603515625f,-0.17720000690519821517909804242663085460662841796875f,1.3501000358700760983765576384030282497406005859375f,0.19210000568032281620389767340384423732757568359375f,0.634300053936243557473062537610530853271484375f,-1.571300043565035498005499903229065239429473876953125f,0.8243999975681308711727979243732988834381103515625f,0.6503999590411779774257183817098848521709442138671875f,0.1602000224918131987550395933794789016246795654296875f,0.3697000006839632479938018150278367102146148681640625f,0.8528000246852638355221642996184527873992919921875f,0.36160000633597377817096685248543508350849151611328125f,-0.1010000277832150405998845599242486059665679931640625f,0.6371999919667838963022177267703227698802947998046875f,-0.586599952617288256107030974817462265491485595703125f,-0.5360999632567171602204325608909130096435546875f,0.8993999778985983795109859784133732318878173828125f,0.76389998488724319969378484529443085193634033203125f,-0.7102000371694572322667227126657962799072265625f,1.31450003075599664015271628159098327159881591796875f,-1.614499989926814205176697214483283460140228271484375f,-0.14460000523328708776915618727798573672771453857421875f,0.90440005717277482943927680025808513164520263671875f,-0.5320999579221010744589648311375640332698822021484375f,0.29699994939565643736756328507908619940280914306640625f,1.085399978375435114941183201153762638568878173828125f,-0.77810002325624250119062708108685910701751708984375f,0.6543000030860304239155311734066344797611236572265625f,-0.32570000879764560153262209496460855007171630859375f,-0.17709999957084665567208503489382565021514892578125f,0.414800022677332569376318360809818841516971588134765625f,-0.753400037324428950569199514575302600860595703125f,-0.81900001930445409659142796954256482422351837158203125f,0.34190001217126830823644922929815948009490966796875f,0.6192000214934350044160282777738757431507110595703125f,-2.102900019353628469076511464663781225681304931640625f,-1.4889000383451584941241208071005530655384063720703125f,-0.19040003798007898438982010702602565288543701171875f,-0.69829995594993177210341173122287727892398834228515625f,-0.855600029551982999009851482696831226348876953125f,-0.74540000761896363368208540123305283486843109130859375f,-0.5999000435531147701340159983374178409576416015625f,-1.765200053597987039921690666233189404010772705078125f,0.635500010222196376474812495871447026729583740234375f,-0.50970001137554643566573986390721984207630157470703125f,-0.24260000427961347924110668827779591083526611328125f,0.494500045806170351880837188218720257282257080078125f,-0.3910000272989275771351458388380706310272216796875f,-1.3189000213950861084555299385101534426212310791015625f,1.0803999850809571370291450875811278820037841796875f,0.30549998652935095311278246299480088055133819580078125f,-0.743800001692771939332260444643907248973846435546875f,-2.82080004809498774420717381872236728668212890625f,0.681900071924925299526876187883317470550537109375f,0.507400091162324873295119687099941074848175048828125f,0.67700003939867059443713515065610408782958984375f,-2.0967000495821235261928450199775397777557373046875f,-0.6926000337839131049122443073429167270660400390625f,-1.142600053751468891505282954312860965728759765625f,0.777900029635429479668573549133725464344024658203125f,0.030899998953938467849411608767695724964141845703125f,-0.2826000080049038132301575387828052043914794921875f,0.264199997204542125217585635255090892314910888671875f,-0.56850000107288334305621901876293122768402099609375f,-0.5516000239938498861391735772485844790935516357421875f,0.270100033485890289597364244400523602962493896484375f,0.04509999980926548968085398882976733148097991943359375f,0.2174999690055852141767900320701301097869873046875f,0.5424999746680263523757048460538499057292938232421875f,0.75040001518130328150846253265626728534698486328125f,0.590800038036704666666310004075057804584503173828125f,-1.2875000399351126390001809340901672840118408203125f,-1.1769999838918450141278526643873192369937896728515625f,1.4985000249147422124451622948981821537017822265625f,0.41989999795556087747172568924725055694580078125f,-0.08209997751712716507199729676358401775360107421875f,-0.3935000051707024937286405474878847599029541015625f,0.20700000289082520765759909409098327159881591796875f,-0.40840001735687270212338262354023754596710205078125f,0.0347000089541078526877981857978738844394683837890625f,-0.82620003207698478053799817644176073372364044189453125f,-1.54540004243180195686591105186380445957183837890625f,1.7618000168323515453039362910203635692596435546875f,0.02359999309629190467063608593889512121677398681640625f,-0.1307999642014501251452429642085917294025421142578125f,0.54790001248046771298305657182936556637287139892578125f,-0.282100015440583273829133759136311709880828857421875f,0.2207000050537286650609303251258097589015960693359375f,0.357899980051815447268381831236183643341064453125f,-0.5495000170096755454096637549810111522674560546875f,0.31550002661347409382841533442842774093151092529296875f,0.4407000173844399792955073280609212815761566162109375f,-1.342100018849224252193153006373904645442962646484375f,-0.6636000139765443339712192027946002781391143798828125f,1.04559998887181304638716028421185910701751708984375f,-0.23989999069124456898549624384031631052494049072265625f,-0.5458999983020127455546344208414666354656219482421875f,-0.9797000454060740270989526834455318748950958251953125f,1.6762000181630252182429785534623079001903533935546875f,0.91559998395442931951038190163671970367431640625f,-0.433700003820657542519256821833550930023193359375f,0.55440000099539743505516753430129028856754302978515625f,-0.477700019586086543910141699598170816898345947265625f,0.19870001820027793915102165556163527071475982666015625f,-1.1883000013053415244712596177123486995697021484375f,-0.340499989397823998871217554551549255847930908203125f,0.18479998682141307586590528444503434002399444580078125f,-0.3364000103123483409461869086953811347484588623046875f,-0.9614000220097602689151017330004833638668060302734375f,0.0163000064425171675708270413451828062534332275390625f,0.8869000271841882554468838861794210970401763916015625f,-0.2169999959617854301541228778660297393798828125f,-1.3130000219941138084323029033839702606201171875f,0.272599992470443197589702322147786617279052734375f,-0.4128999537512658690729949739761650562286376953125f,-0.16909997183978553447758486072416417300701141357421875f,0.398899965739250017549011317896656692028045654296875f,0.12509997291266905516948781951214186847209930419921875f,-0.1876000030875208945957410833216272294521331787109375f,0.30129998828172654956603082609944976866245269775390625f,-0.04749998636543750762939453125f,-0.8597999959856270635327746276743710041046142578125f,0.2634000373989342502767385667539201676845550537109375f,-0.84539998679459105179745392888435162603855133056640625f,0.9650999679580329893013868058915250003337860107421875f,0.8475999804750082855520076918764971196651458740234375f,-1.275399972005188065082847970188595354557037353515625f,-0.66539999119043324871114464258425869047641754150390625f,1.08470000519156428708811290562152862548828125f,-0.65349999973177919088129783631302416324615478515625f,0.00749998569488496968915569595992565155029296875f,-1.31760001506432899986975826323032379150390625f,-0.46960001771301029815930405675317160785198211669921875f,-0.088300004155188738241832879793946631252765655517578125f,-0.06759999172538515288533034208739991299808025360107421875f,-0.06030001887381075675875763408839702606201171875f,-0.8683000145114958545633498943061567842960357666015625f,0.7750000213831664819252864617737941443920135498046875f,-0.143899992300569989911451784792006947100162506103515625f,0.31769997957646811670429087826050817966461181640625f,-0.12390001193285027358825800547492690384387969970703125f,0.4558000299900777374517701900913380086421966552734375f,1.844100021591037918966549113974906504154205322265625f,0.7299000329732887593081613886170089244842529296875f,1.252599985522776737667527413577772676944732666015625f,-0.7126999969363208009554000454954802989959716796875f,0.815900020523369295943894030642695724964141845703125f,-0.313999995052814284690612112171947956085205078125f,-0.29029998372569654208064093836583197116851806640625f,1.63320006229281489140703342854976654052734375f,-0.151899993973225233556689772740355692803859710693359375f,-0.3573999983839690752773776694084517657756805419921875f,-0.33430001549124710447102870602975599467754364013671875f,0.7920000012516974496890043155872263014316558837890625f,-0.66380004028677941363412173814140260219573974609375f,-1.24620000464022151476228827959857881069183349609375f,0.82470003074705566792346189686213620007038116455078125f,1.6790000555515292024466589282383210957050323486328125f,2.0590000074952836683905843528918921947479248046875f,0.4645999334841963701592248980887234210968017578125f,0.5259999853521595269967292551882565021514892578125f,0.3590000129342083567962617962621152400970458984375f,0.60429999842941750642921761027537286281585693359375f,0.82280003459453610048512928187847137451171875f,0.613400017952919096586583691532723605632781982421875f,-1.16350001876801290023877299972809851169586181640625f,0.1742000192955137638506357689038850367069244384765625f,0.14729999302402141392320800150628201663494110107421875f,-0.0888000012457370591079097721376456320285797119140625f,-0.120399978822469666539518584613688290119171142578125f,-0.2788999908477067801726434481679461896419525146484375f,1.47469996935427172246590998838655650615692138671875f,0.75249998629093173274640093950438313186168670654296875f,-1.275600014501809820188782396144233644008636474609375f,-0.354799984586238625894338838406838476657867431640625f,-1.174899978062511340937135173589922487735748291015625f,0.441800004538893009708999670692719519138336181640625f,0.31079997582435670011591355432756245136260986328125f,1.0304999739378697398706208332441747188568115234375f,-0.82940004316568405329235247336328029632568359375f,1.1042000179171562734836697927676141262054443359375f,0.674500012800097525911269258358515799045562744140625f,-0.6851999965637929346229384464095346629619598388671875f,0.1470999856010077877499497844837605953216552734375f,-0.8121999874591825374636755441315472126007080078125f,0.205799986180662752843772977939806878566741943359375f,-1.67569997683465476967512586270458996295928955078125f,-0.254100030066072957168898938107304275035858154296875f,-0.8551000286802652450290906926966272294521331787109375f,-0.40320000689774759816685900659649632871150970458984375f,0.707200021731853922801747103221714496612548828125f,0.185699995367973880178169565624557435512542724609375f,-0.6528000050902367235039491788484156131744384765625f,0.383899998838454603600212067249231040477752685546875f,-1.726600016394257863794337026774883270263671875f,-0.56990001162737602413699278258718550205230712890625f,0.8638999828383326917702333958004601299762725830078125f,-0.544099993278831295384634358924813568592071533203125f,-0.6035000189542769621908746557892300188541412353515625f,-0.9431999749720090253646276323706842958927154541015625f,0.67080003535449517215738524100743234157562255859375f,-0.58870002110600605504941995604895055294036865234375f,1.623300020527839393480462604202330112457275390625f,-1.30050002784282003887028622557409107685089111328125f,1.467400047762692505415316190919838845729827880859375f,0.5359000451847915069691907774540595710277557373046875f,1.654700068446994354331991416984237730503082275390625f,-0.7453000547856103441546338217449374496936798095703125f,-0.589000034093857038897112943232059478759765625f,0.1495000214427715246046091124298982322216033935546875f,1.1345000551939019572955658077262341976165771484375f,-0.739600006313621438636118909926153719425201416015625f,0.9196000257223844176479587986250407993793487548828125f,-0.1375000013783573837855556121212430298328399658203125f,-0.72350000844895845375503995455801486968994140625f,-0.00580000093281274331502572749741375446319580078125f,0.151299985823035942900105510489083826541900634765625f,0.419099996203184499421467990032397210597991943359375f,-0.4851999784588816311270420555956661701202392578125f,0.9986000425308940720015016268007457256317138671875f,-0.29360000773668348816869411166408099234104156494140625f,1.3288000256091354600584963918663561344146728515625f,-1.71700002606213075750929419882595539093017578125f,-0.473599990078806598603478050790727138519287109375f,1.34520003359317730229349763249047100543975830078125f,0.6643000225692985427627945682615973055362701416015625f,-0.8585000508427622634144427138380706310272216796875f,0.673200005519390298758253265987150371074676513671875f,-0.66269999276399571552786937900236807763576507568359375f,0.16870000590682054930624644839554093778133392333984375f,0.60230000384598980645023402757942676544189453125f,-0.6470000030770897136989106002147309482097625732421875f,-0.43070001621097342425770193585776723921298980712890625f,0.9752000180587170330426260989042930305004119873046875f,-0.08639999345540994024617020841105841100215911865234375f,-0.83840000327527519630166352726519107818603515625f,0.337800009318440952466033877499285154044628143310546875f,-0.2586999958112830011458527224021963775157928466796875f,0.18070002541244056804004003424779511988162994384765625f,-1.4880000253468754767283144246903248131275177001953125f,-0.7831000159919263570174052802030928432941436767578125f,0.1581999848067765501724579735309816896915435791015625f,-0.292499963734298429329783175489865243434906005859375f,1.656899982890486011655184483970515429973602294921875f,0.031899992799758525308106982265599071979522705078125f,-0.29449995326995814792780947755090892314910888671875f,0.3077999970249833960878049765597097575664520263671875f,-0.4509000105023392990233332966454327106475830078125f,-1.1611000365890564722803901531733572483062744140625f,-0.3446000041157006865688572361250407993793487548828125f,0.0555000235959888232883940872852690517902374267578125f,-1.72340003531277208281835555681027472019195556640625f,0.091700041945278254473805645829997956752777099609375f,-0.537499984316527790184636614867486059665679931640625f,-0.8893999620661141580768571657245047390460968017578125f,-1.2302000274091959841626930938218720257282257080078125f,-0.552699975702166224067468647263012826442718505859375f,1.23730004366934309700809535570442676544189453125f,0.078200036066771172471590034547261893749237060546875f,-1.233400030916929512869728569057770073413848876953125f,1.053000016778707248477076063863933086395263671875f,-0.30960003458857610070253940648399293422698974609375f,-1.363100057715178170525405221269465982913970947265625f,-1.4313000851392754242397131747566163539886474609375f,-0.359100003951787538625239903922192752361297607421875f,1.1979000200986860136254108510911464691162109375f,-0.33760000457763672532252030578092671930789947509765625f,0.81230002336651097039066371507942676544189453125f,-1.1019000230863691403015991454594768583774566650390625f,-0.44459998347759255299393998939194716513156890869140625f,-0.73989998163878911707769248096155934035778045654296875f,0.867100020805000593071554249036125838756561279296875f,-0.476499978661537593183084027259610593318939208984375f,-0.996499976202845250128348197904415428638458251953125f,-0.63329997843205898533369690994732081890106201171875f,0.3598999832034108781186887426883913576602935791015625f,-0.6980000135749582224065079572028480470180511474609375f,0.01150002701580632447075913660228252410888671875f,-1.1529000256866215323725555208511650562286376953125f,1.8392000065922733398338095867075026035308837890625f,1.513899976021050530761158370296470820903778076171875f,-0.813299974036215900241586496122181415557861328125f,-0.626400017595290847793876309879124164581298828125f,0.0531000089883804093915387056767940521240234375f,0.60289998493194563167207888909615576267242431640625f,-1.473399988336860477744494346552528440952301025390625f,0.83270004670619979236789731658063828945159912109375f,-1.468800039131939794145864652819000184535980224609375f,-0.08799997133016557171458771335892379283905029296875f,-0.556700052040815851484012455330230295658111572265625f,1.31680004454851218298472304013557732105255126953125f,-0.307200042183697785702634064364247024059295654296875f,-0.06649999594688349446869324310682713985443115234375f,-1.363499998316168859702202098560519516468048095703125f,1.2104000069856635235510111670009791851043701171875f,-0.0027999815836541319669095173594541847705841064453125f,0.549600031161308066884885192848742008209228515625f,-0.646800032702088412150942531297914683818817138671875f,-0.260400015926361305673708557151257991790771484375f,0.47249999754130822537945277872495353221893310546875f,0.060800002050399371000821702182292938232421875f,1.16999996900558489443255894002504646778106689453125f,0.11060004713535231957166615757159888744354248046875f,-0.12379997949004195589850496617145836353302001953125f,-1.47429996780753125307228401652537286281585693359375f,0.3028000153720373788246433832682669162750244140625f,-0.24650002396106618363091911305673420429229736328125f,0.767199977922439213529059998109005391597747802734375f,0.44159996610283958062836973113007843494415283203125f,-1.19390001759529074121246594586409628391265869140625f,-0.76329999489784228217104100622236728668212890625f,0.7230999735236167680341168306767940521240234375f,0.859300034117698174895849660970270633697509765625f,-0.09820001028776204066161881200969219207763671875f,-2.23639993116855606558601721189916133880615234375f,0.48330000860691058051088475622236728668212890625f,0.2772999890565870373393408954143524169921875f,-0.184600001358985910115961814881302416324615478515625f,-1.058600011797249162981415793183259665966033935546875f,-0.0957000164717438650541225797496736049652099609375f,0.844300013665854720557035761885344982147216796875f,-0.9106999965041870925830380656407214701175689697265625f,-0.219599990481138096498625600361265242099761962890625f,0.114399996599555020537763994070701301097869873046875f,-1.2355999979615208683725313676404766738414764404296875f,0.422900002366304317291678671608678996562957763671875f,1.5021999797850842384150382713414728641510009765625f,-1.5284999897480009423134106327779591083526611328125f,-0.29409998100399958875783568146289326250553131103515625f,-0.463400031661987343856168308775522746145725250244140625f,0.57290000244081029645570879438309930264949798583984375f,0.0792000076353549520291608132538385689258575439453125f,-0.71570001118183146360252067097462713718414306640625f,0.5220000104904176652098612976260483264923095703125f,0.611200013208389236751827411353588104248046875f,-0.0653999879121780480772940791212022304534912109375f,1.87790003522336501617928661289624869823455810546875f,-0.257099995622038335341130732558667659759521484375f,-0.19060003387331991842046363672125153243541717529296875f,0.0620000017732380559465354963322170078754425048828125f,0.100299989034235370599645875699934549629688262939453125f,0.65310002150535506615369740757159888744354248046875f,1.51030001950711056935006126877851784229278564453125f,-1.4770000241994853951155164395458996295928955078125f,0.5198999994829296422693687418359331786632537841796875f,-0.701200032877922208029985995381139218807220458984375f,0.7567000068902969278639147887588478624820709228515625f,-0.3260000042766331151966596735292114317417144775390625f,-0.356800009977817911277497842092998325824737548828125f,0.09540001384019891617072062217630445957183837890625f,0.20810000898838054705919375919620506465435028076171875f,-0.2964000169992448974198850919492542743682861328125f,0.1488000426709650270140627981163561344146728515625f,0.941399992859363266006766934879124164581298828125f,1.1129000059425830926329581416212022304534912109375f,-0.7138000242680302420694715692661702632904052734375f,0.530100003422796728358434847905300557613372802734375f,-0.522600008936226601008456782437860965728759765625f,-0.25399999102950143736734389676712453365325927734375f,-0.3499999357759957518965165945701301097869873046875f,-0.4783999979853632567028398625552654266357421875f,-0.261200004267692520443233661353588104248046875f,0.46290000095069416818915897238184697926044464111328125f,1.395299942952394456341380646335892379283905029296875f,-0.29439998818039914141309054684825241565704345703125f,0.265999987289309292037842169520445168018341064453125f,-0.6031000116705893532298432546667754650115966796875f,0.5258999804019932611254262155853211879730224609375f,0.0108000015661120496446301331161521375179290771484375f,-0.391299996514618442233768291771411895751953125f,-0.292599992991983637313779809119296260178089141845703125f,1.3076999790176746518710615418967790901660919189453125f,-0.170599986636638778492169876699335873126983642578125f,-1.21260000968128434095660850289277732372283935546875f,0.2532000024646521296745049767196178436279296875f,0.1787999982282519884080329575226642191410064697265625f,-1.123500009641051011755052968510426580905914306640625f,0.6276000270411372650158909891615621745586395263671875f,-0.8588999880164858158337892746203579008579254150390625f,0.879600014695525178609614158631302416324615478515625f,0.406999997675419056264445316628552973270416259765625f,-0.4956000099197030994702117823180742561817169189453125f,0.0003000171974305576583219590247608721256256103515625f,0.5822000075757503001483428306528367102146148681640625f,-0.686899998424947089148417944670654833316802978515625f,0.17939999764263614423498438554815948009490966796875f,0.5602000187665223673860737108043394982814788818359375f,-0.28550003309547899998932507514837197959423065185546875f,-0.0160999944001429806661462862393818795680999755859375f,-0.430099994184077072389982276945374906063079833984375f,0.5447000390172007655564812012016773223876953125f,0.53040002475529934855558167328126728534698486328125f,0.222700029122830045480441185645759105682373046875f,0.2015999567523587554518371689482592046260833740234375f,-0.1982999734401698077590481261722743511199951171875f,-0.22530003757476801951042943983338773250579833984375f,0.12969999997615833109421146218664944171905517578125f,0.2888000004634256878688347569550387561321258544921875f,0.3306000217288735942844368764781393110752105712890625f,-0.6969000249862673523892908633570186793804168701171875f,-0.2328000069528817217445748610771261155605316162109375f,-0.521400025083124774027965031564235687255859375f,0.1392000160545110443166549885063432157039642333984375f,1.256499984286725268844975289539434015750885009765625f,-1.1777999841913577849794592111720703542232513427734375f,0.27510002067089089905493892729282379150390625f,-0.21229999762475504354597433120943605899810791015625f,-0.80619997282624211720758466981351375579833984375f,-0.004399985647200566063474980182945728302001953125f,1.27430006734728795692035419051535427570343017578125f,-1.140400019502639139545863145031034946441650390625f,1.6482000205695630512536808964796364307403564453125f,-1.648800015699862342444248497486114501953125f,-0.2149999864399436688700006925500929355621337890625f,1.311299993199110502928306232206523418426513671875f,-0.8654000097051264273062542997649870812892913818359375f,-0.1418999939173450908214135779417119920253753662109375f,0.4514999967291950522252363953157328069210052490234375f,-0.3973000053361055439182791815255768597126007080078125f,0.1269000012189149817487532345694489777088165283203125f,-0.7577000378400089442010312268394045531749725341796875f,0.142400024330616048473530099727213382720947265625f,1.036000000625848738167178453295491635799407958984375f,-0.3046000028491022693089007589151151478290557861328125f,-1.202000032320618583270288581843487918376922607421875f,0.15050001007318469792295445586205460131168365478515625f,0.7511000322118401850701729927095584571361541748046875f,-0.178300010171532630209867420489899814128875732421875f,0.1027999902635812201623366490821354091167449951171875f,0.34920000957250607598325586877763271331787109375f,-0.1732000015705821294176303126732818782329559326171875f,0.269399989074468049210508979740552604198455810546875f,0.7905000152885917064082832439453341066837310791015625f,0.43690004018545158803732419983134604990482330322265625f,-0.351300020915270128085694523178972303867340087890625f,0.5936000314295295510191863286308944225311279296875f,-0.243300017249584499268166837282478809356689453125f,0.33530004174709393538478252594359219074249267578125f,0.30020000312030303835086897379369474947452545166015625f,-0.113200019228457904318929649889469146728515625f,0.3705000433400276182993593465653248131275177001953125f,0.71900004124641458957967188325710594654083251953125f,-0.1503999843731520513756549917161464691162109375f,1.197000016599893523761011238093487918376922607421875f,-0.7145000163018699890216112180496565997600555419921875f,0.566100022637844091377701261080801486968994140625f,-0.896100021743774899363188524148426949977874755859375f,0.7740000013113021992694484652020037174224853515625f,0.14070002254396685170689806909649632871150970458984375f,0.9618000344157220382612649700604379177093505859375f,-1.7350000372156502148612844393937848508358001708984375f,0.499800032716990028092141074012033641338348388671875f,-0.53910001482963565422323881648480892181396484375f,-0.0963999991178514648026975919492542743682861328125f,0.00600002104043984019199342583306133747100830078125f,-1.5273999981045722762473815237171947956085205078125f,0.72850001150369703140086130588315427303314208984375f,0.87199998068809492224318091757595539093017578125f,0.1651000261843209582224289988516829907894134521484375f,-1.2342999833047390012552568805404007434844970703125f,2.82099999809265167272087637684307992458343505859375f,-0.7141999770879738917983559076674282550811767578125f,-0.50229998786449403525011803139932453632354736328125f,-2.3780999690532684809340935316868126392364501953125f,1.73310002239942573254438684671185910701751708984375f,0.5255999970674505306078572175465524196624755859375f,0.5645000027418134447287911825696937739849090576171875f,-1.2360999520063398193769899080507457256317138671875f,-0.6091000206410885908780983299948275089263916015625f,-0.5668999871373170407196084852330386638641357421875f,0.1506999972864985493092859769603819586336612701416015625f,1.80800002324208630710700163035653531551361083984375f,-0.21119998482167734454861829362926073372364044189453125f,-0.5222000056013464952542335595353506505489349365234375f,0.5599000172525647744947718820185400545597076416015625f,-0.08700001350790287357739316576044075191020965576171875f,-0.17969999535679814783861729665659368038177490234375f,-0.6232999887883661216392283677123486995697021484375f,0.4148999917715787688621276174671947956085205078125f,0.306600006671249813994251098847598768770694732666015625f,0.32609999838620440382186416172771714627742767333984375f,-0.9590000201426447379304818241507746279239654541015625f,0.1488999975532293296165420315446681343019008636474609375f,1.24310001727715135899643428274430334568023681640625f,-0.29769998687803755910152858632500283420085906982421875f,-0.92610000624656674261814259807579219341278076171875f,0.9292999910533428220560381305404007434844970703125f,0.13579999530762432158326191711239516735076904296875f,-0.03800001674145481178612726580468006432056427001953125f,-1.19290001303926107567576764267869293689727783203125f,0.0866000049948691952295121154747903347015380859375f,-0.6367000190719964702878996831714175641536712646484375f,-0.286300004225969129123541279113851487636566162109375f,-0.272800006841122932854659666190855205059051513671875f,0.036700015085935344671952407225035130977630615234375f,0.34739998700320706692679095795028842985630035400390625f,-0.286200008365511848751339130103588104248046875f,0.7149000128194689107630210855859331786632537841796875f,0.52630001137852688852802884866832755506038665771484375f,0.07559999695569263877814591978676617145538330078125f,-0.23900000566989187955613260783138684928417205810546875f,-0.995500010315328420773539619403891265392303466796875f,0.30060000869035707893317521666176617145538330078125f,0.49370001168102017885530585772357881069183349609375f,-0.469599992977082880685202326276339590549468994140625f,0.4336999910987910933357625253847800195217132568359375f,0.235400031851976887509181324276141822338104248046875f,-1.6620000258386138369814943871460855007171630859375f,0.3285000067353252006796537898480892181396484375f,0.16289997785389420670298932236619293689727783203125f,0.120999984681606331804459841805510222911834716796875f,0.163700017753243276530383809586055576801300048828125f,-0.460000007823109680060724713257513940334320068359375f,-0.6704000038564206231939124336349777877330780029296875f,0.889799988907575833962937394971959292888641357421875f,0.47309999546557668370638793930993415415287017822265625f,0.05019998952299342676042215316556394100189208984375f,0.6391999907039107586825821272213943302631378173828125f,-0.852299984418600775626373433624394237995147705078125f,-0.78119999905228620917796433786861598491668701171875f,0.56739999259114259899661192321218550205230712890625f,0.15530000278055666740328888408839702606201171875f,1.177800021108984740436653737560845911502838134765625f,-0.7031999942131335057382557351957075297832489013671875f,-0.245399999739974672419151602298370562493801116943359375f,0.1722999971218406312090110077406279742717742919921875f,1.01420000800788390193929444649256765842437744140625f,0.025700004252791341929906820951146073639392852783203125f,-1.1025000176578760857637462322600185871124267578125f,0.6347000003486871211322295494028367102146148681640625f,0.6010000264421109239521001654793508350849151611328125f,-0.337499993890523819484172918237163685262203216552734375f,0.08769996284991476986903080614865757524967193603515625f,-0.9696999978899956129652082381653599441051483154296875f,-0.19630000057518515177434892393648624420166015625f,-0.6984000179529186169702370534650981426239013671875f,0.6351000254392626942490096553228795528411865234375f,-1.026400030410290042937049292959272861480712890625f,0.3954999954700468833834747783839702606201171875f,0.0373000059321526578060002066195011138916015625f,-0.7669000414706770296646709539345465600490570068359375f,0.235300003842264293663077978635556064546108245849609375f,-0.10200000143796206231172618572600185871124267578125f,-0.228900020128488534965072176419198513031005859375f,-0.558299991582333898776369096594862639904022216796875f,0.9562000228196383400103286476223729550838470458984375f,-0.49810000187307590380214605829678475856781005859375f,-0.20340000737309527067964154412038624286651611328125f,-0.08790000922083862633371609263122081756591796875f,-1.4483000064462416833777069768984802067279815673828125f,1.0799000065237278622731764698983170092105865478515625f,-0.80050001871585774182449313229881227016448974609375f,1.49550004661828328522688025259412825107574462890625f,-0.5556000375986105410675008897669613361358642578125f,-0.34610001302659532118610741235897876322269439697265625f,-0.3839999768733974860879243351519107818603515625f,0.92310000094175348461789099019370041787624359130859375f,-1.0834000074475993447009614101261831820011138916015625f,0.2047000019319356045599533899803645908832550048828125f,-0.2866999953046442950466143884113989770412445068359375f,-0.24980001483559599595452027642750181257724761962890625f,0.60280000166296954233757787733338773250579833984375f,0.21399998857080948511111273546703159809112548828125f,0.02400000771880161920535101671703159809112548828125f,-1.05199998843669906278819325962103903293609619140625f,-0.92459994607567796975899909739382565021514892578125f,-1.034599962392449779002845389186404645442962646484375f,0.78899995645880682815231921267695724964141845703125f,-0.5329000084757797850443239440210163593292236328125f,1.5951999394178404401145598967559635639190673828125f,0.6487000164866447704525853623636066913604736328125f,0.0258999926209444453206742764450609683990478515625f,0.048900002470612491389800879915128462016582489013671875f,-0.2080999876052140873383677899255417287349700927734375f,-0.42930000528395151349059233325533568859100341796875f,0.2388999954670669723100218106992542743682861328125f,-0.88279999227523830995778553187847137451171875f,0.6588999958023424863284844832378439605236053466796875f,-0.9385999847143882135469539207406342029571533203125f,1.162400031706690928245961913489736616611480712890625f,-0.5171999943882223949032095333677716553211212158203125f,-0.88990002854019412126262977835722267627716064453125f,0.238499996826052385046068593510426580905914306640625f,0.1361000032663344239125535750645212829113006591796875f,-0.709999998211860106067661035922355949878692626953125f,1.3302000228643418022755895435693673789501190185546875f,-0.60880000381618748495071713477955199778079986572265625f,-0.6219000088930128899988858393044210970401763916015625f,0.846700005288421930771391998860053718090057373046875f,0.68699999804794753544001650880090892314910888671875f,0.4162000074714422570565375281148590147495269775390625f,1.0531000099197032060516221463331021368503570556640625f,0.433800022554397202156906132586300373077392578125f,0.357799988102912802645505507825873792171478271484375f,0.68440003832280726925318958819843828678131103515625f,-0.5353000500172380160535112736397422850131988525390625f,-0.2198000088006260011042058977182023227214813232421875f,-0.81570002969652433133518343311152420938014984130859375f,0.468700026023388005569358938373625278472900390625f,0.81320002194792040395299181909649632871150970458984375f,0.2589000262379652639310734230093657970428466796875f,-0.129900036102533977810935539309866726398468017578125f,-0.1676000225335358972955646095215342938899993896484375f,-0.799900007604062324162441655062139034271240234375f,0.5169000155240295324432509005418978631496429443359375f,-0.161199987131357136416198727602022700011730194091796875f,-0.71800000351667403464261951739899814128875732421875f,0.98160000521838686182718447525985538959503173828125f,-0.52779998974204023198808499728329479694366455078125f,1.007000042378901838446836336515843868255615234375f,-1.116399929380417432867034221999347209930419921875f,2.12770000617504084772235728451050817966461181640625f,-1.18070000142156938949256073101423680782318115234375f,-0.003500000961124793974477142910473048686981201171875f,-1.35699998407810884515356519841589033603668212890625f,0.06470002426505061521311290562152862548828125f,0.6860000183582304345009106327779591083526611328125f,0.62350002640485779892287609982304275035858154296875f,-1.37530001581907157515161088667809963226318359375f,1.5116999894186857744671215186826884746551513671875f,-0.1923000119298696741765297701931558549404144287109375f,-0.7542999971255659463764686734066344797611236572265625f,-0.2133999752238395775094659256865270435810089111328125f,0.0717000021591782665808523233863525092601776123046875f,-1.43050003253668567282375079230405390262603759765625f,1.1421000209912659517641486672800965607166290283203125f,-0.1793999783083795040994345981744118034839630126953125f,-0.033900017967820217368313251427025534212589263916015625f,-1.2459000234290957820348921814002096652984619140625f,0.475999988034367493838772134040482342243194580078125f,1.391700030992925274375693334150128066539764404296875f,-0.4246000041902064925380955173750407993793487548828125f,-0.228399998804926784856661470257677137851715087890625f,0.645799987372755790460132629959844052791595458984375f,-0.33419998926967331431825414256309159100055694580078125f,0.6475000080466266627610139039461500942707061767578125f,-0.8233000307723881139310151411336846649646759033203125f,0.038700009259581502618630111101083457469940185546875f,0.6130999836564061666877023526467382907867431640625f,0.163900055742264072478064917959272861480712890625f,-0.82620003593266044816800786065869033336639404296875f,0.5558000416129831311451425790437497198581695556640625f,0.5673000164374712994685978628695011138916015625f,-0.218900004854798879705413128249347209930419921875f,0.66019999458938849468125908970250748097896575927734375f,0.6366000327855350082728591587510891258716583251953125f,1.325400007805228153046073202858678996562957763671875f,-0.2890000206083060962924946579732932150363922119140625f,-0.3138999999746676650147492182441055774688720703125f,-0.020700025039911640334366893512196838855743408203125f,-0.4284999998062850590230254965717904269695281982421875f,-0.538400027042628348539210492162965238094329833984375f,0.7518000404879454645623582109692506492137908935546875f,0.61750000953674355486100466805510222911834716796875f,0.0258999736592169238491578653338365256786346435546875f,0.6759000456690795743952548946253955364227294921875f,-1.0684000112473961419112811199738644063472747802734375f,0.0353999835163352205569253783323802053928375244140625f,-0.8062000179395079069166740737273357808589935302734375f,-0.3706999916985627141485792890307493507862091064453125f,0.5523000054478643949806837554206140339374542236328125f,-0.42609999857246894094942035735584795475006103515625f,0.71720002288669348899929900653660297393798828125f,-0.440900003163516629456353257410228252410888671875f,0.847899987837672508561581707908771932125091552734375f,-0.37509999548792816170816877274774014949798583984375f,-0.185300020363926787325681289075873792171478271484375f,0.321099980169533782969892854453064501285552978515625f,-2.21470000630915198058801252045668661594390869140625f,0.808600043722985883931642092647962272167205810546875f,1.64959998750090530705847413628362119197845458984375f,0.966499976873397059762282879091799259185791015625f,0.039699988055229340488949674181640148162841796875f,-1.749999952316283469144764239899814128875732421875f,1.1610999934315682224905685870908200740814208984375f,2.367099985880404044991109913098625838756561279296875f,-0.31689998710006406223982367009739391505718231201171875f,-0.36160000438019590518479162710718810558319091796875f,1.07359997081905600424533986370079219341278076171875f,2.27490001606047176352376482100225985050201416015625f,0.4869999812841410147257192875258624553680419921875f,0.7963000188291069481039130550925619900226593017578125f,-1.9181000718712812869171102647669613361358642578125f,0.728000081151725542127906010136939585208892822265625f,1.0243999564409254077190780662931501865386962890625f,-0.4673000754088167951749710482545197010040283203125f,-1.66519999902248372336543980054557323455810546875f,-1.386699984687566722385554385255090892314910888671875f,-0.967000016897916481184438453055918216705322265625f,-0.40759999921321832250242778172832913696765899658203125f,0.7313000306010248863231026916764676570892333984375f,0.15200001429021359200532970135100185871124267578125f,-0.9578000423431394239059954998083412647247314453125f,0.6058999808490279637140929480665363371372222900390625f,0.337000000860542048286561112035997211933135986328125f,0.83229998758509726730636657521245069801807403564453125f,-1.6577000312089911204793679644353687763214111328125f,0.4494999519288540579253776741097681224346160888671875f,0.191900036311150046230977750383317470550537109375f,-0.299200027900934362179441450280137360095977783203125f,-1.288199959921836779130899230949580669403076171875f,-1.096900002411008046010465477593243122100830078125f,0.7449000165820134355953996418975293636322021484375f,0.459899986481667522042471318854950368404388427734375f,1.070300023567675840041601986740715801715850830078125f,-0.98709998899102247005288290893076919019222259521484375f,-0.435199991571902700115970219485461711883544921875f,-0.40689999518394426303302680025808513164520263671875f,0.481099991792440118132390125538222491741180419921875f,-0.66259996300339718544591960380785167217254638671875f,-0.54440002435296774851991585819632746279239654541015625f,1.872000010490418642206122967763803899288177490234375f,0.245200026310235130733161668104003183543682098388671875f,-0.61609997569918650839326801360584795475006103515625f,1.27130001080110677236234550946392118930816650390625f,-0.7865000150203709239349336712621152400970458984375f,-1.368300035261362790350858631427399814128875732421875f,1.5614000326454646749851917775231413543224334716796875f,1.5659000722117728532367664229241199791431427001953125f,-0.229299989861250441691709056613035500049591064453125f,0.8522999897085128395701758563518524169921875f,1.3137000664427882501428257455700077116489410400390625f,1.7083000604629521301802697053062729537487030029296875f,-1.20110002561807593934872784302569925785064697265625f,0.308900013311207255473078703289502300322055816650390625f,0.05179999984502803300756568205542862415313720703125f,-0.2989999999701975497146122506819665431976318359375f,1.63250004410743709826192571199499070644378662109375f,0.59329996010362950897842893027700483798980712890625f,-0.45470001070499410644742965814657509326934814453125f,0.9668000353097918608824556940817274153232574462890625f,-0.2568999808788294814121400122530758380889892578125f,-1.50450000616908052819553631707094609737396240234375f,1.5706000206112857853213427006267011165618896484375f,2.7646000733315947428536674124188721179962158203125f,-0.715700046497584008164949409547261893749237060546875f,0.72540000292509798640594453900121152400970458984375f,0.255599964173137461642681955709122121334075927734375f,1.280800055582821617150557358399964869022369384765625f,-1.620100040116906559006793031585402786731719970703125f,-0.338899985483288812559976577176712453365325927734375f,0.808400022423267472504448960535228252410888671875f,0.065200005280971584653570971568115055561065673828125f,-0.5035000021159650174240596243180334568023681640625f,0.55190000732839106234450810006819665431976318359375f,0.848199974226950370592703620786778628826141357421875f,0.7341000298798079359841040059109218418598175048828125f,-0.8896000474035741234501983853988349437713623046875f,0.4812999787449836475161646376363933086395263671875f,0.177000035524368826145291677676141262054443359375f,-0.1186999837785958078484327415935695171356201171875f,-1.0686000479698185561261425391421653330326080322265625f,-0.2563000307500369334690049072378315031528472900390625f,-0.48289995490610515549434467175160534679889678955078125f,-0.1694000116497276497984358911708113737404346466064453125f,0.4925999741047615287925509619526565074920654296875f,0.1643999816983934270098188790143467485904693603515625f,-0.5143999954819680198170317453332245349884033203125f,0.2624000383377076417446005507372319698333740234375f,0.03430001176595709466710104607045650482177734375f,-0.3049999354779711069340919493697583675384521484375f,1.9488000529900195711974220103002153337001800537109375f,-0.34540000087618860646898610866628587245941162109375f,0.19029999873861680459441458879155106842517852783203125f,-1.32110000609755484646257173153571784496307373046875f,-0.6252000153765084888846104149706661701202392578125f,0.006399997329712192595252417959272861480712890625f,-1.5917000188857315379209467209875583648681640625f,0.7571000246047976389718314749188721179962158203125f,0.8816000451721250907866078705410473048686981201171875f,-0.59640002646148193576891571865417063236236572265625f,0.016799991239607070525607923627831041812896728515625f,-1.1506999811932445698658966648508794605731964111328125f,0.1424999547377223674260449115536175668239593505859375f,1.026500001478940049537413869984447956085205078125f,-0.61760001703873268485978087483090348541736602783203125f,1.391400018191337295547782559879124164581298828125f,1.352699991720914152182331235962919890880584716796875f,-0.279600045764447191487533928011544048786163330078125f,-0.434499964147805339820251901983283460140228271484375f,0.78300003615021740444035458494909107685089111328125f,-1.44680004909336634710825819638557732105255126953125f,1.2294000437617302878834379953332245349884033203125f,-0.723800030899047985855077058658935129642486572265625f,1.30820004843473469691161881200969219207763671875f,1.04960004487037661391468645888380706310272216796875f,-1.4602999711155888462599250487983226776123046875f,-0.2148999676316984963619916015886701643466949462890625f,1.1484000069260595910947131415014155209064483642578125f,-0.0545000591240840659423838587827049195766448974609375f,-0.20920001318603753670544165288447402417659759521484375f,-0.31269998887106786877865260976250283420085906982421875f,-1.05590001240223596568057473632507026195526123046875f,0.310799985361099206926382976234890520572662353515625f,-0.65950001406669667147752988967113196849822998046875f,1.53289999998211801113257024553604423999786376953125f,-0.146000008672475534154955312260426580905914306640625f,0.046100008258223912349649253883399069309234619140625f,0.182399997657537227269131108187139034271240234375f,1.231600041204691020624295560992322862148284912109375f,-0.0601000147104269188957914593629539012908935546875f,0.26450001883506768507459128159098327159881591796875f,-0.00310006471872359412600417272187769412994384765625f,-0.0404000124990939735170059066149406135082244873046875f,-0.642699959236383566718586735078133642673492431640625f,-0.525499957323074529114137476426549255847930908203125f,-0.3171999540060761457738180979504249989986419677734375f,0.311300009366869689930723552606650628149509429931640625f,0.915800005459039834221357523347251117229461669921875f,-0.88430001100227262345043754976359196007251739501953125f,0.23250001728534730460751234204508364200592041015625f,0.46230000723600372936061830841936171054840087890625f,0.4474000075668096432224274394684471189975738525390625f,-0.39110001131296190379771360312588512897491455078125f,0.864399993675201816500930362963117659091949462890625f,-0.023099966669082760972742107696831226348876953125f,0.17099997749179607353653409518301486968994140625f,0.6310000162720683647421537898480892181396484375f,1.2092000178612767324892729448038153350353240966796875f,-1.311300008547306106265750713646411895751953125f,0.3853999747060239400298087275587022304534912109375f,1.494600009852647293229210845311172306537628173828125f,0.82229997301921209373887222682242281734943389892578125f,0.1224000298440461875770779442973434925079345703125f,1.290200025453418941623340288060717284679412841796875f,-0.5717000137448309260435053147375583648681640625f,1.015500013910233700897833841736428439617156982421875f,0.2044999857544897992056576185859739780426025390625f,-0.2463000175625082999175674558500759303569793701171875f,-0.65120001857280751522694117738865315914154052734375f,1.0477000195860857179042113784817047417163848876953125f,0.620000015795231007587062777020037174224853515625f,-0.216500004366040332826059966464526951313018798828125f,1.074999991953372813213718472979962825775146484375f,0.53509998848438211638267603120766580104827880859375f,-0.03299998384714086796520859934389591217041015625f,0.90129998910129049960460179136134684085845947265625f,2.2552000060260297686909325420856475830078125f,0.37529994824230661354391713757649995386600494384765625f,-0.18659998592138204998036599135957658290863037109375f,1.49910002444088519268916570581495761871337890625f,1.775000046193600411470470135100185871124267578125f,0.5316999881893398338661427260376513004302978515625f,0.38600004100799623074635746888816356658935546875f,-0.2942000091999770194917118715238757431507110595703125f,0.8314999944567684764251680462621152400970458984375f,0.4678999873161318046044243601500056684017181396484375f,-1.2942999971628186273164828890003263950347900390625f,-0.153999994382262084258172762929461896419525146484375f,-0.0020999679803850312964641489088535308837890625f,0.39370000933408721977002642233856022357940673828125f,0.223299943935870714994962327182292938232421875f,1.35420000592172140585489614750258624553680419921875f,-2.131099960947036908720519932103343307971954345703125f,0.16540002002418041637810119937057606875896453857421875f,0.877999985337257538020594438421539962291717529296875f,0.767099992790818863142021655221469700336456298828125f,-1.39910005163550454909682230209000408649444580078125f,1.1458000173240900920745843905024230480194091796875f,-0.467600049579143473010844900272786617279052734375f,1.0748000120043759153531937045045197010040283203125f,0.2329000016212461332543171010911464691162109375f,-0.07230000306367845297472740639932453632354736328125f,-0.19529998678043458681941046961583197116851806640625f,0.06900000190734889571331223123706877231597900390625f,1.700000042114407250437579932622611522674560546875f,-0.07370000337362370146365719847381114959716796875f,0.10759997933879505982446289635845459997653961181640625f,-0.99719996838569624486581233213655650615692138671875f,0.684200027081370176773589264485053718090057373046875f,-0.3078000461429351020115063874982297420501708984375f,-1.29349999140575544487319348263554275035858154296875f,-0.7776000070363282556940021095215342938899993896484375f,-0.89130000759735683590889721017447300255298614501953125f,1.50499996826052662157735539949499070644378662109375f,0.1015999954767521362697380027384497225284576416015625f,0.47230001885890970925174769945442676544189453125f,0.83280000203549864323804285959340631961822509765625f,-0.713299986553192155724900658242404460906982421875f,0.2325000550597906556760108287562616169452667236328125f,0.67109998717308005922177471802569925785064697265625f,1.3364000380098823850261169354780577123165130615234375f,-1.2633000670194629577736122882924973964691162109375f,-0.154399993246792543999390545650385320186614990234375f,1.00489998171329464327072855667211115360260009765625f,-0.091200001138448616444520666846074163913726806640625f,-1.1636999613523482111077100853435695171356201171875f,-1.441599980407954006977888639084994792938232421875f,-0.5274999913573263921051648139837197959423065185546875f,-0.3080999907344577071199864803929813206195831298828125f,0.217899999013543066439524409361183643341064453125f,1.097299991142749231443076496361754834651947021484375f,-0.6289000041842456312224385328590869903564453125f,0.34589999336600296686583533301018178462982177734375f,-0.9414000116348262281462666578590869903564453125f,-0.0436000052779912383726923508220352232456207275390625f,-0.646099975848198226913154940120875835418701171875f,-0.8141999855816368647509762013214640319347381591796875f,-0.43490000257492056334740482270717620849609375f,-0.177800024312734894493814863380976021289825439453125f,-0.2066999511599529881777925766073167324066162109375f,-0.350800012928246918164632006664760410785675048828125f,0.2118999543547630537432269193232059478759765625f,0.828299997654557529358498868532478809356689453125f,0.718000024229287436838831126806326210498809814453125f,-1.057899954235554051962253652163781225681304931640625f,-0.5076999682515861422871239483356475830078125f,-0.822399995571374997638258719234727323055267333984375f,0.0206000213563442002850933931767940521240234375f,-1.1096999907374378580016127671115100383758544921875f,0.618299977016448565336759202182292938232421875f,0.47209998943805686621999484486877918243408203125f,0.098999976873397343979377183131873607635498046875f,-0.852199958550929892453495995141565799713134765625f,0.3957999760851260706573384595685638487339019775390625f,0.60550001722574275930810472345910966396331787109375f,-0.2484999778270715609096441767178475856781005859375f,0.41889997617006269337025514687411487102508544921875f,-1.38920003471821562612831257865764200687408447265625f,-0.7964999911785124453444950631819665431976318359375f,-0.234299974196404026116624663700349628925323486328125f,0.0876999847173696167601519846357405185699462890625f,-0.3838999642677605184104550062329508364200592041015625f,-0.48240000823736206569947171374224126338958740234375f,-0.0533999991774560811563787865452468395233154296875f,-0.6068999418377867272056391811929643154144287109375f,2.16439370870347147501888684928417205810546875e-08f,-1.372000043570995586605931748636066913604736328125f,-0.0486000150941314768004986035521142184734344482421875f,0.903000021696090726663896930404007434844970703125f,0.612099966937301953606720417155884206295013427734375f,0.41669999496936771521404807572253048419952392578125f,-0.083099971064924726960043699364177882671356201171875f,-1.8622000622630121569045513751916587352752685546875f,-1.553000069975853048021008362411521375179290771484375f,0.2478000003218649993641520268283784389495849609375f,-0.295299968880414098748588003218173980712890625f,-0.68959993072748115849890382378362119197845458984375f,-0.08929994150698217225681219133548438549041748046875f,-0.39940000323057223141631766338832676410675048828125f,-1.31559996450841421022914801142178475856781005859375f,0.197100031757354798855885746888816356658935546875f,-0.5258000043600803419252542880713008344173431396484375f,1.3194999720454205771602573804557323455810546875f,-0.03479999613463835572702009812928736209869384765625f,1.688700087118149895104579627513885498046875f,0.218400004392862445712353292037732899188995361328125f,-0.77219999312162457272279425524175167083740234375f,0.826600002536177047574028620147146284580230712890625f,0.871000000774859728380761225707828998565673828125f,-0.769200029838085885813825370860286056995391845703125f,1.21600005143880895985830647987313568592071533203125f,-0.16819998450875228712675379938445985317230224609375f,-0.0180000887513163121411707834340631961822509765625f,0.4358999864369632604166326927952468395233154296875f,-0.21879995967149756808112215367145836353302001953125f,-1.47489996509850040951050686999224126338958740234375f,0.79329996688365955748167834826745092868804931640625f,0.4939000121563672518476550976629368960857391357421875f,-0.39589995685815893011749722063541412353515625f,0.84250003270804807442573292064480483531951904296875f,-1.29450003969669324987989966757595539093017578125f,0.43480002422332830747109255753457546234130859375f,0.37400001483410616476277255060267634689807891845703125f,-0.0338999992296098451305397247779183089733123779296875f,1.212000009708106507133607010473497211933135986328125f,0.515800002348422825804163949214853346347808837890625f,0.95009999520853138932352521806024014949798583984375f,-0.186599991285800737017552819452248513698577880859375f,-0.981199986386298572682562735280953347682952880859375f,1.48949997487664287376674110419116914272308349609375f,0.1090000509135429229701230724458582699298858642578125f,0.21599998300522482708174720755778253078460693359375f,0.575099971571563894912060277420096099376678466796875f,0.47720005782991725684638595339492894709110260009765625f,0.04500003151595655115357885733828879892826080322265625f,-0.39140002027750053770205340697430074214935302734375f,0.127600003031640929496148828548029996454715728759765625f,-0.4453000003218647862013312987983226776123046875f,0.15650000187009582841568544608890078961849212646484375f,0.2409999939948319624960504370392300188541412353515625f,2.14180001225024430056009805412031710147857666015625f,1.13239998745024195159203372895717620849609375f,0.200099971497059048175515272305347025394439697265625f,-1.33820001609921401808378504938445985317230224609375f,-1.018599996039270205727689244667999446392059326171875f,0.00240000257492045676599445869214832782745361328125f,1.57710002341270527637107079499401152133941650390625f,0.38559998365640613116056556464172899723052978515625f,0.520699956755339865566156731802038848400115966796875f,-0.00229996849298430561248096637427806854248046875f,-0.51829999813884508608197165813180617988109588623046875f,-1.552300057937205490787846429157070815563201904296875f,0.6877000411182641048668529037968255579471588134765625f,0.2434000304698944461279097595252096652984619140625f,-0.4878999817281963746751216604025103151798248291015625f,-0.150800016355514543420213158242404460906982421875f,0.08959999435544008239418189987190999090671539306640625f,-1.94259997466355560646888989140279591083526611328125f,-0.2255999924108385012999633545405231416225433349609375f,1.0035999797224999241507248370908200740814208984375f,0.52749998219311267089182138079195283353328704833984375f,0.691999977558851231407288651098497211933135986328125f,0.228999986112117692726997120189480483531951904296875f,-0.76030000881552695091158966533839702606201171875f,-0.677199997368454820190208920394070446491241455078125f,0.1116999713510284930606530906516127288341522216796875f,0.6656999839127071272315561145660467445850372314453125f,1.2953999886199822899612854598672129213809967041015625f,0.52479998197853472419183162855915725231170654296875f,-1.18189998445510813240844072424806654453277587890625f,-1.0607999915450807559835766369360499083995819091796875f,0.275000025033950379338421043939888477325439453125f,-0.1945999799013140574288627249188721179962158203125f,-0.44829997627139039906296602566726505756378173828125f,-0.420400081044435491861577247618697583675384521484375f,-1.270699994958192302618726898799650371074676513671875f,-0.55670003460645745008150697685778141021728515625f,-1.004700047864765632965600161696784198284149169921875f,0.53279997171163540059524166281335055828094482421875f,-1.4206000463344157225265007582493126392364501953125f,0.88950002390146210018428973853588104248046875f,1.8305000064596532727279054597602225840091705322265625f,-0.1483999786138536780555341465515084564685821533203125f,1.29660000033825628662498274934478104114532470703125f,-0.70519999775588504409284951179870404303073883056640625f,0.9042999981500206008178111005690880119800567626953125f,0.2063999798953535247392210294492542743682861328125f,0.31720000686794536104429198530851863324642181396484375f,0.61899999937415106643356921267695724964141845703125f,0.24640000127851957589797393666231073439121246337890625f,-0.993400005733966917631505566532723605632781982421875f,-0.882499991953372475705918986932374536991119384765625f,-0.3938999804913994040589386713691055774688720703125f,-1.7538000601053245208760245077428407967090606689453125f,-1.02929999746084188672057280200533568859100341796875f,-0.405900022870302468191994194057770073413848876953125f,1.73250000149011640360185992904007434844970703125f,2.3674000051081183215728742652572691440582275390625f,-0.203200004029273628702867426909506320953369140625f,0.606700012329220772500093517010100185871124267578125f,-0.223000018268823652078935992904007434844970703125f,0.923400010874866783439074424677528440952301025390625f,-0.0824999926611778011675824018311686813831329345703125f,0.429000029698014540002759531489573419094085693359375f,-0.762800016713141904034500839770771563053131103515625f,-0.01460001193881022008014269886189140379428863525390625f,-1.0000000241398812050874767010100185871124267578125f,0.12219999625086774841520309564657509326934814453125f,0.09430001489520112301079279859550297260284423828125f,-1.3679999808669085492596195763326250016689300537109375f,1.2390000340938573941684808232821524143218994140625f,-0.292899999088048534900963204563595354557037353515625f,-0.3353999610900881833686071331612765789031982421875f,-0.591899986690282986501188133843243122100830078125f,-1.0469000431656834138038902892731130123138427734375f,0.487499959021806983372471222537569701671600341796875f,-0.2138999836206434412844146208954043686389923095703125f,0.1864999959468842671839183822157792747020721435546875f,-0.991199974092841440409529241151176393032073974609375f,-0.36369998012781135088289374834857881069183349609375f,0.052200007016956373462335250223986804485321044921875f,-0.82059999673217542959235970556619577109813690185546875f,0.778600005948543838485420565120875835418701171875f,-0.12940001366138442762121485429815948009490966796875f,0.04410002496242526603964506648480892181396484375f,-0.90449999644607326221290577450417913496494293212890625f,0.5261999722301953230640947367646731436252593994140625f,-0.69310002862065955842041375944972969591617584228515625f,1.0930000105947252908578093411051668226718902587890625f,-1.0562000263214113005005856393836438655853271484375f,-0.352200001801550310887023442774079740047454833984375f,-0.35689997283220264989722636528313159942626953125f,1.925500017300248334350953882676549255847930908203125f,-1.2371000117897974490688284276984632015228271484375f,1.7544000537082553581313959512044675648212432861328125f,-0.2632000183343894406107210670597851276397705078125f,0.280700002576411034027614732622168958187103271484375f,1.247400064861775348390438011847436428070068359375f,0.127299979259073214876707424991764128208160400390625f,0.23010001232624066602738821529783308506011962890625f,-0.2118000012978911161809492114116437733173370361328125f,0.880799997878074947266213712282478809356689453125f,-1.157799952545017330152177237323485314846038818359375f,-0.03900002608448238650140638128505088388919830322265625f,-0.05670001093223697719025722108199261128902435302734375f,1.766899942899495812298482633195817470550537109375f,-0.7883999987304211032324019470252096652984619140625f,-0.3358000011563309072926131193526089191436767578125f,0.18160003107190103577295303693972527980804443359375f,0.0239999756813045195258382591418921947479248046875f,-0.55289998105764315283749965601600706577301025390625f,0.481699931192398622670225449837744235992431640625f,1.536100028896331792793716886080801486968994140625f,0.97779996962547244265806511975824832916259765625f,0.281600007230043303252386976964771747589111328125f,-0.9115000033974638427025638520717620849609375f,0.2714000147640707183427366544492542743682861328125f,-0.720899987107515016759862191975116729736328125f,-0.056099998199939715703976617078296840190887451171875f,1.795000027976929590778354395297355949878692626953125f,-0.7603000029295678974250449755345471203327178955078125f,-0.8449000131919977008010391728021204471588134765625f,0.437399986109137106637945180409587919712066650390625f,-0.5616000014558435449174567111185751855373382568359375f,-0.581599995830654581396856883657164871692657470703125f,1.6671000336572527888989725397550500929355621337890625f,-0.282200005117058783099537322414107620716094970703125f,-1.084800039757788159278106832061894237995147705078125f,0.088300007861852325419249609694816172122955322265625f,0.7752000196233392870226452942006289958953857421875f,-0.6763000180840494923728556386777199804782867431640625f,-1.766800043691695076830683319712989032268524169921875f,0.201599983388185410859705370967276394367218017578125f,0.75090003136396443750300022657029330730438232421875f,-0.8086000405192379503205302171409130096435546875f,0.632399996912479167576748295687139034271240234375f,-0.72380005563497551435148125165142118930816650390625f,-0.683499998502432593028288465575315058231353759765625f,0.3221000010609618158241573837585747241973876953125f,-0.36860000561326788925953223952092230319976806640625f,-0.08849997663497877198324204073287546634674072265625f,0.1238999557927245032118435119627974927425384521484375f,-0.2379000501990322646861386601813137531280517578125f,-0.0442999763384455125247995965764857828617095947265625f,-0.562900007134676361175706915673799812793731689453125f,0.049200010503828384145208474365063011646270751953125f,-1.559900061210990340754278804524801671504974365234375f,1.27560004840195251318846203503198921680450439453125f,-0.2889999998956918059178633484407328069210052490234375f,1.49670004716068572037102057947777211666107177734375f,-0.54120003913641046011662183445878326892852783203125f,0.6697000135362147776874053306528367102146148681640625f,-0.19179998375177387259782335604541003704071044921875f,0.326599920654297104505303650512360036373138427734375f,1.15109998821616255071376144769601523876190185546875f,-1.37499997794628203706679414608515799045562744140625f,0.43660004373788741105499866534955799579620361328125f,0.601300011527538202926734811626374721527099609375f,-1.24630003134608191572851865203119814395904541015625f,0.56929995707869540666479224455542862415313720703125f,1.199300015044212130987943965010344982147216796875f,-2.068299998548627005590105909504927694797515869140625f,-0.782999980717898669269061429076828062534332275390625f,-0.3421000130563973673503141981200315058231353759765625f,1.968599968695639912397155057988129556179046630859375f,-1.0117000181034203620811240398325026035308837890625f,0.41049998299032441906319945701397955417633056640625f,1.00510003229379663025611080229282379150390625f,0.367899992680548848511534743010997772216796875f,-0.250100007408857294421977712772786617279052734375f,-0.5429000374957919650142912360024638473987579345703125f,-1.011900007030368531246722341165877878665924072265625f,-0.049799997475743484898202950716949999332427978515625f,-0.37640001103877995802804434788413345813751220703125f,1.0636000296413901100578414116171188652515411376953125f,-0.718900019532442424718965412466786801815032958984375f,1.167800003711878975565241489675827324390411376953125f,0.15760001985132721102900177356787025928497314453125f,-1.678600006768107277821400202810764312744140625f,0.504799992185831047919464253936894237995147705078125f,0.5931000246718529655964857738581486046314239501953125f,-0.479300015789270561494817002312629483640193939208984375f,0.71350002046301963520846811661613173782825469970703125f,-0.43779998752549265095268538061645813286304473876953125f,-0.82560001606643240723570897898753173649311065673828125f,1.0660000207237907421387035356019623577594757080078125f,0.5324000431060795790472184307873249053955078125f,-0.462400009429455138842968153767287731170654296875f,0.06610001120120312378958260524086654186248779296875f,0.069500005573034595585113493143580853939056396484375f,1.1374000263050201287029494778835214674472808837890625f,0.33739997783899244865324362763203680515289306640625f,-0.7515000153183937214862453402020037174224853515625f,0.5884000030517579915567694115452468395233154296875f,-2.00550002717226760751145775429904460906982421875f,0.633900025641917608254516380839049816131591796875f,-0.1694000151515007157598802223219536244869232177734375f,-0.471600018852949887104841764084994792938232421875f,1.2523000057086346448187441637855954468250274658203125f,-0.47689999641329039459236582843004725873470306396484375f,-0.62470005987882648579301303470856510102748870849609375f,0.54930004145652133740185263377497904002666473388671875f,0.15109997428357557769373897826881147921085357666015625f,0.8802000252485271403202204965054988861083984375f,0.5918000199615953693665915125166065990924835205078125f,-0.3574999797344222685069325962103903293609619140625f,0.31420000808239034739699491183273494243621826171875f,-0.29190000650882641508587767020799219608306884765625f,1.767200006085634100827519432641565799713134765625f,0.08750002786517097064233894343487918376922607421875f,-0.0231000395357607857249604421667754650115966796875f,-0.41449997055530474199258605949580669403076171875f,-0.4635999993920325579210839350707828998565673828125f,-1.56919997112751019585630274377763271331787109375f,0.06660003286004012590382217240403406322002410888671875f,0.11510002603530811171594905317761003971099853515625f,1.10649999380484143074454550514928996562957763671875f,0.45209997718185224169218372480827383697032928466796875f,-0.86290002093687678819122766071814112365245819091796875f,0.04159997483864563694311300423578359186649322509765625f,-0.112700000065565131279754496063105762004852294921875f,-0.77450001015514124969030262946034781634807586669921875f,0.1061999895900485046951189360697753727436065673828125f,-0.01689998706281148344032772001810371875762939453125f,-0.58339999589920044087421047152020037174224853515625f,0.720899992434679948161146967322565615177154541015625f,-0.60839999850690329452618243522010743618011474609375f,-0.989800001685321007016682415269315242767333984375f,-0.4428000172346830964897890225984156131744384765625f,1.7663999955788252105293167915078811347484588623046875f,-1.3762999810546634460450832193600945174694061279296875f,-0.7783000015661121206989037091261707246303558349609375f,-0.036900012245773883279298388515599071979522705078125f,0.6678999909669156576086379573098383843898773193359375f,-1.330100018435716346942854215740226209163665771484375f,0.406300040435791487425376544706523418426513671875f,-0.947000046551227825375462998636066913604736328125f,0.10280001373291014488131622783839702606201171875f,0.1922000011682509779120664461515843868255615234375f,0.70400003092736029852716228560893796384334564208984375f,-0.39049998772144345338119819643907248973846435546875f,0.50159997988641225052930394667782820761203765869140625f,-0.67720001785755101764152641408145427703857421875f,0.9474000248149041514267310049035586416721343994140625f,0.258500010907649624414261779747903347015380859375f,0.470099983045458191810439529945142567157745361328125f,-1.116899997568129521141599980182945728302001953125f,-0.184200016352534579056055008550174534320831298828125f,-0.2256999846577638635380935738794505596160888671875f,1.17260002175122490797321006539277732372283935546875f,-0.95450001883506718769467624952085316181182861328125f,0.2895000125765800902399860206060111522674560546875f,-0.699600043678284322368199354968965053558349609375f,0.13309999617338152955170471614110283553600311279296875f,0.1183999821901320359529563575051724910736083984375f,0.051400012677908168967633173451758921146392822265625f,0.469599999570846193819306790828704833984375f,-0.1331999563455585899873767630197107791900634765625f,-1.0363999818325044799394163419492542743682861328125f,-0.1186000266611575426622948725707828998565673828125f,1.325799975228310501051964820362627506256103515625f,-0.04629995922446283884710283018648624420166015625f,-0.38779998750686672792653553187847137451171875f,0.298700031834840729061397723853588104248046875f,-0.327100027883052035804212209768593311309814453125f,-1.0732999768674371665611033677123486995697021484375f,1.27599996158480788466249578050337731838226318359375f,-0.5623999913990491705817476031370460987091064453125f,0.06189996493458860271630328497849404811859130859375f,0.6189000298142435241288694669492542743682861328125f,0.5848999659180638133193497196771204471588134765625f,0.116699996235966718671761555015109479427337646484375f,0.41119999275654539605540094271418638527393341064453125f,-0.0453999955862760540270528508699499070644378662109375f,-1.1579000295236705486701112022274173796176910400390625f,-0.186400006791949568452082530711777508258819580078125f,0.66319998897910081847584251590888015925884246826171875f,-0.4437999984145164500404234786401502788066864013671875f,-0.3373999719902871508026009905734099447727203369140625f,-0.282300035473704635791136752231977880001068115234375f,-0.3945999907419082131099230537074618041515350341796875f,0.9178999910414216145682075875811278820037841796875f,0.7743999913468953000261763008893467485904693603515625f,0.1977000198841096789692528545856475830078125f,-0.43139999211430524184152091038413345813751220703125f,0.68489999139904966796166263520717620849609375f,-0.52259999325275430237525142729282379150390625f,-0.04930002040863001866455306299030780792236328125f,0.1106999837636948313956963829696178436279296875f,-0.335300011646747719851191504858434200286865234375f,0.505400013840198614190057924133725464344024658203125f,-0.53049998891353578045482208835892379283905029296875f,1.243100011372565649736543491599150002002716064453125f,-0.8125000175833694271432250388897955417633056640625f,1.513800074857474253775535544264130294322967529296875f,0.06759997268915096668706610216759145259857177734375f,-0.5423000079438085041516615092405118048191070556640625f,1.0601000235021107442889842786826193332672119140625f,-0.0442000172093515164561949859489686787128448486328125f,0.0227000110179191860737546448945067822933197021484375f,-0.8383000091284511512412791489623486995697021484375f,-0.48629999692440062375453635468147695064544677734375f,0.5208000192239878867184188493411056697368621826171875f,-0.43260001232624034628315712325274944305419921875f,0.643199989537894367686021723784506320953369140625f,-0.591399987196922438670299015939235687255859375f,-0.522000008255243397314870890113525092601776123046875f,-0.1346999847769740910052860272116959095001220703125f,-0.17649994990229611602217119070701301097869873046875f,-0.2632000368118294630903619690798223018646240234375f,-0.376600003504752312011305548367090523242950439453125f,1.22659996953010619336055242456495761871337890625f,1.52979997400641476446025990298949182033538818359375f,-0.118300009799003902344338712282478809356689453125f,0.49790002561211543774533083706046454608440399169921875f,-0.89919995473623348658520626486279070377349853515625f,0.3485999765187504095820258953608572483062744140625f,-0.4496000095546239805344157502986490726470947265625f,-1.99349997350573548970942283631302416324615478515625f,0.67409999679923071624898511799983680248260498046875f,-0.1495000063180924598782439716160297393798828125f,1.201899940943718547714524902403354644775390625f,-0.1686000013291835131212792475707828998565673828125f,1.0983000252962114018373540602624416351318359375f,-0.7991999912440774789246233922312967479228973388671875f,-1.0511000426799064211280665404046885669231414794921875f,1.053000022143125935514262891956605017185211181640625f,0.20909999065250151062400618684478104114532470703125f,0.6178000090390440846022102050483226776123046875f,-0.27769997603744267511416410343372263014316558837890625f,0.1523999877482655751492757190135307610034942626953125f,-0.72860001965761178599478853357140906155109405517578125f,0.6970999991238111537228405722999013960361480712890625f,-0.263699959936737560184383255545981228351593017578125f,-0.193800032687187151481111868633888661861419677734375f,0.081199927489458811891154255135916173458099365234375f,-0.59859996608793775152435046038590371608734130859375f,1.179600048148632396305401925928890705108642578125f,-0.97070002071857430792078957892954349517822265625f,0.4494000035285949934404925443232059478759765625f,-0.78640003190040630443036206997931003570556640625f,-1.120000063627958564183018097537569701671600341796875f,0.49540003231763929392172940424643456935882568359375f,-0.189900016433000917714934985269792377948760986328125f,0.66780001850128201112966053187847137451171875f,-0.112299956423043578723763857851736247539520263671875f,0.8293000010371205377168735140003263950347900390625f,-1.5705000489279630215833094553090631961822509765625f,0.54190002498626643756551857222802937030792236328125f,0.664100037144125021626450688927434384822845458984375f,0.43159997042417419521598276332952082157135009765625f,1.030299994026124377199948867200873792171478271484375f,0.6474000431060797922100391588173806667327880859375f,-0.0705999734118574195917972247116267681121826171875f,-0.218100011670589566392663982696831226348876953125f,-0.510300032955408600088276216411031782627105712890625f,0.53320003561973550176844582892954349517822265625f,0.667999963283537834257685972261242568492889404296875f,0.566799992394447116339506465010344982147216796875f,-0.32010002901554113208248963928781449794769287109375f,-0.106300036859511948250656132586300373077392578125f,-0.0422999954640861375310123548842966556549072265625f,0.6518000281572344789537964970804750919342041015625f,1.228099964135885624472166455234400928020477294921875f,-1.4533000059247012103469387511722743511199951171875f,-0.427599978351592913128342843265272676944732666015625f,-1.3793999296188363956616740324534475803375244140625f,1.0198999981045719920302872196771204471588134765625f,0.2146000177502624950420795357786118984222412109375f,0.8372000042974949707286214106716215610504150390625f,-1.3771999305367472743455437012016773223876953125f,0.97120003466606164010954671539366245269775390625f,-0.86770001511573724428671994246542453765869140625f,-0.0800000012293458073742158376262523233890533447265625f,0.944499995701014949389673347468487918376922607421875f,-0.9894000033423306916802175692282617092132568359375f,-0.57279998005628574020420273882336914539337158203125f,-0.0126000349760053698133788202540017664432525634765625f,0.471299991001188356420925629208795726299285888671875f,0.27669999966323377549315409851260483264923095703125f,-0.10070003405511374072744956720271147787570953369140625f,0.3197999996364109254187724218354560434818267822265625f,0.5950000704824931485603656255989335477352142333984375f,-0.47570003781765712869145090735401026904582977294921875f,-1.157600050771236954716414402355439960956573486328125f,0.6149000163212419156621990623534657061100006103515625f,-0.1516999913185832493667248854762874543666839599609375f,0.03890000776052460196297033689916133880615234375f,0.0781000057101248756907807546667754650115966796875f,-0.1464000187873839564645095379091799259185791015625f,0.55820001989901069094912600121460855007171630859375f,-1.6442000258892772723839925674838013947010040283203125f,0.458300025147199807662445891764946281909942626953125f,-0.63470002530813207641813278314657509326934814453125f,0.9498000051125881082469959437730722129344940185546875f,0.40319997929334672193135702400468289852142333984375f,-0.9361999823629851835704585027997381985187530517578125f,-0.32770000677108779285617856658063828945159912109375f,-0.256599938610195810184677611687220633029937744140625f,-1.01080006947815537188262169365771114826202392578125f,0.093899999341368722838296889676712453365325927734375f,0.81080002361983105618747913467814214527606964111328125f,-0.367000009521841974446942913345992565155029296875f,0.02970002143382988180064785410650074481964111328125f,0.2305000034421684773633387521840631961822509765625f,-0.33720001368522645179837127216160297393798828125f,1.0401000065892933577060830430127680301666259765625f,-0.0183999965697527745334127757814712822437286376953125f,-1.1088999733760951382777193430229090154170989990234375f,0.4285999761834740073851435226970352232456207275390625f,-0.0699999947845935110990467364899814128875732421875f,1.93609991981983231568165138014592230319976806640625f,0.520299946379662259232645737938582897186279296875f,0.424699985969066773350277799181640148162841796875f,-2.145799972993134741017229316639713943004608154296875f,-0.33019996817708108238775821519084274768829345703125f,-0.2424000058531763368563360927510075271129608154296875f,1.876799973171948909822503992472775280475616455078125f,0.408099960224330260416536475531756877899169921875f,-0.057099999794364197924778636661358177661895751953125f,-0.26599997454881663117021162179298698902130126953125f,-1.602499957978725575458156527020037174224853515625f,0.445300003898143614833315950818359851837158203125f,0.802499972283840179443359375f,0.27749999769031996610380019774311222136020660400390625f,-0.1327999881401653592405409654020331799983978271484375f,0.87109999574124774568417706177569925785064697265625f,-0.55619998321980201883008021468413062393665313720703125f,0.7580000120848418010410796341602690517902374267578125f,0.981399981310963287484128159121610224246978759765625f,0.23450004491210041379645190318115055561065673828125f,-0.571700007560849154941706729005090892314910888671875f,-0.5481999934494492521253050654195249080657958984375f,-0.298900001390278446944392953810165636241436004638671875f,-0.57550001573562614698431616488960571587085723876953125f,0.252100015217065820394282127381302416324615478515625f,-0.5294000150397419890424544064444489777088165283203125f,-0.093999989427626218230216181837022304534912109375f,0.2183999785766008727705411729402840137481689453125f,-0.6268000020802020344490301795303821563720703125f,0.38819999032020557905298119294457137584686279296875f,-0.6362999988615509749934062710963189601898193359375f,0.840300027292966600356294293305836617946624755859375f,-0.455099997350573826082609230070374906063079833984375f,1.1259000456690788638525191345252096652984619140625f,-0.21810003227144514337254577185376547276973724365234375f,0.26070003621578319297213965910486876964569091796875f,-0.5834000179901723814879233032115735113620758056640625f,-0.884999994635581543889202293939888477325439453125f,0.311599989534914190159042846062220633029937744140625f,-0.824800009471177819619924775906838476657867431640625f,1.579500023193657565201419856748543679714202880859375f,-0.9111000049054622795807745205820538103580474853515625f,0.6411999913409351936621760614798404276371002197265625f,-0.706800014001130794127902845502831041812896728515625f,0.87659999951869205236931748004280962049961090087890625f,-0.07680000401735309623063585604541003704071044921875f,0.6060000217109919873337275930680334568023681640625f,-0.5324000234365460215713028446771204471588134765625f,0.25560004743337660926272292272187769412994384765625f,-1.255999975144863611831169691868126392364501953125f,1.229300025326013123816437655477784574031829833984375f,-0.4546999856710431942019567941315472126007080078125f,-1.0220999953985216990304252249188721179962158203125f,-0.282499985098837669283966533839702606201171875f,-1.2395999819874763403504402958787977695465087890625f,-0.118100037002562885390943847596645355224609375f,0.0384000143021345774485553192789666354656219482421875f,0.806299960416555716591346936183981597423553466796875f,0.1546000212892888470150865032337605953216552734375f,0.1240999690085653472948479247861541807651519775390625f,-0.5453999932765960512170977381174452602863311767578125f,1.0259999329745774776512234893743880093097686767578125f,-0.3669999984204768139761654310859739780426025390625f,0.839399981921911120252843829803168773651123046875f,-0.55690000799894345817619978333823382854461669921875f,-0.141999961987136824603794593713246285915374755859375f,1.674800045680999716068981797434389591217041015625f,0.766799978908895951690283254720270633697509765625f,-1.19060004951953946061848910176195204257965087890625f,0.66269998196065404982846303028054535388946533203125f,-1.338500049259514046440244783298112452030181884765625f,-0.7206000148743394451145150014781393110752105712890625f,-1.057099992064386650980623016948811709880828857421875f,-1.713800014247001346490151263424195349216461181640625f,-0.478100002320111183706785595859400928020477294921875f,-0.4300999912038452333007398920017294585704803466796875f,1.6662000551551585658671683631837368011474609375f,1.367600063269585941583272870047949254512786865234375f,0.21900000455230494988967393510392867028713226318359375f,-0.31060000714436164503950976722990162670612335205078125f,-0.45950003988295851087286791880615055561065673828125f,-0.038000007912516675645520081161521375179290771484375f,-1.378100001239776162265115999616682529449462890625f,-0.270399975320696484004656667821109294891357421875f,0.7473000081300735786271616234444081783294677734375f,-0.6066000135630371659090087632648646831512451171875f,0.9242999976471064638161578841391019523143768310546875f,0.00969996898174256549918226255613262765109539031982421875f,0.67839998366162201914875140573712997138500213623046875f,0.2290999871134757892576772064785473048686981201171875f,1.277700040745734622049667450482957065105438232421875f,-0.29999997623264829105238504780572839081287384033203125f,-0.8833000007838018863282059101038612425327301025390625f,0.14640004404485207345487651764415204524993896484375f,-1.2943000309139491843524183423141948878765106201171875f,-0.326300012347102319498759470661752857267856597900390625f,0.456800037861615937373471751925535500049591064453125f,-0.06890000733211631978036848522606305778026580810546875f,-0.62679999017789922444165995329967699944972991943359375f,0.78829997427836051571148345828987658023834228515625f,-0.45339997999221093305521890215459279716014862060546875f,0.7840000188946729675620872512809000909328460693359375f,0.856800049018860310212630793103016912937164306640625f,0.07949999711662503187881156918592751026153564453125f,0.6197000182487073516313103027641773223876953125f,1.3164999955557288302543383906595408916473388671875f,-0.05740001654475934689259020160534419119358062744140625f,-0.295899997389316826712502006557770073413848876953125f,-0.08090002296343457022231859809835441410541534423828125f,-0.1051000431716442307106262887828052043914794921875f,-0.7895000154264273373172500214423052966594696044921875f,0.12030002595186228830925756483338773250579833984375f,-0.427299993135779487829495337791740894317626953125f,-0.24860007240772308279019853216595947742462158203125f,-1.1567000189788643194077621956239454448223114013671875f,0.106100003415346033630584088314208202064037322998046875f,0.5532000262878840057823026654659770429134368896484375f,-0.1709999744929373566293406838667578995227813720703125f,1.275500043898821100896157076931558549404144287109375f,-1.234500071138144416948989601223729550838470458984375f,0.642200032516569763174629770219326019287109375f,-0.9504000607974834036184574870276264846324920654296875f,-0.2436000641934576993463679173146374523639678955078125f,0.1622999496057626611644764125230722129344940185546875f,-0.379600057238339783083347356296144425868988037109375f,0.87270006905794250684493817971087992191314697265625f,0.2013999641001220641811642053653486073017120361328125f,0.46930001504421259284072220907546579837799072265625f,-0.936600020939110766704516208847053349018096923828125f,-2.01270005743503599404675696860067546367645263671875f,-0.770399999833106363666956895031034946441650390625f,0.477300047171116403887936030514538288116455078125f,0.0531999993726610664879217438283376395702362060546875f,-0.09720003426745493779748130691586993634700775146484375f,-0.6149000077903268302037531611858867108821868896484375f,-0.0991999985456468724720480167889036238193511962890625f,-0.5064000218421227383913674202631227672100067138671875f,-0.099699975463747259052382787558599375188350677490234375f,-0.766400033204257535857095717801712453365325927734375f,0.9715000596120955123069506953470408916473388671875f,0.11780001097619552385964425411657430231571197509765625f,0.198199925053119674345225575962103903293609619140625f,1.7344999911189085128881970376824028789997100830078125f,-0.51440001708865157326044936780817806720733642578125f,-0.036100001478195142823324204073287546634674072265625f,1.37660001333951953483847319148480892181396484375f,-0.19770004134178176258274106658063828945159912109375f,-0.1229999898076048481243560672737658023834228515625f,-1.0470999876499167413612667587585747241973876953125f,0.96429999805688826342020547599531710147857666015625f,-0.27259997492432574972553993575274944305419921875f,0.37419999778196266770891043051960878074169158935546875f,0.324699979859590204256392098614014685153961181640625f,-0.50000000528991239701070981027442030608654022216796875f,0.8522999521017080581941627315245568752288818359375f,-0.539700005694478956996817942126654088497161865234375f,1.42829999728202761133388776215724647045135498046875f,-1.21680003620386134599584693205542862415313720703125f,1.3386999723792083472062586224637925624847412109375f,-0.14979997480735163239984331084997393190860748291015625f,0.2790000010132782648497595801018178462982177734375f,0.30089997986182559497336796994204632937908172607421875f,-0.139599997186660829129323246888816356658935546875f,0.0643000066995622621135453300666995346546173095703125f,0.9229999972581861555909199523739516735076904296875f,0.3375000025518237745103533598012290894985198974609375f,0.08789999059438713402414578013122081756591796875f,0.010799990502000422765149778570048511028289794921875f,-0.227300059092045092512535120476968586444854736328125f,0.1084999688863759192969382638693787157535552978515625f,-0.8554999798238274166806149878539144992828369140625f,0.55789995058476815614767474471591413021087646484375f,-1.3310000368356700306549100787378847599029541015625f,-1.3544000503927460243858149624429643154144287109375f,-1.183299986255168079196664621122181415557861328125f,1.16619998213946818310660091810859739780426025390625f,0.7883000023484232343662370112724602222442626953125f,-0.15770004685521055165509096696041524410247802734375f,-0.06219999312162372007151134312152862548828125f,1.2313000107079747902361077649402432143688201904296875f,0.61799998474121053959606797434389591217041015625f,0.0367999840497972297015394360641948878765106201171875f,-1.397300027203559835697888047434389591217041015625f,-0.627600003609061207754393763025291264057159423828125f,0.219500033140182182478383765555918216705322265625f,0.1617999859869476697582513224915601313114166259765625f,0.37499998733401351813654400757513940334320068359375f,-0.3628000160425894904392407624982297420501708984375f,0.2974000431060783711245676386170089244842529296875f,-1.1924999802559614892061290447600185871124267578125f,2.26669997738599793279945515678264200687408447265625f,0.0068999705225234464478489826433360576629638671875f,-0.5347000315666203817954738042317330837249755859375f,0.16940001083016387184443374280817806720733642578125f,0.62099995547533115569649453391321003437042236328125f,1.1726999616950755100930336993769742548465728759765625f,-1.35359998806714987296118124504573643207550048828125f,-0.4350000169873238231588175040087662637233734130859375f,1.06940000054836303888805559836328029632568359375f,-0.1553000179052350659958392498083412647247314453125f,0.5595000084042549559626422706060111522674560546875f,0.8053000049225982337475215899758040904998779296875f,1.1966999973163006121268381320987828075885772705078125f,0.37990004081502615207455164636485278606414794921875f,1.39510003613084609952466053073294460773468017578125f,-1.178899988612533089593625845736823976039886474609375f,0.302200005862116594101962618879042565822601318359375f,-0.1429000539988274365299503187998197972774505615234375f,-2.353700014251470573611868530861102044582366943359375f,1.802700035381317178462268202565610408782958984375f,-0.05619999221637794750705552360159344971179962158203125f,0.27930000163316659467227509594522416591644287109375f,-0.168100030669570355001951611484400928020477294921875f,0.3349000392317780239181956858374178409576416015625f,-0.09779995971620059069806529805646277964115142822265625f,-0.153000044345855457095240126363933086395263671875f,-0.83129999776259067001404901020578108727931976318359375f,0.14149998700618748870283525320701301097869873046875f,1.1667000148810446891189940288313664495944976806640625f,0.2547000010192397301267419607029296457767486572265625f,1.2034000228516765762520890348241664469242095947265625f,0.8449999653361734974765795413986779749393463134765625f,-0.69950001540779993547403137199580669403076171875f,-0.4302999411642549620893305473146028816699981689453125f,-1.8381000247836121186395530457957647740840911865234375f,0.72769999477565239232035310124047100543975830078125f,1.764999994561076590571246924810111522674560546875f,0.43479993541240613552645299932919442653656005859375f,-0.57850007162988159592487136251293122768402099609375f,-0.2019999660849567391096570645458996295928955078125f,0.251400013571977165582893576356582343578338623046875f,-1.3992000009298326546058888197876513004302978515625f,-0.7463999766170978933388369114254601299762725830078125f,0.092100019836426127994855050928890705108642578125f,-0.3427999978631735000789149125921539962291717529296875f,-0.34550001040846078925738993348204530775547027587890625f,0.1435999915316703567924605522421188652515411376953125f,-0.61870001233294658060657411624561063945293426513671875f,0.91039999700188600773032021606923080980777740478515625f,0.404599939668179420237947852001525461673736572265625f,1.201399984589219815944716174271889030933380126953125f,0.771300017711519902974259821348823606967926025390625f,-0.391299970325828194717132646474055945873260498046875f,-0.7343999776005745161455706693232059478759765625f,-0.469700041475892504649891634471714496612548828125f,1.1113999618649490486177455750294029712677001953125f,1.193599978977442521710372602683492004871368408203125f,0.31799997431039717099565677926875650882720947265625f,-0.0567999990999703552319033406092785298824310302734375f,0.168000015139579517153833876363933086395263671875f,-0.9260999680995940508410058100707828998565673828125f,0.28499999016523336337058935896493494510650634765625f,-1.0323999745607377320766318007372319698333740234375f,0.7655999919638036832481020610430277884006500244140625f,0.05369997938275350435333166387863457202911376953125f,-0.2752000042378901500939036850468255579471588134765625f,0.822600030542910420905400314950384199619293212890625f,0.72800003235042110016905780867091380059719085693359375f,-1.9609000622652477119345348910428583621978759765625f,-1.030700040220469571039529910194687545299530029296875f,1.61289999446868836940893743303604423999786376953125f,-0.5075000542588536944776933523826301097869873046875f,-0.906099951112270929343139869160950183868408203125f,0.783299970012902857519065946689806878566741943359375f,-1.49550003778934392784094598027877509593963623046875f,-1.40170002153068740113894818932749330997467041015625f,1.2356000131607043357462316635064780712127685546875f,-0.29450003265589497214449465900543145835399627685546875f,0.0680000002384186785775455064140260219573974609375f,0.94410001915097208069482803693972527980804443359375f,0.9721000208795071984013702603988349437713623046875f,0.06310001114904906227565106746624223887920379638671875f,0.35530000360012081728200428187847137451171875f,0.0667999877937137664929423408466391265392303466796875f,-2.21070003977343443324343752465210855007171630859375f,-1.0155000138171021983879427352803759276866912841796875f,0.2349000217601660001065511096385307610034942626953125f,-0.580200046780706912841196754015982151031494140625f,-0.7299000323772428799884437466971576213836669921875f,-0.014599981391430372212880683946423232555389404296875f,0.97420006768703526489616706385277211666107177734375f,-1.42850004234910077371978331939317286014556884765625f,1.8832999755263326591148143052123486995697021484375f,0.546199957105518052458137390203773975372314453125f,-0.62780000479221342857272247783839702606201171875f,0.365200005578994790766955702565610408782958984375f,-1.574999989271164135828939834027551114559173583984375f,0.12300000716745851381261900314711965620517730712890625f,-0.06339999798536322117570307455025613307952880859375f,-0.3505999908834696032755573469330556690692901611328125f,1.281299977701902292892555124126374721527099609375f,0.38889997665435060980598791502416133880615234375f,-0.6570999577358360710377382929436862468719482421875f,-0.93289998150467834392429722356610000133514404296875f,1.28700006622076035256441173260100185871124267578125f,-0.812499995827674847959087855997495353221893310546875f,-0.607900042152405006845583557151257991790771484375f,1.12909999977946284843710600398480892181396484375f,-0.3168999668717393802808146574534475803375244140625f,0.566100040295719253435891005210578441619873046875f,0.307699984717370256248614168725907802581787109375f,-0.091700065898894678184660733677446842193603515625f,-0.186599996855109806492833968150080181658267974853515625f,-1.258199992499500208253948585479520261287689208984375f,0.4805000252909958913249965917202644050121307373046875f,-0.0524000371456148084092774297459982335567474365234375f,-0.50780001937672525347267082906910218298435211181640625f,-0.86339997943341728614541352726519107818603515625f,0.12679998585656304666002824887982569634914398193359375f,1.9314999688267704414101899601519107818603515625f,-0.77950001434609317296775543582043610513210296630859375f,0.403200028765201778924165409989655017852783203125f,-0.07589999321699114620543014098075218498706817626953125f,0.76910002406835520361028102342970669269561767578125f,-0.2566000029832122830697471727035008370876312255859375f,-0.3986000166773795427843651850707828998565673828125f,-0.89170000165626406296581762944697402417659759521484375f,1.6196000028491024380628005019389092922210693359375f,0.26370002496168021277611615005298517644405364990234375f,0.36270000792592760330279588742996565997600555419921875f,0.08280002526268381846108468380407430231571197509765625f,-0.11069996532350767637353783356957137584686279296875f,-0.4685999899297959014887737794197164475917816162109375f,0.64889999688267696598131806240417063236236572265625f,0.83779999354183676274487879709340631961822509765625f,0.668699988174438431087764911353588104248046875f,-0.803100034767389114875868472154252231121063232421875f,-1.1925000011920925402364446199499070644378662109375f,1.075799983423948003036230147699825465679168701171875f,0.449699948418139427985806833021342754364013671875f,-0.7139000283986323136531382260727696120738983154296875f,-1.13820003174543415980224381200969219207763671875f,0.4231000004947187420611953712068498134613037109375f,1.1607000275731085281449850299395620822906494140625f,0.38179999090433103248187762801535427570343017578125f,-1.79270002644062032004512730054557323455810546875f,0.51120000508725649268626511911861598491668701171875f,-0.681600040962547115697134358924813568592071533203125f,0.49530000246390704088383927228278480470180511474609375f,-0.1650999524235723381337948012514971196651458740234375f,-0.8559999736547470039482732317992486059665679931640625f,-1.23389996939003498965803373721428215503692626953125f,-0.16280000382363812860830876161344349384307861328125f,2.183099955344200537865617661736905574798583984375f,0.128599976295233009437879445613361895084381103515625f,-0.75899999415874486174971025320701301097869873046875f,0.2555999709159129107405306058353744447231292724609375f,1.45729999084472705561665861750952899456024169921875f,0.37769996102452285668960030307061970233917236328125f,-1.1264999893903731020827763131819665431976318359375f,-0.52939998981952651746496485429815948009490966796875f,1.263199976611137032023179926909506320953369140625f,-0.13730001051425944069706019945442676544189453125f,-0.831000023126602371803528512828052043914794921875f,0.7954000431954868499673239057301543653011322021484375f,0.4945000276826323482026737110572867095470428466796875f,0.7342999872535467265066699837916530668735504150390625f,0.61250000108033397783202644859557040035724639892578125f,-1.143200000080466072205354066682048141956329345703125f,-0.347900018217414508381324367292108945548534393310546875f,0.716099980169534688911880948580801486968994140625f,-0.76519998374879349967159214429557323455810546875f,-0.0010000398159022427790887377341277897357940673828125f,-0.7612999646447595747389414100325666368007659912109375f,-0.099099982121586993599748893757350742816925048828125f,-1.092699998612701950406744799693115055561065673828125f,0.27069996431768006406315407730289734899997711181640625f,-0.7291999603241678951093263094662688672542572021484375f,0.1970000015869735054963030052022077143192291259765625f,-0.09720000008791684908970864853472448885440826416015625f,0.99400002723932256998295997618697583675384521484375f,1.2225999898999919679454251308925449848175048828125f,-0.875700030106306481769706806517206132411956787109375f,0.6152000142216678124640338864992372691631317138671875f,-0.914700004148482737065251058083958923816680908203125f,-0.397000025987625182466445039608515799045562744140625f,0.15179999284148237848057760857045650482177734375f,-0.600999989822506019976344759925268590450286865234375f,-1.841899974024295172370102591230534017086029052734375f,-0.13769995774626675455465374398045241832733154296875f,-0.644100015425682403957807764527387917041778564453125f,-1.560600017556547758346141563379205763339996337890625f,-1.3590000163614757155983170378021895885467529296875f,0.125900055243075126298890609177760779857635498046875f,-0.3453999975979329395414652026374824345111846923828125f,-0.271000004760920953916780717918300069868564605712890625f,-0.46220003268420750686118481098674237728118896484375f,0.18380000064969070905362968915142118930816650390625f,-0.1442000024199494578880376138840802013874053955078125f,0.1284999991357320681117926142178475856781005859375f,0.1888000016182658225005752683500759303569793701171875f,-0.5859999946653842162191949682892300188541412353515625f,-0.2650999590545897177662482135929167270660400390625f,-0.15599998438358309016393832280300557613372802734375f,1.31269995789527893492731891456060111522674560546875f,-0.674700014951824744713348991353996098041534423828125f,0.0240999967992305830222221629810519516468048095703125f,-1.1077999977886678895089289653697051107883453369140625f,1.527300023701787523577877436764538288116455078125f,-0.319100014308094870330023695714771747589111328125f,-0.24290001533031502223280995167442597448825836181640625f,-1.118899986898899356901893042959272861480712890625f,0.44480003171116155868958230712451040744781494140625f,-0.3410000075176358169670720599242486059665679931640625f,-0.73219999803900659429700681357644498348236083984375f,1.129700005564092446519453005748800933361053466796875f,1.13439997868090802768392677535302937030792236328125f,0.53630004699230227060979814268648624420166015625f,1.2021000238969927753629463040851987898349761962890625f,-0.068999988384545130060132578364573419094085693359375f,0.592399994975327803814479921129532158374786376953125f,-0.27970001638606156735278318592463620007038116455078125f,-0.576200027886033527835252243676222860813140869140625f,-0.14269997860789285226701395004056394100189208984375f,0.648099990101159040278844258864410221576690673828125f,1.206700009796022765584666558424942195415496826171875f,0.5348999739274378217857019990333355963230133056640625f,-0.2003999859377738612664643369498662650585174560546875f,0.54160001497864740382937043250421993434429168701171875f,-0.68200001168251045413626343361102044582366943359375f,0.175799997803568752718916812227689661085605621337890625f,1.14889996894299972751696259365417063236236572265625f,2.04619995136857024675691718584857881069183349609375f,0.05740000033974634874311959720216691493988037109375f,1.299600037717819844829136854968965053558349609375f,-0.00840004298687002659562494955025613307952880859375f,-0.8112000384099786742808646522462368011474609375f,-0.8818999641336497941068728323443792760372161865234375f,-0.34700000816211262755217603626078926026821136474609375f,1.9466000336423519012640781511436216533184051513671875f,1.48319999620616460589417329174466431140899658203125f,1.41589997116327293014137467253021895885467529296875f,-0.086300068338215130125945506733842194080352783203125f,-0.5767999958962197748491007587290368974208831787109375f,0.52640001182109141808496133307926356792449951171875f,0.93660003487169785074684114078991115093231201171875f,0.0191999989926816549967725222813896834850311279296875f,0.6996000254988672395484172739088535308837890625f,0.346599993148445861823603308948804624378681182861328125f,-0.8302000200331196566594371688552200794219970703125f,0.470199949848652654083025481668300926685333251953125f,-0.47149999684095433849506662227213382720947265625f,0.093699983257054864083102074800990521907806396484375f,1.2250999728381637421392724718316458165645599365234375f,1.0913000082120305478383670561015605926513671875f,-0.700999986730516777555521912290714681148529052734375f,0.26110004960894539483007292801630683243274688720703125f,-1.5507999938547616380901672528125345706939697265625f,0.402599984833597712707842219970189034938812255859375f,1.25579998148679816694084365735761821269989013671875f,0.9755999748647219416142206682707183063030242919921875f,-0.73300004274398133929935283958911895751953125f,0.82780000870376824195773224346339702606201171875f,-0.5466000097632406351522149634547531604766845703125f,0.329800009322166687297794851474463939666748046875f,1.080300038468838152994067058898508548736572265625f,-1.19830009026527495308300785836763679981231689453125f,-0.3577999766290183014660897242720238864421844482421875f,-0.43830004287958246322887134738266468048095703125f,-0.5048999770939348596954232561984099447727203369140625f,-0.102600001448392763592210030765272676944732666015625f,-0.0037999874807892464190217651776038110256195068359375f,0.477099994336813104656158657235209830105304718017578125f,-0.6170000362135470428626149441697634756565093994140625f,1.569500058546662302205731975845992565155029296875f,-0.0327000066220757279467079570167697966098785400390625f,0.3613000206917524881333747543976642191410064697265625f,-0.59800004125386518882834252508473582565784454345703125f,-0.62640000749975432992044943603104911744594573974609375f,0.2797999829471107258171969078830443322658538818359375f,-0.36870001518279316155002334198798052966594696044921875f,1.039200037959218558825114087085239589214324951171875f,-0.8266000017166135194202070124447345733642578125f,0.8911999820649629810986880329437553882598876953125f,0.815499979376792083485270268283784389495849609375f,-1.801500022768974584863599375239573419094085693359375f,1.58930001325607150874930084683001041412353515625f,-1.28950003194808981987762308563105762004852294921875f,1.0543999894097446645702120804344303905963897705078125f,1.070899977943300829252848416217602789402008056640625f,-0.0903999960519377143697283827350474894046783447265625f,0.5124999864399428783912071594386361539363861083984375f,-1.08279992395341384536777695757336914539337158203125f,0.58280001265257563858313005766831338405609130859375f,-1.292599967958032447512550788815133273601531982421875f,-0.38509998932257294423919802284217439591884613037109375f,-0.68520002588182682412565327467746101319789886474609375f,0.26680000098124156959755737261730246245861053466796875f,1.005299986966698977397527414723299443721771240234375f,0.05699999791756270306475329334716661833226680755615234375f,0.152900030250102736406603298746631480753421783447265625f,0.501199983387440095583542642998509109020233154296875f,-0.72400002668052909626794644282199442386627197265625f,0.311099997603892841624428911018185317516326904296875f,-0.2979000127412378962077355026849545538425445556640625f,0.1389999975487587191569360811627120710909366607666015625f,-0.196900035547465268592759457533247768878936767578125f,0.22330000339150390953335545418667607009410858154296875f,0.774600031943619260488276268006302416324615478515625f,-0.678400015196203742817715465207584202289581298828125f,0.75320002261847307511999360940535552799701690673828125f,0.1236000133246182297597215438145212829113006591796875f,-1.284600061820448058114152445341460406780242919921875f,1.0256000342458488905350577624631114304065704345703125f,0.22690002439022105562571596237830817699432373046875f,0.497799998533725585048159700818359851837158203125f,-0.6902999863706524852347001797170378267765045166015625f,-0.1239000419028115640429632549057714641094207763671875f,-0.6612999854505059449394366311025805771350860595703125f,-0.539800020796060930905468921991996467113494873046875f,0.42480002228617674830957184894941747188568115234375f,-0.71190001973360761411413477617315948009490966796875f,0.9460000390335920883444487117230892181396484375f,-0.21829998904913605617394978253287263214588165283203125f,1.080200066468865127689014116185717284679412841796875f,0.08249998152256043937313734204508364200592041015625f,-1.279700034211576298304180454579181969165802001953125f,0.18089999828338587661846759147010743618011474609375f,-1.787500048764050664118485656217671930789947509765625f,1.10990006396770635177517760894261300563812255859375f,-0.42989999501258113756563261631526984274387359619140625f,0.58890001878738384988309917389415204524993896484375f,-0.15439998687654732378859989694319665431976318359375f,0.3355999844759705030838858874631114304065704345703125f,-1.1089000223077831730478237659553997218608856201171875f,0.1804000280335535943976310591096989810466766357421875f,-0.999900007864833018089711913489736616611480712890625f,0.397299996507166408576949834241531789302825927734375f,-0.40660000194013090624167716669035144150257110595703125f,0.746899965493380335601614206098020076751708984375f,-1.1308999701760706901865205509238876402378082275390625f,1.588299998604506324539897832437418401241302490234375f,0.81529998482465781961536777089349925518035888671875f,0.46749995797872667679939695517532527446746826171875f,-0.121699984538555572299856066820211708545684814453125f,-0.59789998150467926762985371169634163379669189453125f,0.693099978739023736551416732254438102245330810546875f,-0.5197000324606886323408616590313613414764404296875f,-0.034999963343143036809124168939888477325439453125f,-0.946899972534179568128820392303168773651123046875f,-0.220699997696280458825413006707094609737396240234375f,0.5489999761283390622423894456005655229091644287109375f,-0.34710000091195081228789831584435887634754180908203125f,0.208400025068223637703113126917742192745208740234375f,0.5391999869972465830869623459875583648681640625f,-0.1386000023722646634638522300519980490207672119140625f,0.996500000268220720300860193674452602863311767578125f,-0.4733999948933724777333509337040595710277557373046875f,1.202799999055267132774815763696096837520599365234375f,0.2084999827817084483427834129543043673038482666015625f,0.112500004395842179150122319697402417659759521484375f,-0.031799985241889761056199859012849628925323486328125f,-0.265600004369020314953786510159261524677276611328125f,0.784799985145032241717899523791857063770294189453125f,-0.258399975559115535617138448287732899188995361328125f,-0.894100000524520766020941664464771747589111328125f,0.5605999851465224281810151296667754650115966796875f,0.7126999912738796183475642465054988861083984375f,-0.936700020599364080453597125597298145294189453125f,0.58630002128779901138244667890830896794795989990234375f,-0.141800022345781595589642165577970445156097412109375f,-0.95029998914599378423417874728329479694366455078125f,-0.2154999808669089134127716533839702606201171875f,-0.6883000192239887571332701554638333618640899658203125f,0.659200007486344219387319753877818584442138671875f,-0.2995999917104839038728414379875175654888153076171875f,-0.03169999943971646416684961877763271331787109375f,-1.077799996447563746215791979921050369739532470703125f,-0.0025000146031377568078823969699442386627197265625f,-0.2804000055328013107924789437674917280673980712890625f,-0.6130000061988827297909665503539144992828369140625f,-0.88959999659061406163118590484373271465301513671875f,-0.0753000044941902757500429288484156131744384765625f,0.7432000243321061905277247205958701670169830322265625f,-0.1973999995946886798492414527572691440582275390625f,0.1969999750256541393156339836423285305500030517578125f,1.810299915683270199906473862938582897186279296875f,-0.3453000232696528115639011957682669162750244140625f,0.7269999788999559342528300476260483264923095703125f,-1.15539996407032052871954874717630445957183837890625f,0.6205999744176864396649762056767940521240234375f,-0.60920005382895414669519595918245613574981689453125f,1.7277999428033847806318590301088988780975341796875f,-1.64409996789097778702171126496978104114532470703125f,-0.3456999733328824930822520400397479534149169921875f,0.26239999542236347451762412674725055694580078125f,0.803899966931343357146033667959272861480712890625f,-0.63260001158118228659077431075274944305419921875f,-0.459699997293949280674496549181640148162841796875f,-0.56579999698400484930971288122236728668212890625f,1.102899972712993559298411128111183643341064453125f,-0.1438999975532293529312255486729554831981658935546875f,-0.2265999882310627544512726672110147774219512939453125f,-0.45560001723989829169880749759613536298274993896484375f,0.29079996362402982246209148797788657248020172119140625f,-1.00930001282766479420160976587794721126556396484375f,0.205900052374601738591763933072797954082489013671875f,-0.516100040742755172828992726863361895084381103515625f,0.985500022105873174638190903351642191410064697265625f,0.743900010442733350402022551861591637134552001953125f,0.02669992418885158702579474265803582966327667236328125f,0.4008000644862652617206322247511707246303558349609375f,0.5577000510275367251011857661069370806217193603515625f,-0.157000030457973505093605126603506505489349365234375f,-1.0161000293061139831962691459921188652515411376953125f,0.72920001158416258846273194649256765842437744140625f,0.0423000134944917505208650254644453525543212890625f,0.4643999874353408330307502183131873607635498046875f,0.446300010335445307418922311626374721527099609375f,-0.92690002114921821174675642396323382854461669921875f,1.4048000221557914724002102957456372678279876708984375f,-1.1291000296004110037273449052008800208568572998046875f,0.99260004581660055134051390268723480403423309326171875f,0.071799949777125338101768647902645170688629150390625f,-0.57069999933540760395089819212444126605987548828125f,0.9881999819010485186510095445555634796619415283203125f,-1.6270000313147907622379761960473842918872833251953125f,-0.2742999688506113642461059498600661754608154296875f,-0.9104999724104996250417798364651389420032501220703125f,-0.32460005768537580905785944196395576000213623046875f,-0.5852999937653537987358731697895564138889312744140625f,0.8844000303506849292034530662931501865386962890625f,-1.2423000222116706847685918546631000936031341552734375f,0.371700007188319858641989412717521190643310546875f,0.28860000255852913397802694817073643207550048828125f,-0.056600009465217482329535414464771747589111328125f,-1.11039998314380650157318086712621152400970458984375f,0.9462999739021065170874180694227106869220733642578125f,-0.615700000099092203953432544949464499950408935546875f,1.1051999533131720010459275727043859660625457763671875f,1.8067999695956711292410545866005122661590576171875f,0.34709998884201009872185750282369554042816162109375f,0.3196999902158967898913033423013985157012939453125f,-0.10639992744326765006235291366465389728546142578125f,-2.418500029087066405253381162765435874462127685546875f,-0.288799982023238754891281132586300373077392578125f,0.31009996196031597293085724231787025928497314453125f,2.094099965953827080511473468504846096038818359375f,0.07069997079968448616682508145458996295928955078125f,-1.330999963521957596412903512828052043914794921875f,-0.011399977958202356376204988919198513031005859375f,-0.35009998639822015320532955229282379150390625f,0.00700000169873238320406017010100185871124267578125f,0.693500000625848489477220937260426580905914306640625f,-0.9439999892413613036268316136556677520275115966796875f,0.9071000278085470380773358556325547397136688232421875f,-0.10230002273321137096218080841936171054840087890625f,-1.8573000319905579313939369967556558549404144287109375f,0.69979999654442048839797507753246463835239410400390625f,0.934100041409582360074637108482420444488525390625f,0.00010000740885740544428017528844065964221954345703125f,-1.37030002952814111694124221685342490673065185546875f,1.12930003918409394003674606210552155971527099609375f,0.12670002663433532763548328148317523300647735595703125f,-0.9987999967753877772480564090074039995670318603515625f,2.04180000364482427954726517782546579837799072265625f,-0.640399988508224549121905511128716170787811279296875f,-0.6795000241994859635497050476260483264923095703125f,0.73510005106926001872125198133289813995361328125f,0.61939999816417667943824199028313159942626953125f,-0.6959999942183490162506132037378847599029541015625f,0.01859997927546508389440305109019391238689422607421875f,-0.82220004736185092752975833718664944171905517578125f,0.173900006121396888403296543401665985584259033203125f,0.698200028429925456663340810337103903293609619140625f,0.2233000074893236597262102804961614310741424560546875f,0.22340001538246856416236596487578935921192169189453125f,-0.7006000533007090691484108901931904256343841552734375f,0.863700080859661767362922546453773975372314453125f,1.1250999953761693372911167898564599454402923583984375f,0.0268000096797944564741555950604379177093505859375f,0.28179994325786805742239948813221417367458343505859375f,0.00999998360872300651180921704508364200592041015625f,-1.3442000490233312337551296877791173756122589111328125f,0.5970000462532052409869720577262341976165771484375f,-1.05610000457018582409318696591071784496307373046875f,-0.1727999785661697984551210538484156131744384765625f,0.41700000005960458082654440659098327159881591796875f,-0.7078000131368629155304006417281925678253173828125f,-0.45520001764893525120214690105058252811431884765625f,1.48669995667338383782407618127763271331787109375f,0.07170000480413440957505599726573564112186431884765625f,-0.177600000144541692037591928965412080287933349609375f,1.5783000317595894390620969716110266745090484619140625f,-0.991199999927729447080082536558620631694793701171875f,-1.2727999681353576733044974389486014842987060546875f,-0.0873999680042256699152858345769345760345458984375f,1.085999976634978825273947222740389406681060791015625f,-0.7575000074505791047840830287896096706390380859375f,1.39680001623630456464297822094522416591644287109375f,0.32769999365806512514609494246542453765869140625f,-1.06000002503395052144696819595992565155029296875f,-0.65979999024867908019587048329412937164306640625f,0.02039999836161732194028672893182374536991119384765625f,-0.13519998825639467643355828840867616236209869384765625f,-0.4675000101141633113144280287087894976139068603515625f,0.0889000060096385436025911985780112445354461669921875f,0.16800000649690616238984830488334409892559051513671875f,1.4616999986574048531196012845612131059169769287109375f,-0.655999987065791767548716961755417287349700927734375f,-0.6882999642014497876374434781610034406185150146484375f,1.127699975478649196958258471568115055561065673828125f,0.610799968522786951297121049719862639904022216796875f,-0.115499958068132624333657076931558549404144287109375f,-0.8257000246301295920403617856209166347980499267578125f,0.647600012400746383178784526535309851169586181640625f,1.5208000157967209720055734578636474907398223876953125f,-0.146499992743134743733435243484564125537872314453125f,-1.2907000079035757522660787799395620822906494140625f,0.93199998307228071325880591757595539093017578125f,0.218100022213161270911996325594373047351837158203125f,0.32850000341981611118313821862102486193180084228515625f,-0.6110000293105841695506796895642764866352081298828125f,0.159500016301870051194100597058422863483428955078125f,0.8359000070005653260096778467413969337940216064453125f,-1.18010002186298290638433172716759145259857177734375f,-0.70390001260340195887721392864477820694446563720703125f,0.4304999647736540424602935672737658023834228515625f,-0.0063999960631135888178278037230484187602996826171875f,-0.2832000076055525283891256549395620822906494140625f,-1.5390000147968529997655195984407328069210052490234375f,0.62020001153945969463165965862572193145751953125f,-0.0925999780163164754043236825964413583278656005859375f,-0.459100031220913251672754995524883270263671875f,-1.45889999420046745370882490533404052257537841796875f,0.2036999980092044637558501563034951686859130859375f,0.2020000121667979708917073367047123610973358154296875f,0.20280001466423291933693917599157430231571197509765625f,1.398900008673221062593938768259249627590179443359375f,-1.49779998737648156748036853969097137451171875f,-0.2816999621123079666773492135689593851566314697265625f,-0.388999961525202220258279339759610593318939208984375f,0.100899965575337535739208760787732899188995361328125f,0.705999945566059050605645097675733268260955810546875f,-1.16730002183914205460268931346945464611053466796875f,1.9325999902725214951715315692126750946044921875f,-0.732900030827522641629911959171295166015625f,-0.2159999718666085755103267729282379150390625f,-0.00060003238320316398812792613171041011810302734375f,0.5969999911189081132079081726260483264923095703125f,-0.38540002859234778753716454957611858844757080078125f,-0.1257999785065653242099870112724602222442626953125f,-1.142400001978874257702045724727213382720947265625f,0.834099990987777317741347360424697399139404296875f,-0.0868000031977891328693885952816344797611236572265625f,-0.4187000254273416377515104613848961889743804931640625f,0.252000009968876614863120266818441450595855712890625f,-0.91349999567121253907231448465608991682529449462890625f,0.3672999882370235891215770607232116162776947021484375f,1.125600004070996806859739081119187176227569580078125f,-0.32320000306069829321131692267954349517822265625f,-0.91159998566061251068504134309478104114532470703125f,1.260199980917573014949084608815610408782958984375f,-0.0338999961376187375350355068803764879703521728515625f,0.71959999130070162465244720806367695331573486328125f,-0.4291000080496074620128865717560984194278717041015625f,0.947600003311037664133209545980207622051239013671875f,-0.263000001840293151644800673238933086395263671875f,0.74989999799281326087196930529898963868618011474609375f,-0.2740000122636556678656916119507513940334320068359375f,0.29480000414401263952868248452432453632354736328125f,-0.670099998319148681957813096232712268829345703125f,-0.182699998277426001180856474093161523342132568359375f,-0.8652000115767120025367376001668162643909454345703125f,-0.17019998203516006896052203956060111522674560546875f,1.0290999932229514257642222219146788120269775390625f,-0.620500010222195808040623887791298329830169677734375f,-0.37610000825822342829241051731514744460582733154296875f,0.751200002777576258949920884333550930023193359375f,-0.558599998684227383449751869193278253078460693359375f,0.248499983489512743517479975707828998565673828125f,-0.51509999019801622210934510803781449794769287109375f,0.854999997466802330592372527462430298328399658203125f,-0.4551999658301462137188764245365746319293975830078125f,0.5156000019103281761800872118328697979450225830078125f,-0.616400004668534062801654727081768214702606201171875f,0.0281999999314546112572088532033376395702362060546875f,0.4858000305861238654614453480462543666362762451171875f,1.0142000164270399409360834397375583648681640625f,-0.809600022444129319154626500676386058330535888671875f,-0.031499982982874374926041127764619886875152587890625f,-0.813399969524145394217384819057770073413848876953125f,0.149999977573752829584918799810111522674560546875f,0.877699984642863473283114217338152229785919189453125f,0.893899940481782895318474402301944792270660400390625f,-0.2404000285923475477289912305423058569431304931640625f,0.1299999779462819304853837820701301097869873046875f,-0.248100000569224210522634166409261524677276611328125f,-0.404099981600047453156321353162638843059539794921875f,0.768199945393205130272917813272215425968170166015625f,0.373300025817751457424265026929788291454315185546875f,-0.5750999928429723073719515014090575277805328369140625f,-1.1652999592691670738275888652424328029155731201171875f,-0.1242000020101665658245337908738292753696441650390625f,0.3904000107482070713871280531748197972774505615234375f,-0.10659999501109129749210069348919205367565155029296875f,-0.228199991065263230183290943386964499950408935546875f,-0.303899981385469697414691836456768214702606201171875f,-0.02930003523528645903439837638870812952518463134765625f,-0.4706000001966950119225430171354673802852630615234375f,-0.8677999652482564219013738693320192396640777587890625f,-0.1047000397250059389175902424540254287421703338623046875f,0.7921000143043699370792865011026151478290557861328125f,-0.47279998624026753351046181705896742641925811767578125f,1.176399970805644290550162622821517288684844970703125f,-0.9643999695539469119154318832443095743656158447265625f,-0.739699997331201952732726567774079740047454833984375f,0.0957999893471599417438255841261707246303558349609375f,0.16779999436140047297527644332149066030979156494140625f,-0.6744000031188128740922138604219071567058563232421875f,0.53460001804828660709745236090384423732757568359375f,-0.5558000118851662563201898592524230480194091796875f,-0.403999979406595599584761657752096652984619140625f,-0.654600003594160373410204556421376764774322509765625f,-0.4040999932229516478088271469459868967533111572265625f,-1.10180001564025786819911445491015911102294921875f,1.0318000306531789345143579339492134749889373779296875f,0.386599985809623181154393023462034761905670166015625f,0.691100009301305107101143221370875835418701171875f,0.28470001610666473279565025222836993634700775146484375f,-0.283900049651415020690592427854426205158233642578125f,-1.12299999874830103152589799719862639904022216796875f,-0.4683000224649911391594514498137868940830230712890625f,0.0194000033050778508680878076120279729366302490234375f,0.6289000279515983837796966327005065977573394775390625f,-0.0131999944180247741343237066757865250110626220703125f,-0.284200014191866134893871276290155947208404541015625f,-0.24600001388788239609084484982304275035858154296875f,-0.0474000029474497264203591839759610593318939208984375f,1.0369999742805955378344151540659368038177490234375f,-1.44479998272359377864404450519941747188568115234375f,-0.0194999810606244494692873558960855007171630859375f,-0.907800023567676195312969866790808737277984619140625f,1.07829993946552349370904266834259033203125f,1.00760001128315934693091548979282379150390625f,0.577900012238323323998656633193604648113250732421875f,0.0273000302024175578452513946103863418102264404296875f,-0.895199943385273311236005611135624349117279052734375f,-0.612300003212690402421003454946912825107574462890625f,0.25229994360804663955377691308967769145965576171875f,-0.5100999918743964745004859651089645922183990478515625f,1.3490999724358327949147451363387517631053924560546875f,0.1754000111207367762489184315199963748455047607421875f,-0.7716000010833139466370766967884264886379241943359375f,0.535600014352798581285242107696831226348876953125f,-0.13600000435113912544693448580801486968994140625f,-0.415199995744228278482523819548077881336212158203125f,0.130299985271692175814450820325873792171478271484375f,0.601499983578920538462853073724545538425445556640625f,0.9635999494731419812154626924893818795680999755859375f,-0.15109999507069549196103253052569925785064697265625f,1.35449999544024368702821448096074163913726806640625f,1.0530999782919880924936251176404766738414764404296875f,0.5727000236839068225691562474821694195270538330078125f,0.22729995053708496044464482110925018787384033203125f,0.227099954941868187319187200046144425868988037109375f,-0.66630000497102770395940751768648624420166015625f,0.32990000182986278787211631424725055694580078125f,0.86209996701180902078931467258371412754058837890625f,1.6457999883413314545776984232361428439617156982421875f,-0.229400001144409504405530242365784943103790283203125f,-0.224299983900785537827005100552923977375030517578125f,-0.003699974912404258020615088753402233123779296875f,-0.016199995774030639950069598853588104248046875f,0.04740002865195247494511932018212974071502685546875f,-0.7870999936103821870148067318950779736042022705078125f,0.3092999633371835699335861136205494403839111328125f,0.9524999704211953854837702238000929355621337890625f,-0.951599993520975129257521984982304275035858154296875f,0.643299982529878189296823620679788291454315185546875f,-0.759800005820393753452890450716949999332427978515625f,0.4551999613970512115201927372254431247711181640625f,1.2176000129222881440682613174431025981903076171875f,-0.3764000142425292239067857735790312290191650390625f,-0.0937000388383875115749788164976052939891815185546875f,1.303599990451334900143365302938036620616912841796875f,-0.8066999933302396730283589931786991655826568603515625f,0.0963999790757898988857732547330670058727264404296875f,0.0765999834626907016854602261446416378021240234375f,0.140300029602646869619775316095910966396331787109375f,-0.62770003158152132272107337485067546367645263671875f,1.6575000390782947956580528625636361539363861083984375f,0.255500003144145171329881804922479204833507537841796875f,-0.08319999724924487249921867260127328336238861083984375f,1.3905000472143296263993761385791003704071044921875f,0.16260001718774408363543670930084772408008575439453125f,0.691400013851374506401725739124231040477752685546875f,-0.2312000158905984648072262643836438655853271484375f,-0.7739999990575017196903218064107932150363922119140625f,0.394200000333785993689161841757595539093017578125f,0.4613000120863315345332011929713189601898193359375f,-2.2139999630525704077399495872668921947479248046875f,-0.456300000724195964352247756323777139186859130859375f,1.265599950315058119798550251289270818233489990234375f,-0.00939998756572624216687472653575241565704345703125f,-0.1918999564409258340447195223532617092132568359375f,0.99609999376684432714768036021268926560878753662109375f,0.1593999942898751687181402303394861519336700439453125f,1.2561000383958227377689809145522303879261016845703125f,0.27150000138580787645281588993384502828121185302734375f,1.9689000344149771848378804861567914485931396484375f,0.3578999698631462056397367632598616182804107666015625f,0.1738999909035865432116452211630530655384063720703125f,-2.54050005198270145712058365461416542530059814453125f,-1.013199973705411149893507172237150371074676513671875f,-1.546300004076957890220000990666449069976806640625f,0.216500062853096864046165137551724910736083984375f,-0.90160001065731076863585258251987397670745849609375f,-1.460099970714747730227145439130254089832305908203125f,0.67749998122453625182259884240920655429363250732421875f,1.1085000297203659869182956754229962825775146484375f,0.268800001059472659648008630028925836086273193359375f,0.332699984940886483997246614308096468448638916015625f,-0.289200036692619733003084547817707061767578125f,0.088400006479024906269614803022705018520355224609375f,-0.2801000178396699791250057387514971196651458740234375f,-0.39510003393292425499794262577779591083526611328125f,-1.3473000193431976523328330586082302033901214599609375f,0.2503000012531872986443204354145564138889312744140625f,1.009400008110702628982835449278354644775390625f,-0.30520004029870018147363452953868545591831207275390625f,0.8675000274181368098425082280300557613372802734375f,-1.2787999967753886920718287001363933086395263671875f,0.044400021111965060072179767303168773651123046875f,-1.085799986255169269355747019289992749691009521484375f,-0.2180999527364975154597459550132043659687042236328125f,0.139700020688771520127602343563921749591827392578125f,-0.5707000015333292264330111720482818782329559326171875f,-1.0693999928742645710855185825494118034839630126953125f,0.118800009441375920005157240666449069976806640625f,0.762799976778030242030581575818359851837158203125f,0.2357999969840047782554393052123486995697021484375f,0.1800999792456625669956338242627680301666259765625f,-1.0255000182874500769258929722127504646778106689453125f,-0.99409999203085863683782008592970669269561767578125f,-0.21649998164176931236823975268634967505931854248046875f,-0.703600083546340737683522092993371188640594482421875f,-1.69010004412159364761691904277540743350982666015625f,0.62710005183666905903550059520057402551174163818359375f,0.19269999462664111344878392628743313252925872802734375f,-0.243400010204315009332276531495153903961181640625f,-1.2376000398747628583606683605466969311237335205078125f,1.012100062304736258766979517531581223011016845703125f,1.27850005562976054562795980018563568592071533203125f,-0.4817000156819819522979742032475769519805908203125f,-0.1600000146403910417092220086487941443920135498046875f,0.7929000203967095927737318561412394046783447265625f,0.45540001166090371409467252306058071553707122802734375f,-0.816100004160404068898060359060764312744140625f,-1.544100002387165826434056725702248513698577880859375f,-0.58359999149441676991045824252068996429443359375f,-0.42339999165236952283208893277333118021488189697265625f,-0.58459997900724403052663546986877918243408203125f,-1.4942000268951065589106974584865383803844451904296875f,0.33370003049373686554446294394438154995441436767578125f,0.32630001812130171590098370870691724121570587158203125f,-1.359600009368360229444760989281348884105682373046875f,-1.369400024688245398607477909536100924015045166015625f,1.4461000493109228415278266766108572483062744140625f,1.929300061535835109083336647017858922481536865234375f,-0.168700003075599624935421161353588104248046875f,-0.371100025469064842553734706598334014415740966796875f,1.2529000102639198388487784541212022304534912109375f,0.738099985817074699667728054919280111789703369140625f,-0.546100000882148606251575984060764312744140625f,-0.439999994412064499016423724242486059665679931640625f,1.324600013279914723085539662861265242099761962890625f,-0.53909998972117889604049878471414558589458465576171875f,-0.16870000545978580674955082940869033336639404296875f,-0.0906999882340431273775038789608515799045562744140625f,0.3110000336319209690572051840717904269695281982421875f,-1.0206999656587836255994261591695249080657958984375f,0.3656000069394707008285649862955324351787567138671875f,-0.5202000268876554311958670950843952596187591552734375f,0.408700032281875014206207197275944054126739501953125f,0.07940005955696094464002499080379493534564971923828125f,-0.816300058615209156442915627849288284778594970703125f,0.44719999818801792201838907203637063503265380859375f,-0.1657999816358088907009005197323858737945556640625f,0.467800025430321841923841930110938847064971923828125f,0.184599962019920127431760192848742008209228515625f,0.537899982547759947237864253111183643341064453125f,-0.3198000031381844632250022186781279742717742919921875f,-1.394300002303719754337407721322961151599884033203125f,-0.24200001969188489336914926752797327935695648193359375f,1.05690000233650227556836398434825241565704345703125f,-0.3199999893270428774627589518786408007144927978515625f,-0.4198000444889071758325371774844825267791748046875f,0.00259996225833825089779338668449781835079193115234375f,0.8757000385999678115922506549395620822906494140625f,0.34870002153441337355843643308617174625396728515625f,0.15589998371750102279520433512516319751739501953125f,-0.7387000128917395880279173070448450744152069091796875f,-0.38949999905377608033774095019907690584659576416015625f,0.159800010793656355190250906161963939666748046875f,-1.1441000681757937940119518316350877285003662109375f,-1.608400058763475026779588006320409476757049560546875f,0.57590000096559545994523432455025613307952880859375f,-0.93690000406876183802751256735064089298248291015625f,0.308600002372264814454183579073287546634674072265625f,-0.49170003632009107175093731711967848241329193115234375f,-0.12469998254179959218390649766661226749420166015625f,0.818800058987737333637824121979065239429473876953125f,0.5959999811053275919903171597979962825775146484375f,-0.376500082150102333145014199544675648212432861328125f,0.3075000283122069077990090590901672840118408203125f,1.112800039586425260296209671651013195514678955078125f,-1.8010000459253792115532633033581078052520751953125f,-0.76140001591891059984362755130860023200511932373046875f,-0.24740005279183463482439719882677309215068817138671875f,-0.3078000231578943424182170929270796477794647216796875f,-1.645100028978287998171481376630254089832305908203125f,-1.745900029277801746019349593552760779857635498046875f,-0.42649995064735346517181824310682713985443115234375f,-0.481499978661537486601673663244582712650299072265625f,-0.405300019395351540651972754858434200286865234375f,0.50079999892115589776864226223551668226718902587890625f,-0.3499999979883430256677456782199442386627197265625f,0.41440000054836279019809808232821524143218994140625f,-0.6883999595403675897387074655853211879730224609375f,-1.535099993774295068504898154060356318950653076171875f,0.4317999807715420246267967741005122661590576171875f,1.055800010916591258336438841070048511028289794921875f,0.081299968612194017936189993633888661861419677734375f,-0.56340006153881605488464856534847058355808258056640625f,0.6305000685602430987586330957128666341304779052734375f,-0.81419999839663514951126899177324958145618438720703125f,0.5810999989822505096270788271795026957988739013671875f,0.0095999836266040805554666803800500929355621337890625f,0.46900003319978811333612611633725464344024658203125f,-1.180499992191790692430686249281279742717742919921875f,0.4665000269412988842532286071218550205230712890625f,-1.000999994814396387710075941868126392364501953125f,-0.2100999953389166563511025742627680301666259765625f,-0.25170004161000303355422147433273494243621826171875f,-1.2038000345312063199543217706377618014812469482421875f,0.82360004901290029266647252370603382587432861328125f,0.409500022970140375999648085780791006982326507568359375f,-0.734900050333143095571131198084913194179534912109375f,-0.982100032837688985409840825013816356658935546875f,0.991800023575127198682821472175419330596923828125f,0.49780001293197295719750172793283127248287200927734375f,-0.074199999476969313860763577395118772983551025390625f,-0.399300009307265002522768782000639475882053375244140625f,1.196700018624961447955001858645118772983551025390625f,0.37760001687109434609368463497958146035671234130859375f,0.11790000249668997067686859736568294465541839599609375f,1.170100029015541220900331609300337731838226318359375f,-1.317200028884411011631527799181640148162841796875f,0.0715999865360557041782385567785240709781646728515625f,-0.5788000012457370502261255751363933086395263671875f,-0.31590000111460703191568200054462067782878875732421875f,1.75840005967617063475927352556027472019195556640625f,-0.2854000403642660277370168842026032507419586181640625f,-1.10420005762875117483190479106269776821136474609375f,-0.655000002011655535483214407577179372310638427734375f,2.00930006406903327587087915162555873394012451171875f,1.4003999879494311020522445687674917280673980712890625f,0.79249996036291048273625392539543099701404571533203125f,-0.46840004190653450333314822273678146302700042724609375f,0.555900005771219785088987919152714312076568603515625f,1.239799970486014313308942291769199073314666748046875f,-1.2915000051297249594739469102933071553707122802734375f,0.68160006350055335477833295954042114317417144775390625f,-1.1018000015959137360965769403264857828617095947265625f,-0.72279998609125595265822994406335055828094482421875f,0.729100019598007520471583120524883270263671875f,0.37180000342130659074513232553726993501186370849609375f,-0.73209999986886931111484955181367695331573486328125f,-0.4215999827176328818012507326784543693065643310546875f,1.2560999964117993688006436059367842972278594970703125f,1.0582999971330158128779430626309476792812347412109375f,-0.6984000389635554029155173338949680328369140625f,0.2931999909907585522006456812960095703601837158203125f,-0.7526999900072810500972764202742837369441986083984375f,0.2787999801605938188231448293663561344146728515625f,0.5179999616816644003591818545828573405742645263671875f,1.1215999748200180530233183162636123597621917724609375f,0.025899994558096051111562019286793656647205352783203125f,-0.205700012709200186211688787807361222803592681884765625f,-0.17490000283569173689102171920239925384521484375f,-0.72819998376369488024550946647650562226772308349609375f,-0.32949997651576978796583716757595539093017578125f,-0.12219997315406772031565196812152862548828125f,0.4090000210180881889954207508708350360393524169921875f,-0.41930002819448752848785488822613842785358428955078125f,0.2021999942392109783639853048953227698802947998046875f,-0.96659999516010319808856365852989256381988525390625f,1.0905000050440427639131257819826714694499969482421875f,-0.77460000009238694218538512359373271465301513671875f,0.35559999317452295475305845684488303959369659423828125f,-0.4794999805018307359461005034972913563251495361328125f,0.16719999122172601868641095279599539935588836669921875f,-0.5180999829486012675516803938080556690692901611328125f,-0.2494999919570982382932555765364668332040309906005859375f,-0.3868000289767976607180344217340461909770965576171875f,-1.299400013270229248263376575778238475322723388671875f,0.79330001188516607868450591922737658023834228515625f,-0.358699985026567802481878288745065219700336456298828125f,-0.75780001179575950942535200738348066806793212890625f,-0.711900008352845947712239649263210594654083251953125f,-0.61450002345442766849004101459286175668239593505859375f,-0.397199990327655516608729158178903162479400634765625f,0.502200035031140412655759064364247024059295654296875f,0.610300044429302079862509344820864498615264892578125f,0.60310005130767851966311354772187769412994384765625f,-0.259799984996021748884231783449649810791015625f,0.8812999947637318864934741213801316916942596435546875f,-0.21079999564290030633628703071735799312591552734375f,0.3478000081822276712273378507234156131744384765625f,-0.4226999840840697242327905769343487918376922607421875f,-0.1635999859064820594056754998746328055858612060546875f,-0.891300022405386638268964816234074532985687255859375f,0.52449999730288965338331763632595539093017578125f,-0.74419999616146004228767196764238178730010986328125f,-1.4105000200569630663949283189140260219573974609375f,-0.36759999802112564015743600975838489830493927001953125f,1.1575000050663941664197409409098327159881591796875f,0.04540000162124624427661956360680051147937774658203125f,-0.1754000039681786748957392774173058569431304931640625f,-0.0381999910652630614293912003631703555583953857421875f,-1.1769000160455700410011559142731130123138427734375f,0.123700013878941916800613398663699626922607421875f,-0.43589996401071606779709100010222755372524261474609375f,0.08020004000067670890672388850362040102481842041015625f,-0.08300001450628047283686328228213824331760406494140625f,0.53359999168068161790046133319265209138393402099609375f,0.040999993845820537075042011565528810024261474609375f,0.022899998715519842562571284361183643341064453125f,0.137800032433867070125188547535799443721771240234375f,-0.403499971121549894093050170340575277805328369140625f,1.427399952208995959068715819739736616611480712890625f,0.28589995909333210111213929849327541887760162353515625f,-0.777999966561793598884833045303821563720703125f,-0.33989999542236315477339303470216691493988037109375f,-0.194099971467255993928802126902155578136444091796875f,1.0339999638348817967425929964520037174224853515625f,-0.60519998495951288219885100261308252811431884765625f,0.2168000056192276314703804018790833652019500732421875f,-0.314299994257092263350017447010031901299953460693359375f,-1.49479998577833139705717258038930594921112060546875f,-1.1027999789759512783149375536595471203327178955078125f,0.746399965590238689827629059436731040477752685546875f,-1.165100000833720006454541362472809851169586181640625f,-1.70770000319480885764278355054557323455810546875f,1.0593000216938552870971079755690880119800567626953125f,0.4371000076174738069312297739088535308837890625f,-0.906900019826740066974934961763210594654083251953125f,-1.430800032150745693115823087282478809356689453125f,0.07349997983872913476233179608243517577648162841796875f,1.08709999212026531267838436178863048553466796875f,0.39759999206066110044588413074961863458156585693359375f,-0.63530001134872460255564874387346208095550537109375f,-0.08579999858587950001265198807232081890106201171875f,-1.076900050355494631304509312030859291553497314453125f,-1.310100007371603592076780842035077512264251708984375f,-0.822000004306435716472378771868534386157989501953125f,-1.46920000092983205064456342370249330997467041015625f,0.212100008511542537092964266776107251644134521484375f,-0.541299989101290623949580549378879368305206298828125f,1.4560000602304938865927397273480892181396484375f,0.498900035402178421151120346621610224246978759765625f,-0.5240999495625491277905894094146788120269775390625f,-1.64370001738071547237041158950887620449066162109375f,-0.2611000759840020890578671242110431194305419921875f,0.96330001069307424899079705937765538692474365234375f,2.0268000314354903679259223281405866146087646484375f,0.1606999675214295653091767235309816896915435791015625f,-1.512200022029876578244511620141565799713134765625f,-0.490000022873282592428267889772541821002960205078125f,0.71270004625916538731900118364137597382068634033203125f,-1.37460000515878189020213540061376988887786865234375f,1.555799960699677964015563702560029923915863037109375f,0.69879997293353091070144955665455199778079986572265625f,0.02950000810623176761282593361102044582366943359375f,0.23730000760853275920680971466936171054840087890625f,-0.0849999897181987762451171875f,-0.648899955829977503896088819601573050022125244140625f,-0.8999000785708428651332724257372319698333740234375f,0.79659999754428856277854720246978104114532470703125f,1.2677001045227047626440253225155174732208251953125f,-0.78199997532367770958217079169116914272308349609375f,-2.24180000699758519289161995402537286281585693359375f,-0.0662000035226342209426775298197753727436065673828125f,1.118299994301796829176964820362627506256103515625f,-0.0827000353813165389738060184754431247711181640625f,-1.2608000542044646152817222173325717449188232421875f,0.8074000431060799343185863108374178409576416015625f,-0.054199974685162022325357611407525837421417236328125f,1.0294000142574315503907200763933360576629638671875f,0.7957000245556233863908346393145620822906494140625f,0.09739997158050517356286945869214832782745361328125f,0.69239998581111394937437353291898034512996673583984375f,-0.5192999795794488449018899700604379177093505859375f,-0.390400041425973398645510314963757991790771484375f,0.090200010496378268953776569105684757232666015625f,0.4066000092417001443578783437260426580905914306640625f,-1.303700028631090912512036084081046283245086669921875f,-1.0360000044070181157707111196941696107387542724609375f,-0.243300054837763735093858485925011336803436279296875f,-1.4684000138737263174704139601089991629123687744140625f,-0.16520000120177880109650914164376445114612579345703125f,-0.0831000028975308335787985924980603158473968505859375f,1.170600042068958401841882732696831226348876953125f,1.275400016820430248998263778048567473888397216796875f,-0.163799989175796412155250436626374721527099609375f,1.1153000027805557436977323959581553936004638671875f,-1.7529000326156616296202628291212022304534912109375f,0.2106000194936985536742213298566639423370361328125f,-0.5653999888062468670568705420009791851043701171875f,-0.9058000461578377127125349943526089191436767578125f,0.70529999793767927940280060283839702606201171875f,0.6293000034958120547656790222390554845333099365234375f,-0.51219997695386343394829964381642639636993408203125f,-2.34890001513659907317332908860407769680023193359375f,-0.58909998726248691269802293390966951847076416015625f,-0.885500000480561766380560584366321563720703125f,0.9786000413946809306509067027946002781391143798828125f,0.06659999532774101005383471374443615786731243133544921875f,0.3540000068619846285145058573107235133647918701171875f,0.511800022494793172000981940072961151599884033203125f,-0.4740000513792050895744978333823382854461669921875f,0.235899993738531321696427767165005207061767578125f,0.2391000240683549549203235073946416378021240234375f,-0.7592999909788371581953470013104379177093505859375f,0.10700000154972055810276287957094609737396240234375f,-0.48719998537302000585214045713655650615692138671875f,0.33670004146099064001873557572253048419952392578125f,0.5092000167250632092219575497438199818134307861328125f,-0.3261000256180770495717524681822396814823150634765625f,0.047700006175041165246142327305278740823268890380859375f,1.2641999960124490343105208012275397777557373046875f,0.1995000145137311164944549091160297393798828125f,-0.250099999660252814148719835429801605641841888427734375f,-0.67119999372512051394323862041346728801727294921875f,-0.295500000648200256136988173238933086395263671875f,0.08089999642074119368118090278585441410541534423828125f,-0.6329000276982785777590834186412394046783447265625f,-0.054099985772370740022552126902155578136444091796875f,-0.2431999940827491712980190641246736049652099609375f,-0.339600007766485578741821882431395351886749267578125f,0.2102999776348466465236697331420145928859710693359375f,-0.81810000138878802289355007815174758434295654296875f,0.46029997130185329456963927441393025219440460205078125f,-1.0874000413924445052060718808206729590892791748046875f,-0.59500000469386560286721987722557969391345977783203125f,0.1333999839782702423462978913448750972747802734375f,0.53560005279779498987835495427134446799755096435546875f,-0.396300032240151978157882695086300373077392578125f,-0.6972000212103133254260001194779761135578155517578125f,-0.34469997971057875219003108213655650615692138671875f,0.03090003397166818377428398889605887234210968017578125f,-0.437700024205446514002915137098170816898345947265625f,0.8102999879911536584842224328895099461078643798828125f,-0.0882000034332275362203290569595992565155029296875f,-1.5226999961912632830518532500718720257282257080078125f,0.4287000003933911784770316444337368011474609375f,-1.004399983561039544355253383400849997997283935546875f,0.832099992418289247098073246888816356658935546875f,-0.83170000301599589231926756838220171630382537841796875f,0.25160000290870687678079775650985538959503173828125f,0.67099993990361728180005229660309851169586181640625f,1.01399991831183466928223424474708735942840576171875f,-2.03669999839663518059751368127763271331787109375f,1.1771000612616528968601414817385375499725341796875f,-0.048800032836199047636682735173963010311126708984375f,0.82189997268319103795874980278313159942626953125f,-0.921999997973441676180073045543394982814788818359375f,-0.649999973103403849705728134722448885440826416015625f,0.500300022599101001929966514580883085727691650390625f,0.750599953034520428474252184969373047351837158203125f,0.051100005576014517760796707079862244427204132080078125f,-0.3536000261396174959571681029046885669231414794921875f,0.2982000254496937063919403954059816896915435791015625f,0.84460001188293121199279767097323201596736907958984375f,-0.95550003436207819884629088846850208938121795654296875f,-1.0934000024743377732505678068264387547969818115234375f,0.60669998662471746886382106822566129267215728759765625f,-0.794999988898634768474948941729962825775146484375f,1.06209999673962585120534640736877918243408203125f,-1.467700001928209818657933283247984945774078369140625f,0.5677000027477741017634116360568441450595855712890625f,1.15569998819678954049550156923942267894744873046875f,-0.69319999631792283967257617405266501009464263916015625f,-0.30629999264031593231294436918688006699085235595703125f,0.44789999785870282522637353395111858844757080078125f,-0.0155000091791155369946864084340631961822509765625f,-0.573800000500678919479469186626374721527099609375f,-0.65560003081858209128540693200193345546722412109375f,-0.2659999957084651089900262377341277897357940673828125f,0.2584000289052730270356050823465920984745025634765625f,-1.74960007504522874199892612523399293422698974609375f,0.2089000176697968225170143341529183089733123779296875f,-0.21840001039057976850443765215459279716014862060546875f,0.4636000195644800481176162065821699798107147216796875f,0.71910001512765830966600333340466022491455078125f,0.2805000006854536653833065429353155195713043212890625f,0.478000033020972381336832768283784389495849609375f,1.4749999921768901511853755437186919152736663818359375f,-0.16560001502335008893851409084163606166839599609375f,0.155299998086691370957623803406022489070892333984375f,-0.18130000038892102320886579036596231162548065185546875f,0.41070001557767354949390892215888015925884246826171875f,-0.6756000173330303226748583256267011165618896484375f,0.6505000091791155458764706054353155195713043212890625f,0.3641999955654142695493646897375583648681640625f,-2.049200048390030559630758943967521190643310546875f,0.091100007587672227060693330713547766208648681640625f,-0.68439999358206982105201632293756119906902313232421875f,1.238399957491456149227815330959856510162353515625f,0.2600999815367164291757262617466039955615997314453125f,1.017499954868107625571838070754893124103546142578125f,0.4279999868273733909518341533839702606201171875f,-0.42069997839927708582763443700969219207763671875f,-0.468099972331524671886882060789503157138824462890625f,-1.2301999435901649349034414626657962799072265625f,2.2020999541223051920724174124188721179962158203125f,0.220699974077940286321108942502178251743316650390625f,-1.911999956585467064229533207253552973270416259765625f,-0.426499961599708488080295865074731409549713134765625f,0.375099981331825205188579275272786617279052734375f,1.107499960660934590350734652020037174224853515625f,0.0796000023275613699524910771287977695465087890625f,-0.4031999955356122544714025934808887541294097900390625f,0.7914999952018264561814930857508443295955657958984375f,-0.3348000104770065821213620438356883823871612548828125f,0.16250000439584244560364822973497211933135986328125f,0.7545000492334372754754667766974307596683502197265625f,0.56039999487847091952374967149808071553707122802734375f,-1.2942000282734635874248851905576884746551513671875f,0.749700033764541640124434707104228436946868896484375f,1.21150004582852144352500545210205018520355224609375f,-0.1376000167220829961678418840165250003337860107421875f,-0.79430003702342555360615961035364307463169097900390625f,-0.2098999936342236249942061476758681237697601318359375f,1.3184000279739500616216218986664898693561553955078125f,-0.718800025311112644743616328923963010311126708984375f,-0.4646000058293342505066902958787977695465087890625f,0.112400000786781362194233224727213382720947265625f,-0.193900023555755751658580265939235687255859375f,0.741100019359588912948311190120875835418701171875f,0.4667000023081897541743501278688199818134307861328125f,0.73680003203153621171850318205542862415313720703125f,-0.8062000194296239374835977287148125469684600830078125f,0.50890000604689067387198520009405910968780517578125f,-0.4890000387877233567479606790584512054920196533203125f,0.62160005018264097298441583916428498923778533935546875f,0.023499984458089262506774730354663915932178497314453125f,0.023099967861174519612177391536533832550048828125f,-0.004100011551380067231775683467276394367218017578125f,-0.1958999902367590806306907325051724910736083984375f,-0.223200023847818673772280817502178251743316650390625f,-0.8036000509500507149596160161308944225311279296875f,0.264900021424889720034201445741928182542324066162109375f,0.0182999993681907113796114572323858737945556640625f,-0.5593000129207970072542366324341855943202972412109375f,-0.561200012612342646889374009333550930023193359375f,0.2593999927997590049244536203332245349884033203125f,0.368199987041951004584916518069803714752197265625f,1.21230000127553960709292368846945464611053466796875f,-1.549999982267618037212741910479962825775146484375f,0.5454000333607191830509464125498197972774505615234375f,-0.151299985450506380146862284163944423198699951171875f,-0.405000038705766396418539443402551114559173583984375f,-0.969599980124830285177495170501060783863067626953125f,0.21309997903704580579642424709163606166839599609375f,0.983600041711330419502701261080801486968994140625f,-1.163300020527839873096809242269955575466156005859375f,-0.91299999472498871710968160186894237995147705078125f,0.2611000023722649387991623370908200740814208984375f,1.49899998357892005884650643565692007541656494140625f,-0.554900010919570974010639474727213382720947265625f,0.087399994826316884655170724727213382720947265625f,0.0453999993115663524889669133699499070644378662109375f,0.26140001755803854432969046683865599334239959716796875f,-0.2251999860584736889990153940743766725063323974609375f,-0.0597000500440605375018776612705551087856292724609375f,0.483000029295683308561137891956605017185211181640625f,1.1906000787630686144069613874307833611965179443359375f,-0.612900047144294024548116794903762638568878173828125f,-1.18390003290623457843366850283928215503692626953125f,0.8490000371485952879169190055108629167079925537109375f,0.158899985446035840741529909792006947100162506103515625f,0.50400000999122818168274307026877067983150482177734375f,-0.9639000283986332018315579261980019509792327880859375f,0.4713000094041228660302067510201595723628997802734375f,0.25030005147010137012131281153415329754352569580078125f,0.7607000051468613133209828447434119880199432373046875f,-0.41190002498626743232534863636828958988189697265625f,0.904900027012825347583202528767287731170654296875f,0.0410000074803829051006687222979962825775146484375f,0.514000019043683753494633492664434015750885009765625f,-1.2691000543177128889738014549948275089263916015625f,-0.23369996552467231509808698319830000400543212890625f,0.7681000410631302344199866638518869876861572265625f,-0.8522000709056865019874749123118817806243896484375f,-0.4252000253230339144039362508920021355152130126953125f,0.3209000268936161859301137155853211879730224609375f,-0.77640003331601636915593189769424498081207275390625f,-0.03310000295341009035610113642178475856781005859375f,0.534600040586293179245558349066413938999176025390625f,0.63870004736930219291224375410820357501506805419921875f,-0.41540001980066332265550954616628587245941162109375f,0.87330002887248969756228689220733940601348876953125f,-0.49910003602653763010721377213485538959503173828125f,0.65670003550052680196813525981269776821136474609375f,-0.12300001074373734244460365516715683043003082275390625f,0.226000011354684726683217377285473048686981201171875f,1.62230000865161372303191456012427806854248046875f,-1.227799975660443632108353995135985314846038818359375f,-1.92170000718831968100630547269247472286224365234375f,-0.41400000131130099134679767303168773651123046875f,0.62490001822113949714321279316209256649017333984375f,2.222299956274031984548855689354240894317626953125f,-0.81700004670023940178680277313105762004852294921875f,-0.836900010383129000501867267303168773651123046875f,-0.237800001215934753151515224089962430298328399658203125f,0.870700011777877147522985978866927325725555419921875f,0.92430001984983700680231777369044721126556396484375f,-0.716100034186243572520425004768185317516326904296875f,1.189400015598535365057841772795654833316802978515625f,-0.7720999888420099210861735627986490726470947265625f,-0.6071000186443331614327689749188721179962158203125f,-0.5247999786257739884831607923842966556549072265625f,0.2000999998837709181742638975265435874462127685546875f,1.42590004708468942595800399431027472019195556640625f,-0.1447999968051909558397483124281279742717742919921875f,-1.26450006115436597298185006366111338138580322265625f,0.33450001555681219134186221708660013973712921142578125f,1.3729000377938154198176334830350242555141448974609375f,-0.514699994239210756319380379864014685153961181640625f,0.2609999609887598825963550552842207252979278564453125f,-0.30529997443109790911108802902163006365299224853515625f,1.2060000225678084717628735234029591083526611328125f,-0.4207000169560313196370771038345992565155029296875f,-1.0794000180758533957714462303556501865386962890625f,0.6942000139310950945770173348137177526950836181640625f,-1.3661000145725907817251254527946002781391143798828125f,0.14859998165965115912712235513026826083660125732421875f,0.737800030124187600222285254858434200286865234375f,-0.65090002748966213630410493351519107818603515625f,-0.09090000275373455451699555851519107818603515625f,-0.08710002698898389184023471898399293422698974609375f,-0.827600014933943750605749301030300557613372802734375f,0.77980004255175661143084653303958475589752197265625f,-0.128500003419816710703571516205556690692901611328125f,0.171399984961748241829582184436731040477752685546875f,-0.8945000208467248636878821343998424708843231201171875f,-0.5571000054568049364434045855887234210968017578125f,-0.28029996400028422964822993890265934169292449951171875f,-0.50700006245821704897736026396160013973712921142578125f,-0.9565999887526033251816670599509961903095245361328125f,0.336200026805698382048603889415971934795379638671875f,-0.8719999946951861868882360795396380126476287841796875f,-0.08609996359199245663518240689882077276706695556640625f,1.7323999978996809101516873852233402431011199951171875f,-0.497700020778179208491565077565610408782958984375f,1.07070000812709320570093041169457137584686279296875f,-0.411199999201297572426483384333550930023193359375f,-0.347400010770559486417141670244745910167694091796875f,-0.17039996034502991051340359263122081756591796875f,0.162600004875659731595760604250244796276092529296875f,-0.0291999963104725424045682302676141262054443359375f,0.4526000039070847691391463740728795528411865234375f,-0.54979999956190539478484424762427806854248046875f,-1.4054000022172925543628707600873894989490509033203125f,0.140900024062394546575660569942556321620941162109375f,0.345099954845011769322127292980439960956573486328125f,1.880199948880076465940192065318115055561065673828125f,-1.11019999648928635593847502605058252811431884765625f,-0.819199949781597158704471439705230295658111572265625f,-0.2277999594740569566653221045271493494510650634765625f,-0.669100036082415972060744024929590523242950439453125f,1.0820000259876252357571502216160297393798828125f,0.5026999674320222766255028545856475830078125f,0.0890000027641653368437602011908893473446369171142578125f,-0.289199997614324022521259394125081598758697509765625f,-0.2523000097319483270297268973081372678279876708984375f,-0.11009997317343922507149045486585237085819244384765625f,-0.8412000180512666513976682836073450744152069091796875f,0.4606000086531043535842400160618126392364501953125f,-0.6789000126406550261748407137929461896419525146484375f,0.255500004187226126273202453376143239438533782958984375f,-0.406199997077881957974199167438200674951076507568359375f,-0.0394999960362911461952961644783499650657176971435546875f,0.066200001883506665034673233094508759677410125732421875f,0.94930001269727959378741388718481175601482391357421875f,0.2640000033974645443635154151706956326961517333984375f,-1.36480002545267353752933559007942676544189453125f,0.9174000154644252891245059799985028803348541259765625f,0.7144999909698963591608844581060111522674560546875f,-0.18319997482299843483133372501470148563385009765625f,0.404800008539855316058719836291857063770294189453125f,0.27820002612024585797456666114157997071743011474609375f,0.9561000211477279631111514390795491635799407958984375f,-0.31720000146627480575034496723674237728118896484375f,0.94709999446719894677926276926882565021514892578125f,-0.4545000084787609306857802948798052966594696044921875f,1.1132999935567375882072838066960684955120086669921875f,-0.0582000050723550099718295314232818782329559326171875f,-0.032500017508864498250176211513462476432323455810546875f,0.3796999966233964851625160008552484214305877685546875f,-0.39990000242590861834202087266021408140659332275390625f,0.1824999877065416331589631226961500942707061767578125f,-0.903400014190375966194324064417742192745208740234375f,1.24790003052949938222582204616628587245941162109375f,-0.45970001223236345300193761431728489696979522705078125f,0.28590000960826866815978064551018178462982177734375f,-0.5365999831274146369963773395284079015254974365234375f,-0.622600016051530591454366003745235502719879150390625f,0.439800046612322859118648921139538288116455078125f,-0.748900005003809265957670504576526582241058349609375f,0.640700027424097040551487225457094609737396240234375f,-1.090099984908103891712016775272786617279052734375f,0.5474000263795251175480416350183077156543731689453125f,0.2727000011831519632465870017767883837223052978515625f,0.6066999818935989718937662473763339221477508544921875f,0.0593999827042226424822501940070651471614837646484375f,0.67089997351020580484970423640334047377109527587890625f,-0.582699972014129485842204303480684757232666015625f,0.247999994352459562918511437601409852504730224609375f,0.24260002350211085087039464269764721393585205078125f,0.12429997913241397355932349455542862415313720703125f,-0.5764999959468841694842922152020037174224853515625f,0.3926000035345555971844078158028423786163330078125f,-1.1262999422371393620778690092265605926513671875f,0.08329999601542947829102558898739516735076904296875f,-0.61420001899749021578855945335817523300647735595703125f,1.0971000012844793758404193795286118984222412109375f,-0.7192999986156820568794501014053821563720703125f,0.397499989271163922666119105997495353221893310546875f,0.2233999700829383261435623353463597595691680908203125f,0.10350002230703780892184795447974465787410736083984375f,0.2244999991282821127924762549810111522674560546875f,0.38629999878704524007844156585633754730224609375f,0.0280000635311015155792802033829502761363983154296875f,-0.803600035974383342107785210828296840190887451171875f,-0.9783000665724281841306719798012636601924896240234375f,0.859500042378902318063182974583469331264495849609375f,1.9365000135302541384163532711681909859180450439453125f,-1.17789998642206139578547663404606282711029052734375f,-0.9929999975562093794678730773739516735076904296875f,0.6066999896049498630645757657475769519805908203125f,-0.709700005042552817258183495141565799713134765625f,0.1493000031977889108247836702503263950347900390625f,0.482100057983398944116970596951432526111602783203125f,0.8135999923139809464345262313145212829113006591796875f,0.10060001185685407687486758732120506465435028076171875f,1.28060002746582046739831639570184051990509033203125f,-1.4798999945655457732840432072407566010951995849609375f,0.657300016623735050558252623886801302433013916015625f,0.09060004344731620307129560387693345546722412109375f,-0.1823000352501882304068203666247427463531494140625f,-0.70580008072853173128891057785949669778347015380859375f,1.1369000753521927293832050054334104061126708984375f,1.3154000308275224373488754281424917280673980712890625f,-1.0538000076532352267122405464760959148406982421875f,-0.8646000337690120662870185697101987898349761962890625f,-0.05460000016689292579030734486877918243408203125f,0.6380000300034882609878650328028015792369842529296875f,-0.23260003587007549441523224231787025928497314453125f,-1.1833999844253060285126366579788736999034881591796875f,0.447100023636220811340535874478518962860107421875f,0.738899978144467173279963390086777508258819580078125f,0.44159999452680320786157608381472527980804443359375f,1.9223999796271329376651237907935865223407745361328125f,-1.8922999400317674911775611690245568752288818359375f,-1.0296999814987177757075187400914728641510009765625f,-0.8353999262213704923851764760911464691162109375f,1.82820000894665657398263647337444126605987548828125f,-1.0211000142931947465285702492110431194305419921875f,-0.00369997267723132239325423142872750759124755859375f,-0.45859994246959701058585778810083866119384765625f,0.33230001707077061467998646548949182033538818359375f,-0.52080006344318263700188254006206989288330078125f,-0.685899994409084001745213754475116729736328125f,1.107499968111515187274562777020037174224853515625f,-0.02659999985396854782493392121978104114532470703125f,0.28809999565184085668079205788671970367431640625f,0.187100011193752635563214425928890705108642578125f,-1.2904000217378139847568263576249592006206512451171875f,0.676099993878602756325335576548241078853607177734375f,-0.600899970231949964727391488850116729736328125f,0.0142999747365717411184959928505122661590576171875f,0.747800001960992499761005092295818030834197998046875f,-0.367999990701675105952972444356419146060943603515625f,-0.598900009362399732282256081816740334033966064453125f,0.10790000512301956003824443541816435754299163818359375f,-0.1660999990567566531041165944770909845829010009765625f,0.3372999873429538553182283067144453525543212890625f,0.375700003749877442515980874304659664630889892578125f,0.44560002027600986718169906453113071620464324951171875f,0.2054999993257224932818871820927597582340240478515625f,-0.12729999270737180072643468520254828035831451416015625f,-0.9115000102519985603066743351519107818603515625f,-0.187300018560886627483341726474463939666748046875f,-1.8462000479653479789732273275149054825305938720703125f,-0.17359997584819808480460778810083866119384765625f,0.6183000121459365505671712526236660778522491455078125f,-0.362799992871284615603144629858434200286865234375f,1.5351999777883282849444412931916303932666778564453125f,-0.1290999984383578436109019094146788120269775390625f,0.08159999039173138069003243799670599400997161865234375f,0.296200037497282409049148554913699626922607421875f,-0.45850003724545285166414032573811709880828857421875f,1.132100026541948967206963061471469700336456298828125f,-0.1643000154539946411347273169667460024356842041015625f,0.990900029613077659718101131147705018520355224609375f,-0.91319999713748689629255750332958996295928955078125f,0.05160000190287827859236813310417346656322479248046875f,-0.312899998883157837159529890413978137075901031494140625f,1.241000034451484879127747262828052043914794921875f,-1.0035000633597377373007475398480892181396484375f,-0.12009999742507915243550087325274944305419921875f,-1.08539998075962085266610301914624869823455810546875f,0.05519999961853017111934605054557323455810546875f,-0.2030999566853051163661803002469241619110107421875f,0.4042999584198003759638595511205494403839111328125f,0.6881999828696248044934691279195249080657958984375f,0.8807999880433090567066756193526089191436767578125f,-0.036999986499548054297292765113525092601776123046875f,-1.0791000093162050887940495158545672893524169921875f,2.141200012090802573538894648663699626922607421875f,-0.3291000332325690891366321011446416378021240234375f,0.06980002094507253840305338599137030541896820068359375f,-1.599399945674836676090535547700710594654083251953125f,0.0824000007309020343626571047934703528881072998046875f,-0.357199986136704783490358749986626207828521728515625f,-0.94470000534057607666227340814657509326934814453125f,0.7377000564664599746578232952742837369441986083984375f,1.3491000374794008820344970445148646831512451171875f,0.4153999664545044989694133619195781648159027099609375f,-0.9365000692605978116489495732821524143218994140625f,-1.052400000563264104158633926999755203723907470703125f,-1.871599979960917981003376553417183458805084228515625f,0.0799000513017171787311099251382984220981597900390625f,-0.04020004208684024860076533514074981212615966796875f,-0.5733999830842009259868063963949680328369140625f,-0.1009000102043149382780029554851353168487548828125f,-0.455799945947525930733945642714388668537139892578125f,0.79100004339218177307202495285309851169586181640625f,-0.2996999975174663433108435128815472126007080078125f,-2.0008000378131871599407531903125345706939697265625f,-0.6079999939054250290837444481439888477325439453125f,-0.93010002186298379456275142729282379150390625f,-0.0659999983161687620025759315467439591884613037109375f,-1.53399998015165284215299834613688290119171142578125f,-1.0230999863386152082256330686504952609539031982421875f,0.30440005419254223539837767020799219608306884765625f,0.128600006172060243869736950728110969066619873046875f,1.6312999659299851629157274146564304828643798828125f,-0.0731000232934950755492309326655231416225433349609375f,-0.63959995829463078820253940648399293422698974609375f,-0.847599976861477699685565312393009662628173828125f,-0.28489996711015663777288864366710186004638671875f,-0.212499998435377701611059819697402417659759521484375f,-1.097600006961821694773107083165086805820465087890625f,-0.852700001928209605495112555217929184436798095703125f,0.80760002558827448382317015784792602062225341796875f,-1.135300007101893271510562044568359851837158203125f,0.913900029814243453074595890939235687255859375f,-1.0579999987483024170842327293939888477325439453125f,-0.26700001257658012576712280861102044582366943359375f,0.50020000244975104664035825408063828945159912109375f,0.02559998512789596336602926385239697992801666259765625f,0.09150001256167883401104745644261129200458526611328125f,-0.07280001002624636452509321316028945147991180419921875f,0.9910999898180363754107702334295026957988739013671875f,0.085099976302682733830806682817637920379638671875f,-1.284999955892562439885296043939888477325439453125f,0.6750000051781537280248812749050557613372802734375f,0.457999976694583921243975055404007434844970703125f,-0.42749998310580827887150690003181807696819305419921875f,-2.03729999386221205526226185611449182033538818359375f,-0.96350000002980229485416430179611779749393463134765625f,0.51039997334629350067558561931946314871311187744140625f,-0.43179995586797570439330229419283568859100341796875f,0.5371000404000287886674414039589464664459228515625f,0.99080003127530236195497082007932476699352264404296875f,0.6275000031292439217622813885100185871124267578125f,0.30359995327293842759530662078759633004665374755859375f,0.908800019425154204100181232206523418426513671875f,-0.09740003591626866796815420457278378307819366455078125f,0.2520000345557933396634098244248889386653900146484375f,0.558599972011147638539796389522962272167205810546875f,0.789700038048625430064930696971714496612548828125f,0.5129000139147041892329070833511650562286376953125f,0.1230999997496606823688125587068498134613037109375f,-1.322599986025690288471423627925105392932891845703125f,-1.383600010567903382252552546560764312744140625f,-1.443499974139033970033096920815296471118927001953125f,0.67370005165338486818882302031852304935455322265625f,1.058200101967156303572892284137196838855743408203125f,0.818600010083615270417567444383166730403900146484375f,0.3138999935299153776213643141090869903564453125f,-1.20070006840229126510166679508984088897705078125f,0.25840001161992554035151670177583582699298858642578125f,-0.090800018739700671943637644289992749691009521484375f,0.88510000852644454827355957604595459997653961181640625f,0.21699995863437637755311016007908619940280914306640625f,0.95509999332726025489392895906348712742328643798828125f,-0.21700000717490919299734741798602044582366943359375f,0.2268000211909415586575278211967088282108306884765625f,-0.904000012338162406422270578332245349884033203125f,0.596199998679756593134015929535962641239166259765625f,1.588699984598159975490716533386148512363433837890625f,-0.30600000621378409260131547853234224021434783935546875f,-0.4234999993592494238470180789590813219547271728515625f,-0.01469999371767016782541759312152862548828125f,-1.2653999979332091196937426502699963748455047607421875f,-1.6338999787777666039545465537230484187602996826171875f,0.829600017452239768545041442848742008209228515625f,0.487099994206428021925603388808667659759521484375f,1.2331000253796577936782341566868126392364501953125f,-1.6370000182390214149563689716160297393798828125f,-0.900000027380883249605858509312383830547332763671875f,0.83950000613182762521091717644594609737396240234375f,0.3394000180572269709244892510469071567058563232421875f,-0.70890001755803788707765988874598406255245208740234375f,-1.330999991610645860617978541995398700237274169921875f,-0.203200017067790117408776495722122490406036376953125f,-0.6878000047735870481346864835359156131744384765625f,-0.0465000350996855882357294831308536231517791748046875f,-1.047400047408789891534297566977329552173614501953125f,1.09220003454685166133231177809648215770721435546875f,1.026400004761665929464697910589165985584259033203125f,-0.39649998641014150280170724727213382720947265625f,-0.326100040407478974913146885228343307971954345703125f,0.3963000404357917005881972727365791797637939453125f,1.012900027213990750141192620503716170787811279296875f,-0.8707000171422958345601728069595992565155029296875f,-1.008300010171531813085721296374686062335968017578125f,2.15099999690055820877887526876293122768402099609375f,0.111899977339804390652489018975757062435150146484375f,-1.910999984681605923242386779747903347015380859375f,-1.8226999796882263904507226470741443336009979248046875f,0.572700004051625466416908238898031413555145263671875f,-0.0639000213578342357578776500304229557514190673828125f,1.086900014741718489830191174405626952648162841796875f,-0.7578000703945761973301387115498073399066925048828125f,0.544800017890334498105175953241996467113494873046875f,1.3841000180333846714120227261446416378021240234375f,-1.83800003510713594323533470742404460906982421875f,-0.303599981585145339835207778378389775753021240234375f,1.7550999974250789392726801452226936817169189453125f,0.848899996957182434442756857606582343578338623046875f,-0.438800013613700912173953838646411895751953125f,-0.132200011226534908104213172919116914272308349609375f,-0.7744999452233312098314854665659368038177490234375f,0.007700019139051494931891284068115055561065673828125f,1.0848999434918169537667154145310632884502410888671875f,0.4036999705165629226399914841749705374240875244140625f,0.6162000022932885290316562532098032534122467041015625f,-0.076599983909726443442878007772378623485565185546875f,-0.61319998853206592048792344940011389553546905517578125f,-0.0112999935716388277828770014821202494204044342041015625f,0.45059999654591109674584004096686840057373046875f,0.4262999999791385885572481129202060401439666748046875f,-0.0093000192165373363195612910203635692596435546875f,-0.372800048825145058373209394630976021289825439453125f,-0.6049999317526815190149136469699442386627197265625f,-0.492799982067942732300025454605929553508758544921875f,0.067399962639807853292950312606990337371826171875f,1.0025999980211253159723128192126750946044921875f,-0.4857000170305372233059415520983748137950897216796875f,0.756300012458860582142960993223823606967926025390625f,-1.198100004592537626280090989894233644008636474609375f,0.8686000259161001935126478201709687709808349609375f,0.05440004419013877434707637803512625396251678466796875f,0.037399973368644534588156602694652974605560302734375f,0.20749999800696949936451574103557504713535308837890625f,-0.0779999571740628283578189439140260219573974609375f,1.3716000329717996653045020138961263000965118408203125f,-0.62360001399517006603900881600566208362579345703125f,1.176200050982833200663435491151176393032073974609375f,-0.81570001267194758298728629597462713718414306640625f,0.1554000092767180429831341825774870812892913818359375f,0.442000007510184644843320711515843868255615234375f,-1.15210001076534407360441036871634423732757568359375f,-0.638300006520747587046571425162255764007568359375f,-1.0328999900169668624272389934048987925052642822265625f,0.37789999893903736261080439362558536231517791748046875f,-0.76790001156777043522794201635406352579593658447265625f,0.125900003610551092236846670857630670070648193359375f,0.5958999907582995803068115492351353168487548828125f,-1.0339000283241279731782924500294029712677001953125f,0.3140000199377535050615506406757049262523651123046875f,-0.1110999967098234009199586580507457256317138671875f,1.1421000827193268367665268669952638447284698486328125f,-0.2973000123023989971215996774844825267791748046875f,0.98050007839500974426982793374918401241302490234375f,0.1828999768853183383043869980610907077789306640625f,-0.359500006392597715176862038788385689258575439453125f,-1.2278000521779066644967315369285643100738525390625f,-1.955800010171532488101320268469862639904022216796875f,-1.81650001055002263683491037227213382720947265625f,-0.454099968338013137980624378542415797710418701171875f,1.20350000762939401965923025272786617279052734375f,-0.0445999965906143103211434208787977695465087890625f,1.0196999625369909114880329070729203522205352783203125f,0.3764999681562184452587871419382281601428985595703125f,0.29679998255968076392719012801535427570343017578125f,0.1768999883294102293262994862743653357028961181640625f,-0.345599990510940369148329409654252231121063232421875f,-0.6410999938786028362613933495595119893550872802734375f,-0.7884000026047228715242454200051724910736083984375f,-0.4116000171393157103238991112448275089263916015625f,-1.02899999564886091008020230219699442386627197265625f,-0.489399986168742362480088559095747768878936767578125f,0.42150002485513748951007073628716170787811279296875f,-0.261600020119547860275588391232304275035858154296875f,2.14740000764131622190689085982739925384521484375f,-0.418500024318694929803541526780463755130767822265625f,-0.78649997210502675670795724727213382720947265625f,-0.58839996728897148159376229159533977508544921875f,-0.003499996751546774476082646287977695465087890625f,0.301600020417570835462584000197239220142364501953125f,0.217999999959021639739376041688956320285797119140625f,0.4966000023126611040424904786050319671630859375f,-0.367499964460729966475582841667346656322479248046875f,1.30730002429783365158755259471945464611053466796875f,-0.345699962101131408331866623484529554843902587890625f,0.0411999630659825033518472992000170052051544189453125f,-0.6739999628663060349254010361619293689727783203125f,0.4482000025391579267619590609683655202388763427734375f,0.578400021678209252939950602012686431407928466796875f,0.212299971845746338061644564731977880001068115234375f,0.08760000409334922633064479668973945081233978271484375f,0.449900007715820837717046742909587919712066650390625f,-0.05670001549571768162394391765701584517955780029296875f,0.1333999899387361409708319115452468395233154296875f,-0.214799993508309139400580534129403531551361083984375f,0.2024999969452618842069568927399814128875732421875f,0.887700010496377700519587961025536060333251953125f,-0.38380000333190000816330211819149553775787353515625f,0.2938000284403565753876819144352339208126068115234375f,-1.067900046622753507108427584171295166015625f,-0.336299999904631885527805934543721377849578857421875f,-2.39280003340244373788436860195361077785491943359375f,-0.1375999612152571938139544727164320647716522216796875f,-0.1682999921411276744720453280024230480194091796875f,0.9065000191926955164234414041857235133647918701171875f,-1.020200024652481385345481612603180110454559326171875f,0.13730000574588807626952302598510868847370147705078125f,-0.02600003035366516002113712602294981479644775390625f,-0.415800050106644281111556438190746121108531951904296875f,-0.53829999623894675409019328071735799312591552734375f,0.1806000026553868931245006024255417287349700927734375f,0.236700023579597740308599895797669887542724609375f,-0.214800007161498107688402114945347420871257781982421875f,-0.36669998677372905859783713822253048419952392578125f,-0.5040000113695859429441270549432374536991119384765625f,0.4559999980926505713796359486877918243408203125f,-0.664800004628301355324992982787080109119415283203125f,1.733799990516901257109338985173963010311126708984375f,-0.4643000285297631535286200232803821563720703125f,-0.0547000109285118440993755939416587352752685546875f,-0.8056999810069800371792325677233748137950897216796875f,-0.1992999803245065759682574935141019523143768310546875f,-0.77250000551342956622846713798935525119304656982421875f,0.0687000220745799250465779550722800195217132568359375f,-0.474499990299344143185322764111333526670932769775390625f,0.94739998722672513764564428129233419895172119140625f,0.6229999797493224189537386337178759276866912841796875f,-0.2421000222206117769019328989088535308837890625f,0.0184000066280365359716597595252096652984619140625f,-0.63260002692937877810663849231787025928497314453125f,-0.24410000112056717824771112645976245403289794921875f,-0.108699997506290502169434830648242495954036712646484375f,-0.21680003218054810787407404859550297260284423828125f,0.6320000028908250300219151540659368038177490234375f,-1.37320006154775597906336770392954349517822265625f,0.08709997334480246156118710132432170212268829345703125f,1.61990001755058710131152110989205539226531982421875f,-0.0904000015653669830584249211824499070644378662109375f,-1.2132999836847184749188954810961149632930755615234375f,-1.3168000009253619797533474411466158926486968994140625f,0.605100003683567866374914956395514309406280517578125f,-0.2497999674499034039598654999281279742717742919921875f,1.1260000232942406483971353736706078052520751953125f,0.017800034147501531123225504416041076183319091796875f,-0.690200018803774906217540774377994239330291748046875f,0.489700018081068944031386536153149791061878204345703125f,-0.11229999829530701305202455841936171054840087890625f,-0.1743000033468007359260809607803821563720703125f,-1.11980001006722407197457869187928736209869384765625f,0.91210005820691641975628272120957262814044952392578125f,0.0768000394821175547122038551606237888336181640625f,-0.07300003018975331769979675300419330596923828125f,0.177400012409687946046688011847436428070068359375f,-0.797999995283781959898306013201363384723663330078125f,-2.641300015960633462697160211973823606967926025390625f,-0.57159997135549789337716219961293973028659820556640625f,1.731900025209783589019707505940459668636322021484375f,0.071600001046060679499305479112081229686737060546875f,-1.70880002315044432492641135468147695064544677734375f,0.301300058875978660211103488109074532985687255859375f,-0.560600021468103282273887089104391634464263916015625f,0.039200002717971804688801285010413266718387603759765625f,0.4477000210948289815604539398918859660625457763671875f,0.252400010696053556102924630977213382720947265625f,0.128200009989738017424087956896983087062835693359375f,-0.51590000402033331372564362027333118021488189697265625f,-1.18709997364282582310579527984373271465301513671875f,-1.5274999801814548305856078513897955417633056640625f,-0.5376999617695812361262142076157033443450927734375f,-0.425899955964088139381829023477621376514434814453125f,2.3826999531269077436945735826157033443450927734375f,-0.1156000500410794551431337140456889756023883819580078125f,-1.0246999690562486051703672274015843868255615234375f,-0.3810999796479943579896598748746328055858612060546875f,0.793999989986418963638925561099313199520111083984375f,-0.3721999928236012777915675542317330837249755859375f,0.332300008092820231553332632756792008876800537109375f,-0.8829000030741098736797312085400335490703582763671875f,-0.35740001386255026982752269759657792747020721435546875f,0.479300025121122796889494566130451858043670654296875f,1.224800013196468029263996868394315242767333984375f,-1.20540000668764069047256271005608141422271728515625f,0.22949998592212772496878869787906296551227569580078125f,1.06589996915161666635185611085034906864166259765625f,-0.33710000063255396440808908664621412754058837890625f,0.1103000206992026388519434476620517671108245849609375f,0.1151000099048020697267702416866086423397064208984375f,0.904199950806051422347309198812581598758697509765625f,-1.108700008179247742390316489036194980144500732421875f,0.9793000600270931332858026507892645895481109619140625f,-0.130000003390014218229708831131574697792530059814453125f,0.04509998714327867475049060885794460773468017578125f,-0.54279999885037522044939350962522439658641815185546875f,1.07070001161023942159999933210201561450958251953125f,0.6446000045068560613259478486725129187107086181640625f,0.24900003344193155641050907433964312076568603515625f,1.100800029580295102960008080117404460906982421875f,-0.6580999890208243385814057546667754650115966796875f,-0.152999990329147461665115770301781594753265380859375f,0.4211000169754033350955069181509315967559814453125f,0.2034999865815045883010725447093136608600616455078125f,0.134499981842935767417657189071178436279296875f,0.14609997484237091658343388189678080379962921142578125f,-1.02559998969137655677741349791176617145538330078125f,-0.107700013122708160295815105200745165348052978515625f,0.0697000211358069776679258211515843868255615234375f,-0.49990004226788986585461316280998289585113525390625f,-0.8821000247351828971176246341201476752758026123046875f,0.1292999976098536674129491075291298329830169677734375f,1.37820002906322525149107605102472007274627685546875f,1.4970000132657592661900025632348842918872833251953125f,0.8599000292479994644878615872585214674472808837890625f,0.7081999929092821499665433293557725846767425537109375f,-0.4979000219799580850832398937200196087360382080078125f,-0.439599994876980648683684194111265242099761962890625f,-0.5953999527454385543023818172514438629150390625f,-0.4138999462187291644710285254404880106449127197265625f,0.42109999521970664648762294746120460331439971923828125f,-0.2203999888807537110846368477723444811999797821044921875f,0.514999951049686099935343008837662637233734130859375f,0.6125999537646775738863880178541876375675201416015625f,0.30830000283271108418858830191311426460742950439453125f,-0.8097999853685495796895565945305861532688140869140625f,1.38780000101104317167255430831573903560638427734375f,0.81069999680221027649196230413508601486682891845703125f,-0.64450001689791758252567888121120631694793701171875f,0.2270000251680610592330822328221984207630157470703125f,-0.73610003936439716287765122615383006632328033447265625f,0.2866999830111864877579819221864454448223114013671875f,0.15889997525736643257943114804220385849475860595703125f,0.97079999944269623757264753294293768703937530517578125f,-0.473500019028783025731854650075547397136688232421875f,0.375099986584484401674899345380254089832305908203125f,-0.9117000366367402630629612758639268577098846435546875f,0.65980003489628391566412801694241352379322052001953125f,-0.04090001135915521945918271740083582699298858642578125f,-1.148000016033649206548261645366437733173370361328125f,-0.37140002171173713829688267651363275945186614990234375f,-1.16020001720190091276663224562071263790130615234375f,-1.8271000275105235832739936085999943315982818603515625f,0.750799993705749901806711932295002043247222900390625f,0.05060005205273665485066203473252244293689727783203125f,-0.8975000228174028205074819197761826217174530029296875f,0.39650000630319104377718986143008805811405181884765625f,0.781400023406744193010808885446749627590179443359375f,0.13720003454685281152336528975865803658962249755859375f,-0.9694000142574310530108050443232059478759765625f,0.32310001180097447370798136034864000976085662841796875f,0.2720000048279762605574205736047588288784027099609375f,-0.895800064020604747128118106047622859477996826171875f,-0.57120001861006020060784749148297123610973358154296875f,0.53330000996664150836323869953048415482044219970703125f,-0.371100006805360482786682041478343307971954345703125f,-1.57700004751980404904543320299126207828521728515625f,-0.5421000029981133394585413043387234210968017578125f,-1.898599993878602010255463028443045914173126220703125f,0.4511000461816792039826395921409130096435546875f,0.895800073799491780590642520110122859477996826171875f,0.0651000108361241558441179222427308559417724609375f,-0.2837000068008901365601559518836438655853271484375f,0.291500005073845380731967225074185989797115325927734375f,-0.12649995798617463815816108763101510703563690185546875f,0.0808999861948189735727510196738876402378082275390625f,0.230799989682436201832160804769955575466156005859375f,1.15970002679824801816721446812152862548828125f,-0.0379999975562095304582044263952411711215972900390625f,1.092100007542967876617012734641321003437042236328125f,0.227899989402294000928606010347721166908740997314453125f,-0.15269999553933732983068694011308252811431884765625f,-0.40110002723857773876403598478646017611026763916015625f,0.83650002694129899083463897113688290119171142578125f,0.401200029599666496693544104346074163913726806640625f,0.31799999368190778881171354441903531551361083984375f,-0.395000003948807520970376572222448885440826416015625f,-0.702100013577938053543903151876293122768402099609375f,-1.016299995210766748954256399883888661861419677734375f,-0.146299990479647978869337521246052347123622894287109375f,0.328700025166571319079622526260209269821643829345703125f,0.50479996603429311274879864868125878274440765380859375f,0.3006999809511003984852095527458004653453826904296875f,-0.970400010263919199360316270031034946441650390625f,0.6727000118374828474543392076157033443450927734375f,0.622499991357326365459812222979962825775146484375f,-1.02970003156662048837688416824676096439361572265625f,0.8760999890729783690090926029370166361331939697265625f,-0.8556000167369841591380463796667754650115966796875f,0.286399991369247430839095613919198513031005859375f,1.0316000204920765526139803114347159862518310546875f,0.02529996064752315643175961668021045625209808349609375f,0.591800057810545343528474404592998325824737548828125f,0.5117999907553192340259329284890554845333099365234375f,0.11720004795789851215204180334694683551788330078125f,1.4385000183954834707122927284217439591884613037109375f,-0.49450003433227518456760662957094609737396240234375f,-0.12419998427778455862835471634753048419952392578125f,0.26669999318122794562668786966241896152496337890625f,-0.0317999685153360633904640053515322506427764892578125f,0.4671000183463096533387215458787977695465087890625f,0.51690002185702332848649120933259837329387664794921875f,-0.9579999954700468833834747783839702606201171875f,-0.51529999525547065530872714589349925518035888671875f,0.0070000006556512062161345966160297393798828125f,0.0108999809980385720109552494250237941741943359375f,1.6203999655604359730887154000811278820037841796875f,0.4841999672532077969577812837087549269199371337890625f,-0.07340000513791977709843195043504238128662109375f,0.169500040441751043118756570038385689258575439453125f,-1.9825999545097356957512602093629539012908935546875f,1.38300005045533236369692531297914683818817138671875f,-0.83730004448890582580133923329412937164306640625f,1.14479999591112235890477677457965910434722900390625f,-0.28740004757642712007736918167211115360260009765625f,0.00210000612735772307360093691386282444000244140625f,0.19340000975727900112133283982984721660614013671875f,-0.6115999960541724789209183654747903347015380859375f,0.1867000101685523105743413907475769519805908203125f,-0.0631999976962804854707655977108515799045562744140625f,0.527200001578032928506445387029089033603668212890625f,-0.3871999994546173606835282043903134763240814208984375f,0.23010000871270885891561874814215116202831268310546875f,-0.203800001953541976007500124978832900524139404296875f,-0.13259998110830739648235976346768438816070556640625f,-0.036999997340142698476483928970992565155029296875f,0.6583999954894179662545639075688086450099945068359375f,-0.814299995598196435508953072712756693363189697265625f,-1.4834000143021344175764397732564248144626617431640625f,0.9635999898552893849767997380695305764675140380859375f,0.7227000158235430404829457984305918216705322265625f,-0.26129999427944439194249071078957058489322662353515625f,0.51120001037716900071927739190869033336639404296875f,0.287199991966783585439770831726491451263427734375f,-0.6504000186085701074034659541212022304534912109375f,-0.03220002083778439327943488024175167083740234375f,-0.01149998909234994926009676419198513031005859375f,-0.19670000361204120764568870072253048419952392578125f,-0.58780001086443689484184460525284521281719207763671875f,0.1826999980539085743913574333419091999530792236328125f,-0.925900020746886820433019238407723605632781982421875f,0.1010000117272131792134359784540720283985137939453125f,0.01809998291134806702729065364110283553600311279296875f,-0.5916000046968452608098232303746044635772705078125f,-0.6074000488430264521610979500110261142253875732421875f,1.2061000324726105503714279620908200740814208984375f,0.7521000179737804547386303966050036251544952392578125f,0.77519999775588477319843150326050817966461181640625f,-0.9217999998450270293659514209139160811901092529296875f,-0.48070004027634982701755461675929836928844451904296875f,0.1234000300601127975141935166902840137481689453125f,-0.5475000342726712432295244070701301097869873046875f,0.953500039704144430885435212985612452030181884765625f,0.4900000074505808100866488530300557613372802734375f,-0.43959999331235888320179583388380706310272216796875f,0.4953999907433992433425373747013509273529052734375f,0.2646999836966397179338628120603971183300018310546875f,0.1287999846309431095647823894978500902652740478515625f,-1.0920999713703996025770948108402080833911895751953125f,-0.3132999491885304355065500203636474907398223876953125f,1.12409997027516350698306268895976245403289794921875f,-0.112700057435034750596969388425350189208984375f,0.3543000213027003297838746220804750919342041015625f,-0.406899995481968090871305321343243122100830078125f,2.1204999644756323817773591144941747188568115234375f,-1.556999952375889062494707104633562266826629638671875f,-1.651200013692676993315444633481092751026153564453125f,-0.592799942132832047292367860791273415088653564453125f,-0.143900011634826796580455265939235687255859375f,1.5741999902009951028958312235772609710693359375f,0.022699999320507036060234895558096468448638916015625f,2.32140000604689067387198520009405910968780517578125f,-0.9617999985039233923345136645366437733173370361328125f,-1.52599999436736055713481619022786617279052734375f,0.75649998853355626682315460129757411777973175048828125f,0.29299998861551268536373981987708248198032379150390625f,0.26840000487714998911314978613518178462982177734375f,0.67749999254941950521669014051440171897411346435546875f,-0.102599981182814048441542809086968190968036651611328125f,1.014400040408969250194104461115784943103790283203125f,0.058100002208352208299402263946831226348876953125f,-1.9977000320285558387922719703055918216705322265625f,0.72350004070997275817944682785309851169586181640625f,1.3184000084906812855223279257188551127910614013671875f,-0.601799989414214753224996456992812454700469970703125f,-0.2631000042200088984145622816868126392364501953125f,-0.385200016307830850337268202565610408782958984375f,1.21179999954700434727783431299030780792236328125f,-0.8528000387668601689483693917281925678253173828125f,-0.0476999714925880891058795896242372691631317138671875f,0.82000001482665485497136614867486059665679931640625f,-0.5450000053644179498490984769887290894985198974609375f,0.99560000790804636938702287807245738804340362548828125f,-0.3226999942541122212702475735568441450595855712890625f,0.2793000075191258702034247107803821563720703125f,-1.001599988119303841216378714307211339473724365234375f,0.0620999795287844325031301195849664509296417236328125f,-1.2431000398337845869178863722481764853000640869140625f,0.58870006506443051108590225339867174625396728515625f,0.021499956235289108263941670884378254413604736328125f,-0.8954000254631040434105671010911464691162109375f,1.1865000285059219553573939265334047377109527587890625f,-0.4332000242948534918241421109996736049652099609375f,-0.150700014385581226861177128739655017852783203125f,-1.117599995636940235499423579312860965728759765625f,1.025099983417988358525008152355439960956573486328125f,1.122199961233139475780262728221714496612548828125f,-0.029500000655650893133241652321885339915752410888671875f,-0.453299987894296929624715630779974162578582763671875f,1.064299990122020478366948736947961151599884033203125f,-0.631599973329902031338178858277387917041778564453125f,-0.0503999905943868231616988850873894989490509033203125f,-0.8675999507099394492115607135929167270660400390625f,0.203999995015561097488898667506873607635498046875f,0.88089999534040686057778657414019107818603515625f,-0.88919999318122933118502260185778141021728515625f,-1.62389997885227188589851721189916133880615234375f,-0.31810002895593658678308202070184051990509033203125f,1.111100001478195764548217994160950183868408203125f,-0.1259000137805941221813554875552654266357421875f,0.65369998042583432606988935731351375579833984375f,-0.0304999973699450477315764373997808434069156646728515625f,0.59990000726878622838711407894152216613292694091796875f,0.4833000055149199170045903883874416351318359375f,-0.8903000350788234928955944269546307623386383056640625f,0.60580001255571769291918826638720929622650146484375f,-0.34209999219477149612345101559185422956943511962890625f,0.1171999978899955774380714501603506505489349365234375f,-0.6095000210702423704134389481623657047748565673828125f,-0.389200004804134191971343170735053718090057373046875f,0.322800011833012323592129177995957434177398681640625f,-0.7235000433176761447384706116281449794769287109375f,0.1122000110402705530532330158166587352752685546875f,0.06849998266994905549154282198287546634674072265625f,-1.435800005887448715924392672604881227016448974609375f,1.4167000124782322956207281094975769519805908203125f,0.6804999844431875999362091533839702606201171875f,0.034300023388862399542631465010344982147216796875f,-0.541000010572374012696172940195538103580474853515625f,0.27079995805472123482360302659799344837665557861328125f,-1.24739999002069179567797618801705539226531982421875f,0.6315000111460680187747129821218550205230712890625f,-1.1373999858483667679820428020320832729339599609375f,-0.03169998543262497747718953178264200687408447265625f,0.625399995772540240324133264948613941669464111328125f,-0.672000003933906242536977515555918216705322265625f,1.528000035479665452697872751741670072078704833984375f,-1.297000058770180430656182579696178436279296875f,1.54060004586875454180017186445184051990509033203125f,-0.474500035226345318051244248636066913604736328125f,0.22520001705288894555678780307061970233917236328125f,1.6273000298857684953190982923842966556549072265625f,0.02020001329034626369463012451888062059879302978515625f,0.519900014793872600193935795687139034271240234375f,-0.7666000082731245779399387174635194242000579833984375f,0.489999993294477409477849505492486059665679931640625f,-0.404500079818069568915461786673404276371002197265625f,1.178199997249245623009983319207094609737396240234375f,-0.52700001063942902845838034409098327159881591796875f,-0.85899993932247298289439640939235687255859375f,-0.314999965280294613734213271527551114559173583984375f,-0.52289999387264263219776694313623011112213134765625f,-0.33920001598000482090355944819748401641845703125f,-0.24980002198815309810697726788930594921112060546875f,-0.08309997695088355840198346413671970367431640625f,0.6896999713659273112398295779712498188018798828125f,0.910899995452165711640191148035228252410888671875f,1.0968999609112728421678184531629085540771484375f,0.6118999640405178297442034818232059478759765625f,-0.3457999510884288696388466632924973964691162109375f,0.227099995994567649404416442848742008209228515625f,-1.481799931001663850338445627130568027496337890625f,-0.07370000061690806791858676660922355949878692626953125f,0.28850001634657385096716097905300557613372802734375f,0.0872999969542025411328722839243710041046142578125f,0.3468999999895689967388534569181501865386962890625f,0.56919998163282858882894288399256765842437744140625f,-1.42700002568960204740733388462103903293609619140625f,1.384899992516636313411027003894560039043426513671875f,0.1309000092357399669396045283065177500247955322265625f,0.479799980190395825019322728621773421764373779296875f,-0.6980000117123126290863410758902318775653839111328125f,0.3571999966233965206896527888602577149868011474609375f,0.2883999981343745577788695300114341080188751220703125f,-0.00170001620352266957780784650822170078754425048828125f,-0.248299996946752088433640892617404460906982421875f,0.18369999525249003369253841810859739780426025390625f,0.053400004839897263764214585535228252410888671875f,0.0757000004529952974507978069595992565155029296875f,-0.1218999992072582472246722318232059478759765625f,-0.739900034612417822899033126304857432842254638671875f,-0.13009998684525425627356298718950711190700531005859375f,-0.6376000528573992909286971553228795528411865234375f,-0.474199991542100907082613048260100185871124267578125f,0.6846999974429603952330580796115100383758544921875f,-0.2080999476328482256093366231652908027172088623046875f,0.1743999776750795316360154174617491662502288818359375f,-0.39569998320490118448589100808021612465381622314453125f,-0.287399995273351915869852746254764497280120849609375f,0.5271000265792016303834088830626569688320159912109375f,-0.676400016105174639591268714866600930690765380859375f,0.8136000195458528150282973001594655215740203857421875f,0.173399996942281919842798743047751486301422119140625f,-0.18749998450279203865420640795491635799407958984375f,0.2355000047087667525147480773739516735076904296875f,-0.61650001442432422749106990522705018520355224609375f,0.1997999921858308880473487079143524169921875f,-0.74500004105269912457742975675500929355621337890625f,0.741100003191828449189415550790727138519287109375f,-0.697600050026178308826274587772786617279052734375f,1.604699956762789980757588637061417102813720703125f,-0.6172000473618499682970650610513985157012939453125f,0.58540001324415147365698430803604423999786376953125f,-0.283299996462464065416497760452330112457275390625f,-0.2935999777853499637103595887310802936553955078125f,-0.716299988952278976483967198873870074748992919921875f,-0.113599993133545407175688524148426949977874755859375f,-0.5150999782770870893955361680127680301666259765625f,1.194699948790669541409670273424126207828521728515625f,0.08839998703300988580622288282029330730438232421875f,0.8467999817401175022268944303505122661590576171875f,-0.188599995219707494698013761080801486968994140625f,-0.59469998418092728798001189716160297393798828125f,0.1328000039726495717928855810896493494510650634765625f,-1.00700003865361242816334197414107620716094970703125f,0.6025000266730788922586725675500929355621337890625f,-0.4279999988973142155401774289202876389026641845703125f,-0.013600010120868422092144101043231785297393798828125f,1.463200063969195330315642422647215425968170166015625f,-0.6246000488191842947571785771287977695465087890625f,-2.249200006778538263318978351890109479427337646484375f,1.4836000012546783910494241354172118008136749267578125f,1.8583000116243955091022144188173115253448486328125f,-0.36629998470544844479945822968147695064544677734375f,-0.7675999983936545589813249534927308559417724609375f,0.9014999881982799223578695091418921947479248046875f,-0.956100044039637442239154552225954830646514892578125f,0.6937000081419946173610924233798868954181671142578125f,0.441300027788430859754242874259944073855876922607421875f,-0.659800038491189955180971082882024347782135009765625f,-0.593900025641918460905799292959272861480712890625f,0.88270007799863936526207908173091709613800048828125f,-0.33550002851337235032502803733223117887973785400390625f,-0.4317999631881708211267323349602520465850830078125f,1.601100029604137109373596103978343307971954345703125f,-1.413600048565864852889717440120875835418701171875f,0.426899974955617711458444318850524723529815673828125f,1.19290003105103981084766928688623011112213134765625f,0.56199999044835546868625897332094609737396240234375f,-0.1480000423341996107495788237429223954677581787109375f,-1.2969999878406515136930465814657509326934814453125f,0.4007999913215627429963205941021442413330078125f,-0.7633000162810088529141694380086846649646759033203125f,0.25600002074241678950983214235748164355754852294921875f,0.892500047851354505468179922900162637233734130859375f,-0.474300037749857461566449501333408989012241363525390625f,0.348800012533366621259034445756697095930576324462890625f,0.1328000018119812608574648038484156131744384765625f,0.479400025582313649152865764335729181766510009765625f,-1.6124000150918949003653324325568974018096923828125f,-0.65629994938969549167495642905123531818389892578125f,-1.0384000225722775923031804268248379230499267578125f,0.77310004162192313970081158913671970367431640625f,1.8188999135851862121171507169492542743682861328125f,-0.4566000327110291134857789074885658919811248779296875f,0.35439998080432477678414215915836393833160400390625f,0.57089993223398938226864629541523754596710205078125f,-0.37849993815273119679432056727819144725799560546875f,-0.27340001541972203114028161508031189441680908203125f,0.06090002421140550126210655434988439083099365234375f,-1.225400020229070907618051933241076767444610595703125f,-0.25849998747557367817506701612728647887706756591796875f,0.63060002839714301980933441882370971143245697021484375f,0.38349999449774541115942838587216101586818695068359375f,-0.066200002554059000914321586606092751026153564453125f,0.87000002909451712440613846410997211933135986328125f,-0.594100028315186268201841812697239220142364501953125f,0.345699995125829995235022806809865869581699371337890625f,0.282699988666177126628298310606623999774456024169921875f,-0.12799999792873872905829557566903531551361083984375f,0.16629999401867390229625698339077644050121307373046875f,0.35220003879368377841529991201241500675678253173828125f,0.7446999884836371652596653802902437746524810791015625f,-0.7937000283144417522862568148411810398101806640625f,0.4981000149674714716496737310080789029598236083984375f,0.82089999552667125559679561774828471243381500244140625f,-0.2663999927103517517679165393928997218608856201171875f,0.3940000142008066585930237124557606875896453857421875f,-0.41500000840052952977288214242435060441493988037109375f,-0.239400000827759473764899667003192007541656494140625f,-0.4533000248678031862681336860987357795238494873046875f,1.01870000426769280466032796539366245269775390625f,-0.899300017726421430097616394050419330596923828125f,0.88759998449832178124552228837274014949798583984375f,-0.46130001055896219241958533530123531818389892578125f,0.09879999983012599518161778178182430565357208251953125f,0.0181999816030267158595279397559352219104766845703125f,-1.074699988614022050370522265438921749591827392578125f,0.109699969018994369918118536588735878467559814453125f,-0.6652999937839805255634928471408784389495849609375f,0.651700017265230524543540013837628066539764404296875f,-0.3953000076979396038012737335520796477794647216796875f,0.06950000512599974200611541164107620716094970703125f,-0.90800002110004474076276892446912825107574462890625f,-0.86060003763586312430788893834687769412994384765625f,0.036800023221224886782465546275489032268524169921875f,-0.5655000349767511647058881862903945147991180419921875f,0.2021000063136220159520917150075547397136688232421875f,-0.88439998385906193334449199028313159942626953125f,0.7165000136792656348916352726519107818603515625f,0.8021999588489538979274584562517702579498291015625f,-1.23029998810291285593621068983338773250579833984375f,1.43799994011223386536357793374918401241302490234375f,0.3113999547496446584915474886656738817691802978515625f,0.5249999579414730277449052664451301097869873046875f,-1.161000001370906975495245205820538103580474853515625f,0.4344000514358270503834091869066469371318817138671875f,-0.1278000423245135852567955225822515785694122314453125f,0.5221000236734749666567267922800965607166290283203125f,-0.327799993318319327073595559340901672840118408203125f,-0.49819999195933295510485550039447844028472900390625f,-1.55900001949071853601935799815692007541656494140625f,0.6677999954789879399896790346247144043445587158203125f,0.0285999949775634920712263919995166361331939697265625f,0.9494999835565678392157451526145450770854949951171875f,-0.008699998530745489677684645357658155262470245361328125f,-0.05749999048188303429895995577680878341197967529296875f,0.61280001788660898309757385504781268537044525146484375f,-0.6777000310972336460935139257344417273998260498046875f,0.48220004361793444846995271291234530508518218994140625f,-0.2509999936223028527138012577779591083526611328125f,0.041099994325637823067154386080801486968994140625f,0.5538000019907947102382195225800387561321258544921875f,0.464700026947260536047679124749265611171722412109375f,-0.68110002964138993686304957009269855916500091552734375f,0.4516000035047527116915944134234450757503509521484375f,0.52290003021284980899707761636818759143352508544921875f,-0.6271000377550726145869930405751802027225494384765625f,-0.6255000394843523991994516109116375446319580078125f,0.139400002485513763161861788830719888210296630859375f,0.83830001963377043239233898930251598358154296875f,-0.90360001667738032438137452118098735809326171875f,-0.54180001496970664476293677580542862415313720703125f,0.62080001304000609341215977110550738871097564697265625f,-0.96980004217922743947610797476954758167266845703125f,0.48220004739910427016269522937363944947719573974609375f,-0.3779000158518550644970446228398941457271575927734375f,0.747300033611059344451632568961940705776214599609375f,0.029099961781501093582846806384623050689697265625f,1.2404000361919411687949832412414252758026123046875f,-1.2842000323712825515798385822563432157039642333984375f,0.31280001581907246333003058680333197116851806640625f,0.23810004627853686987037917788256891071796417236328125f,-0.57300003920495579112781570074730552732944488525390625f,0.6472000150077044100527245973353274166584014892578125f,0.3733000043414538282604553387500345706939697265625f,0.5506000419571999149326302358531393110752105712890625f,-0.344700031864643818835247657261788845062255859375f,-0.37659998081773482514478246230282820761203765869140625f,0.40859999879598607464004089706577360630035400390625f,0.15999999331310321704080479321419261395931243896484375f,0.42510000766962752205557762863463722169399261474609375f,-0.71050002693012370702518865073216147720813751220703125f,-1.10929997523948475901534038712270557880401611328125f,1.83480003793239543341542230336926877498626708984375f,-0.30150000593066206278081153868697583675384521484375f,1.39369996403455775890734003041870892047882080078125f,-1.3177999982357011532485557836480438709259033203125f,-1.231699990648030595963291489169932901859283447265625f,-1.51679994709491783311250401311554014682769775390625f,0.85690003014579418927354481638758443295955657958984375f,0.78519998609572561054648076606099493801593780517578125f,-0.47460001901686121872359080953174270689487457275390625f,-0.07020000321343522475814324934617616236209869384765625f,0.7490000248551371697658396442420780658721923828125f,-0.3956999697566043749930031481198966503143310546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2996999955058097242499570711515843868255615234375f,0.66869998804405295089736682712100446224212646484375f,0.1904000163733956529910074095823802053928375244140625f,0.9528000095792117374315921551897190511226654052734375f,-1.1342999945178620091468246755539439618587493896484375f,0.77090001189932222391831828645081259310245513916015625f,-0.44390000187456646330019793822430074214935302734375f,0.50019999909698942275326771778054535388946533203125f,-0.298999989837408275406005486729554831981658935546875f,-0.4072000112637874646992486304952763020992279052734375f,-0.377900012946128072144347243010997772216796875f,-1.501900049331040154498850824893452227115631103515625f,0.57409999042898307575200078645139001309871673583984375f,-1.37670003296732890163411866524256765842437744140625f,1.477500019781291751286289581912569701671600341796875f,-0.39199999210610980693303417865536175668239593505859375f,0.67840001616477962897988618351519107818603515625f,0.429899982383847412137356513994745910167694091796875f,-0.465399963846803377265359813463874161243438720703125f,-0.51439997927695546575677099099266342818737030029296875f,-1.367099974574148557593389341491274535655975341796875f,1.1270999913379551315983917447738349437713623046875f,0.065300036606192879418131269630976021289825439453125f,0.0884000152334570021839255105078336782753467559814453125f,0.817400006002187762277344518224708735942840576171875f,-0.38199997811764507815723845851607620716094970703125f,1.246799977120757407789142234832979738712310791015625f,0.464399977451563028552072864840738475322723388671875f,1.1024999843165270707601166577660478651523590087890625f,0.16239998558759671798412682619527913630008697509765625f,0.2396999573960894025503876036964356899261474609375f,-0.988700043606758338654572071391157805919647216796875f,-0.8773000419929626314541337706032209098339080810546875f,-0.4590999871134755494495038874447345733642578125f,0.1102000127315516664339156704954802989959716796875f,-0.74800000545382516747849876992404460906982421875f,0.739500019207597336645676477928645908832550048828125f,0.60099997983872821549766740645281970500946044921875f,-0.99910000641047957969931303523480892181396484375f,-0.7824000147193672649592599555035121738910675048828125f,0.3222999935641885915771354120806790888309478759765625f,-0.7233000399366027455272387669538147747516632080078125f,-1.45960005943626214275354868732392787933349609375f,-0.242000016227365222221123985946178436279296875f,-0.5588000240445136768130396376363933086395263671875f,-0.52290000877380382604542319313623011112213134765625f,-0.2055000139474874032430307124741375446319580078125f,0.519300045442582014487697961158119142055511474609375f,2.08220007746219692990052863024175167083740234375f,-0.549700001689791495351755656884051859378814697265625f,0.239199998769163979517315965495072305202484130859375f,-0.143600012207031113575794734060764312744140625f,-0.41670003721416060127324954009964130818843841552734375f,1.23210003578066817908620578236877918243408203125f,-0.44970002311021095575682693379349075257778167724609375f,0.14159997996091799876694494741968810558319091796875f,-0.5205000219568611807829938697977922856807708740234375f,1.392100019314884917775998474098742008209228515625f,-1.2013000528596347038501335191540420055389404296875f,1.001300033730268435050447806133888661861419677734375f,-0.80610002794638280221306558814831078052520751953125f,0.56140003041029018504559644497931003570556640625f,0.8195000458806760423868809084524400532245635986328125f,-1.6401000218629828708571949391625821590423583984375f,1.00720002897754312698452849872410297393798828125f,-0.660100028121471726905156174325384199619293212890625f,0.3975000076554715544574492014362476766109466552734375f,-0.816300045576692223647796708974055945873260498046875f,0.337299995464086510565948628936894237995147705078125f,0.4456000267952677784677462113904766738414764404296875f,-0.8559000196665527138151219332939945161342620849609375f,-0.9889000108152636858704909172956831753253936767578125f,0.43549998429417602352486937888897955417633056640625f,-0.3210000335946683325261119534843601286411285400390625f,-0.84789998809844269178626063876436091959476470947265625f,0.716899974657593919147302585770376026630401611328125f,-0.413800016407668724838231355533935129642486572265625f,1.0288999962493772244442880037240684032440185546875f,-1.030600032234192386937365881749428808689117431640625f,-1.562100020730495142373683847836218774318695068359375f,1.31890001491308250791689715697430074214935302734375f,0.715900069920719861471525291563011705875396728515625f,-0.0794999910444020851940649663447402417659759521484375f,-0.592500014640390926246027447632513940334320068359375f,1.0936000350803138925215307608596049249172210693359375f,0.3600000211596494636978604830801486968994140625f,-0.4234000354617837302839689073152840137481689453125f,-2.27260002335309962973042274825274944305419921875f,0.1502999862030154343273125050473026931285858154296875f,0.35620005388855968675443364190869033336639404296875f,0.048800002586841362273162303608842194080352783203125f,0.59740000548064664531722200990770943462848663330078125f,-1.166799993549287872696140766493044793605804443359375f,0.3029999656677244690428096873802132904529571533203125f,-0.0142999756306405600980724557302892208099365234375f,0.383400044178964005681109483703039586544036865234375f,0.54430001586377618760792529428726993501186370849609375f,0.9597000363349916796096295001916587352752685546875f,-0.7379000296354292220968318360974080860614776611328125f,-1.0980999836564055982535137445665895938873291015625f,0.24439996880888958941113742184825241565704345703125f,-0.63949998888373382754934937111102044582366943359375f,0.7201000066637988084039534442126750946044921875f,-0.936200001939385462179643582203425467014312744140625f,0.05989995757341404214457725174725055694580078125f,-0.2435999705396592052153437180095352232456207275390625f,0.8613000151038165963512938105850480496883392333984375f,0.2184999742507927056323069336940534412860870361328125f,-0.29210000851154382939256493045832030475139617919921875f,-0.2923000206470487416510195544105954468250274658203125f,0.399400000809133626233915492775849997997283935546875f,0.86520003282949320411177041023620404303073883056640625f,-1.371799979430436966509887497522868216037750244140625f,0.3044999837614597026913543231785297393798828125f,0.10309996622204753435880775214172899723052978515625f,-0.9851999998047944462342684346367605030536651611328125f,1.34290000907182704992237631813623011112213134765625f,-0.523299987559020518546049061114899814128875732421875f,0.64990000570416395220263439114205539226531982421875f,-0.72980001252591630844790415721945464611053466796875f,-0.79140008342117074136012888629920780658721923828125f,0.96869999793469929727507405914366245269775390625f,0.2310999981254344692871427469071932137012481689453125f,-1.377499988935887653695999688352458178997039794921875f,-0.30089996967315768561235245215357281267642974853515625f,0.46210000266283746128692655474878847599029541015625f,-0.05900004597753305546348201460205018520355224609375f,-0.15189999324679426262463266539270989596843719482421875f,-1.489399973018466827312522582360543310642242431640625f,0.786099981138109971112726270803250372409820556640625f,0.00679997991472447438354720361530780792236328125f,0.6170000075101855774306613966473378241062164306640625f,-0.33010005874335723863310931847081519663333892822265625f,-0.52740000481009463584314289619214832782745361328125f,0.26739997701942908037153756595216691493988037109375f,-0.835499987274407995840874718851409852504730224609375f,0.059999994784593724261867464520037174224853515625f,0.00089998269304614186836488443077541887760162353515625f,-1.3593000438407070173951751712593249976634979248046875f,0.47930001284629086821809096363722346723079681396484375f,-0.14660000039413534356214086074032820761203765869140625f,-1.01290002598464479177664543385617434978485107421875f,0.2776999726474276286580789019353687763214111328125f,1.226200029301643734669369223411194980144500732421875f,-0.45379997606277466815072330064140260219573974609375f,0.8434000295758250498323604915640316903591156005859375f,0.476700049433112393870715095545165240764617919921875f,-0.011399973711370581241908439551480114459991455078125f,0.6232000219106677008085171109996736049652099609375f,1.0438000068336723558104495168663561344146728515625f,0.7677000708460806066568693495355546474456787109375f,-0.50709999681413098215898571652360260486602783203125f,0.427300020945072134281872422434389591217041015625f,0.1945000000596046607626021796022541821002960205078125f,-0.5116000213488940406847405029111541807651519775390625f,-0.24590000226944697114817017791210673749446868896484375f,-0.030200000065564946982732408287120051681995391845703125f,0.06090002121254844968945008076843805611133575439453125f,0.14910001950487539179590612548054195940494537353515625f,-0.19460000761747320296990437782369554042816162109375f,0.2154000069297847819171920491498894989490509033203125f,-0.881500023066997595577731772209517657756805419921875f,0.1510000183954840480282655335031449794769287109375f,0.430399953266978219090788115863688290119171142578125f,0.52140003139749147020864938895101658999919891357421875f,0.35660001810789054843553458340466022491455078125f,0.155200011409074800639018576475791633129119873046875f,0.83400000041723387766978703439235687255859375f,-0.1393999788485469348842116232845000922679901123046875f,0.93699999380111620439492980949580669403076171875f,-0.36799999073892830647736218452337197959423065185546875f,0.46419999536052281019493648273055441677570343017578125f,0.473899994070082986041825279244221746921539306640625f,0.02250001866370465730682326466194353997707366943359375f,-0.369300028622895848418039577154559083282947540283203125f,0.4876999648615718552235875904443673789501190185546875f,-0.083399964308738194773695795447565615177154541015625f,0.321800070141256444600230679498054087162017822265625f,0.6152999719351528451483090975671075284481048583984375f,-0.89710001409947792705423808001796714961528778076171875f,0.1408000084578990840356027547386474907398223876953125f,-2.0787000197276466195717148366384208202362060546875f,-0.0326999647498131817968669565743766725063323974609375f,-0.33720000098198621429901322699151933193206787109375f,-0.2911000348567955864353962169843725860118865966796875f,-0.580300001029669498819885120610706508159637451171875f,-0.7810999609470368199026779620908200740814208984375f,1.06759997656345273497890957514755427837371826171875f,0.53020005344897480625832031364552676677703857421875f,1.66379998220950309217869289568625390529632568359375f,-0.0870999713703990152691147841323981992900371551513671875f,0.962299988795816574338459759019315242767333984375f,0.532400043553114432626216512289829552173614501953125f,0.87669999735355386150104095577262341976165771484375f,0.8941000076770777571510961934109218418598175048828125f,-0.49280000628232922821325701079331338405609130859375f,-0.0506000619620092795258869955432601273059844970703125f,-1.7025000208616258845495394780300557613372802734375f,0.773100003921985745591882732696831226348876953125f,0.19500002309680031231664543156512081623077392578125f,-0.16409999169558242559929794879280962049961090087890625f,0.72290000581219804320909361194935627281665802001953125f,0.25860000604167565629865066512138582766056060791015625f,0.50040000745132573456430691294372081756591796875f,-1.20189997253417946154741002828814089298248291015625f,1.176699982508271791203924294677563011646270751953125f,-1.1666000136747953774829511530697345733642578125f,1.1740000356957320803985567181371152400970458984375f,-1.2112000095576040958889052490121684968471527099609375f,0.856799998764693260255853601847775280475616455078125f,-0.62409998472929029134093070751987397670745849609375f,-0.7539000538796190031831656597205437719821929931640625f,1.15560001822710045615849594469182193279266357421875f,-1.1440000258423388412865051577682606875896453857421875f,1.45730003584623357681948618846945464611053466796875f,-0.07269997953921514266539816162548959255218505859375f,0.7537999931991095081684761680662631988525390625f,0.57630001761838778318036702330573461949825286865234375f,-0.1637999986752873837758670561015605926513671875f,0.1980999942734837926394675378105603158473968505859375f,0.0916000181823972692285451557836495339870452880859375f,0.29180001303255576061701503931544721126556396484375f,0.13070007331967337105282922493643127381801605224609375f,0.8795999954357738292998192264349199831485748291015625f,-1.36319999691396898100492762750945985317230224609375f,0.84120001659840337726592451872420497238636016845703125f,-0.1133999476939437922595743657439015805721282958984375f,0.5706000025063751035503400999004952609539031982421875f,1.00470000653266833268162372405640780925750732421875f,-0.42770004350245027779919837485067546367645263671875f,-0.586300023485720966931467046379111707210540771484375f,-0.68250003024935690376651109545491635799407958984375f,-0.423900000533460996621215599589049816131591796875f,-0.962099990779161640119809817406348884105682373046875f,-0.94240001362040637378214569253032095730304718017578125f,0.243500035662204183939394397384603507816791534423828125f,0.27350000651180728095113181552733294665813446044921875f,-0.1530000136680900180152775647002272307872772216796875f,0.2302999720096590152706994558684527873992919921875f,0.399600019538401962648777043796144425868988037109375f,1.0396999451398851732619732501916587352752685546875f,0.803499999731778213885036166175268590450286865234375f,0.445099987143277697754228938720189034938812255859375f,-0.858500005692243828292475882335565984249114990234375f,1.33059999527931172025319028762169182300567626953125f,0.451400038903951639213119051419198513031005859375f,0.9094000026345252507553595933131873607635498046875f,0.7855000380873686793847809894941747188568115234375f,0.80190000233650238214977434836328029632568359375f,0.93749999843537779042890178970992565155029296875f,0.03190001500248929033887179684825241565704345703125f,-1.06950003492832212970142791164107620716094970703125f,0.62239998469352730836590126273222267627716064453125f,0.46510002305507658348915356327779591083526611328125f,0.430699991661310122026407043449580669403076171875f,-0.9229999668598178885758898104541003704071044921875f,0.812599965462088125178752306965179741382598876953125f,-0.3204999382495881121712955064140260219573974609375f,-0.0021000336572527533718357517500407993793487548828125f,0.206500021725893656565631317789666354656219482421875f,0.8766000146806238735308625109610147774219512939453125f,-0.8823999764978882609511856571771204471588134765625f,1.576900032697618581067899867775849997997283935546875f,-2.1340000513792034553262055851519107818603515625f,0.807800011795758887700458217295818030834197998046875f,-0.1904000019192704229453738662414252758026123046875f,-0.5544000338710846076395455384044907987117767333984375f,1.3435000207424163676250827847979962825775146484375f,0.3956000242993236160060632755630649626255035400390625f,-0.1811999556899070995541478623636066913604736328125f,0.6364999936744570874225246370770037174224853515625f,-1.2288000404357912742625558166764676570892333984375f,0.60940000121891468864987473352812230587005615234375f,0.1686000338137139120675556114292703568935394287109375f,0.0877999874323602824688350665383040904998779296875f,0.9227000139981498261221304346690885722637176513671875f,-0.347100034514069566426996971131302416324615478515625f,-0.898899962013960163176307105459272861480712890625f,0.01980000820457927801498954067938029766082763671875f,-1.45279996463358340719196348800323903560638427734375f,2.03270001042187242745740149985067546367645263671875f,-0.53219996883273079646414771559648215770721435546875f,0.356600025111436735869574476964771747589111328125f,0.946600003467499728770917499787174165248870849609375f,-0.87010002789795393152161295802216045558452606201171875f,-0.767900004526973045670956707908771932125091552734375f,-0.73379998120367517788764644137700088322162628173828125f,0.52480002206265918829330985317938029766082763671875f,1.1769000063598156202715472318232059478759765625f,-0.353800018531084337070069523178972303867340087890625f,0.49730000909864890967782002917374484241008758544921875f,0.22380001480579370110035597463138401508331298828125f,-0.66740000160634505022017037845216691493988037109375f,0.55000002756714838625384800252504646778106689453125f,-1.120000009760260706315193601767532527446746826171875f,0.53379996488690295564083498902618885040283203125f,0.41570006087720390297590711270458996295928955078125f,1.103000009924173685504911190946586430072784423828125f,-0.19190001038312931580520626084762625396251678466796875f,-0.591900013512372868973443473805673420429229736328125f,0.32690000501871108173901347981882281601428985595703125f,-0.76140003547668388961255914182402193546295166015625f,-0.101600009241700650619577572797425091266632080078125f,0.23859997614622141526297127711586654186248779296875f,0.3395999877989293036506524003925733268260955810546875f,-0.9005999887228011147044526296667754650115966796875f,0.114300004166364654878407236537896096706390380859375f,0.0949000037483872238563975543002015911042690277099609375f,0.1450000084936622357645319425500929355621337890625f,-0.535100018566101898187525875982828438282012939453125f,0.9120999852657316608883775188587605953216552734375f,0.91550003628060216964712481058086268603801727294921875f,-1.6238999880909918971383376629091799259185791015625f,0.99429997993335106354351182744721882045269012451171875f,-1.385800014120339707091034142649732530117034912109375f,-0.61470002460032713287318983930163085460662841796875f,0.749400002932548492395881112315692007541656494140625f,-0.02340000793188867778127359997597523033618927001953125f,0.0149000515997403937262788531370460987091064453125f,-0.29450002436712352515968404986779205501079559326171875f,0.77260003646612229744050637236796319484710693359375f,-0.389599995435774337781964504756615497171878814697265625f,-0.061799989712238101446928340010344982147216796875f,0.0914999987035989281736192424432374536991119384765625f,-0.245599985481798566144817641543340869247913360595703125f,-0.2550000156462193245943126385100185871124267578125f,0.93719998898655187957729140180163085460662841796875f,-0.63019999440312390870388981056748889386653900146484375f,-1.5866000084593896435336546346661634743213653564453125f,0.55580003721714010822552154422737658023834228515625f,0.117400057820975955991116279619745910167694091796875f,1.552199968758225079312751404359005391597747802734375f,0.264599975505470930148277375337784178555011749267578125f,-0.144300007146596964702212062547914683818817138671875f,-0.9435999981254343804693007768946699798107147216796875f,-0.104800009396672777484127436764538288116455078125f,1.3082999540314073083635548755410127341747283935546875f,-0.710600013011693842912563923164270818233489990234375f,0.38899999676644803070502121045137755572795867919921875f,-0.130600020909309222361116553656756877899169921875f,-0.8985000052452090102406145888380706310272216796875f,0.09490000056326408639506553299725055694580078125f,0.2286000259906055820380288423621095716953277587890625f,-0.552700008037686263406840225798077881336212158203125f,-0.585500000983476098781466134823858737945556640625f,-0.49880004632175001244576151293586008250713348388671875f,-1.3474000207543372908958190237171947956085205078125f,0.4037999885052439008603641923400573432445526123046875f,-0.03180000882297806530374373323866166174411773681640625f,-0.34209997777789868944608997480827383697032928466796875f,0.86029997294098148330476760747842490673065185546875f,0.20429999138861842933323487159213982522487640380859375f,0.85340000085383582728582041454501450061798095703125f,-1.0731999938219785040161013967008329927921295166015625f,-0.20380004281997710080531760468147695064544677734375f,0.8824000215739020713812124085961841046810150146484375f,-0.3264999625682829531569950631819665431976318359375f,0.4684999870657922116379268118180334568023681640625f,-0.3676999936766922605357876818743534386157989501953125f,1.0386000183910131244857666388270445168018341064453125f,-0.586800018098950548761649770312942564487457275390625f,0.507699996265768671577234272263012826442718505859375f,-1.1135000196807087746719844290055334568023681640625f,0.891100014442205434761490323580801486968994140625f,0.22060000019669490090024055461981333792209625244140625f,-0.56029998150914928078947241374407894909381866455078125f,0.72459998621568078025489967330940999090671539306640625f,-0.68719999680966115107594305300153791904449462890625f,0.33619998620003432865388504069414921104907989501953125f,-0.175099991911649599529710030765272676944732666015625f,0.198000026259572248221729751094244420528411865234375f,-0.54910003300905163570178046938963234424591064453125f,-0.038999981958419305261287490793620236217975616455078125f,-0.300500014841556861711069359444081783294677734375f,0.2164999887943268586099065942107699811458587646484375f,0.062599998132884859813884759205393493175506591796875f,-1.1674000029474498330017695479909889400005340576171875f,0.26729997318685050355924204268376342952251434326171875f,0.045800000634789395093093844479881227016448974609375f,0.9267999560728663599462606725865043699741363525390625f,-0.672400009503960394141586220939643681049346923828125f,-0.42229996424615456884765762879396788775920867919921875f,-0.10140000880360611201780329793109558522701263427734375f,-0.64189997935146114738103051422513090074062347412109375f,0.79909999832659928120648373806034214794635772705078125f,0.1824000355809918261229540803469717502593994140625f,-0.1310000065490601883766430546529591083526611328125f,-0.27460004513114599777878765962668694555759429931640625f,0.588100031116604693437466266914270818233489990234375f,0.97890003543943249209036139291129074990749359130859375f,-0.029100005888938795806097914464771747589111328125f,0.355999984383583267799622262828052043914794921875f,-1.222600004987417943169702994055114686489105224609375f,0.5148000173687929947874408753705210983753204345703125f,-0.336699991057812564321238824049942195415496826171875f,0.512199968348443945842518587596714496612548828125f,-0.70680002763569349522043694378226064145565032958984375f,0.69950001205503919976536053582094609737396240234375f,-0.785600014986097772151651952299289405345916748046875f,-0.4373999561950567827040003976435400545597076416015625f,0.007399994677304899681757888174615800380706787109375f,-0.8524000130057327595523020136170089244842529296875f,1.4518000297963615796703606974915601313114166259765625f,-0.7365000149458651446110479810158722102642059326171875f,1.1899000744730245937574864001362584531307220458984375f,-0.4964000400587920136530328818480484187602996826171875f,0.406500003695487777122252737171947956085205078125f,0.045400005197524961886301753111183643341064453125f,0.85969997732639313881009002216160297393798828125f,-0.745100000107287829820279512205161154270172119140625f,0.8494999950677151190348013187758624553680419921875f,-0.1122000171124941658717943937517702579498291015625f,-0.4541000358402731418294706600136123597621917724609375f,-1.2478000138446685429727267546695657074451446533203125f,-0.21370001987665876441724321921356022357940673828125f,-0.96779999611228684219810247668647207319736480712890625f,-0.3961999721556905829089600956649519503116607666015625f,0.852800010901688665398978628218173980712890625f,-1.0287999724864957951098176636151038110256195068359375f,1.621200075569749987636214427766390144824981689453125f,-0.2534999797642225605187604742241092026233673095703125f,1.0070000135451551148690896297921426594257354736328125f,1.232800029937923369516283855773508548736572265625f,0.703799999308586166080203838646411895751953125f,0.0557000008255243983512627892196178436279296875f,0.2642000033140181614044195157475769519805908203125f,0.182700032103061626997941857553087174892425537109375f,0.0261999540507797945565471309237182140350341796875f,-0.04080000361502189587525890601682476699352264404296875f,-0.64900002470612516702885841368697583675384521484375f,-0.1699999912828207282444736847537569701671600341796875f,-0.05090004276335235999795258976519107818603515625f,-1.0061000456228856858587050737696699798107147216796875f,-1.215100029090046884761022738530300557613372802734375f,0.7749999924004076223837955694762058556079864501953125f,-1.437600011730193472203609417192637920379638671875f,-0.6617000060707329822662359219975769519805908203125f,0.7381000060081479574591867276467382907867431640625f,1.628400048798322785614800523035228252410888671875f,1.016700031775236379161242439295165240764617919921875f,-0.4298000093966722889859966016956605017185211181640625f,0.655399994170665411985510218073613941669464111328125f,0.3028000000238417754871989018283784389495849609375f,-1.1629000512421134860829852186725474894046783447265625f,0.902900017565489321214045048691332340240478515625f,0.818000066176058027167528052814304828643798828125f,-0.335200066450239031468072425923310220241546630859375f,-0.71820001453459225704278878765762783586978912353515625f,1.0465000100284811423989594914019107818603515625f,0.697100025685131630126534219016321003437042236328125f,0.0227000103101133721139603949268348515033721923828125f,1.061200080561639058629452847526408731937408447265625f,-1.7665000194907189978721362422220408916473388671875f,0.815100038254261249903720454312860965728759765625f,-0.828600033366679866730919457040727138519287109375f,-1.466500070527196886160936628584749996662139892578125f,-0.182199982690810724506036422098986804485321044921875f,-0.4510000167936087933640010305680334568023681640625f,0.0720999801248307647938418085686862468719482421875f,1.30719999818801824176262016408145427703857421875f,0.517800001141428989370751878595910966396331787109375f,0.5422000287353998526640452837455086410045623779296875f,1.1259000473827127652981516803265549242496490478515625f,1.612100017005205554454505545436404645442962646484375f,0.7948000122651459609102175818406976759433746337890625f,0.67670001668781054604551172815263271331787109375f,1.22820004660934234408387055736966431140899658203125f,-0.6404000163733962747159011996700428426265716552734375f,0.068499998390674221582230529747903347015380859375f,-0.1300999760419132567079714135616086423397064208984375f,0.0669999796450131412939299480058252811431884765625f,-1.0066000398635870372032741215662099421024322509765625f,-1.404600039356946883373211676371283829212188720703125f,0.5925000377744442658922707778401672840118408203125f,0.7288000401377683346026969957165420055389404296875f,-0.010499988913535818202404925614246167242527008056640625f,-0.70010001053810100302143837325274944305419921875f,-1.72870001320838895253473310731351375579833984375f,-0.95810000243186976831566425971686840057373046875f,0.92750002510845686032325829728506505489349365234375f,-0.5053999611645940692739031874225474894046783447265625f,0.56739998551309067664760732441209256649017333984375f,0.288599978716671312639618918183259665966033935546875f,-0.0772000057131052574987961634178645908832550048828125f,-0.05910004560053305766587072866968810558319091796875f,0.4343000472307212422862221501418389379978179931640625f,0.58830000428557394087647480773739516735076904296875f,0.27470000303089658899580172146670520305633544921875f,0.8447999979972831141594724613241851329803466796875f,-1.17289997457563810456804276327602565288543701171875f,-0.67120000694990178402576930238865315914154052734375f,0.160099988335371090641956470790319144725799560546875f,-0.84479996581077632100686969351954758167266845703125f,-0.60599999570846563301529386080801486968994140625f,1.378600062721968555479179485701024532318115234375f,-0.06890007541179699046551831997931003570556640625f,0.303599977971612589033867379839648492634296417236328125f,-0.4731999676704405910498962839483283460140228271484375f,-0.58970001290291573781132683507166802883148193359375f,-0.1931999922201039776581410478684119880199432373046875f,-0.4448999642044293434395285657956264913082122802734375f,-0.1109000119552019381075069759390316903591156005859375f,0.6602000005125991943799590444541536271572113037109375f,-1.254900003319978196714146179147064685821533203125f,-1.753600060039758989915981146623380482196807861328125f,1.47920001180767979320762606221251189708709716796875f,0.3361999848961827463966756113222800195217132568359375f,0.94389998898506188851342812995426356792449951171875f,0.866300008696317291878585820086300373077392578125f,-0.45569998599886929468993912450969219207763671875f,0.469500058174133982902276329696178436279296875f,2.070400037533044468318621511571109294891357421875f,-0.13280001052916023951411261805333197116851806640625f,-0.7656000419199464612773908811504952609539031982421875f,0.28580001568496271602271008305251598358154296875f,0.4995000482648619399839162724674679338932037353515625f,1.1936000308334826502942860315670259296894073486328125f,0.9662000125378364945305520450347103178501129150390625f,0.09920002659708193615273330578929744660854339599609375f,-0.3219000237941747055714358793920837342739105224609375f,0.0952000593721873400454569491557776927947998046875f,0.477900049304962326601753375143744051456451416015625f,1.714900042062998064551493371254764497280120849609375f,1.2547999902486797196843326673842966556549072265625f,-0.102999997481703076118719764053821563720703125f,-0.2428000018119806924232761957682669162750244140625f,-0.09880000698268387449019201085320673882961273193359375f,-0.2764999949783086830024103619507513940334320068359375f,-0.776300048594177116711989583563990890979766845703125f,-0.76429999168664231046221857468481175601482391357421875f,0.405600019698590108507829654627130366861820220947265625f,-0.31370000449120938323943619252531789243221282958984375f,-0.50730003107786192373396261245943605899810791015625f,0.32170001314878415854536797269247472286224365234375f,0.461600008794665062072937189441290684044361114501953125f,-0.277799967092275057467531951260752975940704345703125f,0.3784000368028881400306318028015084564685821533203125f,0.455300003898143845759705072850920259952545166015625f,-0.326799980622529684382016057497821748256683349609375f,1.06349999213218637805766775272786617279052734375f,0.3901000372856859854664435260929167270660400390625f,0.607100013875960797804509638808667659759521484375f,-0.058899979448318706687359735951758921146392822265625f,0.1901999972343446643208153545856475830078125f,-0.3659999939203262275810857317992486059665679931640625f,0.95610002768561219710363729973323643207550048828125f,-0.51609999890774493902512176646268926560878753662109375f,0.52610003725960918785631292848847806453704833984375f,-0.457400017494708155396665461012162268161773681640625f,-0.2502000145196916491840966045856475830078125f,0.04010002216100705396684134029783308506011962890625f,0.66010001314580402098641798147582449018955230712890625f,-0.856100021743773975657632036018185317516326904296875f,-0.784900010100006539914829772897064685821533203125f,1.1314999699443575309487641789019107818603515625f,-0.495100055688620699356761178933084011077880859375f,1.485000055432319499004734097979962825775146484375f,-1.267900053179264574509943486191332340240478515625f,1.8445000149607668760154410847462713718414306640625f,-0.4031000396847730371518991887569427490234375f,0.7968000207066534557043269160203635692596435546875f,-0.7844999932795762020987240248359739780426025390625f,0.59950003634393256302104191490798257291316986083984375f,-0.7291000256702309112455395734286867082118988037109375f,-0.5124000302165752085414851535460911691188812255859375f,0.12130000351816420944839336470977286808192729949951171875f,-0.26040000951886188573780600563623011112213134765625f,-1.660700051265955767831883349572308361530303955078125f,-0.4992000176191329074981695157475769519805908203125f,0.352299995017051781331929305451922118663787841796875f,0.31129998366236666385020726011134684085845947265625f,2.051900018951297255398458219133317470550537109375f,-0.022100020283460253267548978328704833984375f,0.853500015527009825433424339280463755130767822265625f,-0.22720004751086353422806496382690966129302978515625f,1.803499977827071720781759722740389406681060791015625f,0.09979997028112475021544014452956616878509521484375f,0.271099994102120600558691876358352601528167724609375f,1.0482999856591224130397677072323858737945556640625f,0.80699999987333959072799416389898397028446197509765625f,1.40619996660500845564456540159881114959716796875f,-0.00099998985975979781670730517362244427204132080078125f,1.3797999865978958222711980852182023227214813232421875f,-0.081599999853968707697049467242322862148284912109375f,1.2041999970555306020969510427676141262054443359375f,-0.02470000161528584925463292165659368038177490234375f,-0.6231000188231465841681711026467382907867431640625f,-1.121100003415346435531319002620875835418701171875f,-1.08420002864599229752684550476260483264923095703125f,0.72600003482401376242449941855738870799541473388671875f,-0.2217999956727023658231701119802892208099365234375f,-0.21019999961853008674239617903367616236209869384765625f,0.97829999709576309641079205903224647045135498046875f,0.2446000040598215463649722778427531011402606964111328125f,0.3147999991521237550529122017906047403812408447265625f,-0.6867999840497969188390925410203635692596435546875f,-1.510400012685357840069855228648521006107330322265625f,0.375899994036555273879685046267695724964141845703125f,-0.3688999885007733237074489807127974927425384521484375f,0.1575999890804291592161234802915714681148529052734375f,-1.022100066775084581394139604526571929454803466796875f,1.1450000494718555188455866300500929355621337890625f,-0.501200004416703759346773949800990521907806396484375f,1.1465999999284743893213089904747903347015380859375f,1.0843000339686874600175769955967552959918975830078125f,0.75439996865987790641838728333823382854461669921875f,0.2563000289618979632422224312904290854930877685546875f,0.15579996673464702183764529763720929622650146484375f,-0.6752000480845576912969363547745160758495330810546875f,0.925900034679472128118504770100116729736328125f,-0.547600021471828402042092420742847025394439697265625f,0.46490002118274598164049393744789995253086090087890625f,-0.679300040674210237057195627130568027496337890625f,-0.8422999913662672089031957511906512081623077392578125f,0.516300001245736606136915725073777139186859130859375f,-0.6467000328928238506165371290990151464939117431640625f,-0.26690002308636973094024824604275636374950408935546875f,-0.869600003147124933633449472836218774318695068359375f,-0.160300004866719358886939517105929553508758544921875f,0.402600028270482912517991280765272676944732666015625f,1.132300015655160141392343575716949999332427978515625f,1.03010000576972959862587231327779591083526611328125f,0.198099949979782419262619441724382340908050537109375f,-0.6938000332832334748900393606163561344146728515625f,0.95500000387430261650223428659955970942974090576171875f,-0.20649999067559787757630829219124279916286468505859375f,0.09630000778362168833979239934706129133701324462890625f,-0.447500006351620294164916913359775207936763763427734375f,0.12890000094324338686391229202854447066783905029296875f,0.87479997773170481423221644945442676544189453125f,-0.03360001541078094788872476783581078052520751953125f,1.18039995453357704491281765513122081756591796875f,0.8325999828219412535190713242627680301666259765625f,-0.13320000984072688510195803246460855007171630859375f,0.813500011056661254116306736250407993793487548828125f,0.794599968278408397281964425928890705108642578125f,-0.214099979737401024948439953732304275035858154296875f,1.84819998793602024989013443700969219207763671875f,-0.2920999898850917819714823053800500929355621337890625f,1.3190999993324279415674027404747903347015380859375f,0.3102000039398671304979870910756289958953857421875f,-0.3790000016093255652549487422220408916473388671875f,0.3111000108659267571198370205820538103580474853515625f,0.2417999798774719266702959430404007434844970703125f,0.1150999846100806411897110592690296471118927001953125f,0.6538000079512595874575708876363933086395263671875f,0.1454000105619428495629108510911464691162109375f,-1.08670002849698033031700106221251189708709716796875f,1.27820004038810708379969582892954349517822265625f,-0.4769999791979782077788740934920497238636016845703125f,0.03400000578165052189660855219699442386627197265625f,-0.1136999831080434120167410583235323429107666015625f,-1.16150000175833678639492063666693866252899169921875f,-0.68459999971985752154068904928863048553466796875f,0.367699943590164224360705702565610408782958984375f,-1.103600069576502473722712238668464124202728271484375f,1.893000031530856830386255751363933086395263671875f,-0.741200028407573707767141968361102044582366943359375f,1.3225000202655792236328125f,0.591300024789572109540358724188990890979766845703125f,-1.02649999788403523126589789171703159809112548828125f,-0.646500009506940642722838674671947956085205078125f,-0.05410001795887975362120414501987397670745849609375f,-0.28330000562667834174135350622236728668212890625f,-1.25839997444152817962503831950016319751739501953125f,-0.575900007298588700876962320762686431407928466796875f,-0.5737999844074253008585628776927478611469268798828125f,0.69369999927580305865149057353846728801727294921875f,-0.60200002092123039432181030861102044582366943359375f,-0.3697000010192397212449577637016773223876953125f,-0.4238000070571903421523529686965048313140869140625f,0.072299978029727984818464392446912825107574462890625f,-0.92150000384449892720795105560682713985443115234375f,0.155799986478686758317735439050011336803436279296875f,-0.16549999025464057211820545489899814128875732421875f,1.1200000052154066576548530065338127315044403076171875f,-0.774000026941299434923848821199499070644378662109375f,-1.21050001096725434734935333835892379283905029296875f,-0.238899963802098991294542429386638104915618896484375f,-0.079199993032216919885968309245072305202484130859375f,-0.09130000512003899615365298814140260219573974609375f,0.488400013408064825881638171267695724964141845703125f,0.0076999999165536792133934795856475830078125f,0.4676000122517347534767395700328052043914794921875f,-0.340700006711482927812539855949580669403076171875f,0.4131999729603526017029935246682725846767425537109375f,0.5160999904513356995039430330507457256317138671875f,0.20170001523494729411822845577262341976165771484375f,1.31819997825026558047056823852472007274627685546875f,-0.5857000070095061960984139659558422863483428955078125f,-0.41240001553893090857627612422220408916473388671875f,-0.432500003054737902630222379229962825775146484375f,-0.1887000194668768671135694603435695171356201171875f,-0.13660001646876285263942918390966951847076416015625f,-0.0722000032544141134849269292317330837249755859375f,0.04619999204874059017811305238865315914154052734375f,-0.8471999665975573634568718262016773223876953125f,0.29600000554323191437333662179298698902130126953125f,0.02020001124143622206474901759065687656402587890625f,-0.01040000236630422847383670159615576267242431640625f,-0.930000029802322547567428046022541821002960205078125f,0.120200008261203805659533827565610408782958984375f,1.0842000591933729225502247572876513004302978515625f,-2.24719999207854304046350080170668661594390869140625f,-1.4727000404477121886515078585944138467311859130859375f,0.54519999991655421212044529966078698635101318359375f,-0.1143000022292135486878805750166065990924835205078125f,0.68769999619126309653438511304557323455810546875f,1.06510002722740182434790767729282379150390625f,-0.04919998707175299301752602332271635532379150390625f,-0.733800018084048932820451227598823606967926025390625f,0.8957000380039215059468915569595992565155029296875f,1.0726000188827509163047579932026565074920654296875f,-0.9479999921917912075741696753539144992828369140625f,-0.0499000136017802020660383277572691440582275390625f,-0.3337000222980979202702656039036810398101806640625f,0.670499999195337093027546870871447026729583740234375f,0.7058000103950501369354242342524230480194091796875f,0.62970003909170646494430911843664944171905517578125f,-0.332200002360343749074900188134051859378814697265625f,-1.01509999764859770721159293316304683685302734375f,-0.0566999860286718782020898288465104997158050537109375f,0.521000030502676736432476900517940521240234375f,0.8063000219583511540832887476426549255847930908203125f,0.18420000212192544353229095577262341976165771484375f,-0.2588999900281425681214386713691055774688720703125f,-0.2748999857366081212006747591658495366573333740234375f,-0.415299989295006444223190555931068956851959228515625f,-0.643499998912215431801087106578052043914794921875f,-0.043800006461143681235625990666449069976806640625f,-0.26939999727010732755161370732821524143218994140625f,-0.3444999973773956725153766456060111522674560546875f,0.621500023290515013485446615959517657756805419921875f,-0.440399996033310703325014401343651115894317626953125f,-0.65079999892115569792849782970733940601348876953125f,-0.40570000879764567258689567097462713718414306640625f,-0.59150002261996259544929444018634967505931854248046875f,-1.34739999154806167069864386576227843761444091796875f,0.85399998473375993146561313551501370966434478759765625f,-0.3378999989017843486038827904849313199520111083984375f,-0.22409999091327204023826880074921064078807830810546875f,1.14139994934797339709575680899433791637420654296875f,0.84289998921602915515904896892607212066650390625f,0.283899987718462920582851438666693866252899169921875f,-1.07930000535845760367692491854541003704071044921875f,0.4458999896407132013109730905853211879730224609375f,0.40769999946951873681655342807061970233917236328125f,-0.4407999370813371342592290602624416351318359375f,0.8369999896287918517145953956060111522674560546875f,0.04949999946355820856780383110162802040576934814453125f,0.025600014203786880528923575184307992458343505859375f,-0.0589000164031985296020366149605251848697662353515625f,0.343499998912215165347561196540482342243194580078125f,0.9555000272095204305600191219127736985683441162109375f,-0.4694000048696995719410551828332245349884033203125f,0.994200049507618199839953376795165240764617919921875f,-1.5361000042349102212568823233596049249172210693359375f,-0.94870000441670343516165075925528071820735931396484375f,0.6697999808236956287288421663106419146060943603515625f,-0.506899999617040020183367232675664126873016357421875f,-0.2765999940052628058850814340985380113124847412109375f,0.048600028449298005028822444728575646877288818359375f,0.151299963843822382614234811626374721527099609375f,0.4144999693632112069963113754056394100189208984375f,-0.5665999686360354559155894094146788120269775390625f,0.530999978125095140057965181767940521240234375f,-0.26670003415942200586385979477199725806713104248046875f,0.6220000132471323883720515368622727692127227783203125f,1.0158000273823739600942417382611893117427825927734375f,-0.07529997431933832530148720252327620983123779296875f,0.705200019884109519097137308563105762004852294921875f,-0.767200008469819838552439250634051859378814697265625f,-0.5340999952346081602883032246609218418598175048828125f,0.27040000072717640389186044558300636708736419677734375f,0.0464000125661490070427817045128904283046722412109375f,-0.02729999874234201318046189044252969324588775634765625f,-0.45269998982101655737864120965241454541683197021484375f,0.53149999184906449034571096490253694355487823486328125f,0.324199976044892590465451576164923608303070068359375f,-1.565000042244792499701588894822634756565093994140625f,-0.7041000216841695191760663874447345733642578125f,-0.18309999587535852327846441767178475856781005859375f,-0.1900999959349629619964616722427308559417724609375f,-0.55100000410899496738892366920481435954570770263671875f,-0.17400005078315672335520503111183643341064453125f,-1.2152000012949109031268335456843487918376922607421875f,0.87829998208284365546205663122236728668212890625f,-0.6844000014983120383504910932970233261585235595703125f,-0.0257999805927280334572060382924973964691162109375f,-0.48490001110583513810325939630274660885334014892578125f,-0.095699996578692658744103027856908738613128662109375f,-1.33399999898299537193224750808440148830413818359375f,0.681299935010075241592630845843814313411712646484375f,-1.1638999814972279533975552112678997218608856201171875f,1.6795000000596047584622283466160297393798828125f,0.91650006268546035936850557845900766551494598388671875f,-0.446500008910894141678227242664434015750885009765625f,0.82640000776052457975850984439603053033351898193359375f,-0.5088000340282921030166107811965048313140869140625f,-0.29809997188448900118373785517178475856781005859375f,1.5623000083163385287576829796307720243930816650390625f,0.2380999769508840291365459052030928432941436767578125f,0.372199953559042118200750337564386427402496337890625f,-0.296999993428588293653547225403599441051483154296875f,0.527399958951770830850591664784587919712066650390625f,-0.277699991795420597640031701303087174892425537109375f,1.2699999904632568448192841970012523233890533447265625f,-0.035900033524632224413153380737639963626861572265625f,0.6733999366298324407154041182366199791431427001953125f,0.3041000075280668735189237850136123597621917724609375f,0.028399982115626887235748654347844421863555908203125f,-1.3385000131428237413189208382391370832920074462890625f,-1.85889998723417448900363524444401264190673828125f,-0.162599964940547625502631490235216915607452392578125f,0.0904000434935101626621190007426775991916656494140625f,0.6208000247001645988120799302123486995697021484375f,-0.8256000048160547777342799236066639423370361328125f,1.001099999690056474577204426168464124202728271484375f,-0.3067999188572161362031920361914671957492828369140625f,-0.601300020393729095502521886373870074748992919921875f,0.193500007331370849072982309735380113124847412109375f,-0.1882999871492383459070651952060870826244354248046875f,0.971100046181678777656998136080801486968994140625f,-0.62560001092553108037463971413671970367431640625f,-0.917899977630376451287474992568604648113250732421875f,-0.65440001120269275514118589853751473128795623779296875f,0.7493000152677298242309689157991670072078704833984375f,-0.226699982824921264512596508211572654545307159423828125f,1.426200007545948000853286430356092751026153564453125f,1.17700002174079454420052570640109479427337646484375f,-0.203299995121360321803649640060029923915863037109375f,0.47980003070533339126768623827956616878509521484375f,1.095200026515126268122912733815610408782958984375f,0.28979998428821573952518519945442676544189453125f,-0.66889999163001778725856638629920780658721923828125f,0.9114999867826696355876947563956491649150848388671875f,0.373700033921003649250991429653367958962917327880859375f,-0.20590000670254227088662446476519107818603515625f,-0.27750001192092899060526178800500929355621337890625f,0.78739997903108616128520225174725055694580078125f,-0.64479999229758966361458760729874484241008758544921875f,0.121699961143731716362026418210007250308990478515625f,0.97930005320981183825068683290737681090831756591796875f,-1.1014999966174363610349473674432374536991119384765625f,1.154100024627149334577325134887360036373138427734375f,-1.29180002435743812583268663729541003704071044921875f,0.831400039071590324368798974319361150264739990234375f,-1.3891000480592250454492386779747903347015380859375f,0.33260000953227264464118206888088025152683258056640625f,0.48340000036954844375713946647010743618011474609375f,0.0729000068366527642638175166212022304534912109375f,0.8097999985188251148571225712657906115055084228515625f,-0.8441000174373385345205633711884729564189910888671875f,-1.861900036534667179921598290093243122100830078125f,0.2937000331014398835094425521674565970897674560546875f,1.27620001469850574693509770440869033336639404296875f,-0.6169000038266181373813878963119350373744964599609375f,-0.4680000310093159754387670545838773250579833984375f,0.248800009143352429674678205628879368305206298828125f,-1.6175000086054203141117113773361779749393463134765625f,0.3998000311523679517478058187407441437244415283203125f,0.5061000278532510332496485716546885669231414794921875f,0.7485000273734334630404418930993415415287017822265625f,-0.5648999786213033669213245957507751882076263427734375f,1.2203000123173002311460777491447515785694122314453125f,0.4990000344663865217142983965459279716014862060546875f,-0.60559998581707485953984360094182193279266357421875f,1.66960002400875140438074595294892787933349609375f,-0.59600000956654586303784526535309851169586181640625f,0.99029997484087939341179662733338773250579833984375f,0.16120000411868051770625243079848587512969970703125f,0.58149996599555020537763994070701301097869873046875f,-0.7838000098884110311558970352052710950374603271484375f,1.13519997801184668873020200408063828945159912109375f,0.912300005298852578761170661891810595989227294921875f,-0.2959999887049207245581783354282379150390625f,-1.448799974647163590901755014783702790737152099609375f,1.2276000153064732245411505573429167270660400390625f,0.339899991846083882052198532619513571262359619140625f,0.5099000079393380957526460406370460987091064453125f,0.370200047376751939509631483815610408782958984375f,1.63780001969337529743597769993357360363006591796875f,-0.6942000154584644366906331924838013947010040283203125f,0.46470001875162125770657439716160297393798828125f,-1.128999977469444360167472041212022304534912109375f,1.00649998488277159225390278152190148830413818359375f,-1.8170000340342529199233467807061970233917236328125f,0.8141999969065187858774379492388106882572174072265625f,-1.44160001304149698597711903857998549938201904296875f,0.765199988554417842578914132900536060333251953125f,-1.06109999372959151742179528810083866119384765625f,-0.09430000684857320347731501897214911878108978271484375f,-0.5203000721454618116013079998083412647247314453125f,0.38440002817139007351698865022626705467700958251953125f,-0.025499986529350593400522484444081783294677734375f,1.100699992480873579125955075141973793506622314453125f,-1.8029999966621392815113722463138401508331298828125f,0.6858000019945207359484129483462311327457427978515625f,-0.00850003147125288904817352886311709880828857421875f,0.228800011769682176687723540453589521348476409912109375f,-0.0527000109434130337149326805956661701202392578125f,-0.13959998899102199487742836936376988887786865234375f,0.48890004210770054982049259706400334835052490234375f,1.5001000220865012835247398470528423786163330078125f,-0.04369996612072046815455905743874609470367431640625f,0.690000013262033196070888152462430298328399658203125f,0.44790002345144674933408168726600706577301025390625f,0.53440000911653051662852931258385069668292999267578125f,0.4343999950349328731391551627893932163715362548828125f,0.840099996232986701016898223315365612506866455078125f,-0.8372000013545146668292318281601183116436004638671875f,-0.532299962532520876123953712522052228450775146484375f,-1.1693999385595332807241675254772417247295379638671875f,1.756699968370795783556559399585239589214324951171875f,-0.40259997622817689233443161356262862682342529296875f,-0.024899984917045259180667926557362079620361328125f,-0.76850002979487197496410999519866891205310821533203125f,0.34850002759695097864778290386311709880828857421875f,0.39140002873390977722323214038624428212642669677734375f,-0.27470003532916342781078355983481742441654205322265625f,-0.9817999996960169450943567426293157041072845458984375f,0.191899989521503666622948003350757062435150146484375f,-1.34189998967051504763503544381819665431976318359375f,0.35869998906850764086584604228846728801727294921875f,0.5368999852001667960621489328332245349884033203125f,0.789100021684169039559719749377109110355377197265625f,1.08629997889399465549331580405123531818389892578125f,0.199799985554814707455761890741996467113494873046875f,-1.013399960061907467689934492227621376514434814453125f,0.7600999814808364618556879577226936817169189453125f,-0.4989999939352269819892171653918921947479248046875f,-0.654200024175644490043168843840248882770538330078125f,1.2506999786973000965417668339796364307403564453125f,0.194600017824769100371895547141321003437042236328125f,0.84220000891685486976712127216160297393798828125f,0.04509999910145967572105973886209540069103240966796875f,-0.979000024016946301941288766101934015750885009765625f,1.036499969087541384027417734614573419094085693359375f,-1.12150000399351146285198410623706877231597900390625f,1.397100005233287589589963317848742008209228515625f,0.9378999995723358296118021826259791851043701171875f,-0.519399932599068137051290250383317470550537109375f,0.14929997175633979988873534239246509969234466552734375f,0.8310999669075016527131083421409130096435546875f,-0.833300000318140110522335817222483456134796142578125f,1.266300014358758829757789499126374721527099609375f,-0.62130003628209296007156581254093907773494720458984375f,1.33300001782178778597653945325873792171478271484375f,0.64790001868307622867604322891565971076488494873046875f,0.1801000352740285137542741722427308559417724609375f,0.64989999871924519769805783653282560408115386962890625f,-0.739000021576881493956534541212022304534912109375f,-0.10010000807940977463061926755472086369991302490234375f,0.23409994620978780144326947265653870999813079833984375f,0.08989999847710183900062475004233419895172119140625f,-0.3599000098764892907610146721708588302135467529296875f,-0.91910001177489786261531889977050013840198516845703125f,-2.29519993159473045807317248545587062835693359375f,0.18029998556971538192783555132336914539337158203125f,0.6028999746501442213997279395698569715023040771484375f,-0.11889999285936359019189012542483396828174591064453125f,-0.571199987019598065529635277925990521907806396484375f,-0.617200022849440532723974683904089033603668212890625f,-0.5865999764218934320325615772162564098834991455078125f,-0.629400018429756347160264340345747768878936767578125f,-1.3668000251770024977560069601167924702167510986328125f,-0.775899967363476772419517146772705018520355224609375f,0.2123000120416282943125452220556326210498809814453125f,0.7856000086903573986774063087068498134613037109375f,0.82949999991059275572524711606092751026153564453125f,-0.178499964341520378496852572425268590450286865234375f,0.47700001719594009585989624611102044582366943359375f,1.0502000099003321320623172141495160758495330810546875f,-1.335499960005282815700411447323858737945556640625f,-0.311000010386109249083119721035473048686981201171875f,0.986200001883506871536155813373625278472900390625f,0.337000025093555422017743694595992565155029296875f,0.07189996150732014257300761528313159942626953125f,0.78389999576508895273718735552392899990081787109375f,-0.231300050866604323118735919706523418426513671875f,0.67709999100267825866694693104363977909088134765625f,-0.5751999957442279765018611215054988861083984375f,-0.713100019568205123476900553214363753795623779296875f,0.1534999861717221136814259807579219341278076171875f,-0.298700015592574974476747229346074163913726806640625f,-0.960399996852874693331614253111183643341064453125f,-0.7495000027418132759748914395458996295928955078125f,0.68350004195794522576790086532128043472766876220703125f,-1.24360004647970168178972016903571784496307373046875f,-0.611900011686981049052747039240784943103790283203125f,0.2246999779224392312926283921115100383758544921875f,0.38990003951117435665452148896292783319950103759765625f,0.264800027203559551480793743394315242767333984375f,0.95259998267292900209213257767260074615478515625f,-1.433699984896183110549827688373625278472900390625f,-0.6807000658877198073781755738309584558010101318359375f,1.3550000539422040191084306570701301097869873046875f,-0.3761000194899739579312836212920956313610076904296875f,0.53290002814531334252023953013122081756591796875f,-0.63640001994222383796540043476852588355541229248046875f,1.2347999830961224887460048194043338298797607421875f,-0.50799999338388446279424215390463359653949737548828125f,0.18610000863075271126945153810083866119384765625f,-0.01869997431635839202357374233542941510677337646484375f,0.5233999918386353922272746785893104970455169677734375f,0.872200026723742638523617642931640148162841796875f,0.67500002525746871651080027731950394809246063232421875f,0.530099968740344262840835654060356318950653076171875f,-0.6297000271335242427284129007603041827678680419921875f,-1.292599962556362225285511158290319144725799560546875f,-0.8347999611914165285497801960445940494537353515625f,1.782399993783235547795129605219699442386627197265625f,1.463500024430452572943295308505184948444366455078125f,-1.0372000479578975529193485272116959095001220703125f,-0.602699970076978619459850960993207991123199462890625f,-0.461800043058395726092157929087989032268524169921875f,0.3979999460726972859703209906001575291156768798828125f,-0.626399973115324915085011525661684572696685791015625f,-0.3146000132799147142037554658600129187107086181640625f,-0.165800009500980394250291283242404460906982421875f,-1.2886999843001358811989121022634208202362060546875f,0.53479999151527923828552957274951040744781494140625f,-0.665300027218461043077013528090901672840118408203125f,0.2022999974757428987004459486342966556549072265625f,2.2245000429004431197199664893560111522674560546875f,-0.415599990361929183535494303214363753795623779296875f,-0.165000052005053188253214102587662637233734130859375f,-0.37940003135651390042681896375142969191074371337890625f,-1.7850000463426116681375788175500929355621337890625f,0.071800000962614962674024354782886803150177001953125f,-0.4765000175163149886969904400757513940334320068359375f,-0.7476999982774252817563365169917233288288116455078125f,-0.615800005999207034079745426424778997898101806640625f,1.20830001195967273730502711259759962558746337890625f,-0.468599932485818992944359706598334014415740966796875f,-0.67210000687241533423588180085062049329280853271484375f,-0.17209999331235881214752225787378847599029541015625f,0.7520999849677085791199715458787977695465087890625f,-0.09969997369423511113240010672598145902156829833984375f,0.4248000151336182028671828447841107845306396484375f,1.179600010560452716390500427223742008209228515625f,-0.57409994777440953583891314337961375713348388671875f,-0.1828000389531256875130793559947051107883453369140625f,-0.299700055855513181057858673739247024059295654296875f,-1.093400023056566805479405957157723605632781982421875f,0.7023000285446647783516027629957534372806549072265625f,-0.484100007211416938357473327414481900632381439208984375f,0.3160000107958913506678300109342671930789947509765625f,-0.026599964463709990791784321118029765784740447998046875f,-0.557600015977024465740896630450151860713958740234375f,-0.39269998745545742213636231099371798336505889892578125f,0.108999963611364325544172970694489777088165283203125f,-0.23849999131262311635737205506302416324615478515625f,0.4404000065386295403868643916212022304534912109375f,0.09809997605681441967817590921185910701751708984375f,-0.7802000338539480939203940579318441450595855712890625f,-0.703599960723519490812805088353343307971954345703125f,0.1785000665634866034992001004866324365139007568359375f,-0.49580002402961209639897788292728364467620849609375f,0.8724999909847970247511739216861315071582794189453125f,0.98320002466738198876328169717453420162200927734375f,0.502699982519447363671361017622984945774078369140625f,-0.48940002494901435614593765421886928379535675048828125f,0.91319999572187671166290101609774865210056304931640625f,-0.67840000890046392445498213419341482222080230712890625f,0.18489994326830017090657065637060441076755523681640625f,0.3688000153273345294024920804076828062534332275390625f,1.107800009746850111724825183046050369739532470703125f,-0.0206000136822457324825563773629255592823028564453125f,-0.742200012791156549241122775129042565822601318359375f,-0.2374999913573265786226329510100185871124267578125f,-1.367799981546401255627642967738211154937744140625f,0.1571999652564528560105827637016773223876953125f,-1.18670001821517967499630685779266059398651123046875f,0.00869999724552006359346023600664921104907989501953125f,0.47849999175965773900998101453296840190887451171875f,0.28759999857991902505460757311084307730197906494140625f,-0.21449999463930746745887745419167913496494293212890625f,0.109700015175342446838158139144070446491241455078125f,1.287399990318715214954181647044606506824493408203125f,-0.32469998272806421812930466330726630985736846923828125f,1.1747999892801048726909129982232116162776947021484375f,-0.0136000012546781956501718013896606862545013427734375f,-0.52780001995414493887182061371277086436748504638671875f,-1.0186000601142648491759246098808944225311279296875f,-0.73270000142529612130459781837998889386653900146484375f,0.02840000236257911314652346845832653343677520751953125f,-0.45710002164319174511319943121634423732757568359375f,0.7619999717473984190974078956060111522674560546875f,-0.863699997860193580123677747906185686588287353515625f,1.415500036597251209968817420303821563720703125f,-0.197000010993331697850550199291319586336612701416015625f,0.52540003902316134709593598017818294465541839599609375f,0.5458000063903629683892404500511474907398223876953125f,-1.20970001035109131493072709417901933193206787109375f,0.6552999998748301191398013543221168220043182373046875f,-0.12700000479072370840327721452922560274600982666015625f,0.8521999962694939423357709529227577149868011474609375f,-0.8382999817475678128175786696374416351318359375f,-0.698300049696862767945049199624918401241302490234375f,0.50549997397512191898982791826711036264896392822265625f,-0.75379999765083194329662319432827644050121307373046875f,0.599200031328201543345812751795165240764617919921875f,0.075999999172985555073722707675187848508358001708984375f,-0.8031000425904994965975447485106997191905975341796875f,0.21269998427033431909194405307061970233917236328125f,-0.008800035518408133583534436183981597423553466796875f,-1.1988999837696550088139701983891427516937255859375f,1.1526000070177015999206560081802308559417724609375f,-1.17660001514628564933673260384239256381988525390625f,-0.9769999646693470207736709198798052966594696044921875f,1.1955999782733621739083673674031160771846771240234375f,-0.2908999684810640307119911085464991629123687744140625f,-0.10929997719526307609072546256356872618198394775390625f,-0.47769995826780820014079154134378768503665924072265625f,0.357900002217292456663244593073613941669464111328125f,1.18029997662901831034787392127327620983123779296875f,0.37089998666048040742992952800705097615718841552734375f,0.18229998516365852889720144958118908107280731201171875f,-1.0312000108428296396567702686297707259654998779296875f,0.053300002720952033286039295489899814128875732421875f,0.8115000166781245471980810179957188665866851806640625f,-0.14809998320937101112804157310165464878082275390625f,-0.250600031042099491429553381749428808689117431640625f,-1.170300023716688286867793067358434200286865234375f,0.24930000178217870399066669051535427570343017578125f,1.019599985563754529493962763808667659759521484375f,0.66240003647506195960659169941209256649017333984375f,0.4336999714851383913583049434237182140350341796875f,0.578999969273805259462051253649406135082244873046875f,-0.749700020539761258220323725254274904727935791015625f,-0.548600034335256392381552359438501298427581787109375f,-0.40340001080036191893896102556027472019195556640625f,-0.390000003650784687891928115277551114559173583984375f,-0.3492000197798017513406421130639500916004180908203125f,0.370200045141577838148094770076568238437175750732421875f,1.027200005266070803600086946971714496612548828125f,-0.216299988728761771739073083153925836086273193359375f,1.3446000195384026909550811978988349437713623046875f,0.7172000038504595664790031150914728641510009765625f,-0.54260000476390091161960071985959075391292572021484375f,0.3956999968767167530359074589796364307403564453125f,0.9247000111892813567493476512026973068714141845703125f,-0.437799943697452675905879004858434200286865234375f,0.642799978212267308919081187923438847064971923828125f,-0.43509999802112542255372318322770297527313232421875f,-1.0644000421598558414615354195120744407176971435546875f,1.701700021791457739794850567704997956752777099609375f,-0.951800022625178154811464992235414683818817138671875f,1.12570002503991162257079849950969219207763671875f,0.059399995891749657328517741916584782302379608154296875f,-0.7131000244110825558863098194706253707408905029296875f,1.058500016495585338560658783535473048686981201171875f,-0.962999994724988983563207511906512081623077392578125f,0.63599999656528238833885779968113638460636138916015625f,-0.0568999853491778395664368872530758380889892578125f,-0.35080002015531086545507832852308638393878936767578125f,-1.0244000560551886902516116606420837342739105224609375f,0.191600000151991967101139380247332155704498291015625f,-1.201299948421121310815351534984074532985687255859375f,-0.282899996033310952014971917378716170787811279296875f,-0.18609997342675921316867970745079219341278076171875f,0.904099977725743908507638479932211339473724365234375f,1.2655999947577705189161179077927954494953155517578125f,-1.163600010418891717023370802053250372409820556640625f,0.800600001128017435547690183739177882671356201171875f,-1.387100022220611794665501292911358177661895751953125f,-0.691099992984533617601528021623380482196807861328125f,0.7386000092267988037519899080507457256317138671875f,-1.5665999795883891465564374811947345733642578125f,0.6929000044524664669864932875498197972774505615234375f,-0.2592000010043384872204796920414082705974578857421875f,0.24759999921321895754999786731787025928497314453125f,-0.550199978160858194087268202565610408782958984375f,0.2475000122189516815751630929298698902130126953125f,-1.16220000236034337603996391408145427703857421875f,0.70590000424385035415753009147010743618011474609375f,-0.4748000149846074435799891944043338298797607421875f,1.56000003367662376518865130492486059665679931640625f,-1.54809997620582606714378925971686840057373046875f,0.47850001664459707040322200555237941443920135498046875f,-0.917700018095969571874093162477947771549224853515625f,1.151800028380751683698690612800419330596923828125f,0.15549995829164942051647813059389591217041015625f,1.03950000140070830667582413298077881336212158203125f,0.046900010681152792813009000383317470550537109375f,0.0908000035405158723023077982361428439617156982421875f,-0.20010000398159011325560641125775873661041259765625f,-1.71040007165670449040817402419634163379669189453125f,1.17840001690983786630795293604023754596710205078125f,-1.65840001974105888393751229159533977508544921875f,0.443799998116492844246749882586300373077392578125f,0.67810001263916408476006836281158030033111572265625f,1.77129999200701693240489476011134684085845947265625f,-0.630699964690208059181486532907001674175262451171875f,0.34880002553463018699630993069149553775787353515625f,-1.293399985989928335783361035282723605632781982421875f,0.221299999755620202535055796033702790737152099609375f,-0.837399995124339824315029545687139034271240234375f,1.51250002235174196840716831502504646778106689453125f,0.0485999831497669898539015775895677506923675537109375f,0.377299953144788702275036484934389591217041015625f,1.05899998015165319742436622618697583675384521484375f,-0.8292000670909882575898564027738757431507110595703125f,0.21389995948076201415233299485407769680023193359375f,-0.38640000716447797657338014687411487102508544921875f,0.300000005811452918891291119507513940334320068359375f,-0.2927999833345413804863710538484156131744384765625f,0.9641000260055070381071118390536867082118988037109375f,-1.035999942287803232687792842625640332698822021484375f,-0.354999999105930275078435442992486059665679931640625f,0.601999992758035062934141024015843868255615234375f,0.790300020736455888226146271335892379283905029296875f,-0.058399992918968024468995281495153903961181640625f,-0.242899986869096640162979383603669703006744384765625f,-0.83889999449849117496569306240417063236236572265625f,0.1140000169575217103812292407383210957050323486328125f,0.952400005331635046701421742909587919712066650390625f,0.146900002783536987038814913830719888210296630859375f,1.4155000155866144240235371398739516735076904296875f,-0.23430000833868991350072974455542862415313720703125f,0.69580004064440714728334569372236728668212890625f,0.53640001766979683583969062965479679405689239501953125f,0.61640001267790811567692799144424498081207275390625f,0.5279999901056289246525921043939888477325439453125f,0.25819997407793893184901889981119893491268157958984375f,-0.367099983030557464047660687356255948543548583984375f,0.20060000168681124677050320315174758434295654296875f,0.147000000134110297977940717828460037708282470703125f,-1.36470004944801370783125094021670520305633544921875f,0.40550001513958022769656963646411895751953125f,0.8404000312745572021100315396324731409549713134765625f,-0.98920004265308403290646310779266059398651123046875f,0.36650000086426703926889558715629391372203826904296875f,-0.757900000429153397618620147113688290119171142578125f,-1.232400057709217566781489949789829552173614501953125f,0.285400013020634890636983982403762638568878173828125f,0.3195000061690806969494360600947402417659759521484375f,0.1065999747455119717187699279747903347015380859375f,-1.1636000630944973721625501639209687709808349609375f,1.121800040599703951471610707812942564487457275390625f,-0.68880000646114325490998453460633754730224609375f,-0.03730001215338696329837375742499716579914093017578125f,1.4263000484079129126513407754828222095966339111328125f,0.41519999231696136376967842807061970233917236328125f,0.2318999843060967069874322987743653357028961181640625f,0.17789998791217787044161013909615576267242431640625f,0.8082000245928764314840009319595992565155029296875f,1.4626000124752522868476489747990854084491729736328125f,0.48900003707408945530232813325710594654083251953125f,-0.59050002869963658103102943641715683043003082275390625f,-0.69960000627636897974070961936376988887786865234375f,-1.102000007510185231041077713598497211933135986328125f,-1.1745000174939630444015392640721984207630157470703125f,-0.779300016310811027864247080287896096706390380859375f,-0.1641000481337311356355712632648646831512451171875f,-0.9768000315099951702535463482490740716457366943359375f,0.325399994617700638599444573628716170787811279296875f,0.916199994134903050735374563373625278472900390625f,-0.0716000182569029863799414670211263000965118408203125f,1.1693999781966208928452033433131873607635498046875f,0.6410999879926440048194535847869701683521270751953125f,0.4757999835729602722267372882924973964691162109375f,0.2216999946713447133816998757538385689258575439453125f,0.8853999733090400781065909541212022304534912109375f,0.4967999761521812995823665914940647780895233154296875f,-0.340899976527690995453667710535228252410888671875f,0.5846000307887793123740038936375640332698822021484375f,0.4939999810457234019622774212621152400970458984375f,0.0035999930590389084272828768007457256317138671875f,1.1602999797582622676372920977883040904998779296875f,1.0091000474631786065771166249760426580905914306640625f,1.0128999643683442855035536922514438629150390625f,-0.34069999851584409356064497842453420162200927734375f,0.6243999957799903910427019582130014896392822265625f,-1.378400000667571845269776531495153903961181640625f,-0.9796000252008436604000962688587605953216552734375f,-0.4200000330805782056131647550500929355621337890625f,0.203200046646596188537614580127410590648651123046875f,-1.27010004227757544725818661390803754329681396484375f,-0.80470001517534228696604259312152862548828125f,-0.44510000755786904846900142729282379150390625f,-0.05069995902776636143016730784438550472259521484375f,-0.52290004945397416946661905967630445957183837890625f,-0.285500015586614974694157353951595723628997802734375f,-0.7806999688625335664937665569595992565155029296875f,-0.2736000122070318951728040701709687709808349609375f,-0.26630001510381617890743655152618885040283203125f,-1.4520000008046611839063189108856022357940673828125f,0.717899956917761983277159743010997772216796875f,-0.6123000429987914827734130085445940494537353515625f,0.5552000163078307792829946265555918216705322265625f,-0.802999979972838673347723670303821563720703125f,-0.17089993376135748093247457290999591350555419921875f,-1.087699998724459771182182521442882716655731201171875f,-0.77589996475577294177128351293504238128662109375f,-0.4579999872744078270869749758276157081127166748046875f,-0.4834999693334101067421215702779591083526611328125f,-1.092299992111324780097447728621773421764373779296875f,0.582100032055377969442133689881302416324615478515625f,1.3527999973416322543329215477569960057735443115234375f,-0.02760003616809836302081748726777732372283935546875f,-0.27629994968771942609464531415142118930816650390625f,0.0421000289261339588620103313587605953216552734375f,0.3978999814301726889453902913373894989490509033203125f,-0.434400009042024581873420174815692007541656494140625f,1.4426000410109764970911783166229724884033203125f,0.5755000190138819249341395334340631961822509765625f,0.1607000111818315257750100499833934009075164794921875f,0.0002999979749322978506143044796772301197052001953125f,-0.8923000218391419213759263584506697952747344970703125f,-0.65280000367462687194120007916353642940521240234375f,-0.234799987715482938455124894971959292888641357421875f,-1.326199998605250840455482830293476581573486328125f,0.0589000193834301466466740748728625476360321044921875f,1.0393999454140665505974538973532617092132568359375f,0.196699991914629723765983726480044424533843994140625f,0.8376999683260919482563622295856475830078125f,-0.4989999690502882057074884869507513940334320068359375f,-0.7590999970972536203817071509547531604766845703125f,-0.01769998061955002643941270434879697859287261962890625f,-0.46880003849863971510103510809130966663360595703125f,0.484000016584992476254001303459517657756805419921875f,0.843699955391883804622921161353588104248046875f,0.11039999610781681127491538063623011112213134765625f,0.914800003361701641324543743394315242767333984375f,-0.2248000050753355960608814712031744420528411865234375f,-0.2292000042825936834134381570038385689258575439453125f,0.9827999821797013257906883154646493494510650634765625f,1.132600034081935991281397946295328438282012939453125f,-0.050200015413761178706408827565610408782958984375f,0.3509000026047228715242454200051724910736083984375f,0.33970000101923947255500024766661226749420166015625f,-2.103700008738039084477122742100618779659271240234375f,2.36699997860193178667032043449580669403076171875f,0.5705999785900106591185476645478047430515289306640625f,0.69599999958276814737700988189317286014556884765625f,0.02310000444352677817505536950193345546722412109375f,1.099999969974159608199215654167346656322479248046875f,-0.236199973571299182140137418173253536224365234375f,0.1058999893426886274028220213949680328369140625f,-1.077400041317940093676952528767287731170654296875f,0.54470000951290131752102752216160297393798828125f,0.766100043201447089558087100158445537090301513671875f,1.6680000356286763807389661451452411711215972900390625f,0.734799979780614176405606485786847770214080810546875f,0.84679997734427470135187832056544721126556396484375f,0.85389999680817130212062693317420780658721923828125f,-0.798500005096196563414423508220352232456207275390625f,0.367199979412556221092245323234237730503082275390625f,-0.14679998166561158967624578508548438549041748046875f,1.384799974900483565676267971866764128208160400390625f,0.78979997035562998775048981769941747188568115234375f,0.834099989646673645182772816042415797710418701171875f,0.5301000278979548507862773476517759263515472412109375f,1.687600008079410685013499460183084011077880859375f,-0.83280001149773585922275742632336914539337158203125f,-0.134100002908706272819472360424697399139404296875f,-0.2402999776720999847157145268283784389495849609375f,0.112900009146332802600909417378716170787811279296875f,0.590300014626979674403628450818359851837158203125f,-0.1218999993562694950099967172718606889247894287109375f,0.696200047779083064369842759333550930023193359375f,-0.665399992904066817089869800838641822338104248046875f,-0.671299959373474308677032240666449069976806640625f,1.0676000117301942538006187533028423786163330078125f,0.138400006031989875054932781495153903961181640625f,-1.0593000308394433517378274700604379177093505859375f,1.3664999786615368293496430851519107818603515625f,0.0750999752223486805036145597114227712154388427734375f,-0.5851000367641443489219454932026565074920654296875f,0.378600026512145859669544734060764312744140625f,-0.6640000287294380854064002051018178462982177734375f,-0.2452000119119883692064831848256289958953857421875f,-0.214400022602081463674039696343243122100830078125f,-0.28919999899268145071573599125258624553680419921875f,-0.7905999911069869767743512056767940521240234375f,-0.732599990570545145374126150272786617279052734375f,-0.399900032824278550691587952314876019954681396484375f,-0.29479998652338945674955539288930594921112060546875f,0.2965999552249893866928687202744185924530029296875f,0.6553000021100050531686065369285643100738525390625f,0.401200030195713708280891296453773975372314453125f,-0.86989998171329574461196898482739925384521484375f,-0.5678000478565703446065526804886758327484130859375f,1.116800033521652579793226323090493679046630859375f,-0.502599985951184269339364618645049631595611572265625f,1.1881000161409378534926872816868126392364501953125f,0.82600005412101840107652606093324720859527587890625f,-0.339900044596195272106342599727213382720947265625f,-0.83390000731348923324048882932402193546295166015625f,0.953199996876715971438898122869431972503662109375f,1.123000028252601367739771376363933086395263671875f,-0.81380000646114325490998453460633754730224609375f,-0.934200013893843106416170485317707061767578125f,-0.52210002594590232405380447744391858577728271484375f,-1.13900000213086638467530065099708735942840576171875f,0.62340001154541990757707026205025613307952880859375f,0.516100004011393220793024738668464124202728271484375f,-1.3015000385642050417800419381819665431976318359375f,-0.273299989458918535234488444984890520572662353515625f,-0.030399956470727573787371511571109294891357421875f,-0.49700002017617261884652180015109479427337646484375f,-0.4025999865472318361980796908028423786163330078125f,0.29349996739625883179769516573287546634674072265625f,0.11979999114274964000514955841936171054840087890625f,1.1041000075280671399724496950511820614337921142578125f,0.89010002722740200198359161731787025928497314453125f,0.44090000334978096674376502051018178462982177734375f,0.5492999885201452769933894160203635692596435546875f,-0.21159999836385257054871544823981821537017822265625f,-0.641299995732307337448219186626374721527099609375f,-0.191900019547343259063154619070701301097869873046875f,-0.326199979084730529166336054913699626922607421875f,1.542900029337406220264483636128716170787811279296875f,1.19199999693036051695571586606092751026153564453125f,-0.688100041174889209827369995764456689357757568359375f,0.14939997357726131355093457386828958988189697265625f,-0.89260002670586136019892364856787025928497314453125f,-1.4115000063031908350552612319006584584712982177734375f,0.373699989180266756161330476970761083066463470458984375f,-0.9119999881386753326495409055496565997600555419921875f,-0.8123999995946877827890375556307844817638397216796875f,-0.117099967682361860710216205916367471218109130859375f,0.659699995058775190415190081694163382053375244140625f,-0.78199992123246264696945218020118772983551025390625f,-0.337699986654520056816863871063105762004852294921875f,-0.09580001307725893866518163122236728668212890625f,0.981399961566925327360877417959272861480712890625f,1.1458000276803972372619000452687032520771026611328125f,0.055000019520520027072052471339702606201171875f,-1.147800050613284827960569600691087543964385986328125f,-0.225899957379697813308894183137454092502593994140625f,-0.8894999970793715515782196234795264899730682373046875f,0.56389996871948167012078556581400334835052490234375f,1.4760000539347533088374575527268461883068084716796875f,-0.5747000102579591374052370156277902424335479736328125f,0.73300004140287688958466105759725905954837799072265625f,-0.964399999766051951866074887220747768878936767578125f,0.64560000936090933265631974791176617145538330078125f,-0.538500002637505303937359713017940521240234375f,0.1972999655127509743834934852202422916889190673828125f,-0.42799996954202601529004823532886803150177001953125f,0.20749999701976751254051123396493494510650634765625f,0.09250000193715113283587925252504646778106689453125f,0.08319998674392792281651054508984088897705078125f,-0.755600004443525374853152243304066359996795654296875f,-0.31050001603364929536610361537896096706390380859375f,-1.8836000380605464332717247089021839201450347900390625f,-0.007799999651312727877439101575873792171478271484375f,1.0845000697970397229852324016974307596683502197265625f,-0.1387000046402218078611667806399054825305938720703125f,0.7028999992370603688840446920949034392833709716796875f,-0.974600034514069957225501639186404645442962646484375f,-0.27600000956654557882075096131302416324615478515625f,-0.4144999935030939663960225516348145902156829833984375f,0.1033999748885623404248690349049866199493408203125f,-1.6656000042945147132655847599380649626255035400390625f,-0.0741000064849852702053567554685287177562713623046875f,0.974800018188357153547940470161847770214080810546875f,0.6048999837994575301536315237171947956085205078125f,0.055800001305341684343375163734890520572662353515625f,-0.6081999843597412080953290569595992565155029296875f,-0.69910002555847172711622761198668740689754486083984375f,-0.90859997942447723318082353216595947742462158203125f,0.93419999377727502842816420525196008384227752685546875f,-0.49619999551326021691721734896418638527393341064453125f,-0.1945999946720899931307258157175965607166290283203125f,0.43139999278485785527692542018485255539417266845703125f,0.6073000016108149878846234059892594814300537109375f,0.9040999753788117043740157896536402404308319091796875f,0.48770001280605779214738504379056394100189208984375f,0.950199965569377358320934945368207991123199462890625f,-0.49420002190321687951524154414073564112186431884765625f,-0.4542000169113278307264636168838478624820709228515625f,0.070400003707408842501536128111183643341064453125f,-1.193700002516806080166134051978588104248046875f,-0.1997000174291434781981280366380815394222736358642578125f,-0.1167999971255657687407847333815880119800567626953125f,1.1563999681234360394910254399292171001434326171875f,0.9548999913990490195914162541157566010951995849609375f,0.67750004619359938118350328295491635799407958984375f,0.02710001590624477874058584347949363291263580322265625f,-0.69159996908307064256149487846414558589458465576171875f,0.4271000185884534783298249749350361526012420654296875f,-0.490800036267190709082797184237278997898101806640625f,-0.99180001228749692376851498920586891472339630126953125f,0.5354000018447633291174270198098383843898773193359375f,-0.5822000063836572092412779966252855956554412841796875f,-0.2838000000908971198754215947701595723628997802734375f,-0.7125000038743014574293965779361315071582794189453125f,0.7415999810785052215322821211884729564189910888671875f,1.3449000460490585506789784631109796464443206787109375f,-1.5415000186711547147666578894131816923618316650390625f,0.415500033374876398539754518424160778522491455078125f,-0.69800004253909053719695521067478694021701812744140625f,0.454399998164177532089524902403354644775390625f,-0.091800004017352332397194913937710225582122802734375f,-0.4570000342577695828794048793497495353221893310546875f,-0.91590002085864574699058948681340552866458892822265625f,-0.34829999396652022713993801517062820494174957275390625f,0.1884000031262633712003662367351353168487548828125f,0.16189998344928024920363895944319665431976318359375f,-0.6866000011950725845366605426534079015254974365234375f,0.43529998907148825804114267157274298369884490966796875f,-0.5653999957352875416205506553524173796176910400390625f,-0.18019998117834301609718750114552676677703857421875f,1.197500059828162211061908237752504646778106689453125f,0.489200018140673709154953030520118772983551025390625f,0.30109998227432355211163894637138582766056060791015625f,-0.88510001197233767467054121880210004746913909912109375f,0.355900032053143300370123824905022047460079193115234375f,-0.230699999521672527436066957307048141956329345703125f,-0.616200012314319423012420884333550930023193359375f,-0.8746000114172696182635036166175268590450286865234375f,-0.5016000187039377777864501695148646831512451171875f,0.580099997127055733159295414225198328495025634765625f,0.916699978876113874548536841757595539093017578125f,-0.64729998473525041102760724243125878274440765380859375f,1.18619995867013994228500450844876468181610107421875f,-0.189700034695864161449208040721714496612548828125f,-0.99809999781250979822289082221686840057373046875f,0.6132000030606981066938487856532447040081024169921875f,-1.3498000191569321515316914883442223072052001953125f,-0.083500012770295561903566294859047047793865203857421875f,-0.119800018337368641141438274644315242767333984375f,0.53030002401471154627188298036344349384307861328125f,1.161999976068735396239617330138571560382843017578125f,0.07300001290440594203801083494909107685089111328125f,1.2285999818086619228552081040106713771820068359375f,0.030999996602535162537606083787977695465087890625f,0.8561000488638885741465855971910059452056884765625f,0.14899996361136391698209990863688290119171142578125f,0.00999999180436095258528439444489777088165283203125f,-0.0129999579191206748873810283839702606201171875f,-1.1727000176489343896690797919291071593761444091796875f,1.85359997286796573945366617408581078052520751953125f,-0.1086000133991242844189173410995863378047943115234375f,1.09760002916455245980387189774774014949798583984375f,0.752500060349703847606406270642764866352081298828125f,-1.17970002515912053553392979665659368038177490234375f,0.58310004452765051841112153852009214460849761962890625f,-1.01310003834366835206992618623189628124237060546875f,-0.3561999922722574041245025000534951686859130859375f,0.015299997341632565195368442800827324390411376953125f,-1.598000066101551652764101163484156131744384765625f,1.316600036585331423566458397544920444488525390625f,0.88570004500746790654375217854976654052734375f,1.130000021904706830611075929482467472553253173828125f,0.2896999882787465452338437899015843868255615234375f,-1.3426000220119949091213129577226936817169189453125f,-0.08030002453625240121937167714349925518035888671875f,-0.3174999928474431243330400320701301097869873046875f,0.083099980527162831123177966219373047351837158203125f,0.1264000251948840247706584705156274139881134033203125f,0.406799966466426798916700136032886803150177001953125f,0.27599998378753642924721134477294981479644775390625f,0.023899997329711819560316143906675279140472412109375f,0.7661000269591813349734366056509315967559814453125f,0.1621000078409908784493609346100129187107086181640625f,-0.489600008362531813332907404401339590549468994140625f,0.8435000358670949882622380755492486059665679931640625f,-0.185299997937679705728442058898508548736572265625f,0.3638000101119278628658548768726177513599395751953125f,0.338499993920326147645027958787977695465087890625f,0.6543000248976051924643115853541530668735504150390625f,-0.21989996331036110177592490799725055694580078125f,0.841299999979138402039779975893907248973846435546875f,-0.266399975983798054102180685731582343578338623046875f,1.2587000088870521352646392188034951686859130859375f,-1.1148999622672797205069628034834749996662139892578125f,0.168399947358667478880533963092602789402008056640625f,-0.96870003794431713828316787839867174625396728515625f,-1.173500034581870021810345861013047397136688232421875f,0.49800003525614844335223097004927694797515869140625f,-0.386699993181228551808459314997890032827854156494140625f,-0.13579998878836629927491230773739516735076904296875f,0.61180000677406776166122881477349437773227691650390625f,-1.46229999918937636493865284137427806854248046875f,0.2788999963797629266792910129879601299762725830078125f,0.5100000038743015551290227449499070644378662109375f,0.17469998462796210159098109215847216546535491943359375f,-1.5540000050365925243767151187057606875896453857421875f,-0.1195999810561539644648831881568185053765773773193359375f,0.27940003431811921696947820237255655229091644287109375f,0.01199998618289842278183954249470843933522701263427734375f,-0.1495999322175987078509251659852452576160430908203125f,-0.1643999874539672223505704096169210970401763916015625f,1.29370000285208153201210734550841152667999267578125f,1.2510000165700905228050032746978104114532470703125f,1.1277999844148751673600372669170610606670379638671875f,-0.281000053599476995458417150075547397136688232421875f,0.90559997531175617790921705818618647754192352294921875f,-0.61409998681545285847960258251987397670745849609375f,-0.7787999510288237114963294516201131045818328857421875f,1.4521999572843313774939133509178645908832550048828125f,0.221500002689659414301104334299452602863311767578125f,-0.54819998108148570992170789395458996295928955078125f,0.951299991261958410859733703546226024627685546875f,-1.318299997133017154027356809820048511028289794921875f,-0.8169000828027730420899388263933360576629638671875f,0.455000017434358472456779054482467472553253173828125f,-0.888400032630562552782294005737639963626861572265625f,1.27530001224279398996941381483338773250579833984375f,0.8573000319719312756205908954143524169921875f,0.416200014922022631935760728083550930023193359375f,0.5071999963253734922119519978878088295459747314453125f,-0.0923000092476609079739091612282209098339080810546875f,-1.0469000147417186763476593114319257438182830810546875f,-0.1648000180020929850144284500856883823871612548828125f,0.0521000257596373472779305302537977695465087890625f,-0.303899995094537356266073402366600930690765380859375f,0.345500025272369892803681068471632897853851318359375f,-0.27029999339282506642945236308150924742221832275390625f,0.74160001170039180351523100398480892181396484375f,0.0165000123381613406081669381819665431976318359375f,-0.3412000409431759084810664717224426567554473876953125f,-0.463699985492228794470292996265925467014312744140625f,0.285100017839670538677410149830393493175506591796875f,-0.960599967339633931118214604794047772884368896484375f,0.8590000059306619473176169776706956326961517333984375f,-0.5583999836802480132291748304851353168487548828125f,-0.41810003907009996026999942841939628124237060546875f,1.1439999818652861751644422838580794632434844970703125f,-0.5499000071756541263567896749009378254413604736328125f,0.6522000130891800750987385981716215610504150390625f,-0.72020001366287456079362527816556394100189208984375f,1.464500016152859185325496582663618028163909912109375f,0.331600032450258819238797514117322862148284912109375f,-0.438400002008676903386685808072797954082489013671875f,0.567100002085417553843171845073811709880828857421875f,-0.64170001851320268571043925476260483264923095703125f,-0.48479999099373838333804087596945464611053466796875f,0.184399978531896380218313424848020076751708984375f,0.8458999899387353860191751664387993514537811279296875f,-0.3805999969184394871035692631267011165618896484375f,0.411099968248605396325956462533213198184967041015625f,-0.086199931102991289577630595886148512363433837890625f,0.31019997473359151030081193312071263790130615234375f,0.744500013954937145399526343680918216705322265625f,-0.57699999566376247717158776140422560274600982666015625f,-1.3089000354021791405756403037230484187602996826171875f,-0.0278000084802510760706439896239317022264003753662109375f,0.5847999781042336753245081126806326210498809814453125f,-1.0698000145748256528577257995493710041046142578125f,-0.9773999995201820834012096383958123624324798583984375f,-0.124199983346462428102086050785146653652191162109375f,-0.297700005169212966738001568955951370298862457275390625f,0.844400013512373259771948141860775649547576904296875f,-0.039800061774254658075733459554612636566162109375f,0.61300000903010332109488444984890520572662353515625f,-1.68950001317262632483107154257595539093017578125f,0.12499998249113541959331996622495353221893310546875f,-1.397900004005432350595583557151257991790771484375f,-0.01650001502037046208215542719699442386627197265625f,-0.252199964362383344251838934724219143390655517578125f,-1.19070001252293611315735688549466431140899658203125f,1.38489999542236308371911945869214832782745361328125f,-0.2406999845087529621423527714796364307403564453125f,-0.63780003191232648163122576079331338405609130859375f,0.0790999937444929468455256937886588275432586669921875f,0.3917000375866894756882174988277256488800048828125f,0.54210002005994351037543310667388141155242919921875f,0.0173000334620478923852715524844825267791748046875f,-0.5464000356256963897294554044492542743682861328125f,-0.5819000038266182173174456693232059478759765625f,0.085800018739700334435838158242404460906982421875f,-1.0148000486239794337706143778632394969463348388671875f,0.204400009191036957645337679423391819000244140625f,-1.2326000111155208660562720979214645922183990478515625f,1.369499993801117199154759873636066913604736328125f,-0.53629999863803401449757757291081361472606658935546875f,0.959600013428926867931068045436404645442962646484375f,0.52559999870657880816082752062357030808925628662109375f,-0.231499972850084390074698603712022304534912109375f,-0.1238000202819702355583331154775805771350860595703125f,-0.66500005751848245694191064103506505489349365234375f,0.459900047148019408549401987329474650323390960693359375f,-0.43509998729228982483618892729282379150390625f,-0.2281000255472960258629200325231067836284637451171875f,0.304099959993362034538222360424697399139404296875f,-0.63449999659508460325696432846598327159881591796875f,1.027100029969215455594166996888816356658935546875f,-0.3602999746918680301632775808684527873992919921875f,1.342700022193789255453566511278040707111358642578125f,0.292599930444359390691033695475198328495025634765625f,0.024299945642054066041737314662896096706390380859375f,-0.758100010776519894761804607696831226348876953125f,-0.595099998170137478581409595790319144725799560546875f,-0.66070001550316792560124667943455278873443603515625f,-0.5384999571144584162851742803468368947505950927734375f,-0.416799989265203674193571714567951858043670654296875f,0.33549997602403180962937767617404460906982421875f,1.308600007289647759733952625538222491741180419921875f,-0.71760000826567360565633180158329196274280548095703125f,-0.3865999873369929673572187311947345733642578125f,0.537799997006356722550890481215901672840118408203125f,0.6913999949455262861164328569429926574230194091796875f,-0.60780003660619286165456287562847137451171875f,-0.943499998092651281922371708787977695465087890625f,-0.3952000417888168470881282701157033443450927734375f,0.6948000140905377097766404403955675661563873291015625f,-0.2692000158682461563586230113287456333637237548828125f,0.2240000112205737803350302783655934035778045654296875f,1.4694999782294033696672386213322170078754425048828125f,-0.005499991670251114239675871431245468556880950927734375f,0.48599997197836619644562006214982829988002777099609375f,0.5219000126555564111896501344745047390460968017578125f,-0.1563000119745729943332435141201131045818328857421875f,-0.007200023519992715392845639144070446491241455078125f,0.4160000007748596573264876496978104114532470703125f,0.9331000260874631191398975715856067836284637451171875f,-1.0135000223815442410568721243180334568023681640625f,0.78710000990852713176337829281692393124103546142578125f,0.3809000252172352052326687044114805757999420166015625f,-0.10219999792724843157287750727846287190914154052734375f,0.80450000384077424708806347553036175668239593505859375f,0.3125000110268592035112078519887290894985198974609375f,0.4727999935977156287236766729620285332202911376953125f,-0.8847999524369833945769414640381000936031341552734375f,1.009799990677088477042389058624394237995147705078125f,-0.5647999763160942432449473926681093871593475341796875f,0.6034999993406235940796022987342439591884613037109375f,-0.61039999048262816305765454671927727758884429931640625f,0.47950002472102681849719374440610408782958984375f,-0.611400003476440634386790407006628811359405517578125f,-0.02209999390840533539659418238443322479724884033203125f,0.2249000034689903060325377737171947956085205078125f,-0.01309999827817087680870145049993880093097686767578125f,-0.7321000359296803150499499679426662623882293701171875f,0.3156000074796379006869528893730603158473968505859375f,0.26530001332312835859994493148406036198139190673828125f,0.16230000121966003945317424950189888477325439453125f,0.58010000731572486376563801968586631119251251220703125f,-0.21429998997300880869687489393982104957103729248046875f,-0.717899948684871613835412063053809106349945068359375f,0.38709995989650447523189313869806937873363494873046875f,-0.36139997784644439793311221365001983940601348876953125f,0.25370000173449513969359259135671891272068023681640625f,-0.11699998608976625025235307475668378174304962158203125f,0.44060000369846819001651283542742021381855010986328125f,-0.0559999965652817621730719110928475856781005859375f,0.381500001609325511964243560214526951313018798828125f,1.6623000228822231161274203259381465613842010498046875f,-0.756899987956880959671934760990552604198455810546875f,0.004800028023124180975855779252015054225921630859375f,0.74819999646693435835231866803951561450958251953125f,-0.79769997914806001215737296661245636641979217529296875f,-0.111000013049691814703834324973286129534244537353515625f,-0.1362000016599893559288148026098497211933135986328125f,-0.518299982623010802029739352292381227016448974609375f,-0.2642999876260763159052658011205494403839111328125f,-0.15159997780024969671330836717970669269561767578125f,1.22190003839731264889678641338832676410675048828125f,0.4283000101715324081652624954585917294025421142578125f,0.19329998685121463353198123513720929622650146484375f,-1.0646000558972363858600829189526848495006561279296875f,-1.5165000159144401692401515902020037174224853515625f,-0.638100038082897835778339867829345166683197021484375f,-2.031800049894302961916991989710368216037750244140625f,0.32010001610741090072309589231736026704311370849609375f,0.315399991562963055713453286443836987018585205078125f,0.6260999939531080027421694467193447053432464599609375f,0.97430000997781718297119368799030780792236328125f,-0.254399990341067638865979461115784943103790283203125f,-0.1054000165224078244818883831612765789031982421875f,-0.469100008794665512823485187254846096038818359375f,-0.088600008630752569160904386080801486968994140625f,0.24029997171163586244801990687847137451171875f,0.894800033406168093819132991484366357326507568359375f,-0.11299995136260942985018118633888661861419677734375f,-0.303600037892907959058419464781763963401317596435546875f,0.77839997384548187397967922152020037174224853515625f,-2.03320005521476332432939670979976654052734375f,2.2983000431776048344545415602624416351318359375f,-0.507799966477601838477085038903169333934783935546875f,-0.5914000219352544274187266637454740703105926513671875f,0.533100010441243110648201763979159295558929443359375f,-0.7491999841474006327501911073341034352779388427734375f,0.450899978669732692804217322191107086837291717529296875f,-0.61669998602867082126977038569748401641845703125f,-0.03920002356097118667577205997076816856861114501953125f,-0.2488000326871873113532274146564304828643798828125f,-0.194500002201646537969281780533492565155029296875f,0.426900013661384747365445946343243122100830078125f,0.2501999996185302332918354295543394982814788818359375f,-1.875399963437020200984761686413548886775970458984375f,-0.240700006413459899334839064977131783962249755859375f,0.80529998799115365404333033438888378441333770751953125f,0.195900003498792774081493917037732899188995361328125f,1.68079999310970240600227043614722788333892822265625f,-1.823000038683414647522340601426549255847930908203125f,-0.608800060328840597634325604303739964962005615234375f,0.241100017347931849798214898328296840190887451171875f,0.80640002845451252966313404613174498081207275390625f,0.0990000203847885007490958741982467472553253173828125f,0.948200042437016055174581197206862270832061767578125f,-0.02820003107488133764491067267954349517822265625f,-0.5676999958559874048802385004819370806217193603515625f,0.86550000098347634747142365085892379283905029296875f,-0.5207000127837062564140069298446178436279296875f,-0.2909999998807908383469111868180334568023681640625f,0.15299999055266422232080003595910966396331787109375f,0.18070000566840160871606713044457137584686279296875f,-0.4773999684512606478392626740969717502593994140625f,-1.132300011706351750007115697371773421764373779296875f,0.786400016626715991918672443716786801815032958984375f,0.739199999141693098181349341757595539093017578125f,-1.499200029018521451718015669030137360095977783203125f,0.746800033223629089462747288052923977375030517578125f,1.78260002540201067944281021482311189174652099609375f,0.13240000631883719339754179600276984274387359619140625f,0.8814999885708092453029394164332188665866851806640625f,0.24569998789876701383860790883773006498813629150390625f,-1.528699992141871799589125657803378999233245849609375f,0.10429996855258938381894040503539144992828369140625f,-1.8763000416278838855532740126363933086395263671875f,0.429599998825788365053313100361265242099761962890625f,0.26820001878142374351909893448464572429656982421875f,0.5273000099182123534546917653642594814300537109375f,0.17150001943484005639817269184277392923831939697265625f,-0.688400000369548070722203192417509853839874267578125f,-0.02369999080076812258965901492047123610973358154296875f,0.4904000075072048048241413198411464691162109375f,0.00420000253170715875938867611694149672985076904296875f,-0.862699996414780567732805138803087174892425537109375f,1.1890000177957116278548710397444665431976318359375f,-0.41439994329065099609721301021636463701725006103515625f,0.6251999879024923156833892790018580853939056396484375f,-0.49710001763850464318039712452446110546588897705078125f,-1.4485000412128867797179054832668043673038482666015625f,1.574900004716962342854458256624639034271240234375f,-0.97509999768584965096351879765279591083526611328125f,1.00760003139972642571819960721768438816070556640625f,-0.09160000406354662505492569835041649639606475830078125f,0.60900004526972839613563337479718029499053955078125f,-0.51129999625384858763510464996215887367725372314453125f,-1.2970000348165628967223028666921891272068023681640625f,0.7617999730974436278785333342966623604297637939453125f,0.5261999578133227384313386210124008357524871826171875f,-0.467399966737627625690265631419606506824493408203125f,-0.5508999998480084148155810908065177500247955322265625f,0.992399982979893735546284005977213382720947265625f,-0.52509994277507043847208478837274014949798583984375f,-0.58370001283586059326324857465806417167186737060546875f,0.132799981267004596485747924816678278148174285888671875f,0.8182000180177393389158169156871736049652099609375f,1.2322999779552221699674419141956605017185211181640625f,-0.592700035399943647718146166880615055561065673828125f,-0.6498000672131780675044865347445011138916015625f,-0.787899991190433635068757212138734757900238037109375f,0.4558000048071144671979482154711149632930755615234375f,-0.787400030142068718674863703199662268161773681640625f,-0.670800033938885320594636141322553157806396484375f,0.8008000037640332635646700509823858737945556640625f,-1.63410000126957921651182914501987397670745849609375f,-0.299100010806321758849435354932211339473724365234375f,-0.515899988895654360021580941975116729736328125f,-0.6337000254645943631004456619848497211933135986328125f,-0.1295000124275684783015094581060111522674560546875f,0.1829000153303150799644072321825660765171051025390625f,-0.3518999771162867329366008561919443309307098388671875f,-1.033499984011053474119989914470352232456207275390625f,-0.538800004374980545662765507586300373077392578125f,0.0858999710887662359937166911549866199493408203125f,-1.03219997941255581253017226117663085460662841796875f,-0.0614999937117095551997181246406398713588714599609375f,0.34040004125833522863331381813623011112213134765625f,-0.3179999978542327188080207633902318775653839111328125f,0.70229999993443481542954032192938029766082763671875f,0.501500002205371941954581416212022304534912109375f,-1.0858000150330362476580603470210917294025421142578125f,0.9495999897360807295854101539589464664459228515625f,-0.47119998474717139647083286035922355949878692626953125f,0.9625000301003456826265392010100185871124267578125f,-0.51700000905618070934366414803662337362766265869140625f,0.5556000060081480729223812886630184948444366455078125f,-0.23579997558221166276126723460038192570209503173828125f,-0.5205000468418001791093274732702411711215972900390625f,-0.68580003384575338731821148030576296150684356689453125f,0.286300006759166336678390507586300373077392578125f,-0.6861000064142042109693875318043865263462066650390625f,0.30450001436472096116858665482141077518463134765625f,0.266900023347139303542263633062248118221759796142578125f,0.005799983572959632738275104202330112457275390625f,0.233500002413988105853803745048935525119304656982421875f,0.24529999272227254181188982329331338405609130859375f,-0.156399998000264162101302645169198513031005859375f,0.423999983243644518182691172114573419094085693359375f,-0.30490003394186515617292343449662439525127410888671875f,-0.5223000130474562130444837748655118048191070556640625f,-0.8822000016897917351599289759178645908832550048828125f,1.476499981045722886818793995189480483531951904296875f,-0.73629999010711888463021068673697300255298614501953125f,0.6999999707564719120256313544814474880695343017578125f,0.02550001085549580270850356100709177553653717041015625f,0.4844999765530226820686721111997030675411224365234375f,0.34990000108480412421840810566209256649017333984375f,-0.30190003575235646593455385300330817699432373046875f,-0.099200055170058920595010931720025837421417236328125f,-1.35769998248219447845031027100048959255218505859375f,0.200500011265277944261242737411521375179290771484375f,-0.36520000528097151804018949405872263014316558837890625f,-0.089299975854158386567860361537896096706390380859375f,-0.18370003690123493100827545276843011379241943359375f,0.43380001957416514102305882261134684085845947265625f,1.8667000281989576393470997572876513004302978515625f,-1.19510000524818860156983646447770297527313232421875f,0.316500017926097054754563941969536244869232177734375f,-0.19759999615848045095845009200274944305419921875f,-0.58859998858869122528858497389592230319976806640625f,-1.23100002320110757381144139799289405345916748046875f,-1.022200016218423712643925682641565799713134765625f,-0.115800025296211916980837486335076391696929931640625f,0.047699977341294275134941926808096468448638916015625f,0.29200000765919664758030194207094609737396240234375f,-0.844200034680962829725103802047669887542724609375f,-0.5751000000327823880041933080065064132213592529296875f,0.54489999706149117031372952624224126338958740234375f,0.3261000230848789538384835395845584571361541748046875f,0.5544000094890595420338286203332245349884033203125f,-0.84719996629953442379701300524175167083740234375f,0.4054999973699455750875131343491375446319580078125f,0.5109000242114074552546298946253955364227294921875f,0.27010002054050563646825366959092207252979278564453125f,-0.868400002902745882238377816975116729736328125f,-0.50699999912828153103561135139898397028446197509765625f,-0.23980006147623011969471917836926877498626708984375f,-0.5362999960117047493213249254040420055389404296875f,-0.068999978661537397783831693232059478759765625f,0.14979998216479994965766309178434312343597412109375f,-0.875899991577863801239800523035228252410888671875f,-0.28070002099797120020951979313394986093044281005859375f,-0.4478000548601155372807625099085271358489990234375f,0.439500027906149670631208437043824233114719390869140625f,-0.28799996119737603095245503936894237995147705078125f,0.56899999561160807814985673758201301097869873046875f,-0.881100013101100643098106957040727138519287109375f,0.75270000297129102673210354623734019696712493896484375f,1.33579997977316367041566991247236728668212890625f,-1.02370000642836078696973345358856022357940673828125f,-0.0171000103369352007831594164599664509296417236328125f,0.460800011140108323193231854020268656313419342041015625f,1.1991999643474822168087712270789779722690582275390625f,0.52580000368952772848984977827058173716068267822265625f,0.20110000538975025818899666774086654186248779296875f,-0.165799996834993468297625440754927694797515869140625f,1.108200040537118180594688965356908738613128662109375f,0.090599981942772911391870138686499558389186859130859375f,0.3506999808207151847483373785507865250110626220703125f,0.94320001028776145890475390842766501009464263916015625f,-0.70090000547319630808118517961702309548854827880859375f,-0.466399988612532734322257965686731040477752685546875f,0.36400002060830594086127121045137755572795867919921875f,0.866199989664554692581077688373625278472900390625f,0.5993999871373176802080706693232059478759765625f,0.35990001099407697982002218850539065897464752197265625f,0.48000001855194529554182736319489777088165283203125f,-1.3067000536054376169659008155576884746551513671875f,1.1332000389724974365179832602734677493572235107421875f,-0.575900011377781684274168583215214312076568603515625f,-0.99179997758641824656677954408223740756511688232421875f,-0.9211999752104269578723005906795151531696319580078125f,0.01989998469352638466034477460198104381561279296875f,1.3451000504732133489227408063015900552272796630859375f,-0.87520000900626104822777051595039665699005126953125f,1.241500058382750015795181752764619886875152587890625f,-1.784299989265203834065687260590493679046630859375f,0.645700004625319845530384554876945912837982177734375f,-0.3658000226140014632392194471322000026702880859375f,-1.07240002820491753254827926866710186004638671875f,1.0849000144958491631541619426570832729339599609375f,0.70739999944567699685649131424725055694580078125f,-0.8839000230714677552867897247779183089733123779296875f,-0.3246000209167601013859894010238349437713623046875f,-1.307200017689913895679865163401700556278228759765625f,0.1663999902330337565814488698379136621952056884765625f,-1.6428000110693272706186007781070657074451446533203125f,-0.8828000140681862006886149174533784389495849609375f,0.5867000788629059826462253113277256488800048828125f,-0.70239999333620151134027764783240854740142822265625f,0.859899998998641112990526380599476397037506103515625f,-1.110600015470386114913026176509447395801544189453125f,-0.38790000728368756455211041611619293689727783203125f,-1.435200001257658453113208452123217284679412841796875f,-0.1249000033199791914739762432873249053955078125f,0.0652999973416330536934992778697051107883453369140625f,-0.69709999957084622934644357883371412754058837890625f,0.3200999904960395436859244000515900552272796630859375f,-0.0579000099658966149718253291212022304534912109375f,-1.1990000442266468549945557242608629167079925537109375f,-0.216000048309564096626900209230370819568634033203125f,-0.3673000036597249273739862474030815064907073974609375f,-0.186599987709521908385568167432211339473724365234375f,0.5117000124037265340604108132538385689258575439453125f,0.9481000143527982260138742276467382907867431640625f,0.4593000008881087392609288144740276038646697998046875f,-0.100100028493999904100064668455161154270172119140625f,-0.70179999723732466865300239078351296484470367431640625f,-0.47760002610981455717364951851777732372283935546875f,0.133000012643635423525978467296226881444454193115234375f,-0.509200001749396502503941519535146653652191162109375f,0.63370002676844616740226001638802699744701385498046875f,0.0427000057280064737597058410756289958953857421875f,-0.157800001066922934711556081310845911502838134765625f,0.93659999486207912156032762140966951847076416015625f,-0.9041999663591386848793263197876513004302978515625f,0.28440003936588820199204974414897151291370391845703125f,-0.385700002539157704717354135937057435512542724609375f,1.459200058969855717805330641567707061767578125f,-0.9429000064641233080919846543110907077789306640625f,-0.721200051206351222532475730986334383487701416015625f,0.81860002323389124967434327118098735809326171875f,-1.21270001302957464162091127946041524410247802734375f,1.168899990028142266140775973326526582241058349609375f,0.80650003863871144726971351701649837195873260498046875f,0.332099981093406437793191798846237361431121826171875f,-0.510100024135411000969497763435356318950653076171875f,0.7804000187948350042432821283000521361827850341796875f,0.647400023287534764904194162227213382720947265625f,-1.23220001204609896916508660069666802883148193359375f,-0.98660004299283077244808737304992973804473876953125f,0.56130000944137581342374687665142118930816650390625f,0.333000025868415150398504920303821563720703125f,1.313700039732455859819992838311009109020233154296875f,-1.2664000053763400099882119320682249963283538818359375f,-0.75640003756284812652666005305945873260498046875f,-1.1953000216305260661187048754072748124599456787109375f,-1.30650001963973050322920244070701301097869873046875f,0.476599969604611484186307279742322862148284912109375f,0.32710002743601862551514614096959121525287628173828125f,1.4316000149786474171520467280060984194278717041015625f,1.0800000055506824292450573921087197959423065185546875f,0.7330000035539268044004757030052132904529571533203125f,0.37419998427778444760605225383187644183635711669921875f,0.63990000019073478565445611820905469357967376708984375f,-0.2165000215768813074390664041857235133647918701171875f,0.16700003589689771388293593190610408782958984375f,0.23460002490282061415882708388380706310272216796875f,0.12870001305937772695386911436798982322216033935546875f,-0.22760000636577615296118892729282379150390625f,-0.3864000132739544568494238774292171001434326171875f,-1.4274999952316278495345613919198513031005859375f,-0.29130000810027123492318423814140260219573974609375f,-1.411199995326995804134639911353588104248046875f,0.25440001179873927839025782304815948009490966796875f,-0.55499999865889559913512130151502788066864013671875f,0.7507000191539521249950439596432261168956756591796875f,0.5983999520897871349234264926053583621978759765625f,-0.880200010421872125476738801808096468448638916015625f,0.045299982440472064126879558898508548736572265625f,0.724200022685528299604129642830230295658111572265625f,0.0901000337839121812066878192126750946044921875f,-0.2752999984592197080246478435583412647247314453125f,0.207200007724761547933667316101491451263427734375f,-0.222299996209144534731194653431884944438934326171875f,-0.687200011301040802891293424181640148162841796875f,-0.3883999571561815855602617375552654266357421875f,0.883900036221742624320540926419198513031005859375f,-0.59170001761913226800970733165740966796875f,0.9463000523567206556663222727365791797637939453125f,-0.045800019335747066406838712282478809356689453125f,0.2888999968826764241924820453277789056301116943359375f,0.01560001524686871476887972676195204257965087890625f,-0.451399997776746442212925103376619517803192138671875f,-0.78950000438094125598809114308096468448638916015625f,-0.239199975597857328324380432604812085628509521484375f,0.1854999373555177299977003713138401508331298828125f,0.74610000699162437598488395451568067073822021484375f,-1.1053999870181083764464347041212022304534912109375f,-1.25800006327033120356873041600920259952545166015625f,1.40660000708103183342245756648480892181396484375f,0.78550002557039277917283470742404460906982421875f,-0.1404999981522561114388736314140260219573974609375f,-1.4642000182151804210661794058978557586669921875f,0.9027000170528882705411888309754431247711181640625f,0.14090001027882070872010444873012602329254150390625f,1.099899998104572063084560795687139034271240234375f,0.8615000002682204893744710716418921947479248046875f,0.6221999746441839729271805481403134763240814208984375f,1.163999996840953343735236558131873607635498046875f,-0.29400001323223090565761594916693866252899169921875f,-0.28339999344050870178080003825016319751739501953125f,1.156600051337480561386428234982304275035858154296875f,-0.1911999845236536099690738410572521388530731201171875f,-0.9393000372469426650923196575604379177093505859375f,-0.208399981817602775180375829222612082958221435546875f,0.9543000451445582399401246220804750919342041015625f,1.323100032830238692582724979729391634464263916015625f,0.42959998615980143910064725787378847599029541015625f,0.01719999289810658638089080341160297393798828125f,-0.3599000062257051713032751649734564125537872314453125f,0.23960000653713942053713026325567625463008880615234375f,0.0720999860107902623695963484351523220539093017578125f,0.437499998658895439263005755492486059665679931640625f,1.341299964663386301566561087383888661861419677734375f,-0.24870000870078812216235064624925144016742706298828125f,0.7985999845281244180483781747170723974704742431640625f,-0.4758999968677757319568399907439015805721282958984375f,0.48300001893937594132921731215901672840118408203125f,0.28749997854232756111514390795491635799407958984375f,-0.10749996423721341898271930404007434844970703125f,-0.20490001188814677135496822302229702472686767578125f,-0.4047999851077788147080127600929699838161468505859375f,0.183500023759901598197075145435519516468048095703125f,0.51279999492019412432597391671151854097843170166015625f,0.170500009234994587270506372078671120107173919677734375f,0.51239998551309107188700409096782095730304718017578125f,0.0428000238656995879438227348146028816699981689453125f,0.965900035722553784722776981652714312076568603515625f,-0.84520000263601513523781250114552676677703857421875f,-1.185900015568732701609633295447565615177154541015625f,0.783900010293721916099229929386638104915618896484375f,-0.4545000085532662215115351500571705400943756103515625f,0.4780000039637088971034017959027551114559173583984375f,0.1888999991923572263630148881929926574230194091796875f,0.4456999922946096592824005711008794605731964111328125f,-0.37340003896355700163667279412038624286651611328125f,0.01760003497600592936578323133289813995361328125f,0.9576000492438681011009293797542341053485870361328125f,-0.5868000436544422715456903461017645895481109619140625f,-0.37080002091899533223084972632932476699352264404296875f,0.995900004728138554810357163660228252410888671875f,-0.861300029911845843599849104066379368305206298828125f,0.975900016667693659400129035930149257183074951171875f,-0.7975000100582843032270830008201301097869873046875f,-0.710700007009506418143018890987150371074676513671875f,-0.11529999249875533262610360907274298369884490966796875f,-1.00319998838305490806988018448464572429656982421875f,0.085699992853403017534219543449580669403076171875f,-0.573499998316168824175065310555510222911834716796875f,-0.67600002476573006759963391232304275035858154296875f,-0.9025000280886892323195525023038499057292938232421875f,-0.96280003984719508824952072245650924742221832275390625f,-1.13010001873373955305623894673772156238555908203125f,-1.255100052261352683302675359300337731838226318359375f,-0.0272999936759468209146461958880536258220672607421875f,0.03409998703896999217022312222979962825775146484375f,-0.43580002253949656765286135851056315004825592041015625f,0.9675999891176820799643110149190761148929595947265625f,-0.323000008359551227243855464621447026729583740234375f,0.2979999941661954920846255845390260219573974609375f,-0.074099985027313408636473468504846096038818359375f,0.348700053218007344213447140646167099475860595703125f,1.1930999776959421154742813087068498134613037109375f,-0.389500028222799610233550993143580853939056396484375f,0.2805999878287313009650461026467382907867431640625f,-0.406700002866983822968904860317707061767578125f,-2.229100024366378107743003056384623050689697265625f,-1.0405000115819273975859005076927132904529571533203125f,-1.15249998683109833308435554499737918376922607421875f,0.09559998190552010388643111582496203482151031494140625f,0.372300035138427798386828726506792008876800537109375f,1.3891999896049498364192231747438199818134307861328125f,0.1091000023126600382283868384547531604766845703125f,-0.307600004726648723618609437835402786731719970703125f,0.1858999619245533807543324655853211879730224609375f,-0.2091999922871590200657010427676141262054443359375f,-0.081800022792815951788725215010344982147216796875f,0.540600002431869341990022803656756877899169921875f,-0.386700039672852380334688859875313937664031982421875f,0.56590003315210424261749722063541412353515625f,-1.531100043499469709473714829073287546634674072265625f,1.963400016462802710748292156495153903961181640625f,0.8394000245392323478199614328332245349884033203125f,-0.4142000059962267499713561846874654293060302734375f,2.572000117704273947083493112586438655853271484375f,-1.857300083716214356144291741657070815563201904296875f,-0.01300004766136531042519663969869725406169891357421875f,-0.21239994455352440017037451980286277830600738525390625f,-0.440699988383054463980670334422029554843902587890625f,-0.97199999707937223547560279257595539093017578125f,-0.36469997985958979569431903655640780925750732421875f,0.20059995802640884221546002663671970367431640625f,0.1260000135898593498495756648480892181396484375f,0.132499992847442626953125f,1.462799999129771322259330190718173980712890625f,-0.756000064998865628496105273370631039142608642578125f,0.416300004822015790040268257143907248973846435546875f,0.631600040459633138567596688517369329929351806640625f,0.96969998291432801806877250783145427703857421875f,1.3243999998033046949785784818232059478759765625f,-0.425200007814168667863441442023031413555145263671875f,0.4424999896809456156887563338386826217174530029296875f,-1.8040000324547287302578979506506584584712982177734375f,-1.306099989818035655986250276328064501285552978515625f,-1.334099992384761179664565133862197399139404296875f,-0.90249998196959513308001987752504646778106689453125f,0.47500001378357392667339809122495353221893310546875f,1.210700037258863659417329472489655017852783203125f,0.8713000118255613823947669516201131045818328857421875f,0.37639999136924728873054846189916133880615234375f,0.063299996984004902600418063229881227016448974609375f,-0.52270000826120366355098667554557323455810546875f,-0.0177999726802102742340139229781925678253173828125f,-0.045699993747473133254288768512196838855743408203125f,-0.207899999609589103410911548053263686597347259521484375f,0.4010000095665446906423312611877918243408203125f,-0.90040002889037129563121197861619293689727783203125f,-0.312499980628490714451572785037569701671600341796875f,1.297300048176944553546263705356977880001068115234375f,1.1659999878481033253052601139643229544162750244140625f,-0.22800002013146858370618019762332551181316375732421875f,1.1343999907508486302276651258580386638641357421875f,-0.77359999412074653690751802059821784496307373046875f,-1.38899999691545961155725308344699442386627197265625f,-0.85250002846121775146315258098184131085872650146484375f,-1.28679999150037804866997248609550297260284423828125f,0.401400009399652812902559162466786801815032958984375f,-0.11519998948574095010144446860067546367645263671875f,-0.379199973213673047212068922817707061767578125f,0.8212000363796949642392064561136066913604736328125f,-0.787400028353929304358871377189643681049346923828125f,-1.00450007173419120221069533727131783962249755859375f,0.635600055405497688099103470449335873126983642578125f,1.373000032499432254695648225606419146060943603515625f,-0.350100007036328264575786306522786617279052734375f,-0.6352000264406203466904798915493302047252655029296875f,0.1490000285059214224503421064582653343677520751953125f,-1.45090004550144069384032263769768178462982177734375f,1.273200014944374292014117600047029554843902587890625f,0.227799988792091401279549245373345911502838134765625f,0.866399992170185040407659471384249627590179443359375f,1.8813000599563125536661800651927478611469268798828125f,-0.19309999289512713716021607979200780391693115234375f,-0.18340002197623306301466072909533977508544921875f,0.37320004202723566777422092854976654052734375f,0.15510005688071260010474361479282379150390625f,0.57549999517202277132810195325873792171478271484375f,0.706100045883655713652160557103343307971954345703125f,0.082100004041194996062813515891321003437042236328125f,-0.7763999799698588777374652636353857815265655517578125f,0.11430001668334011100114366854541003704071044921875f,-0.5296000253498556187281565144076012074947357177734375f,-2.123500042721629288422491299570538103580474853515625f,0.1970000294893987646815958214574493467807769775390625f,-0.08659995239377005216141469645663164556026458740234375f,0.14010000271499156720977907752967439591884613037109375f,1.180400019428133884247245077858678996562957763671875f,0.413700015480817029356330749578773975372314453125f,0.4117000074863437220074047218076884746551513671875f,-0.354600013279914971775497178896330296993255615234375f,-0.7038000285148626744557986967265605926513671875f,0.12539999834299064840337223358801566064357757568359375f,-0.496300003480911300357547588646411895751953125f,-0.401400007760524868416496246936731040477752685546875f,-1.3389999825358387397500337101519107818603515625f,-0.3881999782502649765092428424395620822906494140625f,-0.434699999082088339719120995141565799713134765625f,-0.102600016275048222524901575525291264057159423828125f,-0.5726999861329795660225272513343952596187591552734375f,0.89249999251216671769526556090568192303180694580078125f,-0.9146999473750592191123587326728738844394683837890625f,-0.592400021499395190716086290194652974605560302734375f,-0.520500006198883369279428734444081783294677734375f,0.118800006908178712450308012194000184535980224609375f,-1.0757999963879587568271745112724602222442626953125f,0.07060003364980216300494930692366324365139007568359375f,0.46219999401569378250087538617663085460662841796875f,-0.031299993795156577647276208153925836086273193359375f,0.60799998176097869162504139239899814128875732421875f,0.3156000119686126481610699556767940521240234375f,0.945600042665005080522178104729391634464263916015625f,0.12929998077154181146397604607045650482177734375f,0.605699995088576859103568494901992380619049072265625f,0.32470001502633127898889142670668661594390869140625f,-0.0670000130608678912125242277397774159908294677734375f,-0.2113999830245971711661923109204508364200592041015625f,0.909100051896274408136378042399883270263671875f,-2.02750001620501318910783083993010222911834716796875f,-0.18690004093051104661071804002858698368072509765625f,-0.155799944829940528734368854202330112457275390625f,1.114600064837933057759755683946423232555389404296875f,-1.0365000311136238764220252051018178462982177734375f,-0.1519000211119656551517209663870744407176971435546875f,0.30899998730421085468833553022705018520355224609375f,-0.8115000253021715792556278756819665431976318359375f,0.8107000038802620878186644404195249080657958984375f,-0.4887000251293187602641410194337368011474609375f,0.7937999866425986539297809940762817859649658203125f,0.77119998318254889824174824752844870090484619140625f,0.072599957489967170687350517255254089832305908203125f,-0.219900012558698509934629328199662268161773681640625f,-0.1291999949693678484408110307413153350353240966796875f,-1.1668000099033115635194235437666065990924835205078125f,0.0528999944686887602074421010911464691162109375f,0.1889000430762763915737423303653486073017120361328125f,1.6100000493228439069071100675500929355621337890625f,-0.02129993188083201260951682343147695064544677734375f,0.916999986052511761869254769408144056797027587890625f,0.795100006514787072120498123695142567157745361328125f,0.366599978470802501107073112507350742816925048828125f,0.956800005954504317884357078582979738712310791015625f,0.0290000036954882745021677692420780658721923828125f,0.23979999807178986603872772320755757391452789306640625f,0.81919997857809079278013086877763271331787109375f,1.460600006678701046070045777014456689357757568359375f,0.550100016200541830357906292192637920379638671875f,0.0026000009268525303696151240728795528411865234375f,0.1698000154316421372868717298842966556549072265625f,-0.09090002197623248125779582551331259310245513916015625f,-0.68549996164441129309352618292905390262603759765625f,-0.72520000766515746448703794158063828945159912109375f,-0.814599994504451530019650817848742008209228515625f,0.590699986892938344595904709422029554843902587890625f,-0.3777000033438202564184393850155174732208251953125f,-0.43990003058910398525682694526039995253086090087890625f,-0.37090000320076921269674130599014461040496826171875f,0.3514000090271236498296048011980019509792327880859375f,0.2877999994277953277332926518283784389495849609375f,0.11639999777674692182927174144424498081207275390625f,0.6798000075340269177104346454143524169921875f,0.50300001245737036015270859934389591217041015625f,-1.632600039893388643719163155765272676944732666015625f,0.53870005895495420844554246286861598491668701171875f,0.1424999809265126149426805568509735167026519775390625f,-1.242299978178738939504910376854240894317626953125f,0.037700012955069528430840364308096468448638916015625f,-0.719200016948580600484319802490063011646270751953125f,1.244000005334614744612053982564248144626617431640625f,-1.23149998745322175608407633262686431407928466796875f,0.5338000404357916561792762877303175628185272216796875f,-1.68760003333687791382544673979282379150390625f,0.215300043609739422123539043241180479526519775390625f,-0.193500050619245467231621660175733268260955810546875f,0.462900009071826712414576832088641822338104248046875f,-0.79089996952414498565531175700016319751739501953125f,-0.7510000302791599580132242408581078052520751953125f,0.1992000089764598413921703468076884746551513671875f,-0.40809999483227699101917096413671970367431640625f,-1.384600016558171109437580525991506874561309814453125f,0.5239000335395331831023213453590869903564453125f,0.310199995520710647411988247768022119998931884765625f,-1.0009999739527704054609102968242950737476348876953125f,0.9152000139981504300834558307542465627193450927734375f,-0.0424999923259026246569192153401672840118408203125f,0.542099990779161711174083393416367471218109130859375f,-1.68390000545084461691658361814916133880615234375f,-0.622199977251887137441599406884051859378814697265625f,-1.0386000021487469258119062942569144070148468017578125f,0.8971000555247072849596179366926662623882293701171875f,0.293000006049871419833152685896493494510650634765625f,-0.76400003573298480574749191873706877231597900390625f,-0.7767999935120346588490747308242134749889373779296875f,-0.2408999860644343016247148625552654266357421875f,0.00900001107156267465825294493697583675384521484375f,0.0320999915242197175757610239088535308837890625f,-0.382800000973790854885692169773392379283905029296875f,0.74569998167753137607860480784438550472259521484375f,-0.8755000308789318008706459295353852212429046630859375f,1.52980004732012719870226646889932453632354736328125f,-0.8473000327542425935689607285894453525543212890625f,0.47160003867149402623226706054992973804473876953125f,-1.402100048074126359409774522646330296993255615234375f,0.45580000786185337346978485584259033203125f,-0.267699993006140168905204745897208340466022491455078125f,0.4043000707745552091409990680404007434844970703125f,-0.62410001725107366876699188651400618255138397216796875f,0.087499912977218485821140347979962825775146484375f,-0.75109996746629459352107005543075501918792724609375f,1.58760002782344855631890823133289813995361328125f,0.051300010298192422197871565003879368305206298828125f,-0.23280000896453856285006622783839702606201171875f,0.2177999807268384824254781051422469317913055419921875f,-0.3976000243589279392608659691177308559417724609375f,-0.072499972134828727376998358522541821002960205078125f,1.269399996487795778676854752120561897754669189453125f,-1.5240000134557487587727564459783025085926055908203125f,1.0547999560505143534783201175741851329803466796875f,-0.599900006821751929919628310017287731170654296875f,1.1394999914169308130595936745521612465381622314453125f,0.359699935230612499736935205874033272266387939453125f,0.01360001380890574207427334840758703649044036865234375f,-1.297400013005733487858606167719699442386627197265625f,1.0847999564975501218810904902056790888309478759765625f,0.909300005805492528310196576057933270931243896484375f,-0.1437000205844629618923136149533092975616455078125f,-1.753500000849366191602030085050500929355621337890625f,-0.41520000386536093373024414177052676677703857421875f,0.3845000057220460121243377216160297393798828125f,1.0564000187873840985730566899292171001434326171875f,0.577600030803680386526366419275291264057159423828125f,0.0836999893665311134327566833235323429107666015625f,-0.6475000044703487223074489520513452589511871337890625f,0.5107999995544554305837436913861893117427825927734375f,-0.57530002978891159326479964875034056603908538818359375f,-0.40799999811500331503566485480405390262603759765625f,0.690500002697110204508135211654007434844970703125f,-0.68129997889399618316019768826663494110107421875f,-1.034099970424174674832329401397146284580230712890625f,2.15010002782344855631890823133289813995361328125f,0.021500020161270949614618075429461896419525146484375f,0.8338999962121247389035261221579276025295257568359375f,-0.5094000158220530760644351175869815051555633544921875f,-0.1329000332862140254519545123912394046783447265625f,0.4373000069379802123847866823780350387096405029296875f,-0.1307000242948532786613213829696178436279296875f,-0.40839999053478237556191743351519107818603515625f,0.47639998525977100829464916387223638594150543212890625f,-1.06790004632473056744856876321136951446533203125f,-0.12330001548007130107720286105177365243434906005859375f,0.734100029954314781122093336307443678379058837890625f,-0.6746000149190425343448396233725361526012420654296875f,0.561699993628263616329832075280137360095977783203125f,-0.2018999804504211237343724860693328082561492919921875f,-0.1800000539422033085656948969699442386627197265625f,1.29470004043281061711923030088655650615692138671875f,-0.61669997053146374810239649377763271331787109375f,0.673899988128245031049345925566740334033966064453125f,-0.9950000362843274270829851957387290894985198974609375f,0.0314999906942243779184309460106305778026580810546875f,-0.228399972355366021048439506557770073413848876953125f,0.114499984711408708815127965863212011754512786865234375f,-0.4852000049829481298502287245355546474456787109375f,-0.329799995315074756518924914416857063770294189453125f,-0.6989999800026420739840204987558536231517791748046875f,1.431400009399652173414096978376619517803192138671875f,-0.3581000142037871203370968942181207239627838134765625f,-0.43300001107901342489725493578589521348476409912109375f,0.8351999857604501809049679650343023240566253662109375f,-0.41520000554174163465148694740491919219493865966796875f,-0.839100008906423777688132759067229926586151123046875f,0.426600003281235817809147192747332155704498291015625f,-1.6496999551236621872618570705526508390903472900390625f,-0.16929996952116521669040594133548438549041748046875f,0.39460000851154308776358448085375130176544189453125f,1.209500006690621631832982529886066913604736328125f,0.1029000166714186814687082005548290908336639404296875f,-0.103299996834993468297625440754927694797515869140625f,0.49060001196861235950308355313609354197978973388671875f,0.34749998450279218076275355997495353221893310546875f,-0.0652000027477743770987217430956661701202392578125f,0.1335000038295983681990719560417346656322479248046875f,0.037100024865567515774955609231255948543548583984375f,-0.683600016453862036058808371308259665966033935546875f,-0.764900020381807710379007403389550745487213134765625f,1.50910003367960410258774572866968810558319091796875f,0.50479999531507502297245082445442676544189453125f,0.3112000126868483818043387145735323429107666015625f,0.9447000318646434635638797772116959095001220703125f,-1.346800029274822119162990929908119142055511474609375f,0.27169996740221957765015758923254907131195068359375f,-0.032300001350045182135772847686894237995147705078125f,-1.5784000425398350131445113220252096652984619140625f,1.3930000382736322794841044014901854097843170166015625f,1.1251000091969967087379700387828052043914794921875f,-0.1131999909907579482393202852108515799045562744140625f,0.8737000107124448522455395504948683083057403564453125f,-0.1665999987363815382224174754810519516468048095703125f,-0.7079000168949363125392437723348848521709442138671875f,1.370099985988438628936592067475430667400360107421875f,-0.248600001496822375912643110495992004871368408203125f,0.200900045743583888224748079665005207061767578125f,0.5500999903656544898211677718791179358959197998046875f,-0.315700000751018450273477355949580669403076171875f,0.08929996186569244542852175072766840457916259765625f,0.9909000552058220279150191345252096652984619140625f,-0.90620003814920746521011096774600446224212646484375f,1.6000000214576726165205400320701301097869873046875f,0.12150001123920051160354205421754159033298492431640625f,-1.0855999762058257118724213796667754650115966796875f,0.7339999940097332586930178877082653343677520751953125f,0.0426000325918203248676263683591969311237335205078125f,0.2426999983519315673419214363093487918376922607421875f,0.4431000193074348114663507658406160771846771240234375f,0.30919999493211502983314176162821240723133087158203125f,0.641299978148937466215784297673963010311126708984375f,-0.6779000121638178999461388229974545538425445556640625f,-1.059499988101423095798736540018580853939056396484375f,-0.25569999791979824976806412450969219207763671875f,-0.538300005291402872131811818690039217472076416015625f,0.37459998668134186772249449859373271465301513671875f,-0.95549996697157713043679905240423977375030517578125f,-0.31160003330707564117574293049983680248260498046875f,0.6657999327972523584406872032559476792812347412109375f,-1.209699995952844719937502304674126207828521728515625f,-1.6708000033542516060691696111462078988552093505859375f,1.0955999889463183816218361243954859673976898193359375f,-0.1394999643340708583849618662497960031032562255859375f,-0.18280002725571409261107191923656500875949859619140625f,-0.146999995812773676107099163345992565155029296875f,-0.36229999989718197905830265881377272307872772216796875f,-0.58610000982284571335867440211586654186248779296875f,-0.2004000241965060435944678829400800168514251708984375f,0.02990000853538532510356162674725055694580078125f,-0.364000026196241943665654616779647767543792724609375f,-0.87380004326701143924083226011134684085845947265625f,0.808600004011392314851036644540727138519287109375f,0.390900012886524539368338082567788660526275634765625f,-0.22180001564025875193664205653476528823375701904296875f,-2.09980001587867715073798535740934312343597412109375f,-0.468500053003430227960279808030463755130767822265625f,-0.328599936807155046381012653000652790069580078125f,0.156599976235627824650009642937220633029937744140625f,-1.71860003724098220345695153810083866119384765625f,-0.131200044053793707377053578966297209262847900390625f,-1.380000063031912471700479727587662637233734130859375f,0.34250000014901138545297953896806575357913970947265625f,-0.41150001084804532780481167719699442386627197265625f,0.45030000181198115427605443983338773250579833984375f,-0.6687000321142380077077405076124705374240875244140625f,0.998300020527839393480462604202330112457275390625f,0.08269999056607468812529759816243313252925872802734375f,-0.571700001823902681508116074837744235992431640625f,-0.40810003491640112205374180120998062193393707275390625f,1.086500000566243517852171862614341080188751220703125f,-1.57150001990050025568734781700186431407928466796875f,1.718899981087446349192759953439235687255859375f,-0.967799986035376758053416779148392379283905029296875f,1.667900063014030820340849459171295166015625f,-0.91610004443079284808248985427781008183956146240234375f,0.3744999642968167563594761304557323455810546875f,-1.14800003007799400478461393504403531551361083984375f,1.576900046294927637546834375825710594654083251953125f,-0.341100008109211927376236417330801486968994140625f,0.4328000246107581716614731703884899616241455078125f,0.005999986320734063127702029305510222911834716796875f,0.64130002288967313806011816268437542021274566650390625f,-1.0250000218302013355042845432762987911701202392578125f,-1.4513000068336732173435166259878315031528472900390625f,0.90630000482201567013618159762700088322162628173828125f,0.845600003921985088339852154604159295558929443359375f,-0.7581000100314618794783427802030928432941436767578125f,-0.3107001067727803711449041657033376395702362060546875f,-0.176500000640749998837009115959517657756805419921875f,0.3932999942645432067678257226361893117427825927734375f,-2.08850002856552574570514480001293122768402099609375f,0.1426999879211179322879843311966396868228912353515625f,0.74710000001788134937896757037378847599029541015625f,0.1892999646037807348619708136538974940776824951171875f,-1.297200009289384059485428224434144794940948486328125f,0.375000009834766390159899174250313080847263336181640625f,-0.6213000237464907371531808166764676570892333984375f,-0.49009998086616402002135828297468833625316619873046875f,-0.58660001585409038415974691815790720283985137939453125f,-0.7828000517122450929718979750759899616241455078125f,0.999100019150972240566943582962267100811004638671875f,0.66170000346302959570721213822253048419952392578125f,-0.11419997798204395422771995072253048419952392578125f,0.6059000012636186482950506615452468395233154296875f,-0.6083000194102525792771984924911521375179290771484375f,-0.518800002810359206506518603418953716754913330078125f,-0.32780000680387022526929285959340631961822509765625f,-0.71810001669973144711178747456870041787624359130859375f,0.8641000176608562011182357309735380113124847412109375f,0.02610001978799714184020785978646017611026763916015625f,-1.25920003179386252867288931156508624553680419921875f,-0.285200030184537212107898085378110408782958984375f,0.377899999386072327212104937643744051456451416015625f,-0.84379999349713319389820753713138401508331298828125f,0.643999982722103680998770869337022304534912109375f,-0.128299987167865114923159808313357643783092498779296875f,0.1695000227838752149267520508146844804286956787109375f,0.68050001085549605139846107704215683043003082275390625f,-0.5165999754905694629769641323946416378021240234375f,0.10749999446794389346138132168562151491641998291015625f,0.069299988520145294756957810022868216037750244140625f,-0.745700013435631969826999920769594609737396240234375f,0.384499991342425406770644258358515799045562744140625f,0.236600020641088537587393147987313568592071533203125f,-1.080899998469650569177247234620153903961181640625f,-0.13240002529919170370931169600225985050201416015625f,0.277299982947111001152507014921866357326507568359375f,-0.64889999712482071547725581694976426661014556884765625f,0.9464000151365994373264811656554229557514190673828125f,-0.17319999601989988224914895909023471176624298095703125f,-0.59670001121163362967081411625258624553680419921875f,-0.10360001561567173944400366281115566380321979522705078125f,-0.96050002828985459046862160903401672840118408203125f,-0.1582999739058317256734653710736893117427825927734375f,0.2818000434309244628394708342966623604297637939453125f,-0.11570002138912671707515755770145915448665618896484375f,0.8181000084295872998296772493631578981876373291015625f,-0.08479999121725578792307942421757616102695465087890625f,1.2204999732300638726201214012689888477325439453125f,-0.4067999993234872047054295762791298329830169677734375f,-0.8735999984234570359120652938145212829113006591796875f,0.71839999664425846503945649601519107818603515625f,0.573800029353052298830561994691379368305206298828125f,0.265699977784603802444252096393029205501079559326171875f,0.38259994463771584971567563115968368947505950927734375f,-1.0568999719940126436057425962644629180431365966796875f,0.4396000069469214732720274696475826203823089599609375f,0.431599984580278484003201810992322862148284912109375f,-0.4930999991536140214520855806767940521240234375f,-0.604500016935169970366814595763571560382843017578125f,0.0956000084854664589073536262731067836284637451171875f,0.00709999105855825540789538763419841416180133819580078125f,-0.27849999250471590972466628954862244427204132080078125f,-0.3133000083833932247756592914811335504055023193359375f,-0.3763000256091355577581225588801316916942596435546875f,1.2355000232607127319539586096652783453464508056640625f,-0.492399988418817269320015839184634387493133544921875f,-0.058400018250942320463536816532723605632781982421875f,0.7743000242084259188146688757115043699741363525390625f,-0.189199949222802865644865732974722050130367279052734375f,2.406399998223781810935406610951758921146392822265625f,0.6544000435382122393690451644943095743656158447265625f,1.3797000005722050985923488042317330837249755859375f,-0.106299988430738068245773320086300373077392578125f,1.4161000333666802220022873370908200740814208984375f,0.473899991667270370498954434879124164581298828125f,-0.3412000027962027726147198336548171937465667724609375f,1.078400050195307269262912086560390889644622802734375f,-0.34960000527054069241472689100191928446292877197265625f,0.3583999951727687705016478503239341080188751220703125f,0.20230001667961483580171488938503898680210113525390625f,-0.4233999870330096282344811697839759290218353271484375f,-0.369600020469725176752717743511311709880828857421875f,0.47409998480379567098452753270976245403289794921875f,0.5068000430583958770824892781092785298824310302734375f,0.0707000280946499248813097437960095703601837158203125f,1.640299995758383477806319206138141453266143798828125f,-1.4441999779075391785454485216177999973297119140625f,1.5972000121392315730162181353080086410045623779296875f,-1.8294000199198716671844522352330386638641357421875f,0.600700009728968087330258640577085316181182861328125f,0.1359999989867208824989575077779591083526611328125f,0.186199989068507765210824800305999815464019775390625f,-0.58180002860426915933800273705855943262577056884765625f,-0.8605000178962944890059816316352225840091705322265625f,0.411699992492049915426832740195095539093017578125f,-0.22150000272691239278088914943509735167026519775390625f,-0.59210000854879607512515349299064837396144866943359375f,0.176400012007355666554531126166693866252899169921875f,0.8486000029683113954348527840920723974704742431640625f,0.07439997953772536920524771630880422890186309814453125f,-1.2818000058054923595562968330341391265392303466796875f,-0.7297999896526332630486422203830443322658538818359375f,-0.59729990542382027651768794385134242475032806396484375f,0.67160001669228019949287045164965093135833740234375f,-1.477399990169704135922756904619745910167694091796875f,1.7006999912887810211259420611895620822906494140625f,-0.14319999859035004163843041169457137584686279296875f,0.00670002208948172750524463481269776821136474609375f,0.1627000093787909218434606373193673789501190185546875f,0.1912999864190811916131451653200201690196990966796875f,-0.353999978363514156143310174229554831981658935546875f,0.4144000234961513484677197993732988834381103515625f,0.41230001707076979755584034137427806854248046875f,0.1673000242978333051979689116706140339374542236328125f,-0.1126999956324692409026511086267419159412384033203125f,-0.8349000400140884625699300158885307610034942626953125f,0.4164000174090265460336013347841799259185791015625f,1.034200050625205591359190293587744235992431640625f,-0.3230000050067900474459747783839702606201171875f,0.170699996057152247885824181139469146728515625f,0.7349000344634060155613042297773063182830810546875f,-0.9730000267252327805778122638002969324588775634765625f,-0.387000023603439213815136099583469331264495849609375f,0.36609998687505729275670773859019391238689422607421875f,-0.7551999994322657894230133024393580853939056396484375f,0.3972000069051977444445356013602577149868011474609375f,-0.233000000946223195796846994198858737945556640625f,0.33589997794330184177624687436036765575408935546875f,0.025900001338123868066531940712593495845794677734375f,0.839199993777274499962004483677446842193603515625f,-0.5721000334709884782569133676588535308837890625f,1.5790000322312121028289766400121152400970458984375f,-0.1885999796107407089351681861444376409053802490234375f,0.925900047904253842290245302137918770313262939453125f,0.229999991636722800336656291619874536991119384765625f,-0.5471000305652626849450825829990208148956298828125f,-0.07390001929029832439255187637172639369964599609375f,1.0581000536918647725315167917869985103607177734375f,-0.5191000096514819528437101325835101306438446044921875f,0.7514000022470950224118269034079276025295257568359375f,-0.37629999604895714160335273845703341066837310791015625f,0.0887000001326201381601777029572986066341400146484375f,-0.2334000144511463314955790337990038096904754638671875f,-1.2002000224173070552780018260818906128406524658203125f,0.3213999819815160918778929044492542743682861328125f,-1.52589999485611915730487453402020037174224853515625f,-0.2134999786466362792225481825880706310272216796875f,-0.7942000194445253402619755433988757431507110595703125f,-0.4489000279143458005393085841205902397632598876953125f,-0.4252000314325095065015602813218720257282257080078125f,-1.2942000027552242880091171173262409865856170654296875f,-0.53499998170882445691631801309995353221893310546875f,0.9927000072926277507434633662342093884944915771484375f,-0.57179999429434591462495518499054014682769775390625f,-0.756600039863586371069459346472285687923431396484375f,0.0389999545589088114638798288069665431976318359375f,-0.30209998403638593789111155274440534412860870361328125f,0.412899996294081195191694177992758341133594512939453125f,-0.28960000504702321233452266824315302073955535888671875f,-0.491600037330389483258841210044920444488525390625f,0.2278999990135430753213086063624359667301177978515625f,-0.9686000027820467206396415349445305764675140380859375f,-0.9310000158250328894382619182579219341278076171875f,-0.3478999830320479436096547942725010216236114501953125f,0.0139000010177496857721735068480484187602996826171875f,-1.1972999665185806161815662562730722129344940185546875f,-1.2720000341087580597587702868622727692127227783203125f,-0.625500012159347473783554960391484200954437255859375f,-0.3813000303030011917115871256100945174694061279296875f,-0.6263999857068061061227126629091799259185791015625f,0.063199976760148768306635247427038848400115966796875f,0.205699979665874277401371728046797215938568115234375f,0.9075000353902578087428310027462430298328399658203125f,-0.52140002385377925975262769497931003570556640625f,-0.252100032129884521481244519236497581005096435546875f,-0.6758000407934190434389165602624416351318359375f,0.4732000117778778491839375419658608734607696533203125f,-0.69040000665038803173700898696552030742168426513671875f,0.0661999812081457417178853575023822486400604248046875f,-0.603299989440291728470810994622297585010528564453125f,0.290399970552325470407595275901257991790771484375f,-0.9254000151067973689578138873912394046783447265625f,0.3458999709397552191347813277388922870159149169921875f,-0.49279998027980276287252081601764075458049774169921875f,0.451600002983212789331446401774883270263671875f,-0.477999999511987072597918313476839102804660797119140625f,-0.008900009548664211678214996936731040477752685546875f,0.8027000417888168204427756791119463741779327392578125f,-1.536000028416514506801604511565528810024261474609375f,-0.313600062144548719089698352036066353321075439453125f,0.1486999613523476426735214772634208202362060546875f,-0.82630001482442005933393147643073461949825286865234375f,1.449100044631958184027098468504846096038818359375f,-0.37429999667853108835657849340350367128849029541015625f,1.0322999942719939259205830239807255566120147705078125f,-0.26180001768916871807135748895234428346157073974609375f,0.62579995466470761389388144380063749849796295166015625f,-0.2239999833181498256617913966692867688834667205810546875f,0.3318999972701075495962186323595233261585235595703125f,0.237199980194866544280785092269070446491241455078125f,-0.4981000653892753149420968838967382907867431640625f,-0.399599979975819419308891156106255948543548583984375f,1.359900010807812531510307962889783084392547607421875f,-0.1615000054836268628122297741356305778026580810546875f,-0.46749997407197962928648848901502788066864013671875f,0.4239000300377611107904840537230484187602996826171875f,0.82110006864517892655186415140633471310138702392578125f,1.065899979060887847737149058957584202289581298828125f,-0.2909000254407529961753198222140781581401824951171875f,-1.5102000103473667280695735826157033443450927734375f,0.36759996941089667643609573133289813995361328125f,0.0818999998033052012402777108945883810520172119140625f,1.49449998277425688542052739649079740047454833984375f,0.11049999085068673565501740085892379283905029296875f,0.336600051635503660918402601964771747589111328125f,0.8480000523924833277789048224803991615772247314453125f,-0.95289997867345821447315756813623011112213134765625f,1.462400050258636508004883580724708735942840576171875f,-0.3424000223934644537848726031370460987091064453125f,1.4084000404536727302229337510652840137481689453125f,-1.1652000046849249059732755995355546474456787109375f,-0.0632999704599370716096018441021442413330078125f,0.89809995653629215439650579355657100677490234375f,0.5629999838471402284767464152537286281585693359375f,-0.249399952864646223815725534223020076751708984375f,0.00669997947215961175970733165740966796875f,-0.067299992036819134000324993394315242767333984375f,0.5322999762415889790645451284945011138916015625f,0.3469000093400484985295406659133732318878173828125f,-0.03650003536045520746711190440692007541656494140625f,-0.3674999982118603014669133699499070644378662109375f,0.05040000363290297880070056635304354131221771240234375f,0.50400002157688117421230344916693866252899169921875f,-1.46970000653266996692991597228683531284332275390625f,1.354800024968385141477256183861754834651947021484375f,-0.22390001200735609732106468072743155062198638916015625f,-0.89970003693103794972785181016661226749420166015625f,-1.6909000291287892281388849369250237941741943359375f,0.8020000489354135453368144226260483264923095703125f,0.946399976691603139755670781596563756465911865234375f,-0.0637000784754757631844768184237182140350341796875f,0.49570001282095876860012140241451561450958251953125f,0.4798999987006187240012877737171947956085205078125f,-0.0239999901354313038837062777020037174224853515625f,0.2855999976634979731215935316868126392364501953125f,0.1088999828755852217199162623728625476360321044921875f,0.503099985742569089097742107696831226348876953125f,-0.10559998335838305383305169016239233314990997314453125f,0.2751999609500170862474988098256289958953857421875f,-0.064900007082522026546911320110666565597057342529296875f,-0.757499998882412928224994175252504646778106689453125f,0.3159999935477968424635264454991556704044342041015625f,-0.52399997717142099418197176419198513031005859375f,0.735200049984455095142266145558096468448638916015625f,1.64299996745586440738406963646411895751953125f,-0.079500009968876794719250256093800999224185943603515625f,1.02319999203383904529118808568455278873443603515625f,-0.2086000166028740832047105868696235120296478271484375f,-0.081000014632940153802564964280463755130767822265625f,0.146099987918138385367683440563268959522247314453125f,-0.0746000055313107424126428668387234210968017578125f,-0.9943999999523160937542343162931501865386962890625f,-0.287199984031915267479462272603996098041534423828125f,-0.1188000294089324171409316477365791797637939453125f,-0.70500001482665519692005773322307504713535308837890625f,0.7887000111967328308537616976536810398101806640625f,0.14010000789314513980343690491281449794769287109375f,-0.4200000119581817070724127916037105023860931396484375f,0.8147000551104550680747706792317330837249755859375f,-0.72490004325509094229573747725225985050201416015625f,-0.90080004645586075895380417932756245136260986328125f,-0.062199967193603100668042316101491451263427734375f,-1.3261000334411858769811942693195305764675140380859375f,0.5854999911487102082219280418939888477325439453125f,0.421200008663535019337587073096074163913726806640625f,-0.514999989867210405947162144002504646778106689453125f,0.1185000332593926231083969469182193279266357421875f,-0.356500024855137098711566068232059478759765625f,-0.39559998812675445378772565163671970367431640625f,0.6380000558197505000634919269941747188568115234375f,1.2082000295102595099905329334433190524578094482421875f,0.42659999229162881562160691828466951847076416015625f,0.3487000087007878779132852287148125469684600830078125f,0.63930000979155299223322117541101761162281036376953125f,-0.22070000320971006058101693270145915448665618896484375f,0.680599992150068278107255537179298698902130126953125f,1.4475999881118528644918796999263577163219451904296875f,-0.9082000558853149385640790569595992565155029296875f,-0.179299978610873100848266403772868216037750244140625f,0.10730003823041911470426157393376342952251434326171875f,0.045000008791685086606548793497495353221893310546875f,-0.217099990332126235870191521826200187206268310546875f,-1.033500013217329982495584772550500929355621337890625f,-0.25769998412132277820774106658063828945159912109375f,-0.1274000034689903859685955467284657061100006103515625f,0.2313999951690435263884637606679461896419525146484375f,0.09570001174062492399485790883773006498813629150390625f,-0.4041000023871659241336828927160240709781646728515625f,-0.66459999137520797685141360489069484174251556396484375f,-0.32950002598762484495864555356092751026153564453125f,0.723899995318055378135113642201758921146392822265625f,0.534299993586540011847318965010344982147216796875f,0.315900026446580994843316148035228252410888671875f,0.68509995033740977987690712325274944305419921875f,0.010499990478157972262351904646493494510650634765625f,-0.5805000500082968528658966533839702606201171875f,-1.1597000211358070576039835941628552973270416259765625f,-0.1954000054955482568175284541212022304534912109375f,0.750300013062358051030287242610938847064971923828125f,0.9397999924093485279996684766956605017185211181640625f,0.15010001519471416475681735391844995319843292236328125f,0.6223999836504459093333707642159424722194671630859375f,0.3104000004291533887368359501124359667301177978515625f,-0.83770001865476351188277703840867616236209869384765625f,1.106800001744926209568120611947961151599884033203125f,1.117000008627771823199736900278367102146148681640625f,-0.382999992340803174784014117904007434844970703125f,-0.42609997376203434527042190893553197383880615234375f,0.548699995326995093591904151253402233123779296875f,-1.35180006675124264603482515667565166950225830078125f,1.724500049829483572239041677676141262054443359375f,-0.682499974966047862068307949812151491641998291015625f,0.372499999105929902043499168939888477325439453125f,1.9103000523269173527296516112983226776123046875f,-0.86600000777840602239621148328296840190887451171875f,-1.0384000248447058378786778121138922870159149169921875f,1.2801000322937967634828737573116086423397064208984375f,-0.96370003813058158215199000551365315914154052734375f,1.00150002623349454466961105936206877231597900390625f,0.6061999921232459342945730895735323429107666015625f,0.67199998880922751087751976228901185095310211181640625f,1.17120000188350648073765114531852304935455322265625f,0.3390999998539692139587486963137052953243255615234375f,-0.7005999925971024833160072375903837382793426513671875f,-0.61299997773766534692185814492404460906982421875f,1.6054000264316794499563911813311278820037841796875f,0.40590001422762878036110123503021895885467529296875f,2.021300038722157577097959801903925836086273193359375f,0.9976000194787975061672113952226936817169189453125f,0.16550001767277677799938828684389591217041015625f,0.485900006479024870742478015017695724964141845703125f,-0.830800016131997498547434588545002043247222900390625f,-1.3611000236809258634451680336496792733669281005859375f,1.028599994102119996597366480273194611072540283203125f,-0.29240001613497756949300310225225985050201416015625f,0.2987999994575976092647806581226177513599395751953125f,-0.373700008738040878597530536353588104248046875f,0.691200014922022543117918758071027696132659912109375f,-0.8428000256538386025795261957682669162750244140625f,-1.034200003984570681581089957035146653652191162109375f,0.1483999550700190184215898625552654266357421875f,0.7497000056385993982388527001603506505489349365234375f,-0.31220001055598256556322667165659368038177490234375f,-1.631100013697146966507034449023194611072540283203125f,-1.252900004303455716581083834171295166015625f,0.6367000379592184788890563140739686787128448486328125f,-1.3301999733924869673273860826157033443450927734375f,-1.257200031790137728648915071971714496612548828125f,-0.8170000200271605450552669935859739780426025390625f,-0.662900032764673685647949241683818399906158447265625f,-1.26660002630352952479597661294974386692047119140625f,0.21330000249743452656048248172737658023834228515625f,-0.605000000894069245305217918939888477325439453125f,0.461799995374679639326132019050419330596923828125f,0.06390001953244262011821774649433791637420654296875f,-0.5083000199317933009979242342524230480194091796875f,0.820900047308207536644886204157955944538116455078125f,-0.0522000631570822548610522062517702579498291015625f,-0.8395000109374517460736342400196008384227752685546875f,0.8002999992787831473606274812482297420501708984375f,0.600600022846460745995500474236905574798583984375f,1.34619998415112451795039305579848587512969970703125f,0.948599983745812114221962474402971565723419189453125f,-0.8075999628543844011119290371425449848175048828125f,0.4679000208437447128062558476813137531280517578125f,0.5865999498605720674504482303746044635772705078125f,1.1063000598073016167433024747879244387149810791015625f,0.30309999736547421633758858661167323589324951171875f,0.57610003217458771729297950514592230319976806640625f,-0.32170000346302973781575929024256765842437744140625f,0.299799986076354940678356797434389591217041015625f,0.28510000815391567385859161731787025928497314453125f,0.6502000212252145860958307821420021355152130126953125f,0.8187000401794914949960002559237182140350341796875f,-0.286800013628601302428933195187710225582122802734375f,-0.0751999699652190489729264299967326223850250244140625f,1.4646000331729656096513281227089464664459228515625f,2.21510002707838982161092644673772156238555908203125f,0.77190000635981537158158971578814089298248291015625f,-0.23900005942583124607381250825710594654083251953125f,-0.07339999113082829040877186344005167484283447265625f,1.6809000778555880373232866986654698848724365234375f,0.4065999913603073334655846338137052953243255615234375f,0.3165000045895578484334009772283025085926055908203125f,-0.402100047776103419749915701686404645442962646484375f,-0.34890002176761658603254545596428215503692626953125f,-0.073400009459257287147693205042742192745208740234375f,-0.521399995429813856162581942044198513031005859375f,-1.03900002073869135443828781717456877231597900390625f,-0.0594000148907303238132726619369350373744964599609375f,0.67320000157058235146223523770458996295928955078125f,0.5057000168442726817374932579696178436279296875f,0.165100004428625102281813497029361315071582794189453125f,0.291700000482797872081164314295165240764617919921875f,0.099199978056550008886915748007595539093017578125f,-1.008599981808662615634375470108352601528167724609375f,-0.37830000625997783902931814736803062260150909423828125f,-0.1320999954357742200983238944900222122669219970703125f,0.09200000721216194943252730809035710990428924560546875f,0.04519999958127751682468442595563828945159912109375f,-0.38950000983849164537531351015786640346050262451171875f,-0.6108000100597739123742258016136474907398223876953125f,-0.019400011873245137650201286305673420429229736328125f,-0.62379996652603164619677045266143977642059326171875f,0.407799984414875638094599707983434200286865234375f,0.916500025395304174224975213292054831981658935546875f,0.21240000747367704381218800335773266851902008056640625f,-0.161400004780292594119828208931721746921539306640625f,0.03050002259016082462039776146411895751953125f,-0.85660002906024457214329004273167811334133148193359375f,0.422900011605024328531499122618697583675384521484375f,0.36280001276433448964553463156335055828094482421875f,-0.97289999141395089310435650986619293689727783203125f,1.31720004676580426661303135915659368038177490234375f,-1.3440000225454566962213220904232002794742584228515625f,-0.774600029857456906512425121036358177661895751953125f,0.9530999931186439066976845424505881965160369873046875f,0.3606000199221079949296608901931904256343841552734375f,-0.22209999463483731663160369862453080713748931884765625f,0.92920001955628350742699694819748401641845703125f,-1.06820000596642472601160989142954349517822265625f,0.83530000097379097923067092779092490673065185546875f,0.084499991323799206188738253331393934786319732666015625f,-1.137400041280687279510175358154810965061187744140625f,-1.5615000358819965953216524212621152400970458984375f,-0.170500028327107866488177023711614310741424560546875f,-0.239100001567602138408119571977294981479644775390625f,-0.19769998941123478886083830730058252811431884765625f,0.598800017040967968995346382143907248973846435546875f,-0.435499983549118230286012476426549255847930908203125f,0.1427999910086390489283303395495750010013580322265625f,-0.84880001078248046297858309117145836353302001953125f,0.21550001798942697295302650672965683043003082275390625f,1.071800007724017067545219106250442564487457275390625f,0.280999992225319183969389769117697142064571380615234375f,-1.137100009517371557166143247741274535655975341796875f,0.060100006067752786975688650272786617279052734375f,-0.32819999397099020033152783071273006498813629150390625f,-0.28530000360012053306490997783839702606201171875f,-0.239299978033452875880726651303120888769626617431640625f,-0.05730002543404710824148651227005757391452789306640625f,-0.02400000216811892794765981307136826217174530029296875f,-0.391499991141259773286265044589526951313018798828125f,-0.554600005233287784989215651876293122768402099609375f,0.136599997767806069504104016232304275035858154296875f,-0.05120000441670402580029985983856022357940673828125f,-0.3321999792635441650645589106716215610504150390625f,0.70430001467168334983171007479541003704071044921875f,0.405300008293985936091985422535799443721771240234375f,1.060500015810131646531999649596400558948516845703125f,1.9136999588191514476420707069337368011474609375f,0.3544000249862675477885431973845697939395904541015625f,1.8835999675035477451956467120908200740814208984375f,0.18290001249909426661588440765626728534698486328125f,1.1886000014781945566255672019906342029571533203125f,0.67180000789165461583252181299030780792236328125f,-0.12300001104176061517136986367404460906982421875f,0.70370002389997299285795406831311993300914764404296875f,0.06819999669045195833660955031518824398517608642578125f,-0.00859999985769388519685207938891835510730743408203125f,0.69130001689195641478846710015204735100269317626953125f,-0.245300003898143881286841860855929553508758544921875f,-0.3496000184953214073857452603988349437713623046875f,0.970000005885958582751982248737476766109466552734375f,0.34320001147985446099397677244269289076328277587890625f,-0.384399979686736781303579846280626952648162841796875f,-1.53330005614161546390050716581754386425018310546875f,-0.049499962583184231590394119848497211933135986328125f,-1.027999985337256116935122918221168220043182373046875f,0.4302999550223347569044562987983226776123046875f,-1.64270008634328945618108264170587062835693359375f,1.426500047504902113360003568232059478759765625f,1.5028000274419781590040656737983226776123046875f,0.28189996981471698855870045008487068116664886474609375f,0.1415999994814394202791163479560054838657379150390625f,0.22960000437647110071992528901319019496440887451171875f,0.086600001884251931461022877556388266384601593017578125f,-0.24820000909566874991440954545396380126476287841796875f,-1.04120002132952205187166327959857881069183349609375f,0.038300016057491337306828427244909107685089111328125f,0.8978000091135502902517373513546772301197052001953125f,-0.91320005663037340237764283301657997071743011474609375f,0.20619997252821864464067402877844870090484619140625f,-1.2204999898076049902329032192938029766082763671875f,0.0514000389784576849905306517030112445354461669921875f,-0.531999987244606842295979731716215610504150390625f,0.83780002513229856475618362310342490673065185546875f,-0.377500001490115977276218472979962825775146484375f,0.4016000078260899552873297579935751855373382568359375f,-0.313400013706087943177180932252667844295501708984375f,-0.127399980521202049743578754714690148830413818359375f,-0.0930000212490558197941936668939888477325439453125f,0.164400007998942943032716357265599071979522705078125f,0.432399978733062795299701974727213382720947265625f,-0.4976999760746954049039914025343023240566253662109375f,0.055799952057004276184670743532478809356689453125f,0.231100019992888261466390531495562754571437835693359375f,-0.136300006088614811261550130438990890979766845703125f,-0.091900012320280222155588489840738475322723388671875f,0.2421000055313111687382843228988349437713623046875f,0.473500034675002456907577652600593864917755126953125f,0.26139996693134293082039221189916133880615234375f,-0.8834000034987925520368889920064248144626617431640625f,-0.1833000232845545252047259054961614310741424560546875f,0.11139998887330293531050529054482467472553253173828125f,-0.7841999730646611599382822532788850367069244384765625f,-0.110599994534254175704290901194326579570770263671875f,0.089600034290552965643428251496516168117523193359375f,0.359399946382641832798299219575710594654083251953125f,-0.58440002260208157025545006035827100276947021484375f,1.286000021934508907861527404747903347015380859375f,-0.83730002385079849158699971667374484241008758544921875f,2.07870001969039464029265218414366245269775390625f,0.185100000070035353161301827640272676944732666015625f,-0.0897999775454396687024427592405118048191070556640625f,-0.04980000351071378617007212596945464611053466796875f,0.3971000037431711149338298127986490726470947265625f,-0.80240002894997619620198747725225985050201416015625f,1.0486000256180769696356946951709687709808349609375f,-0.140100002826749892026469979100511409342288970947265625f,0.6800999893784520633488455132464878261089324951171875f,0.9278000251695512901051188237033784389495849609375f,-0.765300002966821324434931739233434200286865234375f,0.96490003237724319973267483874224126338958740234375f,1.085699999409913463210841655381955206394195556640625f,1.5619999961853026082536644025822170078754425048828125f,0.509499997675419535880791954696178436279296875f,2.00610000863075210730812614201568067073822021484375f,1.0482999961271877165103205697960220277309417724609375f,1.695800022651254845840185225824825465679168701171875f,-0.33030004990547967569369802731671370565891265869140625f,-0.3953000193208457968552238526172004640102386474609375f,0.31800003406405519257305058999918401241302490234375f,1.1362000320583580670330547945923171937465667724609375f,0.0528999819517128599954958190210163593292236328125f,1.40210002162456515151234270888380706310272216796875f,0.784800012898445142894843229441903531551361083984375f,-0.620800011736154289110345416702330112457275390625f,-0.1616000006735323069051446509547531604766845703125f,-0.2123000038832425218071620065529714338481426239013671875f,-0.5482000078290700795236034537083469331264495849609375f,0.35979999348968261241310528930625878274440765380859375f,-1.2948999764978881277244227021583355963230133056640625f,-0.0323000077575443800270704741706140339374542236328125f,-0.7289999750107529763454294879920780658721923828125f,-0.112499995268881269883110007867799140512943267822265625f,-1.19659998115301124954612532746978104114532470703125f,-0.710599998259544118894837083644233644008636474609375f,-0.4677000045359136493061669170856475830078125f,-0.247900005570054038361860193617758341133594512939453125f,-0.731999966233968901718753841123543679714202880859375f,0.5900999759674065980874502201913855969905853271484375f,0.212900023674964966602374261128716170787811279296875f,-0.758400018548966325937499277642928063869476318359375f,0.128299969863892471266808570362627506256103515625f,0.01040000251531569830376611207611858844757080078125f,-1.003800038051604559541374328546226024627685546875f,-0.0319999884366988229800199405872263014316558837890625f,1.07680006015747853354014296201057732105255126953125f,-1.47490003308504835644043851061724126338958740234375f,0.25320000484883753433251740716514177620410919189453125f,-0.8689999984055756243606083444319665431976318359375f,0.69640000817030600632762116219964809715747833251953125f,-0.7739000163659450759467972602578811347484588623046875f,0.74830002827644381824256925028748810291290283203125f,-0.413500013962388113242241161060519516468048095703125f,0.43999998070299539687511014562915079295635223388671875f,-0.718000001728534176237417341326363384723663330078125f,1.092300063487888106550371958292089402675628662109375f,-1.909200039598346165803377516567707061767578125f,-1.12790003812909134239816921763122081756591796875f,1.7511999783396721142025853623636066913604736328125f,0.984800076079370256820766371674835681915283203125f,-0.15090002689361625698438729159533977508544921875f,-1.1558000262647869504917252925224602222442626953125f,0.246399998521804608486718279891647398471832275390625f,-0.0869999847114086843902214241097681224346160888671875f,0.8693999987974760923492567599168978631496429443359375f,-0.3138999836578968416489487935905344784259796142578125f,1.4818000199988485832136575481854379177093505859375f,0.9126000428736211755875729068065993487834930419921875f,0.7372999845862384660222232923842966556549072265625f,-0.9219000099360943334403373228269629180431365966796875f,0.059199999737739705807371137780137360095977783203125f,-1.203000016033648922331167341326363384723663330078125f,-1.094500016003846720735737108043394982814788818359375f,-1.0056000545114283095671225964906625449657440185546875f,1.00360005616545766571334752370603382587432861328125f,1.050699983689189043190026495722122490406036376953125f,0.0331999778032301406938131549395620822906494140625f,0.029899975752830432185191966709680855274200439453125f,-1.1464999735951426629299021442420780658721923828125f,1.238399986548721631862690628622658550739288330078125f,-0.61569997392892805709863068841514177620410919189453125f,1.3160000012591479556789408889017067849636077880859375f,-1.721199993408470962918954683118499815464019775390625f,1.021599995681643502365432141232304275035858154296875f,-0.1379999892115595372388270334340631961822509765625f,-0.1350000386871401669708347981213591992855072021484375f,-0.3850000494718557320084073580801486968994140625f,0.66620000789985034383988704576040618121623992919921875f,-1.3153999765872956828616224811412394046783447265625f,-0.44170004343539470692547865837696008384227752685546875f,-0.06970005958080349728334113024175167083740234375f,-1.432600031697750520010004038340412080287933349609375f,0.9318000293493273744616089970804750919342041015625f,-0.44580000490024673798217236253549344837665557861328125f,0.5804000218868257121584974811412394046783447265625f,0.60340000878870458933533882373012602329254150390625f,0.438000046003609089240171670098789036273956298828125f,-0.069499999631196185401194043151917867362499237060546875f,-2.008899935974181261144622112624347209930419921875f,1.7602999748967587123615885502658784389495849609375f,0.4636000509500497468451385429943911731243133544921875f,-0.6027999745428560540716489413171075284481048583984375f,1.086200028668343531279560920665971934795379638671875f,0.488500007182359308188779323245398700237274169921875f,-1.4290999688223007257903418576461263000965118408203125f,0.6340999934464690568347577936947345733642578125f,1.5620999419778576200457109734998084604740142822265625f,0.770700036513805475379967901972122490406036376953125f,0.46329999698400481378257609321735799312591552734375f,-0.50160005595684087342789325703051872551441192626953125f,-0.4029000308647756600777256608125753700733184814453125f,-0.527300003808736761357067734934389591217041015625f,0.4406000030279155765811083256267011165618896484375f,-0.447099975877999877837964959326200187206268310546875f,0.38029999838471439943532459437847137451171875f,1.5281000500410806264284246935858391225337982177734375f,0.76090004179477688239785493351519107818603515625f,-0.13839999880492737105441847234033048152923583984375f,1.16460002833008768874378802138380706310272216796875f,-0.548000018045306180880515967146493494510650634765625f,-1.0915000260472300652736521442420780658721923828125f,0.38020000855922742033499162062071263790130615234375f,2.148900062894821161307845613919198513031005859375f,-0.9300999932527542757298988362890668213367462158203125f,0.8702999623611564761205272588995285332202911376953125f,0.7039000481054193958385667428956367075443267822265625f,1.09680006716102429464854139951057732105255126953125f,-0.7153000109016891538971094632870517671108245849609375f,0.1765000197887420796405422152020037174224853515625f,-0.23059999706745148984055049368180334568023681640625f,0.774100052790344772546404783497564494609832763671875f,-0.7128000159680834002529081772081553936004638671875f,0.81480002394393036269093499868176877498626708984375f,-0.53949999797344272423060829169116914272308349609375f,-0.187300032698363150274190047639422118663787841796875f,0.99090002376437258391206341912038624286651611328125f,0.19180004158690644455731444395496509969234466552734375f,-0.7819000276684757722023277892731130123138427734375f,0.83010002068951693754428333704709075391292572021484375f,-0.195299992573261693706854202901013195514678955078125f,1.212000028390437744718610701966099441051483154296875f,-2.034500053852796863651519743143580853939056396484375f,0.66780002274811300910783984363661147654056549072265625f,-0.928800055187940731826756746158935129642486572265625f,0.174700015808641861969618958028149791061878204345703125f,-0.27260002275705375041070510633289813995361328125f,0.094899999445676730402965404209680855274200439453125f,-1.1532000418037162692286301535204984247684478759765625f,0.628300022390484880219219121499918401241302490234375f,-0.04079994777291962027021554604289121925830841064453125f,0.2331000092864037309681179976905696094036102294921875f,0.8768000164225693282560314401052892208099365234375f,-0.804800030183792092230987691436894237995147705078125f,-1.7711000412270418991056430968455970287322998046875f,0.9597000232964756349929302814416587352752685546875f,1.0427000839218500960470237259869463741779327392578125f,-0.9299999918788668384905804487061686813831329345703125f,-0.7653999996840952224630427735974080860614776611328125f,1.523500052444636931880950214690528810024261474609375f,0.5779000306785118112884447327814996242523193359375f,0.6360000016316771365154636441729962825775146484375f,-0.78280002408921711687384004108025692403316497802734375f,0.5683000294685367492775185382924973964691162109375f,0.918600047969817978810169734060764312744140625f,0.37510005390048117845935848890803754329681396484375f,2.183800034624338248789854333153925836086273193359375f,0.00669999608695481807529858997440896928310394287109375f,-0.09599997127056081325235936674289405345916748046875f,0.09700001034140637301561582717113196849822998046875f,-0.34769999708533294580092842807061970233917236328125f,0.344000006377697342685451076249592006206512451171875f,-0.526200001585483079225014080293476581573486328125f,0.64880000206530130668625133694149553775787353515625f,0.5480000714659691851693423814140260219573974609375f,-0.6945000140666959254076573415659368038177490234375f,-0.1291999895304447587562890475965104997158050537109375f,-0.785599976373463793066775906481780111789703369140625f,0.145299937066435180810231031500734388828277587890625f,0.36160000249892443324739588206284679472446441650390625f,-1.091800005898624359446102971560321748256683349609375f,-1.472200016069411532271260512061417102813720703125f,0.63559998838752440608601546045974828302860260009765625f,0.372399958914516826524732096004299819469451904296875f,0.085900014358014420423614865285344421863555908203125f,-0.12640000761151259922598910634405910968780517578125f,-0.199799996171891702800849088816903531551361083984375f,0.67379998455643619337251948309130966663360595703125f,-0.21569998048543903568230462042265571653842926025390625f,0.815600045049190924828508286736905574798583984375f,-0.5523999967448409709191992078558541834354400634765625f,1.167899988508224851102568209171295166015625f,0.2600000308826564410225046231062151491641998291015625f,-0.6236000131756067599297921333345584571361541748046875f,-0.195400031684339448023735030801617540419101715087890625f,0.21990000294744971309768288847408257424831390380859375f,-0.4393000106483697653203535082866437733173370361328125f,-0.16839996931925316214773147294181399047374725341796875f,0.816500002503395716502154755289666354656219482421875f,0.787500048056245294247901256312616169452667236328125f,-0.128399952238798409354103569057770073413848876953125f,-1.324400025619566267920390600920654833316802978515625f,0.353799986121058562815733239403925836086273193359375f,-0.7791000377029178025622968561947345733642578125f,0.2466000236213208296476295799948275089263916015625f,-0.3982000262320035677277019203756935894489288330078125f,-0.79630000884532936566984062665142118930816650390625f,1.680799989570677777095397686935029923915863037109375f,-1.2189000135719771922282461673603393137454986572265625f,-1.562700002226233397806254288298077881336212158203125f,0.4944000230491163438983903688495047390460968017578125f,0.9368999893911180709693553581018932163715362548828125f,0.052700018375366364242751160418265499174594879150390625f,0.4183000138595700523325149333686567842960357666015625f,-0.040000054780394034281698623090051114559173583984375f,0.4877000058770182278067295555956661701202392578125f,0.63470001040697088257047653314657509326934814453125f,0.00830000234842298567627949523739516735076904296875f,-0.4384000120669606648249327918165363371372222900390625f,1.03650000309944090304270503111183643341064453125f,-0.03829995988011347662904881872236728668212890625f,0.40189998892545730768688372336328029632568359375f,1.22970003134310257308925429242663085460662841796875f,1.32200005072355253332716529257595539093017578125f,-1.15069999099075825910176718025468289852142333984375f,1.41589995790123968077978133806027472019195556640625f,1.191800048944353651592109599732793867588043212890625f,0.8667999907553198823961793095804750919342041015625f,0.252400048097968454641204516519792377948760986328125f,2.45700005564093704180095301126129925251007080078125f,-0.098699994879960950555641829851083457469940185546875f,1.3203000241637230516289491788484156131744384765625f,-0.3785999871730805210745529620908200740814208984375f,0.1951999908268451378035024390555918216705322265625f,-0.40840003196001095631118005258031189441680908203125f,0.164099994321912301398214140135678462684154510498046875f,0.3581000179290771967544060316868126392364501953125f,1.02800002694875036723942685057409107685089111328125f,-1.40620001529455151967340498231351375579833984375f,0.6195999908722933380289532578899525105953216552734375f,0.1306000021338462602216168306767940521240234375f,0.707999988522379997135658413753844797611236572265625f,-0.0303999390363696164740758831612765789031982421875f,1.29039999914392833346710176556371152400970458984375f,-1.56289999357461883988662521005608141422271728515625f,1.584500031780451845264678922831080853939056396484375f,-0.64449996593594516269831729005090892314910888671875f,0.73250000279396776381446443338063545525074005126953125f,-0.3582000067114829988668134319595992565155029296875f,-0.0467999999195336524593358262791298329830169677734375f,-0.109399989968538502438377690850757062435150146484375f,0.6848000018715858505657934074406512081623077392578125f,-0.21249999005347552927247534171328879892826080322265625f,-0.635199989858269642439836388803087174892425537109375f,-1.74399999333918032107249018736183643341064453125f,1.31780000747442205266679593478329479694366455078125f,-0.328599954055249998674526068498380482196807861328125f,0.15469998544752616709274661843664944171905517578125f,-1.14829998644143405073236863245256245136260986328125f,-0.35809998134672704761527484151883982121944427490234375f,-1.1181999815657739372198875571484677493572235107421875f,1.367499979063868220663380270707421004772186279296875f,-1.4333999795451755066011401140713132917881011962890625f,0.198600001105666112977132797823287546634674072265625f,-0.1244000074774029851454315576120279729366302490234375f,0.52450001615285923861620176467113196849822998046875f,-0.2971999862670902103900516522116959095001220703125f,-0.5732000124484297742810667841695249080657958984375f,-0.11159998204708099223125827847979962825775146484375f,-0.2933000070422890903643065030337311327457427978515625f,1.0389000095486640162789626629091799259185791015625f,-0.482600000628829572946187909110449254512786865234375f,0.04879996145963705345138805569149553775787353515625f,-0.1742999906063074089246356379589997231960296630859375f,0.233500034153461655250794137828052043914794921875f,-0.176000035271048016483064202475361526012420654296875f,-2.121699978503585271027986891567707061767578125f,-0.9062000167846679943295384873636066913604736328125f,-0.814100027346610577438923428417183458805084228515625f,-0.003799998340010546371559030376374721527099609375f,0.19689998460412017067255874280817806720733642578125f,0.287600027525424906116313650272786617279052734375f,0.538900007462501662303111515939235687255859375f,0.68189997521638900934703997336328029632568359375f,0.1108000313870609598865257794386707246303558349609375f,-0.35780000061988825876824193983338773250579833984375f,0.137899961332230869270887296806904487311840057373046875f,-0.2040000057816513390207546763122081756591796875f,0.86540001788213893885171046349569223821163177490234375f,-1.60990002075433746853150296374224126338958740234375f,-0.5703000013649457411446519472519867122173309326171875f,0.8863999424934387860730566899292171001434326171875f,-1.440499997798353692957107341499067842960357666015625f,0.9748999992221591970320559994434006512165069580078125f,-0.7368000054515897456752782090916298329830169677734375f,0.6476000231295824249855286325328052043914794921875f,0.6017000229276716627424548278213478624820709228515625f,-1.26219998552203183095343774766661226749420166015625f,0.60089999224841594838153469027020037174224853515625f,-0.9913000088453287617085152305662631988525390625f,-0.3375000012665985149595826442237012088298797607421875f,-0.7394999910071486937113149906508624553680419921875f,-0.0710999911963938746595204065670259296894073486328125f,1.006500063039361680949923538719303905963897705078125f,0.014799996656179370546624340931884944438934326171875f,-1.8884000119552017959989598239189945161342620849609375f,1.308500016719102543305552899255417287349700927734375f,1.752999935418368071537997820996679365634918212890625f,-0.1778999760657544726427659043110907077789306640625f,0.2994999700710179180163095224997960031032562255859375f,-0.5664000102937218894538773383828811347484588623046875f,0.54879996686130783078993999879457987844944000244140625f,-0.73169999943971664180253355880267918109893798828125f,0.049400001032650298071757788420654833316802978515625f,0.238000010967254649330016036401502788066864013671875f,0.3240000244081020497333156527020037174224853515625f,0.9968999805808067549151019193232059478759765625f,0.65760000696182174806381226517260074615478515625f,-0.53619996865391694740310413180850446224212646484375f,1.12649997419118808039684154209680855274200439453125f,0.0962999960303305702069565086276270449161529541015625f,-0.96110000028610187428057543002068996429443359375f,0.507300005596876157909491666941903531551361083984375f,-1.530500049412250263003443251363933086395263671875f,0.070199975851178209040881483815610408782958984375f,-0.30889994517564733911285657086409628391265869140625f,-0.3943000222712746971609476531739346683025360107421875f,-1.06449996191263185352227083058096468448638916015625f,0.3289999789595603463254747111932374536991119384765625f,-0.7256999497890479489115023170597851276397705078125f,0.50870005120635131135031770099885761737823486328125f,0.243800023746491234533095848746597766876220703125f,-0.61100002871453806818635712261311709880828857421875f,0.5495000122785569374173064716160297393798828125f,-0.981800006699561578216162160970270633697509765625f,0.453299999479949367042763697099871933460235595703125f,0.03759999831914928591913849231787025928497314453125f,-0.7993000318825249639331786966067738831043243408203125f,0.2926000132203103731853843783028423786163330078125f,-0.23249999780207863953052083161310292780399322509765625f,-0.3383000100970274814926597173325717449188232421875f,-1.2064000122122460201268268065177835524082183837890625f,2.02770000885725121264613335370086133480072021484375f,-0.9700999692060052748132648048340342938899993896484375f,0.67799997162818836926589938229881227016448974609375f,-0.1917999812185759989091593524790368974208831787109375f,-0.0761000455856315483060825499705970287322998046875f,-0.2096999980762606075046505793579854071140289306640625f,1.1465999883055673080889391712844371795654296875f,-0.0468000032536683985284753362066112458705902099609375f,0.83290004692077701520247501321136951446533203125f,0.544600010746717710929942768416367471218109130859375f,0.673900014838576755238364057731814682483673095703125f,-0.039499972864985632980960872373543679714202880859375f,1.631400038158893561757167844916693866252899169921875f,-0.320700045454502369324245591997168958187103271484375f,-0.9489000404685736800303175186854787170886993408203125f,0.60230001185536341523629744187928736209869384765625f,1.85089994989186568119521325570531189441680908203125f,-1.39930003150999482386396266520023345947265625f,0.17249995958060104772613385648583061993122100830078125f,-0.8755999734491108910106049734167754650115966796875f,-1.033400007820129307134493501507677137851715087890625f,0.8305999849230059073335041830432601273059844970703125f,-1.040499987050889973971834479016251862049102783203125f,-0.57969997427165598224974019103683531284332275390625f,-0.694500039659441181782995045068673789501190185546875f,-0.621900007104873253638288588263094425201416015625f,0.4732000235497952900232121464796364307403564453125f,-0.9366999829739330873934477494913153350353240966796875f,2.14049998380988881763187237083911895751953125f,0.242300021168589285736061356146819889545440673828125f,0.43039998716712002391204805462621152400970458984375f,0.15729999233484281973005636245943605899810791015625f,-0.88299999219179081677566500729881227016448974609375f,0.273799993571639532774497638456523418426513671875f,0.9635000247657306005066857323981821537017822265625f,-0.0677000037163499879255823543644510209560394287109375f,0.978499974362551316886538188555277884006500244140625f,0.87250003401190101559592449120827950537204742431640625f,-0.5174999751150604065941251974436454474925994873046875f,-0.7270000327676535700760496183647774159908294677734375f,0.0132000082015988340344847529195249080657958984375f,-0.440499994128942251592206957866437733173370361328125f,-0.415699994790553972734414855949580669403076171875f,-0.3909999998807907051201482317992486059665679931640625f,1.3823000219136474253645019416580908000469207763671875f,-1.1416999795094138203666034314665012061595916748046875f,0.75949998405948282442778918266412802040576934814453125f,-1.52340002182722056289776446647010743618011474609375f,-0.352099992939829409266394577571190893650054931640625f,0.2376999911248685037890027160756289958953857421875f,0.746300034531205636056938601541332900524139404296875f,-0.508299975973368844961441936902701854705810546875f,1.159100019486249077971251608687452971935272216796875f,-1.47810003085583385740164885646663606166839599609375f,-0.558400022125244532844590139575302600860595703125f,0.565099975036084334334418599610216915607452392578125f,-0.5996999896757302384031618203152902424335479736328125f,0.8679000048995018090636222041212022304534912109375f,0.06450000642985098675552535496535710990428924560546875f,-0.723200021910667345537149230949580669403076171875f,0.18799999349564322503880475778714753687381744384765625f,-0.593699979457259274795433157123625278472900390625f,0.07070004541724894675525092679890803992748260498046875f,0.3904000167086716377440325231873430311679840087890625f,1.58159998644292354441631687222979962825775146484375f,0.16679998092055381420095727662555873394012451171875f,0.1078999882102014140627943561412394046783447265625f,-0.245299973388016123720944960950873792171478271484375f,-0.62379997241199003354950036737136542797088623046875f,-0.18410001363754313619125468903803266584873199462890625f,1.2285000290125613897629364146268926560878753662109375f,1.2653000395491726948193900170736014842987060546875f,-0.0180000237077477809322090251953341066837310791015625f,0.26759995782524337304408845739089883863925933837890625f,-1.2185999818831680929775984623120166361331939697265625f,-0.4632000002294776042077728561707772314548492431640625f,0.901700000631809217566114966757595539093017578125f,-1.186299992305039463502680519013665616512298583984375f,-0.55000006787478949998870803028694353997707366943359375f,-1.82070001177787776924787976895458996295928955078125f,0.087799973276256881860035719000734388828277587890625f,-2.00840005550384592680757123162038624286651611328125f,0.1489999964684243227708293488831259310245513916015625f,-0.5595000420808791119497982435859739780426025390625f,0.27630002121329244602065955405123531818389892578125f,2.44270000259876240988887730054557323455810546875f,-0.300699977132678153424194533727131783962249755859375f,0.870599998110532791173454825184307992458343505859375f,-1.0166999885991214735980747718713246285915374755859375f,0.7574999916180968906331827383837662637233734130859375f,-0.654000007160008056672495513339526951313018798828125f,-0.1148000018715855663486991034005768597126007080078125f,0.239200004357099427210187059245072305202484130859375f,-0.214600011342763963284596684388816356658935546875f,1.290899984127282795753899335977621376514434814453125f,-1.4824000194318596168585600025835447013378143310546875f,1.2080000275075430948845678358338773250579833984375f,-1.444500052716583216039225590066052973270416259765625f,1.04329999877214429915284199523739516735076904296875f,0.207900001304596238060895529997651465237140655517578125f,-0.127599919641017578442188096232712268829345703125f,1.1843999856099485246119229486794210970401763916015625f,-0.535099970602988861401172471232712268829345703125f,0.71439999998956926763327146545634604990482330322265625f,-1.08760002916455267296669262577779591083526611328125f,0.2128000019423663402573509983994881622493267059326171875f,-0.3428000025570392406137898433371447026729583740234375f,0.25900000175833692850346778868697583675384521484375f,-1.1341999900892372199479041228187270462512969970703125f,0.17219999558031562347792942091473378241062164306640625f,-0.7582000236988069019616887089796364307403564453125f,-0.428400013631582421425036955042742192745208740234375f,0.684799954150616230208470369689166545867919921875f,-1.0953000083684918397608498707995750010013580322265625f,0.7811999631777395425302756848395802080631256103515625f,-0.184800020498037564919968644971959292888641357421875f,-1.640199971343577534099722470273263752460479736328125f,-0.36280002967715319073249702341854572296142578125f,0.458500051960348375956755262450315058231353759765625f,-0.219400008147955016823971163830719888210296630859375f,-0.44749999135732654309549616300500929355621337890625f,1.0144000109046695801140458570444025099277496337890625f,0.36109997677952065942008630372583866119384765625f,0.0609000236898658842132903146193712018430233001708984375f,0.2546000099271534811151695976150222122669219970703125f,0.6993999831885100437744995360844768583774566650390625f,0.2796000056803225053414507783600129187107086181640625f,0.7053999996840953912169425166212022304534912109375f,-0.103999989986419905108050443232059478759765625f,-1.948500047098845389115240323008038103580474853515625f,0.36270005505085001207277173307375051081180572509765625f,-0.279199991225450805387708896887488663196563720703125f,0.382399954276532394459309216472320258617401123046875f,-1.826600034350156054330227561877109110355377197265625f,0.551400007164478012100516934879124164581298828125f,-0.6796999785184851816666196100413799285888671875f,-0.8765000370740896329380120732821524143218994140625f,0.73100004264712392654246286838315427303314208984375f,-1.827600056433677622180766775272786617279052734375f,-0.949499996930359912994390469975769519805908203125f,0.519199986624716558480940875597298145294189453125f,1.761400050377845527549425241886638104915618896484375f,-0.999300012362002831878271535970270633697509765625f,0.66030003653168733990241889841854572296142578125f,-0.25200001183152220818328714813105762004852294921875f,-0.30889999613761887076179846189916133880615234375f,-1.49929999053180207368995979777537286281585693359375f,0.583900015173851816285832683206535875797271728515625f,-0.088699998381734257435482504661194980144500732421875f,0.295999979354441278278642357690841890871524810791015625f,-1.52470000578761055720633521559648215770721435546875f,-0.15540002613365633266084842034615576267242431640625f,0.7624000563681132547344532213173806667327880859375f,0.6795000353753566368908423100947402417659759521484375f,0.24370000516176215654695624834857881069183349609375f,-0.749999963417648185526331872097216546535491943359375f,1.11610009714365077115871827118098735809326171875f,-0.38199999760091341016732258140109479427337646484375f,1.1760000019669529791599416057579219341278076171875f,0.29660002160966403295105919823981821537017822265625f,0.360400001472234610133682508603669703006744384765625f,-0.037100006052851686177973533631302416324615478515625f,0.26399999505281446232629605219699442386627197265625f,-0.787199974867700991154606526833958923816680908203125f,1.14110000416040424653374429908581078052520751953125f,0.68669999683201321527548088852199725806713104248046875f,0.78930005348920762031639242195524275302886962890625f,0.824400020739436190098103907075710594654083251953125f,0.13189997372627271232659040833823382854461669921875f,-1.36300000120699404959623279864899814128875732421875f,0.221299997520446822818485088646411895751953125f,1.9203000308692452779979475963045842945575714111328125f,0.682500028461217933539728619507513940334320068359375f,-0.222600015082955682288456955575384199619293212890625f,1.310599983432889192869197358959354460239410400390625f,1.13710003455132291350082596181891858577728271484375f,0.552200039091706340599330360419116914272308349609375f,-0.43749999925494176267193324747495353221893310546875f,0.462600002789496755895015667192637920379638671875f,-0.15879999096393593305265312665142118930816650390625f,-0.227200003999471267235321647603996098041534423828125f,0.7370999877989294901681205374188721179962158203125f,-0.5427000766575356127674467643373645842075347900390625f,0.64370007966756848105660537839867174625396728515625f,-1.47280003712773321922213654033839702606201171875f,0.7412000208079820851025942829437553882598876953125f,1.22580004943609299772333542932756245136260986328125f,-0.16920005457401288140317774377763271331787109375f,-1.15840006571114173539172043092548847198486328125f,-0.494899962043761743046843548654578626155853271484375f,-0.30119997580647372981133003122522495687007904052734375f,-0.8255000483691699031396638019941747188568115234375f,-0.898100033910572914663816845859400928020477294921875f,-0.4673000110730527456581739897956140339374542236328125f,-0.3107000362157830153364557190798223018646240234375f,0.2121000082135200415223152958787977695465087890625f,0.6902000052809711405643611215054988861083984375f,-0.357100055450202180651331218541599810123443603515625f,-0.96990002962052856982921866801916621625423431396484375f,-1.41230001632571156022777358884923160076141357421875f,0.66530000933706734400630011805333197116851806640625f,0.0439999940097326458499082946218550205230712890625f,1.02750002153217767641990576521493494510650634765625f,0.63309998052716220939828417613171041011810302734375f,-0.935800052192807374495941985514946281909942626953125f,-1.950100041085481183955607775715179741382598876953125f,0.14110000736415440059090542490594089031219482421875f,-0.1458999778315421824714803733513690531253814697265625f,-1.2264000157698988768828485262929461896419525146484375f,-0.49360001112669726897053124048397876322269439697265625f,0.9047000538438563221887989129754714667797088623046875f,0.7535000186562541557577787898480892181396484375f,0.056000012993812742223553868825547397136688232421875f,1.42640001819133743765632971189916133880615234375f,0.93700001294910872928767275880090892314910888671875f,1.32229999516606344656111104995943605899810791015625f,0.05870002959966702515970382592058740556240081787109375f,0.09499998867511738609437088598497211933135986328125f,-0.9865000055581329974074833444319665431976318359375f,0.36280000002384138468869423377327620983123779296875f,-0.545599986338614950653891355614177882671356201171875f,-0.138899986600875990916392765939235687255859375f,1.213200015577674317768241962767206132411956787109375f,-0.6805999464035039636655710637569427490234375f,-0.142000002145767467709447373636066913604736328125f,0.162700001034140395717031424283050000667572021484375f,-0.58500004053115894908643213057075627148151397705078125f,0.14849997331947129008966612673248164355754852294921875f,0.010000000502914330180459501207224093377590179443359375f,-0.97919998938143215649887451945687644183635711669921875f,-0.14040003798007916202550404705107212066650390625f,-2.70700011032819798373338926467113196849822998046875f,1.80650004452466905746632619411684572696685791015625f,0.3041000286877160174725531760486774146556854248046875f,-0.195400037235021528658762690611183643341064453125f,-0.541800045144558595211492502130568027496337890625f,1.825900024658441367364503093995153903961181640625f,0.51189998996853791624062068876810371875762939453125f,0.925500037193298652482553734444081783294677734375f,-0.7447000433385380091522165457718074321746826171875f,0.111900038099289389492696500383317470550537109375f,0.753800015699862768769889953546226024627685546875f,-0.930500005453824829970699283876456320285797119140625f,-0.79080001238808017571813024915172718465328216552734375f,0.64140003540217860322769638514728285372257232666015625f,1.281800049708038624629580226610414683818817138671875f,-1.169700007519870244010462556616403162479400634765625f,0.2969999550580979530423064716160297393798828125f,-0.5415999759376035616043054687906987965106964111328125f,1.323200039568543928680810495279729366302490234375f,0.6372999908447258832211446133442223072052001953125f,1.02860002286136165139396325685083866119384765625f,0.76490001062154799882364386576227843761444091796875f,0.618599991047382058440007313038222491741180419921875f,-0.6150000499188905678238370455801486968994140625f,-0.478600021669268738122582362848334014415740966796875f,-0.4152000264406208174250423326157033443450927734375f,1.064500026583671488111804137588478624820709228515625f,0.58619998936653150423126135137863457202911376953125f,0.046399991145729568842170920106582343578338623046875f,0.37550002340972443715116924067842774093151092529296875f,-0.880400009183586629291085046133957803249359130859375f,0.025899977645277072468843471142463386058807373046875f,-0.335899998693168555519150686450302600860595703125f,0.2935999937295914463675217120908200740814208984375f,0.0824999952316283202691238329862244427204132080078125f,0.2298999987006187240012877737171947956085205078125f,0.8085999966353174528421732247807085514068603515625f,0.2471999785184859632636289461515843868255615234375f,-0.3206999792933462600785787799395620822906494140625f,1.184099976682663424298880272544920444488525390625f,0.89770001466870308348688922706060111522674560546875f,0.983300001454352923246915452182292938232421875f,0.135600010553002192636995459906756877899169921875f,1.126899926936626883389180875383317470550537109375f,0.50510000070333516131171336382976733148097991943359375f,1.31869998400211319022901079733856022357940673828125f,0.9389000599145898018349498670431785285472869873046875f,2.11839998338222468277081134147010743618011474609375f,0.08310000138878859132773868623189628124237060546875f,1.2773999989986410952269579865969717502593994140625f,-0.4390000042915342959304325631819665431976318359375f,-0.293199996429681419840562739409506320953369140625f,0.2451999733924861146761031704954802989959716796875f,-0.55190000367760649879755874280817806720733642578125f,-0.189899979031086019176655099727213382720947265625f,0.01100002238154473843678715638816356658935546875f,-0.009600031757354798855885746888816356658935546875f,-1.5416000692188749354727406171150505542755126953125f,1.38570004158020054774169693700969219207763671875f,-0.743600008928775313421510873013176023960113525390625f,0.364800028693675670865559368394315242767333984375f,-0.4913999834716313852567282083327881991863250732421875f,0.1790999970972535493274335749447345733642578125f,-1.720300074678660084970260868431068956851959228515625f,1.1174000459373001348239995422773063182830810546875f,-1.604500074565411438243245356716215610504150390625f,1.177500035464763783465969027020037174224853515625f,-0.4638999976277347059294697828590869903564453125f,0.202500032335520163684350336552597582340240478515625f,0.7960000096410515535438889855868183076381683349609375f,-0.18870000188350655179192472132854163646697998046875f,0.5372000034034247750724944125977344810962677001953125f,-0.034300015193224453469156287610530853271484375f,0.009400006583332842780009741545654833316802978515625f,-0.495899996644258322930909343995153903961181640625f,2.04100001791119556315834415727294981479644775390625f,-0.4165999978423116090198163874447345733642578125f,-0.45599997961521143707841474679298698902130126953125f,-0.753599979722500368239934687153436243534088134765625f,0.954600030639767282991670072078704833984375f,-1.31980003793239664133807309553958475589752197265625f,0.87420005099773445689237405531457625329494476318359375f,-0.206699977982044202917677466757595539093017578125f,0.3096000126838684973762383378925733268260955810546875f,-0.61729998488426218727909144945442676544189453125f,0.0890000178515910622678575236932374536991119384765625f,-0.13449999111890775793654029257595539093017578125f,0.942299998295306640017088284366764128208160400390625f,0.1328999753952029294623571331612765789031982421875f,-0.286399989059567428029140501166693866252899169921875f,0.7663999559044842868615887709893286228179931640625f,-0.237200003924965319157536214333958923816680908203125f,-0.123099960112572404113961965776979923248291015625f,-0.5983000173240899943749582234886474907398223876953125f,-1.921099954167008849736930642393417656421661376953125f,-0.51510000122487531459825049751088954508304595947265625f,-0.88930000625252692003641641349531710147857666015625f,0.576899976333975939013498646090738475322723388671875f,0.05520002882480579131652120850048959255218505859375f,0.0717000003337862068519825697876513004302978515625f,0.7168000019311904225105536170303821563720703125f,0.07479999561309824684940394945442676544189453125f,0.8363000167429444076105937710963189601898193359375f,-1.0902000130295750324194159475155174732208251953125f,-0.98720002784132976358932864968664944171905517578125f,-0.55010000427961358582251705229282379150390625f,0.715900080388786275165102779283188283443450927734375f,-1.0167000855684289462033120798878371715545654296875f,-1.489400065517425986172384000383317470550537109375f,1.3011000202536582293078026850707828998565673828125f,1.814000015467403859048545200494118034839630126953125f,-0.988499983936547810259298785240389406681060791015625f,0.8061000274062160020349665501271374523639678955078125f,-0.546500001683831282406345053459517657756805419921875f,0.876000029087066423016949556767940521240234375f,-1.74739997619986553445414756424725055694580078125f,-0.752900026655197507352568209171295166015625f,0.15650000667572072643451974727213382720947265625f,0.4014999974369999335976899601519107818603515625f,-0.001100033813715040054148630588315427303314208984375f,-0.46930000610351552126076057902537286281585693359375f,0.059600046509505233416348346509039402008056640625f,0.4519000333309179939789146374096162617206573486328125f,0.246200020956993004261903479346074163913726806640625f,-0.32229999024867961310292230336926877498626708984375f,0.62670001448988887915447776322253048419952392578125f,0.210700019302964225431651357212103903293609619140625f,1.2544999973773958146239237976260483264923095703125f,-0.036300000128150156086803690413944423198699951171875f,0.346799990308284566964402984012849628925323486328125f,-1.2036999956995246829904999685822986066341400146484375f,-1.0221999993801116346503476961515843868255615234375f,-1.07809998745620294613445366849191486835479736328125f,-0.163799988579750532835532794706523418426513671875f,-0.00739998715221901193217490799725055694580078125f,0.9598000087261198132182471454143524169921875f,-0.25940001574754756319407533737830817699432373046875f,1.0081999900221827459745327359996736049652099609375f,0.0005000033676623161227325908839702606201171875f,-1.208799991559981634736686828546226024627685546875f,-0.8657000240713357275268435842008329927921295166015625f,-0.4891000011205672848291214904747903347015380859375f,-0.07299998049437927960525485104881227016448974609375f,-0.962099964702129994975621229968965053558349609375f,0.3618000024527312508126897228066809475421905517578125f,-1.566399947559833805144080542959272861480712890625f,-0.49599998982250692591833285405300557613372802734375f,-0.0811999798297886599129924434237182140350341796875f,0.228500037878751616204908714280463755130767822265625f,0.109099980854988842793318326584994792938232421875f,-0.1190999893486495153638315969146788120269775390625f,0.36199997591972277177774230949580669403076171875f,-0.1080000014305113609225372783839702606201171875f,-1.509600058877468509166419607936404645442962646484375f,1.2850000500679019665994928800500929355621337890625f,0.63990001300573329245935383369214832782745361328125f,-0.8483000130772584412852665991522371768951416015625f,1.450300068271161091359999772976152598857879638671875f,-0.2552000329971324532607468427158892154693603515625f,-0.770600055480004098029667147784493863582611083984375f,1.2842000316262254244747964548878371715545654296875f,1.52990003073811475786669689114205539226531982421875f,-1.7175000026822093701639460050500929355621337890625f,0.23349999496340689830731207621283829212188720703125f,0.3549999642372139163626343361102044582366943359375f,-0.46340002376437183784219087101519107818603515625f,1.2414000071644775147206019028089940547943115234375f,-0.57070002906322514490966568700969219207763671875f,0.900400034999847775907255709171295166015625f,0.3197999640226356632410897873342037200927734375f,-0.523700019764900215335501343361102044582366943359375f,-0.87090003553629014021453258465044200420379638671875f,0.22249999940395337461040980997495353221893310546875f,-0.559399992203712059790632338263094425201416015625f,0.43200003090500871394397108815610408782958984375f,-0.12499999731779087852601151098497211933135986328125f,0.366099970930813700675798827433027327060699462890625f,0.3623999981790781976798143659834749996662139892578125f,0.640999984681606793657238085870631039142608642578125f,0.129000031337142218035296536982059478759765625f,1.1865999883055682317944956594146788120269775390625f,1.2150999654620882051148100799764506518840789794921875f,0.34799998846650215256204319302923977375030517578125f,1.8375000482052570305313565768301486968994140625f,0.198299994003772184214540175162255764007568359375f,0.29630000348091112272186364862136542797088623046875f,0.4085999984234571780206124458345584571361541748046875f,0.83910000711828469643904782060417346656322479248046875f,1.5448999732196331269307165712234564125537872314453125f,-0.17270001720190031324619894803618080914020538330078125f,0.88359999670982336539282187004573643207550048828125f,-0.81459998109340625571661576032056473195552825927734375f,-0.2996999880552291273261289461515843868255615234375f,1.063200017216801906982936998247168958187103271484375f,0.39640002564191778589020032086409628391265869140625f,-0.25120003313869243566358591124298982322216033935546875f,1.4598000057831399534080674129654653370380401611328125f,-0.0527999865382910993361065266071818768978118896484375f,-0.53929998084604680030906820320524275302886962890625f,0.636099996411800372442257867078296840190887451171875f,-0.87679999060630819940342917107045650482177734375f,-0.451299999457597689200838431133888661861419677734375f,0.033000002026557950784990680404007434844970703125f,-0.1472000113010405453195517111453227698802947998046875f,-0.92590002238601432082987230387516319751739501953125f,-0.55519997104555418587779058725573122501373291015625f,-0.7819000167906284826102591978269629180431365966796875f,0.04679999150037783550715175806544721126556396484375f,0.3690999978423119554094000704935751855373382568359375f,0.5400999709010125826580406283028423786163330078125f,0.325099983417987292710904512205161154270172119140625f,-0.9340999747455125401529585360549390316009521484375f,-0.93820001550316778349269952741451561450958251953125f,0.4507999988466500074224541094736196100711822509765625f,-0.4003000337749724746316815071622841060161590576171875f,0.7216999979868534698113080594339407980442047119140625f,0.538700033473968886710281367413699626922607421875f,-1.0659999963045123649862944148480892181396484375f,0.337500002682208588566936668939888477325439453125f,-0.5493000147461888360567172640003263950347900390625f,0.5602000056535008098990147118456661701202392578125f,-1.2964000009059901685759541578590869903564453125f,-0.937199972483515608701054588891565799713134765625f,-0.0042999564826491365465699345804750919342041015625f,-0.282199988278746705105959335924126207828521728515625f,0.237899988210200863392174142063595354557037353515625f,-0.47960001357793735632384368727798573672771453857421875f,-1.698900013720988422250002258806489408016204833984375f,-0.49639997602105034868458233177079819142818450927734375f,-0.6641999817818404761027295535313896834850311279296875f,-0.860300048229099001417807812686078250408172607421875f,0.2685999976783992604367767853545956313610076904296875f,0.66210001115649974590127158080576919019222259521484375f,0.1561999869823456066342259873636066913604736328125f,0.02020001168847063155453724903054535388946533203125f,0.79550000977516166500436156638897955417633056640625f,-0.1791999873697758260959744802676141262054443359375f,-1.18749999359250058006409744848497211933135986328125f,-0.3931000267952680093941353334230370819568634033203125f,-1.9912999835133557979816032457165420055389404296875f,-0.86580005204379606453812812105752527713775634765625f,-1.639100012743472944976019789464771747589111328125f,-0.1630999932676555630450820899568498134613037109375f,-0.097000038504599928046445711515843868255615234375f,1.49970001063048830047819137689657509326934814453125f,1.39919996933937085259458399377763271331787109375f,0.299200003686547255643546350256656296551227569580078125f,0.847899979567527939394722125143744051456451416015625f,0.23709999040663254366023693364695645868778228759765625f,0.2985999959647653856364968305570073425769805908203125f,-0.086000003755092369051027390014496631920337677001953125f,-0.0123999938577414869911308414884842932224273681640625f,-0.0314000144660473434754521804279647767543792724609375f,0.0096000076174735937684090458787977695465087890625f,0.60619998728036872392976874834857881069183349609375f,1.745500040471553493404144319356419146060943603515625f,-1.19249999776482606961280907853506505489349365234375f,1.004800059390068778242266489542089402675628662109375f,-0.05080001814365431300757336430251598358154296875f,1.5192000612795357739059909363277256488800048828125f,-0.5873999969124792386310218716971576213836669921875f,-1.7992000299870962720660827471874654293060302734375f,0.29410002496242526603964506648480892181396484375f,-0.69359999574124764354365879626129753887653350830078125f,-0.339300023090839619754888190072961151599884033203125f,-0.52810001312345289381511292958748526871204376220703125f,-0.5601999720513821667822185190743766725063323974609375f,0.486200006726383637811750304535962641239166259765625f,-1.3604000003546470320969774547847919166088104248046875f,0.15780001686215427980641834437847137451171875f,-0.48780002163052582631053155637346208095550537109375f,0.0770000319480903527846749057061970233917236328125f,0.911800027710199856301187537610530853271484375f,-0.541099995219707086135940699023194611072540283203125f,1.497000052362680744266754118143580853939056396484375f,0.2550999848335975261903740829438902437686920166015625f,-0.6230000333189966710278895334340631961822509765625f,-0.407100020805001516777110737166367471218109130859375f,-1.5828000009179117313351525808684527873992919921875f,0.39000001467764366935853104223497211933135986328125f,0.2336000233829020888975946945720352232456207275390625f,0.32380002933442586510182081838138401508331298828125f,0.12880001134127372353077589650638401508331298828125f,-0.5964999642446640049087136503658257424831390380859375f,0.09180000755637840459399967585341073572635650634765625f,-1.1275999832689762580884007547865621745586395263671875f,0.052000025242567193828335803118534386157989501953125f,-0.1735999965608119310900292475707828998565673828125f,0.20589997980594620941019456950016319751739501953125f,0.290500046413392443156453737174160778522491455078125f,-0.47340001795291897224160493351519107818603515625f,0.65900003049895161666427156887948513031005859375f,-1.41670002387762128392978411284275352954864501953125f,0.57220000390633940501317056259722448885440826416015625f,-0.9880000171326102087476783708552829921245574951171875f,-0.83370001967176765589329079375602304935455322265625f,0.785099973601848244442180657642893493175506591796875f,-1.283500000793486695016554222092963755130767822265625f,1.16019997585117806693233433179557323455810546875f,-0.33829998940303929177986219656304456293582916259765625f,1.09170004339814230576166664832271635532379150390625f,0.244900000283867047556896068272180855274200439453125f,0.12300002668798015736939532871474511921405792236328125f,0.20399999810755253815131027295137755572795867919921875f,-0.88640004596337729214639011843246407806873321533203125f,-0.203900009921193170470132827176712453365325927734375f,-0.758200060541927456370103755034506320953369140625f,0.31240004958808498258804320357739925384521484375f,0.812300031040609216148595805861987173557281494140625f,0.54470003059804472655969220795668661594390869140625f,0.72280004061087999023271777332411147654056549072265625f,0.54479999641403542565143425235874019563198089599609375f,1.31309997659698129979233272024430334568023681640625f,-0.184200005307048275682291205157525837421417236328125f,-0.0769000297546378330793004352017305791378021240234375f,-0.4522000114127997516533241650904528796672821044921875f,-0.92710001950114939717195738921873271465301513671875f,-0.8846000091448427138374199785175733268260955810546875f,0.30170002672746754068811014803941361606121063232421875f,-0.6035000117830928711981641754391603171825408935546875f,-0.25530000050812962086865809396840631961822509765625f,1.020500021640211141260579097433947026729583740234375f,-0.506799999863653649612160734250210225582122802734375f,-0.02340003276094770878756889942451380193233489990234375f,-0.02309997613132031002436406197375617921352386474609375f,-0.147499999925493785468688656692393124103546142578125f,0.66390001893639549734871252439916133880615234375f,-0.70120002159029282129409921253682114183902740478515625f,0.09029997746720896945049616988399066030979156494140625f,0.76850004125013959832557475237990729510784149169921875f,-0.036000043820589411325272521935403347015380859375f,0.719600018718838274622839890071190893650054931640625f,0.7768000019311909198904686491005122661590576171875f,-0.1148000317484143550927910837344825267791748046875f,0.0289999710619444073245176696218550205230712890625f,0.4348000046283002628655367516330443322658538818359375f,-1.43960003145933068680051292176358401775360107421875f,-0.3585000586658722010469091401319019496440887451171875f,-1.3269000178337098105885161203332245349884033203125f,0.139899994397908400411978391275624744594097137451171875f,0.1490000428482896044357630671584047377109527587890625f,-0.2836000108100471539529507936094887554645538330078125f,0.3360000084489580540747510895016603171825408935546875f,-0.664100027644634938184253769577480852603912353515625f,0.165899992285668762548311860882677137851715087890625f,-0.907400028689206106236042614909820258617401123046875f,1.0631000055611128374266627361066639423370361328125f,-1.7928000103801500220157549847499467432498931884765625f,0.53049998584017188374417628438095562160015106201171875f,0.27050003451109017049702742951922118663787841796875f,-0.2863000192947691147082878160290420055389404296875f,0.8701000550925730436802041367627680301666259765625f,-1.33730006585344707303875111392699182033538818359375f,1.07490000257492024360317373066209256649017333984375f,-0.854600001899152861284392201923765242099761962890625f,0.687400026118755391735248849727213382720947265625f,0.031600002536177618228663277477608062326908111572265625f,0.3843999817915264571865918696857988834381103515625f,-0.33449998255073942754478366623516194522380828857421875f,1.03970005399286780090051252045668661594390869140625f,0.0754000175654889748244613656424917280673980712890625f,0.54900002077594367566604205421754159033298492431640625f,0.520600012955814595017045576241798698902130126953125f,-1.538399971014261513602150444057770073413848876953125f,1.4980000186972313169775361529900692403316497802734375f,-0.2880000120475891822735547975753434002399444580078125f,0.2275999734342100122574947818065993487834930419921875f,0.683000014059244708874985008151270449161529541015625f,-1.0736000026702878784590211580507457256317138671875f,1.825000002719461722477944931597448885440826416015625f,-0.42960004308223798119570346898399293422698974609375f,-0.38290005574971519575200318286078982055187225341796875f,-0.708900005972385116592704434879124164581298828125f,1.2035999946236604074556453269906342029571533203125f,-0.524599971333146175567208047141321003437042236328125f,0.239299962443114022647705496638081967830657958984375f,-0.20790001458525664279619604712934233248233795166015625f,-0.37640002772808056619169292389415204524993896484375f,-0.3929000018447634801077583688311278820037841796875f,-0.5723000522375105703076769714243710041046142578125f,1.067900012648105079193783240043558180332183837890625f,-1.5430000285506242363453566213138401508331298828125f,0.741300005641579939918983654933981597423553466796875f,1.094600017452240336979230050928890705108642578125f,0.739500002965330249793396433233283460140228271484375f,0.5519999786019322840502354665659368038177490234375f,-0.68780000985860834106233596685342490673065185546875f,0.8823999760508538514613974257372319698333740234375f,0.38730000611841663982204408966936171054840087890625f,-0.249900011217594197887592599727213382720947265625f,-0.608499982297420416443856083787977695465087890625f,0.6851999864123758943179609559592790901660919189453125f,-1.402300005894899559422128731966949999332427978515625f,-0.13139999539256042027091098134405910968780517578125f,-0.2267000055491923404815679532475769519805908203125f,-0.21810002059266064833309428649954497814178466796875f,-0.4645999566555030213521604309789836406707763671875f,0.02009999511540006000842595312860794365406036376953125f,-1.020399992978571646062846411950886249542236328125f,0.78610001364126735889925612355000339448451995849609375f,-0.08419994132518837659517885185778141021728515625f,0.55769997367188339776333805275498889386653900146484375f,-0.348399995899200121129979379475116729736328125f,0.2358000027209519178228447344736196100711822509765625f,0.209100028240680302360487985424697399139404296875f,0.2409999938644469263948622028692625463008880615234375f,-0.99540002665519722313547390513122081756591796875f,-0.226899988105893068990326355560682713985443115234375f,-0.1704999709203838254012453035102225840091705322265625f,1.3314999975860113412551299916231073439121246337890625f,1.160799952280521463165996465249918401241302490234375f,-0.03219998723566508402882391237653791904449462890625f,1.45730000965744199703522099298425018787384033203125f,-0.2349999953061345792093561613000929355621337890625f,0.0306999792933464465960469169658608734607696533203125f,1.1742999437421559605354559607803821563720703125f,-0.3097999493822458649816553588607348501682281494140625f,-0.698799996775388621017555124126374721527099609375f,-0.1055999768391260307254242434282787144184112548828125f,0.6523999799996615944763789229909889400005340576171875f,0.441700009572505880584003534750081598758697509765625f,-0.8953999777793886227783559661475010216236114501953125f,0.446900008483230859468449125415645539760589599609375f,0.9016999949693680349582791677676141262054443359375f,0.75559999915361419908776952070184051990509033203125f,0.1482000165835020144555755905457772314548492431640625f,1.364800009899586274997318469104357063770294189453125f,0.19850004884973204699605275891372002661228179931640625f,1.974500029042363991038655512966215610504150390625f,-0.04360001161098481237132773458142764866352081298828125f,0.0584000196665523940708908412489108741283416748046875f,0.145199954691529686812145882868207991123199462890625f,-2.011600013637543593603140834602527320384979248046875f,0.527400027534365545278660647454671561717987060546875f,1.18029996813535742461453992291353642940521240234375f,0.5481000137567517693781837806454859673976898193359375f,0.16969997255802127256174571812152862548828125f,0.456500017702580418443858434329740703105926513671875f,-0.275899957156180875017525977455079555511474609375f,-0.337500029876828033792435235227458178997039794921875f,-1.228499987065791998475106083787977695465087890625f,-0.44769998352527640150810839259065687656402587890625f,1.332400021741539486441752160317264497280120849609375f,-1.5750000078789885282759541951236315071582794189453125f,1.6522000196084378753624832825153134763240814208984375f,-1.4817000041522090381107545908889733254909515380859375f,-0.325699999111889848535383862326852977275848388671875f,0.6041999400585893464921127815614454448223114013671875f,-0.8803999881356958656652977879275567829608917236328125f,0.73360001511275729768613018677569925785064697265625f,0.883200005277245736579061485826969146728515625f,-0.42519994865655963423023422365076839923858642578125f,0.8015000103265055297896424235659651458263397216796875f,-1.1120000334382051931925161625258624553680419921875f,-1.009500002741813506901280561578460037708282470703125f,1.5466999910950658403407942387275397777557373046875f,-0.62939998775199057501339439113507978618144989013671875f,0.181200009632110692336937063373625278472900390625f,-0.0780999935656787602766826239530928432941436767578125f,-1.029299971383810241576384214567951858043670654296875f,0.986100004309415911762926043593324720859527587890625f,-1.2317999409668143329099621041677892208099365234375f,1.0777000116139647811763779827742837369441986083984375f,0.8723999301552776142187894947710447013378143310546875f,0.2887000198394067407292595817125402390956878662109375f,-0.370599991032481312913660076446831226348876953125f,-0.43899999810755208073942412738688290119171142578125f,-0.0261999287560581439748830234748311340808868408203125f,-0.82349999891221603576241250266321003437042236328125f,-0.08980002854466440087577439044252969324588775634765625f,1.386100010865927156800125885638408362865447998046875f,2.0720999839246285745275599765591323375701904296875f,-1.3701000020444393801000160237890668213367462158203125f,-0.3525000277534127146594755686237476766109466552734375f,-0.167700007963180297565486398525536060333251953125f,0.145400006985664020930926199071109294891357421875f,-0.8339999850690364979755031527020037174224853515625f,1.18489999765753761806763577624224126338958740234375f,-0.563200006860494539751016418449580669403076171875f,-0.0130000063478945548922638408839702606201171875f,0.94980002120584272606862441534758545458316802978515625f,-0.13729997968748197667565591473248787224292755126953125f,-1.489400061959774035358350374735891819000244140625f,-1.4576000568993396910855153691954910755157470703125f,0.988500022158026236951400278485380113124847412109375f,-1.181400046205521192632659221999347209930419921875f,0.175900026297568956579198129475116729736328125f,0.93010003437995880659627800923772156238555908203125f,-0.5447000117480753633714130046428181231021881103515625f,-0.66980003912448893288456019945442676544189453125f,0.826899995332955217719472784665413200855255126953125f,0.096199949431419184975311509333550930023193359375f,-0.1070000003576279112849078956060111522674560546875f,0.2237000000581146341716021197498776018619537353515625f,0.3932000177383425665311733609996736049652099609375f,0.6607000278711321339386586259934119880199432373046875f,-1.328399996495246426775338477455079555511474609375f,-0.47160003109052806014034331383300013840198516845703125f,0.4068999654933804777101613581180572509765625f,-1.4958000406630340695102177051012404263019561767578125f,1.3964000300377608088098213556804694235324859619140625f,-0.18380001996532124852734568776213563978672027587890625f,-0.5095999816894529743649400188587605953216552734375f,0.1660000013709075350476496168994344770908355712890625f,0.66710000374317335314344745711423456668853759765625f,-0.179900025261939877641026441779104061424732208251953125f,0.4595000560879700657324065105058252811431884765625f,-0.14800003268569683623212540624081157147884368896484375f,0.55070003323555027918700943700969219207763671875f,0.2499999907612799887601795489899814128875732421875f,-0.101500013753771600732989099924452602863311767578125f,-0.56109998620450529660530492037651129066944122314453125f,-0.564800000381469491372854463406838476657867431640625f,0.8995000035613778255338957023923285305500030517578125f,1.6514000200726091094338698894716799259185791015625f,0.00019998752996315538599247929596458561718463897705078125f,0.77599994932860028029608656652271747589111328125f,-0.9587999999605119061385494205751456320285797119140625f,-0.7401999753668910830839422487770207226276397705078125f,0.348899967527388721322267883806489408016204833984375f,-0.67450001555681193821101260255090892314910888671875f,-0.1216000274956225979394730529747903347015380859375f,-0.994999955371022704042616169317625463008880615234375f,0.0436999958485362327564871520735323429107666015625f,0.8956000084668389593645088098128326237201690673828125f,-0.8848000275015834859004826284945011138916015625f,-0.12419997969567742046592684346251189708709716796875f,-0.31300001607090222055518324850709177553653717041015625f,0.32790003710463644370776137293432839214801788330078125f,-0.28870002349019063814239416387863457202911376953125f,-0.773500012844801521083581974380649626255035400390625f,-1.2248999831289051343219398404471576213836669921875f,0.822499955967069151796522419317625463008880615234375f,1.307499986961483617875501295202411711215972900390625f,-0.9502000002145774004702616366557776927947998046875f,0.084900009727478131793532156734727323055267333984375f,1.0304999731183048794491696753539144992828369140625f,0.89650001055002181971076424815692007541656494140625f,-0.6807000159502027969438131549395620822906494140625f,-0.818400018474459312045610204222612082958221435546875f,0.13489998290538807168559287674725055694580078125f,0.861900015375018480057178749120794236660003662109375f,-0.1142999804735183033699286170303821563720703125f,-0.21450001138448715476414463410037569701671600341796875f,-0.82950001794099803564819239909411408007144927978515625f,1.688299994469434128774310011067427694797515869140625f,-0.22429999118372823119216263876296579837799072265625f,-0.1405999942362310406451797462068498134613037109375f,0.4781000010348857021114099552505649626255035400390625f,-0.449399976184964078385064567555673420429229736328125f,0.500900015661865705141053695115260779857635498046875f,-1.4613000171899788171003820025362074375152587890625f,0.42359999551773019899059136150754056870937347412109375f,0.277200023221968905318135512061417102813720703125f,-0.1734000194430349583996076034964062273502349853515625f,-0.1532000251889229502921807579696178436279296875f,0.06700003360584416700618248796672560274600982666015625f,0.210600019121169879099397803656756877899169921875f,0.22360000511035316161922992250765673816204071044921875f,-0.765600018525123715562585857696831226348876953125f,0.430799993798881730722172278547077439725399017333984375f,1.1194999726042151877436481299810111522674560546875f,-1.108599997287243343890850155730731785297393798828125f,-1.1175000013783578101111970681813545525074005126953125f,0.0098000058203935314082855256856419146060943603515625f,0.3318999913468960727414014399982988834381103515625f,-0.0257999797731642832587795055587776005268096923828125f,-0.1654999729320409951327519593178294599056243896484375f,1.363199969011545942265684061567299067974090576171875f,0.6036000601887703265191476020845584571361541748046875f,-0.8141999801427128868880345180514268577098846435546875f,0.475800010544061624528922038734890520572662353515625f,0.51219998015761358800546076963655650615692138671875f,0.0299999608471994694269824321963824331760406494140625f,-0.7864999634623526247878544381819665431976318359375f,0.9611000245377423478743139639846049249172210693359375f,0.4924999901652338252233676030300557613372802734375f,-0.09350000822544135559155620285309851169586181640625f,0.2865000399053105883950820498284883797168731689453125f,0.9065000126361848487022143672220408916473388671875f,0.050600003549456484819302204414270818233489990234375f,0.114400008818507092911431755055673420429229736328125f,-1.0638999878674744170581334401504136621952056884765625f,-0.353800045576690980198009128798730671405792236328125f,0.31469997553825379554837127216160297393798828125f,-1.423399979954957483840871645952574908733367919921875f,-0.51990005055665944322385030318400822579860687255859375f,-1.00789998403787617320404024212621152400970458984375f,0.144299992394447684773695073090493679046630859375f,-1.928100016066431887651333454414270818233489990234375f,-0.2310000443980093631779482166166417300701141357421875f,-0.781899959793686871734053056570701301097869873046875f,-0.0324000028729438671604157207184471189975738525390625f,-0.4908000186279417587087436913861893117427825927734375f,-0.72590000960826817077986561344005167484283447265625f,0.2546000099830327823013931265450082719326019287109375f,0.082799993765353629981973426765762269496917724609375f,0.15029999663382753727347562744398601353168487548828125f,-0.60729999211132490444242648663930594921112060546875f,-0.120800013133138539611621808944619260728359222412109375f,-0.278699988844990809155888200621120631694793701171875f,-0.90649999982118545371889695161371491849422454833984375f,0.310799992066622454700564048835076391696929931640625f,-0.79569999059960261345025855916901491582393646240234375f,0.445299997192620811148344728280790150165557861328125f,-1.100799996648728296122499159537255764007568359375f,0.418699934008717189470871744561009109020233154296875f,-0.83769998337626472295625035258126445114612579345703125f,0.8693000151932237518082047245115973055362701416015625f,-0.08659999948143959347390818948042578995227813720703125f,-0.3900000291690235432184863384463824331760406494140625f,0.499300015193224311360609135590493679046630859375f,1.37450002274662352874656789936125278472900390625f,-0.526499976798892266316443055984564125537872314453125f,0.2365000124871727837927437576581723988056182861328125f,0.2600999828964472015968567575328052043914794921875f,0.31869996794611232804328437850927002727985382080078125f,0.3907000099897379730151669718907214701175689697265625f,1.55309995370507269996096511022187769412994384765625f,-0.20790003134906243076329701580107212066650390625f,0.9343999929115176517058216631994582712650299072265625f,-0.6659999942928547245202253179741092026233673095703125f,1.0098999396175154519283978515886701643466949462890625f,0.1950000149756669909351103342487476766109466552734375f,0.759799998779595142650578054599463939666748046875f,-1.1899000038042657489967268702457658946514129638671875f,-0.0202000014811754002863608548068441450595855712890625f,0.38960001473277794392657824573689140379428863525390625f,0.037699982705712731245739632868207991123199462890625f,0.473200005444883498029184920596890151500701904296875f,2.082299957838653625685765291564166545867919921875f,-0.16329995898604376947105265571735799312591552734375f,1.39630001331567843436687326175160706043243408203125f,0.76530001928359292495684940149658359587192535400390625f,-0.12929997859224717782211655503488145768642425537109375f,0.19800000636652137497861758674844168126583099365234375f,0.1337000133015209257791866548359394073486328125f,-0.45329997239708941236813188879750669002532958984375f,1.860399968689680605393732548691332340240478515625f,1.212300034169852214205320706241764128208160400390625f,0.45909997563958171440390287898480892181396484375f,0.3304000170439482797490882148849777877330780029296875f,1.1226000388130550344811808827216736972332000732421875f,-0.3062000075459481163164809913723729550838470458984375f,0.3896000028491020117371590458787977695465087890625f,-0.899199957865477017548982985317707061767578125f,1.1325999915018678532163676209165714681148529052734375f,-0.44539998940229441615201722015626728534698486328125f,1.214600015552341982782991181011311709880828857421875f,-1.47570000373125065351587181794457137584686279296875f,-0.022099998732655723454598728494602255523204803466796875f,-0.105400066255032953677073237486183643341064453125f,0.149800015543400777406901624999591149389743804931640625f,0.5096000443488362119381918091676197946071624755859375f,1.1603999839261167625892312571522779762744903564453125f,-0.5371999291956424116278867586515843868255615234375f,0.91449998810142307803516814601607620716094970703125f,-1.1808000160574911863164970782236196100711822509765625f,0.35880000187903637876019047325826250016689300537109375f,0.5442000277146699094288351261639036238193511962890625f,0.116400000533461611684771241925773210823535919189453125f,0.171400004184246057548079988919198513031005859375f,0.66649998179078107085615556570701301097869873046875f,0.36990002738535299187105920282192528247833251953125f,-0.36419997209608556687499003601260483264923095703125f,-0.432499954625963578536129716667346656322479248046875f,0.870799998325109125829612821689806878566741943359375f,0.1029999586455525584227643776102922856807708740234375f,0.93610001408830323388343686019652523100376129150390625f,-1.61839997734725482558815201628021895885467529296875f,-0.4815000043101609339402102705207653343677520751953125f,-0.56529993259608790623360619065351784229278564453125f,-0.14740001054704254812577346456237137317657470703125f,-0.51499999243766125811561096270452253520488739013671875f,1.2974999896809455979251879398361779749393463134765625f,-0.4323999884188174380739155822084285318851470947265625f,-0.47040001045018431380384527074056677520275115966796875f,-0.34560001420378705372371541670872829854488372802734375f,-0.0395999903321264667965806438587605953216552734375f,-0.3004000171184539880187003291212022304534912109375f,0.09529997752308805303300687228329479694366455078125f,-0.1236000107169150652453026850707828998565673828125f,0.512599991911649066622658210690133273601531982421875f,-0.9873999876737595826625693007372319698333740234375f,-0.149099998177587844683245066335075534880161285400390625f,-0.32220000526607039947890598341473378241062164306640625f,-0.23920000918135053691315761170699261128902435302734375f,0.5844999985694887545406572826323099434375762939453125f,1.4058000486165287412632096675224602222442626953125f,-0.1453000139564277759518517996184527873992919921875f,0.19029998240321799318763851260882802307605743408203125f,0.5925000451132660383990469199488870799541473388671875f,-0.61850002597644959667633202116121537983417510986328125f,-0.39379998567402296760775470829685218632221221923828125f,0.124500027682632907755078122136183083057403564453125f,-0.465000046696514512944986563525162637233734130859375f,1.273100055480004044738961965776979923248291015625f,0.3573999890148638058917640591971576213836669921875f,-1.784400025880336304595630281255580484867095947265625f,-1.321200002181529331579667996265925467014312744140625f,-1.807400040721893308415246792719699442386627197265625f,-0.97100000287964938205931275660987012088298797607421875f,-0.4501000086568296421063450907240621745586395263671875f,0.723300010748952448835780160152353346347808837890625f,-0.094700029871613244925043773037032224237918853759765625f,1.0981999847695231853350605888408608734607696533203125f,1.145900004355608903239271967322565615177154541015625f,0.19429995868056970653725556985591538250446319580078125f,0.6130000070184464799893930830876342952251434326171875f,-0.44870002192556890374675049315555952489376068115234375f,-0.100999994479119614698703344402019865810871124267578125f,0.596600036361814023422311947797425091266632080078125f,-0.1662999971106646768248538137413561344146728515625f,-0.16349999159201977771971314723487012088298797607421875f,0.15379999591857218543822227729833684861660003662109375f,-0.129399947090447664521661863545887172222137451171875f,0.645400001025199454574021729058586061000823974609375f,0.127300023180246579812546769971959292888641357421875f,0.09979999009966811218674820338492281734943389892578125f,-0.057700012917816234647716555627994239330291748046875f,-0.071200013860315269909051494323648512363433837890625f,0.8646999808654192154477868825779296457767486572265625f,-0.38079998707473217134378273840411566197872161865234375f,1.19809997829198788821258858661167323589324951171875f,-0.1036000253945592586291013503796420991420745849609375f,-0.22530000747442269215525811887346208095550537109375f,-0.31200000505149372287405640236102044582366943359375f,-1.55870001946687608551656012423336505889892578125f,-0.290300017607211913173159700818359851837158203125f,-1.25899999833107045787983224727213382720947265625f,0.28519998937398194360781644718372263014316558837890625f,0.53859999436289018870382960813003592193126678466796875f,-0.66709999351724957872278309878311119973659515380859375f,-0.31170002797544060779699748309212736785411834716796875f,-0.876900002485513585526177848805673420429229736328125f,0.187500030584634103103525148981134407222270965576171875f,-1.266200052137673282004470820538699626922607421875f,0.623800042317063319075032268301583826541900634765625f,-0.55300002544000859128203728687367402017116546630859375f,-1.4694000540435314672293998228269629180431365966796875f,2.34680003113746682430473811109550297260284423828125f,-0.0436000230848794245730459806509315967559814453125f,0.5434000496923925282999334740452468395233154296875f,0.378600038731099264310842045233584940433502197265625f,0.037799960684775868458018521778285503387451171875f,1.6211000646591191554080069181509315967559814453125f,-0.9988000174880031778457123436965048313140869140625f,-0.6414999971389765409668370921281166374683380126953125f,-0.86200003258138913064811958975042216479778289794921875f,-1.061000038791447774855214447597973048686981201171875f,-0.6317999998450272158834195579402148723602294921875f,-0.6489000019490713810910165193490684032440185546875f,0.15000003119930671591220061600324697792530059814453125f,-0.9192000025130802054462719752336852252483367919921875f,-0.444499957386403410364295041290461085736751556396484375f,0.12190002718418868621341744074015878140926361083984375f,-0.51600000151991931574002592242322862148284912109375f,0.52829999080002298939007232547737658023834228515625f,-0.708600002223252811717202348518185317516326904296875f,-0.82070004284679909378752427073777653276920318603515625f,0.893800043863058224502537996158935129642486572265625f,0.2637000333994634626577635572175495326519012451171875f,0.5746000126838683996766121708787977695465087890625f,-0.23830000659525385486858795047737658023834228515625f,0.4798999912500381270774596487171947956085205078125f,-0.577899994170665731729741310118697583675384521484375f,0.83799996283650468598125371499918401241302490234375f,-0.38550002579390962864636094309389591217041015625f,-0.265699973332882422027978464029729366302490234375f,-0.091400051048398101016800865181721746921539306640625f,-0.704799979519844299602482351474463939666748046875f,-0.8881000070512288235846654060878790915012359619140625f,-0.3256999855518341036031415569595992565155029296875f,0.822800010082125776733619204605929553508758544921875f,0.7371000108957286300892519648186862468719482421875f,-0.1321000061273576164921905728988349437713623046875f,0.920799967479705827599900658242404460906982421875f,0.4102000038653610403116545057855546474456787109375f,0.2534999814033511711386381648480892181396484375f,0.006299981427192680172311156638897955417633056640625f,0.01740001330375662291771732270717620849609375f,-1.0307999799966811593776583322323858737945556640625f,-0.3118999966740603202453030462493188679218292236328125f,-0.17700003135204323001516968361102044582366943359375f,0.227700009751320209261393756605684757232666015625f,-0.26659999590516125778094647102989256381988525390625f,0.8871000006139269089544541202485561370849609375f,-1.3632999364852906865053228102624416351318359375f,1.190400060182810637599004621733911335468292236328125f,-0.2402000413417813007299628225155174732208251953125f,-0.253600051248074720433578477241098880767822265625f,0.309600066030025544705495121888816356658935546875f,-1.02170000897645874005092991865240037441253662109375f,1.305999980211257849305184208787977695465087890625f,-0.284600001060962171095525263808667659759521484375f,-0.2324999883770946240701960050500929355621337890625f,0.176500017702579725664691068232059478759765625f,-0.6228000223755838504757775808684527873992919921875f,-0.33330000205040022365210461430251598358154296875f,0.1379000275492667793031387191149406135082244873046875f,-1.0172999805629248104565931498655118048191070556640625f,-0.4032999858081343091953385737724602222442626953125f,-1.479100009316205888154627245967276394367218017578125f,-0.62079996658563540989916873513720929622650146484375f,-1.990599981346726554676251907949335873126983642578125f,-0.05119999726414814489316995604895055294036865234375f,-0.622100013354420777744735460146330296993255615234375f,-1.85630001942515310275894080405123531818389892578125f,-0.1040999978423113869752114624134264886379241943359375f,-0.344699972856044922764340299181640148162841796875f,0.494599972972273871363313446636311709880828857421875f,-0.95330001412034004459883362869732081890106201171875f,-1.0736000065445896467508646310307085514068603515625f,-0.01619996880292884355867499834857881069183349609375f,-0.224099994489550535803346065222285687923431396484375f,0.3702999913811684251641054288484156131744384765625f,0.71900000863149760021997281000949442386627197265625f,1.34609999009743308562292440910823643207550048828125f,0.60780004580765989441459851150284521281719207763671875f,2.033599973724782739026295530493371188640594482421875f,-0.6699000063002105154197352021583355963230133056640625f,0.251000020816921409760880123940296471118927001953125f,-1.23230002653300818593606891226954758167266845703125f,-1.1144999707043172776366191101260483264923095703125f,1.120200021225213671272058491013012826442718505859375f,0.5829999737143527482885474455542862415313720703125f,-0.848100022697448441277856545639224350452423095703125f,1.183000008583068307643770822323858737945556640625f,0.42570000812709363202657186775468289852142333984375f,-1.1044999449253083412259002216160297393798828125f,-0.354300004613399277531016196007840335369110107421875f,-1.004999976158142516169391456060111522674560546875f,-0.6051999871015549814501355285756289958953857421875f,-0.53249999664723846048985933521180413663387298583984375f,-0.51760001064985983187938245464465580880641937255859375f,-1.38400005253404412286499791662208735942840576171875f,0.50789999724403023240171251018182374536991119384765625f,-1.358200043815374069566814796417020261287689208984375f,1.706099985682964170763398215058259665966033935546875f,1.752500004023312296652648001327179372310638427734375f,0.091800013032556027070540949353016912937164306640625f,0.112200028288365505346746431314386427402496337890625f,-0.5606999992609029703771739150397479534149169921875f,-0.7811000136971468776891924790106713771820068359375f,1.2668999859452245715374374412931501865386962890625f,-0.347300047767163277967483736574649810791015625f,0.452600048685074085597079829312860965728759765625f,1.431900024688243178161428659223020076751708984375f,-1.0667999980568882989473422640003263950347900390625f,-0.174099979364871870757269789464771747589111328125f,-0.7571000029608609960263265747926197946071624755859375f,0.2463000054180619624588644001050852239131927490234375f,-0.93490000365525471881511521132779307663440704345703125f,0.444600040008873598207372879187460057437419891357421875f,0.75200002077221828056252661554026417434215545654296875f,0.312899983292818095748089035623706877231597900390625f,0.346999978899956484923450261703692376613616943359375f,-0.8487000814557077443822663553874008357524871826171875f,-2.222200034844874938499970085103996098041534423828125f,-0.688000000610947015644569546566344797611236572265625f,-0.36729997981786777216939299250952899456024169921875f,-0.589600040400028557741052281926386058330535888671875f,-0.19020002629160881468806110206060111522674560546875f,0.244000024706125362428110747714526951313018798828125f,0.67439999488592139442744155530817806720733642578125f,-0.281099994549155240974869229830801486968994140625f,0.2081999933004379244039228069595992565155029296875f,0.4108000160574913905975336092524230480194091796875f,0.5401999707102778103262608055956661701202392578125f,-0.5257000002294776042077728561707772314548492431640625f,0.5940000093579289597300885361619293689727783203125f,0.44959999122619620504082149636815302073955535888671875f,-0.536199975769221737209591083228588104248046875f,-0.95590004486814184048171227914281189441680908203125f,-1.5573000265702605648954204298206605017185211181640625f,0.3171999702483410121800488923327066004276275634765625f,-0.8606000125646584564265140215866267681121826171875f,0.273100031265616660647310709464363753795623779296875f,0.69479997802972803810916957445442676544189453125f,-1.36030000784695204174568061716854572296142578125f,0.22819995336532628016357193700969219207763671875f,0.59499998793005914876630413345992565155029296875f,-0.36180002815723355524824000895023345947265625f,0.2036000014781951250597558100707828998565673828125f,-0.937000014364719024939631708548404276371002197265625f,0.3056000215053558122235699556767940521240234375f,0.45419999854564618857466484769247472286224365234375f,-0.67230000291466662787343011586926877498626708984375f,-0.6737000261351469010406844972749240696430206298828125f,0.3622000021740792163882360910065472126007080078125f,0.930700037910789479411732827429659664630889892578125f,-0.46099999345466502465029634549864567816257476806640625f,1.700900102144480730004261204157955944538116455078125f,-1.4263000195741657449843842186965048313140869140625f,-1.298200019079446487779705421417020261287689208984375f,1.4221000082135208941735982079990208148956298828125f,0.23499995872378232064647818333469331264495849609375f,1.1959000343441967828539418405853211879730224609375f,0.044000063896181274003538419492542743682861328125f,0.118199973630904509036554372869431972503662109375f,1.138400002455711046422948129475116729736328125f,-0.12690001738667522346304394886828958988189697265625f,0.0253000234186646633105510773020796477794647216796875f,-0.266800011914968937531966730603016912937164306640625f,-1.42630001718997956317025455064140260219573974609375f,0.18020003383532212648532322418759576976299285888671875f,0.955300028354674868324991621193476021289825439453125f,1.4610000317133966518667875789105892181396484375f,-1.01990002347379959957152095739729702472686767578125f,0.1297000121951104034678792231716215610504150390625f,0.780500006869434326262080503511242568492889404296875f,0.334599955761431200329525381675921380519866943359375f,1.5825999966055144252408126703812740743160247802734375f,0.49210000940561382520854749600403010845184326171875f,-0.691900036534666806886662016040645539760589599609375f,-1.036099998795986465438545565120875835418701171875f,0.046599976161122214080023695714771747589111328125f,0.30570000615268944077485002708272077143192291259765625f,0.190900009477883714215096233601798303425312042236328125f,-0.45969999394118821189891832545981742441654205322265625f,-0.0635000135898593498495756648480892181396484375f,-0.574700005638599353829931715154089033603668212890625f,-0.3044999977499245336076683088322170078754425048828125f,0.722099969768524641011708808946423232555389404296875f,-0.493000024825334559608336348901502788066864013671875f,0.155800041985511228403993300162255764007568359375f,0.393600000211595801857811238733120262622833251953125f,0.58079998178482128423638641834259033203125f,-0.2667000037610531837373173402738757431507110595703125f,-0.2543000019311900672391857369802892208099365234375f,-0.281500000491738067154301461414434015750885009765625f,0.0581000137567517782599679776467382907867431640625f,0.344699987757206116611996549181640148162841796875f,-0.091799953055382133015882573090493679046630859375f,0.45359998598098749678086960557266138494014739990234375f,-1.119000054527074627941374274087138473987579345703125f,1.114500027496367895452067386941052973270416259765625f,1.3516000521570443737573441467247903347015380859375f,-1.176700015234947382936070425785146653652191162109375f,-0.56100000844895792084798813448287546634674072265625f,-0.01489998476803311433513954398222267627716064453125f,0.892800017011165181202159146778285503387451171875f,-0.29650002336502012667551753111183643341064453125f,-0.7347000278413293727908239816315472126007080078125f,0.661800021004676608527006465010344982147216796875f,0.111600028091670022689640973112545907497406005859375f,0.342599953019618652660938096232712268829345703125f,0.388299998474121110803025658242404460906982421875f,-0.174499978601932781430150498636066913604736328125f,0.094899981117247733664044062606990337371826171875f,-0.504599984669685142080197692848742008209228515625f,0.82710000638812797291166134527884423732757568359375f,0.587500000819563883425189487752504646778106689453125f,0.794100012706220326208494952879846096038818359375f,0.5198000014245514055488683879957534372806549072265625f,0.132099990332126271397328309831209480762481689453125f,-0.498600008481741241439522127620875835418701171875f,-1.76969999237656594459622283466160297393798828125f,-0.53859999909400979589690905413590371608734130859375f,-0.453300064188242313178989206789992749691009521484375f,-1.4211000220417968620267856749705970287322998046875f,-0.751500039309263012654582780669443309307098388671875f,0.1327000206291675254988149390555918216705322265625f,0.342599990049004876624394455575384199619293212890625f,-0.76420003207325937211180644226260483264923095703125f,-1.6529000212907789535421443360974080860614776611328125f,-0.61019999186992635031856480054557323455810546875f,-0.187499993070959991570134661742486059665679931640625f,0.6455000160336492598389668273739516735076904296875f,0.6244999979734420225696567285922355949878692626953125f,0.277200034993887012291224891669116914272308349609375f,0.5380999908834696032755573469330556690692901611328125f,-0.160699988085031719720063847489655017852783203125f,-0.9295000109374520480542969380621798336505889892578125f,0.045999978721142031901081281830556690692901611328125f,-0.345100008302926308800806509680114686489105224609375f,-1.33689994347691598619576325290836393833160400390625f,-0.218700008216500318969366389865172095596790313720703125f,-1.193599961692095146048586684628389775753021240234375f,-1.0057000186324118740088806589483283460140228271484375f,-1.339599961721896992372649037861265242099761962890625f,0.22750000447034846029481514051440171897411346435546875f,-1.2841999527245764323168941700714640319347381591796875f,-0.1656000124529006800599972848431207239627838134765625f,-0.2711999989032742863770408803247846662998199462890625f,0.2277999892950060001339807058684527873992919921875f,-0.864300055798889133029661024920642375946044921875f,0.277000056684017170738343338598497211933135986328125f,0.27460001059770544618032772632432170212268829345703125f,-1.088799985972046524551615220843814313411712646484375f,0.02800000105798261529344017617404460906982421875f,-0.029399992278218434194059227593243122100830078125f,0.9368000134050842309108020344865508377552032470703125f,-0.66040000489950134721084396005608141422271728515625f,1.08679999902546331469466167618520557880401611328125f,-1.061000001221894972758263975265435874462127685546875f,1.01020004439652044680997278192080557346343994140625f,-0.1805000544041401866479645832441747188568115234375f,1.0908999888584005599767579042236320674419403076171875f,-0.9984999770820135456261823492241092026233673095703125f,0.51160003304630574660905040218494832515716552734375f,-0.589800034393370697927139190142042934894561767578125f,-0.868400054311752711555527639575302600860595703125f,-1.141400019681453326114706214866600930690765380859375f,-0.490499986156821332627941956161521375179290771484375f,-0.08809999304413818066450403421185910701751708984375f,0.050500023186206444147927641097339801490306854248046875f,0.5242000217914579707212396897375583648681640625f,-1.428199976909160451299385385937057435512542724609375f,0.2683999984323970355859501069062389433383941650390625f,-0.69349999727308730967934025102294981479644775390625f,0.205699995684624159508757657022215425968170166015625f,-1.076500018447636985996496150619350373744964599609375f,1.05020001630783088586440499057061970233917236328125f,-0.477800012764334425252599203304271213710308074951171875f,-0.0830000293701884661601297921151854097843170166015625f,-0.51940001075565778165810115751810371875762939453125f,0.3042000197798018223949156890739686787128448486328125f,-0.1806000026553867543466225242809741757810115814208984375f,0.85119996656775498422575765289366245269775390625f,1.23469997524022989665581917506642639636993408203125f,1.29219997181296353261359399766661226749420166015625f,0.56890003555119061928735391120426356792449951171875f,1.2851999826312070496214801096357405185699462890625f,-0.24720001860260942816438500813092105090618133544921875f,0.26670002860873864847235381603240966796875f,0.0732000066556033335274378259782679378986358642578125f,0.2627999908968805531372936457046307623386383056640625f,0.49790002546310407893770388909615576267242431640625f,1.51319993160963139899877205607481300830841064453125f,0.1176999869525433839445582862026640214025974273681640625f,1.0321999850004910381784384298953227698802947998046875f,0.54480003267973697855808268286637030541896820068359375f,0.7215999893486497729355733099509961903095245361328125f,-0.086399997329711819560316143906675279140472412109375f,0.36439998803138706762183574028313159942626953125f,0.29299998489022271996873314492404460906982421875f,-1.41139999382793757121135058696381747722625732421875f,0.438900009474157304367736287531442940235137939453125f,-0.740200014668702888087636893033050000667572021484375f,-0.4475000149011609806848355219699442386627197265625f,-0.831800022979080733165346828172914683818817138671875f,1.0554999949112531254513669409789144992828369140625f,0.3133999841459085278216889491886831820011138916015625f,0.49670005060657917983490960978087969124317169189453125f,-1.125100025886296872812408764730207622051239013671875f,0.4933000266373159803379166987724602222442626953125f,-0.61209998165219925425617475411854684352874755859375f,0.6334999769330018626334322107140906155109405517578125f,-1.081800051365793269297910228488035500049591064453125f,1.12480003521293436818950794986449182033538818359375f,-0.4564000172972679791882910649292171001434326171875f,0.48400000786781305350814363919198513031005859375f,-0.433900024822353458375800983048975467681884765625f,-1.8630999846249813867160582958604209125041961669921875f,0.018599972867965330891593112028203904628753662109375f,-0.763800013911724473558706449694000184535980224609375f,-0.296799965646862506929437586222775280475616455078125f,0.76830003392025825181832487942301668226718902587890625f,1.2933000071726734603316799621097743511199951171875f,-0.171000042516738115505603445853921584784984588623046875f,0.80459997829943807889918616638169623911380767822265625f,-1.310000015795230954296357595012523233890533447265625f,-0.61049999070167515480278552786330692470073699951171875f,-0.4730999614536763875349834052030928432941436767578125f,0.780400007954239693930276189348660409450531005859375f,-0.478400021678209608211318482062779366970062255859375f,1.02540002225935467095041531138122081756591796875f,0.13740000585317613257529956172220408916473388671875f,0.146000001817941704729264529305510222911834716796875f,0.628899999564886069691738157416693866252899169921875f,0.43270001152083248374680124470614828169345855712890625f,-0.784599949800968143875934401876293122768402099609375f,1.2184999907910822880552359492867253720760345458984375f,-0.97659999024271915146755418390966951847076416015625f,1.323100003977864869142422321601770818233489990234375f,-0.8121999814242120141472014438477344810962677001953125f,0.4878999847829341707239336756174452602863311767578125f,-0.43439999466240397651972671155817806720733642578125f,0.809400041005015946637968227150849997997283935546875f,-1.5207000451564791632108608609996736049652099609375f,1.0280000033862883856983216901426203548908233642578125f,-0.491999977856874437520673382095992565155029296875f,0.49210000707730683355833889436325989663600921630859375f,0.083399984872340571229187844437547028064727783203125f,0.24150000673159954800439663813449442386627197265625f,-0.631600001195072646709149921662174165248870849609375f,-0.235099998766183926335315845790319144725799560546875f,-0.867399971170723649294131973874755203723907470703125f,0.0515999853253361617788641524384729564189910888671875f,-0.308199948596955264346064495839527808129787445068359375f,0.6628999897748233394167982623912394046783447265625f,-1.0228999286428095416567884967662394046783447265625f,0.2886000014781948674880140970344655215740203857421875f,0.59890003584921391865947271071490831673145294189453125f,0.350099998393654576744893347495235502719879150390625f,-0.62430000342130664847672960604541003704071044921875f,0.87040003105104002401049001491628587245941162109375f,-0.5724999153986569666585637605749070644378662109375f,0.479499983891844727690312311096931807696819305419921875f,0.5578999852672221759775084137800149619579315185546875f,-0.6181999629765753123677995972684584558010101318359375f,-0.38190000833421944292922489694319665431976318359375f,0.8097999898761514270262296122382394969463348388671875f,-0.4838000454276805140096939794602803885936737060546875f,0.5877000201821325209294855085317976772785186767578125f,0.88180001962631970879868958945735357701778411865234375f,-0.82549999248981420496562577682198025286197662353515625f,2.14830002110525963843201679992489516735076904296875f,-0.25950000231340542011793104393291287124156951904296875f,-0.393900027001649277735850773751735687255859375f,-0.45060003513991819801276506041176617145538330078125f,-1.987100000762939799869855050928890705108642578125f,-0.01010001664757709249897743575274944305419921875f,-0.55179995990991681509285626816563308238983154296875f,-1.1787999917834992569254382033250294625759124755859375f,-0.95770000516921227617928025210858322679996490478515625f,0.07879996812790623916811227900325320661067962646484375f,0.3492000141918654154693513191887177526950836181640625f,-0.024899967259168764854848632239736616611480712890625f,0.87209998049736014991140109486877918243408203125f,0.0894000222295521229654013950494118034839630126953125f,0.93270004108101212114689815280144102871417999267578125f,-0.88809999174028553259319096468971110880374908447265625f,1.4381000284530218724654559991904534399509429931640625f,-1.385100008824468265089535634615458548069000244140625f,-0.385400027754157659121148071790230460464954376220703125f,-0.4749000568896535323659691130160354077816009521484375f,-1.4318000009439881825556994954240508377552032470703125f,0.6026999614715577990864403545856475830078125f,-0.6734999879971146352630739784217439591884613037109375f,0.36379997040033362765143465367145836353302001953125f,-0.7038000020280479329670697552501223981380462646484375f,0.44719996737986800194875058878096751868724822998046875f,0.430300026641040833652596120373345911502838134765625f,-0.536899973465502444724961605970747768878936767578125f,0.70629998846799135758800503026577644050121307373046875f,-0.861800031137466504560507019050419330596923828125f,0.0752999983847140175186041233246214687824249267578125f,-0.650599979558586394290387033834122121334075927734375f,-0.05850001247227198941658343755989335477352142333984375f,0.12770004691109082539668406752753071486949920654296875f,0.5611000439278786000585341753321699798107147216796875f,2.0652999890342353950245524174533784389495849609375f,0.6182000090397894442872939180233515799045562744140625f,1.171099946828186855185549575253389775753021240234375f,-0.302100002700090186635861755348742008209228515625f,0.1589000047057869124955686856992542743682861328125f,-0.452900003707408682629420582088641822338104248046875f,-0.32370003287792104007536408971645869314670562744140625f,0.39880000996291598003296030583442188799381256103515625f,-0.52759997186958840220682986910105682909488677978515625f,-0.44159999374449210307602697866968810558319091796875f,-0.0267999747365712526203651577816344797611236572265625f,0.9999000097274777232314590946771204471588134765625f,-0.19660002108812335563925444148480892181396484375f,0.2348999884933231641070960904471576213836669921875f,-1.82910000395178773402449223794974386692047119140625f,-0.292000007361173263831233271048404276371002197265625f,-0.63949998292326881710323505103588104248046875f,-0.668500056356192029483054284355603158473968505859375f,-1.979900042025745765528199626714922487735748291015625f,0.34440001839250322657193237319006584584712982177734375f,-0.3500999950408936189916175862890668213367462158203125f,-0.0096999841064207004137642798013985157012939453125f,-0.91209999673962638411239822744391858577728271484375f,0.64810004601776682875424739904701709747314453125f,1.90780008451342641961900881142355501651763916015625f,-0.3688999909222123729790610013878904283046722412109375f,0.30159998338818549967754734097979962825775146484375f,0.650200020107627008059125728323124349117279052734375f,0.80600001358985906563248136080801486968994140625f,0.7079000031113622526390827260911464691162109375f,-0.6897000039435920992758610736927948892116546630859375f,-0.1282000031352037439091873238794505596160888671875f,-1.329700022104382650667275811429135501384735107421875f,1.82939999905824635106910136528313159942626953125f,-1.02740001205578490584002793184481561183929443359375f,0.46800003313273219607282271681469865143299102783203125f,-0.6974000076413149340481822946458123624324798583984375f,0.725699985738099240251131050172261893749237060546875f,0.3029000572212046105136096230125986039638519287109375f,-0.72280001909732760623228386975824832916259765625f,0.6611000240720807941130487961345352232456207275390625f,0.2655000202059749625504991854541003704071044921875f,0.9084000267818570506506148376502096652984619140625f,-1.22130001316666625399420809117145836353302001953125f,0.37669999688789246317099923544446937739849090576171875f,-0.011599992775916945220160414464771747589111328125f,-0.599199992212653409495715095545165240764617919921875f,-0.316700026299059356205134463380090892314910888671875f,0.53599996508657932192676298654987476766109466552734375f,1.4202999520421031309780346418847329914569854736328125f,-1.2553000001728535917067119953571818768978118896484375f,-0.20970003748983234714842183166183531284332275390625f,-0.25290001160502406651886531108175404369831085205078125f,1.2497999855920678413667701533995568752288818359375f,-0.760600018823147205893064892734400928020477294921875f,-1.845300042901932169314704879070632159709930419921875f,0.87980002258420009209061163346632383763790130615234375f,0.5899000574484478676140497555024921894073486328125f,-0.8338000151038169605044458876363933086395263671875f,-1.4306000084668397676068707369267940521240234375f,1.1902000308364630853930066223256289958953857421875f,0.6997000243768096705565540105453692376613616943359375f,-0.1622000074267386793280820711515843868255615234375f,-0.27879999021887780230599673814140260219573974609375f,-0.38550001588463800317185814492404460906982421875f,-0.314200027081370958370598600595258176326751708984375f,-0.2607999942272900550932490659761242568492889404296875f,-0.09549999189376734420875436626374721527099609375f,-0.154100004100799825579315438517369329929351806640625f,0.4428999566197386883459330420009791851043701171875f,0.204999990686773880810278569697402417659759521484375f,0.17670000584721545777711071423254907131195068359375f,-0.31940001112818716677566044381819665431976318359375f,-1.175100046412647092353154221200384199619293212890625f,0.02629999815374584049010309172444976866245269775390625f,-0.6238000195741648656877487155725248157978057861328125f,-0.8319000660389670454009092281921766698360443115234375f,-2.44460005857944562279726596898399293422698974609375f,0.1164999946802856012340043889707885682582855224609375f,-1.22739999929666510070092044770717620849609375f,-0.105400004303455308019010772113688290119171142578125f,-1.103200041542947484884962250362150371074676513671875f,1.815100006589293268888241073000244796276092529296875f,0.315800032169371969192894766820245422422885894775390625f,-0.6987999614596365205443362356163561344146728515625f,0.928399997016787548176353084272705018520355224609375f,-1.2679000072836874579707000521011650562286376953125f,-0.56310002629235389459694260949618183076381683349609375f,0.7512000132083898051860160194337368011474609375f,-0.1466999969996513886183464592249947600066661834716796875f,-0.78700002617389042125495279833558015525341033935546875f,-1.1041000646181411415369666428887285292148590087890625f,1.169800014127790444007359837996773421764373779296875f,-1.768000031512230396657514575053937733173370361328125f,1.22490001300573414511063674581237137317657470703125f,-0.6746999961204827034322306644753552973270416259765625f,0.3157000171422961187772671109996736049652099609375f,0.39019998762309560103034300482249818742275238037109375f,1.3177999714881176718250799240195192396640777587890625f,-1.1233999918758870162349694510339759290218353271484375f,-1.01799998887628273536165579571388661861419677734375f,-0.340300014403462913747944185161031782627105712890625f,-0.2985000052452095875565873939194716513156890869140625f,1.453000022366643495530524887726642191410064697265625f,2.2498000441536305515910498797893524169921875f,-1.13160003546774401428365308674983680248260498046875f,-0.4278000010296703070622470477246679365634918212890625f,0.690800020900368405563085616449825465679168701171875f,1.911999999538063210735572283738292753696441650390625f,-0.61340002644658131458754724008031189441680908203125f,0.68729991145878932545798534192726947367191314697265625f,-0.667900016522407380392678533098660409450531005859375f,0.14310000597089567175856927860877476632595062255859375f,-0.47820002518892312792786469799466431140899658203125f,-0.77709999405741658762281076633371412754058837890625f,-0.398200035396218510186372441239655017852783203125f,-0.32570001192689002067481851554475724697113037109375f,1.3643000573635106764669444601167924702167510986328125f,0.112600013592839776066512058605439960956573486328125f,-0.143800050345064267531824953039176762104034423828125f,-0.14120001223981404336882405914366245269775390625f,0.395600024485588175338079963694326579570770263671875f,1.521899987025558953490644853445701301097869873046875f,0.21970000440925330220665045999339781701564788818359375f,0.3392999776422980229284576125792227685451507568359375f,0.58990001796037005998840641041169874370098114013671875f,0.57319999935403453950044649900519289076328277587890625f,-0.26459999329373229404183121005189605057239532470703125f,0.1461000468708579802523672697134315967559814453125f,-0.92400001733005066029136287397705018520355224609375f,-0.37530000717639921958834747783839702606201171875f,0.797099988842010276357541442848742008209228515625f,0.122100037587434595565127892768941819667816162109375f,0.04710002609491414915510176797397434711456298828125f,-0.0486000345215209517135690475697629153728485107421875f,0.33809999513029964646193548105657100677490234375f,-0.0025000032410023020901235213386826217174530029296875f,0.58970005421638571618814239627681672573089599609375f,1.13020001457557039969969991943798959255218505859375f,-0.63989998845607132249568849147181026637554168701171875f,0.8660000051707033019710024746018461883068084716796875f,-0.954400003193318635652531156665645539760589599609375f,-0.17849999941512939560794848148361779749393463134765625f,0.617199990476667181837910902686417102813720703125f,-0.278500008560717215999602558440528810024261474609375f,-0.0591999972790475670336718394537456333637237548828125f,-0.0191999987505375724339273801888339221477508544921875f,-0.4417000110626219111509271897375583648681640625f,-0.08340001422762888250161950054462067782878875732421875f,-1.191700033675134218213997883140109479427337646484375f,-0.3997999882370228963424096946255303919315338134765625f,0.61800005850195904866239970942842774093151092529296875f,0.189399980655311883648295179227716289460659027099609375f,-1.30410001162588695677868599887005984783172607421875f,-0.4643000259220595449249913144740276038646697998046875f,-0.56529994798153637702853302471339702606201171875f,-0.13029997141361271406623245638911612331867218017578125f,-0.3977999921634793878411073819734156131744384765625f,1.094400000473857215155248923110775649547576904296875f,1.70589998813197052385248753125779330730438232421875f,-1.074899972083419807944437707192264497280120849609375f,0.4146000029236078798788867061375640332698822021484375f,-0.0384000278063119093729937958414666354656219482421875f,-0.999300030653178961159710524952970445156097412109375f,0.78170000689029706109067774377763271331787109375f,-0.10569999192207990557079710924881510436534881591796875f,0.0053999831065530135010277490437147207558155059814453125f,-0.1986999917507168422758923043147660791873931884765625f,-0.5294000084459782318191400918294675648212432861328125f,0.88970003752708404221039018011651933193206787109375f,-0.489399986727535374342323848395608365535736083984375f,0.162700009006261581134822336025536060333251953125f,-0.001099992686510287143164532608352601528167724609375f,0.232700003939867006153008333058096468448638916015625f,-0.10880000746697160440845664197695441544055938720703125f,-0.4412999834761024420259900580276735126972198486328125f,-0.376500014871358479329188639894709922373294830322265625f,-1.052300023031234932346933419466949999332427978515625f,1.1535999878436322685359982642694376409053802490234375f,-0.91079997984766958296631855773739516735076904296875f,-0.43450002606213100619925171486102044582366943359375f,-0.400700019973516674554048222489655017852783203125f,0.52400000902265253532874567099497653543949127197265625f,-0.2364000133484600585376256276504136621952056884765625f,0.329000010736286707668796225334517657756805419921875f,-0.0267999882593745741843349605915136635303497314453125f,0.52699998217821164558927193866111338138580322265625f,0.309600011994689394700941420524031855165958404541015625f,-0.4939000101447106327867686559329740703105926513671875f,0.0844999935403467627992313282447867095470428466796875f,0.1267000008925800447201481802039779722690582275390625f,-0.61559999209418847510022487767855636775493621826171875f,0.7972999727398157876478990147006697952747344970703125f,-0.402199990495294590342467699883854947984218597412109375f,0.170100011357664726574512314982712268829345703125f,-0.799900033401697907464722447912208735942840576171875f,0.8843000001430514345202027470804750919342041015625f,-0.52219999608322975603869053884409368038177490234375f,0.9464000015020364031670396798290312290191650390625f,-0.2305000156611207717816114382003434002399444580078125f,-0.866500026978552995160498539917171001434326171875f,-0.02840001482367515706073390902020037174224853515625f,-0.34990003528296942381103917796281166374683380126953125f,-0.39219998485147922195892533636651933193206787109375f,-1.2220999896615740709648889605887234210968017578125f,0.65010001355558622027075443838839419186115264892578125f,-0.215499994315206944150986601016484200954437255859375f,0.1596999868631362728965683572823763824999332427978515625f,0.499099967704713931571092189187766052782535552978515625f,-0.457299990136921241656864367541857063770294189453125f,-0.16660001233369092776825937107787467539310455322265625f,-0.12869998623728751141470638685859739780426025390625f,1.03320002999454718661809238255955278873443603515625f,-0.36060005220174773388208677715738303959369659423828125f,-0.48670002562850722682696869014762341976165771484375f,-0.883899972742796080638072453439235687255859375f,-0.42679999567270243687744368799030780792236328125f,-0.0782000272750853486769528899458236992359161376953125f,-1.1325000251457095767904093008837662637233734130859375f,0.89869998985081878117853193543851375579833984375f,-0.92479998410195118907495270832441747188568115234375f,-0.12230001796483946918669971637427806854248046875f,-1.325099990868567889634732637205161154270172119140625f,0.49499999158084440065152875831699930131435394287109375f,-0.73639996782541228270702049485407769680023193359375f,-0.70300003264844423966906106215901672840118408203125f,0.877000047445296360137945157475769519805908203125f,0.548100017705559938718806733959354460239410400390625f,-1.9772000309705735077159260981716215610504150390625f,-1.372900025761128350865192260243929922580718994140625f,1.213199963796139257965478464029729366302490234375f,0.511600000226497098054778689402155578136444091796875f,-0.133900028622150557566783390939235687255859375f,0.66669999645948418987018158077262341976165771484375f,-0.2819999909512700408953378428122960031032562255859375f,-0.6937000228196392725976693327538669109344482421875f,-0.371300014805793499039765492852893657982349395751953125f,0.128399966245889896043763656052760779857635498046875f,-1.72290002013593923635426108376123011112213134765625f,2.484999983161686909483023555367253720760345458984375f,-0.0175999458484346060771486008889041841030120849609375f,0.110700013715028688920938293449580669403076171875f,-0.6652000220261513607766801214893348515033721923828125f,1.521599989795685559101912076584994792938232421875f,-0.814900019096584049549392148037441074848175048828125f,0.0860999841928471454366444959305226802825927734375f,0.39149997799098457118560645540128462016582489013671875f,0.3490000895261768931732149212621152400970458984375f,0.29329998357296005462302446176181547343730926513671875f,-0.991300023150444076236453838646411895751953125f,-0.574100004622339898929794799187220633029937744140625f,0.3206999972492454720196519701858051121234893798828125f,0.1943000205948949010714699170421226881444454193115234375f,0.10010000442862498903906498526339419186115264892578125f,0.64529997152537144078365827226662077009677886962890625f,0.891199945333600584973510194686241447925567626953125f,-1.471099985906481943942480938858352601528167724609375f,-1.8144000481948250325103799696080386638641357421875f,-0.429899989238381241563047296949662268161773681640625f,1.832499986328184338191249480587430298328399658203125f,-0.897700001704691885606735013425350189208984375f,-1.1204999751672148899928060927777551114559173583984375f,0.34939999101161856032149444217793643474578857421875f,-0.4662999949872503879788609992829151451587677001953125f,-0.8057000116288663971175765254884026944637298583984375f,0.5812000030755990476194483562721870839595794677734375f,0.8608999929189682376318160095252096652984619140625f,-0.492300032866001657794186030514538288116455078125f,0.2930000289231546872770195477642118930816650390625f,1.033600038433075241073311190120875835418701171875f,0.0451000270783907808436197228729724884033203125f,0.2517000074863435798988575697876513004302978515625f,-0.288700007843971651055881011416204273700714111328125f,-0.02199999350309411738635390065610408782958984375f,-0.04459996686279765754079562611877918243408203125f,-0.316400037264823907889876863919198513031005859375f,0.167500014752149706254158445517532527446746826171875f,0.420000021383166721733459780807606875896453857421875f,-0.033700023937225598302802609396167099475860595703125f,-2.1108000030189746354380986304022371768951416015625f,0.01299994655698488710271476520574651658535003662109375f,-1.227500009667128466617214144207537174224853515625f,-0.03250000534579167155158074820064939558506011962890625f,0.1371999944813548921729307039640843868255615234375f,-0.680900010502337948992135352455079555511474609375f,0.371600029078871241239312439574860036373138427734375f,0.2976000284194952882899087853729724884033203125f,-0.015699997361004580653798257117159664630889892578125f,-0.4205000175237660897664682124741375446319580078125f,0.16269999201893792228901247653993777930736541748046875f,-0.7888999916672698731190394028089940547943115234375f,0.25239999816045166625144702265970408916473388671875f,-0.4657000049233435134965475299395620822906494140625f,-0.3776000185102226236466549380565993487834930419921875f,0.92880003402829203196233720518648624420166015625f,-1.51240004793033033791971320169977843761444091796875f,1.20150000429153447356611650320701301097869873046875f,-0.98890000515282150406193295566481538116931915283203125f,1.381300039727985762283424264751374721527099609375f,-0.052800001420825692921567906523705460131168365478515625f,0.3251000046521423936241035335115157067775726318359375f,0.726700023579597509382210773765109479427337646484375f,1.23810002496987703324293761397711932659149169921875f,-0.9163000054925678483641604543663561344146728515625f,1.316700001153350285676424391567707061767578125f,-0.0906999839872124624662319547496736049652099609375f,-0.2655999921500689087139335242682136595249176025390625f,0.68149997478723545185630428022705018520355224609375f,-0.9220000103786585299303624196909368038177490234375f,0.8225000047683710402424139829236082732677459716796875f,1.01749999634921550750732421875f,-0.9382000221341850743073109697434119880199432373046875f,0.407499962486326250399315540562383830547332763671875f,0.096099957296252203065023422823287546634674072265625f,-0.44890003605410522258267747019999660551548004150390625f,-1.2804000118844209854529481162899173796176910400390625f,-0.0438000053808094147456131395301781594753265380859375f,-0.3245999933309853258123212071950547397136688232421875f,0.30300001007318488888131469138897955417633056640625f,1.3222000421464443764563156946678645908832550048828125f,-1.068199994641542360795938293449580669403076171875f,-1.1692000122547145934959189617075026035308837890625f,0.73550001469254500108263528090901672840118408203125f,-1.14120000665187770749753326526843011379241943359375f,0.442500008493662111419553184532560408115386962890625f,-1.191200034219026093751381267793476581573486328125f,0.671599976161122658169233545777387917041778564453125f,0.97630005816817355679404499824158847332000732421875f,-1.03190002826452342787888483144342899322509765625f,-1.2871000788450250862382517880178056657314300537109375f,1.5398000225842007893106710980646312236785888671875f,-1.01429999582171337380032127839513123035430908203125f,0.04950000892579442535179623519070446491241455078125f,-0.1740000340193521566334311501123011112213134765625f,-0.0956999802619211692444878281094133853912353515625f,-0.4296000091075899973702689749188721179962158203125f,-0.8748000185608864054387368014431558549404144287109375f,-0.905300010678172117906115090590901672840118408203125f,-1.251299991708993974981467545148916542530059814453125f,0.604899965992569121908672968856990337371826171875f,-1.15250002875924106859883977449499070644378662109375f,0.4978999723404637922641313707572408020496368408203125f,-0.223100002580881362490572428214363753795623779296875f,-1.305100009271501715346630589920096099376678466796875f,-1.052199995356797757040112628601491451263427734375f,-0.1943000406742101393575694601167924702167510986328125f,-0.966700030359625817055757579510100185871124267578125f,0.168899977734685347030563207226805388927459716796875f,-1.0616999975770708974920353284687735140323638916015625f,0.3758000357270241398310872682486660778522491455078125f,0.5407000082015986919259376008994877338409423828125f,-0.050899980476498740689095257039298303425312042236328125f,-0.108800018754601435233553274883888661861419677734375f,0.97270003608912258830088148897630162537097930908203125f,0.6146000097408894635719889265601523220539093017578125f,0.49000001057982445207272803600062616169452667236328125f,1.77660000480860436056218532030470669269561767578125f,-0.1958000333614648980073980055749416351318359375f,-0.5184999855384229139332319391542114317417144775390625f,-0.528300025594234501369328427244909107685089111328125f,-1.9669999568462372963040252216160297393798828125f,0.77059999475777107846852231887169182300567626953125f,-1.1903999655604362573058097041212022304534912109375f,-1.295999979130923573933387160650454461574554443359375f,-0.3721000218853347529801567361573688685894012451171875f,0.8960999933570621411860201988019980490207672119140625f,-0.07919997895062014237055336707271635532379150390625f,-0.03469999923110034245610222569666802883148193359375f,1.049799970579147867510982905514538288116455078125f,0.302800015372037822913853233330883085727691650390625f,0.291400039350987405395443374800379388034343719482421875f,-0.8175000263750551443564518194762058556079864501953125f,-1.509800087590516870506007762742228806018829345703125f,-0.0031999643921849507677279689232818782329559326171875f,-0.339899988511949135983059022692032158374786376953125f,-0.57690003524944233337379273507394827902317047119140625f,1.54690005300045019254184808232821524143218994140625f,-1.39780003042221068199069122783839702606201171875f,0.921300033134221596498036888078786432743072509765625f,-1.5558000086069103673480640281923115253448486328125f,0.135100001299381489161532954312860965728759765625f,0.4152999616533519233740889831096865236759185791015625f,0.9377000187665218700061586787342093884944915771484375f,-1.5181999834284185535437927683233283460140228271484375f,-1.150300010156630747815142967738211154937744140625f,0.5740999742984769227405195124447345733642578125f,-0.57980002690553700261943959048949182033538818359375f,-1.487100061634183845171719440259039402008056640625f,-0.05570000656247142689636575596523471176624298095703125f,-1.213400001394003169252755469642579555511474609375f,-1.244899998123198070487660515937022864818572998046875f,0.52620005427971516809293461847119033336639404296875f,-0.82369999245852210112417424170416779816150665283203125f,1.3151000069618223875522744492627680301666259765625f,-0.537100006723404188591075580916367471218109130859375f,0.678599980914592837422105731093324720859527587890625f,0.944800002616643208597224656841717660427093505859375f,1.7596999972194431460792429788853041827678680419921875f,-0.322499988451599772787403708207421004772186279296875f,-1.0280000181943174109022720585926435887813568115234375f,-0.31600001511722808356097402793238870799541473388671875f,0.0103000217795372606133241788484156131744384765625f,0.118400003051757796157517077517695724964141845703125f,-0.736700001525878889196974341757595539093017578125f,-0.768200005742907254813189865672029554843902587890625f,-1.065199986207485238765002577565610408782958984375f,-0.0856999988883730967614837936707772314548492431640625f,-1.2606999840617181263269230839796364307403564453125f,-1.0941000448182227611226835506386123597621917724609375f,-1.3735999964490537283978710547671653330326080322265625f,0.5903000030785798823984578120871447026729583740234375f,-1.0156999819010490426762771676294505596160888671875f,-1.027700007367133849811580148525536060333251953125f,0.3001000346779818528375471942126750946044921875f,1.50920005308389715281691678683273494243621826171875f,0.78240003148317338599326831172220408916473388671875f,-0.7109000038713215730012962012551724910736083984375f,1.18729999024867982626574303139932453632354736328125f,0.50100000360608110128168846131302416324615478515625f,0.491600011849403273345160414464771747589111328125f,-0.270399976587295753915896057151257991790771484375f,1.536100052589178144302195505588315427303314208984375f,-0.759500032246112510847524390555918216705322265625f,-0.17990002090334922257852667826227843761444091796875f,0.49420002521872508083333741524256765842437744140625f,1.8731000477313990426608825146104209125041961669921875f,-0.8863999905496833253693012011353857815265655517578125f,0.2264000404685744261001900667906738817691802978515625f,-0.5388000126078724694167476627626456320285797119140625f,1.107200011748074697237598229548893868923187255859375f,0.004300014150143027791273198090493679046630859375f,-1.4208000881791129632603087884490378201007843017578125f,0.5043000655591498571084230206906795501708984375f,-0.1017000127017499977455372572876513004302978515625f,0.3107999962389467185630564927123486995697021484375f,-0.885200038361549701448893756605684757232666015625f,0.6571000287771227021949016489088535308837890625f,-0.4620999763250352998511516489088535308837890625f,1.034200005251168619224699796177446842193603515625f,0.012800001439452035612021063570864498615264892578125f,0.829300005805492457255923000047914683818817138671875f,0.378600033143162928439551251358352601528167724609375f,1.0062000152200460956208871721173636615276336669921875f,-1.01870003831684652340072716469876468181610107421875f,0.0347999737828963429109307980979792773723602294921875f,0.4052000115767115939746645381092093884944915771484375f,0.33979995865821788214589105336926877498626708984375f,-0.37550003287196265233660597004927694797515869140625f,-1.4958000686585908312054016278125345706939697265625f,0.82090003196001060103981217253021895885467529296875f,-0.02649999073147757400192858767695724964141845703125f,-0.62590001184344290496852636351832188665866851806640625f,-0.9105000088810921710091861314140260219573974609375f,0.171499999001622160932356564444489777088165283203125f,0.192999998003244410682555098901502788066864013671875f,-0.353100028359890405482701680739410221576690673828125f,-0.764500015556812240191675300593487918376922607421875f,0.693800035965443040453237699694000184535980224609375f,-0.3438000005006784931538277305662631988525390625f,-0.99400004750490200677859320421703159809112548828125f,1.42249999284744177430184208787977695465087890625f,0.78880000526905025282076167059130966663360595703125f,-0.8460999910473816498779342509806156158447265625f,-0.64580002473741815460783755042939446866512298583984375f,-0.5115000046268101119295579337631352245807647705078125f,-0.064600016558170381131276371888816356658935546875f,-1.10030001544654343348383918055333197116851806640625f,-1.1318000527441502978120979605591855943202972412109375f,-2.14530000270605114565114490687847137451171875f,-0.06599999660253530464615323580801486968994140625f,-1.1889999822378154448188070091418921947479248046875f,0.516799984347820373642434788052923977375030517578125f,-0.706900020292401176646990279550664126873016357421875f,1.21320002883970712304062544717453420162200927734375f,1.621600066089630587384817772544920444488525390625f,-2.1341000192999839413232621154747903347015380859375f,-0.91810004490017949052571566426195204257965087890625f,1.114900028204917958873920724727213382720947265625f,-0.270499997109174383780327843851409852504730224609375f,-0.425600004518031482803053222596645355224609375f,1.37990006262660092062333205831237137317657470703125f,-1.1992000488370664346149396806140430271625518798828125f,-0.3724000093549488799027358254534192383289337158203125f,0.3344000107929108178694832531618885695934295654296875f,1.842100090020895919451504596509039402008056640625f,-0.8980000243782999547192957834340631961822509765625f,1.4912000439047812250237257103435695171356201171875f,-1.1045000069141384102522351895458996295928955078125f,0.666199986684322453811546438373625278472900390625f,0.7093000216007236957693748991005122661590576171875f,0.19329998461604080972620067768730223178863525390625f,-0.69100004547834448231924397987313568592071533203125f,-0.54519996199309783690978292725048959255218505859375f,-0.19579995600581145964724782970733940601348876953125f,-1.050100047045946727308773915865458548069000244140625f,-0.68980002630949055486553334048949182033538818359375f,-0.416200020360946609798702411353588104248046875f,0.5206000278383500212697754250257275998592376708984375f,-0.869000012934207699544231218169443309307098388671875f,0.7429000517636541456312215814250521361827850341796875f,0.25210003909617706430168482256704010069370269775390625f,1.658000029891729010245171593851409852504730224609375f,-0.77970002955496386931599772651679813861846923828125f,0.17219997017383548154612071812152862548828125f,-0.2917000040590764786685440412838943302631378173828125f,-0.65649999982118600883040926419198513031005859375f,-0.4042000050276517431058209695038385689258575439453125f,0.040800006222724505278165452182292938232421875f,0.28180000804066640540668231551535427570343017578125f,0.110399998492002993089045048691332340240478515625f,-0.43909998793303961495126941372291184961795806884765625f,0.20589995939135530278463193099014461040496826171875f,0.83890001323670115862540797024848870933055877685546875f,1.2021999796733264798120899286004714667797088623046875f,-0.9135999962627885384591763795469887554645538330078125f,1.31459998414814460687694008811376988887786865234375f,0.97129997792541977741365144538576714694499969482421875f,-0.3768999597564341996758230379782617092132568359375f,0.81480001686587966158725748755387030541896820068359375f,-0.5674999873712660569680110711487941443920135498046875f,-0.6751000026404856413364541367627680301666259765625f,0.115999979019164811688824556767940521240234375f,0.61579999400377261054018163122236728668212890625f,0.978900029814243399783890708931721746921539306640625f,-0.23449999544024446862522381707094609737396240234375f,-0.8682000470936304115099346745410002768039703369140625f,1.201600023919343840361761976964771747589111328125f,0.323400022050738922274604192352853715419769287109375f,0.733399982413649897949881051317788660526275634765625f,-0.64830000905394502552780977566726505756378173828125f,0.1808000031679861674405174198909662663936614990234375f,-0.7373000289171931509457635911530815064907073974609375f,1.3393999805435539141029721577069722115993499755859375f,-0.480400015069544272794388461989001370966434478759765625f,-0.0469999553188689134231026400811970233917236328125f,0.086099996877460949207261364790610969066619873046875f,-1.26319998743310524247362991445697844028472900390625f,0.28970001428127289955227752216160297393798828125f,-0.3356000226974487077313824556767940521240234375f,0.78680002055764219903721823357045650482177734375f,-0.38530004279017493473702415940351784229278564453125f,-0.81160005521178302245743907405994832515716552734375f,-0.114299977865814472721694983192719519138336181640625f,1.3886000369429591927428191411308944225311279296875f,-0.0487000179022556434205171171925030648708343505859375f,0.084099950009584478749502522987313568592071533203125f,-1.6015000103265040198863289333530701696872711181640625f,-0.0695000024437899543983121475321240723133087158203125f,-0.5324999912828209946979995947913266718387603759765625f,-0.89570003930777264411489113626885227859020233154296875f,-1.119199998564272657830542812007479369640350341796875f,-0.051099983689934180830505283665843307971954345703125f,0.6098000040881330630071488485555164515972137451171875f,0.76940006801337001984819607969257049262523651123046875f,1.189700024116039145383183495141565799713134765625f,-0.1000999883353710373512512887828052043914794921875f,-0.4376000143006442133497557733790017664432525634765625f,0.168900009548664353786762148956768214702606201171875f,-0.25460002423226857359850328066386282444000244140625f,0.615000013560056846273482733522541821002960205078125f,0.2000999977231021631496332702226936817169189453125f,-0.1884999958574773426533965903217904269695281982421875f,0.4501999880701303968777438058168627321720123291015625f,-0.16209998709112405368415466000442393124103546142578125f,0.3416999819308521590954796920414082705974578857421875f,-0.200199995930492813389633965925895608961582183837890625f,-0.5658999799549579989843550720252096652984619140625f,-0.8565000191181899591441606389707885682582855224609375f,-0.5118999866902829154469145578332245349884033203125f,0.061400007238983977941870762151665985584259033203125f,-0.9564999914765355271129010361619293689727783203125f,-0.1437000011384490516519463199074380099773406982421875f,-0.5390000171065327805308697861619293689727783203125f,0.0542000032395122399719866734812967479228973388671875f,0.2624000311106449156994813165511004626750946044921875f,-0.1645000227466228981398899122723378241062164306640625f,-0.777100011938810286693524176371283829212188720703125f,0.10410000499486926628378569148480892181396484375f,0.0142999895259732223706805598339997231960296630859375f,-0.1755999931931493396319865496479906141757965087890625f,-0.52030000937432030472251653918647207319736480712890625f,0.6819999909326437403933596215210855007171630859375f,0.00579999754279858503736022612429223954677581787109375f,0.3067999907925724745183515551616437733173370361328125f,0.994700003403424570791457881568931043148040771484375f,-0.172299982425570430422112622181884944438934326171875f,0.153000005304813502249317025416530668735504150390625f,0.5017999736189846515799217741005122661590576171875f,0.571700035053492428005483816377818584442138671875f,-0.01120002110600548661523134796880185604095458984375f,0.776300047439337337351616952219046652317047119140625f,0.7343000242084263273767419377691112458705902099609375f,1.152200033988058702760781670804135501384735107421875f,0.485299990636109956909649554290808737277984619140625f,1.06390002050101717401275891461409628391265869140625f,0.6530999993026258554351670682081021368503570556640625f,0.377700011315941441836230296758003532886505126953125f,0.2081999909535046100472754915244877338409423828125f,0.347799991641938532893618685193359851837158203125f,0.4740999859586356723895050890860147774219512939453125f,0.1627999912321567155260026993346400558948516845703125f,0.22569999389648476295633372501470148563385009765625f,-1.0547000270962716417244564581778831779956817626953125f,0.24640000269412976052763042389415204524993896484375f,0.9706999826461080171924322712584398686885833740234375f,0.283199996131659137432734496542252600193023681640625f,0.047499984726310007232541465782560408115386962890625f,-0.74409999545812599563987532746978104114532470703125f,-0.0533000278294089024910817897762171924114227294921875f,-0.96059997330009938565353877493180334568023681640625f,-0.533599996300041734542674021213315427303314208984375f,-0.305199993434548400017547464813105762004852294921875f,-0.6517999900847673000470194892841391265392303466796875f,-1.53409999471306779383894536294974386692047119140625f,0.1691000194489962193955534530687145888805389404296875f,-0.4374999639391905503771340590901672840118408203125f,0.112599992805719306687706193770281970500946044921875f,-0.0560999847888954406016637221910059452056884765625f,-0.307400011068583278728283403324894607067108154296875f,-1.377400017476080762435231008566915988922119140625f,0.017400017252564570213735350989736616611480712890625f,0.768199926543235989129243534989655017852783203125f,0.097299973857402743959710278431884944438934326171875f,0.0768000013351439747566473670303821563720703125f,-0.18920001642703976330039949971251189708709716796875f,1.6493000583320860386038475553505122661590576171875f,-1.8958000352054842352345076506026089191436767578125f,0.035100004912913096433157988940365612506866455078125f,0.7659999512657524878278536561992950737476348876953125f,0.1673999925911429098590588182560168206691741943359375f,1.1948999899089329801427084021270275115966796875f,1.683200042027234832886506410432048141956329345703125f,-0.78969999625086817474084455170668661594390869140625f,-1.986100101166965004040321218781173229217529296875f,1.484800054323674345368999638594686985015869140625f,2.032800090920925839554911362938582897186279296875f,-1.215700050520897690375932143069803714752197265625f,-0.65250001057982398577905769343487918376922607421875f,0.337000024795532482357884873636066913604736328125f,0.052700034487248359482691739685833454132080078125f,-0.5319000342249875501465794513933360576629638671875f,-1.13330000771284034044583677314221858978271484375f,0.0055000289604076879612648554029874503612518310546875f,-1.001300000835954495670421238173730671405792236328125f,0.71620001764148455425385009220917709171772003173828125f,0.8891999765664342358917338060564361512660980224609375f,1.2261000056505209077073459411622025072574615478515625f,0.677700010868907742889177825418300926685333251953125f,-0.27700004014372847649383402313105762004852294921875f,0.540400014957785668201495354878716170787811279296875f,0.58349999794363949234821120626293122768402099609375f,0.3490999751925463190360687804059125483036041259765625f,1.29660002436637977751843209262005984783172607421875f,1.117300022733211495307159566436894237995147705078125f,-0.51940000621080439913157533737830817699432373046875f,-1.040399995809793143308752405573613941669464111328125f,-0.18019995580911629673437346355058252811431884765625f,-0.200599999228119774130618679919280111789703369140625f,0.650500016927718860415552626363933086395263671875f,0.495500022962689801175883985706605017185211181640625f,0.4680000455379484947115997783839702606201171875f,0.764199982154369461539999974775128066539764404296875f,-0.397199960041046296055355924181640148162841796875f,0.2089999871551990739959592247032560408115386962890625f,-0.0345000021457675387637209496460855007171630859375f,-0.76340000670254237746803482878021895885467529296875f,0.4731999748229984703584705130197107791900634765625f,0.6128000075489286047059067641384899616241455078125f,-0.351899992054701016286344383843243122100830078125f,-1.28780000441968400792802640353329479694366455078125f,-1.44990001136660584535320595023222267627716064453125f,-0.19540000236630439278684434611932374536991119384765625f,-1.2742000055864455276832813979126513004302978515625f,0.0612000004120170360000230402874876745045185089111328125f,0.0129999924525618926196557367802597582340240478515625f,0.667800007772445525233706575818359851837158203125f,-0.84100007662177223011212845449335873126983642578125f,0.671200074005128044518642127513885498046875f,0.0267999917984003133142323349602520465850830078125f,-1.19279996485710171327809803187847137451171875f,0.285300013881921277203446152270771563053131103515625f,-1.0638999937534325823662584298290312290191650390625f,0.05490001881718686860267553129233419895172119140625f,0.5328999911904332975609577260911464691162109375f,-0.5796000322043901586965830574627034366130828857421875f,0.3091000531256202776830832590349018573760986328125f,0.70470000899136042704640203737653791904449462890625f,0.596599977949261894849541931762360036373138427734375f,1.4153000510975723980067186857922933995723724365234375f,-0.2695000117942694117800783715210855007171630859375f,-0.114199997558444454881509955157525837421417236328125f,0.24019999945089198778447325821616686880588531494140625f,-0.679800051045418740613968111574649810791015625f,-1.6957000752568252011087679420597851276397705078125f,-0.959800018039346003462242151726968586444854736328125f,-0.40579998685121498880334911518730223178863525390625f,-0.360499951735139045894129594671539962291717529296875f,0.4991000143826003210278940969146788120269775390625f,0.159800014537572199202486444846726953983306884765625f,0.436200005161762049965545884333550930023193359375f,-0.0603000018119816960648904569097794592380523681640625f,-0.265200010719895740152196594863198697566986083984375f,0.4522000364840028652224646066315472126007080078125f,2.04840004410445697402565201628021895885467529296875f,0.582500020936131956972303669317625463008880615234375f,0.29949999236688029657926790605415590107440948486328125f,0.20539998126253511401273499359376728534698486328125f,-0.107700001201778527981645083855255506932735443115234375f,0.08199999473243957393009395673288963735103607177734375f,0.74389999017715435769559917389415204524993896484375f,0.4683000395827001671733569310163147747516632080078125f,-0.7309999912381162090468933456577360630035400390625f,0.53919999489486170052288116494310088455677032470703125f,-1.1610000302791601001217713928781449794769287109375f,-0.99820001781284750652645243462757207453250885009765625f,0.548299974483252849921655069920234382152557373046875f,-0.98550001931190589488096520653925836086273193359375f,1.28440001738667408659466673270799219608306884765625f,-0.734700051012635579894549664459191262722015380859375f,0.5974000061511990367080215946771204471588134765625f,-0.68969999928697889313156110802083276212215423583984375f,0.45550001126527828176904222345910966396331787109375f,-0.9996000084556639908583974829525686800479888916015625f,0.14009995585083867553066738764755427837371826171875f,-0.5207999976359307670037424031761474907398223876953125f,-0.3155000100731850665169986314140260219573974609375f,-0.4216000077515842381359334467560984194278717041015625f,-1.182300020386278571749016919056884944438934326171875f,0.0322000151753429886269941562204621732234954833984375f,0.7048000297367578159679624150157906115055084228515625f,1.013800028887390514142907704808749258518218994140625f,1.23739997560381898011883095023222267627716064453125f,0.0755999538540843563083626577281393110752105712890625f,0.2133000296175480148264114177436567842960357666015625f,-1.207500006556511351618610206060111522674560546875f,-0.20160002019405454110057007710565812885761260986328125f,0.037299979445337694272666340111754834651947021484375f,0.0164999317601319805959292352781631052494049072265625f,-0.365599998781084511989547536359168589115142822265625f,1.793199966739118078606907147332094609737396240234375f,-0.141299974945187312158623171853832900524139404296875f,0.129899990244209728729174457839690148830413818359375f,0.19329998983144758284424824523739516735076904296875f,0.10939999145865453300530134583823382854461669921875f,0.620100031176209487426831401535309851169586181640625f,0.91219999312162425297856316319666802883148193359375f,0.5794000195845960821117159866844303905963897705078125f,0.44070000313520463208760702400468289852142333984375f,0.749500016674399471838796671363525092601776123046875f,0.7144000087812536481379765973542816936969757080078125f,-0.758099973374604996223524722154252231121063232421875f,0.8997000038504605612388331792317330837249755859375f,0.746299975392222147974052859353832900524139404296875f,-1.64399997553229315627731921267695724964141845703125f,-0.753400037398934241394954369752667844295501708984375f,-1.254399988068640059424296850920654833316802978515625f,-0.791300043006240461096467697643674910068511962890625f,0.0690000557005403525323572466732002794742584228515625f,0.285200037840009468226298849913291633129119873046875f,0.518800008249282740280250436626374721527099609375f,-0.528099991703033566636804607696831226348876953125f,0.88070000326558928183118268862017430365085601806640625f,-0.418699965822696640316280536353588104248046875f,0.234599993070215173673886965843848884105682373046875f,0.2828000187993053859969450058997608721256256103515625f,0.658400023466348738310216504032723605632781982421875f,-0.2371999801203605873212154619977809488773345947265625f,-0.03140002009123576964810808931360952556133270263671875f,0.5367000185132031475632174988277256488800048828125f,-1.00620002410486364397002034820616245269775390625f,0.4732999826788901742702364572323858737945556640625f,-1.5369999964460721031400680658407509326934814453125f,2.0419999662712218224669413757510483264923095703125f,0.169300077349693101069760814425535500049591064453125f,-0.353300005365908109666861491859890520572662353515625f,0.70690001137033087186267721335752867162227630615234375f,-1.3154000707626352095758193172514438629150390625f,0.41120000419318658568812452358542941510677337646484375f,-0.7506999784365298022947854406083934009075164794921875f,0.25659999657571297415614708370412699878215789794921875f,0.1199999938160181134350068532512523233890533447265625f,-0.05350001452118158251636259592487476766109466552734375f,-0.96369999033510700403581950013176538050174713134765625f,-0.16139999319463960159026783003355376422405242919921875f,0.9095000050514940870272084794123657047748565673828125f,-0.313000008955598207904813534696586430072784423828125f,0.3168999973818655835344770821393467485904693603515625f,0.16909995991885684585298577076173387467861175537109375f,0.373399907237291994732686362112872302532196044921875f,-1.11579998096525656592348241247236728668212890625f,-0.54870000553429021383777808296144939959049224853515625f,-0.769199968072771245175545118399895727634429931640625f,0.3275000350549823746604261032189242541790008544921875f,1.342600034677982279163188650272786617279052734375f,0.5219000333309171679729843162931501865386962890625f,-1.254000009506941015757774948724545538425445556640625f,-0.38200005072355214252866062452085316181182861328125f,0.9783999971657990624152034797589294612407684326171875f,-0.13309997967034592480928267832496203482151031494140625f,0.10949997238069786309466735474416054785251617431640625f,-1.6609999806582924630760089712566696107387542724609375f,-0.0067000138938426712087448322563432157039642333984375f,-0.0095000163391232295140298447222448885440826416015625f,0.151099987210332908915688676643185317516326904296875f,-0.4449000508919365159954395494423806667327880859375f,0.6326000024169677882213136399514041841030120849609375f,0.5802000043869011136621338664554059505462646484375f,-0.7728999874651432122618643916212022304534912109375f,-0.0622000279903414110549420001916587352752685546875f,0.474300041121245019581920132623054087162017822265625f,1.320300033402443062868769629858434200286865234375f,0.4447999950170515859326769714243710041046142578125f,0.38910001039654018573088478660793043673038482666015625f,-0.72729998875856427531516601447947323322296142578125f,-0.56210002099126554764296770372311584651470184326171875f,0.77229997672587635815233397806878201663494110107421875f,-0.9854000019192696857572855151374824345111846923828125f,0.944099999891221397518847879837267100811004638671875f,-0.1825999959535897776419233196065761148929595947265625f,0.7773000193804495694394063320942223072052001953125f,0.14849998820200538407476642532856203615665435791015625f,-0.178899970805644681348667290876619517803192138671875f,-0.5545000420250001393895900037023238837718963623046875f,1.0385000150799754692343412898480892181396484375f,0.859400046853721732986741699278354644775390625f,-1.5072000178575510886957999900914728641510009765625f,0.76370000676363691827219781771418638527393341064453125f,-0.438800013613700912173953838646411895751953125f,-0.384400017610192268335822518565692007541656494140625f,-1.78630002337396120992707437835633754730224609375f,0.840700022357701914899053008412010967731475830078125f,1.811799983751773535090023869997821748256683349609375f,-0.45350001224875413630144294074852950870990753173828125f,-0.634600012497603671590695739723742008209228515625f,-0.2256000203877686072218011759105138480663299560546875f,-0.3448000158041717444490359412156976759433746337890625f,-0.8690999773159624464113903741235844790935516357421875f,-0.20979999646991565054321426941896788775920867919921875f,0.2881999941572548351587101933546364307403564453125f,1.933800049190222747341749709448777139186859130859375f,-1.302000033512711762995195385883562266826629638671875f,-2.32190001313984328845663185347802937030792236328125f,1.123200023922323875780193702667020261287689208984375f,0.5056000390142207923105388545081950724124908447265625f,0.40010000576972970520728267729282379150390625f,-0.13750002760440127591579084764816798269748687744140625f,0.65799999520927665574987486252211965620517730712890625f,-0.6102000009596346696838509160443209111690521240234375f,0.759700011822582066400855183019302785396575927734375f,0.24790002427101143211984890513122081756591796875f,-0.284500021293759619567254048888571560382843017578125f,-0.07969999908208880157189923920668661594390869140625f,-0.5039999904334544478246016296907328069210052490234375f,0.92830004094243090406735063879750669002532958984375f,-1.154900019860267246230023374664597213268280029296875f,0.900900038516521561859917710535228252410888671875f,-0.87030002572834508356436344911344349384307861328125f,-0.5787000382423403976872577914036810398101806640625f,0.8245999774426220341183579876087605953216552734375f,-2.4625000035762791839033525320701301097869873046875f,-0.39150001747906149063283010036684572696685791015625f,-0.679199981856346113318068091757595539093017578125f,-0.23439998516291382646414831469883210957050323486328125f,0.2874000262677676165168350053136236965656280517578125f,-0.500400009109079224600691304658539593219757080078125f,-0.36979999786689898844116441978258080780506134033203125f,0.9314999789595592716295868740417063236236572265625f,-0.18480000483319181103780692865257151424884796142578125f,0.054600011193752351346120121888816356658935546875f,0.343700020919740778513329360066563822329044342041015625f,-0.47740003401637043367600199417211115360260009765625f,0.65980003212094306963564349644002504646778106689453125f,-1.1483000387072568315716125653125345706939697265625f,-0.3636000243701043022070962251746095716953277587890625f,-0.41420004384517739026705385185778141021728515625f,-1.5171000127024942827347331331111490726470947265625f,0.8486999410867692716919918893836438655853271484375f,-0.7476999794460834625198231151443906128406524658203125f,1.221900016045569969946882338263094425201416015625f,0.11540000557005403170052204586681909859180450439453125f,-0.1813999962493770823357408517040312290191650390625f,0.475799985882640275036692401045002043247222900390625f,1.5034000127002602464898473044740967452526092529296875f,-1.0196999749422073211491124311578460037708282470703125f,-0.0486999984189873114104329943074844777584075927734375f,-0.2621000046372412572992516288650222122669219970703125f,-0.05590000133812356164497714416938833892345428466796875f,0.145300005313754088120958840590901672840118408203125f,-0.4352999891459938819338049142970703542232513427734375f,0.724800036963821003865859893267042934894561767578125f,1.166099995182454218678458346403203904628753662109375f,0.679900022467970988060415038489736616611480712890625f,0.9034999585673215705128313857130706310272216796875f,-0.007000020027160491764561811578460037708282470703125f,-0.439700012977420673365713810198940336704254150390625f,-1.2163999854087830243543066899292171001434326171875f,0.239999997988343150012724436237476766109466552734375f,-0.5263999944984909529210881373728625476360321044921875f,0.4445999898105856829033655230887234210968017578125f,-0.50580000264644553453763364814221858978271484375f,-0.348499992132186520166214904747903347015380859375f,0.718300005403161634376374422572553157806396484375f,-1.1294999847859139574524078852846287190914154052734375f,-0.347000036045908399895409957025549374520778656005859375f,-0.720299967539310248554329518810845911502838134765625f,-0.316000002041458838419885069015435874462127685546875f,0.206400000607967371024642488919198513031005859375f,-0.425000002980232327587373220012523233890533447265625f,-0.04170001553297009166954012471251189708709716796875f,0.62780001529753182154536261805333197116851806640625f,-0.1168000228673218288122370722703635692596435546875f,0.16479999360144148567997035570442676544189453125f,0.1157000162482262339835870079696178436279296875f,-0.26709999405741680078563149436376988887786865234375f,0.41530003483668043173793194000609219074249267578125f,-0.75390003279447537209989604889415204524993896484375f,0.234100025856494919906936047482304275035858154296875f,-1.10850001567602163277115323580801486968994140625f,-0.5738999875880772538749852174078114330768585205078125f,-0.1698000636368994786806752017582766711711883544921875f,-0.483999974489212003714300180945429019629955291748046875f,0.2624999959021803164205266511999070644378662109375f,-0.0624999937415122719386317839962430298328399658203125f,-0.254100023025274790455796392052434384822845458984375f,0.54530000043362336636931786415516398847103118896484375f,-1.291099986204505167819434063858352601528167724609375f,0.21969998846501137546027848657104186713695526123046875f,-0.38550001513957976584379139239899814128875732421875f,-0.131399995746463549295413031359203159809112548828125f,0.520900020933151353119683335535228252410888671875f,-0.022299996656179266185660026167170144617557525634765625f,0.56259997615367252077334114801487885415554046630859375f,0.1472000361859799877350951646803878247737884521484375f,1.02580002008080573006054692086763679981231689453125f,0.588300007191300711184567262534983456134796142578125f,1.673499971829355725816412814310751855373382568359375f,0.2046000084370376281839298826525919139385223388671875f,0.37539998143017327958403939192066900432109832763671875f,-0.4735999882906669622428807997494004666805267333984375f,0.861899991831183598378629540093243122100830078125f,-0.19309997657835487050448364243493415415287017822265625f,-0.5487999742746356712785882336902432143688201904296875f,0.922699995148181795201480781543068587779998779296875f,0.1578999876886604258885427043423987925052642822265625f,0.6354999963641165816596867443877272307872772216796875f,0.16949999666959048028047618572600185871124267578125f,-0.608100005114078356882600928656756877899169921875f,-0.76000003397464777066261376603506505489349365234375f,-0.0095000025182965242009913708898238837718963623046875f,-0.77430001102089818232343532145023345947265625f,-1.0122000203907492998922634797054342925548553466796875f,0.07040001041293209027571720071136951446533203125f,-1.686200013804435737796438843361102044582366943359375f,-0.0630000497102738421517642564140260219573974609375f,1.4017999967157834806386063064564950764179229736328125f,-1.3373999808564784874675979153835214674472808837890625f,-0.9772000158086411314428687546751461923122406005859375f,-0.53139999796301129464382029254920780658721923828125f,0.2097999999344357657804494010633789002895355224609375f,-0.5996999773263933519729107501916587352752685546875f,0.34290001095309896594898191324318759143352508544921875f,-0.3653999590225509574992202033172361552715301513671875f,-0.331300004393607017139089521151618100702762603759765625f,0.525100021695345642314123324467800557613372802734375f,0.42470000772476179662362483213655650615692138671875f,-1.309100056571514070213879676884971559047698974609375f,1.56100004160404193243039117078296840190887451171875f,-0.2350999910175797236178141247364692389965057373046875f,-0.0944999791979785896955945645458996295928955078125f,0.36129999267756895964254226782941259443759918212890625f,-1.2757999847650527414089083322323858737945556640625f,-0.6494000239990647127541478766943328082561492919921875f,-0.103300027456878495968339848332107067108154296875f,-0.34659994401186622337007747773895971477031707763671875f,0.627099975281953714301153013366274535655975341796875f,-0.602399993038177594684157156734727323055267333984375f,0.34319997392892798160346501390449702739715576171875f,-0.0145999694891270070939981451374478638172149658203125f,-0.414200015439838142849993118943530134856700897216796875f,0.3326000127919017668176593360840342938899993896484375f,-0.26220002065151970516154733559233136475086212158203125f,0.03739999713599662101159992744214832782745361328125f,1.474099991397558984118631997262127697467803955078125f,-1.229400039365888375186841585673391819000244140625f,-0.33390005604028727415055755045614205300807952880859375f,-0.48410000290870680572652418049983680248260498046875f,-0.0042000295400625997643828668515197932720184326171875f,1.140300022077560537780982485855929553508758544921875f,1.2134000157177446954648303290014155209064483642578125f,0.51069993306249406028740622787154279649257659912109375f,-0.57930005818307250819998444058001041412353515625f,-0.69439994887858613736852930742315948009490966796875f,1.80770000095963556674405481317080557346343994140625f,-0.37859998851418463772233735653571784496307373046875f,-0.2444999966882159814218766769045032560825347900390625f,-0.01860001066103589284495001265895552933216094970703125f,-0.367700014426559196323296418995596468448638916015625f,0.443900027634948901589240222165244631469249725341796875f,0.8056000271677969948314057546667754650115966796875f,-1.262199982914328000305204113828949630260467529296875f,-0.0939000056743619637700248858891427516937255859375f,0.2359000113964080380668519865139387547969818115234375f,0.0695999914124614260657608610927127301692962646484375f,0.25240000231415005149671060280525125563144683837890625f,0.227899989290536009178822496323846280574798583984375f,0.07639996749013688859264448183239437639713287353515625f,-0.7181000060640283511048664877307601273059844970703125f,1.2393000576429076264872719548293389379978179931640625f,-0.21999999247491353049355211624060757458209991455078125f,0.162000006839633226007890698383562266826629638671875f,0.3789000162541871219445965834893286228179931640625f,0.3107000141248106306335330373258329927921295166015625f,1.682600019460171569818385250982828438282012939453125f,0.33480003593936624728399920059018768370151519775390625f,1.223100026199221534994876492419280111789703369140625f,0.1484999793171883908371455618180334568023681640625f,0.13370001695230604443764832467422820627689361572265625f,-0.7257000067114827146497191279195249080657958984375f,0.87870000436082429384754277634783647954463958740234375f,-1.458499948993325023849365607020445168018341064453125f,-0.06320001613646686333680690950131975114345550537109375f,0.20069998629689234093120830948464572429656982421875f,0.04680004043206598129955864351359196007251739501953125f,-0.458400004187971443769811230595223605632781982421875f,0.08680000338405380322370774592855013906955718994140625f,-0.0787000294320285842530893205548636615276336669921875f,0.26730001161321970126749647533870302140712738037109375f,0.3535000557601453152756221243180334568023681640625f,1.459300045070052842532959402888081967830657958984375f,-1.1817999937169247903767654861439950764179229736328125f,0.8114000089153645678408111052704043686389923095703125f,-0.7001000338770453357284395679016597568988800048828125f,-0.452399985215067967914137625484727323055267333984375f,-0.4757000384137028969888660867582075297832489013671875f,0.7324000200092790624495364681934006512165069580078125f,-0.6966999515324834302276713060564361512660980224609375f,0.87759996974617238141291863939841277897357940673828125f,1.4637999731570483508136248929076828062534332275390625f,-0.0254000047877429580012176302261650562286376953125f,-0.1360000196993349508289838922792114317417144775390625f,-0.098299976122378673881740951401297934353351593017578125f,0.36510001729950403159818961285054683685302734375f,0.420899995303153584558231159462593495845794677734375f,-0.7586999859765171105863146294723264873027801513671875f,-0.5732000118523836729167442172183655202388763427734375f,-0.852099987240135359201076425961218774318695068359375f,0.300900013557076795311928663068101741373538970947265625f,0.35890003117397473175742561579681932926177978515625f,0.7229000217378140913382367216399870812892913818359375f,-0.2074000247031453358914632190135307610034942626953125f,0.55229999663755269256881774708745069801807403564453125f,-0.918900045385957486843153674271889030933380126953125f,0.35780001026838992306267073217895813286304473876953125f,0.03949999393150205317937206928036175668239593505859375f,-0.03339999761283396484401464476832188665866851806640625f,-1.2437000225961210020386715768836438655853271484375f,-0.26190003929138239158191936439834535121917724609375f,-2.0555000127553935129753881483338773250579833984375f,0.065599979335070113251049406244419515132904052734375f,0.11870005850791987001002780743874609470367431640625f,-0.55600000679120398228150179420481435954570770263671875f,0.169600000558048302767844006666564382612705230712890625f,0.8371000086791813288300545536912977695465087890625f,0.0803000471673913640557884718873538076877593994140625f,-0.346300013092160074990033535868860781192779541015625f,0.8563000049710272065794924856163561344146728515625f,-0.1215999924033879242557532052160240709781646728515625f,-0.37540003047362013877119579774443991482257843017578125f,0.21970001659095295121204571842099539935588836669921875f,-0.35280000823810675480984855312271974980831146240234375f,0.71100001774355814365691230705124326050281524658203125f,-1.068600049329549772636482884990982711315155029296875f,0.96269999961852992242938853451050817966461181640625f,-1.6484000244349237274121833252138458192348480224609375f,1.84380003626346677947367425076663494110107421875f,-0.26979996387362448206914677939494140446186065673828125f,0.5458999994754787365991433034650981426239013671875f,0.26380000573471173996864536093198694288730621337890625f,-0.5366000300474469408840150208561681210994720458984375f,0.292400012502819495185946152560063637793064117431640625f,-0.23790000706017011555815088286180980503559112548828125f,0.23180000884160401053435407447977922856807708740234375f,0.4483999848723403403027987224049866199493408203125f,0.070700006562471440219042051467113196849822998046875f,0.53350002236291704793558210440096445381641387939453125f,0.076999980073422069626332131520030088722705841064453125f,0.5921000062763692728395881204050965607166290283203125f,-0.9724000196553770347662748463335447013378143310546875f,0.448699983815848246759827588903135620057582855224609375f,0.1068999859452244294288902892731130123138427734375f,0.242999941080808579130234647891484200954437255859375f,-1.083000046953559003526379456161521375179290771484375f,0.0103999868690973773510677347076125442981719970703125f,-1.149399989596008975212271252530626952648162841796875f,-0.43600001535937205066062460900866426527500152587890625f,0.8104000310324135369910436565987765789031982421875f,-0.751999969679861823834698952850885689258575439453125f,0.284699973284453233812740791108808480203151702880859375f,-0.4025000222399832683350950901512987911701202392578125f,-0.28369995419979066042515114531852304935455322265625f,-0.0803999920845033244631849811412394046783447265625f,0.6780000404715540440747645334340631961822509765625f,-1.42450001287460281673702411353588104248046875f,-0.617700024801492997283958175103180110454559326171875f,-0.1357999820828439396791509352624416351318359375f,0.4800000271201134705023605420137755572795867919921875f,-0.04440002081394123223390124621801078319549560546875f,0.641900006359815034073790229740552604198455810546875f,-0.81639998794198032072699788841418921947479248046875f,0.05380004162788498689451444079168140888214111328125f,-0.3471000427842145796830664039589464664459228515625f,0.6613000274717812931868365922127850353717803955078125f,-0.67590004000663750360899939551018178462982177734375f,0.953200021016597620615584673942066729068756103515625f,-1.2831000180780893771270712022669613361358642578125f,1.5733000565886499089174321852624416351318359375f,-1.446400028622150557566783390939235687255859375f,0.981300036561489008590797311626374721527099609375f,-0.0427000386595732805972147616557776927947998046875f,-0.904500031128526504886622205958701670169830322265625f,-0.11389997855424827122305941884405910968780517578125f,-1.1160999913454059395689910161308944225311279296875f,0.8596999949097623439087101360200904309749603271484375f,0.4699999751895658217648588106385432183742523193359375f,0.695599994683266498185503223794512450695037841796875f,0.89900003364682223860882004373706877231597900390625f,1.65690005791783345756584822083823382854461669921875f,1.063100064495206442671815239009447395801544189453125f,1.06380000959038678587376125506125390529632568359375f,-1.2598000007539980060755624435842037200927734375f,0.663600044244528675108085735701024532318115234375f,-1.176399998149275205605590599589049816131591796875f,0.43510000964403161560767330229282379150390625f,-0.4153000000238418198961198868346400558948516845703125f,0.5256999959826469392965009319595992565155029296875f,0.1281000063806774225128037869581021368503570556640625f,1.16389998540878281119148596189916133880615234375f,-0.0871999922275539518512488257329096086323261260986328125f,0.5631999960571525409847026821807958185672760009765625f,0.68209999230653040047656077149440534412860870361328125f,0.6058000129282470336278265676810406148433685302734375f,-0.110999980509280948837158575770445168018341064453125f,0.473699976253509902335281367413699626922607421875f,-0.205300002184510166358677452080883085727691650390625f,-1.382199981796741639072934049181640148162841796875f,-0.84489999039322116747285917881526984274387359619140625f,-1.3244999919757247663909538459847681224346160888671875f,0.5591000233232976679431658340035937726497650146484375f,-0.38369993996918250189764876267872750759124755859375f,0.008599992090463359772911644540727138519287109375f,0.81360002114772811410148278810083866119384765625f,0.1688000049710272065794924856163561344146728515625f,-0.710500032275915405222121989936567842960357666015625f,1.02750003799796107983866022550500929355621337890625f,-0.8436000325098633734199893297045491635799407958984375f,1.3415000250041486662411216457257978618144989013671875f,-0.03410000249892508605853436165489256381988525390625f,0.12530004252940418751904871896840631961822509765625f,-1.4483000582277778534034951007924973964691162109375f,1.029300034713745759518133127130568027496337890625f,1.047600030654668668006479492760263383388519287109375f,-0.49080003870725619208315038122236728668212890625f,0.323500018060208116565945601905696094036102294921875f,0.7557999662131067442061294059385545551776885986328125f,1.3983000057011831795961143143358640372753143310546875f,-1.9826999954089516986499575068592093884944915771484375f,-0.778400035834312831184433889575302600860595703125f,0.035699984955787300577867426909506320953369140625f,0.0742999999940394673103583045303821563720703125f,-0.996800007146597000229348850552923977375030517578125f,0.2020000023692845037004417463322170078754425048828125f,-0.4752000070691113631227153746294789016246795654296875f,0.31079999892115584003704498172737658023834228515625f,-0.98320002623200419833437990746460855007171630859375f,0.69689998736083413888309223693795502185821533203125f,0.11699998679757062092221531202085316181182861328125f,-0.356099978381395398940867380588315427303314208984375f,1.164600029224157395901784184388816356658935546875f,1.086000012993812990913511384860612452030181884765625f,1.5415999877840278475815694037009961903095245361328125f,0.31009998811185379707922038505785167217254638671875f,-0.7842000732749700997459285645163618028163909912109375f,-1.200200025956332350318689350388012826442718505859375f,-0.057699971902369640019969665445387363433837890625f,-0.05589999075829954477967476123012602329254150390625f,-0.34460000672340385108327609486877918243408203125f,-1.332599989750981261948936662520281970500946044921875f,0.6753999458909039077525449101813137531280517578125f,0.41589998282343165758589975666836835443973541259765625f,-0.6049999624118214125445547324488870799541473388671875f,-0.8636999826610087804823479018523357808589935302734375f,-1.06289999692738046377371574635617434978485107421875f,0.01509999794661986971533451651339419186115264892578125f,-0.14170000919997693955565409851260483264923095703125f,-0.8558000341251494891281481613987125456333160400390625f,0.300499986007809471999507877626456320285797119140625f,1.457499990463256178685469421907328069210052490234375f,-0.837600002789496755895015667192637920379638671875f,-0.49789998791217815465870444313623011112213134765625f,0.7749999755620962105240323580801486968994140625f,0.40150001278519642511355414171703159809112548828125f,1.13140001640319809439461096189916133880615234375f,-0.38740000317096701110131107270717620849609375f,0.568700030940771927845389654976315796375274658203125f,0.0667999572277064856962169869802892208099365234375f,-0.648300022464990632897752220742404460906982421875f,0.6239000126034019988452428151504136621952056884765625f,1.1547000155478712724033130143652670085430145263671875f,0.2305999690905216059633175973431207239627838134765625f,0.22350000844895878682194734210497699677944183349609375f,0.38710000873506000829138429253362119197845458984375f,0.179100001977384071238930118852294981479644775390625f,0.7536000176087023216808802317245863378047943115234375f,-0.58109998747110314099018069100566208362579345703125f,-0.3671999871611593135867224191315472126007080078125f,1.01870006953477965083720846450887620449066162109375f,-1.04550005328655259972947533242404460906982421875f,-1.565700081515313257796151447109878063201904296875f,1.0499000413179402357854996807873249053955078125f,1.41290003082752235741281765513122081756591796875f,0.658100010776519805943962637684307992458343505859375f,-1.222600007632374197186209130450151860713958740234375f,-0.1595999789327382512027497796225361526012420654296875f,0.6846000027745962057679207646287977695465087890625f,1.1690999916583300066719175447360612452030181884765625f,0.51629998749941641289495919409091584384441375732421875f,-0.3695000037476414700421400993946008384227752685546875f,-0.01950000963360043471084281918592751026153564453125f,-0.29749999117106196155901898237061686813831329345703125f,-0.87360005765557346979477415516157634556293487548828125f,-0.5688000070571899158267115126363933086395263671875f,0.923599998944997224725739215500652790069580078125f,0.21259996821880289274986353120766580104827880859375f,-0.656900018876791502719925119890831410884857177734375f,-0.7649000100255012313255065237171947956085205078125f,0.2624000086843967238792174612171947956085205078125f,0.1665999922171235159140678661060519516468048095703125f,-0.7516999876677987746376174982287921011447906494140625f,0.27590002085864551162330826628021895885467529296875f,0.61949999700486646503350129933096468448638916015625f,-0.431499957203865136534659541212022304534912109375f,0.35629999111294730074206427161698229610919952392578125f,-0.41440000233650220451409040833823382854461669921875f,0.82360003664493586228445565211586654186248779296875f,0.27700000095367460772877166164107620716094970703125f,0.2438000158488751178964548671501688659191131591796875f,0.6945000104904170967756726895458996295928955078125f,-0.7832000333100561650923054912709631025791168212890625f,-0.321100006842613083790638484060764312744140625f,-0.685300022822618704054775662370957434177398681640625f,-1.9754999620914457381104512023739516735076904296875f,-0.11219998507499706619228163617663085460662841796875f,-0.884999986886978007305515347979962825775146484375f,0.0506000078707933287347486839280463755130767822265625f,0.4537000073224308760444500876474194228649139404296875f,0.5637999659672383589992250563227571547031402587890625f,-0.5177000440239911682738238596357405185699462890625f,0.53790003142356823673253529705107212066650390625f,0.4469000206649302864292394588119350373744964599609375f,-0.86779999853372569162957006483338773250579833984375f,-0.42999999001622146721501849242486059665679931640625f,-0.906000037580728445618660771287977695465087890625f,-0.323700033622980942737967779976315796375274658203125f,-1.0593000162363050975500300410203635692596435546875f,-0.089799997103213957672096512396819889545440673828125f,-0.3568000373959545612478905241005122661590576171875f,1.47320007212758152803644406958483159542083740234375f,-0.38190006760358841120250872336328029632568359375f,0.94220000355243627154777641408145427703857421875f,-0.200699983763694689287149230949580669403076171875f,-0.787300020647049070277034843456931412220001220703125f,-0.6370000360459096011567226014449261128902435302734375f,-0.85930006075352405314760062537970952689647674560546875f,-0.79050000426173117062944584176875650882720947265625f,0.2645000043064360539801782579161226749420166015625f,0.181200006353855247454021082376129925251007080078125f,0.10530000501573066884475338156335055828094482421875f,0.1050999804377550361778048682026565074920654296875f,0.40310003655552895107661015572375617921352386474609375f,0.232200009065866908031239290721714496612548828125f,-0.574699981647729174483174574561417102813720703125f,0.8802000575840480678380117751657962799072265625f,-1.1990000144243235791208235241356305778026580810546875f,1.369100013190507780791449476964771747589111328125f,0.6346999706953762032668464598827995359897613525390625f,0.2441999586105350061870922218076884746551513671875f,0.007499991357326596386201345012523233890533447265625f,0.1924999989569187164306640625f,0.09389998920857911546278273817733861505985260009765625f,0.30060002910494798555873785517178475856781005859375f,-0.52820000127255894994959817267954349517822265625f,-9.995510578164612525142729282379150390625e-05f,-0.11420000972151722606895418721251189708709716796875f,-1.019599933186173590371481623151339590549468994140625f,-0.16959999815523596478072931859060190618038177490234375f,-0.77939995360970559801216950290836393833160400390625f,0.5455999880522488520995239014155231416225433349609375f,-0.352199978369474475670131141669116914272308349609375f,-0.229300003495812365628125917282886803150177001953125f,0.380599987679719919952958662179298698902130126953125f,-0.29230001207888112180199868816998787224292755126953125f,0.33169999571442598806925161625258624553680419921875f,0.207900005048513580874214312643744051456451416015625f,-0.133599987769127181991279940120875835418701171875f,-0.262800002855062331263980013318359851837158203125f,-0.03389999129474130512562624062411487102508544921875f,-0.706400003662705611162664354196749627590179443359375f,0.45019996773004411494412124739028513431549072265625f,-0.8928999929785721434427614440210163593292236328125f,0.64350001582503335573193226082366891205310821533203125f,-0.649100032860041142868112729047425091266632080078125f,0.9054999839961517249520284167374484241008758544921875f,-0.778399987629055711835235342732630670070648193359375f,0.12990001069605361383452191148535348474979400634765625f,-1.096000000625849235547093485365621745586395263671875f,0.5037000073224302543195562975597567856311798095703125f,-0.97410000715553746175601190770976245403289794921875f,-0.32079999005496429020922732888720929622650146484375f,0.635999982073902625501204965985380113124847412109375f,-0.04210001894235659847254282794892787933349609375f,0.2458000279784204877842057612724602222442626953125f,-0.55710001454651347785329562611877918243408203125f,-0.6346000211030247140087112711626105010509490966796875f,-1.3430000178217891271259532004478387534618377685546875f,-0.46620001633763319404124558786861598491668701171875f,-2.16710001715421629597813080181367695331573486328125f,0.00660001140236865513344355349545367062091827392578125f,-0.481199991080164313217437666025944054126739501953125f,0.062999986678362152048293864936567842960357666015625f,-0.76790000489950127615657038404606282711029052734375f,-0.190899987405538151818973346962593495845794677734375f,-0.44979997735917554013695962566998787224292755126953125f,-0.0187999785214660874999026418663561344146728515625f,-0.8631999858498569100362374229007400572299957275390625f,-0.1022999733358622709289420527056790888309478759765625f,0.114899998402595571178608224727213382720947265625f,-0.36030001671314249023225784185342490673065185546875f,0.0898999981045722762473815237171947956085205078125f,-0.376700009572505933874708716757595539093017578125f,0.28199999640882023044241577736102044582366943359375f,-1.52240001628398857747015426866710186004638671875f,0.78260001016557279029939309111796319484710693359375f,-0.69880000377893480845159501768648624420166015625f,-1.06720004318952543798104670713655650615692138671875f,0.5244999866485589024023283855058252811431884765625f,0.0761999927192926751473578406148590147495269775390625f,-1.114399967393278956251378986053168773651123046875f,-0.802499984093010265695511407102458178997039794921875f,-0.255399976810812834315811414853669703006744384765625f,-0.04050002177059652819224311315338127315044403076171875f,-0.83899993425607721775350000825710594654083251953125f,0.573600032714754792806388650205917656421661376953125f,-0.534899967873841308829696572502143681049346923828125f,-0.158300001212209273671049913900787942111492156982421875f,0.63860000881701639574572482160874642431735992431640625f,0.54200001227855665320021216757595539093017578125f,0.26410001706481001093607119400985538959503173828125f,-0.219299980771541669355428894050419330596923828125f,0.922300059986115083887625587522052228450775146484375f,-2.10200000765919714496021697414107620716094970703125f,0.027699978309868811265914700925350189208984375f,-0.677099961721896903554807067848742008209228515625f,0.65130004214942494034090714194462634623050689697265625f,-0.45270004767477445284384884871542453765869140625f,0.718500009827315988530926915700547397136688232421875f,0.1120999861970541910949350494774989783763885498046875f,0.1553999569177613171433449679170735180377960205078125f,-0.25489995936155285249924418167211115360260009765625f,1.03060003374293440714382086298428475856781005859375f,-1.3909000293895601441107601203839294612407684326171875f,-0.68210004816725933185495023280964232981204986572265625f,0.48280001797974103627808517558150924742221832275390625f,-0.125500010371208503556772484444081783294677734375f,0.652600023502111437068151644780300557613372802734375f,0.5166000517100097777500877782586030662059783935546875f,0.424700003999471054072500919573940336704254150390625f,-0.0079999290853738358464397606439888477325439453125f,0.6380000489652157824593814439140260219573974609375f,-2.10299998913705366021531517617404460906982421875f,0.4248000044420352505625260164379142224788665771484375f,0.81309998205453126729480572976171970367431640625f,0.11749994058161961074659984660684131085872650146484375f,-1.1961000409662723154013974635745398700237274169921875f,-0.15230000514984087800485212937928736209869384765625f,-0.718800008696317771494932458153925836086273193359375f,-0.18300003186613356120915341307409107685089111328125f,0.24060003324002077196297477712505497038364410400390625f,0.25060001871138848361653117535752244293689727783203125f,0.125700010455399968645195940553094260394573211669921875f,-0.148099981570243954820398357696831226348876953125f,-0.5885000074803825587110850392491556704044342041015625f,0.2165000042915341538218854111619293689727783203125f,-0.1531999950885769568031946619157679378986358642578125f,0.67739999579489218461958444095216691493988037109375f,-0.2055000003501767924518617292051203548908233642578125f,0.167900004377961042933975477353669703006744384765625f,-0.237100029987841909306922616451629437506198883056640625f,-0.37049994869902802552275034031481482088565826416015625f,-0.071599999220669285904250500607304275035858154296875f,0.143100026609003894151328495354391634464263916015625f,0.097600022533536279212285080575384199619293212890625f,0.37679999567270261451312762801535427570343017578125f,0.7797000223465266888211999685154296457767486572265625f,1.54050000793114261199434622540138661861419677734375f,0.25810000470429661501015061730868183076381683349609375f,1.182400015799700998542220986564643681049346923828125f,-0.738799984407425114341094740666449069976806640625f,0.184399989372491024397504588705487549304962158203125f,-0.6876999722003933612768378225155174732208251953125f,0.56479998957812693749502841455978341400623321533203125f,-0.62609997636973790946512963273562490940093994140625f,-0.949800017480551428405988190206699073314666748046875f,0.9595999859735366843693782357149757444858551025390625f,1.2939000184148543581841295235790312290191650390625f,-0.92389998514801252582628876552917063236236572265625f,-0.34390001748353260513368923056987114250659942626953125f,0.89900001097843063035952582140453159809112548828125f,0.018299968075751404938955602119676768779754638671875f,0.5399000471293931013860856182873249053955078125f,-0.66480001341998551378509318965370766818523406982421875f,0.01360000613480816245015603271895088255405426025390625f,-1.503900074182451529480886165401898324489593505859375f,0.89900002554431568402293351027765311300754547119140625f,0.63670004146099046238305163569748401641845703125f,0.8275000028312202715596868074499070644378662109375f,-0.88529996537864263927986030466854572296142578125f,-0.83780000170022272953929132199846208095550537109375f,-0.9670999923065302095182005359674803912639617919921875f,-0.36239998234659476228358698790543712675571441650390625f,-0.5200000008195633238727850766736082732677459716796875f,-0.65260002011209705674588121837587095797061920166015625f,-1.212700051325560135495607028133235871791839599609375f,-0.28540001424998051593462378150434233248233795166015625f,0.6519000001944597055825170173193328082561492919921875f,-0.712900034701824392158187038148753345012664794921875f,0.01860000416040430426534157959395088255405426025390625f,-1.6736000394389038969933380940346978604793548583984375f,-0.2773000137180090529653853081981651484966278076171875f,-0.51409999147206508762764087805408053100109100341796875f,0.210500009533018006546711831106222234666347503662109375f,0.065799997356533967973746257484890520572662353515625f,-0.60770003657341054026375104513135738670825958251953125f,-0.42460000661164498669819522547186352312564849853515625f,0.676400008933990992687768084579147398471832275390625f,-0.59490000458657676940532610387890599668025970458984375f,-0.8341000294327738373567626695148646831512451171875f,-0.89480001558065414002385296043939888477325439453125f,-1.0317999888926741025585442912415601313114166259765625f,-1.7576000135183331707366960472427308559417724609375f,1.028000031903386624065888099721632897853851318359375f,-0.72459995963573486932318701292388141155242919921875f,-0.470500013668090133478472125716507434844970703125f,0.5300999995298678602040354235214181244373321533203125f,0.356900046313554375476684299428598023951053619384765625f,0.21950000195950292170010698100668378174304962158203125f,0.012899998939037260470286128111183643341064453125f,0.6765999903917307989331675344146788120269775390625f,0.083600006395578407847324342583306133747100830078125f,0.36700001274421811814363536541350185871124267578125f,0.48590000878870498457473559028585441410541534423828125f,0.21350003894046043484422625624574720859527587890625f,-0.306900006229430477588948633638210594654083251953125f,0.0498999919950957604442010051570832729339599609375f,0.72160001691579811478050032746978104114532470703125f,1.317100053066015963310064762481488287448883056640625f,0.2935999710798267159361785161308944225311279296875f,0.7575000092387196293230999799561686813831329345703125f,-1.18160000082254423858785230549983680248260498046875f,-0.41990002930387915203169768574298359453678131103515625f,0.21300000198930479911041402374394237995147705078125f,0.09850000105425726903973782100365497171878814697265625f,0.09889996026307368293117860957863740622997283935546875f,-0.963300038707256334191697533242404460906982421875f,1.028000011265277180427801795303821563720703125f,-0.72400002545118358199260910623706877231597900390625f,0.01860001585781601018965147886774502694606781005859375f,-0.3308000455245381221658362846937961876392364501953125f,-0.4771000313848261509264148116926662623882293701171875f,0.328099999488889848553441197509528137743473052978515625f,1.59190005061626482785186453838832676410675048828125f,0.70870001842379526380000243079848587512969970703125f,1.26029998095035455918377920170314610004425048828125f,-0.025300014626979727694333632825873792171478271484375f,-0.511800036799907598350500848027877509593963623046875f,0.3687000287801034392742849377100355923175811767578125f,0.63080003457218458695621166043565608561038970947265625f,-0.13209999439269293208809585848939605057239532470703125f,-0.18170001974254879062442569193080998957157135009765625f,1.05050001482293087207153803319670259952545166015625f,-0.0515999832391731416691982303746044635772705078125f,0.3096999964743856192939119864604435861110687255859375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.081000001072883520691902958787977695465087890625f,-1.200899956448376659778887187712825834751129150390625f,0.301099979387223992688404905493371188640594482421875f,0.448400033748149962065099316532723605632781982421875f,1.1219000182993710268419818021357059478759765625f,0.4393000373959541349222490680404007434844970703125f,0.4438000038906932953608475145301781594753265380859375f,-0.205599967861175958461217305739410221576690673828125f,0.083399998246133311141647936892695724964141845703125f,0.1346999829888346766892937012016773223876953125f,-0.122999990999698993743294295200030319392681121826171875f,0.643899932695924714920465703471563756465911865234375f,-1.078400019256771091846758281462825834751129150390625f,-0.5248000576391824711919298351858742535114288330078125f,-1.365100048610568439499957094085402786731719970703125f,-0.881300020170211695358375436626374721527099609375f,0.333500007871538439729874880868010222911834716796875f,1.052999960303306892228647484444081783294677734375f,-1.123399959726631980316824410692788660526275634765625f,0.445799976569414280191949728759936988353729248046875f,0.98260002771168952762081971741281449794769287109375f,0.70159992665201487138659786069183610379695892333984375f,-0.1668000000312928055024030982167460024356842041015625f,-0.1597000065326688034161861651227809488773345947265625f,-1.286800002676248055877294973470270633697509765625f,-0.6901999579325324152279108602670021355152130126953125f,-0.5611000706195827802957865060307085514068603515625f,-0.65799998265504822114735361537896096706390380859375f,0.623599948839842266323785224813036620616912841796875f,-0.348899956575035918859839512151665985584259033203125f,0.06720002262592383868167189575615338981151580810546875f,0.18440002442747382982446424648514948785305023193359375f,0.5919000294566156838982351473532617092132568359375f,0.0047999902486797196843326673842966556549072265625f,0.47569999628067005659204369294457137584686279296875f,1.1059000163137906458388215469312854111194610595703125f,-0.9121000134289267702314418784226290881633758544921875f,0.1508000174731016773677083619986660778522491455078125f,1.162400033047795933072166008059866726398468017578125f,1.1997000267237425408239914759178645908832550048828125f,-0.42629998135268676318077041287324391305446624755859375f,0.005600003959238986084301359369419515132904052734375f,-0.0772000051543118015473510240553878247737884521484375f,0.53449994362145591964008417562581598758697509765625f,1.01020005236864118813855384360067546367645263671875f,0.564700025308132236290248329169116914272308349609375f,0.6412000343680386293954143184237182140350341796875f,0.2106999721780413725724656615057028830051422119140625f,-0.7208000359505419840644435680587776005268096923828125f,0.804800043557584832143447783892042934894561767578125f,-0.495400006762146905003874053363688290119171142578125f,-0.37250000268220873067548382095992565155029296875f,-2.03300002050399797326463158242404460906982421875f,0.147899956526606535245349505203193984925746917724609375f,0.03679995543956682269026714493520557880401611328125f,-0.9112000308662653313973578406148590147495269775390625f,-2.04960000433921862850183970294892787933349609375f,0.3084999744370577889895912448992021381855010986328125f,-0.398900032198429510543746800976805388927459716796875f,-0.15600000518932954918227551388554275035858154296875f,0.05629999185800471650509280152618885040283203125f,-1.0757000371284786144343570413184352219104766845703125f,1.332400028800964975062015582807362079620361328125f,0.8805000576078898522069948739954270422458648681640625f,-0.21119997334778251030229512252844870090484619140625f,-0.225500016685575399577601274359039962291717529296875f,-0.08369999383687964922273749834857881069183349609375f,0.452000018574297668383366044508875347673892974853515625f,1.30039998970031689395909779705107212066650390625f,-1.284500061377884083668732273508794605731964111328125f,-0.107700027334689085733998581417836248874664306640625f,0.5619000389933586081525618283194489777088165283203125f,1.628200002613664043593644237262196838855743408203125f,-0.5078999825105070531350293094874359667301177978515625f,0.1564999850690365068572873497032560408115386962890625f,0.43639999271035156969134050086722709238529205322265625f,-0.8319000146672130835412417582119815051555633544921875f,-0.613899999713897415176688809879124164581298828125f,-1.00449996548891018477434045053087174892425537109375f,1.6762000564217576226155870244838297367095947265625f,0.41859996541738464514992301701568067073822021484375f,-1.98900007201731288120072349556721746921539306640625f,0.21640003041029043373555396101437509059906005859375f,0.10339999746382222411966722575016319751739501953125f,0.8821000044137237150465580270974896848201751708984375f,-0.346400004780292647410533390939235687255859375f,-0.05929993223100904486955187167041003704071044921875f,0.6035000419020661865943111479282379150390625f,0.25009998297095326069694465331849642097949981689453125f,0.74619999298006278731776319546042941510677337646484375f,0.89499999191612078508484273697831667959690093994140625f,-0.06850002171099223158279301060247235000133514404296875f,1.681099992984532942585929049528203904628753662109375f,-0.37279994243085357030764726005145348608493804931640625f,0.871199998195468783279693525400944054126739501953125f,1.137900021886825374650697995093651115894317626953125f,0.019100052194298111629677805467508733272552490234375f,-1.234200028944015148368862355710007250308990478515625f,0.31460006178319421810130052108434028923511505126953125f,-0.061799986582994126393941769492812454700469970703125f,-0.40990000011622829134694256936199963092803955078125f,0.80129999290108688825284843915142118930816650390625f,-0.78910004403591216259883367456495761871337890625f,0.4691999699354170161313959397375583648681640625f,-0.0848999841906128427382327572559006512165069580078125f,-0.488400012923776660755947887082584202289581298828125f,-0.416100007364153867683853604830801486968994140625f,-0.5733000276431443609226334956474602222442626953125f,-0.41560001567527660171208481187932193279266357421875f,-1.3146999721482404144268230083980597555637359619140625f,0.1964999828152358407606925538857467472553253173828125f,0.937799962621927107875308138318359851837158203125f,0.1593999953702083249851284563192166388034820556640625f,-1.15520003165602691552749092807061970233917236328125f,-0.18150000660121445861250322195701301097869873046875f,-1.100300031167268599574526888318359851837158203125f,0.3493999952025708122249625375843606889247894287109375f,0.2077999785661699405636682058684527873992919921875f,-0.446299992733448558368536396301351487636566162109375f,-0.33339995834827362841679132543504238128662109375f,1.857100034737587890276699909009039402008056640625f,0.706899992799758791761632892303168773651123046875f,-0.502400010062754365236514786374755203723907470703125f,-0.747199961978197269019119630684144794940948486328125f,-0.86180000908374854162730116513557732105255126953125f,1.268000004820525550286447469261474907398223876953125f,-0.61859996586441889920848780093365348875522613525390625f,0.11769999771863204340860420415992848575115203857421875f,0.762599987441301419011097095790319144725799560546875f,0.83970001949667949503464114968664944171905517578125f,-1.323700009036063551803863447275944054126739501953125f,1.709800014537572021566802504821680486202239990234375f,0.8101999908268453509663231670856475830078125f,-0.25270001720190027327817006153054535388946533203125f,0.30590001005530353950234712101519107818603515625f,-0.716699965614080891640469417325221002101898193359375f,-0.36499999318271869608309998511685989797115325927734375f,-0.7344000133819872377927140405518002808094024658203125f,0.2235999795548615498574918092344887554645538330078125f,-0.41210003404840922502927469395217485725879669189453125f,0.66820002837404623985406715291901491582393646240234375f,0.3198000238135463302313610256533138453960418701171875f,-0.788100004369020279426649722154252231121063232421875f,-1.2739000564500695400482754848781041800975799560546875f,0.6562999695807689715110200268100015819072723388671875f,0.00190001053214039483663100327248685061931610107421875f,0.008600012430548975572719427873380482196807861328125f,0.5550000112876303237641195664764381945133209228515625f,0.17739998707771231778451692662201821804046630859375f,-0.64709997144490483123036028700880706310272216796875f,-0.56500002145767158623357317992486059665679931640625f,0.783499996416270771959489138680510222911834716796875f,-0.0897000124931341957790209562517702579498291015625f,0.105500003255903695986717139021493494510650634765625f,1.0150999834738672067402376342215575277805328369140625f,-0.2691999812230465582985061701037921011447906494140625f,0.4586000037878747992436956337769515812397003173828125f,-0.37270002319961814674087463572504930198192596435546875f,0.00080004779696540850864039384759962558746337890625f,0.92800005447864608587593693300732411444187164306640625f,1.60479999375045334630840443423949182033538818359375f,-0.88670001080185179009873763789073564112186431884765625f,-0.93809999289512635556320674368180334568023681640625f,-0.35310002290233943877950650858110748231410980224609375f,-0.990800005030632036095994408242404460906982421875f,0.974200008566677322363602797850035130977630615234375f,-1.139900020456313001204762258566915988922119140625f,-0.15729999501705138609253253889619372785091400146484375f,-1.313899972891807266250907559879124164581298828125f,0.49709999221339840058675463296822272241115570068359375f,-0.930999981552362410042178453295491635799407958984375f,0.8144999981410798728376221333746798336505889892578125f,-0.37209999968260565328392885930952616035938262939453125f,0.1426000079490246097435601768665947020053863525390625f,-0.219600000148265916966039412727695889770984649658203125f,0.105700013715029239591558507527224719524383544921875f,-1.3087000158905990332414148724637925624847412109375f,0.13929996221959495272813001065514981746673583984375f,0.43830000681877212542758570634759962558746337890625f,-0.42529996567666472628843621350824832916259765625f,1.7894999974519014696028307298547588288784027099609375f,0.8979000579476359433783727581612765789031982421875f,1.0971999847769726699198145070113241672515869140625f,-0.74099998617172158077437416068278253078460693359375f,1.2120999453306193771595644648186862468719482421875f,-0.03769999395608980563565637567080557346343994140625f,-0.09820001088380791998133645392954349517822265625f,0.46760000398159018430987998726777732372283935546875f,-0.0893000392585998614691789043718017637729644775390625f,-1.0244000120222571670325351078645326197147369384765625f,0.406299998153745789419843958967248909175395965576171875f,-0.02469996931701878839504615825717337429523468017578125f,-0.9142000151604412483408168554888106882572174072265625f,-0.9587000552296647271077745244838297367095947265625f,-1.2071000034451486726538860239088535308837890625f,-0.55390001580715164664070471189916133880615234375f,-0.44099995275586845178139583367737941443920135498046875f,0.739300011914969079640513882623054087162017822265625f,-0.319100011178851783455456825322471559047698974609375f,0.869100034499168838664218128542415797710418701171875f,0.31720003063529811360155008514993824064731597900390625f,-0.398500018022954705099181182959000580012798309326171875f,-0.31829999672323461279432876835926435887813568115234375f,-0.42260000955089915830598101820214651525020599365234375f,0.7461000330686573533967020921409130096435546875f,0.15330004884004555520959911518730223178863525390625f,0.17249998399987820363321588956750929355621337890625f,0.4815000545457006619898265853407792747020721435546875f,0.959100027793646336959909604047425091266632080078125f,0.3395000113844872657864470966160297393798828125f,0.67760000753924221061907928742584772408008575439453125f,-0.6439000390529638906400577980093657970428466796875f,-0.81390004387721470724414984942995943129062652587890625f,-0.483500045627355490296395146287977695465087890625f,-0.05280000725090505664383044859278015792369842529296875f,-0.299699981945753535228504915721714496612548828125f,-0.42510003415644159741049179501715116202831268310546875f,-0.4537999975204467517642115126363933086395263671875f,0.82620001780912277755675177104421891272068023681640625f,-0.1811000059485434743322684880695305764675140380859375f,0.069100000766664615436951635274454019963741302490234375f,-0.131000035047531326881653512828052043914794921875f,-1.2129000088483099517588925664313137531280517578125f,0.90749997489154399232802461483515799045562744140625f,0.63640005957931367053248550291755236685276031494140625f,0.6309999644905326388055755160166881978511810302734375f,-0.0583000238060957798325034673325717449188232421875f,0.2539999795556067141433231881819665431976318359375f,-0.7689000247478485761121191899292171001434326171875f,0.32799999696016346462101864744909107685089111328125f,1.42540002412199928727432052255608141422271728515625f,-0.15900005853176057968312306911684572696685791015625f,-0.0304999653697013428654827293939888477325439453125f,0.114200018364191802078266846365295350551605224609375f,0.28000002192333350858888252332690171897411346435546875f,0.47110000006258412330595319872372783720493316650390625f,-0.453100006082653716799057974640163592994213104248046875f,-0.06350000708922765024766476926743052899837493896484375f,0.481099999690055568635216332040727138519287109375f,-1.0153999907433985949722909936099313199520111083984375f,-1.089500015258789034078290569595992565155029296875f,-0.6697000215083359631051962423953227698802947998046875f,-0.218399995750188313792250482947565615177154541015625f,1.838599982404708566008366688038222491741180419921875f,0.457700030612945507613176232553087174892425537109375f,-0.20309997046887939831094627152197062969207763671875f,-0.42579999012947045144983349018730223178863525390625f,-1.2510999861299989532881227205507457256317138671875f,-0.92830004824400003116124935331754386425018310546875f,-1.150599994795024283433804157539270818233489990234375f,-0.565000026226043505772622665972448885440826416015625f,1.381800013628601941917395379277877509593963623046875f,0.778300044816732761177036081789992749691009521484375f,-0.63739999296367155778852975345216691493988037109375f,-0.79010002052187910948077842476777732372283935546875f,0.682100005531310671358369290828704833984375f,0.192300007534026207167698885314166545867919921875f,0.87319999454840957486823072031256742775440216064453125f,-0.0286000114619735512633269536308944225311279296875f,-0.7207000156335532370377450206433422863483428955078125f,-0.14459997110962863331451444537378847599029541015625f,0.55000000543892391124956020576064474880695343017578125f,1.2009000465631487486461992375552654266357421875f,0.85270002569556269111927804260631091892719268798828125f,0.05530002177953718955905060283839702606201171875f,0.85850000410899507397033403321984224021434783935546875f,-0.891300032687187382407500990666449069976806640625f,-0.63300003033876439673122149542905390262603759765625f,-0.0465999796628953077970436424948275089263916015625f,0.810400012759864285527555693988688290119171142578125f,0.8942000134468077732208257657475769519805908203125f,-0.07799999103695143265468914250959642231464385986328125f,0.40159998204708102775839506648480892181396484375f,-0.1422999965071678474259897484444081783294677734375f,-0.41280002356767653282076935283839702606201171875f,-1.119199991244077097007902921177446842193603515625f,0.041800001335143832648100215010344982147216796875f,-0.250299983036518458590080626890994608402252197265625f,-0.025100023502111046269646976725198328495025634765625f,0.6970000224113466202879862976260483264923095703125f,1.627299956274033121417232905514538288116455078125f,0.29179999895095765083397054695524275302886962890625f,0.435799982976913469201463158242404460906982421875f,-0.97570001848340037753359865746460855007171630859375f,0.0487000000953674572201634873636066913604736328125f,0.543900027579069078598195119411684572696685791015625f,-1.16250000551342935750653850845992565155029296875f,-0.508200025188922932528612363967113196849822998046875f,0.54850001612305643305944613530300557613372802734375f,-0.057500007897615557084236570517532527446746826171875f,-0.125700022022426416423712680625612847506999969482421875f,0.407900009071826996631671136128716170787811279296875f,1.17000000189989794563416580785997211933135986328125f,-0.749500004529953134380093615618534386157989501953125f,2.06119996720105458365424055955372750759124755859375f,1.39790005005002004878633670159615576267242431640625f,-0.186300010149181627383541126619093120098114013671875f,-1.301900026029348733658252967870794236660003662109375f,-0.106999978825449926223001284597557969391345977783203125f,-0.3837000011384485986809522728435695171356201171875f,-0.5919000117987394116170207780669443309307098388671875f,0.999000014722347007278813180164434015750885009765625f,-0.6992999738797542885748725893790833652019500732421875f,-1.10379999573230680454116736655123531818389892578125f,-0.58200001876056173255591374982031993567943572998046875f,0.80699999320507043165662253159098327159881591796875f,0.0486000038623810581839279620908200740814208984375f,-0.05870000084042548138540951185859739780426025390625f,-0.3081000116705894242841168306767940521240234375f,-1.65970000414848328773587127216160297393798828125f,-0.3966000269927085764010143975610844790935516357421875f,0.9163000297069547883666018606163561344146728515625f,0.6190000390298668886401856070733629167079925537109375f,0.2995000031888483960074154310859739780426025390625f,0.161299990125745029700965460506267845630645751953125f,-0.64320001684427285937317719799466431140899658203125f,-0.05320001092106119155999977010651491582393646240234375f,0.75479997132420439953648383379913866519927978515625f,0.212299984735249747114238516587647609412670135498046875f,-0.52630000437498125620550126768648624420166015625f,0.225300006077438386142830495373345911502838134765625f,0.9237999662280085289012276916764676570892333984375f,-0.61759999206066140686743892729282379150390625f,0.023399989044666114068604656495153903961181640625f,-0.157799983334541149559981931815855205059051513671875f,0.344199991542101013664023412275128066539764404296875f,-1.66750004395842577054054345353506505489349365234375f,-0.02679997928142530128070575301535427570343017578125f,0.956300074112415909866058427724055945873260498046875f,-0.05059998708367441366817729431204497814178466796875f,-0.9280000480711461108285220689140260219573974609375f,1.3627999758839610677796372328884899616241455078125f,0.019799994570016909989362829946912825107574462890625f,0.034299991947412511450465899542905390262603759765625f,1.491300031197070996569209455628879368305206298828125f,-0.137399990504980085148645230219699442386627197265625f,-0.7237000021815287453819109941832721233367919921875f,-1.1694999884366996667495186557061970233917236328125f,0.43119998817443860872344885137863457202911376953125f,-0.44359997614622148631724485312588512897491455078125f,-0.0610000144839286662090671597979962825775146484375f,0.324099973888694936619003783562220633029937744140625f,0.4749000121861695067337905129534192383289337158203125f,0.31760002990961078150888852178468368947505950927734375f,-0.4164000053763390329919502619304694235324859619140625f,-0.1767999800637360952038079631165601313114166259765625f,0.26310000854134596437461368623189628124237060546875f,0.25149995385110412993157069649896584451198577880859375f,1.387700011651217923969170442433096468448638916015625f,0.5235000185444944609258755008340813219547271728515625f,-0.800099983566999295447885742760263383388519287109375f,0.356499985069036851026424983501783572137355804443359375f,-0.0600000092014669750284383553662337362766265869140625f,0.39030001935809777169339440661133266985416412353515625f,-0.588499956965446102685746154747903347015380859375f,-1.304600047217309377600713560241274535655975341796875f,0.21289994589090266430275733000598847866058349609375f,-0.2254999993443487227295918273739516735076904296875f,-0.86730000075399882319970856769941747188568115234375f,-0.357900025220960482652543532822164706885814666748046875f,-0.610900025776028865465150374802760779857635498046875f,0.029599989363550738286079422323382459580898284912109375f,-0.02470002948045735280402368516661226749420166015625f,-0.3689999970644712856682190249557606875896453857421875f,0.878999985702335617787639421294443309307098388671875f,-0.62530000531375395933508798407274298369884490966796875f,-0.2848999729588632678911608309135772287845611572265625f,0.785899993012100228639837951050139963626861572265625f,-0.243599999243020892691902190563268959522247314453125f,-0.277300003585219112522963769151829183101654052734375f,0.2710000785589219418625361868180334568023681640625f,1.12150003403797793311014174832962453365325927734375f,1.1426999879956252215151835116557776927947998046875f,-0.685999992579221729016580866300500929355621337890625f,0.5289999780654905947585575631819665431976318359375f,-0.4839999928921461247455226839520037174224853515625f,-0.2987999890267847291625002981163561344146728515625f,0.0051999693691728854361144840368069708347320556640625f,-0.0132999890118836727737061664811335504055023193359375f,0.329099986368417862792057348997332155704498291015625f,-1.177799982589483374084693423355929553508758544921875f,0.208000025086104489702165665221400558948516845703125f,0.13050001432001590018217029864899814128875732421875f,-0.07429999325126412923481211691978387534618377685546875f,0.68959996026903447319256201808457262814044952392578125f,-0.92029999912977256126822567239287309348583221435546875f,0.2627999879911542269184110409696586430072784423828125f,0.90210003563165752638042249600403010845184326171875f,-0.64410002943277344655825800145976245403289794921875f,-0.20680005080699981334646508912555873394012451171875f,-0.47079996151924152769652209826745092868804931640625f,-0.23489999959468832457787357270717620849609375f,-0.5119999937266108691602539693121798336505889892578125f,0.6668999759614469002855230428394861519336700439453125f,0.5254000213280318742903318707249127328395843505859375f,1.181999980017542473120784052298404276371002197265625f,0.4381000041827558533213959890417754650115966796875f,0.429100011402368419766162332962267100811004638671875f,0.407399970947206480076374646159820258617401123046875f,1.0805999710276725789270813038456253707408905029296875f,0.901199971038102898290844677831046283245086669921875f,0.8430999988555907265208588796667754650115966796875f,-0.050500003293156847661293795681558549404144287109375f,-1.471300003518163368454452211153693497180938720703125f,-1.4466000247761605645990812263335101306438446044921875f,-2.13270002059191465804133258643560111522674560546875f,-0.117799993392825097515697052585892379283905029296875f,-0.418499992132186804383309208787977695465087890625f,-0.984799996209144268277668743394315242767333984375f,-1.14829995786845717731239346903748810291290283203125f,-0.79579996557980814397836866191937588155269622802734375f,-0.3312000121280551656610668942448683083057403564453125f,-1.174299995113909034216703730635344982147216796875f,0.033900003290176616843609735951758921146392822265625f,-0.177900004564225611147776362486183643341064453125f,-0.490799977761507133511287293003988452255725860595703125f,0.6912000081047409150158955526421777904033660888671875f,0.66379997129440315717374687665142118930816650390625f,1.249100021386146952551143840537406504154205322265625f,1.519900025671720555919819162227213382720947265625f,1.2277000036045910835724725984618999063968658447265625f,0.18919999269694109944595084016327746212482452392578125f,0.312200004781782780582943814806640148162841796875f,-0.581400019085407393504283390939235687255859375f,-0.537400016433000615734272287227213382720947265625f,-0.380000028610229634296047152020037174224853515625f,-1.0865000333860519443618386503658257424831390380859375f,0.22490002880096471304938177127041853964328765869140625f,0.81340000018328362241248896680190227925777435302734375f,-0.9636000492364169556225306223495863378047943115234375f,-0.99810000418275601763440363356494344770908355712890625f,0.033699993985891130154897155080107040703296661376953125f,-0.6068000246554612342464452012791298329830169677734375f,-0.33400000365823501180528865006635896861553192138671875f,-0.9232000375382602985752100721583701670169830322265625f,0.63380001331567736855276962160132825374603271484375f,0.46319998590573663310721030939021147787570953369140625f,-0.50899998879432661880173327517695724964141845703125f,-0.00539999763518576869625320568957249633967876434326171875f,0.7669000261783602834242401513620279729366302490234375f,0.261199997897445967964813462458550930023193359375f,-0.7890000165849924140815119244507513940334320068359375f,-0.3344000071048736089096564683131873607635498046875f,0.03569998055994449970285131712444126605987548828125f,-0.15019999604225109379740388249047100543975830078125f,-1.78469999856054784004300017841160297393798828125f,-0.951000010684132757177167150075547397136688232421875f,-0.173400013072788627965792329632677137851715087890625f,-0.577799993467330796903524969820864498615264892578125f,0.6280000149533153663838902502902783453464508056640625f,-1.0719000241294496245103573528467677533626556396484375f,-0.468300029990076804864429504959844052791595458984375f,-0.5422000583887101043956135981716215610504150390625f,0.467600031101703894620413848315365612506866455078125f,1.33089999053478180712772882543504238128662109375f,-0.382200027468801106778073517489247024059295654296875f,1.404400030350684946967021460295654833316802978515625f,0.12250000044703479584740080099436454474925994873046875f,1.12390002251267429045356038841418921947479248046875f,-1.00670001821517995921340116183273494243621826171875f,-0.221500000715255662697700245189480483531951904296875f,-0.142499948367475592903019787627272307872772216796875f,-0.11899997776746740196784912768634967505931854248046875f,-0.918100001463293846626356753404252231121063232421875f,-0.3904000107109546480188555506174452602863311767578125f,-0.67719999673515562932379907579161226749420166015625f,0.900200008857249489579999135457910597324371337890625f,0.8010000160112975553516889704042114317417144775390625f,-0.3011999790847301738949681748636066913604736328125f,1.476500052720308264753157345694489777088165283203125f,-0.55380002017021201510260652867145836353302001953125f,-0.693800026726723029213417248683981597423553466796875f,-0.69999998405575691862168241641484200954437255859375f,0.1847000367075211091361097714980132877826690673828125f,-0.074299997684359464500403191777877509593963623046875f,-0.65830001464188103721397737899678759276866912841796875f,0.2834000109493732821874800720252096652984619140625f,0.5950999934203917707264963610214181244373321533203125f,-1.5358000215709211033754399977624416351318359375f,-0.807000001996755589317444901098497211933135986328125f,1.052900001472234947641481994651257991790771484375f,0.21229999132901433700482130007003434002399444580078125f,-0.47240002697557204047740242458530701696872711181640625f,-0.9696000147327776819139444341999478638172149658203125f,-0.1934999896734959090593974906369112432003021240234375f,0.37949998545646701497702224514796398580074310302734375f,-0.120999995708465757360272618825547397136688232421875f,0.73280001831501717646233373670838773250579833984375f,-0.745900028383732038861353430547751486301422119140625f,-0.5950000108964743628092719518463127315044403076171875f,-0.391099989855289376095015541068278253078460693359375f,-0.29029999711811560469953974461532197892665863037109375f,0.484899980372190508859375768224708735942840576171875f,0.04449999439716378901721327565610408782958984375f,-0.074399994364380717115636798553168773651123046875f,-0.95229997773170538266640505753457546234130859375f,-0.4841000090926887988729276912636123597621917724609375f,-0.6965999734044074909178334564785473048686981201171875f,-1.0649999951198696468424031991162337362766265869140625f,0.59809998205453229758177258190698921680450439453125f,0.04009997093826600522703529350110329687595367431640625f,0.5078999617233870278454332947148941457271575927734375f,-0.6888000320166351997386300354264676570892333984375f,1.14419995622634917964433043380267918109893798828125f,-0.87340001623928531504503780524828471243381500244140625f,0.27900003454089183918540584272705018520355224609375f,0.3646999922275540040317309831152670085430145263671875f,-1.29659995999336263849954775650985538959503173828125f,-0.1896999929726124189954816756653599441051483154296875f,-1.0407999779105185922389864572323858737945556640625f,0.941700051295757845082334824837744235992431640625f,0.9381999561965468359403530484996736049652099609375f,-0.4163000444591045123843287001363933086395263671875f,-0.752500007674097748378017058712430298328399658203125f,0.43439999961704034436849042322137393057346343994140625f,-0.47349994500726511059696122174500487744808197021484375f,-0.6144999783784153013499462758773006498813629150390625f,-0.44809999948889023269060771781369112432003021240234375f,-0.1257999845787889370285483892075717449188232421875f,-1.2098999966144556861280534576508216559886932373046875f,-0.75670001642703965671898913569748401641845703125f,0.267899997448921123321952109108678996562957763671875f,-1.1275000365078451380895785405300557613372802734375f,0.443499940276146364936948884860612452030181884765625f,-0.333000026166438534147573591326363384723663330078125f,-0.001100006395578390083755948580801486968994140625f,0.5762999820232390657537280276301316916942596435546875f,0.34100001999735829461002367679611779749393463134765625f,0.388999998163432059161692677662358619272708892822265625f,-0.512000013731420011708905803970992565155029296875f,0.1004000020124018177636315840572933666408061981201171875f,0.373700010060518839072329910777625627815723419189453125f,-0.805900018846988697163169490522705018520355224609375f,0.415000028163194922825596222537569701671600341796875f,1.1949000154085458458297352990484796464443206787109375f,-0.2022000014662739264537094641127623617649078369140625f,0.74480004407912492325749553856439888477325439453125f,-0.5189000261634590938086830647080205380916595458984375f,0.13440005838349566857914396678097546100616455078125f,-0.138900036966801199156407164991833269596099853515625f,0.6316000091485676204428045821259729564189910888671875f,-0.105199981737136294412948700482957065105438232421875f,-0.7750999761164185652972946627414785325527191162109375f,0.33469999461174015920050806016661226749420166015625f,0.2965000048875807703296914041857235133647918701171875f,0.439300027151405980685439089938881807029247283935546875f,-0.40730001565515949746298929312615655362606048583984375f,-0.37660001350715754941944624079042114317417144775390625f,-0.4362000291153791398102157472749240696430206298828125f,1.6685000254362822857956416555680334568023681640625f,0.83480002467036218405382896889932453632354736328125f,-0.726099985608458897701211753883399069309234619140625f,-0.103300005328655242209379139239899814128875732421875f,-1.548600007215142237981808648328296840190887451171875f,-0.57549995985627244721172246499918401241302490234375f,-0.462199991258978482022712341859005391597747802734375f,-0.2424999928474420585189363919198513031005859375f,-1.02369997081458574683665574411861598491668701171875f,-0.8487999519228939249160248436965048313140869140625f,0.0144000088185070040935897850431501865386962890625f,-0.3942999611020088224222490680404007434844970703125f,0.84750001255422835466646347413188777863979339599609375f,0.934200020971893696497545533929951488971710205078125f,0.7260999975480135759653421700932085514068603515625f,-0.18619999910816520394263307025539688766002655029296875f,0.24250001592561620800125865571317262947559356689453125f,0.151099970483779433294557748013176023960113525390625f,0.6859999942183490073688290067366324365139007568359375f,0.396900038546323941091031883843243122100830078125f,0.4086000264376405954891424698871560394763946533203125f,-0.295500007018446364526198522071354091167449951171875f,-1.533500026032328822367389875580556690692901611328125f,-0.90570000149607654549299695645458996295928955078125f,-2.10330007119178841179518713033758103847503662109375f,0.050699966478347402443205282907001674175262451171875f,-1.26950001972913728565117708058096468448638916015625f,-0.574000001013278637884695854154415428638458251953125f,-0.64679999880194660732968259253539144992828369140625f,0.6042000064432624384380687843076884746551513671875f,0.70380001601651331899489605348207987844944000244140625f,-0.04159999299943450967731450873543508350849151611328125f,0.41030001755133305163525392345036379992961883544921875f,-0.04479999118000288493846028359257616102695465087890625f,0.7163000377535826412866981627303175628185272216796875f,0.029199998694658280129488048260100185871124267578125f,0.47270001353248947140173186198808252811431884765625f,-0.28260001068711293470414602779783308506011962890625f,-0.1253000219844292162374443933003931306302547454833984375f,-0.353300005924701121529096781159751117229461669921875f,-0.0140999705918138096283342974857077933847904205322265625f,0.6211999796807763374317801208235323429107666015625f,0.07090002680048380911959071681849309243261814117431640625f,-0.0483000224649908493912420226479298435151576995849609375f,0.05479999863058333808840671963480417616665363311767578125f,0.240200010291487020541723040878423489630222320556640625f,1.384099965059757408880614093504846096038818359375f,-0.161700001898407830225323777995072305202484130859375f,1.6023999677434563881917028993484564125537872314453125f,0.22170001150965734648679017482209019362926483154296875f,1.3465999817863101295500882770284079015254974365234375f,0.578700010228156980218727767351083457469940185546875f,0.650299993765354855668192612938582897186279296875f,-0.5813000134646892913536930791451595723628997802734375f,-1.491400004780292221084891934879124164581298828125f,-0.1577999610573057598372770371497608721256256103515625f,-2.13299998205900198655626809340901672840118408203125f,-0.9944000175356866311204839803394861519336700439453125f,-1.06779997826814732064804047695361077785491943359375f,0.2082000019989909134210392949171364307403564453125f,0.4693000156402591382942546260892413556575775146484375f,0.09469999753609303905221850072848610579967498779296875f,0.05919999614283455446894777196575887501239776611328125f,-0.4010999841928484244135688641108572483062744140625f,-0.22579995250403894768709278650931082665920257568359375f,-0.47009995890557776387908006654470227658748626708984375f,-0.79460000717043877127565565388067625463008880615234375f,-1.095800019484758802690294032800011336803436279296875f,0.28890001893639549734871252439916133880615234375f,-0.53960001521706590921212409739382565021514892578125f,1.081500004738569220563704220694489777088165283203125f,0.14340000737309477329972651205025613307952880859375f,0.52269997026324244160377929802052676677703857421875f,-0.380399997895955355176056400523521006107330322265625f,-0.602800010901688665398978628218173980712890625f,-0.3855000017285343805184538723551668226718902587890625f,-0.41310000615715980476494451067992486059665679931640625f,-1.0530000452585521752979502707603387534618377685546875f,0.0633999928444622107281958278690581209957599639892578125f,-0.01490000382289291547266429915907792747020721435546875f,-0.004000022023916471880511380732059478759765625f,0.319100016431510535852567045367322862148284912109375f,0.4881999985158440580335081904195249080657958984375f,0.25139999034479243444906160220853053033351898193359375f,0.33180000118613239834530759253539144992828369140625f,-0.81250004207715409432211117746192030608654022216796875f,0.58810002374053027551781269721686840057373046875f,1.52040001831054638614659779705107212066650390625f,-0.836499997287988517058465731679461896419525146484375f,0.8665000631324948354716752874082885682582855224609375f,0.027199977922439444455449120141565799713134765625f,0.19700001458823657163321740881656296551227569580078125f,0.2107999914705753763399798117461614310741424560546875f,-1.299100041186064924403353870729915797710418701171875f,1.0449000201582909763686757287359796464443206787109375f,0.935500008601695132171016666688956320285797119140625f,-0.468500007182359734514420779305510222911834716796875f,0.451599959993362887189505272544920444488525390625f,1.021299989324808255020116121158935129642486572265625f,0.526499955415726361707129399292171001434326171875f,-0.540900002753734288063469648477621376514434814453125f,-0.2561000296413897814318261225707828998565673828125f,-0.03259999548792846812972356929094530642032623291015625f,1.69929995797276500724137804354541003704071044921875f,0.00820001371502865339380150544457137584686279296875f,1.8835000076293937354421359486877918243408203125f,0.46120000829100593620069048483856022357940673828125f,0.049800030481815582561466726474463939666748046875f,-0.515600005039572817366888557444326579570770263671875f,-1.4674999809265134587121792719699442386627197265625f,0.6741999942243097621030756272375583648681640625f,0.78369997640252098136670610983856022357940673828125f,0.1892999926924705533792803180404007434844970703125f,1.12970000399947156921598434564657509326934814453125f,-0.9092000250697136465305447927676141262054443359375f,0.2651000026404854992279069847427308559417724609375f,-0.22360001116395000764214273658581078052520751953125f,0.204900017029046477290421535144560039043426513671875f,0.45510001977682090767984846024774014949798583984375f,-1.654399974918365767706518454360775649547576904296875f,-0.30239995295405375230757272220216691493988037109375f,-0.25680000506043398900146712549030780792236328125f,-1.48150001323223090565761594916693866252899169921875f,0.327699996787309988377501213108189404010772705078125f,-1.07090004045367237495156587101519107818603515625f,0.71829999400377264606731841922737658023834228515625f,0.081999998420477115956828129128552973270416259765625f,-1.4096000176012515314738493543700315058231353759765625f,0.17060001673698454993655104772187769412994384765625f,0.5580000146925450099644194779102690517902374267578125f,0.1584999997317790842998874722979962825775146484375f,-0.189000008314847978141415296704508364200592041015625f,0.180100006365776110772713991536875255405902862548828125f,0.18630000020265580218392642564140260219573974609375f,-0.97210002182945609572328748981817625463008880615234375f,0.0757000141620636224359941479633562266826629638671875f,-0.62870002406760983060252101495279930531978607177734375f,0.300499990850686682364312218851409852504730224609375f,1.1033000040620570825211643750662915408611297607421875f,-0.77049997952580451254789295489899814128875732421875f,-0.105699999875575378904812851033057086169719696044921875f,-0.3875000047683716530855235760100185871124267578125f,0.32400002725794951885518457856960594654083251953125f,0.07479996744990291546173466485925018787384033203125f,-0.5080000067017971243643614798202179372310638427734375f,0.54640001848936048389759889687411487102508544921875f,0.17699998996406807183490172974416054785251617431640625f,0.0758000002622605251190179842524230480194091796875f,-0.3177000045359132940347990370355546474456787109375f,-0.68859999633729451762320650232140906155109405517578125f,-0.31690000963807118949944197083823382854461669921875f,-0.25920002648532403100034571252763271331787109375f,0.0750000196695328469331798260100185871124267578125f,0.026800009605289165648400739883072674274444580078125f,1.94699993017315886589813089813105762004852294921875f,-0.07930001467168334983171007479541003704071044921875f,1.6422999701321128185327324899844825267791748046875f,0.10969999699592591912988837066222913563251495361328125f,-0.73429999179840077516701057902537286281585693359375f,-0.04410001244544969889460617196164093911647796630859375f,-0.77250004902481972379746366641484200954437255859375f,1.065299975958466927039580696146003901958465576171875f,-0.35400000518560403861556551419198513031005859375f,-0.1851999958932399881206265490618534386157989501953125f,-0.5736000101208684753828492830507457256317138671875f,-0.07160000693202039911966494400985538959503173828125f,-0.33749999195337299084940241300500929355621337890625f,-0.2861999933898449199887181748636066913604736328125f,-0.10279999614953982955967148882336914539337158203125f,0.3804000338077546672366224811412394046783447265625f,-1.8551999626636510587474049316369928419589996337890625f,-0.24220000176429845595293954829685389995574951171875f,0.332000024795532144850085387588478624820709228515625f,-0.60449998337030397266289583058096468448638916015625f,0.601999988436698885152509319595992565155029296875f,-0.03250000536441799425801946199499070644378662109375f,-0.125999992281198291976807013270445168018341064453125f,-0.384500035077333546240652140113525092601776123046875f,0.74259995793700195321207502274774014949798583984375f,0.8282999759733673528216968406923115253448486328125f,0.2774000019788740445392249966971576213836669921875f,0.769199992883205396765333716757595539093017578125f,-0.641000015079975060672268227790482342243194580078125f,-0.6586999942094087234778498896048404276371002197265625f,0.653500002264976398436147064785473048686981201171875f,-0.27900000762566923295793230863637290894985198974609375f,-0.48290000221729290075245444313623011112213134765625f,-0.66610002988353411712552087919902987778186798095703125f,0.064500004976987934668386515113525092601776123046875f,0.1180999868601559921188481894205324351787567138671875f,0.3508999807745218912913287567789666354656219482421875f,0.214499997824430355120028934834408573806285858154296875f,-0.364899989685416314966204254233161918818950653076171875f,-0.0822000206328931948274174601465347222983837127685546875f,0.30520000159293447783426245223381556570529937744140625f,-0.3404999960102139677786681204452179372310638427734375f,-0.23439997089505215654980929684825241565704345703125f,0.54990002162978102173696015597670339047908782958984375f,0.2745000006556512772704081726260483264923095703125f,-1.0644999812841415831599078956060111522674560546875f,-0.599100011178851143966994641232304275035858154296875f,-0.84449999640882023044241577736102044582366943359375f,0.270599981495737884618080215659574605524539947509765625f,-0.322399988493323075289254120434634387493133544921875f,0.54429999798238271058181680928100831806659698486328125f,-1.0679999929368495514836467918939888477325439453125f,-0.933299986851215290784011813229881227016448974609375f,0.129199955146015010853943749680183827877044677734375f,0.34519999194443229395545813531498424708843231201171875f,1.1630999760940670118003481547930277884006500244140625f,-0.50209998433440927723125923876068554818630218505859375f,0.4978999871298672719177602630225010216236114501953125f,1.5316000037282708756691818052786402404308319091796875f,0.8741000112533559018856976763345301151275634765625f,-0.0495999936848880995654553771601058542728424072265625f,-0.3065999817490574486100740614347159862518310546875f,-0.38400001829862606683718695421703159809112548828125f,-0.5458000005602834381335242142085917294025421142578125f,0.04250001065433028912821100675500929355621337890625f,0.5079999858587982419777517861803062260150909423828125f,0.091499992147088260452392205479554831981658935546875f,0.708899976468086112646460605901665985584259033203125f,1.179900001978874346519887694739736616611480712890625f,-0.84419995130896607893333793981582857668399810791015625f,0.1943000243946908511816218378953635692596435546875f,-0.22979999803453676765485624855500645935535430908203125f,-0.870399987316131440451272283098660409450531005859375f,-0.496199968057871421134308320688433013856410980224609375f,0.002500036247074621797992222127504646778106689453125f,0.2455999572813514220115393982268869876861572265625f,0.06269998993277514642841197201050817966461181640625f,0.57790001875758179039621609263122081756591796875f,0.9643999966740608442705706693232059478759765625f,0.63010003199577369059625198133289813995361328125f,0.554699988055229908923138282261788845062255859375f,-1.5412999990664417726549118015100248157978057861328125f,-0.864600006127357545437916996888816356658935546875f,-0.04730001766681712904727419299888424575328826904296875f,0.8928999682426450590355671010911464691162109375f,-0.29269998045191181201829522251500748097896575927734375f,0.55649997843802001540325363748706877231597900390625f,-0.451599996091425648359063416137360036373138427734375f,0.55989998260736495439005011576227843761444091796875f,-0.803999990433454048144312764634378254413604736328125f,-0.5565000364780416930443607270717620849609375f,-0.9922000071287151445886820511077530682086944580078125f,-0.1500999489963046329421558766625821590423583984375f,0.7418000281572350029790641201543621718883514404296875f,-0.53199995580315562193618461606092751026153564453125f,0.586399997180700349730386733426712453365325927734375f,-1.4961999881744389995219535194337368011474609375f,-1.21690003929138157445777324028313159942626953125f,0.254000037670135014877814683131873607635498046875f,0.29929995603561376782408842700533568859100341796875f,-0.61539997986555050601964467205107212066650390625f,0.7881999986648562384061733609996736049652099609375f,-1.2686999458551415642659776494838297367095947265625f,-0.30470005145966982951222234987653791904449462890625f,-0.92029998601675000458044451079331338405609130859375f,0.9516999972790483042217601905576884746551513671875f,0.0473000271290537899204764471505768597126007080078125f,0.3029000093698499540550983510911464691162109375f,-0.1360999984979631705073188641108572483062744140625f,0.65760006850361829577877870178781449794769287109375f,0.44709996112585059790944796986877918243408203125f,-0.973600036048889894146896040183492004871368408203125f,0.653599984937906608450930434628389775753021240234375f,-0.32129999275207499209727757261134684085845947265625f,-0.5408000165790316504654811069485731422901153564453125f,0.6507999715775250049176747779711149632930755615234375f,1.239700036633015312048655687249265611171722412109375f,0.5884000204861166150038798150490038096904754638671875f,0.62079998744726150317063684269669465720653533935546875f,-1.0705000237822535069653895334340631961822509765625f,1.2306000075727698828131906338967382907867431640625f,0.808100027018785471710771162179298698902130126953125f,1.2039000275418165752938648438430391252040863037109375f,0.7191000266760589898495936722611077129840850830078125f,-0.1918999735027551167831916245631873607635498046875f,-1.3845000312030315559042037421022541821002960205078125f,-1.6248000033989546952994942330406047403812408447265625f,0.2132999783575535435176107057486660778522491455078125f,-0.5696000236362219748542656816425733268260955810546875f,-0.833200040611625336595125190797261893749237060546875f,0.289500010788440675923993694595992565155029296875f,-0.28470005950629673208140957285650074481964111328125f,-1.4376000263519588262539627976366318762302398681640625f,-0.453999996319412790768410559394396841526031494140625f,0.75470003236755778264210903216735459864139556884765625f,-1.5109999918341632252349882037378847599029541015625f,-0.4882000236243009272385506847058422863483428955078125f,-0.3119000209629536612965239328332245349884033203125f,-0.5332000146090984937785606234683655202388763427734375f,0.475000017806888052973590674810111522674560546875f,0.96470003797411951751428205170668661594390869140625f,0.552300047692657170017582757282070815563201904296875f,0.940100036372989666944022246752865612506866455078125f,1.889500030159950938468682579696178436279296875f,0.135000039078295941408214275725185871124267578125f,-0.65139997050762143970814577187411487102508544921875f,0.2015999904662371111641050447360612452030181884765625f,-0.503999992370605554015128291212022304534912109375f,-0.97890004329979429797958800918422639369964599609375f,0.263300062102079124315423541702330112457275390625f,1.0170000668168073243435856056748889386653900146484375f,0.8075999745517961070362389364163391292095184326171875f,-2.099000057935715091161910095252096652984619140625f,0.01170000808239013423417418380267918109893798828125f,0.29329999325871458637493560672737658023834228515625f,0.6584000274896624205211992375552654266357421875f,0.964000039085746163181056544999592006206512451171875f,0.094699993419646855130622498109005391597747802734375f,-0.6117000396728518030187160547939129173755645751953125f,-0.4700000046938657138895223397412337362766265869140625f,1.060000026896596114767135077272541821002960205078125f,0.646799987328052328194871734012849628925323486328125f,-0.9500000235438352813588380740839056670665740966796875f,-0.248600002893805349657441183808259665966033935546875f,0.3328000232696535221066369558684527873992919921875f,0.4202000572115185406119053368456661701202392578125f,-1.310800023806094838363378585199825465679168701171875f,0.12749999314546567319439418497495353221893310546875f,0.3294000263273719752987744868732988834381103515625f,0.75230003592073912965787485518376342952251434326171875f,-0.935299997341632494141094866790808737277984619140625f,-0.4076000151574608043603120677289552986621856689453125f,0.13400000116229104918375014676712453365325927734375f,2.488499963708221685010357759892940521240234375f,0.25930003203153528801294669392518699169158935546875f,0.1866000327110295398114203635486774146556854248046875f,0.0042999715328218002241555950604379177093505859375f,0.6494999937638628040303956368006765842437744140625f,-0.5212999632477757216975078335963189601898193359375f,1.04460005541294886910463901585899293422698974609375f,-0.45230001111030571081528250942938029766082763671875f,0.54220000474452945127268321812152862548828125f,0.752000016748905153463056194595992565155029296875f,0.65220001938492055959528670427971519529819488525390625f,1.2323000233292571437004880863241851329803466796875f,-0.060700007419288215526620433593052439391613006591796875f,0.1758000169515607780112986802123486995697021484375f,0.67310001012459375058227806221111677587032318115234375f,0.4731000012397768017535781837068498134613037109375f,-1.212799999129771322259330190718173980712890625f,-1.19439996329546005426891497336328029632568359375f,0.6656999859616157255715052087907679378986358642578125f,1.00039996168613498639388126321136951446533203125f,0.269600029317289602293072903194115497171878814697265625f,-1.0959999521970757996314205229282379150390625f,-0.018500052463263472191101755015552043914794921875f,-0.8503000146269794612408077227883040904998779296875f,0.6368000223457814801264476045616902410984039306640625f,-0.5923999870777123533116537146270275115966796875f,-0.61890006334185709846451572957448661327362060546875f,0.10140000030994489321756191202439367771148681640625f,-0.105499996364116999103544003446586430072784423828125f,0.0501000297605997957361978478729724884033203125f,-0.700600039237738059227922349236905574798583984375f,-0.4983000002622599566848293761722743511199951171875f,-0.752099982285499013556773206801153719425201416015625f,1.3765000175535675452920258976519107818603515625f,-0.455299994063377511110957129858434200286865234375f,-0.2462999793410300952700708876363933086395263671875f,0.421399989879130743020141380839049816131591796875f,1.63129994089603425067025455064140260219573974609375f,-1.3320999906301498327820809208787977695465087890625f,1.166100008034707258275375352241098880767822265625f,0.02520000468492522571750669158063828945159912109375f,0.5989999938607208918028845801018178462982177734375f,0.16890000414699313235900035579106770455837249755859375f,0.0919000504300004905644527752883732318878173828125f,-0.55000001065432979618918807318550534546375274658203125f,-0.0247000330567364034806132622179575264453887939453125f,1.191000012397765761562595798750407993793487548828125f,0.2160000138133764124859226285479962825775146484375f,1.05010000174641593417845797375775873661041259765625f,-0.8360999752521516281689173410995863378047943115234375f,-1.723199988830088624780501049826852977275848388671875f,-0.183399995638430457489675973192788660526275634765625f,0.6274999764561648429861406839336268603801727294921875f,-0.3986999769613148725255769022624008357524871826171875f,0.473699994656443745810747714131139218807220458984375f,1.3673000364422798202923559074406512081623077392578125f,0.46190006521940318862107233144342899322509765625f,-0.2004999770298596661888268499751575291156768798828125f,-1.0088000270247459155825708876363933086395263671875f,-0.41580002768039747707007336430251598358154296875f,-1.1042000270441170162172284108237363398075103759765625f,-0.2814999907314772453759132986306212842464447021484375f,-0.553400019741058457611870835535228252410888671875f,-0.7269000320084393340636097491369582712650299072265625f,0.3413999746799465384583527338691055774688720703125f,0.81379999081492371271195906956563703715801239013671875f,-1.431599949264526827619192772544920444488525390625f,0.04289998321831289818106824895949102938175201416015625f,-0.7201000421285623787071017432026565074920654296875f,-0.28200002030655768603395472382544539868831634521484375f,-0.99010004689693520418813932337798178195953369140625f,-0.54470005546435762422419202266610227525234222412109375f,-0.857199967789650685290325782261788845062255859375f,0.950899996942281777734251591027714312076568603515625f,0.076900004720688031056852196343243122100830078125f,0.27090001297965649040833113758708350360393524169921875f,0.499400024911761608592541961115784943103790283203125f,-0.07779999287128414042769009029143489897251129150390625f,0.62999997515231331846052853506989777088165283203125f,0.65640001804232550597362205735407769680023193359375f,0.09679999567270325400158981210552155971527099609375f,1.447200037229059876864312172983773052692413330078125f,0.82589993703961450677297762013040482997894287109375f,-1.47799995419382934613850011373870074748992919921875f,0.86499996960163105796937088598497211933135986328125f,0.69949997942149533258771043620072305202484130859375f,0.344599965447188072431572436471469700336456298828125f,0.1227999994277941819831312386668287217617034912109375f,0.949999980852008985010570540907792747020721435546875f,0.241900074160099620712571777403354644775390625f,-0.2501000093460081785678994492627680301666259765625f,-0.6493000216007231983894598670303821563720703125f,0.556700002121925052733786287717521190643310546875f,0.123800022852420976704479471663944423198699951171875f,-0.3129000076934695950825471300049684941768646240234375f,-0.7879000175654891080512243206612765789031982421875f,-1.80289999230802067131662624888122081756591796875f,-0.20270006965398845721892939764074981212615966796875f,0.9976999793902041613335995862144045531749725341796875f,1.064600053513050426090558175928890705108642578125f,-0.9915999771296970521916591678746044635772705078125f,-0.9139000572323805471341984230093657970428466796875f,-1.545400029449165213435435362043790519237518310546875f,-0.70610001131296140641779857105575501918792724609375f,-0.148000016666949729682301040156744420528411865234375f,1.2442000331163409754253734718076884746551513671875f,0.8171000331729654231338599856826476752758026123046875f,-0.8258999857664104382592995534650981426239013671875f,0.3312999879837035255292221336276270449161529541015625f,0.9332000135660170059281881549395620822906494140625f,0.58450003165006680916349068866111338138580322265625f,0.9715000469088561629860123503021895885467529296875f,0.48310004134252704233176700654439628124237060546875f,0.6991999416232115294178584008477628231048583984375f,-0.73210000631362237566435169355827383697032928466796875f,-0.297499965429306456599078956060111522674560546875f,0.26190000041797778518315453766263090074062347412109375f,1.08849998259544378242935636080801486968994140625f,1.0050000339373943436527270023361779749393463134765625f,-0.514000004589558301404395024292171001434326171875f,-0.4852000350832941233392148205894045531749725341796875f,0.015099967622756338414546917192637920379638671875f,-0.75280004707425884902249890728853642940521240234375f,-1.776099985980987838729561190120875835418701171875f,0.1034999896734956070787347925943322479724884033203125f,0.275599979186058163804773357696831226348876953125f,-0.2564999986104667950570501488982699811458587646484375f,-0.63979999382495922333191629149951040744781494140625f,-0.125400023078917843832869039033539593219757080078125f,-0.39799997654557284221255031297914683818817138671875f,0.34629992688894173369362761150114238262176513671875f,0.50010000427961376345820099231787025928497314453125f,-0.04629993433952250825313967652618885040283203125f,-0.90829996032714888087866711430251598358154296875f,1.0790999939680094854566050344146788120269775390625f,0.936399953222274206154907005839049816131591796875f,-0.7873000033617021387044587754644453525543212890625f,-0.7851000340819354050836409442126750946044921875f,0.3776000129222865098199690692126750946044921875f,0.57609998300075648813844964024610817432403564453125f,-0.830799982976913042875821702182292938232421875f,-0.0345999983787539377999564749188721179962158203125f,0.77590000513792034553262055851519107818603515625f,-0.2089999962449073933612453402020037174224853515625f,-0.3428000143289562373638545977883040904998779296875f,-0.8920000132843843854146825833595357835292816162109375f,1.1714000114858149625973737784079276025295257568359375f,0.4462000003188848484825257401098497211933135986328125f,-0.360200017574429320887929861783050000667572021484375f,0.0011000143676997975461517853545956313610076904296875f,-1.428900013273954439085855483426712453365325927734375f,-0.46310002042502185215511190108372829854488372802734375f,0.67110001395791751832575755543075501918792724609375f,-0.27360004826784167786257739862776361405849456787109375f,0.5236000194713472666308007319457828998565673828125f,-0.03659998122751739746405519326799549162387847900390625f,-0.091699975672363986944901625975035130977630615234375f,-1.174500035114585561046851580613292753696441650390625f,0.244699939924478027108989408588968217372894287109375f,0.95240000451207151854760013520717620849609375f,0.291500011444091544632328805164434015750885009765625f,-0.15250001072883634378740680404007434844970703125f,0.675200008559226461102298344485461711883544921875f,0.1636000092826785934363442720496095716953277587890625f,-0.18130000377893384921890174155123531818389892578125f,0.3194000069372353589614021984743885695934295654296875f,-0.2855999674141411759364928002469241619110107421875f,0.055499996941537066419414259144105017185211181640625f,0.484199987705051793085431199870072305202484130859375f,-0.265499999642372142005797286401502788066864013671875f,0.407200009289383491051239616353996098041534423828125f,-0.6176000149525702198616272653453052043914794921875f,0.84820002429485352735127889900468289852142333984375f,0.59800003877654683837050697547965683043003082275390625f,0.5812999568402776873199400142766535282135009765625f,-1.362099992660432690172456204891204833984375f,-0.72599999973177897771847710828296840190887451171875f,0.328999997809529265424544064444489777088165283203125f,0.052199979114533334723091684281826019287109375f,1.433800072547793735822097005438990890979766845703125f,-0.603599964225292939801192915183492004871368408203125f,-1.3762000322818759290299794884049333631992340087890625f,-0.373300020900368512144495980464853346347808837890625f,-0.656600017958879789148340933024883270263671875f,0.42149999922513980976646053022705018520355224609375f,1.08080000506788476144492960884235799312591552734375f,-1.15530001701116535883784308680333197116851806640625f,-0.226000000290572433669922247645445168018341064453125f,-0.98019996672421683125975278016994707286357879638671875f,0.9059000028282415684088846319355070590972900390625f,-1.0154999793022876364290141282253898680210113525390625f,-0.8076000437676906784645325387828052043914794921875f,0.3652000493884077769735085894353687763214111328125f,0.435200011241435813502675955533050000667572021484375f,0.111099981957674120991441668593324720859527587890625f,-0.23050000679492921307200958835892379283905029296875f,-0.98989999676346673851412560907192528247833251953125f,0.590499975949525524043792756856419146060943603515625f,-0.000700046963244826958572275543701834976673126220703125f,0.1293000132746995323174132863641716539859771728515625f,-0.0263999881096186328477415372617542743682861328125f,0.49169999927207841405873978146701119840145111083984375f,0.3524999879300594329833984375f,-0.45010000126212845739104295716970227658748626708984375f,0.82449999458342748681616285466589033603668212890625f,-0.21559998946785940976411666269996203482151031494140625f,-0.35339997444152826400198819101206026971340179443359375f,-1.14610001172274333924860911793075501918792724609375f,1.12250000659376336642480964655987918376922607421875f,1.1162000357463959243631279605324380099773406982421875f,-0.0976000009641051491371399606578052043914794921875f,0.15350000708922750813911761724739335477352142333984375f,-0.259100009763240990423582843504846096038818359375f,-0.1226000229060648649692666367627680301666259765625f,0.107300013066083332891054169522249139845371246337890625f,-0.1668999902851882044529929771670140326023101806640625f,-0.4299000134713948373388348045409657061100006103515625f,1.0392999847203496788239363013417460024356842041015625f,0.0498000142022967662303045699445647187530994415283203125f,-0.569699957992136685192008371814154088497161865234375f,0.349299990662187642698199852020479738712310791015625f,-0.0055999832466249177542749748681671917438507080078125f,0.50989998905211653124069925979711115360260009765625f,-0.651300019611418701259708541329018771648406982421875f,-0.4984000193126500999341033093514852225780487060546875f,-0.1319000176474449670394051281618885695934295654296875f,-0.2176999998234212529979458849993534386157989501953125f,0.8515000150203702045104137141606770455837249755859375f,-0.2189999981075523016738060277930344454944133758544921875f,0.430799994897842708496682462282478809356689453125f,1.1505000298917291701172871398739516735076904296875f,-1.13109999432563856913702693418599665164947509765625f,-1.0008000040061768576293843580060638487339019775390625f,-1.2753999763637782205449866523849777877330780029296875f,0.8233000014729794369117144015035592019557952880859375f,1.04139996329918549378135139704681932926177978515625f,-0.703099980489910070247105977614410221576690673828125f,0.33639998339712651187483061221428215503692626953125f,-0.35600000889599370257343480261624790728092193603515625f,0.22419999061077888757864684521337039768695831298828125f,-0.889400033442675930217546920175664126873016357421875f,0.404400007775425951450642969575710594654083251953125f,0.3680999918334182385848407648154534399509429931640625f,-1.325899999438225673742408616817556321620941162109375f,0.286199998530745236546835030821966938674449920654296875f,0.30860001956447991044996115306275896728038787841796875f,-0.175900012290477913978747892542742192745208740234375f,0.2784000394850971726867783218040131032466888427734375f,0.70570003874898024065487334155477583408355712890625f,0.174899971953034860661091443034820258617401123046875f,0.62669999139308885105492663569748401641845703125f,-1.315599968494475913161068092449568212032318115234375f,-1.300900002604722605070719509967602789402008056640625f,-1.2968000039428464464918988596764393150806427001953125f,-0.388900041139125907108109458931721746921539306640625f,0.908499964080750288530907710082828998565673828125f,0.798300042581558511045614068279974162578582763671875f,0.9412999891757964920913082096376456320285797119140625f,0.629300021600723180625891473027877509593963623046875f,0.5692999815911050642824875467340461909770965576171875f,0.4304000276237724964545350303524173796176910400390625f,0.4497000408798459414327908234554342925548553466796875f,-1.249400019323825716810461017303168773651123046875f,-0.76390004292726576551331163500435650348663330078125f,-0.1509000215291980140364103135652840137481689453125f,0.0450000055693091649544612664612941443920135498046875f,0.28030000836849211509615997783839702606201171875f,0.6381000096403062205041578636155463755130767822265625f,0.2039000240027899479855477693490684032440185546875f,0.72539998809844219440634560669423080980777740478515625f,-0.7281000246345996629315777681767940521240234375f,-0.59900001293420768178066282416693866252899169921875f,-0.73519999797940283059460853110067546367645263671875f,-0.7931000345811247687066725120530463755130767822265625f,0.3424000034689900218154434696771204471588134765625f,-0.177100003501027669639000805545947514474391937255859375f,0.171100011238455085305076863733120262622833251953125f,0.33989998636990748082098434679210186004638671875f,-1.04220000474452945127268321812152862548828125f,0.74550001150742151168060445343144237995147705078125f,0.30040002307891899846481464919634163379669189453125f,1.246500057041645792565986994304694235324859619140625f,0.57079998401999443302656800369732081890106201171875f,0.5101000412344927070762423682026565074920654296875f,-0.02800003272295015221970970742404460906982421875f,0.206799996417760922895467956550419330596923828125f,0.675799946916104232741417945362627506256103515625f,-0.4699000076413160087440701317973434925079345703125f,-0.725600006753206461240779390209354460239410400390625f,-0.663500003457069720269601020845584571361541748046875f,-1.018099998110531867467898337054066359996795654296875f,0.40120001633763280324274091981351375579833984375f,0.74650000026822116439007004373706877231597900390625f,0.2382000440388922779533231732784770429134368896484375f,1.372500000149011523120634592487476766109466552734375f,-0.7829000081777568453844651230610907077789306640625f,0.56819997184276527235624598688445985317230224609375f,0.7872999944210050671244971454143524169921875f,-0.876799992468952904545176352257840335369110107421875f,-0.184299991798400952802694519050419330596923828125f,0.50449996981024725073439185507595539093017578125f,-0.5038999868988991437390723149292171001434326171875f,0.5831999911397698355131069547496736049652099609375f,-0.303600027704238328851715778000652790069580078125f,0.56449998441338511412368461606092751026153564453125f,0.060399998044967428967311207088641822338104248046875f,-0.08299998787045470560741478038835339248180389404296875f,-1.77649994155764634484739872277714312076568603515625f,-1.16329996792674084105101428576745092868804931640625f,-2.35419995265007031548520899377763271331787109375f,-0.4576999621421098396467641578055918216705322265625f,0.68150001755356726107493159361183643341064453125f,0.33519999797940236430093818853492848575115203857421875f,1.15680002830624584220231554354541003704071044921875f,1.01600000017881342273540212772786617279052734375f,0.7284000340461727063257058034650981426239013671875f,1.23420002670884088047387194819748401641845703125f,0.377599977755547211444309141370467841625213623046875f,0.64310001137256644909712122171185910701751708984375f,0.21640000343918774916573966038413345813751220703125f,1.33639998049139929747752830735407769680023193359375f,0.6443999740242958296221331693232059478759765625f,1.3938000241190200423346823299652896821498870849609375f,1.69569997504651581721191178075969219207763671875f,0.58039996205866317691146605284302495419979095458984375f,0.07579999206662169086712310672737658023834228515625f,0.1021999937176705230967854731716215610504150390625f,-1.6809000006675720584325972595252096652984619140625f,-1.27610000192523020956514301360584795475006103515625f,0.068800014656782426669678898178972303867340087890625f,-0.650399990743398603854075190611183643341064453125f,-0.25319999389648462084778657299466431140899658203125f,0.15420000716969373133480303295073099434375762939453125f,0.232499984502792411689142682007513940334320068359375f,-0.1621999992497265008495332949678413569927215576171875f,-0.296600017511844615825111759477294981479644775390625f,-0.6725000154040754463125040274462662637233734130859375f,0.251700028347969340103418289800174534320831298828125f,-0.253500014707446152950609530307701788842678070068359375f,-0.7310000123977662411789424368180334568023681640625f,-0.2865000155977905915705150619032792747020721435546875f,0.301400005376339041873734458931721746921539306640625f,0.113299993724375713810559318517334759235382080078125f,0.335200030612945898411680900608189404010772705078125f,1.6299000417463480605562153868959285318851470947265625f,-0.40810001554489117037149981115362606942653656005859375f,-0.08639996559023825628553794331310200504958629608154296875f,0.6532000067859888847721094862208701670169830322265625f,-0.5453999730110172805552792851813137531280517578125f,-1.4568000204086306581530152470804750919342041015625f,-1.32619996381103977256543657858856022357940673828125f,-0.315899973323940985725499785985448397696018218994140625f,0.365900012066960211853938744752667844295501708984375f,-0.2871000153660776543773636149126105010509490966796875f,0.1693000323295592579597723670303821563720703125f,-0.8202000167548657572069714660756289958953857421875f,-0.190099989825487369898837641812860965728759765625f,0.24200000683963296399525688684661872684955596923828125f,1.3640999976187941911121015436947345733642578125f,1.3876999999910590855023428957792930305004119873046875f,1.947400029471516358370308807934634387493133544921875f,0.004400037205219309299764063325710594654083251953125f,0.33900002843141496811085744411684572696685791015625f,0.1842000014513729411191889084875583648681640625f,-0.088199969533085731399069118197076022624969482421875f,0.25469996480941770045092198415659368038177490234375f,0.1053000033020976555775405358872376382350921630859375f,0.47909999076426057484923148876987397670745849609375f,-1.0255000114887955486864257181878201663494110107421875f,-1.41910003665983719400855989078991115093231201171875f,0.046400005525350618285074233426712453365325927734375f,0.4459000438809397337536211125552654266357421875f,-0.2637999966263766538077106815762817859649658203125f,-0.12369999487996086173779985983856022357940673828125f,-1.3953000177562235872841256423271261155605316162109375f,-0.785500023409724246192809005151502788066864013671875f,0.0778999826222659397245706713874824345111846923828125f,0.27560000325143363397728535346686840057373046875f,0.46629999945759781354581718915142118930816650390625f,-0.4502000342637302310322411358356475830078125f,0.01220000504255314144330668568727560341358184814453125f,-0.100599994906783063441935155424289405345916748046875f,1.063600032174587539657295565120875835418701171875f,0.282999956950544195866115160242770798504352569580078125f,-0.3847999886095518906614643128705210983753204345703125f,0.34579998897463026796827989528537727892398834228515625f,0.183699984747171196630688427831046283245086669921875f,-0.436700045856834140334257199356216005980968475341796875f,0.2625000257790091051646186315338127315044403076171875f,1.088800007746368425642913280171342194080352783203125f,-0.841500025004148444196516720694489777088165283203125f,-1.51770000503882673825728488736785948276519775390625f,0.621800014746188622893896535970270633697509765625f,1.2504999607317142107376639614813029766082763671875f,-0.89249999538063917725594365037977695465087890625f,0.107500028777867295826098370525869540870189666748046875f,0.13339999157786319727847512695007026195526123046875f,-0.258100009901076621332549621001817286014556884765625f,-0.37790000758171071737478996510617434978485107421875f,1.954999979734420634258640347979962825775146484375f,1.12709999212026534820552114979363977909088134765625f,2.458800009441375777896610088646411895751953125f,-0.010300022226572114192322260350920259952545166015625f,1.37509996568560577401285627274774014949798583984375f,1.0775999805867668701608863557339645922183990478515625f,1.9227999860167503953789491788484156131744384765625f,-1.36669998498558964428184481221251189708709716796875f,-0.00199998351931629514410815318115055561065673828125f,0.070499964699149320068727320176549255847930908203125f,1.2718999695539476846306570223532617092132568359375f,-0.49599995592236556518628276535309851169586181640625f,1.527899976587296038132990361191332340240478515625f,-0.96100004644691949806656339205801486968994140625f,0.6453000056862823186065725167281925678253173828125f,0.1066000047713517862035814687260426580905914306640625f,0.6136000274062158865717719891108572483062744140625f,0.2318000370979316215880317031405866146087646484375f,0.20400000935792927947431962820701301097869873046875f,0.3469000352680691179330096929334104061126708984375f,-0.4907999720990654513030904126935638487339019775390625f,0.340000024288892976898068809532560408115386962890625f,-0.27630000377893448870736392564140260219573974609375f,0.4716000321149831364664350985549390316009521484375f,0.4496000304162499627835813953424803912639617919921875f,0.004600008660554433248535133316181600093841552734375f,-1.71209998347759206893670125282369554042816162109375f,-0.34899998253583941476563268224708735942840576171875f,-0.32509997119903655260486630140803754329681396484375f,-1.112199985224008091933001196593977510929107666015625f,-0.8115999926269044095761273638345301151275634765625f,-1.03649997761845558130744393565692007541656494140625f,-0.39030001596808450159414860536344349384307861328125f,-0.876200028109550554944462419371120631694793701171875f,0.0293999937683342427163779575494118034839630126953125f,0.04039998925328214252061798106296919286251068115234375f,-0.974300009456276594477230901247821748256683349609375f,-0.23159998288527117171753388902288861572742462158203125f,-0.1238000176370144256310368291451595723628997802734375f,0.45489999767616329240382810894516296684741973876953125f,0.185699979442358387160538768512196838855743408203125f,0.5080000404529270152664821580401621758937835693359375f,-0.1738000106334689487397326956852339208126068115234375f,-0.0110999960951509157869310229216353036463260650634765625f,-0.28690000039935059650275661624618805944919586181640625f,0.340100013294815095576950625400058925151824951171875f,-0.2813000394672153792186009013676084578037261962890625f,-0.988300010991097099832813910325057804584503173828125f,0.91389999211430517078724733437411487102508544921875f,0.52860000522211192386379252639017067849636077880859375f,-0.281200000095367386165889911353588104248046875f,0.076299994651973346293516442528925836086273193359375f,-0.25629997326880682440020109424949623644351959228515625f,-1.0413999540790916942256671973154880106449127197265625f,-1.005499988615512929612805237411521375179290771484375f,-1.9894999932050705382380328956060111522674560546875f,-0.54649997001886418956928537227213382720947265625f,0.14129995862841671083742767223156988620758056640625f,-0.250600001388787685385750592104159295558929443359375f,-0.3958999660968789413573176716454327106475830078125f,-0.69859997480511726308805009466595947742462158203125f,-1.7277999673902986188522845623083412647247314453125f,-1.321899997866154308212571777403354644775390625f,-1.795299983558058531940559987560845911502838134765625f,0.064399982666969446398752552340738475322723388671875f,0.543999984622001164780158433131873607635498046875f,0.236099996039271342596066460828296840190887451171875f,1.02480001349449167946659144945442676544189453125f,-0.1683999917641278987190389671013690531253814697265625f,0.90609998072832775850571351838880218565464019775390625f,-0.3132000112935902080835148808546364307403564453125f,0.0225000099837782130407504155300557613372802734375f,-0.145699991512298510087930480949580669403076171875f,-1.2641999757468702636487023482914082705974578857421875f,-1.7360999953687183960937545634806156158447265625f,0.68579998208284376204346699523739516735076904296875f,-0.79840000826716472914768019109033048152923583984375f,0.1709999930262566802952051148167811334133148193359375f,-0.5985999942511313065551803447306156158447265625f,0.06500001251697573589893863754696212708950042724609375f,-0.990699986967443546603817594586871564388275146484375f,-1.63980002437233896017687584389932453632354736328125f,-0.9074000082373618880637877737171947956085205078125f,0.23800000128149978451119750388897955417633056640625f,-0.28070001073479655673281740746460855007171630859375f,0.6478000199913978018884108678321354091167449951171875f,0.28690001623183558621121846954338252544403076171875f,-0.020499975204467890677051400416530668735504150390625f,1.2395000206604598336213030052022077143192291259765625f,-0.85690000934004739718830023775808513164520263671875f,-1.140799994990974397524041705764830112457275390625f,1.41959999063014929987502910080365836620330810546875f,1.435700024145841968703507518512196838855743408203125f,-0.96680001593828190920021370402537286281585693359375f,-0.006699994969368183728164467538590542972087860107421875f,0.46940000770092016324497308232821524143218994140625f,0.272399990952014281475612733629532158374786376953125f,-0.008800010335445307418922311626374721527099609375f,-0.84410000099018234198666732481797225773334503173828125f,-0.10520000841021559523369433009065687656402587890625f,-0.24000001283362493165185469479183666408061981201171875f,-0.1279999990463256409611858543939888477325439453125f,1.1878999944686885470446213730610907077789306640625f,1.076299992677569594690112353418953716754913330078125f,1.42850002297759015590372655424289405345916748046875f,-0.2997999856293207532331734910258091986179351806640625f,-1.2545999569535251083607363398186862468719482421875f,-0.169700023445486625206513053853996098041534423828125f,0.1901999829292293497928767465054988861083984375f,0.8028000334024429918144960538484156131744384765625f,1.006299974274636355175971402786672115325927734375f,0.21150000958144676843630804796703159809112548828125f,-1.5160000278949734564548634807579219341278076171875f,-1.339500004157424317696722937398590147495269775390625f,-1.177500028014183186542140902020037174224853515625f,0.101899990564584985719420728855766355991363525390625f,-0.4533999696731569883922929875552654266357421875f,-0.213599994176626228892246217583306133747100830078125f,-0.53490001151561727965599857270717620849609375f,0.219599995845555895357392728328704833984375f,0.3497999816060068933865068174782209098339080810546875f,0.53989997634887654243129873066209256649017333984375f,1.54419999958872811163246296928264200687408447265625f,0.19659998413324419885839233756996691226959228515625f,-1.64460000866055455759351389133371412754058837890625f,0.05849998661875677186117172823287546634674072265625f,0.72740002135038306363412630162201821804046630859375f,-1.0405000374913218053052332834340631961822509765625f,1.06990001121759359392626720364205539226531982421875f,0.73780003489255907567212489084340631961822509765625f,-0.3800999861001965740570085472427308559417724609375f,-0.402099969768523468616194804781116545200347900390625f,0.15119998340606688458365169935859739780426025390625f,-0.1376000007033351124619002803228795528411865234375f,-0.12249999955296519971170710050500929355621337890625f,0.2240000005662441395770656527020037174224853515625f,-0.6825999920606609094875238952226936817169189453125f,0.108800056435913228281009423881187103688716888427734375f,0.56829994661807947675669083764660172164440155029296875f,-0.45660004684850630241044200374744832515716552734375f,1.2707000444859273269315735888085328042507171630859375f,0.3017000092372297270770786781213246285915374755859375f,-0.95890001387000101118474049144424498081207275390625f,-0.39890001781880857212314595017232932150363922119140625f,-0.05689998952150332467425641880254261195659637451171875f,-0.41980000121965976855875624096370302140712738037109375f,0.53250000435858935166066885358304716646671295166015625f,-0.27130000331327275286952271926566027104854583740234375f,-0.0268999738007786870497284326120279729366302490234375f,0.79110001794397877272757568789529614150524139404296875f,-0.150700012522935633541010247427038848400115966796875f,-0.5473000047214326979627685432205908000469207763671875f,-0.11249999612569805407247258699499070644378662109375f,-1.302899949169159299344755709171295166015625f,-0.017799995106458244009672853280790150165557861328125f,-1.21969999818801877466967198415659368038177490234375f,0.05290000124871685471816817880608141422271728515625f,1.412199984255433360402776088449172675609588623046875f,0.58739999840259571328715537674725055694580078125f,0.6492000124037264896514898282475769519805908203125f,0.57990001956224446200138800122658722102642059326171875f,-1.2708000233590601713018486407236196100711822509765625f,0.042799983446300260414574267997522838413715362548828125f,-0.49310002605021008292851547594182193279266357421875f,1.340899983456731003883533048792742192745208740234375f,0.52390000850558227085684848134405910968780517578125f,-1.0236999885469675319882298936136066913604736328125f,-0.7834999627768990393406056682579219341278076171875f,-0.06769999440312390870388981056748889386653900146484375f,-0.17989999706149095715090879821218550205230712890625f,0.02199999290704823806663625873625278472900390625f,-0.229000003099441240550504517159424722194671630859375f,-0.190900061240792950911782099865376949310302734375f,-1.204299998056888920672236054087989032268524169921875f,0.064200039151311560914336951100267469882965087890625f,1.090300020587444240760532920830883085727691650390625f,0.227200017857551284095052324119023978710174560546875f,0.75319999635517564939846124616451561450958251953125f,1.024700016851723294308840195299126207828521728515625f,0.1268000109836460165269045319291763007640838623046875f,1.1833000184044240032932293615886010229587554931640625f,0.381299948421121026598257230944000184535980224609375f,0.71049996127188175254474344910704530775547027587890625f,0.695000020563602216583376502967439591884613037109375f,0.7058999845743176848600342054851353168487548828125f,-0.46280000479221339304558568983338773250579833984375f,-0.5112999985635280353335474501363933086395263671875f,-0.85040000758171085948333711712621152400970458984375f,-0.37509997123628835424824501387774944305419921875f,0.478699971485137876214821517351083457469940185546875f,-0.8689000165894624760909437100053764879703521728515625f,1.1240000208318232655102519856882281601428985595703125f,0.94709998211786106114828953650430776178836822509765625f,-0.70019999717846470588256124756298959255218505859375f,-0.09029998465701949417194782654405571520328521728515625f,0.3745999929398298178284676396287977695465087890625f,-0.1632000137895346369987237267196178436279296875f,0.2459000093102454709281801115139387547969818115234375f,0.261899995779990957256444517042837105691432952880859375f,-0.8424000069707633375770683414884842932224273681640625f,-1.3744000662252313649247525972896255552768707275390625f,0.62610002762973360201925743240281008183956146240234375f,0.493700004677474268977022120452602393925189971923828125f,-0.51850001815333968124832608737051486968994140625f,-1.5047999521017079160856155795045197010040283203125f,0.434700027245283227017580429674126207828521728515625f,-1.98419997738599729331099297269247472286224365234375f,0.16749998308718228035019137678318656980991363525390625f,-0.16080003244876850398981105172424577176570892333984375f,-0.43809997354224350374352070502936840057373046875f,0.6752999872460960784081862584571354091167449951171875f,-0.1462999943912028921744195031351409852504730224609375f,0.64960000087469804697093422873876988887786865234375f,0.136099983485042930198005706188268959522247314453125f,1.3923999643161895090059942958760075271129608154296875f,0.0333999841272831776706198070314712822437286376953125f,-0.381499988757073804634956104564480483531951904296875f,0.222600003292411141586626399657689034938812255859375f,-0.394100008161366144321391402627341449260711669921875f,-0.02820000278130174731217039152397774159908294677734375f,0.47860000149682135894835255385260097682476043701171875f,-0.27610003604888877504208721802569925785064697265625f,0.46429998494386648388854155200533568859100341796875f,0.2564999569058414863320649601519107818603515625f,-0.2859999912753703465995158694568090140819549560546875f,1.215600029384344527016992287826724350452423095703125f,-0.15789998759552859031174421033938415348529815673828125f,-0.6805999904550608103903641676879487931728363037109375f,-0.61949997740983964167327258110162802040576934814453125f,-0.5853999792322517325970920865074731409549713134765625f,-0.82169998003095379157656452662195079028606414794921875f,-0.37059997385889265064662367876735515892505645751953125f,1.262899999535083583879213620093651115894317626953125f,0.3619000071421265563031965939444489777088165283203125f,-0.606100015597045516102525652968324720859527587890625f,1.04110004258677424360257646185345947742462158203125f,0.28770001682937174081189368735067546367645263671875f,0.22820000646933913390768111639772541821002960205078125f,-0.1876000071108341327175139667815528810024261474609375f,0.1494999981597062987948021373085794039070606231689453125f,-0.714800024521351051731699044466949999332427978515625f,-0.0446999927304683186690681395702995359897613525390625f,0.78519999142289176319309262908063828945159912109375f,1.123700028500705716538732303888536989688873291015625f,-0.513499991238117292624565379810519516468048095703125f,-0.16049998820573119218124702456407248973846435546875f,0.272599982374906790738577910815365612506866455078125f,-1.30870004656836425027677250909619033336639404296875f,0.698200023102760525262056034989655017852783203125f,0.147899988321959885428924508232739754021167755126953125f,-0.6171000165581703722494921748875640332698822021484375f,0.6230000304505234343110942063503898680210113525390625f,-0.1563000214368110984963777809753082692623138427734375f,-0.052600005825608864284959054202772676944732666015625f,-0.04499998420476902794007401098497211933135986328125f,-0.5554000016212459200914963730610907077789306640625f,-0.392899989253282289070057231583632528781890869140625f,-0.86910000067353276875792289501987397670745849609375f,-0.10569997094869609810530164395458996295928955078125f,0.47099995994567933621510746888816356658935546875f,0.5953999948412178611079070833511650562286376953125f,0.24879999871253932752779292059130966663360595703125f,0.0296000269889831191250095798750407993793487548828125f,-1.965299973946809330982432584278285503387451171875f,-0.39170000756084932813649857052951119840145111083984375f,-0.79730002258420018534934570197947323322296142578125f,-0.5367999947786330494636786170303821563720703125f,0.610399976214766493143315528868697583675384521484375f,-0.8832000204205510129895628779195249080657958984375f,0.946299956616759363470237076398916542530059814453125f,0.28679999746084217093766710604541003704071044921875f,0.85750000894069700052568805404007434844970703125f,0.276000026851892954482536879368126392364501953125f,0.0736999793827533000722951328498311340808868408203125f,-0.71800003123283406836208087042905390262603759765625f,-1.19009996806979234662549060885794460773468017578125f,-0.77299999651312845116990501992404460906982421875f,0.86229997728467022710674427798949182033538818359375f,-0.885600009435414836644895331119187176227569580078125f,1.086600006932019635286224001902155578136444091796875f,0.6671000067234038599650602918700315058231353759765625f,-0.447899989551305832691241448628716170787811279296875f,1.12300006952881847865910458494909107685089111328125f,-0.4150999665796757387425941487890668213367462158203125f,-0.2901999799489975995214763315743766725063323974609375f,-1.1011999775946137436477556548197753727436065673828125f,-1.44610002204179810547657325514592230319976806640625f,-0.219100017958879789148340933024883270263671875f,-0.887600016349554099548413432785309851169586181640625f,-0.30010001262426388990434134029783308506011962890625f,0.3798000200510021073796451673842966556549072265625f,0.91530001326724885757357697002589702606201171875f,-0.017999990999698400884199145366437733173370361328125f,0.4096000069469212245820699536125175654888153076171875f,0.2669000333309174965989996053394861519336700439453125f,0.6798000131592154549053930168156512081623077392578125f,0.3678000508368004517478766501881182193756103515625f,0.0822999956130983090218933284631930291652679443359375f,0.625099970155954309802837087772786617279052734375f,-0.507300002896041046795971851679496467113494873046875f,0.890400026655196796809832449071109294891357421875f,-0.3039999895021316955734391740406863391399383544921875f,1.815799963009357753662698087282478809356689453125f,-0.102699989336729469169284811869147233664989471435546875f,-0.81549997460842149621385033242404460906982421875f,0.313899992617219847357290518630179576575756072998046875f,0.64579998834133167662230334826745092868804931640625f,1.3805000723227862646780295108328573405742645263671875f,-0.0216000192999843410035509805311448872089385986328125f,-0.7486000106237826745569918784894980490207672119140625f,0.04599997905641772799611999289481900632381439208984375f,-0.742100008884073236714584709261544048786163330078125f,-0.71540001516267626158196435426361858844757080078125f,-0.87640000999569878104011877439916133880615234375f,0.0625000131130217795316639239899814128875732421875f,0.62299997192621248132127220742404460906982421875f,-1.250400012499093715945264193578623235225677490234375f,0.17640001655221038134868649649433791637420654296875f,-1.679699950653315010384858396719209849834442138671875f,1.1576999784588810626928534475155174732208251953125f,1.02599997574090995300366557785309851169586181640625f,-0.76669998871088029801512675476260483264923095703125f,-0.13470000474452969996264073415659368038177490234375f,0.00890000030994375634918469586409628391265869140625f,0.7845999865695831854139896677224896848201751708984375f,-0.5567999774187806227843111628317274153232574462890625f,-1.12890003927648141512918300577439367771148681640625f,1.026800032553076835739602756802923977375030517578125f,-1.382099997633695398491227024351246654987335205078125f,-0.35010005434751523267777884029783308506011962890625f,-0.09410001248270294382791689713485538959503173828125f,-0.8368000104993582599632873098016716539859771728515625f,0.36340000867694610686697842538706026971340179443359375f,0.89990003304779542236957468048785813152790069580078125f,0.2472000117480754877163917626603506505489349365234375f,-0.3761000044956801513507116396795026957988739013671875f,1.1998999840229747082531730484333820641040802001953125f,1.150700035172700808061563293449580669403076171875f,0.457800041635334997724982031286344863474369049072265625f,-0.892200018602609556950255864649079740047454833984375f,-0.429400011575221896009679767303168773651123046875f,0.12310001457631525312308440334163606166839599609375f,0.5282000098779795482784038540557958185672760009765625f,2.05050000888109185126495503936894237995147705078125f,0.139700018304586226491892375634051859378814697265625f,0.66989998618364410276626585982739925384521484375f,0.07710001203194261076845350544317625463008880615234375f,-1.49729998793900076492491280077956616878509521484375f,-0.557500048950314575080255963257513940334320068359375f,-0.440800025668740769191344952560029923915863037109375f,-0.0264000135533513091701252051279880106449127197265625f,-1.480500017523765254878753694356419146060943603515625f,-0.2698000141464174106431528343819081783294677734375f,0.1367000021219251237880598637275397777557373046875f,-0.0577000092856587709633231497718952596187591552734375f,-0.30290000475048994843518812558613717555999755859375f,0.50860001876354221650444742408581078052520751953125f,-0.148199973630905645904931589029729366302490234375f,0.56730000716149842077129505923949182033538818359375f,-0.333500028192997177711731637828052043914794921875f,1.62940003489554019466822865069843828678131103515625f,-0.6645000062435866272636530993622727692127227783203125f,-0.20860002293586799027025335817597806453704833984375f,0.4757999633073808354311040602624416351318359375f,-1.4993000112444168081538009573705494403839111328125f,0.57430000208020270946462915162555873394012451171875f,1.2111000543400649576142313890159130096435546875f,0.8142999798029659785925105097703635692596435546875f,-0.84150001129508000818901791717507876455783843994140625f,-0.2730000121966005277585054500377736985683441162109375f,0.264200017619133031843148273765109479427337646484375f,0.23700003179907813688487294712103903293609619140625f,-0.360800012332200825682093636714853346347808837890625f,0.75849999332427930909261704073287546634674072265625f,-0.4922999994874004414668888784945011138916015625f,-1.244900043627620522812549097579903900623321533203125f,0.1925999861001965740570085472427308559417724609375f,-0.032000016488134885861427392228506505489349365234375f,0.876700031626224784986334270797669887542724609375f,0.742299980562925565408249894971959292888641357421875f,0.751699994522332826107913206215016543865203857421875f,2.1443999916076652567653582082130014896392822265625f,0.083400025403499444820454300497658550739288330078125f,-0.1667999669879680624262618948705494403839111328125f,-0.82209998288154562118279500282369554042816162109375f,0.43069999520033619422321180536528117954730987548828125f,1.17060002061128631822839452070184051990509033203125f,-0.550899966357647929982022105832584202289581298828125f,-1.25260005226135273659338054130785167217254638671875f,0.73610001123845592019279138185083866119384765625f,0.8605000303387644322583582834340631961822509765625f,0.444100023174286206995020620524883270263671875f,0.87069995217323281622157082892954349517822265625f,-1.2431000276707118157304421401931904256343841552734375f,-0.62030001909732845888356678187847137451171875f,-0.856600019188225036970152359572239220142364501953125f,-1.0065000297725195110842832946218550205230712890625f,-0.472500009164213974344193047727458178997039794921875f,-0.94009997224211705457719290279783308506011962890625f,0.55959998509809361078026768154813908040523529052734375f,0.35900001591443952975168940611183643341064453125f,0.01139999250546086528856903896667063236236572265625f,0.5993999925017359231560476473532617092132568359375f,0.01770001424029465919574022336746565997600555419921875f,-0.788600037837028366993763484060764312744140625f,-0.41070001311898252094323424898902885615825653076171875f,-0.729000016212463464171378291212022304534912109375f,0.060499996308237324882384200464002788066864013671875f,-1.4327999886989601918685366399586200714111328125f,-0.0059000537157061216930742375552654266357421875f,-0.067699999320506520916751469485461711883544921875f,0.3470000426769264123549874057061970233917236328125f,0.138400009906291643346776254475116729736328125f,-0.0682999849140646375644791987724602222442626953125f,0.5827999752879140515915423748083412647247314453125f,-0.694600009405613949553526254021562635898590087890625f,-0.263300026041268342424928050604648888111114501953125f,0.1018999826669695352165945223532617092132568359375f,0.66210000970363669381413274095393717288970947265625f,0.41970002113580751057497764122672379016876220703125f,-0.31019996355772061491506974562071263790130615234375f,-0.9625000485777857051061801030300557613372802734375f,1.3428000086665150547560187987983226776123046875f,0.64630003626346610445807527867145836353302001953125f,-0.4271000079154969597539093228988349437713623046875f,-0.40510000558346537236076301269349642097949981689453125f,-0.61020003500878861046885504038073122501373291015625f,1.0998000100299714087981328702881000936031341552734375f,-0.02629998049587001229809857250074855983257293701171875f,-0.7479999942034478710439771020901389420032501220703125f,-0.39869998865872624538297941398923285305500030517578125f,-0.59290000359565009357964981973054818809032440185546875f,-1.4147999906957153815056926760007627308368682861328125f,-0.18110000319182850692101283129886724054813385009765625f,-0.8740000059306616275733858856256119906902313232421875f,-0.7993000478267671127241555950604379177093505859375f,-1.28370004450082841884750450844876468181610107421875f,0.67699999972432822747947511743404902517795562744140625f,-0.531999992590397763336795833311043679714202880859375f,0.135199989690631683370014570755301974713802337646484375f,0.23710001266524205476571296458132565021514892578125f,-0.54510000414922832323583179459092207252979278564453125f,0.171600008086860267297879545367322862148284912109375f,1.446199972230195474054426085785962641239166259765625f,-0.1558000125557190695957388015813194215297698974609375f,-0.16509997902214568199497080058790743350982666015625f,0.6847000027328729032660703524015843868255615234375f,0.4269000259920954221115607651881873607635498046875f,0.6973000164933502720288061027531512081623077392578125f,1.2330000305995347797960448588128201663494110107421875f,0.56460002587139623830836399065447039902210235595703125f,0.277500028610230486947330064140260219573974609375f,0.12020002971887644438453435213887132704257965087890625f,1.593200017179548400036992461537010967731475830078125f,-0.5891000099495051767206632575835101306438446044921875f,-0.2597000107236204780036814554478041827678680419921875f,0.0899999922141432040501030087398248724639415740966796875f,-1.0001999809175730060673004118143580853939056396484375f,0.4242999941453338141883477874216623604297637939453125f,-0.828199994343519652062468594522215425968170166015625f,0.967300006826221636657692215521819889545440673828125f,0.19699999175220772595906737478799186646938323974609375f,-0.49079997649490803013350159744732081890106201171875f,0.31389997542500491789496663841418921947479248046875f,0.2442000084921718272568114116438664495944976806640625f,-0.1904000254631041944008984501124359667301177978515625f,0.94080001128912005281534902678686194121837615966796875f,-0.52570001531690346840974825681769289076328277587890625f,-0.7760000433921820928162560448981821537017822265625f,-1.775800001454353349572556908242404460906982421875f,0.830300007213651714010893556405790150165557861328125f,0.379000012785196904729900779784657061100006103515625f,0.1954000184036790432884345136699266731739044189453125f,0.77569996964484466683842356360401026904582977294921875f,-0.4802999891273678212400000120396725833415985107421875f,-0.030700013174861651155111985644907690584659576416015625f,0.31069999717473972822290306794457137584686279296875f,0.948099994646012245169686138979159295558929443359375f,0.45180000640153838986634582397527992725372314453125f,-0.78169998174458676931664058429305441677570343017578125f,-1.6173000427007675483537241234444081783294677734375f,0.570599985444545154678053222596645355224609375f,0.16610000475645048112482982105575501918792724609375f,-0.04859998612999982814386612517409957945346832275390625f,1.71250008100643835717846741317771375179290771484375f,-1.155300014105438588529750632005743682384490966796875f,-0.4007000517502434799865795866935513913631439208984375f,-0.08210002009719585824853993472061119973659515380859375f,-0.022800004792213002247081021778285503387451171875f,0.7908000072285530368532135980785824358463287353515625f,-0.2381999870419502229879071819595992565155029296875f,0.66940000542849276143897441215813159942626953125f,0.214000023066997613341300166212022304534912109375f,-0.5794000011444087050449525122530758380889892578125f,-0.62809999736547439397327252663671970367431640625f,0.9490999976560470230424471083097159862518310546875f,-1.1388999961376189418160720379091799259185791015625f,-0.0905000023432072087103961166576482355594635009765625f,-1.4389000339865685162976660649292171001434326171875f,-0.79320004370361607559658523314283229410648345947265625f,1.8432000277034934487829787030932493507862091064453125f,0.1148000226214531682700226156157441437244415283203125f,1.257100000781565807272954771178774535655975341796875f,0.3451000197768208099802222932339645922183990478515625f,0.4201999811410903618025258765555918216705322265625f,-0.80670000784024598061705546570010483264923095703125f,-0.52639999673366588694989331997931003570556640625f,-0.105600018152594454790005329414270818233489990234375f,-0.60549999606609361535447533242404460906982421875f,0.24299999263882632316580156839336268603801727294921875f,1.3001999996185293895223367144353687763214111328125f,0.52860002233982061881079062004573643207550048828125f,0.577899951553344948251833557151257991790771484375f,-1.03670000331401812587728272774256765842437744140625f,0.72999997586011833305974505492486059665679931640625f,-0.53930006272792940791305227321572601795196533203125f,-1.74580001367330428507784745306707918643951416015625f,2.39160005396381070141842428711242973804473876953125f,-1.1146999789282674431234454459627158939838409423828125f,0.081599956417083063797690556384623050689697265625f,-1.1581999545760457426268885683384723961353302001953125f,-1.48130002255439752190113722463138401508331298828125f,0.06720001696348187891771885915659368038177490234375f,-0.1276999915719034817129795555956661701202392578125f,1.37479997832775158173035379149951040744781494140625f,0.2832000204205513682609307579696178436279296875f,0.0180999806761743542438125587068498134613037109375f,-0.301099998795986589783524323138408362865447998046875f,0.5235000216737386580234669963829219341278076171875f,1.1155000463575130975613092232379131019115447998046875f,0.9209000439181931341181552852503955364227294921875f,-1.1741000207901002294619274834985844790935516357421875f,-1.4855000047087665304701431523426435887813568115234375f,0.816299986717104797406818761373870074748992919921875f,-0.251000018320978401931142798275686800479888916015625f,0.22880000985115767075939174901577644050121307373046875f,-0.3351999439813189152204131460166536271572113037109375f,-1.4481000233493741546908495365642011165618896484375f,-1.030299982757121757259710648213513195514678955078125f,-0.38340002050474286665604495283332653343677520751953125f,-0.655200004014372172633784430217929184436798095703125f,1.495900031214952630165271330042742192745208740234375f,-0.5820000110119580849499243413447402417659759521484375f,-0.03780001149773593027703100233338773250579833984375f,-0.47880003179311803052087270771153271198272705078125f,0.56469995251595950236378484987653791904449462890625f,-0.28929999943524642436187832572613842785358428955078125f,-1.226299998787045097969894413836300373077392578125f,0.79610002235844723461610783488140441477298736572265625f,1.94919997112751008927489237976260483264923095703125f,-0.087899973458051672281499122618697583675384521484375f,0.207000006653367929487785659148357808589935302734375f,0.392999974291770914902599542983807623386383056640625f,-0.283699998512119133664555192808620631694793701171875f,0.56860002163201583957885532072396017611026763916015625f,-0.6568000511795284435123676303192041814327239990234375f,-0.7669000351563102224616841340321116149425506591796875f,0.4527999730527394461887524812482297420501708984375f,-0.16259999571144578833781224602716974914073944091796875f,-0.2206000226974484945685617276467382907867431640625f,-0.1456000015378000889132437123407726176083087921142578125f,-0.36979994792938253311831431346945464611053466796875f,0.853000030264259745393928824341855943202972412109375f,1.241200016337632661134193767793476581573486328125f,-0.7446000296711918764458459918387234210968017578125f,-0.8495999807208785004064566237502731382846832275390625f,1.84820002258121984795025127823464572429656982421875f,0.2388999950200314525972089541028253734111785888671875f,-0.445100002379715586897646062425337731838226318359375f,-0.186599990764259704434380182647146284580230712890625f,-0.58629999739006100067939541986561380326747894287109375f,0.64499999772757210525497839626041240990161895751953125f,-0.1516000554911796083246144917211495339870452880859375f,-0.3209999920390540495418463251553475856781005859375f,1.8964000737726696144136440125294029712677001953125f,-0.68809997620582663557797786779701709747314453125f,-0.380800006073712893339688889682292938232421875f,-0.536800005954504388938630654592998325824737548828125f,-0.40860001250505462166984216310083866119384765625f,-1.163900039313733980606002660351805388927459716796875f,0.656200003373622831048805892351083457469940185546875f,0.1255999972164627553894433731329627335071563720703125f,0.57320000775456403374619185342453420162200927734375f,-0.07189999019205550734312737404252402484416961669921875f,-0.63110001045614472214850820819265209138393402099609375f,0.3477999451130628916217801815946586430072784423828125f,0.9698000299230218335111430860706605017185211181640625f,1.0934999858364469194071944002644158899784088134765625f,-0.590000032186509315579314716160297393798828125f,0.46700000009685727064834281918592751026153564453125f,1.21980004746913994750912024755962193012237548828125f,0.86070001043677313301571984993643127381801605224609375f,0.414499996632337275315194347058422863483428955078125f,0.53250002160668385986497241901815868914127349853515625f,-0.56530000106692313011080841533839702606201171875f,0.0209000096082689879040117375552654266357421875f,0.21409998204708102775839506648480892181396484375f,-1.0361000172175465205981481631170026957988739013671875f,-0.4897000044465062984500036691315472126007080078125f,0.29439999566823227272749363692128099501132965087890625f,-0.235899989901482864951276496867649257183074951171875f,0.040500008732080539086695125661208294332027435302734375f,0.819900007492303739553562991204671561717987060546875f,-1.18380000156238640585115717840380966663360595703125f,-0.859600047850609172428448800928890705108642578125f,-0.1419000099174685392000583306071348488330841064453125f,0.065699948298931332146821659989655017852783203125f,-0.93500000182539222404187739812186919152736663818359375f,-0.62550000366568525578259141184389591217041015625f,0.20109999339431527953792055996018461883068084716796875f,-0.892399979627132466930561349727213382720947265625f,0.5124999984540046238379318310762755572795867919921875f,-0.566600010657310804162989370524883270263671875f,0.46380001644492185874923961819149553775787353515625f,0.21269999365806580016169391456060111522674560546875f,0.812000011384486963805784398573450744152069091796875f,-0.59810001748204211224901882815174758434295654296875f,-0.607500038407743137014449530397541821002960205078125f,-1.600599979037046427521318037179298698902130126953125f,-0.579099992217122494508885210962034761905670166015625f,-0.06469991701394384175927143587614409625530242919921875f,-0.458800050270557591147735365666449069976806640625f,0.2200000351667394937749122618697583675384521484375f,0.307200029778481820130764390341937541961669921875f,0.5538999880909916129212433588691055774688720703125f,0.29490000719428000053312644013203680515289306640625f,1.15099997574090995300366557785309851169586181640625f,-0.31049998965859426647284635691903531551361083984375f,0.265299986761808437307763597345910966396331787109375f,0.430600001835822698836864219629205763339996337890625f,-0.11350001038610901815673059900291264057159423828125f,-1.0408999969422811915364945889450609683990478515625f,1.4586000059112909088554488334921188652515411376953125f,-0.0483000004112723591020994717837311327457427978515625f,0.0605999883502723601935713304555974900722503662109375f,-0.42080000391304483997600982547737658023834228515625f,0.70889998626559957983772619627416133880615234375f,0.5047999924838535434901132248342037200927734375f,-0.053200006189942694589944949257187545299530029296875f,0.2271000104486946558068893864401616156101226806640625f,-0.4838999979630111969441941255354322493076324462890625f,-0.1511999901860949790943777770735323429107666015625f,0.87279998009353876309290853896527551114559173583984375f,0.69690002751946433789953516679815948009490966796875f,-0.42089999344050876839418151575955562293529510498046875f,-0.79080002529621129525594369624741375446319580078125f,0.80280000151395736196491270675323903560638427734375f,-0.9959999906420708981613643118180334568023681640625f,-0.2371000050470233322386093277600593864917755126953125f,0.15339999470710807827344979159533977508544921875f,0.378800029017776151984975285813561640679836273193359375f,0.20840000841617634108615675359033048152923583984375f,-0.73219998345449510690485794839332811534404754638671875f,0.4786999920487406967595234164036810398101806640625f,0.3119000127486881712712829539668746292591094970703125f,1.177099998974799888173947692848742008209228515625f,-0.429000004738569418183402603972353972494602203369140625f,0.58710000791549621368403677479363977909088134765625f,-0.538900007425248350756419313256628811359405517578125f,0.1568000144481658253425848670303821563720703125f,0.560600013179331835289076479966752231121063232421875f,0.4705999961733819958453750587068498134613037109375f,0.0605000108741222675234894268214702606201171875f,0.41519999246597283359960783855058252811431884765625f,-2.247099998378752072625275104655884206295013427734375f,-0.53889996082186808479264072957448661327362060546875f,-0.116900031617283683971209029550664126873016357421875f,-0.371299982023239660833269226714037358760833740234375f,-0.7414000213205806932847963253152556717395782470703125f,0.065800031480192533450690461904741823673248291015625f,0.929899957573412816458358065574429929256439208984375f,-1.35109999805092773073056378052569925785064697265625f,-0.468900046131015546535536486771889030933380126953125f,-0.997599979543684955984872431145049631595611572265625f,0.0887000094830991958616550618899054825305938720703125f,0.721100023308396398391550974338315427303314208984375f,0.458500030279159975776792634860612452030181884765625f,-0.71749999240040818193619998055510222911834716796875f,-0.52270002971887574716447488754056394100189208984375f,0.47120000069141365628411222132854163646697998046875f,-0.8032999883413314279323458322323858737945556640625f,0.198500010125339354516427192720584571361541748046875f,-0.075000023990869468804021380492486059665679931640625f,0.319300002843886598924427744350396096706390380859375f,0.496100001217425035093810947728343307971954345703125f,0.56810000766590196263905454543419182300567626953125f,2.032599964996427477359475233242847025394439697265625f,0.97569998242259092791073271655477583408355712890625f,1.37539995751380939026375926914624869823455810546875f,0.7069000036031007905279466285719536244869232177734375f,-0.96850003862380962260658634477294981479644775390625f,0.693699985641241578804283562931232154369354248046875f,0.1018999606132506841049689683131873607635498046875f,-0.37269998412132299137056179461069405078887939453125f,0.3838999782562257934159788419492542743682861328125f,0.164299996492266675574001055792905390262603759765625f,-0.0439999783635141028526049922220408916473388671875f,-0.43260000860095004782124306075274944305419921875f,-0.756400007462501466903859181911684572696685791015625f,-1.0446000436410312062207594863139092922210693359375f,0.064099992477893608100458777698804624378681182861328125f,-0.3120000091306864842266577397822402417659759521484375f,-1.4384999707490209175375639460980892181396484375f,0.26790005438998376607884210898191668093204498291015625f,1.11200004342198344176040336606092751026153564453125f,0.755100023502110584416868732660077512264251708984375f,1.05999998882412871381575314444489777088165283203125f,1.5191000274956227844569411900010891258716583251953125f,-0.11019998888969428918471749057061970233917236328125f,1.6869000108301632767648925437242724001407623291015625f,0.35540000847578046005992291611619293689727783203125f,-0.527100011491774989025316244806163012981414794921875f,-0.119800007385015394589800052926875650882720947265625f,-0.284500001996755624844581689103506505489349365234375f,0.53179999686479551002094012801535427570343017578125f,-0.366800022085010279937478117062710225582122802734375f,-0.7917999833047393298812721695867367088794708251953125f,0.6378000107154255804431386422947980463504791259765625f,0.1695000135451552036869315998046658933162689208984375f,1.055900008583813498574954792275093495845794677734375f,0.24920000748634407727877260185778141021728515625f,-0.06999997599050422625310829971567727625370025634765625f,-0.0618000198125840949359144360641948878765106201171875f,0.1439000147268178209092326369500369764864444732666015625f,-0.6438000271737578383834943451802246272563934326171875f,-0.72950001414120213549807658637291751801967620849609375f,0.27360001649111531651925588448648341000080108642578125f,-0.3873000049449502046883253569831140339374542236328125f,0.30699999156594248717055961606092751026153564453125f,0.27479999464452264934521963368752039968967437744140625f,-0.485800000113248753308425875729881227016448974609375f,0.7334999611377721606686463928781449794769287109375f,1.1582000012725597315466075087897479534149169921875f,0.775499934673309798682794280466623604297637939453125f,0.51340000208318270491503199082217179238796234130859375f,-0.05750005766749399782611362752504646778106689453125f,-0.2083999665439133508471059030853211879730224609375f,0.17060001584291484277855488471686840057373046875f,-0.738600013175606751048007936333306133747100830078125f,-0.73719998142421250264533227891661226749420166015625f,-0.778999993525445244557658952544443309307098388671875f,-1.14280001753270621378533178358338773250579833984375f,-0.221899999244511203499996554455719888210296630859375f,-1.0178999747619033033885216354974545538425445556640625f,-0.693800023299455670411362007143907248973846435546875f,0.5395999828815458698727525188587605953216552734375f,0.21049999867379665108302333464962430298328399658203125f,1.09890001446604745893864674144424498081207275390625f,0.50390003696680008005159834283404052257537841796875f,0.59170001657605109102178175817243754863739013671875f,0.4378000169366591531883159404969774186611175537109375f,0.176599989423155800949416516232304275035858154296875f,0.45669997701346842777780921096564270555973052978515625f,0.119500007957219711585139521048404276371002197265625f,-0.4966999980986113172320983721874654293060302734375f,0.3247000247120859217631050341879017651081085205078125f,-0.22899996979534620322738192044198513031005859375f,-0.1680000029951334017397357456502504646778106689453125f,-1.047300026793778204847740198601968586444854736328125f,0.120899979358911335847182044744840823113918304443359375f,0.110400010524690284086091196513734757900238037109375f,-1.2910999908983702422204942195094190537929534912109375f,-0.315900000071525521860849039512686431407928466796875f,-0.024100006186962286136576949502341449260711669921875f,-0.6519000178337097661795951353269629180431365966796875f,0.46120000754594769887262373231351375579833984375f,0.688399986325204604753480452927760779857635498046875f,-0.69889998645186413028795868740417063236236572265625f,-0.20029998767450429841829873112146742641925811767578125f,-0.3081000206112864958640784607268869876861572265625f,-0.35110000935718399528440158974262885749340057373046875f,-0.0636000011801716169657083810307085514068603515625f,-1.30340004982277779532751083024777472019195556640625f,0.672900028443336850614286959171295166015625f,0.6767000066108999067893137180362828075885772705078125f,-1.1803000083684924703675278578884899616241455078125f,-0.2714000048175453372323318035341799259185791015625f,0.7350999975740908354282510117627680301666259765625f,0.8620000214055181242400749397347681224346160888671875f,-1.033900019383430457509120969916693866252899169921875f,-0.8487000144004823720678132303874008357524871826171875f,0.28649999594688413395715542719699442386627197265625f,0.46139996931552929027020581997931003570556640625f,-0.31880000228881844037687187665142118930816650390625f,-1.1027000365734096476444392465054988861083984375f,-0.93199999730288940469336012029089033603668212890625f,-1.873599997305870346053779940120875835418701171875f,-1.15139997199773791436427927692420780658721923828125f,-0.9603999952137467488455513375811278820037841796875f,0.10509999504089317934329983472707681357860565185546875f,1.42179997108578692888158911955542862415313720703125f,0.372299975049495746048933142446912825107574462890625f,0.5463000085473055378315621055662631988525390625f,-0.3557000130444765151338515352108515799045562744140625f,0.3145999962925909443356431438587605953216552734375f,-0.5468000073701144714277688763104379177093505859375f,-0.283199957537650703898179926909506320953369140625f,-0.2032999943017959054714083322323858737945556640625f,-0.9876000024914741715065247262828052043914794921875f,1.086300004374979977228576899506151676177978515625f,1.4541000216841695191760663874447345733642578125f,1.1719000313937666390984304598532617092132568359375f,0.30469999535679814783861729665659368038177490234375f,0.0675999881863589280328596942126750946044921875f,-0.560199983525276667961634302628226578235626220703125f,-0.57949998649954803653372437111102044582366943359375f,-1.552099976772070011321602578391321003437042236328125f,-0.170199931371211476971438969485461711883544921875f,1.0234999900460248767330995178781449794769287109375f,0.024899996167421445392164969234727323055267333984375f,-1.1260000123977658148533009807579219341278076171875f,-0.01060002433657647458176143118180334568023681640625f,1.0373000035107136085343881859444081783294677734375f,-0.56970002024173727050992965814657509326934814453125f,-0.528700016784668047620243669371120631694793701171875f,-0.1637000059068203228207494248636066913604736328125f,0.07380000005364395487816864260821603238582611083984375f,0.72799999884143484774057242248090915381908416748046875f,0.09809999464601271146335648154490627348423004150390625f,0.213800001674145201402410521041019819676876068115234375f,0.3416000009715556284817239429685287177562713623046875f,0.25430000450164069736302963065099902451038360595703125f,0.0847000092893839262586652694153599441051483154296875f,0.675700024387985376250753688509576022624969482421875f,0.168100010553002388036247793934307992458343505859375f,-0.57830001799464270106909680180251598358154296875f,1.01700003239512515307296780520118772983551025390625f,1.18160003800094148829202822525985538959503173828125f,-0.49750002533197423826294425452942959964275360107421875f,0.568800021585821991010334386373870074748992919921875f,0.023699991489947003220351007257704623043537139892578125f,-0.389499987896531896236496095298207364976406097412109375f,0.368099999209493322638309109606780111789703369140625f,-0.1255000145435329894638698533526621758937835693359375f,-0.415600001053512191351302362818387337028980255126953125f,0.074299933758378511328146487358026206493377685546875f,0.2592999963432551790987190543091855943202972412109375f,1.02319997877180579592959475121460855007171630859375f,0.808100008392333979401200849679298698902130126953125f,0.70889999081045385054267171653918921947479248046875f,0.9235000107958912796135564349242486059665679931640625f,0.0573000145748252531774369344930164515972137451171875f,1.315700027573108332745732695912010967731475830078125f,0.117300040912628134037731797434389591217041015625f,-0.3953999924570321677919082503649406135082244873046875f,-0.3870000098943713329191496086423285305500030517578125f,-0.6750999849453569456869672649190761148929595947265625f,-0.55659998778402808294885062423418276011943817138671875f,0.1198999997250735638498753132807905785739421844482421875f,-0.0589999811574817289994143720832653343677520751953125f,0.68039999905079551201225740442168898880481719970703125f,-0.3480999731883407388721707320655696094036102294921875f,-0.142099994839727628015424443219671957194805145263671875f,-0.4562999993085863348341035816702060401439666748046875f,-0.682300001648068477066999548696912825107574462890625f,0.13419999310672270365074609799194149672985076904296875f,-0.1593000042781234082411856434191577136516571044921875f,0.54019996799081593241709242647630162537097930908203125f,0.08989997191578158464153602835722267627716064453125f,0.5519999899268146492659070645458996295928955078125f,1.50780002863407158741892999387346208095550537109375f,0.644700029703975108219538014964200556278228759765625f,0.22370000221878283408472043447545729577541351318359375f,-0.23650000451505182041955777094699442386627197265625f,-1.0634000145070252596468662886763922870159149169921875f,-0.29370000857040279296228391103795729577541351318359375f,-0.4821000018991530300382919449475593864917755126953125f,-0.575800033044815418037387644289992749691009521484375f,-0.249199994354694698284191645143437199294567108154296875f,0.4718000174283982772749368450604379177093505859375f,0.5065000288411976026026195540907792747020721435546875f,-0.64150001502037046208215542719699442386627197265625f,-1.771100060859323033213286180398426949977874755859375f,0.38340003702640590432793032960034906864166259765625f,-0.15710001314953025985943213527207262814044952392578125f,-0.13220000411123067340923853407730348408222198486328125f,0.203100021617114989691543769367854110896587371826171875f,0.6452999931320546611601685071946121752262115478515625f,0.158799961347877494066693770946585573256015777587890625f,0.49310001092553168433596511022187769412994384765625f,0.37679999967738975197306672271224670112133026123046875f,-1.02770001541376121423354561557061970233917236328125f,-0.453800007243454484040512397768907248973846435546875f,-0.0019999961853027770075641456060111522674560546875f,-0.78509999623298654114478267729282379150390625f,-0.3474999924749138102697543217800557613372802734375f,0.4859999814033511000843645888380706310272216796875f,-0.0055999807506796894784883988904766738414764404296875f,-0.5462999653339384309447268606163561344146728515625f,0.3687999993085855354735258515574969351291656494140625f,-1.9170999992728230409966272418387234210968017578125f,0.71319999493956576230857535847462713718414306640625f,-0.3738000085286807827600341624929569661617279052734375f,1.175799988844245635988272624672390520572662353515625f,0.599899998160452252449204024742357432842254638671875f,-0.6323000062674284382779887891956605017185211181640625f,0.7901999689221383960102684795856475830078125f,-0.66519999805390828129247893230058252811431884765625f,0.372899990892410215792551753111183643341064453125f,-0.05069999918639656044661023770458996295928955078125f,-0.21790000817775734276438015513122081756591796875f,-0.8617000322222718722287027048878371715545654296875f,-1.503500004649162224978908852790482342243194580078125f,-1.58990000674724551998906463268212974071502685546875f,1.16300000619888255215528261032886803150177001953125f,-0.4134000094592575891283559030853211879730224609375f,0.67699999529123289221388404257595539093017578125f,1.72090007994175042682627463364042341709136962890625f,0.076899959942699158688128591165877878665924072265625f,0.84029998283162721239847314791404642164707183837890625f,0.417300008372217534624581958269118331372737884521484375f,0.942599976097792335849590017460286617279052734375f,-0.32539996943473870061325214919634163379669189453125f,0.73060004162192360155358983320184051990509033203125f,0.0177000259935855552839711890555918216705322265625f,-0.839899980819225344674805455724708735942840576171875f,-0.36030000658035277183444122783839702606201171875f,-0.36829999430179594099854512023739516735076904296875f,0.1937999911129468788573149140574969351291656494140625f,0.63249999001622214667150956302066333591938018798828125f,-0.6861000090777877868930545446346513926982879638671875f,-1.145300014925002773935602817800827324390411376953125f,0.39350001873075957092851240304298698902130126953125f,1.3107000218734146113064298333483748137950897216796875f,-0.6885000252872706116846757140592671930789947509765625f,0.008000012475997186900400492959306575357913970947265625f,1.2631000084668395633258342058979906141757965087890625f,0.56690001114681354277280433962005190551280975341796875f,-0.423000013276934760941827562419348396360874176025390625f,0.0160000010356304844805208631441928446292877197265625f,-0.484199999234825317895314356064773164689540863037109375f,-0.1252999696068461676912164648456382565200328826904296875f,0.062899955129622764360419751028530299663543701171875f,-0.1854000050112603048546588979661464691162109375f,0.635800017994642363561297315754927694797515869140625f,-0.0007000278711313256962966988794505596160888671875f,-1.10220002478659129252491766237653791904449462890625f,0.134099970573186677569310631952248513698577880859375f,1.164499981284141671977749865618534386157989501953125f,0.645700031596422530100198855507187545299530029296875f,0.09570002764761464941756230473401956260204315185546875f,-0.51580002157092064152266175369732081890106201171875f,-0.21150003550946738783977707498706877231597900390625f,0.632899966715276374173981821513734757900238037109375f,0.48409999661296632122997607439174316823482513427734375f,0.882300019343197572396775285596959292888641357421875f,-0.034900015259534455935863661579787731170654296875f,-1.9920999725624917164878979747300036251544952392578125f,-0.08909999381899780246385489590466022491455078125f,-2.02050001692771896699696299037896096706390380859375f,0.0016000005245209170112730134860612452030181884765625f,-0.6132000096917149534192503779195249080657958984375f,0.7358000120341774419330249656923115253448486328125f,-0.5563000189781186488602315876050852239131927490234375f,-0.8522000419229269763121692449203692376613616943359375f,-0.079400034597516100376424219575710594654083251953125f,0.27189997730255111019204150579753331840038299560546875f,0.797600006589293197833967496990226209163665771484375f,-0.05150004221498993839389868298894725739955902099609375f,-1.196400001055001549588041598326526582241058349609375f,-0.4597999995246524473913041219930164515972137451171875f,0.47329998603165124304581468095420859754085540771484375f,-0.5858000363044444380733466459787450730800628662109375f,1.5066999771066011959419483901001513004302978515625f,0.6701000013552611545009085602941922843456268310546875f,0.2281999854028224916646649944595992565155029296875f,0.089099941366910240247989349882118403911590576171875f,0.02499998673796621773135484545491635799407958984375f,0.2807999941527843379418527547386474907398223876953125f,-0.156999999314546556661298382095992565155029296875f,0.5722000246189538064101043346454389393329620361328125f,-0.4954000101149083068463596646324731409549713134765625f,-0.6041999941311775312868803666788153350353240966796875f,0.36870000873804098517894090036861598491668701171875f,-0.95630002549737724848455400206148624420166015625f,0.59870002408623712586432930038427002727985382080078125f,-0.0335999963372939358663415987393818795680999755859375f,0.08999998915940465860074937154422514140605926513671875f,-0.5078000044569359605617364650242961943149566650390625f,-0.7931000086903576828945006127469241619110107421875f,0.04599995573610049515167474964982829988002777099609375f,-0.42439998564720138318762110429815948009490966796875f,0.1229000073395668246600820339153870008885860443115234375f,-0.337000008106231430105026447563432157039642333984375f,-0.2788999911829830313791944718104787170886993408203125f,0.033499995559453932258975328295491635799407958984375f,-0.51009999802112560018940712325274944305419921875f,-0.80459998265802834538362731109373271465301513671875f,-1.9485999984979631705073188641108572483062744140625f,0.160000007301568825113236016477458178997039794921875f,-1.04799999055266379599515857989899814128875732421875f,1.1744999908953903666741780398297123610973358154296875f,-0.0648999802231788880391150087234564125537872314453125f,-0.851500031299888604507941636256873607635498046875f,0.261299986419081253785634544328786432743072509765625f,0.168799974200129820900428967433981597423553466796875f,0.237599969410897671195925795473158359527587890625f,-1.20889999382793877913400137913413345813751220703125f,-0.8047000294804576014939812012016773223876953125f,-0.8679999865293506200458750754478387534618377685546875f,0.07800002348422996334420531638897955417633056640625f,0.12829999966621397078370137023739516735076904296875f,0.29240001926422110045677982270717620849609375f,0.525700001943111061564195551909506320953369140625f,0.97040000504851331442068840260617434978485107421875f,0.289199996459483799071676912717521190643310546875f,-0.2434000225722787735804786279913969337940216064453125f,-0.18710000895857792357901416835375130176544189453125f,0.5298999789565801421531432424671947956085205078125f,1.44010001772791174090571075794287025928497314453125f,-0.2687999798253175587348096087225712835788726806640625f,1.08269999727159671465415158309042453765869140625f,0.43800003063678705217398601234890520572662353515625f,0.277299989503621446829129126854240894317626953125f,-0.20190001276731539547881766338832676410675048828125f,0.03730000455379478552231375942938029766082763671875f,-0.79719999341964697947560125612653791904449462890625f,-1.315499998897313727042046593851409852504730224609375f,-0.3009000055849551102937766700051724910736083984375f,-0.6409000027537345989259165435214526951313018798828125f,-0.2196000328004366064504893074627034366130828857421875f,-0.56629999919682705300516545321443118155002593994140625f,1.159700019049644037494317672098986804485321044921875f,-0.7155000418871637624107506781001575291156768798828125f,-0.01879997114539133651334168462199158966541290283203125f,0.1077999906733635437916518640122376382350921630859375f,-0.608999989613890679862606702954508364200592041015625f,-0.94180003484413010728104609370348043739795684814453125f,0.123400024956465070857802857062779366970062255859375f,-0.232900055377186332439265470384270884096622467041015625f,0.48740002194643061717016507827793247997760772705078125f,0.82640001929029816007954423184855841100215911865234375f,-0.96460003242790737232326137018389999866485595703125f,-0.76910000853389448582930754128028638660907745361328125f,0.796000011801719420390099912765435874462127685546875f,0.66839999979212894043456572035211138427257537841796875f,-0.3199999910593032126371326739899814128875732421875f,0.1121999576568608603110988042317330837249755859375f,-0.03660003926753940817206967039965093135833740234375f,0.40419999362826342093057974125258624553680419921875f,0.62119998966455458599966732435859739780426025390625f,1.52999998688697846915829359204508364200592041015625f,-0.01630000251233543195894526434130966663360595703125f,0.268800021138786870977810394833795726299285888671875f,1.09830000629723034677454052143730223178863525390625f,0.16019998546242764092539800913073122501373291015625f,0.465399987390636926676279472303576767444610595703125f,0.89789997360706319540213371510617434978485107421875f,-0.29840000722408266398133491748012602329254150390625f,-0.047799998310208646756791495135985314846038818359375f,0.3465999912112953662557401912636123597621917724609375f,0.61199996474385276457041982212103903293609619140625f,0.1869999955892562848447369105997495353221893310546875f,0.9422999971032144372884431504644453525543212890625f,-1.0277000516235830218647606670856475830078125f,-0.3250000174343588010827943435288034379482269287109375f,0.552599987739323950108882854692637920379638671875f,-0.553600002223253451205664532608352601528167724609375f,0.467300005671381857297319584176875650882720947265625f,0.79650000086426719914101113317883573472499847412109375f,0.957300004479288535463865628116764128208160400390625f,0.0402000024497508601228901170543394982814788818359375f,1.0453000253558162313538559828884899616241455078125f,0.011000014185905460095682428800500929355621337890625f,1.514100058862567710349367189337499439716339111328125f,0.86370001767873783027340550688677467405796051025390625f,0.142399994453787481774043044424615800380706787109375f,-1.1212000038206577645638617468648590147495269775390625f,0.011899991309642476977614933275617659091949462890625f,0.37390000492930397513191564939916133880615234375f,-0.018600004607439046822037198580801486968994140625f,-0.1267000003337859226348882657475769519805908203125f,-0.1750999761164191426132674678228795528411865234375f,0.4493999974749984804844871177920140326023101806640625f,-0.6311000217437747750182097661308944225311279296875f,-0.5693000242270531696675561761367134749889373779296875f,0.0477000054299825393400169559754431247711181640625f,-0.0256999929279089212030839917133562266826629638671875f,1.1134999459385870324013012577779591083526611328125f,-1.0915000122822824391022322743083350360393524169921875f,0.5139999998211859377761356881819665431976318359375f,0.3920999978199597446604229844524525105953216552734375f,0.791000009417533789246590458787977695465087890625f,-1.062699993378668050780788689735345542430877685546875f,-1.21900000786781248507395503111183643341064453125f,-0.3638000123098492633033629317651502788066864013671875f,-0.12180001087188685460205306299030780792236328125f,0.206800010983645865536573182907886803150177001953125f,0.5849999648332602220079934340901672840118408203125f,1.05990003163218471371465057018212974071502685546875f,0.014499982178211467953587998636066913604736328125f,-0.643200012224912853753266972489655017852783203125f,-1.224199976342916773575097977300174534320831298828125f,0.15620001514554093802189527195878326892852783203125f,-1.643299985659123052528229891322553157806396484375f,0.9064999680817127369891750277020037174224853515625f,-0.254600007468461786430680149351246654987335205078125f,0.96909999575614858002836626837961375713348388671875f,-2.10460001029968157837402031873352825641632080078125f,-0.08109997584819694793623057194054126739501953125f,0.95830002559423377306302427314221858978271484375f,-0.1840999903917310831502618384547531604766845703125f,-0.703999987602234256200972595252096652984619140625f,0.2253999882102011298457000521011650562286376953125f,0.352099995994567649404416442848742008209228515625f,0.05020001064538970325656919158063828945159912109375f,0.0945999892890456717253755414276383817195892333984375f,0.03369999324083305936028409632854163646697998046875f,0.9772000199809667275729907487402670085430145263671875f,-0.45959998131692392231428812010562978684902191162109375f,0.4923000108867883195529202566831372678279876708984375f,-0.1475999979838726883230037856264971196651458740234375f,0.143599965752661429263525860733352601528167724609375f,1.24170001806616792094928314327262341976165771484375f,2.172099985712765857215345022268593311309814453125f,0.012800014626979105969439842738211154937744140625f,-0.0869999672770498388985060955747030675411224365234375f,-1.406700020748376633861198570230044424533843994140625f,0.18539999446868904442453640513122081756591796875f,0.1183999974638221264200410587363876402378082275390625f,-0.64720001189708742383999151570606045424938201904296875f,-0.03349998795986142141600794275291264057159423828125f,0.086199999797344162288936786353588104248046875f,0.441699992883204828331145108677446842193603515625f,-0.415600012154877074266323688789270818233489990234375f,-1.4177000037908555896137841045856475830078125f,0.439999986551701749437626176586491055786609649658203125f,-0.512600009196996442284444128745235502719879150390625f,-0.34990000302195534143123722969903610646724700927734375f,0.3428000024080279928284653578884899616241455078125f,-0.9613999983914187641431681186077184975147247314453125f,-0.964399955844879031019445392303168773651123046875f,1.323899997720867194317406756454147398471832275390625f,0.072400012856721929210834787227213382720947265625f,0.30800001316517600091771100778714753687381744384765625f,0.393400003573298295833637894247658550739288330078125f,0.358200006413459615117744760937057435512542724609375f,0.7640000075697894743598226341418921947479248046875f,-0.323000006794929517273118335651815868914127349853515625f,-0.5263999901771541090056416578590869903564453125f,-0.089699985074997101719418424181640148162841796875f,1.92480001319646820689968080841936171054840087890625f,-0.3777999913439151757899026051745750010013580322265625f,0.1590999666988850425042301139910705387592315673828125f,-0.52070002831816697419498041199403814971446990966796875f,-0.5557999970957634428003757420810870826244354248046875f,0.354700055706501427010834959219209849834442138671875f,1.4469000006973740113380699767731130123138427734375f,-0.21039999238252704572005313821136951446533203125f,0.20799997252225921329227276146411895751953125f,-0.82460002683997135619620166835375130176544189453125f,0.76369999547600730949170610983856022357940673828125f,0.4056999902456992490584752886206842958927154541015625f,-0.8416000060379502656360273249447345733642578125f,-0.1235999996900556396894899080507457256317138671875f,-0.0302999937653538609083625487983226776123046875f,0.29399999389797437654436862430884502828121185302734375f,0.02609999957829689520849569817073643207550048828125f,-0.6578000040099023504325259636971168220043182373046875f,0.59240000201612696972830462982528842985630035400390625f,0.9389000064939254652784939025877974927425384521484375f,-0.962400014235079037661080292309634387493133544921875f,0.2738000195741658870929313707165420055389404296875f,0.61730001450032023768699218635447323322296142578125f,-0.4523000319719319151090530795045197010040283203125f,1.6743000194773076660936794723966158926486968994140625f,-0.651599945092200538709903412382118403911590576171875f,-0.3127000123217700267019836246618069708347320556640625f,1.4595000148117538429914930020458996295928955078125f,-1.272100024046003863276155243511311709880828857421875f,-0.38940001369863741498278386643505655229091644287109375f,0.258500010125338963717922524665482342243194580078125f,-0.6535000069960952284731092731817625463008880615234375f,-0.063800000239908616350703596253879368305206298828125f,-1.542800000843405516803841237560845911502838134765625f,-0.47730002176463581786691747765871696174144744873046875f,0.118299974706768562526804089429788291454315185546875f,0.51739998888447813474300573943764902651309967041015625f,0.3420999876499166703069931827485561370849609375f,-0.557200018956512277412684852606616914272308349609375f,-1.93280006380081204042653553187847137451171875f,0.07369996166899717149334492205525748431682586669921875f,-2.281299987983704369298720848746597766876220703125f,0.047299974900484098583319791941903531551361083984375f,1.8144999741315839258959385915659368038177490234375f,0.39530002615675308152987099674646742641925811767578125f,-0.017899996219575609046614772523753345012664794921875f,-0.00510001254975811502134774855221621692180633544921875f,-0.966599991434812455537439745967276394367218017578125f,0.46369999605342737680757636553607881069183349609375f,0.741999985009431650695432836073450744152069091796875f,-1.08340000465363228698834063834510743618011474609375f,0.35479998611360785698565223356126807630062103271484375f,1.458599997305868978259013601928018033504486083984375f,-0.4378000098586076749285211917595006525516510009765625f,-0.273399938082693694241243065334856510162353515625f,1.435300021481513255139361717738211154937744140625f,-0.759200009274483278431944199837744235992431640625f,1.5664999572038649233718388131819665431976318359375f,-1.20239997470974824267386793508194386959075927734375f,-0.62390000180006044416813892894424498081207275390625f,2.648000047326088068899707650416530668735504150390625f,0.36319998093247374271186345140449702739715576171875f,-0.560699973891674918746730327256955206394195556640625f,0.594699977475404484295040674624033272266387939453125f,-0.520899998320639046056612642132677137851715087890625f,0.07569997642487280575807062632520683109760284423828125f,-0.1457000174403191294913995079696178436279296875f,0.72909998916238494270913861328153870999813079833984375f,0.332500014305114888202297152020037174224853515625f,0.3032000540226695850520854946807958185672760009765625f,0.593100033128262094095362044754438102245330810546875f,-0.490499999344348847074570585391484200954437255859375f,-0.401900007700919825737173596280626952648162841796875f,-0.2907999602526423199577720879460684955120086669921875f,0.054900022244453783315520922769792377948760986328125f,0.7476999987617134468820268011768348515033721923828125f,1.1532000010490417452047040569595992565155029296875f,-0.7253999607175585229157377398223616182804107666015625f,-0.1761000202536582293078026850707828998565673828125f,-0.5513000096648930092868567953701131045818328857421875f,0.693300028127432188540524293784983456134796142578125f,-0.52960003458857585201258189044892787933349609375f,0.523300015461444889552922177244909107685089111328125f,-0.235000006854534149169921875f,-0.9860000222325329133354898658581078052520751953125f,-0.7545000471845273448678881322848610579967498779296875f,0.55209998824596340227799373678863048553466796875f,0.54579999901428799535807456777547486126422882080078125f,-1.0414000231087214842773391865193843841552734375f,-0.8481000470049682160578186085331253707408905029296875f,-1.11330003587603609815914751379750669002532958984375f,-0.26500000797212119874046720724436454474925994873046875f,-0.95820005558729093309011659584939479827880859375f,-0.5021000122368339813050397424376569688320159912109375f,0.103699964928626542359779705293476581573486328125f,-0.4360000156573951013427858924842439591884613037109375f,-0.3546000021040445204789648414589464664459228515625f,-0.12530001051053385463518452525022439658641815185546875f,0.27720000131726241221485906862653791904449462890625f,-0.400600016438961148423913982696831226348876953125f,0.179399981474876568654508446343243122100830078125f,-1.00620003317594619574038006248883903026580810546875f,-0.284899993932246076155934133566915988922119140625f,1.680600029402970818637186312116682529449462890625f,0.68379997010231097220867013675160706043243408203125f,-1.414500014811754358134976428118534386157989501953125f,0.40610002874732042954519783961586654186248779296875f,1.055800045561791744574975382420234382152557373046875f,0.589599999719857326141436715261079370975494384765625f,-0.2328000363826756569096687599085271358489990234375f,-1.46319999061822869634852395392954349517822265625f,0.72719998269081020936255299602635204792022705078125f,0.75499999374151283149103619507513940334320068359375f,0.93920005412697804558774805627763271331787109375f,-1.1733000229120253976589083322323858737945556640625f,0.108400006926059777612181278527714312076568603515625f,-0.584300001484155284714461231487803161144256591796875f,-0.115500011675059688354139098009909503161907196044921875f,-0.014399996301531547970853353035636246204376220703125f,0.51290001780763250227579419515677727758884429931640625f,-1.2334000354617835615300691642914898693561553955078125f,0.044299985241889938691883799037896096706390380859375f,-0.090900008453429048671523560187779366970062255859375f,-0.9369000131398441677532673566020093858242034912109375f,0.49999998580664428704523061242070980370044708251953125f,-0.031999999687075675325331758358515799045562744140625f,0.3319999892562626175873674583272077143192291259765625f,1.06410002470165476751162714208476245403289794921875f,-0.93179999723732487293403892181231640279293060302734375f,-0.6858000361368061792433081791386939585208892822265625f,0.916900012767314809281060661305673420429229736328125f,-0.01799996412172888415881288892705924808979034423828125f,-0.6373000198274849648072404306731186807155609130859375f,0.57979998831152901317409487091936171054840087890625f,-0.58470001439303154189275346652721054852008819580078125f,0.3165000245571134573907556841732002794742584228515625f,0.90380002294555328301584040673333220183849334716796875f,0.970800016355514827637307462282478809356689453125f,-0.02639998926445830118581170609104447066783905029296875f,-0.38339997421801097488014420378021895885467529296875f,0.123099991647154016760623562731780111789703369140625f,-0.21010001113414755735675498726777732372283935546875f,-0.59399999888986343421493074856698513031005859375f,1.339299999956786724197854709927923977375030517578125f,0.18540002255737786374112374687683768570423126220703125f,-0.2663000205799936903616753625101409852504730224609375f,-0.53580002306103724496466611526557244360446929931640625f,-0.8314999725520613171880768277333118021488189697265625f,0.46030000497847811669061002248781733214855194091796875f,1.2649999761581423030065707280300557613372802734375f,-0.610699990338832154890269521274603903293609619140625f,0.47559999431073685105531012595747597515583038330078125f,1.7410000558346510057816658445517532527446746826171875f,-0.72120002429112817221579234683304093778133392333984375f,-0.14179999254345876380511981551535427570343017578125f,-0.36550001364946371751329934340901672840118408203125f,-0.892000030681491029582730334368534386157989501953125f,-0.33629998962283114138926976011134684085845947265625f,-0.692000027999281730473057905328460037708282470703125f,-0.8060999631077054861094666193821467459201812744140625f,1.380300021369755381073218813980929553508758544921875f,-0.027200005266070359510877096909098327159881591796875f,-0.7479000379987066349229962725075893104076385498046875f,0.81689999164491933214549135300330817699432373046875f,0.126800018545985604401238333593937568366527557373046875f,-1.502499988321214896558331020059995353221893310546875f,0.374499978154897661397626507095992565155029296875f,-0.142199972707033328589432130684144794940948486328125f,0.1938999886870380606751496088691055774688720703125f,0.77360002092421031871793957179761491715908050537109375f,0.068300022930652126706974058834020979702472686767578125f,-0.106299996700882803946086596624809317290782928466796875f,-0.042300007124245142531293595311581157147884368896484375f,-0.11219999040216210861586887403973378241062164306640625f,0.3901000137418508817432893920340575277805328369140625f,-1.70230000924766056158432547817938029766082763671875f,-0.4650000335276127660932843355112709105014801025390625f,0.8197999905467037162765109314932487905025482177734375f,1.0707000266045334502251762387459166347980499267578125f,1.068499968923628085093469053390435874462127685546875f,-0.7632000271260748203161483615986071527004241943359375f,0.105900035946070669723440005327574908733367919921875f,-0.345999957859515205882416921667754650115966796875f,0.30140003689229555305928442976437509059906005859375f,-1.2495000393986703812743144226260483264923095703125f,-0.686600026750564751409910968504846096038818359375f,-1.2748000176109373082766751394956372678279876708984375f,0.464999995045363601064991598832421004772186279296875f,-0.17390001064762461435719842484104447066783905029296875f,-0.270700003097951391595188397332094609737396240234375f,0.74240001382529730022952207946218550205230712890625f,0.62720001977607608978360076434910297393798828125f,0.175599976578354688427907603909261524677276611328125f,-0.73450001788511887212962392368353903293609619140625f,-0.22929998591244160621727132820524275302886962890625f,0.3580000238195066852853187810978852212429046630859375f,-0.129699972185493006548995253979228436946868896484375f,0.80069998313039481008246411875006742775440216064453125f,0.6453000167131417441623852937482297420501708984375f,0.549700002155452782659494914696551859378814697265625f,-1.124599997186660260695134638808667659759521484375f,0.7092000143781305165902040243963710963726043701171875f,1.249000012636184209213752183131873607635498046875f,0.1984000213988130756348482464090920984745025634765625f,-0.2437000349268318988293913207598961889743804931640625f,-0.207900013691186991149351115382160060107707977294921875f,-0.161399991760402927720718935233890078961849212646484375f,1.193500021710991898515885623055510222911834716796875f,-0.54470000861883161036303135915659368038177490234375f,-0.4117000198543070421663969682413153350353240966796875f,0.4604000167459247894186091798474080860614776611328125f,0.417300067511201522307828781777061522006988525390625f,-0.06160002257824004345820867456495761871337890625f,-1.0717000343084348568112318389466963708400726318359375f,0.6314999943822627859191243260283954441547393798828125f,0.06140004531145226707167239510454237461090087890625f,1.98809998246431351986984736868180334568023681640625f,-0.88220001547336579506009002216160297393798828125f,0.41029998840093551137897520675323903560638427734375f,0.0246999885767695825933287778752855956554412841796875f,0.630900030097365061010350473225116729736328125f,0.5634000353872783417585878851241432130336761474609375f,0.58059997754693082328003583825193345546722412109375f,0.3089999873042099665099158301018178462982177734375f,0.1608000038385386520900510731735266745090484619140625f,-0.42409999404251597088233438626048155128955841064453125f,1.1181000207602977969401081281830556690692901611328125f,0.4522000327214603654368829666054807603359222412109375f,-0.83730003625601501227038170327432453632354736328125f,0.13849997395276947287356961169280111789703369140625f,0.56410001732557990550276372232474386692047119140625f,-0.0675999935507771709808366722427308559417724609375f,0.537900048299133626272805486223660409450531005859375f,-0.55499998319894039422450759957428090274333953857421875f,-0.6032000180363652486192904689232818782329559326171875f,0.7241999982848763561804616983863525092601776123046875f,-0.66130003209114107676214189268648624420166015625f,-0.720200019660591994608012100798077881336212158203125f,-0.34530001125559184327329376174020580947399139404296875f,-1.44300000202655809289353783242404460906982421875f,-0.0134000221252439377650489404913969337940216064453125f,-1.94660002794265807324336492456495761871337890625f,0.07979995813667717374784160710987634956836700439453125f,-1.034200000035762290195862078689970076084136962890625f,0.49030001365840447657973300010780803859233856201171875f,0.59899998819827970919504878111183643341064453125f,-0.450500017505139294105021008363110013306140899658203125f,-1.566800044250488355146444519050419330596923828125f,-0.0373000278741126845005737777682952582836151123046875f,-0.911499991588294644628831520094536244869232177734375f,0.60970000386908651535122771747410297393798828125f,0.750499985039234207562230949406512081623077392578125f,-0.56879999834001060410315631088451482355594635009765625f,0.49800001379847547156032305792905390262603759765625f,0.2977000440239911949191764506394974887371063232421875f,1.6305999625712634948371260179555974900722503662109375f,0.016100055718421657502403832040727138519287109375f,1.700599976801871804354959749616682529449462890625f,-0.0809999732449646625553896228666417300701141357421875f,1.2144999563992022739711273970897309482097625732421875f,0.07870002945065557309334280944312922656536102294921875f,-0.9648000251546504646427138141007162630558013916015625f,0.29050001275539383271961924037896096706390380859375f,1.51570002548694571231635563890449702739715576171875f,-0.65409999396800966309228897443972527980804443359375f,0.7195999638080596838562996708787977695465087890625f,0.296600003392994526763004614622332155704498291015625f,-0.553300015908479547732667924719862639904022216796875f,0.3229999984130262902226604637689888477325439453125f,-0.50329997288137684474662592037930153310298919677734375f,-0.60690001101642854219875289345509372651576995849609375f,-0.2886000452503567625939240315346978604793548583984375f,0.7841000244036318012064157301210798323154449462890625f,1.0969999988302590043787176909972913563251495361328125f,-0.00399998976290227847840696995262987911701202392578125f,0.3529999807924031340888859631377272307872772216796875f,0.55539998586326788654332631267607212066650390625f,1.1643000326275825262456464770366437733173370361328125f,0.4156000457942490289298120842431671917438507080078125f,-0.38029998486191030071523755395901389420032501220703125f,-0.2764000331670046772813975621829740703105926513671875f,0.1455999762803319441673011169768869876861572265625f,1.156800022867322308428583710337989032268524169921875f,-1.51370001183003211053801351226866245269775390625f,-1.49700007367134180213952276972122490406036376953125f,0.3085999818459146393223591076093725860118865966796875f,1.0896000648379331465775976539589464664459228515625f,0.208100011409819263263898392324335873126983642578125f,0.744399997717141825859243908780626952648162841796875f,-1.12960001940801735287323026568628847599029541015625f,-0.5245999974846835556263613398186862468719482421875f,-0.303399987200647258678287698785425163805484771728515625f,-0.148900051346421147258070050156675279140472412109375f,-1.1306000243924561043940002491581253707408905029296875f,-0.478499988481402294127065033535473048686981201171875f,0.549700019440800158321280832751654088497161865234375f,0.56419998729527076619660874712280929088592529296875f,-0.21089998364299533673005271339206956326961517333984375f,1.79560005190372518057984052575193345546722412109375f,0.05200004049763184976740149068064056336879730224609375f,-0.1884000045418741109415350365452468395233154296875f,-0.41670003469958916131332671284326352179050445556640625f,-0.697299986076355793329639709554612636566162109375f,-0.16580003177821633908450849048676900565624237060546875f,-1.255600009137392003566446874174289405345916748046875f,-0.236500014461576757440752771799452602863311767578125f,-0.52029995669871531571715195241267792880535125732421875f,-0.5066000135257835434998696655384264886379241943359375f,-0.4367999695956712447042491476167924702167510986328125f,-0.19160004411041786642755369030055589973926544189453125f,0.7523000291034580566673639623331837356090545654296875f,0.8529000317215926774139234112226404249668121337890625f,-0.7459000034242870835754501968040131032466888427734375f,-0.1742000750631103045407144236378371715545654296875f,-1.2815999871134753362866831594146788120269775390625f,0.71130004363954046908702366636134684085845947265625f,-0.4719999977499245158440999148297123610973358154296875f,-0.55700006383657640895989970886148512363433837890625f,1.28659995552301520405080736964009702205657958984375f,0.5684000611662867186169023625552654266357421875f,-0.880500017821788105720770545303821563720703125f,-0.401099971079826644881904940120875835418701171875f,0.41490001501739026679160815547220408916473388671875f,-1.24109996064901384471568235312588512897491455078125f,0.0942999935120345522676643668091855943202972412109375f,-1.5347000104069703496634247130714356899261474609375f,1.10479999158978525741758858202956616878509521484375f,-0.20830005599260292825647411518730223178863525390625f,0.0011000131011009717241222460870631039142608642578125f,-1.7594000267744061005714684142731130123138427734375f,0.9821999987095588746655039358302019536495208740234375f,-0.3936000250220299534475998370908200740814208984375f,-0.541899985349177715221458129235543310642242431640625f,-1.2513999532222754851318313740193843841552734375f,-0.6425000001490115408842029864899814128875732421875f,-1.330399986720085081515208003111183643341064453125f,0.0089000046312808223092361004091799259185791015625f,0.355700007009506435906587284989655017852783203125f,-0.493099988722800475215990445576608180999755859375f,0.803199980932474577599577969522215425968170166015625f,-0.600100034379959179631214283290319144725799560546875f,0.45560001003146144427091712714172899723052978515625f,1.0161000065445897622140591920469887554645538330078125f,0.402800003451108334928676413255743682384490966796875f,1.517900013989209373477251574513502418994903564453125f,1.93090000573396647354229571647010743618011474609375f,1.1432000061154365955218281669658608734607696533203125f,-0.66210002534985523681143604335375130176544189453125f,1.08630003596544355559672112576663494110107421875f,2.02070000164508822848574709496460855007171630859375f,0.22249995723366655653308043838478624820709228515625f,-0.86400000965595236124272560118697583675384521484375f,0.1430999997496602560431711026467382907867431640625f,-0.12760000025630002795651307678781449794769287109375f,-0.327500008046627044677734375f,0.7192000271558764978863109718076884746551513671875f,1.254500028334558425768818779033608734607696533203125f,-1.12560004653930700868613712373189628124237060546875f,-0.69660001725107434378259085860918276011943817138671875f,0.040600017929077136802362701928359456360340118408203125f,-0.687000036697835270160794607363641262054443359375f,0.01360001775771413345950122675276361405849456787109375f,0.79170002026408925477340972065576352179050445556640625f,-0.696000031471252444958963678800500929355621337890625f,-1.0138000064052636872702350956387817859649658203125f,0.785800025445222871667283470742404460906982421875f,0.64889999772086726093078823396353982388973236083984375f,-0.6204999950230121186223186668939888477325439453125f,-0.44260001892000444545516302241594530642032623291015625f,0.3134000071495772754559538952889852225780487060546875f,0.474899998719244986755683157753082923591136932373046875f,-0.0695000006556512062161345966160297393798828125f,1.1491999937772749973419195157475769519805908203125f,-1.150800001752376289232415729202330112457275390625f,0.6932999811887743391025651362724602222442626953125f,0.03329999240189795983724252437241375446319580078125f,-0.025300010901689429232419570325873792171478271484375f,0.18290000076442936016718476821552030742168426513671875f,-0.2360999849751592716273762562195770442485809326171875f,-1.753799969282745951915103432838805019855499267578125f,-0.58039999074339920781540058669634163379669189453125f,0.164899988716840528724105752189643681049346923828125f,1.870699977803230495965181034989655017852783203125f,0.394699992823600798175220916164107620716094970703125f,1.400700011405348721638119968702085316181182861328125f,0.7627000060260293867742120710317976772785186767578125f,0.341999976590276144605695662903599441051483154296875f,0.0100999923586846396261762492940761148929595947265625f,0.17069997899532296514735207892954349517822265625f,0.49220004140138673420779014122672379016876220703125f,0.01500005185604091906270696199499070644378662109375f,-0.974100007975101878088253215537406504154205322265625f,-0.3985000153779985065938262778217904269695281982421875f,-0.6060999910846351923510155756957828998565673828125f,0.259999992996454099003500459730275906622409820556640625f,0.800300031353533558586832441505976021289825439453125f,-0.3213999682724477668926965634454973042011260986328125f,-0.674900002574921220599435400799848139286041259765625f,-2.11089998859763117167176460498012602329254150390625f,-1.24989997888207415854822102119214832782745361328125f,0.4944000005483626392077667333069257438182830810546875f,-0.2455000021755695538416830459027551114559173583984375f,-0.08939998996853848467480929684825241565704345703125f,0.52070001863241210937616187948151491582393646240234375f,0.380400008326768901412151535623706877231597900390625f,0.448900011932849896112429632921703159809112548828125f,-0.28040001950263981900235421562683768570423126220703125f,-0.267399992609769043827583345773746259510517120361328125f,-0.1513999914065002894147937695379368960857391357421875f,-0.502300007105618728786566862254403531551361083984375f,0.0560999905258416919906494513270445168018341064453125f,1.2894000478595497583000906161032617092132568359375f,-0.208200046497584168037064955569803714752197265625f,0.0959000039644541946159961298690177500247955322265625f,-0.29020001392364491721309605054557323455810546875f,-1.717400034966319655183042414137162268161773681640625f,0.2896000159621234804063760748249478638172149658203125f,0.855500012867153269979780816356651484966278076171875f,-0.77030002416372322926463311887346208095550537109375f,0.851400038531422875820453555206768214702606201171875f,0.44369999532699566646698485783417709171772003173828125f,-0.31149999583512533174456393680884502828121185302734375f,0.28290001223832372811983759675058536231517791748046875f,-0.2434000090122221404698166224989108741283416748046875f,-0.55899999978393299926437975955195724964141845703125f,-1.031299968463182725741944523178972303867340087890625f,0.308099999526142698247355156127014197409152984619140625f,-0.39080002753138476379035637364722788333892822265625f,-1.39920001769363810950608240091241896152496337890625f,-0.314300031733512952314413269050419330596923828125f,-1.23709999815523641331083126715384423732757568359375f,-0.53590001914501073088104021735489368438720703125f,0.217499970421195509828748981817625463008880615234375f,-0.3536000145912172598627876141108572483062744140625f,0.095900026074052124869240287807770073413848876953125f,-0.11370005508065172961096322978846728801727294921875f,-0.5524999970197674858951586429611779749393463134765625f,-0.2663000192761419970821634706226177513599395751953125f,0.145899999214708808725760036395513452589511871337890625f,0.107200006160140137723146835924126207828521728515625f,-0.69140001316219590332678990307613275945186614990234375f,0.41899998883157962836776277981698513031005859375f,-0.155799990017712219891876657129614613950252532958984375f,-0.47159999389350393261821636770037002861499786376953125f,-0.280499999791383292091495604836381971836090087890625f,-0.128599981659650808296646573580801486968994140625f,-0.9826000437676913890072682988829910755157470703125f,-0.54399997612834027904682443477213382720947265625f,-1.75980000112652756882880566990934312343597412109375f,-0.27010002067089100563634929130785167217254638671875f,0.3395000249445434548078992520458996295928955078125f,-0.0085999856829646059708238681196235120296478271484375f,-0.5841000404596332629125754465349018573760986328125f,-0.07179999276697657251133932732045650482177734375f,-0.76820001740306642634692479987279511988162994384765625f,-0.2663000106707216207979627142776735126972198486328125f,-0.532900001285970148501291987486183643341064453125f,-0.364600006425380485097775817848742008209228515625f,-0.8431000378035016229461007242207415401935577392578125f,0.408600008183717999799000608618371188640594482421875f,0.468299989421664786259924539990606717765331268310546875f,-0.8350000244379043312648036589962430298328399658203125f,0.0797000152498480163298921752357273362576961517333984375f,0.182699975031614059162166086025536060333251953125f,0.125900033971667768550872779087512753903865814208984375f,-1.2314000338375574727933781105093657970428466796875f,-1.6144000553473836223616899587796069681644439697265625f,1.235700035768747984121773697552271187305450439453125f,0.8035000091195099169993909526965580880641937255859375f,-1.3570000414848326641958919935859739780426025390625f,-0.660499998040497526829994967556558549404144287109375f,0.39389998674988735416491181240417063236236572265625f,-0.117899999367445662556974639301188290119171142578125f,0.4233000080108642038112520822323858737945556640625f,-0.7010999928355214905195680330507457256317138671875f,-0.6918000170558695760547607278567738831043243408203125f,-0.6225000080466269736234607989899814128875732421875f,0.052000007063150111008553722058422863483428955078125f,0.0822999887585638134623877704143524169921875f,-0.9653000274419785142754335538484156131744384765625f,-0.98479998756647102453598563442938029766082763671875f,-0.35250000238418532916284675593487918376922607421875f,-2.28749997243285196901751987752504646778106689453125f,-0.295300001290441205270553837181068956851959228515625f,-1.3939999840259549301890729111619293689727783203125f,-0.2021999856710436915818718262016773223876953125f,-1.1026999748826025182779630995355546474456787109375f,0.298800022554397859408936710678972303867340087890625f,-2.0118000166833400754740068805404007434844970703125f,-0.12470000161528582705017242915346287190914154052734375f,-0.76890002191662798480820129043422639369964599609375f,-0.01909999217987046193911737645976245403289794921875f,-0.3514999834299086245437138131819665431976318359375f,0.545300061714649952904210294946096837520599365234375f,-0.068500008076429086401049062260426580905914306640625f,-0.92490000793933813127978282864205539226531982421875f,0.2685000112056734611343244978343136608600616455078125f,0.0177000090062620074604637920856475830078125f,1.19899996003508579889285101671703159809112548828125f,-0.62140006512999601540059302351437509059906005859375f,-0.78510002886652952014401307678781449794769287109375f,0.89580002678632819623771865735761821269989013671875f,1.381500011891126700191989584709517657756805419921875f,-0.157900002068280809197631242568604648113250732421875f,0.427699991571902859988085765508003532886505126953125f,0.500800003242492408617181354202330112457275390625f,0.092000003039836908413917626603506505489349365234375f,0.541300016444921539005008526146411895751953125f,-0.41880000348091106499026636811322532594203948974609375f,-0.31949998052045669449938714024028740823268890380859375f,0.087599987534433376712428298560553230345249176025390625f,0.55210001247897755316529355695820413529872894287109375f,0.7061000144422056035153900666045956313610076904296875f,-0.2154999912790952210084327589356689713895320892333984375f,0.19960001372694957666453774436376988887786865234375f,-0.75980002869367579965143022491247393190860748291015625f,0.0912000106006860544738401586073450744152069091796875f,0.225299984787404650177222720230929553508758544921875f,-0.271199997040629359190688774106092751026153564453125f,0.803000015363096952825117114116437733173370361328125f,-0.47480000716149783901443015565746463835239410400390625f,-1.5090000180937350737764290897757746279239654541015625f,0.615900013985484662271119304932653903961181640625f,0.5149999941699208161338674472062848508358001708984375f,-0.229400002485513621053314636810682713985443115234375f,-0.8325999753713608786398481242940761148929595947265625f,0.91000002326443818478907132885069586336612701416015625f,-0.43910001097396011981999208728666417300701141357421875f,0.102200015622377460289271766669116914272308349609375f,0.1640999832391738522119339904747903347015380859375f,0.51009998200237749443886059452779591083526611328125f,0.422300010365247668886468090931884944438934326171875f,0.21969998835325255104322650367976166307926177978515625f,-0.08390000209808334830086096189916133880615234375f,0.1378000119447708726738710538484156131744384765625f,0.570699967558681908741391453077085316181182861328125f,-0.57179999837353856495525405989610590040683746337890625f,-0.512499978430569047560538820107467472553253173828125f,1.0235999862603843979513840167783200740814208984375f,-0.29409999454543023222896636070800013840198516845703125f,-0.1287999979674816497521305791451595723628997802734375f,-0.0743000093817709483801081660203635692596435546875f,-0.045199991590529453588942487840540707111358642578125f,-0.0727000125080348169603894348256289958953857421875f,-0.28850002884492320642806362229748629033565521240234375f,-0.03689999339580563031404381035827100276947021484375f,0.24980000306367866613754813442938029766082763671875f,0.2949999985098841204234076940338127315044403076171875f,0.3882000118523836196260390352108515799045562744140625f,0.3149000056296584748594113989383913576602935791015625f,1.29130001964867080488374995184130966663360595703125f,-0.36279999421238906531783641185029409825801849365234375f,0.0372999963954090407725061595556326210498809814453125f,0.7835000115409493925966444294317625463008880615234375f,-0.230199990398436415972582835820503532886505126953125f,-0.19140003093183022286893901764415204524993896484375f,0.76170000336989740485904576416942290961742401123046875f,0.44630001122951501457691847463138401508331298828125f,0.7543999969720840415021712033194489777088165283203125f,-0.67820004277229362088519337703473865985870361328125f,-0.85570001816675056449668090863269753754138946533203125f,0.29260002115517858012339047490968368947505950927734375f,-0.116700005064904799301217508400441147387027740478515625f,-0.01829999940544357883709380985237658023834228515625f,-0.0422999974012374935217195570658077485859394073486328125f,0.114400000399351053914642761810682713985443115234375f,1.009400039421767036884602930513210594654083251953125f,-0.3752000203311445236664667390868999063968658447265625f,-0.52930000807791943717717231265851296484470367431640625f,0.8392000059962274605140919447876513004302978515625f,-0.01279999695047695507099660972016863524913787841796875f,0.43520000319480889316992033855058252811431884765625f,1.4056000143714246775061837979592382907867431640625f,-0.66820000954270408755064636352472007274627685546875f,-0.1222999894477425630867628569831140339374542236328125f,0.14090000424385085153744512354023754596710205078125f,-1.04110001339912372486651293002068996429443359375f,0.0727999994277954698418398038484156131744384765625f,-0.6332000093005596408346491443808190524578094482421875f,-0.051500000435859061287402482776087708771228790283203125f,-0.37909998713210224519087887529167346656322479248046875f,0.7350000070780515759594209157512523233890533447265625f,0.1777999907106160293324137455783784389495849609375f,-0.4798000050753354894794711071881465613842010498046875f,-1.138499981179833309141713471035473048686981201171875f,-0.510799972695112014520191223709844052791595458984375f,0.085499995097517711428736220113933086395263671875f,0.1045999806463715486870569293387234210968017578125f,2.440399952074886247288532103993929922580718994140625f,-0.6456000251561400116173672358854673802852630615234375f,1.771999968469142316962461336515843868255615234375f,0.284700001913309019840880864649079740047454833984375f,0.668300036621094051270119962282478809356689453125f,-0.736900026774407024277024902403354644775390625f,-0.4841000318914654432234101477661170065402984619140625f,-0.54030000760480756394343870852026157081127166748046875f,0.258699996537715193323236917422036640346050262451171875f,0.3083000034473837303039545076899230480194091796875f,-0.10460001491904247217235024436376988887786865234375f,-0.015300017532706267076036965590901672840118408203125f,1.1827000028222800498411970693268813192844390869140625f,0.1055999776214360252879487234167754650115966796875f,-1.50460000731945076068996058893389999866485595703125f,-0.708700009706616729232564466656185686588287353515625f,-1.2652999409407377218172996435896493494510650634765625f,-0.75919996255934218964966930798254907131195068359375f,-0.64280000851750340729040544829331338405609130859375f,-1.71200003150105484195364624611102044582366943359375f,-0.22360001384615912911613122560083866119384765625f,-0.815700022506713917636034238967113196849822998046875f,-0.091600012743473069320998547482304275035858154296875f,0.446999991510062866240104995085857808589935302734375f,-0.84900001099705679763474108767695724964141845703125f,0.28960005451887915750575075435335747897624969482421875f,-0.4090000473558905724758005817420780658721923828125f,0.0723000338159508204594061453462927602231502532958984375f,-0.5897000127911571354388797772116959095001220703125f,-1.3860000396855178816934994756593368947505950927734375f,0.94120003328770429629201998977805487811565399169921875f,1.4113000657491394651543714644503779709339141845703125f,-0.894800054472685513218266351032070815563201904296875f,-0.311800016571581461999329576428863219916820526123046875f,0.19110001131296172616202966310083866119384765625f,-0.756600013861060460840235464274883270263671875f,-0.13989999125003815816370433822157792747020721435546875f,0.09800000031292432911556034014211036264896392822265625f,0.19539999048262834957512268374557606875896453857421875f,-0.25380001458227585686699967482127249240875244140625f,-1.4205999847739934072166079204180277884006500244140625f,-0.592499988004565825150393720832653343677520751953125f,-1.52369997774362531117731123231351375579833984375f,-0.824399992799758507544538588263094425201416015625f,-0.872900006501376335421582552953623235225677490234375f,-1.458100011968612275126133681624196469783782958984375f,0.2859000079318880782608403023914434015750885009765625f,-0.2826999967500569699296875114669091999530792236328125f,-1.001600004920363051752474348177202045917510986328125f,-0.2239000161051745152462899568490684032440185546875f,-0.9280000052303076785165103501640260219573974609375f,-0.6478000140309330134869014727883040904998779296875f,0.8724999952316283557962606209912337362766265869140625f,-0.2876999780118467686662597770919091999530792236328125f,-0.167699997271597400771980801437166519463062286376953125f,-0.646700000259280205483491954510100185871124267578125f,0.011800025400519498219864544807933270931243896484375f,-0.3487000089615583942048715471173636615276336669921875f,-1.33700000192224965900322786183096468448638916015625f,-0.7219999613165857255125956726260483264923095703125f,-1.934300021004677461178289377130568027496337890625f,0.1517999952256678941608925015316344797611236572265625f,-1.554500021331012504788304795511066913604736328125f,-1.0086000397741796685835424796096049249172210693359375f,-0.0064999950900672676112890258082188665866851806640625f,0.415000047832727592123092108522541821002960205078125f,-0.858300035279989881331630385830067098140716552734375f,-0.630900010353327100887099732062779366970062255859375f,-1.031699996161460575194723787717521190643310546875f,0.30129996302425876564257123391143977642059326171875f,0.00519999429136530633144275270751677453517913818359375f,0.2347000045582652294484660160378552973270416259765625f,0.725099997164309062469556010910309851169586181640625f,0.16640001997947717615744522845488972961902618408203125f,-0.059100010079890463732255057038855738937854766845703125f,0.06070000272542219743598934655892662703990936279296875f,-0.86220002521499961023465630205464549362659454345703125f,-0.092499985396862050013311318252817727625370025634765625f,0.196599991043657329470306649454869329929351806640625f,-0.86930001165419812370060981265851296484470367431640625f,-1.057400024740398070122182616614736616611480712890625f,0.05430001444816578981544807902537286281585693359375f,-0.412800040145218982701180721051059663295745849609375f,0.2085000316202647585583918043994344770908355712890625f,0.14439998218268090823812599410302937030792236328125f,0.5241999986201528738405386320664547383785247802734375f,1.429200037176907134295333889895118772983551025390625f,-1.33510001888275109394044193322770297527313232421875f,-0.97940000676959793057818615125142969191074371337890625f,0.702399997210502391453701420687139034271240234375f,-0.1123999966144559881087161556934006512165069580078125f,-0.434100019113719370889015181091963313519954681396484375f,-0.538700016263127690052670004661194980144500732421875f,0.1327999911949040434677726807422004640102386474609375f,0.201500017106532869348711756174452602863311767578125f,-1.0883000084951521824194742293911986052989959716796875f,-0.7249999955669041984407385825761593878269195556640625f,0.6567999963432551435715822663041763007640838623046875f,-0.113099991777539710113842374994419515132904052734375f,0.9711000070661299776730857047368772327899932861328125f,0.031200034964084633060110718361102044582366943359375f,0.11900003610551357358104951345012523233890533447265625f,1.17839999545216489451604502392001450061798095703125f,-0.2047999734848742203752181012532673776149749755859375f,-0.5763999779582018589962899568490684032440185546875f,-0.260500001803040337478023502626456320285797119140625f,-1.55770002644062088847931590862572193145751953125f,1.074500023342669141612759631243534386157989501953125f,0.3557000132307412965104731483734212815761566162109375f,1.31570002671629193713442873558960855007171630859375f,-0.0318000084877014810302853220491670072078704833984375f,0.769100006820261139495187308057211339473724365234375f,0.421900025433302072741525989840738475322723388671875f,0.2754000140264631912856430062674917280673980712890625f,-0.3613000418514011879977942953701131045818328857421875f,0.1164999858886002215285770944319665431976318359375f,0.74709998317956927138538958388380706310272216796875f,1.5062000293761499403188963697175495326519012451171875f,-0.850899978539347845440943274297751486301422119140625f,0.50960002482831467940371794611564837396144866943359375f,1.80080004854202258002260350622236728668212890625f,-0.40289996362328484025283614755608141422271728515625f,-0.48770001094341264291642801254056394100189208984375f,0.17070002250671389987246584496460855007171630859375f,0.740900009012221971715916879475116729736328125f,-0.6140999821402133296288639030535705387592315673828125f,0.20740000156909232931212727635283954441547393798828125f,0.8862000035971402400747365390998311340808868408203125f,0.87060000552386018757289321001735515892505645751953125f,-0.990099938714504190784282400272786617279052734375f,0.6982999960713092146846747709787450730800628662109375f,-0.44529998452663477337409858591854572296142578125f,-0.8657000334963200760540757983108051121234893798828125f,0.358599990153312120355622028000652790069580078125f,0.59799998569115919710981188472942449152469635009765625f,-0.344499980092049185032010427676141262054443359375f,-0.1083999854497605941361371151288039982318878173828125f,0.36059993209838836492053815163671970367431640625f,-1.2310000344328588806064317395794205367565155029296875f,-0.3321000126838686838937064749188721179962158203125f,-0.01730000341758131110481144787627272307872772216796875f,0.40459999420642844825124484486877918243408203125f,-2.03910000002160618493007859797216951847076416015625f,0.3118999982386820857271914064767770469188690185546875f,1.1631000180222088591364126841654069721698760986328125f,1.7475999199576686127244329327368177473545074462890625f,-1.73360000833273009135382380918599665164947509765625f,-0.5330000270605079748520438442938029766082763671875f,-1.73860004081726149394171443418599665164947509765625f,0.62990002909898823446610549581237137317657470703125f,0.679899982011317405294903437606990337371826171875f,0.207999990105630416792337200604379177093505859375f,-1.0036000065445893625337703269906342029571533203125f,-0.0331999655842789564985650940798223018646240234375f,0.69769997294545138544208384701050817966461181640625f,-0.25909998711347537181381994741968810558319091796875f,0.517500033676624671130639399052597582340240478515625f,-0.26260000562071805774166932678781449794769287109375f,0.316899998313188380194560522795654833316802978515625f,0.0622000310450785409699392403126694262027740478515625f,-0.3656999627530570506905860383994877338409423828125f,-1.568200049068032786436788228456862270832061767578125f,-0.104000081628562224267398050869815051555633544921875f,0.7036000008448954901041361154057085514068603515625f,-0.98290000624060613887422732659615576267242431640625f,0.2042000394865868262428421076037921011447906494140625f,0.811300041627884382933189044706523418426513671875f,0.230899967475235445846237780642695724964141845703125f,-0.4115000450462107384197452120133675634860992431640625f,1.1082000274613499346543221690808422863483428955078125f,-0.43059996979832615426886377463233657181262969970703125f,-0.03360001868903683686085059889592230319976806640625f,-0.4171000167444349759904298480250872671604156494140625f,1.042400044819713489374635173589922487735748291015625f,-0.26190001366138471183830915833823382854461669921875f,-0.2087000041186808374504835228435695171356201171875f,0.3330000601410869620622179354541003704071044921875f,1.3668000403203073300772985021467320621013641357421875f,0.8380999861896043512388132512569427490234375f,-0.168999985944479735877621351392008364200592041015625f,-1.7897000124931334852362851961515843868255615234375f,-0.31300003099069029222079052487970329821109771728515625f,0.979300030541420341023695073090493679046630859375f,0.93120004299208580800240042663062922656536102294921875f,-0.29939997886717240049847532645799219608306884765625f,-0.2295000095777213200420874272822402417659759521484375f,0.89910007115602486038596907746978104114532470703125f,1.0490000371485954655526029455359093844890594482421875f,0.292800006878376262164920262875966727733612060546875f,-0.6626000140771270974227036276715807616710662841796875f,-0.5318000129580493506864513619802892208099365234375f,0.52020002264082443321768778332625515758991241455078125f,0.6367999983549115228242953890003263950347900390625f,1.171400051122905239253668696619570255279541015625f,-0.034100033754110636863288164022378623485565185546875f,-0.22450001361966176460782662616111338138580322265625f,0.52730002630948913378006182028912007808685302734375f,-1.97350003549456598506139926030300557613372802734375f,0.143799980384111680820069523178972303867340087890625f,2.3846000666260724898393164039589464664459228515625f,0.297999957919120816995928180404007434844970703125f,-2.05640006863176960649752800236456096172332763671875f,-0.73929998017549447553165009594522416591644287109375f,0.3839000232949854041208936905604787170886993408203125f,0.279899995943904134065860489499755203723907470703125f,-1.475500051945447754775386783876456320285797119140625f,0.9087000493809582746251862772624008357524871826171875f,0.22479999710321418859848563442938029766082763671875f,0.65269999932050648538961468148045241832733154296875f,0.9461000190615660443427259451709687709808349609375f,0.132799989816546837317901008646003901958465576171875f,-2.000900069361926103539417454157955944538116455078125f,0.852500027008354255286803891067393124103546142578125f,0.789300043803454087765203439630568027496337890625f,-0.371899968287348059448049752973020076751708984375f,-0.58500002756714852836239515454508364200592041015625f,-1.0773000050008298966730535539682023227214813232421875f,0.34740000406503668273217044770717620849609375f,-0.368900021506845643415317681501619517803192138671875f,-0.38850002908706660065263349679298698902130126953125f,1.470999995931983495012218554620631039142608642578125f,1.0219000104576345666629322295193560421466827392578125f,0.1363999682165684124157678525079973042011260986328125f,-1.3317000122360891367634394555352628231048583984375f,-0.634700045853108907323303355951793491840362548828125f,0.94680001666471358401366842372226528823375701904296875f,0.658900037860870924077971721999347209930419921875f,-1.7459999645650388089279658743180334568023681640625f,0.137399967724829874526903950027190148830413818359375f,-0.29390002415180216388534972793422639369964599609375f,-0.13430000720247636802895385699230246245861053466796875f,-0.8313000064611433259642581106163561344146728515625f,0.52910001214742674591207105549983680248260498046875f,0.129899998998642018932514474727213382720947265625f,0.48890000859871485250351952345226891338825225830078125f,-0.97080002112388541490872739814221858978271484375f,-1.14530001328587527353874975233338773250579833984375f,0.09540000065267140172409199294634163379669189453125f,0.30880002555325669622021678151213563978672027587890625f,-0.5790999541260306582302064271061681210994720458984375f,0.5250000028684735919881632071337662637233734130859375f,1.1392999717563390138508339077816344797611236572265625f,-0.7173000081673268635285012351232580840587615966796875f,1.086399950838089267790564917959272861480712890625f,-2.433100055032969777357720886357128620147705078125f,0.8123000242233278100911775254644453525543212890625f,0.456400036519766683085208569536916911602020263671875f,1.271399990326166129506191282416693866252899169921875f,0.6321000283300879374337455374188721179962158203125f,0.832899980014561780450321748503483831882476806640625f,1.15920003371238777845064760185778141021728515625f,-0.817500013411045500788532081060111522674560546875f,-2.0291000416517253057691050344146788120269775390625f,-0.366899954950810069931321777403354644775390625f,0.9467000361710784517299543949775397777557373046875f,0.70929998241066893882589283748529851436614990234375f,-0.526000001836568031876595341600477695465087890625f,-0.71700000870227853511096327565610408782958984375f,0.5464999895580116007209880990558303892612457275390625f,-0.452900000037997740864881279776454903185367584228515625f,0.0591000049948693373380592674948275089263916015625f,-0.442899988955259171774514470598660409450531005859375f,-0.14430000878572446509906512801535427570343017578125f,1.292699985499680348510764815728180110454559326171875f,0.05310000809431070223354254267178475856781005859375f,0.450900001040100306681779329665005207061767578125f,0.277700036871433297847033827565610408782958984375f,-0.447000006951391082310465208138339221477508544921875f,-0.603700053441524886466140742413699626922607421875f,1.71450000248104306166396781918592751026153564453125f,-0.270099966132641089444632598315365612506866455078125f,-0.6253000078842043674143269527121447026729583740234375f,0.120300017607212428316643126890994608402252197265625f,0.7592000252559778505911936008487828075885772705078125f,-0.5696999950587748884345273836515843868255615234375f,0.2748000552922496009244923698133789002895355224609375f,-1.32450003343820554846388404257595539093017578125f,0.2705000011324880659913105773739516735076904296875f,0.26629999215602850881623453460633754730224609375f,-0.350500008825212649998803726703044958412647247314453125f,-1.1045999915242195044129402958787977695465087890625f,0.016699993441999183563240194416721351444721221923828125f,0.717100031757354372530244290828704833984375f,0.77810000833645454054732226722990162670612335205078125f,-0.82120002930164304189020185731351375579833984375f,-0.3240000138655305672585882348357699811458587646484375f,-1.12829997344017129989879322238266468048095703125f,1.2800999828591936857691280238213948905467987060546875f,1.0859999895989886908864718861877918243408203125f,0.0410999995596703637801283548469655215740203857421875f,-0.8099000004887582093715536757372319698333740234375f,1.00219999580383234416558480006642639636993408203125f,-0.73300000917911489750622422434389591217041015625f,0.6650000232458115334566173260100185871124267578125f,-0.3480000020265574534050756483338773250579833984375f,-0.18220000027418148391689101117663085460662841796875f,1.01360000833272945186536162509582936763763427734375f,-0.9209999802112580624680049368180334568023681640625f,0.0938999907732007699223686358891427516937255859375f,1.0862000456929212788281802204437553882598876953125f,1.2111999753594400175416012643836438655853271484375f,-1.21060000749826457422386738471686840057373046875f,1.650699983465671039084554649889469146728515625f,-0.00669997008442901886837717029266059398651123046875f,-0.20480002571344346762316490639932453632354736328125f,0.190700049179793662545989718637429177761077880859375f,1.71930000297427110211856415844522416591644287109375f,-0.657899988061188878418761305510997772216796875f,1.110199991720914436399425540002994239330291748046875f,-0.8236000139951702436746927560307085514068603515625f,0.66169999288320457964118759264238178730010986328125f,-0.28670000331401812587728272774256765842437744140625f,0.95159999057799549149194717756472527980804443359375f,0.92640001729726773049833354889415204524993896484375f,0.93079999664872892228828504812554456293582916259765625f,-0.70670003073215514888261168380267918109893798828125f,1.1035000350102774913096936870715580880641937255859375f,0.90150008088350386259435254032723605632781982421875f,-0.36939996076226189192226456725620664656162261962890625f,-1.86930003083944296093932280200533568859100341796875f,-0.0997000227376811931634392749401740729808807373046875f,1.2792999766737216393863718622014857828617095947265625f,0.34709999558478543679740369043429382145404815673828125f,0.2262000298976898360336917903623543679714202880859375f,0.040199967096746558326003651018254458904266357421875f,0.81660001199841492081077376496978104114532470703125f,-0.3684999967515469876389033743180334568023681640625f,-0.81350002133846288643326261080801486968994140625f,-0.5278000018119808345318233477883040904998779296875f,0.299400008296967001797384000383317470550537109375f,0.1772000104069710868515130641753785312175750732421875f,-0.6820000143647195756102519226260483264923095703125f,-0.4858000320017341611134042977937497198581695556640625f,0.97300002676248453781226999126374721527099609375f,0.6303000012904405036096022740821354091167449951171875f,-1.2441999893069262128619811846874654293060302734375f,-0.17999998947605455601461699188803322613239288330078125f,0.22379994141757553194338470348156988620758056640625f,-0.94049999314174048681280737582710571587085723876953125f,-0.121699992957711611296645060065202414989471435546875f,0.151699992547928985686667147092521190643310546875f,1.5568000037193296947179987910203635692596435546875f,-0.03499997233971952059761179043562151491641998291015625f,0.5207999948978425663881353102624416351318359375f,0.0779000092767176965935504995286464691162109375f,-0.0557999931097032941806901362724602222442626953125f,0.64670000301599461778323529870249330997467041015625f,-0.233200005221366524210679926909506320953369140625f,-1.850100077891351446623957599513232707977294921875f,0.1819000136613855289624552824534475803375244140625f,1.36590006027221733120313729159533977508544921875f,0.9928999929785735645282329642213881015777587890625f,-1.61360004260539913190086735994555056095123291015625f,-0.23529996187687007846989217796362936496734619140625f,0.90659998241960959575180822866968810558319091796875f,-0.48369997491240557252467624493874609470367431640625f,0.4333999909818173623676784700364805757999420166015625f,-1.4296999842554327653232348893652670085430145263671875f,0.344400017386675472153001464903354644775390625f,-0.737199951249361884464406102779321372509002685546875f,-0.494300031733512668097318965010344982147216796875f,0.55389999345540985586922033689916133880615234375f,0.6638000094413758489508836646564304828643798828125f,-0.062600020894408192617675013025291264057159423828125f,-1.72920003490448070948559688986279070377349853515625f,-0.71720003275871369030625146478996612131595611572265625f,0.01929998285770384569559610099531710147857666015625f,-0.9129000028878448969038572613499127328395843505859375f,1.10760003408193608009923991630785167217254638671875f,-0.71499996442347768521585749113000929355621337890625f,-1.07330002470016427906784883816726505756378173828125f,0.86739996759444526475135717191733419895172119140625f,1.52130001450777019300630854559130966663360595703125f,0.10160003628730807090363441602676175534725189208984375f,0.3906999828696253729276577359996736049652099609375f,0.254699961158632692814762776833958923816680908203125f,0.3064999974369992941092277760617434978485107421875f,0.944200051258504924334147290210239589214324951171875f,-0.237099986755847425001775263808667659759521484375f,0.384000031709671674207129399292171001434326171875f,-0.702199930536746563802807941101491451263427734375f,1.0254000212162737160070946629275567829608917236328125f,0.5745999754309654150574715458787977695465087890625f,-0.8522000279158359337117190079879947006702423095703125f,1.28979999293088809508844860829412937164306640625f,-0.1091000008970499646210328137385658919811248779296875f,-1.5359000337481489850688376463949680328369140625f,-0.53560000367984184066472153062932193279266357421875f,-0.0498000179648396823495204444043338298797607421875f,-0.9732000050909814792277074957382865250110626220703125f,-0.1047000300765024149995952029712498188018798828125f,-0.3113999736554913244646058956277556717395782470703125f,-1.409899991846085498536922386847436428070068359375f,-0.190100061295181976817048052907921373844146728515625f,-1.41390003011226639273445471189916133880615234375f,0.6172000101834533847267039163853041827678680419921875f,0.817999969244002755885958322323858737945556640625f,-0.7501000090479852389080406283028423786163330078125f,-0.716400035476684848845252417959272861480712890625f,-0.42140008226633174359676559106446802616119384765625f,-1.00409998890161400453280293731950223445892333984375f,1.8764000649809844389892532490193843841552734375f,0.4036000271081920942606302560307085514068603515625f,-1.3298000513434402591883554123342037200927734375f,-0.96139999822378197080752215697430074214935302734375f,0.15349999988079066071122724679298698902130126953125f,-0.52249992877244988420670779305510222911834716796875f,0.62570001326799395968691896996460855007171630859375f,1.6427999478697774549118548748083412647247314453125f,-0.12469998179674224303425944526679813861846923828125f,1.2252000174999242432249957346357405185699462890625f,-0.06879997248649516450313967652618885040283203125f,0.06259997090101165895248414017260074615478515625f,0.37680000271350155838234741167980246245861053466796875f,1.578300000150501869455865744384936988353729248046875f,-0.4065999792531134193751540806260891258716583251953125f,-0.4956999751582740199040699735633097589015960693359375f,-1.0517000198543069444667708012275397777557373046875f,-0.951599960291386270938573943567462265491485595703125f,-0.142999997705220760479960517841391265392303466796875f,0.045499997593462904177386008086614310741424560546875f,0.7892999961197375569099676795303821563720703125f,-0.29089995343089014578907836039434187114238739013671875f,-1.091000008560716327821182858315296471118927001953125f,-1.371000032514334332489625012385658919811248779296875f,-0.65780004912316869525312768018920905888080596923828125f,1.307599986062944363851556772715412080287933349609375f,0.7665000379681590203517771442420780658721923828125f,0.5810000050216912459433160620392300188541412353515625f,0.13469999848604174985666759312152862548828125f,-0.060600005002319878855132628814317286014556884765625f,-0.185199950891733688962403903133235871791839599609375f,1.1569000091165302368523271070444025099277496337890625f,0.9540999935209750759668168029747903347015380859375f,0.17030001922771320632676861350773833692073822021484375f,0.4086999918997289427124997018836438655853271484375f,0.1516000024616716290726259330767788924276828765869140625f,0.7009999966025350914833325077779591083526611328125f,0.1475000010058280519587015078286640346050262451171875f,0.775600044341385963519996948889456689357757568359375f,0.97530006314888684215702596702612936496734619140625f,-0.0265000095441936256435155883082188665866851806640625f,-0.797400034090876896897270853514783084392547607421875f,-0.8586999950289726513119603623636066913604736328125f,-0.425000004358589311692639967077411711215972900390625f,0.3845999906301500459449016489088535308837890625f,0.086000010721385411471828774665482342243194580078125f,0.633999992519617361352857187739573419094085693359375f,-0.601399989394843625944986342801712453365325927734375f,-0.452799980317056149914378693210892379283905029296875f,-0.36380003060102428236177729559130966663360595703125f,0.70919999914169284949139182572253048419952392578125f,0.781099979126453458633250193088315427303314208984375f,-0.338500007927417190245478195720352232456207275390625f,-0.0238000083982949917071891832165420055389404296875f,-0.789099945539236014013795283972285687923431396484375f,0.064600033806265333424789787386544048786163330078125f,0.201799973842500701692870279657654464244842529296875f,-0.73500007063150507580218118164339102804660797119140625f,0.16970000761300407798870537590119056403636932373046875f,-0.5416999738097185712604186846874654293060302734375f,-0.77999996632337609270280154305510222911834716796875f,0.48299998563528045991688486537896096706390380859375f,0.56529999734163283164889435283839702606201171875f,-0.695500008732080399198594022891484200954437255859375f,0.194500011570752473488710165838710963726043701171875f,1.171600007975101664925432487507350742816925048828125f,-0.6694000169768925534441450508893467485904693603515625f,0.32650000429153447356611650320701301097869873046875f,-0.05340003937333881456339668147847987711429595947265625f,1.71960004129409771422842823085375130176544189453125f,0.396199990633129939254786222591064870357513427734375f,-0.9987999996066090346857890835963189601898193359375f,0.9804000065386293538693962545949034392833709716796875f,-0.3938999828010796289134987091529183089733123779296875f,1.2956999919593334968936915174708701670169830322265625f,-0.55309998823851369564863489358685910701751708984375f,0.5556000050768254983069027730380184948444366455078125f,0.091800003272294983247547861537896096706390380859375f,-0.3745999988257884272258024793700315058231353759765625f,-0.04980000202059731151393862091936171054840087890625f,-0.56900003051757863659076974727213382720947265625f,-0.02030006156563768371370315435342490673065185546875f,-0.7849000035807491837402949386159889400005340576171875f,-0.0375000306963926277603604830801486968994140625f,-0.9196000150866805444849205741775222122669219970703125f,0.8725999899744980581317577161826193332672119140625f,0.39499998925253698711657079911674372851848602294921875f,-0.56629997367858919687932939268648624420166015625f,0.2585999861486254491893532758695073425769805908203125f,-0.03039998277127775594408376491628587245941162109375f,0.1829999980218708355295120782102458178997039794921875f,-0.507499980963766805786008262657560408115386962890625f,0.33490000792071217272649619189905934035778045654296875f,0.95299998220801374060329180792905390262603759765625f,0.1232000195264812969497825179132632911205291748046875f,0.8340000001192091616530888131819665431976318359375f,-0.2377999942496418217530163019546307623386383056640625f,-0.4553999703288074130114182480610907077789306640625f,0.598599988663196302951519101043231785297393798828125f,-1.252099961870908995109630268416367471218109130859375f,0.43690001813173306999260603333823382854461669921875f,-0.10579999847412135949298317427746951580047607421875f,1.91020006250143037362931863754056394100189208984375f,-0.26480003942251162385446150437928736209869384765625f,-1.91240000913143148864037357270717620849609375f,0.066299992454052159018829115666449069976806640625f,-0.53249999813735460207908545271493494510650634765625f,1.02330000473260884774617807124741375446319580078125f,-0.7885000078529114109215925054741092026233673095703125f,-0.38839996885359351352917656186036765575408935546875f,0.17399996037036180585033662282512523233890533447265625f,-0.43479998987615109395932222469127736985683441162109375f,0.025799990725517307765812802244909107685089111328125f,0.22460001581311228591175677138380706310272216796875f,0.802299961042404419231388601474463939666748046875f,-0.15790000609159438038631151357549242675304412841796875f,1.64949996876716653559924452565610408782958984375f,-0.29009997479394111774553266513976268470287322998046875f,1.3744999658241867646069067632197402417659759521484375f,0.585200002449750567024011616013012826442718505859375f,1.47060001345872848332874127663671970367431640625f,-0.624100020864605919967971203732304275035858154296875f,0.9725999869942665299049622262828052043914794921875f,0.008299969044327060174737198394723236560821533203125f,1.8342999494791030201668036170303821563720703125f,0.871399988389015778267321366001851856708526611328125f,0.512900021290779051241770503111183643341064453125f,-0.77610000475645080086906091310083866119384765625f,-1.8647000345468516258051749900914728641510009765625f,-0.153999999001622089878082988434471189975738525390625f,-0.63110001175999652645032256259582936763763427734375f,-0.9457000311493874988855168339796364307403564453125f,-0.8561999657109387040776482535875402390956878662109375f,-0.22409997901096989636471334961242973804473876953125f,-0.52859998648390182918177515603019855916500091552734375f,-1.834199990070610741810241961502470076084136962890625f,-0.922299992483854413904964530956931412220001220703125f,0.00250005766749428204320793156512081623077392578125f,0.084700056600570672316052878159098327159881591796875f,0.622500066161155718447162144002504646778106689453125f,-1.58579999787807413014206758816726505756378173828125f,-0.37259995227455977584440915961749851703643798828125f,1.4399000036180016337539200321771204471588134765625f,0.7966999879658214211985978181473910808563232421875f,0.65650001472234720267806551419198513031005859375f,0.84750001847744016458818805404007434844970703125f,-0.3114000128269196476793467809329740703105926513671875f,-0.048000002622604398538896930404007434844970703125f,0.012399984991550372370738841709680855274200439453125f,0.1513000236533585951548275261302478611469268798828125f,-0.078099974119663517768685778719373047351837158203125f,1.1822000661373148755473039273056201636791229248046875f,0.2997999693870543325147082214243710041046142578125f,-0.7219000314123931172360926211695186793804168701171875f,0.363500031620264341114534545340575277805328369140625f,0.41490000646784874760442107799462974071502685546875f,-0.7247000037014483808661680086515843868255615234375f,-0.652500022593886175314992215135134756565093994140625f,1.5108000315546996628057740963413380086421966552734375f,0.1137999882072206758731880427149008028209209442138671875f,-0.349999996349215580782043844010331667959690093994140625f,0.64870001158788814787925502969301305711269378662109375f,-0.09189999432712803173473048445885069668292999267578125f,-0.13550000759586688925395492333336733281612396240234375f,-0.549100009167194258452582289464771747589111328125f,0.19059998097419761364790247171185910701751708984375f,1.0602000006616119076596760351094417273998260498046875f,0.8301000159025182512095852871425449848175048828125f,-0.4760999821811913168545515873120166361331939697265625f,-0.02949999052286056411276149447076022624969482421875f,-0.4204000140637160765066937528899870812892913818359375f,0.004399999356270001271695946343243122100830078125f,0.526500032827258213075083403964526951313018798828125f,0.153600014591216194048683973960578441619873046875f,-0.101900004497171181583325960673391819000244140625f,1.070100026631355394357569821295328438282012939453125f,0.93719999911934148695280555330100469291210174560546875f,2.557699997383355405844440610962919890880584716796875f,-0.8214000362589957138226282040704973042011260986328125f,1.1033000217571851120368364718160592019557952880859375f,0.0651000062167643722688126217690296471118927001953125f,0.6725000271201135859655551030300557613372802734375f,-0.460499996811151834918973690946586430072784423828125f,-0.2241000605762000219556284719146788120269775390625f,-0.8740000501871112437157762542483396828174591064453125f,0.10500003889203046725242529646493494510650634765625f,-0.423500044882298087856042911880649626255035400390625f,-0.207399998849630673447563822264783084392547607421875f,0.0384000447750094675569698665640316903591156005859375f,-0.1200000366568563237024136469699442386627197265625f,-0.90530003712773332580354690435342490673065185546875f,-0.96719997643232336059782028314657509326934814453125f,0.7560000607520349635848333491594530642032623291015625f,-0.5956999976962802367808080816757865250110626220703125f,-0.30119999327808633093894741250551305711269378662109375f,-0.167599987590312249352564322180114686489105224609375f,0.0856000073120000148918506965856067836284637451171875f,-0.311800008040666654096639831550419330596923828125f,-0.75690002409219747647739495732821524143218994140625f,0.12280000985860883844225099892355501651763916015625f,0.00910005102083122352496502571739256381988525390625f,0.20469997911453230443612483213655650615692138671875f,0.2688999893948433861368130237678997218608856201171875f,-0.537100006872415658421004991396330296993255615234375f,0.45500004921108538891161288120201788842678070068359375f,-0.718100029700994735293306803214363753795623779296875f,-0.04259996464252413517215245519764721393585205078125f,0.523900003588199325577079434879124164581298828125f,-0.42870000354126081010264215365168638527393341064453125f,0.691400015211105500867461159941740334033966064453125f,-0.180100012866407421796566268312744796276092529296875f,-0.374700009363889474656161837629042565822601318359375f,-0.5332999820083370590140248168609105050563812255859375f,-0.073899991518259489087228075732127763330936431884765625f,0.24979998089820176776498783510760404169559478759765625f,0.006400020929425986881966537112020887434482574462890625f,0.95300002348422907516578561626374721527099609375f,-0.23550002199411412817653399542905390262603759765625f,0.7843000506579880237723045866005122661590576171875f,-1.141500009134411630640215662424452602863311767578125f,-0.0457000062644485893770252005197107791900634765625f,0.0029000186085703649752076671575196087360382080078125f,-0.8152999621748922010056048748083412647247314453125f,0.538500004500150897257526594330556690692901611328125f,-1.308899999713897699393783113919198513031005859375f,-0.74750001542270183563232421875f,-0.82509995808601388489478267729282379150390625f,-0.50449996876716607374646628159098327159881591796875f,0.65009999362528336774857962154783308506011962890625f,1.2634999983161694370181749036419205367565155029296875f,0.45499997459352048423397718579508364200592041015625f,-0.2163000010408460405120223413177882321178913116455078125f,-0.5870000496432183911821311994572170078754425048828125f,-1.5501000197768206589898909442126750946044921875f,-0.018099962198733887674961806624196469783782958984375f,-0.62709998586177828627796770888380706310272216796875f,-0.719400015896558553407658109790645539760589599609375f,-1.131700017470121277796124559245072305202484130859375f,0.52879998470544808952809034963138401508331298828125f,-1.2574000587522988769961784782935865223407745361328125f,0.21989996055364624538697171374224126338958740234375f,-0.18860004990696932480886971461586654186248779296875f,0.21149999207258218802962801419198513031005859375f,0.93800000798702232174264281638897955417633056640625f,-1.0700999786496165899762900153291411697864532470703125f,-0.841500001385808271692212656489573419094085693359375f,0.695800019522010870787198655307292938232421875f,0.5623000107191493679437144237454049289226531982421875f,0.33700000723078826769807392338407225906848907470703125f,0.8829000168763101452640285060624592006206512451171875f,0.370400015777349711498800388653762638568878173828125f,0.8701000162564216378058290501940064132213592529296875f,-0.575800025892257760773418340249918401241302490234375f,-0.8301000201679766821172279378515668213367462158203125f,0.51040000206828128881397788063623011112213134765625f,0.79890002405867022172714086991618387401103973388671875f,-0.6132000052213673058076892630197107791900634765625f,0.911900029270351186738707838230766355991363525390625f,-0.1530000200569627821778340148739516735076904296875f,-1.103300008569657375545602917554788291454315185546875f,0.5977000048339362603400104489992372691631317138671875f,1.613100033258647147960118672926910221576690673828125f,-0.49320002906322546465389677905477583408355712890625f,0.157500017229468103341361029379186220467090606689453125f,0.491900009489059097944618770270608365535736083984375f,0.177899998026341155110685576801188290119171142578125f,-0.16849999868869769414914117078296840190887451171875f,0.6587999957323074085024927626363933086395263671875f,-0.642400001531839404123047643224708735942840576171875f,0.07520001034736623068965855054557323455810546875f,-0.4721999928981064353905594543903134763240814208984375f,0.425099985504150623683017329312860965728759765625f,-0.14880000646114321938284774660132825374603271484375f,1.208799968910216904305343632586300373077392578125f,-1.34459998519122603255482317763380706310272216796875f,-0.05339998695850312060429132543504238128662109375f,0.584699976804852639133969205431640148162841796875f,1.1476999423980718262328082346357405185699462890625f,-0.922000016823410373234537473763339221477508544921875f,1.014699941116571579868832486681640148162841796875f,-0.3941000356167554397046615122235380113124847412109375f,1.440999985277652939430481637828052043914794921875f,0.306099998646974569282974698580801486968994140625f,1.0647000273942950343553093262016773223876953125f,0.15660001870393802647640768554992973804473876953125f,0.291300014954805508438084871158935129642486572265625f,-0.450700006562471333637631687452085316181182861328125f,-1.032899975097179900984656342188827693462371826171875f,-0.257899970328808070263448826153762638568878173828125f,-0.64789999998211822429539097356610000133514404296875f,-1.38850002506375336253086061333306133747100830078125f,-0.2300999682188038519825568073429167270660400390625f,-1.3077000100493432199755261535756289958953857421875f,-1.37769998903870583006892047706060111522674560546875f,-0.8868999866902829154469145578332245349884033203125f,-0.3701999786078931098387556630768813192844390869140625f,-1.283700044053793565268506426946260035037994384765625f,-1.46130001957416499891451167059130966663360595703125f,-1.7887000392854213171034416518523357808589935302734375f,-0.068600016230345062240303377620875835418701171875f,-0.779100029432773677484647123492322862148284912109375f,-0.142599993103742495037522530765272676944732666015625f,-0.4822000218622386480404884423478506505489349365234375f,0.211700014340877817886621414800174534320831298828125f,-0.44730004385560828694679003092460334300994873046875f,0.11980000723600436884908049250952899456024169921875f,0.445299988568573557046192945563234388828277587890625f,-0.60160001721382183603736848453991115093231201171875f,-0.25109998477026795882238729973323643207550048828125f,-0.2594999888837339341307597351260483264923095703125f,-0.296500009153037868969704504706896841526031494140625f,-0.347800012689828630385591168305836617946624755859375f,-1.03640001712962970259468420408666133880615234375f,0.377600018733739606346944128745235502719879150390625f,-0.02680000768676349398589309203089214861392974853515625f,0.587299994868039743067811286891810595989227294921875f,0.7184999955967070750517677879543043673038482666015625f,-0.410699984657764360918008605949580669403076171875f,-0.514499974727630871029759873636066913604736328125f,-0.0523999907284977695098859840072691440582275390625f,0.403699998903274348549530259333550930023193359375f,-0.3097000148028141719436234780005179345607757568359375f,1.054199996161461427846006699837744235992431640625f,-0.225700020942091850173483180697076022624969482421875f,1.4084999614357940345144015736877918243408203125f,0.6065999756395819009213710160111077129840850830078125f,-0.2020999241709713345471755019389092922210693359375f,-0.467499975487589924938447438762523233890533447265625f,-1.74429998457133716982525584171526134014129638671875f,0.922299977284669836308239609934389591217041015625f,-0.470000014081597417003877126262523233890533447265625f,0.3167999891906980725053699643467552959918975830078125f,1.0534999542832377272816302138380706310272216796875f,0.193299994450807588464158470742404460906982421875f,1.05239998707771320596293662674725055694580078125f,0.446200010526180079750702134333550930023193359375f,-0.839699999827146381647935413639061152935028076171875f,-0.00909998323917360352197647443972527980804443359375f,-0.262699952828884253364094547578133642673492431640625f,0.554400075352192711619636611430905759334564208984375f,-0.9961000062465661386568172019906342029571533203125f,-1.0608000074148176139487986802123486995697021484375f,1.2408999562621119139294023625552654266357421875f,0.63619997014403306678786975680850446224212646484375f,0.5362000088870522152006969918147660791873931884765625f,0.819399991235136848644060592050664126873016357421875f,0.1914999760538342865601180164958350360393524169921875f,1.014100008347630588190213529742322862148284912109375f,-0.5749999822676183924841097905300557613372802734375f,0.8370999843716615540500924907973967492580413818359375f,0.3869000056147573474163436912931501865386962890625f,-0.05030000047087668235690216533839702606201171875f,0.07319999061822901609275504597462713718414306640625f,-0.46019997693151160067515093032852746546268463134765625f,0.36500002905726436353006647550500929355621337890625f,-0.16110003152266172410378430868149735033512115478515625f,0.10070003114938774757547435001470148563385009765625f,0.94480004789754790106570681018638424575328826904296875f,-0.65700005519390192176842901972122490406036376953125f,-0.7357000205881896182091850278084166347980499267578125f,0.37150004109740297764119532075710594654083251953125f,-0.8267000032767650719023322380962781608104705810546875f,0.24379998098015764895762913511134684085845947265625f,0.8868999880313868100500940272468142211437225341796875f,-1.163800000649689803111641595023684203624725341796875f,0.133100077738613709055215394982951693236827850341796875f,-0.16120004941821175492577822296880185604095458984375f,-0.7557000254310668463375577630358748137950897216796875f,0.22290001085996635765695828013122081756591796875f,0.633399992322921434606541879475116729736328125f,1.48410001766085652974425102001987397670745849609375f,0.886400011336803572703502140939235687255859375f,0.3106999876379961111894090208807028830051422119140625f,-0.5825000128149984135461636469699442386627197265625f,-0.278000007838010265714956403826363384723663330078125f,-0.8885000261068345395187861868180334568023681640625f,0.315699992033839027527619691682048141956329345703125f,-1.63779993535280254945973865687847137451171875f,-0.8118000125855207027569804267841391265392303466796875f,-0.8969999788999558631985564716160297393798828125f,-1.1072999442040922790653212359757162630558013916015625f,-0.4726999492526058332941829576157033443450927734375f,-1.37390002623796458891547445091418921947479248046875f,-1.1849999895691869511438198969699442386627197265625f,0.04940000211298478660637556458823382854461669921875f,-0.18720000057220431699533946812152862548828125f,-1.083200020718575018463525339029729366302490234375f,0.3514000023961063590149933588691055774688720703125f,0.1225000104308122672591707669198513031005859375f,0.9368000098288060684126321575604379177093505859375f,1.6341000556588181069628262775950133800506591796875f,0.42040000191926996109259562217630445957183837890625f,-0.39820005826950133354102945304475724697113037109375f,1.11339993569850914667540564551018178462982177734375f,0.950699994045496321604105105507187545299530029296875f,-0.4901999911248680064090876840054988861083984375f,0.85249999657273267672508154646493494510650634765625f,0.15749999091029120990015144343487918376922607421875f,-1.170400008624791876599147144588641822338104248046875f,-0.504499969214200039147044662968255579471588134765625f,-1.4726000166475774477703453158028423786163330078125f,-0.6095000347793106953986352891661226749420166015625f,-0.8086000241279602818167404620908200740814208984375f,0.03239999721050279557488238424411974847316741943359375f,-0.89540001069232799668640154777676798403263092041015625f,0.939800008092820160499059056746773421764373779296875f,1.034299990233778832049438278772868216037750244140625f,-0.7418999935448169491536418718169443309307098388671875f,0.33260000284537627468495202265330590307712554931640625f,-0.264899988269805763962949640699662268161773681640625f,-0.3514999992810192708247996051795780658721923828125f,1.226599985623359589936853808467276394367218017578125f,0.61559999704882473192668612682609818875789642333984375f,-0.1965999927014110415512959662009961903095245361328125f,0.6560000292174519298527002320042811334133148193359375f,-0.34889999710619434836189611814916133880615234375f,-1.34790002633854744118480084580369293689727783203125f,0.708300002199411249392824174719862639904022216796875f,0.249099989516288256030662751072668470442295074462890625f,-0.72449999737739556593396628159098327159881591796875f,0.2905999988555907354026430766680277884006500244140625f,0.8013999677881600280926477353204973042011260986328125f,2.365699965360760881338819672237150371074676513671875f,0.441699992659688067675460843020118772983551025390625f,1.50529998452663438257559391786344349384307861328125f,0.5554999942779537125403521713451482355594635009765625f,-0.71899996569752744335346505977213382720947265625f,0.0259999758154150217848155079991556704044342041015625f,0.5681000000476839062457656837068498134613037109375f,1.0523000291407111905783722249907441437244415283203125f,1.746100017683207550334145707893185317516326904296875f,-0.26240000581592359818472459664917550981044769287109375f,-0.9685999716386197722073347904370166361331939697265625f,-0.3609000116199252428117461022338829934597015380859375f,-0.347000000357628124447728623636066913604736328125f,-0.022699997383356151914313159068115055561065673828125f,0.6448999939322472840785849257372319698333740234375f,-0.56899999494105546471445222778129391372203826904296875f,-1.5609999883696434874735814446466974914073944091796875f,-0.2345999918594958710826858805376105010509490966796875f,0.6666000052928922059436445124447345733642578125f,0.010499992340803121493308935896493494510650634765625f,0.8395999518871313238577158699627034366130828857421875f,0.1563999807894236315775060575106181204319000244140625f,-1.104899997359513719175083679147064685821533203125f,0.3178999895513061613172567376750521361827850341796875f,0.15369998102188053845651438678032718598842620849609375f,-0.3347000278785829951999630793579854071140289306640625f,1.25700001537054806277637908351607620716094970703125f,0.509100021572411520764944725669920444488525390625f,-0.846100002819299756851023630588315427303314208984375f,0.6619000154495235932472496642731130123138427734375f,0.015600007945299143585771162179298698902130126953125f,-0.2199000007867812911399596487171947956085205078125f,-0.46819999568462389305523174698464572429656982421875f,-0.9450000397302213972494655536138452589511871337890625f,0.190399987316131724668366587138734757900238037109375f,0.151500003416091322261394225279218517243862152099609375f,-0.76050002795457860571559649542905390262603759765625f,0.01580000385716584521134109309059567749500274658203125f,0.448699993240833094887420884333550930023193359375f,-0.2460000177435579526985520715243183076381683349609375f,0.94150002127885823455244462820701301097869873046875f,0.2628999929971993498867277594399638473987579345703125f,-0.4891000020146372140317225785111077129840850830078125f,-0.84730001394152576477125649034860543906688690185546875f,0.190700012001394803018428092400426976382732391357421875f,-0.15860004700124263887772713133017532527446746826171875f,-0.114399970746040136049259672290645539760589599609375f,0.55579999877214414372161854771547950804233551025390625f,-0.354600007766485258997590790386311709880828857421875f,-1.2944999668300152251276813331060111522674560546875f,-0.01629999953210370389200534191331826150417327880859375f,-1.59559999960064846646901060012169182300567626953125f,-0.29239998327791649757045888691209256649017333984375f,-1.7532000177383419980969847529195249080657958984375f,-0.2199999847263096608429577827337197959423065185546875f,-0.6073000787615772111394107923842966556549072265625f,-0.02290002009868619126109479111619293689727783203125f,1.55479996998310099343143519945442676544189453125f,-0.285900018548965295650532425497658550739288330078125f,1.545699981230497233042342486442066729068756103515625f,0.564799970839918064058338131872005760669708251953125f,1.2726999778628351744913516085944138467311859130859375f,0.6383999891936777970613547950051724910736083984375f,1.2669999683573844873052394177648238837718963623046875f,0.016999996334314371182472314103506505489349365234375f,0.46719999505877485290739059564657509326934814453125f,0.84589999843239827015395349008031189441680908203125f,0.324299992245435486637461508507840335369110107421875f,1.30970000340342540567917239968664944171905517578125f,1.265600020611285625449227154604159295558929443359375f,0.685000004321336231072336886427365243434906005859375f,-0.9169999839663507401610331726260483264923095703125f,-0.60770000066161156127009235206060111522674560546875f,-0.87809998186826732080589863471686840057373046875f,-0.1033999629676340958894797950051724910736083984375f,-0.493699982362985512196473791846074163913726806640625f,-0.5626999910503627244651170258293859660625457763671875f,-0.4489999943077565003335394067107699811458587646484375f,0.4138000015437597323142426830600015819072723388671875f,-0.076200000654161159641120093510835431516170501708984375f,0.758600015336275301791602032608352601528167724609375f,0.3630000097751617804675561274052597582340240478515625f,0.29940000039935110720534794381819665431976318359375f,-0.741800033521652579793226323090493679046630859375f,-0.1975999744772902744216480641625821590423583984375f,1.481800058481097526197345359832979738712310791015625f,2.089000037372112927869238774292171001434326171875f,0.26610004729926639566173207640531472861766815185546875f,1.3018000276915728807836103442241437733173370361328125f,-0.16859999609515063934139789125765673816204071044921875f,-0.39219999638125246921305233627208508551120758056640625f,-0.38469999090507578376474384640459902584552764892578125f,0.0801999814391134435709318495355546474456787109375f,-1.2254000258728858785417514809523709118366241455078125f,-1.060200021448731266815457274788059294223785400390625f,0.18909999393075682672105131132411770522594451904296875f,-0.8118000032536680787842442441615276038646697998046875f,0.14279999700635659376501962469774298369884490966796875f,0.5295000203996893528568534748046658933162689208984375f,0.0515000025406482098144778092319029383361339569091796875f,0.78260003557205237711968948133289813995361328125f,0.97640000627040901548525653197430074214935302734375f,-0.83000001370906861808407484204508364200592041015625f,0.44839997533559738940311945043504238128662109375f,-1.2569999726414682328368144226260483264923095703125f,-0.26080002127289791946651575926807709038257598876953125f,-0.22639998544603556762666585200349800288677215576171875f,-0.501200022670626577081520736101083457469940185546875f,-0.435100009830296230450841221681912429630756378173828125f,0.7875000335648658644771558101638220250606536865234375f,0.4955000215470788393901102608651854097843170166015625f,0.43320003360807934900122972976532764732837677001953125f,0.1174999800696963081492185665410943329334259033203125f,-1.0785000348053876972898024177993647754192352294921875f,-0.46380001540183979358289434458129107952117919921875f,0.2763000024192032721970235797925852239131927490234375f,0.59440000790581137835033587180078029632568359375f,-0.238999993506818830812932219487265683710575103759765625f,-0.406499979481100448541752712117158807814121246337890625f,0.304399992538989028201257269756752066314220428466796875f,0.1171999839574098256633760684053413569927215576171875f,-0.4078000364571814362335544501547701656818389892578125f,0.71180001616179910950421572124469093978404998779296875f,-0.133399985058605341503579211348551325500011444091796875f,-0.49230002046078435995468680630438029766082763671875f,0.84850001636520000491969994982355274260044097900390625f,-1.1623999478504061233508082295884378254413604736328125f,-1.293300014716386225899213968659751117229461669921875f,-0.588600002335011307508239042363129556179046630859375f,-0.29450003090500853630828714813105762004852294921875f,-1.159000018931926501153384378994815051555633544921875f,-0.2744999935030936200064388685859739780426025390625f,0.38490002153664881756611748642171733081340789794921875f,1.43170004295110775416333126486279070377349853515625f,0.278500002861022721845074556767940521240234375f,0.82550001243874537859568363273865543305873870849609375f,0.14690004631355468855957724372274242341518402099609375f,0.46290002270638996861862324294634163379669189453125f,-0.1631999970257280718755055204383097589015960693359375f,0.57200002181529985278984895558096468448638916015625f,-0.00619999946206795105041464921669103205204010009765625f,1.7789000056743609068377054427401162683963775634765625f,0.219699993009864924520258000484318472445011138916015625f,-0.5561000153362751774466232745908200740814208984375f,0.7442000070579346537869014355237595736980438232421875f,-0.2875000035762784733606167719699442386627197265625f,0.3416000101171434710778385124285705387592315673828125f,-0.0547000303745281968303970643319189548492431640625f,-1.1992000646509237693493332699290476739406585693359375f,0.1458999598383901030729248304851353168487548828125f,-0.391099994046240961864668861380778253078460693359375f,-0.4833999916151163755984043746138922870159149169921875f,0.304600000613928167947364045176072977483272552490234375f,-1.1256999870233241889394548707059584558010101318359375f,-0.957400069406630027657456594170071184635162353515625f,-0.66009997850060386781478882767260074615478515625f,-0.31240003705248320375886805777554400265216827392578125f,-0.43859998896121876743592338243615813553333282470703125f,1.3488000463217495461520911703701131045818328857421875f,1.263200026623159555327902126009576022624969482421875f,-0.097099983030556558105672593228518962860107421875f,0.6117999554395670003259510849602520465850830078125f,-0.01170000584721631042839362635277211666107177734375f,-1.429200023132562780148191450280137360095977783203125f,-0.59610005959272438502694058115594089031219482421875f,-0.4939999679326998460737740970216691493988037109375f,1.04690004495382371629830231540836393833160400390625f,-0.57019998382329983854788224562071263790130615234375f,0.18350000673532473882687554578296840190887451171875f,0.1988000155881037844807224246324039995670318603515625f,-0.719899989014864072345289969234727323055267333984375f,-0.8100999746263024547943132347427308559417724609375f,-0.506199991452693343063629072275944054126739501953125f,-0.58780004744678659989176594535820186138153076171875f,0.04809996994733722885939641855657100677490234375f,-2.1232999880433087724895813153125345706939697265625f,0.45950002747774210121178839472122490406036376953125f,-0.7061000134363764058065271456143818795680999755859375f,0.26789996109008740177159779705107212066650390625f,-0.9596999617919326208692609725403599441051483154296875f,-0.90190000948905879596395607222802937030792236328125f,0.551099995815753640471257313038222491741180419921875f,-0.26809998186826700106166754267178475856781005859375f,-0.593699998791515692886378019466064870357513427734375f,-0.0933000062227247184409861802123486995697021484375f,0.16120000993013405832243734039366245269775390625f,0.8015000205337994287901892676018178462982177734375f,0.7369999941363925000104018181446008384227752685546875f,0.81260001396536818418780967476777732372283935546875f,0.19770002830326571796604184783063828945159912109375f,1.33599998557567545276469900272786617279052734375f,-0.06439998752847364560381038245395757257938385009765625f,2.29429997302293742222900618799030780792236328125f,-0.26289998442903195208231181823066435754299163818359375f,0.35409998264312747551230131648480892181396484375f,0.3619000015914435319785980027518235146999359130859375f,0.081500000715255538352721487171947956085205078125f,0.29069999968931081735235011365148238837718963623046875f,-0.366799959015846610554945073090493679046630859375f,0.5866999937772749973419195157475769519805908203125f,1.134099995160102736235785414464771747589111328125f,-0.34249999089166516252902283667935989797115325927734375f,-0.36859999849796309945304528810083866119384765625f,0.36199996444582893673214130103588104248046875f,0.16959999092817401589172732201404869556427001953125f,0.104099995160103819813457448617555201053619384765625f,-1.373199962455034839337031371542252600193023681640625f,0.142499957680701783147014793939888477325439453125f,-1.9718999385595328277531734784133732318878173828125f,1.5867000182151809184460944379679858684539794921875f,0.1379999951720236595065216533839702606201171875f,-0.0104000060915954151141704642213881015777587890625f,-0.53459999003410363371813218691386282444000244140625f,-0.18959999063014976172780734486877918243408203125f,0.7875999545097354115341659053228795528411865234375f,-0.44820000522136727028055247501470148563385009765625f,-1.41469999252557698810051078908145427703857421875f,-0.02210001536607730798777993186376988887786865234375f,0.25900002127885812797103426419198513031005859375f,0.167500008046626902569187222979962825775146484375f,0.6824000054806471649015975344809703528881072998046875f,0.087400015445798950874944921451970003545284271240234375f,0.58420002126991688040646977242431603372097015380859375f,-0.6168000065132976938997444449341855943202972412109375f,-0.5212999207794661860049245660775341093540191650390625f,-1.7010000148564567989950546689215116202831268310546875f,-1.192800007176398668917727263760752975940704345703125f,1.0441999536931507730486146101611666381359100341796875f,-0.4865000580474745373038558682310394942760467529296875f,0.55640002743005734231473979889415204524993896484375f,-1.461399944430589403765452516381628811359405517578125f,-1.20950000870227736271544927149079740047454833984375f,-1.0159000404536724371240552500239573419094085693359375f,-0.58460004069805204807153131696395576000213623046875f,0.3591000005617741575036916401586495339870452880859375f,1.175700007605552599443399230949580669403076171875f,0.1363999949455259041997123858891427516937255859375f,0.5850000168569386982397872998262755572795867919921875f,0.398800018568337133473278299788944423198699951171875f,0.07110001621171802010934470672509633004665374755859375f,0.876599989274143620576751345652155578136444091796875f,-1.0010999884396785120088679832406342029571533203125f,0.18450002546608512687953407294116914272308349609375f,0.847199980232120619660918237059377133846282958984375f,0.07440003178492238244956524795270524919033050537109375f,0.80790005019903077254639356397092342376708984375f,0.29809996478781053408368961754604242742061614990234375f,1.0222000288844110826858013751916587352752685546875f,0.80880004758834889155849623421090655028820037841796875f,-0.739800006341933880094074993394315242767333984375f,-0.90970002162009488966276649080100469291210174560546875f,0.42629995818138155527776689268648624420166015625f,-0.354899986984580806392841623164713382720947265625f,-0.1429999847412108948674358543939888477325439453125f,-0.1622999739505346372681060529430396854877471923828125f,1.0085999875828621785700534019269980490207672119140625f,-0.378100020760298305422253406504751183092594146728515625f,1.2924999910779304546082357774139381945133209228515625f,-0.804400006583331883547316465410403907299041748046875f,0.315699996280669470394286690861918032169342041015625f,2.0818000069975841626046531018801033496856689453125f,-0.8928000419706105805772722305846400558948516845703125f,-0.0771999907374389948699899832718074321746826171875f,-1.1228999885082231458000023849308490753173828125f,0.58720003007650500848058072733692824840545654296875f,0.44429997659921571795393901993520557880401611328125f,-1.3868000576615333585550615680404007434844970703125f,-1.1461999440670016525700702914036810398101806640625f,0.97110000416040431758801787509582936763763427734375f,-0.262199984180927714305653353221714496612548828125f,0.39859999164342863053889232105575501918792724609375f,1.350199992838501561465136546758003532886505126953125f,-0.554199972021579778669320148765109479427337646484375f,0.206700026857852936501558360760100185871124267578125f,0.06920000465512288201352930627763271331787109375f,-1.140800021012127185571216614334844052791595458984375f,-0.7643000450327994688137778211967088282108306884765625f,0.942200007575750841937178847729228436946868896484375f,0.12800001564249396945882608633837662637233734130859375f,-0.5292000937268139804103839196613989770412445068359375f,1.89790003350973091045261753606610000133514404296875f,0.58059999915361348854503376060165464878082275390625f,-0.58459998019933667734449045383371412754058837890625f,-0.12639998891055570506836147615103982388973236083984375f,-0.15940004703998678081688922247849404811859130859375f,-1.1711000253945595961369008364272303879261016845703125f,0.1062000412225732048909776494838297367095947265625f,-0.353200009691715166582071105949580669403076171875f,-1.348999968528747928075972595252096652984619140625f,0.524999986961483866565458811237476766109466552734375f,1.1811000074386601710330069181509315967559814453125f,0.82970004318952561561673064716160297393798828125f,-0.11170002000331891167661524377763271331787109375f,-1.135000029448419933686409422080032527446746826171875f,-0.9422000160694123938043276211828924715518951416015625f,-0.0529999977983532932768184764427132904529571533203125f,1.4319999839663495322383823804557323455810546875f,-1.08140002532526846579230550560168921947479248046875f,-1.5976999891877170512088923715054988861083984375f,0.120999993044883236148478999893995933234691619873046875f,-0.2927000207781791374372915015555918216705322265625f,1.072399999389797020654668813222087919712066650390625f,0.260600024485587056233271141536533832550048828125f,-0.466500031020492311739644719636999070644378662109375f,1.093999988794326583274596487171947956085205078125f,-0.88629998694062184227959733107127249240875244140625f,-0.211800002825260236249960144050419330596923828125f,1.037799995851516943190517849870957434177398681640625f,-0.36030001477599160608633610536344349384307861328125f,-1.45210005949214160381188776227645576000213623046875f,0.31360004059374346763178209585021249949932098388671875f,0.323200002315640222416703863927978090941905975341796875f,-1.418400011638552005166502567590214312076568603515625f,1.0304000656962397641791540081612765789031982421875f,0.805300010752677764003237825818359851837158203125f,-0.7887999587774263687833808944560587406158447265625f,1.30269994180202441924620870850048959255218505859375f,1.52069998137950701533327446668408811092376708984375f,-0.21370007162094051267331451526843011379241943359375f,-0.4572999649167055480347698903642594814300537109375f,-0.8006000027298920684870608965866267681121826171875f,0.396000002767890768584635452498332597315311431884765625f,0.3867000227414074675635902167414315044879913330078125f,-1.015500025179237209016491760849021375179290771484375f,0.839199977423251031183326631435193121433258056640625f,1.4953000221520660328877738720620982348918914794921875f,0.730800003726779667800883544259704649448394775390625f,-0.9460999730914831928885178058408200740814208984375f,-2.1109999833777539635093489778228104114532470703125f,0.429999965727329680476032081060111522674560546875f,-0.5499000290989879413672269947710447013378143310546875f,-0.67710002520084433541569524095393717288970947265625f,-1.369699996548891629544186798739247024059295654296875f,-0.15889999881982763696441907086409628391265869140625f,0.2249000152036548794143300256109796464443206787109375f,0.06300002169609086877244408242404460906982421875f,-1.2585000190660353691640693796216510236263275146484375f,-0.1341000413537027924348876695148646831512451171875f,-0.0849000048659742656553817141684703528881072998046875f,-2.01520001504123147384461844922043383121490478515625f,-0.32770001641958945715060735892620868980884552001953125f,0.36219998708665368525316807790659368038177490234375f,-0.22859999540597175649025984967011027038097381591796875f,-0.1317999463498598089472579886205494403839111328125f,1.1866000058516856796586580458097159862518310546875f,2.479099977874756888240881380625069141387939453125f,-0.19259994761794752005101827307953499257564544677734375f,-0.181499942302703942687003291212022304534912109375f,0.59129998839348563155482452202704735100269317626953125f,0.7426999999165531107792048715054988861083984375f,-0.50650001693889501463985425289138220250606536865234375f,0.09230003852844248513065394945442676544189453125f,0.901100027480721710304578664363361895084381103515625f,1.285899983978271876594590139575302600860595703125f,-0.57289998647794126984678086955682374536991119384765625f,1.0181999950140714350510506847058422863483428955078125f,-1.5500000367499888742628399995737709105014801025390625f,-0.2554999951906504751519833007478155195713043212890625f,1.06250004313886137197187053970992565155029296875f,0.31529999465942371017490586382336914539337158203125f,-0.107700010719895100663734410773031413555145263671875f,-1.2980999993771311640244903173879720270633697509765625f,1.365000005662441395770656527020037174224853515625f,0.405099967920779846508594346232712268829345703125f,-0.5088000180840506203594486578367650508880615234375f,-1.2024000222444524155207545845769345760345458984375f,0.14419999198913568960733755375258624553680419921875f,0.406999998569489207511651329696178436279296875f,0.6312000215530400026864299434237182140350341796875f,-1.69769998531341581582410071860067546367645263671875f,-0.3377000410437585031786511535756289958953857421875f,-0.126799964380264640340101323090493679046630859375f,0.20720001159906420440393048920668661594390869140625f,-0.71590004764348258436967853413079865276813507080078125f,-0.9171000430077322906896597487502731382846832275390625f,0.322599973806739104276175567065365612506866455078125f,0.3722999982953070219338087554206140339374542236328125f,0.37170001273900232785507569133187644183635711669921875f,-0.685599982538819485711201195954345166683197021484375f,-1.25949998262524598402478659409098327159881591796875f,0.127899995213747619260402643703855574131011962890625f,-0.022899986533820193557176025933586061000823974609375f,0.4873999792546033216211753824609331786632537841796875f,-1.1471999908864500827831989226979203522205352783203125f,0.071000012472272278074569840100593864917755126953125f,0.02389999177902979443643971535493619740009307861328125f,0.85249997552484246821080660083680413663387298583984375f,-0.0090000046268106093094729658332653343677520751953125f,0.3192000110253692124473445801413618028163909912109375f,0.841900045847893352402024902403354644775390625f,0.00010003609359277021439993404783308506011962890625f,0.00089999811574814625458884620456956326961517333984375f,0.51349997775256639442886807955801486968994140625f,-0.467599995860457529062159665045328438282012939453125f,0.09880001361370016610408129054121673107147216796875f,0.7307000031724566468938064645044505596160888671875f,0.9316999584615235363571628113277256488800048828125f,-0.8290999834068120133423462903010658919811248779296875f,0.3813999781817205558809291687794029712677001953125f,-0.342699983935057961303982665413059294223785400390625f,0.71090001407861702631407752051018178462982177734375f,-0.64239999700561123407993591172271408140659332275390625f,0.041399966521562081567253699176944792270660400390625f,-0.5696999911285935969118554567103274166584014892578125f,0.555400013095140199226307231583632528781890869140625f,-0.2006000019103286735600022439029999077320098876953125f,-0.163000034213066413713022484444081783294677734375f,-1.1847000414758923625413444824516773223876953125f,-1.0031999575376513433866421109996736049652099609375f,-0.37400001144409156239589719916693866252899169921875f,0.1210000208169217383868954129866324365139007568359375f,0.4271999833613637775897586834616959095001220703125f,-0.7917999854654074187720880217966623604297637939453125f,0.087099960678816756853848346509039402008056640625f,-1.0405999589204792954433287377469241619110107421875f,0.253699950399993934269105011480860412120819091796875f,-1.426399977809190477984202516381628811359405517578125f,0.65079999631345319954789374605752527713775634765625f,-0.4770999612003563594697652661125175654888153076171875f,0.0488999588474623436695765121839940547943115234375f,-1.33189997685551642092605106881819665431976318359375f,0.3382000053331253308641635157982818782329559326171875f,0.09820001509338649459124326313030906021595001220703125f,0.4369999876916408343419107040972448885440826416015625f,-0.19710000903308344533115814556367695331573486328125f,-0.782399987897276272263979990384541451930999755859375f,0.645799997505544620679529543849639594554901123046875f,-0.39579998476505284799031869624741375446319580078125f,-0.69859997566193243745402696731616742908954620361328125f,-0.7683000246629116958274607895873486995697021484375f,0.39549996041506574329105205833911895751953125f,0.94059998501613650834229929387220181524753570556640625f,0.70659998927414324754181507159955799579620361328125f,-0.6646999879062178706590202637016773223876953125f,-1.137799988140165918792945376480929553508758544921875f,-0.34289999130219162015720257841167040169239044189453125f,0.1952000101611017779390522264293394982814788818359375f,-0.3986000112012029195085460742120631039142608642578125f,0.48220002635121372858151289619854651391506195068359375f,-0.30399999803304655898728015017695724964141845703125f,0.777600001746415525616384911700151860713958740234375f,-0.15489993641376553767941004480235278606414794921875f,0.4043000019684439561018507447442971169948577880859375f,0.483900026834010787979423184879124164581298828125f,0.54699999191984483015716023146524094045162200927734375f,-0.2088000079512593032404765835963189601898193359375f,-0.42879998342022240809257027649437077343463897705078125f,-0.5313000122725959339931023350800387561321258544921875f,0.3061000052966178497371174671570770442485809326171875f,-0.4121999692797668757293649832718074321746826171875f,1.1947999770611517078577890060842037200927734375f,0.970600001351534569238310723449103534221649169921875f,-0.02159999974220994101159476485918276011943817138671875f,-1.113800015848875002433260306133888661861419677734375f,0.2123999843209978255487158094183541834354400634765625f,-0.6252000017046928093122915015555918216705322265625f,0.36340000510066750027959869839833118021488189697265625f,0.330399950659276075981551912263967096805572509765625f,-1.3112999528169628771223642615950666368007659912109375f,-0.315300002333521955932837954605929553508758544921875f,-0.427200028288364563877621549181640148162841796875f,0.4367000018239028946709368028677999973297119140625f,-0.46899998050555480322287849048734642565250396728515625f,1.526700014564393637073180798324756324291229248046875f,0.856200053218006740252121744561009109020233154296875f,0.61929995262697279390096127826836891472339630126953125f,-0.3851000100910670909115651738829910755157470703125f,-0.371100041562317795040826240438036620616912841796875f,0.104800040130318183884128302452154457569122314453125f,1.430500000089406587022722305846400558948516845703125f,-0.709800000791251495257938586291857063770294189453125f,0.57639998634010536360250398502103053033351898193359375f,-0.1101999900817869360025724745355546474456787109375f,0.20719998984336829295216375612653791904449462890625f,-0.01369998668432259592009359039366245269775390625f,1.6365000063031909238731032019131816923618316650390625f,1.2399000493645662146491304156370460987091064453125f,1.3117000214189289408750482834875583648681640625f,1.017600016796588846546001150272786617279052734375f,0.339299996808916315416126963100396096706390380859375f,0.635399992103129473974831853411160409450531005859375f,0.4369000203296542483855091631994582712650299072265625f,-0.79649996197223682514731990522705018520355224609375f,-1.3360999898180361267208127173944376409053802490234375f,-2.320399996554851629326776674133725464344024658203125f,-0.5013999917045233178924945605103857815265655517578125f,-1.0075000112131233809265040690661408007144927978515625f,0.1125999966800208529349447417189367115497589111328125f,0.79799999919533737280374907641089521348476409912109375f,0.0261999964632093906846677100475062616169452667236328125f,0.7979000014349815206315952309523709118366241455078125f,-0.19859998985528903858721605502068996429443359375f,-0.452100068265199439565549255348742008209228515625f,-0.13080002738237350712324769119732081890106201171875f,0.45890000850558276823676351341418921947479248046875f,1.27919999735355371939249380375258624553680419921875f,0.45990001635849486749663128648535348474979400634765625f,0.62410000935345888439798045510542578995227813720703125f,1.23299998818710410120047527016140520572662353515625f,0.78560000984519706701547647753614000976085662841796875f,0.6399000207543374951768555547459982335567474365234375f,0.0041999789878719440139320795424282550811767578125f,-0.6323000069379809406910908364807255566120147705078125f,-0.806100023867190262905069175758399069309234619140625f,-0.8319000043109061603985310284770093858242034912109375f,0.2906000470794742884805827998206950724124908447265625f,0.14580001469776060485372681796434335410594940185546875f,-0.1421999931775033421121889887217548675835132598876953125f,-0.09619999089390052215975401850300841033458709716796875f,-0.274400019454211496761075750328018330037593841552734375f,0.68620001935511876212103743455372750759124755859375f,-0.27280002205893449929163807610166259109973907470703125f,0.4551000191807752504047357433591969311237335205078125f,-0.3358000454314062910299298891914077103137969970703125f,-0.0488999854832884162103567859958275221288204193115234375f,1.474700051552802815280074355541728436946868896484375f,-0.605500060494990055559583197464235126972198486328125f,0.76610002362504603379278478314517997205257415771484375f,-0.14789999387264263219776694313623011112213134765625f,-0.6955000110231341903244128843653015792369842529296875f,0.873500009864569104678366784355603158473968505859375f,0.13010000046119063465965837167459540069103240966796875f,-0.35830000532865557971717862528748810291290283203125f,-0.5677000297188763422440160866244696080684661865234375f,0.1192999831557273182625067420303821563720703125f,-0.2524999812431636403431411963538266718387603759765625f,0.69640002066865547281082626795978285372257232666015625f,0.16820004212036676616293107144883833825588226318359375f,0.471400000458955137361272136331535875797271728515625f,1.072600025290250780329870394780300557613372802734375f,0.54989999026283575833673467059270478785037994384765625f,1.2605000003874298197814596278476528823375701904296875f,-0.54620001624450054134030096975038759410381317138671875f,0.88149994377419382818317217243020422756671905517578125f,0.346599999406933978463030143757350742816925048828125f,-0.23539999446868886678885246510617434978485107421875f,0.9454000277727840906294431988499127328395843505859375f,-0.6860000001788133516811285517178475856781005859375f,-0.1268000434309246582387231683242134749889373779296875f,-1.5594000160455703252182502183131873607635498046875f,-0.3315999956816433336115323982085101306438446044921875f,-1.1561000131011009983694748370908200740814208984375f,0.25289998007044223182759878909564577043056488037109375f,-0.067600035516172718441652023102506063878536224365234375f,0.82399998657777917543398871202953159809112548828125f,-0.0399999776482580227110474879737012088298797607421875f,1.0369000509887940086883872936596162617206573486328125f,-0.07249999083578562153462598871556110680103302001953125f,0.86300005229935106143557277391664683818817138671875f,-0.38549995501339395786999375559389591217041015625f,-1.04140003866180830272014645743183791637420654296875f,0.55799995702505089667511128936894237995147705078125f,-1.147200050938129489708217079169116914272308349609375f,0.249399998909234366095688528730534017086029052734375f,0.6778000131368637770634677508496679365634918212890625f,1.54600000673532544936961130588315427303314208984375f,1.331800035458803765919810757623054087162017822265625f,-0.26480001826286336807925181346945464611053466796875f,0.375999984234571371644051396287977695465087890625f,0.743800013166666218467071303166449069976806640625f,0.38869999154582612899133664541295729577541351318359375f,-0.547499985694885449305502334027551114559173583984375f,-0.188000005546957338342650700724334456026554107666015625f,0.32880000988841029396780868410132825374603271484375f,-0.52070000667422922102645088671124540269374847412109375f,0.666599997991323078849745797924697399139404296875f,1.6349000098206103448461590232909657061100006103515625f,-0.74099999615550071752068106434307992458343505859375f,0.02930001746565114029152709917980246245861053466796875f,1.15169998841285714519244720577262341976165771484375f,0.956100003974138967777207653853110969066619873046875f,-0.355700003284215693355463372427038848400115966796875f,-1.157700015600025889028756864718161523342132568359375f,1.117700019586086224165910607553087174892425537109375f,0.7604999969601629050686142363701947033405303955078125f,-0.42250000774860385632791803800500929355621337890625f,1.2910999698877336783198188641108572483062744140625f,0.97259997142255283364420392899774014949798583984375f,1.5919999662339687773737750831060111522674560546875f,1.347299980860948931393750171991996467113494873046875f,0.18630000988841022291353510809130966663360595703125f,1.42099998103082203471103639458306133747100830078125f,0.372399945354462857949329190887510776519775390625f,0.271899993246794036139135641860775649547576904296875f,-0.4990000358819965953216524212621152400970458984375f,0.0790000031739470642833111924119293689727783203125f,0.030099989080429878640643437393009662628173828125f,0.032300014463066961667436771676875650882720947265625f,0.20890000209808334830086096189916133880615234375f,0.99549998116493210176258799037896096706390380859375f,0.5653999917864798163691375521011650562286376953125f,0.0283999952286482226782027282752096652984619140625f,0.5960999922394751848742089350707828998565673828125f,0.28320003651380520892644199193455278873443603515625f,0.247700002151728160271204615128226578235626220703125f,-0.058999985665082466113062764634378254413604736328125f,0.80659995984435095550679761799983680248260498046875f,1.280499971181153995303247938863933086395263671875f,0.56680002979636157078857650049030780792236328125f,2.00809999930262517153778389911167323589324951171875f,1.295299964112043511477168067358434200286865234375f,0.3087999532639980060366724501363933086395263671875f,-0.700899987107515443085503648035228252410888671875f,-0.196400016403198041103905779891647398471832275390625f,0.5011999935388562477101004333235323429107666015625f,-0.74050000098347634747142365085892379283905029296875f,-0.586200008887052259609617976821027696132659912109375f,0.6809000283837320921520586125552654266357421875f,0.512000011384487141441468338598497211933135986328125f,0.44399999848008153691125698969699442386627197265625f,0.438200001794099680552108111442066729068756103515625f,0.52519998877793538927249983316869474947452545166015625f,-0.03009999731332058114929850489716045558452606201171875f,0.63760001281052824939621359590091742575168609619140625f,-0.0573999994084239961722460066084749996662139892578125f,0.224800007310509197822057103621773421764373779296875f,-0.628299993705749404426796900224871933460235595703125f,0.142800009784102233112434987560845911502838134765625f,-0.08260001053810039906011297716759145259857177734375f,-0.29159998240098372601636356193921528756618499755859375f,1.5294000039756294739845543517731130123138427734375f,1.31210002790167923336639432818628847599029541015625f,-0.63610000326633464595715850009582936763763427734375f,-0.53099997266754439539937493464094586670398712158203125f,0.648600009822845269269464552053250372409820556640625f,-0.254100020603835463628428215088206343352794647216796875f,-0.17430000431537617799904182902537286281585693359375f,0.80869997357130074533415609039366245269775390625f,-0.4053999974489210789130311241024173796176910400390625f,1.8070999748162923648209243765450082719326019287109375f,-0.18120002831444181889963829235057346522808074951171875f,1.999499990019947492925211918191052973270416259765625f,0.53460000120997441808157191189820878207683563232421875f,-0.2240999158114203027025723713450133800506591796875f,0.493900076529383225132363577358773909509181976318359375f,-1.846999964222311607642268427298404276371002197265625f,0.327799971190094741047005300060845911502838134765625f,0.456599948370456587554144789464771747589111328125f,1.0021000150680545726089576419326476752758026123046875f,0.7421000249028200101975016877986490726470947265625f,0.094599975430965432821039939881302416324615478515625f,-1.439099957609176527739691664464771747589111328125f,-0.268799990889430073792709663393907248973846435546875f,-0.6531000051140782858283273526467382907867431640625f,0.0632000123739242525289228069595992565155029296875f,-1.0506999850302936039270207402296364307403564453125f,-0.86489999938979755800261273179785348474979400634765625f,-1.011899991104751261872252143803052604198455810546875f,-1.452400003152340079992654864327050745487213134765625f,1.1969999398589135086723445056122727692127227783203125f,-0.138100038120150703235822220449335873126983642578125f,0.7433000006347889598856681914185173809528350830078125f,0.27449999223649523827361917938105762004852294921875f,-0.95679997764229707257754853344522416591644287109375f,0.42700003656745000313321725116111338138580322265625f,1.03319996007084835554223900544457137584686279296875f,-0.51010005792379420341831064433790743350982666015625f,0.95479998965263401800029896548949182033538818359375f,1.41219999729394896093026545713655650615692138671875f,-0.117000005125999617661136653623543679714202880859375f,0.3251999860584735557722524390555918216705322265625f,-0.29320002489089969088809084496460855007171630859375f,-1.04240000577866975817187267239205539226531982421875f,0.4976000187337394908837495677289552986621856689453125f,0.1253999753952025619785359822344616986811161041259765625f,-0.1058999756336209685514404554851353168487548828125f,0.61470001990646128131601244604098610579967498779296875f,-0.27800001484155656417129875990212894976139068603515625f,-1.5150000461749735247707349117263220250606536865234375f,-0.012599990067630850720803437070571817457675933837890625f,0.9653000292301185947252406549523584544658660888671875f,-0.379600043380261098491246229968965053558349609375f,0.076600007956475479886648827232420444488525390625f,-0.27589997622966766499530422152020037174224853515625f,-0.103599993487447417095381752005778253078460693359375f,-0.3069000039756293318760071997530758380889892578125f,0.015800004155188229759687601472251117229461669921875f,0.80499999999999971578290569595992565155029296875f,-0.2498000134944913241952235694043338298797607421875f,0.238899973487854300202570811961777508258819580078125f,0.6929999817609786560979046043939888477325439453125f,1.029400009340047716932531329803168773651123046875f,1.28409999083876602554710188996978104114532470703125f,-0.5657000166021288745099582229158841073513031005859375f,-0.6118999752163885030853407442918978631496429443359375f,0.84830001588985337601656055994681082665920257568359375f,-1.01260001320168324667747583589516580104827880859375f,-1.513299989458917860218889472889713943004608154296875f,-0.7898000909805293900944889173842966556549072265625f,-0.896899989372491379668872468755580484867095947265625f,-0.17199999946355859492541640065610408782958984375f,0.9060000424981111688538248927216045558452606201171875f,-0.5852000112414348365064142853952944278717041015625f,-0.032400013601779686922554901684634387493133544921875f,-0.834500034332275930637479177676141262054443359375f,-0.81500000387430215909034814103506505489349365234375f,-0.81580000532865515339153716922737658023834228515625f,-0.255899988001585310115615357062779366970062255859375f,-0.1642000228345394940987489462713710963726043701171875f,0.06489998543858543911255765124224126338958740234375f,1.122700024503469506953479140065610408782958984375f,0.748800014079362252772398278466425836086273193359375f,0.4688999968081706271050279610790312290191650390625f,1.139100029488652854325891894404776394367218017578125f,-0.2323999288141731511814214172773063182830810546875f,-0.1075999951899060391724560759030282497406005859375f,-0.4123000161767009785762638784945011138916015625f,-1.400200004833937139636645952123217284679412841796875f,0.28710001626014669540154500282369554042816162109375f,-0.79649999251961656199227945762686431407928466796875f,0.2652999983847141862725038663484156131744384765625f,0.730800025743246095544236595742404460906982421875f,-0.402600019776821305139691276053781621158123016357421875f,-0.8977000106081367558630290659493766725063323974609375f,0.2048000127680602577839863442932255566120147705078125f,-0.143300010097026753186355563229881227016448974609375f,-0.1880000216960906467278391573927365243434906005859375f,-0.98590001068860255717396512409322895109653472900390625f,0.5742000346809621458277206329512409865856170654296875f,1.84659999858736956213078883592970669269561767578125f,-0.60260000070333408217493342817761003971099853515625f,-0.5120999842040243610341576641076244413852691650390625f,1.110000013560056952854893097537569701671600341796875f,1.0239000179864461426149091494153253734111785888671875f,-0.518800011825561568912235088646411895751953125f,-1.05400000106915836539656083914451301097869873046875f,0.96100001597404460795814884477294981479644775390625f,0.8217000089764596726382706037838943302631378173828125f,-0.0073000024676321828565050964243710041046142578125f,-0.36569999389648455423440509548527188599109649658203125f,-0.10690000693723578084615155603387393057346343994140625f,-0.8754999481216072521050364230177365243434906005859375f,-0.74209997505843627862986977561376988887786865234375f,-0.272200026053190047292673625634051859378814697265625f,1.792299964618683016936984131461940705776214599609375f,1.4067999697446822437996161170303821563720703125f,0.101700057330727400284331451985053718090057373046875f,0.006500012636184493430846487171947956085205078125f,0.985500010967254258531511368346400558948516845703125f,0.18660000827312472893027006648480892181396484375f,-0.2288000033318997594733446021564304828643798828125f,-0.58579999221563383571265148930251598358154296875f,1.41729999121725569466434535570442676544189453125f,0.170400012946128498469988699071109294891357421875f,0.033799999979138295458369611878879368305206298828125f,0.237900007581711037119021057151257991790771484375f,-0.9519000305555760377274054917506873607635498046875f,0.35979998046979300152514724686625413596630096435546875f,0.26740001198127882897637164205661974847316741943359375f,-0.757699969965219821688151569105684757232666015625f,-0.3209999928027393867324690290843136608600616455078125f,-0.20930002242028677983398665674030780792236328125f,0.55340000863969318611879089075955562293529510498046875f,0.97109999372959077135192273999564349651336669921875f,0.07749997526407259584857456502504646778106689453125f,-0.10439998534917815931066797929815948009490966796875f,0.43680004216432610775200373609550297260284423828125f,0.991099998348951682913821059628389775753021240234375f,0.1437000023305419205144062289036810398101806640625f,-0.3441000428438183433854646864347159862518310546875f,0.614499977111816253483311811578460037708282470703125f,0.6818000129580497059578192420303821563720703125f,0.81899999102950093998742886469699442386627197265625f,0.0190000096559526099326831172220408916473388671875f,-0.2672999648049465548638181644491851329803466796875f,0.8375000083446499932193773929611779749393463134765625f,0.06369997930824790322024142597001628018915653228759765625f,0.01280000456869601066500763408839702606201171875f,-0.4844000026717776297147111108643002808094024658203125f,-0.2262999873131519290581081804702989757061004638671875f,0.14289997742548565806686156065552495419979095458984375f,-0.0479999804198740775973419658839702606201171875f,-0.7676000402659184373987955041229724884033203125f,0.3895000164508826134834862386924214661121368408203125f,0.3922000249542294536553299622028134763240814208984375f,-0.6815000149458649847389324349933303892612457275390625f,-0.8907000331610444021634975797496736049652099609375f,0.7137000164866453832956949554500170052051544189453125f,0.219600004488229527677134456098428927361965179443359375f,-0.29920001571923504624095357939950190484523773193359375f,0.33430001713037427180097438395023345947265625f,0.7263999807149172927012159561854787170886993408203125f,0.339100013935566213518768563517369329929351806640625f,-0.38690000524222833977461277754628099501132965087890625f,0.581699965912104044463148966315202414989471435546875f,1.4134999650120736447433955618180334568023681640625f,0.66379997114539168734381746617145836353302001953125f,0.4254999872744082978215374168939888477325439453125f,-0.9667999773442748079332886845804750919342041015625f,0.62020001325309215278736019172356463968753814697265625f,-0.554999962002038937924908168497495353221893310546875f,0.34989999132454407959613718048785813152790069580078125f,-1.22679995320439427786141095566563308238983154296875f,0.0753000155955547700870056360145099461078643798828125f,-0.9969999669790265972579845765721984207630157470703125f,-0.069400001032650482368779876196640543639659881591796875f,0.11280000166296932917475714930333197116851806640625f,-0.4796000341415407319800578989088535308837890625f,0.314800012451410538005802663974463939666748046875f,0.9526000001817938045434175364789552986621856689453125f,-0.273399975410104190132187795825302600860595703125f,0.42900001010298705050871603816631250083446502685546875f,0.474099981078505816611823320272378623485565185546875f,0.50979998540580251376042042465996928513050079345703125f,1.276499955564737387447848959709517657756805419921875f,1.60739997843503967800415921374224126338958740234375f,0.344099984282255189071975109982304275035858154296875f,1.4251000220865017720228706821217201650142669677734375f,1.522099970290065495959197505726478993892669677734375f,1.012999976918101285860984717146493494510650634765625f,-0.04230000984370718697391566820442676544189453125f,1.224900018630922460260990192182362079620361328125f,0.14000001233071113215800096440943889319896697998046875f,0.205800006930530077209340333865839056670665740966796875f,-0.07800000515580129967219136233325116336345672607421875f,-0.12079998953342485634721015230752527713775634765625f,1.0952999905616043196943110160646028816699981689453125f,0.21059999052956701603989131399430334568023681640625f,0.10710002371072757654246743186376988887786865234375f,1.179100040142983640834017933229915797710418701171875f,-1.08130003298521071286586447968147695064544677734375f,-0.0317999752208582286971960684240912087261676788330078125f,0.178599986297637347565370191659894771873950958251953125f,-0.55730001254454253345471670400002039968967437744140625f,0.857100028777122435741375738871283829212188720703125f,1.6451000363357366484962085451115854084491729736328125f,-0.64150003319978765592423997077275998890399932861328125f,-0.073399989379942243150622971370466984808444976806640625f,0.32360000352710482918183743095141835510730743408203125f,-0.230400004564225768799445859258412383496761322021484375f,-0.03120000098943709332388607435859739780426025390625f,0.32779998035430946146107089589349925518035888671875f,0.267899999758601570221117071923799812793731689453125f,1.126899984082579297961501652025617659091949462890625f,0.21629996533393835989045328460633754730224609375f,1.970599972331523730417757178656756877899169921875f,-0.743399991000443183253310053260065615177154541015625f,-0.2084999566674234916519026228343136608600616455078125f,-0.81479995461627818809091650109621696174144744873046875f,-0.713000021938233974339027554378844797611236572265625f,0.8083999783158299923258027774863876402378082275390625f,1.4638999389171605258752606459893286228179931640625f,-0.54890000880360556578807518235407769680023193359375f,0.5730999735236166348073538756580092012882232666015625f,-0.5266000102475285604697319286060519516468048095703125f,-0.115399994990229348701404887833632528781890869140625f,0.0692000003337860380980828267638571560382843017578125f,0.190200008559227029536486952565610408782958984375f,-0.9170999981552352409153172629885375499725341796875f,-0.68179998032450706002549623008235357701778411865234375f,-0.020799981114268728532579189050011336803436279296875f,-0.74629999573230687559544094256125390529632568359375f,-0.35679998151659919614075988647527992725372314453125f,0.85609999760389310363706272255512885749340057373046875f,-0.737400051245839183167163355392403900623321533203125f,-0.95979997065365385555679722529021091759204864501953125f,-1.335400013821572695604800173896364867687225341796875f,-1.0349999871850013022367420489899814128875732421875f,-0.5399999625980849149442519774311222136020660400390625f,0.325200010719895349353691926808096468448638916015625f,-0.93790003816634459621326413980568759143352508544921875f,-0.5422000084325671043217198530328460037708282470703125f,-0.9360000124350189576460934404167346656322479248046875f,-0.245000010728836148388154470012523233890533447265625f,-0.47320000969171527316348146996460855007171630859375f,-0.0792999989509581837410223670303821563720703125f,1.46859999402761420839169659302569925785064697265625f,0.2728000061146911292553340899758040904998779296875f,0.46240002179741868104656532523222267627716064453125f,0.25680001532360952065658921128488145768642425537109375f,-0.25069994435012343814150881371460855007171630859375f,0.6824999966286124930547885014675557613372802734375f,-0.358799956169724598709080964908935129642486572265625f,1.09559999121874529492970395949669182300567626953125f,2.186900033926963970998258446343243122100830078125f,-0.246999957442282591646431910703540779650211334228515625f,-0.030199990379809804608157719485461711883544921875f,0.88099999738484580102948484636726789176464080810546875f,-0.589899948707223842347957543097436428070068359375f,-0.092300022062659137223050720422179438173770904541015625f,0.08789996496439034245895527419634163379669189453125f,0.24250002423301320053639074103557504713535308837890625f,-0.6021999972939493517287701251916587352752685546875f,1.426000027447938744984412551275454461574554443359375f,-0.546800004464387701119676421512849628925323486328125f,-0.557799966049194129169563893810845911502838134765625f,0.876299958628415698314029214088805019855499267578125f,-0.4884999621063474961601968971081078052520751953125f,0.77529993907809302555733665940351784229278564453125f,-0.5143000657081613358201366281718946993350982666015625f,-0.7594000395148976512160743368440307676792144775390625f,1.0200000751018531985891968361102044582366943359375f,0.37850002640485680416304603568278253078460693359375f,-0.042100042709709573074405852821655571460723876953125f,0.549999935030938136293343632132746279239654541015625f,-1.4476999722748986609843768746941350400447845458984375f,-0.230699943158030773560085435747168958187103271484375f,0.14309997777044802802492995397187769412994384765625f,0.5974999941140406445327926121535710990428924560546875f,-0.1903000158935788110881048851297236979007720947265625f,-0.1126000245451936887519650554168038070201873779296875f,0.85020004920214475863105008102138526737689971923828125f,0.916500034876168268027640806394629180431365966796875f,-1.3606000430747868801262256965856067836284637451171875f,-0.465199990752339420652106127818115055561065673828125f,-1.164200011453777161563039044267497956752777099609375f,-0.028800014097988491101887120748870074748992919921875f,0.7235000120997430173019893118180334568023681640625f,0.9208999447137120597517423448152840137481689453125f,1.23789999983310661235691441106610000133514404296875f,0.9371000220715988149322583922185003757476806640625f,-0.40559998041540412661021264284499920904636383056640625f,0.46799998038262202726400573737919330596923828125f,1.341500000193715180785147822462022304534912109375f,0.1164000070899720018502421226003207266330718994140625f,-1.0799000001907348433860533987171947956085205078125f,0.2865000080168247365008937777020037174224853515625f,0.957500035874545574188232421875f,0.06420000191703445491242518983199261128902435302734375f,0.0060000379905111866918332452769391238689422607421875f,1.2687000347591936844082738389261066913604736328125f,0.5281000197172163179715198566555045545101165771484375f,-0.52479999892860629273627637303434312343597412109375f,0.421699984836578778413240797817707061767578125f,-1.04379999118745292463472651434130966663360595703125f,-0.200900014078616351298478548415005207061767578125f,-1.2499000036552541104128977167420089244842529296875f,-0.826099984602630232899400652968324720859527587890625f,0.43049998688325263884735250030644237995147705078125f,0.7984999874755744908583210417418740689754486083984375f,-0.155399979269504273649005199331440962851047515869140625f,-0.044599960827827800358136300928890705108642578125f,0.41979999367594722503582715944503434002399444580078125f,-0.39360002219080936214368193759582936763763427734375f,0.1795000381693252489156975570949725806713104248046875f,-0.0367000015258784895166854767012409865856170654296875f,-0.92580005899146289305434720517951063811779022216796875f,-0.50779999991208291110211803243146277964115142822265625f,-1.0834000607937575022532428192789666354656219482421875f,-0.63129995812550154798969970215694047510623931884765625f,0.2917999840497973451647339970804750919342041015625f,-0.8499999621510507807897738530300557613372802734375f,-0.50249999731779126932451617904007434844970703125f,0.3470000176429746119310948415659368038177490234375f,0.614899977801739883176423973054625093936920166015625f,0.210499980494380789508568341261707246303558349609375f,0.32510001858472781233189152771956287324428558349609375f,-0.479399988739192650655240868218243122100830078125f,-0.564400009191036389211149071343243122100830078125f,0.772299981904030374835201655514538288116455078125f,-0.572000001847744243832494248636066913604736328125f,-0.515800000970065841698897202149964869022369384765625f,1.056700018811226104986644713790155947208404541015625f,0.0097999950170517546865767144481651484966278076171875f,-0.4498000085771085565511384629644453525543212890625f,0.05650000112503850147049888619221746921539306640625f,1.11670000439435224137696422985754907131195068359375f,-0.32620005910396654424943108097068034112453460693359375f,0.19130002620518216094325225640204735100269317626953125f,-0.152400009317695595001396213774569332599639892578125f,0.952900006687640743763267892063595354557037353515625f,-0.059600042821467358322706786566413938999176025390625f,-0.591400042927265889858290393021889030933380126953125f,0.31470003745257890681585877246106974780559539794921875f,-0.1489999975860119008075344027020037174224853515625f,-0.8501000096067785616327228126465342938899993896484375f,-0.7294000288605690940357817453332245349884033203125f,-0.288100011968612790269617107696831226348876953125f,1.02279995591640471275240997783839702606201171875f,-0.60879999653324501363016452160081826150417327880859375f,-1.4320000218153001725340800476260483264923095703125f,-0.68300002860650443903267614587093703448772430419921875f,0.577699960540235313288803808973170816898345947265625f,-0.08709997105374911452457808991312049329280853271484375f,-0.1822999793335797757976024513482116162776947021484375f,-1.02360002710819220084204062004573643207550048828125f,1.0152000145196922176182852126657962799072265625f,0.6774000553250318912290595108061097562313079833984375f,-0.785899996346234530619767610915005207061767578125f,-0.156400016701221833415047512971796095371246337890625f,0.616800018918514325605428894050419330596923828125f,1.1345000357106329591516669097472913563251495361328125f,0.004700006532669220860043424181640148162841796875f,-0.03030000119730835717035688503528945147991180419921875f,-0.938599965119362167342842440120875835418701171875f,1.32210000985264830575260930345393717288970947265625f,0.35350000107288348516476617078296840190887451171875f,-1.005500033617019450815632808371447026729583740234375f,-0.75899995854496982161663254373706877231597900390625f,1.651900026960670953002363603445701301097869873046875f,-0.875400016149879167670633250963874161243438720703125f,-0.934499999314545970463541380013339221477508544921875f,-0.48599997100979075437265919390483759343624114990234375f,2.20730007771849745523695673909969627857208251953125f,0.63829997850656372548883155104704201221466064453125f,-1.789600054556131691896325719426386058330535888671875f,-0.06569999046921726204573133145458996295928955078125f,1.19770003195405028151299120509065687656402587890625f,0.625399980014562206775963204563595354557037353515625f,-0.841700020599365217321974341757595539093017578125f,-0.155299997043609749880488379858434200286865234375f,0.2913999923378229883752510431804694235324859619140625f,-0.1195000049769874284066872860421426594257354736328125f,-1.023200020942092169917714272742159664630889892578125f,0.94389995854943886666177377264830283820629119873046875f,1.95620006685256964118480027536861598491668701171875f,-0.6627999445915209175694826626568101346492767333984375f,-1.3942000432491301609161382657475769519805908203125f,-0.6774000396415600366850640057236887514591217041015625f,-0.531400013422965944442921681911684572696685791015625f,0.0441999903127550108905552406213246285915374755859375f,0.87320002150088538162009399457019753754138946533203125f,-0.523900009660422938395640812814235687255859375f,-0.83230002176463624419255893371882848441600799560546875f,0.116500007979572384186894851154647767543792724609375f,0.87540004453659125971398680121637880802154541015625f,-0.29120004803985410912758879931061528623104095458984375f,1.22600001299381222708007044275291264057159423828125f,-0.059200001302361471289259498007595539093017578125f,-1.65699999260902419706553700962103903293609619140625f,-0.194100017064809560185523196196299977600574493408203125f,1.0382999957919114564219853491522371768951416015625f,-0.127100017359108363734776503406465053558349609375f,0.723500013589859491958122816868126392364501953125f,1.252800063316524603607149401796050369739532470703125f,-0.146999983072280571150258765555918216705322265625f,-0.787400025336444375767541714594699442386627197265625f,1.274400015859305401733081453130580484867095947265625f,-0.0189999948665497964128690000507049262523651123046875f,-0.7713000372692950801223332746303640305995941162109375f,-0.8647000465795404711144556131330318748950958251953125f,-0.0144000288605692361443288973532617092132568359375f,-0.631699988431483117068410138017497956752777099609375f,0.7130999848484993464126091566868126392364501953125f,-0.6332000104740262980129728020983748137950897216796875f,0.8475999756693841646892906283028423786163330078125f,-0.978300008830428335926399086019955575466156005859375f,-0.7594000068813564485736833375995047390460968017578125f,0.94020002226829557656628821860067546367645263671875f,-0.47909996744394245382636654539965093135833740234375f,0.605800014269352260498635587282478809356689453125f,0.250499959707259023389269714243710041046142578125f,-1.56700006845593531323856950621120631694793701171875f,-1.53000000149011494698925162083469331264495849609375f,1.32200001574307623997128757764585316181182861328125f,-0.148399986138939343760512201697565615177154541015625f,-1.25460006662607259642072676797397434711456298828125f,-0.57739996655136305303557264778646640479564666748046875f,1.01740004101991754481559837586246430873870849609375f,-1.12339999619722430423962578061036765575408935546875f,0.75379999752044657412852757261134684085845947265625f,-0.6204999915957447598202634253539144992828369140625f,-0.202900011754036047051386049133725464344024658203125f,0.272700015674531837106542297988198697566986083984375f,0.96630003075003667589726319420151412487030029296875f,-1.45390004449188747770449481322430074214935302734375f,-0.6514000582754622126202548315632157027721405029296875f,0.1980000570863484909978069481439888477325439453125f,0.544100031891466162647930104867555201053619384765625f,-0.053200017477572192348134194617159664630889892578125f,-0.03929999623149649234932212493731640279293060302734375f,-1.125099931114912354956913986825384199619293212890625f,-1.11990001188814591870368531090207397937774658203125f,-0.87570004780143440026307644075131975114345550537109375f,-0.430100018845498810460270533440052531659603118896484375f,1.1793999759241937663745147801819257438182830810546875f,0.39420000392869092298298028254066593945026397705078125f,1.282800039772689526529347858740948140621185302734375f,0.182900022780895454843630432151257991790771484375f,-0.58580002499818828454181129927746951580047607421875f,0.3440000159144407376743401982821524143218994140625f,0.719100042359531510527403952437452971935272216796875f,-0.1304000221848493623610920622013509273529052734375f,-0.6949000488244003737037246537511236965656280517578125f,-0.46890002146959286477567729889415204524993896484375f,0.644600010597705885828645477886311709880828857421875f,0.243900046913325230235614071716554462909698486328125f,0.5797999871194361443116349619231186807155609130859375f,0.7893000194028021443415354951866902410984039306640625f,1.180100029797852112523059986415319144725799560546875f,-0.10689997223615677057750872336328029632568359375f,-0.617700009099393820832801793585531413555145263671875f,-0.4126000486850738280253381162765435874462127685546875f,-0.9138000208407646152153347429702989757061004638671875f,-0.22470001666545869056790252216160297393798828125f,0.397800014142692059948558380710892379283905029296875f,0.43620001142025000007151902536861598491668701171875f,1.495600071871281411262089022784493863582611083984375f,-0.61350003549456655349558786838315427303314208984375f,-0.823100048848987686511691208579577505588531494140625f,0.955200015711784811145435014623217284679412841796875f,1.1052000523686411614932012525969184935092926025390625f,0.058799925473332681491456241928972303867340087890625f,-0.237999982319772929173495867871679365634918212890625f,-0.051799997386335672189261458697728812694549560546875f,0.82059999647140513534537831219495274126529693603515625f,0.12119995401352678943140972478431649506092071533203125f,-1.419700005396455022577129057026468217372894287109375f,-1.133199927511812088454234981327317655086517333984375f,-1.094200043025612689717718239990063011646270751953125f,0.47400003962591263917403239247505553066730499267578125f,0.68780001432895687685231678187847137451171875f,0.714599985563754813711057067848742008209228515625f,-0.08980000943392540957432856885134242475032806396484375f,0.8125000251829626041200071995262987911701202392578125f,0.71380003633797228790314193247468210756778717041015625f,1.57890001483857655983911172370426356792449951171875f,-0.1335999791264530500711771310307085514068603515625f,1.1493999902665616996699782248470000922679901123046875f,0.29009999559968646654084523106575943529605865478515625f,0.06270001142770009749938253662548959255218505859375f,-0.42139999721795307063842983552603982388973236083984375f,-0.080199994738400059990368617945932783186435699462890625f,-0.16349999880045640310299859265796840190887451171875f,-0.32880000679641929739460692871944047510623931884765625f,0.4261999859392637812760540327872149646282196044921875f,-1.53220000314265547558534308336675167083740234375f,0.011699973213672887339953376795165240764617919921875f,0.59330000231117008269166035461239516735076904296875f,0.67030003206133859094961735536344349384307861328125f,-0.60240004042387074179032424581237137317657470703125f,-0.6117000435099011479422870252165012061595916748046875f,-0.39210002099126606278645112979575060307979583740234375f,-1.5640999969482418663346834364347159862518310546875f,-0.103299999684840049241074666497297585010528564453125f,-0.269100001083314221972386803827248513698577880859375f,0.33280000702738765649968399884528480470180511474609375f,-0.461199994730949303090028479346074163913726806640625f,0.2068999951839446682644307884402223862707614898681640625f,-0.566699989381432178703335011960007250308990478515625f,-0.2579999999590214532219079046626575291156768798828125f,0.546299999830127003264124141423963010311126708984375f,0.3464000208921731438493907262454740703105926513671875f,0.0754999594092371495435145334340631961822509765625f,-1.112800029248744326793030268163420259952545166015625f,-0.7054999805688861869157335604541003704071044921875f,0.07529998599812415260856823806534521281719207763671875f,0.340000026002526656299096430302597582340240478515625f,0.09190003154277859298559860690147615969181060791015625f,1.028900005786120619433177125756628811359405517578125f,-1.06509997507333675770269110216759145259857177734375f,0.70090000185966427892481078743003308773040771484375f,0.432000013507902291820528262178413569927215576171875f,1.001399981683492423911729929386638104915618896484375f,-0.1259999993592499922812066870392300188541412353515625f,0.43679996672719745287594150795484893023967742919921875f,-0.23400000213086580291843574741506017744541168212890625f,-0.2511999929800630138032602189923636615276336669921875f,1.2553000053882594766463398627820424735546112060546875f,-0.8327000040143730519304199333419091999530792236328125f,-0.4105000418126589778466950519941747188568115234375f,0.18879998202323999834106871276162564754486083984375f,0.473300041091441858753796623204834759235382080078125f,-0.39580000011324845132776317768730223178863525390625f,-0.728500002712010985561619236250407993793487548828125f,-0.2036000122442838122793773436569608747959136962890625f,1.156700011286139417876484003500081598758697509765625f,-1.21449996392428882785452515236102044582366943359375f,-0.1627000179097051191234868383617140352725982666015625f,-0.369999986365437738555783653282560408115386962890625f,-0.165100002193451000920276783290319144725799560546875f,0.100699992518127334761857127887196838855743408203125f,2.184700041997432951035307269194163382053375244140625f,0.515600040541590143305938909179531037807464599609375f,-2.0920000668168068358454547706060111522674560546875f,-0.0104000782504682920404093238175846636295318603515625f,2.0844000229001036217368891811929643154144287109375f,1.5045000383555884315711637100321240723133087158203125f,-0.48129996145963627185437871958129107952117919921875f,-0.7600999902725218415611152522615157067775726318359375f,-0.19589999262094437426640070043504238128662109375f,0.0411999999463557031731397728435695171356201171875f,1.5800999965310094097503679222427308559417724609375f,-0.52419999202638845048340954235754907131195068359375f,0.317499971091747656970483149052597582340240478515625f,-0.55170000383555872769392180998693220317363739013671875f,-1.681800012511015296468031010590493679046630859375f,-0.60060004283264289970389882000745274126529693603515625f,0.64639997125267978361762288841418921947479248046875f,-0.205699988289922419681943210889585316181182861328125f,-2.340000008940695153114575077779591083526611328125f,-0.0091999808132643412506013191887177526950836181640625f,0.802699976968764161711078486405313014984130859375f,-1.154000050187111270361128845252096652984619140625f,-1.186499996542931256726660649292171001434326171875f,1.2952999900773161101596997468732297420501708984375f,1.1564000003099437208220479078590869903564453125f,-0.36569999466016878120200317425769753754138946533203125f,-1.4277999970436088261749318917281925678253173828125f,0.8257999941900366902558516812860034406185150146484375f,-0.7675999464631078073040271192439831793308258056640625f,0.28529998482465723785850286731147207319736480712890625f,0.9979999718144541009934300745953805744647979736328125f,-0.230800016504526528393625994795002043247222900390625f,-0.56709997468590689351231048931367695331573486328125f,1.682500083148480651829004273167811334133148193359375f,0.406900033479927980550883148680441081523895263671875f,-2.2887000689387324570134296664036810398101806640625f,-0.54100003385543882217234568088315427303314208984375f,1.4558000339388836863463438930921256542205810546875f,1.187900017416477993492662790231406688690185546875f,-0.139299997460842206464803894050419330596923828125f,-1.0554000427484506730024804710410535335540771484375f,-0.15790000162124595561863316106610000133514404296875f,0.2143000210046768216898271930404007434844970703125f,0.2694000112771988142412737943232059478759765625f,-1.05060003152638703483034987584687769412994384765625f,-0.9354000442385679292556233122013509273529052734375f,-0.0181000539153810535708544193767011165618896484375f,-0.094600017154217397319371229968965053558349609375f,0.8455999915540222122700697582331486046314239501953125f,0.37040000162124631089000104111619293689727783203125f,-0.1133000116989020256141884601674973964691162109375f,-0.07789998224973704310514222015626728534698486328125f,-0.0974000282049178878196471487171947956085205078125f,-0.4041000140845778521025977170211263000965118408203125f,-0.525099963338673259016786687425337731838226318359375f,0.020999990567565074428557636565528810024261474609375f,-0.02549998883903004109896528461831621825695037841796875f,1.4261000329568986888517656552721746265888214111328125f,0.0647000037014484608022257816628552973270416259765625f,-0.01879999325498926676658584256074391305446624755859375f,0.874300048422814057857976877130568027496337890625f,0.03700003075599767043968313373625278472900390625f,0.43700001903995866481267285053036175668239593505859375f,0.51880000467300391164826578460633754730224609375f,-0.66070000389888894343215497428900562226772308349609375f,0.3396999662995341395799187012016773223876953125f,-0.28759998494535687907358578740968368947505950927734375f,0.293600022339820299066559528000652790069580078125f,0.5251000010013580077128381162765435874462127685546875f,-0.869400023794174359181852196343243122100830078125f,-0.459600022201985591863149238633923232555389404296875f,-0.48060007872581511634280104772187769412994384765625f,-0.710100026202947542941501524182967841625213623046875f,-0.32120003615617775949431234039366245269775390625f,-0.40550001201033591957667567839962430298328399658203125f,-0.3299999922513963124259817050187848508358001708984375f,0.079699995226413078430738323731929995119571685791015625f,0.164599974536894677612508530728518962860107421875f,-0.8103000450253485809071207768283784389495849609375f,-1.5759999998807909804554583388380706310272216796875f,-1.0642000357985494929380365647375583648681640625f,-0.6357999200940138706528159673325717449188232421875f,-0.0963000034809109450861797085963189601898193359375f,0.613899992561339757912719505839049816131591796875f,0.9589000405430798679162762709893286228179931640625f,0.68729997892379746105007143341936171054840087890625f,-1.215100026705860702946893070475198328495025634765625f,-0.465899974739552025226885234587825834751129150390625f,0.5961999631404875099605078503373079001903533935546875f,0.434399994811415890438865972100757062435150146484375f,0.98900004424527299118352630102890543639659881591796875f,-0.60370000255405897870986109410296194255352020263671875f,-0.074299987998604766215038353038835339248180389404296875f,-0.12620000253543250945398312978795729577541351318359375f,-0.5812999952852724305785159231163561344146728515625f,0.19500000528991223713859426425187848508358001708984375f,0.13969994186162981719689923920668661594390869140625f,-0.27799996834993334626773275886080227792263031005859375f,0.95009997760653508436234915279783308506011962890625f,-0.2900999747008083939903144710115157067775726318359375f,1.35239996286332608832481128047220408916473388671875f,1.6874999783188098234631979721598327159881591796875f,1.395600008168816241749254913884215056896209716796875f,-0.419399969851970144674169205245561897754669189453125f,0.425400001025199259174769395031034946441650390625f,0.38480002213716435477408595033921301364898681640625f,0.63410001423358952621356365852989256381988525390625f,-0.17960004814863150812698222580365836620330810546875f,0.59619999346435026410517821204848587512969970703125f,0.06410000350475308916742278597666881978511810302734375f,0.76239999795556112616168320528231561183929443359375f,0.63600000002980205948688308126293122768402099609375f,-0.58939998639225965604282464482821524143218994140625f,0.08929999582171443961442491854541003704071044921875f,-0.222099969917536554930848069489002227783203125f,-0.788999995946884524755660095252096652984619140625f,-1.1747999588072304266717083010007627308368682861328125f,-1.1626999619930984408711083233356475830078125f,0.2949999707192187958781914858263917267322540283203125f,0.316700048390031962952662070165388286113739013671875f,-0.16119998117089284761505041387863457202911376953125f,-0.059500038802624555245301962713710963726043701171875f,-0.019299997013807246304395448532886803150177001953125f,-1.068899985483287906617988483048975467681884765625f,-0.62349999257922117390506855372223071753978729248046875f,0.2051999942541116173089221774716861546039581298828125f,-0.009000007346272820285548732499592006206512451171875f,-0.37350002036988738662870446205488406121730804443359375f,-0.11400000682473176993880770169198513031005859375f,0.24240000009760243049328209963277913630008697509765625f,0.7665000204592942179004921854357235133647918701171875f,-0.51760002612844147051873733289539813995361328125f,0.63260003117620999812942272910731844604015350341796875f,0.0675999812945719258383547867197194136679172515869140625f,-0.404299989805370574291742968853213824331760406494140625f,-0.43390002342537048463100290973670780658721923828125f,-0.3780999952793120399974213796667754650115966796875f,-0.366500020403414783487505701486952602863311767578125f,-0.05120000277757608131423694430850446224212646484375f,-0.427000000245868982506891597950016148388385772705078125f,-0.9434999846071006057712793335667811334133148193359375f,-0.3445000285021964092635471388348378241062164306640625f,0.844699996325374069527924802969209849834442138671875f,0.51050001091137520159435325695085339248180389404296875f,-0.475899996793270219086480210535228252410888671875f,-0.24540001793801735630040639080107212066650390625f,-0.378000012531876483645731923388666473329067230224609375f,0.6765999768689265891907780314795672893524169921875f,1.40569999596401995489713954157195985317230224609375f,2.64750002671033168866188134416006505489349365234375f,0.87159997711107128903762486515915952622890472412109375f,0.68000001998618220255821142927743494510650634765625f,0.016200004938244916274925344623625278472900390625f,-1.0847999783277515462032170034945011138916015625f,-1.43379997472167008965016066213138401508331298828125f,-0.510199993956089059565783827565610408782958984375f,0.5023999860346315404768802181934006512165069580078125f,-0.161699979062378762506568818935193121433258056640625f,-0.39389997635632767458702119256486184895038604736328125f,0.3905999836936595581704523283406160771846771240234375f,0.254299990755319615942653399542905390262603759765625f,1.2540999963521954896350507624447345733642578125f,0.1583000019758937781944041489623486995697021484375f,-1.031300014731287628677591783343814313411712646484375f,-0.172299996469914784569255061796866357326507568359375f,-0.7277000029340385811593705511768348515033721923828125f,0.059500018145889566056894182111136615276336669921875f,1.332600001075864071253818110562860965728759765625f,1.648900029125064836676983759389258921146392822265625f,0.88329999117255209029053730773739516735076904296875f,1.56820001058578473163152011693455278873443603515625f,-0.55379995847940488573613038170151412487030029296875f,0.45910003837347002075830459943972527980804443359375f,0.46749997571110790683945879209204576909542083740234375f,-0.3096999939039346561031607052427716553211212158203125f,-1.180899977310002402219879513722844421863555908203125f,0.287100014360249122802315469016321003437042236328125f,0.15580001233220119871702991076745092868804931640625f,-0.50209998407363887196197538287378847599029541015625f,0.20540000653862922064263329957611858844757080078125f,0.66429999302774678238137084917980246245861053466796875f,-0.8430000069439411714000698339077644050121307373046875f,-0.912999994389712465903130578226409852504730224609375f,0.172000000208615944075063453055918216705322265625f,-0.426700028981268297822992963119759224355220794677734375f,-0.2695000056475398420019473633146844804286956787109375f,-0.28509999131560281870889639321831054985523223876953125f,0.32340001325905287643536212272010743618011474609375f,0.7407000446721923037074475359986536204814910888671875f,-0.6570000164508822404485499646398238837718963623046875f,-0.27039998936504072712949664492043666541576385498046875f,1.06780003474354767689646905637346208095550537109375f,0.5204000150881709085837201200774870812892913818359375f,-0.505200012135505804877766422578133642673492431640625f,0.28200001442059885459201495905290357768535614013671875f,0.417400007939338735241108224727213382720947265625f,-0.127300009322166118863606243394315242767333984375f,-0.4719999932795762020987240248359739780426025390625f,-0.9356000185251236445083122816868126392364501953125f,-0.021400029516219998271253643906675279140472412109375f,-1.48159997206330285024478143895976245403289794921875f,0.434000040873885328807091354974545538425445556640625f,-1.101900022117793032094823502120561897754669189453125f,-0.48240002123862468774717626729398034512996673583984375f,-0.8081000099197035435594216323806904256343841552734375f,0.03170004079043931000114753260277211666107177734375f,-0.1017999977588655013960305950604379177093505859375f,-0.433300016728043857483498868532478809356689453125f,0.4157000149071219397001186734996736049652099609375f,0.5913000115275384160895555396564304828643798828125f,1.19530000002384184654147247783839702606201171875f,0.164099996724724750407631290727294981479644775390625f,0.1320000081062318031399627216160297393798828125f,1.1453000164151188045025264727883040904998779296875f,0.617599992209672876697368337772786617279052734375f,0.14649998044967649235559292719699442386627197265625f,0.7204999834001066805200252929353155195713043212890625f,0.9031999974727629165727194049395620822906494140625f,0.39040000028014099076045795300160534679889678955078125f,0.6381999912515279316238547835382632911205291748046875f,0.33940000520497581870671410797513090074062347412109375f,1.0481000241503115599783768630004487931728363037109375f,-0.18870001350641274484587484039366245269775390625f,-1.1322000758230696959572014748118817806243896484375f,-0.6886999941349027931636328503373079001903533935546875f,-0.018899982726573671953929078881628811359405517578125f,-0.32780001060366625864617162733338773250579833984375f,-1.0999000411689283662752814052510075271129608154296875f,0.27020000036358826633886565105058252811431884765625f,0.18409999858736991740215671597979962825775146484375f,-0.425200014221668087799343993538059294223785400390625f,-0.8705999920569358341282395485904999077320098876953125f,0.1467999985784291805401835517841391265392303466796875f,-0.4934000197969374834627842574263922870159149169921875f,0.71060000913739163053151060012169182300567626953125f,0.82160000631734730891508888817043043673038482666015625f,-0.59830000428557372771365407970733940601348876953125f,-0.704600036581606303798253065906465053558349609375f,0.9485000500977045501116435843869112432003021240234375f,0.25920000269934517778125382392317987978458404541015625f,-0.4715000232905151023032885859720408916473388671875f,-0.35130000854730603077058503913576714694499969482421875f,-0.10910000283420107081155947525985538959503173828125f,-0.78720003570169228002129102605977095663547515869140625f,-0.3813000031828881475348680396564304828643798828125f,-0.14460000500977010506886699658934958279132843017578125f,0.15960000702142718154163958388380706310272216796875f,-0.465399991488456787891436761128716170787811279296875f,0.43490000726878663694918714099912904202938079833984375f,-1.2686999768495557106007254333235323429107666015625f,-0.4814000370413065166275146111729554831981658935546875f,0.4137999352335928193724612356163561344146728515625f,1.431200039732456463781318234396167099475860595703125f,1.35700000929832498286486952565610408782958984375f,0.450299996224045262493973496020771563053131103515625f,-0.4278999920845034665717321331612765789031982421875f,0.24110000572502610083347462932579219341278076171875f,0.173099984587728794593886050279252231121063232421875f,0.11159996509700997879832584658288396894931793212890625f,-0.9943000095680349925686414280789904296398162841796875f,0.3226000161260371701388294241041876375675201416015625f,-1.720000017285347126971828402020037174224853515625f,-0.2832999949723482568941790304961614310741424560546875f,-0.9855000231862067749943889793939888477325439453125f,-0.8068000137776134561562457747641019523143768310546875f,0.8588000011712308978673036108375526964664459228515625f,0.59870001626312696618725794905913062393665313720703125f,-1.4671999965488904393851044005714356899261474609375f,0.2380000093281271489331629709340631961822509765625f,0.70699999067187224710551163298077881336212158203125f,-0.55590004209280063918185987859033048152923583984375f,0.6013000021398067218569849501363933086395263671875f,1.88800007995963170515096862800419330596923828125f,0.14370001946687727123475042390055023133754730224609375f,0.3334000063672661884339731841464526951313018798828125f,1.2033000138595699723964571603573858737945556640625f,1.21360002885907913849905526149086654186248779296875f,0.88419997977018294221807082067243754863739013671875f,1.25269999708533230631246624398045241832733154296875f,0.0817000095725060049289822927676141262054443359375f,0.746300015103817049322287857648916542530059814453125f,-0.5534000161647798510244911085464991629123687744140625f,-0.41530000943019995673921584966592490673065185546875f,0.4464000080585481811112913419492542743682861328125f,1.1190000589601700742292678114608861505985260009765625f,-0.38440003064870864601942912486265413463115692138671875f,-0.217300033387542834706351868589990772306919097900390625f,0.1646000117152934816289189257076941430568695068359375f,0.065700026325136218208200489243608899414539337158203125f,0.8422999864488838195342168546631000936031341552734375f,0.914300006904452455813725464395247399806976318359375f,-0.356999987170099952749069416313432157039642333984375f,-0.386299963117390288136476783620310015976428985595703125f,1.0351999838232988082609153934754431247711181640625f,-0.37900003632903056516312290114001370966434478759765625f,-0.144899987655133066777324302165652625262737274169921875f,-0.1229000258915127208325657193199731409549713134765625f,-0.813900002396106714286361238919198513031005859375f,-1.0970999876499174519040025188587605953216552734375f,0.419000037893653143328265286982059478759765625f,-0.26709999912381210407374965143389999866485595703125f,-0.8134000150471927259587801017914898693561553955078125f,-0.79210000255107804179033337277360260486602783203125f,-0.033099965849519019656099771964363753795623779296875f,-0.713100002804398780398287271964363753795623779296875f,0.08659998927414402913882440770976245403289794921875f,0.916599987895786671998621386592276394367218017578125f,1.278300010953843379724048645584844052791595458984375f,0.7884000226840373048986521098413504660129547119140625f,0.1726999738208954138229955788119696080684661865234375f,-0.4067999876260757208257246020366437733173370361328125f,-1.01430003141686331247228736174292862415313720703125f,0.40029997176751541676509305034414865076541900634765625f,-0.03419999817311758505411489750258624553680419921875f,0.633799991559982345279422588646411895751953125f,-0.014500008553266496846845257095992565155029296875f,-0.6915000314116481749948661672533489763736724853515625f,-0.37710002654194862969916357542388141155242919921875f,0.87450004580616980121021697414107620716094970703125f,-0.3291999990671872211578374844975769519805908203125f,-0.52980003852844281819756133700138889253139495849609375f,-1.003600003191828182735889640753157436847686767578125f,-0.93529997916221574438822017327765934169292449951171875f,-0.4993999888882039162041337476694025099277496337890625f,0.69819995548874158597385530811152420938014984130859375f,-0.63220002907067485153902453021146357059478759765625f,0.31210001313090340868683369990321807563304901123046875f,1.294599990835040070891182040213607251644134521484375f,-0.49409997765123858659563893525046296417713165283203125f,0.41520002228692176604596397737623192369937896728515625f,-0.3221000074684621239384796353988349437713623046875f,-0.673000010073184995462725055404007434844970703125f,-0.1764000140935184646195921231992542743682861328125f,0.0297999855175611616520114921513595618307590484619140625f,-0.407300022956729179668400320224463939666748046875f,-0.7086000164911150367430536789470352232456207275390625f,0.0249000042140483657249205862171947956085205078125f,0.062300004125387019315862602297784178517758846282958984375f,-0.56510000904798507459503298377967439591884613037109375f,0.1309999980367718974694213329712511040270328521728515625f,-0.238599986986815781886406284684198908507823944091796875f,-0.2962999932922423251824284307076595723628997802734375f,-0.04469999524503942556208357927971519529819488525390625f,-0.20039999048262802094910739469924010336399078369140625f,0.00219999505877499501593774766661226749420166015625f,0.6942999936424187890082748708664439618587493896484375f,-0.2422999956130975629520207803579978644847869873046875f,-0.7526999935463074553609885697369463741779327392578125f,0.313099981421232076428395885159261524677276611328125f,-0.448600012765824285310145569383166730403900146484375f,-1.2007000156894325204604001555708236992359161376953125f,-0.4177000196233391360323139451793394982814788818359375f,-0.8148000048518178051182303534005768597126007080078125f,0.3193000180244442987032016390003263950347900390625f,-0.4491999852091073108795171719975769519805908203125f,0.6699999871850017552077360960538499057292938232421875f,-0.048799994316697181684361339648603461682796478271484375f,0.02650000376999361861862780642695724964141845703125f,-0.00049999826401470048864439377211965620517730712890625f,-0.7467999599471693006336181497317738831043243408203125f,0.77470002202987675588019556016661226749420166015625f,-0.559099996575712854252060424187220633029937744140625f,-0.7401999843448405780321763813844881951808929443359375f,-1.638399975931644547699761460535228252410888671875f,-0.1595999968141317282288582646287977695465087890625f,-0.87089998770356213952936741407029330730438232421875f,0.1155000145807865141733827840653248131275177001953125f,-0.56739999985545874761072582259657792747020721435546875f,-0.8074000114038586417564147268421947956085205078125f,-0.89690000591278096209180148434825241565704345703125f,-0.626499998182058703832808532752096652984619140625f,-0.670099973061681009056655966560356318950653076171875f,-1.002000002108514475906986262998543679714202880859375f,0.0729999909624456932988323387689888477325439453125f,-0.507199992898106355454501681379042565822601318359375f,0.399999979138374539555655928779742680490016937255859375f,0.2908999966815113413787230456364341080188751220703125f,0.3533999962717293552572073167539201676845550537109375f,0.35469998321235163274423030088655650615692138671875f,1.1108000005602836068874239572323858737945556640625f,0.464600029671192960023518025991506874561309814453125f,-0.4634000042438504163300194704788736999034881591796875f,-0.1567999938845634488870928180404007434844970703125f,0.254200043398142394579508618335239589214324951171875f,0.0308999884672458702272734853977453894913196563720703125f,0.057699976521730367284845897302147932350635528564453125f,1.18490000281706375773183026467449963092803955078125f,-0.94540000799149226384798794242669828236103057861328125f,-0.744099971038848284621280981809832155704498291015625f,-0.067500020563602713963291535037569701671600341796875f,0.222700029886514438981492958191665820777416229248046875f,0.2225999615132803199912814307026565074920654296875f,-1.12380004429146662658922650734893977642059326171875f,0.20250004887580974610727935214526951313018798828125f,0.78620004066377935370013574356562457978725433349609375f,-0.9638000618934636776202751207165420055389404296875f,-1.532900054650754295693104722886346280574798583984375f,0.922600039744377209416370533290319144725799560546875f,0.810499994389712430375993790221400558948516845703125f,-0.34190001187324536857659040833823382854461669921875f,-0.09289996228218111440355642116628587245941162109375f,0.1453999958470462150472712892224080860614776611328125f,0.030599994683265574479946735664270818233489990234375f,-0.607199976991117384983454030589200556278228759765625f,-0.9349999598041180970398045246838591992855072021484375f,-0.19400001874566097370689021772705018520355224609375f,-2.53189993855953243695466881035827100276947021484375f,-1.199600001582502795116624838556163012981414794921875f,-1.05230000857710859207827525096945464611053466796875f,0.636000030353665035676158368005417287349700927734375f,-0.132099922680854575673947692848742008209228515625f,-1.781699994596838809712835427490063011646270751953125f,-1.432100050234794252901338040828704833984375f,1.162799977672099860370735768810845911502838134765625f,0.891199950623512648917312617413699626922607421875f,0.051299990516901150527928621158935129642486572265625f,0.5955999854445455099494211026467382907867431640625f,1.374400037354231773889523537945933640003204345703125f,0.1053000173091891422672006228822283446788787841796875f,1.1728000075861813034094893737346865236759185791015625f,1.069199986177682859533888404257595539093017578125f,-0.3871000028118485314365671001723967492580413818359375f,0.9604999719262130586372450125054456293582916259765625f,0.37779999242424988636912530637346208095550537109375f,-1.086000016570090931367076336755417287349700927734375f,-0.87150007075071389550657841027714312076568603515625f,0.70089997608065512935127117088995873928070068359375f,0.608900035178661536150457322946749627590179443359375f,-0.0690999251246449119889803114347159862518310546875f,-0.46749998509883905484230126603506505489349365234375f,-1.061600026750564751409910968504846096038818359375f,-0.19169997142553274471765689668245613574981689453125f,0.565300008070468873455638458835892379283905029296875f,0.49890000016093249968207601341418921947479248046875f,-0.29420001791715666428217446082271635532379150390625f,-0.456999985810368603011966115445829927921295166015625f,1.2805000507533552678296473459340631961822509765625f,-0.22999998895451334579576041505788452923297882080078125f,0.390199993509053655316165531985461711883544921875f,0.115999960672110546955337895269622094929218292236328125f,-0.44369998966455437283684659632854163646697998046875f,0.5939000370226807934415091949631460011005401611328125f,-0.673800014805794322825249764719046652317047119140625f,-0.5248999986074858181694935410632751882076263427734375f,0.54790001175403568733912607058300636708736419677734375f,0.316399988612532434562041316894465126097202301025390625f,-1.2235000115782022067634216000442393124103546142578125f,-0.5895000131167476009608208187273703515529632568359375f,0.39019998233318331504193565706373192369937896728515625f,-0.2087999965891243203675031736565870232880115509033203125f,0.194399996320158197082861306626000441610813140869140625f,-0.095900001263618861457871389575302600860595703125f,1.23029999935328948623691758257336914539337158203125f,0.12350002372264867744888761080801486968994140625f,0.08049996477365528591008114744909107685089111328125f,0.608099982166290686791398911736905574798583984375f,-0.480299988698959179345138181815855205059051513671875f,0.8355999607086186387050474877469241619110107421875f,0.149300000068545823950216799858026206493377685546875f,0.17969997017383576576321502216160297393798828125f,-0.1761000259906051468306031893007457256317138671875f,-0.53610001548528618542377444100566208362579345703125f,-0.360600005561113601260103678214363753795623779296875f,-1.859999990984797069160094906692393124103546142578125f,-0.5405999879963696574947107365005649626255035400390625f,-1.9211000008821488282961809090920723974704742431640625f,0.00159999950006606272978615379543043673038482666015625f,-0.20909999547675262032697673930670134723186492919921875f,-0.8428000724434863144551854929886758327484130859375f,0.24999997109174731946268366300500929355621337890625f,0.93239999646544458045127612422220408916473388671875f,0.1099000310361386567592489882372319698333740234375f,1.31710000091195045257563833729363977909088134765625f,-0.3705999754607685048313214792869985103607177734375f,-0.017599986696243519190829829312860965728759765625f,0.176500006675720300108878291212022304534912109375f,-0.856600027644634831602843405562452971935272216796875f,0.10010000249147399387084078625775873661041259765625f,0.555500001356005412844751845113933086395263671875f,1.42899998283386242547976507921703159809112548828125f,0.097299988237023793402613591752015054225921630859375f,-0.25630001778602606865575808114954270422458648681640625f,0.093299992793053565520722258952446281909942626953125f,-0.1231000036984681578200451212978805415332317352294921875f,-0.08719999913796783186370475959847681224346160888671875f,-0.8535000296831132260422236868180334568023681640625f,0.0635000299811370183533654198981821537017822265625f,0.69510004248246615166095807580859400331974029541015625f,0.692700051176547759723689523525536060333251953125f,0.495700009244680384057346600457094609737396240234375f,0.3515999750435359061384588130749762058258056640625f,-1.1286999934829766711885667973547242581844329833984375f,-1.10190003020167370806348117184825241565704345703125f,-0.94710004971325434386386632468202151358127593994140625f,-1.2126000436186803455029803444631397724151611328125f,-0.58540005524680094328715540541452355682849884033203125f,-0.6309000186979769608797141700051724910736083984375f,0.507800015055388076490316962008364498615264892578125f,0.7073999939322472840785849257372319698333740234375f,-0.73209998867437209302266865051933564245700836181640625f,1.205699978995323107255899230949580669403076171875f,-0.682799996671080489107907851575873792171478271484375f,0.3118000287532805003820612910203635692596435546875f,0.6497000521302229714137865812517702579498291015625f,-0.34010003199577365506911519332788884639739990234375f,-0.31700003701448498105719409068115055561065673828125f,-0.294799983394145925785778672434389591217041015625f,-0.2239000126779071564442347153089940547943115234375f,0.07999999016523329231631578295491635799407958984375f,-0.588700032579900334184230814571492373943328857421875f,-0.129599980050324692371077617281116545200347900390625f,-0.49520000915527351281752999057061970233917236328125f,0.93850001418590611734771300689317286014556884765625f,0.5097999883115296171354202670045197010040283203125f,-1.045700005221366524210679926909506320953369140625f,-0.1945000505745415608771509141661226749420166015625f,-1.347699933308363018369391284068115055561065673828125f,0.8656000015377998124677105806767940521240234375f,-0.28739997277259821117922911071218550205230712890625f,0.348400004541873808960872338502667844295501708984375f,-0.6358000012680886658955614620936103165149688720703125f,-0.332600012773275555133523084805347025394439697265625f,-0.92029998527169232236389007084653712809085845947265625f,-0.75520001142770032398487956015742383897304534912109375f,-0.20460003745704813393757603989797644317150115966796875f,0.6520999951377508541128236174699850380420684814453125f,0.5016000017911188546548828526283614337444305419921875f,-1.591200013208389663077468867413699626922607421875f,-0.7006999923691152876159549123258329927921295166015625f,0.855399989104271174511495701153762638568878173828125f,0.47980004091262795640204785740934312343597412109375f,-0.03759996631890472063020069981575943529605865478515625f,1.1681999966904523802213589078746736049652099609375f,1.5186000119835139088309006183408200740814208984375f,-0.3709000038340686256077560756239108741283416748046875f,-0.9018000427603727331415939261205494403839111328125f,0.31790000519752492635916496510617434978485107421875f,0.1877000020027157489721503225155174732208251953125f,0.6498000035293396425828404971980489790439605712890625f,-0.269900003729760751269850516109727323055267333984375f,0.28069999935403444624171243049204349517822265625f,0.221400006121397208147527635446749627590179443359375f,0.05560001681149040475560241247876547276973724365234375f,-1.319899981713295034069233224727213382720947265625f,0.55839998453706451986278125332319177687168121337890625f,0.391199928569793797805687063373625278472900390625f,-1.348500020779669572590364623465575277805328369140625f,-1.668599993543326664990900098928250372409820556640625f,-0.24260000929012848214227915377705357968807220458984375f,0.6391000155374422231346898115589283406734466552734375f,0.2042000444225973154743769555352628231048583984375f,0.742900032615661842783083557151257991790771484375f,0.0729000100590291300051148937200196087360382080078125f,0.46299999547004677680206441436894237995147705078125f,0.242400025075674374619438822264783084392547607421875f,0.44089998323321299977806120296008884906768798828125f,-0.21859999339431457343607689836062490940093994140625f,0.162899986124038331780639055068604648113250732421875f,0.210900065003336134594746908987872302532196044921875f,0.54689999380558618202741172353853471577167510986328125f,-1.2128000521779060960625429288484156131744384765625f,-0.5303000161170956516798469237983226776123046875f,0.26979997311234438228666476788930594921112060546875f,0.76840001288652448607763290056027472019195556640625f,0.087000019691885199790704064071178436279296875f,-0.460799996909499132158316569984890520572662353515625f,0.70780000181198088338163643129519186913967132568359375f,-1.28689998158663509997268192819319665431976318359375f,-0.4805000419616707318937187665142118930816650390625f,0.7361000223398210806635688641108572483062744140625f,0.0988999895811080165231032879091799259185791015625f,1.0678000535935157078171187094994820654392242431640625f,0.23230005104541806559836913947947323322296142578125f,-1.433499997049569429918847163207828998565673828125f,-0.797900034441053396250254081678576767444610595703125f,-0.27749997869133979921940635904320515692234039306640625f,0.921199998679757214858909719623625278472900390625f,-0.0411999975249168759461326771997846662998199462890625f,0.38240001278221613656427280147909186780452728271484375f,-1.05149998875707328949147267849184572696685791015625f,-0.2808000174358487033288156453636474907398223876953125f,-0.29640007403343948677587604834116064012050628662109375f,-0.23250001370906847597552769002504646778106689453125f,1.34899999482929633387584544834680855274200439453125f,1.178500071704387863746887887828052043914794921875f,-0.5490000039190052749660253539332188665866851806640625f,-1.3421000550031667586381445289589464664459228515625f,-0.961500032901764001280753291212022304534912109375f,0.235500021994113239998114295303821563720703125f,1.450800010152905716864779606112278997898101806640625f,-0.200600018934905310885596918524242937564849853515625f,-0.988300005943327786184227079502306878566741943359375f,-0.1496000450566408179753352669649757444858551025390625f,-0.37930001306980865027895788443856872618198394775390625f,-0.4420999895870689755383864394389092922210693359375f,0.22079999100491409791402475093491375446319580078125f,0.681800006699562199941055951057933270931243896484375f,0.67550003020837923184416240474092774093151092529296875f,-0.7668999930977822288014067453332245349884033203125f,-1.3286000432386992908817546776845119893550872802734375f,0.93700005280971510046583716757595539093017578125f,0.95720002564340855855107292882166802883148193359375f,-0.207399985289573152158482116647064685821533203125f,0.31520000680834048267797697917558252811431884765625f,0.710800010991096797852151212282478809356689453125f,1.92400003528594965018783113919198513031005859375f,0.095599963986874314514352590776979923248291015625f,-1.365700030292570144041519597521983087062835693359375f,-0.214299965050817053935361400363035500049591064453125f,1.238699995028972100641340148285962641239166259765625f,0.35639997695237457531192148962873034179210662841796875f,0.3591000350579618416446692208410240709781646728515625f,-0.4968999929487705191633040158194489777088165283203125f,1.075300028634071480837519629858434200286865234375f,-0.4388000465825203644243401868152432143688201904296875f,-2.20810003283023892350911410176195204257965087890625f,-0.59199999201297703876889499952085316181182861328125f,1.8835000824332244206971154198981821537017822265625f,0.7316999669551851326332325697876513004302978515625f,-1.097400044596195556323436903767287731170654296875f,0.3532999966859815543784861802123486995697021484375f,-0.07670000186115510931728067589574493467807769775390625f,-1.681199967759848146187096062931232154369354248046875f,-0.02380000132024229220206734680687077343463897705078125f,-1.1740000279471274335918451470206491649150848388671875f,-0.51060004802942327017944990075193345546722412109375f,0.7202999891087416006740795637597329914569854736328125f,0.587700009006261847588348246063105762004852294921875f,1.45820000425279072686635117861442267894744873046875f,0.02640003771185917003094800747931003570556640625f,1.8234000458180907866534425920690409839153289794921875f,-0.19380000795125962298470767564140260219573974609375f,0.134100036324560800693461715127341449260711669921875f,-0.97790001652240743368338371510617434978485107421875f,-1.43490001252144594445780967362225055694580078125f,0.8045000090003018300421899766661226749420166015625f,0.505799975414574110033072429359890520572662353515625f,1.015800004136561796030946425162255764007568359375f,0.4769000145554542768877581693232059478759765625f,1.3608000191494828978733266922063194215297698974609375f,0.473200029864162541315408816444687545299530029296875f,-0.307499986290931559551609097979962825775146484375f,-0.60040001665279252307527713128365576267242431640625f,-0.1019000164926051610336799058131873607635498046875f,-0.3321000007070600279490690809325315058231353759765625f,-0.48239997962713232482201419770717620849609375f,-0.1978000123545526900414870397071354091167449951171875f,-0.08509999414682312135482789017260074615478515625f,0.518600026325881646727111728978343307971954345703125f,0.2719000163435936201494769193232059478759765625f,0.128100011298060645348328989712172187864780426025390625f,0.685999982595443924537903512828052043914794921875f,-1.5005000087507063266656359701300971210002899169921875f,-0.66250000178813905904462444595992565155029296875f,-0.08020000915527336626809074004995636641979217529296875f,-0.479600013354420706690461884136311709880828857421875f,1.301000006437301959039132270845584571361541748046875f,-0.37560003920048423697863881898229010403156280517578125f,-1.149700039017201191882122657261788845062255859375f,-0.8180999499797820817548199556767940521240234375f,-0.14649999520182621637331976671703159809112548828125f,-0.076399994051456587840220890939235687255859375f,-1.0169000158965584290626793517731130123138427734375f,0.449000018745661311214689703774638473987579345703125f,-0.3686000443935402159922887221910059452056884765625f,0.2155000121593477757642176584340631961822509765625f,-0.2289000545501718164587146020494401454925537109375f,-0.62919999198913512117314894567243754863739013671875f,0.6048000096201899822290215524844825267791748046875f,0.18050001484155675512965899542905390262603759765625f,0.6368000135540956563318104599602520465850830078125f,-0.6642999980568884410558894160203635692596435546875f,-0.0186999969661234555218243258423171937465667724609375f,0.16819999385923145585053362083272077143192291259765625f,-0.352699983748793410853750174283050000667572021484375f,-0.59509999012351055824865397880785167217254638671875f,-0.4827000365734104292414485826157033443450927734375f,-1.609200002382695782188193334150128066539764404296875f,-0.0763000120490786581939346433500759303569793701171875f,0.01540001905560550721219215120072476565837860107421875f,0.5420999801993369171526637728675268590450286865234375f,0.4325999818533661223085573510616086423397064208984375f,0.0379000051230191647988476688624359667301177978515625f,0.4443000119149684845609726835391484200954437255859375f,0.3033000007838009270955126339686103165149688720703125f,-1.0421999823182821476308390629128552973270416259765625f,-0.490299996224045298021110284025780856609344482421875f,-0.21860000226050591010107382317073643207550048828125f,0.0682999889373778756862520822323858737945556640625f,0.301600011700391856805936185992322862148284912109375f,0.63730006147623097234600209048949182033538818359375f,0.438400034530461446546922843481297604739665985107421875f,1.527400013899804065431453636847436428070068359375f,0.4641000391930341084645306182210333645343780517578125f,1.29980000358521952108503683120943605899810791015625f,-0.95419999584481107746114503243006765842437744140625f,0.352899974128604743128789777983911335468292236328125f,-0.154399977246671815667156124618486501276493072509765625f,0.20950001548230634540459504933096468448638916015625f,-0.37400000032410007921868100311257876455783843994140625f,1.04859998270273191423029857105575501918792724609375f,0.39870001196041660040947363086161203682422637939453125f,0.54530000479221296671994423377327620983123779296875f,0.01280000095516442559784309196402318775653839111328125f,-0.279900012707710033055263920687139034271240234375f,-0.052399998402595571178608224727213382720947265625f,-0.413000010073184764536335933371447026729583740234375f,-0.1452999913811676702124486837419681251049041748046875f,-1.006799975928664547808466522837989032268524169921875f,0.43149992173910067805309154209680855274200439453125f,-1.4596999847769733804625502671115100383758544921875f,-0.18210004181563821390454904758371412754058837890625f,-0.67609992801547047491794728557579219341278076171875f,0.4214999674856658717914115186431445181369781494140625f,-0.9625999718695881224306276635616086423397064208984375f,-0.73700001890957356209810313885100185871124267578125f,-0.638399946576357013583447042037732899188995361328125f,0.71909997876882503220485887140966951847076416015625f,0.0585000009983780433486799665843136608600616455078125f,0.1292000146389014059167266168515197932720184326171875f,0.418600004681944692919159933808259665966033935546875f,-0.15400000131130209268803810118697583675384521484375f,-0.3675999974250796498154159053228795528411865234375f,-0.24280001403093365297536365687847137451171875f,-0.07160003803819459111679179841303266584873199462890625f,0.087100006127357243457254298846237361431121826171875f,2.116399995541572653934281333931721746921539306640625f,1.26690007728934350694771637790836393833160400390625f,1.1199000362515458828482906028511933982372283935546875f,-0.8070000104904178073184084496460855007171630859375f,-1.7953000170484190167741189725347794592380523681640625f,0.29209998846948204143103566821082495152950286865234375f,0.50490001524090732942795511917211115360260009765625f,0.3675000320002437259603311758837662637233734130859375f,-1.085500027507543041593862653826363384723663330078125f,-0.485600062781573083015018710284493863582611083984375f,0.127999974776058678127554912862251512706279754638671875f,1.237500028014183239832846084027551114559173583984375f,0.527300041043014200425886883749626576900482177734375f,-0.5707000248722733370954074416658841073513031005859375f,-0.017700001704692880366565077565610408782958984375f,-0.9318000472307215176215322571806609630584716796875f,-1.1082000582508750863297564137610606849193572998046875f,-0.324700028586388356188763282261788845062255859375f,-0.467800011665374437797026985208503901958465576171875f,0.52969999401569367591946502216160297393798828125f,1.1741000509090719905458399807685054838657379150390625f,1.20519996027946518779572215862572193145751953125f,-1.5337000073969362379244785188348032534122467041015625f,-0.306999939262866394784623480518348515033721923828125f,0.86130002808645433898249166304594837129116058349609375f,0.470499997407198833343500155024230480194091796875f,-0.55769998125284880874374948689364828169345855712890625f,0.822199988427758210463025534409098327159881591796875f,-0.68340001359432955840844670092337764799594879150390625f,-0.45490000840499955625517713997396640479564666748046875f,0.444100021311640891230609895501402206718921661376953125f,-0.11780000658035305605153553187847137451171875f,-1.0143000402271749038618509075604379177093505859375f,-0.70229996104240388632433678139932453632354736328125f,-0.298799973082541026059288924443535506725311279296875f,-0.2087999380648142278005252592265605926513671875f,0.6173999790310862323394758277572691440582275390625f,-0.71009999504089460486966345342807471752166748046875f,-0.2337000296369191598699899259372614324092864990234375f,0.5348999940812591002980980192660354077816009521484375f,1.194800039571524141734926161007024347782135009765625f,-1.1463000060513615085966421247576363384723663330078125f,-0.58290006286501938603805683669634163379669189453125f,-1.18909995790720035557797018554992973804473876953125f,0.267899992494285310584700710023753345012664794921875f,0.134900040908158214136847163899801671504974365234375f,-0.188300009035319126926566468682722188532352447509765625f,0.51300000489503083667131022593821398913860321044921875f,0.02900001710653299369369051419198513031005859375f,1.3732999964997174568992477361462078988552093505859375f,-0.1298999891638761283729763817973434925079345703125f,-0.5134000004068017819491842601564712822437286376953125f,0.42319998428523508682275178216514177620410919189453125f,-1.1601999550640587077765530921169556677341461181640625f,-1.4136999996110788391234791561146266758441925048828125f,-0.641700003239511485020329928374849259853363037109375f,0.9095999896056949918232703566900454461574554443359375f,-1.6657000471681360220799206217634491622447967529296875f,-0.38860003101974793793260687380097806453704833984375f,-0.1083000049933791064660226766136474907398223876953125f,0.34089999098181689163311602897010743618011474609375f,0.0785000171288847692352419471717439591884613037109375f,-0.0602999808944755244510815828107297420501708984375f,-0.36179999112784833048550581224844790995121002197265625f,-1.3415000522360209789241025646333582699298858642578125f,-0.579300010275840548956693965010344982147216796875f,0.003800003778934524234500713646411895751953125f,0.610500026110559534942012760438956320285797119140625f,1.6546999661505221723700742586515843868255615234375f,0.5370000170469275690976473924820311367511749267578125f,-0.027299985331297182966636682976968586444854736328125f,0.10439999868571747665413340655504725873470306396484375f,1.4027000087641188219578225471195764839649200439453125f,0.14730005462169726282439796705148182809352874755859375f,0.57100001860037441492323750935611315071582794189453125f,-0.38230000975057476519936017211875878274440765380859375f,-0.2194000199198722356186408433131873607635498046875f,0.47969999848604150116671007708646357059478759765625f,0.5751000372298065155263202541391365230083465576171875f,0.80290003017559674614034292972064577043056488037109375f,-0.08769998026564673754279510831111110746860504150390625f,0.35840001143366106983734198365709744393825531005859375f,0.095400047516822183979456895031034946441650390625f,0.4340000007152564620582779753021895885467529296875f,0.00550001811981104538062936626374721527099609375f,1.1069999781921502357562303586746565997600555419921875f,-0.0825000114738940748537743274937383830547332763671875f,-0.81110003083348214403258680249564349651336669921875f,-0.6987999840721486055628020039875991642475128173828125f,1.2286000324912371706176372754271142184734344482421875f,-1.0118999888882049820182373878196813166141510009765625f,-0.0528000007688997907706607293221168220043182373046875f,0.59080000910982466422183279064483940601348876953125f,0.50060000973343843355678473017178475856781005859375f,-0.205100016051531053307144247810356318950653076171875f,-0.0378999626733353967011908025597222149372100830078125f,0.1297999894663688369433884872705675661563873291015625f,-0.9672000402092937321185672772116959095001220703125f,0.5612000048823653219898233146523125469684600830078125f,-0.1233000018268823883005325114936567842960357666015625f,-1.4280000245459376628787140361964702606201171875f,-0.146000029738992787020634978034649975597858428955078125f,-0.222900051558763578896105173043906688690185546875f,0.42320003016218554758864911491400562226772308349609375f,-0.255000010542571098337560897562070749700069427490234375f,-0.353700030344725302455799464951269328594207763671875f,-0.381699982005358595671395960380323231220245361328125f,0.73630004311800067995363860973156988620758056640625f,0.6159999910891056362771678323042578995227813720703125f,1.236500019639730663101317986729554831981658935546875f,0.6560000397600240784612424249644391238689422607421875f,-0.5220000189468265716641326434910297393798828125f,-0.34639998237267211056433779958751983940601348876953125f,0.5734000070936982051961194883915595710277557373046875f,0.5915000444129117074254509134334512054920196533203125f,-0.63950000870227796667677466757595539093017578125f,0.3973000176668168847982087754644453525543212890625f,0.6626999973647296204859458157443441450595855712890625f,-0.15159999620318398427798456395976245403289794921875f,-0.2322999798923734982025735007482580840587615966796875f,-0.59270002975612878781674908168497495353221893310546875f,-0.72690004301667254793528627487830817699432373046875f,-0.54000002205371888663876234204508364200592041015625f,-0.602300040018558746623966726474463939666748046875f,-0.46370002296865020507965482465806417167186737060546875f,-0.1915000169575211685923932236619293689727783203125f,-0.663800011359900299368064224836416542530059814453125f,-0.1898000027284023616402919287793338298797607421875f,0.478800023299455812519909159163944423198699951171875f,0.28469997369423516442310528873349539935588836669921875f,-0.34800002199411428360775744295096956193447113037109375f,-1.15089998278617855476113618351519107818603515625f,0.5983000071354214188801279306062497198581695556640625f,-0.48200001421570792814463857212103903293609619140625f,0.557000002052634446414458579965867102146148681640625f,-0.362300030332803668642327465931884944438934326171875f,-0.5570999833844598736476427802699618041515350341796875f,-0.5436999935202295741731859379797242581844329833984375f,-0.015000017434358969836694086552597582340240478515625f,0.5414000000119205679993683588691055774688720703125f,1.0064000137209898610990421730093657970428466796875f,-0.7474000234365458794627556926570832729339599609375f,-0.3377000039398669883894399390555918216705322265625f,-0.50779994697570796091667943983338773250579833984375f,0.11499999389052373288677699747495353221893310546875f,0.57920000133961391242110039456747472286224365234375f,0.2098999871708449038720800672308541834354400634765625f,1.04969998984336942982054097228683531284332275390625f,-0.398099981570243954820398357696831226348876953125f,0.21180000044107405443583047599531710147857666015625f,-0.486100002372264583527794457040727138519287109375f,0.2765000255256893080257896144757978618144989013671875f,1.306199980425834183961342205293476581573486328125f,1.181699989008903717291332213790155947208404541015625f,1.510399982249737149686552584171295166015625f,-0.415399977034330181169252682593651115894317626953125f,-1.2740999821960929949682395090349018573760986328125f,-1.1216000048458578675081298570148646831512451171875f,-1.09760001098513537698408981668762862682342529296875f,-1.0072000436365604603139445316628552973270416259765625f,-0.2509000385165212065885498304851353168487548828125f,0.987799981769919011043157297535799443721771240234375f,0.16670001761913244564539127168245613574981689453125f,0.7671999900668857730323679788853041827678680419921875f,-0.31729996551275263527713832445442676544189453125f,-0.262199997219443314833142721909098327159881591796875f,-0.479600015664100709500416996888816356658935546875f,0.336500018298625747092955862171947956085205078125f,-1.489200012105703851972293705330230295658111572265625f,-0.18949998202919982048797464813105762004852294921875f,0.562600033783912767404444821295328438282012939453125f,0.56280001045465422038205360877327620983123779296875f,1.372899988061189180399424003553576767444610595703125f,0.847400005555152713299094102694652974605560302734375f,-0.582599951082467981677837087772786617279052734375f,0.37210005366206200250189795042388141155242919921875f,-0.22429998285770391674986967700533568859100341796875f,-0.33719997211098640121917924261651933193206787109375f,-0.17120006700158096890618253382854163646697998046875f,0.593400009310246279170542038627900183200836181640625f,1.08259999831914921486486491630785167217254638671875f,2.01770001213550553842424051254056394100189208984375f,0.2645000103414059111628375831060111522674560546875f,0.777400002276897428288293667719699442386627197265625f,-0.0922000177085402494725485667004249989986419677734375f,0.6114000341355805279164314924855716526508331298828125f,0.2091999887108799693891114657162688672542572021484375f,0.43360002407208109165281939567648805677890777587890625f,0.06900000250339488605533233567257411777973175048828125f,0.407900033453852672860051598036079667508602142333984375f,0.41909997761398531945786771757411770522594451904296875f,-0.79620000916644884103590129598160274326801300048828125f,0.20990004295706743420169004821218550205230712890625f,0.75939998568445388205105928136617876589298248291015625f,-0.8547000203907486604038012956152670085430145263671875f,-0.99740000007897655631694533440168015658855438232421875f,-0.2451999982029200442212868438218720257282257080078125f,-1.14900004187971394031819727388210594654083251953125f,1.075000029727817274505241584847681224346160888671875f,0.912400028484314784549269461422227323055267333984375f,-0.614200014042854292029005591757595539093017578125f,0.155899981594085801361870835535228252410888671875f,0.0907999988466503538120377925224602222442626953125f,0.34049997565150302847314378595910966396331787109375f,0.1973999938577415402818360234959982335567474365234375f,-0.3121999968469142405780303306528367102146148681640625f,-0.7653999749109143824199463779223151504993438720703125f,-0.023700021925568304226317195571027696132659912109375f,0.979399972683191233358002136810682713985443115234375f,0.55619999830722832712126546539366245269775390625f,-1.276299991783500065167800130438990890979766845703125f,-1.4805999869346617714427338796667754650115966796875f,-0.002300004851818027162835278431884944438934326171875f,-1.841599932612479673110783551237545907497406005859375f,0.26240002250522331816995347253396175801753997802734375f,0.5121000362649554826788289574324153363704681396484375f,0.216299998340010457553717060363851487636566162109375f,1.338200031447410509599649230949580669403076171875f,-0.69790002799630190821744690765626728534698486328125f,-0.2571999974429608482040521266753785312175750732421875f,-0.954400035268068602789526266860775649547576904296875f,0.4273999777644872377635465454659424722194671630859375f,-0.43100003083795324076987753869616426527500152587890625f,-0.8315999753788119175368365176836960017681121826171875f,-0.841800031249225977347805383033119142055511474609375f,0.29479998324513445595584926195442676544189453125f,0.25950002151727691312999013462103903293609619140625f,0.798700009483098938289913348853588104248046875f,0.24020000602603008843516363413073122501373291015625f,0.55449994343519259931696296916925348341464996337890625f,0.1746999868258833910061866845353506505489349365234375f,-1.394200023169815505497126650880090892314910888671875f,0.696200010861456775046463008038699626922607421875f,0.4808000272333625702003701007924973964691162109375f,-0.0548999979555606643089049612171947956085205078125f,0.01799999651312855775131538393907248973846435546875f,-0.2225000084564092706074234229163266718387603759765625f,-0.53720000057220484990239128819666802883148193359375f,0.386100011499225903577325880178250372409820556640625f,0.0896999933451414488416730819153599441051483154296875f,0.203799997799843091161875463512842543423175811767578125f,-0.62940001015961133390419490751810371875762939453125f,-0.45939998028278361541509866583510302007198333740234375f,0.551599978470802110308568444452248513698577880859375f,0.4268000092700123815347978961654007434844970703125f,-0.07330000119358281995829429433797486126422882080078125f,-0.54810001720264567293128266101120971143245697021484375f,0.86090001780390768004735946306027472019195556640625f,0.17150000322982694722639962492394261062145233154296875f,-0.45209999971985848077338232542388141155242919921875f,0.46670002307668367880211235387832857668399810791015625f,0.0979000126481057719729506061412394046783447265625f,-0.095800002050399513109368854202330112457275390625f,1.000399990892410162501846571103669703006744384765625f,0.8440999775767323853870038874447345733642578125f,-0.2511000057995318446302235315670259296894073486328125f,-0.964799960334599138178646171581931412220001220703125f,-0.61509997950643346964483271221979521214962005615234375f,-1.7298999718412757520269451561034657061100006103515625f,0.038099998520314759531402160064317286014556884765625f,-0.776899993395805399387654688325710594654083251953125f,0.9599000401630999323998594263684935867786407470703125f,-0.2380000119730827368158543322351761162281036376953125f,-0.7660999997645616677033331143320538103580474853515625f,-0.6762000164866446283440382103435695171356201171875f,-0.00079998938441239175745067768730223178863525390625f,1.325899961924552172831681673415005207061767578125f,0.308499998837709288324049339280463755130767822265625f,0.87250002443790464212725055404007434844970703125f,0.0396000457644465342355033499188721179962158203125f,-0.4795000294148916708536489750258624553680419921875f,-1.01350001820921864492675013025291264057159423828125f,1.179099991732835750468666446977294981479644775390625f,0.9280999921128152418958734415355138480663299560546875f,0.42019998378604672684133447546628303825855255126953125f,-0.40910000540465052409899726626463234424591064453125f,-0.3396000062763688820410834523499943315982818603515625f,-0.183100013868510824721624885569326579570770263671875f,0.6249999943375581512583494259160943329334259033203125f,-0.114800018300116324354576136101968586444854736328125f,0.11890004635453177428416893235407769680023193359375f,1.302700080084800760005236952565610408782958984375f,-1.25880000213980647316702743410132825374603271484375f,-1.20960001298189201435206996393389999866485595703125f,0.25440000606179236086745731881819665431976318359375f,-0.256700004804134351843458716757595539093017578125f,0.0956000031024217822306354719330556690692901611328125f,0.90160000957697572498972249377402476966381072998046875f,-0.069999979808926138247215931187383830547332763671875f,-0.133199961076676753890524196322076022624969482421875f,0.737799984228610927772251670830883085727691650390625f,-0.422900026990473409949089500514674000442028045654296875f,0.1428999995350839213870131061412394046783447265625f,-0.4750000303238632870517221817863173782825469970703125f,0.071600024366378800522170422482304275035858154296875f,-0.5712000495113438436334263315075077116489410400390625f,-0.1192999818146223134363026474602520465850830078125f,-0.230700030814112000410176506193238310515880584716796875f,-0.54309999185204471672250292613171041011810302734375f,0.7971000144161279887811133448849432170391082763671875f,-0.9092000214934348178985601407475769519805908203125f,-0.90119999122917626710460581307415850460529327392578125f,0.484599995994567933621510746888816356658935546875f,0.188599989967048298211693690973334014415740966796875f,0.81270002632886140236934124914114363491535186767578125f,-0.3387999658927329793556282311328686773777008056640625f,0.043099990287422951240614565904252231121063232421875f,-0.92089994881153103278847993351519107818603515625f,-0.405199994030594812244316926808096468448638916015625f,0.05589998278617880345109369955025613307952880859375f,-0.53610003247261062142570153810083866119384765625f,1.1796000064253799877178607857786118984222412109375f,0.317599996232985848365615311195142567157745361328125f,2.0148000325679777944287707214243710041046142578125f,1.208500000402330964988095729495398700237274169921875f,0.7343000281572340526281550410203635692596435546875f,-0.844299978610873136375403191777877509593963623046875f,-2.192299961042404987665577209554612636566162109375f,-0.24109998784363284585197106935083866119384765625f,-1.287899982547759947237864253111183643341064453125f,0.1105999949440359397812017050455324351787567138671875f,0.77889995035380155119497658233740366995334625244140625f,-0.017300021801888387784629230736754834651947021484375f,0.6060000083744530030571695533581078052520751953125f,0.92470000101923943702786345966160297393798828125f,-0.487399982309341783803802172769792377948760986328125f,0.247200019943714099923681715154089033603668212890625f,-0.897600007408856814805631074705161154270172119140625f,-0.0234999693334099202246534332516603171825408935546875f,-0.92260000323653201803608681075274944305419921875f,-0.08409997727870877071154609438963234424591064453125f,1.00809999870657929221806625719182193279266357421875f,1.074700033056736447889534247224219143390655517578125f,1.53839999396204962778256231104023754596710205078125f,0.830799998921155857800613375729881227016448974609375f,-0.49490000227689723288904133369214832782745361328125f,0.03570000224113467623965334496460855007171630859375f,0.26500001534819628279393555203569121658802032470703125f,-0.41910000106468781044810612002038396894931793212890625f,0.4727000212997196637587649092893116176128387451171875f,-0.075700025039911411628423820729949511587619781494140625f,0.1919000418245797590088841388933360576629638671875f,0.62730001982748484490315377115621231496334075927734375f,-0.168600008034706316806250470108352601528167724609375f,-0.51410002287626299644074379102676175534725189208984375f,0.3096000079154972439710036269389092922210693359375f,-0.50570000969171513549582641644519753754138946533203125f,0.37379999856352785769786351011134684085845947265625f,0.2044999733492730842332463225829997099936008453369140625f,-0.25789999618232239964044083535554818809032440185546875f,0.88190002461373795394905528155504725873470306396484375f,-0.4100000163167718536527672767988406121730804443359375f,-0.243899998875707291201564430593862198293209075927734375f,0.42270000379085548303237374057061970233917236328125f,0.45170000241994845424642335274256765842437744140625f,1.0946999929726126676854391917004249989986419677734375f,1.136400014540552394493033716571517288684844970703125f,1.7746000013589853239182048127986490726470947265625f,1.0738000246405601245669458876363933086395263671875f,0.26479997572004787542709891567938029766082763671875f,-0.59479995262324880656024106428958475589752197265625f,0.385100007632374341515202331720502115786075592041015625f,-0.414299961176514575100782167282886803150177001953125f,0.1601999768197537310499001250718720257282257080078125f,0.575100001597404375530686593265272676944732666015625f,0.41909999441504452999396335144410841166973114013671875f,1.20739996472597166388140976778231561183929443359375f,0.11429999038279081702285111532546579837799072265625f,-0.016500001236796624226599305984564125537872314453125f,-0.020799996909499185449021751992404460906982421875f,-0.4901999927639959508951505995355546474456787109375f,0.18960001171529317076647203066386282444000244140625f,-0.07330002008080460651484600020921789109706878662109375f,0.85949999566376167337011793279089033603668212890625f,0.726599975937605169207245126017369329929351806640625f,1.4570000087022787482737840036861598491668701171875f,1.3903999782264240270279742617276497185230255126953125f,1.547499981150031178600556813762523233890533447265625f,0.0858999773472548522335046072839759290218353271484375f,0.83959999144971408924220668268389999866485595703125f,-0.149899988120794080970199502189643681049346923828125f,1.016900014331936663580790991545654833316802978515625f,-0.2970999878734350563291854996350593864917755126953125f,0.01790000549554859876622003866941668093204498291015625f,0.70559998991489425446133054720121435821056365966796875f,-0.186800027933716972228239683317951858043670654296875f,-0.074300008636713155141251263557933270931243896484375f,0.1123999915480613509544127737171947956085205078125f,-0.35220001502633113688034427468664944171905517578125f,-0.74730001122206413111115352876367978751659393310546875f,0.2487999985635276356532585850800387561321258544921875f,-1.182700048606097453784968820400536060333251953125f,0.353800002735852547885997410048730671405792236328125f,-0.55220003838390052663953611045144498348236083984375f,-0.44669998364448559868833399377763271331787109375f,1.3450999860815697761751152938813902437686920166015625f,-0.7141999893069261862166285936837084591388702392578125f,-0.9758999832704662313886956326314248144626617431640625f,0.41950001432746641949478316746535710990428924560546875f,-0.1147999898761506709643498425066354684531688690185546875f,0.274100008869171407610565438517369329929351806640625f,0.167899997318536053914073136184015311300754547119140625f,0.124100001083314259719969641082570888102054595947265625f,0.020300000154227004767193420775583945214748382568359375f,-0.302300009080022469287740705112810246646404266357421875f,-0.7045000084042545296370008145458996295928955078125f,-0.4428000418215989331116588800796307623386383056640625f,-1.481000037431716265245995600707828998565673828125f,-0.7136000545263285754771231950144283473491668701171875f,0.60029994444251055796257787733338773250579833984375f,1.86749999567866353800127399154007434844970703125f,1.95189996299743739882615045644342899322509765625f,-0.929699981684982912355508233304135501384735107421875f,-1.095499993681908090792376242461614310741424560546875f,1.0156000099942088521487448815605603158473968505859375f,-0.4499999472126374744362919955165125429630279541015625f,0.7013000100374220391330482016201131045818328857421875f,-0.50089995566606493326844429248012602329254150390625f,-0.83630002568364159021285786366206593811511993408203125f,-0.08010000616088552849447523840353824198246002197265625f,0.1340000038444995045239238606882281601428985595703125f,0.66790003162845990036800003508687950670719146728515625f,0.10589998785257304092510821647010743618011474609375f,0.55720001711249356191046899766661226749420166015625f,1.155900028681755120629759403527714312076568603515625f,-1.3821000047862528337105914033600129187107086181640625f,0.458499998986720669336136779747903347015380859375f,0.16149999564886030611887690611183643341064453125f,0.494599980050324905533898345311172306537628173828125f,1.3921000028491017364018489388399757444858551025390625f,0.83760002041012082685256245895288884639739990234375f,-0.27370005359053539706337687675841152667999267578125f,0.0421999755010007415734207825153134763240814208984375f,1.1120000031888483960074154310859739780426025390625f,1.585799999815225458377199174719862639904022216796875f,-0.507100001917778708815376376151107251644134521484375f,0.284700000534952035735614117584191262722015380859375f,-0.145600007796287622685582618942135013639926910400390625f,-0.5824000249266625406363573347334749996662139892578125f,0.37380000064969021167371465708129107952117919921875f,-0.478100006697325718985069897826178930699825286865234375f,-0.472799989593029490464459740906022489070892333984375f,0.67779998597949753236235892472905106842517852783203125f,0.036400010144710670534351493188296444714069366455078125f,0.143500000700354013449810963720665313303470611572265625f,0.13400001729279742423983634580508805811405181884765625f,0.0338000003889202815798853407613933086395263671875f,0.43319999438077239073408009062404744327068328857421875f,1.2080000690445313882293021379155106842517852783203125f,-0.3607000176265839908040788941434584558010101318359375f,-0.176299989045410943067082598645356483757495880126953125f,0.55180000371932980129940915503539144992828369140625f,0.243900001315772330112707777516334317624568939208984375f,0.71260001526921989967178205915843136608600616455078125f,0.5651000240050259026958201502566225826740264892578125f,-0.1486999991640447493779220167198218405246734619140625f,0.476099993133544785450794734060764312744140625f,0.654499982997775564541598214418627321720123291015625f,1.2184999718666080781304117408581078052520751953125f,0.52530000035911772471308722742833197116851806640625f,0.965099946314097234534301605890505015850067138671875f,1.010599968643486334940462256781756877899169921875f,-0.0008999694123870138895426862291060388088226318359375f,0.60050003987550748529855582091840915381908416748046875f,-0.1671999812565745369141723131178878247737884521484375f,-0.325199998743086748920205764079582877457141876220703125f,-0.62419999081566934773235288957948796451091766357421875f,0.0254000016957520724503183373599313199520111083984375f,-0.377700016605853505780032719485461711883544921875f,0.2146999988585711616195794704253785312175750732421875f,1.4643999984435731587240070439293049275875091552734375f,0.31119999869838343986572226640419103205204010009765625f,1.124200033060461567657739578862674534320831298828125f,0.370700008499621702640069997869431972503662109375f,-0.30040000117421161718311850563623011112213134765625f,-0.04640002668500020632791347452439367771148681640625f,-0.2012999924540521679006133126677013933658599853515625f,-0.109100013227760417322542707552202045917510986328125f,1.14359995111227075170745592913590371608734130859375f,0.7786999879136684565850146100274287164211273193359375f,-0.003300011065602681537001217293436639010906219482421875f,0.65489997085407336552265178397647105157375335693359375f,0.9206000160105529683818303965381346642971038818359375f,-0.197700026980788035046998629695735871791839599609375f,-0.554700008767843311119349891669116914272308349609375f,-0.5855000221431254647797004508902318775653839111328125f,-1.06390002303421482565681799314916133880615234375f,-0.66820002125874167209218512653023935854434967041015625f,0.265500008936971509942992497599334456026554107666015625f,-0.34520000839158893057856403174810111522674560546875f,0.0592000049531460348362088552676141262054443359375f,0.572400016134977374093750768224708735942840576171875f,-0.201300007355212973170210943862912245094776153564453125f,-0.3881999943621454729481001777458004653453826904296875f,0.6625000104121860911021713036461733281612396240234375f,0.2205000174678868329891656685504131019115447998046875f,-0.970900027042627566942201156052760779857635498046875f,-0.55350001982972063530041850754059851169586181640625f,-0.1981999842107295961568524944595992565155029296875f,-0.75450001462548998976131997551419772207736968994140625f,0.872399998253583941476563268224708735942840576171875f,0.10419999694377179988435955237946473062038421630859375f,-0.614200013111531717413527076132595539093017578125f,0.7916000000961116445097331961733289062976837158203125f,-0.239400001945346330156638714470318518579006195068359375f,-0.41249998752027783321949527817196212708950042724609375f,0.15609998061656948920017384807579219341278076171875f,0.08599999712407591090368441655300557613372802734375f,0.405500035852193907448537402160582132637500762939453125f,-0.88960001290738544010849864207557402551174163818359375f,-0.42340001312866781812971339604700915515422821044921875f,-0.7487000234529379572023799482849426567554473876953125f,-0.34919999444782712227919319047941826283931732177734375f,-1.22160000842213545269032692885957658290863037109375f,-0.0345000024437904784235797706060111522674560546875f,-2.4186000008821491036314910161308944225311279296875f,-0.84030002163052586183766834437847137451171875f,-0.51689997700452838813589551136828958988189697265625f,0.27690000755190846692954664831631816923618316650390625f,0.85400002248957773698379014604142867028713226318359375f,-1.06970000746399218627402660786174237728118896484375f,-0.27629998580440917610445694663212634623050689697265625f,0.14360000185072419043308400432579219341278076171875f,0.201300042819976710006812936626374721527099609375f,0.171099994847178304979706808808259665966033935546875f,1.05770002007037344782247600960545241832733154296875f,0.404200007318705145653581212172866798937320709228515625f,-0.16949999251589165094600275551783852279186248779296875f,0.57610002783462388453727953674388118088245391845703125f,1.081100027853250100662307886523194611072540283203125f,-0.0432000061154369507931960470159538090229034423828125f,0.79299995046854032665351041941903531551361083984375f,0.4499999970197681165018366300500929355621337890625f,-0.9823000118553633086548870778642594814300537109375f,0.47260000249147449125075581832788884639739990234375f,0.007700000996887557125347711917129345238208770751953125f,-0.511500018298626013546481772209517657756805419921875f,-0.290800010059773350601375341284438036382198333740234375f,-0.80809999799877318960028560468344949185848236083984375f,-0.38820001267194736982446556794457137584686279296875f,-0.737600018733739926091175220790319144725799560546875f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_3d_2item_large_5o3i_padded_strided_uneven_filter_uneven_data_dilation_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,3,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3,4};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,5,9,8,9};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,3,2},        // move_strides
                                                                  Strides{1,1,1},        // filter_dilation
                                                                  CoordinateDiff{2,1,2}, // below_pads
                                                                  CoordinateDiff{1,2,3}, // above_pads
                                                                  Strides{2,3,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3512999974086878918200227417401038110256195068359375f,-0.0852999673157939231060709062148816883563995361328125f,-0.45700004889816059350238219849416054785251617431640625f,-1.247399987785518415961405480629764497280120849609375f,-0.41119998489618314607696447637863457202911376953125f,0.55299996726960021220520502538420259952545166015625f,1.068200010436773705890800556517206132411956787109375f,1.3565000559613109576417855350882746279239654541015625f,0.1871999984860419630194883211515843868255615234375f,0.7983000035405154459766663421760313212871551513671875f,0.0226000394463550247081684574368409812450408935546875f,2.00550005015730814506014212383888661861419677734375f,0.0578000424176454696834071000921539962291717529296875f,-0.0290000228062272658036135908332653343677520751953125f,0.857800015074014599036900108330883085727691650390625f,0.09569991607516925569143495522439479827880859375f,0.084600040064752857205121472361497581005096435546875f,0.7282000009000297868766438114107586443424224853515625f,1.01300005757063704692200190038420259952545166015625f,1.0810999818459148258398272446356713771820068359375f,-0.27699996776133739562197888517403043806552886962890625f,1.55730002459585659124741141567938029766082763671875f,0.77500000264495572110945431631989777088165283203125f,0.911499994717539951949447640799917280673980712890625f,0.90390006192624572367577684417483396828174591064453125f,-0.316699991616606002509115569409914314746856689453125f,-0.254999983310699729344150910037569701671600341796875f,-0.32800000299513332180367797263897955417633056640625f,0.139099954386800117589473302359692752361297607421875f,-0.05599999220669238386705046650604344904422760009765625f,-1.352000060446561402471843393868766725063323974609375f,0.66529998393058809025291111538535915315151214599609375f,0.977100056158005880746486582211218774318695068359375f,-0.4969999913051719175882681156508624553680419921875f,-0.0744999950304627844843707862310111522674560546875f,-0.497199983584880822462537253159098327159881591796875f,-0.61420004805475547637882982598966918885707855224609375f,0.159000005595385740519986939034424722194671630859375f,-0.41380004051029739553513309147092513740062713623046875f,-1.151800014895199897324573612422682344913482666015625f,0.16630005105286882471915532732964493334293365478515625f,-0.28640000321567082863793984870426356792449951171875f,-0.1378999280095103419085944551625289022922515869140625f,-0.5837999888963996131252542909351177513599395751953125f,-0.44539999681562159050685067995800636708736419677734375f,-0.6772000090658660820253089696052484214305877685546875f,-1.8903999827712769654652902318048290908336639404296875f,0.179700022178887863777418942845542915165424346923828125f,0.32009997529685474404459455399774014949798583984375f,0.17219997315406754267996802809648215770721435546875f,-1.58719997684210500921153652598150074481964111328125f,-0.370899944117665025800079092732630670070648193359375f,-0.8487000098556281013628677101223729550838470458984375f,0.3034000049889085115495390709838829934597015380859375f,-0.167100011752546018239939940031035803258419036865234375f,0.231300007168948607017000540508888661861419677734375f,0.8289000383451583520155736550805158913135528564453125f,0.7260999377384773634247494555893354117870330810546875f,0.645699985328317627164551595342345535755157470703125f,0.046000012323260397462121318312711082398891448974609375f,-0.5889999865218988883697193159605376422405242919921875f,-1.418399987033009512771286608767695724964141845703125f,0.26319997869730027417745077400468289852142333984375f,0.1164999925568697136668561142869293689727783203125f,-0.031499968826770530228031930164434015750885009765625f,0.286700015048683365392889754730276763439178466796875f,-0.15370000456571586422427344587049447000026702880859375f,0.853099996583163378005565391504205763339996337890625f,-0.116600001157820154507049892345094121992588043212890625f,0.428000006683170231003288108695414848625659942626953125f,-0.93850005855411378075103812079760245978832244873046875f,-0.17440001589655818037272183573804795742034912109375f,-1.0641000428810720723760141481761820614337921142578125f,1.701799973246454822373152637737803161144256591796875f,0.70759998636096810287199332378804683685302734375f,0.0730999722570182530745341864530928432941436767578125f,0.1473999798133973637703775239060632884502410888671875f,0.6160999939531082159049901747494004666805267333984375f,0.376600003243982683898138930089771747589111328125f,-0.253800031197072062383313095779158174991607666015625f,-0.395199992167949432086970773525536060333251953125f,-0.7377000089317558018109366457792930305004119873046875f,1.04740005830526428098892210982739925384521484375f,0.29409997668266230075317935188650153577327728271484375f,0.534600028702616913989231761661358177661895751953125f,1.85260001184195299828161296318285167217254638671875f,-0.25340003024637713924249737829086370766162872314453125f,-1.2425000226870179798055460196337662637233734130859375f,-0.4172000378251083940739363242755644023418426513671875f,-1.103599977487325300984366549528203904628753662109375f,-0.2779999924153084833733373670838773250579833984375f,0.0533000033915040916099314927123486995697021484375f,0.35770002875029993649746984374360181391239166259765625f,-0.3990999978423117600101477364660240709781646728515625f,1.97160000994950390662552308640442788600921630859375f,0.3137000006914145711078845124575309455394744873046875f,1.02199994920939207787569102947600185871124267578125f,0.54390009102076286584548370228731073439121246337890625f,0.22280000076889994176099207834340631961822509765625f,0.7549000439628965608562793931923806667327880859375f,0.7955999966949216517519971603178419172763824462890625f,0.6557000134915114131217706017196178436279296875f,-0.5277999945104117074379246332682669162750244140625f,-0.502899940712751458704588003456592559814453125f,0.216900001405180287150642470805905759334564208984375f,0.48850008645653753003301744683994911611080169677734375f,-0.8303999545708304896862728128326125442981719970703125f,-0.04179999462962147305233884253539144992828369140625f,-0.6083000328585510541046232901862822473049163818359375f,-0.57590004976689812554724312576581723988056182861328125f,0.2979999984875332241784917641780339181423187255859375f,-0.69239996591806463044349584379233419895172119140625f,-0.691200055192410456328389045665971934795379638671875f,-1.092199978406727556290434222319163382053375244140625f,0.8318000397056344308310826818342320621013641357421875f,0.1086000390291206318948979969718493521213531494140625f,0.50919997515082382477658029529266059398651123046875f,-0.2086999862000340488776828351547010242938995361328125f,-1.1534999752938752681785672393743880093097686767578125f,-0.781299998787044813752800109796226024627685546875f,0.54150000697374256475313814007677137851715087890625f,-0.29790004602670683286902431063936091959476470947265625f,-0.0212999975204468672274060736526735126972198486328125f,-0.7747000523164866425673835692577995359897613525390625f,0.9758999996244910324350030350615270435810089111328125f,0.196599987858534053231096550007350742816925048828125f,-0.013899983024596718195198263856582343578338623046875f,-1.44500000096857572629005517228506505489349365234375f,-0.5255999925784766713121598513680510222911834716796875f,0.8141999623730777901897681658738292753696441650390625f,-0.31409997455924809628413640893995761871337890625f,-0.01779999123215636469552691778517328202724456787109375f,0.731300029110908411666969186626374721527099609375f,0.18609998264685267965745651963516138494014739990234375f,0.70760000275224388399664121607202105224132537841796875f,0.9678000538915403350159749606973491609096527099609375f,0.5871999987095588569019355418276973068714141845703125f,-0.91510001757889991580441346741281449794769287109375f,-1.0991999878913161214910587659687735140323638916015625f,-0.7565999738886957093342289226711727678775787353515625f,0.597700013849138844790331859258003532886505126953125f,1.412299988833069175342416201601736247539520263671875f,0.555400019428134328336454927921295166015625f,-0.46919999720454175218264936120249330997467041015625f,0.343100013943016701745136742829345166683197021484375f,-1.2609000064790250039692409700364805757999420166015625f,-1.230899995098262866832783402060158550739288330078125f,-0.7211000131383540345808569327346049249172210693359375f,0.3572999891683450446322467541904188692569732666015625f,0.02389994145035789774311751898494549095630645751953125f,-0.2776000297605996092187297108466736972332000732421875f,0.485199959646166245619269830058328807353973388671875f,-0.926999957889318881854023857158608734607696533203125f,0.799000009097158514492775793769396841526031494140625f,0.29569992766082275892358666169457137584686279296875f,-0.7846000634968277598346730883349664509296417236328125f,-0.813600007624923904359093285165727138519287109375f,0.4390000755563374656986752597731538116931915283203125f,0.6868000032350420891447129179141484200954437255859375f,-0.089799981009960061495434047174057923257350921630859375f,0.78220001744776990193486199132166802883148193359375f,0.216999997377395725806081827613525092601776123046875f,-0.895300043758750607736374149681068956851959228515625f,0.991299973492325126045443539624102413654327392578125f,0.892400013303756178828507472644560039043426513671875f,0.9510999889612197222277245600707828998565673828125f,0.23350000174343588099645785405300557613372802734375f,0.775700012634694235913457305287010967731475830078125f,-0.093899995057284524335727837751619517803192138671875f,0.86710003868639518032068735919892787933349609375f,-0.056599996650219086546940161497332155704498291015625f,-0.530500021398067733713332927436567842960357666015625f,0.0505999851092704400201682801707647740840911865234375f,-0.8124999808892610086985541784088127315044403076171875f,-0.9891000201940538527622948095086030662059783935546875f,-1.5194000281900168491944214110844768583774566650390625f,-1.6671000270262366083073857225826941430568695068359375f,0.454699993754922981992194763734005391597747802734375f,-0.29229998255595546208240875785122625529766082763671875f,-0.686199973534047824585968555766157805919647216796875f,0.935200001518427637137165220337919890880584716796875f,-0.58510001504570130492766111274249851703643798828125f,-0.08079998238086771866761637284071184694766998291015625f,0.148800003816187853544761310331523418426513671875f,-0.1091000354677447159446046498487703502178192138671875f,-1.516100003713368504776326517458073794841766357421875f,0.7025000178441402898243950403411872684955596923828125f,1.46960000892132480743157429969869554042816162109375f,0.0465999984383582699365433654747903347015380859375f,0.5549000399395829230542176446761004626750946044921875f,1.464499999128282770044506833073683083057403564453125f,-0.912299997997283451667271947371773421764373779296875f,-0.100500049002468561099021826521493494510650634765625f,-1.21439994854330901574712697765789926052093505859375f,-0.4203999817095700475277908481075428426265716552734375f,0.11050003343075598394307235139422118663787841796875f,-0.5003999981194737767253855054150335490703582763671875f,-0.764800002765654962644248371361754834651947021484375f,-0.370800025407970412771874180180020630359649658203125f,0.6350999871432778665081286817439831793308258056640625f,-0.746500009916722273572986523504368960857391357421875f,-0.9108000027954579191913353497511707246303558349609375f,1.589900005145370442960484069772064685821533203125f,1.1739999845102442321831404115073382854461669921875f,1.651199972677230842776907593361102044582366943359375f,0.72890000790953657361370687794988043606281280517578125f,-0.298999979555607087178259462234564125537872314453125f,-0.95490000190436763460866131936199963092803955078125f,-0.722399954183399639617846332839690148830413818359375f,0.5191999496698385119231033968389965593814849853515625f,1.2115999712809912836064540897496044635772705078125f,-0.2574000586777918098135842228657566010951995849609375f,0.1428000019982456070266607639496214687824249267578125f,1.084800017555057394247342017479240894317626953125f,-0.61440002357065648386225120702874846756458282470703125f,0.10639998853802712375227201846428215503692626953125f,0.21820000056475397087751844082958996295928955078125f,-1.05389997970908932956035641836933791637420654296875f,-0.839899970909952831021882957429625093936920166015625f,-0.3546000430822375815154146039276383817195892333984375f,0.673499954767525554899521011975593864917755126953125f,-1.4767000126272435966967577769537456333637237548828125f,0.70680001765191402540722265257500112056732177734375f,-0.0577000052064651214323021122254431247711181640625f,0.40339997906088864709772678907029330730438232421875f,0.1066000060379506120256110079935751855373382568359375f,1.765800022129714097474106893059797585010528564453125f,-0.0855999933794138190279454647679813206195831298828125f,0.4141999974653123306467250586138106882572174072265625f,1.938599989557264979822548411902971565723419189453125f,-0.47370000821649949074299001949839293956756591796875f,0.24400000116229036972725907617132179439067840576171875f,-0.58990002135038366315455959920655004680156707763671875f,0.112200004595517821570638261619023978710174560546875f,0.230799997542798340788294808589853346347808837890625f,0.38570002433210592851509090905892662703990936279296875f,-0.34340000800639292055649320900556631386280059814453125f,0.56720005552023611272716152598150074481964111328125f,0.5476000438794506042228249498293735086917877197265625f,0.2493000336334103561597430598339997231960296630859375f,0.1920999275237325054632719911751337349414825439453125f,-0.865499977104365658675533268251456320285797119140625f,-0.62539999066889262469004506783676333725452423095703125f,0.14619999931305638352085907172295264899730682373046875f,0.7722000514596703002467847909429110586643218994140625f,0.3325000053644178166223355219699442386627197265625f,0.3954000928908587564336585273849777877330780029296875f,0.5881000118568537882168811847805045545101165771484375f,-0.2955999932676542929499419187777675688266754150390625f,0.8909000250309702106932263632188551127910614013671875f,-0.9293000011861332065876695196493528783321380615234375f,-0.2822000010192393659735898836515843868255615234375f,0.67820001649036998347952476251521147787570953369140625f,-1.209100039155781569633063554647378623485565185546875f,-1.0370000202879310347015007209847681224346160888671875f,-0.43579998703747968580302085683797486126422882080078125f,0.63079999087452875983217381872236728668212890625f,0.4067999445989727291816961951553821563720703125f,0.4009000315688553417459161209990270435810089111328125f,-0.83940000125616831549990592975518666207790374755859375f,0.553199975568056334651600991492159664630889892578125f,0.983200031261146190075805861852131783962249755859375f,-0.49590000670254175130224894019193015992641448974609375f,0.13620000665187836919045594186172820627689361572265625f,-0.82560002813637256569023747942992486059665679931640625f,0.0190000326037409461576999092358164489269256591796875f,-0.355400023190677760709377253078855574131011962890625f,-0.12099998248368415421083454930339939892292022705078125f,0.60910002175867550278098860871978104114532470703125f,-0.68979998413920373678820396889932453632354736328125f,0.98560003752410441091313941797125153243541717529296875f,1.5246000125348562193039470002986490726470947265625f,0.789500031575560257124379859305918216705322265625f,0.2520000317618258378615792025811970233917236328125f,-0.789600016595423337406600694521330296993255615234375f,0.956900022713840225918602300225757062435150146484375f,0.1969000313937672164144032649346627295017242431640625f,-0.58639999665915976123642394668422639369964599609375f,0.08379999923408021356152630687574855983257293701171875f,-0.7057000282064083140909360736259259283542633056640625f,0.955900004504620870449116409872658550739288330078125f,0.85529997994452688914179816492833197116851806640625f,2.0437000059813268393327234662137925624847412109375f,-0.19810002578943997075811012109625153243541717529296875f,-0.393400025627016702856053598225116729736328125f,0.00809995597749779694396465856698341667652130126953125f,0.4960000146701932788317890299367718398571014404296875f,-0.45299997431039873418967545148916542530059814453125f,-0.6406000062689181095976209689979441463947296142578125f,-0.3024999792315068436465708145988173782825469970703125f,0.34759996631890555107702311943285167217254638671875f,-1.17059996305555191753455801517702639102935791015625f,1.9231000355496996601090131662203930318355560302734375f,1.0198000004559764075651173698133789002895355224609375f,0.3867000182151780762751513975672423839569091796875f,1.1712999895110736137127105394029058516025543212890625f,0.0908000047326085191201627822010777890682220458984375f,0.5463000038534400193412920998525805771350860595703125f,-1.30240005029588967744302863138727843761444091796875f,-0.65710000080019170809464412741363048553466796875f,0.3209000328168276627849309079465456306934356689453125f,-0.2364000630810865200004400321631692349910736083984375f,-0.59419996639639105495689364033751189708709716796875f,0.12349996907263882661709430976770818233489990234375f,1.1191000368833547984337428715662099421024322509765625f,0.51019999112486846826186592807061970233917236328125f,-0.270500031605363400188934974721632897853851318359375f,-0.1966999143168328689768031836138106882572174072265625f,1.2755000128298998873788150376640260219573974609375f,-0.3826999984264374266018648995668627321720123291015625f,-1.762000016637145893838578558643348515033721923828125f,0.3259000391870738866373358177952468395233154296875f,1.0442999523475762213564621561090461909770965576171875f,-0.7306000318802896575931526967906393110752105712890625f,0.40779995789080913937141303904354572296142578125f,-0.260599998110532471429223733139224350452423095703125f,0.74360003027468890568485448966384865343570709228515625f,0.389000010922550654157703320379368960857391357421875f,0.305599975274503332656195198069326579570770263671875f,0.4837000366404655693486347445286810398101806640625f,-0.8067999847576017291572725298465229570865631103515625f,0.566100012691319154356506260228343307971954345703125f,-0.5956999629765735715380969850230030715465545654296875f,-0.33579998819231970941245890571735799312591552734375f,0.5455000008717174608818822889588773250579833984375f,0.30479996964782485324718663832754828035831451416015625f,1.0386000451013448486747847709921188652515411376953125f,0.8615000077188010862982991966418921947479248046875f,-0.2039999722167851192722309860982932150363922119140625f,0.9339999640956524462609422698733396828174591064453125f,-0.605500009216367818254411758971400558948516845703125f,-0.123300029021501611481426152749918401241302490234375f,0.2061999798297886599129924434237182140350341796875f,-0.075800037142635279252544933115132153034210205078125f,-1.61930004335641886115126908407546579837799072265625f,0.1921999920040378118102353255380876362323760986328125f,0.1783000100597735837482105125673115253448486328125f,-0.703599982963501791033422705368138849735260009765625f,1.260200012582540551875354140065610408782958984375f,1.435199957671760895294710280722938477993011474609375f,0.4949000833019621126140918931923806667327880859375f,-0.4275999966800210216888444847427308559417724609375f,0.372800012205541264620478614233434200286865234375f,-0.775699985514580969692133294302038848400115966796875f,0.3428999801635743693850599811412394046783447265625f,-0.6923999983653421619322898550308309495449066162109375f,-0.83739995362460550865080222138203680515289306640625f,-0.8443000466346755050750516602420248091220855712890625f,0.17010002219826059199903056651237420737743377685546875f,0.5187000161141150922361475750221870839595794677734375f,0.912999966710806187819571277941577136516571044921875f,0.2210000395551332541543843035469762980937957763671875f,0.4145000067278743483001335334847681224346160888671875f,0.829299993474780894331388481077738106250762939453125f,-0.647100031086802029989257789566181600093841552734375f,-0.509299962666630801066958156297914683818817138671875f,1.0487000066146252574839081717072986066341400146484375f,0.2211999912291766623440025796298868954181671142578125f,0.384199953097105695309920747604337520897388458251953125f,0.4411999974131581847558436493272893130779266357421875f,1.263900059057772917725515071651898324489593505859375f,-0.950900043247640436305800903937779366970062255859375f,-1.257800046366453816659713993431068956851959228515625f,-0.44700000797584660272576684292289428412914276123046875f,-0.08980001926869218831228636190644465386867523193359375f,1.1532000019431112303180952949333004653453826904296875f,-0.195400010859965667098236963283852674067020416259765625f,-0.0338999999187883371831730983103625476360321044921875f,0.822400001345574338529331726022064685821533203125f,-0.6179999980777497459172309390851296484470367431640625f,-0.9421000320181256437734873543377034366130828857421875f,-0.0376999992087483359881616706843487918376922607421875f,0.51990000188574148065612234859145246446132659912109375f,0.49570002064406881725489029122400097548961639404296875f,-1.389699986974894496682964017963968217372894287109375f,-1.69430001374036098837905228720046579837799072265625f,0.510300060559810031435290511581115424633026123046875f,0.3988000019907946835928669315762817859649658203125f,0.0972999852195377545882593040005303919315338134765625f,-0.1122000113010399591217947090626694262027740478515625f,-1.141200008924305731028425725526176393032073974609375f,-0.9652000304639332117773165009566582739353179931640625f,-0.2537000394716855122823062629322521388530731201171875f,-1.5186999753221861464425046506221406161785125732421875f,-0.478099983656406823939732930739410221576690673828125f,-1.309299981553852632032430847175419330596923828125f,0.521199994432925972631664990331046283245086669921875f,0.03810000962168003102448210483998991549015045166015625f,0.5109999872893091765746476085041649639606475830078125f,-0.0492999653860917153025411607814021408557891845703125f,-0.3003999826967724828818973037414252758026123046875f,-1.0008000684723261652919745756662450730800628662109375f,-1.57509995748996711739664533524774014949798583984375f,-1.1150000280141838526759556771139614284038543701171875f,-0.5344999670907848443590637543820776045322418212890625f,0.661199970293045335978376897401176393032073974609375f,0.737600009718537119596248885500244796276092529296875f,-0.235099985951185086463510742760263383388519287109375f,-0.539200062993168582892167250975035130977630615234375f,0.68940004955455680413223262803512625396251678466796875f,0.01939996776580754644925264074117876589298248291015625f,1.47489998245835263190883779316209256649017333984375f,0.46749998845159990157327456472557969391345977783203125f,-1.2254000395447006699356506942422129213809967041015625f,-0.880399956768751934532701852731406688690185546875f,0.30240001362040658250407432205975055694580078125f,0.014400039663911012866037708590738475322723388671875f,1.0125999743655318408031007493264041841030120849609375f,-1.443600017459690132426430864143185317516326904296875f,-0.2546000702023512030081064949627034366130828857421875f,-1.928399985617399448045716781052760779857635498046875f,0.1700000227615234837941216028411872684955596923828125f,-1.7969000154122711787607613587169907987117767333984375f,0.03279999078512219057301990687847137451171875f,-1.10910005532354194457411722396500408649444580078125f,-1.1569999882876869090608806800446473062038421630859375f,0.85849998114258074810578591495868749916553497314453125f,-0.6176000187337391533759500816813670098781585693359375f,-0.06610001656562047855913988314568996429443359375f,-0.2257000178873538320800662404508329927921295166015625f,0.240500004187225613350165076553821563720703125f,-0.1738000136509534332418525082175619900226593017578125f,0.4972000048190350351973165743402205407619476318359375f,-0.873100006753205537535222902079112827777862548828125f,-1.181299956467749101562958458089269697666168212890625f,0.08090000521242701747581804738729260861873626708984375f,0.97820001287683766566516396778752095997333526611328125f,-0.1239000373020766598841646555229090154170989990234375f,-1.38729999009966764589307786081917583942413330078125f,0.2586000250592823679340881426469422876834869384765625f,-0.3059999659806482696922103059478104114532470703125f,-0.111999992311000884370741914608515799045562744140625f,-0.265900010912120510209177837168681435286998748779296875f,-0.6119000313937659196739105027518235146999359130859375f,-0.43520001001209107638345585655770264565944671630859375f,-0.3384000175058847759146374301053583621978759765625f,-1.248500002115964235827050288207828998565673828125f,0.068299979065356897223182386369444429874420166015625f,-1.2520000100806345511017525495844893157482147216796875f,-0.30179992731362637048420083374367095530033111572265625f,0.5917999631881720734583041121368296444416046142578125f,-0.59970002568066138604052639493602328002452850341796875f,-0.3390000025033952368858081172220408916473388671875f,-1.12210001210644794156223724712617695331573486328125f,0.42610000855624619031658539825002662837505340576171875f,1.06879996943175736845432766131125390529632568359375f,1.993299978581070330818647562409751117229461669921875f,0.51649998369067962045164676965214312076568603515625f,0.0496999825417991925036176326102577149868011474609375f,0.15100000807642910416461745626293122768402099609375f,0.075600009323656625070952941314317286014556884765625f,0.9985999654173853823380113681196235120296478271484375f,0.099900042547286016514362927409820258617401123046875f,0.0531999307155606171448880559182725846767425537109375f,0.515200011427700221844361294643022119998931884765625f,-0.12920008873492516698888721293769776821136474609375f,-0.5519999795705066159001717096543870866298675537109375f,-0.4104000315353275585295023120124824345111846923828125f,-0.75379998634457567874278538511134684085845947265625f,0.1818000014841556488676133085391484200954437255859375f,-0.52439998002201371196662194051896221935749053955078125f,0.4308999981343755170115628061466850340366363525390625f,-1.1875000591203577204879593409714289009571075439453125f,-0.56089999407380819462787258089520037174224853515625f,0.4806999820128077249847819985006935894489288330078125f,0.30070005204826666389550382518791593611240386962890625f,0.52559996000081188327612835564650595188140869140625f,0.3085999716758731636900847661308944225311279296875f,0.579200029800832627557838350185193121433258056640625f,-0.96760005486905653615536948564113117754459381103515625f,0.24720005466342076516639281180687248706817626953125f,-0.75970001215785776249589389408356510102748870849609375f,-1.014199957716465316792664452805183827877044677734375f,-1.5808000168025493348267218607361428439617156982421875f,-0.78869995250478430737217649948433972895145416259765625f,1.8905000081360350261405756100430153310298919677734375f,0.9995999761760234747498543583787977695465087890625f,0.790700051116944191420543575077317655086517333984375f,1.31470003961324710672897708718664944171905517578125f,1.039100017623602933980464513297192752361297607421875f,0.9363999703958632014888507910654880106449127197265625f,-0.013100004965066858186872877922724001109600067138671875f,0.0370000054612752149463261730488738976418972015380859375f,-0.76010004246383899850769694239716045558452606201171875f,0.34170002413839117849647664115764200687408447265625f,-0.1563999714016921505077561960206367075443267822265625f,-0.51870000110119551806064919219352304935455322265625f,0.46300000049918843547658298120950348675251007080078125f,0.6149000086843965373617493241908960044384002685546875f,-0.149599929572641787700604254496283829212188720703125f,0.347300006304681385671528914826922118663787841796875f,-0.3478000146642330481228100325097329914569854736328125f,2.146000013738870393353863619267940521240234375f,-1.3956999947905532888370316868531517684459686279296875f,0.391000010013579757384150070720352232456207275390625f,-1.19970006200224243997354278690181672573089599609375f,-2.23880003112256442676653023227117955684661865234375f,0.4393000102758410907455299820867367088794708251953125f,1.3551999545052648699083874817006289958953857421875f,-0.029100023919343964706740734982304275035858154296875f,-0.165299976889788258205271631595678627490997314453125f,1.006200012128056098248407579376362264156341552734375f,-0.11639997913166910681326271514990366995334625244140625f,-0.25159998558610696672843687338172458112239837646484375f,1.119200036282837373846632544882595539093017578125f,0.2829999287128439622307496392750181257724761962890625f,-0.362400033122300957444394953199662268161773681640625f,0.805199962812661595989993656985461711883544921875f,1.25190003213882494748077078838832676410675048828125f,1.9990000607296829482351085971458815038204193115234375f,-0.012800007884204323005405967705883085727691650390625f,0.31950000315159510222429162240587174892425537109375f,0.79559997993111597480719865416176617145538330078125f,-0.2508000042483209401922295000986196100711822509765625f,-1.0277999942496414131909432398970238864421844482421875f,-0.93389998715966904274665694174473173916339874267578125f,-0.20570001460909892454509417802910320460796356201171875f,-0.7529000012859692336775196963571943342685699462890625f,1.6676000185102226591737917260616086423397064208984375f,0.1606000260874622664886146594653837382793426513671875f,-0.2746000491544606791904925557901151478290557861328125f,0.42260000323653201803608681075274944305419921875f,1.5206000411003828265421589094330556690692901611328125f,-0.50330002119839167828985182495671324431896209716796875f,-0.0313999605238441947818728294805623590946197509765625f,0.5423999991849068891269780579023063182830810546875f,-0.81309996592402489223871953072375617921352386474609375f,-1.3743000493541364104288504677242599427700042724609375f,0.46030003183782053355344032752327620983123779296875f,0.4988999400347464696636734515777789056301116943359375f,-0.62139999747872298740958285634405910968780517578125f,-0.7452000242054463985397205760818906128406524658203125f,-1.4210000371709468058867287254543043673038482666015625f,-0.8130999446526170348903406193130649626255035400390625f,0.70079999392926672907577767546172253787517547607421875f,1.2045000207722187379744127611047588288784027099609375f,0.805699989165365781929040167597122490406036376953125f,0.127999991185962613826632150448858737945556640625f,1.238799998302756666390678219613619148731231689453125f,-0.22540000221729261653536013909615576267242431640625f,-0.39290001484602699033388262250809930264949798583984375f,-0.73150003185868339716790842430782504379749298095703125f,-1.074600061373411907794661601656116545200347900390625f,0.102800021332502211635073763318359851837158203125f,-0.225200015115737617321656216518022119998931884765625f,0.3647000159204003555402096026227809488773345947265625f,0.2487999732315546719263466002303175628185272216796875f,-0.24089998267442014334704936118214391171932220458984375f,-0.481599995905160671583189468947239220142364501953125f,-0.24700000941008337651538795398664660751819610595703125f,0.57300005030632117364319810803863219916820526123046875f,1.5685000199973586632040678523480892181396484375f,-0.342800075386464175863920900155790150165557861328125f,1.2026999827429649680965439983992837369441986083984375f,-1.242100029745698552829935579211451113224029541015625f,-0.51450002263486371845147004933096468448638916015625f,0.934500012446195515991576030501164495944976806640625f,0.114000003769993807356541992703569121658802032470703125f,0.9948000226959583525143671067780815064907073974609375f,0.515799944271147836616364656947553157806396484375f,-0.6016000425085412661729833416757173836231231689453125f,-1.1209999636337160122678824336617253720760345458984375f,-0.84370005679428661249374954422819428145885467529296875f,-0.20979999844431862499050112091936171054840087890625f,-0.988400003126262749475472446647472679615020751953125f,-0.9643000064760449685508092443342320621013641357421875f,-0.3279999912604687484218857207451947033405303955078125f,-0.66950002837181121773113545714295469224452972412109375f,-0.6689000324778262562830377646605484187602996826171875f,-0.08149998201429864419509385697892867028713226318359375f,0.88540000370740923330004079616628587245941162109375f,-0.0942000039100643338230156587087549269199371337890625f,0.44540004215240502905004404965438880026340484619140625f,0.3937999913550911390558439961750991642475128173828125f,0.6237000348523265014222261015675030648708343505859375f,-0.483599959158897974020874244160950183868408203125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.474499999836087038573850804823450744152069091796875f,0.1140999971717595595777083872235380113124847412109375f,-0.42810000306516837742520920073729939758777618408203125f,-1.1313000031828881475348680396564304828643798828125f,-0.082399998514354588774466492395731620490550994873046875f,0.722300007086992490457078019971959292888641357421875f,0.5951000121399754316797725550713948905467987060546875f,0.841100015000999068348619402968324720859527587890625f,0.90280001358389849741570287733338773250579833984375f,-0.299800024670362486034491666941903531551361083984375f,0.957900008177757111837991033098660409450531005859375f,0.48820001207590102421107758345897309482097625732421875f,1.0622000042229888805422888253815472126007080078125f,0.797800015558302266782675360445864498615264892578125f,0.809300018322467895615091038052923977375030517578125f,-0.16290000072717614187922663404606282711029052734375f,-0.517100014919042560990192214376293122768402099609375f,0.35990000056326376665083444095216691493988037109375f,1.02270001682937117237770507927052676677703857421875f,0.6952000035673382427603428368456661701202392578125f,0.1760000080764292373913804112817160785198211669921875f,0.5275999998837705984300328054814599454402923583984375f,1.28399998994916675343347378657199442386627197265625f,-0.0149999897927048930768023637938313186168670654296875f,-0.031800008040666849495892165577970445156097412109375f,0.814300005209446009502016750047914683818817138671875f,-0.00489998305439964809693265124224126338958740234375f,0.397500008791685122133685581502504646778106689453125f,-0.07499999947845947811941869076690636575222015380859375f,-0.2934999998994169967403422560892067849636077880859375f,0.0454000063896182748379715121700428426265716552734375f,-0.88030002207756030685459336382336914539337158203125f,-0.030000034868717051494968472979962825775146484375f,-0.0577000255465514033659246706520207226276397705078125f,-0.4896000271379941093385923522873781621456146240234375f,0.499399980729818171454326147795654833316802978515625f,-0.5128999823242423872216022573411464691162109375f,-0.735300020177662272402585585950873792171478271484375f,-0.524600016409158431685000323341228067874908447265625f,-1.1254000185340646833509481439250521361827850341796875f,-0.2071000035196544075688507291488349437713623046875f,0.410400025463104611844755709171295166015625f,-0.259199996459483994470929246745072305202484130859375f,-0.19820000015497252210394663052284158766269683837890625f,-0.342899997616559204516306635923683643341064453125f,-0.86059998820126093477256290498189628124237060546875f,-1.22599997313320674408032573410309851169586181640625f,-0.40459998966157417754629932460375130176544189453125f,0.5201000176161529875429323510616086423397064208984375f,-1.0387000155180687865907884770422242581844329833984375f,-1.338900014913082525680465550976805388927459716796875f,-0.36849999831616886414309419706114567816257476806640625f,0.43150004459917556953740813696640543639659881591796875f,-0.6881000024318695285074909406830556690692901611328125f,-0.1157000048488379118083457797183655202388763427734375f,-0.1641000082731245424128019294585101306438446044921875f,1.132699992614984552119494765065610408782958984375f,0.6353000147759912952238892103196121752262115478515625f,-0.58340002756416797780048000277020037174224853515625f,0.3686999997973441356435841953498311340808868408203125f,-0.4430999816447502848149042620207183063030242919921875f,-0.298299980443716084010929989744909107685089111328125f,-0.125999994814395943620866091805510222911834716796875f,-0.1935000183582304966734000117867253720760345458984375f,0.349199994373321498386530947755090892314910888671875f,0.63680000312328310929643748750095255672931671142578125f,0.5842000251442194258544304830138571560382843017578125f,-0.0257999834612012701740013653761707246303558349609375f,-0.0884000001087783815467702197565813548862934112548828125f,0.010900010092556644547556743418681435286998748779296875f,0.11139998354613833697612790274433791637420654296875f,-0.794000020682811946670653924229554831981658935546875f,0.19350000628829011617426658631302416324615478515625f,0.1925999601721766207873542953166179358959197998046875f,1.58949998001754266852003638632595539093017578125f,0.479700024041533001906145727843977510929107666015625f,0.2022000189006332160346346427104435861110687255859375f,0.15809997486472138206181625719182193279266357421875f,0.3709999796152116946501564598293043673038482666015625f,-0.037900013244151598001963066053576767444610595703125f,-0.4680000343620778213704625159152783453464508056640625f,-0.15000000879168506884298039949499070644378662109375f,0.104899992889165627474312714184634387493133544921875f,0.50640000045895583458133160092984326183795928955078125f,0.6224999786913396615517513055237941443920135498046875f,1.276699991840124059905292597250081598758697509765625f,0.0361000183165074428615071155945770442485809326171875f,-0.86009999474287013310203064975212328135967254638671875f,-1.52020002107620211262428711052052676677703857421875f,-0.63380002233088028606999841940705664455890655517578125f,0.02900004020333302179324164171703159809112548828125f,-0.26339995991289610888230754426331259310245513916015625f,-0.4308999663949013569919088695314712822437286376953125f,-0.00449996049702161560190916134160943329334259033203125f,-0.0160000235736366125394170012441463768482208251953125f,1.228499986022710377397970660240389406681060791015625f,-0.463999984845518742559988822904415428638458251953125f,-0.00820001364052291847883680020458996295928955078125f,0.541800006401538691847008522017858922481536865234375f,0.2119000032678244327399852409143932163715362548828125f,0.8089999901726845354943407073733396828174591064453125f,0.66899998007714744918672522544511593878269195556640625f,-0.5854999650716783410331345294252969324588775634765625f,-0.55410002093911125964353914241655729711055755615234375f,0.398099984848499344192163107436499558389186859130859375f,0.14800001752376534813748776286956854164600372314453125f,0.268200043591857006930467832717113196849822998046875f,-0.4639000026941300092175879399292171001434326171875f,-0.11789999595880484850596303658676333725452423095703125f,-0.5970000284463166106974085778347216546535491943359375f,-0.6202000211507083804463036358356475830078125f,0.161900024092197281078142623300664126873016357421875f,-1.137500005811453274162658999557606875896453857421875f,-0.5717999931395052470861628535203635692596435546875f,0.122500013858080070150435858522541821002960205078125f,0.945999999806284908032694147550500929355621337890625f,0.5181999994099140138814618694595992565155029296875f,-0.07389999777674649550363028538413345813751220703125f,-0.6555999891698365278358551222481764853000640869140625f,-0.59069996819198100634906722916639409959316253662109375f,0.297299986150860728884026684681884944438934326171875f,0.37560000578463093034997655195184051990509033203125f,0.090400000913441136418668975238688290119171142578125f,-0.0268000043898819484411433222703635692596435546875f,-0.4020000302344561848855164498672820627689361572265625f,0.7252999916791915691050007808371447026729583740234375f,-0.18020001824498166342891636304557323455810546875f,-0.211899989819527068135585068375803530216217041015625f,0.02100001161545517192053011967800557613372802734375f,-0.2903999979145821530579496538848616182804107666015625f,-0.2323000031754374195003265413106419146060943603515625f,-0.044999985881149895394770510392845608294010162353515625f,0.3915999956443905194447552275960333645343780517578125f,1.08400003889948148838584529585205018520355224609375f,-0.0261999991826713241049873204246978275477886199951171875f,0.58280000576078860863304953454644419252872467041015625f,0.5496000096291304704010372006450779736042022705078125f,-0.631699985507130801209996207035146653652191162109375f,-1.18160000678300836085554692544974386692047119140625f,-0.162899994021654226372675111633725464344024658203125f,1.1105000169277194910222306134528480470180511474609375f,0.632699998724460588306328645558096468448638916015625f,-0.31890001185834437880117775421240366995334625244140625f,-0.10380000064969063799935611314140260219573974609375f,-0.12870000844001772133395888886298052966594696044921875f,-0.6798999923676254386606387924985028803348541259765625f,-1.1491000009343028676056519543635658919811248779296875f,-0.25799996121600254905814608719083480536937713623046875f,-0.024400016045570627198912916355766355991363525390625f,0.464800005820393824507164026726968586444854736328125f,0.2737000091850756433586866478435695171356201171875f,-0.9787000038951634284245528760948218405246734619140625f,-0.370899994111061115376060115522705018520355224609375f,0.655500000983476382998560438863933086395263671875f,0.8102999910086392532093668705783784389495849609375f,-0.9641000015676020051813566169585101306438446044921875f,-1.439899998402595304725082314689643681049346923828125f,0.26679998539090143960805789902224205434322357177734375f,0.58909999277591695854283670996665023267269134521484375f,0.59139999639838947143033465181360952556133270263671875f,0.4843000065505502771401324935141019523143768310546875f,0.0075000001490115320024187894887290894985198974609375f,0.3092000015258786760341536137275397777557373046875f,0.000799983796477388153789433999918401241302490234375f,0.78199998337030418582571655861102044582366943359375f,0.796999981582164895854702990618534386157989501953125f,0.9341000228762628143641677525010891258716583251953125f,0.155999992281198540666764529305510222911834716796875f,-0.144499982699751949866140421363525092601776123046875f,-0.0140000233650210414992898222408257424831390380859375f,0.772700015115737937065887308563105762004852294921875f,0.2064999986290929623322654151706956326961517333984375f,-0.7276999847546221644734032452106475830078125f,-0.69759998051226135284963447702466510236263275146484375f,-0.65699999368935813048864247321034781634807586669921875f,-1.251900001777708215655593448900617659091949462890625f,-0.626299987462162643936380845843814313411712646484375f,-0.7370000209957361381185592108522541821002960205078125f,-0.57050000701844683081986886463710106909275054931640625f,-0.0101000003494322587727083373465575277805328369140625f,-0.1503000417470932159602625688421539962291717529296875f,-0.1062000085890293377133275498636066913604736328125f,-0.1732000266790389986226728069595992565155029296875f,1.6203000173091890445675744558684527873992919921875f,0.22820002632513680662640354057657532393932342529296875f,-1.1358999849282211958012567265541292726993560791015625f,-0.313699974390864166906567334081046283245086669921875f,0.66789998418688778514251680462621152400970458984375f,0.514300001186132504926717956550419330596923828125f,0.69930000021755633898834503270336426794528961181640625f,0.2803000030040740941927879248396493494510650634765625f,0.55580000950098007450606019119732081890106201171875f,-0.8996000008374454015580568011500872671604156494140625f,-1.0865999760493638692793183508911170065402984619140625f,-0.52659999165833004663994643124169670045375823974609375f,0.3499999958090482810035837246687151491641998291015625f,0.10270001332759814971495870850048959255218505859375f,-0.38639999569058414152777913841418921947479248046875f,0.176099994102120405159439542330801486968994140625f,0.01529996597468874508507497012033127248287200927734375f,0.265399996033310880960698341368697583675384521484375f,0.2393000187695026870215997405466623604297637939453125f,0.4199999823048710112516346271149814128875732421875f,1.26089997846484092036689617088995873928070068359375f,0.782900002962351404534047105698846280574798583984375f,0.379599983924627348841340790386311709880828857421875f,0.173099998408555943996134374174289405345916748046875f,-1.0525999937742955836483815801329910755157470703125f,-0.3957999952703714630075637614936567842960357666015625f,0.7677000038653611913019858548068441450595855712890625f,0.512699971306324275843735449598170816898345947265625f,0.7417000097215173948228539302363060414791107177734375f,-0.134099954032897705769045160195673815906047821044921875f,0.0299999903142451618265340584912337362766265869140625f,0.1271000114172695649727984346100129187107086181640625f,-0.541200006800889976688040405861102044582366943359375f,-0.2196999955803151660660432753502391278743743896484375f,-0.433400011694431430697704854537732899188995361328125f,-0.0111999928683046423572022831649519503116607666015625f,0.76830001434385764014223241247236728668212890625f,-0.0153000152975319991810465580783784389495849609375f,0.1046000564932825227515422739088535308837890625f,-0.815700039494037465459541635937057435512542724609375f,0.834199973288178409092097354005090892314910888671875f,0.34509999042153349790851279976777732372283935546875f,0.356000024914741697301678868825547397136688232421875f,-0.069300018769502980120478241587989032268524169921875f,0.6739000015020371048279912429279647767543792724609375f,0.76770000993758469309824477022630162537097930908203125f,0.3056999649137261965137213337584398686885833740234375f,0.40259999526441081485472750500775873661041259765625f,-0.34550000433623784257264333064085803925991058349609375f,0.04080002164542662068669187647174112498760223388671875f,-0.4428999931275843238154266146011650562286376953125f,-0.3398000099927187545034712456981651484966278076171875f,0.517899972005188669044173366273753345012664794921875f,0.441500020906328671799201401881873607635498046875f,-0.91640000589787984353051797370426356792449951171875f,0.35819999508857680581286331289447844028472900390625f,1.61560001360774041501144893118180334568023681640625f,-0.27500000007450609018633258529007434844970703125f,-1.3183000124067068004052316609886474907398223876953125f,-0.032500004880130273221539027872495353221893310546875f,0.049699997740983992144947478664107620716094970703125f,0.8677000224918127724293981373193673789501190185546875f,0.17240003103613854573694652572157792747020721435546875f,-0.019499992311001079769994248636066913604736328125f,0.9168999735027549835564286695444025099277496337890625f,0.9052000294953586045920701508293859660625457763671875f,0.293000005304813182505085933371447026729583740234375f,0.2697000183790922989146565669216215610504150390625f,-0.1752999994277952833243716668221168220043182373046875f,0.25759998922944082888619732329971157014369964599609375f,-0.0355999957449735504599885871357400901615619659423828125f,-1.29020000207722151941425181576050817966461181640625f,-0.11000001132488257393760022750939242541790008544921875f,0.8281000031024217111763618959230370819568634033203125f,0.05150000537186871785166886184015311300754547119140625f,-0.39810001506060366249784010506118647754192352294921875f,0.88370001732483505207937923842109739780426025390625f,-0.24220000087022786061652368516661226749420166015625f,-0.051100029492378684636832986143417656421661376953125f,0.61720000580623757269194129548850469291210174560546875f,0.1705000006295740722084275375891593284904956817626953125f,-0.269299987626075765234645587042905390262603759765625f,-0.75440002029240160741352383411140181124210357666015625f,0.0297000060111287655928435924579389393329620361328125f,-0.38979999635815676395367290751892141997814178466796875f,-0.860699997547268669251252504182048141956329345703125f,-0.1987000059068204649292965768836438655853271484375f,0.423200001347064702628131271922029554843902587890625f,0.62939999004304458818381817764020524919033050537109375f,0.3583999603599306027490456472150981426239013671875f,0.50149996748566605386798755716881714761257171630859375f,0.6567000224992636248089183936826884746551513671875f,-0.5530000365227467629125612802454270422458648681640625f,0.36580000592470174325399057124741375446319580078125f,0.6564000130504370478234932306804694235324859619140625f,-0.1642999925062060828651055999216623604297637939453125f,0.3818000122874976032250060598016716539859771728515625f,0.29419999590069068062803125940263271331787109375f,1.1191999894559383488257253702613525092601776123046875f,0.9578999780401584729361275094561278820037841796875f,0.985200011688470933535199947073124349117279052734375f,0.072600017616152978661148154060356318950653076171875f,-0.496099980542063168087452140753157436847686767578125f,-0.53010003914833081495316946529783308506011962890625f,0.5705000200569632085034754709340631961822509765625f,-0.86829999020397696796180753153748810291290283203125f,-0.54919999105781325265951409164699725806713104248046875f,-0.68419999744668569263694735127501189708709716796875f,-0.3140999716162677923847468264284543693065643310546875f,0.847599992656707712512798025272786617279052734375f,0.976800049987435414777792175300419330596923828125f,0.348600001627206612653253614553250372409820556640625f,0.70179999641776069640997093301848508417606353759765625f,0.5326000188827515469114359802915714681148529052734375f,-0.0608000071167946742889398592524230480194091796875f,-0.51260002104341939599407851346768438816070556640625f,-1.5316000322639948816316746160737238824367523193359375f,0.582299999040365445779343644971959292888641357421875f,-0.00710000396668863942295502056367695331573486328125f,-0.4961999869823454645256788353435695171356201171875f,-0.0260999903023240442667685101696406491100788116455078125f,-0.342900024010241055716363689498393796384334564208984375f,0.598099999302625473518446597154252231121063232421875f,-1.094399997866152940417805439210496842861175537109375f,-0.26080003095865311735224167932756245136260986328125f,0.5208000001877548523765426580212078988552093505859375f,-0.066300029110908820229042248683981597423553466796875f,-0.68820000760555277707908317097462713718414306640625f,0.1188999845147133971323682999354787170886993408203125f,0.759799991068244029435163611196912825107574462890625f,0.2458999746277925613213710676063783466815948486328125f,-0.1175000029243526977751344020362012088298797607421875f,-0.0109999786466359328329644995392300188541412353515625f,-0.3059000032007694880320514130289666354656219482421875f,0.659700006458163290545826384914107620716094970703125f,-0.340100018435716577869243337772786617279052734375f,-0.61940002282559891710889132809825241565704345703125f,0.50590002264678435306421988570946268737316131591796875f,-1.1464000042587521033254915892030112445354461669921875f,-0.60839999969899682952245711931027472019195556640625f,-0.24689999443888677177483259583823382854461669921875f,0.5122999840646984548442333107232116162776947021484375f,0.4683000027954580257727457137661986052989959716796875f,0.578400022721290429927876175497658550739288330078125f,0.168100027167797261284931664704345166683197021484375f,0.241999973610043550564796532853506505489349365234375f,0.87339998777806737706441708724014461040496826171875f,0.61700001294910900018209076733910478651523590087890625f,-0.4245999803483488932442924124188721179962158203125f,-0.5155000094771384766545452293939888477325439453125f,-0.717000012725591329143526309053413569927215576171875f,-0.356400033167004526291066213161684572696685791015625f,0.09710000508427629739571784739382565021514892578125f,-0.4110999986469749956086161546409130096435546875f,0.3148999935597185384494878235273063182830810546875f,1.131099993059038855136577694793231785297393798828125f,0.3131999737054116383916380073060281574726104736328125f,0.279099957683681676456899367622099816799163818359375f,0.55660002049207690788534819148480892181396484375f,-0.2025999957114457128426465715165250003337860107421875f,-0.001499973036349189214888610877096652984619140625f,-0.64399999691545961599814518194762058556079864501953125f,-0.3697999947190282465925292854080908000469207763671875f,-0.614899958392978174259724255534820258617401123046875f,-0.885199992130696333703099298872984945774078369140625f,-0.409800035212934066208845251821912825107574462890625f,1.1214000129014254358850166681804694235324859619140625f,0.415099988335370984060546106775291264057159423828125f,0.6282999951958654349937205552123486995697021484375f,1.021800038588046799503672446007840335369110107421875f,0.5980999940872191444896088796667754650115966796875f,-0.331199982511997337297771082376129925251007080078125f,-0.555099979692697420574631905765272676944732666015625f,0.2609999951124191142071140347979962825775146484375f,0.69239999810457197870761092417524196207523345947265625f,0.222800018277764355634218418344971723854541778564453125f,-0.2725999963819980553836330727790482342243194580078125f,0.407499995604157394524236224242486059665679931640625f,0.625300028708577482206010245135985314846038818359375f,-1.2295999896615743551819832646287977695465087890625f,-1.0823000076644120337476806525955908000469207763671875f,0.178199982161820258408368999880622141063213348388671875f,0.35879995058178881794930248361197300255298614501953125f,0.467499982006847669691040891848388127982616424560546875f,0.50530002151876718397005561200785450637340545654296875f,0.1771999988771975065304786767228506505489349365234375f,-0.10069999696984888171158445402397774159908294677734375f,-0.44669998157694890128510678550810553133487701416015625f,-0.1337999923795460954778491213801316916942596435546875f,0.39150002999603772391168376998393796384334564208984375f,-0.9476999956138438818697977694682776927947998046875f,-1.0710999995410441787413446945720352232456207275390625f,-0.3202999757349491716240663663484156131744384765625f,0.224300005656480561100352133507840335369110107421875f,0.719400037056207697361287500825710594654083251953125f,0.219900016060471603651649274979718029499053955078125f,-0.2323999912500382958313593917409889400005340576171875f,-0.4518999776750802777058879655669443309307098388671875f,-0.2898999759763487560348949045874178409576416015625f,-1.2784999954849478598362111370079219341278076171875f,-1.1182999741852288622112610028125345706939697265625f,-0.9047999874919649965221424281480722129344940185546875f,0.0524000280559062669993863892159424722194671630859375f,-0.433799994689226142696725219138897955417633056640625f,0.0559000132590530274256934717413969337940216064453125f,-0.40329999312832931490646615202422253787517547607421875f,-0.6712999963469805653204502959852106869220733642578125f,-0.0035000122487540696880614632391370832920074462890625f,-0.27530000404715526673982139982399530708789825439453125f,-1.07829997262060661711302600451745092868804931640625f,-1.355900003647803941930760629475116729736328125f,-0.0175000293552878272151929195388220250606536865234375f,0.613800007504224698351436018128879368305206298828125f,-0.20420001285076150310260345577262341976165771484375f,0.65259999481737607229803188602090813219547271728515625f,-0.307699985685944366053945486783050000667572021484375f,-0.28900001740455649734684584473143331706523895263671875f,1.527500020358711463330791957559995353221893310546875f,-0.187500017881393699070713410037569701671600341796875f,0.29689995748400743735828655189834535121917724609375f,-0.019899995943904347228681217529810965061187744140625f,-0.72859997510313956325944673153571784496307373046875f,-0.47430001411288980062153086691978387534618377685546875f,0.790199991888552943919421522878110408782958984375f,0.6466000036537646877832230529747903347015380859375f,0.0784000122159718149106311102514155209064483642578125f,-1.585099995338916212261892724200151860713958740234375f,-0.7577999927967786764071433935896493494510650634765625f,-0.666300003182887845554205341613851487636566162109375f,-0.0739999797046184681903469027020037174224853515625f,-0.88080000458359730686197508475743234157562255859375f,-0.77110000550150825660011832951568067073822021484375f,-0.6650000222027305785132966775563545525074005126953125f,0.14330000219941130268352935672737658023834228515625f,0.33040000475048980632664097356610000133514404296875f,-0.175899989417195090624090880737639963626861572265625f,0.4448000252664090492515924779581837356090545654296875f,0.072299982574582255523409912711940705776214599609375f,0.453700015965104175297284427870181389153003692626953125f,0.09060000150054692280576773555367253720760345458984375f,-0.9481999647647143536488556492258794605731964111328125f,-0.94469997337758526700923766838968731462955474853515625f,-0.353499996751546863293924616300500929355621337890625f,-0.249000036031007787595825675452942959964275360107421875f,0.0516999742381273053037915587992756627500057220458984375f,-0.418899976095557402544500291696749627590179443359375f,-1.20130000072419651502286797040142118930816650390625f,-0.1500000046566130063041555331437848508358001708984375f,0.2224000058531760970481627737171947956085205078125f,0.29009999444484702024737998726777732372283935546875f,-0.582199977773428134497635255684144794940948486328125f,-1.2512000012129542714234275990747846662998199462890625f,-0.541700015011429503175577337970025837421417236328125f,-0.08619998295903175122845141231664456427097320556640625f,-1.25139999978840332772733745514415204524993896484375f,-1.1331000109255313645917340181767940521240234375f,-0.19579999981522588914373272928060032427310943603515625f,0.24729999613463871099838797817938029766082763671875f,-0.152799994286894946782240367610938847064971923828125f,-0.77529999514371106261734212239389307796955108642578125f,-0.38279997845441104953323474546778015792369842529296875f,0.1056000049091876669127287868832354433834552764892578125f,0.96499999120831514431984032853506505489349365234375f,1.049799975049495959211753870476968586444854736328125f,1.1295999720409513056296191280125640332698822021484375f,-0.105599969016015482470294273298350162804126739501953125f,-0.27990001255869856322533451020717620849609375f,0.756099999615549744902409656788222491741180419921875f,0.19229999449551116175172182920505292713642120361328125f,0.865699988234042816515056983917020261287689208984375f,0.6596000165954232219434061335050500929355621337890625f,0.251099972905218848939767894989927299320697784423828125f,-0.30560004281401631498482629467616789042949676513671875f,-0.5389000259399414627381474929279647767543792724609375f,-0.37149999855458748498904242296703159809112548828125f,-0.7919000047206878889483050443232059478759765625f,-0.43340000692605962218095783100579865276813507080078125f,0.6330999884247778819457153076655231416225433349609375f,0.2587000216275452402214796165935695171356201171875f,-0.945999992355704311108866022550500929355621337890625f,-0.10589999329149668572114251219318248331546783447265625f,0.15030000892728556660671301870024763047695159912109375f,0.459100007863342318703558930792496539652347564697265625f,1.21790001637339617701627503265626728534698486328125f,0.204099992030859045399893147987313568592071533203125f,0.23529998571872690060757804531021974980831146240234375f,-0.055299987283348617239653322030790150165557861328125f,-0.7894000340759761424891394199221394956111907958984375f,0.01730003703832683203955866702017374336719512939453125f,-0.8711999850079419349668796712649054825305938720703125f,-1.2467999830439684405547495771315880119800567626953125f,-0.50450000661611593688604671115172095596790313720703125f,1.003499995093792396261278554447926580905914306640625f,0.895800006222724931603806908242404460906982421875f,0.40660000421255781954954500179155729711055755615234375f,0.6991000168040393258905851325835101306438446044921875f,1.030599989691376894285212983959354460239410400390625f,0.57319997787773591113591464818455278873443603515625f,0.0132999901667239794900154947754344902932643890380859375f,0.353699983592331623771798376765218563377857208251953125f,-0.00640000172555466040336114019737578928470611572265625f,-0.39030000151395760621397812428767792880535125732421875f,0.6170000101923942548154400355997495353221893310546875f,0.6503000239402059179383286391384899616241455078125f,0.0438000280678270403900143037390080280601978302001953125f,-0.400000006593763390849716188313323073089122772216796875f,0.8579999869018790636943094796151854097843170166015625f,-0.0501999982029202040934023898444138467311859130859375f,0.805699987339973944244775339029729366302490234375f,0.5908999716103071619954789639450609683990478515625f,0.5400000528991222292773954904987476766109466552734375f,-0.575899982190132053716524751507677137851715087890625f,-1.038900027728080655009534893906675279140472412109375f,-1.03960001916587341241893227561376988887786865234375f,0.025799989011883184275575331412255764007568359375f,0.51100000151991853414301658631302416324615478515625f,-0.1879000262081624850196703846449963748455047607421875f,0.152399992293119124209255232926807366311550140380859375f,0.77789999931156617041239087484427727758884429931640625f,0.49430002994537358684823402654728852212429046630859375f,0.0410000046305358523124340308640967123210430145263671875f,0.6848999880462882661191770239383913576602935791015625f,0.8517000143408781642762050978490151464939117431640625f,-0.34580002835094969526608110754750669002532958984375f,0.0083999729514128151919294396066106855869293212890625f,1.2729999917447563095862506088451482355594635009765625f,1.51560001710951297582141705788671970367431640625f,1.350000051334500739130817237310111522674560546875f,0.480000017359852648723972379229962825775146484375f,0.1818000072956085233499834430404007434844970703125f,-0.54710000411570092193613845665822736918926239013671875f,-1.017200001540780274211783762439154088497161865234375f,-0.7407000062644482074603047294658608734607696533203125f,-0.630700015391409518628051955602131783962249755859375f,0.347999984368681847257676054141484200954437255859375f,0.32640000261962420768924175717984326183795928955078125f,1.283499987699091793302841324475593864917755126953125f,0.4482000179246072857353055951534770429134368896484375f,-0.54970002411603946512741458718664944171905517578125f,0.666400000086426747003542914171703159809112548828125f,0.392800022673606807899204795830883085727691650390625f,-0.477100004674494204692791754496283829212188720703125f,0.47209999662786739094144650152884423732757568359375f,-0.0255999857425691923484833978363894857466220855712890625f,-0.7958999923601746129264711271389387547969818115234375f,0.15899999244511153762005051248706877231597900390625f,0.272499988898635248091295579797588288784027099609375f,-0.795300023716688286867793067358434200286865234375f,-0.2443999915704131442595326006994582712650299072265625f,-1.2800999969407909073737528160563670098781585693359375f,-0.5494000109791759012267675643670372664928436279296875f,0.5606000122666359164469440656830556690692901611328125f,0.73819999784529244379882584325969219207763671875f,0.767400020083785161517653250484727323055267333984375f,0.431100017645955180256578387343324720859527587890625f,0.191199975955486767276170212426222860813140869140625f,0.67349999019503581365597710828296840190887451171875f,-0.045899987331032565673893941493588499724864959716796875f,-0.19920000391006442708174972722190432250499725341796875f,-0.52959999144971359186229165061376988887786865234375f,-0.568500006884336439583194078295491635799407958984375f,0.26520001660585401648262404705747030675411224365234375f,0.31440000755190833370278369329753331840038299560546875f,-0.045899975857138841650595395549316890537738800048828125f,-0.73429999969601633669213924804353155195713043212890625f,-0.114799997103214312943464392446912825107574462890625f,-0.03710000813901448424303453066386282444000244140625f,0.80370000598132562696918057554285041987895965576171875f,1.4494000296056268606292860567918978631496429443359375f,0.527400021350383330087652211659587919712066650390625f,-0.5619000141084187216478085247217677533626556396484375f,0.402699985872209398340970665230997838079929351806640625f,-0.28170001896023766363441609428264200687408447265625f,0.0773999971546233123120828167884610593318939208984375f,0.1610000011101363204257808092734194360673427581787109375f,0.653899983248114491374280987656675279140472412109375f,0.8297999808609481942056618208880536258220672607421875f,-0.9194999756217006048331086276448331773281097412109375f,-0.6592999840870497063605171206290833652019500732421875f,-0.53319998888596875641354699837393127381801605224609375f,-1.0403000515632332412252480935421772301197052001953125f,-0.3632999952703717116975212775287218391895294189453125f,-0.98449998614564504517687737461528740823268890380859375f,-0.6756000198103488951772988002630881965160369873046875f,-0.1741999966830014567875650755013339221477508544921875f,-0.64470000731498000590136143728159368038177490234375f,0.0165999916397037150517235204461030662059783935546875f,0.6541999766409392069732575691887177526950836181640625f,0.777200006756186834167010601959191262722015380859375f,0.449200016799569112890821998007595539093017578125f,0.3233000008396802460453045569011010229587554931640625f,-0.38419999608695487580689587048254907131195068359375f,0.1910000098645685540077465702779591083526611328125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.33310000302791564763538190163671970367431640625f,0.26159998256862149190737909520976245403289794921875f,0.215499989248812140463229525266797281801700592041015625f,-0.768599990451335646213237851043231785297393798828125f,0.11519999250322576767047166867996565997600555419921875f,0.640299989891051968271540317800827324390411376953125f,0.106200010488926910312557083670981228351593017578125f,0.17830001136362572111693225451745092868804931640625f,-0.07520000271052129647841866244561970233917236328125f,0.65739999899864187682396732270717620849609375f,-0.25060003346353820763425801487755961716175079345703125f,1.3594000469654801133145838321070186793804168701171875f,-0.37020001854300499388727985206060111522674560546875f,-0.69900005003810006254383324630907736718654632568359375f,0.45849999179691069972619743566610850393772125244140625f,0.80850001329183596165961489532492123544216156005859375f,0.76250003058463367011654554517008364200592041015625f,1.064400033405423329213590477593243122100830078125f,0.7123000063419340222026221454143524169921875f,0.410600032941997650226539917639456689357757568359375f,-0.48509998461008052572651649825274944305419921875f,1.579100002983211847862321519642136991024017333984375f,0.712699975366890559058674625703133642673492431640625f,1.490700044336914942277871887199580669403076171875f,0.8165000166594982289325344027020037174224853515625f,-1.007800004456937070784761090180836617946624755859375f,-0.63579998234361401188152740360237658023834228515625f,-0.25850000412762152990353570203296840190887451171875f,-0.098800020319223857967472213204018771648406982421875f,-0.1412999974459407948046418823651038110256195068359375f,-0.8885000305771828532641620768117718398571014404296875f,0.6860000170916318307234860185417346656322479248046875f,0.6347000329449781208523972964030690491199493408203125f,-0.4330000049322844102306362401577644050121307373046875f,-0.4085000383257868517006272668368183076381683349609375f,-0.6488000029966232151679150774725712835788726806640625f,0.281499989092349522934455308131873607635498046875f,0.907699989932775341827664306038059294223785400390625f,0.389600001433491993640956252420437522232532501220703125f,-0.6293000240221624519421084187342785298824310302734375f,-0.6128000236794355348735052757547236979007720947265625f,-1.2387000214040282397576220319024287164211273193359375f,-0.4537999499857414686943002379848621785640716552734375f,-0.258099989914894134557243887684307992458343505859375f,-0.349100018201023598596322017328930087387561798095703125f,-0.17679998762607596063389792107045650482177734375f,-1.3277000266268854655749009907594881951808929443359375f,0.061299975951015550634792816708795726299285888671875f,1.1108999972403046374580526389763690531253814697265625f,1.1603000423431402143847890329197980463504791259765625f,-1.2401999911621210959111749616567976772785186767578125f,-1.02480002094507227639041957445442676544189453125f,-0.302499986812472254626982248737476766109466552734375f,0.7769000265508891889254527995944954454898834228515625f,-0.2945000151097776353026347351260483264923095703125f,0.13899999505281446232629605219699442386627197265625f,0.525299990300834185319445168715901672840118408203125f,1.64910001263171412944075200357474386692047119140625f,1.27419998301118564398848320706747472286224365234375f,-0.4131000145763159547840359664405696094036102294921875f,-0.70049998712539685019606849891715683043003082275390625f,-1.49850003664940611969313977169804275035858154296875f,0.0261999994993207252491629333235323429107666015625f,-0.39850001478195196114029386080801486968994140625f,-0.39220000552684053385377183076343499124050140380859375f,-0.05539999849200250015002211512182839214801788330078125f,-0.741300009590387443125791833153925836086273193359375f,0.9650000138953325201640609520836733281612396240234375f,0.399999995268881303189800746622495353221893310546875f,0.619299987812339747250689470092765986919403076171875f,-0.4990000061914316997757623539655469357967376708984375f,-0.20549999666214002758124479441903531551361083984375f,-0.64270002048015584250606480054557323455810546875f,1.313599982888996375862689092173241078853607177734375f,0.1252999949574473692592846418847329914569854736328125f,-0.1040999970600011703680820573936216533184051513671875f,0.05739999475181090105024850345216691493988037109375f,-0.378799978446959784150749328546226024627685546875f,-0.700000014603137987734271519002504646778106689453125f,0.286000010460615516905136246350593864917755126953125f,0.13519999663829818103977231658063828945159912109375f,-0.9443999939918512609438039362430572509765625f,0.33320002820640814977792842910275794565677642822265625f,0.4787000092968345388300122067448683083057403564453125f,0.350299977523088390540806358330883085727691650390625f,1.422300037783384762946070623002015054225921630859375f,0.5464000325709583716360384642030112445354461669921875f,-0.19499998930841666577862270059995353221893310546875f,0.01270003024041699291046825237572193145751953125f,-0.9760000157505273055136285620392300188541412353515625f,0.7570000379085541908352752216160297393798828125f,1.0744000438362359961530501095694489777088165283203125f,1.04830006068646941486122159403748810291290283203125f,0.4564000153973697404552467560279183089733123779296875f,1.700800023657083936967637782800011336803436279296875f,0.795000000223517577779830389772541821002960205078125f,1.687299994905291811164715909399092197418212890625f,0.7858000494733463625607328140176832675933837890625f,0.39170000689029682572339652324444614350795745849609375f,0.697900025798380507779938852763734757900238037109375f,0.008099976131319852612477916409261524677276611328125f,-0.330300006915628774351034735445864498615264892578125f,-0.1206999901711947043025929815485142171382904052734375f,-0.7067000165760510821399975611711852252483367919921875f,-0.6015000167712571954581335376133210957050323486328125f,-0.36019997506886713978246916667558252811431884765625f,-0.36759996762275637394168370519764721393585205078125f,0.4033000073030590382217042133561335504055023193359375f,0.08030000509023693666676990687847137451171875f,0.140800007303059526719835048425011336803436279296875f,-0.3275999980956318946567762395716272294521331787109375f,-0.3578999688014381508338601634022779762744903564453125f,-0.077900028517843278308419030508957803249359130859375f,-1.686500066876412073924029755289666354656219482421875f,0.30280001678764822958811464559403248131275177001953125f,-0.325600009733438167103258820134215056896209716796875f,0.205699997211992613443953814567066729068756103515625f,-0.86249998986721010396649944595992565155029296875f,-1.346199966157972216507232587900944054126739501953125f,0.0849999814108006734869604770210571587085723876953125f,0.53679999995678695512424383196048438549041748046875f,-0.89380003849864042564377086819149553775787353515625f,-0.238400004728138326104414090877980925142765045166015625f,-0.4720999934241170148396804506774060428142547607421875f,0.5206000017985699113154396400204859673976898193359375f,0.042299989503622015263317734934389591217041015625f,-0.75659998681545204135545645840466022491455078125f,-2.10700002058595448062305877101607620716094970703125f,-0.526200028370320982418206767761148512363433837890625f,1.118500012174248325891312560997903347015380859375f,0.062600002938509202721206747810356318950653076171875f,0.65790000966787320901829616559552960097789764404296875f,0.23149998976290220298324129544198513031005859375f,-0.0647000012427569881623412584303878247737884521484375f,0.8293000396125023154780819822917692363262176513671875f,0.4698000066399572460795752704143524169921875f,0.1025000109896065936254672124050557613372802734375f,-0.356600019188225925148572059697471559047698974609375f,-0.8131000059336421781353010374004952609539031982421875f,-1.631700041610002926972811110317707061767578125f,0.068799982395768122245272024883888661861419677734375f,1.4936000220045446695849022944457828998565673828125f,1.0813000266522170278449266334064304828643798828125f,-0.08130002713650508194831445507588796317577362060546875f,-0.06409999843835834099081694148480892181396484375f,-0.915299994324147547786196810193359851837158203125f,-0.75760002394914660595048871982726268470287322998046875f,0.405499998617916801446625640892307274043560028076171875f,0.730999982297420469734561265795491635799407958984375f,-0.54120005310624907934879956883378326892852783203125f,-0.192300004479288855208096720161847770214080810546875f,0.77250001400709111809561591144301928579807281494140625f,-0.82269999950677130673426518114865757524967193603515625f,0.96849997842311807971782400272786617279052734375f,0.403799942162632152786727601778693497180938720703125f,-0.49810001018047334042648799368180334568023681640625f,-0.4474000116273766369801023756735958158969879150390625f,-0.07750000260770295579249022921430878341197967529296875f,0.094299997125566026312526446417905390262603759765625f,-0.66730003539919879873565378147759474813938140869140625f,1.0290000454559926623687715618871152400970458984375f,0.623800004971027277633766061626374721527099609375f,-0.6848000468730930379024357534945011138916015625f,0.275499962873756221171106517431326210498809814453125f,-0.13430001094639276715270170825533568859100341796875f,0.37819996677637046644804286188445985317230224609375f,0.8095999953612682542569700672174803912639617919921875f,0.7725000347569590530838468112051486968994140625f,0.2445000391379001936087433932698331773281097412109375f,0.4725000023841856577888620449812151491641998291015625f,-0.505800008420646207696336205117404460906982421875f,-0.62150003193318870131633957498706877231597900390625f,0.298600004905461791082643685513176023960113525390625f,-0.16550000314414514690497526316903531551361083984375f,-0.49640001021921642543333064168109558522701263427734375f,-1.1586999922722573952427183030522428452968597412109375f,-1.4275000286102297319956733190338127315044403076171875f,0.344100031295419217514108822797425091266632080078125f,-0.381400001800060284296023382921703159809112548828125f,-0.64320000738203508827695031868643127381801605224609375f,0.5106999897241593089347588829696178436279296875f,-0.1233000173240903496463261035387404263019561767578125f,-0.488000040508806964822952068061567842960357666015625f,-1.0309000567704444950578590578516013920307159423828125f,-0.1716999397046852227077806674060411751270294189453125f,-0.951999986462295222366947200498543679714202880859375f,1.0194999996498221861429556156508624553680419921875f,1.34200000616908088346690419712103903293609619140625f,-0.0892000101685523905103991637588478624820709228515625f,0.3404999957680701072604279033839702606201171875f,1.4322000349193813040216127774328924715518951416015625f,-0.6796000165209175047920098222675733268260955810546875f,-0.16539999566078211756803284515626728534698486328125f,-0.515199975031614076925734480028040707111358642578125f,-0.0264999958723783901604065249557606875896453857421875f,-0.98830001613199713883517460999428294599056243896484375f,-1.2560999941766259890840728985494934022426605224609375f,-0.8962000370502476442879924434237182140350341796875f,-0.6647000363349917506639030762016773223876953125f,-0.339700026686489575666882956284098327159881591796875f,-1.0986000170499086170394775763270445168018341064453125f,-0.96280001417994498513763801383902318775653839111328125f,1.5180000389441854480310212238691747188568115234375f,0.865299962212144446738193437340669333934783935546875f,1.0685999808400865607183050087769515812397003173828125f,0.7150000181049109837516652987687848508358001708984375f,-0.172399997322261133714249581316835246980190277099609375f,-0.56719998060464860145657439716160297393798828125f,-0.465200023832917253230334608815610408782958984375f,1.0536999946936962402332937926985323429107666015625f,1.0041999948576094681129688979126513004302978515625f,-0.2361000174596910028412821702659130096435546875f,0.54510002588629713482504257626715116202831268310546875f,0.8531000326812272493981481602531857788562774658203125f,-0.15290000530928382627138262250809930264949798583984375f,0.5122999969169494960397059912793338298797607421875f,0.0531000089883804093915387056767940521240234375f,-1.473399988336860477744494346552528440952301025390625f,-1.468800039131939794145864652819000184535980224609375f,-0.556700052040815851484012455330230295658111572265625f,-0.307200042183697785702634064364247024059295654296875f,-1.363499998316168859702202098560519516468048095703125f,-0.0027999815836541319669095173594541847705841064453125f,-0.646800032702088412150942531297914683818817138671875f,0.47249999754130822537945277872495353221893310546875f,0.611200013208389236751827411353588104248046875f,1.87790003522336501617928661289624869823455810546875f,-0.19060003387331991842046363672125153243541717529296875f,0.100299989034235370599645875699934549629688262939453125f,1.51030001950711056935006126877851784229278564453125f,0.5198999994829296422693687418359331786632537841796875f,0.7567000068902969278639147887588478624820709228515625f,-0.356800009977817911277497842092998325824737548828125f,0.20810000898838054705919375919620506465435028076171875f,0.17939999764263614423498438554815948009490966796875f,-0.28550003309547899998932507514837197959423065185546875f,-0.430099994184077072389982276945374906063079833984375f,0.53040002475529934855558167328126728534698486328125f,0.2015999567523587554518371689482592046260833740234375f,-0.22530003757476801951042943983338773250579833984375f,0.2888000004634256878688347569550387561321258544921875f,-0.6969000249862673523892908633570186793804168701171875f,-0.521400025083124774027965031564235687255859375f,-0.243300017249584499268166837282478809356689453125f,0.30020000312030303835086897379369474947452545166015625f,0.3705000433400276182993593465653248131275177001953125f,-0.1503999843731520513756549917161464691162109375f,-0.7145000163018699890216112180496565997600555419921875f,-0.896100021743774899363188524148426949977874755859375f,0.14070002254396685170689806909649632871150970458984375f,-1.7350000372156502148612844393937848508358001708984375f,-0.53910001482963565422323881648480892181396484375f,0.13579999530762432158326191711239516735076904296875f,-1.19290001303926107567576764267869293689727783203125f,-0.6367000190719964702878996831714175641536712646484375f,-0.272800006841122932854659666190855205059051513671875f,0.34739998700320706692679095795028842985630035400390625f,0.7149000128194689107630210855859331786632537841796875f,0.07559999695569263877814591978676617145538330078125f,-0.995500010315328420773539619403891265392303466796875f,0.49370001168102017885530585772357881069183349609375f,0.3954999954700468833834747783839702606201171875f,-0.7669000414706770296646709539345465600490570068359375f,-0.10200000143796206231172618572600185871124267578125f,-0.558299991582333898776369096594862639904022216796875f,-0.49810000187307590380214605829678475856781005859375f,-0.08790000922083862633371609263122081756591796875f,1.0799000065237278622731764698983170092105865478515625f,1.49550004661828328522688025259412825107574462890625f,-0.34610001302659532118610741235897876322269439697265625f,0.68699999804794753544001650880090892314910888671875f,1.0531000099197032060516221463331021368503570556640625f,0.357799988102912802645505507825873792171478271484375f,-0.5353000500172380160535112736397422850131988525390625f,-0.81570002969652433133518343311152420938014984130859375f,0.81320002194792040395299181909649632871150970458984375f,-0.129900036102533977810935539309866726398468017578125f,-0.799900007604062324162441655062139034271240234375f,-0.161199987131357136416198727602022700011730194091796875f,-0.82620003593266044816800786065869033336639404296875f,0.5673000164374712994685978628695011138916015625f,0.66019999458938849468125908970250748097896575927734375f,1.325400007805228153046073202858678996562957763671875f,-0.3138999999746676650147492182441055774688720703125f,-0.4284999998062850590230254965717904269695281982421875f,0.7518000404879454645623582109692506492137908935546875f,0.0258999736592169238491578653338365256786346435546875f,-1.0684000112473961419112811199738644063472747802734375f,0.04410002496242526603964506648480892181396484375f,0.5261999722301953230640947367646731436252593994140625f,1.0930000105947252908578093411051668226718902587890625f,-0.352200001801550310887023442774079740047454833984375f,1.925500017300248334350953882676549255847930908203125f,1.7544000537082553581313959512044675648212432861328125f,0.280700002576411034027614732622168958187103271484375f,0.127299979259073214876707424991764128208160400390625f,-0.2118000012978911161809492114116437733173370361328125f,0.632399996912479167576748295687139034271240234375f,-0.683499998502432593028288465575315058231353759765625f,-0.36860000561326788925953223952092230319976806640625f,0.1238999557927245032118435119627974927425384521484375f,-0.0442999763384455125247995965764857828617095947265625f,0.049200010503828384145208474365063011646270751953125f,1.27560004840195251318846203503198921680450439453125f,1.49670004716068572037102057947777211666107177734375f,0.6697000135362147776874053306528367102146148681640625f,0.5324000431060795790472184307873249053955078125f,0.06610001120120312378958260524086654186248779296875f,1.1374000263050201287029494778835214674472808837890625f,-0.7515000153183937214862453402020037174224853515625f,-2.00550002717226760751145775429904460906982421875f,-0.1694000151515007157598802223219536244869232177734375f,1.2523000057086346448187441637855954468250274658203125f,-0.62470005987882648579301303470856510102748870849609375f,0.15109997428357557769373897826881147921085357666015625f,0.10280001373291014488131622783839702606201171875f,0.70400003092736029852716228560893796384334564208984375f,0.50159997988641225052930394667782820761203765869140625f,0.9474000248149041514267310049035586416721343994140625f,0.470099983045458191810439529945142567157745361328125f,-0.184200016352534579056055008550174534320831298828125f,1.17260002175122490797321006539277732372283935546875f,0.2895000125765800902399860206060111522674560546875f,0.13309999617338152955170471614110283553600311279296875f,0.1106999837636948313956963829696178436279296875f,0.505400013840198614190057924133725464344024658203125f,1.243100011372565649736543491599150002002716064453125f,1.513800074857474253775535544264130294322967529296875f,-0.5423000079438085041516615092405118048191070556640625f,-0.0442000172093515164561949859489686787128448486328125f,-0.8383000091284511512412791489623486995697021484375f,0.5208000192239878867184188493411056697368621826171875f,0.643199989537894367686021723784506320953369140625f,0.4494000035285949934404925443232059478759765625f,-1.120000063627958564183018097537569701671600341796875f,-0.189900016433000917714934985269792377948760986328125f,-0.112299956423043578723763857851736247539520263671875f,-1.5705000489279630215833094553090631961822509765625f,0.664100037144125021626450688927434384822845458984375f,1.030299994026124377199948867200873792171478271484375f,-0.0705999734118574195917972247116267681121826171875f,-0.510300032955408600088276216411031782627105712890625f,0.55820001989901069094912600121460855007171630859375f,0.458300025147199807662445891764946281909942626953125f,0.9498000051125881082469959437730722129344940185546875f,-0.9361999823629851835704585027997381985187530517578125f,-0.256599938610195810184677611687220633029937744140625f,0.093899999341368722838296889676712453365325927734375f,-0.367000009521841974446942913345992565155029296875f,0.2305000034421684773633387521840631961822509765625f,1.0401000065892933577060830430127680301666259765625f,-0.6362999988615509749934062710963189601898193359375f,-0.455099997350573826082609230070374906063079833984375f,-0.21810003227144514337254577185376547276973724365234375f,-0.5834000179901723814879233032115735113620758056640625f,0.311599989534914190159042846062220633029937744140625f,1.579500023193657565201419856748543679714202880859375f,0.6411999913409351936621760614798404276371002197265625f,0.87659999951869205236931748004280962049961090087890625f,0.6060000217109919873337275930680334568023681640625f,1.080300038468838152994067058898508548736572265625f,-0.3577999766290183014660897242720238864421844482421875f,-0.5048999770939348596954232561984099447727203369140625f,-0.0037999874807892464190217651776038110256195068359375f,-0.6170000362135470428626149441697634756565093994140625f,-0.0327000066220757279467079570167697966098785400390625f,-0.59800004125386518882834252508473582565784454345703125f,0.2797999829471107258171969078830443322658538818359375f,1.039200037959218558825114087085239589214324951171875f,-0.6902999863706524852347001797170378267765045166015625f,-0.6612999854505059449394366311025805771350860595703125f,0.42480002228617674830957184894941747188568115234375f,0.9460000390335920883444487117230892181396484375f,1.080200066468865127689014116185717284679412841796875f,-1.279700034211576298304180454579181969165802001953125f,-1.787500048764050664118485656217671930789947509765625f,-0.42989999501258113756563261631526984274387359619140625f,-0.15439998687654732378859989694319665431976318359375f,0.7126999912738796183475642465054988861083984375f,0.58630002128779901138244667890830896794795989990234375f,-0.95029998914599378423417874728329479694366455078125f,-0.6883000192239887571332701554638333618640899658203125f,-0.2995999917104839038728414379875175654888153076171875f,-1.077799996447563746215791979921050369739532470703125f,-0.2804000055328013107924789437674917280673980712890625f,-0.88959999659061406163118590484373271465301513671875f,0.7432000243321061905277247205958701670169830322265625f,0.446300010335445307418922311626374721527099609375f,1.4048000221557914724002102957456372678279876708984375f,0.99260004581660055134051390268723480403423309326171875f,-0.57069999933540760395089819212444126605987548828125f,-1.6270000313147907622379761960473842918872833251953125f,-0.9104999724104996250417798364651389420032501220703125f,-0.5852999937653537987358731697895564138889312744140625f,-1.2423000222116706847685918546631000936031341552734375f,0.28860000255852913397802694817073643207550048828125f,0.61939999816417667943824199028313159942626953125f,0.01859997927546508389440305109019391238689422607421875f,0.173900006121396888403296543401665985584259033203125f,0.2233000074893236597262102804961614310741424560546875f,-0.7006000533007090691484108901931904256343841552734375f,1.1250999953761693372911167898564599454402923583984375f,0.28179994325786805742239948813221417367458343505859375f,-1.3442000490233312337551296877791173756122589111328125f,-1.05610000457018582409318696591071784496307373046875f,0.218100022213161270911996325594373047351837158203125f,-0.6110000293105841695506796895642764866352081298828125f,0.8359000070005653260096778467413969337940216064453125f,-0.70390001260340195887721392864477820694446563720703125f,-0.0063999960631135888178278037230484187602996826171875f,-1.5390000147968529997655195984407328069210052490234375f,-0.0925999780163164754043236825964413583278656005859375f,-1.45889999420046745370882490533404052257537841796875f,0.2020000121667979708917073367047123610973358154296875f,-0.670099998319148681957813096232712268829345703125f,-0.8652000115767120025367376001668162643909454345703125f,1.0290999932229514257642222219146788120269775390625f,-0.37610000825822342829241051731514744460582733154296875f,-0.558599998684227383449751869193278253078460693359375f,-0.51509999019801622210934510803781449794769287109375f,-0.4551999658301462137188764245365746319293975830078125f,-0.616400004668534062801654727081768214702606201171875f,0.4858000305861238654614453480462543666362762451171875f,-0.403999979406595599584761657752096652984619140625f,-0.4040999932229516478088271469459868967533111572265625f,1.0318000306531789345143579339492134749889373779296875f,0.691100009301305107101143221370875835418701171875f,-0.283900049651415020690592427854426205158233642578125f,-0.4683000224649911391594514498137868940830230712890625f,0.6289000279515983837796966327005065977573394775390625f,-0.284200014191866134893871276290155947208404541015625f,-0.0474000029474497264203591839759610593318939208984375f,-0.6329000276982785777590834186412394046783447265625f,-0.2431999940827491712980190641246736049652099609375f,0.2102999776348466465236697331420145928859710693359375f,0.46029997130185329456963927441393025219440460205078125f,-0.59500000469386560286721987722557969391345977783203125f,0.53560005279779498987835495427134446799755096435546875f,-0.6972000212103133254260001194779761135578155517578125f,0.03090003397166818377428398889605887234210968017578125f,0.8102999879911536584842224328895099461078643798828125f,-0.65560003081858209128540693200193345546722412109375f,0.2584000289052730270356050823465920984745025634765625f,0.2089000176697968225170143341529183089733123779296875f,0.4636000195644800481176162065821699798107147216796875f,0.2805000006854536653833065429353155195713043212890625f,1.4749999921768901511853755437186919152736663818359375f,0.155299998086691370957623803406022489070892333984375f,0.41070001557767354949390892215888015925884246826171875f,0.6505000091791155458764706054353155195713043212890625f,-0.193900023555755751658580265939235687255859375f,0.4667000023081897541743501278688199818134307861328125f,-0.8062000194296239374835977287148125469684600830078125f,-0.4890000387877233567479606790584512054920196533203125f,0.023499984458089262506774730354663915932178497314453125f,-0.004100011551380067231775683467276394367218017578125f,-0.223200023847818673772280817502178251743316650390625f,0.264900021424889720034201445741928182542324066162109375f,-0.5593000129207970072542366324341855943202972412109375f,0.0410000074803829051006687222979962825775146484375f,-1.2691000543177128889738014549948275089263916015625f,0.7681000410631302344199866638518869876861572265625f,-0.4252000253230339144039362508920021355152130126953125f,-0.77640003331601636915593189769424498081207275390625f,0.534600040586293179245558349066413938999176025390625f,-0.41540001980066332265550954616628587245941162109375f,-0.49910003602653763010721377213485538959503173828125f,-0.12300001074373734244460365516715683043003082275390625f,-0.09090000275373455451699555851519107818603515625f,-0.827600014933943750605749301030300557613372802734375f,-0.128500003419816710703571516205556690692901611328125f,-0.8945000208467248636878821343998424708843231201171875f,-0.28029996400028422964822993890265934169292449951171875f,-0.9565999887526033251816670599509961903095245361328125f,-0.8719999946951861868882360795396380126476287841796875f,1.7323999978996809101516873852233402431011199951171875f,1.07070000812709320570093041169457137584686279296875f,0.404800008539855316058719836291857063770294189453125f,0.9561000211477279631111514390795491635799407958984375f,0.94709999446719894677926276926882565021514892578125f,1.1132999935567375882072838066960684955120086669921875f,-0.032500017508864498250176211513462476432323455810546875f,-0.39990000242590861834202087266021408140659332275390625f,-0.903400014190375966194324064417742192745208740234375f,-0.45970001223236345300193761431728489696979522705078125f,-0.5365999831274146369963773395284079015254974365234375f,-0.709700005042552817258183495141565799713134765625f,0.482100057983398944116970596951432526111602783203125f,0.10060001185685407687486758732120506465435028076171875f,-1.4798999945655457732840432072407566010951995849609375f,0.09060004344731620307129560387693345546722412109375f,-0.70580008072853173128891057785949669778347015380859375f,1.3154000308275224373488754281424917280673980712890625f,-0.8646000337690120662870185697101987898349761962890625f,0.6380000300034882609878650328028015792369842529296875f,-0.9115000102519985603066743351519107818603515625f,-1.8462000479653479789732273275149054825305938720703125f,0.6183000121459365505671712526236660778522491455078125f,1.5351999777883282849444412931916303932666778564453125f,0.08159999039173138069003243799670599400997161865234375f,-0.45850003724545285166414032573811709880828857421875f,-0.1643000154539946411347273169667460024356842041015625f,-0.91319999713748689629255750332958996295928955078125f,-0.312899998883157837159529890413978137075901031494140625f,0.622499991357326365459812222979962825775146484375f,0.8760999890729783690090926029370166361331939697265625f,0.286399991369247430839095613919198513031005859375f,0.02529996064752315643175961668021045625209808349609375f,0.5117999907553192340259329284890554845333099365234375f,1.4385000183954834707122927284217439591884613037109375f,-0.12419998427778455862835471634753048419952392578125f,-0.0317999685153360633904640053515322506427764892578125f,0.51690002185702332848649120933259837329387664794921875f,-0.01149998909234994926009676419198513031005859375f,-0.58780001086443689484184460525284521281719207763671875f,-0.925900020746886820433019238407723605632781982421875f,0.01809998291134806702729065364110283553600311279296875f,-0.6074000488430264521610979500110261142253875732421875f,0.7521000179737804547386303966050036251544952392578125f,-0.9217999998450270293659514209139160811901092529296875f,0.1234000300601127975141935166902840137481689453125f,0.953500039704144430885435212985612452030181884765625f,1.21179999954700434727783431299030780792236328125f,-0.0476999714925880891058795896242372691631317138671875f,-0.5450000053644179498490984769887290894985198974609375f,-0.3226999942541122212702475735568441450595855712890625f,-1.001599988119303841216378714307211339473724365234375f,-1.2431000398337845869178863722481764853000640869140625f,0.021499956235289108263941670884378254413604736328125f,1.1865000285059219553573939265334047377109527587890625f,-0.150700014385581226861177128739655017852783203125f,-0.541000010572374012696172940195538103580474853515625f,-1.24739999002069179567797618801705539226531982421875f,-1.1373999858483667679820428020320832729339599609375f,0.625399995772540240324133264948613941669464111328125f,1.528000035479665452697872751741670072078704833984375f,1.54060004586875454180017186445184051990509033203125f,0.22520001705288894555678780307061970233917236328125f,0.02020001329034626369463012451888062059879302978515625f,-0.7666000082731245779399387174635194242000579833984375f,-0.1218999992072582472246722318232059478759765625f,-0.13009998684525425627356298718950711190700531005859375f,-0.474199991542100907082613048260100185871124267578125f,-0.2080999476328482256093366231652908027172088623046875f,-0.39569998320490118448589100808021612465381622314453125f,0.5271000265792016303834088830626569688320159912109375f,0.8136000195458528150282973001594655215740203857421875f,-0.18749998450279203865420640795491635799407958984375f,-0.61650001442432422749106990522705018520355224609375f,-0.593900025641918460905799292959272861480712890625f,-0.33550002851337235032502803733223117887973785400390625f,1.601100029604137109373596103978343307971954345703125f,0.426899974955617711458444318850524723529815673828125f,0.56199999044835546868625897332094609737396240234375f,-1.2969999878406515136930465814657509326934814453125f,-0.7633000162810088529141694380086846649646759033203125f,0.892500047851354505468179922900162637233734130859375f,0.348800012533366621259034445756697095930576324462890625f,1.01870000426769280466032796539366245269775390625f,0.88759998449832178124552228837274014949798583984375f,0.09879999983012599518161778178182430565357208251953125f,-1.074699988614022050370522265438921749591827392578125f,-0.6652999937839805255634928471408784389495849609375f,-0.3953000076979396038012737335520796477794647216796875f,-0.90800002110004474076276892446912825107574462890625f,0.036800023221224886782465546275489032268524169921875f,0.2021000063136220159520917150075547397136688232421875f,-0.54180001496970664476293677580542862415313720703125f,-0.96980004217922743947610797476954758167266845703125f,-0.3779000158518550644970446228398941457271575927734375f,0.029099961781501093582846806384623050689697265625f,-1.2842000323712825515798385822563432157039642333984375f,0.23810004627853686987037917788256891071796417236328125f,0.6472000150077044100527245973353274166584014892578125f,0.5506000419571999149326302358531393110752105712890625f,-0.37659998081773482514478246230282820761203765869140625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.355999984383583267799622262828052043914794921875f,0.5148000173687929947874408753705210983753204345703125f,0.512199968348443945842518587596714496612548828125f,0.69950001205503919976536053582094609737396240234375f,-0.4373999561950567827040003976435400545597076416015625f,-0.8524000130057327595523020136170089244842529296875f,-0.7365000149458651446110479810158722102642059326171875f,-0.4964000400587920136530328818480484187602996826171875f,0.045400005197524961886301753111183643341064453125f,0.3028000000238417754871989018283784389495849609375f,0.902900017565489321214045048691332340240478515625f,-0.335200066450239031468072425923310220241546630859375f,1.0465000100284811423989594914019107818603515625f,0.0227000103101133721139603949268348515033721923828125f,-1.7665000194907189978721362422220408916473388671875f,-0.828600033366679866730919457040727138519287109375f,-0.182199982690810724506036422098986804485321044921875f,0.0720999801248307647938418085686862468719482421875f,-0.60599999570846563301529386080801486968994140625f,-0.06890007541179699046551831997931003570556640625f,-0.4731999676704405910498962839483283460140228271484375f,-0.1931999922201039776581410478684119880199432373046875f,-0.1109000119552019381075069759390316903591156005859375f,-1.254900003319978196714146179147064685821533203125f,1.47920001180767979320762606221251189708709716796875f,0.94389998898506188851342812995426356792449951171875f,-0.45569998599886929468993912450969219207763671875f,-0.3659999939203262275810857317992486059665679931640625f,-0.51609999890774493902512176646268926560878753662109375f,-0.457400017494708155396665461012162268161773681640625f,0.04010002216100705396684134029783308506011962890625f,-0.856100021743773975657632036018185317516326904296875f,1.1314999699443575309487641789019107818603515625f,1.485000055432319499004734097979962825775146484375f,1.8445000149607668760154410847462713718414306640625f,0.7968000207066534557043269160203635692596435546875f,-0.6867999840497969188390925410203635692596435546875f,0.375899994036555273879685046267695724964141845703125f,0.1575999890804291592161234802915714681148529052734375f,1.1450000494718555188455866300500929355621337890625f,1.1465999999284743893213089904747903347015380859375f,0.75439996865987790641838728333823382854461669921875f,0.15579996673464702183764529763720929622650146484375f,0.925900034679472128118504770100116729736328125f,0.46490002118274598164049393744789995253086090087890625f,0.1454000105619428495629108510911464691162109375f,1.27820004038810708379969582892954349517822265625f,0.03400000578165052189660855219699442386627197265625f,-1.16150000175833678639492063666693866252899169921875f,0.367699943590164224360705702565610408782958984375f,1.893000031530856830386255751363933086395263671875f,1.3225000202655792236328125f,-1.02649999788403523126589789171703159809112548828125f,-0.05410001795887975362120414501987397670745849609375f,0.29600000554323191437333662179298698902130126953125f,-0.01040000236630422847383670159615576267242431640625f,0.120200008261203805659533827565610408782958984375f,-2.24719999207854304046350080170668661594390869140625f,0.54519999991655421212044529966078698635101318359375f,0.68769999619126309653438511304557323455810546875f,-0.04919998707175299301752602332271635532379150390625f,0.8957000380039215059468915569595992565155029296875f,-0.9479999921917912075741696753539144992828369140625f,0.8369999896287918517145953956060111522674560546875f,0.025600014203786880528923575184307992458343505859375f,0.343499998912215165347561196540482342243194580078125f,-0.4694000048696995719410551828332245349884033203125f,-1.5361000042349102212568823233596049249172210693359375f,0.6697999808236956287288421663106419146060943603515625f,-0.2765999940052628058850814340985380113124847412109375f,0.151299963843822382614234811626374721527099609375f,-0.5665999686360354559155894094146788120269775390625f,-0.550199978160858194087268202565610408782958984375f,-1.16220000236034337603996391408145427703857421875f,-0.4748000149846074435799891944043338298797607421875f,-1.54809997620582606714378925971686840057373046875f,-0.917700018095969571874093162477947771549224853515625f,0.15549995829164942051647813059389591217041015625f,0.046900010681152792813009000383317470550537109375f,-0.20010000398159011325560641125775873661041259765625f,1.17840001690983786630795293604023754596710205078125f,0.5279999901056289246525921043939888477325439453125f,-0.367099983030557464047660687356255948543548583984375f,0.147000000134110297977940717828460037708282470703125f,0.40550001513958022769656963646411895751953125f,-0.98920004265308403290646310779266059398651123046875f,-0.757900000429153397618620147113688290119171142578125f,0.285400013020634890636983982403762638568878173828125f,0.1065999747455119717187699279747903347015380859375f,1.121800040599703951471610707812942564487457275390625f,-1.378400000667571845269776531495153903961181640625f,-0.4200000330805782056131647550500929355621337890625f,-1.27010004227757544725818661390803754329681396484375f,-0.44510000755786904846900142729282379150390625f,-0.52290004945397416946661905967630445957183837890625f,-0.7806999688625335664937665569595992565155029296875f,-0.26630001510381617890743655152618885040283203125f,0.717899956917761983277159743010997772216796875f,0.5552000163078307792829946265555918216705322265625f,-0.2248000050753355960608814712031744420528411865234375f,0.9827999821797013257906883154646493494510650634765625f,-0.050200015413761178706408827565610408782958984375f,0.33970000101923947255500024766661226749420166015625f,2.36699997860193178667032043449580669403076171875f,0.69599999958276814737700988189317286014556884765625f,1.099999969974159608199215654167346656322479248046875f,0.1058999893426886274028220213949680328369140625f,0.54470000951290131752102752216160297393798828125f,-0.732599990570545145374126150272786617279052734375f,-0.29479998652338945674955539288930594921112060546875f,0.6553000021100050531686065369285643100738525390625f,-0.86989998171329574461196898482739925384521484375f,1.116800033521652579793226323090493679046630859375f,1.1881000161409378534926872816868126392364501953125f,-0.339900044596195272106342599727213382720947265625f,0.953199996876715971438898122869431972503662109375f,-0.81380000646114325490998453460633754730224609375f,0.981399961566925327360877417959272861480712890625f,0.055000019520520027072052471339702606201171875f,-0.225899957379697813308894183137454092502593994140625f,0.56389996871948167012078556581400334835052490234375f,-0.5747000102579591374052370156277902424335479736328125f,-0.964399999766051951866074887220747768878936767578125f,-0.538500002637505303937359713017940521240234375f,-0.42799996954202601529004823532886803150177001953125f,0.09250000193715113283587925252504646778106689453125f,1.1563999681234360394910254399292171001434326171875f,0.67750004619359938118350328295491635799407958984375f,-0.69159996908307064256149487846414558589458465576171875f,-0.490800036267190709082797184237278997898101806640625f,0.5354000018447633291174270198098383843898773193359375f,-0.2838000000908971198754215947701595723628997802734375f,0.7415999810785052215322821211884729564189910888671875f,-1.5415000186711547147666578894131816923618316650390625f,-0.69800004253909053719695521067478694021701812744140625f,0.030999996602535162537606083787977695465087890625f,0.14899996361136391698209990863688290119171142578125f,-0.0129999579191206748873810283839702606201171875f,1.85359997286796573945366617408581078052520751953125f,1.09760002916455245980387189774774014949798583984375f,-1.17970002515912053553392979665659368038177490234375f,-1.01310003834366835206992618623189628124237060546875f,0.015299997341632565195368442800827324390411376953125f,1.316600036585331423566458397544920444488525390625f,0.5993999871373176802080706693232059478759765625f,0.48000001855194529554182736319489777088165283203125f,1.1332000389724974365179832602734677493572235107421875f,-0.99179997758641824656677954408223740756511688232421875f,0.01989998469352638466034477460198104381561279296875f,-0.87520000900626104822777051595039665699005126953125f,-1.784299989265203834065687260590493679046630859375f,-0.3658000226140014632392194471322000026702880859375f,1.0849000144958491631541619426570832729339599609375f,-0.9041999663591386848793263197876513004302978515625f,-0.385700002539157704717354135937057435512542724609375f,-0.9429000064641233080919846543110907077789306640625f,0.81860002323389124967434327118098735809326171875f,1.168899990028142266140775973326526582241058349609375f,0.332099981093406437793191798846237361431121826171875f,0.7804000187948350042432821283000521361827850341796875f,-1.23220001204609896916508660069666802883148193359375f,0.56130000944137581342374687665142118930816650390625f,-0.3883999571561815855602617375552654266357421875f,-0.59170001761913226800970733165740966796875f,-0.045800019335747066406838712282478809356689453125f,0.01560001524686871476887972676195204257965087890625f,-0.78950000438094125598809114308096468448638916015625f,0.1854999373555177299977003713138401508331298828125f,-1.1053999870181083764464347041212022304534912109375f,1.40660000708103183342245756648480892181396484375f,-0.1404999981522561114388736314140260219573974609375f,0.0428000238656995879438227348146028816699981689453125f,-0.84520000263601513523781250114552676677703857421875f,0.783900010293721916099229929386638104915618896484375f,0.4780000039637088971034017959027551114559173583984375f,0.4456999922946096592824005711008794605731964111328125f,0.01760003497600592936578323133289813995361328125f,-0.5868000436544422715456903461017645895481109619140625f,0.995900004728138554810357163660228252410888671875f,0.975900016667693659400129035930149257183074951171875f,0.540600002431869341990022803656756877899169921875f,0.56590003315210424261749722063541412353515625f,1.963400016462802710748292156495153903961181640625f,-0.4142000059962267499713561846874654293060302734375f,-1.857300083716214356144291741657070815563201904296875f,-0.21239994455352440017037451980286277830600738525390625f,-0.97199999707937223547560279257595539093017578125f,0.20059995802640884221546002663671970367431640625f,0.132499992847442626953125f,-0.379199973213673047212068922817707061767578125f,-0.787400028353929304358871377189643681049346923828125f,0.635600055405497688099103470449335873126983642578125f,-0.350100007036328264575786306522786617279052734375f,0.1490000285059214224503421064582653343677520751953125f,1.273200014944374292014117600047029554843902587890625f,0.866399992170185040407659471384249627590179443359375f,-0.19309999289512713716021607979200780391693115234375f,0.37320004202723566777422092854976654052734375f,0.3156000119686126481610699556767940521240234375f,0.12929998077154181146397604607045650482177734375f,0.32470001502633127898889142670668661594390869140625f,-0.2113999830245971711661923109204508364200592041015625f,-2.02750001620501318910783083993010222911834716796875f,-0.155799944829940528734368854202330112457275390625f,-1.0365000311136238764220252051018178462982177734375f,0.30899998730421085468833553022705018520355224609375f,0.8107000038802620878186644404195249080657958984375f,0.50300001245737036015270859934389591217041015625f,0.53870005895495420844554246286861598491668701171875f,-1.242299978178738939504910376854240894317626953125f,-0.719200016948580600484319802490063011646270751953125f,-1.23149998745322175608407633262686431407928466796875f,-1.68760003333687791382544673979282379150390625f,-0.193500050619245467231621660175733268260955810546875f,-0.79089996952414498565531175700016319751739501953125f,0.1992000089764598413921703468076884746551513671875f,0.36759996941089667643609573133289813995361328125f,1.49449998277425688542052739649079740047454833984375f,0.336600051635503660918402601964771747589111328125f,-0.95289997867345821447315756813623011112213134765625f,-0.3424000223934644537848726031370460987091064453125f,-1.1652000046849249059732755995355546474456787109375f,0.89809995653629215439650579355657100677490234375f,-0.249399952864646223815725534223020076751708984375f,-0.067299992036819134000324993394315242767333984375f,-0.9943999999523160937542343162931501865386962890625f,-0.1188000294089324171409316477365791797637939453125f,0.7887000111967328308537616976536810398101806640625f,-0.4200000119581817070724127916037105023860931396484375f,-0.72490004325509094229573747725225985050201416015625f,-0.062199967193603100668042316101491451263427734375f,0.5854999911487102082219280418939888477325439453125f,-0.514999989867210405947162144002504646778106689453125f,-0.356500024855137098711566068232059478759765625f,0.3104000004291533887368359501124359667301177978515625f,1.106800001744926209568120611947961151599884033203125f,-0.382999992340803174784014117904007434844970703125f,0.548699995326995093591904151253402233123779296875f,1.724500049829483572239041677676141262054443359375f,0.372499999105929902043499168939888477325439453125f,-0.86600000777840602239621148328296840190887451171875f,1.2801000322937967634828737573116086423397064208984375f,1.00150002623349454466961105936206877231597900390625f,0.461799995374679639326132019050419330596923828125f,-0.5083000199317933009979242342524230480194091796875f,-0.0522000631570822548610522062517702579498291015625f,0.8002999992787831473606274812482297420501708984375f,1.34619998415112451795039305579848587512969970703125f,-0.8075999628543844011119290371425449848175048828125f,0.5865999498605720674504482303746044635772705078125f,0.30309999736547421633758858661167323589324951171875f,-0.32170000346302973781575929024256765842437744140625f,0.407799984414875638094599707983434200286865234375f,0.21240000747367704381218800335773266851902008056640625f,0.03050002259016082462039776146411895751953125f,0.422900011605024328531499122618697583675384521484375f,-0.97289999141395089310435650986619293689727783203125f,-1.3440000225454566962213220904232002794742584228515625f,0.9530999931186439066976845424505881965160369873046875f,-0.22209999463483731663160369862453080713748931884765625f,-1.06820000596642472601160989142954349517822265625f,0.67180000789165461583252181299030780792236328125f,0.70370002389997299285795406831311993300914764404296875f,-0.00859999985769388519685207938891835510730743408203125f,-0.245300003898143881286841860855929553508758544921875f,0.970000005885958582751982248737476766109466552734375f,-0.384399979686736781303579846280626952648162841796875f,-0.049499962583184231590394119848497211933135986328125f,0.4302999550223347569044562987983226776123046875f,1.426500047504902113360003568232059478759765625f,-0.110599994534254175704290901194326579570770263671875f,0.359399946382641832798299219575710594654083251953125f,1.286000021934508907861527404747903347015380859375f,2.07870001969039464029265218414366245269775390625f,-0.0897999775454396687024427592405118048191070556640625f,0.3971000037431711149338298127986490726470947265625f,1.0486000256180769696356946951709687709808349609375f,0.6800999893784520633488455132464878261089324951171875f,-0.765300002966821324434931739233434200286865234375f,-0.0319999884366988229800199405872263014316558837890625f,-1.47490003308504835644043851061724126338958740234375f,-0.8689999984055756243606083444319665431976318359375f,-0.7739000163659450759467972602578811347484588623046875f,-0.413500013962388113242241161060519516468048095703125f,-0.718000001728534176237417341326363384723663330078125f,-1.909200039598346165803377516567707061767578125f,1.7511999783396721142025853623636066913604736328125f,-0.15090002689361625698438729159533977508544921875f,-0.103999989986419905108050443232059478759765625f,0.36270005505085001207277173307375051081180572509765625f,0.382399954276532394459309216472320258617401123046875f,0.551400007164478012100516934879124164581298828125f,-0.8765000370740896329380120732821524143218994140625f,-1.827600056433677622180766775272786617279052734375f,0.519199986624716558480940875597298145294189453125f,-0.999300012362002831878271535970270633697509765625f,-0.25200001183152220818328714813105762004852294921875f,-0.15879999096393593305265312665142118930816650390625f,0.7370999877989294901681205374188721179962158203125f,0.64370007966756848105660537839867174625396728515625f,0.7412000208079820851025942829437553882598876953125f,-0.16920005457401288140317774377763271331787109375f,-0.494899962043761743046843548654578626155853271484375f,-0.8255000483691699031396638019941747188568115234375f,-0.4673000110730527456581739897956140339374542236328125f,0.2121000082135200415223152958787977695465087890625f,-0.14040003798007916202550404705107212066650390625f,1.80650004452466905746632619411684572696685791015625f,-0.195400037235021528658762690611183643341064453125f,1.825900024658441367364503093995153903961181640625f,0.925500037193298652482553734444081783294677734375f,0.111900038099289389492696500383317470550537109375f,-0.930500005453824829970699283876456320285797119140625f,0.64140003540217860322769638514728285372257232666015625f,-1.169700007519870244010462556616403162479400634765625f,-0.4390000042915342959304325631819665431976318359375f,0.2451999733924861146761031704954802989959716796875f,-0.189899979031086019176655099727213382720947265625f,-0.009600031757354798855885746888816356658935546875f,1.38570004158020054774169693700969219207763671875f,0.364800028693675670865559368394315242767333984375f,0.1790999970972535493274335749447345733642578125f,1.1174000459373001348239995422773063182830810546875f,1.177500035464763783465969027020037174224853515625f,0.07479999561309824684940394945442676544189453125f,-1.0902000130295750324194159475155174732208251953125f,-0.55010000427961358582251705229282379150390625f,-1.0167000855684289462033120798878371715545654296875f,1.3011000202536582293078026850707828998565673828125f,-0.988499983936547810259298785240389406681060791015625f,-0.546500001683831282406345053459517657756805419921875f,-1.74739997619986553445414756424725055694580078125f,0.15650000667572072643451974727213382720947265625f,-0.1080000014305113609225372783839702606201171875f,1.2850000500679019665994928800500929355621337890625f,-0.8483000130772584412852665991522371768951416015625f,-0.2552000329971324532607468427158892154693603515625f,1.2842000316262254244747964548878371715545654296875f,-1.7175000026822093701639460050500929355621337890625f,0.3549999642372139163626343361102044582366943359375f,1.2414000071644775147206019028089940547943115234375f,0.900400034999847775907255709171295166015625f,-0.1472000113010405453195517111453227698802947998046875f,-0.55519997104555418587779058725573122501373291015625f,0.04679999150037783550715175806544721126556396484375f,0.5400999709010125826580406283028423786163330078125f,-0.9340999747455125401529585360549390316009521484375f,0.4507999988466500074224541094736196100711822509765625f,0.7216999979868534698113080594339407980442047119140625f,-1.0659999963045123649862944148480892181396484375f,-0.5493000147461888360567172640003263950347900390625f,0.60619998728036872392976874834857881069183349609375f,-1.19249999776482606961280907853506505489349365234375f,-0.05080001814365431300757336430251598358154296875f,-0.5873999969124792386310218716971576213836669921875f,0.29410002496242526603964506648480892181396484375f,-0.339300023090839619754888190072961151599884033203125f,-0.5601999720513821667822185190743766725063323974609375f,-1.3604000003546470320969774547847919166088104248046875f,-0.48780002163052582631053155637346208095550537109375f,-0.3004000171184539880187003291212022304534912109375f,-0.1236000107169150652453026850707828998565673828125f,-0.9873999876737595826625693007372319698333740234375f,-0.32220000526607039947890598341473378241062164306640625f,0.5844999985694887545406572826323099434375762939453125f,-0.1453000139564277759518517996184527873992919921875f,0.5925000451132660383990469199488870799541473388671875f,-0.39379998567402296760775470829685218632221221923828125f,-0.465000046696514512944986563525162637233734130859375f,0.28519998937398194360781644718372263014316558837890625f,-0.66709999351724957872278309878311119973659515380859375f,-0.876900002485513585526177848805673420429229736328125f,-1.266200052137673282004470820538699626922607421875f,-0.55300002544000859128203728687367402017116546630859375f,2.34680003113746682430473811109550297260284423828125f,0.5434000496923925282999334740452468395233154296875f,0.037799960684775868458018521778285503387451171875f,-0.9988000174880031778457123436965048313140869140625f,-1.0307999799966811593776583322323858737945556640625f,-0.17700003135204323001516968361102044582366943359375f,-0.26659999590516125778094647102989256381988525390625f,-1.3632999364852906865053228102624416351318359375f,-0.2402000413417813007299628225155174732208251953125f,0.309600066030025544705495121888816356658935546875f,1.305999980211257849305184208787977695465087890625f,-0.2324999883770946240701960050500929355621337890625f,-0.6228000223755838504757775808684527873992919921875f,-0.6051999871015549814501355285756289958953857421875f,-0.51760001064985983187938245464465580880641937255859375f,0.50789999724403023240171251018182374536991119384765625f,1.706099985682964170763398215058259665966033935546875f,0.091800013032556027070540949353016912937164306640625f,-0.5606999992609029703771739150397479534149169921875f,1.2668999859452245715374374412931501865386962890625f,0.452600048685074085597079829312860965728759765625f,-1.0667999980568882989473422640003263950347900390625f,0.2036000014781951250597558100707828998565673828125f,0.3056000215053558122235699556767940521240234375f,-0.67230000291466662787343011586926877498626708984375f,0.3622000021740792163882360910065472126007080078125f,-0.46099999345466502465029634549864567816257476806640625f,-1.4263000195741657449843842186965048313140869140625f,1.4221000082135208941735982079990208148956298828125f,1.1959000343441967828539418405853211879730224609375f,0.118199973630904509036554372869431972503662109375f,0.45359998598098749678086960557266138494014739990234375f,1.114500027496367895452067386941052973270416259765625f,-1.176700015234947382936070425785146653652191162109375f,-0.01489998476803311433513954398222267627716064453125f,-0.29650002336502012667551753111183643341064453125f,0.661800021004676608527006465010344982147216796875f,0.342599953019618652660938096232712268829345703125f,-0.174499978601932781430150498636066913604736328125f,-0.504599984669685142080197692848742008209228515625f,0.2277999892950060001339807058684527873992919921875f,0.277000056684017170738343338598497211933135986328125f,-1.088799985972046524551615220843814313411712646484375f,-0.029399992278218434194059227593243122100830078125f,-0.66040000489950134721084396005608141422271728515625f,-1.061000001221894972758263975265435874462127685546875f,-0.1805000544041401866479645832441747188568115234375f,-0.9984999770820135456261823492241092026233673095703125f,-0.589800034393370697927139190142042934894561767578125f,0.29299998489022271996873314492404460906982421875f,0.438900009474157304367736287531442940235137939453125f,-0.4475000149011609806848355219699442386627197265625f,1.0554999949112531254513669409789144992828369140625f,0.49670005060657917983490960978087969124317169189453125f,0.4933000266373159803379166987724602222442626953125f,0.6334999769330018626334322107140906155109405517578125f,1.12480003521293436818950794986449182033538818359375f,0.48400000786781305350814363919198513031005859375f,0.30300001007318488888131469138897955417633056640625f,-1.068199994641542360795938293449580669403076171875f,0.73550001469254500108263528090901672840118408203125f,0.442500008493662111419553184532560408115386962890625f,0.671599976161122658169233545777387917041778564453125f,-1.03190002826452342787888483144342899322509765625f,1.5398000225842007893106710980646312236785888671875f,0.04950000892579442535179623519070446491241455078125f,-0.0956999802619211692444878281094133853912353515625f,0.302800015372037822913853233330883085727691650390625f,-0.8175000263750551443564518194762058556079864501953125f,-0.0031999643921849507677279689232818782329559326171875f,-0.57690003524944233337379273507394827902317047119140625f,-1.39780003042221068199069122783839702606201171875f,-1.5558000086069103673480640281923115253448486328125f,0.4152999616533519233740889831096865236759185791015625f,-1.5181999834284185535437927683233283460140228271484375f,0.5740999742984769227405195124447345733642578125f,-0.270399976587295753915896057151257991790771484375f,-0.759500032246112510847524390555918216705322265625f,0.49420002521872508083333741524256765842437744140625f,-0.8863999905496833253693012011353857815265655517578125f,-0.5388000126078724694167476627626456320285797119140625f,0.004300014150143027791273198090493679046630859375f,0.5043000655591498571084230206906795501708984375f,0.3107999962389467185630564927123486995697021484375f,0.6571000287771227021949016489088535308837890625f,0.516799984347820373642434788052923977375030517578125f,1.21320002883970712304062544717453420162200927734375f,-2.1341000192999839413232621154747903347015380859375f,1.114900028204917958873920724727213382720947265625f,-0.425600004518031482803053222596645355224609375f,-1.1992000488370664346149396806140430271625518798828125f,0.3344000107929108178694832531618885695934295654296875f,-0.8980000243782999547192957834340631961822509765625f,-1.1045000069141384102522351895458996295928955078125f,0.61579999400377261054018163122236728668212890625f,-0.23449999544024446862522381707094609737396240234375f,1.201600023919343840361761976964771747589111328125f,0.733399982413649897949881051317788660526275634765625f,0.1808000031679861674405174198909662663936614990234375f,1.3393999805435539141029721577069722115993499755859375f,-0.0469999553188689134231026400811970233917236328125f,-1.26319998743310524247362991445697844028472900390625f,-0.3356000226974487077313824556767940521240234375f,0.0542000032395122399719866734812967479228973388671875f,-0.1645000227466228981398899122723378241062164306640625f,0.10410000499486926628378569148480892181396484375f,-0.1755999931931493396319865496479906141757965087890625f,0.6819999909326437403933596215210855007171630859375f,0.3067999907925724745183515551616437733173370361328125f,-0.172299982425570430422112622181884944438934326171875f,0.5017999736189846515799217741005122661590576171875f,-0.01120002110600548661523134796880185604095458984375f,-0.18920001642703976330039949971251189708709716796875f,-1.8958000352054842352345076506026089191436767578125f,0.7659999512657524878278536561992950737476348876953125f,1.1948999899089329801427084021270275115966796875f,-0.78969999625086817474084455170668661594390869140625f,1.484800054323674345368999638594686985015869140625f,-1.215700050520897690375932143069803714752197265625f,0.337000024795532482357884873636066913604736328125f,-0.5319000342249875501465794513933360576629638671875f,0.667800007772445525233706575818359851837158203125f,0.671200074005128044518642127513885498046875f,-1.19279996485710171327809803187847137451171875f,-1.0638999937534325823662584298290312290191650390625f,0.5328999911904332975609577260911464691162109375f,0.3091000531256202776830832590349018573760986328125f,0.596599977949261894849541931762360036373138427734375f,-0.2695000117942694117800783715210855007171630859375f,0.24019999945089198778447325821616686880588531494140625f,-0.0803999920845033244631849811412394046783447265625f,-1.42450001287460281673702411353588104248046875f,-0.1357999820828439396791509352624416351318359375f,-0.04440002081394123223390124621801078319549560546875f,-0.81639998794198032072699788841418921947479248046875f,-0.3471000427842145796830664039589464664459228515625f,-0.67590004000663750360899939551018178462982177734375f,-1.2831000180780893771270712022669613361358642578125f,-1.446400028622150557566783390939235687255859375f,0.1688000049710272065794924856163561344146728515625f,1.02750003799796107983866022550500929355621337890625f,1.3415000250041486662411216457257978618144989013671875f,0.12530004252940418751904871896840631961822509765625f,1.029300034713745759518133127130568027496337890625f,-0.49080003870725619208315038122236728668212890625f,0.7557999662131067442061294059385545551776885986328125f,-1.9826999954089516986499575068592093884944915771484375f,0.035699984955787300577867426909506320953369140625f,-0.38740000317096701110131107270717620849609375f,0.0667999572277064856962169869802892208099365234375f,0.6239000126034019988452428151504136621952056884765625f,0.2305999690905216059633175973431207239627838134765625f,0.38710000873506000829138429253362119197845458984375f,0.7536000176087023216808802317245863378047943115234375f,-0.3671999871611593135867224191315472126007080078125f,-1.04550005328655259972947533242404460906982421875f,1.0499000413179402357854996807873249053955078125f,0.0506000078707933287347486839280463755130767822265625f,0.5637999659672383589992250563227571547031402587890625f,0.53790003142356823673253529705107212066650390625f,-0.86779999853372569162957006483338773250579833984375f,-0.906000037580728445618660771287977695465087890625f,-1.0593000162363050975500300410203635692596435546875f,-0.3568000373959545612478905241005122661590576171875f,-0.38190006760358841120250872336328029632568359375f,-0.200699983763694689287149230949580669403076171875f,-0.262800002855062331263980013318359851837158203125f,-0.706400003662705611162664354196749627590179443359375f,-0.8928999929785721434427614440210163593292236328125f,-0.649100032860041142868112729047425091266632080078125f,-0.778399987629055711835235342732630670070648193359375f,-1.096000000625849235547093485365621745586395263671875f,-0.97410000715553746175601190770976245403289794921875f,0.635999982073902625501204965985380113124847412109375f,0.2458000279784204877842057612724602222442626953125f,0.54200001227855665320021216757595539093017578125f,-0.219299980771541669355428894050419330596923828125f,-2.10200000765919714496021697414107620716094970703125f,-0.677099961721896903554807067848742008209228515625f,-0.45270004767477445284384884871542453765869140625f,0.1120999861970541910949350494774989783763885498046875f,-0.25489995936155285249924418167211115360260009765625f,-1.3909000293895601441107601203839294612407684326171875f,0.48280001797974103627808517558150924742221832275390625f,-0.738799984407425114341094740666449069976806640625f,-0.6876999722003933612768378225155174732208251953125f,-0.62609997636973790946512963273562490940093994140625f,0.9595999859735366843693782357149757444858551025390625f,-0.92389998514801252582628876552917063236236572265625f,0.89900001097843063035952582140453159809112548828125f,0.5399000471293931013860856182873249053955078125f,0.01360000613480816245015603271895088255405426025390625f,0.89900002554431568402293351027765311300754547119140625f,0.083600006395578407847324342583306133747100830078125f,0.48590000878870498457473559028585441410541534423828125f,-0.306900006229430477588948633638210594654083251953125f,0.72160001691579811478050032746978104114532470703125f,0.2935999710798267159361785161308944225311279296875f,-1.18160000082254423858785230549983680248260498046875f,0.21300000198930479911041402374394237995147705078125f,0.09889996026307368293117860957863740622997283935546875f,1.028000011265277180427801795303821563720703125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.23489999959468832457787357270717620849609375f,0.6668999759614469002855230428394861519336700439453125f,1.181999980017542473120784052298404276371002197265625f,0.429100011402368419766162332962267100811004638671875f,1.0805999710276725789270813038456253707408905029296875f,0.8430999988555907265208588796667754650115966796875f,-1.471300003518163368454452211153693497180938720703125f,-2.13270002059191465804133258643560111522674560546875f,-0.418499992132186804383309208787977695465087890625f,-0.3344000071048736089096564683131873607635498046875f,-0.15019999604225109379740388249047100543975830078125f,-0.951000010684132757177167150075547397136688232421875f,-0.577799993467330796903524969820864498615264892578125f,-1.0719000241294496245103573528467677533626556396484375f,-0.5422000583887101043956135981716215610504150390625f,1.33089999053478180712772882543504238128662109375f,1.404400030350684946967021460295654833316802978515625f,1.12390002251267429045356038841418921947479248046875f,0.04449999439716378901721327565610408782958984375f,-0.95229997773170538266640505753457546234130859375f,-0.6965999734044074909178334564785473048686981201171875f,0.59809998205453229758177258190698921680450439453125f,0.5078999617233870278454332947148941457271575927734375f,1.14419995622634917964433043380267918109893798828125f,0.27900003454089183918540584272705018520355224609375f,-1.29659995999336263849954775650985538959503173828125f,-1.0407999779105185922389864572323858737945556640625f,0.2965000048875807703296914041857235133647918701171875f,-0.40730001565515949746298929312615655362606048583984375f,-0.4362000291153791398102157472749240696430206298828125f,0.83480002467036218405382896889932453632354736328125f,-0.103300005328655242209379139239899814128875732421875f,-0.57549995985627244721172246499918401241302490234375f,-0.2424999928474420585189363919198513031005859375f,-0.8487999519228939249160248436965048313140869140625f,-0.3942999611020088224222490680404007434844970703125f,1.384099965059757408880614093504846096038818359375f,1.6023999677434563881917028993484564125537872314453125f,1.3465999817863101295500882770284079015254974365234375f,0.650299993765354855668192612938582897186279296875f,-1.491400004780292221084891934879124164581298828125f,-2.13299998205900198655626809340901672840118408203125f,-1.06779997826814732064804047695361077785491943359375f,0.4693000156402591382942546260892413556575775146484375f,0.05919999614283455446894777196575887501239776611328125f,0.451599959993362887189505272544920444488525390625f,0.526499955415726361707129399292171001434326171875f,-0.2561000296413897814318261225707828998565673828125f,1.69929995797276500724137804354541003704071044921875f,1.8835000076293937354421359486877918243408203125f,0.049800030481815582561466726474463939666748046875f,-1.4674999809265134587121792719699442386627197265625f,0.78369997640252098136670610983856022357940673828125f,1.12970000399947156921598434564657509326934814453125f,-0.3177000045359132940347990370355546474456787109375f,-0.31690000963807118949944197083823382854461669921875f,0.0750000196695328469331798260100185871124267578125f,1.94699993017315886589813089813105762004852294921875f,1.6422999701321128185327324899844825267791748046875f,-0.73429999179840077516701057902537286281585693359375f,-0.77250004902481972379746366641484200954437255859375f,-0.35400000518560403861556551419198513031005859375f,-0.5736000101208684753828492830507457256317138671875f,-1.0644999812841415831599078956060111522674560546875f,-0.84449999640882023044241577736102044582366943359375f,-0.322399988493323075289254120434634387493133544921875f,-1.0679999929368495514836467918939888477325439453125f,0.129199955146015010853943749680183827877044677734375f,1.1630999760940670118003481547930277884006500244140625f,0.4978999871298672719177602630225010216236114501953125f,0.8741000112533559018856976763345301151275634765625f,-0.3065999817490574486100740614347159862518310546875f,0.7872999944210050671244971454143524169921875f,-0.184299991798400952802694519050419330596923828125f,-0.5038999868988991437390723149292171001434326171875f,-0.303600027704238328851715778000652790069580078125f,0.060399998044967428967311207088641822338104248046875f,-1.77649994155764634484739872277714312076568603515625f,-2.35419995265007031548520899377763271331787109375f,0.68150001755356726107493159361183643341064453125f,1.15680002830624584220231554354541003704071044921875f,-0.5453999730110172805552792851813137531280517578125f,-1.32619996381103977256543657858856022357940673828125f,0.365900012066960211853938744752667844295501708984375f,0.1693000323295592579597723670303821563720703125f,-0.190099989825487369898837641812860965728759765625f,1.3640999976187941911121015436947345733642578125f,1.947400029471516358370308807934634387493133544921875f,0.33900002843141496811085744411684572696685791015625f,-0.088199969533085731399069118197076022624969482421875f,1.954999979734420634258640347979962825775146484375f,2.458800009441375777896610088646411895751953125f,1.37509996568560577401285627274774014949798583984375f,1.9227999860167503953789491788484156131744384765625f,-0.00199998351931629514410815318115055561065673828125f,1.2718999695539476846306570223532617092132568359375f,1.527899976587296038132990361191332340240478515625f,0.6453000056862823186065725167281925678253173828125f,0.6136000274062158865717719891108572483062744140625f,0.076299994651973346293516442528925836086273193359375f,-1.0413999540790916942256671973154880106449127197265625f,-1.9894999932050705382380328956060111522674560546875f,0.14129995862841671083742767223156988620758056640625f,-0.3958999660968789413573176716454327106475830078125f,-1.7277999673902986188522845623083412647247314453125f,-1.795299983558058531940559987560845911502838134765625f,0.543999984622001164780158433131873607635498046875f,1.02480001349449167946659144945442676544189453125f,1.1878999944686885470446213730610907077789306640625f,1.42850002297759015590372655424289405345916748046875f,-1.2545999569535251083607363398186862468719482421875f,0.1901999829292293497928767465054988861083984375f,1.006299974274636355175971402786672115325927734375f,-1.5160000278949734564548634807579219341278076171875f,-1.177500028014183186542140902020037174224853515625f,-0.4533999696731569883922929875552654266357421875f,-0.53490001151561727965599857270717620849609375f,-1.302899949169159299344755709171295166015625f,-1.21969999818801877466967198415659368038177490234375f,1.412199984255433360402776088449172675609588623046875f,0.6492000124037264896514898282475769519805908203125f,-1.2708000233590601713018486407236196100711822509765625f,-0.49310002605021008292851547594182193279266357421875f,0.52390000850558227085684848134405910968780517578125f,-0.7834999627768990393406056682579219341278076171875f,-0.17989999706149095715090879821218550205230712890625f,-1.5047999521017079160856155795045197010040283203125f,-1.98419997738599729331099297269247472286224365234375f,-0.16080003244876850398981105172424577176570892333984375f,0.6752999872460960784081862584571354091167449951171875f,0.64960000087469804697093422873876988887786865234375f,1.3923999643161895090059942958760075271129608154296875f,-0.381499988757073804634956104564480483531951904296875f,-0.394100008161366144321391402627341449260711669921875f,0.47860000149682135894835255385260097682476043701171875f,-0.5554000016212459200914963730610907077789306640625f,-0.86910000067353276875792289501987397670745849609375f,0.47099995994567933621510746888816356658935546875f,0.24879999871253932752779292059130966663360595703125f,-1.965299973946809330982432584278285503387451171875f,-0.79730002258420018534934570197947323322296142578125f,0.610399976214766493143315528868697583675384521484375f,0.946299956616759363470237076398916542530059814453125f,0.85750000894069700052568805404007434844970703125f,-0.8032999883413314279323458322323858737945556640625f,-0.075000023990869468804021380492486059665679931640625f,0.496100001217425035093810947728343307971954345703125f,2.032599964996427477359475233242847025394439697265625f,1.37539995751380939026375926914624869823455810546875f,-0.96850003862380962260658634477294981479644775390625f,0.1018999606132506841049689683131873607635498046875f,0.3838999782562257934159788419492542743682861328125f,-0.0439999783635141028526049922220408916473388671875f,0.7334999611377721606686463928781449794769287109375f,0.775499934673309798682794280466623604297637939453125f,-0.05750005766749399782611362752504646778106689453125f,0.17060001584291484277855488471686840057373046875f,-0.73719998142421250264533227891661226749420166015625f,-1.14280001753270621378533178358338773250579833984375f,-1.0178999747619033033885216354974545538425445556640625f,0.5395999828815458698727525188587605953216552734375f,1.09890001446604745893864674144424498081207275390625f,0.46139996931552929027020581997931003570556640625f,-1.1027000365734096476444392465054988861083984375f,-1.873599997305870346053779940120875835418701171875f,-0.9603999952137467488455513375811278820037841796875f,1.42179997108578692888158911955542862415313720703125f,0.5463000085473055378315621055662631988525390625f,0.3145999962925909443356431438587605953216552734375f,-0.283199957537650703898179926909506320953369140625f,-0.9876000024914741715065247262828052043914794921875f,-0.1255000145435329894638698533526621758937835693359375f,0.074299933758378511328146487358026206493377685546875f,1.02319997877180579592959475121460855007171630859375f,0.70889999081045385054267171653918921947479248046875f,0.0573000145748252531774369344930164515972137451171875f,0.117300040912628134037731797434389591217041015625f,-0.3870000098943713329191496086423285305500030517578125f,-0.55659998778402808294885062423418276011943817138671875f,-0.0589999811574817289994143720832653343677520751953125f,-0.78509999623298654114478267729282379150390625f,0.4859999814033511000843645888380706310272216796875f,-0.5462999653339384309447268606163561344146728515625f,-1.9170999992728230409966272418387234210968017578125f,-0.3738000085286807827600341624929569661617279052734375f,0.599899998160452252449204024742357432842254638671875f,0.7901999689221383960102684795856475830078125f,0.372899990892410215792551753111183643341064453125f,-0.21790000817775734276438015513122081756591796875f,-0.0007000278711313256962966988794505596160888671875f,0.134099970573186677569310631952248513698577880859375f,0.645700031596422530100198855507187545299530029296875f,-0.51580002157092064152266175369732081890106201171875f,0.632899966715276374173981821513734757900238037109375f,0.882300019343197572396775285596959292888641357421875f,-1.9920999725624917164878979747300036251544952392578125f,-2.02050001692771896699696299037896096706390380859375f,-0.6132000096917149534192503779195249080657958984375f,-0.51009999802112560018940712325274944305419921875f,-1.9485999984979631705073188641108572483062744140625f,-1.04799999055266379599515857989899814128875732421875f,-0.0648999802231788880391150087234564125537872314453125f,0.261299986419081253785634544328786432743072509765625f,0.237599969410897671195925795473158359527587890625f,-0.8047000294804576014939812012016773223876953125f,0.07800002348422996334420531638897955417633056640625f,0.29240001926422110045677982270717620849609375f,0.1121999576568608603110988042317330837249755859375f,0.40419999362826342093057974125258624553680419921875f,1.52999998688697846915829359204508364200592041015625f,0.268800021138786870977810394833795726299285888671875f,0.16019998546242764092539800913073122501373291015625f,0.89789997360706319540213371510617434978485107421875f,-0.047799998310208646756791495135985314846038818359375f,0.61199996474385276457041982212103903293609619140625f,0.9422999971032144372884431504644453525543212890625f,-0.401099971079826644881904940120875835418701171875f,-1.24109996064901384471568235312588512897491455078125f,-1.5347000104069703496634247130714356899261474609375f,-0.20830005599260292825647411518730223178863525390625f,-1.7594000267744061005714684142731130123138427734375f,-0.3936000250220299534475998370908200740814208984375f,-1.2513999532222754851318313740193843841552734375f,-1.330399986720085081515208003111183643341064453125f,0.355700007009506435906587284989655017852783203125f,1.1491999937772749973419195157475769519805908203125f,0.6932999811887743391025651362724602222442626953125f,-0.025300010901689429232419570325873792171478271484375f,-0.2360999849751592716273762562195770442485809326171875f,-0.58039999074339920781540058669634163379669189453125f,1.870699977803230495965181034989655017852783203125f,1.400700011405348721638119968702085316181182861328125f,0.341999976590276144605695662903599441051483154296875f,0.17069997899532296514735207892954349517822265625f,-0.2434000090122221404698166224989108741283416748046875f,-1.031299968463182725741944523178972303867340087890625f,-0.39080002753138476379035637364722788333892822265625f,-0.314300031733512952314413269050419330596923828125f,-0.53590001914501073088104021735489368438720703125f,-0.3536000145912172598627876141108572483062744140625f,-0.11370005508065172961096322978846728801727294921875f,-0.2663000192761419970821634706226177513599395751953125f,0.107200006160140137723146835924126207828521728515625f,-0.7010999928355214905195680330507457256317138671875f,-0.6225000080466269736234607989899814128875732421875f,0.0822999887585638134623877704143524169921875f,-0.98479998756647102453598563442938029766082763671875f,-2.28749997243285196901751987752504646778106689453125f,-1.3939999840259549301890729111619293689727783203125f,-1.1026999748826025182779630995355546474456787109375f,-2.0118000166833400754740068805404007434844970703125f,-0.76890002191662798480820129043422639369964599609375f,-0.8325999753713608786398481242940761148929595947265625f,-0.43910001097396011981999208728666417300701141357421875f,0.1640999832391738522119339904747903347015380859375f,0.422300010365247668886468090931884944438934326171875f,-0.08390000209808334830086096189916133880615234375f,0.570699967558681908741391453077085316181182861328125f,-0.512499978430569047560538820107467472553253173828125f,-0.29409999454543023222896636070800013840198516845703125f,-0.0743000093817709483801081660203635692596435546875f,-1.04110001339912372486651293002068996429443359375f,-0.6332000093005596408346491443808190524578094482421875f,-0.37909998713210224519087887529167346656322479248046875f,0.1777999907106160293324137455783784389495849609375f,-1.138499981179833309141713471035473048686981201171875f,0.085499995097517711428736220113933086395263671875f,2.440399952074886247288532103993929922580718994140625f,1.771999968469142316962461336515843868255615234375f,0.668300036621094051270119962282478809356689453125f,-0.25380001458227585686699967482127249240875244140625f,-0.592499988004565825150393720832653343677520751953125f,-0.824399992799758507544538588263094425201416015625f,-1.458100011968612275126133681624196469783782958984375f,-0.2826999967500569699296875114669091999530792236328125f,-0.2239000161051745152462899568490684032440185546875f,-0.6478000140309330134869014727883040904998779296875f,-0.2876999780118467686662597770919091999530792236328125f,-0.646700000259280205483491954510100185871124267578125f,-0.1123999966144559881087161556934006512165069580078125f,-0.538700016263127690052670004661194980144500732421875f,0.201500017106532869348711756174452602863311767578125f,-0.7249999955669041984407385825761593878269195556640625f,-0.113099991777539710113842374994419515132904052734375f,0.031200034964084633060110718361102044582366943359375f,1.17839999545216489451604502392001450061798095703125f,-0.5763999779582018589962899568490684032440185546875f,-1.55770002644062088847931590862572193145751953125f,0.802299961042404419231388601474463939666748046875f,1.64949996876716653559924452565610408782958984375f,1.3744999658241867646069067632197402417659759521484375f,1.47060001345872848332874127663671970367431640625f,0.9725999869942665299049622262828052043914794921875f,1.8342999494791030201668036170303821563720703125f,0.512900021290779051241770503111183643341064453125f,-1.8647000345468516258051749900914728641510009765625f,-0.63110001175999652645032256259582936763763427734375f,0.19059998097419761364790247171185910701751708984375f,0.8301000159025182512095852871425449848175048828125f,-0.02949999052286056411276149447076022624969482421875f,0.004399999356270001271695946343243122100830078125f,0.153600014591216194048683973960578441619873046875f,1.070100026631355394357569821295328438282012939453125f,2.557699997383355405844440610962919890880584716796875f,1.1033000217571851120368364718160592019557952880859375f,0.6725000271201135859655551030300557613372802734375f,0.95300002348422907516578561626374721527099609375f,0.7843000506579880237723045866005122661590576171875f,-0.0457000062644485893770252005197107791900634765625f,-0.8152999621748922010056048748083412647247314453125f,-1.308899999713897699393783113919198513031005859375f,-0.82509995808601388489478267729282379150390625f,0.65009999362528336774857962154783308506011962890625f,0.45499997459352048423397718579508364200592041015625f,-0.5870000496432183911821311994572170078754425048828125f,0.6587999957323074085024927626363933086395263671875f,0.07520001034736623068965855054557323455810546875f,0.425099985504150623683017329312860965728759765625f,1.208799968910216904305343632586300373077392578125f,-0.05339998695850312060429132543504238128662109375f,1.1476999423980718262328082346357405185699462890625f,1.014699941116571579868832486681640148162841796875f,1.440999985277652939430481637828052043914794921875f,1.0647000273942950343553093262016773223876953125f,-0.514499974727630871029759873636066913604736328125f,0.403699998903274348549530259333550930023193359375f,1.054199996161461427846006699837744235992431640625f,1.4084999614357940345144015736877918243408203125f,-0.2020999241709713345471755019389092922210693359375f,-1.74429998457133716982525584171526134014129638671875f,-0.470000014081597417003877126262523233890533447265625f,1.0534999542832377272816302138380706310272216796875f,1.05239998707771320596293662674725055694580078125f,0.633399992322921434606541879475116729736328125f,0.886400011336803572703502140939235687255859375f,-0.5825000128149984135461636469699442386627197265625f,-0.8885000261068345395187861868180334568023681640625f,-1.63779993535280254945973865687847137451171875f,-0.8969999788999558631985564716160297393798828125f,-0.4726999492526058332941829576157033443450927734375f,-1.1849999895691869511438198969699442386627197265625f,-0.18720000057220431699533946812152862548828125f,0.2905999988555907354026430766680277884006500244140625f,2.365699965360760881338819672237150371074676513671875f,1.50529998452663438257559391786344349384307861328125f,-0.71899996569752744335346505977213382720947265625f,0.5681000000476839062457656837068498134613037109375f,1.746100017683207550334145707893185317516326904296875f,-0.9685999716386197722073347904370166361331939697265625f,-0.347000000357628124447728623636066913604736328125f,0.6448999939322472840785849257372319698333740234375f,-1.2944999668300152251276813331060111522674560546875f,-1.59559999960064846646901060012169182300567626953125f,-1.7532000177383419980969847529195249080657958984375f,-0.6073000787615772111394107923842966556549072265625f,1.55479996998310099343143519945442676544189453125f,1.545699981230497233042342486442066729068756103515625f,1.2726999778628351744913516085944138467311859130859375f,1.2669999683573844873052394177648238837718963623046875f,0.46719999505877485290739059564657509326934814453125f,-0.01369998668432259592009359039366245269775390625f,1.2399000493645662146491304156370460987091064453125f,1.017600016796588846546001150272786617279052734375f,0.635399992103129473974831853411160409450531005859375f,-0.79649996197223682514731990522705018520355224609375f,-2.320399996554851629326776674133725464344024658203125f,-1.0075000112131233809265040690661408007144927978515625f,0.79799999919533737280374907641089521348476409912109375f,0.7979000014349815206315952309523709118366241455078125f,-0.2524999812431636403431411963538266718387603759765625f,0.16820004212036676616293107144883833825588226318359375f,1.072600025290250780329870394780300557613372802734375f,1.2605000003874298197814596278476528823375701904296875f,0.88149994377419382818317217243020422756671905517578125f,-0.23539999446868886678885246510617434978485107421875f,-0.6860000001788133516811285517178475856781005859375f,-1.5594000160455703252182502183131873607635498046875f,-1.1561000131011009983694748370908200740814208984375f,1.2910999698877336783198188641108572483062744140625f,1.5919999662339687773737750831060111522674560546875f,0.18630000988841022291353510809130966663360595703125f,0.372399945354462857949329190887510776519775390625f,-0.4990000358819965953216524212621152400970458984375f,0.030099989080429878640643437393009662628173828125f,0.20890000209808334830086096189916133880615234375f,0.5653999917864798163691375521011650562286376953125f,0.5960999922394751848742089350707828998565673828125f,0.80869997357130074533415609039366245269775390625f,1.8070999748162923648209243765450082719326019287109375f,1.999499990019947492925211918191052973270416259765625f,-0.2240999158114203027025723713450133800506591796875f,-1.846999964222311607642268427298404276371002197265625f,0.456599948370456587554144789464771747589111328125f,0.7421000249028200101975016877986490726470947265625f,-1.439099957609176527739691664464771747589111328125f,-0.6531000051140782858283273526467382907867431640625f,-0.2498000134944913241952235694043338298797607421875f,0.6929999817609786560979046043939888477325439453125f,1.28409999083876602554710188996978104114532470703125f,-0.6118999752163885030853407442918978631496429443359375f,-1.01260001320168324667747583589516580104827880859375f,-0.7898000909805293900944889173842966556549072265625f,-0.17199999946355859492541640065610408782958984375f,-0.5852000112414348365064142853952944278717041015625f,-0.834500034332275930637479177676141262054443359375f,-0.36569999389648455423440509548527188599109649658203125f,-0.8754999481216072521050364230177365243434906005859375f,-0.272200026053190047292673625634051859378814697265625f,1.4067999697446822437996161170303821563720703125f,0.006500012636184493430846487171947956085205078125f,0.18660000827312472893027006648480892181396484375f,-0.58579999221563383571265148930251598358154296875f,0.170400012946128498469988699071109294891357421875f,0.237900007581711037119021057151257991790771484375f,0.33430001713037427180097438395023345947265625f,0.339100013935566213518768563517369329929351806640625f,0.581699965912104044463148966315202414989471435546875f,0.66379997114539168734381746617145836353302001953125f,-0.9667999773442748079332886845804750919342041015625f,-0.554999962002038937924908168497495353221893310546875f,-1.22679995320439427786141095566563308238983154296875f,-0.9969999669790265972579845765721984207630157470703125f,0.11280000166296932917475714930333197116851806640625f,0.32779998035430946146107089589349925518035888671875f,1.126899984082579297961501652025617659091949462890625f,1.970599972331523730417757178656756877899169921875f,-0.2084999566674234916519026228343136608600616455078125f,-0.713000021938233974339027554378844797611236572265625f,1.4638999389171605258752606459893286228179931640625f,0.5730999735236166348073538756580092012882232666015625f,-0.115399994990229348701404887833632528781890869140625f,0.190200008559227029536486952565610408782958984375f,-0.5812999952852724305785159231163561344146728515625f,0.13969994186162981719689923920668661594390869140625f,0.95009997760653508436234915279783308506011962890625f,1.35239996286332608832481128047220408916473388671875f,1.395600008168816241749254913884215056896209716796875f,0.425400001025199259174769395031034946441650390625f,0.63410001423358952621356365852989256381988525390625f,0.59619999346435026410517821204848587512969970703125f,0.76239999795556112616168320528231561183929443359375f,-0.24540001793801735630040639080107212066650390625f,0.6765999768689265891907780314795672893524169921875f,2.64750002671033168866188134416006505489349365234375f,0.68000001998618220255821142927743494510650634765625f,-1.0847999783277515462032170034945011138916015625f,-0.510199993956089059565783827565610408782958984375f,-0.161699979062378762506568818935193121433258056640625f,0.3905999836936595581704523283406160771846771240234375f,1.2540999963521954896350507624447345733642578125f,-0.127300009322166118863606243394315242767333984375f,-0.9356000185251236445083122816868126392364501953125f,-1.48159997206330285024478143895976245403289794921875f,-1.101900022117793032094823502120561897754669189453125f,-0.8081000099197035435594216323806904256343841552734375f,-0.1017999977588655013960305950604379177093505859375f,0.4157000149071219397001186734996736049652099609375f,1.19530000002384184654147247783839702606201171875f,0.1320000081062318031399627216160297393798828125f,-0.465399991488456787891436761128716170787811279296875f,-1.2686999768495557106007254333235323429107666015625f,0.4137999352335928193724612356163561344146728515625f,1.35700000929832498286486952565610408782958984375f,-0.4278999920845034665717321331612765789031982421875f,0.173099984587728794593886050279252231121063232421875f,-0.9943000095680349925686414280789904296398162841796875f,-1.720000017285347126971828402020037174224853515625f,-0.9855000231862067749943889793939888477325439453125f,-1.0970999876499174519040025188587605953216552734375f,-0.26709999912381210407374965143389999866485595703125f,-0.79210000255107804179033337277360260486602783203125f,-0.713100002804398780398287271964363753795623779296875f,0.916599987895786671998621386592276394367218017578125f,0.7884000226840373048986521098413504660129547119140625f,-0.4067999876260757208257246020366437733173370361328125f,0.40029997176751541676509305034414865076541900634765625f,0.633799991559982345279422588646411895751953125f,-0.448600012765824285310145569383166730403900146484375f,-0.4177000196233391360323139451793394982814788818359375f,0.3193000180244442987032016390003263950347900390625f,0.6699999871850017552077360960538499057292938232421875f,0.02650000376999361861862780642695724964141845703125f,-0.7467999599471693006336181497317738831043243408203125f,-0.559099996575712854252060424187220633029937744140625f,-1.638399975931644547699761460535228252410888671875f,-0.87089998770356213952936741407029330730438232421875f,-0.09289996228218111440355642116628587245941162109375f,0.030599994683265574479946735664270818233489990234375f,-0.9349999598041180970398045246838591992855072021484375f,-2.53189993855953243695466881035827100276947021484375f,-1.05230000857710859207827525096945464611053466796875f,-0.132099922680854575673947692848742008209228515625f,-1.432100050234794252901338040828704833984375f,0.891199950623512648917312617413699626922607421875f,0.5955999854445455099494211026467382907867431640625f,-0.095900001263618861457871389575302600860595703125f,0.12350002372264867744888761080801486968994140625f,0.608099982166290686791398911736905574798583984375f,0.8355999607086186387050474877469241619110107421875f,0.17969997017383576576321502216160297393798828125f,-0.53610001548528618542377444100566208362579345703125f,-1.859999990984797069160094906692393124103546142578125f,-1.9211000008821488282961809090920723974704742431640625f,-0.20909999547675262032697673930670134723186492919921875f,-0.486100002372264583527794457040727138519287109375f,1.306199980425834183961342205293476581573486328125f,1.510399982249737149686552584171295166015625f,-1.2740999821960929949682395090349018573760986328125f,-1.09760001098513537698408981668762862682342529296875f,-0.2509000385165212065885498304851353168487548828125f,0.16670001761913244564539127168245613574981689453125f,-0.31729996551275263527713832445442676544189453125f,-0.479600015664100709500416996888816356658935546875f,0.155899981594085801361870835535228252410888671875f,0.34049997565150302847314378595910966396331787109375f,-0.3121999968469142405780303306528367102146148681640625f,-0.023700021925568304226317195571027696132659912109375f,0.55619999830722832712126546539366245269775390625f,-1.4805999869346617714427338796667754650115966796875f,-1.841599932612479673110783551237545907497406005859375f,0.5121000362649554826788289574324153363704681396484375f,1.338200031447410509599649230949580669403076171875f,-0.095800002050399513109368854202330112457275390625f,0.8440999775767323853870038874447345733642578125f,-0.964799960334599138178646171581931412220001220703125f,-1.7298999718412757520269451561034657061100006103515625f,-0.776899993395805399387654688325710594654083251953125f,-0.2380000119730827368158543322351761162281036376953125f,-0.6762000164866446283440382103435695171356201171875f,1.325899961924552172831681673415005207061767578125f,0.87250002443790464212725055404007434844970703125f,0.188599989967048298211693690973334014415740966796875f,-0.3387999658927329793556282311328686773777008056640625f,-0.92089994881153103278847993351519107818603515625f,0.05589998278617880345109369955025613307952880859375f,1.1796000064253799877178607857786118984222412109375f,2.0148000325679777944287707214243710041046142578125f,0.7343000281572340526281550410203635692596435546875f,-2.192299961042404987665577209554612636566162109375f,-1.287899982547759947237864253111183643341064453125f,0.45170000241994845424642335274256765842437744140625f,1.136400014540552394493033716571517288684844970703125f,1.0738000246405601245669458876363933086395263671875f,-0.59479995262324880656024106428958475589752197265625f,-0.414299961176514575100782167282886803150177001953125f,0.575100001597404375530686593265272676944732666015625f,1.20739996472597166388140976778231561183929443359375f,-0.016500001236796624226599305984564125537872314453125f,-0.4901999927639959508951505995355546474456787109375f,-0.7045000084042545296370008145458996295928955078125f,-1.481000037431716265245995600707828998565673828125f,0.60029994444251055796257787733338773250579833984375f,1.95189996299743739882615045644342899322509765625f,-1.095499993681908090792376242461614310741424560546875f,-0.4499999472126374744362919955165125429630279541015625f,-0.50089995566606493326844429248012602329254150390625f,-0.08010000616088552849447523840353824198246002197265625f,0.66790003162845990036800003508687950670719146728515625f,0.476099993133544785450794734060764312744140625f,1.2184999718666080781304117408581078052520751953125f,0.965099946314097234534301605890505015850067138671875f,-0.0008999694123870138895426862291060388088226318359375f,-0.1671999812565745369141723131178878247737884521484375f,-0.62419999081566934773235288957948796451091766357421875f,-0.377700016605853505780032719485461711883544921875f,1.4643999984435731587240070439293049275875091552734375f,1.124200033060461567657739578862674534320831298828125f,0.08599999712407591090368441655300557613372802734375f,-0.88960001290738544010849864207557402551174163818359375f,-0.7487000234529379572023799482849426567554473876953125f,-1.22160000842213545269032692885957658290863037109375f,-2.4186000008821491036314910161308944225311279296875f,-0.51689997700452838813589551136828958988189697265625f,0.85400002248957773698379014604142867028713226318359375f,-0.27629998580440917610445694663212634623050689697265625f,0.201300042819976710006812936626374721527099609375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.021300013352930857735145764308981597423553466796875f,0.6460000353828068142547635943628847599029541015625f,0.58629996496141145723157706015626899898052215576171875f,0.23189999674856520694987693786970339715480804443359375f,0.5205999745666982647662734962068498134613037109375f,-0.47400002612173519622729145339690148830413818359375f,-0.367199996697902708575611541164107620716094970703125f,-0.5686000469267369350490071155945770442485809326171875f,-0.136399997031688702264773382921703159809112548828125f,-0.2953000022962688486671822829521261155605316162109375f,0.95509998889416503065064034672104753553867340087890625f,0.156300009106100201705658037099055945873260498046875f,-1.90429999228268886923842728720046579837799072265625f,-1.417399934811890194197303571854718029499053955078125f,0.0181999605923890417358279592008329927921295166015625f,-0.1139999944567680056906056051957421004772186279296875f,0.014600021810829921786734075794811360538005828857421875f,1.0305999687552453991656875587068498134613037109375f,-1.1647000665843496580720284327981062233448028564453125f,-0.0561999946191897858938091303571127355098724365234375f,-0.879299931523203071037642075680196285247802734375f,0.658899999862908369863134794286452233791351318359375f,0.3902999746173616557598506915383040904998779296875f,0.5241999879658225669487592313089407980442047119140625f,-0.4854999893233171714967966181575320661067962646484375f,-1.078899903936683557503783958964049816131591796875f,0.699200011807679544517668546177446842193603515625f,-0.1249000067472458841422167097334749996662139892578125f,-1.24730002066567546847863923176191747188568115234375f,0.376600004734097992820096578725497238337993621826171875f,0.5601999679908151730245435828692279756069183349609375f,-0.83070007041394777758114287280477583408355712890625f,1.126899954354763533359573557390831410884857177734375f,-0.631600028277933489562201430089771747589111328125f,-0.14540001048743766975945845842943526804447174072265625f,-0.024399991011619714953440052340738475322723388671875f,0.43549998414516355449421780576813034713268280029296875f,0.503199940717965699121805300819687545299530029296875f,-1.16200008171796920208862502477131783962249755859375f,-1.1657999983623614781436117482371628284454345703125f,1.035200025472045037844281978323124349117279052734375f,-0.93330001084208469297465171621297486126422882080078125f,-1.0659000428751121969384030307992361485958099365234375f,-0.5006999434187999753476105979643762111663818359375f,-0.2491999934792517024106928147375583648681640625f,-0.6944000115007160633950888950494118034839630126953125f,-0.0040999882683154798002078678109683096408843994140625f,-0.56169994624257146842438714884337969124317169189453125f,0.323700003224610899366098237805999815464019775390625f,-0.1767000089392058992388001570361666381359100341796875f,0.26640004099011438398747486644424498081207275390625f,0.1445000231936577339553195997723378241062164306640625f,0.4196000056058164506822549810749478638172149658203125f,-0.756399979448318493524539007921703159809112548828125f,-0.56410002745836995696748772388673387467861175537109375f,-0.49049999915808495387636867235414683818817138671875f,-1.0519000227510926226415222117793746292591094970703125f,-0.71780006823390696268916144617833197116851806640625f,0.00830006921738457581483316971571184694766998291015625f,-0.6883000383347279704793209020863287150859832763671875f,0.1650000076740985033296738038188777863979339599609375f,-0.00790003216862700252676177115063183009624481201171875f,-0.846999978303909273336103069595992565155029296875f,-0.0721999974429607949133469446678645908832550048828125f,0.474599953041971556189082548371516168117523193359375f,-0.13329996327012738621675680406042374670505523681640625f,-0.1712000336229806407573050819337368011474609375f,0.5008000121459363906950557066011242568492889404296875f,0.310999989077449023877619538325234316289424896240234375f,1.17890002228915680149157196865417063236236572265625f,1.156999999612569496321157203055918216705322265625f,1.1039999543726448649749727337621152400970458984375f,0.3531000066414475835330222253105603158473968505859375f,-0.5477999633669856649476059828884899616241455078125f,-1.911700010205805266849665713380090892314910888671875f,-1.387200084987283421611437006504274904727935791015625f,-0.6432000276476135258718613840756006538867950439453125f,0.93490003442615243756108611705712974071502685546875f,2.020700030776857669678747697616927325725555419921875f,0.47279993904084083311545327887870371341705322265625f,0.041400009250640579239188809879124164581298828125f,-0.02859999890774489461620078145642764866352081298828125f,0.61789998385161160054934725849307142198085784912109375f,-1.469100004175304174935945411561988294124603271484375f,-1.3326000483497983939429332167492248117923736572265625f,0.6767999834537505865483808520366437733173370361328125f,0.03199999827146526865107034609536640346050262451171875f,1.46540001037567879649259339203126728534698486328125f,0.41349998825788514267287609982304275035858154296875f,-0.08539999387264263219776694313623011112213134765625f,-0.4595000106021760455377034304547123610973358154296875f,-0.965699999260902330888711730949580669403076171875f,0.1954999831952151101432946234126575291156768798828125f,-0.186200008328258392875653726150630973279476165771484375f,0.816900053261220282507792944670654833316802978515625f,-0.943100056225061766923545292229391634464263916015625f,-0.1949000479862090795535323195508681237697601318359375f,-0.754300045144558506393650532118044793605804443359375f,-1.1237999702513217226140795901301316916942596435546875f,-1.32390001841485460687408703961409628391265869140625f,-1.345100011767446090971134253777563571929931640625f,-0.95749999798834384279189180233515799045562744140625f,0.9854000048995017468911328251124359667301177978515625f,0.16990000052601017532794003273011185228824615478515625f,0.01599994787573855159479307985748164355754852294921875f,-0.583599974395335063803713637753389775753021240234375f,0.38819999367296686987316434169770218431949615478515625f,0.53080000562667795094284883816726505756378173828125f,-0.064700011301040749600588242174126207828521728515625f,-0.127199946816265896387676548329181969165802001953125f,0.25689999600350954533922731570783071219921112060546875f,1.340100010463594504273032725905068218708038330078125f,0.265499959632754300997703467146493494510650634765625f,-0.85360000263303448253537908385624177753925323486328125f,-0.84490001319199847795715641041169874370098114013671875f,0.8377999974533916205388095477246679365634918212890625f,0.012799986240268068637959686384419910609722137451171875f,0.4683000062227247184409861802123486995697021484375f,0.03239998229071527902078742044977843761444091796875f,-1.8150000363215799126237470773048698902130126953125f,-2.609000014238059161897353988024406135082244873046875f,-0.7519999916031963049789510478149168193340301513671875f,-1.4015999935582275082168735025334171950817108154296875f,0.8502000081866982750256056533544324338436126708984375f,-0.22139996532946748519776747343712486326694488525390625f,0.25899999982118604435754605219699442386627197265625f,-0.6587999981164927021382027305662631988525390625f,-0.216699990703910660982955960207618772983551025390625f,-0.5299000460118066246906209926237352192401885986328125f,-1.2134000237271178601616838932386599481105804443359375f,0.55379999239817190304080440910183824598789215087890625f,-0.6110000254362824012588362165843136608600616455078125f,-0.958499996937810738728558135335333645343780517578125f,-0.412699974435568162078880050103180110454559326171875f,0.619399972832203271622120155370794236660003662109375f,-0.068699992942809817719762577326036989688873291015625f,-0.49919997537434124268429513904266059398651123046875f,-1.46240000305920769818612825474701821804046630859375f,-0.77320000026673074700056531582958996295928955078125f,-0.334599979789555135312184575013816356658935546875f,-0.489999992214143365032441579387523233890533447265625f,-0.9191999746665355797148322380962781608104705810546875f,-0.02289998828470685143798846183926798403263092041015625f,0.6669999705553057989249055026448331773281097412109375f,0.04430000520944599173844835604541003704071044921875f,2.18270000129491048568297628662548959255218505859375f,-1.2250000727921726006997005242737941443920135498046875f,0.229700054402649289642113217269070446491241455078125f,1.34540002011731285591622508945874869823455810546875f,1.066500040203333554700293461792171001434326171875f,-0.1392999852232639224069998817867599427700042724609375f,-0.121599988119303892286637847064412198960781097412109375f,-0.18999998688697861126684074406512081623077392578125f,0.28200001317262668454333152112667448818683624267578125f,0.45890001334846031166847524218610487878322601318359375f,0.22540003049224643927317401903565041720867156982421875f,0.64220002802759434956669792882166802883148193359375f,-0.91660000592619128578775189453153870999813079833984375f,0.7789000679239632685124661293230019509792327880859375f,1.2314000020980826466399093988002277910709381103515625f,-0.745300028578192108597022524918429553508758544921875f,-0.5283999983951448431440667263814248144626617431640625f,0.58640000463128050256500500836409628391265869140625f,0.633799936388432794132086200988851487636566162109375f,-0.3843999892234801762924689683131873607635498046875f,0.475800001566111685491478056064806878566741943359375f,0.6697000129401680101892679886077530682086944580078125f,1.770299976573138867053103240323252975940704345703125f,1.618799944360554743383318054839037358760833740234375f,-0.9816000556588175829375586545211263000965118408203125f,0.080800012779235430571134202182292938232421875f,1.15010002923905840788165733101777732372283935546875f,-1.19400000984221588140599124017171561717987060546875f,0.63220004996955359022337006535963155329227447509765625f,-0.674499975472689694555583628243766725063323974609375f,-0.6847000535085789874045758551801554858684539794921875f,0.5225999916881314266703384419088251888751983642578125f,-0.2145000170841806497179504731320776045322418212890625f,1.7443000014096485816850190531113184988498687744140625f,0.78419999094605419287518088822253048419952392578125f,0.30880002553463015146917314268648624420166015625f,1.32620004301071237051701245945878326892852783203125f,-1.0767000381827351862540353977237828075885772705078125f,-1.602399987487494126270348715479485690593719482421875f,-1.4555000319778927941882784580229781568050384521484375f,1.6496000276967894837554240439203567802906036376953125f,2.18760004261285079252274954342283308506011962890625f,1.114700029890239196816992262029089033603668212890625f,0.584799984884261991879839115426875650882720947265625f,0.7534999994710080972737387128290720283985137939453125f,-1.35790001518130143409734955639578402042388916015625f,-0.1489999393597243315667810747982002794742584228515625f,-0.487099982732534630969212230411358177661895751953125f,0.00050003253668473579551800867193378508090972900390625f,-0.867800012615322913234194857068359851837158203125f,0.29770003966540159456855008102138526737689971923828125f,1.0846999720364802755057098693214356899261474609375f,0.40899996793270165795775028527714312076568603515625f,0.0120999953612683786019488252350129187107086181640625f,-0.59090007938295752154544970835559070110321044921875f,-0.937999966152012643050284168566577136516571044921875f,0.66339999466985499765314671094529330730438232421875f,0.06359996664672984412192136005614884197711944580078125f,0.7333000083088878628956308602937497198581695556640625f,0.682200020204484669505973215564154088497161865234375f,0.908200021500885412706338684074580669403076171875f,1.048999977320432375194059204659424722194671630859375f,0.5327000042006371227643057864042930305004119873046875f,-1.3705999982967969064162616632529534399509429931640625f,0.021700018401443532667371982824988663196563720703125f,-1.10360003798604022762219756259582936763763427734375f,0.28330003626719169051995095287566073238849639892578125f,0.60819999952167247414536177529953420162200927734375f,0.4008999962717296750014384087990038096904754638671875f,-0.827000034928321525740102515555918216705322265625f,-0.959999991506337746471899663447402417659759521484375f,0.6649000059276815033371121899108402431011199951171875f,1.1325000194832688382717833519564010202884674072265625f,1.5217000486135485726180149868014268577098846435546875f,1.51339999403655500742615913623012602329254150390625f,-0.02920000266209232808733986530569382011890411376953125f,0.22669995350688754154333537371712736785411834716796875f,0.772499997466801868739594283397309482097625732421875f,-0.16020001552551921264466727734543383121490478515625f,-1.16819996916055668378930931794457137584686279296875f,0.4766000227458777604994111243286170065402984619140625f,0.621900043500960730824544953065924346446990966796875f,0.01879998723864539922345784361823461949825286865234375f,-0.9631000446394086811352508448180742561817169189453125f,-0.9659999722018834944492482463829219341278076171875f,-0.77630003492236177020657805769587866961956024169921875f,-0.74690000490695229728999038343317806720733642578125f,-0.69940003761500191270528148379526101052761077880859375f,0.12259998431205776370234161731787025928497314453125f,-0.0571000007629394179531345798750407993793487548828125f,-0.41840001014843586357727645008708350360393524169921875f,-0.2653000630557531547282223982620052993297576904296875f,-1.2924999712035056820269574018311686813831329345703125f,-0.31559996722787619916061885305680334568023681640625f,1.6104000200614330129411655434523709118366241455078125f,0.5453000341102478554233812246820889413356781005859375f,-0.2577000871628525313639102023444138467311859130859375f,0.21579999087452905737194441826432012021541595458984375f,1.3205000441595913240888648942927829921245574951171875f,0.5142999901965261688729924571816809475421905517578125f,0.250599985593557283980459260419593192636966705322265625f,-0.684700032572448602508075055084191262722015380859375f,-0.4112999595224844373575479039573110640048980712890625f,0.22640000939965176929291601481963880360126495361328125f,-0.6673000246703626459066072129644453525543212890625f,0.57859998683780367034756864086375571787357330322265625f,0.538599962139130195026837100158445537090301513671875f,0.172599975594877275142380312900058925151824951171875f,0.917799975138902990323686026385985314846038818359375f,0.46649999244511153317915841398644261062145233154296875f,-1.0875000169500717017712076994939707219600677490234375f,1.798300049138070288989865730400197207927703857421875f,0.3990000265687694280813957448117434978485107421875f,-0.30880003589093707461188387242145836353302001953125f,-1.600699954482913245357167397742159664630889892578125f,-0.2595999857500197460780100300326012074947357177734375f,-0.4289000221401455537062474832055158913135528564453125f,0.08490004072189372141821195327793247997760772705078125f,1.4915999952718614895985638213460333645343780517578125f,0.202399958355725662073609782964922487735748291015625f,0.513400009272992452480366409872658550739288330078125f,0.12269998834952711153079007999622263014316558837890625f,-0.53280005646198969060378658468835055828094482421875f,-1.0040999432295565352291077942936681210994720458984375f,-0.41340001512169788355777200195007026195526123046875f,0.64940000814795462158457439727499149739742279052734375f,-0.2460999970450987728298741785692982375621795654296875f,-0.356099959717692371441444265656173229217529296875f,-0.21950000173598527286600301522412337362766265869140625f,0.30210001246035111943655238064820878207683563232421875f,-0.7937000004306427047851002498646266758441925048828125f,0.18060005093514863716563922935165464878082275390625f,-0.980800009091197910748860522289760410785675048828125f,-0.990999944895506690301090202410705387592315673828125f,-0.8213000198349356129057241560076363384723663330078125f,-1.009700032907724587261100168689154088497161865234375f,0.32809999162852676590773626230657100677490234375f,0.377799952228367263984409873955883085727691650390625f,-0.2232000134542584390828778850845992565155029296875f,0.19680006194561661114761363933212123811244964599609375f,-0.8072000213593242268217409218777902424335479736328125f,-1.012299945470691131532703366247005760669708251953125f,-0.4473999851405618954913734341971576213836669921875f,0.32669999187737663870478854732937179505825042724609375f,-0.509400008594989461840896183275617659091949462890625f,-0.759299964044988229261434753425419330596923828125f,1.7562000255763525746743880517897196114063262939453125f,-0.00430000390559392986489228860591538250446319580078125f,1.0829000364899632469217749530798755586147308349609375f,1.45979996744990359047733363695442676544189453125f,0.997400017662346982660892535932362079620361328125f,-0.3461999616503721455273989704437553882598876953125f,-1.2052000047966828777390446703066118061542510986328125f,-1.36949997852742821891069979756139218807220458984375f,-0.80409999605417237233950800145976245403289794921875f,-1.309599986495076695547368217376060783863067626953125f,-1.394400015598535436112115348805673420429229736328125f,-0.17249999515712222120100705069489777088165283203125f,0.5298999679483478164598864168510772287845611572265625f,0.251300030973554300306460618230630643665790557861328125f,-0.475599982166290402574304607696831226348876953125f,0.58100000757351499824920892933732829988002777099609375f,-0.57629998695924899987375056298333220183849334716796875f,0.1882999690443269980022478193859569728374481201171875f,0.073900000533461740470642098443931899964809417724609375f,-0.987600022608041694383018693770281970500946044921875f,0.997699971343577463045448894263245165348052978515625f,1.6128999755814674710308054272900335490703582763671875f,1.50499995153397403413464417099021375179290771484375f,-0.072999993085861358821375688421539962291717529296875f,-0.2707000047743319814941287404508329927921295166015625f,0.6749000044748176829756403094506822526454925537109375f,1.416799986582994552719583225552923977375030517578125f,0.4405000078752634440348856514901854097843170166015625f,0.7045000420063745050214265575050376355648040771484375f,0.695900020784139261564860134967602789402008056640625f,0.5219999686926597437519603772670961916446685791015625f,-0.00649999281764002123651380316005088388919830322265625f,0.008600000733137047603804603568278253078460693359375f,0.76220002635121364420456302468664944171905517578125f,-0.43550000657141285653750628625857643783092498779296875f,0.67530001328587452302798510572756640613079071044921875f,0.64689994973540365652553418840398080646991729736328125f,-0.19509998814910634035157954713213257491588592529296875f,0.11380007757693544334642865578643977642059326171875f,0.7994999952539794119132920968695543706417083740234375f,0.9012999698042876151049540567328222095966339111328125f,0.74600000308453984221301880097598768770694732666015625f,-0.32450003086775591754076231154613196849822998046875f,0.01930004466027035459063654343481175601482391357421875f,0.3708999982833851571939476343686692416667938232421875f,-1.472900055563449495110717180068604648113250732421875f,-0.5311000106796621711424677414470352232456207275390625f,-0.099899971841276080652960445149801671504974365234375f,-1.3107999973565334084213418464059941470623016357421875f,1.05320003800392214543535374104976654052734375f,0.97519997413754477832981137908063828945159912109375f,1.103000057682395951275111656286753714084625244140625f,-0.18799998176097931779082728098728694021701812744140625f,-0.0950999684795730271247293785563670098781585693359375f,0.039700022737681361917339017963968217372894287109375f,-0.0047000051915639939892344045802019536495208740234375f,-0.40389997408390054811633262943360023200511932373046875f,0.39270003798902120006886207193019799888134002685546875f,0.36480000831633796476438647005124948918819427490234375f,-0.09020000349283208151973667554557323455810546875f,0.301300012347102352805450209416449069976806640625f,0.1683000458039345392791119593312032520771026611328125f,1.4081999769464139671271141196484677493572235107421875f,-1.3801000613138081352104791221790947020053863525390625f,-0.4721999837711463587908156114281155169010162353515625f,1.52750004433095565303801777190528810024261474609375f,0.6406999842479823570329244830645620822906494140625f,1.101700020860135964539949782192707061767578125f,-0.5068000071465963429773182724602520465850830078125f,0.6724000029474492823311493339133448898792266845703125f,-0.04999997973441960841256559433531947433948516845703125f,-0.6411999778181314280089964086073450744152069091796875f,-0.63140002124607608191553254073369316756725311279296875f,-0.1822999877154825032476992419105954468250274658203125f,-0.010900052821634975686038160347379744052886962890625f,0.104000089935957884534900586004368960857391357421875f,1.324899980856478531876518900389783084392547607421875f,-0.1339999853670601481780977337621152400970458984375f,1.058100045011938217243141480139456689357757568359375f,-1.073399994073807928174346670857630670070648193359375f,-1.12550004434585471102536757825873792171478271484375f,0.6404000326529153408472438968601636588573455810546875f,-1.351500017888845217584048441494815051555633544921875f,0.02640000817030685453801197581924498081207275390625f,-1.04530001593083188282662376877851784229278564453125f,1.2676999960422528079817539037321694195270538330078125f,-0.57099998930096607097084415727294981479644775390625f,0.1687999987497923459756066222325898706912994384765625f,0.492699962812662040079203507048077881336212158203125f,-0.0599999916926028387109681716538034379482269287109375f,-1.04990003695935030236796592362225055694580078125f,-0.34189999864846498667247942648828029632568359375f,0.2055000006854534877476226029102690517902374267578125f,-1.8283000187024482929842861267388798296451568603515625f,0.517200053620339161852825782261788845062255859375f,1.073600003862380081187666291953064501285552978515625f,-0.4320000135824084708247028174810111522674560546875f,-1.31060001278817583880709207733161747455596923828125f,-1.039299974736570764122234322712756693363189697265625f,-2.08930001154243871752669292618520557880401611328125f,-0.66949997305124941959775242139585316181182861328125f,1.9838000541076066252088594410452060401439666748046875f,0.65289999934881903786987322746426798403263092041015625f,0.93710002344995702028285222695558331906795501708984375f,0.0747999860763552959497246774844825267791748046875f,0.61139999563470415200328034188714809715747833251953125f,-0.0808000712662941822372886235825717449188232421875f,-1.6251999917581663179788620254839770495891571044921875f,0.65760006198436027347042909241281449794769287109375f,0.59949998068809506879262016809661872684955596923828125f,0.5064999970272179297126058372668921947479248046875f,0.27919996945113056963094777529477141797542572021484375f,-0.1515999458372598862609947900637052953243255615234375f,-0.624399958527088738691190883400849997997283935546875f,-0.3840000256747011508906552990083582699298858642578125f,0.433200014124810905968843144364655017852783203125f,-0.3123999960556631538821648064185865223407745361328125f,1.819600010560453728913898885366506874561309814453125f,-0.70570001233667112305880664280266501009464263916015625f,-1.21670000230818953212974520283751189708709716796875f,0.43000003740191505841039543156512081623077392578125f,-0.294000017553567971617667353712022304534912109375f,0.55459998206198213299700228162691928446292877197265625f,-0.427800014328956645925927659845910966396331787109375f,0.12990004865676230139115432393737137317657470703125f,1.203199999745190762467927925172261893749237060546875f,0.7771999874219293058530411144602112472057342529296875f,0.41769997324347485356810238954494707286357879638671875f,-0.6636999818786974980611148566822521388530731201171875f,-0.02410002421736745503721977001987397670745849609375f,-0.49640000610277057457864202660857699811458587646484375f,0.0289999982938170808299815917052910663187503814697265625f,-1.6260000513270498334605917989392764866352081298828125f,0.295899985393880182726888961042277514934539794921875f,0.8749999924004081552908473895513452589511871337890625f,0.6019000387325885714773221479845233261585235595703125f,-0.1488999964728947578151974084903486073017120361328125f,0.9666000453397634029073515193886123597621917724609375f,-0.038400026781857887758775405018241144716739654541015625f,-0.732199999454616889948965763323940336704254150390625f,0.457199976581335487679780271719209849834442138671875f,-0.6182000224322081738392853367258794605731964111328125f,0.066900043687225352329051020205952227115631103515625f,-0.16159996293634237840564082944183610379695892333984375f,1.0235999919414531245109856172348372638225555419921875f,-0.2246999936431650635171308749704621732234954833984375f,1.3042999845713378892497757988166995346546173095703125f,-1.2492000162780290128949900463339872658252716064453125f,0.084699990737438224375210893413168378174304962158203125f,0.14059998961687103502526952070184051990509033203125f,0.0832000215753918581640391494147479534149169921875f,0.23199997744709144331665129357133992016315460205078125f,0.2930999830976130926529776843381114304065704345703125f,1.421400085321069983734787456342019140720367431640625f,1.962200051012635615421686452464200556278228759765625f,0.92320005409717520450385563890449702739715576171875f,0.91729995117038487517646672131377272307872772216796875f,0.6564000415489079642838987638242542743682861328125f,-0.314999971389771094010257002082653343677520751953125f,-0.114300007332861497388876159675419330596923828125f,-0.81109998735934463187646770165883935987949371337890625f,1.1963999974787233870898717214004136621952056884765625f,0.25330002660006289971761361812241375446319580078125f,1.107899972228705731680520329973660409450531005859375f,-0.447900066776573790860993540263734757900238037109375f,-1.154600055971741312532685697078704833984375f,0.9084000144138928423132028910913504660129547119140625f,1.0061999774456023004631788353435695171356201171875f,0.4364999977722761048681832107831723988056182861328125f,-0.7088999852970248038985801031230948865413665771484375f,0.0854999779611824717306944876327179372310638427734375f,1.5238999557554719910257290393928997218608856201171875f,-0.732400045564770341144367193919606506824493408203125f,0.022299999673664583355048307566903531551361083984375f,0.899199977274239614644102402962744235992431640625f,-0.34910000644773242495233489535166881978511810302734375f,0.2219999912679195830378375831060111522674560546875f,-0.1814000069037080553613350275554694235324859619140625f,-0.51000001609325396056959789348184131085872650146484375f,-1.2329999920427792137189726418000645935535430908203125f,0.10239999709874314515190008023637346923351287841796875f,0.44750001586973697786930870279320515692234039306640625f,-0.287299945805967293921412419877015054225921630859375f,0.235899979321658737063671651412732899188995361328125f,0.2927000004380932995928787931916303932666778564453125f,0.64939996851086656537432872937642969191074371337890625f,0.108900006344914535016954459933913312852382659912109375f,-0.0157000086858871679140747801284305751323699951171875f,-1.065299986016750022344012904795818030834197998046875f,-0.2900999578624967600859463345841504633426666259765625f,-1.289799980562924996974061286891810595989227294921875f,-0.1997999879390006672252866337657906115055084228515625f,-0.615699995312094738864061582717113196849822998046875f,0.6956999390602105481917760698706842958927154541015625f,-0.62019997726678877114636634360067546367645263671875f,-1.275100025327503328043121655355207622051239013671875f,0.2187000617861747375769709833548404276371002197265625f,1.8387999841839082204586475199903361499309539794921875f,0.8614000597283244520241396458004601299762725830078125f,-0.210099990049004592407300151535309851169586181640625f,-0.0640999848037957509205853057210333645343780517578125f,0.5664999872669576053141327065532095730304718017578125f,0.880600020276009143316287008929066359996795654296875f,0.1741999928832054234106863077613525092601776123046875f,1.2041000442966822703283469309099018573760986328125f,0.35380001838207253417323272515204735100269317626953125f,0.441300001543760256339510306133888661861419677734375f,1.5642999641194930138254903795314021408557891845703125f,1.6062999935343873492143984549329616129398345947265625f,-1.4114999980330475981560311993234790861606597900390625f,-1.04219998663961899154628554242663085460662841796875f,-0.00129997636079759448790582609944976866245269775390625f,0.03499999552965160631856633699499070644378662109375f,0.07850000558048486620776884592487476766109466552734375f,1.086900005540251346047853076015599071979522705078125f,0.726399972966313089983714235131628811359405517578125f,0.2748000008657573989268030345556326210498809814453125f,0.739899987822770555112583679147064685821533203125f,0.70379993888437730742424491836572997272014617919921875f,-0.1672000001996754026123426228878088295459747314453125f,-0.00460009858906340607376250773086212575435638427734375f,0.17919998617768317927811949630267918109893798828125f,-1.101900044283271373757315814145840704441070556640625f,0.00319996949583301049102601609774865210056304931640625f,-1.14160001080632245162860272102989256381988525390625f,-1.283699997003376669368890361511148512363433837890625f,-0.1430999935656789290305823669768869876861572265625f,-0.654999988675117439385076067992486059665679931640625f,0.0483999994941048949925743727362714707851409912109375f,-0.306499983392656272229714886634610593318939208984375f,0.491799978238344426273442877572961151599884033203125f,0.7722000478461392702911325613968074321746826171875f,-0.139000012524427118965064664735109545290470123291015625f,-0.1590000630393617608859813117305748164653778076171875f,0.7397000197574492563745707229827530682086944580078125f,0.86459997707009306200376386186690069735050201416015625f,1.48500006746500812226940979599021375179290771484375f,-0.3719999823272239769522684582625515758991241455078125f,-0.245700030255317614091836730949580669403076171875f,-0.757799963814020482999467276385985314846038818359375f,-0.405400006128847412156801510718651115894317626953125f,0.822099994727967686714009687420912086963653564453125f,-0.06739999454691913971515759840258397161960601806640625f,-0.26139997128993208708180873145465739071369171142578125f,0.7013000459492200189259847320499829947948455810546875f,0.76789994458705290725220038439147174358367919921875f,1.552600023241340831958723356365226209163665771484375f,-0.2404000221475952603356063264072872698307037353515625f,-0.56219999014139165893766403314657509326934814453125f,-0.87660000931620551956058307041530497372150421142578125f,0.311500067099929278668923871009610593318939208984375f,0.64349997425079308310813530624727718532085418701171875f,-0.0133999672889706378242635764763690531253814697265625f,0.5772999812707302336178827317780815064907073974609375f,-0.306700003015994926425236144496011547744274139404296875f,-0.80229997158497479148309139418415725231170654296875f,1.39819999952167162149407886317931115627288818359375f,-0.025799998176097904689640927244909107685089111328125f,0.42320002090483932466469241262529976665973663330078125f,-1.1284000007420775801847412367351353168487548828125f,-2.170500023595988370317400040221400558948516845703125f,-1.4260999811381100954577050288207828998565673828125f,-0.7595000277012589062763936453848145902156829833984375f,0.8110000271871682375746104298741556704044342041015625f,-0.81039996865242758250502674854942597448825836181640625f,-0.4428000078469513933754342360771261155605316162109375f,-0.84239998186230646837202584720216691493988037109375f,0.64629999543428351405083276404184289276599884033203125f,0.67609999603927162237226866636774502694606781005859375f,-0.2495999899595970905608055545599199831485748291015625f,-0.19709994819909348873210319652571342885494232177734375f,-0.3780000202432274303276926730177365243434906005859375f,-0.9357999855473639883030045893974602222442626953125f,-0.5453999917492267091034818804473616182804107666015625f,0.4800000892207025859903524178662337362766265869140625f,-0.1561999866843226669743671664036810398101806640625f,0.80630004449635794827599966083653271198272705078125f,1.2864999650269748343589526484720408916473388671875f,-0.2882000108465547771885439942707307636737823486328125f,0.9066000274956234061818349800887517631053924560546875f,0.3947999747328465591778012822032906115055084228515625f,0.31390005000531695511511998120113275945186614990234375f,-1.0823000368520616643053244843031279742717742919921875f,-0.81960007802546031019375050163944251835346221923828125f,-1.1492000021219244132453241036273539066314697265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.157700022789835969660998671315610408782958984375f,1.016699980664253377682371137780137360095977783203125f,0.7439000131994479847463708210852928459644317626953125f,0.00849999988079108703686870285309851169586181640625f,0.16129995788335771411681207609944976866245269775390625f,-0.0805999835073947235031255331705324351787567138671875f,-0.2361999913036825216039460428874008357524871826171875f,-0.7259999851286409455752846042742021381855010986328125f,-0.6014000055253505561125848544179461896419525146484375f,-0.122099987798929277005299809388816356658935546875f,-0.6497000282883647059861687012016773223876953125f,-0.3275000008195636524988003657199442386627197265625f,-0.918699975731968709879993184586055576801300048828125f,-0.6022999970287088444820255972445011138916015625f,-0.115099970230459813791412670980207622051239013671875f,-1.0793000357568269809149796856218017637729644775390625f,0.439199991504848286094642162424861453473567962646484375f,0.7694999870955943066519466810859739780426025390625f,-0.13939998773336359505492509924806654453277587890625f,-1.6020000161528589188719706726260483264923095703125f,-0.62089995954036769632011782960034906864166259765625f,0.455900007522106687218865772592835128307342529296875f,0.906099983894824934083089829073287546634674072265625f,0.0533999862134455771656149636328336782753467559814453125f,-0.658999971695244379787936850334517657756805419921875f,-0.53149993894994285170696457498706877231597900390625f,-0.36599997872114187202896573580801486968994140625f,-0.27020002062916770313449887908063828945159912109375f,-0.15299999796599139667563349576084874570369720458984375f,0.2469999985508624096297580763348378241062164306640625f,0.971899963518977560994471787125803530216217041015625f,0.57149998641014077005451099466881714761257171630859375f,-0.2491000197842716534779583525960333645343780517578125f,0.76420001959353722753576221293769776821136474609375f,-0.6298999675571919087957439842284657061100006103515625f,-0.66940000144243239077468388131819665431976318359375f,0.81039997099936034175016175140626728534698486328125f,1.0077999935045840462777277934947051107883453369140625f,-0.4651999981842929177133783014141954481601715087890625f,-0.896000010907650246139155569835565984249114990234375f,0.02830000048577774318658839547424577176570892333984375f,0.6310000456273552327246534332516603171825408935546875f,-1.65019999045431564610453278874047100543975830078125f,-1.239300029945373804451946853077970445156097412109375f,0.26980000261664383032211844692938029766082763671875f,0.08240000413954273739136624499224126338958740234375f,-0.892899972265958741246549834613688290119171142578125f,-0.5259999831542374604254064252017997205257415771484375f,-0.129299991388619084364819400434498675167560577392578125f,-0.387999980568885849407934074406512081623077392578125f,-0.7617999852418899653372363900416530668735504150390625f,-0.245899972951412859600850424612872302532196044921875f,-0.667000003635882965369319208548404276371002197265625f,0.069399996674060826507002275320701301097869873046875f,-0.789099989497661358228697281447239220142364501953125f,-1.213700019615888692214866750873625278472900390625f,-0.3717000171720983292544815412838943302631378173828125f,-1.169500011198222555464099059463478624820709228515625f,-0.02199999886751202726742349113919772207736968994140625f,0.6036000294178724967508742338395677506923675537109375f,-0.019899990803003253514447123961872421205043792724609375f,-0.26539998724162583432217843437683768570423126220703125f,-0.74169999765157701432372050476260483264923095703125f,0.61179999425709230553849238276598043739795684814453125f,-0.119400012245774345132076632580719888210296630859375f,-0.296099992872774553376302719698287546634674072265625f,-0.3102000047594308806964136238093487918376922607421875f,-0.203700034666061124966063289321027696132659912109375f,0.470799991172551945961544106467044912278652191162109375f,0.26929996799379607441693451619357801973819732666015625f,1.672299973037838949352362760691903531551361083984375f,0.91279998258948324973971466533839702606201171875f,-0.38530000248253326589065181906335055828094482421875f,-0.30210001000165942475206293238443322479724884033203125f,-0.244799994085729000214968209547805599868297576904296875f,-1.080000010803341847775982387247495353221893310546875f,-1.2182000087976458946315005960059352219104766845703125f,0.06850001209974287963433425829862244427204132080078125f,1.7765999799609186737825439195148646831512451171875f,1.108399995899200352056368501507677137851715087890625f,-0.311099985831976066918969081598334014415740966796875f,0.4179000086620450193919396042474545538425445556640625f,-0.305400030082464635228234328678809106349945068359375f,0.025200019362568770731058975798077881336212158203125f,-0.450599982389807252047830843366682529449462890625f,-0.706300042968988517344541833153925836086273193359375f,0.64360000006258477611709167831577360630035400390625f,0.298499982744455660821358833345584571361541748046875f,0.4178999987900253731964994585723616182804107666015625f,-0.19769999783039082785762730054557323455810546875f,-0.4126999874740839846509743438218720257282257080078125f,-0.04439998244345166877877773004001937806606292724609375f,0.14150001379102461473991070306510664522647857666015625f,-0.45439997171461621316979062612517736852169036865234375f,0.03970000453963873354723546071909368038177490234375f,0.6535000025630000042298206608393229544162750244140625f,-0.711200027886033314672431515646167099475860595703125f,-0.82040000050365879946667746480670757591724395751953125f,-0.8009000182509422582910474375239573419094085693359375f,-0.2955999918520451075210075941868126392364501953125f,-1.4767999767482284934061453895992599427700042724609375f,-0.944399967467784318131407417240552604198455810546875f,-0.6613000320166351198025722624151967465877532958984375f,0.71929997496008912793996614709612913429737091064453125f,0.00159998882710932210926557672792114317417144775390625f,-0.319700000870227707405746286895009689033031463623046875f,0.42110001690089726711363482536398805677890777587890625f,0.12659999843835834099081694148480892181396484375f,-0.664799990621208980456913195666857063770294189453125f,0.047999988466500997930097582866437733173370361328125f,0.7033000036150216516261934884823858737945556640625f,0.574199993684143095151739544235169887542724609375f,0.25449995466694230739079785053036175668239593505859375f,-0.52810001956820495916389290869119577109813690185546875f,0.2514000013530253152538307404029183089733123779296875f,0.1775999782025814710539890484142233617603778839111328125f,0.556800000254809912547671046922914683818817138671875f,0.32180000319778923056901476229541003704071044921875f,0.14149998145550479744514404956134967505931854248046875f,-1.2277999976955358274466334478347562253475189208984375f,-1.78849997916817660126298505929298698902130126953125f,-1.815600016438960739861840920639224350452423095703125f,-0.294000008612871399638066804982372559607028961181640625f,-0.67630000623762620559631386640830896794795989990234375f,-0.107299985331297254020910258986987173557281494140625f,-0.146299994093179730469955757143907248973846435546875f,-0.16479999173879633644901332445442676544189453125f,-0.23949997875094408694707226459286175668239593505859375f,-0.3494999984018500871485457537346519529819488525390625f,-0.9126999967873101748949693501344881951808929443359375f,-0.73939997920244915352583348067128099501132965087890625f,0.1579999963827434239505009827553294599056243896484375f,-0.946599982792139638121398093062452971935272216796875f,-0.451599974894523636947951672482304275035858154296875f,0.09469998880028729359992212266661226749420166015625f,-0.103200010362267224905963303172029554843902587890625f,-0.5721000121623280865179594911751337349414825439453125f,-0.7540999868527056282374587681260891258716583251953125f,-1.3493999919429422451599975829594768583774566650390625f,-0.8208999994009731349109415532439015805721282958984375f,-0.4936999977484344270806104759685695171356201171875f,-0.5591000102847816233264666152535937726497650146484375f,0.29860000509172646143696283616009168326854705810546875f,0.2476999996185302865825406115618534386157989501953125f,0.36160000641047940206362909520976245403289794921875f,0.10679996072947961494037372176535427570343017578125f,-0.03720001271665029918267464381642639636993408203125f,-0.211100035005807651344866826548241078853607177734375f,1.014599992828070895711789489723742008209228515625f,0.2612999918393785758752301262575201690196990966796875f,0.53960002464205036876165877401945181190967559814453125f,0.19370000037476398091484952601604163646697998046875f,0.2414000247478487182206663419492542743682861328125f,0.00220001197159280792448043939657509326934814453125f,-0.20340000349879261420937837101519107818603515625f,0.28280000501573088200757410959340631961822509765625f,0.449400018876791040867146875825710594654083251953125f,-0.42879997166693240018275901093147695064544677734375f,-0.1887999610871080591323334374465048313140869140625f,0.711800016236305399530692739062942564487457275390625f,-0.19190001191049821382961226845509372651576995849609375f,-0.2665999975629149698619357877760194242000579833984375f,0.5843999951839446982404524533194489777088165283203125f,-0.169100007081031566968931656447239220142364501953125f,-0.49970002873539909327149644013843499124050140380859375f,0.07069996506273745762172211470897309482097625732421875f,1.247099978858232649514548029401339590549468994140625f,1.864999950379133242250873081502504646778106689453125f,1.0781999875076113681870992877520620822906494140625f,0.208000016555190736511349314241670072078704833984375f,-0.81480000514984141091190394945442676544189453125f,0.011799967733025606975161281297914683818817138671875f,0.318400033301114715467150517724803648889064788818359375f,-0.94149998722970440478974296638625673949718475341796875f,0.56480000425577159273160532393376342952251434326171875f,-0.0179999738633636052753672629478387534618377685546875f,-0.709200022760033466084905739990063011646270751953125f,0.6541000204175713150789306382648646831512451171875f,0.782999996289611477351400026236660778522491455078125f,0.63149998268485063590560457669198513031005859375f,0.58119998117089277656077683786861598491668701171875f,0.67619998839795503187133363098837435245513916015625f,0.16320000939369183612370761693455278873443603515625f,-0.87379996941313164970921434360207058489322662353515625f,-0.5785999810077251392925745676620863378047943115234375f,0.143499990120530274140264737070538103580474853515625f,1.4410000099763269343355887031066231429576873779296875f,1.499800008614361246372936875559389591217041015625f,0.790400012201070811812542160623706877231597900390625f,0.25589996140301263238825413282029330730438232421875f,0.1710000107586377993129644892178475856781005859375f,-0.9499999768286946366657730322913266718387603759765625f,-0.035099973434209508216241601985529996454715728759765625f,0.00679997004270582738882922058110125362873077392578125f,0.3692999798029659164200211307615973055362701416015625f,0.51660001054555149124780655256472527980804443359375f,1.246999983370304487806379256653599441051483154296875f,0.44709998824596386413077198085375130176544189453125f,0.20130001890361359784265005146153271198272705078125f,-0.7733000082343810532847783179022371768951416015625f,-0.60590001735687259998286435802583582699298858642578125f,0.5284000195920468545551784700364805757999420166015625f,0.05600000964105145140337072007241658866405487060546875f,-0.2048000291407108541807957635683123953640460968017578125f,0.8881000091746447111518136807717382907867431640625f,1.1960000138878823516819238648167811334133148193359375f,0.64350000666081907940707651505363173782825469970703125f,-0.209300009679794118966356109012849628925323486328125f,0.372500033676624209277861154987476766109466552734375f,-0.242599981331825365060694821295328438282012939453125f,-0.27839996840655789611673753825016319751739501953125f,-0.63619999573677776805169514773297123610973358154296875f,0.2627000028036532430775196189642883837223052978515625f,-0.0022000002741815460893803901853971183300018310546875f,-0.8808999996989970160399252563365735113620758056640625f,-0.909599994653463195248832562356255948543548583984375f,0.6263999827265742670334702779655344784259796142578125f,1.1301999925218517706326792904292233288288116455078125f,1.387100025144964110523915223893709480762481689453125f,1.020799975600839015754672800539992749691009521484375f,1.1969999875426291513491605655872263014316558837890625f,0.45600000362470727299779582608607597649097442626953125f,-0.1579000004291533088007781771011650562286376953125f,-0.60880001562535779324747409191331826150417327880859375f,-0.70709999524950983840199114638380706310272216796875f,-0.18600001049786818452247416644240729510784149169921875f,-0.0292000201709574636055322116590104997158050537109375f,0.6768999919056895020474939883570186793804168701171875f,-0.185900012886524024224854656495153903961181640625f,-0.543900001241266917162420213571749627590179443359375f,-0.5688999947592614514491060617729090154170989990234375f,-0.8643999951094389899708403390832245349884033203125f,-0.1375000064074989314377717164461500942707061767578125f,-0.410900011843442936054771053022705018520355224609375f,0.35519999194443219181493986980058252811431884765625f,0.54810001628994908795533547163358889520168304443359375f,-1.1357000021480023743691845083958469331264495849609375f,-1.0895000085532664524379242720897309482097625732421875f,0.1702000052064655821748573316654074005782604217529296875f,0.6538999983355402445539539257879368960857391357421875f,1.293400001934170706618942858767695724964141845703125f,0.069000008314848371160366014009923674166202545166015625f,-0.73790002643167984519578794788685627281665802001953125f,0.482499983012676381122219027020037174224853515625f,1.22600000703334810481237582280300557613372802734375f,0.162100004450976886705149127010372467339038848876953125f,-0.6839000176697973554240661542280577123165130615234375f,-0.17939999980330478823731255033635534346103668212890625f,0.8313000149548053219206167341326363384723663330078125f,0.361200023713708240347131095404620282351970672607421875f,0.11039996995627887610424977538059465587139129638671875f,0.463499987810850111458194078295491635799407958984375f,0.726799978610873420592497495817951858043670654296875f,0.95419999884367001641294336877763271331787109375f,-0.45599998296797306096550528309307992458343505859375f,-0.67939998505115450910807339823804795742034912109375f,-0.6056000384926800705898131127469241619110107421875f,1.0872999944955112905375926857232116162776947021484375f,-0.37019999105036216491271261475048959255218505859375f,-1.0904999969415367644387515611015260219573974609375f,-0.53059999242946498387851761435740627348423004150390625f,-0.751400017148256438304088078439235687255859375f,0.54629998790919820361722258894587866961956024169921875f,1.58719998544752538549573728232644498348236083984375f,0.449400007700920145481404688325710594654083251953125f,0.1116000000029802041723314687260426580905914306640625f,0.5363999959141014794994362091529183089733123779296875f,-0.80919999394491293198683479204191826283931732177734375f,-0.512200024786591878722674664459191262722015380859375f,0.375900013929605592011284898035228252410888671875f,0.18919997321367276743586671727825887501239776611328125f,-0.2816999831229447526226294939988292753696441650390625f,-0.5335000195503230013827078437316231429576873779296875f,0.632299986970424665599921354441903531551361083984375f,-0.10199998702108870052285283236415125429630279541015625f,-0.9307999950468541783266118727624416351318359375f,-0.624199994000792290904655601480044424533843994140625f,-0.3164999855905772374597972884657792747020721435546875f,-0.879699982988834339181494215154089033603668212890625f,0.027100000539422186562887873151339590549468994140625f,-0.5583999900132419202947176017914898693561553955078125f,0.51429999507665702385139638863620348274707794189453125f,0.82939999041557310732741825631819665431976318359375f,-0.79259998475909199111555381023208610713481903076171875f,0.0120000063925982392021296618622727692127227783203125f,-0.42989997836053372992637378047220408916473388671875f,-1.59059998537003988161586676142178475856781005859375f,-0.398300007861852822799164641764946281909942626953125f,-0.152500001117587213883552976767532527446746826171875f,-0.35220001368522613205414018011651933193206787109375f,0.655300027218461256239834256120957434177398681640625f,0.67439999503493286425737096578814089298248291015625f,0.82830003937780871670071292101056315004825592041015625f,0.5980000059008594437415240463451482355594635009765625f,0.67749999672174465725760228451690636575222015380859375f,0.1146999966979029839109216482029296457767486572265625f,0.26510000442862491354389931075274944305419921875f,-0.77989997061192983807131895446218550205230712890625f,-1.42449997837841468850683668279089033603668212890625f,-1.12380003246367010660833329893648624420166015625f,-0.5474000048473477431087985678459517657756805419921875f,-0.060199969853460544300105539150536060333251953125f,-0.3593999972701080736214862554334104061126708984375f,-0.72780000758618135225930245724157430231571197509765625f,-0.389100009297579763067886915450799278914928436279296875f,0.3702000063240529215136120910756289958953857421875f,0.01199998858571084403568107745741144753992557525634765625f,0.51939999795928593062654954337631352245807647705078125f,0.10639997659847189037662928967620246112346649169921875f,0.48379997226297855039689466138952411711215972900390625f,0.663499996453523532835561127285473048686981201171875f,0.1404999806433913089875886726076714694499969482421875f,1.3995999652981756522507339468575082719326019287109375f,0.9231999912515280737324019355583004653453826904296875f,0.169300005656480845317446437547914683818817138671875f,-0.238800011155009261898385375388897955417633056640625f,0.7443999971956021255437008221633732318878173828125f,0.8891999717980623163526843200088478624820709228515625f,0.07929997130930421800343310678726993501186370849609375f,1.043800018605589574605119196348823606967926025390625f,0.814300051403045177522699304972775280475616455078125f,-0.608499988034367333966656588017940521240234375f,0.0131999753445388723349651627358980476856231689453125f,0.3313000063121318561343287001363933086395263671875f,0.094199984166026151655159992515109479427337646484375f,0.084800018188357473292171562206931412220001220703125f,0.2560999525278810917683358638896606862545013427734375f,0.612099973791837559389250600361265242099761962890625f,-0.75540000415444363834893692910554818809032440185546875f,0.449300019365548752858785519492812454700469970703125f,1.29500001028180111717347244848497211933135986328125f,0.6547000232219699622504549552104435861110687255859375f,0.125000001452863163109441302367486059665679931640625f,-0.686699998843669501269459942704997956752777099609375f,-0.4533000209003688052433744815061800181865692138671875f,0.41339995648562943841852757032029330730438232421875f,0.096899990080296927175140808685682713985443115234375f,-1.2058000099852679287693035803386010229587554931640625f,-0.206500024929642478355162893421947956085205078125f,0.509600019910930956967831662041135132312774658203125f,-0.010799978879094229711199659504927694797515869140625f,0.62360000609755505962539245956577360630035400390625f,0.087400006300210719700771733187139034271240234375f,0.262000030830502605994070108863525092601776123046875f,-1.12229999896586019048072557779960334300994873046875f,-0.091300011825560911660204510553739964962005615234375f,-0.0968000145971779080156238705967552959918975830078125f,-0.3770999944299460171492910376400686800479888916015625f,0.72390000686645483707337689338601194322109222412109375f,0.60760001947879782591144248726777732372283935546875f,0.132300011259317340517327465931884944438934326171875f,-0.412400001084804568307617955724708735942840576171875f,0.29359999443739626112659379941760562360286712646484375f,0.1269999823458489718319697203696705400943756103515625f,-0.450899990758299562543243155232630670070648193359375f,-0.4519000294566148934194416142418049275875091552734375f,1.14929998755156947964906066772527992725372314453125f,1.096300020393729202083932250388897955417633056640625f,-0.1446000309750441470413306888076476752758026123046875f,0.51700000333786011452730235760100185871124267578125f,0.0145999879479408178895027958787977695465087890625f,-0.2222000001251700496140983887016773223876953125f,0.012800027069449271266421419568359851837158203125f,-0.048399991056322644311649128212593495845794677734375f,0.15979999211132478453833982712239958345890045166015625f,-0.58049997334182279473679955117404460906982421875f,-0.16959999584555607299307666835375130176544189453125f,1.0559999914616344796058911015279591083526611328125f,0.4458000124812127040740961092524230480194091796875f,-0.012900005793571533985186761128716170787811279296875f,0.631700014191865388823998728184960782527923583984375f,-0.644499972268939291808464986388571560382843017578125f,-0.81429998390078572878536533607984893023967742919921875f,-0.043499976336955514799598176978179253637790679931640625f,-0.99970003577619792611841376128722913563251495361328125f,0.3716999939262866092803960782475769519805908203125f,-0.748500034935772884381322000990621745586395263671875f,0.28190000963807104739089481881819665431976318359375f,0.866300004225969200177814855123870074748992919921875f,0.369100018145144037617910726112313568592071533203125f,-0.387199991296231671444871835774392820894718170166015625f,-0.01259998192787159521088824476464651525020599365234375f,-0.2412999844074246169611797085963189601898193359375f,-0.8391999957144260502417409952613525092601776123046875f,-0.9261999980092046058643973083235323429107666015625f,0.1993999986112118261161185728269629180431365966796875f,0.938400020039081628198118778527714312076568603515625f,-0.6538000139117243758590802826802246272563934326171875f,-1.4924000133410098012376465703709982335567474365234375f,-1.041000032663345731265280846855603158473968505859375f,-0.742099992567301303125759659451432526111602783203125f,-0.57119998005330552270919497459544800221920013427734375f,1.0417000005573033849515240945038385689258575439453125f,0.9410000220090151135110545510542578995227813720703125f,0.03129997680783258573455896112136542797088623046875f,0.598199994343519225736827138462103903293609619140625f,0.61309998440146495912728141775005497038364410400390625f,0.1223999876178805401583105094687198288738727569580078125f,-1.04539999804496819280075214919634163379669189453125f,0.100200036871433029173061868277727626264095306396484375f,0.85810001614093789346071616819244809448719024658203125f,0.311599986591935163016131582480738870799541473388671875f,-0.19589998420178933447033386983093805611133575439453125f,-1.0148999829053872989703677376382984220981597900390625f,0.143399999773502262456759126507677137851715087890625f,0.4273000158041717622126043352182023227214813232421875f,-0.038000004447996449385982487001456320285797119140625f,0.150600018599628793225519984844140708446502685546875f,0.213399979656934579708149612997658550739288330078125f,0.5398000213921070322697914889431558549404144287109375f,-0.7874999827891588921602306072600185871124267578125f,-0.3515999939680101160632830215035937726497650146484375f,0.68930001504421223312846223052474670112133026123046875f,-0.112499996609985775108953021117486059665679931640625f,-0.32619999148994682780511311648297123610973358154296875f,0.426900001628696290634223942106473259627819061279296875f,1.193900031453370758072196622379124164581298828125f,0.80120001194179091275060500265681184828281402587890625f,0.2958999766767027583824756220565177500247955322265625f,0.17820001729130741097151258145458996295928955078125f,-0.5507000075310470865730394507409073412418365478515625f,-0.26269999555796375023675182092119939625263214111328125f,-0.1337000026099383898081640609234455041587352752685546875f,-0.0116999516814946247222906094975769519805908203125f,-0.29210000985264716888423208729363977909088134765625f,0.739499970927834482381513225845992565155029296875f,0.66429998993575534171895924373529851436614990234375f,0.962900044834614110556003652163781225681304931640625f,0.464500004008411959688373826793394982814788818359375f,0.14760003855228454039405505682225339114665985107421875f,-0.467500000223517897524061481817625463008880615234375f,-0.658199996727705016752452138462103903293609619140625f,-0.681999998196959555940566133358515799045562744140625f,-0.5826000080049038576390785237890668213367462158203125f,1.053600016826390817215042261523194611072540283203125f,0.1694999945461745927133279110421426594257354736328125f,0.636600027644634192114381221472285687923431396484375f,0.235299974393844646414208909845910966396331787109375f,0.4769000237941745101721835453645326197147369384765625f,-0.401600020790100042944459346472285687923431396484375f,0.1812999973714350954168139651301316916942596435546875f,0.4051000045403834182167202015989460051059722900390625f,0.8664000063076622293323225676431320607662200927734375f,0.067199989396333759117396766669116914272308349609375f,0.44279999659657420796321503075887449085712432861328125f,1.10600002454221257863764549256302416324615478515625f,1.2696000016570085922040789228049106895923614501953125f,0.3498999992966655980808354797773063182830810546875f,-0.2281000201642513491862018781830556690692901611328125f,0.659900008982420072101149344234727323055267333984375f,-0.435800012183189533487848166259936988353729248046875f,0.16220001558512453510019213354098610579967498779296875f,-0.1817999963432550547537402962916530668735504150390625f,1.00769998754859013700979630812071263790130615234375f,1.1354999976307151854371113586239516735076904296875f,0.0644999987930059415219830043497495353221893310546875f,-0.780799998027086328278301152749918401241302490234375f,0.05310001167058953086552719469182193279266357421875f,0.5864999937117101325156909297220408916473388671875f,-0.379300014373659788446957463747821748256683349609375f,0.7102999909341338735657700453884899616241455078125f,0.913199991884827255717027583159506320953369140625f,0.4818999741360550093105530322645790874958038330078125f,0.588699991042911818794891587458550930023193359375f,-0.062300001946091809745809086962253786623477935791015625f,0.53769999499917020546746471154619939625263214111328125f,0.473200020048022196306192199699580669403076171875f,-0.213799993050098446900619819643907248973846435546875f,-0.1212000106751920380787623798823915421962738037109375f,-0.176600012631714431421414701617322862148284912109375f,0.0236000502794981859011613778420723974704742431640625f,-0.53629999202564360150091715695452876389026641845703125f,-0.54519999021217191437216342819738201797008514404296875f,0.180399996256828121232729245093651115894317626953125f,0.1029999899193645429562593562877736985683441162109375f,0.654199979621172378330129504320211708545684814453125f,0.57009997902214515352881107901339419186115264892578125f,-0.0830000021010637301088763706502504646778106689453125f,-0.5112000042676922983986287363222800195217132568359375f,-0.8878000139564277137793624206096865236759185791015625f,-0.6297000010937450209524968158802948892116546630859375f,0.018500003308057788586893366300500929355621337890625f,-0.9075999917626382806901119693065993487834930419921875f,0.3515999927759170251562181874760426580905914306640625f,-0.04230002288222234341219518682919442653656005859375f,0.252899986124038633761301753111183643341064453125f,-0.486099975252151761395680296118371188640594482421875f,-0.3307999898314477604799321852624416351318359375f,0.4132999812632800473721772505086846649646759033203125f,0.58229997855126836014960645115934312343597412109375f,0.499499985381960609487350666313432157039642333984375f,1.0401000039815908593254789593629539012908935546875f,0.68920003877878155407188387471251189708709716796875f,-0.07070000835061029942352206489886157214641571044921875f,0.6048999966144561479808317017159424722194671630859375f,-0.126300014433265328506195146474055945873260498046875f,0.7242000011906031264885541531839407980442047119140625f,0.8939999772086737461762595557956956326961517333984375f,1.2511999761790040253117695101536810398101806640625f,0.649299975258112116449638051562942564487457275390625f,-0.341999979496002470824578267638571560382843017578125f,-1.005300005760788817354978164075873792171478271484375f,0.18679998494386662599708870402537286281585693359375f,0.608500011503696480730241091805510222911834716796875f,0.408099988275766545342548852204345166683197021484375f,0.715399986198544723947634338401257991790771484375f,0.4246999920785425874925067546428181231021881103515625f,0.1988999685704710973510600524605251848697662353515625f,0.908899995392560722251573679386638104915618896484375f,0.3598999971359972960271988995373249053955078125f,0.1030999738961455580721349178929813206195831298828125f,-0.2822999873057003838994205580092966556549072265625f,-0.65840003896355625556680024601519107818603515625f,0.014600006797909514943967224098742008209228515625f,-0.945199998128413820808191303513012826442718505859375f,-0.61950000639259794610325116082094609737396240234375f,-0.763699992421269513442894094623625278472900390625f,-0.755199991571902984333064523525536060333251953125f,0.2097000175595287174701297772116959095001220703125f,-0.52210000087469821128394187326193787157535552978515625f,0.106099998367577885716173113905824720859527587890625f,0.328899988538026821771609320421703159809112548828125f,0.1071999966979021723378906472134985961019992828369140625f,0.7571000222578645466597890845150686800479888916015625f,-0.22619999778568711246151679006288759410381317138671875f,0.0120000041574239713071392543497495353221893310546875f,0.23760001247525242007441192981787025928497314453125f,-0.223300009202957117082632976234890520572662353515625f,0.5615000302940611476287813275121152400970458984375f,-1.2866000184804200490162884307210333645343780517578125f,-0.669500009149313068945730265113525092601776123046875f,0.0960000275224448806898891461969469673931598663330078125f,-0.109600038723647374983016788974055089056491851806640625f,0.67959998705387103523634095836314372718334197998046875f,-0.46709997008517400995941670771571807563304901123046875f,0.352199990811943752788693018374033272266387939453125f,1.1062999858975410827355290166451595723628997802734375f,-0.2308000344231728728772168324212543666362762451171875f,0.25379999394416774549654292059130966663360595703125f,-0.26210000270009015110872496734373271465301513671875f,-0.547199985969066826640982981189154088497161865234375f,-0.2051999831527471229719594703055918216705322265625f,0.05590000923573956725931566325016319751739501953125f,0.2328999926060435488039956908323802053928375244140625f,0.51510001763477941683078142887097783386707305908203125f,-0.449700007817894376049849824994453229010105133056640625f,0.370900021156668646682419421267695724964141845703125f,0.2865000121891501105864108467358164489269256591796875f,0.42169998617768289506102519226260483264923095703125f,-0.154500006765127206875831689103506505489349365234375f,-0.7644999970048671489308844684273935854434967041015625f,-1.346299978148937537270057873683981597423553466796875f,-0.538399971684813127836832791217602789402008056640625f,-0.5588999678254127356780145419179461896419525146484375f,-1.00699999797344208474214610760100185871124267578125f,0.18040000236630449048647051313309930264949798583984375f,-0.9616999950438740452085539800464175641536712646484375f,-0.48280000255703914291416367632336914539337158203125f,0.28219998071640761683198661557980813086032867431640625f,0.398399984630197268042905989204882644116878509521484375f,-0.18440000626668318073342334173503331840038299560546875f,0.034900019450485597616307131829671561717987060546875f,-0.460499999865889630967785706161521375179290771484375f,-1.288199998441339033661279245279729366302490234375f,-0.99339999262094469401063179248012602329254150390625f,-0.009400009489059613088102196343243122100830078125f,0.85650001844763767877566351671703159809112548828125f,0.41369999249577471545080697978846728801727294921875f,0.109799980935453955765979117131792008876800537109375f,0.44419997954666634587539419953827746212482452392578125f,0.930499997556208935378663227311335504055023193359375f,0.16620002617239937325877008333918638527393341064453125f,-0.4891000241242353663295716614811681210994720458984375f,0.9384000120669602207357229417539201676845550537109375f,-1.043900023965537382508728114771656692028045654296875f,-1.0662000167846681364380856393836438655853271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.7376999972343440958866267465054988861083984375f,0.0200000008195644340958097018301486968994140625f,1.41420003289282281144778607995249330997467041015625f,-0.0705000184923409989323772606439888477325439453125f,-0.2764000074252483951653402982628904283046722412109375f,0.01419995399117357504081837760168127715587615966796875f,0.7792000127017499888637530602863989770412445068359375f,0.1613999896556135293934630681178532540798187255859375f,-0.029199995043873716582538691000081598758697509765625f,0.82650003826618245739155099727213382720947265625f,1.6939000162914392699775589790078811347484588623046875f,0.336300009627640117404467900996678508818149566650390625f,-1.3883999789118763867890038454788736999034881591796875f,-1.7125999900490038729827801944338716566562652587890625f,0.28190001891404381506589515993255190551280975341796875f,0.251400024859607551519502521841786801815032958984375f,-0.10290004256218698852620718753314577043056488037109375f,1.064800014090538038402655729441903531551361083984375f,-0.50100002133846288643326261080801486968994140625f,0.7589000615164642749022050338680855929851531982421875f,0.2162000348895787293912462700973264873027801513671875f,0.40219998347312202469083786127157509326934814453125f,-0.3774000029847022208429052625433541834354400634765625f,0.009299993325769750907028310393798165023326873779296875f,-0.091099980281293735373537856503389775753021240234375f,0.3425000646337872378666133954538963735103607177734375f,1.37930004328191291307348365080542862415313720703125f,-0.64660001661777499748495756648480892181396484375f,-1.946100056910515352370794062153436243534088134765625f,0.000599970468878385787547813379205763339996337890625f,-0.0650000266358252698495334698236547410488128662109375f,-1.2224000486940147514047794174985028803348541259765625f,1.032699961434303759233443997800350189208984375f,-0.48680002659261234487786396130104549229145050048828125f,0.46780005716979544683198355414788238704204559326171875f,0.412000011682510436372695039608515799045562744140625f,-0.301400044119358057059798738919198513031005859375f,-0.01420003069490238800653969519771635532379150390625f,-0.8250000100582834949847210737061686813831329345703125f,-1.04079999307244985828901917557232081890106201171875f,0.17000000663101655362652309122495353221893310546875f,-1.0271000240460044228285596545902080833911895751953125f,-0.5766999939635393257475470818462781608104705810546875f,-0.238999990209936896690123830921947956085205078125f,-0.03880001178830883024062359254457987844944000244140625f,-0.87220000295639010801096446812152862548828125f,0.20980005167871718096961330957128666341304779052734375f,0.145600023442506998350154390209354460239410400390625f,1.04609998732209152905170412850566208362579345703125f,0.9921000076547261681270128974574618041515350341796875f,1.2192000451490285772848665146739222109317779541015625f,1.2483000153124326203624150366522371768951416015625f,0.70139999066144209649564800201915204524993896484375f,-0.83640001789331430703811065541231073439121246337890625f,0.245900022423267472504448960535228252410888671875f,0.53709999282807086018465270171873271465301513671875f,-0.0021000279948115707639999527600593864917755126953125f,-0.49390003823340011823717077277251519262790679931640625f,-0.69850001388788240941352114532492123544216156005859375f,-0.71620000996738664156282538897357881069183349609375f,-0.022500009462237624546787628787569701671600341796875f,0.16460003421604663120803024867200292646884918212890625f,-0.7688000092551112718552985825226642191410064697265625f,-0.04720000742673846616526134312152862548828125f,0.144700004632770884427372948266565799713134765625f,0.06730004031658232133139563302393071353435516357421875f,0.7765000091344116395219998594257049262523651123046875f,0.737400003729760999959808032144792377948760986328125f,0.438600004123151254731283188448287546634674072265625f,0.6340000066012148050020869050058536231517791748046875f,0.616799972985684785697912957402877509593963623046875f,1.0342999979451297232202477971441112458705902099609375f,0.173500009715557013123543583787977695465087890625f,-0.825599996769427857401524306624196469783782958984375f,-1.50049999841302561520706149167381227016448974609375f,-0.698300019708276042962324936524964869022369384765625f,0.0036999966308470799702945441822521388530731201171875f,0.8588999777346841835168334000627510249614715576171875f,1.7961000154852875265731881881947629153728485107421875f,0.4169000037148593573732568984269164502620697021484375f,-0.19619997267723066070033155483542941510677337646484375f,0.109899996241927144779992886469699442386627197265625f,0.2056000061385334642949374028830789029598236083984375f,-1.6827000420868400976104339861194603145122528076171875f,-0.78039997830092833197568324976600706577301025390625f,0.13069999661594611239934238255955278873443603515625f,0.08819999315142634799258303246460855007171630859375f,1.8510000253990295249195696669630706310272216796875f,0.89060001547038536440226152990362606942653656005859375f,0.10479999609738588350893451206502504646778106689453125f,0.04270001057088368412451018230058252811431884765625f,-0.713000011600553929014267851016484200954437255859375f,-0.0383999801039693888782267094939015805721282958984375f,0.187300019138306073074318192084319889545440673828125f,0.519299995225667831988403122522868216037750244140625f,-0.6957999918058506150231323772459290921688079833984375f,-0.7137000414088376931687207616050727665424346923828125f,-0.7197000063836573868769619366503320634365081787109375f,-0.81530001514852035171543320757336914539337158203125f,-0.9411999822139733851145138032734394073486328125f,-0.8358999708279968299251549979089759290218353271484375f,0.237800034035742680060820930521003901958465576171875f,1.628000044830143711038772380561567842960357666015625f,-0.134200025479495221869541410342208109796047210693359375f,-0.0385999981626869992368256134795956313610076904296875f,-0.64100000602751949774216200239607132971286773681640625f,0.51300000329315642577654443812207318842411041259765625f,0.6796000275850300198499098769389092922210693359375f,0.3454000135421750883324421010911464691162109375f,0.0789999915510417594077807734720408916473388671875f,-0.18340000089108932090908865575329400599002838134765625f,0.51330000598058111993537977468804456293582916259765625f,0.282400002686679751917608882649801671504974365234375f,-0.948600038731098660349516649148426949977874755859375f,-1.259100048133731331034823597292415797710418701171875f,-0.01790002188682560113619501862558536231517791748046875f,-0.27610001876354239858102346261148341000080108642578125f,0.6547999996364115560254504089243710041046142578125f,-0.089899994267522931323810553294606506824493408203125f,-1.962100012311339458648262734641321003437042236328125f,-2.234900031185150570678388248779810965061187744140625f,-0.1704999722987408095065120505751110613346099853515625f,-0.5815000206083060785289262639707885682582855224609375f,1.351200041595101541958001689636148512363433837890625f,0.69470002351999282641514810165972448885440826416015625f,0.72200000788271412321961406632908619940280914306640625f,-0.5927999863147732639845344237983226776123046875f,-0.2186999894782901687761977882473729550838470458984375f,0.337000021424145035364716704862075857818126678466796875f,-0.0178999947294595784796911175362765789031982421875f,1.0336999948427074702550498841446824371814727783203125f,-0.5558000198572872196933758459636010229587554931640625f,-1.0138999849617482684749347754404880106449127197265625f,0.0919999891445038020520996724371798336505889892578125f,1.0023999939694998584371887773158960044384002685546875f,-0.6801999924659725138553767465054988861083984375f,-0.860699969086050398203724398626945912837982177734375f,-0.579600013577938444342407819931395351886749267578125f,0.342300015953183045525065608671866357326507568359375f,0.1105999918520448321856974871479906141757965087890625f,0.020100026184320884947709373591351322829723358154296875f,-0.8325999894529576561552630664664320647716522216796875f,-0.157400003618002048977331241985666565597057342529296875f,0.7031999981805678867630149397882632911205291748046875f,0.323000001132488279154131305404007434844970703125f,1.8557999993681904005171645621885545551776885986328125f,-0.875700043368339731131300140987150371074676513671875f,-0.0194000007346268876773365263943560421466827392578125f,1.5484000598996885411651192043791525065898895263671875f,0.6984000125139948611519002952263690531253814697265625f,-0.3922000139087440384599858589353971183300018310546875f,-0.7608000355035071660125822745612822473049163818359375f,-0.7136000052779911673184187748120166361331939697265625f,0.7168000168323516163582098670303821563720703125f,0.9125000289455058588572455846588127315044403076171875f,0.2925000150129200005011398388887755572795867919921875f,1.13450006368756373120731950621120631694793701171875f,-0.05909994516670658004642291416530497372150421142578125f,0.7546000147327773799332817361573688685894012451171875f,1.37210002207160020049059312441386282444000244140625f,-0.96550004535168454378180058483849279582500457763671875f,-0.8556000218778849752965243169455789029598236083984375f,0.65239998886585226500756107270717620849609375f,1.4939000095486638652886313138878904283046722412109375f,-0.1026000101655719642934627700014971196651458740234375f,-0.5190000166967509986903905883082188665866851806640625f,-0.2400000520050529217996881925500929355621337890625f,1.0686999734967950193009755821549333631992340087890625f,1.584899974914639297907115178531967103481292724609375f,-0.28900001315772516630175914542633108794689178466796875f,0.1264000384569172741322518049855716526508331298828125f,0.640400014734268552274443209171295166015625f,-0.58860000535249756836719825514592230319976806640625f,0.607200041326880768366436313954181969165802001953125f,-0.3999999807029954723702758201397955417633056640625f,0.05749997306615017844677595348912291228771209716796875f,0.0006000015005473979812222751206718385219573974609375f,-0.3260000382140286756538216650369577109813690185546875f,1.89550003212690398868289776146411895751953125f,0.82830002544522329799292492680251598358154296875f,-0.6376000033855433457574690692126750946044921875f,0.1078000203639268850253074560896493494510650634765625f,-1.6033000711545357663823097027488984167575836181640625f,-1.38890002025887415726401741267181932926177978515625f,-0.78960003172010206906605844778823666274547576904296875f,1.3994000129163264301013214208069257438182830810546875f,1.7824000360280274346536089069559238851070404052734375f,0.3531999843224882251746521433233283460140228271484375f,0.1974000047728419193759918925934471189975738525390625f,0.03509998848438300456109573133289813995361328125f,-1.4638999991923571375451729181804694235324859619140625f,-0.83699999402463465258961150539107620716094970703125f,-0.921000009566545596584319355315528810024261474609375f,-0.830700008760392183404519528266973793506622314453125f,-1.7743000256612895260133200281416065990924835205078125f,-0.238900016254186997599617825471796095371246337890625f,0.436399992673099479389975385856814682483673095703125f,-0.035200000251829788311397351208142936229705810546875f,0.60690002439022094904430559836328029632568359375f,-0.049799992185830976865190677926875650882720947265625f,-0.72059998872280051074312723358161747455596923828125f,0.85250000782310930702578843920491635799407958984375f,0.4975000309199100190227227358263917267322540283203125f,0.35759999068230374774657320813275873661041259765625f,0.34069999970793707344540734993643127381801605224609375f,0.6766999942988156657719400755013339221477508544921875f,0.9086000202536583802981340340920723974704742431640625f,0.411700005996227247351271216757595539093017578125f,-1.0381000172212722088005421028356067836284637451171875f,-0.410699981230497002115953364409506320953369140625f,-0.98779999417513597581574913419899530708789825439453125f,0.7427000529646878845824176096357405185699462890625f,1.108600027331709814149007797823287546634674072265625f,0.99660000521838643106065092069911770522594451904296875f,-0.096300013911724546833426074954331852495670318603515625f,-0.796500038526952724993179799639619886875152587890625f,-0.01960001864433280616140109486877918243408203125f,0.2541000060938298776846977489185519516468048095703125f,1.08049999990314216091746857273392379283905029296875f,0.6222000101462012988662308998755179345607757568359375f,-0.7663000247150659749451051538926549255847930908203125f,0.2503000001169739530126889803796075284481048583984375f,0.8132999951213595313248561069485731422901153564453125f,0.43460001842081563694364376715384423732757568359375f,-0.87480002139210721878725962596945464611053466796875f,0.620200028862059493661718079238198697566986083984375f,0.634900015315413401850719310459680855274200439453125f,-0.584900053983927303846712675294838845729827880859375f,-1.414100010955333441842185493442229926586151123046875f,-1.340300013956427616079736253595910966396331787109375f,-0.5689999974370001378787264911807142198085784912109375f,-0.10340002637207579283540326287038624286651611328125f,-0.226800026443600533099242966272868216037750244140625f,0.060600001388788182765665624174289405345916748046875f,-0.8194000322878363107292898348532617092132568359375f,-0.533300006110966062777833940344862639904022216796875f,0.158799992174654625021190668121562339365482330322265625f,-1.5369000136241315335183799106744118034839630126953125f,-0.7432000071957707287850780630833469331264495849609375f,0.89309998160749604068087137420661747455596923828125f,0.67520001470595669701424412778578698635101318359375f,0.7644999957382676569750401540659368038177490234375f,0.5648000190451742952291169785894453525543212890625f,0.99650002612173604887857436551712453365325927734375f,0.6187000085890292933044065648573450744152069091796875f,0.29590001597851534054228750392212532460689544677734375f,-1.1008000239923598773117419113987125456333160400390625f,-1.4352000492766505690411804607720114290714263916015625f,-0.8422000343605879901787147900904528796672821044921875f,-0.531200013580918284361587211606092751026153564453125f,0.4127999900400640687081477153697051107883453369140625f,0.2289999893158671806503434709156863391399383544921875f,-0.626600057744980176721583120524883270263671875f,0.5322999923720954651429337900481186807155609130859375f,1.4228000403314826627365619060583412647247314453125f,0.4481000001594426240814073025831021368503570556640625f,1.8342000442549586392004812296363525092601776123046875f,-0.605000037141144364483125173137523233890533447265625f,-0.0158999964579942254516708999290131032466888427734375f,-0.5315999920122322919269208796322345733642578125f,0.012899999348819246591801856993697583675384521484375f,-0.329399998909234881239171954803168773651123046875f,-0.255000027827918529510498046875f,1.0713000249758366511088070183177478611469268798828125f,-0.1368000124737622780202173089492134749889373779296875f,0.1065000052601098590177031155690201558172702789306640625f,0.22619999689161784939273047712049447000026702880859375f,-0.55890002718791442770651656246627680957317352294921875f,-2.113300034199654842126392395584844052791595458984375f,-1.03950004096329262637254942092113196849822998046875f,0.051599991583824333929442218504846096038818359375f,0.1045000378340483759842527433647774159908294677734375f,-0.39659998666644025178129595587961375713348388671875f,-0.56450000665336830252272193320095539093017578125f,1.1146000602558261416419327360927127301692962646484375f,-0.1794999644085761847378535094321705400943756103515625f,0.02320003155916960935201132087968289852142333984375f,-0.496399980603158486047732367296703159809112548828125f,-0.8794999847486610899949255326646380126476287841796875f,-0.9557000071585175504651488154195249080657958984375f,-1.1329000074326993630080551156424917280673980712890625f,-0.29880003913193953213323084128205664455890655517578125f,0.015400021514297090874379136948846280574798583984375f,-0.35649998298287399745021275521139614284038543701171875f,0.324900034463406151008513234046404249966144561767578125f,-0.2206999712839721183854635455645620822906494140625f,-0.2656999432697884078180550204706378281116485595703125f,0.029100000748038201692224902217276394367218017578125f,0.53789998672008465518956654705107212066650390625f,-0.6593000183224677623883280830341391265392303466796875f,-1.4783000151634215768581270822323858737945556640625f,0.8957000464975839459924600305384956300258636474609375f,0.25229999270737124561492237262427806854248046875f,1.491800038178265452870618901215493679046630859375f,1.41679999943524581595966083114035427570343017578125f,0.8973000419557097817602198119857348501682281494140625f,-0.25479997437894297274141308662365190684795379638671875f,-0.92949997055530531042677466757595539093017578125f,-0.8782999646857374553832187302759848535060882568359375f,-0.325799989011883450729101241449825465679168701171875f,-1.45770001343935717841304722242057323455810546875f,-1.2671999777734275216545256625977344810962677001953125f,0.245900046749413458968547274707816541194915771484375f,0.535699994418024516562581993639469146728515625f,0.07629997967630564037477824967936612665653228759765625f,-0.38860002178102737158127411021268926560878753662109375f,0.06299999766796826605741443927399814128875732421875f,-0.9519000029139217389229088439606130123138427734375f,-0.64710002870261662533124535912065766751766204833984375f,-0.7524000076040622264628154880483634769916534423828125f,-1.10710000560581622863765005604363977909088134765625f,0.8883000127047300242821847859886474907398223876953125f,1.1630999732255933309943429776467382907867431640625f,1.127299992483853596780818406841717660427093505859375f,-0.1002000029712915019075580858043394982814788818359375f,-0.610600002729892565866975928656756877899169921875f,0.2481999709486959471149702949333004653453826904296875f,1.56970003722906170651185675524175167083740234375f,0.05309999662041686718794153421185910701751708984375f,0.1735999796107403625455845030955970287322998046875f,1.32590002927780137298441331950016319751739501953125f,1.38539998333007030595354081015102565288543701171875f,-0.185700030069053223513719785842113196849822998046875f,-0.31949999804794781965711081284098327159881591796875f,-0.058800003480911300357547588646411895751953125f,-0.173899999378621661350052818306721746921539306640625f,0.8594999778568748194373938531498424708843231201171875f,0.626099997231364113758900202810764312744140625f,-0.241899986429512436902911076685995794832706451416015625f,-0.0184000060319904346073371925740502774715423583984375f,0.637099982434511158402301589376293122768402099609375f,1.0537000011011954381245914191822521388530731201171875f,0.972700039106607849959118539118207991123199462890625f,0.1907000117778778758292901329696178436279296875f,0.0916999989554285743764694416313432157039642333984375f,-0.62630002069175183532223627480561845004558563232421875f,-1.643300048653781875174217930180020630359649658203125f,-0.1681999908417461941922965706908144056797027587890625f,-0.68460001536607728578331943936063908040523529052734375f,-1.36970000608563413635465622064657509326934814453125f,1.189900027944147620218018346349708735942840576171875f,1.1419000141456721042487743034143932163715362548828125f,0.826600027644634138823676039464771747589111328125f,0.370200027111172946803208105848170816898345947265625f,0.8711000302374360959589694175519980490207672119140625f,0.0782999702736735248009836141136474907398223876953125f,0.167200026164949111517898927559144794940948486328125f,-0.06880002147406383272709717857651412487030029296875f,0.00119999573677775916991095073171891272068023681640625f,-0.459399987286329025693021321785636246204376220703125f,-0.45560000250637544549903168444870971143245697021484375f,0.5520000247955323402493377216160297393798828125f,0.8199000206798319201340063955285586416721343994140625f,1.8540000036209816069998623788706026971340179443359375f,-0.89770005378425199449310412092017941176891326904296875f,-1.0235000245049590272827799708466045558452606201171875f,0.823500016085803476784121812670491635799407958984375f,0.512799996782839162534628485445864498615264892578125f,0.51210000705868008452625872450880706310272216796875f,-0.680700041803718125521527326782234013080596923828125f,0.666199994432925990395233384333550930023193359375f,0.5396000272870065117558624478988349437713623046875f,-0.6730000298917300227685700519941747188568115234375f,-0.98969998310059226209745020241825841367244720458984375f,-0.352800001215934688758579795830883085727691650390625f,0.495600018785893325912184081971645355224609375f,0.1216000282406815014013545805937610566616058349609375f,0.59089999664425807424095182796008884906768798828125f,-0.5716999978005883331633185662212781608104705810546875f,0.960100034081936115626376704312860965728759765625f,-0.7287000016972420279870448212022893130779266357421875f,-0.81520000468492537226694594210130162537097930908203125f,0.10990001218616907152636485989205539226531982421875f,-1.10409999530911395737575730890966951847076416015625f,-0.031499991066753718627069247304461896419525146484375f,-0.51390001189559664229733471074723638594150543212890625f,1.360000018179416514385593472979962825775146484375f,-0.5663000251621010949776291454327292740345001220703125f,-0.36470004646778164669740363024175167083740234375f,0.1843999955192205941756355969118885695934295654296875f,0.08239999698698508012739694095216691493988037109375f,-0.8707000432193285899273860195535235106945037841796875f,-0.453999994382261906622488822904415428638458251953125f,0.3920000502392653185523840875248424708843231201171875f,-1.2422000197947025146305577436578460037708282470703125f,0.236199995662271788887665024958550930023193359375f,0.8924000417649751160098503532935865223407745361328125f,-0.1961999955505131509880811790935695171356201171875f,-0.67420000938624158326462065815576352179050445556640625f,-1.3163000368222597113998517670552246272563934326171875f,-2.176400026722252523114775613066740334033966064453125f,-0.667300024782121692368264120887033641338348388671875f,1.554200042355061039955899104825221002101898193359375f,0.85020004003793037128389187273569405078887939453125f,0.75820000894665728896626433197525329887866973876953125f,-0.048200019489228917990431000362150371074676513671875f,0.23469998645335454767746341531164944171905517578125f,0.5362000229127710237975179552449844777584075927734375f,-0.76679996762126645393209400936029851436614990234375f,0.2933000196337698373127977902186103165149688720703125f,0.4006000393867499287381406247732229530811309814453125f,0.165800001752376691133150643508997745811939239501953125f,0.7730000158101313356695527545525692403316497802734375f,0.1210000221580269652577044325880706310272216796875f,-0.59739998737573574150161448415019549429416656494140625f,-0.48530003675520472228299695416353642940521240234375f,-0.126899962252377651594770213705487549304962158203125f,-0.07320000607818410998106628539972007274627685546875f,1.581399963764845484348597892676480114459991455078125f,-0.6693000118404626164192450232803821563720703125f,-1.0066999980613589027456100666313432157039642333984375f,0.5529000356703999141672056794050149619579315185546875f,-0.14300001577287935194959800355718471109867095947265625f,0.411800011914968511206325274542905390262603759765625f,-0.059399994997680227726277735200710594654083251953125f,-0.075500013947487065735231226426549255847930908203125f,0.4265999879330391042486780861509032547473907470703125f,0.123099999302625828789814477204345166683197021484375f,-0.281300006572902105972389108501374721527099609375f,-0.096699956487118488013265960034914314746856689453125f,0.29090000770837054488993089762516319751739501953125f,-0.59630001152753830950814517564140260219573974609375f,0.10230000151768325888923527600127272307872772216796875f,-1.07930003530979234938058652915060520172119140625f,-0.3527000181332227146668856221367605030536651611328125f,-0.5920000155195592750345667809597216546535491943359375f,-0.709800006304681208035844974801875650882720947265625f,-0.0520000125765804899202748856623657047748565673828125f,0.6815000459030275958838274164008907973766326904296875f,0.174900011664628873830906741204671561717987060546875f,0.621900027258694532150684608495794236660003662109375f,0.9801000015601510906293469815864227712154388427734375f,0.03860001518726363656242028810083866119384765625f,-0.195900012476742713118937899707816541194915771484375f,-0.6229000017330046379271379919373430311679840087890625f,0.31089998550563979762273447704501450061798095703125f,-0.1936000321000819202055254208971746265888214111328125f,0.545399958817660124310577884898521006107330322265625f,-1.29000001620501247856509507982991635799407958984375f,0.025000000223517560016261995770037174224853515625f,-0.08279999842196705817087831746903248131275177001953125f,-0.562700023125112469557507210993207991123199462890625f,-0.926900034895539182144830192555673420429229736328125f,0.305000036843121602458950292202644050121307373046875f,1.48530003079473971183688263408839702606201171875f,1.82130005827993191047653454006649553775787353515625f,0.71710000161975617771759061724878847599029541015625f,1.1610999993175266098432985018007457256317138671875f,0.40429999600797861258882903712219558656215667724609375f,-0.484600002029537790804170072078704833984375f,-0.25880000549256809705411797040142118930816650390625f,-0.547400020307302614952504882239736616611480712890625f,1.7024000336065887584169331603334285318851470947265625f,0.0669000018894678749603599499096162617206573486328125f,0.5656000203877689092024638739530928432941436767578125f,-0.0988000002026556689571634706226177513599395751953125f,-0.9173000388264662685600114855333231389522552490234375f,0.9806999943435188438201066674082539975643157958984375f,1.265400006315112069188444365863688290119171142578125f,0.13030000240802763755709747783839702606201171875f,-1.14320002675354448484767999616451561450958251953125f,-0.315600004331767269860620217514224350452423095703125f,0.649299984906613669721764381392858922481536865234375f,-0.9543000149697065115361738207866437733173370361328125f,0.02619999350160362450168349823798052966594696044921875f,0.4947999874919648544135952761280350387096405029296875f,-0.63789999707639200021702663434552960097789764404296875f,0.12430000878572455835779919652850367128849029541015625f,-0.38860001213252548524224039283581078052520751953125f,-1.2999000496253376724098416161723434925079345703125f,-0.779799993303418315093722412711940705776214599609375f,0.3538000207476319491917138293501920998096466064453125f,0.423500035084784176575567471445538103580474853515625f,0.074600029112399379727094128611497581005096435546875f,-0.058400008863210839393786955042742192745208740234375f,-0.299700015324353807866231136358692310750484466552734375f,0.485400003223121156992192481993697583675384521484375f,0.1288000092923642370124071021564304828643798828125f,0.32220001577138912551845351117663085460662841796875f,-0.85579999486058933033660878209047950804233551025390625f,-0.6365000036954879814032892682007513940334320068359375f,-1.364899984768032314974561813869513571262359619140625f,-0.213799989995360650851807804428972303867340087890625f,-0.36640003614723692937360510768485255539417266845703125f,-0.12430003292560609651218328508548438549041748046875f,-1.1637999973341817128158481864375062286853790283203125f,-1.0011000133991245775177958421409130096435546875f,0.564200028534234121480039902962744235992431640625f,1.6902000147804614460511629658867605030536651611328125f,0.049200014676153980275330468430183827877044677734375f,-0.984300031100213690393729848437942564487457275390625f,-1.064100047984719576987799882772378623485565185546875f,0.9217000414982436939936860653688199818134307861328125f,0.7254999996051194610657830708078108727931976318359375f,-0.175800000560283553596718775224871933460235595703125f,0.9765000078678127692910493351519107818603515625f,0.3752999786034225682129772394546307623386383056640625f,-0.03610001515001048932873572994139976799488067626953125f,0.869499958671628991879742898163385689258575439453125f,1.3652000248014928285300584320793859660625457763671875f,-1.2936000114247203640616135089658200740814208984375f,-1.1678000223383306899194167272071354091167449951171875f,-0.905700034315884305868848969112150371074676513671875f,-0.485400007842481162612102707498706877231597900390625f,-0.526100008928775508820763207040727138519287109375f,0.4858000221297149590071740021812729537487030029296875f,1.2975000053644176745137883699499070644378662109375f,0.19169998483657868515450672930455766618251800537109375f,0.48249997798353394529158322256989777088165283203125f,1.5598999995946882801689525877009145915508270263671875f,0.191000008635223483821619083755649626255035400390625f,0.797399984842538156470936883124522864818572998046875f,0.76789999957233678884449545876123011112213134765625f,-1.010400045430660753709162236191332340240478515625f,0.1677999909341332251955236642970703542232513427734375f,-1.100800016281008542051722542964853346347808837890625f,-1.41760000789314499769488975289277732372283935546875f,0.181300001916289144077154560363851487636566162109375f,-0.129799993862211471284950903282151557505130767822265625f,-0.25709996362179465823061264018178917467594146728515625f,-0.62810001994073394460116332993493415415287017822265625f,0.5242999973863362583870184607803821563720703125f,0.1732000027626749982800902216695249080657958984375f,0.05629996268898207478770245870691724121570587158203125f,0.89739998569935519601159512603771872818470001220703125f,1.2560000182464718943009529539267532527446746826171875f,1.82710004978776030526432805345393717288970947265625f,1.6001000631392006123832061348366551101207733154296875f,0.6538000761985791609021134718204848468303680419921875f,0.95110005009323383973196541774086654186248779296875f,-0.483499988183378803796585998497903347015380859375f,-0.0697999965816737955037751817144453525543212890625f,-0.08510001322030991133260613423772156238555908203125f,-0.9790000362172726067200301258708350360393524169921875f,0.463900055127591304238166003415244631469249725341796875f,1.133699982251226590079795641941018402576446533203125f,0.6716999924361706231223934082663618028163909912109375f,0.804200010317563940276386347250081598758697509765625f,0.071299942982195663176980815478600561618804931640625f,-0.19760003519952318296049043055973015725612640380859375f,-0.531999988138675661275556194595992565155029296875f,0.5940000372976068643282587800058536231517791748046875f,1.373500033780932216842529669520445168018341064453125f,-0.1603000345945362337118922368972562253475189208984375f,0.316400000999122799072438283474184572696685791015625f,-0.25289999443143595936334122598054818809032440185546875f,-1.17940001612007616671462528756819665431976318359375f,0.830000032037496726644576483522541821002960205078125f,-0.06160000536739806964448007420287467539310455322265625f,0.28490001866817493691996787674725055694580078125f,-0.5795999891400336334612575228675268590450286865234375f,-1.861000073902310258944226006860844790935516357421875f,-1.5821000059783454361195254023186862468719482421875f,-0.41349997551739214873833816454862244427204132080078125f,0.88250001382082654544092292780987918376922607421875f,-0.6969000094145532120393227160093374550342559814453125f,-0.07279999898076094932974910989287309348583221435546875f,-0.5479000284060837611121996815199963748455047607421875f,0.338399987815320546502562137902714312076568603515625f,0.8768000133119524530656008209916763007640838623046875f,-0.27569998067170342626042156553012318909168243408203125f,-0.196799957190453511657324270345270633697509765625f,-0.121199977743625453285858384333550930023193359375f,-0.66480001561790746933411355712451040744781494140625f,-0.0416000361010440133924248584662564098834991455078125f,0.00929999574720863364518663729541003704071044921875f,-0.4388000136882066470889185438863933086395263671875f,0.98190002439022094904430559836328029632568359375f,1.5091999762311569810435685212723910808563232421875f,-0.47440006663501310679720290863770060241222381591796875f,0.43659999091327161835351944318972527980804443359375f,0.477199998523294421914897611713968217372894287109375f,-0.256200010656565446698351706800167448818683624267578125f,-1.43270008828044037585414116620086133480072021484375f,-0.602800003935396233600840787403285503387451171875f,-1.0652000341147189743651324533857405185699462890625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.15399997985363000907454988919198513031005859375f,-0.73399996316432947196517488919198513031005859375f,-1.1846999791145318425833465880714356899261474609375f,0.6944999806880947090803601895458996295928955078125f,-0.546300000500679061588016338646411895751953125f,-0.24919994117617516593554682913236320018768310546875f,-0.259800008726120523760982905514538288116455078125f,-0.22599999809265103323241419275291264057159423828125f,-0.15420000286698343217040019226260483264923095703125f,-1.36859998359680190560538903810083866119384765625f,0.39280002088546783767242231988348066806793212890625f,0.82979999129176196248636188101954758167266845703125f,1.558599989110231121003380394540727138519287109375f,0.9172000184535971101240647840313613414764404296875f,-1.721500001609324925766486558131873607635498046875f,0.93480000969469490001984013360925018787384033203125f,0.3563000488922007935599367556278593838214874267578125f,-0.56830001695156084906557225622236728668212890625f,-0.58719999163150760068674571812152862548828125f,-0.462300024819374044682263047434389591217041015625f,0.495300045919417897266612271778285503387451171875f,0.668899969315528863944564363919198513031005859375f,-1.03830001814365413537188942427746951580047607421875f,-0.3673000001579520557015712256543338298797607421875f,0.2174999993667008357789427464012987911701202392578125f,0.007400004959105643820294062606990337371826171875f,0.150599976205825214492506347596645355224609375f,0.949999999403953410137546597979962825775146484375f,0.85590002629756967156282598807592876255512237548828125f,-0.710799997375160419466055827797390520572662353515625f,0.429199956077337407833738325280137360095977783203125f,-1.202699955138564202883344478323124349117279052734375f,-1.4386999879509201338834145644796080887317657470703125f,0.0612999725982550369707269055652432143688201904296875f,-1.9739000298142439504545109230093657970428466796875f,0.314999994039535380352390347979962825775146484375f,0.2570000063180923888239703956060111522674560546875f,-1.467100002849101247903718103771097958087921142578125f,-0.596700029819458688251643252442590892314910888671875f,-0.3404999808669089134127716533839702606201171875f,-0.0933000133752823757049554842524230480194091796875f,-0.42599998363852531468864981434307992458343505859375f,0.0448999593615528880263809696771204471588134765625f,1.0436000420093538565424751141108572483062744140625f,1.007200026500225220615902799181640148162841796875f,0.2901999894857407724657605285756289958953857421875f,-0.82400001889467233695540926419198513031005859375f,0.9038000053435555347647323287674225866794586181640625f,-0.2080999836191537166740772590856067836284637451171875f,-0.21870000858902916007764360983856022357940673828125f,0.1015999890506265046496991999447345733642578125f,-0.04419996024966277303747119731269776821136474609375f,1.36019996102452278563532672706060111522674560546875f,0.8874000198602676192649596487171947956085205078125f,-0.3518000055074690379797175410203635692596435546875f,0.82140003637075409415047033689916133880615234375f,0.4150000362843275780733165447600185871124267578125f,0.6540000159516938804671326579409651458263397216796875f,0.3689000152111052699410720379091799259185791015625f,-0.40180001400113152243420699960552155971527099609375f,-0.9450999913156028497951410827226936817169189453125f,-0.590400039544701993321496047428809106349945068359375f,0.585500026464462308695146930404007434844970703125f,-0.525399979269504768808474182151257991790771484375f,-0.3766000114023679845587366799009032547473907470703125f,-0.5417999853536485943550360389053821563720703125f,0.26720002724528324478114882367663085460662841796875f,0.398799990516901292636475773178972303867340087890625f,-0.0728999894767998757316718183574266731739044189453125f,0.426899977935850660770711328950710594654083251953125f,-0.3864999688267705124644635361619293689727783203125f,-0.7464000187873836011931416578590869903564453125f,0.50079997761249561705199084826745092868804931640625f,1.488199995461105817895486325141973793506622314453125f,0.3215000237748026901130060650757513940334320068359375f,-0.6039999440908427885688070091418921947479248046875f,-0.71939997476935335640746416174806654453277587890625f,0.125600004592537839442911717924289405345916748046875f,-1.0964000435233121066858075209893286228179931640625f,-0.351100005054473029986183973960578441619873046875f,-0.8296999999761585087298954022116959095001220703125f,-0.52679999109059572059976517266477458178997039794921875f,0.29749998755753004342494705269928090274333953857421875f,0.39420005174279282300631166435778141021728515625f,-0.9723000096201896980119272484444081783294677734375f,0.058700014400482558585281367413699626922607421875f,-0.9099999833106995339448985760100185871124267578125f,-0.39620002214908556226191649329848587512969970703125f,0.03830001292824736225384185672737658023834228515625f,0.32149999147653574027572176419198513031005859375f,-0.262599993997812308776929057785309851169586181640625f,0.30980000149905695394636495620943605899810791015625f,-1.393700020286441354500084344181232154369354248046875f,-1.1206999979756773200278985314071178436279296875f,0.51249998917803141207372163989930413663387298583984375f,-0.64190001083016401395298089482821524143218994140625f,0.16330005211830211919732391834259033203125f,-0.4348000251173971264506690204143524169921875f,1.36710002892613413649769427138380706310272216796875f,-0.051199999499321080520530813373625278472900390625f,-0.27769996623992909690059605054557323455810546875f,-0.9301999903798101598795255995355546474456787109375f,0.33549998474121078828602549037896096706390380859375f,0.64260000174641618286841548979282379150390625f,-0.45820000775456382058337112539447844028472900390625f,0.4906000036239621664435617276467382907867431640625f,-0.840100011655688039269307409995235502719879150390625f,-0.2706000393867498132749460637569427490234375f,0.00299998950958269006150658242404460906982421875f,0.9157000195264810571416091988794505596160888671875f,0.106299978893995561435303898178972303867340087890625f,-0.80690000192672017842454579295008443295955657958984375f,0.30599999736621985579887450512615032494068145751953125f,1.417800048303604842914182881941087543964385986328125f,0.440100025215744761197811385500244796276092529296875f,-0.443600016677379471730091609060764312744140625f,-0.0995999870911237206172472724574618041515350341796875f,-0.596599992179870586284096134477294981479644775390625f,0.50129997732937325860547161937574855983257293701171875f,0.579600040493160495458369041443802416324615478515625f,0.8686000521421437525759756681509315967559814453125f,0.5422999988913534963330675964243710041046142578125f,0.977700065630674686190104694105684757232666015625f,-0.513900016924738967105668052681721746921539306640625f,0.2396000341415405188172371708787977695465087890625f,0.4807000016450881929586103069595992565155029296875f,0.157599995040893503528423025272786617279052734375f,-0.3959999902322888232220066129229962825775146484375f,-0.93420001149103060189560210346826352179050445556640625f,-0.300199956107139342975642648525536060333251953125f,-0.71110000594854361199992354158894158899784088134765625f,0.515499966580420743156309981714002788066864013671875f,-0.2828999886572365340953183476813137531280517578125f,-0.52239996263980881252564358874224126338958740234375f,0.066699991840124539521639235317707061767578125f,0.9245999673843385835425578989088535308837890625f,0.38370001559257449486040059127844870090484619140625f,0.6242000619500880365109196645789779722690582275390625f,0.5373000123396514116080879830406047403812408447265625f,0.9991000064849855366588826655060984194278717041015625f,0.44679998386353236838886004989035427570343017578125f,1.2341000151276586560555870164535008370876312255859375f,-0.648799957697093532260623760521411895751953125f,-0.56910000618696177099309352342970669269561767578125f,-0.68259998311996472608598196529783308506011962890625f,-0.09030000702738760764987091533839702606201171875f,1.225600003623963374366212519817054271697998046875f,0.7034000125885011556192694115452468395233154296875f,-0.95600003176927594861211900933994911611080169677734375f,1.232800006263703007647336562513373792171478271484375f,0.15720000087022756307675308562465943396091461181640625f,1.123699995755404135167054846533574163913726806640625f,-0.60820002310276066737060318700969219207763671875f,-0.63819999389648440768496584496460855007171630859375f,0.359199979323148621546124559245072305202484130859375f,0.288100006380677342576746013946831226348876953125f,-0.94930002577304861688389792107045650482177734375f,0.0746999943137165445250502671115100383758544921875f,-0.7130999770998958098289222107268869876861572265625f,-0.549400064325333392645234198425896465778350830078125f,1.197000015631318081688050369848497211933135986328125f,-0.228799991578608530318206248921342194080352783203125f,-0.669599983477592530789479496888816356658935546875f,-0.9782000156521792888497657258994877338409423828125f,1.400100001448393083336441122810356318950653076171875f,0.955599995353817899257364842924289405345916748046875f,-0.0660999689936634382547708810307085514068603515625f,-0.617900015218555864748850581236183643341064453125f,-1.61439997624084252692000518436543643474578857421875f,0.081199961277842902518386836163699626922607421875f,-0.7612000185728078349711722694337368011474609375f,0.486199983406066849056514911353588104248046875f,0.1223000413596633251245293649844825267791748046875f,0.753699938479065689733715771581046283245086669921875f,0.0182000114798537282467805198393762111663818359375f,-0.90289996362328484025283614755608141422271728515625f,-0.01600005114078584256276371888816356658935546875f,-0.987999956727026784619738464243710041046142578125f,-0.97280007035732296571950428187847137451171875f,-0.352900035595894223661161959171295166015625f,0.750799980592727678185838158242404460906982421875f,-0.509799978923798136065670405514538288116455078125f,1.50620004122257267198392582940869033336639404296875f,-0.11849996780604110657719729715608991682529449462890625f,0.00550003207102378599557823690702207386493682861328125f,-1.299000017404555507027907879091799259185791015625f,-0.4383000211983925087366742445738054811954498291015625f,0.3256999771699307100192299913032911717891693115234375f,-0.8350000175833702797945079510100185871124267578125f,0.63479998309612284401737269945442676544189453125f,0.139400029307604089723326978855766355991363525390625f,0.847700004759430303380440818727947771549224853515625f,0.584099986368417756210646984982304275035858154296875f,1.624299981814622650944102133507840335369110107421875f,0.7912000191688537409362425023573450744152069091796875f,-0.73989996319860207307783639407716691493988037109375f,-0.24720003991127015297024627216160297393798828125f,-1.038099982762336281894022249616682529449462890625f,-0.973200014758109688273179926909506320953369140625f,0.443999960184097375304190791212022304534912109375f,-0.33719999550581025715700889122672379016876220703125f,-0.86210001357793775156324045383371412754058837890625f,-1.1203000354886061273873565369285643100738525390625f,0.28070000291168728523416575626470148563385009765625f,-0.3937999998673797907855487210326828062534332275390625f,-0.47010001441240323316606009029783308506011962890625f,-1.067900036117434670046577593893744051456451416015625f,0.24619998579025281770782385137863457202911376953125f,0.3400000147521495819091796875f,-0.4679000427484520940879519912414252758026123046875f,-1.1186000005841254534288964350707828998565673828125f,-0.81720000385045998836375247265095822513103485107421875f,-0.2620999886371198073220512014813721179962158203125f,-0.4809999883323907710064304410479962825775146484375f,0.615900023764371695733643718995153903961181640625f,0.7544000306487088636231419513933360576629638671875f,0.298799986791610905356719740666449069976806640625f,-0.57570001226216571676985722660901956260204315185546875f,-0.96139999839141998538849520627991296350955963134765625f,0.5879999310970305259615997783839702606201171875f,-0.513399943782388845647801645100116729736328125f,-0.99620000268444375279841551673598587512969970703125f,-0.778099997812509602823638488189317286014556884765625f,0.553099982389807731664177481434307992458343505859375f,1.480300044727325570192988379858434200286865234375f,-0.12180000371932997449420099655981175601482391357421875f,0.960600033035129197145352009101770818233489990234375f,1.3373000260114675796785377315245568752288818359375f,0.2807999853610994023256353102624416351318359375f,0.4234000323325395331863774117664434015750885009765625f,-0.1167000047668815820856025311513803899288177490234375f,0.03110004916191133617076047812588512897491455078125f,0.32179998092055317471249509253539144992828369140625f,-0.6659000003695485503385498304851353168487548828125f,0.74939999965429304751296513131819665431976318359375f,-1.61829997988492291227657915442250669002532958984375f,-0.02599999501928629275226967365597374737262725830078125f,1.610600014054775375171857376699335873126983642578125f,0.30460002534985530786570961936376988887786865234375f,0.0445999860480428278464160030125640332698822021484375f,-0.596500022768974957898535649292171001434326171875f,-0.9009999872148028998708468861877918243408203125f,-0.241900003603100710591888855560682713985443115234375f,-0.181600018256902639990357783972285687923431396484375f,-0.379399989819526606282806824310682713985443115234375f,-0.5171000224813819823310723222675733268260955810546875f,0.67700001697242218057226637029089033603668212890625f,0.841800027263165162594305002130568027496337890625f,0.43289999461770012345596114755608141422271728515625f,-1.1304000080287455176630828646011650562286376953125f,-0.099500004157424104533902209368534386157989501953125f,-0.9150999977231020210410861182026565074920654296875f,0.454600009703636231961354496888816356658935546875f,-2.002400016283988559706585874664597213268280029296875f,-0.7772000128656628703538444824516773223876953125f,0.1851000114321710299236656283028423786163330078125f,0.6131999813795090403800713829696178436279296875f,1.713899993753432937637626309879124164581298828125f,-0.3808000154614443744094387511722743511199951171875f,1.667400015054642636869175476022064685821533203125f,0.41239999555274831077866792838904075324535369873046875f,0.55640000224709496023933752439916133880615234375f,0.458900027653575737218716312781907618045806884765625f,-1.204700008916855580309857032261788845062255859375f,-0.689900003468989719834780771634541451930999755859375f,-0.534899993261694906010461636469699442386627197265625f,-0.466299990554154053512547761783935129642486572265625f,0.64019999103173674814826199508388526737689971923828125f,1.7346000082135191178167588077485561370849609375f,-0.4036999732732766688059200532734394073486328125f,0.5439999896883964680682765902020037174224853515625f,-0.21740001017451238141120484215207397937774658203125f,2.115599983358382729647928499616682529449462890625f,-0.0453999904453752378685749135911464691162109375f,-1.409800016251206766781933765741996467113494873046875f,1.294299991202354505048788269050419330596923828125f,0.072200009512901175412480370141565799713134765625f,1.08159998919963840080527006648480892181396484375f,0.7152999985337249455596975167281925678253173828125f,0.29839999381303794478981217253021895885467529296875f,-0.530399980759620692793987473123706877231597900390625f,-0.549700010332464739093438765848986804485321044921875f,-1.415100047194957966212314204312860965728759765625f,-0.30910002317428553197942164842970669269561767578125f,0.01109998538494050279723523999564349651336669921875f,0.5776999689221380407389005995355546474456787109375f,0.2145999989747995329025798127986490726470947265625f,1.1997000348448754181163167231716215610504150390625f,-0.21559998335838326255498031969182193279266357421875f,1.4647999680459502513940606149844825267791748046875f,0.053800008249282882388797588646411895751953125f,-1.255600053244830149878907832317054271697998046875f,-0.856999990150332013882916726288385689258575439453125f,-1.0288000180840501940338072017766535282135009765625f,-0.880900013482570187761666602455079555511474609375f,-0.01100003936886739808187485323287546634674072265625f,0.88450000661611483110391418449580669403076171875f,0.47239997396469135537699912674725055694580078125f,0.360099996232985830602046917192637920379638671875f,1.00630002218186831441926187835633754730224609375f,1.397400018705428070830976139404810965061187744140625f,-0.80039998563975078837984256097115576267242431640625f,-1.47299997282028272138632019050419330596923828125f,-0.20669998520910703998509916345938108861446380615234375f,0.9554000335283581168965838514850474894046783447265625f,1.141200006949901535335811786353588104248046875f,0.5933999749258163092235918156802654266357421875f,0.3808000002436335851285775788710452616214752197265625f,0.35609998462125691531809934531338512897491455078125f,-0.11520000110864625497697488754056394100189208984375f,-0.49169997798204434502622461877763271331787109375f,-1.895799965691566058012540452182292938232421875f,-0.8685999693661923259924151352606713771820068359375f,-1.1601000066265456656111609845538623631000518798828125f,0.1950999947428702085971963242627680301666259765625f,0.70259999027252195702430981327779591083526611328125f,-0.6112999875366682545063667930662631988525390625f,-0.741800052744150395511724127572961151599884033203125f,0.940599996694922335649380329414270818233489990234375f,-0.2623999718785281487498650676570832729339599609375f,-1.399900023436547513711047940887510776519775390625f,-0.23990000972747793639427982270717620849609375f,1.2705999893188479887840003357268869876861572265625f,-0.38230000038146982888065394945442676544189453125f,0.90539997837543495506906765513122081756591796875f,-0.3443999898195269082634695223532617092132568359375f,0.784700000274181519444027799181640148162841796875f,-0.37510000144839228397586339269764721393585205078125f,0.820299986389279478515845767105929553508758544921875f,0.5362000273644920156357329688034951686859130859375f,-0.51610002114772868253567139618098735809326171875f,-1.8336000205516806982330990649643354117870330810546875f,-0.39280003641992866647569826454855501651763916015625f,-0.077400012111663585301357670687139034271240234375f,1.271200009632110550228389911353588104248046875f,1.488599970185756404816856957040727138519287109375f,-0.0742999385267488765549614981864579021930694580078125f,0.2468999875470998528470545352320186793804168701171875f,1.5733999988794327151708785095252096652984619140625f,-0.3911999691009526713969535194337368011474609375f,-1.474799997997283895756481797434389591217041015625f,-0.357200005936622488889042870141565799713134765625f,0.1159999843835830546368015347979962825775146484375f,0.167599987441300779522634911700151860713958740234375f,0.507400024554133022292035093414597213268280029296875f,1.75640000090599102122723706997931003570556640625f,0.0368000096797942433113348670303821563720703125f,0.935500035852193878582738761906512081623077392578125f,0.388299986031651389595253931474871933460235595703125f,-0.3501000251412396124806036823429167270660400390625f,-0.5130999857425688759349213796667754650115966796875f,-0.050800007414817827111619408242404460906982421875f,0.06990003178119630344866664017899893224239349365234375f,-0.12199997947737539760737490723840892314910888671875f,1.1468000158637767782465743948705494403839111328125f,0.690399977332353476100479383603669703006744384765625f,-1.040499988839030276466246505151502788066864013671875f,-0.3051000150084497430924557193065993487834930419921875f,0.214299999807774810278715449385344982147216796875f,0.779600041890144979106480604968965053558349609375f,-0.11719999341964726369269556016661226749420166015625f,0.3013999974787235913709082524292171001434326171875f,1.26340003196001049445840180851519107818603515625f,-0.492200052279234245844463657704181969165802001953125f,-0.945599942305684226795392532949335873126983642578125f,-0.79030001432895602420103386975824832916259765625f,0.46529999644756259158384636975824832916259765625f,0.203899987792969028532752417959272861480712890625f,-0.31640001819133800609051831997931003570556640625f,-0.0354000165224075402647940791212022304534912109375f,0.568600035750865373529450153000652790069580078125f,-0.9848999718785282908584122196771204471588134765625f,-0.1099000033944843579547523404471576213836669921875f,0.252799980093538323444590787403285503387451171875f,0.6600000097975138313444176674238406121730804443359375f,-0.24529995673596882710398858762346208095550537109375f,-0.0832999939665193256388420195435173809528350830078125f,1.015100033783912891749423579312860965728759765625f,-0.0193999814748764265459612943232059478759765625f,-1.4319000413030387175439273050869815051555633544921875f,-0.1409000045046211457844265169114805757999420166015625f,0.426100008407235275598168300348334014415740966796875f,-0.917400038039684506685489395749755203723907470703125f,-0.3636000329941513342646430828608572483062744140625f,1.2935000311732292033184421597979962825775146484375f,-0.026000001668929684228714904747903347015380859375f,-0.9128000617146501127763258409686386585235595703125f,0.292000014290213272261098609305918216705322265625f,1.0792000447764997783650642304564826190471649169921875f,2.5162000570178033598267575143836438655853271484375f,0.892700053858757058833361952565610408782958984375f,0.1853000480055815302193877869285643100738525390625f,-0.63160003911852857783060244400985538959503173828125f,-0.5981000271677974211570472107268869876861572265625f,0.3876000012993809917816179222427308559417724609375f,0.1992000089764598413921703468076884746551513671875f,-0.098000012047588569430445204488933086395263671875f,-1.273300018274038603038889050367288291454315185546875f,0.4869000181317328923569220933131873607635498046875f,-0.83389999032616657359540113247931003570556640625f,-1.055100050994754123934171730070374906063079833984375f,-0.6388999737858771510445876629091799259185791015625f,-0.845199993881582845034472484258003532886505126953125f,1.532099950993060843984494567848742008209228515625f,1.0034000030517571389054864994250237941741943359375f,-0.1497999926328656972174258044105954468250274658203125f,-0.4695000706538561274783205590210855007171630859375f,-0.949899935370684289637210895307362079620361328125f,0.282899994170666246873224736191332340240478515625f,0.4192999858379362621008112910203635692596435546875f,0.12180002070665363334001085604541003704071044921875f,-0.607399995720386609576735281734727323055267333984375f,0.0754000182360410331483535628649406135082244873046875f,-1.12230002306848763993230022606439888477325439453125f,-0.1579999971836805183755814141477458178997039794921875f,0.1400000301375980971130275065661408007144927978515625f,-1.1376000159025192459694153512828052043914794921875f,-0.9153000304222107530449648038484156131744384765625f,0.17189998371005099642161439987830817699432373046875f,-1.1948999909520150453090536757372319698333740234375f,0.6008000053286550734554793962161056697368621826171875f,-0.4131999916613100509721334674395620822906494140625f,-0.00990000002309676663259097040281631052494049072265625f,-0.455999982595443498212262056767940521240234375f,-0.5527000627994542725218707346357405185699462890625f,-0.4021000320553778095700181438587605953216552734375f,0.28800000143051107670544297434389591217041015625f,-0.849799983692168581228543189354240894317626953125f,0.114399950033425401585418512695468962192535400390625f,-1.410500003740191576895313119166530668735504150390625f,-0.65749997332692178275692640454508364200592041015625f,-0.1257999904274935687453762511722743511199951171875f,-0.223399991726875413178277085535228252410888671875f,1.57080001024603888026831555180251598358154296875f,-0.223699993762374926831171251251362264156341552734375f,-0.087899998492002140437762136571109294891357421875f,-0.62419996427297608221351765678264200687408447265625f,0.936499970763922107153121032752096652984619140625f,-0.576300005269050785727813490666449069976806640625f,0.07860000028610247824190082610584795475006103515625f,-0.47709999495148647241649086936376988887786865234375f,0.5270000081807373337738908958272077143192291259765625f,-0.3121000364884733979664588332525454461574554443359375f,-1.488500013440847258294752464280463755130767822265625f,0.328800019499659423871662511373870074748992919921875f,-0.105800058972835842041604337282478809356689453125f,0.4328000295281402287628225167281925678253173828125f,-0.6446000284790986967209391877986490726470947265625f,1.4569999979734422623778300476260483264923095703125f,0.6692999825596803731286854599602520465850830078125f,0.5738000016927717883419290956226177513599395751953125f,-0.12849996713548872406818190938793122768402099609375f,-0.887600020968914105168323658290319144725799560546875f,0.635599979111552304544829894439317286014556884765625f,-1.714000000715255822569815791212022304534912109375f,-1.196599995160102736235785414464771747589111328125f,-0.17599996680021234851665212772786617279052734375f,-1.7082000025391579356437432579696178436279296875f,-0.241300045502185867007938213646411895751953125f,0.653199976611137600457368534989655017852783203125f,0.858000005006790189554521930404007434844970703125f,-1.271599993371963943644686878542415797710418701171875f,-1.1057000254869453481632035618531517684459686279296875f,-0.3600999796554444909446601741365157067775726318359375f,-1.082599976563453747502308033290319144725799560546875f,-0.407000005871057002337920494028367102146148681640625f,-0.6641000052928927033235595445148646831512451171875f,-0.6880000014305114319768108543939888477325439453125f,0.4110000070333479360584760797792114317417144775390625f,0.96600001727789663874546022270806133747100830078125f,-1.289599988543987052480588317848742008209228515625f,0.17879997341781794784054682168061845004558563232421875f,-0.51790003390088645840449999013799242675304412841796875f,0.9410000074803821501490119771915487945079803466796875f,0.477099996329844344700177316553890705108642578125f,-0.262599999213218637805766775272786617279052734375f,-0.661900002858042402209548527025617659091949462890625f,0.02299999576807021384183826739899814128875732421875f,0.24429999313950556683039394556544721126556396484375f,1.755400006091594633517161128111183643341064453125f,1.1227000040888786003279165015555918216705322265625f,1.17009998610019660958414533524774014949798583984375f,-0.033599970930815015179859983618371188640594482421875f,0.5024000238090753356345885549671947956085205078125f,0.4362999537110336945033850497566163539886474609375f,-0.076700000929832867768709547817707061767578125f,0.24460001268386832862233859486877918243408203125f,-1.07999999105930388765273164608515799045562744140625f,0.355099980884791222024432499893009662628173828125f,1.1863999958395954337220246088691055774688720703125f,1.5333000187397001212730174302123486995697021484375f,0.351300003182888787023330223746597766876220703125f,-2.200500020503998399590273038484156131744384765625f,0.640399944698810230647723074071109294891357421875f,-0.39210000508427622634144427138380706310272216796875f,0.208800013613700485848312382586300373077392578125f,0.8869999990910295561974407974048517644405364990234375f,0.5252000120982527331392475389293394982814788818359375f,-1.28549998891353567387341172434389591217041015625f,-0.2512000573158257310524277272634208202362060546875f,1.4260000192522994666433078236877918243408203125f,0.71530000240802760202996068983338773250579833984375f,1.0958000165045262974672368727624416351318359375f,-0.5579000025153160180479972041212022304534912109375f,0.4693999862432480796314848703332245349884033203125f,-1.05080001218318930256145904422737658023834228515625f,-0.68949997964501363867384498007595539093017578125f,1.168799971592426434341405183658935129642486572265625f,-1.262699966016411767810723176808096468448638916015625f,-1.2735000284910196199916754267178475856781005859375f,0.1572000038504597352329028581152670085430145263671875f,-2.11449997171014469898864263086579740047454833984375f,0.37279998691082028017262928187847137451171875f,0.0502000324010847265299162245355546474456787109375f,-0.8897000181555743125727531150914728641510009765625f,1.41469999282360081593878931016661226749420166015625f,0.2590000121891495865611432236619293689727783203125f,-0.15460000076293933801707680686376988887786865234375f,0.4292999797284604568403665325604379177093505859375f,0.12799995404481812055763612079317681491374969482421875f,-0.212499975170940214042047955445013940334320068359375f,-0.2731000098824498678595773526467382907867431640625f,0.592399999669193988438564701937139034271240234375f,-0.6081999678567056033529070191434584558010101318359375f,-0.822700006175040243761031888425350189208984375f,-0.76070000537037874011048188549466431140899658203125f,-1.5046000421881675634949715458787977695465087890625f,0.8295999593377114678105499478988349437713623046875f,-0.684300021004677461178289377130568027496337890625f,0.577999996066093757463022484444081783294677734375f,0.9393000117659564551786388619802892208099365234375f,-0.4650000037997958912683316157199442386627197265625f,0.8777999609455460205964527631294913589954376220703125f,-0.0224999799951905998796064523048698902130126953125f,-0.46990001101270351124838953182916156947612762451171875f,0.5788000574231144668146953335963189601898193359375f,-0.5763000282168382337744105825549922883510589599609375f,-0.1282999807789924062717545893974602222442626953125f,0.8779000618219381379958576872013509273529052734375f,1.244500001251697796078587998636066913604736328125f,0.1320000027418135601919857435859739780426025390625f,-0.4031999953866005625968682579696178436279296875f,-0.3411000041604044241694282391108572483062744140625f,0.9525000211596488242093982989899814128875732421875f,-0.98180003184527164794559439542354084551334381103515625f,0.39480000092163669567213446498499251902103424072265625f,0.48320001565218007044677506200969219207763671875f,-1.286000043392181879653435316868126392364501953125f,0.347899992382526335177317378111183643341064453125f,0.24299998712539672141019764239899814128875732421875f,0.7017000372886652570514343096874654293060302734375f,1.732600063958764646798726971610449254512786865234375f,-0.1034000259995462300821600365452468395233154296875f,-1.1221000007629395867070343228988349437713623046875f,0.1168999799847600939983749412931501865386962890625f,1.361699979770182977745207608677446842193603515625f,0.15990002358555788219973692321218550205230712890625f,1.263200020718574734246431034989655017852783203125f,0.0753999929040671812430218778899870812892913818359375f,-0.52170000174939612502811314698192290961742401123046875f,1.333999969627708281905142939649522304534912109375f,-0.46779998739510819660836205002851784229278564453125f,0.7017999846458433665929987910203635692596435546875f,0.566900052106380059058210463263094425201416015625f,1.428300004434586156776276766322553157806396484375f,1.93089998457431732958866632543504238128662109375f,0.409899993038177257176357670687139034271240234375f,-0.7145999793052677517835036269389092922210693359375f,0.189800035250187182356285120476968586444854736328125f,1.1853000273674734188489310326986014842987060546875f,-0.69449997726082823845672464813105762004852294921875f,-0.506600009651481997252631117589771747589111328125f,1.432800016880780180628107700613327324390411376953125f,0.4062000719189649000639974474324844777584075927734375f,-0.55750001799315196393536098184995353221893310546875f,0.69380007500648588347758050076663494110107421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.5208999649047854063610429875552654266357421875f,-1.0924999558925634346451261080801486968994140625f,1.968299980890751754714074195362627506256103515625f,-0.095899964010715876838730764575302600860595703125f,-1.7994000181317328923569220933131873607635498046875f,0.65139996380209996829080409952439367771148681640625f,2.4049999415874481201171875f,1.391799976450204923139608581550419330596923828125f,0.10239999646544450939700254821218550205230712890625f,2.042899970626830707942644949071109294891357421875f,1.86800000530481380422997972345910966396331787109375f,0.19200001347064965528943503159098327159881591796875f,0.3427000404477116290991034475155174732208251953125f,-0.8228999694347383098147474811412394046783447265625f,0.230199993062019103717830148525536060333251953125f,0.519600000166893227770970042911358177661895751953125f,-0.0513999851480126945801885085529647767543792724609375f,0.51730000574588785866581019945442676544189453125f,1.071600002908706272819472360424697399139404296875f,1.1566999848365782099790521897375583648681640625f,1.978899986004829969488127971999347209930419921875f,0.236600039267539585807753610424697399139404296875f,-1.38119999294281026180897242738865315914154052734375f,-1.048700006353854963236926778336055576801300048828125f,0.1961999738320712172168214237899519503116607666015625f,2.607799975585937346522769075818359851837158203125f,1.831200012016296341244014911353588104248046875f,-1.14079999251365649115541600622236728668212890625f,-1.872099996888637374325980999856255948543548583984375f,-0.7659000147864223340121725414064712822437286376953125f,-0.66459997371733248172631647321395576000213623046875f,-0.947700017127394378491089810268022119998931884765625f,-0.087100063310562969576267278171144425868988037109375f,0.6029999991580841101068699572351761162281036376953125f,0.8828999703288076261742389760911464691162109375f,0.95900000190734857596908113919198513031005859375f,-0.951999967277050274105931748636066913604736328125f,-0.84060001643896076206630141314235515892505645751953125f,0.2877000013694159807897676728316582739353179931640625f,-0.0868999898195266240463752183131873607635498046875f,-1.463499982893466722089215181767940521240234375f,-0.43290004706978724158261684351600706577301025390625f,0.2000999607682221181903514661826193332672119140625f,0.0369000112771988852955473703332245349884033203125f,0.5421000192403795381324016489088535308837890625f,-0.3697999729633334453637871774844825267791748046875f,-0.13779999525547026451022247783839702606201171875f,1.271699998359382188795052570640109479427337646484375f,1.5474999428167943715806131876888684928417205810546875f,2.290699931162595959222016972489655017852783203125f,1.9310999858319757294111695955507457256317138671875f,2.259899948483706566548789851367473602294921875f,0.67249997451901410983055029646493494510650634765625f,-0.3733999884486198794775191345252096652984619140625f,1.088299986851215095384759479202330112457275390625f,1.7573999870777132770172102027572691440582275390625f,1.992999949276447413382129525416530668735504150390625f,0.1593999898567799444748516179970465600490570068359375f,-1.4384000051379199902612526784650981426239013671875f,-0.12710002296566980106717892340384423732757568359375f,-0.21330000443458541070640421821735799312591552734375f,0.141199983331560918742297872086055576801300048828125f,-0.177599990868568369251079275272786617279052734375f,0.3455999964714049355052338796667754650115966796875f,-0.5870999955475328935250445283600129187107086181640625f,0.028100020685791537999875799869187176227569580078125f,1.89840002316832556772396856104023754596710205078125f,0.72420002231299829276167656644247472286224365234375f,0.4491000117748977782383690282586030662059783935546875f,-0.71609996944069909119434669264592230319976806640625f,-0.4443999692559241765366095933131873607635498046875f,0.30169998781681073296567774377763271331787109375f,-0.49729998667240149501367341144941747188568115234375f,-0.87249997705221193911029331502504646778106689453125f,0.2876000195160501871072256108163855969905853271484375f,0.952200019347667847569027799181640148162841796875f,0.87169995697140745249953397433273494243621826171875f,0.285699976089597296180500052287243306636810302734375f,0.4773999796271315432250048615969717502593994140625f,0.4553000331044199100460900808684527873992919921875f,-0.6720999867558479223816902958787977695465087890625f,0.46929999656677245933877884453977458178997039794921875f,-0.76330001346841436227208532727672718465328216552734375f,-0.8602000070691104838260798715054988861083984375f,0.7514000238537779097214297507889568805694580078125f,-0.94719994633197757138987071812152862548828125f,-0.002600031101703592639751150272786617279052734375f,1.3258000120341772998244778136722743511199951171875f,1.19930000595450447775647262460552155971527099609375f,0.421599989199638258696722914464771747589111328125f,0.70149997813999664941775336046703159809112548828125f,0.6479000212162731031639850698411464691162109375f,-0.34679999969601638110106023304979316890239715576171875f,0.448899945231527652822478557936847209930419921875f,-0.16420000817552204974703045081696473062038421630859375f,0.36600001046061425569178027217276394367218017578125f,-1.051699962782858932541785179637372493743896484375f,-0.7990000329017643565521211712621152400970458984375f,0.33250002101063724779805852449499070644378662109375f,0.442200007128715100179761066101491451263427734375f,0.321900001740456076504415250383317470550537109375f,1.9569000139594070475368425832130014896392822265625f,1.63180003352165226004899523104541003704071044921875f,-0.2267999563336378088251876761205494403839111328125f,-0.30989999616742114341150227119214832782745361328125f,-0.248999985814094770830706693232059478759765625f,0.429299999919533714631825205287896096706390380859375f,0.54059999885559051335803815163671970367431640625f,0.44159998860359195305136381648480892181396484375f,0.6185999967098236851370529620908200740814208984375f,-0.50869999629557138831614793161861598491668701171875f,-1.3964999607987704788314431425533257424831390380859375f,-0.0794999943785370533078094013035297393798828125f,-0.475099977159499697876299251220189034938812255859375f,-0.889800008130073205592225349391810595989227294921875f,-1.605400025612115921802569573628716170787811279296875f,-0.6093000046506522604516931096441112458705902099609375f,0.564599990034103438318879852886311709880828857421875f,-0.02989999139904997438321743175038136541843414306640625f,-1.2749000081256027794296414867858402431011199951171875f,-0.407700010049342864704158273525536060333251953125f,0.3664000107407563433525865548290312290191650390625f,1.48720002024173769683557111420668661594390869140625f,0.996299977031350092460115774883888661861419677734375f,1.81029998691082028017262928187847137451171875f,1.2177000064730645334520886535756289958953857421875f,-0.025599994087219357652429607696831226348876953125f,-0.1086999837040900640516838393523357808589935302734375f,1.46169999945834216958928664098493754863739013671875f,2.1963999758720404997802688740193843841552734375f,1.0501999572992328779719173326157033443450927734375f,0.2266999949321146790026659800787456333637237548828125f,-0.7450000351667398490462801419198513031005859375f,0.77149995422363293329226507921703159809112548828125f,1.1559999870657922116379268118180334568023681640625f,-1.2223999513149266249456559307873249053955078125f,-1.40769997518062606189914731658063828945159912109375f,1.148999988310039466199441449134610593318939208984375f,1.8303999703660609466027153757750056684017181396484375f,0.7554999996423721331240130894002504646778106689453125f,0.695300003674625965999211985035799443721771240234375f,0.0147000390172002948219187601353041827678680419921875f,-0.222599998989700953444526021485216915607452392578125f,0.31279998556971566614492985536344349384307861328125f,0.71559998335838326255498031969182193279266357421875f,-0.1488000204682347060725078335963189601898193359375f,0.19920001165866807468773913569748401641845703125f,-0.328999935448170077734175720252096652984619140625f,0.79230001051425891223090047787991352379322052001953125f,-0.2561999717459075842640459086396731436252593994140625f,-0.65550001737475394048004773139837197959423065185546875f,-1.0844000060245388983304337671143002808094024658203125f,-1.33440003094673187433727434836328029632568359375f,0.85799997788667692333319791941903531551361083984375f,1.074900002872943627352242401684634387493133544921875f,0.34550003287196151546822875388897955417633056640625f,1.2076000185847277634820784442126750946044921875f,1.617000026285648317525556194595992565155029296875f,0.2372999869704255360147726605646312236785888671875f,0.683599949622153868489249362028203904628753662109375f,-0.53370000441670384372372382131288759410381317138671875f,-1.0305000204667449370532494867802597582340240478515625f,0.478799990367889449771610088646411895751953125f,1.6868999615073203557358283433131873607635498046875f,0.6787000310152766502369559020735323429107666015625f,-1.950999962404370347002213748055510222911834716796875f,-1.5636999896645544794182569603435695171356201171875f,-0.4508999958992008227909309425740502774715423583984375f,0.33599996175244528640035923672257922589778900146484375f,1.48380000213980611789565955405123531818389892578125f,-0.05239998707771320596293662674725055694580078125f,-0.84379997725486788340276689268648624420166015625f,0.86239998618364399618485549581237137317657470703125f,0.001500028505920791843664119369350373744964599609375f,0.46600002551078745227641775272786617279052734375f,1.59989995861649614283805931336246430873870849609375f,-0.961700024771690920033506699837744235992431640625f,-0.31800000262260397221325547434389591217041015625f,1.089600023412703677649915334768593311309814453125f,0.45070000998973824835047707892954349517822265625f,-1.478399949407577906868027639575302600860595703125f,-2.395000007748603110258045489899814128875732421875f,-1.39930002696514055315901714493520557880401611328125f,-0.4087000602774317759013911199872381985187530517578125f,1.1330999640800059058420856672455556690692901611328125f,0.02149998700618738212142488919198513031005859375f,0.018899999602139505583409118116833269596099853515625f,-0.8025000009313225657336943186237476766109466552734375f,-0.6687000123143196361752416123636066913604736328125f,-1.160499954342842698906679288484156131744384765625f,-0.69320000171959339496652319212444126605987548828125f,-1.646200006875395249750226867035962641239166259765625f,-1.324700004446506707012076731189154088497161865234375f,-1.626299936202169060805999833974055945873260498046875f,-2.146399968719482131973563809879124164581298828125f,-1.001200015667080389647480842540971934795379638671875f,-0.56669997291565010755221010185778141021728515625f,-0.936999989628791496443227515555918216705322265625f,1.2337000161886209070871700532734394073486328125f,0.959300057959556085052099660970270633697509765625f,0.07670000271797139390628217370249330997467041015625f,0.88120000233054174287872228887863457202911376953125f,0.9033999902367586543050492764450609683990478515625f,-0.476299980384111254494428067118860781192779541015625f,-0.4696000136896965049260188607149757444858551025390625f,0.2223999946773052016624205862171947956085205078125f,0.044299985316395673606848504277877509593963623046875f,-0.27909998979568495514058668049983680248260498046875f,0.5841999994397166773296703468076884746551513671875f,-0.7334000137805940511270819115452468395233154296875f,-0.297900017416477425058474182151257991790771484375f,0.89430001981258355758797051748842932283878326416015625f,1.3338000160351397571645293282927013933658599853515625f,1.32079996479749706139728004927746951580047607421875f,1.1168999418377882903996578534133732318878173828125f,-0.0251000227570532530307900742627680301666259765625f,-0.920199960279465045687175006605684757232666015625f,-1.3808999986745413846023211590363644063472747802734375f,-0.26680003890469616845138034477713517844676971435546875f,-1.11309997322559439680844661779701709747314453125f,-1.3686999565839765846675391003373079001903533935546875f,0.21789998547211320012451096772565506398677825927734375f,0.16769998061954982659926827182061970233917236328125f,0.850999963745474818921366022550500929355621337890625f,0.304700018453598175938168424181640148162841796875f,0.23019999544620495246505242903367616236209869384765625f,0.1487000008776788728681594875524751842021942138671875f,-0.9354999647736548240573029033839702606201171875f,-1.424099982643127759729395620524883270263671875f,-1.145599983842670699374366449774242937564849853515625f,-0.0889000148758294361783782733255065977573394775390625f,0.90659997295737326794551336206495761871337890625f,1.05649998477101281224577178363688290119171142578125f,-0.2247000074267386793280820711515843868255615234375f,-1.2791999817073342882167708012275397777557373046875f,-0.4375000520050524865922625394887290894985198974609375f,0.50099996495619458780623745042248629033565521240234375f,-1.050599975311756306695087914704345166683197021484375f,-0.969899979031085823777402765699662268161773681640625f,-0.9861000084444875302125410598819144070148468017578125f,0.57969996182918492877433891408145427703857421875f,2.18060000958442667950976101565174758434295654296875f,0.777200013610720663592701384914107620716094970703125f,-0.23749997325241611889623527531512081623077392578125f,0.6804000252395869807742201373912394046783447265625f,0.2947000043347476339050672322628088295459747314453125f,-1.2102000142216677858186812954954802989959716796875f,-2.3167000170230860334186218096874654293060302734375f,-2.01260000413060158308553582173772156238555908203125f,-0.03780003906488449416656294488348066806793212890625f,0.161400020948052169700304148136638104915618896484375f,-0.5561000119090078186445680330507457256317138671875f,-1.139799974751472433354138047434389591217041015625f,-0.613700038242340095706595093361102044582366943359375f,1.77199994418025053022347492515109479427337646484375f,2.9222999947190277936215352383442223072052001953125f,0.55270001630783127666290965862572193145751953125f,-1.67299996149539964562791283242404460906982421875f,-0.1871000207304955864628936978988349437713623046875f,1.7463999938465655237251894504879601299762725830078125f,0.51319999089762557975546997113269753754138946533203125f,-0.23080000801086431039266244624741375446319580078125f,-0.342399950197339375534966166014783084392547607421875f,-0.04129999811649298635529703460633754730224609375f,-0.535500011414289378564035359886474907398223876953125f,-0.74879998537600034325123488088138401508331298828125f,0.3532999845415354389643880494986660778522491455078125f,-0.38820000635758000750996643546386621892452239990234375f,-2.5781000000476836930829449556767940521240234375f,-1.62070002667903878545985207892954349517822265625f,-0.9751999730944636013418858055956661701202392578125f,0.13730000231862060644516532192938029766082763671875f,-0.0190999063491830156635842286050319671630859375f,-0.68779995770454416259553909185342490673065185546875f,1.503899978479742660653073471621610224246978759765625f,1.14150003647804254569564363919198513031005859375f,-0.608699951815605544425125117413699626922607421875f,0.631000020146370133033997262828052043914794921875f,-0.1591999875187877222515453468076884746551513671875f,-0.3280999754607680785056800232268869876861572265625f,-0.6925999991387126186737077659927308559417724609375f,-0.836599967667460919784616635297425091266632080078125f,-0.522399996912479736010936903767287731170654296875f,-0.66959997870922105533963986090384423732757568359375f,0.25960002385973890426384969032369554042816162109375f,0.9016000026106834042138871154747903347015380859375f,0.722900007879733408344691270031034946441650390625f,1.0219000366091730569451101473532617092132568359375f,0.35989996830225035040484726778231561183929443359375f,-0.3959999945163730217245756648480892181396484375f,-1.36109994604587569710929528810083866119384765625f,-1.33020000438690200184055356658063828945159912109375f,0.594099980333447685865166931762360036373138427734375f,1.5076999722003936454939321265555918216705322265625f,0.502399986481666616100483224727213382720947265625f,-0.10089996386170430042739099008031189441680908203125f,0.211900047636031985121007892303168773651123046875f,0.009100010061263930083441664464771747589111328125f,0.216800043654441054741255356930196285247802734375f,0.519800007534027219691097343456931412220001220703125f,-0.600799967442453120014533851644955575466156005859375f,-0.100599945881963304117334701004438102245330810546875f,0.86009999146461524333062698133289813995361328125f,-0.06730003256797745248007913687615655362606048583984375f,-0.593600019881128648790991064743138849735260009765625f,0.193499998092651281922371708787977695465087890625f,-0.9189999673366548993413971402333118021488189697265625f,-0.8392999777354301915721634941291995346546173095703125f,-1.5834999550655488587125319099868647754192352294921875f,-1.7430999709904195782428359962068498134613037109375f,-0.2154999784827236197770616854541003704071044921875f,0.057900000429153664072146057151257991790771484375f,0.050200010049343379847641699598170816898345947265625f,-0.3555000240430230551424983787001110613346099853515625f,-0.0483000041365624355194086092524230480194091796875f,-0.79439998266696942863518415833823382854461669921875f,-0.5899999603629115796366022550500929355621337890625f,0.81880000087320770063570307684130966663360595703125f,-0.584499970927834677780765559873543679714202880859375f,-0.7371000076174727411171261337585747241973876953125f,1.554099906945229037091849022544920444488525390625f,1.813799927783012577720000990666449069976806640625f,-0.090299991977214943972285254858434200286865234375f,-0.763299998474121110803025658242404460906982421875f,-1.4355999720335006486493512056767940521240234375f,-0.012700021374225656245471327565610408782958984375f,0.9489000289201729998467271798290312290191650390625f,0.168500006735325058571106637828052043914794921875f,-0.456499984100461109193247466464526951313018798828125f,-0.096100002074241075433747028000652790069580078125f,0.362999955534935025980303180404007434844970703125f,0.6229999686479568143937513013952411711215972900390625f,0.792600009420513895719295760500244796276092529296875f,0.603500002264976131982621154747903347015380859375f,0.1948000245213510339681306504644453525543212890625f,-1.82179995305538255934152402915060520172119140625f,-1.004099995122849531270503575797192752361297607421875f,0.2625999978348609875666852531139738857746124267578125f,-1.1182999409556391157138932612724602222442626953125f,-0.441200030493735795289467205293476581573486328125f,0.593899966335296625175033113919198513031005859375f,0.810100005769729847315829829312860965728759765625f,-0.367000004231929466413930640555918216705322265625f,0.99799999196827382519359161960892379283905029296875f,1.595899951121211746141170806367881596088409423828125f,0.34780003399848880007994011975824832916259765625f,0.02310000004768397730003925971686840057373046875f,0.58979999315440689855449818423949182033538818359375f,-0.297000008031725659662924954318441450595855712890625f,-1.506399993753433363963267765939235687255859375f,-0.9292999986529348888097956660203635692596435546875f,0.302599990868568369251079275272786617279052734375f,1.5324999988079071044921875f,1.3200999806985265205838686597417108714580535888671875f,0.59229998145699536138408802798949182033538818359375f,-1.16779996850788680973209920921362936496734619140625f,-0.950000005885958120899204004672355949878692626953125f,0.2003999850809574656551603766274638473987579345703125f,-1.115100014784931925504452010500244796276092529296875f,-0.449799999785422954801106243394315242767333984375f,0.06579998834133160556802977225743234157562255859375f,0.9965999677419663527189186424948275089263916015625f,0.1438000186800962865163455717265605926513671875f,-0.98609999454915620020756250596605241298675537109375f,-0.293999985888599990602187972399406135082244873046875f,0.71219995348453579708802863024175167083740234375f,0.23760003497600479249740601517260074615478515625f,-0.841799961400031548919287160970270633697509765625f,-1.05280000896453884706716053187847137451171875f,-0.08229999591112147072635707445442676544189453125f,0.7884000003695490477184648625552654266357421875f,0.3072999673008922627559513784945011138916015625f,-1.137100008213519419797421505791135132312774658203125f,-0.0244999908208856087554750047274865210056304931640625f,-0.3211999569192529602190688819973729550838470458984375f,0.915999999731778924427771926275454461574554443359375f,0.3239999798908834716115734408958815038204193115234375f,-0.061499989986419478782408987171947956085205078125f,-0.564900001978873689267857116647064685821533203125f,-0.631900061978399651962945426930673420429229736328125f,0.1205999722942709961870377810555510222911834716796875f,0.305999985799193741087265152600593864917755126953125f,-0.329500019133091459622164620668627321720123291015625f,-0.01030000017285370716990655637346208095550537109375f,0.5728000295281407971970111248083412647247314453125f,-0.1622999795198438732768408954143524169921875f,0.0139999926686282805121663841418921947479248046875f,0.076999975323677194438687365618534386157989501953125f,0.4407999806299807143972202538861893117427825927734375f,-0.8974999734759325775712568429298698902130126953125f,-1.14480003376007033466521534137427806854248046875f,-0.2090999945640561463733320124447345733642578125f,-0.09069997944235819176128643448464572429656982421875f,0.66690001246929142553199199028313159942626953125f,-0.039499980986118288228681194595992565155029296875f,-0.1587000033736227777581007103435695171356201171875f,-0.3466999609947210014837537528364919126033782958984375f,1.02390001642182415508841586415655910968780517578125f,1.4213000133156779014598214416764676570892333984375f,-0.33439999339580506187985520227812230587005615234375f,-0.095099997648597334176656659110449254512786865234375f,-0.42999995768070231605406661401502788066864013671875f,0.744699951472879373426394522539339959621429443359375f,0.339100016319751063065268681384623050689697265625f,-0.0493999656796457742302663973532617092132568359375f,-0.0194999967813489494261602885671891272068023681640625f,-1.074199994149804826548688652110286056995391845703125f,0.2538999675273903022798549500294029712677001953125f,0.1726999388217933528721914626657962799072265625f,-0.08920003401041043389341211877763271331787109375f,0.0048000047028065040422006859444081783294677734375f,0.20840000059306618140908540226519107818603515625f,0.6790000122636559165556491279858164489269256591796875f,-0.257700021001696644162848315318115055561065673828125f,0.493399988113343734852378474897705018520355224609375f,0.0482999800339347640232290359563194215297698974609375f,-0.961700048315524469444426358677446842193603515625f,-1.041600000822544558332083397544920444488525390625f,-1.30999995276331926419288720353506505489349365234375f,0.7857000055193896770333594758994877338409423828125f,0.8728000266969206943912240603822283446788787841796875f,-0.5218000157147646422828302092966623604297637939453125f,0.16420000020340086432923953907447867095470428466796875f,0.864499975919724050754666677676141262054443359375f,-0.998199997174740616401322768069803714752197265625f,-2.7525999881863594254127747262828052043914794921875f,-2.652400000190734630223232670687139034271240234375f,0.1956999921083450999503838829696178436279296875f,0.10660001475512981272686374722979962825775146484375f,0.121199979159235748937817334081046283245086669921875f,2.64619995822310460908965978887863457202911376953125f,1.3008999902367595069563321885652840137481689453125f,0.84459997692108146338796359486877918243408203125f,-0.41249997496604873248315925593487918376922607421875f,-0.769500012502074337561452921363525092601776123046875f,-1.2086000309824935783353794249705970287322998046875f,-0.02180004752874342699442422599531710147857666015625f,-0.9458000130772585833938137511722743511199951171875f,-0.599800027203560404132076655514538288116455078125f,0.0368000265181063213049128535203635692596435546875f,-0.49719999625086774841520309564657509326934814453125f,-1.018100005300342392189349993714131414890289306640625f,-2.1239999827966098422393770306371152400970458984375f,-0.31829999869763891950924517004750669002532958984375f,0.793400014302135137000959730357863008975982666015625f,0.3197000065326687234801283921115100383758544921875f,0.2395000113844876210578149766661226749420166015625f,0.582699969816207641315486398525536060333251953125f,-0.3473999688982960520888809696771204471588134765625f,-0.3588999982237819352803853689692914485931396484375f,-0.5046000067234037711472183218575082719326019287109375f,0.385399987688660683460284417378716170787811279296875f,1.23269997823536403558364327182061970233917236328125f,0.350900054982303810646726560662500560283660888671875f,-0.605699970054626390947305480949580669403076171875f,0.5654999823570250327975372783839702606201171875f,0.2437999885797506038898063707165420055389404296875f,0.3126999474644662768696434795856475830078125f,1.046499974489211837180846487171947956085205078125f,-0.726499977469444502276019193232059478759765625f,-0.7598999453544621474065934307873249053955078125f,-0.468900005078315640361097393906675279140472412109375f,-1.310699981267750668934013447142206132411956787109375f,-0.5846999886140225033415163125027902424335479736328125f,0.060799976867437433014629277749918401241302490234375f,-0.329300004389881717514754200237803161144256591796875f,-0.606800006103515698896444519050419330596923828125f,-0.2464000053763388820016189129091799259185791015625f,-0.4738000034809111138400794516201131045818328857421875f,-1.777500002607702800361266781692393124103546142578125f,0.3555999589204787980634137056767940521240234375f,0.786200014009326597630433752783574163913726806640625f,-0.41859999914988910152402468156651593744754791259765625f,0.8545000364184376362430839435546658933162689208984375f,-0.420799971726536714555777507484890520572662353515625f,-1.00699996012449322080328784068115055561065673828125f,-0.130599993416666837475759166409261524677276611328125f,0.0078000019609927306873942143283784389495849609375f,0.39260001426339119490194207173772156238555908203125f,-0.002299971771240194584606797434389591217041015625f,-1.0664999718070031775596362422220408916473388671875f,-0.4548999642789368635931168682873249053955078125f,-0.521799994033574687790633106487803161144256591796875f,0.63859998046755794831597086158581078052520751953125f,-1.4320999581456188565198317519389092922210693359375f,-1.48779998154640225038747303187847137451171875f,0.14529997260570581829597358591854572296142578125f,0.66809995832443203056527636363171041011810302734375f,0.3494000336289406760670317453332245349884033203125f,-1.4689999569058418416034328402020037174224853515625f,-1.6473999882698056040908340946771204471588134765625f,-1.737099956059456218326886300928890705108642578125f,0.71549995791912124332156963646411895751953125f,0.2318000049114221639001698349602520465850830078125f,-0.71569999821782115390078615746460855007171630859375f,-0.370300038766860240002642967738211154937744140625f,0.281199994321167157096397204441018402576446533203125f,-0.4382999627485870686172120258561335504055023193359375f,-0.846700000929832441443068091757595539093017578125f,-0.1323000021696092431966462754644453525543212890625f,-0.2126999912738796183475642465054988861083984375f,-0.81820002697706240013530987198464572429656982421875f,-1.685499976098537189272974501363933086395263671875f,-1.1862000105261802929135228623636066913604736328125f,-0.9384999614357951713827787898480892181396484375f,-1.022899962729216127854670048691332340240478515625f,1.28109999134540597509612780413590371608734130859375f,0.306500022992491238937873276881873607635498046875f,-0.272800011721253010676946360035799443721771240234375f,1.970899961030482927526463754475116729736328125f,1.1512000141024587573923554373322986066341400146484375f,1.348699971410632247881267176126129925251007080078125f,1.40209999659061423926686984486877918243408203125f,0.0661999735713010295512503944337368011474609375f,-0.091200050759315587356468313373625278472900390625f,-0.19329997284412403502074084826745092868804931640625f,-0.89460000374317161231374484486877918243408203125f,0.26659998145103447342307845246978104114532470703125f,1.230800026190280505034024827182292938232421875f,-0.68649997329711942573027272374019958078861236572265625f,-0.8263999957278371777391612340579740703105926513671875f,0.2794999923110008666071735206060111522674560546875f,-1.005600012564658474190082415589131414890289306640625f,0.3652999441072346531456105367396958172321319580078125f,2.1856999551534652681539228069595992565155029296875f,1.51349999734759332881139926030300557613372802734375f,2.147399975454806764219028991647064685821533203125f,0.7603000593304631848923236248083412647247314453125f,1.9155000520944582831361913122236728668212890625f,2.28830006046295153510072850622236728668212890625f,0.3643000308394430675207331660203635692596435546875f,0.644199997949600255964242023765109479427337646484375f,-1.4792999641194943816202567177242599427700042724609375f,-2.05140001401901184152620771783404052257537841796875f,0.96039996353015288921284309253678657114505767822265625f,1.5555000118613246939958116854541003704071044921875f,0.3940999839842318497318274239660240709781646728515625f,-1.446199997189640651384934244561009109020233154296875f,0.3363999407053004375711680040694773197174072265625f,0.73829997403621661078432225622236728668212890625f,0.2907999832749368351869634352624416351318359375f,0.657000010788440391706899390555918216705322265625f,1.6020999992728235383765422739088535308837890625f,0.1182999928116803545208313153125345706939697265625f,-0.4506000360153616934866249721380881965160369873046875f,-0.12320000415965914886129439764772541821002960205078125f,-0.621899969553947329359289142303168773651123046875f,-0.9836999872803691147282734164036810398101806640625f,-0.000299995851516854372675879858434200286865234375f,-0.29609999223947536250989287509582936763763427734375f,0.44669996933937117233881508582271635532379150390625f,-0.012000034853814955937423292198218405246734619140625f,-0.729000015616417584851660649292171001434326171875f,0.3187000033736229198666478623636066913604736328125f,0.0104999802708629630387804354541003704071044921875f,0.127199987161159100423901691101491451263427734375f,0.17749996528029488018773918156512081623077392578125f,0.438899994945526117362533113919198513031005859375f,-0.377399977354705651322319681639783084392547607421875f,0.471999994602054329106977093033492565155029296875f,0.01980004849359351393189854206866584718227386474609375f,-0.217199964436888937058256487944163382053375244140625f,0.3855000216960906556096233543939888477325439453125f,-0.237900042152404012085753493010997772216796875f,0.5796999534845355128709343262016773223876953125f,-0.749399996972083926038976642303168773651123046875f,-0.564700008916855011875668424181640148162841796875f,0.6250999717950822542889000033028423786163330078125f,0.737299953889847703436544179567135870456695556640625f,-0.19240001315474497545210397220216691493988037109375f,-0.50910003584027219147856158087961375713348388671875f,0.1395999411582951044152878239401616156101226806640625f,-1.19280001073405106382097073947079479694366455078125f,-0.7450000253319732923529272738960571587085723876953125f,-0.190200041118264717709962496883235871791839599609375f,-0.31159997906684822055467520840466022491455078125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

TEST (${BACKEND_NAME}, convolution_3d_2item_large_5o3i_padded_strided_uneven_filter_uneven_data_dilation_filter_dilated_data_dilated)
{
    SKIP_TEST_FOR("GPU", "${BACKEND_NAME}");
    Shape shape_a{2,3,8,8,8};
    auto A = make_shared<op::Parameter>(element::f32, shape_a);
    Shape shape_b{5,3,2,3,4};
    auto B = make_shared<op::Parameter>(element::f32, shape_b);
    Shape shape_r{2,5,8,7,7};
    auto make_graph = [A, B] {
        return make_shared<Function>(make_shared<op::Convolution>(A, B,
                                                                  Strides{2,3,2},        // move_strides
                                                                  Strides{3,2,2},        // filter_dilation
                                                                  CoordinateDiff{2,1,2}, // below_pads
                                                                  CoordinateDiff{1,2,3}, // above_pads
                                                                  Strides{2,3,2}),       // data_dilation
                                     op::Parameters{A, B});
    };

    auto manager = runtime::Manager::get("${BACKEND_NAME}");
    auto external = manager->compile(make_graph());
    auto backend = manager->allocate_backend();
    auto cf = backend->make_call_frame(external);

    // Create some tensors for input/output
    auto a = backend->make_primary_tensor_view(element::f32, shape_a);
    copy_data(a, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f,-.87f,.43f,-.17f,.95f,-.96f,.36f,-.57f,.57f,-.47f,.58f,.35f,.52f,.23f,.66f,-.18f,.54f,-.78f,.38f,.34f,-.73f,-.74f,.40f,-.92f,-.28f,.21f,-.20f,-.47f,-.12f,.72f,-.61f,-.59f,.53f,-.10f,-.06f,.98f,.87f,.50f,.34f,.05f,.30f,.89f,-.56f,.45f,-.02f,.39f,-.97f,.34f,-.43f,-.82f,.35f,.32f,-.31f,.97f,-.05f,.47f,.29f,-.37f,-.22f,-.74f,-.59f,-.02f,-.58f,.59f,.63f,.06f,-.34f,.35f,-.18f,-.21f,.07f,-.09f,-.53f,.97f,.34f,-.90f,-.17f,-.85f,.91f,-.92f,.80f,-.97f,.50f,.84f,.25f,-.60f,.39f,-.57f,-.32f,.29f,.68f,.52f,-.24f,.40f,.19f,.85f,.87f,-.55f,-.61f,-.46f,-.54f,.69f,-.58f,.05f,.79f,.30f,-.22f,.02f,-.77f,-.15f,-.47f,.74f,-.30f,.02f,-.71f,.19f,.34f,.68f,.85f,.73f,-.63f,.27f,.76f,.73f,.12f,.36f,.82f,.18f,-.49f,-.27f,.48f,.96f,.89f,-.52f,.16f,.50f,.01f,.71f,.53f,-.24f,.23f,-.01f,-.79f,-.34f,-.03f,.04f,-.44f,.21f,-.36f,-.32f,.40f,.03f,-.71f,-.00f,-.05f,.74f,.50f,.16f,-.22f,-.72f,-.99f,-.29f,-.60f,-.74f,-.49f,-.33f,-.40f,.97f,-.10f,-.85f,-.26f,-.05f,-.61f,-.38f,.52f,-.87f,-.23f,.05f,.03f,.08f,.66f,-.07f,-.70f,.32f,-.59f,.99f,.02f,.79f,.45f,-.92f,-.20f,-.11f,.35f,.83f,.65f,-.44f,.56f,.67f,.48f,-.74f,.94f,.00f,-.13f,-.87f,.99f,-.18f,-.40f,.53f,.00f,-.35f,-.28f,-.36f,-.20f,.13f,.24f,-.19f,.72f,.61f,-.96f,-.43f,.26f,-.12f,-.64f,-.64f,.72f,.15f,-.93f,.36f,-.88f,-.56f,.99f,.74f,.60f,.82f,.42f,.45f,.48f,-.90f,.29f,.25f,-.40f,-.36f,-.89f,-.56f,.84f,-.71f,-.43f,-.72f,.69f,.73f,.54f,-.55f,.62f,.64f,-.46f,-.36f,-.63f,-.37f,.63f,.47f,-.50f,.05f,-.07f,.67f,-.76f,-.47f,.24f,-.01f,.23f,.10f,.39f,-.93f,.14f,-.85f,-.13f,.82f,.24f,-.62f,-.47f,-.10f,.29f,-.37f,.94f,.28f,.88f,-.23f,-.14f,.40f,.35f,-.34f,.12f,.93f,.28f,-.99f,-.23f,-.63f,.24f,.37f,-.74f,-.76f,-.57f,.29f,-.31f,.56f,-.99f,.06f,.52f,.41f,.56f,.80f,.36f,.05f,-.01f,-.58f,.63f,-.82f,-.86f,.86f,.77f,-.28f,-.56f,-.54f,-.07f,.12f,.47f,.71f,-.01f,-.92f,.24f,-.13f,-.70f,.22f,.05f,-.39f,-.66f,-.58f,-.11f,.90f,-.29f,.31f,-.65f,.48f,.26f,-.47f,.75f,-.13f,.11f,-.36f,.11f,-.62f,.95f,-.90f,-.26f,.22f,-.46f,-.20f,-.41f,-.59f,.82f,-.12f,.48f,-.02f,-.95f,.53f,-.31f,-.72f,-.69f,.82f,.05f,-.34f,-.67f,.39f,-.92f,-.88f,-.15f,-.67f,-.00f,-.69f,.81f,.49f,-.10f,-.44f,.62f,-.68f,.23f,-.74f,-.00f,-.25f,.75f,-.62f,.54f,.53f,.34f,-.67f,-.64f,-.56f,-.80f,.33f,-.36f,-.49f,.02f,.05f,-.41f,-.39f,-.99f,.30f,.07f,.91f,-.92f,.95f,.03f,.63f,.34f,-.93f,.65f,-.63f,-.67f,-.11f,.97f,.21f,-.81f,.85f,.12f,.86f,-.44f,.97f,-.67f,-.24f,-.19f,.56f,-.95f,.27f,-.01f,.35f,-.34f,-.19f,.62f,-.62f,.65f,.24f,.48f,-.64f,.40f,-.60f,.69f,-.83f,-.50f,.21f,.75f,.60f,.36f,-.36f,.29f,-.89f,-.41f,-.53f,-.60f,.06f,.95f,.08f,.14f,.77f,.72f,-.74f,-.26f,-.86f,-.33f,-.81f,.89f,.13f,-.07f,.06f,.38f,-.68f,.92f,.99f,.31f,-.58f,-.82f,-.07f,.01f,.19f,.60f,.12f,.16f,.65f,.11f,.35f,-.97f,.34f,.52f,.43f,.59f,.40f,.71f,-.00f,.57f,.26f,.46f,.52f,-.59f,.43f,.96f,.54f,-.75f,-.28f,.00f,.93f,-.99f,-.31f,-.15f,-.59f,.53f,-.03f,-.94f,.10f,.82f,.54f,.57f,-.28f,.93f,.91f,.12f,-.10f,.39f,-.26f,.07f,-.39f,-.06f,.90f,.43f,.51f,-.77f,.20f,.84f,.06f,-.18f,-.25f,.95f,.27f,.46f,.48f,.66f,.48f,-.07f,.21f,.67f,.88f,-.44f,-.58f,-.77f,-.28f,.56f,-.43f,-.19f,-.93f,.20f,-.64f,.72f,-.14f,.81f,-.04f,-.75f,.70f,.74f,-.38f,.38f,-.97f,-.42f,-.74f,.85f,-.72f,.40f,.70f,.81f,-.22f,-.74f,-.66f,-.49f,-.58f,.02f,-.13f,-.39f,.47f,-.81f,-.14f,.32f,-.57f,-.99f,.40f,-.96f,-.54f,.71f,.91f,.55f,-.18f,-.16f,.25f,-.49f,-.98f,-.09f,-.93f,.72f,.35f,-.84f,-.79f,-.97f,.17f,-.28f,.41f,.51f,.02f,.30f,-.18f,-.93f,-.77f,.85f,-.93f,.95f,-.82f,-.37f,.21f,.62f,-.94f,.98f,.47f,.73f,-.37f,-.14f,-.26f,.28f,-.09f,-.96f,.34f,.60f,.81f,-.29f,.17f,-.07f,.58f,.00f,-.15f,-.97f,-.15f,.76f,-.83f,.66f,-.21f,-.01f,-.51f,.28f,.96f,-.08f,.79f,-.24f,.45f,.92f,.37f,.03f,-.24f,-.92f,-.47f,-.11f,.03f,-.15f,-.45f,-.84f,.14f,.06f,-.32f,.58f,.80f,-.36f,-.70f,.04f,-.75f,.05f,.97f,-.44f,-.00f,-.98f,-.37f,.77f,-.55f,-.84f,-.08f,-.97f,-.40f,.84f,.35f,.66f,-.30f,.86f,-.44f,-.79f,-.28f,-.40f,-.53f,-.58f,-.31f,.27f,.25f,.68f,-.66f,.01f,-.13f,.49f,-.39f,-.89f,-.33f,.38f,.11f,-.23f,.27f,-.50f,-.20f,-.90f,-.25f,.50f,.13f,.82f,.40f,-.43f,.51f,.75f,-.96f,.02f,-.65f,.95f,.62f,-.42f,.71f,.43f,.42f,.95f,-.22f,-.88f,.90f,-.73f,-.35f,-.28f,-.36f,-.49f,.35f,.04f,-.86f,.23f,-.56f,.11f,-.70f,-.68f,-.59f,-.97f,-.61f,.55f,-.11f,-.64f,.14f,.12f,-.31f,-.55f,-.20f,.39f,.57f,-.80f,.66f,.60f,-.68f,.31f,.90f,.12f,-.88f,.67f,.86f,-.00f,.84f,-.41f,.78f,.80f,-.21f,-.35f,-.45f,-.47f,.56f,-.01f,-.23f,.19f,-.59f,.64f,.66f,-.32f,-.45f,-.99f,.08f,-.05f,-.03f,-.71f,.66f,.93f,.18f,-.15f,-.78f,.91f,.29f,-.26f,-.21f,-.05f,.11f,-.92f,.72f,.43f,.00f,-.58f,-.70f,.58f,-.73f,.16f,.67f,-.44f,-.24f,.37f,.64f,.67f,.00f,-.34f,-.48f,-.68f,.54f,.80f,.59f,.50f,-.89f,-.26f,-.11f,-.72f,.57f,.46f,-.85f,.31f,.89f,.50f,.73f,.23f,.29f,.75f,-.80f,.36f,-.56f,-.62f,.21f,-.49f,-.07f,.17f,.68f,-.12f,.17f,-.31f,-.09f,.88f,.33f,.79f,.99f,.32f,.56f,-.43f,-.56f,-.70f,.02f,.23f,-.95f,-.87f,.14f,.33f,-.26f,.04f,.25f,-.87f,-.64f,.20f,.27f,-.44f,.30f,-.47f,.24f,-.40f,.95f,-.27f,.74f,.73f,.01f,.74f,-.21f,.24f,.18f,-.08f,-.01f,.63f,-.32f,.13f,.11f,.41f,.24f,-.82f,-.89f,-.85f,-.51f,-.62f,-.77f,.06f,-.09f,-.35f,-.56f,.82f,.92f,-.18f,.06f,-.28f,.45f,.69f,-.58f,.66f,-.37f,-.90f,-.41f,-.42f,-.58f,-.73f,.92f,.14f,-.64f,.53f,-.42f,-.38f,.51f,-.94f,-.05f,-.91f,-.92f,-.38f,-.73f,.56f,.64f,-.46f,-.58f,.91f,-.15f,-.28f,-.71f,-.36f,-.89f,-.21f,-.87f,-.97f,.09f,-.34f,-.58f,-.54f,.77f,.30f,.70f,-.38f,-.70f,.79f,.44f,.26f,.77f,-.49f,-.29f,-.61f,-.62f,-.56f,.17f,.78f,-.67f,.33f,.35f,-.47f,.90f,.40f,.66f,-.26f,-.82f,-.56f,-.06f,.08f,-.91f,.77f,.49f,.03f,.80f,-.06f,-.19f,-.44f,.65f,.21f,-.13f,.01f,-.56f,.48f,.66f,.64f,-.71f,.42f,.89f,.73f,.65f,-.38f,.15f,.16f,-.47f,-.94f,-.79f,.83f,.97f,-.49f,.82f,.42f,-.64f,-.23f,.35f,-.01f,.97f,-.84f,-.50f,.97f,-.33f,.22f,-.52f,.70f,.30f,.33f,.26f,.64f,.38f,.22f,-.51f,.95f,.71f,.50f,.96f,.10f,.07f,-.52f,-.54f,-.03f,-.53f,-.19f,-.34f,-.46f,.74f,-.16f,-.53f,-.84f,.83f,-.51f,.10f,-.61f,.60f,-.85f,-.01f,.69f,.50f,-.96f,.77f,-.83f,.43f,.21f,.48f,.72f,-.18f,.20f,-.80f,.19f,-.14f,.79f,-.91f,.91f,.23f,.23f,.56f,.92f,.24f,.15f,-.12f,.09f,-.56f,.89f,-.03f,-.87f,-.23f,-.68f,-.75f,-.42f,.06f,.29f,.74f,.14f,.89f,.02f,.24f,.77f,-.79f,.60f,-.64f,.22f,.56f,.51f,-.25f,-.52f,.28f,-.10f,.82f,.01f,-.35f,-.48f,.55f,.40f,-.64f,-.43f,.65f,.75f,-.17f,-.11f,.29f,-.77f,.57f,-.17f,.71f,-.49f,-.95f,.58f,.22f,.29f,.06f,.85f,-.59f,-.43f,.43f,.77f,.16f,-.42f,-.85f,-.43f,.93f,.44f,.07f,.29f,.95f,.53f,.29f,-.20f,-.04f,.29f,.61f,.11f,-.98f,.92f,-.02f,.71f,.68f,-.72f,-.81f,.22f,.02f,.84f,.30f,.16f,-.09f,-.61f,.46f,.72f,-.31f,.54f,.83f,-.85f,.15f,-.66f,-.87f,.77f,.46f,.22f,-.72f,.30f,.64f,.20f,.83f,-.33f,-.77f,-.65f,.39f,.30f,-.97f,.75f,.68f,-.69f,.50f,.36f,.26f,-.37f,-.77f,.99f,-.80f,-.35f,.40f,.07f,.78f,.06f,.85f,.34f,.48f,.49f,.09f,.47f,-.38f,.40f,-.93f,-.49f,.18f,-.59f,.02f,.26f,.17f,-.00f,.21f,-.94f,-.35f,-.27f,.53f,-.69f,.15f,.26f,.80f,-.56f,.96f,-.56f,.11f,.00f,.07f,.11f,-.40f,.76f,-.83f,-.81f,.45f,.42f,.08f,.27f,.90f,.27f,.75f,-.97f,.34f,.69f,-.44f,.48f,.23f,.00f,-.07f,.86f,.50f,.03f,.08f,.52f,-.19f,-.88f,.26f,-.25f,.86f,-.44f,.41f,-.48f,.17f,.54f,-.64f,.39f,.10f,.01f,-.79f,-.41f,.29f,-.30f,.50f,-.30f,-.98f,-.35f,-.93f,.69f,-.46f,.35f,-.00f,.27f,.35f,.69f,.15f,.15f,-.49f,.64f,-.64f,-.98f,.05f,-.37f,-.26f,-.41f,-.18f,.64f,.76f,-.79f,-.70f,.33f,.16f,-.69f,-.08f,-.32f,.28f,-.28f,.27f,.49f,.15f,.67f,-.18f,-.43f,.76f,-.25f,-.12f,-.93f,.44f,.59f,.28f,.60f,-.74f,.33f,-.78f,.64f,.99f,-.66f,.91f,.54f,-.00f,-.72f,-.80f,.03f,.71f,.55f,-.05f,.25f,-.43f,-.12f,-.81f,-.64f,-.69f,-.49f,.69f,.26f,.79f,-.53f,.63f,.79f,.29f,-.19f,.17f,.87f,-.57f,.79f,.13f,.28f,.40f,.24f,.26f,-.60f,.18f,.01f,.69f,-.74f,.76f,.28f,.12f,-.99f,-.30f,-.60f,-.16f,-.69f,.05f,-.46f,.23f,.41f,-.08f,.84f,-.87f,-.69f,.14f,-.39f,.71f,.68f,.07f,.62f,.20f,-.91f,-.32f,-.28f,.62f,.39f,.38f,.62f,-.34f,.81f,-.88f,-.50f,-.90f,-.01f,-.92f,.92f,.59f,.11f,.07f,.39f,-.96f,-.66f,-.40f,-.65f,-.51f,.33f,.38f,-.57f,-.26f,-.65f,.17f,.04f,.98f,.25f,-.53f,.27f,-.58f,-.34f,-.15f,-.76f,.97f,-.30f,.79f,.96f,.16f,-.59f,-.59f,-.26f,-.67f,-.22f,.86f,.78f,-.73f,-.81f,-.58f,.12f,-.35f,-.14f,.96f,.65f,.90f,-.19f,-.56f,.79f,-.04f,-.55f,.54f,.64f,-.18f,-.16f,-.76f,-.22f,.48f,-.22f,-.95f,.49f,.71f,.79f,-.51f,-.29f,-.92f,.06f,-.57f,-.51f,.58f,.76f,-.42f,-.04f,-.67f,-.13f,-.68f,.04f,.34f,-.15f,-.50f,.71f,-.36f,-.39f,.07f,.38f,.24f,-.91f,-.18f,-.40f,.30f,-.10f,.96f,-.64f,.63f,-.39f,-.41f,.65f,.48f,-.64f,-.05f,-.63f,.90f,-.86f,.81f,-.06f,-.07f,.33f,.49f,-.88f,-.19f,.72f,-.66f,-.92f,-.98f,-.66f,.49f,.27f,.15f,-.27f,.43f,.15f,.31f,.87f,.16f,.51f,-.45f,-.84f,.67f,-.35f,.59f,-.37f,-.15f,.11f,.12f,.75f,-.06f,.83f,.41f,.19f,-.48f,.81f,-.23f,.11f,.91f,.34f,-.44f,-.67f,.91f,.07f,.23f,.90f,-.30f,-.52f,.65f,-.68f,.70f,-.82f,.44f,.08f,.05f,.15f,-.07f,.34f,-.42f,.52f,-.41f,-.77f,-.74f,.81f,-.12f,.87f,.63f,.39f,.34f,.81f,-.95f,.55f,.96f,-.06f,.24f,.77f,.09f,.41f,-.30f,-.62f,.92f,.01f,-.65f,.26f,-.35f,.44f,-.14f,-.57f,-.88f,-.70f,-.24f,-.56f,.70f,-.84f,-.13f,.84f,-.70f,-.81f,-.86f,.66f,.55f,-.13f,-.73f,-.02f,.26f,.57f,-.71f,-.56f,-.46f,-.28f,.09f,.14f,.14f,-.07f,.76f,.46f,.71f,.56f,.42f,-.35f,-.48f,-.25f,-.26f,.72f,-.69f,.38f,-.56f,.65f,.55f,.87f,.40f,-.53f,.68f,-.52f,.54f,-.80f,-.19f,-.34f,.96f,.24f,-.97f,-.55f,-.28f,.62f,-.28f,.44f,-.01f,-.49f,.73f,-.29f,-.47f,-.72f,-.35f,.46f,.30f,-.83f,-.15f,-.74f,-.02f,-.01f,-.58f,.37f,-.69f,-.73f,-.83f,-.69f,.53f,-.72f,.13f,.52f,-.40f,-.97f,.70f,-.72f,-.50f,-.78f,.78f,-.71f,-.16f,.44f,.70f,.62f,.93f,-.11f,.08f,.94f,-.27f,.61f,.24f,-.64f,.91f,.70f,.53f,-.83f,-.21f,-.28f,.71f,.78f,-.25f,-.01f,-.45f,-.58f,.36f,-.28f,.22f,.88f,-.90f,-.03f,.97f,.26f,-.93f,.59f,.98f,-.15f,.09f,.51f,.26f,.21f,.83f,.24f,.53f,-.16f,.99f,.43f,.75f,.40f,-.85f,.62f,-.40f,.69f,-.13f,.02f,.18f,.61f,-.39f,.06f,-.89f,-.83f,-.14f,-.10f,.86f,.91f,.64f,-.80f,-.83f,.16f,-.01f,-.20f,.26f,.97f,-.76f,.25f,-.42f,.29f,.63f,.07f,.03f,.65f,-.93f,.91f,-.44f,.86f,.81f,.89f,.85f,-.11f,-.59f,.47f,-.16f,.00f,.50f,-.40f,-.27f,-.90f,.36f,-.10f,-.10f,-.69f,-.12f,.01f,.88f,.94f,.54f,.44f,.16f,.42f,.32f,-.00f,.97f,.96f,-.03f,.20f,.59f,.17f,-.38f,.23f,.88f,.82f,-.02f,.56f,.02f,-.17f,.08f,-.88f,.91f,-.95f,.03f,-.79f,.03f,-.54f,.21f,.07f,-.28f,.22f,.95f,-.72f,.04f,.03f,-.32f,.80f,-.21f,-.76f,-.95f,-.96f,-.10f,.80f,-.76f,.96f,-.50f,.33f,.71f,-.45f,.00f,-.52f,-.94f,-.79f,.21f,.71f,.68f,-.93f,-.79f,-.93f,-.84f,.78f,-.60f,.07f,-.83f,-.06f,-.61f,-.65f,.36f,.72f,.14f,-.18f,.61f,-.75f,-.30f,-.62f,-.21f,.52f,-.27f,.87f,-.01f,.10f,.24f,.89f,-.89f,-.67f,-.79f,-.83f,-.45f,.32f,-.26f,.76f,-.01f,-.36f,-.64f,.49f,-.18f,.74f,.38f,.36f,-.62f,.41f,.69f,-.07f,-.34f,-.99f,-.00f,.91f,-.82f,.00f,.07f,.19f,-.79f,-.62f,.62f,-.58f,-.83f,-.10f,.83f,.91f,-.85f,-.95f,.93f,.64f,.43f,-.95f,-.43f,.04f,-.41f,.14f,.41f,-.74f,-.70f,.76f,-.53f,-.13f,.69f,.51f,-.39f,.83f,.18f,.47f,-.90f,.28f,.20f,-.25f,.95f,.62f,.08f,-.00f,-.56f,-.68f,.29f,-.27f,-.32f,-.22f,.83f,.60f,.44f,.02f,.61f,.39f,.03f,-.33f,-.57f,-.01f,-.59f,-.38f,.30f,-.26f,-.95f,-.18f,-.08f,-.78f,.00f,-.51f,-.72f,.53f,.37f,-.50f,-.87f,.68f,-.78f,.17f,.08f,-.60f,-.31f,.06f,.67f,-.46f,-.78f,.68f,.40f,-.76f,.65f,-.43f,-.60f,.56f,.59f,-.65f,.51f,-.09f,-.60f,-.34f,.19f,-.03f,-.10f,.81f,-.65f,-.85f,-.05f,-.62f,-.24f,.52f,-.33f,.36f,-.25f,.96f,-.76f,-.68f,-.43f,.48f,.76f,-.60f,.85f,-.29f,.61f,-.30f,-.68f,.26f,.08f,.38f,.07f,.44f,.73f,-.92f,.50f,.81f,-.37f,.89f,-.28f,-.99f,.28f,-.63f,-.65f,.97f,.84f,-.86f,.20f,-.52f,.78f,-.16f,-.06f,-.23f,.35f,-.98f,.57f,.44f,-.15f,.06f,.50f,-.66f,-.60f,-.39f,-.39f,.75f,.05f,.17f,.95f,-.71f,-.87f,-.80f,.19f,-.60f,.61f,.95f,.87f,-.73f,-.23f,-.54f,.11f,-.99f,.74f,-.29f,-.60f,-.65f,.46f,-.97f,-.49f,-.74f,.33f,-.74f,.06f,.19f,.08f,-.70f,-.32f,.80f,.44f,-.34f,-.26f,-.39f,-.10f,-.84f,.47f,-.12f,-.46f,.16f,-.05f,-.34f,.89f,-.06f,.11f,-.06f,-.24f,-.59f,-.00f,.33f,.32f,-.38f,.38f,.10f,.00f,-.59f,.77f,-.17f,-.63f,-.87f,-.34f,-.95f,.71f,-.76f,-.54f,-.29f,-.63f,-.84f,-.27f,.57f,.11f,-.87f,-.57f,-.52f,.34f,-.51f,-.32f,.61f,-.06f,.97f,.44f,.34f,.23f,.38f,.23f,.22f,.01f,-.26f,-.72f,.27f,.81f,-.28f,.68f,-.80f,.97f,.74f,.35f,-.96f,-.33f,.89f,-.82f,.06f,-.88f,.42f,.70f,-.34f,.56f,-.67f,.07f,.22f,-.36f,-.42f,-.20f,.95f,.52f,.22f,.75f,-.30f,.16f,.57f,-.52f,.10f,-.05f,.00f,.47f,.45f,.26f,-.98f,-.27f,-.10f,-.38f,-.60f,.39f,-.74f,.05f,.98f,.77f,-.92f,-.33f,.86f,.08f,-.84f,-.22f,.40f,-.98f,-.50f,-.55f,.57f,.89f,-.43f,.02f,-.62f,.36f,-.60f,-.74f,.68f,.56f,.02f,-.56f,-.97f,.62f,.86f,-.59f,.94f,.91f,-.65f,.94f,-.28f,.61f,.33f,.93f,.64f,.17f,.54f,.31f,-.92f,.89f,-.68f,-.51f,-.84f,-.56f,-.92f,-.96f,-.16f,-.56f,-.84f,-.02f,.80f,-.14f,.87f,.04f,.38f,.21f,.64f,-.97f,-.39f,-.49f,-.12f,-.30f,-.51f,-.25f,.06f,.53f,.67f,-.12f,-.53f,-.92f,.96f,-.01f,.47f,.51f,.71f,.30f,-.63f,-.98f,-.26f,-.22f,.82f,.56f,-.13f,-.22f,.78f,.91f,-.21f,.78f,.42f,-.86f,-.47f,.14f,.23f,.10f,-.32f,-.57f,-.32f,.84f,-.57f,-.15f,.20f,.85f,.05f,-.57f,.83f,.94f,.50f,-.74f,-.04f,-.80f,-.86f,.52f,-.01f,-.28f,.28f,-.53f,.94f,.50f,-.82f,.71f,-.27f,-.50f,-.38f,-.38f,-.30f,.52f,-.14f,-.25f,-.89f,.25f,-.56f,-.91f,-.00f,-.26f,.92f,-.75f,-.36f,-.30f,-.72f,.65f,-.71f,-.38f,.92f,-.48f,-.76f,.21f,.40f,-.52f,-.01f,-.90f,.88f,-.55f,-.04f,-.30f,-.20f,.60f,.69f,-.51f,-.57f,.90f,-.37f,.98f,-.05f,.73f,-.79f,-.63f,.22f,.13f,.83f,.34f,-.10f,.38f,.94f,-.44f,.70f,-.61f,.85f,-.20f,-.00f,.17f,-.02f,-.38f,-.04f,-.44f,.68f,.94f,.51f,-.72f,-.38f,-.63f,.33f,.28f,.02f,-.76f,.59f,.03f,.47f,-.71f,-.30f,-.67f,-.61f,.46f,.88f,.28f,.25f,-.59f,.68f,.99f,.03f,.34f,.42f,-.02f,-.53f,-.67f,.10f,-.79f,-.05f,.35f,.98f,.24f,.60f,-.07f,.40f,.91f,-.44f,-.88f,-.56f,-.74f,.33f,-.95f,-.13f,.10f,-.36f,.25f,.64f,-.19f,-.03f,.73f,.93f,.08f,-.79f,.16f,.10f,-.98f,.78f,-.93f,.41f,.95f,.74f,.33f,.82f,.08f,-.81f,.57f,-.85f,-.59f,.70f,.69f,-.10f,-.44f,-.34f,-.20f,-.66f,.65f,-.25f,-.05f,.78f,.65f,.26f,-.02f,-.82f,.01f,.44f,.68f,.21f,.96f,-.27f,-.02f,-.16f,.57f,-.77f,-.40f,.16f,-.51f,-.58f,-.65f,.41f,.19f,-.83f,-.47f,.04f,-.15f,-.64f,.36f,-.43f,-.44f,-.35f,.63f,.33f,-.15f,.31f,.69f,-.79f,-.74f,-.56f,-.94f,-.18f,-.40f,-.42f,.45f,.70f,-.68f,-.92f,-.76f,.66f,.47f,-.68f,.28f,-.37f,-.64f,-.60f,.67f,.90f,-.00f,.13f,-.60f,.49f,-.92f,.76f,.95f,-.06f,.32f,-.23f,-.02f,-.55f,.66f,.27f,-.32f,.91f,.38f,-.33f,.30f,-.28f,-.02f,-.33f,.50f,.70f,-.23f,-.67f,-.27f,-.38f,-.01f,-.31f,.84f,-.10f,.53f,-.07f,-.48f,-.46f,.60f,.53f,-.53f,-.58f,.48f,.51f,-.67f,.68f,-.08f,.31f,.50f,.21f,.44f,-.66f,.74f,-.01f,.97f,.98f,.69f,.13f,-.56f,.64f,-.51f,-.33f,-.31f,.12f,.25f,.28f,-.52f,.54f,.16f,.38f,.74f,.87f,-.67f,-.99f,-.91f,-.42f,-.56f,-.48f,.15f,-.20f,-.88f,.53f,.25f,.47f,.98f,-.96f,-.19f,.50f,.81f,.13f,.81f,-.07f,.44f,.57f,.37f,.86f,-.33f,.68f,-.86f,-.27f,.47f,-.04f,-.15f,-.56f,.35f,-.41f,.78f,-.52f,.15f,.80f,.80f,-.98f,-.33f,-.43f,.43f,.38f,-.52f,.20f,-.95f,-.55f,.50f,-.53f,-.54f,.66f,.19f,.56f,.70f,-.55f,-.96f,.51f,-.59f,-.63f,.64f,-.59f,.97f,-.38f,.81f,.43f,-.76f,.26f,.75f,-.29f,-.06f,-.53f,-.17f,-.51f,-.86f,-.72f,-.97f,-.15f,.05f,.73f,-.03f,-.91f,-.77f,.71f,-.82f,-.78f,.32f,-.91f,-.51f,.33f,-.02f,.13f,.83f,-.42f,-.30f,-.07f,.97f,.77f,.34f,-.84f,.24f,-.45f,-.21f,-.81f,.59f,-.61f,.57f,-.53f,-.64f,-.38f,-.86f,-.65f,.20f,-.16f,-.01f,-.09f,.66f,.64f});
    auto b = backend->make_primary_tensor_view(element::f32, shape_b);
    copy_data(b, vector<float>{-.83f,.21f,-.45f,-.22f,.90f,.99f,-.06f,-.31f,-.80f,.54f,-.88f,.13f,.25f,.88f,-.25f,.61f,.99f,-.75f,.34f,.02f,.65f,-.17f,-.46f,-.26f,.75f,-.09f,-.11f,.75f,-.42f,-.93f,-.97f,-.89f,.15f,.06f,-.30f,-.40f,-.47f,-.78f,-.44f,-.33f,-.68f,-.18f,.08f,.74f,-.76f,-.87f,-.38f,.74f,-.23f,-.25f,.81f,.65f,-.14f,-.14f,-.67f,.42f,-.75f,.12f,.11f,.40f,.05f,-.08f,.27f,.29f,.95f,.38f,.63f,.58f,.67f,-.62f,-.51f,-.24f,-.18f,-.12f,-.40f,.74f,.11f,.10f,.55f,-.94f,.98f,.72f,.34f,.92f,-.31f,.96f,-.53f,-.09f,.05f,-.32f,-.50f,-.96f,-.49f,-.71f,-.96f,.19f,.57f,.42f,-.83f,.52f,-.87f,-.13f,-.58f,.32f,-.59f,-.66f,.66f,.24f,.09f,.48f,.39f,-.20f,-.88f,-.46f,.82f,-.52f,.05f,.34f,.06f,.85f,.18f,-.62f,-.25f,-.82f,-.21f,.86f,.04f,-.54f,-.63f,-.67f,-.19f,.98f,.59f,.40f,.96f,-.89f,.89f,.52f,-.36f,-.65f,-.07f,-.80f,.18f,.21f,.90f,-.13f,-.36f,-.24f,.98f,-.09f,.13f,-.50f,.39f,-.47f,-.07f,-.06f,-.44f,.79f,-.62f,-.82f,-.85f,-.41f,-.40f,.97f,.03f,.29f,.55f,-.25f,.71f,-.55f,.82f,-.93f,-.06f,-.18f,-.26f,-.66f,.13f,-.68f,-.19f,.93f,.59f,.53f,-.16f,-.48f,-.98f,.71f,.94f,.51f,.74f,.91f,-.27f,.42f,-.53f,-.87f,.26f,.96f,-.47f,.66f,-.98f,.26f,.92f,-.61f,.14f,.61f,-.68f,.50f,.10f,.40f,-.62f,-.07f,-.13f,-.97f,-.69f,.95f,-.86f,-.74f,-.29f,.54f,-.64f,-.14f,-.91f,.66f,-.70f,-.20f,-.26f,-.30f,-.19f,.41f,.88f,-.05f,-.48f,.82f,-.85f,-.01f,.09f,-.10f,-.68f,-.07f,.47f,-.74f,.10f,-.76f,-.98f,-.92f,.49f,-.97f,-.32f,.23f,.68f,.41f,.82f,-.04f,.41f,.72f,.33f,-.83f,-.55f,.57f,-.96f,.85f,.81f,.66f,.52f,-.19f,-.18f,-.33f,-.73f,.55f,-.69f,.97f,.50f,.22f,-.90f,.21f,-.50f,-.92f,-.61f,.50f,-.07f,.59f,.36f,-.47f,.50f,-.61f,-.75f,.45f,-.91f,-.30f,.25f,.27f,.91f,-.28f,.45f,-.07f,.63f,.52f,-.62f,.48f,.54f,-.97f,.33f,.48f,-.90f,-.85f,-.26f,.95f,-.00f,-.39f,.34f,.02f,-.29f,.82f,.17f,.43f,-.27f,-.62f,.62f,.21f,.96f,.68f,.09f,.61f,.40f,.03f,.15f,-.85f,-.36f,.85f,-.03f,.36f,.85f,.85f,-.91f,-.87f,.27f,.09f,.81f,.20f,.51f,.96f,-.93f,-.19f,-.64f,.87f,.41f,-.03f,-.24f,-.12f,-.71f,.28f,-.68f,.45f,-.62f,.64f,.18f,-.00f,.81f,-.72f,-.46f,-.58f});
    auto result = backend->make_primary_tensor_view(element::f32, shape_r);

    vector<float> expected_result{0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.94260001295953987909825855240342207252979278564453125f,2.52870004181861940395492638344876468181610107421875f,-0.5710000071451066805394702896592207252979278564453125f,1.2864000110015272326791091472841799259185791015625f,-1.106699997353553843737472561770118772983551025390625f,-0.92230004508495400106227180003770627081394195556640625f,-1.125400026245415130432547812233678996562957763671875f,1.106100012020766687470541000948287546634674072265625f,-1.4043000310257076623798866421566344797611236572265625f,0.375600042255223254272777921869419515132904052734375f,-1.078499977529049136393268781830556690692901611328125f,0.85619997811615444760491300257854163646697998046875f,0.32200001771748087975311136688105762004852294921875f,0.9831000029906629134046625040355138480663299560546875f,0.426500013753771778368673039949499070644378662109375f,-0.60530001637786645662941964474157430231571197509765625f,-1.4751000270597638408531793174915947020053863525390625f,1.0581000036984680168217209939029999077320098876953125f,-0.8544000077381725954950297818868421018123626708984375f,0.703599982926248923575940352748148143291473388671875f,1.609499975919722825068447491503320634365081787109375f,-0.5248000021323562247488325738231651484966278076171875f,0.258899985632300155824481180388829670846462249755859375f,0.89179999440610402405837930928100831806659698486328125f,1.402100039543211718040538471541367471218109130859375f,0.4499999556690454927121436412562616169452667236328125f,-0.13240001091957032119950099513516761362552642822265625f,-0.0529000223897399257655393967070267535746097564697265625f,-0.2853000095978377448346918754396028816699981689453125f,0.301500004626810369501299646799452602863311767578125f,-1.5719000179082158741294961146195419132709503173828125f,0.7672999910682440916076529902056790888309478759765625f,0.703699972639978188482245968771167099475860595703125f,0.75729998920559882247260929943877272307872772216796875f,-0.71190000218749027727227485229377634823322296142578125f,0.4854999937564132839185049306252039968967437744140625f,-1.3666000039145347333402469303109683096408843994140625f,-0.09749996174126873693666084363940171897411346435546875f,0.16540000299960322394099421217106282711029052734375f,0.2382000122249132711971242315485142171382904052734375f,1.060400006985664500547272837138734757900238037109375f,-0.91680001560300627971855647047050297260284423828125f,-0.08460000303536652221936265050317160785198211669921875f,0.74320001900494114810413748273276723921298980712890625f,-0.355499993272125447418829935486428439617156982421875f,-0.51749998442828648581581774124060757458209991455078125f,-1.14120000233054152971590156084857881069183349609375f,0.55069996036887147727867386493016965687274932861328125f,-0.2252999932438137786760989911272190511226654052734375f,-0.01190001064389945018007210819632746279239654541015625f,-0.1749999785795808282529151256312616169452667236328125f,-0.0561000027447942084535270623746328055858612060546875f,0.4023000228077175943752763487282209098339080810546875f,-1.0548000440418723311353232929832302033901214599609375f,-0.8611000202909113721005951447295956313610076904296875f,1.471000007256865416138680302537977695465087890625f,-0.6188999954670659775501917465589940547943115234375f,-0.0985999865770335048864581040106713771820068359375f,-1.44390004028230922727971119456924498081207275390625f,0.95690001612007602460607813554815948009490966796875f,-2.46339999075830018426813694532029330730438232421875f,1.178999964021146151793573153554461896419525146484375f,-0.463999973259865416963521056459285318851470947265625f,0.6563000067591666653044057966326363384723663330078125f,0.082400024144351657895413154619745910167694091796875f,0.19189995040595564379515280961641110479831695556640625f,0.6410000128448010148218827453092671930789947509765625f,1.889799975831806033710336123476736247539520263671875f,0.3410999890729795591681750011048279702663421630859375f,-0.620000025033950574737673377967439591884613037109375f,0.17900002228468603338029652149998582899570465087890625f,0.5671999783694745556061889146803878247737884521484375f,-1.796999959044157879617387152393348515033721923828125f,1.42930001750290358586426009424030780792236328125f,0.0398000121533872430745759629644453525543212890625f,1.2323999902814630758030034485273063182830810546875f,-0.88990001300573329245935383369214832782745361328125f,-0.28199999547749710071542494915775023400783538818359375f,-0.882100041405857293597136958851478993892669677734375f,1.159700088004768758764839731156826019287109375f,0.473299967703222801418405651929788291454315185546875f,-0.897400018034875568417874092119745910167694091796875f,-0.79230001133382377265235163577017374336719512939453125f,0.31600000625103685791827956563793122768402099609375f,0.037599980102479724219932677442557178437709808349609375f,-1.130799976904690584689205934409983456134796142578125f,0.35759999731332070549427726291469298303127288818359375f,-1.2323000088192517242902113139280118048191070556640625f,0.82790000329762702957481224075309000909328460693359375f,0.8014000125661493445505811905604787170886993408203125f,-0.60830002620890744058357313406304456293582916259765625f,0.02760001467317374501675431019975803792476654052734375f,-0.456300031532347549756423177313990890979766845703125f,1.3821000192403796180684594219201244413852691650390625f,1.02909997489452376129293043049983680248260498046875f,-0.6451999829292296428917552475468255579471588134765625f,-0.65910000234991283907248771356535144150257110595703125f,-0.27760001173019432929578442781348712742328643798828125f,-0.32969997203648138128784239597734995186328887939453125f,-0.01160001907646712737687266780994832515716552734375f,-0.4220000435337423194681605309597216546535491943359375f,0.148100049221516538722198674804531037807464599609375f,1.7214999967664474933570772918756119906902313232421875f,-0.78080004746168885088053457366186194121837615966796875f,0.35760003035664589265962831632350571453571319580078125f,1.235500015698372866523868651711381971836090087890625f,-0.757899987614154557746815044083632528781890869140625f,-0.51970002635121392842165732872672379016876220703125f,-0.7414999966546889087481986280181445181369781494140625f,0.741000004909933007724021081230603158473968505859375f,-0.1808000351682308437517576749087311327457427978515625f,0.3613000253111124937532849799026735126972198486328125f,0.234699997442960661686583989649079740047454833984375f,-1.2525000182539220983102268291986547410488128662109375f,0.5462999843701716873312079769675619900226593017578125f,0.787399979925155424353988564689643681049346923828125f,-1.088400032705068465332942651002667844295501708984375f,0.8126000515162947746006238958216272294521331787109375f,-0.7251000610530378143181451378040947020053863525390625f,0.03759997663795910938233646447770297527313232421875f,2.13530004565864839349842441151849925518035888671875f,0.1203999780587849954827106557786464691162109375f,0.0704000119775540778022104859701357781887054443359375f,-1.97010004894584422885372987366281449794769287109375f,-0.48509996900111429507518323589465580880641937255859375f,0.04130003499686740742191659592208452522754669189453125f,1.05020000557899439996845103451050817966461181640625f,-1.7739000021353366065568479825742542743682861328125f,1.5841000355795025189564739775960333645343780517578125f,-1.6334000260367986800957851301063783466815948486328125f,1.263400029575824756733481990522705018520355224609375f,1.352300017964839895512341172434389591217041015625f,0.3850000201351944095762291908613406121730804443359375f,0.9870000079199667819551677894196473062038421630859375f,-0.5186000350058079799708821155945770442485809326171875f,-0.62630001890361286509545379885821603238582611083984375f,0.549400070993602440694303368218243122100830078125f,-1.3413999772876508131957962177693843841552734375f,0.3461000152617688119249805822619237005710601806640625f,-0.5942000112861391958318790784687735140323638916015625f,-1.0715000209808354103557803682633675634860992431640625f,0.5351000329270954036786633878364227712154388427734375f,0.214900051711499673334770932342507876455783843994140625f,-1.1503000043079258940537101807422004640102386474609375f,-0.7940999959796659624089443241246044635772705078125f,-0.5094999846369023810410681107896380126476287841796875f,-1.381300040324032973870771456859074532985687255859375f,1.14680005073249358105158535181544721126556396484375f,-0.4497000324234370349785194775904528796672821044921875f,0.38909999799132355402520033749169670045375823974609375f,0.23530001060366612097851657381397671997547149658203125f,-1.903300040681660476593606290407478809356689453125f,1.33820002354979550318603287450969219207763671875f,-0.02060000019669516735376646465738303959369659423828125f,-0.145099995413422722112528617799398489296436309814453125f,0.2307999980270867279585900178062729537487030029296875f,0.668300040756166335853549753664992749691009521484375f,0.5814999755695458016901966402656398713588714599609375f,-0.0493999461591239086288851467543281614780426025390625f,1.667900003744661407978355782688595354557037353515625f,-0.8085000303536655774649943850818090140819549560546875f,-0.61009999615848020670938467446831054985523223876953125f,0.8134000010028488159008475122391246259212493896484375f,0.49549998090416214058251398455468006432056427001953125f,1.41880002121329251707493313006125390529632568359375f,-0.248700031164288848373189466656185686588287353515625f,1.3629000761643064176809048149152658879756927490234375f,-0.4211000287845731993030540252220816910266876220703125f,-0.382900004266202353964132498731487430632114410400390625f,0.2587999644212419791955426262575201690196990966796875f,-0.23389998358339048056819819976226426661014556884765625f,-0.28189998620599521217400251771323382854461669921875f,0.002200021061301071778615323637495748698711395263671875f,-0.0125000233761964896217477871687151491641998291015625f,-0.82949999439716315396964319006656296551227569580078125f,-0.3630000278800731283723735032253898680210113525390625f,0.67810001394301677724030241733998991549015045166015625f,0.1210000058040023862560019551892764866352081298828125f,0.105899987144768115143733666627667844295501708984375f,-1.18810001502335094158979700296185910701751708984375f,-1.0754999927133319648220322051201947033405303955078125f,1.386500010885298284080135999829508364200592041015625f,0.5338000060513616862323260647826828062534332275390625f,0.84890000925064101888750656144111417233943939208984375f,1.2736999899253242940488917156471870839595794677734375f,0.0776999750316143267259150206882623024284839630126953125f,1.6981000694870951317483331877156160771846771240234375f,-0.14550003935396726983952930822852067649364471435546875f,0.5969000200316314508341974942595697939395904541015625f,0.50960001778751606860140554999816231429576873779296875f,-1.370500000424683673116987847606651484966278076171875f,0.5868000188812612094579890253953635692596435546875f,-0.933800002959370711863584801903925836086273193359375f,-0.7449000380396844089858632287359796464443206787109375f,-0.2477999546125524421569252808694727718830108642578125f,0.15919998867362750161191797815263271331787109375f,-0.469800012637675179494323174367309547960758209228515625f,1.32779999678283910924392330343835055828094482421875f,-0.928499986506998720159344884450547397136688232421875f,1.24370000594407326133250535349361598491668701171875f,-1.8289000747784971423470778972841799259185791015625f,-0.147699966798722381877695397633942775428295135498046875f,0.1008999788373704797894703233396285213530063629150390625f,-0.8247999953523272864686077809892594814300537109375f,2.19520002059191465804133258643560111522674560546875f,0.525399988880753454623118159361183643341064453125f,1.1706999828323716439371082742582075297832489013671875f,-0.4861999959975482621388209736323915421962738037109375f,0.611300051425397672488770695053972303867340087890625f,-0.70479997311234487966657979995943605899810791015625f,0.53920001832693864596279809120460413396358489990234375f,-1.3632000243693591645666174372308887541294097900390625f,0.8141000276818870506900793770910240709781646728515625f,0.2311999657481909054013158311136066913604736328125f,-0.8743999701499933774329065272468142211437225341796875f,1.44539999763518522968297475017607212066650390625f,-0.67310002903044274802368818200193345546722412109375f,-0.1612999922305344835393725588801316916942596435546875f,-1.4475000114738933998381753553985618054866790771484375f,1.16330001248121295276405362528748810291290283203125f,-0.3090000355467206194504115046584047377109527587890625f,1.5406000292912123139643654212704859673976898193359375f,-2.01859998918473682039120831177569925785064697265625f,-0.5315999928131700524858160861185751855373382568359375f,-0.4360999971568588762238505296409130096435546875f,1.0297000067561867808763054199516773223876953125f,0.9024000274598602544529057922773063182830810546875f,-1.19950008071214053728681392385624349117279052734375f,0.467400069443881793773698518634773790836334228515625f,0.00539998042434419733837103194673545658588409423828125f,0.2303000358983882112084984328248538076877593994140625f,-1.357799990189075156621356654795818030834197998046875f,1.3848000079065554412949268225929699838161468505859375f,-0.31710002527534941751952146660187281668186187744140625f,-0.981400022549926642767559314961545169353485107421875f,0.0984000173941250722009499440900981426239013671875f,0.97790000162124623983572746510617434978485107421875f,-1.009799993228912118325979463406838476657867431640625f,0.46570002377331309872943165828473865985870361328125f,-0.66699995442479831364579467845032922923564910888671875f,-0.8581000356242063364931027535931207239627838134765625f,-1.5566000348344444237369543770910240709781646728515625f,0.92010002644509081282109264066093601286411285400390625f,-0.720499993830919560622305652941577136516571044921875f,0.0527000068455931447441997761416132561862468719482421875f,-0.39000003077089806513555458877817727625370025634765625f,-0.9271999842554328186139400713727809488773345947265625f,-1.302099987445025863763703455333597958087921142578125f,0.484400018057227488288418726369854994118213653564453125f,-0.238600051117689115898201634990982711315155029296875f,1.2256000172212722088005421028356067836284637451171875f,0.880700050409138679441412023152224719524383544921875f,0.33710000955462471328161200290196575224399566650390625f,-0.1740000106990333694767514316481538116931915283203125f,-0.9513999741211536242957436115830205380916595458984375f,-0.0603999796047809400789674327825196087360382080078125f,-1.738000033691525114676323937601409852504730224609375f,0.7397000149145724900989762318204157054424285888671875f,-0.7031000130861995156550392493954859673976898193359375f,-0.5896000074684621949927532114088535308837890625f,0.4827999835953116214426472652121447026729583740234375f,1.10240000935494908418377235648222267627716064453125f,0.9207999798849224593055851073586381971836090087890625f,-1.4209000036850565340529328750562854111194610595703125f,1.1080999646574254402509041028679348528385162353515625f,-0.0736999956995252336611201826599426567554473876953125f,-1.2255000054538254250502404829603619873523712158203125f,1.245099994966387679795616350020281970500946044921875f,0.76870004336461483784859183288062922656536102294921875f,-1.21029997875243378047116493689827620983123779296875f,0.9566000321336087797163827417534776031970977783203125f,-1.29320002582222315368198906071484088897705078125f,1.2977999867245559872941385037847794592380523681640625f,-0.8535000119507312188460446122917346656322479248046875f,1.2791000305317343777034011509385891258716583251953125f,-2.024000008389354121618453064002096652984619140625f,1.853799970698356336384904352598823606967926025390625f,-1.92160000760257165808297941111959517002105712890625f,1.4644999862387777511685271747410297393798828125f,-1.45360003932714487717703377711586654186248779296875f,-0.794299994592368552304151307907886803150177001953125f,0.5318000292003153273157067815191112458705902099609375f,-0.54139997564852249123390492968610487878322601318359375f,0.717199985708295795205913236713968217372894287109375f,-0.9064000056743619637700248858891427516937255859375f,0.18030004089027673597200873700785450637340545654296875f,-1.344100055770576229718926697387360036373138427734375f,-1.25520001511573742192240388249047100543975830078125f,0.5645999883577221822861247346736490726470947265625f,0.7818000205017627823878001436241902410984039306640625f,-0.06160000361651231381987514623688184656202793121337890625f,-0.86470000070258923319244104277458973228931427001953125f,1.868900015360117183860211298451758921146392822265625f,-0.3357000353217129973160126610309816896915435791015625f,-0.43970002445131572965664190633106045424938201904296875f,-0.99510003611221886643534162431024014949798583984375f,1.30610005512237581370982297812588512897491455078125f,-0.3611000000812116805803952956921420991420745849609375f,-0.7315999996677045036364006591611541807651519775390625f,-1.2320999920830129070736802532337605953216552734375f,-0.08820004143118842510062904693768359720706939697265625f,1.70180001463443009157572305412031710147857666015625f,-1.246399984514713121797058192896656692028045654296875f,0.778300003577768517715185225824825465679168701171875f,-1.276999988511204886520999934873543679714202880859375f,0.096399976021050548524726764298975467681884765625f,-1.378700030195713299718818234396167099475860595703125f,0.96950004439055970095751035842113196849822998046875f,-1.0605000133514403959367200513952411711215972900390625f,0.4298000197157263446712249788106419146060943603515625f,-0.3584000255525114919663565160590223968029022216796875f,-0.2630999979987740378106764183030463755130767822265625f,0.83350004547834488644042494343011640012264251708984375f,-0.46940001120269292389508564156130887567996978759765625f,0.1593000185087320996757398461340926587581634521484375f,-0.11650001181662095195434858396765775978565216064453125f,0.60850002875179065586763726969365961849689483642578125f,-0.363700050945579944627894519726396538317203521728515625f,-0.00719999908208801553399780459585599601268768310546875f,0.9464999507963656188991308226832188665866851806640625f,-0.0433999786697325529161162194213829934597015380859375f,-0.20829997262060706564312795308069325983524322509765625f,0.31409999970495705579054401823668740689754486083984375f,-0.68500000055879384230905770891695283353328704833984375f,0.92760002357661697214297191749210469424724578857421875f,0.560599982427060439249544288031756877899169921875f,0.53160000246167193882484980349545367062091827392578125f,0.4834000106513498540294904159964062273502349853515625f,0.46119998225122704749168178750551305711269378662109375f,0.4362000362679367970741850513149984180927276611328125f,0.41199996951222395136227305556531064212322235107421875f,-0.78560001651346633710915057235979475080966949462890625f,1.67910000294595906922268113703466951847076416015625f,-1.72620001685917401346159749664366245269775390625f,2.64750011414289598832283445517532527446746826171875f,-2.820800087061524852316551914555020630359649658203125f,-0.8697000511989003701529554746230132877826690673828125f,0.0325000846013432376224727704538963735103607177734375f,-0.5823999845445160250534399892785586416721343994140625f,0.24200003522634527808321536213043145835399627685546875f,0.32929999779611784926913742310716770589351654052734375f,1.3609000391498209392437956921639852225780487060546875f,-0.26900001339986878701182604345376603305339813232421875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-1.857800070580841289569207219756208360195159912109375f,-0.008000017225741284931928021251223981380462646484375f,2.02800007589906527982748229987919330596923828125f,0.3954999740496274451828639939776621758937835693359375f,0.254499962545930547008765643113292753696441650390625f,1.153000061929226216506094715441577136516571044921875f,0.882999954789875918237385121756233274936676025390625f,0.7106000717222700213682173853158019483089447021484375f,0.056400011262297766734263859689235687255859375f,-0.22820001997351668787672451799153350293636322021484375f,0.6921999646231526970296954459627158939838409423828125f,1.126799957898257265043184816022403538227081298828125f,0.9700999676227570756026352682965807616710662841796875f,0.24890005127191461298252761480398476123809814453125f,-0.23970000634640464820535044054850004613399505615234375f,1.721800034490227115924199097207747399806976318359375f,-0.7598999721206711743803907666006125509738922119140625f,-2.341000081539155619481107351020909845829010009765625f,-1.875700050073861202548641813336871564388275146484375f,-1.189200009796023582708812682540155947208404541015625f,-1.338700083243846705727264634333550930023193359375f,-0.43670006187558263466286234688595868647098541259765625f,0.7983000352054843151705654236138798296451568603515625f,-0.4158000599041591360816028100089170038700103759765625f,-2.24440004286766026098121074028313159942626953125f,-2.33070002444386492612693473347462713718414306640625f,-1.267100052283704147981779897236265242099761962890625f,-0.75989997171088974337038735029636882245540618896484375f,-1.089800029587745910930607351474463939666748046875f,-2.207200016367435235764560275129042565822601318359375f,-0.33079998044371716758860202389769256114959716796875f,1.276900002336502470967616318375803530216217041015625f,-0.283400001859664740777589031495153903961181640625f,-2.0875000457465642256238425034098327159881591796875f,-0.3183999514937401187353316345252096652984619140625f,-0.081900011128187344411344383843243122100830078125f,-0.46190004256963757001130943535827100276947021484375f,2.06650005421042504138995354878716170787811279296875f,1.996999963551758128232904709875583648681640625f,0.036999946862458887864022472058422863483428955078125f,1.2419999884366976772298585274256765842437744140625f,-1.14850003266334521612179742078296840190887451171875f,-0.1247000349938873764443769687204621732234954833984375f,0.5367000039100646713308151447563432157039642333984375f,-0.4732999846160403922823434186284430325031280517578125f,0.20520001139044741211847622253117151558399200439453125f,0.3241000470533970112541055641486309468746185302734375f,1.8081999877497552109417711108108051121234893798828125f,0.916000064291060400734068025485612452030181884765625f,-1.9721000554502012835911273214151151478290557861328125f,0.54790009579658605076701860525645315647125244140625f,-0.0621000757157812444120281725190579891204833984375f,0.5029000026643277720950209186412394046783447265625f,3.917900084620715261962686781771481037139892578125f,0.339200004357099516028029029257595539093017578125f,-0.435499946668743032063275677501223981380462646484375f,1.09090005401372991400421597063541412353515625f,2.693900132781268386139572612592019140720367431640625f,0.05740009466409734528724584379233419895172119140625f,0.412400018072129004309545052819885313510894775390625f,1.03010008325576762899800087325274944305419921875f,0.591099926674365949708089829073287546634674072265625f,0.93649998939037271128427164512686431407928466796875f,-0.8205999803781498513899350655265152454376220703125f,-1.1933000241786242412445062655024230480194091796875f,-0.1861000327706339163569282391108572483062744140625f,-2.2077000404477100659050847752951085567474365234375f,-2.919000008985400906880158800049684941768646240234375f,-1.398600098410248460112370594288222491741180419921875f,0.09850005255639704415671076276339590549468994140625f,0.15070001922845843722598146996460855007171630859375f,-0.9717000648558151709721641964279115200042724609375f,-0.0311999589681608568980664131231606006622314453125f,0.012400040125846611971383026684634387493133544921875f,-2.59140009120702785594403394497931003570556640625f,-1.6966000221312054208055997150950133800506591796875f,-0.324300001037119756119864177890121936798095703125f,-0.7573999610751851463419370702467858791351318359375f,-0.439099973776936547409377453732304275035858154296875f,0.2224000404983763612420943900360725820064544677734375f,1.3028000389903784395073671475984156131744384765625f,1.0234999384134997324480309544014744460582733154296875f,-0.4442999627038834997705407658941112458705902099609375f,-0.70420004421770610036901416606269776821136474609375f,-0.44979999650716739889588779988116584718227386474609375f,-2.55620004021674418481779866851866245269775390625f,-0.6881999777659775219262883183546364307403564453125f,0.764399991365522613051552980323322117328643798828125f,0.6898000227332115041889437634381465613842010498046875f,0.6818000194028026594850189212593249976634979248046875f,1.064300059729069491964992266730405390262603759765625f,-0.099399944072960266794325434602797031402587890625f,0.8335999792754640314029757064417935907840728759765625f,-0.9137000024795529640186941833235323429107666015625f,-1.77590001065134917013210724689997732639312744140625f,-1.91780000017285345847994904033839702606201171875f,-0.4763000450551511111285662991576828062534332275390625f,1.913900007313489748383972255396656692028045654296875f,1.647099989140032416656822533695958554744720458984375f,0.90039998090863182511611739755608141422271728515625f,-1.69640004970729396660544807673431932926177978515625f,-3.0403000658124685884331483975984156131744384765625f,-0.1867999605432146825734207595814950764179229736328125f,-0.76920009737759897916475892998278141021728515625f,0.25949999421089842144283466041088104248046875f,-1.206500073134900929971990990452468395233154296875f,-1.233200009244679762332452810369431972503662109375f,0.21829999385476117623738900874741375446319580078125f,1.75639995843768037531162917730398476123809814453125f,0.642899967646599179715849459171295166015625f,0.9932000563323502007051502005197107791900634765625f,-0.9114000226616862931905416189692914485931396484375f,-0.1419000277429829370845482117147184908390045166015625f,0.2720999965533617004354027812951244413852691650390625f,0.042900011381506342189595670788548886775970458984375f,0.90660007622242044789118153857998549938201904296875f,-2.075400016000866543208758230321109294891357421875f,-2.691900002261995883401368701015599071979522705078125f,-0.56049999394267724373008832117193378508090972900390625f,0.4624000016808498259024418075568974018096923828125f,0.24639996752739090624118034611456096172332763671875f,-0.03130004095733329716466641912120394408702850341796875f,1.88890004512518761004002953995950520038604736328125f,0.0911000302374365134028266766108572483062744140625f,-1.93199998459964916719400207512080669403076171875f,0.2868000493168825215661854599602520465850830078125f,2.186700039970874254180444040684960782527923583984375f,1.173999995648860039665350996074266731739044189453125f,-0.7260000400394208686094543736544437706470489501953125f,-0.168800030004979451092594899819232523441314697265625f,-1.135299984414874785443316795863211154937744140625f,-1.1972000307098047411358265890157781541347503662109375f,-0.70309998889043867631443163190851919353008270263671875f,-0.884400045102835097310389755875803530216217041015625f,1.1906000060081478597595605606329627335071563720703125f,-1.0027000812023880715884160963469184935092926025390625f,0.4126000226825479177961142340791411697864532470703125f,0.4685999997273071482339901194791309535503387451171875f,0.4818999658286571285970012468169443309307098388671875f,1.2067999849438653114930275478400290012359619140625f,-0.4266000441849229218860273249447345733642578125f,0.424500053033230795307417793083004653453826904296875f,2.86920003378689347783847551909275352954864501953125f,2.68590007480084835833622491918504238128662109375f,0.845799980667234052589265047572553157806396484375f,0.405900052300096181312483167857863008975982666015625f,1.51260006172358973941527438000775873661041259765625f,0.6148000189334157372655909057357348501682281494140625f,2.0580000612214206512362579815089702606201171875f,-1.4711000485286118699690405264846049249172210693359375f,0.162400003059209208089441744959913194179534912109375f,2.308000017672776404964452012791298329830169677734375f,1.4166999781310571915327045644517056643962860107421875f,1.001800043952464580598871179972775280475616455078125f,-0.664800042849778893838674775906838476657867431640625f,-1.4435000756904499130683916519046761095523834228515625f,-1.94810006687939019087707492872141301631927490234375f,-0.19230001453757505913699787924997508525848388671875f,-0.3126000688016414841285950387828052043914794921875f,0.2546999850749955829343207369674928486347198486328125f,-0.2642000829607253908903885530889965593814849853515625f,-0.61470002843737603370755095966160297393798828125f,0.12590004581809122186086824513040482997894287109375f,-0.354600051277876193722704556421376764774322509765625f,-0.998199942338465984192907853866927325725555419921875f,0.04379997859597128950781552703119814395904541015625f,-0.50540000013112962307104680803604423999786376953125f,0.141100038135052674448388643213547766208648681640625f,0.29050003467872775875235902276472188532352447509765625f,-0.21330005306825061683895228270557709038257598876953125f,-0.649400001889467670679323418880812823772430419921875f,-0.1435999996900554354084533770219422876834869384765625f,0.905800053273141170251392395584844052791595458984375f,1.433899967806785635815458590514026582241058349609375f,0.05640003851279684621289334245375357568264007568359375f,0.193299993668496039589399515534751117229461669921875f,0.04539993411898546327165604452602565288543701171875f,-0.0916999820426107614679267499013803899288177490234375f,-2.099700058984756534385951454169116914272308349609375f,-0.8641999663591375391291649066261015832424163818359375f,-1.062700120764973110709661341388709843158721923828125f,0.8486000221908103213763752137310802936553955078125f,-1.49770000021457594385765332845039665699005126953125f,-0.43090006794631552367036420037038624286651611328125f,-0.2071000800743705294593155485927127301692962646484375f,-1.2398000980213288446663000286207534372806549072265625f,-1.733800084729493651281018173904158174991607666015625f,0.8100999911665913710834274752414785325527191162109375f,-0.1139000210970644300090270917280577123165130615234375f,0.4628000015139577261180647838045842945575714111328125f,0.2049000368475919486854763817973434925079345703125f,1.84109998326152446423975561629049479961395263671875f,3.415400096504391580509718551184050738811492919921875f,0.630500013798475311688207511906512081623077392578125f,0.436600127035382090667781085358001291751861572265625f,-0.759600011417270071234497663681395351886749267578125f,-0.2336999889194963930805215568398125469684600830078125f,-1.03180006868839324596365258912555873394012451171875f,-1.81880005399584820935388052021153271198272705078125f,0.84000008180737584240205251262523233890533447265625f,1.061799956706166003783664564252831041812896728515625f,-0.2978999368384467327786069290596060454845428466796875f,1.2552000221565366988585310537018813192844390869140625f,-2.20340009663105096393564963364042341709136962890625f,-0.5595999648511398305572583922185003757476806640625f,0.7837999593734732428629286005161702632904052734375f,-0.9789000435233123909029018250294029712677001953125f,-3.3175000940263270621244373614899814128875732421875f,1.66060008245110513058762080618180334568023681640625f,1.5213999530360098333403584547340869903564453125f,-0.4836999737203111493499818607233464717864990234375f,-0.271300013017654606528594740666449069976806640625f,-0.5632999990701677006654790602624416351318359375f,0.3112999175012109986937502981163561344146728515625f,-1.2816000677287593401132426151889376342296600341796875f,-0.869900117164851049977869479334913194179534912109375f,0.35819999642968003428222800721414387226104736328125f,1.08730007160902086837950264452956616878509521484375f,0.12649992159008949244025643565692007541656494140625f,3.11480011705756254514199099503457546234130859375f,0.884500037014485318564993576728738844394683837890625f,0.919800028991698326308323885314166545867919921875f,1.02999996915459579582829974242486059665679931640625f,0.62740000212788604727620622725225985050201416015625f,0.71889999911785107400419292389415204524993896484375f,-1.647500124722720560299649150692857801914215087890625f,0.7487000470340259283830164349637925624847412109375f,-1.358100034767389718837193868239410221576690673828125f,-2.23009998446106916247799745178781449794769287109375f,-0.8166000246644014026742297573946416378021240234375f,0.347999933779238546094347839243710041046142578125f,1.430800031927228044281719121499918401241302490234375f,0.99230001315921523286078809178434312343597412109375f,0.20920001821517875129075036966241896152496337890625f,0.2156000377476219309613725272356532514095306396484375f,-0.4527998999625424403348006308078765869140625f,-0.80300002199411579795196303166449069976806640625f,-0.371499984547494221942542935721576213836669921875f,0.14340005326867188983896994614042341709136962890625f,-0.99700001510977731555840364308096468448638916015625f,0.48610000311732193267744150944054126739501953125f,-0.98210010045170814674264647692325524985790252685546875f,0.9115000292509787271910681738518178462982177734375f,2.79270003951638923211930887191556394100189208984375f,1.758500018395482644706362407305277884006500244140625f,0.579400077736379248705134159536100924015045166015625f,0.39460006029307859165555782965384423732757568359375f,-0.35089998172446967839022136104176752269268035888671875f,1.266200047574192577570784123963676393032073974609375f,0.6881000610865661837323159488732926547527313232421875f,-1.26850003698468327684167888946831226348876953125f,-2.429900065904856631959773949347436428070068359375f,2.610500019460915144264845366706140339374542236328125f,2.58820005514025819337575740064494311809539794921875f,-0.433800013017656471703276110929436981678009033203125f,-1.05439998818039892825026981881819665431976318359375f,-1.2019000074028980407092603854835033416748046875f,-1.05979998920559825847931278985925018787384033203125f,-0.285399973309040433377958834171295166015625f,0.0402999881401659987290031494922004640102386474609375f,1.822200023519991329834510906948707997798919677734375f,0.1174000860586790206951945947366766631603240966796875f,-0.43149997627735192651243778527714312076568603515625f,-0.6078000663340095144349106703884899616241455078125f,-1.0492999639332278860592850833199918270111083984375f,-1.80050003242492806521113379858434200286865234375f,1.160299992871284047168956021778285503387451171875f,0.9368000166460870303808405878953635692596435546875f,1.023300010078400656965413872967474162578582763671875f,-1.567799970091133587857257225550711154937744140625f,-0.5107000638760637034607725581736303865909576416015625f,0.60239999944567745870926955831237137317657470703125f,0.27359992011785294607761898078024387359619140625f,-1.54350002793222618180379868135787546634674072265625f,2.019400019863992756796733374358154833316802978515625f,1.269000020533800121569356633699499070644378662109375f,0.540200044545532609419069558498449623584747314453125f,-1.143300001752377337282950975350104272365570068359375f,-0.83970001323819065675024830852635204792022705078125f,0.09480006535053320249062380753457546234130859375f,-1.16080005822777820867486298084259033203125f,0.60699999037384966271702069207094609737396240234375f,2.11740001725256288267473792075179517269134521484375f,1.089500035002828326469170860946178436279296875f,0.399700042295454860408199238008819520473480224609375f,0.104900075590612207321328241960145533084869384765625f,-0.0998999603301282679268524589133448898792266845703125f,-0.784700027468799188312686965218745172023773193359375f,-1.33229997031837843479706862126477062702178955078125f,-1.186599996706096504794913926161825656890869140625f,0.73970000766888166499057888358947820961475372314453125f,-0.9906001442722993655110030886135064065456390380859375f,-1.6086000598162406571844940117443911731243133544921875f,-0.5846999927490961201925756540731526911258697509765625f,1.073700028295815567247473154566250741481781005859375f,1.771100057189911591848385796765796840190887451171875f,0.89800006258115272483877333797863684594631195068359375f,-0.2185998893655825003889958679792471230030059814453125f,1.608500010460615303742315518320538103580474853515625f,1.19630002024471782107184480992145836353302001953125f,1.72649998670816540169425934436731040477752685546875f,0.9420000131726258274511565105058252811431884765625f,0.61969999312162382665292170713655650615692138671875f,0.06440001164972752878412620702874846756458282470703125f,-0.7207000955224049931757690501399338245391845703125f,-0.1647999879390000810275296316831372678279876708984375f,-0.2742001371264475295674856170080602169036865234375f,-0.27589998800158443970076405094005167484283447265625f,-1.425700088667869902536722293007187545299530029296875f,1.053800033357739085460025307838805019855499267578125f,1.902600072824956356498660170473158359527587890625f,-0.5415000224709520804111662073410116136074066162109375f,0.41990006612837416088979125561309047043323516845703125f,1.069800011110305870687398055451922118663787841796875f,-0.16809994960725216373020884930156171321868896484375f,-0.452299975645541252333714510314166545867919921875f,-1.9597000966101898455917762476019561290740966796875f,-1.1416999866992221246420058378134854137897491455078125f,0.12069997050166048069286262034438550472259521484375f,-0.343199961672722597683105050236918032169342041015625f,-2.01780003383085126955620580702088773250579833984375f,-1.125799984820933019591393531300127506256103515625f,-0.2290999866664407136340741999447345733642578125f,0.4336000107914210754955774973495863378047943115234375f,1.3878000043451776956970888932119123637676239013671875f,1.3826999658301470930155119276605546474456787109375f,0.4372999966561799123354603580082766711711883544921875f,-0.459199964459239939884582781814970076084136962890625f,-1.49620006059408172660596392233856022357940673828125f,-0.2120999356076109787494488045922480523586273193359375f,0.1255000558570034119298952646204270422458648681640625f,-2.618500076733530246286818510270677506923675537109375f,0.63810010748505785471706985845230519771575927734375f,-2.45570007108449939181582521996460855007171630859375f,0.88989998394846825391368838609196245670318603515625f,1.901799996268749737282632850110530853271484375f,-0.75350008898973630522277744603343307971954345703125f,-2.288700048375129636468727767351083457469940185546875f,0.5959000115454191615071977139450609683990478515625f,1.77189998058080533382963039912283420562744140625f,-0.245400078585744640946586514473892748355865478515625f,-1.5433000387445101608818731619976460933685302734375f,0.25120000152960442196814483395428396761417388916015625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.01090002123117483900927027207217179238796234130859375f,-1.19969999163150742305106177809648215770721435546875f,1.9045000376477838610611570402397774159908294677734375f,-1.399999990127980709075927734375f,0.4181000126764178315141862185555510222911834716796875f,0.179600023002922881687482004053890705108642578125f,-0.816600049549341289178983060992322862148284912109375f,0.14570000682324180107940492234774865210056304931640625f,-1.177400040908157308194859069772064685821533203125f,0.14750003166496739037683028072933666408061981201171875f,0.4554999412670721614659896658849902451038360595703125f,-2.09079999746829248152835134533233940601348876953125f,0.9041000134140257937787055197986774146556854248046875f,0.34019997294545145649635742302052676677703857421875f,-0.609000009637326034095394788892008364200592041015625f,1.2842000156819819434161900062463246285915374755859375f,0.39050003851577719249377196319983340799808502197265625f,-2.73040007743090473724123512511141598224639892578125f,1.9554000006154179658324210322462022304534912109375f,0.0459000426515938642069158959202468395233154296875f,-0.653000006124377474492348483181558549404144287109375f,0.0539999961704011877117181938956491649150848388671875f,0.44630001372545979876349520054645836353302001953125f,-1.4093000186577437915502741816453635692596435546875f,-0.0639000035881995831488211479154415428638458251953125f,0.5542000006318092975021727397688664495944976806640625f,0.32559998140260570931303618635865859687328338623046875f,0.93760006794482475545038369091344065964221954345703125f,-0.8596999823927877759643934041378088295459747314453125f,-0.97629999208152240530722565381438471376895904541015625f,1.149200017321109879020468724775128066539764404296875f,0.03959999305158845572805148549377918243408203125f,-0.929900030142068789729137279209680855274200439453125f,0.2098999974340196583710849154158495366573333740234375f,0.4248000178158284345641959589556790888309478759765625f,-1.0641000245526435197263026566361077129840850830078125f,1.0829000383898621517886340370750986039638519287109375f,-0.912599996381997957684006905765272676944732666015625f,-0.460500017002225536799642213736660778522491455078125f,0.6630999714747076723142527043819427490234375f,0.16420001359581937183662603274569846689701080322265625f,0.3140000352859499965774148222408257424831390380859375f,0.30079997243434153375574169331230223178863525390625f,0.805900047606229907870556417037732899188995361328125f,-0.8559000082299108580485835773288272321224212646484375f,0.033399988299608107666927026002667844295501708984375f,0.6671000196129084347518301001400686800479888916015625f,0.17330004321485803497893130042939446866512298583984375f,0.393099985370039650689477639389224350452423095703125f,-0.3381999900966878858099562421557493507862091064453125f,0.78840001825094230269996842253021895885467529296875f,-1.353400056211651047988198115490376949310302734375f,0.6543000074446201352884600055404007434844970703125f,1.070300026436150186981421938980929553508758544921875f,-0.273699996519088717406020805356092751026153564453125f,-1.3009000424653294203380937688052654266357421875f,-0.5351000028640029437809744194964878261089324951171875f,-0.7961000542283063197146475431509315967559814453125f,-0.962799967651069010798892122693359851837158203125f,-1.2580000451654196336193081151577644050121307373046875f,1.393999993674456927550409091054461896419525146484375f,-2.166799998466670817975909812957979738712310791015625f,0.1719999299496410838372639773297123610973358154296875f,0.30400001811236176951780407762271352112293243408203125f,0.40599993174523074657145116361789405345916748046875f,0.220900064407289153933788838912732899188995361328125f,-0.193900007425248599446376829291693866252899169921875f,-0.87920003229677712308642867355956695973873138427734375f,0.8536999992385505553471602979698218405246734619140625f,0.6861000386565923037096581538207828998565673828125f,-0.833700026582181674683624805766157805919647216796875f,-0.369500035300851070729777347878552973270416259765625f,1.792999997295439573719022519071586430072784423828125f,-0.071099936471879399135787025443278253078460693359375f,-1.4158000884771351213231582732987590134143829345703125f,-0.0431999696075913153237024744157679378986358642578125f,1.514400053149462355150944858905859291553497314453125f,-0.21610000945031593300171834926004521548748016357421875f,0.029000039383768605461000333889387547969818115234375f,0.5796000145465141084599736132076941430568695068359375f,-0.66190002256482893816524892827146686613559722900390625f,1.135900018139183398346858666627667844295501708984375f,-0.3270999704018231479807354844524525105953216552734375f,-0.87220001882612663290927912385086528956890106201171875f,0.16940001267418314245816191032645292580127716064453125f,-0.17500000657513792301500643588951788842678070068359375f,0.09650003122538375333050453264149837195873260498046875f,0.396899960408359842034542452893219888210296630859375f,-1.002499989382922951364207619917578995227813720703125f,-0.195799984448402852876824908889830112457275390625f,-0.0740000128597020634391157045683939941227436065673828125f,1.146100030498207189566528541035950183868408203125f,0.4538999937534329287558421128778718411922454833984375f,-0.2240999840214847882435833525960333645343780517578125f,-1.160899988895653933695939485915005207061767578125f,0.03229998615086027147214053911739028990268707275390625f,0.206700016501546013358847631025128066539764404296875f,0.1477000002890822560885908387717790901660919189453125f,1.42030002528130960826047157752327620983123779296875f,-1.265900018437206231425307123572565615177154541015625f,-0.6731999956473699864289983452181331813335418701171875f,0.48819996957033851003870950080454349517822265625f,-0.058999963387847131901509101226110942661762237548828125f,-0.014400011724233774401682239840738475322723388671875f,-0.30029999816119656852464459007023833692073822021484375f,-1.589100016543269600077792347292415797710418701171875f,0.66360001518726452474083998822607100009918212890625f,0.76250001572072534372637164779007434844970703125f,0.46419999463409145068482075657811947166919708251953125f,-0.7180000040009619777237048765528015792369842529296875f,-1.3356000209093092934153901296667754650115966796875f,0.707100013428926477132563377381302416324615478515625f,-0.5212000077322127555845554525149054825305938720703125f,1.5555000444576150275821646573604084551334381103515625f,0.4904000149950384912500567224924452602863311767578125f,-0.10859998173415619238113549727131612598896026611328125f,-0.2840000066757200070099997901706956326961517333984375f,-0.63720004667267271525332716919365338981151580810546875f,1.240400023358315007016017261776141822338104248046875f,0.49130002560913565989864082439453341066837310791015625f,-0.7799999802932136017119546522735618054866790771484375f,1.0779999807178965998133435277850367128849029541015625f,0.795900013724714749940858382615260779857635498046875f,-0.889199984966963175025966847897507250308990478515625f,-1.0729000170625748733499449372175149619579315185546875f,1.235499997034668950846025836654007434844970703125f,0.4617000307321548202565963947563432157039642333984375f,1.6239000268712648900049089206731878221035003662109375f,-1.51510007148385117403677213587798178195953369140625f,0.63759998358562575138108741157338954508304595947265625f,-0.05369995902404145482478270423598587512969970703125f,0.07459995974749278335735880318679846823215484619140625f,0.4892000202827160304508424815139733254909515380859375f,0.187699974621831966459239993127994239330291748046875f,1.821000033110380389445026594330556690692901611328125f,-1.237499987855553484905613004229962825775146484375f,-1.2125000311806799491165520521462894976139068603515625f,0.80240003561824568834026649710722267627716064453125f,-0.98520003523230548214684176855371333658695220947265625f,1.2309999985396868904530265353969298303127288818359375f,-0.1534999793916937971260949780116789042949676513671875f,-0.57549996078759380058187389295198954641819000244140625f,0.1068999693304302223140211935970000922679901123046875f,0.5072999692752955258612246325355954468250274658203125f,-1.265300011982023686840648224460892379283905029296875f,1.0605000015050174422270856666727922856807708740234375f,0.4058000010073181673675435376935638487339019775390625f,-0.390499999158084420969316852279007434844970703125f,0.6521000044509766002676087737199850380420684814453125f,-1.9036999818786970450901208096183836460113525390625f,0.1273999673336743132523452004534192383289337158203125f,1.351700008231401017866346592199988663196563720703125f,-0.8186999711871141283126007692771963775157928466796875f,-0.62260003013312825714820064604282379150390625f,1.30779997495263788920283332117833197116851806640625f,-0.6465000025033951214226135562057606875896453857421875f,-1.5794000159338115185647666294244118034839630126953125f,0.9644000424578786923035522704594768583774566650390625f,1.0568000163853172868044794086017645895481109619140625f,-1.2860999796107408510437153381644748151302337646484375f,0.1273999866679311754324999128584749996662139892578125f,0.755699976760148661725224883412010967731475830078125f,-0.4796999921530484822795870059053413569927215576171875f,-0.47540000098794710226002280251123011112213134765625f,0.9774000177741049011359564246959052979946136474609375f,-0.2808000086814164131254756284761242568492889404296875f,1.0984000411987311363049002466141246259212493896484375f,0.438899983024596540559514323831535875797271728515625f,0.89500001566484588266803257283754646778106689453125f,-1.1235000087097291032733892279793508350849151611328125f,0.225000011920929221531650910037569701671600341796875f,0.103300017286837297891821663142764009535312652587890625f,-0.5917000219963493012897970402264036238193511962890625f,0.81270000252425678155532295932061970233917236328125f,0.6687000022374094410082534523098729550838470458984375f,0.946000020910054306710890159592963755130767822265625f,0.2135000154338781985785544748068787157535552978515625f,-0.2912000105075540634658182170824147760868072509765625f,-0.4040999899819490703833935185684822499752044677734375f,0.7397999825373293703023591660894453525543212890625f,-0.375200015935302388925265404395759105682373046875f,-0.881699988040328008764845435507595539093017578125f,-0.1239999944567677925277848771656863391399383544921875f,0.0682999876335264044513451153761707246303558349609375f,0.80850003039091777878866196260787546634674072265625f,-0.82329998573362761504768059239722788333892822265625f,-1.498200020942091814646346392692066729068756103515625f,1.6075000247359272353975256919511593878269195556640625f,0.17300001718849011833611939437105320394039154052734375f,-1.2661000173106791155674955007270909845829010009765625f,0.250000008903444037589025583656621165573596954345703125f,0.464799995166063018015023544649011455476284027099609375f,1.6072000124186278657845150519278831779956817626953125f,-0.56749997820705122553164301280048675835132598876953125f,-0.874400032921135217378605375415645539760589599609375f,0.829899982011317316477061467594467103481292724609375f,0.4456999957218765739952459625783376395702362060546875f,-2.566800013516842948746443653362803161144256591796875f,2.14670001691132750210044832783751189708709716796875f,-0.68150000634044349823170705349184572696685791015625f,-0.28820001010149720599429201683960855007171630859375f,-0.8364000202775005998745427859830670058727264404296875f,-1.3194000369444494058512873380095697939395904541015625f,-0.25450003239512464237037647762917913496494293212890625f,-0.2840999818980700641901648850762285292148590087890625f,1.6154000209555032796515661175362765789031982421875f,-1.84600002998113676966340790386311709880828857421875f,-0.3701999730572096414249472218216396868228912353515625f,1.498100041417032901591710469801910221576690673828125f,-0.06060002167299438635694741606130264699459075927734375f,-0.8971999949097628768157619560952298343181610107421875f,0.966099995033442571212844995898194611072540283203125f,-1.35780001436620967325552555848844349384307861328125f,0.0969000000640753977876329372520558536052703857421875f,0.391199981729685841713717309175990521907806396484375f,-1.9483000069677831600500894637661986052989959716796875f,0.656000013291836214790464509860612452030181884765625f,-0.84610003496855501481377359596081078052520751953125f,0.20379998612105809652206289683817885816097259521484375f,-0.33769999317377852321442333050072193145751953125f,-0.3086999783769244221076633039047010242938995361328125f,0.093299972508847250907138004549778997898101806640625f,1.4143999877333637282816880542668513953685760498046875f,-0.9142999741405244407133068307302892208099365234375f,0.220199974361061645566906008752994239330291748046875f,-1.181400031788647719821483406121842563152313232421875f,0.35480002940148158696587188387638889253139495849609375f,0.21350000442564465163997056151856668293476104736328125f,-0.7610999942511311733284173897118307650089263916015625f,0.31749997001141228025744567275978624820709228515625f,0.73149994818866215240404926589690148830413818359375f,0.3123000143140555184828599522006697952747344970703125f,-0.7768999954819684194973206103895790874958038330078125f,0.217599925303459063030686593265272676944732666015625f,0.38490004847049785752233219682238996028900146484375f,-0.2402999921262267690735825453884899616241455078125f,-1.451800071780384282504883231013081967830657958984375f,0.757200021247566024129582729074172675609588623046875f,-0.28999996323138399478835935951792635023593902587890625f,1.5322999906957146531993885218980722129344940185546875f,0.189299991574883197387180189252831041812896728515625f,-0.3607999672189350359730042328010313212871551513671875f,-0.36309998514652230827692847014986909925937652587890625f,0.069199998731911238625258420142927207052707672119140625f,-1.006400004146993154563460848294198513031005859375f,-0.6401000007219608267661214995314367115497589111328125f,-0.0909000404164199693468617624603211879730224609375f,0.23879996034204886701246550728683359920978546142578125f,0.0221999967165287248604954584152437746524810791015625f,1.50790005001276750107308544102124869823455810546875f,-1.25440004245787850578608413343317806720733642578125f,-0.3833000192612414647186369620612822473049163818359375f,0.67940001004785244287376144711743108928203582763671875f,-0.0532999906510107646084861698909662663936614990234375f,0.2041999944850798609508046865812502801418304443359375f,0.2958999721691009110458026043488644063472747802734375f,0.84640001215636839759781651082448661327362060546875f,0.7669000075146552575233727111481130123138427734375f,-0.268900016030669153366261525661684572696685791015625f,-0.283600027015060540680480016817455179989337921142578125f,-0.191799991183728035792910304735414683818817138671875f,0.695300047353656136550625888048671185970306396484375f,-0.12060007172226949734294976224191486835479736328125f,0.862299992558359207350804354064166545867919921875f,-0.2225999283954498420001755221164785325527191162109375f,-0.59480005380213274435163839370943605899810791015625f,1.8628000193208453794113665935583412647247314453125f,-2.09459999228790394454335910268127918243408203125f,-0.4510000111311669446223504564841277897357940673828125f,0.7217000126458705810961191673413850367069244384765625f,-1.7643000050604344952631663545616902410984039306640625f,3.314500047519803427320539412903599441051483154296875f,-0.074999983608722065042684334912337362766265869140625f,0.595900016500056306512078663217835128307342529296875f,0.70879999951347671949264395152567885816097259521484375f,1.1955000341385602080634953381377272307872772216796875f,-1.24639997907578869984490665956400334835052490234375f,0.8671999336287374848808440219727344810962677001953125f,0.5643000330373639794601103858440183103084564208984375f,0.690800014790892813465461586019955575466156005859375f,-1.087400028987228761678807131829671561717987060546875f,0.6303000143289561041370916427695192396640777587890625f,0.858800001133978252454426183248870074748992919921875f,-0.971599994713067349749735512887127697467803955078125f,1.0072999926328660702523620784631930291652679443359375f,0.24019997059851849741107798763550817966461181640625f,0.47830000260919314225560583508922718465328216552734375f,-0.34470002933144583412428119117976166307926177978515625f,-1.163499981347472012060961787938140332698822021484375f,-0.3511999673500645346990722828195430338382720947265625f,1.1679999497793607243778524207300506532192230224609375f,-0.18790000188201705366708438305067829787731170654296875f,-1.819299984236061096254388758097775280475616455078125f,0.678800002139806846201963708153925836086273193359375f,0.0495999769397077461263734221574850380420684814453125f,0.35370005057305153872704295281437225639820098876953125f,-1.9163000294275587265246940660290420055389404296875f,-1.2201000120282170513519304222427308559417724609375f,0.9656999802991743653279854697757400572299957275390625f,1.682999999530613077780571984476409852504730224609375f,-1.39709998865574558379876179969869554042816162109375f,-0.882600007818639564760587745695374906063079833984375f,1.7899000282794244043316211900673806667327880859375f,-0.58599996940791598820652552603860385715961456298828125f,0.849299991500377604580762636032886803150177001953125f,0.36280002628713881041022659701411612331867218017578125f,0.533399973584711251106682539102621376514434814453125f,0.225500029817224056927216224721632897853851318359375f,0.80329996546804860457768882042728364467620849609375f,0.3889000391647219334601004447904415428638458251953125f,0.431700076590479486782214735285378992557525634765625f,-1.0008000908985741350676335059688426554203033447265625f,-0.1504999988973139135595147308777086436748504638671875f,0.5483000395268200488629872779711149632930755615234375f,0.5269000236079099952490878422395326197147369384765625f,-0.2928000204384326732309773433371447026729583740234375f,0.04829999031573517509485782284173183143138885498046875f,1.3618000220477581851952209035516716539859771728515625f,0.57879999711066487666499824626953341066837310791015625f,-0.46729996445104504676493206716259010136127471923828125f,-2.51350006332248465668044445919804275035858154296875f,0.5632000257663423159470994505682028830051422119140625f,-0.43529996984899010925573747954331338405609130859375f,0.3877000132530932230423559303744696080684661865234375f,-0.25620000147372490317820847849361598491668701171875f,-0.443900005711615364134559058584272861480712890625f,0.700200013066828663710339242243207991123199462890625f,-0.6037000041559339447161391944973729550838470458984375f,-0.2747000205397609473578768302104435861110687255859375f,-1.315100013071298423739108329755254089832305908203125f,0.553399979433416344676288645132444798946380615234375f,1.1056999624922869696064253730583004653453826904296875f,-0.6293999261543150591791118131368421018123626708984375f,0.4880999989300967012439969039405696094036102294921875f,-0.3233000795364386892316588273388333618640899658203125f,1.81200006681680658715549725457094609737396240234375f,-0.9429000324666503285442331616650335490703582763671875f,-1.205499990515410235758508861181326210498809814453125f,0.995200022938847350673086111783050000667572021484375f,-0.12370004222840069729727474623359739780426025390625f,-0.78570000682324181440208121784962713718414306640625f,0.54950001900270617483101887046359479427337646484375f,0.47139999550432010178013797485618852078914642333984375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.2816999755978581987392317387275397777557373046875f,0.20349994117021574790982185731991194188594818115234375f,-2.00950005310773871514129496063105762004852294921875f,0.149699936087429652520341960553196258842945098876953125f,-0.835100014114380062579812147305347025394439697265625f,0.3158000352799898990951987798325717449188232421875f,0.660000009089708239429228342487476766109466552734375f,-0.53149998216330995859379981993697583675384521484375f,0.75249998036771958975776897204923443496227264404296875f,-0.6415000044405463341945505817420780658721923828125f,0.346900061717629437652021806570701301097869873046875f,1.1262999508798132719533668932854197919368743896484375f,-1.591099976667761151105651151738129556179046630859375f,0.408900011634826032747014323831535875797271728515625f,-0.19160000901818252661001906744786538183689117431640625f,-0.571700002438575438645784743130207061767578125f,-0.34329998506307557448735678917728364467620849609375f,1.195100026333332010608501150272786617279052734375f,-0.6704999353438612086364400965976528823375701904296875f,0.8971000157386068707410231581889092922210693359375f,0.725500038087367737915656107361428439617156982421875f,0.675200001220404910640127127408050000667572021484375f,1.40899998197704601210489272489212453365325927734375f,0.7928999662309890172906534644425846636295318603515625f,-0.47209999614357955888266360489069484174251556396484375f,0.05870001536905833372514962320565246045589447021484375f,-0.26870001022815748203953489792183972895145416259765625f,-0.252200010146200082061795910703949630260467529296875f,1.2483999660968780887060347595252096652984619140625f,2.200399975395202378791736919083632528781890869140625f,0.490300014254450911010962954605929553508758544921875f,-0.0150000144541262425690320014837197959423065185546875f,1.564299976897240096462837755098007619380950927734375f,1.375600018376112121387677689199335873126983642578125f,0.7894999967813489671897286825696937739849090576171875f,0.867500012293457967160748012247495353221893310546875f,0.0210999911218880953356347163207828998565673828125f,1.1029999789297579848579289318877272307872772216796875f,-1.0977000067710875885751420355518348515033721923828125f,0.262999981388450265740175382234156131744384765625f,0.03669994698762923945878355880267918109893798828125f,-0.7297999915897843692391688819043338298797607421875f,-0.281599983462691216828943652217276394367218017578125f,-1.733200015801191096187494622427038848400115966796875f,0.37539997967928606659171464343671686947345733642578125f,-0.26060001640170848968836025960627011954784393310546875f,-0.1599999900907282945894394288188777863979339599609375f,0.399999933056534029418571662972681224346160888671875f,-0.069099971243739677362327711307443678379058837890625f,0.073300003913045141956672523519955575466156005859375f,-0.017399937605858450950790938804857432842254638671875f,-0.073599970036745343549000608618371188640594482421875f,0.1743000186204912704823755120742134749889373779296875f,0.034700003477931229411979074939154088497161865234375f,0.06870000844001733497634631930850446224212646484375f,1.950700011628865748747330144396983087062835693359375f,0.590799972471594880829570683999918401241302490234375f,1.9224999645352358612626630929298698902130126953125f,0.5729999729692938359448817209340631961822509765625f,1.343999989315867171768559273914434015750885009765625f,0.27610002882182538286315320874564349651336669921875f,1.2588999931573880974866597171057946979999542236328125f,0.87130002031922249017270587501116096973419189453125f,-1.3160999917179345697348935573245398700237274169921875f,-0.009799991962314624771579474327154457569122314453125f,-0.2298000389009706712073466405854560434818267822265625f,-0.59719998038113164540163779747672379016876220703125f,0.9247999774336819189812786135007627308368682861328125f,-0.11619999487996068854300801831413991749286651611328125f,0.5165999823451041805810746154747903347015380859375f,-0.5883999675124884642940514822839759290218353271484375f,1.05160003122091350036271251155994832515716552734375f,-0.1296999777734288983310761977918446063995361328125f,-0.9678000624597073997534835143596865236759185791015625f,1.4181000244855876957217333256267011165618896484375f,0.069999957010150382075153174810111522674560546875f,-1.66780000896453817205156155978329479694366455078125f,1.8577000031203028118653719502617605030536651611328125f,-0.22470000541508217128949809193727560341358184814453125f,-1.2450000336021194158320213318802416324615478515625f,0.5971999628722670649949577637016773223876953125f,-1.6195999973729247045639567659236490726470947265625f,-0.5551999663144346897070136037655174732208251953125f,-0.218499988704919712034779877285473048686981201171875f,0.64909998010993008588087604948668740689754486083984375f,-0.3477999552831061436108939233236014842987060546875f,0.1614999900795519582885617637657560408115386962890625f,0.99950000620633339121212657119031064212322235107421875f,0.57489995790869041769610703340731561183929443359375f,-0.282700021709501481126380895148031413555145263671875f,0.2763999768778682142311708958004601299762725830078125f,-0.1271000173032288405039480494451709091663360595703125f,-0.16490003133416164526892089270404540002346038818359375f,-0.446800023537874846368822545628063380718231201171875f,1.4996999450653796248644766819779761135578155517578125f,-0.2065999853998430335622060738387517631053924560546875f,-0.6657000152051442132261627193656750023365020751953125f,0.4326999914973976846255254713469184935092926025390625f,-1.47720000794827921453133967588655650615692138671875f,0.971199973645806569066962765646167099475860595703125f,0.3647999968051915953282104965182952582836151123046875f,-0.24609997465610555789083946365281008183956146240234375f,0.1505999781429772088614527092431671917438507080078125f,-0.013699977184831624299476970918476581573486328125f,0.4242999935120341792327280927565880119800567626953125f,-0.8814000146150586800786186358891427516937255859375f,-0.9012999786704778415469263563863933086395263671875f,-0.1575999896019697477100862670340575277805328369140625f,-0.863199996951222292551619830192066729068756103515625f,0.050099990272522543222066815360449254512786865234375f,1.08099997894465982284373239963315427303314208984375f,-1.4688999754250051665849241544492542743682861328125f,0.95969997579902421858122352205100469291210174560546875f,-0.31970001118928170757982343275216408073902130126953125f,0.494899974448979040886342772864736616611480712890625f,0.9509999761134382278982002389966510236263275146484375f,0.21170000748634354437172078178264200687408447265625f,0.26950003947317535679673028425895608961582183837890625f,-0.03949997711181563175841802149079740047454833984375f,0.2228999943196772193232391146011650562286376953125f,-0.673200028318166943108735722489655017852783203125f,-0.8102000005312259833800681008142419159412384033203125f,-1.38260002784207447490416598157025873661041259765625f,0.29229997307509225645816286487388424575328826904296875f,0.6497000281766052154353019432164728641510009765625f,-0.75019999470114751005667130812071263790130615234375f,-0.243400001114606412411234259707271121442317962646484375f,-1.633400010949373371005322042037732899188995361328125f,1.727400020344555198192892930819652974605560302734375f,0.58370001771599044904093034347170032560825347900390625f,-0.73960000847428986059384214968304149806499481201171875f,-0.51180002504661725737378219491802155971527099609375f,-0.27910000939071177850081539872917346656322479248046875f,-0.8522999813079834563467329644481651484966278076171875f,-1.0530999901384110462032595023629255592823028564453125f,0.1847000084698204869226856317254714667797088623046875f,-0.28240001073330700531727188717923127114772796630859375f,-1.73679996825456584019775618799030780792236328125f,0.160200044247507999983781701303087174892425537109375f,-0.9070000009536745011473612976260483264923095703125f,1.0168000183969738703382290623267181217670440673828125f,0.43229999121725537492011426365934312343597412109375f,-1.166699986848235948144747453625313937664031982421875f,0.0317000119566916982449811257538385689258575439453125f,-0.7580000216215847519407589061302132904529571533203125f,-0.66130000571608549275737232164829038083553314208984375f,0.6623999991476539062063011442660354077816009521484375f,-0.16770002256631866277558629008126445114612579345703125f,0.1161000057995320577930442595970816910266876220703125f,0.407699964265525294226932828678400255739688873291015625f,0.5998000269800420891641579146380536258220672607421875f,0.047600024172663901733670854810043238103389739990234375f,0.2878999555021526379761098723975010216236114501953125f,1.3133999694496392152132102637551724910736083984375f,-0.96699997044354690789447204224416054785251617431640625f,0.77089997146129662919378233709721826016902923583984375f,0.34479999457001664353583691990934312343597412109375f,0.244199970158935020236867785570211708545684814453125f,-0.878300022763014442972462347825057804584503173828125f,0.5400999225467446596127274460741318762302398681640625f,0.1563000192016363865121775234001688659191131591796875f,-0.9052000010341405555891469703055918216705322265625f,-0.292400031185149789081378912669606506824493408203125f,-1.5104000531792636241590344070573337376117706298828125f,-0.7022999646931886719158910636906512081623077392578125f,-1.171900023719668393340498369070701301097869873046875f,-3.073000037491322444793695467524230480194091796875f,-0.26690001440644284258496554684825241565704345703125f,0.244099995700269634113510619499720633029937744140625f,0.6610999926492573397496244069770909845829010009765625f,0.1095000118315219150844086470897309482097625732421875f,-0.13170000566095085847706513959565199911594390869140625f,-0.39969998816698815158332536157104186713695526123046875f,-1.151600008831918042773168053827248513698577880859375f,0.087500013466924375649114153929986059665679931640625f,-0.23299997842684361248899449492455460131168365478515625f,-0.68920003106743099596798174388823099434375762939453125f,-0.2665999983265996409187437166110612452030181884765625f,0.3361000066190957813461182013270445168018341064453125f,2.02360003679394662157164930249564349651336669921875f,0.8647000148773196226414938792004249989986419677734375f,1.8571999943137171840135124512016773223876953125f,-0.15319999508857762293700943700969219207763671875f,-0.4756000286579131763886607586755417287349700927734375f,-0.0175999832689759383441696627414785325527191162109375f,0.6887999696180229225461744135827757418155670166015625f,0.07829998625516854104944286518730223178863525390625f,-0.4739999777674668290927684211055748164653778076171875f,-0.3374000223189594915851330370060168206691741943359375f,-0.01780000810772197628040203198906965553760528564453125f,-1.524499964520334760464947976288385689258575439453125f,-1.5387999946519730354310695474850945174694061279296875f,-1.3843999641895294860916010293294675648212432861328125f,-0.10820002973377729205139985424466431140899658203125f,-0.2872000063091528776482164175831712782382965087890625f,-1.534999999701976758359478480997495353221893310546875f,0.54630003484785516487676204633316956460475921630859375f,2.284399970969558157918299912125803530216217041015625f,-0.87949991445243380638885355438105762004852294921875f,1.9645000066161151242027926855371333658695220947265625f,0.9114999828338625764700964282383210957050323486328125f,1.4394000655174254976742531653144396841526031494140625f,0.91989999225586682740640753763727843761444091796875f,0.55140003592371955587481124894111417233943939208984375f,0.682500002309680109391365476767532527446746826171875f,-1.39220001368522616758127696812152862548828125f,1.860800029990076875918703080969862639904022216796875f,0.003100026571750458259657534654252231121063232421875f,-0.74290001853406517628997107749455608427524566650390625f,-0.92739999218136104364162974889040924608707427978515625f,0.820299976256489316028819303028285503387451171875f,0.382700016158819433798043974093161523342132568359375f,1.1064000015020365452755868318490684032440185546875f,1.172199963542819745043743751011788845062255859375f,1.658300015610456146220030859694816172122955322265625f,2.2166999785780916454314137808978557586669921875f,1.638300008904934212949910943279974162578582763671875f,1.058900009995698443532319288351573050022125244140625f,-0.59009999392330581002852341043762862682342529296875f,0.265299982962012403930884829605929553508758544921875f,-1.3049000320047137169154893854283727705478668212890625f,-0.22640000060796749981051334543735720217227935791015625f,-1.260099953988194965148750270600430667400360107421875f,0.158499972760677731997702721855603158473968505859375f,-0.07499997287988691141435992904007434844970703125f,0.1023999758273370641603605690761469304561614990234375f,-1.05359999514520108476745008374564349651336669921875f,-0.8301999518603080385759085402241908013820648193359375f,-0.350199976149202285569117520935833454132080078125f,-1.6808999263107775590242454200051724910736083984375f,0.003700019019841516154656346770934760570526123046875f,0.6912999873504042103178335310076363384723663330078125f,-0.24920001106262146262082524117431603372097015380859375f,1.4954000362664461309947228073724545538425445556640625f,0.817199959780275708709496029769070446491241455078125f,-0.4220999607905751016545536913326941430568695068359375f,1.4490000019818538579130517973680980503559112548828125f,-0.089499970480799273531147264293394982814788818359375f,-1.452400067171454534076247000484727323055267333984375f,-2.612799995329976976421448853216134011745452880859375f,1.1722999883860349967790170921944081783294677734375f,0.4245999886184936844557569202152080833911895751953125f,1.3245999636590479742181969413650222122669219970703125f,0.542699984642863508810251005343161523342132568359375f,0.268199981230497541684343332235584966838359832763671875f,1.0053000106409191172218697829521261155605316162109375f,0.06890003163963476229270099793211556971073150634765625f,-0.0268999867275356852047707434394396841526031494140625f,-0.02650004294142094263264652909128926694393157958984375f,-0.194300017391145107836791794397868216037750244140625f,0.98209996313750735907888156361877918243408203125f,-1.06240000436306036846190181677229702472686767578125f,-0.701200021180510280061071171076036989688873291015625f,0.41009992984831367124343159957788884639739990234375f,-0.9251000142633911682565894807339645922183990478515625f,-0.110999951079488123895089302095584571361541748046875f,0.48199998053908377215748259914107620716094970703125f,0.430400023600458592198947371798567473888397216796875f,1.319799971324205234424198351916857063770294189453125f,1.345400034049898607690920471213757991790771484375f,0.57119998776465674694691188051365315914154052734375f,-0.790800021608174308340721836430020630359649658203125f,1.6250999929547305100641096942126750946044921875f,-0.599799977582693433220129008986987173557281494140625f,-0.199200006815789976144515094347298145294189453125f,0.70810001703500713432504198863171041011810302734375f,-0.355199974659085260242363801808096468448638916015625f,0.90869996239543004978855833542183972895145416259765625f,-0.91330004559904409688897430896759033203125f,-0.5469000068999829711202664839220233261585235595703125f,-0.1018000328138473076222680901992134749889373779296875f,-2.614099978843331228972601820714771747589111328125f,0.6920999298334127303178320289589464664459228515625f,-0.3783999884299935612119725192314945161342620849609375f,0.12159997541606404336533842069911770522594451904296875f,-1.5154999916702511786326112996903248131275177001953125f,1.0747999395102265651757988962344825267791748046875f,0.0253000438332551258469038657494820654392242431640625f,-0.1018000116914503649923062766902148723602294921875f,0.44210000553131134637396826292388141155242919921875f,1.088800007355212873250138727598823606967926025390625f,-0.2494999891817564208196245090221054852008819580078125f,-1.3227999775975940455197132905595935881137847900390625f,-0.3007000137895341485005928916507400572299957275390625f,-2.010899975112080628747435184777714312076568603515625f,-1.5736999472707520197900521452538669109344482421875f,-0.6800999891549344145147415474639274179935455322265625f,1.773900012230873013407972393906675279140472412109375f,1.08759997507333761035397401428781449794769287109375f,0.35379997542947527744416902351076714694499969482421875f,1.51620002179518298390803465736098587512969970703125f,0.52690000244826096231776091372012160718441009521484375f,1.6994999591559178941935215334524400532245635986328125f,1.002999999605119541001840843819081783294677734375f,1.603700024943053303871920434175990521907806396484375f,0.0228999720796949635115424825926311314105987548828125f,0.1979999901615077106953322072513401508331298828125f,0.2889000068292018053028868962428532540798187255859375f,-2.32189996102303286562573703122325241565704345703125f,-1.3535999466046693129328559734858572483062744140625f,1.1010999462693937811508249069447629153728485107421875f,-0.727300008502602235438416755641810595989227294921875f,-2.54330001415759365812618852942250669002532958984375f,-0.4292000125899908535842541823512874543666839599609375f,-2.061599972435832484052298241294920444488525390625f,0.63170004645288069244912776412093080580234527587890625f,0.0907000482112170214321622552233748137950897216796875f,0.885300020438432699876329934340901672840118408203125f,0.06410004187524354080096600227989256381988525390625f,0.039699949014186497464606873109005391597747802734375f,-0.90010002037286707121666040620766580104827880859375f,0.94110000308006913627423273283056914806365966796875f,-0.711300032053887587579765749978832900524139404296875f,-2.37390005149543270590584143064916133880615234375f,-1.216200008067488624874386005103588104248046875f,0.3576000103890895065461563717690296471118927001953125f,-0.9075000056624416000516930580488406121730804443359375f,1.27590002871900853875786197022534906864166259765625f,-0.0605000399872663852107734783203341066837310791015625f,1.127499974817038008723102393560111522674560546875f,2.48899999956041551030239133979193866252899169921875f,-0.664399963649362046425039807218126952648162841796875f,0.522599998691678280238193110562860965728759765625f,-0.3544000049442057598270139351370744407176971435546875f,-0.008600013399124417645680296118371188640594482421875f,-0.15559999069720487074874881727737374603748321533203125f,-0.7539000225499272733742373020504601299762725830078125f,-0.48899999460577969756513994070701301097869873046875f,-0.80279998407959940465161707834340631961822509765625f,-0.54070002340078371361187237198464572429656982421875f,-2.269899964912235557079611680819652974605560302734375f,-0.604299958494305400336088496260344982147216796875f,0.181199978786706683564489139826036989688873291015625f,-2.095799956154823728837754970300011336803436279296875f,0.812599981108306668176055609364993870258331298828125f,-1.189000003695486640253875521011650562286376953125f,1.464300026816129918216802252572961151599884033203125f,0.9547000177085396277476547766127623617649078369140625f,-1.0255999832838771368415109463967382907867431640625f,0.084900005629658270578374867909587919712066650390625f,-0.739300016236304369243725886917673051357269287109375f,-1.50699999789893634982718140236102044582366943359375f,-0.4212000144377352484070797800086438655853271484375f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.4235000322163109398587721443618647754192352294921875f,-0.6631000278197232145060979746631346642971038818359375f,-1.7303000404432420378242341030272655189037322998046875f,-1.1091000415213405627667953012860380113124847412109375f,0.47780000021010604616122918741893954575061798095703125f,0.180499963469802848781142756706685759127140045166015625f,0.2343000522971159238494465171243064105510711669921875f,-0.9913000512018805832070711403503082692623138427734375f,-1.0250000022351739570325435124686919152736663818359375f,0.2425999743469067215784207292017526924610137939453125f,-0.26300001987069909770156073136604391038417816162109375f,0.48800000327453030291025015685590915381908416748046875f,-0.56120000981837581122135816258378326892852783203125f,-1.4190000133812425975321502846782095730304718017578125f,0.330099989918619385331766125091235153377056121826171875f,-1.12800002964958512308157878578640520572662353515625f,1.254800042328238163236164837144315242767333984375f,1.5159000434339053864363222601241432130336761474609375f,0.1074000453412538025332878532935865223407745361328125f,-0.21100002595782374470445574843324720859527587890625f,-0.6929000069111577175817728857509791851043701171875f,1.22330003311932156151442541158758103847503662109375f,0.43470002677962182868753870934597216546535491943359375f,0.37150002930685899205087707741768099367618560791015625f,0.191599989479034282791047871796763502061367034912109375f,-0.5567999979823825551505933617590926587581634521484375f,0.00050002946332085296266001250842236913740634918212890625f,0.4452999761447304916117673201370052993297576904296875f,0.18499998932704297960327721739304251968860626220703125f,0.10499996358528684847755130249424837529659271240234375f,1.786800030727684340803307350142858922481536865234375f,1.2239000430576505440427581561380065977573394775390625f,-0.75529997817501459689282228282536379992961883544921875f,-0.9296000391334298118550805156701244413852691650390625f,0.932100020041316756902460838318802416324615478515625f,-0.97330003181546931312340120712178759276866912841796875f,-0.2863999793924383308052483698702417314052581787109375f,0.925799984057247282720481962314806878566741943359375f,0.256599988268316148154468692155205644667148590087890625f,-0.55779994716197212056840726290829479694366455078125f,-1.04490003323406011048746222513727843761444091796875f,0.2835999948658043390281591200619004666805267333984375f,-0.7081999950885773387199151329696178436279296875f,-0.1938000194810333975947713724963250569999217987060546875f,-0.4314000102192163677017333611729554831981658935546875f,0.54779997631236898580908700751024298369884490966796875f,0.031999964427202598482580242489348165690898895263671875f,-0.453399980178474992786874508965411223471164703369140625f,0.32920002464130504460371184904943220317363739013671875f,0.1573000124886634321086376075982116162776947021484375f,-0.8075000591762366841663833838538266718387603759765625f,-0.235999980416148391615394075415679253637790679931640625f,-0.123000006925314819827832479859353043138980865478515625f,-0.916500035919249445015566379879601299762725830078125f,1.8344000675104565889483865248621441423892974853515625f,0.253900012137741104556454274643328972160816192626953125f,1.24890001917853954438442087848670780658721923828125f,-0.68470005103126274637048709337250329554080963134765625f,1.1603000142171977504546021009446121752262115478515625f,-0.8691000221684570536950786845409311354160308837890625f,0.23109997638836399236339502749615348875522613525390625f,0.62939999056458562076699081444530747830867767333984375f,-0.907899976643175232737803526106290519237518310546875f,-0.35670000228956333376828524706070311367511749267578125f,-0.4609000179715458944684769448940642178058624267578125f,0.1738999972552066897168998593770083971321582794189453125f,0.364500006075947180494978283604723401367664337158203125f,0.0985000017061829213904644575450220145285129547119140625f,-1.139000014591962095522603704011999070644378662109375f,0.07059998494163066684592422461719252169132232666015625f,1.7943000416986638612115712021477520465850830078125f,-1.0451999931924031983498935005627572536468505859375f,0.70180003741458019117516187179717235267162322998046875f,1.018699954460561496460968555766157805919647216796875f,-0.40639999244958191493282129158615134656429290771484375f,-0.153900006140023304368469325709156692028045654296875f,-0.34599999473989073717206110814004205167293548583984375f,-1.18960006075874158426586291170679032802581787109375f,1.19279999581426476851220286334864795207977294921875f,0.72480002386942476988451744546182453632354736328125f,-1.54310002273470114886322335223667323589324951171875f,-0.09899998095259066044349083313136361539363861083984375f,0.9864000188991426387730143687804229557514190673828125f,-0.04679999552369074056201725397841073572635650634765625f,-1.847400024405121854442768380977213382720947265625f,1.7671999921716743386923553771339356899261474609375f,0.93229999125450901953371385388891212642192840576171875f,-0.7808999937944116620514023452415131032466888427734375f,0.59720000128000971795216855753096751868724822998046875f,-1.08440005212500789610885476577095687389373779296875f,-0.40629998813271483992792809658567421138286590576171875f,-0.51970003672614739631541169728734530508518218994140625f,0.39540000108107908882715264553553424775600433349609375f,0.45400000404939044873486864162259735167026519775390625f,-1.6092000195749107671616684456239454448223114013671875f,0.65900000250339541008059995874646119773387908935546875f,1.221600022112578454169806718709878623485565185546875f,-0.67300004212930863989328145180479623377323150634765625f,0.69589999979212902037062349336338229477405548095703125f,0.17139999276623052448798034674837253987789154052734375f,0.334100030848384743631385163098457269370555877685546875f,0.68410005314424660038952197282924316823482513427734375f,-1.02830006333142609520336918649263679981231689453125f,-0.9218999972514805296697204539668746292591094970703125f,0.58380000469163040310860424142447300255298614501953125f,0.49310000384748065016538021154701709747314453125f,-0.969300008711219351908994212863035500049591064453125f,-1.522400072703510520710779019282199442386627197265625f,0.53090004203692142681347831967286765575408935546875f,0.194500003598629955803289703908376395702362060546875f,-0.21269999386295712628935916654882021248340606689453125f,-0.012800020457059080314365928643383085727691650390625f,0.372499972656369304768730899013462476432323455810546875f,0.35750000990927188748713660970679484307765960693359375f,0.85830002626478663074749420047737658023834228515625f,-0.62350001960620338170571130831376649439334869384765625f,1.5096000860907146101652642755652777850627899169921875f,1.7070000128187234889054479936021380126476287841796875f,-0.815000019185244894970310269854962825775146484375f,-0.254999991171061257677621370021370239555835723876953125f,-1.25040003436654867385868783458136022090911865234375f,-0.76259997393712330993054138161824084818363189697265625f,0.213000013295560730597344445413909852504730224609375f,0.079999966975300818017302617590758018195629119873046875f,1.550099997872114609975824350840412080287933349609375f,-0.0822000162370504217079769659903831779956817626953125f,-0.997700068890303715107847892795689404010772705078125f,0.39600005764141688047175193787552416324615478515625f,0.246099967876076686223996148328296840190887451171875f,0.1553000172346831464498251307304599322378635406494140625f,1.1795000065229828134505396519671194255352020263671875f,-0.48379996708482420064711959639680571854114532470703125f,-0.8553000361591587275000847512274049222469329833984375f,-0.585100015623122526875476978602819144725799560546875f,-1.6281000212818383943158551119267940521240234375f,0.341400050154329226526073171044117771089076995849609375f,0.48840000273510841832802498174714855849742889404296875f,-0.05450004647672213042852717990172095596790313720703125f,-0.5352000321216880962538198218680918216705322265625f,-0.80859998702406843396062186002382077276706695556640625f,-0.436800019924343196908722575244610197842121124267578125f,1.06710001454651415286889459821395576000213623046875f,-0.83520001606568705643240946301375515758991241455078125f,-1.172500037346035473007077598595060408115386962890625f,1.17420004948899237007253759657032787799835205078125f,-0.2601000213973228003538906705216504633426666259765625f,-0.41020002187714010855046353754005394876003265380859375f,0.102999962314962834053488904828554950654506683349609375f,-1.973400055522472218427765255910344421863555908203125f,0.86050005453452504955436097588972188532352447509765625f,-0.792000041875988269879371728166006505489349365234375f,-0.2891999962173399385534366956562735140323638916015625f,0.304500003132968988239781538140960037708282470703125f,-0.29349999030679452260983453015796840190887451171875f,-0.61739999968782066641637129578157328069210052490234375f,0.779500051785261494075029986561276018619537353515625f,0.5131999626971770478434109463705681264400482177734375f,-1.871000064589082523269780722330324351787567138671875f,0.48100005043297999751672477941610850393772125244140625f,0.70059998419657343315947173323365859687328338623046875f,-0.7638000126637518594208131617051549255847930908203125f,0.3845000146441159838417434002622030675411224365234375f,0.045799980387836336515050561501993797719478607177734375f,-0.2695999667137860544841032606200315058231353759765625f,-1.7698001038886623792478758332435972988605499267578125f,-0.2307999808534975105800413075485266745090484619140625f,0.390700000173598738228974980302155017852783203125f,-1.583500036798417998085142244235612452030181884765625f,-0.33740000471696285355704958419664762914180755615234375f,0.14539999798908864736546320273191668093204498291015625f,0.0117000165574254466616110903487424366176128387451171875f,0.53710001268386842188107266338192857801914215087890625f,0.67039998683184365280141037146677263081073760986328125f,-1.547700014277547442276272704475559294223785400390625f,-0.207299978141486984650754266112926416099071502685546875f,-0.99440002291873097473029474713257513940334320068359375f,0.3130000282898546259957583970390260219573974609375f,0.70670002728626146737411772846826352179050445556640625f,-0.126300014675408800446376744730514474213123321533203125f,-0.3724000200279053984786514774896204471588134765625f,2.294000046461821096244193540769629180431365966796875f,0.2717999773256474238536384291364811360836029052734375f,0.16320001030638875416656219385913573205471038818359375f,-0.77519998885244112418746453840867616236209869384765625f,-1.91530007277876190840970593853853642940521240234375f,0.7004000849932443539813675670302473008632659912109375f,0.7412999757274978396281994719174690544605255126953125f,-1.5410000444166362942866044249967671930789947509765625f,-0.07479996953606615761600551195442676544189453125f,-0.38440001148208968739794499924755655229091644287109375f,0.227300004237145081287252423862810246646404266357421875f,-1.0041000289484853791321938842884264886379241943359375f,-1.67930004717484226972601391025818884372711181640625f,0.2152000120237473890227875017444603145122528076171875f,1.704600032893568428704611505963839590549468994140625f,-0.40090001717060752550736424382193945348262786865234375f,0.71310004555210515508889557168004103004932403564453125f,-0.46490001825839299964826523137162439525127410888671875f,0.9973000284887856281557105830870568752288818359375f,-0.8989000436350711442656802319106645882129669189453125f,-0.0534999581389120903995859634960652329027652740478515625f,1.7890000213906167569888339130557142198085784912109375f,-0.0486999983631075661349996153148822486400604248046875f,0.68930005438327779376805892752599902451038360595703125f,0.60030000101104408205543450094410218298435211181640625f,-0.88629998427703970964586233094451017677783966064453125f,-1.3025000254996121906714279248262755572795867919921875f,0.9585000314898792783679937201668508350849151611328125f,-0.1071000009119498219689603502047248184680938720703125f,-0.339399999244511307860960869220434688031673431396484375f,-0.337300028246641347973167057716636918485164642333984375f,0.340100020037591821431277594456332735717296600341796875f,1.112099996646493504925956585793755948543548583984375f,-1.35320001358464292451344590517692267894744873046875f,1.122600049337000083227167124277912080287933349609375f,0.89829997625276380990300140183535404503345489501953125f,-0.3081000068835917371501409434131346642971038818359375f,-0.71680004311427492069697109400294721126556396484375f,0.95580003365948851268285579863004386425018310546875f,-0.48200002455338886164781797560863196849822998046875f,-0.81019998918771796159177256413386203348636627197265625f,-1.297700030016898598006491738487966358661651611328125f,-0.56569999577775542665136754294508136808872222900390625f,1.202899995474516980920043351943604648113250732421875f,-0.28439999686032557679737919897888787090778350830078125f,1.53810009619742782405182879301719367504119873046875f,-0.82430004706308401818404263394768349826335906982421875f,-1.2650000046379858975598153847386129200458526611328125f,1.4312000228568904081072332701296545565128326416015625f,0.2773000093035415947184674223535694181919097900390625f,-1.8346000124789778595868483535014092922210693359375f,0.93980001967847304200631697312928736209869384765625f,1.2447999887585632361464149653329513967037200927734375f,-1.2574999962747097637105753165087662637233734130859375f,0.5667000211954114075041388787212781608104705810546875f,1.6638999931387601360910366565803997218608856201171875f,-1.2607000041969111947537385276518762111663818359375f,0.038200013845413660629191099360468797385692596435546875f,-0.30739999685659979977714328924776054918766021728515625f,0.210399966286867690090645055533968843519687652587890625f,-1.19130006910190022750839489162899553775787353515625f,1.063200028932839824591383148799650371074676513671875f,0.0569999974146488119775000313893542625010013580322265625f,-0.73500002507120443251409369622706435620784759521484375f,0.6331000028416517988461009736056439578533172607421875f,-0.339300014206022293450359939015470445156097412109375f,0.551999995868653225983280208311043679714202880859375f,1.8920000722929846137532194916275329887866973876953125f,0.628399976434557583360174248809926211833953857421875f,-1.403600062684715599203855163068510591983795166015625f,-0.037599963580816686548047300675534643232822418212890625f,0.57639999811202258239717366450349800288677215576171875f,0.2441000038400293892237868931260891258716583251953125f,-1.1293999854423104611811368158669210970401763916015625f,1.99070003139153062221566869993694126605987548828125f,0.63919995894581049800109440184314735233783721923828125f,0.410700024536996721735704340972006320953369140625f,0.00479996270015836667965913875377736985683441162109375f,0.373600040500610852678420314987306483089923858642578125f,1.1060999928168950834361794477445073425769805908203125f,0.138000014487653810402179033189895562827587127685546875f,-0.0144999795705087475283789899549447000026702880859375f,-1.1989000073321154360428408836014568805694580078125f,1.48420005075559036100685261772014200687408447265625f,0.7539000000491735686836136665078811347484588623046875f,-0.46749997314065694364870751087437383830547332763671875f,-0.23710000629499605739880507826455868780612945556640625f,0.76760001895725749054832931506098248064517974853515625f,-1.15290001149326570839548367075622081756591796875f,0.387300032065064581132673993124626576900482177734375f,0.318699958204477662437881235746317543089389801025390625f,-0.44660002326741843337032378258300013840198516845703125f,-0.8111999713733784034275231533683836460113525390625f,-0.39229998000413146552745047301868908107280731201171875f,-0.8801000339888032097945824716589413583278656005859375f,1.3184000203743568846448397380299866199493408203125f,0.0902999804474415856958557924372144043445587158203125f,0.48790006498843452309444046477437950670719146728515625f,0.193399992397427500367967923011747188866138458251953125f,0.037400023920833636026372914784587919712066650390625f,0.345200032848119897632699348832829855382442474365234375f,0.538799994819612049212764759431593120098114013671875f,-0.2436000033594674096804055807297118008136749267578125f,-0.8244000342249877544276159824221394956111907958984375f,0.34480005339235153538623990243650041520595550537109375f,-0.551600038187206820339270052500069141387939453125f,-0.30739999445378718423427244488266296684741973876953125f,0.42480001140832823747217616983107291162014007568359375f,2.259400041172653583743112903903238475322723388671875f,0.1728000151112674409592528945722733624279499053955078125f,-1.737900058208405429382992224418558180332183837890625f,1.6863000681519515477901904887403361499309539794921875f,0.89979999738261018382701195150730200111865997314453125f,-0.7579000125735999571219281278899870812892913818359375f,-0.421300037008524530524056217473116703331470489501953125f,0.67600003793463148138442875278997235000133514404296875f,0.32230001669824115406726150467875413596630096435546875f,-0.72370002563223245761747648430173285305500030517578125f,-0.468200010734796112643607557401992380619049072265625f,-0.87930002398490925141771867856732569634914398193359375f,-0.25839998775944084119515764541574753820896148681640625f,-0.7988000359654423565558545305975712835788726806640625f,-1.110399993667751772363772033713757991790771484375f,-0.429000024184583994557584674112149514257907867431640625f,0.8612000592529780451656051809550262987613677978515625f,-0.39400003543496164848392027124646119773387908935546875f,0.3326000544779013079477181236143223941326141357421875f,0.89620001782774927345798232636298052966594696044921875f,-0.894599991300701891105973118101246654987335205078125f,1.01909999629631631279380599153228104114532470703125f,-0.4224000547476121081302835591486655175685882568359375f,0.1157000532589862462629071160336025059223175048828125f,0.3764000026196239190312553546391427516937255859375f,-1.013300024085491912728684837929904460906982421875f,-0.45560002873241944865156938249128870666027069091796875f,0.259600024902821024941346195191727019846439361572265625f,0.58429998082742018450375098836957477033138275146484375f,-0.58900001315772498866607520540128462016582489013671875f,1.186600034871696518479211590602062642574310302734375f,0.09480001166910008436605750148373772390186786651611328125f,-0.2562999870337552277277382017928175628185272216796875f,1.2002999642983080974545373464934527873992919921875f,-0.850200050003082186123037899960763752460479736328125f,-0.27000000335276130858375154275563545525074005126953125f,-0.0712000174924730944159279033556231297552585601806640625f,-0.9324999970383938130424894552561454474925994873046875f,-0.415899966115504338137753848059219308197498321533203125f,0.36489998745024132542624784036888740956783294677734375f,-0.3821000007256858399529164671548642218112945556640625f,1.48740001498013629799288537469692528247833251953125f,-0.6201000267244871633209868377889506518840789794921875f,-0.70199994377046792681795750468154437839984893798828125f,-1.19640010974034805002474968205206096172332763671875f,1.10810006693527274279631456010974943637847900390625f,-1.268000065561384559487123624421656131744384765625f,-0.4932000135660172812634982619783841073513031005859375f,-0.349899949564038248439601375139318406581878662109375f,1.7298000253222880484571533088455908000469207763671875f,1.0012000054225336231894516458851285278797149658203125f,-1.2737000122211872277233624117798171937465667724609375f,-0.062799986799061680020628273268812336027622222900390625f,-0.189500019375234918239669923423207364976406097412109375f,-1.4341000160589814083067494721035473048686981201171875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.3027000180587167132983950068592093884944915771484375f,0.326299991969764224819527953513897955417633056640625f,1.0128999735325583397838045129901729524135589599609375f,0.2730000023245813256522751544252969324588775634765625f,-0.1737000240489845470648333503049798309803009033203125f,0.016300035667419610074801994414883665740489959716796875f,-0.86920003222227182337888962138094939291477203369140625f,0.3532000048115853108043893371359445154666900634765625f,-1.1097000222533937030533479628502391278743743896484375f,1.668800030563771574776410488993860781192779541015625f,0.42840002998560666735983204489457421004772186279296875f,-0.923800011378526964023194523178972303867340087890625f,-1.0420000441670413948713758145458996295928955078125f,-1.4133000074893227182570853983634151518344879150390625f,0.02899997359514216999087921067257411777973175048828125f,0.38630001197457308759197758263326250016689300537109375f,0.2889000082634389787727968723629601299762725830078125f,-0.6636999994620682574719694457598961889743804931640625f,0.06220001316368506394383075530640780925750732421875f,-2.44530002297162951663267449475824832916259765625f,0.38849999801814616429140869513503275811672210693359375f,-0.11180000621527474979899352547363378107547760009765625f,-0.728900017893314267070081768906675279140472412109375f,-0.4562000070802867401908997635473497211933135986328125f,0.3863000308245425618025592484627850353717803955078125f,-1.916100006432830848979165239143185317516326904296875f,0.5399999800696966278934496585861779749393463134765625f,-0.667700042235851665139989563613198697566986083984375f,0.19539997710883671988568721644696779549121856689453125f,-1.705499988876282291272445945651270449161529541015625f,0.1056000041455027460557403173879720270633697509765625f,-0.466500018782913861148387013599858619272708892822265625f,-0.2140000044032925874404327259981073439121246337890625f,-0.259099982643127224601897751199430786073207855224609375f,1.613600011573731674019427373423241078853607177734375f,-0.41970001022070635432470453451969660818576812744140625f,0.004400029382109205133843943258398212492465972900390625f,2.117999993272125180965304025448858737945556640625f,-0.30769998192340064502303675908478908240795135498046875f,-0.01389998969286676544498959628981538116931915283203125f,-0.63690001731216938640756097811390645802021026611328125f,-0.027099979901313353547465112569625489413738250732421875f,0.647299983729421324341046783956699073314666748046875f,-1.8269000179827210583738406057818792760372161865234375f,-0.33199999942630553650957381250918842852115631103515625f,0.60010002517849192482657372238463722169399261474609375f,0.39559999352842567521548744480242021381855010986328125f,1.11440001112818709572138686780817806720733642578125f,-0.770299994808435695148318700375966727733612060546875f,1.5127000399261711915954720097943209111690521240234375f,-1.408800075006485741369033348746597766876220703125f,0.200600031638145548384954963694326579570770263671875f,0.259100005926191645500011873082257807254791259765625f,-1.946999993205070556001601289608515799045562744140625f,-0.192600001411140198115390376187860965728759765625f,-0.00819999009668792577798512866138480603694915771484375f,-0.437000010751187772939374553970992565155029296875f,0.1870000349842016740264938334803446196019649505615234375f,-0.8947999987423418399856700489181093871593475341796875f,-0.57590002098903081417802241048775613307952880859375f,0.00250001493841367494752603306551463901996612548828125f,0.2473999878600234847425554107758216559886932373046875f,-0.390999983564018993575928107020445168018341064453125f,-0.2904000429347159961679381012800149619579315185546875f,-2.01030000527650098973708736593835055828094482421875f,1.144500032059848404486501749488525092601776123046875f,-0.542900000242889202439755536033771932125091552734375f,1.3235999942883847069197145174257457256317138671875f,-0.87330000603646007473201962056919001042842864990234375f,-0.134200013744831092576959008511039428412914276123046875f,-0.185399982175230793046694088843651115894317626953125f,1.38139999377205935360279909218661487102508544921875f,-0.10160000980049377350411532461293973028659820556640625f,-0.23009996885210315387126911446102894842624664306640625f,0.69199997450411310229156924833660013973712921142578125f,0.72040004446208516508676211742567829787731170654296875f,0.8921000179365272675369169519399292767047882080078125f,0.44330002361983067427075866362429223954677581787109375f,0.42799997423589208889183055362082086503505706787109375f,-1.29650004417076747387227442231960594654083251953125f,0.6111000094503165058767990558408200740814208984375f,-1.700000024735927706132088133017532527446746826171875f,1.576500036329031129156419410719536244869232177734375f,-1.319400014033913492994543048553168773651123046875f,-0.61160003448054200969608018567669205367565155029296875f,-1.0094999850094319437943113371147774159908294677734375f,-1.03979999805316314365200014435686171054840087890625f,0.67350002331286684675859532944741658866405487060546875f,-0.7906999909162519823979664579383097589015960693359375f,0.47519997337386055136221330030821263790130615234375f,-1.6686000081464651412233024529996328055858612060546875f,-0.06500000905245517657249365583993494510650634765625f,-1.5164000316396359924198122826055623590946197509765625f,0.900800035950541921891954189050011336803436279296875f,0.0664999756813047238068747901706956326961517333984375f,0.8531999915122987498961037999833934009075164794921875f,0.57099998240917926306536855918238870799541473388671875f,0.075800021421909169472286293967044912278652191162109375f,0.2867000011160969474843795978813432157039642333984375f,-0.3191000347599388664576736118760891258716583251953125f,-0.07939999078810189292454424503375776112079620361328125f,-0.09599998714029733815067402247223071753978729248046875f,-0.246099982888996038354889606125652790069580078125f,0.173100061663985560489464887723443098366260528564453125f,0.2186999531194562251812385511584579944610595703125f,0.38239999996721696362556031090207397937774658203125f,0.060599992895126408853911925689317286014556884765625f,2.15740001207441078889814889407716691493988037109375f,-0.295500013351440771192102374698151834309101104736328125f,-0.06769994182065153243144095540628768503665924072265625f,-0.050100021266937966313292918130173347890377044677734375f,-1.24250000903382851191736335749737918376922607421875f,-1.11540003038048762817879833164624869823455810546875f,1.79609999551773125148201870615594089031219482421875f,-1.1766999922871586026218437837087549269199371337890625f,0.4673999962791797191385967380483634769916534423828125f,1.0427000562801957972425270781968720257282257080078125f,-2.242800087679923226602340946556068956851959228515625f,-0.00869994876086632729084158199839293956756591796875f,1.0855000505298380009122638512053526937961578369140625f,-0.84790000689253253174371138811693526804447174072265625f,0.3006000000476840572360970327281393110752105712890625f,-0.631699989157915364756945564295165240764617919921875f,0.39740002030730214865883453967398963868618011474609375f,-0.1628999566569915291580628036172129213809967041015625f,-0.053799991895258525431700036278925836086273193359375f,-0.0333999971657987781981091757188551127910614013671875f,0.981100021147728540427124244160950183868408203125f,0.08339997794330089586622989372699521481990814208984375f,1.1839000000491737285557292125304229557514190673828125f,0.9309999944046138953268609839142300188541412353515625f,0.08829999787807452094057225622236728668212890625f,0.00440003854632353696985092028626240789890289306640625f,0.664799978998303675581382776726968586444854736328125f,-0.7051999980539080947750107952742837369441986083984375f,-0.56040003581941133514732200637809000909328460693359375f,-0.06939996918141810855473750052624382078647613525390625f,-0.065300015595555704894792370396316982805728912353515625f,-0.1742999846830960153365452924845158122479915618896484375f,-0.50129998966008415539619136325200088322162628173828125f,-0.6487000121280546149904466801672242581844329833984375f,1.25630002613067670580448975670151412487030029296875f,-1.5805000643506652568959225391154177486896514892578125f,-1.173900005301832916160265085636638104915618896484375f,-0.94419997954666590178618434947566129267215728759765625f,-0.3635999676339327635332665522582828998565673828125f,0.4845000137686727992303303835797123610973358154296875f,0.7451000178396700590610635117627680301666259765625f,0.77189999967291900162535966956056654453277587890625f,1.5614000234439970871136438290704973042011260986328125f,-0.765600027205049826761751319281756877899169921875f,-0.1324999856948851917337606209912337362766265869140625f,-0.6510000255107881717009377098293043673038482666015625f,-0.02629997397661210101205142564140260219573974609375f,-1.0735999747119844283105294380220584571361541748046875f,0.4125999850198620588770381800713948905467987060546875f,-0.20000001264736033679270121865556575357913970947265625f,0.3280000261664393068627987304353155195713043212890625f,0.6855000061988831827619605974177829921245574951171875f,-1.4156999962806702253459434359683655202388763427734375f,0.00359994619488623879277611194993369281291961669921875f,-0.5192999999567866620253653309191577136516571044921875f,0.78150003789365329875948873450397513806819915771484375f,0.91770002726018429228815875831060111522674560546875f,0.2221999929353593028480418070103041827678680419921875f,0.13310001610368538127460169562255032360553741455078125f,-0.45550002288818391971148002994596026837825775146484375f,-0.080599985742569157931569634456536732614040374755859375f,-1.0275999911293387523159026386565528810024261474609375f,-1.364300031509995125844625363242812454700469970703125f,-1.538200027908385170150040721637196838855743408203125f,2.876300044794380728063742935773916542530059814453125f,-0.81420001147240395056314810062758624553680419921875f,0.9432999636240302354650566485361196100711822509765625f,-0.731700014005600696265219085034914314746856689453125f,-0.674500032208860123006388676003552973270416259765625f,1.013700087453425435768394891056232154369354248046875f,-0.2270000200644142207551112733199261128902435302734375f,-0.5621999916315074674599827631027437746524810791015625f,-0.9568999557517468002032501317444257438182830810546875f,1.624699986639619009309853936429135501384735107421875f,0.23040002961680350335171851838822476565837860107421875f,0.21839995216429219926368432425078935921192169189453125f,0.5150999919861558584699423590791411697864532470703125f,0.07949997483938908704459436194156296551227569580078125f,-0.2243999885901806212729070466593839228153228759765625f,-0.0449000359162690099168457891209982335567474365234375f,0.34190000162869704780632673646323382854461669921875f,0.040199997122586150766210266738198697566986083984375f,0.7411999980092047746182970513473264873027801513671875f,-0.9905000021383170150102159823291003704071044921875f,-0.3323999697923658569465032996959052979946136474609375f,-0.69000007197260859737042437700438313186168670654296875f,-1.516699992585181622217760377679951488971710205078125f,0.0834999548420309878338230191729962825775146484375f,0.1858999712750317723219950494240038096904754638671875f,-0.45260003024488637546340896733454428613185882568359375f,0.7021000086233015746728369776974432170391082763671875f,0.54529995546936926853476279575261287391185760498046875f,-0.2034000148609276248379273965838365256786346435546875f,-0.71669992672204918537914863918558694422245025634765625f,-0.278299978022277960665320506450370885431766510009765625f,-0.43369999763667588243976069861673749983310699462890625f,0.062100002551079003243472698159166611731052398681640625f,1.26559998330250333964386300067417323589324951171875f,0.078799984910339071486617967821075581014156341552734375f,-1.14040001041293148631439180462621152400970458984375f,0.51440002655088845617825654699117876589298248291015625f,-1.66589993446916384556288903695531189441680908203125f,0.27370002956241357150446447121794335544109344482421875f,-0.9499000317811969740233735137735493481159210205078125f,0.48360004782080689977874499163590371608734130859375f,0.5676999860584734935997630600468255579471588134765625f,-0.6525000048056239787541699115536175668239593505859375f,-0.7866999363332976002993746078573167324066162109375f,0.469399975626169752018768122070468962192535400390625f,0.1853999882474539617760456167161464691162109375f,-0.35510003389567135201332348515279591083526611328125f,-0.163299990800023664405671297572553157806396484375f,0.988000001206993605507022948586381971836090087890625f,0.590899995116890064394965520477853715419769287109375f,-0.315999996155499562888735454180277884006500244140625f,0.45080004675388318791107167271547950804233551025390625f,-0.354500042639673385025389507063664495944976806640625f,0.5110999931707980170614291637321002781391143798828125f,1.5930000131651758099593507722602225840091705322265625f,-0.55710005187392308556582065648399293422698974609375f,0.9861000193595887974851166291045956313610076904296875f,0.30699998612701928646373517040046863257884979248046875f,-0.329999992474913295126270895707421004772186279296875f,-0.130199990044534530397868365980684757232666015625f,-1.245100006663798719586111474200151860713958740234375f,1.2260999995410439833420923605444841086864471435546875f,-0.0660000242814426929527371612493880093097686767578125f,1.473300024662912210970944215659983456134796142578125f,0.1666999981358647797691219238913618028163909912109375f,1.21310003689080492250695897382684051990509033203125f,0.782100003221631201455465998151339590549468994140625f,-1.367999988690018486892086002626456320285797119140625f,1.3622000426679845563882054193527437746524810791015625f,0.4557999829582868311916854509036056697368621826171875f,1.062700013066828486074655302218161523342132568359375f,0.00270000509470713634385674595250748097896575927734375f,-0.6170999851539729075255991119774989783763885498046875f,1.1848000249311325671186523322830908000469207763671875f,-0.8515999896839259175607139695785008370876312255859375f,1.011400029367207853425725261331535875797271728515625f,-0.081099997901916298648217207301058806478977203369140625f,-1.605000010281801170464177630492486059665679931640625f,1.010499961867928231384894388611428439617156982421875f,-1.6466999802172181954773577672312967479228973388671875f,1.939500027589499353553037508390843868255615234375f,0.3112000213295222916798365986323915421962738037109375f,-0.97210002110302440314626437611877918243408203125f,-0.3035000204443931703934822508017532527446746826171875f,-0.214600001806020568295707562356255948543548583984375f,-0.1657000369235872572204470998258329927921295166015625f,-0.8241000331580641269368925350136123597621917724609375f,0.6063000110432508193980538635514676570892333984375f,0.42209997908174934355685081754927523434162139892578125f,0.996800018322467895615091038052923977375030517578125f,1.08810001677423695554125515627674758434295654296875f,-0.0749000131919983491712855538935400545597076416015625f,0.1419000541180377439332005451433360576629638671875f,-0.43630000791400647131013101898133754730224609375f,0.58820001393854604021527165969018824398517608642578125f,-0.22480001047700659544403833933756686747074127197265625f,-0.3473000039391214155415354980505071580410003662109375f,0.21080000763833550703196806352934800088405609130859375f,-0.241999974802136585960710135623230598866939544677734375f,-1.451000000551343038779350536060519516468048095703125f,-0.379699979189038250293464216156280599534511566162109375f,0.0492999712347990115546281231218017637729644775390625f,0.590200020778179901270732443663291633129119873046875f,-1.704800003659725060600749202421866357326507568359375f,0.62430004138201411478803493082523345947265625f,-0.2941000177353618738607110572047531604766845703125f,0.22969996365457756581918147276155650615692138671875f,0.20370002430975453489025994713301770389080047607421875f,-1.0322999870076774442395617370493710041046142578125f,-0.01699995267391163356052175004151649773120880126953125f,0.767700021970271873072988455533050000667572021484375f,-0.5075999968290332287068622463266365230083465576171875f,-0.55950004070252223886683395903673954308032989501953125f,-1.36210000169426148630691386642865836620330810546875f,0.6098000150963664989234302993281744420528411865234375f,-0.2217999969579281527298775245071738027036190032958984375f,1.2532000472053888007195610043709166347980499267578125f,-0.0454000219240786595520376067725010216236114501953125f,-0.65830001304000607120769927860237658023834228515625f,-1.1045000340342523426073739756247960031032562255859375f,0.5248000561490664406250061801983974874019622802734375f,-1.03740005994439155045938605326227843761444091796875f,-1.639699971477687423515590126044116914272308349609375f,-0.2226000203728679682768643033341504633426666259765625f,-0.6525000116601586963582803946337662637233734130859375f,-1.30029999972581844502883541281335055828094482421875f,1.246200026731193677420606036321260035037994384765625f,-0.73149998883157962836776277981698513031005859375f,1.3351000167220823389158113059238530695438385009765625f,0.36599998077005191365884684273623861372470855712890625f,-1.569400023049116299489469383843243122100830078125f,1.503200015875697292955237571732141077518463134765625f,-0.8225000087916851665426065665087662637233734130859375f,0.7472000117480754877163917626603506505489349365234375f,-1.1187999724119903621755156564177013933658599853515625f,-0.8906000074237585550918083754368126392364501953125f,-0.9332999677777287228508384941960684955120086669921875f,-0.3746000204324719806692201018449850380420684814453125f,0.851600036883354061245654520462267100811004638671875f,0.22239999259114184848584727660636417567729949951171875f,-1.6007000023156401358193079431657679378986358642578125f,-0.19809996931403883735356430406682193279266357421875f,-0.004299987700581764382690153070143423974514007568359375f,-0.853799988356232386621513796853832900524139404296875f,0.2531999851420518865552367060445249080657958984375f,-1.1126000183053310621517084655351936817169189453125f,-0.0778999802008268904529586507123894989490509033203125f,0.12259995313137750372334267012774944305419921875f,-0.9765999931484456997310417136759497225284576416015625f,1.9947999846607442631096773766330443322658538818359375f,-0.37609998914748443699096469572396017611026763916015625f,0.1449999850243328669563425137312151491641998291015625f,0.48319999311417360932097153636277653276920318603515625f,1.325199972572922657576555138803087174892425537109375f,-0.098300001156329752660667509189806878566741943359375f,1.353899984589219229746959172189235687255859375f,-0.847499963715673043651577245327644050121307373046875f,2.2531999769091601848458594758994877338409423828125f,-0.694999975450337093008101874147541821002960205078125f,0.7846999910727137095278749256976880133152008056640625f,0.7048000364795334871104159901733510196208953857421875f,1.69570000376850327228339665452949702739715576171875f,-0.26770002033114448369843785258126445114612579345703125f,0.2724000393435368483352476687286980450153350830078125f,-0.31390000787228355960678527480922639369964599609375f,-0.0107000207185750095817411420284770429134368896484375f,1.334600023896992393446225833031348884105682373046875f,0.0394000234588973707872128215967677533626556396484375f,0.1259000157922509632868468543165363371372222900390625f,-0.28230003051906737976395334044354967772960662841796875f,1.6718000072583552029215070433565415441989898681640625f,0.38849998732656221278602970414794981479644775390625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6776000159025195035411570643191225826740264892578125f,1.13380003387927974500826167059130966663360595703125f,1.275100009942054857248194821295328438282012939453125f,-2.13930002905130312029768901993520557880401611328125f,-1.9168999602407215121502304100431501865386962890625f,-0.1640999978050599050760638419887982308864593505859375f,-0.7148000289544460539303827317780815064907073974609375f,-0.1488000450551520970066121662966907024383544921875f,0.80470006300807117582962746382690966129302978515625f,-0.0221999283015719139910970625351183116436004638671875f,0.450300045993923259146640702965669333934783935546875f,-1.0365000494047993395696494189905934035778045654296875f,0.3914999564960590650031235782080329954624176025390625f,0.177899991674721480450216404278762638568878173828125f,-1.4336000065818428339525780756957828998565673828125f,-1.051200045804679295002870276221074163913726806640625f,1.25960003591105351716805671458132565021514892578125f,0.5134999887049207512035309264319948852062225341796875f,1.24799996842443849942583256051875650882720947265625f,1.14739999482631649385666605667211115360260009765625f,1.22010001940429280153921354212798178195953369140625f,1.842000028260051269768382553593255579471588134765625f,2.384700009773672579882486388669349253177642822265625f,-1.104900036307424837644930448732338845729827880859375f,-1.068499970078466532186212134547531604766845703125f,0.2365000342428675850214858655817806720733642578125f,-1.510700047540664225920181706896983087062835693359375f,-1.9112000552296652955419631325639784336090087890625f,-0.6798999902069564615914032401633448898792266845703125f,-1.001499965026973359982775946264155209064483642578125f,-0.0727999885499475141159564373083412647247314453125f,0.6668999743968233584467952823615632951259613037109375f,0.32269998501539287616424189764074981212615966796875f,0.03130003507137235629897986655123531818389892578125f,0.48780001805424699767854690435342490673065185546875f,0.46939995591938465935299973352812230587005615234375f,0.4345000119432802510033297949121333658695220947265625f,-0.4149000382259491193082112658885307610034942626953125f,-0.450100073458255867109301107120700180530548095703125f,0.358100084984303901336488706874661147594451904296875f,-0.045000056847931180215027779922820627689361572265625f,0.346900071552396216389979599625803530216217041015625f,-0.4687000296741736793393329207901842892169952392578125f,0.1446999966233970535967046089353971183300018310546875f,-0.3861999614641067335440993701922707259654998779296875f,0.3288999880909917461480063138878904283046722412109375f,-0.2539000070154668886601712074480019509792327880859375f,0.0611999850451951932228666919399984180927276611328125f,0.4558999984696507912218521596514619886875152587890625f,0.8772000081904229862317379229352809488773345947265625f,1.6195999750211818035694477657671086490154266357421875f,-0.95049996700882832811885236878879368305206298828125f,-1.58460001938939143428797251544892787933349609375f,-0.761299996030330827778698221663944423198699951171875f,0.232899971073866396409357548691332340240478515625f,2.06000003993511171529462444595992565155029296875f,0.880000021606683890951217108522541821002960205078125f,-0.1384999642297619626418736515915952622890472412109375f,0.34860004342496342388812990975566208362579345703125f,0.71559995912536911166768049952224828302860260009765625f,1.78720014924854098126161261461675167083740234375f,-1.320000077784062142427501385100185871124267578125f,-0.66649997642636282790817858767695724964141845703125f,-0.295000001341104933771930518560111522674560546875f,-1.6382999476611603828501984025933779776096343994140625f,-0.1603999302446847963210529997013509273529052734375f,0.573800052058698550894177969894371926784515380859375f,0.839900015091895379981679070624522864818572998046875f,-0.03180002152621597133475006558001041412353515625f,-0.487199974718689698960361056379042565822601318359375f,0.370899955815076509679784066975116729736328125f,-0.87309997613132039884220603198627941310405731201171875f,-0.762500048689543685753733370802365243434906005859375f,0.50350004875659870595683287319843657314777374267578125f,1.02639993761330838850653890403918921947479248046875f,-1.77680006086528319286799160181544721126556396484375f,-1.3304000529184942802629620928200893104076385498046875f,0.79879999636560616860236905267811380326747894287109375f,0.60379999383240934296424029525951482355594635009765625f,-3.783800096594543305172919644974172115325927734375f,-2.39720001353621459116993719362653791904449462890625f,-2.37900004638731576989130189758725464344024658203125f,-0.8494000503927470191456450265832245349884033203125f,1.100600025603174714206033968366682529449462890625f,-0.18549999951198670178342808867455460131168365478515625f,1.636700091212243801663817066582851111888885498046875f,0.4967000216983252780522661851136945188045501708984375f,0.86180002908855801724286038734135217964649200439453125f,0.60150004344433460889973730445490218698978424072265625f,-0.201400042983145066699535163934342563152313232421875f,-1.154800009471177446584988501854240894317626953125f,-1.0486000123560421659618668854818679392337799072265625f,-1.350000033006072186481105745770037174224853515625f,-1.2646000238597394638162541014025919139385223388671875f,-1.2744000255823138001431971133570186793804168701171875f,0.109600010970234418294921852066181600093841552734375f,-0.2034000168725846879880236883764155209064483642578125f,-0.9023999998182057336038042194559238851070404052734375f,0.4721000208795071984013702603988349437713623046875f,0.37839998874664271255596759147010743618011474609375f,1.942900041779875852654413392883725464344024658203125f,1.61110011889934678919189536827616393566131591796875f,-0.79140001163482642798641109038726426661014556884765625f,1.160900140589476325914120025117881596088409423828125f,1.3260999980136745080017135478556156158447265625f,1.7124000444471831894333035961608402431011199951171875f,0.1718999525666233818554928802768699824810028076171875f,-1.55329999758005055099374658311717212200164794921875f,-0.63800005779415425166689601610414683818817138671875f,-0.5023000340767200810887516126967966556549072265625f,4.1910000023953610792659674189053475856781005859375f,2.586300001152604721710304147563874721527099609375f,-0.040899982264638534701362004852853715419769287109375f,-0.365700048658251120059503591619431972503662109375f,1.89420014658868485213361054775305092334747314453125f,0.748799991261957842425545095466077327728271484375f,0.0856000305950654905018382123671472072601318359375f,-0.2474999549984939761770874611102044582366943359375f,0.1296000001668939916044109850190579891204833984375f,-0.4037000127613545874538658608798868954181671142578125f,0.06789997852444618064993164807674475014209747314453125f,0.64249991677701245862408541142940521240234375f,-1.8787000244215128486047206024522893130779266357421875f,-1.9144000418990867462554206213098950684070587158203125f,1.17750005133450041938658614526502788066864013671875f,1.92230002452135106949526743846945464611053466796875f,-0.0820000127255930966185815123026259243488311767578125f,-2.57100009107589588808195912861265242099761962890625f,-2.1296999429792169422626102459616959095001220703125f,-1.492600038775801607471294119022786617279052734375f,-1.1367000063687562327885416380013339221477508544921875f,-0.6841000907510526030108621853287331759929656982421875f,-0.3163999655529849963642163857002742588520050048828125f,1.2906000808119786338323820018558762967586517333984375f,0.54800001767277795039490229100920259952545166015625f,-0.2548999721765508041926295845769345760345458984375f,1.44290008588731399896687435102649033069610595703125f,-0.2711000106051566493903237642371095716953277587890625f,0.7332000254496933155934357273508794605731964111328125f,0.73809998876005433743330286233685910701751708984375f,1.481600019448995109172528827912174165248870849609375f,0.26609999402761541631434738519601523876190185546875f,-0.0864999390244485244494399012182839214801788330078125f,-0.881200012798607712483089926536194980144500732421875f,0.11600006846338584143296657202881760895252227783203125f,1.564400007235258183158066458418034017086029052734375f,1.187200003328918729295082812313921749591827392578125f,0.06380000187903600572525419920566491782665252685546875f,0.12929998909756523772784930770285427570343017578125f,0.2064999901354303002420920165604911744594573974609375f,-1.634400097331404566602941486053168773651123046875f,-1.54339994970559946096955172833986580371856689453125f,0.562799980428814183852637142990715801715850830078125f,-0.48699998754262896483169242856092751026153564453125f,-0.06280001501813547992725261792656965553760528564453125f,-0.16720001973882414603167489985935389995574951171875f,-1.0802000454582281196991289107245393097400665283203125f,-3.49469999971538669569781632162630558013916015625f,-1.2496000589892275467462923188577406108379364013671875f,2.388800036673247806362496703513897955417633056640625f,2.308700038912892882336791444686241447925567626953125f,1.8850000570341947092600776159088127315044403076171875f,1.7465000103637577932857993801007978618144989013671875f,-0.11659998968392581986108780256472527980804443359375f,-1.997200064423681364900176049559377133846282958984375f,-1.4589000265359874930481964838691055774688720703125f,1.051800018993019225632679081172682344913482666015625f,-0.528700036826729391492563081555999815464019775390625f,-0.535900045371056510390417315647937357425689697265625f,2.110199998724460623833465433563105762004852294921875f,-0.015599991330503382158667591284029185771942138671875f,-0.99490001166463049031563059543259441852569580078125f,-1.36259996330141763820620326441712677478790283203125f,0.656499995425342763866183304344303905963897705078125f,-0.319000071831047282699955758289434015750885009765625f,-0.574400072148443019415253729675896465778350830078125f,0.91960002050697775555221369359060190618038177490234375f,1.952000027179717989156415569595992565155029296875f,0.4506999647274607667668533395044505596160888671875f,0.29060006101205992923297571906005032360553741455078125f,0.57079998485818450593143325022538192570209503173828125f,-2.02190008531734388697032045456580817699432373046875f,0.11150002574920581110973216709680855274200439453125f,-0.6528999946176980984091642312705516815185546875f,-1.150100001075864408761617596610449254512786865234375f,1.101699984762073203370391638600267469882965087890625f,-0.43769996750652762074196289177052676677703857421875f,-0.85299997505545466225385098368860781192779541015625f,-0.4086999243974691609082583454437553882598876953125f,0.5593000637710099365307314656092785298824310302734375f,-0.538000000238417985798378140316344797611236572265625f,0.1270000750124455901612918751197867095470428466796875f,0.060499996662141342085305950604379177093505859375f,-0.670299983930586762426173663698136806488037109375f,-0.46119996731281265311963579733856022357940673828125f,-0.04900000011920813136612196103669703006744384765625f,-1.61280000158846181790295304381288588047027587890625f,-1.254199977721274450459532090462744235992431640625f,0.174199988897144830701790851890109479427337646484375f,0.0586000007703899594702079411945305764675140380859375f,-1.1484999996200209171348660674993880093097686767578125f,-1.64919997552335217960717272944748401641845703125f,-0.49890004650354569104564461667905561625957489013671875f,0.10530003561899103914356601308099925518035888671875f,-1.09200001706555394065389918978326022624969482421875f,1.085799968206135890369523622212000191211700439453125f,0.19899996539950404184082799474708735942840576171875f,0.48579997992217460733854750287719070911407470703125f,0.2875000065192574449923768042935989797115325927734375f,2.09589999735206244935170616372488439083099365234375f,-1.261000031769274887238907467690296471118927001953125f,0.3296000155150882182653049312648363411426544189453125f,1.228900006568434122300459421239793300628662109375f,-0.000900054013730056112763122655451297760009765625f,0.8306000210583217580051496042869985103607177734375f,2.216000043690204535096199833787977695465087890625f,0.27929997659921479424838253180496394634246826171875f,0.22520002793073690128267116961069405078887939453125f,-0.7870000560879704121219901935546658933162689208984375f,0.1094000018149607900141973004792816936969757080078125f,0.367199965777994297155828462564386427402496337890625f,-0.1536000285238028340017990558408200740814208984375f,0.58210005396008579481303968350403010845184326171875f,2.10190001507699353311409140587784349918365478515625f,-1.26650006061792463896154004032723605632781982421875f,-0.6940999702751617927987126677180640399456024169921875f,-0.53090004574358573563586105592548847198486328125f,-1.71410007085800142334619522443972527980804443359375f,-0.49230002832114649891082081012427806854248046875f,-0.45650002649426557610468080383725464344024658203125f,-0.595700046944617866984117426909506320953369140625f,-0.3441999884128559283880122166010551154613494873046875f,0.2735999882161603835584173793904483318328857421875f,-1.593000038683413777107489295303821563720703125f,-0.17149998864531657005727538489736616611480712890625f,0.4550999486982829633774372268817387521266937255859375f,-0.245200026738645160406804279773496091365814208984375f,0.28780002371668889082911846344359219074249267578125f,-0.73340002983659502433511079289019107818603515625f,1.41210005764812240869332526926882565021514892578125f,2.254700028083472584938817817601375281810760498046875f,-2.08790010332167152995452852337621152400970458984375f,-0.9308000233590612015888154928688891232013702392578125f,0.0929999839216477663939031117479316890239715576171875f,0.6721999633938062768123700152500532567501068115234375f,2.553400034940242591119385906495153903961181640625f,-0.23379999923408067985519664944149553775787353515625f,-1.048100028471650180250662742764689028263092041015625f,-0.9924999115243544611786319364910013973712921142578125f,-0.453900050713123004442195451701991260051727294921875f,-1.5500000238046045542006368123111315071582794189453125f,-0.4285000578835618245676641890895552933216094970703125f,0.3309000168353317672398361537489108741283416748046875f,1.7599000337183483555492102823336608707904815673828125f,0.75900003342330435440743485742132179439067840576171875f,-1.423900048738718115970414146431721746921539306640625f,-0.073799937319754871367649684543721377849578857421875f,0.024000070825220820580625513684935867786407470703125f,-2.0429000642061243553371241432614624500274658203125f,-1.0077000588133941949564587048371322453022003173828125f,-2.285100135745109728446777808130718767642974853515625f,-1.579800028395653921364782945602200925350189208984375f,-3.20900011187791900368893038830719888210296630859375f,-1.414899994975329367008498593349941074848175048828125f,1.60210008331537334669292249600403010845184326171875f,0.93949999499321013018970916164107620716094970703125f,2.805400052266568078351838266826234757900238037109375f,1.4828999852858479879813558000023476779460906982421875f,-0.6622000194221724367338310912600718438625335693359375f,0.901500148013233637556140820379368960857391357421875f,0.197400003916026189898502707364968955516815185546875f,-0.34679997056424749501957194297574460506439208984375f,0.02969997486770292738356147310696542263031005859375f,-0.1440999950855970102026049062260426580905914306640625f,-0.7529999848529682626718795290798880159854888916015625f,-0.1855000613331814207640491076745092868804931640625f,2.41520001004934226074283287744037806987762451171875f,-0.576100076133013505597091352683492004871368408203125f,0.1013000352203843323906085061025805771350860595703125f,0.3266000582665207208066249222611077129840850830078125f,0.191299959820508291841179016046226024627685546875f,0.45750002540647916049465493415482342243194580078125f,1.02700004416704171461560690659098327159881591796875f,0.46849998646974455596136976964771747589111328125f,1.3224000192642215711913422637735493481159210205078125f,-0.040699979554115639501787882181815803050994873046875f,0.22420002646669712209614999665063805878162384033203125f,-1.09830000160336371806124589056707918643951416015625f,-1.75890000269413082634173406404443085193634033203125f,-0.51670003445744416836760137812234461307525634765625f,0.694699989247321791907552324119023978710174560546875f,0.0028999935746190086405249530798755586147308349609375f,2.460400071507693464667454463778994977474212646484375f,1.1198000121533855377720101387239992618560791015625f,-1.21770001392364424219749707845039665699005126953125f,-0.5922000124186286296179559940355829894542694091796875f,-1.65500009879469889284564487752504646778106689453125f,1.22630007664561357927368590026162564754486083984375f,1.081300003033876411251412719138897955417633056640625f,0.42340005878210096312841415056027472019195556640625f,0.4139999265074720824486576020717620849609375f,-1.1877000405222180035025303368456661701202392578125f,1.06770000867098513452901897835545241832733154296875f,1.7390000494420529975059253047220408916473388671875f,2.0898000427007676904622712754644453525543212890625f,3.37910010066032473474706421256996691226959228515625f,0.029199991393089153035589333740063011646270751953125f,-0.854700030374527131016293424181640148162841796875f,1.102500004172325187568048932007513940334320068359375f,-0.3624000016808508473076244627009145915508270263671875f,1.196500017181039599023506525554694235324859619140625f,1.762500009909270914931767038069665431976318359375f,0.274299948734045617726451382623054087162017822265625f,0.745700016918777297547649141051806509494781494140625f,1.079800012898445071840569653431884944438934326171875f,-0.2929000371605161578969500624225474894046783447265625f,0.328299991805850732706772987512522377073764801025390625f,-2.232500010021029712703466429957188665866851806640625f,0.1508000465303653836457442594110034406185150146484375f,-0.1664999752342690708672989785554818809032440185546875f,0.1049000461608173839778146430035121738910675048828125f,-0.1804000089600676925982725151698105037212371826171875f,-0.25969999401569321406668677809648215770721435546875f,-1.5872000154733658661143635981716215610504150390625f,-1.25590005371570523351465453743003308773040771484375f,-2.22999998211860628316571819595992565155029296875f,-0.384300016981362180246151183382607996463775634765625f,-0.992499946057796567089326345012523233890533447265625f,-0.703700013431906246097469193045981228351593017578125f,-1.89680005483031255408832294051535427570343017578125f,-1.67859996765256003214972224668599665164947509765625f,0.454499969810246984280865945038385689258575439453125f,0.32580002484917702787470261682756245136260986328125f,-1.00050005492567972709139212383888661861419677734375f,0.5829000329136846403343952260911464691162109375f,0.65990001434683964731675587245263159275054931640625f,0.95160001013576966943929846820537932217121124267578125f,0.128199990022181964377523399889469146728515625f,-0.741100038358568635743495178758166730403900146484375f,1.58279999957680583833052878617309033870697021484375f,1.2666999677002446222928710994892753660678863525390625f,-1.4554000291883948392523961956612765789031982421875f,-2.607800056946278033365160808898508548736572265625f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,-0.6655999950930475694832466615480370819568634033203125f,1.300499960936605692296552660991437733173370361328125f,0.255099991799891068211536548915319144725799560546875f,-1.0417000155329707578033548998064361512660980224609375f,-0.0529999771416189702222254709340631961822509765625f,-0.351000016607344089702991141166421584784984588623046875f,0.063499994255602043580211102380417287349700927734375f,-1.315200079041720204742205169168300926685333251953125f,0.311000063657761227631226574885658919811248779296875f,-0.515699983242154047502481262199580669403076171875f,-0.56240006411671661368245622725225985050201416015625f,0.7838999902889127735505780947278253734111785888671875f,0.856299962949752302421302374568767845630645751953125f,-0.2948999846190212270613528744434006512165069580078125f,-0.4877999875999983192542686083470471203327178955078125f,-0.12269997871965154789819507641368545591831207275390625f,2.21130006343945861857491763657890260219573974609375f,-1.3132999982506030622886328274034895002841949462890625f,0.3352000053554766711982892957166768610477447509765625f,2.404400011835992501119108055718243122100830078125f,-1.9637000097438688239748216801672242581844329833984375f,1.49590003052577369402342810644768178462982177734375f,0.7637999950990079778279095990001223981380462646484375f,-1.509500040944665943953850728576071560382843017578125f,1.3099000267148019904794864487485028803348541259765625f,2.60500003214925524019918157136999070644378662109375f,-1.780300024275482062563469298765994608402252197265625f,0.24509999422132988655675944755785167217254638671875f,-0.9100000187009571472884772447287105023860931396484375f,0.791100009636581003036326364963315427303314208984375f,-0.420100010016561220549391464373911730945110321044921875f,-0.76620002121776387138396557929809205234050750732421875f,-1.2265000090226523266068170414655469357967376708984375f,-0.38279998393058745076444893129519186913967132568359375f,-0.614900013452768678945403735269792377948760986328125f,0.3060000041276216276031618690467439591884613037109375f,0.542900014846026568449133264948613941669464111328125f,-2.369700011375546200298458643374033272266387939453125f,1.0731999772071840748566273759934119880199432373046875f,0.7020000503510239742155363273923285305500030517578125f,-0.32260001634955426386142107730847783386707305908203125f,0.815300013360381381488650731625966727733612060546875f,-0.7968000144481661717321685500792227685451507568359375f,0.561599992105365597439003977342508733272552490234375f,-0.0456999716937540600980582894408144056797027587890625f,-1.026400067514182001815470357541926205158233642578125f,-0.77089999265819775242647438062704168260097503662109375f,-1.467000009596347798179749588598497211933135986328125f,0.94960000504702346546537228277884423732757568359375f,0.364599998024850879829728000913746654987335205078125f,1.879200031868368370169264380820095539093017578125f,-1.2501999795764675571518864671816118061542510986328125f,-0.74480003256797788768750478993752039968967437744140625f,0.4174999512359491848911829947610385715961456298828125f,0.03190002729594720864980672558885999023914337158203125f,0.323900003141165182540817113476805388927459716796875f,0.11950000927969817166029997679288499057292938232421875f,-0.53870003475919359114953977041295729577541351318359375f,-0.64580001734271685887023295435938052833080291748046875f,1.768500040747225110493445754400454461574554443359375f,0.762800032713264908323935742373578250408172607421875f,-0.68229998778998801611805902211926877498626708984375f,0.7408999421060078471867882399237714707851409912109375f,-1.027299985480308208707356243394315242767333984375f,-0.5243000324785709320707383085391484200954437255859375f,-0.554999991767108458162738315877504646778106689453125f,0.9321999702483412253428696203627623617649078369140625f,-0.6167000066295267313165595624013803899288177490234375f,-0.217400013899803123962328754714690148830413818359375f,-0.02919998878538621056577540002763271331787109375f,0.56769997875690403343895695797982625663280487060546875f,-1.44849996620416643366979769780300557613372802734375f,-0.1888999918349079099044729446177370846271514892578125f,1.7511999838344747271179357994697056710720062255859375f,0.06430003279522089609798740639234893023967742919921875f,-0.736100006879865986775257624685764312744140625f,-0.416600008124113407870225955775822512805461883544921875f,1.780200016251951122825403217575512826442718505859375f,0.5161000101581219023927360467496328055858612060546875f,-0.0407999999083579201197835573111660778522491455078125f,0.0195999979689720493780669130501337349414825439453125f,0.51789999163746802679497704957611858844757080078125f,-1.8376000397071248215752348187379539012908935546875f,0.6602000116139652430291562268394045531749725341796875f,0.1552000356048350848681138813844881951808929443359375f,1.1733999735288318522208328431588597595691680908203125f,-1.108900001297146165057938560494221746921539306640625f,-0.471499990005046054619697315501980483531951904296875f,1.890899961440264842593705907347612082958221435546875f,-1.6011999954573805649005180384847335517406463623046875f,0.28150002485513692107588212820701301097869873046875f,0.623599989371002028093471380998380482196807861328125f,1.135400022110342632686297292821109294891357421875f,-1.3355000035166739724701301383902318775653839111328125f,-0.14680001664608732792061118743731640279293060302734375f,0.20719998883753998342172053526155650615692138671875f,-0.6589999957978724953733262736932374536991119384765625f,-0.641400002992153162040267488919198513031005859375f,-0.32060000343769801123272600307245738804340362548828125f,-0.5216000348344447257176170751336030662059783935546875f,0.90929997954219532463326913784840144217014312744140625f,-1.390700019191205338842109995312057435512542724609375f,-1.7490000368505718153500083644757978618144989013671875f,1.95450002168491465681654517538845539093017578125f,-0.04309998576119522528671268446487374603748321533203125f,-0.1476999929875137951285068993456661701202392578125f,-0.4790000100657341164378522080369293689727783203125f,-1.58319999851584380934355067438445985317230224609375f,-0.0283999927513303163095770287327468395233154296875f,0.1438000188663604017591524097952060401439666748046875f,1.069500041577965188110965755186043679714202880859375f,0.51069998622238632624004139870521612465381622314453125f,0.38780000523924818001120229382649995386600494384765625f,-1.422799999651312763404575889580883085727691650390625f,1.9377000015184286940694846634869463741779327392578125f,-1.1265999649107463209674051540787331759929656982421875f,-1.3430000406578181948447081595077179372310638427734375f,0.2269000221550470097753304798970930278301239013671875f,0.183499996602535464518268781830556690692901611328125f,1.0800000309944157717012558350688777863979339599609375f,2.588400029017031034328510941122658550739288330078125f,-0.1973999869845803889489843641058541834354400634765625f,-1.3059999831356103872082030648016370832920074462890625f,-2.271800066322833711041084825410507619380950927734375f,1.212400031855703108618627084069885313510894775390625f,0.50179999302774624947431902910466305911540985107421875f,0.3683999770119783523369960676063783466815948486328125f,-0.8767999797284600216329408794990740716457366943359375f,0.91830001751035450485716182811302132904529571533203125f,1.0393000207066538376210473870742134749889373779296875f,-0.375400019279122698634409971418790519237518310546875f,-0.5167999672114851339443930555717088282108306884765625f,1.228100031377375334074031343334354460239410400390625f,-0.254300007705390296308678443892858922481536865234375f,-1.27880009985417242290850481367669999599456787109375f,1.032100082682072983430998647236265242099761962890625f,-0.9797000267237423454247391418903134763240814208984375f,-0.738100033836067037640304988599382340908050537109375f,0.29899999386072162455008083270513452589511871337890625f,0.21720001837909219677413830140721984207630157470703125f,0.93470000291913724055348211550153791904449462890625f,-0.51689999198019531689851646660827100276947021484375f,-0.035600017798692373816038525546900928020477294921875f,-0.43400000632181734427916808272129856050014495849609375f,0.683800010260939838957483516423963010311126708984375f,0.4562999708287411859686244497424922883510589599609375f,-0.224999986216425806873075998737476766109466552734375f,0.2765000462383035984004209240083582699298858642578125f,1.3953000314094128331277033794322051107883453369140625f,-0.533400046823918838612144099897705018520355224609375f,0.67010002115517863785498775541782379150390625f,0.4386000278532506957418490856071002781391143798828125f,0.3683000287979842912733374760136939585208892822265625f,0.1382999961644408859484656204585917294025421142578125f,0.6240999988108872909009505747235380113124847412109375f,0.317000041838735313604047405533492565155029296875f,-0.07560004121214125571981412576860748231410980224609375f,-0.9896999909237020887076141661964356899261474609375f,-0.0243000044643884027806279846117831766605377197265625f,1.894800028246641065976518802926875650882720947265625f,-0.89989994043707788318187112963641993701457977294921875f,1.2228000269204379879539601461146958172321319580078125f,-0.21620000493824498288830682213301770389080047607421875f,-0.563600031802058420993262188858352601528167724609375f,0.12209999934732895798816798560437746345996856689453125f,1.8307000633358965213659530490986071527004241943359375f,-0.30929998572617789509564545369357801973819732666015625f,-0.169199972766637340981787929194979369640350341796875f,0.00019994835853509584921994246542453765869140625f,1.121500071235001616543058844399638473987579345703125f,-1.54090004552006742954972651205025613307952880859375f,-2.02490001099407646023564666393212974071502685546875f,1.963100014278293059533098130486905574798583984375f,-0.18500002749264277124296995680197142064571380615234375f,0.2161000181488692550857422247645445168018341064453125f,-0.1381000155076386182173564520780928432941436767578125f,0.71870001659840365704212672426365315914154052734375f,-0.66040000448972036029005039381445385515689849853515625f,0.0697999883674091936569539029733277857303619384765625f,1.8248000174060463596248382600606419146060943603515625f,-0.450799985901266353494065697304904460906982421875f,0.18679995819628236741749560678726993501186370849609375f,0.04100001951307064018692472018301486968994140625f,0.17630001931339556175970528784091584384441375732421875f,-0.34230006415844071998577646809280849993228912353515625f,0.4410999925747505212569876675843261182308197021484375f,-0.70839997920989983271056189551018178462982177734375f,-0.195300053705274923032675360445864498615264892578125f,1.461500003322958374241125056869350373744964599609375f,-1.4647999813079828346218391743605025112628936767578125f,1.0680999892815943308477244499954394996166229248046875f,1.01829998919814812552431249059736728668212890625f,-0.97180000874847183300886399592855013906955718994140625f,2.133799980235099535974541140603832900524139404296875f,0.44850001277029438728760624144342727959156036376953125f,-1.6065000281333923215498771241982467472553253173828125f,-0.3955000063478941729755433698301203548908233642578125f,0.80639997553676234787189969210885465145111083984375f,0.8258000194847578967483059386722743511199951171875f,-0.246700010541081038439870098954997956752777099609375f,-0.61390002040788560488948633064865134656429290771484375f,-0.493600016509742001158400626081856898963451385498046875f,0.933599980504811188808389488258399069309234619140625f,0.59630002411901983361275370043586008250713348388671875f,-0.59419998982846766732990317905205301940441131591796875f,1.5195999854706221388056519572273828089237213134765625f,0.173200028206407896647078814567066729068756103515625f,-2.135199993695318543274197509163059294223785400390625f,1.193699980537593408058683053241111338138580322265625f,0.231599981711804847606117618852294981479644775390625f,-0.28660002026855935230997829421539790928363800048828125f,0.52749999117106138868393827578984200954437255859375f,-1.0942000181406736913913846365176141262054443359375f,0.25239997463524321830163898994214832782745361328125f,0.1450000449270014701852460348163731396198272705078125f,-1.01359997495412823553806447307579219341278076171875f,0.1779999750554563942017693989328108727931976318359375f,-0.43799997878074614643395534585579298436641693115234375f,0.15710000597834550273290687982807867228984832763671875f,-0.14090000837892302509857245240709744393825531005859375f,-1.330700041393935606492959777824580669403076171875f,0.3355000181570650941154099200502969324588775634765625f,-0.124299981665611625203382573090493679046630859375f,-0.9451000027522444835170745136565528810024261474609375f,-0.17469998887479309956916040391661226749420166015625f,1.27690002915859235343987165833823382854461669921875f,0.31109999872148019761652903980575501918792724609375f,-1.116699994894862157934767310507595539093017578125f,1.7168999977543950574698783384519629180431365966796875f,-0.735800035578013211789993874845094978809356689453125f,-1.23009996777176855431434887577779591083526611328125f,0.89029999834746131881502151372842490673065185546875f,-1.8462999892130496082387480782927013933658599853515625f,0.4075999971270560795488790972740389406681060791015625f,-0.88410002287626221484373445491655729711055755615234375f,-0.4324000098764901878212185692973434925079345703125f,0.00469999114721975086439442748087458312511444091796875f,-0.2356999767601484219170515643781982362270355224609375f,-0.18719999481663085472149532506591640412807464599609375f,0.56910005614310554822310450617806054651737213134765625f,-0.2288000076718628150729273329488933086395263671875f,-2.66640009165406244306950611644424498081207275390625f,0.832400030775368282576209821854718029499053955078125f,0.47360003508031367491781793432892300188541412353515625f,-0.7078000246852635957139909805846400558948516845703125f,2.2735000370591880169968135305680334568023681640625f,-2.758900036631524610442056655301712453365325927734375f,0.832300012861192595181591968866996467113494873046875f,1.419999997504055588848359548137523233890533447265625f,-0.56439999034106735908977725557633675634860992431640625f,0.2743999689951535092546919258893467485904693603515625f,-1.280399995325506079524302549543790519237518310546875f,-0.44490001322925110116557334549725055694580078125f,1.355000048652291955164628234342671930789947509765625f,0.732100022593140664639577153138816356658935546875f,-1.01360004715025464605560046038590371608734130859375f,1.3107000636711723107197258286760188639163970947265625f,-0.287500021457672172431330182007513940334320068359375f,-1.094500038206577929855711772688664495944976806640625f,-0.133599991233646964161607684218324720859527587890625f,1.375300007399916868422451443620957434177398681640625f,-0.604900011180341312666541853104718029499053955078125f,-2.051200009743869401290794485248625278472900390625f,0.764799987007677817274498011101968586444854736328125f,-0.00489998517781475850796368831652216613292694091796875f,-0.6506000188976528431084034309606067836284637451171875f,1.2827999593995504934440532451844774186611175537109375f,-1.443700037609041419983668674831278622150421142578125f,-0.74169999847114098656675196252763271331787109375f,0.7079000065013765219390506899799220263957977294921875f,0.11389996979981698022044156459742225706577301025390625f,0.1696999709188931060310778775601647794246673583984375f,0.065599996471405130904486213694326579570770263671875f,-0.5309000252172351164148267343989573419094085693359375f,-0.621899975290893802792879796470515429973602294921875f,1.091999996744095202672042432823218405246734619140625f,1.6652000208154327243192938112770207226276397705078125f,-1.990500011190771800784204970113933086395263671875f,0.147100004190206690157793900652904994785785675048828125f,1.054900023399293562675893554114736616611480712890625f,-0.4299000179231169394000744432560168206691741943359375f,-0.2530999617516993538401948171667754650115966796875f,0.45810001398027011543234721102635376155376434326171875f,1.00529997279196958714919674093835055828094482421875f,0.6590999948620794857134796984610147774219512939453125f,-0.0655999549902974710136049907305277884006500244140625f,-0.612500064857304682419680830207653343677520751953125f,-1.778900047099590597809992686961777508258819580078125f,0.7641999783173198945718240793212316930294036865234375f,1.317000026471912832448651897720992565155029296875f,0.7797000141695138442088364172377623617649078369140625f,-0.0198999807074660139960542437620460987091064453125f,0.27389998950660210397245464264415204524993896484375f,0.036599985995888317802382516674697399139404296875f,0.19119998966455453714985424085170961916446685791015625f,-1.23960006554573798354113023378886282444000244140625f,-0.06050000575184866224986990346224047243595123291015625f,-0.4606999963551758892066345651983283460140228271484375f,0.21670001162141605544064759669709019362926483154296875f,-1.5969000268861657243490981272771023213863372802734375f,0.157399988642335397770466443034820258617401123046875f,-0.36050001282989985185167824965901672840118408203125f,-1.040599983209371526271524999174289405345916748046875f,0.6587999918207525507085620120051316916942596435546875f,-0.043199980224668532713394597521983087062835693359375f,1.378000001132487994937037001363933086395263671875f,-1.500500017933547258763837817241437733173370361328125f,-0.494800033238530279078304374706931412220001220703125f,1.1882999990701674786208741352311335504055023193359375f,-1.510499980047345314204676469671539962291717529296875f,0.84530001704841828402692271993146277964115142822265625f,1.4334999875128271451529826663318090140819549560546875f,-1.1452000122845173990526745910756289958953857421875f,1.276900020404160063236531641450710594654083251953125f,0.2430999841034415798191048452281393110752105712890625f,0.748899981385469093453366440371610224246978759765625f,0.29360001198351393103536111084395088255405426025390625f,-0.608499977063387209597067339927889406681060791015625f,-0.5397000570662326968118804870755411684513092041015625f,1.41690003379657891713350181817077100276947021484375f,-1.4106999757170679554718617509934119880199432373046875f,-0.03810003875344991630669255755492486059665679931640625f,0.619999998733401280759380824747495353221893310546875f,-0.3192000142663717898727782085188664495944976806640625f,0.19529998661279623917153003276325762271881103515625f,0.7211000127285718264147362788207828998565673828125f,-1.3240999723613258165499928509234450757503509521484375f,1.1475000220537181494506739909411408007144927978515625f,-0.821399989506601979627475884626619517803192138671875f,-1.8594000298291446515719371745944954454898834228515625f,0.2037000361189249542093193667824380099773406982421875f,0.5220999544203284159493705374188721179962158203125f,0.0698000293642280222883300666580908000469207763671875f,0.0460999828144907919380557359545491635799407958984375f,-0.7031999789580698489999122102744877338409423828125f,0.2409000058084720397033606786862947046756744384765625f,0.97280003656893965224838893846026621758937835693359375f,-1.653999962866305128983412942034192383289337158203125f,-1.0427000562429433738742545756394974887371063232421875f,2.097800005909800091785655240528285503387451171875f,0.74130004155337891891264234800473786890506744384765625f,-1.748600035154820542260267757228575646877288818359375f,0.7517000399708753111127634838339872658252716064453125f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.33610003098249474629000133063527755439281463623046875f,0.905900037920474687780370004475116729736328125f,0.5704999715536833715390230281627736985683441162109375f,1.20339999716579892918844052474014461040496826171875f,0.48690000174045600100924957587267272174358367919921875f,0.25840003829300339788233031868003308773040771484375f,0.760100047381222498898978301440365612506866455078125f,2.1144000138849019521103400620631873607635498046875f,0.750799987670780044624052607105113565921783447265625f,-0.38190002252757604406241398464771918952465057373046875f,-0.069600014322996894833295300486497581005096435546875f,-1.31469996019005730403250709059648215770721435546875f,1.56160004303008381754125366569496691226959228515625f,1.228799976882338551575912788393907248973846435546875f,0.62079998793154922420711727681919001042842864990234375f,-1.2014000110574067914370743892504833638668060302734375f,-2.608900022810697993946860151481814682483673095703125f,0.38539996209591642628566887651686556637287139892578125f,-1.594999981261789656628025113604962825775146484375f,-1.1355999621242280905875077223754487931728363037109375f,1.20859992645084890483531125937588512897491455078125f,-1.525100018174946558957572051440365612506866455078125f,-1.4689999755136666781396570513607002794742584228515625f,-0.34990000287294364955670289418776519596576690673828125f,-1.232600003311037806241756698000244796276092529296875f,-1.3209000124394896236168506220565177500247955322265625f,1.0255000028461218608555327591602690517902374267578125f,0.98720000340342561884199312771670520305633544921875f,1.5070999738290904446103013469837605953216552734375f,0.79210000459998763933100462963921017944812774658203125f,2.2001999708592894222647373680956661701202392578125f,2.2273999950498346578342534485273063182830810546875f,2.607799945858121137831631131120957434177398681640625f,1.5722999512076378803016041274531744420528411865234375f,1.07129998738765674914930059458129107952117919921875f,1.4228999839633702517716073998599313199520111083984375f,0.8384999966770407819893762280116789042949676513671875f,1.54629996328502983260477776639163494110107421875f,-0.444900014570355661902567590004764497280120849609375f,-0.360299999874829968149470005300827324390411376953125f,-1.238900032570958043010023175156675279140472412109375f,-0.34239999087750916384464971997658722102642059326171875f,0.435900042167306711604624069877900183200836181640625f,0.72970004047006320480051044796709902584552764892578125f,0.61060004087686525764411271666176617145538330078125f,1.2000000376254309486512283910997211933135986328125f,0.7179000105619437466231147482176311314105987548828125f,0.434200019817053917137172902585007250308990478515625f,-0.7576999710083007766314722175593487918376922607421875f,-0.78610000486820930110098970544640906155109405517578125f,-0.40059997750967657470511085193720646202564239501953125f,-0.31179999343752928808726210263557732105255126953125f,0.0460000160485503073459767620079219341278076171875f,1.1661001144289968589617956240545026957988739013671875f,-0.5166999864012000731605667169787921011447906494140625f,0.5329999814629555743294986314140260219573974609375f,-0.5943000122129917794921993845491670072078704833984375f,0.3429999709762630022424900744226761162281036376953125f,-0.069400003714859381798163440180360339581966400146484375f,-1.5919999905601136536148487721220590174198150634765625f,-0.540400020806492076275162617093883454799652099609375f,-1.0560000268518923149940746952779591083526611328125f,0.82670008154511354092619512812234461307525634765625f,2.09500000469386638002333711483515799045562744140625f,1.52930007427632741467959931469522416591644287109375f,0.493400002679228233404273851192556321620941162109375f,0.0577000536352392234817898497567512094974517822265625f,0.472799977970123297410509621840901672840118408203125f,0.0535000299811361212931615227716974914073944091796875f,0.19919998967945595769180044953827746212482452392578125f,-0.685799984616040969598316223709844052791595458984375f,0.57529998320415653978443515370599925518035888671875f,-0.70490000538751473868614993989467620849609375f,-1.28509993569702007931709886179305613040924072265625f,-0.511700034196674646835845123860053718090057373046875f,0.0056999988138672907922455124207772314548492431640625f,-0.6246999981880188013150245751603506505489349365234375f,-1.970500006385147440113314587506465613842010498046875f,0.366499972347170466235866115312092006206512451171875f,1.055200051102042291262250728323124349117279052734375f,1.8942000325202936750201843096874654293060302734375f,1.4159000544607642790850832170690409839153289794921875f,2.776500019863247903373348890454508364200592041015625f,0.89930001102089907050185502157546579837799072265625f,0.55639998879879737359033242682926356792449951171875f,0.52230003225132737298963547800667583942413330078125f,1.239099968226253789538304772577248513698577880859375f,0.4665999785080554573823974351398646831512451171875f,-0.9661999961279337245656506638624705374240875244140625f,1.9888999259531505714448940125294029712677001953125f,0.1727000213742257983540184795856475830078125f,1.14040001536756729905164320371113717555999755859375f,0.7278000368669628183937447829521261155605316162109375f,2.071499993339180445417468945379368960857391357421875f,1.7113000098884108002295079131727106869220733642578125f,1.1198999982163313493543910226435400545597076416015625f,0.874199988710880671050063028815202414989471435546875f,0.73780000527650113628652661645901389420032501220703125f,0.5331000234425069539412334052030928432941436767578125f,1.505700030776858877601398489787243306636810302734375f,0.05840002160370350026141750277020037174224853515625f,1.18359998091459228675148551701568067073822021484375f,1.062699959534406435324171980028040707111358642578125f,-1.390200011986495010063435984193347394466400146484375f,-0.38790006029456802760790878892294131219387054443359375f,0.38950000557303432469069548460538499057292938232421875f,-0.713100000271201128754228193429298698902130126953125f,1.3372999827235936276537131561781279742717742919921875f,-0.56229997557103639227449320969753898680210113525390625f,-1.21319996733516521913998076342977583408355712890625f,-1.145900024677067197131918874219991266727447509765625f,-1.009199969972670363205224930425174534320831298828125f,0.51560002284646078152263726224191486835479736328125f,0.77819997437596299949547073993016965687274932861328125f,0.643499976262449369102114360430277884006500244140625f,-0.7592000659734006173806619699462316930294036865234375f,0.6118000189185135440084195579402148723602294921875f,0.69250000849366255550876303459517657756805419921875f,-1.379499976366759028678643517196178436279296875f,-1.760400047889351782259836909361183643341064453125f,-1.77069996299520138194338869652710855007171630859375f,0.708699920206516509324501384980976581573486328125f,0.787600019925833283451765964855439960956573486328125f,0.85770000483393660228870203354745171964168548583984375f,-1.4774000050708646636365983795258216559886932373046875f,-1.4257000069350007631641119587584398686885833740234375f,-0.561600008943676787254162263707257807254791259765625f,-0.7276000131458040254273100799764506518840789794921875f,-0.487299983319640528378613453241996467113494873046875f,-2.106500015020370764062818125239573419094085693359375f,0.5365000087618827517843556051957421004772186279296875f,-1.329899999184906089766400327789597213268280029296875f,-2.241299952369928405460086651146411895751953125f,0.48529999138116808321541384430020116269588470458984375f,1.647300028395652038426533181336708366870880126953125f,-0.6708000104695546195188171623158268630504608154296875f,0.1311999980837101897890306645422242581844329833984375f,-0.2665999956816433869022375802160240709781646728515625f,0.261700014303624717282303890897310338914394378662109375f,-1.350100010314583709458702287520281970500946044921875f,-0.8719000293821095315394131830544210970401763916015625f,-0.5849000495880840588824867154471576213836669921875f,0.81590003449320869233218900262727402150630950927734375f,0.0362000209197399502869529896997846662998199462890625f,0.68750002628192297127185383942560292780399322509765625f,0.30359998352229522478040735222748480737209320068359375f,0.91449997234344504448699808563105762004852294921875f,-0.394200005195289693293858590550371445715427398681640625f,-1.25799998563528081518825274542905390262603759765625f,0.20539999331384806158240508011658675968647003173828125f,-0.9602000018164511541129968463792465627193450927734375f,0.8817000135213131084555016059312038123607635498046875f,0.19749996624886989593505859375f,0.2380000248253342221005368628539144992828369140625f,-0.557800009448826461522230601985938847064971923828125f,-1.474800000698118562780791762634180486202239990234375f,0.6018000277474520043341499331290833652019500732421875f,0.7277999920517201903891191250295378267765045166015625f,0.003000008545816168492592623806558549404144287109375f,-0.299299983751774245632759630098007619380950927734375f,0.11849995398521517842027606093324720859527587890625f,-0.8586999932408330149513631113222800195217132568359375f,-1.499200047086179932165350692230276763439178466796875f,-0.5557000076800584498215584972058422863483428955078125f,-1.111700012999773168331785200280137360095977783203125f,-1.1551000132203099735050955132464878261089324951171875f,-0.473500029981136716372702721855603158473968505859375f,-1.585000007376074382392516781692393124103546142578125f,0.39400001658499217427333860541693866252899169921875f,0.1944999899268158305432052657124586403369903564453125f,1.650600036481022936385443244944326579570770263671875f,0.826899938485026186896220679045654833316802978515625f,0.291100052663683772635749846813268959522247314453125f,1.64509995948299714285667505464516580104827880859375f,0.23049998576566588237568566910340450704097747802734375f,-0.27139997736215570878215430639102123677730560302734375f,-1.43449999827146523756482565659098327159881591796875f,-0.95019998531341531844418568653054535388946533203125f,-0.6256000008672477630256025804555974900722503662109375f,-1.8322999814197429291340313284308649599552154541015625f,-1.13490005940422378216680954210460186004638671875f,-0.3161999938741331117597610500524751842021942138671875f,0.6564000091761352795316497577005065977573394775390625f,-0.472499974146485701709252680302597582340240478515625f,2.17660003576576688288923833169974386692047119140625f,-0.49119996686577749311908291929285041987895965576171875f,0.6000000251084571800674893893301486968994140625f,1.314000015392899012312000195379368960857391357421875f,0.11490002239346441825773581513203680515289306640625f,0.364299967136978875004160727257840335369110107421875f,-2.19299995419383098038679236196912825107574462890625f,-1.477299974453449404876437256461940705776214599609375f,0.138800013986230652562881004996597766876220703125f,-1.0062999625772242051624516534502618014812469482421875f,1.0702000124335295794253397616557776927947998046875f,2.16999998047947872947815994848497211933135986328125f,1.0738999904751775904543364958954043686389923095703125f,0.1088000400632618269725071513676084578037261962890625f,0.4699000470921406158453237367211841046810150146484375f,0.833199999298155358218309629592113196849822998046875f,-0.74309998604059279703193396926508285105228424072265625f,0.19550005399435754771531037476961500942707061767578125f,0.34880002354159922450804742766194976866245269775390625f,-1.054100006801635291964203133829869329929351806640625f,-1.1861999900929627305146141225122846662998199462890625f,-1.319699972185492509169080221909098327159881591796875f,-0.8829999527037137863061388998175971210002899169921875f,-0.1506999950140712751789351386833004653453826904296875f,-2.13139990486800723346050290274433791637420654296875f,-0.79799998209625488954088723403401672840118408203125f,0.2018000331491232035574512337916530668735504150390625f,-1.284999976754189088268276464077644050121307373046875f,0.21090000320076995876661385409533977508544921875f,0.696699971202015877480562267010100185871124267578125f,0.20029994645416682175209643901325762271881103515625f,-0.0277000671207900950321345590054988861083984375f,0.2005999838054179917889996431767940521240234375f,1.585999996304511938660652958787977695465087890625f,-1.024099967667460919784616635297425091266632080078125f,0.61060000422000937359001682125381194055080413818359375f,1.046700007411837329840409438475035130977630615234375f,0.130299954500794346046177452080883085727691650390625f,0.05699999856948834153769212207407690584659576416015625f,-0.1098000478416676362058979066205210983753204345703125f,-0.73740000928044313610598692321218550205230712890625f,-0.27689998672753601827167813098640181124210357666015625f,0.435600005672872381268234676099382340908050537109375f,-0.4806999708741905852349418637459166347980499267578125f,-0.650999995559453648041881024255417287349700927734375f,1.595100017467141473304081955575384199619293212890625f,-0.52780000851750319412758472026325762271881103515625f,1.03790000579357144516734479111619293689727783203125f,1.710599950501322741303056318429298698902130126953125f,0.79120003965795060452137477113865315914154052734375f,2.30989995742440168413622814114205539226531982421875f,-0.039099987336992825248671579174697399139404296875f,-0.7270000194311141417102817285922355949878692626953125f,-0.54740003625154454169887685566209256649017333984375f,0.9412000214412807874708732924773357808589935302734375f,0.0195999554075296789079629888874478638172149658203125f,0.560999991983174961518443524255417287349700927734375f,1.33140001118779149891224733437411487102508544921875f,-0.717000016972421772010193308233283460140228271484375f,-1.72299997900426316022048922604881227016448974609375f,-0.249000024445355183644323915359564125537872314453125f,-1.0640000063776966499062837101519107818603515625f,0.6773999708354472293336812072084285318851470947265625f,-0.84880000187903636987840627625701017677783966064453125f,-0.018199935558438962157623564053210429847240447998046875f,0.602999966487288485694762130151502788066864013671875f,0.8294000786304474015508958473219536244869232177734375f,0.341899993954599190626453264485462568700313568115234375f,-0.03110002610236428655099416573648341000080108642578125f,-1.5639000548481940544576218599104322493076324462890625f,-0.4216000264525410212712586144334636628627777099609375f,0.30799999770522124009630715590901672840118408203125f,-0.3750000309944148124685625589336268603801727294921875f,1.027600040489434984891659041750244796276092529296875f,0.49309999289512607134611243964172899723052978515625f,0.5558000050678852943519814289174973964691162109375f,-0.24899999102950065577033456065692007541656494140625f,2.263200021314620613566148676909506320953369140625f,0.82210003317296553859705454669892787933349609375f,-0.39269996027946518779572215862572193145751953125f,-0.13820000693500045230166506371460855007171630859375f,-0.402600011879206132192621225840412080287933349609375f,-0.1549999663420020823423328693024814128875732421875f,0.4358000071540477637910271369037218391895294189453125f,0.07690000367760763122504386046784929931163787841796875f,-0.73709996924698428788502724273712374269962310791015625f,-0.838099990659952442939584216219373047351837158203125f,0.28680004223883115432869317373842932283878326416015625f,0.189200009125471080295710635255090892314910888671875f,0.22879996961802173682798411391559056937694549560546875f,0.2279999656677251795855454474803991615772247314453125f,-1.656399980565905760698797166696749627590179443359375f,-0.03680002234578072517479085945524275302886962890625f,1.087799967241287912855796093936078250408172607421875f,0.11779999622404535575270756453392095863819122314453125f,-0.8598999814897776428068709719809703528881072998046875f,-0.915600024262070544267544391914270818233489990234375f,-0.94650002932548460421458003111183643341064453125f,-2.254500001028180378170873154886066913604736328125f,-0.6766999543637035596788109614863060414791107177734375f,-0.413099986115098349870322635979391634464263916015625f,-0.8483000068932768922280729384510777890682220458984375f,1.26180002212226316515852886368520557880401611328125f,-1.59700002144277153348639330943115055561065673828125f,-1.987999998748301688777928575291298329830169677734375f,-1.245999962106346448109661650960333645343780517578125f,-1.4710999887377018691125840632594190537929534912109375f,-1.08570003308653895146562717854976654052734375f,1.61009998289644773450390857760794460773468017578125f,-0.15519999008178775312671859865076839923858642578125f,-0.5327999635159967883879517103196121752262115478515625f,0.598700030270218785943825423601083457469940185546875f,-0.875499934002756052819904653006233274936676025390625f,0.988599988812207897126427269540727138519287109375f,-0.2847000297039750105199118479504249989986419677734375f,0.831400004631280165057205522316507995128631591796875f,-0.34020002584457387229122105054557323455810546875f,1.13720002281218768303006072528660297393798828125f,0.3165999918073421071085249423049390316009521484375f,-0.980599967563152485894306664704345166683197021484375f,-0.370700044858455424190424309927038848400115966796875f,-1.879799979966879863724216193077154457569122314453125f,0.7383999938130376694545020654913969337940216064453125f,0.3511000136971471619062867830507457256317138671875f,-0.88340000007152585936864852556027472019195556640625f,-0.75870001451224078348190005272044800221920013427734375f,-2.4250999778673048012933577410876750946044921875f,-1.025999970898032298549651386565528810024261474609375f,-0.9409999853521584523008414180367253720760345458984375f,-1.743699986535310220148176085785962641239166259765625f,-0.573699994283914715964556307881139218807220458984375f,0.20140001986771849384894039758364669978618621826171875f,0.03909997915998097983703019053791649639606475830078125f,-0.69519998976513708299762583919800817966461181640625f,1.77930001198947440599340552580542862415313720703125f,-1.125799981151521134137283297604881227016448974609375f,0.73399998085945850068156914858263917267322540283203125f,-0.3074999624490744043470158430864103138446807861328125f,-0.026199986237287475887569598853588104248046875f,0.72010001485943764265584832173772156238555908203125f,0.54050001201033648356997218797914683818817138671875f,0.311500011295079648476757938624359667301177978515625f,1.281599939280748667869147539022378623485565185546875f,-1.5461999559879313625998520365101285278797149658203125f,-0.22110002427697139637530199252068996429443359375f,0.75550002728402620988390481215901672840118408203125f,-0.810399975320696075442583605763502418994903564453125f,-0.6486999977111824744469004144775681197643280029296875f,-0.524799992558360184347066024201922118663787841796875f,-0.203999996319413678946830259519629180431365966796875f,-0.37150005435943622700278865522705018520355224609375f,-1.8411999893665313976498509873636066913604736328125f,-0.4890999829038975565964619818259961903095245361328125f,0.3880999384313816857883239208604209125041961669921875f,-0.2438999421954155533143193679279647767543792724609375f,-2.248800001022220218516167733469046652317047119140625f,-0.37750004820525562276856135213165543973445892333984375f,0.51469997725188676440666313283145427703857421875f,-0.778000007092952028386889651301316916942596435546875f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,1.7081999867811792359617584224906750023365020751953125f,-0.70470000765025619937631518041598610579967498779296875f,-0.91959995635747848741203824829426594078540802001953125f,0.17130000258684041636314532297546975314617156982421875f,2.172000042993575963379271343001164495944976806640625f,-0.4771000425420706125834158228826709091663360595703125f,-0.329399994066357282296308994773426093161106109619140625f,0.5417999969951809990931224092491902410984039306640625f,0.12559999272748745280381399425095878541469573974609375f,0.20380000264272124521625073612085543572902679443359375f,-0.64949998307228085092646097109536640346050262451171875f,0.479599971128254642938060214873985387384891510009765625f,-1.117400050668418298727146975579671561717987060546875f,1.37210003879815278793330435291863977909088134765625f,-1.2852000248201187471153161823167465627193450927734375f,0.50730002264007956203073490542010404169559478759765625f,-0.77089998561739825344574228438432328402996063232421875f,-0.5205999884247780595813992476905696094036102294921875f,0.002199930555373430518528721222537569701671600341796875f,-0.14899996005371163310115889544249512255191802978515625f,0.78790002833157746220393846670049242675304412841796875f,-0.333299998474120451330549030899419449269771575927734375f,-0.232999978855251976828100168859236873686313629150390625f,0.96030004932805901329828657253528945147991180419921875f,-0.27530003688558946084441458879155106842517852783203125f,0.45630001575574297145720947810332290828227996826171875f,1.1538999992109832959386039874516427516937255859375f,0.06480001347586494819807967360247857868671417236328125f,0.2128000016815965178551550707197748124599456787109375f,0.40179999543055888722165036597289144992828369140625f,0.6695999632492657394067236964474432170391082763671875f,0.75579999594092350800877966321422718465328216552734375f,0.36220001135691981541953055057092569768428802490234375f,1.3957000069350000703849445926607586443424224853515625f,-0.43590003985762615368315664454712532460689544677734375f,0.1913000213064261767481610831964644603431224822998046875f,1.4107000285416837481733409731532447040081024169921875f,0.0153999973185363348004983663486200384795665740966796875f,-0.94389996924102259612254783860407769680023193359375f,-0.263700044314564230329978045119787566363811492919921875f,1.030700034166872836038919558632187545299530029296875f,0.272499987035989488237675004711491055786609649658203125f,-0.0577000273533164464989653197335428558290004730224609375f,-0.367299984120577516222283520619384944438934326171875f,1.11990000376701370754517483874224126338958740234375f,-0.156999988939612700189485394730581901967525482177734375f,0.214900032675266583481743509764783084392547607421875f,-1.387300028451531996864787288359366357326507568359375f,-0.6370999930888412432494760651024989783763885498046875f,-0.74560005505159576699725221260450780391693115234375f,-1.631900011873244960014517346280626952648162841796875f,2.04080006595775476085918853641487658023834228515625f,-0.07280000147670417920409136058879084885120391845703125f,-0.34090001527071012166203445303835906088352203369140625f,0.9605000022500755907373104491853155195713043212890625f,-0.471399994386732412721130458521656692028045654296875f,0.101600026229024142931933738509542308747768402099609375f,0.59440001399666064774152118843630887567996978759765625f,0.27909996830075967100270872833789326250553131103515625f,-0.50960000927522786096091067520319484174251556396484375f,-0.016199986088276496776217072692816145718097686767578125f,0.96540002687871517839113266745698638260364532470703125f,-0.38840000955238884738918159200693480670452117919921875f,0.0101999677486712680973823808017186820507049560546875f,2.1325000491924583911895751953125f,0.7634000022508207194960050401277840137481689453125f,-0.66959998873025206034270695454324595630168914794921875f,-0.405200004275142522391917054846999235451221466064453125f,-0.68210001612976245422004240026581101119518280029296875f,0.7836000167891388201724112150259315967559814453125f,-0.063599986316264234265105415033758617937564849853515625f,-0.06300001318380255010964674511342309415340423583984375f,0.02640004311353028043640733812935650348663330078125f,-0.1449000278137624331264987631584517657756805419921875f,-1.17169999234303734425566290155984461307525634765625f,0.37380004164651070563962775850086472928524017333984375f,-0.070100025793165154919250880993786267936229705810546875f,0.69090003812536604499427994596771895885467529296875f,-0.8124000165447597954226921501685865223407745361328125f,1.639500046793372245446107626776210963726043701171875f,0.7027000021144746977341810634243302047252655029296875f,1.51700006156414790581266061053611338138580322265625f,0.706499985925853390966722145094536244869232177734375f,0.05960000128447984213408972209435887634754180908203125f,0.0510000079832973740590063016497879289090633392333984375f,0.341999995253980226816992171734455041587352752685546875f,1.203200013733655371339636985794641077518463134765625f,0.60779999033808707054049591533839702606201171875f,0.21140003705993326565959478102740831673145294189453125f,-0.8131000117078424072047937443130649626255035400390625f,1.030499987311660703426241525448858737945556640625f,-0.89480001906380124410134158097207546234130859375f,1.259800004237145554242260914179496467113494873046875f,1.610000008381903047194327882607467472553253173828125f,0.570500017486512689401934039779007434844970703125f,0.209200031700730371131413676266674883663654327392578125f,-0.66569999074861396781699340863269753754138946533203125f,0.4030000012442467305362470142426900565624237060546875f,0.013899974102526357899733966405619867146015167236328125f,-0.8196000066861512944882406372926197946071624755859375f,0.3305000084899363255175330778001807630062103271484375f,-0.205900034418702748695295667857863008975982666015625f,0.51840000113323358998940193487214855849742889404296875f,0.4802999664030964677152724107145331799983978271484375f,-1.262900021477043832618392116273753345012664794921875f,0.63629998673573162637495670423959381878376007080078125f,0.326100066149235257029204149148426949977874755859375f,-1.5717000211209057880523687344975769519805908203125f,-1.39789999342560644635113931144587695598602294921875f,0.0898999817505477250012546619473141618072986602783203125f,-1.2100000554695726773246633456437848508358001708984375f,0.076900047133118654141270553736831061542034149169921875f,0.5471000046931202565048124597524292767047882080078125f,-0.22599999945238169463124222602345980703830718994140625f,-1.2904000196702789526881360870902426540851593017578125f,2.0486000459767890191642436548136174678802490234375f,-1.0823000775694862074516322536510415375232696533203125f,-0.77310001560077046267593914308235980570316314697265625f,1.324100040552765644719102056114934384822845458984375f,-0.55170002125129136150150088724330998957157135009765625f,-1.1170000723116102481213829378248192369937896728515625f,-0.5053999864965670330008151722722686827182769775390625f,1.379299995560943070671555688022635877132415771484375f,0.9735000073686246313542369534843601286411285400390625f,-0.84790002482980530995604340205318294465541839599609375f,-1.5254000463061039294387910558725707232952117919921875f,0.0522000315852466645338836315204389393329620361328125f,0.5134999742507926345780333576840348541736602783203125f,0.025099988279491503018192588569945655763149261474609375f,-0.10899996625632031588271075861484860070049762725830078125f,-1.6358000273264943214002187232836149632930755615234375f,0.08740000587180285496202714057289995253086090087890625f,0.368499990157782786326379209640435874462127685546875f,-0.1679999792836596450573694028207683004438877105712890625f,-0.87570000300481976029942643435788340866565704345703125f,1.206900014611334004399623154313303530216217041015625f,0.33340000996217089568318669989821501076221466064453125f,-0.5259000237829980495263271222938783466815948486328125f,-1.3980000347718597453194888657890260219573974609375f,0.18349999928474414190304742078296840190887451171875f,-0.164099960310011561137599755966220982372760772705078125f,0.25999999137595219966812010170542635023593902587890625f,-0.219800026756525601623337706769234500825405120849609375f,1.130900021175295311337549719610251486301422119140625f,0.41269999416098013256259946501813828945159912109375f,0.6002000157862907858685730388970114290714263916015625f,-0.052700029327720887550867701065726578235626220703125f,-0.37080003371536796041851857808069325983524322509765625f,-0.453299993314593752113950131388264708220958709716796875f,-0.866799988110363184290463323122821748256683349609375f,0.497700041900575484987712115980684757232666015625f,0.5851999717906115616727902306593023240566253662109375f,0.2400000059790914352930712993838824331760406494140625f,1.016400027523189919520518742501735687255859375f,0.9879000249788159049302294079097919166088104248046875f,1.3934999821670353004066100766067393124103546142578125f,-1.1234999846257267552829262058367021381855010986328125f,0.711000010106712210244950256310403347015380859375f,-0.63360003299415013078288438919116742908954620361328125f,-1.0009000345304619461472839248017407953739166259765625f,1.003500018898397794231414081878028810024261474609375f,-0.329999999068677329905341366611537523567676544189453125f,0.68080004820674755450227166875265538692474365234375f,-0.66940003459751673542399430516525171697139739990234375f,-0.6899000132292505416131689344183541834354400634765625f,-0.7772000293686989191854763703304342925548553466796875f,-0.1489999952390785864508870872668921947479248046875f,0.92430001901164615674133528955280780792236328125f,0.509500012725592199558377615176141262054443359375f,-0.0190000432394448193207381336833350360393524169921875f,1.6484000507540998992084269048064015805721282958984375f,0.6647000135920944074996441486291587352752685546875f,-1.2829000286482283854638808406889438629150390625f,-0.505200014035402933387786106322892010211944580078125f,0.8054000381849704126580036245286464691162109375f,-0.59429994893893456353595183827565051615238189697265625f,0.42730000440478266288124586935737170279026031494140625f,-0.92570004482120271660505750332958996295928955078125f,-0.6950999886706465957786349463276565074920654296875f,1.11669999459683833009648878942243754863739013671875f,0.09210002877712397673093391858856193721294403076171875f,-1.3497000011309989719876512026530690491199493408203125f,-0.0191999932371077208781429135342477820813655853271484375f,-0.468699990893900741983912894284003414213657379150390625f,-0.55349998730793525947291300326469354331493377685546875f,1.601200021310895227344417435233481228351593017578125f,0.51849999162927229434671971830539405345916748046875f,-0.02819999287203012183766759335412643849849700927734375f,0.26689999509081296924506432333146221935749053955078125f,-0.82230003456100864145383866343763656914234161376953125f,0.3415000263080008036098433876759372651576995849609375f,0.79310000476017517012650159813347272574901580810546875f,-0.68470002309158417563850207443465478718280792236328125f,0.8345000263787800687254048170871101319789886474609375f,0.76119999869838361750140620642923749983310699462890625f,0.03040001177266182796898874585167504847049713134765625f,1.0323000680513683136041436227969825267791748046875f,0.9022999950729306828378639693255536258220672607421875f,-1.130900022423267703430838082567788660526275634765625f,0.09400007043406477524882802754291333258152008056640625f,2.449600042877346783853909073513932526111602783203125f,0.8894999496005475680959762030397541821002960205078125f,-1.0460000240392981485371137750917114317417144775390625f,-0.4309999790750447257181576787843368947505950927734375f,-0.138300000374018849935708885823260061442852020263671875f,-1.7585999893896271384363672041217796504497528076171875f,-0.0604999708272514202800351768019027076661586761474609375f,0.1663999916486448016339494415660738013684749603271484375f,0.2957999899618328765171781924436800181865692138671875f,-0.82020002770721844864709737521479837596416473388671875f,0.05150003195181562798410368486656807363033294677734375f,1.0736000337764626255676603250321932137012481689453125f,-1.906500060841441968051412914064712822437286376953125f,-1.1284000092543651216914213364361785352230072021484375f,1.1564999811202281154720594713580794632434844970703125f,0.33699995036423147087845109126647002995014190673828125f,-0.8128999818213282768653016319149173796176910400390625f,-0.9814000113926824031551632288028486073017120361328125f,1.0786000222653149283047468998120166361331939697265625f,0.5075000227428974053367483065812848508358001708984375f,-1.42510002774894228849689170601777732372283935546875f,-0.79759998781382979160525792394764721393585205078125f,-0.116800073754788957813843808253295719623565673828125f,0.07449999938905282892420700591173954308032989501953125f,0.71390006449669674015723330740001983940601348876953125f,-1.25060002077892473693054853356443345546722412109375f,-0.133299978078156355909555941252619959414005279541015625f,1.5151000266499818902588003766140900552272796630859375f,-1.145499978035687593802549599786289036273956298828125f,-0.63750000644475246502906884416006505489349365234375f,1.4570999822296197834248232538811862468719482421875f,0.57180002031549814756061778098228387534618377685546875f,0.47019996298029942405349856926477514207363128662109375f,0.34800002540275476459186165811843238770961761474609375f,-0.42869999631419763996831306940293870866298675537109375f,-0.370400017416477267406804685379029251635074615478515625f,0.167300017517805155176091602697852067649364471435546875f,-0.80640002107843822276578293894999660551548004150390625f,0.92590003020912392539543134262203238904476165771484375f,0.66979995672106706905424289288930594921112060546875f,-1.2198999971732489289166778689832426607608795166015625f,-0.45779995675459483894798040637397207319736480712890625f,-0.031200002516806046859443313223891891539096832275390625f,0.7260000180229548849553111722343601286411285400390625f,-1.165200050841272538804105352028273046016693115234375f,-0.62769999864995529303968169188010506331920623779296875f,1.3654999936819069983329200113075785338878631591796875f,0.081699985805154085038992661793599836528301239013671875f,0.355100018752365798047065936771105043590068817138671875f,-0.4501999943099905809873462203540839254856109619140625f,1.3923000656299293620321577691356651484966278076171875f,0.62249996237456795000753118074499070644378662109375f,0.0696999745324253516809420716526801697909832000732421875f,-0.5180000295750801786454076136578805744647979736328125f,-0.2109000054173171267990483102039434015750885009765625f,1.3901000500820577254756926777190528810024261474609375f,0.4193999889440831463360837005893699824810028076171875f,-0.8608000354476279536442007156438194215297698974609375f,0.88300004103034812619199556138482876121997833251953125f,0.77470002800896808903274859403609298169612884521484375f,0.294100050368905685527209925567149184644222259521484375f,-0.82760003821700889314882942926487885415554046630859375f,0.98100000802054998505496996585861779749393463134765625f,0.6535000010915099633024283320992253720760345458984375f,-0.33110000110566606412731971431639976799488067626953125f,-0.58690002727732082377087863278575241565704345703125f,-0.02269998050779060971837708393650245852768421173095703125f,-0.2120999902948736970387244582525454461574554443359375f,0.44020001086890669927953467777115292847156524658203125f,-0.3389000351414086509294065763242542743682861328125f,0.12679999811276754240196851242217235267162322998046875f,1.1151999663330613987710648871143348515033721923828125f,-0.5246999967724084168452236554003320634365081787109375f,-0.63939999386146606497050015605054795742034912109375f,0.48589999621584956113196085425443015992641448974609375f,0.57859994533807068695097086674650199711322784423828125f,-1.432499998249114003812110240687616169452667236328125f,0.237200019627064218052936439562472514808177947998046875f,0.0052999813787637517492612460046075284481048583984375f,0.4325000050105154425494902170612476766109466552734375f,-0.0873000034362081678285250063709099777042865753173828125f,-0.32390004460364618665835223509930074214935302734375f,-0.33380001299902861688906341441906988620758056640625f,-1.46100002830475528270426366361789405345916748046875f,0.48330004818812033473562905783182941377162933349609375f,0.92679998891130088711776124910102225840091705322265625f,-0.67760000554621202528693402200588025152683258056640625f,0.9695000360459087307418712953221984207630157470703125f,-0.52119998344331930351103210341534577310085296630859375f,-2.158300093059243085491516467300243675708770751953125f,0.20300000659003902825361365103162825107574462890625f,0.46679999326988996788401209414587356150150299072265625f,-1.44260003339275666434105005464516580104827880859375f,1.608000012103467657453848005388863384723663330078125f,-0.589900005033612284677246861974708735942840576171875f,-0.0653000068783760045931785498396493494510650634765625f,1.007399995608627474297236403799615800380706787109375f,-1.2693000412888821859525023683090694248676300048828125f,-0.61680001765191649898412151742377318441867828369140625f,0.481299951028823558285552053348510526120662689208984375f,-1.136499988347291267842820161604322493076324462890625f,0.462700019604712797782752886632806621491909027099609375f,0.28629997405111773378649786536698229610919952392578125f,-1.1375999842375514869985408950014971196651458740234375f,-0.56299998725578148661696786803076975047588348388671875f,-0.30180003149136902518279157447977922856807708740234375f,-0.21210001236721820472297395099303685128688812255859375f,0.95690001667886992464673312497325241565704345703125f,-1.619299962610750487357336169225163757801055908203125f,0.800200016307830441775195140508003532886505126953125f,-0.51110002774149132509506898713880218565464019775390625f,-1.4360999833546568282827138318680226802825927734375f,0.6231999994657930752595120793557725846767425537109375f,-0.178200021072476733063894016595440916717052459716796875f,-0.61889999129474182471000176519737578928470611572265625f,-1.6889000645712022308231325951055623590946197509765625f,0.64130003736242680023593720761709846556186676025390625f,2.25600000647455356528325864928774535655975341796875f,-1.0000000164285294257382474825135432183742523193359375f,0.5052000276699655234580177420866675674915313720703125f,-0.6809000314757225336137480553588829934597015380859375f,-0.59459997734948977221591803754563443362712860107421875f,0.7171000194825232210149579259450547397136688232421875f,0.25540001447349769403416530622052960097789764404296875f,-0.365199991180747807195672294255928136408329010009765625f,0.6062000321514904310760130101698450744152069091796875f,-0.509700021247565526749667697004042565822601318359375f,1.353699998959152939193018028163351118564605712890625f,-0.407099994448571533833813873570761643350124359130859375f,1.7038000144146370207209884028998203575611114501953125f,1.0671999920971695363647313570254482328891754150390625f,-1.558099972983449266195066229556687176227569580078125f,-2.08250008873641601070403339690528810024261474609375f,-0.950100005695223348567424181965179741382598876953125f,1.1899000190034509927272665663622319698333740234375f,-0.0484000296503308558015987728140316903591156005859375f,-0.9772000180810689329291562899015843868255615234375f,-0.90159999052211625691910512614413164556026458740234375f,0.330699988103657693816472828984842635691165924072265625f,-0.11649999665468835363668631543987430632114410400390625f,-0.94169997107163083693848193433950655162334442138671875f,0.18189998805001350579146901509375311434268951416015625f,-0.12389999503865845031924663999234326183795928955078125f};

    cf->call({a, b}, {result});
    EXPECT_TRUE(all_close<float>(vector<float>{expected_result}, read_vector<float>(result)));
    // only test backprop for certain cases as it takes significant compute resources
    if(false) {
        EXPECT_TRUE(autodiff_numeric_compare<float>(manager, backend, make_graph, {a, b}, .01f, .01f));
    }
}

// clang-format on
